"""
This module provides enumeration classes for results variables of 'Elm' classes (network elements). This is convenient to get results variable strings and supports code completion in your IDE. If you hover over a variable, it's unit and description will be shown.  

This file is automatically created using the .txt files in '.\result_variables' and using the module 'results_varaibles_parser'. The .txt files are created by copying from the output window of PF (manually) using the printed results variables when clicking on 'Variable List' in an .IntMon file of an 'Elm' class.

Currently, the following 'Elm' classes are supported: 

ElmArea
ElmAsm
ElmAsmsc
ElmBoundary
ElmCoup
ElmGenstat
ElmIac
ElmLne
ElmLod
ElmLodlv
ElmLodmv
ElmPvsys
ElmSecctrl
ElmShnt
ElmSind
ElmStactrl
ElmSym
ElmTerm
ElmTr2
ElmTr3
ElmVac
ElmVsc
ElmVscmono
ElmXnet
ElmZone
ElmZpu

The follwing simulation types/result variable types are supported:

Basic Data balanced: Basic
Load Flow AC balanced: LF_Bal
Load Flow AC unbalanced: LF_Unbal
Simulation RMS balanced: RMS_Bal
Simulation RMS unbalanced: RMS_Unbal
Simulation EMT unbalanced: EMT
Sensitivities / Distribution Factors AC balanced: Sensitivities_Bal
"""
from aenum import Enum

class ResVar:


  class Basic:

    class ElmArea(Enum):
      _init_ = 'value __doc__'

      b_loc_name = "b:loc_name", "Name"
      b_n_ele = "b:n_ele", "Number of Elements"
      b_n_areas = "b:n_areas", "Number of Areas"
      e_loc_name = "e:loc_name", "Name"
      e_icolor = "e:icolor", "Colour"

    class ElmAsm(Enum):
      _init_ = 'value __doc__'

      b_loc_name = "b:loc_name", "Name"
      b_i_mot = "b:i_mot", "Motor"
      b_iWindGen = "b:iWindGen", "Wind Generator"
      e_loc_name = "e:loc_name", "Name"
      e_typ_id = "e:typ_id", "TypAsm*,TypAsmo*,TypAsm1*, Type"
      e_bus1 = "e:bus1", "StaCubic, Terminal"
      e_bus1_bar = "e:bus1_bar", "Terminal"
      e_cpZone = "e:cpZone", "Zone"
      e_cpArea = "e:cpArea", "Area"
      e_outserv = "e:outserv", "Out of Service"
      e_phtech = "e:phtech", "Technology"
      e_ngnum = "e:ngnum", "parallel machines"
      e_i_mot = "e:i_mot", "Generator/Motor"
      e_cCategory = "e:cCategory", "Plant Category"
      e_cSubCategory = "e:cSubCategory", "Subcategory"
      e_c_pmod = "e:c_pmod", "Model"
      e_idfig = "e:idfig", "Machine type"
      e_Inom = "e:Inom", "kA, Nominal Current"
      e_iintgnd = "e:iintgnd", "N-Connection"
      e_bus1n = "e:bus1n", "StaCubic, Neutral"
      e_bus1n_bar = "e:bus1n_bar", "Neutral"
      e_cgnd = "e:cgnd", "Star Point"
      e_Re = "e:Re", "Ohm, Resistance, Re"
      e_Xe = "e:Xe", "Ohm, Reactance, Xe"
      e_cpGrid = "e:cpGrid", "Grid"
      e_cpOwner = "e:cpOwner", "Owner"
      e_cpOperator = "e:cpOperator", "Operator"
      e_cpBranch = "e:cpBranch", "Branch"
      e_cpSubstat = "e:cpSubstat", "Substation"
      e_cpSite = "e:cpSite", "Site"
      e_cpMeteostat = "e:cpMeteostat", "Meteo Station"
      e_cpHeadFold = "e:cpHeadFold", "Head Folder"
      e_fold_id = "e:fold_id", "In Folder"
      e_ciOutaged = "e:ciOutaged", "Planned Outage"
      e_ciEnergized = "e:ciEnergized", "Energized"
      e_ciEarthed = "e:ciEarthed", "Earthed"
      e_cDisplayName = "e:cDisplayName", "Display Name"
      e_cpSupplyTransformer = "e:cpSupplyTransformer", "Supplying Transformer"
      e_cpSupplyTrfStation = "e:cpSupplyTrfStation", "Supplying Secondary Substation"
      e_cpSupplySubstation = "e:cpSupplySubstation", "Supplying Substation"
      r_typ_id = "r:typ_id", "TypAsm*,TypAsmo*,TypAsm1*, Type"
      r_bus1 = "r:bus1", "StaCubic, Terminal"
      r_cpZone = "r:cpZone", "Zone"
      r_cpArea = "r:cpArea", "Area"
      r_c_pmod = "r:c_pmod", "Model"
      r_bus1n = "r:bus1n", "StaCubic, Neutral"
      r_cpGrid = "r:cpGrid", "Grid"
      r_cpOwner = "r:cpOwner", "Owner"
      r_cpOperator = "r:cpOperator", "Operator"
      r_cpBranch = "r:cpBranch", "Branch"
      r_cpSubstat = "r:cpSubstat", "Substation"
      r_cpSite = "r:cpSite", "Site"
      r_cpMeteostat = "r:cpMeteostat", "Meteo Station"
      r_cpHeadFold = "r:cpHeadFold", "Head Folder"
      r_fold_id = "r:fold_id", "In Folder"
      r_cpSupplyTransformer = "r:cpSupplyTransformer", "Supplying Transformer"
      r_cpSupplyTrfStation = "r:cpSupplyTrfStation", "Supplying Secondary Substation"
      r_cpSupplySubstation = "r:cpSupplySubstation", "Supplying Substation"

    class ElmAsmsc(Enum):
      _init_ = 'value __doc__'

      b_loc_name = "b:loc_name", "Name"
      b_i_mot = "b:i_mot", "Motor"
      b_iWindGen = "b:iWindGen", "Wind Generator"
      e_loc_name = "e:loc_name", "Name"
      e_typ_id = "e:typ_id", "TypAsmo*, Type"
      e_bus1 = "e:bus1", "StaCubic, Terminal AC"
      e_bus1_bar = "e:bus1_bar", "Terminal AC"
      e_bus2 = "e:bus2", "StaCubic, Terminal R"
      e_bus2_bar = "e:bus2_bar", "Terminal R"
      e_iZoneBus = "e:iZoneBus", "Zone"
      e_cpZone = "e:cpZone", "Zone"
      e_iAreaBus = "e:iAreaBus", "Area"
      e_cpArea = "e:cpArea", "Area"
      e_outserv = "e:outserv", "Out of Service"
      e_i_conv = "e:i_conv", "Use Integrated PWM Converter"
      e_ngnum = "e:ngnum", "parallel Machines"
      e_i_mot = "e:i_mot", "Generator/Motor"
      e_cCategory = "e:cCategory", "Plant Category"
      e_cSubCategory = "e:cSubCategory", "Subcategory"
      e_c_pmod = "e:c_pmod", "Model"
      e_Inom = "e:Inom", "kA, Nominal Current"
      e_cgnd = "e:cgnd", "Star Point"
      e_Re = "e:Re", "Ohm, Resistance, Re"
      e_Xe = "e:Xe", "Ohm, Reactance, Xe"
      e_cpGrid = "e:cpGrid", "Grid"
      e_cpOwner = "e:cpOwner", "Owner"
      e_cpOperator = "e:cpOperator", "Operator"
      e_cpBranch = "e:cpBranch", "Branch"
      e_cpSubstat = "e:cpSubstat", "Substation"
      e_cpSite = "e:cpSite", "Site"
      e_cpMeteostat = "e:cpMeteostat", "Meteo Station"
      e_cpHeadFold = "e:cpHeadFold", "Head Folder"
      e_fold_id = "e:fold_id", "In Folder"
      e_ciOutaged = "e:ciOutaged", "Planned Outage"
      e_ciEnergized = "e:ciEnergized", "Energized"
      e_ciEarthed = "e:ciEarthed", "Earthed"
      e_cDisplayName = "e:cDisplayName", "Display Name"
      e_cpSupplyTransformer = "e:cpSupplyTransformer", "Supplying Transformer"
      e_cpSupplyTrfStation = "e:cpSupplyTrfStation", "Supplying Secondary Substation"
      e_cpSupplySubstation = "e:cpSupplySubstation", "Supplying Substation"
      r_typ_id = "r:typ_id", "TypAsmo*, Type"
      r_bus1 = "r:bus1", "StaCubic, Terminal AC"
      r_bus2 = "r:bus2", "StaCubic, Terminal R"
      r_cpZone = "r:cpZone", "Zone"
      r_cpArea = "r:cpArea", "Area"
      r_c_pmod = "r:c_pmod", "Model"
      r_cpGrid = "r:cpGrid", "Grid"
      r_cpOwner = "r:cpOwner", "Owner"
      r_cpOperator = "r:cpOperator", "Operator"
      r_cpBranch = "r:cpBranch", "Branch"
      r_cpSubstat = "r:cpSubstat", "Substation"
      r_cpSite = "r:cpSite", "Site"
      r_cpMeteostat = "r:cpMeteostat", "Meteo Station"
      r_cpHeadFold = "r:cpHeadFold", "Head Folder"
      r_fold_id = "r:fold_id", "In Folder"
      r_cpSupplyTransformer = "r:cpSupplyTransformer", "Supplying Transformer"
      r_cpSupplyTrfStation = "r:cpSupplyTrfStation", "Supplying Secondary Substation"
      r_cpSupplySubstation = "r:cpSupplySubstation", "Supplying Substation"

    class ElmBoundary(Enum):
      _init_ = 'value __doc__'

      b_loc_name = "b:loc_name", "Name"
      e_loc_name = "e:loc_name", "Name"
      e_icolor = "e:icolor", "Colour"
      e_outserv = "e:outserv", "Out of Service"
      e_cubicles = "e:cubicles", "StaCubic, Boundary Cubicle"
      e_headfold = "e:headfold", "Head Folder"
      e_buses = "e:buses", "Busbar"
      e_branches = "e:branches", "Branch"
      e_ciorient = "e:ciorient", "Orientation"
      e_iSwitchStates = "e:iSwitchStates", "Topological search: Stop at open breakers"
      e_iNeutral = "e:iNeutral", "Topological search: consider neutral wire connections"
      r_cubicles = "r:cubicles", "StaCubic, Boundary Cubicle"
      r_headfold = "r:headfold", "Head Folder"
      r_buses = "r:buses", "Busbar"
      r_branches = "r:branches", "Branch"

    class ElmCoup(Enum):
      _init_ = 'value __doc__'

      b_loc_name = "b:loc_name", "Name"
      e_loc_name = "e:loc_name", "Name"
      e_typ_id = "e:typ_id", "TypSwitch, Type"
      e_bus1 = "e:bus1", "StaCubic, Terminal i"
      e_bus1_bar = "e:bus1_bar", "Terminal i"
      e_bus2 = "e:bus2", "StaCubic, Terminal j"
      e_bus2_bar = "e:bus2_bar", "Terminal j"
      e_iZoneBus = "e:iZoneBus", "Zone"
      e_cpZone = "e:cpZone", "Zone"
      e_iAreaBus = "e:iAreaBus", "Area"
      e_cpArea = "e:cpArea", "Area"
      e_cpBay = "e:cpBay", "Bay"
      e_on_off = "e:on_off", "Closed"
      e_isclosed = "e:isclosed", "Actual State"
      e_idetail = "e:idetail", "Detailed for calculation"
      e_pRating = "e:pRating", "IntThrating, Thermal Rating"
      e_nphase = "e:nphase", "No. of Phases"
      e_nneutral = "e:nneutral", "No. of Neutrals"
      e_aUsage = "e:aUsage", "Switch Type"
      e_iNeutInter = "e:iNeutInter", "Switch interrupts neutral wire"
      e_iExclRA = "e:iExclRA", "Excluded from Running Arrangement"
      e_Inom = "e:Inom", "kA, Nominal Current"
      e_ciScenarioType = "e:ciScenarioType", "Scenario Type"
      e_cpGrid = "e:cpGrid", "Grid"
      e_cpOwner = "e:cpOwner", "Owner"
      e_cpOperator = "e:cpOperator", "Operator"
      e_cpBranch = "e:cpBranch", "Branch"
      e_cpSubstat = "e:cpSubstat", "Substation"
      e_cpSite = "e:cpSite", "Site"
      e_cpMeteostat = "e:cpMeteostat", "Meteo Station"
      e_cpHeadFold = "e:cpHeadFold", "Head Folder"
      e_fold_id = "e:fold_id", "In Folder"
      e_ciOutaged = "e:ciOutaged", "Planned Outage"
      e_ciEnergized = "e:ciEnergized", "Energized"
      e_ciEarthed = "e:ciEarthed", "Earthed"
      e_cDisplayName = "e:cDisplayName", "Display Name"
      e_cpSupplyTransformer = "e:cpSupplyTransformer", "Supplying Transformer"
      e_cpSupplyTrfStation = "e:cpSupplyTrfStation", "Supplying Secondary Substation"
      e_cpSupplySubstation = "e:cpSupplySubstation", "Supplying Substation"
      r_typ_id = "r:typ_id", "TypSwitch, Type"
      r_bus1 = "r:bus1", "StaCubic, Terminal i"
      r_bus2 = "r:bus2", "StaCubic, Terminal j"
      r_cpZone = "r:cpZone", "Zone"
      r_cpArea = "r:cpArea", "Area"
      r_cpBay = "r:cpBay", "Bay"
      r_pRating = "r:pRating", "IntThrating, Thermal Rating"
      r_cpGrid = "r:cpGrid", "Grid"
      r_cpOwner = "r:cpOwner", "Owner"
      r_cpOperator = "r:cpOperator", "Operator"
      r_cpBranch = "r:cpBranch", "Branch"
      r_cpSubstat = "r:cpSubstat", "Substation"
      r_cpSite = "r:cpSite", "Site"
      r_cpMeteostat = "r:cpMeteostat", "Meteo Station"
      r_cpHeadFold = "r:cpHeadFold", "Head Folder"
      r_fold_id = "r:fold_id", "In Folder"
      r_cpSupplyTransformer = "r:cpSupplyTransformer", "Supplying Transformer"
      r_cpSupplyTrfStation = "r:cpSupplyTrfStation", "Supplying Secondary Substation"
      r_cpSupplySubstation = "r:cpSupplySubstation", "Supplying Substation"

    class ElmGenstat(Enum):
      _init_ = 'value __doc__'

      b_loc_name = "b:loc_name", "Name"
      b_iWindGen = "b:iWindGen", "Wind Generator"
      e_loc_name = "e:loc_name", "Name"
      e_bus1 = "e:bus1", "StaCubic, Terminal"
      e_bus1_bar = "e:bus1_bar", "Terminal"
      e_cpZone = "e:cpZone", "Zone"
      e_cpArea = "e:cpArea", "Area"
      e_outserv = "e:outserv", "Out of Service"
      e_phtech = "e:phtech", "Technology"
      e_cCategory = "e:cCategory", "Plant Category"
      e_cSubCategory = "e:cSubCategory", "Subcategory"
      e_ngnum = "e:ngnum", "parallel units"
      e_sgn = "e:sgn", "MVA, Rated Apparent Power"
      e_cosn = "e:cosn", "Rated Power Factor"
      e_c_pmod = "e:c_pmod", "Model"
      e_Inom = "e:Inom", "kA, Nominal Current"
      e_nneutral = "e:nneutral", "N-Connection"
      e_bus1n = "e:bus1n", "StaCubic, Neutral"
      e_bus1n_bar = "e:bus1n_bar", "Neutral"
      e_r0 = "e:r0", "p.u., Resistance, r0"
      e_x0 = "e:x0", "p.u., Reactance, x0"
      e_ciScenarioType = "e:ciScenarioType", "Scenario Type"
      r_bus1 = "r:bus1", "StaCubic, Terminal"
      r_cpZone = "r:cpZone", "Zone"
      r_cpArea = "r:cpArea", "Area"
      r_c_pmod = "r:c_pmod", "Model"
      r_bus1n = "r:bus1n", "StaCubic, Neutral"

    class ElmIac(Enum):
      _init_ = 'value __doc__'

      b_loc_name = "b:loc_name", "Name"
      e_loc_name = "e:loc_name", "Name"
      e_bus1 = "e:bus1", "StaCubic, Terminal"
      e_bus1_bar = "e:bus1_bar", "Terminal"
      e_cpZone = "e:cpZone", "Zone"
      e_cpArea = "e:cpArea", "Area"
      e_outserv = "e:outserv", "Out of Service"
      e_nphase = "e:nphase", "No. of Phases"
      e_Ir = "e:Ir", "A, Rated Current"
      e_c_pmod = "e:c_pmod", "Model"
      e_Inom = "e:Inom", "kA, Nominal Current"
      e_cpGrid = "e:cpGrid", "Grid"
      e_cpOwner = "e:cpOwner", "Owner"
      e_cpOperator = "e:cpOperator", "Operator"
      e_cpBranch = "e:cpBranch", "Branch"
      e_cpSubstat = "e:cpSubstat", "Substation"
      e_cpSite = "e:cpSite", "Site"
      e_cpMeteostat = "e:cpMeteostat", "Meteo Station"
      e_cpHeadFold = "e:cpHeadFold", "Head Folder"
      e_fold_id = "e:fold_id", "In Folder"
      e_ciOutaged = "e:ciOutaged", "Planned Outage"
      e_ciEnergized = "e:ciEnergized", "Energized"
      e_ciEarthed = "e:ciEarthed", "Earthed"
      e_cDisplayName = "e:cDisplayName", "Display Name"
      e_cpSupplyTransformer = "e:cpSupplyTransformer", "Supplying Transformer"
      e_cpSupplyTrfStation = "e:cpSupplyTrfStation", "Supplying Secondary Substation"
      e_cpSupplySubstation = "e:cpSupplySubstation", "Supplying Substation"
      r_bus1 = "r:bus1", "StaCubic, Terminal"
      r_cpZone = "r:cpZone", "Zone"
      r_cpArea = "r:cpArea", "Area"
      r_c_pmod = "r:c_pmod", "Model"
      r_cpGrid = "r:cpGrid", "Grid"
      r_cpOwner = "r:cpOwner", "Owner"
      r_cpOperator = "r:cpOperator", "Operator"
      r_cpBranch = "r:cpBranch", "Branch"
      r_cpSubstat = "r:cpSubstat", "Substation"
      r_cpSite = "r:cpSite", "Site"
      r_cpMeteostat = "r:cpMeteostat", "Meteo Station"
      r_cpHeadFold = "r:cpHeadFold", "Head Folder"
      r_fold_id = "r:fold_id", "In Folder"
      r_cpSupplyTransformer = "r:cpSupplyTransformer", "Supplying Transformer"
      r_cpSupplyTrfStation = "r:cpSupplyTrfStation", "Supplying Secondary Substation"
      r_cpSupplySubstation = "r:cpSupplySubstation", "Supplying Substation"

    class ElmLne(Enum):
      _init_ = 'value __doc__'

      b_loc_name = "b:loc_name", "Name"
      b_i_ohl = "b:i_ohl", "Overhead Line/Cable"
      b_dline = "b:dline", "km, Length"
      b_fline = "b:fline", "Derating Factor"
      e_loc_name = "e:loc_name", "Name"
      e_typ_id = "e:typ_id", "TypLne,TypTow,TypGeo,TypCabsys, Type"
      e_bus1 = "e:bus1", "StaCubic, Terminal i"
      e_bus1_bar = "e:bus1_bar", "Terminal i"
      e_bus2 = "e:bus2", "StaCubic, Terminal j"
      e_bus2_bar = "e:bus2_bar", "Terminal j"
      e_iZoneBus = "e:iZoneBus", "Zone"
      e_cpZone = "e:cpZone", "Zone"
      e_iAreaBus = "e:iAreaBus", "Area"
      e_cpArea = "e:cpArea", "Area"
      e_outserv = "e:outserv", "Out of Service"
      e_nlnum = "e:nlnum", "parallel Lines"
      e_pRating = "e:pRating", "IntThrating, Thermal Rating"
      e_pMeteo = "e:pMeteo", "ElmMeteostat, Meteo. Station"
      e_dline = "e:dline", "km, Length of Line"
      e_fline = "e:fline", "Derating Factor"
      e_inAir = "e:inAir", "Laying"
      e_txt_typ = "e:txt_typ", "Type of Line"
      e_i_dist = "e:i_dist", "Line Model"
      e_cubsecs = "e:cubsecs", "Sections/Line Loads/Compensation"
      e_pCondCir = "e:pCondCir", "TypCon, Type of phase conductors"
      e_pCondGnd = "e:pCondGnd", "TypCon, Type of earth conductors"
      e_sagCir = "e:sagCir", "m, Max.sag, phase conductors"
      e_sagGnd = "e:sagGnd", "m, Max.sag, earth conductors"
      e_rearth = "e:rearth", "Ohm*m, Earth resistivity"
      e_earthReturn = "e:earthReturn", "Earth return formula"
      e_ktrto = "e:ktrto", "Transposition"
      e_c_pCompi = "e:c_pCompi", "Compensation i"
      e_c_pCompj = "e:c_pCompj", "Compensation j"
      e_forTrainSim = "e:forTrainSim", "Line for train simulation"
      e_minResTrainDist = "e:minResTrainDist", "m, Min. resolution of train distances"
      e_nconTrains = "e:nconTrains", "No. of trains"
      e_conTrain = "e:conTrain", "Train"
      e_conTrainPos = "e:conTrainPos", "km, Train position"
      e_Inom_a = "e:Inom_a", "kA, Nominal Current (act.)"
      e_Inom = "e:Inom", "kA, Nominal Current"
      e_Z1 = "e:Z1", "Ohm, Pos. Seq. Impedance, Z1"
      e_phiz1 = "e:phiz1", "deg, Pos. Seq. Impedance, Angle"
      e_R1 = "e:R1", "Ohm, Pos. Seq. Resistance, R1"
      e_X1 = "e:X1", "Ohm, Pos. Seq. Reactance, X1"
      e_Ldc = "e:Ldc", "mH, Inductance, Ldc"
      e_R0 = "e:R0", "Ohm, Zero Seq. Resistance, R0"
      e_X0 = "e:X0", "Ohm, Zero Seq. Reactance, X0"
      e_Ice = "e:Ice", "A, Earth-Fault Current, Ice"
      e_k0 = "e:k0", "Earth Factor, Magnitude"
      e_phik0 = "e:phik0", "deg, Earth Factor, Angle"
      e_B1 = "e:B1", "uS, Susceptance B1"
      e_G1 = "e:G1", "uS, Conductance G1"
      e_C1 = "e:C1", "uF, Capacitance C1"
      e_B0 = "e:B0", "uS, Susceptance B0"
      e_G0 = "e:G0", "uS, Conductance G0"
      e_C0 = "e:C0", "uF, Capacitance C0"
      e_Unom = "e:Unom", "kV, Rated Voltage"
      e_rSbasepu = "e:rSbasepu", "p.u., r (Sbase)"
      e_xSbasepu = "e:xSbasepu", "p.u., x (Sbase)"
      e_bSbasepu = "e:bSbasepu", "p.u., b (Sbase)"
      e_r0Sbasepu = "e:r0Sbasepu", "p.u., r0 (Sbase)"
      e_x0Sbasepu = "e:x0Sbasepu", "p.u., x0 (Sbase)"
      e_b0Sbasepu = "e:b0Sbasepu", "p.u., b0 (Sbase)"
      e_r0mSbasepu = "e:r0mSbasepu", "p.u., r0m (Sbase)"
      e_x0mSbasepu = "e:x0mSbasepu", "p.u., x0m (Sbase)"
      e_ReRl = "e:ReRl", "Earth Factor, Re/Rl"
      e_XeXl = "e:XeXl", "Earth Factor, Xe/Xl"
      e_R0m = "e:R0m", "Ohm, R0m"
      e_X0m = "e:X0m", "Ohm, X0m"
      e_R1_tmax = "e:R1_tmax", "Ohm, Pos. Seq. Resistance at Max. Operating Temperature, R1 at tmax"
      e_c_ptow = "e:c_ptow", "Line Couplings"
      e_c_pcond = "e:c_pcond", "Conductor Type"
      e_c_pcoup = "e:c_pcoup", "Coupled Line"
      e_cpGrid = "e:cpGrid", "Grid"
      e_cpOwner = "e:cpOwner", "Owner"
      e_cpOperator = "e:cpOperator", "Operator"
      e_cpBranch = "e:cpBranch", "Branch"
      e_cpSubstat = "e:cpSubstat", "Substation"
      e_cpSite = "e:cpSite", "Site"
      e_cpMeteostat = "e:cpMeteostat", "Meteo Station"
      e_cpHeadFold = "e:cpHeadFold", "Head Folder"
      e_fold_id = "e:fold_id", "In Folder"
      e_ciOutaged = "e:ciOutaged", "Planned Outage"
      e_ciEnergized = "e:ciEnergized", "Energized"
      e_ciEarthed = "e:ciEarthed", "Earthed"
      e_cDisplayName = "e:cDisplayName", "Display Name"
      e_cpSupplyTransformer = "e:cpSupplyTransformer", "Supplying Transformer"
      e_cpSupplyTrfStation = "e:cpSupplyTrfStation", "Supplying Secondary Substation"
      e_cpSupplySubstation = "e:cpSupplySubstation", "Supplying Substation"
      t_loc_name = "t:loc_name", "Name"
      t_uline = "t:uline", "kV, Rated Voltage"
      t_sline = "t:sline", "kA, Rated Current"
      t_InomAir = "t:InomAir", "kA, Rated Current (in air)"
      t_cohl_ = "t:cohl_", "Cable / OHL"
      t_systp = "t:systp", "System Type"
      t_nlnph = "t:nlnph", "Phases"
      t_nneutral = "t:nneutral", "Number of Neutrals"
      t_frnom = "t:frnom", "Hz, Nominal Frequency"
      t_rline = "t:rline", "Ohm/km, AC-Resistance R'(20°C)"
      t_xline = "t:xline", "Ohm/km, Reactance X'"
      t_lline = "t:lline", "mH/km, Inductance L'"
      t_rline0 = "t:rline0", "Ohm/km, AC-Resistance R0'"
      t_xline0 = "t:xline0", "Ohm/km, Reactance X0'"
      t_lline0 = "t:lline0", "mH/km, Inductance L0'"
      t_rnline = "t:rnline", "Ohm/km, AC-Resistance Rn'"
      t_xnline = "t:xnline", "Ohm/km, Reactance Xn'"
      t_lnline = "t:lnline", "mH/km, Inductance Ln'"
      t_rpnline = "t:rpnline", "Ohm/km, AC-Resistance Rpn'"
      t_xpnline = "t:xpnline", "Ohm/km, Reactance Xpn'"
      t_lpnline = "t:lpnline", "mH/km, Inductance Lpn'"
      r_typ_id = "r:typ_id", "TypLne,TypTow,TypGeo,TypCabsys, Type"
      r_bus1 = "r:bus1", "StaCubic, Terminal i"
      r_bus2 = "r:bus2", "StaCubic, Terminal j"
      r_cpZone = "r:cpZone", "Zone"
      r_cpArea = "r:cpArea", "Area"
      r_pRating = "r:pRating", "IntThrating, Thermal Rating"
      r_pMeteo = "r:pMeteo", "ElmMeteostat, Meteo. Station"
      r_pCondCir = "r:pCondCir", "TypCon, Type of phase conductors"
      r_pCondGnd = "r:pCondGnd", "TypCon, Type of earth conductors"
      r_c_pCompi = "r:c_pCompi", "Compensation i"
      r_c_pCompj = "r:c_pCompj", "Compensation j"
      r_conTrain = "r:conTrain", "Train"
      r_c_ptow = "r:c_ptow", "Line Couplings"
      r_c_pcond = "r:c_pcond", "Conductor Type"
      r_c_pcoup = "r:c_pcoup", "Coupled Line"
      r_cpGrid = "r:cpGrid", "Grid"
      r_cpOwner = "r:cpOwner", "Owner"
      r_cpOperator = "r:cpOperator", "Operator"
      r_cpBranch = "r:cpBranch", "Branch"
      r_cpSubstat = "r:cpSubstat", "Substation"
      r_cpSite = "r:cpSite", "Site"
      r_cpMeteostat = "r:cpMeteostat", "Meteo Station"
      r_cpHeadFold = "r:cpHeadFold", "Head Folder"
      r_fold_id = "r:fold_id", "In Folder"
      r_cpSupplyTransformer = "r:cpSupplyTransformer", "Supplying Transformer"
      r_cpSupplyTrfStation = "r:cpSupplyTrfStation", "Supplying Secondary Substation"
      r_cpSupplySubstation = "r:cpSupplySubstation", "Supplying Substation"

    class ElmLod(Enum):
      _init_ = 'value __doc__'

      b_loc_name = "b:loc_name", "Name"
      e_loc_name = "e:loc_name", "Name"
      e_typ_id = "e:typ_id", "TypLod,TypLodind, Type"
      e_bus1 = "e:bus1", "StaCubic, Terminal"
      e_bus1_bar = "e:bus1_bar", "Terminal"
      e_cpZone = "e:cpZone", "Zone"
      e_cpArea = "e:cpArea", "Area"
      e_outserv = "e:outserv", "Out of Service"
      e_phtech = "e:phtech", "Technology"
      e_iLoadTrf = "e:iLoadTrf", "Consider Load Transformer"
      e_Strat = "e:Strat", "MVA, Rated Power"
      e_r0 = "e:r0", "p.u., r0"
      e_x0 = "e:x0", "p.u., x0"
      e_classif = "e:classif", "Load Classification"
      e_Inom = "e:Inom", "kA, Nominal Current"
      e_cpGrid = "e:cpGrid", "Grid"
      e_cpOwner = "e:cpOwner", "Owner"
      e_cpOperator = "e:cpOperator", "Operator"
      e_cpBranch = "e:cpBranch", "Branch"
      e_cpSubstat = "e:cpSubstat", "Substation"
      e_cpSite = "e:cpSite", "Site"
      e_cpMeteostat = "e:cpMeteostat", "Meteo Station"
      e_cpHeadFold = "e:cpHeadFold", "Head Folder"
      e_fold_id = "e:fold_id", "In Folder"
      e_ciOutaged = "e:ciOutaged", "Planned Outage"
      e_ciEnergized = "e:ciEnergized", "Energized"
      e_ciEarthed = "e:ciEarthed", "Earthed"
      e_cDisplayName = "e:cDisplayName", "Display Name"
      e_cpSupplyTransformer = "e:cpSupplyTransformer", "Supplying Transformer"
      e_cpSupplyTrfStation = "e:cpSupplyTrfStation", "Supplying Secondary Substation"
      e_cpSupplySubstation = "e:cpSupplySubstation", "Supplying Substation"
      r_typ_id = "r:typ_id", "TypLod,TypLodind, Type"
      r_bus1 = "r:bus1", "StaCubic, Terminal"
      r_cpZone = "r:cpZone", "Zone"
      r_cpArea = "r:cpArea", "Area"
      r_cpGrid = "r:cpGrid", "Grid"
      r_cpOwner = "r:cpOwner", "Owner"
      r_cpOperator = "r:cpOperator", "Operator"
      r_cpBranch = "r:cpBranch", "Branch"
      r_cpSubstat = "r:cpSubstat", "Substation"
      r_cpSite = "r:cpSite", "Site"
      r_cpMeteostat = "r:cpMeteostat", "Meteo Station"
      r_cpHeadFold = "r:cpHeadFold", "Head Folder"
      r_fold_id = "r:fold_id", "In Folder"
      r_cpSupplyTransformer = "r:cpSupplyTransformer", "Supplying Transformer"
      r_cpSupplyTrfStation = "r:cpSupplyTrfStation", "Supplying Secondary Substation"
      r_cpSupplySubstation = "r:cpSupplySubstation", "Supplying Substation"

    class ElmLodlv(Enum):
      _init_ = 'value __doc__'

      b_loc_name = "b:loc_name", "Name"
      e_loc_name = "e:loc_name", "Name"
      e_typ_id = "e:typ_id", "TypLodlv,TypLodlvcons, Type"
      e_bus1 = "e:bus1", "StaCubic, Terminal"
      e_bus1_bar = "e:bus1_bar", "Terminal"
      e_cpZone = "e:cpZone", "Zone"
      e_cpArea = "e:cpArea", "Area"
      e_outserv = "e:outserv", "Out of Service"
      e_phtech = "e:phtech", "Technology"
      e_NrCust = "e:NrCust", "Number of Customers"
      e_classif = "e:classif", "Load Classification"
      e_cpGrid = "e:cpGrid", "Grid"
      e_cpOwner = "e:cpOwner", "Owner"
      e_cpOperator = "e:cpOperator", "Operator"
      e_cpBranch = "e:cpBranch", "Branch"
      e_cpSubstat = "e:cpSubstat", "Substation"
      e_cpSite = "e:cpSite", "Site"
      e_cpMeteostat = "e:cpMeteostat", "Meteo Station"
      e_cpHeadFold = "e:cpHeadFold", "Head Folder"
      e_fold_id = "e:fold_id", "In Folder"
      e_ciOutaged = "e:ciOutaged", "Planned Outage"
      e_ciEnergized = "e:ciEnergized", "Energized"
      e_ciEarthed = "e:ciEarthed", "Earthed"
      e_cDisplayName = "e:cDisplayName", "Display Name"
      e_cpSupplyTransformer = "e:cpSupplyTransformer", "Supplying Transformer"
      e_cpSupplyTrfStation = "e:cpSupplyTrfStation", "Supplying Secondary Substation"
      e_cpSupplySubstation = "e:cpSupplySubstation", "Supplying Substation"
      r_typ_id = "r:typ_id", "TypLodlv,TypLodlvcons, Type"
      r_bus1 = "r:bus1", "StaCubic, Terminal"
      r_cpZone = "r:cpZone", "Zone"
      r_cpArea = "r:cpArea", "Area"
      r_cpGrid = "r:cpGrid", "Grid"
      r_cpOwner = "r:cpOwner", "Owner"
      r_cpOperator = "r:cpOperator", "Operator"
      r_cpBranch = "r:cpBranch", "Branch"
      r_cpSubstat = "r:cpSubstat", "Substation"
      r_cpSite = "r:cpSite", "Site"
      r_cpMeteostat = "r:cpMeteostat", "Meteo Station"
      r_cpHeadFold = "r:cpHeadFold", "Head Folder"
      r_fold_id = "r:fold_id", "In Folder"
      r_cpSupplyTransformer = "r:cpSupplyTransformer", "Supplying Transformer"
      r_cpSupplyTrfStation = "r:cpSupplyTrfStation", "Supplying Secondary Substation"
      r_cpSupplySubstation = "r:cpSupplySubstation", "Supplying Substation"

    class ElmLodmv(Enum):
      _init_ = 'value __doc__'

      b_loc_name = "b:loc_name", "Name"
      e_loc_name = "e:loc_name", "Name"
      e_typ_id = "e:typ_id", "TypDistrf, Type"
      e_bus1 = "e:bus1", "StaCubic, Terminal"
      e_bus1_bar = "e:bus1_bar", "Terminal"
      e_cpZone = "e:cpZone", "Zone"
      e_cpArea = "e:cpArea", "Area"
      e_outserv = "e:outserv", "Out of Service"
      e_phtech = "e:phtech", "Technology"
      e_classif = "e:classif", "Load Classification"
      e_Inom = "e:Inom", "kA, Nominal Current"
      e_cpGrid = "e:cpGrid", "Grid"
      e_cpOwner = "e:cpOwner", "Owner"
      e_cpOperator = "e:cpOperator", "Operator"
      e_cpBranch = "e:cpBranch", "Branch"
      e_cpSubstat = "e:cpSubstat", "Substation"
      e_cpSite = "e:cpSite", "Site"
      e_cpMeteostat = "e:cpMeteostat", "Meteo Station"
      e_cpHeadFold = "e:cpHeadFold", "Head Folder"
      e_fold_id = "e:fold_id", "In Folder"
      e_ciOutaged = "e:ciOutaged", "Planned Outage"
      e_ciEnergized = "e:ciEnergized", "Energized"
      e_ciEarthed = "e:ciEarthed", "Earthed"
      e_cDisplayName = "e:cDisplayName", "Display Name"
      e_cpSupplyTransformer = "e:cpSupplyTransformer", "Supplying Transformer"
      e_cpSupplyTrfStation = "e:cpSupplyTrfStation", "Supplying Secondary Substation"
      e_cpSupplySubstation = "e:cpSupplySubstation", "Supplying Substation"
      r_typ_id = "r:typ_id", "TypDistrf, Type"
      r_bus1 = "r:bus1", "StaCubic, Terminal"
      r_cpZone = "r:cpZone", "Zone"
      r_cpArea = "r:cpArea", "Area"
      r_cpGrid = "r:cpGrid", "Grid"
      r_cpOwner = "r:cpOwner", "Owner"
      r_cpOperator = "r:cpOperator", "Operator"
      r_cpBranch = "r:cpBranch", "Branch"
      r_cpSubstat = "r:cpSubstat", "Substation"
      r_cpSite = "r:cpSite", "Site"
      r_cpMeteostat = "r:cpMeteostat", "Meteo Station"
      r_cpHeadFold = "r:cpHeadFold", "Head Folder"
      r_fold_id = "r:fold_id", "In Folder"
      r_cpSupplyTransformer = "r:cpSupplyTransformer", "Supplying Transformer"
      r_cpSupplyTrfStation = "r:cpSupplyTrfStation", "Supplying Secondary Substation"
      r_cpSupplySubstation = "r:cpSupplySubstation", "Supplying Substation"

    class ElmPvsys(Enum):
      _init_ = 'value __doc__'

      b_loc_name = "b:loc_name", "Name"
      b_iWindGen = "b:iWindGen", "Wind Generator"
      e_loc_name = "e:loc_name", "Name"
      e_typ_id = "e:typ_id", "TypPvpanel, Type"
      e_bus1 = "e:bus1", "StaCubic, Terminal"
      e_bus1_bar = "e:bus1_bar", "Terminal"
      e_cpZone = "e:cpZone", "Zone"
      e_cpArea = "e:cpArea", "Area"
      e_outserv = "e:outserv", "Out of Service"
      e_mode_pgi = "e:mode_pgi", "Model"
      e_phtech = "e:phtech", "Technology"
      e_ngnum = "e:ngnum", "Parallel Inverters"
      e_npnum = "e:npnum", "Panels per Inverter"
      e_sgn = "e:sgn", "kVA, Rated Apparent Power"
      e_cosn = "e:cosn", "Rated Power Factor"
      e_c_pmod = "e:c_pmod", "Model"
      e_Inom = "e:Inom", "kA, Nominal Current"
      e_cGPSlat = "e:cGPSlat", "deg, Latitude"
      e_cGPSlon = "e:cGPSlon", "deg, Longitude"
      e_timezone = "e:timezone", "Time Zone (Offset)"
      e_mount = "e:mount", "Mounting System"
      e_orient = "e:orient", "deg, Orientation Angle"
      e_tilt = "e:tilt", "deg, Tilt Angle"
      e_inveff = "e:inveff", "%, Efficiency Factor"
      e_nneutral = "e:nneutral", "N-Connection"
      e_bus1n = "e:bus1n", "StaCubic, Neutral"
      e_bus1n_bar = "e:bus1n_bar", "Neutral"
      e_r0 = "e:r0", "p.u., Resistance, r0"
      e_x0 = "e:x0", "p.u., Reactance, x0"
      r_typ_id = "r:typ_id", "TypPvpanel, Type"
      r_bus1 = "r:bus1", "StaCubic, Terminal"
      r_cpZone = "r:cpZone", "Zone"
      r_cpArea = "r:cpArea", "Area"
      r_c_pmod = "r:c_pmod", "Model"
      r_bus1n = "r:bus1n", "StaCubic, Neutral"

    class ElmSecctrl(Enum):
      _init_ = 'value __doc__'

      b_loc_name = "b:loc_name", "Name"
      b_n_sym = "b:n_sym", "Number of Machines"
      b_i_modus = "b:i_modus", "Control Mode 0=phi 1=p"
      b_i_net = "b:i_net", "Control Area Exchange"
      e_loc_name = "e:loc_name", "Name"
      e_outserv = "e:outserv", "Out of Service"
      e_cpGrid = "e:cpGrid", "Grid"
      e_cpOwner = "e:cpOwner", "Owner"
      e_cpOperator = "e:cpOperator", "Operator"
      e_cpBranch = "e:cpBranch", "Branch"
      e_cpSubstat = "e:cpSubstat", "Substation"
      e_cpSite = "e:cpSite", "Site"
      e_cpHeadFold = "e:cpHeadFold", "Head Folder"
      e_fold_id = "e:fold_id", "In Folder"
      e_cDisplayName = "e:cDisplayName", "Display Name"
      r_cpGrid = "r:cpGrid", "Grid"
      r_cpOwner = "r:cpOwner", "Owner"
      r_cpOperator = "r:cpOperator", "Operator"
      r_cpBranch = "r:cpBranch", "Branch"
      r_cpSubstat = "r:cpSubstat", "Substation"
      r_cpSite = "r:cpSite", "Site"
      r_cpHeadFold = "r:cpHeadFold", "Head Folder"
      r_fold_id = "r:fold_id", "In Folder"

    class ElmShnt(Enum):
      _init_ = 'value __doc__'

      b_loc_name = "b:loc_name", "Name"
      e_loc_name = "e:loc_name", "Name"
      e_bus1 = "e:bus1", "StaCubic, Terminal"
      e_bus1_bar = "e:bus1_bar", "Terminal"
      e_cpZone = "e:cpZone", "Zone"
      e_cpArea = "e:cpArea", "Area"
      e_outserv = "e:outserv", "Out of Service"
      e_systp = "e:systp", "System Type"
      e_ctech = "e:ctech", "Technology"
      e_ushnm = "e:ushnm", "kV, Rated Voltage"
      e_shtype = "e:shtype", "Shunt Type"
      e_mode_inp = "e:mode_inp", "Input Mode"
      e_ncapx = "e:ncapx", "Max. No. of Steps"
      e_ncapa = "e:ncapa", "Act.No. of Step"
      e_Qmax = "e:Qmax", "Mvar, Max. Rated Reactive Power"
      e_Qact = "e:Qact", "Mvar, Actual Reactive Power"
      e_iTaps = "e:iTaps", "According to Measurement Report"
      e_qcapn = "e:qcapn", "Mvar, Rated Reactive Power, C"
      e_cucap = "e:cucap", "A, Rated Current, C"
      e_qtotn = "e:qtotn", "Mvar, Rated Reactive Power, L-C"
      e_cutot = "e:cutot", "A, Rated Current, L-C"
      e_qrean = "e:qrean", "Mvar, Rated Reactive Power, L"
      e_curea = "e:curea", "A, Rated Current, L"
      e_pgrad = "e:pgrad", "%, Degree"
      e_fres = "e:fres", "Hz, Resonant Frequency"
      e_nres = "e:nres", "Tuning Order"
      e_grea = "e:grea", "Quality Factor (at fn)"
      e_greaf0 = "e:greaf0", "Quality Factor (at fr)"
      e_tandc = "e:tandc", "Loss Factor, tan(delta)"
      e_ccap = "e:ccap", "uF, Capacitance"
      e_bcap = "e:bcap", "uS, Susceptance"
      e_c1 = "e:c1", "uF, Capacitance C1"
      e_c2 = "e:c2", "uF, Capacitance C2"
      e_xrea = "e:xrea", "Ohm, Reactance"
      e_rlrea = "e:rlrea", "mH, Inductance"
      e_rrea = "e:rrea", "Ohm, Resistance"
      e_rpara = "e:rpara", "Ohm, Parallel Resistance"
      e_gparac = "e:gparac", "uS, Parallel Conductance"
      e_iintgnd = "e:iintgnd", "N-Connection"
      e_bus1n = "e:bus1n", "StaCubic, Neutral"
      e_bus1n_bar = "e:bus1n_bar", "Neutral"
      e_Bg = "e:Bg", "nS, Susceptance to Ground"
      e_cgnd = "e:cgnd", "Star Point"
      e_Re = "e:Re", "Ohm, Resistance, Re"
      e_Xe = "e:Xe", "Ohm, Reactance, Xe"
      e_iZeConfig = "e:iZeConfig", "Configuration"
      e_R0toR1 = "e:R0toR1", "R0/R1 ratio"
      e_X0toX1 = "e:X0toX1", "X0/X1 ratio"
      e_G0toG1 = "e:G0toG1", "G0/G1 ratio"
      e_B0toB1 = "e:B0toB1", "B0/B1 ratio"
      e_cpGrid = "e:cpGrid", "Grid"
      e_cpOwner = "e:cpOwner", "Owner"
      e_cpOperator = "e:cpOperator", "Operator"
      e_cpBranch = "e:cpBranch", "Branch"
      e_cpSubstat = "e:cpSubstat", "Substation"
      e_cpSite = "e:cpSite", "Site"
      e_cpMeteostat = "e:cpMeteostat", "Meteo Station"
      e_cpHeadFold = "e:cpHeadFold", "Head Folder"
      e_fold_id = "e:fold_id", "In Folder"
      e_ciOutaged = "e:ciOutaged", "Planned Outage"
      e_ciEnergized = "e:ciEnergized", "Energized"
      e_ciEarthed = "e:ciEarthed", "Earthed"
      e_cDisplayName = "e:cDisplayName", "Display Name"
      e_cpSupplyTransformer = "e:cpSupplyTransformer", "Supplying Transformer"
      e_cpSupplyTrfStation = "e:cpSupplyTrfStation", "Supplying Secondary Substation"
      e_cpSupplySubstation = "e:cpSupplySubstation", "Supplying Substation"
      e_mTaps = "e:mTaps", "Measurement Report"
      r_bus1 = "r:bus1", "StaCubic, Terminal"
      r_cpZone = "r:cpZone", "Zone"
      r_cpArea = "r:cpArea", "Area"
      r_bus1n = "r:bus1n", "StaCubic, Neutral"
      r_cpGrid = "r:cpGrid", "Grid"
      r_cpOwner = "r:cpOwner", "Owner"
      r_cpOperator = "r:cpOperator", "Operator"
      r_cpBranch = "r:cpBranch", "Branch"
      r_cpSubstat = "r:cpSubstat", "Substation"
      r_cpSite = "r:cpSite", "Site"
      r_cpMeteostat = "r:cpMeteostat", "Meteo Station"
      r_cpHeadFold = "r:cpHeadFold", "Head Folder"
      r_fold_id = "r:fold_id", "In Folder"
      r_cpSupplyTransformer = "r:cpSupplyTransformer", "Supplying Transformer"
      r_cpSupplyTrfStation = "r:cpSupplyTrfStation", "Supplying Secondary Substation"
      r_cpSupplySubstation = "r:cpSupplySubstation", "Supplying Substation"

    class ElmSind(Enum):
      _init_ = 'value __doc__'

      b_loc_name = "b:loc_name", "Name"
      e_loc_name = "e:loc_name", "Name"
      e_bus1 = "e:bus1", "StaCubic, Terminal i"
      e_bus1_bar = "e:bus1_bar", "Terminal i"
      e_bus2 = "e:bus2", "StaCubic, Terminal j"
      e_bus2_bar = "e:bus2_bar", "Terminal j"
      e_iZoneBus = "e:iZoneBus", "Zone"
      e_cpZone = "e:cpZone", "Zone"
      e_iAreaBus = "e:iAreaBus", "Area"
      e_cpArea = "e:cpArea", "Area"
      e_outserv = "e:outserv", "Out of Service"
      e_ucn = "e:ucn", "kV, Rated Voltage"
      e_Sn = "e:Sn", "MVA, Rated Power"
      e_Curn = "e:Curn", "kA, Rated Current"
      e_pRating = "e:pRating", "IntThrating, Thermal Rating"
      e_systp = "e:systp", "System Type"
      e_nphases = "e:nphases", "Phases"
      e_Inom = "e:Inom", "kA, Nominal Current"
      e_cpGrid = "e:cpGrid", "Grid"
      e_cpOwner = "e:cpOwner", "Owner"
      e_cpOperator = "e:cpOperator", "Operator"
      e_cpBranch = "e:cpBranch", "Branch"
      e_cpSubstat = "e:cpSubstat", "Substation"
      e_cpSite = "e:cpSite", "Site"
      e_cpMeteostat = "e:cpMeteostat", "Meteo Station"
      e_cpHeadFold = "e:cpHeadFold", "Head Folder"
      e_fold_id = "e:fold_id", "In Folder"
      e_ciOutaged = "e:ciOutaged", "Planned Outage"
      e_ciEnergized = "e:ciEnergized", "Energized"
      e_ciEarthed = "e:ciEarthed", "Earthed"
      e_cDisplayName = "e:cDisplayName", "Display Name"
      e_cpSupplyTransformer = "e:cpSupplyTransformer", "Supplying Transformer"
      e_cpSupplyTrfStation = "e:cpSupplyTrfStation", "Supplying Secondary Substation"
      e_cpSupplySubstation = "e:cpSupplySubstation", "Supplying Substation"
      r_bus1 = "r:bus1", "StaCubic, Terminal i"
      r_bus2 = "r:bus2", "StaCubic, Terminal j"
      r_cpZone = "r:cpZone", "Zone"
      r_cpArea = "r:cpArea", "Area"
      r_pRating = "r:pRating", "IntThrating, Thermal Rating"
      r_cpGrid = "r:cpGrid", "Grid"
      r_cpOwner = "r:cpOwner", "Owner"
      r_cpOperator = "r:cpOperator", "Operator"
      r_cpBranch = "r:cpBranch", "Branch"
      r_cpSubstat = "r:cpSubstat", "Substation"
      r_cpSite = "r:cpSite", "Site"
      r_cpMeteostat = "r:cpMeteostat", "Meteo Station"
      r_cpHeadFold = "r:cpHeadFold", "Head Folder"
      r_fold_id = "r:fold_id", "In Folder"
      r_cpSupplyTransformer = "r:cpSupplyTransformer", "Supplying Transformer"
      r_cpSupplyTrfStation = "r:cpSupplyTrfStation", "Supplying Secondary Substation"
      r_cpSupplySubstation = "r:cpSupplySubstation", "Supplying Substation"

    class ElmStactrl(Enum):
      _init_ = 'value __doc__'

      b_loc_name = "b:loc_name", "Name"
      b_n_q = "b:n_q", "Number of Reactive Power Outputs"
      b_n_sym = "b:n_sym", "Number of Machines"
      b_i_ctrl = "b:i_ctrl", "Load Flow Control"
      e_loc_name = "e:loc_name", "Name"
      e_outserv = "e:outserv", "Out of Service"
      e_psym = "e:psym", "ElmSym,ElmGenstat,ElmPvsys,ElmSvs,ElmAsm,ElmVsc*,ElmXnet, Machines"
      e_cconn = "e:cconn", "Connected"
      e_cpGrid = "e:cpGrid", "Grid"
      e_cpOwner = "e:cpOwner", "Owner"
      e_cpOperator = "e:cpOperator", "Operator"
      e_cpBranch = "e:cpBranch", "Branch"
      e_cpSubstat = "e:cpSubstat", "Substation"
      e_cpSite = "e:cpSite", "Site"
      e_cpHeadFold = "e:cpHeadFold", "Head Folder"
      e_fold_id = "e:fold_id", "In Folder"
      e_cDisplayName = "e:cDisplayName", "Display Name"
      r_psym = "r:psym", "ElmSym,ElmGenstat,ElmPvsys,ElmSvs,ElmAsm,ElmVsc*,ElmXnet, Machines"
      r_cpGrid = "r:cpGrid", "Grid"
      r_cpOwner = "r:cpOwner", "Owner"
      r_cpOperator = "r:cpOperator", "Operator"
      r_cpBranch = "r:cpBranch", "Branch"
      r_cpSubstat = "r:cpSubstat", "Substation"
      r_cpSite = "r:cpSite", "Site"
      r_cpHeadFold = "r:cpHeadFold", "Head Folder"
      r_fold_id = "r:fold_id", "In Folder"

    class ElmSym(Enum):
      _init_ = 'value __doc__'

      b_loc_name = "b:loc_name", "Name"
      b_Kf = "b:Kf", "Correction Factor (Short-Circuit)"
      b_ipsu = "b:ipsu", "Power Station Unit"
      b_i_spin = "b:i_spin", "Flag"
      b_i_mot = "b:i_mot", "Motor"
      b_iWindGen = "b:iWindGen", "Wind Generator"
      e_loc_name = "e:loc_name", "Name"
      e_typ_id = "e:typ_id", "TypSym, Type"
      e_bus1 = "e:bus1", "StaCubic, Terminal"
      e_bus1_bar = "e:bus1_bar", "Terminal"
      e_cpZone = "e:cpZone", "Zone"
      e_cpArea = "e:cpArea", "Area"
      e_outserv = "e:outserv", "Out of Service"
      e_phtech = "e:phtech", "Technology"
      e_ngnum = "e:ngnum", "parallel Machines"
      e_i_mot = "e:i_mot", "Generator/Motor"
      e_cCategory = "e:cCategory", "Plant Category"
      e_cSubCategory = "e:cSubCategory", "Subcategory"
      e_c_pmod = "e:c_pmod", "Model"
      e_Inom = "e:Inom", "kA, Nominal Current"
      e_iintgnd = "e:iintgnd", "N-Connection"
      e_bus1n = "e:bus1n", "StaCubic, Neutral"
      e_bus1n_bar = "e:bus1n_bar", "Neutral"
      e_cgnd = "e:cgnd", "Star Point"
      e_resy = "e:resy", "Ohm, Resistance, Re"
      e_xesy = "e:xesy", "Ohm, Reactance, Xe"
      e_cpGrid = "e:cpGrid", "Grid"
      e_cpOwner = "e:cpOwner", "Owner"
      e_cpOperator = "e:cpOperator", "Operator"
      e_cpBranch = "e:cpBranch", "Branch"
      e_cpSubstat = "e:cpSubstat", "Substation"
      e_cpSite = "e:cpSite", "Site"
      e_cpMeteostat = "e:cpMeteostat", "Meteo Station"
      e_cpHeadFold = "e:cpHeadFold", "Head Folder"
      e_fold_id = "e:fold_id", "In Folder"
      e_ciOutaged = "e:ciOutaged", "Planned Outage"
      e_ciEnergized = "e:ciEnergized", "Energized"
      e_ciEarthed = "e:ciEarthed", "Earthed"
      e_cDisplayName = "e:cDisplayName", "Display Name"
      e_cpSupplyTransformer = "e:cpSupplyTransformer", "Supplying Transformer"
      e_cpSupplyTrfStation = "e:cpSupplyTrfStation", "Supplying Secondary Substation"
      e_cpSupplySubstation = "e:cpSupplySubstation", "Supplying Substation"
      t_loc_name = "t:loc_name", "Name"
      t_sgn = "t:sgn", "MVA, Rated apparent power"
      t_ugn = "t:ugn", "kV, Rated voltage"
      t_cosn = "t:cosn", "Rated power factor"
      t_nphase = "t:nphase", "No. of phases"
      t_nslty = "t:nslty", "Connection"
      r_typ_id = "r:typ_id", "TypSym, Type"
      r_bus1 = "r:bus1", "StaCubic, Terminal"
      r_cpZone = "r:cpZone", "Zone"
      r_cpArea = "r:cpArea", "Area"
      r_c_pmod = "r:c_pmod", "Model"
      r_bus1n = "r:bus1n", "StaCubic, Neutral"
      r_cpGrid = "r:cpGrid", "Grid"
      r_cpOwner = "r:cpOwner", "Owner"
      r_cpOperator = "r:cpOperator", "Operator"
      r_cpBranch = "r:cpBranch", "Branch"
      r_cpSubstat = "r:cpSubstat", "Substation"
      r_cpSite = "r:cpSite", "Site"
      r_cpMeteostat = "r:cpMeteostat", "Meteo Station"
      r_cpHeadFold = "r:cpHeadFold", "Head Folder"
      r_fold_id = "r:fold_id", "In Folder"
      r_cpSupplyTransformer = "r:cpSupplyTransformer", "Supplying Transformer"
      r_cpSupplyTrfStation = "r:cpSupplyTrfStation", "Supplying Secondary Substation"
      r_cpSupplySubstation = "r:cpSupplySubstation", "Supplying Substation"

    class ElmTerm(Enum):
      _init_ = 'value __doc__'

      e_loc_name = "e:loc_name", "Name"
      e_typ_id = "e:typ_id", "TypBar, Type"
      e_cpZone = "e:cpZone", "ElmZone, Zone"
      e_cpArea = "e:cpArea", "ElmArea, Area"
      e_cpSubstat = "e:cpSubstat", "Substation"
      e_cpBranch = "e:cpBranch", "Branch"
      e_outserv = "e:outserv", "Out of Service"
      e_systype = "e:systype", "System Type"
      e_iUsage = "e:iUsage", "Usage"
      e_phtech = "e:phtech", "Phase Technology"
      e_uknom = "e:uknom", "kV, Line-Line"
      e_unknom = "e:unknom", "kV, Line-Ground"
      e_iminus = "e:iminus", "DC-Polarity"
      e_cPosLne = "e:cPosLne", "km, Position on Line"
      e_iEarth = "e:iEarth", "Earthed"
      e_cStatName = "e:cStatName", "Station/Name"
      e_NodeName = "e:NodeName", "Node Name"
      e_UcteNodeName = "e:UcteNodeName", "Ucte Node Name"
      e_dpl1 = "e:dpl1", "dpl1"
      e_dpl2 = "e:dpl2", "dpl2"
      e_dpl3 = "e:dpl3", "dpl3"
      e_dpl4 = "e:dpl4", "dpl4"
      e_dpl5 = "e:dpl5", "dpl5"
      e_cpGrid = "e:cpGrid", "Grid"
      e_cpOwner = "e:cpOwner", "Owner"
      e_cpOperator = "e:cpOperator", "Operator"
      e_cpSite = "e:cpSite", "Site"
      e_cpMeteostat = "e:cpMeteostat", "Meteo Station"
      e_cpHeadFold = "e:cpHeadFold", "Head Folder"
      e_fold_id = "e:fold_id", "In Folder"
      e_ciOutaged = "e:ciOutaged", "Planned Outage"
      e_ciEnergized = "e:ciEnergized", "Energized"
      e_ciEarthed = "e:ciEarthed", "Earthed"
      e_cDisplayName = "e:cDisplayName", "Display Name"
      e_cpSupplyTransformer = "e:cpSupplyTransformer", "Supplying Transformer"
      e_cpSupplyTrfStation = "e:cpSupplyTrfStation", "Supplying Secondary Substation"
      e_cpSupplySubstation = "e:cpSupplySubstation", "Supplying Substation"
      r_typ_id = "r:typ_id", "TypBar, Type"
      r_cpZone = "r:cpZone", "ElmZone, Zone"
      r_cpArea = "r:cpArea", "ElmArea, Area"
      r_cpSubstat = "r:cpSubstat", "Substation"
      r_cpBranch = "r:cpBranch", "Branch"
      r_cpGrid = "r:cpGrid", "Grid"
      r_cpOwner = "r:cpOwner", "Owner"
      r_cpOperator = "r:cpOperator", "Operator"
      r_cpSite = "r:cpSite", "Site"
      r_cpMeteostat = "r:cpMeteostat", "Meteo Station"
      r_cpHeadFold = "r:cpHeadFold", "Head Folder"
      r_fold_id = "r:fold_id", "In Folder"
      r_cpSupplyTransformer = "r:cpSupplyTransformer", "Supplying Transformer"
      r_cpSupplyTrfStation = "r:cpSupplyTrfStation", "Supplying Secondary Substation"
      r_cpSupplySubstation = "r:cpSupplySubstation", "Supplying Substation"

    class ElmTr2(Enum):
      _init_ = 'value __doc__'

      b_loc_name = "b:loc_name", "Name"
      b_Kf = "b:Kf", "Correction Factor (Short-Circuit)"
      b_Ugn = "b:Ugn", "Rated Voltage of Generator (for PSU)"
      b_ipsu = "b:ipsu", "Power Station Unit"
      b_nt2ag = "b:nt2ag", "Rel.Phase Displacement"
      b_ntrcn = "b:ntrcn", "Automatic Tap Changer on/off"
      b_i_cont = "b:i_cont", "Continuous Tap"
      b_iExtCtrl = "b:iExtCtrl", "External Controlled"
      b_isat = "b:isat", "Saturated Model"
      e_loc_name = "e:loc_name", "Name"
      e_typ_id = "e:typ_id", "TypTr2, Type"
      e_bushv = "e:bushv", "StaCubic, HV-Side"
      e_bushv_bar = "e:bushv_bar", "HV-Side"
      e_buslv = "e:buslv", "StaCubic, LV-Side"
      e_buslv_bar = "e:buslv_bar", "LV-Side"
      e_iZoneBus = "e:iZoneBus", "Zone"
      e_cpZone = "e:cpZone", "Zone"
      e_iAreaBus = "e:iAreaBus", "Area"
      e_cpArea = "e:cpArea", "Area"
      e_outserv = "e:outserv", "Out of Service"
      e_ntnum = "e:ntnum", "parallel Transformers"
      e_pRating = "e:pRating", "IntThrating, Thermal Rating"
      e_pMeteo = "e:pMeteo", "ElmMeteostat, Meteo. Station"
      e_ratfac = "e:ratfac", "Rating Factor"
      e_Snom_a = "e:Snom_a", "MVA, Nominal Power (act.)"
      e_Snom = "e:Snom", "MVA, Nominal Power"
      e_i_auto = "e:i_auto", "Auto Transformer"
      e_cneutcon = "e:cneutcon", "N-Connection"
      e_bushvn = "e:bushvn", "StaCubic, HV-Neutral"
      e_bushvn_bar = "e:bushvn_bar", "HV-Neutral"
      e_buslvn = "e:buslvn", "StaCubic, LV-Neutral"
      e_buslvn_bar = "e:buslvn_bar", "LV-Neutral"
      e_iintgnd = "e:iintgnd", "Neutral Connection"
      e_i_eahv = "e:i_eahv", "HV-side, phase 2 internally grounded"
      e_i_hvcon = "e:i_hvcon", "HV-side, phase 2 connected"
      e_cgnd_h = "e:cgnd_h", "Star Point"
      e_cpeter_h = "e:cpeter_h", "Petersen Coil"
      e_re0tr_h = "e:re0tr_h", "Ohm, Resistance, Re"
      e_xe0tr_h = "e:xe0tr_h", "Ohm, Reactance, Xe"
      e_i_ealv = "e:i_ealv", "LV-side, phase 2 internally grounded"
      e_i_lvcon = "e:i_lvcon", "LV-side, phase 2 connected"
      e_cgnd_l = "e:cgnd_l", "Star Point"
      e_cpeter_l = "e:cpeter_l", "Petersen Coil"
      e_re0tr_l = "e:re0tr_l", "Ohm, Resistance, Re"
      e_xe0tr_l = "e:xe0tr_l", "Ohm, Reactance, Xe"
      e_rSbasepu = "e:rSbasepu", "p.u., r (Sbase)"
      e_xSbasepu = "e:xSbasepu", "p.u., x (Sbase)"
      e_bSbasepu = "e:bSbasepu", "p.u., b (Sbase)"
      e_r0Sbasepu = "e:r0Sbasepu", "p.u., r0 (Sbase)"
      e_x0Sbasepu = "e:x0Sbasepu", "p.u., x0 (Sbase)"
      e_Inom_h_a = "e:Inom_h_a", "kA, HV-Side, Nominal Current (act.)"
      e_Inom_l_a = "e:Inom_l_a", "kA, LV-Side, Nominal Current"
      e_Inom_h = "e:Inom_h", "kA, HV-Side, Nominal Current"
      e_Inom_l = "e:Inom_l", "kA, LV-Side, Nominal Current"
      e_cpGrid = "e:cpGrid", "Grid"
      e_cpOwner = "e:cpOwner", "Owner"
      e_cpOperator = "e:cpOperator", "Operator"
      e_cpBranch = "e:cpBranch", "Branch"
      e_cpSubstat = "e:cpSubstat", "Substation"
      e_cpSite = "e:cpSite", "Site"
      e_cpMeteostat = "e:cpMeteostat", "Meteo Station"
      e_cpHeadFold = "e:cpHeadFold", "Head Folder"
      e_fold_id = "e:fold_id", "In Folder"
      e_ciOutaged = "e:ciOutaged", "Planned Outage"
      e_ciEnergized = "e:ciEnergized", "Energized"
      e_ciEarthed = "e:ciEarthed", "Earthed"
      e_cDisplayName = "e:cDisplayName", "Display Name"
      e_cpSupplyTransformer = "e:cpSupplyTransformer", "Supplying Transformer"
      e_cpSupplyTrfStation = "e:cpSupplyTrfStation", "Supplying Secondary Substation"
      e_cpSupplySubstation = "e:cpSupplySubstation", "Supplying Substation"
      t_loc_name = "t:loc_name", "Name"
      t_nt2ph = "t:nt2ph", "Technology"
      t_strn = "t:strn", "MVA, Rated Power"
      t_frnom = "t:frnom", "Hz, Nominal Frequency"
      t_utrn_h = "t:utrn_h", "kV, HV-Side"
      t_utrn_l = "t:utrn_l", "kV, LV-Side"
      t_uktr = "t:uktr", "%, Short-Circuit Voltage uk"
      t_pcutr = "t:pcutr", "kW, Copper Losses"
      t_uktrr = "t:uktrr", "%, SHC-Voltage (Re(uk)) ukr"
      t_xtor = "t:xtor", "Ratio X/R"
      t_x1pu = "t:x1pu", "p.u., Reactance x1"
      t_r1pu = "t:r1pu", "p.u., Resistance r1"
      t_tr2cn_h = "t:tr2cn_h", "HV-Side"
      t_tr2cn_l = "t:tr2cn_l", "LV-Side"
      t_iIntDelta = "t:iIntDelta", "Internal Delta Winding"
      t_nt2ag = "t:nt2ag", "*30deg, Phase Shift"
      t_vecgrp = "t:vecgrp", "Name"
      t_uk0tr = "t:uk0tr", "%, Short-Circuit Voltage uk0"
      t_ur0tr = "t:ur0tr", "%, SHC-Voltage (Re(uk0)) uk0r"
      t_x0tor0 = "t:x0tor0", "Ratio X0/R0"
      t_x0pu = "t:x0pu", "p.u., Reactance x0"
      t_r0pu = "t:r0pu", "p.u., Resistance r0"
      t_uk0_hls = "t:uk0_hls", "%, HV-SHC-Voltage uk0 (LV short-circuit)"
      t_ur0_hls = "t:ur0_hls", "%, HV-SHC-Voltage Re(uk0) (LV short-circuit)"
      t_uk0_hlo = "t:uk0_hlo", "%, HV-SHC-Voltage uk0 (LV open)"
      t_ur0_hlo = "t:ur0_hlo", "%, HV-SHC-Voltage Re(uk0) (LV open)"
      t_uk0_lho = "t:uk0_lho", "%, LV-SHC-Voltage uk0 (HV open)"
      t_ur0_lho = "t:ur0_lho", "%, LV-SHC-Voltage Re(uk0) (HV open)"
      t_x0pu_hls = "t:x0pu_hls", "p.u., HV-Reactance x0 (LV short-circuit)"
      t_xtr0_hls = "t:xtr0_hls", "HV-Ratio X0/R0 (LV short-circuit)"
      t_r0pu_hls = "t:r0pu_hls", "p.u., HV-Resistance r0 (LV short-circuit)"
      t_x0pu_hlo = "t:x0pu_hlo", "p.u., HV-Reactance x0 (LV open)"
      t_xtr0_hlo = "t:xtr0_hlo", "HV-Ratio X0/R0 (LV open)"
      t_r0pu_hlo = "t:r0pu_hlo", "p.u., HV-Resistance r0 (LV open)"
      t_x0pu_lho = "t:x0pu_lho", "p.u., LV-Reactance x0 (HV open)"
      t_xtr0_lho = "t:xtr0_lho", "LV-Ratio X0/R0 (HV open)"
      t_r0pu_lho = "t:r0pu_lho", "p.u., LV-Resistance r0 (HV open)"
      t_uk0delta = "t:uk0delta", "%, Delta Winding, uk0"
      t_ur0delta = "t:ur0delta", "%, Delta Winding, Re(uk0)"
      t_x0tor0delta = "t:x0tor0delta", "Delta Winding, X0/R0"
      t_x0delta = "t:x0delta", "p.u., Delta Winding, x0"
      t_r0delta = "t:r0delta", "p.u., Delta Winding, r0"
      r_typ_id = "r:typ_id", "TypTr2, Type"
      r_bushv = "r:bushv", "StaCubic, HV-Side"
      r_buslv = "r:buslv", "StaCubic, LV-Side"
      r_cpZone = "r:cpZone", "Zone"
      r_cpArea = "r:cpArea", "Area"
      r_pRating = "r:pRating", "IntThrating, Thermal Rating"
      r_pMeteo = "r:pMeteo", "ElmMeteostat, Meteo. Station"
      r_bushvn = "r:bushvn", "StaCubic, HV-Neutral"
      r_buslvn = "r:buslvn", "StaCubic, LV-Neutral"
      r_cpGrid = "r:cpGrid", "Grid"
      r_cpOwner = "r:cpOwner", "Owner"
      r_cpOperator = "r:cpOperator", "Operator"
      r_cpBranch = "r:cpBranch", "Branch"
      r_cpSubstat = "r:cpSubstat", "Substation"
      r_cpSite = "r:cpSite", "Site"
      r_cpMeteostat = "r:cpMeteostat", "Meteo Station"
      r_cpHeadFold = "r:cpHeadFold", "Head Folder"
      r_fold_id = "r:fold_id", "In Folder"
      r_cpSupplyTransformer = "r:cpSupplyTransformer", "Supplying Transformer"
      r_cpSupplyTrfStation = "r:cpSupplyTrfStation", "Supplying Secondary Substation"
      r_cpSupplySubstation = "r:cpSupplySubstation", "Supplying Substation"

    class ElmTr3(Enum):
      _init_ = 'value __doc__'

      b_loc_name = "b:loc_name", "Name"
      b_nt3ag_h = "b:nt3ag_h", "Rel.Phase Displacement"
      b_nt3ag_m = "b:nt3ag_m", "Rel.Phase Displacement"
      b_nt3ag_l = "b:nt3ag_l", "Rel.Phase Displacement"
      b_itrcn_A = "b:itrcn:A", "Automatic Tap-Changer on/off"
      b_itrcn_B = "b:itrcn:B", "Automatic Tap-Changer on/off"
      b_itrcn_C = "b:itrcn:C", "Automatic Tap-Changer on/off"
      b_itrcn_N = "b:itrcn:N", "Automatic Tap-Changer on/off"
      b_icont_A = "b:icont:A", "Continuous Tap"
      b_icont_B = "b:icont:B", "Continuous Tap"
      b_icont_C = "b:icont:C", "Continuous Tap"
      b_icont_N = "b:icont:N", "Continuous Tap"
      b_iExtCtrl_A = "b:iExtCtrl:A", "External Controlled"
      b_iExtCtrl_B = "b:iExtCtrl:B", "External Controlled"
      b_iExtCtrl_C = "b:iExtCtrl:C", "External Controlled"
      b_iExtCtrl_N = "b:iExtCtrl:N", "External Controlled"
      b_isat = "b:isat", "Saturated Model"
      b_ipsu_A = "b:ipsu:A", "Power Station Unit"
      b_ipsu_B = "b:ipsu:B", "Power Station Unit"
      b_ipsu_C = "b:ipsu:C", "Power Station Unit"
      b_ipsu_N = "b:ipsu:N", "Power Station Unit"
      e_loc_name = "e:loc_name", "Name"
      e_typ_id = "e:typ_id", "TypTr3, Type"
      e_bushv = "e:bushv", "StaCubic, HV-Side"
      e_bushv_bar = "e:bushv_bar", "HV-Side"
      e_busmv = "e:busmv", "StaCubic, MV-Side"
      e_busmv_bar = "e:busmv_bar", "MV-Side"
      e_buslv = "e:buslv", "StaCubic, LV-Side"
      e_buslv_bar = "e:buslv_bar", "LV-Side"
      e_iZoneBus = "e:iZoneBus", "Zone"
      e_cpZone = "e:cpZone", "Zone"
      e_iAreaBus = "e:iAreaBus", "Area"
      e_cpArea = "e:cpArea", "Area"
      e_outserv = "e:outserv", "Out of Service"
      e_nt3nm = "e:nt3nm", "parallel Transformers"
      e_i_auto_hl = "e:i_auto_hl", "Auto Transformer"
      e_pRating_h = "e:pRating_h", "IntThrating, HV-Side"
      e_pRating_m = "e:pRating_m", "IntThrating, MV-Side"
      e_pRating_l = "e:pRating_l", "IntThrating, LV-Side"
      e_pMeteo = "e:pMeteo", "ElmMeteostat, Meteo. Station"
      e_ratfac_h = "e:ratfac_h", "HV-Side"
      e_ratfac_m = "e:ratfac_m", "MV-Side"
      e_ratfac_l = "e:ratfac_l", "LV-Side"
      e_Snom_h_a = "e:Snom_h_a", "MVA, HV-Side, Nom.Power (act.)"
      e_Snom_m_a = "e:Snom_m_a", "MVA, MV-Side, Nom.Power (act.)"
      e_Snom_l_a = "e:Snom_l_a", "MVA, LV-Side, Nom.Power (act.)"
      e_Snom_h = "e:Snom_h", "MVA, HV-Side, Nominal Power"
      e_Snom_m = "e:Snom_m", "MVA, MV-Side, Nominal Power"
      e_Snom_l = "e:Snom_l", "MVA, LV-Side, Nominal Power"
      e_cneutcon = "e:cneutcon", "N-Connection"
      e_bushvn = "e:bushvn", "StaCubic, HV-Neutral"
      e_bushvn_bar = "e:bushvn_bar", "HV-Neutral"
      e_busmvn = "e:busmvn", "StaCubic, MV-Neutral"
      e_busmvn_bar = "e:busmvn_bar", "MV-Neutral"
      e_buslvn = "e:buslvn", "StaCubic, LV-Neutral"
      e_buslvn_bar = "e:buslvn_bar", "LV-Neutral"
      e_i_neuthv = "e:i_neuthv", "Neutral Connection"
      e_i_eahv = "e:i_eahv", "Phase 2 internally grounded"
      e_i_hvcon = "e:i_hvcon", "Phase 2 connected"
      e_cgnd_h = "e:cgnd_h", "Star Point"
      e_cpeter_h = "e:cpeter_h", "Petersen Coil"
      e_re0h = "e:re0h", "Ohm, Resistance, Re"
      e_xe0h = "e:xe0h", "Ohm, Reactance, Xe"
      e_i_neutmv = "e:i_neutmv", "Neutral Connection"
      e_i_eamv = "e:i_eamv", "Phase 2 internally grounded"
      e_i_mvcon = "e:i_mvcon", "Phase 2 connected"
      e_cgnd_m = "e:cgnd_m", "Star Point"
      e_cpeter_m = "e:cpeter_m", "Petersen Coil"
      e_re0m = "e:re0m", "Ohm, Resistance, Re"
      e_xe0m = "e:xe0m", "Ohm, Reactance, Xe"
      e_i_neutlv = "e:i_neutlv", "Neutral Connection"
      e_i_ealv = "e:i_ealv", "Phase 2 internally grounded"
      e_i_lvcon = "e:i_lvcon", "Phase 2 connected"
      e_cgnd_l = "e:cgnd_l", "Star Point"
      e_cpeter_l = "e:cpeter_l", "Petersen Coil"
      e_re0l = "e:re0l", "Ohm, Resistance, Re"
      e_xe0l = "e:xe0l", "Ohm, Reactance, Xe"
      e_rSbasepu_hm = "e:rSbasepu_hm", "p.u., r(HV-MV) (Sbase)"
      e_rSbasepu_ml = "e:rSbasepu_ml", "p.u., r(MV-LV) (Sbase)"
      e_rSbasepu_lh = "e:rSbasepu_lh", "p.u., r(LV-HV) (Sbase)"
      e_xSbasepu_hm = "e:xSbasepu_hm", "p.u., x(HV-MV) (Sbase)"
      e_xSbasepu_ml = "e:xSbasepu_ml", "p.u., x(MV-LV) (Sbase)"
      e_xSbasepu_lh = "e:xSbasepu_lh", "p.u., x(LV-HV) (Sbase)"
      e_rSbasepu_h = "e:rSbasepu_h", "p.u., r(HV) (Sbase)"
      e_rSbasepu_m = "e:rSbasepu_m", "p.u., r(MV) (Sbase)"
      e_rSbasepu_l = "e:rSbasepu_l", "p.u., r(LV) (Sbase)"
      e_xSbasepu_h = "e:xSbasepu_h", "p.u., x(HV) (Sbase)"
      e_xSbasepu_m = "e:xSbasepu_m", "p.u., x(MV) (Sbase)"
      e_xSbasepu_l = "e:xSbasepu_l", "p.u., x(LV) (Sbase)"
      e_bSbasepu = "e:bSbasepu", "p.u., b (Sbase)"
      e_r0Sbasepu_hm = "e:r0Sbasepu_hm", "p.u., r0(HV-MV) (Sbase)"
      e_r0Sbasepu_ml = "e:r0Sbasepu_ml", "p.u., r0(MV-LV) (Sbase)"
      e_r0Sbasepu_lh = "e:r0Sbasepu_lh", "p.u., r0(LV-HV) (Sbase)"
      e_x0Sbasepu_hm = "e:x0Sbasepu_hm", "p.u., x0(HV-MV) (Sbase)"
      e_x0Sbasepu_ml = "e:x0Sbasepu_ml", "p.u., x0(MV-LV) (Sbase)"
      e_x0Sbasepu_lh = "e:x0Sbasepu_lh", "p.u., x0(LV-HV) (Sbase)"
      e_r0Sbasepu_h = "e:r0Sbasepu_h", "p.u., r0(HV) (Sbase)"
      e_r0Sbasepu_m = "e:r0Sbasepu_m", "p.u., r0(MV) (Sbase)"
      e_r0Sbasepu_l = "e:r0Sbasepu_l", "p.u., r0(LV) (Sbase)"
      e_x0Sbasepu_h = "e:x0Sbasepu_h", "p.u., x0(HV) (Sbase)"
      e_x0Sbasepu_m = "e:x0Sbasepu_m", "p.u., x0(MV) (Sbase)"
      e_x0Sbasepu_l = "e:x0Sbasepu_l", "p.u., x0(LV) (Sbase)"
      e_Inom_h_a = "e:Inom_h_a", "kA, HV-Side, Nominal Current (act.)"
      e_Inom_m_a = "e:Inom_m_a", "kA, MV-Side, Nominal Current (act.)"
      e_Inom_l_a = "e:Inom_l_a", "kA, LV-Side, Nominal Current (act.)"
      e_Inom_h = "e:Inom_h", "kA, HV-Side, Nominal Current"
      e_Inom_m = "e:Inom_m", "kA, MV-Side, Nominal Current"
      e_Inom_l = "e:Inom_l", "kA, LV-Side, Nominal Current"
      e_cpGrid = "e:cpGrid", "Grid"
      e_cpOwner = "e:cpOwner", "Owner"
      e_cpOperator = "e:cpOperator", "Operator"
      e_cpBranch = "e:cpBranch", "Branch"
      e_cpSubstat = "e:cpSubstat", "Substation"
      e_cpSite = "e:cpSite", "Site"
      e_cpMeteostat = "e:cpMeteostat", "Meteo Station"
      e_cpHeadFold = "e:cpHeadFold", "Head Folder"
      e_fold_id = "e:fold_id", "In Folder"
      e_ciOutaged = "e:ciOutaged", "Planned Outage"
      e_ciEnergized = "e:ciEnergized", "Energized"
      e_ciEarthed = "e:ciEarthed", "Earthed"
      e_cDisplayName = "e:cDisplayName", "Display Name"
      e_cpSupplyTransformer = "e:cpSupplyTransformer", "Supplying Transformer"
      e_cpSupplyTrfStation = "e:cpSupplyTrfStation", "Supplying Secondary Substation"
      e_cpSupplySubstation = "e:cpSupplySubstation", "Supplying Substation"
      t_loc_name = "t:loc_name", "Name"
      t_nt3ph = "t:nt3ph", "Technology"
      t_strn3_h = "t:strn3_h", "MVA, HV-Side"
      t_strn3_m = "t:strn3_m", "MVA, MV-Side"
      t_strn3_l = "t:strn3_l", "MVA, LV-Side"
      t_utrn3_h = "t:utrn3_h", "kV, HV-Side"
      t_utrn3_m = "t:utrn3_m", "kV, MV-Side"
      t_utrn3_l = "t:utrn3_l", "kV, LV-Side"
      t_tr3cn_h = "t:tr3cn_h", "HV-Side"
      t_nt3ag_h = "t:nt3ag_h", "*30deg, Phase Shift"
      t_tr3cn_m = "t:tr3cn_m", "MV-Side"
      t_nt3ag_m = "t:nt3ag_m", "*30deg, Phase Shift"
      t_tr3cn_l = "t:tr3cn_l", "LV-Side"
      t_nt3ag_l = "t:nt3ag_l", "*30deg, Phase Shift"
      t_vecgrp = "t:vecgrp", "Name"
      t_uktr3_h = "t:uktr3_h", "%, HV-MV"
      t_uktr3_m = "t:uktr3_m", "%, MV-LV"
      t_uktr3_l = "t:uktr3_l", "%, LV-HV"
      t_pcut3_h = "t:pcut3_h", "kW, HV-MV"
      t_pcut3_m = "t:pcut3_m", "kW, MV-LV"
      t_pcut3_l = "t:pcut3_l", "kW, LV-HV"
      t_uktrr3_h = "t:uktrr3_h", "%, HV-MV"
      t_uktrr3_m = "t:uktrr3_m", "%, MV-LV"
      t_uktrr3_l = "t:uktrr3_l", "%, LV-HV"
      t_xtor3_h = "t:xtor3_h", "HV-MV"
      t_xtor3_m = "t:xtor3_m", "MV-LV"
      t_xtor3_l = "t:xtor3_l", "LV-HV"
      t_x1pu_h = "t:x1pu_h", "p.u., HV-MV"
      t_x1pu_m = "t:x1pu_m", "p.u., MV-LV"
      t_x1pu_l = "t:x1pu_l", "p.u., LV-HV"
      t_r1pu_h = "t:r1pu_h", "p.u., HV-MV"
      t_r1pu_m = "t:r1pu_m", "p.u., MV-LV"
      t_r1pu_l = "t:r1pu_l", "p.u., LV-HV"
      t_uk0hm = "t:uk0hm", "%, HV-MV"
      t_uk0ml = "t:uk0ml", "%, MV-LV"
      t_uk0hl = "t:uk0hl", "%, LV-HV"
      t_ur0hm = "t:ur0hm", "%, HV-MV"
      t_ur0ml = "t:ur0ml", "%, MV-LV"
      t_ur0hl = "t:ur0hl", "%, LV-HV"
      t_xtr0hm = "t:xtr0hm", "HV-MV"
      t_xtr0ml = "t:xtr0ml", "MV-LV"
      t_xtr0hl = "t:xtr0hl", "LV-HV"
      t_x0puhm = "t:x0puhm", "p.u., HV-MV"
      t_x0puml = "t:x0puml", "p.u., MV-LV"
      t_x0puhl = "t:x0puhl", "p.u., LV-HV"
      t_r0puhm = "t:r0puhm", "p.u., HV-MV"
      t_r0puml = "t:r0puml", "p.u., MV-LV"
      t_r0puhl = "t:r0puhl", "p.u., LV-HV"
      r_typ_id = "r:typ_id", "TypTr3, Type"
      r_bushv = "r:bushv", "StaCubic, HV-Side"
      r_busmv = "r:busmv", "StaCubic, MV-Side"
      r_buslv = "r:buslv", "StaCubic, LV-Side"
      r_cpZone = "r:cpZone", "Zone"
      r_cpArea = "r:cpArea", "Area"
      r_pRating_h = "r:pRating_h", "IntThrating, HV-Side"
      r_pRating_m = "r:pRating_m", "IntThrating, MV-Side"
      r_pRating_l = "r:pRating_l", "IntThrating, LV-Side"
      r_pMeteo = "r:pMeteo", "ElmMeteostat, Meteo. Station"
      r_bushvn = "r:bushvn", "StaCubic, HV-Neutral"
      r_busmvn = "r:busmvn", "StaCubic, MV-Neutral"
      r_buslvn = "r:buslvn", "StaCubic, LV-Neutral"
      r_cpGrid = "r:cpGrid", "Grid"
      r_cpOwner = "r:cpOwner", "Owner"
      r_cpOperator = "r:cpOperator", "Operator"
      r_cpBranch = "r:cpBranch", "Branch"
      r_cpSubstat = "r:cpSubstat", "Substation"
      r_cpSite = "r:cpSite", "Site"
      r_cpMeteostat = "r:cpMeteostat", "Meteo Station"
      r_cpHeadFold = "r:cpHeadFold", "Head Folder"
      r_fold_id = "r:fold_id", "In Folder"
      r_cpSupplyTransformer = "r:cpSupplyTransformer", "Supplying Transformer"
      r_cpSupplyTrfStation = "r:cpSupplyTrfStation", "Supplying Secondary Substation"
      r_cpSupplySubstation = "r:cpSupplySubstation", "Supplying Substation"

    class ElmVac(Enum):
      _init_ = 'value __doc__'

      b_loc_name = "b:loc_name", "Name"
      e_loc_name = "e:loc_name", "Name"
      e_bus1 = "e:bus1", "StaCubic, Terminal"
      e_bus1_bar = "e:bus1_bar", "Terminal"
      e_cpZone = "e:cpZone", "Zone"
      e_cpArea = "e:cpArea", "Area"
      e_outserv = "e:outserv", "Out of Service"
      e_nphase = "e:nphase", "No. of Phases"
      e_itype = "e:itype", "Type"
      e_Unom = "e:Unom", "kV, Line-Line"
      e_Unnom = "e:Unnom", "kV, Line-Ground"
      e_c_pmod = "e:c_pmod", "Model"
      e_cpGrid = "e:cpGrid", "Grid"
      e_cpOwner = "e:cpOwner", "Owner"
      e_cpOperator = "e:cpOperator", "Operator"
      e_cpBranch = "e:cpBranch", "Branch"
      e_cpSubstat = "e:cpSubstat", "Substation"
      e_cpSite = "e:cpSite", "Site"
      e_cpMeteostat = "e:cpMeteostat", "Meteo Station"
      e_cpHeadFold = "e:cpHeadFold", "Head Folder"
      e_fold_id = "e:fold_id", "In Folder"
      e_ciOutaged = "e:ciOutaged", "Planned Outage"
      e_ciEnergized = "e:ciEnergized", "Energized"
      e_ciEarthed = "e:ciEarthed", "Earthed"
      e_cDisplayName = "e:cDisplayName", "Display Name"
      e_cpSupplyTransformer = "e:cpSupplyTransformer", "Supplying Transformer"
      e_cpSupplyTrfStation = "e:cpSupplyTrfStation", "Supplying Secondary Substation"
      e_cpSupplySubstation = "e:cpSupplySubstation", "Supplying Substation"
      r_bus1 = "r:bus1", "StaCubic, Terminal"
      r_cpZone = "r:cpZone", "Zone"
      r_cpArea = "r:cpArea", "Area"
      r_c_pmod = "r:c_pmod", "Model"
      r_cpGrid = "r:cpGrid", "Grid"
      r_cpOwner = "r:cpOwner", "Owner"
      r_cpOperator = "r:cpOperator", "Operator"
      r_cpBranch = "r:cpBranch", "Branch"
      r_cpSubstat = "r:cpSubstat", "Substation"
      r_cpSite = "r:cpSite", "Site"
      r_cpMeteostat = "r:cpMeteostat", "Meteo Station"
      r_cpHeadFold = "r:cpHeadFold", "Head Folder"
      r_fold_id = "r:fold_id", "In Folder"
      r_cpSupplyTransformer = "r:cpSupplyTransformer", "Supplying Transformer"
      r_cpSupplyTrfStation = "r:cpSupplyTrfStation", "Supplying Secondary Substation"
      r_cpSupplySubstation = "r:cpSupplySubstation", "Supplying Substation"

    class ElmVsc(Enum):
      _init_ = 'value __doc__'

      b_loc_name = "b:loc_name", "Name"
      b_iWindGen = "b:iWindGen", "Wind Generator"
      e_loc_name = "e:loc_name", "Name"
      e_busac = "e:busac", "StaCubic, Terminal AC"
      e_busac_bar = "e:busac_bar", "Terminal AC"
      e_busdp = "e:busdp", "StaCubic, Terminal DC+"
      e_busdp_bar = "e:busdp_bar", "Terminal DC+"
      e_busdm = "e:busdm", "StaCubic, Terminal DC-"
      e_busdm_bar = "e:busdm_bar", "Terminal DC-"
      e_iZoneBus = "e:iZoneBus", "Zone"
      e_cpZone = "e:cpZone", "Zone"
      e_iAreaBus = "e:iAreaBus", "Area"
      e_cpArea = "e:cpArea", "Area"
      e_outserv = "e:outserv", "Out of Service"
      e_nparnum = "e:nparnum", "parallel converters"
      e_Unom = "e:Unom", "kV, Rated AC-voltage"
      e_Unomdc = "e:Unomdc", "kV, Rated DC-voltage (DC)"
      e_Snom = "e:Snom", "MVA, Rated power"
      e_i_mod = "e:i_mod", "Modulation"
      e_uk = "e:uk", "%, Short circuit impedance"
      e_Pcu = "e:Pcu", "kW, Copper losses"
      e_R0toR1 = "e:R0toR1", "R0/R1 ratio"
      e_X0toX1 = "e:X0toX1", "X0/X1 ratio"
      e_vsctype = "e:vsctype", "Converter type"
      e_Rarm = "e:Rarm", "Ohm, Resistance, Rarm"
      e_Larm = "e:Larm", "mH, Inductance, Larm"
      e_cCategory = "e:cCategory", "Category"
      e_cSubCategory = "e:cSubCategory", "Subcategory"
      e_c_pmod = "e:c_pmod", "Model"
      e_Inom = "e:Inom", "kA, Nominal Current (AC)"
      e_Inomdc = "e:Inomdc", "kA, Nominal Current (DC)"
      e_r0 = "e:r0", "p.u., Resistance, r0"
      e_x0 = "e:x0", "p.u., Reactance, x0"
      e_iThiPWM = "e:iThiPWM", "Third harmonic injection"
      e_iZarmDCside = "e:iZarmDCside", "Consider arm reactor on DC side"
      e_cpGrid = "e:cpGrid", "Grid"
      e_cpOwner = "e:cpOwner", "Owner"
      e_cpOperator = "e:cpOperator", "Operator"
      e_cpBranch = "e:cpBranch", "Branch"
      e_cpSubstat = "e:cpSubstat", "Substation"
      e_cpSite = "e:cpSite", "Site"
      e_cpMeteostat = "e:cpMeteostat", "Meteo Station"
      e_cpHeadFold = "e:cpHeadFold", "Head Folder"
      e_fold_id = "e:fold_id", "In Folder"
      e_ciOutaged = "e:ciOutaged", "Planned Outage"
      e_ciEnergized = "e:ciEnergized", "Energized"
      e_ciEarthed = "e:ciEarthed", "Earthed"
      e_cDisplayName = "e:cDisplayName", "Display Name"
      e_cpSupplyTransformer = "e:cpSupplyTransformer", "Supplying Transformer"
      e_cpSupplyTrfStation = "e:cpSupplyTrfStation", "Supplying Secondary Substation"
      e_cpSupplySubstation = "e:cpSupplySubstation", "Supplying Substation"
      r_busac = "r:busac", "StaCubic, Terminal AC"
      r_busdp = "r:busdp", "StaCubic, Terminal DC+"
      r_busdm = "r:busdm", "StaCubic, Terminal DC-"
      r_cpZone = "r:cpZone", "Zone"
      r_cpArea = "r:cpArea", "Area"
      r_c_pmod = "r:c_pmod", "Model"
      r_cpGrid = "r:cpGrid", "Grid"
      r_cpOwner = "r:cpOwner", "Owner"
      r_cpOperator = "r:cpOperator", "Operator"
      r_cpBranch = "r:cpBranch", "Branch"
      r_cpSubstat = "r:cpSubstat", "Substation"
      r_cpSite = "r:cpSite", "Site"
      r_cpMeteostat = "r:cpMeteostat", "Meteo Station"
      r_cpHeadFold = "r:cpHeadFold", "Head Folder"
      r_fold_id = "r:fold_id", "In Folder"
      r_cpSupplyTransformer = "r:cpSupplyTransformer", "Supplying Transformer"
      r_cpSupplyTrfStation = "r:cpSupplyTrfStation", "Supplying Secondary Substation"
      r_cpSupplySubstation = "r:cpSupplySubstation", "Supplying Substation"

    class ElmVscmono(Enum):
      _init_ = 'value __doc__'

      b_loc_name = "b:loc_name", "Name"
      b_iWindGen = "b:iWindGen", "Wind Generator"
      e_loc_name = "e:loc_name", "Name"
      e_busac = "e:busac", "StaCubic, Terminal AC"
      e_busac_bar = "e:busac_bar", "Terminal AC"
      e_busdc = "e:busdc", "StaCubic, Terminal DC"
      e_busdc_bar = "e:busdc_bar", "Terminal DC"
      e_iZoneBus = "e:iZoneBus", "Zone"
      e_cpZone = "e:cpZone", "Zone"
      e_iAreaBus = "e:iAreaBus", "Area"
      e_cpArea = "e:cpArea", "Area"
      e_outserv = "e:outserv", "Out of Service"
      e_nparnum = "e:nparnum", "parallel converters"
      e_Unom = "e:Unom", "kV, Rated AC-voltage"
      e_Unomdc = "e:Unomdc", "kV, Rated DC-voltage (DC)"
      e_Snom = "e:Snom", "MVA, Rated power"
      e_i_mod = "e:i_mod", "Modulation"
      e_uk = "e:uk", "%, Short circuit impedance"
      e_Pcu = "e:Pcu", "kW, Copper losses"
      e_R0toR1 = "e:R0toR1", "R0/R1 ratio"
      e_X0toX1 = "e:X0toX1", "X0/X1 ratio"
      e_vsctype = "e:vsctype", "Converter type"
      e_Rarm = "e:Rarm", "Ohm, Resistance, Rarm"
      e_Larm = "e:Larm", "mH, Inductance, Larm"
      e_cCategory = "e:cCategory", "Category"
      e_cSubCategory = "e:cSubCategory", "Subcategory"
      e_c_pmod = "e:c_pmod", "Model"
      e_Inom = "e:Inom", "kA, Nominal Current (AC)"
      e_Inomdc = "e:Inomdc", "kA, Nominal Current (DC)"
      e_r0 = "e:r0", "p.u., Resistance, r0"
      e_x0 = "e:x0", "p.u., Reactance, x0"
      e_iThiPWM = "e:iThiPWM", "Third harmonic injection"
      e_iZarmDCside = "e:iZarmDCside", "Consider arm reactor on DC side"
      e_cpGrid = "e:cpGrid", "Grid"
      e_cpOwner = "e:cpOwner", "Owner"
      e_cpOperator = "e:cpOperator", "Operator"
      e_cpBranch = "e:cpBranch", "Branch"
      e_cpSubstat = "e:cpSubstat", "Substation"
      e_cpSite = "e:cpSite", "Site"
      e_cpMeteostat = "e:cpMeteostat", "Meteo Station"
      e_cpHeadFold = "e:cpHeadFold", "Head Folder"
      e_fold_id = "e:fold_id", "In Folder"
      e_ciOutaged = "e:ciOutaged", "Planned Outage"
      e_ciEnergized = "e:ciEnergized", "Energized"
      e_ciEarthed = "e:ciEarthed", "Earthed"
      e_cDisplayName = "e:cDisplayName", "Display Name"
      e_cpSupplyTransformer = "e:cpSupplyTransformer", "Supplying Transformer"
      e_cpSupplyTrfStation = "e:cpSupplyTrfStation", "Supplying Secondary Substation"
      e_cpSupplySubstation = "e:cpSupplySubstation", "Supplying Substation"
      r_busac = "r:busac", "StaCubic, Terminal AC"
      r_busdc = "r:busdc", "StaCubic, Terminal DC"
      r_cpZone = "r:cpZone", "Zone"
      r_cpArea = "r:cpArea", "Area"
      r_c_pmod = "r:c_pmod", "Model"
      r_cpGrid = "r:cpGrid", "Grid"
      r_cpOwner = "r:cpOwner", "Owner"
      r_cpOperator = "r:cpOperator", "Operator"
      r_cpBranch = "r:cpBranch", "Branch"
      r_cpSubstat = "r:cpSubstat", "Substation"
      r_cpSite = "r:cpSite", "Site"
      r_cpMeteostat = "r:cpMeteostat", "Meteo Station"
      r_cpHeadFold = "r:cpHeadFold", "Head Folder"
      r_fold_id = "r:fold_id", "In Folder"
      r_cpSupplyTransformer = "r:cpSupplyTransformer", "Supplying Transformer"
      r_cpSupplyTrfStation = "r:cpSupplyTrfStation", "Supplying Secondary Substation"
      r_cpSupplySubstation = "r:cpSupplySubstation", "Supplying Substation"

    class ElmXnet(Enum):
      _init_ = 'value __doc__'

      b_loc_name = "b:loc_name", "Name"
      e_loc_name = "e:loc_name", "Name"
      e_bus1 = "e:bus1", "StaCubic, Terminal"
      e_bus1_bar = "e:bus1_bar", "Terminal"
      e_cpZone = "e:cpZone", "Zone"
      e_cpArea = "e:cpArea", "Area"
      e_outserv = "e:outserv", "Out of Service"
      e_iintgnd = "e:iintgnd", "N-Connection"
      e_bus1n = "e:bus1n", "StaCubic, Neutral"
      e_bus1n_bar = "e:bus1n_bar", "Neutral"
      e_cgnd = "e:cgnd", "Star Point"
      e_cpeter = "e:cpeter", "Petersen Coil"
      e_Re = "e:Re", "Ohm, Resistance, Re"
      e_Xe = "e:Xe", "Ohm, Reactance, Xe"
      e_cpGrid = "e:cpGrid", "Grid"
      e_cpOwner = "e:cpOwner", "Owner"
      e_cpOperator = "e:cpOperator", "Operator"
      e_cpBranch = "e:cpBranch", "Branch"
      e_cpSubstat = "e:cpSubstat", "Substation"
      e_cpSite = "e:cpSite", "Site"
      e_cpMeteostat = "e:cpMeteostat", "Meteo Station"
      e_cpHeadFold = "e:cpHeadFold", "Head Folder"
      e_fold_id = "e:fold_id", "In Folder"
      e_ciOutaged = "e:ciOutaged", "Planned Outage"
      e_ciEnergized = "e:ciEnergized", "Energized"
      e_ciEarthed = "e:ciEarthed", "Earthed"
      e_cDisplayName = "e:cDisplayName", "Display Name"
      e_cpSupplyTransformer = "e:cpSupplyTransformer", "Supplying Transformer"
      e_cpSupplyTrfStation = "e:cpSupplyTrfStation", "Supplying Secondary Substation"
      e_cpSupplySubstation = "e:cpSupplySubstation", "Supplying Substation"
      r_bus1 = "r:bus1", "StaCubic, Terminal"
      r_cpZone = "r:cpZone", "Zone"
      r_cpArea = "r:cpArea", "Area"
      r_bus1n = "r:bus1n", "StaCubic, Neutral"
      r_cpGrid = "r:cpGrid", "Grid"
      r_cpOwner = "r:cpOwner", "Owner"
      r_cpOperator = "r:cpOperator", "Operator"
      r_cpBranch = "r:cpBranch", "Branch"
      r_cpSubstat = "r:cpSubstat", "Substation"
      r_cpSite = "r:cpSite", "Site"
      r_cpMeteostat = "r:cpMeteostat", "Meteo Station"
      r_cpHeadFold = "r:cpHeadFold", "Head Folder"
      r_fold_id = "r:fold_id", "In Folder"
      r_cpSupplyTransformer = "r:cpSupplyTransformer", "Supplying Transformer"
      r_cpSupplyTrfStation = "r:cpSupplyTrfStation", "Supplying Secondary Substation"
      r_cpSupplySubstation = "r:cpSupplySubstation", "Supplying Substation"

    class ElmZone(Enum):
      _init_ = 'value __doc__'

      b_loc_name = "b:loc_name", "Name"
      b_n_ele = "b:n_ele", "Number of Elements"
      b_n_zones = "b:n_zones", "Number of Connected Zones"
      b_scale = "b:scale", "Load Scaling Factor"
      b_scaleWind = "b:scaleWind", "Wind Generation Scaling Factor"
      e_loc_name = "e:loc_name", "Name"
      e_icolor = "e:icolor", "Colour"
      e_curscale = "e:curscale", "Load Scaling Factor"
      e_windscale = "e:windscale", "Wind Generation Scaling Factor"

    class ElmZpu(Enum):
      _init_ = 'value __doc__'

      b_loc_name = "b:loc_name", "Name"
      b_nphshift = "b:nphshift", "Rel.Phase Displacement"
      b_uratio = "b:uratio", "Rating Factor"
      b_addang = "b:addang", "rad, Additional Phase Shift"
      e_loc_name = "e:loc_name", "Name"
      e_bus1 = "e:bus1", "StaCubic, Terminal i"
      e_bus1_bar = "e:bus1_bar", "Terminal i"
      e_bus2 = "e:bus2", "StaCubic, Terminal j"
      e_bus2_bar = "e:bus2_bar", "Terminal j"
      e_iZoneBus = "e:iZoneBus", "Zone"
      e_cpZone = "e:cpZone", "Zone"
      e_iAreaBus = "e:iAreaBus", "Area"
      e_cpArea = "e:cpArea", "Area"
      e_outserv = "e:outserv", "Out of Service"
      e_Sn = "e:Sn", "MVA, Rated Power"
      e_nphases = "e:nphases", "Phases"
      e_nphshift = "e:nphshift", "*30deg, Nominal Phase Shift"
      e_uratio = "e:uratio", "Tap Ratio"
      e_ag = "e:ag", "deg, Additional Phase Shift"
      e_cpGrid = "e:cpGrid", "Grid"
      e_cpOwner = "e:cpOwner", "Owner"
      e_cpOperator = "e:cpOperator", "Operator"
      e_cpBranch = "e:cpBranch", "Branch"
      e_cpSubstat = "e:cpSubstat", "Substation"
      e_cpSite = "e:cpSite", "Site"
      e_cpMeteostat = "e:cpMeteostat", "Meteo Station"
      e_cpHeadFold = "e:cpHeadFold", "Head Folder"
      e_fold_id = "e:fold_id", "In Folder"
      e_ciOutaged = "e:ciOutaged", "Planned Outage"
      e_ciEnergized = "e:ciEnergized", "Energized"
      e_ciEarthed = "e:ciEarthed", "Earthed"
      e_cDisplayName = "e:cDisplayName", "Display Name"
      e_cpSupplyTransformer = "e:cpSupplyTransformer", "Supplying Transformer"
      e_cpSupplyTrfStation = "e:cpSupplyTrfStation", "Supplying Secondary Substation"
      e_cpSupplySubstation = "e:cpSupplySubstation", "Supplying Substation"
      r_bus1 = "r:bus1", "StaCubic, Terminal i"
      r_bus2 = "r:bus2", "StaCubic, Terminal j"
      r_cpZone = "r:cpZone", "Zone"
      r_cpArea = "r:cpArea", "Area"
      r_cpGrid = "r:cpGrid", "Grid"
      r_cpOwner = "r:cpOwner", "Owner"
      r_cpOperator = "r:cpOperator", "Operator"
      r_cpBranch = "r:cpBranch", "Branch"
      r_cpSubstat = "r:cpSubstat", "Substation"
      r_cpSite = "r:cpSite", "Site"
      r_cpMeteostat = "r:cpMeteostat", "Meteo Station"
      r_cpHeadFold = "r:cpHeadFold", "Head Folder"
      r_fold_id = "r:fold_id", "In Folder"
      r_cpSupplyTransformer = "r:cpSupplyTransformer", "Supplying Transformer"
      r_cpSupplyTrfStation = "r:cpSupplyTrfStation", "Supplying Secondary Substation"
      r_cpSupplySubstation = "r:cpSupplySubstation", "Supplying Substation"


  class LF_Bal:

    class ElmArea(Enum):
      _init_ = 'value __doc__'

      c_n_ele = "c:n_ele", "Number of Elements"
      c_n_areas = "c:n_areas", "Number of Connected Areas"
      c_n_volt = "c:n_volt", "Number of Voltage Levels"
      c_n_bar = "c:n_bar", "No. of Busbars"
      c_n_term = "c:n_term", "No. of Terminals"
      c_n_lne = "c:n_lne", "No. of Lines"
      c_n_sym = "c:n_sym", "No. of syn. Machines"
      c_n_asm = "c:n_asm", "No. of asyn. Machines"
      c_n_stg = "c:n_stg", "No. of Static Generators"
      c_n_pvs = "c:n_pvs", "No. of PV Systems"
      c_n_lod = "c:n_lod", "No. of Loads"
      c_n_cust = "c:n_cust", "No. of Customers"
      c_TotgenP = "c:TotgenP", "MW, Total Generation, Active Power"
      c_TotgenQ = "c:TotgenQ", "Mvar, Total Generation, Reactive Power"
      c_TotconsP = "c:TotconsP", "MW, Total Consumption, Active Power"
      c_TotconsQ = "c:TotconsQ", "Mvar, Total Consumption, Reactive Power"
      c_GenP = "c:GenP", "MW, Generators, Active Power"
      c_GenQ = "c:GenQ", "Mvar, Generators, Reactive Power"
      c_GenS = "c:GenS", "MVA, Generators, Apparent Power"
      c_Gencos = "c:Gencos", "Generators, Power Factor"
      c_GenPnom = "c:GenPnom", "MW, Generators, Nominal Active Power"
      c_GenQnom = "c:GenQnom", "Mvar, Generators, Nominal Reactive Power"
      c_GenSnom = "c:GenSnom", "MVA, Generators, Nominal Apparent Power"
      c_GenPdiff = "c:GenPdiff", "MW, Generators, difference between nominal and actual active power"
      c_GenQdiff = "c:GenQdiff", "Mvar, Generators, difference between nominal and actual reactive power"
      c_Gencosnom = "c:Gencosnom", "Generators, Nominal Power Factor"
      c_GenPdisp = "c:GenPdisp", "MW, Generators, Dispatch Active Power"
      c_GenPmism = "c:GenPmism", "MW, Generators, difference between actual and dispatched active power"
      c_GenPspinres = "c:GenPspinres", "MW, Generators, difference between maximum and actual active power"
      c_GenQspinres = "c:GenQspinres", "Mvar, Generators, difference between maximum and actual reactive power"
      c_MotP = "c:MotP", "MW, Motor Loads, Active Power"
      c_MotQ = "c:MotQ", "Mvar, Motor Loads, Reactive Power"
      c_MotS = "c:MotS", "MVA, Motor Loads, Apparent Power"
      c_Motcos = "c:Motcos", "Motor Loads, Power Factor"
      c_LoadP = "c:LoadP", "MW, Loads, Active Power"
      c_LoadQ = "c:LoadQ", "Mvar, Loads, Reactive Power"
      c_LoadS = "c:LoadS", "MVA, Loads, Apparent Power"
      c_Loadcos = "c:Loadcos", "Loads, Power Factor"
      c_LoadPnom = "c:LoadPnom", "MW, Loads, Nominal Active Power"
      c_LoadQnom = "c:LoadQnom", "Mvar, Loads, Nominal Reactive Power"
      c_LoadSnom = "c:LoadSnom", "MVA, Loads, Nominal Apparent Power"
      c_Loadcosnom = "c:Loadcosnom", "Loads, Nominal Power Factor"
      c_LoadPdiff = "c:LoadPdiff", "MW, Loads, difference between nominal and actual active power"
      c_LoadQdiff = "c:LoadQdiff", "Mvar, Loads, difference between nominal and actual reactive power"
      c_LoadPdem = "c:LoadPdem", "MW, Loads, Active Power Demand"
      c_LoadPmism = "c:LoadPmism", "MW, Loads, difference between actual active power and demand"
      c_CompP = "c:CompP", "MW, Compensation (Losses)"
      c_CompQ = "c:CompQ", "Mvar, Compensation, C"
      c_ComprQ = "c:ComprQ", "Mvar, Compensation, L"
      c_NetP = "c:NetP", "MW, External Networks, Active Power"
      c_NetQ = "c:NetQ", "Mvar, External Networks, Reactive Power"
      c_NetS = "c:NetS", "MVA, External Networks, Apparent Power"
      c_Netcos = "c:Netcos", "External Networks, Power Factor"
      c_InterP = "c:InterP", "MW, Interchange Flow, Active Power"
      c_InterQ = "c:InterQ", "Mvar, Interchange Flow, Reactive Power"
      c_LossP = "c:LossP", "MW, Losses, Active Power"
      c_LossQ = "c:LossQ", "Mvar, Losses, Reactive Power"
      c_LossPld = "c:LossPld", "MW, Losses, Active Power (load)"
      c_LossQld = "c:LossQld", "Mvar, Losses, Reactive Power (load)"
      c_LossPnld = "c:LossPnld", "MW, Losses, Active Power (no load)"
      c_LossQnld = "c:LossQnld", "Mvar, Losses, Reactive Power (no load)"
      c_LossPlne = "c:LossPlne", "MW, Line Losses, Active Power"
      c_LossQlne = "c:LossQlne", "Mvar, Line Losses, Reactive Power"
      c_LossPldlne = "c:LossPldlne", "MW, Line Losses, Active Power (load)"
      c_LossQldlne = "c:LossQldlne", "Mvar, Line Losses, Reactive Power (load)"
      c_LossPnldlne = "c:LossPnldlne", "MW, Line Losses, Active Power (no load)"
      c_LossQnldlne = "c:LossQnldlne", "Mvar, Line Losses, Reactive Power (no load)"
      c_LossPtrf = "c:LossPtrf", "MW, Transformer Losses, Active Power"
      c_LossQtrf = "c:LossQtrf", "Mvar, Transformer Losses, Reactive Power"
      c_LossPldtrf = "c:LossPldtrf", "MW, Transformer Losses, Active Power (load)"
      c_LossQldtrf = "c:LossQldtrf", "Mvar, Transformer Losses, Reactive Power (load)"
      c_LossPnldtrf = "c:LossPnldtrf", "MW, Transformer Losses, Active Power (no load)"
      c_LossQnldtrf = "c:LossQnldtrf", "Mvar, Transformer Losses, Reactive Power (no load)"
      c_TotLoadP = "c:TotLoadP", "MW, Total Load, Active Power"
      c_TotLoadQ = "c:TotLoadQ", "Mvar, Total Load, Reactive Power"
      c_TotLoadS = "c:TotLoadS", "MVA, Total Load, Apparent Power"
      c_TotLoadcos = "c:TotLoadcos", "Total Load, Power Factor"
      c_TotP = "c:TotP", "MW, Total Demand"
      c_UnsupP = "c:UnsupP", "MW, Unsupplied Demand"
      c_SupP = "c:SupP", "MW, Supplied Demand"
      c_ExportP = "c:ExportP", "MW, Export, Active Power"
      c_ExportQ = "c:ExportQ", "Mvar, Export, Reactive Power"
      c_ImportP = "c:ImportP", "MW, Import, Active Power"
      c_ImportQ = "c:ImportQ", "Mvar, Import, Reactive Power"
      c_cst_disp = "c:cst_disp", "USD/h, Total Production Costs"
      c_NrOvlBranch = "c:NrOvlBranch", "Number of overloaded branch components"
      c_NrGenQMax = "c:NrGenQMax", "Number of generators above their maximum reactive power limit"
      c_NrGenQMin = "c:NrGenQMin", "Number of generators below their minimum reactive power limit"
      c_NrTermMin = "c:NrTermMin", "Number of terminals below their minimum voltage limit"
      c_NrTermMax = "c:NrTermMax", "Number of terminals above their maximum voltage limit"
      c_RUC_Lead = "c:RUC_Lead", "Mvar, Used Reactive Capacity Lead"
      c_RUC_Lag = "c:RUC_Lag", "Mvar, Used Reactive Capacity Lag"
      c_RSC_Lead = "c:RSC_Lead", "Mvar, Spare Reactive Capacity Lead"
      c_RSC_Lag = "c:RSC_Lag", "Mvar, Spare Reactive Capacity Lag"
      c_Q_shgain = "c:Q_shgain", "Mvar, Reactive Power Shunt Gain"
      c_ConnMvar_max = "c:ConnMvar_max", "Mvar, Connected MVAR Capacity (max)"
      c_ConnMvar_min = "c:ConnMvar_min", "Mvar, Connected MVAR Capacity (min)"
      c_Maxloading = "c:Maxloading", "%, Maximum Loading"
      c_umin = "c:umin", "p.u., Minimum voltage of all terminals"
      c_umax = "c:umax", "p.u., Maximum voltage of all terminals"
      e_iInterChg = "e:iInterChg", "Consider Interchange Schedule"
      e_InterPset = "e:InterPset", "MW, Scheduled Active Power Interchange"

    class ElmAsm(Enum):
      _init_ = 'value __doc__'

      m_u1r_bus1 = "m:u1r:bus1", "p.u., Positive-Sequence-Voltage, Real Part"
      m_u1i_bus1 = "m:u1i:bus1", "p.u., Positive-Sequence-Voltage, Imaginary Part"
      m_u1_bus1 = "m:u1:bus1", "p.u., Positive-Sequence-Voltage, Magnitude"
      m_U1_bus1 = "m:U1:bus1", "kV, Line-Ground Positive-Sequence-Voltage, Magnitude"
      m_U1l_bus1 = "m:U1l:bus1", "kV, Line-Line Positive-Sequence-Voltage, Magnitude"
      m_phiu1_bus1 = "m:phiu1:bus1", "deg, Positive-Sequence-Voltage, Angle"
      m_inet_bus1 = "m:inet:bus1", "p.u., Current, Magnitude, referred to network"
      m_I_bus1 = "m:I:bus1", "kA, Current, Magnitude"
      m_phii_bus1 = "m:phii:bus1", "deg, Current, Angle"
      m_i1r_bus1 = "m:i1r:bus1", "p.u., Positive-Sequence Current, Real Part"
      m_i1i_bus1 = "m:i1i:bus1", "p.u., Positive-Sequence Current, Imaginary Part"
      m_i1_bus1 = "m:i1:bus1", "p.u., Positive-Sequence Current, Magnitude"
      m_I1_bus1 = "m:I1:bus1", "kA, Positive-Sequence Current, Magnitude"
      m_phii1_bus1 = "m:phii1:bus1", "deg, Positive-Sequence Current, Angle"
      m_P_bus1 = "m:P:bus1", "MW, Active Power"
      m_Q_bus1 = "m:Q:bus1", "Mvar, Reactive Power"
      m_S_bus1 = "m:S:bus1", "MVA, Apparent Power"
      m_cosphi_bus1 = "m:cosphi:bus1", "Power Factor"
      m_tanphi_bus1 = "m:tanphi:bus1", "tan(phi)"
      m_Psum_bus1 = "m:Psum:bus1", "MW, Total Active Power"
      m_Qsum_bus1 = "m:Qsum:bus1", "Mvar, Total Reactive Power"
      m_Ssum_bus1 = "m:Ssum:bus1", "MVA, Total Apparent Power"
      m_cosphisum_bus1 = "m:cosphisum:bus1", "Total Power Factor"
      m_tanphisum_bus1 = "m:tanphisum:bus1", "Total tan(phi)"
      m_Spu_bus1 = "m:Spu:bus1", "MVA/p.u., Apparent Power per p.u. Voltage"
      m_i1P_bus1 = "m:i1P:bus1", "p.u., Positive-Sequence Active Current"
      m_i1Q_bus1 = "m:i1Q:bus1", "p.u., Positive-Sequence Reactive Current"
      m_I1P_bus1 = "m:I1P:bus1", "kA, Positive-Sequence Active Current"
      m_I1Q_bus1 = "m:I1Q:bus1", "kA, Positive-Sequence Reactive Current"
      m_ur_bus1 = "m:ur:bus1", "p.u., Voltage, Real Part"
      m_ui_bus1 = "m:ui:bus1", "p.u., Voltage, Imaginary Part"
      m_u_bus1 = "m:u:bus1", "p.u., Voltage, Magnitude"
      m_ir_bus1 = "m:ir:bus1", "p.u., Current, Real Part"
      m_ii_bus1 = "m:ii:bus1", "p.u., Current, Imaginary Part"
      m_i_bus1 = "m:i:bus1", "p.u., Current, Magnitude"
      m_phiui_bus1 = "m:phiui:bus1", "deg, Angle between Voltage and Current"
      m_phiu1i1_bus1 = "m:phiu1i1:bus1", "deg, Angle between Voltage and Current in positive sequence system"
      m_Tfct_bus1 = "m:Tfct:bus1", "s, Fault Clearing Time"
      m_Brkload_bus1 = "m:Brkload:bus1", "%, Breaker Loading"
      n_ur_bus1 = "n:ur:bus1", "p.u., Voltage, Real Part"
      n_ui_bus1 = "n:ui:bus1", "p.u., Voltage, Imaginary Part"
      n_u_bus1 = "n:u:bus1", "p.u., Voltage, Magnitude"
      n_upc_bus1 = "n:upc:bus1", "%, Voltage, Magnitude"
      n_u1_bus1 = "n:u1:bus1", "p.u., Voltage, Magnitude"
      n_u1pc_bus1 = "n:u1pc:bus1", "%, Voltage, Magnitude"
      n_u1r_bus1 = "n:u1r:bus1", "p.u., Positive-Sequence Voltage, Real Part"
      n_u1i_bus1 = "n:u1i:bus1", "p.u., Positive-Sequence Voltage, Imaginary Part"
      n_U_bus1 = "n:U:bus1", "kV, Line-Ground Voltage, Magnitude"
      n_Ul_bus1 = "n:Ul:bus1", "kV, Line-Line Voltage, Magnitude"
      n_phiu_bus1 = "n:phiu:bus1", "deg, Voltage, Angle"
      n_phiurel_bus1 = "n:phiurel:bus1", "deg, Voltage, Relative Angle"
      n_du_bus1 = "n:du:bus1", "%, Voltage Deviation"
      n_Pgen_bus1 = "n:Pgen:bus1", "MW, Generation, Active Power"
      n_Qgen_bus1 = "n:Qgen:bus1", "Mvar, Generation, Reactive Power"
      n_Pmot_bus1 = "n:Pmot:bus1", "MW, Motor Load, Active Power"
      n_Qmot_bus1 = "n:Qmot:bus1", "Mvar, Motor Load, Reactive Power"
      n_Pload_bus1 = "n:Pload:bus1", "MW, General Load, Active Power"
      n_Qload_bus1 = "n:Qload:bus1", "Mvar, General Load, Reactive Power"
      n_Pcomp_bus1 = "n:Pcomp:bus1", "MW, Compensation (Losses)"
      n_Qcomp_bus1 = "n:Qcomp:bus1", "Mvar, Compensation"
      n_Pnet_bus1 = "n:Pnet:bus1", "MW, External Networks, Active Power"
      n_Qnet_bus1 = "n:Qnet:bus1", "Mvar, External Networks, Reactive Power"
      n_Pflow_bus1 = "n:Pflow:bus1", "MW, Power Flow, Active Power"
      n_Qflow_bus1 = "n:Qflow:bus1", "Mvar, Power Flow, Reactive Power"
      n_Pout_bus1 = "n:Pout:bus1", "MW, Outgoing Flow, Active Power"
      n_Qout_bus1 = "n:Qout:bus1", "Mvar, Outgoing Power, Reactive Power"
      n_Sout_bus1 = "n:Sout:bus1", "MVA, Outgoing Power, Apparent Power"
      n_cosphiout_bus1 = "n:cosphiout:bus1", "Outgoing Power, Power Factor"
      n_Pbalance_bus1 = "n:Pbalance:bus1", "MW, Active Power Balance (=0)"
      n_Qbalance_bus1 = "n:Qbalance:bus1", "Mvar, Reactive Power Balance (=0)"
      n_LossPdown_bus1 = "n:LossPdown:bus1", "MW, Losses, downstream"
      n_LossQdown_bus1 = "n:LossQdown:bus1", "Mvar, Losses, downstream (Reactive Power)"
      n_LossPdownload_bus1 = "n:LossPdownload:bus1", "MW, Load losses, downstream"
      n_LossQdownload_bus1 = "n:LossQdownload:bus1", "Mvar, Load losses, downstream"
      n_LossPdownnoload_bus1 = "n:LossPdownnoload:bus1", "MW, No load losses, downstream"
      n_LossQdownnoload_bus1 = "n:LossQdownnoload:bus1", "Mvar, No load losses, downstream"
      n_du_feed_bus1 = "n:du_feed:bus1", "%, Voltage difference relative to feeder begin"
      s_pset = "s:pset", "MW, IN, Active Power Setpoint, continuous, double"
      s_slip = "s:slip", "%, OUT, Slip, continuous, double"
      s_rradd = "s:rradd", "p.u., OUT, Additional Rotor Resistance, continuous, double"
      c_cosn = "c:cosn", "Rated Power Factor"
      c_sgnn = "c:sgnn", "MVA, Total Nominal Apparent power (ngnum*sgn)"
      c_Pnom = "c:Pnom", "MW, Total Nominal Active power"
      c_p = "c:p", "MW, Active Power (Electrical)"
      c_q = "c:q", "MW, Reactive Power"
      c_qcomp = "c:qcomp", "Mvar, Reactive Power for Compensation"
      c_pm = "c:pm", "MW, Mechanical Power"
      c_rstr = "c:rstr", "p.u., Stator Resistance"
      c_xstr = "c:xstr", "p.u., Stator Reactance"
      c_xrtrA = "c:xrtrA", "p.u., Rotor Reactance A"
      c_xrtrB = "c:xrtrB", "p.u., Rotor Reactance B"
      c_xm = "c:xm", "p.u., Magnetising Reactance"
      c_xmrtr = "c:xmrtr", "p.u., Rotor Leakage Reactance"
      c_frnom = "c:frnom", "Hz, Nominal Frequency"
      c_loading = "c:loading", "%, Loading"
      c_i_con = "c:i_con", "flag"
      c_i_slip = "c:i_slip", "flag"
      c_slipn = "c:slipn", "%, Nominal Slip"
      c_slipset = "c:slipset", "%, Slip Setpoint"
      c_i_mot = "c:i_mot", "i_mot=0"
      c_mk = "c:mk", "Stalling Torque"
      c_xme = "c:xme", "p.u., Electrical Torque"
      c_pmset = "c:pmset", "MW, Mechanical Power Setpoint"
      c_xmtset = "c:xmtset", "p.u., Mechanical Torque Setpoint"
      c_rrtrA0 = "c:rrtrA0", "p.u., RrA0"
      c_xrtrA0 = "c:xrtrA0", "p.u., XrA0"
      c_rr0 = "c:rr0", "p.u., Rr0"
      c_xr0 = "c:xr0", "p.u., Xr0"
      c_rr1 = "c:rr1", "p.u., Rr1"
      c_xr1 = "c:xr1", "p.u., Xr1"
      c_zonewscale = "c:zonewscale", "Wind generation zone scaling factor"
      c_i_rem = "c:i_rem", "Remote Control"
      c_psiM = "c:psiM", "p.u., Magnetising Flux"
      c_ksat = "c:ksat", "Saturation Factor"
      c_xmsat = "c:xmsat", "p.u., Saturated Magnetising Reactance"
      c_Ploss = "c:Ploss", "MW, Losses (total)"
      c_Qloss = "c:Qloss", "Mvar, Reactive-Losses (total)"
      c_Plossld = "c:Plossld", "MW, Losses (load)"
      c_Qlossld = "c:Qlossld", "Mvar, Reactive-Losses (load)"
      c_Plossnld = "c:Plossnld", "MW, Losses (no load)"
      c_Qlossnld = "c:Qlossnld", "Mvar, Reactive-Losses (no load)"
      e_bustp = "e:bustp", "Bus type"
      e_pmode = "e:pmode", "Input mode"
      e_av_mode = "e:av_mode", "Local controller"
      e_outServPzero = "e:outServPzero", "Out of service when active power is zero"
      e_c_pstac = "e:c_pstac", "ElmStactrl, External station controller"
      e_qdslCtrl = "e:qdslCtrl", "Quasi-Dynamic Model"
      e_mode_pgi = "e:mode_pgi", "Model"
      e_pPowerCrv = "e:pPowerCrv", "TypPowercurve, Wind Power Curve"
      e_mode_inp = "e:mode_inp", "Input mode"
      e_pgini = "e:pgini", "MW, Active power"
      e_qgini = "e:qgini", "Mvar, Reactive power"
      e_sgini = "e:sgini", "MVA, Apparent power"
      e_cosgini = "e:cosgini", "Power factor"
      e_pf_recap = "e:pf_recap", "Power factor"
      e_p_direc = "e:p_direc", "Power direction"
      e_windspeed = "e:windspeed", "m/s, Wind speed"
      e_usetp = "e:usetp", "p.u., Voltage"
      e_Kpf = "e:Kpf", "MW/Hz, Prim. frequency bias"
      e_pgini_a = "e:pgini_a", "MW, Active Power (act.)"
      e_qgini_a = "e:qgini_a", "Mvar, Reactive Power (act.)"
      e_sgini_a = "e:sgini_a", "MVA, Apparent Power (act.)"
      e_cosgini_a = "e:cosgini_a", "Power Factor (act.)"
      e_pf_recap_a = "e:pf_recap_a", "Power Factor Ind/Cap (act.)"
      e_windspeed_a = "e:windspeed_a", "m/s, Wind speed (act.)"
      e_usp_min = "e:usp_min", "p.u., Min. voltage setpoint"
      e_usp_max = "e:usp_max", "p.u., Max. voltage setpoint"
      e_ddroop = "e:ddroop", "%, Droop"
      e_pQPcurve = "e:pQPcurve", "IntQpcurve, Q(P)-curve"
      e_pmini = "e:pmini", "MW, Mechanical power"
      e_xmtini = "e:xmtini", "kNm, Mechanical torque"
      e_xmtn = "e:xmtn", "kNm, Rated mechanical torque"
      e_iopt_slip = "e:iopt_slip", "Slip Settings"
      e_slipset = "e:slipset", "%, Slip"
      e_chr_p = "e:chr_p", "MW, Active Power"
      e_chr_slip = "e:chr_slip", "%, slip"
      e_cOperSpeed = "e:cOperSpeed", "p.u., Operating speed"
      e_pQlimType = "e:pQlimType", "IntQlim, Capability curve"
      e_q_min = "e:q_min", "p.u., Min."
      e_q_max = "e:q_max", "p.u., Max."
      e_cQ_min = "e:cQ_min", "Mvar, Minimum"
      e_cQ_max = "e:cQ_max", "Mvar, Maximum"
      e_scaleQmin = "e:scaleQmin", "%, Scaling factor (min.)"
      e_scaleQmax = "e:scaleQmax", "%, Scaling factor (max.)"
      e_Pmin_uc = "e:Pmin_uc", "MW, Min."
      e_Pmin_ucPU = "e:Pmin_ucPU", "p.u., Min."
      e_Pmax_uc = "e:Pmax_uc", "MW, Max."
      e_Pmax_ucPU = "e:Pmax_ucPU", "p.u., Max."
      e_Pnom = "e:Pnom", "MW, Pr(rated)"
      e_P_max = "e:P_max", "MW, Max."
      e_pmaxratf = "e:pmaxratf", "Rating factor"
      e_is4SpinReserve = "e:is4SpinReserve", "Consider for region spinning reserve"
      e_genShiftKey = "e:genShiftKey", "%, Generation shift key"
      e_i_rem = "e:i_rem", "Remote control"
      e_p_cub = "e:p_cub", "StaCubic*, Controlled branch (Cubicle)"
      e_pBMU = "e:pBMU", "ElmBmu, Virtual Power Plant"
      e_isDispatchable = "e:isDispatchable", "Generator Dispatch"
      e_priority = "e:priority", "Merit Order"
      e_fixed = "e:fixed", "Must run"
      e_cpFeed = "e:cpFeed", "Feeder"
      e_ciDist = "e:ciDist", "Distance from infeed in number of buses"
      e_ciLater = "e:ciLater", "Lateral Index"
      e_ciDistRoot = "e:ciDistRoot", "Distance from first infeed in number of buses"
      e_ciDistAll = "e:ciDistAll", "Distance from infeed in number of buses including switches"
      e_ciDistAllRoot = "e:ciDistAllRoot", "Distance from first infeed in number of buses including switches"
      r_c_pstac = "r:c_pstac", "ElmStactrl, External station controller"
      r_qdslCtrl = "r:qdslCtrl", "Quasi-Dynamic Model"
      r_pPowerCrv = "r:pPowerCrv", "TypPowercurve, Wind Power Curve"
      r_pQPcurve = "r:pQPcurve", "IntQpcurve, Q(P)-curve"
      r_pQlimType = "r:pQlimType", "IntQlim, Capability curve"
      r_p_cub = "r:p_cub", "StaCubic*, Controlled branch (Cubicle)"
      r_pBMU = "r:pBMU", "ElmBmu, Virtual Power Plant"
      r_cpFeed = "r:cpFeed", "Feeder"

    class ElmAsmsc(Enum):
      _init_ = 'value __doc__'

      m_u1r_bus1 = "m:u1r:bus1", "p.u., Positive-Sequence-Voltage, Real Part"
      m_u1r_bus2 = "m:u1r:bus2", "p.u., Positive-Sequence-Voltage, Real Part"
      m_u1i_bus1 = "m:u1i:bus1", "p.u., Positive-Sequence-Voltage, Imaginary Part"
      m_u1i_bus2 = "m:u1i:bus2", "p.u., Positive-Sequence-Voltage, Imaginary Part"
      m_u1_bus1 = "m:u1:bus1", "p.u., Positive-Sequence-Voltage, Magnitude"
      m_u1_bus2 = "m:u1:bus2", "p.u., Positive-Sequence-Voltage, Magnitude"
      m_U1_bus1 = "m:U1:bus1", "kV, Line-Ground Positive-Sequence-Voltage, Magnitude"
      m_U1_bus2 = "m:U1:bus2", "kV, Line-Ground Positive-Sequence-Voltage, Magnitude"
      m_U1l_bus1 = "m:U1l:bus1", "kV, Line-Line Positive-Sequence-Voltage, Magnitude"
      m_U1l_bus2 = "m:U1l:bus2", "kV, Line-Line Positive-Sequence-Voltage, Magnitude"
      m_phiu1_bus1 = "m:phiu1:bus1", "deg, Positive-Sequence-Voltage, Angle"
      m_phiu1_bus2 = "m:phiu1:bus2", "deg, Positive-Sequence-Voltage, Angle"
      m_inet_bus1 = "m:inet:bus1", "p.u., Current, Magnitude, referred to network"
      m_inet_bus2 = "m:inet:bus2", "p.u., Current, Magnitude, referred to network"
      m_I_bus1 = "m:I:bus1", "kA, Current, Magnitude"
      m_I_bus2 = "m:I:bus2", "kA, Current, Magnitude"
      m_phii_bus1 = "m:phii:bus1", "deg, Current, Angle"
      m_phii_bus2 = "m:phii:bus2", "deg, Current, Angle"
      m_i1r_bus1 = "m:i1r:bus1", "p.u., Positive-Sequence Current, Real Part"
      m_i1r_bus2 = "m:i1r:bus2", "p.u., Positive-Sequence Current, Real Part"
      m_i1i_bus1 = "m:i1i:bus1", "p.u., Positive-Sequence Current, Imaginary Part"
      m_i1i_bus2 = "m:i1i:bus2", "p.u., Positive-Sequence Current, Imaginary Part"
      m_i1_bus1 = "m:i1:bus1", "p.u., Positive-Sequence Current, Magnitude"
      m_i1_bus2 = "m:i1:bus2", "p.u., Positive-Sequence Current, Magnitude"
      m_I1_bus1 = "m:I1:bus1", "kA, Positive-Sequence Current, Magnitude"
      m_I1_bus2 = "m:I1:bus2", "kA, Positive-Sequence Current, Magnitude"
      m_phii1_bus1 = "m:phii1:bus1", "deg, Positive-Sequence Current, Angle"
      m_phii1_bus2 = "m:phii1:bus2", "deg, Positive-Sequence Current, Angle"
      m_P_bus1 = "m:P:bus1", "MW, Active Power"
      m_P_bus2 = "m:P:bus2", "MW, Active Power"
      m_Q_bus1 = "m:Q:bus1", "Mvar, Reactive Power"
      m_Q_bus2 = "m:Q:bus2", "Mvar, Reactive Power"
      m_S_bus1 = "m:S:bus1", "MVA, Apparent Power"
      m_S_bus2 = "m:S:bus2", "MVA, Apparent Power"
      m_cosphi_bus1 = "m:cosphi:bus1", "Power Factor"
      m_cosphi_bus2 = "m:cosphi:bus2", "Power Factor"
      m_tanphi_bus1 = "m:tanphi:bus1", "tan(phi)"
      m_tanphi_bus2 = "m:tanphi:bus2", "tan(phi)"
      m_Psum_bus1 = "m:Psum:bus1", "MW, Total Active Power"
      m_Psum_bus2 = "m:Psum:bus2", "MW, Total Active Power"
      m_Qsum_bus1 = "m:Qsum:bus1", "Mvar, Total Reactive Power"
      m_Qsum_bus2 = "m:Qsum:bus2", "Mvar, Total Reactive Power"
      m_Ssum_bus1 = "m:Ssum:bus1", "MVA, Total Apparent Power"
      m_Ssum_bus2 = "m:Ssum:bus2", "MVA, Total Apparent Power"
      m_cosphisum_bus1 = "m:cosphisum:bus1", "Total Power Factor"
      m_cosphisum_bus2 = "m:cosphisum:bus2", "Total Power Factor"
      m_tanphisum_bus1 = "m:tanphisum:bus1", "Total tan(phi)"
      m_tanphisum_bus2 = "m:tanphisum:bus2", "Total tan(phi)"
      m_Spu_bus1 = "m:Spu:bus1", "MVA/p.u., Apparent Power per p.u. Voltage"
      m_Spu_bus2 = "m:Spu:bus2", "MVA/p.u., Apparent Power per p.u. Voltage"
      m_i1P_bus1 = "m:i1P:bus1", "p.u., Positive-Sequence Active Current"
      m_i1P_bus2 = "m:i1P:bus2", "p.u., Positive-Sequence Active Current"
      m_i1Q_bus1 = "m:i1Q:bus1", "p.u., Positive-Sequence Reactive Current"
      m_i1Q_bus2 = "m:i1Q:bus2", "p.u., Positive-Sequence Reactive Current"
      m_I1P_bus1 = "m:I1P:bus1", "kA, Positive-Sequence Active Current"
      m_I1P_bus2 = "m:I1P:bus2", "kA, Positive-Sequence Active Current"
      m_I1Q_bus1 = "m:I1Q:bus1", "kA, Positive-Sequence Reactive Current"
      m_I1Q_bus2 = "m:I1Q:bus2", "kA, Positive-Sequence Reactive Current"
      m_ur_bus1 = "m:ur:bus1", "p.u., Voltage, Real Part"
      m_ur_bus2 = "m:ur:bus2", "p.u., Voltage, Real Part"
      m_ui_bus1 = "m:ui:bus1", "p.u., Voltage, Imaginary Part"
      m_ui_bus2 = "m:ui:bus2", "p.u., Voltage, Imaginary Part"
      m_u_bus1 = "m:u:bus1", "p.u., Voltage, Magnitude"
      m_u_bus2 = "m:u:bus2", "p.u., Voltage, Magnitude"
      m_ir_bus1 = "m:ir:bus1", "p.u., Current, Real Part"
      m_ir_bus2 = "m:ir:bus2", "p.u., Current, Real Part"
      m_ii_bus1 = "m:ii:bus1", "p.u., Current, Imaginary Part"
      m_ii_bus2 = "m:ii:bus2", "p.u., Current, Imaginary Part"
      m_i_bus1 = "m:i:bus1", "p.u., Current, Magnitude"
      m_i_bus2 = "m:i:bus2", "p.u., Current, Magnitude"
      m_phiui_bus1 = "m:phiui:bus1", "deg, Angle between Voltage and Current"
      m_phiui_bus2 = "m:phiui:bus2", "deg, Angle between Voltage and Current"
      m_phiu1i1_bus1 = "m:phiu1i1:bus1", "deg, Angle between Voltage and Current in positive sequence system"
      m_phiu1i1_bus2 = "m:phiu1i1:bus2", "deg, Angle between Voltage and Current in positive sequence system"
      m_Tfct_bus1 = "m:Tfct:bus1", "s, Fault Clearing Time"
      m_Tfct_bus2 = "m:Tfct:bus2", "s, Fault Clearing Time"
      m_Brkload_bus1 = "m:Brkload:bus1", "%, Breaker Loading"
      m_Brkload_bus2 = "m:Brkload:bus2", "%, Breaker Loading"
      n_ur_bus1 = "n:ur:bus1", "p.u., Voltage, Real Part"
      n_ur_bus2 = "n:ur:bus2", "p.u., Voltage, Real Part"
      n_ui_bus1 = "n:ui:bus1", "p.u., Voltage, Imaginary Part"
      n_ui_bus2 = "n:ui:bus2", "p.u., Voltage, Imaginary Part"
      n_u_bus1 = "n:u:bus1", "p.u., Voltage, Magnitude"
      n_u_bus2 = "n:u:bus2", "p.u., Voltage, Magnitude"
      n_upc_bus1 = "n:upc:bus1", "%, Voltage, Magnitude"
      n_upc_bus2 = "n:upc:bus2", "%, Voltage, Magnitude"
      n_u1_bus1 = "n:u1:bus1", "p.u., Voltage, Magnitude"
      n_u1_bus2 = "n:u1:bus2", "p.u., Voltage, Magnitude"
      n_u1pc_bus1 = "n:u1pc:bus1", "%, Voltage, Magnitude"
      n_u1pc_bus2 = "n:u1pc:bus2", "%, Voltage, Magnitude"
      n_u1r_bus1 = "n:u1r:bus1", "p.u., Positive-Sequence Voltage, Real Part"
      n_u1r_bus2 = "n:u1r:bus2", "p.u., Positive-Sequence Voltage, Real Part"
      n_u1i_bus1 = "n:u1i:bus1", "p.u., Positive-Sequence Voltage, Imaginary Part"
      n_u1i_bus2 = "n:u1i:bus2", "p.u., Positive-Sequence Voltage, Imaginary Part"
      n_U_bus1 = "n:U:bus1", "kV, Line-Ground Voltage, Magnitude"
      n_U_bus2 = "n:U:bus2", "kV, Line-Ground Voltage, Magnitude"
      n_Ul_bus1 = "n:Ul:bus1", "kV, Line-Line Voltage, Magnitude"
      n_Ul_bus2 = "n:Ul:bus2", "kV, Line-Line Voltage, Magnitude"
      n_phiu_bus1 = "n:phiu:bus1", "deg, Voltage, Angle"
      n_phiu_bus2 = "n:phiu:bus2", "deg, Voltage, Angle"
      n_phiurel_bus1 = "n:phiurel:bus1", "deg, Voltage, Relative Angle"
      n_phiurel_bus2 = "n:phiurel:bus2", "deg, Voltage, Relative Angle"
      n_du_bus1 = "n:du:bus1", "%, Voltage Deviation"
      n_du_bus2 = "n:du:bus2", "%, Voltage Deviation"
      n_Pgen_bus1 = "n:Pgen:bus1", "MW, Generation, Active Power"
      n_Pgen_bus2 = "n:Pgen:bus2", "MW, Generation, Active Power"
      n_Qgen_bus1 = "n:Qgen:bus1", "Mvar, Generation, Reactive Power"
      n_Qgen_bus2 = "n:Qgen:bus2", "Mvar, Generation, Reactive Power"
      n_Pmot_bus1 = "n:Pmot:bus1", "MW, Motor Load, Active Power"
      n_Pmot_bus2 = "n:Pmot:bus2", "MW, Motor Load, Active Power"
      n_Qmot_bus1 = "n:Qmot:bus1", "Mvar, Motor Load, Reactive Power"
      n_Qmot_bus2 = "n:Qmot:bus2", "Mvar, Motor Load, Reactive Power"
      n_Pload_bus1 = "n:Pload:bus1", "MW, General Load, Active Power"
      n_Pload_bus2 = "n:Pload:bus2", "MW, General Load, Active Power"
      n_Qload_bus1 = "n:Qload:bus1", "Mvar, General Load, Reactive Power"
      n_Qload_bus2 = "n:Qload:bus2", "Mvar, General Load, Reactive Power"
      n_Pcomp_bus1 = "n:Pcomp:bus1", "MW, Compensation (Losses)"
      n_Pcomp_bus2 = "n:Pcomp:bus2", "MW, Compensation (Losses)"
      n_Qcomp_bus1 = "n:Qcomp:bus1", "Mvar, Compensation"
      n_Qcomp_bus2 = "n:Qcomp:bus2", "Mvar, Compensation"
      n_Pnet_bus1 = "n:Pnet:bus1", "MW, External Networks, Active Power"
      n_Pnet_bus2 = "n:Pnet:bus2", "MW, External Networks, Active Power"
      n_Qnet_bus1 = "n:Qnet:bus1", "Mvar, External Networks, Reactive Power"
      n_Qnet_bus2 = "n:Qnet:bus2", "Mvar, External Networks, Reactive Power"
      n_Pflow_bus1 = "n:Pflow:bus1", "MW, Power Flow, Active Power"
      n_Pflow_bus2 = "n:Pflow:bus2", "MW, Power Flow, Active Power"
      n_Qflow_bus1 = "n:Qflow:bus1", "Mvar, Power Flow, Reactive Power"
      n_Qflow_bus2 = "n:Qflow:bus2", "Mvar, Power Flow, Reactive Power"
      n_Pout_bus1 = "n:Pout:bus1", "MW, Outgoing Flow, Active Power"
      n_Pout_bus2 = "n:Pout:bus2", "MW, Outgoing Flow, Active Power"
      n_Qout_bus1 = "n:Qout:bus1", "Mvar, Outgoing Power, Reactive Power"
      n_Qout_bus2 = "n:Qout:bus2", "Mvar, Outgoing Power, Reactive Power"
      n_Sout_bus1 = "n:Sout:bus1", "MVA, Outgoing Power, Apparent Power"
      n_Sout_bus2 = "n:Sout:bus2", "MVA, Outgoing Power, Apparent Power"
      n_cosphiout_bus1 = "n:cosphiout:bus1", "Outgoing Power, Power Factor"
      n_cosphiout_bus2 = "n:cosphiout:bus2", "Outgoing Power, Power Factor"
      n_Pbalance_bus1 = "n:Pbalance:bus1", "MW, Active Power Balance (=0)"
      n_Pbalance_bus2 = "n:Pbalance:bus2", "MW, Active Power Balance (=0)"
      n_Qbalance_bus1 = "n:Qbalance:bus1", "Mvar, Reactive Power Balance (=0)"
      n_Qbalance_bus2 = "n:Qbalance:bus2", "Mvar, Reactive Power Balance (=0)"
      n_LossPdown_bus1 = "n:LossPdown:bus1", "MW, Losses, downstream"
      n_LossPdown_bus2 = "n:LossPdown:bus2", "MW, Losses, downstream"
      n_LossQdown_bus1 = "n:LossQdown:bus1", "Mvar, Losses, downstream (Reactive Power)"
      n_LossQdown_bus2 = "n:LossQdown:bus2", "Mvar, Losses, downstream (Reactive Power)"
      n_LossPdownload_bus1 = "n:LossPdownload:bus1", "MW, Load losses, downstream"
      n_LossPdownload_bus2 = "n:LossPdownload:bus2", "MW, Load losses, downstream"
      n_LossQdownload_bus1 = "n:LossQdownload:bus1", "Mvar, Load losses, downstream"
      n_LossQdownload_bus2 = "n:LossQdownload:bus2", "Mvar, Load losses, downstream"
      n_LossPdownnoload_bus1 = "n:LossPdownnoload:bus1", "MW, No load losses, downstream"
      n_LossPdownnoload_bus2 = "n:LossPdownnoload:bus2", "MW, No load losses, downstream"
      n_LossQdownnoload_bus1 = "n:LossQdownnoload:bus1", "Mvar, No load losses, downstream"
      n_LossQdownnoload_bus2 = "n:LossQdownnoload:bus2", "Mvar, No load losses, downstream"
      n_du_feed_bus1 = "n:du_feed:bus1", "%, Voltage difference relative to feeder begin"
      n_du_feed_bus2 = "n:du_feed:bus2", "%, Voltage difference relative to feeder begin"
      s_usr = "s:usr", "p.u., STATE, Slip-ring Voltage, Real Part, continuous, double"
      s_usi = "s:usi", "p.u., STATE, Slip-ring Voltage, Imaginary Part, continuous, double"
      c_cosn = "c:cosn", "Rated Power Factor"
      c_sgnn = "c:sgnn", "MVA, Total rated apparent power (ngnum*sgn)"
      c_Pnom = "c:Pnom", "MW, Total Nominal Active power"
      c_p = "c:p", "MW, Active Power (Electrical)"
      c_q = "c:q", "Mvar, Reactive Power"
      c_rstr = "c:rstr", "p.u., Stator Resistance"
      c_xstr = "c:xstr", "p.u., Stator Reactance"
      c_xrtrA = "c:xrtrA", "p.u., Rotor Reactance A"
      c_xrtrB = "c:xrtrB", "p.u., Rotor Reactance B"
      c_xm = "c:xm", "p.u., Magnetising Reactance"
      c_xmrtr = "c:xmrtr", "p.u., Rotor Leakage Reactance"
      c_frnom = "c:frnom", "Hz, Nominal Frequency"
      c_loading = "c:loading", "%, Loading"
      c_i_con = "c:i_con", "flag"
      c_i_slip = "c:i_slip", "flag"
      c_slipn = "c:slipn", "%, Nominal Slip"
      c_i_mot = "c:i_mot", "i_mot=0"
      c_mk = "c:mk", "Stalling Torque"
      c_xme = "c:xme", "p.u., Electrical Torque"
      c_i_cdisp = "c:i_cdisp", "Current Displacement"
      c_rrtrA0 = "c:rrtrA0", "p.u., RrA0"
      c_xrtrA0 = "c:xrtrA0", "p.u., XrA0"
      c_rr0 = "c:rr0", "p.u., Rr0"
      c_xr0 = "c:xr0", "p.u., Xr0"
      c_rr1 = "c:rr1", "p.u., Rr1"
      c_xr1 = "c:xr1", "p.u., Xr1"
      c_Pmr = "c:Pmr", "Amplification, Real Part"
      c_Pmi = "c:Pmi", "Amplification, Imaginary Part"
      c_slip = "c:slip", "%, Slip"
      c_Unrot = "c:Unrot", "kV, Rated Slip Ring Voltage"
      c_zonewscale = "c:zonewscale", "Wind generation zone scaling factor"
      c_psiM = "c:psiM", "p.u., Magnetising Flux"
      c_ksat = "c:ksat", "Saturation Factor"
      c_xmsat = "c:xmsat", "p.u., Saturated Magnetising Reactance"
      c_Ploss = "c:Ploss", "MW, Losses (total)"
      c_Qloss = "c:Qloss", "Mvar, Reactive-Losses (total)"
      c_Plossld = "c:Plossld", "MW, Losses (load)"
      c_Qlossld = "c:Qlossld", "Mvar, Reactive-Losses (load)"
      c_Plossnld = "c:Plossnld", "MW, Losses (no load)"
      c_Qlossnld = "c:Qlossnld", "Mvar, Reactive-Losses (no load)"
      e_pgini = "e:pgini", "MW, Active Power"
      e_qgini = "e:qgini", "Mvar, Reactive Power"
      e_p_pctrl = "e:p_pctrl", "StaCubic*, Controlled Flow"
      e_slipset = "e:slipset", "%, Slip"
      e_Urot = "e:Urot", "V, Rated Slip Ring Voltage"
      e_cOperSpeed = "e:cOperSpeed", "p.u., Operating speed"
      e_cpFeed = "e:cpFeed", "Feeder"
      e_ciDist = "e:ciDist", "Distance from infeed in number of buses"
      e_ciLater = "e:ciLater", "Lateral Index"
      e_ciDistRoot = "e:ciDistRoot", "Distance from first infeed in number of buses"
      e_ciDistAll = "e:ciDistAll", "Distance from infeed in number of buses including switches"
      e_ciDistAllRoot = "e:ciDistAllRoot", "Distance from first infeed in number of buses including switches"
      r_p_pctrl = "r:p_pctrl", "StaCubic*, Controlled Flow"
      r_cpFeed = "r:cpFeed", "Feeder"

    class ElmBoundary(Enum):
      _init_ = 'value __doc__'

      c_Pinter = "c:Pinter", "MW, Active Power Interchange"
      c_Qinter = "c:Qinter", "Mvar, Reactive Power Interchange"
      c_ExportP = "c:ExportP", "MW, Export, Active Power"
      c_ExportQ = "c:ExportQ", "Mvar, Export, Reactive Power"
      c_ImportP = "c:ImportP", "MW, Import, Active Power"
      c_ImportQ = "c:ImportQ", "Mvar, Import, Reactive Power"
      e_iInterChg = "e:iInterChg", "Consider Interchange Schedule"
      e_InterPset = "e:InterPset", "MW, Scheduled Active Power Interchange"

    class ElmCoup(Enum):
      _init_ = 'value __doc__'

      m_u1r_bus1 = "m:u1r:bus1", "p.u., Positive-Sequence-Voltage, Real Part"
      m_u1r_bus2 = "m:u1r:bus2", "p.u., Positive-Sequence-Voltage, Real Part"
      m_u1i_bus1 = "m:u1i:bus1", "p.u., Positive-Sequence-Voltage, Imaginary Part"
      m_u1i_bus2 = "m:u1i:bus2", "p.u., Positive-Sequence-Voltage, Imaginary Part"
      m_u1_bus1 = "m:u1:bus1", "p.u., Positive-Sequence-Voltage, Magnitude"
      m_u1_bus2 = "m:u1:bus2", "p.u., Positive-Sequence-Voltage, Magnitude"
      m_U1_bus1 = "m:U1:bus1", "kV, Line-Ground Positive-Sequence-Voltage, Magnitude"
      m_U1_bus2 = "m:U1:bus2", "kV, Line-Ground Positive-Sequence-Voltage, Magnitude"
      m_U1l_bus1 = "m:U1l:bus1", "kV, Line-Line Positive-Sequence-Voltage, Magnitude"
      m_U1l_bus2 = "m:U1l:bus2", "kV, Line-Line Positive-Sequence-Voltage, Magnitude"
      m_phiu1_bus1 = "m:phiu1:bus1", "deg, Positive-Sequence-Voltage, Angle"
      m_phiu1_bus2 = "m:phiu1:bus2", "deg, Positive-Sequence-Voltage, Angle"
      m_inet_bus1 = "m:inet:bus1", "p.u., Current, Magnitude, referred to network"
      m_inet_bus2 = "m:inet:bus2", "p.u., Current, Magnitude, referred to network"
      m_I_bus1 = "m:I:bus1", "kA, Current, Magnitude"
      m_I_bus2 = "m:I:bus2", "kA, Current, Magnitude"
      m_phii_bus1 = "m:phii:bus1", "deg, Current, Angle"
      m_phii_bus2 = "m:phii:bus2", "deg, Current, Angle"
      m_i1r_bus1 = "m:i1r:bus1", "p.u., Positive-Sequence Current, Real Part"
      m_i1r_bus2 = "m:i1r:bus2", "p.u., Positive-Sequence Current, Real Part"
      m_i1i_bus1 = "m:i1i:bus1", "p.u., Positive-Sequence Current, Imaginary Part"
      m_i1i_bus2 = "m:i1i:bus2", "p.u., Positive-Sequence Current, Imaginary Part"
      m_i1_bus1 = "m:i1:bus1", "p.u., Positive-Sequence Current, Magnitude"
      m_i1_bus2 = "m:i1:bus2", "p.u., Positive-Sequence Current, Magnitude"
      m_I1_bus1 = "m:I1:bus1", "kA, Positive-Sequence Current, Magnitude"
      m_I1_bus2 = "m:I1:bus2", "kA, Positive-Sequence Current, Magnitude"
      m_phii1_bus1 = "m:phii1:bus1", "deg, Positive-Sequence Current, Angle"
      m_phii1_bus2 = "m:phii1:bus2", "deg, Positive-Sequence Current, Angle"
      m_P_bus1 = "m:P:bus1", "MW, Active Power"
      m_P_bus2 = "m:P:bus2", "MW, Active Power"
      m_Q_bus1 = "m:Q:bus1", "Mvar, Reactive Power"
      m_Q_bus2 = "m:Q:bus2", "Mvar, Reactive Power"
      m_S_bus1 = "m:S:bus1", "MVA, Apparent Power"
      m_S_bus2 = "m:S:bus2", "MVA, Apparent Power"
      m_cosphi_bus1 = "m:cosphi:bus1", "Power Factor"
      m_cosphi_bus2 = "m:cosphi:bus2", "Power Factor"
      m_tanphi_bus1 = "m:tanphi:bus1", "tan(phi)"
      m_tanphi_bus2 = "m:tanphi:bus2", "tan(phi)"
      m_Psum_bus1 = "m:Psum:bus1", "MW, Total Active Power"
      m_Psum_bus2 = "m:Psum:bus2", "MW, Total Active Power"
      m_Qsum_bus1 = "m:Qsum:bus1", "Mvar, Total Reactive Power"
      m_Qsum_bus2 = "m:Qsum:bus2", "Mvar, Total Reactive Power"
      m_Ssum_bus1 = "m:Ssum:bus1", "MVA, Total Apparent Power"
      m_Ssum_bus2 = "m:Ssum:bus2", "MVA, Total Apparent Power"
      m_cosphisum_bus1 = "m:cosphisum:bus1", "Total Power Factor"
      m_cosphisum_bus2 = "m:cosphisum:bus2", "Total Power Factor"
      m_tanphisum_bus1 = "m:tanphisum:bus1", "Total tan(phi)"
      m_tanphisum_bus2 = "m:tanphisum:bus2", "Total tan(phi)"
      m_Spu_bus1 = "m:Spu:bus1", "MVA/p.u., Apparent Power per p.u. Voltage"
      m_Spu_bus2 = "m:Spu:bus2", "MVA/p.u., Apparent Power per p.u. Voltage"
      m_i1P_bus1 = "m:i1P:bus1", "p.u., Positive-Sequence Active Current"
      m_i1P_bus2 = "m:i1P:bus2", "p.u., Positive-Sequence Active Current"
      m_i1Q_bus1 = "m:i1Q:bus1", "p.u., Positive-Sequence Reactive Current"
      m_i1Q_bus2 = "m:i1Q:bus2", "p.u., Positive-Sequence Reactive Current"
      m_I1P_bus1 = "m:I1P:bus1", "kA, Positive-Sequence Active Current"
      m_I1P_bus2 = "m:I1P:bus2", "kA, Positive-Sequence Active Current"
      m_I1Q_bus1 = "m:I1Q:bus1", "kA, Positive-Sequence Reactive Current"
      m_I1Q_bus2 = "m:I1Q:bus2", "kA, Positive-Sequence Reactive Current"
      m_ur_bus1 = "m:ur:bus1", "p.u., Voltage, Real Part"
      m_ur_bus2 = "m:ur:bus2", "p.u., Voltage, Real Part"
      m_ui_bus1 = "m:ui:bus1", "p.u., Voltage, Imaginary Part"
      m_ui_bus2 = "m:ui:bus2", "p.u., Voltage, Imaginary Part"
      m_u_bus1 = "m:u:bus1", "p.u., Voltage, Magnitude"
      m_u_bus2 = "m:u:bus2", "p.u., Voltage, Magnitude"
      m_ir_bus1 = "m:ir:bus1", "p.u., Current, Real Part"
      m_ir_bus2 = "m:ir:bus2", "p.u., Current, Real Part"
      m_ii_bus1 = "m:ii:bus1", "p.u., Current, Imaginary Part"
      m_ii_bus2 = "m:ii:bus2", "p.u., Current, Imaginary Part"
      m_i_bus1 = "m:i:bus1", "p.u., Current, Magnitude"
      m_i_bus2 = "m:i:bus2", "p.u., Current, Magnitude"
      m_phiui_bus1 = "m:phiui:bus1", "deg, Angle between Voltage and Current"
      m_phiui_bus2 = "m:phiui:bus2", "deg, Angle between Voltage and Current"
      m_phiu1i1_bus1 = "m:phiu1i1:bus1", "deg, Angle between Voltage and Current in positive sequence system"
      m_phiu1i1_bus2 = "m:phiu1i1:bus2", "deg, Angle between Voltage and Current in positive sequence system"
      m_Tfct_bus1 = "m:Tfct:bus1", "s, Fault Clearing Time"
      m_Tfct_bus2 = "m:Tfct:bus2", "s, Fault Clearing Time"
      m_Brkload_bus1 = "m:Brkload:bus1", "%, Breaker Loading"
      m_Brkload_bus2 = "m:Brkload:bus2", "%, Breaker Loading"
      n_ur_bus1 = "n:ur:bus1", "p.u., Voltage, Real Part"
      n_ur_bus2 = "n:ur:bus2", "p.u., Voltage, Real Part"
      n_ui_bus1 = "n:ui:bus1", "p.u., Voltage, Imaginary Part"
      n_ui_bus2 = "n:ui:bus2", "p.u., Voltage, Imaginary Part"
      n_u_bus1 = "n:u:bus1", "p.u., Voltage, Magnitude"
      n_u_bus2 = "n:u:bus2", "p.u., Voltage, Magnitude"
      n_upc_bus1 = "n:upc:bus1", "%, Voltage, Magnitude"
      n_upc_bus2 = "n:upc:bus2", "%, Voltage, Magnitude"
      n_u1_bus1 = "n:u1:bus1", "p.u., Voltage, Magnitude"
      n_u1_bus2 = "n:u1:bus2", "p.u., Voltage, Magnitude"
      n_u1pc_bus1 = "n:u1pc:bus1", "%, Voltage, Magnitude"
      n_u1pc_bus2 = "n:u1pc:bus2", "%, Voltage, Magnitude"
      n_u1r_bus1 = "n:u1r:bus1", "p.u., Positive-Sequence Voltage, Real Part"
      n_u1r_bus2 = "n:u1r:bus2", "p.u., Positive-Sequence Voltage, Real Part"
      n_u1i_bus1 = "n:u1i:bus1", "p.u., Positive-Sequence Voltage, Imaginary Part"
      n_u1i_bus2 = "n:u1i:bus2", "p.u., Positive-Sequence Voltage, Imaginary Part"
      n_U_bus1 = "n:U:bus1", "kV, Line-Ground Voltage, Magnitude"
      n_U_bus2 = "n:U:bus2", "kV, Line-Ground Voltage, Magnitude"
      n_Ul_bus1 = "n:Ul:bus1", "kV, Line-Line Voltage, Magnitude"
      n_Ul_bus2 = "n:Ul:bus2", "kV, Line-Line Voltage, Magnitude"
      n_phiu_bus1 = "n:phiu:bus1", "deg, Voltage, Angle"
      n_phiu_bus2 = "n:phiu:bus2", "deg, Voltage, Angle"
      n_phiurel_bus1 = "n:phiurel:bus1", "deg, Voltage, Relative Angle"
      n_phiurel_bus2 = "n:phiurel:bus2", "deg, Voltage, Relative Angle"
      n_du_bus1 = "n:du:bus1", "%, Voltage Deviation"
      n_du_bus2 = "n:du:bus2", "%, Voltage Deviation"
      n_Pgen_bus1 = "n:Pgen:bus1", "MW, Generation, Active Power"
      n_Pgen_bus2 = "n:Pgen:bus2", "MW, Generation, Active Power"
      n_Qgen_bus1 = "n:Qgen:bus1", "Mvar, Generation, Reactive Power"
      n_Qgen_bus2 = "n:Qgen:bus2", "Mvar, Generation, Reactive Power"
      n_Pmot_bus1 = "n:Pmot:bus1", "MW, Motor Load, Active Power"
      n_Pmot_bus2 = "n:Pmot:bus2", "MW, Motor Load, Active Power"
      n_Qmot_bus1 = "n:Qmot:bus1", "Mvar, Motor Load, Reactive Power"
      n_Qmot_bus2 = "n:Qmot:bus2", "Mvar, Motor Load, Reactive Power"
      n_Pload_bus1 = "n:Pload:bus1", "MW, General Load, Active Power"
      n_Pload_bus2 = "n:Pload:bus2", "MW, General Load, Active Power"
      n_Qload_bus1 = "n:Qload:bus1", "Mvar, General Load, Reactive Power"
      n_Qload_bus2 = "n:Qload:bus2", "Mvar, General Load, Reactive Power"
      n_Pcomp_bus1 = "n:Pcomp:bus1", "MW, Compensation (Losses)"
      n_Pcomp_bus2 = "n:Pcomp:bus2", "MW, Compensation (Losses)"
      n_Qcomp_bus1 = "n:Qcomp:bus1", "Mvar, Compensation"
      n_Qcomp_bus2 = "n:Qcomp:bus2", "Mvar, Compensation"
      n_Pnet_bus1 = "n:Pnet:bus1", "MW, External Networks, Active Power"
      n_Pnet_bus2 = "n:Pnet:bus2", "MW, External Networks, Active Power"
      n_Qnet_bus1 = "n:Qnet:bus1", "Mvar, External Networks, Reactive Power"
      n_Qnet_bus2 = "n:Qnet:bus2", "Mvar, External Networks, Reactive Power"
      n_Pflow_bus1 = "n:Pflow:bus1", "MW, Power Flow, Active Power"
      n_Pflow_bus2 = "n:Pflow:bus2", "MW, Power Flow, Active Power"
      n_Qflow_bus1 = "n:Qflow:bus1", "Mvar, Power Flow, Reactive Power"
      n_Qflow_bus2 = "n:Qflow:bus2", "Mvar, Power Flow, Reactive Power"
      n_Pout_bus1 = "n:Pout:bus1", "MW, Outgoing Flow, Active Power"
      n_Pout_bus2 = "n:Pout:bus2", "MW, Outgoing Flow, Active Power"
      n_Qout_bus1 = "n:Qout:bus1", "Mvar, Outgoing Power, Reactive Power"
      n_Qout_bus2 = "n:Qout:bus2", "Mvar, Outgoing Power, Reactive Power"
      n_Sout_bus1 = "n:Sout:bus1", "MVA, Outgoing Power, Apparent Power"
      n_Sout_bus2 = "n:Sout:bus2", "MVA, Outgoing Power, Apparent Power"
      n_cosphiout_bus1 = "n:cosphiout:bus1", "Outgoing Power, Power Factor"
      n_cosphiout_bus2 = "n:cosphiout:bus2", "Outgoing Power, Power Factor"
      n_Pbalance_bus1 = "n:Pbalance:bus1", "MW, Active Power Balance (=0)"
      n_Pbalance_bus2 = "n:Pbalance:bus2", "MW, Active Power Balance (=0)"
      n_Qbalance_bus1 = "n:Qbalance:bus1", "Mvar, Reactive Power Balance (=0)"
      n_Qbalance_bus2 = "n:Qbalance:bus2", "Mvar, Reactive Power Balance (=0)"
      n_LossPdown_bus1 = "n:LossPdown:bus1", "MW, Losses, downstream"
      n_LossPdown_bus2 = "n:LossPdown:bus2", "MW, Losses, downstream"
      n_LossQdown_bus1 = "n:LossQdown:bus1", "Mvar, Losses, downstream (Reactive Power)"
      n_LossQdown_bus2 = "n:LossQdown:bus2", "Mvar, Losses, downstream (Reactive Power)"
      n_LossPdownload_bus1 = "n:LossPdownload:bus1", "MW, Load losses, downstream"
      n_LossPdownload_bus2 = "n:LossPdownload:bus2", "MW, Load losses, downstream"
      n_LossQdownload_bus1 = "n:LossQdownload:bus1", "Mvar, Load losses, downstream"
      n_LossQdownload_bus2 = "n:LossQdownload:bus2", "Mvar, Load losses, downstream"
      n_LossPdownnoload_bus1 = "n:LossPdownnoload:bus1", "MW, No load losses, downstream"
      n_LossPdownnoload_bus2 = "n:LossPdownnoload:bus2", "MW, No load losses, downstream"
      n_LossQdownnoload_bus1 = "n:LossQdownnoload:bus1", "Mvar, No load losses, downstream"
      n_LossQdownnoload_bus2 = "n:LossQdownnoload:bus2", "Mvar, No load losses, downstream"
      n_du_feed_bus1 = "n:du_feed:bus1", "%, Voltage difference relative to feeder begin"
      n_du_feed_bus2 = "n:du_feed:bus2", "%, Voltage difference relative to feeder begin"
      c_R_on = "c:R_on", "Ohm, On Resistance"
      c_X_on = "c:X_on", "Ohm, On Reactance"
      c_on_off = "c:on_off", "Flag Closed"
      c_Inom = "c:Inom", "kA, Rated Current"
      c_loading = "c:loading", "%, Loading"
      c_maxload = "c:maxload", "%, Max. Loading"
      c_Ploss = "c:Ploss", "MW, Losses (total)"
      c_Qloss = "c:Qloss", "Mvar, Reactive-Losses (total)"
      c_Plossld = "c:Plossld", "MW, Losses (load)"
      c_Qlossld = "c:Qlossld", "Mvar, Reactive-Losses (load)"
      c_Plossnld = "c:Plossnld", "MW, Losses (no load)"
      c_Qlossnld = "c:Qlossnld", "Mvar, Reactive-Losses (no load)"
      e_cpFeed = "e:cpFeed", "Feeder"
      e_ciDist = "e:ciDist", "Distance from infeed in number of buses"
      e_ciLater = "e:ciLater", "Lateral Index"
      e_ciDistRoot = "e:ciDistRoot", "Distance from first infeed in number of buses"
      e_ciDistAll = "e:ciDistAll", "Distance from infeed in number of buses including switches"
      e_ciDistAllRoot = "e:ciDistAllRoot", "Distance from first infeed in number of buses including switches"
      r_cpFeed = "r:cpFeed", "Feeder"

    class ElmGenstat(Enum):
      _init_ = 'value __doc__'

      m_u1r_bus1 = "m:u1r:bus1", "p.u., Positive-Sequence-Voltage, Real Part"
      m_u1i_bus1 = "m:u1i:bus1", "p.u., Positive-Sequence-Voltage, Imaginary Part"
      m_u1_bus1 = "m:u1:bus1", "p.u., Positive-Sequence-Voltage, Magnitude"
      m_U1_bus1 = "m:U1:bus1", "kV, Line-Ground Positive-Sequence-Voltage, Magnitude"
      m_U1l_bus1 = "m:U1l:bus1", "kV, Line-Line Positive-Sequence-Voltage, Magnitude"
      m_phiu1_bus1 = "m:phiu1:bus1", "deg, Positive-Sequence-Voltage, Angle"
      m_inet_bus1 = "m:inet:bus1", "p.u., Current, Magnitude, referred to network"
      m_I_bus1 = "m:I:bus1", "kA, Current, Magnitude"
      m_phii_bus1 = "m:phii:bus1", "deg, Current, Angle"
      m_i1r_bus1 = "m:i1r:bus1", "p.u., Positive-Sequence Current, Real Part"
      m_i1i_bus1 = "m:i1i:bus1", "p.u., Positive-Sequence Current, Imaginary Part"
      m_i1_bus1 = "m:i1:bus1", "p.u., Positive-Sequence Current, Magnitude"
      m_I1_bus1 = "m:I1:bus1", "kA, Positive-Sequence Current, Magnitude"
      m_phii1_bus1 = "m:phii1:bus1", "deg, Positive-Sequence Current, Angle"
      m_P_bus1 = "m:P:bus1", "MW, Active Power"
      m_Q_bus1 = "m:Q:bus1", "Mvar, Reactive Power"
      m_S_bus1 = "m:S:bus1", "MVA, Apparent Power"
      m_cosphi_bus1 = "m:cosphi:bus1", "Power Factor"
      m_tanphi_bus1 = "m:tanphi:bus1", "tan(phi)"
      m_Psum_bus1 = "m:Psum:bus1", "MW, Total Active Power"
      m_Qsum_bus1 = "m:Qsum:bus1", "Mvar, Total Reactive Power"
      m_Ssum_bus1 = "m:Ssum:bus1", "MVA, Total Apparent Power"
      m_cosphisum_bus1 = "m:cosphisum:bus1", "Total Power Factor"
      m_tanphisum_bus1 = "m:tanphisum:bus1", "Total tan(phi)"
      m_Spu_bus1 = "m:Spu:bus1", "MVA/p.u., Apparent Power per p.u. Voltage"
      m_i1P_bus1 = "m:i1P:bus1", "p.u., Positive-Sequence Active Current"
      m_i1Q_bus1 = "m:i1Q:bus1", "p.u., Positive-Sequence Reactive Current"
      m_I1P_bus1 = "m:I1P:bus1", "kA, Positive-Sequence Active Current"
      m_I1Q_bus1 = "m:I1Q:bus1", "kA, Positive-Sequence Reactive Current"
      m_ur_bus1 = "m:ur:bus1", "p.u., Voltage, Real Part"
      m_ui_bus1 = "m:ui:bus1", "p.u., Voltage, Imaginary Part"
      m_u_bus1 = "m:u:bus1", "p.u., Voltage, Magnitude"
      m_ir_bus1 = "m:ir:bus1", "p.u., Current, Real Part"
      m_ii_bus1 = "m:ii:bus1", "p.u., Current, Imaginary Part"
      m_i_bus1 = "m:i:bus1", "p.u., Current, Magnitude"
      m_phiui_bus1 = "m:phiui:bus1", "deg, Angle between Voltage and Current"
      m_phiu1i1_bus1 = "m:phiu1i1:bus1", "deg, Angle between Voltage and Current in positive sequence system"
      m_Tfct_bus1 = "m:Tfct:bus1", "s, Fault Clearing Time"
      m_Brkload_bus1 = "m:Brkload:bus1", "%, Breaker Loading"
      n_ur_bus1 = "n:ur:bus1", "p.u., Voltage, Real Part"
      n_ui_bus1 = "n:ui:bus1", "p.u., Voltage, Imaginary Part"
      n_u_bus1 = "n:u:bus1", "p.u., Voltage, Magnitude"
      n_upc_bus1 = "n:upc:bus1", "%, Voltage, Magnitude"
      n_u1_bus1 = "n:u1:bus1", "p.u., Voltage, Magnitude"
      n_u1pc_bus1 = "n:u1pc:bus1", "%, Voltage, Magnitude"
      n_u1r_bus1 = "n:u1r:bus1", "p.u., Positive-Sequence Voltage, Real Part"
      n_u1i_bus1 = "n:u1i:bus1", "p.u., Positive-Sequence Voltage, Imaginary Part"
      n_U_bus1 = "n:U:bus1", "kV, Line-Ground Voltage, Magnitude"
      n_Ul_bus1 = "n:Ul:bus1", "kV, Line-Line Voltage, Magnitude"
      n_phiu_bus1 = "n:phiu:bus1", "deg, Voltage, Angle"
      n_phiurel_bus1 = "n:phiurel:bus1", "deg, Voltage, Relative Angle"
      n_du_bus1 = "n:du:bus1", "%, Voltage Deviation"
      n_Pgen_bus1 = "n:Pgen:bus1", "MW, Generation, Active Power"
      n_Qgen_bus1 = "n:Qgen:bus1", "Mvar, Generation, Reactive Power"
      n_Pmot_bus1 = "n:Pmot:bus1", "MW, Motor Load, Active Power"
      n_Qmot_bus1 = "n:Qmot:bus1", "Mvar, Motor Load, Reactive Power"
      n_Pload_bus1 = "n:Pload:bus1", "MW, General Load, Active Power"
      n_Qload_bus1 = "n:Qload:bus1", "Mvar, General Load, Reactive Power"
      n_Pcomp_bus1 = "n:Pcomp:bus1", "MW, Compensation (Losses)"
      n_Qcomp_bus1 = "n:Qcomp:bus1", "Mvar, Compensation"
      n_Pnet_bus1 = "n:Pnet:bus1", "MW, External Networks, Active Power"
      n_Qnet_bus1 = "n:Qnet:bus1", "Mvar, External Networks, Reactive Power"
      n_Pflow_bus1 = "n:Pflow:bus1", "MW, Power Flow, Active Power"
      n_Qflow_bus1 = "n:Qflow:bus1", "Mvar, Power Flow, Reactive Power"
      n_Pout_bus1 = "n:Pout:bus1", "MW, Outgoing Flow, Active Power"
      n_Qout_bus1 = "n:Qout:bus1", "Mvar, Outgoing Power, Reactive Power"
      n_Sout_bus1 = "n:Sout:bus1", "MVA, Outgoing Power, Apparent Power"
      n_cosphiout_bus1 = "n:cosphiout:bus1", "Outgoing Power, Power Factor"
      n_Pbalance_bus1 = "n:Pbalance:bus1", "MW, Active Power Balance (=0)"
      n_Qbalance_bus1 = "n:Qbalance:bus1", "Mvar, Reactive Power Balance (=0)"
      n_LossPdown_bus1 = "n:LossPdown:bus1", "MW, Losses, downstream"
      n_LossQdown_bus1 = "n:LossQdown:bus1", "Mvar, Losses, downstream (Reactive Power)"
      n_LossPdownload_bus1 = "n:LossPdownload:bus1", "MW, Load losses, downstream"
      n_LossQdownload_bus1 = "n:LossQdownload:bus1", "Mvar, Load losses, downstream"
      n_LossPdownnoload_bus1 = "n:LossPdownnoload:bus1", "MW, No load losses, downstream"
      n_LossQdownnoload_bus1 = "n:LossQdownnoload:bus1", "Mvar, No load losses, downstream"
      n_du_feed_bus1 = "n:du_feed:bus1", "%, Voltage difference relative to feeder begin"
      s_pset = "s:pset", "MW, IN, Active Power Setpoint, continuous, double"
      s_qset = "s:qset", "Mvar, IN, Reactive Power Setpoint continuous, double"
      s_dFin = "s:dFin", "Hz, IN, Frequency Deviation, continuous, double"
      c_p = "c:p", "MW, Total Active Power"
      c_q = "c:q", "Mvar, Total Reactive Power"
      c_sgnn = "c:sgnn", "MVA, Total rated apparent power (ngnum*sgn)"
      c_cosn = "c:cosn", "Nominal-Power factor"
      c_Pnom = "c:Pnom", "MW, Total Nominal Active power"
      c_c2 = "c:c2", "Voltage Coefficient (uknom/ugn)"
      c_uini_r = "c:uini:r", "p.u., Voltage Setpoint for PV and Slack, Real Part"
      c_uini_i = "c:uini:i", "p.u., Voltage Setpoint for PV and Slack, Imaginary Part"
      c_uini_phi = "c:uini:phi", "deg, Voltage Setpoint for PV and Slack, Angle"
      c_uini_mag = "c:uini:mag", "p.u., Voltage Setpoint for PV and Slack, Magnitude"
      c_uini_phirad = "c:uini:phirad", "rad, Voltage Setpoint for PV and Slack, Angle"
      c_phiini = "c:phiini", "Initial Angle"
      c_loading = "c:loading", "%, Loading"
      c_iPowerOn = "c:iPowerOn", "Flag"
      c_Qmin = "c:Qmin", "Mvar, Lower limit of reactive power"
      c_Qmax = "c:Qmax", "Mvar, Upper limit of reactive power"
      c_Qsparelead = "c:Qsparelead", "Mvar, Q Spare Lead"
      c_Qsparelag = "c:Qsparelag", "Mvar, Q Spare Lag"
      c_i_qlim = "c:i_qlim", "Reactive power limit reached"
      c_i_vlim = "c:i_vlim", "Voltage setpoint limit reached"
      c_Pmin = "c:Pmin", "MW, Lower limit of active power"
      c_Pmax = "c:Pmax", "MW, Upper limit of active power"
      c_i_plim = "c:i_plim", "Active power limit reached"
      c_sm_r = "c:sm:r", "MW, Complex Power, Real Part"
      c_sm_i = "c:sm:i", "Mvar, Complex Power, Imaginary Part"
      c_sm_phi = "c:sm:phi", "deg, Complex Power, Angle"
      c_sm_mag = "c:sm:mag", "MVA, Complex Power, Magnitude"
      c_sm_phirad = "c:sm:phirad", "rad, Complex Power, Angle"
      c_u1_r = "c:u1:r", "p.u., Internal, Real Part"
      c_u1_i = "c:u1:i", "p.u., Internal, Imaginary Part"
      c_u1_phi = "c:u1:phi", "deg, Internal, Angle"
      c_u1_mag = "c:u1:mag", "p.u., Internal, Magnitude"
      c_u1_phirad = "c:u1:phirad", "rad, Internal, Angle"
      c_iRefElement = "c:iRefElement", "Reference Element"
      c_Snom = "c:Snom", "MVA, Nom. Apparent Power (1 mach.)"
      c_Nnom = "c:Nnom", "Number of parallel units"
      c_Kp = "c:Kp", "p.u., Active Power Participation"
      c_Kq = "c:Kq", "p.u., Reactive Power Participation"
      c_Kpf = "c:Kpf", "MW/Hz, Frequency Bias"
      c_zonewscale = "c:zonewscale", "Wind generation zone scaling factor"
      c_Pmism = "c:Pmism", "MW, Active Power Mismatch"
      c_nCtrlIter = "c:nCtrlIter", "No. of CTRL-iterations"
      c_Ploss = "c:Ploss", "MW, Losses (total)"
      c_Qloss = "c:Qloss", "Mvar, Reactive-Losses (total)"
      c_Plossld = "c:Plossld", "MW, Losses (load)"
      c_Qlossld = "c:Qlossld", "Mvar, Reactive-Losses (load)"
      c_Plossnld = "c:Plossnld", "MW, Losses (no load)"
      c_Qlossnld = "c:Qlossnld", "Mvar, Reactive-Losses (no load)"
      e_ip_ctrl = "e:ip_ctrl", "Reference Machine"
      e_outServPzero = "e:outServPzero", "Out of service when active power is zero"
      e_av_mode = "e:av_mode", "Local Controller"
      e_c_psecc = "e:c_psecc", "ElmSecctrl, External Secondary Controller"
      e_c_pstac = "e:c_pstac", "ElmStactrl, External Station Controller"
      e_qdslCtrl = "e:qdslCtrl", "Quasi-Dynamic Model"
      e_mode_pgi = "e:mode_pgi", "Model"
      e_pPowerCrv = "e:pPowerCrv", "TypPowercurve, Wind Power Curve"
      e_mode_inp = "e:mode_inp", "Input Mode"
      e_pgini = "e:pgini", "MW, Active Power"
      e_qgini = "e:qgini", "Mvar, Reactive Power"
      e_sgini = "e:sgini", "MVA, Apparent Power"
      e_cosgini = "e:cosgini", "Power Factor"
      e_pf_recap = "e:pf_recap", "Power Factor"
      e_p_direc = "e:p_direc", "Power Direction"
      e_windspeed = "e:windspeed", "m/s, Wind speed"
      e_usetp = "e:usetp", "p.u., Voltage"
      e_phiini = "e:phiini", "deg, Angle"
      e_Kpf = "e:Kpf", "MW/Hz, Prim. Frequency Bias"
      e_scale0 = "e:scale0", "Scaling Factor"
      e_pgini_a = "e:pgini_a", "MW, Active Power (act.)"
      e_qgini_a = "e:qgini_a", "Mvar, Reactive Power (act.)"
      e_sgini_a = "e:sgini_a", "MVA, Apparent Power (act.)"
      e_cosgini_a = "e:cosgini_a", "Power Factor (act.)"
      e_pf_recap_a = "e:pf_recap_a", "Power Factor Ind/Cap (act.)"
      e_scale0_a = "e:scale0_a", "Scaling Factor (act.)"
      e_windspeed_a = "e:windspeed_a", "m/s, Wind speed (act.)"
      e_usp_min = "e:usp_min", "p.u., Min. Voltage Setpoint"
      e_usp_max = "e:usp_max", "p.u., Max. Voltage Setpoint"
      e_ddroop = "e:ddroop", "%, Droop"
      e_pQPcurve = "e:pQPcurve", "IntQpcurve, Q(P)-Curve"
      e_iopt_tdr = "e:iopt_tdr", "Different droop values"
      e_ddroopoe = "e:ddroopoe", "%, Droop (overexcited)"
      e_ddroopue = "e:ddroopue", "%, Droop (underexcited)"
      e_Qfu_max = "e:Qfu_max", "Mvar, Q max"
      e_Qfu_min = "e:Qfu_min", "Mvar, Q min"
      e_udeadblow = "e:udeadblow", "p.u., Lower Voltage Limit"
      e_udeadbup = "e:udeadbup", "p.u., Upper Voltage Limit"
      e_pf_over = "e:pf_over", "Min. Power Factor"
      e_p_over = "e:p_over", "MW, Active Power"
      e_pf_under = "e:pf_under", "Min. Power Factor"
      e_p_under = "e:p_under", "MW, Active Power"
      e_pQlimType = "e:pQlimType", "IntQlim, Capability Curve"
      e_q_min = "e:q_min", "p.u., Min."
      e_q_max = "e:q_max", "p.u., Max."
      e_cQ_min = "e:cQ_min", "Mvar, Minimum"
      e_cQ_max = "e:cQ_max", "Mvar, Maximum"
      e_scaleQmin = "e:scaleQmin", "%, Scaling Factor (min.)"
      e_scaleQmax = "e:scaleQmax", "%, Scaling Factor (max.)"
      e_Pmin_uc = "e:Pmin_uc", "MW, Min."
      e_Pmin_ucPU = "e:Pmin_ucPU", "p.u., Min."
      e_Pmax_uc = "e:Pmax_uc", "MW, Max."
      e_Pmax_ucPU = "e:Pmax_ucPU", "p.u., Max."
      e_Pnom = "e:Pnom", "MW, Pr(rated)"
      e_P_max = "e:P_max", "MW, Max."
      e_pmaxratf = "e:pmaxratf", "Rating Factor"
      e_c_pCtrlHV = "e:c_pCtrlHV", "Controlled HV-busbar"
      e_Uctrl = "e:Uctrl", "kV, HV-Voltage Setpoint"
      e_Qmin_a = "e:Qmin_a", "Mvar, Qmin(act.)"
      e_Qmax_a = "e:Qmax_a", "Mvar, Qmax(act.)"
      e_Pmin_a = "e:Pmin_a", "MW, Pmin(act.)"
      e_Pmax_a = "e:Pmax_a", "MW, Pmax(act.)"
      e_r2 = "e:r2", "p.u., Resistance r2"
      e_x2 = "e:x2", "p.u., Reactance x2"
      e_is4SpinReserve = "e:is4SpinReserve", "Consider for region spinning reserve"
      e_genShiftKey = "e:genShiftKey", "%, Generation shift key"
      e_pBMU = "e:pBMU", "ElmBmu, Virtual Power Plant"
      e_dispatch = "e:dispatch", "Generator Dispatch"
      e_priority = "e:priority", "Merit Order"
      e_fixed = "e:fixed", "Must run"
      r_c_psecc = "r:c_psecc", "ElmSecctrl, External Secondary Controller"
      r_c_pstac = "r:c_pstac", "ElmStactrl, External Station Controller"
      r_qdslCtrl = "r:qdslCtrl", "Quasi-Dynamic Model"
      r_pPowerCrv = "r:pPowerCrv", "TypPowercurve, Wind Power Curve"
      r_pQPcurve = "r:pQPcurve", "IntQpcurve, Q(P)-Curve"
      r_pQlimType = "r:pQlimType", "IntQlim, Capability Curve"
      r_c_pCtrlHV = "r:c_pCtrlHV", "Controlled HV-busbar"
      r_pBMU = "r:pBMU", "ElmBmu, Virtual Power Plant"

    class ElmIac(Enum):
      _init_ = 'value __doc__'

      m_u1r_bus1 = "m:u1r:bus1", "p.u., Positive-Sequence-Voltage, Real Part"
      m_u1i_bus1 = "m:u1i:bus1", "p.u., Positive-Sequence-Voltage, Imaginary Part"
      m_u1_bus1 = "m:u1:bus1", "p.u., Positive-Sequence-Voltage, Magnitude"
      m_U1_bus1 = "m:U1:bus1", "kV, Line-Ground Positive-Sequence-Voltage, Magnitude"
      m_U1l_bus1 = "m:U1l:bus1", "kV, Line-Line Positive-Sequence-Voltage, Magnitude"
      m_phiu1_bus1 = "m:phiu1:bus1", "deg, Positive-Sequence-Voltage, Angle"
      m_inet_bus1 = "m:inet:bus1", "p.u., Current, Magnitude, referred to network"
      m_I_bus1 = "m:I:bus1", "kA, Current, Magnitude"
      m_phii_bus1 = "m:phii:bus1", "deg, Current, Angle"
      m_i1r_bus1 = "m:i1r:bus1", "p.u., Positive-Sequence Current, Real Part"
      m_i1i_bus1 = "m:i1i:bus1", "p.u., Positive-Sequence Current, Imaginary Part"
      m_i1_bus1 = "m:i1:bus1", "p.u., Positive-Sequence Current, Magnitude"
      m_I1_bus1 = "m:I1:bus1", "kA, Positive-Sequence Current, Magnitude"
      m_phii1_bus1 = "m:phii1:bus1", "deg, Positive-Sequence Current, Angle"
      m_P_bus1 = "m:P:bus1", "MW, Active Power"
      m_Q_bus1 = "m:Q:bus1", "Mvar, Reactive Power"
      m_S_bus1 = "m:S:bus1", "MVA, Apparent Power"
      m_cosphi_bus1 = "m:cosphi:bus1", "Power Factor"
      m_tanphi_bus1 = "m:tanphi:bus1", "tan(phi)"
      m_Psum_bus1 = "m:Psum:bus1", "MW, Total Active Power"
      m_Qsum_bus1 = "m:Qsum:bus1", "Mvar, Total Reactive Power"
      m_Ssum_bus1 = "m:Ssum:bus1", "MVA, Total Apparent Power"
      m_cosphisum_bus1 = "m:cosphisum:bus1", "Total Power Factor"
      m_tanphisum_bus1 = "m:tanphisum:bus1", "Total tan(phi)"
      m_Spu_bus1 = "m:Spu:bus1", "MVA/p.u., Apparent Power per p.u. Voltage"
      m_i1P_bus1 = "m:i1P:bus1", "p.u., Positive-Sequence Active Current"
      m_i1Q_bus1 = "m:i1Q:bus1", "p.u., Positive-Sequence Reactive Current"
      m_I1P_bus1 = "m:I1P:bus1", "kA, Positive-Sequence Active Current"
      m_I1Q_bus1 = "m:I1Q:bus1", "kA, Positive-Sequence Reactive Current"
      m_ur_bus1 = "m:ur:bus1", "p.u., Voltage, Real Part"
      m_ui_bus1 = "m:ui:bus1", "p.u., Voltage, Imaginary Part"
      m_u_bus1 = "m:u:bus1", "p.u., Voltage, Magnitude"
      m_ir_bus1 = "m:ir:bus1", "p.u., Current, Real Part"
      m_ii_bus1 = "m:ii:bus1", "p.u., Current, Imaginary Part"
      m_i_bus1 = "m:i:bus1", "p.u., Current, Magnitude"
      m_phiui_bus1 = "m:phiui:bus1", "deg, Angle between Voltage and Current"
      m_phiu1i1_bus1 = "m:phiu1i1:bus1", "deg, Angle between Voltage and Current in positive sequence system"
      m_Tfct_bus1 = "m:Tfct:bus1", "s, Fault Clearing Time"
      m_Brkload_bus1 = "m:Brkload:bus1", "%, Breaker Loading"
      n_ur_bus1 = "n:ur:bus1", "p.u., Voltage, Real Part"
      n_ui_bus1 = "n:ui:bus1", "p.u., Voltage, Imaginary Part"
      n_u_bus1 = "n:u:bus1", "p.u., Voltage, Magnitude"
      n_upc_bus1 = "n:upc:bus1", "%, Voltage, Magnitude"
      n_u1_bus1 = "n:u1:bus1", "p.u., Voltage, Magnitude"
      n_u1pc_bus1 = "n:u1pc:bus1", "%, Voltage, Magnitude"
      n_u1r_bus1 = "n:u1r:bus1", "p.u., Positive-Sequence Voltage, Real Part"
      n_u1i_bus1 = "n:u1i:bus1", "p.u., Positive-Sequence Voltage, Imaginary Part"
      n_U_bus1 = "n:U:bus1", "kV, Line-Ground Voltage, Magnitude"
      n_Ul_bus1 = "n:Ul:bus1", "kV, Line-Line Voltage, Magnitude"
      n_phiu_bus1 = "n:phiu:bus1", "deg, Voltage, Angle"
      n_phiurel_bus1 = "n:phiurel:bus1", "deg, Voltage, Relative Angle"
      n_du_bus1 = "n:du:bus1", "%, Voltage Deviation"
      n_Pgen_bus1 = "n:Pgen:bus1", "MW, Generation, Active Power"
      n_Qgen_bus1 = "n:Qgen:bus1", "Mvar, Generation, Reactive Power"
      n_Pmot_bus1 = "n:Pmot:bus1", "MW, Motor Load, Active Power"
      n_Qmot_bus1 = "n:Qmot:bus1", "Mvar, Motor Load, Reactive Power"
      n_Pload_bus1 = "n:Pload:bus1", "MW, General Load, Active Power"
      n_Qload_bus1 = "n:Qload:bus1", "Mvar, General Load, Reactive Power"
      n_Pcomp_bus1 = "n:Pcomp:bus1", "MW, Compensation (Losses)"
      n_Qcomp_bus1 = "n:Qcomp:bus1", "Mvar, Compensation"
      n_Pnet_bus1 = "n:Pnet:bus1", "MW, External Networks, Active Power"
      n_Qnet_bus1 = "n:Qnet:bus1", "Mvar, External Networks, Reactive Power"
      n_Pflow_bus1 = "n:Pflow:bus1", "MW, Power Flow, Active Power"
      n_Qflow_bus1 = "n:Qflow:bus1", "Mvar, Power Flow, Reactive Power"
      n_Pout_bus1 = "n:Pout:bus1", "MW, Outgoing Flow, Active Power"
      n_Qout_bus1 = "n:Qout:bus1", "Mvar, Outgoing Power, Reactive Power"
      n_Sout_bus1 = "n:Sout:bus1", "MVA, Outgoing Power, Apparent Power"
      n_cosphiout_bus1 = "n:cosphiout:bus1", "Outgoing Power, Power Factor"
      n_Pbalance_bus1 = "n:Pbalance:bus1", "MW, Active Power Balance (=0)"
      n_Qbalance_bus1 = "n:Qbalance:bus1", "Mvar, Reactive Power Balance (=0)"
      n_LossPdown_bus1 = "n:LossPdown:bus1", "MW, Losses, downstream"
      n_LossQdown_bus1 = "n:LossQdown:bus1", "Mvar, Losses, downstream (Reactive Power)"
      n_LossPdownload_bus1 = "n:LossPdownload:bus1", "MW, Load losses, downstream"
      n_LossQdownload_bus1 = "n:LossQdownload:bus1", "Mvar, Load losses, downstream"
      n_LossPdownnoload_bus1 = "n:LossPdownnoload:bus1", "MW, No load losses, downstream"
      n_LossQdownnoload_bus1 = "n:LossQdownnoload:bus1", "Mvar, No load losses, downstream"
      n_du_feed_bus1 = "n:du_feed:bus1", "%, Voltage difference relative to feeder begin"
      s_i0 = "s:i0", "p.u., IN, Current-Input, continuous, double"
      s_phiui = "s:phiui", "rad, IN, Angle-Input (u,i), continuous, double"
      c_Ir = "c:Ir", "A, Nominal Current"
      c_isetp = "c:isetp", "p.u., Current Setpoint"
      c_cosini = "c:cosini", "Power Factor"
      c_Y1_r = "c:Y1:r", "S, Internal Admittance, Real Part"
      c_Y1_i = "c:Y1:i", "S, Internal Admittance, Imaginary Part"
      c_Y1_phi = "c:Y1:phi", "deg, Internal Admittance, Angle"
      c_Y1_mag = "c:Y1:mag", "S, Internal Admittance, Magnitude"
      c_Y1_phirad = "c:Y1:phirad", "rad, Internal Admittance, Angle"
      c_Ploss = "c:Ploss", "MW, Losses (total)"
      c_Qloss = "c:Qloss", "Mvar, Reactive-Losses (total)"
      c_Plossld = "c:Plossld", "MW, Losses (load)"
      c_Qlossld = "c:Qlossld", "Mvar, Reactive-Losses (load)"
      c_Plossnld = "c:Plossnld", "MW, Losses (no load)"
      c_Qlossnld = "c:Qlossnld", "Mvar, Reactive-Losses (no load)"
      e_isetp = "e:isetp", "p.u., Current Setpoint"
      e_cosini = "e:cosini", "Power Factor"
      e_i_cap = "e:i_cap", "Reactive Power"
      e_G1 = "e:G1", "S, Internal Conductance, G1"
      e_B1 = "e:B1", "S, Internal Susceptance, B1"
      e_isetp2 = "e:isetp2", "p.u., Current, Magnitude"
      e_phisetp2 = "e:phisetp2", "deg, Current, Angle"
      e_G2 = "e:G2", "S, Internal Conductance, G2"
      e_B2 = "e:B2", "S, Internal Susceptance, B2"
      e_isetp0 = "e:isetp0", "p.u., Current, Magnitude"
      e_phisetp0 = "e:phisetp0", "deg, Current, Angle"
      e_G0 = "e:G0", "S, Internal Conductance, G0"
      e_B0 = "e:B0", "S, Internal Susceptance, B0"
      e_qdslCtrl = "e:qdslCtrl", "Quasi-Dynamic Model"
      e_cpFeed = "e:cpFeed", "Feeder"
      e_ciDist = "e:ciDist", "Distance from infeed in number of buses"
      e_ciLater = "e:ciLater", "Lateral Index"
      e_ciDistRoot = "e:ciDistRoot", "Distance from first infeed in number of buses"
      e_ciDistAll = "e:ciDistAll", "Distance from infeed in number of buses including switches"
      e_ciDistAllRoot = "e:ciDistAllRoot", "Distance from first infeed in number of buses including switches"
      r_qdslCtrl = "r:qdslCtrl", "Quasi-Dynamic Model"
      r_cpFeed = "r:cpFeed", "Feeder"

    class ElmLne(Enum):
      _init_ = 'value __doc__'

      m_u1r_bus1 = "m:u1r:bus1", "p.u., Positive-Sequence-Voltage, Real Part"
      m_u1r_bus2 = "m:u1r:bus2", "p.u., Positive-Sequence-Voltage, Real Part"
      m_u1i_bus1 = "m:u1i:bus1", "p.u., Positive-Sequence-Voltage, Imaginary Part"
      m_u1i_bus2 = "m:u1i:bus2", "p.u., Positive-Sequence-Voltage, Imaginary Part"
      m_u1_bus1 = "m:u1:bus1", "p.u., Positive-Sequence-Voltage, Magnitude"
      m_u1_bus2 = "m:u1:bus2", "p.u., Positive-Sequence-Voltage, Magnitude"
      m_U1_bus1 = "m:U1:bus1", "kV, Line-Ground Positive-Sequence-Voltage, Magnitude"
      m_U1_bus2 = "m:U1:bus2", "kV, Line-Ground Positive-Sequence-Voltage, Magnitude"
      m_U1l_bus1 = "m:U1l:bus1", "kV, Line-Line Positive-Sequence-Voltage, Magnitude"
      m_U1l_bus2 = "m:U1l:bus2", "kV, Line-Line Positive-Sequence-Voltage, Magnitude"
      m_phiu1_bus1 = "m:phiu1:bus1", "deg, Positive-Sequence-Voltage, Angle"
      m_phiu1_bus2 = "m:phiu1:bus2", "deg, Positive-Sequence-Voltage, Angle"
      m_inet_bus1 = "m:inet:bus1", "p.u., Current, Magnitude, referred to network"
      m_inet_bus2 = "m:inet:bus2", "p.u., Current, Magnitude, referred to network"
      m_I_bus1 = "m:I:bus1", "kA, Current, Magnitude"
      m_I_bus2 = "m:I:bus2", "kA, Current, Magnitude"
      m_phii_bus1 = "m:phii:bus1", "deg, Current, Angle"
      m_phii_bus2 = "m:phii:bus2", "deg, Current, Angle"
      m_i1r_bus1 = "m:i1r:bus1", "p.u., Positive-Sequence Current, Real Part"
      m_i1r_bus2 = "m:i1r:bus2", "p.u., Positive-Sequence Current, Real Part"
      m_i1i_bus1 = "m:i1i:bus1", "p.u., Positive-Sequence Current, Imaginary Part"
      m_i1i_bus2 = "m:i1i:bus2", "p.u., Positive-Sequence Current, Imaginary Part"
      m_i1_bus1 = "m:i1:bus1", "p.u., Positive-Sequence Current, Magnitude"
      m_i1_bus2 = "m:i1:bus2", "p.u., Positive-Sequence Current, Magnitude"
      m_I1_bus1 = "m:I1:bus1", "kA, Positive-Sequence Current, Magnitude"
      m_I1_bus2 = "m:I1:bus2", "kA, Positive-Sequence Current, Magnitude"
      m_phii1_bus1 = "m:phii1:bus1", "deg, Positive-Sequence Current, Angle"
      m_phii1_bus2 = "m:phii1:bus2", "deg, Positive-Sequence Current, Angle"
      m_P_bus1 = "m:P:bus1", "MW, Active Power"
      m_P_bus2 = "m:P:bus2", "MW, Active Power"
      m_Q_bus1 = "m:Q:bus1", "Mvar, Reactive Power"
      m_Q_bus2 = "m:Q:bus2", "Mvar, Reactive Power"
      m_S_bus1 = "m:S:bus1", "MVA, Apparent Power"
      m_S_bus2 = "m:S:bus2", "MVA, Apparent Power"
      m_cosphi_bus1 = "m:cosphi:bus1", "Power Factor"
      m_cosphi_bus2 = "m:cosphi:bus2", "Power Factor"
      m_tanphi_bus1 = "m:tanphi:bus1", "tan(phi)"
      m_tanphi_bus2 = "m:tanphi:bus2", "tan(phi)"
      m_Psum_bus1 = "m:Psum:bus1", "MW, Total Active Power"
      m_Psum_bus2 = "m:Psum:bus2", "MW, Total Active Power"
      m_Qsum_bus1 = "m:Qsum:bus1", "Mvar, Total Reactive Power"
      m_Qsum_bus2 = "m:Qsum:bus2", "Mvar, Total Reactive Power"
      m_Ssum_bus1 = "m:Ssum:bus1", "MVA, Total Apparent Power"
      m_Ssum_bus2 = "m:Ssum:bus2", "MVA, Total Apparent Power"
      m_cosphisum_bus1 = "m:cosphisum:bus1", "Total Power Factor"
      m_cosphisum_bus2 = "m:cosphisum:bus2", "Total Power Factor"
      m_tanphisum_bus1 = "m:tanphisum:bus1", "Total tan(phi)"
      m_tanphisum_bus2 = "m:tanphisum:bus2", "Total tan(phi)"
      m_Spu_bus1 = "m:Spu:bus1", "MVA/p.u., Apparent Power per p.u. Voltage"
      m_Spu_bus2 = "m:Spu:bus2", "MVA/p.u., Apparent Power per p.u. Voltage"
      m_i1P_bus1 = "m:i1P:bus1", "p.u., Positive-Sequence Active Current"
      m_i1P_bus2 = "m:i1P:bus2", "p.u., Positive-Sequence Active Current"
      m_i1Q_bus1 = "m:i1Q:bus1", "p.u., Positive-Sequence Reactive Current"
      m_i1Q_bus2 = "m:i1Q:bus2", "p.u., Positive-Sequence Reactive Current"
      m_I1P_bus1 = "m:I1P:bus1", "kA, Positive-Sequence Active Current"
      m_I1P_bus2 = "m:I1P:bus2", "kA, Positive-Sequence Active Current"
      m_I1Q_bus1 = "m:I1Q:bus1", "kA, Positive-Sequence Reactive Current"
      m_I1Q_bus2 = "m:I1Q:bus2", "kA, Positive-Sequence Reactive Current"
      m_ur_bus1 = "m:ur:bus1", "p.u., Voltage, Real Part"
      m_ur_bus2 = "m:ur:bus2", "p.u., Voltage, Real Part"
      m_ui_bus1 = "m:ui:bus1", "p.u., Voltage, Imaginary Part"
      m_ui_bus2 = "m:ui:bus2", "p.u., Voltage, Imaginary Part"
      m_u_bus1 = "m:u:bus1", "p.u., Voltage, Magnitude"
      m_u_bus2 = "m:u:bus2", "p.u., Voltage, Magnitude"
      m_ir_bus1 = "m:ir:bus1", "p.u., Current, Real Part"
      m_ir_bus2 = "m:ir:bus2", "p.u., Current, Real Part"
      m_ii_bus1 = "m:ii:bus1", "p.u., Current, Imaginary Part"
      m_ii_bus2 = "m:ii:bus2", "p.u., Current, Imaginary Part"
      m_i_bus1 = "m:i:bus1", "p.u., Current, Magnitude"
      m_i_bus2 = "m:i:bus2", "p.u., Current, Magnitude"
      m_phiui_bus1 = "m:phiui:bus1", "deg, Angle between Voltage and Current"
      m_phiui_bus2 = "m:phiui:bus2", "deg, Angle between Voltage and Current"
      m_phiu1i1_bus1 = "m:phiu1i1:bus1", "deg, Angle between Voltage and Current in positive sequence system"
      m_phiu1i1_bus2 = "m:phiu1i1:bus2", "deg, Angle between Voltage and Current in positive sequence system"
      m_Tfct_bus1 = "m:Tfct:bus1", "s, Fault Clearing Time"
      m_Tfct_bus2 = "m:Tfct:bus2", "s, Fault Clearing Time"
      m_Brkload_bus1 = "m:Brkload:bus1", "%, Breaker Loading"
      m_Brkload_bus2 = "m:Brkload:bus2", "%, Breaker Loading"
      n_ur_bus1 = "n:ur:bus1", "p.u., Voltage, Real Part"
      n_ur_bus2 = "n:ur:bus2", "p.u., Voltage, Real Part"
      n_ui_bus1 = "n:ui:bus1", "p.u., Voltage, Imaginary Part"
      n_ui_bus2 = "n:ui:bus2", "p.u., Voltage, Imaginary Part"
      n_u_bus1 = "n:u:bus1", "p.u., Voltage, Magnitude"
      n_u_bus2 = "n:u:bus2", "p.u., Voltage, Magnitude"
      n_upc_bus1 = "n:upc:bus1", "%, Voltage, Magnitude"
      n_upc_bus2 = "n:upc:bus2", "%, Voltage, Magnitude"
      n_u1_bus1 = "n:u1:bus1", "p.u., Voltage, Magnitude"
      n_u1_bus2 = "n:u1:bus2", "p.u., Voltage, Magnitude"
      n_u1pc_bus1 = "n:u1pc:bus1", "%, Voltage, Magnitude"
      n_u1pc_bus2 = "n:u1pc:bus2", "%, Voltage, Magnitude"
      n_u1r_bus1 = "n:u1r:bus1", "p.u., Positive-Sequence Voltage, Real Part"
      n_u1r_bus2 = "n:u1r:bus2", "p.u., Positive-Sequence Voltage, Real Part"
      n_u1i_bus1 = "n:u1i:bus1", "p.u., Positive-Sequence Voltage, Imaginary Part"
      n_u1i_bus2 = "n:u1i:bus2", "p.u., Positive-Sequence Voltage, Imaginary Part"
      n_U_bus1 = "n:U:bus1", "kV, Line-Ground Voltage, Magnitude"
      n_U_bus2 = "n:U:bus2", "kV, Line-Ground Voltage, Magnitude"
      n_Ul_bus1 = "n:Ul:bus1", "kV, Line-Line Voltage, Magnitude"
      n_Ul_bus2 = "n:Ul:bus2", "kV, Line-Line Voltage, Magnitude"
      n_phiu_bus1 = "n:phiu:bus1", "deg, Voltage, Angle"
      n_phiu_bus2 = "n:phiu:bus2", "deg, Voltage, Angle"
      n_phiurel_bus1 = "n:phiurel:bus1", "deg, Voltage, Relative Angle"
      n_phiurel_bus2 = "n:phiurel:bus2", "deg, Voltage, Relative Angle"
      n_du_bus1 = "n:du:bus1", "%, Voltage Deviation"
      n_du_bus2 = "n:du:bus2", "%, Voltage Deviation"
      n_Pgen_bus1 = "n:Pgen:bus1", "MW, Generation, Active Power"
      n_Pgen_bus2 = "n:Pgen:bus2", "MW, Generation, Active Power"
      n_Qgen_bus1 = "n:Qgen:bus1", "Mvar, Generation, Reactive Power"
      n_Qgen_bus2 = "n:Qgen:bus2", "Mvar, Generation, Reactive Power"
      n_Pmot_bus1 = "n:Pmot:bus1", "MW, Motor Load, Active Power"
      n_Pmot_bus2 = "n:Pmot:bus2", "MW, Motor Load, Active Power"
      n_Qmot_bus1 = "n:Qmot:bus1", "Mvar, Motor Load, Reactive Power"
      n_Qmot_bus2 = "n:Qmot:bus2", "Mvar, Motor Load, Reactive Power"
      n_Pload_bus1 = "n:Pload:bus1", "MW, General Load, Active Power"
      n_Pload_bus2 = "n:Pload:bus2", "MW, General Load, Active Power"
      n_Qload_bus1 = "n:Qload:bus1", "Mvar, General Load, Reactive Power"
      n_Qload_bus2 = "n:Qload:bus2", "Mvar, General Load, Reactive Power"
      n_Pcomp_bus1 = "n:Pcomp:bus1", "MW, Compensation (Losses)"
      n_Pcomp_bus2 = "n:Pcomp:bus2", "MW, Compensation (Losses)"
      n_Qcomp_bus1 = "n:Qcomp:bus1", "Mvar, Compensation"
      n_Qcomp_bus2 = "n:Qcomp:bus2", "Mvar, Compensation"
      n_Pnet_bus1 = "n:Pnet:bus1", "MW, External Networks, Active Power"
      n_Pnet_bus2 = "n:Pnet:bus2", "MW, External Networks, Active Power"
      n_Qnet_bus1 = "n:Qnet:bus1", "Mvar, External Networks, Reactive Power"
      n_Qnet_bus2 = "n:Qnet:bus2", "Mvar, External Networks, Reactive Power"
      n_Pflow_bus1 = "n:Pflow:bus1", "MW, Power Flow, Active Power"
      n_Pflow_bus2 = "n:Pflow:bus2", "MW, Power Flow, Active Power"
      n_Qflow_bus1 = "n:Qflow:bus1", "Mvar, Power Flow, Reactive Power"
      n_Qflow_bus2 = "n:Qflow:bus2", "Mvar, Power Flow, Reactive Power"
      n_Pout_bus1 = "n:Pout:bus1", "MW, Outgoing Flow, Active Power"
      n_Pout_bus2 = "n:Pout:bus2", "MW, Outgoing Flow, Active Power"
      n_Qout_bus1 = "n:Qout:bus1", "Mvar, Outgoing Power, Reactive Power"
      n_Qout_bus2 = "n:Qout:bus2", "Mvar, Outgoing Power, Reactive Power"
      n_Sout_bus1 = "n:Sout:bus1", "MVA, Outgoing Power, Apparent Power"
      n_Sout_bus2 = "n:Sout:bus2", "MVA, Outgoing Power, Apparent Power"
      n_cosphiout_bus1 = "n:cosphiout:bus1", "Outgoing Power, Power Factor"
      n_cosphiout_bus2 = "n:cosphiout:bus2", "Outgoing Power, Power Factor"
      n_Pbalance_bus1 = "n:Pbalance:bus1", "MW, Active Power Balance (=0)"
      n_Pbalance_bus2 = "n:Pbalance:bus2", "MW, Active Power Balance (=0)"
      n_Qbalance_bus1 = "n:Qbalance:bus1", "Mvar, Reactive Power Balance (=0)"
      n_Qbalance_bus2 = "n:Qbalance:bus2", "Mvar, Reactive Power Balance (=0)"
      n_LossPdown_bus1 = "n:LossPdown:bus1", "MW, Losses, downstream"
      n_LossPdown_bus2 = "n:LossPdown:bus2", "MW, Losses, downstream"
      n_LossQdown_bus1 = "n:LossQdown:bus1", "Mvar, Losses, downstream (Reactive Power)"
      n_LossQdown_bus2 = "n:LossQdown:bus2", "Mvar, Losses, downstream (Reactive Power)"
      n_LossPdownload_bus1 = "n:LossPdownload:bus1", "MW, Load losses, downstream"
      n_LossPdownload_bus2 = "n:LossPdownload:bus2", "MW, Load losses, downstream"
      n_LossQdownload_bus1 = "n:LossQdownload:bus1", "Mvar, Load losses, downstream"
      n_LossQdownload_bus2 = "n:LossQdownload:bus2", "Mvar, Load losses, downstream"
      n_LossPdownnoload_bus1 = "n:LossPdownnoload:bus1", "MW, No load losses, downstream"
      n_LossPdownnoload_bus2 = "n:LossPdownnoload:bus2", "MW, No load losses, downstream"
      n_LossQdownnoload_bus1 = "n:LossQdownnoload:bus1", "Mvar, No load losses, downstream"
      n_LossQdownnoload_bus2 = "n:LossQdownnoload:bus2", "Mvar, No load losses, downstream"
      n_du_feed_bus1 = "n:du_feed:bus1", "%, Voltage difference relative to feeder begin"
      n_du_feed_bus2 = "n:du_feed:bus2", "%, Voltage difference relative to feeder begin"
      c_zline_r = "c:zline:r", "p.u., Impedance, Real Part"
      c_zline_i = "c:zline:i", "p.u., Impedance, Imaginary Part"
      c_zline_phi = "c:zline:phi", "deg, Impedance, Angle"
      c_zline_mag = "c:zline:mag", "p.u., Impedance, Magnitude"
      c_zline_phirad = "c:zline:phirad", "rad, Impedance, Angle"
      c_yline_r = "c:yline:r", "p.u., Admittance, Real Part"
      c_yline_i = "c:yline:i", "p.u., Admittance, Imaginary Part"
      c_yline_phi = "c:yline:phi", "deg, Admittance, Angle"
      c_yline_mag = "c:yline:mag", "p.u., Admittance, Magnitude"
      c_yline_phirad = "c:yline:phirad", "rad, Admittance, Angle"
      c_frnom = "c:frnom", "Hz, Nominal frequency"
      c_hpi = "c:hpi", "rad/s, Nominal angular frequency"
      c_loading = "c:loading", "%, Loading"
      c_curnom = "c:curnom", "kA, Nominal Current"
      c_rtfac = "c:rtfac", "Temperature Factor"
      c_maxloadpu = "c:maxloadpu", "p.u., Max. Loading"
      c_maxload = "c:maxload", "%, Max. Loading"
      c_du = "c:du", "p.u., Voltage Drop"
      c_dupc = "c:dupc", "%, Voltage Drop"
      c_dphiu = "c:dphiu", "deg, Voltage Drop Angle"
      c_du1 = "c:du1", "p.u., Positive Sequence Voltage Drop"
      c_du1pc = "c:du1pc", "%, Positive Sequence Voltage Drop"
      c_dphiu1 = "c:dphiu1", "deg, Positive Sequence Voltage Drop Angle"
      c_Cload = "c:Cload", "Mvar, Capacitive Losses"
      c_Gload = "c:Gload", "kW, Active no load losses"
      c_Losses = "c:Losses", "kW, Losses"
      c_Imax = "c:Imax", "kA, Maximum Current"
      c_Ploss = "c:Ploss", "MW, Losses (total)"
      c_Qloss = "c:Qloss", "Mvar, Reactive-Losses (total)"
      c_Plossld = "c:Plossld", "MW, Losses (load)"
      c_Qlossld = "c:Qlossld", "Mvar, Reactive-Losses (load)"
      c_Plossnld = "c:Plossnld", "MW, Losses (no load)"
      c_Qlossnld = "c:Qlossnld", "Mvar, Reactive-Losses (no load)"
      c_ncust = "c:ncust", "Number of Customers"
      c_Sl_r = "c:Sl:r", "MW, Load, variable Part, Apparent Power, Real Part"
      c_Sl_i = "c:Sl:i", "Mvar, Load, variable Part, Apparent Power, Imaginary Part"
      c_Sl_phi = "c:Sl:phi", "deg, Load, variable Part, Apparent Power, Angle"
      c_Sl_mag = "c:Sl:mag", "MVA, Load, variable Part, Apparent Power, Magnitude"
      c_Sl_phirad = "c:Sl:phirad", "rad, Load, variable Part, Apparent Power, Angle"
      c_ldcosphi = "c:ldcosphi", "Load, Variable Part, Power Factor"
      c_posld = "c:posld", "Weighted Load Position"
      c_ildlv = "c:ildlv", "Line Loads Present"
      c_Sldlv = "c:Sldlv", "kVA, Load, Apparent Power"
      c_Pldlv = "c:Pldlv", "kW, Load, Active Power"
      c_Qldlv = "c:Qldlv", "kvar, Load, Reactive Power"
      c_Ildlv = "c:Ildlv", "A, Load, Current"
      c_cosphildlv = "c:cosphildlv", "k, Load, Power Factor"
      e_cNrCust = "e:cNrCust", "Number of Customers"
      e_cSmax = "e:cSmax", "kVA, Max. Load"
      e_cSav = "e:cSav", "kVA, Average Load"
      e_ccosphi = "e:ccosphi", "Power Factor"
      e_maxload = "e:maxload", "%, Max. loading"
      e_deltau1 = "e:deltau1", "p.u., Voltage Difference across Line"
      e_deltaphiu1 = "e:deltaphiu1", "deg, Voltage Angle Difference across Line"
      e_lossAssign = "e:lossAssign", "Loss assignment"
      e_Top = "e:Top", "degC, Operating Temperature"
      e_Imaxlim = "e:Imaxlim", "kA, Max. permissible current"
      e_cpFeed = "e:cpFeed", "Feeder"
      e_ciDist = "e:ciDist", "Distance from infeed in number of buses"
      e_ciLater = "e:ciLater", "Lateral Index"
      e_ciDistRoot = "e:ciDistRoot", "Distance from first infeed in number of buses"
      e_ciDistAll = "e:ciDistAll", "Distance from infeed in number of buses including switches"
      e_ciDistAllRoot = "e:ciDistAllRoot", "Distance from first infeed in number of buses including switches"
      t_tmax = "t:tmax", "degC, Max. Operating Temperature"
      t_rline = "t:rline", "Ohm/km, AC-Resistance R'(20°C)"
      t_rline_tmax = "t:rline_tmax", "Ohm/km, AC-Resistance R' at max. operating temperature"
      t_alpha = "t:alpha", "1/K, Temperature Coefficient"
      t_mlei = "t:mlei", "Conductor Material"
      t_bline = "t:bline", "uS/km, Susceptance B'"
      t_cline = "t:cline", "uF/km, Capacitance C'"
      t_tline = "t:tline", "Ins. Factor"
      t_gline = "t:gline", "uS/km, Conductance G'"
      t_bline0 = "t:bline0", "uS/km, Susceptance B0'"
      t_cline0 = "t:cline0", "uF/km, Capacitance C0'"
      t_Ices = "t:Ices", "A/km, Earth-Fault Current"
      t_tline0 = "t:tline0", "Ins. Factor"
      t_gline0 = "t:gline0", "uS/km, Conductance G0'"
      t_bnline = "t:bnline", "uS/km, Susceptance Bn'"
      t_cnline = "t:cnline", "uF/km, Capacitance Cn'"
      t_bpnline = "t:bpnline", "uS/km, Susceptance Bpn'"
      t_cpnline = "t:cpnline", "uF/km, Capacitance Cpn'"
      r_cpFeed = "r:cpFeed", "Feeder"

    class ElmLod(Enum):
      _init_ = 'value __doc__'

      m_u1r_bus1 = "m:u1r:bus1", "p.u., Positive-Sequence-Voltage, Real Part"
      m_u1i_bus1 = "m:u1i:bus1", "p.u., Positive-Sequence-Voltage, Imaginary Part"
      m_u1_bus1 = "m:u1:bus1", "p.u., Positive-Sequence-Voltage, Magnitude"
      m_U1_bus1 = "m:U1:bus1", "kV, Line-Ground Positive-Sequence-Voltage, Magnitude"
      m_U1l_bus1 = "m:U1l:bus1", "kV, Line-Line Positive-Sequence-Voltage, Magnitude"
      m_phiu1_bus1 = "m:phiu1:bus1", "deg, Positive-Sequence-Voltage, Angle"
      m_inet_bus1 = "m:inet:bus1", "p.u., Current, Magnitude, referred to network"
      m_I_bus1 = "m:I:bus1", "kA, Current, Magnitude"
      m_phii_bus1 = "m:phii:bus1", "deg, Current, Angle"
      m_i1r_bus1 = "m:i1r:bus1", "p.u., Positive-Sequence Current, Real Part"
      m_i1i_bus1 = "m:i1i:bus1", "p.u., Positive-Sequence Current, Imaginary Part"
      m_i1_bus1 = "m:i1:bus1", "p.u., Positive-Sequence Current, Magnitude"
      m_I1_bus1 = "m:I1:bus1", "kA, Positive-Sequence Current, Magnitude"
      m_phii1_bus1 = "m:phii1:bus1", "deg, Positive-Sequence Current, Angle"
      m_P_bus1 = "m:P:bus1", "MW, Active Power"
      m_Q_bus1 = "m:Q:bus1", "Mvar, Reactive Power"
      m_S_bus1 = "m:S:bus1", "MVA, Apparent Power"
      m_cosphi_bus1 = "m:cosphi:bus1", "Power Factor"
      m_tanphi_bus1 = "m:tanphi:bus1", "tan(phi)"
      m_Psum_bus1 = "m:Psum:bus1", "MW, Total Active Power"
      m_Qsum_bus1 = "m:Qsum:bus1", "Mvar, Total Reactive Power"
      m_Ssum_bus1 = "m:Ssum:bus1", "MVA, Total Apparent Power"
      m_cosphisum_bus1 = "m:cosphisum:bus1", "Total Power Factor"
      m_tanphisum_bus1 = "m:tanphisum:bus1", "Total tan(phi)"
      m_Spu_bus1 = "m:Spu:bus1", "MVA/p.u., Apparent Power per p.u. Voltage"
      m_i1P_bus1 = "m:i1P:bus1", "p.u., Positive-Sequence Active Current"
      m_i1Q_bus1 = "m:i1Q:bus1", "p.u., Positive-Sequence Reactive Current"
      m_I1P_bus1 = "m:I1P:bus1", "kA, Positive-Sequence Active Current"
      m_I1Q_bus1 = "m:I1Q:bus1", "kA, Positive-Sequence Reactive Current"
      m_ur_bus1 = "m:ur:bus1", "p.u., Voltage, Real Part"
      m_ui_bus1 = "m:ui:bus1", "p.u., Voltage, Imaginary Part"
      m_u_bus1 = "m:u:bus1", "p.u., Voltage, Magnitude"
      m_ir_bus1 = "m:ir:bus1", "p.u., Current, Real Part"
      m_ii_bus1 = "m:ii:bus1", "p.u., Current, Imaginary Part"
      m_i_bus1 = "m:i:bus1", "p.u., Current, Magnitude"
      m_phiui_bus1 = "m:phiui:bus1", "deg, Angle between Voltage and Current"
      m_phiu1i1_bus1 = "m:phiu1i1:bus1", "deg, Angle between Voltage and Current in positive sequence system"
      m_Tfct_bus1 = "m:Tfct:bus1", "s, Fault Clearing Time"
      m_Brkload_bus1 = "m:Brkload:bus1", "%, Breaker Loading"
      n_ur_bus1 = "n:ur:bus1", "p.u., Voltage, Real Part"
      n_ui_bus1 = "n:ui:bus1", "p.u., Voltage, Imaginary Part"
      n_u_bus1 = "n:u:bus1", "p.u., Voltage, Magnitude"
      n_upc_bus1 = "n:upc:bus1", "%, Voltage, Magnitude"
      n_u1_bus1 = "n:u1:bus1", "p.u., Voltage, Magnitude"
      n_u1pc_bus1 = "n:u1pc:bus1", "%, Voltage, Magnitude"
      n_u1r_bus1 = "n:u1r:bus1", "p.u., Positive-Sequence Voltage, Real Part"
      n_u1i_bus1 = "n:u1i:bus1", "p.u., Positive-Sequence Voltage, Imaginary Part"
      n_U_bus1 = "n:U:bus1", "kV, Line-Ground Voltage, Magnitude"
      n_Ul_bus1 = "n:Ul:bus1", "kV, Line-Line Voltage, Magnitude"
      n_phiu_bus1 = "n:phiu:bus1", "deg, Voltage, Angle"
      n_phiurel_bus1 = "n:phiurel:bus1", "deg, Voltage, Relative Angle"
      n_du_bus1 = "n:du:bus1", "%, Voltage Deviation"
      n_Pgen_bus1 = "n:Pgen:bus1", "MW, Generation, Active Power"
      n_Qgen_bus1 = "n:Qgen:bus1", "Mvar, Generation, Reactive Power"
      n_Pmot_bus1 = "n:Pmot:bus1", "MW, Motor Load, Active Power"
      n_Qmot_bus1 = "n:Qmot:bus1", "Mvar, Motor Load, Reactive Power"
      n_Pload_bus1 = "n:Pload:bus1", "MW, General Load, Active Power"
      n_Qload_bus1 = "n:Qload:bus1", "Mvar, General Load, Reactive Power"
      n_Pcomp_bus1 = "n:Pcomp:bus1", "MW, Compensation (Losses)"
      n_Qcomp_bus1 = "n:Qcomp:bus1", "Mvar, Compensation"
      n_Pnet_bus1 = "n:Pnet:bus1", "MW, External Networks, Active Power"
      n_Qnet_bus1 = "n:Qnet:bus1", "Mvar, External Networks, Reactive Power"
      n_Pflow_bus1 = "n:Pflow:bus1", "MW, Power Flow, Active Power"
      n_Qflow_bus1 = "n:Qflow:bus1", "Mvar, Power Flow, Reactive Power"
      n_Pout_bus1 = "n:Pout:bus1", "MW, Outgoing Flow, Active Power"
      n_Qout_bus1 = "n:Qout:bus1", "Mvar, Outgoing Power, Reactive Power"
      n_Sout_bus1 = "n:Sout:bus1", "MVA, Outgoing Power, Apparent Power"
      n_cosphiout_bus1 = "n:cosphiout:bus1", "Outgoing Power, Power Factor"
      n_Pbalance_bus1 = "n:Pbalance:bus1", "MW, Active Power Balance (=0)"
      n_Qbalance_bus1 = "n:Qbalance:bus1", "Mvar, Reactive Power Balance (=0)"
      n_LossPdown_bus1 = "n:LossPdown:bus1", "MW, Losses, downstream"
      n_LossQdown_bus1 = "n:LossQdown:bus1", "Mvar, Losses, downstream (Reactive Power)"
      n_LossPdownload_bus1 = "n:LossPdownload:bus1", "MW, Load losses, downstream"
      n_LossQdownload_bus1 = "n:LossQdownload:bus1", "Mvar, Load losses, downstream"
      n_LossPdownnoload_bus1 = "n:LossPdownnoload:bus1", "MW, No load losses, downstream"
      n_LossQdownnoload_bus1 = "n:LossQdownnoload:bus1", "Mvar, No load losses, downstream"
      n_du_feed_bus1 = "n:du_feed:bus1", "%, Voltage difference relative to feeder begin"
      s_scale = "s:scale", "IN, Scale Factor, continuous, double"
      s_scalephi = "s:scalephi", "rad, IN, Load Scaling: Power Angle, continuous, double"
      s_pini_set = "s:pini_set", "IN, Nominal Active Power, continuous, double"
      s_qini_set = "s:qini_set", "IN, Nominal Reactive Power, continuous, double"
      c_pini = "c:pini", "MW, Nominal Active Power"
      c_qini = "c:qini", "Mvar, Nominal Reactive Power"
      c_s_r = "c:s:r", "Complex Power, Real Part"
      c_s_i = "c:s:i", "Complex Power, Imaginary Part"
      c_s_phi = "c:s:phi", "Complex Power, Angle"
      c_s_mag = "c:s:mag", "Complex Power, Magnitude"
      c_s_phirad = "c:s:phirad", "Complex Power, Angle"
      c_frnom = "c:frnom", "Hz, Nominal Frequency"
      c_hpi = "c:hpi", "rad/s, Nominal angular frequency"
      c_u0 = "c:u0", "p.u., Nominal Voltage"
      c_ncust = "c:ncust", "Number of Connected Customers"
      c_scale0 = "c:scale0", "Initial Scale Factor"
      c_i_rem = "c:i_rem", "Remote Control"
      c_zonescale = "c:zonescale", "Zone Scale Factor"
      c_Pnom = "c:Pnom", "MW, Nominal Active Power"
      c_Qnom = "c:Qnom", "Mvar, Nominal Reactive Power"
      c_kpu0 = "c:kpu0", "Volt. Dependence I on P"
      c_kqu0 = "c:kqu0", "Volt. Dependence I on Q"
      c_kpu1 = "c:kpu1", "Volt. Dependence II on P"
      c_kqu1 = "c:kqu1", "Volt. Dependence II on Q"
      c_kpu = "c:kpu", "Volt. Dependence III on P"
      c_kqu = "c:kqu", "Volt. Dependence III on Q"
      c_aP = "c:aP", "Percent of Volt. Dep. I on P"
      c_bP = "c:bP", "Percent of Volt. Dep. II on P"
      c_aQ = "c:aQ", "Percent of Volt. Dep. I on Q"
      c_bQ = "c:bQ", "Percent of Volt. Dep. II on Q"
      c_iRefElement = "c:iRefElement", "Reference Element"
      c_Pmism = "c:Pmism", "MW, Active Power Mismatch"
      c_Ppeak = "c:Ppeak", "MW, Peak Active Power"
      c_Qpeak = "c:Qpeak", "Mvar, Peak Reactive Power"
      c_Ploss = "c:Ploss", "MW, Losses (total)"
      c_Qloss = "c:Qloss", "Mvar, Reactive-Losses (total)"
      c_Plossld = "c:Plossld", "MW, Losses (load)"
      c_Qlossld = "c:Qlossld", "Mvar, Reactive-Losses (load)"
      c_Plossnld = "c:Plossnld", "MW, Losses (no load)"
      c_Qlossnld = "c:Qlossnld", "Mvar, Reactive-Losses (no load)"
      e_mode_inp = "e:mode_inp", "Input Mode"
      e_i_sym = "e:i_sym", "Balanced/Unbalanced"
      e_plini = "e:plini", "MW, Active Power"
      e_qlini = "e:qlini", "Mvar, Reactive Power"
      e_slini = "e:slini", "MVA, Apparent Power"
      e_ilini = "e:ilini", "kA, Current"
      e_coslini = "e:coslini", "Power Factor"
      e_pf_recap = "e:pf_recap", "Power Factor"
      e_p_direc = "e:p_direc", "Power Direction"
      e_u0 = "e:u0", "p.u., Voltage"
      e_scale0 = "e:scale0", "Scaling Factor"
      e_i_scale = "e:i_scale", "Adjusted by Load Scaling"
      e_plinir = "e:plinir", "MW, Active Power"
      e_qlinir = "e:qlinir", "Mvar, Reactive Power"
      e_slinir = "e:slinir", "MVA, Apparent Power"
      e_ilinir = "e:ilinir", "kA, Current"
      e_coslinir = "e:coslinir", "Power Factor"
      e_pf_recapr = "e:pf_recapr", "Power Factor"
      e_p_direcr = "e:p_direcr", "Power Direction"
      e_plinis = "e:plinis", "MW, Active Power"
      e_qlinis = "e:qlinis", "Mvar, Reactive Power"
      e_slinis = "e:slinis", "MVA, Apparent Power"
      e_ilinis = "e:ilinis", "kA, Current"
      e_coslinis = "e:coslinis", "Power Factor"
      e_pf_recaps = "e:pf_recaps", "Power Factor"
      e_p_direcs = "e:p_direcs", "Power Direction"
      e_plinit = "e:plinit", "MW, Active Power"
      e_qlinit = "e:qlinit", "Mvar, Reactive Power"
      e_slinit = "e:slinit", "MVA, Apparent Power"
      e_ilinit = "e:ilinit", "kA, Current"
      e_coslinit = "e:coslinit", "Power Factor"
      e_pf_recapt = "e:pf_recapt", "Power Factor"
      e_p_direct = "e:p_direct", "Power Direction"
      e_qdslCtrl = "e:qdslCtrl", "Quasi-Dynamic Model"
      e_zonefact = "e:zonefact", "Zone Scaling Factor:"
      e_i_rem = "e:i_rem", "Remote Control"
      e_p_cub = "e:p_cub", "StaCubic*, Controlled Branch (Cubicle)"
      e_plini_a = "e:plini_a", "MW, P(act.)"
      e_qlini_a = "e:qlini_a", "Mvar, Q(act.)"
      e_slini_a = "e:slini_a", "MVA, S(act.)"
      e_coslini_a = "e:coslini_a", "cos(phi)(act.)"
      e_ilini_a = "e:ilini_a", "kA, I(act.)"
      e_scale0_a = "e:scale0_a", "Scaling Factor(act.)"
      e_plinir_a = "e:plinir_a", "MW, Ph 1:P(act.)"
      e_qlinir_a = "e:qlinir_a", "Mvar, Ph 1:Q(act.)"
      e_slinir_a = "e:slinir_a", "MVA, Ph 1:S(act.)"
      e_coslinir_a = "e:coslinir_a", "Ph 1:cos(phi)(act.)"
      e_ilinir_a = "e:ilinir_a", "kA, Ph 1:I(act.)"
      e_plinis_a = "e:plinis_a", "MW, Ph 2:P(act.)"
      e_qlinis_a = "e:qlinis_a", "Mvar, Ph 2:Q(act.)"
      e_slinis_a = "e:slinis_a", "MVA, Ph 2:S(act.)"
      e_coslinis_a = "e:coslinis_a", "Ph 2:cos(phi)(act.)"
      e_ilinis_a = "e:ilinis_a", "kA, Ph 2:I(act.)"
      e_plinit_a = "e:plinit_a", "MW, Ph 3:P(act.)"
      e_qlinit_a = "e:qlinit_a", "Mvar, Ph 3:Q(act.)"
      e_slinit_a = "e:slinit_a", "MVA, Ph 3:S(act.)"
      e_coslinit_a = "e:coslinit_a", "Ph 3:cos(phi)(act.)"
      e_ilinit_a = "e:ilinit_a", "kA, Ph 3:I(act.)"
      e_cpFeed = "e:cpFeed", "Feeder"
      e_ciDist = "e:ciDist", "Distance from infeed in number of buses"
      e_ciLater = "e:ciLater", "Lateral Index"
      e_ciDistRoot = "e:ciDistRoot", "Distance from first infeed in number of buses"
      e_ciDistAll = "e:ciDistAll", "Distance from infeed in number of buses including switches"
      e_ciDistAllRoot = "e:ciDistAllRoot", "Distance from first infeed in number of buses including switches"
      r_qdslCtrl = "r:qdslCtrl", "Quasi-Dynamic Model"
      r_p_cub = "r:p_cub", "StaCubic*, Controlled Branch (Cubicle)"
      r_cpFeed = "r:cpFeed", "Feeder"

    class ElmLodlv(Enum):
      _init_ = 'value __doc__'

      m_u1r_bus1 = "m:u1r:bus1", "p.u., Positive-Sequence-Voltage, Real Part"
      m_u1i_bus1 = "m:u1i:bus1", "p.u., Positive-Sequence-Voltage, Imaginary Part"
      m_u1_bus1 = "m:u1:bus1", "p.u., Positive-Sequence-Voltage, Magnitude"
      m_U1_bus1 = "m:U1:bus1", "kV, Line-Ground Positive-Sequence-Voltage, Magnitude"
      m_U1l_bus1 = "m:U1l:bus1", "kV, Line-Line Positive-Sequence-Voltage, Magnitude"
      m_phiu1_bus1 = "m:phiu1:bus1", "deg, Positive-Sequence-Voltage, Angle"
      m_inet_bus1 = "m:inet:bus1", "p.u., Current, Magnitude, referred to network"
      m_I_bus1 = "m:I:bus1", "kA, Current, Magnitude"
      m_phii_bus1 = "m:phii:bus1", "deg, Current, Angle"
      m_i1r_bus1 = "m:i1r:bus1", "p.u., Positive-Sequence Current, Real Part"
      m_i1i_bus1 = "m:i1i:bus1", "p.u., Positive-Sequence Current, Imaginary Part"
      m_i1_bus1 = "m:i1:bus1", "p.u., Positive-Sequence Current, Magnitude"
      m_I1_bus1 = "m:I1:bus1", "kA, Positive-Sequence Current, Magnitude"
      m_phii1_bus1 = "m:phii1:bus1", "deg, Positive-Sequence Current, Angle"
      m_P_bus1 = "m:P:bus1", "MW, Active Power"
      m_Q_bus1 = "m:Q:bus1", "Mvar, Reactive Power"
      m_S_bus1 = "m:S:bus1", "MVA, Apparent Power"
      m_cosphi_bus1 = "m:cosphi:bus1", "Power Factor"
      m_tanphi_bus1 = "m:tanphi:bus1", "tan(phi)"
      m_Psum_bus1 = "m:Psum:bus1", "MW, Total Active Power"
      m_Qsum_bus1 = "m:Qsum:bus1", "Mvar, Total Reactive Power"
      m_Ssum_bus1 = "m:Ssum:bus1", "MVA, Total Apparent Power"
      m_cosphisum_bus1 = "m:cosphisum:bus1", "Total Power Factor"
      m_tanphisum_bus1 = "m:tanphisum:bus1", "Total tan(phi)"
      m_Spu_bus1 = "m:Spu:bus1", "MVA/p.u., Apparent Power per p.u. Voltage"
      m_i1P_bus1 = "m:i1P:bus1", "p.u., Positive-Sequence Active Current"
      m_i1Q_bus1 = "m:i1Q:bus1", "p.u., Positive-Sequence Reactive Current"
      m_I1P_bus1 = "m:I1P:bus1", "kA, Positive-Sequence Active Current"
      m_I1Q_bus1 = "m:I1Q:bus1", "kA, Positive-Sequence Reactive Current"
      m_ur_bus1 = "m:ur:bus1", "p.u., Voltage, Real Part"
      m_ui_bus1 = "m:ui:bus1", "p.u., Voltage, Imaginary Part"
      m_u_bus1 = "m:u:bus1", "p.u., Voltage, Magnitude"
      m_ir_bus1 = "m:ir:bus1", "p.u., Current, Real Part"
      m_ii_bus1 = "m:ii:bus1", "p.u., Current, Imaginary Part"
      m_i_bus1 = "m:i:bus1", "p.u., Current, Magnitude"
      m_phiui_bus1 = "m:phiui:bus1", "deg, Angle between Voltage and Current"
      m_phiu1i1_bus1 = "m:phiu1i1:bus1", "deg, Angle between Voltage and Current in positive sequence system"
      m_Tfct_bus1 = "m:Tfct:bus1", "s, Fault Clearing Time"
      m_Brkload_bus1 = "m:Brkload:bus1", "%, Breaker Loading"
      n_ur_bus1 = "n:ur:bus1", "p.u., Voltage, Real Part"
      n_ui_bus1 = "n:ui:bus1", "p.u., Voltage, Imaginary Part"
      n_u_bus1 = "n:u:bus1", "p.u., Voltage, Magnitude"
      n_upc_bus1 = "n:upc:bus1", "%, Voltage, Magnitude"
      n_u1_bus1 = "n:u1:bus1", "p.u., Voltage, Magnitude"
      n_u1pc_bus1 = "n:u1pc:bus1", "%, Voltage, Magnitude"
      n_u1r_bus1 = "n:u1r:bus1", "p.u., Positive-Sequence Voltage, Real Part"
      n_u1i_bus1 = "n:u1i:bus1", "p.u., Positive-Sequence Voltage, Imaginary Part"
      n_U_bus1 = "n:U:bus1", "kV, Line-Ground Voltage, Magnitude"
      n_Ul_bus1 = "n:Ul:bus1", "kV, Line-Line Voltage, Magnitude"
      n_phiu_bus1 = "n:phiu:bus1", "deg, Voltage, Angle"
      n_phiurel_bus1 = "n:phiurel:bus1", "deg, Voltage, Relative Angle"
      n_du_bus1 = "n:du:bus1", "%, Voltage Deviation"
      n_Pgen_bus1 = "n:Pgen:bus1", "MW, Generation, Active Power"
      n_Qgen_bus1 = "n:Qgen:bus1", "Mvar, Generation, Reactive Power"
      n_Pmot_bus1 = "n:Pmot:bus1", "MW, Motor Load, Active Power"
      n_Qmot_bus1 = "n:Qmot:bus1", "Mvar, Motor Load, Reactive Power"
      n_Pload_bus1 = "n:Pload:bus1", "MW, General Load, Active Power"
      n_Qload_bus1 = "n:Qload:bus1", "Mvar, General Load, Reactive Power"
      n_Pcomp_bus1 = "n:Pcomp:bus1", "MW, Compensation (Losses)"
      n_Qcomp_bus1 = "n:Qcomp:bus1", "Mvar, Compensation"
      n_Pnet_bus1 = "n:Pnet:bus1", "MW, External Networks, Active Power"
      n_Qnet_bus1 = "n:Qnet:bus1", "Mvar, External Networks, Reactive Power"
      n_Pflow_bus1 = "n:Pflow:bus1", "MW, Power Flow, Active Power"
      n_Qflow_bus1 = "n:Qflow:bus1", "Mvar, Power Flow, Reactive Power"
      n_Pout_bus1 = "n:Pout:bus1", "MW, Outgoing Flow, Active Power"
      n_Qout_bus1 = "n:Qout:bus1", "Mvar, Outgoing Power, Reactive Power"
      n_Sout_bus1 = "n:Sout:bus1", "MVA, Outgoing Power, Apparent Power"
      n_cosphiout_bus1 = "n:cosphiout:bus1", "Outgoing Power, Power Factor"
      n_Pbalance_bus1 = "n:Pbalance:bus1", "MW, Active Power Balance (=0)"
      n_Qbalance_bus1 = "n:Qbalance:bus1", "Mvar, Reactive Power Balance (=0)"
      n_LossPdown_bus1 = "n:LossPdown:bus1", "MW, Losses, downstream"
      n_LossQdown_bus1 = "n:LossQdown:bus1", "Mvar, Losses, downstream (Reactive Power)"
      n_LossPdownload_bus1 = "n:LossPdownload:bus1", "MW, Load losses, downstream"
      n_LossQdownload_bus1 = "n:LossQdownload:bus1", "Mvar, Load losses, downstream"
      n_LossPdownnoload_bus1 = "n:LossPdownnoload:bus1", "MW, No load losses, downstream"
      n_LossQdownnoload_bus1 = "n:LossQdownnoload:bus1", "Mvar, No load losses, downstream"
      n_du_feed_bus1 = "n:du_feed:bus1", "%, Voltage difference relative to feeder begin"
      s_scale = "s:scale", "IN, Scale Factor, continuous, double"
      s_scalephi = "s:scalephi", "rad, IN, Load Scaling: Power Angle, continuous, double"
      s_pini_set = "s:pini_set", "IN, Nominal Active Power, continuous, double"
      s_qini_set = "s:qini_set", "IN, Nominal Reactive Power, continuous, double"
      c_pini = "c:pini", "MW, Nominal Active Power"
      c_qini = "c:qini", "Mvar, Nominal Reactive Power"
      c_s_r = "c:s:r", "Complex Power, Real Part"
      c_s_i = "c:s:i", "Complex Power, Imaginary Part"
      c_s_phi = "c:s:phi", "Complex Power, Angle"
      c_s_mag = "c:s:mag", "Complex Power, Magnitude"
      c_s_phirad = "c:s:phirad", "Complex Power, Angle"
      c_frnom = "c:frnom", "Hz, Nominal Frequency"
      c_hpi = "c:hpi", "rad/s, Nominal angular frequency"
      c_u0 = "c:u0", "p.u., Nominal Voltage"
      c_ncust = "c:ncust", "Number of Connected Customers"
      c_scale0 = "c:scale0", "Initial Scale Factor"
      c_zonescale = "c:zonescale", "Zone Scale Factor"
      c_Pnom = "c:Pnom", "MW, Nominal Active Power"
      c_Qnom = "c:Qnom", "Mvar, Nominal Reactive Power"
      c_kpu0 = "c:kpu0", "Volt. Dependence I on P"
      c_kqu0 = "c:kqu0", "Volt. Dependence I on Q"
      c_kpu1 = "c:kpu1", "Volt. Dependence II on P"
      c_kqu1 = "c:kqu1", "Volt. Dependence II on Q"
      c_kpu = "c:kpu", "Volt. Dependence III on P"
      c_kqu = "c:kqu", "Volt. Dependence III on Q"
      c_aP = "c:aP", "Percent of Volt. Dep. I on P"
      c_bP = "c:bP", "Percent of Volt. Dep. II on P"
      c_aQ = "c:aQ", "Percent of Volt. Dep. I on Q"
      c_bQ = "c:bQ", "Percent of Volt. Dep. II on Q"
      c_Pmism = "c:Pmism", "MW, Active Power Mismatch"
      c_Ppeak = "c:Ppeak", "MW, Peak Active Power"
      c_Qpeak = "c:Qpeak", "Mvar, Peak Reactive Power"
      c_Ploss = "c:Ploss", "MW, Losses (total)"
      c_Qloss = "c:Qloss", "Mvar, Reactive-Losses (total)"
      c_Plossld = "c:Plossld", "MW, Losses (load)"
      c_Qlossld = "c:Qlossld", "Mvar, Reactive-Losses (load)"
      c_Plossnld = "c:Plossnld", "MW, Losses (no load)"
      c_Qlossnld = "c:Qlossnld", "Mvar, Reactive-Losses (no load)"
      e_iopt_inp = "e:iopt_inp", "Input Mode"
      e_i_sym = "e:i_sym", "Balanced/Unbalanced"
      e_ulini = "e:ulini", "kV, Voltage, U(L-L)"
      e_plini = "e:plini", "kW, Active Power, P"
      e_slini = "e:slini", "kVA, Apparent Power, S"
      e_ilini = "e:ilini", "A, Current, I"
      e_cplinia = "e:cplinia", "kW, P, Load"
      e_coslini = "e:coslini", "Power Factor, cos(phi)"
      e_pf_recap = "e:pf_recap", "Power Factor"
      e_elini = "e:elini", "kWh, Yearly Energy"
      e_pProfile = "e:pProfile", "ChaTime,ChaProfile, Consumption Profile"
      e_scale0 = "e:scale0", "Scaling Factor"
      e_i_scale = "e:i_scale", "Adjusted by Load Scaling"
      e_plinir = "e:plinir", "kW, Active Power, P"
      e_slinir = "e:slinir", "kVA, Apparent Power, S"
      e_ilinir = "e:ilinir", "A, Current, I"
      e_coslinir = "e:coslinir", "Power Factor, cos(phi)"
      e_pf_recapr = "e:pf_recapr", "Power Factor"
      e_plinis = "e:plinis", "kW, Active Power, P"
      e_slinis = "e:slinis", "kVA, Apparent Power, S"
      e_ilinis = "e:ilinis", "A, Current, I"
      e_coslinis = "e:coslinis", "Power Factor, cos(phi)"
      e_pf_recaps = "e:pf_recaps", "Power Factor"
      e_plinit = "e:plinit", "kW, Active Power, P"
      e_slinit = "e:slinit", "kVA, Apparent Power, S"
      e_ilinit = "e:ilinit", "A, Current, I"
      e_coslinit = "e:coslinit", "Power Factor, cos(phi)"
      e_pf_recapt = "e:pf_recapt", "Power Factor"
      e_qdslCtrl = "e:qdslCtrl", "Quasi-Dynamic Model"
      e_cHasPartLod = "e:cHasPartLod", "Partial Loads Present"
      e_cPartLod = "e:cPartLod", "Partial Loads:"
      e_ulini_a = "e:ulini_a", "kV, U(L-L)(act.)"
      e_plini_a = "e:plini_a", "kW, P(act.)"
      e_qlini_a = "e:qlini_a", "kvar, Q(act.)"
      e_slini_a = "e:slini_a", "kVA, S(act.)"
      e_coslini_a = "e:coslini_a", "cos(phi)(act.)"
      e_ilini_a = "e:ilini_a", "A, I(act.)"
      e_scale0_a = "e:scale0_a", "Scaling Factor(act.)"
      e_pnight_a = "e:pnight_a", "kW, P(night)(act.)"
      e_plinir_a = "e:plinir_a", "kW, Ph 1:P(act.)"
      e_slinir_a = "e:slinir_a", "kVA, Ph 1:S(act.)"
      e_coslinir_a = "e:coslinir_a", "Ph 1:cos(phi)(act.)"
      e_ilinir_a = "e:ilinir_a", "A, Ph 1:I(act.)"
      e_plinis_a = "e:plinis_a", "kW, Ph 2:P(act.)"
      e_slinis_a = "e:slinis_a", "kVA, Ph 2:S(act.)"
      e_coslinis_a = "e:coslinis_a", "Ph 2:cos(phi)(act.)"
      e_ilinis_a = "e:ilinis_a", "A, Ph 2:I(act.)"
      e_plinit_a = "e:plinit_a", "kW, Ph 3:P(act.)"
      e_slinit_a = "e:slinit_a", "kVA, Ph 3:S(act.)"
      e_coslinit_a = "e:coslinit_a", "Ph 3:cos(phi)(act.)"
      e_ilinit_a = "e:ilinit_a", "A, Ph 3:I(act.)"
      e_pnight = "e:pnight", "kW, P"
      e_cNrCust = "e:cNrCust", "Number of Customers"
      e_cPrCust = "e:cPrCust", "kW, P/Customer"
      e_UtilFactor = "e:UtilFactor", "Utilisation Factor"
      e_cSmax = "e:cSmax", "kVA, Max. Load per Customer"
      e_cSav = "e:cSav", "kVA, Average Load"
      e_ccosphi = "e:ccosphi", "Power Factor"
      e_cpFeed = "e:cpFeed", "Feeder"
      e_ciDist = "e:ciDist", "Distance from infeed in number of buses"
      e_ciLater = "e:ciLater", "Lateral Index"
      e_ciDistRoot = "e:ciDistRoot", "Distance from first infeed in number of buses"
      e_ciDistAll = "e:ciDistAll", "Distance from infeed in number of buses including switches"
      e_ciDistAllRoot = "e:ciDistAllRoot", "Distance from first infeed in number of buses including switches"
      r_pProfile = "r:pProfile", "ChaTime,ChaProfile, Consumption Profile"
      r_qdslCtrl = "r:qdslCtrl", "Quasi-Dynamic Model"
      r_cpFeed = "r:cpFeed", "Feeder"

    class ElmLodmv(Enum):
      _init_ = 'value __doc__'

      m_u1r_bus1 = "m:u1r:bus1", "p.u., Positive-Sequence-Voltage, Real Part"
      m_u1i_bus1 = "m:u1i:bus1", "p.u., Positive-Sequence-Voltage, Imaginary Part"
      m_u1_bus1 = "m:u1:bus1", "p.u., Positive-Sequence-Voltage, Magnitude"
      m_U1_bus1 = "m:U1:bus1", "kV, Line-Ground Positive-Sequence-Voltage, Magnitude"
      m_U1l_bus1 = "m:U1l:bus1", "kV, Line-Line Positive-Sequence-Voltage, Magnitude"
      m_phiu1_bus1 = "m:phiu1:bus1", "deg, Positive-Sequence-Voltage, Angle"
      m_inet_bus1 = "m:inet:bus1", "p.u., Current, Magnitude, referred to network"
      m_I_bus1 = "m:I:bus1", "kA, Current, Magnitude"
      m_phii_bus1 = "m:phii:bus1", "deg, Current, Angle"
      m_i1r_bus1 = "m:i1r:bus1", "p.u., Positive-Sequence Current, Real Part"
      m_i1i_bus1 = "m:i1i:bus1", "p.u., Positive-Sequence Current, Imaginary Part"
      m_i1_bus1 = "m:i1:bus1", "p.u., Positive-Sequence Current, Magnitude"
      m_I1_bus1 = "m:I1:bus1", "kA, Positive-Sequence Current, Magnitude"
      m_phii1_bus1 = "m:phii1:bus1", "deg, Positive-Sequence Current, Angle"
      m_P_bus1 = "m:P:bus1", "MW, Active Power"
      m_Q_bus1 = "m:Q:bus1", "Mvar, Reactive Power"
      m_S_bus1 = "m:S:bus1", "MVA, Apparent Power"
      m_cosphi_bus1 = "m:cosphi:bus1", "Power Factor"
      m_tanphi_bus1 = "m:tanphi:bus1", "tan(phi)"
      m_Psum_bus1 = "m:Psum:bus1", "MW, Total Active Power"
      m_Qsum_bus1 = "m:Qsum:bus1", "Mvar, Total Reactive Power"
      m_Ssum_bus1 = "m:Ssum:bus1", "MVA, Total Apparent Power"
      m_cosphisum_bus1 = "m:cosphisum:bus1", "Total Power Factor"
      m_tanphisum_bus1 = "m:tanphisum:bus1", "Total tan(phi)"
      m_Spu_bus1 = "m:Spu:bus1", "MVA/p.u., Apparent Power per p.u. Voltage"
      m_i1P_bus1 = "m:i1P:bus1", "p.u., Positive-Sequence Active Current"
      m_i1Q_bus1 = "m:i1Q:bus1", "p.u., Positive-Sequence Reactive Current"
      m_I1P_bus1 = "m:I1P:bus1", "kA, Positive-Sequence Active Current"
      m_I1Q_bus1 = "m:I1Q:bus1", "kA, Positive-Sequence Reactive Current"
      m_ur_bus1 = "m:ur:bus1", "p.u., Voltage, Real Part"
      m_ui_bus1 = "m:ui:bus1", "p.u., Voltage, Imaginary Part"
      m_u_bus1 = "m:u:bus1", "p.u., Voltage, Magnitude"
      m_ir_bus1 = "m:ir:bus1", "p.u., Current, Real Part"
      m_ii_bus1 = "m:ii:bus1", "p.u., Current, Imaginary Part"
      m_i_bus1 = "m:i:bus1", "p.u., Current, Magnitude"
      m_phiui_bus1 = "m:phiui:bus1", "deg, Angle between Voltage and Current"
      m_phiu1i1_bus1 = "m:phiu1i1:bus1", "deg, Angle between Voltage and Current in positive sequence system"
      m_Tfct_bus1 = "m:Tfct:bus1", "s, Fault Clearing Time"
      m_Brkload_bus1 = "m:Brkload:bus1", "%, Breaker Loading"
      n_ur_bus1 = "n:ur:bus1", "p.u., Voltage, Real Part"
      n_ui_bus1 = "n:ui:bus1", "p.u., Voltage, Imaginary Part"
      n_u_bus1 = "n:u:bus1", "p.u., Voltage, Magnitude"
      n_upc_bus1 = "n:upc:bus1", "%, Voltage, Magnitude"
      n_u1_bus1 = "n:u1:bus1", "p.u., Voltage, Magnitude"
      n_u1pc_bus1 = "n:u1pc:bus1", "%, Voltage, Magnitude"
      n_u1r_bus1 = "n:u1r:bus1", "p.u., Positive-Sequence Voltage, Real Part"
      n_u1i_bus1 = "n:u1i:bus1", "p.u., Positive-Sequence Voltage, Imaginary Part"
      n_U_bus1 = "n:U:bus1", "kV, Line-Ground Voltage, Magnitude"
      n_Ul_bus1 = "n:Ul:bus1", "kV, Line-Line Voltage, Magnitude"
      n_phiu_bus1 = "n:phiu:bus1", "deg, Voltage, Angle"
      n_phiurel_bus1 = "n:phiurel:bus1", "deg, Voltage, Relative Angle"
      n_du_bus1 = "n:du:bus1", "%, Voltage Deviation"
      n_Pgen_bus1 = "n:Pgen:bus1", "MW, Generation, Active Power"
      n_Qgen_bus1 = "n:Qgen:bus1", "Mvar, Generation, Reactive Power"
      n_Pmot_bus1 = "n:Pmot:bus1", "MW, Motor Load, Active Power"
      n_Qmot_bus1 = "n:Qmot:bus1", "Mvar, Motor Load, Reactive Power"
      n_Pload_bus1 = "n:Pload:bus1", "MW, General Load, Active Power"
      n_Qload_bus1 = "n:Qload:bus1", "Mvar, General Load, Reactive Power"
      n_Pcomp_bus1 = "n:Pcomp:bus1", "MW, Compensation (Losses)"
      n_Qcomp_bus1 = "n:Qcomp:bus1", "Mvar, Compensation"
      n_Pnet_bus1 = "n:Pnet:bus1", "MW, External Networks, Active Power"
      n_Qnet_bus1 = "n:Qnet:bus1", "Mvar, External Networks, Reactive Power"
      n_Pflow_bus1 = "n:Pflow:bus1", "MW, Power Flow, Active Power"
      n_Qflow_bus1 = "n:Qflow:bus1", "Mvar, Power Flow, Reactive Power"
      n_Pout_bus1 = "n:Pout:bus1", "MW, Outgoing Flow, Active Power"
      n_Qout_bus1 = "n:Qout:bus1", "Mvar, Outgoing Power, Reactive Power"
      n_Sout_bus1 = "n:Sout:bus1", "MVA, Outgoing Power, Apparent Power"
      n_cosphiout_bus1 = "n:cosphiout:bus1", "Outgoing Power, Power Factor"
      n_Pbalance_bus1 = "n:Pbalance:bus1", "MW, Active Power Balance (=0)"
      n_Qbalance_bus1 = "n:Qbalance:bus1", "Mvar, Reactive Power Balance (=0)"
      n_LossPdown_bus1 = "n:LossPdown:bus1", "MW, Losses, downstream"
      n_LossQdown_bus1 = "n:LossQdown:bus1", "Mvar, Losses, downstream (Reactive Power)"
      n_LossPdownload_bus1 = "n:LossPdownload:bus1", "MW, Load losses, downstream"
      n_LossQdownload_bus1 = "n:LossQdownload:bus1", "Mvar, Load losses, downstream"
      n_LossPdownnoload_bus1 = "n:LossPdownnoload:bus1", "MW, No load losses, downstream"
      n_LossQdownnoload_bus1 = "n:LossQdownnoload:bus1", "Mvar, No load losses, downstream"
      n_du_feed_bus1 = "n:du_feed:bus1", "%, Voltage difference relative to feeder begin"
      s_scale = "s:scale", "IN, Scale Factor, continuous, double"
      s_scalephi = "s:scalephi", "rad, IN, Load Scaling: Power Angle, continuous, double"
      s_pini_set = "s:pini_set", "IN, Nominal Active Power, continuous, double"
      s_qini_set = "s:qini_set", "IN, Nominal Reactive Power, continuous, double"
      c_pini = "c:pini", "MW, Nominal Active Power"
      c_qini = "c:qini", "Mvar, Nominal Reactive Power"
      c_s_r = "c:s:r", "Complex Power, Real Part"
      c_s_i = "c:s:i", "Complex Power, Imaginary Part"
      c_s_phi = "c:s:phi", "Complex Power, Angle"
      c_s_mag = "c:s:mag", "Complex Power, Magnitude"
      c_s_phirad = "c:s:phirad", "Complex Power, Angle"
      c_frnom = "c:frnom", "Hz, Nominal Frequency"
      c_hpi = "c:hpi", "rad/s, Nominal angular frequency"
      c_u0 = "c:u0", "p.u., Nominal Voltage"
      c_nntap = "c:nntap", "Current Tap Position"
      c_ulv = "c:ulv", "p.u., LV Voltage"
      c_Gmload = "c:Gmload", "kW, Iron Losses"
      c_dudropTrf = "c:dudropTrf", "%, max. Voltage drop across transformer"
      c_duriseTrf = "c:duriseTrf", "%, max. Voltage rise across transformer"
      c_ncust = "c:ncust", "Number of Connected Customers"
      c_scale0 = "c:scale0", "Initial Scale Factor"
      c_zonescale = "c:zonescale", "Zone Scale Factor"
      c_gscale = "c:gscale", "Generation Scale Factor"
      c_tempscFac = "c:tempscFac", "Temperature dependent load scaling factor"
      c_Pnom = "c:Pnom", "MW, Nominal Active Power"
      c_Qnom = "c:Qnom", "Mvar, Nominal Reactive Power"
      c_kpu0 = "c:kpu0", "Volt. Dependence I on P"
      c_kqu0 = "c:kqu0", "Volt. Dependence I on Q"
      c_kpu1 = "c:kpu1", "Volt. Dependence II on P"
      c_kqu1 = "c:kqu1", "Volt. Dependence II on Q"
      c_kpu = "c:kpu", "Volt. Dependence III on P"
      c_kqu = "c:kqu", "Volt. Dependence III on Q"
      c_aP = "c:aP", "Percent of Volt. Dep. I on P"
      c_bP = "c:bP", "Percent of Volt. Dep. II on P"
      c_aQ = "c:aQ", "Percent of Volt. Dep. I on Q"
      c_bQ = "c:bQ", "Percent of Volt. Dep. II on Q"
      c_iRefElement = "c:iRefElement", "Reference Element"
      c_Ppeak = "c:Ppeak", "MW, Peak Active Power"
      c_Qpeak = "c:Qpeak", "Mvar, Peak Reactive Power"
      c_Ploss = "c:Ploss", "MW, Losses (total)"
      c_Qloss = "c:Qloss", "Mvar, Reactive-Losses (total)"
      c_Plossld = "c:Plossld", "MW, Losses (load)"
      c_Qlossld = "c:Qlossld", "Mvar, Reactive-Losses (load)"
      c_Plossnld = "c:Plossnld", "MW, Losses (no load)"
      c_Qlossnld = "c:Qlossnld", "Mvar, Reactive-Losses (no load)"
      e_mode_inp = "e:mode_inp", "Input Mode"
      e_ci_sym = "e:ci_sym", "Balanced/Unbalanced"
      e_plini = "e:plini", "MW, P, Load"
      e_slini = "e:slini", "MVA, S, Load"
      e_coslini = "e:coslini", "cos(phi), Load"
      e_pf_recap = "e:pf_recap", "cos(phi), Load"
      e_cplinia = "e:cplinia", "MW, P, Load"
      e_elini = "e:elini", "MWh, Yearly Energy"
      e_pProfile = "e:pProfile", "ChaTime,ChaProfile, Consumption Profile"
      e_scale0 = "e:scale0", "Load Scaling Factor"
      e_i_scale = "e:i_scale", "Adjusted by Load Scaling"
      e_pgini = "e:pgini", "MW, P, Generation"
      e_sgini = "e:sgini", "MVA, S, Generation"
      e_cosgini = "e:cosgini", "cos(phi), Generation"
      e_pfg_recap = "e:pfg_recap", "cos(phi), Generation"
      e_gscale = "e:gscale", "Generation Scaling Factor"
      e_plinir = "e:plinir", "MW, P, Load"
      e_slinir = "e:slinir", "MVA, S, Load"
      e_coslinir = "e:coslinir", "cos(phi), Load"
      e_pf_recapr = "e:pf_recapr", "cos(phi), Load"
      e_pginir = "e:pginir", "MW, P, Generation"
      e_sginir = "e:sginir", "MVA, S, Generation"
      e_cosginir = "e:cosginir", "cos(phi), Generation"
      e_pfg_recapr = "e:pfg_recapr", "cos(phi), Generation"
      e_plinis = "e:plinis", "MW, P, Load"
      e_slinis = "e:slinis", "MVA, S, Load"
      e_coslinis = "e:coslinis", "cos(phi), Load"
      e_pf_recaps = "e:pf_recaps", "cos(phi), Load"
      e_pginis = "e:pginis", "MW, P, Generation"
      e_sginis = "e:sginis", "MVA, S, Generation"
      e_cosginis = "e:cosginis", "cos(phi), Generation"
      e_pfg_recaps = "e:pfg_recaps", "cos(phi), Generation"
      e_plinit = "e:plinit", "MW, P, Load"
      e_slinit = "e:slinit", "MVA, S, Load"
      e_coslinit = "e:coslinit", "cos(phi), Load"
      e_pf_recapt = "e:pf_recapt", "cos(phi), Load"
      e_pginit = "e:pginit", "MW, P, Generation"
      e_sginit = "e:sginit", "MVA, S, Generation"
      e_cosginit = "e:cosginit", "cos(phi), Generation"
      e_pfg_recapt = "e:pfg_recapt", "cos(phi), Generation"
      e_nntap = "e:nntap", "Tap Position"
      e_dutap = "e:dutap", "%, Additional Voltage per Tap"
      e_qdslCtrl = "e:qdslCtrl", "Quasi-Dynamic Model"
      e_plini_a = "e:plini_a", "MW, P(act.)"
      e_slini_a = "e:slini_a", "MVA, S(act.)"
      e_coslini_a = "e:coslini_a", "cos(phi)(act.)"
      e_scale0_a = "e:scale0_a", "Scaling Factor(act.)"
      e_plinir_a = "e:plinir_a", "MW, Ph 1:P(act.)"
      e_slinir_a = "e:slinir_a", "MVA, Ph 1:S(act.)"
      e_coslinir_a = "e:coslinir_a", "Ph 1:cos(phi)(act.)"
      e_plinis_a = "e:plinis_a", "MW, Ph 2:P(act.)"
      e_slinis_a = "e:slinis_a", "MVA, Ph 2:S(act.)"
      e_coslinis_a = "e:coslinis_a", "Ph 2:cos(phi)(act.)"
      e_plinit_a = "e:plinit_a", "MW, Ph 3:P(act.)"
      e_slinit_a = "e:slinit_a", "MVA, Ph 3:S(act.)"
      e_coslinit_a = "e:coslinit_a", "Ph 3:cos(phi)(act.)"
      e_gscale_a = "e:gscale_a", "Gen. Scaling Factor(act.)"
      e_pgini_a = "e:pgini_a", "MW, Pgen(act.)"
      e_sgini_a = "e:sgini_a", "MVA, Sgen(act.)"
      e_cosgini_a = "e:cosgini_a", "cos(phi)(act.), Gen."
      e_pginir_a = "e:pginir_a", "MW, Ph 1:Pgen(act.)"
      e_sginir_a = "e:sginir_a", "MVA, Ph 1:Sgen(act.)"
      e_cosginir_a = "e:cosginir_a", "Ph 1:cos(phi)(act.), Gen."
      e_pginis_a = "e:pginis_a", "MW, Ph 2:Pgen(act.)"
      e_sginis_a = "e:sginis_a", "MVA, Ph 2:Sgen(act.)"
      e_cosginis_a = "e:cosginis_a", "Ph 2:cos(phi)(act.), Gen."
      e_pginit_a = "e:pginit_a", "MW, Ph 3:Pgen(act.)"
      e_sginit_a = "e:sginit_a", "MVA, Ph 3:Sgen(act.)"
      e_cosginit_a = "e:cosginit_a", "Ph 3:cos(phi)(act.), Gen."
      e_cpFeed = "e:cpFeed", "Feeder"
      e_ciDist = "e:ciDist", "Distance from infeed in number of buses"
      e_ciLater = "e:ciLater", "Lateral Index"
      e_ciDistRoot = "e:ciDistRoot", "Distance from first infeed in number of buses"
      e_ciDistAll = "e:ciDistAll", "Distance from infeed in number of buses including switches"
      e_ciDistAllRoot = "e:ciDistAllRoot", "Distance from first infeed in number of buses including switches"
      r_pProfile = "r:pProfile", "ChaTime,ChaProfile, Consumption Profile"
      r_qdslCtrl = "r:qdslCtrl", "Quasi-Dynamic Model"
      r_cpFeed = "r:cpFeed", "Feeder"

    class ElmPvsys(Enum):
      _init_ = 'value __doc__'

      m_u1r_bus1 = "m:u1r:bus1", "p.u., Positive-Sequence-Voltage, Real Part"
      m_u1i_bus1 = "m:u1i:bus1", "p.u., Positive-Sequence-Voltage, Imaginary Part"
      m_u1_bus1 = "m:u1:bus1", "p.u., Positive-Sequence-Voltage, Magnitude"
      m_U1_bus1 = "m:U1:bus1", "kV, Line-Ground Positive-Sequence-Voltage, Magnitude"
      m_U1l_bus1 = "m:U1l:bus1", "kV, Line-Line Positive-Sequence-Voltage, Magnitude"
      m_phiu1_bus1 = "m:phiu1:bus1", "deg, Positive-Sequence-Voltage, Angle"
      m_inet_bus1 = "m:inet:bus1", "p.u., Current, Magnitude, referred to network"
      m_I_bus1 = "m:I:bus1", "kA, Current, Magnitude"
      m_phii_bus1 = "m:phii:bus1", "deg, Current, Angle"
      m_i1r_bus1 = "m:i1r:bus1", "p.u., Positive-Sequence Current, Real Part"
      m_i1i_bus1 = "m:i1i:bus1", "p.u., Positive-Sequence Current, Imaginary Part"
      m_i1_bus1 = "m:i1:bus1", "p.u., Positive-Sequence Current, Magnitude"
      m_I1_bus1 = "m:I1:bus1", "kA, Positive-Sequence Current, Magnitude"
      m_phii1_bus1 = "m:phii1:bus1", "deg, Positive-Sequence Current, Angle"
      m_P_bus1 = "m:P:bus1", "MW, Active Power"
      m_Q_bus1 = "m:Q:bus1", "Mvar, Reactive Power"
      m_S_bus1 = "m:S:bus1", "MVA, Apparent Power"
      m_cosphi_bus1 = "m:cosphi:bus1", "Power Factor"
      m_tanphi_bus1 = "m:tanphi:bus1", "tan(phi)"
      m_Psum_bus1 = "m:Psum:bus1", "MW, Total Active Power"
      m_Qsum_bus1 = "m:Qsum:bus1", "Mvar, Total Reactive Power"
      m_Ssum_bus1 = "m:Ssum:bus1", "MVA, Total Apparent Power"
      m_cosphisum_bus1 = "m:cosphisum:bus1", "Total Power Factor"
      m_tanphisum_bus1 = "m:tanphisum:bus1", "Total tan(phi)"
      m_Spu_bus1 = "m:Spu:bus1", "MVA/p.u., Apparent Power per p.u. Voltage"
      m_i1P_bus1 = "m:i1P:bus1", "p.u., Positive-Sequence Active Current"
      m_i1Q_bus1 = "m:i1Q:bus1", "p.u., Positive-Sequence Reactive Current"
      m_I1P_bus1 = "m:I1P:bus1", "kA, Positive-Sequence Active Current"
      m_I1Q_bus1 = "m:I1Q:bus1", "kA, Positive-Sequence Reactive Current"
      m_ur_bus1 = "m:ur:bus1", "p.u., Voltage, Real Part"
      m_ui_bus1 = "m:ui:bus1", "p.u., Voltage, Imaginary Part"
      m_u_bus1 = "m:u:bus1", "p.u., Voltage, Magnitude"
      m_ir_bus1 = "m:ir:bus1", "p.u., Current, Real Part"
      m_ii_bus1 = "m:ii:bus1", "p.u., Current, Imaginary Part"
      m_i_bus1 = "m:i:bus1", "p.u., Current, Magnitude"
      m_phiui_bus1 = "m:phiui:bus1", "deg, Angle between Voltage and Current"
      m_phiu1i1_bus1 = "m:phiu1i1:bus1", "deg, Angle between Voltage and Current in positive sequence system"
      m_Tfct_bus1 = "m:Tfct:bus1", "s, Fault Clearing Time"
      m_Brkload_bus1 = "m:Brkload:bus1", "%, Breaker Loading"
      n_ur_bus1 = "n:ur:bus1", "p.u., Voltage, Real Part"
      n_ui_bus1 = "n:ui:bus1", "p.u., Voltage, Imaginary Part"
      n_u_bus1 = "n:u:bus1", "p.u., Voltage, Magnitude"
      n_upc_bus1 = "n:upc:bus1", "%, Voltage, Magnitude"
      n_u1_bus1 = "n:u1:bus1", "p.u., Voltage, Magnitude"
      n_u1pc_bus1 = "n:u1pc:bus1", "%, Voltage, Magnitude"
      n_u1r_bus1 = "n:u1r:bus1", "p.u., Positive-Sequence Voltage, Real Part"
      n_u1i_bus1 = "n:u1i:bus1", "p.u., Positive-Sequence Voltage, Imaginary Part"
      n_U_bus1 = "n:U:bus1", "kV, Line-Ground Voltage, Magnitude"
      n_Ul_bus1 = "n:Ul:bus1", "kV, Line-Line Voltage, Magnitude"
      n_phiu_bus1 = "n:phiu:bus1", "deg, Voltage, Angle"
      n_phiurel_bus1 = "n:phiurel:bus1", "deg, Voltage, Relative Angle"
      n_du_bus1 = "n:du:bus1", "%, Voltage Deviation"
      n_Pgen_bus1 = "n:Pgen:bus1", "MW, Generation, Active Power"
      n_Qgen_bus1 = "n:Qgen:bus1", "Mvar, Generation, Reactive Power"
      n_Pmot_bus1 = "n:Pmot:bus1", "MW, Motor Load, Active Power"
      n_Qmot_bus1 = "n:Qmot:bus1", "Mvar, Motor Load, Reactive Power"
      n_Pload_bus1 = "n:Pload:bus1", "MW, General Load, Active Power"
      n_Qload_bus1 = "n:Qload:bus1", "Mvar, General Load, Reactive Power"
      n_Pcomp_bus1 = "n:Pcomp:bus1", "MW, Compensation (Losses)"
      n_Qcomp_bus1 = "n:Qcomp:bus1", "Mvar, Compensation"
      n_Pnet_bus1 = "n:Pnet:bus1", "MW, External Networks, Active Power"
      n_Qnet_bus1 = "n:Qnet:bus1", "Mvar, External Networks, Reactive Power"
      n_Pflow_bus1 = "n:Pflow:bus1", "MW, Power Flow, Active Power"
      n_Qflow_bus1 = "n:Qflow:bus1", "Mvar, Power Flow, Reactive Power"
      n_Pout_bus1 = "n:Pout:bus1", "MW, Outgoing Flow, Active Power"
      n_Qout_bus1 = "n:Qout:bus1", "Mvar, Outgoing Power, Reactive Power"
      n_Sout_bus1 = "n:Sout:bus1", "MVA, Outgoing Power, Apparent Power"
      n_cosphiout_bus1 = "n:cosphiout:bus1", "Outgoing Power, Power Factor"
      n_Pbalance_bus1 = "n:Pbalance:bus1", "MW, Active Power Balance (=0)"
      n_Qbalance_bus1 = "n:Qbalance:bus1", "Mvar, Reactive Power Balance (=0)"
      n_LossPdown_bus1 = "n:LossPdown:bus1", "MW, Losses, downstream"
      n_LossQdown_bus1 = "n:LossQdown:bus1", "Mvar, Losses, downstream (Reactive Power)"
      n_LossPdownload_bus1 = "n:LossPdownload:bus1", "MW, Load losses, downstream"
      n_LossQdownload_bus1 = "n:LossQdownload:bus1", "Mvar, Load losses, downstream"
      n_LossPdownnoload_bus1 = "n:LossPdownnoload:bus1", "MW, No load losses, downstream"
      n_LossQdownnoload_bus1 = "n:LossQdownnoload:bus1", "Mvar, No load losses, downstream"
      n_du_feed_bus1 = "n:du_feed:bus1", "%, Voltage difference relative to feeder begin"
      s_pset = "s:pset", "MW, IN, Active Power Setpoint, continuous, double"
      s_qset = "s:qset", "Mvar, IN, Reactive Power Setpoint continuous, double"
      s_dFin = "s:dFin", "Hz, IN, Frequency Deviation, continuous, double"
      c_p = "c:p", "MW, Total Active Power"
      c_q = "c:q", "Mvar, Total Reactive Power"
      c_sgnn = "c:sgnn", "MVA, Total rated apparent power (ngnum*sgn)"
      c_cosn = "c:cosn", "Nominal-Power factor"
      c_Pnom = "c:Pnom", "MW, Total Nominal Active power"
      c_c2 = "c:c2", "Voltage Coefficient (uknom/ugn)"
      c_uini_r = "c:uini:r", "p.u., Voltage Setpoint for PV and Slack, Real Part"
      c_uini_i = "c:uini:i", "p.u., Voltage Setpoint for PV and Slack, Imaginary Part"
      c_uini_phi = "c:uini:phi", "deg, Voltage Setpoint for PV and Slack, Angle"
      c_uini_mag = "c:uini:mag", "p.u., Voltage Setpoint for PV and Slack, Magnitude"
      c_uini_phirad = "c:uini:phirad", "rad, Voltage Setpoint for PV and Slack, Angle"
      c_phiini = "c:phiini", "Initial Angle"
      c_loading = "c:loading", "%, Loading"
      c_iPowerOn = "c:iPowerOn", "Flag"
      c_Qmin = "c:Qmin", "Mvar, Lower limit of reactive power"
      c_Qmax = "c:Qmax", "Mvar, Upper limit of reactive power"
      c_Qsparelead = "c:Qsparelead", "Mvar, Q Spare Lead"
      c_Qsparelag = "c:Qsparelag", "Mvar, Q Spare Lag"
      c_i_qlim = "c:i_qlim", "Reactive power limit reached"
      c_i_vlim = "c:i_vlim", "Voltage setpoint limit reached"
      c_Pmin = "c:Pmin", "MW, Lower limit of active power"
      c_Pmax = "c:Pmax", "MW, Upper limit of active power"
      c_i_plim = "c:i_plim", "Active power limit reached"
      c_sm_r = "c:sm:r", "MW, Complex Power, Real Part"
      c_sm_i = "c:sm:i", "Mvar, Complex Power, Imaginary Part"
      c_sm_phi = "c:sm:phi", "deg, Complex Power, Angle"
      c_sm_mag = "c:sm:mag", "MVA, Complex Power, Magnitude"
      c_sm_phirad = "c:sm:phirad", "rad, Complex Power, Angle"
      c_u1_r = "c:u1:r", "p.u., Internal, Real Part"
      c_u1_i = "c:u1:i", "p.u., Internal, Imaginary Part"
      c_u1_phi = "c:u1:phi", "deg, Internal, Angle"
      c_u1_mag = "c:u1:mag", "p.u., Internal, Magnitude"
      c_u1_phirad = "c:u1:phirad", "rad, Internal, Angle"
      c_iRefElement = "c:iRefElement", "Reference Element"
      c_Snom = "c:Snom", "MVA, Nom. Apparent Power (1 mach.)"
      c_Nnom = "c:Nnom", "Number of parallel units"
      c_Kp = "c:Kp", "p.u., Active Power Participation"
      c_Kq = "c:Kq", "p.u., Reactive Power Participation"
      c_Kpf = "c:Kpf", "MW/Hz, Frequency Bias"
      c_zonewscale = "c:zonewscale", "Wind generation zone scaling factor"
      c_Pmism = "c:Pmism", "MW, Active Power Mismatch"
      c_nCtrlIter = "c:nCtrlIter", "No. of CTRL-iterations"
      c_Ploss = "c:Ploss", "MW, Losses (total)"
      c_Qloss = "c:Qloss", "Mvar, Reactive-Losses (total)"
      c_Plossld = "c:Plossld", "MW, Losses (load)"
      c_Qlossld = "c:Qlossld", "Mvar, Reactive-Losses (load)"
      c_Plossnld = "c:Plossnld", "MW, Losses (no load)"
      c_Qlossnld = "c:Qlossnld", "Mvar, Reactive-Losses (no load)"
      e_ip_ctrl = "e:ip_ctrl", "Reference Machine"
      e_outServPzero = "e:outServPzero", "Out of service when active power is zero"
      e_av_mode = "e:av_mode", "Local Controller"
      e_c_psecc = "e:c_psecc", "ElmSecctrl, External Secondary Controller"
      e_c_pstac = "e:c_pstac", "ElmStactrl, External Station Controller"
      e_qdslCtrl = "e:qdslCtrl", "Quasi-Dynamic Model"
      e_pgini = "e:pgini", "kW, Active Power"
      e_Kpf = "e:Kpf", "kW/Hz, Prim. Frequency Bias"
      e_mode_inp = "e:mode_inp", "Input Mode"
      e_qgini = "e:qgini", "kvar, Reactive Power"
      e_cosgini = "e:cosgini", "Power Factor"
      e_pf_recap = "e:pf_recap", "Power Factor"
      e_usetp = "e:usetp", "p.u., Voltage"
      e_phiini = "e:phiini", "deg, Angle"
      e_scale0 = "e:scale0", "Scaling Factor"
      e_pgini_a = "e:pgini_a", "kW, Active Power (act.)"
      e_qgini_a = "e:qgini_a", "kvar, Reactive Power (act.)"
      e_sgini_a = "e:sgini_a", "kVA, Apparent Power (act.)"
      e_cosgini_a = "e:cosgini_a", "Power Factor (act.)"
      e_pf_recap_a = "e:pf_recap_a", "Power Factor Ind/Cap (act.)"
      e_scale0_a = "e:scale0_a", "Scaling Factor(act.)"
      e_usp_min = "e:usp_min", "p.u., Min. Voltage Setpoint"
      e_usp_max = "e:usp_max", "p.u., Max. Voltage Setpoint"
      e_ddroop = "e:ddroop", "%, Droop"
      e_pQPcurve = "e:pQPcurve", "IntQpcurve, Q(P)-Curve"
      e_iopt_tdr = "e:iopt_tdr", "Different droop values"
      e_ddroopoe = "e:ddroopoe", "%, Droop (overexcited)"
      e_ddroopue = "e:ddroopue", "%, Droop (underexcited)"
      e_Qfu_max = "e:Qfu_max", "kvar, Q max"
      e_Qfu_min = "e:Qfu_min", "kvar, Q min"
      e_udeadblow = "e:udeadblow", "p.u., Lower Voltage Limit"
      e_udeadbup = "e:udeadbup", "p.u., Upper Voltage Limit"
      e_pf_over = "e:pf_over", "Min. Power Factor"
      e_p_over = "e:p_over", "kW, Active Power"
      e_pf_under = "e:pf_under", "Min. Power Factor"
      e_p_under = "e:p_under", "kW, Active Power"
      e_c_pCtrlHV = "e:c_pCtrlHV", "Controlled HV-busbar"
      e_Uctrl = "e:Uctrl", "kV, HV-Voltage Setpoint"
      e_pQlimType = "e:pQlimType", "IntQlim, Capability Curve"
      e_q_min = "e:q_min", "p.u., Min."
      e_q_max = "e:q_max", "p.u., Max."
      e_cQ_min = "e:cQ_min", "kvar, Minimum"
      e_cQ_max = "e:cQ_max", "kvar, Maximum"
      e_scaleQmin = "e:scaleQmin", "%, Scaling Factor (min.)"
      e_scaleQmax = "e:scaleQmax", "%, Scaling Factor (max.)"
      e_Pmin_uc = "e:Pmin_uc", "kW, Min."
      e_Pmin_ucPU = "e:Pmin_ucPU", "p.u., Min."
      e_Pmax_uc = "e:Pmax_uc", "kW, Max."
      e_Pmax_ucPU = "e:Pmax_ucPU", "p.u., Max."
      e_Pnom = "e:Pnom", "kW, Pr(rated)"
      e_P_max = "e:P_max", "kW, Max."
      e_pmaxratf = "e:pmaxratf", "Rating Factor"
      e_Qmin_a = "e:Qmin_a", "kvar, Qmin(act.)"
      e_Qmax_a = "e:Qmax_a", "kvar, Qmax(act.)"
      e_Pmin_a = "e:Pmin_a", "kW, Pmin(act.)"
      e_Pmax_a = "e:Pmax_a", "kW, Pmax(act.)"
      e_cGPSlat = "e:cGPSlat", "deg, Latitude"
      e_cGPSlon = "e:cGPSlon", "deg, Longitude"
      e_iopt_rad = "e:iopt_rad", "Specified Components"
      e_iopt_glo = "e:iopt_glo", "Global Irradiance Data"
      e_ghi = "e:ghi", "W/m^2, Global Horizontal Irradiance"
      e_kt = "e:kt", "Clearness Index"
      e_iopt_dif = "e:iopt_dif", "Diffuse Irradiance Data"
      e_iopt_dir = "e:iopt_dir", "Direct Irradiance Data"
      e_dhi = "e:dhi", "W/m^2, Direct Horizontal Irradiance"
      e_dni = "e:dni", "W/m^2, Direct Normal Irradiance"
      e_ghi_a = "e:ghi_a", "W/m^2, Global Horizontal Irradiance (act.)"
      e_kt_a = "e:kt_a", "Clearness Index (act.)"
      e_dhi_a = "e:dhi_a", "W/m^2, Direct Horizontal Irradiance (act.)"
      e_dni_a = "e:dni_a", "W/m^2, Direct Normal Irradiance (act.)"
      e_Tamb = "e:Tamb", "degC, Ambient Temperature"
      e_shfdir = "e:shfdir", "Shading Factor (Direct)"
      e_shfdiff = "e:shfdiff", "Shading Factor (Diffuse)"
      e_albedo = "e:albedo", "Ground Albedo"
      e_Tamb_a = "e:Tamb_a", "degC, Amb. Temp. (act.)"
      e_shfdir_a = "e:shfdir_a", "Shading Factor (Dir.) (act.)"
      e_shfdiff_a = "e:shfdiff_a", "Shading Factor (Diff.) (act.)"
      e_albedo_a = "e:albedo_a", "Albedo (act.)"
      e_Egh_a = "e:Egh_a", "W/m^2, Global Horizontal Irradiance (act.)"
      e_Ebh_a = "e:Ebh_a", "W/m^2, Direct Horizontal Irradiance (act.)"
      e_Edh_a = "e:Edh_a", "W/m^2, Diffuse Horizontal Irradiance (act.)"
      e_Egpv_a = "e:Egpv_a", "W/m^2, Global Irradiance, PV Panel (act.)"
      e_Ebpv_a = "e:Ebpv_a", "W/m^2, Direct Irradiance, PV Panel (act.)"
      e_Edpv_a = "e:Edpv_a", "W/m^2, Diffuse Irradiance, PV Panel (act.)"
      e_Erpv_a = "e:Erpv_a", "W/m^2, Reflected Irradiance, PV Panel (act.)"
      e_RelEff_a = "e:RelEff_a", "Relative Efficiency (act.)"
      e_ktc_a = "e:ktc_a", "W/m^2, Calculated Clearness Index"
      e_solalt_a = "e:solalt_a", "deg, Solar Altitude Angle (act.)"
      e_anginc_a = "e:anginc_a", "deg, Angle of Incidence (act.)"
      e_orient_a = "e:orient_a", "deg, Orientation Angle (act.)"
      e_tilt_a = "e:tilt_a", "deg, Tilt Angle (act.)"
      e_r2 = "e:r2", "p.u., Resistance r2"
      e_x2 = "e:x2", "p.u., Reactance x2"
      e_is4SpinReserve = "e:is4SpinReserve", "Consider for region spinning reserve"
      e_genShiftKey = "e:genShiftKey", "%, Generation shift key"
      e_pBMU = "e:pBMU", "ElmBmu, Virtual Power Plant"
      e_dispatch = "e:dispatch", "Generator Dispatch"
      e_priority = "e:priority", "Merit Order"
      e_fixed = "e:fixed", "Must run"
      r_c_psecc = "r:c_psecc", "ElmSecctrl, External Secondary Controller"
      r_c_pstac = "r:c_pstac", "ElmStactrl, External Station Controller"
      r_qdslCtrl = "r:qdslCtrl", "Quasi-Dynamic Model"
      r_pQPcurve = "r:pQPcurve", "IntQpcurve, Q(P)-Curve"
      r_c_pCtrlHV = "r:c_pCtrlHV", "Controlled HV-busbar"
      r_pQlimType = "r:pQlimType", "IntQlim, Capability Curve"
      r_pBMU = "r:pBMU", "ElmBmu, Virtual Power Plant"

    class ElmSecctrl(Enum):
      _init_ = 'value __doc__'

      s_pset = "s:pset", "MW, IN, Active Power Setpoint, continuous, double"
      s_dpsco = "s:dpsco", "MW, OUT, Controller Output, continuous, double"
      c_rembar = "c:rembar", "Controlled Busbar"
      c_pini = "c:pini", "MW, Active Power Setpoint"
      c_imode = "c:imode", "Control Mode"
      c_i_modus = "c:i_modus", "Control Mode 0=phi 1=p"
      c_phiu = "c:phiu", "rad, Phase Angle"
      c_PExch = "c:PExch", "MW, Power Exchange"
      c_PctrlLim = "c:PctrlLim", "MW, Controller Output Limit"
      e_i_net = "e:i_net", "Control Mode"
      e_rembar = "e:rembar", "StaBar*,ElmTerm*, Busbar for Frequency Measurement"
      e_iexchange = "e:iexchange", "Exchange for"
      e_cpGrid = "e:cpGrid", "Grid"
      e_pPmeas = "e:pPmeas", "ElmBoundary,ElmArea,ElmZone, Boundary/Zone/Area"
      e_psetp = "e:psetp", "MW, Power Setpoint"
      e_Kpf = "e:Kpf", "MW/Hz, Frequency Bias"
      e_imode = "e:imode", "Active Power Distribution"
      e_psym = "e:psym", "ElmSym,ElmGenstat,ElmVscmono,ElmVsc,ElmVac,ElmXnet,ElmPvsys, Machines"
      e_cvpp = "e:cvpp", "%, Active Power Percentage"
      e_cmo = "e:cmo", "Merit Order"
      e_qdslCtrl = "e:qdslCtrl", "Quasi-Dynamic Model"
      r_rembar = "r:rembar", "StaBar*,ElmTerm*, Busbar for Frequency Measurement"
      r_cpGrid = "r:cpGrid", "Grid"
      r_pPmeas = "r:pPmeas", "ElmBoundary,ElmArea,ElmZone, Boundary/Zone/Area"
      r_psym = "r:psym", "ElmSym,ElmGenstat,ElmVscmono,ElmVsc,ElmVac,ElmXnet,ElmPvsys, Machines"
      r_qdslCtrl = "r:qdslCtrl", "Quasi-Dynamic Model"

    class ElmShnt(Enum):
      _init_ = 'value __doc__'

      m_u1r_bus1 = "m:u1r:bus1", "p.u., Positive-Sequence-Voltage, Real Part"
      m_u1i_bus1 = "m:u1i:bus1", "p.u., Positive-Sequence-Voltage, Imaginary Part"
      m_u1_bus1 = "m:u1:bus1", "p.u., Positive-Sequence-Voltage, Magnitude"
      m_U1_bus1 = "m:U1:bus1", "kV, Line-Ground Positive-Sequence-Voltage, Magnitude"
      m_U1l_bus1 = "m:U1l:bus1", "kV, Line-Line Positive-Sequence-Voltage, Magnitude"
      m_phiu1_bus1 = "m:phiu1:bus1", "deg, Positive-Sequence-Voltage, Angle"
      m_inet_bus1 = "m:inet:bus1", "p.u., Current, Magnitude, referred to network"
      m_I_bus1 = "m:I:bus1", "kA, Current, Magnitude"
      m_phii_bus1 = "m:phii:bus1", "deg, Current, Angle"
      m_i1r_bus1 = "m:i1r:bus1", "p.u., Positive-Sequence Current, Real Part"
      m_i1i_bus1 = "m:i1i:bus1", "p.u., Positive-Sequence Current, Imaginary Part"
      m_i1_bus1 = "m:i1:bus1", "p.u., Positive-Sequence Current, Magnitude"
      m_I1_bus1 = "m:I1:bus1", "kA, Positive-Sequence Current, Magnitude"
      m_phii1_bus1 = "m:phii1:bus1", "deg, Positive-Sequence Current, Angle"
      m_P_bus1 = "m:P:bus1", "MW, Active Power"
      m_Q_bus1 = "m:Q:bus1", "Mvar, Reactive Power"
      m_S_bus1 = "m:S:bus1", "MVA, Apparent Power"
      m_cosphi_bus1 = "m:cosphi:bus1", "Power Factor"
      m_tanphi_bus1 = "m:tanphi:bus1", "tan(phi)"
      m_Psum_bus1 = "m:Psum:bus1", "MW, Total Active Power"
      m_Qsum_bus1 = "m:Qsum:bus1", "Mvar, Total Reactive Power"
      m_Ssum_bus1 = "m:Ssum:bus1", "MVA, Total Apparent Power"
      m_cosphisum_bus1 = "m:cosphisum:bus1", "Total Power Factor"
      m_tanphisum_bus1 = "m:tanphisum:bus1", "Total tan(phi)"
      m_Spu_bus1 = "m:Spu:bus1", "MVA/p.u., Apparent Power per p.u. Voltage"
      m_i1P_bus1 = "m:i1P:bus1", "p.u., Positive-Sequence Active Current"
      m_i1Q_bus1 = "m:i1Q:bus1", "p.u., Positive-Sequence Reactive Current"
      m_I1P_bus1 = "m:I1P:bus1", "kA, Positive-Sequence Active Current"
      m_I1Q_bus1 = "m:I1Q:bus1", "kA, Positive-Sequence Reactive Current"
      m_ur_bus1 = "m:ur:bus1", "p.u., Voltage, Real Part"
      m_ui_bus1 = "m:ui:bus1", "p.u., Voltage, Imaginary Part"
      m_u_bus1 = "m:u:bus1", "p.u., Voltage, Magnitude"
      m_ir_bus1 = "m:ir:bus1", "p.u., Current, Real Part"
      m_ii_bus1 = "m:ii:bus1", "p.u., Current, Imaginary Part"
      m_i_bus1 = "m:i:bus1", "p.u., Current, Magnitude"
      m_phiui_bus1 = "m:phiui:bus1", "deg, Angle between Voltage and Current"
      m_phiu1i1_bus1 = "m:phiu1i1:bus1", "deg, Angle between Voltage and Current in positive sequence system"
      m_Tfct_bus1 = "m:Tfct:bus1", "s, Fault Clearing Time"
      m_Brkload_bus1 = "m:Brkload:bus1", "%, Breaker Loading"
      n_ur_bus1 = "n:ur:bus1", "p.u., Voltage, Real Part"
      n_ui_bus1 = "n:ui:bus1", "p.u., Voltage, Imaginary Part"
      n_u_bus1 = "n:u:bus1", "p.u., Voltage, Magnitude"
      n_upc_bus1 = "n:upc:bus1", "%, Voltage, Magnitude"
      n_u1_bus1 = "n:u1:bus1", "p.u., Voltage, Magnitude"
      n_u1pc_bus1 = "n:u1pc:bus1", "%, Voltage, Magnitude"
      n_u1r_bus1 = "n:u1r:bus1", "p.u., Positive-Sequence Voltage, Real Part"
      n_u1i_bus1 = "n:u1i:bus1", "p.u., Positive-Sequence Voltage, Imaginary Part"
      n_U_bus1 = "n:U:bus1", "kV, Line-Ground Voltage, Magnitude"
      n_Ul_bus1 = "n:Ul:bus1", "kV, Line-Line Voltage, Magnitude"
      n_phiu_bus1 = "n:phiu:bus1", "deg, Voltage, Angle"
      n_phiurel_bus1 = "n:phiurel:bus1", "deg, Voltage, Relative Angle"
      n_du_bus1 = "n:du:bus1", "%, Voltage Deviation"
      n_Pgen_bus1 = "n:Pgen:bus1", "MW, Generation, Active Power"
      n_Qgen_bus1 = "n:Qgen:bus1", "Mvar, Generation, Reactive Power"
      n_Pmot_bus1 = "n:Pmot:bus1", "MW, Motor Load, Active Power"
      n_Qmot_bus1 = "n:Qmot:bus1", "Mvar, Motor Load, Reactive Power"
      n_Pload_bus1 = "n:Pload:bus1", "MW, General Load, Active Power"
      n_Qload_bus1 = "n:Qload:bus1", "Mvar, General Load, Reactive Power"
      n_Pcomp_bus1 = "n:Pcomp:bus1", "MW, Compensation (Losses)"
      n_Qcomp_bus1 = "n:Qcomp:bus1", "Mvar, Compensation"
      n_Pnet_bus1 = "n:Pnet:bus1", "MW, External Networks, Active Power"
      n_Qnet_bus1 = "n:Qnet:bus1", "Mvar, External Networks, Reactive Power"
      n_Pflow_bus1 = "n:Pflow:bus1", "MW, Power Flow, Active Power"
      n_Qflow_bus1 = "n:Qflow:bus1", "Mvar, Power Flow, Reactive Power"
      n_Pout_bus1 = "n:Pout:bus1", "MW, Outgoing Flow, Active Power"
      n_Qout_bus1 = "n:Qout:bus1", "Mvar, Outgoing Power, Reactive Power"
      n_Sout_bus1 = "n:Sout:bus1", "MVA, Outgoing Power, Apparent Power"
      n_cosphiout_bus1 = "n:cosphiout:bus1", "Outgoing Power, Power Factor"
      n_Pbalance_bus1 = "n:Pbalance:bus1", "MW, Active Power Balance (=0)"
      n_Qbalance_bus1 = "n:Qbalance:bus1", "Mvar, Reactive Power Balance (=0)"
      n_LossPdown_bus1 = "n:LossPdown:bus1", "MW, Losses, downstream"
      n_LossQdown_bus1 = "n:LossQdown:bus1", "Mvar, Losses, downstream (Reactive Power)"
      n_LossPdownload_bus1 = "n:LossPdownload:bus1", "MW, Load losses, downstream"
      n_LossQdownload_bus1 = "n:LossQdownload:bus1", "Mvar, Load losses, downstream"
      n_LossPdownnoload_bus1 = "n:LossPdownnoload:bus1", "MW, No load losses, downstream"
      n_LossQdownnoload_bus1 = "n:LossQdownnoload:bus1", "Mvar, No load losses, downstream"
      n_du_feed_bus1 = "n:du_feed:bus1", "%, Voltage difference relative to feeder begin"
      s_ncapain = "s:ncapain", "IN, Number of Stages in Parallel, input, continuous, double"
      c_z_r = "c:z:r", "p.u., Shunt-Impedance, Real Part"
      c_z_i = "c:z:i", "p.u., Shunt-Impedance, Imaginary Part"
      c_z_phi = "c:z:phi", "deg, Shunt-Impedance, Angle"
      c_z_mag = "c:z:mag", "p.u., Shunt-Impedance, Magnitude"
      c_z_phirad = "c:z:phirad", "rad, Shunt-Impedance, Angle"
      c_cn = "c:cn", "Connection Mode"
      c_frnom = "c:frnom", "Hz, Nominal Frequency"
      c_hpi = "c:hpi", "rad/s, Nominal Angular Frequency"
      c_rs = "c:rs", "p.u., Rs"
      c_gp = "c:gp", "p.u., Gp"
      c_xl = "c:xl", "p.u., Xl"
      c_bc = "c:bc", "p.u., Bc"
      c_b0 = "c:b0", "p.u., Susceptance to Ground"
      c_gpc = "c:gpc", "p.u., Gp(C)"
      c_uc = "c:uc", "p.u., Voltage across Capacitor"
      c_IL = "c:IL", "A, Current through Inductor L"
      c_IC = "c:IC", "A, Current through Capacitor C"
      c_IRp = "c:IRp", "A, Current through Rp"
      c_PRp = "c:PRp", "kW, Losses in Rp"
      c_PL = "c:PL", "kW, Losses in Inductor (Rs)"
      c_url = "c:url", "p.u., Voltage across Inductor (Rs-L)"
      c_Uc = "c:Uc", "kV, Voltage across Capacitor"
      c_Url = "c:Url", "kV, Voltage across Inductor (Rs-L)"
      c_ncapa = "c:ncapa", "Number of Stages in Parallel"
      c_acost = "c:acost", "Annual Cost"
      c_ncapx = "c:ncapx", "Max. No. of Steps"
      c_nCtrlIter = "c:nCtrlIter", "No. of CTRL-iterations"
      c_Tctrl = "c:Tctrl", "Controller Time Constant"
      c_Kctrl = "c:Kctrl", "p.u./%, Controller Sensitivity dq/dv"
      c_Ploss = "c:Ploss", "MW, Losses (total)"
      c_Qloss = "c:Qloss", "Mvar, Reactive-Losses (total)"
      c_Plossld = "c:Plossld", "MW, Losses (load)"
      c_Qlossld = "c:Qlossld", "Mvar, Reactive-Losses (load)"
      c_Plossnld = "c:Plossnld", "MW, Losses (no load)"
      c_Qlossnld = "c:Qlossnld", "Mvar, Reactive-Losses (no load)"
      e_ncapx = "e:ncapx", "Max. No. of Steps"
      e_ncapa = "e:ncapa", "Act.No. of Step"
      e_Qmax = "e:Qmax", "Mvar, Max. Rated Reactive Power"
      e_Qact = "e:Qact", "Mvar, Actual Reactive Power"
      e_c_pctrl = "e:c_pctrl", "ElmShntctrl, Shunt Controller"
      e_qdslCtrl = "e:qdslCtrl", "Quasi-Dynamic Model"
      e_iswitch = "e:iswitch", "Switchable"
      e_i_cont = "e:i_cont", "Tap Changer"
      e_imldc = "e:imldc", "Control Mode"
      e_ilcph = "e:ilcph", "Phase"
      e_iQorient = "e:iQorient", "Orientation"
      e_uset_mode = "e:uset_mode", "Setpoint"
      e_i_rem = "e:i_rem", "Remote Control"
      e_p_rem = "e:p_rem", "StaBar,ElmTerm, Controlled Node"
      e_p_cub = "e:p_cub", "StaCubic,ElmBoundary, Q Controlled at"
      e_cpCtrlNode = "e:cpCtrlNode", "Target Node"
      e_usetp_mx = "e:usetp_mx", "p.u., Upper Voltage Limit"
      e_usetp_mn = "e:usetp_mn", "p.u., Lower Voltage Limit"
      e_qsetp_mx = "e:qsetp_mx", "Mvar, Upper Reactive Power Limit"
      e_qsetp_mn = "e:qsetp_mn", "Mvar, Lower Reactive Power Limit"
      e_pfsetp_mx = "e:pfsetp_mx", "Upper Power Factor Limit"
      e_pf_recap_mx = "e:pf_recap_mx", "Power Factor"
      e_pfsetp_mn = "e:pfsetp_mn", "Lower Power Factor Limit"
      e_pf_recap_mn = "e:pf_recap_mn", "Power Factor"
      e_Tctrl = "e:Tctrl", "s, Controller Time Constant"
      e_Kctrl = "e:Kctrl", "p.u./%, Controller Sensitivity dq/dv"
      e_cpFeed = "e:cpFeed", "Feeder"
      e_ciDist = "e:ciDist", "Distance from infeed in number of buses"
      e_ciLater = "e:ciLater", "Lateral Index"
      e_ciDistRoot = "e:ciDistRoot", "Distance from first infeed in number of buses"
      e_ciDistAll = "e:ciDistAll", "Distance from infeed in number of buses including switches"
      e_ciDistAllRoot = "e:ciDistAllRoot", "Distance from first infeed in number of buses including switches"
      r_c_pctrl = "r:c_pctrl", "ElmShntctrl, Shunt Controller"
      r_qdslCtrl = "r:qdslCtrl", "Quasi-Dynamic Model"
      r_p_rem = "r:p_rem", "StaBar,ElmTerm, Controlled Node"
      r_p_cub = "r:p_cub", "StaCubic,ElmBoundary, Q Controlled at"
      r_cpCtrlNode = "r:cpCtrlNode", "Target Node"
      r_cpFeed = "r:cpFeed", "Feeder"

    class ElmSind(Enum):
      _init_ = 'value __doc__'

      m_u1r_bus1 = "m:u1r:bus1", "p.u., Positive-Sequence-Voltage, Real Part"
      m_u1r_bus2 = "m:u1r:bus2", "p.u., Positive-Sequence-Voltage, Real Part"
      m_u1i_bus1 = "m:u1i:bus1", "p.u., Positive-Sequence-Voltage, Imaginary Part"
      m_u1i_bus2 = "m:u1i:bus2", "p.u., Positive-Sequence-Voltage, Imaginary Part"
      m_u1_bus1 = "m:u1:bus1", "p.u., Positive-Sequence-Voltage, Magnitude"
      m_u1_bus2 = "m:u1:bus2", "p.u., Positive-Sequence-Voltage, Magnitude"
      m_U1_bus1 = "m:U1:bus1", "kV, Line-Ground Positive-Sequence-Voltage, Magnitude"
      m_U1_bus2 = "m:U1:bus2", "kV, Line-Ground Positive-Sequence-Voltage, Magnitude"
      m_U1l_bus1 = "m:U1l:bus1", "kV, Line-Line Positive-Sequence-Voltage, Magnitude"
      m_U1l_bus2 = "m:U1l:bus2", "kV, Line-Line Positive-Sequence-Voltage, Magnitude"
      m_phiu1_bus1 = "m:phiu1:bus1", "deg, Positive-Sequence-Voltage, Angle"
      m_phiu1_bus2 = "m:phiu1:bus2", "deg, Positive-Sequence-Voltage, Angle"
      m_inet_bus1 = "m:inet:bus1", "p.u., Current, Magnitude, referred to network"
      m_inet_bus2 = "m:inet:bus2", "p.u., Current, Magnitude, referred to network"
      m_I_bus1 = "m:I:bus1", "kA, Current, Magnitude"
      m_I_bus2 = "m:I:bus2", "kA, Current, Magnitude"
      m_phii_bus1 = "m:phii:bus1", "deg, Current, Angle"
      m_phii_bus2 = "m:phii:bus2", "deg, Current, Angle"
      m_i1r_bus1 = "m:i1r:bus1", "p.u., Positive-Sequence Current, Real Part"
      m_i1r_bus2 = "m:i1r:bus2", "p.u., Positive-Sequence Current, Real Part"
      m_i1i_bus1 = "m:i1i:bus1", "p.u., Positive-Sequence Current, Imaginary Part"
      m_i1i_bus2 = "m:i1i:bus2", "p.u., Positive-Sequence Current, Imaginary Part"
      m_i1_bus1 = "m:i1:bus1", "p.u., Positive-Sequence Current, Magnitude"
      m_i1_bus2 = "m:i1:bus2", "p.u., Positive-Sequence Current, Magnitude"
      m_I1_bus1 = "m:I1:bus1", "kA, Positive-Sequence Current, Magnitude"
      m_I1_bus2 = "m:I1:bus2", "kA, Positive-Sequence Current, Magnitude"
      m_phii1_bus1 = "m:phii1:bus1", "deg, Positive-Sequence Current, Angle"
      m_phii1_bus2 = "m:phii1:bus2", "deg, Positive-Sequence Current, Angle"
      m_P_bus1 = "m:P:bus1", "MW, Active Power"
      m_P_bus2 = "m:P:bus2", "MW, Active Power"
      m_Q_bus1 = "m:Q:bus1", "Mvar, Reactive Power"
      m_Q_bus2 = "m:Q:bus2", "Mvar, Reactive Power"
      m_S_bus1 = "m:S:bus1", "MVA, Apparent Power"
      m_S_bus2 = "m:S:bus2", "MVA, Apparent Power"
      m_cosphi_bus1 = "m:cosphi:bus1", "Power Factor"
      m_cosphi_bus2 = "m:cosphi:bus2", "Power Factor"
      m_tanphi_bus1 = "m:tanphi:bus1", "tan(phi)"
      m_tanphi_bus2 = "m:tanphi:bus2", "tan(phi)"
      m_Psum_bus1 = "m:Psum:bus1", "MW, Total Active Power"
      m_Psum_bus2 = "m:Psum:bus2", "MW, Total Active Power"
      m_Qsum_bus1 = "m:Qsum:bus1", "Mvar, Total Reactive Power"
      m_Qsum_bus2 = "m:Qsum:bus2", "Mvar, Total Reactive Power"
      m_Ssum_bus1 = "m:Ssum:bus1", "MVA, Total Apparent Power"
      m_Ssum_bus2 = "m:Ssum:bus2", "MVA, Total Apparent Power"
      m_cosphisum_bus1 = "m:cosphisum:bus1", "Total Power Factor"
      m_cosphisum_bus2 = "m:cosphisum:bus2", "Total Power Factor"
      m_tanphisum_bus1 = "m:tanphisum:bus1", "Total tan(phi)"
      m_tanphisum_bus2 = "m:tanphisum:bus2", "Total tan(phi)"
      m_Spu_bus1 = "m:Spu:bus1", "MVA/p.u., Apparent Power per p.u. Voltage"
      m_Spu_bus2 = "m:Spu:bus2", "MVA/p.u., Apparent Power per p.u. Voltage"
      m_i1P_bus1 = "m:i1P:bus1", "p.u., Positive-Sequence Active Current"
      m_i1P_bus2 = "m:i1P:bus2", "p.u., Positive-Sequence Active Current"
      m_i1Q_bus1 = "m:i1Q:bus1", "p.u., Positive-Sequence Reactive Current"
      m_i1Q_bus2 = "m:i1Q:bus2", "p.u., Positive-Sequence Reactive Current"
      m_I1P_bus1 = "m:I1P:bus1", "kA, Positive-Sequence Active Current"
      m_I1P_bus2 = "m:I1P:bus2", "kA, Positive-Sequence Active Current"
      m_I1Q_bus1 = "m:I1Q:bus1", "kA, Positive-Sequence Reactive Current"
      m_I1Q_bus2 = "m:I1Q:bus2", "kA, Positive-Sequence Reactive Current"
      m_ur_bus1 = "m:ur:bus1", "p.u., Voltage, Real Part"
      m_ur_bus2 = "m:ur:bus2", "p.u., Voltage, Real Part"
      m_ui_bus1 = "m:ui:bus1", "p.u., Voltage, Imaginary Part"
      m_ui_bus2 = "m:ui:bus2", "p.u., Voltage, Imaginary Part"
      m_u_bus1 = "m:u:bus1", "p.u., Voltage, Magnitude"
      m_u_bus2 = "m:u:bus2", "p.u., Voltage, Magnitude"
      m_ir_bus1 = "m:ir:bus1", "p.u., Current, Real Part"
      m_ir_bus2 = "m:ir:bus2", "p.u., Current, Real Part"
      m_ii_bus1 = "m:ii:bus1", "p.u., Current, Imaginary Part"
      m_ii_bus2 = "m:ii:bus2", "p.u., Current, Imaginary Part"
      m_i_bus1 = "m:i:bus1", "p.u., Current, Magnitude"
      m_i_bus2 = "m:i:bus2", "p.u., Current, Magnitude"
      m_phiui_bus1 = "m:phiui:bus1", "deg, Angle between Voltage and Current"
      m_phiui_bus2 = "m:phiui:bus2", "deg, Angle between Voltage and Current"
      m_phiu1i1_bus1 = "m:phiu1i1:bus1", "deg, Angle between Voltage and Current in positive sequence system"
      m_phiu1i1_bus2 = "m:phiu1i1:bus2", "deg, Angle between Voltage and Current in positive sequence system"
      m_Tfct_bus1 = "m:Tfct:bus1", "s, Fault Clearing Time"
      m_Tfct_bus2 = "m:Tfct:bus2", "s, Fault Clearing Time"
      m_Brkload_bus1 = "m:Brkload:bus1", "%, Breaker Loading"
      m_Brkload_bus2 = "m:Brkload:bus2", "%, Breaker Loading"
      n_ur_bus1 = "n:ur:bus1", "p.u., Voltage, Real Part"
      n_ur_bus2 = "n:ur:bus2", "p.u., Voltage, Real Part"
      n_ui_bus1 = "n:ui:bus1", "p.u., Voltage, Imaginary Part"
      n_ui_bus2 = "n:ui:bus2", "p.u., Voltage, Imaginary Part"
      n_u_bus1 = "n:u:bus1", "p.u., Voltage, Magnitude"
      n_u_bus2 = "n:u:bus2", "p.u., Voltage, Magnitude"
      n_upc_bus1 = "n:upc:bus1", "%, Voltage, Magnitude"
      n_upc_bus2 = "n:upc:bus2", "%, Voltage, Magnitude"
      n_u1_bus1 = "n:u1:bus1", "p.u., Voltage, Magnitude"
      n_u1_bus2 = "n:u1:bus2", "p.u., Voltage, Magnitude"
      n_u1pc_bus1 = "n:u1pc:bus1", "%, Voltage, Magnitude"
      n_u1pc_bus2 = "n:u1pc:bus2", "%, Voltage, Magnitude"
      n_u1r_bus1 = "n:u1r:bus1", "p.u., Positive-Sequence Voltage, Real Part"
      n_u1r_bus2 = "n:u1r:bus2", "p.u., Positive-Sequence Voltage, Real Part"
      n_u1i_bus1 = "n:u1i:bus1", "p.u., Positive-Sequence Voltage, Imaginary Part"
      n_u1i_bus2 = "n:u1i:bus2", "p.u., Positive-Sequence Voltage, Imaginary Part"
      n_U_bus1 = "n:U:bus1", "kV, Line-Ground Voltage, Magnitude"
      n_U_bus2 = "n:U:bus2", "kV, Line-Ground Voltage, Magnitude"
      n_Ul_bus1 = "n:Ul:bus1", "kV, Line-Line Voltage, Magnitude"
      n_Ul_bus2 = "n:Ul:bus2", "kV, Line-Line Voltage, Magnitude"
      n_phiu_bus1 = "n:phiu:bus1", "deg, Voltage, Angle"
      n_phiu_bus2 = "n:phiu:bus2", "deg, Voltage, Angle"
      n_phiurel_bus1 = "n:phiurel:bus1", "deg, Voltage, Relative Angle"
      n_phiurel_bus2 = "n:phiurel:bus2", "deg, Voltage, Relative Angle"
      n_du_bus1 = "n:du:bus1", "%, Voltage Deviation"
      n_du_bus2 = "n:du:bus2", "%, Voltage Deviation"
      n_Pgen_bus1 = "n:Pgen:bus1", "MW, Generation, Active Power"
      n_Pgen_bus2 = "n:Pgen:bus2", "MW, Generation, Active Power"
      n_Qgen_bus1 = "n:Qgen:bus1", "Mvar, Generation, Reactive Power"
      n_Qgen_bus2 = "n:Qgen:bus2", "Mvar, Generation, Reactive Power"
      n_Pmot_bus1 = "n:Pmot:bus1", "MW, Motor Load, Active Power"
      n_Pmot_bus2 = "n:Pmot:bus2", "MW, Motor Load, Active Power"
      n_Qmot_bus1 = "n:Qmot:bus1", "Mvar, Motor Load, Reactive Power"
      n_Qmot_bus2 = "n:Qmot:bus2", "Mvar, Motor Load, Reactive Power"
      n_Pload_bus1 = "n:Pload:bus1", "MW, General Load, Active Power"
      n_Pload_bus2 = "n:Pload:bus2", "MW, General Load, Active Power"
      n_Qload_bus1 = "n:Qload:bus1", "Mvar, General Load, Reactive Power"
      n_Qload_bus2 = "n:Qload:bus2", "Mvar, General Load, Reactive Power"
      n_Pcomp_bus1 = "n:Pcomp:bus1", "MW, Compensation (Losses)"
      n_Pcomp_bus2 = "n:Pcomp:bus2", "MW, Compensation (Losses)"
      n_Qcomp_bus1 = "n:Qcomp:bus1", "Mvar, Compensation"
      n_Qcomp_bus2 = "n:Qcomp:bus2", "Mvar, Compensation"
      n_Pnet_bus1 = "n:Pnet:bus1", "MW, External Networks, Active Power"
      n_Pnet_bus2 = "n:Pnet:bus2", "MW, External Networks, Active Power"
      n_Qnet_bus1 = "n:Qnet:bus1", "Mvar, External Networks, Reactive Power"
      n_Qnet_bus2 = "n:Qnet:bus2", "Mvar, External Networks, Reactive Power"
      n_Pflow_bus1 = "n:Pflow:bus1", "MW, Power Flow, Active Power"
      n_Pflow_bus2 = "n:Pflow:bus2", "MW, Power Flow, Active Power"
      n_Qflow_bus1 = "n:Qflow:bus1", "Mvar, Power Flow, Reactive Power"
      n_Qflow_bus2 = "n:Qflow:bus2", "Mvar, Power Flow, Reactive Power"
      n_Pout_bus1 = "n:Pout:bus1", "MW, Outgoing Flow, Active Power"
      n_Pout_bus2 = "n:Pout:bus2", "MW, Outgoing Flow, Active Power"
      n_Qout_bus1 = "n:Qout:bus1", "Mvar, Outgoing Power, Reactive Power"
      n_Qout_bus2 = "n:Qout:bus2", "Mvar, Outgoing Power, Reactive Power"
      n_Sout_bus1 = "n:Sout:bus1", "MVA, Outgoing Power, Apparent Power"
      n_Sout_bus2 = "n:Sout:bus2", "MVA, Outgoing Power, Apparent Power"
      n_cosphiout_bus1 = "n:cosphiout:bus1", "Outgoing Power, Power Factor"
      n_cosphiout_bus2 = "n:cosphiout:bus2", "Outgoing Power, Power Factor"
      n_Pbalance_bus1 = "n:Pbalance:bus1", "MW, Active Power Balance (=0)"
      n_Pbalance_bus2 = "n:Pbalance:bus2", "MW, Active Power Balance (=0)"
      n_Qbalance_bus1 = "n:Qbalance:bus1", "Mvar, Reactive Power Balance (=0)"
      n_Qbalance_bus2 = "n:Qbalance:bus2", "Mvar, Reactive Power Balance (=0)"
      n_LossPdown_bus1 = "n:LossPdown:bus1", "MW, Losses, downstream"
      n_LossPdown_bus2 = "n:LossPdown:bus2", "MW, Losses, downstream"
      n_LossQdown_bus1 = "n:LossQdown:bus1", "Mvar, Losses, downstream (Reactive Power)"
      n_LossQdown_bus2 = "n:LossQdown:bus2", "Mvar, Losses, downstream (Reactive Power)"
      n_LossPdownload_bus1 = "n:LossPdownload:bus1", "MW, Load losses, downstream"
      n_LossPdownload_bus2 = "n:LossPdownload:bus2", "MW, Load losses, downstream"
      n_LossQdownload_bus1 = "n:LossQdownload:bus1", "Mvar, Load losses, downstream"
      n_LossQdownload_bus2 = "n:LossQdownload:bus2", "Mvar, Load losses, downstream"
      n_LossPdownnoload_bus1 = "n:LossPdownnoload:bus1", "MW, No load losses, downstream"
      n_LossPdownnoload_bus2 = "n:LossPdownnoload:bus2", "MW, No load losses, downstream"
      n_LossQdownnoload_bus1 = "n:LossQdownnoload:bus1", "Mvar, No load losses, downstream"
      n_LossQdownnoload_bus2 = "n:LossQdownnoload:bus2", "Mvar, No load losses, downstream"
      n_du_feed_bus1 = "n:du_feed:bus1", "%, Voltage difference relative to feeder begin"
      n_du_feed_bus2 = "n:du_feed:bus2", "%, Voltage difference relative to feeder begin"
      s_Rin = "s:Rin", "Ohm, IN, Resistance (Input), continuous, double"
      s_Xin = "s:Xin", "Ohm, IN, Reactance (Input), continuous, double"
      c_Zind_1_r = "c:Zind_1:r", "Ohm, Impedance, Real Part"
      c_Zind_1_i = "c:Zind_1:i", "Ohm, Impedance, Imaginary Part"
      c_Zind_1_phi = "c:Zind_1:phi", "deg, Impedance, Angle"
      c_Zind_1_mag = "c:Zind_1:mag", "Ohm, Impedance, Magnitude"
      c_Zind_1_phirad = "c:Zind_1:phirad", "rad, Impedance, Angle"
      c_loading = "c:loading", "%, Loading"
      c_maxload = "c:maxload", "%, Max. Loading"
      c_du = "c:du", "p.u., Voltage Drop"
      c_dupc = "c:dupc", "%, Voltage Drop"
      c_dphiu = "c:dphiu", "deg, Voltage Drop Angle"
      c_du1 = "c:du1", "p.u., Positive Sequence Voltage Drop"
      c_du1pc = "c:du1pc", "%, Positive Sequence Voltage Drop"
      c_dphiu1 = "c:dphiu1", "deg, Positive Sequence Voltage Drop Angle"
      c_Ploss = "c:Ploss", "MW, Losses (total)"
      c_Qloss = "c:Qloss", "Mvar, Reactive-Losses (total)"
      c_Plossld = "c:Plossld", "MW, Losses (load)"
      c_Qlossld = "c:Qlossld", "Mvar, Reactive-Losses (load)"
      c_Plossnld = "c:Plossnld", "MW, Losses (no load)"
      c_Qlossnld = "c:Qlossnld", "Mvar, Reactive-Losses (no load)"
      e_uk = "e:uk", "%, Short-Circuit Voltage uk"
      e_Zd = "e:Zd", "Ohm, Impedance (absolute) Zd"
      e_lrea = "e:lrea", "mH, Inductance, L"
      e_xrea = "e:xrea", "Ohm, Reactance, X"
      e_Pcu = "e:Pcu", "kW, Copper Losses"
      e_ukr = "e:ukr", "%, Short-Circuit Voltage (Re(uk)) ukr"
      e_rrea = "e:rrea", "Ohm, Resistance, R"
      e_R0toR1 = "e:R0toR1", "R0/R1 ratio"
      e_X0toX1 = "e:X0toX1", "X0/X1 ratio"
      e_maxload = "e:maxload", "%, Max. Loading"
      e_lossAssign = "e:lossAssign", "Loss assignment"
      e_cpFeed = "e:cpFeed", "Feeder"
      e_ciDist = "e:ciDist", "Distance from infeed in number of buses"
      e_ciLater = "e:ciLater", "Lateral Index"
      e_ciDistRoot = "e:ciDistRoot", "Distance from first infeed in number of buses"
      e_ciDistAll = "e:ciDistAll", "Distance from infeed in number of buses including switches"
      e_ciDistAllRoot = "e:ciDistAllRoot", "Distance from first infeed in number of buses including switches"
      r_cpFeed = "r:cpFeed", "Feeder"

    class ElmStactrl(Enum):
      _init_ = 'value __doc__'

      c_rembar = "c:rembar", "Controlled Busbar"
      c_uini_r = "c:uini:r", "p.u., Voltage Setpoint, Real Part"
      c_uini_i = "c:uini:i", "p.u., Voltage Setpoint, Imaginary Part"
      c_uini_phi = "c:uini:phi", "deg, Voltage Setpoint, Angle"
      c_uini_mag = "c:uini:mag", "p.u., Voltage Setpoint, Magnitude"
      c_uini_phirad = "c:uini:phirad", "rad, Voltage Setpoint, Angle"
      c_qset = "c:qset", "Reactive Power Percentage"
      c_imode = "c:imode", "Control Mode"
      c_i_ctrl = "c:i_ctrl", "Load Flow Control"
      c_qini = "c:qini", "Mvar, Reactive Power Setpoint"
      c_qres = "c:qres", "Mvar, Reactive Power Reserve"
      c_Qdmeas = "c:Qdmeas", "Mvar, Q(measured) for Droop"
      c_Qdroop = "c:Qdroop", "Mvar/p.u., Droop"
      c_umin = "c:umin", "p.u., Lower Deadband Limit"
      c_umax = "c:umax", "p.u., Upper Deadband Limit"
      c_Qmin = "c:Qmin", "Mvar, Lower limit of reactive power"
      c_Qmax = "c:Qmax", "Mvar, Upper limit of reactive power"
      c_i_qlim = "c:i_qlim", "Reactive power limit reached"
      c_nCtrlIter = "c:nCtrlIter", "No. of CTRL-iterations"
      e_i_ctrl = "e:i_ctrl", "Control Mode"
      e_i_phase = "e:i_phase", "Phase"
      e_qu_char = "e:qu_char", "Q-Control"
      e_cosphi_char = "e:cosphi_char", "PF-Control"
      e_selBus = "e:selBus", "Controlled Node"
      e_uset_mode = "e:uset_mode", "Setpoint"
      e_rembar = "e:rembar", "StaBar*,ElmTerm*, Controlled Node"
      e_cpCtrlNode = "e:cpCtrlNode", "Target Node"
      e_usetp = "e:usetp", "p.u., Voltage Setpoint"
      e_selAutoUn = "e:selAutoUn", "kV, Busbar Search Criteria >="
      e_p_cub = "e:p_cub", "StaCubic,ElmBoundary, Control Q at"
      e_qsetp = "e:qsetp", "Mvar, Q Setpoint"
      e_tansetp = "e:tansetp", "tan(phi)"
      e_pfsetp = "e:pfsetp", "Power Factor"
      e_pf_recap = "e:pf_recap", "Power Factor"
      e_iQorient = "e:iQorient", "Orientation"
      e_i_droop = "e:i_droop", "Enable Droop"
      e_Srated = "e:Srated", "Mvar, Rated Reactive Power"
      e_ddroop = "e:ddroop", "%, Droop"
      e_Qdroop = "e:Qdroop", "Mvar/p.u., Droop"
      e_deltaV = "e:deltaV", "p.u., delta(V)"
      e_pQmeas = "e:pQmeas", "StaCubic,ElmBoundary, Q measured at"
      e_refbar = "e:refbar", "StaBar*,ElmTerm*, Reference Node"
      e_iopt_tdr = "e:iopt_tdr", "Different droop values"
      e_Qdroopoe = "e:Qdroopoe", "Mvar/p.u., Droop (overexcited)"
      e_Qdroopue = "e:Qdroopue", "Mvar/p.u., Droop (underexcited)"
      e_ddroopoe = "e:ddroopoe", "%, Droop (overexcited)"
      e_ddroopue = "e:ddroopue", "%, Droop (underexcited)"
      e_deltaVoe = "e:deltaVoe", "p.u., delta(V) (overexcited)"
      e_deltaVue = "e:deltaVue", "p.u., delta(V) (underexcited)"
      e_Qmin = "e:Qmin", "Mvar, Qmin"
      e_Qmax = "e:Qmax", "Mvar, Qmax"
      e_udeadbup = "e:udeadbup", "p.u., Upper Voltage Limit"
      e_udeadblow = "e:udeadblow", "p.u., Lower Voltage Limit"
      e_pQPcurve = "e:pQPcurve", "IntQpcurve, Q(P)-Curve"
      e_pf_over = "e:pf_over", "Min. Power Factor"
      e_p_over = "e:p_over", "MW, Active Power"
      e_pf_under = "e:pf_under", "Min. Power Factor"
      e_p_under = "e:p_under", "MW, Active Power"
      e_u_over = "e:u_over", "p.u., Voltage"
      e_u_under = "e:u_under", "p.u., Voltage"
      e_imode = "e:imode", "Reactive Power Distribution"
      e_consQdisp = "e:consQdisp", "Consider reactive power dispatch"
      e_iTrfCtrl = "e:iTrfCtrl", "Step-up Transformer Control"
      e_Tctrl = "e:Tctrl", "s, Controller Time Constant"
      e_psym = "e:psym", "ElmSym,ElmGenstat,ElmPvsys,ElmSvs,ElmAsm,ElmVsc*,ElmXnet, Machines"
      e_cvqq = "e:cvqq", "%, Reactive Power Percentage"
      e_cvgen = "e:cvgen", "p.u., Voltage Setpoint"
      e_cvgenmin = "e:cvgenmin", "p.u., Min. Voltage"
      e_cvgenmax = "e:cvgenmax", "p.u., Max. Voltage"
      r_rembar = "r:rembar", "StaBar*,ElmTerm*, Controlled Node"
      r_cpCtrlNode = "r:cpCtrlNode", "Target Node"
      r_p_cub = "r:p_cub", "StaCubic,ElmBoundary, Control Q at"
      r_pQmeas = "r:pQmeas", "StaCubic,ElmBoundary, Q measured at"
      r_refbar = "r:refbar", "StaBar*,ElmTerm*, Reference Node"
      r_pQPcurve = "r:pQPcurve", "IntQpcurve, Q(P)-Curve"
      r_psym = "r:psym", "ElmSym,ElmGenstat,ElmPvsys,ElmSvs,ElmAsm,ElmVsc*,ElmXnet, Machines"

    class ElmSym(Enum):
      _init_ = 'value __doc__'

      m_u1r_bus1 = "m:u1r:bus1", "p.u., Positive-Sequence-Voltage, Real Part"
      m_u1i_bus1 = "m:u1i:bus1", "p.u., Positive-Sequence-Voltage, Imaginary Part"
      m_u1_bus1 = "m:u1:bus1", "p.u., Positive-Sequence-Voltage, Magnitude"
      m_U1_bus1 = "m:U1:bus1", "kV, Line-Ground Positive-Sequence-Voltage, Magnitude"
      m_U1l_bus1 = "m:U1l:bus1", "kV, Line-Line Positive-Sequence-Voltage, Magnitude"
      m_phiu1_bus1 = "m:phiu1:bus1", "deg, Positive-Sequence-Voltage, Angle"
      m_inet_bus1 = "m:inet:bus1", "p.u., Current, Magnitude, referred to network"
      m_I_bus1 = "m:I:bus1", "kA, Current, Magnitude"
      m_phii_bus1 = "m:phii:bus1", "deg, Current, Angle"
      m_i1r_bus1 = "m:i1r:bus1", "p.u., Positive-Sequence Current, Real Part"
      m_i1i_bus1 = "m:i1i:bus1", "p.u., Positive-Sequence Current, Imaginary Part"
      m_i1_bus1 = "m:i1:bus1", "p.u., Positive-Sequence Current, Magnitude"
      m_I1_bus1 = "m:I1:bus1", "kA, Positive-Sequence Current, Magnitude"
      m_phii1_bus1 = "m:phii1:bus1", "deg, Positive-Sequence Current, Angle"
      m_P_bus1 = "m:P:bus1", "MW, Active Power"
      m_Q_bus1 = "m:Q:bus1", "Mvar, Reactive Power"
      m_S_bus1 = "m:S:bus1", "MVA, Apparent Power"
      m_cosphi_bus1 = "m:cosphi:bus1", "Power Factor"
      m_tanphi_bus1 = "m:tanphi:bus1", "tan(phi)"
      m_Psum_bus1 = "m:Psum:bus1", "MW, Total Active Power"
      m_Qsum_bus1 = "m:Qsum:bus1", "Mvar, Total Reactive Power"
      m_Ssum_bus1 = "m:Ssum:bus1", "MVA, Total Apparent Power"
      m_cosphisum_bus1 = "m:cosphisum:bus1", "Total Power Factor"
      m_tanphisum_bus1 = "m:tanphisum:bus1", "Total tan(phi)"
      m_Spu_bus1 = "m:Spu:bus1", "MVA/p.u., Apparent Power per p.u. Voltage"
      m_i1P_bus1 = "m:i1P:bus1", "p.u., Positive-Sequence Active Current"
      m_i1Q_bus1 = "m:i1Q:bus1", "p.u., Positive-Sequence Reactive Current"
      m_I1P_bus1 = "m:I1P:bus1", "kA, Positive-Sequence Active Current"
      m_I1Q_bus1 = "m:I1Q:bus1", "kA, Positive-Sequence Reactive Current"
      m_ur_bus1 = "m:ur:bus1", "p.u., Voltage, Real Part"
      m_ui_bus1 = "m:ui:bus1", "p.u., Voltage, Imaginary Part"
      m_u_bus1 = "m:u:bus1", "p.u., Voltage, Magnitude"
      m_ir_bus1 = "m:ir:bus1", "p.u., Current, Real Part"
      m_ii_bus1 = "m:ii:bus1", "p.u., Current, Imaginary Part"
      m_i_bus1 = "m:i:bus1", "p.u., Current, Magnitude"
      m_phiui_bus1 = "m:phiui:bus1", "deg, Angle between Voltage and Current"
      m_phiu1i1_bus1 = "m:phiu1i1:bus1", "deg, Angle between Voltage and Current in positive sequence system"
      m_Tfct_bus1 = "m:Tfct:bus1", "s, Fault Clearing Time"
      m_Brkload_bus1 = "m:Brkload:bus1", "%, Breaker Loading"
      n_ur_bus1 = "n:ur:bus1", "p.u., Voltage, Real Part"
      n_ui_bus1 = "n:ui:bus1", "p.u., Voltage, Imaginary Part"
      n_u_bus1 = "n:u:bus1", "p.u., Voltage, Magnitude"
      n_upc_bus1 = "n:upc:bus1", "%, Voltage, Magnitude"
      n_u1_bus1 = "n:u1:bus1", "p.u., Voltage, Magnitude"
      n_u1pc_bus1 = "n:u1pc:bus1", "%, Voltage, Magnitude"
      n_u1r_bus1 = "n:u1r:bus1", "p.u., Positive-Sequence Voltage, Real Part"
      n_u1i_bus1 = "n:u1i:bus1", "p.u., Positive-Sequence Voltage, Imaginary Part"
      n_U_bus1 = "n:U:bus1", "kV, Line-Ground Voltage, Magnitude"
      n_Ul_bus1 = "n:Ul:bus1", "kV, Line-Line Voltage, Magnitude"
      n_phiu_bus1 = "n:phiu:bus1", "deg, Voltage, Angle"
      n_phiurel_bus1 = "n:phiurel:bus1", "deg, Voltage, Relative Angle"
      n_du_bus1 = "n:du:bus1", "%, Voltage Deviation"
      n_Pgen_bus1 = "n:Pgen:bus1", "MW, Generation, Active Power"
      n_Qgen_bus1 = "n:Qgen:bus1", "Mvar, Generation, Reactive Power"
      n_Pmot_bus1 = "n:Pmot:bus1", "MW, Motor Load, Active Power"
      n_Qmot_bus1 = "n:Qmot:bus1", "Mvar, Motor Load, Reactive Power"
      n_Pload_bus1 = "n:Pload:bus1", "MW, General Load, Active Power"
      n_Qload_bus1 = "n:Qload:bus1", "Mvar, General Load, Reactive Power"
      n_Pcomp_bus1 = "n:Pcomp:bus1", "MW, Compensation (Losses)"
      n_Qcomp_bus1 = "n:Qcomp:bus1", "Mvar, Compensation"
      n_Pnet_bus1 = "n:Pnet:bus1", "MW, External Networks, Active Power"
      n_Qnet_bus1 = "n:Qnet:bus1", "Mvar, External Networks, Reactive Power"
      n_Pflow_bus1 = "n:Pflow:bus1", "MW, Power Flow, Active Power"
      n_Qflow_bus1 = "n:Qflow:bus1", "Mvar, Power Flow, Reactive Power"
      n_Pout_bus1 = "n:Pout:bus1", "MW, Outgoing Flow, Active Power"
      n_Qout_bus1 = "n:Qout:bus1", "Mvar, Outgoing Power, Reactive Power"
      n_Sout_bus1 = "n:Sout:bus1", "MVA, Outgoing Power, Apparent Power"
      n_cosphiout_bus1 = "n:cosphiout:bus1", "Outgoing Power, Power Factor"
      n_Pbalance_bus1 = "n:Pbalance:bus1", "MW, Active Power Balance (=0)"
      n_Qbalance_bus1 = "n:Qbalance:bus1", "Mvar, Reactive Power Balance (=0)"
      n_LossPdown_bus1 = "n:LossPdown:bus1", "MW, Losses, downstream"
      n_LossQdown_bus1 = "n:LossQdown:bus1", "Mvar, Losses, downstream (Reactive Power)"
      n_LossPdownload_bus1 = "n:LossPdownload:bus1", "MW, Load losses, downstream"
      n_LossQdownload_bus1 = "n:LossQdownload:bus1", "Mvar, Load losses, downstream"
      n_LossPdownnoload_bus1 = "n:LossPdownnoload:bus1", "MW, No load losses, downstream"
      n_LossQdownnoload_bus1 = "n:LossQdownnoload:bus1", "Mvar, No load losses, downstream"
      n_du_feed_bus1 = "n:du_feed:bus1", "%, Voltage difference relative to feeder begin"
      s_pset = "s:pset", "MW, IN, Active Power Setpoint, continuous, double"
      s_qset = "s:qset", "Mvar, IN, Reactive Power Setpoint continuous, double"
      s_dFin = "s:dFin", "Hz, IN, Frequency Deviation, continuous, double"
      s_dF = "s:dF", "Hz, OUT, Frequency Deviation Output, continuous, double"
      c_p = "c:p", "MW, Total Active Power"
      c_q = "c:q", "Mvar, Total Reactive Power"
      c_sgnn = "c:sgnn", "MVA, Total rated apparent power (ngnum*sgn)"
      c_cosn = "c:cosn", "Nominal-Power factor"
      c_Pnom = "c:Pnom", "MW, Total Nominal Active power"
      c_c2 = "c:c2", "Voltage Coefficient (uknom/ugn)"
      c_uini_r = "c:uini:r", "p.u., Voltage Setpoint for PV and Slack, Real Part"
      c_uini_i = "c:uini:i", "p.u., Voltage Setpoint for PV and Slack, Imaginary Part"
      c_uini_phi = "c:uini:phi", "deg, Voltage Setpoint for PV and Slack, Angle"
      c_uini_mag = "c:uini:mag", "p.u., Voltage Setpoint for PV and Slack, Magnitude"
      c_uini_phirad = "c:uini:phirad", "rad, Voltage Setpoint for PV and Slack, Angle"
      c_phiini = "c:phiini", "Initial Angle"
      c_loading = "c:loading", "%, Loading"
      c_iPowerOn = "c:iPowerOn", "Flag"
      c_xdq = "c:xdq", "p.u., Synchronous Reactance"
      c_Qmin = "c:Qmin", "Mvar, Lower limit of reactive power"
      c_Qmax = "c:Qmax", "Mvar, Upper limit of reactive power"
      c_Qsparelead = "c:Qsparelead", "Mvar, Q spare lead"
      c_Qsparelag = "c:Qsparelag", "Mvar, Q spare lag"
      c_i_qlim = "c:i_qlim", "Reactive power limit reached"
      c_i_vlim = "c:i_vlim", "Voltage setpoint limit reached"
      c_Pmin = "c:Pmin", "MW, Lower limit of active power"
      c_Pmax = "c:Pmax", "MW, Upper limit of active power"
      c_i_plim = "c:i_plim", "Active power limit reached"
      c_sm_r = "c:sm:r", "MW, Complex Power, Real Part"
      c_sm_i = "c:sm:i", "Mvar, Complex Power, Imaginary Part"
      c_sm_phi = "c:sm:phi", "deg, Complex Power, Angle"
      c_sm_mag = "c:sm:mag", "MVA, Complex Power, Magnitude"
      c_sm_phirad = "c:sm:phirad", "rad, Complex Power, Angle"
      c_u1_r = "c:u1:r", "p.u., Internal, Real Part"
      c_u1_i = "c:u1:i", "p.u., Internal, Imaginary Part"
      c_u1_phi = "c:u1:phi", "deg, Internal, Angle"
      c_u1_mag = "c:u1:mag", "p.u., Internal, Magnitude"
      c_u1_phirad = "c:u1:phirad", "rad, Internal, Angle"
      c_i_spin = "c:i_spin", "Flag"
      c_iRefElement = "c:iRefElement", "Reference Element"
      c_Snom = "c:Snom", "MVA, Nom. Apparent Power (1 mach.)"
      c_Nnom = "c:Nnom", "Number of Parallel Machines"
      c_Kp = "c:Kp", "p.u., Active Power Participation"
      c_Kq = "c:Kq", "p.u., Reactive Power Participation"
      c_Kpf = "c:Kpf", "MW/Hz, Frequency Bias"
      c_Pdisp = "c:Pdisp", "MW, Power (Eco-Dispatch)"
      c_cst_disp = "c:cst_disp", "USD/h, Operating Costs"
      c_zonewscale = "c:zonewscale", "Wind generation zone scaling factor"
      c_Pmism = "c:Pmism", "MW, Active Power Mismatch"
      c_nCtrlIter = "c:nCtrlIter", "No. of CTRL-iterations"
      c_Ploss = "c:Ploss", "MW, Losses (total)"
      c_Qloss = "c:Qloss", "Mvar, Reactive-Losses (total)"
      c_Plossld = "c:Plossld", "MW, Losses (load)"
      c_Qlossld = "c:Qlossld", "Mvar, Reactive-Losses (load)"
      c_Plossnld = "c:Plossnld", "MW, Losses (no load)"
      c_Qlossnld = "c:Qlossnld", "Mvar, Reactive-Losses (no load)"
      e_i_spin = "e:i_spin", "Spinning if circuit-breaker is open"
      e_ip_ctrl = "e:ip_ctrl", "Reference Machine"
      e_outServPzero = "e:outServPzero", "Out of service when active power is zero"
      e_av_mode = "e:av_mode", "Local Controller"
      e_c_psecc = "e:c_psecc", "ElmSecctrl, External Secondary Controller"
      e_c_pstac = "e:c_pstac", "ElmStactrl, External Station Controller"
      e_qdslCtrl = "e:qdslCtrl", "Quasi-Dynamic Model"
      e_mode_pgi = "e:mode_pgi", "Model"
      e_pPowerCrv = "e:pPowerCrv", "TypPowercurve, Wind Power Curve"
      e_mode_inp = "e:mode_inp", "Input Mode"
      e_pgini = "e:pgini", "MW, Active Power"
      e_qgini = "e:qgini", "Mvar, Reactive Power"
      e_sgini = "e:sgini", "MVA, Apparent Power"
      e_cosgini = "e:cosgini", "Power Factor"
      e_pf_recap = "e:pf_recap", "Power Factor"
      e_p_direc = "e:p_direc", "Power Direction"
      e_windspeed = "e:windspeed", "m/s, Wind speed"
      e_usetp = "e:usetp", "p.u., Voltage"
      e_phiini = "e:phiini", "deg, Angle"
      e_Kpf = "e:Kpf", "MW/Hz, Prim. Frequency Bias"
      e_pgini_a = "e:pgini_a", "MW, Active Power (act.)"
      e_qgini_a = "e:qgini_a", "Mvar, Reactive Power (act.)"
      e_sgini_a = "e:sgini_a", "MVA, Apparent Power (act.)"
      e_cosgini_a = "e:cosgini_a", "Power Factor (act.)"
      e_pf_recap_a = "e:pf_recap_a", "Power Factor Ind/Cap (act.)"
      e_windspeed_a = "e:windspeed_a", "m/s, Wind speed (act.)"
      e_usp_min = "e:usp_min", "p.u., Min. Voltage Setpoint"
      e_usp_max = "e:usp_max", "p.u., Max. Voltage Setpoint"
      e_ddroop = "e:ddroop", "%, Droop"
      e_pQPcurve = "e:pQPcurve", "IntQpcurve, Q(P)-Curve"
      e_pQlimType = "e:pQlimType", "IntQlim, Capability Curve"
      e_iqtype = "e:iqtype", "Use limits specified in type"
      e_q_min = "e:q_min", "p.u., Min."
      e_q_max = "e:q_max", "p.u., Max."
      e_cQ_min = "e:cQ_min", "Mvar, Minimum"
      e_cQ_max = "e:cQ_max", "Mvar, Maximum"
      e_cq_min = "e:cq_min", "p.u., Min."
      e_cq_max = "e:cq_max", "p.u., Max."
      e_scaleQmin = "e:scaleQmin", "%, Scaling Factor (min.)"
      e_scaleQmax = "e:scaleQmax", "%, Scaling Factor (max.)"
      e_Pmin_uc = "e:Pmin_uc", "MW, Min."
      e_Pmin_ucPU = "e:Pmin_ucPU", "p.u., Min."
      e_Pmax_uc = "e:Pmax_uc", "MW, Max."
      e_Pmax_ucPU = "e:Pmax_ucPU", "p.u., Max."
      e_Pnom = "e:Pnom", "MW, Pr(rated)"
      e_P_max = "e:P_max", "MW, Max."
      e_pmaxratf = "e:pmaxratf", "Rating Factor"
      e_i_cap = "e:i_cap", "User defined Capability Curve"
      e_cap_P = "e:cap_P", "MW, Act.Power"
      e_cap_Qmn = "e:cap_Qmn", "Mvar, Min."
      e_cap_Qmx = "e:cap_Qmx", "Mvar, Max."
      e_is4SpinReserve = "e:is4SpinReserve", "Consider for region spinning reserve"
      e_genShiftKey = "e:genShiftKey", "%, Generation shift key"
      e_c_pCtrlHV = "e:c_pCtrlHV", "Controlled HV-busbar"
      e_Uctrl = "e:Uctrl", "kV, HV-Voltage Setpoint"
      e_Qmin_a = "e:Qmin_a", "Mvar, Qmin(act.)"
      e_Qmax_a = "e:Qmax_a", "Mvar, Qmax(act.)"
      e_Pmin_a = "e:Pmin_a", "MW, Pmin(act.)"
      e_Pmax_a = "e:Pmax_a", "MW, Pmax(act.)"
      e_pBMU = "e:pBMU", "ElmBmu, Virtual Power Plant"
      e_dispatch = "e:dispatch", "Generator Dispatch"
      e_priority = "e:priority", "Merit Order"
      e_fixed = "e:fixed", "Must run"
      e_cpFeed = "e:cpFeed", "Feeder"
      e_ciDist = "e:ciDist", "Distance from infeed in number of buses"
      e_ciLater = "e:ciLater", "Lateral Index"
      e_ciDistRoot = "e:ciDistRoot", "Distance from first infeed in number of buses"
      e_ciDistAll = "e:ciDistAll", "Distance from infeed in number of buses including switches"
      e_ciDistAllRoot = "e:ciDistAllRoot", "Distance from first infeed in number of buses including switches"
      t_model_inp = "t:model_inp", "Model"
      t_rstr = "t:rstr", "p.u., rstr"
      t_iturbo = "t:iturbo", "Rotor type"
      t_x0sy = "t:x0sy", "p.u., Reactance x0"
      t_r0sy = "t:r0sy", "p.u., Resistance r0"
      t_x2sy = "t:x2sy", "p.u., Reactance x2"
      t_r2sy = "t:r2sy", "p.u., Resistance r2"
      t_q_min = "t:q_min", "p.u., Minimum value"
      t_q_max = "t:q_max", "p.u., Maximum value"
      t_Q_min = "t:Q_min", "Mvar, Minimum value"
      t_Q_max = "t:Q_max", "Mvar, Maximum value"
      t_xd = "t:xd", "p.u., xd"
      t_xq = "t:xq", "p.u., xq"
      t_psim = "t:psim", "p.u., Rotor flux"
      r_c_psecc = "r:c_psecc", "ElmSecctrl, External Secondary Controller"
      r_c_pstac = "r:c_pstac", "ElmStactrl, External Station Controller"
      r_qdslCtrl = "r:qdslCtrl", "Quasi-Dynamic Model"
      r_pPowerCrv = "r:pPowerCrv", "TypPowercurve, Wind Power Curve"
      r_pQPcurve = "r:pQPcurve", "IntQpcurve, Q(P)-Curve"
      r_pQlimType = "r:pQlimType", "IntQlim, Capability Curve"
      r_c_pCtrlHV = "r:c_pCtrlHV", "Controlled HV-busbar"
      r_pBMU = "r:pBMU", "ElmBmu, Virtual Power Plant"
      r_cpFeed = "r:cpFeed", "Feeder"

    class ElmTerm(Enum):
      _init_ = 'value __doc__'

      m_ur = "m:ur", "p.u., Voltage, Real Part"
      m_ui = "m:ui", "p.u., Voltage, Imaginary Part"
      m_u = "m:u", "p.u., Voltage, Magnitude"
      m_upc = "m:upc", "%, Voltage, Magnitude"
      m_u1 = "m:u1", "p.u., Voltage, Magnitude"
      m_u1pc = "m:u1pc", "%, Voltage, Magnitude"
      m_u1r = "m:u1r", "p.u., Positive-Sequence Voltage, Real Part"
      m_u1i = "m:u1i", "p.u., Positive-Sequence Voltage, Imaginary Part"
      m_U = "m:U", "kV, Line-Ground Voltage, Magnitude"
      m_Ul = "m:Ul", "kV, Line-Line Voltage, Magnitude"
      m_phiu = "m:phiu", "deg, Voltage, Angle"
      m_phiurel = "m:phiurel", "deg, Voltage, Relative Angle"
      m_du = "m:du", "%, Voltage Deviation"
      m_Pgen = "m:Pgen", "MW, Generation, Active Power"
      m_Qgen = "m:Qgen", "Mvar, Generation, Reactive Power"
      m_Pmot = "m:Pmot", "MW, Motor Load, Active Power"
      m_Qmot = "m:Qmot", "Mvar, Motor Load, Reactive Power"
      m_Pload = "m:Pload", "MW, General Load, Active Power"
      m_Qload = "m:Qload", "Mvar, General Load, Reactive Power"
      m_Pcomp = "m:Pcomp", "MW, Compensation (Losses)"
      m_Qcomp = "m:Qcomp", "Mvar, Compensation"
      m_Pnet = "m:Pnet", "MW, External Networks, Active Power"
      m_Qnet = "m:Qnet", "Mvar, External Networks, Reactive Power"
      m_Pflow = "m:Pflow", "MW, Power Flow, Active Power"
      m_Qflow = "m:Qflow", "Mvar, Power Flow, Reactive Power"
      m_Pout = "m:Pout", "MW, Outgoing Flow, Active Power"
      m_Qout = "m:Qout", "Mvar, Outgoing Power, Reactive Power"
      m_Sout = "m:Sout", "MVA, Outgoing Power, Apparent Power"
      m_cosphiout = "m:cosphiout", "Outgoing Power, Power Factor"
      m_Pbalance = "m:Pbalance", "MW, Active Power Balance (=0)"
      m_Qbalance = "m:Qbalance", "Mvar, Reactive Power Balance (=0)"
      m_LossPdown = "m:LossPdown", "MW, Losses, downstream"
      m_LossQdown = "m:LossQdown", "Mvar, Losses, downstream (Reactive Power)"
      m_LossPdownload = "m:LossPdownload", "MW, Load losses, downstream"
      m_LossQdownload = "m:LossQdownload", "Mvar, Load losses, downstream"
      m_LossPdownnoload = "m:LossPdownnoload", "MW, No load losses, downstream"
      m_LossQdownnoload = "m:LossQdownnoload", "Mvar, No load losses, downstream"
      m_du_feed = "m:du_feed", "%, Voltage difference relative to feeder begin"
      m_Imax = "m:Imax", "kA, Max. Current"
      m_loading = "m:loading", "%, Loading"
      e_vtarget = "e:vtarget", "p.u., Target Voltage"
      e_Vtarget = "e:Vtarget", "kV, Target Voltage"
      e_dvmax = "e:dvmax", "%, Delta V max"
      e_dvmin = "e:dvmin", "%, Delta V min"
      e_ivpriority = "e:ivpriority", "Priority"
      e_vmax = "e:vmax", "p.u., Upper Voltage Limit"
      e_vmin = "e:vmin", "p.u., Lower Voltage Limit"
      e_cpFeed = "e:cpFeed", "Feeder"
      e_ciDist = "e:ciDist", "Distance from infeed in number of buses"
      e_ciLater = "e:ciLater", "Lateral Index"
      e_ciDistRoot = "e:ciDistRoot", "Distance from first infeed in number of buses"
      e_ciDistAll = "e:ciDistAll", "Distance from infeed in number of buses including switches"
      e_ciDistAllRoot = "e:ciDistAllRoot", "Distance from first infeed in number of buses including switches"
      r_cpFeed = "r:cpFeed", "Feeder"

    class ElmTr2(Enum):
      _init_ = 'value __doc__'

      m_u1r_bushv = "m:u1r:bushv", "p.u., Positive-Sequence-Voltage, Real Part"
      m_u1r_buslv = "m:u1r:buslv", "p.u., Positive-Sequence-Voltage, Real Part"
      m_u1i_bushv = "m:u1i:bushv", "p.u., Positive-Sequence-Voltage, Imaginary Part"
      m_u1i_buslv = "m:u1i:buslv", "p.u., Positive-Sequence-Voltage, Imaginary Part"
      m_u1_bushv = "m:u1:bushv", "p.u., Positive-Sequence-Voltage, Magnitude"
      m_u1_buslv = "m:u1:buslv", "p.u., Positive-Sequence-Voltage, Magnitude"
      m_U1_bushv = "m:U1:bushv", "kV, Line-Ground Positive-Sequence-Voltage, Magnitude"
      m_U1_buslv = "m:U1:buslv", "kV, Line-Ground Positive-Sequence-Voltage, Magnitude"
      m_U1l_bushv = "m:U1l:bushv", "kV, Line-Line Positive-Sequence-Voltage, Magnitude"
      m_U1l_buslv = "m:U1l:buslv", "kV, Line-Line Positive-Sequence-Voltage, Magnitude"
      m_phiu1_bushv = "m:phiu1:bushv", "deg, Positive-Sequence-Voltage, Angle"
      m_phiu1_buslv = "m:phiu1:buslv", "deg, Positive-Sequence-Voltage, Angle"
      m_inet_bushv = "m:inet:bushv", "p.u., Current, Magnitude, referred to network"
      m_inet_buslv = "m:inet:buslv", "p.u., Current, Magnitude, referred to network"
      m_I_bushv = "m:I:bushv", "kA, Current, Magnitude"
      m_I_buslv = "m:I:buslv", "kA, Current, Magnitude"
      m_phii_bushv = "m:phii:bushv", "deg, Current, Angle"
      m_phii_buslv = "m:phii:buslv", "deg, Current, Angle"
      m_i1r_bushv = "m:i1r:bushv", "p.u., Positive-Sequence Current, Real Part"
      m_i1r_buslv = "m:i1r:buslv", "p.u., Positive-Sequence Current, Real Part"
      m_i1i_bushv = "m:i1i:bushv", "p.u., Positive-Sequence Current, Imaginary Part"
      m_i1i_buslv = "m:i1i:buslv", "p.u., Positive-Sequence Current, Imaginary Part"
      m_i1_bushv = "m:i1:bushv", "p.u., Positive-Sequence Current, Magnitude"
      m_i1_buslv = "m:i1:buslv", "p.u., Positive-Sequence Current, Magnitude"
      m_I1_bushv = "m:I1:bushv", "kA, Positive-Sequence Current, Magnitude"
      m_I1_buslv = "m:I1:buslv", "kA, Positive-Sequence Current, Magnitude"
      m_phii1_bushv = "m:phii1:bushv", "deg, Positive-Sequence Current, Angle"
      m_phii1_buslv = "m:phii1:buslv", "deg, Positive-Sequence Current, Angle"
      m_P_bushv = "m:P:bushv", "MW, Active Power"
      m_P_buslv = "m:P:buslv", "MW, Active Power"
      m_Q_bushv = "m:Q:bushv", "Mvar, Reactive Power"
      m_Q_buslv = "m:Q:buslv", "Mvar, Reactive Power"
      m_S_bushv = "m:S:bushv", "MVA, Apparent Power"
      m_S_buslv = "m:S:buslv", "MVA, Apparent Power"
      m_cosphi_bushv = "m:cosphi:bushv", "Power Factor"
      m_cosphi_buslv = "m:cosphi:buslv", "Power Factor"
      m_tanphi_bushv = "m:tanphi:bushv", "tan(phi)"
      m_tanphi_buslv = "m:tanphi:buslv", "tan(phi)"
      m_Psum_bushv = "m:Psum:bushv", "MW, Total Active Power"
      m_Psum_buslv = "m:Psum:buslv", "MW, Total Active Power"
      m_Qsum_bushv = "m:Qsum:bushv", "Mvar, Total Reactive Power"
      m_Qsum_buslv = "m:Qsum:buslv", "Mvar, Total Reactive Power"
      m_Ssum_bushv = "m:Ssum:bushv", "MVA, Total Apparent Power"
      m_Ssum_buslv = "m:Ssum:buslv", "MVA, Total Apparent Power"
      m_cosphisum_bushv = "m:cosphisum:bushv", "Total Power Factor"
      m_cosphisum_buslv = "m:cosphisum:buslv", "Total Power Factor"
      m_tanphisum_bushv = "m:tanphisum:bushv", "Total tan(phi)"
      m_tanphisum_buslv = "m:tanphisum:buslv", "Total tan(phi)"
      m_Spu_bushv = "m:Spu:bushv", "MVA/p.u., Apparent Power per p.u. Voltage"
      m_Spu_buslv = "m:Spu:buslv", "MVA/p.u., Apparent Power per p.u. Voltage"
      m_i1P_bushv = "m:i1P:bushv", "p.u., Positive-Sequence Active Current"
      m_i1P_buslv = "m:i1P:buslv", "p.u., Positive-Sequence Active Current"
      m_i1Q_bushv = "m:i1Q:bushv", "p.u., Positive-Sequence Reactive Current"
      m_i1Q_buslv = "m:i1Q:buslv", "p.u., Positive-Sequence Reactive Current"
      m_I1P_bushv = "m:I1P:bushv", "kA, Positive-Sequence Active Current"
      m_I1P_buslv = "m:I1P:buslv", "kA, Positive-Sequence Active Current"
      m_I1Q_bushv = "m:I1Q:bushv", "kA, Positive-Sequence Reactive Current"
      m_I1Q_buslv = "m:I1Q:buslv", "kA, Positive-Sequence Reactive Current"
      m_ur_bushv = "m:ur:bushv", "p.u., Voltage, Real Part"
      m_ur_buslv = "m:ur:buslv", "p.u., Voltage, Real Part"
      m_ui_bushv = "m:ui:bushv", "p.u., Voltage, Imaginary Part"
      m_ui_buslv = "m:ui:buslv", "p.u., Voltage, Imaginary Part"
      m_u_bushv = "m:u:bushv", "p.u., Voltage, Magnitude"
      m_u_buslv = "m:u:buslv", "p.u., Voltage, Magnitude"
      m_ir_bushv = "m:ir:bushv", "p.u., Current, Real Part"
      m_ir_buslv = "m:ir:buslv", "p.u., Current, Real Part"
      m_ii_bushv = "m:ii:bushv", "p.u., Current, Imaginary Part"
      m_ii_buslv = "m:ii:buslv", "p.u., Current, Imaginary Part"
      m_i_bushv = "m:i:bushv", "p.u., Current, Magnitude"
      m_i_buslv = "m:i:buslv", "p.u., Current, Magnitude"
      m_phiui_bushv = "m:phiui:bushv", "deg, Angle between Voltage and Current"
      m_phiui_buslv = "m:phiui:buslv", "deg, Angle between Voltage and Current"
      m_phiu1i1_bushv = "m:phiu1i1:bushv", "deg, Angle between Voltage and Current in positive sequence system"
      m_phiu1i1_buslv = "m:phiu1i1:buslv", "deg, Angle between Voltage and Current in positive sequence system"
      m_Tfct_bushv = "m:Tfct:bushv", "s, Fault Clearing Time"
      m_Tfct_buslv = "m:Tfct:buslv", "s, Fault Clearing Time"
      m_Brkload_bushv = "m:Brkload:bushv", "%, Breaker Loading"
      m_Brkload_buslv = "m:Brkload:buslv", "%, Breaker Loading"
      n_ur_bushv = "n:ur:bushv", "p.u., Voltage, Real Part"
      n_ur_buslv = "n:ur:buslv", "p.u., Voltage, Real Part"
      n_ui_bushv = "n:ui:bushv", "p.u., Voltage, Imaginary Part"
      n_ui_buslv = "n:ui:buslv", "p.u., Voltage, Imaginary Part"
      n_u_bushv = "n:u:bushv", "p.u., Voltage, Magnitude"
      n_u_buslv = "n:u:buslv", "p.u., Voltage, Magnitude"
      n_upc_bushv = "n:upc:bushv", "%, Voltage, Magnitude"
      n_upc_buslv = "n:upc:buslv", "%, Voltage, Magnitude"
      n_u1_bushv = "n:u1:bushv", "p.u., Voltage, Magnitude"
      n_u1_buslv = "n:u1:buslv", "p.u., Voltage, Magnitude"
      n_u1pc_bushv = "n:u1pc:bushv", "%, Voltage, Magnitude"
      n_u1pc_buslv = "n:u1pc:buslv", "%, Voltage, Magnitude"
      n_u1r_bushv = "n:u1r:bushv", "p.u., Positive-Sequence Voltage, Real Part"
      n_u1r_buslv = "n:u1r:buslv", "p.u., Positive-Sequence Voltage, Real Part"
      n_u1i_bushv = "n:u1i:bushv", "p.u., Positive-Sequence Voltage, Imaginary Part"
      n_u1i_buslv = "n:u1i:buslv", "p.u., Positive-Sequence Voltage, Imaginary Part"
      n_U_bushv = "n:U:bushv", "kV, Line-Ground Voltage, Magnitude"
      n_U_buslv = "n:U:buslv", "kV, Line-Ground Voltage, Magnitude"
      n_Ul_bushv = "n:Ul:bushv", "kV, Line-Line Voltage, Magnitude"
      n_Ul_buslv = "n:Ul:buslv", "kV, Line-Line Voltage, Magnitude"
      n_phiu_bushv = "n:phiu:bushv", "deg, Voltage, Angle"
      n_phiu_buslv = "n:phiu:buslv", "deg, Voltage, Angle"
      n_phiurel_bushv = "n:phiurel:bushv", "deg, Voltage, Relative Angle"
      n_phiurel_buslv = "n:phiurel:buslv", "deg, Voltage, Relative Angle"
      n_du_bushv = "n:du:bushv", "%, Voltage Deviation"
      n_du_buslv = "n:du:buslv", "%, Voltage Deviation"
      n_Pgen_bushv = "n:Pgen:bushv", "MW, Generation, Active Power"
      n_Pgen_buslv = "n:Pgen:buslv", "MW, Generation, Active Power"
      n_Qgen_bushv = "n:Qgen:bushv", "Mvar, Generation, Reactive Power"
      n_Qgen_buslv = "n:Qgen:buslv", "Mvar, Generation, Reactive Power"
      n_Pmot_bushv = "n:Pmot:bushv", "MW, Motor Load, Active Power"
      n_Pmot_buslv = "n:Pmot:buslv", "MW, Motor Load, Active Power"
      n_Qmot_bushv = "n:Qmot:bushv", "Mvar, Motor Load, Reactive Power"
      n_Qmot_buslv = "n:Qmot:buslv", "Mvar, Motor Load, Reactive Power"
      n_Pload_bushv = "n:Pload:bushv", "MW, General Load, Active Power"
      n_Pload_buslv = "n:Pload:buslv", "MW, General Load, Active Power"
      n_Qload_bushv = "n:Qload:bushv", "Mvar, General Load, Reactive Power"
      n_Qload_buslv = "n:Qload:buslv", "Mvar, General Load, Reactive Power"
      n_Pcomp_bushv = "n:Pcomp:bushv", "MW, Compensation (Losses)"
      n_Pcomp_buslv = "n:Pcomp:buslv", "MW, Compensation (Losses)"
      n_Qcomp_bushv = "n:Qcomp:bushv", "Mvar, Compensation"
      n_Qcomp_buslv = "n:Qcomp:buslv", "Mvar, Compensation"
      n_Pnet_bushv = "n:Pnet:bushv", "MW, External Networks, Active Power"
      n_Pnet_buslv = "n:Pnet:buslv", "MW, External Networks, Active Power"
      n_Qnet_bushv = "n:Qnet:bushv", "Mvar, External Networks, Reactive Power"
      n_Qnet_buslv = "n:Qnet:buslv", "Mvar, External Networks, Reactive Power"
      n_Pflow_bushv = "n:Pflow:bushv", "MW, Power Flow, Active Power"
      n_Pflow_buslv = "n:Pflow:buslv", "MW, Power Flow, Active Power"
      n_Qflow_bushv = "n:Qflow:bushv", "Mvar, Power Flow, Reactive Power"
      n_Qflow_buslv = "n:Qflow:buslv", "Mvar, Power Flow, Reactive Power"
      n_Pout_bushv = "n:Pout:bushv", "MW, Outgoing Flow, Active Power"
      n_Pout_buslv = "n:Pout:buslv", "MW, Outgoing Flow, Active Power"
      n_Qout_bushv = "n:Qout:bushv", "Mvar, Outgoing Power, Reactive Power"
      n_Qout_buslv = "n:Qout:buslv", "Mvar, Outgoing Power, Reactive Power"
      n_Sout_bushv = "n:Sout:bushv", "MVA, Outgoing Power, Apparent Power"
      n_Sout_buslv = "n:Sout:buslv", "MVA, Outgoing Power, Apparent Power"
      n_cosphiout_bushv = "n:cosphiout:bushv", "Outgoing Power, Power Factor"
      n_cosphiout_buslv = "n:cosphiout:buslv", "Outgoing Power, Power Factor"
      n_Pbalance_bushv = "n:Pbalance:bushv", "MW, Active Power Balance (=0)"
      n_Pbalance_buslv = "n:Pbalance:buslv", "MW, Active Power Balance (=0)"
      n_Qbalance_bushv = "n:Qbalance:bushv", "Mvar, Reactive Power Balance (=0)"
      n_Qbalance_buslv = "n:Qbalance:buslv", "Mvar, Reactive Power Balance (=0)"
      n_LossPdown_bushv = "n:LossPdown:bushv", "MW, Losses, downstream"
      n_LossPdown_buslv = "n:LossPdown:buslv", "MW, Losses, downstream"
      n_LossQdown_bushv = "n:LossQdown:bushv", "Mvar, Losses, downstream (Reactive Power)"
      n_LossQdown_buslv = "n:LossQdown:buslv", "Mvar, Losses, downstream (Reactive Power)"
      n_LossPdownload_bushv = "n:LossPdownload:bushv", "MW, Load losses, downstream"
      n_LossPdownload_buslv = "n:LossPdownload:buslv", "MW, Load losses, downstream"
      n_LossQdownload_bushv = "n:LossQdownload:bushv", "Mvar, Load losses, downstream"
      n_LossQdownload_buslv = "n:LossQdownload:buslv", "Mvar, Load losses, downstream"
      n_LossPdownnoload_bushv = "n:LossPdownnoload:bushv", "MW, No load losses, downstream"
      n_LossPdownnoload_buslv = "n:LossPdownnoload:buslv", "MW, No load losses, downstream"
      n_LossQdownnoload_bushv = "n:LossQdownnoload:bushv", "Mvar, No load losses, downstream"
      n_LossQdownnoload_buslv = "n:LossQdownnoload:buslv", "Mvar, No load losses, downstream"
      n_du_feed_bushv = "n:du_feed:bushv", "%, Voltage difference relative to feeder begin"
      n_du_feed_buslv = "n:du_feed:buslv", "%, Voltage difference relative to feeder begin"
      s_nt_in = "s:nt_in", "IN, Tap 1, Input, continuous, double"
      s_nt2_in = "s:nt2_in", "IN, Tap 2, Input, continuous, double"
      c_loading_h = "c:loading_h", "%, Loading (HV)"
      c_loading_l = "c:loading_l", "%, Loading (LV)"
      c_loading = "c:loading", "%, Loading"
      c_maxload = "c:maxload", "%, Max. Loading"
      c_maxloadpu = "c:maxloadpu", "p.u., Max. Loading"
      c_nntap = "c:nntap", "Tap 1, Current Position"
      c_ntpmn = "c:ntpmn", "Tap 1, Min. Position"
      c_ntpmx = "c:ntpmx", "Tap 1, Max. Position"
      c_dut = "c:dut", "Tap 1: Voltage/Tap"
      c_phitrrd = "c:phitrrd", "rad, Tap 1:, Phase/Tap"
      c_nt2agrd = "c:nt2agrd", "rad, Phase Angle"
      c_ratfac = "c:ratfac", "Rating Factor"
      c_snom = "c:snom", "MVA, Rated Power"
      c_zshv_r = "c:zshv:r", "p.u., Leakage Impedance (HV-Side), Real Part"
      c_zshv_i = "c:zshv:i", "p.u., Leakage Impedance (HV-Side), Imaginary Part"
      c_zshv_phi = "c:zshv:phi", "deg, Leakage Impedance (HV-Side), Angle"
      c_zshv_mag = "c:zshv:mag", "p.u., Leakage Impedance (HV-Side), Magnitude"
      c_zshv_phirad = "c:zshv:phirad", "rad, Leakage Impedance (HV-Side), Angle"
      c_zslv_r = "c:zslv:r", "p.u., Leakage Impedance (LV-Side), Real Part"
      c_zslv_i = "c:zslv:i", "p.u., Leakage Impedance (LV-Side), Imaginary Part"
      c_zslv_phi = "c:zslv:phi", "deg, Leakage Impedance (LV-Side), Angle"
      c_zslv_mag = "c:zslv:mag", "p.u., Leakage Impedance (LV-Side), Magnitude"
      c_zslv_phirad = "c:zslv:phirad", "rad, Leakage Impedance (LV-Side), Angle"
      c_ym_r = "c:ym:r", "p.u., Magnetising Admittance, Real Part"
      c_ym_i = "c:ym:i", "p.u., Magnetising Admittance, Imaginary Part"
      c_ym_phi = "c:ym:phi", "deg, Magnetising Admittance, Angle"
      c_ym_mag = "c:ym:mag", "p.u., Magnetising Admittance, Magnitude"
      c_ym_phirad = "c:ym:phirad", "rad, Magnetising Admittance, Angle"
      c_thv_r = "c:thv:r", "Complex Winding Ratio HV, Real Part"
      c_thv_i = "c:thv:i", "Complex Winding Ratio HV, Imaginary Part"
      c_thv_phi = "c:thv:phi", "Complex Winding Ratio HV, Angle"
      c_thv_mag = "c:thv:mag", "Complex Winding Ratio HV, Magnitude"
      c_thv_phirad = "c:thv:phirad", "Complex Winding Ratio HV, Angle"
      c_tlv_r = "c:tlv:r", "Complex Winding Ratio LV, Real Part"
      c_tlv_i = "c:tlv:i", "Complex Winding Ratio LV, Imaginary Part"
      c_tlv_phi = "c:tlv:phi", "Complex Winding Ratio LV, Angle"
      c_tlv_mag = "c:tlv:mag", "Complex Winding Ratio LV, Magnitude"
      c_tlv_phirad = "c:tlv:phirad", "Complex Winding Ratio LV, Angle"
      c_tap_side = "c:tap_side", "0"
      c_tap_side2 = "c:tap_side2", "0"
      c_nntap2 = "c:nntap2", "Tap 2, Current Position"
      c_ntpmn2 = "c:ntpmn2", "Tap 2, Min. Position"
      c_ntpmx2 = "c:ntpmx2", "Tap 2, Max. Position"
      c_dut2 = "c:dut2", "Tap 2: Voltage/Tap"
      c_phitrrd2 = "c:phitrrd2", "rad, Tap 2:, Phase/Tap"
      c_Inom_HV = "c:Inom_HV", "kA, Nominal Current (HV-Side)"
      c_Inom_LV = "c:Inom_LV", "kA, Nominal Current (LV-Side)"
      c_du = "c:du", "p.u., Voltage Drop"
      c_dupc = "c:dupc", "%, Voltage Drop"
      c_dphiu = "c:dphiu", "deg, Voltage Drop Angle"
      c_du1 = "c:du1", "p.u., Positive Sequence Voltage Drop"
      c_du1pc = "c:du1pc", "%, Positive Sequence Voltage Drop"
      c_dphiu1 = "c:dphiu1", "deg, Positive Sequence Voltage Drop Angle"
      c_Pmeas = "c:Pmeas", "MW, Measured active power"
      c_du_ldc = "c:du_ldc", "p.u., LDC Input"
      c_ctrl_set = "c:ctrl_set", "Setpoint"
      c_i_uplim = "c:i_uplim", "Tap 1 at Maximum"
      c_i_lowlim = "c:i_lowlim", "Tap 1 at Minimum"
      c_i_uplim2 = "c:i_uplim2", "Tap 2 at Maximum"
      c_i_lowlim2 = "c:i_lowlim2", "Tap 2 at Minimum"
      c_nCtrlIter = "c:nCtrlIter", "No. of CTRL-iterations"
      c_Tctrl = "c:Tctrl", "s, Controller Time Constant"
      c_i_fixedtap = "c:i_fixedtap", "Flag"
      c_w_0 = "c:w_0", "Nominal winding ratio"
      c_Gmload = "c:Gmload", "kW, Iron Losses"
      c_Xmload = "c:Xmload", "kvar, Magnetising Losses"
      c_Ploss = "c:Ploss", "MW, Losses (total)"
      c_Qloss = "c:Qloss", "Mvar, Reactive-Losses (total)"
      c_Plossld = "c:Plossld", "MW, Losses (load)"
      c_Qlossld = "c:Qlossld", "Mvar, Reactive-Losses (load)"
      c_Plossnld = "c:Plossnld", "MW, Losses (no load)"
      c_Qlossnld = "c:Qlossnld", "Mvar, Reactive-Losses (no load)"
      c_isat = "c:isat", "Saturated Model"
      e_dutap = "e:dutap", "%, Additional Voltage per Tap"
      e_dphitap = "e:dphitap", "deg, Additional Angle per Tap"
      e_phitr = "e:phitr", "deg, Phase of du"
      e_nntap = "e:nntap", "Tap Position"
      e_iTaps = "e:iTaps", "According to Measurement Report"
      e_dutap2 = "e:dutap2", "%, Additional Voltage per Tap"
      e_dphitap2 = "e:dphitap2", "deg, Additional Angle per Tap"
      e_phitr2 = "e:phitr2", "deg, Phase of du"
      e_nntap2 = "e:nntap2", "Tap Position"
      e_optaplimit = "e:optaplimit", "Operational limits for tap changer"
      e_optapmin = "e:optapmin", "Minimum Position"
      e_optapmax = "e:optapmax", "Maximum Position"
      e_optap2min = "e:optap2min", "Minimum Position"
      e_optap2max = "e:optap2max", "Maximum Position"
      e_maxload = "e:maxload", "%, Max. loading"
      e_lossAssign = "e:lossAssign", "Loss assignment"
      e_c_ptapc = "e:c_ptapc", "ElmTapctrl, External Tap Controller"
      e_c_pstac = "e:c_pstac", "External Station Controller"
      e_c_plcc = "e:c_plcc", "External LCC Controller"
      e_qdslCtrl = "e:qdslCtrl", "Quasi-Dynamic Model"
      e_ntrcn = "e:ntrcn", "Automatic Tap Changing"
      e_i_cont = "e:i_cont", "Tap Changer"
      e_t2ldc = "e:t2ldc", "Controlled Node is at"
      e_ilcph = "e:ilcph", "Phase"
      e_imldc = "e:imldc", "Control Mode"
      e_uset_mode = "e:uset_mode", "Setpoint"
      e_tapctrl = "e:tapctrl", "ElmTr2*, Tap Controller"
      e_iPpart = "e:iPpart", "Active power participation"
      e_i_rem = "e:i_rem", "Remote Control"
      e_p_rem = "e:p_rem", "ElmTerm,StaBar, Controlled Node"
      e_p_cub = "e:p_cub", "StaCubic,ElmBoundary, Controlled Branch (Cubicle)"
      e_cpCtrlNode = "e:cpCtrlNode", "Target Node"
      e_Kpart = "e:Kpart", "Participation factor"
      e_Ptolerance = "e:Ptolerance", "MW, Tolerance (+/-)"
      e_p_pmeas = "e:p_pmeas", "StaCubic,ElmBoundary, P measured at"
      e_usetp = "e:usetp", "p.u., Voltage Setpoint"
      e_usp_low = "e:usp_low", "p.u., Lower Voltage Bound"
      e_usp_up = "e:usp_up", "p.u., Upper Voltage Bound"
      e_psetp = "e:psetp", "MW, Active Power Setpoint"
      e_psp_low = "e:psp_low", "MW, Lower Active Power Bound"
      e_psp_up = "e:psp_up", "MW, Upper Active Power Bound"
      e_qsetp = "e:qsetp", "Mvar, Reactive Power Setpoint"
      e_qsp_low = "e:qsp_low", "Mvar, Lower Reactive Power Bound"
      e_qsp_up = "e:qsp_up", "Mvar, Upper Reactive Power Bound"
      e_Tctrl = "e:Tctrl", "s, Controller Time Constant"
      e_Kqctrl = "e:Kqctrl", "tap/Mvar, Controller Sensitivity dtap/dQ"
      e_Kpctrl = "e:Kpctrl", "tap/MW, Controller Sensitivity dtap/dP"
      e_ildc = "e:ildc", "Compensation"
      e_pldc = "e:pldc", "StaLdc, External LDC"
      e_ldcct = "e:ldcct", "A, Current Transformer Rating"
      e_ldcpt = "e:ldcpt", "Voltage Transformer Ratio"
      e_ldcrs = "e:ldcrs", "V, Rset"
      e_ldcxs = "e:ldcxs", "V, Xset"
      e_icomp = "e:icomp", "Compounding"
      e_Vtolerance = "e:Vtolerance", "%, Tolerance (+/-)"
      e_pVccurve = "e:pVccurve", "IntVctrlcurve, V-Control-Curve"
      e_ntrcn2 = "e:ntrcn2", "Automatic Tap Changing"
      e_i_cont2 = "e:i_cont2", "Tap Changer"
      e_t2ldc2 = "e:t2ldc2", "Controlled Node is at"
      e_ilcph2 = "e:ilcph2", "Phase"
      e_imldc2 = "e:imldc2", "Control Mode"
      e_uset_mode2 = "e:uset_mode2", "Setpoint"
      e_iPpart2 = "e:iPpart2", "Active power participation"
      e_i_rem2 = "e:i_rem2", "Remote Control"
      e_p_rem2 = "e:p_rem2", "ElmTerm,StaBar, Controlled Node"
      e_p_cub2 = "e:p_cub2", "StaCubic,ElmBoundary, Controlled Branch (Cubicle)"
      e_cpCtrlNode2 = "e:cpCtrlNode2", "Target Node"
      e_Kpart2 = "e:Kpart2", "Participation factor"
      e_Ptolerance2 = "e:Ptolerance2", "MW, Tolerance (+/-)"
      e_p_pmeas2 = "e:p_pmeas2", "StaCubic,ElmBoundary, P measured at"
      e_usetp2 = "e:usetp2", "p.u., Voltage Setpoint"
      e_usp_low2 = "e:usp_low2", "p.u., Lower Voltage Bound"
      e_usp_up2 = "e:usp_up2", "p.u., Upper Voltage Bound"
      e_psetp2 = "e:psetp2", "MW, Active Power Setpoint"
      e_psp_low2 = "e:psp_low2", "MW, Lower Active Power Bound"
      e_psp_up2 = "e:psp_up2", "MW, Upper Active Power Bound"
      e_qsetp2 = "e:qsetp2", "Mvar, Reactive Power Setpoint"
      e_qsp_low2 = "e:qsp_low2", "Mvar, Lower Reactive Power Bound"
      e_qsp_up2 = "e:qsp_up2", "Mvar, Upper Reactive Power Bound"
      e_Tctrl2 = "e:Tctrl2", "s, Controller Time Constant"
      e_Kqctrl2 = "e:Kqctrl2", "tap/Mvar, Controller Sensitivity dtap/dQ"
      e_Kpctrl2 = "e:Kpctrl2", "tap/MW, Controller Sensitivity dtap/dP"
      e_ildc2 = "e:ildc2", "Compensation"
      e_pldc2 = "e:pldc2", "StaLdc, External LDC"
      e_ldcct2 = "e:ldcct2", "A, Current Transformer Rating"
      e_ldcpt2 = "e:ldcpt2", "Voltage Transformer Ratio"
      e_ldcrs2 = "e:ldcrs2", "V, Rset"
      e_ldcxs2 = "e:ldcxs2", "V, Xset"
      e_icomp2 = "e:icomp2", "Compounding"
      e_Vtolerance2 = "e:Vtolerance2", "%, Tolerance (+/-)"
      e_pVccurve2 = "e:pVccurve2", "IntVctrlcurve, V-Control-Curve"
      e_iMeasLoc = "e:iMeasLoc", "Measured at"
      e_mTaps = "e:mTaps", "Measurement Report"
      e_iopt_hf = "e:iopt_hf", "Consider Capacitances"
      e_Cg_h = "e:Cg_h", "uF, Capacitance HV-Ground"
      e_Cg_l = "e:Cg_l", "uF, Capacitance LV-Ground"
      e_Cc1_hl = "e:Cc1_hl", "uF, Capacitance HV-LV, 1-Sequence"
      e_Cc0_hl = "e:Cc0_hl", "uF, Capacitance HV-LV, 0-Sequence"
      e_cpFeed = "e:cpFeed", "Feeder"
      e_ciDist = "e:ciDist", "Distance from infeed in number of buses"
      e_ciLater = "e:ciLater", "Lateral Index"
      e_ciDistRoot = "e:ciDistRoot", "Distance from first infeed in number of buses"
      e_ciDistAll = "e:ciDistAll", "Distance from infeed in number of buses including switches"
      e_ciDistAllRoot = "e:ciDistAllRoot", "Distance from first infeed in number of buses including switches"
      t_curmg = "t:curmg", "%, No Load Current"
      t_pfe = "t:pfe", "kW, No Load Losses"
      t_zx0hl_n = "t:zx0hl_n", "Mag. Impedance/uk0"
      t_rtox0_n = "t:rtox0_n", "Mag. R/X"
      t_itrdl = "t:itrdl", "x,Pos.Seq. HV-Side"
      t_itrdl_lv = "t:itrdl_lv", "x,Pos.Seq. LV-Side"
      t_itrdr = "t:itrdr", "r,Pos.Seq. HV-Side"
      t_itrdr_lv = "t:itrdr_lv", "r,Pos.Seq. LV-Side"
      t_zx0hl_h = "t:zx0hl_h", "z, Zero Sequ. HV-Side"
      t_zx0hl_l = "t:zx0hl_l", "z, Zero Sequ. LV-Side"
      t_itapch = "t:itapch", "Tap Changer 1"
      t_tapchtype = "t:tapchtype", "Type"
      t_tap_side = "t:tap_side", "at Side"
      t_dutap = "t:dutap", "%, Additional Voltage per Tap"
      t_dphitap = "t:dphitap", "deg, Additional Angle per Tap"
      t_phitr = "t:phitr", "deg, Phase of du"
      t_nntap0 = "t:nntap0", "Neutral Position"
      t_ntpmn = "t:ntpmn", "Minimum Position"
      t_ntpmx = "t:ntpmx", "Maximum Position"
      t_itapch2 = "t:itapch2", "Tap Changer 2"
      t_tapchtype2 = "t:tapchtype2", "Type"
      t_tap_side2 = "t:tap_side2", "at Side"
      t_dutap2 = "t:dutap2", "%, Additional Voltage per Tap"
      t_dphitap2 = "t:dphitap2", "deg, Additional Angle per Tap"
      t_phitr2 = "t:phitr2", "deg, Phase of du"
      t_nntap02 = "t:nntap02", "Neutral Position"
      t_ntpmn2 = "t:ntpmn2", "Minimum Position"
      t_ntpmx2 = "t:ntpmx2", "Maximum Position"
      t_itapzdep = "t:itapzdep", "Tap dependent impedance"
      t_uktr = "t:uktr", "%, Short-Circuit Voltage uk"
      t_uktmn = "t:uktmn", "%, uk (min. tap)"
      t_uktmx = "t:uktmx", "%, uk (max. tap)"
      t_x1pu = "t:x1pu", "p.u., Reactance x1"
      t_x1putmn = "t:x1putmn", "p.u., x1 (min. tap)"
      t_x1putmx = "t:x1putmx", "p.u., x1 (max. tap)"
      t_pcutr = "t:pcutr", "kW, Copper Losses"
      t_uktrr = "t:uktrr", "%, SHC-Voltage (Re(uk)) ukr"
      t_xtor = "t:xtor", "Ratio X/R"
      t_r1pu = "t:r1pu", "p.u., Resistance r1"
      t_pcutmn = "t:pcutmn", "kW, Pcu (min. tap)"
      t_ukrtmn = "t:ukrtmn", "%, Re(uk) (min. tap)"
      t_xtortmn = "t:xtortmn", "X/R (min. tap)"
      t_r1putmn = "t:r1putmn", "p.u., r1 (min. tap)"
      t_pcutmx = "t:pcutmx", "kW, Pcu (max. tap)"
      t_ukrtmx = "t:ukrtmx", "%, Re(uk) (max. tap)"
      t_xtortmx = "t:xtortmx", "X/R (max. tap)"
      t_r1putmx = "t:r1putmx", "p.u., r1 (max. tap)"
      t_uk0tr = "t:uk0tr", "%, Short-Circuit Voltage uk0"
      t_uk0tmn = "t:uk0tmn", "%, uk0 (min. tap)"
      t_uk0tmx = "t:uk0tmx", "%, uk0 (max. tap)"
      t_x0pu = "t:x0pu", "p.u., Reactance x0"
      t_x0putmn = "t:x0putmn", "p.u., x0 (min. tap)"
      t_x0putmx = "t:x0putmx", "p.u., x0 (max. tap)"
      t_ur0tr = "t:ur0tr", "%, SHC-Voltage (Re(uk0)) uk0r"
      t_x0tor0 = "t:x0tor0", "Ratio X0/R0"
      t_r0pu = "t:r0pu", "p.u., Resistance r0"
      t_uk0rtmn = "t:uk0rtmn", "%, Re(uk0) (min. tap)"
      t_x0tor0tmn = "t:x0tor0tmn", "X0/R0 (min. tap)"
      t_r0putmn = "t:r0putmn", "p.u., r0 (min. tap)"
      t_uk0rtmx = "t:uk0rtmx", "%, Re(uk0) (max. tap)"
      t_x0tor0tmx = "t:x0tor0tmx", "X0/R0 (max. tap)"
      t_r0putmx = "t:r0putmx", "p.u., r0 (max. tap)"
      t_iTapLoading = "t:iTapLoading", "Configuration"
      t_itrldf = "t:itrldf", "Type"
      t_satcue = "t:satcue", "%, Current (RMS)"
      t_satvol = "t:satvol", "p.u., Voltage (RMS)"
      t_iIntPola = "t:iIntPola", "Interpolation"
      t_smoothf = "t:smoothf", "%, Smoothing Factor"
      t_itratioadpt = "t:itratioadpt", "Transformer Ratio Adaptation"
      r_c_ptapc = "r:c_ptapc", "ElmTapctrl, External Tap Controller"
      r_c_pstac = "r:c_pstac", "External Station Controller"
      r_c_plcc = "r:c_plcc", "External LCC Controller"
      r_qdslCtrl = "r:qdslCtrl", "Quasi-Dynamic Model"
      r_tapctrl = "r:tapctrl", "ElmTr2*, Tap Controller"
      r_p_rem = "r:p_rem", "ElmTerm,StaBar, Controlled Node"
      r_p_cub = "r:p_cub", "StaCubic,ElmBoundary, Controlled Branch (Cubicle)"
      r_cpCtrlNode = "r:cpCtrlNode", "Target Node"
      r_p_pmeas = "r:p_pmeas", "StaCubic,ElmBoundary, P measured at"
      r_pldc = "r:pldc", "StaLdc, External LDC"
      r_pVccurve = "r:pVccurve", "IntVctrlcurve, V-Control-Curve"
      r_p_rem2 = "r:p_rem2", "ElmTerm,StaBar, Controlled Node"
      r_p_cub2 = "r:p_cub2", "StaCubic,ElmBoundary, Controlled Branch (Cubicle)"
      r_cpCtrlNode2 = "r:cpCtrlNode2", "Target Node"
      r_p_pmeas2 = "r:p_pmeas2", "StaCubic,ElmBoundary, P measured at"
      r_pldc2 = "r:pldc2", "StaLdc, External LDC"
      r_pVccurve2 = "r:pVccurve2", "IntVctrlcurve, V-Control-Curve"
      r_cpFeed = "r:cpFeed", "Feeder"

    class ElmTr3(Enum):
      _init_ = 'value __doc__'

      m_u1r_bushv = "m:u1r:bushv", "p.u., Positive-Sequence-Voltage, Real Part"
      m_u1r_busmv = "m:u1r:busmv", "p.u., Positive-Sequence-Voltage, Real Part"
      m_u1r_buslv = "m:u1r:buslv", "p.u., Positive-Sequence-Voltage, Real Part"
      m_u1i_bushv = "m:u1i:bushv", "p.u., Positive-Sequence-Voltage, Imaginary Part"
      m_u1i_busmv = "m:u1i:busmv", "p.u., Positive-Sequence-Voltage, Imaginary Part"
      m_u1i_buslv = "m:u1i:buslv", "p.u., Positive-Sequence-Voltage, Imaginary Part"
      m_u1_bushv = "m:u1:bushv", "p.u., Positive-Sequence-Voltage, Magnitude"
      m_u1_busmv = "m:u1:busmv", "p.u., Positive-Sequence-Voltage, Magnitude"
      m_u1_buslv = "m:u1:buslv", "p.u., Positive-Sequence-Voltage, Magnitude"
      m_U1_bushv = "m:U1:bushv", "kV, Line-Ground Positive-Sequence-Voltage, Magnitude"
      m_U1_busmv = "m:U1:busmv", "kV, Line-Ground Positive-Sequence-Voltage, Magnitude"
      m_U1_buslv = "m:U1:buslv", "kV, Line-Ground Positive-Sequence-Voltage, Magnitude"
      m_U1l_bushv = "m:U1l:bushv", "kV, Line-Line Positive-Sequence-Voltage, Magnitude"
      m_U1l_busmv = "m:U1l:busmv", "kV, Line-Line Positive-Sequence-Voltage, Magnitude"
      m_U1l_buslv = "m:U1l:buslv", "kV, Line-Line Positive-Sequence-Voltage, Magnitude"
      m_phiu1_bushv = "m:phiu1:bushv", "deg, Positive-Sequence-Voltage, Angle"
      m_phiu1_busmv = "m:phiu1:busmv", "deg, Positive-Sequence-Voltage, Angle"
      m_phiu1_buslv = "m:phiu1:buslv", "deg, Positive-Sequence-Voltage, Angle"
      m_inet_bushv = "m:inet:bushv", "p.u., Current, Magnitude, referred to network"
      m_inet_busmv = "m:inet:busmv", "p.u., Current, Magnitude, referred to network"
      m_inet_buslv = "m:inet:buslv", "p.u., Current, Magnitude, referred to network"
      m_I_bushv = "m:I:bushv", "kA, Current, Magnitude"
      m_I_busmv = "m:I:busmv", "kA, Current, Magnitude"
      m_I_buslv = "m:I:buslv", "kA, Current, Magnitude"
      m_phii_bushv = "m:phii:bushv", "deg, Current, Angle"
      m_phii_busmv = "m:phii:busmv", "deg, Current, Angle"
      m_phii_buslv = "m:phii:buslv", "deg, Current, Angle"
      m_i1r_bushv = "m:i1r:bushv", "p.u., Positive-Sequence Current, Real Part"
      m_i1r_busmv = "m:i1r:busmv", "p.u., Positive-Sequence Current, Real Part"
      m_i1r_buslv = "m:i1r:buslv", "p.u., Positive-Sequence Current, Real Part"
      m_i1i_bushv = "m:i1i:bushv", "p.u., Positive-Sequence Current, Imaginary Part"
      m_i1i_busmv = "m:i1i:busmv", "p.u., Positive-Sequence Current, Imaginary Part"
      m_i1i_buslv = "m:i1i:buslv", "p.u., Positive-Sequence Current, Imaginary Part"
      m_i1_bushv = "m:i1:bushv", "p.u., Positive-Sequence Current, Magnitude"
      m_i1_busmv = "m:i1:busmv", "p.u., Positive-Sequence Current, Magnitude"
      m_i1_buslv = "m:i1:buslv", "p.u., Positive-Sequence Current, Magnitude"
      m_I1_bushv = "m:I1:bushv", "kA, Positive-Sequence Current, Magnitude"
      m_I1_busmv = "m:I1:busmv", "kA, Positive-Sequence Current, Magnitude"
      m_I1_buslv = "m:I1:buslv", "kA, Positive-Sequence Current, Magnitude"
      m_phii1_bushv = "m:phii1:bushv", "deg, Positive-Sequence Current, Angle"
      m_phii1_busmv = "m:phii1:busmv", "deg, Positive-Sequence Current, Angle"
      m_phii1_buslv = "m:phii1:buslv", "deg, Positive-Sequence Current, Angle"
      m_P_bushv = "m:P:bushv", "MW, Active Power"
      m_P_busmv = "m:P:busmv", "MW, Active Power"
      m_P_buslv = "m:P:buslv", "MW, Active Power"
      m_Q_bushv = "m:Q:bushv", "Mvar, Reactive Power"
      m_Q_busmv = "m:Q:busmv", "Mvar, Reactive Power"
      m_Q_buslv = "m:Q:buslv", "Mvar, Reactive Power"
      m_S_bushv = "m:S:bushv", "MVA, Apparent Power"
      m_S_busmv = "m:S:busmv", "MVA, Apparent Power"
      m_S_buslv = "m:S:buslv", "MVA, Apparent Power"
      m_cosphi_bushv = "m:cosphi:bushv", "Power Factor"
      m_cosphi_busmv = "m:cosphi:busmv", "Power Factor"
      m_cosphi_buslv = "m:cosphi:buslv", "Power Factor"
      m_tanphi_bushv = "m:tanphi:bushv", "tan(phi)"
      m_tanphi_busmv = "m:tanphi:busmv", "tan(phi)"
      m_tanphi_buslv = "m:tanphi:buslv", "tan(phi)"
      m_Psum_bushv = "m:Psum:bushv", "MW, Total Active Power"
      m_Psum_busmv = "m:Psum:busmv", "MW, Total Active Power"
      m_Psum_buslv = "m:Psum:buslv", "MW, Total Active Power"
      m_Qsum_bushv = "m:Qsum:bushv", "Mvar, Total Reactive Power"
      m_Qsum_busmv = "m:Qsum:busmv", "Mvar, Total Reactive Power"
      m_Qsum_buslv = "m:Qsum:buslv", "Mvar, Total Reactive Power"
      m_Ssum_bushv = "m:Ssum:bushv", "MVA, Total Apparent Power"
      m_Ssum_busmv = "m:Ssum:busmv", "MVA, Total Apparent Power"
      m_Ssum_buslv = "m:Ssum:buslv", "MVA, Total Apparent Power"
      m_cosphisum_bushv = "m:cosphisum:bushv", "Total Power Factor"
      m_cosphisum_busmv = "m:cosphisum:busmv", "Total Power Factor"
      m_cosphisum_buslv = "m:cosphisum:buslv", "Total Power Factor"
      m_tanphisum_bushv = "m:tanphisum:bushv", "Total tan(phi)"
      m_tanphisum_busmv = "m:tanphisum:busmv", "Total tan(phi)"
      m_tanphisum_buslv = "m:tanphisum:buslv", "Total tan(phi)"
      m_Spu_bushv = "m:Spu:bushv", "MVA/p.u., Apparent Power per p.u. Voltage"
      m_Spu_busmv = "m:Spu:busmv", "MVA/p.u., Apparent Power per p.u. Voltage"
      m_Spu_buslv = "m:Spu:buslv", "MVA/p.u., Apparent Power per p.u. Voltage"
      m_i1P_bushv = "m:i1P:bushv", "p.u., Positive-Sequence Active Current"
      m_i1P_busmv = "m:i1P:busmv", "p.u., Positive-Sequence Active Current"
      m_i1P_buslv = "m:i1P:buslv", "p.u., Positive-Sequence Active Current"
      m_i1Q_bushv = "m:i1Q:bushv", "p.u., Positive-Sequence Reactive Current"
      m_i1Q_busmv = "m:i1Q:busmv", "p.u., Positive-Sequence Reactive Current"
      m_i1Q_buslv = "m:i1Q:buslv", "p.u., Positive-Sequence Reactive Current"
      m_I1P_bushv = "m:I1P:bushv", "kA, Positive-Sequence Active Current"
      m_I1P_busmv = "m:I1P:busmv", "kA, Positive-Sequence Active Current"
      m_I1P_buslv = "m:I1P:buslv", "kA, Positive-Sequence Active Current"
      m_I1Q_bushv = "m:I1Q:bushv", "kA, Positive-Sequence Reactive Current"
      m_I1Q_busmv = "m:I1Q:busmv", "kA, Positive-Sequence Reactive Current"
      m_I1Q_buslv = "m:I1Q:buslv", "kA, Positive-Sequence Reactive Current"
      m_ur_bushv = "m:ur:bushv", "p.u., Voltage, Real Part"
      m_ur_busmv = "m:ur:busmv", "p.u., Voltage, Real Part"
      m_ur_buslv = "m:ur:buslv", "p.u., Voltage, Real Part"
      m_ui_bushv = "m:ui:bushv", "p.u., Voltage, Imaginary Part"
      m_ui_busmv = "m:ui:busmv", "p.u., Voltage, Imaginary Part"
      m_ui_buslv = "m:ui:buslv", "p.u., Voltage, Imaginary Part"
      m_u_bushv = "m:u:bushv", "p.u., Voltage, Magnitude"
      m_u_busmv = "m:u:busmv", "p.u., Voltage, Magnitude"
      m_u_buslv = "m:u:buslv", "p.u., Voltage, Magnitude"
      m_ir_bushv = "m:ir:bushv", "p.u., Current, Real Part"
      m_ir_busmv = "m:ir:busmv", "p.u., Current, Real Part"
      m_ir_buslv = "m:ir:buslv", "p.u., Current, Real Part"
      m_ii_bushv = "m:ii:bushv", "p.u., Current, Imaginary Part"
      m_ii_busmv = "m:ii:busmv", "p.u., Current, Imaginary Part"
      m_ii_buslv = "m:ii:buslv", "p.u., Current, Imaginary Part"
      m_i_bushv = "m:i:bushv", "p.u., Current, Magnitude"
      m_i_busmv = "m:i:busmv", "p.u., Current, Magnitude"
      m_i_buslv = "m:i:buslv", "p.u., Current, Magnitude"
      m_phiui_bushv = "m:phiui:bushv", "deg, Angle between Voltage and Current"
      m_phiui_busmv = "m:phiui:busmv", "deg, Angle between Voltage and Current"
      m_phiui_buslv = "m:phiui:buslv", "deg, Angle between Voltage and Current"
      m_phiu1i1_bushv = "m:phiu1i1:bushv", "deg, Angle between Voltage and Current in positive sequence system"
      m_phiu1i1_busmv = "m:phiu1i1:busmv", "deg, Angle between Voltage and Current in positive sequence system"
      m_phiu1i1_buslv = "m:phiu1i1:buslv", "deg, Angle between Voltage and Current in positive sequence system"
      m_Tfct_bushv = "m:Tfct:bushv", "s, Fault Clearing Time"
      m_Tfct_busmv = "m:Tfct:busmv", "s, Fault Clearing Time"
      m_Tfct_buslv = "m:Tfct:buslv", "s, Fault Clearing Time"
      m_Brkload_bushv = "m:Brkload:bushv", "%, Breaker Loading"
      m_Brkload_busmv = "m:Brkload:busmv", "%, Breaker Loading"
      m_Brkload_buslv = "m:Brkload:buslv", "%, Breaker Loading"
      n_ur_bushv = "n:ur:bushv", "p.u., Voltage, Real Part"
      n_ur_busmv = "n:ur:busmv", "p.u., Voltage, Real Part"
      n_ur_buslv = "n:ur:buslv", "p.u., Voltage, Real Part"
      n_ui_bushv = "n:ui:bushv", "p.u., Voltage, Imaginary Part"
      n_ui_busmv = "n:ui:busmv", "p.u., Voltage, Imaginary Part"
      n_ui_buslv = "n:ui:buslv", "p.u., Voltage, Imaginary Part"
      n_u_bushv = "n:u:bushv", "p.u., Voltage, Magnitude"
      n_u_busmv = "n:u:busmv", "p.u., Voltage, Magnitude"
      n_u_buslv = "n:u:buslv", "p.u., Voltage, Magnitude"
      n_upc_bushv = "n:upc:bushv", "%, Voltage, Magnitude"
      n_upc_busmv = "n:upc:busmv", "%, Voltage, Magnitude"
      n_upc_buslv = "n:upc:buslv", "%, Voltage, Magnitude"
      n_u1_bushv = "n:u1:bushv", "p.u., Voltage, Magnitude"
      n_u1_busmv = "n:u1:busmv", "p.u., Voltage, Magnitude"
      n_u1_buslv = "n:u1:buslv", "p.u., Voltage, Magnitude"
      n_u1pc_bushv = "n:u1pc:bushv", "%, Voltage, Magnitude"
      n_u1pc_busmv = "n:u1pc:busmv", "%, Voltage, Magnitude"
      n_u1pc_buslv = "n:u1pc:buslv", "%, Voltage, Magnitude"
      n_u1r_bushv = "n:u1r:bushv", "p.u., Positive-Sequence Voltage, Real Part"
      n_u1r_busmv = "n:u1r:busmv", "p.u., Positive-Sequence Voltage, Real Part"
      n_u1r_buslv = "n:u1r:buslv", "p.u., Positive-Sequence Voltage, Real Part"
      n_u1i_bushv = "n:u1i:bushv", "p.u., Positive-Sequence Voltage, Imaginary Part"
      n_u1i_busmv = "n:u1i:busmv", "p.u., Positive-Sequence Voltage, Imaginary Part"
      n_u1i_buslv = "n:u1i:buslv", "p.u., Positive-Sequence Voltage, Imaginary Part"
      n_U_bushv = "n:U:bushv", "kV, Line-Ground Voltage, Magnitude"
      n_U_busmv = "n:U:busmv", "kV, Line-Ground Voltage, Magnitude"
      n_U_buslv = "n:U:buslv", "kV, Line-Ground Voltage, Magnitude"
      n_Ul_bushv = "n:Ul:bushv", "kV, Line-Line Voltage, Magnitude"
      n_Ul_busmv = "n:Ul:busmv", "kV, Line-Line Voltage, Magnitude"
      n_Ul_buslv = "n:Ul:buslv", "kV, Line-Line Voltage, Magnitude"
      n_phiu_bushv = "n:phiu:bushv", "deg, Voltage, Angle"
      n_phiu_busmv = "n:phiu:busmv", "deg, Voltage, Angle"
      n_phiu_buslv = "n:phiu:buslv", "deg, Voltage, Angle"
      n_phiurel_bushv = "n:phiurel:bushv", "deg, Voltage, Relative Angle"
      n_phiurel_busmv = "n:phiurel:busmv", "deg, Voltage, Relative Angle"
      n_phiurel_buslv = "n:phiurel:buslv", "deg, Voltage, Relative Angle"
      n_du_bushv = "n:du:bushv", "%, Voltage Deviation"
      n_du_busmv = "n:du:busmv", "%, Voltage Deviation"
      n_du_buslv = "n:du:buslv", "%, Voltage Deviation"
      n_Pgen_bushv = "n:Pgen:bushv", "MW, Generation, Active Power"
      n_Pgen_busmv = "n:Pgen:busmv", "MW, Generation, Active Power"
      n_Pgen_buslv = "n:Pgen:buslv", "MW, Generation, Active Power"
      n_Qgen_bushv = "n:Qgen:bushv", "Mvar, Generation, Reactive Power"
      n_Qgen_busmv = "n:Qgen:busmv", "Mvar, Generation, Reactive Power"
      n_Qgen_buslv = "n:Qgen:buslv", "Mvar, Generation, Reactive Power"
      n_Pmot_bushv = "n:Pmot:bushv", "MW, Motor Load, Active Power"
      n_Pmot_busmv = "n:Pmot:busmv", "MW, Motor Load, Active Power"
      n_Pmot_buslv = "n:Pmot:buslv", "MW, Motor Load, Active Power"
      n_Qmot_bushv = "n:Qmot:bushv", "Mvar, Motor Load, Reactive Power"
      n_Qmot_busmv = "n:Qmot:busmv", "Mvar, Motor Load, Reactive Power"
      n_Qmot_buslv = "n:Qmot:buslv", "Mvar, Motor Load, Reactive Power"
      n_Pload_bushv = "n:Pload:bushv", "MW, General Load, Active Power"
      n_Pload_busmv = "n:Pload:busmv", "MW, General Load, Active Power"
      n_Pload_buslv = "n:Pload:buslv", "MW, General Load, Active Power"
      n_Qload_bushv = "n:Qload:bushv", "Mvar, General Load, Reactive Power"
      n_Qload_busmv = "n:Qload:busmv", "Mvar, General Load, Reactive Power"
      n_Qload_buslv = "n:Qload:buslv", "Mvar, General Load, Reactive Power"
      n_Pcomp_bushv = "n:Pcomp:bushv", "MW, Compensation (Losses)"
      n_Pcomp_busmv = "n:Pcomp:busmv", "MW, Compensation (Losses)"
      n_Pcomp_buslv = "n:Pcomp:buslv", "MW, Compensation (Losses)"
      n_Qcomp_bushv = "n:Qcomp:bushv", "Mvar, Compensation"
      n_Qcomp_busmv = "n:Qcomp:busmv", "Mvar, Compensation"
      n_Qcomp_buslv = "n:Qcomp:buslv", "Mvar, Compensation"
      n_Pnet_bushv = "n:Pnet:bushv", "MW, External Networks, Active Power"
      n_Pnet_busmv = "n:Pnet:busmv", "MW, External Networks, Active Power"
      n_Pnet_buslv = "n:Pnet:buslv", "MW, External Networks, Active Power"
      n_Qnet_bushv = "n:Qnet:bushv", "Mvar, External Networks, Reactive Power"
      n_Qnet_busmv = "n:Qnet:busmv", "Mvar, External Networks, Reactive Power"
      n_Qnet_buslv = "n:Qnet:buslv", "Mvar, External Networks, Reactive Power"
      n_Pflow_bushv = "n:Pflow:bushv", "MW, Power Flow, Active Power"
      n_Pflow_busmv = "n:Pflow:busmv", "MW, Power Flow, Active Power"
      n_Pflow_buslv = "n:Pflow:buslv", "MW, Power Flow, Active Power"
      n_Qflow_bushv = "n:Qflow:bushv", "Mvar, Power Flow, Reactive Power"
      n_Qflow_busmv = "n:Qflow:busmv", "Mvar, Power Flow, Reactive Power"
      n_Qflow_buslv = "n:Qflow:buslv", "Mvar, Power Flow, Reactive Power"
      n_Pout_bushv = "n:Pout:bushv", "MW, Outgoing Flow, Active Power"
      n_Pout_busmv = "n:Pout:busmv", "MW, Outgoing Flow, Active Power"
      n_Pout_buslv = "n:Pout:buslv", "MW, Outgoing Flow, Active Power"
      n_Qout_bushv = "n:Qout:bushv", "Mvar, Outgoing Power, Reactive Power"
      n_Qout_busmv = "n:Qout:busmv", "Mvar, Outgoing Power, Reactive Power"
      n_Qout_buslv = "n:Qout:buslv", "Mvar, Outgoing Power, Reactive Power"
      n_Sout_bushv = "n:Sout:bushv", "MVA, Outgoing Power, Apparent Power"
      n_Sout_busmv = "n:Sout:busmv", "MVA, Outgoing Power, Apparent Power"
      n_Sout_buslv = "n:Sout:buslv", "MVA, Outgoing Power, Apparent Power"
      n_cosphiout_bushv = "n:cosphiout:bushv", "Outgoing Power, Power Factor"
      n_cosphiout_busmv = "n:cosphiout:busmv", "Outgoing Power, Power Factor"
      n_cosphiout_buslv = "n:cosphiout:buslv", "Outgoing Power, Power Factor"
      n_Pbalance_bushv = "n:Pbalance:bushv", "MW, Active Power Balance (=0)"
      n_Pbalance_busmv = "n:Pbalance:busmv", "MW, Active Power Balance (=0)"
      n_Pbalance_buslv = "n:Pbalance:buslv", "MW, Active Power Balance (=0)"
      n_Qbalance_bushv = "n:Qbalance:bushv", "Mvar, Reactive Power Balance (=0)"
      n_Qbalance_busmv = "n:Qbalance:busmv", "Mvar, Reactive Power Balance (=0)"
      n_Qbalance_buslv = "n:Qbalance:buslv", "Mvar, Reactive Power Balance (=0)"
      n_LossPdown_bushv = "n:LossPdown:bushv", "MW, Losses, downstream"
      n_LossPdown_busmv = "n:LossPdown:busmv", "MW, Losses, downstream"
      n_LossPdown_buslv = "n:LossPdown:buslv", "MW, Losses, downstream"
      n_LossQdown_bushv = "n:LossQdown:bushv", "Mvar, Losses, downstream (Reactive Power)"
      n_LossQdown_busmv = "n:LossQdown:busmv", "Mvar, Losses, downstream (Reactive Power)"
      n_LossQdown_buslv = "n:LossQdown:buslv", "Mvar, Losses, downstream (Reactive Power)"
      n_LossPdownload_bushv = "n:LossPdownload:bushv", "MW, Load losses, downstream"
      n_LossPdownload_busmv = "n:LossPdownload:busmv", "MW, Load losses, downstream"
      n_LossPdownload_buslv = "n:LossPdownload:buslv", "MW, Load losses, downstream"
      n_LossQdownload_bushv = "n:LossQdownload:bushv", "Mvar, Load losses, downstream"
      n_LossQdownload_busmv = "n:LossQdownload:busmv", "Mvar, Load losses, downstream"
      n_LossQdownload_buslv = "n:LossQdownload:buslv", "Mvar, Load losses, downstream"
      n_LossPdownnoload_bushv = "n:LossPdownnoload:bushv", "MW, No load losses, downstream"
      n_LossPdownnoload_busmv = "n:LossPdownnoload:busmv", "MW, No load losses, downstream"
      n_LossPdownnoload_buslv = "n:LossPdownnoload:buslv", "MW, No load losses, downstream"
      n_LossQdownnoload_bushv = "n:LossQdownnoload:bushv", "Mvar, No load losses, downstream"
      n_LossQdownnoload_busmv = "n:LossQdownnoload:busmv", "Mvar, No load losses, downstream"
      n_LossQdownnoload_buslv = "n:LossQdownnoload:buslv", "Mvar, No load losses, downstream"
      n_du_feed_bushv = "n:du_feed:bushv", "%, Voltage difference relative to feeder begin"
      n_du_feed_busmv = "n:du_feed:busmv", "%, Voltage difference relative to feeder begin"
      n_du_feed_buslv = "n:du_feed:buslv", "%, Voltage difference relative to feeder begin"
      s_nt_in_h = "s:nt_in_h", "IN, Tap Position (HV), Input, continuous, double"
      s_nt_in_m = "s:nt_in_m", "IN, Tap Position (MV), Input, continuous, double"
      s_nt_in_l = "s:nt_in_l", "IN, Tap Position (LV), Input, continuous, double"
      s_du_ldc = "s:du_ldc", "IN, LDC Input, continuous, double"
      c_loading_h = "c:loading_h", "%, Loading (HV)"
      c_loading_m = "c:loading_m", "%, Loading (MV)"
      c_loading_l = "c:loading_l", "%, Loading (LV)"
      c_loading = "c:loading", "%, Maximum Loading"
      c_maxload = "c:maxload", "%, Max. Loading"
      c_maxloadpu = "c:maxloadpu", "p.u., Max. Loading"
      c_n3tap_h = "c:n3tap_h", "Current Tap Position (HV)"
      c_n3tap_m = "c:n3tap_m", "Current Tap Position (MV)"
      c_n3tap_l = "c:n3tap_l", "Current Tap Position (LV)"
      c_nntap_int_h = "c:nntap_int_h", "Tap (HV), relative Position"
      c_nntap_int_m = "c:nntap_int_m", "Tap (MV), relative Position"
      c_nntap_int_l = "c:nntap_int_l", "Tap (LV), relative Position"
      c_zs_h_r = "c:zs_h:r", "p.u., Short Circuit Impedance (HV), Real Part"
      c_zs_h_i = "c:zs_h:i", "p.u., Short Circuit Impedance (HV), Imaginary Part"
      c_zs_h_phi = "c:zs_h:phi", "deg, Short Circuit Impedance (HV), Angle"
      c_zs_h_mag = "c:zs_h:mag", "p.u., Short Circuit Impedance (HV), Magnitude"
      c_zs_h_phirad = "c:zs_h:phirad", "rad, Short Circuit Impedance (HV), Angle"
      c_zs_m_r = "c:zs_m:r", "p.u., Short Circuit Impedance (MV), Real Part"
      c_zs_m_i = "c:zs_m:i", "p.u., Short Circuit Impedance (MV), Imaginary Part"
      c_zs_m_phi = "c:zs_m:phi", "deg, Short Circuit Impedance (MV), Angle"
      c_zs_m_mag = "c:zs_m:mag", "p.u., Short Circuit Impedance (MV), Magnitude"
      c_zs_m_phirad = "c:zs_m:phirad", "rad, Short Circuit Impedance (MV), Angle"
      c_zs_l_r = "c:zs_l:r", "p.u., Short Circuit Impedance (LV), Real Part"
      c_zs_l_i = "c:zs_l:i", "p.u., Short Circuit Impedance (LV), Imaginary Part"
      c_zs_l_phi = "c:zs_l:phi", "deg, Short Circuit Impedance (LV), Angle"
      c_zs_l_mag = "c:zs_l:mag", "p.u., Short Circuit Impedance (LV), Magnitude"
      c_zs_l_phirad = "c:zs_l:phirad", "rad, Short Circuit Impedance (LV), Angle"
      c_ym_r = "c:ym:r", "p.u., Magnetising Admittance, Real Part"
      c_ym_i = "c:ym:i", "p.u., Magnetising Admittance, Imaginary Part"
      c_ym_phi = "c:ym:phi", "deg, Magnetising Admittance, Angle"
      c_ym_mag = "c:ym:mag", "p.u., Magnetising Admittance, Magnitude"
      c_ym_phirad = "c:ym:phirad", "rad, Magnetising Admittance, Angle"
      c_t_h_r = "c:t_h:r", "Complex Winding Ratio (HV/S), Real Part"
      c_t_h_i = "c:t_h:i", "Complex Winding Ratio (HV/S), Imaginary Part"
      c_t_h_phi = "c:t_h:phi", "Complex Winding Ratio (HV/S), Angle"
      c_t_h_mag = "c:t_h:mag", "Complex Winding Ratio (HV/S), Magnitude"
      c_t_h_phirad = "c:t_h:phirad", "Complex Winding Ratio (HV/S), Angle"
      c_t_m_r = "c:t_m:r", "Complex Winding Ratio (MV/S), Real Part"
      c_t_m_i = "c:t_m:i", "Complex Winding Ratio (MV/S), Imaginary Part"
      c_t_m_phi = "c:t_m:phi", "Complex Winding Ratio (MV/S), Angle"
      c_t_m_mag = "c:t_m:mag", "Complex Winding Ratio (MV/S), Magnitude"
      c_t_m_phirad = "c:t_m:phirad", "Complex Winding Ratio (MV/S), Angle"
      c_t_l_r = "c:t_l:r", "Complex Winding Ratio (LV/S), Real Part"
      c_t_l_i = "c:t_l:i", "Complex Winding Ratio (LV/S), Imaginary Part"
      c_t_l_phi = "c:t_l:phi", "Complex Winding Ratio (LV/S), Angle"
      c_t_l_mag = "c:t_l:mag", "Complex Winding Ratio (LV/S), Magnitude"
      c_t_l_phirad = "c:t_l:phirad", "Complex Winding Ratio (LV/S), Angle"
      c_i_uplim = "c:i_uplim", "Tap at Maximum"
      c_i_lowlim = "c:i_lowlim", "Tap at Minimum"
      c_nCtrlIter = "c:nCtrlIter", "No. of CTRL-iterations"
      c_Pmeas = "c:Pmeas", "MW, Measured active power"
      c_i_fixedtap = "c:i_fixedtap", "Flag"
      c_um_1_r = "c:um_1:r", "Magnetising Flux, Real Part"
      c_um_1_i = "c:um_1:i", "Magnetising Flux, Imaginary Part"
      c_um_1_phi = "c:um_1:phi", "Magnetising Flux, Angle"
      c_um_1_mag = "c:um_1:mag", "Magnetising Flux, Magnitude"
      c_um_1_phirad = "c:um_1:phirad", "Magnetising Flux, Angle"
      c_u_h_r = "c:u_h:r", "Internal, Real Part"
      c_u_h_i = "c:u_h:i", "Internal, Imaginary Part"
      c_u_h_phi = "c:u_h:phi", "Internal, Angle"
      c_u_h_mag = "c:u_h:mag", "Internal, Magnitude"
      c_u_h_phirad = "c:u_h:phirad", "Internal, Angle"
      c_u_m_r = "c:u_m:r", "Internal, Real Part"
      c_u_m_i = "c:u_m:i", "Internal, Imaginary Part"
      c_u_m_phi = "c:u_m:phi", "Internal, Angle"
      c_u_m_mag = "c:u_m:mag", "Internal, Magnitude"
      c_u_m_phirad = "c:u_m:phirad", "Internal, Angle"
      c_u_l_r = "c:u_l:r", "Internal, Real Part"
      c_u_l_i = "c:u_l:i", "Internal, Imaginary Part"
      c_u_l_phi = "c:u_l:phi", "Internal, Angle"
      c_u_l_mag = "c:u_l:mag", "Internal, Magnitude"
      c_u_l_phirad = "c:u_l:phirad", "Internal, Angle"
      c_w_0 = "c:w_0", "Nominal winding ratio"
      c_Gmload = "c:Gmload", "kW, Iron Losses"
      c_Xmload = "c:Xmload", "kvar, Magnetising Losses"
      c_Ploss = "c:Ploss", "MW, Losses (total)"
      c_Qloss = "c:Qloss", "Mvar, Reactive-Losses (total)"
      c_Plossld = "c:Plossld", "MW, Losses (load)"
      c_Qlossld = "c:Qlossld", "Mvar, Reactive-Losses (load)"
      c_Plossnld = "c:Plossnld", "MW, Losses (no load)"
      c_Qlossnld = "c:Qlossnld", "Mvar, Reactive-Losses (no load)"
      c_itrcn = "c:itrcn", "Flag f. auto. Tap"
      c_i3loc = "c:i3loc", "Position of Magn.ind."
      c_isat = "c:isat", "Saturated Model"
      c_dphiu_hm = "c:dphiu_hm", "deg, Voltage Drop Angle (HV-MV)"
      c_dphiu_hl = "c:dphiu_hl", "deg, Voltage Drop Angle (HV-LV)"
      c_dphiu_ml = "c:dphiu_ml", "deg, Voltage Drop Angle (MV-LV)"
      c_Inom_HV = "c:Inom_HV", "kA, Nominal Current (HV-Side)"
      c_Inom_MV = "c:Inom_MV", "kA, Nominal Current (MV-Side)"
      c_Inom_LV = "c:Inom_LV", "kA, Nominal Current (LV-Side)"
      e_du3tp_h = "e:du3tp_h", "%, Add. Voltage per Tap"
      e_ph3tr_h = "e:ph3tr_h", "deg, Phase of du"
      e_n3tap_h = "e:n3tap_h", "Act. Position"
      e_du3tp_m = "e:du3tp_m", "%, Add. Voltage per Tap"
      e_ph3tr_m = "e:ph3tr_m", "deg, Phase of du"
      e_n3tap_m = "e:n3tap_m", "Act. Position"
      e_du3tp_l = "e:du3tp_l", "%, Add. Voltage per Tap"
      e_ph3tr_l = "e:ph3tr_l", "deg, Phase of du"
      e_n3tap_l = "e:n3tap_l", "Act. Position"
      e_optaplimit = "e:optaplimit", "Operational limits for tap changer"
      e_optapmin_h = "e:optapmin_h", "Minimum Position"
      e_optapmax_h = "e:optapmax_h", "Maximum Position"
      e_optapmin_m = "e:optapmin_m", "Minimum Position"
      e_optapmax_m = "e:optapmax_m", "Maximum Position"
      e_optapmin_l = "e:optapmin_l", "Minimum Position"
      e_optapmax_l = "e:optapmax_l", "Maximum Position"
      e_maxload = "e:maxload", "%, Max. loading"
      e_lossAssign = "e:lossAssign", "Loss assignment"
      e_ictrlside = "e:ictrlside", "Controller for tap at"
      e_c_ptapc = "e:c_ptapc", "ElmTapctrl, External Tap Controller"
      e_c_pstac = "e:c_pstac", "External Station Controller"
      e_c_plcc = "e:c_plcc", "External LCC Controller"
      e_qdslCtrl = "e:qdslCtrl", "Quasi-Dynamic Model"
      e_ntrcn = "e:ntrcn", "Automatic Tap Changing"
      e_i_cont = "e:i_cont", "Tap Changer"
      e_t3ldc = "e:t3ldc", "Controlled Node is at"
      e_ilcph = "e:ilcph", "Phase"
      e_imldc = "e:imldc", "Control Mode"
      e_uset_mode = "e:uset_mode", "Setpoint"
      e_tapctrl = "e:tapctrl", "ElmTr3,ElmTr2, Tap Controller"
      e_iPpart = "e:iPpart", "Active power participation"
      e_i_rem = "e:i_rem", "Remote Control"
      e_p_rem = "e:p_rem", "StaBar,ElmTerm, Controlled Node"
      e_p_cub = "e:p_cub", "StaCubic,ElmBoundary, Controlled Branch (Cubicle)"
      e_cpCtrlNode = "e:cpCtrlNode", "Target Node"
      e_Kpart = "e:Kpart", "Participation factor"
      e_Ptolerance = "e:Ptolerance", "MW, Tolerance (+/-)"
      e_p_pmeas = "e:p_pmeas", "StaCubic,ElmBoundary, P measured at"
      e_usetp = "e:usetp", "p.u., Voltage Setpoint"
      e_usp_low = "e:usp_low", "p.u., Lower Voltage Bound"
      e_usp_up = "e:usp_up", "p.u., Upper Voltage Bound"
      e_psetp = "e:psetp", "MW, Active Power Setpoint"
      e_psp_low = "e:psp_low", "MW, Lower Active Power Bound"
      e_psp_up = "e:psp_up", "MW, Upper Active Power Bound"
      e_qsetp = "e:qsetp", "Mvar, Reactive Power Setpoint"
      e_qsp_low = "e:qsp_low", "Mvar, Lower Reactive Power Bound"
      e_qsp_up = "e:qsp_up", "Mvar, Upper Reactive Power Bound"
      e_Tctrl = "e:Tctrl", "s, Controller Time Constant"
      e_Kqctrl = "e:Kqctrl", "tap/Mvar, Controller Sensitivity dtap/dQ"
      e_Kpctrl = "e:Kpctrl", "tap/MW, Controller Sensitivity dtap/dP"
      e_ildc = "e:ildc", "Compensation"
      e_pldc = "e:pldc", "StaLdc, External LDC"
      e_icomp = "e:icomp", "Compounding"
      e_Vtolerance = "e:Vtolerance", "%, Tolerance (+/-)"
      e_pVccurve = "e:pVccurve", "IntVctrlcurve, V-Control-Curve"
      e_ictrlside2 = "e:ictrlside2", "Controller for tap at"
      e_ntrcn2 = "e:ntrcn2", "Automatic Tap Changing"
      e_i_cont2 = "e:i_cont2", "Tap Changer"
      e_t3ldc2 = "e:t3ldc2", "Controlled Node is at"
      e_ilcph2 = "e:ilcph2", "Phase"
      e_imldc2 = "e:imldc2", "Control Mode"
      e_uset_mode2 = "e:uset_mode2", "Setpoint"
      e_iPpart2 = "e:iPpart2", "Active power participation"
      e_i_rem2 = "e:i_rem2", "Remote Control"
      e_p_rem2 = "e:p_rem2", "StaBar,ElmTerm, Controlled Node"
      e_p_cub2 = "e:p_cub2", "StaCubic,ElmBoundary, Controlled Branch (Cubicle)"
      e_cpCtrlNode2 = "e:cpCtrlNode2", "Target Node"
      e_Kpart2 = "e:Kpart2", "Participation factor"
      e_Ptolerance2 = "e:Ptolerance2", "MW, Tolerance (+/-)"
      e_p_pmeas2 = "e:p_pmeas2", "StaCubic,ElmBoundary, P measured at"
      e_usetp2 = "e:usetp2", "p.u., Voltage Setpoint"
      e_usp_low2 = "e:usp_low2", "p.u., Lower Voltage Bound"
      e_usp_up2 = "e:usp_up2", "p.u., Upper Voltage Bound"
      e_psetp2 = "e:psetp2", "MW, Active Power Setpoint"
      e_psp_low2 = "e:psp_low2", "MW, Lower Active Power Bound"
      e_psp_up2 = "e:psp_up2", "MW, Upper Active Power Bound"
      e_qsetp2 = "e:qsetp2", "Mvar, Reactive Power Setpoint"
      e_qsp_low2 = "e:qsp_low2", "Mvar, Lower Reactive Power Bound"
      e_qsp_up2 = "e:qsp_up2", "Mvar, Upper Reactive Power Bound"
      e_Tctrl2 = "e:Tctrl2", "s, Controller Time Constant"
      e_Kqctrl2 = "e:Kqctrl2", "tap/Mvar, Controller Sensitivity dtap/dQ"
      e_Kpctrl2 = "e:Kpctrl2", "tap/MW, Controller Sensitivity dtap/dP"
      e_ildc2 = "e:ildc2", "Compensation"
      e_pldc2 = "e:pldc2", "StaLdc, External LDC"
      e_icomp2 = "e:icomp2", "Compounding"
      e_Vtolerance2 = "e:Vtolerance2", "%, Tolerance (+/-)"
      e_pVccurve2 = "e:pVccurve2", "IntVctrlcurve, V-Control-Curve"
      e_iTaps = "e:iTaps", "According to Measurement Report"
      e_iMeasTap = "e:iMeasTap", "for Tap at"
      e_iMeasLoc = "e:iMeasLoc", "Measured at"
      e_mTaps = "e:mTaps", "Measurement Report"
      e_iopt_hf = "e:iopt_hf", "Consider Capacitances"
      e_Cg_h = "e:Cg_h", "uF, HV-Ground"
      e_Cg_m = "e:Cg_m", "uF, MV-Ground"
      e_Cg_l = "e:Cg_l", "uF, LV-Ground"
      e_Cc1_hm = "e:Cc1_hm", "uF, HV-MV"
      e_Cc1_ml = "e:Cc1_ml", "uF, MV-LV"
      e_Cc1_lh = "e:Cc1_lh", "uF, LV-HV"
      e_Cc0_hm = "e:Cc0_hm", "uF, HV-MV"
      e_Cc0_ml = "e:Cc0_ml", "uF, MV-LV"
      e_Cc0_lh = "e:Cc0_lh", "uF, LV-HV"
      e_cpFeed = "e:cpFeed", "Feeder"
      e_ciDist = "e:ciDist", "Distance from infeed in number of buses"
      e_ciLater = "e:ciLater", "Lateral Index"
      e_ciDistRoot = "e:ciDistRoot", "Distance from first infeed in number of buses"
      e_ciDistAll = "e:ciDistAll", "Distance from infeed in number of buses including switches"
      e_ciDistAllRoot = "e:ciDistAllRoot", "Distance from first infeed in number of buses including switches"
      t_du3tp_h = "t:du3tp_h", "%, Add. Voltage per Tap"
      t_ph3tr_h = "t:ph3tr_h", "deg, Phase of du"
      t_n3tp0_h = "t:n3tp0_h", "Neutral Position"
      t_n3tmn_h = "t:n3tmn_h", "Min. Position"
      t_n3tmx_h = "t:n3tmx_h", "Max. Position"
      t_du3tp_m = "t:du3tp_m", "%, Add. Voltage per Tap"
      t_ph3tr_m = "t:ph3tr_m", "deg, Phase of du"
      t_n3tp0_m = "t:n3tp0_m", "Neutral Position"
      t_n3tmn_m = "t:n3tmn_m", "Min. Position"
      t_n3tmx_m = "t:n3tmx_m", "Max. Position"
      t_du3tp_l = "t:du3tp_l", "%, Add. Voltage per Tap"
      t_ph3tr_l = "t:ph3tr_l", "deg, Phase of du"
      t_n3tp0_l = "t:n3tp0_l", "Neutral Position"
      t_n3tmn_l = "t:n3tmn_l", "Min. Position"
      t_n3tmx_l = "t:n3tmx_l", "Max. Position"
      t_itapos = "t:itapos", "Tap Modelled at"
      t_i3loc = "t:i3loc", "Position"
      t_curm3 = "t:curm3", "%, No Load Current"
      t_pfe = "t:pfe", "kW, No Load Losses"
      t_i30lc = "t:i30lc", "Position"
      t_cr0m3 = "t:cr0m3", "%, No Load Current"
      t_rtox0_n = "t:rtox0_n", "Mag. R/X"
      t_iTapLoading = "t:iTapLoading", "Configuration"
      t_itapzdep = "t:itapzdep", "Tap dependent impedance"
      t_itapzside = "t:itapzside", "for Tap at"
      t_uktr3_h = "t:uktr3_h", "%, HV-MV"
      t_uktr3mn_h = "t:uktr3mn_h", "%, uk(HV-MV)(min. tap)"
      t_uktr3mx_h = "t:uktr3mx_h", "%, uk(HV-MV)(max. tap)"
      t_uktr3_m = "t:uktr3_m", "%, MV-LV"
      t_uktr3mn_m = "t:uktr3mn_m", "%, uk(MV-LV)(min. tap)"
      t_uktr3mx_m = "t:uktr3mx_m", "%, uk(MV-LV)(max. tap)"
      t_uktr3_l = "t:uktr3_l", "%, LV-HV"
      t_uktr3mn_l = "t:uktr3mn_l", "%, uk(LV-HV)(min. tap)"
      t_uktr3mx_l = "t:uktr3mx_l", "%, uk(LV-HV)(max. tap)"
      t_x1pu_h = "t:x1pu_h", "p.u., HV-MV"
      t_x1putmn_h = "t:x1putmn_h", "p.u., x1(HV-MV) (min. tap)"
      t_x1putmx_h = "t:x1putmx_h", "p.u., x1(HV-MV) (max. tap)"
      t_x1pu_m = "t:x1pu_m", "p.u., MV-LV"
      t_x1putmn_m = "t:x1putmn_m", "p.u., x1(MV-LV) (min. tap)"
      t_x1putmx_m = "t:x1putmx_m", "p.u., x1(MV-LV) (max. tap)"
      t_x1pu_l = "t:x1pu_l", "p.u., LV-HV"
      t_x1putmn_l = "t:x1putmn_l", "p.u., x1(LV-HV) (min. tap)"
      t_x1putmx_l = "t:x1putmx_l", "p.u., x1(LV-HV) (max. tap)"
      t_pcut3_h = "t:pcut3_h", "kW, HV-MV"
      t_uktrr3_h = "t:uktrr3_h", "%, HV-MV"
      t_xtor3_h = "t:xtor3_h", "HV-MV"
      t_r1pu_h = "t:r1pu_h", "p.u., HV-MV"
      t_pcut3mn_h = "t:pcut3mn_h", "kW, Pcu(HV-MV)(min. tap)"
      t_uktrr3mn_h = "t:uktrr3mn_h", "%, Re(uk)(HV-MV) (min. tap)"
      t_xtor3mn_h = "t:xtor3mn_h", "X/R (HV-MV)(min. tap)"
      t_r1putmn_h = "t:r1putmn_h", "p.u., r1(HV-MV) (min. tap)"
      t_pcut3mx_h = "t:pcut3mx_h", "kW, Pcu(HV-MV)(max. tap)"
      t_uktrr3mx_h = "t:uktrr3mx_h", "%, Re(uk)(HV-MV) (max. tap)"
      t_xtor3mx_h = "t:xtor3mx_h", "X/R (HV-MV)(max. tap)"
      t_r1putmx_h = "t:r1putmx_h", "p.u., r1(HV-MV) (max. tap)"
      t_pcut3_m = "t:pcut3_m", "kW, MV-LV"
      t_uktrr3_m = "t:uktrr3_m", "%, MV-LV"
      t_xtor3_m = "t:xtor3_m", "MV-LV"
      t_r1pu_m = "t:r1pu_m", "p.u., MV-LV"
      t_pcut3mn_m = "t:pcut3mn_m", "kW, Pcu(MV-LV)(min. tap)"
      t_uktrr3mn_m = "t:uktrr3mn_m", "%, Re(uk)(MV-LV) (min. tap)"
      t_xtor3mn_m = "t:xtor3mn_m", "X/R (MV-LV)(min. tap)"
      t_r1putmn_m = "t:r1putmn_m", "p.u., r1(MV-LV) (min. tap)"
      t_pcut3mx_m = "t:pcut3mx_m", "kW, Pcu(MV-LV)(max. tap)"
      t_uktrr3mx_m = "t:uktrr3mx_m", "%, Re(uk)(MV-LV) (max. tap)"
      t_xtor3mx_m = "t:xtor3mx_m", "X/R (MV-LV)(max. tap)"
      t_r1putmx_m = "t:r1putmx_m", "p.u., r1(MV-LV) (max. tap)"
      t_pcut3_l = "t:pcut3_l", "kW, LV-HV"
      t_uktrr3_l = "t:uktrr3_l", "%, LV-HV"
      t_xtor3_l = "t:xtor3_l", "LV-HV"
      t_r1pu_l = "t:r1pu_l", "p.u., LV-HV"
      t_pcut3mn_l = "t:pcut3mn_l", "kW, Pcu(LV-HV)(min. tap)"
      t_uktrr3mn_l = "t:uktrr3mn_l", "%, Re(uk)(LV-HV) (min. tap)"
      t_xtor3mn_l = "t:xtor3mn_l", "X/R (LV-HV)(min. tap)"
      t_r1putmn_l = "t:r1putmn_l", "p.u., r1(LV-HV) (min. tap)"
      t_pcut3mx_l = "t:pcut3mx_l", "kW, Pcu(LV-HV)(max. tap)"
      t_uktrr3mx_l = "t:uktrr3mx_l", "%, Re(uk)(LV-HV) (max. tap)"
      t_xtor3mx_l = "t:xtor3mx_l", "X/R (LV-HV)(max. tap)"
      t_r1putmx_l = "t:r1putmx_l", "p.u., r1(LV-HV) (max. tap)"
      t_uk0hm = "t:uk0hm", "%, HV-MV"
      t_uk0mnhm = "t:uk0mnhm", "%, uk0(HV-MV)(min. tap)"
      t_uk0mxhm = "t:uk0mxhm", "%, uk0(HV-MV)(max. tap)"
      t_uk0ml = "t:uk0ml", "%, MV-LV"
      t_uk0mnml = "t:uk0mnml", "%, uk0(MV-LV)(min. tap)"
      t_uk0mxml = "t:uk0mxml", "%, uk0(MV-LV)(max. tap)"
      t_uk0hl = "t:uk0hl", "%, LV-HV"
      t_uk0mnhl = "t:uk0mnhl", "%, uk0(LV-HV)(min. tap)"
      t_uk0mxhl = "t:uk0mxhl", "%, uk0(LV-HV)(max. tap)"
      t_x0puhm = "t:x0puhm", "p.u., HV-MV"
      t_x0pumnhm = "t:x0pumnhm", "p.u., x0(HV-MV) (min. tap)"
      t_x0pumxhm = "t:x0pumxhm", "p.u., x0(HV-MV) (max. tap)"
      t_x0puml = "t:x0puml", "p.u., MV-LV"
      t_x0pumnml = "t:x0pumnml", "p.u., x0(MV-LV) (min. tap)"
      t_x0pumxml = "t:x0pumxml", "p.u., x0(MV-LV) (max. tap)"
      t_x0puhl = "t:x0puhl", "p.u., LV-HV"
      t_x0pumnhl = "t:x0pumnhl", "p.u., x0(LV-HV) (min. tap)"
      t_x0pumxhl = "t:x0pumxhl", "p.u., x0(LV-HV) (max. tap)"
      t_ur0hm = "t:ur0hm", "%, HV-MV"
      t_ur0mnhm = "t:ur0mnhm", "%, Re(uk0)(HV-MV)(min. tap)"
      t_ur0mxhm = "t:ur0mxhm", "%, Re(uk0)(HV-MV)(max. tap)"
      t_ur0ml = "t:ur0ml", "%, MV-LV"
      t_ur0mnml = "t:ur0mnml", "%, Re(uk0)(MV-LV)(min. tap)"
      t_ur0mxml = "t:ur0mxml", "%, Re(uk0)(MV-LV)(max. tap)"
      t_ur0hl = "t:ur0hl", "%, LV-HV"
      t_ur0mnhl = "t:ur0mnhl", "%, Re(uk0)(LV-HV)(min. tap)"
      t_ur0mxhl = "t:ur0mxhl", "%, Re(uk0)(LV-HV)(max. tap)"
      t_xtr0hm = "t:xtr0hm", "HV-MV"
      t_xtr0mnhm = "t:xtr0mnhm", "X0/R0 (HV-MV)(min. tap)"
      t_xtr0mxhm = "t:xtr0mxhm", "X0/R0 (HV-MV)(max. tap)"
      t_xtr0ml = "t:xtr0ml", "MV-LV"
      t_xtr0mnml = "t:xtr0mnml", "X0/R0 (MV-LV)(min. tap)"
      t_xtr0mxml = "t:xtr0mxml", "X0/R0 (MV-LV)(max. tap)"
      t_xtr0hl = "t:xtr0hl", "LV-HV"
      t_xtr0mnhl = "t:xtr0mnhl", "X0/R0 (LV-HV)(min. tap)"
      t_xtr0mxhl = "t:xtr0mxhl", "X0/R0 (LV-HV)(max. tap)"
      t_r0puhm = "t:r0puhm", "p.u., HV-MV"
      t_r0pumnhm = "t:r0pumnhm", "p.u., r0(HV-MV) (min. tap)"
      t_r0pumxhm = "t:r0pumxhm", "p.u., r0(HV-MV) (max. tap)"
      t_r0puml = "t:r0puml", "p.u., MV-LV"
      t_r0pumnml = "t:r0pumnml", "p.u., r0(MV-LV) (min. tap)"
      t_r0pumxml = "t:r0pumxml", "p.u., r0(MV-LV) (max. tap)"
      t_r0puhl = "t:r0puhl", "p.u., LV-HV"
      t_r0pumnhl = "t:r0pumnhl", "p.u., r0(LV-HV) (min. tap)"
      t_r0pumxhl = "t:r0pumxhl", "p.u., r0(LV-HV) (max. tap)"
      t_itrldf = "t:itrldf", "Type"
      t_satcue = "t:satcue", "%, Current (RMS)"
      t_satvol = "t:satvol", "p.u., Voltage (RMS)"
      t_iIntPola = "t:iIntPola", "Interpolation"
      t_smoothf = "t:smoothf", "%, Smoothing Factor"
      t_itratioadpt = "t:itratioadpt", "Transformer Ratio Adaptation"
      r_c_ptapc = "r:c_ptapc", "ElmTapctrl, External Tap Controller"
      r_c_pstac = "r:c_pstac", "External Station Controller"
      r_c_plcc = "r:c_plcc", "External LCC Controller"
      r_qdslCtrl = "r:qdslCtrl", "Quasi-Dynamic Model"
      r_tapctrl = "r:tapctrl", "ElmTr3,ElmTr2, Tap Controller"
      r_p_rem = "r:p_rem", "StaBar,ElmTerm, Controlled Node"
      r_p_cub = "r:p_cub", "StaCubic,ElmBoundary, Controlled Branch (Cubicle)"
      r_cpCtrlNode = "r:cpCtrlNode", "Target Node"
      r_p_pmeas = "r:p_pmeas", "StaCubic,ElmBoundary, P measured at"
      r_pldc = "r:pldc", "StaLdc, External LDC"
      r_pVccurve = "r:pVccurve", "IntVctrlcurve, V-Control-Curve"
      r_p_rem2 = "r:p_rem2", "StaBar,ElmTerm, Controlled Node"
      r_p_cub2 = "r:p_cub2", "StaCubic,ElmBoundary, Controlled Branch (Cubicle)"
      r_cpCtrlNode2 = "r:cpCtrlNode2", "Target Node"
      r_p_pmeas2 = "r:p_pmeas2", "StaCubic,ElmBoundary, P measured at"
      r_pldc2 = "r:pldc2", "StaLdc, External LDC"
      r_pVccurve2 = "r:pVccurve2", "IntVctrlcurve, V-Control-Curve"
      r_cpFeed = "r:cpFeed", "Feeder"

    class ElmVac(Enum):
      _init_ = 'value __doc__'

      m_u1r_bus1 = "m:u1r:bus1", "p.u., Positive-Sequence-Voltage, Real Part"
      m_u1i_bus1 = "m:u1i:bus1", "p.u., Positive-Sequence-Voltage, Imaginary Part"
      m_u1_bus1 = "m:u1:bus1", "p.u., Positive-Sequence-Voltage, Magnitude"
      m_U1_bus1 = "m:U1:bus1", "kV, Line-Ground Positive-Sequence-Voltage, Magnitude"
      m_U1l_bus1 = "m:U1l:bus1", "kV, Line-Line Positive-Sequence-Voltage, Magnitude"
      m_phiu1_bus1 = "m:phiu1:bus1", "deg, Positive-Sequence-Voltage, Angle"
      m_inet_bus1 = "m:inet:bus1", "p.u., Current, Magnitude, referred to network"
      m_I_bus1 = "m:I:bus1", "kA, Current, Magnitude"
      m_phii_bus1 = "m:phii:bus1", "deg, Current, Angle"
      m_i1r_bus1 = "m:i1r:bus1", "p.u., Positive-Sequence Current, Real Part"
      m_i1i_bus1 = "m:i1i:bus1", "p.u., Positive-Sequence Current, Imaginary Part"
      m_i1_bus1 = "m:i1:bus1", "p.u., Positive-Sequence Current, Magnitude"
      m_I1_bus1 = "m:I1:bus1", "kA, Positive-Sequence Current, Magnitude"
      m_phii1_bus1 = "m:phii1:bus1", "deg, Positive-Sequence Current, Angle"
      m_P_bus1 = "m:P:bus1", "MW, Active Power"
      m_Q_bus1 = "m:Q:bus1", "Mvar, Reactive Power"
      m_S_bus1 = "m:S:bus1", "MVA, Apparent Power"
      m_cosphi_bus1 = "m:cosphi:bus1", "Power Factor"
      m_tanphi_bus1 = "m:tanphi:bus1", "tan(phi)"
      m_Psum_bus1 = "m:Psum:bus1", "MW, Total Active Power"
      m_Qsum_bus1 = "m:Qsum:bus1", "Mvar, Total Reactive Power"
      m_Ssum_bus1 = "m:Ssum:bus1", "MVA, Total Apparent Power"
      m_cosphisum_bus1 = "m:cosphisum:bus1", "Total Power Factor"
      m_tanphisum_bus1 = "m:tanphisum:bus1", "Total tan(phi)"
      m_Spu_bus1 = "m:Spu:bus1", "MVA/p.u., Apparent Power per p.u. Voltage"
      m_i1P_bus1 = "m:i1P:bus1", "p.u., Positive-Sequence Active Current"
      m_i1Q_bus1 = "m:i1Q:bus1", "p.u., Positive-Sequence Reactive Current"
      m_I1P_bus1 = "m:I1P:bus1", "kA, Positive-Sequence Active Current"
      m_I1Q_bus1 = "m:I1Q:bus1", "kA, Positive-Sequence Reactive Current"
      m_ur_bus1 = "m:ur:bus1", "p.u., Voltage, Real Part"
      m_ui_bus1 = "m:ui:bus1", "p.u., Voltage, Imaginary Part"
      m_u_bus1 = "m:u:bus1", "p.u., Voltage, Magnitude"
      m_ir_bus1 = "m:ir:bus1", "p.u., Current, Real Part"
      m_ii_bus1 = "m:ii:bus1", "p.u., Current, Imaginary Part"
      m_i_bus1 = "m:i:bus1", "p.u., Current, Magnitude"
      m_phiui_bus1 = "m:phiui:bus1", "deg, Angle between Voltage and Current"
      m_phiu1i1_bus1 = "m:phiu1i1:bus1", "deg, Angle between Voltage and Current in positive sequence system"
      m_Tfct_bus1 = "m:Tfct:bus1", "s, Fault Clearing Time"
      m_Brkload_bus1 = "m:Brkload:bus1", "%, Breaker Loading"
      n_ur_bus1 = "n:ur:bus1", "p.u., Voltage, Real Part"
      n_ui_bus1 = "n:ui:bus1", "p.u., Voltage, Imaginary Part"
      n_u_bus1 = "n:u:bus1", "p.u., Voltage, Magnitude"
      n_upc_bus1 = "n:upc:bus1", "%, Voltage, Magnitude"
      n_u1_bus1 = "n:u1:bus1", "p.u., Voltage, Magnitude"
      n_u1pc_bus1 = "n:u1pc:bus1", "%, Voltage, Magnitude"
      n_u1r_bus1 = "n:u1r:bus1", "p.u., Positive-Sequence Voltage, Real Part"
      n_u1i_bus1 = "n:u1i:bus1", "p.u., Positive-Sequence Voltage, Imaginary Part"
      n_U_bus1 = "n:U:bus1", "kV, Line-Ground Voltage, Magnitude"
      n_Ul_bus1 = "n:Ul:bus1", "kV, Line-Line Voltage, Magnitude"
      n_phiu_bus1 = "n:phiu:bus1", "deg, Voltage, Angle"
      n_phiurel_bus1 = "n:phiurel:bus1", "deg, Voltage, Relative Angle"
      n_du_bus1 = "n:du:bus1", "%, Voltage Deviation"
      n_Pgen_bus1 = "n:Pgen:bus1", "MW, Generation, Active Power"
      n_Qgen_bus1 = "n:Qgen:bus1", "Mvar, Generation, Reactive Power"
      n_Pmot_bus1 = "n:Pmot:bus1", "MW, Motor Load, Active Power"
      n_Qmot_bus1 = "n:Qmot:bus1", "Mvar, Motor Load, Reactive Power"
      n_Pload_bus1 = "n:Pload:bus1", "MW, General Load, Active Power"
      n_Qload_bus1 = "n:Qload:bus1", "Mvar, General Load, Reactive Power"
      n_Pcomp_bus1 = "n:Pcomp:bus1", "MW, Compensation (Losses)"
      n_Qcomp_bus1 = "n:Qcomp:bus1", "Mvar, Compensation"
      n_Pnet_bus1 = "n:Pnet:bus1", "MW, External Networks, Active Power"
      n_Qnet_bus1 = "n:Qnet:bus1", "Mvar, External Networks, Reactive Power"
      n_Pflow_bus1 = "n:Pflow:bus1", "MW, Power Flow, Active Power"
      n_Qflow_bus1 = "n:Qflow:bus1", "Mvar, Power Flow, Reactive Power"
      n_Pout_bus1 = "n:Pout:bus1", "MW, Outgoing Flow, Active Power"
      n_Qout_bus1 = "n:Qout:bus1", "Mvar, Outgoing Power, Reactive Power"
      n_Sout_bus1 = "n:Sout:bus1", "MVA, Outgoing Power, Apparent Power"
      n_cosphiout_bus1 = "n:cosphiout:bus1", "Outgoing Power, Power Factor"
      n_Pbalance_bus1 = "n:Pbalance:bus1", "MW, Active Power Balance (=0)"
      n_Qbalance_bus1 = "n:Qbalance:bus1", "Mvar, Reactive Power Balance (=0)"
      n_LossPdown_bus1 = "n:LossPdown:bus1", "MW, Losses, downstream"
      n_LossQdown_bus1 = "n:LossQdown:bus1", "Mvar, Losses, downstream (Reactive Power)"
      n_LossPdownload_bus1 = "n:LossPdownload:bus1", "MW, Load losses, downstream"
      n_LossQdownload_bus1 = "n:LossQdownload:bus1", "Mvar, Load losses, downstream"
      n_LossPdownnoload_bus1 = "n:LossPdownnoload:bus1", "MW, No load losses, downstream"
      n_LossQdownnoload_bus1 = "n:LossQdownnoload:bus1", "Mvar, No load losses, downstream"
      n_du_feed_bus1 = "n:du_feed:bus1", "%, Voltage difference relative to feeder begin"
      s_u0 = "s:u0", "p.u., IN, Voltage-Input, continuous, double"
      s_Ul0 = "s:Ul0", "kV, IN, Voltage-Input (Line-Line), continuous, double"
      s_U_A = "s:U_A", "kV, IN, Voltage, Magnitude, Phase a, continuous, double"
      s_U_B = "s:U_B", "kV, IN, Voltage, Magnitude, Phase b, continuous, double"
      s_U_C = "s:U_C", "kV, IN, Voltage, Magnitude, Phase c, continuous, double"
      s_phiu_B = "s:phiu_B", "deg, IN, Voltage, Angle, Phase b continuous, double"
      s_phiu_C = "s:phiu_C", "deg, IN, Voltage, Angle, Phase c continuous, double"
      s_dphiu = "s:dphiu", "rad, IN, Voltage Angle Input, continuous, double"
      s_dFin = "s:dFin", "Hz, IN, Frequency Deviation, continuous, double"
      s_dF = "s:dF", "Hz, OUT, Frequency Deviation Output, continuous, double"
      c_uini = "c:uini", "p.u., Voltage"
      c_phiu = "c:phiu", "rad, Voltage Angle"
      c_Kpf = "c:Kpf", "MW/Hz, Primary Frequency Bias"
      c_Ksf = "c:Ksf", "MW/Hz, Secondary Frequency Bias"
      c_Kp = "c:Kp", "Secondary Factor"
      c_iRefElement = "c:iRefElement", "Reference Element"
      c_Ploss = "c:Ploss", "MW, Losses (total)"
      c_Qloss = "c:Qloss", "Mvar, Reactive-Losses (total)"
      c_Plossld = "c:Plossld", "MW, Losses (load)"
      c_Qlossld = "c:Qlossld", "Mvar, Reactive-Losses (load)"
      c_Plossnld = "c:Plossnld", "MW, Losses (no load)"
      c_Qlossnld = "c:Qlossnld", "Mvar, Reactive-Losses (no load)"
      e_usetp = "e:usetp", "p.u., Voltage, Magnitude"
      e_phisetp = "e:phisetp", "deg, Voltage, Angle"
      e_contbar = "e:contbar", "ElmTerm, Controlled Node"
      e_R1 = "e:R1", "Ohm, Resistance, R1"
      e_X1 = "e:X1", "Ohm, Reactance, X1"
      e_p_uctrl = "e:p_uctrl", "ElmLdfctrl*, Voltage Control"
      e_p_phictrl = "e:p_phictrl", "ElmLdfctrl*, Angle Control"
      e_Pload = "e:Pload", "MW, Active Power"
      e_Qload = "e:Qload", "Mvar, Reactive Power"
      e_Pzload = "e:Pzload", "MW, Active Power"
      e_Qzload = "e:Qzload", "Mvar, Reactive Power"
      e_Kpf = "e:Kpf", "MW/Hz, Prim. Frequency Bias"
      e_Ksf = "e:Ksf", "MW/Hz, Sec. Frequency Bias"
      e_c_psecc = "e:c_psecc", "ElmSecctrl, Ext. Secondary Controller"
      e_Pgen = "e:Pgen", "MW, Active Power"
      e_Qgen = "e:Qgen", "Mvar, Reactive Power"
      e_Rext = "e:Rext", "Ohm, Resistance"
      e_Xext = "e:Xext", "Ohm, Reactance"
      e_usetp0 = "e:usetp0", "p.u., Voltage, Magnitude"
      e_phisetp0 = "e:phisetp0", "deg, Voltage, Angle"
      e_R0 = "e:R0", "Ohm, Resistance, R0"
      e_X0 = "e:X0", "Ohm, Reactance, X0"
      e_usetp2 = "e:usetp2", "p.u., Voltage, Magnitude"
      e_phisetp2 = "e:phisetp2", "deg, Voltage, Angle"
      e_R2 = "e:R2", "Ohm, Resistance, R2"
      e_X2 = "e:X2", "Ohm, Reactance, X2"
      e_iPossibleSlack = "e:iPossibleSlack", "Can be considered as a potential slack"
      e_qdslCtrl = "e:qdslCtrl", "Quasi-Dynamic Model"
      e_cpFeed = "e:cpFeed", "Feeder"
      e_ciDist = "e:ciDist", "Distance from infeed in number of buses"
      e_ciLater = "e:ciLater", "Lateral Index"
      e_ciDistRoot = "e:ciDistRoot", "Distance from first infeed in number of buses"
      e_ciDistAll = "e:ciDistAll", "Distance from infeed in number of buses including switches"
      e_ciDistAllRoot = "e:ciDistAllRoot", "Distance from first infeed in number of buses including switches"
      r_contbar = "r:contbar", "ElmTerm, Controlled Node"
      r_p_uctrl = "r:p_uctrl", "ElmLdfctrl*, Voltage Control"
      r_p_phictrl = "r:p_phictrl", "ElmLdfctrl*, Angle Control"
      r_c_psecc = "r:c_psecc", "ElmSecctrl, Ext. Secondary Controller"
      r_qdslCtrl = "r:qdslCtrl", "Quasi-Dynamic Model"
      r_cpFeed = "r:cpFeed", "Feeder"

    class ElmVsc(Enum):
      _init_ = 'value __doc__'

      m_u1r_busac = "m:u1r:busac", "p.u., Positive-Sequence-Voltage, Real Part"
      m_u1r_busdp = "m:u1r:busdp", "p.u., Positive-Sequence-Voltage, Real Part"
      m_u1r_busdm = "m:u1r:busdm", "p.u., Positive-Sequence-Voltage, Real Part"
      m_u1i_busac = "m:u1i:busac", "p.u., Positive-Sequence-Voltage, Imaginary Part"
      m_u1i_busdp = "m:u1i:busdp", "p.u., Positive-Sequence-Voltage, Imaginary Part"
      m_u1i_busdm = "m:u1i:busdm", "p.u., Positive-Sequence-Voltage, Imaginary Part"
      m_u1_busac = "m:u1:busac", "p.u., Positive-Sequence-Voltage, Magnitude"
      m_u1_busdp = "m:u1:busdp", "p.u., Positive-Sequence-Voltage, Magnitude"
      m_u1_busdm = "m:u1:busdm", "p.u., Positive-Sequence-Voltage, Magnitude"
      m_U1_busac = "m:U1:busac", "kV, Line-Ground Positive-Sequence-Voltage, Magnitude"
      m_U1_busdp = "m:U1:busdp", "kV, Line-Ground Positive-Sequence-Voltage, Magnitude"
      m_U1_busdm = "m:U1:busdm", "kV, Line-Ground Positive-Sequence-Voltage, Magnitude"
      m_U1l_busac = "m:U1l:busac", "kV, Line-Line Positive-Sequence-Voltage, Magnitude"
      m_U1l_busdp = "m:U1l:busdp", "kV, Line-Line Positive-Sequence-Voltage, Magnitude"
      m_U1l_busdm = "m:U1l:busdm", "kV, Line-Line Positive-Sequence-Voltage, Magnitude"
      m_phiu1_busac = "m:phiu1:busac", "deg, Positive-Sequence-Voltage, Angle"
      m_phiu1_busdp = "m:phiu1:busdp", "deg, Positive-Sequence-Voltage, Angle"
      m_phiu1_busdm = "m:phiu1:busdm", "deg, Positive-Sequence-Voltage, Angle"
      m_inet_busac = "m:inet:busac", "p.u., Current, Magnitude, referred to network"
      m_inet_busdp = "m:inet:busdp", "p.u., Current, Magnitude, referred to network"
      m_inet_busdm = "m:inet:busdm", "p.u., Current, Magnitude, referred to network"
      m_I_busac = "m:I:busac", "kA, Current, Magnitude"
      m_I_busdp = "m:I:busdp", "kA, Current, Magnitude"
      m_I_busdm = "m:I:busdm", "kA, Current, Magnitude"
      m_phii_busac = "m:phii:busac", "deg, Current, Angle"
      m_phii_busdp = "m:phii:busdp", "deg, Current, Angle"
      m_phii_busdm = "m:phii:busdm", "deg, Current, Angle"
      m_i1r_busac = "m:i1r:busac", "p.u., Positive-Sequence Current, Real Part"
      m_i1r_busdp = "m:i1r:busdp", "p.u., Positive-Sequence Current, Real Part"
      m_i1r_busdm = "m:i1r:busdm", "p.u., Positive-Sequence Current, Real Part"
      m_i1i_busac = "m:i1i:busac", "p.u., Positive-Sequence Current, Imaginary Part"
      m_i1i_busdp = "m:i1i:busdp", "p.u., Positive-Sequence Current, Imaginary Part"
      m_i1i_busdm = "m:i1i:busdm", "p.u., Positive-Sequence Current, Imaginary Part"
      m_i1_busac = "m:i1:busac", "p.u., Positive-Sequence Current, Magnitude"
      m_i1_busdp = "m:i1:busdp", "p.u., Positive-Sequence Current, Magnitude"
      m_i1_busdm = "m:i1:busdm", "p.u., Positive-Sequence Current, Magnitude"
      m_I1_busac = "m:I1:busac", "kA, Positive-Sequence Current, Magnitude"
      m_I1_busdp = "m:I1:busdp", "kA, Positive-Sequence Current, Magnitude"
      m_I1_busdm = "m:I1:busdm", "kA, Positive-Sequence Current, Magnitude"
      m_phii1_busac = "m:phii1:busac", "deg, Positive-Sequence Current, Angle"
      m_phii1_busdp = "m:phii1:busdp", "deg, Positive-Sequence Current, Angle"
      m_phii1_busdm = "m:phii1:busdm", "deg, Positive-Sequence Current, Angle"
      m_P_busac = "m:P:busac", "MW, Active Power"
      m_P_busdp = "m:P:busdp", "MW, Active Power"
      m_P_busdm = "m:P:busdm", "MW, Active Power"
      m_Q_busac = "m:Q:busac", "Mvar, Reactive Power"
      m_Q_busdp = "m:Q:busdp", "Mvar, Reactive Power"
      m_Q_busdm = "m:Q:busdm", "Mvar, Reactive Power"
      m_S_busac = "m:S:busac", "MVA, Apparent Power"
      m_S_busdp = "m:S:busdp", "MVA, Apparent Power"
      m_S_busdm = "m:S:busdm", "MVA, Apparent Power"
      m_cosphi_busac = "m:cosphi:busac", "Power Factor"
      m_cosphi_busdp = "m:cosphi:busdp", "Power Factor"
      m_cosphi_busdm = "m:cosphi:busdm", "Power Factor"
      m_tanphi_busac = "m:tanphi:busac", "tan(phi)"
      m_tanphi_busdp = "m:tanphi:busdp", "tan(phi)"
      m_tanphi_busdm = "m:tanphi:busdm", "tan(phi)"
      m_Psum_busac = "m:Psum:busac", "MW, Total Active Power"
      m_Psum_busdp = "m:Psum:busdp", "MW, Total Active Power"
      m_Psum_busdm = "m:Psum:busdm", "MW, Total Active Power"
      m_Qsum_busac = "m:Qsum:busac", "Mvar, Total Reactive Power"
      m_Qsum_busdp = "m:Qsum:busdp", "Mvar, Total Reactive Power"
      m_Qsum_busdm = "m:Qsum:busdm", "Mvar, Total Reactive Power"
      m_Ssum_busac = "m:Ssum:busac", "MVA, Total Apparent Power"
      m_Ssum_busdp = "m:Ssum:busdp", "MVA, Total Apparent Power"
      m_Ssum_busdm = "m:Ssum:busdm", "MVA, Total Apparent Power"
      m_cosphisum_busac = "m:cosphisum:busac", "Total Power Factor"
      m_cosphisum_busdp = "m:cosphisum:busdp", "Total Power Factor"
      m_cosphisum_busdm = "m:cosphisum:busdm", "Total Power Factor"
      m_tanphisum_busac = "m:tanphisum:busac", "Total tan(phi)"
      m_tanphisum_busdp = "m:tanphisum:busdp", "Total tan(phi)"
      m_tanphisum_busdm = "m:tanphisum:busdm", "Total tan(phi)"
      m_Spu_busac = "m:Spu:busac", "MVA/p.u., Apparent Power per p.u. Voltage"
      m_Spu_busdp = "m:Spu:busdp", "MVA/p.u., Apparent Power per p.u. Voltage"
      m_Spu_busdm = "m:Spu:busdm", "MVA/p.u., Apparent Power per p.u. Voltage"
      m_i1P_busac = "m:i1P:busac", "p.u., Positive-Sequence Active Current"
      m_i1P_busdp = "m:i1P:busdp", "p.u., Positive-Sequence Active Current"
      m_i1P_busdm = "m:i1P:busdm", "p.u., Positive-Sequence Active Current"
      m_i1Q_busac = "m:i1Q:busac", "p.u., Positive-Sequence Reactive Current"
      m_i1Q_busdp = "m:i1Q:busdp", "p.u., Positive-Sequence Reactive Current"
      m_i1Q_busdm = "m:i1Q:busdm", "p.u., Positive-Sequence Reactive Current"
      m_I1P_busac = "m:I1P:busac", "kA, Positive-Sequence Active Current"
      m_I1P_busdp = "m:I1P:busdp", "kA, Positive-Sequence Active Current"
      m_I1P_busdm = "m:I1P:busdm", "kA, Positive-Sequence Active Current"
      m_I1Q_busac = "m:I1Q:busac", "kA, Positive-Sequence Reactive Current"
      m_I1Q_busdp = "m:I1Q:busdp", "kA, Positive-Sequence Reactive Current"
      m_I1Q_busdm = "m:I1Q:busdm", "kA, Positive-Sequence Reactive Current"
      m_ur_busac = "m:ur:busac", "p.u., Voltage, Real Part"
      m_ur_busdp = "m:ur:busdp", "p.u., Voltage, Real Part"
      m_ur_busdm = "m:ur:busdm", "p.u., Voltage, Real Part"
      m_ui_busac = "m:ui:busac", "p.u., Voltage, Imaginary Part"
      m_ui_busdp = "m:ui:busdp", "p.u., Voltage, Imaginary Part"
      m_ui_busdm = "m:ui:busdm", "p.u., Voltage, Imaginary Part"
      m_u_busac = "m:u:busac", "p.u., Voltage, Magnitude"
      m_u_busdp = "m:u:busdp", "p.u., Voltage, Magnitude"
      m_u_busdm = "m:u:busdm", "p.u., Voltage, Magnitude"
      m_ir_busac = "m:ir:busac", "p.u., Current, Real Part"
      m_ir_busdp = "m:ir:busdp", "p.u., Current, Real Part"
      m_ir_busdm = "m:ir:busdm", "p.u., Current, Real Part"
      m_ii_busac = "m:ii:busac", "p.u., Current, Imaginary Part"
      m_ii_busdp = "m:ii:busdp", "p.u., Current, Imaginary Part"
      m_ii_busdm = "m:ii:busdm", "p.u., Current, Imaginary Part"
      m_i_busac = "m:i:busac", "p.u., Current, Magnitude"
      m_i_busdp = "m:i:busdp", "p.u., Current, Magnitude"
      m_i_busdm = "m:i:busdm", "p.u., Current, Magnitude"
      m_phiui_busac = "m:phiui:busac", "deg, Angle between Voltage and Current"
      m_phiui_busdp = "m:phiui:busdp", "deg, Angle between Voltage and Current"
      m_phiui_busdm = "m:phiui:busdm", "deg, Angle between Voltage and Current"
      m_phiu1i1_busac = "m:phiu1i1:busac", "deg, Angle between Voltage and Current in positive sequence system"
      m_phiu1i1_busdp = "m:phiu1i1:busdp", "deg, Angle between Voltage and Current in positive sequence system"
      m_phiu1i1_busdm = "m:phiu1i1:busdm", "deg, Angle between Voltage and Current in positive sequence system"
      m_Tfct_busac = "m:Tfct:busac", "s, Fault Clearing Time"
      m_Tfct_busdp = "m:Tfct:busdp", "s, Fault Clearing Time"
      m_Tfct_busdm = "m:Tfct:busdm", "s, Fault Clearing Time"
      m_Brkload_busac = "m:Brkload:busac", "%, Breaker Loading"
      m_Brkload_busdp = "m:Brkload:busdp", "%, Breaker Loading"
      m_Brkload_busdm = "m:Brkload:busdm", "%, Breaker Loading"
      n_ur_busac = "n:ur:busac", "p.u., Voltage, Real Part"
      n_ur_busdp = "n:ur:busdp", "p.u., Voltage, Real Part"
      n_ur_busdm = "n:ur:busdm", "p.u., Voltage, Real Part"
      n_ui_busac = "n:ui:busac", "p.u., Voltage, Imaginary Part"
      n_ui_busdp = "n:ui:busdp", "p.u., Voltage, Imaginary Part"
      n_ui_busdm = "n:ui:busdm", "p.u., Voltage, Imaginary Part"
      n_u_busac = "n:u:busac", "p.u., Voltage, Magnitude"
      n_u_busdp = "n:u:busdp", "p.u., Voltage, Magnitude"
      n_u_busdm = "n:u:busdm", "p.u., Voltage, Magnitude"
      n_upc_busac = "n:upc:busac", "%, Voltage, Magnitude"
      n_upc_busdp = "n:upc:busdp", "%, Voltage, Magnitude"
      n_upc_busdm = "n:upc:busdm", "%, Voltage, Magnitude"
      n_u1_busac = "n:u1:busac", "p.u., Voltage, Magnitude"
      n_u1_busdp = "n:u1:busdp", "p.u., Voltage, Magnitude"
      n_u1_busdm = "n:u1:busdm", "p.u., Voltage, Magnitude"
      n_u1pc_busac = "n:u1pc:busac", "%, Voltage, Magnitude"
      n_u1pc_busdp = "n:u1pc:busdp", "%, Voltage, Magnitude"
      n_u1pc_busdm = "n:u1pc:busdm", "%, Voltage, Magnitude"
      n_u1r_busac = "n:u1r:busac", "p.u., Positive-Sequence Voltage, Real Part"
      n_u1r_busdp = "n:u1r:busdp", "p.u., Positive-Sequence Voltage, Real Part"
      n_u1r_busdm = "n:u1r:busdm", "p.u., Positive-Sequence Voltage, Real Part"
      n_u1i_busac = "n:u1i:busac", "p.u., Positive-Sequence Voltage, Imaginary Part"
      n_u1i_busdp = "n:u1i:busdp", "p.u., Positive-Sequence Voltage, Imaginary Part"
      n_u1i_busdm = "n:u1i:busdm", "p.u., Positive-Sequence Voltage, Imaginary Part"
      n_U_busac = "n:U:busac", "kV, Line-Ground Voltage, Magnitude"
      n_U_busdp = "n:U:busdp", "kV, Line-Ground Voltage, Magnitude"
      n_U_busdm = "n:U:busdm", "kV, Line-Ground Voltage, Magnitude"
      n_Ul_busac = "n:Ul:busac", "kV, Line-Line Voltage, Magnitude"
      n_Ul_busdp = "n:Ul:busdp", "kV, Line-Line Voltage, Magnitude"
      n_Ul_busdm = "n:Ul:busdm", "kV, Line-Line Voltage, Magnitude"
      n_phiu_busac = "n:phiu:busac", "deg, Voltage, Angle"
      n_phiu_busdp = "n:phiu:busdp", "deg, Voltage, Angle"
      n_phiu_busdm = "n:phiu:busdm", "deg, Voltage, Angle"
      n_phiurel_busac = "n:phiurel:busac", "deg, Voltage, Relative Angle"
      n_phiurel_busdp = "n:phiurel:busdp", "deg, Voltage, Relative Angle"
      n_phiurel_busdm = "n:phiurel:busdm", "deg, Voltage, Relative Angle"
      n_du_busac = "n:du:busac", "%, Voltage Deviation"
      n_du_busdp = "n:du:busdp", "%, Voltage Deviation"
      n_du_busdm = "n:du:busdm", "%, Voltage Deviation"
      n_Pgen_busac = "n:Pgen:busac", "MW, Generation, Active Power"
      n_Pgen_busdp = "n:Pgen:busdp", "MW, Generation, Active Power"
      n_Pgen_busdm = "n:Pgen:busdm", "MW, Generation, Active Power"
      n_Qgen_busac = "n:Qgen:busac", "Mvar, Generation, Reactive Power"
      n_Qgen_busdp = "n:Qgen:busdp", "Mvar, Generation, Reactive Power"
      n_Qgen_busdm = "n:Qgen:busdm", "Mvar, Generation, Reactive Power"
      n_Pmot_busac = "n:Pmot:busac", "MW, Motor Load, Active Power"
      n_Pmot_busdp = "n:Pmot:busdp", "MW, Motor Load, Active Power"
      n_Pmot_busdm = "n:Pmot:busdm", "MW, Motor Load, Active Power"
      n_Qmot_busac = "n:Qmot:busac", "Mvar, Motor Load, Reactive Power"
      n_Qmot_busdp = "n:Qmot:busdp", "Mvar, Motor Load, Reactive Power"
      n_Qmot_busdm = "n:Qmot:busdm", "Mvar, Motor Load, Reactive Power"
      n_Pload_busac = "n:Pload:busac", "MW, General Load, Active Power"
      n_Pload_busdp = "n:Pload:busdp", "MW, General Load, Active Power"
      n_Pload_busdm = "n:Pload:busdm", "MW, General Load, Active Power"
      n_Qload_busac = "n:Qload:busac", "Mvar, General Load, Reactive Power"
      n_Qload_busdp = "n:Qload:busdp", "Mvar, General Load, Reactive Power"
      n_Qload_busdm = "n:Qload:busdm", "Mvar, General Load, Reactive Power"
      n_Pcomp_busac = "n:Pcomp:busac", "MW, Compensation (Losses)"
      n_Pcomp_busdp = "n:Pcomp:busdp", "MW, Compensation (Losses)"
      n_Pcomp_busdm = "n:Pcomp:busdm", "MW, Compensation (Losses)"
      n_Qcomp_busac = "n:Qcomp:busac", "Mvar, Compensation"
      n_Qcomp_busdp = "n:Qcomp:busdp", "Mvar, Compensation"
      n_Qcomp_busdm = "n:Qcomp:busdm", "Mvar, Compensation"
      n_Pnet_busac = "n:Pnet:busac", "MW, External Networks, Active Power"
      n_Pnet_busdp = "n:Pnet:busdp", "MW, External Networks, Active Power"
      n_Pnet_busdm = "n:Pnet:busdm", "MW, External Networks, Active Power"
      n_Qnet_busac = "n:Qnet:busac", "Mvar, External Networks, Reactive Power"
      n_Qnet_busdp = "n:Qnet:busdp", "Mvar, External Networks, Reactive Power"
      n_Qnet_busdm = "n:Qnet:busdm", "Mvar, External Networks, Reactive Power"
      n_Pflow_busac = "n:Pflow:busac", "MW, Power Flow, Active Power"
      n_Pflow_busdp = "n:Pflow:busdp", "MW, Power Flow, Active Power"
      n_Pflow_busdm = "n:Pflow:busdm", "MW, Power Flow, Active Power"
      n_Qflow_busac = "n:Qflow:busac", "Mvar, Power Flow, Reactive Power"
      n_Qflow_busdp = "n:Qflow:busdp", "Mvar, Power Flow, Reactive Power"
      n_Qflow_busdm = "n:Qflow:busdm", "Mvar, Power Flow, Reactive Power"
      n_Pout_busac = "n:Pout:busac", "MW, Outgoing Flow, Active Power"
      n_Pout_busdp = "n:Pout:busdp", "MW, Outgoing Flow, Active Power"
      n_Pout_busdm = "n:Pout:busdm", "MW, Outgoing Flow, Active Power"
      n_Qout_busac = "n:Qout:busac", "Mvar, Outgoing Power, Reactive Power"
      n_Qout_busdp = "n:Qout:busdp", "Mvar, Outgoing Power, Reactive Power"
      n_Qout_busdm = "n:Qout:busdm", "Mvar, Outgoing Power, Reactive Power"
      n_Sout_busac = "n:Sout:busac", "MVA, Outgoing Power, Apparent Power"
      n_Sout_busdp = "n:Sout:busdp", "MVA, Outgoing Power, Apparent Power"
      n_Sout_busdm = "n:Sout:busdm", "MVA, Outgoing Power, Apparent Power"
      n_cosphiout_busac = "n:cosphiout:busac", "Outgoing Power, Power Factor"
      n_cosphiout_busdp = "n:cosphiout:busdp", "Outgoing Power, Power Factor"
      n_cosphiout_busdm = "n:cosphiout:busdm", "Outgoing Power, Power Factor"
      n_Pbalance_busac = "n:Pbalance:busac", "MW, Active Power Balance (=0)"
      n_Pbalance_busdp = "n:Pbalance:busdp", "MW, Active Power Balance (=0)"
      n_Pbalance_busdm = "n:Pbalance:busdm", "MW, Active Power Balance (=0)"
      n_Qbalance_busac = "n:Qbalance:busac", "Mvar, Reactive Power Balance (=0)"
      n_Qbalance_busdp = "n:Qbalance:busdp", "Mvar, Reactive Power Balance (=0)"
      n_Qbalance_busdm = "n:Qbalance:busdm", "Mvar, Reactive Power Balance (=0)"
      n_LossPdown_busac = "n:LossPdown:busac", "MW, Losses, downstream"
      n_LossPdown_busdp = "n:LossPdown:busdp", "MW, Losses, downstream"
      n_LossPdown_busdm = "n:LossPdown:busdm", "MW, Losses, downstream"
      n_LossQdown_busac = "n:LossQdown:busac", "Mvar, Losses, downstream (Reactive Power)"
      n_LossQdown_busdp = "n:LossQdown:busdp", "Mvar, Losses, downstream (Reactive Power)"
      n_LossQdown_busdm = "n:LossQdown:busdm", "Mvar, Losses, downstream (Reactive Power)"
      n_LossPdownload_busac = "n:LossPdownload:busac", "MW, Load losses, downstream"
      n_LossPdownload_busdp = "n:LossPdownload:busdp", "MW, Load losses, downstream"
      n_LossPdownload_busdm = "n:LossPdownload:busdm", "MW, Load losses, downstream"
      n_LossQdownload_busac = "n:LossQdownload:busac", "Mvar, Load losses, downstream"
      n_LossQdownload_busdp = "n:LossQdownload:busdp", "Mvar, Load losses, downstream"
      n_LossQdownload_busdm = "n:LossQdownload:busdm", "Mvar, Load losses, downstream"
      n_LossPdownnoload_busac = "n:LossPdownnoload:busac", "MW, No load losses, downstream"
      n_LossPdownnoload_busdp = "n:LossPdownnoload:busdp", "MW, No load losses, downstream"
      n_LossPdownnoload_busdm = "n:LossPdownnoload:busdm", "MW, No load losses, downstream"
      n_LossQdownnoload_busac = "n:LossQdownnoload:busac", "Mvar, No load losses, downstream"
      n_LossQdownnoload_busdp = "n:LossQdownnoload:busdp", "Mvar, No load losses, downstream"
      n_LossQdownnoload_busdm = "n:LossQdownnoload:busdm", "Mvar, No load losses, downstream"
      n_du_feed_busac = "n:du_feed:busac", "%, Voltage difference relative to feeder begin"
      n_du_feed_busdp = "n:du_feed:busdp", "%, Voltage difference relative to feeder begin"
      n_du_feed_busdm = "n:du_feed:busdm", "%, Voltage difference relative to feeder begin"
      s_pset = "s:pset", "MW, IN, Active power setpoint, continuous, double"
      s_qset = "s:qset", "Mvar, IN, Reactive power setpoint continuous, double"
      s_udcset = "s:udcset", "p.u., IN, DC Voltage setpoint, continuous, double"
      s_dFin = "s:dFin", "Hz, IN, Frequency Deviation, continuous, double"
      c_phiu = "c:phiu", "rad, Voltage angle"
      c_uini = "c:uini", "p.u., Voltage setpoint"
      c_snomn = "c:snomn", "MVA, Total rated apparent power (nparnum*Snom)"
      c_uinidc = "c:uinidc", "p.u., Voltage setpoint (DC)"
      c_pmsetp = "c:pmsetp", "PWM factor"
      c_i_acdc = "c:i_acdc", "Control mode"
      c_i_mod = "c:i_mod", "Modulation"
      c_K0 = "c:K0", "Vac/Vdc at PWM=1"
      c_Pm = "c:Pm", "Pulse-modulation factor"
      c_uDC = "c:uDC", "p.u., DC-voltage"
      c_pini = "c:pini", "Active power setpoint"
      c_qini = "c:qini", "Reactive power setpoint"
      c_DCPoleLosses = "c:DCPoleLosses", "MW, DC pole losses"
      c_Qmax = "c:Qmax", "Mvar, Upper limit of reactive power"
      c_Qmin = "c:Qmin", "Mvar, Lower limit of reactive power"
      c_Pmin = "c:Pmin", "MW, Lower limit of active power"
      c_Pmax = "c:Pmax", "MW, Upper limit of active power"
      c_i_qlim = "c:i_qlim", "Reactive power limited"
      c_i_plim = "c:i_plim", "Active power limited"
      c_Kpf = "c:Kpf", "MW/Hz, Frequency bias"
      c_Kp = "c:Kp", "p.u., Active power participation"
      c_zonewscale = "c:zonewscale", "Wind generation zone scaling factor"
      c_iRefElement = "c:iRefElement", "Reference element"
      c_Kq = "c:Kq", "p.u., Reactive power participation"
      c_iPowerOn = "c:iPowerOn", "Flag"
      c_i_vlim = "c:i_vlim", "Voltage setpoint limit reached"
      c_dphi = "c:dphi", "deg, Measured angle difference"
      c_Pmeas = "c:Pmeas", "MW, Measured active power"
      c_nCtrlIter = "c:nCtrlIter", "No. of CTRL-iterations"
      c_Ploss = "c:Ploss", "MW, Losses (total)"
      c_Qloss = "c:Qloss", "Mvar, Reactive-Losses (total)"
      c_Plossld = "c:Plossld", "MW, Losses (load)"
      c_Qlossld = "c:Qlossld", "Mvar, Reactive-Losses (load)"
      c_Plossnld = "c:Plossnld", "MW, Losses (no load)"
      c_Qlossnld = "c:Qlossnld", "Mvar, Reactive-Losses (no load)"
      e_i_acdc = "e:i_acdc", "Control mode"
      e_ivacdroop = "e:ivacdroop", "AC-Voltage Droop"
      e_usetp = "e:usetp", "p.u., AC voltage setpoint"
      e_pmsetp = "e:pmsetp", "p.u., PWM factor"
      e_p_uctrl = "e:p_uctrl", "ElmTerm*,StaBar*, Controlled node (AC)"
      e_usetpdc = "e:usetpdc", "p.u., DC voltage setpoint"
      e_phisetp = "e:phisetp", "deg, Phase setpoint"
      e_psetp = "e:psetp", "MW, Active power setpoint"
      e_Kpf = "e:Kpf", "MW/Hz, Prim. frequency bias"
      e_qsetp = "e:qsetp", "Mvar, Reactive power setpoint"
      e_iQorient = "e:iQorient", "Orientation"
      e_pfsetp = "e:pfsetp", "Power factor"
      e_pf_recap = "e:pf_recap", "Power factor"
      e_p_pctrl = "e:p_pctrl", "StaCubic,ElmBoundary, Controlled flow"
      e_c_pstac = "e:c_pstac", "ElmStactrl, External station controller"
      e_c_psecc = "e:c_psecc", "ElmSecctrl, Ext. secondary controller"
      e_iVacMax = "e:iVacMax", "Consider modulation index limit"
      e_Pmmax = "e:Pmmax", "Max. PWM factor"
      e_Pnold = "e:Pnold", "kW, No-load losses"
      e_swtLossFactor = "e:swtLossFactor", "kW/A, Switching loss factor"
      e_resLossFactor = "e:resLossFactor", "Ohm, Resistive loss factor"
      e_ddroop = "e:ddroop", "%, Droop"
      e_usp_min = "e:usp_min", "p.u., Min. voltage setpoint"
      e_usp_max = "e:usp_max", "p.u., Max. voltage setpoint"
      e_idroop = "e:idroop", "DC-voltage dependent P-droop"
      e_K = "e:K", "p.u./MW, K"
      e_iPphidrp = "e:iPphidrp", "Angle difference dependent P-droop"
      e_Kpphi = "e:Kpphi", "MW/deg, Kpphi"
      e_p_b1phiu = "e:p_b1phiu", "ElmTerm*, Remote AC busbar"
      e_p_b2phiu = "e:p_b2phiu", "ElmTerm*, Local AC busbar"
      e_considerPset = "e:considerPset", "Consider active power setpoint"
      e_iPpart = "e:iPpart", "Active power participation"
      e_Kpart = "e:Kpart", "Participation factor"
      e_p_pmeas = "e:p_pmeas", "StaCubic*,ElmBoundary, P(AC) measured at"
      e_pQlimType = "e:pQlimType", "IntQlim, Capability curve"
      e_q_min = "e:q_min", "p.u., Min."
      e_q_max = "e:q_max", "p.u., Max."
      e_cQ_min = "e:cQ_min", "Mvar, Min."
      e_cQ_max = "e:cQ_max", "Mvar, Max."
      e_scaleQmin = "e:scaleQmin", "%, Scaling factor (min.)"
      e_scaleQmax = "e:scaleQmax", "%, Scaling factor (max.)"
      e_Pmin_uc = "e:Pmin_uc", "MW, Min."
      e_Pmax_uc = "e:Pmax_uc", "MW, Max."
      e_Pnom = "e:Pnom", "MW, Pr(rated)"
      e_Qmin_a = "e:Qmin_a", "Mvar, Qmin(act.)"
      e_Qmax_a = "e:Qmax_a", "Mvar, Qmax(act.)"
      e_P_max = "e:P_max", "MW, Max."
      e_r2 = "e:r2", "p.u., Resistance, r2"
      e_x2 = "e:x2", "p.u., Reactance, x2"
      e_priority = "e:priority", "Merit Order"
      e_cpFeed = "e:cpFeed", "Feeder"
      e_ciDist = "e:ciDist", "Distance from infeed in number of buses"
      e_ciLater = "e:ciLater", "Lateral Index"
      e_ciDistRoot = "e:ciDistRoot", "Distance from first infeed in number of buses"
      e_ciDistAll = "e:ciDistAll", "Distance from infeed in number of buses including switches"
      e_ciDistAllRoot = "e:ciDistAllRoot", "Distance from first infeed in number of buses including switches"
      r_p_uctrl = "r:p_uctrl", "ElmTerm*,StaBar*, Controlled node (AC)"
      r_p_pctrl = "r:p_pctrl", "StaCubic,ElmBoundary, Controlled flow"
      r_c_pstac = "r:c_pstac", "ElmStactrl, External station controller"
      r_c_psecc = "r:c_psecc", "ElmSecctrl, Ext. secondary controller"
      r_p_b1phiu = "r:p_b1phiu", "ElmTerm*, Remote AC busbar"
      r_p_b2phiu = "r:p_b2phiu", "ElmTerm*, Local AC busbar"
      r_p_pmeas = "r:p_pmeas", "StaCubic*,ElmBoundary, P(AC) measured at"
      r_pQlimType = "r:pQlimType", "IntQlim, Capability curve"
      r_cpFeed = "r:cpFeed", "Feeder"

    class ElmVscmono(Enum):
      _init_ = 'value __doc__'

      m_u1r_busac = "m:u1r:busac", "p.u., Positive-Sequence-Voltage, Real Part"
      m_u1r_busdc = "m:u1r:busdc", "p.u., Positive-Sequence-Voltage, Real Part"
      m_u1i_busac = "m:u1i:busac", "p.u., Positive-Sequence-Voltage, Imaginary Part"
      m_u1i_busdc = "m:u1i:busdc", "p.u., Positive-Sequence-Voltage, Imaginary Part"
      m_u1_busac = "m:u1:busac", "p.u., Positive-Sequence-Voltage, Magnitude"
      m_u1_busdc = "m:u1:busdc", "p.u., Positive-Sequence-Voltage, Magnitude"
      m_U1_busac = "m:U1:busac", "kV, Line-Ground Positive-Sequence-Voltage, Magnitude"
      m_U1_busdc = "m:U1:busdc", "kV, Line-Ground Positive-Sequence-Voltage, Magnitude"
      m_U1l_busac = "m:U1l:busac", "kV, Line-Line Positive-Sequence-Voltage, Magnitude"
      m_U1l_busdc = "m:U1l:busdc", "kV, Line-Line Positive-Sequence-Voltage, Magnitude"
      m_phiu1_busac = "m:phiu1:busac", "deg, Positive-Sequence-Voltage, Angle"
      m_phiu1_busdc = "m:phiu1:busdc", "deg, Positive-Sequence-Voltage, Angle"
      m_inet_busac = "m:inet:busac", "p.u., Current, Magnitude, referred to network"
      m_inet_busdc = "m:inet:busdc", "p.u., Current, Magnitude, referred to network"
      m_I_busac = "m:I:busac", "kA, Current, Magnitude"
      m_I_busdc = "m:I:busdc", "kA, Current, Magnitude"
      m_phii_busac = "m:phii:busac", "deg, Current, Angle"
      m_phii_busdc = "m:phii:busdc", "deg, Current, Angle"
      m_i1r_busac = "m:i1r:busac", "p.u., Positive-Sequence Current, Real Part"
      m_i1r_busdc = "m:i1r:busdc", "p.u., Positive-Sequence Current, Real Part"
      m_i1i_busac = "m:i1i:busac", "p.u., Positive-Sequence Current, Imaginary Part"
      m_i1i_busdc = "m:i1i:busdc", "p.u., Positive-Sequence Current, Imaginary Part"
      m_i1_busac = "m:i1:busac", "p.u., Positive-Sequence Current, Magnitude"
      m_i1_busdc = "m:i1:busdc", "p.u., Positive-Sequence Current, Magnitude"
      m_I1_busac = "m:I1:busac", "kA, Positive-Sequence Current, Magnitude"
      m_I1_busdc = "m:I1:busdc", "kA, Positive-Sequence Current, Magnitude"
      m_phii1_busac = "m:phii1:busac", "deg, Positive-Sequence Current, Angle"
      m_phii1_busdc = "m:phii1:busdc", "deg, Positive-Sequence Current, Angle"
      m_P_busac = "m:P:busac", "MW, Active Power"
      m_P_busdc = "m:P:busdc", "MW, Active Power"
      m_Q_busac = "m:Q:busac", "Mvar, Reactive Power"
      m_Q_busdc = "m:Q:busdc", "Mvar, Reactive Power"
      m_S_busac = "m:S:busac", "MVA, Apparent Power"
      m_S_busdc = "m:S:busdc", "MVA, Apparent Power"
      m_cosphi_busac = "m:cosphi:busac", "Power Factor"
      m_cosphi_busdc = "m:cosphi:busdc", "Power Factor"
      m_tanphi_busac = "m:tanphi:busac", "tan(phi)"
      m_tanphi_busdc = "m:tanphi:busdc", "tan(phi)"
      m_Psum_busac = "m:Psum:busac", "MW, Total Active Power"
      m_Psum_busdc = "m:Psum:busdc", "MW, Total Active Power"
      m_Qsum_busac = "m:Qsum:busac", "Mvar, Total Reactive Power"
      m_Qsum_busdc = "m:Qsum:busdc", "Mvar, Total Reactive Power"
      m_Ssum_busac = "m:Ssum:busac", "MVA, Total Apparent Power"
      m_Ssum_busdc = "m:Ssum:busdc", "MVA, Total Apparent Power"
      m_cosphisum_busac = "m:cosphisum:busac", "Total Power Factor"
      m_cosphisum_busdc = "m:cosphisum:busdc", "Total Power Factor"
      m_tanphisum_busac = "m:tanphisum:busac", "Total tan(phi)"
      m_tanphisum_busdc = "m:tanphisum:busdc", "Total tan(phi)"
      m_Spu_busac = "m:Spu:busac", "MVA/p.u., Apparent Power per p.u. Voltage"
      m_Spu_busdc = "m:Spu:busdc", "MVA/p.u., Apparent Power per p.u. Voltage"
      m_i1P_busac = "m:i1P:busac", "p.u., Positive-Sequence Active Current"
      m_i1P_busdc = "m:i1P:busdc", "p.u., Positive-Sequence Active Current"
      m_i1Q_busac = "m:i1Q:busac", "p.u., Positive-Sequence Reactive Current"
      m_i1Q_busdc = "m:i1Q:busdc", "p.u., Positive-Sequence Reactive Current"
      m_I1P_busac = "m:I1P:busac", "kA, Positive-Sequence Active Current"
      m_I1P_busdc = "m:I1P:busdc", "kA, Positive-Sequence Active Current"
      m_I1Q_busac = "m:I1Q:busac", "kA, Positive-Sequence Reactive Current"
      m_I1Q_busdc = "m:I1Q:busdc", "kA, Positive-Sequence Reactive Current"
      m_ur_busac = "m:ur:busac", "p.u., Voltage, Real Part"
      m_ur_busdc = "m:ur:busdc", "p.u., Voltage, Real Part"
      m_ui_busac = "m:ui:busac", "p.u., Voltage, Imaginary Part"
      m_ui_busdc = "m:ui:busdc", "p.u., Voltage, Imaginary Part"
      m_u_busac = "m:u:busac", "p.u., Voltage, Magnitude"
      m_u_busdc = "m:u:busdc", "p.u., Voltage, Magnitude"
      m_ir_busac = "m:ir:busac", "p.u., Current, Real Part"
      m_ir_busdc = "m:ir:busdc", "p.u., Current, Real Part"
      m_ii_busac = "m:ii:busac", "p.u., Current, Imaginary Part"
      m_ii_busdc = "m:ii:busdc", "p.u., Current, Imaginary Part"
      m_i_busac = "m:i:busac", "p.u., Current, Magnitude"
      m_i_busdc = "m:i:busdc", "p.u., Current, Magnitude"
      m_phiui_busac = "m:phiui:busac", "deg, Angle between Voltage and Current"
      m_phiui_busdc = "m:phiui:busdc", "deg, Angle between Voltage and Current"
      m_phiu1i1_busac = "m:phiu1i1:busac", "deg, Angle between Voltage and Current in positive sequence system"
      m_phiu1i1_busdc = "m:phiu1i1:busdc", "deg, Angle between Voltage and Current in positive sequence system"
      m_Tfct_busac = "m:Tfct:busac", "s, Fault Clearing Time"
      m_Tfct_busdc = "m:Tfct:busdc", "s, Fault Clearing Time"
      m_Brkload_busac = "m:Brkload:busac", "%, Breaker Loading"
      m_Brkload_busdc = "m:Brkload:busdc", "%, Breaker Loading"
      n_ur_busac = "n:ur:busac", "p.u., Voltage, Real Part"
      n_ur_busdc = "n:ur:busdc", "p.u., Voltage, Real Part"
      n_ui_busac = "n:ui:busac", "p.u., Voltage, Imaginary Part"
      n_ui_busdc = "n:ui:busdc", "p.u., Voltage, Imaginary Part"
      n_u_busac = "n:u:busac", "p.u., Voltage, Magnitude"
      n_u_busdc = "n:u:busdc", "p.u., Voltage, Magnitude"
      n_upc_busac = "n:upc:busac", "%, Voltage, Magnitude"
      n_upc_busdc = "n:upc:busdc", "%, Voltage, Magnitude"
      n_u1_busac = "n:u1:busac", "p.u., Voltage, Magnitude"
      n_u1_busdc = "n:u1:busdc", "p.u., Voltage, Magnitude"
      n_u1pc_busac = "n:u1pc:busac", "%, Voltage, Magnitude"
      n_u1pc_busdc = "n:u1pc:busdc", "%, Voltage, Magnitude"
      n_u1r_busac = "n:u1r:busac", "p.u., Positive-Sequence Voltage, Real Part"
      n_u1r_busdc = "n:u1r:busdc", "p.u., Positive-Sequence Voltage, Real Part"
      n_u1i_busac = "n:u1i:busac", "p.u., Positive-Sequence Voltage, Imaginary Part"
      n_u1i_busdc = "n:u1i:busdc", "p.u., Positive-Sequence Voltage, Imaginary Part"
      n_U_busac = "n:U:busac", "kV, Line-Ground Voltage, Magnitude"
      n_U_busdc = "n:U:busdc", "kV, Line-Ground Voltage, Magnitude"
      n_Ul_busac = "n:Ul:busac", "kV, Line-Line Voltage, Magnitude"
      n_Ul_busdc = "n:Ul:busdc", "kV, Line-Line Voltage, Magnitude"
      n_phiu_busac = "n:phiu:busac", "deg, Voltage, Angle"
      n_phiu_busdc = "n:phiu:busdc", "deg, Voltage, Angle"
      n_phiurel_busac = "n:phiurel:busac", "deg, Voltage, Relative Angle"
      n_phiurel_busdc = "n:phiurel:busdc", "deg, Voltage, Relative Angle"
      n_du_busac = "n:du:busac", "%, Voltage Deviation"
      n_du_busdc = "n:du:busdc", "%, Voltage Deviation"
      n_Pgen_busac = "n:Pgen:busac", "MW, Generation, Active Power"
      n_Pgen_busdc = "n:Pgen:busdc", "MW, Generation, Active Power"
      n_Qgen_busac = "n:Qgen:busac", "Mvar, Generation, Reactive Power"
      n_Qgen_busdc = "n:Qgen:busdc", "Mvar, Generation, Reactive Power"
      n_Pmot_busac = "n:Pmot:busac", "MW, Motor Load, Active Power"
      n_Pmot_busdc = "n:Pmot:busdc", "MW, Motor Load, Active Power"
      n_Qmot_busac = "n:Qmot:busac", "Mvar, Motor Load, Reactive Power"
      n_Qmot_busdc = "n:Qmot:busdc", "Mvar, Motor Load, Reactive Power"
      n_Pload_busac = "n:Pload:busac", "MW, General Load, Active Power"
      n_Pload_busdc = "n:Pload:busdc", "MW, General Load, Active Power"
      n_Qload_busac = "n:Qload:busac", "Mvar, General Load, Reactive Power"
      n_Qload_busdc = "n:Qload:busdc", "Mvar, General Load, Reactive Power"
      n_Pcomp_busac = "n:Pcomp:busac", "MW, Compensation (Losses)"
      n_Pcomp_busdc = "n:Pcomp:busdc", "MW, Compensation (Losses)"
      n_Qcomp_busac = "n:Qcomp:busac", "Mvar, Compensation"
      n_Qcomp_busdc = "n:Qcomp:busdc", "Mvar, Compensation"
      n_Pnet_busac = "n:Pnet:busac", "MW, External Networks, Active Power"
      n_Pnet_busdc = "n:Pnet:busdc", "MW, External Networks, Active Power"
      n_Qnet_busac = "n:Qnet:busac", "Mvar, External Networks, Reactive Power"
      n_Qnet_busdc = "n:Qnet:busdc", "Mvar, External Networks, Reactive Power"
      n_Pflow_busac = "n:Pflow:busac", "MW, Power Flow, Active Power"
      n_Pflow_busdc = "n:Pflow:busdc", "MW, Power Flow, Active Power"
      n_Qflow_busac = "n:Qflow:busac", "Mvar, Power Flow, Reactive Power"
      n_Qflow_busdc = "n:Qflow:busdc", "Mvar, Power Flow, Reactive Power"
      n_Pout_busac = "n:Pout:busac", "MW, Outgoing Flow, Active Power"
      n_Pout_busdc = "n:Pout:busdc", "MW, Outgoing Flow, Active Power"
      n_Qout_busac = "n:Qout:busac", "Mvar, Outgoing Power, Reactive Power"
      n_Qout_busdc = "n:Qout:busdc", "Mvar, Outgoing Power, Reactive Power"
      n_Sout_busac = "n:Sout:busac", "MVA, Outgoing Power, Apparent Power"
      n_Sout_busdc = "n:Sout:busdc", "MVA, Outgoing Power, Apparent Power"
      n_cosphiout_busac = "n:cosphiout:busac", "Outgoing Power, Power Factor"
      n_cosphiout_busdc = "n:cosphiout:busdc", "Outgoing Power, Power Factor"
      n_Pbalance_busac = "n:Pbalance:busac", "MW, Active Power Balance (=0)"
      n_Pbalance_busdc = "n:Pbalance:busdc", "MW, Active Power Balance (=0)"
      n_Qbalance_busac = "n:Qbalance:busac", "Mvar, Reactive Power Balance (=0)"
      n_Qbalance_busdc = "n:Qbalance:busdc", "Mvar, Reactive Power Balance (=0)"
      n_LossPdown_busac = "n:LossPdown:busac", "MW, Losses, downstream"
      n_LossPdown_busdc = "n:LossPdown:busdc", "MW, Losses, downstream"
      n_LossQdown_busac = "n:LossQdown:busac", "Mvar, Losses, downstream (Reactive Power)"
      n_LossQdown_busdc = "n:LossQdown:busdc", "Mvar, Losses, downstream (Reactive Power)"
      n_LossPdownload_busac = "n:LossPdownload:busac", "MW, Load losses, downstream"
      n_LossPdownload_busdc = "n:LossPdownload:busdc", "MW, Load losses, downstream"
      n_LossQdownload_busac = "n:LossQdownload:busac", "Mvar, Load losses, downstream"
      n_LossQdownload_busdc = "n:LossQdownload:busdc", "Mvar, Load losses, downstream"
      n_LossPdownnoload_busac = "n:LossPdownnoload:busac", "MW, No load losses, downstream"
      n_LossPdownnoload_busdc = "n:LossPdownnoload:busdc", "MW, No load losses, downstream"
      n_LossQdownnoload_busac = "n:LossQdownnoload:busac", "Mvar, No load losses, downstream"
      n_LossQdownnoload_busdc = "n:LossQdownnoload:busdc", "Mvar, No load losses, downstream"
      n_du_feed_busac = "n:du_feed:busac", "%, Voltage difference relative to feeder begin"
      n_du_feed_busdc = "n:du_feed:busdc", "%, Voltage difference relative to feeder begin"
      s_pset = "s:pset", "MW, IN, Active power setpoint, continuous, double"
      s_qset = "s:qset", "Mvar, IN, Reactive power setpoint continuous, double"
      s_udcset = "s:udcset", "p.u., IN, DC Voltage setpoint, continuous, double"
      s_dFin = "s:dFin", "Hz, IN, Frequency Deviation, continuous, double"
      c_phiu = "c:phiu", "rad, Voltage angle"
      c_uini = "c:uini", "p.u., Voltage setpoint"
      c_snomn = "c:snomn", "MVA, Total rated apparent power (nparnum*Snom)"
      c_uinidc = "c:uinidc", "p.u., Voltage setpoint (DC)"
      c_pmsetp = "c:pmsetp", "PWM factor"
      c_i_acdc = "c:i_acdc", "Control mode"
      c_i_mod = "c:i_mod", "Modulation"
      c_K0 = "c:K0", "Vac/Vdc at PWM=1"
      c_Pm = "c:Pm", "Pulse-modulation factor"
      c_uDC = "c:uDC", "p.u., DC-voltage"
      c_pini = "c:pini", "Active power setpoint"
      c_qini = "c:qini", "Reactive power setpoint"
      c_DCPoleLosses = "c:DCPoleLosses", "MW, DC pole losses"
      c_Qmax = "c:Qmax", "Mvar, Upper limit of reactive power"
      c_Qmin = "c:Qmin", "Mvar, Lower limit of reactive power"
      c_Pmin = "c:Pmin", "MW, Lower limit of active power"
      c_Pmax = "c:Pmax", "MW, Upper limit of active power"
      c_i_qlim = "c:i_qlim", "Reactive power limited"
      c_i_plim = "c:i_plim", "Active power limited"
      c_Kpf = "c:Kpf", "MW/Hz, Frequency bias"
      c_Kp = "c:Kp", "p.u., Active power participation"
      c_zonewscale = "c:zonewscale", "Wind generation zone scaling factor"
      c_iRefElement = "c:iRefElement", "Reference element"
      c_Kq = "c:Kq", "p.u., Reactive power participation"
      c_iPowerOn = "c:iPowerOn", "Flag"
      c_i_vlim = "c:i_vlim", "Voltage setpoint limit reached"
      c_dphi = "c:dphi", "deg, Measured angle difference"
      c_Pmeas = "c:Pmeas", "MW, Measured active power"
      c_nCtrlIter = "c:nCtrlIter", "No. of CTRL-iterations"
      c_Ploss = "c:Ploss", "MW, Losses (total)"
      c_Qloss = "c:Qloss", "Mvar, Reactive-Losses (total)"
      c_Plossld = "c:Plossld", "MW, Losses (load)"
      c_Qlossld = "c:Qlossld", "Mvar, Reactive-Losses (load)"
      c_Plossnld = "c:Plossnld", "MW, Losses (no load)"
      c_Qlossnld = "c:Qlossnld", "Mvar, Reactive-Losses (no load)"
      e_i_acdc = "e:i_acdc", "Control mode"
      e_ivacdroop = "e:ivacdroop", "AC-Voltage Droop"
      e_usetp = "e:usetp", "p.u., AC voltage setpoint"
      e_pmsetp = "e:pmsetp", "p.u., PWM factor"
      e_p_uctrl = "e:p_uctrl", "ElmTerm*,StaBar*, Controlled node (AC)"
      e_usetpdc = "e:usetpdc", "p.u., DC voltage setpoint"
      e_p_uctrldc = "e:p_uctrldc", "ElmTerm*,StaBar*, Controlled node (DC)"
      e_phisetp = "e:phisetp", "deg, Phase setpoint"
      e_psetp = "e:psetp", "MW, Active power setpoint"
      e_Kpf = "e:Kpf", "MW/Hz, Prim. frequency bias"
      e_qsetp = "e:qsetp", "Mvar, Reactive power setpoint"
      e_iQorient = "e:iQorient", "Orientation"
      e_pfsetp = "e:pfsetp", "Power factor"
      e_pf_recap = "e:pf_recap", "Power factor"
      e_p_pctrl = "e:p_pctrl", "StaCubic,ElmBoundary, Controlled flow"
      e_c_pstac = "e:c_pstac", "ElmStactrl, External station controller"
      e_c_psecc = "e:c_psecc", "ElmSecctrl, Ext. secondary controller"
      e_iVacMax = "e:iVacMax", "Consider modulation index limit"
      e_Pmmax = "e:Pmmax", "Max. PWM factor"
      e_Pnold = "e:Pnold", "kW, No-load losses"
      e_swtLossFactor = "e:swtLossFactor", "kW/A, Switching loss factor"
      e_resLossFactor = "e:resLossFactor", "Ohm, Resistive loss factor"
      e_ddroop = "e:ddroop", "%, Droop"
      e_usp_min = "e:usp_min", "p.u., Min. voltage setpoint"
      e_usp_max = "e:usp_max", "p.u., Max. voltage setpoint"
      e_idroop = "e:idroop", "DC-voltage dependent P-droop"
      e_K = "e:K", "p.u./MW, K"
      e_iPphidrp = "e:iPphidrp", "Angle difference dependent P-droop"
      e_Kpphi = "e:Kpphi", "MW/deg, Kpphi"
      e_p_b1phiu = "e:p_b1phiu", "ElmTerm*, Remote AC busbar"
      e_p_b2phiu = "e:p_b2phiu", "ElmTerm*, Local AC busbar"
      e_considerPset = "e:considerPset", "Consider active power setpoint"
      e_iPpart = "e:iPpart", "Active power participation"
      e_Kpart = "e:Kpart", "Participation factor"
      e_p_pmeas = "e:p_pmeas", "StaCubic*,ElmBoundary, P(AC) measured at"
      e_pQlimType = "e:pQlimType", "IntQlim, Capability curve"
      e_q_min = "e:q_min", "p.u., Min."
      e_q_max = "e:q_max", "p.u., Max."
      e_cQ_min = "e:cQ_min", "Mvar, Min."
      e_cQ_max = "e:cQ_max", "Mvar, Max."
      e_scaleQmin = "e:scaleQmin", "%, Scaling factor (min.)"
      e_scaleQmax = "e:scaleQmax", "%, Scaling factor (max.)"
      e_Pmin_uc = "e:Pmin_uc", "MW, Min."
      e_Pmax_uc = "e:Pmax_uc", "MW, Max."
      e_Pnom = "e:Pnom", "MW, Pr(rated)"
      e_Qmin_a = "e:Qmin_a", "Mvar, Qmin(act.)"
      e_Qmax_a = "e:Qmax_a", "Mvar, Qmax(act.)"
      e_P_max = "e:P_max", "MW, Max."
      e_r2 = "e:r2", "p.u., Resistance, r2"
      e_x2 = "e:x2", "p.u., Reactance, x2"
      e_priority = "e:priority", "Merit Order"
      e_cpFeed = "e:cpFeed", "Feeder"
      e_ciDist = "e:ciDist", "Distance from infeed in number of buses"
      e_ciLater = "e:ciLater", "Lateral Index"
      e_ciDistRoot = "e:ciDistRoot", "Distance from first infeed in number of buses"
      e_ciDistAll = "e:ciDistAll", "Distance from infeed in number of buses including switches"
      e_ciDistAllRoot = "e:ciDistAllRoot", "Distance from first infeed in number of buses including switches"
      r_p_uctrl = "r:p_uctrl", "ElmTerm*,StaBar*, Controlled node (AC)"
      r_p_uctrldc = "r:p_uctrldc", "ElmTerm*,StaBar*, Controlled node (DC)"
      r_p_pctrl = "r:p_pctrl", "StaCubic,ElmBoundary, Controlled flow"
      r_c_pstac = "r:c_pstac", "ElmStactrl, External station controller"
      r_c_psecc = "r:c_psecc", "ElmSecctrl, Ext. secondary controller"
      r_p_b1phiu = "r:p_b1phiu", "ElmTerm*, Remote AC busbar"
      r_p_b2phiu = "r:p_b2phiu", "ElmTerm*, Local AC busbar"
      r_p_pmeas = "r:p_pmeas", "StaCubic*,ElmBoundary, P(AC) measured at"
      r_pQlimType = "r:pQlimType", "IntQlim, Capability curve"
      r_cpFeed = "r:cpFeed", "Feeder"

    class ElmXnet(Enum):
      _init_ = 'value __doc__'

      m_u1r_bus1 = "m:u1r:bus1", "p.u., Positive-Sequence-Voltage, Real Part"
      m_u1i_bus1 = "m:u1i:bus1", "p.u., Positive-Sequence-Voltage, Imaginary Part"
      m_u1_bus1 = "m:u1:bus1", "p.u., Positive-Sequence-Voltage, Magnitude"
      m_U1_bus1 = "m:U1:bus1", "kV, Line-Ground Positive-Sequence-Voltage, Magnitude"
      m_U1l_bus1 = "m:U1l:bus1", "kV, Line-Line Positive-Sequence-Voltage, Magnitude"
      m_phiu1_bus1 = "m:phiu1:bus1", "deg, Positive-Sequence-Voltage, Angle"
      m_inet_bus1 = "m:inet:bus1", "p.u., Current, Magnitude, referred to network"
      m_I_bus1 = "m:I:bus1", "kA, Current, Magnitude"
      m_phii_bus1 = "m:phii:bus1", "deg, Current, Angle"
      m_i1r_bus1 = "m:i1r:bus1", "p.u., Positive-Sequence Current, Real Part"
      m_i1i_bus1 = "m:i1i:bus1", "p.u., Positive-Sequence Current, Imaginary Part"
      m_i1_bus1 = "m:i1:bus1", "p.u., Positive-Sequence Current, Magnitude"
      m_I1_bus1 = "m:I1:bus1", "kA, Positive-Sequence Current, Magnitude"
      m_phii1_bus1 = "m:phii1:bus1", "deg, Positive-Sequence Current, Angle"
      m_P_bus1 = "m:P:bus1", "MW, Active Power"
      m_Q_bus1 = "m:Q:bus1", "Mvar, Reactive Power"
      m_S_bus1 = "m:S:bus1", "MVA, Apparent Power"
      m_cosphi_bus1 = "m:cosphi:bus1", "Power Factor"
      m_tanphi_bus1 = "m:tanphi:bus1", "tan(phi)"
      m_Psum_bus1 = "m:Psum:bus1", "MW, Total Active Power"
      m_Qsum_bus1 = "m:Qsum:bus1", "Mvar, Total Reactive Power"
      m_Ssum_bus1 = "m:Ssum:bus1", "MVA, Total Apparent Power"
      m_cosphisum_bus1 = "m:cosphisum:bus1", "Total Power Factor"
      m_tanphisum_bus1 = "m:tanphisum:bus1", "Total tan(phi)"
      m_Spu_bus1 = "m:Spu:bus1", "MVA/p.u., Apparent Power per p.u. Voltage"
      m_i1P_bus1 = "m:i1P:bus1", "p.u., Positive-Sequence Active Current"
      m_i1Q_bus1 = "m:i1Q:bus1", "p.u., Positive-Sequence Reactive Current"
      m_I1P_bus1 = "m:I1P:bus1", "kA, Positive-Sequence Active Current"
      m_I1Q_bus1 = "m:I1Q:bus1", "kA, Positive-Sequence Reactive Current"
      m_ur_bus1 = "m:ur:bus1", "p.u., Voltage, Real Part"
      m_ui_bus1 = "m:ui:bus1", "p.u., Voltage, Imaginary Part"
      m_u_bus1 = "m:u:bus1", "p.u., Voltage, Magnitude"
      m_ir_bus1 = "m:ir:bus1", "p.u., Current, Real Part"
      m_ii_bus1 = "m:ii:bus1", "p.u., Current, Imaginary Part"
      m_i_bus1 = "m:i:bus1", "p.u., Current, Magnitude"
      m_phiui_bus1 = "m:phiui:bus1", "deg, Angle between Voltage and Current"
      m_phiu1i1_bus1 = "m:phiu1i1:bus1", "deg, Angle between Voltage and Current in positive sequence system"
      m_Tfct_bus1 = "m:Tfct:bus1", "s, Fault Clearing Time"
      m_Brkload_bus1 = "m:Brkload:bus1", "%, Breaker Loading"
      n_ur_bus1 = "n:ur:bus1", "p.u., Voltage, Real Part"
      n_ui_bus1 = "n:ui:bus1", "p.u., Voltage, Imaginary Part"
      n_u_bus1 = "n:u:bus1", "p.u., Voltage, Magnitude"
      n_upc_bus1 = "n:upc:bus1", "%, Voltage, Magnitude"
      n_u1_bus1 = "n:u1:bus1", "p.u., Voltage, Magnitude"
      n_u1pc_bus1 = "n:u1pc:bus1", "%, Voltage, Magnitude"
      n_u1r_bus1 = "n:u1r:bus1", "p.u., Positive-Sequence Voltage, Real Part"
      n_u1i_bus1 = "n:u1i:bus1", "p.u., Positive-Sequence Voltage, Imaginary Part"
      n_U_bus1 = "n:U:bus1", "kV, Line-Ground Voltage, Magnitude"
      n_Ul_bus1 = "n:Ul:bus1", "kV, Line-Line Voltage, Magnitude"
      n_phiu_bus1 = "n:phiu:bus1", "deg, Voltage, Angle"
      n_phiurel_bus1 = "n:phiurel:bus1", "deg, Voltage, Relative Angle"
      n_du_bus1 = "n:du:bus1", "%, Voltage Deviation"
      n_Pgen_bus1 = "n:Pgen:bus1", "MW, Generation, Active Power"
      n_Qgen_bus1 = "n:Qgen:bus1", "Mvar, Generation, Reactive Power"
      n_Pmot_bus1 = "n:Pmot:bus1", "MW, Motor Load, Active Power"
      n_Qmot_bus1 = "n:Qmot:bus1", "Mvar, Motor Load, Reactive Power"
      n_Pload_bus1 = "n:Pload:bus1", "MW, General Load, Active Power"
      n_Qload_bus1 = "n:Qload:bus1", "Mvar, General Load, Reactive Power"
      n_Pcomp_bus1 = "n:Pcomp:bus1", "MW, Compensation (Losses)"
      n_Qcomp_bus1 = "n:Qcomp:bus1", "Mvar, Compensation"
      n_Pnet_bus1 = "n:Pnet:bus1", "MW, External Networks, Active Power"
      n_Qnet_bus1 = "n:Qnet:bus1", "Mvar, External Networks, Reactive Power"
      n_Pflow_bus1 = "n:Pflow:bus1", "MW, Power Flow, Active Power"
      n_Qflow_bus1 = "n:Qflow:bus1", "Mvar, Power Flow, Reactive Power"
      n_Pout_bus1 = "n:Pout:bus1", "MW, Outgoing Flow, Active Power"
      n_Qout_bus1 = "n:Qout:bus1", "Mvar, Outgoing Power, Reactive Power"
      n_Sout_bus1 = "n:Sout:bus1", "MVA, Outgoing Power, Apparent Power"
      n_cosphiout_bus1 = "n:cosphiout:bus1", "Outgoing Power, Power Factor"
      n_Pbalance_bus1 = "n:Pbalance:bus1", "MW, Active Power Balance (=0)"
      n_Qbalance_bus1 = "n:Qbalance:bus1", "Mvar, Reactive Power Balance (=0)"
      n_LossPdown_bus1 = "n:LossPdown:bus1", "MW, Losses, downstream"
      n_LossQdown_bus1 = "n:LossQdown:bus1", "Mvar, Losses, downstream (Reactive Power)"
      n_LossPdownload_bus1 = "n:LossPdownload:bus1", "MW, Load losses, downstream"
      n_LossQdownload_bus1 = "n:LossQdownload:bus1", "Mvar, Load losses, downstream"
      n_LossPdownnoload_bus1 = "n:LossPdownnoload:bus1", "MW, No load losses, downstream"
      n_LossQdownnoload_bus1 = "n:LossQdownnoload:bus1", "Mvar, No load losses, downstream"
      n_du_feed_bus1 = "n:du_feed:bus1", "%, Voltage difference relative to feeder begin"
      s_pset = "s:pset", "MW, IN, Active Power Setpoint, continuous, double"
      s_qset = "s:qset", "Mvar, IN, Reactive Power Setpoint continuous, double"
      s_dFin = "s:dFin", "Hz, IN, Frequency Deviation, continuous, double"
      s_dF = "s:dF", "Hz, OUT, Frequency Deviation Output, continuous, double"
      c_phiini = "c:phiini", "Initial Angle"
      c_uini_r = "c:uini:r", "p.u., Voltage Setpoint for PV and Slack, Real Part"
      c_uini_i = "c:uini:i", "p.u., Voltage Setpoint for PV and Slack, Imaginary Part"
      c_uini_phi = "c:uini:phi", "deg, Voltage Setpoint for PV and Slack, Angle"
      c_uini_mag = "c:uini:mag", "p.u., Voltage Setpoint for PV and Slack, Magnitude"
      c_uini_phirad = "c:uini:phirad", "rad, Voltage Setpoint for PV and Slack, Angle"
      c_p = "c:p", "MW, Total Active Power"
      c_q = "c:q", "Mvar, Total Reactive Power"
      c_sgnn = "c:sgnn", "MVA, Total rated apparent power (ngnum*sgn)"
      c_cosn = "c:cosn", "Nominal-Power factor"
      c_c2 = "c:c2", "Voltage Coefficient (uknom/ugn)"
      c_iPowerOn = "c:iPowerOn", "Flag"
      c_Pdisp = "c:Pdisp", "MW, Power (Eco-Dispatch)"
      c_xdq = "c:xdq", "p.u., Synchronous Reactance"
      c_rs = "c:rs", "p.u., Resistance"
      c_i_qlim = "c:i_qlim", "Reactive power limit reached"
      c_Qmin = "c:Qmin", "Mvar, Lower limit of reactive power"
      c_Qmax = "c:Qmax", "Mvar, Upper limit of reactive power"
      c_sm_r = "c:sm:r", "MW, Complex Power, Real Part"
      c_sm_i = "c:sm:i", "Mvar, Complex Power, Imaginary Part"
      c_sm_phi = "c:sm:phi", "deg, Complex Power, Angle"
      c_sm_mag = "c:sm:mag", "MVA, Complex Power, Magnitude"
      c_sm_phirad = "c:sm:phirad", "rad, Complex Power, Angle"
      c_Kf = "c:Kf", "MW/Hz, Frequency Bias"
      c_Kp = "c:Kp", "p.u., Active Power Participation"
      c_iRefElement = "c:iRefElement", "Reference Element"
      c_nCtrlIter = "c:nCtrlIter", "No. of CTRL-iterations"
      c_Ploss = "c:Ploss", "MW, Losses (total)"
      c_Qloss = "c:Qloss", "Mvar, Reactive-Losses (total)"
      c_Plossld = "c:Plossld", "MW, Losses (load)"
      c_Qlossld = "c:Qlossld", "Mvar, Reactive-Losses (load)"
      c_Plossnld = "c:Plossnld", "MW, Losses (no load)"
      c_Qlossnld = "c:Qlossnld", "Mvar, Reactive-Losses (no load)"
      e_c_pstac = "e:c_pstac", "ElmStactrl, External Station Controller"
      e_c_psecc = "e:c_psecc", "ElmSecctrl, External Secondary Controller"
      e_bustp = "e:bustp", "Bus Type"
      e_uset_mode = "e:uset_mode", "Setpoint"
      e_outServPzero = "e:outServPzero", "Out of service when active power is zero"
      e_mode_inp = "e:mode_inp", "Input Mode"
      e_pgini = "e:pgini", "MW, Active Power"
      e_qgini = "e:qgini", "Mvar, Reactive Power"
      e_sgini = "e:sgini", "MVA, Apparent Power"
      e_cosgini = "e:cosgini", "Power Factor"
      e_pf_recap = "e:pf_recap", "Power Factor"
      e_p_direc = "e:p_direc", "Power Direction"
      e_usetp = "e:usetp", "p.u., Voltage Setpoint"
      e_phiini = "e:phiini", "deg, Angle"
      e_p_uctrl = "e:p_uctrl", "ElmTerm*,StaBar*,ElmLdfctrl*, Reference Busbar"
      e_cpCtrlNode = "e:cpCtrlNode", "Target Node"
      e_Kpf = "e:Kpf", "MW/Hz, Primary Frequency Bias"
      e_K = "e:K", "MW/Hz, Secondary Frequency Bias"
      e_pQlimType = "e:pQlimType", "IntQlim, Capability Curve"
      e_cQ_min = "e:cQ_min", "Mvar, Min."
      e_cQ_max = "e:cQ_max", "Mvar, Max."
      e_scaleQmin = "e:scaleQmin", "%, Scaling Factor (min.)"
      e_scaleQmax = "e:scaleQmax", "%, Scaling Factor (max.)"
      e_qdslCtrl = "e:qdslCtrl", "Quasi-Dynamic Model"
      e_cused = "e:cused", "Use for calculation"
      e_snss = "e:snss", "MVA, Short-Circuit Power Sk''max"
      e_ikss = "e:ikss", "kA, Short-Circuit Current Ik''max"
      e_cmax = "e:cmax", "c-Factor (max.)"
      e_rntxn = "e:rntxn", "R/X Ratio (max.)"
      e_xntrn = "e:xntrn", "X/R Ratio (max.)"
      e_z2tz1 = "e:z2tz1", "Z2/Z1 max."
      e_x0tx1 = "e:x0tx1", "X0/X1 max."
      e_r0tx0 = "e:r0tx0", "R0/X0 max."
      e_snssmin = "e:snssmin", "MVA, Short-Circuit Power Sk''min"
      e_ikssmin = "e:ikssmin", "kA, Short-Circuit Current Ik''min"
      e_cmin = "e:cmin", "c-Factor (min.)"
      e_rntxnmin = "e:rntxnmin", "R/X Ratio (min.)"
      e_xntrnmin = "e:xntrnmin", "X/R Ratio (min.)"
      e_z2tz1min = "e:z2tz1min", "Z2/Z1 min."
      e_x0tx1min = "e:x0tx1min", "X0/X1 min."
      e_r0tx0min = "e:r0tx0min", "R0/X0 min."
      e_is4SpinReserve = "e:is4SpinReserve", "Consider for region spinning reserve"
      e_genShiftKey = "e:genShiftKey", "%, Generation shift key"
      e_pBMU = "e:pBMU", "ElmBmu, Virtual Power Plant"
      e_dispatch = "e:dispatch", "Generator Dispatch"
      e_Pmin_uc = "e:Pmin_uc", "MW, Min."
      e_Pmin_ucPU = "e:Pmin_ucPU", "p.u., Min."
      e_MaxS = "e:MaxS", "MW, Max."
      e_Pmax_ucPU = "e:Pmax_ucPU", "p.u., Max."
      e_priority = "e:priority", "Merit Order"
      e_fixed = "e:fixed", "Must run"
      e_cpFeed = "e:cpFeed", "Feeder"
      e_ciDist = "e:ciDist", "Distance from infeed in number of buses"
      e_ciLater = "e:ciLater", "Lateral Index"
      e_ciDistRoot = "e:ciDistRoot", "Distance from first infeed in number of buses"
      e_ciDistAll = "e:ciDistAll", "Distance from infeed in number of buses including switches"
      e_ciDistAllRoot = "e:ciDistAllRoot", "Distance from first infeed in number of buses including switches"
      r_c_pstac = "r:c_pstac", "ElmStactrl, External Station Controller"
      r_c_psecc = "r:c_psecc", "ElmSecctrl, External Secondary Controller"
      r_p_uctrl = "r:p_uctrl", "ElmTerm*,StaBar*,ElmLdfctrl*, Reference Busbar"
      r_cpCtrlNode = "r:cpCtrlNode", "Target Node"
      r_pQlimType = "r:pQlimType", "IntQlim, Capability Curve"
      r_qdslCtrl = "r:qdslCtrl", "Quasi-Dynamic Model"
      r_pBMU = "r:pBMU", "ElmBmu, Virtual Power Plant"
      r_cpFeed = "r:cpFeed", "Feeder"

    class ElmZone(Enum):
      _init_ = 'value __doc__'

      c_n_ele = "c:n_ele", "Number of Elements"
      c_n_zones = "c:n_zones", "Number of Connected Zones"
      c_n_volt = "c:n_volt", "Number of Voltage Levels"
      c_n_bar = "c:n_bar", "No. of Busbars"
      c_n_term = "c:n_term", "No. of Terminals"
      c_n_lne = "c:n_lne", "No. of Lines"
      c_n_sym = "c:n_sym", "No. of syn. Machines"
      c_n_asm = "c:n_asm", "No. of asyn. Machines"
      c_n_stg = "c:n_stg", "No. of Static Generators"
      c_n_pvs = "c:n_pvs", "No. of PV Systems"
      c_n_lod = "c:n_lod", "No. of Loads"
      c_n_cust = "c:n_cust", "No. of Customers"
      c_TotgenP = "c:TotgenP", "MW, Total Generation, Active Power"
      c_TotgenQ = "c:TotgenQ", "Mvar, Total Generation, Reactive Power"
      c_TotconsP = "c:TotconsP", "MW, Total Consumption, Active Power"
      c_TotconsQ = "c:TotconsQ", "Mvar, Total Consumption, Reactive Power"
      c_GenP = "c:GenP", "MW, Generators, Active Power"
      c_GenQ = "c:GenQ", "Mvar, Generators, Reactive Power"
      c_GenS = "c:GenS", "MVA, Generators, Apparent Power"
      c_Gencos = "c:Gencos", "Generators, Power Factor"
      c_GenPnom = "c:GenPnom", "MW, Generators, Nominal Active Power"
      c_GenQnom = "c:GenQnom", "Mvar, Generators, Nominal Reactive Power"
      c_GenSnom = "c:GenSnom", "MVA, Generators, Nominal Apparent Power"
      c_GenPdiff = "c:GenPdiff", "MW, Generators, difference between nominal and actual active power"
      c_GenQdiff = "c:GenQdiff", "Mvar, Generators, difference between nominal and actual reactive power"
      c_Gencosnom = "c:Gencosnom", "Generators, Nominal Power Factor"
      c_GenPdisp = "c:GenPdisp", "MW, Generators, Dispatch Active Power"
      c_GenPmism = "c:GenPmism", "MW, Generators, difference between actual and dispatched active power"
      c_GenPspinres = "c:GenPspinres", "MW, Generators, difference between maximum and actual active power"
      c_GenQspinres = "c:GenQspinres", "Mvar, Generators, difference between maximum and actual reactive power"
      c_MotP = "c:MotP", "MW, Motor Loads, Active Power"
      c_MotQ = "c:MotQ", "Mvar, Motor Loads, Reactive Power"
      c_MotS = "c:MotS", "MVA, Motor Loads, Apparent Power"
      c_Motcos = "c:Motcos", "Motor Loads, Power Factor"
      c_LoadP = "c:LoadP", "MW, Loads, Active Power"
      c_LoadQ = "c:LoadQ", "Mvar, Loads, Reactive Power"
      c_LoadS = "c:LoadS", "MVA, Loads, Apparent Power"
      c_Loadcos = "c:Loadcos", "Loads, Power Factor"
      c_LoadPnom = "c:LoadPnom", "MW, Loads, Nominal Active Power"
      c_LoadQnom = "c:LoadQnom", "Mvar, Loads, Nominal Reactive Power"
      c_LoadSnom = "c:LoadSnom", "MVA, Loads, Nominal Apparent Power"
      c_Loadcosnom = "c:Loadcosnom", "Loads, Nominal Power Factor"
      c_LoadPdiff = "c:LoadPdiff", "MW, Loads, difference between nominal and actual active power"
      c_LoadQdiff = "c:LoadQdiff", "Mvar, Loads, difference between nominal and actual reactive power"
      c_LoadPdem = "c:LoadPdem", "MW, Loads, Active Power Demand"
      c_LoadPmism = "c:LoadPmism", "MW, Loads, difference between actual active power and demand"
      c_CompP = "c:CompP", "MW, Compensation (Losses)"
      c_CompQ = "c:CompQ", "Mvar, Compensation, C"
      c_ComprQ = "c:ComprQ", "Mvar, Compensation, L"
      c_NetP = "c:NetP", "MW, External Networks, Active Power"
      c_NetQ = "c:NetQ", "Mvar, External Networks, Reactive Power"
      c_NetS = "c:NetS", "MVA, External Networks, Apparent Power"
      c_Netcos = "c:Netcos", "External Networks, Power Factor"
      c_InterP = "c:InterP", "MW, Interchange Flow, Active Power"
      c_InterQ = "c:InterQ", "Mvar, Interchange Flow, Reactive Power"
      c_LossP = "c:LossP", "MW, Losses, Active Power"
      c_LossQ = "c:LossQ", "Mvar, Losses, Reactive Power"
      c_LossPld = "c:LossPld", "MW, Losses, Active Power (load)"
      c_LossQld = "c:LossQld", "Mvar, Losses, Reactive Power (load)"
      c_LossPnld = "c:LossPnld", "MW, Losses, Active Power (no load)"
      c_LossQnld = "c:LossQnld", "Mvar, Losses, Reactive Power (no load)"
      c_LossPlne = "c:LossPlne", "MW, Line Losses, Active Power"
      c_LossQlne = "c:LossQlne", "Mvar, Line Losses, Reactive Power"
      c_LossPldlne = "c:LossPldlne", "MW, Line Losses, Active Power (load)"
      c_LossQldlne = "c:LossQldlne", "Mvar, Line Losses, Reactive Power (load)"
      c_LossPnldlne = "c:LossPnldlne", "MW, Line Losses, Active Power (no load)"
      c_LossQnldlne = "c:LossQnldlne", "Mvar, Line Losses, Reactive Power (no load)"
      c_LossPtrf = "c:LossPtrf", "MW, Transformer Losses, Active Power"
      c_LossQtrf = "c:LossQtrf", "Mvar, Transformer Losses, Reactive Power"
      c_LossPldtrf = "c:LossPldtrf", "MW, Transformer Losses, Active Power (load)"
      c_LossQldtrf = "c:LossQldtrf", "Mvar, Transformer Losses, Reactive Power (load)"
      c_LossPnldtrf = "c:LossPnldtrf", "MW, Transformer Losses, Active Power (no load)"
      c_LossQnldtrf = "c:LossQnldtrf", "Mvar, Transformer Losses, Reactive Power (no load)"
      c_TotLoadP = "c:TotLoadP", "MW, Total Load, Active Power"
      c_TotLoadQ = "c:TotLoadQ", "Mvar, Total Load, Reactive Power"
      c_TotLoadS = "c:TotLoadS", "MVA, Total Load, Apparent Power"
      c_TotLoadcos = "c:TotLoadcos", "Total Load, Power Factor"
      c_TotP = "c:TotP", "MW, Total Demand"
      c_UnsupP = "c:UnsupP", "MW, Unsupplied Demand"
      c_SupP = "c:SupP", "MW, Supplied Demand"
      c_ExportP = "c:ExportP", "MW, Export, Active Power"
      c_ExportQ = "c:ExportQ", "Mvar, Export, Reactive Power"
      c_ImportP = "c:ImportP", "MW, Import, Active Power"
      c_ImportQ = "c:ImportQ", "Mvar, Import, Reactive Power"
      c_cst_disp = "c:cst_disp", "USD/h, Total Production Costs"
      c_NrOvlBranch = "c:NrOvlBranch", "Number of overloaded branch components"
      c_NrGenQMax = "c:NrGenQMax", "Number of generators above their maximum reactive power limit"
      c_NrGenQMin = "c:NrGenQMin", "Number of generators below their minimum reactive power limit"
      c_NrTermMin = "c:NrTermMin", "Number of terminals below their minimum voltage limit"
      c_NrTermMax = "c:NrTermMax", "Number of terminals above their maximum voltage limit"
      c_RUC_Lead = "c:RUC_Lead", "Mvar, Used Reactive Capacity Lead"
      c_RUC_Lag = "c:RUC_Lag", "Mvar, Used Reactive Capacity Lag"
      c_RSC_Lead = "c:RSC_Lead", "Mvar, Spare Reactive Capacity Lead"
      c_RSC_Lag = "c:RSC_Lag", "Mvar, Spare Reactive Capacity Lag"
      c_Q_shgain = "c:Q_shgain", "Mvar, Reactive Power Shunt Gain"
      c_ConnMvar_max = "c:ConnMvar_max", "Mvar, Connected MVAR Capacity (max)"
      c_ConnMvar_min = "c:ConnMvar_min", "Mvar, Connected MVAR Capacity (min)"
      c_Maxloading = "c:Maxloading", "%, Maximum Loading"
      c_umin = "c:umin", "p.u., Minimum voltage of all terminals"
      c_umax = "c:umax", "p.u., Maximum voltage of all terminals"
      e_iInterChg = "e:iInterChg", "Consider Interchange Schedule"
      e_InterPset = "e:InterPset", "MW, Scheduled Active Power Interchange"

    class ElmZpu(Enum):
      _init_ = 'value __doc__'

      m_u1r_bus1 = "m:u1r:bus1", "p.u., Positive-Sequence-Voltage, Real Part"
      m_u1r_bus2 = "m:u1r:bus2", "p.u., Positive-Sequence-Voltage, Real Part"
      m_u1i_bus1 = "m:u1i:bus1", "p.u., Positive-Sequence-Voltage, Imaginary Part"
      m_u1i_bus2 = "m:u1i:bus2", "p.u., Positive-Sequence-Voltage, Imaginary Part"
      m_u1_bus1 = "m:u1:bus1", "p.u., Positive-Sequence-Voltage, Magnitude"
      m_u1_bus2 = "m:u1:bus2", "p.u., Positive-Sequence-Voltage, Magnitude"
      m_U1_bus1 = "m:U1:bus1", "kV, Line-Ground Positive-Sequence-Voltage, Magnitude"
      m_U1_bus2 = "m:U1:bus2", "kV, Line-Ground Positive-Sequence-Voltage, Magnitude"
      m_U1l_bus1 = "m:U1l:bus1", "kV, Line-Line Positive-Sequence-Voltage, Magnitude"
      m_U1l_bus2 = "m:U1l:bus2", "kV, Line-Line Positive-Sequence-Voltage, Magnitude"
      m_phiu1_bus1 = "m:phiu1:bus1", "deg, Positive-Sequence-Voltage, Angle"
      m_phiu1_bus2 = "m:phiu1:bus2", "deg, Positive-Sequence-Voltage, Angle"
      m_inet_bus1 = "m:inet:bus1", "p.u., Current, Magnitude, referred to network"
      m_inet_bus2 = "m:inet:bus2", "p.u., Current, Magnitude, referred to network"
      m_I_bus1 = "m:I:bus1", "kA, Current, Magnitude"
      m_I_bus2 = "m:I:bus2", "kA, Current, Magnitude"
      m_phii_bus1 = "m:phii:bus1", "deg, Current, Angle"
      m_phii_bus2 = "m:phii:bus2", "deg, Current, Angle"
      m_i1r_bus1 = "m:i1r:bus1", "p.u., Positive-Sequence Current, Real Part"
      m_i1r_bus2 = "m:i1r:bus2", "p.u., Positive-Sequence Current, Real Part"
      m_i1i_bus1 = "m:i1i:bus1", "p.u., Positive-Sequence Current, Imaginary Part"
      m_i1i_bus2 = "m:i1i:bus2", "p.u., Positive-Sequence Current, Imaginary Part"
      m_i1_bus1 = "m:i1:bus1", "p.u., Positive-Sequence Current, Magnitude"
      m_i1_bus2 = "m:i1:bus2", "p.u., Positive-Sequence Current, Magnitude"
      m_I1_bus1 = "m:I1:bus1", "kA, Positive-Sequence Current, Magnitude"
      m_I1_bus2 = "m:I1:bus2", "kA, Positive-Sequence Current, Magnitude"
      m_phii1_bus1 = "m:phii1:bus1", "deg, Positive-Sequence Current, Angle"
      m_phii1_bus2 = "m:phii1:bus2", "deg, Positive-Sequence Current, Angle"
      m_P_bus1 = "m:P:bus1", "MW, Active Power"
      m_P_bus2 = "m:P:bus2", "MW, Active Power"
      m_Q_bus1 = "m:Q:bus1", "Mvar, Reactive Power"
      m_Q_bus2 = "m:Q:bus2", "Mvar, Reactive Power"
      m_S_bus1 = "m:S:bus1", "MVA, Apparent Power"
      m_S_bus2 = "m:S:bus2", "MVA, Apparent Power"
      m_cosphi_bus1 = "m:cosphi:bus1", "Power Factor"
      m_cosphi_bus2 = "m:cosphi:bus2", "Power Factor"
      m_tanphi_bus1 = "m:tanphi:bus1", "tan(phi)"
      m_tanphi_bus2 = "m:tanphi:bus2", "tan(phi)"
      m_Psum_bus1 = "m:Psum:bus1", "MW, Total Active Power"
      m_Psum_bus2 = "m:Psum:bus2", "MW, Total Active Power"
      m_Qsum_bus1 = "m:Qsum:bus1", "Mvar, Total Reactive Power"
      m_Qsum_bus2 = "m:Qsum:bus2", "Mvar, Total Reactive Power"
      m_Ssum_bus1 = "m:Ssum:bus1", "MVA, Total Apparent Power"
      m_Ssum_bus2 = "m:Ssum:bus2", "MVA, Total Apparent Power"
      m_cosphisum_bus1 = "m:cosphisum:bus1", "Total Power Factor"
      m_cosphisum_bus2 = "m:cosphisum:bus2", "Total Power Factor"
      m_tanphisum_bus1 = "m:tanphisum:bus1", "Total tan(phi)"
      m_tanphisum_bus2 = "m:tanphisum:bus2", "Total tan(phi)"
      m_Spu_bus1 = "m:Spu:bus1", "MVA/p.u., Apparent Power per p.u. Voltage"
      m_Spu_bus2 = "m:Spu:bus2", "MVA/p.u., Apparent Power per p.u. Voltage"
      m_i1P_bus1 = "m:i1P:bus1", "p.u., Positive-Sequence Active Current"
      m_i1P_bus2 = "m:i1P:bus2", "p.u., Positive-Sequence Active Current"
      m_i1Q_bus1 = "m:i1Q:bus1", "p.u., Positive-Sequence Reactive Current"
      m_i1Q_bus2 = "m:i1Q:bus2", "p.u., Positive-Sequence Reactive Current"
      m_I1P_bus1 = "m:I1P:bus1", "kA, Positive-Sequence Active Current"
      m_I1P_bus2 = "m:I1P:bus2", "kA, Positive-Sequence Active Current"
      m_I1Q_bus1 = "m:I1Q:bus1", "kA, Positive-Sequence Reactive Current"
      m_I1Q_bus2 = "m:I1Q:bus2", "kA, Positive-Sequence Reactive Current"
      m_ur_bus1 = "m:ur:bus1", "p.u., Voltage, Real Part"
      m_ur_bus2 = "m:ur:bus2", "p.u., Voltage, Real Part"
      m_ui_bus1 = "m:ui:bus1", "p.u., Voltage, Imaginary Part"
      m_ui_bus2 = "m:ui:bus2", "p.u., Voltage, Imaginary Part"
      m_u_bus1 = "m:u:bus1", "p.u., Voltage, Magnitude"
      m_u_bus2 = "m:u:bus2", "p.u., Voltage, Magnitude"
      m_ir_bus1 = "m:ir:bus1", "p.u., Current, Real Part"
      m_ir_bus2 = "m:ir:bus2", "p.u., Current, Real Part"
      m_ii_bus1 = "m:ii:bus1", "p.u., Current, Imaginary Part"
      m_ii_bus2 = "m:ii:bus2", "p.u., Current, Imaginary Part"
      m_i_bus1 = "m:i:bus1", "p.u., Current, Magnitude"
      m_i_bus2 = "m:i:bus2", "p.u., Current, Magnitude"
      m_phiui_bus1 = "m:phiui:bus1", "deg, Angle between Voltage and Current"
      m_phiui_bus2 = "m:phiui:bus2", "deg, Angle between Voltage and Current"
      m_phiu1i1_bus1 = "m:phiu1i1:bus1", "deg, Angle between Voltage and Current in positive sequence system"
      m_phiu1i1_bus2 = "m:phiu1i1:bus2", "deg, Angle between Voltage and Current in positive sequence system"
      m_Tfct_bus1 = "m:Tfct:bus1", "s, Fault Clearing Time"
      m_Tfct_bus2 = "m:Tfct:bus2", "s, Fault Clearing Time"
      m_Brkload_bus1 = "m:Brkload:bus1", "%, Breaker Loading"
      m_Brkload_bus2 = "m:Brkload:bus2", "%, Breaker Loading"
      n_ur_bus1 = "n:ur:bus1", "p.u., Voltage, Real Part"
      n_ur_bus2 = "n:ur:bus2", "p.u., Voltage, Real Part"
      n_ui_bus1 = "n:ui:bus1", "p.u., Voltage, Imaginary Part"
      n_ui_bus2 = "n:ui:bus2", "p.u., Voltage, Imaginary Part"
      n_u_bus1 = "n:u:bus1", "p.u., Voltage, Magnitude"
      n_u_bus2 = "n:u:bus2", "p.u., Voltage, Magnitude"
      n_upc_bus1 = "n:upc:bus1", "%, Voltage, Magnitude"
      n_upc_bus2 = "n:upc:bus2", "%, Voltage, Magnitude"
      n_u1_bus1 = "n:u1:bus1", "p.u., Voltage, Magnitude"
      n_u1_bus2 = "n:u1:bus2", "p.u., Voltage, Magnitude"
      n_u1pc_bus1 = "n:u1pc:bus1", "%, Voltage, Magnitude"
      n_u1pc_bus2 = "n:u1pc:bus2", "%, Voltage, Magnitude"
      n_u1r_bus1 = "n:u1r:bus1", "p.u., Positive-Sequence Voltage, Real Part"
      n_u1r_bus2 = "n:u1r:bus2", "p.u., Positive-Sequence Voltage, Real Part"
      n_u1i_bus1 = "n:u1i:bus1", "p.u., Positive-Sequence Voltage, Imaginary Part"
      n_u1i_bus2 = "n:u1i:bus2", "p.u., Positive-Sequence Voltage, Imaginary Part"
      n_U_bus1 = "n:U:bus1", "kV, Line-Ground Voltage, Magnitude"
      n_U_bus2 = "n:U:bus2", "kV, Line-Ground Voltage, Magnitude"
      n_Ul_bus1 = "n:Ul:bus1", "kV, Line-Line Voltage, Magnitude"
      n_Ul_bus2 = "n:Ul:bus2", "kV, Line-Line Voltage, Magnitude"
      n_phiu_bus1 = "n:phiu:bus1", "deg, Voltage, Angle"
      n_phiu_bus2 = "n:phiu:bus2", "deg, Voltage, Angle"
      n_phiurel_bus1 = "n:phiurel:bus1", "deg, Voltage, Relative Angle"
      n_phiurel_bus2 = "n:phiurel:bus2", "deg, Voltage, Relative Angle"
      n_du_bus1 = "n:du:bus1", "%, Voltage Deviation"
      n_du_bus2 = "n:du:bus2", "%, Voltage Deviation"
      n_Pgen_bus1 = "n:Pgen:bus1", "MW, Generation, Active Power"
      n_Pgen_bus2 = "n:Pgen:bus2", "MW, Generation, Active Power"
      n_Qgen_bus1 = "n:Qgen:bus1", "Mvar, Generation, Reactive Power"
      n_Qgen_bus2 = "n:Qgen:bus2", "Mvar, Generation, Reactive Power"
      n_Pmot_bus1 = "n:Pmot:bus1", "MW, Motor Load, Active Power"
      n_Pmot_bus2 = "n:Pmot:bus2", "MW, Motor Load, Active Power"
      n_Qmot_bus1 = "n:Qmot:bus1", "Mvar, Motor Load, Reactive Power"
      n_Qmot_bus2 = "n:Qmot:bus2", "Mvar, Motor Load, Reactive Power"
      n_Pload_bus1 = "n:Pload:bus1", "MW, General Load, Active Power"
      n_Pload_bus2 = "n:Pload:bus2", "MW, General Load, Active Power"
      n_Qload_bus1 = "n:Qload:bus1", "Mvar, General Load, Reactive Power"
      n_Qload_bus2 = "n:Qload:bus2", "Mvar, General Load, Reactive Power"
      n_Pcomp_bus1 = "n:Pcomp:bus1", "MW, Compensation (Losses)"
      n_Pcomp_bus2 = "n:Pcomp:bus2", "MW, Compensation (Losses)"
      n_Qcomp_bus1 = "n:Qcomp:bus1", "Mvar, Compensation"
      n_Qcomp_bus2 = "n:Qcomp:bus2", "Mvar, Compensation"
      n_Pnet_bus1 = "n:Pnet:bus1", "MW, External Networks, Active Power"
      n_Pnet_bus2 = "n:Pnet:bus2", "MW, External Networks, Active Power"
      n_Qnet_bus1 = "n:Qnet:bus1", "Mvar, External Networks, Reactive Power"
      n_Qnet_bus2 = "n:Qnet:bus2", "Mvar, External Networks, Reactive Power"
      n_Pflow_bus1 = "n:Pflow:bus1", "MW, Power Flow, Active Power"
      n_Pflow_bus2 = "n:Pflow:bus2", "MW, Power Flow, Active Power"
      n_Qflow_bus1 = "n:Qflow:bus1", "Mvar, Power Flow, Reactive Power"
      n_Qflow_bus2 = "n:Qflow:bus2", "Mvar, Power Flow, Reactive Power"
      n_Pout_bus1 = "n:Pout:bus1", "MW, Outgoing Flow, Active Power"
      n_Pout_bus2 = "n:Pout:bus2", "MW, Outgoing Flow, Active Power"
      n_Qout_bus1 = "n:Qout:bus1", "Mvar, Outgoing Power, Reactive Power"
      n_Qout_bus2 = "n:Qout:bus2", "Mvar, Outgoing Power, Reactive Power"
      n_Sout_bus1 = "n:Sout:bus1", "MVA, Outgoing Power, Apparent Power"
      n_Sout_bus2 = "n:Sout:bus2", "MVA, Outgoing Power, Apparent Power"
      n_cosphiout_bus1 = "n:cosphiout:bus1", "Outgoing Power, Power Factor"
      n_cosphiout_bus2 = "n:cosphiout:bus2", "Outgoing Power, Power Factor"
      n_Pbalance_bus1 = "n:Pbalance:bus1", "MW, Active Power Balance (=0)"
      n_Pbalance_bus2 = "n:Pbalance:bus2", "MW, Active Power Balance (=0)"
      n_Qbalance_bus1 = "n:Qbalance:bus1", "Mvar, Reactive Power Balance (=0)"
      n_Qbalance_bus2 = "n:Qbalance:bus2", "Mvar, Reactive Power Balance (=0)"
      n_LossPdown_bus1 = "n:LossPdown:bus1", "MW, Losses, downstream"
      n_LossPdown_bus2 = "n:LossPdown:bus2", "MW, Losses, downstream"
      n_LossQdown_bus1 = "n:LossQdown:bus1", "Mvar, Losses, downstream (Reactive Power)"
      n_LossQdown_bus2 = "n:LossQdown:bus2", "Mvar, Losses, downstream (Reactive Power)"
      n_LossPdownload_bus1 = "n:LossPdownload:bus1", "MW, Load losses, downstream"
      n_LossPdownload_bus2 = "n:LossPdownload:bus2", "MW, Load losses, downstream"
      n_LossQdownload_bus1 = "n:LossQdownload:bus1", "Mvar, Load losses, downstream"
      n_LossQdownload_bus2 = "n:LossQdownload:bus2", "Mvar, Load losses, downstream"
      n_LossPdownnoload_bus1 = "n:LossPdownnoload:bus1", "MW, No load losses, downstream"
      n_LossPdownnoload_bus2 = "n:LossPdownnoload:bus2", "MW, No load losses, downstream"
      n_LossQdownnoload_bus1 = "n:LossQdownnoload:bus1", "Mvar, No load losses, downstream"
      n_LossQdownnoload_bus2 = "n:LossQdownnoload:bus2", "Mvar, No load losses, downstream"
      n_du_feed_bus1 = "n:du_feed:bus1", "%, Voltage difference relative to feeder begin"
      n_du_feed_bus2 = "n:du_feed:bus2", "%, Voltage difference relative to feeder begin"
      c_z1_ij_r = "c:z1_ij:r", "p.u., Impedance i-j, Real Part"
      c_z1_ij_i = "c:z1_ij:i", "p.u., Impedance i-j, Imaginary Part"
      c_z1_ij_phi = "c:z1_ij:phi", "deg, Impedance i-j, Angle"
      c_z1_ij_mag = "c:z1_ij:mag", "p.u., Impedance i-j, Magnitude"
      c_z1_ij_phirad = "c:z1_ij:phirad", "rad, Impedance i-j, Angle"
      c_z1_ji_r = "c:z1_ji:r", "p.u., Impedance j-i, Real Part"
      c_z1_ji_i = "c:z1_ji:i", "p.u., Impedance j-i, Imaginary Part"
      c_z1_ji_phi = "c:z1_ji:phi", "deg, Impedance j-i, Angle"
      c_z1_ji_mag = "c:z1_ji:mag", "p.u., Impedance j-i, Magnitude"
      c_z1_ji_phirad = "c:z1_ji:phirad", "rad, Impedance j-i, Angle"
      c_y1_ij_r = "c:y1_ij:r", "p.u., Admittance i-j, Real Part"
      c_y1_ij_i = "c:y1_ij:i", "p.u., Admittance i-j, Imaginary Part"
      c_y1_ij_phi = "c:y1_ij:phi", "deg, Admittance i-j, Angle"
      c_y1_ij_mag = "c:y1_ij:mag", "p.u., Admittance i-j, Magnitude"
      c_y1_ij_phirad = "c:y1_ij:phirad", "rad, Admittance i-j, Angle"
      c_y1_ji_r = "c:y1_ji:r", "p.u., Admittance j-i, Real Part"
      c_y1_ji_i = "c:y1_ji:i", "p.u., Admittance j-i, Imaginary Part"
      c_y1_ji_phi = "c:y1_ji:phi", "deg, Admittance j-i, Angle"
      c_y1_ji_mag = "c:y1_ji:mag", "p.u., Admittance j-i, Magnitude"
      c_y1_ji_phirad = "c:y1_ji:phirad", "rad, Admittance j-i, Angle"
      c_du = "c:du", "p.u., Voltage Drop"
      c_dupc = "c:dupc", "%, Voltage Drop"
      c_dphiu = "c:dphiu", "deg, Voltage Drop Angle"
      c_du1 = "c:du1", "p.u., Positive Sequence Voltage Drop"
      c_du1pc = "c:du1pc", "%, Positive Sequence Voltage Drop"
      c_dphiu1 = "c:dphiu1", "deg, Positive Sequence Voltage Drop Angle"
      c_Ploss = "c:Ploss", "MW, Losses (total)"
      c_Qloss = "c:Qloss", "Mvar, Reactive-Losses (total)"
      c_Plossld = "c:Plossld", "MW, Losses (load)"
      c_Qlossld = "c:Qlossld", "Mvar, Reactive-Losses (load)"
      c_Plossnld = "c:Plossnld", "MW, Losses (no load)"
      c_Qlossnld = "c:Qlossnld", "Mvar, Reactive-Losses (no load)"
      e_iequalz = "e:iequalz", "Use equal Impedances (zij = zji)"
      e_r_pu = "e:r_pu", "p.u., Real Part"
      e_x_pu = "e:x_pu", "p.u., Imaginary Part"
      e_r_pu_ji = "e:r_pu_ji", "p.u., Real Part"
      e_x_pu_ji = "e:x_pu_ji", "p.u., Imaginary Part"
      e_r0_pu = "e:r0_pu", "p.u., Real Part"
      e_x0_pu = "e:x0_pu", "p.u., Imaginary Part"
      e_r0_pu_ji = "e:r0_pu_ji", "p.u., Real Part"
      e_x0_pu_ji = "e:x0_pu_ji", "p.u., Imaginary Part"
      e_iz2eqz1 = "e:iz2eqz1", "Use Impedance z2 = z1"
      e_r2_pu = "e:r2_pu", "p.u., Real Part"
      e_x2_pu = "e:x2_pu", "p.u., Imaginary Part"
      e_r2_pu_ji = "e:r2_pu_ji", "p.u., Real Part"
      e_x2_pu_ji = "e:x2_pu_ji", "p.u., Imaginary Part"
      e_gi_pu = "e:gi_pu", "p.u., Conductance i"
      e_bi_pu = "e:bi_pu", "p.u., Susceptance i"
      e_gj_pu = "e:gj_pu", "p.u., Conductance j"
      e_bj_pu = "e:bj_pu", "p.u., Susceptance j"
      e_gi0_pu = "e:gi0_pu", "p.u., Conductance i"
      e_bi0_pu = "e:bi0_pu", "p.u., Susceptance i"
      e_gj0_pu = "e:gj0_pu", "p.u., Conductance j"
      e_bj0_pu = "e:bj0_pu", "p.u., Susceptance j"
      e_iy2eqy1 = "e:iy2eqy1", "Use Admittance y2 = y1"
      e_gi2_pu = "e:gi2_pu", "p.u., Conductance i"
      e_bi2_pu = "e:bi2_pu", "p.u., Susceptance i"
      e_gj2_pu = "e:gj2_pu", "p.u., Conductance j"
      e_bj2_pu = "e:bj2_pu", "p.u., Susceptance j"
      e_cpFeed = "e:cpFeed", "Feeder"
      e_ciDist = "e:ciDist", "Distance from infeed in number of buses"
      e_ciLater = "e:ciLater", "Lateral Index"
      e_ciDistRoot = "e:ciDistRoot", "Distance from first infeed in number of buses"
      e_ciDistAll = "e:ciDistAll", "Distance from infeed in number of buses including switches"
      e_ciDistAllRoot = "e:ciDistAllRoot", "Distance from first infeed in number of buses including switches"
      r_cpFeed = "r:cpFeed", "Feeder"


  class LF_Unbal:

    class ElmArea(Enum):
      _init_ = 'value __doc__'

      c_n_ele = "c:n_ele", "Number of Elements"
      c_n_areas = "c:n_areas", "Number of Connected Areas"
      c_n_volt = "c:n_volt", "Number of Voltage Levels"
      c_n_bar = "c:n_bar", "No. of Busbars"
      c_n_term = "c:n_term", "No. of Terminals"
      c_n_lne = "c:n_lne", "No. of Lines"
      c_n_sym = "c:n_sym", "No. of syn. Machines"
      c_n_asm = "c:n_asm", "No. of asyn. Machines"
      c_n_stg = "c:n_stg", "No. of Static Generators"
      c_n_pvs = "c:n_pvs", "No. of PV Systems"
      c_n_lod = "c:n_lod", "No. of Loads"
      c_n_cust = "c:n_cust", "No. of Customers"
      c_TotgenP = "c:TotgenP", "MW, Total Generation, Active Power"
      c_TotgenQ = "c:TotgenQ", "Mvar, Total Generation, Reactive Power"
      c_TotconsP = "c:TotconsP", "MW, Total Consumption, Active Power"
      c_TotconsQ = "c:TotconsQ", "Mvar, Total Consumption, Reactive Power"
      c_GenP = "c:GenP", "MW, Generators, Active Power"
      c_GenQ = "c:GenQ", "Mvar, Generators, Reactive Power"
      c_GenS = "c:GenS", "MVA, Generators, Apparent Power"
      c_Gencos = "c:Gencos", "Generators, Power Factor"
      c_GenPnom = "c:GenPnom", "MW, Generators, Nominal Active Power"
      c_GenQnom = "c:GenQnom", "Mvar, Generators, Nominal Reactive Power"
      c_GenSnom = "c:GenSnom", "MVA, Generators, Nominal Apparent Power"
      c_GenPdiff = "c:GenPdiff", "MW, Generators, difference between nominal and actual active power"
      c_GenQdiff = "c:GenQdiff", "Mvar, Generators, difference between nominal and actual reactive power"
      c_Gencosnom = "c:Gencosnom", "Generators, Nominal Power Factor"
      c_GenPdisp = "c:GenPdisp", "MW, Generators, Dispatch Active Power"
      c_GenPmism = "c:GenPmism", "MW, Generators, difference between actual and dispatched active power"
      c_GenPspinres = "c:GenPspinres", "MW, Generators, difference between maximum and actual active power"
      c_GenQspinres = "c:GenQspinres", "Mvar, Generators, difference between maximum and actual reactive power"
      c_MotP = "c:MotP", "MW, Motor Loads, Active Power"
      c_MotQ = "c:MotQ", "Mvar, Motor Loads, Reactive Power"
      c_MotS = "c:MotS", "MVA, Motor Loads, Apparent Power"
      c_Motcos = "c:Motcos", "Motor Loads, Power Factor"
      c_LoadP = "c:LoadP", "MW, Loads, Active Power"
      c_LoadQ = "c:LoadQ", "Mvar, Loads, Reactive Power"
      c_LoadS = "c:LoadS", "MVA, Loads, Apparent Power"
      c_Loadcos = "c:Loadcos", "Loads, Power Factor"
      c_LoadPnom = "c:LoadPnom", "MW, Loads, Nominal Active Power"
      c_LoadQnom = "c:LoadQnom", "Mvar, Loads, Nominal Reactive Power"
      c_LoadSnom = "c:LoadSnom", "MVA, Loads, Nominal Apparent Power"
      c_Loadcosnom = "c:Loadcosnom", "Loads, Nominal Power Factor"
      c_LoadPdiff = "c:LoadPdiff", "MW, Loads, difference between nominal and actual active power"
      c_LoadQdiff = "c:LoadQdiff", "Mvar, Loads, difference between nominal and actual reactive power"
      c_LoadPdem = "c:LoadPdem", "MW, Loads, Active Power Demand"
      c_LoadPmism = "c:LoadPmism", "MW, Loads, difference between actual active power and demand"
      c_CompP = "c:CompP", "MW, Compensation (Losses)"
      c_CompQ = "c:CompQ", "Mvar, Compensation, C"
      c_ComprQ = "c:ComprQ", "Mvar, Compensation, L"
      c_NetP = "c:NetP", "MW, External Networks, Active Power"
      c_NetQ = "c:NetQ", "Mvar, External Networks, Reactive Power"
      c_NetS = "c:NetS", "MVA, External Networks, Apparent Power"
      c_Netcos = "c:Netcos", "External Networks, Power Factor"
      c_InterP = "c:InterP", "MW, Interchange Flow, Active Power"
      c_InterQ = "c:InterQ", "Mvar, Interchange Flow, Reactive Power"
      c_LossP = "c:LossP", "MW, Losses, Active Power"
      c_LossQ = "c:LossQ", "Mvar, Losses, Reactive Power"
      c_LossPld = "c:LossPld", "MW, Losses, Active Power (load)"
      c_LossQld = "c:LossQld", "Mvar, Losses, Reactive Power (load)"
      c_LossPnld = "c:LossPnld", "MW, Losses, Active Power (no load)"
      c_LossQnld = "c:LossQnld", "Mvar, Losses, Reactive Power (no load)"
      c_LossPlne = "c:LossPlne", "MW, Line Losses, Active Power"
      c_LossQlne = "c:LossQlne", "Mvar, Line Losses, Reactive Power"
      c_LossPldlne = "c:LossPldlne", "MW, Line Losses, Active Power (load)"
      c_LossQldlne = "c:LossQldlne", "Mvar, Line Losses, Reactive Power (load)"
      c_LossPnldlne = "c:LossPnldlne", "MW, Line Losses, Active Power (no load)"
      c_LossQnldlne = "c:LossQnldlne", "Mvar, Line Losses, Reactive Power (no load)"
      c_LossPtrf = "c:LossPtrf", "MW, Transformer Losses, Active Power"
      c_LossQtrf = "c:LossQtrf", "Mvar, Transformer Losses, Reactive Power"
      c_LossPldtrf = "c:LossPldtrf", "MW, Transformer Losses, Active Power (load)"
      c_LossQldtrf = "c:LossQldtrf", "Mvar, Transformer Losses, Reactive Power (load)"
      c_LossPnldtrf = "c:LossPnldtrf", "MW, Transformer Losses, Active Power (no load)"
      c_LossQnldtrf = "c:LossQnldtrf", "Mvar, Transformer Losses, Reactive Power (no load)"
      c_TotLoadP = "c:TotLoadP", "MW, Total Load, Active Power"
      c_TotLoadQ = "c:TotLoadQ", "Mvar, Total Load, Reactive Power"
      c_TotLoadS = "c:TotLoadS", "MVA, Total Load, Apparent Power"
      c_TotLoadcos = "c:TotLoadcos", "Total Load, Power Factor"
      c_TotP = "c:TotP", "MW, Total Demand"
      c_UnsupP = "c:UnsupP", "MW, Unsupplied Demand"
      c_SupP = "c:SupP", "MW, Supplied Demand"
      c_ExportP = "c:ExportP", "MW, Export, Active Power"
      c_ExportQ = "c:ExportQ", "Mvar, Export, Reactive Power"
      c_ImportP = "c:ImportP", "MW, Import, Active Power"
      c_ImportQ = "c:ImportQ", "Mvar, Import, Reactive Power"
      c_cst_disp = "c:cst_disp", "USD/h, Total Production Costs"
      c_RUC_Lead = "c:RUC_Lead", "Mvar, Used Reactive Capacity Lead"
      c_RUC_Lag = "c:RUC_Lag", "Mvar, Used Reactive Capacity Lag"
      c_RSC_Lead = "c:RSC_Lead", "Mvar, Spare Reactive Capacity Lead"
      c_RSC_Lag = "c:RSC_Lag", "Mvar, Spare Reactive Capacity Lag"
      c_Q_shgain = "c:Q_shgain", "Mvar, Reactive Power Shunt Gain"
      c_ConnMvar_max = "c:ConnMvar_max", "Mvar, Connected MVAR Capacity (max)"
      c_ConnMvar_min = "c:ConnMvar_min", "Mvar, Connected MVAR Capacity (min)"
      c_Maxloading = "c:Maxloading", "%, Maximum Loading"
      c_uphtmin = "c:uphtmin", "p.u., Minimum voltage of all terminals"
      c_uphtmax = "c:uphtmax", "p.u., Maximum voltage of all terminals"
      e_iInterChg = "e:iInterChg", "Consider Interchange Schedule"
      e_InterPset = "e:InterPset", "MW, Scheduled Active Power Interchange"

    class ElmAsm(Enum):
      _init_ = 'value __doc__'

      m_u0_bus1 = "m:u0:bus1", "p.u., Zero-Sequence-Voltage, Magnitude"
      m_u0_bus1n = "m:u0:bus1n", "p.u., Zero-Sequence-Voltage, Magnitude"
      m_phiu0_bus1 = "m:phiu0:bus1", "deg, Zero-Sequence-Voltage, Angle"
      m_phiu0_bus1n = "m:phiu0:bus1n", "deg, Zero-Sequence-Voltage, Angle"
      m_u1r_bus1 = "m:u1r:bus1", "p.u., Positive-Sequence-Voltage, Real Part"
      m_u1r_bus1n = "m:u1r:bus1n", "p.u., Positive-Sequence-Voltage, Real Part"
      m_u1i_bus1 = "m:u1i:bus1", "p.u., Positive-Sequence-Voltage, Imaginary Part"
      m_u1i_bus1n = "m:u1i:bus1n", "p.u., Positive-Sequence-Voltage, Imaginary Part"
      m_u1_bus1 = "m:u1:bus1", "p.u., Positive-Sequence-Voltage, Magnitude"
      m_u1_bus1n = "m:u1:bus1n", "p.u., Positive-Sequence-Voltage, Magnitude"
      m_U1_bus1 = "m:U1:bus1", "kV, Line-Ground Positive-Sequence-Voltage, Magnitude"
      m_U1_bus1n = "m:U1:bus1n", "kV, Line-Ground Positive-Sequence-Voltage, Magnitude"
      m_U1l_bus1 = "m:U1l:bus1", "kV, Line-Line Positive-Sequence-Voltage, Magnitude"
      m_U1l_bus1n = "m:U1l:bus1n", "kV, Line-Line Positive-Sequence-Voltage, Magnitude"
      m_phiu1_bus1 = "m:phiu1:bus1", "deg, Positive-Sequence-Voltage, Angle"
      m_phiu1_bus1n = "m:phiu1:bus1n", "deg, Positive-Sequence-Voltage, Angle"
      m_u2_bus1 = "m:u2:bus1", "p.u., Negative-Sequence-Voltage, Magnitude"
      m_u2_bus1n = "m:u2:bus1n", "p.u., Negative-Sequence-Voltage, Magnitude"
      m_phiu2_bus1 = "m:phiu2:bus1", "deg, Negative-Sequence-Voltage, Angle"
      m_phiu2_bus1n = "m:phiu2:bus1n", "deg, Negative-Sequence-Voltage, Angle"
      m_inet_bus1_A = "m:inet:bus1:A", "p.u., Phase Current, Magnitude, referred to network"
      m_inet_bus1_B = "m:inet:bus1:B", "p.u., Phase Current, Magnitude, referred to network"
      m_inet_bus1_C = "m:inet:bus1:C", "p.u., Phase Current, Magnitude, referred to network"
      m_inet_bus1_N = "m:inet:bus1:N", "p.u., Phase Current, Magnitude, referred to network"
      m_inet_bus1n = "m:inet:bus1n", "p.u., Phase Current, Magnitude, referred to network"
      m_I_bus1_A = "m:I:bus1:A", "kA, Phase Current, Magnitude"
      m_I_bus1_B = "m:I:bus1:B", "kA, Phase Current, Magnitude"
      m_I_bus1_C = "m:I:bus1:C", "kA, Phase Current, Magnitude"
      m_I_bus1_N = "m:I:bus1:N", "kA, Phase Current, Magnitude"
      m_I_bus1n = "m:I:bus1n", "kA, Phase Current, Magnitude"
      m_phii_bus1_A = "m:phii:bus1:A", "deg, Phase Current, Angle"
      m_phii_bus1_B = "m:phii:bus1:B", "deg, Phase Current, Angle"
      m_phii_bus1_C = "m:phii:bus1:C", "deg, Phase Current, Angle"
      m_phii_bus1_N = "m:phii:bus1:N", "deg, Phase Current, Angle"
      m_phii_bus1n = "m:phii:bus1n", "deg, Phase Current, Angle"
      m_i0r_bus1 = "m:i0r:bus1", "p.u., Zero-Sequence Current, Real Part"
      m_i0r_bus1n = "m:i0r:bus1n", "p.u., Zero-Sequence Current, Real Part"
      m_i0i_bus1 = "m:i0i:bus1", "p.u., Zero-Sequence Current, Imaginary Part"
      m_i0i_bus1n = "m:i0i:bus1n", "p.u., Zero-Sequence Current, Imaginary Part"
      m_i0_bus1 = "m:i0:bus1", "p.u., Zero-Sequence Current, Magnitude"
      m_i0_bus1n = "m:i0:bus1n", "p.u., Zero-Sequence Current, Magnitude"
      m_I0_bus1 = "m:I0:bus1", "kA, Zero-Sequence Current, Magnitude"
      m_I0_bus1n = "m:I0:bus1n", "kA, Zero-Sequence Current, Magnitude"
      m_I0x3_bus1 = "m:I0x3:bus1", "kA, 3*I0"
      m_I0x3_bus1n = "m:I0x3:bus1n", "kA, 3*I0"
      m_phii0_bus1 = "m:phii0:bus1", "deg, Zero-Sequence Current, Angle"
      m_phii0_bus1n = "m:phii0:bus1n", "deg, Zero-Sequence Current, Angle"
      m_i1r_bus1 = "m:i1r:bus1", "p.u., Positive-Sequence Current, Real Part"
      m_i1r_bus1n = "m:i1r:bus1n", "p.u., Positive-Sequence Current, Real Part"
      m_i1i_bus1 = "m:i1i:bus1", "p.u., Positive-Sequence Current, Imaginary Part"
      m_i1i_bus1n = "m:i1i:bus1n", "p.u., Positive-Sequence Current, Imaginary Part"
      m_i1_bus1 = "m:i1:bus1", "p.u., Positive-Sequence Current, Magnitude"
      m_i1_bus1n = "m:i1:bus1n", "p.u., Positive-Sequence Current, Magnitude"
      m_I1_bus1 = "m:I1:bus1", "kA, Positive-Sequence Current, Magnitude"
      m_I1_bus1n = "m:I1:bus1n", "kA, Positive-Sequence Current, Magnitude"
      m_phii1_bus1 = "m:phii1:bus1", "deg, Positive-Sequence Current, Angle"
      m_phii1_bus1n = "m:phii1:bus1n", "deg, Positive-Sequence Current, Angle"
      m_i2r_bus1 = "m:i2r:bus1", "p.u., Negative-Sequence Current, Real Part"
      m_i2r_bus1n = "m:i2r:bus1n", "p.u., Negative-Sequence Current, Real Part"
      m_i2i_bus1 = "m:i2i:bus1", "p.u., Negative-Sequence Current, Imaginary Part"
      m_i2i_bus1n = "m:i2i:bus1n", "p.u., Negative-Sequence Current, Imaginary Part"
      m_i2_bus1 = "m:i2:bus1", "p.u., Negative-Sequence Current, Magnitude"
      m_i2_bus1n = "m:i2:bus1n", "p.u., Negative-Sequence Current, Magnitude"
      m_I2_bus1 = "m:I2:bus1", "kA, Negative-Sequence Current, Magnitude"
      m_I2_bus1n = "m:I2:bus1n", "kA, Negative-Sequence Current, Magnitude"
      m_phii2_bus1 = "m:phii2:bus1", "deg, Negative-Sequence Current, Angle"
      m_phii2_bus1n = "m:phii2:bus1n", "deg, Negative-Sequence Current, Angle"
      m_ubfacI_bus1 = "m:ubfacI:bus1", "%, Current Unbalance Factor"
      m_ubfacI_bus1n = "m:ubfacI:bus1n", "%, Current Unbalance Factor"
      m_ubfacS_bus1 = "m:ubfacS:bus1", "%, Power Unbalance Factor"
      m_ubfacS_bus1n = "m:ubfacS:bus1n", "%, Power Unbalance Factor"
      m_P_bus1_A = "m:P:bus1:A", "MW, Active Power"
      m_P_bus1_B = "m:P:bus1:B", "MW, Active Power"
      m_P_bus1_C = "m:P:bus1:C", "MW, Active Power"
      m_P_bus1_N = "m:P:bus1:N", "MW, Active Power"
      m_P_bus1n = "m:P:bus1n", "MW, Active Power"
      m_Q_bus1_A = "m:Q:bus1:A", "Mvar, Reactive Power"
      m_Q_bus1_B = "m:Q:bus1:B", "Mvar, Reactive Power"
      m_Q_bus1_C = "m:Q:bus1:C", "Mvar, Reactive Power"
      m_Q_bus1_N = "m:Q:bus1:N", "Mvar, Reactive Power"
      m_Q_bus1n = "m:Q:bus1n", "Mvar, Reactive Power"
      m_S_bus1_A = "m:S:bus1:A", "MVA, Apparent Power"
      m_S_bus1_B = "m:S:bus1:B", "MVA, Apparent Power"
      m_S_bus1_C = "m:S:bus1:C", "MVA, Apparent Power"
      m_S_bus1_N = "m:S:bus1:N", "MVA, Apparent Power"
      m_S_bus1n = "m:S:bus1n", "MVA, Apparent Power"
      m_cosphi_bus1_A = "m:cosphi:bus1:A", "Power Factor"
      m_cosphi_bus1_B = "m:cosphi:bus1:B", "Power Factor"
      m_cosphi_bus1_C = "m:cosphi:bus1:C", "Power Factor"
      m_cosphi_bus1_N = "m:cosphi:bus1:N", "Power Factor"
      m_cosphi_bus1n = "m:cosphi:bus1n", "Power Factor"
      m_tanphi_bus1_A = "m:tanphi:bus1:A", "tan(phi)"
      m_tanphi_bus1_B = "m:tanphi:bus1:B", "tan(phi)"
      m_tanphi_bus1_C = "m:tanphi:bus1:C", "tan(phi)"
      m_tanphi_bus1_N = "m:tanphi:bus1:N", "tan(phi)"
      m_tanphi_bus1n = "m:tanphi:bus1n", "tan(phi)"
      m_Psum_bus1 = "m:Psum:bus1", "MW, Total Active Power"
      m_Psum_bus1n = "m:Psum:bus1n", "MW, Total Active Power"
      m_Qsum_bus1 = "m:Qsum:bus1", "Mvar, Total Reactive Power"
      m_Qsum_bus1n = "m:Qsum:bus1n", "Mvar, Total Reactive Power"
      m_Ssum_bus1 = "m:Ssum:bus1", "MVA, Total Apparent Power"
      m_Ssum_bus1n = "m:Ssum:bus1n", "MVA, Total Apparent Power"
      m_cosphisum_bus1 = "m:cosphisum:bus1", "Total Power Factor"
      m_cosphisum_bus1n = "m:cosphisum:bus1n", "Total Power Factor"
      m_tanphisum_bus1 = "m:tanphisum:bus1", "Total tan(phi)"
      m_tanphisum_bus1n = "m:tanphisum:bus1n", "Total tan(phi)"
      m_Pln_bus1_A = "m:Pln:bus1:A", "MW, Active Power (Line-Neutral)"
      m_Pln_bus1_B = "m:Pln:bus1:B", "MW, Active Power (Line-Neutral)"
      m_Pln_bus1_C = "m:Pln:bus1:C", "MW, Active Power (Line-Neutral)"
      m_Pln_bus1_N = "m:Pln:bus1:N", "MW, Active Power (Line-Neutral)"
      m_Pln_bus1n = "m:Pln:bus1n", "MW, Active Power (Line-Neutral)"
      m_Qln_bus1_A = "m:Qln:bus1:A", "Mvar, Reactive Power (Line-Neutral)"
      m_Qln_bus1_B = "m:Qln:bus1:B", "Mvar, Reactive Power (Line-Neutral)"
      m_Qln_bus1_C = "m:Qln:bus1:C", "Mvar, Reactive Power (Line-Neutral)"
      m_Qln_bus1_N = "m:Qln:bus1:N", "Mvar, Reactive Power (Line-Neutral)"
      m_Qln_bus1n = "m:Qln:bus1n", "Mvar, Reactive Power (Line-Neutral)"
      m_P1_bus1 = "m:P1:bus1", "MW, Positive Sequence Active Power"
      m_P1_bus1n = "m:P1:bus1n", "MW, Positive Sequence Active Power"
      m_Q1_bus1 = "m:Q1:bus1", "Mvar, Positive Sequence Reactive Power"
      m_Q1_bus1n = "m:Q1:bus1n", "Mvar, Positive Sequence Reactive Power"
      m_P2_bus1 = "m:P2:bus1", "MW, Negative Sequence Active Power"
      m_P2_bus1n = "m:P2:bus1n", "MW, Negative Sequence Active Power"
      m_Q2_bus1 = "m:Q2:bus1", "Mvar, Negative Sequence Reactive Power"
      m_Q2_bus1n = "m:Q2:bus1n", "Mvar, Negative Sequence Reactive Power"
      m_P0_bus1 = "m:P0:bus1", "MW, Zero Sequence Active Power"
      m_P0_bus1n = "m:P0:bus1n", "MW, Zero Sequence Active Power"
      m_Q0_bus1 = "m:Q0:bus1", "Mvar, Zero Sequence Reactive Power"
      m_Q0_bus1n = "m:Q0:bus1n", "Mvar, Zero Sequence Reactive Power"
      m_i1P_bus1 = "m:i1P:bus1", "p.u., Positive-Sequence Active Current"
      m_i1P_bus1n = "m:i1P:bus1n", "p.u., Positive-Sequence Active Current"
      m_i1Q_bus1 = "m:i1Q:bus1", "p.u., Positive-Sequence Reactive Current"
      m_i1Q_bus1n = "m:i1Q:bus1n", "p.u., Positive-Sequence Reactive Current"
      m_I1P_bus1 = "m:I1P:bus1", "kA, Positive-Sequence Active Current"
      m_I1P_bus1n = "m:I1P:bus1n", "kA, Positive-Sequence Active Current"
      m_I1Q_bus1 = "m:I1Q:bus1", "kA, Positive-Sequence Reactive Current"
      m_I1Q_bus1n = "m:I1Q:bus1n", "kA, Positive-Sequence Reactive Current"
      m_i2P_bus1 = "m:i2P:bus1", "p.u., Negative-Sequence Active Current"
      m_i2P_bus1n = "m:i2P:bus1n", "p.u., Negative-Sequence Active Current"
      m_i2Q_bus1 = "m:i2Q:bus1", "p.u., Negative-Sequence Reactive Current"
      m_i2Q_bus1n = "m:i2Q:bus1n", "p.u., Negative-Sequence Reactive Current"
      m_I2P_bus1 = "m:I2P:bus1", "kA, Negative-Sequence Active Current"
      m_I2P_bus1n = "m:I2P:bus1n", "kA, Negative-Sequence Active Current"
      m_I2Q_bus1 = "m:I2Q:bus1", "kA, Negative-Sequence Reactive Current"
      m_I2Q_bus1n = "m:I2Q:bus1n", "kA, Negative-Sequence Reactive Current"
      m_ur_bus1_A = "m:ur:bus1:A", "p.u., Phase Voltage, Real Part"
      m_ur_bus1_B = "m:ur:bus1:B", "p.u., Phase Voltage, Real Part"
      m_ur_bus1_C = "m:ur:bus1:C", "p.u., Phase Voltage, Real Part"
      m_ur_bus1_N = "m:ur:bus1:N", "p.u., Phase Voltage, Real Part"
      m_ur_bus1n = "m:ur:bus1n", "p.u., Phase Voltage, Real Part"
      m_ui_bus1_A = "m:ui:bus1:A", "p.u., Phase Voltage, Imaginary Part"
      m_ui_bus1_B = "m:ui:bus1:B", "p.u., Phase Voltage, Imaginary Part"
      m_ui_bus1_C = "m:ui:bus1:C", "p.u., Phase Voltage, Imaginary Part"
      m_ui_bus1_N = "m:ui:bus1:N", "p.u., Phase Voltage, Imaginary Part"
      m_ui_bus1n = "m:ui:bus1n", "p.u., Phase Voltage, Imaginary Part"
      m_u_bus1_A = "m:u:bus1:A", "p.u., Phase Voltage, Magnitude"
      m_u_bus1_B = "m:u:bus1:B", "p.u., Phase Voltage, Magnitude"
      m_u_bus1_C = "m:u:bus1:C", "p.u., Phase Voltage, Magnitude"
      m_u_bus1_N = "m:u:bus1:N", "p.u., Phase Voltage, Magnitude"
      m_u_bus1n = "m:u:bus1n", "p.u., Phase Voltage, Magnitude"
      m_ir_bus1_A = "m:ir:bus1:A", "p.u., Phase Current, Real Part"
      m_ir_bus1_B = "m:ir:bus1:B", "p.u., Phase Current, Real Part"
      m_ir_bus1_C = "m:ir:bus1:C", "p.u., Phase Current, Real Part"
      m_ir_bus1_N = "m:ir:bus1:N", "p.u., Phase Current, Real Part"
      m_ir_bus1n = "m:ir:bus1n", "p.u., Phase Current, Real Part"
      m_ii_bus1_A = "m:ii:bus1:A", "p.u., Phase Current, Imaginary Part"
      m_ii_bus1_B = "m:ii:bus1:B", "p.u., Phase Current, Imaginary Part"
      m_ii_bus1_C = "m:ii:bus1:C", "p.u., Phase Current, Imaginary Part"
      m_ii_bus1_N = "m:ii:bus1:N", "p.u., Phase Current, Imaginary Part"
      m_ii_bus1n = "m:ii:bus1n", "p.u., Phase Current, Imaginary Part"
      m_i_bus1_A = "m:i:bus1:A", "p.u., Phase Current, Magnitude"
      m_i_bus1_B = "m:i:bus1:B", "p.u., Phase Current, Magnitude"
      m_i_bus1_C = "m:i:bus1:C", "p.u., Phase Current, Magnitude"
      m_i_bus1_N = "m:i:bus1:N", "p.u., Phase Current, Magnitude"
      m_i_bus1n = "m:i:bus1n", "p.u., Phase Current, Magnitude"
      m_phiui_bus1_A = "m:phiui:bus1:A", "deg, Angle between Voltage and Current"
      m_phiui_bus1_B = "m:phiui:bus1:B", "deg, Angle between Voltage and Current"
      m_phiui_bus1_C = "m:phiui:bus1:C", "deg, Angle between Voltage and Current"
      m_phiui_bus1_N = "m:phiui:bus1:N", "deg, Angle between Voltage and Current"
      m_phiui_bus1n = "m:phiui:bus1n", "deg, Angle between Voltage and Current"
      m_phiu0i0_bus1 = "m:phiu0i0:bus1", "deg, Angle between Voltage and Current in zero-sequence system"
      m_phiu0i0_bus1n = "m:phiu0i0:bus1n", "deg, Angle between Voltage and Current in zero-sequence system"
      m_phiu1i1_bus1 = "m:phiu1i1:bus1", "deg, Angle between Voltage and Current in positive sequence system"
      m_phiu1i1_bus1n = "m:phiu1i1:bus1n", "deg, Angle between Voltage and Current in positive sequence system"
      m_phiu2i2_bus1 = "m:phiu2i2:bus1", "deg, Angle between Voltage and Current in negative sequence system"
      m_phiu2i2_bus1n = "m:phiu2i2:bus1n", "deg, Angle between Voltage and Current in negative sequence system"
      m_Tfct_bus1 = "m:Tfct:bus1", "s, Fault Clearing Time"
      m_Tfct_bus1n = "m:Tfct:bus1n", "s, Fault Clearing Time"
      m_TfctPh_bus1_A = "m:TfctPh:bus1:A", "s, Fault Clearing Time"
      m_TfctPh_bus1_B = "m:TfctPh:bus1:B", "s, Fault Clearing Time"
      m_TfctPh_bus1_C = "m:TfctPh:bus1:C", "s, Fault Clearing Time"
      m_TfctPh_bus1_N = "m:TfctPh:bus1:N", "s, Fault Clearing Time"
      m_TfctPh_bus1n = "m:TfctPh:bus1n", "s, Fault Clearing Time"
      m_Brkload_bus1 = "m:Brkload:bus1", "%, Max. Breaker Loading"
      m_Brkload_bus1n = "m:Brkload:bus1n", "%, Max. Breaker Loading"
      m_BrkloadPh_bus1_A = "m:BrkloadPh:bus1:A", "%, Breaker Loading"
      m_BrkloadPh_bus1_B = "m:BrkloadPh:bus1:B", "%, Breaker Loading"
      m_BrkloadPh_bus1_C = "m:BrkloadPh:bus1:C", "%, Breaker Loading"
      m_BrkloadPh_bus1_N = "m:BrkloadPh:bus1:N", "%, Breaker Loading"
      m_BrkloadPh_bus1n = "m:BrkloadPh:bus1n", "%, Breaker Loading"
      n_ur_bus1_A = "n:ur:bus1:A", "p.u., Line-Ground Voltage, Real Part"
      n_ur_bus1_B = "n:ur:bus1:B", "p.u., Line-Ground Voltage, Real Part"
      n_ur_bus1_C = "n:ur:bus1:C", "p.u., Line-Ground Voltage, Real Part"
      n_ur_bus1_N = "n:ur:bus1:N", "p.u., Line-Ground Voltage, Real Part"
      n_ur_bus1n = "n:ur:bus1n", "p.u., Line-Ground Voltage, Real Part"
      n_ui_bus1_A = "n:ui:bus1:A", "p.u., Line-Ground Voltage, Imaginary Part"
      n_ui_bus1_B = "n:ui:bus1:B", "p.u., Line-Ground Voltage, Imaginary Part"
      n_ui_bus1_C = "n:ui:bus1:C", "p.u., Line-Ground Voltage, Imaginary Part"
      n_ui_bus1_N = "n:ui:bus1:N", "p.u., Line-Ground Voltage, Imaginary Part"
      n_ui_bus1n = "n:ui:bus1n", "p.u., Line-Ground Voltage, Imaginary Part"
      n_u_bus1_A = "n:u:bus1:A", "p.u., Line-Ground Voltage, Magnitude"
      n_u_bus1_B = "n:u:bus1:B", "p.u., Line-Ground Voltage, Magnitude"
      n_u_bus1_C = "n:u:bus1:C", "p.u., Line-Ground Voltage, Magnitude"
      n_u_bus1_N = "n:u:bus1:N", "p.u., Line-Ground Voltage, Magnitude"
      n_u_bus1n = "n:u:bus1n", "p.u., Line-Ground Voltage, Magnitude"
      n_upc_bus1_A = "n:upc:bus1:A", "%, Line-Ground Voltage, Magnitude"
      n_upc_bus1_B = "n:upc:bus1:B", "%, Line-Ground Voltage, Magnitude"
      n_upc_bus1_C = "n:upc:bus1:C", "%, Line-Ground Voltage, Magnitude"
      n_upc_bus1_N = "n:upc:bus1:N", "%, Line-Ground Voltage, Magnitude"
      n_upc_bus1n = "n:upc:bus1n", "%, Line-Ground Voltage, Magnitude"
      n_U_bus1_A = "n:U:bus1:A", "kV, Line-Ground Voltage, Magnitude"
      n_U_bus1_B = "n:U:bus1:B", "kV, Line-Ground Voltage, Magnitude"
      n_U_bus1_C = "n:U:bus1:C", "kV, Line-Ground Voltage, Magnitude"
      n_U_bus1_N = "n:U:bus1:N", "kV, Line-Ground Voltage, Magnitude"
      n_U_bus1n = "n:U:bus1n", "kV, Line-Ground Voltage, Magnitude"
      n_phiu_bus1_A = "n:phiu:bus1:A", "deg, Line-Ground Voltage, Angle"
      n_phiu_bus1_B = "n:phiu:bus1:B", "deg, Line-Ground Voltage, Angle"
      n_phiu_bus1_C = "n:phiu:bus1:C", "deg, Line-Ground Voltage, Angle"
      n_phiu_bus1_N = "n:phiu:bus1:N", "deg, Line-Ground Voltage, Angle"
      n_phiu_bus1n = "n:phiu:bus1n", "deg, Line-Ground Voltage, Angle"
      n_phiurel_bus1 = "n:phiurel:bus1", "deg, Voltage, Relative Angle"
      n_phiurel_bus1n = "n:phiurel:bus1n", "deg, Voltage, Relative Angle"
      n_ul_bus1_A = "n:ul:bus1:A", "p.u., Line-Line Voltage, Magnitude"
      n_ul_bus1_B = "n:ul:bus1:B", "p.u., Line-Line Voltage, Magnitude"
      n_ul_bus1_C = "n:ul:bus1:C", "p.u., Line-Line Voltage, Magnitude"
      n_ul_bus1_N = "n:ul:bus1:N", "p.u., Line-Line Voltage, Magnitude"
      n_ul_bus1n = "n:ul:bus1n", "p.u., Line-Line Voltage, Magnitude"
      n_ulpc_bus1_A = "n:ulpc:bus1:A", "%, Line-Line Voltage, Magnitude"
      n_ulpc_bus1_B = "n:ulpc:bus1:B", "%, Line-Line Voltage, Magnitude"
      n_ulpc_bus1_C = "n:ulpc:bus1:C", "%, Line-Line Voltage, Magnitude"
      n_ulpc_bus1_N = "n:ulpc:bus1:N", "%, Line-Line Voltage, Magnitude"
      n_ulpc_bus1n = "n:ulpc:bus1n", "%, Line-Line Voltage, Magnitude"
      n_Ul_bus1_A = "n:Ul:bus1:A", "kV, Line-Line Voltage, Magnitude"
      n_Ul_bus1_B = "n:Ul:bus1:B", "kV, Line-Line Voltage, Magnitude"
      n_Ul_bus1_C = "n:Ul:bus1:C", "kV, Line-Line Voltage, Magnitude"
      n_Ul_bus1_N = "n:Ul:bus1:N", "kV, Line-Line Voltage, Magnitude"
      n_Ul_bus1n = "n:Ul:bus1n", "kV, Line-Line Voltage, Magnitude"
      n_phiul_bus1_A = "n:phiul:bus1:A", "deg, Line-Line Voltage, Angle"
      n_phiul_bus1_B = "n:phiul:bus1:B", "deg, Line-Line Voltage, Angle"
      n_phiul_bus1_C = "n:phiul:bus1:C", "deg, Line-Line Voltage, Angle"
      n_phiul_bus1_N = "n:phiul:bus1:N", "deg, Line-Line Voltage, Angle"
      n_phiul_bus1n = "n:phiul:bus1n", "deg, Line-Line Voltage, Angle"
      n_u0_bus1 = "n:u0:bus1", "p.u., Zero-Sequence Voltage, Magnitude"
      n_u0_bus1n = "n:u0:bus1n", "p.u., Zero-Sequence Voltage, Magnitude"
      n_U0_bus1 = "n:U0:bus1", "kV, Zero-Sequence Voltage, Magnitude"
      n_U0_bus1n = "n:U0:bus1n", "kV, Zero-Sequence Voltage, Magnitude"
      n_U0x3_bus1 = "n:U0x3:bus1", "kV, 3*U0"
      n_U0x3_bus1n = "n:U0x3:bus1n", "kV, 3*U0"
      n_phiu0_bus1 = "n:phiu0:bus1", "deg, Zero-Sequence Voltage, Angle"
      n_phiu0_bus1n = "n:phiu0:bus1n", "deg, Zero-Sequence Voltage, Angle"
      n_u1_bus1 = "n:u1:bus1", "p.u., Positive-Sequence Voltage, Magnitude"
      n_u1_bus1n = "n:u1:bus1n", "p.u., Positive-Sequence Voltage, Magnitude"
      n_u1pc_bus1 = "n:u1pc:bus1", "%, Positive-Sequence Voltage, Magnitude"
      n_u1pc_bus1n = "n:u1pc:bus1n", "%, Positive-Sequence Voltage, Magnitude"
      n_u1r_bus1 = "n:u1r:bus1", "p.u., Positive-Sequence Voltage, Real Part"
      n_u1r_bus1n = "n:u1r:bus1n", "p.u., Positive-Sequence Voltage, Real Part"
      n_u1i_bus1 = "n:u1i:bus1", "p.u., Positive-Sequence Voltage, Imaginary Part"
      n_u1i_bus1n = "n:u1i:bus1n", "p.u., Positive-Sequence Voltage, Imaginary Part"
      n_U1_bus1 = "n:U1:bus1", "kV, Line-Ground Positive-Sequence Voltage, Magnitude"
      n_U1_bus1n = "n:U1:bus1n", "kV, Line-Ground Positive-Sequence Voltage, Magnitude"
      n_phiu1_bus1 = "n:phiu1:bus1", "deg, Positive-Sequence Voltage, Angle"
      n_phiu1_bus1n = "n:phiu1:bus1n", "deg, Positive-Sequence Voltage, Angle"
      n_u2_bus1 = "n:u2:bus1", "p.u., Negative-Sequence Voltage, Magnitude"
      n_u2_bus1n = "n:u2:bus1n", "p.u., Negative-Sequence Voltage, Magnitude"
      n_U2_bus1 = "n:U2:bus1", "kV, Line-Ground Negative-Sequence Voltage, Magnitude"
      n_U2_bus1n = "n:U2:bus1n", "kV, Line-Ground Negative-Sequence Voltage, Magnitude"
      n_phiu2_bus1 = "n:phiu2:bus1", "deg, Negative-Sequence Voltage, Angle"
      n_phiu2_bus1n = "n:phiu2:bus1n", "deg, Negative-Sequence Voltage, Angle"
      n_U1l_bus1 = "n:U1l:bus1", "kV, Line-Line Positive-Sequence Voltage, Magnitude"
      n_U1l_bus1n = "n:U1l:bus1n", "kV, Line-Line Positive-Sequence Voltage, Magnitude"
      n_U2l_bus1 = "n:U2l:bus1", "kV, Line-Line Negative-Sequence Voltage, Magnitude"
      n_U2l_bus1n = "n:U2l:bus1n", "kV, Line-Line Negative-Sequence Voltage, Magnitude"
      n_uln_bus1_A = "n:uln:bus1:A", "p.u., Line-Neutral Voltage, Magnitude"
      n_uln_bus1_B = "n:uln:bus1:B", "p.u., Line-Neutral Voltage, Magnitude"
      n_uln_bus1_C = "n:uln:bus1:C", "p.u., Line-Neutral Voltage, Magnitude"
      n_uln_bus1_N = "n:uln:bus1:N", "p.u., Line-Neutral Voltage, Magnitude"
      n_uln_bus1n = "n:uln:bus1n", "p.u., Line-Neutral Voltage, Magnitude"
      n_Uln_bus1_A = "n:Uln:bus1:A", "kV, Line-Neutral Voltage, Magnitude"
      n_Uln_bus1_B = "n:Uln:bus1:B", "kV, Line-Neutral Voltage, Magnitude"
      n_Uln_bus1_C = "n:Uln:bus1:C", "kV, Line-Neutral Voltage, Magnitude"
      n_Uln_bus1_N = "n:Uln:bus1:N", "kV, Line-Neutral Voltage, Magnitude"
      n_Uln_bus1n = "n:Uln:bus1n", "kV, Line-Neutral Voltage, Magnitude"
      n_phiuln_bus1_A = "n:phiuln:bus1:A", "deg, Line-Neutral Voltage, Angle"
      n_phiuln_bus1_B = "n:phiuln:bus1:B", "deg, Line-Neutral Voltage, Angle"
      n_phiuln_bus1_C = "n:phiuln:bus1:C", "deg, Line-Neutral Voltage, Angle"
      n_phiuln_bus1_N = "n:phiuln:bus1:N", "deg, Line-Neutral Voltage, Angle"
      n_phiuln_bus1n = "n:phiuln:bus1n", "deg, Line-Neutral Voltage, Angle"
      n_un_bus1 = "n:un:bus1", "p.u., Neutral-Ground Voltage, Magnitude"
      n_un_bus1n = "n:un:bus1n", "p.u., Neutral-Ground Voltage, Magnitude"
      n_Un_bus1 = "n:Un:bus1", "kV, Neutral-Ground Voltage, Magnitude"
      n_Un_bus1n = "n:Un:bus1n", "kV, Neutral-Ground Voltage, Magnitude"
      n_phiun_bus1 = "n:phiun:bus1", "deg, Neutral-Ground Voltage, Angle"
      n_phiun_bus1n = "n:phiun:bus1n", "deg, Neutral-Ground Voltage, Angle"
      n_upht_bus1_A = "n:upht:bus1:A", "p.u., Phase Technology dependent Voltage, Magnitude"
      n_upht_bus1_B = "n:upht:bus1:B", "p.u., Phase Technology dependent Voltage, Magnitude"
      n_upht_bus1_C = "n:upht:bus1:C", "p.u., Phase Technology dependent Voltage, Magnitude"
      n_upht_bus1_N = "n:upht:bus1:N", "p.u., Phase Technology dependent Voltage, Magnitude"
      n_upht_bus1n = "n:upht:bus1n", "p.u., Phase Technology dependent Voltage, Magnitude"
      n_Upht_bus1_A = "n:Upht:bus1:A", "kV, Phase Technology dependent Voltage, Magnitude"
      n_Upht_bus1_B = "n:Upht:bus1:B", "kV, Phase Technology dependent Voltage, Magnitude"
      n_Upht_bus1_C = "n:Upht:bus1:C", "kV, Phase Technology dependent Voltage, Magnitude"
      n_Upht_bus1_N = "n:Upht:bus1:N", "kV, Phase Technology dependent Voltage, Magnitude"
      n_Upht_bus1n = "n:Upht:bus1n", "kV, Phase Technology dependent Voltage, Magnitude"
      n_um_bus1 = "n:um:bus1", "p.u., Average-Voltage, Magnitude"
      n_um_bus1n = "n:um:bus1n", "p.u., Average-Voltage, Magnitude"
      n_Um_bus1 = "n:Um:bus1", "kV, Average-Voltage, Magnitude"
      n_Um_bus1n = "n:Um:bus1n", "kV, Average-Voltage, Magnitude"
      n_uphtmin_bus1 = "n:uphtmin:bus1", "p.u., Minimum of Phase Technology dependent Voltage, Magnitude"
      n_uphtmin_bus1n = "n:uphtmin:bus1n", "p.u., Minimum of Phase Technology dependent Voltage, Magnitude"
      n_uphtmax_bus1 = "n:uphtmax:bus1", "p.u., Maximum of Phase Technology dependent Voltage, Magnitude"
      n_uphtmax_bus1n = "n:uphtmax:bus1n", "p.u., Maximum of Phase Technology dependent Voltage, Magnitude"
      n_Pgen_bus1 = "n:Pgen:bus1", "MW, Generation, Active Power"
      n_Pgen_bus1n = "n:Pgen:bus1n", "MW, Generation, Active Power"
      n_Qgen_bus1 = "n:Qgen:bus1", "Mvar, Generation, Reactive Power"
      n_Qgen_bus1n = "n:Qgen:bus1n", "Mvar, Generation, Reactive Power"
      n_Pmot_bus1 = "n:Pmot:bus1", "MW, Motor Load, Active Power"
      n_Pmot_bus1n = "n:Pmot:bus1n", "MW, Motor Load, Active Power"
      n_Qmot_bus1 = "n:Qmot:bus1", "Mvar, Motor Load, Reactive Power"
      n_Qmot_bus1n = "n:Qmot:bus1n", "Mvar, Motor Load, Reactive Power"
      n_Pload_bus1 = "n:Pload:bus1", "MW, General Load, Active Power"
      n_Pload_bus1n = "n:Pload:bus1n", "MW, General Load, Active Power"
      n_Qload_bus1 = "n:Qload:bus1", "Mvar, General Load, Reactive Power"
      n_Qload_bus1n = "n:Qload:bus1n", "Mvar, General Load, Reactive Power"
      n_Pcomp_bus1 = "n:Pcomp:bus1", "MW, Compensation (Losses)"
      n_Pcomp_bus1n = "n:Pcomp:bus1n", "MW, Compensation (Losses)"
      n_Qcomp_bus1 = "n:Qcomp:bus1", "Mvar, Compensation"
      n_Qcomp_bus1n = "n:Qcomp:bus1n", "Mvar, Compensation"
      n_Pnet_bus1 = "n:Pnet:bus1", "MW, External Networks, Active Power"
      n_Pnet_bus1n = "n:Pnet:bus1n", "MW, External Networks, Active Power"
      n_Qnet_bus1 = "n:Qnet:bus1", "Mvar, External Networks, Reactive Power"
      n_Qnet_bus1n = "n:Qnet:bus1n", "Mvar, External Networks, Reactive Power"
      n_Pflow_bus1 = "n:Pflow:bus1", "MW, Power Flow, Active Power"
      n_Pflow_bus1n = "n:Pflow:bus1n", "MW, Power Flow, Active Power"
      n_Qflow_bus1 = "n:Qflow:bus1", "Mvar, Power Flow, Reactive Power"
      n_Qflow_bus1n = "n:Qflow:bus1n", "Mvar, Power Flow, Reactive Power"
      n_Pout_bus1 = "n:Pout:bus1", "MW, Outgoing Flow, Active Power"
      n_Pout_bus1n = "n:Pout:bus1n", "MW, Outgoing Flow, Active Power"
      n_Qout_bus1 = "n:Qout:bus1", "Mvar, Outgoing Power, Reactive Power"
      n_Qout_bus1n = "n:Qout:bus1n", "Mvar, Outgoing Power, Reactive Power"
      n_Sout_bus1 = "n:Sout:bus1", "MVA, Outgoing Power, Apparent Power"
      n_Sout_bus1n = "n:Sout:bus1n", "MVA, Outgoing Power, Apparent Power"
      n_cosphiout_bus1 = "n:cosphiout:bus1", "Outgoing Power, Power Factor"
      n_cosphiout_bus1n = "n:cosphiout:bus1n", "Outgoing Power, Power Factor"
      n_Pbalance_bus1 = "n:Pbalance:bus1", "MW, Active Power Balance (=0)"
      n_Pbalance_bus1n = "n:Pbalance:bus1n", "MW, Active Power Balance (=0)"
      n_Qbalance_bus1 = "n:Qbalance:bus1", "Mvar, Reactive Power Balance (=0)"
      n_Qbalance_bus1n = "n:Qbalance:bus1n", "Mvar, Reactive Power Balance (=0)"
      n_ubfac_bus1 = "n:ubfac:bus1", "%, Unbalance Factor"
      n_ubfac_bus1n = "n:ubfac:bus1n", "%, Unbalance Factor"
      n_LossPdown_bus1 = "n:LossPdown:bus1", "MW, Losses, downstream"
      n_LossPdown_bus1n = "n:LossPdown:bus1n", "MW, Losses, downstream"
      n_LossQdown_bus1 = "n:LossQdown:bus1", "Mvar, Losses, downstream (Reactive Power)"
      n_LossQdown_bus1n = "n:LossQdown:bus1n", "Mvar, Losses, downstream (Reactive Power)"
      n_LossPdownload_bus1 = "n:LossPdownload:bus1", "MW, Load losses, downstream"
      n_LossPdownload_bus1n = "n:LossPdownload:bus1n", "MW, Load losses, downstream"
      n_LossQdownload_bus1 = "n:LossQdownload:bus1", "Mvar, Load losses, downstream"
      n_LossQdownload_bus1n = "n:LossQdownload:bus1n", "Mvar, Load losses, downstream"
      n_LossPdownnoload_bus1 = "n:LossPdownnoload:bus1", "MW, No load losses, downstream"
      n_LossPdownnoload_bus1n = "n:LossPdownnoload:bus1n", "MW, No load losses, downstream"
      n_LossQdownnoload_bus1 = "n:LossQdownnoload:bus1", "Mvar, No load losses, downstream"
      n_LossQdownnoload_bus1n = "n:LossQdownnoload:bus1n", "Mvar, No load losses, downstream"
      s_pset = "s:pset", "MW, IN, Active Power Setpoint, continuous, double"
      s_slip = "s:slip", "%, OUT, Slip, continuous, double"
      s_rradd = "s:rradd", "p.u., OUT, Additional Rotor Resistance, continuous, double"
      c_cosn = "c:cosn", "Rated Power Factor"
      c_p = "c:p", "MW, Active Power (Electrical)"
      c_q = "c:q", "Mvar, Reactive Power"
      c_sgnn = "c:sgnn", "MVA, Total Nominal Apparent power (ngnum*sgn)"
      c_Pnom = "c:Pnom", "MW, Total Nominal Active power"
      c_pm = "c:pm", "MW, Mechanical Power"
      c_rstr = "c:rstr", "p.u., Stator Resistance"
      c_xstr = "c:xstr", "p.u., Stator Reactance"
      c_xrtrA = "c:xrtrA", "p.u., Rotor Reactance A"
      c_xrtrB = "c:xrtrB", "p.u., Rotor Reactance B"
      c_xm = "c:xm", "p.u., Magnetising Reactance"
      c_xmrtr = "c:xmrtr", "p.u., Rotor Leakage Reactance"
      c_r2 = "c:r2", "p.u., Negative Sequence Resistance"
      c_x2 = "c:x2", "p.u., Negative Sequence Reactance"
      c_frnom = "c:frnom", "Hz, Nominal Frequency"
      c_loading = "c:loading", "%, Loading"
      c_i_con = "c:i_con", "flag"
      c_i_slip = "c:i_slip", "flag"
      c_slipn = "c:slipn", "%, Nominal Slip"
      c_slipset = "c:slipset", "%, Slip Setpoint"
      c_i_mot = "c:i_mot", "i_mot=0"
      c_mk = "c:mk", "Stalling Torque"
      c_z0_r = "c:z0:r", "p.u., Zero Sequence Impedance, Real Part"
      c_z0_i = "c:z0:i", "p.u., Zero Sequence Impedance, Imaginary Part"
      c_z0_phi = "c:z0:phi", "deg, Zero Sequence Impedance, Angle"
      c_z0_mag = "c:z0:mag", "p.u., Zero Sequence Impedance, Magnitude"
      c_z0_phirad = "c:z0:phirad", "rad, Zero Sequence Impedance, Angle"
      c_ze_r = "c:ze:r", "p.u., Grounding Impedance, Real Part"
      c_ze_i = "c:ze:i", "p.u., Grounding Impedance, Imaginary Part"
      c_ze_phi = "c:ze:phi", "deg, Grounding Impedance, Angle"
      c_ze_mag = "c:ze:mag", "p.u., Grounding Impedance, Magnitude"
      c_ze_phirad = "c:ze:phirad", "rad, Grounding Impedance, Angle"
      c_xme = "c:xme", "p.u., Electrical Torque"
      c_pmset = "c:pmset", "MW, Mechanical Power Setpoint"
      c_xmtset = "c:xmtset", "p.u., Mechanical Torque Setpoint"
      c_rrtrA0 = "c:rrtrA0", "p.u., RrA0"
      c_xrtrA0 = "c:xrtrA0", "p.u., XrA0"
      c_rr0 = "c:rr0", "p.u., Rr0"
      c_xr0 = "c:xr0", "p.u., Xr0"
      c_rr1 = "c:rr1", "p.u., Rr1"
      c_xr1 = "c:xr1", "p.u., Xr1"
      c_qcomp = "c:qcomp", "Mvar, Reactive Power for Compensation"
      c_zonewscale = "c:zonewscale", "Wind generation zone scaling factor"
      c_psiM = "c:psiM", "p.u., Magnetising Flux"
      c_ksat = "c:ksat", "Saturation Factor"
      c_xmsat = "c:xmsat", "p.u., Saturated Magnetising Reactance"
      c_Ploss = "c:Ploss", "MW, Losses (total)"
      c_Qloss = "c:Qloss", "Mvar, Reactive-Losses (total)"
      c_Plossld = "c:Plossld", "MW, Losses (load)"
      c_Qlossld = "c:Qlossld", "Mvar, Reactive-Losses (load)"
      c_Plossnld = "c:Plossnld", "MW, Losses (no load)"
      c_Qlossnld = "c:Qlossnld", "Mvar, Reactive-Losses (no load)"
      e_bustp = "e:bustp", "Bus type"
      e_pmode = "e:pmode", "Input mode"
      e_av_mode = "e:av_mode", "Local controller"
      e_outServPzero = "e:outServPzero", "Out of service when active power is zero"
      e_c_pstac = "e:c_pstac", "ElmStactrl, External station controller"
      e_qdslCtrl = "e:qdslCtrl", "Quasi-Dynamic Model"
      e_mode_pgi = "e:mode_pgi", "Model"
      e_pPowerCrv = "e:pPowerCrv", "TypPowercurve, Wind Power Curve"
      e_mode_inp = "e:mode_inp", "Input mode"
      e_pgini = "e:pgini", "MW, Active power"
      e_qgini = "e:qgini", "Mvar, Reactive power"
      e_sgini = "e:sgini", "MVA, Apparent power"
      e_cosgini = "e:cosgini", "Power factor"
      e_pf_recap = "e:pf_recap", "Power factor"
      e_p_direc = "e:p_direc", "Power direction"
      e_windspeed = "e:windspeed", "m/s, Wind speed"
      e_usetp = "e:usetp", "p.u., Voltage"
      e_Kpf = "e:Kpf", "MW/Hz, Prim. frequency bias"
      e_pgini_a = "e:pgini_a", "MW, Active Power (act.)"
      e_qgini_a = "e:qgini_a", "Mvar, Reactive Power (act.)"
      e_sgini_a = "e:sgini_a", "MVA, Apparent Power (act.)"
      e_cosgini_a = "e:cosgini_a", "Power Factor (act.)"
      e_pf_recap_a = "e:pf_recap_a", "Power Factor Ind/Cap (act.)"
      e_windspeed_a = "e:windspeed_a", "m/s, Wind speed (act.)"
      e_usp_min = "e:usp_min", "p.u., Min. voltage setpoint"
      e_usp_max = "e:usp_max", "p.u., Max. voltage setpoint"
      e_ddroop = "e:ddroop", "%, Droop"
      e_pQPcurve = "e:pQPcurve", "IntQpcurve, Q(P)-curve"
      e_pmini = "e:pmini", "MW, Mechanical power"
      e_xmtini = "e:xmtini", "kNm, Mechanical torque"
      e_xmtn = "e:xmtn", "kNm, Rated mechanical torque"
      e_iopt_slip = "e:iopt_slip", "Slip Settings"
      e_slipset = "e:slipset", "%, Slip"
      e_chr_p = "e:chr_p", "MW, Active Power"
      e_chr_slip = "e:chr_slip", "%, slip"
      e_cOperSpeed = "e:cOperSpeed", "p.u., Operating speed"
      e_pQlimType = "e:pQlimType", "IntQlim, Capability curve"
      e_q_min = "e:q_min", "p.u., Min."
      e_q_max = "e:q_max", "p.u., Max."
      e_cQ_min = "e:cQ_min", "Mvar, Minimum"
      e_cQ_max = "e:cQ_max", "Mvar, Maximum"
      e_scaleQmin = "e:scaleQmin", "%, Scaling factor (min.)"
      e_scaleQmax = "e:scaleQmax", "%, Scaling factor (max.)"
      e_Pmin_uc = "e:Pmin_uc", "MW, Min."
      e_Pmin_ucPU = "e:Pmin_ucPU", "p.u., Min."
      e_Pmax_uc = "e:Pmax_uc", "MW, Max."
      e_Pmax_ucPU = "e:Pmax_ucPU", "p.u., Max."
      e_Pnom = "e:Pnom", "MW, Pr(rated)"
      e_P_max = "e:P_max", "MW, Max."
      e_pmaxratf = "e:pmaxratf", "Rating factor"
      e_is4SpinReserve = "e:is4SpinReserve", "Consider for region spinning reserve"
      e_genShiftKey = "e:genShiftKey", "%, Generation shift key"
      e_i_rem = "e:i_rem", "Remote control"
      e_p_cub = "e:p_cub", "StaCubic*, Controlled branch (Cubicle)"
      e_pBMU = "e:pBMU", "ElmBmu, Virtual Power Plant"
      e_isDispatchable = "e:isDispatchable", "Generator Dispatch"
      e_priority = "e:priority", "Merit Order"
      e_fixed = "e:fixed", "Must run"
      e_cpFeed = "e:cpFeed", "Feeder"
      e_ciDist = "e:ciDist", "Distance from infeed in number of buses"
      e_ciLater = "e:ciLater", "Lateral Index"
      e_ciDistRoot = "e:ciDistRoot", "Distance from first infeed in number of buses"
      e_ciDistAll = "e:ciDistAll", "Distance from infeed in number of buses including switches"
      e_ciDistAllRoot = "e:ciDistAllRoot", "Distance from first infeed in number of buses including switches"
      r_c_pstac = "r:c_pstac", "ElmStactrl, External station controller"
      r_qdslCtrl = "r:qdslCtrl", "Quasi-Dynamic Model"
      r_pPowerCrv = "r:pPowerCrv", "TypPowercurve, Wind Power Curve"
      r_pQPcurve = "r:pQPcurve", "IntQpcurve, Q(P)-curve"
      r_pQlimType = "r:pQlimType", "IntQlim, Capability curve"
      r_p_cub = "r:p_cub", "StaCubic*, Controlled branch (Cubicle)"
      r_pBMU = "r:pBMU", "ElmBmu, Virtual Power Plant"
      r_cpFeed = "r:cpFeed", "Feeder"
      m_Ikss_bus1 = "m:Ikss:bus1", "kA, Initial Short-Circuit Current"
      m_ikss_bus1 = "m:ikss:bus1", "p.u., Initial Short-Circuit Current, Magnitude"
      m_phii_bus1 = "m:phii:bus1", "deg, Phase Current, Angle"
      m_maxIkss_bus1 = "m:maxIkss:bus1", "kA, Max. Initial Short-Circuit Current"
      m_Skss_bus1 = "m:Skss:bus1", "MVA, Initial Short-Circuit Power"
      m_ip_bus1 = "m:ip:bus1", "kA, Peak Short-Circuit Current"
      m_maxip_bus1 = "m:maxip:bus1", "kA, Max. Peak Short-Circuit Current"
      m_maxIb_bus1 = "m:maxIb:bus1", "kA, Max. Breaking Current"
      m_maxIth_bus1 = "m:maxIth:bus1", "kA, Max. Thermal equivalent Short-Circuit Current"
      m_Ib_bus1 = "m:Ib:bus1", "kA, Short-Circuit Breaking Current"
      m_Sb_bus1 = "m:Sb:bus1", "MVA, Short-Circuit Breaking Power"
      m_Ik_bus1 = "m:Ik:bus1", "kA, Steady-State Short-Circuit Current"
      m_Ith_bus1 = "m:Ith:bus1", "kA, Thermal equivalent Short-Circuit Current"
      m_I_bus1 = "m:I:bus1", "kA, Short-Circuit Current"
      m_R_bus1 = "m:R:bus1", "Ohm, Fault Impedance, Real Part"
      m_X_bus1 = "m:X:bus1", "Ohm, Fault Impedance, Imaginary Part"
      m_Z_bus1 = "m:Z:bus1", "Ohm, Fault Impedance, Magnitude"
      m_phiz_bus1 = "m:phiz:bus1", "deg, Fault Impedance, Angle"
      m_zSbase_bus1 = "m:zSbase:bus1", "p.u., Fault Impedance, Magnitude"
      m_XtoR_bus1 = "m:XtoR:bus1", "X/R ratio"
      m_Zs_bus1 = "m:Zs:bus1", "Ohm, Source Impedance, Magnitude"
      m_phizs_bus1 = "m:phizs:bus1", "deg, Source Impedance, Angle"
      m_Rs_bus1 = "m:Rs:bus1", "Ohm, Source Impedance, Real Part"
      m_Xs_bus1 = "m:Xs:bus1", "Ohm, Source Impedance, Imaginary Part"
      m_phiui_bus1 = "m:phiui:bus1", "deg, Angle between Voltage and Current"
      m_cosphiui_bus1 = "m:cosphiui:bus1", "cosphi"
      m_P_bus1 = "m:P:bus1", "MW, Active Power"
      m_Q_bus1 = "m:Q:bus1", "Mvar, Reactive Power"
      m_BrkIpload_bus1 = "m:BrkIpload:bus1", "%, Breaker Loading (peak)"
      m_BrkIbload_bus1 = "m:BrkIbload:bus1", "%, Breaker Loading (breaking)"
      m_BrkmaxIb_bus1 = "m:BrkmaxIb:bus1", "kA, Max. Breaking Current (breaker time)"
      n_Ikss_bus1 = "n:Ikss:bus1", "kA, Initial Short-Circuit Current"
      n_I_bus1 = "n:I:bus1", "kA, Short-Circuit Current"
      n_phii_bus1 = "n:phii:bus1", "deg, Phase Current, Angle"
      n_Skss_bus1 = "n:Skss:bus1", "MVA, Initial Short-Circuit Power"
      n_ip_bus1 = "n:ip:bus1", "kA, Peak Short-Circuit Current"
      n_Ib_bus1 = "n:Ib:bus1", "kA, Short-Circuit Breaking Current"
      n_Sb_bus1 = "n:Sb:bus1", "MVA, Short-Circuit Breaking Power"
      n_Ik_bus1 = "n:Ik:bus1", "kA, Steady-State Short-Circuit Current"
      n_Ith_bus1 = "n:Ith:bus1", "kA, Thermal equivalent Short-Circuit Current"
      n_R_bus1 = "n:R:bus1", "Ohm, Short-Circuit Impedance, Real Part"
      n_X_bus1 = "n:X:bus1", "Ohm, Short-Circuit Impedance, Imaginary Part"
      n_Z_bus1 = "n:Z:bus1", "Ohm, Short-Circuit Impedance, Magnitude"
      n_phiz_bus1 = "n:phiz:bus1", "deg, Short-Circuit Impedance, Angle"
      n_rSbase_bus1 = "n:rSbase:bus1", "p.u., Short-Circuit Impedance, Real Part"
      n_xSbase_bus1 = "n:xSbase:bus1", "p.u., Short-Circuit Impedance, Imaginary Part"
      n_zSbase_bus1 = "n:zSbase:bus1", "p.u., Short-Circuit Impedance, Magnitude"
      n_XtoR_bus1 = "n:XtoR:bus1", "X/R ratio (peak)"
      n_RtoX_bus1 = "n:RtoX:bus1", "R/X ratio (peak)"
      n_ur_bus1 = "n:ur:bus1", "p.u., Voltage, Real Part"
      n_ui_bus1 = "n:ui:bus1", "p.u., Voltage, Imaginary Part"
      n_u_bus1 = "n:u:bus1", "p.u., Voltage, Magnitude"
      n_upc_bus1 = "n:upc:bus1", "%, Voltage, Magnitude"
      n_U_bus1 = "n:U:bus1", "kV, Line-Ground Voltage, Magnitude"
      n_Ul_bus1 = "n:Ul:bus1", "kV, Line-Line Voltage, Magnitude"
      n_phiu_bus1 = "n:phiu:bus1", "deg, Voltage, Angle"
      n_du_bus1 = "n:du:bus1", "%, Voltage Deviation"
      n_urpref_bus1 = "n:urpref:bus1", "p.u., Pre-fault Voltage, Real Part"
      n_uipref_bus1 = "n:uipref:bus1", "p.u., Pre-fault Voltage, Imaginary Part"
      n_upref_bus1 = "n:upref:bus1", "p.u., Pre-fault Voltage, Magnitude"
      n_uprefpc_bus1 = "n:uprefpc:bus1", "%, Pre-fault Voltage, Magnitude"
      n_Upref_bus1 = "n:Upref:bus1", "kV, Line-Ground Pre-fault Voltage, Magnitude"
      n_Ulpref_bus1 = "n:Ulpref:bus1", "kV, Line-Line Pre-fault Voltage, Magnitude"
      n_phiupref_bus1 = "n:phiupref:bus1", "deg, Pre-fault Voltage, Angle"
      n_phiui_bus1 = "n:phiui:bus1", "deg, Angle between Voltage and Current"
      n_cfac_bus1 = "n:cfac:bus1", "Voltage Factor"
      n_idc_bus1 = "n:idc:bus1", "kA, Decaying Component"
      n_Iasy_bus1 = "n:Iasy:bus1", "kA, Asymmetrical Breaking Current"
      n_Ithload_bus1 = "n:Ithload:bus1", "%, Loading, Thermal equivalent Current"
      n_Ipload_bus1 = "n:Ipload:bus1", "%, Loading, Peak Short-Circuit Current"
      n_Ithrtk_bus1 = "n:Ithrtk:bus1", "kA, Rated Short-Time Current (Tk)"
      n_Tn_bus1 = "n:Tn:bus1", "ms, Grid Time Constant"
      n_XtoR_b_bus1 = "n:XtoR_b:bus1", "X/R ratio (dc)"
      n_RtoX_b_bus1 = "n:RtoX_b:bus1", "R/X ratio (dc)"
      n_fctof_b_bus1 = "n:fctof_b:bus1", "Equivalent Frequency fc/f (dc)"
      c_z1_r = "c:z1:r", "p.u., Positive-sequence impedance, Real Part"
      c_z1_i = "c:z1:i", "p.u., Positive-sequence impedance, Imaginary Part"
      c_z1_phi = "c:z1:phi", "deg, Positive-sequence impedance, Angle"
      c_z1_mag = "c:z1:mag", "p.u., Positive-sequence impedance, Magnitude"
      c_z1_phirad = "c:z1:phirad", "rad, Positive-sequence impedance, Angle"
      c_r1 = "c:r1", "p.u., Positive-sequence resistance"
      c_x1 = "c:x1", "p.u., Positive-sequence reactance"
      c_uini_r = "c:uini:r", "p.u., Internal positive-sequence voltage, Real Part"
      c_uini_i = "c:uini:i", "p.u., Internal positive-sequence voltage, Imaginary Part"
      c_uini_phi = "c:uini:phi", "deg, Internal positive-sequence voltage, Angle"
      c_uini_mag = "c:uini:mag", "p.u., Internal positive-sequence voltage, Magnitude"
      c_uini_phirad = "c:uini:phirad", "rad, Internal positive-sequence voltage, Angle"
      c_Prm = "c:Prm", "MW, Rated Mechanical Power"
      e_iNoShcContr = "e:iNoShcContr", "No Short-Circuit Contribution"
      e_iconfed = "e:iconfed", "Static converter-fed drive"
      e_searchBlockTrf = "e:searchBlockTrf", "Externally modelled unit transformer"
      e_pblocktrf = "e:pblocktrf", "Unit transformer"
      e_iWDmax = "e:iWDmax", "kA, Max. instantaneous short-circuit current, iWDmax"
      e_kWD = "e:kWD", "Factor kappaWD"
      e_rxWD = "e:rxWD", "Ratio RWD/XWD"
      e_IkWDmax = "e:IkWDmax", "kA, Maximum current"
      e_IkWDmin = "e:IkWDmin", "kA, Minimum current"
      e_r2iec = "e:r2iec", "p.u., Resistance, r2"
      e_x2iec = "e:x2iec", "p.u., Reactance, x2"
      e_r0iec = "e:r0iec", "p.u., Resistance, r0"
      e_x0iec = "e:x0iec", "p.u., Reactance, x0"
      r_pblocktrf = "r:pblocktrf", "Unit transformer"
      m_Ikss_bus1_A = "m:Ikss:bus1:A", "kA, Initial Short-Circuit Current"
      m_Ikss_bus1_B = "m:Ikss:bus1:B", "kA, Initial Short-Circuit Current"
      m_Ikss_bus1_C = "m:Ikss:bus1:C", "kA, Initial Short-Circuit Current"
      m_Ikss_bus1_N = "m:Ikss:bus1:N", "kA, Initial Short-Circuit Current"
      m_Ikss_bus1n = "m:Ikss:bus1n", "kA, Initial Short-Circuit Current"
      m_ikss_bus1_A = "m:ikss:bus1:A", "p.u., Initial Short-Circuit Current, Magnitude"
      m_ikss_bus1_B = "m:ikss:bus1:B", "p.u., Initial Short-Circuit Current, Magnitude"
      m_ikss_bus1_C = "m:ikss:bus1:C", "p.u., Initial Short-Circuit Current, Magnitude"
      m_ikss_bus1_N = "m:ikss:bus1:N", "p.u., Initial Short-Circuit Current, Magnitude"
      m_ikss_bus1n = "m:ikss:bus1n", "p.u., Initial Short-Circuit Current, Magnitude"
      m_maxIkss_bus1n = "m:maxIkss:bus1n", "kA, Max. Initial Short-Circuit Current"
      m_Skss_bus1_A = "m:Skss:bus1:A", "MVA, Initial Short-Circuit Power"
      m_Skss_bus1_B = "m:Skss:bus1:B", "MVA, Initial Short-Circuit Power"
      m_Skss_bus1_C = "m:Skss:bus1:C", "MVA, Initial Short-Circuit Power"
      m_Skss_bus1_N = "m:Skss:bus1:N", "MVA, Initial Short-Circuit Power"
      m_Skss_bus1n = "m:Skss:bus1n", "MVA, Initial Short-Circuit Power"
      m_Skssx3_bus1_A = "m:Skssx3:bus1:A", "MVA, 3*Initial Short-Circuit Power"
      m_Skssx3_bus1_B = "m:Skssx3:bus1:B", "MVA, 3*Initial Short-Circuit Power"
      m_Skssx3_bus1_C = "m:Skssx3:bus1:C", "MVA, 3*Initial Short-Circuit Power"
      m_Skssx3_bus1_N = "m:Skssx3:bus1:N", "MVA, 3*Initial Short-Circuit Power"
      m_Skssx3_bus1n = "m:Skssx3:bus1n", "MVA, 3*Initial Short-Circuit Power"
      m_ip_bus1_A = "m:ip:bus1:A", "kA, Peak Short-Circuit Current"
      m_ip_bus1_B = "m:ip:bus1:B", "kA, Peak Short-Circuit Current"
      m_ip_bus1_C = "m:ip:bus1:C", "kA, Peak Short-Circuit Current"
      m_ip_bus1_N = "m:ip:bus1:N", "kA, Peak Short-Circuit Current"
      m_ip_bus1n = "m:ip:bus1n", "kA, Peak Short-Circuit Current"
      m_maxip_bus1n = "m:maxip:bus1n", "kA, Max. Peak Short-Circuit Current"
      m_maxIth_bus1n = "m:maxIth:bus1n", "kA, Max. Thermal equivalent Short-Circuit Current"
      m_Z_bus1_A = "m:Z:bus1:A", "Ohm, Fault Impedance, Magnitude"
      m_Z_bus1_B = "m:Z:bus1:B", "Ohm, Fault Impedance, Magnitude"
      m_Z_bus1_C = "m:Z:bus1:C", "Ohm, Fault Impedance, Magnitude"
      m_Z_bus1_N = "m:Z:bus1:N", "Ohm, Fault Impedance, Magnitude"
      m_Z_bus1n = "m:Z:bus1n", "Ohm, Fault Impedance, Magnitude"
      m_phiz_bus1_A = "m:phiz:bus1:A", "deg, Fault Impedance, Angle"
      m_phiz_bus1_B = "m:phiz:bus1:B", "deg, Fault Impedance, Angle"
      m_phiz_bus1_C = "m:phiz:bus1:C", "deg, Fault Impedance, Angle"
      m_phiz_bus1_N = "m:phiz:bus1:N", "deg, Fault Impedance, Angle"
      m_phiz_bus1n = "m:phiz:bus1n", "deg, Fault Impedance, Angle"
      m_Zl_bus1_A = "m:Zl:bus1:A", "Ohm, Line-Line Fault Impedance, Magnitude"
      m_Zl_bus1_B = "m:Zl:bus1:B", "Ohm, Line-Line Fault Impedance, Magnitude"
      m_Zl_bus1_C = "m:Zl:bus1:C", "Ohm, Line-Line Fault Impedance, Magnitude"
      m_Zl_bus1_N = "m:Zl:bus1:N", "Ohm, Line-Line Fault Impedance, Magnitude"
      m_Zl_bus1n = "m:Zl:bus1n", "Ohm, Line-Line Fault Impedance, Magnitude"
      m_phizl_bus1_A = "m:phizl:bus1:A", "deg, Line-Line Fault Impedance, Angle"
      m_phizl_bus1_B = "m:phizl:bus1:B", "deg, Line-Line Fault Impedance, Angle"
      m_phizl_bus1_C = "m:phizl:bus1:C", "deg, Line-Line Fault Impedance, Angle"
      m_phizl_bus1_N = "m:phizl:bus1:N", "deg, Line-Line Fault Impedance, Angle"
      m_phizl_bus1n = "m:phizl:bus1n", "deg, Line-Line Fault Impedance, Angle"
      m_Z0_bus1 = "m:Z0:bus1", "Ohm, Zero-Sequence Fault Impedance, Magnitude"
      m_Z0_bus1n = "m:Z0:bus1n", "Ohm, Zero-Sequence Fault Impedance, Magnitude"
      m_phiz0_bus1 = "m:phiz0:bus1", "deg, Zero-Sequence Fault Impedance, Angle"
      m_phiz0_bus1n = "m:phiz0:bus1n", "deg, Zero-Sequence Fault Impedance, Angle"
      m_z0Sbase_bus1 = "m:z0Sbase:bus1", "p.u., Zero-Sequence Fault Impedance, Magnitude"
      m_z0Sbase_bus1n = "m:z0Sbase:bus1n", "p.u., Zero-Sequence Fault Impedance, Magnitude"
      m_cosphiu1i1_bus1 = "m:cosphiu1i1:bus1", "Positive-Sequence cosphi"
      m_cosphiu1i1_bus1n = "m:cosphiu1i1:bus1n", "Positive-Sequence cosphi"
      m_I0x3watt_bus1 = "m:I0x3watt:bus1", "kA, Earth-Fault Residual Current"
      m_I0x3watt_bus1n = "m:I0x3watt:bus1n", "kA, Earth-Fault Residual Current"
      m_I0x3var_bus1 = "m:I0x3var:bus1", "kA, Capacitive Earth-Fault Current"
      m_I0x3var_bus1n = "m:I0x3var:bus1n", "kA, Capacitive Earth-Fault Current"
      m_Q0cap_bus1 = "m:Q0cap:bus1", "Mvar, Q0 capacitive"
      m_Q0cap_bus1n = "m:Q0cap:bus1n", "Mvar, Q0 capacitive"
      m_Zfrel_bus1 = "m:Zfrel:bus1", "Ohm, Relay Impedance, Magnitude"
      m_Zfrel_bus1n = "m:Zfrel:bus1n", "Ohm, Relay Impedance, Magnitude"
      m_phizfrel_bus1 = "m:phizfrel:bus1", "deg, Relay Impedance, Angle"
      m_phizfrel_bus1n = "m:phizfrel:bus1n", "deg, Relay Impedance, Angle"
      m_Rfrel_bus1 = "m:Rfrel:bus1", "Ohm, Relay Impedance, Real Part"
      m_Rfrel_bus1n = "m:Rfrel:bus1n", "Ohm, Relay Impedance, Real Part"
      m_Xfrel_bus1 = "m:Xfrel:bus1", "Ohm, Relay Impedance, Imaginary Part"
      m_Xfrel_bus1n = "m:Xfrel:bus1n", "Ohm, Relay Impedance, Imaginary Part"
      m_Zs_bus1n = "m:Zs:bus1n", "Ohm, Source Impedance, Magnitude"
      m_phizs_bus1n = "m:phizs:bus1n", "deg, Source Impedance, Angle"
      m_Rs_bus1n = "m:Rs:bus1n", "Ohm, Source Impedance, Real Part"
      m_Xs_bus1n = "m:Xs:bus1n", "Ohm, Source Impedance, Imaginary Part"
      m_Zf_bus1 = "m:Zf:bus1", "Ohm, Meas. Fault Impedance, Magnitude"
      m_Zf_bus1n = "m:Zf:bus1n", "Ohm, Meas. Fault Impedance, Magnitude"
      m_phizf_bus1 = "m:phizf:bus1", "deg, Meas. Fault Impedance, Angle"
      m_phizf_bus1n = "m:phizf:bus1n", "deg, Meas. Fault Impedance, Angle"
      m_BrkIpload_bus1n = "m:BrkIpload:bus1n", "%, Max. Breaker Loading (peak)"
      n_Ikss_bus1_A = "n:Ikss:bus1:A", "kA, Initial Short-Circuit Current"
      n_Ikss_bus1_B = "n:Ikss:bus1:B", "kA, Initial Short-Circuit Current"
      n_Ikss_bus1_C = "n:Ikss:bus1:C", "kA, Initial Short-Circuit Current"
      n_Ikss_bus1_N = "n:Ikss:bus1:N", "kA, Initial Short-Circuit Current"
      n_Ikss_bus1n = "n:Ikss:bus1n", "kA, Initial Short-Circuit Current"
      n_I_bus1_A = "n:I:bus1:A", "kA, Short-Circuit Current"
      n_I_bus1_B = "n:I:bus1:B", "kA, Short-Circuit Current"
      n_I_bus1_C = "n:I:bus1:C", "kA, Short-Circuit Current"
      n_I_bus1_N = "n:I:bus1:N", "kA, Short-Circuit Current"
      n_I_bus1n = "n:I:bus1n", "kA, Short-Circuit Current"
      n_phii_bus1_A = "n:phii:bus1:A", "deg, Phase Current, Angle"
      n_phii_bus1_B = "n:phii:bus1:B", "deg, Phase Current, Angle"
      n_phii_bus1_C = "n:phii:bus1:C", "deg, Phase Current, Angle"
      n_phii_bus1_N = "n:phii:bus1:N", "deg, Phase Current, Angle"
      n_phii_bus1n = "n:phii:bus1n", "deg, Phase Current, Angle"
      n_Skss_bus1_A = "n:Skss:bus1:A", "MVA, Initial Short-Circuit Power"
      n_Skss_bus1_B = "n:Skss:bus1:B", "MVA, Initial Short-Circuit Power"
      n_Skss_bus1_C = "n:Skss:bus1:C", "MVA, Initial Short-Circuit Power"
      n_Skss_bus1_N = "n:Skss:bus1:N", "MVA, Initial Short-Circuit Power"
      n_Skss_bus1n = "n:Skss:bus1n", "MVA, Initial Short-Circuit Power"
      n_Skssx3_bus1_A = "n:Skssx3:bus1:A", "MVA, 3*Initial Short-Circuit Power"
      n_Skssx3_bus1_B = "n:Skssx3:bus1:B", "MVA, 3*Initial Short-Circuit Power"
      n_Skssx3_bus1_C = "n:Skssx3:bus1:C", "MVA, 3*Initial Short-Circuit Power"
      n_Skssx3_bus1_N = "n:Skssx3:bus1:N", "MVA, 3*Initial Short-Circuit Power"
      n_Skssx3_bus1n = "n:Skssx3:bus1n", "MVA, 3*Initial Short-Circuit Power"
      n_ip_bus1_A = "n:ip:bus1:A", "kA, Peak Short-Circuit Current"
      n_ip_bus1_B = "n:ip:bus1:B", "kA, Peak Short-Circuit Current"
      n_ip_bus1_C = "n:ip:bus1:C", "kA, Peak Short-Circuit Current"
      n_ip_bus1_N = "n:ip:bus1:N", "kA, Peak Short-Circuit Current"
      n_ip_bus1n = "n:ip:bus1n", "kA, Peak Short-Circuit Current"
      n_Ib_bus1_A = "n:Ib:bus1:A", "kA, Short-Circuit Breaking Current"
      n_Ib_bus1_B = "n:Ib:bus1:B", "kA, Short-Circuit Breaking Current"
      n_Ib_bus1_C = "n:Ib:bus1:C", "kA, Short-Circuit Breaking Current"
      n_Ib_bus1_N = "n:Ib:bus1:N", "kA, Short-Circuit Breaking Current"
      n_Ib_bus1n = "n:Ib:bus1n", "kA, Short-Circuit Breaking Current"
      n_Sb_bus1_A = "n:Sb:bus1:A", "MVA, Short-Circuit Breaking Power"
      n_Sb_bus1_B = "n:Sb:bus1:B", "MVA, Short-Circuit Breaking Power"
      n_Sb_bus1_C = "n:Sb:bus1:C", "MVA, Short-Circuit Breaking Power"
      n_Sb_bus1_N = "n:Sb:bus1:N", "MVA, Short-Circuit Breaking Power"
      n_Sb_bus1n = "n:Sb:bus1n", "MVA, Short-Circuit Breaking Power"
      n_Ith_bus1_A = "n:Ith:bus1:A", "kA, Thermal equivalent Short-Circuit Current"
      n_Ith_bus1_B = "n:Ith:bus1:B", "kA, Thermal equivalent Short-Circuit Current"
      n_Ith_bus1_C = "n:Ith:bus1:C", "kA, Thermal equivalent Short-Circuit Current"
      n_Ith_bus1_N = "n:Ith:bus1:N", "kA, Thermal equivalent Short-Circuit Current"
      n_Ith_bus1n = "n:Ith:bus1n", "kA, Thermal equivalent Short-Circuit Current"
      n_Ithmax_bus1 = "n:Ithmax:bus1", "kA, Max. Thermal equivalent Short-Circuit Current"
      n_Ithmax_bus1n = "n:Ithmax:bus1n", "kA, Max. Thermal equivalent Short-Circuit Current"
      n_Ik_bus1_A = "n:Ik:bus1:A", "kA, Steady-State Short-Circuit Current"
      n_Ik_bus1_B = "n:Ik:bus1:B", "kA, Steady-State Short-Circuit Current"
      n_Ik_bus1_C = "n:Ik:bus1:C", "kA, Steady-State Short-Circuit Current"
      n_Ik_bus1_N = "n:Ik:bus1:N", "kA, Steady-State Short-Circuit Current"
      n_Ik_bus1n = "n:Ik:bus1n", "kA, Steady-State Short-Circuit Current"
      n_R0_bus1 = "n:R0:bus1", "Ohm, Zero-Sequence Impedance, Real Part"
      n_R0_bus1n = "n:R0:bus1n", "Ohm, Zero-Sequence Impedance, Real Part"
      n_X0_bus1 = "n:X0:bus1", "Ohm, Zero-Sequence Impedance, Imaginary Part"
      n_X0_bus1n = "n:X0:bus1n", "Ohm, Zero-Sequence Impedance, Imaginary Part"
      n_Z0_bus1 = "n:Z0:bus1", "Ohm, Zero-Sequence Impedance, Magnitude"
      n_Z0_bus1n = "n:Z0:bus1n", "Ohm, Zero-Sequence Impedance, Magnitude"
      n_phiz0_bus1 = "n:phiz0:bus1", "deg, Zero-Sequence Impedance, Angle"
      n_phiz0_bus1n = "n:phiz0:bus1n", "deg, Zero-Sequence Impedance, Angle"
      n_r0Sbase_bus1 = "n:r0Sbase:bus1", "p.u., Zero-Sequence Impedance, Real Part"
      n_r0Sbase_bus1n = "n:r0Sbase:bus1n", "p.u., Zero-Sequence Impedance, Real Part"
      n_x0Sbase_bus1 = "n:x0Sbase:bus1", "p.u., Zero-Sequence Impedance, Imaginary Part"
      n_x0Sbase_bus1n = "n:x0Sbase:bus1n", "p.u., Zero-Sequence Impedance, Imaginary Part"
      n_z0Sbase_bus1 = "n:z0Sbase:bus1", "p.u., Zero-Sequence Impedance, Magnitude"
      n_z0Sbase_bus1n = "n:z0Sbase:bus1n", "p.u., Zero-Sequence Impedance, Magnitude"
      n_R1_bus1 = "n:R1:bus1", "Ohm, Positive-Sequence Impedance, Real Part"
      n_R1_bus1n = "n:R1:bus1n", "Ohm, Positive-Sequence Impedance, Real Part"
      n_X1_bus1 = "n:X1:bus1", "Ohm, Positive-Sequence Impedance, Imaginary Part"
      n_X1_bus1n = "n:X1:bus1n", "Ohm, Positive-Sequence Impedance, Imaginary Part"
      n_Z1_bus1 = "n:Z1:bus1", "Ohm, Positive-Sequence Impedance, Magnitude"
      n_Z1_bus1n = "n:Z1:bus1n", "Ohm, Positive-Sequence Impedance, Magnitude"
      n_phiz1_bus1 = "n:phiz1:bus1", "deg, Positive-Sequence Impedance, Angle"
      n_phiz1_bus1n = "n:phiz1:bus1n", "deg, Positive-Sequence Impedance, Angle"
      n_r1Sbase_bus1 = "n:r1Sbase:bus1", "p.u., Positive-Sequence Impedance, Real Part"
      n_r1Sbase_bus1n = "n:r1Sbase:bus1n", "p.u., Positive-Sequence Impedance, Real Part"
      n_x1Sbase_bus1 = "n:x1Sbase:bus1", "p.u., Positive-Sequence Impedance, Imaginary Part"
      n_x1Sbase_bus1n = "n:x1Sbase:bus1n", "p.u., Positive-Sequence Impedance, Imaginary Part"
      n_z1Sbase_bus1 = "n:z1Sbase:bus1", "p.u., Positive-Sequence Impedance, Magnitude"
      n_z1Sbase_bus1n = "n:z1Sbase:bus1n", "p.u., Positive-Sequence Impedance, Magnitude"
      n_R2_bus1 = "n:R2:bus1", "Ohm, Negative-Sequence Impedance, Real Part"
      n_R2_bus1n = "n:R2:bus1n", "Ohm, Negative-Sequence Impedance, Real Part"
      n_X2_bus1 = "n:X2:bus1", "Ohm, Negative-Sequence Impedance, Imaginary Part"
      n_X2_bus1n = "n:X2:bus1n", "Ohm, Negative-Sequence Impedance, Imaginary Part"
      n_Z2_bus1 = "n:Z2:bus1", "Ohm, Negative-Sequence Impedance, Magnitude"
      n_Z2_bus1n = "n:Z2:bus1n", "Ohm, Negative-Sequence Impedance, Magnitude"
      n_phiz2_bus1 = "n:phiz2:bus1", "deg, Negative-Sequence Impedance, Angle"
      n_phiz2_bus1n = "n:phiz2:bus1n", "deg, Negative-Sequence Impedance, Angle"
      n_r2Sbase_bus1 = "n:r2Sbase:bus1", "p.u., Negative-Sequence Impedance, Real Part"
      n_r2Sbase_bus1n = "n:r2Sbase:bus1n", "p.u., Negative-Sequence Impedance, Real Part"
      n_x2Sbase_bus1 = "n:x2Sbase:bus1", "p.u., Negative-Sequence Impedance, Imaginary Part"
      n_x2Sbase_bus1n = "n:x2Sbase:bus1n", "p.u., Negative-Sequence Impedance, Imaginary Part"
      n_z2Sbase_bus1 = "n:z2Sbase:bus1", "p.u., Negative-Sequence Impedance, Magnitude"
      n_z2Sbase_bus1n = "n:z2Sbase:bus1n", "p.u., Negative-Sequence Impedance, Magnitude"
      n_R_bus1_A = "n:R:bus1:A", "Ohm, Network Resistance, Line-Ground"
      n_R_bus1_B = "n:R:bus1:B", "Ohm, Network Resistance, Line-Ground"
      n_R_bus1_C = "n:R:bus1:C", "Ohm, Network Resistance, Line-Ground"
      n_R_bus1_N = "n:R:bus1:N", "Ohm, Network Resistance, Line-Ground"
      n_R_bus1n = "n:R:bus1n", "Ohm, Network Resistance, Line-Ground"
      n_X_bus1_A = "n:X:bus1:A", "Ohm, Network Reactance, Line-Ground"
      n_X_bus1_B = "n:X:bus1:B", "Ohm, Network Reactance, Line-Ground"
      n_X_bus1_C = "n:X:bus1:C", "Ohm, Network Reactance, Line-Ground"
      n_X_bus1_N = "n:X:bus1:N", "Ohm, Network Reactance, Line-Ground"
      n_X_bus1n = "n:X:bus1n", "Ohm, Network Reactance, Line-Ground"
      n_Z_bus1_A = "n:Z:bus1:A", "Ohm, Network Impedance, Line-Ground"
      n_Z_bus1_B = "n:Z:bus1:B", "Ohm, Network Impedance, Line-Ground"
      n_Z_bus1_C = "n:Z:bus1:C", "Ohm, Network Impedance, Line-Ground"
      n_Z_bus1_N = "n:Z:bus1:N", "Ohm, Network Impedance, Line-Ground"
      n_Z_bus1n = "n:Z:bus1n", "Ohm, Network Impedance, Line-Ground"
      n_phiz_bus1_A = "n:phiz:bus1:A", "deg, Angle of Network Impedance, Line-Ground"
      n_phiz_bus1_B = "n:phiz:bus1:B", "deg, Angle of Network Impedance, Line-Ground"
      n_phiz_bus1_C = "n:phiz:bus1:C", "deg, Angle of Network Impedance, Line-Ground"
      n_phiz_bus1_N = "n:phiz:bus1:N", "deg, Angle of Network Impedance, Line-Ground"
      n_phiz_bus1n = "n:phiz:bus1n", "deg, Angle of Network Impedance, Line-Ground"
      n_Rl_bus1_A = "n:Rl:bus1:A", "Ohm, Network Resistance, Line-Line"
      n_Rl_bus1_B = "n:Rl:bus1:B", "Ohm, Network Resistance, Line-Line"
      n_Rl_bus1_C = "n:Rl:bus1:C", "Ohm, Network Resistance, Line-Line"
      n_Rl_bus1_N = "n:Rl:bus1:N", "Ohm, Network Resistance, Line-Line"
      n_Rl_bus1n = "n:Rl:bus1n", "Ohm, Network Resistance, Line-Line"
      n_Xl_bus1_A = "n:Xl:bus1:A", "Ohm, Network Reactance, Line-Line"
      n_Xl_bus1_B = "n:Xl:bus1:B", "Ohm, Network Reactance, Line-Line"
      n_Xl_bus1_C = "n:Xl:bus1:C", "Ohm, Network Reactance, Line-Line"
      n_Xl_bus1_N = "n:Xl:bus1:N", "Ohm, Network Reactance, Line-Line"
      n_Xl_bus1n = "n:Xl:bus1n", "Ohm, Network Reactance, Line-Line"
      n_Zl_bus1_A = "n:Zl:bus1:A", "Ohm, Network Impedance, Line-Line"
      n_Zl_bus1_B = "n:Zl:bus1:B", "Ohm, Network Impedance, Line-Line"
      n_Zl_bus1_C = "n:Zl:bus1:C", "Ohm, Network Impedance, Line-Line"
      n_Zl_bus1_N = "n:Zl:bus1:N", "Ohm, Network Impedance, Line-Line"
      n_Zl_bus1n = "n:Zl:bus1n", "Ohm, Network Impedance, Line-Line"
      n_phizl_bus1_A = "n:phizl:bus1:A", "deg, Angle of Network Impedance, Line-Line"
      n_phizl_bus1_B = "n:phizl:bus1:B", "deg, Angle of Network Impedance, Line-Line"
      n_phizl_bus1_C = "n:phizl:bus1:C", "deg, Angle of Network Impedance, Line-Line"
      n_phizl_bus1_N = "n:phizl:bus1:N", "deg, Angle of Network Impedance, Line-Line"
      n_phizl_bus1n = "n:phizl:bus1n", "deg, Angle of Network Impedance, Line-Line"
      n_Rln_bus1_A = "n:Rln:bus1:A", "Ohm, Network Resistance, Line-Neutral"
      n_Rln_bus1_B = "n:Rln:bus1:B", "Ohm, Network Resistance, Line-Neutral"
      n_Rln_bus1_C = "n:Rln:bus1:C", "Ohm, Network Resistance, Line-Neutral"
      n_Rln_bus1_N = "n:Rln:bus1:N", "Ohm, Network Resistance, Line-Neutral"
      n_Rln_bus1n = "n:Rln:bus1n", "Ohm, Network Resistance, Line-Neutral"
      n_Xln_bus1_A = "n:Xln:bus1:A", "Ohm, Network Reactance, Line-Neutral"
      n_Xln_bus1_B = "n:Xln:bus1:B", "Ohm, Network Reactance, Line-Neutral"
      n_Xln_bus1_C = "n:Xln:bus1:C", "Ohm, Network Reactance, Line-Neutral"
      n_Xln_bus1_N = "n:Xln:bus1:N", "Ohm, Network Reactance, Line-Neutral"
      n_Xln_bus1n = "n:Xln:bus1n", "Ohm, Network Reactance, Line-Neutral"
      n_Zln_bus1_A = "n:Zln:bus1:A", "Ohm, Network Impedance, Line-Neutral"
      n_Zln_bus1_B = "n:Zln:bus1:B", "Ohm, Network Impedance, Line-Neutral"
      n_Zln_bus1_C = "n:Zln:bus1:C", "Ohm, Network Impedance, Line-Neutral"
      n_Zln_bus1_N = "n:Zln:bus1:N", "Ohm, Network Impedance, Line-Neutral"
      n_Zln_bus1n = "n:Zln:bus1n", "Ohm, Network Impedance, Line-Neutral"
      n_phizln_bus1_A = "n:phizln:bus1:A", "deg, Angle of Network Impedance, Line-Neutral"
      n_phizln_bus1_B = "n:phizln:bus1:B", "deg, Angle of Network Impedance, Line-Neutral"
      n_phizln_bus1_C = "n:phizln:bus1:C", "deg, Angle of Network Impedance, Line-Neutral"
      n_phizln_bus1_N = "n:phizln:bus1:N", "deg, Angle of Network Impedance, Line-Neutral"
      n_phizln_bus1n = "n:phizln:bus1n", "deg, Angle of Network Impedance, Line-Neutral"
      n_Rn_bus1 = "n:Rn:bus1", "Ohm, Network Resistance, Neutral"
      n_Rn_bus1n = "n:Rn:bus1n", "Ohm, Network Resistance, Neutral"
      n_Xn_bus1 = "n:Xn:bus1", "Ohm, Network Reactance, Neutral"
      n_Xn_bus1n = "n:Xn:bus1n", "Ohm, Network Reactance, Neutral"
      n_Zn_bus1 = "n:Zn:bus1", "Ohm, Network Impedance, Neutral"
      n_Zn_bus1n = "n:Zn:bus1n", "Ohm, Network Impedance, Neutral"
      n_phizn_bus1 = "n:phizn:bus1", "deg, Angle of Network Impedance, Neutral"
      n_phizn_bus1n = "n:phizn:bus1n", "deg, Angle of Network Impedance, Neutral"
      n_eff_bus1_A = "n:eff:bus1:A", "Earth Fault Factor"
      n_eff_bus1_B = "n:eff:bus1:B", "Earth Fault Factor"
      n_eff_bus1_C = "n:eff:bus1:C", "Earth Fault Factor"
      n_eff_bus1_N = "n:eff:bus1:N", "Earth Fault Factor"
      n_eff_bus1n = "n:eff:bus1n", "Earth Fault Factor"
      n_X0toX1_bus1 = "n:X0toX1:bus1", "X0/X1"
      n_X0toX1_bus1n = "n:X0toX1:bus1n", "X0/X1"
      n_in_bus1 = "n:in:bus1", "p.u., Neutral Current, Magnitude"
      n_in_bus1n = "n:in:bus1n", "p.u., Neutral Current, Magnitude"
      n_In_bus1 = "n:In:bus1", "kA, Neutral Current, Magnitude"
      n_In_bus1n = "n:In:bus1n", "kA, Neutral Current, Magnitude"
      n_phiin_bus1 = "n:phiin:bus1", "deg, Neutral Current, Angle"
      n_phiin_bus1n = "n:phiin:bus1n", "deg, Neutral Current, Angle"
      n_i0r_bus1 = "n:i0r:bus1", "p.u., Zero-Sequence Current, Real Part"
      n_i0r_bus1n = "n:i0r:bus1n", "p.u., Zero-Sequence Current, Real Part"
      n_i0i_bus1 = "n:i0i:bus1", "p.u., Zero-Sequence Current, Imaginary Part"
      n_i0i_bus1n = "n:i0i:bus1n", "p.u., Zero-Sequence Current, Imaginary Part"
      n_i0_bus1 = "n:i0:bus1", "p.u., Zero-Sequence Current, Magnitude"
      n_i0_bus1n = "n:i0:bus1n", "p.u., Zero-Sequence Current, Magnitude"
      n_I0_bus1 = "n:I0:bus1", "kA, Zero-Sequence Current, Magnitude"
      n_I0_bus1n = "n:I0:bus1n", "kA, Zero-Sequence Current, Magnitude"
      n_I0x3_bus1 = "n:I0x3:bus1", "kA, 3*I0"
      n_I0x3_bus1n = "n:I0x3:bus1n", "kA, 3*I0"
      n_phii0_bus1 = "n:phii0:bus1", "deg, Zero-Sequence Current, Angle"
      n_phii0_bus1n = "n:phii0:bus1n", "deg, Zero-Sequence Current, Angle"
      n_i1r_bus1 = "n:i1r:bus1", "p.u., Positive-Sequence Current, Real Part"
      n_i1r_bus1n = "n:i1r:bus1n", "p.u., Positive-Sequence Current, Real Part"
      n_i1i_bus1 = "n:i1i:bus1", "p.u., Positive-Sequence Current, Imaginary Part"
      n_i1i_bus1n = "n:i1i:bus1n", "p.u., Positive-Sequence Current, Imaginary Part"
      n_i1_bus1 = "n:i1:bus1", "p.u., Positive-Sequence Current, Magnitude"
      n_i1_bus1n = "n:i1:bus1n", "p.u., Positive-Sequence Current, Magnitude"
      n_I1_bus1 = "n:I1:bus1", "kA, Positive-Sequence Current, Magnitude"
      n_I1_bus1n = "n:I1:bus1n", "kA, Positive-Sequence Current, Magnitude"
      n_phii1_bus1 = "n:phii1:bus1", "deg, Positive-Sequence Current, Angle"
      n_phii1_bus1n = "n:phii1:bus1n", "deg, Positive-Sequence Current, Angle"
      n_i2r_bus1 = "n:i2r:bus1", "p.u., Negative-Sequence Current, Real Part"
      n_i2r_bus1n = "n:i2r:bus1n", "p.u., Negative-Sequence Current, Real Part"
      n_i2i_bus1 = "n:i2i:bus1", "p.u., Negative-Sequence Current, Imaginary Part"
      n_i2i_bus1n = "n:i2i:bus1n", "p.u., Negative-Sequence Current, Imaginary Part"
      n_i2_bus1 = "n:i2:bus1", "p.u., Negative-Sequence Current, Magnitude"
      n_i2_bus1n = "n:i2:bus1n", "p.u., Negative-Sequence Current, Magnitude"
      n_I2_bus1 = "n:I2:bus1", "kA, Negative-Sequence Current, Magnitude"
      n_I2_bus1n = "n:I2:bus1n", "kA, Negative-Sequence Current, Magnitude"
      n_phii2_bus1 = "n:phii2:bus1", "deg, Negative-Sequence Current, Angle"
      n_phii2_bus1n = "n:phii2:bus1n", "deg, Negative-Sequence Current, Angle"
      n_du_bus1_A = "n:du:bus1:A", "%, Voltage Deviation"
      n_du_bus1_B = "n:du:bus1:B", "%, Voltage Deviation"
      n_du_bus1_C = "n:du:bus1:C", "%, Voltage Deviation"
      n_du_bus1_N = "n:du:bus1:N", "%, Voltage Deviation"
      n_du_bus1n = "n:du:bus1n", "%, Voltage Deviation"
      n_urpref_bus1_A = "n:urpref:bus1:A", "p.u., Pre-fault Voltage, Real Part"
      n_urpref_bus1_B = "n:urpref:bus1:B", "p.u., Pre-fault Voltage, Real Part"
      n_urpref_bus1_C = "n:urpref:bus1:C", "p.u., Pre-fault Voltage, Real Part"
      n_urpref_bus1_N = "n:urpref:bus1:N", "p.u., Pre-fault Voltage, Real Part"
      n_urpref_bus1n = "n:urpref:bus1n", "p.u., Pre-fault Voltage, Real Part"
      n_uipref_bus1_A = "n:uipref:bus1:A", "p.u., Pre-fault Voltage, Imaginary Part"
      n_uipref_bus1_B = "n:uipref:bus1:B", "p.u., Pre-fault Voltage, Imaginary Part"
      n_uipref_bus1_C = "n:uipref:bus1:C", "p.u., Pre-fault Voltage, Imaginary Part"
      n_uipref_bus1_N = "n:uipref:bus1:N", "p.u., Pre-fault Voltage, Imaginary Part"
      n_uipref_bus1n = "n:uipref:bus1n", "p.u., Pre-fault Voltage, Imaginary Part"
      n_upref_bus1_A = "n:upref:bus1:A", "p.u., Pre-fault Voltage, Magnitude"
      n_upref_bus1_B = "n:upref:bus1:B", "p.u., Pre-fault Voltage, Magnitude"
      n_upref_bus1_C = "n:upref:bus1:C", "p.u., Pre-fault Voltage, Magnitude"
      n_upref_bus1_N = "n:upref:bus1:N", "p.u., Pre-fault Voltage, Magnitude"
      n_upref_bus1n = "n:upref:bus1n", "p.u., Pre-fault Voltage, Magnitude"
      n_uprefpc_bus1_A = "n:uprefpc:bus1:A", "%, Pre-fault Voltage, Magnitude"
      n_uprefpc_bus1_B = "n:uprefpc:bus1:B", "%, Pre-fault Voltage, Magnitude"
      n_uprefpc_bus1_C = "n:uprefpc:bus1:C", "%, Pre-fault Voltage, Magnitude"
      n_uprefpc_bus1_N = "n:uprefpc:bus1:N", "%, Pre-fault Voltage, Magnitude"
      n_uprefpc_bus1n = "n:uprefpc:bus1n", "%, Pre-fault Voltage, Magnitude"
      n_Upref_bus1_A = "n:Upref:bus1:A", "kV, Line-Ground Pre-fault Voltage, Magnitude"
      n_Upref_bus1_B = "n:Upref:bus1:B", "kV, Line-Ground Pre-fault Voltage, Magnitude"
      n_Upref_bus1_C = "n:Upref:bus1:C", "kV, Line-Ground Pre-fault Voltage, Magnitude"
      n_Upref_bus1_N = "n:Upref:bus1:N", "kV, Line-Ground Pre-fault Voltage, Magnitude"
      n_Upref_bus1n = "n:Upref:bus1n", "kV, Line-Ground Pre-fault Voltage, Magnitude"
      n_Ulpref_bus1_A = "n:Ulpref:bus1:A", "kV, Line-Line Pre-fault Voltage, Magnitude"
      n_Ulpref_bus1_B = "n:Ulpref:bus1:B", "kV, Line-Line Pre-fault Voltage, Magnitude"
      n_Ulpref_bus1_C = "n:Ulpref:bus1:C", "kV, Line-Line Pre-fault Voltage, Magnitude"
      n_Ulpref_bus1_N = "n:Ulpref:bus1:N", "kV, Line-Line Pre-fault Voltage, Magnitude"
      n_Ulpref_bus1n = "n:Ulpref:bus1n", "kV, Line-Line Pre-fault Voltage, Magnitude"
      n_phiupref_bus1_A = "n:phiupref:bus1:A", "deg, Pre-fault Voltage, Angle"
      n_phiupref_bus1_B = "n:phiupref:bus1:B", "deg, Pre-fault Voltage, Angle"
      n_phiupref_bus1_C = "n:phiupref:bus1:C", "deg, Pre-fault Voltage, Angle"
      n_phiupref_bus1_N = "n:phiupref:bus1:N", "deg, Pre-fault Voltage, Angle"
      n_phiupref_bus1n = "n:phiupref:bus1n", "deg, Pre-fault Voltage, Angle"
      n_phiui_bus1_A = "n:phiui:bus1:A", "deg, Angle between Voltage and Current"
      n_phiui_bus1_B = "n:phiui:bus1:B", "deg, Angle between Voltage and Current"
      n_phiui_bus1_C = "n:phiui:bus1:C", "deg, Angle between Voltage and Current"
      n_phiui_bus1_N = "n:phiui:bus1:N", "deg, Angle between Voltage and Current"
      n_phiui_bus1n = "n:phiui:bus1n", "deg, Angle between Voltage and Current"
      n_phiu1i1_bus1 = "n:phiu1i1:bus1", "deg, Angle between Voltage and Current in positive sequence system"
      n_phiu1i1_bus1n = "n:phiu1i1:bus1n", "deg, Angle between Voltage and Current in positive sequence system"
      n_phiu2i2_bus1 = "n:phiu2i2:bus1", "deg, Angle between Voltage and Current in negative sequence system"
      n_phiu2i2_bus1n = "n:phiu2i2:bus1n", "deg, Angle between Voltage and Current in negative sequence system"
      n_phiu0i0_bus1 = "n:phiu0i0:bus1", "deg, Angle between Voltage and Current in zero-sequence system"
      n_phiu0i0_bus1n = "n:phiu0i0:bus1n", "deg, Angle between Voltage and Current in zero-sequence system"
      n_Ithload_bus1n = "n:Ithload:bus1n", "%, Loading, Thermal equivalent Current"
      n_Ipload_bus1n = "n:Ipload:bus1n", "%, Loading, Peak Short-Circuit Current"
      n_Ithrtk_bus1n = "n:Ithrtk:bus1n", "kA, Rated Short-Time Current (Tk)"
      n_cfac_bus1n = "n:cfac:bus1n", "Voltage Factor"
      n_XtoR_bus1n = "n:XtoR:bus1n", "X/R ratio (peak)"
      n_RtoX_bus1n = "n:RtoX:bus1n", "R/X ratio (peak)"
      c_z2_r = "c:z2:r", "p.u., Negative-sequence impedance, Real Part"
      c_z2_i = "c:z2:i", "p.u., Negative-sequence impedance, Imaginary Part"
      c_z2_phi = "c:z2:phi", "deg, Negative-sequence impedance, Angle"
      c_z2_mag = "c:z2:mag", "p.u., Negative-sequence impedance, Magnitude"
      c_z2_phirad = "c:z2:phirad", "rad, Negative-sequence impedance, Angle"

    class ElmAsmsc(Enum):
      _init_ = 'value __doc__'

      m_u0_bus1 = "m:u0:bus1", "p.u., Zero-Sequence-Voltage, Magnitude"
      m_u0_bus2 = "m:u0:bus2", "p.u., Zero-Sequence-Voltage, Magnitude"
      m_phiu0_bus1 = "m:phiu0:bus1", "deg, Zero-Sequence-Voltage, Angle"
      m_phiu0_bus2 = "m:phiu0:bus2", "deg, Zero-Sequence-Voltage, Angle"
      m_u1r_bus1 = "m:u1r:bus1", "p.u., Positive-Sequence-Voltage, Real Part"
      m_u1r_bus2 = "m:u1r:bus2", "p.u., Positive-Sequence-Voltage, Real Part"
      m_u1i_bus1 = "m:u1i:bus1", "p.u., Positive-Sequence-Voltage, Imaginary Part"
      m_u1i_bus2 = "m:u1i:bus2", "p.u., Positive-Sequence-Voltage, Imaginary Part"
      m_u1_bus1 = "m:u1:bus1", "p.u., Positive-Sequence-Voltage, Magnitude"
      m_u1_bus2 = "m:u1:bus2", "p.u., Positive-Sequence-Voltage, Magnitude"
      m_U1_bus1 = "m:U1:bus1", "kV, Line-Ground Positive-Sequence-Voltage, Magnitude"
      m_U1_bus2 = "m:U1:bus2", "kV, Line-Ground Positive-Sequence-Voltage, Magnitude"
      m_U1l_bus1 = "m:U1l:bus1", "kV, Line-Line Positive-Sequence-Voltage, Magnitude"
      m_U1l_bus2 = "m:U1l:bus2", "kV, Line-Line Positive-Sequence-Voltage, Magnitude"
      m_phiu1_bus1 = "m:phiu1:bus1", "deg, Positive-Sequence-Voltage, Angle"
      m_phiu1_bus2 = "m:phiu1:bus2", "deg, Positive-Sequence-Voltage, Angle"
      m_u2_bus1 = "m:u2:bus1", "p.u., Negative-Sequence-Voltage, Magnitude"
      m_u2_bus2 = "m:u2:bus2", "p.u., Negative-Sequence-Voltage, Magnitude"
      m_phiu2_bus1 = "m:phiu2:bus1", "deg, Negative-Sequence-Voltage, Angle"
      m_phiu2_bus2 = "m:phiu2:bus2", "deg, Negative-Sequence-Voltage, Angle"
      m_inet_bus1_A = "m:inet:bus1:A", "p.u., Phase Current, Magnitude, referred to network"
      m_inet_bus1_B = "m:inet:bus1:B", "p.u., Phase Current, Magnitude, referred to network"
      m_inet_bus1_C = "m:inet:bus1:C", "p.u., Phase Current, Magnitude, referred to network"
      m_inet_bus2 = "m:inet:bus2", "p.u., Phase Current, Magnitude, referred to network"
      m_I_bus1_A = "m:I:bus1:A", "kA, Phase Current, Magnitude"
      m_I_bus1_B = "m:I:bus1:B", "kA, Phase Current, Magnitude"
      m_I_bus1_C = "m:I:bus1:C", "kA, Phase Current, Magnitude"
      m_I_bus2 = "m:I:bus2", "kA, Phase Current, Magnitude"
      m_phii_bus1_A = "m:phii:bus1:A", "deg, Phase Current, Angle"
      m_phii_bus1_B = "m:phii:bus1:B", "deg, Phase Current, Angle"
      m_phii_bus1_C = "m:phii:bus1:C", "deg, Phase Current, Angle"
      m_phii_bus2 = "m:phii:bus2", "deg, Phase Current, Angle"
      m_i0r_bus1 = "m:i0r:bus1", "p.u., Zero-Sequence Current, Real Part"
      m_i0r_bus2 = "m:i0r:bus2", "p.u., Zero-Sequence Current, Real Part"
      m_i0i_bus1 = "m:i0i:bus1", "p.u., Zero-Sequence Current, Imaginary Part"
      m_i0i_bus2 = "m:i0i:bus2", "p.u., Zero-Sequence Current, Imaginary Part"
      m_i0_bus1 = "m:i0:bus1", "p.u., Zero-Sequence Current, Magnitude"
      m_i0_bus2 = "m:i0:bus2", "p.u., Zero-Sequence Current, Magnitude"
      m_I0_bus1 = "m:I0:bus1", "kA, Zero-Sequence Current, Magnitude"
      m_I0_bus2 = "m:I0:bus2", "kA, Zero-Sequence Current, Magnitude"
      m_I0x3_bus1 = "m:I0x3:bus1", "kA, 3*I0"
      m_I0x3_bus2 = "m:I0x3:bus2", "kA, 3*I0"
      m_phii0_bus1 = "m:phii0:bus1", "deg, Zero-Sequence Current, Angle"
      m_phii0_bus2 = "m:phii0:bus2", "deg, Zero-Sequence Current, Angle"
      m_i1r_bus1 = "m:i1r:bus1", "p.u., Positive-Sequence Current, Real Part"
      m_i1r_bus2 = "m:i1r:bus2", "p.u., Positive-Sequence Current, Real Part"
      m_i1i_bus1 = "m:i1i:bus1", "p.u., Positive-Sequence Current, Imaginary Part"
      m_i1i_bus2 = "m:i1i:bus2", "p.u., Positive-Sequence Current, Imaginary Part"
      m_i1_bus1 = "m:i1:bus1", "p.u., Positive-Sequence Current, Magnitude"
      m_i1_bus2 = "m:i1:bus2", "p.u., Positive-Sequence Current, Magnitude"
      m_I1_bus1 = "m:I1:bus1", "kA, Positive-Sequence Current, Magnitude"
      m_I1_bus2 = "m:I1:bus2", "kA, Positive-Sequence Current, Magnitude"
      m_phii1_bus1 = "m:phii1:bus1", "deg, Positive-Sequence Current, Angle"
      m_phii1_bus2 = "m:phii1:bus2", "deg, Positive-Sequence Current, Angle"
      m_i2r_bus1 = "m:i2r:bus1", "p.u., Negative-Sequence Current, Real Part"
      m_i2r_bus2 = "m:i2r:bus2", "p.u., Negative-Sequence Current, Real Part"
      m_i2i_bus1 = "m:i2i:bus1", "p.u., Negative-Sequence Current, Imaginary Part"
      m_i2i_bus2 = "m:i2i:bus2", "p.u., Negative-Sequence Current, Imaginary Part"
      m_i2_bus1 = "m:i2:bus1", "p.u., Negative-Sequence Current, Magnitude"
      m_i2_bus2 = "m:i2:bus2", "p.u., Negative-Sequence Current, Magnitude"
      m_I2_bus1 = "m:I2:bus1", "kA, Negative-Sequence Current, Magnitude"
      m_I2_bus2 = "m:I2:bus2", "kA, Negative-Sequence Current, Magnitude"
      m_phii2_bus1 = "m:phii2:bus1", "deg, Negative-Sequence Current, Angle"
      m_phii2_bus2 = "m:phii2:bus2", "deg, Negative-Sequence Current, Angle"
      m_ubfacI_bus1 = "m:ubfacI:bus1", "%, Current Unbalance Factor"
      m_ubfacI_bus2 = "m:ubfacI:bus2", "%, Current Unbalance Factor"
      m_ubfacS_bus1 = "m:ubfacS:bus1", "%, Power Unbalance Factor"
      m_ubfacS_bus2 = "m:ubfacS:bus2", "%, Power Unbalance Factor"
      m_P_bus1_A = "m:P:bus1:A", "MW, Active Power"
      m_P_bus1_B = "m:P:bus1:B", "MW, Active Power"
      m_P_bus1_C = "m:P:bus1:C", "MW, Active Power"
      m_P_bus2 = "m:P:bus2", "MW, Active Power"
      m_Q_bus1_A = "m:Q:bus1:A", "Mvar, Reactive Power"
      m_Q_bus1_B = "m:Q:bus1:B", "Mvar, Reactive Power"
      m_Q_bus1_C = "m:Q:bus1:C", "Mvar, Reactive Power"
      m_Q_bus2 = "m:Q:bus2", "Mvar, Reactive Power"
      m_S_bus1_A = "m:S:bus1:A", "MVA, Apparent Power"
      m_S_bus1_B = "m:S:bus1:B", "MVA, Apparent Power"
      m_S_bus1_C = "m:S:bus1:C", "MVA, Apparent Power"
      m_S_bus2 = "m:S:bus2", "MVA, Apparent Power"
      m_cosphi_bus1_A = "m:cosphi:bus1:A", "Power Factor"
      m_cosphi_bus1_B = "m:cosphi:bus1:B", "Power Factor"
      m_cosphi_bus1_C = "m:cosphi:bus1:C", "Power Factor"
      m_cosphi_bus2 = "m:cosphi:bus2", "Power Factor"
      m_tanphi_bus1_A = "m:tanphi:bus1:A", "tan(phi)"
      m_tanphi_bus1_B = "m:tanphi:bus1:B", "tan(phi)"
      m_tanphi_bus1_C = "m:tanphi:bus1:C", "tan(phi)"
      m_tanphi_bus2 = "m:tanphi:bus2", "tan(phi)"
      m_Psum_bus1 = "m:Psum:bus1", "MW, Total Active Power"
      m_Psum_bus2 = "m:Psum:bus2", "MW, Total Active Power"
      m_Qsum_bus1 = "m:Qsum:bus1", "Mvar, Total Reactive Power"
      m_Qsum_bus2 = "m:Qsum:bus2", "Mvar, Total Reactive Power"
      m_Ssum_bus1 = "m:Ssum:bus1", "MVA, Total Apparent Power"
      m_Ssum_bus2 = "m:Ssum:bus2", "MVA, Total Apparent Power"
      m_cosphisum_bus1 = "m:cosphisum:bus1", "Total Power Factor"
      m_cosphisum_bus2 = "m:cosphisum:bus2", "Total Power Factor"
      m_tanphisum_bus1 = "m:tanphisum:bus1", "Total tan(phi)"
      m_tanphisum_bus2 = "m:tanphisum:bus2", "Total tan(phi)"
      m_Pln_bus1_A = "m:Pln:bus1:A", "MW, Active Power (Line-Neutral)"
      m_Pln_bus1_B = "m:Pln:bus1:B", "MW, Active Power (Line-Neutral)"
      m_Pln_bus1_C = "m:Pln:bus1:C", "MW, Active Power (Line-Neutral)"
      m_Pln_bus2 = "m:Pln:bus2", "MW, Active Power (Line-Neutral)"
      m_Qln_bus1_A = "m:Qln:bus1:A", "Mvar, Reactive Power (Line-Neutral)"
      m_Qln_bus1_B = "m:Qln:bus1:B", "Mvar, Reactive Power (Line-Neutral)"
      m_Qln_bus1_C = "m:Qln:bus1:C", "Mvar, Reactive Power (Line-Neutral)"
      m_Qln_bus2 = "m:Qln:bus2", "Mvar, Reactive Power (Line-Neutral)"
      m_P1_bus1 = "m:P1:bus1", "MW, Positive Sequence Active Power"
      m_P1_bus2 = "m:P1:bus2", "MW, Positive Sequence Active Power"
      m_Q1_bus1 = "m:Q1:bus1", "Mvar, Positive Sequence Reactive Power"
      m_Q1_bus2 = "m:Q1:bus2", "Mvar, Positive Sequence Reactive Power"
      m_P2_bus1 = "m:P2:bus1", "MW, Negative Sequence Active Power"
      m_P2_bus2 = "m:P2:bus2", "MW, Negative Sequence Active Power"
      m_Q2_bus1 = "m:Q2:bus1", "Mvar, Negative Sequence Reactive Power"
      m_Q2_bus2 = "m:Q2:bus2", "Mvar, Negative Sequence Reactive Power"
      m_P0_bus1 = "m:P0:bus1", "MW, Zero Sequence Active Power"
      m_P0_bus2 = "m:P0:bus2", "MW, Zero Sequence Active Power"
      m_Q0_bus1 = "m:Q0:bus1", "Mvar, Zero Sequence Reactive Power"
      m_Q0_bus2 = "m:Q0:bus2", "Mvar, Zero Sequence Reactive Power"
      m_i1P_bus1 = "m:i1P:bus1", "p.u., Positive-Sequence Active Current"
      m_i1P_bus2 = "m:i1P:bus2", "p.u., Positive-Sequence Active Current"
      m_i1Q_bus1 = "m:i1Q:bus1", "p.u., Positive-Sequence Reactive Current"
      m_i1Q_bus2 = "m:i1Q:bus2", "p.u., Positive-Sequence Reactive Current"
      m_I1P_bus1 = "m:I1P:bus1", "kA, Positive-Sequence Active Current"
      m_I1P_bus2 = "m:I1P:bus2", "kA, Positive-Sequence Active Current"
      m_I1Q_bus1 = "m:I1Q:bus1", "kA, Positive-Sequence Reactive Current"
      m_I1Q_bus2 = "m:I1Q:bus2", "kA, Positive-Sequence Reactive Current"
      m_i2P_bus1 = "m:i2P:bus1", "p.u., Negative-Sequence Active Current"
      m_i2P_bus2 = "m:i2P:bus2", "p.u., Negative-Sequence Active Current"
      m_i2Q_bus1 = "m:i2Q:bus1", "p.u., Negative-Sequence Reactive Current"
      m_i2Q_bus2 = "m:i2Q:bus2", "p.u., Negative-Sequence Reactive Current"
      m_I2P_bus1 = "m:I2P:bus1", "kA, Negative-Sequence Active Current"
      m_I2P_bus2 = "m:I2P:bus2", "kA, Negative-Sequence Active Current"
      m_I2Q_bus1 = "m:I2Q:bus1", "kA, Negative-Sequence Reactive Current"
      m_I2Q_bus2 = "m:I2Q:bus2", "kA, Negative-Sequence Reactive Current"
      m_ur_bus1_A = "m:ur:bus1:A", "p.u., Phase Voltage, Real Part"
      m_ur_bus1_B = "m:ur:bus1:B", "p.u., Phase Voltage, Real Part"
      m_ur_bus1_C = "m:ur:bus1:C", "p.u., Phase Voltage, Real Part"
      m_ur_bus2 = "m:ur:bus2", "p.u., Phase Voltage, Real Part"
      m_ui_bus1_A = "m:ui:bus1:A", "p.u., Phase Voltage, Imaginary Part"
      m_ui_bus1_B = "m:ui:bus1:B", "p.u., Phase Voltage, Imaginary Part"
      m_ui_bus1_C = "m:ui:bus1:C", "p.u., Phase Voltage, Imaginary Part"
      m_ui_bus2 = "m:ui:bus2", "p.u., Phase Voltage, Imaginary Part"
      m_u_bus1_A = "m:u:bus1:A", "p.u., Phase Voltage, Magnitude"
      m_u_bus1_B = "m:u:bus1:B", "p.u., Phase Voltage, Magnitude"
      m_u_bus1_C = "m:u:bus1:C", "p.u., Phase Voltage, Magnitude"
      m_u_bus2 = "m:u:bus2", "p.u., Phase Voltage, Magnitude"
      m_ir_bus1_A = "m:ir:bus1:A", "p.u., Phase Current, Real Part"
      m_ir_bus1_B = "m:ir:bus1:B", "p.u., Phase Current, Real Part"
      m_ir_bus1_C = "m:ir:bus1:C", "p.u., Phase Current, Real Part"
      m_ir_bus2 = "m:ir:bus2", "p.u., Phase Current, Real Part"
      m_ii_bus1_A = "m:ii:bus1:A", "p.u., Phase Current, Imaginary Part"
      m_ii_bus1_B = "m:ii:bus1:B", "p.u., Phase Current, Imaginary Part"
      m_ii_bus1_C = "m:ii:bus1:C", "p.u., Phase Current, Imaginary Part"
      m_ii_bus2 = "m:ii:bus2", "p.u., Phase Current, Imaginary Part"
      m_i_bus1_A = "m:i:bus1:A", "p.u., Phase Current, Magnitude"
      m_i_bus1_B = "m:i:bus1:B", "p.u., Phase Current, Magnitude"
      m_i_bus1_C = "m:i:bus1:C", "p.u., Phase Current, Magnitude"
      m_i_bus2 = "m:i:bus2", "p.u., Phase Current, Magnitude"
      m_phiui_bus1_A = "m:phiui:bus1:A", "deg, Angle between Voltage and Current"
      m_phiui_bus1_B = "m:phiui:bus1:B", "deg, Angle between Voltage and Current"
      m_phiui_bus1_C = "m:phiui:bus1:C", "deg, Angle between Voltage and Current"
      m_phiui_bus2 = "m:phiui:bus2", "deg, Angle between Voltage and Current"
      m_phiu0i0_bus1 = "m:phiu0i0:bus1", "deg, Angle between Voltage and Current in zero-sequence system"
      m_phiu0i0_bus2 = "m:phiu0i0:bus2", "deg, Angle between Voltage and Current in zero-sequence system"
      m_phiu1i1_bus1 = "m:phiu1i1:bus1", "deg, Angle between Voltage and Current in positive sequence system"
      m_phiu1i1_bus2 = "m:phiu1i1:bus2", "deg, Angle between Voltage and Current in positive sequence system"
      m_phiu2i2_bus1 = "m:phiu2i2:bus1", "deg, Angle between Voltage and Current in negative sequence system"
      m_phiu2i2_bus2 = "m:phiu2i2:bus2", "deg, Angle between Voltage and Current in negative sequence system"
      m_Tfct_bus1 = "m:Tfct:bus1", "s, Fault Clearing Time"
      m_Tfct_bus2 = "m:Tfct:bus2", "s, Fault Clearing Time"
      m_TfctPh_bus1_A = "m:TfctPh:bus1:A", "s, Fault Clearing Time"
      m_TfctPh_bus1_B = "m:TfctPh:bus1:B", "s, Fault Clearing Time"
      m_TfctPh_bus1_C = "m:TfctPh:bus1:C", "s, Fault Clearing Time"
      m_TfctPh_bus2 = "m:TfctPh:bus2", "s, Fault Clearing Time"
      m_Brkload_bus1 = "m:Brkload:bus1", "%, Max. Breaker Loading"
      m_Brkload_bus2 = "m:Brkload:bus2", "%, Max. Breaker Loading"
      m_BrkloadPh_bus1_A = "m:BrkloadPh:bus1:A", "%, Breaker Loading"
      m_BrkloadPh_bus1_B = "m:BrkloadPh:bus1:B", "%, Breaker Loading"
      m_BrkloadPh_bus1_C = "m:BrkloadPh:bus1:C", "%, Breaker Loading"
      m_BrkloadPh_bus2 = "m:BrkloadPh:bus2", "%, Breaker Loading"
      n_ur_bus1_A = "n:ur:bus1:A", "p.u., Line-Ground Voltage, Real Part"
      n_ur_bus1_B = "n:ur:bus1:B", "p.u., Line-Ground Voltage, Real Part"
      n_ur_bus1_C = "n:ur:bus1:C", "p.u., Line-Ground Voltage, Real Part"
      n_ur_bus2 = "n:ur:bus2", "p.u., Line-Ground Voltage, Real Part"
      n_ui_bus1_A = "n:ui:bus1:A", "p.u., Line-Ground Voltage, Imaginary Part"
      n_ui_bus1_B = "n:ui:bus1:B", "p.u., Line-Ground Voltage, Imaginary Part"
      n_ui_bus1_C = "n:ui:bus1:C", "p.u., Line-Ground Voltage, Imaginary Part"
      n_ui_bus2 = "n:ui:bus2", "p.u., Line-Ground Voltage, Imaginary Part"
      n_u_bus1_A = "n:u:bus1:A", "p.u., Line-Ground Voltage, Magnitude"
      n_u_bus1_B = "n:u:bus1:B", "p.u., Line-Ground Voltage, Magnitude"
      n_u_bus1_C = "n:u:bus1:C", "p.u., Line-Ground Voltage, Magnitude"
      n_u_bus2 = "n:u:bus2", "p.u., Line-Ground Voltage, Magnitude"
      n_upc_bus1_A = "n:upc:bus1:A", "%, Line-Ground Voltage, Magnitude"
      n_upc_bus1_B = "n:upc:bus1:B", "%, Line-Ground Voltage, Magnitude"
      n_upc_bus1_C = "n:upc:bus1:C", "%, Line-Ground Voltage, Magnitude"
      n_upc_bus2 = "n:upc:bus2", "%, Line-Ground Voltage, Magnitude"
      n_U_bus1_A = "n:U:bus1:A", "kV, Line-Ground Voltage, Magnitude"
      n_U_bus1_B = "n:U:bus1:B", "kV, Line-Ground Voltage, Magnitude"
      n_U_bus1_C = "n:U:bus1:C", "kV, Line-Ground Voltage, Magnitude"
      n_U_bus2 = "n:U:bus2", "kV, Line-Ground Voltage, Magnitude"
      n_phiu_bus1_A = "n:phiu:bus1:A", "deg, Line-Ground Voltage, Angle"
      n_phiu_bus1_B = "n:phiu:bus1:B", "deg, Line-Ground Voltage, Angle"
      n_phiu_bus1_C = "n:phiu:bus1:C", "deg, Line-Ground Voltage, Angle"
      n_phiu_bus2 = "n:phiu:bus2", "deg, Line-Ground Voltage, Angle"
      n_phiurel_bus1 = "n:phiurel:bus1", "deg, Voltage, Relative Angle"
      n_phiurel_bus2 = "n:phiurel:bus2", "deg, Voltage, Relative Angle"
      n_ul_bus1_A = "n:ul:bus1:A", "p.u., Line-Line Voltage, Magnitude"
      n_ul_bus1_B = "n:ul:bus1:B", "p.u., Line-Line Voltage, Magnitude"
      n_ul_bus1_C = "n:ul:bus1:C", "p.u., Line-Line Voltage, Magnitude"
      n_ul_bus2 = "n:ul:bus2", "p.u., Line-Line Voltage, Magnitude"
      n_ulpc_bus1_A = "n:ulpc:bus1:A", "%, Line-Line Voltage, Magnitude"
      n_ulpc_bus1_B = "n:ulpc:bus1:B", "%, Line-Line Voltage, Magnitude"
      n_ulpc_bus1_C = "n:ulpc:bus1:C", "%, Line-Line Voltage, Magnitude"
      n_ulpc_bus2 = "n:ulpc:bus2", "%, Line-Line Voltage, Magnitude"
      n_Ul_bus1_A = "n:Ul:bus1:A", "kV, Line-Line Voltage, Magnitude"
      n_Ul_bus1_B = "n:Ul:bus1:B", "kV, Line-Line Voltage, Magnitude"
      n_Ul_bus1_C = "n:Ul:bus1:C", "kV, Line-Line Voltage, Magnitude"
      n_Ul_bus2 = "n:Ul:bus2", "kV, Line-Line Voltage, Magnitude"
      n_phiul_bus1_A = "n:phiul:bus1:A", "deg, Line-Line Voltage, Angle"
      n_phiul_bus1_B = "n:phiul:bus1:B", "deg, Line-Line Voltage, Angle"
      n_phiul_bus1_C = "n:phiul:bus1:C", "deg, Line-Line Voltage, Angle"
      n_phiul_bus2 = "n:phiul:bus2", "deg, Line-Line Voltage, Angle"
      n_u0_bus1 = "n:u0:bus1", "p.u., Zero-Sequence Voltage, Magnitude"
      n_u0_bus2 = "n:u0:bus2", "p.u., Zero-Sequence Voltage, Magnitude"
      n_U0_bus1 = "n:U0:bus1", "kV, Zero-Sequence Voltage, Magnitude"
      n_U0_bus2 = "n:U0:bus2", "kV, Zero-Sequence Voltage, Magnitude"
      n_U0x3_bus1 = "n:U0x3:bus1", "kV, 3*U0"
      n_U0x3_bus2 = "n:U0x3:bus2", "kV, 3*U0"
      n_phiu0_bus1 = "n:phiu0:bus1", "deg, Zero-Sequence Voltage, Angle"
      n_phiu0_bus2 = "n:phiu0:bus2", "deg, Zero-Sequence Voltage, Angle"
      n_u1_bus1 = "n:u1:bus1", "p.u., Positive-Sequence Voltage, Magnitude"
      n_u1_bus2 = "n:u1:bus2", "p.u., Positive-Sequence Voltage, Magnitude"
      n_u1pc_bus1 = "n:u1pc:bus1", "%, Positive-Sequence Voltage, Magnitude"
      n_u1pc_bus2 = "n:u1pc:bus2", "%, Positive-Sequence Voltage, Magnitude"
      n_u1r_bus1 = "n:u1r:bus1", "p.u., Positive-Sequence Voltage, Real Part"
      n_u1r_bus2 = "n:u1r:bus2", "p.u., Positive-Sequence Voltage, Real Part"
      n_u1i_bus1 = "n:u1i:bus1", "p.u., Positive-Sequence Voltage, Imaginary Part"
      n_u1i_bus2 = "n:u1i:bus2", "p.u., Positive-Sequence Voltage, Imaginary Part"
      n_U1_bus1 = "n:U1:bus1", "kV, Line-Ground Positive-Sequence Voltage, Magnitude"
      n_U1_bus2 = "n:U1:bus2", "kV, Line-Ground Positive-Sequence Voltage, Magnitude"
      n_phiu1_bus1 = "n:phiu1:bus1", "deg, Positive-Sequence Voltage, Angle"
      n_phiu1_bus2 = "n:phiu1:bus2", "deg, Positive-Sequence Voltage, Angle"
      n_u2_bus1 = "n:u2:bus1", "p.u., Negative-Sequence Voltage, Magnitude"
      n_u2_bus2 = "n:u2:bus2", "p.u., Negative-Sequence Voltage, Magnitude"
      n_U2_bus1 = "n:U2:bus1", "kV, Line-Ground Negative-Sequence Voltage, Magnitude"
      n_U2_bus2 = "n:U2:bus2", "kV, Line-Ground Negative-Sequence Voltage, Magnitude"
      n_phiu2_bus1 = "n:phiu2:bus1", "deg, Negative-Sequence Voltage, Angle"
      n_phiu2_bus2 = "n:phiu2:bus2", "deg, Negative-Sequence Voltage, Angle"
      n_U1l_bus1 = "n:U1l:bus1", "kV, Line-Line Positive-Sequence Voltage, Magnitude"
      n_U1l_bus2 = "n:U1l:bus2", "kV, Line-Line Positive-Sequence Voltage, Magnitude"
      n_U2l_bus1 = "n:U2l:bus1", "kV, Line-Line Negative-Sequence Voltage, Magnitude"
      n_U2l_bus2 = "n:U2l:bus2", "kV, Line-Line Negative-Sequence Voltage, Magnitude"
      n_uln_bus1_A = "n:uln:bus1:A", "p.u., Line-Neutral Voltage, Magnitude"
      n_uln_bus1_B = "n:uln:bus1:B", "p.u., Line-Neutral Voltage, Magnitude"
      n_uln_bus1_C = "n:uln:bus1:C", "p.u., Line-Neutral Voltage, Magnitude"
      n_uln_bus2 = "n:uln:bus2", "p.u., Line-Neutral Voltage, Magnitude"
      n_Uln_bus1_A = "n:Uln:bus1:A", "kV, Line-Neutral Voltage, Magnitude"
      n_Uln_bus1_B = "n:Uln:bus1:B", "kV, Line-Neutral Voltage, Magnitude"
      n_Uln_bus1_C = "n:Uln:bus1:C", "kV, Line-Neutral Voltage, Magnitude"
      n_Uln_bus2 = "n:Uln:bus2", "kV, Line-Neutral Voltage, Magnitude"
      n_phiuln_bus1_A = "n:phiuln:bus1:A", "deg, Line-Neutral Voltage, Angle"
      n_phiuln_bus1_B = "n:phiuln:bus1:B", "deg, Line-Neutral Voltage, Angle"
      n_phiuln_bus1_C = "n:phiuln:bus1:C", "deg, Line-Neutral Voltage, Angle"
      n_phiuln_bus2 = "n:phiuln:bus2", "deg, Line-Neutral Voltage, Angle"
      n_un_bus1 = "n:un:bus1", "p.u., Neutral-Ground Voltage, Magnitude"
      n_un_bus2 = "n:un:bus2", "p.u., Neutral-Ground Voltage, Magnitude"
      n_Un_bus1 = "n:Un:bus1", "kV, Neutral-Ground Voltage, Magnitude"
      n_Un_bus2 = "n:Un:bus2", "kV, Neutral-Ground Voltage, Magnitude"
      n_phiun_bus1 = "n:phiun:bus1", "deg, Neutral-Ground Voltage, Angle"
      n_phiun_bus2 = "n:phiun:bus2", "deg, Neutral-Ground Voltage, Angle"
      n_upht_bus1_A = "n:upht:bus1:A", "p.u., Phase Technology dependent Voltage, Magnitude"
      n_upht_bus1_B = "n:upht:bus1:B", "p.u., Phase Technology dependent Voltage, Magnitude"
      n_upht_bus1_C = "n:upht:bus1:C", "p.u., Phase Technology dependent Voltage, Magnitude"
      n_upht_bus2 = "n:upht:bus2", "p.u., Phase Technology dependent Voltage, Magnitude"
      n_Upht_bus1_A = "n:Upht:bus1:A", "kV, Phase Technology dependent Voltage, Magnitude"
      n_Upht_bus1_B = "n:Upht:bus1:B", "kV, Phase Technology dependent Voltage, Magnitude"
      n_Upht_bus1_C = "n:Upht:bus1:C", "kV, Phase Technology dependent Voltage, Magnitude"
      n_Upht_bus2 = "n:Upht:bus2", "kV, Phase Technology dependent Voltage, Magnitude"
      n_um_bus1 = "n:um:bus1", "p.u., Average-Voltage, Magnitude"
      n_um_bus2 = "n:um:bus2", "p.u., Average-Voltage, Magnitude"
      n_Um_bus1 = "n:Um:bus1", "kV, Average-Voltage, Magnitude"
      n_Um_bus2 = "n:Um:bus2", "kV, Average-Voltage, Magnitude"
      n_uphtmin_bus1 = "n:uphtmin:bus1", "p.u., Minimum of Phase Technology dependent Voltage, Magnitude"
      n_uphtmin_bus2 = "n:uphtmin:bus2", "p.u., Minimum of Phase Technology dependent Voltage, Magnitude"
      n_uphtmax_bus1 = "n:uphtmax:bus1", "p.u., Maximum of Phase Technology dependent Voltage, Magnitude"
      n_uphtmax_bus2 = "n:uphtmax:bus2", "p.u., Maximum of Phase Technology dependent Voltage, Magnitude"
      n_Pgen_bus1 = "n:Pgen:bus1", "MW, Generation, Active Power"
      n_Pgen_bus2 = "n:Pgen:bus2", "MW, Generation, Active Power"
      n_Qgen_bus1 = "n:Qgen:bus1", "Mvar, Generation, Reactive Power"
      n_Qgen_bus2 = "n:Qgen:bus2", "Mvar, Generation, Reactive Power"
      n_Pmot_bus1 = "n:Pmot:bus1", "MW, Motor Load, Active Power"
      n_Pmot_bus2 = "n:Pmot:bus2", "MW, Motor Load, Active Power"
      n_Qmot_bus1 = "n:Qmot:bus1", "Mvar, Motor Load, Reactive Power"
      n_Qmot_bus2 = "n:Qmot:bus2", "Mvar, Motor Load, Reactive Power"
      n_Pload_bus1 = "n:Pload:bus1", "MW, General Load, Active Power"
      n_Pload_bus2 = "n:Pload:bus2", "MW, General Load, Active Power"
      n_Qload_bus1 = "n:Qload:bus1", "Mvar, General Load, Reactive Power"
      n_Qload_bus2 = "n:Qload:bus2", "Mvar, General Load, Reactive Power"
      n_Pcomp_bus1 = "n:Pcomp:bus1", "MW, Compensation (Losses)"
      n_Pcomp_bus2 = "n:Pcomp:bus2", "MW, Compensation (Losses)"
      n_Qcomp_bus1 = "n:Qcomp:bus1", "Mvar, Compensation"
      n_Qcomp_bus2 = "n:Qcomp:bus2", "Mvar, Compensation"
      n_Pnet_bus1 = "n:Pnet:bus1", "MW, External Networks, Active Power"
      n_Pnet_bus2 = "n:Pnet:bus2", "MW, External Networks, Active Power"
      n_Qnet_bus1 = "n:Qnet:bus1", "Mvar, External Networks, Reactive Power"
      n_Qnet_bus2 = "n:Qnet:bus2", "Mvar, External Networks, Reactive Power"
      n_Pflow_bus1 = "n:Pflow:bus1", "MW, Power Flow, Active Power"
      n_Pflow_bus2 = "n:Pflow:bus2", "MW, Power Flow, Active Power"
      n_Qflow_bus1 = "n:Qflow:bus1", "Mvar, Power Flow, Reactive Power"
      n_Qflow_bus2 = "n:Qflow:bus2", "Mvar, Power Flow, Reactive Power"
      n_Pout_bus1 = "n:Pout:bus1", "MW, Outgoing Flow, Active Power"
      n_Pout_bus2 = "n:Pout:bus2", "MW, Outgoing Flow, Active Power"
      n_Qout_bus1 = "n:Qout:bus1", "Mvar, Outgoing Power, Reactive Power"
      n_Qout_bus2 = "n:Qout:bus2", "Mvar, Outgoing Power, Reactive Power"
      n_Sout_bus1 = "n:Sout:bus1", "MVA, Outgoing Power, Apparent Power"
      n_Sout_bus2 = "n:Sout:bus2", "MVA, Outgoing Power, Apparent Power"
      n_cosphiout_bus1 = "n:cosphiout:bus1", "Outgoing Power, Power Factor"
      n_cosphiout_bus2 = "n:cosphiout:bus2", "Outgoing Power, Power Factor"
      n_Pbalance_bus1 = "n:Pbalance:bus1", "MW, Active Power Balance (=0)"
      n_Pbalance_bus2 = "n:Pbalance:bus2", "MW, Active Power Balance (=0)"
      n_Qbalance_bus1 = "n:Qbalance:bus1", "Mvar, Reactive Power Balance (=0)"
      n_Qbalance_bus2 = "n:Qbalance:bus2", "Mvar, Reactive Power Balance (=0)"
      n_ubfac_bus1 = "n:ubfac:bus1", "%, Unbalance Factor"
      n_ubfac_bus2 = "n:ubfac:bus2", "%, Unbalance Factor"
      n_LossPdown_bus1 = "n:LossPdown:bus1", "MW, Losses, downstream"
      n_LossPdown_bus2 = "n:LossPdown:bus2", "MW, Losses, downstream"
      n_LossQdown_bus1 = "n:LossQdown:bus1", "Mvar, Losses, downstream (Reactive Power)"
      n_LossQdown_bus2 = "n:LossQdown:bus2", "Mvar, Losses, downstream (Reactive Power)"
      n_LossPdownload_bus1 = "n:LossPdownload:bus1", "MW, Load losses, downstream"
      n_LossPdownload_bus2 = "n:LossPdownload:bus2", "MW, Load losses, downstream"
      n_LossQdownload_bus1 = "n:LossQdownload:bus1", "Mvar, Load losses, downstream"
      n_LossQdownload_bus2 = "n:LossQdownload:bus2", "Mvar, Load losses, downstream"
      n_LossPdownnoload_bus1 = "n:LossPdownnoload:bus1", "MW, No load losses, downstream"
      n_LossPdownnoload_bus2 = "n:LossPdownnoload:bus2", "MW, No load losses, downstream"
      n_LossQdownnoload_bus1 = "n:LossQdownnoload:bus1", "Mvar, No load losses, downstream"
      n_LossQdownnoload_bus2 = "n:LossQdownnoload:bus2", "Mvar, No load losses, downstream"
      s_usr = "s:usr", "p.u., STATE, Slip-ring Voltage, Real Part, continuous, double"
      s_usi = "s:usi", "p.u., STATE, Slip-ring Voltage, Imaginary Part, continuous, double"
      c_cosn = "c:cosn", "Rated Power Factor"
      c_p = "c:p", "MW, Active Power (Electrical)"
      c_q = "c:q", "Mvar, Reactive Power"
      c_sgnn = "c:sgnn", "MVA, Total rated apparent power (ngnum*sgn)"
      c_Pnom = "c:Pnom", "MW, Total Nominal Active power"
      c_rstr = "c:rstr", "p.u., Stator Resistance"
      c_xstr = "c:xstr", "p.u., Stator Reactance"
      c_xrtrA = "c:xrtrA", "p.u., Rotor Reactance A"
      c_xrtrB = "c:xrtrB", "p.u., Rotor Reactance B"
      c_xm = "c:xm", "p.u., Magnetising Reactance"
      c_xmrtr = "c:xmrtr", "p.u., Rotor Leakage Reactance"
      c_r2 = "c:r2", "p.u., Negative Sequence Resistance"
      c_x2 = "c:x2", "p.u., Negative Sequence Reactance"
      c_frnom = "c:frnom", "Hz, Nominal Frequency"
      c_loading = "c:loading", "%, Loading"
      c_i_con = "c:i_con", "flag"
      c_i_slip = "c:i_slip", "flag"
      c_slipn = "c:slipn", "%, Nominal Slip"
      c_i_mot = "c:i_mot", "i_mot=0"
      c_mk = "c:mk", "Stalling Torque"
      c_xme = "c:xme", "p.u., Electrical Torque"
      c_i_cdisp = "c:i_cdisp", "Current Displacement"
      c_rrtrA0 = "c:rrtrA0", "p.u., RrA0"
      c_xrtrA0 = "c:xrtrA0", "p.u., XrA0"
      c_rr0 = "c:rr0", "p.u., Rr0"
      c_xr0 = "c:xr0", "p.u., Xr0"
      c_rr1 = "c:rr1", "p.u., Rr1"
      c_xr1 = "c:xr1", "p.u., Xr1"
      c_Pmr = "c:Pmr", "Amplification, Real Part"
      c_Pmi = "c:Pmi", "Amplification, Imaginary Part"
      c_slip = "c:slip", "%, Slip"
      c_z0_r = "c:z0:r", "p.u., Zero Sequence Impedance, Real Part"
      c_z0_i = "c:z0:i", "p.u., Zero Sequence Impedance, Imaginary Part"
      c_z0_phi = "c:z0:phi", "deg, Zero Sequence Impedance, Angle"
      c_z0_mag = "c:z0:mag", "p.u., Zero Sequence Impedance, Magnitude"
      c_z0_phirad = "c:z0:phirad", "rad, Zero Sequence Impedance, Angle"
      c_ze_r = "c:ze:r", "p.u., Grounding Impedance, Real Part"
      c_ze_i = "c:ze:i", "p.u., Grounding Impedance, Imaginary Part"
      c_ze_phi = "c:ze:phi", "deg, Grounding Impedance, Angle"
      c_ze_mag = "c:ze:mag", "p.u., Grounding Impedance, Magnitude"
      c_ze_phirad = "c:ze:phirad", "rad, Grounding Impedance, Angle"
      c_Unrot = "c:Unrot", "kV, Rated Slip Ring Voltage"
      c_zonewscale = "c:zonewscale", "Wind generation zone scaling factor"
      c_psiM = "c:psiM", "p.u., Magnetising Flux"
      c_ksat = "c:ksat", "Saturation Factor"
      c_xmsat = "c:xmsat", "p.u., Saturated Magnetising Reactance"
      c_Ploss = "c:Ploss", "MW, Losses (total)"
      c_Qloss = "c:Qloss", "Mvar, Reactive-Losses (total)"
      c_Plossld = "c:Plossld", "MW, Losses (load)"
      c_Qlossld = "c:Qlossld", "Mvar, Reactive-Losses (load)"
      c_Plossnld = "c:Plossnld", "MW, Losses (no load)"
      c_Qlossnld = "c:Qlossnld", "Mvar, Reactive-Losses (no load)"
      e_pgini = "e:pgini", "MW, Active Power"
      e_qgini = "e:qgini", "Mvar, Reactive Power"
      e_p_pctrl = "e:p_pctrl", "StaCubic*, Controlled Flow"
      e_slipset = "e:slipset", "%, Slip"
      e_Urot = "e:Urot", "V, Rated Slip Ring Voltage"
      e_cOperSpeed = "e:cOperSpeed", "p.u., Operating speed"
      e_cpFeed = "e:cpFeed", "Feeder"
      e_ciDist = "e:ciDist", "Distance from infeed in number of buses"
      e_ciLater = "e:ciLater", "Lateral Index"
      e_ciDistRoot = "e:ciDistRoot", "Distance from first infeed in number of buses"
      e_ciDistAll = "e:ciDistAll", "Distance from infeed in number of buses including switches"
      e_ciDistAllRoot = "e:ciDistAllRoot", "Distance from first infeed in number of buses including switches"
      r_p_pctrl = "r:p_pctrl", "StaCubic*, Controlled Flow"
      r_cpFeed = "r:cpFeed", "Feeder"
      m_Ikss_bus1 = "m:Ikss:bus1", "kA, Initial Short-Circuit Current"
      m_Ikss_bus2 = "m:Ikss:bus2", "kA, Initial Short-Circuit Current"
      m_ikss_bus1 = "m:ikss:bus1", "p.u., Initial Short-Circuit Current, Magnitude"
      m_ikss_bus2 = "m:ikss:bus2", "p.u., Initial Short-Circuit Current, Magnitude"
      m_phii_bus1 = "m:phii:bus1", "deg, Phase Current, Angle"
      m_maxIkss_bus1 = "m:maxIkss:bus1", "kA, Max. Initial Short-Circuit Current"
      m_maxIkss_bus2 = "m:maxIkss:bus2", "kA, Max. Initial Short-Circuit Current"
      m_Skss_bus1 = "m:Skss:bus1", "MVA, Initial Short-Circuit Power"
      m_Skss_bus2 = "m:Skss:bus2", "MVA, Initial Short-Circuit Power"
      m_ip_bus1 = "m:ip:bus1", "kA, Peak Short-Circuit Current"
      m_ip_bus2 = "m:ip:bus2", "kA, Peak Short-Circuit Current"
      m_maxip_bus1 = "m:maxip:bus1", "kA, Max. Peak Short-Circuit Current"
      m_maxip_bus2 = "m:maxip:bus2", "kA, Max. Peak Short-Circuit Current"
      m_maxIb_bus1 = "m:maxIb:bus1", "kA, Max. Breaking Current"
      m_maxIb_bus2 = "m:maxIb:bus2", "kA, Max. Breaking Current"
      m_maxIth_bus1 = "m:maxIth:bus1", "kA, Max. Thermal equivalent Short-Circuit Current"
      m_maxIth_bus2 = "m:maxIth:bus2", "kA, Max. Thermal equivalent Short-Circuit Current"
      m_Ib_bus1 = "m:Ib:bus1", "kA, Short-Circuit Breaking Current"
      m_Ib_bus2 = "m:Ib:bus2", "kA, Short-Circuit Breaking Current"
      m_Sb_bus1 = "m:Sb:bus1", "MVA, Short-Circuit Breaking Power"
      m_Sb_bus2 = "m:Sb:bus2", "MVA, Short-Circuit Breaking Power"
      m_Ik_bus1 = "m:Ik:bus1", "kA, Steady-State Short-Circuit Current"
      m_Ik_bus2 = "m:Ik:bus2", "kA, Steady-State Short-Circuit Current"
      m_Ith_bus1 = "m:Ith:bus1", "kA, Thermal equivalent Short-Circuit Current"
      m_Ith_bus2 = "m:Ith:bus2", "kA, Thermal equivalent Short-Circuit Current"
      m_I_bus1 = "m:I:bus1", "kA, Short-Circuit Current"
      m_R_bus1 = "m:R:bus1", "Ohm, Fault Impedance, Real Part"
      m_R_bus2 = "m:R:bus2", "Ohm, Fault Impedance, Real Part"
      m_X_bus1 = "m:X:bus1", "Ohm, Fault Impedance, Imaginary Part"
      m_X_bus2 = "m:X:bus2", "Ohm, Fault Impedance, Imaginary Part"
      m_Z_bus1 = "m:Z:bus1", "Ohm, Fault Impedance, Magnitude"
      m_Z_bus2 = "m:Z:bus2", "Ohm, Fault Impedance, Magnitude"
      m_phiz_bus1 = "m:phiz:bus1", "deg, Fault Impedance, Angle"
      m_phiz_bus2 = "m:phiz:bus2", "deg, Fault Impedance, Angle"
      m_zSbase_bus1 = "m:zSbase:bus1", "p.u., Fault Impedance, Magnitude"
      m_zSbase_bus2 = "m:zSbase:bus2", "p.u., Fault Impedance, Magnitude"
      m_XtoR_bus1 = "m:XtoR:bus1", "X/R ratio"
      m_XtoR_bus2 = "m:XtoR:bus2", "X/R ratio"
      m_Zs_bus1 = "m:Zs:bus1", "Ohm, Source Impedance, Magnitude"
      m_Zs_bus2 = "m:Zs:bus2", "Ohm, Source Impedance, Magnitude"
      m_phizs_bus1 = "m:phizs:bus1", "deg, Source Impedance, Angle"
      m_phizs_bus2 = "m:phizs:bus2", "deg, Source Impedance, Angle"
      m_Rs_bus1 = "m:Rs:bus1", "Ohm, Source Impedance, Real Part"
      m_Rs_bus2 = "m:Rs:bus2", "Ohm, Source Impedance, Real Part"
      m_Xs_bus1 = "m:Xs:bus1", "Ohm, Source Impedance, Imaginary Part"
      m_Xs_bus2 = "m:Xs:bus2", "Ohm, Source Impedance, Imaginary Part"
      m_phiui_bus1 = "m:phiui:bus1", "deg, Angle between Voltage and Current"
      m_cosphiui_bus1 = "m:cosphiui:bus1", "cosphi"
      m_cosphiui_bus2 = "m:cosphiui:bus2", "cosphi"
      m_P_bus1 = "m:P:bus1", "MW, Active Power"
      m_Q_bus1 = "m:Q:bus1", "Mvar, Reactive Power"
      m_BrkIpload_bus1 = "m:BrkIpload:bus1", "%, Breaker Loading (peak)"
      m_BrkIpload_bus2 = "m:BrkIpload:bus2", "%, Breaker Loading (peak)"
      m_BrkIbload_bus1 = "m:BrkIbload:bus1", "%, Breaker Loading (breaking)"
      m_BrkIbload_bus2 = "m:BrkIbload:bus2", "%, Breaker Loading (breaking)"
      m_BrkmaxIb_bus1 = "m:BrkmaxIb:bus1", "kA, Max. Breaking Current (breaker time)"
      m_BrkmaxIb_bus2 = "m:BrkmaxIb:bus2", "kA, Max. Breaking Current (breaker time)"
      n_Ikss_bus1 = "n:Ikss:bus1", "kA, Initial Short-Circuit Current"
      n_Ikss_bus2 = "n:Ikss:bus2", "kA, Initial Short-Circuit Current"
      n_I_bus1 = "n:I:bus1", "kA, Short-Circuit Current"
      n_I_bus2 = "n:I:bus2", "kA, Short-Circuit Current"
      n_phii_bus1 = "n:phii:bus1", "deg, Phase Current, Angle"
      n_phii_bus2 = "n:phii:bus2", "deg, Phase Current, Angle"
      n_Skss_bus1 = "n:Skss:bus1", "MVA, Initial Short-Circuit Power"
      n_Skss_bus2 = "n:Skss:bus2", "MVA, Initial Short-Circuit Power"
      n_ip_bus1 = "n:ip:bus1", "kA, Peak Short-Circuit Current"
      n_ip_bus2 = "n:ip:bus2", "kA, Peak Short-Circuit Current"
      n_Ib_bus1 = "n:Ib:bus1", "kA, Short-Circuit Breaking Current"
      n_Ib_bus2 = "n:Ib:bus2", "kA, Short-Circuit Breaking Current"
      n_Sb_bus1 = "n:Sb:bus1", "MVA, Short-Circuit Breaking Power"
      n_Sb_bus2 = "n:Sb:bus2", "MVA, Short-Circuit Breaking Power"
      n_Ik_bus1 = "n:Ik:bus1", "kA, Steady-State Short-Circuit Current"
      n_Ik_bus2 = "n:Ik:bus2", "kA, Steady-State Short-Circuit Current"
      n_Ith_bus1 = "n:Ith:bus1", "kA, Thermal equivalent Short-Circuit Current"
      n_Ith_bus2 = "n:Ith:bus2", "kA, Thermal equivalent Short-Circuit Current"
      n_R_bus1 = "n:R:bus1", "Ohm, Short-Circuit Impedance, Real Part"
      n_R_bus2 = "n:R:bus2", "Ohm, Short-Circuit Impedance, Real Part"
      n_X_bus1 = "n:X:bus1", "Ohm, Short-Circuit Impedance, Imaginary Part"
      n_X_bus2 = "n:X:bus2", "Ohm, Short-Circuit Impedance, Imaginary Part"
      n_Z_bus1 = "n:Z:bus1", "Ohm, Short-Circuit Impedance, Magnitude"
      n_Z_bus2 = "n:Z:bus2", "Ohm, Short-Circuit Impedance, Magnitude"
      n_phiz_bus1 = "n:phiz:bus1", "deg, Short-Circuit Impedance, Angle"
      n_phiz_bus2 = "n:phiz:bus2", "deg, Short-Circuit Impedance, Angle"
      n_rSbase_bus1 = "n:rSbase:bus1", "p.u., Short-Circuit Impedance, Real Part"
      n_rSbase_bus2 = "n:rSbase:bus2", "p.u., Short-Circuit Impedance, Real Part"
      n_xSbase_bus1 = "n:xSbase:bus1", "p.u., Short-Circuit Impedance, Imaginary Part"
      n_xSbase_bus2 = "n:xSbase:bus2", "p.u., Short-Circuit Impedance, Imaginary Part"
      n_zSbase_bus1 = "n:zSbase:bus1", "p.u., Short-Circuit Impedance, Magnitude"
      n_zSbase_bus2 = "n:zSbase:bus2", "p.u., Short-Circuit Impedance, Magnitude"
      n_XtoR_bus1 = "n:XtoR:bus1", "X/R ratio (peak)"
      n_XtoR_bus2 = "n:XtoR:bus2", "X/R ratio (peak)"
      n_RtoX_bus1 = "n:RtoX:bus1", "R/X ratio (peak)"
      n_RtoX_bus2 = "n:RtoX:bus2", "R/X ratio (peak)"
      n_ur_bus1 = "n:ur:bus1", "p.u., Voltage, Real Part"
      n_ui_bus1 = "n:ui:bus1", "p.u., Voltage, Imaginary Part"
      n_u_bus1 = "n:u:bus1", "p.u., Voltage, Magnitude"
      n_upc_bus1 = "n:upc:bus1", "%, Voltage, Magnitude"
      n_U_bus1 = "n:U:bus1", "kV, Line-Ground Voltage, Magnitude"
      n_Ul_bus1 = "n:Ul:bus1", "kV, Line-Line Voltage, Magnitude"
      n_phiu_bus1 = "n:phiu:bus1", "deg, Voltage, Angle"
      n_du_bus1 = "n:du:bus1", "%, Voltage Deviation"
      n_du_bus2 = "n:du:bus2", "%, Voltage Deviation"
      n_urpref_bus1 = "n:urpref:bus1", "p.u., Pre-fault Voltage, Real Part"
      n_urpref_bus2 = "n:urpref:bus2", "p.u., Pre-fault Voltage, Real Part"
      n_uipref_bus1 = "n:uipref:bus1", "p.u., Pre-fault Voltage, Imaginary Part"
      n_uipref_bus2 = "n:uipref:bus2", "p.u., Pre-fault Voltage, Imaginary Part"
      n_upref_bus1 = "n:upref:bus1", "p.u., Pre-fault Voltage, Magnitude"
      n_upref_bus2 = "n:upref:bus2", "p.u., Pre-fault Voltage, Magnitude"
      n_uprefpc_bus1 = "n:uprefpc:bus1", "%, Pre-fault Voltage, Magnitude"
      n_uprefpc_bus2 = "n:uprefpc:bus2", "%, Pre-fault Voltage, Magnitude"
      n_Upref_bus1 = "n:Upref:bus1", "kV, Line-Ground Pre-fault Voltage, Magnitude"
      n_Upref_bus2 = "n:Upref:bus2", "kV, Line-Ground Pre-fault Voltage, Magnitude"
      n_Ulpref_bus1 = "n:Ulpref:bus1", "kV, Line-Line Pre-fault Voltage, Magnitude"
      n_Ulpref_bus2 = "n:Ulpref:bus2", "kV, Line-Line Pre-fault Voltage, Magnitude"
      n_phiupref_bus1 = "n:phiupref:bus1", "deg, Pre-fault Voltage, Angle"
      n_phiupref_bus2 = "n:phiupref:bus2", "deg, Pre-fault Voltage, Angle"
      n_phiui_bus1 = "n:phiui:bus1", "deg, Angle between Voltage and Current"
      n_phiui_bus2 = "n:phiui:bus2", "deg, Angle between Voltage and Current"
      n_cfac_bus1 = "n:cfac:bus1", "Voltage Factor"
      n_cfac_bus2 = "n:cfac:bus2", "Voltage Factor"
      n_idc_bus1 = "n:idc:bus1", "kA, Decaying Component"
      n_idc_bus2 = "n:idc:bus2", "kA, Decaying Component"
      n_Iasy_bus1 = "n:Iasy:bus1", "kA, Asymmetrical Breaking Current"
      n_Iasy_bus2 = "n:Iasy:bus2", "kA, Asymmetrical Breaking Current"
      n_Ithload_bus1 = "n:Ithload:bus1", "%, Loading, Thermal equivalent Current"
      n_Ithload_bus2 = "n:Ithload:bus2", "%, Loading, Thermal equivalent Current"
      n_Ipload_bus1 = "n:Ipload:bus1", "%, Loading, Peak Short-Circuit Current"
      n_Ipload_bus2 = "n:Ipload:bus2", "%, Loading, Peak Short-Circuit Current"
      n_Ithrtk_bus1 = "n:Ithrtk:bus1", "kA, Rated Short-Time Current (Tk)"
      n_Ithrtk_bus2 = "n:Ithrtk:bus2", "kA, Rated Short-Time Current (Tk)"
      n_Tn_bus1 = "n:Tn:bus1", "ms, Grid Time Constant"
      n_Tn_bus2 = "n:Tn:bus2", "ms, Grid Time Constant"
      n_XtoR_b_bus1 = "n:XtoR_b:bus1", "X/R ratio (dc)"
      n_XtoR_b_bus2 = "n:XtoR_b:bus2", "X/R ratio (dc)"
      n_RtoX_b_bus1 = "n:RtoX_b:bus1", "R/X ratio (dc)"
      n_RtoX_b_bus2 = "n:RtoX_b:bus2", "R/X ratio (dc)"
      n_fctof_b_bus1 = "n:fctof_b:bus1", "Equivalent Frequency fc/f (dc)"
      n_fctof_b_bus2 = "n:fctof_b:bus2", "Equivalent Frequency fc/f (dc)"
      c_z1_r = "c:z1:r", "p.u., Positive-sequence impedance, Real Part"
      c_z1_i = "c:z1:i", "p.u., Positive-sequence impedance, Imaginary Part"
      c_z1_phi = "c:z1:phi", "deg, Positive-sequence impedance, Angle"
      c_z1_mag = "c:z1:mag", "p.u., Positive-sequence impedance, Magnitude"
      c_z1_phirad = "c:z1:phirad", "rad, Positive-sequence impedance, Angle"
      c_uini_r = "c:uini:r", "p.u., Internal positive-sequence voltage, Real Part"
      c_uini_i = "c:uini:i", "p.u., Internal positive-sequence voltage, Imaginary Part"
      c_uini_phi = "c:uini:phi", "deg, Internal positive-sequence voltage, Angle"
      c_uini_mag = "c:uini:mag", "p.u., Internal positive-sequence voltage, Magnitude"
      c_uini_phirad = "c:uini:phirad", "rad, Internal positive-sequence voltage, Angle"
      c_Prm = "c:Prm", "MW, Rated Mechanical Power"
      c_imax = "c:imax", "p.u., Max. Current"
      c_Zwd_r = "c:Zwd:r", "p.u., Impedance of doubly-fed asynchronous generator, Real Part"
      c_Zwd_i = "c:Zwd:i", "p.u., Impedance of doubly-fed asynchronous generator, Imaginary Part"
      c_Zwd_phi = "c:Zwd:phi", "deg, Impedance of doubly-fed asynchronous generator, Angle"
      c_Zwd_mag = "c:Zwd:mag", "p.u., Impedance of doubly-fed asynchronous generator, Magnitude"
      c_Zwd_phirad = "c:Zwd:phirad", "rad, Impedance of doubly-fed asynchronous generator, Angle"
      e_iNoShcContr = "e:iNoShcContr", "No Short-Circuit Contribution"
      e_iconfed = "e:iconfed", "Static converter-fed drive"
      e_searchBlockTrf = "e:searchBlockTrf", "Externally modelled unit transformer"
      e_pblocktrf = "e:pblocktrf", "Unit transformer"
      e_iWDmax = "e:iWDmax", "kA, Max. instantaneous short-circuit current, iWDmax"
      e_kWD = "e:kWD", "Factor kappaWD"
      e_rxWD = "e:rxWD", "Ratio RWD/XWD"
      e_IkWDmax = "e:IkWDmax", "kA, Maximum current"
      e_IkWDmin = "e:IkWDmin", "kA, Minimum current"
      e_r2iec = "e:r2iec", "p.u., Resistance, r2"
      e_x2iec = "e:x2iec", "p.u., Reactance, x2"
      e_r0iec = "e:r0iec", "p.u., Resistance, r0"
      e_x0iec = "e:x0iec", "p.u., Reactance, x0"
      r_pblocktrf = "r:pblocktrf", "Unit transformer"
      m_Ikss_bus1_A = "m:Ikss:bus1:A", "kA, Initial Short-Circuit Current"
      m_Ikss_bus1_B = "m:Ikss:bus1:B", "kA, Initial Short-Circuit Current"
      m_Ikss_bus1_C = "m:Ikss:bus1:C", "kA, Initial Short-Circuit Current"
      m_ikss_bus1_A = "m:ikss:bus1:A", "p.u., Initial Short-Circuit Current, Magnitude"
      m_ikss_bus1_B = "m:ikss:bus1:B", "p.u., Initial Short-Circuit Current, Magnitude"
      m_ikss_bus1_C = "m:ikss:bus1:C", "p.u., Initial Short-Circuit Current, Magnitude"
      m_Skss_bus1_A = "m:Skss:bus1:A", "MVA, Initial Short-Circuit Power"
      m_Skss_bus1_B = "m:Skss:bus1:B", "MVA, Initial Short-Circuit Power"
      m_Skss_bus1_C = "m:Skss:bus1:C", "MVA, Initial Short-Circuit Power"
      m_Skssx3_bus1_A = "m:Skssx3:bus1:A", "MVA, 3*Initial Short-Circuit Power"
      m_Skssx3_bus1_B = "m:Skssx3:bus1:B", "MVA, 3*Initial Short-Circuit Power"
      m_Skssx3_bus1_C = "m:Skssx3:bus1:C", "MVA, 3*Initial Short-Circuit Power"
      m_Skssx3_bus2 = "m:Skssx3:bus2", "MVA, 3*Initial Short-Circuit Power"
      m_ip_bus1_A = "m:ip:bus1:A", "kA, Peak Short-Circuit Current"
      m_ip_bus1_B = "m:ip:bus1:B", "kA, Peak Short-Circuit Current"
      m_ip_bus1_C = "m:ip:bus1:C", "kA, Peak Short-Circuit Current"
      m_Z_bus1_A = "m:Z:bus1:A", "Ohm, Fault Impedance, Magnitude"
      m_Z_bus1_B = "m:Z:bus1:B", "Ohm, Fault Impedance, Magnitude"
      m_Z_bus1_C = "m:Z:bus1:C", "Ohm, Fault Impedance, Magnitude"
      m_phiz_bus1_A = "m:phiz:bus1:A", "deg, Fault Impedance, Angle"
      m_phiz_bus1_B = "m:phiz:bus1:B", "deg, Fault Impedance, Angle"
      m_phiz_bus1_C = "m:phiz:bus1:C", "deg, Fault Impedance, Angle"
      m_Zl_bus1_A = "m:Zl:bus1:A", "Ohm, Line-Line Fault Impedance, Magnitude"
      m_Zl_bus1_B = "m:Zl:bus1:B", "Ohm, Line-Line Fault Impedance, Magnitude"
      m_Zl_bus1_C = "m:Zl:bus1:C", "Ohm, Line-Line Fault Impedance, Magnitude"
      m_Zl_bus2 = "m:Zl:bus2", "Ohm, Line-Line Fault Impedance, Magnitude"
      m_phizl_bus1_A = "m:phizl:bus1:A", "deg, Line-Line Fault Impedance, Angle"
      m_phizl_bus1_B = "m:phizl:bus1:B", "deg, Line-Line Fault Impedance, Angle"
      m_phizl_bus1_C = "m:phizl:bus1:C", "deg, Line-Line Fault Impedance, Angle"
      m_phizl_bus2 = "m:phizl:bus2", "deg, Line-Line Fault Impedance, Angle"
      m_Z0_bus1 = "m:Z0:bus1", "Ohm, Zero-Sequence Fault Impedance, Magnitude"
      m_Z0_bus2 = "m:Z0:bus2", "Ohm, Zero-Sequence Fault Impedance, Magnitude"
      m_phiz0_bus1 = "m:phiz0:bus1", "deg, Zero-Sequence Fault Impedance, Angle"
      m_phiz0_bus2 = "m:phiz0:bus2", "deg, Zero-Sequence Fault Impedance, Angle"
      m_z0Sbase_bus1 = "m:z0Sbase:bus1", "p.u., Zero-Sequence Fault Impedance, Magnitude"
      m_z0Sbase_bus2 = "m:z0Sbase:bus2", "p.u., Zero-Sequence Fault Impedance, Magnitude"
      m_cosphiu1i1_bus1 = "m:cosphiu1i1:bus1", "Positive-Sequence cosphi"
      m_cosphiu1i1_bus2 = "m:cosphiu1i1:bus2", "Positive-Sequence cosphi"
      m_I0x3watt_bus1 = "m:I0x3watt:bus1", "kA, Earth-Fault Residual Current"
      m_I0x3watt_bus2 = "m:I0x3watt:bus2", "kA, Earth-Fault Residual Current"
      m_I0x3var_bus1 = "m:I0x3var:bus1", "kA, Capacitive Earth-Fault Current"
      m_I0x3var_bus2 = "m:I0x3var:bus2", "kA, Capacitive Earth-Fault Current"
      m_Q0cap_bus1 = "m:Q0cap:bus1", "Mvar, Q0 capacitive"
      m_Q0cap_bus2 = "m:Q0cap:bus2", "Mvar, Q0 capacitive"
      m_Zfrel_bus1 = "m:Zfrel:bus1", "Ohm, Relay Impedance, Magnitude"
      m_Zfrel_bus2 = "m:Zfrel:bus2", "Ohm, Relay Impedance, Magnitude"
      m_phizfrel_bus1 = "m:phizfrel:bus1", "deg, Relay Impedance, Angle"
      m_phizfrel_bus2 = "m:phizfrel:bus2", "deg, Relay Impedance, Angle"
      m_Rfrel_bus1 = "m:Rfrel:bus1", "Ohm, Relay Impedance, Real Part"
      m_Rfrel_bus2 = "m:Rfrel:bus2", "Ohm, Relay Impedance, Real Part"
      m_Xfrel_bus1 = "m:Xfrel:bus1", "Ohm, Relay Impedance, Imaginary Part"
      m_Xfrel_bus2 = "m:Xfrel:bus2", "Ohm, Relay Impedance, Imaginary Part"
      m_Zf_bus1 = "m:Zf:bus1", "Ohm, Meas. Fault Impedance, Magnitude"
      m_Zf_bus2 = "m:Zf:bus2", "Ohm, Meas. Fault Impedance, Magnitude"
      m_phizf_bus1 = "m:phizf:bus1", "deg, Meas. Fault Impedance, Angle"
      m_phizf_bus2 = "m:phizf:bus2", "deg, Meas. Fault Impedance, Angle"
      n_Ikss_bus1_A = "n:Ikss:bus1:A", "kA, Initial Short-Circuit Current"
      n_Ikss_bus1_B = "n:Ikss:bus1:B", "kA, Initial Short-Circuit Current"
      n_Ikss_bus1_C = "n:Ikss:bus1:C", "kA, Initial Short-Circuit Current"
      n_I_bus1_A = "n:I:bus1:A", "kA, Short-Circuit Current"
      n_I_bus1_B = "n:I:bus1:B", "kA, Short-Circuit Current"
      n_I_bus1_C = "n:I:bus1:C", "kA, Short-Circuit Current"
      n_phii_bus1_A = "n:phii:bus1:A", "deg, Phase Current, Angle"
      n_phii_bus1_B = "n:phii:bus1:B", "deg, Phase Current, Angle"
      n_phii_bus1_C = "n:phii:bus1:C", "deg, Phase Current, Angle"
      n_Skss_bus1_A = "n:Skss:bus1:A", "MVA, Initial Short-Circuit Power"
      n_Skss_bus1_B = "n:Skss:bus1:B", "MVA, Initial Short-Circuit Power"
      n_Skss_bus1_C = "n:Skss:bus1:C", "MVA, Initial Short-Circuit Power"
      n_Skssx3_bus1_A = "n:Skssx3:bus1:A", "MVA, 3*Initial Short-Circuit Power"
      n_Skssx3_bus1_B = "n:Skssx3:bus1:B", "MVA, 3*Initial Short-Circuit Power"
      n_Skssx3_bus1_C = "n:Skssx3:bus1:C", "MVA, 3*Initial Short-Circuit Power"
      n_Skssx3_bus2 = "n:Skssx3:bus2", "MVA, 3*Initial Short-Circuit Power"
      n_ip_bus1_A = "n:ip:bus1:A", "kA, Peak Short-Circuit Current"
      n_ip_bus1_B = "n:ip:bus1:B", "kA, Peak Short-Circuit Current"
      n_ip_bus1_C = "n:ip:bus1:C", "kA, Peak Short-Circuit Current"
      n_Ib_bus1_A = "n:Ib:bus1:A", "kA, Short-Circuit Breaking Current"
      n_Ib_bus1_B = "n:Ib:bus1:B", "kA, Short-Circuit Breaking Current"
      n_Ib_bus1_C = "n:Ib:bus1:C", "kA, Short-Circuit Breaking Current"
      n_Sb_bus1_A = "n:Sb:bus1:A", "MVA, Short-Circuit Breaking Power"
      n_Sb_bus1_B = "n:Sb:bus1:B", "MVA, Short-Circuit Breaking Power"
      n_Sb_bus1_C = "n:Sb:bus1:C", "MVA, Short-Circuit Breaking Power"
      n_Ith_bus1_A = "n:Ith:bus1:A", "kA, Thermal equivalent Short-Circuit Current"
      n_Ith_bus1_B = "n:Ith:bus1:B", "kA, Thermal equivalent Short-Circuit Current"
      n_Ith_bus1_C = "n:Ith:bus1:C", "kA, Thermal equivalent Short-Circuit Current"
      n_Ithmax_bus1 = "n:Ithmax:bus1", "kA, Max. Thermal equivalent Short-Circuit Current"
      n_Ithmax_bus2 = "n:Ithmax:bus2", "kA, Max. Thermal equivalent Short-Circuit Current"
      n_Ik_bus1_A = "n:Ik:bus1:A", "kA, Steady-State Short-Circuit Current"
      n_Ik_bus1_B = "n:Ik:bus1:B", "kA, Steady-State Short-Circuit Current"
      n_Ik_bus1_C = "n:Ik:bus1:C", "kA, Steady-State Short-Circuit Current"
      n_R0_bus1 = "n:R0:bus1", "Ohm, Zero-Sequence Impedance, Real Part"
      n_R0_bus2 = "n:R0:bus2", "Ohm, Zero-Sequence Impedance, Real Part"
      n_X0_bus1 = "n:X0:bus1", "Ohm, Zero-Sequence Impedance, Imaginary Part"
      n_X0_bus2 = "n:X0:bus2", "Ohm, Zero-Sequence Impedance, Imaginary Part"
      n_Z0_bus1 = "n:Z0:bus1", "Ohm, Zero-Sequence Impedance, Magnitude"
      n_Z0_bus2 = "n:Z0:bus2", "Ohm, Zero-Sequence Impedance, Magnitude"
      n_phiz0_bus1 = "n:phiz0:bus1", "deg, Zero-Sequence Impedance, Angle"
      n_phiz0_bus2 = "n:phiz0:bus2", "deg, Zero-Sequence Impedance, Angle"
      n_r0Sbase_bus1 = "n:r0Sbase:bus1", "p.u., Zero-Sequence Impedance, Real Part"
      n_r0Sbase_bus2 = "n:r0Sbase:bus2", "p.u., Zero-Sequence Impedance, Real Part"
      n_x0Sbase_bus1 = "n:x0Sbase:bus1", "p.u., Zero-Sequence Impedance, Imaginary Part"
      n_x0Sbase_bus2 = "n:x0Sbase:bus2", "p.u., Zero-Sequence Impedance, Imaginary Part"
      n_z0Sbase_bus1 = "n:z0Sbase:bus1", "p.u., Zero-Sequence Impedance, Magnitude"
      n_z0Sbase_bus2 = "n:z0Sbase:bus2", "p.u., Zero-Sequence Impedance, Magnitude"
      n_R1_bus1 = "n:R1:bus1", "Ohm, Positive-Sequence Impedance, Real Part"
      n_R1_bus2 = "n:R1:bus2", "Ohm, Positive-Sequence Impedance, Real Part"
      n_X1_bus1 = "n:X1:bus1", "Ohm, Positive-Sequence Impedance, Imaginary Part"
      n_X1_bus2 = "n:X1:bus2", "Ohm, Positive-Sequence Impedance, Imaginary Part"
      n_Z1_bus1 = "n:Z1:bus1", "Ohm, Positive-Sequence Impedance, Magnitude"
      n_Z1_bus2 = "n:Z1:bus2", "Ohm, Positive-Sequence Impedance, Magnitude"
      n_phiz1_bus1 = "n:phiz1:bus1", "deg, Positive-Sequence Impedance, Angle"
      n_phiz1_bus2 = "n:phiz1:bus2", "deg, Positive-Sequence Impedance, Angle"
      n_r1Sbase_bus1 = "n:r1Sbase:bus1", "p.u., Positive-Sequence Impedance, Real Part"
      n_r1Sbase_bus2 = "n:r1Sbase:bus2", "p.u., Positive-Sequence Impedance, Real Part"
      n_x1Sbase_bus1 = "n:x1Sbase:bus1", "p.u., Positive-Sequence Impedance, Imaginary Part"
      n_x1Sbase_bus2 = "n:x1Sbase:bus2", "p.u., Positive-Sequence Impedance, Imaginary Part"
      n_z1Sbase_bus1 = "n:z1Sbase:bus1", "p.u., Positive-Sequence Impedance, Magnitude"
      n_z1Sbase_bus2 = "n:z1Sbase:bus2", "p.u., Positive-Sequence Impedance, Magnitude"
      n_R2_bus1 = "n:R2:bus1", "Ohm, Negative-Sequence Impedance, Real Part"
      n_R2_bus2 = "n:R2:bus2", "Ohm, Negative-Sequence Impedance, Real Part"
      n_X2_bus1 = "n:X2:bus1", "Ohm, Negative-Sequence Impedance, Imaginary Part"
      n_X2_bus2 = "n:X2:bus2", "Ohm, Negative-Sequence Impedance, Imaginary Part"
      n_Z2_bus1 = "n:Z2:bus1", "Ohm, Negative-Sequence Impedance, Magnitude"
      n_Z2_bus2 = "n:Z2:bus2", "Ohm, Negative-Sequence Impedance, Magnitude"
      n_phiz2_bus1 = "n:phiz2:bus1", "deg, Negative-Sequence Impedance, Angle"
      n_phiz2_bus2 = "n:phiz2:bus2", "deg, Negative-Sequence Impedance, Angle"
      n_r2Sbase_bus1 = "n:r2Sbase:bus1", "p.u., Negative-Sequence Impedance, Real Part"
      n_r2Sbase_bus2 = "n:r2Sbase:bus2", "p.u., Negative-Sequence Impedance, Real Part"
      n_x2Sbase_bus1 = "n:x2Sbase:bus1", "p.u., Negative-Sequence Impedance, Imaginary Part"
      n_x2Sbase_bus2 = "n:x2Sbase:bus2", "p.u., Negative-Sequence Impedance, Imaginary Part"
      n_z2Sbase_bus1 = "n:z2Sbase:bus1", "p.u., Negative-Sequence Impedance, Magnitude"
      n_z2Sbase_bus2 = "n:z2Sbase:bus2", "p.u., Negative-Sequence Impedance, Magnitude"
      n_R_bus1_A = "n:R:bus1:A", "Ohm, Network Resistance, Line-Ground"
      n_R_bus1_B = "n:R:bus1:B", "Ohm, Network Resistance, Line-Ground"
      n_R_bus1_C = "n:R:bus1:C", "Ohm, Network Resistance, Line-Ground"
      n_X_bus1_A = "n:X:bus1:A", "Ohm, Network Reactance, Line-Ground"
      n_X_bus1_B = "n:X:bus1:B", "Ohm, Network Reactance, Line-Ground"
      n_X_bus1_C = "n:X:bus1:C", "Ohm, Network Reactance, Line-Ground"
      n_Z_bus1_A = "n:Z:bus1:A", "Ohm, Network Impedance, Line-Ground"
      n_Z_bus1_B = "n:Z:bus1:B", "Ohm, Network Impedance, Line-Ground"
      n_Z_bus1_C = "n:Z:bus1:C", "Ohm, Network Impedance, Line-Ground"
      n_phiz_bus1_A = "n:phiz:bus1:A", "deg, Angle of Network Impedance, Line-Ground"
      n_phiz_bus1_B = "n:phiz:bus1:B", "deg, Angle of Network Impedance, Line-Ground"
      n_phiz_bus1_C = "n:phiz:bus1:C", "deg, Angle of Network Impedance, Line-Ground"
      n_Rl_bus1_A = "n:Rl:bus1:A", "Ohm, Network Resistance, Line-Line"
      n_Rl_bus1_B = "n:Rl:bus1:B", "Ohm, Network Resistance, Line-Line"
      n_Rl_bus1_C = "n:Rl:bus1:C", "Ohm, Network Resistance, Line-Line"
      n_Rl_bus2 = "n:Rl:bus2", "Ohm, Network Resistance, Line-Line"
      n_Xl_bus1_A = "n:Xl:bus1:A", "Ohm, Network Reactance, Line-Line"
      n_Xl_bus1_B = "n:Xl:bus1:B", "Ohm, Network Reactance, Line-Line"
      n_Xl_bus1_C = "n:Xl:bus1:C", "Ohm, Network Reactance, Line-Line"
      n_Xl_bus2 = "n:Xl:bus2", "Ohm, Network Reactance, Line-Line"
      n_Zl_bus1_A = "n:Zl:bus1:A", "Ohm, Network Impedance, Line-Line"
      n_Zl_bus1_B = "n:Zl:bus1:B", "Ohm, Network Impedance, Line-Line"
      n_Zl_bus1_C = "n:Zl:bus1:C", "Ohm, Network Impedance, Line-Line"
      n_Zl_bus2 = "n:Zl:bus2", "Ohm, Network Impedance, Line-Line"
      n_phizl_bus1_A = "n:phizl:bus1:A", "deg, Angle of Network Impedance, Line-Line"
      n_phizl_bus1_B = "n:phizl:bus1:B", "deg, Angle of Network Impedance, Line-Line"
      n_phizl_bus1_C = "n:phizl:bus1:C", "deg, Angle of Network Impedance, Line-Line"
      n_phizl_bus2 = "n:phizl:bus2", "deg, Angle of Network Impedance, Line-Line"
      n_Rln_bus1_A = "n:Rln:bus1:A", "Ohm, Network Resistance, Line-Neutral"
      n_Rln_bus1_B = "n:Rln:bus1:B", "Ohm, Network Resistance, Line-Neutral"
      n_Rln_bus1_C = "n:Rln:bus1:C", "Ohm, Network Resistance, Line-Neutral"
      n_Rln_bus2 = "n:Rln:bus2", "Ohm, Network Resistance, Line-Neutral"
      n_Xln_bus1_A = "n:Xln:bus1:A", "Ohm, Network Reactance, Line-Neutral"
      n_Xln_bus1_B = "n:Xln:bus1:B", "Ohm, Network Reactance, Line-Neutral"
      n_Xln_bus1_C = "n:Xln:bus1:C", "Ohm, Network Reactance, Line-Neutral"
      n_Xln_bus2 = "n:Xln:bus2", "Ohm, Network Reactance, Line-Neutral"
      n_Zln_bus1_A = "n:Zln:bus1:A", "Ohm, Network Impedance, Line-Neutral"
      n_Zln_bus1_B = "n:Zln:bus1:B", "Ohm, Network Impedance, Line-Neutral"
      n_Zln_bus1_C = "n:Zln:bus1:C", "Ohm, Network Impedance, Line-Neutral"
      n_Zln_bus2 = "n:Zln:bus2", "Ohm, Network Impedance, Line-Neutral"
      n_phizln_bus1_A = "n:phizln:bus1:A", "deg, Angle of Network Impedance, Line-Neutral"
      n_phizln_bus1_B = "n:phizln:bus1:B", "deg, Angle of Network Impedance, Line-Neutral"
      n_phizln_bus1_C = "n:phizln:bus1:C", "deg, Angle of Network Impedance, Line-Neutral"
      n_phizln_bus2 = "n:phizln:bus2", "deg, Angle of Network Impedance, Line-Neutral"
      n_Rn_bus1 = "n:Rn:bus1", "Ohm, Network Resistance, Neutral"
      n_Rn_bus2 = "n:Rn:bus2", "Ohm, Network Resistance, Neutral"
      n_Xn_bus1 = "n:Xn:bus1", "Ohm, Network Reactance, Neutral"
      n_Xn_bus2 = "n:Xn:bus2", "Ohm, Network Reactance, Neutral"
      n_Zn_bus1 = "n:Zn:bus1", "Ohm, Network Impedance, Neutral"
      n_Zn_bus2 = "n:Zn:bus2", "Ohm, Network Impedance, Neutral"
      n_phizn_bus1 = "n:phizn:bus1", "deg, Angle of Network Impedance, Neutral"
      n_phizn_bus2 = "n:phizn:bus2", "deg, Angle of Network Impedance, Neutral"
      n_eff_bus1_A = "n:eff:bus1:A", "Earth Fault Factor"
      n_eff_bus1_B = "n:eff:bus1:B", "Earth Fault Factor"
      n_eff_bus1_C = "n:eff:bus1:C", "Earth Fault Factor"
      n_eff_bus2 = "n:eff:bus2", "Earth Fault Factor"
      n_X0toX1_bus1 = "n:X0toX1:bus1", "X0/X1"
      n_X0toX1_bus2 = "n:X0toX1:bus2", "X0/X1"
      n_in_bus1 = "n:in:bus1", "p.u., Neutral Current, Magnitude"
      n_in_bus2 = "n:in:bus2", "p.u., Neutral Current, Magnitude"
      n_In_bus1 = "n:In:bus1", "kA, Neutral Current, Magnitude"
      n_In_bus2 = "n:In:bus2", "kA, Neutral Current, Magnitude"
      n_phiin_bus1 = "n:phiin:bus1", "deg, Neutral Current, Angle"
      n_phiin_bus2 = "n:phiin:bus2", "deg, Neutral Current, Angle"
      n_i0r_bus1 = "n:i0r:bus1", "p.u., Zero-Sequence Current, Real Part"
      n_i0r_bus2 = "n:i0r:bus2", "p.u., Zero-Sequence Current, Real Part"
      n_i0i_bus1 = "n:i0i:bus1", "p.u., Zero-Sequence Current, Imaginary Part"
      n_i0i_bus2 = "n:i0i:bus2", "p.u., Zero-Sequence Current, Imaginary Part"
      n_i0_bus1 = "n:i0:bus1", "p.u., Zero-Sequence Current, Magnitude"
      n_i0_bus2 = "n:i0:bus2", "p.u., Zero-Sequence Current, Magnitude"
      n_I0_bus1 = "n:I0:bus1", "kA, Zero-Sequence Current, Magnitude"
      n_I0_bus2 = "n:I0:bus2", "kA, Zero-Sequence Current, Magnitude"
      n_I0x3_bus1 = "n:I0x3:bus1", "kA, 3*I0"
      n_I0x3_bus2 = "n:I0x3:bus2", "kA, 3*I0"
      n_phii0_bus1 = "n:phii0:bus1", "deg, Zero-Sequence Current, Angle"
      n_phii0_bus2 = "n:phii0:bus2", "deg, Zero-Sequence Current, Angle"
      n_i1r_bus1 = "n:i1r:bus1", "p.u., Positive-Sequence Current, Real Part"
      n_i1r_bus2 = "n:i1r:bus2", "p.u., Positive-Sequence Current, Real Part"
      n_i1i_bus1 = "n:i1i:bus1", "p.u., Positive-Sequence Current, Imaginary Part"
      n_i1i_bus2 = "n:i1i:bus2", "p.u., Positive-Sequence Current, Imaginary Part"
      n_i1_bus1 = "n:i1:bus1", "p.u., Positive-Sequence Current, Magnitude"
      n_i1_bus2 = "n:i1:bus2", "p.u., Positive-Sequence Current, Magnitude"
      n_I1_bus1 = "n:I1:bus1", "kA, Positive-Sequence Current, Magnitude"
      n_I1_bus2 = "n:I1:bus2", "kA, Positive-Sequence Current, Magnitude"
      n_phii1_bus1 = "n:phii1:bus1", "deg, Positive-Sequence Current, Angle"
      n_phii1_bus2 = "n:phii1:bus2", "deg, Positive-Sequence Current, Angle"
      n_i2r_bus1 = "n:i2r:bus1", "p.u., Negative-Sequence Current, Real Part"
      n_i2r_bus2 = "n:i2r:bus2", "p.u., Negative-Sequence Current, Real Part"
      n_i2i_bus1 = "n:i2i:bus1", "p.u., Negative-Sequence Current, Imaginary Part"
      n_i2i_bus2 = "n:i2i:bus2", "p.u., Negative-Sequence Current, Imaginary Part"
      n_i2_bus1 = "n:i2:bus1", "p.u., Negative-Sequence Current, Magnitude"
      n_i2_bus2 = "n:i2:bus2", "p.u., Negative-Sequence Current, Magnitude"
      n_I2_bus1 = "n:I2:bus1", "kA, Negative-Sequence Current, Magnitude"
      n_I2_bus2 = "n:I2:bus2", "kA, Negative-Sequence Current, Magnitude"
      n_phii2_bus1 = "n:phii2:bus1", "deg, Negative-Sequence Current, Angle"
      n_phii2_bus2 = "n:phii2:bus2", "deg, Negative-Sequence Current, Angle"
      n_du_bus1_A = "n:du:bus1:A", "%, Voltage Deviation"
      n_du_bus1_B = "n:du:bus1:B", "%, Voltage Deviation"
      n_du_bus1_C = "n:du:bus1:C", "%, Voltage Deviation"
      n_urpref_bus1_A = "n:urpref:bus1:A", "p.u., Pre-fault Voltage, Real Part"
      n_urpref_bus1_B = "n:urpref:bus1:B", "p.u., Pre-fault Voltage, Real Part"
      n_urpref_bus1_C = "n:urpref:bus1:C", "p.u., Pre-fault Voltage, Real Part"
      n_uipref_bus1_A = "n:uipref:bus1:A", "p.u., Pre-fault Voltage, Imaginary Part"
      n_uipref_bus1_B = "n:uipref:bus1:B", "p.u., Pre-fault Voltage, Imaginary Part"
      n_uipref_bus1_C = "n:uipref:bus1:C", "p.u., Pre-fault Voltage, Imaginary Part"
      n_upref_bus1_A = "n:upref:bus1:A", "p.u., Pre-fault Voltage, Magnitude"
      n_upref_bus1_B = "n:upref:bus1:B", "p.u., Pre-fault Voltage, Magnitude"
      n_upref_bus1_C = "n:upref:bus1:C", "p.u., Pre-fault Voltage, Magnitude"
      n_uprefpc_bus1_A = "n:uprefpc:bus1:A", "%, Pre-fault Voltage, Magnitude"
      n_uprefpc_bus1_B = "n:uprefpc:bus1:B", "%, Pre-fault Voltage, Magnitude"
      n_uprefpc_bus1_C = "n:uprefpc:bus1:C", "%, Pre-fault Voltage, Magnitude"
      n_Upref_bus1_A = "n:Upref:bus1:A", "kV, Line-Ground Pre-fault Voltage, Magnitude"
      n_Upref_bus1_B = "n:Upref:bus1:B", "kV, Line-Ground Pre-fault Voltage, Magnitude"
      n_Upref_bus1_C = "n:Upref:bus1:C", "kV, Line-Ground Pre-fault Voltage, Magnitude"
      n_Ulpref_bus1_A = "n:Ulpref:bus1:A", "kV, Line-Line Pre-fault Voltage, Magnitude"
      n_Ulpref_bus1_B = "n:Ulpref:bus1:B", "kV, Line-Line Pre-fault Voltage, Magnitude"
      n_Ulpref_bus1_C = "n:Ulpref:bus1:C", "kV, Line-Line Pre-fault Voltage, Magnitude"
      n_phiupref_bus1_A = "n:phiupref:bus1:A", "deg, Pre-fault Voltage, Angle"
      n_phiupref_bus1_B = "n:phiupref:bus1:B", "deg, Pre-fault Voltage, Angle"
      n_phiupref_bus1_C = "n:phiupref:bus1:C", "deg, Pre-fault Voltage, Angle"
      n_phiui_bus1_A = "n:phiui:bus1:A", "deg, Angle between Voltage and Current"
      n_phiui_bus1_B = "n:phiui:bus1:B", "deg, Angle between Voltage and Current"
      n_phiui_bus1_C = "n:phiui:bus1:C", "deg, Angle between Voltage and Current"
      n_phiu1i1_bus1 = "n:phiu1i1:bus1", "deg, Angle between Voltage and Current in positive sequence system"
      n_phiu1i1_bus2 = "n:phiu1i1:bus2", "deg, Angle between Voltage and Current in positive sequence system"
      n_phiu2i2_bus1 = "n:phiu2i2:bus1", "deg, Angle between Voltage and Current in negative sequence system"
      n_phiu2i2_bus2 = "n:phiu2i2:bus2", "deg, Angle between Voltage and Current in negative sequence system"
      n_phiu0i0_bus1 = "n:phiu0i0:bus1", "deg, Angle between Voltage and Current in zero-sequence system"
      n_phiu0i0_bus2 = "n:phiu0i0:bus2", "deg, Angle between Voltage and Current in zero-sequence system"
      c_z2_r = "c:z2:r", "p.u., Negative-sequence impedance, Real Part"
      c_z2_i = "c:z2:i", "p.u., Negative-sequence impedance, Imaginary Part"
      c_z2_phi = "c:z2:phi", "deg, Negative-sequence impedance, Angle"
      c_z2_mag = "c:z2:mag", "p.u., Negative-sequence impedance, Magnitude"
      c_z2_phirad = "c:z2:phirad", "rad, Negative-sequence impedance, Angle"

    class ElmBoundary(Enum):
      _init_ = 'value __doc__'

      c_Pinter = "c:Pinter", "MW, Active Power Interchange"
      c_Qinter = "c:Qinter", "Mvar, Reactive Power Interchange"
      c_ExportP = "c:ExportP", "MW, Export, Active Power"
      c_ExportQ = "c:ExportQ", "Mvar, Export, Reactive Power"
      c_ImportP = "c:ImportP", "MW, Import, Active Power"
      c_ImportQ = "c:ImportQ", "Mvar, Import, Reactive Power"
      e_iInterChg = "e:iInterChg", "Consider Interchange Schedule"
      e_InterPset = "e:InterPset", "MW, Scheduled Active Power Interchange"

    class ElmCoup(Enum):
      _init_ = 'value __doc__'

      m_u0_bus1 = "m:u0:bus1", "p.u., Zero-Sequence-Voltage, Magnitude"
      m_u0_bus2 = "m:u0:bus2", "p.u., Zero-Sequence-Voltage, Magnitude"
      m_phiu0_bus1 = "m:phiu0:bus1", "deg, Zero-Sequence-Voltage, Angle"
      m_phiu0_bus2 = "m:phiu0:bus2", "deg, Zero-Sequence-Voltage, Angle"
      m_u1r_bus1 = "m:u1r:bus1", "p.u., Positive-Sequence-Voltage, Real Part"
      m_u1r_bus2 = "m:u1r:bus2", "p.u., Positive-Sequence-Voltage, Real Part"
      m_u1i_bus1 = "m:u1i:bus1", "p.u., Positive-Sequence-Voltage, Imaginary Part"
      m_u1i_bus2 = "m:u1i:bus2", "p.u., Positive-Sequence-Voltage, Imaginary Part"
      m_u1_bus1 = "m:u1:bus1", "p.u., Positive-Sequence-Voltage, Magnitude"
      m_u1_bus2 = "m:u1:bus2", "p.u., Positive-Sequence-Voltage, Magnitude"
      m_U1_bus1 = "m:U1:bus1", "kV, Line-Ground Positive-Sequence-Voltage, Magnitude"
      m_U1_bus2 = "m:U1:bus2", "kV, Line-Ground Positive-Sequence-Voltage, Magnitude"
      m_U1l_bus1 = "m:U1l:bus1", "kV, Line-Line Positive-Sequence-Voltage, Magnitude"
      m_U1l_bus2 = "m:U1l:bus2", "kV, Line-Line Positive-Sequence-Voltage, Magnitude"
      m_phiu1_bus1 = "m:phiu1:bus1", "deg, Positive-Sequence-Voltage, Angle"
      m_phiu1_bus2 = "m:phiu1:bus2", "deg, Positive-Sequence-Voltage, Angle"
      m_u2_bus1 = "m:u2:bus1", "p.u., Negative-Sequence-Voltage, Magnitude"
      m_u2_bus2 = "m:u2:bus2", "p.u., Negative-Sequence-Voltage, Magnitude"
      m_phiu2_bus1 = "m:phiu2:bus1", "deg, Negative-Sequence-Voltage, Angle"
      m_phiu2_bus2 = "m:phiu2:bus2", "deg, Negative-Sequence-Voltage, Angle"
      m_inet_bus1_A = "m:inet:bus1:A", "p.u., Phase Current, Magnitude, referred to network"
      m_inet_bus1_B = "m:inet:bus1:B", "p.u., Phase Current, Magnitude, referred to network"
      m_inet_bus1_C = "m:inet:bus1:C", "p.u., Phase Current, Magnitude, referred to network"
      m_inet_bus1_N = "m:inet:bus1:N", "p.u., Phase Current, Magnitude, referred to network"
      m_inet_bus2_A = "m:inet:bus2:A", "p.u., Phase Current, Magnitude, referred to network"
      m_inet_bus2_B = "m:inet:bus2:B", "p.u., Phase Current, Magnitude, referred to network"
      m_inet_bus2_C = "m:inet:bus2:C", "p.u., Phase Current, Magnitude, referred to network"
      m_inet_bus2_N = "m:inet:bus2:N", "p.u., Phase Current, Magnitude, referred to network"
      m_I_bus1_A = "m:I:bus1:A", "kA, Phase Current, Magnitude"
      m_I_bus1_B = "m:I:bus1:B", "kA, Phase Current, Magnitude"
      m_I_bus1_C = "m:I:bus1:C", "kA, Phase Current, Magnitude"
      m_I_bus1_N = "m:I:bus1:N", "kA, Phase Current, Magnitude"
      m_I_bus2_A = "m:I:bus2:A", "kA, Phase Current, Magnitude"
      m_I_bus2_B = "m:I:bus2:B", "kA, Phase Current, Magnitude"
      m_I_bus2_C = "m:I:bus2:C", "kA, Phase Current, Magnitude"
      m_I_bus2_N = "m:I:bus2:N", "kA, Phase Current, Magnitude"
      m_phii_bus1_A = "m:phii:bus1:A", "deg, Phase Current, Angle"
      m_phii_bus1_B = "m:phii:bus1:B", "deg, Phase Current, Angle"
      m_phii_bus1_C = "m:phii:bus1:C", "deg, Phase Current, Angle"
      m_phii_bus1_N = "m:phii:bus1:N", "deg, Phase Current, Angle"
      m_phii_bus2_A = "m:phii:bus2:A", "deg, Phase Current, Angle"
      m_phii_bus2_B = "m:phii:bus2:B", "deg, Phase Current, Angle"
      m_phii_bus2_C = "m:phii:bus2:C", "deg, Phase Current, Angle"
      m_phii_bus2_N = "m:phii:bus2:N", "deg, Phase Current, Angle"
      m_i0r_bus1 = "m:i0r:bus1", "p.u., Zero-Sequence Current, Real Part"
      m_i0r_bus2 = "m:i0r:bus2", "p.u., Zero-Sequence Current, Real Part"
      m_i0i_bus1 = "m:i0i:bus1", "p.u., Zero-Sequence Current, Imaginary Part"
      m_i0i_bus2 = "m:i0i:bus2", "p.u., Zero-Sequence Current, Imaginary Part"
      m_i0_bus1 = "m:i0:bus1", "p.u., Zero-Sequence Current, Magnitude"
      m_i0_bus2 = "m:i0:bus2", "p.u., Zero-Sequence Current, Magnitude"
      m_I0_bus1 = "m:I0:bus1", "kA, Zero-Sequence Current, Magnitude"
      m_I0_bus2 = "m:I0:bus2", "kA, Zero-Sequence Current, Magnitude"
      m_I0x3_bus1 = "m:I0x3:bus1", "kA, 3*I0"
      m_I0x3_bus2 = "m:I0x3:bus2", "kA, 3*I0"
      m_phii0_bus1 = "m:phii0:bus1", "deg, Zero-Sequence Current, Angle"
      m_phii0_bus2 = "m:phii0:bus2", "deg, Zero-Sequence Current, Angle"
      m_i1r_bus1 = "m:i1r:bus1", "p.u., Positive-Sequence Current, Real Part"
      m_i1r_bus2 = "m:i1r:bus2", "p.u., Positive-Sequence Current, Real Part"
      m_i1i_bus1 = "m:i1i:bus1", "p.u., Positive-Sequence Current, Imaginary Part"
      m_i1i_bus2 = "m:i1i:bus2", "p.u., Positive-Sequence Current, Imaginary Part"
      m_i1_bus1 = "m:i1:bus1", "p.u., Positive-Sequence Current, Magnitude"
      m_i1_bus2 = "m:i1:bus2", "p.u., Positive-Sequence Current, Magnitude"
      m_I1_bus1 = "m:I1:bus1", "kA, Positive-Sequence Current, Magnitude"
      m_I1_bus2 = "m:I1:bus2", "kA, Positive-Sequence Current, Magnitude"
      m_phii1_bus1 = "m:phii1:bus1", "deg, Positive-Sequence Current, Angle"
      m_phii1_bus2 = "m:phii1:bus2", "deg, Positive-Sequence Current, Angle"
      m_i2r_bus1 = "m:i2r:bus1", "p.u., Negative-Sequence Current, Real Part"
      m_i2r_bus2 = "m:i2r:bus2", "p.u., Negative-Sequence Current, Real Part"
      m_i2i_bus1 = "m:i2i:bus1", "p.u., Negative-Sequence Current, Imaginary Part"
      m_i2i_bus2 = "m:i2i:bus2", "p.u., Negative-Sequence Current, Imaginary Part"
      m_i2_bus1 = "m:i2:bus1", "p.u., Negative-Sequence Current, Magnitude"
      m_i2_bus2 = "m:i2:bus2", "p.u., Negative-Sequence Current, Magnitude"
      m_I2_bus1 = "m:I2:bus1", "kA, Negative-Sequence Current, Magnitude"
      m_I2_bus2 = "m:I2:bus2", "kA, Negative-Sequence Current, Magnitude"
      m_phii2_bus1 = "m:phii2:bus1", "deg, Negative-Sequence Current, Angle"
      m_phii2_bus2 = "m:phii2:bus2", "deg, Negative-Sequence Current, Angle"
      m_ubfacI_bus1 = "m:ubfacI:bus1", "%, Current Unbalance Factor"
      m_ubfacI_bus2 = "m:ubfacI:bus2", "%, Current Unbalance Factor"
      m_ubfacS_bus1 = "m:ubfacS:bus1", "%, Power Unbalance Factor"
      m_ubfacS_bus2 = "m:ubfacS:bus2", "%, Power Unbalance Factor"
      m_P_bus1_A = "m:P:bus1:A", "MW, Active Power"
      m_P_bus1_B = "m:P:bus1:B", "MW, Active Power"
      m_P_bus1_C = "m:P:bus1:C", "MW, Active Power"
      m_P_bus1_N = "m:P:bus1:N", "MW, Active Power"
      m_P_bus2_A = "m:P:bus2:A", "MW, Active Power"
      m_P_bus2_B = "m:P:bus2:B", "MW, Active Power"
      m_P_bus2_C = "m:P:bus2:C", "MW, Active Power"
      m_P_bus2_N = "m:P:bus2:N", "MW, Active Power"
      m_Q_bus1_A = "m:Q:bus1:A", "Mvar, Reactive Power"
      m_Q_bus1_B = "m:Q:bus1:B", "Mvar, Reactive Power"
      m_Q_bus1_C = "m:Q:bus1:C", "Mvar, Reactive Power"
      m_Q_bus1_N = "m:Q:bus1:N", "Mvar, Reactive Power"
      m_Q_bus2_A = "m:Q:bus2:A", "Mvar, Reactive Power"
      m_Q_bus2_B = "m:Q:bus2:B", "Mvar, Reactive Power"
      m_Q_bus2_C = "m:Q:bus2:C", "Mvar, Reactive Power"
      m_Q_bus2_N = "m:Q:bus2:N", "Mvar, Reactive Power"
      m_S_bus1_A = "m:S:bus1:A", "MVA, Apparent Power"
      m_S_bus1_B = "m:S:bus1:B", "MVA, Apparent Power"
      m_S_bus1_C = "m:S:bus1:C", "MVA, Apparent Power"
      m_S_bus1_N = "m:S:bus1:N", "MVA, Apparent Power"
      m_S_bus2_A = "m:S:bus2:A", "MVA, Apparent Power"
      m_S_bus2_B = "m:S:bus2:B", "MVA, Apparent Power"
      m_S_bus2_C = "m:S:bus2:C", "MVA, Apparent Power"
      m_S_bus2_N = "m:S:bus2:N", "MVA, Apparent Power"
      m_cosphi_bus1_A = "m:cosphi:bus1:A", "Power Factor"
      m_cosphi_bus1_B = "m:cosphi:bus1:B", "Power Factor"
      m_cosphi_bus1_C = "m:cosphi:bus1:C", "Power Factor"
      m_cosphi_bus1_N = "m:cosphi:bus1:N", "Power Factor"
      m_cosphi_bus2_A = "m:cosphi:bus2:A", "Power Factor"
      m_cosphi_bus2_B = "m:cosphi:bus2:B", "Power Factor"
      m_cosphi_bus2_C = "m:cosphi:bus2:C", "Power Factor"
      m_cosphi_bus2_N = "m:cosphi:bus2:N", "Power Factor"
      m_tanphi_bus1_A = "m:tanphi:bus1:A", "tan(phi)"
      m_tanphi_bus1_B = "m:tanphi:bus1:B", "tan(phi)"
      m_tanphi_bus1_C = "m:tanphi:bus1:C", "tan(phi)"
      m_tanphi_bus1_N = "m:tanphi:bus1:N", "tan(phi)"
      m_tanphi_bus2_A = "m:tanphi:bus2:A", "tan(phi)"
      m_tanphi_bus2_B = "m:tanphi:bus2:B", "tan(phi)"
      m_tanphi_bus2_C = "m:tanphi:bus2:C", "tan(phi)"
      m_tanphi_bus2_N = "m:tanphi:bus2:N", "tan(phi)"
      m_Psum_bus1 = "m:Psum:bus1", "MW, Total Active Power"
      m_Psum_bus2 = "m:Psum:bus2", "MW, Total Active Power"
      m_Qsum_bus1 = "m:Qsum:bus1", "Mvar, Total Reactive Power"
      m_Qsum_bus2 = "m:Qsum:bus2", "Mvar, Total Reactive Power"
      m_Ssum_bus1 = "m:Ssum:bus1", "MVA, Total Apparent Power"
      m_Ssum_bus2 = "m:Ssum:bus2", "MVA, Total Apparent Power"
      m_cosphisum_bus1 = "m:cosphisum:bus1", "Total Power Factor"
      m_cosphisum_bus2 = "m:cosphisum:bus2", "Total Power Factor"
      m_tanphisum_bus1 = "m:tanphisum:bus1", "Total tan(phi)"
      m_tanphisum_bus2 = "m:tanphisum:bus2", "Total tan(phi)"
      m_Pln_bus1_A = "m:Pln:bus1:A", "MW, Active Power (Line-Neutral)"
      m_Pln_bus1_B = "m:Pln:bus1:B", "MW, Active Power (Line-Neutral)"
      m_Pln_bus1_C = "m:Pln:bus1:C", "MW, Active Power (Line-Neutral)"
      m_Pln_bus1_N = "m:Pln:bus1:N", "MW, Active Power (Line-Neutral)"
      m_Pln_bus2_A = "m:Pln:bus2:A", "MW, Active Power (Line-Neutral)"
      m_Pln_bus2_B = "m:Pln:bus2:B", "MW, Active Power (Line-Neutral)"
      m_Pln_bus2_C = "m:Pln:bus2:C", "MW, Active Power (Line-Neutral)"
      m_Pln_bus2_N = "m:Pln:bus2:N", "MW, Active Power (Line-Neutral)"
      m_Qln_bus1_A = "m:Qln:bus1:A", "Mvar, Reactive Power (Line-Neutral)"
      m_Qln_bus1_B = "m:Qln:bus1:B", "Mvar, Reactive Power (Line-Neutral)"
      m_Qln_bus1_C = "m:Qln:bus1:C", "Mvar, Reactive Power (Line-Neutral)"
      m_Qln_bus1_N = "m:Qln:bus1:N", "Mvar, Reactive Power (Line-Neutral)"
      m_Qln_bus2_A = "m:Qln:bus2:A", "Mvar, Reactive Power (Line-Neutral)"
      m_Qln_bus2_B = "m:Qln:bus2:B", "Mvar, Reactive Power (Line-Neutral)"
      m_Qln_bus2_C = "m:Qln:bus2:C", "Mvar, Reactive Power (Line-Neutral)"
      m_Qln_bus2_N = "m:Qln:bus2:N", "Mvar, Reactive Power (Line-Neutral)"
      m_P1_bus1 = "m:P1:bus1", "MW, Positive Sequence Active Power"
      m_P1_bus2 = "m:P1:bus2", "MW, Positive Sequence Active Power"
      m_Q1_bus1 = "m:Q1:bus1", "Mvar, Positive Sequence Reactive Power"
      m_Q1_bus2 = "m:Q1:bus2", "Mvar, Positive Sequence Reactive Power"
      m_P2_bus1 = "m:P2:bus1", "MW, Negative Sequence Active Power"
      m_P2_bus2 = "m:P2:bus2", "MW, Negative Sequence Active Power"
      m_Q2_bus1 = "m:Q2:bus1", "Mvar, Negative Sequence Reactive Power"
      m_Q2_bus2 = "m:Q2:bus2", "Mvar, Negative Sequence Reactive Power"
      m_P0_bus1 = "m:P0:bus1", "MW, Zero Sequence Active Power"
      m_P0_bus2 = "m:P0:bus2", "MW, Zero Sequence Active Power"
      m_Q0_bus1 = "m:Q0:bus1", "Mvar, Zero Sequence Reactive Power"
      m_Q0_bus2 = "m:Q0:bus2", "Mvar, Zero Sequence Reactive Power"
      m_i1P_bus1 = "m:i1P:bus1", "p.u., Positive-Sequence Active Current"
      m_i1P_bus2 = "m:i1P:bus2", "p.u., Positive-Sequence Active Current"
      m_i1Q_bus1 = "m:i1Q:bus1", "p.u., Positive-Sequence Reactive Current"
      m_i1Q_bus2 = "m:i1Q:bus2", "p.u., Positive-Sequence Reactive Current"
      m_I1P_bus1 = "m:I1P:bus1", "kA, Positive-Sequence Active Current"
      m_I1P_bus2 = "m:I1P:bus2", "kA, Positive-Sequence Active Current"
      m_I1Q_bus1 = "m:I1Q:bus1", "kA, Positive-Sequence Reactive Current"
      m_I1Q_bus2 = "m:I1Q:bus2", "kA, Positive-Sequence Reactive Current"
      m_i2P_bus1 = "m:i2P:bus1", "p.u., Negative-Sequence Active Current"
      m_i2P_bus2 = "m:i2P:bus2", "p.u., Negative-Sequence Active Current"
      m_i2Q_bus1 = "m:i2Q:bus1", "p.u., Negative-Sequence Reactive Current"
      m_i2Q_bus2 = "m:i2Q:bus2", "p.u., Negative-Sequence Reactive Current"
      m_I2P_bus1 = "m:I2P:bus1", "kA, Negative-Sequence Active Current"
      m_I2P_bus2 = "m:I2P:bus2", "kA, Negative-Sequence Active Current"
      m_I2Q_bus1 = "m:I2Q:bus1", "kA, Negative-Sequence Reactive Current"
      m_I2Q_bus2 = "m:I2Q:bus2", "kA, Negative-Sequence Reactive Current"
      m_ur_bus1_A = "m:ur:bus1:A", "p.u., Phase Voltage, Real Part"
      m_ur_bus1_B = "m:ur:bus1:B", "p.u., Phase Voltage, Real Part"
      m_ur_bus1_C = "m:ur:bus1:C", "p.u., Phase Voltage, Real Part"
      m_ur_bus1_N = "m:ur:bus1:N", "p.u., Phase Voltage, Real Part"
      m_ur_bus2_A = "m:ur:bus2:A", "p.u., Phase Voltage, Real Part"
      m_ur_bus2_B = "m:ur:bus2:B", "p.u., Phase Voltage, Real Part"
      m_ur_bus2_C = "m:ur:bus2:C", "p.u., Phase Voltage, Real Part"
      m_ur_bus2_N = "m:ur:bus2:N", "p.u., Phase Voltage, Real Part"
      m_ui_bus1_A = "m:ui:bus1:A", "p.u., Phase Voltage, Imaginary Part"
      m_ui_bus1_B = "m:ui:bus1:B", "p.u., Phase Voltage, Imaginary Part"
      m_ui_bus1_C = "m:ui:bus1:C", "p.u., Phase Voltage, Imaginary Part"
      m_ui_bus1_N = "m:ui:bus1:N", "p.u., Phase Voltage, Imaginary Part"
      m_ui_bus2_A = "m:ui:bus2:A", "p.u., Phase Voltage, Imaginary Part"
      m_ui_bus2_B = "m:ui:bus2:B", "p.u., Phase Voltage, Imaginary Part"
      m_ui_bus2_C = "m:ui:bus2:C", "p.u., Phase Voltage, Imaginary Part"
      m_ui_bus2_N = "m:ui:bus2:N", "p.u., Phase Voltage, Imaginary Part"
      m_u_bus1_A = "m:u:bus1:A", "p.u., Phase Voltage, Magnitude"
      m_u_bus1_B = "m:u:bus1:B", "p.u., Phase Voltage, Magnitude"
      m_u_bus1_C = "m:u:bus1:C", "p.u., Phase Voltage, Magnitude"
      m_u_bus1_N = "m:u:bus1:N", "p.u., Phase Voltage, Magnitude"
      m_u_bus2_A = "m:u:bus2:A", "p.u., Phase Voltage, Magnitude"
      m_u_bus2_B = "m:u:bus2:B", "p.u., Phase Voltage, Magnitude"
      m_u_bus2_C = "m:u:bus2:C", "p.u., Phase Voltage, Magnitude"
      m_u_bus2_N = "m:u:bus2:N", "p.u., Phase Voltage, Magnitude"
      m_ir_bus1_A = "m:ir:bus1:A", "p.u., Phase Current, Real Part"
      m_ir_bus1_B = "m:ir:bus1:B", "p.u., Phase Current, Real Part"
      m_ir_bus1_C = "m:ir:bus1:C", "p.u., Phase Current, Real Part"
      m_ir_bus1_N = "m:ir:bus1:N", "p.u., Phase Current, Real Part"
      m_ir_bus2_A = "m:ir:bus2:A", "p.u., Phase Current, Real Part"
      m_ir_bus2_B = "m:ir:bus2:B", "p.u., Phase Current, Real Part"
      m_ir_bus2_C = "m:ir:bus2:C", "p.u., Phase Current, Real Part"
      m_ir_bus2_N = "m:ir:bus2:N", "p.u., Phase Current, Real Part"
      m_ii_bus1_A = "m:ii:bus1:A", "p.u., Phase Current, Imaginary Part"
      m_ii_bus1_B = "m:ii:bus1:B", "p.u., Phase Current, Imaginary Part"
      m_ii_bus1_C = "m:ii:bus1:C", "p.u., Phase Current, Imaginary Part"
      m_ii_bus1_N = "m:ii:bus1:N", "p.u., Phase Current, Imaginary Part"
      m_ii_bus2_A = "m:ii:bus2:A", "p.u., Phase Current, Imaginary Part"
      m_ii_bus2_B = "m:ii:bus2:B", "p.u., Phase Current, Imaginary Part"
      m_ii_bus2_C = "m:ii:bus2:C", "p.u., Phase Current, Imaginary Part"
      m_ii_bus2_N = "m:ii:bus2:N", "p.u., Phase Current, Imaginary Part"
      m_i_bus1_A = "m:i:bus1:A", "p.u., Phase Current, Magnitude"
      m_i_bus1_B = "m:i:bus1:B", "p.u., Phase Current, Magnitude"
      m_i_bus1_C = "m:i:bus1:C", "p.u., Phase Current, Magnitude"
      m_i_bus1_N = "m:i:bus1:N", "p.u., Phase Current, Magnitude"
      m_i_bus2_A = "m:i:bus2:A", "p.u., Phase Current, Magnitude"
      m_i_bus2_B = "m:i:bus2:B", "p.u., Phase Current, Magnitude"
      m_i_bus2_C = "m:i:bus2:C", "p.u., Phase Current, Magnitude"
      m_i_bus2_N = "m:i:bus2:N", "p.u., Phase Current, Magnitude"
      m_phiui_bus1_A = "m:phiui:bus1:A", "deg, Angle between Voltage and Current"
      m_phiui_bus1_B = "m:phiui:bus1:B", "deg, Angle between Voltage and Current"
      m_phiui_bus1_C = "m:phiui:bus1:C", "deg, Angle between Voltage and Current"
      m_phiui_bus1_N = "m:phiui:bus1:N", "deg, Angle between Voltage and Current"
      m_phiui_bus2_A = "m:phiui:bus2:A", "deg, Angle between Voltage and Current"
      m_phiui_bus2_B = "m:phiui:bus2:B", "deg, Angle between Voltage and Current"
      m_phiui_bus2_C = "m:phiui:bus2:C", "deg, Angle between Voltage and Current"
      m_phiui_bus2_N = "m:phiui:bus2:N", "deg, Angle between Voltage and Current"
      m_phiu0i0_bus1 = "m:phiu0i0:bus1", "deg, Angle between Voltage and Current in zero-sequence system"
      m_phiu0i0_bus2 = "m:phiu0i0:bus2", "deg, Angle between Voltage and Current in zero-sequence system"
      m_phiu1i1_bus1 = "m:phiu1i1:bus1", "deg, Angle between Voltage and Current in positive sequence system"
      m_phiu1i1_bus2 = "m:phiu1i1:bus2", "deg, Angle between Voltage and Current in positive sequence system"
      m_phiu2i2_bus1 = "m:phiu2i2:bus1", "deg, Angle between Voltage and Current in negative sequence system"
      m_phiu2i2_bus2 = "m:phiu2i2:bus2", "deg, Angle between Voltage and Current in negative sequence system"
      m_Tfct_bus1 = "m:Tfct:bus1", "s, Fault Clearing Time"
      m_Tfct_bus2 = "m:Tfct:bus2", "s, Fault Clearing Time"
      m_TfctPh_bus1_A = "m:TfctPh:bus1:A", "s, Fault Clearing Time"
      m_TfctPh_bus1_B = "m:TfctPh:bus1:B", "s, Fault Clearing Time"
      m_TfctPh_bus1_C = "m:TfctPh:bus1:C", "s, Fault Clearing Time"
      m_TfctPh_bus1_N = "m:TfctPh:bus1:N", "s, Fault Clearing Time"
      m_TfctPh_bus2_A = "m:TfctPh:bus2:A", "s, Fault Clearing Time"
      m_TfctPh_bus2_B = "m:TfctPh:bus2:B", "s, Fault Clearing Time"
      m_TfctPh_bus2_C = "m:TfctPh:bus2:C", "s, Fault Clearing Time"
      m_TfctPh_bus2_N = "m:TfctPh:bus2:N", "s, Fault Clearing Time"
      m_Brkload_bus1 = "m:Brkload:bus1", "%, Max. Breaker Loading"
      m_Brkload_bus2 = "m:Brkload:bus2", "%, Max. Breaker Loading"
      m_BrkloadPh_bus1_A = "m:BrkloadPh:bus1:A", "%, Breaker Loading"
      m_BrkloadPh_bus1_B = "m:BrkloadPh:bus1:B", "%, Breaker Loading"
      m_BrkloadPh_bus1_C = "m:BrkloadPh:bus1:C", "%, Breaker Loading"
      m_BrkloadPh_bus1_N = "m:BrkloadPh:bus1:N", "%, Breaker Loading"
      m_BrkloadPh_bus2_A = "m:BrkloadPh:bus2:A", "%, Breaker Loading"
      m_BrkloadPh_bus2_B = "m:BrkloadPh:bus2:B", "%, Breaker Loading"
      m_BrkloadPh_bus2_C = "m:BrkloadPh:bus2:C", "%, Breaker Loading"
      m_BrkloadPh_bus2_N = "m:BrkloadPh:bus2:N", "%, Breaker Loading"
      n_ur_bus1_A = "n:ur:bus1:A", "p.u., Line-Ground Voltage, Real Part"
      n_ur_bus1_B = "n:ur:bus1:B", "p.u., Line-Ground Voltage, Real Part"
      n_ur_bus1_C = "n:ur:bus1:C", "p.u., Line-Ground Voltage, Real Part"
      n_ur_bus1_N = "n:ur:bus1:N", "p.u., Line-Ground Voltage, Real Part"
      n_ur_bus2_A = "n:ur:bus2:A", "p.u., Line-Ground Voltage, Real Part"
      n_ur_bus2_B = "n:ur:bus2:B", "p.u., Line-Ground Voltage, Real Part"
      n_ur_bus2_C = "n:ur:bus2:C", "p.u., Line-Ground Voltage, Real Part"
      n_ur_bus2_N = "n:ur:bus2:N", "p.u., Line-Ground Voltage, Real Part"
      n_ui_bus1_A = "n:ui:bus1:A", "p.u., Line-Ground Voltage, Imaginary Part"
      n_ui_bus1_B = "n:ui:bus1:B", "p.u., Line-Ground Voltage, Imaginary Part"
      n_ui_bus1_C = "n:ui:bus1:C", "p.u., Line-Ground Voltage, Imaginary Part"
      n_ui_bus1_N = "n:ui:bus1:N", "p.u., Line-Ground Voltage, Imaginary Part"
      n_ui_bus2_A = "n:ui:bus2:A", "p.u., Line-Ground Voltage, Imaginary Part"
      n_ui_bus2_B = "n:ui:bus2:B", "p.u., Line-Ground Voltage, Imaginary Part"
      n_ui_bus2_C = "n:ui:bus2:C", "p.u., Line-Ground Voltage, Imaginary Part"
      n_ui_bus2_N = "n:ui:bus2:N", "p.u., Line-Ground Voltage, Imaginary Part"
      n_u_bus1_A = "n:u:bus1:A", "p.u., Line-Ground Voltage, Magnitude"
      n_u_bus1_B = "n:u:bus1:B", "p.u., Line-Ground Voltage, Magnitude"
      n_u_bus1_C = "n:u:bus1:C", "p.u., Line-Ground Voltage, Magnitude"
      n_u_bus1_N = "n:u:bus1:N", "p.u., Line-Ground Voltage, Magnitude"
      n_u_bus2_A = "n:u:bus2:A", "p.u., Line-Ground Voltage, Magnitude"
      n_u_bus2_B = "n:u:bus2:B", "p.u., Line-Ground Voltage, Magnitude"
      n_u_bus2_C = "n:u:bus2:C", "p.u., Line-Ground Voltage, Magnitude"
      n_u_bus2_N = "n:u:bus2:N", "p.u., Line-Ground Voltage, Magnitude"
      n_upc_bus1_A = "n:upc:bus1:A", "%, Line-Ground Voltage, Magnitude"
      n_upc_bus1_B = "n:upc:bus1:B", "%, Line-Ground Voltage, Magnitude"
      n_upc_bus1_C = "n:upc:bus1:C", "%, Line-Ground Voltage, Magnitude"
      n_upc_bus1_N = "n:upc:bus1:N", "%, Line-Ground Voltage, Magnitude"
      n_upc_bus2_A = "n:upc:bus2:A", "%, Line-Ground Voltage, Magnitude"
      n_upc_bus2_B = "n:upc:bus2:B", "%, Line-Ground Voltage, Magnitude"
      n_upc_bus2_C = "n:upc:bus2:C", "%, Line-Ground Voltage, Magnitude"
      n_upc_bus2_N = "n:upc:bus2:N", "%, Line-Ground Voltage, Magnitude"
      n_U_bus1_A = "n:U:bus1:A", "kV, Line-Ground Voltage, Magnitude"
      n_U_bus1_B = "n:U:bus1:B", "kV, Line-Ground Voltage, Magnitude"
      n_U_bus1_C = "n:U:bus1:C", "kV, Line-Ground Voltage, Magnitude"
      n_U_bus1_N = "n:U:bus1:N", "kV, Line-Ground Voltage, Magnitude"
      n_U_bus2_A = "n:U:bus2:A", "kV, Line-Ground Voltage, Magnitude"
      n_U_bus2_B = "n:U:bus2:B", "kV, Line-Ground Voltage, Magnitude"
      n_U_bus2_C = "n:U:bus2:C", "kV, Line-Ground Voltage, Magnitude"
      n_U_bus2_N = "n:U:bus2:N", "kV, Line-Ground Voltage, Magnitude"
      n_phiu_bus1_A = "n:phiu:bus1:A", "deg, Line-Ground Voltage, Angle"
      n_phiu_bus1_B = "n:phiu:bus1:B", "deg, Line-Ground Voltage, Angle"
      n_phiu_bus1_C = "n:phiu:bus1:C", "deg, Line-Ground Voltage, Angle"
      n_phiu_bus1_N = "n:phiu:bus1:N", "deg, Line-Ground Voltage, Angle"
      n_phiu_bus2_A = "n:phiu:bus2:A", "deg, Line-Ground Voltage, Angle"
      n_phiu_bus2_B = "n:phiu:bus2:B", "deg, Line-Ground Voltage, Angle"
      n_phiu_bus2_C = "n:phiu:bus2:C", "deg, Line-Ground Voltage, Angle"
      n_phiu_bus2_N = "n:phiu:bus2:N", "deg, Line-Ground Voltage, Angle"
      n_phiurel_bus1 = "n:phiurel:bus1", "deg, Voltage, Relative Angle"
      n_phiurel_bus2 = "n:phiurel:bus2", "deg, Voltage, Relative Angle"
      n_ul_bus1_A = "n:ul:bus1:A", "p.u., Line-Line Voltage, Magnitude"
      n_ul_bus1_B = "n:ul:bus1:B", "p.u., Line-Line Voltage, Magnitude"
      n_ul_bus1_C = "n:ul:bus1:C", "p.u., Line-Line Voltage, Magnitude"
      n_ul_bus1_N = "n:ul:bus1:N", "p.u., Line-Line Voltage, Magnitude"
      n_ul_bus2_A = "n:ul:bus2:A", "p.u., Line-Line Voltage, Magnitude"
      n_ul_bus2_B = "n:ul:bus2:B", "p.u., Line-Line Voltage, Magnitude"
      n_ul_bus2_C = "n:ul:bus2:C", "p.u., Line-Line Voltage, Magnitude"
      n_ul_bus2_N = "n:ul:bus2:N", "p.u., Line-Line Voltage, Magnitude"
      n_ulpc_bus1_A = "n:ulpc:bus1:A", "%, Line-Line Voltage, Magnitude"
      n_ulpc_bus1_B = "n:ulpc:bus1:B", "%, Line-Line Voltage, Magnitude"
      n_ulpc_bus1_C = "n:ulpc:bus1:C", "%, Line-Line Voltage, Magnitude"
      n_ulpc_bus1_N = "n:ulpc:bus1:N", "%, Line-Line Voltage, Magnitude"
      n_ulpc_bus2_A = "n:ulpc:bus2:A", "%, Line-Line Voltage, Magnitude"
      n_ulpc_bus2_B = "n:ulpc:bus2:B", "%, Line-Line Voltage, Magnitude"
      n_ulpc_bus2_C = "n:ulpc:bus2:C", "%, Line-Line Voltage, Magnitude"
      n_ulpc_bus2_N = "n:ulpc:bus2:N", "%, Line-Line Voltage, Magnitude"
      n_Ul_bus1_A = "n:Ul:bus1:A", "kV, Line-Line Voltage, Magnitude"
      n_Ul_bus1_B = "n:Ul:bus1:B", "kV, Line-Line Voltage, Magnitude"
      n_Ul_bus1_C = "n:Ul:bus1:C", "kV, Line-Line Voltage, Magnitude"
      n_Ul_bus1_N = "n:Ul:bus1:N", "kV, Line-Line Voltage, Magnitude"
      n_Ul_bus2_A = "n:Ul:bus2:A", "kV, Line-Line Voltage, Magnitude"
      n_Ul_bus2_B = "n:Ul:bus2:B", "kV, Line-Line Voltage, Magnitude"
      n_Ul_bus2_C = "n:Ul:bus2:C", "kV, Line-Line Voltage, Magnitude"
      n_Ul_bus2_N = "n:Ul:bus2:N", "kV, Line-Line Voltage, Magnitude"
      n_phiul_bus1_A = "n:phiul:bus1:A", "deg, Line-Line Voltage, Angle"
      n_phiul_bus1_B = "n:phiul:bus1:B", "deg, Line-Line Voltage, Angle"
      n_phiul_bus1_C = "n:phiul:bus1:C", "deg, Line-Line Voltage, Angle"
      n_phiul_bus1_N = "n:phiul:bus1:N", "deg, Line-Line Voltage, Angle"
      n_phiul_bus2_A = "n:phiul:bus2:A", "deg, Line-Line Voltage, Angle"
      n_phiul_bus2_B = "n:phiul:bus2:B", "deg, Line-Line Voltage, Angle"
      n_phiul_bus2_C = "n:phiul:bus2:C", "deg, Line-Line Voltage, Angle"
      n_phiul_bus2_N = "n:phiul:bus2:N", "deg, Line-Line Voltage, Angle"
      n_u0_bus1 = "n:u0:bus1", "p.u., Zero-Sequence Voltage, Magnitude"
      n_u0_bus2 = "n:u0:bus2", "p.u., Zero-Sequence Voltage, Magnitude"
      n_U0_bus1 = "n:U0:bus1", "kV, Zero-Sequence Voltage, Magnitude"
      n_U0_bus2 = "n:U0:bus2", "kV, Zero-Sequence Voltage, Magnitude"
      n_U0x3_bus1 = "n:U0x3:bus1", "kV, 3*U0"
      n_U0x3_bus2 = "n:U0x3:bus2", "kV, 3*U0"
      n_phiu0_bus1 = "n:phiu0:bus1", "deg, Zero-Sequence Voltage, Angle"
      n_phiu0_bus2 = "n:phiu0:bus2", "deg, Zero-Sequence Voltage, Angle"
      n_u1_bus1 = "n:u1:bus1", "p.u., Positive-Sequence Voltage, Magnitude"
      n_u1_bus2 = "n:u1:bus2", "p.u., Positive-Sequence Voltage, Magnitude"
      n_u1pc_bus1 = "n:u1pc:bus1", "%, Positive-Sequence Voltage, Magnitude"
      n_u1pc_bus2 = "n:u1pc:bus2", "%, Positive-Sequence Voltage, Magnitude"
      n_u1r_bus1 = "n:u1r:bus1", "p.u., Positive-Sequence Voltage, Real Part"
      n_u1r_bus2 = "n:u1r:bus2", "p.u., Positive-Sequence Voltage, Real Part"
      n_u1i_bus1 = "n:u1i:bus1", "p.u., Positive-Sequence Voltage, Imaginary Part"
      n_u1i_bus2 = "n:u1i:bus2", "p.u., Positive-Sequence Voltage, Imaginary Part"
      n_U1_bus1 = "n:U1:bus1", "kV, Line-Ground Positive-Sequence Voltage, Magnitude"
      n_U1_bus2 = "n:U1:bus2", "kV, Line-Ground Positive-Sequence Voltage, Magnitude"
      n_phiu1_bus1 = "n:phiu1:bus1", "deg, Positive-Sequence Voltage, Angle"
      n_phiu1_bus2 = "n:phiu1:bus2", "deg, Positive-Sequence Voltage, Angle"
      n_u2_bus1 = "n:u2:bus1", "p.u., Negative-Sequence Voltage, Magnitude"
      n_u2_bus2 = "n:u2:bus2", "p.u., Negative-Sequence Voltage, Magnitude"
      n_U2_bus1 = "n:U2:bus1", "kV, Line-Ground Negative-Sequence Voltage, Magnitude"
      n_U2_bus2 = "n:U2:bus2", "kV, Line-Ground Negative-Sequence Voltage, Magnitude"
      n_phiu2_bus1 = "n:phiu2:bus1", "deg, Negative-Sequence Voltage, Angle"
      n_phiu2_bus2 = "n:phiu2:bus2", "deg, Negative-Sequence Voltage, Angle"
      n_U1l_bus1 = "n:U1l:bus1", "kV, Line-Line Positive-Sequence Voltage, Magnitude"
      n_U1l_bus2 = "n:U1l:bus2", "kV, Line-Line Positive-Sequence Voltage, Magnitude"
      n_U2l_bus1 = "n:U2l:bus1", "kV, Line-Line Negative-Sequence Voltage, Magnitude"
      n_U2l_bus2 = "n:U2l:bus2", "kV, Line-Line Negative-Sequence Voltage, Magnitude"
      n_uln_bus1_A = "n:uln:bus1:A", "p.u., Line-Neutral Voltage, Magnitude"
      n_uln_bus1_B = "n:uln:bus1:B", "p.u., Line-Neutral Voltage, Magnitude"
      n_uln_bus1_C = "n:uln:bus1:C", "p.u., Line-Neutral Voltage, Magnitude"
      n_uln_bus1_N = "n:uln:bus1:N", "p.u., Line-Neutral Voltage, Magnitude"
      n_uln_bus2_A = "n:uln:bus2:A", "p.u., Line-Neutral Voltage, Magnitude"
      n_uln_bus2_B = "n:uln:bus2:B", "p.u., Line-Neutral Voltage, Magnitude"
      n_uln_bus2_C = "n:uln:bus2:C", "p.u., Line-Neutral Voltage, Magnitude"
      n_uln_bus2_N = "n:uln:bus2:N", "p.u., Line-Neutral Voltage, Magnitude"
      n_Uln_bus1_A = "n:Uln:bus1:A", "kV, Line-Neutral Voltage, Magnitude"
      n_Uln_bus1_B = "n:Uln:bus1:B", "kV, Line-Neutral Voltage, Magnitude"
      n_Uln_bus1_C = "n:Uln:bus1:C", "kV, Line-Neutral Voltage, Magnitude"
      n_Uln_bus1_N = "n:Uln:bus1:N", "kV, Line-Neutral Voltage, Magnitude"
      n_Uln_bus2_A = "n:Uln:bus2:A", "kV, Line-Neutral Voltage, Magnitude"
      n_Uln_bus2_B = "n:Uln:bus2:B", "kV, Line-Neutral Voltage, Magnitude"
      n_Uln_bus2_C = "n:Uln:bus2:C", "kV, Line-Neutral Voltage, Magnitude"
      n_Uln_bus2_N = "n:Uln:bus2:N", "kV, Line-Neutral Voltage, Magnitude"
      n_phiuln_bus1_A = "n:phiuln:bus1:A", "deg, Line-Neutral Voltage, Angle"
      n_phiuln_bus1_B = "n:phiuln:bus1:B", "deg, Line-Neutral Voltage, Angle"
      n_phiuln_bus1_C = "n:phiuln:bus1:C", "deg, Line-Neutral Voltage, Angle"
      n_phiuln_bus1_N = "n:phiuln:bus1:N", "deg, Line-Neutral Voltage, Angle"
      n_phiuln_bus2_A = "n:phiuln:bus2:A", "deg, Line-Neutral Voltage, Angle"
      n_phiuln_bus2_B = "n:phiuln:bus2:B", "deg, Line-Neutral Voltage, Angle"
      n_phiuln_bus2_C = "n:phiuln:bus2:C", "deg, Line-Neutral Voltage, Angle"
      n_phiuln_bus2_N = "n:phiuln:bus2:N", "deg, Line-Neutral Voltage, Angle"
      n_un_bus1 = "n:un:bus1", "p.u., Neutral-Ground Voltage, Magnitude"
      n_un_bus2 = "n:un:bus2", "p.u., Neutral-Ground Voltage, Magnitude"
      n_Un_bus1 = "n:Un:bus1", "kV, Neutral-Ground Voltage, Magnitude"
      n_Un_bus2 = "n:Un:bus2", "kV, Neutral-Ground Voltage, Magnitude"
      n_phiun_bus1 = "n:phiun:bus1", "deg, Neutral-Ground Voltage, Angle"
      n_phiun_bus2 = "n:phiun:bus2", "deg, Neutral-Ground Voltage, Angle"
      n_upht_bus1_A = "n:upht:bus1:A", "p.u., Phase Technology dependent Voltage, Magnitude"
      n_upht_bus1_B = "n:upht:bus1:B", "p.u., Phase Technology dependent Voltage, Magnitude"
      n_upht_bus1_C = "n:upht:bus1:C", "p.u., Phase Technology dependent Voltage, Magnitude"
      n_upht_bus1_N = "n:upht:bus1:N", "p.u., Phase Technology dependent Voltage, Magnitude"
      n_upht_bus2_A = "n:upht:bus2:A", "p.u., Phase Technology dependent Voltage, Magnitude"
      n_upht_bus2_B = "n:upht:bus2:B", "p.u., Phase Technology dependent Voltage, Magnitude"
      n_upht_bus2_C = "n:upht:bus2:C", "p.u., Phase Technology dependent Voltage, Magnitude"
      n_upht_bus2_N = "n:upht:bus2:N", "p.u., Phase Technology dependent Voltage, Magnitude"
      n_Upht_bus1_A = "n:Upht:bus1:A", "kV, Phase Technology dependent Voltage, Magnitude"
      n_Upht_bus1_B = "n:Upht:bus1:B", "kV, Phase Technology dependent Voltage, Magnitude"
      n_Upht_bus1_C = "n:Upht:bus1:C", "kV, Phase Technology dependent Voltage, Magnitude"
      n_Upht_bus1_N = "n:Upht:bus1:N", "kV, Phase Technology dependent Voltage, Magnitude"
      n_Upht_bus2_A = "n:Upht:bus2:A", "kV, Phase Technology dependent Voltage, Magnitude"
      n_Upht_bus2_B = "n:Upht:bus2:B", "kV, Phase Technology dependent Voltage, Magnitude"
      n_Upht_bus2_C = "n:Upht:bus2:C", "kV, Phase Technology dependent Voltage, Magnitude"
      n_Upht_bus2_N = "n:Upht:bus2:N", "kV, Phase Technology dependent Voltage, Magnitude"
      n_um_bus1 = "n:um:bus1", "p.u., Average-Voltage, Magnitude"
      n_um_bus2 = "n:um:bus2", "p.u., Average-Voltage, Magnitude"
      n_Um_bus1 = "n:Um:bus1", "kV, Average-Voltage, Magnitude"
      n_Um_bus2 = "n:Um:bus2", "kV, Average-Voltage, Magnitude"
      n_uphtmin_bus1 = "n:uphtmin:bus1", "p.u., Minimum of Phase Technology dependent Voltage, Magnitude"
      n_uphtmin_bus2 = "n:uphtmin:bus2", "p.u., Minimum of Phase Technology dependent Voltage, Magnitude"
      n_uphtmax_bus1 = "n:uphtmax:bus1", "p.u., Maximum of Phase Technology dependent Voltage, Magnitude"
      n_uphtmax_bus2 = "n:uphtmax:bus2", "p.u., Maximum of Phase Technology dependent Voltage, Magnitude"
      n_Pgen_bus1 = "n:Pgen:bus1", "MW, Generation, Active Power"
      n_Pgen_bus2 = "n:Pgen:bus2", "MW, Generation, Active Power"
      n_Qgen_bus1 = "n:Qgen:bus1", "Mvar, Generation, Reactive Power"
      n_Qgen_bus2 = "n:Qgen:bus2", "Mvar, Generation, Reactive Power"
      n_Pmot_bus1 = "n:Pmot:bus1", "MW, Motor Load, Active Power"
      n_Pmot_bus2 = "n:Pmot:bus2", "MW, Motor Load, Active Power"
      n_Qmot_bus1 = "n:Qmot:bus1", "Mvar, Motor Load, Reactive Power"
      n_Qmot_bus2 = "n:Qmot:bus2", "Mvar, Motor Load, Reactive Power"
      n_Pload_bus1 = "n:Pload:bus1", "MW, General Load, Active Power"
      n_Pload_bus2 = "n:Pload:bus2", "MW, General Load, Active Power"
      n_Qload_bus1 = "n:Qload:bus1", "Mvar, General Load, Reactive Power"
      n_Qload_bus2 = "n:Qload:bus2", "Mvar, General Load, Reactive Power"
      n_Pcomp_bus1 = "n:Pcomp:bus1", "MW, Compensation (Losses)"
      n_Pcomp_bus2 = "n:Pcomp:bus2", "MW, Compensation (Losses)"
      n_Qcomp_bus1 = "n:Qcomp:bus1", "Mvar, Compensation"
      n_Qcomp_bus2 = "n:Qcomp:bus2", "Mvar, Compensation"
      n_Pnet_bus1 = "n:Pnet:bus1", "MW, External Networks, Active Power"
      n_Pnet_bus2 = "n:Pnet:bus2", "MW, External Networks, Active Power"
      n_Qnet_bus1 = "n:Qnet:bus1", "Mvar, External Networks, Reactive Power"
      n_Qnet_bus2 = "n:Qnet:bus2", "Mvar, External Networks, Reactive Power"
      n_Pflow_bus1 = "n:Pflow:bus1", "MW, Power Flow, Active Power"
      n_Pflow_bus2 = "n:Pflow:bus2", "MW, Power Flow, Active Power"
      n_Qflow_bus1 = "n:Qflow:bus1", "Mvar, Power Flow, Reactive Power"
      n_Qflow_bus2 = "n:Qflow:bus2", "Mvar, Power Flow, Reactive Power"
      n_Pout_bus1 = "n:Pout:bus1", "MW, Outgoing Flow, Active Power"
      n_Pout_bus2 = "n:Pout:bus2", "MW, Outgoing Flow, Active Power"
      n_Qout_bus1 = "n:Qout:bus1", "Mvar, Outgoing Power, Reactive Power"
      n_Qout_bus2 = "n:Qout:bus2", "Mvar, Outgoing Power, Reactive Power"
      n_Sout_bus1 = "n:Sout:bus1", "MVA, Outgoing Power, Apparent Power"
      n_Sout_bus2 = "n:Sout:bus2", "MVA, Outgoing Power, Apparent Power"
      n_cosphiout_bus1 = "n:cosphiout:bus1", "Outgoing Power, Power Factor"
      n_cosphiout_bus2 = "n:cosphiout:bus2", "Outgoing Power, Power Factor"
      n_Pbalance_bus1 = "n:Pbalance:bus1", "MW, Active Power Balance (=0)"
      n_Pbalance_bus2 = "n:Pbalance:bus2", "MW, Active Power Balance (=0)"
      n_Qbalance_bus1 = "n:Qbalance:bus1", "Mvar, Reactive Power Balance (=0)"
      n_Qbalance_bus2 = "n:Qbalance:bus2", "Mvar, Reactive Power Balance (=0)"
      n_ubfac_bus1 = "n:ubfac:bus1", "%, Unbalance Factor"
      n_ubfac_bus2 = "n:ubfac:bus2", "%, Unbalance Factor"
      n_LossPdown_bus1 = "n:LossPdown:bus1", "MW, Losses, downstream"
      n_LossPdown_bus2 = "n:LossPdown:bus2", "MW, Losses, downstream"
      n_LossQdown_bus1 = "n:LossQdown:bus1", "Mvar, Losses, downstream (Reactive Power)"
      n_LossQdown_bus2 = "n:LossQdown:bus2", "Mvar, Losses, downstream (Reactive Power)"
      n_LossPdownload_bus1 = "n:LossPdownload:bus1", "MW, Load losses, downstream"
      n_LossPdownload_bus2 = "n:LossPdownload:bus2", "MW, Load losses, downstream"
      n_LossQdownload_bus1 = "n:LossQdownload:bus1", "Mvar, Load losses, downstream"
      n_LossQdownload_bus2 = "n:LossQdownload:bus2", "Mvar, Load losses, downstream"
      n_LossPdownnoload_bus1 = "n:LossPdownnoload:bus1", "MW, No load losses, downstream"
      n_LossPdownnoload_bus2 = "n:LossPdownnoload:bus2", "MW, No load losses, downstream"
      n_LossQdownnoload_bus1 = "n:LossQdownnoload:bus1", "Mvar, No load losses, downstream"
      n_LossQdownnoload_bus2 = "n:LossQdownnoload:bus2", "Mvar, No load losses, downstream"
      c_R_on = "c:R_on", "Ohm, On Resistance"
      c_X_on = "c:X_on", "Ohm, On Reactance"
      c_on_off = "c:on_off", "Flag Closed"
      c_Inom = "c:Inom", "kA, Rated Current"
      c_loading = "c:loading", "%, Max. Loading"
      c_loadingPh_A = "c:loadingPh:A", "%, Loading"
      c_loadingPh_B = "c:loadingPh:B", "%, Loading"
      c_loadingPh_C = "c:loadingPh:C", "%, Loading"
      c_loadingPh_N = "c:loadingPh:N", "%, Loading"
      c_loading_st = "c:loading_st", "%, Short Term Loading"
      c_maxload = "c:maxload", "%, Max. Loading"
      c_Ploss = "c:Ploss", "MW, Losses (total)"
      c_Qloss = "c:Qloss", "Mvar, Reactive-Losses (total)"
      c_Plossld = "c:Plossld", "MW, Losses (load)"
      c_Qlossld = "c:Qlossld", "Mvar, Reactive-Losses (load)"
      c_Plossnld = "c:Plossnld", "MW, Losses (no load)"
      c_Qlossnld = "c:Qlossnld", "Mvar, Reactive-Losses (no load)"
      e_cpFeed = "e:cpFeed", "Feeder"
      e_ciDist = "e:ciDist", "Distance from infeed in number of buses"
      e_ciLater = "e:ciLater", "Lateral Index"
      e_ciDistRoot = "e:ciDistRoot", "Distance from first infeed in number of buses"
      e_ciDistAll = "e:ciDistAll", "Distance from infeed in number of buses including switches"
      e_ciDistAllRoot = "e:ciDistAllRoot", "Distance from first infeed in number of buses including switches"
      r_cpFeed = "r:cpFeed", "Feeder"
      m_Ikss_bus1 = "m:Ikss:bus1", "kA, Initial Short-Circuit Current"
      m_Ikss_bus2 = "m:Ikss:bus2", "kA, Initial Short-Circuit Current"
      m_ikss_bus1 = "m:ikss:bus1", "p.u., Initial Short-Circuit Current, Magnitude"
      m_ikss_bus2 = "m:ikss:bus2", "p.u., Initial Short-Circuit Current, Magnitude"
      m_phii_bus1 = "m:phii:bus1", "deg, Phase Current, Angle"
      m_phii_bus2 = "m:phii:bus2", "deg, Phase Current, Angle"
      m_maxIkss_bus1 = "m:maxIkss:bus1", "kA, Max. Initial Short-Circuit Current"
      m_maxIkss_bus2 = "m:maxIkss:bus2", "kA, Max. Initial Short-Circuit Current"
      m_Skss_bus1 = "m:Skss:bus1", "MVA, Initial Short-Circuit Power"
      m_Skss_bus2 = "m:Skss:bus2", "MVA, Initial Short-Circuit Power"
      m_ip_bus1 = "m:ip:bus1", "kA, Peak Short-Circuit Current"
      m_ip_bus2 = "m:ip:bus2", "kA, Peak Short-Circuit Current"
      m_maxip_bus1 = "m:maxip:bus1", "kA, Max. Peak Short-Circuit Current"
      m_maxip_bus2 = "m:maxip:bus2", "kA, Max. Peak Short-Circuit Current"
      m_maxIb_bus1 = "m:maxIb:bus1", "kA, Max. Breaking Current"
      m_maxIb_bus2 = "m:maxIb:bus2", "kA, Max. Breaking Current"
      m_maxIth_bus1 = "m:maxIth:bus1", "kA, Max. Thermal equivalent Short-Circuit Current"
      m_maxIth_bus2 = "m:maxIth:bus2", "kA, Max. Thermal equivalent Short-Circuit Current"
      m_Ib_bus1 = "m:Ib:bus1", "kA, Short-Circuit Breaking Current"
      m_Ib_bus2 = "m:Ib:bus2", "kA, Short-Circuit Breaking Current"
      m_Sb_bus1 = "m:Sb:bus1", "MVA, Short-Circuit Breaking Power"
      m_Sb_bus2 = "m:Sb:bus2", "MVA, Short-Circuit Breaking Power"
      m_Ik_bus1 = "m:Ik:bus1", "kA, Steady-State Short-Circuit Current"
      m_Ik_bus2 = "m:Ik:bus2", "kA, Steady-State Short-Circuit Current"
      m_Ith_bus1 = "m:Ith:bus1", "kA, Thermal equivalent Short-Circuit Current"
      m_Ith_bus2 = "m:Ith:bus2", "kA, Thermal equivalent Short-Circuit Current"
      m_I_bus1 = "m:I:bus1", "kA, Short-Circuit Current"
      m_I_bus2 = "m:I:bus2", "kA, Short-Circuit Current"
      m_R_bus1 = "m:R:bus1", "Ohm, Fault Impedance, Real Part"
      m_R_bus2 = "m:R:bus2", "Ohm, Fault Impedance, Real Part"
      m_X_bus1 = "m:X:bus1", "Ohm, Fault Impedance, Imaginary Part"
      m_X_bus2 = "m:X:bus2", "Ohm, Fault Impedance, Imaginary Part"
      m_Z_bus1 = "m:Z:bus1", "Ohm, Fault Impedance, Magnitude"
      m_Z_bus2 = "m:Z:bus2", "Ohm, Fault Impedance, Magnitude"
      m_phiz_bus1 = "m:phiz:bus1", "deg, Fault Impedance, Angle"
      m_phiz_bus2 = "m:phiz:bus2", "deg, Fault Impedance, Angle"
      m_zSbase_bus1 = "m:zSbase:bus1", "p.u., Fault Impedance, Magnitude"
      m_zSbase_bus2 = "m:zSbase:bus2", "p.u., Fault Impedance, Magnitude"
      m_XtoR_bus1 = "m:XtoR:bus1", "X/R ratio"
      m_XtoR_bus2 = "m:XtoR:bus2", "X/R ratio"
      m_Zs_bus1 = "m:Zs:bus1", "Ohm, Source Impedance, Magnitude"
      m_Zs_bus2 = "m:Zs:bus2", "Ohm, Source Impedance, Magnitude"
      m_phizs_bus1 = "m:phizs:bus1", "deg, Source Impedance, Angle"
      m_phizs_bus2 = "m:phizs:bus2", "deg, Source Impedance, Angle"
      m_Rs_bus1 = "m:Rs:bus1", "Ohm, Source Impedance, Real Part"
      m_Rs_bus2 = "m:Rs:bus2", "Ohm, Source Impedance, Real Part"
      m_Xs_bus1 = "m:Xs:bus1", "Ohm, Source Impedance, Imaginary Part"
      m_Xs_bus2 = "m:Xs:bus2", "Ohm, Source Impedance, Imaginary Part"
      m_phiui_bus1 = "m:phiui:bus1", "deg, Angle between Voltage and Current"
      m_phiui_bus2 = "m:phiui:bus2", "deg, Angle between Voltage and Current"
      m_cosphiui_bus1 = "m:cosphiui:bus1", "cosphi"
      m_cosphiui_bus2 = "m:cosphiui:bus2", "cosphi"
      m_P_bus1 = "m:P:bus1", "MW, Active Power"
      m_P_bus2 = "m:P:bus2", "MW, Active Power"
      m_Q_bus1 = "m:Q:bus1", "Mvar, Reactive Power"
      m_Q_bus2 = "m:Q:bus2", "Mvar, Reactive Power"
      m_BrkIpload_bus1 = "m:BrkIpload:bus1", "%, Breaker Loading (peak)"
      m_BrkIpload_bus2 = "m:BrkIpload:bus2", "%, Breaker Loading (peak)"
      m_BrkIbload_bus1 = "m:BrkIbload:bus1", "%, Breaker Loading (breaking)"
      m_BrkIbload_bus2 = "m:BrkIbload:bus2", "%, Breaker Loading (breaking)"
      m_BrkmaxIb_bus1 = "m:BrkmaxIb:bus1", "kA, Max. Breaking Current (breaker time)"
      m_BrkmaxIb_bus2 = "m:BrkmaxIb:bus2", "kA, Max. Breaking Current (breaker time)"
      n_Ikss_bus1 = "n:Ikss:bus1", "kA, Initial Short-Circuit Current"
      n_Ikss_bus2 = "n:Ikss:bus2", "kA, Initial Short-Circuit Current"
      n_I_bus1 = "n:I:bus1", "kA, Short-Circuit Current"
      n_I_bus2 = "n:I:bus2", "kA, Short-Circuit Current"
      n_phii_bus1 = "n:phii:bus1", "deg, Phase Current, Angle"
      n_phii_bus2 = "n:phii:bus2", "deg, Phase Current, Angle"
      n_Skss_bus1 = "n:Skss:bus1", "MVA, Initial Short-Circuit Power"
      n_Skss_bus2 = "n:Skss:bus2", "MVA, Initial Short-Circuit Power"
      n_ip_bus1 = "n:ip:bus1", "kA, Peak Short-Circuit Current"
      n_ip_bus2 = "n:ip:bus2", "kA, Peak Short-Circuit Current"
      n_Ib_bus1 = "n:Ib:bus1", "kA, Short-Circuit Breaking Current"
      n_Ib_bus2 = "n:Ib:bus2", "kA, Short-Circuit Breaking Current"
      n_Sb_bus1 = "n:Sb:bus1", "MVA, Short-Circuit Breaking Power"
      n_Sb_bus2 = "n:Sb:bus2", "MVA, Short-Circuit Breaking Power"
      n_Ik_bus1 = "n:Ik:bus1", "kA, Steady-State Short-Circuit Current"
      n_Ik_bus2 = "n:Ik:bus2", "kA, Steady-State Short-Circuit Current"
      n_Ith_bus1 = "n:Ith:bus1", "kA, Thermal equivalent Short-Circuit Current"
      n_Ith_bus2 = "n:Ith:bus2", "kA, Thermal equivalent Short-Circuit Current"
      n_R_bus1 = "n:R:bus1", "Ohm, Short-Circuit Impedance, Real Part"
      n_R_bus2 = "n:R:bus2", "Ohm, Short-Circuit Impedance, Real Part"
      n_X_bus1 = "n:X:bus1", "Ohm, Short-Circuit Impedance, Imaginary Part"
      n_X_bus2 = "n:X:bus2", "Ohm, Short-Circuit Impedance, Imaginary Part"
      n_Z_bus1 = "n:Z:bus1", "Ohm, Short-Circuit Impedance, Magnitude"
      n_Z_bus2 = "n:Z:bus2", "Ohm, Short-Circuit Impedance, Magnitude"
      n_phiz_bus1 = "n:phiz:bus1", "deg, Short-Circuit Impedance, Angle"
      n_phiz_bus2 = "n:phiz:bus2", "deg, Short-Circuit Impedance, Angle"
      n_rSbase_bus1 = "n:rSbase:bus1", "p.u., Short-Circuit Impedance, Real Part"
      n_rSbase_bus2 = "n:rSbase:bus2", "p.u., Short-Circuit Impedance, Real Part"
      n_xSbase_bus1 = "n:xSbase:bus1", "p.u., Short-Circuit Impedance, Imaginary Part"
      n_xSbase_bus2 = "n:xSbase:bus2", "p.u., Short-Circuit Impedance, Imaginary Part"
      n_zSbase_bus1 = "n:zSbase:bus1", "p.u., Short-Circuit Impedance, Magnitude"
      n_zSbase_bus2 = "n:zSbase:bus2", "p.u., Short-Circuit Impedance, Magnitude"
      n_XtoR_bus1 = "n:XtoR:bus1", "X/R ratio (peak)"
      n_XtoR_bus2 = "n:XtoR:bus2", "X/R ratio (peak)"
      n_RtoX_bus1 = "n:RtoX:bus1", "R/X ratio (peak)"
      n_RtoX_bus2 = "n:RtoX:bus2", "R/X ratio (peak)"
      n_ur_bus1 = "n:ur:bus1", "p.u., Voltage, Real Part"
      n_ur_bus2 = "n:ur:bus2", "p.u., Voltage, Real Part"
      n_ui_bus1 = "n:ui:bus1", "p.u., Voltage, Imaginary Part"
      n_ui_bus2 = "n:ui:bus2", "p.u., Voltage, Imaginary Part"
      n_u_bus1 = "n:u:bus1", "p.u., Voltage, Magnitude"
      n_u_bus2 = "n:u:bus2", "p.u., Voltage, Magnitude"
      n_upc_bus1 = "n:upc:bus1", "%, Voltage, Magnitude"
      n_upc_bus2 = "n:upc:bus2", "%, Voltage, Magnitude"
      n_U_bus1 = "n:U:bus1", "kV, Line-Ground Voltage, Magnitude"
      n_U_bus2 = "n:U:bus2", "kV, Line-Ground Voltage, Magnitude"
      n_Ul_bus1 = "n:Ul:bus1", "kV, Line-Line Voltage, Magnitude"
      n_Ul_bus2 = "n:Ul:bus2", "kV, Line-Line Voltage, Magnitude"
      n_phiu_bus1 = "n:phiu:bus1", "deg, Voltage, Angle"
      n_phiu_bus2 = "n:phiu:bus2", "deg, Voltage, Angle"
      n_du_bus1 = "n:du:bus1", "%, Voltage Deviation"
      n_du_bus2 = "n:du:bus2", "%, Voltage Deviation"
      n_urpref_bus1 = "n:urpref:bus1", "p.u., Pre-fault Voltage, Real Part"
      n_urpref_bus2 = "n:urpref:bus2", "p.u., Pre-fault Voltage, Real Part"
      n_uipref_bus1 = "n:uipref:bus1", "p.u., Pre-fault Voltage, Imaginary Part"
      n_uipref_bus2 = "n:uipref:bus2", "p.u., Pre-fault Voltage, Imaginary Part"
      n_upref_bus1 = "n:upref:bus1", "p.u., Pre-fault Voltage, Magnitude"
      n_upref_bus2 = "n:upref:bus2", "p.u., Pre-fault Voltage, Magnitude"
      n_uprefpc_bus1 = "n:uprefpc:bus1", "%, Pre-fault Voltage, Magnitude"
      n_uprefpc_bus2 = "n:uprefpc:bus2", "%, Pre-fault Voltage, Magnitude"
      n_Upref_bus1 = "n:Upref:bus1", "kV, Line-Ground Pre-fault Voltage, Magnitude"
      n_Upref_bus2 = "n:Upref:bus2", "kV, Line-Ground Pre-fault Voltage, Magnitude"
      n_Ulpref_bus1 = "n:Ulpref:bus1", "kV, Line-Line Pre-fault Voltage, Magnitude"
      n_Ulpref_bus2 = "n:Ulpref:bus2", "kV, Line-Line Pre-fault Voltage, Magnitude"
      n_phiupref_bus1 = "n:phiupref:bus1", "deg, Pre-fault Voltage, Angle"
      n_phiupref_bus2 = "n:phiupref:bus2", "deg, Pre-fault Voltage, Angle"
      n_phiui_bus1 = "n:phiui:bus1", "deg, Angle between Voltage and Current"
      n_phiui_bus2 = "n:phiui:bus2", "deg, Angle between Voltage and Current"
      n_cfac_bus1 = "n:cfac:bus1", "Voltage Factor"
      n_cfac_bus2 = "n:cfac:bus2", "Voltage Factor"
      n_idc_bus1 = "n:idc:bus1", "kA, Decaying Component"
      n_idc_bus2 = "n:idc:bus2", "kA, Decaying Component"
      n_Iasy_bus1 = "n:Iasy:bus1", "kA, Asymmetrical Breaking Current"
      n_Iasy_bus2 = "n:Iasy:bus2", "kA, Asymmetrical Breaking Current"
      n_Ithload_bus1 = "n:Ithload:bus1", "%, Loading, Thermal equivalent Current"
      n_Ithload_bus2 = "n:Ithload:bus2", "%, Loading, Thermal equivalent Current"
      n_Ipload_bus1 = "n:Ipload:bus1", "%, Loading, Peak Short-Circuit Current"
      n_Ipload_bus2 = "n:Ipload:bus2", "%, Loading, Peak Short-Circuit Current"
      n_Ithrtk_bus1 = "n:Ithrtk:bus1", "kA, Rated Short-Time Current (Tk)"
      n_Ithrtk_bus2 = "n:Ithrtk:bus2", "kA, Rated Short-Time Current (Tk)"
      n_Tn_bus1 = "n:Tn:bus1", "ms, Grid Time Constant"
      n_Tn_bus2 = "n:Tn:bus2", "ms, Grid Time Constant"
      n_XtoR_b_bus1 = "n:XtoR_b:bus1", "X/R ratio (dc)"
      n_XtoR_b_bus2 = "n:XtoR_b:bus2", "X/R ratio (dc)"
      n_RtoX_b_bus1 = "n:RtoX_b:bus1", "R/X ratio (dc)"
      n_RtoX_b_bus2 = "n:RtoX_b:bus2", "R/X ratio (dc)"
      n_fctof_b_bus1 = "n:fctof_b:bus1", "Equivalent Frequency fc/f (dc)"
      n_fctof_b_bus2 = "n:fctof_b:bus2", "Equivalent Frequency fc/f (dc)"
      c_t_open = "c:t_open", "s, Opening Time"
      c_Ibload = "c:Ibload", "%, Loading, Interrupting"
      c_Ipload = "c:Ipload", "%, Loading, Peak Short-Circuit Current"
      c_Tb = "c:Tb", "s, Break Time"
      c_XtoR = "c:XtoR", "X/R ratio (dc)"
      c_XtoR_p = "c:XtoR_p", "X/R ratio (peak)"
      c_Ibmax = "c:Ibmax", "kA, Breaking Current"
      c_ibmax = "c:ibmax", "kA, Peak Breaking Current"
      c_Ibasymmax = "c:Ibasymmax", "kA, Asymmetrical Breaking Current"
      c_idcmax = "c:idcmax", "kA, Decaying Component"
      c_Ikssmax = "c:Ikssmax", "kA, Initial Short-Circuit Current"
      c_ipmax = "c:ipmax", "kA, Peak Short-Circuit Current"
      c_fctof = "c:fctof", "Equivalent Frequency fc/f (dc)"
      e_Tb = "e:Tb", "s, Break Time"
      m_Ikss_bus1_A = "m:Ikss:bus1:A", "kA, Initial Short-Circuit Current"
      m_Ikss_bus1_B = "m:Ikss:bus1:B", "kA, Initial Short-Circuit Current"
      m_Ikss_bus1_C = "m:Ikss:bus1:C", "kA, Initial Short-Circuit Current"
      m_Ikss_bus1_N = "m:Ikss:bus1:N", "kA, Initial Short-Circuit Current"
      m_Ikss_bus2_A = "m:Ikss:bus2:A", "kA, Initial Short-Circuit Current"
      m_Ikss_bus2_B = "m:Ikss:bus2:B", "kA, Initial Short-Circuit Current"
      m_Ikss_bus2_C = "m:Ikss:bus2:C", "kA, Initial Short-Circuit Current"
      m_Ikss_bus2_N = "m:Ikss:bus2:N", "kA, Initial Short-Circuit Current"
      m_ikss_bus1_A = "m:ikss:bus1:A", "p.u., Initial Short-Circuit Current, Magnitude"
      m_ikss_bus1_B = "m:ikss:bus1:B", "p.u., Initial Short-Circuit Current, Magnitude"
      m_ikss_bus1_C = "m:ikss:bus1:C", "p.u., Initial Short-Circuit Current, Magnitude"
      m_ikss_bus1_N = "m:ikss:bus1:N", "p.u., Initial Short-Circuit Current, Magnitude"
      m_ikss_bus2_A = "m:ikss:bus2:A", "p.u., Initial Short-Circuit Current, Magnitude"
      m_ikss_bus2_B = "m:ikss:bus2:B", "p.u., Initial Short-Circuit Current, Magnitude"
      m_ikss_bus2_C = "m:ikss:bus2:C", "p.u., Initial Short-Circuit Current, Magnitude"
      m_ikss_bus2_N = "m:ikss:bus2:N", "p.u., Initial Short-Circuit Current, Magnitude"
      m_Skss_bus1_A = "m:Skss:bus1:A", "MVA, Initial Short-Circuit Power"
      m_Skss_bus1_B = "m:Skss:bus1:B", "MVA, Initial Short-Circuit Power"
      m_Skss_bus1_C = "m:Skss:bus1:C", "MVA, Initial Short-Circuit Power"
      m_Skss_bus1_N = "m:Skss:bus1:N", "MVA, Initial Short-Circuit Power"
      m_Skss_bus2_A = "m:Skss:bus2:A", "MVA, Initial Short-Circuit Power"
      m_Skss_bus2_B = "m:Skss:bus2:B", "MVA, Initial Short-Circuit Power"
      m_Skss_bus2_C = "m:Skss:bus2:C", "MVA, Initial Short-Circuit Power"
      m_Skss_bus2_N = "m:Skss:bus2:N", "MVA, Initial Short-Circuit Power"
      m_Skssx3_bus1_A = "m:Skssx3:bus1:A", "MVA, 3*Initial Short-Circuit Power"
      m_Skssx3_bus1_B = "m:Skssx3:bus1:B", "MVA, 3*Initial Short-Circuit Power"
      m_Skssx3_bus1_C = "m:Skssx3:bus1:C", "MVA, 3*Initial Short-Circuit Power"
      m_Skssx3_bus1_N = "m:Skssx3:bus1:N", "MVA, 3*Initial Short-Circuit Power"
      m_Skssx3_bus2_A = "m:Skssx3:bus2:A", "MVA, 3*Initial Short-Circuit Power"
      m_Skssx3_bus2_B = "m:Skssx3:bus2:B", "MVA, 3*Initial Short-Circuit Power"
      m_Skssx3_bus2_C = "m:Skssx3:bus2:C", "MVA, 3*Initial Short-Circuit Power"
      m_Skssx3_bus2_N = "m:Skssx3:bus2:N", "MVA, 3*Initial Short-Circuit Power"
      m_ip_bus1_A = "m:ip:bus1:A", "kA, Peak Short-Circuit Current"
      m_ip_bus1_B = "m:ip:bus1:B", "kA, Peak Short-Circuit Current"
      m_ip_bus1_C = "m:ip:bus1:C", "kA, Peak Short-Circuit Current"
      m_ip_bus1_N = "m:ip:bus1:N", "kA, Peak Short-Circuit Current"
      m_ip_bus2_A = "m:ip:bus2:A", "kA, Peak Short-Circuit Current"
      m_ip_bus2_B = "m:ip:bus2:B", "kA, Peak Short-Circuit Current"
      m_ip_bus2_C = "m:ip:bus2:C", "kA, Peak Short-Circuit Current"
      m_ip_bus2_N = "m:ip:bus2:N", "kA, Peak Short-Circuit Current"
      m_Z_bus1_A = "m:Z:bus1:A", "Ohm, Fault Impedance, Magnitude"
      m_Z_bus1_B = "m:Z:bus1:B", "Ohm, Fault Impedance, Magnitude"
      m_Z_bus1_C = "m:Z:bus1:C", "Ohm, Fault Impedance, Magnitude"
      m_Z_bus1_N = "m:Z:bus1:N", "Ohm, Fault Impedance, Magnitude"
      m_Z_bus2_A = "m:Z:bus2:A", "Ohm, Fault Impedance, Magnitude"
      m_Z_bus2_B = "m:Z:bus2:B", "Ohm, Fault Impedance, Magnitude"
      m_Z_bus2_C = "m:Z:bus2:C", "Ohm, Fault Impedance, Magnitude"
      m_Z_bus2_N = "m:Z:bus2:N", "Ohm, Fault Impedance, Magnitude"
      m_phiz_bus1_A = "m:phiz:bus1:A", "deg, Fault Impedance, Angle"
      m_phiz_bus1_B = "m:phiz:bus1:B", "deg, Fault Impedance, Angle"
      m_phiz_bus1_C = "m:phiz:bus1:C", "deg, Fault Impedance, Angle"
      m_phiz_bus1_N = "m:phiz:bus1:N", "deg, Fault Impedance, Angle"
      m_phiz_bus2_A = "m:phiz:bus2:A", "deg, Fault Impedance, Angle"
      m_phiz_bus2_B = "m:phiz:bus2:B", "deg, Fault Impedance, Angle"
      m_phiz_bus2_C = "m:phiz:bus2:C", "deg, Fault Impedance, Angle"
      m_phiz_bus2_N = "m:phiz:bus2:N", "deg, Fault Impedance, Angle"
      m_Zl_bus1_A = "m:Zl:bus1:A", "Ohm, Line-Line Fault Impedance, Magnitude"
      m_Zl_bus1_B = "m:Zl:bus1:B", "Ohm, Line-Line Fault Impedance, Magnitude"
      m_Zl_bus1_C = "m:Zl:bus1:C", "Ohm, Line-Line Fault Impedance, Magnitude"
      m_Zl_bus1_N = "m:Zl:bus1:N", "Ohm, Line-Line Fault Impedance, Magnitude"
      m_Zl_bus2_A = "m:Zl:bus2:A", "Ohm, Line-Line Fault Impedance, Magnitude"
      m_Zl_bus2_B = "m:Zl:bus2:B", "Ohm, Line-Line Fault Impedance, Magnitude"
      m_Zl_bus2_C = "m:Zl:bus2:C", "Ohm, Line-Line Fault Impedance, Magnitude"
      m_Zl_bus2_N = "m:Zl:bus2:N", "Ohm, Line-Line Fault Impedance, Magnitude"
      m_phizl_bus1_A = "m:phizl:bus1:A", "deg, Line-Line Fault Impedance, Angle"
      m_phizl_bus1_B = "m:phizl:bus1:B", "deg, Line-Line Fault Impedance, Angle"
      m_phizl_bus1_C = "m:phizl:bus1:C", "deg, Line-Line Fault Impedance, Angle"
      m_phizl_bus1_N = "m:phizl:bus1:N", "deg, Line-Line Fault Impedance, Angle"
      m_phizl_bus2_A = "m:phizl:bus2:A", "deg, Line-Line Fault Impedance, Angle"
      m_phizl_bus2_B = "m:phizl:bus2:B", "deg, Line-Line Fault Impedance, Angle"
      m_phizl_bus2_C = "m:phizl:bus2:C", "deg, Line-Line Fault Impedance, Angle"
      m_phizl_bus2_N = "m:phizl:bus2:N", "deg, Line-Line Fault Impedance, Angle"
      m_Z0_bus1 = "m:Z0:bus1", "Ohm, Zero-Sequence Fault Impedance, Magnitude"
      m_Z0_bus2 = "m:Z0:bus2", "Ohm, Zero-Sequence Fault Impedance, Magnitude"
      m_phiz0_bus1 = "m:phiz0:bus1", "deg, Zero-Sequence Fault Impedance, Angle"
      m_phiz0_bus2 = "m:phiz0:bus2", "deg, Zero-Sequence Fault Impedance, Angle"
      m_z0Sbase_bus1 = "m:z0Sbase:bus1", "p.u., Zero-Sequence Fault Impedance, Magnitude"
      m_z0Sbase_bus2 = "m:z0Sbase:bus2", "p.u., Zero-Sequence Fault Impedance, Magnitude"
      m_cosphiu1i1_bus1 = "m:cosphiu1i1:bus1", "Positive-Sequence cosphi"
      m_cosphiu1i1_bus2 = "m:cosphiu1i1:bus2", "Positive-Sequence cosphi"
      m_I0x3watt_bus1 = "m:I0x3watt:bus1", "kA, Earth-Fault Residual Current"
      m_I0x3watt_bus2 = "m:I0x3watt:bus2", "kA, Earth-Fault Residual Current"
      m_I0x3var_bus1 = "m:I0x3var:bus1", "kA, Capacitive Earth-Fault Current"
      m_I0x3var_bus2 = "m:I0x3var:bus2", "kA, Capacitive Earth-Fault Current"
      m_Q0cap_bus1 = "m:Q0cap:bus1", "Mvar, Q0 capacitive"
      m_Q0cap_bus2 = "m:Q0cap:bus2", "Mvar, Q0 capacitive"
      m_Zfrel_bus1 = "m:Zfrel:bus1", "Ohm, Relay Impedance, Magnitude"
      m_Zfrel_bus2 = "m:Zfrel:bus2", "Ohm, Relay Impedance, Magnitude"
      m_phizfrel_bus1 = "m:phizfrel:bus1", "deg, Relay Impedance, Angle"
      m_phizfrel_bus2 = "m:phizfrel:bus2", "deg, Relay Impedance, Angle"
      m_Rfrel_bus1 = "m:Rfrel:bus1", "Ohm, Relay Impedance, Real Part"
      m_Rfrel_bus2 = "m:Rfrel:bus2", "Ohm, Relay Impedance, Real Part"
      m_Xfrel_bus1 = "m:Xfrel:bus1", "Ohm, Relay Impedance, Imaginary Part"
      m_Xfrel_bus2 = "m:Xfrel:bus2", "Ohm, Relay Impedance, Imaginary Part"
      m_Zf_bus1 = "m:Zf:bus1", "Ohm, Meas. Fault Impedance, Magnitude"
      m_Zf_bus2 = "m:Zf:bus2", "Ohm, Meas. Fault Impedance, Magnitude"
      m_phizf_bus1 = "m:phizf:bus1", "deg, Meas. Fault Impedance, Angle"
      m_phizf_bus2 = "m:phizf:bus2", "deg, Meas. Fault Impedance, Angle"
      n_Ikss_bus1_A = "n:Ikss:bus1:A", "kA, Initial Short-Circuit Current"
      n_Ikss_bus1_B = "n:Ikss:bus1:B", "kA, Initial Short-Circuit Current"
      n_Ikss_bus1_C = "n:Ikss:bus1:C", "kA, Initial Short-Circuit Current"
      n_Ikss_bus1_N = "n:Ikss:bus1:N", "kA, Initial Short-Circuit Current"
      n_Ikss_bus2_A = "n:Ikss:bus2:A", "kA, Initial Short-Circuit Current"
      n_Ikss_bus2_B = "n:Ikss:bus2:B", "kA, Initial Short-Circuit Current"
      n_Ikss_bus2_C = "n:Ikss:bus2:C", "kA, Initial Short-Circuit Current"
      n_Ikss_bus2_N = "n:Ikss:bus2:N", "kA, Initial Short-Circuit Current"
      n_I_bus1_A = "n:I:bus1:A", "kA, Short-Circuit Current"
      n_I_bus1_B = "n:I:bus1:B", "kA, Short-Circuit Current"
      n_I_bus1_C = "n:I:bus1:C", "kA, Short-Circuit Current"
      n_I_bus1_N = "n:I:bus1:N", "kA, Short-Circuit Current"
      n_I_bus2_A = "n:I:bus2:A", "kA, Short-Circuit Current"
      n_I_bus2_B = "n:I:bus2:B", "kA, Short-Circuit Current"
      n_I_bus2_C = "n:I:bus2:C", "kA, Short-Circuit Current"
      n_I_bus2_N = "n:I:bus2:N", "kA, Short-Circuit Current"
      n_phii_bus1_A = "n:phii:bus1:A", "deg, Phase Current, Angle"
      n_phii_bus1_B = "n:phii:bus1:B", "deg, Phase Current, Angle"
      n_phii_bus1_C = "n:phii:bus1:C", "deg, Phase Current, Angle"
      n_phii_bus1_N = "n:phii:bus1:N", "deg, Phase Current, Angle"
      n_phii_bus2_A = "n:phii:bus2:A", "deg, Phase Current, Angle"
      n_phii_bus2_B = "n:phii:bus2:B", "deg, Phase Current, Angle"
      n_phii_bus2_C = "n:phii:bus2:C", "deg, Phase Current, Angle"
      n_phii_bus2_N = "n:phii:bus2:N", "deg, Phase Current, Angle"
      n_Skss_bus1_A = "n:Skss:bus1:A", "MVA, Initial Short-Circuit Power"
      n_Skss_bus1_B = "n:Skss:bus1:B", "MVA, Initial Short-Circuit Power"
      n_Skss_bus1_C = "n:Skss:bus1:C", "MVA, Initial Short-Circuit Power"
      n_Skss_bus1_N = "n:Skss:bus1:N", "MVA, Initial Short-Circuit Power"
      n_Skss_bus2_A = "n:Skss:bus2:A", "MVA, Initial Short-Circuit Power"
      n_Skss_bus2_B = "n:Skss:bus2:B", "MVA, Initial Short-Circuit Power"
      n_Skss_bus2_C = "n:Skss:bus2:C", "MVA, Initial Short-Circuit Power"
      n_Skss_bus2_N = "n:Skss:bus2:N", "MVA, Initial Short-Circuit Power"
      n_Skssx3_bus1_A = "n:Skssx3:bus1:A", "MVA, 3*Initial Short-Circuit Power"
      n_Skssx3_bus1_B = "n:Skssx3:bus1:B", "MVA, 3*Initial Short-Circuit Power"
      n_Skssx3_bus1_C = "n:Skssx3:bus1:C", "MVA, 3*Initial Short-Circuit Power"
      n_Skssx3_bus1_N = "n:Skssx3:bus1:N", "MVA, 3*Initial Short-Circuit Power"
      n_Skssx3_bus2_A = "n:Skssx3:bus2:A", "MVA, 3*Initial Short-Circuit Power"
      n_Skssx3_bus2_B = "n:Skssx3:bus2:B", "MVA, 3*Initial Short-Circuit Power"
      n_Skssx3_bus2_C = "n:Skssx3:bus2:C", "MVA, 3*Initial Short-Circuit Power"
      n_Skssx3_bus2_N = "n:Skssx3:bus2:N", "MVA, 3*Initial Short-Circuit Power"
      n_ip_bus1_A = "n:ip:bus1:A", "kA, Peak Short-Circuit Current"
      n_ip_bus1_B = "n:ip:bus1:B", "kA, Peak Short-Circuit Current"
      n_ip_bus1_C = "n:ip:bus1:C", "kA, Peak Short-Circuit Current"
      n_ip_bus1_N = "n:ip:bus1:N", "kA, Peak Short-Circuit Current"
      n_ip_bus2_A = "n:ip:bus2:A", "kA, Peak Short-Circuit Current"
      n_ip_bus2_B = "n:ip:bus2:B", "kA, Peak Short-Circuit Current"
      n_ip_bus2_C = "n:ip:bus2:C", "kA, Peak Short-Circuit Current"
      n_ip_bus2_N = "n:ip:bus2:N", "kA, Peak Short-Circuit Current"
      n_Ib_bus1_A = "n:Ib:bus1:A", "kA, Short-Circuit Breaking Current"
      n_Ib_bus1_B = "n:Ib:bus1:B", "kA, Short-Circuit Breaking Current"
      n_Ib_bus1_C = "n:Ib:bus1:C", "kA, Short-Circuit Breaking Current"
      n_Ib_bus1_N = "n:Ib:bus1:N", "kA, Short-Circuit Breaking Current"
      n_Ib_bus2_A = "n:Ib:bus2:A", "kA, Short-Circuit Breaking Current"
      n_Ib_bus2_B = "n:Ib:bus2:B", "kA, Short-Circuit Breaking Current"
      n_Ib_bus2_C = "n:Ib:bus2:C", "kA, Short-Circuit Breaking Current"
      n_Ib_bus2_N = "n:Ib:bus2:N", "kA, Short-Circuit Breaking Current"
      n_Sb_bus1_A = "n:Sb:bus1:A", "MVA, Short-Circuit Breaking Power"
      n_Sb_bus1_B = "n:Sb:bus1:B", "MVA, Short-Circuit Breaking Power"
      n_Sb_bus1_C = "n:Sb:bus1:C", "MVA, Short-Circuit Breaking Power"
      n_Sb_bus1_N = "n:Sb:bus1:N", "MVA, Short-Circuit Breaking Power"
      n_Sb_bus2_A = "n:Sb:bus2:A", "MVA, Short-Circuit Breaking Power"
      n_Sb_bus2_B = "n:Sb:bus2:B", "MVA, Short-Circuit Breaking Power"
      n_Sb_bus2_C = "n:Sb:bus2:C", "MVA, Short-Circuit Breaking Power"
      n_Sb_bus2_N = "n:Sb:bus2:N", "MVA, Short-Circuit Breaking Power"
      n_Ith_bus1_A = "n:Ith:bus1:A", "kA, Thermal equivalent Short-Circuit Current"
      n_Ith_bus1_B = "n:Ith:bus1:B", "kA, Thermal equivalent Short-Circuit Current"
      n_Ith_bus1_C = "n:Ith:bus1:C", "kA, Thermal equivalent Short-Circuit Current"
      n_Ith_bus1_N = "n:Ith:bus1:N", "kA, Thermal equivalent Short-Circuit Current"
      n_Ith_bus2_A = "n:Ith:bus2:A", "kA, Thermal equivalent Short-Circuit Current"
      n_Ith_bus2_B = "n:Ith:bus2:B", "kA, Thermal equivalent Short-Circuit Current"
      n_Ith_bus2_C = "n:Ith:bus2:C", "kA, Thermal equivalent Short-Circuit Current"
      n_Ith_bus2_N = "n:Ith:bus2:N", "kA, Thermal equivalent Short-Circuit Current"
      n_Ithmax_bus1 = "n:Ithmax:bus1", "kA, Max. Thermal equivalent Short-Circuit Current"
      n_Ithmax_bus2 = "n:Ithmax:bus2", "kA, Max. Thermal equivalent Short-Circuit Current"
      n_Ik_bus1_A = "n:Ik:bus1:A", "kA, Steady-State Short-Circuit Current"
      n_Ik_bus1_B = "n:Ik:bus1:B", "kA, Steady-State Short-Circuit Current"
      n_Ik_bus1_C = "n:Ik:bus1:C", "kA, Steady-State Short-Circuit Current"
      n_Ik_bus1_N = "n:Ik:bus1:N", "kA, Steady-State Short-Circuit Current"
      n_Ik_bus2_A = "n:Ik:bus2:A", "kA, Steady-State Short-Circuit Current"
      n_Ik_bus2_B = "n:Ik:bus2:B", "kA, Steady-State Short-Circuit Current"
      n_Ik_bus2_C = "n:Ik:bus2:C", "kA, Steady-State Short-Circuit Current"
      n_Ik_bus2_N = "n:Ik:bus2:N", "kA, Steady-State Short-Circuit Current"
      n_R0_bus1 = "n:R0:bus1", "Ohm, Zero-Sequence Impedance, Real Part"
      n_R0_bus2 = "n:R0:bus2", "Ohm, Zero-Sequence Impedance, Real Part"
      n_X0_bus1 = "n:X0:bus1", "Ohm, Zero-Sequence Impedance, Imaginary Part"
      n_X0_bus2 = "n:X0:bus2", "Ohm, Zero-Sequence Impedance, Imaginary Part"
      n_Z0_bus1 = "n:Z0:bus1", "Ohm, Zero-Sequence Impedance, Magnitude"
      n_Z0_bus2 = "n:Z0:bus2", "Ohm, Zero-Sequence Impedance, Magnitude"
      n_phiz0_bus1 = "n:phiz0:bus1", "deg, Zero-Sequence Impedance, Angle"
      n_phiz0_bus2 = "n:phiz0:bus2", "deg, Zero-Sequence Impedance, Angle"
      n_r0Sbase_bus1 = "n:r0Sbase:bus1", "p.u., Zero-Sequence Impedance, Real Part"
      n_r0Sbase_bus2 = "n:r0Sbase:bus2", "p.u., Zero-Sequence Impedance, Real Part"
      n_x0Sbase_bus1 = "n:x0Sbase:bus1", "p.u., Zero-Sequence Impedance, Imaginary Part"
      n_x0Sbase_bus2 = "n:x0Sbase:bus2", "p.u., Zero-Sequence Impedance, Imaginary Part"
      n_z0Sbase_bus1 = "n:z0Sbase:bus1", "p.u., Zero-Sequence Impedance, Magnitude"
      n_z0Sbase_bus2 = "n:z0Sbase:bus2", "p.u., Zero-Sequence Impedance, Magnitude"
      n_R1_bus1 = "n:R1:bus1", "Ohm, Positive-Sequence Impedance, Real Part"
      n_R1_bus2 = "n:R1:bus2", "Ohm, Positive-Sequence Impedance, Real Part"
      n_X1_bus1 = "n:X1:bus1", "Ohm, Positive-Sequence Impedance, Imaginary Part"
      n_X1_bus2 = "n:X1:bus2", "Ohm, Positive-Sequence Impedance, Imaginary Part"
      n_Z1_bus1 = "n:Z1:bus1", "Ohm, Positive-Sequence Impedance, Magnitude"
      n_Z1_bus2 = "n:Z1:bus2", "Ohm, Positive-Sequence Impedance, Magnitude"
      n_phiz1_bus1 = "n:phiz1:bus1", "deg, Positive-Sequence Impedance, Angle"
      n_phiz1_bus2 = "n:phiz1:bus2", "deg, Positive-Sequence Impedance, Angle"
      n_r1Sbase_bus1 = "n:r1Sbase:bus1", "p.u., Positive-Sequence Impedance, Real Part"
      n_r1Sbase_bus2 = "n:r1Sbase:bus2", "p.u., Positive-Sequence Impedance, Real Part"
      n_x1Sbase_bus1 = "n:x1Sbase:bus1", "p.u., Positive-Sequence Impedance, Imaginary Part"
      n_x1Sbase_bus2 = "n:x1Sbase:bus2", "p.u., Positive-Sequence Impedance, Imaginary Part"
      n_z1Sbase_bus1 = "n:z1Sbase:bus1", "p.u., Positive-Sequence Impedance, Magnitude"
      n_z1Sbase_bus2 = "n:z1Sbase:bus2", "p.u., Positive-Sequence Impedance, Magnitude"
      n_R2_bus1 = "n:R2:bus1", "Ohm, Negative-Sequence Impedance, Real Part"
      n_R2_bus2 = "n:R2:bus2", "Ohm, Negative-Sequence Impedance, Real Part"
      n_X2_bus1 = "n:X2:bus1", "Ohm, Negative-Sequence Impedance, Imaginary Part"
      n_X2_bus2 = "n:X2:bus2", "Ohm, Negative-Sequence Impedance, Imaginary Part"
      n_Z2_bus1 = "n:Z2:bus1", "Ohm, Negative-Sequence Impedance, Magnitude"
      n_Z2_bus2 = "n:Z2:bus2", "Ohm, Negative-Sequence Impedance, Magnitude"
      n_phiz2_bus1 = "n:phiz2:bus1", "deg, Negative-Sequence Impedance, Angle"
      n_phiz2_bus2 = "n:phiz2:bus2", "deg, Negative-Sequence Impedance, Angle"
      n_r2Sbase_bus1 = "n:r2Sbase:bus1", "p.u., Negative-Sequence Impedance, Real Part"
      n_r2Sbase_bus2 = "n:r2Sbase:bus2", "p.u., Negative-Sequence Impedance, Real Part"
      n_x2Sbase_bus1 = "n:x2Sbase:bus1", "p.u., Negative-Sequence Impedance, Imaginary Part"
      n_x2Sbase_bus2 = "n:x2Sbase:bus2", "p.u., Negative-Sequence Impedance, Imaginary Part"
      n_z2Sbase_bus1 = "n:z2Sbase:bus1", "p.u., Negative-Sequence Impedance, Magnitude"
      n_z2Sbase_bus2 = "n:z2Sbase:bus2", "p.u., Negative-Sequence Impedance, Magnitude"
      n_R_bus1_A = "n:R:bus1:A", "Ohm, Network Resistance, Line-Ground"
      n_R_bus1_B = "n:R:bus1:B", "Ohm, Network Resistance, Line-Ground"
      n_R_bus1_C = "n:R:bus1:C", "Ohm, Network Resistance, Line-Ground"
      n_R_bus1_N = "n:R:bus1:N", "Ohm, Network Resistance, Line-Ground"
      n_R_bus2_A = "n:R:bus2:A", "Ohm, Network Resistance, Line-Ground"
      n_R_bus2_B = "n:R:bus2:B", "Ohm, Network Resistance, Line-Ground"
      n_R_bus2_C = "n:R:bus2:C", "Ohm, Network Resistance, Line-Ground"
      n_R_bus2_N = "n:R:bus2:N", "Ohm, Network Resistance, Line-Ground"
      n_X_bus1_A = "n:X:bus1:A", "Ohm, Network Reactance, Line-Ground"
      n_X_bus1_B = "n:X:bus1:B", "Ohm, Network Reactance, Line-Ground"
      n_X_bus1_C = "n:X:bus1:C", "Ohm, Network Reactance, Line-Ground"
      n_X_bus1_N = "n:X:bus1:N", "Ohm, Network Reactance, Line-Ground"
      n_X_bus2_A = "n:X:bus2:A", "Ohm, Network Reactance, Line-Ground"
      n_X_bus2_B = "n:X:bus2:B", "Ohm, Network Reactance, Line-Ground"
      n_X_bus2_C = "n:X:bus2:C", "Ohm, Network Reactance, Line-Ground"
      n_X_bus2_N = "n:X:bus2:N", "Ohm, Network Reactance, Line-Ground"
      n_Z_bus1_A = "n:Z:bus1:A", "Ohm, Network Impedance, Line-Ground"
      n_Z_bus1_B = "n:Z:bus1:B", "Ohm, Network Impedance, Line-Ground"
      n_Z_bus1_C = "n:Z:bus1:C", "Ohm, Network Impedance, Line-Ground"
      n_Z_bus1_N = "n:Z:bus1:N", "Ohm, Network Impedance, Line-Ground"
      n_Z_bus2_A = "n:Z:bus2:A", "Ohm, Network Impedance, Line-Ground"
      n_Z_bus2_B = "n:Z:bus2:B", "Ohm, Network Impedance, Line-Ground"
      n_Z_bus2_C = "n:Z:bus2:C", "Ohm, Network Impedance, Line-Ground"
      n_Z_bus2_N = "n:Z:bus2:N", "Ohm, Network Impedance, Line-Ground"
      n_phiz_bus1_A = "n:phiz:bus1:A", "deg, Angle of Network Impedance, Line-Ground"
      n_phiz_bus1_B = "n:phiz:bus1:B", "deg, Angle of Network Impedance, Line-Ground"
      n_phiz_bus1_C = "n:phiz:bus1:C", "deg, Angle of Network Impedance, Line-Ground"
      n_phiz_bus1_N = "n:phiz:bus1:N", "deg, Angle of Network Impedance, Line-Ground"
      n_phiz_bus2_A = "n:phiz:bus2:A", "deg, Angle of Network Impedance, Line-Ground"
      n_phiz_bus2_B = "n:phiz:bus2:B", "deg, Angle of Network Impedance, Line-Ground"
      n_phiz_bus2_C = "n:phiz:bus2:C", "deg, Angle of Network Impedance, Line-Ground"
      n_phiz_bus2_N = "n:phiz:bus2:N", "deg, Angle of Network Impedance, Line-Ground"
      n_Rl_bus1_A = "n:Rl:bus1:A", "Ohm, Network Resistance, Line-Line"
      n_Rl_bus1_B = "n:Rl:bus1:B", "Ohm, Network Resistance, Line-Line"
      n_Rl_bus1_C = "n:Rl:bus1:C", "Ohm, Network Resistance, Line-Line"
      n_Rl_bus1_N = "n:Rl:bus1:N", "Ohm, Network Resistance, Line-Line"
      n_Rl_bus2_A = "n:Rl:bus2:A", "Ohm, Network Resistance, Line-Line"
      n_Rl_bus2_B = "n:Rl:bus2:B", "Ohm, Network Resistance, Line-Line"
      n_Rl_bus2_C = "n:Rl:bus2:C", "Ohm, Network Resistance, Line-Line"
      n_Rl_bus2_N = "n:Rl:bus2:N", "Ohm, Network Resistance, Line-Line"
      n_Xl_bus1_A = "n:Xl:bus1:A", "Ohm, Network Reactance, Line-Line"
      n_Xl_bus1_B = "n:Xl:bus1:B", "Ohm, Network Reactance, Line-Line"
      n_Xl_bus1_C = "n:Xl:bus1:C", "Ohm, Network Reactance, Line-Line"
      n_Xl_bus1_N = "n:Xl:bus1:N", "Ohm, Network Reactance, Line-Line"
      n_Xl_bus2_A = "n:Xl:bus2:A", "Ohm, Network Reactance, Line-Line"
      n_Xl_bus2_B = "n:Xl:bus2:B", "Ohm, Network Reactance, Line-Line"
      n_Xl_bus2_C = "n:Xl:bus2:C", "Ohm, Network Reactance, Line-Line"
      n_Xl_bus2_N = "n:Xl:bus2:N", "Ohm, Network Reactance, Line-Line"
      n_Zl_bus1_A = "n:Zl:bus1:A", "Ohm, Network Impedance, Line-Line"
      n_Zl_bus1_B = "n:Zl:bus1:B", "Ohm, Network Impedance, Line-Line"
      n_Zl_bus1_C = "n:Zl:bus1:C", "Ohm, Network Impedance, Line-Line"
      n_Zl_bus1_N = "n:Zl:bus1:N", "Ohm, Network Impedance, Line-Line"
      n_Zl_bus2_A = "n:Zl:bus2:A", "Ohm, Network Impedance, Line-Line"
      n_Zl_bus2_B = "n:Zl:bus2:B", "Ohm, Network Impedance, Line-Line"
      n_Zl_bus2_C = "n:Zl:bus2:C", "Ohm, Network Impedance, Line-Line"
      n_Zl_bus2_N = "n:Zl:bus2:N", "Ohm, Network Impedance, Line-Line"
      n_phizl_bus1_A = "n:phizl:bus1:A", "deg, Angle of Network Impedance, Line-Line"
      n_phizl_bus1_B = "n:phizl:bus1:B", "deg, Angle of Network Impedance, Line-Line"
      n_phizl_bus1_C = "n:phizl:bus1:C", "deg, Angle of Network Impedance, Line-Line"
      n_phizl_bus1_N = "n:phizl:bus1:N", "deg, Angle of Network Impedance, Line-Line"
      n_phizl_bus2_A = "n:phizl:bus2:A", "deg, Angle of Network Impedance, Line-Line"
      n_phizl_bus2_B = "n:phizl:bus2:B", "deg, Angle of Network Impedance, Line-Line"
      n_phizl_bus2_C = "n:phizl:bus2:C", "deg, Angle of Network Impedance, Line-Line"
      n_phizl_bus2_N = "n:phizl:bus2:N", "deg, Angle of Network Impedance, Line-Line"
      n_Rln_bus1_A = "n:Rln:bus1:A", "Ohm, Network Resistance, Line-Neutral"
      n_Rln_bus1_B = "n:Rln:bus1:B", "Ohm, Network Resistance, Line-Neutral"
      n_Rln_bus1_C = "n:Rln:bus1:C", "Ohm, Network Resistance, Line-Neutral"
      n_Rln_bus1_N = "n:Rln:bus1:N", "Ohm, Network Resistance, Line-Neutral"
      n_Rln_bus2_A = "n:Rln:bus2:A", "Ohm, Network Resistance, Line-Neutral"
      n_Rln_bus2_B = "n:Rln:bus2:B", "Ohm, Network Resistance, Line-Neutral"
      n_Rln_bus2_C = "n:Rln:bus2:C", "Ohm, Network Resistance, Line-Neutral"
      n_Rln_bus2_N = "n:Rln:bus2:N", "Ohm, Network Resistance, Line-Neutral"
      n_Xln_bus1_A = "n:Xln:bus1:A", "Ohm, Network Reactance, Line-Neutral"
      n_Xln_bus1_B = "n:Xln:bus1:B", "Ohm, Network Reactance, Line-Neutral"
      n_Xln_bus1_C = "n:Xln:bus1:C", "Ohm, Network Reactance, Line-Neutral"
      n_Xln_bus1_N = "n:Xln:bus1:N", "Ohm, Network Reactance, Line-Neutral"
      n_Xln_bus2_A = "n:Xln:bus2:A", "Ohm, Network Reactance, Line-Neutral"
      n_Xln_bus2_B = "n:Xln:bus2:B", "Ohm, Network Reactance, Line-Neutral"
      n_Xln_bus2_C = "n:Xln:bus2:C", "Ohm, Network Reactance, Line-Neutral"
      n_Xln_bus2_N = "n:Xln:bus2:N", "Ohm, Network Reactance, Line-Neutral"
      n_Zln_bus1_A = "n:Zln:bus1:A", "Ohm, Network Impedance, Line-Neutral"
      n_Zln_bus1_B = "n:Zln:bus1:B", "Ohm, Network Impedance, Line-Neutral"
      n_Zln_bus1_C = "n:Zln:bus1:C", "Ohm, Network Impedance, Line-Neutral"
      n_Zln_bus1_N = "n:Zln:bus1:N", "Ohm, Network Impedance, Line-Neutral"
      n_Zln_bus2_A = "n:Zln:bus2:A", "Ohm, Network Impedance, Line-Neutral"
      n_Zln_bus2_B = "n:Zln:bus2:B", "Ohm, Network Impedance, Line-Neutral"
      n_Zln_bus2_C = "n:Zln:bus2:C", "Ohm, Network Impedance, Line-Neutral"
      n_Zln_bus2_N = "n:Zln:bus2:N", "Ohm, Network Impedance, Line-Neutral"
      n_phizln_bus1_A = "n:phizln:bus1:A", "deg, Angle of Network Impedance, Line-Neutral"
      n_phizln_bus1_B = "n:phizln:bus1:B", "deg, Angle of Network Impedance, Line-Neutral"
      n_phizln_bus1_C = "n:phizln:bus1:C", "deg, Angle of Network Impedance, Line-Neutral"
      n_phizln_bus1_N = "n:phizln:bus1:N", "deg, Angle of Network Impedance, Line-Neutral"
      n_phizln_bus2_A = "n:phizln:bus2:A", "deg, Angle of Network Impedance, Line-Neutral"
      n_phizln_bus2_B = "n:phizln:bus2:B", "deg, Angle of Network Impedance, Line-Neutral"
      n_phizln_bus2_C = "n:phizln:bus2:C", "deg, Angle of Network Impedance, Line-Neutral"
      n_phizln_bus2_N = "n:phizln:bus2:N", "deg, Angle of Network Impedance, Line-Neutral"
      n_Rn_bus1 = "n:Rn:bus1", "Ohm, Network Resistance, Neutral"
      n_Rn_bus2 = "n:Rn:bus2", "Ohm, Network Resistance, Neutral"
      n_Xn_bus1 = "n:Xn:bus1", "Ohm, Network Reactance, Neutral"
      n_Xn_bus2 = "n:Xn:bus2", "Ohm, Network Reactance, Neutral"
      n_Zn_bus1 = "n:Zn:bus1", "Ohm, Network Impedance, Neutral"
      n_Zn_bus2 = "n:Zn:bus2", "Ohm, Network Impedance, Neutral"
      n_phizn_bus1 = "n:phizn:bus1", "deg, Angle of Network Impedance, Neutral"
      n_phizn_bus2 = "n:phizn:bus2", "deg, Angle of Network Impedance, Neutral"
      n_eff_bus1_A = "n:eff:bus1:A", "Earth Fault Factor"
      n_eff_bus1_B = "n:eff:bus1:B", "Earth Fault Factor"
      n_eff_bus1_C = "n:eff:bus1:C", "Earth Fault Factor"
      n_eff_bus1_N = "n:eff:bus1:N", "Earth Fault Factor"
      n_eff_bus2_A = "n:eff:bus2:A", "Earth Fault Factor"
      n_eff_bus2_B = "n:eff:bus2:B", "Earth Fault Factor"
      n_eff_bus2_C = "n:eff:bus2:C", "Earth Fault Factor"
      n_eff_bus2_N = "n:eff:bus2:N", "Earth Fault Factor"
      n_X0toX1_bus1 = "n:X0toX1:bus1", "X0/X1"
      n_X0toX1_bus2 = "n:X0toX1:bus2", "X0/X1"
      n_in_bus1 = "n:in:bus1", "p.u., Neutral Current, Magnitude"
      n_in_bus2 = "n:in:bus2", "p.u., Neutral Current, Magnitude"
      n_In_bus1 = "n:In:bus1", "kA, Neutral Current, Magnitude"
      n_In_bus2 = "n:In:bus2", "kA, Neutral Current, Magnitude"
      n_phiin_bus1 = "n:phiin:bus1", "deg, Neutral Current, Angle"
      n_phiin_bus2 = "n:phiin:bus2", "deg, Neutral Current, Angle"
      n_i0r_bus1 = "n:i0r:bus1", "p.u., Zero-Sequence Current, Real Part"
      n_i0r_bus2 = "n:i0r:bus2", "p.u., Zero-Sequence Current, Real Part"
      n_i0i_bus1 = "n:i0i:bus1", "p.u., Zero-Sequence Current, Imaginary Part"
      n_i0i_bus2 = "n:i0i:bus2", "p.u., Zero-Sequence Current, Imaginary Part"
      n_i0_bus1 = "n:i0:bus1", "p.u., Zero-Sequence Current, Magnitude"
      n_i0_bus2 = "n:i0:bus2", "p.u., Zero-Sequence Current, Magnitude"
      n_I0_bus1 = "n:I0:bus1", "kA, Zero-Sequence Current, Magnitude"
      n_I0_bus2 = "n:I0:bus2", "kA, Zero-Sequence Current, Magnitude"
      n_I0x3_bus1 = "n:I0x3:bus1", "kA, 3*I0"
      n_I0x3_bus2 = "n:I0x3:bus2", "kA, 3*I0"
      n_phii0_bus1 = "n:phii0:bus1", "deg, Zero-Sequence Current, Angle"
      n_phii0_bus2 = "n:phii0:bus2", "deg, Zero-Sequence Current, Angle"
      n_i1r_bus1 = "n:i1r:bus1", "p.u., Positive-Sequence Current, Real Part"
      n_i1r_bus2 = "n:i1r:bus2", "p.u., Positive-Sequence Current, Real Part"
      n_i1i_bus1 = "n:i1i:bus1", "p.u., Positive-Sequence Current, Imaginary Part"
      n_i1i_bus2 = "n:i1i:bus2", "p.u., Positive-Sequence Current, Imaginary Part"
      n_i1_bus1 = "n:i1:bus1", "p.u., Positive-Sequence Current, Magnitude"
      n_i1_bus2 = "n:i1:bus2", "p.u., Positive-Sequence Current, Magnitude"
      n_I1_bus1 = "n:I1:bus1", "kA, Positive-Sequence Current, Magnitude"
      n_I1_bus2 = "n:I1:bus2", "kA, Positive-Sequence Current, Magnitude"
      n_phii1_bus1 = "n:phii1:bus1", "deg, Positive-Sequence Current, Angle"
      n_phii1_bus2 = "n:phii1:bus2", "deg, Positive-Sequence Current, Angle"
      n_i2r_bus1 = "n:i2r:bus1", "p.u., Negative-Sequence Current, Real Part"
      n_i2r_bus2 = "n:i2r:bus2", "p.u., Negative-Sequence Current, Real Part"
      n_i2i_bus1 = "n:i2i:bus1", "p.u., Negative-Sequence Current, Imaginary Part"
      n_i2i_bus2 = "n:i2i:bus2", "p.u., Negative-Sequence Current, Imaginary Part"
      n_i2_bus1 = "n:i2:bus1", "p.u., Negative-Sequence Current, Magnitude"
      n_i2_bus2 = "n:i2:bus2", "p.u., Negative-Sequence Current, Magnitude"
      n_I2_bus1 = "n:I2:bus1", "kA, Negative-Sequence Current, Magnitude"
      n_I2_bus2 = "n:I2:bus2", "kA, Negative-Sequence Current, Magnitude"
      n_phii2_bus1 = "n:phii2:bus1", "deg, Negative-Sequence Current, Angle"
      n_phii2_bus2 = "n:phii2:bus2", "deg, Negative-Sequence Current, Angle"
      n_du_bus1_A = "n:du:bus1:A", "%, Voltage Deviation"
      n_du_bus1_B = "n:du:bus1:B", "%, Voltage Deviation"
      n_du_bus1_C = "n:du:bus1:C", "%, Voltage Deviation"
      n_du_bus1_N = "n:du:bus1:N", "%, Voltage Deviation"
      n_du_bus2_A = "n:du:bus2:A", "%, Voltage Deviation"
      n_du_bus2_B = "n:du:bus2:B", "%, Voltage Deviation"
      n_du_bus2_C = "n:du:bus2:C", "%, Voltage Deviation"
      n_du_bus2_N = "n:du:bus2:N", "%, Voltage Deviation"
      n_urpref_bus1_A = "n:urpref:bus1:A", "p.u., Pre-fault Voltage, Real Part"
      n_urpref_bus1_B = "n:urpref:bus1:B", "p.u., Pre-fault Voltage, Real Part"
      n_urpref_bus1_C = "n:urpref:bus1:C", "p.u., Pre-fault Voltage, Real Part"
      n_urpref_bus1_N = "n:urpref:bus1:N", "p.u., Pre-fault Voltage, Real Part"
      n_urpref_bus2_A = "n:urpref:bus2:A", "p.u., Pre-fault Voltage, Real Part"
      n_urpref_bus2_B = "n:urpref:bus2:B", "p.u., Pre-fault Voltage, Real Part"
      n_urpref_bus2_C = "n:urpref:bus2:C", "p.u., Pre-fault Voltage, Real Part"
      n_urpref_bus2_N = "n:urpref:bus2:N", "p.u., Pre-fault Voltage, Real Part"
      n_uipref_bus1_A = "n:uipref:bus1:A", "p.u., Pre-fault Voltage, Imaginary Part"
      n_uipref_bus1_B = "n:uipref:bus1:B", "p.u., Pre-fault Voltage, Imaginary Part"
      n_uipref_bus1_C = "n:uipref:bus1:C", "p.u., Pre-fault Voltage, Imaginary Part"
      n_uipref_bus1_N = "n:uipref:bus1:N", "p.u., Pre-fault Voltage, Imaginary Part"
      n_uipref_bus2_A = "n:uipref:bus2:A", "p.u., Pre-fault Voltage, Imaginary Part"
      n_uipref_bus2_B = "n:uipref:bus2:B", "p.u., Pre-fault Voltage, Imaginary Part"
      n_uipref_bus2_C = "n:uipref:bus2:C", "p.u., Pre-fault Voltage, Imaginary Part"
      n_uipref_bus2_N = "n:uipref:bus2:N", "p.u., Pre-fault Voltage, Imaginary Part"
      n_upref_bus1_A = "n:upref:bus1:A", "p.u., Pre-fault Voltage, Magnitude"
      n_upref_bus1_B = "n:upref:bus1:B", "p.u., Pre-fault Voltage, Magnitude"
      n_upref_bus1_C = "n:upref:bus1:C", "p.u., Pre-fault Voltage, Magnitude"
      n_upref_bus1_N = "n:upref:bus1:N", "p.u., Pre-fault Voltage, Magnitude"
      n_upref_bus2_A = "n:upref:bus2:A", "p.u., Pre-fault Voltage, Magnitude"
      n_upref_bus2_B = "n:upref:bus2:B", "p.u., Pre-fault Voltage, Magnitude"
      n_upref_bus2_C = "n:upref:bus2:C", "p.u., Pre-fault Voltage, Magnitude"
      n_upref_bus2_N = "n:upref:bus2:N", "p.u., Pre-fault Voltage, Magnitude"
      n_uprefpc_bus1_A = "n:uprefpc:bus1:A", "%, Pre-fault Voltage, Magnitude"
      n_uprefpc_bus1_B = "n:uprefpc:bus1:B", "%, Pre-fault Voltage, Magnitude"
      n_uprefpc_bus1_C = "n:uprefpc:bus1:C", "%, Pre-fault Voltage, Magnitude"
      n_uprefpc_bus1_N = "n:uprefpc:bus1:N", "%, Pre-fault Voltage, Magnitude"
      n_uprefpc_bus2_A = "n:uprefpc:bus2:A", "%, Pre-fault Voltage, Magnitude"
      n_uprefpc_bus2_B = "n:uprefpc:bus2:B", "%, Pre-fault Voltage, Magnitude"
      n_uprefpc_bus2_C = "n:uprefpc:bus2:C", "%, Pre-fault Voltage, Magnitude"
      n_uprefpc_bus2_N = "n:uprefpc:bus2:N", "%, Pre-fault Voltage, Magnitude"
      n_Upref_bus1_A = "n:Upref:bus1:A", "kV, Line-Ground Pre-fault Voltage, Magnitude"
      n_Upref_bus1_B = "n:Upref:bus1:B", "kV, Line-Ground Pre-fault Voltage, Magnitude"
      n_Upref_bus1_C = "n:Upref:bus1:C", "kV, Line-Ground Pre-fault Voltage, Magnitude"
      n_Upref_bus1_N = "n:Upref:bus1:N", "kV, Line-Ground Pre-fault Voltage, Magnitude"
      n_Upref_bus2_A = "n:Upref:bus2:A", "kV, Line-Ground Pre-fault Voltage, Magnitude"
      n_Upref_bus2_B = "n:Upref:bus2:B", "kV, Line-Ground Pre-fault Voltage, Magnitude"
      n_Upref_bus2_C = "n:Upref:bus2:C", "kV, Line-Ground Pre-fault Voltage, Magnitude"
      n_Upref_bus2_N = "n:Upref:bus2:N", "kV, Line-Ground Pre-fault Voltage, Magnitude"
      n_Ulpref_bus1_A = "n:Ulpref:bus1:A", "kV, Line-Line Pre-fault Voltage, Magnitude"
      n_Ulpref_bus1_B = "n:Ulpref:bus1:B", "kV, Line-Line Pre-fault Voltage, Magnitude"
      n_Ulpref_bus1_C = "n:Ulpref:bus1:C", "kV, Line-Line Pre-fault Voltage, Magnitude"
      n_Ulpref_bus1_N = "n:Ulpref:bus1:N", "kV, Line-Line Pre-fault Voltage, Magnitude"
      n_Ulpref_bus2_A = "n:Ulpref:bus2:A", "kV, Line-Line Pre-fault Voltage, Magnitude"
      n_Ulpref_bus2_B = "n:Ulpref:bus2:B", "kV, Line-Line Pre-fault Voltage, Magnitude"
      n_Ulpref_bus2_C = "n:Ulpref:bus2:C", "kV, Line-Line Pre-fault Voltage, Magnitude"
      n_Ulpref_bus2_N = "n:Ulpref:bus2:N", "kV, Line-Line Pre-fault Voltage, Magnitude"
      n_phiupref_bus1_A = "n:phiupref:bus1:A", "deg, Pre-fault Voltage, Angle"
      n_phiupref_bus1_B = "n:phiupref:bus1:B", "deg, Pre-fault Voltage, Angle"
      n_phiupref_bus1_C = "n:phiupref:bus1:C", "deg, Pre-fault Voltage, Angle"
      n_phiupref_bus1_N = "n:phiupref:bus1:N", "deg, Pre-fault Voltage, Angle"
      n_phiupref_bus2_A = "n:phiupref:bus2:A", "deg, Pre-fault Voltage, Angle"
      n_phiupref_bus2_B = "n:phiupref:bus2:B", "deg, Pre-fault Voltage, Angle"
      n_phiupref_bus2_C = "n:phiupref:bus2:C", "deg, Pre-fault Voltage, Angle"
      n_phiupref_bus2_N = "n:phiupref:bus2:N", "deg, Pre-fault Voltage, Angle"
      n_phiui_bus1_A = "n:phiui:bus1:A", "deg, Angle between Voltage and Current"
      n_phiui_bus1_B = "n:phiui:bus1:B", "deg, Angle between Voltage and Current"
      n_phiui_bus1_C = "n:phiui:bus1:C", "deg, Angle between Voltage and Current"
      n_phiui_bus1_N = "n:phiui:bus1:N", "deg, Angle between Voltage and Current"
      n_phiui_bus2_A = "n:phiui:bus2:A", "deg, Angle between Voltage and Current"
      n_phiui_bus2_B = "n:phiui:bus2:B", "deg, Angle between Voltage and Current"
      n_phiui_bus2_C = "n:phiui:bus2:C", "deg, Angle between Voltage and Current"
      n_phiui_bus2_N = "n:phiui:bus2:N", "deg, Angle between Voltage and Current"
      n_phiu1i1_bus1 = "n:phiu1i1:bus1", "deg, Angle between Voltage and Current in positive sequence system"
      n_phiu1i1_bus2 = "n:phiu1i1:bus2", "deg, Angle between Voltage and Current in positive sequence system"
      n_phiu2i2_bus1 = "n:phiu2i2:bus1", "deg, Angle between Voltage and Current in negative sequence system"
      n_phiu2i2_bus2 = "n:phiu2i2:bus2", "deg, Angle between Voltage and Current in negative sequence system"
      n_phiu0i0_bus1 = "n:phiu0i0:bus1", "deg, Angle between Voltage and Current in zero-sequence system"
      n_phiu0i0_bus2 = "n:phiu0i0:bus2", "deg, Angle between Voltage and Current in zero-sequence system"

    class ElmGenstat(Enum):
      _init_ = 'value __doc__'

      m_u0_bus1 = "m:u0:bus1", "p.u., Zero-Sequence-Voltage, Magnitude"
      m_u0_bus1n = "m:u0:bus1n", "p.u., Zero-Sequence-Voltage, Magnitude"
      m_phiu0_bus1 = "m:phiu0:bus1", "deg, Zero-Sequence-Voltage, Angle"
      m_phiu0_bus1n = "m:phiu0:bus1n", "deg, Zero-Sequence-Voltage, Angle"
      m_u1r_bus1 = "m:u1r:bus1", "p.u., Positive-Sequence-Voltage, Real Part"
      m_u1r_bus1n = "m:u1r:bus1n", "p.u., Positive-Sequence-Voltage, Real Part"
      m_u1i_bus1 = "m:u1i:bus1", "p.u., Positive-Sequence-Voltage, Imaginary Part"
      m_u1i_bus1n = "m:u1i:bus1n", "p.u., Positive-Sequence-Voltage, Imaginary Part"
      m_u1_bus1 = "m:u1:bus1", "p.u., Positive-Sequence-Voltage, Magnitude"
      m_u1_bus1n = "m:u1:bus1n", "p.u., Positive-Sequence-Voltage, Magnitude"
      m_U1_bus1 = "m:U1:bus1", "kV, Line-Ground Positive-Sequence-Voltage, Magnitude"
      m_U1_bus1n = "m:U1:bus1n", "kV, Line-Ground Positive-Sequence-Voltage, Magnitude"
      m_U1l_bus1 = "m:U1l:bus1", "kV, Line-Line Positive-Sequence-Voltage, Magnitude"
      m_U1l_bus1n = "m:U1l:bus1n", "kV, Line-Line Positive-Sequence-Voltage, Magnitude"
      m_phiu1_bus1 = "m:phiu1:bus1", "deg, Positive-Sequence-Voltage, Angle"
      m_phiu1_bus1n = "m:phiu1:bus1n", "deg, Positive-Sequence-Voltage, Angle"
      m_u2_bus1 = "m:u2:bus1", "p.u., Negative-Sequence-Voltage, Magnitude"
      m_u2_bus1n = "m:u2:bus1n", "p.u., Negative-Sequence-Voltage, Magnitude"
      m_phiu2_bus1 = "m:phiu2:bus1", "deg, Negative-Sequence-Voltage, Angle"
      m_phiu2_bus1n = "m:phiu2:bus1n", "deg, Negative-Sequence-Voltage, Angle"
      m_inet_bus1_A = "m:inet:bus1:A", "p.u., Phase Current, Magnitude, referred to network"
      m_inet_bus1_B = "m:inet:bus1:B", "p.u., Phase Current, Magnitude, referred to network"
      m_inet_bus1_C = "m:inet:bus1:C", "p.u., Phase Current, Magnitude, referred to network"
      m_inet_bus1_N = "m:inet:bus1:N", "p.u., Phase Current, Magnitude, referred to network"
      m_inet_bus1n = "m:inet:bus1n", "p.u., Phase Current, Magnitude, referred to network"
      m_I_bus1_A = "m:I:bus1:A", "kA, Phase Current, Magnitude"
      m_I_bus1_B = "m:I:bus1:B", "kA, Phase Current, Magnitude"
      m_I_bus1_C = "m:I:bus1:C", "kA, Phase Current, Magnitude"
      m_I_bus1_N = "m:I:bus1:N", "kA, Phase Current, Magnitude"
      m_I_bus1n = "m:I:bus1n", "kA, Phase Current, Magnitude"
      m_phii_bus1_A = "m:phii:bus1:A", "deg, Phase Current, Angle"
      m_phii_bus1_B = "m:phii:bus1:B", "deg, Phase Current, Angle"
      m_phii_bus1_C = "m:phii:bus1:C", "deg, Phase Current, Angle"
      m_phii_bus1_N = "m:phii:bus1:N", "deg, Phase Current, Angle"
      m_phii_bus1n = "m:phii:bus1n", "deg, Phase Current, Angle"
      m_i0r_bus1 = "m:i0r:bus1", "p.u., Zero-Sequence Current, Real Part"
      m_i0r_bus1n = "m:i0r:bus1n", "p.u., Zero-Sequence Current, Real Part"
      m_i0i_bus1 = "m:i0i:bus1", "p.u., Zero-Sequence Current, Imaginary Part"
      m_i0i_bus1n = "m:i0i:bus1n", "p.u., Zero-Sequence Current, Imaginary Part"
      m_i0_bus1 = "m:i0:bus1", "p.u., Zero-Sequence Current, Magnitude"
      m_i0_bus1n = "m:i0:bus1n", "p.u., Zero-Sequence Current, Magnitude"
      m_I0_bus1 = "m:I0:bus1", "kA, Zero-Sequence Current, Magnitude"
      m_I0_bus1n = "m:I0:bus1n", "kA, Zero-Sequence Current, Magnitude"
      m_I0x3_bus1 = "m:I0x3:bus1", "kA, 3*I0"
      m_I0x3_bus1n = "m:I0x3:bus1n", "kA, 3*I0"
      m_phii0_bus1 = "m:phii0:bus1", "deg, Zero-Sequence Current, Angle"
      m_phii0_bus1n = "m:phii0:bus1n", "deg, Zero-Sequence Current, Angle"
      m_i1r_bus1 = "m:i1r:bus1", "p.u., Positive-Sequence Current, Real Part"
      m_i1r_bus1n = "m:i1r:bus1n", "p.u., Positive-Sequence Current, Real Part"
      m_i1i_bus1 = "m:i1i:bus1", "p.u., Positive-Sequence Current, Imaginary Part"
      m_i1i_bus1n = "m:i1i:bus1n", "p.u., Positive-Sequence Current, Imaginary Part"
      m_i1_bus1 = "m:i1:bus1", "p.u., Positive-Sequence Current, Magnitude"
      m_i1_bus1n = "m:i1:bus1n", "p.u., Positive-Sequence Current, Magnitude"
      m_I1_bus1 = "m:I1:bus1", "kA, Positive-Sequence Current, Magnitude"
      m_I1_bus1n = "m:I1:bus1n", "kA, Positive-Sequence Current, Magnitude"
      m_phii1_bus1 = "m:phii1:bus1", "deg, Positive-Sequence Current, Angle"
      m_phii1_bus1n = "m:phii1:bus1n", "deg, Positive-Sequence Current, Angle"
      m_i2r_bus1 = "m:i2r:bus1", "p.u., Negative-Sequence Current, Real Part"
      m_i2r_bus1n = "m:i2r:bus1n", "p.u., Negative-Sequence Current, Real Part"
      m_i2i_bus1 = "m:i2i:bus1", "p.u., Negative-Sequence Current, Imaginary Part"
      m_i2i_bus1n = "m:i2i:bus1n", "p.u., Negative-Sequence Current, Imaginary Part"
      m_i2_bus1 = "m:i2:bus1", "p.u., Negative-Sequence Current, Magnitude"
      m_i2_bus1n = "m:i2:bus1n", "p.u., Negative-Sequence Current, Magnitude"
      m_I2_bus1 = "m:I2:bus1", "kA, Negative-Sequence Current, Magnitude"
      m_I2_bus1n = "m:I2:bus1n", "kA, Negative-Sequence Current, Magnitude"
      m_phii2_bus1 = "m:phii2:bus1", "deg, Negative-Sequence Current, Angle"
      m_phii2_bus1n = "m:phii2:bus1n", "deg, Negative-Sequence Current, Angle"
      m_ubfacI_bus1 = "m:ubfacI:bus1", "%, Current Unbalance Factor"
      m_ubfacI_bus1n = "m:ubfacI:bus1n", "%, Current Unbalance Factor"
      m_ubfacS_bus1 = "m:ubfacS:bus1", "%, Power Unbalance Factor"
      m_ubfacS_bus1n = "m:ubfacS:bus1n", "%, Power Unbalance Factor"
      m_P_bus1_A = "m:P:bus1:A", "MW, Active Power"
      m_P_bus1_B = "m:P:bus1:B", "MW, Active Power"
      m_P_bus1_C = "m:P:bus1:C", "MW, Active Power"
      m_P_bus1_N = "m:P:bus1:N", "MW, Active Power"
      m_P_bus1n = "m:P:bus1n", "MW, Active Power"
      m_Q_bus1_A = "m:Q:bus1:A", "Mvar, Reactive Power"
      m_Q_bus1_B = "m:Q:bus1:B", "Mvar, Reactive Power"
      m_Q_bus1_C = "m:Q:bus1:C", "Mvar, Reactive Power"
      m_Q_bus1_N = "m:Q:bus1:N", "Mvar, Reactive Power"
      m_Q_bus1n = "m:Q:bus1n", "Mvar, Reactive Power"
      m_S_bus1_A = "m:S:bus1:A", "MVA, Apparent Power"
      m_S_bus1_B = "m:S:bus1:B", "MVA, Apparent Power"
      m_S_bus1_C = "m:S:bus1:C", "MVA, Apparent Power"
      m_S_bus1_N = "m:S:bus1:N", "MVA, Apparent Power"
      m_S_bus1n = "m:S:bus1n", "MVA, Apparent Power"
      m_cosphi_bus1_A = "m:cosphi:bus1:A", "Power Factor"
      m_cosphi_bus1_B = "m:cosphi:bus1:B", "Power Factor"
      m_cosphi_bus1_C = "m:cosphi:bus1:C", "Power Factor"
      m_cosphi_bus1_N = "m:cosphi:bus1:N", "Power Factor"
      m_cosphi_bus1n = "m:cosphi:bus1n", "Power Factor"
      m_tanphi_bus1_A = "m:tanphi:bus1:A", "tan(phi)"
      m_tanphi_bus1_B = "m:tanphi:bus1:B", "tan(phi)"
      m_tanphi_bus1_C = "m:tanphi:bus1:C", "tan(phi)"
      m_tanphi_bus1_N = "m:tanphi:bus1:N", "tan(phi)"
      m_tanphi_bus1n = "m:tanphi:bus1n", "tan(phi)"
      m_Psum_bus1 = "m:Psum:bus1", "MW, Total Active Power"
      m_Psum_bus1n = "m:Psum:bus1n", "MW, Total Active Power"
      m_Qsum_bus1 = "m:Qsum:bus1", "Mvar, Total Reactive Power"
      m_Qsum_bus1n = "m:Qsum:bus1n", "Mvar, Total Reactive Power"
      m_Ssum_bus1 = "m:Ssum:bus1", "MVA, Total Apparent Power"
      m_Ssum_bus1n = "m:Ssum:bus1n", "MVA, Total Apparent Power"
      m_cosphisum_bus1 = "m:cosphisum:bus1", "Total Power Factor"
      m_cosphisum_bus1n = "m:cosphisum:bus1n", "Total Power Factor"
      m_tanphisum_bus1 = "m:tanphisum:bus1", "Total tan(phi)"
      m_tanphisum_bus1n = "m:tanphisum:bus1n", "Total tan(phi)"
      m_Pln_bus1_A = "m:Pln:bus1:A", "MW, Active Power (Line-Neutral)"
      m_Pln_bus1_B = "m:Pln:bus1:B", "MW, Active Power (Line-Neutral)"
      m_Pln_bus1_C = "m:Pln:bus1:C", "MW, Active Power (Line-Neutral)"
      m_Pln_bus1_N = "m:Pln:bus1:N", "MW, Active Power (Line-Neutral)"
      m_Pln_bus1n = "m:Pln:bus1n", "MW, Active Power (Line-Neutral)"
      m_Qln_bus1_A = "m:Qln:bus1:A", "Mvar, Reactive Power (Line-Neutral)"
      m_Qln_bus1_B = "m:Qln:bus1:B", "Mvar, Reactive Power (Line-Neutral)"
      m_Qln_bus1_C = "m:Qln:bus1:C", "Mvar, Reactive Power (Line-Neutral)"
      m_Qln_bus1_N = "m:Qln:bus1:N", "Mvar, Reactive Power (Line-Neutral)"
      m_Qln_bus1n = "m:Qln:bus1n", "Mvar, Reactive Power (Line-Neutral)"
      m_P1_bus1 = "m:P1:bus1", "MW, Positive Sequence Active Power"
      m_P1_bus1n = "m:P1:bus1n", "MW, Positive Sequence Active Power"
      m_Q1_bus1 = "m:Q1:bus1", "Mvar, Positive Sequence Reactive Power"
      m_Q1_bus1n = "m:Q1:bus1n", "Mvar, Positive Sequence Reactive Power"
      m_P2_bus1 = "m:P2:bus1", "MW, Negative Sequence Active Power"
      m_P2_bus1n = "m:P2:bus1n", "MW, Negative Sequence Active Power"
      m_Q2_bus1 = "m:Q2:bus1", "Mvar, Negative Sequence Reactive Power"
      m_Q2_bus1n = "m:Q2:bus1n", "Mvar, Negative Sequence Reactive Power"
      m_P0_bus1 = "m:P0:bus1", "MW, Zero Sequence Active Power"
      m_P0_bus1n = "m:P0:bus1n", "MW, Zero Sequence Active Power"
      m_Q0_bus1 = "m:Q0:bus1", "Mvar, Zero Sequence Reactive Power"
      m_Q0_bus1n = "m:Q0:bus1n", "Mvar, Zero Sequence Reactive Power"
      m_i1P_bus1 = "m:i1P:bus1", "p.u., Positive-Sequence Active Current"
      m_i1P_bus1n = "m:i1P:bus1n", "p.u., Positive-Sequence Active Current"
      m_i1Q_bus1 = "m:i1Q:bus1", "p.u., Positive-Sequence Reactive Current"
      m_i1Q_bus1n = "m:i1Q:bus1n", "p.u., Positive-Sequence Reactive Current"
      m_I1P_bus1 = "m:I1P:bus1", "kA, Positive-Sequence Active Current"
      m_I1P_bus1n = "m:I1P:bus1n", "kA, Positive-Sequence Active Current"
      m_I1Q_bus1 = "m:I1Q:bus1", "kA, Positive-Sequence Reactive Current"
      m_I1Q_bus1n = "m:I1Q:bus1n", "kA, Positive-Sequence Reactive Current"
      m_i2P_bus1 = "m:i2P:bus1", "p.u., Negative-Sequence Active Current"
      m_i2P_bus1n = "m:i2P:bus1n", "p.u., Negative-Sequence Active Current"
      m_i2Q_bus1 = "m:i2Q:bus1", "p.u., Negative-Sequence Reactive Current"
      m_i2Q_bus1n = "m:i2Q:bus1n", "p.u., Negative-Sequence Reactive Current"
      m_I2P_bus1 = "m:I2P:bus1", "kA, Negative-Sequence Active Current"
      m_I2P_bus1n = "m:I2P:bus1n", "kA, Negative-Sequence Active Current"
      m_I2Q_bus1 = "m:I2Q:bus1", "kA, Negative-Sequence Reactive Current"
      m_I2Q_bus1n = "m:I2Q:bus1n", "kA, Negative-Sequence Reactive Current"
      m_ur_bus1_A = "m:ur:bus1:A", "p.u., Phase Voltage, Real Part"
      m_ur_bus1_B = "m:ur:bus1:B", "p.u., Phase Voltage, Real Part"
      m_ur_bus1_C = "m:ur:bus1:C", "p.u., Phase Voltage, Real Part"
      m_ur_bus1_N = "m:ur:bus1:N", "p.u., Phase Voltage, Real Part"
      m_ur_bus1n = "m:ur:bus1n", "p.u., Phase Voltage, Real Part"
      m_ui_bus1_A = "m:ui:bus1:A", "p.u., Phase Voltage, Imaginary Part"
      m_ui_bus1_B = "m:ui:bus1:B", "p.u., Phase Voltage, Imaginary Part"
      m_ui_bus1_C = "m:ui:bus1:C", "p.u., Phase Voltage, Imaginary Part"
      m_ui_bus1_N = "m:ui:bus1:N", "p.u., Phase Voltage, Imaginary Part"
      m_ui_bus1n = "m:ui:bus1n", "p.u., Phase Voltage, Imaginary Part"
      m_u_bus1_A = "m:u:bus1:A", "p.u., Phase Voltage, Magnitude"
      m_u_bus1_B = "m:u:bus1:B", "p.u., Phase Voltage, Magnitude"
      m_u_bus1_C = "m:u:bus1:C", "p.u., Phase Voltage, Magnitude"
      m_u_bus1_N = "m:u:bus1:N", "p.u., Phase Voltage, Magnitude"
      m_u_bus1n = "m:u:bus1n", "p.u., Phase Voltage, Magnitude"
      m_ir_bus1_A = "m:ir:bus1:A", "p.u., Phase Current, Real Part"
      m_ir_bus1_B = "m:ir:bus1:B", "p.u., Phase Current, Real Part"
      m_ir_bus1_C = "m:ir:bus1:C", "p.u., Phase Current, Real Part"
      m_ir_bus1_N = "m:ir:bus1:N", "p.u., Phase Current, Real Part"
      m_ir_bus1n = "m:ir:bus1n", "p.u., Phase Current, Real Part"
      m_ii_bus1_A = "m:ii:bus1:A", "p.u., Phase Current, Imaginary Part"
      m_ii_bus1_B = "m:ii:bus1:B", "p.u., Phase Current, Imaginary Part"
      m_ii_bus1_C = "m:ii:bus1:C", "p.u., Phase Current, Imaginary Part"
      m_ii_bus1_N = "m:ii:bus1:N", "p.u., Phase Current, Imaginary Part"
      m_ii_bus1n = "m:ii:bus1n", "p.u., Phase Current, Imaginary Part"
      m_i_bus1_A = "m:i:bus1:A", "p.u., Phase Current, Magnitude"
      m_i_bus1_B = "m:i:bus1:B", "p.u., Phase Current, Magnitude"
      m_i_bus1_C = "m:i:bus1:C", "p.u., Phase Current, Magnitude"
      m_i_bus1_N = "m:i:bus1:N", "p.u., Phase Current, Magnitude"
      m_i_bus1n = "m:i:bus1n", "p.u., Phase Current, Magnitude"
      m_phiui_bus1_A = "m:phiui:bus1:A", "deg, Angle between Voltage and Current"
      m_phiui_bus1_B = "m:phiui:bus1:B", "deg, Angle between Voltage and Current"
      m_phiui_bus1_C = "m:phiui:bus1:C", "deg, Angle between Voltage and Current"
      m_phiui_bus1_N = "m:phiui:bus1:N", "deg, Angle between Voltage and Current"
      m_phiui_bus1n = "m:phiui:bus1n", "deg, Angle between Voltage and Current"
      m_phiu0i0_bus1 = "m:phiu0i0:bus1", "deg, Angle between Voltage and Current in zero-sequence system"
      m_phiu0i0_bus1n = "m:phiu0i0:bus1n", "deg, Angle between Voltage and Current in zero-sequence system"
      m_phiu1i1_bus1 = "m:phiu1i1:bus1", "deg, Angle between Voltage and Current in positive sequence system"
      m_phiu1i1_bus1n = "m:phiu1i1:bus1n", "deg, Angle between Voltage and Current in positive sequence system"
      m_phiu2i2_bus1 = "m:phiu2i2:bus1", "deg, Angle between Voltage and Current in negative sequence system"
      m_phiu2i2_bus1n = "m:phiu2i2:bus1n", "deg, Angle between Voltage and Current in negative sequence system"
      m_Tfct_bus1 = "m:Tfct:bus1", "s, Fault Clearing Time"
      m_Tfct_bus1n = "m:Tfct:bus1n", "s, Fault Clearing Time"
      m_TfctPh_bus1_A = "m:TfctPh:bus1:A", "s, Fault Clearing Time"
      m_TfctPh_bus1_B = "m:TfctPh:bus1:B", "s, Fault Clearing Time"
      m_TfctPh_bus1_C = "m:TfctPh:bus1:C", "s, Fault Clearing Time"
      m_TfctPh_bus1_N = "m:TfctPh:bus1:N", "s, Fault Clearing Time"
      m_TfctPh_bus1n = "m:TfctPh:bus1n", "s, Fault Clearing Time"
      m_Brkload_bus1 = "m:Brkload:bus1", "%, Max. Breaker Loading"
      m_Brkload_bus1n = "m:Brkload:bus1n", "%, Max. Breaker Loading"
      m_BrkloadPh_bus1_A = "m:BrkloadPh:bus1:A", "%, Breaker Loading"
      m_BrkloadPh_bus1_B = "m:BrkloadPh:bus1:B", "%, Breaker Loading"
      m_BrkloadPh_bus1_C = "m:BrkloadPh:bus1:C", "%, Breaker Loading"
      m_BrkloadPh_bus1_N = "m:BrkloadPh:bus1:N", "%, Breaker Loading"
      m_BrkloadPh_bus1n = "m:BrkloadPh:bus1n", "%, Breaker Loading"
      n_ur_bus1_A = "n:ur:bus1:A", "p.u., Line-Ground Voltage, Real Part"
      n_ur_bus1_B = "n:ur:bus1:B", "p.u., Line-Ground Voltage, Real Part"
      n_ur_bus1_C = "n:ur:bus1:C", "p.u., Line-Ground Voltage, Real Part"
      n_ur_bus1_N = "n:ur:bus1:N", "p.u., Line-Ground Voltage, Real Part"
      n_ur_bus1n = "n:ur:bus1n", "p.u., Line-Ground Voltage, Real Part"
      n_ui_bus1_A = "n:ui:bus1:A", "p.u., Line-Ground Voltage, Imaginary Part"
      n_ui_bus1_B = "n:ui:bus1:B", "p.u., Line-Ground Voltage, Imaginary Part"
      n_ui_bus1_C = "n:ui:bus1:C", "p.u., Line-Ground Voltage, Imaginary Part"
      n_ui_bus1_N = "n:ui:bus1:N", "p.u., Line-Ground Voltage, Imaginary Part"
      n_ui_bus1n = "n:ui:bus1n", "p.u., Line-Ground Voltage, Imaginary Part"
      n_u_bus1_A = "n:u:bus1:A", "p.u., Line-Ground Voltage, Magnitude"
      n_u_bus1_B = "n:u:bus1:B", "p.u., Line-Ground Voltage, Magnitude"
      n_u_bus1_C = "n:u:bus1:C", "p.u., Line-Ground Voltage, Magnitude"
      n_u_bus1_N = "n:u:bus1:N", "p.u., Line-Ground Voltage, Magnitude"
      n_u_bus1n = "n:u:bus1n", "p.u., Line-Ground Voltage, Magnitude"
      n_upc_bus1_A = "n:upc:bus1:A", "%, Line-Ground Voltage, Magnitude"
      n_upc_bus1_B = "n:upc:bus1:B", "%, Line-Ground Voltage, Magnitude"
      n_upc_bus1_C = "n:upc:bus1:C", "%, Line-Ground Voltage, Magnitude"
      n_upc_bus1_N = "n:upc:bus1:N", "%, Line-Ground Voltage, Magnitude"
      n_upc_bus1n = "n:upc:bus1n", "%, Line-Ground Voltage, Magnitude"
      n_U_bus1_A = "n:U:bus1:A", "kV, Line-Ground Voltage, Magnitude"
      n_U_bus1_B = "n:U:bus1:B", "kV, Line-Ground Voltage, Magnitude"
      n_U_bus1_C = "n:U:bus1:C", "kV, Line-Ground Voltage, Magnitude"
      n_U_bus1_N = "n:U:bus1:N", "kV, Line-Ground Voltage, Magnitude"
      n_U_bus1n = "n:U:bus1n", "kV, Line-Ground Voltage, Magnitude"
      n_phiu_bus1_A = "n:phiu:bus1:A", "deg, Line-Ground Voltage, Angle"
      n_phiu_bus1_B = "n:phiu:bus1:B", "deg, Line-Ground Voltage, Angle"
      n_phiu_bus1_C = "n:phiu:bus1:C", "deg, Line-Ground Voltage, Angle"
      n_phiu_bus1_N = "n:phiu:bus1:N", "deg, Line-Ground Voltage, Angle"
      n_phiu_bus1n = "n:phiu:bus1n", "deg, Line-Ground Voltage, Angle"
      n_phiurel_bus1 = "n:phiurel:bus1", "deg, Voltage, Relative Angle"
      n_phiurel_bus1n = "n:phiurel:bus1n", "deg, Voltage, Relative Angle"
      n_ul_bus1_A = "n:ul:bus1:A", "p.u., Line-Line Voltage, Magnitude"
      n_ul_bus1_B = "n:ul:bus1:B", "p.u., Line-Line Voltage, Magnitude"
      n_ul_bus1_C = "n:ul:bus1:C", "p.u., Line-Line Voltage, Magnitude"
      n_ul_bus1_N = "n:ul:bus1:N", "p.u., Line-Line Voltage, Magnitude"
      n_ul_bus1n = "n:ul:bus1n", "p.u., Line-Line Voltage, Magnitude"
      n_ulpc_bus1_A = "n:ulpc:bus1:A", "%, Line-Line Voltage, Magnitude"
      n_ulpc_bus1_B = "n:ulpc:bus1:B", "%, Line-Line Voltage, Magnitude"
      n_ulpc_bus1_C = "n:ulpc:bus1:C", "%, Line-Line Voltage, Magnitude"
      n_ulpc_bus1_N = "n:ulpc:bus1:N", "%, Line-Line Voltage, Magnitude"
      n_ulpc_bus1n = "n:ulpc:bus1n", "%, Line-Line Voltage, Magnitude"
      n_Ul_bus1_A = "n:Ul:bus1:A", "kV, Line-Line Voltage, Magnitude"
      n_Ul_bus1_B = "n:Ul:bus1:B", "kV, Line-Line Voltage, Magnitude"
      n_Ul_bus1_C = "n:Ul:bus1:C", "kV, Line-Line Voltage, Magnitude"
      n_Ul_bus1_N = "n:Ul:bus1:N", "kV, Line-Line Voltage, Magnitude"
      n_Ul_bus1n = "n:Ul:bus1n", "kV, Line-Line Voltage, Magnitude"
      n_phiul_bus1_A = "n:phiul:bus1:A", "deg, Line-Line Voltage, Angle"
      n_phiul_bus1_B = "n:phiul:bus1:B", "deg, Line-Line Voltage, Angle"
      n_phiul_bus1_C = "n:phiul:bus1:C", "deg, Line-Line Voltage, Angle"
      n_phiul_bus1_N = "n:phiul:bus1:N", "deg, Line-Line Voltage, Angle"
      n_phiul_bus1n = "n:phiul:bus1n", "deg, Line-Line Voltage, Angle"
      n_u0_bus1 = "n:u0:bus1", "p.u., Zero-Sequence Voltage, Magnitude"
      n_u0_bus1n = "n:u0:bus1n", "p.u., Zero-Sequence Voltage, Magnitude"
      n_U0_bus1 = "n:U0:bus1", "kV, Zero-Sequence Voltage, Magnitude"
      n_U0_bus1n = "n:U0:bus1n", "kV, Zero-Sequence Voltage, Magnitude"
      n_U0x3_bus1 = "n:U0x3:bus1", "kV, 3*U0"
      n_U0x3_bus1n = "n:U0x3:bus1n", "kV, 3*U0"
      n_phiu0_bus1 = "n:phiu0:bus1", "deg, Zero-Sequence Voltage, Angle"
      n_phiu0_bus1n = "n:phiu0:bus1n", "deg, Zero-Sequence Voltage, Angle"
      n_u1_bus1 = "n:u1:bus1", "p.u., Positive-Sequence Voltage, Magnitude"
      n_u1_bus1n = "n:u1:bus1n", "p.u., Positive-Sequence Voltage, Magnitude"
      n_u1pc_bus1 = "n:u1pc:bus1", "%, Positive-Sequence Voltage, Magnitude"
      n_u1pc_bus1n = "n:u1pc:bus1n", "%, Positive-Sequence Voltage, Magnitude"
      n_u1r_bus1 = "n:u1r:bus1", "p.u., Positive-Sequence Voltage, Real Part"
      n_u1r_bus1n = "n:u1r:bus1n", "p.u., Positive-Sequence Voltage, Real Part"
      n_u1i_bus1 = "n:u1i:bus1", "p.u., Positive-Sequence Voltage, Imaginary Part"
      n_u1i_bus1n = "n:u1i:bus1n", "p.u., Positive-Sequence Voltage, Imaginary Part"
      n_U1_bus1 = "n:U1:bus1", "kV, Line-Ground Positive-Sequence Voltage, Magnitude"
      n_U1_bus1n = "n:U1:bus1n", "kV, Line-Ground Positive-Sequence Voltage, Magnitude"
      n_phiu1_bus1 = "n:phiu1:bus1", "deg, Positive-Sequence Voltage, Angle"
      n_phiu1_bus1n = "n:phiu1:bus1n", "deg, Positive-Sequence Voltage, Angle"
      n_u2_bus1 = "n:u2:bus1", "p.u., Negative-Sequence Voltage, Magnitude"
      n_u2_bus1n = "n:u2:bus1n", "p.u., Negative-Sequence Voltage, Magnitude"
      n_U2_bus1 = "n:U2:bus1", "kV, Line-Ground Negative-Sequence Voltage, Magnitude"
      n_U2_bus1n = "n:U2:bus1n", "kV, Line-Ground Negative-Sequence Voltage, Magnitude"
      n_phiu2_bus1 = "n:phiu2:bus1", "deg, Negative-Sequence Voltage, Angle"
      n_phiu2_bus1n = "n:phiu2:bus1n", "deg, Negative-Sequence Voltage, Angle"
      n_U1l_bus1 = "n:U1l:bus1", "kV, Line-Line Positive-Sequence Voltage, Magnitude"
      n_U1l_bus1n = "n:U1l:bus1n", "kV, Line-Line Positive-Sequence Voltage, Magnitude"
      n_U2l_bus1 = "n:U2l:bus1", "kV, Line-Line Negative-Sequence Voltage, Magnitude"
      n_U2l_bus1n = "n:U2l:bus1n", "kV, Line-Line Negative-Sequence Voltage, Magnitude"
      n_uln_bus1_A = "n:uln:bus1:A", "p.u., Line-Neutral Voltage, Magnitude"
      n_uln_bus1_B = "n:uln:bus1:B", "p.u., Line-Neutral Voltage, Magnitude"
      n_uln_bus1_C = "n:uln:bus1:C", "p.u., Line-Neutral Voltage, Magnitude"
      n_uln_bus1_N = "n:uln:bus1:N", "p.u., Line-Neutral Voltage, Magnitude"
      n_uln_bus1n = "n:uln:bus1n", "p.u., Line-Neutral Voltage, Magnitude"
      n_Uln_bus1_A = "n:Uln:bus1:A", "kV, Line-Neutral Voltage, Magnitude"
      n_Uln_bus1_B = "n:Uln:bus1:B", "kV, Line-Neutral Voltage, Magnitude"
      n_Uln_bus1_C = "n:Uln:bus1:C", "kV, Line-Neutral Voltage, Magnitude"
      n_Uln_bus1_N = "n:Uln:bus1:N", "kV, Line-Neutral Voltage, Magnitude"
      n_Uln_bus1n = "n:Uln:bus1n", "kV, Line-Neutral Voltage, Magnitude"
      n_phiuln_bus1_A = "n:phiuln:bus1:A", "deg, Line-Neutral Voltage, Angle"
      n_phiuln_bus1_B = "n:phiuln:bus1:B", "deg, Line-Neutral Voltage, Angle"
      n_phiuln_bus1_C = "n:phiuln:bus1:C", "deg, Line-Neutral Voltage, Angle"
      n_phiuln_bus1_N = "n:phiuln:bus1:N", "deg, Line-Neutral Voltage, Angle"
      n_phiuln_bus1n = "n:phiuln:bus1n", "deg, Line-Neutral Voltage, Angle"
      n_un_bus1 = "n:un:bus1", "p.u., Neutral-Ground Voltage, Magnitude"
      n_un_bus1n = "n:un:bus1n", "p.u., Neutral-Ground Voltage, Magnitude"
      n_Un_bus1 = "n:Un:bus1", "kV, Neutral-Ground Voltage, Magnitude"
      n_Un_bus1n = "n:Un:bus1n", "kV, Neutral-Ground Voltage, Magnitude"
      n_phiun_bus1 = "n:phiun:bus1", "deg, Neutral-Ground Voltage, Angle"
      n_phiun_bus1n = "n:phiun:bus1n", "deg, Neutral-Ground Voltage, Angle"
      n_upht_bus1_A = "n:upht:bus1:A", "p.u., Phase Technology dependent Voltage, Magnitude"
      n_upht_bus1_B = "n:upht:bus1:B", "p.u., Phase Technology dependent Voltage, Magnitude"
      n_upht_bus1_C = "n:upht:bus1:C", "p.u., Phase Technology dependent Voltage, Magnitude"
      n_upht_bus1_N = "n:upht:bus1:N", "p.u., Phase Technology dependent Voltage, Magnitude"
      n_upht_bus1n = "n:upht:bus1n", "p.u., Phase Technology dependent Voltage, Magnitude"
      n_Upht_bus1_A = "n:Upht:bus1:A", "kV, Phase Technology dependent Voltage, Magnitude"
      n_Upht_bus1_B = "n:Upht:bus1:B", "kV, Phase Technology dependent Voltage, Magnitude"
      n_Upht_bus1_C = "n:Upht:bus1:C", "kV, Phase Technology dependent Voltage, Magnitude"
      n_Upht_bus1_N = "n:Upht:bus1:N", "kV, Phase Technology dependent Voltage, Magnitude"
      n_Upht_bus1n = "n:Upht:bus1n", "kV, Phase Technology dependent Voltage, Magnitude"
      n_um_bus1 = "n:um:bus1", "p.u., Average-Voltage, Magnitude"
      n_um_bus1n = "n:um:bus1n", "p.u., Average-Voltage, Magnitude"
      n_Um_bus1 = "n:Um:bus1", "kV, Average-Voltage, Magnitude"
      n_Um_bus1n = "n:Um:bus1n", "kV, Average-Voltage, Magnitude"
      n_uphtmin_bus1 = "n:uphtmin:bus1", "p.u., Minimum of Phase Technology dependent Voltage, Magnitude"
      n_uphtmin_bus1n = "n:uphtmin:bus1n", "p.u., Minimum of Phase Technology dependent Voltage, Magnitude"
      n_uphtmax_bus1 = "n:uphtmax:bus1", "p.u., Maximum of Phase Technology dependent Voltage, Magnitude"
      n_uphtmax_bus1n = "n:uphtmax:bus1n", "p.u., Maximum of Phase Technology dependent Voltage, Magnitude"
      n_Pgen_bus1 = "n:Pgen:bus1", "MW, Generation, Active Power"
      n_Pgen_bus1n = "n:Pgen:bus1n", "MW, Generation, Active Power"
      n_Qgen_bus1 = "n:Qgen:bus1", "Mvar, Generation, Reactive Power"
      n_Qgen_bus1n = "n:Qgen:bus1n", "Mvar, Generation, Reactive Power"
      n_Pmot_bus1 = "n:Pmot:bus1", "MW, Motor Load, Active Power"
      n_Pmot_bus1n = "n:Pmot:bus1n", "MW, Motor Load, Active Power"
      n_Qmot_bus1 = "n:Qmot:bus1", "Mvar, Motor Load, Reactive Power"
      n_Qmot_bus1n = "n:Qmot:bus1n", "Mvar, Motor Load, Reactive Power"
      n_Pload_bus1 = "n:Pload:bus1", "MW, General Load, Active Power"
      n_Pload_bus1n = "n:Pload:bus1n", "MW, General Load, Active Power"
      n_Qload_bus1 = "n:Qload:bus1", "Mvar, General Load, Reactive Power"
      n_Qload_bus1n = "n:Qload:bus1n", "Mvar, General Load, Reactive Power"
      n_Pcomp_bus1 = "n:Pcomp:bus1", "MW, Compensation (Losses)"
      n_Pcomp_bus1n = "n:Pcomp:bus1n", "MW, Compensation (Losses)"
      n_Qcomp_bus1 = "n:Qcomp:bus1", "Mvar, Compensation"
      n_Qcomp_bus1n = "n:Qcomp:bus1n", "Mvar, Compensation"
      n_Pnet_bus1 = "n:Pnet:bus1", "MW, External Networks, Active Power"
      n_Pnet_bus1n = "n:Pnet:bus1n", "MW, External Networks, Active Power"
      n_Qnet_bus1 = "n:Qnet:bus1", "Mvar, External Networks, Reactive Power"
      n_Qnet_bus1n = "n:Qnet:bus1n", "Mvar, External Networks, Reactive Power"
      n_Pflow_bus1 = "n:Pflow:bus1", "MW, Power Flow, Active Power"
      n_Pflow_bus1n = "n:Pflow:bus1n", "MW, Power Flow, Active Power"
      n_Qflow_bus1 = "n:Qflow:bus1", "Mvar, Power Flow, Reactive Power"
      n_Qflow_bus1n = "n:Qflow:bus1n", "Mvar, Power Flow, Reactive Power"
      n_Pout_bus1 = "n:Pout:bus1", "MW, Outgoing Flow, Active Power"
      n_Pout_bus1n = "n:Pout:bus1n", "MW, Outgoing Flow, Active Power"
      n_Qout_bus1 = "n:Qout:bus1", "Mvar, Outgoing Power, Reactive Power"
      n_Qout_bus1n = "n:Qout:bus1n", "Mvar, Outgoing Power, Reactive Power"
      n_Sout_bus1 = "n:Sout:bus1", "MVA, Outgoing Power, Apparent Power"
      n_Sout_bus1n = "n:Sout:bus1n", "MVA, Outgoing Power, Apparent Power"
      n_cosphiout_bus1 = "n:cosphiout:bus1", "Outgoing Power, Power Factor"
      n_cosphiout_bus1n = "n:cosphiout:bus1n", "Outgoing Power, Power Factor"
      n_Pbalance_bus1 = "n:Pbalance:bus1", "MW, Active Power Balance (=0)"
      n_Pbalance_bus1n = "n:Pbalance:bus1n", "MW, Active Power Balance (=0)"
      n_Qbalance_bus1 = "n:Qbalance:bus1", "Mvar, Reactive Power Balance (=0)"
      n_Qbalance_bus1n = "n:Qbalance:bus1n", "Mvar, Reactive Power Balance (=0)"
      n_ubfac_bus1 = "n:ubfac:bus1", "%, Unbalance Factor"
      n_ubfac_bus1n = "n:ubfac:bus1n", "%, Unbalance Factor"
      n_LossPdown_bus1 = "n:LossPdown:bus1", "MW, Losses, downstream"
      n_LossPdown_bus1n = "n:LossPdown:bus1n", "MW, Losses, downstream"
      n_LossQdown_bus1 = "n:LossQdown:bus1", "Mvar, Losses, downstream (Reactive Power)"
      n_LossQdown_bus1n = "n:LossQdown:bus1n", "Mvar, Losses, downstream (Reactive Power)"
      n_LossPdownload_bus1 = "n:LossPdownload:bus1", "MW, Load losses, downstream"
      n_LossPdownload_bus1n = "n:LossPdownload:bus1n", "MW, Load losses, downstream"
      n_LossQdownload_bus1 = "n:LossQdownload:bus1", "Mvar, Load losses, downstream"
      n_LossQdownload_bus1n = "n:LossQdownload:bus1n", "Mvar, Load losses, downstream"
      n_LossPdownnoload_bus1 = "n:LossPdownnoload:bus1", "MW, No load losses, downstream"
      n_LossPdownnoload_bus1n = "n:LossPdownnoload:bus1n", "MW, No load losses, downstream"
      n_LossQdownnoload_bus1 = "n:LossQdownnoload:bus1", "Mvar, No load losses, downstream"
      n_LossQdownnoload_bus1n = "n:LossQdownnoload:bus1n", "Mvar, No load losses, downstream"
      s_pset = "s:pset", "MW, IN, Active Power Setpoint, continuous, double"
      s_qset = "s:qset", "Mvar, IN, Reactive Power Setpoint continuous, double"
      s_dFin = "s:dFin", "Hz, IN, Frequency Deviation, continuous, double"
      s_i2r_set = "s:i2r_set", "p.u., IN, Negative sequence current setpoint, Real Part, continuous, double"
      s_i2i_set = "s:i2i_set", "p.u., IN, Negative sequence current setpoint, Imaginary Part, continuous, double"
      c_p = "c:p", "MW, Total Active Power"
      c_q = "c:q", "Mvar, Total Reactive Power"
      c_sgnn = "c:sgnn", "MVA, Total rated apparent power (ngnum*sgn)"
      c_cosn = "c:cosn", "Nominal-Power factor"
      c_Pnom = "c:Pnom", "MW, Total Nominal Active power"
      c_c2 = "c:c2", "Voltage Coefficient (uknom/ugn)"
      c_uini_r = "c:uini:r", "p.u., Voltage Setpoint for PV and Slack, Real Part"
      c_uini_i = "c:uini:i", "p.u., Voltage Setpoint for PV and Slack, Imaginary Part"
      c_uini_phi = "c:uini:phi", "deg, Voltage Setpoint for PV and Slack, Angle"
      c_uini_mag = "c:uini:mag", "p.u., Voltage Setpoint for PV and Slack, Magnitude"
      c_uini_phirad = "c:uini:phirad", "rad, Voltage Setpoint for PV and Slack, Angle"
      c_phiini = "c:phiini", "Initial Angle"
      c_loading = "c:loading", "%, Loading"
      c_iPowerOn = "c:iPowerOn", "Flag"
      c_Qmin = "c:Qmin", "Mvar, Lower limit of reactive power"
      c_Qmax = "c:Qmax", "Mvar, Upper limit of reactive power"
      c_Qsparelead = "c:Qsparelead", "Mvar, Q Spare Lead"
      c_Qsparelag = "c:Qsparelag", "Mvar, Q Spare Lag"
      c_i_qlim = "c:i_qlim", "Reactive power limit reached"
      c_i_vlim = "c:i_vlim", "Voltage setpoint limit reached"
      c_i_plim = "c:i_plim", "Active power limit reached"
      c_sm_r = "c:sm:r", "MW, Complex Power, Real Part"
      c_sm_i = "c:sm:i", "Mvar, Complex Power, Imaginary Part"
      c_sm_phi = "c:sm:phi", "deg, Complex Power, Angle"
      c_sm_mag = "c:sm:mag", "MVA, Complex Power, Magnitude"
      c_sm_phirad = "c:sm:phirad", "rad, Complex Power, Angle"
      c_u1_r = "c:u1:r", "p.u., Internal, Real Part"
      c_u1_i = "c:u1:i", "p.u., Internal, Imaginary Part"
      c_u1_phi = "c:u1:phi", "deg, Internal, Angle"
      c_u1_mag = "c:u1:mag", "p.u., Internal, Magnitude"
      c_u1_phirad = "c:u1:phirad", "rad, Internal, Angle"
      c_Kp = "c:Kp", "p.u., Active Power Participation"
      c_Kq = "c:Kq", "p.u., Reactive Power Participation"
      c_Kpf = "c:Kpf", "MW/Hz, Frequency Bias"
      c_iRefElement = "c:iRefElement", "Reference Element"
      c_zonewscale = "c:zonewscale", "Wind generation zone scaling factor"
      c_Pmism = "c:Pmism", "MW, Active Power Mismatch"
      c_nCtrlIter = "c:nCtrlIter", "No. of CTRL-iterations"
      c_Ploss = "c:Ploss", "MW, Losses (total)"
      c_Qloss = "c:Qloss", "Mvar, Reactive-Losses (total)"
      c_Plossld = "c:Plossld", "MW, Losses (load)"
      c_Qlossld = "c:Qlossld", "Mvar, Reactive-Losses (load)"
      c_Plossnld = "c:Plossnld", "MW, Losses (no load)"
      c_Qlossnld = "c:Qlossnld", "Mvar, Reactive-Losses (no load)"
      e_ip_ctrl = "e:ip_ctrl", "Reference Machine"
      e_outServPzero = "e:outServPzero", "Out of service when active power is zero"
      e_av_mode = "e:av_mode", "Local Controller"
      e_c_psecc = "e:c_psecc", "ElmSecctrl, External Secondary Controller"
      e_c_pstac = "e:c_pstac", "ElmStactrl, External Station Controller"
      e_qdslCtrl = "e:qdslCtrl", "Quasi-Dynamic Model"
      e_mode_pgi = "e:mode_pgi", "Model"
      e_pPowerCrv = "e:pPowerCrv", "TypPowercurve, Wind Power Curve"
      e_mode_inp = "e:mode_inp", "Input Mode"
      e_pgini = "e:pgini", "MW, Active Power"
      e_qgini = "e:qgini", "Mvar, Reactive Power"
      e_sgini = "e:sgini", "MVA, Apparent Power"
      e_cosgini = "e:cosgini", "Power Factor"
      e_pf_recap = "e:pf_recap", "Power Factor"
      e_p_direc = "e:p_direc", "Power Direction"
      e_windspeed = "e:windspeed", "m/s, Wind speed"
      e_usetp = "e:usetp", "p.u., Voltage"
      e_phiini = "e:phiini", "deg, Angle"
      e_Kpf = "e:Kpf", "MW/Hz, Prim. Frequency Bias"
      e_scale0 = "e:scale0", "Scaling Factor"
      e_pgini_a = "e:pgini_a", "MW, Active Power (act.)"
      e_qgini_a = "e:qgini_a", "Mvar, Reactive Power (act.)"
      e_sgini_a = "e:sgini_a", "MVA, Apparent Power (act.)"
      e_cosgini_a = "e:cosgini_a", "Power Factor (act.)"
      e_pf_recap_a = "e:pf_recap_a", "Power Factor Ind/Cap (act.)"
      e_scale0_a = "e:scale0_a", "Scaling Factor (act.)"
      e_windspeed_a = "e:windspeed_a", "m/s, Wind speed (act.)"
      e_usp_min = "e:usp_min", "p.u., Min. Voltage Setpoint"
      e_usp_max = "e:usp_max", "p.u., Max. Voltage Setpoint"
      e_ddroop = "e:ddroop", "%, Droop"
      e_pQPcurve = "e:pQPcurve", "IntQpcurve, Q(P)-Curve"
      e_iopt_tdr = "e:iopt_tdr", "Different droop values"
      e_ddroopoe = "e:ddroopoe", "%, Droop (overexcited)"
      e_ddroopue = "e:ddroopue", "%, Droop (underexcited)"
      e_Qfu_max = "e:Qfu_max", "Mvar, Q max"
      e_Qfu_min = "e:Qfu_min", "Mvar, Q min"
      e_udeadblow = "e:udeadblow", "p.u., Lower Voltage Limit"
      e_udeadbup = "e:udeadbup", "p.u., Upper Voltage Limit"
      e_pf_over = "e:pf_over", "Min. Power Factor"
      e_p_over = "e:p_over", "MW, Active Power"
      e_pf_under = "e:pf_under", "Min. Power Factor"
      e_p_under = "e:p_under", "MW, Active Power"
      e_pQlimType = "e:pQlimType", "IntQlim, Capability Curve"
      e_q_min = "e:q_min", "p.u., Min."
      e_q_max = "e:q_max", "p.u., Max."
      e_cQ_min = "e:cQ_min", "Mvar, Minimum"
      e_cQ_max = "e:cQ_max", "Mvar, Maximum"
      e_scaleQmin = "e:scaleQmin", "%, Scaling Factor (min.)"
      e_scaleQmax = "e:scaleQmax", "%, Scaling Factor (max.)"
      e_Pmin_uc = "e:Pmin_uc", "MW, Min."
      e_Pmin_ucPU = "e:Pmin_ucPU", "p.u., Min."
      e_Pmax_uc = "e:Pmax_uc", "MW, Max."
      e_Pmax_ucPU = "e:Pmax_ucPU", "p.u., Max."
      e_Pnom = "e:Pnom", "MW, Pr(rated)"
      e_P_max = "e:P_max", "MW, Max."
      e_pmaxratf = "e:pmaxratf", "Rating Factor"
      e_c_pCtrlHV = "e:c_pCtrlHV", "Controlled HV-busbar"
      e_Uctrl = "e:Uctrl", "kV, HV-Voltage Setpoint"
      e_Qmin_a = "e:Qmin_a", "Mvar, Qmin(act.)"
      e_Qmax_a = "e:Qmax_a", "Mvar, Qmax(act.)"
      e_Pmin_a = "e:Pmin_a", "MW, Pmin(act.)"
      e_Pmax_a = "e:Pmax_a", "MW, Pmax(act.)"
      e_r2 = "e:r2", "p.u., Resistance r2"
      e_x2 = "e:x2", "p.u., Reactance x2"
      e_is4SpinReserve = "e:is4SpinReserve", "Consider for region spinning reserve"
      e_genShiftKey = "e:genShiftKey", "%, Generation shift key"
      e_pBMU = "e:pBMU", "ElmBmu, Virtual Power Plant"
      e_dispatch = "e:dispatch", "Generator Dispatch"
      e_priority = "e:priority", "Merit Order"
      e_fixed = "e:fixed", "Must run"
      r_c_psecc = "r:c_psecc", "ElmSecctrl, External Secondary Controller"
      r_c_pstac = "r:c_pstac", "ElmStactrl, External Station Controller"
      r_qdslCtrl = "r:qdslCtrl", "Quasi-Dynamic Model"
      r_pPowerCrv = "r:pPowerCrv", "TypPowercurve, Wind Power Curve"
      r_pQPcurve = "r:pQPcurve", "IntQpcurve, Q(P)-Curve"
      r_pQlimType = "r:pQlimType", "IntQlim, Capability Curve"
      r_c_pCtrlHV = "r:c_pCtrlHV", "Controlled HV-busbar"
      r_pBMU = "r:pBMU", "ElmBmu, Virtual Power Plant"
      m_Ikss_bus1 = "m:Ikss:bus1", "kA, Initial Short-Circuit Current"
      m_ikss_bus1 = "m:ikss:bus1", "p.u., Initial Short-Circuit Current, Magnitude"
      m_phii_bus1 = "m:phii:bus1", "deg, Phase Current, Angle"
      m_maxIkss_bus1 = "m:maxIkss:bus1", "kA, Max. Initial Short-Circuit Current"
      m_Skss_bus1 = "m:Skss:bus1", "MVA, Initial Short-Circuit Power"
      m_ip_bus1 = "m:ip:bus1", "kA, Peak Short-Circuit Current"
      m_maxip_bus1 = "m:maxip:bus1", "kA, Max. Peak Short-Circuit Current"
      m_maxIb_bus1 = "m:maxIb:bus1", "kA, Max. Breaking Current"
      m_maxIth_bus1 = "m:maxIth:bus1", "kA, Max. Thermal equivalent Short-Circuit Current"
      m_Ib_bus1 = "m:Ib:bus1", "kA, Short-Circuit Breaking Current"
      m_Sb_bus1 = "m:Sb:bus1", "MVA, Short-Circuit Breaking Power"
      m_Ik_bus1 = "m:Ik:bus1", "kA, Steady-State Short-Circuit Current"
      m_Ith_bus1 = "m:Ith:bus1", "kA, Thermal equivalent Short-Circuit Current"
      m_I_bus1 = "m:I:bus1", "kA, Short-Circuit Current"
      m_R_bus1 = "m:R:bus1", "Ohm, Fault Impedance, Real Part"
      m_X_bus1 = "m:X:bus1", "Ohm, Fault Impedance, Imaginary Part"
      m_Z_bus1 = "m:Z:bus1", "Ohm, Fault Impedance, Magnitude"
      m_phiz_bus1 = "m:phiz:bus1", "deg, Fault Impedance, Angle"
      m_zSbase_bus1 = "m:zSbase:bus1", "p.u., Fault Impedance, Magnitude"
      m_XtoR_bus1 = "m:XtoR:bus1", "X/R ratio"
      m_Zs_bus1 = "m:Zs:bus1", "Ohm, Source Impedance, Magnitude"
      m_phizs_bus1 = "m:phizs:bus1", "deg, Source Impedance, Angle"
      m_Rs_bus1 = "m:Rs:bus1", "Ohm, Source Impedance, Real Part"
      m_Xs_bus1 = "m:Xs:bus1", "Ohm, Source Impedance, Imaginary Part"
      m_phiui_bus1 = "m:phiui:bus1", "deg, Angle between Voltage and Current"
      m_cosphiui_bus1 = "m:cosphiui:bus1", "cosphi"
      m_P_bus1 = "m:P:bus1", "MW, Active Power"
      m_Q_bus1 = "m:Q:bus1", "Mvar, Reactive Power"
      m_BrkIpload_bus1 = "m:BrkIpload:bus1", "%, Breaker Loading (peak)"
      m_BrkIbload_bus1 = "m:BrkIbload:bus1", "%, Breaker Loading (breaking)"
      m_BrkmaxIb_bus1 = "m:BrkmaxIb:bus1", "kA, Max. Breaking Current (breaker time)"
      n_Ikss_bus1 = "n:Ikss:bus1", "kA, Initial Short-Circuit Current"
      n_I_bus1 = "n:I:bus1", "kA, Short-Circuit Current"
      n_phii_bus1 = "n:phii:bus1", "deg, Phase Current, Angle"
      n_Skss_bus1 = "n:Skss:bus1", "MVA, Initial Short-Circuit Power"
      n_ip_bus1 = "n:ip:bus1", "kA, Peak Short-Circuit Current"
      n_Ib_bus1 = "n:Ib:bus1", "kA, Short-Circuit Breaking Current"
      n_Sb_bus1 = "n:Sb:bus1", "MVA, Short-Circuit Breaking Power"
      n_Ik_bus1 = "n:Ik:bus1", "kA, Steady-State Short-Circuit Current"
      n_Ith_bus1 = "n:Ith:bus1", "kA, Thermal equivalent Short-Circuit Current"
      n_R_bus1 = "n:R:bus1", "Ohm, Short-Circuit Impedance, Real Part"
      n_X_bus1 = "n:X:bus1", "Ohm, Short-Circuit Impedance, Imaginary Part"
      n_Z_bus1 = "n:Z:bus1", "Ohm, Short-Circuit Impedance, Magnitude"
      n_phiz_bus1 = "n:phiz:bus1", "deg, Short-Circuit Impedance, Angle"
      n_rSbase_bus1 = "n:rSbase:bus1", "p.u., Short-Circuit Impedance, Real Part"
      n_xSbase_bus1 = "n:xSbase:bus1", "p.u., Short-Circuit Impedance, Imaginary Part"
      n_zSbase_bus1 = "n:zSbase:bus1", "p.u., Short-Circuit Impedance, Magnitude"
      n_XtoR_bus1 = "n:XtoR:bus1", "X/R ratio (peak)"
      n_RtoX_bus1 = "n:RtoX:bus1", "R/X ratio (peak)"
      n_ur_bus1 = "n:ur:bus1", "p.u., Voltage, Real Part"
      n_ui_bus1 = "n:ui:bus1", "p.u., Voltage, Imaginary Part"
      n_u_bus1 = "n:u:bus1", "p.u., Voltage, Magnitude"
      n_upc_bus1 = "n:upc:bus1", "%, Voltage, Magnitude"
      n_U_bus1 = "n:U:bus1", "kV, Line-Ground Voltage, Magnitude"
      n_Ul_bus1 = "n:Ul:bus1", "kV, Line-Line Voltage, Magnitude"
      n_phiu_bus1 = "n:phiu:bus1", "deg, Voltage, Angle"
      n_du_bus1 = "n:du:bus1", "%, Voltage Deviation"
      n_urpref_bus1 = "n:urpref:bus1", "p.u., Pre-fault Voltage, Real Part"
      n_uipref_bus1 = "n:uipref:bus1", "p.u., Pre-fault Voltage, Imaginary Part"
      n_upref_bus1 = "n:upref:bus1", "p.u., Pre-fault Voltage, Magnitude"
      n_uprefpc_bus1 = "n:uprefpc:bus1", "%, Pre-fault Voltage, Magnitude"
      n_Upref_bus1 = "n:Upref:bus1", "kV, Line-Ground Pre-fault Voltage, Magnitude"
      n_Ulpref_bus1 = "n:Ulpref:bus1", "kV, Line-Line Pre-fault Voltage, Magnitude"
      n_phiupref_bus1 = "n:phiupref:bus1", "deg, Pre-fault Voltage, Angle"
      n_phiui_bus1 = "n:phiui:bus1", "deg, Angle between Voltage and Current"
      n_cfac_bus1 = "n:cfac:bus1", "Voltage Factor"
      n_idc_bus1 = "n:idc:bus1", "kA, Decaying Component"
      n_Iasy_bus1 = "n:Iasy:bus1", "kA, Asymmetrical Breaking Current"
      n_Ithload_bus1 = "n:Ithload:bus1", "%, Loading, Thermal equivalent Current"
      n_Ipload_bus1 = "n:Ipload:bus1", "%, Loading, Peak Short-Circuit Current"
      n_Ithrtk_bus1 = "n:Ithrtk:bus1", "kA, Rated Short-Time Current (Tk)"
      n_Tn_bus1 = "n:Tn:bus1", "ms, Grid Time Constant"
      n_XtoR_b_bus1 = "n:XtoR_b:bus1", "X/R ratio (dc)"
      n_RtoX_b_bus1 = "n:RtoX_b:bus1", "R/X ratio (dc)"
      n_fctof_b_bus1 = "n:fctof_b:bus1", "Equivalent Frequency fc/f (dc)"
      c_z1_r = "c:z1:r", "p.u., Positive-sequence impedance, Real Part"
      c_z1_i = "c:z1:i", "p.u., Positive-sequence impedance, Imaginary Part"
      c_z1_phi = "c:z1:phi", "deg, Positive-sequence impedance, Angle"
      c_z1_mag = "c:z1:mag", "p.u., Positive-sequence impedance, Magnitude"
      c_z1_phirad = "c:z1:phirad", "rad, Positive-sequence impedance, Angle"
      c_imax = "c:imax", "p.u., Max. current"
      c_Zwd_r = "c:Zwd:r", "p.u., Impedance of doubly-fed asynchronous generator, Real Part"
      c_Zwd_i = "c:Zwd:i", "p.u., Impedance of doubly-fed asynchronous generator, Imaginary Part"
      c_Zwd_phi = "c:Zwd:phi", "deg, Impedance of doubly-fed asynchronous generator, Angle"
      c_Zwd_mag = "c:Zwd:mag", "p.u., Impedance of doubly-fed asynchronous generator, Magnitude"
      c_Zwd_phirad = "c:Zwd:phirad", "rad, Impedance of doubly-fed asynchronous generator, Angle"
      e_iNoShcContr = "e:iNoShcContr", "No Short-Circuit Contribution"
      e_iconfed = "e:iconfed", "Static Converter-Fed Drive"
      e_psutype = "e:psutype", "Power station unit type"
      e_searchBlockTrf = "e:searchBlockTrf", "Externally modelled unit transformer"
      e_pblocktrf = "e:pblocktrf", "Unit transformer"
      e_Ikss3PF = "e:Ikss3PF", "kA, Three-phase faults, Ik''3PF"
      e_Ikss2PF = "e:Ikss2PF", "kA, Two-phase faults, Ik''2PF"
      e_Ikss1PF = "e:Ikss1PF", "kA, Single-phase faults, Ik''1PF"
      e_iWDmax = "e:iWDmax", "kA, Max. instantaneous short-circuit current, iWDmax"
      e_kWD = "e:kWD", "Factor kappaWD"
      e_rxWD = "e:rxWD", "Ratio RWD/XWD"
      e_IkPFmax = "e:IkPFmax", "kA, Maximum current"
      e_IkPFmin = "e:IkPFmin", "kA, Minimum current"
      e_IkWDmax = "e:IkWDmax", "kA, Maximum current"
      e_IkWDmin = "e:IkWDmin", "kA, Minimum current"
      e_r2iec = "e:r2iec", "p.u., Resistance, r2"
      e_x2iec = "e:x2iec", "p.u., Reactance, x2"
      e_r0iec = "e:r0iec", "p.u., Resistance, r0"
      e_x0iec = "e:x0iec", "p.u., Reactance, x0"
      e_Skss = "e:Skss", "MVA, Subtransient Short-Circuit Level"
      e_Ikss = "e:Ikss", "kA, Subtransient Short-Circuit Current"
      e_rtox = "e:rtox", "R to X'' ratio"
      e_xtor = "e:xtor", "X'' to R ratio"
      e_r2shc = "e:r2shc", "p.u., Resistance, r2"
      e_x2shc = "e:x2shc", "p.u., Reactance, x2"
      r_pblocktrf = "r:pblocktrf", "Unit transformer"
      m_Ikss_bus1_A = "m:Ikss:bus1:A", "kA, Initial Short-Circuit Current"
      m_Ikss_bus1_B = "m:Ikss:bus1:B", "kA, Initial Short-Circuit Current"
      m_Ikss_bus1_C = "m:Ikss:bus1:C", "kA, Initial Short-Circuit Current"
      m_Ikss_bus1_N = "m:Ikss:bus1:N", "kA, Initial Short-Circuit Current"
      m_Ikss_bus1n = "m:Ikss:bus1n", "kA, Initial Short-Circuit Current"
      m_ikss_bus1_A = "m:ikss:bus1:A", "p.u., Initial Short-Circuit Current, Magnitude"
      m_ikss_bus1_B = "m:ikss:bus1:B", "p.u., Initial Short-Circuit Current, Magnitude"
      m_ikss_bus1_C = "m:ikss:bus1:C", "p.u., Initial Short-Circuit Current, Magnitude"
      m_ikss_bus1_N = "m:ikss:bus1:N", "p.u., Initial Short-Circuit Current, Magnitude"
      m_ikss_bus1n = "m:ikss:bus1n", "p.u., Initial Short-Circuit Current, Magnitude"
      m_maxIkss_bus1n = "m:maxIkss:bus1n", "kA, Max. Initial Short-Circuit Current"
      m_Skss_bus1_A = "m:Skss:bus1:A", "MVA, Initial Short-Circuit Power"
      m_Skss_bus1_B = "m:Skss:bus1:B", "MVA, Initial Short-Circuit Power"
      m_Skss_bus1_C = "m:Skss:bus1:C", "MVA, Initial Short-Circuit Power"
      m_Skss_bus1_N = "m:Skss:bus1:N", "MVA, Initial Short-Circuit Power"
      m_Skss_bus1n = "m:Skss:bus1n", "MVA, Initial Short-Circuit Power"
      m_Skssx3_bus1_A = "m:Skssx3:bus1:A", "MVA, 3*Initial Short-Circuit Power"
      m_Skssx3_bus1_B = "m:Skssx3:bus1:B", "MVA, 3*Initial Short-Circuit Power"
      m_Skssx3_bus1_C = "m:Skssx3:bus1:C", "MVA, 3*Initial Short-Circuit Power"
      m_Skssx3_bus1_N = "m:Skssx3:bus1:N", "MVA, 3*Initial Short-Circuit Power"
      m_Skssx3_bus1n = "m:Skssx3:bus1n", "MVA, 3*Initial Short-Circuit Power"
      m_ip_bus1_A = "m:ip:bus1:A", "kA, Peak Short-Circuit Current"
      m_ip_bus1_B = "m:ip:bus1:B", "kA, Peak Short-Circuit Current"
      m_ip_bus1_C = "m:ip:bus1:C", "kA, Peak Short-Circuit Current"
      m_ip_bus1_N = "m:ip:bus1:N", "kA, Peak Short-Circuit Current"
      m_ip_bus1n = "m:ip:bus1n", "kA, Peak Short-Circuit Current"
      m_maxip_bus1n = "m:maxip:bus1n", "kA, Max. Peak Short-Circuit Current"
      m_maxIth_bus1n = "m:maxIth:bus1n", "kA, Max. Thermal equivalent Short-Circuit Current"
      m_Z_bus1_A = "m:Z:bus1:A", "Ohm, Fault Impedance, Magnitude"
      m_Z_bus1_B = "m:Z:bus1:B", "Ohm, Fault Impedance, Magnitude"
      m_Z_bus1_C = "m:Z:bus1:C", "Ohm, Fault Impedance, Magnitude"
      m_Z_bus1_N = "m:Z:bus1:N", "Ohm, Fault Impedance, Magnitude"
      m_Z_bus1n = "m:Z:bus1n", "Ohm, Fault Impedance, Magnitude"
      m_phiz_bus1_A = "m:phiz:bus1:A", "deg, Fault Impedance, Angle"
      m_phiz_bus1_B = "m:phiz:bus1:B", "deg, Fault Impedance, Angle"
      m_phiz_bus1_C = "m:phiz:bus1:C", "deg, Fault Impedance, Angle"
      m_phiz_bus1_N = "m:phiz:bus1:N", "deg, Fault Impedance, Angle"
      m_phiz_bus1n = "m:phiz:bus1n", "deg, Fault Impedance, Angle"
      m_Zl_bus1_A = "m:Zl:bus1:A", "Ohm, Line-Line Fault Impedance, Magnitude"
      m_Zl_bus1_B = "m:Zl:bus1:B", "Ohm, Line-Line Fault Impedance, Magnitude"
      m_Zl_bus1_C = "m:Zl:bus1:C", "Ohm, Line-Line Fault Impedance, Magnitude"
      m_Zl_bus1_N = "m:Zl:bus1:N", "Ohm, Line-Line Fault Impedance, Magnitude"
      m_Zl_bus1n = "m:Zl:bus1n", "Ohm, Line-Line Fault Impedance, Magnitude"
      m_phizl_bus1_A = "m:phizl:bus1:A", "deg, Line-Line Fault Impedance, Angle"
      m_phizl_bus1_B = "m:phizl:bus1:B", "deg, Line-Line Fault Impedance, Angle"
      m_phizl_bus1_C = "m:phizl:bus1:C", "deg, Line-Line Fault Impedance, Angle"
      m_phizl_bus1_N = "m:phizl:bus1:N", "deg, Line-Line Fault Impedance, Angle"
      m_phizl_bus1n = "m:phizl:bus1n", "deg, Line-Line Fault Impedance, Angle"
      m_Z0_bus1 = "m:Z0:bus1", "Ohm, Zero-Sequence Fault Impedance, Magnitude"
      m_Z0_bus1n = "m:Z0:bus1n", "Ohm, Zero-Sequence Fault Impedance, Magnitude"
      m_phiz0_bus1 = "m:phiz0:bus1", "deg, Zero-Sequence Fault Impedance, Angle"
      m_phiz0_bus1n = "m:phiz0:bus1n", "deg, Zero-Sequence Fault Impedance, Angle"
      m_z0Sbase_bus1 = "m:z0Sbase:bus1", "p.u., Zero-Sequence Fault Impedance, Magnitude"
      m_z0Sbase_bus1n = "m:z0Sbase:bus1n", "p.u., Zero-Sequence Fault Impedance, Magnitude"
      m_cosphiu1i1_bus1 = "m:cosphiu1i1:bus1", "Positive-Sequence cosphi"
      m_cosphiu1i1_bus1n = "m:cosphiu1i1:bus1n", "Positive-Sequence cosphi"
      m_I0x3watt_bus1 = "m:I0x3watt:bus1", "kA, Earth-Fault Residual Current"
      m_I0x3watt_bus1n = "m:I0x3watt:bus1n", "kA, Earth-Fault Residual Current"
      m_I0x3var_bus1 = "m:I0x3var:bus1", "kA, Capacitive Earth-Fault Current"
      m_I0x3var_bus1n = "m:I0x3var:bus1n", "kA, Capacitive Earth-Fault Current"
      m_Q0cap_bus1 = "m:Q0cap:bus1", "Mvar, Q0 capacitive"
      m_Q0cap_bus1n = "m:Q0cap:bus1n", "Mvar, Q0 capacitive"
      m_Zfrel_bus1 = "m:Zfrel:bus1", "Ohm, Relay Impedance, Magnitude"
      m_Zfrel_bus1n = "m:Zfrel:bus1n", "Ohm, Relay Impedance, Magnitude"
      m_phizfrel_bus1 = "m:phizfrel:bus1", "deg, Relay Impedance, Angle"
      m_phizfrel_bus1n = "m:phizfrel:bus1n", "deg, Relay Impedance, Angle"
      m_Rfrel_bus1 = "m:Rfrel:bus1", "Ohm, Relay Impedance, Real Part"
      m_Rfrel_bus1n = "m:Rfrel:bus1n", "Ohm, Relay Impedance, Real Part"
      m_Xfrel_bus1 = "m:Xfrel:bus1", "Ohm, Relay Impedance, Imaginary Part"
      m_Xfrel_bus1n = "m:Xfrel:bus1n", "Ohm, Relay Impedance, Imaginary Part"
      m_Zs_bus1n = "m:Zs:bus1n", "Ohm, Source Impedance, Magnitude"
      m_phizs_bus1n = "m:phizs:bus1n", "deg, Source Impedance, Angle"
      m_Rs_bus1n = "m:Rs:bus1n", "Ohm, Source Impedance, Real Part"
      m_Xs_bus1n = "m:Xs:bus1n", "Ohm, Source Impedance, Imaginary Part"
      m_Zf_bus1 = "m:Zf:bus1", "Ohm, Meas. Fault Impedance, Magnitude"
      m_Zf_bus1n = "m:Zf:bus1n", "Ohm, Meas. Fault Impedance, Magnitude"
      m_phizf_bus1 = "m:phizf:bus1", "deg, Meas. Fault Impedance, Angle"
      m_phizf_bus1n = "m:phizf:bus1n", "deg, Meas. Fault Impedance, Angle"
      m_BrkIpload_bus1n = "m:BrkIpload:bus1n", "%, Max. Breaker Loading (peak)"
      n_Ikss_bus1_A = "n:Ikss:bus1:A", "kA, Initial Short-Circuit Current"
      n_Ikss_bus1_B = "n:Ikss:bus1:B", "kA, Initial Short-Circuit Current"
      n_Ikss_bus1_C = "n:Ikss:bus1:C", "kA, Initial Short-Circuit Current"
      n_Ikss_bus1_N = "n:Ikss:bus1:N", "kA, Initial Short-Circuit Current"
      n_Ikss_bus1n = "n:Ikss:bus1n", "kA, Initial Short-Circuit Current"
      n_I_bus1_A = "n:I:bus1:A", "kA, Short-Circuit Current"
      n_I_bus1_B = "n:I:bus1:B", "kA, Short-Circuit Current"
      n_I_bus1_C = "n:I:bus1:C", "kA, Short-Circuit Current"
      n_I_bus1_N = "n:I:bus1:N", "kA, Short-Circuit Current"
      n_I_bus1n = "n:I:bus1n", "kA, Short-Circuit Current"
      n_phii_bus1_A = "n:phii:bus1:A", "deg, Phase Current, Angle"
      n_phii_bus1_B = "n:phii:bus1:B", "deg, Phase Current, Angle"
      n_phii_bus1_C = "n:phii:bus1:C", "deg, Phase Current, Angle"
      n_phii_bus1_N = "n:phii:bus1:N", "deg, Phase Current, Angle"
      n_phii_bus1n = "n:phii:bus1n", "deg, Phase Current, Angle"
      n_Skss_bus1_A = "n:Skss:bus1:A", "MVA, Initial Short-Circuit Power"
      n_Skss_bus1_B = "n:Skss:bus1:B", "MVA, Initial Short-Circuit Power"
      n_Skss_bus1_C = "n:Skss:bus1:C", "MVA, Initial Short-Circuit Power"
      n_Skss_bus1_N = "n:Skss:bus1:N", "MVA, Initial Short-Circuit Power"
      n_Skss_bus1n = "n:Skss:bus1n", "MVA, Initial Short-Circuit Power"
      n_Skssx3_bus1_A = "n:Skssx3:bus1:A", "MVA, 3*Initial Short-Circuit Power"
      n_Skssx3_bus1_B = "n:Skssx3:bus1:B", "MVA, 3*Initial Short-Circuit Power"
      n_Skssx3_bus1_C = "n:Skssx3:bus1:C", "MVA, 3*Initial Short-Circuit Power"
      n_Skssx3_bus1_N = "n:Skssx3:bus1:N", "MVA, 3*Initial Short-Circuit Power"
      n_Skssx3_bus1n = "n:Skssx3:bus1n", "MVA, 3*Initial Short-Circuit Power"
      n_ip_bus1_A = "n:ip:bus1:A", "kA, Peak Short-Circuit Current"
      n_ip_bus1_B = "n:ip:bus1:B", "kA, Peak Short-Circuit Current"
      n_ip_bus1_C = "n:ip:bus1:C", "kA, Peak Short-Circuit Current"
      n_ip_bus1_N = "n:ip:bus1:N", "kA, Peak Short-Circuit Current"
      n_ip_bus1n = "n:ip:bus1n", "kA, Peak Short-Circuit Current"
      n_Ib_bus1_A = "n:Ib:bus1:A", "kA, Short-Circuit Breaking Current"
      n_Ib_bus1_B = "n:Ib:bus1:B", "kA, Short-Circuit Breaking Current"
      n_Ib_bus1_C = "n:Ib:bus1:C", "kA, Short-Circuit Breaking Current"
      n_Ib_bus1_N = "n:Ib:bus1:N", "kA, Short-Circuit Breaking Current"
      n_Ib_bus1n = "n:Ib:bus1n", "kA, Short-Circuit Breaking Current"
      n_Sb_bus1_A = "n:Sb:bus1:A", "MVA, Short-Circuit Breaking Power"
      n_Sb_bus1_B = "n:Sb:bus1:B", "MVA, Short-Circuit Breaking Power"
      n_Sb_bus1_C = "n:Sb:bus1:C", "MVA, Short-Circuit Breaking Power"
      n_Sb_bus1_N = "n:Sb:bus1:N", "MVA, Short-Circuit Breaking Power"
      n_Sb_bus1n = "n:Sb:bus1n", "MVA, Short-Circuit Breaking Power"
      n_Ith_bus1_A = "n:Ith:bus1:A", "kA, Thermal equivalent Short-Circuit Current"
      n_Ith_bus1_B = "n:Ith:bus1:B", "kA, Thermal equivalent Short-Circuit Current"
      n_Ith_bus1_C = "n:Ith:bus1:C", "kA, Thermal equivalent Short-Circuit Current"
      n_Ith_bus1_N = "n:Ith:bus1:N", "kA, Thermal equivalent Short-Circuit Current"
      n_Ith_bus1n = "n:Ith:bus1n", "kA, Thermal equivalent Short-Circuit Current"
      n_Ithmax_bus1 = "n:Ithmax:bus1", "kA, Max. Thermal equivalent Short-Circuit Current"
      n_Ithmax_bus1n = "n:Ithmax:bus1n", "kA, Max. Thermal equivalent Short-Circuit Current"
      n_Ik_bus1_A = "n:Ik:bus1:A", "kA, Steady-State Short-Circuit Current"
      n_Ik_bus1_B = "n:Ik:bus1:B", "kA, Steady-State Short-Circuit Current"
      n_Ik_bus1_C = "n:Ik:bus1:C", "kA, Steady-State Short-Circuit Current"
      n_Ik_bus1_N = "n:Ik:bus1:N", "kA, Steady-State Short-Circuit Current"
      n_Ik_bus1n = "n:Ik:bus1n", "kA, Steady-State Short-Circuit Current"
      n_R0_bus1 = "n:R0:bus1", "Ohm, Zero-Sequence Impedance, Real Part"
      n_R0_bus1n = "n:R0:bus1n", "Ohm, Zero-Sequence Impedance, Real Part"
      n_X0_bus1 = "n:X0:bus1", "Ohm, Zero-Sequence Impedance, Imaginary Part"
      n_X0_bus1n = "n:X0:bus1n", "Ohm, Zero-Sequence Impedance, Imaginary Part"
      n_Z0_bus1 = "n:Z0:bus1", "Ohm, Zero-Sequence Impedance, Magnitude"
      n_Z0_bus1n = "n:Z0:bus1n", "Ohm, Zero-Sequence Impedance, Magnitude"
      n_phiz0_bus1 = "n:phiz0:bus1", "deg, Zero-Sequence Impedance, Angle"
      n_phiz0_bus1n = "n:phiz0:bus1n", "deg, Zero-Sequence Impedance, Angle"
      n_r0Sbase_bus1 = "n:r0Sbase:bus1", "p.u., Zero-Sequence Impedance, Real Part"
      n_r0Sbase_bus1n = "n:r0Sbase:bus1n", "p.u., Zero-Sequence Impedance, Real Part"
      n_x0Sbase_bus1 = "n:x0Sbase:bus1", "p.u., Zero-Sequence Impedance, Imaginary Part"
      n_x0Sbase_bus1n = "n:x0Sbase:bus1n", "p.u., Zero-Sequence Impedance, Imaginary Part"
      n_z0Sbase_bus1 = "n:z0Sbase:bus1", "p.u., Zero-Sequence Impedance, Magnitude"
      n_z0Sbase_bus1n = "n:z0Sbase:bus1n", "p.u., Zero-Sequence Impedance, Magnitude"
      n_R1_bus1 = "n:R1:bus1", "Ohm, Positive-Sequence Impedance, Real Part"
      n_R1_bus1n = "n:R1:bus1n", "Ohm, Positive-Sequence Impedance, Real Part"
      n_X1_bus1 = "n:X1:bus1", "Ohm, Positive-Sequence Impedance, Imaginary Part"
      n_X1_bus1n = "n:X1:bus1n", "Ohm, Positive-Sequence Impedance, Imaginary Part"
      n_Z1_bus1 = "n:Z1:bus1", "Ohm, Positive-Sequence Impedance, Magnitude"
      n_Z1_bus1n = "n:Z1:bus1n", "Ohm, Positive-Sequence Impedance, Magnitude"
      n_phiz1_bus1 = "n:phiz1:bus1", "deg, Positive-Sequence Impedance, Angle"
      n_phiz1_bus1n = "n:phiz1:bus1n", "deg, Positive-Sequence Impedance, Angle"
      n_r1Sbase_bus1 = "n:r1Sbase:bus1", "p.u., Positive-Sequence Impedance, Real Part"
      n_r1Sbase_bus1n = "n:r1Sbase:bus1n", "p.u., Positive-Sequence Impedance, Real Part"
      n_x1Sbase_bus1 = "n:x1Sbase:bus1", "p.u., Positive-Sequence Impedance, Imaginary Part"
      n_x1Sbase_bus1n = "n:x1Sbase:bus1n", "p.u., Positive-Sequence Impedance, Imaginary Part"
      n_z1Sbase_bus1 = "n:z1Sbase:bus1", "p.u., Positive-Sequence Impedance, Magnitude"
      n_z1Sbase_bus1n = "n:z1Sbase:bus1n", "p.u., Positive-Sequence Impedance, Magnitude"
      n_R2_bus1 = "n:R2:bus1", "Ohm, Negative-Sequence Impedance, Real Part"
      n_R2_bus1n = "n:R2:bus1n", "Ohm, Negative-Sequence Impedance, Real Part"
      n_X2_bus1 = "n:X2:bus1", "Ohm, Negative-Sequence Impedance, Imaginary Part"
      n_X2_bus1n = "n:X2:bus1n", "Ohm, Negative-Sequence Impedance, Imaginary Part"
      n_Z2_bus1 = "n:Z2:bus1", "Ohm, Negative-Sequence Impedance, Magnitude"
      n_Z2_bus1n = "n:Z2:bus1n", "Ohm, Negative-Sequence Impedance, Magnitude"
      n_phiz2_bus1 = "n:phiz2:bus1", "deg, Negative-Sequence Impedance, Angle"
      n_phiz2_bus1n = "n:phiz2:bus1n", "deg, Negative-Sequence Impedance, Angle"
      n_r2Sbase_bus1 = "n:r2Sbase:bus1", "p.u., Negative-Sequence Impedance, Real Part"
      n_r2Sbase_bus1n = "n:r2Sbase:bus1n", "p.u., Negative-Sequence Impedance, Real Part"
      n_x2Sbase_bus1 = "n:x2Sbase:bus1", "p.u., Negative-Sequence Impedance, Imaginary Part"
      n_x2Sbase_bus1n = "n:x2Sbase:bus1n", "p.u., Negative-Sequence Impedance, Imaginary Part"
      n_z2Sbase_bus1 = "n:z2Sbase:bus1", "p.u., Negative-Sequence Impedance, Magnitude"
      n_z2Sbase_bus1n = "n:z2Sbase:bus1n", "p.u., Negative-Sequence Impedance, Magnitude"
      n_R_bus1_A = "n:R:bus1:A", "Ohm, Network Resistance, Line-Ground"
      n_R_bus1_B = "n:R:bus1:B", "Ohm, Network Resistance, Line-Ground"
      n_R_bus1_C = "n:R:bus1:C", "Ohm, Network Resistance, Line-Ground"
      n_R_bus1_N = "n:R:bus1:N", "Ohm, Network Resistance, Line-Ground"
      n_R_bus1n = "n:R:bus1n", "Ohm, Network Resistance, Line-Ground"
      n_X_bus1_A = "n:X:bus1:A", "Ohm, Network Reactance, Line-Ground"
      n_X_bus1_B = "n:X:bus1:B", "Ohm, Network Reactance, Line-Ground"
      n_X_bus1_C = "n:X:bus1:C", "Ohm, Network Reactance, Line-Ground"
      n_X_bus1_N = "n:X:bus1:N", "Ohm, Network Reactance, Line-Ground"
      n_X_bus1n = "n:X:bus1n", "Ohm, Network Reactance, Line-Ground"
      n_Z_bus1_A = "n:Z:bus1:A", "Ohm, Network Impedance, Line-Ground"
      n_Z_bus1_B = "n:Z:bus1:B", "Ohm, Network Impedance, Line-Ground"
      n_Z_bus1_C = "n:Z:bus1:C", "Ohm, Network Impedance, Line-Ground"
      n_Z_bus1_N = "n:Z:bus1:N", "Ohm, Network Impedance, Line-Ground"
      n_Z_bus1n = "n:Z:bus1n", "Ohm, Network Impedance, Line-Ground"
      n_phiz_bus1_A = "n:phiz:bus1:A", "deg, Angle of Network Impedance, Line-Ground"
      n_phiz_bus1_B = "n:phiz:bus1:B", "deg, Angle of Network Impedance, Line-Ground"
      n_phiz_bus1_C = "n:phiz:bus1:C", "deg, Angle of Network Impedance, Line-Ground"
      n_phiz_bus1_N = "n:phiz:bus1:N", "deg, Angle of Network Impedance, Line-Ground"
      n_phiz_bus1n = "n:phiz:bus1n", "deg, Angle of Network Impedance, Line-Ground"
      n_Rl_bus1_A = "n:Rl:bus1:A", "Ohm, Network Resistance, Line-Line"
      n_Rl_bus1_B = "n:Rl:bus1:B", "Ohm, Network Resistance, Line-Line"
      n_Rl_bus1_C = "n:Rl:bus1:C", "Ohm, Network Resistance, Line-Line"
      n_Rl_bus1_N = "n:Rl:bus1:N", "Ohm, Network Resistance, Line-Line"
      n_Rl_bus1n = "n:Rl:bus1n", "Ohm, Network Resistance, Line-Line"
      n_Xl_bus1_A = "n:Xl:bus1:A", "Ohm, Network Reactance, Line-Line"
      n_Xl_bus1_B = "n:Xl:bus1:B", "Ohm, Network Reactance, Line-Line"
      n_Xl_bus1_C = "n:Xl:bus1:C", "Ohm, Network Reactance, Line-Line"
      n_Xl_bus1_N = "n:Xl:bus1:N", "Ohm, Network Reactance, Line-Line"
      n_Xl_bus1n = "n:Xl:bus1n", "Ohm, Network Reactance, Line-Line"
      n_Zl_bus1_A = "n:Zl:bus1:A", "Ohm, Network Impedance, Line-Line"
      n_Zl_bus1_B = "n:Zl:bus1:B", "Ohm, Network Impedance, Line-Line"
      n_Zl_bus1_C = "n:Zl:bus1:C", "Ohm, Network Impedance, Line-Line"
      n_Zl_bus1_N = "n:Zl:bus1:N", "Ohm, Network Impedance, Line-Line"
      n_Zl_bus1n = "n:Zl:bus1n", "Ohm, Network Impedance, Line-Line"
      n_phizl_bus1_A = "n:phizl:bus1:A", "deg, Angle of Network Impedance, Line-Line"
      n_phizl_bus1_B = "n:phizl:bus1:B", "deg, Angle of Network Impedance, Line-Line"
      n_phizl_bus1_C = "n:phizl:bus1:C", "deg, Angle of Network Impedance, Line-Line"
      n_phizl_bus1_N = "n:phizl:bus1:N", "deg, Angle of Network Impedance, Line-Line"
      n_phizl_bus1n = "n:phizl:bus1n", "deg, Angle of Network Impedance, Line-Line"
      n_Rln_bus1_A = "n:Rln:bus1:A", "Ohm, Network Resistance, Line-Neutral"
      n_Rln_bus1_B = "n:Rln:bus1:B", "Ohm, Network Resistance, Line-Neutral"
      n_Rln_bus1_C = "n:Rln:bus1:C", "Ohm, Network Resistance, Line-Neutral"
      n_Rln_bus1_N = "n:Rln:bus1:N", "Ohm, Network Resistance, Line-Neutral"
      n_Rln_bus1n = "n:Rln:bus1n", "Ohm, Network Resistance, Line-Neutral"
      n_Xln_bus1_A = "n:Xln:bus1:A", "Ohm, Network Reactance, Line-Neutral"
      n_Xln_bus1_B = "n:Xln:bus1:B", "Ohm, Network Reactance, Line-Neutral"
      n_Xln_bus1_C = "n:Xln:bus1:C", "Ohm, Network Reactance, Line-Neutral"
      n_Xln_bus1_N = "n:Xln:bus1:N", "Ohm, Network Reactance, Line-Neutral"
      n_Xln_bus1n = "n:Xln:bus1n", "Ohm, Network Reactance, Line-Neutral"
      n_Zln_bus1_A = "n:Zln:bus1:A", "Ohm, Network Impedance, Line-Neutral"
      n_Zln_bus1_B = "n:Zln:bus1:B", "Ohm, Network Impedance, Line-Neutral"
      n_Zln_bus1_C = "n:Zln:bus1:C", "Ohm, Network Impedance, Line-Neutral"
      n_Zln_bus1_N = "n:Zln:bus1:N", "Ohm, Network Impedance, Line-Neutral"
      n_Zln_bus1n = "n:Zln:bus1n", "Ohm, Network Impedance, Line-Neutral"
      n_phizln_bus1_A = "n:phizln:bus1:A", "deg, Angle of Network Impedance, Line-Neutral"
      n_phizln_bus1_B = "n:phizln:bus1:B", "deg, Angle of Network Impedance, Line-Neutral"
      n_phizln_bus1_C = "n:phizln:bus1:C", "deg, Angle of Network Impedance, Line-Neutral"
      n_phizln_bus1_N = "n:phizln:bus1:N", "deg, Angle of Network Impedance, Line-Neutral"
      n_phizln_bus1n = "n:phizln:bus1n", "deg, Angle of Network Impedance, Line-Neutral"
      n_Rn_bus1 = "n:Rn:bus1", "Ohm, Network Resistance, Neutral"
      n_Rn_bus1n = "n:Rn:bus1n", "Ohm, Network Resistance, Neutral"
      n_Xn_bus1 = "n:Xn:bus1", "Ohm, Network Reactance, Neutral"
      n_Xn_bus1n = "n:Xn:bus1n", "Ohm, Network Reactance, Neutral"
      n_Zn_bus1 = "n:Zn:bus1", "Ohm, Network Impedance, Neutral"
      n_Zn_bus1n = "n:Zn:bus1n", "Ohm, Network Impedance, Neutral"
      n_phizn_bus1 = "n:phizn:bus1", "deg, Angle of Network Impedance, Neutral"
      n_phizn_bus1n = "n:phizn:bus1n", "deg, Angle of Network Impedance, Neutral"
      n_eff_bus1_A = "n:eff:bus1:A", "Earth Fault Factor"
      n_eff_bus1_B = "n:eff:bus1:B", "Earth Fault Factor"
      n_eff_bus1_C = "n:eff:bus1:C", "Earth Fault Factor"
      n_eff_bus1_N = "n:eff:bus1:N", "Earth Fault Factor"
      n_eff_bus1n = "n:eff:bus1n", "Earth Fault Factor"
      n_X0toX1_bus1 = "n:X0toX1:bus1", "X0/X1"
      n_X0toX1_bus1n = "n:X0toX1:bus1n", "X0/X1"
      n_in_bus1 = "n:in:bus1", "p.u., Neutral Current, Magnitude"
      n_in_bus1n = "n:in:bus1n", "p.u., Neutral Current, Magnitude"
      n_In_bus1 = "n:In:bus1", "kA, Neutral Current, Magnitude"
      n_In_bus1n = "n:In:bus1n", "kA, Neutral Current, Magnitude"
      n_phiin_bus1 = "n:phiin:bus1", "deg, Neutral Current, Angle"
      n_phiin_bus1n = "n:phiin:bus1n", "deg, Neutral Current, Angle"
      n_i0r_bus1 = "n:i0r:bus1", "p.u., Zero-Sequence Current, Real Part"
      n_i0r_bus1n = "n:i0r:bus1n", "p.u., Zero-Sequence Current, Real Part"
      n_i0i_bus1 = "n:i0i:bus1", "p.u., Zero-Sequence Current, Imaginary Part"
      n_i0i_bus1n = "n:i0i:bus1n", "p.u., Zero-Sequence Current, Imaginary Part"
      n_i0_bus1 = "n:i0:bus1", "p.u., Zero-Sequence Current, Magnitude"
      n_i0_bus1n = "n:i0:bus1n", "p.u., Zero-Sequence Current, Magnitude"
      n_I0_bus1 = "n:I0:bus1", "kA, Zero-Sequence Current, Magnitude"
      n_I0_bus1n = "n:I0:bus1n", "kA, Zero-Sequence Current, Magnitude"
      n_I0x3_bus1 = "n:I0x3:bus1", "kA, 3*I0"
      n_I0x3_bus1n = "n:I0x3:bus1n", "kA, 3*I0"
      n_phii0_bus1 = "n:phii0:bus1", "deg, Zero-Sequence Current, Angle"
      n_phii0_bus1n = "n:phii0:bus1n", "deg, Zero-Sequence Current, Angle"
      n_i1r_bus1 = "n:i1r:bus1", "p.u., Positive-Sequence Current, Real Part"
      n_i1r_bus1n = "n:i1r:bus1n", "p.u., Positive-Sequence Current, Real Part"
      n_i1i_bus1 = "n:i1i:bus1", "p.u., Positive-Sequence Current, Imaginary Part"
      n_i1i_bus1n = "n:i1i:bus1n", "p.u., Positive-Sequence Current, Imaginary Part"
      n_i1_bus1 = "n:i1:bus1", "p.u., Positive-Sequence Current, Magnitude"
      n_i1_bus1n = "n:i1:bus1n", "p.u., Positive-Sequence Current, Magnitude"
      n_I1_bus1 = "n:I1:bus1", "kA, Positive-Sequence Current, Magnitude"
      n_I1_bus1n = "n:I1:bus1n", "kA, Positive-Sequence Current, Magnitude"
      n_phii1_bus1 = "n:phii1:bus1", "deg, Positive-Sequence Current, Angle"
      n_phii1_bus1n = "n:phii1:bus1n", "deg, Positive-Sequence Current, Angle"
      n_i2r_bus1 = "n:i2r:bus1", "p.u., Negative-Sequence Current, Real Part"
      n_i2r_bus1n = "n:i2r:bus1n", "p.u., Negative-Sequence Current, Real Part"
      n_i2i_bus1 = "n:i2i:bus1", "p.u., Negative-Sequence Current, Imaginary Part"
      n_i2i_bus1n = "n:i2i:bus1n", "p.u., Negative-Sequence Current, Imaginary Part"
      n_i2_bus1 = "n:i2:bus1", "p.u., Negative-Sequence Current, Magnitude"
      n_i2_bus1n = "n:i2:bus1n", "p.u., Negative-Sequence Current, Magnitude"
      n_I2_bus1 = "n:I2:bus1", "kA, Negative-Sequence Current, Magnitude"
      n_I2_bus1n = "n:I2:bus1n", "kA, Negative-Sequence Current, Magnitude"
      n_phii2_bus1 = "n:phii2:bus1", "deg, Negative-Sequence Current, Angle"
      n_phii2_bus1n = "n:phii2:bus1n", "deg, Negative-Sequence Current, Angle"
      n_du_bus1_A = "n:du:bus1:A", "%, Voltage Deviation"
      n_du_bus1_B = "n:du:bus1:B", "%, Voltage Deviation"
      n_du_bus1_C = "n:du:bus1:C", "%, Voltage Deviation"
      n_du_bus1_N = "n:du:bus1:N", "%, Voltage Deviation"
      n_du_bus1n = "n:du:bus1n", "%, Voltage Deviation"
      n_urpref_bus1_A = "n:urpref:bus1:A", "p.u., Pre-fault Voltage, Real Part"
      n_urpref_bus1_B = "n:urpref:bus1:B", "p.u., Pre-fault Voltage, Real Part"
      n_urpref_bus1_C = "n:urpref:bus1:C", "p.u., Pre-fault Voltage, Real Part"
      n_urpref_bus1_N = "n:urpref:bus1:N", "p.u., Pre-fault Voltage, Real Part"
      n_urpref_bus1n = "n:urpref:bus1n", "p.u., Pre-fault Voltage, Real Part"
      n_uipref_bus1_A = "n:uipref:bus1:A", "p.u., Pre-fault Voltage, Imaginary Part"
      n_uipref_bus1_B = "n:uipref:bus1:B", "p.u., Pre-fault Voltage, Imaginary Part"
      n_uipref_bus1_C = "n:uipref:bus1:C", "p.u., Pre-fault Voltage, Imaginary Part"
      n_uipref_bus1_N = "n:uipref:bus1:N", "p.u., Pre-fault Voltage, Imaginary Part"
      n_uipref_bus1n = "n:uipref:bus1n", "p.u., Pre-fault Voltage, Imaginary Part"
      n_upref_bus1_A = "n:upref:bus1:A", "p.u., Pre-fault Voltage, Magnitude"
      n_upref_bus1_B = "n:upref:bus1:B", "p.u., Pre-fault Voltage, Magnitude"
      n_upref_bus1_C = "n:upref:bus1:C", "p.u., Pre-fault Voltage, Magnitude"
      n_upref_bus1_N = "n:upref:bus1:N", "p.u., Pre-fault Voltage, Magnitude"
      n_upref_bus1n = "n:upref:bus1n", "p.u., Pre-fault Voltage, Magnitude"
      n_uprefpc_bus1_A = "n:uprefpc:bus1:A", "%, Pre-fault Voltage, Magnitude"
      n_uprefpc_bus1_B = "n:uprefpc:bus1:B", "%, Pre-fault Voltage, Magnitude"
      n_uprefpc_bus1_C = "n:uprefpc:bus1:C", "%, Pre-fault Voltage, Magnitude"
      n_uprefpc_bus1_N = "n:uprefpc:bus1:N", "%, Pre-fault Voltage, Magnitude"
      n_uprefpc_bus1n = "n:uprefpc:bus1n", "%, Pre-fault Voltage, Magnitude"
      n_Upref_bus1_A = "n:Upref:bus1:A", "kV, Line-Ground Pre-fault Voltage, Magnitude"
      n_Upref_bus1_B = "n:Upref:bus1:B", "kV, Line-Ground Pre-fault Voltage, Magnitude"
      n_Upref_bus1_C = "n:Upref:bus1:C", "kV, Line-Ground Pre-fault Voltage, Magnitude"
      n_Upref_bus1_N = "n:Upref:bus1:N", "kV, Line-Ground Pre-fault Voltage, Magnitude"
      n_Upref_bus1n = "n:Upref:bus1n", "kV, Line-Ground Pre-fault Voltage, Magnitude"
      n_Ulpref_bus1_A = "n:Ulpref:bus1:A", "kV, Line-Line Pre-fault Voltage, Magnitude"
      n_Ulpref_bus1_B = "n:Ulpref:bus1:B", "kV, Line-Line Pre-fault Voltage, Magnitude"
      n_Ulpref_bus1_C = "n:Ulpref:bus1:C", "kV, Line-Line Pre-fault Voltage, Magnitude"
      n_Ulpref_bus1_N = "n:Ulpref:bus1:N", "kV, Line-Line Pre-fault Voltage, Magnitude"
      n_Ulpref_bus1n = "n:Ulpref:bus1n", "kV, Line-Line Pre-fault Voltage, Magnitude"
      n_phiupref_bus1_A = "n:phiupref:bus1:A", "deg, Pre-fault Voltage, Angle"
      n_phiupref_bus1_B = "n:phiupref:bus1:B", "deg, Pre-fault Voltage, Angle"
      n_phiupref_bus1_C = "n:phiupref:bus1:C", "deg, Pre-fault Voltage, Angle"
      n_phiupref_bus1_N = "n:phiupref:bus1:N", "deg, Pre-fault Voltage, Angle"
      n_phiupref_bus1n = "n:phiupref:bus1n", "deg, Pre-fault Voltage, Angle"
      n_phiui_bus1_A = "n:phiui:bus1:A", "deg, Angle between Voltage and Current"
      n_phiui_bus1_B = "n:phiui:bus1:B", "deg, Angle between Voltage and Current"
      n_phiui_bus1_C = "n:phiui:bus1:C", "deg, Angle between Voltage and Current"
      n_phiui_bus1_N = "n:phiui:bus1:N", "deg, Angle between Voltage and Current"
      n_phiui_bus1n = "n:phiui:bus1n", "deg, Angle between Voltage and Current"
      n_phiu1i1_bus1 = "n:phiu1i1:bus1", "deg, Angle between Voltage and Current in positive sequence system"
      n_phiu1i1_bus1n = "n:phiu1i1:bus1n", "deg, Angle between Voltage and Current in positive sequence system"
      n_phiu2i2_bus1 = "n:phiu2i2:bus1", "deg, Angle between Voltage and Current in negative sequence system"
      n_phiu2i2_bus1n = "n:phiu2i2:bus1n", "deg, Angle between Voltage and Current in negative sequence system"
      n_phiu0i0_bus1 = "n:phiu0i0:bus1", "deg, Angle between Voltage and Current in zero-sequence system"
      n_phiu0i0_bus1n = "n:phiu0i0:bus1n", "deg, Angle between Voltage and Current in zero-sequence system"
      n_Ithload_bus1n = "n:Ithload:bus1n", "%, Loading, Thermal equivalent Current"
      n_Ipload_bus1n = "n:Ipload:bus1n", "%, Loading, Peak Short-Circuit Current"
      n_Ithrtk_bus1n = "n:Ithrtk:bus1n", "kA, Rated Short-Time Current (Tk)"
      n_cfac_bus1n = "n:cfac:bus1n", "Voltage Factor"
      n_XtoR_bus1n = "n:XtoR:bus1n", "X/R ratio (peak)"
      n_RtoX_bus1n = "n:RtoX:bus1n", "R/X ratio (peak)"
      c_z2_r = "c:z2:r", "p.u., Negative-sequence impedance, Real Part"
      c_z2_i = "c:z2:i", "p.u., Negative-sequence impedance, Imaginary Part"
      c_z2_phi = "c:z2:phi", "deg, Negative-sequence impedance, Angle"
      c_z2_mag = "c:z2:mag", "p.u., Negative-sequence impedance, Magnitude"
      c_z2_phirad = "c:z2:phirad", "rad, Negative-sequence impedance, Angle"
      c_z0_r = "c:z0:r", "p.u., Zero-sequence impedance, Real Part"
      c_z0_i = "c:z0:i", "p.u., Zero-sequence impedance, Imaginary Part"
      c_z0_phi = "c:z0:phi", "deg, Zero-sequence impedance, Angle"
      c_z0_mag = "c:z0:mag", "p.u., Zero-sequence impedance, Magnitude"
      c_z0_phirad = "c:z0:phirad", "rad, Zero-sequence impedance, Angle"

    class ElmIac(Enum):
      _init_ = 'value __doc__'

      m_u0_bus1 = "m:u0:bus1", "p.u., Zero-Sequence-Voltage, Magnitude"
      m_phiu0_bus1 = "m:phiu0:bus1", "deg, Zero-Sequence-Voltage, Angle"
      m_u1r_bus1 = "m:u1r:bus1", "p.u., Positive-Sequence-Voltage, Real Part"
      m_u1i_bus1 = "m:u1i:bus1", "p.u., Positive-Sequence-Voltage, Imaginary Part"
      m_u1_bus1 = "m:u1:bus1", "p.u., Positive-Sequence-Voltage, Magnitude"
      m_U1_bus1 = "m:U1:bus1", "kV, Line-Ground Positive-Sequence-Voltage, Magnitude"
      m_U1l_bus1 = "m:U1l:bus1", "kV, Line-Line Positive-Sequence-Voltage, Magnitude"
      m_phiu1_bus1 = "m:phiu1:bus1", "deg, Positive-Sequence-Voltage, Angle"
      m_u2_bus1 = "m:u2:bus1", "p.u., Negative-Sequence-Voltage, Magnitude"
      m_phiu2_bus1 = "m:phiu2:bus1", "deg, Negative-Sequence-Voltage, Angle"
      m_inet_bus1_A = "m:inet:bus1:A", "p.u., Phase Current, Magnitude, referred to network"
      m_inet_bus1_B = "m:inet:bus1:B", "p.u., Phase Current, Magnitude, referred to network"
      m_inet_bus1_C = "m:inet:bus1:C", "p.u., Phase Current, Magnitude, referred to network"
      m_I_bus1_A = "m:I:bus1:A", "kA, Phase Current, Magnitude"
      m_I_bus1_B = "m:I:bus1:B", "kA, Phase Current, Magnitude"
      m_I_bus1_C = "m:I:bus1:C", "kA, Phase Current, Magnitude"
      m_phii_bus1_A = "m:phii:bus1:A", "deg, Phase Current, Angle"
      m_phii_bus1_B = "m:phii:bus1:B", "deg, Phase Current, Angle"
      m_phii_bus1_C = "m:phii:bus1:C", "deg, Phase Current, Angle"
      m_i0r_bus1 = "m:i0r:bus1", "p.u., Zero-Sequence Current, Real Part"
      m_i0i_bus1 = "m:i0i:bus1", "p.u., Zero-Sequence Current, Imaginary Part"
      m_i0_bus1 = "m:i0:bus1", "p.u., Zero-Sequence Current, Magnitude"
      m_I0_bus1 = "m:I0:bus1", "kA, Zero-Sequence Current, Magnitude"
      m_I0x3_bus1 = "m:I0x3:bus1", "kA, 3*I0"
      m_phii0_bus1 = "m:phii0:bus1", "deg, Zero-Sequence Current, Angle"
      m_i1r_bus1 = "m:i1r:bus1", "p.u., Positive-Sequence Current, Real Part"
      m_i1i_bus1 = "m:i1i:bus1", "p.u., Positive-Sequence Current, Imaginary Part"
      m_i1_bus1 = "m:i1:bus1", "p.u., Positive-Sequence Current, Magnitude"
      m_I1_bus1 = "m:I1:bus1", "kA, Positive-Sequence Current, Magnitude"
      m_phii1_bus1 = "m:phii1:bus1", "deg, Positive-Sequence Current, Angle"
      m_i2r_bus1 = "m:i2r:bus1", "p.u., Negative-Sequence Current, Real Part"
      m_i2i_bus1 = "m:i2i:bus1", "p.u., Negative-Sequence Current, Imaginary Part"
      m_i2_bus1 = "m:i2:bus1", "p.u., Negative-Sequence Current, Magnitude"
      m_I2_bus1 = "m:I2:bus1", "kA, Negative-Sequence Current, Magnitude"
      m_phii2_bus1 = "m:phii2:bus1", "deg, Negative-Sequence Current, Angle"
      m_ubfacI_bus1 = "m:ubfacI:bus1", "%, Current Unbalance Factor"
      m_ubfacS_bus1 = "m:ubfacS:bus1", "%, Power Unbalance Factor"
      m_P_bus1_A = "m:P:bus1:A", "MW, Active Power"
      m_P_bus1_B = "m:P:bus1:B", "MW, Active Power"
      m_P_bus1_C = "m:P:bus1:C", "MW, Active Power"
      m_Q_bus1_A = "m:Q:bus1:A", "Mvar, Reactive Power"
      m_Q_bus1_B = "m:Q:bus1:B", "Mvar, Reactive Power"
      m_Q_bus1_C = "m:Q:bus1:C", "Mvar, Reactive Power"
      m_S_bus1_A = "m:S:bus1:A", "MVA, Apparent Power"
      m_S_bus1_B = "m:S:bus1:B", "MVA, Apparent Power"
      m_S_bus1_C = "m:S:bus1:C", "MVA, Apparent Power"
      m_cosphi_bus1_A = "m:cosphi:bus1:A", "Power Factor"
      m_cosphi_bus1_B = "m:cosphi:bus1:B", "Power Factor"
      m_cosphi_bus1_C = "m:cosphi:bus1:C", "Power Factor"
      m_tanphi_bus1_A = "m:tanphi:bus1:A", "tan(phi)"
      m_tanphi_bus1_B = "m:tanphi:bus1:B", "tan(phi)"
      m_tanphi_bus1_C = "m:tanphi:bus1:C", "tan(phi)"
      m_Psum_bus1 = "m:Psum:bus1", "MW, Total Active Power"
      m_Qsum_bus1 = "m:Qsum:bus1", "Mvar, Total Reactive Power"
      m_Ssum_bus1 = "m:Ssum:bus1", "MVA, Total Apparent Power"
      m_cosphisum_bus1 = "m:cosphisum:bus1", "Total Power Factor"
      m_tanphisum_bus1 = "m:tanphisum:bus1", "Total tan(phi)"
      m_Pln_bus1_A = "m:Pln:bus1:A", "MW, Active Power (Line-Neutral)"
      m_Pln_bus1_B = "m:Pln:bus1:B", "MW, Active Power (Line-Neutral)"
      m_Pln_bus1_C = "m:Pln:bus1:C", "MW, Active Power (Line-Neutral)"
      m_Qln_bus1_A = "m:Qln:bus1:A", "Mvar, Reactive Power (Line-Neutral)"
      m_Qln_bus1_B = "m:Qln:bus1:B", "Mvar, Reactive Power (Line-Neutral)"
      m_Qln_bus1_C = "m:Qln:bus1:C", "Mvar, Reactive Power (Line-Neutral)"
      m_P1_bus1 = "m:P1:bus1", "MW, Positive Sequence Active Power"
      m_Q1_bus1 = "m:Q1:bus1", "Mvar, Positive Sequence Reactive Power"
      m_P2_bus1 = "m:P2:bus1", "MW, Negative Sequence Active Power"
      m_Q2_bus1 = "m:Q2:bus1", "Mvar, Negative Sequence Reactive Power"
      m_P0_bus1 = "m:P0:bus1", "MW, Zero Sequence Active Power"
      m_Q0_bus1 = "m:Q0:bus1", "Mvar, Zero Sequence Reactive Power"
      m_i1P_bus1 = "m:i1P:bus1", "p.u., Positive-Sequence Active Current"
      m_i1Q_bus1 = "m:i1Q:bus1", "p.u., Positive-Sequence Reactive Current"
      m_I1P_bus1 = "m:I1P:bus1", "kA, Positive-Sequence Active Current"
      m_I1Q_bus1 = "m:I1Q:bus1", "kA, Positive-Sequence Reactive Current"
      m_i2P_bus1 = "m:i2P:bus1", "p.u., Negative-Sequence Active Current"
      m_i2Q_bus1 = "m:i2Q:bus1", "p.u., Negative-Sequence Reactive Current"
      m_I2P_bus1 = "m:I2P:bus1", "kA, Negative-Sequence Active Current"
      m_I2Q_bus1 = "m:I2Q:bus1", "kA, Negative-Sequence Reactive Current"
      m_ur_bus1_A = "m:ur:bus1:A", "p.u., Phase Voltage, Real Part"
      m_ur_bus1_B = "m:ur:bus1:B", "p.u., Phase Voltage, Real Part"
      m_ur_bus1_C = "m:ur:bus1:C", "p.u., Phase Voltage, Real Part"
      m_ui_bus1_A = "m:ui:bus1:A", "p.u., Phase Voltage, Imaginary Part"
      m_ui_bus1_B = "m:ui:bus1:B", "p.u., Phase Voltage, Imaginary Part"
      m_ui_bus1_C = "m:ui:bus1:C", "p.u., Phase Voltage, Imaginary Part"
      m_u_bus1_A = "m:u:bus1:A", "p.u., Phase Voltage, Magnitude"
      m_u_bus1_B = "m:u:bus1:B", "p.u., Phase Voltage, Magnitude"
      m_u_bus1_C = "m:u:bus1:C", "p.u., Phase Voltage, Magnitude"
      m_ir_bus1_A = "m:ir:bus1:A", "p.u., Phase Current, Real Part"
      m_ir_bus1_B = "m:ir:bus1:B", "p.u., Phase Current, Real Part"
      m_ir_bus1_C = "m:ir:bus1:C", "p.u., Phase Current, Real Part"
      m_ii_bus1_A = "m:ii:bus1:A", "p.u., Phase Current, Imaginary Part"
      m_ii_bus1_B = "m:ii:bus1:B", "p.u., Phase Current, Imaginary Part"
      m_ii_bus1_C = "m:ii:bus1:C", "p.u., Phase Current, Imaginary Part"
      m_i_bus1_A = "m:i:bus1:A", "p.u., Phase Current, Magnitude"
      m_i_bus1_B = "m:i:bus1:B", "p.u., Phase Current, Magnitude"
      m_i_bus1_C = "m:i:bus1:C", "p.u., Phase Current, Magnitude"
      m_phiui_bus1_A = "m:phiui:bus1:A", "deg, Angle between Voltage and Current"
      m_phiui_bus1_B = "m:phiui:bus1:B", "deg, Angle between Voltage and Current"
      m_phiui_bus1_C = "m:phiui:bus1:C", "deg, Angle between Voltage and Current"
      m_phiu0i0_bus1 = "m:phiu0i0:bus1", "deg, Angle between Voltage and Current in zero-sequence system"
      m_phiu1i1_bus1 = "m:phiu1i1:bus1", "deg, Angle between Voltage and Current in positive sequence system"
      m_phiu2i2_bus1 = "m:phiu2i2:bus1", "deg, Angle between Voltage and Current in negative sequence system"
      m_Tfct_bus1 = "m:Tfct:bus1", "s, Fault Clearing Time"
      m_TfctPh_bus1_A = "m:TfctPh:bus1:A", "s, Fault Clearing Time"
      m_TfctPh_bus1_B = "m:TfctPh:bus1:B", "s, Fault Clearing Time"
      m_TfctPh_bus1_C = "m:TfctPh:bus1:C", "s, Fault Clearing Time"
      m_Brkload_bus1 = "m:Brkload:bus1", "%, Max. Breaker Loading"
      m_BrkloadPh_bus1_A = "m:BrkloadPh:bus1:A", "%, Breaker Loading"
      m_BrkloadPh_bus1_B = "m:BrkloadPh:bus1:B", "%, Breaker Loading"
      m_BrkloadPh_bus1_C = "m:BrkloadPh:bus1:C", "%, Breaker Loading"
      n_ur_bus1_A = "n:ur:bus1:A", "p.u., Line-Ground Voltage, Real Part"
      n_ur_bus1_B = "n:ur:bus1:B", "p.u., Line-Ground Voltage, Real Part"
      n_ur_bus1_C = "n:ur:bus1:C", "p.u., Line-Ground Voltage, Real Part"
      n_ui_bus1_A = "n:ui:bus1:A", "p.u., Line-Ground Voltage, Imaginary Part"
      n_ui_bus1_B = "n:ui:bus1:B", "p.u., Line-Ground Voltage, Imaginary Part"
      n_ui_bus1_C = "n:ui:bus1:C", "p.u., Line-Ground Voltage, Imaginary Part"
      n_u_bus1_A = "n:u:bus1:A", "p.u., Line-Ground Voltage, Magnitude"
      n_u_bus1_B = "n:u:bus1:B", "p.u., Line-Ground Voltage, Magnitude"
      n_u_bus1_C = "n:u:bus1:C", "p.u., Line-Ground Voltage, Magnitude"
      n_upc_bus1_A = "n:upc:bus1:A", "%, Line-Ground Voltage, Magnitude"
      n_upc_bus1_B = "n:upc:bus1:B", "%, Line-Ground Voltage, Magnitude"
      n_upc_bus1_C = "n:upc:bus1:C", "%, Line-Ground Voltage, Magnitude"
      n_U_bus1_A = "n:U:bus1:A", "kV, Line-Ground Voltage, Magnitude"
      n_U_bus1_B = "n:U:bus1:B", "kV, Line-Ground Voltage, Magnitude"
      n_U_bus1_C = "n:U:bus1:C", "kV, Line-Ground Voltage, Magnitude"
      n_phiu_bus1_A = "n:phiu:bus1:A", "deg, Line-Ground Voltage, Angle"
      n_phiu_bus1_B = "n:phiu:bus1:B", "deg, Line-Ground Voltage, Angle"
      n_phiu_bus1_C = "n:phiu:bus1:C", "deg, Line-Ground Voltage, Angle"
      n_phiurel_bus1 = "n:phiurel:bus1", "deg, Voltage, Relative Angle"
      n_ul_bus1_A = "n:ul:bus1:A", "p.u., Line-Line Voltage, Magnitude"
      n_ul_bus1_B = "n:ul:bus1:B", "p.u., Line-Line Voltage, Magnitude"
      n_ul_bus1_C = "n:ul:bus1:C", "p.u., Line-Line Voltage, Magnitude"
      n_ulpc_bus1_A = "n:ulpc:bus1:A", "%, Line-Line Voltage, Magnitude"
      n_ulpc_bus1_B = "n:ulpc:bus1:B", "%, Line-Line Voltage, Magnitude"
      n_ulpc_bus1_C = "n:ulpc:bus1:C", "%, Line-Line Voltage, Magnitude"
      n_Ul_bus1_A = "n:Ul:bus1:A", "kV, Line-Line Voltage, Magnitude"
      n_Ul_bus1_B = "n:Ul:bus1:B", "kV, Line-Line Voltage, Magnitude"
      n_Ul_bus1_C = "n:Ul:bus1:C", "kV, Line-Line Voltage, Magnitude"
      n_phiul_bus1_A = "n:phiul:bus1:A", "deg, Line-Line Voltage, Angle"
      n_phiul_bus1_B = "n:phiul:bus1:B", "deg, Line-Line Voltage, Angle"
      n_phiul_bus1_C = "n:phiul:bus1:C", "deg, Line-Line Voltage, Angle"
      n_u0_bus1 = "n:u0:bus1", "p.u., Zero-Sequence Voltage, Magnitude"
      n_U0_bus1 = "n:U0:bus1", "kV, Zero-Sequence Voltage, Magnitude"
      n_U0x3_bus1 = "n:U0x3:bus1", "kV, 3*U0"
      n_phiu0_bus1 = "n:phiu0:bus1", "deg, Zero-Sequence Voltage, Angle"
      n_u1_bus1 = "n:u1:bus1", "p.u., Positive-Sequence Voltage, Magnitude"
      n_u1pc_bus1 = "n:u1pc:bus1", "%, Positive-Sequence Voltage, Magnitude"
      n_u1r_bus1 = "n:u1r:bus1", "p.u., Positive-Sequence Voltage, Real Part"
      n_u1i_bus1 = "n:u1i:bus1", "p.u., Positive-Sequence Voltage, Imaginary Part"
      n_U1_bus1 = "n:U1:bus1", "kV, Line-Ground Positive-Sequence Voltage, Magnitude"
      n_phiu1_bus1 = "n:phiu1:bus1", "deg, Positive-Sequence Voltage, Angle"
      n_u2_bus1 = "n:u2:bus1", "p.u., Negative-Sequence Voltage, Magnitude"
      n_U2_bus1 = "n:U2:bus1", "kV, Line-Ground Negative-Sequence Voltage, Magnitude"
      n_phiu2_bus1 = "n:phiu2:bus1", "deg, Negative-Sequence Voltage, Angle"
      n_U1l_bus1 = "n:U1l:bus1", "kV, Line-Line Positive-Sequence Voltage, Magnitude"
      n_U2l_bus1 = "n:U2l:bus1", "kV, Line-Line Negative-Sequence Voltage, Magnitude"
      n_uln_bus1_A = "n:uln:bus1:A", "p.u., Line-Neutral Voltage, Magnitude"
      n_uln_bus1_B = "n:uln:bus1:B", "p.u., Line-Neutral Voltage, Magnitude"
      n_uln_bus1_C = "n:uln:bus1:C", "p.u., Line-Neutral Voltage, Magnitude"
      n_Uln_bus1_A = "n:Uln:bus1:A", "kV, Line-Neutral Voltage, Magnitude"
      n_Uln_bus1_B = "n:Uln:bus1:B", "kV, Line-Neutral Voltage, Magnitude"
      n_Uln_bus1_C = "n:Uln:bus1:C", "kV, Line-Neutral Voltage, Magnitude"
      n_phiuln_bus1_A = "n:phiuln:bus1:A", "deg, Line-Neutral Voltage, Angle"
      n_phiuln_bus1_B = "n:phiuln:bus1:B", "deg, Line-Neutral Voltage, Angle"
      n_phiuln_bus1_C = "n:phiuln:bus1:C", "deg, Line-Neutral Voltage, Angle"
      n_un_bus1 = "n:un:bus1", "p.u., Neutral-Ground Voltage, Magnitude"
      n_Un_bus1 = "n:Un:bus1", "kV, Neutral-Ground Voltage, Magnitude"
      n_phiun_bus1 = "n:phiun:bus1", "deg, Neutral-Ground Voltage, Angle"
      n_upht_bus1_A = "n:upht:bus1:A", "p.u., Phase Technology dependent Voltage, Magnitude"
      n_upht_bus1_B = "n:upht:bus1:B", "p.u., Phase Technology dependent Voltage, Magnitude"
      n_upht_bus1_C = "n:upht:bus1:C", "p.u., Phase Technology dependent Voltage, Magnitude"
      n_Upht_bus1_A = "n:Upht:bus1:A", "kV, Phase Technology dependent Voltage, Magnitude"
      n_Upht_bus1_B = "n:Upht:bus1:B", "kV, Phase Technology dependent Voltage, Magnitude"
      n_Upht_bus1_C = "n:Upht:bus1:C", "kV, Phase Technology dependent Voltage, Magnitude"
      n_um_bus1 = "n:um:bus1", "p.u., Average-Voltage, Magnitude"
      n_Um_bus1 = "n:Um:bus1", "kV, Average-Voltage, Magnitude"
      n_uphtmin_bus1 = "n:uphtmin:bus1", "p.u., Minimum of Phase Technology dependent Voltage, Magnitude"
      n_uphtmax_bus1 = "n:uphtmax:bus1", "p.u., Maximum of Phase Technology dependent Voltage, Magnitude"
      n_Pgen_bus1 = "n:Pgen:bus1", "MW, Generation, Active Power"
      n_Qgen_bus1 = "n:Qgen:bus1", "Mvar, Generation, Reactive Power"
      n_Pmot_bus1 = "n:Pmot:bus1", "MW, Motor Load, Active Power"
      n_Qmot_bus1 = "n:Qmot:bus1", "Mvar, Motor Load, Reactive Power"
      n_Pload_bus1 = "n:Pload:bus1", "MW, General Load, Active Power"
      n_Qload_bus1 = "n:Qload:bus1", "Mvar, General Load, Reactive Power"
      n_Pcomp_bus1 = "n:Pcomp:bus1", "MW, Compensation (Losses)"
      n_Qcomp_bus1 = "n:Qcomp:bus1", "Mvar, Compensation"
      n_Pnet_bus1 = "n:Pnet:bus1", "MW, External Networks, Active Power"
      n_Qnet_bus1 = "n:Qnet:bus1", "Mvar, External Networks, Reactive Power"
      n_Pflow_bus1 = "n:Pflow:bus1", "MW, Power Flow, Active Power"
      n_Qflow_bus1 = "n:Qflow:bus1", "Mvar, Power Flow, Reactive Power"
      n_Pout_bus1 = "n:Pout:bus1", "MW, Outgoing Flow, Active Power"
      n_Qout_bus1 = "n:Qout:bus1", "Mvar, Outgoing Power, Reactive Power"
      n_Sout_bus1 = "n:Sout:bus1", "MVA, Outgoing Power, Apparent Power"
      n_cosphiout_bus1 = "n:cosphiout:bus1", "Outgoing Power, Power Factor"
      n_Pbalance_bus1 = "n:Pbalance:bus1", "MW, Active Power Balance (=0)"
      n_Qbalance_bus1 = "n:Qbalance:bus1", "Mvar, Reactive Power Balance (=0)"
      n_ubfac_bus1 = "n:ubfac:bus1", "%, Unbalance Factor"
      n_LossPdown_bus1 = "n:LossPdown:bus1", "MW, Losses, downstream"
      n_LossQdown_bus1 = "n:LossQdown:bus1", "Mvar, Losses, downstream (Reactive Power)"
      n_LossPdownload_bus1 = "n:LossPdownload:bus1", "MW, Load losses, downstream"
      n_LossQdownload_bus1 = "n:LossQdownload:bus1", "Mvar, Load losses, downstream"
      n_LossPdownnoload_bus1 = "n:LossPdownnoload:bus1", "MW, No load losses, downstream"
      n_LossQdownnoload_bus1 = "n:LossQdownnoload:bus1", "Mvar, No load losses, downstream"
      s_i0 = "s:i0", "p.u., IN, Current-Input, continuous, double"
      s_phiui = "s:phiui", "rad, IN, Angle-Input (u,i), continuous, double"
      c_Ir = "c:Ir", "A, Nominal Current"
      c_isetp = "c:isetp", "p.u., Current Setpoint"
      c_cosini = "c:cosini", "Power Factor"
      c_Y0_r = "c:Y0:r", "S, Internal Admittance, Real Part"
      c_Y0_i = "c:Y0:i", "S, Internal Admittance, Imaginary Part"
      c_Y0_phi = "c:Y0:phi", "deg, Internal Admittance, Angle"
      c_Y0_mag = "c:Y0:mag", "S, Internal Admittance, Magnitude"
      c_Y0_phirad = "c:Y0:phirad", "rad, Internal Admittance, Angle"
      c_Y1_r = "c:Y1:r", "S, Internal Admittance, Real Part"
      c_Y1_i = "c:Y1:i", "S, Internal Admittance, Imaginary Part"
      c_Y1_phi = "c:Y1:phi", "deg, Internal Admittance, Angle"
      c_Y1_mag = "c:Y1:mag", "S, Internal Admittance, Magnitude"
      c_Y1_phirad = "c:Y1:phirad", "rad, Internal Admittance, Angle"
      c_Y2_r = "c:Y2:r", "S, Internal Admittance, Real Part"
      c_Y2_i = "c:Y2:i", "S, Internal Admittance, Imaginary Part"
      c_Y2_phi = "c:Y2:phi", "deg, Internal Admittance, Angle"
      c_Y2_mag = "c:Y2:mag", "S, Internal Admittance, Magnitude"
      c_Y2_phirad = "c:Y2:phirad", "rad, Internal Admittance, Angle"
      c_Ploss = "c:Ploss", "MW, Losses (total)"
      c_Qloss = "c:Qloss", "Mvar, Reactive-Losses (total)"
      c_Plossld = "c:Plossld", "MW, Losses (load)"
      c_Qlossld = "c:Qlossld", "Mvar, Reactive-Losses (load)"
      c_Plossnld = "c:Plossnld", "MW, Losses (no load)"
      c_Qlossnld = "c:Qlossnld", "Mvar, Reactive-Losses (no load)"
      e_isetp = "e:isetp", "p.u., Current Setpoint"
      e_cosini = "e:cosini", "Power Factor"
      e_i_cap = "e:i_cap", "Reactive Power"
      e_G1 = "e:G1", "S, Internal Conductance, G1"
      e_B1 = "e:B1", "S, Internal Susceptance, B1"
      e_isetp2 = "e:isetp2", "p.u., Current, Magnitude"
      e_phisetp2 = "e:phisetp2", "deg, Current, Angle"
      e_G2 = "e:G2", "S, Internal Conductance, G2"
      e_B2 = "e:B2", "S, Internal Susceptance, B2"
      e_isetp0 = "e:isetp0", "p.u., Current, Magnitude"
      e_phisetp0 = "e:phisetp0", "deg, Current, Angle"
      e_G0 = "e:G0", "S, Internal Conductance, G0"
      e_B0 = "e:B0", "S, Internal Susceptance, B0"
      e_qdslCtrl = "e:qdslCtrl", "Quasi-Dynamic Model"
      e_cpFeed = "e:cpFeed", "Feeder"
      e_ciDist = "e:ciDist", "Distance from infeed in number of buses"
      e_ciLater = "e:ciLater", "Lateral Index"
      e_ciDistRoot = "e:ciDistRoot", "Distance from first infeed in number of buses"
      e_ciDistAll = "e:ciDistAll", "Distance from infeed in number of buses including switches"
      e_ciDistAllRoot = "e:ciDistAllRoot", "Distance from first infeed in number of buses including switches"
      r_qdslCtrl = "r:qdslCtrl", "Quasi-Dynamic Model"
      r_cpFeed = "r:cpFeed", "Feeder"
      m_Ikss_bus1 = "m:Ikss:bus1", "kA, Initial Short-Circuit Current"
      m_ikss_bus1 = "m:ikss:bus1", "p.u., Initial Short-Circuit Current, Magnitude"
      m_phii_bus1 = "m:phii:bus1", "deg, Phase Current, Angle"
      m_maxIkss_bus1 = "m:maxIkss:bus1", "kA, Max. Initial Short-Circuit Current"
      m_Skss_bus1 = "m:Skss:bus1", "MVA, Initial Short-Circuit Power"
      m_ip_bus1 = "m:ip:bus1", "kA, Peak Short-Circuit Current"
      m_maxip_bus1 = "m:maxip:bus1", "kA, Max. Peak Short-Circuit Current"
      m_maxIb_bus1 = "m:maxIb:bus1", "kA, Max. Breaking Current"
      m_maxIth_bus1 = "m:maxIth:bus1", "kA, Max. Thermal equivalent Short-Circuit Current"
      m_Ib_bus1 = "m:Ib:bus1", "kA, Short-Circuit Breaking Current"
      m_Sb_bus1 = "m:Sb:bus1", "MVA, Short-Circuit Breaking Power"
      m_Ik_bus1 = "m:Ik:bus1", "kA, Steady-State Short-Circuit Current"
      m_Ith_bus1 = "m:Ith:bus1", "kA, Thermal equivalent Short-Circuit Current"
      m_I_bus1 = "m:I:bus1", "kA, Short-Circuit Current"
      m_R_bus1 = "m:R:bus1", "Ohm, Fault Impedance, Real Part"
      m_X_bus1 = "m:X:bus1", "Ohm, Fault Impedance, Imaginary Part"
      m_Z_bus1 = "m:Z:bus1", "Ohm, Fault Impedance, Magnitude"
      m_phiz_bus1 = "m:phiz:bus1", "deg, Fault Impedance, Angle"
      m_zSbase_bus1 = "m:zSbase:bus1", "p.u., Fault Impedance, Magnitude"
      m_XtoR_bus1 = "m:XtoR:bus1", "X/R ratio"
      m_Zs_bus1 = "m:Zs:bus1", "Ohm, Source Impedance, Magnitude"
      m_phizs_bus1 = "m:phizs:bus1", "deg, Source Impedance, Angle"
      m_Rs_bus1 = "m:Rs:bus1", "Ohm, Source Impedance, Real Part"
      m_Xs_bus1 = "m:Xs:bus1", "Ohm, Source Impedance, Imaginary Part"
      m_phiui_bus1 = "m:phiui:bus1", "deg, Angle between Voltage and Current"
      m_cosphiui_bus1 = "m:cosphiui:bus1", "cosphi"
      m_P_bus1 = "m:P:bus1", "MW, Active Power"
      m_Q_bus1 = "m:Q:bus1", "Mvar, Reactive Power"
      m_BrkIpload_bus1 = "m:BrkIpload:bus1", "%, Breaker Loading (peak)"
      m_BrkIbload_bus1 = "m:BrkIbload:bus1", "%, Breaker Loading (breaking)"
      m_BrkmaxIb_bus1 = "m:BrkmaxIb:bus1", "kA, Max. Breaking Current (breaker time)"
      n_Ikss_bus1 = "n:Ikss:bus1", "kA, Initial Short-Circuit Current"
      n_I_bus1 = "n:I:bus1", "kA, Short-Circuit Current"
      n_phii_bus1 = "n:phii:bus1", "deg, Phase Current, Angle"
      n_Skss_bus1 = "n:Skss:bus1", "MVA, Initial Short-Circuit Power"
      n_ip_bus1 = "n:ip:bus1", "kA, Peak Short-Circuit Current"
      n_Ib_bus1 = "n:Ib:bus1", "kA, Short-Circuit Breaking Current"
      n_Sb_bus1 = "n:Sb:bus1", "MVA, Short-Circuit Breaking Power"
      n_Ik_bus1 = "n:Ik:bus1", "kA, Steady-State Short-Circuit Current"
      n_Ith_bus1 = "n:Ith:bus1", "kA, Thermal equivalent Short-Circuit Current"
      n_R_bus1 = "n:R:bus1", "Ohm, Short-Circuit Impedance, Real Part"
      n_X_bus1 = "n:X:bus1", "Ohm, Short-Circuit Impedance, Imaginary Part"
      n_Z_bus1 = "n:Z:bus1", "Ohm, Short-Circuit Impedance, Magnitude"
      n_phiz_bus1 = "n:phiz:bus1", "deg, Short-Circuit Impedance, Angle"
      n_rSbase_bus1 = "n:rSbase:bus1", "p.u., Short-Circuit Impedance, Real Part"
      n_xSbase_bus1 = "n:xSbase:bus1", "p.u., Short-Circuit Impedance, Imaginary Part"
      n_zSbase_bus1 = "n:zSbase:bus1", "p.u., Short-Circuit Impedance, Magnitude"
      n_XtoR_bus1 = "n:XtoR:bus1", "X/R ratio (peak)"
      n_RtoX_bus1 = "n:RtoX:bus1", "R/X ratio (peak)"
      n_ur_bus1 = "n:ur:bus1", "p.u., Voltage, Real Part"
      n_ui_bus1 = "n:ui:bus1", "p.u., Voltage, Imaginary Part"
      n_u_bus1 = "n:u:bus1", "p.u., Voltage, Magnitude"
      n_upc_bus1 = "n:upc:bus1", "%, Voltage, Magnitude"
      n_U_bus1 = "n:U:bus1", "kV, Line-Ground Voltage, Magnitude"
      n_Ul_bus1 = "n:Ul:bus1", "kV, Line-Line Voltage, Magnitude"
      n_phiu_bus1 = "n:phiu:bus1", "deg, Voltage, Angle"
      n_du_bus1 = "n:du:bus1", "%, Voltage Deviation"
      n_urpref_bus1 = "n:urpref:bus1", "p.u., Pre-fault Voltage, Real Part"
      n_uipref_bus1 = "n:uipref:bus1", "p.u., Pre-fault Voltage, Imaginary Part"
      n_upref_bus1 = "n:upref:bus1", "p.u., Pre-fault Voltage, Magnitude"
      n_uprefpc_bus1 = "n:uprefpc:bus1", "%, Pre-fault Voltage, Magnitude"
      n_Upref_bus1 = "n:Upref:bus1", "kV, Line-Ground Pre-fault Voltage, Magnitude"
      n_Ulpref_bus1 = "n:Ulpref:bus1", "kV, Line-Line Pre-fault Voltage, Magnitude"
      n_phiupref_bus1 = "n:phiupref:bus1", "deg, Pre-fault Voltage, Angle"
      n_phiui_bus1 = "n:phiui:bus1", "deg, Angle between Voltage and Current"
      n_cfac_bus1 = "n:cfac:bus1", "Voltage Factor"
      n_idc_bus1 = "n:idc:bus1", "kA, Decaying Component"
      n_Iasy_bus1 = "n:Iasy:bus1", "kA, Asymmetrical Breaking Current"
      n_Ithload_bus1 = "n:Ithload:bus1", "%, Loading, Thermal equivalent Current"
      n_Ipload_bus1 = "n:Ipload:bus1", "%, Loading, Peak Short-Circuit Current"
      n_Ithrtk_bus1 = "n:Ithrtk:bus1", "kA, Rated Short-Time Current (Tk)"
      n_Tn_bus1 = "n:Tn:bus1", "ms, Grid Time Constant"
      n_XtoR_b_bus1 = "n:XtoR_b:bus1", "X/R ratio (dc)"
      n_RtoX_b_bus1 = "n:RtoX_b:bus1", "R/X ratio (dc)"
      n_fctof_b_bus1 = "n:fctof_b:bus1", "Equivalent Frequency fc/f (dc)"
      m_Ikss_bus1_A = "m:Ikss:bus1:A", "kA, Initial Short-Circuit Current"
      m_Ikss_bus1_B = "m:Ikss:bus1:B", "kA, Initial Short-Circuit Current"
      m_Ikss_bus1_C = "m:Ikss:bus1:C", "kA, Initial Short-Circuit Current"
      m_ikss_bus1_A = "m:ikss:bus1:A", "p.u., Initial Short-Circuit Current, Magnitude"
      m_ikss_bus1_B = "m:ikss:bus1:B", "p.u., Initial Short-Circuit Current, Magnitude"
      m_ikss_bus1_C = "m:ikss:bus1:C", "p.u., Initial Short-Circuit Current, Magnitude"
      m_Skss_bus1_A = "m:Skss:bus1:A", "MVA, Initial Short-Circuit Power"
      m_Skss_bus1_B = "m:Skss:bus1:B", "MVA, Initial Short-Circuit Power"
      m_Skss_bus1_C = "m:Skss:bus1:C", "MVA, Initial Short-Circuit Power"
      m_Skssx3_bus1_A = "m:Skssx3:bus1:A", "MVA, 3*Initial Short-Circuit Power"
      m_Skssx3_bus1_B = "m:Skssx3:bus1:B", "MVA, 3*Initial Short-Circuit Power"
      m_Skssx3_bus1_C = "m:Skssx3:bus1:C", "MVA, 3*Initial Short-Circuit Power"
      m_ip_bus1_A = "m:ip:bus1:A", "kA, Peak Short-Circuit Current"
      m_ip_bus1_B = "m:ip:bus1:B", "kA, Peak Short-Circuit Current"
      m_ip_bus1_C = "m:ip:bus1:C", "kA, Peak Short-Circuit Current"
      m_Z_bus1_A = "m:Z:bus1:A", "Ohm, Fault Impedance, Magnitude"
      m_Z_bus1_B = "m:Z:bus1:B", "Ohm, Fault Impedance, Magnitude"
      m_Z_bus1_C = "m:Z:bus1:C", "Ohm, Fault Impedance, Magnitude"
      m_phiz_bus1_A = "m:phiz:bus1:A", "deg, Fault Impedance, Angle"
      m_phiz_bus1_B = "m:phiz:bus1:B", "deg, Fault Impedance, Angle"
      m_phiz_bus1_C = "m:phiz:bus1:C", "deg, Fault Impedance, Angle"
      m_Zl_bus1_A = "m:Zl:bus1:A", "Ohm, Line-Line Fault Impedance, Magnitude"
      m_Zl_bus1_B = "m:Zl:bus1:B", "Ohm, Line-Line Fault Impedance, Magnitude"
      m_Zl_bus1_C = "m:Zl:bus1:C", "Ohm, Line-Line Fault Impedance, Magnitude"
      m_phizl_bus1_A = "m:phizl:bus1:A", "deg, Line-Line Fault Impedance, Angle"
      m_phizl_bus1_B = "m:phizl:bus1:B", "deg, Line-Line Fault Impedance, Angle"
      m_phizl_bus1_C = "m:phizl:bus1:C", "deg, Line-Line Fault Impedance, Angle"
      m_Z0_bus1 = "m:Z0:bus1", "Ohm, Zero-Sequence Fault Impedance, Magnitude"
      m_phiz0_bus1 = "m:phiz0:bus1", "deg, Zero-Sequence Fault Impedance, Angle"
      m_z0Sbase_bus1 = "m:z0Sbase:bus1", "p.u., Zero-Sequence Fault Impedance, Magnitude"
      m_cosphiu1i1_bus1 = "m:cosphiu1i1:bus1", "Positive-Sequence cosphi"
      m_I0x3watt_bus1 = "m:I0x3watt:bus1", "kA, Earth-Fault Residual Current"
      m_I0x3var_bus1 = "m:I0x3var:bus1", "kA, Capacitive Earth-Fault Current"
      m_Q0cap_bus1 = "m:Q0cap:bus1", "Mvar, Q0 capacitive"
      m_Zfrel_bus1 = "m:Zfrel:bus1", "Ohm, Relay Impedance, Magnitude"
      m_phizfrel_bus1 = "m:phizfrel:bus1", "deg, Relay Impedance, Angle"
      m_Rfrel_bus1 = "m:Rfrel:bus1", "Ohm, Relay Impedance, Real Part"
      m_Xfrel_bus1 = "m:Xfrel:bus1", "Ohm, Relay Impedance, Imaginary Part"
      m_Zf_bus1 = "m:Zf:bus1", "Ohm, Meas. Fault Impedance, Magnitude"
      m_phizf_bus1 = "m:phizf:bus1", "deg, Meas. Fault Impedance, Angle"
      n_Ikss_bus1_A = "n:Ikss:bus1:A", "kA, Initial Short-Circuit Current"
      n_Ikss_bus1_B = "n:Ikss:bus1:B", "kA, Initial Short-Circuit Current"
      n_Ikss_bus1_C = "n:Ikss:bus1:C", "kA, Initial Short-Circuit Current"
      n_I_bus1_A = "n:I:bus1:A", "kA, Short-Circuit Current"
      n_I_bus1_B = "n:I:bus1:B", "kA, Short-Circuit Current"
      n_I_bus1_C = "n:I:bus1:C", "kA, Short-Circuit Current"
      n_phii_bus1_A = "n:phii:bus1:A", "deg, Phase Current, Angle"
      n_phii_bus1_B = "n:phii:bus1:B", "deg, Phase Current, Angle"
      n_phii_bus1_C = "n:phii:bus1:C", "deg, Phase Current, Angle"
      n_Skss_bus1_A = "n:Skss:bus1:A", "MVA, Initial Short-Circuit Power"
      n_Skss_bus1_B = "n:Skss:bus1:B", "MVA, Initial Short-Circuit Power"
      n_Skss_bus1_C = "n:Skss:bus1:C", "MVA, Initial Short-Circuit Power"
      n_Skssx3_bus1_A = "n:Skssx3:bus1:A", "MVA, 3*Initial Short-Circuit Power"
      n_Skssx3_bus1_B = "n:Skssx3:bus1:B", "MVA, 3*Initial Short-Circuit Power"
      n_Skssx3_bus1_C = "n:Skssx3:bus1:C", "MVA, 3*Initial Short-Circuit Power"
      n_ip_bus1_A = "n:ip:bus1:A", "kA, Peak Short-Circuit Current"
      n_ip_bus1_B = "n:ip:bus1:B", "kA, Peak Short-Circuit Current"
      n_ip_bus1_C = "n:ip:bus1:C", "kA, Peak Short-Circuit Current"
      n_Ib_bus1_A = "n:Ib:bus1:A", "kA, Short-Circuit Breaking Current"
      n_Ib_bus1_B = "n:Ib:bus1:B", "kA, Short-Circuit Breaking Current"
      n_Ib_bus1_C = "n:Ib:bus1:C", "kA, Short-Circuit Breaking Current"
      n_Sb_bus1_A = "n:Sb:bus1:A", "MVA, Short-Circuit Breaking Power"
      n_Sb_bus1_B = "n:Sb:bus1:B", "MVA, Short-Circuit Breaking Power"
      n_Sb_bus1_C = "n:Sb:bus1:C", "MVA, Short-Circuit Breaking Power"
      n_Ith_bus1_A = "n:Ith:bus1:A", "kA, Thermal equivalent Short-Circuit Current"
      n_Ith_bus1_B = "n:Ith:bus1:B", "kA, Thermal equivalent Short-Circuit Current"
      n_Ith_bus1_C = "n:Ith:bus1:C", "kA, Thermal equivalent Short-Circuit Current"
      n_Ithmax_bus1 = "n:Ithmax:bus1", "kA, Max. Thermal equivalent Short-Circuit Current"
      n_Ik_bus1_A = "n:Ik:bus1:A", "kA, Steady-State Short-Circuit Current"
      n_Ik_bus1_B = "n:Ik:bus1:B", "kA, Steady-State Short-Circuit Current"
      n_Ik_bus1_C = "n:Ik:bus1:C", "kA, Steady-State Short-Circuit Current"
      n_R0_bus1 = "n:R0:bus1", "Ohm, Zero-Sequence Impedance, Real Part"
      n_X0_bus1 = "n:X0:bus1", "Ohm, Zero-Sequence Impedance, Imaginary Part"
      n_Z0_bus1 = "n:Z0:bus1", "Ohm, Zero-Sequence Impedance, Magnitude"
      n_phiz0_bus1 = "n:phiz0:bus1", "deg, Zero-Sequence Impedance, Angle"
      n_r0Sbase_bus1 = "n:r0Sbase:bus1", "p.u., Zero-Sequence Impedance, Real Part"
      n_x0Sbase_bus1 = "n:x0Sbase:bus1", "p.u., Zero-Sequence Impedance, Imaginary Part"
      n_z0Sbase_bus1 = "n:z0Sbase:bus1", "p.u., Zero-Sequence Impedance, Magnitude"
      n_R1_bus1 = "n:R1:bus1", "Ohm, Positive-Sequence Impedance, Real Part"
      n_X1_bus1 = "n:X1:bus1", "Ohm, Positive-Sequence Impedance, Imaginary Part"
      n_Z1_bus1 = "n:Z1:bus1", "Ohm, Positive-Sequence Impedance, Magnitude"
      n_phiz1_bus1 = "n:phiz1:bus1", "deg, Positive-Sequence Impedance, Angle"
      n_r1Sbase_bus1 = "n:r1Sbase:bus1", "p.u., Positive-Sequence Impedance, Real Part"
      n_x1Sbase_bus1 = "n:x1Sbase:bus1", "p.u., Positive-Sequence Impedance, Imaginary Part"
      n_z1Sbase_bus1 = "n:z1Sbase:bus1", "p.u., Positive-Sequence Impedance, Magnitude"
      n_R2_bus1 = "n:R2:bus1", "Ohm, Negative-Sequence Impedance, Real Part"
      n_X2_bus1 = "n:X2:bus1", "Ohm, Negative-Sequence Impedance, Imaginary Part"
      n_Z2_bus1 = "n:Z2:bus1", "Ohm, Negative-Sequence Impedance, Magnitude"
      n_phiz2_bus1 = "n:phiz2:bus1", "deg, Negative-Sequence Impedance, Angle"
      n_r2Sbase_bus1 = "n:r2Sbase:bus1", "p.u., Negative-Sequence Impedance, Real Part"
      n_x2Sbase_bus1 = "n:x2Sbase:bus1", "p.u., Negative-Sequence Impedance, Imaginary Part"
      n_z2Sbase_bus1 = "n:z2Sbase:bus1", "p.u., Negative-Sequence Impedance, Magnitude"
      n_R_bus1_A = "n:R:bus1:A", "Ohm, Network Resistance, Line-Ground"
      n_R_bus1_B = "n:R:bus1:B", "Ohm, Network Resistance, Line-Ground"
      n_R_bus1_C = "n:R:bus1:C", "Ohm, Network Resistance, Line-Ground"
      n_X_bus1_A = "n:X:bus1:A", "Ohm, Network Reactance, Line-Ground"
      n_X_bus1_B = "n:X:bus1:B", "Ohm, Network Reactance, Line-Ground"
      n_X_bus1_C = "n:X:bus1:C", "Ohm, Network Reactance, Line-Ground"
      n_Z_bus1_A = "n:Z:bus1:A", "Ohm, Network Impedance, Line-Ground"
      n_Z_bus1_B = "n:Z:bus1:B", "Ohm, Network Impedance, Line-Ground"
      n_Z_bus1_C = "n:Z:bus1:C", "Ohm, Network Impedance, Line-Ground"
      n_phiz_bus1_A = "n:phiz:bus1:A", "deg, Angle of Network Impedance, Line-Ground"
      n_phiz_bus1_B = "n:phiz:bus1:B", "deg, Angle of Network Impedance, Line-Ground"
      n_phiz_bus1_C = "n:phiz:bus1:C", "deg, Angle of Network Impedance, Line-Ground"
      n_Rl_bus1_A = "n:Rl:bus1:A", "Ohm, Network Resistance, Line-Line"
      n_Rl_bus1_B = "n:Rl:bus1:B", "Ohm, Network Resistance, Line-Line"
      n_Rl_bus1_C = "n:Rl:bus1:C", "Ohm, Network Resistance, Line-Line"
      n_Xl_bus1_A = "n:Xl:bus1:A", "Ohm, Network Reactance, Line-Line"
      n_Xl_bus1_B = "n:Xl:bus1:B", "Ohm, Network Reactance, Line-Line"
      n_Xl_bus1_C = "n:Xl:bus1:C", "Ohm, Network Reactance, Line-Line"
      n_Zl_bus1_A = "n:Zl:bus1:A", "Ohm, Network Impedance, Line-Line"
      n_Zl_bus1_B = "n:Zl:bus1:B", "Ohm, Network Impedance, Line-Line"
      n_Zl_bus1_C = "n:Zl:bus1:C", "Ohm, Network Impedance, Line-Line"
      n_phizl_bus1_A = "n:phizl:bus1:A", "deg, Angle of Network Impedance, Line-Line"
      n_phizl_bus1_B = "n:phizl:bus1:B", "deg, Angle of Network Impedance, Line-Line"
      n_phizl_bus1_C = "n:phizl:bus1:C", "deg, Angle of Network Impedance, Line-Line"
      n_Rln_bus1_A = "n:Rln:bus1:A", "Ohm, Network Resistance, Line-Neutral"
      n_Rln_bus1_B = "n:Rln:bus1:B", "Ohm, Network Resistance, Line-Neutral"
      n_Rln_bus1_C = "n:Rln:bus1:C", "Ohm, Network Resistance, Line-Neutral"
      n_Xln_bus1_A = "n:Xln:bus1:A", "Ohm, Network Reactance, Line-Neutral"
      n_Xln_bus1_B = "n:Xln:bus1:B", "Ohm, Network Reactance, Line-Neutral"
      n_Xln_bus1_C = "n:Xln:bus1:C", "Ohm, Network Reactance, Line-Neutral"
      n_Zln_bus1_A = "n:Zln:bus1:A", "Ohm, Network Impedance, Line-Neutral"
      n_Zln_bus1_B = "n:Zln:bus1:B", "Ohm, Network Impedance, Line-Neutral"
      n_Zln_bus1_C = "n:Zln:bus1:C", "Ohm, Network Impedance, Line-Neutral"
      n_phizln_bus1_A = "n:phizln:bus1:A", "deg, Angle of Network Impedance, Line-Neutral"
      n_phizln_bus1_B = "n:phizln:bus1:B", "deg, Angle of Network Impedance, Line-Neutral"
      n_phizln_bus1_C = "n:phizln:bus1:C", "deg, Angle of Network Impedance, Line-Neutral"
      n_Rn_bus1 = "n:Rn:bus1", "Ohm, Network Resistance, Neutral"
      n_Xn_bus1 = "n:Xn:bus1", "Ohm, Network Reactance, Neutral"
      n_Zn_bus1 = "n:Zn:bus1", "Ohm, Network Impedance, Neutral"
      n_phizn_bus1 = "n:phizn:bus1", "deg, Angle of Network Impedance, Neutral"
      n_eff_bus1_A = "n:eff:bus1:A", "Earth Fault Factor"
      n_eff_bus1_B = "n:eff:bus1:B", "Earth Fault Factor"
      n_eff_bus1_C = "n:eff:bus1:C", "Earth Fault Factor"
      n_X0toX1_bus1 = "n:X0toX1:bus1", "X0/X1"
      n_in_bus1 = "n:in:bus1", "p.u., Neutral Current, Magnitude"
      n_In_bus1 = "n:In:bus1", "kA, Neutral Current, Magnitude"
      n_phiin_bus1 = "n:phiin:bus1", "deg, Neutral Current, Angle"
      n_i0r_bus1 = "n:i0r:bus1", "p.u., Zero-Sequence Current, Real Part"
      n_i0i_bus1 = "n:i0i:bus1", "p.u., Zero-Sequence Current, Imaginary Part"
      n_i0_bus1 = "n:i0:bus1", "p.u., Zero-Sequence Current, Magnitude"
      n_I0_bus1 = "n:I0:bus1", "kA, Zero-Sequence Current, Magnitude"
      n_I0x3_bus1 = "n:I0x3:bus1", "kA, 3*I0"
      n_phii0_bus1 = "n:phii0:bus1", "deg, Zero-Sequence Current, Angle"
      n_i1r_bus1 = "n:i1r:bus1", "p.u., Positive-Sequence Current, Real Part"
      n_i1i_bus1 = "n:i1i:bus1", "p.u., Positive-Sequence Current, Imaginary Part"
      n_i1_bus1 = "n:i1:bus1", "p.u., Positive-Sequence Current, Magnitude"
      n_I1_bus1 = "n:I1:bus1", "kA, Positive-Sequence Current, Magnitude"
      n_phii1_bus1 = "n:phii1:bus1", "deg, Positive-Sequence Current, Angle"
      n_i2r_bus1 = "n:i2r:bus1", "p.u., Negative-Sequence Current, Real Part"
      n_i2i_bus1 = "n:i2i:bus1", "p.u., Negative-Sequence Current, Imaginary Part"
      n_i2_bus1 = "n:i2:bus1", "p.u., Negative-Sequence Current, Magnitude"
      n_I2_bus1 = "n:I2:bus1", "kA, Negative-Sequence Current, Magnitude"
      n_phii2_bus1 = "n:phii2:bus1", "deg, Negative-Sequence Current, Angle"
      n_du_bus1_A = "n:du:bus1:A", "%, Voltage Deviation"
      n_du_bus1_B = "n:du:bus1:B", "%, Voltage Deviation"
      n_du_bus1_C = "n:du:bus1:C", "%, Voltage Deviation"
      n_urpref_bus1_A = "n:urpref:bus1:A", "p.u., Pre-fault Voltage, Real Part"
      n_urpref_bus1_B = "n:urpref:bus1:B", "p.u., Pre-fault Voltage, Real Part"
      n_urpref_bus1_C = "n:urpref:bus1:C", "p.u., Pre-fault Voltage, Real Part"
      n_uipref_bus1_A = "n:uipref:bus1:A", "p.u., Pre-fault Voltage, Imaginary Part"
      n_uipref_bus1_B = "n:uipref:bus1:B", "p.u., Pre-fault Voltage, Imaginary Part"
      n_uipref_bus1_C = "n:uipref:bus1:C", "p.u., Pre-fault Voltage, Imaginary Part"
      n_upref_bus1_A = "n:upref:bus1:A", "p.u., Pre-fault Voltage, Magnitude"
      n_upref_bus1_B = "n:upref:bus1:B", "p.u., Pre-fault Voltage, Magnitude"
      n_upref_bus1_C = "n:upref:bus1:C", "p.u., Pre-fault Voltage, Magnitude"
      n_uprefpc_bus1_A = "n:uprefpc:bus1:A", "%, Pre-fault Voltage, Magnitude"
      n_uprefpc_bus1_B = "n:uprefpc:bus1:B", "%, Pre-fault Voltage, Magnitude"
      n_uprefpc_bus1_C = "n:uprefpc:bus1:C", "%, Pre-fault Voltage, Magnitude"
      n_Upref_bus1_A = "n:Upref:bus1:A", "kV, Line-Ground Pre-fault Voltage, Magnitude"
      n_Upref_bus1_B = "n:Upref:bus1:B", "kV, Line-Ground Pre-fault Voltage, Magnitude"
      n_Upref_bus1_C = "n:Upref:bus1:C", "kV, Line-Ground Pre-fault Voltage, Magnitude"
      n_Ulpref_bus1_A = "n:Ulpref:bus1:A", "kV, Line-Line Pre-fault Voltage, Magnitude"
      n_Ulpref_bus1_B = "n:Ulpref:bus1:B", "kV, Line-Line Pre-fault Voltage, Magnitude"
      n_Ulpref_bus1_C = "n:Ulpref:bus1:C", "kV, Line-Line Pre-fault Voltage, Magnitude"
      n_phiupref_bus1_A = "n:phiupref:bus1:A", "deg, Pre-fault Voltage, Angle"
      n_phiupref_bus1_B = "n:phiupref:bus1:B", "deg, Pre-fault Voltage, Angle"
      n_phiupref_bus1_C = "n:phiupref:bus1:C", "deg, Pre-fault Voltage, Angle"
      n_phiui_bus1_A = "n:phiui:bus1:A", "deg, Angle between Voltage and Current"
      n_phiui_bus1_B = "n:phiui:bus1:B", "deg, Angle between Voltage and Current"
      n_phiui_bus1_C = "n:phiui:bus1:C", "deg, Angle between Voltage and Current"
      n_phiu1i1_bus1 = "n:phiu1i1:bus1", "deg, Angle between Voltage and Current in positive sequence system"
      n_phiu2i2_bus1 = "n:phiu2i2:bus1", "deg, Angle between Voltage and Current in negative sequence system"
      n_phiu0i0_bus1 = "n:phiu0i0:bus1", "deg, Angle between Voltage and Current in zero-sequence system"

    class ElmLne(Enum):
      _init_ = 'value __doc__'

      m_u0_bus1 = "m:u0:bus1", "p.u., Zero-Sequence-Voltage, Magnitude"
      m_u0_bus2 = "m:u0:bus2", "p.u., Zero-Sequence-Voltage, Magnitude"
      m_phiu0_bus1 = "m:phiu0:bus1", "deg, Zero-Sequence-Voltage, Angle"
      m_phiu0_bus2 = "m:phiu0:bus2", "deg, Zero-Sequence-Voltage, Angle"
      m_u1r_bus1 = "m:u1r:bus1", "p.u., Positive-Sequence-Voltage, Real Part"
      m_u1r_bus2 = "m:u1r:bus2", "p.u., Positive-Sequence-Voltage, Real Part"
      m_u1i_bus1 = "m:u1i:bus1", "p.u., Positive-Sequence-Voltage, Imaginary Part"
      m_u1i_bus2 = "m:u1i:bus2", "p.u., Positive-Sequence-Voltage, Imaginary Part"
      m_u1_bus1 = "m:u1:bus1", "p.u., Positive-Sequence-Voltage, Magnitude"
      m_u1_bus2 = "m:u1:bus2", "p.u., Positive-Sequence-Voltage, Magnitude"
      m_U1_bus1 = "m:U1:bus1", "kV, Line-Ground Positive-Sequence-Voltage, Magnitude"
      m_U1_bus2 = "m:U1:bus2", "kV, Line-Ground Positive-Sequence-Voltage, Magnitude"
      m_U1l_bus1 = "m:U1l:bus1", "kV, Line-Line Positive-Sequence-Voltage, Magnitude"
      m_U1l_bus2 = "m:U1l:bus2", "kV, Line-Line Positive-Sequence-Voltage, Magnitude"
      m_phiu1_bus1 = "m:phiu1:bus1", "deg, Positive-Sequence-Voltage, Angle"
      m_phiu1_bus2 = "m:phiu1:bus2", "deg, Positive-Sequence-Voltage, Angle"
      m_u2_bus1 = "m:u2:bus1", "p.u., Negative-Sequence-Voltage, Magnitude"
      m_u2_bus2 = "m:u2:bus2", "p.u., Negative-Sequence-Voltage, Magnitude"
      m_phiu2_bus1 = "m:phiu2:bus1", "deg, Negative-Sequence-Voltage, Angle"
      m_phiu2_bus2 = "m:phiu2:bus2", "deg, Negative-Sequence-Voltage, Angle"
      m_inet_bus1_A = "m:inet:bus1:A", "p.u., Phase Current, Magnitude, referred to network"
      m_inet_bus1_B = "m:inet:bus1:B", "p.u., Phase Current, Magnitude, referred to network"
      m_inet_bus1_C = "m:inet:bus1:C", "p.u., Phase Current, Magnitude, referred to network"
      m_inet_bus1_N = "m:inet:bus1:N", "p.u., Phase Current, Magnitude, referred to network"
      m_inet_bus2_A = "m:inet:bus2:A", "p.u., Phase Current, Magnitude, referred to network"
      m_inet_bus2_B = "m:inet:bus2:B", "p.u., Phase Current, Magnitude, referred to network"
      m_inet_bus2_C = "m:inet:bus2:C", "p.u., Phase Current, Magnitude, referred to network"
      m_inet_bus2_N = "m:inet:bus2:N", "p.u., Phase Current, Magnitude, referred to network"
      m_I_bus1_A = "m:I:bus1:A", "kA, Phase Current, Magnitude"
      m_I_bus1_B = "m:I:bus1:B", "kA, Phase Current, Magnitude"
      m_I_bus1_C = "m:I:bus1:C", "kA, Phase Current, Magnitude"
      m_I_bus1_N = "m:I:bus1:N", "kA, Phase Current, Magnitude"
      m_I_bus2_A = "m:I:bus2:A", "kA, Phase Current, Magnitude"
      m_I_bus2_B = "m:I:bus2:B", "kA, Phase Current, Magnitude"
      m_I_bus2_C = "m:I:bus2:C", "kA, Phase Current, Magnitude"
      m_I_bus2_N = "m:I:bus2:N", "kA, Phase Current, Magnitude"
      m_phii_bus1_A = "m:phii:bus1:A", "deg, Phase Current, Angle"
      m_phii_bus1_B = "m:phii:bus1:B", "deg, Phase Current, Angle"
      m_phii_bus1_C = "m:phii:bus1:C", "deg, Phase Current, Angle"
      m_phii_bus1_N = "m:phii:bus1:N", "deg, Phase Current, Angle"
      m_phii_bus2_A = "m:phii:bus2:A", "deg, Phase Current, Angle"
      m_phii_bus2_B = "m:phii:bus2:B", "deg, Phase Current, Angle"
      m_phii_bus2_C = "m:phii:bus2:C", "deg, Phase Current, Angle"
      m_phii_bus2_N = "m:phii:bus2:N", "deg, Phase Current, Angle"
      m_i0r_bus1 = "m:i0r:bus1", "p.u., Zero-Sequence Current, Real Part"
      m_i0r_bus2 = "m:i0r:bus2", "p.u., Zero-Sequence Current, Real Part"
      m_i0i_bus1 = "m:i0i:bus1", "p.u., Zero-Sequence Current, Imaginary Part"
      m_i0i_bus2 = "m:i0i:bus2", "p.u., Zero-Sequence Current, Imaginary Part"
      m_i0_bus1 = "m:i0:bus1", "p.u., Zero-Sequence Current, Magnitude"
      m_i0_bus2 = "m:i0:bus2", "p.u., Zero-Sequence Current, Magnitude"
      m_I0_bus1 = "m:I0:bus1", "kA, Zero-Sequence Current, Magnitude"
      m_I0_bus2 = "m:I0:bus2", "kA, Zero-Sequence Current, Magnitude"
      m_I0x3_bus1 = "m:I0x3:bus1", "kA, 3*I0"
      m_I0x3_bus2 = "m:I0x3:bus2", "kA, 3*I0"
      m_phii0_bus1 = "m:phii0:bus1", "deg, Zero-Sequence Current, Angle"
      m_phii0_bus2 = "m:phii0:bus2", "deg, Zero-Sequence Current, Angle"
      m_i1r_bus1 = "m:i1r:bus1", "p.u., Positive-Sequence Current, Real Part"
      m_i1r_bus2 = "m:i1r:bus2", "p.u., Positive-Sequence Current, Real Part"
      m_i1i_bus1 = "m:i1i:bus1", "p.u., Positive-Sequence Current, Imaginary Part"
      m_i1i_bus2 = "m:i1i:bus2", "p.u., Positive-Sequence Current, Imaginary Part"
      m_i1_bus1 = "m:i1:bus1", "p.u., Positive-Sequence Current, Magnitude"
      m_i1_bus2 = "m:i1:bus2", "p.u., Positive-Sequence Current, Magnitude"
      m_I1_bus1 = "m:I1:bus1", "kA, Positive-Sequence Current, Magnitude"
      m_I1_bus2 = "m:I1:bus2", "kA, Positive-Sequence Current, Magnitude"
      m_phii1_bus1 = "m:phii1:bus1", "deg, Positive-Sequence Current, Angle"
      m_phii1_bus2 = "m:phii1:bus2", "deg, Positive-Sequence Current, Angle"
      m_i2r_bus1 = "m:i2r:bus1", "p.u., Negative-Sequence Current, Real Part"
      m_i2r_bus2 = "m:i2r:bus2", "p.u., Negative-Sequence Current, Real Part"
      m_i2i_bus1 = "m:i2i:bus1", "p.u., Negative-Sequence Current, Imaginary Part"
      m_i2i_bus2 = "m:i2i:bus2", "p.u., Negative-Sequence Current, Imaginary Part"
      m_i2_bus1 = "m:i2:bus1", "p.u., Negative-Sequence Current, Magnitude"
      m_i2_bus2 = "m:i2:bus2", "p.u., Negative-Sequence Current, Magnitude"
      m_I2_bus1 = "m:I2:bus1", "kA, Negative-Sequence Current, Magnitude"
      m_I2_bus2 = "m:I2:bus2", "kA, Negative-Sequence Current, Magnitude"
      m_phii2_bus1 = "m:phii2:bus1", "deg, Negative-Sequence Current, Angle"
      m_phii2_bus2 = "m:phii2:bus2", "deg, Negative-Sequence Current, Angle"
      m_ubfacI_bus1 = "m:ubfacI:bus1", "%, Current Unbalance Factor"
      m_ubfacI_bus2 = "m:ubfacI:bus2", "%, Current Unbalance Factor"
      m_ubfacS_bus1 = "m:ubfacS:bus1", "%, Power Unbalance Factor"
      m_ubfacS_bus2 = "m:ubfacS:bus2", "%, Power Unbalance Factor"
      m_P_bus1_A = "m:P:bus1:A", "MW, Active Power"
      m_P_bus1_B = "m:P:bus1:B", "MW, Active Power"
      m_P_bus1_C = "m:P:bus1:C", "MW, Active Power"
      m_P_bus1_N = "m:P:bus1:N", "MW, Active Power"
      m_P_bus2_A = "m:P:bus2:A", "MW, Active Power"
      m_P_bus2_B = "m:P:bus2:B", "MW, Active Power"
      m_P_bus2_C = "m:P:bus2:C", "MW, Active Power"
      m_P_bus2_N = "m:P:bus2:N", "MW, Active Power"
      m_Q_bus1_A = "m:Q:bus1:A", "Mvar, Reactive Power"
      m_Q_bus1_B = "m:Q:bus1:B", "Mvar, Reactive Power"
      m_Q_bus1_C = "m:Q:bus1:C", "Mvar, Reactive Power"
      m_Q_bus1_N = "m:Q:bus1:N", "Mvar, Reactive Power"
      m_Q_bus2_A = "m:Q:bus2:A", "Mvar, Reactive Power"
      m_Q_bus2_B = "m:Q:bus2:B", "Mvar, Reactive Power"
      m_Q_bus2_C = "m:Q:bus2:C", "Mvar, Reactive Power"
      m_Q_bus2_N = "m:Q:bus2:N", "Mvar, Reactive Power"
      m_S_bus1_A = "m:S:bus1:A", "MVA, Apparent Power"
      m_S_bus1_B = "m:S:bus1:B", "MVA, Apparent Power"
      m_S_bus1_C = "m:S:bus1:C", "MVA, Apparent Power"
      m_S_bus1_N = "m:S:bus1:N", "MVA, Apparent Power"
      m_S_bus2_A = "m:S:bus2:A", "MVA, Apparent Power"
      m_S_bus2_B = "m:S:bus2:B", "MVA, Apparent Power"
      m_S_bus2_C = "m:S:bus2:C", "MVA, Apparent Power"
      m_S_bus2_N = "m:S:bus2:N", "MVA, Apparent Power"
      m_cosphi_bus1_A = "m:cosphi:bus1:A", "Power Factor"
      m_cosphi_bus1_B = "m:cosphi:bus1:B", "Power Factor"
      m_cosphi_bus1_C = "m:cosphi:bus1:C", "Power Factor"
      m_cosphi_bus1_N = "m:cosphi:bus1:N", "Power Factor"
      m_cosphi_bus2_A = "m:cosphi:bus2:A", "Power Factor"
      m_cosphi_bus2_B = "m:cosphi:bus2:B", "Power Factor"
      m_cosphi_bus2_C = "m:cosphi:bus2:C", "Power Factor"
      m_cosphi_bus2_N = "m:cosphi:bus2:N", "Power Factor"
      m_tanphi_bus1_A = "m:tanphi:bus1:A", "tan(phi)"
      m_tanphi_bus1_B = "m:tanphi:bus1:B", "tan(phi)"
      m_tanphi_bus1_C = "m:tanphi:bus1:C", "tan(phi)"
      m_tanphi_bus1_N = "m:tanphi:bus1:N", "tan(phi)"
      m_tanphi_bus2_A = "m:tanphi:bus2:A", "tan(phi)"
      m_tanphi_bus2_B = "m:tanphi:bus2:B", "tan(phi)"
      m_tanphi_bus2_C = "m:tanphi:bus2:C", "tan(phi)"
      m_tanphi_bus2_N = "m:tanphi:bus2:N", "tan(phi)"
      m_Psum_bus1 = "m:Psum:bus1", "MW, Total Active Power"
      m_Psum_bus2 = "m:Psum:bus2", "MW, Total Active Power"
      m_Qsum_bus1 = "m:Qsum:bus1", "Mvar, Total Reactive Power"
      m_Qsum_bus2 = "m:Qsum:bus2", "Mvar, Total Reactive Power"
      m_Ssum_bus1 = "m:Ssum:bus1", "MVA, Total Apparent Power"
      m_Ssum_bus2 = "m:Ssum:bus2", "MVA, Total Apparent Power"
      m_cosphisum_bus1 = "m:cosphisum:bus1", "Total Power Factor"
      m_cosphisum_bus2 = "m:cosphisum:bus2", "Total Power Factor"
      m_tanphisum_bus1 = "m:tanphisum:bus1", "Total tan(phi)"
      m_tanphisum_bus2 = "m:tanphisum:bus2", "Total tan(phi)"
      m_Pln_bus1_A = "m:Pln:bus1:A", "MW, Active Power (Line-Neutral)"
      m_Pln_bus1_B = "m:Pln:bus1:B", "MW, Active Power (Line-Neutral)"
      m_Pln_bus1_C = "m:Pln:bus1:C", "MW, Active Power (Line-Neutral)"
      m_Pln_bus1_N = "m:Pln:bus1:N", "MW, Active Power (Line-Neutral)"
      m_Pln_bus2_A = "m:Pln:bus2:A", "MW, Active Power (Line-Neutral)"
      m_Pln_bus2_B = "m:Pln:bus2:B", "MW, Active Power (Line-Neutral)"
      m_Pln_bus2_C = "m:Pln:bus2:C", "MW, Active Power (Line-Neutral)"
      m_Pln_bus2_N = "m:Pln:bus2:N", "MW, Active Power (Line-Neutral)"
      m_Qln_bus1_A = "m:Qln:bus1:A", "Mvar, Reactive Power (Line-Neutral)"
      m_Qln_bus1_B = "m:Qln:bus1:B", "Mvar, Reactive Power (Line-Neutral)"
      m_Qln_bus1_C = "m:Qln:bus1:C", "Mvar, Reactive Power (Line-Neutral)"
      m_Qln_bus1_N = "m:Qln:bus1:N", "Mvar, Reactive Power (Line-Neutral)"
      m_Qln_bus2_A = "m:Qln:bus2:A", "Mvar, Reactive Power (Line-Neutral)"
      m_Qln_bus2_B = "m:Qln:bus2:B", "Mvar, Reactive Power (Line-Neutral)"
      m_Qln_bus2_C = "m:Qln:bus2:C", "Mvar, Reactive Power (Line-Neutral)"
      m_Qln_bus2_N = "m:Qln:bus2:N", "Mvar, Reactive Power (Line-Neutral)"
      m_P1_bus1 = "m:P1:bus1", "MW, Positive Sequence Active Power"
      m_P1_bus2 = "m:P1:bus2", "MW, Positive Sequence Active Power"
      m_Q1_bus1 = "m:Q1:bus1", "Mvar, Positive Sequence Reactive Power"
      m_Q1_bus2 = "m:Q1:bus2", "Mvar, Positive Sequence Reactive Power"
      m_P2_bus1 = "m:P2:bus1", "MW, Negative Sequence Active Power"
      m_P2_bus2 = "m:P2:bus2", "MW, Negative Sequence Active Power"
      m_Q2_bus1 = "m:Q2:bus1", "Mvar, Negative Sequence Reactive Power"
      m_Q2_bus2 = "m:Q2:bus2", "Mvar, Negative Sequence Reactive Power"
      m_P0_bus1 = "m:P0:bus1", "MW, Zero Sequence Active Power"
      m_P0_bus2 = "m:P0:bus2", "MW, Zero Sequence Active Power"
      m_Q0_bus1 = "m:Q0:bus1", "Mvar, Zero Sequence Reactive Power"
      m_Q0_bus2 = "m:Q0:bus2", "Mvar, Zero Sequence Reactive Power"
      m_i1P_bus1 = "m:i1P:bus1", "p.u., Positive-Sequence Active Current"
      m_i1P_bus2 = "m:i1P:bus2", "p.u., Positive-Sequence Active Current"
      m_i1Q_bus1 = "m:i1Q:bus1", "p.u., Positive-Sequence Reactive Current"
      m_i1Q_bus2 = "m:i1Q:bus2", "p.u., Positive-Sequence Reactive Current"
      m_I1P_bus1 = "m:I1P:bus1", "kA, Positive-Sequence Active Current"
      m_I1P_bus2 = "m:I1P:bus2", "kA, Positive-Sequence Active Current"
      m_I1Q_bus1 = "m:I1Q:bus1", "kA, Positive-Sequence Reactive Current"
      m_I1Q_bus2 = "m:I1Q:bus2", "kA, Positive-Sequence Reactive Current"
      m_i2P_bus1 = "m:i2P:bus1", "p.u., Negative-Sequence Active Current"
      m_i2P_bus2 = "m:i2P:bus2", "p.u., Negative-Sequence Active Current"
      m_i2Q_bus1 = "m:i2Q:bus1", "p.u., Negative-Sequence Reactive Current"
      m_i2Q_bus2 = "m:i2Q:bus2", "p.u., Negative-Sequence Reactive Current"
      m_I2P_bus1 = "m:I2P:bus1", "kA, Negative-Sequence Active Current"
      m_I2P_bus2 = "m:I2P:bus2", "kA, Negative-Sequence Active Current"
      m_I2Q_bus1 = "m:I2Q:bus1", "kA, Negative-Sequence Reactive Current"
      m_I2Q_bus2 = "m:I2Q:bus2", "kA, Negative-Sequence Reactive Current"
      m_ur_bus1_A = "m:ur:bus1:A", "p.u., Phase Voltage, Real Part"
      m_ur_bus1_B = "m:ur:bus1:B", "p.u., Phase Voltage, Real Part"
      m_ur_bus1_C = "m:ur:bus1:C", "p.u., Phase Voltage, Real Part"
      m_ur_bus1_N = "m:ur:bus1:N", "p.u., Phase Voltage, Real Part"
      m_ur_bus2_A = "m:ur:bus2:A", "p.u., Phase Voltage, Real Part"
      m_ur_bus2_B = "m:ur:bus2:B", "p.u., Phase Voltage, Real Part"
      m_ur_bus2_C = "m:ur:bus2:C", "p.u., Phase Voltage, Real Part"
      m_ur_bus2_N = "m:ur:bus2:N", "p.u., Phase Voltage, Real Part"
      m_ui_bus1_A = "m:ui:bus1:A", "p.u., Phase Voltage, Imaginary Part"
      m_ui_bus1_B = "m:ui:bus1:B", "p.u., Phase Voltage, Imaginary Part"
      m_ui_bus1_C = "m:ui:bus1:C", "p.u., Phase Voltage, Imaginary Part"
      m_ui_bus1_N = "m:ui:bus1:N", "p.u., Phase Voltage, Imaginary Part"
      m_ui_bus2_A = "m:ui:bus2:A", "p.u., Phase Voltage, Imaginary Part"
      m_ui_bus2_B = "m:ui:bus2:B", "p.u., Phase Voltage, Imaginary Part"
      m_ui_bus2_C = "m:ui:bus2:C", "p.u., Phase Voltage, Imaginary Part"
      m_ui_bus2_N = "m:ui:bus2:N", "p.u., Phase Voltage, Imaginary Part"
      m_u_bus1_A = "m:u:bus1:A", "p.u., Phase Voltage, Magnitude"
      m_u_bus1_B = "m:u:bus1:B", "p.u., Phase Voltage, Magnitude"
      m_u_bus1_C = "m:u:bus1:C", "p.u., Phase Voltage, Magnitude"
      m_u_bus1_N = "m:u:bus1:N", "p.u., Phase Voltage, Magnitude"
      m_u_bus2_A = "m:u:bus2:A", "p.u., Phase Voltage, Magnitude"
      m_u_bus2_B = "m:u:bus2:B", "p.u., Phase Voltage, Magnitude"
      m_u_bus2_C = "m:u:bus2:C", "p.u., Phase Voltage, Magnitude"
      m_u_bus2_N = "m:u:bus2:N", "p.u., Phase Voltage, Magnitude"
      m_ir_bus1_A = "m:ir:bus1:A", "p.u., Phase Current, Real Part"
      m_ir_bus1_B = "m:ir:bus1:B", "p.u., Phase Current, Real Part"
      m_ir_bus1_C = "m:ir:bus1:C", "p.u., Phase Current, Real Part"
      m_ir_bus1_N = "m:ir:bus1:N", "p.u., Phase Current, Real Part"
      m_ir_bus2_A = "m:ir:bus2:A", "p.u., Phase Current, Real Part"
      m_ir_bus2_B = "m:ir:bus2:B", "p.u., Phase Current, Real Part"
      m_ir_bus2_C = "m:ir:bus2:C", "p.u., Phase Current, Real Part"
      m_ir_bus2_N = "m:ir:bus2:N", "p.u., Phase Current, Real Part"
      m_ii_bus1_A = "m:ii:bus1:A", "p.u., Phase Current, Imaginary Part"
      m_ii_bus1_B = "m:ii:bus1:B", "p.u., Phase Current, Imaginary Part"
      m_ii_bus1_C = "m:ii:bus1:C", "p.u., Phase Current, Imaginary Part"
      m_ii_bus1_N = "m:ii:bus1:N", "p.u., Phase Current, Imaginary Part"
      m_ii_bus2_A = "m:ii:bus2:A", "p.u., Phase Current, Imaginary Part"
      m_ii_bus2_B = "m:ii:bus2:B", "p.u., Phase Current, Imaginary Part"
      m_ii_bus2_C = "m:ii:bus2:C", "p.u., Phase Current, Imaginary Part"
      m_ii_bus2_N = "m:ii:bus2:N", "p.u., Phase Current, Imaginary Part"
      m_i_bus1_A = "m:i:bus1:A", "p.u., Phase Current, Magnitude"
      m_i_bus1_B = "m:i:bus1:B", "p.u., Phase Current, Magnitude"
      m_i_bus1_C = "m:i:bus1:C", "p.u., Phase Current, Magnitude"
      m_i_bus1_N = "m:i:bus1:N", "p.u., Phase Current, Magnitude"
      m_i_bus2_A = "m:i:bus2:A", "p.u., Phase Current, Magnitude"
      m_i_bus2_B = "m:i:bus2:B", "p.u., Phase Current, Magnitude"
      m_i_bus2_C = "m:i:bus2:C", "p.u., Phase Current, Magnitude"
      m_i_bus2_N = "m:i:bus2:N", "p.u., Phase Current, Magnitude"
      m_phiui_bus1_A = "m:phiui:bus1:A", "deg, Angle between Voltage and Current"
      m_phiui_bus1_B = "m:phiui:bus1:B", "deg, Angle between Voltage and Current"
      m_phiui_bus1_C = "m:phiui:bus1:C", "deg, Angle between Voltage and Current"
      m_phiui_bus1_N = "m:phiui:bus1:N", "deg, Angle between Voltage and Current"
      m_phiui_bus2_A = "m:phiui:bus2:A", "deg, Angle between Voltage and Current"
      m_phiui_bus2_B = "m:phiui:bus2:B", "deg, Angle between Voltage and Current"
      m_phiui_bus2_C = "m:phiui:bus2:C", "deg, Angle between Voltage and Current"
      m_phiui_bus2_N = "m:phiui:bus2:N", "deg, Angle between Voltage and Current"
      m_phiu0i0_bus1 = "m:phiu0i0:bus1", "deg, Angle between Voltage and Current in zero-sequence system"
      m_phiu0i0_bus2 = "m:phiu0i0:bus2", "deg, Angle between Voltage and Current in zero-sequence system"
      m_phiu1i1_bus1 = "m:phiu1i1:bus1", "deg, Angle between Voltage and Current in positive sequence system"
      m_phiu1i1_bus2 = "m:phiu1i1:bus2", "deg, Angle between Voltage and Current in positive sequence system"
      m_phiu2i2_bus1 = "m:phiu2i2:bus1", "deg, Angle between Voltage and Current in negative sequence system"
      m_phiu2i2_bus2 = "m:phiu2i2:bus2", "deg, Angle between Voltage and Current in negative sequence system"
      m_Tfct_bus1 = "m:Tfct:bus1", "s, Fault Clearing Time"
      m_Tfct_bus2 = "m:Tfct:bus2", "s, Fault Clearing Time"
      m_TfctPh_bus1_A = "m:TfctPh:bus1:A", "s, Fault Clearing Time"
      m_TfctPh_bus1_B = "m:TfctPh:bus1:B", "s, Fault Clearing Time"
      m_TfctPh_bus1_C = "m:TfctPh:bus1:C", "s, Fault Clearing Time"
      m_TfctPh_bus1_N = "m:TfctPh:bus1:N", "s, Fault Clearing Time"
      m_TfctPh_bus2_A = "m:TfctPh:bus2:A", "s, Fault Clearing Time"
      m_TfctPh_bus2_B = "m:TfctPh:bus2:B", "s, Fault Clearing Time"
      m_TfctPh_bus2_C = "m:TfctPh:bus2:C", "s, Fault Clearing Time"
      m_TfctPh_bus2_N = "m:TfctPh:bus2:N", "s, Fault Clearing Time"
      m_Brkload_bus1 = "m:Brkload:bus1", "%, Max. Breaker Loading"
      m_Brkload_bus2 = "m:Brkload:bus2", "%, Max. Breaker Loading"
      m_BrkloadPh_bus1_A = "m:BrkloadPh:bus1:A", "%, Breaker Loading"
      m_BrkloadPh_bus1_B = "m:BrkloadPh:bus1:B", "%, Breaker Loading"
      m_BrkloadPh_bus1_C = "m:BrkloadPh:bus1:C", "%, Breaker Loading"
      m_BrkloadPh_bus1_N = "m:BrkloadPh:bus1:N", "%, Breaker Loading"
      m_BrkloadPh_bus2_A = "m:BrkloadPh:bus2:A", "%, Breaker Loading"
      m_BrkloadPh_bus2_B = "m:BrkloadPh:bus2:B", "%, Breaker Loading"
      m_BrkloadPh_bus2_C = "m:BrkloadPh:bus2:C", "%, Breaker Loading"
      m_BrkloadPh_bus2_N = "m:BrkloadPh:bus2:N", "%, Breaker Loading"
      n_ur_bus1_A = "n:ur:bus1:A", "p.u., Line-Ground Voltage, Real Part"
      n_ur_bus1_B = "n:ur:bus1:B", "p.u., Line-Ground Voltage, Real Part"
      n_ur_bus1_C = "n:ur:bus1:C", "p.u., Line-Ground Voltage, Real Part"
      n_ur_bus1_N = "n:ur:bus1:N", "p.u., Line-Ground Voltage, Real Part"
      n_ur_bus2_A = "n:ur:bus2:A", "p.u., Line-Ground Voltage, Real Part"
      n_ur_bus2_B = "n:ur:bus2:B", "p.u., Line-Ground Voltage, Real Part"
      n_ur_bus2_C = "n:ur:bus2:C", "p.u., Line-Ground Voltage, Real Part"
      n_ur_bus2_N = "n:ur:bus2:N", "p.u., Line-Ground Voltage, Real Part"
      n_ui_bus1_A = "n:ui:bus1:A", "p.u., Line-Ground Voltage, Imaginary Part"
      n_ui_bus1_B = "n:ui:bus1:B", "p.u., Line-Ground Voltage, Imaginary Part"
      n_ui_bus1_C = "n:ui:bus1:C", "p.u., Line-Ground Voltage, Imaginary Part"
      n_ui_bus1_N = "n:ui:bus1:N", "p.u., Line-Ground Voltage, Imaginary Part"
      n_ui_bus2_A = "n:ui:bus2:A", "p.u., Line-Ground Voltage, Imaginary Part"
      n_ui_bus2_B = "n:ui:bus2:B", "p.u., Line-Ground Voltage, Imaginary Part"
      n_ui_bus2_C = "n:ui:bus2:C", "p.u., Line-Ground Voltage, Imaginary Part"
      n_ui_bus2_N = "n:ui:bus2:N", "p.u., Line-Ground Voltage, Imaginary Part"
      n_u_bus1_A = "n:u:bus1:A", "p.u., Line-Ground Voltage, Magnitude"
      n_u_bus1_B = "n:u:bus1:B", "p.u., Line-Ground Voltage, Magnitude"
      n_u_bus1_C = "n:u:bus1:C", "p.u., Line-Ground Voltage, Magnitude"
      n_u_bus1_N = "n:u:bus1:N", "p.u., Line-Ground Voltage, Magnitude"
      n_u_bus2_A = "n:u:bus2:A", "p.u., Line-Ground Voltage, Magnitude"
      n_u_bus2_B = "n:u:bus2:B", "p.u., Line-Ground Voltage, Magnitude"
      n_u_bus2_C = "n:u:bus2:C", "p.u., Line-Ground Voltage, Magnitude"
      n_u_bus2_N = "n:u:bus2:N", "p.u., Line-Ground Voltage, Magnitude"
      n_upc_bus1_A = "n:upc:bus1:A", "%, Line-Ground Voltage, Magnitude"
      n_upc_bus1_B = "n:upc:bus1:B", "%, Line-Ground Voltage, Magnitude"
      n_upc_bus1_C = "n:upc:bus1:C", "%, Line-Ground Voltage, Magnitude"
      n_upc_bus1_N = "n:upc:bus1:N", "%, Line-Ground Voltage, Magnitude"
      n_upc_bus2_A = "n:upc:bus2:A", "%, Line-Ground Voltage, Magnitude"
      n_upc_bus2_B = "n:upc:bus2:B", "%, Line-Ground Voltage, Magnitude"
      n_upc_bus2_C = "n:upc:bus2:C", "%, Line-Ground Voltage, Magnitude"
      n_upc_bus2_N = "n:upc:bus2:N", "%, Line-Ground Voltage, Magnitude"
      n_U_bus1_A = "n:U:bus1:A", "kV, Line-Ground Voltage, Magnitude"
      n_U_bus1_B = "n:U:bus1:B", "kV, Line-Ground Voltage, Magnitude"
      n_U_bus1_C = "n:U:bus1:C", "kV, Line-Ground Voltage, Magnitude"
      n_U_bus1_N = "n:U:bus1:N", "kV, Line-Ground Voltage, Magnitude"
      n_U_bus2_A = "n:U:bus2:A", "kV, Line-Ground Voltage, Magnitude"
      n_U_bus2_B = "n:U:bus2:B", "kV, Line-Ground Voltage, Magnitude"
      n_U_bus2_C = "n:U:bus2:C", "kV, Line-Ground Voltage, Magnitude"
      n_U_bus2_N = "n:U:bus2:N", "kV, Line-Ground Voltage, Magnitude"
      n_phiu_bus1_A = "n:phiu:bus1:A", "deg, Line-Ground Voltage, Angle"
      n_phiu_bus1_B = "n:phiu:bus1:B", "deg, Line-Ground Voltage, Angle"
      n_phiu_bus1_C = "n:phiu:bus1:C", "deg, Line-Ground Voltage, Angle"
      n_phiu_bus1_N = "n:phiu:bus1:N", "deg, Line-Ground Voltage, Angle"
      n_phiu_bus2_A = "n:phiu:bus2:A", "deg, Line-Ground Voltage, Angle"
      n_phiu_bus2_B = "n:phiu:bus2:B", "deg, Line-Ground Voltage, Angle"
      n_phiu_bus2_C = "n:phiu:bus2:C", "deg, Line-Ground Voltage, Angle"
      n_phiu_bus2_N = "n:phiu:bus2:N", "deg, Line-Ground Voltage, Angle"
      n_phiurel_bus1 = "n:phiurel:bus1", "deg, Voltage, Relative Angle"
      n_phiurel_bus2 = "n:phiurel:bus2", "deg, Voltage, Relative Angle"
      n_ul_bus1_A = "n:ul:bus1:A", "p.u., Line-Line Voltage, Magnitude"
      n_ul_bus1_B = "n:ul:bus1:B", "p.u., Line-Line Voltage, Magnitude"
      n_ul_bus1_C = "n:ul:bus1:C", "p.u., Line-Line Voltage, Magnitude"
      n_ul_bus1_N = "n:ul:bus1:N", "p.u., Line-Line Voltage, Magnitude"
      n_ul_bus2_A = "n:ul:bus2:A", "p.u., Line-Line Voltage, Magnitude"
      n_ul_bus2_B = "n:ul:bus2:B", "p.u., Line-Line Voltage, Magnitude"
      n_ul_bus2_C = "n:ul:bus2:C", "p.u., Line-Line Voltage, Magnitude"
      n_ul_bus2_N = "n:ul:bus2:N", "p.u., Line-Line Voltage, Magnitude"
      n_ulpc_bus1_A = "n:ulpc:bus1:A", "%, Line-Line Voltage, Magnitude"
      n_ulpc_bus1_B = "n:ulpc:bus1:B", "%, Line-Line Voltage, Magnitude"
      n_ulpc_bus1_C = "n:ulpc:bus1:C", "%, Line-Line Voltage, Magnitude"
      n_ulpc_bus1_N = "n:ulpc:bus1:N", "%, Line-Line Voltage, Magnitude"
      n_ulpc_bus2_A = "n:ulpc:bus2:A", "%, Line-Line Voltage, Magnitude"
      n_ulpc_bus2_B = "n:ulpc:bus2:B", "%, Line-Line Voltage, Magnitude"
      n_ulpc_bus2_C = "n:ulpc:bus2:C", "%, Line-Line Voltage, Magnitude"
      n_ulpc_bus2_N = "n:ulpc:bus2:N", "%, Line-Line Voltage, Magnitude"
      n_Ul_bus1_A = "n:Ul:bus1:A", "kV, Line-Line Voltage, Magnitude"
      n_Ul_bus1_B = "n:Ul:bus1:B", "kV, Line-Line Voltage, Magnitude"
      n_Ul_bus1_C = "n:Ul:bus1:C", "kV, Line-Line Voltage, Magnitude"
      n_Ul_bus1_N = "n:Ul:bus1:N", "kV, Line-Line Voltage, Magnitude"
      n_Ul_bus2_A = "n:Ul:bus2:A", "kV, Line-Line Voltage, Magnitude"
      n_Ul_bus2_B = "n:Ul:bus2:B", "kV, Line-Line Voltage, Magnitude"
      n_Ul_bus2_C = "n:Ul:bus2:C", "kV, Line-Line Voltage, Magnitude"
      n_Ul_bus2_N = "n:Ul:bus2:N", "kV, Line-Line Voltage, Magnitude"
      n_phiul_bus1_A = "n:phiul:bus1:A", "deg, Line-Line Voltage, Angle"
      n_phiul_bus1_B = "n:phiul:bus1:B", "deg, Line-Line Voltage, Angle"
      n_phiul_bus1_C = "n:phiul:bus1:C", "deg, Line-Line Voltage, Angle"
      n_phiul_bus1_N = "n:phiul:bus1:N", "deg, Line-Line Voltage, Angle"
      n_phiul_bus2_A = "n:phiul:bus2:A", "deg, Line-Line Voltage, Angle"
      n_phiul_bus2_B = "n:phiul:bus2:B", "deg, Line-Line Voltage, Angle"
      n_phiul_bus2_C = "n:phiul:bus2:C", "deg, Line-Line Voltage, Angle"
      n_phiul_bus2_N = "n:phiul:bus2:N", "deg, Line-Line Voltage, Angle"
      n_u0_bus1 = "n:u0:bus1", "p.u., Zero-Sequence Voltage, Magnitude"
      n_u0_bus2 = "n:u0:bus2", "p.u., Zero-Sequence Voltage, Magnitude"
      n_U0_bus1 = "n:U0:bus1", "kV, Zero-Sequence Voltage, Magnitude"
      n_U0_bus2 = "n:U0:bus2", "kV, Zero-Sequence Voltage, Magnitude"
      n_U0x3_bus1 = "n:U0x3:bus1", "kV, 3*U0"
      n_U0x3_bus2 = "n:U0x3:bus2", "kV, 3*U0"
      n_phiu0_bus1 = "n:phiu0:bus1", "deg, Zero-Sequence Voltage, Angle"
      n_phiu0_bus2 = "n:phiu0:bus2", "deg, Zero-Sequence Voltage, Angle"
      n_u1_bus1 = "n:u1:bus1", "p.u., Positive-Sequence Voltage, Magnitude"
      n_u1_bus2 = "n:u1:bus2", "p.u., Positive-Sequence Voltage, Magnitude"
      n_u1pc_bus1 = "n:u1pc:bus1", "%, Positive-Sequence Voltage, Magnitude"
      n_u1pc_bus2 = "n:u1pc:bus2", "%, Positive-Sequence Voltage, Magnitude"
      n_u1r_bus1 = "n:u1r:bus1", "p.u., Positive-Sequence Voltage, Real Part"
      n_u1r_bus2 = "n:u1r:bus2", "p.u., Positive-Sequence Voltage, Real Part"
      n_u1i_bus1 = "n:u1i:bus1", "p.u., Positive-Sequence Voltage, Imaginary Part"
      n_u1i_bus2 = "n:u1i:bus2", "p.u., Positive-Sequence Voltage, Imaginary Part"
      n_U1_bus1 = "n:U1:bus1", "kV, Line-Ground Positive-Sequence Voltage, Magnitude"
      n_U1_bus2 = "n:U1:bus2", "kV, Line-Ground Positive-Sequence Voltage, Magnitude"
      n_phiu1_bus1 = "n:phiu1:bus1", "deg, Positive-Sequence Voltage, Angle"
      n_phiu1_bus2 = "n:phiu1:bus2", "deg, Positive-Sequence Voltage, Angle"
      n_u2_bus1 = "n:u2:bus1", "p.u., Negative-Sequence Voltage, Magnitude"
      n_u2_bus2 = "n:u2:bus2", "p.u., Negative-Sequence Voltage, Magnitude"
      n_U2_bus1 = "n:U2:bus1", "kV, Line-Ground Negative-Sequence Voltage, Magnitude"
      n_U2_bus2 = "n:U2:bus2", "kV, Line-Ground Negative-Sequence Voltage, Magnitude"
      n_phiu2_bus1 = "n:phiu2:bus1", "deg, Negative-Sequence Voltage, Angle"
      n_phiu2_bus2 = "n:phiu2:bus2", "deg, Negative-Sequence Voltage, Angle"
      n_U1l_bus1 = "n:U1l:bus1", "kV, Line-Line Positive-Sequence Voltage, Magnitude"
      n_U1l_bus2 = "n:U1l:bus2", "kV, Line-Line Positive-Sequence Voltage, Magnitude"
      n_U2l_bus1 = "n:U2l:bus1", "kV, Line-Line Negative-Sequence Voltage, Magnitude"
      n_U2l_bus2 = "n:U2l:bus2", "kV, Line-Line Negative-Sequence Voltage, Magnitude"
      n_uln_bus1_A = "n:uln:bus1:A", "p.u., Line-Neutral Voltage, Magnitude"
      n_uln_bus1_B = "n:uln:bus1:B", "p.u., Line-Neutral Voltage, Magnitude"
      n_uln_bus1_C = "n:uln:bus1:C", "p.u., Line-Neutral Voltage, Magnitude"
      n_uln_bus1_N = "n:uln:bus1:N", "p.u., Line-Neutral Voltage, Magnitude"
      n_uln_bus2_A = "n:uln:bus2:A", "p.u., Line-Neutral Voltage, Magnitude"
      n_uln_bus2_B = "n:uln:bus2:B", "p.u., Line-Neutral Voltage, Magnitude"
      n_uln_bus2_C = "n:uln:bus2:C", "p.u., Line-Neutral Voltage, Magnitude"
      n_uln_bus2_N = "n:uln:bus2:N", "p.u., Line-Neutral Voltage, Magnitude"
      n_Uln_bus1_A = "n:Uln:bus1:A", "kV, Line-Neutral Voltage, Magnitude"
      n_Uln_bus1_B = "n:Uln:bus1:B", "kV, Line-Neutral Voltage, Magnitude"
      n_Uln_bus1_C = "n:Uln:bus1:C", "kV, Line-Neutral Voltage, Magnitude"
      n_Uln_bus1_N = "n:Uln:bus1:N", "kV, Line-Neutral Voltage, Magnitude"
      n_Uln_bus2_A = "n:Uln:bus2:A", "kV, Line-Neutral Voltage, Magnitude"
      n_Uln_bus2_B = "n:Uln:bus2:B", "kV, Line-Neutral Voltage, Magnitude"
      n_Uln_bus2_C = "n:Uln:bus2:C", "kV, Line-Neutral Voltage, Magnitude"
      n_Uln_bus2_N = "n:Uln:bus2:N", "kV, Line-Neutral Voltage, Magnitude"
      n_phiuln_bus1_A = "n:phiuln:bus1:A", "deg, Line-Neutral Voltage, Angle"
      n_phiuln_bus1_B = "n:phiuln:bus1:B", "deg, Line-Neutral Voltage, Angle"
      n_phiuln_bus1_C = "n:phiuln:bus1:C", "deg, Line-Neutral Voltage, Angle"
      n_phiuln_bus1_N = "n:phiuln:bus1:N", "deg, Line-Neutral Voltage, Angle"
      n_phiuln_bus2_A = "n:phiuln:bus2:A", "deg, Line-Neutral Voltage, Angle"
      n_phiuln_bus2_B = "n:phiuln:bus2:B", "deg, Line-Neutral Voltage, Angle"
      n_phiuln_bus2_C = "n:phiuln:bus2:C", "deg, Line-Neutral Voltage, Angle"
      n_phiuln_bus2_N = "n:phiuln:bus2:N", "deg, Line-Neutral Voltage, Angle"
      n_un_bus1 = "n:un:bus1", "p.u., Neutral-Ground Voltage, Magnitude"
      n_un_bus2 = "n:un:bus2", "p.u., Neutral-Ground Voltage, Magnitude"
      n_Un_bus1 = "n:Un:bus1", "kV, Neutral-Ground Voltage, Magnitude"
      n_Un_bus2 = "n:Un:bus2", "kV, Neutral-Ground Voltage, Magnitude"
      n_phiun_bus1 = "n:phiun:bus1", "deg, Neutral-Ground Voltage, Angle"
      n_phiun_bus2 = "n:phiun:bus2", "deg, Neutral-Ground Voltage, Angle"
      n_upht_bus1_A = "n:upht:bus1:A", "p.u., Phase Technology dependent Voltage, Magnitude"
      n_upht_bus1_B = "n:upht:bus1:B", "p.u., Phase Technology dependent Voltage, Magnitude"
      n_upht_bus1_C = "n:upht:bus1:C", "p.u., Phase Technology dependent Voltage, Magnitude"
      n_upht_bus1_N = "n:upht:bus1:N", "p.u., Phase Technology dependent Voltage, Magnitude"
      n_upht_bus2_A = "n:upht:bus2:A", "p.u., Phase Technology dependent Voltage, Magnitude"
      n_upht_bus2_B = "n:upht:bus2:B", "p.u., Phase Technology dependent Voltage, Magnitude"
      n_upht_bus2_C = "n:upht:bus2:C", "p.u., Phase Technology dependent Voltage, Magnitude"
      n_upht_bus2_N = "n:upht:bus2:N", "p.u., Phase Technology dependent Voltage, Magnitude"
      n_Upht_bus1_A = "n:Upht:bus1:A", "kV, Phase Technology dependent Voltage, Magnitude"
      n_Upht_bus1_B = "n:Upht:bus1:B", "kV, Phase Technology dependent Voltage, Magnitude"
      n_Upht_bus1_C = "n:Upht:bus1:C", "kV, Phase Technology dependent Voltage, Magnitude"
      n_Upht_bus1_N = "n:Upht:bus1:N", "kV, Phase Technology dependent Voltage, Magnitude"
      n_Upht_bus2_A = "n:Upht:bus2:A", "kV, Phase Technology dependent Voltage, Magnitude"
      n_Upht_bus2_B = "n:Upht:bus2:B", "kV, Phase Technology dependent Voltage, Magnitude"
      n_Upht_bus2_C = "n:Upht:bus2:C", "kV, Phase Technology dependent Voltage, Magnitude"
      n_Upht_bus2_N = "n:Upht:bus2:N", "kV, Phase Technology dependent Voltage, Magnitude"
      n_um_bus1 = "n:um:bus1", "p.u., Average-Voltage, Magnitude"
      n_um_bus2 = "n:um:bus2", "p.u., Average-Voltage, Magnitude"
      n_Um_bus1 = "n:Um:bus1", "kV, Average-Voltage, Magnitude"
      n_Um_bus2 = "n:Um:bus2", "kV, Average-Voltage, Magnitude"
      n_uphtmin_bus1 = "n:uphtmin:bus1", "p.u., Minimum of Phase Technology dependent Voltage, Magnitude"
      n_uphtmin_bus2 = "n:uphtmin:bus2", "p.u., Minimum of Phase Technology dependent Voltage, Magnitude"
      n_uphtmax_bus1 = "n:uphtmax:bus1", "p.u., Maximum of Phase Technology dependent Voltage, Magnitude"
      n_uphtmax_bus2 = "n:uphtmax:bus2", "p.u., Maximum of Phase Technology dependent Voltage, Magnitude"
      n_Pgen_bus1 = "n:Pgen:bus1", "MW, Generation, Active Power"
      n_Pgen_bus2 = "n:Pgen:bus2", "MW, Generation, Active Power"
      n_Qgen_bus1 = "n:Qgen:bus1", "Mvar, Generation, Reactive Power"
      n_Qgen_bus2 = "n:Qgen:bus2", "Mvar, Generation, Reactive Power"
      n_Pmot_bus1 = "n:Pmot:bus1", "MW, Motor Load, Active Power"
      n_Pmot_bus2 = "n:Pmot:bus2", "MW, Motor Load, Active Power"
      n_Qmot_bus1 = "n:Qmot:bus1", "Mvar, Motor Load, Reactive Power"
      n_Qmot_bus2 = "n:Qmot:bus2", "Mvar, Motor Load, Reactive Power"
      n_Pload_bus1 = "n:Pload:bus1", "MW, General Load, Active Power"
      n_Pload_bus2 = "n:Pload:bus2", "MW, General Load, Active Power"
      n_Qload_bus1 = "n:Qload:bus1", "Mvar, General Load, Reactive Power"
      n_Qload_bus2 = "n:Qload:bus2", "Mvar, General Load, Reactive Power"
      n_Pcomp_bus1 = "n:Pcomp:bus1", "MW, Compensation (Losses)"
      n_Pcomp_bus2 = "n:Pcomp:bus2", "MW, Compensation (Losses)"
      n_Qcomp_bus1 = "n:Qcomp:bus1", "Mvar, Compensation"
      n_Qcomp_bus2 = "n:Qcomp:bus2", "Mvar, Compensation"
      n_Pnet_bus1 = "n:Pnet:bus1", "MW, External Networks, Active Power"
      n_Pnet_bus2 = "n:Pnet:bus2", "MW, External Networks, Active Power"
      n_Qnet_bus1 = "n:Qnet:bus1", "Mvar, External Networks, Reactive Power"
      n_Qnet_bus2 = "n:Qnet:bus2", "Mvar, External Networks, Reactive Power"
      n_Pflow_bus1 = "n:Pflow:bus1", "MW, Power Flow, Active Power"
      n_Pflow_bus2 = "n:Pflow:bus2", "MW, Power Flow, Active Power"
      n_Qflow_bus1 = "n:Qflow:bus1", "Mvar, Power Flow, Reactive Power"
      n_Qflow_bus2 = "n:Qflow:bus2", "Mvar, Power Flow, Reactive Power"
      n_Pout_bus1 = "n:Pout:bus1", "MW, Outgoing Flow, Active Power"
      n_Pout_bus2 = "n:Pout:bus2", "MW, Outgoing Flow, Active Power"
      n_Qout_bus1 = "n:Qout:bus1", "Mvar, Outgoing Power, Reactive Power"
      n_Qout_bus2 = "n:Qout:bus2", "Mvar, Outgoing Power, Reactive Power"
      n_Sout_bus1 = "n:Sout:bus1", "MVA, Outgoing Power, Apparent Power"
      n_Sout_bus2 = "n:Sout:bus2", "MVA, Outgoing Power, Apparent Power"
      n_cosphiout_bus1 = "n:cosphiout:bus1", "Outgoing Power, Power Factor"
      n_cosphiout_bus2 = "n:cosphiout:bus2", "Outgoing Power, Power Factor"
      n_Pbalance_bus1 = "n:Pbalance:bus1", "MW, Active Power Balance (=0)"
      n_Pbalance_bus2 = "n:Pbalance:bus2", "MW, Active Power Balance (=0)"
      n_Qbalance_bus1 = "n:Qbalance:bus1", "Mvar, Reactive Power Balance (=0)"
      n_Qbalance_bus2 = "n:Qbalance:bus2", "Mvar, Reactive Power Balance (=0)"
      n_ubfac_bus1 = "n:ubfac:bus1", "%, Unbalance Factor"
      n_ubfac_bus2 = "n:ubfac:bus2", "%, Unbalance Factor"
      n_LossPdown_bus1 = "n:LossPdown:bus1", "MW, Losses, downstream"
      n_LossPdown_bus2 = "n:LossPdown:bus2", "MW, Losses, downstream"
      n_LossQdown_bus1 = "n:LossQdown:bus1", "Mvar, Losses, downstream (Reactive Power)"
      n_LossQdown_bus2 = "n:LossQdown:bus2", "Mvar, Losses, downstream (Reactive Power)"
      n_LossPdownload_bus1 = "n:LossPdownload:bus1", "MW, Load losses, downstream"
      n_LossPdownload_bus2 = "n:LossPdownload:bus2", "MW, Load losses, downstream"
      n_LossQdownload_bus1 = "n:LossQdownload:bus1", "Mvar, Load losses, downstream"
      n_LossQdownload_bus2 = "n:LossQdownload:bus2", "Mvar, Load losses, downstream"
      n_LossPdownnoload_bus1 = "n:LossPdownnoload:bus1", "MW, No load losses, downstream"
      n_LossPdownnoload_bus2 = "n:LossPdownnoload:bus2", "MW, No load losses, downstream"
      n_LossQdownnoload_bus1 = "n:LossQdownnoload:bus1", "Mvar, No load losses, downstream"
      n_LossQdownnoload_bus2 = "n:LossQdownnoload:bus2", "Mvar, No load losses, downstream"
      c_zline_r = "c:zline:r", "p.u., Impedance matrix, Real Part"
      c_zline_i = "c:zline:i", "p.u., Impedance matrix, Imaginary Part"
      c_zline_phi = "c:zline:phi", "deg, Impedance matrix, Angle"
      c_zline_mag = "c:zline:mag", "p.u., Impedance matrix, Magnitude"
      c_zline_phirad = "c:zline:phirad", "rad, Impedance matrix, Angle"
      c_yline_r = "c:yline:r", "p.u., Admittance matrix, Real Part"
      c_yline_i = "c:yline:i", "p.u., Admittance matrix, Imaginary Part"
      c_yline_phi = "c:yline:phi", "deg, Admittance matrix, Angle"
      c_yline_mag = "c:yline:mag", "p.u., Admittance matrix, Magnitude"
      c_yline_phirad = "c:yline:phirad", "rad, Admittance matrix, Angle"
      c_frnom = "c:frnom", "Hz, Nominal frequency"
      c_hpi = "c:hpi", "rad/s, Nominal angular frequency"
      c_loadingPh_A = "c:loadingPh:A", "%, Loading"
      c_loadingPh_B = "c:loadingPh:B", "%, Loading"
      c_loadingPh_C = "c:loadingPh:C", "%, Loading"
      c_loadingPh_N = "c:loadingPh:N", "%, Loading"
      c_loading = "c:loading", "%, Loading, max."
      c_loading_st = "c:loading_st", "%, Short Term Loading"
      c_CloadPh_A = "c:CloadPh:A", "Mvar, Capacitive Losses"
      c_CloadPh_B = "c:CloadPh:B", "Mvar, Capacitive Losses"
      c_CloadPh_C = "c:CloadPh:C", "Mvar, Capacitive Losses"
      c_CloadPh_N = "c:CloadPh:N", "Mvar, Capacitive Losses"
      c_Cload = "c:Cload", "Mvar, Capacitive Losses, total"
      c_GloadPh_A = "c:GloadPh:A", "kW, Active no load losses"
      c_GloadPh_B = "c:GloadPh:B", "kW, Active no load losses"
      c_GloadPh_C = "c:GloadPh:C", "kW, Active no load losses"
      c_GloadPh_N = "c:GloadPh:N", "kW, Active no load losses"
      c_Gload = "c:Gload", "kW, Active no load losses, total"
      c_LossesPh_A = "c:LossesPh:A", "kW, Losses"
      c_LossesPh_B = "c:LossesPh:B", "kW, Losses"
      c_LossesPh_C = "c:LossesPh:C", "kW, Losses"
      c_LossesPh_N = "c:LossesPh:N", "kW, Losses"
      c_Losses = "c:Losses", "kW, Losses, total"
      c_maxload = "c:maxload", "%, Max. Loading"
      c_curnom = "c:curnom", "kA, Nominal Current"
      c_rtfac = "c:rtfac", "Temperature Factor"
      c_du_A = "c:du:A", "p.u., Voltage Drop"
      c_du_B = "c:du:B", "p.u., Voltage Drop"
      c_du_C = "c:du:C", "p.u., Voltage Drop"
      c_du_N = "c:du:N", "p.u., Voltage Drop"
      c_dupc_A = "c:dupc:A", "%, Voltage Drop"
      c_dupc_B = "c:dupc:B", "%, Voltage Drop"
      c_dupc_C = "c:dupc:C", "%, Voltage Drop"
      c_dupc_N = "c:dupc:N", "%, Voltage Drop"
      c_dphiu_A = "c:dphiu:A", "deg, Voltage Drop Angle"
      c_dphiu_B = "c:dphiu:B", "deg, Voltage Drop Angle"
      c_dphiu_C = "c:dphiu:C", "deg, Voltage Drop Angle"
      c_dphiu_N = "c:dphiu:N", "deg, Voltage Drop Angle"
      c_du1 = "c:du1", "p.u., Positive Sequence Voltage Drop"
      c_du1pc = "c:du1pc", "%, Positive Sequence Voltage Drop"
      c_dphiu1 = "c:dphiu1", "deg, Positive Sequence Voltage Drop Angle"
      c_Ploss = "c:Ploss", "MW, Losses (total)"
      c_Qloss = "c:Qloss", "Mvar, Reactive-Losses (total)"
      c_Plossld = "c:Plossld", "MW, Losses (load)"
      c_Qlossld = "c:Qlossld", "Mvar, Reactive-Losses (load)"
      c_Plossnld = "c:Plossnld", "MW, Losses (no load)"
      c_Qlossnld = "c:Qlossnld", "Mvar, Reactive-Losses (no load)"
      c_ncust = "c:ncust", "Number of Customers"
      c_Sl_nom = "c:Sl_nom", "MVA, Load, variable part, Nominal Apparent Power"
      c_ildlv = "c:ildlv", "Line Loads Present"
      c_Sldlv = "c:Sldlv", "kVA, Load, Apparent Power"
      c_Pldlv = "c:Pldlv", "kW, Load, Active Power"
      c_Qldlv = "c:Qldlv", "kvar, Load, Reactive Power"
      c_Ildlv = "c:Ildlv", "A, Load, Current"
      c_cosphildlv = "c:cosphildlv", "k, Load, Power Factor"
      c_Sl_a_r = "c:Sl_a:r", "MW, Load, variable part, Apparent Power, Phase A, Real Part"
      c_Sl_a_i = "c:Sl_a:i", "Mvar, Load, variable part, Apparent Power, Phase A, Imaginary Part"
      c_Sl_a_phi = "c:Sl_a:phi", "deg, Load, variable part, Apparent Power, Phase A, Angle"
      c_Sl_a_mag = "c:Sl_a:mag", "MVA, Load, variable part, Apparent Power, Phase A, Magnitude"
      c_Sl_a_phirad = "c:Sl_a:phirad", "rad, Load, variable part, Apparent Power, Phase A, Angle"
      c_Sl_b_r = "c:Sl_b:r", "MW, Load, variable part, Apparent Power, Phase B, Real Part"
      c_Sl_b_i = "c:Sl_b:i", "Mvar, Load, variable part, Apparent Power, Phase B, Imaginary Part"
      c_Sl_b_phi = "c:Sl_b:phi", "deg, Load, variable part, Apparent Power, Phase B, Angle"
      c_Sl_b_mag = "c:Sl_b:mag", "MVA, Load, variable part, Apparent Power, Phase B, Magnitude"
      c_Sl_b_phirad = "c:Sl_b:phirad", "rad, Load, variable part, Apparent Power, Phase B, Angle"
      c_Sl_c_r = "c:Sl_c:r", "MW, Load, variable part, Apparent Power, Phase C, Real Part"
      c_Sl_c_i = "c:Sl_c:i", "Mvar, Load, variable part, Apparent Power, Phase C, Imaginary Part"
      c_Sl_c_phi = "c:Sl_c:phi", "deg, Load, variable part, Apparent Power, Phase C, Angle"
      c_Sl_c_mag = "c:Sl_c:mag", "MVA, Load, variable part, Apparent Power, Phase C, Magnitude"
      c_Sl_c_phirad = "c:Sl_c:phirad", "rad, Load, variable part, Apparent Power, Phase C, Angle"
      c_Imax = "c:Imax", "kA, Maximum Current"
      e_cNrCust = "e:cNrCust", "Number of Customers"
      e_cSmax = "e:cSmax", "kVA, Max. Load"
      e_cSav = "e:cSav", "kVA, Average Load"
      e_ccosphi = "e:ccosphi", "Power Factor"
      e_maxload = "e:maxload", "%, Max. loading"
      e_deltau1 = "e:deltau1", "p.u., Voltage Difference across Line"
      e_deltaphiu1 = "e:deltaphiu1", "deg, Voltage Angle Difference across Line"
      e_lossAssign = "e:lossAssign", "Loss assignment"
      e_Top = "e:Top", "degC, Operating Temperature"
      e_Imaxlim = "e:Imaxlim", "kA, Max. permissible current"
      e_cpFeed = "e:cpFeed", "Feeder"
      e_ciDist = "e:ciDist", "Distance from infeed in number of buses"
      e_ciLater = "e:ciLater", "Lateral Index"
      e_ciDistRoot = "e:ciDistRoot", "Distance from first infeed in number of buses"
      e_ciDistAll = "e:ciDistAll", "Distance from infeed in number of buses including switches"
      e_ciDistAllRoot = "e:ciDistAllRoot", "Distance from first infeed in number of buses including switches"
      t_tmax = "t:tmax", "degC, Max. Operating Temperature"
      t_rline = "t:rline", "Ohm/km, AC-Resistance R'(20°C)"
      t_rline_tmax = "t:rline_tmax", "Ohm/km, AC-Resistance R' at max. operating temperature"
      t_alpha = "t:alpha", "1/K, Temperature Coefficient"
      t_mlei = "t:mlei", "Conductor Material"
      t_bline = "t:bline", "uS/km, Susceptance B'"
      t_cline = "t:cline", "uF/km, Capacitance C'"
      t_tline = "t:tline", "Ins. Factor"
      t_gline = "t:gline", "uS/km, Conductance G'"
      t_bline0 = "t:bline0", "uS/km, Susceptance B0'"
      t_cline0 = "t:cline0", "uF/km, Capacitance C0'"
      t_Ices = "t:Ices", "A/km, Earth-Fault Current"
      t_tline0 = "t:tline0", "Ins. Factor"
      t_gline0 = "t:gline0", "uS/km, Conductance G0'"
      t_bnline = "t:bnline", "uS/km, Susceptance Bn'"
      t_cnline = "t:cnline", "uF/km, Capacitance Cn'"
      t_bpnline = "t:bpnline", "uS/km, Susceptance Bpn'"
      t_cpnline = "t:cpnline", "uF/km, Capacitance Cpn'"
      r_cpFeed = "r:cpFeed", "Feeder"
      m_Ikss_bus1 = "m:Ikss:bus1", "kA, Initial Short-Circuit Current"
      m_Ikss_bus2 = "m:Ikss:bus2", "kA, Initial Short-Circuit Current"
      m_ikss_bus1 = "m:ikss:bus1", "p.u., Initial Short-Circuit Current, Magnitude"
      m_ikss_bus2 = "m:ikss:bus2", "p.u., Initial Short-Circuit Current, Magnitude"
      m_phii_bus1 = "m:phii:bus1", "deg, Phase Current, Angle"
      m_phii_bus2 = "m:phii:bus2", "deg, Phase Current, Angle"
      m_maxIkss_bus1 = "m:maxIkss:bus1", "kA, Max. Initial Short-Circuit Current"
      m_maxIkss_bus2 = "m:maxIkss:bus2", "kA, Max. Initial Short-Circuit Current"
      m_Skss_bus1 = "m:Skss:bus1", "MVA, Initial Short-Circuit Power"
      m_Skss_bus2 = "m:Skss:bus2", "MVA, Initial Short-Circuit Power"
      m_ip_bus1 = "m:ip:bus1", "kA, Peak Short-Circuit Current"
      m_ip_bus2 = "m:ip:bus2", "kA, Peak Short-Circuit Current"
      m_maxip_bus1 = "m:maxip:bus1", "kA, Max. Peak Short-Circuit Current"
      m_maxip_bus2 = "m:maxip:bus2", "kA, Max. Peak Short-Circuit Current"
      m_maxIb_bus1 = "m:maxIb:bus1", "kA, Max. Breaking Current"
      m_maxIb_bus2 = "m:maxIb:bus2", "kA, Max. Breaking Current"
      m_maxIth_bus1 = "m:maxIth:bus1", "kA, Max. Thermal equivalent Short-Circuit Current"
      m_maxIth_bus2 = "m:maxIth:bus2", "kA, Max. Thermal equivalent Short-Circuit Current"
      m_Ib_bus1 = "m:Ib:bus1", "kA, Short-Circuit Breaking Current"
      m_Ib_bus2 = "m:Ib:bus2", "kA, Short-Circuit Breaking Current"
      m_Sb_bus1 = "m:Sb:bus1", "MVA, Short-Circuit Breaking Power"
      m_Sb_bus2 = "m:Sb:bus2", "MVA, Short-Circuit Breaking Power"
      m_Ik_bus1 = "m:Ik:bus1", "kA, Steady-State Short-Circuit Current"
      m_Ik_bus2 = "m:Ik:bus2", "kA, Steady-State Short-Circuit Current"
      m_Ith_bus1 = "m:Ith:bus1", "kA, Thermal equivalent Short-Circuit Current"
      m_Ith_bus2 = "m:Ith:bus2", "kA, Thermal equivalent Short-Circuit Current"
      m_I_bus1 = "m:I:bus1", "kA, Short-Circuit Current"
      m_I_bus2 = "m:I:bus2", "kA, Short-Circuit Current"
      m_R_bus1 = "m:R:bus1", "Ohm, Fault Impedance, Real Part"
      m_R_bus2 = "m:R:bus2", "Ohm, Fault Impedance, Real Part"
      m_X_bus1 = "m:X:bus1", "Ohm, Fault Impedance, Imaginary Part"
      m_X_bus2 = "m:X:bus2", "Ohm, Fault Impedance, Imaginary Part"
      m_Z_bus1 = "m:Z:bus1", "Ohm, Fault Impedance, Magnitude"
      m_Z_bus2 = "m:Z:bus2", "Ohm, Fault Impedance, Magnitude"
      m_phiz_bus1 = "m:phiz:bus1", "deg, Fault Impedance, Angle"
      m_phiz_bus2 = "m:phiz:bus2", "deg, Fault Impedance, Angle"
      m_zSbase_bus1 = "m:zSbase:bus1", "p.u., Fault Impedance, Magnitude"
      m_zSbase_bus2 = "m:zSbase:bus2", "p.u., Fault Impedance, Magnitude"
      m_XtoR_bus1 = "m:XtoR:bus1", "X/R ratio"
      m_XtoR_bus2 = "m:XtoR:bus2", "X/R ratio"
      m_Zs_bus1 = "m:Zs:bus1", "Ohm, Source Impedance, Magnitude"
      m_Zs_bus2 = "m:Zs:bus2", "Ohm, Source Impedance, Magnitude"
      m_phizs_bus1 = "m:phizs:bus1", "deg, Source Impedance, Angle"
      m_phizs_bus2 = "m:phizs:bus2", "deg, Source Impedance, Angle"
      m_Rs_bus1 = "m:Rs:bus1", "Ohm, Source Impedance, Real Part"
      m_Rs_bus2 = "m:Rs:bus2", "Ohm, Source Impedance, Real Part"
      m_Xs_bus1 = "m:Xs:bus1", "Ohm, Source Impedance, Imaginary Part"
      m_Xs_bus2 = "m:Xs:bus2", "Ohm, Source Impedance, Imaginary Part"
      m_phiui_bus1 = "m:phiui:bus1", "deg, Angle between Voltage and Current"
      m_phiui_bus2 = "m:phiui:bus2", "deg, Angle between Voltage and Current"
      m_cosphiui_bus1 = "m:cosphiui:bus1", "cosphi"
      m_cosphiui_bus2 = "m:cosphiui:bus2", "cosphi"
      m_P_bus1 = "m:P:bus1", "MW, Active Power"
      m_P_bus2 = "m:P:bus2", "MW, Active Power"
      m_Q_bus1 = "m:Q:bus1", "Mvar, Reactive Power"
      m_Q_bus2 = "m:Q:bus2", "Mvar, Reactive Power"
      m_BrkIpload_bus1 = "m:BrkIpload:bus1", "%, Breaker Loading (peak)"
      m_BrkIpload_bus2 = "m:BrkIpload:bus2", "%, Breaker Loading (peak)"
      m_BrkIbload_bus1 = "m:BrkIbload:bus1", "%, Breaker Loading (breaking)"
      m_BrkIbload_bus2 = "m:BrkIbload:bus2", "%, Breaker Loading (breaking)"
      m_BrkmaxIb_bus1 = "m:BrkmaxIb:bus1", "kA, Max. Breaking Current (breaker time)"
      m_BrkmaxIb_bus2 = "m:BrkmaxIb:bus2", "kA, Max. Breaking Current (breaker time)"
      n_Ikss_bus1 = "n:Ikss:bus1", "kA, Initial Short-Circuit Current"
      n_Ikss_bus2 = "n:Ikss:bus2", "kA, Initial Short-Circuit Current"
      n_I_bus1 = "n:I:bus1", "kA, Short-Circuit Current"
      n_I_bus2 = "n:I:bus2", "kA, Short-Circuit Current"
      n_phii_bus1 = "n:phii:bus1", "deg, Phase Current, Angle"
      n_phii_bus2 = "n:phii:bus2", "deg, Phase Current, Angle"
      n_Skss_bus1 = "n:Skss:bus1", "MVA, Initial Short-Circuit Power"
      n_Skss_bus2 = "n:Skss:bus2", "MVA, Initial Short-Circuit Power"
      n_ip_bus1 = "n:ip:bus1", "kA, Peak Short-Circuit Current"
      n_ip_bus2 = "n:ip:bus2", "kA, Peak Short-Circuit Current"
      n_Ib_bus1 = "n:Ib:bus1", "kA, Short-Circuit Breaking Current"
      n_Ib_bus2 = "n:Ib:bus2", "kA, Short-Circuit Breaking Current"
      n_Sb_bus1 = "n:Sb:bus1", "MVA, Short-Circuit Breaking Power"
      n_Sb_bus2 = "n:Sb:bus2", "MVA, Short-Circuit Breaking Power"
      n_Ik_bus1 = "n:Ik:bus1", "kA, Steady-State Short-Circuit Current"
      n_Ik_bus2 = "n:Ik:bus2", "kA, Steady-State Short-Circuit Current"
      n_Ith_bus1 = "n:Ith:bus1", "kA, Thermal equivalent Short-Circuit Current"
      n_Ith_bus2 = "n:Ith:bus2", "kA, Thermal equivalent Short-Circuit Current"
      n_R_bus1 = "n:R:bus1", "Ohm, Short-Circuit Impedance, Real Part"
      n_R_bus2 = "n:R:bus2", "Ohm, Short-Circuit Impedance, Real Part"
      n_X_bus1 = "n:X:bus1", "Ohm, Short-Circuit Impedance, Imaginary Part"
      n_X_bus2 = "n:X:bus2", "Ohm, Short-Circuit Impedance, Imaginary Part"
      n_Z_bus1 = "n:Z:bus1", "Ohm, Short-Circuit Impedance, Magnitude"
      n_Z_bus2 = "n:Z:bus2", "Ohm, Short-Circuit Impedance, Magnitude"
      n_phiz_bus1 = "n:phiz:bus1", "deg, Short-Circuit Impedance, Angle"
      n_phiz_bus2 = "n:phiz:bus2", "deg, Short-Circuit Impedance, Angle"
      n_rSbase_bus1 = "n:rSbase:bus1", "p.u., Short-Circuit Impedance, Real Part"
      n_rSbase_bus2 = "n:rSbase:bus2", "p.u., Short-Circuit Impedance, Real Part"
      n_xSbase_bus1 = "n:xSbase:bus1", "p.u., Short-Circuit Impedance, Imaginary Part"
      n_xSbase_bus2 = "n:xSbase:bus2", "p.u., Short-Circuit Impedance, Imaginary Part"
      n_zSbase_bus1 = "n:zSbase:bus1", "p.u., Short-Circuit Impedance, Magnitude"
      n_zSbase_bus2 = "n:zSbase:bus2", "p.u., Short-Circuit Impedance, Magnitude"
      n_XtoR_bus1 = "n:XtoR:bus1", "X/R ratio (peak)"
      n_XtoR_bus2 = "n:XtoR:bus2", "X/R ratio (peak)"
      n_RtoX_bus1 = "n:RtoX:bus1", "R/X ratio (peak)"
      n_RtoX_bus2 = "n:RtoX:bus2", "R/X ratio (peak)"
      n_ur_bus1 = "n:ur:bus1", "p.u., Voltage, Real Part"
      n_ur_bus2 = "n:ur:bus2", "p.u., Voltage, Real Part"
      n_ui_bus1 = "n:ui:bus1", "p.u., Voltage, Imaginary Part"
      n_ui_bus2 = "n:ui:bus2", "p.u., Voltage, Imaginary Part"
      n_u_bus1 = "n:u:bus1", "p.u., Voltage, Magnitude"
      n_u_bus2 = "n:u:bus2", "p.u., Voltage, Magnitude"
      n_upc_bus1 = "n:upc:bus1", "%, Voltage, Magnitude"
      n_upc_bus2 = "n:upc:bus2", "%, Voltage, Magnitude"
      n_U_bus1 = "n:U:bus1", "kV, Line-Ground Voltage, Magnitude"
      n_U_bus2 = "n:U:bus2", "kV, Line-Ground Voltage, Magnitude"
      n_Ul_bus1 = "n:Ul:bus1", "kV, Line-Line Voltage, Magnitude"
      n_Ul_bus2 = "n:Ul:bus2", "kV, Line-Line Voltage, Magnitude"
      n_phiu_bus1 = "n:phiu:bus1", "deg, Voltage, Angle"
      n_phiu_bus2 = "n:phiu:bus2", "deg, Voltage, Angle"
      n_du_bus1 = "n:du:bus1", "%, Voltage Deviation"
      n_du_bus2 = "n:du:bus2", "%, Voltage Deviation"
      n_urpref_bus1 = "n:urpref:bus1", "p.u., Pre-fault Voltage, Real Part"
      n_urpref_bus2 = "n:urpref:bus2", "p.u., Pre-fault Voltage, Real Part"
      n_uipref_bus1 = "n:uipref:bus1", "p.u., Pre-fault Voltage, Imaginary Part"
      n_uipref_bus2 = "n:uipref:bus2", "p.u., Pre-fault Voltage, Imaginary Part"
      n_upref_bus1 = "n:upref:bus1", "p.u., Pre-fault Voltage, Magnitude"
      n_upref_bus2 = "n:upref:bus2", "p.u., Pre-fault Voltage, Magnitude"
      n_uprefpc_bus1 = "n:uprefpc:bus1", "%, Pre-fault Voltage, Magnitude"
      n_uprefpc_bus2 = "n:uprefpc:bus2", "%, Pre-fault Voltage, Magnitude"
      n_Upref_bus1 = "n:Upref:bus1", "kV, Line-Ground Pre-fault Voltage, Magnitude"
      n_Upref_bus2 = "n:Upref:bus2", "kV, Line-Ground Pre-fault Voltage, Magnitude"
      n_Ulpref_bus1 = "n:Ulpref:bus1", "kV, Line-Line Pre-fault Voltage, Magnitude"
      n_Ulpref_bus2 = "n:Ulpref:bus2", "kV, Line-Line Pre-fault Voltage, Magnitude"
      n_phiupref_bus1 = "n:phiupref:bus1", "deg, Pre-fault Voltage, Angle"
      n_phiupref_bus2 = "n:phiupref:bus2", "deg, Pre-fault Voltage, Angle"
      n_phiui_bus1 = "n:phiui:bus1", "deg, Angle between Voltage and Current"
      n_phiui_bus2 = "n:phiui:bus2", "deg, Angle between Voltage and Current"
      n_cfac_bus1 = "n:cfac:bus1", "Voltage Factor"
      n_cfac_bus2 = "n:cfac:bus2", "Voltage Factor"
      n_idc_bus1 = "n:idc:bus1", "kA, Decaying Component"
      n_idc_bus2 = "n:idc:bus2", "kA, Decaying Component"
      n_Iasy_bus1 = "n:Iasy:bus1", "kA, Asymmetrical Breaking Current"
      n_Iasy_bus2 = "n:Iasy:bus2", "kA, Asymmetrical Breaking Current"
      n_Ithload_bus1 = "n:Ithload:bus1", "%, Loading, Thermal equivalent Current"
      n_Ithload_bus2 = "n:Ithload:bus2", "%, Loading, Thermal equivalent Current"
      n_Ipload_bus1 = "n:Ipload:bus1", "%, Loading, Peak Short-Circuit Current"
      n_Ipload_bus2 = "n:Ipload:bus2", "%, Loading, Peak Short-Circuit Current"
      n_Ithrtk_bus1 = "n:Ithrtk:bus1", "kA, Rated Short-Time Current (Tk)"
      n_Ithrtk_bus2 = "n:Ithrtk:bus2", "kA, Rated Short-Time Current (Tk)"
      n_Tn_bus1 = "n:Tn:bus1", "ms, Grid Time Constant"
      n_Tn_bus2 = "n:Tn:bus2", "ms, Grid Time Constant"
      n_XtoR_b_bus1 = "n:XtoR_b:bus1", "X/R ratio (dc)"
      n_XtoR_b_bus2 = "n:XtoR_b:bus2", "X/R ratio (dc)"
      n_RtoX_b_bus1 = "n:RtoX_b:bus1", "R/X ratio (dc)"
      n_RtoX_b_bus2 = "n:RtoX_b:bus2", "R/X ratio (dc)"
      n_fctof_b_bus1 = "n:fctof_b:bus1", "Equivalent Frequency fc/f (dc)"
      n_fctof_b_bus2 = "n:fctof_b:bus2", "Equivalent Frequency fc/f (dc)"
      c_Ithr = "c:Ithr", "kA, Rated Short-Time (1s) Current (Conductor)"
      c_Ithrtk = "c:Ithrtk", "kA, Rated Short-Time Current (Tk)"
      c_Ithmax = "c:Ithmax", "kA, Max. Thermal equivalent Short-Circuit Current"
      t_rtemp = "t:rtemp", "degC, Max. End Temperature"
      t_Ithr = "t:Ithr", "kA, Rated Short-Time (1s) Current (Conductor)"
      m_Ikss_bus1_A = "m:Ikss:bus1:A", "kA, Initial Short-Circuit Current"
      m_Ikss_bus1_B = "m:Ikss:bus1:B", "kA, Initial Short-Circuit Current"
      m_Ikss_bus1_C = "m:Ikss:bus1:C", "kA, Initial Short-Circuit Current"
      m_Ikss_bus1_N = "m:Ikss:bus1:N", "kA, Initial Short-Circuit Current"
      m_Ikss_bus2_A = "m:Ikss:bus2:A", "kA, Initial Short-Circuit Current"
      m_Ikss_bus2_B = "m:Ikss:bus2:B", "kA, Initial Short-Circuit Current"
      m_Ikss_bus2_C = "m:Ikss:bus2:C", "kA, Initial Short-Circuit Current"
      m_Ikss_bus2_N = "m:Ikss:bus2:N", "kA, Initial Short-Circuit Current"
      m_ikss_bus1_A = "m:ikss:bus1:A", "p.u., Initial Short-Circuit Current, Magnitude"
      m_ikss_bus1_B = "m:ikss:bus1:B", "p.u., Initial Short-Circuit Current, Magnitude"
      m_ikss_bus1_C = "m:ikss:bus1:C", "p.u., Initial Short-Circuit Current, Magnitude"
      m_ikss_bus1_N = "m:ikss:bus1:N", "p.u., Initial Short-Circuit Current, Magnitude"
      m_ikss_bus2_A = "m:ikss:bus2:A", "p.u., Initial Short-Circuit Current, Magnitude"
      m_ikss_bus2_B = "m:ikss:bus2:B", "p.u., Initial Short-Circuit Current, Magnitude"
      m_ikss_bus2_C = "m:ikss:bus2:C", "p.u., Initial Short-Circuit Current, Magnitude"
      m_ikss_bus2_N = "m:ikss:bus2:N", "p.u., Initial Short-Circuit Current, Magnitude"
      m_Skss_bus1_A = "m:Skss:bus1:A", "MVA, Initial Short-Circuit Power"
      m_Skss_bus1_B = "m:Skss:bus1:B", "MVA, Initial Short-Circuit Power"
      m_Skss_bus1_C = "m:Skss:bus1:C", "MVA, Initial Short-Circuit Power"
      m_Skss_bus1_N = "m:Skss:bus1:N", "MVA, Initial Short-Circuit Power"
      m_Skss_bus2_A = "m:Skss:bus2:A", "MVA, Initial Short-Circuit Power"
      m_Skss_bus2_B = "m:Skss:bus2:B", "MVA, Initial Short-Circuit Power"
      m_Skss_bus2_C = "m:Skss:bus2:C", "MVA, Initial Short-Circuit Power"
      m_Skss_bus2_N = "m:Skss:bus2:N", "MVA, Initial Short-Circuit Power"
      m_Skssx3_bus1_A = "m:Skssx3:bus1:A", "MVA, 3*Initial Short-Circuit Power"
      m_Skssx3_bus1_B = "m:Skssx3:bus1:B", "MVA, 3*Initial Short-Circuit Power"
      m_Skssx3_bus1_C = "m:Skssx3:bus1:C", "MVA, 3*Initial Short-Circuit Power"
      m_Skssx3_bus1_N = "m:Skssx3:bus1:N", "MVA, 3*Initial Short-Circuit Power"
      m_Skssx3_bus2_A = "m:Skssx3:bus2:A", "MVA, 3*Initial Short-Circuit Power"
      m_Skssx3_bus2_B = "m:Skssx3:bus2:B", "MVA, 3*Initial Short-Circuit Power"
      m_Skssx3_bus2_C = "m:Skssx3:bus2:C", "MVA, 3*Initial Short-Circuit Power"
      m_Skssx3_bus2_N = "m:Skssx3:bus2:N", "MVA, 3*Initial Short-Circuit Power"
      m_ip_bus1_A = "m:ip:bus1:A", "kA, Peak Short-Circuit Current"
      m_ip_bus1_B = "m:ip:bus1:B", "kA, Peak Short-Circuit Current"
      m_ip_bus1_C = "m:ip:bus1:C", "kA, Peak Short-Circuit Current"
      m_ip_bus1_N = "m:ip:bus1:N", "kA, Peak Short-Circuit Current"
      m_ip_bus2_A = "m:ip:bus2:A", "kA, Peak Short-Circuit Current"
      m_ip_bus2_B = "m:ip:bus2:B", "kA, Peak Short-Circuit Current"
      m_ip_bus2_C = "m:ip:bus2:C", "kA, Peak Short-Circuit Current"
      m_ip_bus2_N = "m:ip:bus2:N", "kA, Peak Short-Circuit Current"
      m_Z_bus1_A = "m:Z:bus1:A", "Ohm, Fault Impedance, Magnitude"
      m_Z_bus1_B = "m:Z:bus1:B", "Ohm, Fault Impedance, Magnitude"
      m_Z_bus1_C = "m:Z:bus1:C", "Ohm, Fault Impedance, Magnitude"
      m_Z_bus1_N = "m:Z:bus1:N", "Ohm, Fault Impedance, Magnitude"
      m_Z_bus2_A = "m:Z:bus2:A", "Ohm, Fault Impedance, Magnitude"
      m_Z_bus2_B = "m:Z:bus2:B", "Ohm, Fault Impedance, Magnitude"
      m_Z_bus2_C = "m:Z:bus2:C", "Ohm, Fault Impedance, Magnitude"
      m_Z_bus2_N = "m:Z:bus2:N", "Ohm, Fault Impedance, Magnitude"
      m_phiz_bus1_A = "m:phiz:bus1:A", "deg, Fault Impedance, Angle"
      m_phiz_bus1_B = "m:phiz:bus1:B", "deg, Fault Impedance, Angle"
      m_phiz_bus1_C = "m:phiz:bus1:C", "deg, Fault Impedance, Angle"
      m_phiz_bus1_N = "m:phiz:bus1:N", "deg, Fault Impedance, Angle"
      m_phiz_bus2_A = "m:phiz:bus2:A", "deg, Fault Impedance, Angle"
      m_phiz_bus2_B = "m:phiz:bus2:B", "deg, Fault Impedance, Angle"
      m_phiz_bus2_C = "m:phiz:bus2:C", "deg, Fault Impedance, Angle"
      m_phiz_bus2_N = "m:phiz:bus2:N", "deg, Fault Impedance, Angle"
      m_Zl_bus1_A = "m:Zl:bus1:A", "Ohm, Line-Line Fault Impedance, Magnitude"
      m_Zl_bus1_B = "m:Zl:bus1:B", "Ohm, Line-Line Fault Impedance, Magnitude"
      m_Zl_bus1_C = "m:Zl:bus1:C", "Ohm, Line-Line Fault Impedance, Magnitude"
      m_Zl_bus1_N = "m:Zl:bus1:N", "Ohm, Line-Line Fault Impedance, Magnitude"
      m_Zl_bus2_A = "m:Zl:bus2:A", "Ohm, Line-Line Fault Impedance, Magnitude"
      m_Zl_bus2_B = "m:Zl:bus2:B", "Ohm, Line-Line Fault Impedance, Magnitude"
      m_Zl_bus2_C = "m:Zl:bus2:C", "Ohm, Line-Line Fault Impedance, Magnitude"
      m_Zl_bus2_N = "m:Zl:bus2:N", "Ohm, Line-Line Fault Impedance, Magnitude"
      m_phizl_bus1_A = "m:phizl:bus1:A", "deg, Line-Line Fault Impedance, Angle"
      m_phizl_bus1_B = "m:phizl:bus1:B", "deg, Line-Line Fault Impedance, Angle"
      m_phizl_bus1_C = "m:phizl:bus1:C", "deg, Line-Line Fault Impedance, Angle"
      m_phizl_bus1_N = "m:phizl:bus1:N", "deg, Line-Line Fault Impedance, Angle"
      m_phizl_bus2_A = "m:phizl:bus2:A", "deg, Line-Line Fault Impedance, Angle"
      m_phizl_bus2_B = "m:phizl:bus2:B", "deg, Line-Line Fault Impedance, Angle"
      m_phizl_bus2_C = "m:phizl:bus2:C", "deg, Line-Line Fault Impedance, Angle"
      m_phizl_bus2_N = "m:phizl:bus2:N", "deg, Line-Line Fault Impedance, Angle"
      m_Z0_bus1 = "m:Z0:bus1", "Ohm, Zero-Sequence Fault Impedance, Magnitude"
      m_Z0_bus2 = "m:Z0:bus2", "Ohm, Zero-Sequence Fault Impedance, Magnitude"
      m_phiz0_bus1 = "m:phiz0:bus1", "deg, Zero-Sequence Fault Impedance, Angle"
      m_phiz0_bus2 = "m:phiz0:bus2", "deg, Zero-Sequence Fault Impedance, Angle"
      m_z0Sbase_bus1 = "m:z0Sbase:bus1", "p.u., Zero-Sequence Fault Impedance, Magnitude"
      m_z0Sbase_bus2 = "m:z0Sbase:bus2", "p.u., Zero-Sequence Fault Impedance, Magnitude"
      m_cosphiu1i1_bus1 = "m:cosphiu1i1:bus1", "Positive-Sequence cosphi"
      m_cosphiu1i1_bus2 = "m:cosphiu1i1:bus2", "Positive-Sequence cosphi"
      m_I0x3watt_bus1 = "m:I0x3watt:bus1", "kA, Earth-Fault Residual Current"
      m_I0x3watt_bus2 = "m:I0x3watt:bus2", "kA, Earth-Fault Residual Current"
      m_I0x3var_bus1 = "m:I0x3var:bus1", "kA, Capacitive Earth-Fault Current"
      m_I0x3var_bus2 = "m:I0x3var:bus2", "kA, Capacitive Earth-Fault Current"
      m_Q0cap_bus1 = "m:Q0cap:bus1", "Mvar, Q0 capacitive"
      m_Q0cap_bus2 = "m:Q0cap:bus2", "Mvar, Q0 capacitive"
      m_Zfrel_bus1 = "m:Zfrel:bus1", "Ohm, Relay Impedance, Magnitude"
      m_Zfrel_bus2 = "m:Zfrel:bus2", "Ohm, Relay Impedance, Magnitude"
      m_phizfrel_bus1 = "m:phizfrel:bus1", "deg, Relay Impedance, Angle"
      m_phizfrel_bus2 = "m:phizfrel:bus2", "deg, Relay Impedance, Angle"
      m_Rfrel_bus1 = "m:Rfrel:bus1", "Ohm, Relay Impedance, Real Part"
      m_Rfrel_bus2 = "m:Rfrel:bus2", "Ohm, Relay Impedance, Real Part"
      m_Xfrel_bus1 = "m:Xfrel:bus1", "Ohm, Relay Impedance, Imaginary Part"
      m_Xfrel_bus2 = "m:Xfrel:bus2", "Ohm, Relay Impedance, Imaginary Part"
      m_Zf_bus1 = "m:Zf:bus1", "Ohm, Meas. Fault Impedance, Magnitude"
      m_Zf_bus2 = "m:Zf:bus2", "Ohm, Meas. Fault Impedance, Magnitude"
      m_phizf_bus1 = "m:phizf:bus1", "deg, Meas. Fault Impedance, Angle"
      m_phizf_bus2 = "m:phizf:bus2", "deg, Meas. Fault Impedance, Angle"
      n_Ikss_bus1_A = "n:Ikss:bus1:A", "kA, Initial Short-Circuit Current"
      n_Ikss_bus1_B = "n:Ikss:bus1:B", "kA, Initial Short-Circuit Current"
      n_Ikss_bus1_C = "n:Ikss:bus1:C", "kA, Initial Short-Circuit Current"
      n_Ikss_bus1_N = "n:Ikss:bus1:N", "kA, Initial Short-Circuit Current"
      n_Ikss_bus2_A = "n:Ikss:bus2:A", "kA, Initial Short-Circuit Current"
      n_Ikss_bus2_B = "n:Ikss:bus2:B", "kA, Initial Short-Circuit Current"
      n_Ikss_bus2_C = "n:Ikss:bus2:C", "kA, Initial Short-Circuit Current"
      n_Ikss_bus2_N = "n:Ikss:bus2:N", "kA, Initial Short-Circuit Current"
      n_I_bus1_A = "n:I:bus1:A", "kA, Short-Circuit Current"
      n_I_bus1_B = "n:I:bus1:B", "kA, Short-Circuit Current"
      n_I_bus1_C = "n:I:bus1:C", "kA, Short-Circuit Current"
      n_I_bus1_N = "n:I:bus1:N", "kA, Short-Circuit Current"
      n_I_bus2_A = "n:I:bus2:A", "kA, Short-Circuit Current"
      n_I_bus2_B = "n:I:bus2:B", "kA, Short-Circuit Current"
      n_I_bus2_C = "n:I:bus2:C", "kA, Short-Circuit Current"
      n_I_bus2_N = "n:I:bus2:N", "kA, Short-Circuit Current"
      n_phii_bus1_A = "n:phii:bus1:A", "deg, Phase Current, Angle"
      n_phii_bus1_B = "n:phii:bus1:B", "deg, Phase Current, Angle"
      n_phii_bus1_C = "n:phii:bus1:C", "deg, Phase Current, Angle"
      n_phii_bus1_N = "n:phii:bus1:N", "deg, Phase Current, Angle"
      n_phii_bus2_A = "n:phii:bus2:A", "deg, Phase Current, Angle"
      n_phii_bus2_B = "n:phii:bus2:B", "deg, Phase Current, Angle"
      n_phii_bus2_C = "n:phii:bus2:C", "deg, Phase Current, Angle"
      n_phii_bus2_N = "n:phii:bus2:N", "deg, Phase Current, Angle"
      n_Skss_bus1_A = "n:Skss:bus1:A", "MVA, Initial Short-Circuit Power"
      n_Skss_bus1_B = "n:Skss:bus1:B", "MVA, Initial Short-Circuit Power"
      n_Skss_bus1_C = "n:Skss:bus1:C", "MVA, Initial Short-Circuit Power"
      n_Skss_bus1_N = "n:Skss:bus1:N", "MVA, Initial Short-Circuit Power"
      n_Skss_bus2_A = "n:Skss:bus2:A", "MVA, Initial Short-Circuit Power"
      n_Skss_bus2_B = "n:Skss:bus2:B", "MVA, Initial Short-Circuit Power"
      n_Skss_bus2_C = "n:Skss:bus2:C", "MVA, Initial Short-Circuit Power"
      n_Skss_bus2_N = "n:Skss:bus2:N", "MVA, Initial Short-Circuit Power"
      n_Skssx3_bus1_A = "n:Skssx3:bus1:A", "MVA, 3*Initial Short-Circuit Power"
      n_Skssx3_bus1_B = "n:Skssx3:bus1:B", "MVA, 3*Initial Short-Circuit Power"
      n_Skssx3_bus1_C = "n:Skssx3:bus1:C", "MVA, 3*Initial Short-Circuit Power"
      n_Skssx3_bus1_N = "n:Skssx3:bus1:N", "MVA, 3*Initial Short-Circuit Power"
      n_Skssx3_bus2_A = "n:Skssx3:bus2:A", "MVA, 3*Initial Short-Circuit Power"
      n_Skssx3_bus2_B = "n:Skssx3:bus2:B", "MVA, 3*Initial Short-Circuit Power"
      n_Skssx3_bus2_C = "n:Skssx3:bus2:C", "MVA, 3*Initial Short-Circuit Power"
      n_Skssx3_bus2_N = "n:Skssx3:bus2:N", "MVA, 3*Initial Short-Circuit Power"
      n_ip_bus1_A = "n:ip:bus1:A", "kA, Peak Short-Circuit Current"
      n_ip_bus1_B = "n:ip:bus1:B", "kA, Peak Short-Circuit Current"
      n_ip_bus1_C = "n:ip:bus1:C", "kA, Peak Short-Circuit Current"
      n_ip_bus1_N = "n:ip:bus1:N", "kA, Peak Short-Circuit Current"
      n_ip_bus2_A = "n:ip:bus2:A", "kA, Peak Short-Circuit Current"
      n_ip_bus2_B = "n:ip:bus2:B", "kA, Peak Short-Circuit Current"
      n_ip_bus2_C = "n:ip:bus2:C", "kA, Peak Short-Circuit Current"
      n_ip_bus2_N = "n:ip:bus2:N", "kA, Peak Short-Circuit Current"
      n_Ib_bus1_A = "n:Ib:bus1:A", "kA, Short-Circuit Breaking Current"
      n_Ib_bus1_B = "n:Ib:bus1:B", "kA, Short-Circuit Breaking Current"
      n_Ib_bus1_C = "n:Ib:bus1:C", "kA, Short-Circuit Breaking Current"
      n_Ib_bus1_N = "n:Ib:bus1:N", "kA, Short-Circuit Breaking Current"
      n_Ib_bus2_A = "n:Ib:bus2:A", "kA, Short-Circuit Breaking Current"
      n_Ib_bus2_B = "n:Ib:bus2:B", "kA, Short-Circuit Breaking Current"
      n_Ib_bus2_C = "n:Ib:bus2:C", "kA, Short-Circuit Breaking Current"
      n_Ib_bus2_N = "n:Ib:bus2:N", "kA, Short-Circuit Breaking Current"
      n_Sb_bus1_A = "n:Sb:bus1:A", "MVA, Short-Circuit Breaking Power"
      n_Sb_bus1_B = "n:Sb:bus1:B", "MVA, Short-Circuit Breaking Power"
      n_Sb_bus1_C = "n:Sb:bus1:C", "MVA, Short-Circuit Breaking Power"
      n_Sb_bus1_N = "n:Sb:bus1:N", "MVA, Short-Circuit Breaking Power"
      n_Sb_bus2_A = "n:Sb:bus2:A", "MVA, Short-Circuit Breaking Power"
      n_Sb_bus2_B = "n:Sb:bus2:B", "MVA, Short-Circuit Breaking Power"
      n_Sb_bus2_C = "n:Sb:bus2:C", "MVA, Short-Circuit Breaking Power"
      n_Sb_bus2_N = "n:Sb:bus2:N", "MVA, Short-Circuit Breaking Power"
      n_Ith_bus1_A = "n:Ith:bus1:A", "kA, Thermal equivalent Short-Circuit Current"
      n_Ith_bus1_B = "n:Ith:bus1:B", "kA, Thermal equivalent Short-Circuit Current"
      n_Ith_bus1_C = "n:Ith:bus1:C", "kA, Thermal equivalent Short-Circuit Current"
      n_Ith_bus1_N = "n:Ith:bus1:N", "kA, Thermal equivalent Short-Circuit Current"
      n_Ith_bus2_A = "n:Ith:bus2:A", "kA, Thermal equivalent Short-Circuit Current"
      n_Ith_bus2_B = "n:Ith:bus2:B", "kA, Thermal equivalent Short-Circuit Current"
      n_Ith_bus2_C = "n:Ith:bus2:C", "kA, Thermal equivalent Short-Circuit Current"
      n_Ith_bus2_N = "n:Ith:bus2:N", "kA, Thermal equivalent Short-Circuit Current"
      n_Ithmax_bus1 = "n:Ithmax:bus1", "kA, Max. Thermal equivalent Short-Circuit Current"
      n_Ithmax_bus2 = "n:Ithmax:bus2", "kA, Max. Thermal equivalent Short-Circuit Current"
      n_Ik_bus1_A = "n:Ik:bus1:A", "kA, Steady-State Short-Circuit Current"
      n_Ik_bus1_B = "n:Ik:bus1:B", "kA, Steady-State Short-Circuit Current"
      n_Ik_bus1_C = "n:Ik:bus1:C", "kA, Steady-State Short-Circuit Current"
      n_Ik_bus1_N = "n:Ik:bus1:N", "kA, Steady-State Short-Circuit Current"
      n_Ik_bus2_A = "n:Ik:bus2:A", "kA, Steady-State Short-Circuit Current"
      n_Ik_bus2_B = "n:Ik:bus2:B", "kA, Steady-State Short-Circuit Current"
      n_Ik_bus2_C = "n:Ik:bus2:C", "kA, Steady-State Short-Circuit Current"
      n_Ik_bus2_N = "n:Ik:bus2:N", "kA, Steady-State Short-Circuit Current"
      n_R0_bus1 = "n:R0:bus1", "Ohm, Zero-Sequence Impedance, Real Part"
      n_R0_bus2 = "n:R0:bus2", "Ohm, Zero-Sequence Impedance, Real Part"
      n_X0_bus1 = "n:X0:bus1", "Ohm, Zero-Sequence Impedance, Imaginary Part"
      n_X0_bus2 = "n:X0:bus2", "Ohm, Zero-Sequence Impedance, Imaginary Part"
      n_Z0_bus1 = "n:Z0:bus1", "Ohm, Zero-Sequence Impedance, Magnitude"
      n_Z0_bus2 = "n:Z0:bus2", "Ohm, Zero-Sequence Impedance, Magnitude"
      n_phiz0_bus1 = "n:phiz0:bus1", "deg, Zero-Sequence Impedance, Angle"
      n_phiz0_bus2 = "n:phiz0:bus2", "deg, Zero-Sequence Impedance, Angle"
      n_r0Sbase_bus1 = "n:r0Sbase:bus1", "p.u., Zero-Sequence Impedance, Real Part"
      n_r0Sbase_bus2 = "n:r0Sbase:bus2", "p.u., Zero-Sequence Impedance, Real Part"
      n_x0Sbase_bus1 = "n:x0Sbase:bus1", "p.u., Zero-Sequence Impedance, Imaginary Part"
      n_x0Sbase_bus2 = "n:x0Sbase:bus2", "p.u., Zero-Sequence Impedance, Imaginary Part"
      n_z0Sbase_bus1 = "n:z0Sbase:bus1", "p.u., Zero-Sequence Impedance, Magnitude"
      n_z0Sbase_bus2 = "n:z0Sbase:bus2", "p.u., Zero-Sequence Impedance, Magnitude"
      n_R1_bus1 = "n:R1:bus1", "Ohm, Positive-Sequence Impedance, Real Part"
      n_R1_bus2 = "n:R1:bus2", "Ohm, Positive-Sequence Impedance, Real Part"
      n_X1_bus1 = "n:X1:bus1", "Ohm, Positive-Sequence Impedance, Imaginary Part"
      n_X1_bus2 = "n:X1:bus2", "Ohm, Positive-Sequence Impedance, Imaginary Part"
      n_Z1_bus1 = "n:Z1:bus1", "Ohm, Positive-Sequence Impedance, Magnitude"
      n_Z1_bus2 = "n:Z1:bus2", "Ohm, Positive-Sequence Impedance, Magnitude"
      n_phiz1_bus1 = "n:phiz1:bus1", "deg, Positive-Sequence Impedance, Angle"
      n_phiz1_bus2 = "n:phiz1:bus2", "deg, Positive-Sequence Impedance, Angle"
      n_r1Sbase_bus1 = "n:r1Sbase:bus1", "p.u., Positive-Sequence Impedance, Real Part"
      n_r1Sbase_bus2 = "n:r1Sbase:bus2", "p.u., Positive-Sequence Impedance, Real Part"
      n_x1Sbase_bus1 = "n:x1Sbase:bus1", "p.u., Positive-Sequence Impedance, Imaginary Part"
      n_x1Sbase_bus2 = "n:x1Sbase:bus2", "p.u., Positive-Sequence Impedance, Imaginary Part"
      n_z1Sbase_bus1 = "n:z1Sbase:bus1", "p.u., Positive-Sequence Impedance, Magnitude"
      n_z1Sbase_bus2 = "n:z1Sbase:bus2", "p.u., Positive-Sequence Impedance, Magnitude"
      n_R2_bus1 = "n:R2:bus1", "Ohm, Negative-Sequence Impedance, Real Part"
      n_R2_bus2 = "n:R2:bus2", "Ohm, Negative-Sequence Impedance, Real Part"
      n_X2_bus1 = "n:X2:bus1", "Ohm, Negative-Sequence Impedance, Imaginary Part"
      n_X2_bus2 = "n:X2:bus2", "Ohm, Negative-Sequence Impedance, Imaginary Part"
      n_Z2_bus1 = "n:Z2:bus1", "Ohm, Negative-Sequence Impedance, Magnitude"
      n_Z2_bus2 = "n:Z2:bus2", "Ohm, Negative-Sequence Impedance, Magnitude"
      n_phiz2_bus1 = "n:phiz2:bus1", "deg, Negative-Sequence Impedance, Angle"
      n_phiz2_bus2 = "n:phiz2:bus2", "deg, Negative-Sequence Impedance, Angle"
      n_r2Sbase_bus1 = "n:r2Sbase:bus1", "p.u., Negative-Sequence Impedance, Real Part"
      n_r2Sbase_bus2 = "n:r2Sbase:bus2", "p.u., Negative-Sequence Impedance, Real Part"
      n_x2Sbase_bus1 = "n:x2Sbase:bus1", "p.u., Negative-Sequence Impedance, Imaginary Part"
      n_x2Sbase_bus2 = "n:x2Sbase:bus2", "p.u., Negative-Sequence Impedance, Imaginary Part"
      n_z2Sbase_bus1 = "n:z2Sbase:bus1", "p.u., Negative-Sequence Impedance, Magnitude"
      n_z2Sbase_bus2 = "n:z2Sbase:bus2", "p.u., Negative-Sequence Impedance, Magnitude"
      n_R_bus1_A = "n:R:bus1:A", "Ohm, Network Resistance, Line-Ground"
      n_R_bus1_B = "n:R:bus1:B", "Ohm, Network Resistance, Line-Ground"
      n_R_bus1_C = "n:R:bus1:C", "Ohm, Network Resistance, Line-Ground"
      n_R_bus1_N = "n:R:bus1:N", "Ohm, Network Resistance, Line-Ground"
      n_R_bus2_A = "n:R:bus2:A", "Ohm, Network Resistance, Line-Ground"
      n_R_bus2_B = "n:R:bus2:B", "Ohm, Network Resistance, Line-Ground"
      n_R_bus2_C = "n:R:bus2:C", "Ohm, Network Resistance, Line-Ground"
      n_R_bus2_N = "n:R:bus2:N", "Ohm, Network Resistance, Line-Ground"
      n_X_bus1_A = "n:X:bus1:A", "Ohm, Network Reactance, Line-Ground"
      n_X_bus1_B = "n:X:bus1:B", "Ohm, Network Reactance, Line-Ground"
      n_X_bus1_C = "n:X:bus1:C", "Ohm, Network Reactance, Line-Ground"
      n_X_bus1_N = "n:X:bus1:N", "Ohm, Network Reactance, Line-Ground"
      n_X_bus2_A = "n:X:bus2:A", "Ohm, Network Reactance, Line-Ground"
      n_X_bus2_B = "n:X:bus2:B", "Ohm, Network Reactance, Line-Ground"
      n_X_bus2_C = "n:X:bus2:C", "Ohm, Network Reactance, Line-Ground"
      n_X_bus2_N = "n:X:bus2:N", "Ohm, Network Reactance, Line-Ground"
      n_Z_bus1_A = "n:Z:bus1:A", "Ohm, Network Impedance, Line-Ground"
      n_Z_bus1_B = "n:Z:bus1:B", "Ohm, Network Impedance, Line-Ground"
      n_Z_bus1_C = "n:Z:bus1:C", "Ohm, Network Impedance, Line-Ground"
      n_Z_bus1_N = "n:Z:bus1:N", "Ohm, Network Impedance, Line-Ground"
      n_Z_bus2_A = "n:Z:bus2:A", "Ohm, Network Impedance, Line-Ground"
      n_Z_bus2_B = "n:Z:bus2:B", "Ohm, Network Impedance, Line-Ground"
      n_Z_bus2_C = "n:Z:bus2:C", "Ohm, Network Impedance, Line-Ground"
      n_Z_bus2_N = "n:Z:bus2:N", "Ohm, Network Impedance, Line-Ground"
      n_phiz_bus1_A = "n:phiz:bus1:A", "deg, Angle of Network Impedance, Line-Ground"
      n_phiz_bus1_B = "n:phiz:bus1:B", "deg, Angle of Network Impedance, Line-Ground"
      n_phiz_bus1_C = "n:phiz:bus1:C", "deg, Angle of Network Impedance, Line-Ground"
      n_phiz_bus1_N = "n:phiz:bus1:N", "deg, Angle of Network Impedance, Line-Ground"
      n_phiz_bus2_A = "n:phiz:bus2:A", "deg, Angle of Network Impedance, Line-Ground"
      n_phiz_bus2_B = "n:phiz:bus2:B", "deg, Angle of Network Impedance, Line-Ground"
      n_phiz_bus2_C = "n:phiz:bus2:C", "deg, Angle of Network Impedance, Line-Ground"
      n_phiz_bus2_N = "n:phiz:bus2:N", "deg, Angle of Network Impedance, Line-Ground"
      n_Rl_bus1_A = "n:Rl:bus1:A", "Ohm, Network Resistance, Line-Line"
      n_Rl_bus1_B = "n:Rl:bus1:B", "Ohm, Network Resistance, Line-Line"
      n_Rl_bus1_C = "n:Rl:bus1:C", "Ohm, Network Resistance, Line-Line"
      n_Rl_bus1_N = "n:Rl:bus1:N", "Ohm, Network Resistance, Line-Line"
      n_Rl_bus2_A = "n:Rl:bus2:A", "Ohm, Network Resistance, Line-Line"
      n_Rl_bus2_B = "n:Rl:bus2:B", "Ohm, Network Resistance, Line-Line"
      n_Rl_bus2_C = "n:Rl:bus2:C", "Ohm, Network Resistance, Line-Line"
      n_Rl_bus2_N = "n:Rl:bus2:N", "Ohm, Network Resistance, Line-Line"
      n_Xl_bus1_A = "n:Xl:bus1:A", "Ohm, Network Reactance, Line-Line"
      n_Xl_bus1_B = "n:Xl:bus1:B", "Ohm, Network Reactance, Line-Line"
      n_Xl_bus1_C = "n:Xl:bus1:C", "Ohm, Network Reactance, Line-Line"
      n_Xl_bus1_N = "n:Xl:bus1:N", "Ohm, Network Reactance, Line-Line"
      n_Xl_bus2_A = "n:Xl:bus2:A", "Ohm, Network Reactance, Line-Line"
      n_Xl_bus2_B = "n:Xl:bus2:B", "Ohm, Network Reactance, Line-Line"
      n_Xl_bus2_C = "n:Xl:bus2:C", "Ohm, Network Reactance, Line-Line"
      n_Xl_bus2_N = "n:Xl:bus2:N", "Ohm, Network Reactance, Line-Line"
      n_Zl_bus1_A = "n:Zl:bus1:A", "Ohm, Network Impedance, Line-Line"
      n_Zl_bus1_B = "n:Zl:bus1:B", "Ohm, Network Impedance, Line-Line"
      n_Zl_bus1_C = "n:Zl:bus1:C", "Ohm, Network Impedance, Line-Line"
      n_Zl_bus1_N = "n:Zl:bus1:N", "Ohm, Network Impedance, Line-Line"
      n_Zl_bus2_A = "n:Zl:bus2:A", "Ohm, Network Impedance, Line-Line"
      n_Zl_bus2_B = "n:Zl:bus2:B", "Ohm, Network Impedance, Line-Line"
      n_Zl_bus2_C = "n:Zl:bus2:C", "Ohm, Network Impedance, Line-Line"
      n_Zl_bus2_N = "n:Zl:bus2:N", "Ohm, Network Impedance, Line-Line"
      n_phizl_bus1_A = "n:phizl:bus1:A", "deg, Angle of Network Impedance, Line-Line"
      n_phizl_bus1_B = "n:phizl:bus1:B", "deg, Angle of Network Impedance, Line-Line"
      n_phizl_bus1_C = "n:phizl:bus1:C", "deg, Angle of Network Impedance, Line-Line"
      n_phizl_bus1_N = "n:phizl:bus1:N", "deg, Angle of Network Impedance, Line-Line"
      n_phizl_bus2_A = "n:phizl:bus2:A", "deg, Angle of Network Impedance, Line-Line"
      n_phizl_bus2_B = "n:phizl:bus2:B", "deg, Angle of Network Impedance, Line-Line"
      n_phizl_bus2_C = "n:phizl:bus2:C", "deg, Angle of Network Impedance, Line-Line"
      n_phizl_bus2_N = "n:phizl:bus2:N", "deg, Angle of Network Impedance, Line-Line"
      n_Rln_bus1_A = "n:Rln:bus1:A", "Ohm, Network Resistance, Line-Neutral"
      n_Rln_bus1_B = "n:Rln:bus1:B", "Ohm, Network Resistance, Line-Neutral"
      n_Rln_bus1_C = "n:Rln:bus1:C", "Ohm, Network Resistance, Line-Neutral"
      n_Rln_bus1_N = "n:Rln:bus1:N", "Ohm, Network Resistance, Line-Neutral"
      n_Rln_bus2_A = "n:Rln:bus2:A", "Ohm, Network Resistance, Line-Neutral"
      n_Rln_bus2_B = "n:Rln:bus2:B", "Ohm, Network Resistance, Line-Neutral"
      n_Rln_bus2_C = "n:Rln:bus2:C", "Ohm, Network Resistance, Line-Neutral"
      n_Rln_bus2_N = "n:Rln:bus2:N", "Ohm, Network Resistance, Line-Neutral"
      n_Xln_bus1_A = "n:Xln:bus1:A", "Ohm, Network Reactance, Line-Neutral"
      n_Xln_bus1_B = "n:Xln:bus1:B", "Ohm, Network Reactance, Line-Neutral"
      n_Xln_bus1_C = "n:Xln:bus1:C", "Ohm, Network Reactance, Line-Neutral"
      n_Xln_bus1_N = "n:Xln:bus1:N", "Ohm, Network Reactance, Line-Neutral"
      n_Xln_bus2_A = "n:Xln:bus2:A", "Ohm, Network Reactance, Line-Neutral"
      n_Xln_bus2_B = "n:Xln:bus2:B", "Ohm, Network Reactance, Line-Neutral"
      n_Xln_bus2_C = "n:Xln:bus2:C", "Ohm, Network Reactance, Line-Neutral"
      n_Xln_bus2_N = "n:Xln:bus2:N", "Ohm, Network Reactance, Line-Neutral"
      n_Zln_bus1_A = "n:Zln:bus1:A", "Ohm, Network Impedance, Line-Neutral"
      n_Zln_bus1_B = "n:Zln:bus1:B", "Ohm, Network Impedance, Line-Neutral"
      n_Zln_bus1_C = "n:Zln:bus1:C", "Ohm, Network Impedance, Line-Neutral"
      n_Zln_bus1_N = "n:Zln:bus1:N", "Ohm, Network Impedance, Line-Neutral"
      n_Zln_bus2_A = "n:Zln:bus2:A", "Ohm, Network Impedance, Line-Neutral"
      n_Zln_bus2_B = "n:Zln:bus2:B", "Ohm, Network Impedance, Line-Neutral"
      n_Zln_bus2_C = "n:Zln:bus2:C", "Ohm, Network Impedance, Line-Neutral"
      n_Zln_bus2_N = "n:Zln:bus2:N", "Ohm, Network Impedance, Line-Neutral"
      n_phizln_bus1_A = "n:phizln:bus1:A", "deg, Angle of Network Impedance, Line-Neutral"
      n_phizln_bus1_B = "n:phizln:bus1:B", "deg, Angle of Network Impedance, Line-Neutral"
      n_phizln_bus1_C = "n:phizln:bus1:C", "deg, Angle of Network Impedance, Line-Neutral"
      n_phizln_bus1_N = "n:phizln:bus1:N", "deg, Angle of Network Impedance, Line-Neutral"
      n_phizln_bus2_A = "n:phizln:bus2:A", "deg, Angle of Network Impedance, Line-Neutral"
      n_phizln_bus2_B = "n:phizln:bus2:B", "deg, Angle of Network Impedance, Line-Neutral"
      n_phizln_bus2_C = "n:phizln:bus2:C", "deg, Angle of Network Impedance, Line-Neutral"
      n_phizln_bus2_N = "n:phizln:bus2:N", "deg, Angle of Network Impedance, Line-Neutral"
      n_Rn_bus1 = "n:Rn:bus1", "Ohm, Network Resistance, Neutral"
      n_Rn_bus2 = "n:Rn:bus2", "Ohm, Network Resistance, Neutral"
      n_Xn_bus1 = "n:Xn:bus1", "Ohm, Network Reactance, Neutral"
      n_Xn_bus2 = "n:Xn:bus2", "Ohm, Network Reactance, Neutral"
      n_Zn_bus1 = "n:Zn:bus1", "Ohm, Network Impedance, Neutral"
      n_Zn_bus2 = "n:Zn:bus2", "Ohm, Network Impedance, Neutral"
      n_phizn_bus1 = "n:phizn:bus1", "deg, Angle of Network Impedance, Neutral"
      n_phizn_bus2 = "n:phizn:bus2", "deg, Angle of Network Impedance, Neutral"
      n_eff_bus1_A = "n:eff:bus1:A", "Earth Fault Factor"
      n_eff_bus1_B = "n:eff:bus1:B", "Earth Fault Factor"
      n_eff_bus1_C = "n:eff:bus1:C", "Earth Fault Factor"
      n_eff_bus1_N = "n:eff:bus1:N", "Earth Fault Factor"
      n_eff_bus2_A = "n:eff:bus2:A", "Earth Fault Factor"
      n_eff_bus2_B = "n:eff:bus2:B", "Earth Fault Factor"
      n_eff_bus2_C = "n:eff:bus2:C", "Earth Fault Factor"
      n_eff_bus2_N = "n:eff:bus2:N", "Earth Fault Factor"
      n_X0toX1_bus1 = "n:X0toX1:bus1", "X0/X1"
      n_X0toX1_bus2 = "n:X0toX1:bus2", "X0/X1"
      n_in_bus1 = "n:in:bus1", "p.u., Neutral Current, Magnitude"
      n_in_bus2 = "n:in:bus2", "p.u., Neutral Current, Magnitude"
      n_In_bus1 = "n:In:bus1", "kA, Neutral Current, Magnitude"
      n_In_bus2 = "n:In:bus2", "kA, Neutral Current, Magnitude"
      n_phiin_bus1 = "n:phiin:bus1", "deg, Neutral Current, Angle"
      n_phiin_bus2 = "n:phiin:bus2", "deg, Neutral Current, Angle"
      n_i0r_bus1 = "n:i0r:bus1", "p.u., Zero-Sequence Current, Real Part"
      n_i0r_bus2 = "n:i0r:bus2", "p.u., Zero-Sequence Current, Real Part"
      n_i0i_bus1 = "n:i0i:bus1", "p.u., Zero-Sequence Current, Imaginary Part"
      n_i0i_bus2 = "n:i0i:bus2", "p.u., Zero-Sequence Current, Imaginary Part"
      n_i0_bus1 = "n:i0:bus1", "p.u., Zero-Sequence Current, Magnitude"
      n_i0_bus2 = "n:i0:bus2", "p.u., Zero-Sequence Current, Magnitude"
      n_I0_bus1 = "n:I0:bus1", "kA, Zero-Sequence Current, Magnitude"
      n_I0_bus2 = "n:I0:bus2", "kA, Zero-Sequence Current, Magnitude"
      n_I0x3_bus1 = "n:I0x3:bus1", "kA, 3*I0"
      n_I0x3_bus2 = "n:I0x3:bus2", "kA, 3*I0"
      n_phii0_bus1 = "n:phii0:bus1", "deg, Zero-Sequence Current, Angle"
      n_phii0_bus2 = "n:phii0:bus2", "deg, Zero-Sequence Current, Angle"
      n_i1r_bus1 = "n:i1r:bus1", "p.u., Positive-Sequence Current, Real Part"
      n_i1r_bus2 = "n:i1r:bus2", "p.u., Positive-Sequence Current, Real Part"
      n_i1i_bus1 = "n:i1i:bus1", "p.u., Positive-Sequence Current, Imaginary Part"
      n_i1i_bus2 = "n:i1i:bus2", "p.u., Positive-Sequence Current, Imaginary Part"
      n_i1_bus1 = "n:i1:bus1", "p.u., Positive-Sequence Current, Magnitude"
      n_i1_bus2 = "n:i1:bus2", "p.u., Positive-Sequence Current, Magnitude"
      n_I1_bus1 = "n:I1:bus1", "kA, Positive-Sequence Current, Magnitude"
      n_I1_bus2 = "n:I1:bus2", "kA, Positive-Sequence Current, Magnitude"
      n_phii1_bus1 = "n:phii1:bus1", "deg, Positive-Sequence Current, Angle"
      n_phii1_bus2 = "n:phii1:bus2", "deg, Positive-Sequence Current, Angle"
      n_i2r_bus1 = "n:i2r:bus1", "p.u., Negative-Sequence Current, Real Part"
      n_i2r_bus2 = "n:i2r:bus2", "p.u., Negative-Sequence Current, Real Part"
      n_i2i_bus1 = "n:i2i:bus1", "p.u., Negative-Sequence Current, Imaginary Part"
      n_i2i_bus2 = "n:i2i:bus2", "p.u., Negative-Sequence Current, Imaginary Part"
      n_i2_bus1 = "n:i2:bus1", "p.u., Negative-Sequence Current, Magnitude"
      n_i2_bus2 = "n:i2:bus2", "p.u., Negative-Sequence Current, Magnitude"
      n_I2_bus1 = "n:I2:bus1", "kA, Negative-Sequence Current, Magnitude"
      n_I2_bus2 = "n:I2:bus2", "kA, Negative-Sequence Current, Magnitude"
      n_phii2_bus1 = "n:phii2:bus1", "deg, Negative-Sequence Current, Angle"
      n_phii2_bus2 = "n:phii2:bus2", "deg, Negative-Sequence Current, Angle"
      n_du_bus1_A = "n:du:bus1:A", "%, Voltage Deviation"
      n_du_bus1_B = "n:du:bus1:B", "%, Voltage Deviation"
      n_du_bus1_C = "n:du:bus1:C", "%, Voltage Deviation"
      n_du_bus1_N = "n:du:bus1:N", "%, Voltage Deviation"
      n_du_bus2_A = "n:du:bus2:A", "%, Voltage Deviation"
      n_du_bus2_B = "n:du:bus2:B", "%, Voltage Deviation"
      n_du_bus2_C = "n:du:bus2:C", "%, Voltage Deviation"
      n_du_bus2_N = "n:du:bus2:N", "%, Voltage Deviation"
      n_urpref_bus1_A = "n:urpref:bus1:A", "p.u., Pre-fault Voltage, Real Part"
      n_urpref_bus1_B = "n:urpref:bus1:B", "p.u., Pre-fault Voltage, Real Part"
      n_urpref_bus1_C = "n:urpref:bus1:C", "p.u., Pre-fault Voltage, Real Part"
      n_urpref_bus1_N = "n:urpref:bus1:N", "p.u., Pre-fault Voltage, Real Part"
      n_urpref_bus2_A = "n:urpref:bus2:A", "p.u., Pre-fault Voltage, Real Part"
      n_urpref_bus2_B = "n:urpref:bus2:B", "p.u., Pre-fault Voltage, Real Part"
      n_urpref_bus2_C = "n:urpref:bus2:C", "p.u., Pre-fault Voltage, Real Part"
      n_urpref_bus2_N = "n:urpref:bus2:N", "p.u., Pre-fault Voltage, Real Part"
      n_uipref_bus1_A = "n:uipref:bus1:A", "p.u., Pre-fault Voltage, Imaginary Part"
      n_uipref_bus1_B = "n:uipref:bus1:B", "p.u., Pre-fault Voltage, Imaginary Part"
      n_uipref_bus1_C = "n:uipref:bus1:C", "p.u., Pre-fault Voltage, Imaginary Part"
      n_uipref_bus1_N = "n:uipref:bus1:N", "p.u., Pre-fault Voltage, Imaginary Part"
      n_uipref_bus2_A = "n:uipref:bus2:A", "p.u., Pre-fault Voltage, Imaginary Part"
      n_uipref_bus2_B = "n:uipref:bus2:B", "p.u., Pre-fault Voltage, Imaginary Part"
      n_uipref_bus2_C = "n:uipref:bus2:C", "p.u., Pre-fault Voltage, Imaginary Part"
      n_uipref_bus2_N = "n:uipref:bus2:N", "p.u., Pre-fault Voltage, Imaginary Part"
      n_upref_bus1_A = "n:upref:bus1:A", "p.u., Pre-fault Voltage, Magnitude"
      n_upref_bus1_B = "n:upref:bus1:B", "p.u., Pre-fault Voltage, Magnitude"
      n_upref_bus1_C = "n:upref:bus1:C", "p.u., Pre-fault Voltage, Magnitude"
      n_upref_bus1_N = "n:upref:bus1:N", "p.u., Pre-fault Voltage, Magnitude"
      n_upref_bus2_A = "n:upref:bus2:A", "p.u., Pre-fault Voltage, Magnitude"
      n_upref_bus2_B = "n:upref:bus2:B", "p.u., Pre-fault Voltage, Magnitude"
      n_upref_bus2_C = "n:upref:bus2:C", "p.u., Pre-fault Voltage, Magnitude"
      n_upref_bus2_N = "n:upref:bus2:N", "p.u., Pre-fault Voltage, Magnitude"
      n_uprefpc_bus1_A = "n:uprefpc:bus1:A", "%, Pre-fault Voltage, Magnitude"
      n_uprefpc_bus1_B = "n:uprefpc:bus1:B", "%, Pre-fault Voltage, Magnitude"
      n_uprefpc_bus1_C = "n:uprefpc:bus1:C", "%, Pre-fault Voltage, Magnitude"
      n_uprefpc_bus1_N = "n:uprefpc:bus1:N", "%, Pre-fault Voltage, Magnitude"
      n_uprefpc_bus2_A = "n:uprefpc:bus2:A", "%, Pre-fault Voltage, Magnitude"
      n_uprefpc_bus2_B = "n:uprefpc:bus2:B", "%, Pre-fault Voltage, Magnitude"
      n_uprefpc_bus2_C = "n:uprefpc:bus2:C", "%, Pre-fault Voltage, Magnitude"
      n_uprefpc_bus2_N = "n:uprefpc:bus2:N", "%, Pre-fault Voltage, Magnitude"
      n_Upref_bus1_A = "n:Upref:bus1:A", "kV, Line-Ground Pre-fault Voltage, Magnitude"
      n_Upref_bus1_B = "n:Upref:bus1:B", "kV, Line-Ground Pre-fault Voltage, Magnitude"
      n_Upref_bus1_C = "n:Upref:bus1:C", "kV, Line-Ground Pre-fault Voltage, Magnitude"
      n_Upref_bus1_N = "n:Upref:bus1:N", "kV, Line-Ground Pre-fault Voltage, Magnitude"
      n_Upref_bus2_A = "n:Upref:bus2:A", "kV, Line-Ground Pre-fault Voltage, Magnitude"
      n_Upref_bus2_B = "n:Upref:bus2:B", "kV, Line-Ground Pre-fault Voltage, Magnitude"
      n_Upref_bus2_C = "n:Upref:bus2:C", "kV, Line-Ground Pre-fault Voltage, Magnitude"
      n_Upref_bus2_N = "n:Upref:bus2:N", "kV, Line-Ground Pre-fault Voltage, Magnitude"
      n_Ulpref_bus1_A = "n:Ulpref:bus1:A", "kV, Line-Line Pre-fault Voltage, Magnitude"
      n_Ulpref_bus1_B = "n:Ulpref:bus1:B", "kV, Line-Line Pre-fault Voltage, Magnitude"
      n_Ulpref_bus1_C = "n:Ulpref:bus1:C", "kV, Line-Line Pre-fault Voltage, Magnitude"
      n_Ulpref_bus1_N = "n:Ulpref:bus1:N", "kV, Line-Line Pre-fault Voltage, Magnitude"
      n_Ulpref_bus2_A = "n:Ulpref:bus2:A", "kV, Line-Line Pre-fault Voltage, Magnitude"
      n_Ulpref_bus2_B = "n:Ulpref:bus2:B", "kV, Line-Line Pre-fault Voltage, Magnitude"
      n_Ulpref_bus2_C = "n:Ulpref:bus2:C", "kV, Line-Line Pre-fault Voltage, Magnitude"
      n_Ulpref_bus2_N = "n:Ulpref:bus2:N", "kV, Line-Line Pre-fault Voltage, Magnitude"
      n_phiupref_bus1_A = "n:phiupref:bus1:A", "deg, Pre-fault Voltage, Angle"
      n_phiupref_bus1_B = "n:phiupref:bus1:B", "deg, Pre-fault Voltage, Angle"
      n_phiupref_bus1_C = "n:phiupref:bus1:C", "deg, Pre-fault Voltage, Angle"
      n_phiupref_bus1_N = "n:phiupref:bus1:N", "deg, Pre-fault Voltage, Angle"
      n_phiupref_bus2_A = "n:phiupref:bus2:A", "deg, Pre-fault Voltage, Angle"
      n_phiupref_bus2_B = "n:phiupref:bus2:B", "deg, Pre-fault Voltage, Angle"
      n_phiupref_bus2_C = "n:phiupref:bus2:C", "deg, Pre-fault Voltage, Angle"
      n_phiupref_bus2_N = "n:phiupref:bus2:N", "deg, Pre-fault Voltage, Angle"
      n_phiui_bus1_A = "n:phiui:bus1:A", "deg, Angle between Voltage and Current"
      n_phiui_bus1_B = "n:phiui:bus1:B", "deg, Angle between Voltage and Current"
      n_phiui_bus1_C = "n:phiui:bus1:C", "deg, Angle between Voltage and Current"
      n_phiui_bus1_N = "n:phiui:bus1:N", "deg, Angle between Voltage and Current"
      n_phiui_bus2_A = "n:phiui:bus2:A", "deg, Angle between Voltage and Current"
      n_phiui_bus2_B = "n:phiui:bus2:B", "deg, Angle between Voltage and Current"
      n_phiui_bus2_C = "n:phiui:bus2:C", "deg, Angle between Voltage and Current"
      n_phiui_bus2_N = "n:phiui:bus2:N", "deg, Angle between Voltage and Current"
      n_phiu1i1_bus1 = "n:phiu1i1:bus1", "deg, Angle between Voltage and Current in positive sequence system"
      n_phiu1i1_bus2 = "n:phiu1i1:bus2", "deg, Angle between Voltage and Current in positive sequence system"
      n_phiu2i2_bus1 = "n:phiu2i2:bus1", "deg, Angle between Voltage and Current in negative sequence system"
      n_phiu2i2_bus2 = "n:phiu2i2:bus2", "deg, Angle between Voltage and Current in negative sequence system"
      n_phiu0i0_bus1 = "n:phiu0i0:bus1", "deg, Angle between Voltage and Current in zero-sequence system"
      n_phiu0i0_bus2 = "n:phiu0i0:bus2", "deg, Angle between Voltage and Current in zero-sequence system"
      c_k0r = "c:k0r", "Earth Factor, Real Part"
      c_k0i = "c:k0i", "Earth Factor, Imaginary Part"

    class ElmLod(Enum):
      _init_ = 'value __doc__'

      m_u0_bus1 = "m:u0:bus1", "p.u., Zero-Sequence-Voltage, Magnitude"
      m_phiu0_bus1 = "m:phiu0:bus1", "deg, Zero-Sequence-Voltage, Angle"
      m_u1r_bus1 = "m:u1r:bus1", "p.u., Positive-Sequence-Voltage, Real Part"
      m_u1i_bus1 = "m:u1i:bus1", "p.u., Positive-Sequence-Voltage, Imaginary Part"
      m_u1_bus1 = "m:u1:bus1", "p.u., Positive-Sequence-Voltage, Magnitude"
      m_U1_bus1 = "m:U1:bus1", "kV, Line-Ground Positive-Sequence-Voltage, Magnitude"
      m_U1l_bus1 = "m:U1l:bus1", "kV, Line-Line Positive-Sequence-Voltage, Magnitude"
      m_phiu1_bus1 = "m:phiu1:bus1", "deg, Positive-Sequence-Voltage, Angle"
      m_u2_bus1 = "m:u2:bus1", "p.u., Negative-Sequence-Voltage, Magnitude"
      m_phiu2_bus1 = "m:phiu2:bus1", "deg, Negative-Sequence-Voltage, Angle"
      m_inet_bus1_A = "m:inet:bus1:A", "p.u., Phase Current, Magnitude, referred to network"
      m_inet_bus1_B = "m:inet:bus1:B", "p.u., Phase Current, Magnitude, referred to network"
      m_inet_bus1_C = "m:inet:bus1:C", "p.u., Phase Current, Magnitude, referred to network"
      m_inet_bus1_N = "m:inet:bus1:N", "p.u., Phase Current, Magnitude, referred to network"
      m_I_bus1_A = "m:I:bus1:A", "kA, Phase Current, Magnitude"
      m_I_bus1_B = "m:I:bus1:B", "kA, Phase Current, Magnitude"
      m_I_bus1_C = "m:I:bus1:C", "kA, Phase Current, Magnitude"
      m_I_bus1_N = "m:I:bus1:N", "kA, Phase Current, Magnitude"
      m_phii_bus1_A = "m:phii:bus1:A", "deg, Phase Current, Angle"
      m_phii_bus1_B = "m:phii:bus1:B", "deg, Phase Current, Angle"
      m_phii_bus1_C = "m:phii:bus1:C", "deg, Phase Current, Angle"
      m_phii_bus1_N = "m:phii:bus1:N", "deg, Phase Current, Angle"
      m_i0r_bus1 = "m:i0r:bus1", "p.u., Zero-Sequence Current, Real Part"
      m_i0i_bus1 = "m:i0i:bus1", "p.u., Zero-Sequence Current, Imaginary Part"
      m_i0_bus1 = "m:i0:bus1", "p.u., Zero-Sequence Current, Magnitude"
      m_I0_bus1 = "m:I0:bus1", "kA, Zero-Sequence Current, Magnitude"
      m_I0x3_bus1 = "m:I0x3:bus1", "kA, 3*I0"
      m_phii0_bus1 = "m:phii0:bus1", "deg, Zero-Sequence Current, Angle"
      m_i1r_bus1 = "m:i1r:bus1", "p.u., Positive-Sequence Current, Real Part"
      m_i1i_bus1 = "m:i1i:bus1", "p.u., Positive-Sequence Current, Imaginary Part"
      m_i1_bus1 = "m:i1:bus1", "p.u., Positive-Sequence Current, Magnitude"
      m_I1_bus1 = "m:I1:bus1", "kA, Positive-Sequence Current, Magnitude"
      m_phii1_bus1 = "m:phii1:bus1", "deg, Positive-Sequence Current, Angle"
      m_i2r_bus1 = "m:i2r:bus1", "p.u., Negative-Sequence Current, Real Part"
      m_i2i_bus1 = "m:i2i:bus1", "p.u., Negative-Sequence Current, Imaginary Part"
      m_i2_bus1 = "m:i2:bus1", "p.u., Negative-Sequence Current, Magnitude"
      m_I2_bus1 = "m:I2:bus1", "kA, Negative-Sequence Current, Magnitude"
      m_phii2_bus1 = "m:phii2:bus1", "deg, Negative-Sequence Current, Angle"
      m_ubfacI_bus1 = "m:ubfacI:bus1", "%, Current Unbalance Factor"
      m_ubfacS_bus1 = "m:ubfacS:bus1", "%, Power Unbalance Factor"
      m_P_bus1_A = "m:P:bus1:A", "MW, Active Power"
      m_P_bus1_B = "m:P:bus1:B", "MW, Active Power"
      m_P_bus1_C = "m:P:bus1:C", "MW, Active Power"
      m_P_bus1_N = "m:P:bus1:N", "MW, Active Power"
      m_Q_bus1_A = "m:Q:bus1:A", "Mvar, Reactive Power"
      m_Q_bus1_B = "m:Q:bus1:B", "Mvar, Reactive Power"
      m_Q_bus1_C = "m:Q:bus1:C", "Mvar, Reactive Power"
      m_Q_bus1_N = "m:Q:bus1:N", "Mvar, Reactive Power"
      m_S_bus1_A = "m:S:bus1:A", "MVA, Apparent Power"
      m_S_bus1_B = "m:S:bus1:B", "MVA, Apparent Power"
      m_S_bus1_C = "m:S:bus1:C", "MVA, Apparent Power"
      m_S_bus1_N = "m:S:bus1:N", "MVA, Apparent Power"
      m_cosphi_bus1_A = "m:cosphi:bus1:A", "Power Factor"
      m_cosphi_bus1_B = "m:cosphi:bus1:B", "Power Factor"
      m_cosphi_bus1_C = "m:cosphi:bus1:C", "Power Factor"
      m_cosphi_bus1_N = "m:cosphi:bus1:N", "Power Factor"
      m_tanphi_bus1_A = "m:tanphi:bus1:A", "tan(phi)"
      m_tanphi_bus1_B = "m:tanphi:bus1:B", "tan(phi)"
      m_tanphi_bus1_C = "m:tanphi:bus1:C", "tan(phi)"
      m_tanphi_bus1_N = "m:tanphi:bus1:N", "tan(phi)"
      m_Psum_bus1 = "m:Psum:bus1", "MW, Total Active Power"
      m_Qsum_bus1 = "m:Qsum:bus1", "Mvar, Total Reactive Power"
      m_Ssum_bus1 = "m:Ssum:bus1", "MVA, Total Apparent Power"
      m_cosphisum_bus1 = "m:cosphisum:bus1", "Total Power Factor"
      m_tanphisum_bus1 = "m:tanphisum:bus1", "Total tan(phi)"
      m_Pln_bus1_A = "m:Pln:bus1:A", "MW, Active Power (Line-Neutral)"
      m_Pln_bus1_B = "m:Pln:bus1:B", "MW, Active Power (Line-Neutral)"
      m_Pln_bus1_C = "m:Pln:bus1:C", "MW, Active Power (Line-Neutral)"
      m_Pln_bus1_N = "m:Pln:bus1:N", "MW, Active Power (Line-Neutral)"
      m_Qln_bus1_A = "m:Qln:bus1:A", "Mvar, Reactive Power (Line-Neutral)"
      m_Qln_bus1_B = "m:Qln:bus1:B", "Mvar, Reactive Power (Line-Neutral)"
      m_Qln_bus1_C = "m:Qln:bus1:C", "Mvar, Reactive Power (Line-Neutral)"
      m_Qln_bus1_N = "m:Qln:bus1:N", "Mvar, Reactive Power (Line-Neutral)"
      m_P1_bus1 = "m:P1:bus1", "MW, Positive Sequence Active Power"
      m_Q1_bus1 = "m:Q1:bus1", "Mvar, Positive Sequence Reactive Power"
      m_P2_bus1 = "m:P2:bus1", "MW, Negative Sequence Active Power"
      m_Q2_bus1 = "m:Q2:bus1", "Mvar, Negative Sequence Reactive Power"
      m_P0_bus1 = "m:P0:bus1", "MW, Zero Sequence Active Power"
      m_Q0_bus1 = "m:Q0:bus1", "Mvar, Zero Sequence Reactive Power"
      m_i1P_bus1 = "m:i1P:bus1", "p.u., Positive-Sequence Active Current"
      m_i1Q_bus1 = "m:i1Q:bus1", "p.u., Positive-Sequence Reactive Current"
      m_I1P_bus1 = "m:I1P:bus1", "kA, Positive-Sequence Active Current"
      m_I1Q_bus1 = "m:I1Q:bus1", "kA, Positive-Sequence Reactive Current"
      m_i2P_bus1 = "m:i2P:bus1", "p.u., Negative-Sequence Active Current"
      m_i2Q_bus1 = "m:i2Q:bus1", "p.u., Negative-Sequence Reactive Current"
      m_I2P_bus1 = "m:I2P:bus1", "kA, Negative-Sequence Active Current"
      m_I2Q_bus1 = "m:I2Q:bus1", "kA, Negative-Sequence Reactive Current"
      m_ur_bus1_A = "m:ur:bus1:A", "p.u., Phase Voltage, Real Part"
      m_ur_bus1_B = "m:ur:bus1:B", "p.u., Phase Voltage, Real Part"
      m_ur_bus1_C = "m:ur:bus1:C", "p.u., Phase Voltage, Real Part"
      m_ur_bus1_N = "m:ur:bus1:N", "p.u., Phase Voltage, Real Part"
      m_ui_bus1_A = "m:ui:bus1:A", "p.u., Phase Voltage, Imaginary Part"
      m_ui_bus1_B = "m:ui:bus1:B", "p.u., Phase Voltage, Imaginary Part"
      m_ui_bus1_C = "m:ui:bus1:C", "p.u., Phase Voltage, Imaginary Part"
      m_ui_bus1_N = "m:ui:bus1:N", "p.u., Phase Voltage, Imaginary Part"
      m_u_bus1_A = "m:u:bus1:A", "p.u., Phase Voltage, Magnitude"
      m_u_bus1_B = "m:u:bus1:B", "p.u., Phase Voltage, Magnitude"
      m_u_bus1_C = "m:u:bus1:C", "p.u., Phase Voltage, Magnitude"
      m_u_bus1_N = "m:u:bus1:N", "p.u., Phase Voltage, Magnitude"
      m_ir_bus1_A = "m:ir:bus1:A", "p.u., Phase Current, Real Part"
      m_ir_bus1_B = "m:ir:bus1:B", "p.u., Phase Current, Real Part"
      m_ir_bus1_C = "m:ir:bus1:C", "p.u., Phase Current, Real Part"
      m_ir_bus1_N = "m:ir:bus1:N", "p.u., Phase Current, Real Part"
      m_ii_bus1_A = "m:ii:bus1:A", "p.u., Phase Current, Imaginary Part"
      m_ii_bus1_B = "m:ii:bus1:B", "p.u., Phase Current, Imaginary Part"
      m_ii_bus1_C = "m:ii:bus1:C", "p.u., Phase Current, Imaginary Part"
      m_ii_bus1_N = "m:ii:bus1:N", "p.u., Phase Current, Imaginary Part"
      m_i_bus1_A = "m:i:bus1:A", "p.u., Phase Current, Magnitude"
      m_i_bus1_B = "m:i:bus1:B", "p.u., Phase Current, Magnitude"
      m_i_bus1_C = "m:i:bus1:C", "p.u., Phase Current, Magnitude"
      m_i_bus1_N = "m:i:bus1:N", "p.u., Phase Current, Magnitude"
      m_phiui_bus1_A = "m:phiui:bus1:A", "deg, Angle between Voltage and Current"
      m_phiui_bus1_B = "m:phiui:bus1:B", "deg, Angle between Voltage and Current"
      m_phiui_bus1_C = "m:phiui:bus1:C", "deg, Angle between Voltage and Current"
      m_phiui_bus1_N = "m:phiui:bus1:N", "deg, Angle between Voltage and Current"
      m_phiu0i0_bus1 = "m:phiu0i0:bus1", "deg, Angle between Voltage and Current in zero-sequence system"
      m_phiu1i1_bus1 = "m:phiu1i1:bus1", "deg, Angle between Voltage and Current in positive sequence system"
      m_phiu2i2_bus1 = "m:phiu2i2:bus1", "deg, Angle between Voltage and Current in negative sequence system"
      m_Tfct_bus1 = "m:Tfct:bus1", "s, Fault Clearing Time"
      m_TfctPh_bus1_A = "m:TfctPh:bus1:A", "s, Fault Clearing Time"
      m_TfctPh_bus1_B = "m:TfctPh:bus1:B", "s, Fault Clearing Time"
      m_TfctPh_bus1_C = "m:TfctPh:bus1:C", "s, Fault Clearing Time"
      m_TfctPh_bus1_N = "m:TfctPh:bus1:N", "s, Fault Clearing Time"
      m_Brkload_bus1 = "m:Brkload:bus1", "%, Max. Breaker Loading"
      m_BrkloadPh_bus1_A = "m:BrkloadPh:bus1:A", "%, Breaker Loading"
      m_BrkloadPh_bus1_B = "m:BrkloadPh:bus1:B", "%, Breaker Loading"
      m_BrkloadPh_bus1_C = "m:BrkloadPh:bus1:C", "%, Breaker Loading"
      m_BrkloadPh_bus1_N = "m:BrkloadPh:bus1:N", "%, Breaker Loading"
      n_ur_bus1_A = "n:ur:bus1:A", "p.u., Line-Ground Voltage, Real Part"
      n_ur_bus1_B = "n:ur:bus1:B", "p.u., Line-Ground Voltage, Real Part"
      n_ur_bus1_C = "n:ur:bus1:C", "p.u., Line-Ground Voltage, Real Part"
      n_ur_bus1_N = "n:ur:bus1:N", "p.u., Line-Ground Voltage, Real Part"
      n_ui_bus1_A = "n:ui:bus1:A", "p.u., Line-Ground Voltage, Imaginary Part"
      n_ui_bus1_B = "n:ui:bus1:B", "p.u., Line-Ground Voltage, Imaginary Part"
      n_ui_bus1_C = "n:ui:bus1:C", "p.u., Line-Ground Voltage, Imaginary Part"
      n_ui_bus1_N = "n:ui:bus1:N", "p.u., Line-Ground Voltage, Imaginary Part"
      n_u_bus1_A = "n:u:bus1:A", "p.u., Line-Ground Voltage, Magnitude"
      n_u_bus1_B = "n:u:bus1:B", "p.u., Line-Ground Voltage, Magnitude"
      n_u_bus1_C = "n:u:bus1:C", "p.u., Line-Ground Voltage, Magnitude"
      n_u_bus1_N = "n:u:bus1:N", "p.u., Line-Ground Voltage, Magnitude"
      n_upc_bus1_A = "n:upc:bus1:A", "%, Line-Ground Voltage, Magnitude"
      n_upc_bus1_B = "n:upc:bus1:B", "%, Line-Ground Voltage, Magnitude"
      n_upc_bus1_C = "n:upc:bus1:C", "%, Line-Ground Voltage, Magnitude"
      n_upc_bus1_N = "n:upc:bus1:N", "%, Line-Ground Voltage, Magnitude"
      n_U_bus1_A = "n:U:bus1:A", "kV, Line-Ground Voltage, Magnitude"
      n_U_bus1_B = "n:U:bus1:B", "kV, Line-Ground Voltage, Magnitude"
      n_U_bus1_C = "n:U:bus1:C", "kV, Line-Ground Voltage, Magnitude"
      n_U_bus1_N = "n:U:bus1:N", "kV, Line-Ground Voltage, Magnitude"
      n_phiu_bus1_A = "n:phiu:bus1:A", "deg, Line-Ground Voltage, Angle"
      n_phiu_bus1_B = "n:phiu:bus1:B", "deg, Line-Ground Voltage, Angle"
      n_phiu_bus1_C = "n:phiu:bus1:C", "deg, Line-Ground Voltage, Angle"
      n_phiu_bus1_N = "n:phiu:bus1:N", "deg, Line-Ground Voltage, Angle"
      n_phiurel_bus1 = "n:phiurel:bus1", "deg, Voltage, Relative Angle"
      n_ul_bus1_A = "n:ul:bus1:A", "p.u., Line-Line Voltage, Magnitude"
      n_ul_bus1_B = "n:ul:bus1:B", "p.u., Line-Line Voltage, Magnitude"
      n_ul_bus1_C = "n:ul:bus1:C", "p.u., Line-Line Voltage, Magnitude"
      n_ul_bus1_N = "n:ul:bus1:N", "p.u., Line-Line Voltage, Magnitude"
      n_ulpc_bus1_A = "n:ulpc:bus1:A", "%, Line-Line Voltage, Magnitude"
      n_ulpc_bus1_B = "n:ulpc:bus1:B", "%, Line-Line Voltage, Magnitude"
      n_ulpc_bus1_C = "n:ulpc:bus1:C", "%, Line-Line Voltage, Magnitude"
      n_ulpc_bus1_N = "n:ulpc:bus1:N", "%, Line-Line Voltage, Magnitude"
      n_Ul_bus1_A = "n:Ul:bus1:A", "kV, Line-Line Voltage, Magnitude"
      n_Ul_bus1_B = "n:Ul:bus1:B", "kV, Line-Line Voltage, Magnitude"
      n_Ul_bus1_C = "n:Ul:bus1:C", "kV, Line-Line Voltage, Magnitude"
      n_Ul_bus1_N = "n:Ul:bus1:N", "kV, Line-Line Voltage, Magnitude"
      n_phiul_bus1_A = "n:phiul:bus1:A", "deg, Line-Line Voltage, Angle"
      n_phiul_bus1_B = "n:phiul:bus1:B", "deg, Line-Line Voltage, Angle"
      n_phiul_bus1_C = "n:phiul:bus1:C", "deg, Line-Line Voltage, Angle"
      n_phiul_bus1_N = "n:phiul:bus1:N", "deg, Line-Line Voltage, Angle"
      n_u0_bus1 = "n:u0:bus1", "p.u., Zero-Sequence Voltage, Magnitude"
      n_U0_bus1 = "n:U0:bus1", "kV, Zero-Sequence Voltage, Magnitude"
      n_U0x3_bus1 = "n:U0x3:bus1", "kV, 3*U0"
      n_phiu0_bus1 = "n:phiu0:bus1", "deg, Zero-Sequence Voltage, Angle"
      n_u1_bus1 = "n:u1:bus1", "p.u., Positive-Sequence Voltage, Magnitude"
      n_u1pc_bus1 = "n:u1pc:bus1", "%, Positive-Sequence Voltage, Magnitude"
      n_u1r_bus1 = "n:u1r:bus1", "p.u., Positive-Sequence Voltage, Real Part"
      n_u1i_bus1 = "n:u1i:bus1", "p.u., Positive-Sequence Voltage, Imaginary Part"
      n_U1_bus1 = "n:U1:bus1", "kV, Line-Ground Positive-Sequence Voltage, Magnitude"
      n_phiu1_bus1 = "n:phiu1:bus1", "deg, Positive-Sequence Voltage, Angle"
      n_u2_bus1 = "n:u2:bus1", "p.u., Negative-Sequence Voltage, Magnitude"
      n_U2_bus1 = "n:U2:bus1", "kV, Line-Ground Negative-Sequence Voltage, Magnitude"
      n_phiu2_bus1 = "n:phiu2:bus1", "deg, Negative-Sequence Voltage, Angle"
      n_U1l_bus1 = "n:U1l:bus1", "kV, Line-Line Positive-Sequence Voltage, Magnitude"
      n_U2l_bus1 = "n:U2l:bus1", "kV, Line-Line Negative-Sequence Voltage, Magnitude"
      n_uln_bus1_A = "n:uln:bus1:A", "p.u., Line-Neutral Voltage, Magnitude"
      n_uln_bus1_B = "n:uln:bus1:B", "p.u., Line-Neutral Voltage, Magnitude"
      n_uln_bus1_C = "n:uln:bus1:C", "p.u., Line-Neutral Voltage, Magnitude"
      n_uln_bus1_N = "n:uln:bus1:N", "p.u., Line-Neutral Voltage, Magnitude"
      n_Uln_bus1_A = "n:Uln:bus1:A", "kV, Line-Neutral Voltage, Magnitude"
      n_Uln_bus1_B = "n:Uln:bus1:B", "kV, Line-Neutral Voltage, Magnitude"
      n_Uln_bus1_C = "n:Uln:bus1:C", "kV, Line-Neutral Voltage, Magnitude"
      n_Uln_bus1_N = "n:Uln:bus1:N", "kV, Line-Neutral Voltage, Magnitude"
      n_phiuln_bus1_A = "n:phiuln:bus1:A", "deg, Line-Neutral Voltage, Angle"
      n_phiuln_bus1_B = "n:phiuln:bus1:B", "deg, Line-Neutral Voltage, Angle"
      n_phiuln_bus1_C = "n:phiuln:bus1:C", "deg, Line-Neutral Voltage, Angle"
      n_phiuln_bus1_N = "n:phiuln:bus1:N", "deg, Line-Neutral Voltage, Angle"
      n_un_bus1 = "n:un:bus1", "p.u., Neutral-Ground Voltage, Magnitude"
      n_Un_bus1 = "n:Un:bus1", "kV, Neutral-Ground Voltage, Magnitude"
      n_phiun_bus1 = "n:phiun:bus1", "deg, Neutral-Ground Voltage, Angle"
      n_upht_bus1_A = "n:upht:bus1:A", "p.u., Phase Technology dependent Voltage, Magnitude"
      n_upht_bus1_B = "n:upht:bus1:B", "p.u., Phase Technology dependent Voltage, Magnitude"
      n_upht_bus1_C = "n:upht:bus1:C", "p.u., Phase Technology dependent Voltage, Magnitude"
      n_upht_bus1_N = "n:upht:bus1:N", "p.u., Phase Technology dependent Voltage, Magnitude"
      n_Upht_bus1_A = "n:Upht:bus1:A", "kV, Phase Technology dependent Voltage, Magnitude"
      n_Upht_bus1_B = "n:Upht:bus1:B", "kV, Phase Technology dependent Voltage, Magnitude"
      n_Upht_bus1_C = "n:Upht:bus1:C", "kV, Phase Technology dependent Voltage, Magnitude"
      n_Upht_bus1_N = "n:Upht:bus1:N", "kV, Phase Technology dependent Voltage, Magnitude"
      n_um_bus1 = "n:um:bus1", "p.u., Average-Voltage, Magnitude"
      n_Um_bus1 = "n:Um:bus1", "kV, Average-Voltage, Magnitude"
      n_uphtmin_bus1 = "n:uphtmin:bus1", "p.u., Minimum of Phase Technology dependent Voltage, Magnitude"
      n_uphtmax_bus1 = "n:uphtmax:bus1", "p.u., Maximum of Phase Technology dependent Voltage, Magnitude"
      n_Pgen_bus1 = "n:Pgen:bus1", "MW, Generation, Active Power"
      n_Qgen_bus1 = "n:Qgen:bus1", "Mvar, Generation, Reactive Power"
      n_Pmot_bus1 = "n:Pmot:bus1", "MW, Motor Load, Active Power"
      n_Qmot_bus1 = "n:Qmot:bus1", "Mvar, Motor Load, Reactive Power"
      n_Pload_bus1 = "n:Pload:bus1", "MW, General Load, Active Power"
      n_Qload_bus1 = "n:Qload:bus1", "Mvar, General Load, Reactive Power"
      n_Pcomp_bus1 = "n:Pcomp:bus1", "MW, Compensation (Losses)"
      n_Qcomp_bus1 = "n:Qcomp:bus1", "Mvar, Compensation"
      n_Pnet_bus1 = "n:Pnet:bus1", "MW, External Networks, Active Power"
      n_Qnet_bus1 = "n:Qnet:bus1", "Mvar, External Networks, Reactive Power"
      n_Pflow_bus1 = "n:Pflow:bus1", "MW, Power Flow, Active Power"
      n_Qflow_bus1 = "n:Qflow:bus1", "Mvar, Power Flow, Reactive Power"
      n_Pout_bus1 = "n:Pout:bus1", "MW, Outgoing Flow, Active Power"
      n_Qout_bus1 = "n:Qout:bus1", "Mvar, Outgoing Power, Reactive Power"
      n_Sout_bus1 = "n:Sout:bus1", "MVA, Outgoing Power, Apparent Power"
      n_cosphiout_bus1 = "n:cosphiout:bus1", "Outgoing Power, Power Factor"
      n_Pbalance_bus1 = "n:Pbalance:bus1", "MW, Active Power Balance (=0)"
      n_Qbalance_bus1 = "n:Qbalance:bus1", "Mvar, Reactive Power Balance (=0)"
      n_ubfac_bus1 = "n:ubfac:bus1", "%, Unbalance Factor"
      n_LossPdown_bus1 = "n:LossPdown:bus1", "MW, Losses, downstream"
      n_LossQdown_bus1 = "n:LossQdown:bus1", "Mvar, Losses, downstream (Reactive Power)"
      n_LossPdownload_bus1 = "n:LossPdownload:bus1", "MW, Load losses, downstream"
      n_LossQdownload_bus1 = "n:LossQdownload:bus1", "Mvar, Load losses, downstream"
      n_LossPdownnoload_bus1 = "n:LossPdownnoload:bus1", "MW, No load losses, downstream"
      n_LossQdownnoload_bus1 = "n:LossQdownnoload:bus1", "Mvar, No load losses, downstream"
      s_scale = "s:scale", "IN, Scale Factor, continuous, double"
      s_scalephi = "s:scalephi", "rad, IN, Load Scaling: Power Angle, continuous, double"
      s_scale1r = "s:scale1r", "IN, Feeder load scaling, control 1, ph. 1, continuous, double"
      s_scale1s = "s:scale1s", "IN, Feeder load scaling, control 1, ph. 2, continuous, double"
      s_scale1t = "s:scale1t", "IN, Feeder load scaling, control 1, ph. 3, continuous, double"
      s_scale2r = "s:scale2r", "IN, Feeder load scaling, control 2, ph. 1, continuous, double"
      s_scale2s = "s:scale2s", "IN, Feeder load scaling, control 2, ph. 2, continuous, double"
      s_scale2t = "s:scale2t", "IN, Feeder load scaling, control 2, ph. 3, continuous, double"
      c_prini = "c:prini", "MW, Nominal Active Power, in phase A"
      c_qrini = "c:qrini", "Mvar, Nominal Reactive Power in phase A"
      c_psini = "c:psini", "MW, Nominal Active Power, in phase B"
      c_qsini = "c:qsini", "Mvar, Nominal Reactive Power in phase B"
      c_ptini = "c:ptini", "MW, Nominal Active Power, in phase C"
      c_qtini = "c:qtini", "Mvar, Nominal Reactive Power in phase C"
      c_sr_r = "c:sr:r", "MW, Complex Power in phase A, Real Part"
      c_sr_i = "c:sr:i", "Mvar, Complex Power in phase A, Imaginary Part"
      c_sr_phi = "c:sr:phi", "deg, Complex Power in phase A, Angle"
      c_sr_mag = "c:sr:mag", "MVA, Complex Power in phase A, Magnitude"
      c_sr_phirad = "c:sr:phirad", "rad, Complex Power in phase A, Angle"
      c_ss_r = "c:ss:r", "MW, Complex Power in phase B, Real Part"
      c_ss_i = "c:ss:i", "Mvar, Complex Power in phase B, Imaginary Part"
      c_ss_phi = "c:ss:phi", "deg, Complex Power in phase B, Angle"
      c_ss_mag = "c:ss:mag", "MVA, Complex Power in phase B, Magnitude"
      c_ss_phirad = "c:ss:phirad", "rad, Complex Power in phase B, Angle"
      c_st_r = "c:st:r", "MW, Complex Power in phase C, Real Part"
      c_st_i = "c:st:i", "Mvar, Complex Power in phase C, Imaginary Part"
      c_st_phi = "c:st:phi", "deg, Complex Power in phase C, Angle"
      c_st_mag = "c:st:mag", "MVA, Complex Power in phase C, Magnitude"
      c_st_phirad = "c:st:phirad", "rad, Complex Power in phase C, Angle"
      c_frnom = "c:frnom", "Hz, Nominal Frequency"
      c_hpi = "c:hpi", "rad/s, Nominal angular frequency"
      c_u0 = "c:u0", "p.u., Nominal Voltage"
      c_ncust = "c:ncust", "Number of Connected Customers"
      c_scale0 = "c:scale0", "Initial Scale Factor"
      c_zonescale = "c:zonescale", "Zone Scale Factor"
      c_kpu0 = "c:kpu0", "Volt. Dependence I on P"
      c_kqu0 = "c:kqu0", "Volt. Dependence I on Q"
      c_kpu1 = "c:kpu1", "Volt. Dependence II on P"
      c_kqu1 = "c:kqu1", "Volt. Dependence II on Q"
      c_kpu = "c:kpu", "Volt. Dependence III on P"
      c_kqu = "c:kqu", "Volt. Dependence III on Q"
      c_aP = "c:aP", "Percent of Volt. Dep. I on P"
      c_bP = "c:bP", "Percent of Volt. Dep. II on P"
      c_aQ = "c:aQ", "Percent of Volt. Dep. I on Q"
      c_bQ = "c:bQ", "Percent of Volt. Dep. II on Q"
      c_z0_r = "c:z0:r", "p.u., z0, Real Part"
      c_z0_i = "c:z0:i", "p.u., z0, Imaginary Part"
      c_z0_phi = "c:z0:phi", "deg, z0, Angle"
      c_z0_mag = "c:z0:mag", "p.u., z0, Magnitude"
      c_z0_phirad = "c:z0:phirad", "rad, z0, Angle"
      c_iZzero = "c:iZzero", "..."
      c_iRefElement = "c:iRefElement", "Reference Element"
      c_Pmism = "c:Pmism", "MW, Active Power Mismatch"
      c_Ploss = "c:Ploss", "MW, Losses (total)"
      c_Qloss = "c:Qloss", "Mvar, Reactive-Losses (total)"
      c_Plossld = "c:Plossld", "MW, Losses (load)"
      c_Qlossld = "c:Qlossld", "Mvar, Reactive-Losses (load)"
      c_Plossnld = "c:Plossnld", "MW, Losses (no load)"
      c_Qlossnld = "c:Qlossnld", "Mvar, Reactive-Losses (no load)"
      e_mode_inp = "e:mode_inp", "Input Mode"
      e_i_sym = "e:i_sym", "Balanced/Unbalanced"
      e_plini = "e:plini", "MW, Active Power"
      e_qlini = "e:qlini", "Mvar, Reactive Power"
      e_slini = "e:slini", "MVA, Apparent Power"
      e_ilini = "e:ilini", "kA, Current"
      e_coslini = "e:coslini", "Power Factor"
      e_pf_recap = "e:pf_recap", "Power Factor"
      e_p_direc = "e:p_direc", "Power Direction"
      e_u0 = "e:u0", "p.u., Voltage"
      e_scale0 = "e:scale0", "Scaling Factor"
      e_i_scale = "e:i_scale", "Adjusted by Load Scaling"
      e_plinir = "e:plinir", "MW, Active Power"
      e_qlinir = "e:qlinir", "Mvar, Reactive Power"
      e_slinir = "e:slinir", "MVA, Apparent Power"
      e_ilinir = "e:ilinir", "kA, Current"
      e_coslinir = "e:coslinir", "Power Factor"
      e_pf_recapr = "e:pf_recapr", "Power Factor"
      e_p_direcr = "e:p_direcr", "Power Direction"
      e_plinis = "e:plinis", "MW, Active Power"
      e_qlinis = "e:qlinis", "Mvar, Reactive Power"
      e_slinis = "e:slinis", "MVA, Apparent Power"
      e_ilinis = "e:ilinis", "kA, Current"
      e_coslinis = "e:coslinis", "Power Factor"
      e_pf_recaps = "e:pf_recaps", "Power Factor"
      e_p_direcs = "e:p_direcs", "Power Direction"
      e_plinit = "e:plinit", "MW, Active Power"
      e_qlinit = "e:qlinit", "Mvar, Reactive Power"
      e_slinit = "e:slinit", "MVA, Apparent Power"
      e_ilinit = "e:ilinit", "kA, Current"
      e_coslinit = "e:coslinit", "Power Factor"
      e_pf_recapt = "e:pf_recapt", "Power Factor"
      e_p_direct = "e:p_direct", "Power Direction"
      e_qdslCtrl = "e:qdslCtrl", "Quasi-Dynamic Model"
      e_zonefact = "e:zonefact", "Zone Scaling Factor:"
      e_i_rem = "e:i_rem", "Remote Control"
      e_p_cub = "e:p_cub", "StaCubic*, Controlled Branch (Cubicle)"
      e_plini_a = "e:plini_a", "MW, P(act.)"
      e_qlini_a = "e:qlini_a", "Mvar, Q(act.)"
      e_slini_a = "e:slini_a", "MVA, S(act.)"
      e_coslini_a = "e:coslini_a", "cos(phi)(act.)"
      e_ilini_a = "e:ilini_a", "kA, I(act.)"
      e_scale0_a = "e:scale0_a", "Scaling Factor(act.)"
      e_plinir_a = "e:plinir_a", "MW, Ph 1:P(act.)"
      e_qlinir_a = "e:qlinir_a", "Mvar, Ph 1:Q(act.)"
      e_slinir_a = "e:slinir_a", "MVA, Ph 1:S(act.)"
      e_coslinir_a = "e:coslinir_a", "Ph 1:cos(phi)(act.)"
      e_ilinir_a = "e:ilinir_a", "kA, Ph 1:I(act.)"
      e_plinis_a = "e:plinis_a", "MW, Ph 2:P(act.)"
      e_qlinis_a = "e:qlinis_a", "Mvar, Ph 2:Q(act.)"
      e_slinis_a = "e:slinis_a", "MVA, Ph 2:S(act.)"
      e_coslinis_a = "e:coslinis_a", "Ph 2:cos(phi)(act.)"
      e_ilinis_a = "e:ilinis_a", "kA, Ph 2:I(act.)"
      e_plinit_a = "e:plinit_a", "MW, Ph 3:P(act.)"
      e_qlinit_a = "e:qlinit_a", "Mvar, Ph 3:Q(act.)"
      e_slinit_a = "e:slinit_a", "MVA, Ph 3:S(act.)"
      e_coslinit_a = "e:coslinit_a", "Ph 3:cos(phi)(act.)"
      e_ilinit_a = "e:ilinit_a", "kA, Ph 3:I(act.)"
      e_cpFeed = "e:cpFeed", "Feeder"
      e_ciDist = "e:ciDist", "Distance from infeed in number of buses"
      e_ciLater = "e:ciLater", "Lateral Index"
      e_ciDistRoot = "e:ciDistRoot", "Distance from first infeed in number of buses"
      e_ciDistAll = "e:ciDistAll", "Distance from infeed in number of buses including switches"
      e_ciDistAllRoot = "e:ciDistAllRoot", "Distance from first infeed in number of buses including switches"
      r_qdslCtrl = "r:qdslCtrl", "Quasi-Dynamic Model"
      r_p_cub = "r:p_cub", "StaCubic*, Controlled Branch (Cubicle)"
      r_cpFeed = "r:cpFeed", "Feeder"
      m_Ikss_bus1 = "m:Ikss:bus1", "kA, Initial Short-Circuit Current"
      m_ikss_bus1 = "m:ikss:bus1", "p.u., Initial Short-Circuit Current, Magnitude"
      m_phii_bus1 = "m:phii:bus1", "deg, Phase Current, Angle"
      m_maxIkss_bus1 = "m:maxIkss:bus1", "kA, Max. Initial Short-Circuit Current"
      m_Skss_bus1 = "m:Skss:bus1", "MVA, Initial Short-Circuit Power"
      m_ip_bus1 = "m:ip:bus1", "kA, Peak Short-Circuit Current"
      m_maxip_bus1 = "m:maxip:bus1", "kA, Max. Peak Short-Circuit Current"
      m_maxIb_bus1 = "m:maxIb:bus1", "kA, Max. Breaking Current"
      m_maxIth_bus1 = "m:maxIth:bus1", "kA, Max. Thermal equivalent Short-Circuit Current"
      m_Ib_bus1 = "m:Ib:bus1", "kA, Short-Circuit Breaking Current"
      m_Sb_bus1 = "m:Sb:bus1", "MVA, Short-Circuit Breaking Power"
      m_Ik_bus1 = "m:Ik:bus1", "kA, Steady-State Short-Circuit Current"
      m_Ith_bus1 = "m:Ith:bus1", "kA, Thermal equivalent Short-Circuit Current"
      m_I_bus1 = "m:I:bus1", "kA, Short-Circuit Current"
      m_R_bus1 = "m:R:bus1", "Ohm, Fault Impedance, Real Part"
      m_X_bus1 = "m:X:bus1", "Ohm, Fault Impedance, Imaginary Part"
      m_Z_bus1 = "m:Z:bus1", "Ohm, Fault Impedance, Magnitude"
      m_phiz_bus1 = "m:phiz:bus1", "deg, Fault Impedance, Angle"
      m_zSbase_bus1 = "m:zSbase:bus1", "p.u., Fault Impedance, Magnitude"
      m_XtoR_bus1 = "m:XtoR:bus1", "X/R ratio"
      m_Zs_bus1 = "m:Zs:bus1", "Ohm, Source Impedance, Magnitude"
      m_phizs_bus1 = "m:phizs:bus1", "deg, Source Impedance, Angle"
      m_Rs_bus1 = "m:Rs:bus1", "Ohm, Source Impedance, Real Part"
      m_Xs_bus1 = "m:Xs:bus1", "Ohm, Source Impedance, Imaginary Part"
      m_phiui_bus1 = "m:phiui:bus1", "deg, Angle between Voltage and Current"
      m_cosphiui_bus1 = "m:cosphiui:bus1", "cosphi"
      m_P_bus1 = "m:P:bus1", "MW, Active Power"
      m_Q_bus1 = "m:Q:bus1", "Mvar, Reactive Power"
      m_BrkIpload_bus1 = "m:BrkIpload:bus1", "%, Breaker Loading (peak)"
      m_BrkIbload_bus1 = "m:BrkIbload:bus1", "%, Breaker Loading (breaking)"
      m_BrkmaxIb_bus1 = "m:BrkmaxIb:bus1", "kA, Max. Breaking Current (breaker time)"
      n_Ikss_bus1 = "n:Ikss:bus1", "kA, Initial Short-Circuit Current"
      n_I_bus1 = "n:I:bus1", "kA, Short-Circuit Current"
      n_phii_bus1 = "n:phii:bus1", "deg, Phase Current, Angle"
      n_Skss_bus1 = "n:Skss:bus1", "MVA, Initial Short-Circuit Power"
      n_ip_bus1 = "n:ip:bus1", "kA, Peak Short-Circuit Current"
      n_Ib_bus1 = "n:Ib:bus1", "kA, Short-Circuit Breaking Current"
      n_Sb_bus1 = "n:Sb:bus1", "MVA, Short-Circuit Breaking Power"
      n_Ik_bus1 = "n:Ik:bus1", "kA, Steady-State Short-Circuit Current"
      n_Ith_bus1 = "n:Ith:bus1", "kA, Thermal equivalent Short-Circuit Current"
      n_R_bus1 = "n:R:bus1", "Ohm, Short-Circuit Impedance, Real Part"
      n_X_bus1 = "n:X:bus1", "Ohm, Short-Circuit Impedance, Imaginary Part"
      n_Z_bus1 = "n:Z:bus1", "Ohm, Short-Circuit Impedance, Magnitude"
      n_phiz_bus1 = "n:phiz:bus1", "deg, Short-Circuit Impedance, Angle"
      n_rSbase_bus1 = "n:rSbase:bus1", "p.u., Short-Circuit Impedance, Real Part"
      n_xSbase_bus1 = "n:xSbase:bus1", "p.u., Short-Circuit Impedance, Imaginary Part"
      n_zSbase_bus1 = "n:zSbase:bus1", "p.u., Short-Circuit Impedance, Magnitude"
      n_XtoR_bus1 = "n:XtoR:bus1", "X/R ratio (peak)"
      n_RtoX_bus1 = "n:RtoX:bus1", "R/X ratio (peak)"
      n_ur_bus1 = "n:ur:bus1", "p.u., Voltage, Real Part"
      n_ui_bus1 = "n:ui:bus1", "p.u., Voltage, Imaginary Part"
      n_u_bus1 = "n:u:bus1", "p.u., Voltage, Magnitude"
      n_upc_bus1 = "n:upc:bus1", "%, Voltage, Magnitude"
      n_U_bus1 = "n:U:bus1", "kV, Line-Ground Voltage, Magnitude"
      n_Ul_bus1 = "n:Ul:bus1", "kV, Line-Line Voltage, Magnitude"
      n_phiu_bus1 = "n:phiu:bus1", "deg, Voltage, Angle"
      n_du_bus1 = "n:du:bus1", "%, Voltage Deviation"
      n_urpref_bus1 = "n:urpref:bus1", "p.u., Pre-fault Voltage, Real Part"
      n_uipref_bus1 = "n:uipref:bus1", "p.u., Pre-fault Voltage, Imaginary Part"
      n_upref_bus1 = "n:upref:bus1", "p.u., Pre-fault Voltage, Magnitude"
      n_uprefpc_bus1 = "n:uprefpc:bus1", "%, Pre-fault Voltage, Magnitude"
      n_Upref_bus1 = "n:Upref:bus1", "kV, Line-Ground Pre-fault Voltage, Magnitude"
      n_Ulpref_bus1 = "n:Ulpref:bus1", "kV, Line-Line Pre-fault Voltage, Magnitude"
      n_phiupref_bus1 = "n:phiupref:bus1", "deg, Pre-fault Voltage, Angle"
      n_phiui_bus1 = "n:phiui:bus1", "deg, Angle between Voltage and Current"
      n_cfac_bus1 = "n:cfac:bus1", "Voltage Factor"
      n_idc_bus1 = "n:idc:bus1", "kA, Decaying Component"
      n_Iasy_bus1 = "n:Iasy:bus1", "kA, Asymmetrical Breaking Current"
      n_Ithload_bus1 = "n:Ithload:bus1", "%, Loading, Thermal equivalent Current"
      n_Ipload_bus1 = "n:Ipload:bus1", "%, Loading, Peak Short-Circuit Current"
      n_Ithrtk_bus1 = "n:Ithrtk:bus1", "kA, Rated Short-Time Current (Tk)"
      n_Tn_bus1 = "n:Tn:bus1", "ms, Grid Time Constant"
      n_XtoR_b_bus1 = "n:XtoR_b:bus1", "X/R ratio (dc)"
      n_RtoX_b_bus1 = "n:RtoX_b:bus1", "R/X ratio (dc)"
      n_fctof_b_bus1 = "n:fctof_b:bus1", "Equivalent Frequency fc/f (dc)"
      c_i_pure = "c:i_pure", "Purely inductive/capacitive"
      c_icsrc = "c:icsrc", "Current Source/Impedance"
      c_R = "c:R", "p.u., Resistance"
      c_L = "c:L", "p.u., Inductance"
      c_C = "c:C", "p.u., Capacitance"
      c_C_lv = "c:C_lv", "p.u., LV-Capacitance"
      c_y1_r = "c:y1:r", "p.u., Admittance, Real Part"
      c_y1_i = "c:y1:i", "p.u., Admittance, Imaginary Part"
      c_y1_phi = "c:y1:phi", "deg, Admittance, Angle"
      c_y1_mag = "c:y1:mag", "p.u., Admittance, Magnitude"
      c_y1_phirad = "c:y1:phirad", "rad, Admittance, Angle"
      c_i1_r = "c:i1:r", "p.u., Harmonic current, Real Part"
      c_i1_i = "c:i1:i", "p.u., Harmonic current, Imaginary Part"
      c_i1_phi = "c:i1:phi", "deg, Harmonic current, Angle"
      c_i1_mag = "c:i1:mag", "p.u., Harmonic current, Magnitude"
      c_i1_phirad = "c:i1:phirad", "rad, Harmonic current, Angle"
      c_i1iec_r = "c:i1iec:r", "p.u., Harmonic IEC current, Real Part"
      c_i1iec_i = "c:i1iec:i", "p.u., Harmonic IEC current, Imaginary Part"
      c_i1iec_phi = "c:i1iec:phi", "deg, Harmonic IEC current, Angle"
      c_i1iec_mag = "c:i1iec:mag", "p.u., Harmonic IEC current, Magnitude"
      c_i1iec_phirad = "c:i1iec:phirad", "rad, Harmonic IEC current, Angle"
      e_iecfltcont = "e:iecfltcont", "Fault Contribution"
      e_iecShcModel = "e:iecShcModel", "Short-Circuit Model"
      e_Skss = "e:Skss", "MVA/MW, Subtransient Short-Circuit Level"
      e_Ikss = "e:Ikss", "kA/MW, Subtransient Short-Circuit Current"
      e_Skssfix = "e:Skssfix", "MVA, Subtransient Short-Circuit Level"
      e_Ikssfix = "e:Ikssfix", "kA, Subtransient Short-Circuit Level"
      e_Skssfixmin = "e:Skssfixmin", "MVA, Subtransient Short-Circuit Level"
      e_Ikssfixmin = "e:Ikssfixmin", "kA, Subtransient Short-Circuit Level"
      e_rtox = "e:rtox", "R to X'' ratio"
      e_xtor = "e:xtor", "X'' to R ratio"
      e_rtoxmin = "e:rtoxmin", "R to X'' ratio"
      e_xtormin = "e:xtormin", "X'' to R ratio"
      e_ctotSkss = "e:ctotSkss", "MVA, Total contribution (subtransient)"
      e_ctotIkss = "e:ctotIkss", "kA, Total contribution (subtransient)"
      e_ctotSkssmin = "e:ctotSkssmin", "MVA, Total contribution (subtransient)"
      e_ctotIkssmin = "e:ctotIkssmin", "kA, Total contribution (subtransient)"
      m_Ikss_bus1_A = "m:Ikss:bus1:A", "kA, Initial Short-Circuit Current"
      m_Ikss_bus1_B = "m:Ikss:bus1:B", "kA, Initial Short-Circuit Current"
      m_Ikss_bus1_C = "m:Ikss:bus1:C", "kA, Initial Short-Circuit Current"
      m_Ikss_bus1_N = "m:Ikss:bus1:N", "kA, Initial Short-Circuit Current"
      m_ikss_bus1_A = "m:ikss:bus1:A", "p.u., Initial Short-Circuit Current, Magnitude"
      m_ikss_bus1_B = "m:ikss:bus1:B", "p.u., Initial Short-Circuit Current, Magnitude"
      m_ikss_bus1_C = "m:ikss:bus1:C", "p.u., Initial Short-Circuit Current, Magnitude"
      m_ikss_bus1_N = "m:ikss:bus1:N", "p.u., Initial Short-Circuit Current, Magnitude"
      m_Skss_bus1_A = "m:Skss:bus1:A", "MVA, Initial Short-Circuit Power"
      m_Skss_bus1_B = "m:Skss:bus1:B", "MVA, Initial Short-Circuit Power"
      m_Skss_bus1_C = "m:Skss:bus1:C", "MVA, Initial Short-Circuit Power"
      m_Skss_bus1_N = "m:Skss:bus1:N", "MVA, Initial Short-Circuit Power"
      m_Skssx3_bus1_A = "m:Skssx3:bus1:A", "MVA, 3*Initial Short-Circuit Power"
      m_Skssx3_bus1_B = "m:Skssx3:bus1:B", "MVA, 3*Initial Short-Circuit Power"
      m_Skssx3_bus1_C = "m:Skssx3:bus1:C", "MVA, 3*Initial Short-Circuit Power"
      m_Skssx3_bus1_N = "m:Skssx3:bus1:N", "MVA, 3*Initial Short-Circuit Power"
      m_ip_bus1_A = "m:ip:bus1:A", "kA, Peak Short-Circuit Current"
      m_ip_bus1_B = "m:ip:bus1:B", "kA, Peak Short-Circuit Current"
      m_ip_bus1_C = "m:ip:bus1:C", "kA, Peak Short-Circuit Current"
      m_ip_bus1_N = "m:ip:bus1:N", "kA, Peak Short-Circuit Current"
      m_Z_bus1_A = "m:Z:bus1:A", "Ohm, Fault Impedance, Magnitude"
      m_Z_bus1_B = "m:Z:bus1:B", "Ohm, Fault Impedance, Magnitude"
      m_Z_bus1_C = "m:Z:bus1:C", "Ohm, Fault Impedance, Magnitude"
      m_Z_bus1_N = "m:Z:bus1:N", "Ohm, Fault Impedance, Magnitude"
      m_phiz_bus1_A = "m:phiz:bus1:A", "deg, Fault Impedance, Angle"
      m_phiz_bus1_B = "m:phiz:bus1:B", "deg, Fault Impedance, Angle"
      m_phiz_bus1_C = "m:phiz:bus1:C", "deg, Fault Impedance, Angle"
      m_phiz_bus1_N = "m:phiz:bus1:N", "deg, Fault Impedance, Angle"
      m_Zl_bus1_A = "m:Zl:bus1:A", "Ohm, Line-Line Fault Impedance, Magnitude"
      m_Zl_bus1_B = "m:Zl:bus1:B", "Ohm, Line-Line Fault Impedance, Magnitude"
      m_Zl_bus1_C = "m:Zl:bus1:C", "Ohm, Line-Line Fault Impedance, Magnitude"
      m_Zl_bus1_N = "m:Zl:bus1:N", "Ohm, Line-Line Fault Impedance, Magnitude"
      m_phizl_bus1_A = "m:phizl:bus1:A", "deg, Line-Line Fault Impedance, Angle"
      m_phizl_bus1_B = "m:phizl:bus1:B", "deg, Line-Line Fault Impedance, Angle"
      m_phizl_bus1_C = "m:phizl:bus1:C", "deg, Line-Line Fault Impedance, Angle"
      m_phizl_bus1_N = "m:phizl:bus1:N", "deg, Line-Line Fault Impedance, Angle"
      m_Z0_bus1 = "m:Z0:bus1", "Ohm, Zero-Sequence Fault Impedance, Magnitude"
      m_phiz0_bus1 = "m:phiz0:bus1", "deg, Zero-Sequence Fault Impedance, Angle"
      m_z0Sbase_bus1 = "m:z0Sbase:bus1", "p.u., Zero-Sequence Fault Impedance, Magnitude"
      m_cosphiu1i1_bus1 = "m:cosphiu1i1:bus1", "Positive-Sequence cosphi"
      m_I0x3watt_bus1 = "m:I0x3watt:bus1", "kA, Earth-Fault Residual Current"
      m_I0x3var_bus1 = "m:I0x3var:bus1", "kA, Capacitive Earth-Fault Current"
      m_Q0cap_bus1 = "m:Q0cap:bus1", "Mvar, Q0 capacitive"
      m_Zfrel_bus1 = "m:Zfrel:bus1", "Ohm, Relay Impedance, Magnitude"
      m_phizfrel_bus1 = "m:phizfrel:bus1", "deg, Relay Impedance, Angle"
      m_Rfrel_bus1 = "m:Rfrel:bus1", "Ohm, Relay Impedance, Real Part"
      m_Xfrel_bus1 = "m:Xfrel:bus1", "Ohm, Relay Impedance, Imaginary Part"
      m_Zf_bus1 = "m:Zf:bus1", "Ohm, Meas. Fault Impedance, Magnitude"
      m_phizf_bus1 = "m:phizf:bus1", "deg, Meas. Fault Impedance, Angle"
      n_Ikss_bus1_A = "n:Ikss:bus1:A", "kA, Initial Short-Circuit Current"
      n_Ikss_bus1_B = "n:Ikss:bus1:B", "kA, Initial Short-Circuit Current"
      n_Ikss_bus1_C = "n:Ikss:bus1:C", "kA, Initial Short-Circuit Current"
      n_Ikss_bus1_N = "n:Ikss:bus1:N", "kA, Initial Short-Circuit Current"
      n_I_bus1_A = "n:I:bus1:A", "kA, Short-Circuit Current"
      n_I_bus1_B = "n:I:bus1:B", "kA, Short-Circuit Current"
      n_I_bus1_C = "n:I:bus1:C", "kA, Short-Circuit Current"
      n_I_bus1_N = "n:I:bus1:N", "kA, Short-Circuit Current"
      n_phii_bus1_A = "n:phii:bus1:A", "deg, Phase Current, Angle"
      n_phii_bus1_B = "n:phii:bus1:B", "deg, Phase Current, Angle"
      n_phii_bus1_C = "n:phii:bus1:C", "deg, Phase Current, Angle"
      n_phii_bus1_N = "n:phii:bus1:N", "deg, Phase Current, Angle"
      n_Skss_bus1_A = "n:Skss:bus1:A", "MVA, Initial Short-Circuit Power"
      n_Skss_bus1_B = "n:Skss:bus1:B", "MVA, Initial Short-Circuit Power"
      n_Skss_bus1_C = "n:Skss:bus1:C", "MVA, Initial Short-Circuit Power"
      n_Skss_bus1_N = "n:Skss:bus1:N", "MVA, Initial Short-Circuit Power"
      n_Skssx3_bus1_A = "n:Skssx3:bus1:A", "MVA, 3*Initial Short-Circuit Power"
      n_Skssx3_bus1_B = "n:Skssx3:bus1:B", "MVA, 3*Initial Short-Circuit Power"
      n_Skssx3_bus1_C = "n:Skssx3:bus1:C", "MVA, 3*Initial Short-Circuit Power"
      n_Skssx3_bus1_N = "n:Skssx3:bus1:N", "MVA, 3*Initial Short-Circuit Power"
      n_ip_bus1_A = "n:ip:bus1:A", "kA, Peak Short-Circuit Current"
      n_ip_bus1_B = "n:ip:bus1:B", "kA, Peak Short-Circuit Current"
      n_ip_bus1_C = "n:ip:bus1:C", "kA, Peak Short-Circuit Current"
      n_ip_bus1_N = "n:ip:bus1:N", "kA, Peak Short-Circuit Current"
      n_Ib_bus1_A = "n:Ib:bus1:A", "kA, Short-Circuit Breaking Current"
      n_Ib_bus1_B = "n:Ib:bus1:B", "kA, Short-Circuit Breaking Current"
      n_Ib_bus1_C = "n:Ib:bus1:C", "kA, Short-Circuit Breaking Current"
      n_Ib_bus1_N = "n:Ib:bus1:N", "kA, Short-Circuit Breaking Current"
      n_Sb_bus1_A = "n:Sb:bus1:A", "MVA, Short-Circuit Breaking Power"
      n_Sb_bus1_B = "n:Sb:bus1:B", "MVA, Short-Circuit Breaking Power"
      n_Sb_bus1_C = "n:Sb:bus1:C", "MVA, Short-Circuit Breaking Power"
      n_Sb_bus1_N = "n:Sb:bus1:N", "MVA, Short-Circuit Breaking Power"
      n_Ith_bus1_A = "n:Ith:bus1:A", "kA, Thermal equivalent Short-Circuit Current"
      n_Ith_bus1_B = "n:Ith:bus1:B", "kA, Thermal equivalent Short-Circuit Current"
      n_Ith_bus1_C = "n:Ith:bus1:C", "kA, Thermal equivalent Short-Circuit Current"
      n_Ith_bus1_N = "n:Ith:bus1:N", "kA, Thermal equivalent Short-Circuit Current"
      n_Ithmax_bus1 = "n:Ithmax:bus1", "kA, Max. Thermal equivalent Short-Circuit Current"
      n_Ik_bus1_A = "n:Ik:bus1:A", "kA, Steady-State Short-Circuit Current"
      n_Ik_bus1_B = "n:Ik:bus1:B", "kA, Steady-State Short-Circuit Current"
      n_Ik_bus1_C = "n:Ik:bus1:C", "kA, Steady-State Short-Circuit Current"
      n_Ik_bus1_N = "n:Ik:bus1:N", "kA, Steady-State Short-Circuit Current"
      n_R0_bus1 = "n:R0:bus1", "Ohm, Zero-Sequence Impedance, Real Part"
      n_X0_bus1 = "n:X0:bus1", "Ohm, Zero-Sequence Impedance, Imaginary Part"
      n_Z0_bus1 = "n:Z0:bus1", "Ohm, Zero-Sequence Impedance, Magnitude"
      n_phiz0_bus1 = "n:phiz0:bus1", "deg, Zero-Sequence Impedance, Angle"
      n_r0Sbase_bus1 = "n:r0Sbase:bus1", "p.u., Zero-Sequence Impedance, Real Part"
      n_x0Sbase_bus1 = "n:x0Sbase:bus1", "p.u., Zero-Sequence Impedance, Imaginary Part"
      n_z0Sbase_bus1 = "n:z0Sbase:bus1", "p.u., Zero-Sequence Impedance, Magnitude"
      n_R1_bus1 = "n:R1:bus1", "Ohm, Positive-Sequence Impedance, Real Part"
      n_X1_bus1 = "n:X1:bus1", "Ohm, Positive-Sequence Impedance, Imaginary Part"
      n_Z1_bus1 = "n:Z1:bus1", "Ohm, Positive-Sequence Impedance, Magnitude"
      n_phiz1_bus1 = "n:phiz1:bus1", "deg, Positive-Sequence Impedance, Angle"
      n_r1Sbase_bus1 = "n:r1Sbase:bus1", "p.u., Positive-Sequence Impedance, Real Part"
      n_x1Sbase_bus1 = "n:x1Sbase:bus1", "p.u., Positive-Sequence Impedance, Imaginary Part"
      n_z1Sbase_bus1 = "n:z1Sbase:bus1", "p.u., Positive-Sequence Impedance, Magnitude"
      n_R2_bus1 = "n:R2:bus1", "Ohm, Negative-Sequence Impedance, Real Part"
      n_X2_bus1 = "n:X2:bus1", "Ohm, Negative-Sequence Impedance, Imaginary Part"
      n_Z2_bus1 = "n:Z2:bus1", "Ohm, Negative-Sequence Impedance, Magnitude"
      n_phiz2_bus1 = "n:phiz2:bus1", "deg, Negative-Sequence Impedance, Angle"
      n_r2Sbase_bus1 = "n:r2Sbase:bus1", "p.u., Negative-Sequence Impedance, Real Part"
      n_x2Sbase_bus1 = "n:x2Sbase:bus1", "p.u., Negative-Sequence Impedance, Imaginary Part"
      n_z2Sbase_bus1 = "n:z2Sbase:bus1", "p.u., Negative-Sequence Impedance, Magnitude"
      n_R_bus1_A = "n:R:bus1:A", "Ohm, Network Resistance, Line-Ground"
      n_R_bus1_B = "n:R:bus1:B", "Ohm, Network Resistance, Line-Ground"
      n_R_bus1_C = "n:R:bus1:C", "Ohm, Network Resistance, Line-Ground"
      n_R_bus1_N = "n:R:bus1:N", "Ohm, Network Resistance, Line-Ground"
      n_X_bus1_A = "n:X:bus1:A", "Ohm, Network Reactance, Line-Ground"
      n_X_bus1_B = "n:X:bus1:B", "Ohm, Network Reactance, Line-Ground"
      n_X_bus1_C = "n:X:bus1:C", "Ohm, Network Reactance, Line-Ground"
      n_X_bus1_N = "n:X:bus1:N", "Ohm, Network Reactance, Line-Ground"
      n_Z_bus1_A = "n:Z:bus1:A", "Ohm, Network Impedance, Line-Ground"
      n_Z_bus1_B = "n:Z:bus1:B", "Ohm, Network Impedance, Line-Ground"
      n_Z_bus1_C = "n:Z:bus1:C", "Ohm, Network Impedance, Line-Ground"
      n_Z_bus1_N = "n:Z:bus1:N", "Ohm, Network Impedance, Line-Ground"
      n_phiz_bus1_A = "n:phiz:bus1:A", "deg, Angle of Network Impedance, Line-Ground"
      n_phiz_bus1_B = "n:phiz:bus1:B", "deg, Angle of Network Impedance, Line-Ground"
      n_phiz_bus1_C = "n:phiz:bus1:C", "deg, Angle of Network Impedance, Line-Ground"
      n_phiz_bus1_N = "n:phiz:bus1:N", "deg, Angle of Network Impedance, Line-Ground"
      n_Rl_bus1_A = "n:Rl:bus1:A", "Ohm, Network Resistance, Line-Line"
      n_Rl_bus1_B = "n:Rl:bus1:B", "Ohm, Network Resistance, Line-Line"
      n_Rl_bus1_C = "n:Rl:bus1:C", "Ohm, Network Resistance, Line-Line"
      n_Rl_bus1_N = "n:Rl:bus1:N", "Ohm, Network Resistance, Line-Line"
      n_Xl_bus1_A = "n:Xl:bus1:A", "Ohm, Network Reactance, Line-Line"
      n_Xl_bus1_B = "n:Xl:bus1:B", "Ohm, Network Reactance, Line-Line"
      n_Xl_bus1_C = "n:Xl:bus1:C", "Ohm, Network Reactance, Line-Line"
      n_Xl_bus1_N = "n:Xl:bus1:N", "Ohm, Network Reactance, Line-Line"
      n_Zl_bus1_A = "n:Zl:bus1:A", "Ohm, Network Impedance, Line-Line"
      n_Zl_bus1_B = "n:Zl:bus1:B", "Ohm, Network Impedance, Line-Line"
      n_Zl_bus1_C = "n:Zl:bus1:C", "Ohm, Network Impedance, Line-Line"
      n_Zl_bus1_N = "n:Zl:bus1:N", "Ohm, Network Impedance, Line-Line"
      n_phizl_bus1_A = "n:phizl:bus1:A", "deg, Angle of Network Impedance, Line-Line"
      n_phizl_bus1_B = "n:phizl:bus1:B", "deg, Angle of Network Impedance, Line-Line"
      n_phizl_bus1_C = "n:phizl:bus1:C", "deg, Angle of Network Impedance, Line-Line"
      n_phizl_bus1_N = "n:phizl:bus1:N", "deg, Angle of Network Impedance, Line-Line"
      n_Rln_bus1_A = "n:Rln:bus1:A", "Ohm, Network Resistance, Line-Neutral"
      n_Rln_bus1_B = "n:Rln:bus1:B", "Ohm, Network Resistance, Line-Neutral"
      n_Rln_bus1_C = "n:Rln:bus1:C", "Ohm, Network Resistance, Line-Neutral"
      n_Rln_bus1_N = "n:Rln:bus1:N", "Ohm, Network Resistance, Line-Neutral"
      n_Xln_bus1_A = "n:Xln:bus1:A", "Ohm, Network Reactance, Line-Neutral"
      n_Xln_bus1_B = "n:Xln:bus1:B", "Ohm, Network Reactance, Line-Neutral"
      n_Xln_bus1_C = "n:Xln:bus1:C", "Ohm, Network Reactance, Line-Neutral"
      n_Xln_bus1_N = "n:Xln:bus1:N", "Ohm, Network Reactance, Line-Neutral"
      n_Zln_bus1_A = "n:Zln:bus1:A", "Ohm, Network Impedance, Line-Neutral"
      n_Zln_bus1_B = "n:Zln:bus1:B", "Ohm, Network Impedance, Line-Neutral"
      n_Zln_bus1_C = "n:Zln:bus1:C", "Ohm, Network Impedance, Line-Neutral"
      n_Zln_bus1_N = "n:Zln:bus1:N", "Ohm, Network Impedance, Line-Neutral"
      n_phizln_bus1_A = "n:phizln:bus1:A", "deg, Angle of Network Impedance, Line-Neutral"
      n_phizln_bus1_B = "n:phizln:bus1:B", "deg, Angle of Network Impedance, Line-Neutral"
      n_phizln_bus1_C = "n:phizln:bus1:C", "deg, Angle of Network Impedance, Line-Neutral"
      n_phizln_bus1_N = "n:phizln:bus1:N", "deg, Angle of Network Impedance, Line-Neutral"
      n_Rn_bus1 = "n:Rn:bus1", "Ohm, Network Resistance, Neutral"
      n_Xn_bus1 = "n:Xn:bus1", "Ohm, Network Reactance, Neutral"
      n_Zn_bus1 = "n:Zn:bus1", "Ohm, Network Impedance, Neutral"
      n_phizn_bus1 = "n:phizn:bus1", "deg, Angle of Network Impedance, Neutral"
      n_eff_bus1_A = "n:eff:bus1:A", "Earth Fault Factor"
      n_eff_bus1_B = "n:eff:bus1:B", "Earth Fault Factor"
      n_eff_bus1_C = "n:eff:bus1:C", "Earth Fault Factor"
      n_eff_bus1_N = "n:eff:bus1:N", "Earth Fault Factor"
      n_X0toX1_bus1 = "n:X0toX1:bus1", "X0/X1"
      n_in_bus1 = "n:in:bus1", "p.u., Neutral Current, Magnitude"
      n_In_bus1 = "n:In:bus1", "kA, Neutral Current, Magnitude"
      n_phiin_bus1 = "n:phiin:bus1", "deg, Neutral Current, Angle"
      n_i0r_bus1 = "n:i0r:bus1", "p.u., Zero-Sequence Current, Real Part"
      n_i0i_bus1 = "n:i0i:bus1", "p.u., Zero-Sequence Current, Imaginary Part"
      n_i0_bus1 = "n:i0:bus1", "p.u., Zero-Sequence Current, Magnitude"
      n_I0_bus1 = "n:I0:bus1", "kA, Zero-Sequence Current, Magnitude"
      n_I0x3_bus1 = "n:I0x3:bus1", "kA, 3*I0"
      n_phii0_bus1 = "n:phii0:bus1", "deg, Zero-Sequence Current, Angle"
      n_i1r_bus1 = "n:i1r:bus1", "p.u., Positive-Sequence Current, Real Part"
      n_i1i_bus1 = "n:i1i:bus1", "p.u., Positive-Sequence Current, Imaginary Part"
      n_i1_bus1 = "n:i1:bus1", "p.u., Positive-Sequence Current, Magnitude"
      n_I1_bus1 = "n:I1:bus1", "kA, Positive-Sequence Current, Magnitude"
      n_phii1_bus1 = "n:phii1:bus1", "deg, Positive-Sequence Current, Angle"
      n_i2r_bus1 = "n:i2r:bus1", "p.u., Negative-Sequence Current, Real Part"
      n_i2i_bus1 = "n:i2i:bus1", "p.u., Negative-Sequence Current, Imaginary Part"
      n_i2_bus1 = "n:i2:bus1", "p.u., Negative-Sequence Current, Magnitude"
      n_I2_bus1 = "n:I2:bus1", "kA, Negative-Sequence Current, Magnitude"
      n_phii2_bus1 = "n:phii2:bus1", "deg, Negative-Sequence Current, Angle"
      n_du_bus1_A = "n:du:bus1:A", "%, Voltage Deviation"
      n_du_bus1_B = "n:du:bus1:B", "%, Voltage Deviation"
      n_du_bus1_C = "n:du:bus1:C", "%, Voltage Deviation"
      n_du_bus1_N = "n:du:bus1:N", "%, Voltage Deviation"
      n_urpref_bus1_A = "n:urpref:bus1:A", "p.u., Pre-fault Voltage, Real Part"
      n_urpref_bus1_B = "n:urpref:bus1:B", "p.u., Pre-fault Voltage, Real Part"
      n_urpref_bus1_C = "n:urpref:bus1:C", "p.u., Pre-fault Voltage, Real Part"
      n_urpref_bus1_N = "n:urpref:bus1:N", "p.u., Pre-fault Voltage, Real Part"
      n_uipref_bus1_A = "n:uipref:bus1:A", "p.u., Pre-fault Voltage, Imaginary Part"
      n_uipref_bus1_B = "n:uipref:bus1:B", "p.u., Pre-fault Voltage, Imaginary Part"
      n_uipref_bus1_C = "n:uipref:bus1:C", "p.u., Pre-fault Voltage, Imaginary Part"
      n_uipref_bus1_N = "n:uipref:bus1:N", "p.u., Pre-fault Voltage, Imaginary Part"
      n_upref_bus1_A = "n:upref:bus1:A", "p.u., Pre-fault Voltage, Magnitude"
      n_upref_bus1_B = "n:upref:bus1:B", "p.u., Pre-fault Voltage, Magnitude"
      n_upref_bus1_C = "n:upref:bus1:C", "p.u., Pre-fault Voltage, Magnitude"
      n_upref_bus1_N = "n:upref:bus1:N", "p.u., Pre-fault Voltage, Magnitude"
      n_uprefpc_bus1_A = "n:uprefpc:bus1:A", "%, Pre-fault Voltage, Magnitude"
      n_uprefpc_bus1_B = "n:uprefpc:bus1:B", "%, Pre-fault Voltage, Magnitude"
      n_uprefpc_bus1_C = "n:uprefpc:bus1:C", "%, Pre-fault Voltage, Magnitude"
      n_uprefpc_bus1_N = "n:uprefpc:bus1:N", "%, Pre-fault Voltage, Magnitude"
      n_Upref_bus1_A = "n:Upref:bus1:A", "kV, Line-Ground Pre-fault Voltage, Magnitude"
      n_Upref_bus1_B = "n:Upref:bus1:B", "kV, Line-Ground Pre-fault Voltage, Magnitude"
      n_Upref_bus1_C = "n:Upref:bus1:C", "kV, Line-Ground Pre-fault Voltage, Magnitude"
      n_Upref_bus1_N = "n:Upref:bus1:N", "kV, Line-Ground Pre-fault Voltage, Magnitude"
      n_Ulpref_bus1_A = "n:Ulpref:bus1:A", "kV, Line-Line Pre-fault Voltage, Magnitude"
      n_Ulpref_bus1_B = "n:Ulpref:bus1:B", "kV, Line-Line Pre-fault Voltage, Magnitude"
      n_Ulpref_bus1_C = "n:Ulpref:bus1:C", "kV, Line-Line Pre-fault Voltage, Magnitude"
      n_Ulpref_bus1_N = "n:Ulpref:bus1:N", "kV, Line-Line Pre-fault Voltage, Magnitude"
      n_phiupref_bus1_A = "n:phiupref:bus1:A", "deg, Pre-fault Voltage, Angle"
      n_phiupref_bus1_B = "n:phiupref:bus1:B", "deg, Pre-fault Voltage, Angle"
      n_phiupref_bus1_C = "n:phiupref:bus1:C", "deg, Pre-fault Voltage, Angle"
      n_phiupref_bus1_N = "n:phiupref:bus1:N", "deg, Pre-fault Voltage, Angle"
      n_phiui_bus1_A = "n:phiui:bus1:A", "deg, Angle between Voltage and Current"
      n_phiui_bus1_B = "n:phiui:bus1:B", "deg, Angle between Voltage and Current"
      n_phiui_bus1_C = "n:phiui:bus1:C", "deg, Angle between Voltage and Current"
      n_phiui_bus1_N = "n:phiui:bus1:N", "deg, Angle between Voltage and Current"
      n_phiu1i1_bus1 = "n:phiu1i1:bus1", "deg, Angle between Voltage and Current in positive sequence system"
      n_phiu2i2_bus1 = "n:phiu2i2:bus1", "deg, Angle between Voltage and Current in negative sequence system"
      n_phiu0i0_bus1 = "n:phiu0i0:bus1", "deg, Angle between Voltage and Current in zero-sequence system"
      c_y_a_r = "c:y_a:r", "p.u., Admittance, Phase a, Real Part"
      c_y_a_i = "c:y_a:i", "p.u., Admittance, Phase a, Imaginary Part"
      c_y_a_phi = "c:y_a:phi", "deg, Admittance, Phase a, Angle"
      c_y_a_mag = "c:y_a:mag", "p.u., Admittance, Phase a, Magnitude"
      c_y_a_phirad = "c:y_a:phirad", "rad, Admittance, Phase a, Angle"
      c_y_b_r = "c:y_b:r", "p.u., Admittance, Phase b, Real Part"
      c_y_b_i = "c:y_b:i", "p.u., Admittance, Phase b, Imaginary Part"
      c_y_b_phi = "c:y_b:phi", "deg, Admittance, Phase b, Angle"
      c_y_b_mag = "c:y_b:mag", "p.u., Admittance, Phase b, Magnitude"
      c_y_b_phirad = "c:y_b:phirad", "rad, Admittance, Phase b, Angle"
      c_y_c_r = "c:y_c:r", "p.u., Admittance, Phase c, Real Part"
      c_y_c_i = "c:y_c:i", "p.u., Admittance, Phase c, Imaginary Part"
      c_y_c_phi = "c:y_c:phi", "deg, Admittance, Phase c, Angle"
      c_y_c_mag = "c:y_c:mag", "p.u., Admittance, Phase c, Magnitude"
      c_y_c_phirad = "c:y_c:phirad", "rad, Admittance, Phase c, Angle"
      c_cn = "c:cn", "Connection Mode"
      c_Ra = "c:Ra", "p.u., Resistance, Phase a"
      c_La = "c:La", "p.u., Inductance, Phase a"
      c_Ca = "c:Ca", "p.u., Capacitance, Phase a"
      c_Ca_lv = "c:Ca_lv", "p.u., LV-Capacitance, Phase a"
      c_Rb = "c:Rb", "p.u., Resistance, Phase b"
      c_Lb = "c:Lb", "p.u., Inductance, Phase b"
      c_Cb = "c:Cb", "p.u., Capacitance, Phase b"
      c_Cb_lv = "c:Cb_lv", "p.u., LV-Capacitance, Phase b"
      c_Rc = "c:Rc", "p.u., Resistance, Phase c"
      c_Lc = "c:Lc", "p.u., Inductance, Phase c"
      c_Cc = "c:Cc", "p.u., Capacitance, Phase c"
      c_Cc_lv = "c:Cc_lv", "p.u., LV-Capacitance, Phase c"
      c_i_abc_r = "c:i_abc:r", "p.u., Harmonic currents, Real Part"
      c_i_abc_i = "c:i_abc:i", "p.u., Harmonic currents, Imaginary Part"
      c_i_abc_phi = "c:i_abc:phi", "deg, Harmonic currents, Angle"
      c_i_abc_mag = "c:i_abc:mag", "p.u., Harmonic currents, Magnitude"
      c_i_abc_phirad = "c:i_abc:phirad", "rad, Harmonic currents, Angle"
      c_i_abc_iec_r = "c:i_abc_iec:r", "p.u., Harmonic IEC current, Real Part"
      c_i_abc_iec_i = "c:i_abc_iec:i", "p.u., Harmonic IEC current, Imaginary Part"
      c_i_abc_iec_phi = "c:i_abc_iec:phi", "deg, Harmonic IEC current, Angle"
      c_i_abc_iec_mag = "c:i_abc_iec:mag", "p.u., Harmonic IEC current, Magnitude"
      c_i_abc_iec_phirad = "c:i_abc_iec:phirad", "rad, Harmonic IEC current, Angle"

    class ElmLodlv(Enum):
      _init_ = 'value __doc__'

      m_u0_bus1 = "m:u0:bus1", "p.u., Zero-Sequence-Voltage, Magnitude"
      m_phiu0_bus1 = "m:phiu0:bus1", "deg, Zero-Sequence-Voltage, Angle"
      m_u1r_bus1 = "m:u1r:bus1", "p.u., Positive-Sequence-Voltage, Real Part"
      m_u1i_bus1 = "m:u1i:bus1", "p.u., Positive-Sequence-Voltage, Imaginary Part"
      m_u1_bus1 = "m:u1:bus1", "p.u., Positive-Sequence-Voltage, Magnitude"
      m_U1_bus1 = "m:U1:bus1", "kV, Line-Ground Positive-Sequence-Voltage, Magnitude"
      m_U1l_bus1 = "m:U1l:bus1", "kV, Line-Line Positive-Sequence-Voltage, Magnitude"
      m_phiu1_bus1 = "m:phiu1:bus1", "deg, Positive-Sequence-Voltage, Angle"
      m_u2_bus1 = "m:u2:bus1", "p.u., Negative-Sequence-Voltage, Magnitude"
      m_phiu2_bus1 = "m:phiu2:bus1", "deg, Negative-Sequence-Voltage, Angle"
      m_inet_bus1_A = "m:inet:bus1:A", "p.u., Phase Current, Magnitude, referred to network"
      m_inet_bus1_B = "m:inet:bus1:B", "p.u., Phase Current, Magnitude, referred to network"
      m_inet_bus1_C = "m:inet:bus1:C", "p.u., Phase Current, Magnitude, referred to network"
      m_inet_bus1_N = "m:inet:bus1:N", "p.u., Phase Current, Magnitude, referred to network"
      m_I_bus1_A = "m:I:bus1:A", "kA, Phase Current, Magnitude"
      m_I_bus1_B = "m:I:bus1:B", "kA, Phase Current, Magnitude"
      m_I_bus1_C = "m:I:bus1:C", "kA, Phase Current, Magnitude"
      m_I_bus1_N = "m:I:bus1:N", "kA, Phase Current, Magnitude"
      m_phii_bus1_A = "m:phii:bus1:A", "deg, Phase Current, Angle"
      m_phii_bus1_B = "m:phii:bus1:B", "deg, Phase Current, Angle"
      m_phii_bus1_C = "m:phii:bus1:C", "deg, Phase Current, Angle"
      m_phii_bus1_N = "m:phii:bus1:N", "deg, Phase Current, Angle"
      m_i0r_bus1 = "m:i0r:bus1", "p.u., Zero-Sequence Current, Real Part"
      m_i0i_bus1 = "m:i0i:bus1", "p.u., Zero-Sequence Current, Imaginary Part"
      m_i0_bus1 = "m:i0:bus1", "p.u., Zero-Sequence Current, Magnitude"
      m_I0_bus1 = "m:I0:bus1", "kA, Zero-Sequence Current, Magnitude"
      m_I0x3_bus1 = "m:I0x3:bus1", "kA, 3*I0"
      m_phii0_bus1 = "m:phii0:bus1", "deg, Zero-Sequence Current, Angle"
      m_i1r_bus1 = "m:i1r:bus1", "p.u., Positive-Sequence Current, Real Part"
      m_i1i_bus1 = "m:i1i:bus1", "p.u., Positive-Sequence Current, Imaginary Part"
      m_i1_bus1 = "m:i1:bus1", "p.u., Positive-Sequence Current, Magnitude"
      m_I1_bus1 = "m:I1:bus1", "kA, Positive-Sequence Current, Magnitude"
      m_phii1_bus1 = "m:phii1:bus1", "deg, Positive-Sequence Current, Angle"
      m_i2r_bus1 = "m:i2r:bus1", "p.u., Negative-Sequence Current, Real Part"
      m_i2i_bus1 = "m:i2i:bus1", "p.u., Negative-Sequence Current, Imaginary Part"
      m_i2_bus1 = "m:i2:bus1", "p.u., Negative-Sequence Current, Magnitude"
      m_I2_bus1 = "m:I2:bus1", "kA, Negative-Sequence Current, Magnitude"
      m_phii2_bus1 = "m:phii2:bus1", "deg, Negative-Sequence Current, Angle"
      m_ubfacI_bus1 = "m:ubfacI:bus1", "%, Current Unbalance Factor"
      m_ubfacS_bus1 = "m:ubfacS:bus1", "%, Power Unbalance Factor"
      m_P_bus1_A = "m:P:bus1:A", "MW, Active Power"
      m_P_bus1_B = "m:P:bus1:B", "MW, Active Power"
      m_P_bus1_C = "m:P:bus1:C", "MW, Active Power"
      m_P_bus1_N = "m:P:bus1:N", "MW, Active Power"
      m_Q_bus1_A = "m:Q:bus1:A", "Mvar, Reactive Power"
      m_Q_bus1_B = "m:Q:bus1:B", "Mvar, Reactive Power"
      m_Q_bus1_C = "m:Q:bus1:C", "Mvar, Reactive Power"
      m_Q_bus1_N = "m:Q:bus1:N", "Mvar, Reactive Power"
      m_S_bus1_A = "m:S:bus1:A", "MVA, Apparent Power"
      m_S_bus1_B = "m:S:bus1:B", "MVA, Apparent Power"
      m_S_bus1_C = "m:S:bus1:C", "MVA, Apparent Power"
      m_S_bus1_N = "m:S:bus1:N", "MVA, Apparent Power"
      m_cosphi_bus1_A = "m:cosphi:bus1:A", "Power Factor"
      m_cosphi_bus1_B = "m:cosphi:bus1:B", "Power Factor"
      m_cosphi_bus1_C = "m:cosphi:bus1:C", "Power Factor"
      m_cosphi_bus1_N = "m:cosphi:bus1:N", "Power Factor"
      m_tanphi_bus1_A = "m:tanphi:bus1:A", "tan(phi)"
      m_tanphi_bus1_B = "m:tanphi:bus1:B", "tan(phi)"
      m_tanphi_bus1_C = "m:tanphi:bus1:C", "tan(phi)"
      m_tanphi_bus1_N = "m:tanphi:bus1:N", "tan(phi)"
      m_Psum_bus1 = "m:Psum:bus1", "MW, Total Active Power"
      m_Qsum_bus1 = "m:Qsum:bus1", "Mvar, Total Reactive Power"
      m_Ssum_bus1 = "m:Ssum:bus1", "MVA, Total Apparent Power"
      m_cosphisum_bus1 = "m:cosphisum:bus1", "Total Power Factor"
      m_tanphisum_bus1 = "m:tanphisum:bus1", "Total tan(phi)"
      m_Pln_bus1_A = "m:Pln:bus1:A", "MW, Active Power (Line-Neutral)"
      m_Pln_bus1_B = "m:Pln:bus1:B", "MW, Active Power (Line-Neutral)"
      m_Pln_bus1_C = "m:Pln:bus1:C", "MW, Active Power (Line-Neutral)"
      m_Pln_bus1_N = "m:Pln:bus1:N", "MW, Active Power (Line-Neutral)"
      m_Qln_bus1_A = "m:Qln:bus1:A", "Mvar, Reactive Power (Line-Neutral)"
      m_Qln_bus1_B = "m:Qln:bus1:B", "Mvar, Reactive Power (Line-Neutral)"
      m_Qln_bus1_C = "m:Qln:bus1:C", "Mvar, Reactive Power (Line-Neutral)"
      m_Qln_bus1_N = "m:Qln:bus1:N", "Mvar, Reactive Power (Line-Neutral)"
      m_P1_bus1 = "m:P1:bus1", "MW, Positive Sequence Active Power"
      m_Q1_bus1 = "m:Q1:bus1", "Mvar, Positive Sequence Reactive Power"
      m_P2_bus1 = "m:P2:bus1", "MW, Negative Sequence Active Power"
      m_Q2_bus1 = "m:Q2:bus1", "Mvar, Negative Sequence Reactive Power"
      m_P0_bus1 = "m:P0:bus1", "MW, Zero Sequence Active Power"
      m_Q0_bus1 = "m:Q0:bus1", "Mvar, Zero Sequence Reactive Power"
      m_i1P_bus1 = "m:i1P:bus1", "p.u., Positive-Sequence Active Current"
      m_i1Q_bus1 = "m:i1Q:bus1", "p.u., Positive-Sequence Reactive Current"
      m_I1P_bus1 = "m:I1P:bus1", "kA, Positive-Sequence Active Current"
      m_I1Q_bus1 = "m:I1Q:bus1", "kA, Positive-Sequence Reactive Current"
      m_i2P_bus1 = "m:i2P:bus1", "p.u., Negative-Sequence Active Current"
      m_i2Q_bus1 = "m:i2Q:bus1", "p.u., Negative-Sequence Reactive Current"
      m_I2P_bus1 = "m:I2P:bus1", "kA, Negative-Sequence Active Current"
      m_I2Q_bus1 = "m:I2Q:bus1", "kA, Negative-Sequence Reactive Current"
      m_ur_bus1_A = "m:ur:bus1:A", "p.u., Phase Voltage, Real Part"
      m_ur_bus1_B = "m:ur:bus1:B", "p.u., Phase Voltage, Real Part"
      m_ur_bus1_C = "m:ur:bus1:C", "p.u., Phase Voltage, Real Part"
      m_ur_bus1_N = "m:ur:bus1:N", "p.u., Phase Voltage, Real Part"
      m_ui_bus1_A = "m:ui:bus1:A", "p.u., Phase Voltage, Imaginary Part"
      m_ui_bus1_B = "m:ui:bus1:B", "p.u., Phase Voltage, Imaginary Part"
      m_ui_bus1_C = "m:ui:bus1:C", "p.u., Phase Voltage, Imaginary Part"
      m_ui_bus1_N = "m:ui:bus1:N", "p.u., Phase Voltage, Imaginary Part"
      m_u_bus1_A = "m:u:bus1:A", "p.u., Phase Voltage, Magnitude"
      m_u_bus1_B = "m:u:bus1:B", "p.u., Phase Voltage, Magnitude"
      m_u_bus1_C = "m:u:bus1:C", "p.u., Phase Voltage, Magnitude"
      m_u_bus1_N = "m:u:bus1:N", "p.u., Phase Voltage, Magnitude"
      m_ir_bus1_A = "m:ir:bus1:A", "p.u., Phase Current, Real Part"
      m_ir_bus1_B = "m:ir:bus1:B", "p.u., Phase Current, Real Part"
      m_ir_bus1_C = "m:ir:bus1:C", "p.u., Phase Current, Real Part"
      m_ir_bus1_N = "m:ir:bus1:N", "p.u., Phase Current, Real Part"
      m_ii_bus1_A = "m:ii:bus1:A", "p.u., Phase Current, Imaginary Part"
      m_ii_bus1_B = "m:ii:bus1:B", "p.u., Phase Current, Imaginary Part"
      m_ii_bus1_C = "m:ii:bus1:C", "p.u., Phase Current, Imaginary Part"
      m_ii_bus1_N = "m:ii:bus1:N", "p.u., Phase Current, Imaginary Part"
      m_i_bus1_A = "m:i:bus1:A", "p.u., Phase Current, Magnitude"
      m_i_bus1_B = "m:i:bus1:B", "p.u., Phase Current, Magnitude"
      m_i_bus1_C = "m:i:bus1:C", "p.u., Phase Current, Magnitude"
      m_i_bus1_N = "m:i:bus1:N", "p.u., Phase Current, Magnitude"
      m_phiui_bus1_A = "m:phiui:bus1:A", "deg, Angle between Voltage and Current"
      m_phiui_bus1_B = "m:phiui:bus1:B", "deg, Angle between Voltage and Current"
      m_phiui_bus1_C = "m:phiui:bus1:C", "deg, Angle between Voltage and Current"
      m_phiui_bus1_N = "m:phiui:bus1:N", "deg, Angle between Voltage and Current"
      m_phiu0i0_bus1 = "m:phiu0i0:bus1", "deg, Angle between Voltage and Current in zero-sequence system"
      m_phiu1i1_bus1 = "m:phiu1i1:bus1", "deg, Angle between Voltage and Current in positive sequence system"
      m_phiu2i2_bus1 = "m:phiu2i2:bus1", "deg, Angle between Voltage and Current in negative sequence system"
      m_Tfct_bus1 = "m:Tfct:bus1", "s, Fault Clearing Time"
      m_TfctPh_bus1_A = "m:TfctPh:bus1:A", "s, Fault Clearing Time"
      m_TfctPh_bus1_B = "m:TfctPh:bus1:B", "s, Fault Clearing Time"
      m_TfctPh_bus1_C = "m:TfctPh:bus1:C", "s, Fault Clearing Time"
      m_TfctPh_bus1_N = "m:TfctPh:bus1:N", "s, Fault Clearing Time"
      m_Brkload_bus1 = "m:Brkload:bus1", "%, Max. Breaker Loading"
      m_BrkloadPh_bus1_A = "m:BrkloadPh:bus1:A", "%, Breaker Loading"
      m_BrkloadPh_bus1_B = "m:BrkloadPh:bus1:B", "%, Breaker Loading"
      m_BrkloadPh_bus1_C = "m:BrkloadPh:bus1:C", "%, Breaker Loading"
      m_BrkloadPh_bus1_N = "m:BrkloadPh:bus1:N", "%, Breaker Loading"
      n_ur_bus1_A = "n:ur:bus1:A", "p.u., Line-Ground Voltage, Real Part"
      n_ur_bus1_B = "n:ur:bus1:B", "p.u., Line-Ground Voltage, Real Part"
      n_ur_bus1_C = "n:ur:bus1:C", "p.u., Line-Ground Voltage, Real Part"
      n_ur_bus1_N = "n:ur:bus1:N", "p.u., Line-Ground Voltage, Real Part"
      n_ui_bus1_A = "n:ui:bus1:A", "p.u., Line-Ground Voltage, Imaginary Part"
      n_ui_bus1_B = "n:ui:bus1:B", "p.u., Line-Ground Voltage, Imaginary Part"
      n_ui_bus1_C = "n:ui:bus1:C", "p.u., Line-Ground Voltage, Imaginary Part"
      n_ui_bus1_N = "n:ui:bus1:N", "p.u., Line-Ground Voltage, Imaginary Part"
      n_u_bus1_A = "n:u:bus1:A", "p.u., Line-Ground Voltage, Magnitude"
      n_u_bus1_B = "n:u:bus1:B", "p.u., Line-Ground Voltage, Magnitude"
      n_u_bus1_C = "n:u:bus1:C", "p.u., Line-Ground Voltage, Magnitude"
      n_u_bus1_N = "n:u:bus1:N", "p.u., Line-Ground Voltage, Magnitude"
      n_upc_bus1_A = "n:upc:bus1:A", "%, Line-Ground Voltage, Magnitude"
      n_upc_bus1_B = "n:upc:bus1:B", "%, Line-Ground Voltage, Magnitude"
      n_upc_bus1_C = "n:upc:bus1:C", "%, Line-Ground Voltage, Magnitude"
      n_upc_bus1_N = "n:upc:bus1:N", "%, Line-Ground Voltage, Magnitude"
      n_U_bus1_A = "n:U:bus1:A", "kV, Line-Ground Voltage, Magnitude"
      n_U_bus1_B = "n:U:bus1:B", "kV, Line-Ground Voltage, Magnitude"
      n_U_bus1_C = "n:U:bus1:C", "kV, Line-Ground Voltage, Magnitude"
      n_U_bus1_N = "n:U:bus1:N", "kV, Line-Ground Voltage, Magnitude"
      n_phiu_bus1_A = "n:phiu:bus1:A", "deg, Line-Ground Voltage, Angle"
      n_phiu_bus1_B = "n:phiu:bus1:B", "deg, Line-Ground Voltage, Angle"
      n_phiu_bus1_C = "n:phiu:bus1:C", "deg, Line-Ground Voltage, Angle"
      n_phiu_bus1_N = "n:phiu:bus1:N", "deg, Line-Ground Voltage, Angle"
      n_phiurel_bus1 = "n:phiurel:bus1", "deg, Voltage, Relative Angle"
      n_ul_bus1_A = "n:ul:bus1:A", "p.u., Line-Line Voltage, Magnitude"
      n_ul_bus1_B = "n:ul:bus1:B", "p.u., Line-Line Voltage, Magnitude"
      n_ul_bus1_C = "n:ul:bus1:C", "p.u., Line-Line Voltage, Magnitude"
      n_ul_bus1_N = "n:ul:bus1:N", "p.u., Line-Line Voltage, Magnitude"
      n_ulpc_bus1_A = "n:ulpc:bus1:A", "%, Line-Line Voltage, Magnitude"
      n_ulpc_bus1_B = "n:ulpc:bus1:B", "%, Line-Line Voltage, Magnitude"
      n_ulpc_bus1_C = "n:ulpc:bus1:C", "%, Line-Line Voltage, Magnitude"
      n_ulpc_bus1_N = "n:ulpc:bus1:N", "%, Line-Line Voltage, Magnitude"
      n_Ul_bus1_A = "n:Ul:bus1:A", "kV, Line-Line Voltage, Magnitude"
      n_Ul_bus1_B = "n:Ul:bus1:B", "kV, Line-Line Voltage, Magnitude"
      n_Ul_bus1_C = "n:Ul:bus1:C", "kV, Line-Line Voltage, Magnitude"
      n_Ul_bus1_N = "n:Ul:bus1:N", "kV, Line-Line Voltage, Magnitude"
      n_phiul_bus1_A = "n:phiul:bus1:A", "deg, Line-Line Voltage, Angle"
      n_phiul_bus1_B = "n:phiul:bus1:B", "deg, Line-Line Voltage, Angle"
      n_phiul_bus1_C = "n:phiul:bus1:C", "deg, Line-Line Voltage, Angle"
      n_phiul_bus1_N = "n:phiul:bus1:N", "deg, Line-Line Voltage, Angle"
      n_u0_bus1 = "n:u0:bus1", "p.u., Zero-Sequence Voltage, Magnitude"
      n_U0_bus1 = "n:U0:bus1", "kV, Zero-Sequence Voltage, Magnitude"
      n_U0x3_bus1 = "n:U0x3:bus1", "kV, 3*U0"
      n_phiu0_bus1 = "n:phiu0:bus1", "deg, Zero-Sequence Voltage, Angle"
      n_u1_bus1 = "n:u1:bus1", "p.u., Positive-Sequence Voltage, Magnitude"
      n_u1pc_bus1 = "n:u1pc:bus1", "%, Positive-Sequence Voltage, Magnitude"
      n_u1r_bus1 = "n:u1r:bus1", "p.u., Positive-Sequence Voltage, Real Part"
      n_u1i_bus1 = "n:u1i:bus1", "p.u., Positive-Sequence Voltage, Imaginary Part"
      n_U1_bus1 = "n:U1:bus1", "kV, Line-Ground Positive-Sequence Voltage, Magnitude"
      n_phiu1_bus1 = "n:phiu1:bus1", "deg, Positive-Sequence Voltage, Angle"
      n_u2_bus1 = "n:u2:bus1", "p.u., Negative-Sequence Voltage, Magnitude"
      n_U2_bus1 = "n:U2:bus1", "kV, Line-Ground Negative-Sequence Voltage, Magnitude"
      n_phiu2_bus1 = "n:phiu2:bus1", "deg, Negative-Sequence Voltage, Angle"
      n_U1l_bus1 = "n:U1l:bus1", "kV, Line-Line Positive-Sequence Voltage, Magnitude"
      n_U2l_bus1 = "n:U2l:bus1", "kV, Line-Line Negative-Sequence Voltage, Magnitude"
      n_uln_bus1_A = "n:uln:bus1:A", "p.u., Line-Neutral Voltage, Magnitude"
      n_uln_bus1_B = "n:uln:bus1:B", "p.u., Line-Neutral Voltage, Magnitude"
      n_uln_bus1_C = "n:uln:bus1:C", "p.u., Line-Neutral Voltage, Magnitude"
      n_uln_bus1_N = "n:uln:bus1:N", "p.u., Line-Neutral Voltage, Magnitude"
      n_Uln_bus1_A = "n:Uln:bus1:A", "kV, Line-Neutral Voltage, Magnitude"
      n_Uln_bus1_B = "n:Uln:bus1:B", "kV, Line-Neutral Voltage, Magnitude"
      n_Uln_bus1_C = "n:Uln:bus1:C", "kV, Line-Neutral Voltage, Magnitude"
      n_Uln_bus1_N = "n:Uln:bus1:N", "kV, Line-Neutral Voltage, Magnitude"
      n_phiuln_bus1_A = "n:phiuln:bus1:A", "deg, Line-Neutral Voltage, Angle"
      n_phiuln_bus1_B = "n:phiuln:bus1:B", "deg, Line-Neutral Voltage, Angle"
      n_phiuln_bus1_C = "n:phiuln:bus1:C", "deg, Line-Neutral Voltage, Angle"
      n_phiuln_bus1_N = "n:phiuln:bus1:N", "deg, Line-Neutral Voltage, Angle"
      n_un_bus1 = "n:un:bus1", "p.u., Neutral-Ground Voltage, Magnitude"
      n_Un_bus1 = "n:Un:bus1", "kV, Neutral-Ground Voltage, Magnitude"
      n_phiun_bus1 = "n:phiun:bus1", "deg, Neutral-Ground Voltage, Angle"
      n_upht_bus1_A = "n:upht:bus1:A", "p.u., Phase Technology dependent Voltage, Magnitude"
      n_upht_bus1_B = "n:upht:bus1:B", "p.u., Phase Technology dependent Voltage, Magnitude"
      n_upht_bus1_C = "n:upht:bus1:C", "p.u., Phase Technology dependent Voltage, Magnitude"
      n_upht_bus1_N = "n:upht:bus1:N", "p.u., Phase Technology dependent Voltage, Magnitude"
      n_Upht_bus1_A = "n:Upht:bus1:A", "kV, Phase Technology dependent Voltage, Magnitude"
      n_Upht_bus1_B = "n:Upht:bus1:B", "kV, Phase Technology dependent Voltage, Magnitude"
      n_Upht_bus1_C = "n:Upht:bus1:C", "kV, Phase Technology dependent Voltage, Magnitude"
      n_Upht_bus1_N = "n:Upht:bus1:N", "kV, Phase Technology dependent Voltage, Magnitude"
      n_um_bus1 = "n:um:bus1", "p.u., Average-Voltage, Magnitude"
      n_Um_bus1 = "n:Um:bus1", "kV, Average-Voltage, Magnitude"
      n_uphtmin_bus1 = "n:uphtmin:bus1", "p.u., Minimum of Phase Technology dependent Voltage, Magnitude"
      n_uphtmax_bus1 = "n:uphtmax:bus1", "p.u., Maximum of Phase Technology dependent Voltage, Magnitude"
      n_Pgen_bus1 = "n:Pgen:bus1", "MW, Generation, Active Power"
      n_Qgen_bus1 = "n:Qgen:bus1", "Mvar, Generation, Reactive Power"
      n_Pmot_bus1 = "n:Pmot:bus1", "MW, Motor Load, Active Power"
      n_Qmot_bus1 = "n:Qmot:bus1", "Mvar, Motor Load, Reactive Power"
      n_Pload_bus1 = "n:Pload:bus1", "MW, General Load, Active Power"
      n_Qload_bus1 = "n:Qload:bus1", "Mvar, General Load, Reactive Power"
      n_Pcomp_bus1 = "n:Pcomp:bus1", "MW, Compensation (Losses)"
      n_Qcomp_bus1 = "n:Qcomp:bus1", "Mvar, Compensation"
      n_Pnet_bus1 = "n:Pnet:bus1", "MW, External Networks, Active Power"
      n_Qnet_bus1 = "n:Qnet:bus1", "Mvar, External Networks, Reactive Power"
      n_Pflow_bus1 = "n:Pflow:bus1", "MW, Power Flow, Active Power"
      n_Qflow_bus1 = "n:Qflow:bus1", "Mvar, Power Flow, Reactive Power"
      n_Pout_bus1 = "n:Pout:bus1", "MW, Outgoing Flow, Active Power"
      n_Qout_bus1 = "n:Qout:bus1", "Mvar, Outgoing Power, Reactive Power"
      n_Sout_bus1 = "n:Sout:bus1", "MVA, Outgoing Power, Apparent Power"
      n_cosphiout_bus1 = "n:cosphiout:bus1", "Outgoing Power, Power Factor"
      n_Pbalance_bus1 = "n:Pbalance:bus1", "MW, Active Power Balance (=0)"
      n_Qbalance_bus1 = "n:Qbalance:bus1", "Mvar, Reactive Power Balance (=0)"
      n_ubfac_bus1 = "n:ubfac:bus1", "%, Unbalance Factor"
      n_LossPdown_bus1 = "n:LossPdown:bus1", "MW, Losses, downstream"
      n_LossQdown_bus1 = "n:LossQdown:bus1", "Mvar, Losses, downstream (Reactive Power)"
      n_LossPdownload_bus1 = "n:LossPdownload:bus1", "MW, Load losses, downstream"
      n_LossQdownload_bus1 = "n:LossQdownload:bus1", "Mvar, Load losses, downstream"
      n_LossPdownnoload_bus1 = "n:LossPdownnoload:bus1", "MW, No load losses, downstream"
      n_LossQdownnoload_bus1 = "n:LossQdownnoload:bus1", "Mvar, No load losses, downstream"
      s_scale = "s:scale", "IN, Scale Factor, continuous, double"
      s_scalephi = "s:scalephi", "rad, IN, Load Scaling: Power Angle, continuous, double"
      s_scale1r = "s:scale1r", "IN, Feeder load scaling, control 1, ph. 1, continuous, double"
      s_scale1s = "s:scale1s", "IN, Feeder load scaling, control 1, ph. 2, continuous, double"
      s_scale1t = "s:scale1t", "IN, Feeder load scaling, control 1, ph. 3, continuous, double"
      s_scale2r = "s:scale2r", "IN, Feeder load scaling, control 2, ph. 1, continuous, double"
      s_scale2s = "s:scale2s", "IN, Feeder load scaling, control 2, ph. 2, continuous, double"
      s_scale2t = "s:scale2t", "IN, Feeder load scaling, control 2, ph. 3, continuous, double"
      c_prini = "c:prini", "MW, Nominal Active Power, in phase A"
      c_qrini = "c:qrini", "Mvar, Nominal Reactive Power in phase A"
      c_psini = "c:psini", "MW, Nominal Active Power, in phase B"
      c_qsini = "c:qsini", "Mvar, Nominal Reactive Power in phase B"
      c_ptini = "c:ptini", "MW, Nominal Active Power, in phase C"
      c_qtini = "c:qtini", "Mvar, Nominal Reactive Power in phase C"
      c_sr_r = "c:sr:r", "MW, Complex Power in phase A, Real Part"
      c_sr_i = "c:sr:i", "Mvar, Complex Power in phase A, Imaginary Part"
      c_sr_phi = "c:sr:phi", "deg, Complex Power in phase A, Angle"
      c_sr_mag = "c:sr:mag", "MVA, Complex Power in phase A, Magnitude"
      c_sr_phirad = "c:sr:phirad", "rad, Complex Power in phase A, Angle"
      c_ss_r = "c:ss:r", "MW, Complex Power in phase B, Real Part"
      c_ss_i = "c:ss:i", "Mvar, Complex Power in phase B, Imaginary Part"
      c_ss_phi = "c:ss:phi", "deg, Complex Power in phase B, Angle"
      c_ss_mag = "c:ss:mag", "MVA, Complex Power in phase B, Magnitude"
      c_ss_phirad = "c:ss:phirad", "rad, Complex Power in phase B, Angle"
      c_st_r = "c:st:r", "MW, Complex Power in phase C, Real Part"
      c_st_i = "c:st:i", "Mvar, Complex Power in phase C, Imaginary Part"
      c_st_phi = "c:st:phi", "deg, Complex Power in phase C, Angle"
      c_st_mag = "c:st:mag", "MVA, Complex Power in phase C, Magnitude"
      c_st_phirad = "c:st:phirad", "rad, Complex Power in phase C, Angle"
      c_frnom = "c:frnom", "Hz, Nominal Frequency"
      c_hpi = "c:hpi", "rad/s, Nominal angular frequency"
      c_u0 = "c:u0", "p.u., Nominal Voltage"
      c_ncust = "c:ncust", "Number of Connected Customers"
      c_scale0 = "c:scale0", "Initial Scale Factor"
      c_zonescale = "c:zonescale", "Zone Scale Factor"
      c_kpu0 = "c:kpu0", "Volt. Dependence I on P"
      c_kqu0 = "c:kqu0", "Volt. Dependence I on Q"
      c_kpu1 = "c:kpu1", "Volt. Dependence II on P"
      c_kqu1 = "c:kqu1", "Volt. Dependence II on Q"
      c_kpu = "c:kpu", "Volt. Dependence III on P"
      c_kqu = "c:kqu", "Volt. Dependence III on Q"
      c_aP = "c:aP", "Percent of Volt. Dep. I on P"
      c_bP = "c:bP", "Percent of Volt. Dep. II on P"
      c_aQ = "c:aQ", "Percent of Volt. Dep. I on Q"
      c_bQ = "c:bQ", "Percent of Volt. Dep. II on Q"
      c_Pmism = "c:Pmism", "MW, Active Power Mismatch"
      c_Ploss = "c:Ploss", "MW, Losses (total)"
      c_Qloss = "c:Qloss", "Mvar, Reactive-Losses (total)"
      c_Plossld = "c:Plossld", "MW, Losses (load)"
      c_Qlossld = "c:Qlossld", "Mvar, Reactive-Losses (load)"
      c_Plossnld = "c:Plossnld", "MW, Losses (no load)"
      c_Qlossnld = "c:Qlossnld", "Mvar, Reactive-Losses (no load)"
      e_iopt_inp = "e:iopt_inp", "Input Mode"
      e_i_sym = "e:i_sym", "Balanced/Unbalanced"
      e_ulini = "e:ulini", "kV, Voltage, U(L-L)"
      e_plini = "e:plini", "kW, Active Power, P"
      e_slini = "e:slini", "kVA, Apparent Power, S"
      e_ilini = "e:ilini", "A, Current, I"
      e_cplinia = "e:cplinia", "kW, P, Load"
      e_coslini = "e:coslini", "Power Factor, cos(phi)"
      e_pf_recap = "e:pf_recap", "Power Factor"
      e_elini = "e:elini", "kWh, Yearly Energy"
      e_pProfile = "e:pProfile", "ChaTime,ChaProfile, Consumption Profile"
      e_scale0 = "e:scale0", "Scaling Factor"
      e_i_scale = "e:i_scale", "Adjusted by Load Scaling"
      e_plinir = "e:plinir", "kW, Active Power, P"
      e_slinir = "e:slinir", "kVA, Apparent Power, S"
      e_ilinir = "e:ilinir", "A, Current, I"
      e_coslinir = "e:coslinir", "Power Factor, cos(phi)"
      e_pf_recapr = "e:pf_recapr", "Power Factor"
      e_plinis = "e:plinis", "kW, Active Power, P"
      e_slinis = "e:slinis", "kVA, Apparent Power, S"
      e_ilinis = "e:ilinis", "A, Current, I"
      e_coslinis = "e:coslinis", "Power Factor, cos(phi)"
      e_pf_recaps = "e:pf_recaps", "Power Factor"
      e_plinit = "e:plinit", "kW, Active Power, P"
      e_slinit = "e:slinit", "kVA, Apparent Power, S"
      e_ilinit = "e:ilinit", "A, Current, I"
      e_coslinit = "e:coslinit", "Power Factor, cos(phi)"
      e_pf_recapt = "e:pf_recapt", "Power Factor"
      e_qdslCtrl = "e:qdslCtrl", "Quasi-Dynamic Model"
      e_cHasPartLod = "e:cHasPartLod", "Partial Loads Present"
      e_cPartLod = "e:cPartLod", "Partial Loads:"
      e_ulini_a = "e:ulini_a", "kV, U(L-L)(act.)"
      e_plini_a = "e:plini_a", "kW, P(act.)"
      e_qlini_a = "e:qlini_a", "kvar, Q(act.)"
      e_slini_a = "e:slini_a", "kVA, S(act.)"
      e_coslini_a = "e:coslini_a", "cos(phi)(act.)"
      e_ilini_a = "e:ilini_a", "A, I(act.)"
      e_scale0_a = "e:scale0_a", "Scaling Factor(act.)"
      e_pnight_a = "e:pnight_a", "kW, P(night)(act.)"
      e_plinir_a = "e:plinir_a", "kW, Ph 1:P(act.)"
      e_slinir_a = "e:slinir_a", "kVA, Ph 1:S(act.)"
      e_coslinir_a = "e:coslinir_a", "Ph 1:cos(phi)(act.)"
      e_ilinir_a = "e:ilinir_a", "A, Ph 1:I(act.)"
      e_plinis_a = "e:plinis_a", "kW, Ph 2:P(act.)"
      e_slinis_a = "e:slinis_a", "kVA, Ph 2:S(act.)"
      e_coslinis_a = "e:coslinis_a", "Ph 2:cos(phi)(act.)"
      e_ilinis_a = "e:ilinis_a", "A, Ph 2:I(act.)"
      e_plinit_a = "e:plinit_a", "kW, Ph 3:P(act.)"
      e_slinit_a = "e:slinit_a", "kVA, Ph 3:S(act.)"
      e_coslinit_a = "e:coslinit_a", "Ph 3:cos(phi)(act.)"
      e_ilinit_a = "e:ilinit_a", "A, Ph 3:I(act.)"
      e_pnight = "e:pnight", "kW, P"
      e_cNrCust = "e:cNrCust", "Number of Customers"
      e_cPrCust = "e:cPrCust", "kW, P/Customer"
      e_UtilFactor = "e:UtilFactor", "Utilisation Factor"
      e_cSmax = "e:cSmax", "kVA, Max. Load per Customer"
      e_cSav = "e:cSav", "kVA, Average Load"
      e_ccosphi = "e:ccosphi", "Power Factor"
      e_cpFeed = "e:cpFeed", "Feeder"
      e_ciDist = "e:ciDist", "Distance from infeed in number of buses"
      e_ciLater = "e:ciLater", "Lateral Index"
      e_ciDistRoot = "e:ciDistRoot", "Distance from first infeed in number of buses"
      e_ciDistAll = "e:ciDistAll", "Distance from infeed in number of buses including switches"
      e_ciDistAllRoot = "e:ciDistAllRoot", "Distance from first infeed in number of buses including switches"
      r_pProfile = "r:pProfile", "ChaTime,ChaProfile, Consumption Profile"
      r_qdslCtrl = "r:qdslCtrl", "Quasi-Dynamic Model"
      r_cpFeed = "r:cpFeed", "Feeder"
      m_Ikss_bus1 = "m:Ikss:bus1", "kA, Initial Short-Circuit Current"
      m_ikss_bus1 = "m:ikss:bus1", "p.u., Initial Short-Circuit Current, Magnitude"
      m_phii_bus1 = "m:phii:bus1", "deg, Phase Current, Angle"
      m_maxIkss_bus1 = "m:maxIkss:bus1", "kA, Max. Initial Short-Circuit Current"
      m_Skss_bus1 = "m:Skss:bus1", "MVA, Initial Short-Circuit Power"
      m_ip_bus1 = "m:ip:bus1", "kA, Peak Short-Circuit Current"
      m_maxip_bus1 = "m:maxip:bus1", "kA, Max. Peak Short-Circuit Current"
      m_maxIb_bus1 = "m:maxIb:bus1", "kA, Max. Breaking Current"
      m_maxIth_bus1 = "m:maxIth:bus1", "kA, Max. Thermal equivalent Short-Circuit Current"
      m_Ib_bus1 = "m:Ib:bus1", "kA, Short-Circuit Breaking Current"
      m_Sb_bus1 = "m:Sb:bus1", "MVA, Short-Circuit Breaking Power"
      m_Ik_bus1 = "m:Ik:bus1", "kA, Steady-State Short-Circuit Current"
      m_Ith_bus1 = "m:Ith:bus1", "kA, Thermal equivalent Short-Circuit Current"
      m_I_bus1 = "m:I:bus1", "kA, Short-Circuit Current"
      m_R_bus1 = "m:R:bus1", "Ohm, Fault Impedance, Real Part"
      m_X_bus1 = "m:X:bus1", "Ohm, Fault Impedance, Imaginary Part"
      m_Z_bus1 = "m:Z:bus1", "Ohm, Fault Impedance, Magnitude"
      m_phiz_bus1 = "m:phiz:bus1", "deg, Fault Impedance, Angle"
      m_zSbase_bus1 = "m:zSbase:bus1", "p.u., Fault Impedance, Magnitude"
      m_XtoR_bus1 = "m:XtoR:bus1", "X/R ratio"
      m_Zs_bus1 = "m:Zs:bus1", "Ohm, Source Impedance, Magnitude"
      m_phizs_bus1 = "m:phizs:bus1", "deg, Source Impedance, Angle"
      m_Rs_bus1 = "m:Rs:bus1", "Ohm, Source Impedance, Real Part"
      m_Xs_bus1 = "m:Xs:bus1", "Ohm, Source Impedance, Imaginary Part"
      m_phiui_bus1 = "m:phiui:bus1", "deg, Angle between Voltage and Current"
      m_cosphiui_bus1 = "m:cosphiui:bus1", "cosphi"
      m_P_bus1 = "m:P:bus1", "MW, Active Power"
      m_Q_bus1 = "m:Q:bus1", "Mvar, Reactive Power"
      m_BrkIpload_bus1 = "m:BrkIpload:bus1", "%, Breaker Loading (peak)"
      m_BrkIbload_bus1 = "m:BrkIbload:bus1", "%, Breaker Loading (breaking)"
      m_BrkmaxIb_bus1 = "m:BrkmaxIb:bus1", "kA, Max. Breaking Current (breaker time)"
      n_Ikss_bus1 = "n:Ikss:bus1", "kA, Initial Short-Circuit Current"
      n_I_bus1 = "n:I:bus1", "kA, Short-Circuit Current"
      n_phii_bus1 = "n:phii:bus1", "deg, Phase Current, Angle"
      n_Skss_bus1 = "n:Skss:bus1", "MVA, Initial Short-Circuit Power"
      n_ip_bus1 = "n:ip:bus1", "kA, Peak Short-Circuit Current"
      n_Ib_bus1 = "n:Ib:bus1", "kA, Short-Circuit Breaking Current"
      n_Sb_bus1 = "n:Sb:bus1", "MVA, Short-Circuit Breaking Power"
      n_Ik_bus1 = "n:Ik:bus1", "kA, Steady-State Short-Circuit Current"
      n_Ith_bus1 = "n:Ith:bus1", "kA, Thermal equivalent Short-Circuit Current"
      n_R_bus1 = "n:R:bus1", "Ohm, Short-Circuit Impedance, Real Part"
      n_X_bus1 = "n:X:bus1", "Ohm, Short-Circuit Impedance, Imaginary Part"
      n_Z_bus1 = "n:Z:bus1", "Ohm, Short-Circuit Impedance, Magnitude"
      n_phiz_bus1 = "n:phiz:bus1", "deg, Short-Circuit Impedance, Angle"
      n_rSbase_bus1 = "n:rSbase:bus1", "p.u., Short-Circuit Impedance, Real Part"
      n_xSbase_bus1 = "n:xSbase:bus1", "p.u., Short-Circuit Impedance, Imaginary Part"
      n_zSbase_bus1 = "n:zSbase:bus1", "p.u., Short-Circuit Impedance, Magnitude"
      n_XtoR_bus1 = "n:XtoR:bus1", "X/R ratio (peak)"
      n_RtoX_bus1 = "n:RtoX:bus1", "R/X ratio (peak)"
      n_ur_bus1 = "n:ur:bus1", "p.u., Voltage, Real Part"
      n_ui_bus1 = "n:ui:bus1", "p.u., Voltage, Imaginary Part"
      n_u_bus1 = "n:u:bus1", "p.u., Voltage, Magnitude"
      n_upc_bus1 = "n:upc:bus1", "%, Voltage, Magnitude"
      n_U_bus1 = "n:U:bus1", "kV, Line-Ground Voltage, Magnitude"
      n_Ul_bus1 = "n:Ul:bus1", "kV, Line-Line Voltage, Magnitude"
      n_phiu_bus1 = "n:phiu:bus1", "deg, Voltage, Angle"
      n_du_bus1 = "n:du:bus1", "%, Voltage Deviation"
      n_urpref_bus1 = "n:urpref:bus1", "p.u., Pre-fault Voltage, Real Part"
      n_uipref_bus1 = "n:uipref:bus1", "p.u., Pre-fault Voltage, Imaginary Part"
      n_upref_bus1 = "n:upref:bus1", "p.u., Pre-fault Voltage, Magnitude"
      n_uprefpc_bus1 = "n:uprefpc:bus1", "%, Pre-fault Voltage, Magnitude"
      n_Upref_bus1 = "n:Upref:bus1", "kV, Line-Ground Pre-fault Voltage, Magnitude"
      n_Ulpref_bus1 = "n:Ulpref:bus1", "kV, Line-Line Pre-fault Voltage, Magnitude"
      n_phiupref_bus1 = "n:phiupref:bus1", "deg, Pre-fault Voltage, Angle"
      n_phiui_bus1 = "n:phiui:bus1", "deg, Angle between Voltage and Current"
      n_cfac_bus1 = "n:cfac:bus1", "Voltage Factor"
      n_idc_bus1 = "n:idc:bus1", "kA, Decaying Component"
      n_Iasy_bus1 = "n:Iasy:bus1", "kA, Asymmetrical Breaking Current"
      n_Ithload_bus1 = "n:Ithload:bus1", "%, Loading, Thermal equivalent Current"
      n_Ipload_bus1 = "n:Ipload:bus1", "%, Loading, Peak Short-Circuit Current"
      n_Ithrtk_bus1 = "n:Ithrtk:bus1", "kA, Rated Short-Time Current (Tk)"
      n_Tn_bus1 = "n:Tn:bus1", "ms, Grid Time Constant"
      n_XtoR_b_bus1 = "n:XtoR_b:bus1", "X/R ratio (dc)"
      n_RtoX_b_bus1 = "n:RtoX_b:bus1", "R/X ratio (dc)"
      n_fctof_b_bus1 = "n:fctof_b:bus1", "Equivalent Frequency fc/f (dc)"
      c_i_pure = "c:i_pure", "Purely inductive/capacitive"
      c_icsrc = "c:icsrc", "Current Source/Impedance"
      c_R = "c:R", "Resistance"
      c_L = "c:L", "Inductance"
      c_C = "c:C", "Capacitance"
      c_y1_r = "c:y1:r", "p.u., Admittance, Real Part"
      c_y1_i = "c:y1:i", "p.u., Admittance, Imaginary Part"
      c_y1_phi = "c:y1:phi", "deg, Admittance, Angle"
      c_y1_mag = "c:y1:mag", "p.u., Admittance, Magnitude"
      c_y1_phirad = "c:y1:phirad", "rad, Admittance, Angle"
      c_i1_r = "c:i1:r", "p.u., Harmonic current, Real Part"
      c_i1_i = "c:i1:i", "p.u., Harmonic current, Imaginary Part"
      c_i1_phi = "c:i1:phi", "deg, Harmonic current, Angle"
      c_i1_mag = "c:i1:mag", "p.u., Harmonic current, Magnitude"
      c_i1_phirad = "c:i1:phirad", "rad, Harmonic current, Angle"
      m_Ikss_bus1_A = "m:Ikss:bus1:A", "kA, Initial Short-Circuit Current"
      m_Ikss_bus1_B = "m:Ikss:bus1:B", "kA, Initial Short-Circuit Current"
      m_Ikss_bus1_C = "m:Ikss:bus1:C", "kA, Initial Short-Circuit Current"
      m_Ikss_bus1_N = "m:Ikss:bus1:N", "kA, Initial Short-Circuit Current"
      m_ikss_bus1_A = "m:ikss:bus1:A", "p.u., Initial Short-Circuit Current, Magnitude"
      m_ikss_bus1_B = "m:ikss:bus1:B", "p.u., Initial Short-Circuit Current, Magnitude"
      m_ikss_bus1_C = "m:ikss:bus1:C", "p.u., Initial Short-Circuit Current, Magnitude"
      m_ikss_bus1_N = "m:ikss:bus1:N", "p.u., Initial Short-Circuit Current, Magnitude"
      m_Skss_bus1_A = "m:Skss:bus1:A", "MVA, Initial Short-Circuit Power"
      m_Skss_bus1_B = "m:Skss:bus1:B", "MVA, Initial Short-Circuit Power"
      m_Skss_bus1_C = "m:Skss:bus1:C", "MVA, Initial Short-Circuit Power"
      m_Skss_bus1_N = "m:Skss:bus1:N", "MVA, Initial Short-Circuit Power"
      m_Skssx3_bus1_A = "m:Skssx3:bus1:A", "MVA, 3*Initial Short-Circuit Power"
      m_Skssx3_bus1_B = "m:Skssx3:bus1:B", "MVA, 3*Initial Short-Circuit Power"
      m_Skssx3_bus1_C = "m:Skssx3:bus1:C", "MVA, 3*Initial Short-Circuit Power"
      m_Skssx3_bus1_N = "m:Skssx3:bus1:N", "MVA, 3*Initial Short-Circuit Power"
      m_ip_bus1_A = "m:ip:bus1:A", "kA, Peak Short-Circuit Current"
      m_ip_bus1_B = "m:ip:bus1:B", "kA, Peak Short-Circuit Current"
      m_ip_bus1_C = "m:ip:bus1:C", "kA, Peak Short-Circuit Current"
      m_ip_bus1_N = "m:ip:bus1:N", "kA, Peak Short-Circuit Current"
      m_Z_bus1_A = "m:Z:bus1:A", "Ohm, Fault Impedance, Magnitude"
      m_Z_bus1_B = "m:Z:bus1:B", "Ohm, Fault Impedance, Magnitude"
      m_Z_bus1_C = "m:Z:bus1:C", "Ohm, Fault Impedance, Magnitude"
      m_Z_bus1_N = "m:Z:bus1:N", "Ohm, Fault Impedance, Magnitude"
      m_phiz_bus1_A = "m:phiz:bus1:A", "deg, Fault Impedance, Angle"
      m_phiz_bus1_B = "m:phiz:bus1:B", "deg, Fault Impedance, Angle"
      m_phiz_bus1_C = "m:phiz:bus1:C", "deg, Fault Impedance, Angle"
      m_phiz_bus1_N = "m:phiz:bus1:N", "deg, Fault Impedance, Angle"
      m_Zl_bus1_A = "m:Zl:bus1:A", "Ohm, Line-Line Fault Impedance, Magnitude"
      m_Zl_bus1_B = "m:Zl:bus1:B", "Ohm, Line-Line Fault Impedance, Magnitude"
      m_Zl_bus1_C = "m:Zl:bus1:C", "Ohm, Line-Line Fault Impedance, Magnitude"
      m_Zl_bus1_N = "m:Zl:bus1:N", "Ohm, Line-Line Fault Impedance, Magnitude"
      m_phizl_bus1_A = "m:phizl:bus1:A", "deg, Line-Line Fault Impedance, Angle"
      m_phizl_bus1_B = "m:phizl:bus1:B", "deg, Line-Line Fault Impedance, Angle"
      m_phizl_bus1_C = "m:phizl:bus1:C", "deg, Line-Line Fault Impedance, Angle"
      m_phizl_bus1_N = "m:phizl:bus1:N", "deg, Line-Line Fault Impedance, Angle"
      m_Z0_bus1 = "m:Z0:bus1", "Ohm, Zero-Sequence Fault Impedance, Magnitude"
      m_phiz0_bus1 = "m:phiz0:bus1", "deg, Zero-Sequence Fault Impedance, Angle"
      m_z0Sbase_bus1 = "m:z0Sbase:bus1", "p.u., Zero-Sequence Fault Impedance, Magnitude"
      m_cosphiu1i1_bus1 = "m:cosphiu1i1:bus1", "Positive-Sequence cosphi"
      m_I0x3watt_bus1 = "m:I0x3watt:bus1", "kA, Earth-Fault Residual Current"
      m_I0x3var_bus1 = "m:I0x3var:bus1", "kA, Capacitive Earth-Fault Current"
      m_Q0cap_bus1 = "m:Q0cap:bus1", "Mvar, Q0 capacitive"
      m_Zfrel_bus1 = "m:Zfrel:bus1", "Ohm, Relay Impedance, Magnitude"
      m_phizfrel_bus1 = "m:phizfrel:bus1", "deg, Relay Impedance, Angle"
      m_Rfrel_bus1 = "m:Rfrel:bus1", "Ohm, Relay Impedance, Real Part"
      m_Xfrel_bus1 = "m:Xfrel:bus1", "Ohm, Relay Impedance, Imaginary Part"
      m_Zf_bus1 = "m:Zf:bus1", "Ohm, Meas. Fault Impedance, Magnitude"
      m_phizf_bus1 = "m:phizf:bus1", "deg, Meas. Fault Impedance, Angle"
      n_Ikss_bus1_A = "n:Ikss:bus1:A", "kA, Initial Short-Circuit Current"
      n_Ikss_bus1_B = "n:Ikss:bus1:B", "kA, Initial Short-Circuit Current"
      n_Ikss_bus1_C = "n:Ikss:bus1:C", "kA, Initial Short-Circuit Current"
      n_Ikss_bus1_N = "n:Ikss:bus1:N", "kA, Initial Short-Circuit Current"
      n_I_bus1_A = "n:I:bus1:A", "kA, Short-Circuit Current"
      n_I_bus1_B = "n:I:bus1:B", "kA, Short-Circuit Current"
      n_I_bus1_C = "n:I:bus1:C", "kA, Short-Circuit Current"
      n_I_bus1_N = "n:I:bus1:N", "kA, Short-Circuit Current"
      n_phii_bus1_A = "n:phii:bus1:A", "deg, Phase Current, Angle"
      n_phii_bus1_B = "n:phii:bus1:B", "deg, Phase Current, Angle"
      n_phii_bus1_C = "n:phii:bus1:C", "deg, Phase Current, Angle"
      n_phii_bus1_N = "n:phii:bus1:N", "deg, Phase Current, Angle"
      n_Skss_bus1_A = "n:Skss:bus1:A", "MVA, Initial Short-Circuit Power"
      n_Skss_bus1_B = "n:Skss:bus1:B", "MVA, Initial Short-Circuit Power"
      n_Skss_bus1_C = "n:Skss:bus1:C", "MVA, Initial Short-Circuit Power"
      n_Skss_bus1_N = "n:Skss:bus1:N", "MVA, Initial Short-Circuit Power"
      n_Skssx3_bus1_A = "n:Skssx3:bus1:A", "MVA, 3*Initial Short-Circuit Power"
      n_Skssx3_bus1_B = "n:Skssx3:bus1:B", "MVA, 3*Initial Short-Circuit Power"
      n_Skssx3_bus1_C = "n:Skssx3:bus1:C", "MVA, 3*Initial Short-Circuit Power"
      n_Skssx3_bus1_N = "n:Skssx3:bus1:N", "MVA, 3*Initial Short-Circuit Power"
      n_ip_bus1_A = "n:ip:bus1:A", "kA, Peak Short-Circuit Current"
      n_ip_bus1_B = "n:ip:bus1:B", "kA, Peak Short-Circuit Current"
      n_ip_bus1_C = "n:ip:bus1:C", "kA, Peak Short-Circuit Current"
      n_ip_bus1_N = "n:ip:bus1:N", "kA, Peak Short-Circuit Current"
      n_Ib_bus1_A = "n:Ib:bus1:A", "kA, Short-Circuit Breaking Current"
      n_Ib_bus1_B = "n:Ib:bus1:B", "kA, Short-Circuit Breaking Current"
      n_Ib_bus1_C = "n:Ib:bus1:C", "kA, Short-Circuit Breaking Current"
      n_Ib_bus1_N = "n:Ib:bus1:N", "kA, Short-Circuit Breaking Current"
      n_Sb_bus1_A = "n:Sb:bus1:A", "MVA, Short-Circuit Breaking Power"
      n_Sb_bus1_B = "n:Sb:bus1:B", "MVA, Short-Circuit Breaking Power"
      n_Sb_bus1_C = "n:Sb:bus1:C", "MVA, Short-Circuit Breaking Power"
      n_Sb_bus1_N = "n:Sb:bus1:N", "MVA, Short-Circuit Breaking Power"
      n_Ith_bus1_A = "n:Ith:bus1:A", "kA, Thermal equivalent Short-Circuit Current"
      n_Ith_bus1_B = "n:Ith:bus1:B", "kA, Thermal equivalent Short-Circuit Current"
      n_Ith_bus1_C = "n:Ith:bus1:C", "kA, Thermal equivalent Short-Circuit Current"
      n_Ith_bus1_N = "n:Ith:bus1:N", "kA, Thermal equivalent Short-Circuit Current"
      n_Ithmax_bus1 = "n:Ithmax:bus1", "kA, Max. Thermal equivalent Short-Circuit Current"
      n_Ik_bus1_A = "n:Ik:bus1:A", "kA, Steady-State Short-Circuit Current"
      n_Ik_bus1_B = "n:Ik:bus1:B", "kA, Steady-State Short-Circuit Current"
      n_Ik_bus1_C = "n:Ik:bus1:C", "kA, Steady-State Short-Circuit Current"
      n_Ik_bus1_N = "n:Ik:bus1:N", "kA, Steady-State Short-Circuit Current"
      n_R0_bus1 = "n:R0:bus1", "Ohm, Zero-Sequence Impedance, Real Part"
      n_X0_bus1 = "n:X0:bus1", "Ohm, Zero-Sequence Impedance, Imaginary Part"
      n_Z0_bus1 = "n:Z0:bus1", "Ohm, Zero-Sequence Impedance, Magnitude"
      n_phiz0_bus1 = "n:phiz0:bus1", "deg, Zero-Sequence Impedance, Angle"
      n_r0Sbase_bus1 = "n:r0Sbase:bus1", "p.u., Zero-Sequence Impedance, Real Part"
      n_x0Sbase_bus1 = "n:x0Sbase:bus1", "p.u., Zero-Sequence Impedance, Imaginary Part"
      n_z0Sbase_bus1 = "n:z0Sbase:bus1", "p.u., Zero-Sequence Impedance, Magnitude"
      n_R1_bus1 = "n:R1:bus1", "Ohm, Positive-Sequence Impedance, Real Part"
      n_X1_bus1 = "n:X1:bus1", "Ohm, Positive-Sequence Impedance, Imaginary Part"
      n_Z1_bus1 = "n:Z1:bus1", "Ohm, Positive-Sequence Impedance, Magnitude"
      n_phiz1_bus1 = "n:phiz1:bus1", "deg, Positive-Sequence Impedance, Angle"
      n_r1Sbase_bus1 = "n:r1Sbase:bus1", "p.u., Positive-Sequence Impedance, Real Part"
      n_x1Sbase_bus1 = "n:x1Sbase:bus1", "p.u., Positive-Sequence Impedance, Imaginary Part"
      n_z1Sbase_bus1 = "n:z1Sbase:bus1", "p.u., Positive-Sequence Impedance, Magnitude"
      n_R2_bus1 = "n:R2:bus1", "Ohm, Negative-Sequence Impedance, Real Part"
      n_X2_bus1 = "n:X2:bus1", "Ohm, Negative-Sequence Impedance, Imaginary Part"
      n_Z2_bus1 = "n:Z2:bus1", "Ohm, Negative-Sequence Impedance, Magnitude"
      n_phiz2_bus1 = "n:phiz2:bus1", "deg, Negative-Sequence Impedance, Angle"
      n_r2Sbase_bus1 = "n:r2Sbase:bus1", "p.u., Negative-Sequence Impedance, Real Part"
      n_x2Sbase_bus1 = "n:x2Sbase:bus1", "p.u., Negative-Sequence Impedance, Imaginary Part"
      n_z2Sbase_bus1 = "n:z2Sbase:bus1", "p.u., Negative-Sequence Impedance, Magnitude"
      n_R_bus1_A = "n:R:bus1:A", "Ohm, Network Resistance, Line-Ground"
      n_R_bus1_B = "n:R:bus1:B", "Ohm, Network Resistance, Line-Ground"
      n_R_bus1_C = "n:R:bus1:C", "Ohm, Network Resistance, Line-Ground"
      n_R_bus1_N = "n:R:bus1:N", "Ohm, Network Resistance, Line-Ground"
      n_X_bus1_A = "n:X:bus1:A", "Ohm, Network Reactance, Line-Ground"
      n_X_bus1_B = "n:X:bus1:B", "Ohm, Network Reactance, Line-Ground"
      n_X_bus1_C = "n:X:bus1:C", "Ohm, Network Reactance, Line-Ground"
      n_X_bus1_N = "n:X:bus1:N", "Ohm, Network Reactance, Line-Ground"
      n_Z_bus1_A = "n:Z:bus1:A", "Ohm, Network Impedance, Line-Ground"
      n_Z_bus1_B = "n:Z:bus1:B", "Ohm, Network Impedance, Line-Ground"
      n_Z_bus1_C = "n:Z:bus1:C", "Ohm, Network Impedance, Line-Ground"
      n_Z_bus1_N = "n:Z:bus1:N", "Ohm, Network Impedance, Line-Ground"
      n_phiz_bus1_A = "n:phiz:bus1:A", "deg, Angle of Network Impedance, Line-Ground"
      n_phiz_bus1_B = "n:phiz:bus1:B", "deg, Angle of Network Impedance, Line-Ground"
      n_phiz_bus1_C = "n:phiz:bus1:C", "deg, Angle of Network Impedance, Line-Ground"
      n_phiz_bus1_N = "n:phiz:bus1:N", "deg, Angle of Network Impedance, Line-Ground"
      n_Rl_bus1_A = "n:Rl:bus1:A", "Ohm, Network Resistance, Line-Line"
      n_Rl_bus1_B = "n:Rl:bus1:B", "Ohm, Network Resistance, Line-Line"
      n_Rl_bus1_C = "n:Rl:bus1:C", "Ohm, Network Resistance, Line-Line"
      n_Rl_bus1_N = "n:Rl:bus1:N", "Ohm, Network Resistance, Line-Line"
      n_Xl_bus1_A = "n:Xl:bus1:A", "Ohm, Network Reactance, Line-Line"
      n_Xl_bus1_B = "n:Xl:bus1:B", "Ohm, Network Reactance, Line-Line"
      n_Xl_bus1_C = "n:Xl:bus1:C", "Ohm, Network Reactance, Line-Line"
      n_Xl_bus1_N = "n:Xl:bus1:N", "Ohm, Network Reactance, Line-Line"
      n_Zl_bus1_A = "n:Zl:bus1:A", "Ohm, Network Impedance, Line-Line"
      n_Zl_bus1_B = "n:Zl:bus1:B", "Ohm, Network Impedance, Line-Line"
      n_Zl_bus1_C = "n:Zl:bus1:C", "Ohm, Network Impedance, Line-Line"
      n_Zl_bus1_N = "n:Zl:bus1:N", "Ohm, Network Impedance, Line-Line"
      n_phizl_bus1_A = "n:phizl:bus1:A", "deg, Angle of Network Impedance, Line-Line"
      n_phizl_bus1_B = "n:phizl:bus1:B", "deg, Angle of Network Impedance, Line-Line"
      n_phizl_bus1_C = "n:phizl:bus1:C", "deg, Angle of Network Impedance, Line-Line"
      n_phizl_bus1_N = "n:phizl:bus1:N", "deg, Angle of Network Impedance, Line-Line"
      n_Rln_bus1_A = "n:Rln:bus1:A", "Ohm, Network Resistance, Line-Neutral"
      n_Rln_bus1_B = "n:Rln:bus1:B", "Ohm, Network Resistance, Line-Neutral"
      n_Rln_bus1_C = "n:Rln:bus1:C", "Ohm, Network Resistance, Line-Neutral"
      n_Rln_bus1_N = "n:Rln:bus1:N", "Ohm, Network Resistance, Line-Neutral"
      n_Xln_bus1_A = "n:Xln:bus1:A", "Ohm, Network Reactance, Line-Neutral"
      n_Xln_bus1_B = "n:Xln:bus1:B", "Ohm, Network Reactance, Line-Neutral"
      n_Xln_bus1_C = "n:Xln:bus1:C", "Ohm, Network Reactance, Line-Neutral"
      n_Xln_bus1_N = "n:Xln:bus1:N", "Ohm, Network Reactance, Line-Neutral"
      n_Zln_bus1_A = "n:Zln:bus1:A", "Ohm, Network Impedance, Line-Neutral"
      n_Zln_bus1_B = "n:Zln:bus1:B", "Ohm, Network Impedance, Line-Neutral"
      n_Zln_bus1_C = "n:Zln:bus1:C", "Ohm, Network Impedance, Line-Neutral"
      n_Zln_bus1_N = "n:Zln:bus1:N", "Ohm, Network Impedance, Line-Neutral"
      n_phizln_bus1_A = "n:phizln:bus1:A", "deg, Angle of Network Impedance, Line-Neutral"
      n_phizln_bus1_B = "n:phizln:bus1:B", "deg, Angle of Network Impedance, Line-Neutral"
      n_phizln_bus1_C = "n:phizln:bus1:C", "deg, Angle of Network Impedance, Line-Neutral"
      n_phizln_bus1_N = "n:phizln:bus1:N", "deg, Angle of Network Impedance, Line-Neutral"
      n_Rn_bus1 = "n:Rn:bus1", "Ohm, Network Resistance, Neutral"
      n_Xn_bus1 = "n:Xn:bus1", "Ohm, Network Reactance, Neutral"
      n_Zn_bus1 = "n:Zn:bus1", "Ohm, Network Impedance, Neutral"
      n_phizn_bus1 = "n:phizn:bus1", "deg, Angle of Network Impedance, Neutral"
      n_eff_bus1_A = "n:eff:bus1:A", "Earth Fault Factor"
      n_eff_bus1_B = "n:eff:bus1:B", "Earth Fault Factor"
      n_eff_bus1_C = "n:eff:bus1:C", "Earth Fault Factor"
      n_eff_bus1_N = "n:eff:bus1:N", "Earth Fault Factor"
      n_X0toX1_bus1 = "n:X0toX1:bus1", "X0/X1"
      n_in_bus1 = "n:in:bus1", "p.u., Neutral Current, Magnitude"
      n_In_bus1 = "n:In:bus1", "kA, Neutral Current, Magnitude"
      n_phiin_bus1 = "n:phiin:bus1", "deg, Neutral Current, Angle"
      n_i0r_bus1 = "n:i0r:bus1", "p.u., Zero-Sequence Current, Real Part"
      n_i0i_bus1 = "n:i0i:bus1", "p.u., Zero-Sequence Current, Imaginary Part"
      n_i0_bus1 = "n:i0:bus1", "p.u., Zero-Sequence Current, Magnitude"
      n_I0_bus1 = "n:I0:bus1", "kA, Zero-Sequence Current, Magnitude"
      n_I0x3_bus1 = "n:I0x3:bus1", "kA, 3*I0"
      n_phii0_bus1 = "n:phii0:bus1", "deg, Zero-Sequence Current, Angle"
      n_i1r_bus1 = "n:i1r:bus1", "p.u., Positive-Sequence Current, Real Part"
      n_i1i_bus1 = "n:i1i:bus1", "p.u., Positive-Sequence Current, Imaginary Part"
      n_i1_bus1 = "n:i1:bus1", "p.u., Positive-Sequence Current, Magnitude"
      n_I1_bus1 = "n:I1:bus1", "kA, Positive-Sequence Current, Magnitude"
      n_phii1_bus1 = "n:phii1:bus1", "deg, Positive-Sequence Current, Angle"
      n_i2r_bus1 = "n:i2r:bus1", "p.u., Negative-Sequence Current, Real Part"
      n_i2i_bus1 = "n:i2i:bus1", "p.u., Negative-Sequence Current, Imaginary Part"
      n_i2_bus1 = "n:i2:bus1", "p.u., Negative-Sequence Current, Magnitude"
      n_I2_bus1 = "n:I2:bus1", "kA, Negative-Sequence Current, Magnitude"
      n_phii2_bus1 = "n:phii2:bus1", "deg, Negative-Sequence Current, Angle"
      n_du_bus1_A = "n:du:bus1:A", "%, Voltage Deviation"
      n_du_bus1_B = "n:du:bus1:B", "%, Voltage Deviation"
      n_du_bus1_C = "n:du:bus1:C", "%, Voltage Deviation"
      n_du_bus1_N = "n:du:bus1:N", "%, Voltage Deviation"
      n_urpref_bus1_A = "n:urpref:bus1:A", "p.u., Pre-fault Voltage, Real Part"
      n_urpref_bus1_B = "n:urpref:bus1:B", "p.u., Pre-fault Voltage, Real Part"
      n_urpref_bus1_C = "n:urpref:bus1:C", "p.u., Pre-fault Voltage, Real Part"
      n_urpref_bus1_N = "n:urpref:bus1:N", "p.u., Pre-fault Voltage, Real Part"
      n_uipref_bus1_A = "n:uipref:bus1:A", "p.u., Pre-fault Voltage, Imaginary Part"
      n_uipref_bus1_B = "n:uipref:bus1:B", "p.u., Pre-fault Voltage, Imaginary Part"
      n_uipref_bus1_C = "n:uipref:bus1:C", "p.u., Pre-fault Voltage, Imaginary Part"
      n_uipref_bus1_N = "n:uipref:bus1:N", "p.u., Pre-fault Voltage, Imaginary Part"
      n_upref_bus1_A = "n:upref:bus1:A", "p.u., Pre-fault Voltage, Magnitude"
      n_upref_bus1_B = "n:upref:bus1:B", "p.u., Pre-fault Voltage, Magnitude"
      n_upref_bus1_C = "n:upref:bus1:C", "p.u., Pre-fault Voltage, Magnitude"
      n_upref_bus1_N = "n:upref:bus1:N", "p.u., Pre-fault Voltage, Magnitude"
      n_uprefpc_bus1_A = "n:uprefpc:bus1:A", "%, Pre-fault Voltage, Magnitude"
      n_uprefpc_bus1_B = "n:uprefpc:bus1:B", "%, Pre-fault Voltage, Magnitude"
      n_uprefpc_bus1_C = "n:uprefpc:bus1:C", "%, Pre-fault Voltage, Magnitude"
      n_uprefpc_bus1_N = "n:uprefpc:bus1:N", "%, Pre-fault Voltage, Magnitude"
      n_Upref_bus1_A = "n:Upref:bus1:A", "kV, Line-Ground Pre-fault Voltage, Magnitude"
      n_Upref_bus1_B = "n:Upref:bus1:B", "kV, Line-Ground Pre-fault Voltage, Magnitude"
      n_Upref_bus1_C = "n:Upref:bus1:C", "kV, Line-Ground Pre-fault Voltage, Magnitude"
      n_Upref_bus1_N = "n:Upref:bus1:N", "kV, Line-Ground Pre-fault Voltage, Magnitude"
      n_Ulpref_bus1_A = "n:Ulpref:bus1:A", "kV, Line-Line Pre-fault Voltage, Magnitude"
      n_Ulpref_bus1_B = "n:Ulpref:bus1:B", "kV, Line-Line Pre-fault Voltage, Magnitude"
      n_Ulpref_bus1_C = "n:Ulpref:bus1:C", "kV, Line-Line Pre-fault Voltage, Magnitude"
      n_Ulpref_bus1_N = "n:Ulpref:bus1:N", "kV, Line-Line Pre-fault Voltage, Magnitude"
      n_phiupref_bus1_A = "n:phiupref:bus1:A", "deg, Pre-fault Voltage, Angle"
      n_phiupref_bus1_B = "n:phiupref:bus1:B", "deg, Pre-fault Voltage, Angle"
      n_phiupref_bus1_C = "n:phiupref:bus1:C", "deg, Pre-fault Voltage, Angle"
      n_phiupref_bus1_N = "n:phiupref:bus1:N", "deg, Pre-fault Voltage, Angle"
      n_phiui_bus1_A = "n:phiui:bus1:A", "deg, Angle between Voltage and Current"
      n_phiui_bus1_B = "n:phiui:bus1:B", "deg, Angle between Voltage and Current"
      n_phiui_bus1_C = "n:phiui:bus1:C", "deg, Angle between Voltage and Current"
      n_phiui_bus1_N = "n:phiui:bus1:N", "deg, Angle between Voltage and Current"
      n_phiu1i1_bus1 = "n:phiu1i1:bus1", "deg, Angle between Voltage and Current in positive sequence system"
      n_phiu2i2_bus1 = "n:phiu2i2:bus1", "deg, Angle between Voltage and Current in negative sequence system"
      n_phiu0i0_bus1 = "n:phiu0i0:bus1", "deg, Angle between Voltage and Current in zero-sequence system"
      c_y_a_r = "c:y_a:r", "Admittance, Phase a, Real Part"
      c_y_a_i = "c:y_a:i", "Admittance, Phase a, Imaginary Part"
      c_y_a_phi = "c:y_a:phi", "Admittance, Phase a, Angle"
      c_y_a_mag = "c:y_a:mag", "Admittance, Phase a, Magnitude"
      c_y_a_phirad = "c:y_a:phirad", "Admittance, Phase a, Angle"
      c_y_b_r = "c:y_b:r", "Admittance, Phase b, Real Part"
      c_y_b_i = "c:y_b:i", "Admittance, Phase b, Imaginary Part"
      c_y_b_phi = "c:y_b:phi", "Admittance, Phase b, Angle"
      c_y_b_mag = "c:y_b:mag", "Admittance, Phase b, Magnitude"
      c_y_b_phirad = "c:y_b:phirad", "Admittance, Phase b, Angle"
      c_y_c_r = "c:y_c:r", "Admittance, Phase c, Real Part"
      c_y_c_i = "c:y_c:i", "Admittance, Phase c, Imaginary Part"
      c_y_c_phi = "c:y_c:phi", "Admittance, Phase c, Angle"
      c_y_c_mag = "c:y_c:mag", "Admittance, Phase c, Magnitude"
      c_y_c_phirad = "c:y_c:phirad", "Admittance, Phase c, Angle"
      c_cn = "c:cn", "Connection Mode"
      c_Ra = "c:Ra", "Resistance, Phase a"
      c_La = "c:La", "Inductance, Phase a"
      c_Ca = "c:Ca", "Capacitance, Phase a"
      c_Rb = "c:Rb", "Resistance, Phase b"
      c_Lb = "c:Lb", "Inductance, Phase b"
      c_Cb = "c:Cb", "Capacitance, Phase b"
      c_Rc = "c:Rc", "Resistance, Phase c"
      c_Lc = "c:Lc", "Inductance, Phase c"
      c_Cc = "c:Cc", "Capacitance, Phase c"
      c_i_abc_r = "c:i_abc:r", "p.u., Harmonic currents, Real Part"
      c_i_abc_i = "c:i_abc:i", "p.u., Harmonic currents, Imaginary Part"
      c_i_abc_phi = "c:i_abc:phi", "deg, Harmonic currents, Angle"
      c_i_abc_mag = "c:i_abc:mag", "p.u., Harmonic currents, Magnitude"
      c_i_abc_phirad = "c:i_abc:phirad", "rad, Harmonic currents, Angle"

    class ElmLodmv(Enum):
      _init_ = 'value __doc__'

      m_u0_bus1 = "m:u0:bus1", "p.u., Zero-Sequence-Voltage, Magnitude"
      m_phiu0_bus1 = "m:phiu0:bus1", "deg, Zero-Sequence-Voltage, Angle"
      m_u1r_bus1 = "m:u1r:bus1", "p.u., Positive-Sequence-Voltage, Real Part"
      m_u1i_bus1 = "m:u1i:bus1", "p.u., Positive-Sequence-Voltage, Imaginary Part"
      m_u1_bus1 = "m:u1:bus1", "p.u., Positive-Sequence-Voltage, Magnitude"
      m_U1_bus1 = "m:U1:bus1", "kV, Line-Ground Positive-Sequence-Voltage, Magnitude"
      m_U1l_bus1 = "m:U1l:bus1", "kV, Line-Line Positive-Sequence-Voltage, Magnitude"
      m_phiu1_bus1 = "m:phiu1:bus1", "deg, Positive-Sequence-Voltage, Angle"
      m_u2_bus1 = "m:u2:bus1", "p.u., Negative-Sequence-Voltage, Magnitude"
      m_phiu2_bus1 = "m:phiu2:bus1", "deg, Negative-Sequence-Voltage, Angle"
      m_inet_bus1_A = "m:inet:bus1:A", "p.u., Phase Current, Magnitude, referred to network"
      m_inet_bus1_B = "m:inet:bus1:B", "p.u., Phase Current, Magnitude, referred to network"
      m_inet_bus1_C = "m:inet:bus1:C", "p.u., Phase Current, Magnitude, referred to network"
      m_inet_bus1_N = "m:inet:bus1:N", "p.u., Phase Current, Magnitude, referred to network"
      m_I_bus1_A = "m:I:bus1:A", "kA, Phase Current, Magnitude"
      m_I_bus1_B = "m:I:bus1:B", "kA, Phase Current, Magnitude"
      m_I_bus1_C = "m:I:bus1:C", "kA, Phase Current, Magnitude"
      m_I_bus1_N = "m:I:bus1:N", "kA, Phase Current, Magnitude"
      m_phii_bus1_A = "m:phii:bus1:A", "deg, Phase Current, Angle"
      m_phii_bus1_B = "m:phii:bus1:B", "deg, Phase Current, Angle"
      m_phii_bus1_C = "m:phii:bus1:C", "deg, Phase Current, Angle"
      m_phii_bus1_N = "m:phii:bus1:N", "deg, Phase Current, Angle"
      m_i0r_bus1 = "m:i0r:bus1", "p.u., Zero-Sequence Current, Real Part"
      m_i0i_bus1 = "m:i0i:bus1", "p.u., Zero-Sequence Current, Imaginary Part"
      m_i0_bus1 = "m:i0:bus1", "p.u., Zero-Sequence Current, Magnitude"
      m_I0_bus1 = "m:I0:bus1", "kA, Zero-Sequence Current, Magnitude"
      m_I0x3_bus1 = "m:I0x3:bus1", "kA, 3*I0"
      m_phii0_bus1 = "m:phii0:bus1", "deg, Zero-Sequence Current, Angle"
      m_i1r_bus1 = "m:i1r:bus1", "p.u., Positive-Sequence Current, Real Part"
      m_i1i_bus1 = "m:i1i:bus1", "p.u., Positive-Sequence Current, Imaginary Part"
      m_i1_bus1 = "m:i1:bus1", "p.u., Positive-Sequence Current, Magnitude"
      m_I1_bus1 = "m:I1:bus1", "kA, Positive-Sequence Current, Magnitude"
      m_phii1_bus1 = "m:phii1:bus1", "deg, Positive-Sequence Current, Angle"
      m_i2r_bus1 = "m:i2r:bus1", "p.u., Negative-Sequence Current, Real Part"
      m_i2i_bus1 = "m:i2i:bus1", "p.u., Negative-Sequence Current, Imaginary Part"
      m_i2_bus1 = "m:i2:bus1", "p.u., Negative-Sequence Current, Magnitude"
      m_I2_bus1 = "m:I2:bus1", "kA, Negative-Sequence Current, Magnitude"
      m_phii2_bus1 = "m:phii2:bus1", "deg, Negative-Sequence Current, Angle"
      m_ubfacI_bus1 = "m:ubfacI:bus1", "%, Current Unbalance Factor"
      m_ubfacS_bus1 = "m:ubfacS:bus1", "%, Power Unbalance Factor"
      m_P_bus1_A = "m:P:bus1:A", "MW, Active Power"
      m_P_bus1_B = "m:P:bus1:B", "MW, Active Power"
      m_P_bus1_C = "m:P:bus1:C", "MW, Active Power"
      m_P_bus1_N = "m:P:bus1:N", "MW, Active Power"
      m_Q_bus1_A = "m:Q:bus1:A", "Mvar, Reactive Power"
      m_Q_bus1_B = "m:Q:bus1:B", "Mvar, Reactive Power"
      m_Q_bus1_C = "m:Q:bus1:C", "Mvar, Reactive Power"
      m_Q_bus1_N = "m:Q:bus1:N", "Mvar, Reactive Power"
      m_S_bus1_A = "m:S:bus1:A", "MVA, Apparent Power"
      m_S_bus1_B = "m:S:bus1:B", "MVA, Apparent Power"
      m_S_bus1_C = "m:S:bus1:C", "MVA, Apparent Power"
      m_S_bus1_N = "m:S:bus1:N", "MVA, Apparent Power"
      m_cosphi_bus1_A = "m:cosphi:bus1:A", "Power Factor"
      m_cosphi_bus1_B = "m:cosphi:bus1:B", "Power Factor"
      m_cosphi_bus1_C = "m:cosphi:bus1:C", "Power Factor"
      m_cosphi_bus1_N = "m:cosphi:bus1:N", "Power Factor"
      m_tanphi_bus1_A = "m:tanphi:bus1:A", "tan(phi)"
      m_tanphi_bus1_B = "m:tanphi:bus1:B", "tan(phi)"
      m_tanphi_bus1_C = "m:tanphi:bus1:C", "tan(phi)"
      m_tanphi_bus1_N = "m:tanphi:bus1:N", "tan(phi)"
      m_Psum_bus1 = "m:Psum:bus1", "MW, Total Active Power"
      m_Qsum_bus1 = "m:Qsum:bus1", "Mvar, Total Reactive Power"
      m_Ssum_bus1 = "m:Ssum:bus1", "MVA, Total Apparent Power"
      m_cosphisum_bus1 = "m:cosphisum:bus1", "Total Power Factor"
      m_tanphisum_bus1 = "m:tanphisum:bus1", "Total tan(phi)"
      m_Pln_bus1_A = "m:Pln:bus1:A", "MW, Active Power (Line-Neutral)"
      m_Pln_bus1_B = "m:Pln:bus1:B", "MW, Active Power (Line-Neutral)"
      m_Pln_bus1_C = "m:Pln:bus1:C", "MW, Active Power (Line-Neutral)"
      m_Pln_bus1_N = "m:Pln:bus1:N", "MW, Active Power (Line-Neutral)"
      m_Qln_bus1_A = "m:Qln:bus1:A", "Mvar, Reactive Power (Line-Neutral)"
      m_Qln_bus1_B = "m:Qln:bus1:B", "Mvar, Reactive Power (Line-Neutral)"
      m_Qln_bus1_C = "m:Qln:bus1:C", "Mvar, Reactive Power (Line-Neutral)"
      m_Qln_bus1_N = "m:Qln:bus1:N", "Mvar, Reactive Power (Line-Neutral)"
      m_P1_bus1 = "m:P1:bus1", "MW, Positive Sequence Active Power"
      m_Q1_bus1 = "m:Q1:bus1", "Mvar, Positive Sequence Reactive Power"
      m_P2_bus1 = "m:P2:bus1", "MW, Negative Sequence Active Power"
      m_Q2_bus1 = "m:Q2:bus1", "Mvar, Negative Sequence Reactive Power"
      m_P0_bus1 = "m:P0:bus1", "MW, Zero Sequence Active Power"
      m_Q0_bus1 = "m:Q0:bus1", "Mvar, Zero Sequence Reactive Power"
      m_i1P_bus1 = "m:i1P:bus1", "p.u., Positive-Sequence Active Current"
      m_i1Q_bus1 = "m:i1Q:bus1", "p.u., Positive-Sequence Reactive Current"
      m_I1P_bus1 = "m:I1P:bus1", "kA, Positive-Sequence Active Current"
      m_I1Q_bus1 = "m:I1Q:bus1", "kA, Positive-Sequence Reactive Current"
      m_i2P_bus1 = "m:i2P:bus1", "p.u., Negative-Sequence Active Current"
      m_i2Q_bus1 = "m:i2Q:bus1", "p.u., Negative-Sequence Reactive Current"
      m_I2P_bus1 = "m:I2P:bus1", "kA, Negative-Sequence Active Current"
      m_I2Q_bus1 = "m:I2Q:bus1", "kA, Negative-Sequence Reactive Current"
      m_ur_bus1_A = "m:ur:bus1:A", "p.u., Phase Voltage, Real Part"
      m_ur_bus1_B = "m:ur:bus1:B", "p.u., Phase Voltage, Real Part"
      m_ur_bus1_C = "m:ur:bus1:C", "p.u., Phase Voltage, Real Part"
      m_ur_bus1_N = "m:ur:bus1:N", "p.u., Phase Voltage, Real Part"
      m_ui_bus1_A = "m:ui:bus1:A", "p.u., Phase Voltage, Imaginary Part"
      m_ui_bus1_B = "m:ui:bus1:B", "p.u., Phase Voltage, Imaginary Part"
      m_ui_bus1_C = "m:ui:bus1:C", "p.u., Phase Voltage, Imaginary Part"
      m_ui_bus1_N = "m:ui:bus1:N", "p.u., Phase Voltage, Imaginary Part"
      m_u_bus1_A = "m:u:bus1:A", "p.u., Phase Voltage, Magnitude"
      m_u_bus1_B = "m:u:bus1:B", "p.u., Phase Voltage, Magnitude"
      m_u_bus1_C = "m:u:bus1:C", "p.u., Phase Voltage, Magnitude"
      m_u_bus1_N = "m:u:bus1:N", "p.u., Phase Voltage, Magnitude"
      m_ir_bus1_A = "m:ir:bus1:A", "p.u., Phase Current, Real Part"
      m_ir_bus1_B = "m:ir:bus1:B", "p.u., Phase Current, Real Part"
      m_ir_bus1_C = "m:ir:bus1:C", "p.u., Phase Current, Real Part"
      m_ir_bus1_N = "m:ir:bus1:N", "p.u., Phase Current, Real Part"
      m_ii_bus1_A = "m:ii:bus1:A", "p.u., Phase Current, Imaginary Part"
      m_ii_bus1_B = "m:ii:bus1:B", "p.u., Phase Current, Imaginary Part"
      m_ii_bus1_C = "m:ii:bus1:C", "p.u., Phase Current, Imaginary Part"
      m_ii_bus1_N = "m:ii:bus1:N", "p.u., Phase Current, Imaginary Part"
      m_i_bus1_A = "m:i:bus1:A", "p.u., Phase Current, Magnitude"
      m_i_bus1_B = "m:i:bus1:B", "p.u., Phase Current, Magnitude"
      m_i_bus1_C = "m:i:bus1:C", "p.u., Phase Current, Magnitude"
      m_i_bus1_N = "m:i:bus1:N", "p.u., Phase Current, Magnitude"
      m_phiui_bus1_A = "m:phiui:bus1:A", "deg, Angle between Voltage and Current"
      m_phiui_bus1_B = "m:phiui:bus1:B", "deg, Angle between Voltage and Current"
      m_phiui_bus1_C = "m:phiui:bus1:C", "deg, Angle between Voltage and Current"
      m_phiui_bus1_N = "m:phiui:bus1:N", "deg, Angle between Voltage and Current"
      m_phiu0i0_bus1 = "m:phiu0i0:bus1", "deg, Angle between Voltage and Current in zero-sequence system"
      m_phiu1i1_bus1 = "m:phiu1i1:bus1", "deg, Angle between Voltage and Current in positive sequence system"
      m_phiu2i2_bus1 = "m:phiu2i2:bus1", "deg, Angle between Voltage and Current in negative sequence system"
      m_Tfct_bus1 = "m:Tfct:bus1", "s, Fault Clearing Time"
      m_TfctPh_bus1_A = "m:TfctPh:bus1:A", "s, Fault Clearing Time"
      m_TfctPh_bus1_B = "m:TfctPh:bus1:B", "s, Fault Clearing Time"
      m_TfctPh_bus1_C = "m:TfctPh:bus1:C", "s, Fault Clearing Time"
      m_TfctPh_bus1_N = "m:TfctPh:bus1:N", "s, Fault Clearing Time"
      m_Brkload_bus1 = "m:Brkload:bus1", "%, Max. Breaker Loading"
      m_BrkloadPh_bus1_A = "m:BrkloadPh:bus1:A", "%, Breaker Loading"
      m_BrkloadPh_bus1_B = "m:BrkloadPh:bus1:B", "%, Breaker Loading"
      m_BrkloadPh_bus1_C = "m:BrkloadPh:bus1:C", "%, Breaker Loading"
      m_BrkloadPh_bus1_N = "m:BrkloadPh:bus1:N", "%, Breaker Loading"
      n_ur_bus1_A = "n:ur:bus1:A", "p.u., Line-Ground Voltage, Real Part"
      n_ur_bus1_B = "n:ur:bus1:B", "p.u., Line-Ground Voltage, Real Part"
      n_ur_bus1_C = "n:ur:bus1:C", "p.u., Line-Ground Voltage, Real Part"
      n_ur_bus1_N = "n:ur:bus1:N", "p.u., Line-Ground Voltage, Real Part"
      n_ui_bus1_A = "n:ui:bus1:A", "p.u., Line-Ground Voltage, Imaginary Part"
      n_ui_bus1_B = "n:ui:bus1:B", "p.u., Line-Ground Voltage, Imaginary Part"
      n_ui_bus1_C = "n:ui:bus1:C", "p.u., Line-Ground Voltage, Imaginary Part"
      n_ui_bus1_N = "n:ui:bus1:N", "p.u., Line-Ground Voltage, Imaginary Part"
      n_u_bus1_A = "n:u:bus1:A", "p.u., Line-Ground Voltage, Magnitude"
      n_u_bus1_B = "n:u:bus1:B", "p.u., Line-Ground Voltage, Magnitude"
      n_u_bus1_C = "n:u:bus1:C", "p.u., Line-Ground Voltage, Magnitude"
      n_u_bus1_N = "n:u:bus1:N", "p.u., Line-Ground Voltage, Magnitude"
      n_upc_bus1_A = "n:upc:bus1:A", "%, Line-Ground Voltage, Magnitude"
      n_upc_bus1_B = "n:upc:bus1:B", "%, Line-Ground Voltage, Magnitude"
      n_upc_bus1_C = "n:upc:bus1:C", "%, Line-Ground Voltage, Magnitude"
      n_upc_bus1_N = "n:upc:bus1:N", "%, Line-Ground Voltage, Magnitude"
      n_U_bus1_A = "n:U:bus1:A", "kV, Line-Ground Voltage, Magnitude"
      n_U_bus1_B = "n:U:bus1:B", "kV, Line-Ground Voltage, Magnitude"
      n_U_bus1_C = "n:U:bus1:C", "kV, Line-Ground Voltage, Magnitude"
      n_U_bus1_N = "n:U:bus1:N", "kV, Line-Ground Voltage, Magnitude"
      n_phiu_bus1_A = "n:phiu:bus1:A", "deg, Line-Ground Voltage, Angle"
      n_phiu_bus1_B = "n:phiu:bus1:B", "deg, Line-Ground Voltage, Angle"
      n_phiu_bus1_C = "n:phiu:bus1:C", "deg, Line-Ground Voltage, Angle"
      n_phiu_bus1_N = "n:phiu:bus1:N", "deg, Line-Ground Voltage, Angle"
      n_phiurel_bus1 = "n:phiurel:bus1", "deg, Voltage, Relative Angle"
      n_ul_bus1_A = "n:ul:bus1:A", "p.u., Line-Line Voltage, Magnitude"
      n_ul_bus1_B = "n:ul:bus1:B", "p.u., Line-Line Voltage, Magnitude"
      n_ul_bus1_C = "n:ul:bus1:C", "p.u., Line-Line Voltage, Magnitude"
      n_ul_bus1_N = "n:ul:bus1:N", "p.u., Line-Line Voltage, Magnitude"
      n_ulpc_bus1_A = "n:ulpc:bus1:A", "%, Line-Line Voltage, Magnitude"
      n_ulpc_bus1_B = "n:ulpc:bus1:B", "%, Line-Line Voltage, Magnitude"
      n_ulpc_bus1_C = "n:ulpc:bus1:C", "%, Line-Line Voltage, Magnitude"
      n_ulpc_bus1_N = "n:ulpc:bus1:N", "%, Line-Line Voltage, Magnitude"
      n_Ul_bus1_A = "n:Ul:bus1:A", "kV, Line-Line Voltage, Magnitude"
      n_Ul_bus1_B = "n:Ul:bus1:B", "kV, Line-Line Voltage, Magnitude"
      n_Ul_bus1_C = "n:Ul:bus1:C", "kV, Line-Line Voltage, Magnitude"
      n_Ul_bus1_N = "n:Ul:bus1:N", "kV, Line-Line Voltage, Magnitude"
      n_phiul_bus1_A = "n:phiul:bus1:A", "deg, Line-Line Voltage, Angle"
      n_phiul_bus1_B = "n:phiul:bus1:B", "deg, Line-Line Voltage, Angle"
      n_phiul_bus1_C = "n:phiul:bus1:C", "deg, Line-Line Voltage, Angle"
      n_phiul_bus1_N = "n:phiul:bus1:N", "deg, Line-Line Voltage, Angle"
      n_u0_bus1 = "n:u0:bus1", "p.u., Zero-Sequence Voltage, Magnitude"
      n_U0_bus1 = "n:U0:bus1", "kV, Zero-Sequence Voltage, Magnitude"
      n_U0x3_bus1 = "n:U0x3:bus1", "kV, 3*U0"
      n_phiu0_bus1 = "n:phiu0:bus1", "deg, Zero-Sequence Voltage, Angle"
      n_u1_bus1 = "n:u1:bus1", "p.u., Positive-Sequence Voltage, Magnitude"
      n_u1pc_bus1 = "n:u1pc:bus1", "%, Positive-Sequence Voltage, Magnitude"
      n_u1r_bus1 = "n:u1r:bus1", "p.u., Positive-Sequence Voltage, Real Part"
      n_u1i_bus1 = "n:u1i:bus1", "p.u., Positive-Sequence Voltage, Imaginary Part"
      n_U1_bus1 = "n:U1:bus1", "kV, Line-Ground Positive-Sequence Voltage, Magnitude"
      n_phiu1_bus1 = "n:phiu1:bus1", "deg, Positive-Sequence Voltage, Angle"
      n_u2_bus1 = "n:u2:bus1", "p.u., Negative-Sequence Voltage, Magnitude"
      n_U2_bus1 = "n:U2:bus1", "kV, Line-Ground Negative-Sequence Voltage, Magnitude"
      n_phiu2_bus1 = "n:phiu2:bus1", "deg, Negative-Sequence Voltage, Angle"
      n_U1l_bus1 = "n:U1l:bus1", "kV, Line-Line Positive-Sequence Voltage, Magnitude"
      n_U2l_bus1 = "n:U2l:bus1", "kV, Line-Line Negative-Sequence Voltage, Magnitude"
      n_uln_bus1_A = "n:uln:bus1:A", "p.u., Line-Neutral Voltage, Magnitude"
      n_uln_bus1_B = "n:uln:bus1:B", "p.u., Line-Neutral Voltage, Magnitude"
      n_uln_bus1_C = "n:uln:bus1:C", "p.u., Line-Neutral Voltage, Magnitude"
      n_uln_bus1_N = "n:uln:bus1:N", "p.u., Line-Neutral Voltage, Magnitude"
      n_Uln_bus1_A = "n:Uln:bus1:A", "kV, Line-Neutral Voltage, Magnitude"
      n_Uln_bus1_B = "n:Uln:bus1:B", "kV, Line-Neutral Voltage, Magnitude"
      n_Uln_bus1_C = "n:Uln:bus1:C", "kV, Line-Neutral Voltage, Magnitude"
      n_Uln_bus1_N = "n:Uln:bus1:N", "kV, Line-Neutral Voltage, Magnitude"
      n_phiuln_bus1_A = "n:phiuln:bus1:A", "deg, Line-Neutral Voltage, Angle"
      n_phiuln_bus1_B = "n:phiuln:bus1:B", "deg, Line-Neutral Voltage, Angle"
      n_phiuln_bus1_C = "n:phiuln:bus1:C", "deg, Line-Neutral Voltage, Angle"
      n_phiuln_bus1_N = "n:phiuln:bus1:N", "deg, Line-Neutral Voltage, Angle"
      n_un_bus1 = "n:un:bus1", "p.u., Neutral-Ground Voltage, Magnitude"
      n_Un_bus1 = "n:Un:bus1", "kV, Neutral-Ground Voltage, Magnitude"
      n_phiun_bus1 = "n:phiun:bus1", "deg, Neutral-Ground Voltage, Angle"
      n_upht_bus1_A = "n:upht:bus1:A", "p.u., Phase Technology dependent Voltage, Magnitude"
      n_upht_bus1_B = "n:upht:bus1:B", "p.u., Phase Technology dependent Voltage, Magnitude"
      n_upht_bus1_C = "n:upht:bus1:C", "p.u., Phase Technology dependent Voltage, Magnitude"
      n_upht_bus1_N = "n:upht:bus1:N", "p.u., Phase Technology dependent Voltage, Magnitude"
      n_Upht_bus1_A = "n:Upht:bus1:A", "kV, Phase Technology dependent Voltage, Magnitude"
      n_Upht_bus1_B = "n:Upht:bus1:B", "kV, Phase Technology dependent Voltage, Magnitude"
      n_Upht_bus1_C = "n:Upht:bus1:C", "kV, Phase Technology dependent Voltage, Magnitude"
      n_Upht_bus1_N = "n:Upht:bus1:N", "kV, Phase Technology dependent Voltage, Magnitude"
      n_um_bus1 = "n:um:bus1", "p.u., Average-Voltage, Magnitude"
      n_Um_bus1 = "n:Um:bus1", "kV, Average-Voltage, Magnitude"
      n_uphtmin_bus1 = "n:uphtmin:bus1", "p.u., Minimum of Phase Technology dependent Voltage, Magnitude"
      n_uphtmax_bus1 = "n:uphtmax:bus1", "p.u., Maximum of Phase Technology dependent Voltage, Magnitude"
      n_Pgen_bus1 = "n:Pgen:bus1", "MW, Generation, Active Power"
      n_Qgen_bus1 = "n:Qgen:bus1", "Mvar, Generation, Reactive Power"
      n_Pmot_bus1 = "n:Pmot:bus1", "MW, Motor Load, Active Power"
      n_Qmot_bus1 = "n:Qmot:bus1", "Mvar, Motor Load, Reactive Power"
      n_Pload_bus1 = "n:Pload:bus1", "MW, General Load, Active Power"
      n_Qload_bus1 = "n:Qload:bus1", "Mvar, General Load, Reactive Power"
      n_Pcomp_bus1 = "n:Pcomp:bus1", "MW, Compensation (Losses)"
      n_Qcomp_bus1 = "n:Qcomp:bus1", "Mvar, Compensation"
      n_Pnet_bus1 = "n:Pnet:bus1", "MW, External Networks, Active Power"
      n_Qnet_bus1 = "n:Qnet:bus1", "Mvar, External Networks, Reactive Power"
      n_Pflow_bus1 = "n:Pflow:bus1", "MW, Power Flow, Active Power"
      n_Qflow_bus1 = "n:Qflow:bus1", "Mvar, Power Flow, Reactive Power"
      n_Pout_bus1 = "n:Pout:bus1", "MW, Outgoing Flow, Active Power"
      n_Qout_bus1 = "n:Qout:bus1", "Mvar, Outgoing Power, Reactive Power"
      n_Sout_bus1 = "n:Sout:bus1", "MVA, Outgoing Power, Apparent Power"
      n_cosphiout_bus1 = "n:cosphiout:bus1", "Outgoing Power, Power Factor"
      n_Pbalance_bus1 = "n:Pbalance:bus1", "MW, Active Power Balance (=0)"
      n_Qbalance_bus1 = "n:Qbalance:bus1", "Mvar, Reactive Power Balance (=0)"
      n_ubfac_bus1 = "n:ubfac:bus1", "%, Unbalance Factor"
      n_LossPdown_bus1 = "n:LossPdown:bus1", "MW, Losses, downstream"
      n_LossQdown_bus1 = "n:LossQdown:bus1", "Mvar, Losses, downstream (Reactive Power)"
      n_LossPdownload_bus1 = "n:LossPdownload:bus1", "MW, Load losses, downstream"
      n_LossQdownload_bus1 = "n:LossQdownload:bus1", "Mvar, Load losses, downstream"
      n_LossPdownnoload_bus1 = "n:LossPdownnoload:bus1", "MW, No load losses, downstream"
      n_LossQdownnoload_bus1 = "n:LossQdownnoload:bus1", "Mvar, No load losses, downstream"
      s_scale = "s:scale", "IN, Scale Factor, continuous, double"
      s_scalephi = "s:scalephi", "rad, IN, Load Scaling: Power Angle, continuous, double"
      s_scale1r = "s:scale1r", "IN, Feeder load scaling, control 1, ph. 1, continuous, double"
      s_scale1s = "s:scale1s", "IN, Feeder load scaling, control 1, ph. 2, continuous, double"
      s_scale1t = "s:scale1t", "IN, Feeder load scaling, control 1, ph. 3, continuous, double"
      s_scale2r = "s:scale2r", "IN, Feeder load scaling, control 2, ph. 1, continuous, double"
      s_scale2s = "s:scale2s", "IN, Feeder load scaling, control 2, ph. 2, continuous, double"
      s_scale2t = "s:scale2t", "IN, Feeder load scaling, control 2, ph. 3, continuous, double"
      c_prini = "c:prini", "MW, Nominal Active Power, in phase A"
      c_qrini = "c:qrini", "Mvar, Nominal Reactive Power in phase A"
      c_psini = "c:psini", "MW, Nominal Active Power, in phase B"
      c_qsini = "c:qsini", "Mvar, Nominal Reactive Power in phase B"
      c_ptini = "c:ptini", "MW, Nominal Active Power, in phase C"
      c_qtini = "c:qtini", "Mvar, Nominal Reactive Power in phase C"
      c_sr_r = "c:sr:r", "MW, Complex Power in phase A, Real Part"
      c_sr_i = "c:sr:i", "Mvar, Complex Power in phase A, Imaginary Part"
      c_sr_phi = "c:sr:phi", "deg, Complex Power in phase A, Angle"
      c_sr_mag = "c:sr:mag", "MVA, Complex Power in phase A, Magnitude"
      c_sr_phirad = "c:sr:phirad", "rad, Complex Power in phase A, Angle"
      c_ss_r = "c:ss:r", "MW, Complex Power in phase B, Real Part"
      c_ss_i = "c:ss:i", "Mvar, Complex Power in phase B, Imaginary Part"
      c_ss_phi = "c:ss:phi", "deg, Complex Power in phase B, Angle"
      c_ss_mag = "c:ss:mag", "MVA, Complex Power in phase B, Magnitude"
      c_ss_phirad = "c:ss:phirad", "rad, Complex Power in phase B, Angle"
      c_st_r = "c:st:r", "MW, Complex Power in phase C, Real Part"
      c_st_i = "c:st:i", "Mvar, Complex Power in phase C, Imaginary Part"
      c_st_phi = "c:st:phi", "deg, Complex Power in phase C, Angle"
      c_st_mag = "c:st:mag", "MVA, Complex Power in phase C, Magnitude"
      c_st_phirad = "c:st:phirad", "rad, Complex Power in phase C, Angle"
      c_frnom = "c:frnom", "Hz, Nominal Frequency"
      c_hpi = "c:hpi", "rad/s, Nominal angular frequency"
      c_u0 = "c:u0", "p.u., Nominal Voltage"
      c_ncust = "c:ncust", "Number of Connected Customers"
      c_scale0 = "c:scale0", "Initial Scale Factor"
      c_zonescale = "c:zonescale", "Zone Scale Factor"
      c_gscale = "c:gscale", "Generation Scale Factor"
      c_tempscFac = "c:tempscFac", "Temperature dependent load scaling factor"
      c_kpu0 = "c:kpu0", "Volt. Dependence I on P"
      c_kqu0 = "c:kqu0", "Volt. Dependence I on Q"
      c_kpu1 = "c:kpu1", "Volt. Dependence II on P"
      c_kqu1 = "c:kqu1", "Volt. Dependence II on Q"
      c_kpu = "c:kpu", "Volt. Dependence III on P"
      c_kqu = "c:kqu", "Volt. Dependence III on Q"
      c_aP = "c:aP", "Percent of Volt. Dep. I on P"
      c_bP = "c:bP", "Percent of Volt. Dep. II on P"
      c_aQ = "c:aQ", "Percent of Volt. Dep. I on Q"
      c_bQ = "c:bQ", "Percent of Volt. Dep. II on Q"
      c_z0_r = "c:z0:r", "p.u., z0, Real Part"
      c_z0_i = "c:z0:i", "p.u., z0, Imaginary Part"
      c_z0_phi = "c:z0:phi", "deg, z0, Angle"
      c_z0_mag = "c:z0:mag", "p.u., z0, Magnitude"
      c_z0_phirad = "c:z0:phirad", "rad, z0, Angle"
      c_iRefElement = "c:iRefElement", "Reference Element"
      c_ulv_A = "c:ulv:A", "p.u., LV Voltage"
      c_ulv_B = "c:ulv:B", "p.u., LV Voltage"
      c_ulv_C = "c:ulv:C", "p.u., LV Voltage"
      c_ulv_N = "c:ulv:N", "p.u., LV Voltage"
      c_Gmload = "c:Gmload", "kW, Iron Losses"
      c_Ploss = "c:Ploss", "MW, Losses (total)"
      c_Qloss = "c:Qloss", "Mvar, Reactive-Losses (total)"
      c_Plossld = "c:Plossld", "MW, Losses (load)"
      c_Qlossld = "c:Qlossld", "Mvar, Reactive-Losses (load)"
      c_Plossnld = "c:Plossnld", "MW, Losses (no load)"
      c_Qlossnld = "c:Qlossnld", "Mvar, Reactive-Losses (no load)"
      e_mode_inp = "e:mode_inp", "Input Mode"
      e_ci_sym = "e:ci_sym", "Balanced/Unbalanced"
      e_plini = "e:plini", "MW, P, Load"
      e_slini = "e:slini", "MVA, S, Load"
      e_coslini = "e:coslini", "cos(phi), Load"
      e_pf_recap = "e:pf_recap", "cos(phi), Load"
      e_cplinia = "e:cplinia", "MW, P, Load"
      e_elini = "e:elini", "MWh, Yearly Energy"
      e_pProfile = "e:pProfile", "ChaTime,ChaProfile, Consumption Profile"
      e_scale0 = "e:scale0", "Load Scaling Factor"
      e_i_scale = "e:i_scale", "Adjusted by Load Scaling"
      e_pgini = "e:pgini", "MW, P, Generation"
      e_sgini = "e:sgini", "MVA, S, Generation"
      e_cosgini = "e:cosgini", "cos(phi), Generation"
      e_pfg_recap = "e:pfg_recap", "cos(phi), Generation"
      e_gscale = "e:gscale", "Generation Scaling Factor"
      e_plinir = "e:plinir", "MW, P, Load"
      e_slinir = "e:slinir", "MVA, S, Load"
      e_coslinir = "e:coslinir", "cos(phi), Load"
      e_pf_recapr = "e:pf_recapr", "cos(phi), Load"
      e_pginir = "e:pginir", "MW, P, Generation"
      e_sginir = "e:sginir", "MVA, S, Generation"
      e_cosginir = "e:cosginir", "cos(phi), Generation"
      e_pfg_recapr = "e:pfg_recapr", "cos(phi), Generation"
      e_plinis = "e:plinis", "MW, P, Load"
      e_slinis = "e:slinis", "MVA, S, Load"
      e_coslinis = "e:coslinis", "cos(phi), Load"
      e_pf_recaps = "e:pf_recaps", "cos(phi), Load"
      e_pginis = "e:pginis", "MW, P, Generation"
      e_sginis = "e:sginis", "MVA, S, Generation"
      e_cosginis = "e:cosginis", "cos(phi), Generation"
      e_pfg_recaps = "e:pfg_recaps", "cos(phi), Generation"
      e_plinit = "e:plinit", "MW, P, Load"
      e_slinit = "e:slinit", "MVA, S, Load"
      e_coslinit = "e:coslinit", "cos(phi), Load"
      e_pf_recapt = "e:pf_recapt", "cos(phi), Load"
      e_pginit = "e:pginit", "MW, P, Generation"
      e_sginit = "e:sginit", "MVA, S, Generation"
      e_cosginit = "e:cosginit", "cos(phi), Generation"
      e_pfg_recapt = "e:pfg_recapt", "cos(phi), Generation"
      e_nntap = "e:nntap", "Tap Position"
      e_dutap = "e:dutap", "%, Additional Voltage per Tap"
      e_qdslCtrl = "e:qdslCtrl", "Quasi-Dynamic Model"
      e_plini_a = "e:plini_a", "MW, P(act.)"
      e_slini_a = "e:slini_a", "MVA, S(act.)"
      e_coslini_a = "e:coslini_a", "cos(phi)(act.)"
      e_scale0_a = "e:scale0_a", "Scaling Factor(act.)"
      e_plinir_a = "e:plinir_a", "MW, Ph 1:P(act.)"
      e_slinir_a = "e:slinir_a", "MVA, Ph 1:S(act.)"
      e_coslinir_a = "e:coslinir_a", "Ph 1:cos(phi)(act.)"
      e_plinis_a = "e:plinis_a", "MW, Ph 2:P(act.)"
      e_slinis_a = "e:slinis_a", "MVA, Ph 2:S(act.)"
      e_coslinis_a = "e:coslinis_a", "Ph 2:cos(phi)(act.)"
      e_plinit_a = "e:plinit_a", "MW, Ph 3:P(act.)"
      e_slinit_a = "e:slinit_a", "MVA, Ph 3:S(act.)"
      e_coslinit_a = "e:coslinit_a", "Ph 3:cos(phi)(act.)"
      e_gscale_a = "e:gscale_a", "Gen. Scaling Factor(act.)"
      e_pgini_a = "e:pgini_a", "MW, Pgen(act.)"
      e_sgini_a = "e:sgini_a", "MVA, Sgen(act.)"
      e_cosgini_a = "e:cosgini_a", "cos(phi)(act.), Gen."
      e_pginir_a = "e:pginir_a", "MW, Ph 1:Pgen(act.)"
      e_sginir_a = "e:sginir_a", "MVA, Ph 1:Sgen(act.)"
      e_cosginir_a = "e:cosginir_a", "Ph 1:cos(phi)(act.), Gen."
      e_pginis_a = "e:pginis_a", "MW, Ph 2:Pgen(act.)"
      e_sginis_a = "e:sginis_a", "MVA, Ph 2:Sgen(act.)"
      e_cosginis_a = "e:cosginis_a", "Ph 2:cos(phi)(act.), Gen."
      e_pginit_a = "e:pginit_a", "MW, Ph 3:Pgen(act.)"
      e_sginit_a = "e:sginit_a", "MVA, Ph 3:Sgen(act.)"
      e_cosginit_a = "e:cosginit_a", "Ph 3:cos(phi)(act.), Gen."
      e_cpFeed = "e:cpFeed", "Feeder"
      e_ciDist = "e:ciDist", "Distance from infeed in number of buses"
      e_ciLater = "e:ciLater", "Lateral Index"
      e_ciDistRoot = "e:ciDistRoot", "Distance from first infeed in number of buses"
      e_ciDistAll = "e:ciDistAll", "Distance from infeed in number of buses including switches"
      e_ciDistAllRoot = "e:ciDistAllRoot", "Distance from first infeed in number of buses including switches"
      r_pProfile = "r:pProfile", "ChaTime,ChaProfile, Consumption Profile"
      r_qdslCtrl = "r:qdslCtrl", "Quasi-Dynamic Model"
      r_cpFeed = "r:cpFeed", "Feeder"
      m_Ikss_bus1 = "m:Ikss:bus1", "kA, Initial Short-Circuit Current"
      m_ikss_bus1 = "m:ikss:bus1", "p.u., Initial Short-Circuit Current, Magnitude"
      m_phii_bus1 = "m:phii:bus1", "deg, Phase Current, Angle"
      m_maxIkss_bus1 = "m:maxIkss:bus1", "kA, Max. Initial Short-Circuit Current"
      m_Skss_bus1 = "m:Skss:bus1", "MVA, Initial Short-Circuit Power"
      m_ip_bus1 = "m:ip:bus1", "kA, Peak Short-Circuit Current"
      m_maxip_bus1 = "m:maxip:bus1", "kA, Max. Peak Short-Circuit Current"
      m_maxIb_bus1 = "m:maxIb:bus1", "kA, Max. Breaking Current"
      m_maxIth_bus1 = "m:maxIth:bus1", "kA, Max. Thermal equivalent Short-Circuit Current"
      m_Ib_bus1 = "m:Ib:bus1", "kA, Short-Circuit Breaking Current"
      m_Sb_bus1 = "m:Sb:bus1", "MVA, Short-Circuit Breaking Power"
      m_Ik_bus1 = "m:Ik:bus1", "kA, Steady-State Short-Circuit Current"
      m_Ith_bus1 = "m:Ith:bus1", "kA, Thermal equivalent Short-Circuit Current"
      m_I_bus1 = "m:I:bus1", "kA, Short-Circuit Current"
      m_R_bus1 = "m:R:bus1", "Ohm, Fault Impedance, Real Part"
      m_X_bus1 = "m:X:bus1", "Ohm, Fault Impedance, Imaginary Part"
      m_Z_bus1 = "m:Z:bus1", "Ohm, Fault Impedance, Magnitude"
      m_phiz_bus1 = "m:phiz:bus1", "deg, Fault Impedance, Angle"
      m_zSbase_bus1 = "m:zSbase:bus1", "p.u., Fault Impedance, Magnitude"
      m_XtoR_bus1 = "m:XtoR:bus1", "X/R ratio"
      m_Zs_bus1 = "m:Zs:bus1", "Ohm, Source Impedance, Magnitude"
      m_phizs_bus1 = "m:phizs:bus1", "deg, Source Impedance, Angle"
      m_Rs_bus1 = "m:Rs:bus1", "Ohm, Source Impedance, Real Part"
      m_Xs_bus1 = "m:Xs:bus1", "Ohm, Source Impedance, Imaginary Part"
      m_phiui_bus1 = "m:phiui:bus1", "deg, Angle between Voltage and Current"
      m_cosphiui_bus1 = "m:cosphiui:bus1", "cosphi"
      m_P_bus1 = "m:P:bus1", "MW, Active Power"
      m_Q_bus1 = "m:Q:bus1", "Mvar, Reactive Power"
      m_BrkIpload_bus1 = "m:BrkIpload:bus1", "%, Breaker Loading (peak)"
      m_BrkIbload_bus1 = "m:BrkIbload:bus1", "%, Breaker Loading (breaking)"
      m_BrkmaxIb_bus1 = "m:BrkmaxIb:bus1", "kA, Max. Breaking Current (breaker time)"
      n_Ikss_bus1 = "n:Ikss:bus1", "kA, Initial Short-Circuit Current"
      n_I_bus1 = "n:I:bus1", "kA, Short-Circuit Current"
      n_phii_bus1 = "n:phii:bus1", "deg, Phase Current, Angle"
      n_Skss_bus1 = "n:Skss:bus1", "MVA, Initial Short-Circuit Power"
      n_ip_bus1 = "n:ip:bus1", "kA, Peak Short-Circuit Current"
      n_Ib_bus1 = "n:Ib:bus1", "kA, Short-Circuit Breaking Current"
      n_Sb_bus1 = "n:Sb:bus1", "MVA, Short-Circuit Breaking Power"
      n_Ik_bus1 = "n:Ik:bus1", "kA, Steady-State Short-Circuit Current"
      n_Ith_bus1 = "n:Ith:bus1", "kA, Thermal equivalent Short-Circuit Current"
      n_R_bus1 = "n:R:bus1", "Ohm, Short-Circuit Impedance, Real Part"
      n_X_bus1 = "n:X:bus1", "Ohm, Short-Circuit Impedance, Imaginary Part"
      n_Z_bus1 = "n:Z:bus1", "Ohm, Short-Circuit Impedance, Magnitude"
      n_phiz_bus1 = "n:phiz:bus1", "deg, Short-Circuit Impedance, Angle"
      n_rSbase_bus1 = "n:rSbase:bus1", "p.u., Short-Circuit Impedance, Real Part"
      n_xSbase_bus1 = "n:xSbase:bus1", "p.u., Short-Circuit Impedance, Imaginary Part"
      n_zSbase_bus1 = "n:zSbase:bus1", "p.u., Short-Circuit Impedance, Magnitude"
      n_XtoR_bus1 = "n:XtoR:bus1", "X/R ratio (peak)"
      n_RtoX_bus1 = "n:RtoX:bus1", "R/X ratio (peak)"
      n_ur_bus1 = "n:ur:bus1", "p.u., Voltage, Real Part"
      n_ui_bus1 = "n:ui:bus1", "p.u., Voltage, Imaginary Part"
      n_u_bus1 = "n:u:bus1", "p.u., Voltage, Magnitude"
      n_upc_bus1 = "n:upc:bus1", "%, Voltage, Magnitude"
      n_U_bus1 = "n:U:bus1", "kV, Line-Ground Voltage, Magnitude"
      n_Ul_bus1 = "n:Ul:bus1", "kV, Line-Line Voltage, Magnitude"
      n_phiu_bus1 = "n:phiu:bus1", "deg, Voltage, Angle"
      n_du_bus1 = "n:du:bus1", "%, Voltage Deviation"
      n_urpref_bus1 = "n:urpref:bus1", "p.u., Pre-fault Voltage, Real Part"
      n_uipref_bus1 = "n:uipref:bus1", "p.u., Pre-fault Voltage, Imaginary Part"
      n_upref_bus1 = "n:upref:bus1", "p.u., Pre-fault Voltage, Magnitude"
      n_uprefpc_bus1 = "n:uprefpc:bus1", "%, Pre-fault Voltage, Magnitude"
      n_Upref_bus1 = "n:Upref:bus1", "kV, Line-Ground Pre-fault Voltage, Magnitude"
      n_Ulpref_bus1 = "n:Ulpref:bus1", "kV, Line-Line Pre-fault Voltage, Magnitude"
      n_phiupref_bus1 = "n:phiupref:bus1", "deg, Pre-fault Voltage, Angle"
      n_phiui_bus1 = "n:phiui:bus1", "deg, Angle between Voltage and Current"
      n_cfac_bus1 = "n:cfac:bus1", "Voltage Factor"
      n_idc_bus1 = "n:idc:bus1", "kA, Decaying Component"
      n_Iasy_bus1 = "n:Iasy:bus1", "kA, Asymmetrical Breaking Current"
      n_Ithload_bus1 = "n:Ithload:bus1", "%, Loading, Thermal equivalent Current"
      n_Ipload_bus1 = "n:Ipload:bus1", "%, Loading, Peak Short-Circuit Current"
      n_Ithrtk_bus1 = "n:Ithrtk:bus1", "kA, Rated Short-Time Current (Tk)"
      n_Tn_bus1 = "n:Tn:bus1", "ms, Grid Time Constant"
      n_XtoR_b_bus1 = "n:XtoR_b:bus1", "X/R ratio (dc)"
      n_RtoX_b_bus1 = "n:RtoX_b:bus1", "R/X ratio (dc)"
      n_fctof_b_bus1 = "n:fctof_b:bus1", "Equivalent Frequency fc/f (dc)"
      c_i_pure = "c:i_pure", "Purely inductive/capacitive"
      c_icsrc = "c:icsrc", "Current Source/Impedance"
      c_R = "c:R", "p.u., Resistance"
      c_L = "c:L", "p.u., Inductance"
      c_C = "c:C", "p.u., Capacitance"
      c_C_lv = "c:C_lv", "p.u., LV-Capacitance"
      c_y1_r = "c:y1:r", "p.u., Admittance, Real Part"
      c_y1_i = "c:y1:i", "p.u., Admittance, Imaginary Part"
      c_y1_phi = "c:y1:phi", "deg, Admittance, Angle"
      c_y1_mag = "c:y1:mag", "p.u., Admittance, Magnitude"
      c_y1_phirad = "c:y1:phirad", "rad, Admittance, Angle"
      c_i1_r = "c:i1:r", "p.u., Harmonic current, Real Part"
      c_i1_i = "c:i1:i", "p.u., Harmonic current, Imaginary Part"
      c_i1_phi = "c:i1:phi", "deg, Harmonic current, Angle"
      c_i1_mag = "c:i1:mag", "p.u., Harmonic current, Magnitude"
      c_i1_phirad = "c:i1:phirad", "rad, Harmonic current, Angle"
      c_z1_r = "c:z1:r", "p.u., Positive-sequence impedance, Real Part"
      c_z1_i = "c:z1:i", "p.u., Positive-sequence impedance, Imaginary Part"
      c_z1_phi = "c:z1:phi", "deg, Positive-sequence impedance, Angle"
      c_z1_mag = "c:z1:mag", "p.u., Positive-sequence impedance, Magnitude"
      c_z1_phirad = "c:z1:phirad", "rad, Positive-sequence impedance, Angle"
      c_z2_r = "c:z2:r", "p.u., Negative-sequence Impedance, Real Part"
      c_z2_i = "c:z2:i", "p.u., Negative-sequence Impedance, Imaginary Part"
      c_z2_phi = "c:z2:phi", "deg, Negative-sequence Impedance, Angle"
      c_z2_mag = "c:z2:mag", "p.u., Negative-sequence Impedance, Magnitude"
      c_z2_phirad = "c:z2:phirad", "rad, Negative-sequence Impedance, Angle"
      m_Ikss_bus1_A = "m:Ikss:bus1:A", "kA, Initial Short-Circuit Current"
      m_Ikss_bus1_B = "m:Ikss:bus1:B", "kA, Initial Short-Circuit Current"
      m_Ikss_bus1_C = "m:Ikss:bus1:C", "kA, Initial Short-Circuit Current"
      m_Ikss_bus1_N = "m:Ikss:bus1:N", "kA, Initial Short-Circuit Current"
      m_ikss_bus1_A = "m:ikss:bus1:A", "p.u., Initial Short-Circuit Current, Magnitude"
      m_ikss_bus1_B = "m:ikss:bus1:B", "p.u., Initial Short-Circuit Current, Magnitude"
      m_ikss_bus1_C = "m:ikss:bus1:C", "p.u., Initial Short-Circuit Current, Magnitude"
      m_ikss_bus1_N = "m:ikss:bus1:N", "p.u., Initial Short-Circuit Current, Magnitude"
      m_Skss_bus1_A = "m:Skss:bus1:A", "MVA, Initial Short-Circuit Power"
      m_Skss_bus1_B = "m:Skss:bus1:B", "MVA, Initial Short-Circuit Power"
      m_Skss_bus1_C = "m:Skss:bus1:C", "MVA, Initial Short-Circuit Power"
      m_Skss_bus1_N = "m:Skss:bus1:N", "MVA, Initial Short-Circuit Power"
      m_Skssx3_bus1_A = "m:Skssx3:bus1:A", "MVA, 3*Initial Short-Circuit Power"
      m_Skssx3_bus1_B = "m:Skssx3:bus1:B", "MVA, 3*Initial Short-Circuit Power"
      m_Skssx3_bus1_C = "m:Skssx3:bus1:C", "MVA, 3*Initial Short-Circuit Power"
      m_Skssx3_bus1_N = "m:Skssx3:bus1:N", "MVA, 3*Initial Short-Circuit Power"
      m_ip_bus1_A = "m:ip:bus1:A", "kA, Peak Short-Circuit Current"
      m_ip_bus1_B = "m:ip:bus1:B", "kA, Peak Short-Circuit Current"
      m_ip_bus1_C = "m:ip:bus1:C", "kA, Peak Short-Circuit Current"
      m_ip_bus1_N = "m:ip:bus1:N", "kA, Peak Short-Circuit Current"
      m_Z_bus1_A = "m:Z:bus1:A", "Ohm, Fault Impedance, Magnitude"
      m_Z_bus1_B = "m:Z:bus1:B", "Ohm, Fault Impedance, Magnitude"
      m_Z_bus1_C = "m:Z:bus1:C", "Ohm, Fault Impedance, Magnitude"
      m_Z_bus1_N = "m:Z:bus1:N", "Ohm, Fault Impedance, Magnitude"
      m_phiz_bus1_A = "m:phiz:bus1:A", "deg, Fault Impedance, Angle"
      m_phiz_bus1_B = "m:phiz:bus1:B", "deg, Fault Impedance, Angle"
      m_phiz_bus1_C = "m:phiz:bus1:C", "deg, Fault Impedance, Angle"
      m_phiz_bus1_N = "m:phiz:bus1:N", "deg, Fault Impedance, Angle"
      m_Zl_bus1_A = "m:Zl:bus1:A", "Ohm, Line-Line Fault Impedance, Magnitude"
      m_Zl_bus1_B = "m:Zl:bus1:B", "Ohm, Line-Line Fault Impedance, Magnitude"
      m_Zl_bus1_C = "m:Zl:bus1:C", "Ohm, Line-Line Fault Impedance, Magnitude"
      m_Zl_bus1_N = "m:Zl:bus1:N", "Ohm, Line-Line Fault Impedance, Magnitude"
      m_phizl_bus1_A = "m:phizl:bus1:A", "deg, Line-Line Fault Impedance, Angle"
      m_phizl_bus1_B = "m:phizl:bus1:B", "deg, Line-Line Fault Impedance, Angle"
      m_phizl_bus1_C = "m:phizl:bus1:C", "deg, Line-Line Fault Impedance, Angle"
      m_phizl_bus1_N = "m:phizl:bus1:N", "deg, Line-Line Fault Impedance, Angle"
      m_Z0_bus1 = "m:Z0:bus1", "Ohm, Zero-Sequence Fault Impedance, Magnitude"
      m_phiz0_bus1 = "m:phiz0:bus1", "deg, Zero-Sequence Fault Impedance, Angle"
      m_z0Sbase_bus1 = "m:z0Sbase:bus1", "p.u., Zero-Sequence Fault Impedance, Magnitude"
      m_cosphiu1i1_bus1 = "m:cosphiu1i1:bus1", "Positive-Sequence cosphi"
      m_I0x3watt_bus1 = "m:I0x3watt:bus1", "kA, Earth-Fault Residual Current"
      m_I0x3var_bus1 = "m:I0x3var:bus1", "kA, Capacitive Earth-Fault Current"
      m_Q0cap_bus1 = "m:Q0cap:bus1", "Mvar, Q0 capacitive"
      m_Zfrel_bus1 = "m:Zfrel:bus1", "Ohm, Relay Impedance, Magnitude"
      m_phizfrel_bus1 = "m:phizfrel:bus1", "deg, Relay Impedance, Angle"
      m_Rfrel_bus1 = "m:Rfrel:bus1", "Ohm, Relay Impedance, Real Part"
      m_Xfrel_bus1 = "m:Xfrel:bus1", "Ohm, Relay Impedance, Imaginary Part"
      m_Zf_bus1 = "m:Zf:bus1", "Ohm, Meas. Fault Impedance, Magnitude"
      m_phizf_bus1 = "m:phizf:bus1", "deg, Meas. Fault Impedance, Angle"
      n_Ikss_bus1_A = "n:Ikss:bus1:A", "kA, Initial Short-Circuit Current"
      n_Ikss_bus1_B = "n:Ikss:bus1:B", "kA, Initial Short-Circuit Current"
      n_Ikss_bus1_C = "n:Ikss:bus1:C", "kA, Initial Short-Circuit Current"
      n_Ikss_bus1_N = "n:Ikss:bus1:N", "kA, Initial Short-Circuit Current"
      n_I_bus1_A = "n:I:bus1:A", "kA, Short-Circuit Current"
      n_I_bus1_B = "n:I:bus1:B", "kA, Short-Circuit Current"
      n_I_bus1_C = "n:I:bus1:C", "kA, Short-Circuit Current"
      n_I_bus1_N = "n:I:bus1:N", "kA, Short-Circuit Current"
      n_phii_bus1_A = "n:phii:bus1:A", "deg, Phase Current, Angle"
      n_phii_bus1_B = "n:phii:bus1:B", "deg, Phase Current, Angle"
      n_phii_bus1_C = "n:phii:bus1:C", "deg, Phase Current, Angle"
      n_phii_bus1_N = "n:phii:bus1:N", "deg, Phase Current, Angle"
      n_Skss_bus1_A = "n:Skss:bus1:A", "MVA, Initial Short-Circuit Power"
      n_Skss_bus1_B = "n:Skss:bus1:B", "MVA, Initial Short-Circuit Power"
      n_Skss_bus1_C = "n:Skss:bus1:C", "MVA, Initial Short-Circuit Power"
      n_Skss_bus1_N = "n:Skss:bus1:N", "MVA, Initial Short-Circuit Power"
      n_Skssx3_bus1_A = "n:Skssx3:bus1:A", "MVA, 3*Initial Short-Circuit Power"
      n_Skssx3_bus1_B = "n:Skssx3:bus1:B", "MVA, 3*Initial Short-Circuit Power"
      n_Skssx3_bus1_C = "n:Skssx3:bus1:C", "MVA, 3*Initial Short-Circuit Power"
      n_Skssx3_bus1_N = "n:Skssx3:bus1:N", "MVA, 3*Initial Short-Circuit Power"
      n_ip_bus1_A = "n:ip:bus1:A", "kA, Peak Short-Circuit Current"
      n_ip_bus1_B = "n:ip:bus1:B", "kA, Peak Short-Circuit Current"
      n_ip_bus1_C = "n:ip:bus1:C", "kA, Peak Short-Circuit Current"
      n_ip_bus1_N = "n:ip:bus1:N", "kA, Peak Short-Circuit Current"
      n_Ib_bus1_A = "n:Ib:bus1:A", "kA, Short-Circuit Breaking Current"
      n_Ib_bus1_B = "n:Ib:bus1:B", "kA, Short-Circuit Breaking Current"
      n_Ib_bus1_C = "n:Ib:bus1:C", "kA, Short-Circuit Breaking Current"
      n_Ib_bus1_N = "n:Ib:bus1:N", "kA, Short-Circuit Breaking Current"
      n_Sb_bus1_A = "n:Sb:bus1:A", "MVA, Short-Circuit Breaking Power"
      n_Sb_bus1_B = "n:Sb:bus1:B", "MVA, Short-Circuit Breaking Power"
      n_Sb_bus1_C = "n:Sb:bus1:C", "MVA, Short-Circuit Breaking Power"
      n_Sb_bus1_N = "n:Sb:bus1:N", "MVA, Short-Circuit Breaking Power"
      n_Ith_bus1_A = "n:Ith:bus1:A", "kA, Thermal equivalent Short-Circuit Current"
      n_Ith_bus1_B = "n:Ith:bus1:B", "kA, Thermal equivalent Short-Circuit Current"
      n_Ith_bus1_C = "n:Ith:bus1:C", "kA, Thermal equivalent Short-Circuit Current"
      n_Ith_bus1_N = "n:Ith:bus1:N", "kA, Thermal equivalent Short-Circuit Current"
      n_Ithmax_bus1 = "n:Ithmax:bus1", "kA, Max. Thermal equivalent Short-Circuit Current"
      n_Ik_bus1_A = "n:Ik:bus1:A", "kA, Steady-State Short-Circuit Current"
      n_Ik_bus1_B = "n:Ik:bus1:B", "kA, Steady-State Short-Circuit Current"
      n_Ik_bus1_C = "n:Ik:bus1:C", "kA, Steady-State Short-Circuit Current"
      n_Ik_bus1_N = "n:Ik:bus1:N", "kA, Steady-State Short-Circuit Current"
      n_R0_bus1 = "n:R0:bus1", "Ohm, Zero-Sequence Impedance, Real Part"
      n_X0_bus1 = "n:X0:bus1", "Ohm, Zero-Sequence Impedance, Imaginary Part"
      n_Z0_bus1 = "n:Z0:bus1", "Ohm, Zero-Sequence Impedance, Magnitude"
      n_phiz0_bus1 = "n:phiz0:bus1", "deg, Zero-Sequence Impedance, Angle"
      n_r0Sbase_bus1 = "n:r0Sbase:bus1", "p.u., Zero-Sequence Impedance, Real Part"
      n_x0Sbase_bus1 = "n:x0Sbase:bus1", "p.u., Zero-Sequence Impedance, Imaginary Part"
      n_z0Sbase_bus1 = "n:z0Sbase:bus1", "p.u., Zero-Sequence Impedance, Magnitude"
      n_R1_bus1 = "n:R1:bus1", "Ohm, Positive-Sequence Impedance, Real Part"
      n_X1_bus1 = "n:X1:bus1", "Ohm, Positive-Sequence Impedance, Imaginary Part"
      n_Z1_bus1 = "n:Z1:bus1", "Ohm, Positive-Sequence Impedance, Magnitude"
      n_phiz1_bus1 = "n:phiz1:bus1", "deg, Positive-Sequence Impedance, Angle"
      n_r1Sbase_bus1 = "n:r1Sbase:bus1", "p.u., Positive-Sequence Impedance, Real Part"
      n_x1Sbase_bus1 = "n:x1Sbase:bus1", "p.u., Positive-Sequence Impedance, Imaginary Part"
      n_z1Sbase_bus1 = "n:z1Sbase:bus1", "p.u., Positive-Sequence Impedance, Magnitude"
      n_R2_bus1 = "n:R2:bus1", "Ohm, Negative-Sequence Impedance, Real Part"
      n_X2_bus1 = "n:X2:bus1", "Ohm, Negative-Sequence Impedance, Imaginary Part"
      n_Z2_bus1 = "n:Z2:bus1", "Ohm, Negative-Sequence Impedance, Magnitude"
      n_phiz2_bus1 = "n:phiz2:bus1", "deg, Negative-Sequence Impedance, Angle"
      n_r2Sbase_bus1 = "n:r2Sbase:bus1", "p.u., Negative-Sequence Impedance, Real Part"
      n_x2Sbase_bus1 = "n:x2Sbase:bus1", "p.u., Negative-Sequence Impedance, Imaginary Part"
      n_z2Sbase_bus1 = "n:z2Sbase:bus1", "p.u., Negative-Sequence Impedance, Magnitude"
      n_R_bus1_A = "n:R:bus1:A", "Ohm, Network Resistance, Line-Ground"
      n_R_bus1_B = "n:R:bus1:B", "Ohm, Network Resistance, Line-Ground"
      n_R_bus1_C = "n:R:bus1:C", "Ohm, Network Resistance, Line-Ground"
      n_R_bus1_N = "n:R:bus1:N", "Ohm, Network Resistance, Line-Ground"
      n_X_bus1_A = "n:X:bus1:A", "Ohm, Network Reactance, Line-Ground"
      n_X_bus1_B = "n:X:bus1:B", "Ohm, Network Reactance, Line-Ground"
      n_X_bus1_C = "n:X:bus1:C", "Ohm, Network Reactance, Line-Ground"
      n_X_bus1_N = "n:X:bus1:N", "Ohm, Network Reactance, Line-Ground"
      n_Z_bus1_A = "n:Z:bus1:A", "Ohm, Network Impedance, Line-Ground"
      n_Z_bus1_B = "n:Z:bus1:B", "Ohm, Network Impedance, Line-Ground"
      n_Z_bus1_C = "n:Z:bus1:C", "Ohm, Network Impedance, Line-Ground"
      n_Z_bus1_N = "n:Z:bus1:N", "Ohm, Network Impedance, Line-Ground"
      n_phiz_bus1_A = "n:phiz:bus1:A", "deg, Angle of Network Impedance, Line-Ground"
      n_phiz_bus1_B = "n:phiz:bus1:B", "deg, Angle of Network Impedance, Line-Ground"
      n_phiz_bus1_C = "n:phiz:bus1:C", "deg, Angle of Network Impedance, Line-Ground"
      n_phiz_bus1_N = "n:phiz:bus1:N", "deg, Angle of Network Impedance, Line-Ground"
      n_Rl_bus1_A = "n:Rl:bus1:A", "Ohm, Network Resistance, Line-Line"
      n_Rl_bus1_B = "n:Rl:bus1:B", "Ohm, Network Resistance, Line-Line"
      n_Rl_bus1_C = "n:Rl:bus1:C", "Ohm, Network Resistance, Line-Line"
      n_Rl_bus1_N = "n:Rl:bus1:N", "Ohm, Network Resistance, Line-Line"
      n_Xl_bus1_A = "n:Xl:bus1:A", "Ohm, Network Reactance, Line-Line"
      n_Xl_bus1_B = "n:Xl:bus1:B", "Ohm, Network Reactance, Line-Line"
      n_Xl_bus1_C = "n:Xl:bus1:C", "Ohm, Network Reactance, Line-Line"
      n_Xl_bus1_N = "n:Xl:bus1:N", "Ohm, Network Reactance, Line-Line"
      n_Zl_bus1_A = "n:Zl:bus1:A", "Ohm, Network Impedance, Line-Line"
      n_Zl_bus1_B = "n:Zl:bus1:B", "Ohm, Network Impedance, Line-Line"
      n_Zl_bus1_C = "n:Zl:bus1:C", "Ohm, Network Impedance, Line-Line"
      n_Zl_bus1_N = "n:Zl:bus1:N", "Ohm, Network Impedance, Line-Line"
      n_phizl_bus1_A = "n:phizl:bus1:A", "deg, Angle of Network Impedance, Line-Line"
      n_phizl_bus1_B = "n:phizl:bus1:B", "deg, Angle of Network Impedance, Line-Line"
      n_phizl_bus1_C = "n:phizl:bus1:C", "deg, Angle of Network Impedance, Line-Line"
      n_phizl_bus1_N = "n:phizl:bus1:N", "deg, Angle of Network Impedance, Line-Line"
      n_Rln_bus1_A = "n:Rln:bus1:A", "Ohm, Network Resistance, Line-Neutral"
      n_Rln_bus1_B = "n:Rln:bus1:B", "Ohm, Network Resistance, Line-Neutral"
      n_Rln_bus1_C = "n:Rln:bus1:C", "Ohm, Network Resistance, Line-Neutral"
      n_Rln_bus1_N = "n:Rln:bus1:N", "Ohm, Network Resistance, Line-Neutral"
      n_Xln_bus1_A = "n:Xln:bus1:A", "Ohm, Network Reactance, Line-Neutral"
      n_Xln_bus1_B = "n:Xln:bus1:B", "Ohm, Network Reactance, Line-Neutral"
      n_Xln_bus1_C = "n:Xln:bus1:C", "Ohm, Network Reactance, Line-Neutral"
      n_Xln_bus1_N = "n:Xln:bus1:N", "Ohm, Network Reactance, Line-Neutral"
      n_Zln_bus1_A = "n:Zln:bus1:A", "Ohm, Network Impedance, Line-Neutral"
      n_Zln_bus1_B = "n:Zln:bus1:B", "Ohm, Network Impedance, Line-Neutral"
      n_Zln_bus1_C = "n:Zln:bus1:C", "Ohm, Network Impedance, Line-Neutral"
      n_Zln_bus1_N = "n:Zln:bus1:N", "Ohm, Network Impedance, Line-Neutral"
      n_phizln_bus1_A = "n:phizln:bus1:A", "deg, Angle of Network Impedance, Line-Neutral"
      n_phizln_bus1_B = "n:phizln:bus1:B", "deg, Angle of Network Impedance, Line-Neutral"
      n_phizln_bus1_C = "n:phizln:bus1:C", "deg, Angle of Network Impedance, Line-Neutral"
      n_phizln_bus1_N = "n:phizln:bus1:N", "deg, Angle of Network Impedance, Line-Neutral"
      n_Rn_bus1 = "n:Rn:bus1", "Ohm, Network Resistance, Neutral"
      n_Xn_bus1 = "n:Xn:bus1", "Ohm, Network Reactance, Neutral"
      n_Zn_bus1 = "n:Zn:bus1", "Ohm, Network Impedance, Neutral"
      n_phizn_bus1 = "n:phizn:bus1", "deg, Angle of Network Impedance, Neutral"
      n_eff_bus1_A = "n:eff:bus1:A", "Earth Fault Factor"
      n_eff_bus1_B = "n:eff:bus1:B", "Earth Fault Factor"
      n_eff_bus1_C = "n:eff:bus1:C", "Earth Fault Factor"
      n_eff_bus1_N = "n:eff:bus1:N", "Earth Fault Factor"
      n_X0toX1_bus1 = "n:X0toX1:bus1", "X0/X1"
      n_in_bus1 = "n:in:bus1", "p.u., Neutral Current, Magnitude"
      n_In_bus1 = "n:In:bus1", "kA, Neutral Current, Magnitude"
      n_phiin_bus1 = "n:phiin:bus1", "deg, Neutral Current, Angle"
      n_i0r_bus1 = "n:i0r:bus1", "p.u., Zero-Sequence Current, Real Part"
      n_i0i_bus1 = "n:i0i:bus1", "p.u., Zero-Sequence Current, Imaginary Part"
      n_i0_bus1 = "n:i0:bus1", "p.u., Zero-Sequence Current, Magnitude"
      n_I0_bus1 = "n:I0:bus1", "kA, Zero-Sequence Current, Magnitude"
      n_I0x3_bus1 = "n:I0x3:bus1", "kA, 3*I0"
      n_phii0_bus1 = "n:phii0:bus1", "deg, Zero-Sequence Current, Angle"
      n_i1r_bus1 = "n:i1r:bus1", "p.u., Positive-Sequence Current, Real Part"
      n_i1i_bus1 = "n:i1i:bus1", "p.u., Positive-Sequence Current, Imaginary Part"
      n_i1_bus1 = "n:i1:bus1", "p.u., Positive-Sequence Current, Magnitude"
      n_I1_bus1 = "n:I1:bus1", "kA, Positive-Sequence Current, Magnitude"
      n_phii1_bus1 = "n:phii1:bus1", "deg, Positive-Sequence Current, Angle"
      n_i2r_bus1 = "n:i2r:bus1", "p.u., Negative-Sequence Current, Real Part"
      n_i2i_bus1 = "n:i2i:bus1", "p.u., Negative-Sequence Current, Imaginary Part"
      n_i2_bus1 = "n:i2:bus1", "p.u., Negative-Sequence Current, Magnitude"
      n_I2_bus1 = "n:I2:bus1", "kA, Negative-Sequence Current, Magnitude"
      n_phii2_bus1 = "n:phii2:bus1", "deg, Negative-Sequence Current, Angle"
      n_du_bus1_A = "n:du:bus1:A", "%, Voltage Deviation"
      n_du_bus1_B = "n:du:bus1:B", "%, Voltage Deviation"
      n_du_bus1_C = "n:du:bus1:C", "%, Voltage Deviation"
      n_du_bus1_N = "n:du:bus1:N", "%, Voltage Deviation"
      n_urpref_bus1_A = "n:urpref:bus1:A", "p.u., Pre-fault Voltage, Real Part"
      n_urpref_bus1_B = "n:urpref:bus1:B", "p.u., Pre-fault Voltage, Real Part"
      n_urpref_bus1_C = "n:urpref:bus1:C", "p.u., Pre-fault Voltage, Real Part"
      n_urpref_bus1_N = "n:urpref:bus1:N", "p.u., Pre-fault Voltage, Real Part"
      n_uipref_bus1_A = "n:uipref:bus1:A", "p.u., Pre-fault Voltage, Imaginary Part"
      n_uipref_bus1_B = "n:uipref:bus1:B", "p.u., Pre-fault Voltage, Imaginary Part"
      n_uipref_bus1_C = "n:uipref:bus1:C", "p.u., Pre-fault Voltage, Imaginary Part"
      n_uipref_bus1_N = "n:uipref:bus1:N", "p.u., Pre-fault Voltage, Imaginary Part"
      n_upref_bus1_A = "n:upref:bus1:A", "p.u., Pre-fault Voltage, Magnitude"
      n_upref_bus1_B = "n:upref:bus1:B", "p.u., Pre-fault Voltage, Magnitude"
      n_upref_bus1_C = "n:upref:bus1:C", "p.u., Pre-fault Voltage, Magnitude"
      n_upref_bus1_N = "n:upref:bus1:N", "p.u., Pre-fault Voltage, Magnitude"
      n_uprefpc_bus1_A = "n:uprefpc:bus1:A", "%, Pre-fault Voltage, Magnitude"
      n_uprefpc_bus1_B = "n:uprefpc:bus1:B", "%, Pre-fault Voltage, Magnitude"
      n_uprefpc_bus1_C = "n:uprefpc:bus1:C", "%, Pre-fault Voltage, Magnitude"
      n_uprefpc_bus1_N = "n:uprefpc:bus1:N", "%, Pre-fault Voltage, Magnitude"
      n_Upref_bus1_A = "n:Upref:bus1:A", "kV, Line-Ground Pre-fault Voltage, Magnitude"
      n_Upref_bus1_B = "n:Upref:bus1:B", "kV, Line-Ground Pre-fault Voltage, Magnitude"
      n_Upref_bus1_C = "n:Upref:bus1:C", "kV, Line-Ground Pre-fault Voltage, Magnitude"
      n_Upref_bus1_N = "n:Upref:bus1:N", "kV, Line-Ground Pre-fault Voltage, Magnitude"
      n_Ulpref_bus1_A = "n:Ulpref:bus1:A", "kV, Line-Line Pre-fault Voltage, Magnitude"
      n_Ulpref_bus1_B = "n:Ulpref:bus1:B", "kV, Line-Line Pre-fault Voltage, Magnitude"
      n_Ulpref_bus1_C = "n:Ulpref:bus1:C", "kV, Line-Line Pre-fault Voltage, Magnitude"
      n_Ulpref_bus1_N = "n:Ulpref:bus1:N", "kV, Line-Line Pre-fault Voltage, Magnitude"
      n_phiupref_bus1_A = "n:phiupref:bus1:A", "deg, Pre-fault Voltage, Angle"
      n_phiupref_bus1_B = "n:phiupref:bus1:B", "deg, Pre-fault Voltage, Angle"
      n_phiupref_bus1_C = "n:phiupref:bus1:C", "deg, Pre-fault Voltage, Angle"
      n_phiupref_bus1_N = "n:phiupref:bus1:N", "deg, Pre-fault Voltage, Angle"
      n_phiui_bus1_A = "n:phiui:bus1:A", "deg, Angle between Voltage and Current"
      n_phiui_bus1_B = "n:phiui:bus1:B", "deg, Angle between Voltage and Current"
      n_phiui_bus1_C = "n:phiui:bus1:C", "deg, Angle between Voltage and Current"
      n_phiui_bus1_N = "n:phiui:bus1:N", "deg, Angle between Voltage and Current"
      n_phiu1i1_bus1 = "n:phiu1i1:bus1", "deg, Angle between Voltage and Current in positive sequence system"
      n_phiu2i2_bus1 = "n:phiu2i2:bus1", "deg, Angle between Voltage and Current in negative sequence system"
      n_phiu0i0_bus1 = "n:phiu0i0:bus1", "deg, Angle between Voltage and Current in zero-sequence system"
      c_y_a_r = "c:y_a:r", "p.u., Admittance, Phase a, Real Part"
      c_y_a_i = "c:y_a:i", "p.u., Admittance, Phase a, Imaginary Part"
      c_y_a_phi = "c:y_a:phi", "deg, Admittance, Phase a, Angle"
      c_y_a_mag = "c:y_a:mag", "p.u., Admittance, Phase a, Magnitude"
      c_y_a_phirad = "c:y_a:phirad", "rad, Admittance, Phase a, Angle"
      c_y_b_r = "c:y_b:r", "p.u., Admittance, Phase b, Real Part"
      c_y_b_i = "c:y_b:i", "p.u., Admittance, Phase b, Imaginary Part"
      c_y_b_phi = "c:y_b:phi", "deg, Admittance, Phase b, Angle"
      c_y_b_mag = "c:y_b:mag", "p.u., Admittance, Phase b, Magnitude"
      c_y_b_phirad = "c:y_b:phirad", "rad, Admittance, Phase b, Angle"
      c_y_c_r = "c:y_c:r", "p.u., Admittance, Phase c, Real Part"
      c_y_c_i = "c:y_c:i", "p.u., Admittance, Phase c, Imaginary Part"
      c_y_c_phi = "c:y_c:phi", "deg, Admittance, Phase c, Angle"
      c_y_c_mag = "c:y_c:mag", "p.u., Admittance, Phase c, Magnitude"
      c_y_c_phirad = "c:y_c:phirad", "rad, Admittance, Phase c, Angle"
      c_cn = "c:cn", "Connection Mode"
      c_Ra = "c:Ra", "p.u., Resistance, Phase a"
      c_La = "c:La", "p.u., Inductance, Phase a"
      c_Ca = "c:Ca", "p.u., Capacitance, Phase a"
      c_Ca_lv = "c:Ca_lv", "p.u., LV-Capacitance, Phase a"
      c_Rb = "c:Rb", "p.u., Resistance, Phase b"
      c_Lb = "c:Lb", "p.u., Inductance, Phase b"
      c_Cb = "c:Cb", "p.u., Capacitance, Phase b"
      c_Cb_lv = "c:Cb_lv", "p.u., LV-Capacitance, Phase b"
      c_Rc = "c:Rc", "p.u., Resistance, Phase c"
      c_Lc = "c:Lc", "p.u., Inductance, Phase c"
      c_Cc = "c:Cc", "p.u., Capacitance, Phase c"
      c_Cc_lv = "c:Cc_lv", "p.u., LV-Capacitance, Phase c"
      c_i_abc_r = "c:i_abc:r", "p.u., Harmonic currents, Real Part"
      c_i_abc_i = "c:i_abc:i", "p.u., Harmonic currents, Imaginary Part"
      c_i_abc_phi = "c:i_abc:phi", "deg, Harmonic currents, Angle"
      c_i_abc_mag = "c:i_abc:mag", "p.u., Harmonic currents, Magnitude"
      c_i_abc_phirad = "c:i_abc:phirad", "rad, Harmonic currents, Angle"
      c_i_abc_iec_r = "c:i_abc_iec:r", "p.u., Harmonic IEC current, Real Part"
      c_i_abc_iec_i = "c:i_abc_iec:i", "p.u., Harmonic IEC current, Imaginary Part"
      c_i_abc_iec_phi = "c:i_abc_iec:phi", "deg, Harmonic IEC current, Angle"
      c_i_abc_iec_mag = "c:i_abc_iec:mag", "p.u., Harmonic IEC current, Magnitude"
      c_i_abc_iec_phirad = "c:i_abc_iec:phirad", "rad, Harmonic IEC current, Angle"

    class ElmPvsys(Enum):
      _init_ = 'value __doc__'

      m_u0_bus1 = "m:u0:bus1", "p.u., Zero-Sequence-Voltage, Magnitude"
      m_u0_bus1n = "m:u0:bus1n", "p.u., Zero-Sequence-Voltage, Magnitude"
      m_phiu0_bus1 = "m:phiu0:bus1", "deg, Zero-Sequence-Voltage, Angle"
      m_phiu0_bus1n = "m:phiu0:bus1n", "deg, Zero-Sequence-Voltage, Angle"
      m_u1r_bus1 = "m:u1r:bus1", "p.u., Positive-Sequence-Voltage, Real Part"
      m_u1r_bus1n = "m:u1r:bus1n", "p.u., Positive-Sequence-Voltage, Real Part"
      m_u1i_bus1 = "m:u1i:bus1", "p.u., Positive-Sequence-Voltage, Imaginary Part"
      m_u1i_bus1n = "m:u1i:bus1n", "p.u., Positive-Sequence-Voltage, Imaginary Part"
      m_u1_bus1 = "m:u1:bus1", "p.u., Positive-Sequence-Voltage, Magnitude"
      m_u1_bus1n = "m:u1:bus1n", "p.u., Positive-Sequence-Voltage, Magnitude"
      m_U1_bus1 = "m:U1:bus1", "kV, Line-Ground Positive-Sequence-Voltage, Magnitude"
      m_U1_bus1n = "m:U1:bus1n", "kV, Line-Ground Positive-Sequence-Voltage, Magnitude"
      m_U1l_bus1 = "m:U1l:bus1", "kV, Line-Line Positive-Sequence-Voltage, Magnitude"
      m_U1l_bus1n = "m:U1l:bus1n", "kV, Line-Line Positive-Sequence-Voltage, Magnitude"
      m_phiu1_bus1 = "m:phiu1:bus1", "deg, Positive-Sequence-Voltage, Angle"
      m_phiu1_bus1n = "m:phiu1:bus1n", "deg, Positive-Sequence-Voltage, Angle"
      m_u2_bus1 = "m:u2:bus1", "p.u., Negative-Sequence-Voltage, Magnitude"
      m_u2_bus1n = "m:u2:bus1n", "p.u., Negative-Sequence-Voltage, Magnitude"
      m_phiu2_bus1 = "m:phiu2:bus1", "deg, Negative-Sequence-Voltage, Angle"
      m_phiu2_bus1n = "m:phiu2:bus1n", "deg, Negative-Sequence-Voltage, Angle"
      m_inet_bus1_A = "m:inet:bus1:A", "p.u., Phase Current, Magnitude, referred to network"
      m_inet_bus1_B = "m:inet:bus1:B", "p.u., Phase Current, Magnitude, referred to network"
      m_inet_bus1_C = "m:inet:bus1:C", "p.u., Phase Current, Magnitude, referred to network"
      m_inet_bus1_N = "m:inet:bus1:N", "p.u., Phase Current, Magnitude, referred to network"
      m_inet_bus1n = "m:inet:bus1n", "p.u., Phase Current, Magnitude, referred to network"
      m_I_bus1_A = "m:I:bus1:A", "kA, Phase Current, Magnitude"
      m_I_bus1_B = "m:I:bus1:B", "kA, Phase Current, Magnitude"
      m_I_bus1_C = "m:I:bus1:C", "kA, Phase Current, Magnitude"
      m_I_bus1_N = "m:I:bus1:N", "kA, Phase Current, Magnitude"
      m_I_bus1n = "m:I:bus1n", "kA, Phase Current, Magnitude"
      m_phii_bus1_A = "m:phii:bus1:A", "deg, Phase Current, Angle"
      m_phii_bus1_B = "m:phii:bus1:B", "deg, Phase Current, Angle"
      m_phii_bus1_C = "m:phii:bus1:C", "deg, Phase Current, Angle"
      m_phii_bus1_N = "m:phii:bus1:N", "deg, Phase Current, Angle"
      m_phii_bus1n = "m:phii:bus1n", "deg, Phase Current, Angle"
      m_i0r_bus1 = "m:i0r:bus1", "p.u., Zero-Sequence Current, Real Part"
      m_i0r_bus1n = "m:i0r:bus1n", "p.u., Zero-Sequence Current, Real Part"
      m_i0i_bus1 = "m:i0i:bus1", "p.u., Zero-Sequence Current, Imaginary Part"
      m_i0i_bus1n = "m:i0i:bus1n", "p.u., Zero-Sequence Current, Imaginary Part"
      m_i0_bus1 = "m:i0:bus1", "p.u., Zero-Sequence Current, Magnitude"
      m_i0_bus1n = "m:i0:bus1n", "p.u., Zero-Sequence Current, Magnitude"
      m_I0_bus1 = "m:I0:bus1", "kA, Zero-Sequence Current, Magnitude"
      m_I0_bus1n = "m:I0:bus1n", "kA, Zero-Sequence Current, Magnitude"
      m_I0x3_bus1 = "m:I0x3:bus1", "kA, 3*I0"
      m_I0x3_bus1n = "m:I0x3:bus1n", "kA, 3*I0"
      m_phii0_bus1 = "m:phii0:bus1", "deg, Zero-Sequence Current, Angle"
      m_phii0_bus1n = "m:phii0:bus1n", "deg, Zero-Sequence Current, Angle"
      m_i1r_bus1 = "m:i1r:bus1", "p.u., Positive-Sequence Current, Real Part"
      m_i1r_bus1n = "m:i1r:bus1n", "p.u., Positive-Sequence Current, Real Part"
      m_i1i_bus1 = "m:i1i:bus1", "p.u., Positive-Sequence Current, Imaginary Part"
      m_i1i_bus1n = "m:i1i:bus1n", "p.u., Positive-Sequence Current, Imaginary Part"
      m_i1_bus1 = "m:i1:bus1", "p.u., Positive-Sequence Current, Magnitude"
      m_i1_bus1n = "m:i1:bus1n", "p.u., Positive-Sequence Current, Magnitude"
      m_I1_bus1 = "m:I1:bus1", "kA, Positive-Sequence Current, Magnitude"
      m_I1_bus1n = "m:I1:bus1n", "kA, Positive-Sequence Current, Magnitude"
      m_phii1_bus1 = "m:phii1:bus1", "deg, Positive-Sequence Current, Angle"
      m_phii1_bus1n = "m:phii1:bus1n", "deg, Positive-Sequence Current, Angle"
      m_i2r_bus1 = "m:i2r:bus1", "p.u., Negative-Sequence Current, Real Part"
      m_i2r_bus1n = "m:i2r:bus1n", "p.u., Negative-Sequence Current, Real Part"
      m_i2i_bus1 = "m:i2i:bus1", "p.u., Negative-Sequence Current, Imaginary Part"
      m_i2i_bus1n = "m:i2i:bus1n", "p.u., Negative-Sequence Current, Imaginary Part"
      m_i2_bus1 = "m:i2:bus1", "p.u., Negative-Sequence Current, Magnitude"
      m_i2_bus1n = "m:i2:bus1n", "p.u., Negative-Sequence Current, Magnitude"
      m_I2_bus1 = "m:I2:bus1", "kA, Negative-Sequence Current, Magnitude"
      m_I2_bus1n = "m:I2:bus1n", "kA, Negative-Sequence Current, Magnitude"
      m_phii2_bus1 = "m:phii2:bus1", "deg, Negative-Sequence Current, Angle"
      m_phii2_bus1n = "m:phii2:bus1n", "deg, Negative-Sequence Current, Angle"
      m_ubfacI_bus1 = "m:ubfacI:bus1", "%, Current Unbalance Factor"
      m_ubfacI_bus1n = "m:ubfacI:bus1n", "%, Current Unbalance Factor"
      m_ubfacS_bus1 = "m:ubfacS:bus1", "%, Power Unbalance Factor"
      m_ubfacS_bus1n = "m:ubfacS:bus1n", "%, Power Unbalance Factor"
      m_P_bus1_A = "m:P:bus1:A", "MW, Active Power"
      m_P_bus1_B = "m:P:bus1:B", "MW, Active Power"
      m_P_bus1_C = "m:P:bus1:C", "MW, Active Power"
      m_P_bus1_N = "m:P:bus1:N", "MW, Active Power"
      m_P_bus1n = "m:P:bus1n", "MW, Active Power"
      m_Q_bus1_A = "m:Q:bus1:A", "Mvar, Reactive Power"
      m_Q_bus1_B = "m:Q:bus1:B", "Mvar, Reactive Power"
      m_Q_bus1_C = "m:Q:bus1:C", "Mvar, Reactive Power"
      m_Q_bus1_N = "m:Q:bus1:N", "Mvar, Reactive Power"
      m_Q_bus1n = "m:Q:bus1n", "Mvar, Reactive Power"
      m_S_bus1_A = "m:S:bus1:A", "MVA, Apparent Power"
      m_S_bus1_B = "m:S:bus1:B", "MVA, Apparent Power"
      m_S_bus1_C = "m:S:bus1:C", "MVA, Apparent Power"
      m_S_bus1_N = "m:S:bus1:N", "MVA, Apparent Power"
      m_S_bus1n = "m:S:bus1n", "MVA, Apparent Power"
      m_cosphi_bus1_A = "m:cosphi:bus1:A", "Power Factor"
      m_cosphi_bus1_B = "m:cosphi:bus1:B", "Power Factor"
      m_cosphi_bus1_C = "m:cosphi:bus1:C", "Power Factor"
      m_cosphi_bus1_N = "m:cosphi:bus1:N", "Power Factor"
      m_cosphi_bus1n = "m:cosphi:bus1n", "Power Factor"
      m_tanphi_bus1_A = "m:tanphi:bus1:A", "tan(phi)"
      m_tanphi_bus1_B = "m:tanphi:bus1:B", "tan(phi)"
      m_tanphi_bus1_C = "m:tanphi:bus1:C", "tan(phi)"
      m_tanphi_bus1_N = "m:tanphi:bus1:N", "tan(phi)"
      m_tanphi_bus1n = "m:tanphi:bus1n", "tan(phi)"
      m_Psum_bus1 = "m:Psum:bus1", "MW, Total Active Power"
      m_Psum_bus1n = "m:Psum:bus1n", "MW, Total Active Power"
      m_Qsum_bus1 = "m:Qsum:bus1", "Mvar, Total Reactive Power"
      m_Qsum_bus1n = "m:Qsum:bus1n", "Mvar, Total Reactive Power"
      m_Ssum_bus1 = "m:Ssum:bus1", "MVA, Total Apparent Power"
      m_Ssum_bus1n = "m:Ssum:bus1n", "MVA, Total Apparent Power"
      m_cosphisum_bus1 = "m:cosphisum:bus1", "Total Power Factor"
      m_cosphisum_bus1n = "m:cosphisum:bus1n", "Total Power Factor"
      m_tanphisum_bus1 = "m:tanphisum:bus1", "Total tan(phi)"
      m_tanphisum_bus1n = "m:tanphisum:bus1n", "Total tan(phi)"
      m_Pln_bus1_A = "m:Pln:bus1:A", "MW, Active Power (Line-Neutral)"
      m_Pln_bus1_B = "m:Pln:bus1:B", "MW, Active Power (Line-Neutral)"
      m_Pln_bus1_C = "m:Pln:bus1:C", "MW, Active Power (Line-Neutral)"
      m_Pln_bus1_N = "m:Pln:bus1:N", "MW, Active Power (Line-Neutral)"
      m_Pln_bus1n = "m:Pln:bus1n", "MW, Active Power (Line-Neutral)"
      m_Qln_bus1_A = "m:Qln:bus1:A", "Mvar, Reactive Power (Line-Neutral)"
      m_Qln_bus1_B = "m:Qln:bus1:B", "Mvar, Reactive Power (Line-Neutral)"
      m_Qln_bus1_C = "m:Qln:bus1:C", "Mvar, Reactive Power (Line-Neutral)"
      m_Qln_bus1_N = "m:Qln:bus1:N", "Mvar, Reactive Power (Line-Neutral)"
      m_Qln_bus1n = "m:Qln:bus1n", "Mvar, Reactive Power (Line-Neutral)"
      m_P1_bus1 = "m:P1:bus1", "MW, Positive Sequence Active Power"
      m_P1_bus1n = "m:P1:bus1n", "MW, Positive Sequence Active Power"
      m_Q1_bus1 = "m:Q1:bus1", "Mvar, Positive Sequence Reactive Power"
      m_Q1_bus1n = "m:Q1:bus1n", "Mvar, Positive Sequence Reactive Power"
      m_P2_bus1 = "m:P2:bus1", "MW, Negative Sequence Active Power"
      m_P2_bus1n = "m:P2:bus1n", "MW, Negative Sequence Active Power"
      m_Q2_bus1 = "m:Q2:bus1", "Mvar, Negative Sequence Reactive Power"
      m_Q2_bus1n = "m:Q2:bus1n", "Mvar, Negative Sequence Reactive Power"
      m_P0_bus1 = "m:P0:bus1", "MW, Zero Sequence Active Power"
      m_P0_bus1n = "m:P0:bus1n", "MW, Zero Sequence Active Power"
      m_Q0_bus1 = "m:Q0:bus1", "Mvar, Zero Sequence Reactive Power"
      m_Q0_bus1n = "m:Q0:bus1n", "Mvar, Zero Sequence Reactive Power"
      m_i1P_bus1 = "m:i1P:bus1", "p.u., Positive-Sequence Active Current"
      m_i1P_bus1n = "m:i1P:bus1n", "p.u., Positive-Sequence Active Current"
      m_i1Q_bus1 = "m:i1Q:bus1", "p.u., Positive-Sequence Reactive Current"
      m_i1Q_bus1n = "m:i1Q:bus1n", "p.u., Positive-Sequence Reactive Current"
      m_I1P_bus1 = "m:I1P:bus1", "kA, Positive-Sequence Active Current"
      m_I1P_bus1n = "m:I1P:bus1n", "kA, Positive-Sequence Active Current"
      m_I1Q_bus1 = "m:I1Q:bus1", "kA, Positive-Sequence Reactive Current"
      m_I1Q_bus1n = "m:I1Q:bus1n", "kA, Positive-Sequence Reactive Current"
      m_i2P_bus1 = "m:i2P:bus1", "p.u., Negative-Sequence Active Current"
      m_i2P_bus1n = "m:i2P:bus1n", "p.u., Negative-Sequence Active Current"
      m_i2Q_bus1 = "m:i2Q:bus1", "p.u., Negative-Sequence Reactive Current"
      m_i2Q_bus1n = "m:i2Q:bus1n", "p.u., Negative-Sequence Reactive Current"
      m_I2P_bus1 = "m:I2P:bus1", "kA, Negative-Sequence Active Current"
      m_I2P_bus1n = "m:I2P:bus1n", "kA, Negative-Sequence Active Current"
      m_I2Q_bus1 = "m:I2Q:bus1", "kA, Negative-Sequence Reactive Current"
      m_I2Q_bus1n = "m:I2Q:bus1n", "kA, Negative-Sequence Reactive Current"
      m_ur_bus1_A = "m:ur:bus1:A", "p.u., Phase Voltage, Real Part"
      m_ur_bus1_B = "m:ur:bus1:B", "p.u., Phase Voltage, Real Part"
      m_ur_bus1_C = "m:ur:bus1:C", "p.u., Phase Voltage, Real Part"
      m_ur_bus1_N = "m:ur:bus1:N", "p.u., Phase Voltage, Real Part"
      m_ur_bus1n = "m:ur:bus1n", "p.u., Phase Voltage, Real Part"
      m_ui_bus1_A = "m:ui:bus1:A", "p.u., Phase Voltage, Imaginary Part"
      m_ui_bus1_B = "m:ui:bus1:B", "p.u., Phase Voltage, Imaginary Part"
      m_ui_bus1_C = "m:ui:bus1:C", "p.u., Phase Voltage, Imaginary Part"
      m_ui_bus1_N = "m:ui:bus1:N", "p.u., Phase Voltage, Imaginary Part"
      m_ui_bus1n = "m:ui:bus1n", "p.u., Phase Voltage, Imaginary Part"
      m_u_bus1_A = "m:u:bus1:A", "p.u., Phase Voltage, Magnitude"
      m_u_bus1_B = "m:u:bus1:B", "p.u., Phase Voltage, Magnitude"
      m_u_bus1_C = "m:u:bus1:C", "p.u., Phase Voltage, Magnitude"
      m_u_bus1_N = "m:u:bus1:N", "p.u., Phase Voltage, Magnitude"
      m_u_bus1n = "m:u:bus1n", "p.u., Phase Voltage, Magnitude"
      m_ir_bus1_A = "m:ir:bus1:A", "p.u., Phase Current, Real Part"
      m_ir_bus1_B = "m:ir:bus1:B", "p.u., Phase Current, Real Part"
      m_ir_bus1_C = "m:ir:bus1:C", "p.u., Phase Current, Real Part"
      m_ir_bus1_N = "m:ir:bus1:N", "p.u., Phase Current, Real Part"
      m_ir_bus1n = "m:ir:bus1n", "p.u., Phase Current, Real Part"
      m_ii_bus1_A = "m:ii:bus1:A", "p.u., Phase Current, Imaginary Part"
      m_ii_bus1_B = "m:ii:bus1:B", "p.u., Phase Current, Imaginary Part"
      m_ii_bus1_C = "m:ii:bus1:C", "p.u., Phase Current, Imaginary Part"
      m_ii_bus1_N = "m:ii:bus1:N", "p.u., Phase Current, Imaginary Part"
      m_ii_bus1n = "m:ii:bus1n", "p.u., Phase Current, Imaginary Part"
      m_i_bus1_A = "m:i:bus1:A", "p.u., Phase Current, Magnitude"
      m_i_bus1_B = "m:i:bus1:B", "p.u., Phase Current, Magnitude"
      m_i_bus1_C = "m:i:bus1:C", "p.u., Phase Current, Magnitude"
      m_i_bus1_N = "m:i:bus1:N", "p.u., Phase Current, Magnitude"
      m_i_bus1n = "m:i:bus1n", "p.u., Phase Current, Magnitude"
      m_phiui_bus1_A = "m:phiui:bus1:A", "deg, Angle between Voltage and Current"
      m_phiui_bus1_B = "m:phiui:bus1:B", "deg, Angle between Voltage and Current"
      m_phiui_bus1_C = "m:phiui:bus1:C", "deg, Angle between Voltage and Current"
      m_phiui_bus1_N = "m:phiui:bus1:N", "deg, Angle between Voltage and Current"
      m_phiui_bus1n = "m:phiui:bus1n", "deg, Angle between Voltage and Current"
      m_phiu0i0_bus1 = "m:phiu0i0:bus1", "deg, Angle between Voltage and Current in zero-sequence system"
      m_phiu0i0_bus1n = "m:phiu0i0:bus1n", "deg, Angle between Voltage and Current in zero-sequence system"
      m_phiu1i1_bus1 = "m:phiu1i1:bus1", "deg, Angle between Voltage and Current in positive sequence system"
      m_phiu1i1_bus1n = "m:phiu1i1:bus1n", "deg, Angle between Voltage and Current in positive sequence system"
      m_phiu2i2_bus1 = "m:phiu2i2:bus1", "deg, Angle between Voltage and Current in negative sequence system"
      m_phiu2i2_bus1n = "m:phiu2i2:bus1n", "deg, Angle between Voltage and Current in negative sequence system"
      m_Tfct_bus1 = "m:Tfct:bus1", "s, Fault Clearing Time"
      m_Tfct_bus1n = "m:Tfct:bus1n", "s, Fault Clearing Time"
      m_TfctPh_bus1_A = "m:TfctPh:bus1:A", "s, Fault Clearing Time"
      m_TfctPh_bus1_B = "m:TfctPh:bus1:B", "s, Fault Clearing Time"
      m_TfctPh_bus1_C = "m:TfctPh:bus1:C", "s, Fault Clearing Time"
      m_TfctPh_bus1_N = "m:TfctPh:bus1:N", "s, Fault Clearing Time"
      m_TfctPh_bus1n = "m:TfctPh:bus1n", "s, Fault Clearing Time"
      m_Brkload_bus1 = "m:Brkload:bus1", "%, Max. Breaker Loading"
      m_Brkload_bus1n = "m:Brkload:bus1n", "%, Max. Breaker Loading"
      m_BrkloadPh_bus1_A = "m:BrkloadPh:bus1:A", "%, Breaker Loading"
      m_BrkloadPh_bus1_B = "m:BrkloadPh:bus1:B", "%, Breaker Loading"
      m_BrkloadPh_bus1_C = "m:BrkloadPh:bus1:C", "%, Breaker Loading"
      m_BrkloadPh_bus1_N = "m:BrkloadPh:bus1:N", "%, Breaker Loading"
      m_BrkloadPh_bus1n = "m:BrkloadPh:bus1n", "%, Breaker Loading"
      n_ur_bus1_A = "n:ur:bus1:A", "p.u., Line-Ground Voltage, Real Part"
      n_ur_bus1_B = "n:ur:bus1:B", "p.u., Line-Ground Voltage, Real Part"
      n_ur_bus1_C = "n:ur:bus1:C", "p.u., Line-Ground Voltage, Real Part"
      n_ur_bus1_N = "n:ur:bus1:N", "p.u., Line-Ground Voltage, Real Part"
      n_ur_bus1n = "n:ur:bus1n", "p.u., Line-Ground Voltage, Real Part"
      n_ui_bus1_A = "n:ui:bus1:A", "p.u., Line-Ground Voltage, Imaginary Part"
      n_ui_bus1_B = "n:ui:bus1:B", "p.u., Line-Ground Voltage, Imaginary Part"
      n_ui_bus1_C = "n:ui:bus1:C", "p.u., Line-Ground Voltage, Imaginary Part"
      n_ui_bus1_N = "n:ui:bus1:N", "p.u., Line-Ground Voltage, Imaginary Part"
      n_ui_bus1n = "n:ui:bus1n", "p.u., Line-Ground Voltage, Imaginary Part"
      n_u_bus1_A = "n:u:bus1:A", "p.u., Line-Ground Voltage, Magnitude"
      n_u_bus1_B = "n:u:bus1:B", "p.u., Line-Ground Voltage, Magnitude"
      n_u_bus1_C = "n:u:bus1:C", "p.u., Line-Ground Voltage, Magnitude"
      n_u_bus1_N = "n:u:bus1:N", "p.u., Line-Ground Voltage, Magnitude"
      n_u_bus1n = "n:u:bus1n", "p.u., Line-Ground Voltage, Magnitude"
      n_upc_bus1_A = "n:upc:bus1:A", "%, Line-Ground Voltage, Magnitude"
      n_upc_bus1_B = "n:upc:bus1:B", "%, Line-Ground Voltage, Magnitude"
      n_upc_bus1_C = "n:upc:bus1:C", "%, Line-Ground Voltage, Magnitude"
      n_upc_bus1_N = "n:upc:bus1:N", "%, Line-Ground Voltage, Magnitude"
      n_upc_bus1n = "n:upc:bus1n", "%, Line-Ground Voltage, Magnitude"
      n_U_bus1_A = "n:U:bus1:A", "kV, Line-Ground Voltage, Magnitude"
      n_U_bus1_B = "n:U:bus1:B", "kV, Line-Ground Voltage, Magnitude"
      n_U_bus1_C = "n:U:bus1:C", "kV, Line-Ground Voltage, Magnitude"
      n_U_bus1_N = "n:U:bus1:N", "kV, Line-Ground Voltage, Magnitude"
      n_U_bus1n = "n:U:bus1n", "kV, Line-Ground Voltage, Magnitude"
      n_phiu_bus1_A = "n:phiu:bus1:A", "deg, Line-Ground Voltage, Angle"
      n_phiu_bus1_B = "n:phiu:bus1:B", "deg, Line-Ground Voltage, Angle"
      n_phiu_bus1_C = "n:phiu:bus1:C", "deg, Line-Ground Voltage, Angle"
      n_phiu_bus1_N = "n:phiu:bus1:N", "deg, Line-Ground Voltage, Angle"
      n_phiu_bus1n = "n:phiu:bus1n", "deg, Line-Ground Voltage, Angle"
      n_phiurel_bus1 = "n:phiurel:bus1", "deg, Voltage, Relative Angle"
      n_phiurel_bus1n = "n:phiurel:bus1n", "deg, Voltage, Relative Angle"
      n_ul_bus1_A = "n:ul:bus1:A", "p.u., Line-Line Voltage, Magnitude"
      n_ul_bus1_B = "n:ul:bus1:B", "p.u., Line-Line Voltage, Magnitude"
      n_ul_bus1_C = "n:ul:bus1:C", "p.u., Line-Line Voltage, Magnitude"
      n_ul_bus1_N = "n:ul:bus1:N", "p.u., Line-Line Voltage, Magnitude"
      n_ul_bus1n = "n:ul:bus1n", "p.u., Line-Line Voltage, Magnitude"
      n_ulpc_bus1_A = "n:ulpc:bus1:A", "%, Line-Line Voltage, Magnitude"
      n_ulpc_bus1_B = "n:ulpc:bus1:B", "%, Line-Line Voltage, Magnitude"
      n_ulpc_bus1_C = "n:ulpc:bus1:C", "%, Line-Line Voltage, Magnitude"
      n_ulpc_bus1_N = "n:ulpc:bus1:N", "%, Line-Line Voltage, Magnitude"
      n_ulpc_bus1n = "n:ulpc:bus1n", "%, Line-Line Voltage, Magnitude"
      n_Ul_bus1_A = "n:Ul:bus1:A", "kV, Line-Line Voltage, Magnitude"
      n_Ul_bus1_B = "n:Ul:bus1:B", "kV, Line-Line Voltage, Magnitude"
      n_Ul_bus1_C = "n:Ul:bus1:C", "kV, Line-Line Voltage, Magnitude"
      n_Ul_bus1_N = "n:Ul:bus1:N", "kV, Line-Line Voltage, Magnitude"
      n_Ul_bus1n = "n:Ul:bus1n", "kV, Line-Line Voltage, Magnitude"
      n_phiul_bus1_A = "n:phiul:bus1:A", "deg, Line-Line Voltage, Angle"
      n_phiul_bus1_B = "n:phiul:bus1:B", "deg, Line-Line Voltage, Angle"
      n_phiul_bus1_C = "n:phiul:bus1:C", "deg, Line-Line Voltage, Angle"
      n_phiul_bus1_N = "n:phiul:bus1:N", "deg, Line-Line Voltage, Angle"
      n_phiul_bus1n = "n:phiul:bus1n", "deg, Line-Line Voltage, Angle"
      n_u0_bus1 = "n:u0:bus1", "p.u., Zero-Sequence Voltage, Magnitude"
      n_u0_bus1n = "n:u0:bus1n", "p.u., Zero-Sequence Voltage, Magnitude"
      n_U0_bus1 = "n:U0:bus1", "kV, Zero-Sequence Voltage, Magnitude"
      n_U0_bus1n = "n:U0:bus1n", "kV, Zero-Sequence Voltage, Magnitude"
      n_U0x3_bus1 = "n:U0x3:bus1", "kV, 3*U0"
      n_U0x3_bus1n = "n:U0x3:bus1n", "kV, 3*U0"
      n_phiu0_bus1 = "n:phiu0:bus1", "deg, Zero-Sequence Voltage, Angle"
      n_phiu0_bus1n = "n:phiu0:bus1n", "deg, Zero-Sequence Voltage, Angle"
      n_u1_bus1 = "n:u1:bus1", "p.u., Positive-Sequence Voltage, Magnitude"
      n_u1_bus1n = "n:u1:bus1n", "p.u., Positive-Sequence Voltage, Magnitude"
      n_u1pc_bus1 = "n:u1pc:bus1", "%, Positive-Sequence Voltage, Magnitude"
      n_u1pc_bus1n = "n:u1pc:bus1n", "%, Positive-Sequence Voltage, Magnitude"
      n_u1r_bus1 = "n:u1r:bus1", "p.u., Positive-Sequence Voltage, Real Part"
      n_u1r_bus1n = "n:u1r:bus1n", "p.u., Positive-Sequence Voltage, Real Part"
      n_u1i_bus1 = "n:u1i:bus1", "p.u., Positive-Sequence Voltage, Imaginary Part"
      n_u1i_bus1n = "n:u1i:bus1n", "p.u., Positive-Sequence Voltage, Imaginary Part"
      n_U1_bus1 = "n:U1:bus1", "kV, Line-Ground Positive-Sequence Voltage, Magnitude"
      n_U1_bus1n = "n:U1:bus1n", "kV, Line-Ground Positive-Sequence Voltage, Magnitude"
      n_phiu1_bus1 = "n:phiu1:bus1", "deg, Positive-Sequence Voltage, Angle"
      n_phiu1_bus1n = "n:phiu1:bus1n", "deg, Positive-Sequence Voltage, Angle"
      n_u2_bus1 = "n:u2:bus1", "p.u., Negative-Sequence Voltage, Magnitude"
      n_u2_bus1n = "n:u2:bus1n", "p.u., Negative-Sequence Voltage, Magnitude"
      n_U2_bus1 = "n:U2:bus1", "kV, Line-Ground Negative-Sequence Voltage, Magnitude"
      n_U2_bus1n = "n:U2:bus1n", "kV, Line-Ground Negative-Sequence Voltage, Magnitude"
      n_phiu2_bus1 = "n:phiu2:bus1", "deg, Negative-Sequence Voltage, Angle"
      n_phiu2_bus1n = "n:phiu2:bus1n", "deg, Negative-Sequence Voltage, Angle"
      n_U1l_bus1 = "n:U1l:bus1", "kV, Line-Line Positive-Sequence Voltage, Magnitude"
      n_U1l_bus1n = "n:U1l:bus1n", "kV, Line-Line Positive-Sequence Voltage, Magnitude"
      n_U2l_bus1 = "n:U2l:bus1", "kV, Line-Line Negative-Sequence Voltage, Magnitude"
      n_U2l_bus1n = "n:U2l:bus1n", "kV, Line-Line Negative-Sequence Voltage, Magnitude"
      n_uln_bus1_A = "n:uln:bus1:A", "p.u., Line-Neutral Voltage, Magnitude"
      n_uln_bus1_B = "n:uln:bus1:B", "p.u., Line-Neutral Voltage, Magnitude"
      n_uln_bus1_C = "n:uln:bus1:C", "p.u., Line-Neutral Voltage, Magnitude"
      n_uln_bus1_N = "n:uln:bus1:N", "p.u., Line-Neutral Voltage, Magnitude"
      n_uln_bus1n = "n:uln:bus1n", "p.u., Line-Neutral Voltage, Magnitude"
      n_Uln_bus1_A = "n:Uln:bus1:A", "kV, Line-Neutral Voltage, Magnitude"
      n_Uln_bus1_B = "n:Uln:bus1:B", "kV, Line-Neutral Voltage, Magnitude"
      n_Uln_bus1_C = "n:Uln:bus1:C", "kV, Line-Neutral Voltage, Magnitude"
      n_Uln_bus1_N = "n:Uln:bus1:N", "kV, Line-Neutral Voltage, Magnitude"
      n_Uln_bus1n = "n:Uln:bus1n", "kV, Line-Neutral Voltage, Magnitude"
      n_phiuln_bus1_A = "n:phiuln:bus1:A", "deg, Line-Neutral Voltage, Angle"
      n_phiuln_bus1_B = "n:phiuln:bus1:B", "deg, Line-Neutral Voltage, Angle"
      n_phiuln_bus1_C = "n:phiuln:bus1:C", "deg, Line-Neutral Voltage, Angle"
      n_phiuln_bus1_N = "n:phiuln:bus1:N", "deg, Line-Neutral Voltage, Angle"
      n_phiuln_bus1n = "n:phiuln:bus1n", "deg, Line-Neutral Voltage, Angle"
      n_un_bus1 = "n:un:bus1", "p.u., Neutral-Ground Voltage, Magnitude"
      n_un_bus1n = "n:un:bus1n", "p.u., Neutral-Ground Voltage, Magnitude"
      n_Un_bus1 = "n:Un:bus1", "kV, Neutral-Ground Voltage, Magnitude"
      n_Un_bus1n = "n:Un:bus1n", "kV, Neutral-Ground Voltage, Magnitude"
      n_phiun_bus1 = "n:phiun:bus1", "deg, Neutral-Ground Voltage, Angle"
      n_phiun_bus1n = "n:phiun:bus1n", "deg, Neutral-Ground Voltage, Angle"
      n_upht_bus1_A = "n:upht:bus1:A", "p.u., Phase Technology dependent Voltage, Magnitude"
      n_upht_bus1_B = "n:upht:bus1:B", "p.u., Phase Technology dependent Voltage, Magnitude"
      n_upht_bus1_C = "n:upht:bus1:C", "p.u., Phase Technology dependent Voltage, Magnitude"
      n_upht_bus1_N = "n:upht:bus1:N", "p.u., Phase Technology dependent Voltage, Magnitude"
      n_upht_bus1n = "n:upht:bus1n", "p.u., Phase Technology dependent Voltage, Magnitude"
      n_Upht_bus1_A = "n:Upht:bus1:A", "kV, Phase Technology dependent Voltage, Magnitude"
      n_Upht_bus1_B = "n:Upht:bus1:B", "kV, Phase Technology dependent Voltage, Magnitude"
      n_Upht_bus1_C = "n:Upht:bus1:C", "kV, Phase Technology dependent Voltage, Magnitude"
      n_Upht_bus1_N = "n:Upht:bus1:N", "kV, Phase Technology dependent Voltage, Magnitude"
      n_Upht_bus1n = "n:Upht:bus1n", "kV, Phase Technology dependent Voltage, Magnitude"
      n_um_bus1 = "n:um:bus1", "p.u., Average-Voltage, Magnitude"
      n_um_bus1n = "n:um:bus1n", "p.u., Average-Voltage, Magnitude"
      n_Um_bus1 = "n:Um:bus1", "kV, Average-Voltage, Magnitude"
      n_Um_bus1n = "n:Um:bus1n", "kV, Average-Voltage, Magnitude"
      n_uphtmin_bus1 = "n:uphtmin:bus1", "p.u., Minimum of Phase Technology dependent Voltage, Magnitude"
      n_uphtmin_bus1n = "n:uphtmin:bus1n", "p.u., Minimum of Phase Technology dependent Voltage, Magnitude"
      n_uphtmax_bus1 = "n:uphtmax:bus1", "p.u., Maximum of Phase Technology dependent Voltage, Magnitude"
      n_uphtmax_bus1n = "n:uphtmax:bus1n", "p.u., Maximum of Phase Technology dependent Voltage, Magnitude"
      n_Pgen_bus1 = "n:Pgen:bus1", "MW, Generation, Active Power"
      n_Pgen_bus1n = "n:Pgen:bus1n", "MW, Generation, Active Power"
      n_Qgen_bus1 = "n:Qgen:bus1", "Mvar, Generation, Reactive Power"
      n_Qgen_bus1n = "n:Qgen:bus1n", "Mvar, Generation, Reactive Power"
      n_Pmot_bus1 = "n:Pmot:bus1", "MW, Motor Load, Active Power"
      n_Pmot_bus1n = "n:Pmot:bus1n", "MW, Motor Load, Active Power"
      n_Qmot_bus1 = "n:Qmot:bus1", "Mvar, Motor Load, Reactive Power"
      n_Qmot_bus1n = "n:Qmot:bus1n", "Mvar, Motor Load, Reactive Power"
      n_Pload_bus1 = "n:Pload:bus1", "MW, General Load, Active Power"
      n_Pload_bus1n = "n:Pload:bus1n", "MW, General Load, Active Power"
      n_Qload_bus1 = "n:Qload:bus1", "Mvar, General Load, Reactive Power"
      n_Qload_bus1n = "n:Qload:bus1n", "Mvar, General Load, Reactive Power"
      n_Pcomp_bus1 = "n:Pcomp:bus1", "MW, Compensation (Losses)"
      n_Pcomp_bus1n = "n:Pcomp:bus1n", "MW, Compensation (Losses)"
      n_Qcomp_bus1 = "n:Qcomp:bus1", "Mvar, Compensation"
      n_Qcomp_bus1n = "n:Qcomp:bus1n", "Mvar, Compensation"
      n_Pnet_bus1 = "n:Pnet:bus1", "MW, External Networks, Active Power"
      n_Pnet_bus1n = "n:Pnet:bus1n", "MW, External Networks, Active Power"
      n_Qnet_bus1 = "n:Qnet:bus1", "Mvar, External Networks, Reactive Power"
      n_Qnet_bus1n = "n:Qnet:bus1n", "Mvar, External Networks, Reactive Power"
      n_Pflow_bus1 = "n:Pflow:bus1", "MW, Power Flow, Active Power"
      n_Pflow_bus1n = "n:Pflow:bus1n", "MW, Power Flow, Active Power"
      n_Qflow_bus1 = "n:Qflow:bus1", "Mvar, Power Flow, Reactive Power"
      n_Qflow_bus1n = "n:Qflow:bus1n", "Mvar, Power Flow, Reactive Power"
      n_Pout_bus1 = "n:Pout:bus1", "MW, Outgoing Flow, Active Power"
      n_Pout_bus1n = "n:Pout:bus1n", "MW, Outgoing Flow, Active Power"
      n_Qout_bus1 = "n:Qout:bus1", "Mvar, Outgoing Power, Reactive Power"
      n_Qout_bus1n = "n:Qout:bus1n", "Mvar, Outgoing Power, Reactive Power"
      n_Sout_bus1 = "n:Sout:bus1", "MVA, Outgoing Power, Apparent Power"
      n_Sout_bus1n = "n:Sout:bus1n", "MVA, Outgoing Power, Apparent Power"
      n_cosphiout_bus1 = "n:cosphiout:bus1", "Outgoing Power, Power Factor"
      n_cosphiout_bus1n = "n:cosphiout:bus1n", "Outgoing Power, Power Factor"
      n_Pbalance_bus1 = "n:Pbalance:bus1", "MW, Active Power Balance (=0)"
      n_Pbalance_bus1n = "n:Pbalance:bus1n", "MW, Active Power Balance (=0)"
      n_Qbalance_bus1 = "n:Qbalance:bus1", "Mvar, Reactive Power Balance (=0)"
      n_Qbalance_bus1n = "n:Qbalance:bus1n", "Mvar, Reactive Power Balance (=0)"
      n_ubfac_bus1 = "n:ubfac:bus1", "%, Unbalance Factor"
      n_ubfac_bus1n = "n:ubfac:bus1n", "%, Unbalance Factor"
      n_LossPdown_bus1 = "n:LossPdown:bus1", "MW, Losses, downstream"
      n_LossPdown_bus1n = "n:LossPdown:bus1n", "MW, Losses, downstream"
      n_LossQdown_bus1 = "n:LossQdown:bus1", "Mvar, Losses, downstream (Reactive Power)"
      n_LossQdown_bus1n = "n:LossQdown:bus1n", "Mvar, Losses, downstream (Reactive Power)"
      n_LossPdownload_bus1 = "n:LossPdownload:bus1", "MW, Load losses, downstream"
      n_LossPdownload_bus1n = "n:LossPdownload:bus1n", "MW, Load losses, downstream"
      n_LossQdownload_bus1 = "n:LossQdownload:bus1", "Mvar, Load losses, downstream"
      n_LossQdownload_bus1n = "n:LossQdownload:bus1n", "Mvar, Load losses, downstream"
      n_LossPdownnoload_bus1 = "n:LossPdownnoload:bus1", "MW, No load losses, downstream"
      n_LossPdownnoload_bus1n = "n:LossPdownnoload:bus1n", "MW, No load losses, downstream"
      n_LossQdownnoload_bus1 = "n:LossQdownnoload:bus1", "Mvar, No load losses, downstream"
      n_LossQdownnoload_bus1n = "n:LossQdownnoload:bus1n", "Mvar, No load losses, downstream"
      s_pset = "s:pset", "MW, IN, Active Power Setpoint, continuous, double"
      s_qset = "s:qset", "Mvar, IN, Reactive Power Setpoint continuous, double"
      s_dFin = "s:dFin", "Hz, IN, Frequency Deviation, continuous, double"
      s_i2r_set = "s:i2r_set", "p.u., IN, Negative sequence current setpoint, Real Part, continuous, double"
      s_i2i_set = "s:i2i_set", "p.u., IN, Negative sequence current setpoint, Imaginary Part, continuous, double"
      c_p = "c:p", "MW, Total Active Power"
      c_q = "c:q", "Mvar, Total Reactive Power"
      c_sgnn = "c:sgnn", "MVA, Total rated apparent power (ngnum*sgn)"
      c_cosn = "c:cosn", "Nominal-Power factor"
      c_Pnom = "c:Pnom", "MW, Total Nominal Active power"
      c_c2 = "c:c2", "Voltage Coefficient (uknom/ugn)"
      c_uini_r = "c:uini:r", "p.u., Voltage Setpoint for PV and Slack, Real Part"
      c_uini_i = "c:uini:i", "p.u., Voltage Setpoint for PV and Slack, Imaginary Part"
      c_uini_phi = "c:uini:phi", "deg, Voltage Setpoint for PV and Slack, Angle"
      c_uini_mag = "c:uini:mag", "p.u., Voltage Setpoint for PV and Slack, Magnitude"
      c_uini_phirad = "c:uini:phirad", "rad, Voltage Setpoint for PV and Slack, Angle"
      c_phiini = "c:phiini", "Initial Angle"
      c_loading = "c:loading", "%, Loading"
      c_iPowerOn = "c:iPowerOn", "Flag"
      c_Qmin = "c:Qmin", "Mvar, Lower limit of reactive power"
      c_Qmax = "c:Qmax", "Mvar, Upper limit of reactive power"
      c_Qsparelead = "c:Qsparelead", "Mvar, Q Spare Lead"
      c_Qsparelag = "c:Qsparelag", "Mvar, Q Spare Lag"
      c_i_qlim = "c:i_qlim", "Reactive power limit reached"
      c_i_vlim = "c:i_vlim", "Voltage setpoint limit reached"
      c_i_plim = "c:i_plim", "Active power limit reached"
      c_sm_r = "c:sm:r", "MW, Complex Power, Real Part"
      c_sm_i = "c:sm:i", "Mvar, Complex Power, Imaginary Part"
      c_sm_phi = "c:sm:phi", "deg, Complex Power, Angle"
      c_sm_mag = "c:sm:mag", "MVA, Complex Power, Magnitude"
      c_sm_phirad = "c:sm:phirad", "rad, Complex Power, Angle"
      c_u1_r = "c:u1:r", "p.u., Internal, Real Part"
      c_u1_i = "c:u1:i", "p.u., Internal, Imaginary Part"
      c_u1_phi = "c:u1:phi", "deg, Internal, Angle"
      c_u1_mag = "c:u1:mag", "p.u., Internal, Magnitude"
      c_u1_phirad = "c:u1:phirad", "rad, Internal, Angle"
      c_Kp = "c:Kp", "p.u., Active Power Participation"
      c_Kq = "c:Kq", "p.u., Reactive Power Participation"
      c_Kpf = "c:Kpf", "MW/Hz, Frequency Bias"
      c_iRefElement = "c:iRefElement", "Reference Element"
      c_zonewscale = "c:zonewscale", "Wind generation zone scaling factor"
      c_Pmism = "c:Pmism", "MW, Active Power Mismatch"
      c_nCtrlIter = "c:nCtrlIter", "No. of CTRL-iterations"
      c_Ploss = "c:Ploss", "MW, Losses (total)"
      c_Qloss = "c:Qloss", "Mvar, Reactive-Losses (total)"
      c_Plossld = "c:Plossld", "MW, Losses (load)"
      c_Qlossld = "c:Qlossld", "Mvar, Reactive-Losses (load)"
      c_Plossnld = "c:Plossnld", "MW, Losses (no load)"
      c_Qlossnld = "c:Qlossnld", "Mvar, Reactive-Losses (no load)"
      e_ip_ctrl = "e:ip_ctrl", "Reference Machine"
      e_outServPzero = "e:outServPzero", "Out of service when active power is zero"
      e_av_mode = "e:av_mode", "Local Controller"
      e_c_psecc = "e:c_psecc", "ElmSecctrl, External Secondary Controller"
      e_c_pstac = "e:c_pstac", "ElmStactrl, External Station Controller"
      e_qdslCtrl = "e:qdslCtrl", "Quasi-Dynamic Model"
      e_pgini = "e:pgini", "kW, Active Power"
      e_Kpf = "e:Kpf", "kW/Hz, Prim. Frequency Bias"
      e_mode_inp = "e:mode_inp", "Input Mode"
      e_qgini = "e:qgini", "kvar, Reactive Power"
      e_cosgini = "e:cosgini", "Power Factor"
      e_pf_recap = "e:pf_recap", "Power Factor"
      e_usetp = "e:usetp", "p.u., Voltage"
      e_phiini = "e:phiini", "deg, Angle"
      e_scale0 = "e:scale0", "Scaling Factor"
      e_pgini_a = "e:pgini_a", "kW, Active Power (act.)"
      e_qgini_a = "e:qgini_a", "kvar, Reactive Power (act.)"
      e_sgini_a = "e:sgini_a", "kVA, Apparent Power (act.)"
      e_cosgini_a = "e:cosgini_a", "Power Factor (act.)"
      e_pf_recap_a = "e:pf_recap_a", "Power Factor Ind/Cap (act.)"
      e_scale0_a = "e:scale0_a", "Scaling Factor(act.)"
      e_usp_min = "e:usp_min", "p.u., Min. Voltage Setpoint"
      e_usp_max = "e:usp_max", "p.u., Max. Voltage Setpoint"
      e_ddroop = "e:ddroop", "%, Droop"
      e_pQPcurve = "e:pQPcurve", "IntQpcurve, Q(P)-Curve"
      e_iopt_tdr = "e:iopt_tdr", "Different droop values"
      e_ddroopoe = "e:ddroopoe", "%, Droop (overexcited)"
      e_ddroopue = "e:ddroopue", "%, Droop (underexcited)"
      e_Qfu_max = "e:Qfu_max", "kvar, Q max"
      e_Qfu_min = "e:Qfu_min", "kvar, Q min"
      e_udeadblow = "e:udeadblow", "p.u., Lower Voltage Limit"
      e_udeadbup = "e:udeadbup", "p.u., Upper Voltage Limit"
      e_pf_over = "e:pf_over", "Min. Power Factor"
      e_p_over = "e:p_over", "kW, Active Power"
      e_pf_under = "e:pf_under", "Min. Power Factor"
      e_p_under = "e:p_under", "kW, Active Power"
      e_c_pCtrlHV = "e:c_pCtrlHV", "Controlled HV-busbar"
      e_Uctrl = "e:Uctrl", "kV, HV-Voltage Setpoint"
      e_pQlimType = "e:pQlimType", "IntQlim, Capability Curve"
      e_q_min = "e:q_min", "p.u., Min."
      e_q_max = "e:q_max", "p.u., Max."
      e_cQ_min = "e:cQ_min", "kvar, Minimum"
      e_cQ_max = "e:cQ_max", "kvar, Maximum"
      e_scaleQmin = "e:scaleQmin", "%, Scaling Factor (min.)"
      e_scaleQmax = "e:scaleQmax", "%, Scaling Factor (max.)"
      e_Pmin_uc = "e:Pmin_uc", "kW, Min."
      e_Pmin_ucPU = "e:Pmin_ucPU", "p.u., Min."
      e_Pmax_uc = "e:Pmax_uc", "kW, Max."
      e_Pmax_ucPU = "e:Pmax_ucPU", "p.u., Max."
      e_Pnom = "e:Pnom", "kW, Pr(rated)"
      e_P_max = "e:P_max", "kW, Max."
      e_pmaxratf = "e:pmaxratf", "Rating Factor"
      e_Qmin_a = "e:Qmin_a", "kvar, Qmin(act.)"
      e_Qmax_a = "e:Qmax_a", "kvar, Qmax(act.)"
      e_Pmin_a = "e:Pmin_a", "kW, Pmin(act.)"
      e_Pmax_a = "e:Pmax_a", "kW, Pmax(act.)"
      e_cGPSlat = "e:cGPSlat", "deg, Latitude"
      e_cGPSlon = "e:cGPSlon", "deg, Longitude"
      e_iopt_rad = "e:iopt_rad", "Specified Components"
      e_iopt_glo = "e:iopt_glo", "Global Irradiance Data"
      e_ghi = "e:ghi", "W/m^2, Global Horizontal Irradiance"
      e_kt = "e:kt", "Clearness Index"
      e_iopt_dif = "e:iopt_dif", "Diffuse Irradiance Data"
      e_iopt_dir = "e:iopt_dir", "Direct Irradiance Data"
      e_dhi = "e:dhi", "W/m^2, Direct Horizontal Irradiance"
      e_dni = "e:dni", "W/m^2, Direct Normal Irradiance"
      e_ghi_a = "e:ghi_a", "W/m^2, Global Horizontal Irradiance (act.)"
      e_kt_a = "e:kt_a", "Clearness Index (act.)"
      e_dhi_a = "e:dhi_a", "W/m^2, Direct Horizontal Irradiance (act.)"
      e_dni_a = "e:dni_a", "W/m^2, Direct Normal Irradiance (act.)"
      e_Tamb = "e:Tamb", "degC, Ambient Temperature"
      e_shfdir = "e:shfdir", "Shading Factor (Direct)"
      e_shfdiff = "e:shfdiff", "Shading Factor (Diffuse)"
      e_albedo = "e:albedo", "Ground Albedo"
      e_Tamb_a = "e:Tamb_a", "degC, Amb. Temp. (act.)"
      e_shfdir_a = "e:shfdir_a", "Shading Factor (Dir.) (act.)"
      e_shfdiff_a = "e:shfdiff_a", "Shading Factor (Diff.) (act.)"
      e_albedo_a = "e:albedo_a", "Albedo (act.)"
      e_Egh_a = "e:Egh_a", "W/m^2, Global Horizontal Irradiance (act.)"
      e_Ebh_a = "e:Ebh_a", "W/m^2, Direct Horizontal Irradiance (act.)"
      e_Edh_a = "e:Edh_a", "W/m^2, Diffuse Horizontal Irradiance (act.)"
      e_Egpv_a = "e:Egpv_a", "W/m^2, Global Irradiance, PV Panel (act.)"
      e_Ebpv_a = "e:Ebpv_a", "W/m^2, Direct Irradiance, PV Panel (act.)"
      e_Edpv_a = "e:Edpv_a", "W/m^2, Diffuse Irradiance, PV Panel (act.)"
      e_Erpv_a = "e:Erpv_a", "W/m^2, Reflected Irradiance, PV Panel (act.)"
      e_RelEff_a = "e:RelEff_a", "Relative Efficiency (act.)"
      e_ktc_a = "e:ktc_a", "W/m^2, Calculated Clearness Index"
      e_solalt_a = "e:solalt_a", "deg, Solar Altitude Angle (act.)"
      e_anginc_a = "e:anginc_a", "deg, Angle of Incidence (act.)"
      e_orient_a = "e:orient_a", "deg, Orientation Angle (act.)"
      e_tilt_a = "e:tilt_a", "deg, Tilt Angle (act.)"
      e_r2 = "e:r2", "p.u., Resistance r2"
      e_x2 = "e:x2", "p.u., Reactance x2"
      e_is4SpinReserve = "e:is4SpinReserve", "Consider for region spinning reserve"
      e_genShiftKey = "e:genShiftKey", "%, Generation shift key"
      e_pBMU = "e:pBMU", "ElmBmu, Virtual Power Plant"
      e_dispatch = "e:dispatch", "Generator Dispatch"
      e_priority = "e:priority", "Merit Order"
      e_fixed = "e:fixed", "Must run"
      r_c_psecc = "r:c_psecc", "ElmSecctrl, External Secondary Controller"
      r_c_pstac = "r:c_pstac", "ElmStactrl, External Station Controller"
      r_qdslCtrl = "r:qdslCtrl", "Quasi-Dynamic Model"
      r_pQPcurve = "r:pQPcurve", "IntQpcurve, Q(P)-Curve"
      r_c_pCtrlHV = "r:c_pCtrlHV", "Controlled HV-busbar"
      r_pQlimType = "r:pQlimType", "IntQlim, Capability Curve"
      r_pBMU = "r:pBMU", "ElmBmu, Virtual Power Plant"
      m_Ikss_bus1 = "m:Ikss:bus1", "kA, Initial Short-Circuit Current"
      m_ikss_bus1 = "m:ikss:bus1", "p.u., Initial Short-Circuit Current, Magnitude"
      m_phii_bus1 = "m:phii:bus1", "deg, Phase Current, Angle"
      m_maxIkss_bus1 = "m:maxIkss:bus1", "kA, Max. Initial Short-Circuit Current"
      m_Skss_bus1 = "m:Skss:bus1", "MVA, Initial Short-Circuit Power"
      m_ip_bus1 = "m:ip:bus1", "kA, Peak Short-Circuit Current"
      m_maxip_bus1 = "m:maxip:bus1", "kA, Max. Peak Short-Circuit Current"
      m_maxIb_bus1 = "m:maxIb:bus1", "kA, Max. Breaking Current"
      m_maxIth_bus1 = "m:maxIth:bus1", "kA, Max. Thermal equivalent Short-Circuit Current"
      m_Ib_bus1 = "m:Ib:bus1", "kA, Short-Circuit Breaking Current"
      m_Sb_bus1 = "m:Sb:bus1", "MVA, Short-Circuit Breaking Power"
      m_Ik_bus1 = "m:Ik:bus1", "kA, Steady-State Short-Circuit Current"
      m_Ith_bus1 = "m:Ith:bus1", "kA, Thermal equivalent Short-Circuit Current"
      m_I_bus1 = "m:I:bus1", "kA, Short-Circuit Current"
      m_R_bus1 = "m:R:bus1", "Ohm, Fault Impedance, Real Part"
      m_X_bus1 = "m:X:bus1", "Ohm, Fault Impedance, Imaginary Part"
      m_Z_bus1 = "m:Z:bus1", "Ohm, Fault Impedance, Magnitude"
      m_phiz_bus1 = "m:phiz:bus1", "deg, Fault Impedance, Angle"
      m_zSbase_bus1 = "m:zSbase:bus1", "p.u., Fault Impedance, Magnitude"
      m_XtoR_bus1 = "m:XtoR:bus1", "X/R ratio"
      m_Zs_bus1 = "m:Zs:bus1", "Ohm, Source Impedance, Magnitude"
      m_phizs_bus1 = "m:phizs:bus1", "deg, Source Impedance, Angle"
      m_Rs_bus1 = "m:Rs:bus1", "Ohm, Source Impedance, Real Part"
      m_Xs_bus1 = "m:Xs:bus1", "Ohm, Source Impedance, Imaginary Part"
      m_phiui_bus1 = "m:phiui:bus1", "deg, Angle between Voltage and Current"
      m_cosphiui_bus1 = "m:cosphiui:bus1", "cosphi"
      m_P_bus1 = "m:P:bus1", "MW, Active Power"
      m_Q_bus1 = "m:Q:bus1", "Mvar, Reactive Power"
      m_BrkIpload_bus1 = "m:BrkIpload:bus1", "%, Breaker Loading (peak)"
      m_BrkIbload_bus1 = "m:BrkIbload:bus1", "%, Breaker Loading (breaking)"
      m_BrkmaxIb_bus1 = "m:BrkmaxIb:bus1", "kA, Max. Breaking Current (breaker time)"
      n_Ikss_bus1 = "n:Ikss:bus1", "kA, Initial Short-Circuit Current"
      n_I_bus1 = "n:I:bus1", "kA, Short-Circuit Current"
      n_phii_bus1 = "n:phii:bus1", "deg, Phase Current, Angle"
      n_Skss_bus1 = "n:Skss:bus1", "MVA, Initial Short-Circuit Power"
      n_ip_bus1 = "n:ip:bus1", "kA, Peak Short-Circuit Current"
      n_Ib_bus1 = "n:Ib:bus1", "kA, Short-Circuit Breaking Current"
      n_Sb_bus1 = "n:Sb:bus1", "MVA, Short-Circuit Breaking Power"
      n_Ik_bus1 = "n:Ik:bus1", "kA, Steady-State Short-Circuit Current"
      n_Ith_bus1 = "n:Ith:bus1", "kA, Thermal equivalent Short-Circuit Current"
      n_R_bus1 = "n:R:bus1", "Ohm, Short-Circuit Impedance, Real Part"
      n_X_bus1 = "n:X:bus1", "Ohm, Short-Circuit Impedance, Imaginary Part"
      n_Z_bus1 = "n:Z:bus1", "Ohm, Short-Circuit Impedance, Magnitude"
      n_phiz_bus1 = "n:phiz:bus1", "deg, Short-Circuit Impedance, Angle"
      n_rSbase_bus1 = "n:rSbase:bus1", "p.u., Short-Circuit Impedance, Real Part"
      n_xSbase_bus1 = "n:xSbase:bus1", "p.u., Short-Circuit Impedance, Imaginary Part"
      n_zSbase_bus1 = "n:zSbase:bus1", "p.u., Short-Circuit Impedance, Magnitude"
      n_XtoR_bus1 = "n:XtoR:bus1", "X/R ratio (peak)"
      n_RtoX_bus1 = "n:RtoX:bus1", "R/X ratio (peak)"
      n_ur_bus1 = "n:ur:bus1", "p.u., Voltage, Real Part"
      n_ui_bus1 = "n:ui:bus1", "p.u., Voltage, Imaginary Part"
      n_u_bus1 = "n:u:bus1", "p.u., Voltage, Magnitude"
      n_upc_bus1 = "n:upc:bus1", "%, Voltage, Magnitude"
      n_U_bus1 = "n:U:bus1", "kV, Line-Ground Voltage, Magnitude"
      n_Ul_bus1 = "n:Ul:bus1", "kV, Line-Line Voltage, Magnitude"
      n_phiu_bus1 = "n:phiu:bus1", "deg, Voltage, Angle"
      n_du_bus1 = "n:du:bus1", "%, Voltage Deviation"
      n_urpref_bus1 = "n:urpref:bus1", "p.u., Pre-fault Voltage, Real Part"
      n_uipref_bus1 = "n:uipref:bus1", "p.u., Pre-fault Voltage, Imaginary Part"
      n_upref_bus1 = "n:upref:bus1", "p.u., Pre-fault Voltage, Magnitude"
      n_uprefpc_bus1 = "n:uprefpc:bus1", "%, Pre-fault Voltage, Magnitude"
      n_Upref_bus1 = "n:Upref:bus1", "kV, Line-Ground Pre-fault Voltage, Magnitude"
      n_Ulpref_bus1 = "n:Ulpref:bus1", "kV, Line-Line Pre-fault Voltage, Magnitude"
      n_phiupref_bus1 = "n:phiupref:bus1", "deg, Pre-fault Voltage, Angle"
      n_phiui_bus1 = "n:phiui:bus1", "deg, Angle between Voltage and Current"
      n_cfac_bus1 = "n:cfac:bus1", "Voltage Factor"
      n_idc_bus1 = "n:idc:bus1", "kA, Decaying Component"
      n_Iasy_bus1 = "n:Iasy:bus1", "kA, Asymmetrical Breaking Current"
      n_Ithload_bus1 = "n:Ithload:bus1", "%, Loading, Thermal equivalent Current"
      n_Ipload_bus1 = "n:Ipload:bus1", "%, Loading, Peak Short-Circuit Current"
      n_Ithrtk_bus1 = "n:Ithrtk:bus1", "kA, Rated Short-Time Current (Tk)"
      n_Tn_bus1 = "n:Tn:bus1", "ms, Grid Time Constant"
      n_XtoR_b_bus1 = "n:XtoR_b:bus1", "X/R ratio (dc)"
      n_RtoX_b_bus1 = "n:RtoX_b:bus1", "R/X ratio (dc)"
      n_fctof_b_bus1 = "n:fctof_b:bus1", "Equivalent Frequency fc/f (dc)"
      c_z1_r = "c:z1:r", "p.u., Positive-sequence impedance, Real Part"
      c_z1_i = "c:z1:i", "p.u., Positive-sequence impedance, Imaginary Part"
      c_z1_phi = "c:z1:phi", "deg, Positive-sequence impedance, Angle"
      c_z1_mag = "c:z1:mag", "p.u., Positive-sequence impedance, Magnitude"
      c_z1_phirad = "c:z1:phirad", "rad, Positive-sequence impedance, Angle"
      c_imax = "c:imax", "p.u., Max. current"
      c_Zwd_r = "c:Zwd:r", "p.u., Impedance of doubly-fed asynchronous generator, Real Part"
      c_Zwd_i = "c:Zwd:i", "p.u., Impedance of doubly-fed asynchronous generator, Imaginary Part"
      c_Zwd_phi = "c:Zwd:phi", "deg, Impedance of doubly-fed asynchronous generator, Angle"
      c_Zwd_mag = "c:Zwd:mag", "p.u., Impedance of doubly-fed asynchronous generator, Magnitude"
      c_Zwd_phirad = "c:Zwd:phirad", "rad, Impedance of doubly-fed asynchronous generator, Angle"
      e_iNoShcContr = "e:iNoShcContr", "No Short-Circuit Contribution"
      e_iconfed = "e:iconfed", "Static Converter-Fed Drive"
      e_psutype = "e:psutype", "Power station unit type"
      e_searchBlockTrf = "e:searchBlockTrf", "Externally modelled unit transformer"
      e_pblocktrf = "e:pblocktrf", "Unit transformer"
      e_Ikss3PF = "e:Ikss3PF", "A, Three-phase faults, Ik''3PF"
      e_Ikss2PF = "e:Ikss2PF", "A, Two-phase faults, Ik''2PF"
      e_Ikss1PF = "e:Ikss1PF", "A, Single-phase faults, Ik''1PF"
      e_IkPFmax = "e:IkPFmax", "A, Maximum current"
      e_IkPFmin = "e:IkPFmin", "A, Minimum current"
      e_r2iec = "e:r2iec", "p.u., Resistance, r2"
      e_x2iec = "e:x2iec", "p.u., Reactance, x2"
      e_r0iec = "e:r0iec", "p.u., Resistance, r0"
      e_x0iec = "e:x0iec", "p.u., Reactance, x0"
      e_Skss = "e:Skss", "kVA, Subtransient Short-Circuit Level"
      e_Ikss = "e:Ikss", "A, Subtransient Short-Circuit Current"
      e_rtox = "e:rtox", "R to X'' ratio"
      e_xtor = "e:xtor", "X'' to R ratio"
      e_r2shc = "e:r2shc", "p.u., Resistance, r2"
      e_x2shc = "e:x2shc", "p.u., Reactance, x2"
      r_pblocktrf = "r:pblocktrf", "Unit transformer"
      m_Ikss_bus1_A = "m:Ikss:bus1:A", "kA, Initial Short-Circuit Current"
      m_Ikss_bus1_B = "m:Ikss:bus1:B", "kA, Initial Short-Circuit Current"
      m_Ikss_bus1_C = "m:Ikss:bus1:C", "kA, Initial Short-Circuit Current"
      m_Ikss_bus1_N = "m:Ikss:bus1:N", "kA, Initial Short-Circuit Current"
      m_Ikss_bus1n = "m:Ikss:bus1n", "kA, Initial Short-Circuit Current"
      m_ikss_bus1_A = "m:ikss:bus1:A", "p.u., Initial Short-Circuit Current, Magnitude"
      m_ikss_bus1_B = "m:ikss:bus1:B", "p.u., Initial Short-Circuit Current, Magnitude"
      m_ikss_bus1_C = "m:ikss:bus1:C", "p.u., Initial Short-Circuit Current, Magnitude"
      m_ikss_bus1_N = "m:ikss:bus1:N", "p.u., Initial Short-Circuit Current, Magnitude"
      m_ikss_bus1n = "m:ikss:bus1n", "p.u., Initial Short-Circuit Current, Magnitude"
      m_maxIkss_bus1n = "m:maxIkss:bus1n", "kA, Max. Initial Short-Circuit Current"
      m_Skss_bus1_A = "m:Skss:bus1:A", "MVA, Initial Short-Circuit Power"
      m_Skss_bus1_B = "m:Skss:bus1:B", "MVA, Initial Short-Circuit Power"
      m_Skss_bus1_C = "m:Skss:bus1:C", "MVA, Initial Short-Circuit Power"
      m_Skss_bus1_N = "m:Skss:bus1:N", "MVA, Initial Short-Circuit Power"
      m_Skss_bus1n = "m:Skss:bus1n", "MVA, Initial Short-Circuit Power"
      m_Skssx3_bus1_A = "m:Skssx3:bus1:A", "MVA, 3*Initial Short-Circuit Power"
      m_Skssx3_bus1_B = "m:Skssx3:bus1:B", "MVA, 3*Initial Short-Circuit Power"
      m_Skssx3_bus1_C = "m:Skssx3:bus1:C", "MVA, 3*Initial Short-Circuit Power"
      m_Skssx3_bus1_N = "m:Skssx3:bus1:N", "MVA, 3*Initial Short-Circuit Power"
      m_Skssx3_bus1n = "m:Skssx3:bus1n", "MVA, 3*Initial Short-Circuit Power"
      m_ip_bus1_A = "m:ip:bus1:A", "kA, Peak Short-Circuit Current"
      m_ip_bus1_B = "m:ip:bus1:B", "kA, Peak Short-Circuit Current"
      m_ip_bus1_C = "m:ip:bus1:C", "kA, Peak Short-Circuit Current"
      m_ip_bus1_N = "m:ip:bus1:N", "kA, Peak Short-Circuit Current"
      m_ip_bus1n = "m:ip:bus1n", "kA, Peak Short-Circuit Current"
      m_maxip_bus1n = "m:maxip:bus1n", "kA, Max. Peak Short-Circuit Current"
      m_maxIth_bus1n = "m:maxIth:bus1n", "kA, Max. Thermal equivalent Short-Circuit Current"
      m_Z_bus1_A = "m:Z:bus1:A", "Ohm, Fault Impedance, Magnitude"
      m_Z_bus1_B = "m:Z:bus1:B", "Ohm, Fault Impedance, Magnitude"
      m_Z_bus1_C = "m:Z:bus1:C", "Ohm, Fault Impedance, Magnitude"
      m_Z_bus1_N = "m:Z:bus1:N", "Ohm, Fault Impedance, Magnitude"
      m_Z_bus1n = "m:Z:bus1n", "Ohm, Fault Impedance, Magnitude"
      m_phiz_bus1_A = "m:phiz:bus1:A", "deg, Fault Impedance, Angle"
      m_phiz_bus1_B = "m:phiz:bus1:B", "deg, Fault Impedance, Angle"
      m_phiz_bus1_C = "m:phiz:bus1:C", "deg, Fault Impedance, Angle"
      m_phiz_bus1_N = "m:phiz:bus1:N", "deg, Fault Impedance, Angle"
      m_phiz_bus1n = "m:phiz:bus1n", "deg, Fault Impedance, Angle"
      m_Zl_bus1_A = "m:Zl:bus1:A", "Ohm, Line-Line Fault Impedance, Magnitude"
      m_Zl_bus1_B = "m:Zl:bus1:B", "Ohm, Line-Line Fault Impedance, Magnitude"
      m_Zl_bus1_C = "m:Zl:bus1:C", "Ohm, Line-Line Fault Impedance, Magnitude"
      m_Zl_bus1_N = "m:Zl:bus1:N", "Ohm, Line-Line Fault Impedance, Magnitude"
      m_Zl_bus1n = "m:Zl:bus1n", "Ohm, Line-Line Fault Impedance, Magnitude"
      m_phizl_bus1_A = "m:phizl:bus1:A", "deg, Line-Line Fault Impedance, Angle"
      m_phizl_bus1_B = "m:phizl:bus1:B", "deg, Line-Line Fault Impedance, Angle"
      m_phizl_bus1_C = "m:phizl:bus1:C", "deg, Line-Line Fault Impedance, Angle"
      m_phizl_bus1_N = "m:phizl:bus1:N", "deg, Line-Line Fault Impedance, Angle"
      m_phizl_bus1n = "m:phizl:bus1n", "deg, Line-Line Fault Impedance, Angle"
      m_Z0_bus1 = "m:Z0:bus1", "Ohm, Zero-Sequence Fault Impedance, Magnitude"
      m_Z0_bus1n = "m:Z0:bus1n", "Ohm, Zero-Sequence Fault Impedance, Magnitude"
      m_phiz0_bus1 = "m:phiz0:bus1", "deg, Zero-Sequence Fault Impedance, Angle"
      m_phiz0_bus1n = "m:phiz0:bus1n", "deg, Zero-Sequence Fault Impedance, Angle"
      m_z0Sbase_bus1 = "m:z0Sbase:bus1", "p.u., Zero-Sequence Fault Impedance, Magnitude"
      m_z0Sbase_bus1n = "m:z0Sbase:bus1n", "p.u., Zero-Sequence Fault Impedance, Magnitude"
      m_cosphiu1i1_bus1 = "m:cosphiu1i1:bus1", "Positive-Sequence cosphi"
      m_cosphiu1i1_bus1n = "m:cosphiu1i1:bus1n", "Positive-Sequence cosphi"
      m_I0x3watt_bus1 = "m:I0x3watt:bus1", "kA, Earth-Fault Residual Current"
      m_I0x3watt_bus1n = "m:I0x3watt:bus1n", "kA, Earth-Fault Residual Current"
      m_I0x3var_bus1 = "m:I0x3var:bus1", "kA, Capacitive Earth-Fault Current"
      m_I0x3var_bus1n = "m:I0x3var:bus1n", "kA, Capacitive Earth-Fault Current"
      m_Q0cap_bus1 = "m:Q0cap:bus1", "Mvar, Q0 capacitive"
      m_Q0cap_bus1n = "m:Q0cap:bus1n", "Mvar, Q0 capacitive"
      m_Zfrel_bus1 = "m:Zfrel:bus1", "Ohm, Relay Impedance, Magnitude"
      m_Zfrel_bus1n = "m:Zfrel:bus1n", "Ohm, Relay Impedance, Magnitude"
      m_phizfrel_bus1 = "m:phizfrel:bus1", "deg, Relay Impedance, Angle"
      m_phizfrel_bus1n = "m:phizfrel:bus1n", "deg, Relay Impedance, Angle"
      m_Rfrel_bus1 = "m:Rfrel:bus1", "Ohm, Relay Impedance, Real Part"
      m_Rfrel_bus1n = "m:Rfrel:bus1n", "Ohm, Relay Impedance, Real Part"
      m_Xfrel_bus1 = "m:Xfrel:bus1", "Ohm, Relay Impedance, Imaginary Part"
      m_Xfrel_bus1n = "m:Xfrel:bus1n", "Ohm, Relay Impedance, Imaginary Part"
      m_Zs_bus1n = "m:Zs:bus1n", "Ohm, Source Impedance, Magnitude"
      m_phizs_bus1n = "m:phizs:bus1n", "deg, Source Impedance, Angle"
      m_Rs_bus1n = "m:Rs:bus1n", "Ohm, Source Impedance, Real Part"
      m_Xs_bus1n = "m:Xs:bus1n", "Ohm, Source Impedance, Imaginary Part"
      m_Zf_bus1 = "m:Zf:bus1", "Ohm, Meas. Fault Impedance, Magnitude"
      m_Zf_bus1n = "m:Zf:bus1n", "Ohm, Meas. Fault Impedance, Magnitude"
      m_phizf_bus1 = "m:phizf:bus1", "deg, Meas. Fault Impedance, Angle"
      m_phizf_bus1n = "m:phizf:bus1n", "deg, Meas. Fault Impedance, Angle"
      m_BrkIpload_bus1n = "m:BrkIpload:bus1n", "%, Max. Breaker Loading (peak)"
      n_Ikss_bus1_A = "n:Ikss:bus1:A", "kA, Initial Short-Circuit Current"
      n_Ikss_bus1_B = "n:Ikss:bus1:B", "kA, Initial Short-Circuit Current"
      n_Ikss_bus1_C = "n:Ikss:bus1:C", "kA, Initial Short-Circuit Current"
      n_Ikss_bus1_N = "n:Ikss:bus1:N", "kA, Initial Short-Circuit Current"
      n_Ikss_bus1n = "n:Ikss:bus1n", "kA, Initial Short-Circuit Current"
      n_I_bus1_A = "n:I:bus1:A", "kA, Short-Circuit Current"
      n_I_bus1_B = "n:I:bus1:B", "kA, Short-Circuit Current"
      n_I_bus1_C = "n:I:bus1:C", "kA, Short-Circuit Current"
      n_I_bus1_N = "n:I:bus1:N", "kA, Short-Circuit Current"
      n_I_bus1n = "n:I:bus1n", "kA, Short-Circuit Current"
      n_phii_bus1_A = "n:phii:bus1:A", "deg, Phase Current, Angle"
      n_phii_bus1_B = "n:phii:bus1:B", "deg, Phase Current, Angle"
      n_phii_bus1_C = "n:phii:bus1:C", "deg, Phase Current, Angle"
      n_phii_bus1_N = "n:phii:bus1:N", "deg, Phase Current, Angle"
      n_phii_bus1n = "n:phii:bus1n", "deg, Phase Current, Angle"
      n_Skss_bus1_A = "n:Skss:bus1:A", "MVA, Initial Short-Circuit Power"
      n_Skss_bus1_B = "n:Skss:bus1:B", "MVA, Initial Short-Circuit Power"
      n_Skss_bus1_C = "n:Skss:bus1:C", "MVA, Initial Short-Circuit Power"
      n_Skss_bus1_N = "n:Skss:bus1:N", "MVA, Initial Short-Circuit Power"
      n_Skss_bus1n = "n:Skss:bus1n", "MVA, Initial Short-Circuit Power"
      n_Skssx3_bus1_A = "n:Skssx3:bus1:A", "MVA, 3*Initial Short-Circuit Power"
      n_Skssx3_bus1_B = "n:Skssx3:bus1:B", "MVA, 3*Initial Short-Circuit Power"
      n_Skssx3_bus1_C = "n:Skssx3:bus1:C", "MVA, 3*Initial Short-Circuit Power"
      n_Skssx3_bus1_N = "n:Skssx3:bus1:N", "MVA, 3*Initial Short-Circuit Power"
      n_Skssx3_bus1n = "n:Skssx3:bus1n", "MVA, 3*Initial Short-Circuit Power"
      n_ip_bus1_A = "n:ip:bus1:A", "kA, Peak Short-Circuit Current"
      n_ip_bus1_B = "n:ip:bus1:B", "kA, Peak Short-Circuit Current"
      n_ip_bus1_C = "n:ip:bus1:C", "kA, Peak Short-Circuit Current"
      n_ip_bus1_N = "n:ip:bus1:N", "kA, Peak Short-Circuit Current"
      n_ip_bus1n = "n:ip:bus1n", "kA, Peak Short-Circuit Current"
      n_Ib_bus1_A = "n:Ib:bus1:A", "kA, Short-Circuit Breaking Current"
      n_Ib_bus1_B = "n:Ib:bus1:B", "kA, Short-Circuit Breaking Current"
      n_Ib_bus1_C = "n:Ib:bus1:C", "kA, Short-Circuit Breaking Current"
      n_Ib_bus1_N = "n:Ib:bus1:N", "kA, Short-Circuit Breaking Current"
      n_Ib_bus1n = "n:Ib:bus1n", "kA, Short-Circuit Breaking Current"
      n_Sb_bus1_A = "n:Sb:bus1:A", "MVA, Short-Circuit Breaking Power"
      n_Sb_bus1_B = "n:Sb:bus1:B", "MVA, Short-Circuit Breaking Power"
      n_Sb_bus1_C = "n:Sb:bus1:C", "MVA, Short-Circuit Breaking Power"
      n_Sb_bus1_N = "n:Sb:bus1:N", "MVA, Short-Circuit Breaking Power"
      n_Sb_bus1n = "n:Sb:bus1n", "MVA, Short-Circuit Breaking Power"
      n_Ith_bus1_A = "n:Ith:bus1:A", "kA, Thermal equivalent Short-Circuit Current"
      n_Ith_bus1_B = "n:Ith:bus1:B", "kA, Thermal equivalent Short-Circuit Current"
      n_Ith_bus1_C = "n:Ith:bus1:C", "kA, Thermal equivalent Short-Circuit Current"
      n_Ith_bus1_N = "n:Ith:bus1:N", "kA, Thermal equivalent Short-Circuit Current"
      n_Ith_bus1n = "n:Ith:bus1n", "kA, Thermal equivalent Short-Circuit Current"
      n_Ithmax_bus1 = "n:Ithmax:bus1", "kA, Max. Thermal equivalent Short-Circuit Current"
      n_Ithmax_bus1n = "n:Ithmax:bus1n", "kA, Max. Thermal equivalent Short-Circuit Current"
      n_Ik_bus1_A = "n:Ik:bus1:A", "kA, Steady-State Short-Circuit Current"
      n_Ik_bus1_B = "n:Ik:bus1:B", "kA, Steady-State Short-Circuit Current"
      n_Ik_bus1_C = "n:Ik:bus1:C", "kA, Steady-State Short-Circuit Current"
      n_Ik_bus1_N = "n:Ik:bus1:N", "kA, Steady-State Short-Circuit Current"
      n_Ik_bus1n = "n:Ik:bus1n", "kA, Steady-State Short-Circuit Current"
      n_R0_bus1 = "n:R0:bus1", "Ohm, Zero-Sequence Impedance, Real Part"
      n_R0_bus1n = "n:R0:bus1n", "Ohm, Zero-Sequence Impedance, Real Part"
      n_X0_bus1 = "n:X0:bus1", "Ohm, Zero-Sequence Impedance, Imaginary Part"
      n_X0_bus1n = "n:X0:bus1n", "Ohm, Zero-Sequence Impedance, Imaginary Part"
      n_Z0_bus1 = "n:Z0:bus1", "Ohm, Zero-Sequence Impedance, Magnitude"
      n_Z0_bus1n = "n:Z0:bus1n", "Ohm, Zero-Sequence Impedance, Magnitude"
      n_phiz0_bus1 = "n:phiz0:bus1", "deg, Zero-Sequence Impedance, Angle"
      n_phiz0_bus1n = "n:phiz0:bus1n", "deg, Zero-Sequence Impedance, Angle"
      n_r0Sbase_bus1 = "n:r0Sbase:bus1", "p.u., Zero-Sequence Impedance, Real Part"
      n_r0Sbase_bus1n = "n:r0Sbase:bus1n", "p.u., Zero-Sequence Impedance, Real Part"
      n_x0Sbase_bus1 = "n:x0Sbase:bus1", "p.u., Zero-Sequence Impedance, Imaginary Part"
      n_x0Sbase_bus1n = "n:x0Sbase:bus1n", "p.u., Zero-Sequence Impedance, Imaginary Part"
      n_z0Sbase_bus1 = "n:z0Sbase:bus1", "p.u., Zero-Sequence Impedance, Magnitude"
      n_z0Sbase_bus1n = "n:z0Sbase:bus1n", "p.u., Zero-Sequence Impedance, Magnitude"
      n_R1_bus1 = "n:R1:bus1", "Ohm, Positive-Sequence Impedance, Real Part"
      n_R1_bus1n = "n:R1:bus1n", "Ohm, Positive-Sequence Impedance, Real Part"
      n_X1_bus1 = "n:X1:bus1", "Ohm, Positive-Sequence Impedance, Imaginary Part"
      n_X1_bus1n = "n:X1:bus1n", "Ohm, Positive-Sequence Impedance, Imaginary Part"
      n_Z1_bus1 = "n:Z1:bus1", "Ohm, Positive-Sequence Impedance, Magnitude"
      n_Z1_bus1n = "n:Z1:bus1n", "Ohm, Positive-Sequence Impedance, Magnitude"
      n_phiz1_bus1 = "n:phiz1:bus1", "deg, Positive-Sequence Impedance, Angle"
      n_phiz1_bus1n = "n:phiz1:bus1n", "deg, Positive-Sequence Impedance, Angle"
      n_r1Sbase_bus1 = "n:r1Sbase:bus1", "p.u., Positive-Sequence Impedance, Real Part"
      n_r1Sbase_bus1n = "n:r1Sbase:bus1n", "p.u., Positive-Sequence Impedance, Real Part"
      n_x1Sbase_bus1 = "n:x1Sbase:bus1", "p.u., Positive-Sequence Impedance, Imaginary Part"
      n_x1Sbase_bus1n = "n:x1Sbase:bus1n", "p.u., Positive-Sequence Impedance, Imaginary Part"
      n_z1Sbase_bus1 = "n:z1Sbase:bus1", "p.u., Positive-Sequence Impedance, Magnitude"
      n_z1Sbase_bus1n = "n:z1Sbase:bus1n", "p.u., Positive-Sequence Impedance, Magnitude"
      n_R2_bus1 = "n:R2:bus1", "Ohm, Negative-Sequence Impedance, Real Part"
      n_R2_bus1n = "n:R2:bus1n", "Ohm, Negative-Sequence Impedance, Real Part"
      n_X2_bus1 = "n:X2:bus1", "Ohm, Negative-Sequence Impedance, Imaginary Part"
      n_X2_bus1n = "n:X2:bus1n", "Ohm, Negative-Sequence Impedance, Imaginary Part"
      n_Z2_bus1 = "n:Z2:bus1", "Ohm, Negative-Sequence Impedance, Magnitude"
      n_Z2_bus1n = "n:Z2:bus1n", "Ohm, Negative-Sequence Impedance, Magnitude"
      n_phiz2_bus1 = "n:phiz2:bus1", "deg, Negative-Sequence Impedance, Angle"
      n_phiz2_bus1n = "n:phiz2:bus1n", "deg, Negative-Sequence Impedance, Angle"
      n_r2Sbase_bus1 = "n:r2Sbase:bus1", "p.u., Negative-Sequence Impedance, Real Part"
      n_r2Sbase_bus1n = "n:r2Sbase:bus1n", "p.u., Negative-Sequence Impedance, Real Part"
      n_x2Sbase_bus1 = "n:x2Sbase:bus1", "p.u., Negative-Sequence Impedance, Imaginary Part"
      n_x2Sbase_bus1n = "n:x2Sbase:bus1n", "p.u., Negative-Sequence Impedance, Imaginary Part"
      n_z2Sbase_bus1 = "n:z2Sbase:bus1", "p.u., Negative-Sequence Impedance, Magnitude"
      n_z2Sbase_bus1n = "n:z2Sbase:bus1n", "p.u., Negative-Sequence Impedance, Magnitude"
      n_R_bus1_A = "n:R:bus1:A", "Ohm, Network Resistance, Line-Ground"
      n_R_bus1_B = "n:R:bus1:B", "Ohm, Network Resistance, Line-Ground"
      n_R_bus1_C = "n:R:bus1:C", "Ohm, Network Resistance, Line-Ground"
      n_R_bus1_N = "n:R:bus1:N", "Ohm, Network Resistance, Line-Ground"
      n_R_bus1n = "n:R:bus1n", "Ohm, Network Resistance, Line-Ground"
      n_X_bus1_A = "n:X:bus1:A", "Ohm, Network Reactance, Line-Ground"
      n_X_bus1_B = "n:X:bus1:B", "Ohm, Network Reactance, Line-Ground"
      n_X_bus1_C = "n:X:bus1:C", "Ohm, Network Reactance, Line-Ground"
      n_X_bus1_N = "n:X:bus1:N", "Ohm, Network Reactance, Line-Ground"
      n_X_bus1n = "n:X:bus1n", "Ohm, Network Reactance, Line-Ground"
      n_Z_bus1_A = "n:Z:bus1:A", "Ohm, Network Impedance, Line-Ground"
      n_Z_bus1_B = "n:Z:bus1:B", "Ohm, Network Impedance, Line-Ground"
      n_Z_bus1_C = "n:Z:bus1:C", "Ohm, Network Impedance, Line-Ground"
      n_Z_bus1_N = "n:Z:bus1:N", "Ohm, Network Impedance, Line-Ground"
      n_Z_bus1n = "n:Z:bus1n", "Ohm, Network Impedance, Line-Ground"
      n_phiz_bus1_A = "n:phiz:bus1:A", "deg, Angle of Network Impedance, Line-Ground"
      n_phiz_bus1_B = "n:phiz:bus1:B", "deg, Angle of Network Impedance, Line-Ground"
      n_phiz_bus1_C = "n:phiz:bus1:C", "deg, Angle of Network Impedance, Line-Ground"
      n_phiz_bus1_N = "n:phiz:bus1:N", "deg, Angle of Network Impedance, Line-Ground"
      n_phiz_bus1n = "n:phiz:bus1n", "deg, Angle of Network Impedance, Line-Ground"
      n_Rl_bus1_A = "n:Rl:bus1:A", "Ohm, Network Resistance, Line-Line"
      n_Rl_bus1_B = "n:Rl:bus1:B", "Ohm, Network Resistance, Line-Line"
      n_Rl_bus1_C = "n:Rl:bus1:C", "Ohm, Network Resistance, Line-Line"
      n_Rl_bus1_N = "n:Rl:bus1:N", "Ohm, Network Resistance, Line-Line"
      n_Rl_bus1n = "n:Rl:bus1n", "Ohm, Network Resistance, Line-Line"
      n_Xl_bus1_A = "n:Xl:bus1:A", "Ohm, Network Reactance, Line-Line"
      n_Xl_bus1_B = "n:Xl:bus1:B", "Ohm, Network Reactance, Line-Line"
      n_Xl_bus1_C = "n:Xl:bus1:C", "Ohm, Network Reactance, Line-Line"
      n_Xl_bus1_N = "n:Xl:bus1:N", "Ohm, Network Reactance, Line-Line"
      n_Xl_bus1n = "n:Xl:bus1n", "Ohm, Network Reactance, Line-Line"
      n_Zl_bus1_A = "n:Zl:bus1:A", "Ohm, Network Impedance, Line-Line"
      n_Zl_bus1_B = "n:Zl:bus1:B", "Ohm, Network Impedance, Line-Line"
      n_Zl_bus1_C = "n:Zl:bus1:C", "Ohm, Network Impedance, Line-Line"
      n_Zl_bus1_N = "n:Zl:bus1:N", "Ohm, Network Impedance, Line-Line"
      n_Zl_bus1n = "n:Zl:bus1n", "Ohm, Network Impedance, Line-Line"
      n_phizl_bus1_A = "n:phizl:bus1:A", "deg, Angle of Network Impedance, Line-Line"
      n_phizl_bus1_B = "n:phizl:bus1:B", "deg, Angle of Network Impedance, Line-Line"
      n_phizl_bus1_C = "n:phizl:bus1:C", "deg, Angle of Network Impedance, Line-Line"
      n_phizl_bus1_N = "n:phizl:bus1:N", "deg, Angle of Network Impedance, Line-Line"
      n_phizl_bus1n = "n:phizl:bus1n", "deg, Angle of Network Impedance, Line-Line"
      n_Rln_bus1_A = "n:Rln:bus1:A", "Ohm, Network Resistance, Line-Neutral"
      n_Rln_bus1_B = "n:Rln:bus1:B", "Ohm, Network Resistance, Line-Neutral"
      n_Rln_bus1_C = "n:Rln:bus1:C", "Ohm, Network Resistance, Line-Neutral"
      n_Rln_bus1_N = "n:Rln:bus1:N", "Ohm, Network Resistance, Line-Neutral"
      n_Rln_bus1n = "n:Rln:bus1n", "Ohm, Network Resistance, Line-Neutral"
      n_Xln_bus1_A = "n:Xln:bus1:A", "Ohm, Network Reactance, Line-Neutral"
      n_Xln_bus1_B = "n:Xln:bus1:B", "Ohm, Network Reactance, Line-Neutral"
      n_Xln_bus1_C = "n:Xln:bus1:C", "Ohm, Network Reactance, Line-Neutral"
      n_Xln_bus1_N = "n:Xln:bus1:N", "Ohm, Network Reactance, Line-Neutral"
      n_Xln_bus1n = "n:Xln:bus1n", "Ohm, Network Reactance, Line-Neutral"
      n_Zln_bus1_A = "n:Zln:bus1:A", "Ohm, Network Impedance, Line-Neutral"
      n_Zln_bus1_B = "n:Zln:bus1:B", "Ohm, Network Impedance, Line-Neutral"
      n_Zln_bus1_C = "n:Zln:bus1:C", "Ohm, Network Impedance, Line-Neutral"
      n_Zln_bus1_N = "n:Zln:bus1:N", "Ohm, Network Impedance, Line-Neutral"
      n_Zln_bus1n = "n:Zln:bus1n", "Ohm, Network Impedance, Line-Neutral"
      n_phizln_bus1_A = "n:phizln:bus1:A", "deg, Angle of Network Impedance, Line-Neutral"
      n_phizln_bus1_B = "n:phizln:bus1:B", "deg, Angle of Network Impedance, Line-Neutral"
      n_phizln_bus1_C = "n:phizln:bus1:C", "deg, Angle of Network Impedance, Line-Neutral"
      n_phizln_bus1_N = "n:phizln:bus1:N", "deg, Angle of Network Impedance, Line-Neutral"
      n_phizln_bus1n = "n:phizln:bus1n", "deg, Angle of Network Impedance, Line-Neutral"
      n_Rn_bus1 = "n:Rn:bus1", "Ohm, Network Resistance, Neutral"
      n_Rn_bus1n = "n:Rn:bus1n", "Ohm, Network Resistance, Neutral"
      n_Xn_bus1 = "n:Xn:bus1", "Ohm, Network Reactance, Neutral"
      n_Xn_bus1n = "n:Xn:bus1n", "Ohm, Network Reactance, Neutral"
      n_Zn_bus1 = "n:Zn:bus1", "Ohm, Network Impedance, Neutral"
      n_Zn_bus1n = "n:Zn:bus1n", "Ohm, Network Impedance, Neutral"
      n_phizn_bus1 = "n:phizn:bus1", "deg, Angle of Network Impedance, Neutral"
      n_phizn_bus1n = "n:phizn:bus1n", "deg, Angle of Network Impedance, Neutral"
      n_eff_bus1_A = "n:eff:bus1:A", "Earth Fault Factor"
      n_eff_bus1_B = "n:eff:bus1:B", "Earth Fault Factor"
      n_eff_bus1_C = "n:eff:bus1:C", "Earth Fault Factor"
      n_eff_bus1_N = "n:eff:bus1:N", "Earth Fault Factor"
      n_eff_bus1n = "n:eff:bus1n", "Earth Fault Factor"
      n_X0toX1_bus1 = "n:X0toX1:bus1", "X0/X1"
      n_X0toX1_bus1n = "n:X0toX1:bus1n", "X0/X1"
      n_in_bus1 = "n:in:bus1", "p.u., Neutral Current, Magnitude"
      n_in_bus1n = "n:in:bus1n", "p.u., Neutral Current, Magnitude"
      n_In_bus1 = "n:In:bus1", "kA, Neutral Current, Magnitude"
      n_In_bus1n = "n:In:bus1n", "kA, Neutral Current, Magnitude"
      n_phiin_bus1 = "n:phiin:bus1", "deg, Neutral Current, Angle"
      n_phiin_bus1n = "n:phiin:bus1n", "deg, Neutral Current, Angle"
      n_i0r_bus1 = "n:i0r:bus1", "p.u., Zero-Sequence Current, Real Part"
      n_i0r_bus1n = "n:i0r:bus1n", "p.u., Zero-Sequence Current, Real Part"
      n_i0i_bus1 = "n:i0i:bus1", "p.u., Zero-Sequence Current, Imaginary Part"
      n_i0i_bus1n = "n:i0i:bus1n", "p.u., Zero-Sequence Current, Imaginary Part"
      n_i0_bus1 = "n:i0:bus1", "p.u., Zero-Sequence Current, Magnitude"
      n_i0_bus1n = "n:i0:bus1n", "p.u., Zero-Sequence Current, Magnitude"
      n_I0_bus1 = "n:I0:bus1", "kA, Zero-Sequence Current, Magnitude"
      n_I0_bus1n = "n:I0:bus1n", "kA, Zero-Sequence Current, Magnitude"
      n_I0x3_bus1 = "n:I0x3:bus1", "kA, 3*I0"
      n_I0x3_bus1n = "n:I0x3:bus1n", "kA, 3*I0"
      n_phii0_bus1 = "n:phii0:bus1", "deg, Zero-Sequence Current, Angle"
      n_phii0_bus1n = "n:phii0:bus1n", "deg, Zero-Sequence Current, Angle"
      n_i1r_bus1 = "n:i1r:bus1", "p.u., Positive-Sequence Current, Real Part"
      n_i1r_bus1n = "n:i1r:bus1n", "p.u., Positive-Sequence Current, Real Part"
      n_i1i_bus1 = "n:i1i:bus1", "p.u., Positive-Sequence Current, Imaginary Part"
      n_i1i_bus1n = "n:i1i:bus1n", "p.u., Positive-Sequence Current, Imaginary Part"
      n_i1_bus1 = "n:i1:bus1", "p.u., Positive-Sequence Current, Magnitude"
      n_i1_bus1n = "n:i1:bus1n", "p.u., Positive-Sequence Current, Magnitude"
      n_I1_bus1 = "n:I1:bus1", "kA, Positive-Sequence Current, Magnitude"
      n_I1_bus1n = "n:I1:bus1n", "kA, Positive-Sequence Current, Magnitude"
      n_phii1_bus1 = "n:phii1:bus1", "deg, Positive-Sequence Current, Angle"
      n_phii1_bus1n = "n:phii1:bus1n", "deg, Positive-Sequence Current, Angle"
      n_i2r_bus1 = "n:i2r:bus1", "p.u., Negative-Sequence Current, Real Part"
      n_i2r_bus1n = "n:i2r:bus1n", "p.u., Negative-Sequence Current, Real Part"
      n_i2i_bus1 = "n:i2i:bus1", "p.u., Negative-Sequence Current, Imaginary Part"
      n_i2i_bus1n = "n:i2i:bus1n", "p.u., Negative-Sequence Current, Imaginary Part"
      n_i2_bus1 = "n:i2:bus1", "p.u., Negative-Sequence Current, Magnitude"
      n_i2_bus1n = "n:i2:bus1n", "p.u., Negative-Sequence Current, Magnitude"
      n_I2_bus1 = "n:I2:bus1", "kA, Negative-Sequence Current, Magnitude"
      n_I2_bus1n = "n:I2:bus1n", "kA, Negative-Sequence Current, Magnitude"
      n_phii2_bus1 = "n:phii2:bus1", "deg, Negative-Sequence Current, Angle"
      n_phii2_bus1n = "n:phii2:bus1n", "deg, Negative-Sequence Current, Angle"
      n_du_bus1_A = "n:du:bus1:A", "%, Voltage Deviation"
      n_du_bus1_B = "n:du:bus1:B", "%, Voltage Deviation"
      n_du_bus1_C = "n:du:bus1:C", "%, Voltage Deviation"
      n_du_bus1_N = "n:du:bus1:N", "%, Voltage Deviation"
      n_du_bus1n = "n:du:bus1n", "%, Voltage Deviation"
      n_urpref_bus1_A = "n:urpref:bus1:A", "p.u., Pre-fault Voltage, Real Part"
      n_urpref_bus1_B = "n:urpref:bus1:B", "p.u., Pre-fault Voltage, Real Part"
      n_urpref_bus1_C = "n:urpref:bus1:C", "p.u., Pre-fault Voltage, Real Part"
      n_urpref_bus1_N = "n:urpref:bus1:N", "p.u., Pre-fault Voltage, Real Part"
      n_urpref_bus1n = "n:urpref:bus1n", "p.u., Pre-fault Voltage, Real Part"
      n_uipref_bus1_A = "n:uipref:bus1:A", "p.u., Pre-fault Voltage, Imaginary Part"
      n_uipref_bus1_B = "n:uipref:bus1:B", "p.u., Pre-fault Voltage, Imaginary Part"
      n_uipref_bus1_C = "n:uipref:bus1:C", "p.u., Pre-fault Voltage, Imaginary Part"
      n_uipref_bus1_N = "n:uipref:bus1:N", "p.u., Pre-fault Voltage, Imaginary Part"
      n_uipref_bus1n = "n:uipref:bus1n", "p.u., Pre-fault Voltage, Imaginary Part"
      n_upref_bus1_A = "n:upref:bus1:A", "p.u., Pre-fault Voltage, Magnitude"
      n_upref_bus1_B = "n:upref:bus1:B", "p.u., Pre-fault Voltage, Magnitude"
      n_upref_bus1_C = "n:upref:bus1:C", "p.u., Pre-fault Voltage, Magnitude"
      n_upref_bus1_N = "n:upref:bus1:N", "p.u., Pre-fault Voltage, Magnitude"
      n_upref_bus1n = "n:upref:bus1n", "p.u., Pre-fault Voltage, Magnitude"
      n_uprefpc_bus1_A = "n:uprefpc:bus1:A", "%, Pre-fault Voltage, Magnitude"
      n_uprefpc_bus1_B = "n:uprefpc:bus1:B", "%, Pre-fault Voltage, Magnitude"
      n_uprefpc_bus1_C = "n:uprefpc:bus1:C", "%, Pre-fault Voltage, Magnitude"
      n_uprefpc_bus1_N = "n:uprefpc:bus1:N", "%, Pre-fault Voltage, Magnitude"
      n_uprefpc_bus1n = "n:uprefpc:bus1n", "%, Pre-fault Voltage, Magnitude"
      n_Upref_bus1_A = "n:Upref:bus1:A", "kV, Line-Ground Pre-fault Voltage, Magnitude"
      n_Upref_bus1_B = "n:Upref:bus1:B", "kV, Line-Ground Pre-fault Voltage, Magnitude"
      n_Upref_bus1_C = "n:Upref:bus1:C", "kV, Line-Ground Pre-fault Voltage, Magnitude"
      n_Upref_bus1_N = "n:Upref:bus1:N", "kV, Line-Ground Pre-fault Voltage, Magnitude"
      n_Upref_bus1n = "n:Upref:bus1n", "kV, Line-Ground Pre-fault Voltage, Magnitude"
      n_Ulpref_bus1_A = "n:Ulpref:bus1:A", "kV, Line-Line Pre-fault Voltage, Magnitude"
      n_Ulpref_bus1_B = "n:Ulpref:bus1:B", "kV, Line-Line Pre-fault Voltage, Magnitude"
      n_Ulpref_bus1_C = "n:Ulpref:bus1:C", "kV, Line-Line Pre-fault Voltage, Magnitude"
      n_Ulpref_bus1_N = "n:Ulpref:bus1:N", "kV, Line-Line Pre-fault Voltage, Magnitude"
      n_Ulpref_bus1n = "n:Ulpref:bus1n", "kV, Line-Line Pre-fault Voltage, Magnitude"
      n_phiupref_bus1_A = "n:phiupref:bus1:A", "deg, Pre-fault Voltage, Angle"
      n_phiupref_bus1_B = "n:phiupref:bus1:B", "deg, Pre-fault Voltage, Angle"
      n_phiupref_bus1_C = "n:phiupref:bus1:C", "deg, Pre-fault Voltage, Angle"
      n_phiupref_bus1_N = "n:phiupref:bus1:N", "deg, Pre-fault Voltage, Angle"
      n_phiupref_bus1n = "n:phiupref:bus1n", "deg, Pre-fault Voltage, Angle"
      n_phiui_bus1_A = "n:phiui:bus1:A", "deg, Angle between Voltage and Current"
      n_phiui_bus1_B = "n:phiui:bus1:B", "deg, Angle between Voltage and Current"
      n_phiui_bus1_C = "n:phiui:bus1:C", "deg, Angle between Voltage and Current"
      n_phiui_bus1_N = "n:phiui:bus1:N", "deg, Angle between Voltage and Current"
      n_phiui_bus1n = "n:phiui:bus1n", "deg, Angle between Voltage and Current"
      n_phiu1i1_bus1 = "n:phiu1i1:bus1", "deg, Angle between Voltage and Current in positive sequence system"
      n_phiu1i1_bus1n = "n:phiu1i1:bus1n", "deg, Angle between Voltage and Current in positive sequence system"
      n_phiu2i2_bus1 = "n:phiu2i2:bus1", "deg, Angle between Voltage and Current in negative sequence system"
      n_phiu2i2_bus1n = "n:phiu2i2:bus1n", "deg, Angle between Voltage and Current in negative sequence system"
      n_phiu0i0_bus1 = "n:phiu0i0:bus1", "deg, Angle between Voltage and Current in zero-sequence system"
      n_phiu0i0_bus1n = "n:phiu0i0:bus1n", "deg, Angle between Voltage and Current in zero-sequence system"
      n_Ithload_bus1n = "n:Ithload:bus1n", "%, Loading, Thermal equivalent Current"
      n_Ipload_bus1n = "n:Ipload:bus1n", "%, Loading, Peak Short-Circuit Current"
      n_Ithrtk_bus1n = "n:Ithrtk:bus1n", "kA, Rated Short-Time Current (Tk)"
      n_cfac_bus1n = "n:cfac:bus1n", "Voltage Factor"
      n_XtoR_bus1n = "n:XtoR:bus1n", "X/R ratio (peak)"
      n_RtoX_bus1n = "n:RtoX:bus1n", "R/X ratio (peak)"
      c_z2_r = "c:z2:r", "p.u., Negative-sequence impedance, Real Part"
      c_z2_i = "c:z2:i", "p.u., Negative-sequence impedance, Imaginary Part"
      c_z2_phi = "c:z2:phi", "deg, Negative-sequence impedance, Angle"
      c_z2_mag = "c:z2:mag", "p.u., Negative-sequence impedance, Magnitude"
      c_z2_phirad = "c:z2:phirad", "rad, Negative-sequence impedance, Angle"
      c_z0_r = "c:z0:r", "p.u., Zero-sequence impedance, Real Part"
      c_z0_i = "c:z0:i", "p.u., Zero-sequence impedance, Imaginary Part"
      c_z0_phi = "c:z0:phi", "deg, Zero-sequence impedance, Angle"
      c_z0_mag = "c:z0:mag", "p.u., Zero-sequence impedance, Magnitude"
      c_z0_phirad = "c:z0:phirad", "rad, Zero-sequence impedance, Angle"

    class ElmSecctrl(Enum):
      _init_ = 'value __doc__'

      s_pset = "s:pset", "MW, IN, Active Power Setpoint, continuous, double"
      s_dpsco = "s:dpsco", "MW, OUT, Controller Output, continuous, double"
      c_rembar = "c:rembar", "Controlled Busbar"
      c_i_phase = "c:i_phase", "Controlled Phase"
      c_pini = "c:pini", "MW, Active Power Setpoint"
      c_imode = "c:imode", "Control Mode"
      c_i_modus = "c:i_modus", "Control Mode 0=phi 1=p"
      c_phiu = "c:phiu", "rad, Phase Angle"
      c_PExch = "c:PExch", "MW, Power Exchange"
      c_PctrlLim = "c:PctrlLim", "MW, Controller Output Limit"
      e_i_net = "e:i_net", "Control Mode"
      e_rembar = "e:rembar", "StaBar*,ElmTerm*, Busbar for Frequency Measurement"
      e_iexchange = "e:iexchange", "Exchange for"
      e_cpGrid = "e:cpGrid", "Grid"
      e_pPmeas = "e:pPmeas", "ElmBoundary,ElmArea,ElmZone, Boundary/Zone/Area"
      e_psetp = "e:psetp", "MW, Power Setpoint"
      e_Kpf = "e:Kpf", "MW/Hz, Frequency Bias"
      e_imode = "e:imode", "Active Power Distribution"
      e_psym = "e:psym", "ElmSym,ElmGenstat,ElmVscmono,ElmVsc,ElmVac,ElmXnet,ElmPvsys, Machines"
      e_cvpp = "e:cvpp", "%, Active Power Percentage"
      e_cmo = "e:cmo", "Merit Order"
      e_qdslCtrl = "e:qdslCtrl", "Quasi-Dynamic Model"
      r_rembar = "r:rembar", "StaBar*,ElmTerm*, Busbar for Frequency Measurement"
      r_cpGrid = "r:cpGrid", "Grid"
      r_pPmeas = "r:pPmeas", "ElmBoundary,ElmArea,ElmZone, Boundary/Zone/Area"
      r_psym = "r:psym", "ElmSym,ElmGenstat,ElmVscmono,ElmVsc,ElmVac,ElmXnet,ElmPvsys, Machines"
      r_qdslCtrl = "r:qdslCtrl", "Quasi-Dynamic Model"

    class ElmShnt(Enum):
      _init_ = 'value __doc__'

      m_u0_bus1 = "m:u0:bus1", "p.u., Zero-Sequence-Voltage, Magnitude"
      m_u0_bus1n = "m:u0:bus1n", "p.u., Zero-Sequence-Voltage, Magnitude"
      m_phiu0_bus1 = "m:phiu0:bus1", "deg, Zero-Sequence-Voltage, Angle"
      m_phiu0_bus1n = "m:phiu0:bus1n", "deg, Zero-Sequence-Voltage, Angle"
      m_u1r_bus1 = "m:u1r:bus1", "p.u., Positive-Sequence-Voltage, Real Part"
      m_u1r_bus1n = "m:u1r:bus1n", "p.u., Positive-Sequence-Voltage, Real Part"
      m_u1i_bus1 = "m:u1i:bus1", "p.u., Positive-Sequence-Voltage, Imaginary Part"
      m_u1i_bus1n = "m:u1i:bus1n", "p.u., Positive-Sequence-Voltage, Imaginary Part"
      m_u1_bus1 = "m:u1:bus1", "p.u., Positive-Sequence-Voltage, Magnitude"
      m_u1_bus1n = "m:u1:bus1n", "p.u., Positive-Sequence-Voltage, Magnitude"
      m_U1_bus1 = "m:U1:bus1", "kV, Line-Ground Positive-Sequence-Voltage, Magnitude"
      m_U1_bus1n = "m:U1:bus1n", "kV, Line-Ground Positive-Sequence-Voltage, Magnitude"
      m_U1l_bus1 = "m:U1l:bus1", "kV, Line-Line Positive-Sequence-Voltage, Magnitude"
      m_U1l_bus1n = "m:U1l:bus1n", "kV, Line-Line Positive-Sequence-Voltage, Magnitude"
      m_phiu1_bus1 = "m:phiu1:bus1", "deg, Positive-Sequence-Voltage, Angle"
      m_phiu1_bus1n = "m:phiu1:bus1n", "deg, Positive-Sequence-Voltage, Angle"
      m_u2_bus1 = "m:u2:bus1", "p.u., Negative-Sequence-Voltage, Magnitude"
      m_u2_bus1n = "m:u2:bus1n", "p.u., Negative-Sequence-Voltage, Magnitude"
      m_phiu2_bus1 = "m:phiu2:bus1", "deg, Negative-Sequence-Voltage, Angle"
      m_phiu2_bus1n = "m:phiu2:bus1n", "deg, Negative-Sequence-Voltage, Angle"
      m_inet_bus1_A = "m:inet:bus1:A", "p.u., Phase Current, Magnitude, referred to network"
      m_inet_bus1_B = "m:inet:bus1:B", "p.u., Phase Current, Magnitude, referred to network"
      m_inet_bus1_C = "m:inet:bus1:C", "p.u., Phase Current, Magnitude, referred to network"
      m_inet_bus1_N = "m:inet:bus1:N", "p.u., Phase Current, Magnitude, referred to network"
      m_inet_bus1n = "m:inet:bus1n", "p.u., Phase Current, Magnitude, referred to network"
      m_I_bus1_A = "m:I:bus1:A", "kA, Phase Current, Magnitude"
      m_I_bus1_B = "m:I:bus1:B", "kA, Phase Current, Magnitude"
      m_I_bus1_C = "m:I:bus1:C", "kA, Phase Current, Magnitude"
      m_I_bus1_N = "m:I:bus1:N", "kA, Phase Current, Magnitude"
      m_I_bus1n = "m:I:bus1n", "kA, Phase Current, Magnitude"
      m_phii_bus1_A = "m:phii:bus1:A", "deg, Phase Current, Angle"
      m_phii_bus1_B = "m:phii:bus1:B", "deg, Phase Current, Angle"
      m_phii_bus1_C = "m:phii:bus1:C", "deg, Phase Current, Angle"
      m_phii_bus1_N = "m:phii:bus1:N", "deg, Phase Current, Angle"
      m_phii_bus1n = "m:phii:bus1n", "deg, Phase Current, Angle"
      m_i0r_bus1 = "m:i0r:bus1", "p.u., Zero-Sequence Current, Real Part"
      m_i0r_bus1n = "m:i0r:bus1n", "p.u., Zero-Sequence Current, Real Part"
      m_i0i_bus1 = "m:i0i:bus1", "p.u., Zero-Sequence Current, Imaginary Part"
      m_i0i_bus1n = "m:i0i:bus1n", "p.u., Zero-Sequence Current, Imaginary Part"
      m_i0_bus1 = "m:i0:bus1", "p.u., Zero-Sequence Current, Magnitude"
      m_i0_bus1n = "m:i0:bus1n", "p.u., Zero-Sequence Current, Magnitude"
      m_I0_bus1 = "m:I0:bus1", "kA, Zero-Sequence Current, Magnitude"
      m_I0_bus1n = "m:I0:bus1n", "kA, Zero-Sequence Current, Magnitude"
      m_I0x3_bus1 = "m:I0x3:bus1", "kA, 3*I0"
      m_I0x3_bus1n = "m:I0x3:bus1n", "kA, 3*I0"
      m_phii0_bus1 = "m:phii0:bus1", "deg, Zero-Sequence Current, Angle"
      m_phii0_bus1n = "m:phii0:bus1n", "deg, Zero-Sequence Current, Angle"
      m_i1r_bus1 = "m:i1r:bus1", "p.u., Positive-Sequence Current, Real Part"
      m_i1r_bus1n = "m:i1r:bus1n", "p.u., Positive-Sequence Current, Real Part"
      m_i1i_bus1 = "m:i1i:bus1", "p.u., Positive-Sequence Current, Imaginary Part"
      m_i1i_bus1n = "m:i1i:bus1n", "p.u., Positive-Sequence Current, Imaginary Part"
      m_i1_bus1 = "m:i1:bus1", "p.u., Positive-Sequence Current, Magnitude"
      m_i1_bus1n = "m:i1:bus1n", "p.u., Positive-Sequence Current, Magnitude"
      m_I1_bus1 = "m:I1:bus1", "kA, Positive-Sequence Current, Magnitude"
      m_I1_bus1n = "m:I1:bus1n", "kA, Positive-Sequence Current, Magnitude"
      m_phii1_bus1 = "m:phii1:bus1", "deg, Positive-Sequence Current, Angle"
      m_phii1_bus1n = "m:phii1:bus1n", "deg, Positive-Sequence Current, Angle"
      m_i2r_bus1 = "m:i2r:bus1", "p.u., Negative-Sequence Current, Real Part"
      m_i2r_bus1n = "m:i2r:bus1n", "p.u., Negative-Sequence Current, Real Part"
      m_i2i_bus1 = "m:i2i:bus1", "p.u., Negative-Sequence Current, Imaginary Part"
      m_i2i_bus1n = "m:i2i:bus1n", "p.u., Negative-Sequence Current, Imaginary Part"
      m_i2_bus1 = "m:i2:bus1", "p.u., Negative-Sequence Current, Magnitude"
      m_i2_bus1n = "m:i2:bus1n", "p.u., Negative-Sequence Current, Magnitude"
      m_I2_bus1 = "m:I2:bus1", "kA, Negative-Sequence Current, Magnitude"
      m_I2_bus1n = "m:I2:bus1n", "kA, Negative-Sequence Current, Magnitude"
      m_phii2_bus1 = "m:phii2:bus1", "deg, Negative-Sequence Current, Angle"
      m_phii2_bus1n = "m:phii2:bus1n", "deg, Negative-Sequence Current, Angle"
      m_ubfacI_bus1 = "m:ubfacI:bus1", "%, Current Unbalance Factor"
      m_ubfacI_bus1n = "m:ubfacI:bus1n", "%, Current Unbalance Factor"
      m_ubfacS_bus1 = "m:ubfacS:bus1", "%, Power Unbalance Factor"
      m_ubfacS_bus1n = "m:ubfacS:bus1n", "%, Power Unbalance Factor"
      m_P_bus1_A = "m:P:bus1:A", "MW, Active Power"
      m_P_bus1_B = "m:P:bus1:B", "MW, Active Power"
      m_P_bus1_C = "m:P:bus1:C", "MW, Active Power"
      m_P_bus1_N = "m:P:bus1:N", "MW, Active Power"
      m_P_bus1n = "m:P:bus1n", "MW, Active Power"
      m_Q_bus1_A = "m:Q:bus1:A", "Mvar, Reactive Power"
      m_Q_bus1_B = "m:Q:bus1:B", "Mvar, Reactive Power"
      m_Q_bus1_C = "m:Q:bus1:C", "Mvar, Reactive Power"
      m_Q_bus1_N = "m:Q:bus1:N", "Mvar, Reactive Power"
      m_Q_bus1n = "m:Q:bus1n", "Mvar, Reactive Power"
      m_S_bus1_A = "m:S:bus1:A", "MVA, Apparent Power"
      m_S_bus1_B = "m:S:bus1:B", "MVA, Apparent Power"
      m_S_bus1_C = "m:S:bus1:C", "MVA, Apparent Power"
      m_S_bus1_N = "m:S:bus1:N", "MVA, Apparent Power"
      m_S_bus1n = "m:S:bus1n", "MVA, Apparent Power"
      m_cosphi_bus1_A = "m:cosphi:bus1:A", "Power Factor"
      m_cosphi_bus1_B = "m:cosphi:bus1:B", "Power Factor"
      m_cosphi_bus1_C = "m:cosphi:bus1:C", "Power Factor"
      m_cosphi_bus1_N = "m:cosphi:bus1:N", "Power Factor"
      m_cosphi_bus1n = "m:cosphi:bus1n", "Power Factor"
      m_tanphi_bus1_A = "m:tanphi:bus1:A", "tan(phi)"
      m_tanphi_bus1_B = "m:tanphi:bus1:B", "tan(phi)"
      m_tanphi_bus1_C = "m:tanphi:bus1:C", "tan(phi)"
      m_tanphi_bus1_N = "m:tanphi:bus1:N", "tan(phi)"
      m_tanphi_bus1n = "m:tanphi:bus1n", "tan(phi)"
      m_Psum_bus1 = "m:Psum:bus1", "MW, Total Active Power"
      m_Psum_bus1n = "m:Psum:bus1n", "MW, Total Active Power"
      m_Qsum_bus1 = "m:Qsum:bus1", "Mvar, Total Reactive Power"
      m_Qsum_bus1n = "m:Qsum:bus1n", "Mvar, Total Reactive Power"
      m_Ssum_bus1 = "m:Ssum:bus1", "MVA, Total Apparent Power"
      m_Ssum_bus1n = "m:Ssum:bus1n", "MVA, Total Apparent Power"
      m_cosphisum_bus1 = "m:cosphisum:bus1", "Total Power Factor"
      m_cosphisum_bus1n = "m:cosphisum:bus1n", "Total Power Factor"
      m_tanphisum_bus1 = "m:tanphisum:bus1", "Total tan(phi)"
      m_tanphisum_bus1n = "m:tanphisum:bus1n", "Total tan(phi)"
      m_Pln_bus1_A = "m:Pln:bus1:A", "MW, Active Power (Line-Neutral)"
      m_Pln_bus1_B = "m:Pln:bus1:B", "MW, Active Power (Line-Neutral)"
      m_Pln_bus1_C = "m:Pln:bus1:C", "MW, Active Power (Line-Neutral)"
      m_Pln_bus1_N = "m:Pln:bus1:N", "MW, Active Power (Line-Neutral)"
      m_Pln_bus1n = "m:Pln:bus1n", "MW, Active Power (Line-Neutral)"
      m_Qln_bus1_A = "m:Qln:bus1:A", "Mvar, Reactive Power (Line-Neutral)"
      m_Qln_bus1_B = "m:Qln:bus1:B", "Mvar, Reactive Power (Line-Neutral)"
      m_Qln_bus1_C = "m:Qln:bus1:C", "Mvar, Reactive Power (Line-Neutral)"
      m_Qln_bus1_N = "m:Qln:bus1:N", "Mvar, Reactive Power (Line-Neutral)"
      m_Qln_bus1n = "m:Qln:bus1n", "Mvar, Reactive Power (Line-Neutral)"
      m_P1_bus1 = "m:P1:bus1", "MW, Positive Sequence Active Power"
      m_P1_bus1n = "m:P1:bus1n", "MW, Positive Sequence Active Power"
      m_Q1_bus1 = "m:Q1:bus1", "Mvar, Positive Sequence Reactive Power"
      m_Q1_bus1n = "m:Q1:bus1n", "Mvar, Positive Sequence Reactive Power"
      m_P2_bus1 = "m:P2:bus1", "MW, Negative Sequence Active Power"
      m_P2_bus1n = "m:P2:bus1n", "MW, Negative Sequence Active Power"
      m_Q2_bus1 = "m:Q2:bus1", "Mvar, Negative Sequence Reactive Power"
      m_Q2_bus1n = "m:Q2:bus1n", "Mvar, Negative Sequence Reactive Power"
      m_P0_bus1 = "m:P0:bus1", "MW, Zero Sequence Active Power"
      m_P0_bus1n = "m:P0:bus1n", "MW, Zero Sequence Active Power"
      m_Q0_bus1 = "m:Q0:bus1", "Mvar, Zero Sequence Reactive Power"
      m_Q0_bus1n = "m:Q0:bus1n", "Mvar, Zero Sequence Reactive Power"
      m_i1P_bus1 = "m:i1P:bus1", "p.u., Positive-Sequence Active Current"
      m_i1P_bus1n = "m:i1P:bus1n", "p.u., Positive-Sequence Active Current"
      m_i1Q_bus1 = "m:i1Q:bus1", "p.u., Positive-Sequence Reactive Current"
      m_i1Q_bus1n = "m:i1Q:bus1n", "p.u., Positive-Sequence Reactive Current"
      m_I1P_bus1 = "m:I1P:bus1", "kA, Positive-Sequence Active Current"
      m_I1P_bus1n = "m:I1P:bus1n", "kA, Positive-Sequence Active Current"
      m_I1Q_bus1 = "m:I1Q:bus1", "kA, Positive-Sequence Reactive Current"
      m_I1Q_bus1n = "m:I1Q:bus1n", "kA, Positive-Sequence Reactive Current"
      m_i2P_bus1 = "m:i2P:bus1", "p.u., Negative-Sequence Active Current"
      m_i2P_bus1n = "m:i2P:bus1n", "p.u., Negative-Sequence Active Current"
      m_i2Q_bus1 = "m:i2Q:bus1", "p.u., Negative-Sequence Reactive Current"
      m_i2Q_bus1n = "m:i2Q:bus1n", "p.u., Negative-Sequence Reactive Current"
      m_I2P_bus1 = "m:I2P:bus1", "kA, Negative-Sequence Active Current"
      m_I2P_bus1n = "m:I2P:bus1n", "kA, Negative-Sequence Active Current"
      m_I2Q_bus1 = "m:I2Q:bus1", "kA, Negative-Sequence Reactive Current"
      m_I2Q_bus1n = "m:I2Q:bus1n", "kA, Negative-Sequence Reactive Current"
      m_ur_bus1_A = "m:ur:bus1:A", "p.u., Phase Voltage, Real Part"
      m_ur_bus1_B = "m:ur:bus1:B", "p.u., Phase Voltage, Real Part"
      m_ur_bus1_C = "m:ur:bus1:C", "p.u., Phase Voltage, Real Part"
      m_ur_bus1_N = "m:ur:bus1:N", "p.u., Phase Voltage, Real Part"
      m_ur_bus1n = "m:ur:bus1n", "p.u., Phase Voltage, Real Part"
      m_ui_bus1_A = "m:ui:bus1:A", "p.u., Phase Voltage, Imaginary Part"
      m_ui_bus1_B = "m:ui:bus1:B", "p.u., Phase Voltage, Imaginary Part"
      m_ui_bus1_C = "m:ui:bus1:C", "p.u., Phase Voltage, Imaginary Part"
      m_ui_bus1_N = "m:ui:bus1:N", "p.u., Phase Voltage, Imaginary Part"
      m_ui_bus1n = "m:ui:bus1n", "p.u., Phase Voltage, Imaginary Part"
      m_u_bus1_A = "m:u:bus1:A", "p.u., Phase Voltage, Magnitude"
      m_u_bus1_B = "m:u:bus1:B", "p.u., Phase Voltage, Magnitude"
      m_u_bus1_C = "m:u:bus1:C", "p.u., Phase Voltage, Magnitude"
      m_u_bus1_N = "m:u:bus1:N", "p.u., Phase Voltage, Magnitude"
      m_u_bus1n = "m:u:bus1n", "p.u., Phase Voltage, Magnitude"
      m_ir_bus1_A = "m:ir:bus1:A", "p.u., Phase Current, Real Part"
      m_ir_bus1_B = "m:ir:bus1:B", "p.u., Phase Current, Real Part"
      m_ir_bus1_C = "m:ir:bus1:C", "p.u., Phase Current, Real Part"
      m_ir_bus1_N = "m:ir:bus1:N", "p.u., Phase Current, Real Part"
      m_ir_bus1n = "m:ir:bus1n", "p.u., Phase Current, Real Part"
      m_ii_bus1_A = "m:ii:bus1:A", "p.u., Phase Current, Imaginary Part"
      m_ii_bus1_B = "m:ii:bus1:B", "p.u., Phase Current, Imaginary Part"
      m_ii_bus1_C = "m:ii:bus1:C", "p.u., Phase Current, Imaginary Part"
      m_ii_bus1_N = "m:ii:bus1:N", "p.u., Phase Current, Imaginary Part"
      m_ii_bus1n = "m:ii:bus1n", "p.u., Phase Current, Imaginary Part"
      m_i_bus1_A = "m:i:bus1:A", "p.u., Phase Current, Magnitude"
      m_i_bus1_B = "m:i:bus1:B", "p.u., Phase Current, Magnitude"
      m_i_bus1_C = "m:i:bus1:C", "p.u., Phase Current, Magnitude"
      m_i_bus1_N = "m:i:bus1:N", "p.u., Phase Current, Magnitude"
      m_i_bus1n = "m:i:bus1n", "p.u., Phase Current, Magnitude"
      m_phiui_bus1_A = "m:phiui:bus1:A", "deg, Angle between Voltage and Current"
      m_phiui_bus1_B = "m:phiui:bus1:B", "deg, Angle between Voltage and Current"
      m_phiui_bus1_C = "m:phiui:bus1:C", "deg, Angle between Voltage and Current"
      m_phiui_bus1_N = "m:phiui:bus1:N", "deg, Angle between Voltage and Current"
      m_phiui_bus1n = "m:phiui:bus1n", "deg, Angle between Voltage and Current"
      m_phiu0i0_bus1 = "m:phiu0i0:bus1", "deg, Angle between Voltage and Current in zero-sequence system"
      m_phiu0i0_bus1n = "m:phiu0i0:bus1n", "deg, Angle between Voltage and Current in zero-sequence system"
      m_phiu1i1_bus1 = "m:phiu1i1:bus1", "deg, Angle between Voltage and Current in positive sequence system"
      m_phiu1i1_bus1n = "m:phiu1i1:bus1n", "deg, Angle between Voltage and Current in positive sequence system"
      m_phiu2i2_bus1 = "m:phiu2i2:bus1", "deg, Angle between Voltage and Current in negative sequence system"
      m_phiu2i2_bus1n = "m:phiu2i2:bus1n", "deg, Angle between Voltage and Current in negative sequence system"
      m_Tfct_bus1 = "m:Tfct:bus1", "s, Fault Clearing Time"
      m_Tfct_bus1n = "m:Tfct:bus1n", "s, Fault Clearing Time"
      m_TfctPh_bus1_A = "m:TfctPh:bus1:A", "s, Fault Clearing Time"
      m_TfctPh_bus1_B = "m:TfctPh:bus1:B", "s, Fault Clearing Time"
      m_TfctPh_bus1_C = "m:TfctPh:bus1:C", "s, Fault Clearing Time"
      m_TfctPh_bus1_N = "m:TfctPh:bus1:N", "s, Fault Clearing Time"
      m_TfctPh_bus1n = "m:TfctPh:bus1n", "s, Fault Clearing Time"
      m_Brkload_bus1 = "m:Brkload:bus1", "%, Max. Breaker Loading"
      m_Brkload_bus1n = "m:Brkload:bus1n", "%, Max. Breaker Loading"
      m_BrkloadPh_bus1_A = "m:BrkloadPh:bus1:A", "%, Breaker Loading"
      m_BrkloadPh_bus1_B = "m:BrkloadPh:bus1:B", "%, Breaker Loading"
      m_BrkloadPh_bus1_C = "m:BrkloadPh:bus1:C", "%, Breaker Loading"
      m_BrkloadPh_bus1_N = "m:BrkloadPh:bus1:N", "%, Breaker Loading"
      m_BrkloadPh_bus1n = "m:BrkloadPh:bus1n", "%, Breaker Loading"
      n_ur_bus1_A = "n:ur:bus1:A", "p.u., Line-Ground Voltage, Real Part"
      n_ur_bus1_B = "n:ur:bus1:B", "p.u., Line-Ground Voltage, Real Part"
      n_ur_bus1_C = "n:ur:bus1:C", "p.u., Line-Ground Voltage, Real Part"
      n_ur_bus1_N = "n:ur:bus1:N", "p.u., Line-Ground Voltage, Real Part"
      n_ur_bus1n = "n:ur:bus1n", "p.u., Line-Ground Voltage, Real Part"
      n_ui_bus1_A = "n:ui:bus1:A", "p.u., Line-Ground Voltage, Imaginary Part"
      n_ui_bus1_B = "n:ui:bus1:B", "p.u., Line-Ground Voltage, Imaginary Part"
      n_ui_bus1_C = "n:ui:bus1:C", "p.u., Line-Ground Voltage, Imaginary Part"
      n_ui_bus1_N = "n:ui:bus1:N", "p.u., Line-Ground Voltage, Imaginary Part"
      n_ui_bus1n = "n:ui:bus1n", "p.u., Line-Ground Voltage, Imaginary Part"
      n_u_bus1_A = "n:u:bus1:A", "p.u., Line-Ground Voltage, Magnitude"
      n_u_bus1_B = "n:u:bus1:B", "p.u., Line-Ground Voltage, Magnitude"
      n_u_bus1_C = "n:u:bus1:C", "p.u., Line-Ground Voltage, Magnitude"
      n_u_bus1_N = "n:u:bus1:N", "p.u., Line-Ground Voltage, Magnitude"
      n_u_bus1n = "n:u:bus1n", "p.u., Line-Ground Voltage, Magnitude"
      n_upc_bus1_A = "n:upc:bus1:A", "%, Line-Ground Voltage, Magnitude"
      n_upc_bus1_B = "n:upc:bus1:B", "%, Line-Ground Voltage, Magnitude"
      n_upc_bus1_C = "n:upc:bus1:C", "%, Line-Ground Voltage, Magnitude"
      n_upc_bus1_N = "n:upc:bus1:N", "%, Line-Ground Voltage, Magnitude"
      n_upc_bus1n = "n:upc:bus1n", "%, Line-Ground Voltage, Magnitude"
      n_U_bus1_A = "n:U:bus1:A", "kV, Line-Ground Voltage, Magnitude"
      n_U_bus1_B = "n:U:bus1:B", "kV, Line-Ground Voltage, Magnitude"
      n_U_bus1_C = "n:U:bus1:C", "kV, Line-Ground Voltage, Magnitude"
      n_U_bus1_N = "n:U:bus1:N", "kV, Line-Ground Voltage, Magnitude"
      n_U_bus1n = "n:U:bus1n", "kV, Line-Ground Voltage, Magnitude"
      n_phiu_bus1_A = "n:phiu:bus1:A", "deg, Line-Ground Voltage, Angle"
      n_phiu_bus1_B = "n:phiu:bus1:B", "deg, Line-Ground Voltage, Angle"
      n_phiu_bus1_C = "n:phiu:bus1:C", "deg, Line-Ground Voltage, Angle"
      n_phiu_bus1_N = "n:phiu:bus1:N", "deg, Line-Ground Voltage, Angle"
      n_phiu_bus1n = "n:phiu:bus1n", "deg, Line-Ground Voltage, Angle"
      n_phiurel_bus1 = "n:phiurel:bus1", "deg, Voltage, Relative Angle"
      n_phiurel_bus1n = "n:phiurel:bus1n", "deg, Voltage, Relative Angle"
      n_ul_bus1_A = "n:ul:bus1:A", "p.u., Line-Line Voltage, Magnitude"
      n_ul_bus1_B = "n:ul:bus1:B", "p.u., Line-Line Voltage, Magnitude"
      n_ul_bus1_C = "n:ul:bus1:C", "p.u., Line-Line Voltage, Magnitude"
      n_ul_bus1_N = "n:ul:bus1:N", "p.u., Line-Line Voltage, Magnitude"
      n_ul_bus1n = "n:ul:bus1n", "p.u., Line-Line Voltage, Magnitude"
      n_ulpc_bus1_A = "n:ulpc:bus1:A", "%, Line-Line Voltage, Magnitude"
      n_ulpc_bus1_B = "n:ulpc:bus1:B", "%, Line-Line Voltage, Magnitude"
      n_ulpc_bus1_C = "n:ulpc:bus1:C", "%, Line-Line Voltage, Magnitude"
      n_ulpc_bus1_N = "n:ulpc:bus1:N", "%, Line-Line Voltage, Magnitude"
      n_ulpc_bus1n = "n:ulpc:bus1n", "%, Line-Line Voltage, Magnitude"
      n_Ul_bus1_A = "n:Ul:bus1:A", "kV, Line-Line Voltage, Magnitude"
      n_Ul_bus1_B = "n:Ul:bus1:B", "kV, Line-Line Voltage, Magnitude"
      n_Ul_bus1_C = "n:Ul:bus1:C", "kV, Line-Line Voltage, Magnitude"
      n_Ul_bus1_N = "n:Ul:bus1:N", "kV, Line-Line Voltage, Magnitude"
      n_Ul_bus1n = "n:Ul:bus1n", "kV, Line-Line Voltage, Magnitude"
      n_phiul_bus1_A = "n:phiul:bus1:A", "deg, Line-Line Voltage, Angle"
      n_phiul_bus1_B = "n:phiul:bus1:B", "deg, Line-Line Voltage, Angle"
      n_phiul_bus1_C = "n:phiul:bus1:C", "deg, Line-Line Voltage, Angle"
      n_phiul_bus1_N = "n:phiul:bus1:N", "deg, Line-Line Voltage, Angle"
      n_phiul_bus1n = "n:phiul:bus1n", "deg, Line-Line Voltage, Angle"
      n_u0_bus1 = "n:u0:bus1", "p.u., Zero-Sequence Voltage, Magnitude"
      n_u0_bus1n = "n:u0:bus1n", "p.u., Zero-Sequence Voltage, Magnitude"
      n_U0_bus1 = "n:U0:bus1", "kV, Zero-Sequence Voltage, Magnitude"
      n_U0_bus1n = "n:U0:bus1n", "kV, Zero-Sequence Voltage, Magnitude"
      n_U0x3_bus1 = "n:U0x3:bus1", "kV, 3*U0"
      n_U0x3_bus1n = "n:U0x3:bus1n", "kV, 3*U0"
      n_phiu0_bus1 = "n:phiu0:bus1", "deg, Zero-Sequence Voltage, Angle"
      n_phiu0_bus1n = "n:phiu0:bus1n", "deg, Zero-Sequence Voltage, Angle"
      n_u1_bus1 = "n:u1:bus1", "p.u., Positive-Sequence Voltage, Magnitude"
      n_u1_bus1n = "n:u1:bus1n", "p.u., Positive-Sequence Voltage, Magnitude"
      n_u1pc_bus1 = "n:u1pc:bus1", "%, Positive-Sequence Voltage, Magnitude"
      n_u1pc_bus1n = "n:u1pc:bus1n", "%, Positive-Sequence Voltage, Magnitude"
      n_u1r_bus1 = "n:u1r:bus1", "p.u., Positive-Sequence Voltage, Real Part"
      n_u1r_bus1n = "n:u1r:bus1n", "p.u., Positive-Sequence Voltage, Real Part"
      n_u1i_bus1 = "n:u1i:bus1", "p.u., Positive-Sequence Voltage, Imaginary Part"
      n_u1i_bus1n = "n:u1i:bus1n", "p.u., Positive-Sequence Voltage, Imaginary Part"
      n_U1_bus1 = "n:U1:bus1", "kV, Line-Ground Positive-Sequence Voltage, Magnitude"
      n_U1_bus1n = "n:U1:bus1n", "kV, Line-Ground Positive-Sequence Voltage, Magnitude"
      n_phiu1_bus1 = "n:phiu1:bus1", "deg, Positive-Sequence Voltage, Angle"
      n_phiu1_bus1n = "n:phiu1:bus1n", "deg, Positive-Sequence Voltage, Angle"
      n_u2_bus1 = "n:u2:bus1", "p.u., Negative-Sequence Voltage, Magnitude"
      n_u2_bus1n = "n:u2:bus1n", "p.u., Negative-Sequence Voltage, Magnitude"
      n_U2_bus1 = "n:U2:bus1", "kV, Line-Ground Negative-Sequence Voltage, Magnitude"
      n_U2_bus1n = "n:U2:bus1n", "kV, Line-Ground Negative-Sequence Voltage, Magnitude"
      n_phiu2_bus1 = "n:phiu2:bus1", "deg, Negative-Sequence Voltage, Angle"
      n_phiu2_bus1n = "n:phiu2:bus1n", "deg, Negative-Sequence Voltage, Angle"
      n_U1l_bus1 = "n:U1l:bus1", "kV, Line-Line Positive-Sequence Voltage, Magnitude"
      n_U1l_bus1n = "n:U1l:bus1n", "kV, Line-Line Positive-Sequence Voltage, Magnitude"
      n_U2l_bus1 = "n:U2l:bus1", "kV, Line-Line Negative-Sequence Voltage, Magnitude"
      n_U2l_bus1n = "n:U2l:bus1n", "kV, Line-Line Negative-Sequence Voltage, Magnitude"
      n_uln_bus1_A = "n:uln:bus1:A", "p.u., Line-Neutral Voltage, Magnitude"
      n_uln_bus1_B = "n:uln:bus1:B", "p.u., Line-Neutral Voltage, Magnitude"
      n_uln_bus1_C = "n:uln:bus1:C", "p.u., Line-Neutral Voltage, Magnitude"
      n_uln_bus1_N = "n:uln:bus1:N", "p.u., Line-Neutral Voltage, Magnitude"
      n_uln_bus1n = "n:uln:bus1n", "p.u., Line-Neutral Voltage, Magnitude"
      n_Uln_bus1_A = "n:Uln:bus1:A", "kV, Line-Neutral Voltage, Magnitude"
      n_Uln_bus1_B = "n:Uln:bus1:B", "kV, Line-Neutral Voltage, Magnitude"
      n_Uln_bus1_C = "n:Uln:bus1:C", "kV, Line-Neutral Voltage, Magnitude"
      n_Uln_bus1_N = "n:Uln:bus1:N", "kV, Line-Neutral Voltage, Magnitude"
      n_Uln_bus1n = "n:Uln:bus1n", "kV, Line-Neutral Voltage, Magnitude"
      n_phiuln_bus1_A = "n:phiuln:bus1:A", "deg, Line-Neutral Voltage, Angle"
      n_phiuln_bus1_B = "n:phiuln:bus1:B", "deg, Line-Neutral Voltage, Angle"
      n_phiuln_bus1_C = "n:phiuln:bus1:C", "deg, Line-Neutral Voltage, Angle"
      n_phiuln_bus1_N = "n:phiuln:bus1:N", "deg, Line-Neutral Voltage, Angle"
      n_phiuln_bus1n = "n:phiuln:bus1n", "deg, Line-Neutral Voltage, Angle"
      n_un_bus1 = "n:un:bus1", "p.u., Neutral-Ground Voltage, Magnitude"
      n_un_bus1n = "n:un:bus1n", "p.u., Neutral-Ground Voltage, Magnitude"
      n_Un_bus1 = "n:Un:bus1", "kV, Neutral-Ground Voltage, Magnitude"
      n_Un_bus1n = "n:Un:bus1n", "kV, Neutral-Ground Voltage, Magnitude"
      n_phiun_bus1 = "n:phiun:bus1", "deg, Neutral-Ground Voltage, Angle"
      n_phiun_bus1n = "n:phiun:bus1n", "deg, Neutral-Ground Voltage, Angle"
      n_upht_bus1_A = "n:upht:bus1:A", "p.u., Phase Technology dependent Voltage, Magnitude"
      n_upht_bus1_B = "n:upht:bus1:B", "p.u., Phase Technology dependent Voltage, Magnitude"
      n_upht_bus1_C = "n:upht:bus1:C", "p.u., Phase Technology dependent Voltage, Magnitude"
      n_upht_bus1_N = "n:upht:bus1:N", "p.u., Phase Technology dependent Voltage, Magnitude"
      n_upht_bus1n = "n:upht:bus1n", "p.u., Phase Technology dependent Voltage, Magnitude"
      n_Upht_bus1_A = "n:Upht:bus1:A", "kV, Phase Technology dependent Voltage, Magnitude"
      n_Upht_bus1_B = "n:Upht:bus1:B", "kV, Phase Technology dependent Voltage, Magnitude"
      n_Upht_bus1_C = "n:Upht:bus1:C", "kV, Phase Technology dependent Voltage, Magnitude"
      n_Upht_bus1_N = "n:Upht:bus1:N", "kV, Phase Technology dependent Voltage, Magnitude"
      n_Upht_bus1n = "n:Upht:bus1n", "kV, Phase Technology dependent Voltage, Magnitude"
      n_um_bus1 = "n:um:bus1", "p.u., Average-Voltage, Magnitude"
      n_um_bus1n = "n:um:bus1n", "p.u., Average-Voltage, Magnitude"
      n_Um_bus1 = "n:Um:bus1", "kV, Average-Voltage, Magnitude"
      n_Um_bus1n = "n:Um:bus1n", "kV, Average-Voltage, Magnitude"
      n_uphtmin_bus1 = "n:uphtmin:bus1", "p.u., Minimum of Phase Technology dependent Voltage, Magnitude"
      n_uphtmin_bus1n = "n:uphtmin:bus1n", "p.u., Minimum of Phase Technology dependent Voltage, Magnitude"
      n_uphtmax_bus1 = "n:uphtmax:bus1", "p.u., Maximum of Phase Technology dependent Voltage, Magnitude"
      n_uphtmax_bus1n = "n:uphtmax:bus1n", "p.u., Maximum of Phase Technology dependent Voltage, Magnitude"
      n_Pgen_bus1 = "n:Pgen:bus1", "MW, Generation, Active Power"
      n_Pgen_bus1n = "n:Pgen:bus1n", "MW, Generation, Active Power"
      n_Qgen_bus1 = "n:Qgen:bus1", "Mvar, Generation, Reactive Power"
      n_Qgen_bus1n = "n:Qgen:bus1n", "Mvar, Generation, Reactive Power"
      n_Pmot_bus1 = "n:Pmot:bus1", "MW, Motor Load, Active Power"
      n_Pmot_bus1n = "n:Pmot:bus1n", "MW, Motor Load, Active Power"
      n_Qmot_bus1 = "n:Qmot:bus1", "Mvar, Motor Load, Reactive Power"
      n_Qmot_bus1n = "n:Qmot:bus1n", "Mvar, Motor Load, Reactive Power"
      n_Pload_bus1 = "n:Pload:bus1", "MW, General Load, Active Power"
      n_Pload_bus1n = "n:Pload:bus1n", "MW, General Load, Active Power"
      n_Qload_bus1 = "n:Qload:bus1", "Mvar, General Load, Reactive Power"
      n_Qload_bus1n = "n:Qload:bus1n", "Mvar, General Load, Reactive Power"
      n_Pcomp_bus1 = "n:Pcomp:bus1", "MW, Compensation (Losses)"
      n_Pcomp_bus1n = "n:Pcomp:bus1n", "MW, Compensation (Losses)"
      n_Qcomp_bus1 = "n:Qcomp:bus1", "Mvar, Compensation"
      n_Qcomp_bus1n = "n:Qcomp:bus1n", "Mvar, Compensation"
      n_Pnet_bus1 = "n:Pnet:bus1", "MW, External Networks, Active Power"
      n_Pnet_bus1n = "n:Pnet:bus1n", "MW, External Networks, Active Power"
      n_Qnet_bus1 = "n:Qnet:bus1", "Mvar, External Networks, Reactive Power"
      n_Qnet_bus1n = "n:Qnet:bus1n", "Mvar, External Networks, Reactive Power"
      n_Pflow_bus1 = "n:Pflow:bus1", "MW, Power Flow, Active Power"
      n_Pflow_bus1n = "n:Pflow:bus1n", "MW, Power Flow, Active Power"
      n_Qflow_bus1 = "n:Qflow:bus1", "Mvar, Power Flow, Reactive Power"
      n_Qflow_bus1n = "n:Qflow:bus1n", "Mvar, Power Flow, Reactive Power"
      n_Pout_bus1 = "n:Pout:bus1", "MW, Outgoing Flow, Active Power"
      n_Pout_bus1n = "n:Pout:bus1n", "MW, Outgoing Flow, Active Power"
      n_Qout_bus1 = "n:Qout:bus1", "Mvar, Outgoing Power, Reactive Power"
      n_Qout_bus1n = "n:Qout:bus1n", "Mvar, Outgoing Power, Reactive Power"
      n_Sout_bus1 = "n:Sout:bus1", "MVA, Outgoing Power, Apparent Power"
      n_Sout_bus1n = "n:Sout:bus1n", "MVA, Outgoing Power, Apparent Power"
      n_cosphiout_bus1 = "n:cosphiout:bus1", "Outgoing Power, Power Factor"
      n_cosphiout_bus1n = "n:cosphiout:bus1n", "Outgoing Power, Power Factor"
      n_Pbalance_bus1 = "n:Pbalance:bus1", "MW, Active Power Balance (=0)"
      n_Pbalance_bus1n = "n:Pbalance:bus1n", "MW, Active Power Balance (=0)"
      n_Qbalance_bus1 = "n:Qbalance:bus1", "Mvar, Reactive Power Balance (=0)"
      n_Qbalance_bus1n = "n:Qbalance:bus1n", "Mvar, Reactive Power Balance (=0)"
      n_ubfac_bus1 = "n:ubfac:bus1", "%, Unbalance Factor"
      n_ubfac_bus1n = "n:ubfac:bus1n", "%, Unbalance Factor"
      n_LossPdown_bus1 = "n:LossPdown:bus1", "MW, Losses, downstream"
      n_LossPdown_bus1n = "n:LossPdown:bus1n", "MW, Losses, downstream"
      n_LossQdown_bus1 = "n:LossQdown:bus1", "Mvar, Losses, downstream (Reactive Power)"
      n_LossQdown_bus1n = "n:LossQdown:bus1n", "Mvar, Losses, downstream (Reactive Power)"
      n_LossPdownload_bus1 = "n:LossPdownload:bus1", "MW, Load losses, downstream"
      n_LossPdownload_bus1n = "n:LossPdownload:bus1n", "MW, Load losses, downstream"
      n_LossQdownload_bus1 = "n:LossQdownload:bus1", "Mvar, Load losses, downstream"
      n_LossQdownload_bus1n = "n:LossQdownload:bus1n", "Mvar, Load losses, downstream"
      n_LossPdownnoload_bus1 = "n:LossPdownnoload:bus1", "MW, No load losses, downstream"
      n_LossPdownnoload_bus1n = "n:LossPdownnoload:bus1n", "MW, No load losses, downstream"
      n_LossQdownnoload_bus1 = "n:LossQdownnoload:bus1", "Mvar, No load losses, downstream"
      n_LossQdownnoload_bus1n = "n:LossQdownnoload:bus1n", "Mvar, No load losses, downstream"
      s_ncapain = "s:ncapain", "IN, Number of Stages in Parallel, input, continuous, double"
      c_z_r = "c:z:r", "p.u., Shunt-Impedance, Real Part"
      c_z_i = "c:z:i", "p.u., Shunt-Impedance, Imaginary Part"
      c_z_phi = "c:z:phi", "deg, Shunt-Impedance, Angle"
      c_z_mag = "c:z:mag", "p.u., Shunt-Impedance, Magnitude"
      c_z_phirad = "c:z:phirad", "rad, Shunt-Impedance, Angle"
      c_cn = "c:cn", "Connection Mode"
      c_frnom = "c:frnom", "Hz, Nominal Frequency"
      c_hpi = "c:hpi", "rad/s, Nominal Angular Frequency"
      c_y0_r = "c:y0:r", "p.u., Zero Sequence Admittance, Real Part"
      c_y0_i = "c:y0:i", "p.u., Zero Sequence Admittance, Imaginary Part"
      c_y0_phi = "c:y0:phi", "deg, Zero Sequence Admittance, Angle"
      c_y0_mag = "c:y0:mag", "p.u., Zero Sequence Admittance, Magnitude"
      c_y0_phirad = "c:y0:phirad", "rad, Zero Sequence Admittance, Angle"
      c_ze_r = "c:ze:r", "p.u., Grounding Impedance, Real Part"
      c_ze_i = "c:ze:i", "p.u., Grounding Impedance, Imaginary Part"
      c_ze_phi = "c:ze:phi", "deg, Grounding Impedance, Angle"
      c_ze_mag = "c:ze:mag", "p.u., Grounding Impedance, Magnitude"
      c_ze_phirad = "c:ze:phirad", "rad, Grounding Impedance, Angle"
      c_z0_r = "c:z0:r", "p.u., Zero Sequence Impedance, Real Part"
      c_z0_i = "c:z0:i", "p.u., Zero Sequence Impedance, Imaginary Part"
      c_z0_phi = "c:z0:phi", "deg, Zero Sequence Impedance, Angle"
      c_z0_mag = "c:z0:mag", "p.u., Zero Sequence Impedance, Magnitude"
      c_z0_phirad = "c:z0:phirad", "rad, Zero Sequence Impedance, Angle"
      c_b0 = "c:b0", "p.u., Susceptance to Ground"
      c_ucph_A = "c:ucph:A", "p.u., Voltage across Capacitor (Phase Value)"
      c_ucph_B = "c:ucph:B", "p.u., Voltage across Capacitor (Phase Value)"
      c_ucph_C = "c:ucph:C", "p.u., Voltage across Capacitor (Phase Value)"
      c_ucph_N = "c:ucph:N", "p.u., Voltage across Capacitor (Phase Value)"
      c_urlph_A = "c:urlph:A", "p.u., Voltage across Inductor (Phase Value)"
      c_urlph_B = "c:urlph:B", "p.u., Voltage across Inductor (Phase Value)"
      c_urlph_C = "c:urlph:C", "p.u., Voltage across Inductor (Phase Value)"
      c_urlph_N = "c:urlph:N", "p.u., Voltage across Inductor (Phase Value)"
      c_ILph_A = "c:ILph:A", "A, Current through Inductor L (Phase Value)"
      c_ILph_B = "c:ILph:B", "A, Current through Inductor L (Phase Value)"
      c_ILph_C = "c:ILph:C", "A, Current through Inductor L (Phase Value)"
      c_ILph_N = "c:ILph:N", "A, Current through Inductor L (Phase Value)"
      c_ICph_A = "c:ICph:A", "A, Current through Capacitor C (Phase Value)"
      c_ICph_B = "c:ICph:B", "A, Current through Capacitor C (Phase Value)"
      c_ICph_C = "c:ICph:C", "A, Current through Capacitor C (Phase Value)"
      c_ICph_N = "c:ICph:N", "A, Current through Capacitor C (Phase Value)"
      c_IRpph_A = "c:IRpph:A", "A, Current through Rp (Phase Value)"
      c_IRpph_B = "c:IRpph:B", "A, Current through Rp (Phase Value)"
      c_IRpph_C = "c:IRpph:C", "A, Current through Rp (Phase Value)"
      c_IRpph_N = "c:IRpph:N", "A, Current through Rp (Phase Value)"
      c_uc = "c:uc", "p.u., Voltage across Capacitor (max abc)"
      c_url = "c:url", "p.u., Voltage across Inductor (max abc)"
      c_IL = "c:IL", "A, Current through Inductor (max abc)"
      c_IC = "c:IC", "A, Current through Capacitor C (max abc)"
      c_IRp = "c:IRp", "A, Current through Rp (max abc)"
      c_PRp = "c:PRp", "kW, Losses in Rp"
      c_PL = "c:PL", "kW, Losses in Inductor (Rs)"
      c_Ustar = "c:Ustar", "kV, Star-Point Voltage"
      c_Ucph_A = "c:Ucph:A", "kV, Voltage across Capacitor (Phase Value)"
      c_Ucph_B = "c:Ucph:B", "kV, Voltage across Capacitor (Phase Value)"
      c_Ucph_C = "c:Ucph:C", "kV, Voltage across Capacitor (Phase Value)"
      c_Ucph_N = "c:Ucph:N", "kV, Voltage across Capacitor (Phase Value)"
      c_Urlph_A = "c:Urlph:A", "kV, Voltage across Inductor (Phase Value)"
      c_Urlph_B = "c:Urlph:B", "kV, Voltage across Inductor (Phase Value)"
      c_Urlph_C = "c:Urlph:C", "kV, Voltage across Inductor (Phase Value)"
      c_Urlph_N = "c:Urlph:N", "kV, Voltage across Inductor (Phase Value)"
      c_Uc = "c:Uc", "kV, Voltage across Capacitor (max abc)"
      c_Url = "c:Url", "kV, Voltage across Inductor (max abc)"
      c_ncapa = "c:ncapa", "Number of Stages in Parallel"
      c_acost = "c:acost", "Annual Cost"
      c_ncapx = "c:ncapx", "Max. No. of Steps"
      c_nCtrlIter = "c:nCtrlIter", "No. of CTRL-iterations"
      c_Tctrl = "c:Tctrl", "Controller Time Constant"
      c_Kctrl = "c:Kctrl", "p.u./%, Controller Sensitivity dq/dv"
      c_Ploss = "c:Ploss", "MW, Losses (total)"
      c_Qloss = "c:Qloss", "Mvar, Reactive-Losses (total)"
      c_Plossld = "c:Plossld", "MW, Losses (load)"
      c_Qlossld = "c:Qlossld", "Mvar, Reactive-Losses (load)"
      c_Plossnld = "c:Plossnld", "MW, Losses (no load)"
      c_Qlossnld = "c:Qlossnld", "Mvar, Reactive-Losses (no load)"
      e_ncapx = "e:ncapx", "Max. No. of Steps"
      e_ncapa = "e:ncapa", "Act.No. of Step"
      e_Qmax = "e:Qmax", "Mvar, Max. Rated Reactive Power"
      e_Qact = "e:Qact", "Mvar, Actual Reactive Power"
      e_c_pctrl = "e:c_pctrl", "ElmShntctrl, Shunt Controller"
      e_qdslCtrl = "e:qdslCtrl", "Quasi-Dynamic Model"
      e_iswitch = "e:iswitch", "Switchable"
      e_i_cont = "e:i_cont", "Tap Changer"
      e_imldc = "e:imldc", "Control Mode"
      e_ilcph = "e:ilcph", "Phase"
      e_iQorient = "e:iQorient", "Orientation"
      e_uset_mode = "e:uset_mode", "Setpoint"
      e_i_rem = "e:i_rem", "Remote Control"
      e_p_rem = "e:p_rem", "StaBar,ElmTerm, Controlled Node"
      e_p_cub = "e:p_cub", "StaCubic,ElmBoundary, Q Controlled at"
      e_cpCtrlNode = "e:cpCtrlNode", "Target Node"
      e_usetp_mx = "e:usetp_mx", "p.u., Upper Voltage Limit"
      e_usetp_mn = "e:usetp_mn", "p.u., Lower Voltage Limit"
      e_qsetp_mx = "e:qsetp_mx", "Mvar, Upper Reactive Power Limit"
      e_qsetp_mn = "e:qsetp_mn", "Mvar, Lower Reactive Power Limit"
      e_pfsetp_mx = "e:pfsetp_mx", "Upper Power Factor Limit"
      e_pf_recap_mx = "e:pf_recap_mx", "Power Factor"
      e_pfsetp_mn = "e:pfsetp_mn", "Lower Power Factor Limit"
      e_pf_recap_mn = "e:pf_recap_mn", "Power Factor"
      e_Tctrl = "e:Tctrl", "s, Controller Time Constant"
      e_Kctrl = "e:Kctrl", "p.u./%, Controller Sensitivity dq/dv"
      e_cpFeed = "e:cpFeed", "Feeder"
      e_ciDist = "e:ciDist", "Distance from infeed in number of buses"
      e_ciLater = "e:ciLater", "Lateral Index"
      e_ciDistRoot = "e:ciDistRoot", "Distance from first infeed in number of buses"
      e_ciDistAll = "e:ciDistAll", "Distance from infeed in number of buses including switches"
      e_ciDistAllRoot = "e:ciDistAllRoot", "Distance from first infeed in number of buses including switches"
      r_c_pctrl = "r:c_pctrl", "ElmShntctrl, Shunt Controller"
      r_qdslCtrl = "r:qdslCtrl", "Quasi-Dynamic Model"
      r_p_rem = "r:p_rem", "StaBar,ElmTerm, Controlled Node"
      r_p_cub = "r:p_cub", "StaCubic,ElmBoundary, Q Controlled at"
      r_cpCtrlNode = "r:cpCtrlNode", "Target Node"
      r_cpFeed = "r:cpFeed", "Feeder"
      m_Ikss_bus1 = "m:Ikss:bus1", "kA, Initial Short-Circuit Current"
      m_ikss_bus1 = "m:ikss:bus1", "p.u., Initial Short-Circuit Current, Magnitude"
      m_phii_bus1 = "m:phii:bus1", "deg, Phase Current, Angle"
      m_maxIkss_bus1 = "m:maxIkss:bus1", "kA, Max. Initial Short-Circuit Current"
      m_Skss_bus1 = "m:Skss:bus1", "MVA, Initial Short-Circuit Power"
      m_ip_bus1 = "m:ip:bus1", "kA, Peak Short-Circuit Current"
      m_maxip_bus1 = "m:maxip:bus1", "kA, Max. Peak Short-Circuit Current"
      m_maxIb_bus1 = "m:maxIb:bus1", "kA, Max. Breaking Current"
      m_maxIth_bus1 = "m:maxIth:bus1", "kA, Max. Thermal equivalent Short-Circuit Current"
      m_Ib_bus1 = "m:Ib:bus1", "kA, Short-Circuit Breaking Current"
      m_Sb_bus1 = "m:Sb:bus1", "MVA, Short-Circuit Breaking Power"
      m_Ik_bus1 = "m:Ik:bus1", "kA, Steady-State Short-Circuit Current"
      m_Ith_bus1 = "m:Ith:bus1", "kA, Thermal equivalent Short-Circuit Current"
      m_I_bus1 = "m:I:bus1", "kA, Short-Circuit Current"
      m_R_bus1 = "m:R:bus1", "Ohm, Fault Impedance, Real Part"
      m_X_bus1 = "m:X:bus1", "Ohm, Fault Impedance, Imaginary Part"
      m_Z_bus1 = "m:Z:bus1", "Ohm, Fault Impedance, Magnitude"
      m_phiz_bus1 = "m:phiz:bus1", "deg, Fault Impedance, Angle"
      m_zSbase_bus1 = "m:zSbase:bus1", "p.u., Fault Impedance, Magnitude"
      m_XtoR_bus1 = "m:XtoR:bus1", "X/R ratio"
      m_Zs_bus1 = "m:Zs:bus1", "Ohm, Source Impedance, Magnitude"
      m_phizs_bus1 = "m:phizs:bus1", "deg, Source Impedance, Angle"
      m_Rs_bus1 = "m:Rs:bus1", "Ohm, Source Impedance, Real Part"
      m_Xs_bus1 = "m:Xs:bus1", "Ohm, Source Impedance, Imaginary Part"
      m_phiui_bus1 = "m:phiui:bus1", "deg, Angle between Voltage and Current"
      m_cosphiui_bus1 = "m:cosphiui:bus1", "cosphi"
      m_P_bus1 = "m:P:bus1", "MW, Active Power"
      m_Q_bus1 = "m:Q:bus1", "Mvar, Reactive Power"
      m_BrkIpload_bus1 = "m:BrkIpload:bus1", "%, Breaker Loading (peak)"
      m_BrkIbload_bus1 = "m:BrkIbload:bus1", "%, Breaker Loading (breaking)"
      m_BrkmaxIb_bus1 = "m:BrkmaxIb:bus1", "kA, Max. Breaking Current (breaker time)"
      n_Ikss_bus1 = "n:Ikss:bus1", "kA, Initial Short-Circuit Current"
      n_I_bus1 = "n:I:bus1", "kA, Short-Circuit Current"
      n_phii_bus1 = "n:phii:bus1", "deg, Phase Current, Angle"
      n_Skss_bus1 = "n:Skss:bus1", "MVA, Initial Short-Circuit Power"
      n_ip_bus1 = "n:ip:bus1", "kA, Peak Short-Circuit Current"
      n_Ib_bus1 = "n:Ib:bus1", "kA, Short-Circuit Breaking Current"
      n_Sb_bus1 = "n:Sb:bus1", "MVA, Short-Circuit Breaking Power"
      n_Ik_bus1 = "n:Ik:bus1", "kA, Steady-State Short-Circuit Current"
      n_Ith_bus1 = "n:Ith:bus1", "kA, Thermal equivalent Short-Circuit Current"
      n_R_bus1 = "n:R:bus1", "Ohm, Short-Circuit Impedance, Real Part"
      n_X_bus1 = "n:X:bus1", "Ohm, Short-Circuit Impedance, Imaginary Part"
      n_Z_bus1 = "n:Z:bus1", "Ohm, Short-Circuit Impedance, Magnitude"
      n_phiz_bus1 = "n:phiz:bus1", "deg, Short-Circuit Impedance, Angle"
      n_rSbase_bus1 = "n:rSbase:bus1", "p.u., Short-Circuit Impedance, Real Part"
      n_xSbase_bus1 = "n:xSbase:bus1", "p.u., Short-Circuit Impedance, Imaginary Part"
      n_zSbase_bus1 = "n:zSbase:bus1", "p.u., Short-Circuit Impedance, Magnitude"
      n_XtoR_bus1 = "n:XtoR:bus1", "X/R ratio (peak)"
      n_RtoX_bus1 = "n:RtoX:bus1", "R/X ratio (peak)"
      n_ur_bus1 = "n:ur:bus1", "p.u., Voltage, Real Part"
      n_ui_bus1 = "n:ui:bus1", "p.u., Voltage, Imaginary Part"
      n_u_bus1 = "n:u:bus1", "p.u., Voltage, Magnitude"
      n_upc_bus1 = "n:upc:bus1", "%, Voltage, Magnitude"
      n_U_bus1 = "n:U:bus1", "kV, Line-Ground Voltage, Magnitude"
      n_Ul_bus1 = "n:Ul:bus1", "kV, Line-Line Voltage, Magnitude"
      n_phiu_bus1 = "n:phiu:bus1", "deg, Voltage, Angle"
      n_du_bus1 = "n:du:bus1", "%, Voltage Deviation"
      n_urpref_bus1 = "n:urpref:bus1", "p.u., Pre-fault Voltage, Real Part"
      n_uipref_bus1 = "n:uipref:bus1", "p.u., Pre-fault Voltage, Imaginary Part"
      n_upref_bus1 = "n:upref:bus1", "p.u., Pre-fault Voltage, Magnitude"
      n_uprefpc_bus1 = "n:uprefpc:bus1", "%, Pre-fault Voltage, Magnitude"
      n_Upref_bus1 = "n:Upref:bus1", "kV, Line-Ground Pre-fault Voltage, Magnitude"
      n_Ulpref_bus1 = "n:Ulpref:bus1", "kV, Line-Line Pre-fault Voltage, Magnitude"
      n_phiupref_bus1 = "n:phiupref:bus1", "deg, Pre-fault Voltage, Angle"
      n_phiui_bus1 = "n:phiui:bus1", "deg, Angle between Voltage and Current"
      n_cfac_bus1 = "n:cfac:bus1", "Voltage Factor"
      n_idc_bus1 = "n:idc:bus1", "kA, Decaying Component"
      n_Iasy_bus1 = "n:Iasy:bus1", "kA, Asymmetrical Breaking Current"
      n_Ithload_bus1 = "n:Ithload:bus1", "%, Loading, Thermal equivalent Current"
      n_Ipload_bus1 = "n:Ipload:bus1", "%, Loading, Peak Short-Circuit Current"
      n_Ithrtk_bus1 = "n:Ithrtk:bus1", "kA, Rated Short-Time Current (Tk)"
      n_Tn_bus1 = "n:Tn:bus1", "ms, Grid Time Constant"
      n_XtoR_b_bus1 = "n:XtoR_b:bus1", "X/R ratio (dc)"
      n_RtoX_b_bus1 = "n:RtoX_b:bus1", "R/X ratio (dc)"
      n_fctof_b_bus1 = "n:fctof_b:bus1", "Equivalent Frequency fc/f (dc)"
      c_rs = "c:rs", "p.u., Rs"
      c_gp = "c:gp", "p.u., Gp"
      c_xl = "c:xl", "p.u., Xl"
      c_bc = "c:bc", "p.u., Bc"
      c_gpc = "c:gpc", "p.u., Gp(C)"
      c_uha_c = "c:uha_c", "p.u., Harmonic Voltage (arith.) across Capacitor"
      c_uhrms_c = "c:uhrms_c", "p.u., Harmonic Voltage (RMS) across Capacitor"
      c_ut_c = "c:ut_c", "p.u., Total Voltage (u_fn+uhrms) across Capacitor"
      c_uha_rl = "c:uha_rl", "p.u., Harmonic Voltage (arith.) across Inductor (Rs-L)"
      c_uhrms_rl = "c:uhrms_rl", "p.u., Harmonic Voltage (rms) across Inductor (Rs-L)"
      c_ut_rl = "c:ut_rl", "p.u., Total Voltage (u_fn+uhrms) across Inductor (Rs-L)"
      c_Ihrms_L = "c:Ihrms_L", "A, Harmonic Current (rms) through Inductor (Rs-L)"
      c_Irms_L = "c:Irms_L", "A, RMS Value of Current through Inductor (Rs-L)"
      c_Ihrms_Rp = "c:Ihrms_Rp", "A, Harmonic Current (rms) through Rp"
      c_Irms_Rp = "c:Irms_Rp", "A, RMS Value of Current through Rp"
      c_Ph_Rp = "c:Ph_Rp", "kW, Harmonic Losses of Rp"
      c_Pt_Rp = "c:Pt_Rp", "kW, Total Losses of Rp"
      c_Ph_L = "c:Ph_L", "kW, Harmonic Losses of Inductor (Rs)"
      c_Pt_L = "c:Pt_L", "kW, Total Losses of Inductor"
      c_ufn_c = "c:ufn_c", "p.u., Fundamental Frequency Voltage across Capacitor"
      c_ufn_rl = "c:ufn_rl", "p.u., Fundamental Frequency Voltage across Inductor (Rs-L)"
      c_Ifn_Rp = "c:Ifn_Rp", "A, Fundamental Frequency Current through Rp"
      c_Ifn_L = "c:Ifn_L", "A, Fundamental Frequency Current through Inductor (Rs-L)"
      c_Pfn_L = "c:Pfn_L", "kW, Fundamental Frequency Losses of Inductor"
      c_Pfn_Rp = "c:Pfn_Rp", "kW, Fundamental Frequency Losses of Rp"
      c_Ifn_c = "c:Ifn_c", "A, Fundamental Frequency Current through Capacitor"
      c_Ihrms_c = "c:Ihrms_c", "A, Harmonic Current (rms) through Capacitor"
      c_Irms_c = "c:Irms_c", "A, RMS Value of Current through Capacitor"
      c_C = "c:C", "uF, Capacitance C"
      c_L = "c:L", "mH, Inductance L"
      c_Rp = "c:Rp", "Ohm, Parallel-Resistance Rp"
      c_Rs = "c:Rs", "Ohm, Resistance Rs"
      c_Uha_c = "c:Uha_c", "kV, Harmonic Voltage (arith.) across Capacitor"
      c_Uhrms_c = "c:Uhrms_c", "kV, Harmonic Voltage (RMS) across Capacitor"
      c_Ut_c = "c:Ut_c", "kV, Total Voltage (U_fn+Uhrms) across Capacitor"
      c_Uha_rl = "c:Uha_rl", "kV, Harmonic Voltage (arith.) across Inductor (Rs-L)"
      c_Uhrms_rl = "c:Uhrms_rl", "kV, Harmonic Voltage (rms) across Inductor (Rs-L)"
      c_Ut_rl = "c:Ut_rl", "kV, Total Voltage (U_fn+Uhrms) across Inductor (Rs-L)"
      c_Ufn_c = "c:Ufn_c", "kV, Fundamental Frequency Voltage across Capacitor"
      c_Ufn_rl = "c:Ufn_rl", "kV, Fundamental Frequency Voltage across Inductor (Rs-L)"
      m_Ikss_bus1_A = "m:Ikss:bus1:A", "kA, Initial Short-Circuit Current"
      m_Ikss_bus1_B = "m:Ikss:bus1:B", "kA, Initial Short-Circuit Current"
      m_Ikss_bus1_C = "m:Ikss:bus1:C", "kA, Initial Short-Circuit Current"
      m_Ikss_bus1_N = "m:Ikss:bus1:N", "kA, Initial Short-Circuit Current"
      m_Ikss_bus1n = "m:Ikss:bus1n", "kA, Initial Short-Circuit Current"
      m_ikss_bus1_A = "m:ikss:bus1:A", "p.u., Initial Short-Circuit Current, Magnitude"
      m_ikss_bus1_B = "m:ikss:bus1:B", "p.u., Initial Short-Circuit Current, Magnitude"
      m_ikss_bus1_C = "m:ikss:bus1:C", "p.u., Initial Short-Circuit Current, Magnitude"
      m_ikss_bus1_N = "m:ikss:bus1:N", "p.u., Initial Short-Circuit Current, Magnitude"
      m_ikss_bus1n = "m:ikss:bus1n", "p.u., Initial Short-Circuit Current, Magnitude"
      m_maxIkss_bus1n = "m:maxIkss:bus1n", "kA, Max. Initial Short-Circuit Current"
      m_Skss_bus1_A = "m:Skss:bus1:A", "MVA, Initial Short-Circuit Power"
      m_Skss_bus1_B = "m:Skss:bus1:B", "MVA, Initial Short-Circuit Power"
      m_Skss_bus1_C = "m:Skss:bus1:C", "MVA, Initial Short-Circuit Power"
      m_Skss_bus1_N = "m:Skss:bus1:N", "MVA, Initial Short-Circuit Power"
      m_Skss_bus1n = "m:Skss:bus1n", "MVA, Initial Short-Circuit Power"
      m_Skssx3_bus1_A = "m:Skssx3:bus1:A", "MVA, 3*Initial Short-Circuit Power"
      m_Skssx3_bus1_B = "m:Skssx3:bus1:B", "MVA, 3*Initial Short-Circuit Power"
      m_Skssx3_bus1_C = "m:Skssx3:bus1:C", "MVA, 3*Initial Short-Circuit Power"
      m_Skssx3_bus1_N = "m:Skssx3:bus1:N", "MVA, 3*Initial Short-Circuit Power"
      m_Skssx3_bus1n = "m:Skssx3:bus1n", "MVA, 3*Initial Short-Circuit Power"
      m_ip_bus1_A = "m:ip:bus1:A", "kA, Peak Short-Circuit Current"
      m_ip_bus1_B = "m:ip:bus1:B", "kA, Peak Short-Circuit Current"
      m_ip_bus1_C = "m:ip:bus1:C", "kA, Peak Short-Circuit Current"
      m_ip_bus1_N = "m:ip:bus1:N", "kA, Peak Short-Circuit Current"
      m_ip_bus1n = "m:ip:bus1n", "kA, Peak Short-Circuit Current"
      m_maxip_bus1n = "m:maxip:bus1n", "kA, Max. Peak Short-Circuit Current"
      m_maxIth_bus1n = "m:maxIth:bus1n", "kA, Max. Thermal equivalent Short-Circuit Current"
      m_Z_bus1_A = "m:Z:bus1:A", "Ohm, Fault Impedance, Magnitude"
      m_Z_bus1_B = "m:Z:bus1:B", "Ohm, Fault Impedance, Magnitude"
      m_Z_bus1_C = "m:Z:bus1:C", "Ohm, Fault Impedance, Magnitude"
      m_Z_bus1_N = "m:Z:bus1:N", "Ohm, Fault Impedance, Magnitude"
      m_Z_bus1n = "m:Z:bus1n", "Ohm, Fault Impedance, Magnitude"
      m_phiz_bus1_A = "m:phiz:bus1:A", "deg, Fault Impedance, Angle"
      m_phiz_bus1_B = "m:phiz:bus1:B", "deg, Fault Impedance, Angle"
      m_phiz_bus1_C = "m:phiz:bus1:C", "deg, Fault Impedance, Angle"
      m_phiz_bus1_N = "m:phiz:bus1:N", "deg, Fault Impedance, Angle"
      m_phiz_bus1n = "m:phiz:bus1n", "deg, Fault Impedance, Angle"
      m_Zl_bus1_A = "m:Zl:bus1:A", "Ohm, Line-Line Fault Impedance, Magnitude"
      m_Zl_bus1_B = "m:Zl:bus1:B", "Ohm, Line-Line Fault Impedance, Magnitude"
      m_Zl_bus1_C = "m:Zl:bus1:C", "Ohm, Line-Line Fault Impedance, Magnitude"
      m_Zl_bus1_N = "m:Zl:bus1:N", "Ohm, Line-Line Fault Impedance, Magnitude"
      m_Zl_bus1n = "m:Zl:bus1n", "Ohm, Line-Line Fault Impedance, Magnitude"
      m_phizl_bus1_A = "m:phizl:bus1:A", "deg, Line-Line Fault Impedance, Angle"
      m_phizl_bus1_B = "m:phizl:bus1:B", "deg, Line-Line Fault Impedance, Angle"
      m_phizl_bus1_C = "m:phizl:bus1:C", "deg, Line-Line Fault Impedance, Angle"
      m_phizl_bus1_N = "m:phizl:bus1:N", "deg, Line-Line Fault Impedance, Angle"
      m_phizl_bus1n = "m:phizl:bus1n", "deg, Line-Line Fault Impedance, Angle"
      m_Z0_bus1 = "m:Z0:bus1", "Ohm, Zero-Sequence Fault Impedance, Magnitude"
      m_Z0_bus1n = "m:Z0:bus1n", "Ohm, Zero-Sequence Fault Impedance, Magnitude"
      m_phiz0_bus1 = "m:phiz0:bus1", "deg, Zero-Sequence Fault Impedance, Angle"
      m_phiz0_bus1n = "m:phiz0:bus1n", "deg, Zero-Sequence Fault Impedance, Angle"
      m_z0Sbase_bus1 = "m:z0Sbase:bus1", "p.u., Zero-Sequence Fault Impedance, Magnitude"
      m_z0Sbase_bus1n = "m:z0Sbase:bus1n", "p.u., Zero-Sequence Fault Impedance, Magnitude"
      m_cosphiu1i1_bus1 = "m:cosphiu1i1:bus1", "Positive-Sequence cosphi"
      m_cosphiu1i1_bus1n = "m:cosphiu1i1:bus1n", "Positive-Sequence cosphi"
      m_I0x3watt_bus1 = "m:I0x3watt:bus1", "kA, Earth-Fault Residual Current"
      m_I0x3watt_bus1n = "m:I0x3watt:bus1n", "kA, Earth-Fault Residual Current"
      m_I0x3var_bus1 = "m:I0x3var:bus1", "kA, Capacitive Earth-Fault Current"
      m_I0x3var_bus1n = "m:I0x3var:bus1n", "kA, Capacitive Earth-Fault Current"
      m_Q0cap_bus1 = "m:Q0cap:bus1", "Mvar, Q0 capacitive"
      m_Q0cap_bus1n = "m:Q0cap:bus1n", "Mvar, Q0 capacitive"
      m_Zfrel_bus1 = "m:Zfrel:bus1", "Ohm, Relay Impedance, Magnitude"
      m_Zfrel_bus1n = "m:Zfrel:bus1n", "Ohm, Relay Impedance, Magnitude"
      m_phizfrel_bus1 = "m:phizfrel:bus1", "deg, Relay Impedance, Angle"
      m_phizfrel_bus1n = "m:phizfrel:bus1n", "deg, Relay Impedance, Angle"
      m_Rfrel_bus1 = "m:Rfrel:bus1", "Ohm, Relay Impedance, Real Part"
      m_Rfrel_bus1n = "m:Rfrel:bus1n", "Ohm, Relay Impedance, Real Part"
      m_Xfrel_bus1 = "m:Xfrel:bus1", "Ohm, Relay Impedance, Imaginary Part"
      m_Xfrel_bus1n = "m:Xfrel:bus1n", "Ohm, Relay Impedance, Imaginary Part"
      m_Zs_bus1n = "m:Zs:bus1n", "Ohm, Source Impedance, Magnitude"
      m_phizs_bus1n = "m:phizs:bus1n", "deg, Source Impedance, Angle"
      m_Rs_bus1n = "m:Rs:bus1n", "Ohm, Source Impedance, Real Part"
      m_Xs_bus1n = "m:Xs:bus1n", "Ohm, Source Impedance, Imaginary Part"
      m_Zf_bus1 = "m:Zf:bus1", "Ohm, Meas. Fault Impedance, Magnitude"
      m_Zf_bus1n = "m:Zf:bus1n", "Ohm, Meas. Fault Impedance, Magnitude"
      m_phizf_bus1 = "m:phizf:bus1", "deg, Meas. Fault Impedance, Angle"
      m_phizf_bus1n = "m:phizf:bus1n", "deg, Meas. Fault Impedance, Angle"
      m_BrkIpload_bus1n = "m:BrkIpload:bus1n", "%, Max. Breaker Loading (peak)"
      n_Ikss_bus1_A = "n:Ikss:bus1:A", "kA, Initial Short-Circuit Current"
      n_Ikss_bus1_B = "n:Ikss:bus1:B", "kA, Initial Short-Circuit Current"
      n_Ikss_bus1_C = "n:Ikss:bus1:C", "kA, Initial Short-Circuit Current"
      n_Ikss_bus1_N = "n:Ikss:bus1:N", "kA, Initial Short-Circuit Current"
      n_Ikss_bus1n = "n:Ikss:bus1n", "kA, Initial Short-Circuit Current"
      n_I_bus1_A = "n:I:bus1:A", "kA, Short-Circuit Current"
      n_I_bus1_B = "n:I:bus1:B", "kA, Short-Circuit Current"
      n_I_bus1_C = "n:I:bus1:C", "kA, Short-Circuit Current"
      n_I_bus1_N = "n:I:bus1:N", "kA, Short-Circuit Current"
      n_I_bus1n = "n:I:bus1n", "kA, Short-Circuit Current"
      n_phii_bus1_A = "n:phii:bus1:A", "deg, Phase Current, Angle"
      n_phii_bus1_B = "n:phii:bus1:B", "deg, Phase Current, Angle"
      n_phii_bus1_C = "n:phii:bus1:C", "deg, Phase Current, Angle"
      n_phii_bus1_N = "n:phii:bus1:N", "deg, Phase Current, Angle"
      n_phii_bus1n = "n:phii:bus1n", "deg, Phase Current, Angle"
      n_Skss_bus1_A = "n:Skss:bus1:A", "MVA, Initial Short-Circuit Power"
      n_Skss_bus1_B = "n:Skss:bus1:B", "MVA, Initial Short-Circuit Power"
      n_Skss_bus1_C = "n:Skss:bus1:C", "MVA, Initial Short-Circuit Power"
      n_Skss_bus1_N = "n:Skss:bus1:N", "MVA, Initial Short-Circuit Power"
      n_Skss_bus1n = "n:Skss:bus1n", "MVA, Initial Short-Circuit Power"
      n_Skssx3_bus1_A = "n:Skssx3:bus1:A", "MVA, 3*Initial Short-Circuit Power"
      n_Skssx3_bus1_B = "n:Skssx3:bus1:B", "MVA, 3*Initial Short-Circuit Power"
      n_Skssx3_bus1_C = "n:Skssx3:bus1:C", "MVA, 3*Initial Short-Circuit Power"
      n_Skssx3_bus1_N = "n:Skssx3:bus1:N", "MVA, 3*Initial Short-Circuit Power"
      n_Skssx3_bus1n = "n:Skssx3:bus1n", "MVA, 3*Initial Short-Circuit Power"
      n_ip_bus1_A = "n:ip:bus1:A", "kA, Peak Short-Circuit Current"
      n_ip_bus1_B = "n:ip:bus1:B", "kA, Peak Short-Circuit Current"
      n_ip_bus1_C = "n:ip:bus1:C", "kA, Peak Short-Circuit Current"
      n_ip_bus1_N = "n:ip:bus1:N", "kA, Peak Short-Circuit Current"
      n_ip_bus1n = "n:ip:bus1n", "kA, Peak Short-Circuit Current"
      n_Ib_bus1_A = "n:Ib:bus1:A", "kA, Short-Circuit Breaking Current"
      n_Ib_bus1_B = "n:Ib:bus1:B", "kA, Short-Circuit Breaking Current"
      n_Ib_bus1_C = "n:Ib:bus1:C", "kA, Short-Circuit Breaking Current"
      n_Ib_bus1_N = "n:Ib:bus1:N", "kA, Short-Circuit Breaking Current"
      n_Ib_bus1n = "n:Ib:bus1n", "kA, Short-Circuit Breaking Current"
      n_Sb_bus1_A = "n:Sb:bus1:A", "MVA, Short-Circuit Breaking Power"
      n_Sb_bus1_B = "n:Sb:bus1:B", "MVA, Short-Circuit Breaking Power"
      n_Sb_bus1_C = "n:Sb:bus1:C", "MVA, Short-Circuit Breaking Power"
      n_Sb_bus1_N = "n:Sb:bus1:N", "MVA, Short-Circuit Breaking Power"
      n_Sb_bus1n = "n:Sb:bus1n", "MVA, Short-Circuit Breaking Power"
      n_Ith_bus1_A = "n:Ith:bus1:A", "kA, Thermal equivalent Short-Circuit Current"
      n_Ith_bus1_B = "n:Ith:bus1:B", "kA, Thermal equivalent Short-Circuit Current"
      n_Ith_bus1_C = "n:Ith:bus1:C", "kA, Thermal equivalent Short-Circuit Current"
      n_Ith_bus1_N = "n:Ith:bus1:N", "kA, Thermal equivalent Short-Circuit Current"
      n_Ith_bus1n = "n:Ith:bus1n", "kA, Thermal equivalent Short-Circuit Current"
      n_Ithmax_bus1 = "n:Ithmax:bus1", "kA, Max. Thermal equivalent Short-Circuit Current"
      n_Ithmax_bus1n = "n:Ithmax:bus1n", "kA, Max. Thermal equivalent Short-Circuit Current"
      n_Ik_bus1_A = "n:Ik:bus1:A", "kA, Steady-State Short-Circuit Current"
      n_Ik_bus1_B = "n:Ik:bus1:B", "kA, Steady-State Short-Circuit Current"
      n_Ik_bus1_C = "n:Ik:bus1:C", "kA, Steady-State Short-Circuit Current"
      n_Ik_bus1_N = "n:Ik:bus1:N", "kA, Steady-State Short-Circuit Current"
      n_Ik_bus1n = "n:Ik:bus1n", "kA, Steady-State Short-Circuit Current"
      n_R0_bus1 = "n:R0:bus1", "Ohm, Zero-Sequence Impedance, Real Part"
      n_R0_bus1n = "n:R0:bus1n", "Ohm, Zero-Sequence Impedance, Real Part"
      n_X0_bus1 = "n:X0:bus1", "Ohm, Zero-Sequence Impedance, Imaginary Part"
      n_X0_bus1n = "n:X0:bus1n", "Ohm, Zero-Sequence Impedance, Imaginary Part"
      n_Z0_bus1 = "n:Z0:bus1", "Ohm, Zero-Sequence Impedance, Magnitude"
      n_Z0_bus1n = "n:Z0:bus1n", "Ohm, Zero-Sequence Impedance, Magnitude"
      n_phiz0_bus1 = "n:phiz0:bus1", "deg, Zero-Sequence Impedance, Angle"
      n_phiz0_bus1n = "n:phiz0:bus1n", "deg, Zero-Sequence Impedance, Angle"
      n_r0Sbase_bus1 = "n:r0Sbase:bus1", "p.u., Zero-Sequence Impedance, Real Part"
      n_r0Sbase_bus1n = "n:r0Sbase:bus1n", "p.u., Zero-Sequence Impedance, Real Part"
      n_x0Sbase_bus1 = "n:x0Sbase:bus1", "p.u., Zero-Sequence Impedance, Imaginary Part"
      n_x0Sbase_bus1n = "n:x0Sbase:bus1n", "p.u., Zero-Sequence Impedance, Imaginary Part"
      n_z0Sbase_bus1 = "n:z0Sbase:bus1", "p.u., Zero-Sequence Impedance, Magnitude"
      n_z0Sbase_bus1n = "n:z0Sbase:bus1n", "p.u., Zero-Sequence Impedance, Magnitude"
      n_R1_bus1 = "n:R1:bus1", "Ohm, Positive-Sequence Impedance, Real Part"
      n_R1_bus1n = "n:R1:bus1n", "Ohm, Positive-Sequence Impedance, Real Part"
      n_X1_bus1 = "n:X1:bus1", "Ohm, Positive-Sequence Impedance, Imaginary Part"
      n_X1_bus1n = "n:X1:bus1n", "Ohm, Positive-Sequence Impedance, Imaginary Part"
      n_Z1_bus1 = "n:Z1:bus1", "Ohm, Positive-Sequence Impedance, Magnitude"
      n_Z1_bus1n = "n:Z1:bus1n", "Ohm, Positive-Sequence Impedance, Magnitude"
      n_phiz1_bus1 = "n:phiz1:bus1", "deg, Positive-Sequence Impedance, Angle"
      n_phiz1_bus1n = "n:phiz1:bus1n", "deg, Positive-Sequence Impedance, Angle"
      n_r1Sbase_bus1 = "n:r1Sbase:bus1", "p.u., Positive-Sequence Impedance, Real Part"
      n_r1Sbase_bus1n = "n:r1Sbase:bus1n", "p.u., Positive-Sequence Impedance, Real Part"
      n_x1Sbase_bus1 = "n:x1Sbase:bus1", "p.u., Positive-Sequence Impedance, Imaginary Part"
      n_x1Sbase_bus1n = "n:x1Sbase:bus1n", "p.u., Positive-Sequence Impedance, Imaginary Part"
      n_z1Sbase_bus1 = "n:z1Sbase:bus1", "p.u., Positive-Sequence Impedance, Magnitude"
      n_z1Sbase_bus1n = "n:z1Sbase:bus1n", "p.u., Positive-Sequence Impedance, Magnitude"
      n_R2_bus1 = "n:R2:bus1", "Ohm, Negative-Sequence Impedance, Real Part"
      n_R2_bus1n = "n:R2:bus1n", "Ohm, Negative-Sequence Impedance, Real Part"
      n_X2_bus1 = "n:X2:bus1", "Ohm, Negative-Sequence Impedance, Imaginary Part"
      n_X2_bus1n = "n:X2:bus1n", "Ohm, Negative-Sequence Impedance, Imaginary Part"
      n_Z2_bus1 = "n:Z2:bus1", "Ohm, Negative-Sequence Impedance, Magnitude"
      n_Z2_bus1n = "n:Z2:bus1n", "Ohm, Negative-Sequence Impedance, Magnitude"
      n_phiz2_bus1 = "n:phiz2:bus1", "deg, Negative-Sequence Impedance, Angle"
      n_phiz2_bus1n = "n:phiz2:bus1n", "deg, Negative-Sequence Impedance, Angle"
      n_r2Sbase_bus1 = "n:r2Sbase:bus1", "p.u., Negative-Sequence Impedance, Real Part"
      n_r2Sbase_bus1n = "n:r2Sbase:bus1n", "p.u., Negative-Sequence Impedance, Real Part"
      n_x2Sbase_bus1 = "n:x2Sbase:bus1", "p.u., Negative-Sequence Impedance, Imaginary Part"
      n_x2Sbase_bus1n = "n:x2Sbase:bus1n", "p.u., Negative-Sequence Impedance, Imaginary Part"
      n_z2Sbase_bus1 = "n:z2Sbase:bus1", "p.u., Negative-Sequence Impedance, Magnitude"
      n_z2Sbase_bus1n = "n:z2Sbase:bus1n", "p.u., Negative-Sequence Impedance, Magnitude"
      n_R_bus1_A = "n:R:bus1:A", "Ohm, Network Resistance, Line-Ground"
      n_R_bus1_B = "n:R:bus1:B", "Ohm, Network Resistance, Line-Ground"
      n_R_bus1_C = "n:R:bus1:C", "Ohm, Network Resistance, Line-Ground"
      n_R_bus1_N = "n:R:bus1:N", "Ohm, Network Resistance, Line-Ground"
      n_R_bus1n = "n:R:bus1n", "Ohm, Network Resistance, Line-Ground"
      n_X_bus1_A = "n:X:bus1:A", "Ohm, Network Reactance, Line-Ground"
      n_X_bus1_B = "n:X:bus1:B", "Ohm, Network Reactance, Line-Ground"
      n_X_bus1_C = "n:X:bus1:C", "Ohm, Network Reactance, Line-Ground"
      n_X_bus1_N = "n:X:bus1:N", "Ohm, Network Reactance, Line-Ground"
      n_X_bus1n = "n:X:bus1n", "Ohm, Network Reactance, Line-Ground"
      n_Z_bus1_A = "n:Z:bus1:A", "Ohm, Network Impedance, Line-Ground"
      n_Z_bus1_B = "n:Z:bus1:B", "Ohm, Network Impedance, Line-Ground"
      n_Z_bus1_C = "n:Z:bus1:C", "Ohm, Network Impedance, Line-Ground"
      n_Z_bus1_N = "n:Z:bus1:N", "Ohm, Network Impedance, Line-Ground"
      n_Z_bus1n = "n:Z:bus1n", "Ohm, Network Impedance, Line-Ground"
      n_phiz_bus1_A = "n:phiz:bus1:A", "deg, Angle of Network Impedance, Line-Ground"
      n_phiz_bus1_B = "n:phiz:bus1:B", "deg, Angle of Network Impedance, Line-Ground"
      n_phiz_bus1_C = "n:phiz:bus1:C", "deg, Angle of Network Impedance, Line-Ground"
      n_phiz_bus1_N = "n:phiz:bus1:N", "deg, Angle of Network Impedance, Line-Ground"
      n_phiz_bus1n = "n:phiz:bus1n", "deg, Angle of Network Impedance, Line-Ground"
      n_Rl_bus1_A = "n:Rl:bus1:A", "Ohm, Network Resistance, Line-Line"
      n_Rl_bus1_B = "n:Rl:bus1:B", "Ohm, Network Resistance, Line-Line"
      n_Rl_bus1_C = "n:Rl:bus1:C", "Ohm, Network Resistance, Line-Line"
      n_Rl_bus1_N = "n:Rl:bus1:N", "Ohm, Network Resistance, Line-Line"
      n_Rl_bus1n = "n:Rl:bus1n", "Ohm, Network Resistance, Line-Line"
      n_Xl_bus1_A = "n:Xl:bus1:A", "Ohm, Network Reactance, Line-Line"
      n_Xl_bus1_B = "n:Xl:bus1:B", "Ohm, Network Reactance, Line-Line"
      n_Xl_bus1_C = "n:Xl:bus1:C", "Ohm, Network Reactance, Line-Line"
      n_Xl_bus1_N = "n:Xl:bus1:N", "Ohm, Network Reactance, Line-Line"
      n_Xl_bus1n = "n:Xl:bus1n", "Ohm, Network Reactance, Line-Line"
      n_Zl_bus1_A = "n:Zl:bus1:A", "Ohm, Network Impedance, Line-Line"
      n_Zl_bus1_B = "n:Zl:bus1:B", "Ohm, Network Impedance, Line-Line"
      n_Zl_bus1_C = "n:Zl:bus1:C", "Ohm, Network Impedance, Line-Line"
      n_Zl_bus1_N = "n:Zl:bus1:N", "Ohm, Network Impedance, Line-Line"
      n_Zl_bus1n = "n:Zl:bus1n", "Ohm, Network Impedance, Line-Line"
      n_phizl_bus1_A = "n:phizl:bus1:A", "deg, Angle of Network Impedance, Line-Line"
      n_phizl_bus1_B = "n:phizl:bus1:B", "deg, Angle of Network Impedance, Line-Line"
      n_phizl_bus1_C = "n:phizl:bus1:C", "deg, Angle of Network Impedance, Line-Line"
      n_phizl_bus1_N = "n:phizl:bus1:N", "deg, Angle of Network Impedance, Line-Line"
      n_phizl_bus1n = "n:phizl:bus1n", "deg, Angle of Network Impedance, Line-Line"
      n_Rln_bus1_A = "n:Rln:bus1:A", "Ohm, Network Resistance, Line-Neutral"
      n_Rln_bus1_B = "n:Rln:bus1:B", "Ohm, Network Resistance, Line-Neutral"
      n_Rln_bus1_C = "n:Rln:bus1:C", "Ohm, Network Resistance, Line-Neutral"
      n_Rln_bus1_N = "n:Rln:bus1:N", "Ohm, Network Resistance, Line-Neutral"
      n_Rln_bus1n = "n:Rln:bus1n", "Ohm, Network Resistance, Line-Neutral"
      n_Xln_bus1_A = "n:Xln:bus1:A", "Ohm, Network Reactance, Line-Neutral"
      n_Xln_bus1_B = "n:Xln:bus1:B", "Ohm, Network Reactance, Line-Neutral"
      n_Xln_bus1_C = "n:Xln:bus1:C", "Ohm, Network Reactance, Line-Neutral"
      n_Xln_bus1_N = "n:Xln:bus1:N", "Ohm, Network Reactance, Line-Neutral"
      n_Xln_bus1n = "n:Xln:bus1n", "Ohm, Network Reactance, Line-Neutral"
      n_Zln_bus1_A = "n:Zln:bus1:A", "Ohm, Network Impedance, Line-Neutral"
      n_Zln_bus1_B = "n:Zln:bus1:B", "Ohm, Network Impedance, Line-Neutral"
      n_Zln_bus1_C = "n:Zln:bus1:C", "Ohm, Network Impedance, Line-Neutral"
      n_Zln_bus1_N = "n:Zln:bus1:N", "Ohm, Network Impedance, Line-Neutral"
      n_Zln_bus1n = "n:Zln:bus1n", "Ohm, Network Impedance, Line-Neutral"
      n_phizln_bus1_A = "n:phizln:bus1:A", "deg, Angle of Network Impedance, Line-Neutral"
      n_phizln_bus1_B = "n:phizln:bus1:B", "deg, Angle of Network Impedance, Line-Neutral"
      n_phizln_bus1_C = "n:phizln:bus1:C", "deg, Angle of Network Impedance, Line-Neutral"
      n_phizln_bus1_N = "n:phizln:bus1:N", "deg, Angle of Network Impedance, Line-Neutral"
      n_phizln_bus1n = "n:phizln:bus1n", "deg, Angle of Network Impedance, Line-Neutral"
      n_Rn_bus1 = "n:Rn:bus1", "Ohm, Network Resistance, Neutral"
      n_Rn_bus1n = "n:Rn:bus1n", "Ohm, Network Resistance, Neutral"
      n_Xn_bus1 = "n:Xn:bus1", "Ohm, Network Reactance, Neutral"
      n_Xn_bus1n = "n:Xn:bus1n", "Ohm, Network Reactance, Neutral"
      n_Zn_bus1 = "n:Zn:bus1", "Ohm, Network Impedance, Neutral"
      n_Zn_bus1n = "n:Zn:bus1n", "Ohm, Network Impedance, Neutral"
      n_phizn_bus1 = "n:phizn:bus1", "deg, Angle of Network Impedance, Neutral"
      n_phizn_bus1n = "n:phizn:bus1n", "deg, Angle of Network Impedance, Neutral"
      n_eff_bus1_A = "n:eff:bus1:A", "Earth Fault Factor"
      n_eff_bus1_B = "n:eff:bus1:B", "Earth Fault Factor"
      n_eff_bus1_C = "n:eff:bus1:C", "Earth Fault Factor"
      n_eff_bus1_N = "n:eff:bus1:N", "Earth Fault Factor"
      n_eff_bus1n = "n:eff:bus1n", "Earth Fault Factor"
      n_X0toX1_bus1 = "n:X0toX1:bus1", "X0/X1"
      n_X0toX1_bus1n = "n:X0toX1:bus1n", "X0/X1"
      n_in_bus1 = "n:in:bus1", "p.u., Neutral Current, Magnitude"
      n_in_bus1n = "n:in:bus1n", "p.u., Neutral Current, Magnitude"
      n_In_bus1 = "n:In:bus1", "kA, Neutral Current, Magnitude"
      n_In_bus1n = "n:In:bus1n", "kA, Neutral Current, Magnitude"
      n_phiin_bus1 = "n:phiin:bus1", "deg, Neutral Current, Angle"
      n_phiin_bus1n = "n:phiin:bus1n", "deg, Neutral Current, Angle"
      n_i0r_bus1 = "n:i0r:bus1", "p.u., Zero-Sequence Current, Real Part"
      n_i0r_bus1n = "n:i0r:bus1n", "p.u., Zero-Sequence Current, Real Part"
      n_i0i_bus1 = "n:i0i:bus1", "p.u., Zero-Sequence Current, Imaginary Part"
      n_i0i_bus1n = "n:i0i:bus1n", "p.u., Zero-Sequence Current, Imaginary Part"
      n_i0_bus1 = "n:i0:bus1", "p.u., Zero-Sequence Current, Magnitude"
      n_i0_bus1n = "n:i0:bus1n", "p.u., Zero-Sequence Current, Magnitude"
      n_I0_bus1 = "n:I0:bus1", "kA, Zero-Sequence Current, Magnitude"
      n_I0_bus1n = "n:I0:bus1n", "kA, Zero-Sequence Current, Magnitude"
      n_I0x3_bus1 = "n:I0x3:bus1", "kA, 3*I0"
      n_I0x3_bus1n = "n:I0x3:bus1n", "kA, 3*I0"
      n_phii0_bus1 = "n:phii0:bus1", "deg, Zero-Sequence Current, Angle"
      n_phii0_bus1n = "n:phii0:bus1n", "deg, Zero-Sequence Current, Angle"
      n_i1r_bus1 = "n:i1r:bus1", "p.u., Positive-Sequence Current, Real Part"
      n_i1r_bus1n = "n:i1r:bus1n", "p.u., Positive-Sequence Current, Real Part"
      n_i1i_bus1 = "n:i1i:bus1", "p.u., Positive-Sequence Current, Imaginary Part"
      n_i1i_bus1n = "n:i1i:bus1n", "p.u., Positive-Sequence Current, Imaginary Part"
      n_i1_bus1 = "n:i1:bus1", "p.u., Positive-Sequence Current, Magnitude"
      n_i1_bus1n = "n:i1:bus1n", "p.u., Positive-Sequence Current, Magnitude"
      n_I1_bus1 = "n:I1:bus1", "kA, Positive-Sequence Current, Magnitude"
      n_I1_bus1n = "n:I1:bus1n", "kA, Positive-Sequence Current, Magnitude"
      n_phii1_bus1 = "n:phii1:bus1", "deg, Positive-Sequence Current, Angle"
      n_phii1_bus1n = "n:phii1:bus1n", "deg, Positive-Sequence Current, Angle"
      n_i2r_bus1 = "n:i2r:bus1", "p.u., Negative-Sequence Current, Real Part"
      n_i2r_bus1n = "n:i2r:bus1n", "p.u., Negative-Sequence Current, Real Part"
      n_i2i_bus1 = "n:i2i:bus1", "p.u., Negative-Sequence Current, Imaginary Part"
      n_i2i_bus1n = "n:i2i:bus1n", "p.u., Negative-Sequence Current, Imaginary Part"
      n_i2_bus1 = "n:i2:bus1", "p.u., Negative-Sequence Current, Magnitude"
      n_i2_bus1n = "n:i2:bus1n", "p.u., Negative-Sequence Current, Magnitude"
      n_I2_bus1 = "n:I2:bus1", "kA, Negative-Sequence Current, Magnitude"
      n_I2_bus1n = "n:I2:bus1n", "kA, Negative-Sequence Current, Magnitude"
      n_phii2_bus1 = "n:phii2:bus1", "deg, Negative-Sequence Current, Angle"
      n_phii2_bus1n = "n:phii2:bus1n", "deg, Negative-Sequence Current, Angle"
      n_du_bus1_A = "n:du:bus1:A", "%, Voltage Deviation"
      n_du_bus1_B = "n:du:bus1:B", "%, Voltage Deviation"
      n_du_bus1_C = "n:du:bus1:C", "%, Voltage Deviation"
      n_du_bus1_N = "n:du:bus1:N", "%, Voltage Deviation"
      n_du_bus1n = "n:du:bus1n", "%, Voltage Deviation"
      n_urpref_bus1_A = "n:urpref:bus1:A", "p.u., Pre-fault Voltage, Real Part"
      n_urpref_bus1_B = "n:urpref:bus1:B", "p.u., Pre-fault Voltage, Real Part"
      n_urpref_bus1_C = "n:urpref:bus1:C", "p.u., Pre-fault Voltage, Real Part"
      n_urpref_bus1_N = "n:urpref:bus1:N", "p.u., Pre-fault Voltage, Real Part"
      n_urpref_bus1n = "n:urpref:bus1n", "p.u., Pre-fault Voltage, Real Part"
      n_uipref_bus1_A = "n:uipref:bus1:A", "p.u., Pre-fault Voltage, Imaginary Part"
      n_uipref_bus1_B = "n:uipref:bus1:B", "p.u., Pre-fault Voltage, Imaginary Part"
      n_uipref_bus1_C = "n:uipref:bus1:C", "p.u., Pre-fault Voltage, Imaginary Part"
      n_uipref_bus1_N = "n:uipref:bus1:N", "p.u., Pre-fault Voltage, Imaginary Part"
      n_uipref_bus1n = "n:uipref:bus1n", "p.u., Pre-fault Voltage, Imaginary Part"
      n_upref_bus1_A = "n:upref:bus1:A", "p.u., Pre-fault Voltage, Magnitude"
      n_upref_bus1_B = "n:upref:bus1:B", "p.u., Pre-fault Voltage, Magnitude"
      n_upref_bus1_C = "n:upref:bus1:C", "p.u., Pre-fault Voltage, Magnitude"
      n_upref_bus1_N = "n:upref:bus1:N", "p.u., Pre-fault Voltage, Magnitude"
      n_upref_bus1n = "n:upref:bus1n", "p.u., Pre-fault Voltage, Magnitude"
      n_uprefpc_bus1_A = "n:uprefpc:bus1:A", "%, Pre-fault Voltage, Magnitude"
      n_uprefpc_bus1_B = "n:uprefpc:bus1:B", "%, Pre-fault Voltage, Magnitude"
      n_uprefpc_bus1_C = "n:uprefpc:bus1:C", "%, Pre-fault Voltage, Magnitude"
      n_uprefpc_bus1_N = "n:uprefpc:bus1:N", "%, Pre-fault Voltage, Magnitude"
      n_uprefpc_bus1n = "n:uprefpc:bus1n", "%, Pre-fault Voltage, Magnitude"
      n_Upref_bus1_A = "n:Upref:bus1:A", "kV, Line-Ground Pre-fault Voltage, Magnitude"
      n_Upref_bus1_B = "n:Upref:bus1:B", "kV, Line-Ground Pre-fault Voltage, Magnitude"
      n_Upref_bus1_C = "n:Upref:bus1:C", "kV, Line-Ground Pre-fault Voltage, Magnitude"
      n_Upref_bus1_N = "n:Upref:bus1:N", "kV, Line-Ground Pre-fault Voltage, Magnitude"
      n_Upref_bus1n = "n:Upref:bus1n", "kV, Line-Ground Pre-fault Voltage, Magnitude"
      n_Ulpref_bus1_A = "n:Ulpref:bus1:A", "kV, Line-Line Pre-fault Voltage, Magnitude"
      n_Ulpref_bus1_B = "n:Ulpref:bus1:B", "kV, Line-Line Pre-fault Voltage, Magnitude"
      n_Ulpref_bus1_C = "n:Ulpref:bus1:C", "kV, Line-Line Pre-fault Voltage, Magnitude"
      n_Ulpref_bus1_N = "n:Ulpref:bus1:N", "kV, Line-Line Pre-fault Voltage, Magnitude"
      n_Ulpref_bus1n = "n:Ulpref:bus1n", "kV, Line-Line Pre-fault Voltage, Magnitude"
      n_phiupref_bus1_A = "n:phiupref:bus1:A", "deg, Pre-fault Voltage, Angle"
      n_phiupref_bus1_B = "n:phiupref:bus1:B", "deg, Pre-fault Voltage, Angle"
      n_phiupref_bus1_C = "n:phiupref:bus1:C", "deg, Pre-fault Voltage, Angle"
      n_phiupref_bus1_N = "n:phiupref:bus1:N", "deg, Pre-fault Voltage, Angle"
      n_phiupref_bus1n = "n:phiupref:bus1n", "deg, Pre-fault Voltage, Angle"
      n_phiui_bus1_A = "n:phiui:bus1:A", "deg, Angle between Voltage and Current"
      n_phiui_bus1_B = "n:phiui:bus1:B", "deg, Angle between Voltage and Current"
      n_phiui_bus1_C = "n:phiui:bus1:C", "deg, Angle between Voltage and Current"
      n_phiui_bus1_N = "n:phiui:bus1:N", "deg, Angle between Voltage and Current"
      n_phiui_bus1n = "n:phiui:bus1n", "deg, Angle between Voltage and Current"
      n_phiu1i1_bus1 = "n:phiu1i1:bus1", "deg, Angle between Voltage and Current in positive sequence system"
      n_phiu1i1_bus1n = "n:phiu1i1:bus1n", "deg, Angle between Voltage and Current in positive sequence system"
      n_phiu2i2_bus1 = "n:phiu2i2:bus1", "deg, Angle between Voltage and Current in negative sequence system"
      n_phiu2i2_bus1n = "n:phiu2i2:bus1n", "deg, Angle between Voltage and Current in negative sequence system"
      n_phiu0i0_bus1 = "n:phiu0i0:bus1", "deg, Angle between Voltage and Current in zero-sequence system"
      n_phiu0i0_bus1n = "n:phiu0i0:bus1n", "deg, Angle between Voltage and Current in zero-sequence system"
      n_Ithload_bus1n = "n:Ithload:bus1n", "%, Loading, Thermal equivalent Current"
      n_Ipload_bus1n = "n:Ipload:bus1n", "%, Loading, Peak Short-Circuit Current"
      n_Ithrtk_bus1n = "n:Ithrtk:bus1n", "kA, Rated Short-Time Current (Tk)"
      n_cfac_bus1n = "n:cfac:bus1n", "Voltage Factor"
      n_XtoR_bus1n = "n:XtoR:bus1n", "X/R ratio (peak)"
      n_RtoX_bus1n = "n:RtoX:bus1n", "R/X ratio (peak)"
      c_i_earth = "c:i_earth", "Internal"
      c_Uhaph_c_A = "c:Uhaph_c:A", "kV"
      c_Uhaph_c_B = "c:Uhaph_c:B", "kV"
      c_Uhaph_c_C = "c:Uhaph_c:C", "kV"
      c_Uhaph_c_N = "c:Uhaph_c:N", "kV"
      c_Uhaph_rl_A = "c:Uhaph_rl:A", "kV"
      c_Uhaph_rl_B = "c:Uhaph_rl:B", "kV"
      c_Uhaph_rl_C = "c:Uhaph_rl:C", "kV"
      c_Uhaph_rl_N = "c:Uhaph_rl:N", "kV"
      c_Ufnph_c_A = "c:Ufnph_c:A", "kV"
      c_Ufnph_c_B = "c:Ufnph_c:B", "kV"
      c_Ufnph_c_C = "c:Ufnph_c:C", "kV"
      c_Ufnph_c_N = "c:Ufnph_c:N", "kV"
      c_Ufnph_rl_A = "c:Ufnph_rl:A", "kV"
      c_Ufnph_rl_B = "c:Ufnph_rl:B", "kV"
      c_Ufnph_rl_C = "c:Ufnph_rl:C", "kV"
      c_Ufnph_rl_N = "c:Ufnph_rl:N", "kV"

    class ElmSind(Enum):
      _init_ = 'value __doc__'

      m_u0_bus1 = "m:u0:bus1", "p.u., Zero-Sequence-Voltage, Magnitude"
      m_u0_bus2 = "m:u0:bus2", "p.u., Zero-Sequence-Voltage, Magnitude"
      m_phiu0_bus1 = "m:phiu0:bus1", "deg, Zero-Sequence-Voltage, Angle"
      m_phiu0_bus2 = "m:phiu0:bus2", "deg, Zero-Sequence-Voltage, Angle"
      m_u1r_bus1 = "m:u1r:bus1", "p.u., Positive-Sequence-Voltage, Real Part"
      m_u1r_bus2 = "m:u1r:bus2", "p.u., Positive-Sequence-Voltage, Real Part"
      m_u1i_bus1 = "m:u1i:bus1", "p.u., Positive-Sequence-Voltage, Imaginary Part"
      m_u1i_bus2 = "m:u1i:bus2", "p.u., Positive-Sequence-Voltage, Imaginary Part"
      m_u1_bus1 = "m:u1:bus1", "p.u., Positive-Sequence-Voltage, Magnitude"
      m_u1_bus2 = "m:u1:bus2", "p.u., Positive-Sequence-Voltage, Magnitude"
      m_U1_bus1 = "m:U1:bus1", "kV, Line-Ground Positive-Sequence-Voltage, Magnitude"
      m_U1_bus2 = "m:U1:bus2", "kV, Line-Ground Positive-Sequence-Voltage, Magnitude"
      m_U1l_bus1 = "m:U1l:bus1", "kV, Line-Line Positive-Sequence-Voltage, Magnitude"
      m_U1l_bus2 = "m:U1l:bus2", "kV, Line-Line Positive-Sequence-Voltage, Magnitude"
      m_phiu1_bus1 = "m:phiu1:bus1", "deg, Positive-Sequence-Voltage, Angle"
      m_phiu1_bus2 = "m:phiu1:bus2", "deg, Positive-Sequence-Voltage, Angle"
      m_u2_bus1 = "m:u2:bus1", "p.u., Negative-Sequence-Voltage, Magnitude"
      m_u2_bus2 = "m:u2:bus2", "p.u., Negative-Sequence-Voltage, Magnitude"
      m_phiu2_bus1 = "m:phiu2:bus1", "deg, Negative-Sequence-Voltage, Angle"
      m_phiu2_bus2 = "m:phiu2:bus2", "deg, Negative-Sequence-Voltage, Angle"
      m_inet_bus1_A = "m:inet:bus1:A", "p.u., Phase Current, Magnitude, referred to network"
      m_inet_bus1_B = "m:inet:bus1:B", "p.u., Phase Current, Magnitude, referred to network"
      m_inet_bus1_C = "m:inet:bus1:C", "p.u., Phase Current, Magnitude, referred to network"
      m_inet_bus2_A = "m:inet:bus2:A", "p.u., Phase Current, Magnitude, referred to network"
      m_inet_bus2_B = "m:inet:bus2:B", "p.u., Phase Current, Magnitude, referred to network"
      m_inet_bus2_C = "m:inet:bus2:C", "p.u., Phase Current, Magnitude, referred to network"
      m_I_bus1_A = "m:I:bus1:A", "kA, Phase Current, Magnitude"
      m_I_bus1_B = "m:I:bus1:B", "kA, Phase Current, Magnitude"
      m_I_bus1_C = "m:I:bus1:C", "kA, Phase Current, Magnitude"
      m_I_bus2_A = "m:I:bus2:A", "kA, Phase Current, Magnitude"
      m_I_bus2_B = "m:I:bus2:B", "kA, Phase Current, Magnitude"
      m_I_bus2_C = "m:I:bus2:C", "kA, Phase Current, Magnitude"
      m_phii_bus1_A = "m:phii:bus1:A", "deg, Phase Current, Angle"
      m_phii_bus1_B = "m:phii:bus1:B", "deg, Phase Current, Angle"
      m_phii_bus1_C = "m:phii:bus1:C", "deg, Phase Current, Angle"
      m_phii_bus2_A = "m:phii:bus2:A", "deg, Phase Current, Angle"
      m_phii_bus2_B = "m:phii:bus2:B", "deg, Phase Current, Angle"
      m_phii_bus2_C = "m:phii:bus2:C", "deg, Phase Current, Angle"
      m_i0r_bus1 = "m:i0r:bus1", "p.u., Zero-Sequence Current, Real Part"
      m_i0r_bus2 = "m:i0r:bus2", "p.u., Zero-Sequence Current, Real Part"
      m_i0i_bus1 = "m:i0i:bus1", "p.u., Zero-Sequence Current, Imaginary Part"
      m_i0i_bus2 = "m:i0i:bus2", "p.u., Zero-Sequence Current, Imaginary Part"
      m_i0_bus1 = "m:i0:bus1", "p.u., Zero-Sequence Current, Magnitude"
      m_i0_bus2 = "m:i0:bus2", "p.u., Zero-Sequence Current, Magnitude"
      m_I0_bus1 = "m:I0:bus1", "kA, Zero-Sequence Current, Magnitude"
      m_I0_bus2 = "m:I0:bus2", "kA, Zero-Sequence Current, Magnitude"
      m_I0x3_bus1 = "m:I0x3:bus1", "kA, 3*I0"
      m_I0x3_bus2 = "m:I0x3:bus2", "kA, 3*I0"
      m_phii0_bus1 = "m:phii0:bus1", "deg, Zero-Sequence Current, Angle"
      m_phii0_bus2 = "m:phii0:bus2", "deg, Zero-Sequence Current, Angle"
      m_i1r_bus1 = "m:i1r:bus1", "p.u., Positive-Sequence Current, Real Part"
      m_i1r_bus2 = "m:i1r:bus2", "p.u., Positive-Sequence Current, Real Part"
      m_i1i_bus1 = "m:i1i:bus1", "p.u., Positive-Sequence Current, Imaginary Part"
      m_i1i_bus2 = "m:i1i:bus2", "p.u., Positive-Sequence Current, Imaginary Part"
      m_i1_bus1 = "m:i1:bus1", "p.u., Positive-Sequence Current, Magnitude"
      m_i1_bus2 = "m:i1:bus2", "p.u., Positive-Sequence Current, Magnitude"
      m_I1_bus1 = "m:I1:bus1", "kA, Positive-Sequence Current, Magnitude"
      m_I1_bus2 = "m:I1:bus2", "kA, Positive-Sequence Current, Magnitude"
      m_phii1_bus1 = "m:phii1:bus1", "deg, Positive-Sequence Current, Angle"
      m_phii1_bus2 = "m:phii1:bus2", "deg, Positive-Sequence Current, Angle"
      m_i2r_bus1 = "m:i2r:bus1", "p.u., Negative-Sequence Current, Real Part"
      m_i2r_bus2 = "m:i2r:bus2", "p.u., Negative-Sequence Current, Real Part"
      m_i2i_bus1 = "m:i2i:bus1", "p.u., Negative-Sequence Current, Imaginary Part"
      m_i2i_bus2 = "m:i2i:bus2", "p.u., Negative-Sequence Current, Imaginary Part"
      m_i2_bus1 = "m:i2:bus1", "p.u., Negative-Sequence Current, Magnitude"
      m_i2_bus2 = "m:i2:bus2", "p.u., Negative-Sequence Current, Magnitude"
      m_I2_bus1 = "m:I2:bus1", "kA, Negative-Sequence Current, Magnitude"
      m_I2_bus2 = "m:I2:bus2", "kA, Negative-Sequence Current, Magnitude"
      m_phii2_bus1 = "m:phii2:bus1", "deg, Negative-Sequence Current, Angle"
      m_phii2_bus2 = "m:phii2:bus2", "deg, Negative-Sequence Current, Angle"
      m_ubfacI_bus1 = "m:ubfacI:bus1", "%, Current Unbalance Factor"
      m_ubfacI_bus2 = "m:ubfacI:bus2", "%, Current Unbalance Factor"
      m_ubfacS_bus1 = "m:ubfacS:bus1", "%, Power Unbalance Factor"
      m_ubfacS_bus2 = "m:ubfacS:bus2", "%, Power Unbalance Factor"
      m_P_bus1_A = "m:P:bus1:A", "MW, Active Power"
      m_P_bus1_B = "m:P:bus1:B", "MW, Active Power"
      m_P_bus1_C = "m:P:bus1:C", "MW, Active Power"
      m_P_bus2_A = "m:P:bus2:A", "MW, Active Power"
      m_P_bus2_B = "m:P:bus2:B", "MW, Active Power"
      m_P_bus2_C = "m:P:bus2:C", "MW, Active Power"
      m_Q_bus1_A = "m:Q:bus1:A", "Mvar, Reactive Power"
      m_Q_bus1_B = "m:Q:bus1:B", "Mvar, Reactive Power"
      m_Q_bus1_C = "m:Q:bus1:C", "Mvar, Reactive Power"
      m_Q_bus2_A = "m:Q:bus2:A", "Mvar, Reactive Power"
      m_Q_bus2_B = "m:Q:bus2:B", "Mvar, Reactive Power"
      m_Q_bus2_C = "m:Q:bus2:C", "Mvar, Reactive Power"
      m_S_bus1_A = "m:S:bus1:A", "MVA, Apparent Power"
      m_S_bus1_B = "m:S:bus1:B", "MVA, Apparent Power"
      m_S_bus1_C = "m:S:bus1:C", "MVA, Apparent Power"
      m_S_bus2_A = "m:S:bus2:A", "MVA, Apparent Power"
      m_S_bus2_B = "m:S:bus2:B", "MVA, Apparent Power"
      m_S_bus2_C = "m:S:bus2:C", "MVA, Apparent Power"
      m_cosphi_bus1_A = "m:cosphi:bus1:A", "Power Factor"
      m_cosphi_bus1_B = "m:cosphi:bus1:B", "Power Factor"
      m_cosphi_bus1_C = "m:cosphi:bus1:C", "Power Factor"
      m_cosphi_bus2_A = "m:cosphi:bus2:A", "Power Factor"
      m_cosphi_bus2_B = "m:cosphi:bus2:B", "Power Factor"
      m_cosphi_bus2_C = "m:cosphi:bus2:C", "Power Factor"
      m_tanphi_bus1_A = "m:tanphi:bus1:A", "tan(phi)"
      m_tanphi_bus1_B = "m:tanphi:bus1:B", "tan(phi)"
      m_tanphi_bus1_C = "m:tanphi:bus1:C", "tan(phi)"
      m_tanphi_bus2_A = "m:tanphi:bus2:A", "tan(phi)"
      m_tanphi_bus2_B = "m:tanphi:bus2:B", "tan(phi)"
      m_tanphi_bus2_C = "m:tanphi:bus2:C", "tan(phi)"
      m_Psum_bus1 = "m:Psum:bus1", "MW, Total Active Power"
      m_Psum_bus2 = "m:Psum:bus2", "MW, Total Active Power"
      m_Qsum_bus1 = "m:Qsum:bus1", "Mvar, Total Reactive Power"
      m_Qsum_bus2 = "m:Qsum:bus2", "Mvar, Total Reactive Power"
      m_Ssum_bus1 = "m:Ssum:bus1", "MVA, Total Apparent Power"
      m_Ssum_bus2 = "m:Ssum:bus2", "MVA, Total Apparent Power"
      m_cosphisum_bus1 = "m:cosphisum:bus1", "Total Power Factor"
      m_cosphisum_bus2 = "m:cosphisum:bus2", "Total Power Factor"
      m_tanphisum_bus1 = "m:tanphisum:bus1", "Total tan(phi)"
      m_tanphisum_bus2 = "m:tanphisum:bus2", "Total tan(phi)"
      m_Pln_bus1_A = "m:Pln:bus1:A", "MW, Active Power (Line-Neutral)"
      m_Pln_bus1_B = "m:Pln:bus1:B", "MW, Active Power (Line-Neutral)"
      m_Pln_bus1_C = "m:Pln:bus1:C", "MW, Active Power (Line-Neutral)"
      m_Pln_bus2_A = "m:Pln:bus2:A", "MW, Active Power (Line-Neutral)"
      m_Pln_bus2_B = "m:Pln:bus2:B", "MW, Active Power (Line-Neutral)"
      m_Pln_bus2_C = "m:Pln:bus2:C", "MW, Active Power (Line-Neutral)"
      m_Qln_bus1_A = "m:Qln:bus1:A", "Mvar, Reactive Power (Line-Neutral)"
      m_Qln_bus1_B = "m:Qln:bus1:B", "Mvar, Reactive Power (Line-Neutral)"
      m_Qln_bus1_C = "m:Qln:bus1:C", "Mvar, Reactive Power (Line-Neutral)"
      m_Qln_bus2_A = "m:Qln:bus2:A", "Mvar, Reactive Power (Line-Neutral)"
      m_Qln_bus2_B = "m:Qln:bus2:B", "Mvar, Reactive Power (Line-Neutral)"
      m_Qln_bus2_C = "m:Qln:bus2:C", "Mvar, Reactive Power (Line-Neutral)"
      m_P1_bus1 = "m:P1:bus1", "MW, Positive Sequence Active Power"
      m_P1_bus2 = "m:P1:bus2", "MW, Positive Sequence Active Power"
      m_Q1_bus1 = "m:Q1:bus1", "Mvar, Positive Sequence Reactive Power"
      m_Q1_bus2 = "m:Q1:bus2", "Mvar, Positive Sequence Reactive Power"
      m_P2_bus1 = "m:P2:bus1", "MW, Negative Sequence Active Power"
      m_P2_bus2 = "m:P2:bus2", "MW, Negative Sequence Active Power"
      m_Q2_bus1 = "m:Q2:bus1", "Mvar, Negative Sequence Reactive Power"
      m_Q2_bus2 = "m:Q2:bus2", "Mvar, Negative Sequence Reactive Power"
      m_P0_bus1 = "m:P0:bus1", "MW, Zero Sequence Active Power"
      m_P0_bus2 = "m:P0:bus2", "MW, Zero Sequence Active Power"
      m_Q0_bus1 = "m:Q0:bus1", "Mvar, Zero Sequence Reactive Power"
      m_Q0_bus2 = "m:Q0:bus2", "Mvar, Zero Sequence Reactive Power"
      m_i1P_bus1 = "m:i1P:bus1", "p.u., Positive-Sequence Active Current"
      m_i1P_bus2 = "m:i1P:bus2", "p.u., Positive-Sequence Active Current"
      m_i1Q_bus1 = "m:i1Q:bus1", "p.u., Positive-Sequence Reactive Current"
      m_i1Q_bus2 = "m:i1Q:bus2", "p.u., Positive-Sequence Reactive Current"
      m_I1P_bus1 = "m:I1P:bus1", "kA, Positive-Sequence Active Current"
      m_I1P_bus2 = "m:I1P:bus2", "kA, Positive-Sequence Active Current"
      m_I1Q_bus1 = "m:I1Q:bus1", "kA, Positive-Sequence Reactive Current"
      m_I1Q_bus2 = "m:I1Q:bus2", "kA, Positive-Sequence Reactive Current"
      m_i2P_bus1 = "m:i2P:bus1", "p.u., Negative-Sequence Active Current"
      m_i2P_bus2 = "m:i2P:bus2", "p.u., Negative-Sequence Active Current"
      m_i2Q_bus1 = "m:i2Q:bus1", "p.u., Negative-Sequence Reactive Current"
      m_i2Q_bus2 = "m:i2Q:bus2", "p.u., Negative-Sequence Reactive Current"
      m_I2P_bus1 = "m:I2P:bus1", "kA, Negative-Sequence Active Current"
      m_I2P_bus2 = "m:I2P:bus2", "kA, Negative-Sequence Active Current"
      m_I2Q_bus1 = "m:I2Q:bus1", "kA, Negative-Sequence Reactive Current"
      m_I2Q_bus2 = "m:I2Q:bus2", "kA, Negative-Sequence Reactive Current"
      m_ur_bus1_A = "m:ur:bus1:A", "p.u., Phase Voltage, Real Part"
      m_ur_bus1_B = "m:ur:bus1:B", "p.u., Phase Voltage, Real Part"
      m_ur_bus1_C = "m:ur:bus1:C", "p.u., Phase Voltage, Real Part"
      m_ur_bus2_A = "m:ur:bus2:A", "p.u., Phase Voltage, Real Part"
      m_ur_bus2_B = "m:ur:bus2:B", "p.u., Phase Voltage, Real Part"
      m_ur_bus2_C = "m:ur:bus2:C", "p.u., Phase Voltage, Real Part"
      m_ui_bus1_A = "m:ui:bus1:A", "p.u., Phase Voltage, Imaginary Part"
      m_ui_bus1_B = "m:ui:bus1:B", "p.u., Phase Voltage, Imaginary Part"
      m_ui_bus1_C = "m:ui:bus1:C", "p.u., Phase Voltage, Imaginary Part"
      m_ui_bus2_A = "m:ui:bus2:A", "p.u., Phase Voltage, Imaginary Part"
      m_ui_bus2_B = "m:ui:bus2:B", "p.u., Phase Voltage, Imaginary Part"
      m_ui_bus2_C = "m:ui:bus2:C", "p.u., Phase Voltage, Imaginary Part"
      m_u_bus1_A = "m:u:bus1:A", "p.u., Phase Voltage, Magnitude"
      m_u_bus1_B = "m:u:bus1:B", "p.u., Phase Voltage, Magnitude"
      m_u_bus1_C = "m:u:bus1:C", "p.u., Phase Voltage, Magnitude"
      m_u_bus2_A = "m:u:bus2:A", "p.u., Phase Voltage, Magnitude"
      m_u_bus2_B = "m:u:bus2:B", "p.u., Phase Voltage, Magnitude"
      m_u_bus2_C = "m:u:bus2:C", "p.u., Phase Voltage, Magnitude"
      m_ir_bus1_A = "m:ir:bus1:A", "p.u., Phase Current, Real Part"
      m_ir_bus1_B = "m:ir:bus1:B", "p.u., Phase Current, Real Part"
      m_ir_bus1_C = "m:ir:bus1:C", "p.u., Phase Current, Real Part"
      m_ir_bus2_A = "m:ir:bus2:A", "p.u., Phase Current, Real Part"
      m_ir_bus2_B = "m:ir:bus2:B", "p.u., Phase Current, Real Part"
      m_ir_bus2_C = "m:ir:bus2:C", "p.u., Phase Current, Real Part"
      m_ii_bus1_A = "m:ii:bus1:A", "p.u., Phase Current, Imaginary Part"
      m_ii_bus1_B = "m:ii:bus1:B", "p.u., Phase Current, Imaginary Part"
      m_ii_bus1_C = "m:ii:bus1:C", "p.u., Phase Current, Imaginary Part"
      m_ii_bus2_A = "m:ii:bus2:A", "p.u., Phase Current, Imaginary Part"
      m_ii_bus2_B = "m:ii:bus2:B", "p.u., Phase Current, Imaginary Part"
      m_ii_bus2_C = "m:ii:bus2:C", "p.u., Phase Current, Imaginary Part"
      m_i_bus1_A = "m:i:bus1:A", "p.u., Phase Current, Magnitude"
      m_i_bus1_B = "m:i:bus1:B", "p.u., Phase Current, Magnitude"
      m_i_bus1_C = "m:i:bus1:C", "p.u., Phase Current, Magnitude"
      m_i_bus2_A = "m:i:bus2:A", "p.u., Phase Current, Magnitude"
      m_i_bus2_B = "m:i:bus2:B", "p.u., Phase Current, Magnitude"
      m_i_bus2_C = "m:i:bus2:C", "p.u., Phase Current, Magnitude"
      m_phiui_bus1_A = "m:phiui:bus1:A", "deg, Angle between Voltage and Current"
      m_phiui_bus1_B = "m:phiui:bus1:B", "deg, Angle between Voltage and Current"
      m_phiui_bus1_C = "m:phiui:bus1:C", "deg, Angle between Voltage and Current"
      m_phiui_bus2_A = "m:phiui:bus2:A", "deg, Angle between Voltage and Current"
      m_phiui_bus2_B = "m:phiui:bus2:B", "deg, Angle between Voltage and Current"
      m_phiui_bus2_C = "m:phiui:bus2:C", "deg, Angle between Voltage and Current"
      m_phiu0i0_bus1 = "m:phiu0i0:bus1", "deg, Angle between Voltage and Current in zero-sequence system"
      m_phiu0i0_bus2 = "m:phiu0i0:bus2", "deg, Angle between Voltage and Current in zero-sequence system"
      m_phiu1i1_bus1 = "m:phiu1i1:bus1", "deg, Angle between Voltage and Current in positive sequence system"
      m_phiu1i1_bus2 = "m:phiu1i1:bus2", "deg, Angle between Voltage and Current in positive sequence system"
      m_phiu2i2_bus1 = "m:phiu2i2:bus1", "deg, Angle between Voltage and Current in negative sequence system"
      m_phiu2i2_bus2 = "m:phiu2i2:bus2", "deg, Angle between Voltage and Current in negative sequence system"
      m_Tfct_bus1 = "m:Tfct:bus1", "s, Fault Clearing Time"
      m_Tfct_bus2 = "m:Tfct:bus2", "s, Fault Clearing Time"
      m_TfctPh_bus1_A = "m:TfctPh:bus1:A", "s, Fault Clearing Time"
      m_TfctPh_bus1_B = "m:TfctPh:bus1:B", "s, Fault Clearing Time"
      m_TfctPh_bus1_C = "m:TfctPh:bus1:C", "s, Fault Clearing Time"
      m_TfctPh_bus2_A = "m:TfctPh:bus2:A", "s, Fault Clearing Time"
      m_TfctPh_bus2_B = "m:TfctPh:bus2:B", "s, Fault Clearing Time"
      m_TfctPh_bus2_C = "m:TfctPh:bus2:C", "s, Fault Clearing Time"
      m_Brkload_bus1 = "m:Brkload:bus1", "%, Max. Breaker Loading"
      m_Brkload_bus2 = "m:Brkload:bus2", "%, Max. Breaker Loading"
      m_BrkloadPh_bus1_A = "m:BrkloadPh:bus1:A", "%, Breaker Loading"
      m_BrkloadPh_bus1_B = "m:BrkloadPh:bus1:B", "%, Breaker Loading"
      m_BrkloadPh_bus1_C = "m:BrkloadPh:bus1:C", "%, Breaker Loading"
      m_BrkloadPh_bus2_A = "m:BrkloadPh:bus2:A", "%, Breaker Loading"
      m_BrkloadPh_bus2_B = "m:BrkloadPh:bus2:B", "%, Breaker Loading"
      m_BrkloadPh_bus2_C = "m:BrkloadPh:bus2:C", "%, Breaker Loading"
      n_ur_bus1_A = "n:ur:bus1:A", "p.u., Line-Ground Voltage, Real Part"
      n_ur_bus1_B = "n:ur:bus1:B", "p.u., Line-Ground Voltage, Real Part"
      n_ur_bus1_C = "n:ur:bus1:C", "p.u., Line-Ground Voltage, Real Part"
      n_ur_bus2_A = "n:ur:bus2:A", "p.u., Line-Ground Voltage, Real Part"
      n_ur_bus2_B = "n:ur:bus2:B", "p.u., Line-Ground Voltage, Real Part"
      n_ur_bus2_C = "n:ur:bus2:C", "p.u., Line-Ground Voltage, Real Part"
      n_ui_bus1_A = "n:ui:bus1:A", "p.u., Line-Ground Voltage, Imaginary Part"
      n_ui_bus1_B = "n:ui:bus1:B", "p.u., Line-Ground Voltage, Imaginary Part"
      n_ui_bus1_C = "n:ui:bus1:C", "p.u., Line-Ground Voltage, Imaginary Part"
      n_ui_bus2_A = "n:ui:bus2:A", "p.u., Line-Ground Voltage, Imaginary Part"
      n_ui_bus2_B = "n:ui:bus2:B", "p.u., Line-Ground Voltage, Imaginary Part"
      n_ui_bus2_C = "n:ui:bus2:C", "p.u., Line-Ground Voltage, Imaginary Part"
      n_u_bus1_A = "n:u:bus1:A", "p.u., Line-Ground Voltage, Magnitude"
      n_u_bus1_B = "n:u:bus1:B", "p.u., Line-Ground Voltage, Magnitude"
      n_u_bus1_C = "n:u:bus1:C", "p.u., Line-Ground Voltage, Magnitude"
      n_u_bus2_A = "n:u:bus2:A", "p.u., Line-Ground Voltage, Magnitude"
      n_u_bus2_B = "n:u:bus2:B", "p.u., Line-Ground Voltage, Magnitude"
      n_u_bus2_C = "n:u:bus2:C", "p.u., Line-Ground Voltage, Magnitude"
      n_upc_bus1_A = "n:upc:bus1:A", "%, Line-Ground Voltage, Magnitude"
      n_upc_bus1_B = "n:upc:bus1:B", "%, Line-Ground Voltage, Magnitude"
      n_upc_bus1_C = "n:upc:bus1:C", "%, Line-Ground Voltage, Magnitude"
      n_upc_bus2_A = "n:upc:bus2:A", "%, Line-Ground Voltage, Magnitude"
      n_upc_bus2_B = "n:upc:bus2:B", "%, Line-Ground Voltage, Magnitude"
      n_upc_bus2_C = "n:upc:bus2:C", "%, Line-Ground Voltage, Magnitude"
      n_U_bus1_A = "n:U:bus1:A", "kV, Line-Ground Voltage, Magnitude"
      n_U_bus1_B = "n:U:bus1:B", "kV, Line-Ground Voltage, Magnitude"
      n_U_bus1_C = "n:U:bus1:C", "kV, Line-Ground Voltage, Magnitude"
      n_U_bus2_A = "n:U:bus2:A", "kV, Line-Ground Voltage, Magnitude"
      n_U_bus2_B = "n:U:bus2:B", "kV, Line-Ground Voltage, Magnitude"
      n_U_bus2_C = "n:U:bus2:C", "kV, Line-Ground Voltage, Magnitude"
      n_phiu_bus1_A = "n:phiu:bus1:A", "deg, Line-Ground Voltage, Angle"
      n_phiu_bus1_B = "n:phiu:bus1:B", "deg, Line-Ground Voltage, Angle"
      n_phiu_bus1_C = "n:phiu:bus1:C", "deg, Line-Ground Voltage, Angle"
      n_phiu_bus2_A = "n:phiu:bus2:A", "deg, Line-Ground Voltage, Angle"
      n_phiu_bus2_B = "n:phiu:bus2:B", "deg, Line-Ground Voltage, Angle"
      n_phiu_bus2_C = "n:phiu:bus2:C", "deg, Line-Ground Voltage, Angle"
      n_phiurel_bus1 = "n:phiurel:bus1", "deg, Voltage, Relative Angle"
      n_phiurel_bus2 = "n:phiurel:bus2", "deg, Voltage, Relative Angle"
      n_ul_bus1_A = "n:ul:bus1:A", "p.u., Line-Line Voltage, Magnitude"
      n_ul_bus1_B = "n:ul:bus1:B", "p.u., Line-Line Voltage, Magnitude"
      n_ul_bus1_C = "n:ul:bus1:C", "p.u., Line-Line Voltage, Magnitude"
      n_ul_bus2_A = "n:ul:bus2:A", "p.u., Line-Line Voltage, Magnitude"
      n_ul_bus2_B = "n:ul:bus2:B", "p.u., Line-Line Voltage, Magnitude"
      n_ul_bus2_C = "n:ul:bus2:C", "p.u., Line-Line Voltage, Magnitude"
      n_ulpc_bus1_A = "n:ulpc:bus1:A", "%, Line-Line Voltage, Magnitude"
      n_ulpc_bus1_B = "n:ulpc:bus1:B", "%, Line-Line Voltage, Magnitude"
      n_ulpc_bus1_C = "n:ulpc:bus1:C", "%, Line-Line Voltage, Magnitude"
      n_ulpc_bus2_A = "n:ulpc:bus2:A", "%, Line-Line Voltage, Magnitude"
      n_ulpc_bus2_B = "n:ulpc:bus2:B", "%, Line-Line Voltage, Magnitude"
      n_ulpc_bus2_C = "n:ulpc:bus2:C", "%, Line-Line Voltage, Magnitude"
      n_Ul_bus1_A = "n:Ul:bus1:A", "kV, Line-Line Voltage, Magnitude"
      n_Ul_bus1_B = "n:Ul:bus1:B", "kV, Line-Line Voltage, Magnitude"
      n_Ul_bus1_C = "n:Ul:bus1:C", "kV, Line-Line Voltage, Magnitude"
      n_Ul_bus2_A = "n:Ul:bus2:A", "kV, Line-Line Voltage, Magnitude"
      n_Ul_bus2_B = "n:Ul:bus2:B", "kV, Line-Line Voltage, Magnitude"
      n_Ul_bus2_C = "n:Ul:bus2:C", "kV, Line-Line Voltage, Magnitude"
      n_phiul_bus1_A = "n:phiul:bus1:A", "deg, Line-Line Voltage, Angle"
      n_phiul_bus1_B = "n:phiul:bus1:B", "deg, Line-Line Voltage, Angle"
      n_phiul_bus1_C = "n:phiul:bus1:C", "deg, Line-Line Voltage, Angle"
      n_phiul_bus2_A = "n:phiul:bus2:A", "deg, Line-Line Voltage, Angle"
      n_phiul_bus2_B = "n:phiul:bus2:B", "deg, Line-Line Voltage, Angle"
      n_phiul_bus2_C = "n:phiul:bus2:C", "deg, Line-Line Voltage, Angle"
      n_u0_bus1 = "n:u0:bus1", "p.u., Zero-Sequence Voltage, Magnitude"
      n_u0_bus2 = "n:u0:bus2", "p.u., Zero-Sequence Voltage, Magnitude"
      n_U0_bus1 = "n:U0:bus1", "kV, Zero-Sequence Voltage, Magnitude"
      n_U0_bus2 = "n:U0:bus2", "kV, Zero-Sequence Voltage, Magnitude"
      n_U0x3_bus1 = "n:U0x3:bus1", "kV, 3*U0"
      n_U0x3_bus2 = "n:U0x3:bus2", "kV, 3*U0"
      n_phiu0_bus1 = "n:phiu0:bus1", "deg, Zero-Sequence Voltage, Angle"
      n_phiu0_bus2 = "n:phiu0:bus2", "deg, Zero-Sequence Voltage, Angle"
      n_u1_bus1 = "n:u1:bus1", "p.u., Positive-Sequence Voltage, Magnitude"
      n_u1_bus2 = "n:u1:bus2", "p.u., Positive-Sequence Voltage, Magnitude"
      n_u1pc_bus1 = "n:u1pc:bus1", "%, Positive-Sequence Voltage, Magnitude"
      n_u1pc_bus2 = "n:u1pc:bus2", "%, Positive-Sequence Voltage, Magnitude"
      n_u1r_bus1 = "n:u1r:bus1", "p.u., Positive-Sequence Voltage, Real Part"
      n_u1r_bus2 = "n:u1r:bus2", "p.u., Positive-Sequence Voltage, Real Part"
      n_u1i_bus1 = "n:u1i:bus1", "p.u., Positive-Sequence Voltage, Imaginary Part"
      n_u1i_bus2 = "n:u1i:bus2", "p.u., Positive-Sequence Voltage, Imaginary Part"
      n_U1_bus1 = "n:U1:bus1", "kV, Line-Ground Positive-Sequence Voltage, Magnitude"
      n_U1_bus2 = "n:U1:bus2", "kV, Line-Ground Positive-Sequence Voltage, Magnitude"
      n_phiu1_bus1 = "n:phiu1:bus1", "deg, Positive-Sequence Voltage, Angle"
      n_phiu1_bus2 = "n:phiu1:bus2", "deg, Positive-Sequence Voltage, Angle"
      n_u2_bus1 = "n:u2:bus1", "p.u., Negative-Sequence Voltage, Magnitude"
      n_u2_bus2 = "n:u2:bus2", "p.u., Negative-Sequence Voltage, Magnitude"
      n_U2_bus1 = "n:U2:bus1", "kV, Line-Ground Negative-Sequence Voltage, Magnitude"
      n_U2_bus2 = "n:U2:bus2", "kV, Line-Ground Negative-Sequence Voltage, Magnitude"
      n_phiu2_bus1 = "n:phiu2:bus1", "deg, Negative-Sequence Voltage, Angle"
      n_phiu2_bus2 = "n:phiu2:bus2", "deg, Negative-Sequence Voltage, Angle"
      n_U1l_bus1 = "n:U1l:bus1", "kV, Line-Line Positive-Sequence Voltage, Magnitude"
      n_U1l_bus2 = "n:U1l:bus2", "kV, Line-Line Positive-Sequence Voltage, Magnitude"
      n_U2l_bus1 = "n:U2l:bus1", "kV, Line-Line Negative-Sequence Voltage, Magnitude"
      n_U2l_bus2 = "n:U2l:bus2", "kV, Line-Line Negative-Sequence Voltage, Magnitude"
      n_uln_bus1_A = "n:uln:bus1:A", "p.u., Line-Neutral Voltage, Magnitude"
      n_uln_bus1_B = "n:uln:bus1:B", "p.u., Line-Neutral Voltage, Magnitude"
      n_uln_bus1_C = "n:uln:bus1:C", "p.u., Line-Neutral Voltage, Magnitude"
      n_uln_bus2_A = "n:uln:bus2:A", "p.u., Line-Neutral Voltage, Magnitude"
      n_uln_bus2_B = "n:uln:bus2:B", "p.u., Line-Neutral Voltage, Magnitude"
      n_uln_bus2_C = "n:uln:bus2:C", "p.u., Line-Neutral Voltage, Magnitude"
      n_Uln_bus1_A = "n:Uln:bus1:A", "kV, Line-Neutral Voltage, Magnitude"
      n_Uln_bus1_B = "n:Uln:bus1:B", "kV, Line-Neutral Voltage, Magnitude"
      n_Uln_bus1_C = "n:Uln:bus1:C", "kV, Line-Neutral Voltage, Magnitude"
      n_Uln_bus2_A = "n:Uln:bus2:A", "kV, Line-Neutral Voltage, Magnitude"
      n_Uln_bus2_B = "n:Uln:bus2:B", "kV, Line-Neutral Voltage, Magnitude"
      n_Uln_bus2_C = "n:Uln:bus2:C", "kV, Line-Neutral Voltage, Magnitude"
      n_phiuln_bus1_A = "n:phiuln:bus1:A", "deg, Line-Neutral Voltage, Angle"
      n_phiuln_bus1_B = "n:phiuln:bus1:B", "deg, Line-Neutral Voltage, Angle"
      n_phiuln_bus1_C = "n:phiuln:bus1:C", "deg, Line-Neutral Voltage, Angle"
      n_phiuln_bus2_A = "n:phiuln:bus2:A", "deg, Line-Neutral Voltage, Angle"
      n_phiuln_bus2_B = "n:phiuln:bus2:B", "deg, Line-Neutral Voltage, Angle"
      n_phiuln_bus2_C = "n:phiuln:bus2:C", "deg, Line-Neutral Voltage, Angle"
      n_un_bus1 = "n:un:bus1", "p.u., Neutral-Ground Voltage, Magnitude"
      n_un_bus2 = "n:un:bus2", "p.u., Neutral-Ground Voltage, Magnitude"
      n_Un_bus1 = "n:Un:bus1", "kV, Neutral-Ground Voltage, Magnitude"
      n_Un_bus2 = "n:Un:bus2", "kV, Neutral-Ground Voltage, Magnitude"
      n_phiun_bus1 = "n:phiun:bus1", "deg, Neutral-Ground Voltage, Angle"
      n_phiun_bus2 = "n:phiun:bus2", "deg, Neutral-Ground Voltage, Angle"
      n_upht_bus1_A = "n:upht:bus1:A", "p.u., Phase Technology dependent Voltage, Magnitude"
      n_upht_bus1_B = "n:upht:bus1:B", "p.u., Phase Technology dependent Voltage, Magnitude"
      n_upht_bus1_C = "n:upht:bus1:C", "p.u., Phase Technology dependent Voltage, Magnitude"
      n_upht_bus2_A = "n:upht:bus2:A", "p.u., Phase Technology dependent Voltage, Magnitude"
      n_upht_bus2_B = "n:upht:bus2:B", "p.u., Phase Technology dependent Voltage, Magnitude"
      n_upht_bus2_C = "n:upht:bus2:C", "p.u., Phase Technology dependent Voltage, Magnitude"
      n_Upht_bus1_A = "n:Upht:bus1:A", "kV, Phase Technology dependent Voltage, Magnitude"
      n_Upht_bus1_B = "n:Upht:bus1:B", "kV, Phase Technology dependent Voltage, Magnitude"
      n_Upht_bus1_C = "n:Upht:bus1:C", "kV, Phase Technology dependent Voltage, Magnitude"
      n_Upht_bus2_A = "n:Upht:bus2:A", "kV, Phase Technology dependent Voltage, Magnitude"
      n_Upht_bus2_B = "n:Upht:bus2:B", "kV, Phase Technology dependent Voltage, Magnitude"
      n_Upht_bus2_C = "n:Upht:bus2:C", "kV, Phase Technology dependent Voltage, Magnitude"
      n_um_bus1 = "n:um:bus1", "p.u., Average-Voltage, Magnitude"
      n_um_bus2 = "n:um:bus2", "p.u., Average-Voltage, Magnitude"
      n_Um_bus1 = "n:Um:bus1", "kV, Average-Voltage, Magnitude"
      n_Um_bus2 = "n:Um:bus2", "kV, Average-Voltage, Magnitude"
      n_uphtmin_bus1 = "n:uphtmin:bus1", "p.u., Minimum of Phase Technology dependent Voltage, Magnitude"
      n_uphtmin_bus2 = "n:uphtmin:bus2", "p.u., Minimum of Phase Technology dependent Voltage, Magnitude"
      n_uphtmax_bus1 = "n:uphtmax:bus1", "p.u., Maximum of Phase Technology dependent Voltage, Magnitude"
      n_uphtmax_bus2 = "n:uphtmax:bus2", "p.u., Maximum of Phase Technology dependent Voltage, Magnitude"
      n_Pgen_bus1 = "n:Pgen:bus1", "MW, Generation, Active Power"
      n_Pgen_bus2 = "n:Pgen:bus2", "MW, Generation, Active Power"
      n_Qgen_bus1 = "n:Qgen:bus1", "Mvar, Generation, Reactive Power"
      n_Qgen_bus2 = "n:Qgen:bus2", "Mvar, Generation, Reactive Power"
      n_Pmot_bus1 = "n:Pmot:bus1", "MW, Motor Load, Active Power"
      n_Pmot_bus2 = "n:Pmot:bus2", "MW, Motor Load, Active Power"
      n_Qmot_bus1 = "n:Qmot:bus1", "Mvar, Motor Load, Reactive Power"
      n_Qmot_bus2 = "n:Qmot:bus2", "Mvar, Motor Load, Reactive Power"
      n_Pload_bus1 = "n:Pload:bus1", "MW, General Load, Active Power"
      n_Pload_bus2 = "n:Pload:bus2", "MW, General Load, Active Power"
      n_Qload_bus1 = "n:Qload:bus1", "Mvar, General Load, Reactive Power"
      n_Qload_bus2 = "n:Qload:bus2", "Mvar, General Load, Reactive Power"
      n_Pcomp_bus1 = "n:Pcomp:bus1", "MW, Compensation (Losses)"
      n_Pcomp_bus2 = "n:Pcomp:bus2", "MW, Compensation (Losses)"
      n_Qcomp_bus1 = "n:Qcomp:bus1", "Mvar, Compensation"
      n_Qcomp_bus2 = "n:Qcomp:bus2", "Mvar, Compensation"
      n_Pnet_bus1 = "n:Pnet:bus1", "MW, External Networks, Active Power"
      n_Pnet_bus2 = "n:Pnet:bus2", "MW, External Networks, Active Power"
      n_Qnet_bus1 = "n:Qnet:bus1", "Mvar, External Networks, Reactive Power"
      n_Qnet_bus2 = "n:Qnet:bus2", "Mvar, External Networks, Reactive Power"
      n_Pflow_bus1 = "n:Pflow:bus1", "MW, Power Flow, Active Power"
      n_Pflow_bus2 = "n:Pflow:bus2", "MW, Power Flow, Active Power"
      n_Qflow_bus1 = "n:Qflow:bus1", "Mvar, Power Flow, Reactive Power"
      n_Qflow_bus2 = "n:Qflow:bus2", "Mvar, Power Flow, Reactive Power"
      n_Pout_bus1 = "n:Pout:bus1", "MW, Outgoing Flow, Active Power"
      n_Pout_bus2 = "n:Pout:bus2", "MW, Outgoing Flow, Active Power"
      n_Qout_bus1 = "n:Qout:bus1", "Mvar, Outgoing Power, Reactive Power"
      n_Qout_bus2 = "n:Qout:bus2", "Mvar, Outgoing Power, Reactive Power"
      n_Sout_bus1 = "n:Sout:bus1", "MVA, Outgoing Power, Apparent Power"
      n_Sout_bus2 = "n:Sout:bus2", "MVA, Outgoing Power, Apparent Power"
      n_cosphiout_bus1 = "n:cosphiout:bus1", "Outgoing Power, Power Factor"
      n_cosphiout_bus2 = "n:cosphiout:bus2", "Outgoing Power, Power Factor"
      n_Pbalance_bus1 = "n:Pbalance:bus1", "MW, Active Power Balance (=0)"
      n_Pbalance_bus2 = "n:Pbalance:bus2", "MW, Active Power Balance (=0)"
      n_Qbalance_bus1 = "n:Qbalance:bus1", "Mvar, Reactive Power Balance (=0)"
      n_Qbalance_bus2 = "n:Qbalance:bus2", "Mvar, Reactive Power Balance (=0)"
      n_ubfac_bus1 = "n:ubfac:bus1", "%, Unbalance Factor"
      n_ubfac_bus2 = "n:ubfac:bus2", "%, Unbalance Factor"
      n_LossPdown_bus1 = "n:LossPdown:bus1", "MW, Losses, downstream"
      n_LossPdown_bus2 = "n:LossPdown:bus2", "MW, Losses, downstream"
      n_LossQdown_bus1 = "n:LossQdown:bus1", "Mvar, Losses, downstream (Reactive Power)"
      n_LossQdown_bus2 = "n:LossQdown:bus2", "Mvar, Losses, downstream (Reactive Power)"
      n_LossPdownload_bus1 = "n:LossPdownload:bus1", "MW, Load losses, downstream"
      n_LossPdownload_bus2 = "n:LossPdownload:bus2", "MW, Load losses, downstream"
      n_LossQdownload_bus1 = "n:LossQdownload:bus1", "Mvar, Load losses, downstream"
      n_LossQdownload_bus2 = "n:LossQdownload:bus2", "Mvar, Load losses, downstream"
      n_LossPdownnoload_bus1 = "n:LossPdownnoload:bus1", "MW, No load losses, downstream"
      n_LossPdownnoload_bus2 = "n:LossPdownnoload:bus2", "MW, No load losses, downstream"
      n_LossQdownnoload_bus1 = "n:LossQdownnoload:bus1", "Mvar, No load losses, downstream"
      n_LossQdownnoload_bus2 = "n:LossQdownnoload:bus2", "Mvar, No load losses, downstream"
      s_Rin = "s:Rin", "Ohm, IN, Resistance (Input), continuous, double"
      s_Xin = "s:Xin", "Ohm, IN, Reactance (Input), continuous, double"
      c_Zind_r_r = "c:Zind_r:r", "Ohm, Impedance, Phase a, Real Part"
      c_Zind_r_i = "c:Zind_r:i", "Ohm, Impedance, Phase a, Imaginary Part"
      c_Zind_r_phi = "c:Zind_r:phi", "deg, Impedance, Phase a, Angle"
      c_Zind_r_mag = "c:Zind_r:mag", "Ohm, Impedance, Phase a, Magnitude"
      c_Zind_r_phirad = "c:Zind_r:phirad", "rad, Impedance, Phase a, Angle"
      c_Zind_s_r = "c:Zind_s:r", "Ohm, Impedance, Phase b, Real Part"
      c_Zind_s_i = "c:Zind_s:i", "Ohm, Impedance, Phase b, Imaginary Part"
      c_Zind_s_phi = "c:Zind_s:phi", "deg, Impedance, Phase b, Angle"
      c_Zind_s_mag = "c:Zind_s:mag", "Ohm, Impedance, Phase b, Magnitude"
      c_Zind_s_phirad = "c:Zind_s:phirad", "rad, Impedance, Phase b, Angle"
      c_Zind_t_r = "c:Zind_t:r", "Ohm, Impedance, Phase c, Real Part"
      c_Zind_t_i = "c:Zind_t:i", "Ohm, Impedance, Phase c, Imaginary Part"
      c_Zind_t_phi = "c:Zind_t:phi", "deg, Impedance, Phase c, Angle"
      c_Zind_t_mag = "c:Zind_t:mag", "Ohm, Impedance, Phase c, Magnitude"
      c_Zind_t_phirad = "c:Zind_t:phirad", "rad, Impedance, Phase c, Angle"
      c_Z0_r = "c:Z0:r", "Ohm, Zero Sequence Impedance, Real Part"
      c_Z0_i = "c:Z0:i", "Ohm, Zero Sequence Impedance, Imaginary Part"
      c_Z0_phi = "c:Z0:phi", "deg, Zero Sequence Impedance, Angle"
      c_Z0_mag = "c:Z0:mag", "Ohm, Zero Sequence Impedance, Magnitude"
      c_Z0_phirad = "c:Z0:phirad", "rad, Zero Sequence Impedance, Angle"
      c_loading = "c:loading", "%, Loading"
      c_loading_st = "c:loading_st", "%, Short Term Loading"
      c_maxload = "c:maxload", "%, Max. Loading"
      c_du_A = "c:du:A", "p.u., Voltage Drop"
      c_du_B = "c:du:B", "p.u., Voltage Drop"
      c_du_C = "c:du:C", "p.u., Voltage Drop"
      c_du_N = "c:du:N", "p.u., Voltage Drop"
      c_dupc_A = "c:dupc:A", "%, Voltage Drop"
      c_dupc_B = "c:dupc:B", "%, Voltage Drop"
      c_dupc_C = "c:dupc:C", "%, Voltage Drop"
      c_dupc_N = "c:dupc:N", "%, Voltage Drop"
      c_dphiu_A = "c:dphiu:A", "deg, Voltage Drop Angle"
      c_dphiu_B = "c:dphiu:B", "deg, Voltage Drop Angle"
      c_dphiu_C = "c:dphiu:C", "deg, Voltage Drop Angle"
      c_dphiu_N = "c:dphiu:N", "deg, Voltage Drop Angle"
      c_du1 = "c:du1", "p.u., Positive Sequence Voltage Drop"
      c_du1pc = "c:du1pc", "%, Positive Sequence Voltage Drop"
      c_dphiu1 = "c:dphiu1", "deg, Positive Sequence Voltage Drop Angle"
      c_Ploss = "c:Ploss", "MW, Losses (total)"
      c_Qloss = "c:Qloss", "Mvar, Reactive-Losses (total)"
      c_Plossld = "c:Plossld", "MW, Losses (load)"
      c_Qlossld = "c:Qlossld", "Mvar, Reactive-Losses (load)"
      c_Plossnld = "c:Plossnld", "MW, Losses (no load)"
      c_Qlossnld = "c:Qlossnld", "Mvar, Reactive-Losses (no load)"
      e_uk = "e:uk", "%, Short-Circuit Voltage uk"
      e_Zd = "e:Zd", "Ohm, Impedance (absolute) Zd"
      e_lrea = "e:lrea", "mH, Inductance, L"
      e_xrea = "e:xrea", "Ohm, Reactance, X"
      e_Pcu = "e:Pcu", "kW, Copper Losses"
      e_ukr = "e:ukr", "%, Short-Circuit Voltage (Re(uk)) ukr"
      e_rrea = "e:rrea", "Ohm, Resistance, R"
      e_R0toR1 = "e:R0toR1", "R0/R1 ratio"
      e_X0toX1 = "e:X0toX1", "X0/X1 ratio"
      e_maxload = "e:maxload", "%, Max. Loading"
      e_lossAssign = "e:lossAssign", "Loss assignment"
      e_cpFeed = "e:cpFeed", "Feeder"
      e_ciDist = "e:ciDist", "Distance from infeed in number of buses"
      e_ciLater = "e:ciLater", "Lateral Index"
      e_ciDistRoot = "e:ciDistRoot", "Distance from first infeed in number of buses"
      e_ciDistAll = "e:ciDistAll", "Distance from infeed in number of buses including switches"
      e_ciDistAllRoot = "e:ciDistAllRoot", "Distance from first infeed in number of buses including switches"
      r_cpFeed = "r:cpFeed", "Feeder"
      m_Ikss_bus1 = "m:Ikss:bus1", "kA, Initial Short-Circuit Current"
      m_Ikss_bus2 = "m:Ikss:bus2", "kA, Initial Short-Circuit Current"
      m_ikss_bus1 = "m:ikss:bus1", "p.u., Initial Short-Circuit Current, Magnitude"
      m_ikss_bus2 = "m:ikss:bus2", "p.u., Initial Short-Circuit Current, Magnitude"
      m_phii_bus1 = "m:phii:bus1", "deg, Phase Current, Angle"
      m_phii_bus2 = "m:phii:bus2", "deg, Phase Current, Angle"
      m_maxIkss_bus1 = "m:maxIkss:bus1", "kA, Max. Initial Short-Circuit Current"
      m_maxIkss_bus2 = "m:maxIkss:bus2", "kA, Max. Initial Short-Circuit Current"
      m_Skss_bus1 = "m:Skss:bus1", "MVA, Initial Short-Circuit Power"
      m_Skss_bus2 = "m:Skss:bus2", "MVA, Initial Short-Circuit Power"
      m_ip_bus1 = "m:ip:bus1", "kA, Peak Short-Circuit Current"
      m_ip_bus2 = "m:ip:bus2", "kA, Peak Short-Circuit Current"
      m_maxip_bus1 = "m:maxip:bus1", "kA, Max. Peak Short-Circuit Current"
      m_maxip_bus2 = "m:maxip:bus2", "kA, Max. Peak Short-Circuit Current"
      m_maxIb_bus1 = "m:maxIb:bus1", "kA, Max. Breaking Current"
      m_maxIb_bus2 = "m:maxIb:bus2", "kA, Max. Breaking Current"
      m_maxIth_bus1 = "m:maxIth:bus1", "kA, Max. Thermal equivalent Short-Circuit Current"
      m_maxIth_bus2 = "m:maxIth:bus2", "kA, Max. Thermal equivalent Short-Circuit Current"
      m_Ib_bus1 = "m:Ib:bus1", "kA, Short-Circuit Breaking Current"
      m_Ib_bus2 = "m:Ib:bus2", "kA, Short-Circuit Breaking Current"
      m_Sb_bus1 = "m:Sb:bus1", "MVA, Short-Circuit Breaking Power"
      m_Sb_bus2 = "m:Sb:bus2", "MVA, Short-Circuit Breaking Power"
      m_Ik_bus1 = "m:Ik:bus1", "kA, Steady-State Short-Circuit Current"
      m_Ik_bus2 = "m:Ik:bus2", "kA, Steady-State Short-Circuit Current"
      m_Ith_bus1 = "m:Ith:bus1", "kA, Thermal equivalent Short-Circuit Current"
      m_Ith_bus2 = "m:Ith:bus2", "kA, Thermal equivalent Short-Circuit Current"
      m_I_bus1 = "m:I:bus1", "kA, Short-Circuit Current"
      m_I_bus2 = "m:I:bus2", "kA, Short-Circuit Current"
      m_R_bus1 = "m:R:bus1", "Ohm, Fault Impedance, Real Part"
      m_R_bus2 = "m:R:bus2", "Ohm, Fault Impedance, Real Part"
      m_X_bus1 = "m:X:bus1", "Ohm, Fault Impedance, Imaginary Part"
      m_X_bus2 = "m:X:bus2", "Ohm, Fault Impedance, Imaginary Part"
      m_Z_bus1 = "m:Z:bus1", "Ohm, Fault Impedance, Magnitude"
      m_Z_bus2 = "m:Z:bus2", "Ohm, Fault Impedance, Magnitude"
      m_phiz_bus1 = "m:phiz:bus1", "deg, Fault Impedance, Angle"
      m_phiz_bus2 = "m:phiz:bus2", "deg, Fault Impedance, Angle"
      m_zSbase_bus1 = "m:zSbase:bus1", "p.u., Fault Impedance, Magnitude"
      m_zSbase_bus2 = "m:zSbase:bus2", "p.u., Fault Impedance, Magnitude"
      m_XtoR_bus1 = "m:XtoR:bus1", "X/R ratio"
      m_XtoR_bus2 = "m:XtoR:bus2", "X/R ratio"
      m_Zs_bus1 = "m:Zs:bus1", "Ohm, Source Impedance, Magnitude"
      m_Zs_bus2 = "m:Zs:bus2", "Ohm, Source Impedance, Magnitude"
      m_phizs_bus1 = "m:phizs:bus1", "deg, Source Impedance, Angle"
      m_phizs_bus2 = "m:phizs:bus2", "deg, Source Impedance, Angle"
      m_Rs_bus1 = "m:Rs:bus1", "Ohm, Source Impedance, Real Part"
      m_Rs_bus2 = "m:Rs:bus2", "Ohm, Source Impedance, Real Part"
      m_Xs_bus1 = "m:Xs:bus1", "Ohm, Source Impedance, Imaginary Part"
      m_Xs_bus2 = "m:Xs:bus2", "Ohm, Source Impedance, Imaginary Part"
      m_phiui_bus1 = "m:phiui:bus1", "deg, Angle between Voltage and Current"
      m_phiui_bus2 = "m:phiui:bus2", "deg, Angle between Voltage and Current"
      m_cosphiui_bus1 = "m:cosphiui:bus1", "cosphi"
      m_cosphiui_bus2 = "m:cosphiui:bus2", "cosphi"
      m_P_bus1 = "m:P:bus1", "MW, Active Power"
      m_P_bus2 = "m:P:bus2", "MW, Active Power"
      m_Q_bus1 = "m:Q:bus1", "Mvar, Reactive Power"
      m_Q_bus2 = "m:Q:bus2", "Mvar, Reactive Power"
      m_BrkIpload_bus1 = "m:BrkIpload:bus1", "%, Breaker Loading (peak)"
      m_BrkIpload_bus2 = "m:BrkIpload:bus2", "%, Breaker Loading (peak)"
      m_BrkIbload_bus1 = "m:BrkIbload:bus1", "%, Breaker Loading (breaking)"
      m_BrkIbload_bus2 = "m:BrkIbload:bus2", "%, Breaker Loading (breaking)"
      m_BrkmaxIb_bus1 = "m:BrkmaxIb:bus1", "kA, Max. Breaking Current (breaker time)"
      m_BrkmaxIb_bus2 = "m:BrkmaxIb:bus2", "kA, Max. Breaking Current (breaker time)"
      n_Ikss_bus1 = "n:Ikss:bus1", "kA, Initial Short-Circuit Current"
      n_Ikss_bus2 = "n:Ikss:bus2", "kA, Initial Short-Circuit Current"
      n_I_bus1 = "n:I:bus1", "kA, Short-Circuit Current"
      n_I_bus2 = "n:I:bus2", "kA, Short-Circuit Current"
      n_phii_bus1 = "n:phii:bus1", "deg, Phase Current, Angle"
      n_phii_bus2 = "n:phii:bus2", "deg, Phase Current, Angle"
      n_Skss_bus1 = "n:Skss:bus1", "MVA, Initial Short-Circuit Power"
      n_Skss_bus2 = "n:Skss:bus2", "MVA, Initial Short-Circuit Power"
      n_ip_bus1 = "n:ip:bus1", "kA, Peak Short-Circuit Current"
      n_ip_bus2 = "n:ip:bus2", "kA, Peak Short-Circuit Current"
      n_Ib_bus1 = "n:Ib:bus1", "kA, Short-Circuit Breaking Current"
      n_Ib_bus2 = "n:Ib:bus2", "kA, Short-Circuit Breaking Current"
      n_Sb_bus1 = "n:Sb:bus1", "MVA, Short-Circuit Breaking Power"
      n_Sb_bus2 = "n:Sb:bus2", "MVA, Short-Circuit Breaking Power"
      n_Ik_bus1 = "n:Ik:bus1", "kA, Steady-State Short-Circuit Current"
      n_Ik_bus2 = "n:Ik:bus2", "kA, Steady-State Short-Circuit Current"
      n_Ith_bus1 = "n:Ith:bus1", "kA, Thermal equivalent Short-Circuit Current"
      n_Ith_bus2 = "n:Ith:bus2", "kA, Thermal equivalent Short-Circuit Current"
      n_R_bus1 = "n:R:bus1", "Ohm, Short-Circuit Impedance, Real Part"
      n_R_bus2 = "n:R:bus2", "Ohm, Short-Circuit Impedance, Real Part"
      n_X_bus1 = "n:X:bus1", "Ohm, Short-Circuit Impedance, Imaginary Part"
      n_X_bus2 = "n:X:bus2", "Ohm, Short-Circuit Impedance, Imaginary Part"
      n_Z_bus1 = "n:Z:bus1", "Ohm, Short-Circuit Impedance, Magnitude"
      n_Z_bus2 = "n:Z:bus2", "Ohm, Short-Circuit Impedance, Magnitude"
      n_phiz_bus1 = "n:phiz:bus1", "deg, Short-Circuit Impedance, Angle"
      n_phiz_bus2 = "n:phiz:bus2", "deg, Short-Circuit Impedance, Angle"
      n_rSbase_bus1 = "n:rSbase:bus1", "p.u., Short-Circuit Impedance, Real Part"
      n_rSbase_bus2 = "n:rSbase:bus2", "p.u., Short-Circuit Impedance, Real Part"
      n_xSbase_bus1 = "n:xSbase:bus1", "p.u., Short-Circuit Impedance, Imaginary Part"
      n_xSbase_bus2 = "n:xSbase:bus2", "p.u., Short-Circuit Impedance, Imaginary Part"
      n_zSbase_bus1 = "n:zSbase:bus1", "p.u., Short-Circuit Impedance, Magnitude"
      n_zSbase_bus2 = "n:zSbase:bus2", "p.u., Short-Circuit Impedance, Magnitude"
      n_XtoR_bus1 = "n:XtoR:bus1", "X/R ratio (peak)"
      n_XtoR_bus2 = "n:XtoR:bus2", "X/R ratio (peak)"
      n_RtoX_bus1 = "n:RtoX:bus1", "R/X ratio (peak)"
      n_RtoX_bus2 = "n:RtoX:bus2", "R/X ratio (peak)"
      n_ur_bus1 = "n:ur:bus1", "p.u., Voltage, Real Part"
      n_ur_bus2 = "n:ur:bus2", "p.u., Voltage, Real Part"
      n_ui_bus1 = "n:ui:bus1", "p.u., Voltage, Imaginary Part"
      n_ui_bus2 = "n:ui:bus2", "p.u., Voltage, Imaginary Part"
      n_u_bus1 = "n:u:bus1", "p.u., Voltage, Magnitude"
      n_u_bus2 = "n:u:bus2", "p.u., Voltage, Magnitude"
      n_upc_bus1 = "n:upc:bus1", "%, Voltage, Magnitude"
      n_upc_bus2 = "n:upc:bus2", "%, Voltage, Magnitude"
      n_U_bus1 = "n:U:bus1", "kV, Line-Ground Voltage, Magnitude"
      n_U_bus2 = "n:U:bus2", "kV, Line-Ground Voltage, Magnitude"
      n_Ul_bus1 = "n:Ul:bus1", "kV, Line-Line Voltage, Magnitude"
      n_Ul_bus2 = "n:Ul:bus2", "kV, Line-Line Voltage, Magnitude"
      n_phiu_bus1 = "n:phiu:bus1", "deg, Voltage, Angle"
      n_phiu_bus2 = "n:phiu:bus2", "deg, Voltage, Angle"
      n_du_bus1 = "n:du:bus1", "%, Voltage Deviation"
      n_du_bus2 = "n:du:bus2", "%, Voltage Deviation"
      n_urpref_bus1 = "n:urpref:bus1", "p.u., Pre-fault Voltage, Real Part"
      n_urpref_bus2 = "n:urpref:bus2", "p.u., Pre-fault Voltage, Real Part"
      n_uipref_bus1 = "n:uipref:bus1", "p.u., Pre-fault Voltage, Imaginary Part"
      n_uipref_bus2 = "n:uipref:bus2", "p.u., Pre-fault Voltage, Imaginary Part"
      n_upref_bus1 = "n:upref:bus1", "p.u., Pre-fault Voltage, Magnitude"
      n_upref_bus2 = "n:upref:bus2", "p.u., Pre-fault Voltage, Magnitude"
      n_uprefpc_bus1 = "n:uprefpc:bus1", "%, Pre-fault Voltage, Magnitude"
      n_uprefpc_bus2 = "n:uprefpc:bus2", "%, Pre-fault Voltage, Magnitude"
      n_Upref_bus1 = "n:Upref:bus1", "kV, Line-Ground Pre-fault Voltage, Magnitude"
      n_Upref_bus2 = "n:Upref:bus2", "kV, Line-Ground Pre-fault Voltage, Magnitude"
      n_Ulpref_bus1 = "n:Ulpref:bus1", "kV, Line-Line Pre-fault Voltage, Magnitude"
      n_Ulpref_bus2 = "n:Ulpref:bus2", "kV, Line-Line Pre-fault Voltage, Magnitude"
      n_phiupref_bus1 = "n:phiupref:bus1", "deg, Pre-fault Voltage, Angle"
      n_phiupref_bus2 = "n:phiupref:bus2", "deg, Pre-fault Voltage, Angle"
      n_phiui_bus1 = "n:phiui:bus1", "deg, Angle between Voltage and Current"
      n_phiui_bus2 = "n:phiui:bus2", "deg, Angle between Voltage and Current"
      n_cfac_bus1 = "n:cfac:bus1", "Voltage Factor"
      n_cfac_bus2 = "n:cfac:bus2", "Voltage Factor"
      n_idc_bus1 = "n:idc:bus1", "kA, Decaying Component"
      n_idc_bus2 = "n:idc:bus2", "kA, Decaying Component"
      n_Iasy_bus1 = "n:Iasy:bus1", "kA, Asymmetrical Breaking Current"
      n_Iasy_bus2 = "n:Iasy:bus2", "kA, Asymmetrical Breaking Current"
      n_Ithload_bus1 = "n:Ithload:bus1", "%, Loading, Thermal equivalent Current"
      n_Ithload_bus2 = "n:Ithload:bus2", "%, Loading, Thermal equivalent Current"
      n_Ipload_bus1 = "n:Ipload:bus1", "%, Loading, Peak Short-Circuit Current"
      n_Ipload_bus2 = "n:Ipload:bus2", "%, Loading, Peak Short-Circuit Current"
      n_Ithrtk_bus1 = "n:Ithrtk:bus1", "kA, Rated Short-Time Current (Tk)"
      n_Ithrtk_bus2 = "n:Ithrtk:bus2", "kA, Rated Short-Time Current (Tk)"
      n_Tn_bus1 = "n:Tn:bus1", "ms, Grid Time Constant"
      n_Tn_bus2 = "n:Tn:bus2", "ms, Grid Time Constant"
      n_XtoR_b_bus1 = "n:XtoR_b:bus1", "X/R ratio (dc)"
      n_XtoR_b_bus2 = "n:XtoR_b:bus2", "X/R ratio (dc)"
      n_RtoX_b_bus1 = "n:RtoX_b:bus1", "R/X ratio (dc)"
      n_RtoX_b_bus2 = "n:RtoX_b:bus2", "R/X ratio (dc)"
      n_fctof_b_bus1 = "n:fctof_b:bus1", "Equivalent Frequency fc/f (dc)"
      n_fctof_b_bus2 = "n:fctof_b:bus2", "Equivalent Frequency fc/f (dc)"
      c_Zind_1_r = "c:Zind_1:r", "Ohm, Impedance, Real Part"
      c_Zind_1_i = "c:Zind_1:i", "Ohm, Impedance, Imaginary Part"
      c_Zind_1_phi = "c:Zind_1:phi", "deg, Impedance, Angle"
      c_Zind_1_mag = "c:Zind_1:mag", "Ohm, Impedance, Magnitude"
      c_Zind_1_phirad = "c:Zind_1:phirad", "rad, Impedance, Angle"
      m_Ikss_bus1_A = "m:Ikss:bus1:A", "kA, Initial Short-Circuit Current"
      m_Ikss_bus1_B = "m:Ikss:bus1:B", "kA, Initial Short-Circuit Current"
      m_Ikss_bus1_C = "m:Ikss:bus1:C", "kA, Initial Short-Circuit Current"
      m_Ikss_bus2_A = "m:Ikss:bus2:A", "kA, Initial Short-Circuit Current"
      m_Ikss_bus2_B = "m:Ikss:bus2:B", "kA, Initial Short-Circuit Current"
      m_Ikss_bus2_C = "m:Ikss:bus2:C", "kA, Initial Short-Circuit Current"
      m_ikss_bus1_A = "m:ikss:bus1:A", "p.u., Initial Short-Circuit Current, Magnitude"
      m_ikss_bus1_B = "m:ikss:bus1:B", "p.u., Initial Short-Circuit Current, Magnitude"
      m_ikss_bus1_C = "m:ikss:bus1:C", "p.u., Initial Short-Circuit Current, Magnitude"
      m_ikss_bus2_A = "m:ikss:bus2:A", "p.u., Initial Short-Circuit Current, Magnitude"
      m_ikss_bus2_B = "m:ikss:bus2:B", "p.u., Initial Short-Circuit Current, Magnitude"
      m_ikss_bus2_C = "m:ikss:bus2:C", "p.u., Initial Short-Circuit Current, Magnitude"
      m_Skss_bus1_A = "m:Skss:bus1:A", "MVA, Initial Short-Circuit Power"
      m_Skss_bus1_B = "m:Skss:bus1:B", "MVA, Initial Short-Circuit Power"
      m_Skss_bus1_C = "m:Skss:bus1:C", "MVA, Initial Short-Circuit Power"
      m_Skss_bus2_A = "m:Skss:bus2:A", "MVA, Initial Short-Circuit Power"
      m_Skss_bus2_B = "m:Skss:bus2:B", "MVA, Initial Short-Circuit Power"
      m_Skss_bus2_C = "m:Skss:bus2:C", "MVA, Initial Short-Circuit Power"
      m_Skssx3_bus1_A = "m:Skssx3:bus1:A", "MVA, 3*Initial Short-Circuit Power"
      m_Skssx3_bus1_B = "m:Skssx3:bus1:B", "MVA, 3*Initial Short-Circuit Power"
      m_Skssx3_bus1_C = "m:Skssx3:bus1:C", "MVA, 3*Initial Short-Circuit Power"
      m_Skssx3_bus2_A = "m:Skssx3:bus2:A", "MVA, 3*Initial Short-Circuit Power"
      m_Skssx3_bus2_B = "m:Skssx3:bus2:B", "MVA, 3*Initial Short-Circuit Power"
      m_Skssx3_bus2_C = "m:Skssx3:bus2:C", "MVA, 3*Initial Short-Circuit Power"
      m_ip_bus1_A = "m:ip:bus1:A", "kA, Peak Short-Circuit Current"
      m_ip_bus1_B = "m:ip:bus1:B", "kA, Peak Short-Circuit Current"
      m_ip_bus1_C = "m:ip:bus1:C", "kA, Peak Short-Circuit Current"
      m_ip_bus2_A = "m:ip:bus2:A", "kA, Peak Short-Circuit Current"
      m_ip_bus2_B = "m:ip:bus2:B", "kA, Peak Short-Circuit Current"
      m_ip_bus2_C = "m:ip:bus2:C", "kA, Peak Short-Circuit Current"
      m_Z_bus1_A = "m:Z:bus1:A", "Ohm, Fault Impedance, Magnitude"
      m_Z_bus1_B = "m:Z:bus1:B", "Ohm, Fault Impedance, Magnitude"
      m_Z_bus1_C = "m:Z:bus1:C", "Ohm, Fault Impedance, Magnitude"
      m_Z_bus2_A = "m:Z:bus2:A", "Ohm, Fault Impedance, Magnitude"
      m_Z_bus2_B = "m:Z:bus2:B", "Ohm, Fault Impedance, Magnitude"
      m_Z_bus2_C = "m:Z:bus2:C", "Ohm, Fault Impedance, Magnitude"
      m_phiz_bus1_A = "m:phiz:bus1:A", "deg, Fault Impedance, Angle"
      m_phiz_bus1_B = "m:phiz:bus1:B", "deg, Fault Impedance, Angle"
      m_phiz_bus1_C = "m:phiz:bus1:C", "deg, Fault Impedance, Angle"
      m_phiz_bus2_A = "m:phiz:bus2:A", "deg, Fault Impedance, Angle"
      m_phiz_bus2_B = "m:phiz:bus2:B", "deg, Fault Impedance, Angle"
      m_phiz_bus2_C = "m:phiz:bus2:C", "deg, Fault Impedance, Angle"
      m_Zl_bus1_A = "m:Zl:bus1:A", "Ohm, Line-Line Fault Impedance, Magnitude"
      m_Zl_bus1_B = "m:Zl:bus1:B", "Ohm, Line-Line Fault Impedance, Magnitude"
      m_Zl_bus1_C = "m:Zl:bus1:C", "Ohm, Line-Line Fault Impedance, Magnitude"
      m_Zl_bus2_A = "m:Zl:bus2:A", "Ohm, Line-Line Fault Impedance, Magnitude"
      m_Zl_bus2_B = "m:Zl:bus2:B", "Ohm, Line-Line Fault Impedance, Magnitude"
      m_Zl_bus2_C = "m:Zl:bus2:C", "Ohm, Line-Line Fault Impedance, Magnitude"
      m_phizl_bus1_A = "m:phizl:bus1:A", "deg, Line-Line Fault Impedance, Angle"
      m_phizl_bus1_B = "m:phizl:bus1:B", "deg, Line-Line Fault Impedance, Angle"
      m_phizl_bus1_C = "m:phizl:bus1:C", "deg, Line-Line Fault Impedance, Angle"
      m_phizl_bus2_A = "m:phizl:bus2:A", "deg, Line-Line Fault Impedance, Angle"
      m_phizl_bus2_B = "m:phizl:bus2:B", "deg, Line-Line Fault Impedance, Angle"
      m_phizl_bus2_C = "m:phizl:bus2:C", "deg, Line-Line Fault Impedance, Angle"
      m_Z0_bus1 = "m:Z0:bus1", "Ohm, Zero-Sequence Fault Impedance, Magnitude"
      m_Z0_bus2 = "m:Z0:bus2", "Ohm, Zero-Sequence Fault Impedance, Magnitude"
      m_phiz0_bus1 = "m:phiz0:bus1", "deg, Zero-Sequence Fault Impedance, Angle"
      m_phiz0_bus2 = "m:phiz0:bus2", "deg, Zero-Sequence Fault Impedance, Angle"
      m_z0Sbase_bus1 = "m:z0Sbase:bus1", "p.u., Zero-Sequence Fault Impedance, Magnitude"
      m_z0Sbase_bus2 = "m:z0Sbase:bus2", "p.u., Zero-Sequence Fault Impedance, Magnitude"
      m_cosphiu1i1_bus1 = "m:cosphiu1i1:bus1", "Positive-Sequence cosphi"
      m_cosphiu1i1_bus2 = "m:cosphiu1i1:bus2", "Positive-Sequence cosphi"
      m_I0x3watt_bus1 = "m:I0x3watt:bus1", "kA, Earth-Fault Residual Current"
      m_I0x3watt_bus2 = "m:I0x3watt:bus2", "kA, Earth-Fault Residual Current"
      m_I0x3var_bus1 = "m:I0x3var:bus1", "kA, Capacitive Earth-Fault Current"
      m_I0x3var_bus2 = "m:I0x3var:bus2", "kA, Capacitive Earth-Fault Current"
      m_Q0cap_bus1 = "m:Q0cap:bus1", "Mvar, Q0 capacitive"
      m_Q0cap_bus2 = "m:Q0cap:bus2", "Mvar, Q0 capacitive"
      m_Zfrel_bus1 = "m:Zfrel:bus1", "Ohm, Relay Impedance, Magnitude"
      m_Zfrel_bus2 = "m:Zfrel:bus2", "Ohm, Relay Impedance, Magnitude"
      m_phizfrel_bus1 = "m:phizfrel:bus1", "deg, Relay Impedance, Angle"
      m_phizfrel_bus2 = "m:phizfrel:bus2", "deg, Relay Impedance, Angle"
      m_Rfrel_bus1 = "m:Rfrel:bus1", "Ohm, Relay Impedance, Real Part"
      m_Rfrel_bus2 = "m:Rfrel:bus2", "Ohm, Relay Impedance, Real Part"
      m_Xfrel_bus1 = "m:Xfrel:bus1", "Ohm, Relay Impedance, Imaginary Part"
      m_Xfrel_bus2 = "m:Xfrel:bus2", "Ohm, Relay Impedance, Imaginary Part"
      m_Zf_bus1 = "m:Zf:bus1", "Ohm, Meas. Fault Impedance, Magnitude"
      m_Zf_bus2 = "m:Zf:bus2", "Ohm, Meas. Fault Impedance, Magnitude"
      m_phizf_bus1 = "m:phizf:bus1", "deg, Meas. Fault Impedance, Angle"
      m_phizf_bus2 = "m:phizf:bus2", "deg, Meas. Fault Impedance, Angle"
      n_Ikss_bus1_A = "n:Ikss:bus1:A", "kA, Initial Short-Circuit Current"
      n_Ikss_bus1_B = "n:Ikss:bus1:B", "kA, Initial Short-Circuit Current"
      n_Ikss_bus1_C = "n:Ikss:bus1:C", "kA, Initial Short-Circuit Current"
      n_Ikss_bus2_A = "n:Ikss:bus2:A", "kA, Initial Short-Circuit Current"
      n_Ikss_bus2_B = "n:Ikss:bus2:B", "kA, Initial Short-Circuit Current"
      n_Ikss_bus2_C = "n:Ikss:bus2:C", "kA, Initial Short-Circuit Current"
      n_I_bus1_A = "n:I:bus1:A", "kA, Short-Circuit Current"
      n_I_bus1_B = "n:I:bus1:B", "kA, Short-Circuit Current"
      n_I_bus1_C = "n:I:bus1:C", "kA, Short-Circuit Current"
      n_I_bus2_A = "n:I:bus2:A", "kA, Short-Circuit Current"
      n_I_bus2_B = "n:I:bus2:B", "kA, Short-Circuit Current"
      n_I_bus2_C = "n:I:bus2:C", "kA, Short-Circuit Current"
      n_phii_bus1_A = "n:phii:bus1:A", "deg, Phase Current, Angle"
      n_phii_bus1_B = "n:phii:bus1:B", "deg, Phase Current, Angle"
      n_phii_bus1_C = "n:phii:bus1:C", "deg, Phase Current, Angle"
      n_phii_bus2_A = "n:phii:bus2:A", "deg, Phase Current, Angle"
      n_phii_bus2_B = "n:phii:bus2:B", "deg, Phase Current, Angle"
      n_phii_bus2_C = "n:phii:bus2:C", "deg, Phase Current, Angle"
      n_Skss_bus1_A = "n:Skss:bus1:A", "MVA, Initial Short-Circuit Power"
      n_Skss_bus1_B = "n:Skss:bus1:B", "MVA, Initial Short-Circuit Power"
      n_Skss_bus1_C = "n:Skss:bus1:C", "MVA, Initial Short-Circuit Power"
      n_Skss_bus2_A = "n:Skss:bus2:A", "MVA, Initial Short-Circuit Power"
      n_Skss_bus2_B = "n:Skss:bus2:B", "MVA, Initial Short-Circuit Power"
      n_Skss_bus2_C = "n:Skss:bus2:C", "MVA, Initial Short-Circuit Power"
      n_Skssx3_bus1_A = "n:Skssx3:bus1:A", "MVA, 3*Initial Short-Circuit Power"
      n_Skssx3_bus1_B = "n:Skssx3:bus1:B", "MVA, 3*Initial Short-Circuit Power"
      n_Skssx3_bus1_C = "n:Skssx3:bus1:C", "MVA, 3*Initial Short-Circuit Power"
      n_Skssx3_bus2_A = "n:Skssx3:bus2:A", "MVA, 3*Initial Short-Circuit Power"
      n_Skssx3_bus2_B = "n:Skssx3:bus2:B", "MVA, 3*Initial Short-Circuit Power"
      n_Skssx3_bus2_C = "n:Skssx3:bus2:C", "MVA, 3*Initial Short-Circuit Power"
      n_ip_bus1_A = "n:ip:bus1:A", "kA, Peak Short-Circuit Current"
      n_ip_bus1_B = "n:ip:bus1:B", "kA, Peak Short-Circuit Current"
      n_ip_bus1_C = "n:ip:bus1:C", "kA, Peak Short-Circuit Current"
      n_ip_bus2_A = "n:ip:bus2:A", "kA, Peak Short-Circuit Current"
      n_ip_bus2_B = "n:ip:bus2:B", "kA, Peak Short-Circuit Current"
      n_ip_bus2_C = "n:ip:bus2:C", "kA, Peak Short-Circuit Current"
      n_Ib_bus1_A = "n:Ib:bus1:A", "kA, Short-Circuit Breaking Current"
      n_Ib_bus1_B = "n:Ib:bus1:B", "kA, Short-Circuit Breaking Current"
      n_Ib_bus1_C = "n:Ib:bus1:C", "kA, Short-Circuit Breaking Current"
      n_Ib_bus2_A = "n:Ib:bus2:A", "kA, Short-Circuit Breaking Current"
      n_Ib_bus2_B = "n:Ib:bus2:B", "kA, Short-Circuit Breaking Current"
      n_Ib_bus2_C = "n:Ib:bus2:C", "kA, Short-Circuit Breaking Current"
      n_Sb_bus1_A = "n:Sb:bus1:A", "MVA, Short-Circuit Breaking Power"
      n_Sb_bus1_B = "n:Sb:bus1:B", "MVA, Short-Circuit Breaking Power"
      n_Sb_bus1_C = "n:Sb:bus1:C", "MVA, Short-Circuit Breaking Power"
      n_Sb_bus2_A = "n:Sb:bus2:A", "MVA, Short-Circuit Breaking Power"
      n_Sb_bus2_B = "n:Sb:bus2:B", "MVA, Short-Circuit Breaking Power"
      n_Sb_bus2_C = "n:Sb:bus2:C", "MVA, Short-Circuit Breaking Power"
      n_Ith_bus1_A = "n:Ith:bus1:A", "kA, Thermal equivalent Short-Circuit Current"
      n_Ith_bus1_B = "n:Ith:bus1:B", "kA, Thermal equivalent Short-Circuit Current"
      n_Ith_bus1_C = "n:Ith:bus1:C", "kA, Thermal equivalent Short-Circuit Current"
      n_Ith_bus2_A = "n:Ith:bus2:A", "kA, Thermal equivalent Short-Circuit Current"
      n_Ith_bus2_B = "n:Ith:bus2:B", "kA, Thermal equivalent Short-Circuit Current"
      n_Ith_bus2_C = "n:Ith:bus2:C", "kA, Thermal equivalent Short-Circuit Current"
      n_Ithmax_bus1 = "n:Ithmax:bus1", "kA, Max. Thermal equivalent Short-Circuit Current"
      n_Ithmax_bus2 = "n:Ithmax:bus2", "kA, Max. Thermal equivalent Short-Circuit Current"
      n_Ik_bus1_A = "n:Ik:bus1:A", "kA, Steady-State Short-Circuit Current"
      n_Ik_bus1_B = "n:Ik:bus1:B", "kA, Steady-State Short-Circuit Current"
      n_Ik_bus1_C = "n:Ik:bus1:C", "kA, Steady-State Short-Circuit Current"
      n_Ik_bus2_A = "n:Ik:bus2:A", "kA, Steady-State Short-Circuit Current"
      n_Ik_bus2_B = "n:Ik:bus2:B", "kA, Steady-State Short-Circuit Current"
      n_Ik_bus2_C = "n:Ik:bus2:C", "kA, Steady-State Short-Circuit Current"
      n_R0_bus1 = "n:R0:bus1", "Ohm, Zero-Sequence Impedance, Real Part"
      n_R0_bus2 = "n:R0:bus2", "Ohm, Zero-Sequence Impedance, Real Part"
      n_X0_bus1 = "n:X0:bus1", "Ohm, Zero-Sequence Impedance, Imaginary Part"
      n_X0_bus2 = "n:X0:bus2", "Ohm, Zero-Sequence Impedance, Imaginary Part"
      n_Z0_bus1 = "n:Z0:bus1", "Ohm, Zero-Sequence Impedance, Magnitude"
      n_Z0_bus2 = "n:Z0:bus2", "Ohm, Zero-Sequence Impedance, Magnitude"
      n_phiz0_bus1 = "n:phiz0:bus1", "deg, Zero-Sequence Impedance, Angle"
      n_phiz0_bus2 = "n:phiz0:bus2", "deg, Zero-Sequence Impedance, Angle"
      n_r0Sbase_bus1 = "n:r0Sbase:bus1", "p.u., Zero-Sequence Impedance, Real Part"
      n_r0Sbase_bus2 = "n:r0Sbase:bus2", "p.u., Zero-Sequence Impedance, Real Part"
      n_x0Sbase_bus1 = "n:x0Sbase:bus1", "p.u., Zero-Sequence Impedance, Imaginary Part"
      n_x0Sbase_bus2 = "n:x0Sbase:bus2", "p.u., Zero-Sequence Impedance, Imaginary Part"
      n_z0Sbase_bus1 = "n:z0Sbase:bus1", "p.u., Zero-Sequence Impedance, Magnitude"
      n_z0Sbase_bus2 = "n:z0Sbase:bus2", "p.u., Zero-Sequence Impedance, Magnitude"
      n_R1_bus1 = "n:R1:bus1", "Ohm, Positive-Sequence Impedance, Real Part"
      n_R1_bus2 = "n:R1:bus2", "Ohm, Positive-Sequence Impedance, Real Part"
      n_X1_bus1 = "n:X1:bus1", "Ohm, Positive-Sequence Impedance, Imaginary Part"
      n_X1_bus2 = "n:X1:bus2", "Ohm, Positive-Sequence Impedance, Imaginary Part"
      n_Z1_bus1 = "n:Z1:bus1", "Ohm, Positive-Sequence Impedance, Magnitude"
      n_Z1_bus2 = "n:Z1:bus2", "Ohm, Positive-Sequence Impedance, Magnitude"
      n_phiz1_bus1 = "n:phiz1:bus1", "deg, Positive-Sequence Impedance, Angle"
      n_phiz1_bus2 = "n:phiz1:bus2", "deg, Positive-Sequence Impedance, Angle"
      n_r1Sbase_bus1 = "n:r1Sbase:bus1", "p.u., Positive-Sequence Impedance, Real Part"
      n_r1Sbase_bus2 = "n:r1Sbase:bus2", "p.u., Positive-Sequence Impedance, Real Part"
      n_x1Sbase_bus1 = "n:x1Sbase:bus1", "p.u., Positive-Sequence Impedance, Imaginary Part"
      n_x1Sbase_bus2 = "n:x1Sbase:bus2", "p.u., Positive-Sequence Impedance, Imaginary Part"
      n_z1Sbase_bus1 = "n:z1Sbase:bus1", "p.u., Positive-Sequence Impedance, Magnitude"
      n_z1Sbase_bus2 = "n:z1Sbase:bus2", "p.u., Positive-Sequence Impedance, Magnitude"
      n_R2_bus1 = "n:R2:bus1", "Ohm, Negative-Sequence Impedance, Real Part"
      n_R2_bus2 = "n:R2:bus2", "Ohm, Negative-Sequence Impedance, Real Part"
      n_X2_bus1 = "n:X2:bus1", "Ohm, Negative-Sequence Impedance, Imaginary Part"
      n_X2_bus2 = "n:X2:bus2", "Ohm, Negative-Sequence Impedance, Imaginary Part"
      n_Z2_bus1 = "n:Z2:bus1", "Ohm, Negative-Sequence Impedance, Magnitude"
      n_Z2_bus2 = "n:Z2:bus2", "Ohm, Negative-Sequence Impedance, Magnitude"
      n_phiz2_bus1 = "n:phiz2:bus1", "deg, Negative-Sequence Impedance, Angle"
      n_phiz2_bus2 = "n:phiz2:bus2", "deg, Negative-Sequence Impedance, Angle"
      n_r2Sbase_bus1 = "n:r2Sbase:bus1", "p.u., Negative-Sequence Impedance, Real Part"
      n_r2Sbase_bus2 = "n:r2Sbase:bus2", "p.u., Negative-Sequence Impedance, Real Part"
      n_x2Sbase_bus1 = "n:x2Sbase:bus1", "p.u., Negative-Sequence Impedance, Imaginary Part"
      n_x2Sbase_bus2 = "n:x2Sbase:bus2", "p.u., Negative-Sequence Impedance, Imaginary Part"
      n_z2Sbase_bus1 = "n:z2Sbase:bus1", "p.u., Negative-Sequence Impedance, Magnitude"
      n_z2Sbase_bus2 = "n:z2Sbase:bus2", "p.u., Negative-Sequence Impedance, Magnitude"
      n_R_bus1_A = "n:R:bus1:A", "Ohm, Network Resistance, Line-Ground"
      n_R_bus1_B = "n:R:bus1:B", "Ohm, Network Resistance, Line-Ground"
      n_R_bus1_C = "n:R:bus1:C", "Ohm, Network Resistance, Line-Ground"
      n_R_bus2_A = "n:R:bus2:A", "Ohm, Network Resistance, Line-Ground"
      n_R_bus2_B = "n:R:bus2:B", "Ohm, Network Resistance, Line-Ground"
      n_R_bus2_C = "n:R:bus2:C", "Ohm, Network Resistance, Line-Ground"
      n_X_bus1_A = "n:X:bus1:A", "Ohm, Network Reactance, Line-Ground"
      n_X_bus1_B = "n:X:bus1:B", "Ohm, Network Reactance, Line-Ground"
      n_X_bus1_C = "n:X:bus1:C", "Ohm, Network Reactance, Line-Ground"
      n_X_bus2_A = "n:X:bus2:A", "Ohm, Network Reactance, Line-Ground"
      n_X_bus2_B = "n:X:bus2:B", "Ohm, Network Reactance, Line-Ground"
      n_X_bus2_C = "n:X:bus2:C", "Ohm, Network Reactance, Line-Ground"
      n_Z_bus1_A = "n:Z:bus1:A", "Ohm, Network Impedance, Line-Ground"
      n_Z_bus1_B = "n:Z:bus1:B", "Ohm, Network Impedance, Line-Ground"
      n_Z_bus1_C = "n:Z:bus1:C", "Ohm, Network Impedance, Line-Ground"
      n_Z_bus2_A = "n:Z:bus2:A", "Ohm, Network Impedance, Line-Ground"
      n_Z_bus2_B = "n:Z:bus2:B", "Ohm, Network Impedance, Line-Ground"
      n_Z_bus2_C = "n:Z:bus2:C", "Ohm, Network Impedance, Line-Ground"
      n_phiz_bus1_A = "n:phiz:bus1:A", "deg, Angle of Network Impedance, Line-Ground"
      n_phiz_bus1_B = "n:phiz:bus1:B", "deg, Angle of Network Impedance, Line-Ground"
      n_phiz_bus1_C = "n:phiz:bus1:C", "deg, Angle of Network Impedance, Line-Ground"
      n_phiz_bus2_A = "n:phiz:bus2:A", "deg, Angle of Network Impedance, Line-Ground"
      n_phiz_bus2_B = "n:phiz:bus2:B", "deg, Angle of Network Impedance, Line-Ground"
      n_phiz_bus2_C = "n:phiz:bus2:C", "deg, Angle of Network Impedance, Line-Ground"
      n_Rl_bus1_A = "n:Rl:bus1:A", "Ohm, Network Resistance, Line-Line"
      n_Rl_bus1_B = "n:Rl:bus1:B", "Ohm, Network Resistance, Line-Line"
      n_Rl_bus1_C = "n:Rl:bus1:C", "Ohm, Network Resistance, Line-Line"
      n_Rl_bus2_A = "n:Rl:bus2:A", "Ohm, Network Resistance, Line-Line"
      n_Rl_bus2_B = "n:Rl:bus2:B", "Ohm, Network Resistance, Line-Line"
      n_Rl_bus2_C = "n:Rl:bus2:C", "Ohm, Network Resistance, Line-Line"
      n_Xl_bus1_A = "n:Xl:bus1:A", "Ohm, Network Reactance, Line-Line"
      n_Xl_bus1_B = "n:Xl:bus1:B", "Ohm, Network Reactance, Line-Line"
      n_Xl_bus1_C = "n:Xl:bus1:C", "Ohm, Network Reactance, Line-Line"
      n_Xl_bus2_A = "n:Xl:bus2:A", "Ohm, Network Reactance, Line-Line"
      n_Xl_bus2_B = "n:Xl:bus2:B", "Ohm, Network Reactance, Line-Line"
      n_Xl_bus2_C = "n:Xl:bus2:C", "Ohm, Network Reactance, Line-Line"
      n_Zl_bus1_A = "n:Zl:bus1:A", "Ohm, Network Impedance, Line-Line"
      n_Zl_bus1_B = "n:Zl:bus1:B", "Ohm, Network Impedance, Line-Line"
      n_Zl_bus1_C = "n:Zl:bus1:C", "Ohm, Network Impedance, Line-Line"
      n_Zl_bus2_A = "n:Zl:bus2:A", "Ohm, Network Impedance, Line-Line"
      n_Zl_bus2_B = "n:Zl:bus2:B", "Ohm, Network Impedance, Line-Line"
      n_Zl_bus2_C = "n:Zl:bus2:C", "Ohm, Network Impedance, Line-Line"
      n_phizl_bus1_A = "n:phizl:bus1:A", "deg, Angle of Network Impedance, Line-Line"
      n_phizl_bus1_B = "n:phizl:bus1:B", "deg, Angle of Network Impedance, Line-Line"
      n_phizl_bus1_C = "n:phizl:bus1:C", "deg, Angle of Network Impedance, Line-Line"
      n_phizl_bus2_A = "n:phizl:bus2:A", "deg, Angle of Network Impedance, Line-Line"
      n_phizl_bus2_B = "n:phizl:bus2:B", "deg, Angle of Network Impedance, Line-Line"
      n_phizl_bus2_C = "n:phizl:bus2:C", "deg, Angle of Network Impedance, Line-Line"
      n_Rln_bus1_A = "n:Rln:bus1:A", "Ohm, Network Resistance, Line-Neutral"
      n_Rln_bus1_B = "n:Rln:bus1:B", "Ohm, Network Resistance, Line-Neutral"
      n_Rln_bus1_C = "n:Rln:bus1:C", "Ohm, Network Resistance, Line-Neutral"
      n_Rln_bus2_A = "n:Rln:bus2:A", "Ohm, Network Resistance, Line-Neutral"
      n_Rln_bus2_B = "n:Rln:bus2:B", "Ohm, Network Resistance, Line-Neutral"
      n_Rln_bus2_C = "n:Rln:bus2:C", "Ohm, Network Resistance, Line-Neutral"
      n_Xln_bus1_A = "n:Xln:bus1:A", "Ohm, Network Reactance, Line-Neutral"
      n_Xln_bus1_B = "n:Xln:bus1:B", "Ohm, Network Reactance, Line-Neutral"
      n_Xln_bus1_C = "n:Xln:bus1:C", "Ohm, Network Reactance, Line-Neutral"
      n_Xln_bus2_A = "n:Xln:bus2:A", "Ohm, Network Reactance, Line-Neutral"
      n_Xln_bus2_B = "n:Xln:bus2:B", "Ohm, Network Reactance, Line-Neutral"
      n_Xln_bus2_C = "n:Xln:bus2:C", "Ohm, Network Reactance, Line-Neutral"
      n_Zln_bus1_A = "n:Zln:bus1:A", "Ohm, Network Impedance, Line-Neutral"
      n_Zln_bus1_B = "n:Zln:bus1:B", "Ohm, Network Impedance, Line-Neutral"
      n_Zln_bus1_C = "n:Zln:bus1:C", "Ohm, Network Impedance, Line-Neutral"
      n_Zln_bus2_A = "n:Zln:bus2:A", "Ohm, Network Impedance, Line-Neutral"
      n_Zln_bus2_B = "n:Zln:bus2:B", "Ohm, Network Impedance, Line-Neutral"
      n_Zln_bus2_C = "n:Zln:bus2:C", "Ohm, Network Impedance, Line-Neutral"
      n_phizln_bus1_A = "n:phizln:bus1:A", "deg, Angle of Network Impedance, Line-Neutral"
      n_phizln_bus1_B = "n:phizln:bus1:B", "deg, Angle of Network Impedance, Line-Neutral"
      n_phizln_bus1_C = "n:phizln:bus1:C", "deg, Angle of Network Impedance, Line-Neutral"
      n_phizln_bus2_A = "n:phizln:bus2:A", "deg, Angle of Network Impedance, Line-Neutral"
      n_phizln_bus2_B = "n:phizln:bus2:B", "deg, Angle of Network Impedance, Line-Neutral"
      n_phizln_bus2_C = "n:phizln:bus2:C", "deg, Angle of Network Impedance, Line-Neutral"
      n_Rn_bus1 = "n:Rn:bus1", "Ohm, Network Resistance, Neutral"
      n_Rn_bus2 = "n:Rn:bus2", "Ohm, Network Resistance, Neutral"
      n_Xn_bus1 = "n:Xn:bus1", "Ohm, Network Reactance, Neutral"
      n_Xn_bus2 = "n:Xn:bus2", "Ohm, Network Reactance, Neutral"
      n_Zn_bus1 = "n:Zn:bus1", "Ohm, Network Impedance, Neutral"
      n_Zn_bus2 = "n:Zn:bus2", "Ohm, Network Impedance, Neutral"
      n_phizn_bus1 = "n:phizn:bus1", "deg, Angle of Network Impedance, Neutral"
      n_phizn_bus2 = "n:phizn:bus2", "deg, Angle of Network Impedance, Neutral"
      n_eff_bus1_A = "n:eff:bus1:A", "Earth Fault Factor"
      n_eff_bus1_B = "n:eff:bus1:B", "Earth Fault Factor"
      n_eff_bus1_C = "n:eff:bus1:C", "Earth Fault Factor"
      n_eff_bus2_A = "n:eff:bus2:A", "Earth Fault Factor"
      n_eff_bus2_B = "n:eff:bus2:B", "Earth Fault Factor"
      n_eff_bus2_C = "n:eff:bus2:C", "Earth Fault Factor"
      n_X0toX1_bus1 = "n:X0toX1:bus1", "X0/X1"
      n_X0toX1_bus2 = "n:X0toX1:bus2", "X0/X1"
      n_in_bus1 = "n:in:bus1", "p.u., Neutral Current, Magnitude"
      n_in_bus2 = "n:in:bus2", "p.u., Neutral Current, Magnitude"
      n_In_bus1 = "n:In:bus1", "kA, Neutral Current, Magnitude"
      n_In_bus2 = "n:In:bus2", "kA, Neutral Current, Magnitude"
      n_phiin_bus1 = "n:phiin:bus1", "deg, Neutral Current, Angle"
      n_phiin_bus2 = "n:phiin:bus2", "deg, Neutral Current, Angle"
      n_i0r_bus1 = "n:i0r:bus1", "p.u., Zero-Sequence Current, Real Part"
      n_i0r_bus2 = "n:i0r:bus2", "p.u., Zero-Sequence Current, Real Part"
      n_i0i_bus1 = "n:i0i:bus1", "p.u., Zero-Sequence Current, Imaginary Part"
      n_i0i_bus2 = "n:i0i:bus2", "p.u., Zero-Sequence Current, Imaginary Part"
      n_i0_bus1 = "n:i0:bus1", "p.u., Zero-Sequence Current, Magnitude"
      n_i0_bus2 = "n:i0:bus2", "p.u., Zero-Sequence Current, Magnitude"
      n_I0_bus1 = "n:I0:bus1", "kA, Zero-Sequence Current, Magnitude"
      n_I0_bus2 = "n:I0:bus2", "kA, Zero-Sequence Current, Magnitude"
      n_I0x3_bus1 = "n:I0x3:bus1", "kA, 3*I0"
      n_I0x3_bus2 = "n:I0x3:bus2", "kA, 3*I0"
      n_phii0_bus1 = "n:phii0:bus1", "deg, Zero-Sequence Current, Angle"
      n_phii0_bus2 = "n:phii0:bus2", "deg, Zero-Sequence Current, Angle"
      n_i1r_bus1 = "n:i1r:bus1", "p.u., Positive-Sequence Current, Real Part"
      n_i1r_bus2 = "n:i1r:bus2", "p.u., Positive-Sequence Current, Real Part"
      n_i1i_bus1 = "n:i1i:bus1", "p.u., Positive-Sequence Current, Imaginary Part"
      n_i1i_bus2 = "n:i1i:bus2", "p.u., Positive-Sequence Current, Imaginary Part"
      n_i1_bus1 = "n:i1:bus1", "p.u., Positive-Sequence Current, Magnitude"
      n_i1_bus2 = "n:i1:bus2", "p.u., Positive-Sequence Current, Magnitude"
      n_I1_bus1 = "n:I1:bus1", "kA, Positive-Sequence Current, Magnitude"
      n_I1_bus2 = "n:I1:bus2", "kA, Positive-Sequence Current, Magnitude"
      n_phii1_bus1 = "n:phii1:bus1", "deg, Positive-Sequence Current, Angle"
      n_phii1_bus2 = "n:phii1:bus2", "deg, Positive-Sequence Current, Angle"
      n_i2r_bus1 = "n:i2r:bus1", "p.u., Negative-Sequence Current, Real Part"
      n_i2r_bus2 = "n:i2r:bus2", "p.u., Negative-Sequence Current, Real Part"
      n_i2i_bus1 = "n:i2i:bus1", "p.u., Negative-Sequence Current, Imaginary Part"
      n_i2i_bus2 = "n:i2i:bus2", "p.u., Negative-Sequence Current, Imaginary Part"
      n_i2_bus1 = "n:i2:bus1", "p.u., Negative-Sequence Current, Magnitude"
      n_i2_bus2 = "n:i2:bus2", "p.u., Negative-Sequence Current, Magnitude"
      n_I2_bus1 = "n:I2:bus1", "kA, Negative-Sequence Current, Magnitude"
      n_I2_bus2 = "n:I2:bus2", "kA, Negative-Sequence Current, Magnitude"
      n_phii2_bus1 = "n:phii2:bus1", "deg, Negative-Sequence Current, Angle"
      n_phii2_bus2 = "n:phii2:bus2", "deg, Negative-Sequence Current, Angle"
      n_du_bus1_A = "n:du:bus1:A", "%, Voltage Deviation"
      n_du_bus1_B = "n:du:bus1:B", "%, Voltage Deviation"
      n_du_bus1_C = "n:du:bus1:C", "%, Voltage Deviation"
      n_du_bus2_A = "n:du:bus2:A", "%, Voltage Deviation"
      n_du_bus2_B = "n:du:bus2:B", "%, Voltage Deviation"
      n_du_bus2_C = "n:du:bus2:C", "%, Voltage Deviation"
      n_urpref_bus1_A = "n:urpref:bus1:A", "p.u., Pre-fault Voltage, Real Part"
      n_urpref_bus1_B = "n:urpref:bus1:B", "p.u., Pre-fault Voltage, Real Part"
      n_urpref_bus1_C = "n:urpref:bus1:C", "p.u., Pre-fault Voltage, Real Part"
      n_urpref_bus2_A = "n:urpref:bus2:A", "p.u., Pre-fault Voltage, Real Part"
      n_urpref_bus2_B = "n:urpref:bus2:B", "p.u., Pre-fault Voltage, Real Part"
      n_urpref_bus2_C = "n:urpref:bus2:C", "p.u., Pre-fault Voltage, Real Part"
      n_uipref_bus1_A = "n:uipref:bus1:A", "p.u., Pre-fault Voltage, Imaginary Part"
      n_uipref_bus1_B = "n:uipref:bus1:B", "p.u., Pre-fault Voltage, Imaginary Part"
      n_uipref_bus1_C = "n:uipref:bus1:C", "p.u., Pre-fault Voltage, Imaginary Part"
      n_uipref_bus2_A = "n:uipref:bus2:A", "p.u., Pre-fault Voltage, Imaginary Part"
      n_uipref_bus2_B = "n:uipref:bus2:B", "p.u., Pre-fault Voltage, Imaginary Part"
      n_uipref_bus2_C = "n:uipref:bus2:C", "p.u., Pre-fault Voltage, Imaginary Part"
      n_upref_bus1_A = "n:upref:bus1:A", "p.u., Pre-fault Voltage, Magnitude"
      n_upref_bus1_B = "n:upref:bus1:B", "p.u., Pre-fault Voltage, Magnitude"
      n_upref_bus1_C = "n:upref:bus1:C", "p.u., Pre-fault Voltage, Magnitude"
      n_upref_bus2_A = "n:upref:bus2:A", "p.u., Pre-fault Voltage, Magnitude"
      n_upref_bus2_B = "n:upref:bus2:B", "p.u., Pre-fault Voltage, Magnitude"
      n_upref_bus2_C = "n:upref:bus2:C", "p.u., Pre-fault Voltage, Magnitude"
      n_uprefpc_bus1_A = "n:uprefpc:bus1:A", "%, Pre-fault Voltage, Magnitude"
      n_uprefpc_bus1_B = "n:uprefpc:bus1:B", "%, Pre-fault Voltage, Magnitude"
      n_uprefpc_bus1_C = "n:uprefpc:bus1:C", "%, Pre-fault Voltage, Magnitude"
      n_uprefpc_bus2_A = "n:uprefpc:bus2:A", "%, Pre-fault Voltage, Magnitude"
      n_uprefpc_bus2_B = "n:uprefpc:bus2:B", "%, Pre-fault Voltage, Magnitude"
      n_uprefpc_bus2_C = "n:uprefpc:bus2:C", "%, Pre-fault Voltage, Magnitude"
      n_Upref_bus1_A = "n:Upref:bus1:A", "kV, Line-Ground Pre-fault Voltage, Magnitude"
      n_Upref_bus1_B = "n:Upref:bus1:B", "kV, Line-Ground Pre-fault Voltage, Magnitude"
      n_Upref_bus1_C = "n:Upref:bus1:C", "kV, Line-Ground Pre-fault Voltage, Magnitude"
      n_Upref_bus2_A = "n:Upref:bus2:A", "kV, Line-Ground Pre-fault Voltage, Magnitude"
      n_Upref_bus2_B = "n:Upref:bus2:B", "kV, Line-Ground Pre-fault Voltage, Magnitude"
      n_Upref_bus2_C = "n:Upref:bus2:C", "kV, Line-Ground Pre-fault Voltage, Magnitude"
      n_Ulpref_bus1_A = "n:Ulpref:bus1:A", "kV, Line-Line Pre-fault Voltage, Magnitude"
      n_Ulpref_bus1_B = "n:Ulpref:bus1:B", "kV, Line-Line Pre-fault Voltage, Magnitude"
      n_Ulpref_bus1_C = "n:Ulpref:bus1:C", "kV, Line-Line Pre-fault Voltage, Magnitude"
      n_Ulpref_bus2_A = "n:Ulpref:bus2:A", "kV, Line-Line Pre-fault Voltage, Magnitude"
      n_Ulpref_bus2_B = "n:Ulpref:bus2:B", "kV, Line-Line Pre-fault Voltage, Magnitude"
      n_Ulpref_bus2_C = "n:Ulpref:bus2:C", "kV, Line-Line Pre-fault Voltage, Magnitude"
      n_phiupref_bus1_A = "n:phiupref:bus1:A", "deg, Pre-fault Voltage, Angle"
      n_phiupref_bus1_B = "n:phiupref:bus1:B", "deg, Pre-fault Voltage, Angle"
      n_phiupref_bus1_C = "n:phiupref:bus1:C", "deg, Pre-fault Voltage, Angle"
      n_phiupref_bus2_A = "n:phiupref:bus2:A", "deg, Pre-fault Voltage, Angle"
      n_phiupref_bus2_B = "n:phiupref:bus2:B", "deg, Pre-fault Voltage, Angle"
      n_phiupref_bus2_C = "n:phiupref:bus2:C", "deg, Pre-fault Voltage, Angle"
      n_phiui_bus1_A = "n:phiui:bus1:A", "deg, Angle between Voltage and Current"
      n_phiui_bus1_B = "n:phiui:bus1:B", "deg, Angle between Voltage and Current"
      n_phiui_bus1_C = "n:phiui:bus1:C", "deg, Angle between Voltage and Current"
      n_phiui_bus2_A = "n:phiui:bus2:A", "deg, Angle between Voltage and Current"
      n_phiui_bus2_B = "n:phiui:bus2:B", "deg, Angle between Voltage and Current"
      n_phiui_bus2_C = "n:phiui:bus2:C", "deg, Angle between Voltage and Current"
      n_phiu1i1_bus1 = "n:phiu1i1:bus1", "deg, Angle between Voltage and Current in positive sequence system"
      n_phiu1i1_bus2 = "n:phiu1i1:bus2", "deg, Angle between Voltage and Current in positive sequence system"
      n_phiu2i2_bus1 = "n:phiu2i2:bus1", "deg, Angle between Voltage and Current in negative sequence system"
      n_phiu2i2_bus2 = "n:phiu2i2:bus2", "deg, Angle between Voltage and Current in negative sequence system"
      n_phiu0i0_bus1 = "n:phiu0i0:bus1", "deg, Angle between Voltage and Current in zero-sequence system"
      n_phiu0i0_bus2 = "n:phiu0i0:bus2", "deg, Angle between Voltage and Current in zero-sequence system"

    class ElmStactrl(Enum):
      _init_ = 'value __doc__'

      c_rembar = "c:rembar", "Controlled Busbar"
      c_i_phase = "c:i_phase", "Controlled Phase"
      c_uini_r = "c:uini:r", "p.u., Voltage Setpoint, Real Part"
      c_uini_i = "c:uini:i", "p.u., Voltage Setpoint, Imaginary Part"
      c_uini_phi = "c:uini:phi", "deg, Voltage Setpoint, Angle"
      c_uini_mag = "c:uini:mag", "p.u., Voltage Setpoint, Magnitude"
      c_uini_phirad = "c:uini:phirad", "rad, Voltage Setpoint, Angle"
      c_qset = "c:qset", "Reactive Power Percentage"
      c_qres = "c:qres", "Mvar, Reactive Power Reserve"
      c_imode = "c:imode", "Control Mode"
      c_i_ctrl = "c:i_ctrl", "Load Flow Control"
      c_qini = "c:qini", "Mvar, Reactive Power Setpoint"
      c_Qdmeas = "c:Qdmeas", "Mvar, Q(measured) for Droop"
      c_Qdroop = "c:Qdroop", "Mvar/p.u., Droop"
      c_umin = "c:umin", "p.u., Lower Deadband Limit"
      c_umax = "c:umax", "p.u., Upper Deadband Limit"
      c_Qmin = "c:Qmin", "Mvar, Lower limit of reactive power"
      c_Qmax = "c:Qmax", "Mvar, Upper limit of reactive power"
      c_i_qlim = "c:i_qlim", "Reactive power limit reached"
      c_nCtrlIter = "c:nCtrlIter", "No. of CTRL-iterations"
      e_i_ctrl = "e:i_ctrl", "Control Mode"
      e_i_phase = "e:i_phase", "Phase"
      e_qu_char = "e:qu_char", "Q-Control"
      e_cosphi_char = "e:cosphi_char", "PF-Control"
      e_selBus = "e:selBus", "Controlled Node"
      e_uset_mode = "e:uset_mode", "Setpoint"
      e_rembar = "e:rembar", "StaBar*,ElmTerm*, Controlled Node"
      e_cpCtrlNode = "e:cpCtrlNode", "Target Node"
      e_usetp = "e:usetp", "p.u., Voltage Setpoint"
      e_selAutoUn = "e:selAutoUn", "kV, Busbar Search Criteria >="
      e_p_cub = "e:p_cub", "StaCubic,ElmBoundary, Control Q at"
      e_qsetp = "e:qsetp", "Mvar, Q Setpoint"
      e_tansetp = "e:tansetp", "tan(phi)"
      e_pfsetp = "e:pfsetp", "Power Factor"
      e_pf_recap = "e:pf_recap", "Power Factor"
      e_iQorient = "e:iQorient", "Orientation"
      e_i_droop = "e:i_droop", "Enable Droop"
      e_Srated = "e:Srated", "Mvar, Rated Reactive Power"
      e_ddroop = "e:ddroop", "%, Droop"
      e_Qdroop = "e:Qdroop", "Mvar/p.u., Droop"
      e_deltaV = "e:deltaV", "p.u., delta(V)"
      e_pQmeas = "e:pQmeas", "StaCubic,ElmBoundary, Q measured at"
      e_refbar = "e:refbar", "StaBar*,ElmTerm*, Reference Node"
      e_iopt_tdr = "e:iopt_tdr", "Different droop values"
      e_Qdroopoe = "e:Qdroopoe", "Mvar/p.u., Droop (overexcited)"
      e_Qdroopue = "e:Qdroopue", "Mvar/p.u., Droop (underexcited)"
      e_ddroopoe = "e:ddroopoe", "%, Droop (overexcited)"
      e_ddroopue = "e:ddroopue", "%, Droop (underexcited)"
      e_deltaVoe = "e:deltaVoe", "p.u., delta(V) (overexcited)"
      e_deltaVue = "e:deltaVue", "p.u., delta(V) (underexcited)"
      e_Qmin = "e:Qmin", "Mvar, Qmin"
      e_Qmax = "e:Qmax", "Mvar, Qmax"
      e_udeadbup = "e:udeadbup", "p.u., Upper Voltage Limit"
      e_udeadblow = "e:udeadblow", "p.u., Lower Voltage Limit"
      e_pQPcurve = "e:pQPcurve", "IntQpcurve, Q(P)-Curve"
      e_pf_over = "e:pf_over", "Min. Power Factor"
      e_p_over = "e:p_over", "MW, Active Power"
      e_pf_under = "e:pf_under", "Min. Power Factor"
      e_p_under = "e:p_under", "MW, Active Power"
      e_u_over = "e:u_over", "p.u., Voltage"
      e_u_under = "e:u_under", "p.u., Voltage"
      e_imode = "e:imode", "Reactive Power Distribution"
      e_consQdisp = "e:consQdisp", "Consider reactive power dispatch"
      e_iTrfCtrl = "e:iTrfCtrl", "Step-up Transformer Control"
      e_Tctrl = "e:Tctrl", "s, Controller Time Constant"
      e_psym = "e:psym", "ElmSym,ElmGenstat,ElmPvsys,ElmSvs,ElmAsm,ElmVsc*,ElmXnet, Machines"
      e_cvqq = "e:cvqq", "%, Reactive Power Percentage"
      e_cvgen = "e:cvgen", "p.u., Voltage Setpoint"
      e_cvgenmin = "e:cvgenmin", "p.u., Min. Voltage"
      e_cvgenmax = "e:cvgenmax", "p.u., Max. Voltage"
      r_rembar = "r:rembar", "StaBar*,ElmTerm*, Controlled Node"
      r_cpCtrlNode = "r:cpCtrlNode", "Target Node"
      r_p_cub = "r:p_cub", "StaCubic,ElmBoundary, Control Q at"
      r_pQmeas = "r:pQmeas", "StaCubic,ElmBoundary, Q measured at"
      r_refbar = "r:refbar", "StaBar*,ElmTerm*, Reference Node"
      r_pQPcurve = "r:pQPcurve", "IntQpcurve, Q(P)-Curve"
      r_psym = "r:psym", "ElmSym,ElmGenstat,ElmPvsys,ElmSvs,ElmAsm,ElmVsc*,ElmXnet, Machines"

    class ElmSym(Enum):
      _init_ = 'value __doc__'

      m_u0_bus1 = "m:u0:bus1", "p.u., Zero-Sequence-Voltage, Magnitude"
      m_u0_bus1n = "m:u0:bus1n", "p.u., Zero-Sequence-Voltage, Magnitude"
      m_phiu0_bus1 = "m:phiu0:bus1", "deg, Zero-Sequence-Voltage, Angle"
      m_phiu0_bus1n = "m:phiu0:bus1n", "deg, Zero-Sequence-Voltage, Angle"
      m_u1r_bus1 = "m:u1r:bus1", "p.u., Positive-Sequence-Voltage, Real Part"
      m_u1r_bus1n = "m:u1r:bus1n", "p.u., Positive-Sequence-Voltage, Real Part"
      m_u1i_bus1 = "m:u1i:bus1", "p.u., Positive-Sequence-Voltage, Imaginary Part"
      m_u1i_bus1n = "m:u1i:bus1n", "p.u., Positive-Sequence-Voltage, Imaginary Part"
      m_u1_bus1 = "m:u1:bus1", "p.u., Positive-Sequence-Voltage, Magnitude"
      m_u1_bus1n = "m:u1:bus1n", "p.u., Positive-Sequence-Voltage, Magnitude"
      m_U1_bus1 = "m:U1:bus1", "kV, Line-Ground Positive-Sequence-Voltage, Magnitude"
      m_U1_bus1n = "m:U1:bus1n", "kV, Line-Ground Positive-Sequence-Voltage, Magnitude"
      m_U1l_bus1 = "m:U1l:bus1", "kV, Line-Line Positive-Sequence-Voltage, Magnitude"
      m_U1l_bus1n = "m:U1l:bus1n", "kV, Line-Line Positive-Sequence-Voltage, Magnitude"
      m_phiu1_bus1 = "m:phiu1:bus1", "deg, Positive-Sequence-Voltage, Angle"
      m_phiu1_bus1n = "m:phiu1:bus1n", "deg, Positive-Sequence-Voltage, Angle"
      m_u2_bus1 = "m:u2:bus1", "p.u., Negative-Sequence-Voltage, Magnitude"
      m_u2_bus1n = "m:u2:bus1n", "p.u., Negative-Sequence-Voltage, Magnitude"
      m_phiu2_bus1 = "m:phiu2:bus1", "deg, Negative-Sequence-Voltage, Angle"
      m_phiu2_bus1n = "m:phiu2:bus1n", "deg, Negative-Sequence-Voltage, Angle"
      m_inet_bus1_A = "m:inet:bus1:A", "p.u., Phase Current, Magnitude, referred to network"
      m_inet_bus1_B = "m:inet:bus1:B", "p.u., Phase Current, Magnitude, referred to network"
      m_inet_bus1_C = "m:inet:bus1:C", "p.u., Phase Current, Magnitude, referred to network"
      m_inet_bus1_N = "m:inet:bus1:N", "p.u., Phase Current, Magnitude, referred to network"
      m_inet_bus1n = "m:inet:bus1n", "p.u., Phase Current, Magnitude, referred to network"
      m_I_bus1_A = "m:I:bus1:A", "kA, Phase Current, Magnitude"
      m_I_bus1_B = "m:I:bus1:B", "kA, Phase Current, Magnitude"
      m_I_bus1_C = "m:I:bus1:C", "kA, Phase Current, Magnitude"
      m_I_bus1_N = "m:I:bus1:N", "kA, Phase Current, Magnitude"
      m_I_bus1n = "m:I:bus1n", "kA, Phase Current, Magnitude"
      m_phii_bus1_A = "m:phii:bus1:A", "deg, Phase Current, Angle"
      m_phii_bus1_B = "m:phii:bus1:B", "deg, Phase Current, Angle"
      m_phii_bus1_C = "m:phii:bus1:C", "deg, Phase Current, Angle"
      m_phii_bus1_N = "m:phii:bus1:N", "deg, Phase Current, Angle"
      m_phii_bus1n = "m:phii:bus1n", "deg, Phase Current, Angle"
      m_i0r_bus1 = "m:i0r:bus1", "p.u., Zero-Sequence Current, Real Part"
      m_i0r_bus1n = "m:i0r:bus1n", "p.u., Zero-Sequence Current, Real Part"
      m_i0i_bus1 = "m:i0i:bus1", "p.u., Zero-Sequence Current, Imaginary Part"
      m_i0i_bus1n = "m:i0i:bus1n", "p.u., Zero-Sequence Current, Imaginary Part"
      m_i0_bus1 = "m:i0:bus1", "p.u., Zero-Sequence Current, Magnitude"
      m_i0_bus1n = "m:i0:bus1n", "p.u., Zero-Sequence Current, Magnitude"
      m_I0_bus1 = "m:I0:bus1", "kA, Zero-Sequence Current, Magnitude"
      m_I0_bus1n = "m:I0:bus1n", "kA, Zero-Sequence Current, Magnitude"
      m_I0x3_bus1 = "m:I0x3:bus1", "kA, 3*I0"
      m_I0x3_bus1n = "m:I0x3:bus1n", "kA, 3*I0"
      m_phii0_bus1 = "m:phii0:bus1", "deg, Zero-Sequence Current, Angle"
      m_phii0_bus1n = "m:phii0:bus1n", "deg, Zero-Sequence Current, Angle"
      m_i1r_bus1 = "m:i1r:bus1", "p.u., Positive-Sequence Current, Real Part"
      m_i1r_bus1n = "m:i1r:bus1n", "p.u., Positive-Sequence Current, Real Part"
      m_i1i_bus1 = "m:i1i:bus1", "p.u., Positive-Sequence Current, Imaginary Part"
      m_i1i_bus1n = "m:i1i:bus1n", "p.u., Positive-Sequence Current, Imaginary Part"
      m_i1_bus1 = "m:i1:bus1", "p.u., Positive-Sequence Current, Magnitude"
      m_i1_bus1n = "m:i1:bus1n", "p.u., Positive-Sequence Current, Magnitude"
      m_I1_bus1 = "m:I1:bus1", "kA, Positive-Sequence Current, Magnitude"
      m_I1_bus1n = "m:I1:bus1n", "kA, Positive-Sequence Current, Magnitude"
      m_phii1_bus1 = "m:phii1:bus1", "deg, Positive-Sequence Current, Angle"
      m_phii1_bus1n = "m:phii1:bus1n", "deg, Positive-Sequence Current, Angle"
      m_i2r_bus1 = "m:i2r:bus1", "p.u., Negative-Sequence Current, Real Part"
      m_i2r_bus1n = "m:i2r:bus1n", "p.u., Negative-Sequence Current, Real Part"
      m_i2i_bus1 = "m:i2i:bus1", "p.u., Negative-Sequence Current, Imaginary Part"
      m_i2i_bus1n = "m:i2i:bus1n", "p.u., Negative-Sequence Current, Imaginary Part"
      m_i2_bus1 = "m:i2:bus1", "p.u., Negative-Sequence Current, Magnitude"
      m_i2_bus1n = "m:i2:bus1n", "p.u., Negative-Sequence Current, Magnitude"
      m_I2_bus1 = "m:I2:bus1", "kA, Negative-Sequence Current, Magnitude"
      m_I2_bus1n = "m:I2:bus1n", "kA, Negative-Sequence Current, Magnitude"
      m_phii2_bus1 = "m:phii2:bus1", "deg, Negative-Sequence Current, Angle"
      m_phii2_bus1n = "m:phii2:bus1n", "deg, Negative-Sequence Current, Angle"
      m_ubfacI_bus1 = "m:ubfacI:bus1", "%, Current Unbalance Factor"
      m_ubfacI_bus1n = "m:ubfacI:bus1n", "%, Current Unbalance Factor"
      m_ubfacS_bus1 = "m:ubfacS:bus1", "%, Power Unbalance Factor"
      m_ubfacS_bus1n = "m:ubfacS:bus1n", "%, Power Unbalance Factor"
      m_P_bus1_A = "m:P:bus1:A", "MW, Active Power"
      m_P_bus1_B = "m:P:bus1:B", "MW, Active Power"
      m_P_bus1_C = "m:P:bus1:C", "MW, Active Power"
      m_P_bus1_N = "m:P:bus1:N", "MW, Active Power"
      m_P_bus1n = "m:P:bus1n", "MW, Active Power"
      m_Q_bus1_A = "m:Q:bus1:A", "Mvar, Reactive Power"
      m_Q_bus1_B = "m:Q:bus1:B", "Mvar, Reactive Power"
      m_Q_bus1_C = "m:Q:bus1:C", "Mvar, Reactive Power"
      m_Q_bus1_N = "m:Q:bus1:N", "Mvar, Reactive Power"
      m_Q_bus1n = "m:Q:bus1n", "Mvar, Reactive Power"
      m_S_bus1_A = "m:S:bus1:A", "MVA, Apparent Power"
      m_S_bus1_B = "m:S:bus1:B", "MVA, Apparent Power"
      m_S_bus1_C = "m:S:bus1:C", "MVA, Apparent Power"
      m_S_bus1_N = "m:S:bus1:N", "MVA, Apparent Power"
      m_S_bus1n = "m:S:bus1n", "MVA, Apparent Power"
      m_cosphi_bus1_A = "m:cosphi:bus1:A", "Power Factor"
      m_cosphi_bus1_B = "m:cosphi:bus1:B", "Power Factor"
      m_cosphi_bus1_C = "m:cosphi:bus1:C", "Power Factor"
      m_cosphi_bus1_N = "m:cosphi:bus1:N", "Power Factor"
      m_cosphi_bus1n = "m:cosphi:bus1n", "Power Factor"
      m_tanphi_bus1_A = "m:tanphi:bus1:A", "tan(phi)"
      m_tanphi_bus1_B = "m:tanphi:bus1:B", "tan(phi)"
      m_tanphi_bus1_C = "m:tanphi:bus1:C", "tan(phi)"
      m_tanphi_bus1_N = "m:tanphi:bus1:N", "tan(phi)"
      m_tanphi_bus1n = "m:tanphi:bus1n", "tan(phi)"
      m_Psum_bus1 = "m:Psum:bus1", "MW, Total Active Power"
      m_Psum_bus1n = "m:Psum:bus1n", "MW, Total Active Power"
      m_Qsum_bus1 = "m:Qsum:bus1", "Mvar, Total Reactive Power"
      m_Qsum_bus1n = "m:Qsum:bus1n", "Mvar, Total Reactive Power"
      m_Ssum_bus1 = "m:Ssum:bus1", "MVA, Total Apparent Power"
      m_Ssum_bus1n = "m:Ssum:bus1n", "MVA, Total Apparent Power"
      m_cosphisum_bus1 = "m:cosphisum:bus1", "Total Power Factor"
      m_cosphisum_bus1n = "m:cosphisum:bus1n", "Total Power Factor"
      m_tanphisum_bus1 = "m:tanphisum:bus1", "Total tan(phi)"
      m_tanphisum_bus1n = "m:tanphisum:bus1n", "Total tan(phi)"
      m_Pln_bus1_A = "m:Pln:bus1:A", "MW, Active Power (Line-Neutral)"
      m_Pln_bus1_B = "m:Pln:bus1:B", "MW, Active Power (Line-Neutral)"
      m_Pln_bus1_C = "m:Pln:bus1:C", "MW, Active Power (Line-Neutral)"
      m_Pln_bus1_N = "m:Pln:bus1:N", "MW, Active Power (Line-Neutral)"
      m_Pln_bus1n = "m:Pln:bus1n", "MW, Active Power (Line-Neutral)"
      m_Qln_bus1_A = "m:Qln:bus1:A", "Mvar, Reactive Power (Line-Neutral)"
      m_Qln_bus1_B = "m:Qln:bus1:B", "Mvar, Reactive Power (Line-Neutral)"
      m_Qln_bus1_C = "m:Qln:bus1:C", "Mvar, Reactive Power (Line-Neutral)"
      m_Qln_bus1_N = "m:Qln:bus1:N", "Mvar, Reactive Power (Line-Neutral)"
      m_Qln_bus1n = "m:Qln:bus1n", "Mvar, Reactive Power (Line-Neutral)"
      m_P1_bus1 = "m:P1:bus1", "MW, Positive Sequence Active Power"
      m_P1_bus1n = "m:P1:bus1n", "MW, Positive Sequence Active Power"
      m_Q1_bus1 = "m:Q1:bus1", "Mvar, Positive Sequence Reactive Power"
      m_Q1_bus1n = "m:Q1:bus1n", "Mvar, Positive Sequence Reactive Power"
      m_P2_bus1 = "m:P2:bus1", "MW, Negative Sequence Active Power"
      m_P2_bus1n = "m:P2:bus1n", "MW, Negative Sequence Active Power"
      m_Q2_bus1 = "m:Q2:bus1", "Mvar, Negative Sequence Reactive Power"
      m_Q2_bus1n = "m:Q2:bus1n", "Mvar, Negative Sequence Reactive Power"
      m_P0_bus1 = "m:P0:bus1", "MW, Zero Sequence Active Power"
      m_P0_bus1n = "m:P0:bus1n", "MW, Zero Sequence Active Power"
      m_Q0_bus1 = "m:Q0:bus1", "Mvar, Zero Sequence Reactive Power"
      m_Q0_bus1n = "m:Q0:bus1n", "Mvar, Zero Sequence Reactive Power"
      m_i1P_bus1 = "m:i1P:bus1", "p.u., Positive-Sequence Active Current"
      m_i1P_bus1n = "m:i1P:bus1n", "p.u., Positive-Sequence Active Current"
      m_i1Q_bus1 = "m:i1Q:bus1", "p.u., Positive-Sequence Reactive Current"
      m_i1Q_bus1n = "m:i1Q:bus1n", "p.u., Positive-Sequence Reactive Current"
      m_I1P_bus1 = "m:I1P:bus1", "kA, Positive-Sequence Active Current"
      m_I1P_bus1n = "m:I1P:bus1n", "kA, Positive-Sequence Active Current"
      m_I1Q_bus1 = "m:I1Q:bus1", "kA, Positive-Sequence Reactive Current"
      m_I1Q_bus1n = "m:I1Q:bus1n", "kA, Positive-Sequence Reactive Current"
      m_i2P_bus1 = "m:i2P:bus1", "p.u., Negative-Sequence Active Current"
      m_i2P_bus1n = "m:i2P:bus1n", "p.u., Negative-Sequence Active Current"
      m_i2Q_bus1 = "m:i2Q:bus1", "p.u., Negative-Sequence Reactive Current"
      m_i2Q_bus1n = "m:i2Q:bus1n", "p.u., Negative-Sequence Reactive Current"
      m_I2P_bus1 = "m:I2P:bus1", "kA, Negative-Sequence Active Current"
      m_I2P_bus1n = "m:I2P:bus1n", "kA, Negative-Sequence Active Current"
      m_I2Q_bus1 = "m:I2Q:bus1", "kA, Negative-Sequence Reactive Current"
      m_I2Q_bus1n = "m:I2Q:bus1n", "kA, Negative-Sequence Reactive Current"
      m_ur_bus1_A = "m:ur:bus1:A", "p.u., Phase Voltage, Real Part"
      m_ur_bus1_B = "m:ur:bus1:B", "p.u., Phase Voltage, Real Part"
      m_ur_bus1_C = "m:ur:bus1:C", "p.u., Phase Voltage, Real Part"
      m_ur_bus1_N = "m:ur:bus1:N", "p.u., Phase Voltage, Real Part"
      m_ur_bus1n = "m:ur:bus1n", "p.u., Phase Voltage, Real Part"
      m_ui_bus1_A = "m:ui:bus1:A", "p.u., Phase Voltage, Imaginary Part"
      m_ui_bus1_B = "m:ui:bus1:B", "p.u., Phase Voltage, Imaginary Part"
      m_ui_bus1_C = "m:ui:bus1:C", "p.u., Phase Voltage, Imaginary Part"
      m_ui_bus1_N = "m:ui:bus1:N", "p.u., Phase Voltage, Imaginary Part"
      m_ui_bus1n = "m:ui:bus1n", "p.u., Phase Voltage, Imaginary Part"
      m_u_bus1_A = "m:u:bus1:A", "p.u., Phase Voltage, Magnitude"
      m_u_bus1_B = "m:u:bus1:B", "p.u., Phase Voltage, Magnitude"
      m_u_bus1_C = "m:u:bus1:C", "p.u., Phase Voltage, Magnitude"
      m_u_bus1_N = "m:u:bus1:N", "p.u., Phase Voltage, Magnitude"
      m_u_bus1n = "m:u:bus1n", "p.u., Phase Voltage, Magnitude"
      m_ir_bus1_A = "m:ir:bus1:A", "p.u., Phase Current, Real Part"
      m_ir_bus1_B = "m:ir:bus1:B", "p.u., Phase Current, Real Part"
      m_ir_bus1_C = "m:ir:bus1:C", "p.u., Phase Current, Real Part"
      m_ir_bus1_N = "m:ir:bus1:N", "p.u., Phase Current, Real Part"
      m_ir_bus1n = "m:ir:bus1n", "p.u., Phase Current, Real Part"
      m_ii_bus1_A = "m:ii:bus1:A", "p.u., Phase Current, Imaginary Part"
      m_ii_bus1_B = "m:ii:bus1:B", "p.u., Phase Current, Imaginary Part"
      m_ii_bus1_C = "m:ii:bus1:C", "p.u., Phase Current, Imaginary Part"
      m_ii_bus1_N = "m:ii:bus1:N", "p.u., Phase Current, Imaginary Part"
      m_ii_bus1n = "m:ii:bus1n", "p.u., Phase Current, Imaginary Part"
      m_i_bus1_A = "m:i:bus1:A", "p.u., Phase Current, Magnitude"
      m_i_bus1_B = "m:i:bus1:B", "p.u., Phase Current, Magnitude"
      m_i_bus1_C = "m:i:bus1:C", "p.u., Phase Current, Magnitude"
      m_i_bus1_N = "m:i:bus1:N", "p.u., Phase Current, Magnitude"
      m_i_bus1n = "m:i:bus1n", "p.u., Phase Current, Magnitude"
      m_phiui_bus1_A = "m:phiui:bus1:A", "deg, Angle between Voltage and Current"
      m_phiui_bus1_B = "m:phiui:bus1:B", "deg, Angle between Voltage and Current"
      m_phiui_bus1_C = "m:phiui:bus1:C", "deg, Angle between Voltage and Current"
      m_phiui_bus1_N = "m:phiui:bus1:N", "deg, Angle between Voltage and Current"
      m_phiui_bus1n = "m:phiui:bus1n", "deg, Angle between Voltage and Current"
      m_phiu0i0_bus1 = "m:phiu0i0:bus1", "deg, Angle between Voltage and Current in zero-sequence system"
      m_phiu0i0_bus1n = "m:phiu0i0:bus1n", "deg, Angle between Voltage and Current in zero-sequence system"
      m_phiu1i1_bus1 = "m:phiu1i1:bus1", "deg, Angle between Voltage and Current in positive sequence system"
      m_phiu1i1_bus1n = "m:phiu1i1:bus1n", "deg, Angle between Voltage and Current in positive sequence system"
      m_phiu2i2_bus1 = "m:phiu2i2:bus1", "deg, Angle between Voltage and Current in negative sequence system"
      m_phiu2i2_bus1n = "m:phiu2i2:bus1n", "deg, Angle between Voltage and Current in negative sequence system"
      m_Tfct_bus1 = "m:Tfct:bus1", "s, Fault Clearing Time"
      m_Tfct_bus1n = "m:Tfct:bus1n", "s, Fault Clearing Time"
      m_TfctPh_bus1_A = "m:TfctPh:bus1:A", "s, Fault Clearing Time"
      m_TfctPh_bus1_B = "m:TfctPh:bus1:B", "s, Fault Clearing Time"
      m_TfctPh_bus1_C = "m:TfctPh:bus1:C", "s, Fault Clearing Time"
      m_TfctPh_bus1_N = "m:TfctPh:bus1:N", "s, Fault Clearing Time"
      m_TfctPh_bus1n = "m:TfctPh:bus1n", "s, Fault Clearing Time"
      m_Brkload_bus1 = "m:Brkload:bus1", "%, Max. Breaker Loading"
      m_Brkload_bus1n = "m:Brkload:bus1n", "%, Max. Breaker Loading"
      m_BrkloadPh_bus1_A = "m:BrkloadPh:bus1:A", "%, Breaker Loading"
      m_BrkloadPh_bus1_B = "m:BrkloadPh:bus1:B", "%, Breaker Loading"
      m_BrkloadPh_bus1_C = "m:BrkloadPh:bus1:C", "%, Breaker Loading"
      m_BrkloadPh_bus1_N = "m:BrkloadPh:bus1:N", "%, Breaker Loading"
      m_BrkloadPh_bus1n = "m:BrkloadPh:bus1n", "%, Breaker Loading"
      n_ur_bus1_A = "n:ur:bus1:A", "p.u., Line-Ground Voltage, Real Part"
      n_ur_bus1_B = "n:ur:bus1:B", "p.u., Line-Ground Voltage, Real Part"
      n_ur_bus1_C = "n:ur:bus1:C", "p.u., Line-Ground Voltage, Real Part"
      n_ur_bus1_N = "n:ur:bus1:N", "p.u., Line-Ground Voltage, Real Part"
      n_ur_bus1n = "n:ur:bus1n", "p.u., Line-Ground Voltage, Real Part"
      n_ui_bus1_A = "n:ui:bus1:A", "p.u., Line-Ground Voltage, Imaginary Part"
      n_ui_bus1_B = "n:ui:bus1:B", "p.u., Line-Ground Voltage, Imaginary Part"
      n_ui_bus1_C = "n:ui:bus1:C", "p.u., Line-Ground Voltage, Imaginary Part"
      n_ui_bus1_N = "n:ui:bus1:N", "p.u., Line-Ground Voltage, Imaginary Part"
      n_ui_bus1n = "n:ui:bus1n", "p.u., Line-Ground Voltage, Imaginary Part"
      n_u_bus1_A = "n:u:bus1:A", "p.u., Line-Ground Voltage, Magnitude"
      n_u_bus1_B = "n:u:bus1:B", "p.u., Line-Ground Voltage, Magnitude"
      n_u_bus1_C = "n:u:bus1:C", "p.u., Line-Ground Voltage, Magnitude"
      n_u_bus1_N = "n:u:bus1:N", "p.u., Line-Ground Voltage, Magnitude"
      n_u_bus1n = "n:u:bus1n", "p.u., Line-Ground Voltage, Magnitude"
      n_upc_bus1_A = "n:upc:bus1:A", "%, Line-Ground Voltage, Magnitude"
      n_upc_bus1_B = "n:upc:bus1:B", "%, Line-Ground Voltage, Magnitude"
      n_upc_bus1_C = "n:upc:bus1:C", "%, Line-Ground Voltage, Magnitude"
      n_upc_bus1_N = "n:upc:bus1:N", "%, Line-Ground Voltage, Magnitude"
      n_upc_bus1n = "n:upc:bus1n", "%, Line-Ground Voltage, Magnitude"
      n_U_bus1_A = "n:U:bus1:A", "kV, Line-Ground Voltage, Magnitude"
      n_U_bus1_B = "n:U:bus1:B", "kV, Line-Ground Voltage, Magnitude"
      n_U_bus1_C = "n:U:bus1:C", "kV, Line-Ground Voltage, Magnitude"
      n_U_bus1_N = "n:U:bus1:N", "kV, Line-Ground Voltage, Magnitude"
      n_U_bus1n = "n:U:bus1n", "kV, Line-Ground Voltage, Magnitude"
      n_phiu_bus1_A = "n:phiu:bus1:A", "deg, Line-Ground Voltage, Angle"
      n_phiu_bus1_B = "n:phiu:bus1:B", "deg, Line-Ground Voltage, Angle"
      n_phiu_bus1_C = "n:phiu:bus1:C", "deg, Line-Ground Voltage, Angle"
      n_phiu_bus1_N = "n:phiu:bus1:N", "deg, Line-Ground Voltage, Angle"
      n_phiu_bus1n = "n:phiu:bus1n", "deg, Line-Ground Voltage, Angle"
      n_phiurel_bus1 = "n:phiurel:bus1", "deg, Voltage, Relative Angle"
      n_phiurel_bus1n = "n:phiurel:bus1n", "deg, Voltage, Relative Angle"
      n_ul_bus1_A = "n:ul:bus1:A", "p.u., Line-Line Voltage, Magnitude"
      n_ul_bus1_B = "n:ul:bus1:B", "p.u., Line-Line Voltage, Magnitude"
      n_ul_bus1_C = "n:ul:bus1:C", "p.u., Line-Line Voltage, Magnitude"
      n_ul_bus1_N = "n:ul:bus1:N", "p.u., Line-Line Voltage, Magnitude"
      n_ul_bus1n = "n:ul:bus1n", "p.u., Line-Line Voltage, Magnitude"
      n_ulpc_bus1_A = "n:ulpc:bus1:A", "%, Line-Line Voltage, Magnitude"
      n_ulpc_bus1_B = "n:ulpc:bus1:B", "%, Line-Line Voltage, Magnitude"
      n_ulpc_bus1_C = "n:ulpc:bus1:C", "%, Line-Line Voltage, Magnitude"
      n_ulpc_bus1_N = "n:ulpc:bus1:N", "%, Line-Line Voltage, Magnitude"
      n_ulpc_bus1n = "n:ulpc:bus1n", "%, Line-Line Voltage, Magnitude"
      n_Ul_bus1_A = "n:Ul:bus1:A", "kV, Line-Line Voltage, Magnitude"
      n_Ul_bus1_B = "n:Ul:bus1:B", "kV, Line-Line Voltage, Magnitude"
      n_Ul_bus1_C = "n:Ul:bus1:C", "kV, Line-Line Voltage, Magnitude"
      n_Ul_bus1_N = "n:Ul:bus1:N", "kV, Line-Line Voltage, Magnitude"
      n_Ul_bus1n = "n:Ul:bus1n", "kV, Line-Line Voltage, Magnitude"
      n_phiul_bus1_A = "n:phiul:bus1:A", "deg, Line-Line Voltage, Angle"
      n_phiul_bus1_B = "n:phiul:bus1:B", "deg, Line-Line Voltage, Angle"
      n_phiul_bus1_C = "n:phiul:bus1:C", "deg, Line-Line Voltage, Angle"
      n_phiul_bus1_N = "n:phiul:bus1:N", "deg, Line-Line Voltage, Angle"
      n_phiul_bus1n = "n:phiul:bus1n", "deg, Line-Line Voltage, Angle"
      n_u0_bus1 = "n:u0:bus1", "p.u., Zero-Sequence Voltage, Magnitude"
      n_u0_bus1n = "n:u0:bus1n", "p.u., Zero-Sequence Voltage, Magnitude"
      n_U0_bus1 = "n:U0:bus1", "kV, Zero-Sequence Voltage, Magnitude"
      n_U0_bus1n = "n:U0:bus1n", "kV, Zero-Sequence Voltage, Magnitude"
      n_U0x3_bus1 = "n:U0x3:bus1", "kV, 3*U0"
      n_U0x3_bus1n = "n:U0x3:bus1n", "kV, 3*U0"
      n_phiu0_bus1 = "n:phiu0:bus1", "deg, Zero-Sequence Voltage, Angle"
      n_phiu0_bus1n = "n:phiu0:bus1n", "deg, Zero-Sequence Voltage, Angle"
      n_u1_bus1 = "n:u1:bus1", "p.u., Positive-Sequence Voltage, Magnitude"
      n_u1_bus1n = "n:u1:bus1n", "p.u., Positive-Sequence Voltage, Magnitude"
      n_u1pc_bus1 = "n:u1pc:bus1", "%, Positive-Sequence Voltage, Magnitude"
      n_u1pc_bus1n = "n:u1pc:bus1n", "%, Positive-Sequence Voltage, Magnitude"
      n_u1r_bus1 = "n:u1r:bus1", "p.u., Positive-Sequence Voltage, Real Part"
      n_u1r_bus1n = "n:u1r:bus1n", "p.u., Positive-Sequence Voltage, Real Part"
      n_u1i_bus1 = "n:u1i:bus1", "p.u., Positive-Sequence Voltage, Imaginary Part"
      n_u1i_bus1n = "n:u1i:bus1n", "p.u., Positive-Sequence Voltage, Imaginary Part"
      n_U1_bus1 = "n:U1:bus1", "kV, Line-Ground Positive-Sequence Voltage, Magnitude"
      n_U1_bus1n = "n:U1:bus1n", "kV, Line-Ground Positive-Sequence Voltage, Magnitude"
      n_phiu1_bus1 = "n:phiu1:bus1", "deg, Positive-Sequence Voltage, Angle"
      n_phiu1_bus1n = "n:phiu1:bus1n", "deg, Positive-Sequence Voltage, Angle"
      n_u2_bus1 = "n:u2:bus1", "p.u., Negative-Sequence Voltage, Magnitude"
      n_u2_bus1n = "n:u2:bus1n", "p.u., Negative-Sequence Voltage, Magnitude"
      n_U2_bus1 = "n:U2:bus1", "kV, Line-Ground Negative-Sequence Voltage, Magnitude"
      n_U2_bus1n = "n:U2:bus1n", "kV, Line-Ground Negative-Sequence Voltage, Magnitude"
      n_phiu2_bus1 = "n:phiu2:bus1", "deg, Negative-Sequence Voltage, Angle"
      n_phiu2_bus1n = "n:phiu2:bus1n", "deg, Negative-Sequence Voltage, Angle"
      n_U1l_bus1 = "n:U1l:bus1", "kV, Line-Line Positive-Sequence Voltage, Magnitude"
      n_U1l_bus1n = "n:U1l:bus1n", "kV, Line-Line Positive-Sequence Voltage, Magnitude"
      n_U2l_bus1 = "n:U2l:bus1", "kV, Line-Line Negative-Sequence Voltage, Magnitude"
      n_U2l_bus1n = "n:U2l:bus1n", "kV, Line-Line Negative-Sequence Voltage, Magnitude"
      n_uln_bus1_A = "n:uln:bus1:A", "p.u., Line-Neutral Voltage, Magnitude"
      n_uln_bus1_B = "n:uln:bus1:B", "p.u., Line-Neutral Voltage, Magnitude"
      n_uln_bus1_C = "n:uln:bus1:C", "p.u., Line-Neutral Voltage, Magnitude"
      n_uln_bus1_N = "n:uln:bus1:N", "p.u., Line-Neutral Voltage, Magnitude"
      n_uln_bus1n = "n:uln:bus1n", "p.u., Line-Neutral Voltage, Magnitude"
      n_Uln_bus1_A = "n:Uln:bus1:A", "kV, Line-Neutral Voltage, Magnitude"
      n_Uln_bus1_B = "n:Uln:bus1:B", "kV, Line-Neutral Voltage, Magnitude"
      n_Uln_bus1_C = "n:Uln:bus1:C", "kV, Line-Neutral Voltage, Magnitude"
      n_Uln_bus1_N = "n:Uln:bus1:N", "kV, Line-Neutral Voltage, Magnitude"
      n_Uln_bus1n = "n:Uln:bus1n", "kV, Line-Neutral Voltage, Magnitude"
      n_phiuln_bus1_A = "n:phiuln:bus1:A", "deg, Line-Neutral Voltage, Angle"
      n_phiuln_bus1_B = "n:phiuln:bus1:B", "deg, Line-Neutral Voltage, Angle"
      n_phiuln_bus1_C = "n:phiuln:bus1:C", "deg, Line-Neutral Voltage, Angle"
      n_phiuln_bus1_N = "n:phiuln:bus1:N", "deg, Line-Neutral Voltage, Angle"
      n_phiuln_bus1n = "n:phiuln:bus1n", "deg, Line-Neutral Voltage, Angle"
      n_un_bus1 = "n:un:bus1", "p.u., Neutral-Ground Voltage, Magnitude"
      n_un_bus1n = "n:un:bus1n", "p.u., Neutral-Ground Voltage, Magnitude"
      n_Un_bus1 = "n:Un:bus1", "kV, Neutral-Ground Voltage, Magnitude"
      n_Un_bus1n = "n:Un:bus1n", "kV, Neutral-Ground Voltage, Magnitude"
      n_phiun_bus1 = "n:phiun:bus1", "deg, Neutral-Ground Voltage, Angle"
      n_phiun_bus1n = "n:phiun:bus1n", "deg, Neutral-Ground Voltage, Angle"
      n_upht_bus1_A = "n:upht:bus1:A", "p.u., Phase Technology dependent Voltage, Magnitude"
      n_upht_bus1_B = "n:upht:bus1:B", "p.u., Phase Technology dependent Voltage, Magnitude"
      n_upht_bus1_C = "n:upht:bus1:C", "p.u., Phase Technology dependent Voltage, Magnitude"
      n_upht_bus1_N = "n:upht:bus1:N", "p.u., Phase Technology dependent Voltage, Magnitude"
      n_upht_bus1n = "n:upht:bus1n", "p.u., Phase Technology dependent Voltage, Magnitude"
      n_Upht_bus1_A = "n:Upht:bus1:A", "kV, Phase Technology dependent Voltage, Magnitude"
      n_Upht_bus1_B = "n:Upht:bus1:B", "kV, Phase Technology dependent Voltage, Magnitude"
      n_Upht_bus1_C = "n:Upht:bus1:C", "kV, Phase Technology dependent Voltage, Magnitude"
      n_Upht_bus1_N = "n:Upht:bus1:N", "kV, Phase Technology dependent Voltage, Magnitude"
      n_Upht_bus1n = "n:Upht:bus1n", "kV, Phase Technology dependent Voltage, Magnitude"
      n_um_bus1 = "n:um:bus1", "p.u., Average-Voltage, Magnitude"
      n_um_bus1n = "n:um:bus1n", "p.u., Average-Voltage, Magnitude"
      n_Um_bus1 = "n:Um:bus1", "kV, Average-Voltage, Magnitude"
      n_Um_bus1n = "n:Um:bus1n", "kV, Average-Voltage, Magnitude"
      n_uphtmin_bus1 = "n:uphtmin:bus1", "p.u., Minimum of Phase Technology dependent Voltage, Magnitude"
      n_uphtmin_bus1n = "n:uphtmin:bus1n", "p.u., Minimum of Phase Technology dependent Voltage, Magnitude"
      n_uphtmax_bus1 = "n:uphtmax:bus1", "p.u., Maximum of Phase Technology dependent Voltage, Magnitude"
      n_uphtmax_bus1n = "n:uphtmax:bus1n", "p.u., Maximum of Phase Technology dependent Voltage, Magnitude"
      n_Pgen_bus1 = "n:Pgen:bus1", "MW, Generation, Active Power"
      n_Pgen_bus1n = "n:Pgen:bus1n", "MW, Generation, Active Power"
      n_Qgen_bus1 = "n:Qgen:bus1", "Mvar, Generation, Reactive Power"
      n_Qgen_bus1n = "n:Qgen:bus1n", "Mvar, Generation, Reactive Power"
      n_Pmot_bus1 = "n:Pmot:bus1", "MW, Motor Load, Active Power"
      n_Pmot_bus1n = "n:Pmot:bus1n", "MW, Motor Load, Active Power"
      n_Qmot_bus1 = "n:Qmot:bus1", "Mvar, Motor Load, Reactive Power"
      n_Qmot_bus1n = "n:Qmot:bus1n", "Mvar, Motor Load, Reactive Power"
      n_Pload_bus1 = "n:Pload:bus1", "MW, General Load, Active Power"
      n_Pload_bus1n = "n:Pload:bus1n", "MW, General Load, Active Power"
      n_Qload_bus1 = "n:Qload:bus1", "Mvar, General Load, Reactive Power"
      n_Qload_bus1n = "n:Qload:bus1n", "Mvar, General Load, Reactive Power"
      n_Pcomp_bus1 = "n:Pcomp:bus1", "MW, Compensation (Losses)"
      n_Pcomp_bus1n = "n:Pcomp:bus1n", "MW, Compensation (Losses)"
      n_Qcomp_bus1 = "n:Qcomp:bus1", "Mvar, Compensation"
      n_Qcomp_bus1n = "n:Qcomp:bus1n", "Mvar, Compensation"
      n_Pnet_bus1 = "n:Pnet:bus1", "MW, External Networks, Active Power"
      n_Pnet_bus1n = "n:Pnet:bus1n", "MW, External Networks, Active Power"
      n_Qnet_bus1 = "n:Qnet:bus1", "Mvar, External Networks, Reactive Power"
      n_Qnet_bus1n = "n:Qnet:bus1n", "Mvar, External Networks, Reactive Power"
      n_Pflow_bus1 = "n:Pflow:bus1", "MW, Power Flow, Active Power"
      n_Pflow_bus1n = "n:Pflow:bus1n", "MW, Power Flow, Active Power"
      n_Qflow_bus1 = "n:Qflow:bus1", "Mvar, Power Flow, Reactive Power"
      n_Qflow_bus1n = "n:Qflow:bus1n", "Mvar, Power Flow, Reactive Power"
      n_Pout_bus1 = "n:Pout:bus1", "MW, Outgoing Flow, Active Power"
      n_Pout_bus1n = "n:Pout:bus1n", "MW, Outgoing Flow, Active Power"
      n_Qout_bus1 = "n:Qout:bus1", "Mvar, Outgoing Power, Reactive Power"
      n_Qout_bus1n = "n:Qout:bus1n", "Mvar, Outgoing Power, Reactive Power"
      n_Sout_bus1 = "n:Sout:bus1", "MVA, Outgoing Power, Apparent Power"
      n_Sout_bus1n = "n:Sout:bus1n", "MVA, Outgoing Power, Apparent Power"
      n_cosphiout_bus1 = "n:cosphiout:bus1", "Outgoing Power, Power Factor"
      n_cosphiout_bus1n = "n:cosphiout:bus1n", "Outgoing Power, Power Factor"
      n_Pbalance_bus1 = "n:Pbalance:bus1", "MW, Active Power Balance (=0)"
      n_Pbalance_bus1n = "n:Pbalance:bus1n", "MW, Active Power Balance (=0)"
      n_Qbalance_bus1 = "n:Qbalance:bus1", "Mvar, Reactive Power Balance (=0)"
      n_Qbalance_bus1n = "n:Qbalance:bus1n", "Mvar, Reactive Power Balance (=0)"
      n_ubfac_bus1 = "n:ubfac:bus1", "%, Unbalance Factor"
      n_ubfac_bus1n = "n:ubfac:bus1n", "%, Unbalance Factor"
      n_LossPdown_bus1 = "n:LossPdown:bus1", "MW, Losses, downstream"
      n_LossPdown_bus1n = "n:LossPdown:bus1n", "MW, Losses, downstream"
      n_LossQdown_bus1 = "n:LossQdown:bus1", "Mvar, Losses, downstream (Reactive Power)"
      n_LossQdown_bus1n = "n:LossQdown:bus1n", "Mvar, Losses, downstream (Reactive Power)"
      n_LossPdownload_bus1 = "n:LossPdownload:bus1", "MW, Load losses, downstream"
      n_LossPdownload_bus1n = "n:LossPdownload:bus1n", "MW, Load losses, downstream"
      n_LossQdownload_bus1 = "n:LossQdownload:bus1", "Mvar, Load losses, downstream"
      n_LossQdownload_bus1n = "n:LossQdownload:bus1n", "Mvar, Load losses, downstream"
      n_LossPdownnoload_bus1 = "n:LossPdownnoload:bus1", "MW, No load losses, downstream"
      n_LossPdownnoload_bus1n = "n:LossPdownnoload:bus1n", "MW, No load losses, downstream"
      n_LossQdownnoload_bus1 = "n:LossQdownnoload:bus1", "Mvar, No load losses, downstream"
      n_LossQdownnoload_bus1n = "n:LossQdownnoload:bus1n", "Mvar, No load losses, downstream"
      s_pset = "s:pset", "MW, IN, Active Power Setpoint, continuous, double"
      s_qset = "s:qset", "Mvar, IN, Reactive Power Setpoint continuous, double"
      s_dFin = "s:dFin", "Hz, IN, Frequency Deviation, continuous, double"
      s_dF = "s:dF", "Hz, OUT, Frequency Deviation Output, continuous, double"
      c_p = "c:p", "MW, Total Active Power"
      c_q = "c:q", "Mvar, Total Reactive Power"
      c_sgnn = "c:sgnn", "MVA, Total rated apparent power (ngnum*sgn)"
      c_cosn = "c:cosn", "Nominal-Power factor"
      c_Pnom = "c:Pnom", "MW, Total Nominal Active power"
      c_c2 = "c:c2", "Voltage Coefficient (uknom/ugn)"
      c_r0 = "c:r0", "p.u., Zero Sequence Resistance"
      c_x0 = "c:x0", "p.u., Zero Sequence Reactance"
      c_rearth = "c:rearth", "p.u., Grounding Resistance"
      c_xearth = "c:xearth", "p.u., Grounding Reactance"
      c_r2 = "c:r2", "p.u., Negative Sequence Resistance"
      c_x2 = "c:x2", "p.u., Negative Sequence Reactance"
      c_uini_r = "c:uini:r", "p.u., Voltage Setpoint for PV and Slack, Real Part"
      c_uini_i = "c:uini:i", "p.u., Voltage Setpoint for PV and Slack, Imaginary Part"
      c_uini_phi = "c:uini:phi", "deg, Voltage Setpoint for PV and Slack, Angle"
      c_uini_mag = "c:uini:mag", "p.u., Voltage Setpoint for PV and Slack, Magnitude"
      c_uini_phirad = "c:uini:phirad", "rad, Voltage Setpoint for PV and Slack, Angle"
      c_loading = "c:loading", "%, Loading"
      c_iPowerOn = "c:iPowerOn", "Flag"
      c_xdq = "c:xdq", "p.u., Synchronous Reactance"
      c_Qmin = "c:Qmin", "Mvar, Lower limit of reactive power"
      c_Qmax = "c:Qmax", "Mvar, Upper limit of reactive power"
      c_Qsparelead = "c:Qsparelead", "Mvar, Q spare lead"
      c_Qsparelag = "c:Qsparelag", "Mvar, Q spare lag"
      c_i_qlim = "c:i_qlim", "Reactive power limit reached"
      c_i_vlim = "c:i_vlim", "Voltage setpoint limit reached"
      c_i_plim = "c:i_plim", "Active power limit reached"
      c_sm_r = "c:sm:r", "MW, Complex Power, Real Part"
      c_sm_i = "c:sm:i", "Mvar, Complex Power, Imaginary Part"
      c_sm_phi = "c:sm:phi", "deg, Complex Power, Angle"
      c_sm_mag = "c:sm:mag", "MVA, Complex Power, Magnitude"
      c_sm_phirad = "c:sm:phirad", "rad, Complex Power, Angle"
      c_u1_r = "c:u1:r", "p.u., Internal, Real Part"
      c_u1_i = "c:u1:i", "p.u., Internal, Imaginary Part"
      c_u1_phi = "c:u1:phi", "deg, Internal, Angle"
      c_u1_mag = "c:u1:mag", "p.u., Internal, Magnitude"
      c_u1_phirad = "c:u1:phirad", "rad, Internal, Angle"
      c_i_spin = "c:i_spin", "Flag"
      c_iRefElement = "c:iRefElement", "Reference Element"
      c_phiini = "c:phiini", "Initial Angle"
      c_Pdisp = "c:Pdisp", "MW, Power (Eco-Dispatch)"
      c_cst_disp = "c:cst_disp", "USD/h, Operating Costs"
      c_Kp = "c:Kp", "p.u., Active Power Participation"
      c_Kq = "c:Kq", "p.u., Reactive Power Participation"
      c_Kpf = "c:Kpf", "MW/Hz, Frequency Bias"
      c_zonewscale = "c:zonewscale", "Wind generation zone scaling factor"
      c_Pmism = "c:Pmism", "MW, Active Power Mismatch"
      c_nCtrlIter = "c:nCtrlIter", "No. of CTRL-iterations"
      c_Ploss = "c:Ploss", "MW, Losses (total)"
      c_Qloss = "c:Qloss", "Mvar, Reactive-Losses (total)"
      c_Plossld = "c:Plossld", "MW, Losses (load)"
      c_Qlossld = "c:Qlossld", "Mvar, Reactive-Losses (load)"
      c_Plossnld = "c:Plossnld", "MW, Losses (no load)"
      c_Qlossnld = "c:Qlossnld", "Mvar, Reactive-Losses (no load)"
      e_i_spin = "e:i_spin", "Spinning if circuit-breaker is open"
      e_ip_ctrl = "e:ip_ctrl", "Reference Machine"
      e_outServPzero = "e:outServPzero", "Out of service when active power is zero"
      e_av_mode = "e:av_mode", "Local Controller"
      e_c_psecc = "e:c_psecc", "ElmSecctrl, External Secondary Controller"
      e_c_pstac = "e:c_pstac", "ElmStactrl, External Station Controller"
      e_qdslCtrl = "e:qdslCtrl", "Quasi-Dynamic Model"
      e_mode_pgi = "e:mode_pgi", "Model"
      e_pPowerCrv = "e:pPowerCrv", "TypPowercurve, Wind Power Curve"
      e_mode_inp = "e:mode_inp", "Input Mode"
      e_pgini = "e:pgini", "MW, Active Power"
      e_qgini = "e:qgini", "Mvar, Reactive Power"
      e_sgini = "e:sgini", "MVA, Apparent Power"
      e_cosgini = "e:cosgini", "Power Factor"
      e_pf_recap = "e:pf_recap", "Power Factor"
      e_p_direc = "e:p_direc", "Power Direction"
      e_windspeed = "e:windspeed", "m/s, Wind speed"
      e_usetp = "e:usetp", "p.u., Voltage"
      e_phiini = "e:phiini", "deg, Angle"
      e_Kpf = "e:Kpf", "MW/Hz, Prim. Frequency Bias"
      e_pgini_a = "e:pgini_a", "MW, Active Power (act.)"
      e_qgini_a = "e:qgini_a", "Mvar, Reactive Power (act.)"
      e_sgini_a = "e:sgini_a", "MVA, Apparent Power (act.)"
      e_cosgini_a = "e:cosgini_a", "Power Factor (act.)"
      e_pf_recap_a = "e:pf_recap_a", "Power Factor Ind/Cap (act.)"
      e_windspeed_a = "e:windspeed_a", "m/s, Wind speed (act.)"
      e_usp_min = "e:usp_min", "p.u., Min. Voltage Setpoint"
      e_usp_max = "e:usp_max", "p.u., Max. Voltage Setpoint"
      e_ddroop = "e:ddroop", "%, Droop"
      e_pQPcurve = "e:pQPcurve", "IntQpcurve, Q(P)-Curve"
      e_pQlimType = "e:pQlimType", "IntQlim, Capability Curve"
      e_iqtype = "e:iqtype", "Use limits specified in type"
      e_q_min = "e:q_min", "p.u., Min."
      e_q_max = "e:q_max", "p.u., Max."
      e_cQ_min = "e:cQ_min", "Mvar, Minimum"
      e_cQ_max = "e:cQ_max", "Mvar, Maximum"
      e_cq_min = "e:cq_min", "p.u., Min."
      e_cq_max = "e:cq_max", "p.u., Max."
      e_scaleQmin = "e:scaleQmin", "%, Scaling Factor (min.)"
      e_scaleQmax = "e:scaleQmax", "%, Scaling Factor (max.)"
      e_Pmin_uc = "e:Pmin_uc", "MW, Min."
      e_Pmin_ucPU = "e:Pmin_ucPU", "p.u., Min."
      e_Pmax_uc = "e:Pmax_uc", "MW, Max."
      e_Pmax_ucPU = "e:Pmax_ucPU", "p.u., Max."
      e_Pnom = "e:Pnom", "MW, Pr(rated)"
      e_P_max = "e:P_max", "MW, Max."
      e_pmaxratf = "e:pmaxratf", "Rating Factor"
      e_i_cap = "e:i_cap", "User defined Capability Curve"
      e_cap_P = "e:cap_P", "MW, Act.Power"
      e_cap_Qmn = "e:cap_Qmn", "Mvar, Min."
      e_cap_Qmx = "e:cap_Qmx", "Mvar, Max."
      e_is4SpinReserve = "e:is4SpinReserve", "Consider for region spinning reserve"
      e_genShiftKey = "e:genShiftKey", "%, Generation shift key"
      e_c_pCtrlHV = "e:c_pCtrlHV", "Controlled HV-busbar"
      e_Uctrl = "e:Uctrl", "kV, HV-Voltage Setpoint"
      e_Qmin_a = "e:Qmin_a", "Mvar, Qmin(act.)"
      e_Qmax_a = "e:Qmax_a", "Mvar, Qmax(act.)"
      e_Pmin_a = "e:Pmin_a", "MW, Pmin(act.)"
      e_Pmax_a = "e:Pmax_a", "MW, Pmax(act.)"
      e_pBMU = "e:pBMU", "ElmBmu, Virtual Power Plant"
      e_dispatch = "e:dispatch", "Generator Dispatch"
      e_priority = "e:priority", "Merit Order"
      e_fixed = "e:fixed", "Must run"
      e_cpFeed = "e:cpFeed", "Feeder"
      e_ciDist = "e:ciDist", "Distance from infeed in number of buses"
      e_ciLater = "e:ciLater", "Lateral Index"
      e_ciDistRoot = "e:ciDistRoot", "Distance from first infeed in number of buses"
      e_ciDistAll = "e:ciDistAll", "Distance from infeed in number of buses including switches"
      e_ciDistAllRoot = "e:ciDistAllRoot", "Distance from first infeed in number of buses including switches"
      t_model_inp = "t:model_inp", "Model"
      t_rstr = "t:rstr", "p.u., rstr"
      t_iturbo = "t:iturbo", "Rotor type"
      t_x0sy = "t:x0sy", "p.u., Reactance x0"
      t_r0sy = "t:r0sy", "p.u., Resistance r0"
      t_x2sy = "t:x2sy", "p.u., Reactance x2"
      t_r2sy = "t:r2sy", "p.u., Resistance r2"
      t_q_min = "t:q_min", "p.u., Minimum value"
      t_q_max = "t:q_max", "p.u., Maximum value"
      t_Q_min = "t:Q_min", "Mvar, Minimum value"
      t_Q_max = "t:Q_max", "Mvar, Maximum value"
      t_xd = "t:xd", "p.u., xd"
      t_xq = "t:xq", "p.u., xq"
      t_psim = "t:psim", "p.u., Rotor flux"
      r_c_psecc = "r:c_psecc", "ElmSecctrl, External Secondary Controller"
      r_c_pstac = "r:c_pstac", "ElmStactrl, External Station Controller"
      r_qdslCtrl = "r:qdslCtrl", "Quasi-Dynamic Model"
      r_pPowerCrv = "r:pPowerCrv", "TypPowercurve, Wind Power Curve"
      r_pQPcurve = "r:pQPcurve", "IntQpcurve, Q(P)-Curve"
      r_pQlimType = "r:pQlimType", "IntQlim, Capability Curve"
      r_c_pCtrlHV = "r:c_pCtrlHV", "Controlled HV-busbar"
      r_pBMU = "r:pBMU", "ElmBmu, Virtual Power Plant"
      r_cpFeed = "r:cpFeed", "Feeder"
      m_Ikss_bus1 = "m:Ikss:bus1", "kA, Initial Short-Circuit Current"
      m_ikss_bus1 = "m:ikss:bus1", "p.u., Initial Short-Circuit Current, Magnitude"
      m_phii_bus1 = "m:phii:bus1", "deg, Phase Current, Angle"
      m_maxIkss_bus1 = "m:maxIkss:bus1", "kA, Max. Initial Short-Circuit Current"
      m_Skss_bus1 = "m:Skss:bus1", "MVA, Initial Short-Circuit Power"
      m_ip_bus1 = "m:ip:bus1", "kA, Peak Short-Circuit Current"
      m_maxip_bus1 = "m:maxip:bus1", "kA, Max. Peak Short-Circuit Current"
      m_maxIb_bus1 = "m:maxIb:bus1", "kA, Max. Breaking Current"
      m_maxIth_bus1 = "m:maxIth:bus1", "kA, Max. Thermal equivalent Short-Circuit Current"
      m_Ib_bus1 = "m:Ib:bus1", "kA, Short-Circuit Breaking Current"
      m_Sb_bus1 = "m:Sb:bus1", "MVA, Short-Circuit Breaking Power"
      m_Ik_bus1 = "m:Ik:bus1", "kA, Steady-State Short-Circuit Current"
      m_Ith_bus1 = "m:Ith:bus1", "kA, Thermal equivalent Short-Circuit Current"
      m_I_bus1 = "m:I:bus1", "kA, Short-Circuit Current"
      m_R_bus1 = "m:R:bus1", "Ohm, Fault Impedance, Real Part"
      m_X_bus1 = "m:X:bus1", "Ohm, Fault Impedance, Imaginary Part"
      m_Z_bus1 = "m:Z:bus1", "Ohm, Fault Impedance, Magnitude"
      m_phiz_bus1 = "m:phiz:bus1", "deg, Fault Impedance, Angle"
      m_zSbase_bus1 = "m:zSbase:bus1", "p.u., Fault Impedance, Magnitude"
      m_XtoR_bus1 = "m:XtoR:bus1", "X/R ratio"
      m_Zs_bus1 = "m:Zs:bus1", "Ohm, Source Impedance, Magnitude"
      m_phizs_bus1 = "m:phizs:bus1", "deg, Source Impedance, Angle"
      m_Rs_bus1 = "m:Rs:bus1", "Ohm, Source Impedance, Real Part"
      m_Xs_bus1 = "m:Xs:bus1", "Ohm, Source Impedance, Imaginary Part"
      m_phiui_bus1 = "m:phiui:bus1", "deg, Angle between Voltage and Current"
      m_cosphiui_bus1 = "m:cosphiui:bus1", "cosphi"
      m_P_bus1 = "m:P:bus1", "MW, Active Power"
      m_Q_bus1 = "m:Q:bus1", "Mvar, Reactive Power"
      m_BrkIpload_bus1 = "m:BrkIpload:bus1", "%, Breaker Loading (peak)"
      m_BrkIbload_bus1 = "m:BrkIbload:bus1", "%, Breaker Loading (breaking)"
      m_BrkmaxIb_bus1 = "m:BrkmaxIb:bus1", "kA, Max. Breaking Current (breaker time)"
      n_Ikss_bus1 = "n:Ikss:bus1", "kA, Initial Short-Circuit Current"
      n_I_bus1 = "n:I:bus1", "kA, Short-Circuit Current"
      n_phii_bus1 = "n:phii:bus1", "deg, Phase Current, Angle"
      n_Skss_bus1 = "n:Skss:bus1", "MVA, Initial Short-Circuit Power"
      n_ip_bus1 = "n:ip:bus1", "kA, Peak Short-Circuit Current"
      n_Ib_bus1 = "n:Ib:bus1", "kA, Short-Circuit Breaking Current"
      n_Sb_bus1 = "n:Sb:bus1", "MVA, Short-Circuit Breaking Power"
      n_Ik_bus1 = "n:Ik:bus1", "kA, Steady-State Short-Circuit Current"
      n_Ith_bus1 = "n:Ith:bus1", "kA, Thermal equivalent Short-Circuit Current"
      n_R_bus1 = "n:R:bus1", "Ohm, Short-Circuit Impedance, Real Part"
      n_X_bus1 = "n:X:bus1", "Ohm, Short-Circuit Impedance, Imaginary Part"
      n_Z_bus1 = "n:Z:bus1", "Ohm, Short-Circuit Impedance, Magnitude"
      n_phiz_bus1 = "n:phiz:bus1", "deg, Short-Circuit Impedance, Angle"
      n_rSbase_bus1 = "n:rSbase:bus1", "p.u., Short-Circuit Impedance, Real Part"
      n_xSbase_bus1 = "n:xSbase:bus1", "p.u., Short-Circuit Impedance, Imaginary Part"
      n_zSbase_bus1 = "n:zSbase:bus1", "p.u., Short-Circuit Impedance, Magnitude"
      n_XtoR_bus1 = "n:XtoR:bus1", "X/R ratio (peak)"
      n_RtoX_bus1 = "n:RtoX:bus1", "R/X ratio (peak)"
      n_ur_bus1 = "n:ur:bus1", "p.u., Voltage, Real Part"
      n_ui_bus1 = "n:ui:bus1", "p.u., Voltage, Imaginary Part"
      n_u_bus1 = "n:u:bus1", "p.u., Voltage, Magnitude"
      n_upc_bus1 = "n:upc:bus1", "%, Voltage, Magnitude"
      n_U_bus1 = "n:U:bus1", "kV, Line-Ground Voltage, Magnitude"
      n_Ul_bus1 = "n:Ul:bus1", "kV, Line-Line Voltage, Magnitude"
      n_phiu_bus1 = "n:phiu:bus1", "deg, Voltage, Angle"
      n_du_bus1 = "n:du:bus1", "%, Voltage Deviation"
      n_urpref_bus1 = "n:urpref:bus1", "p.u., Pre-fault Voltage, Real Part"
      n_uipref_bus1 = "n:uipref:bus1", "p.u., Pre-fault Voltage, Imaginary Part"
      n_upref_bus1 = "n:upref:bus1", "p.u., Pre-fault Voltage, Magnitude"
      n_uprefpc_bus1 = "n:uprefpc:bus1", "%, Pre-fault Voltage, Magnitude"
      n_Upref_bus1 = "n:Upref:bus1", "kV, Line-Ground Pre-fault Voltage, Magnitude"
      n_Ulpref_bus1 = "n:Ulpref:bus1", "kV, Line-Line Pre-fault Voltage, Magnitude"
      n_phiupref_bus1 = "n:phiupref:bus1", "deg, Pre-fault Voltage, Angle"
      n_phiui_bus1 = "n:phiui:bus1", "deg, Angle between Voltage and Current"
      n_cfac_bus1 = "n:cfac:bus1", "Voltage Factor"
      n_idc_bus1 = "n:idc:bus1", "kA, Decaying Component"
      n_Iasy_bus1 = "n:Iasy:bus1", "kA, Asymmetrical Breaking Current"
      n_Ithload_bus1 = "n:Ithload:bus1", "%, Loading, Thermal equivalent Current"
      n_Ipload_bus1 = "n:Ipload:bus1", "%, Loading, Peak Short-Circuit Current"
      n_Ithrtk_bus1 = "n:Ithrtk:bus1", "kA, Rated Short-Time Current (Tk)"
      n_Tn_bus1 = "n:Tn:bus1", "ms, Grid Time Constant"
      n_XtoR_b_bus1 = "n:XtoR_b:bus1", "X/R ratio (dc)"
      n_RtoX_b_bus1 = "n:RtoX_b:bus1", "R/X ratio (dc)"
      n_fctof_b_bus1 = "n:fctof_b:bus1", "Equivalent Frequency fc/f (dc)"
      c_z1_r = "c:z1:r", "p.u., Positive-sequence impedance, Real Part"
      c_z1_i = "c:z1:i", "p.u., Positive-sequence impedance, Imaginary Part"
      c_z1_phi = "c:z1:phi", "deg, Positive-sequence impedance, Angle"
      c_z1_mag = "c:z1:mag", "p.u., Positive-sequence impedance, Magnitude"
      c_z1_phirad = "c:z1:phirad", "rad, Positive-sequence impedance, Angle"
      c_uini1_r = "c:uini1:r", "p.u., Internal positive-sequence voltage, Real Part"
      c_uini1_i = "c:uini1:i", "p.u., Internal positive-sequence voltage, Imaginary Part"
      c_uini1_phi = "c:uini1:phi", "deg, Internal positive-sequence voltage, Angle"
      c_uini1_mag = "c:uini1:mag", "p.u., Internal positive-sequence voltage, Magnitude"
      c_uini1_phirad = "c:uini1:phirad", "rad, Internal positive-sequence voltage, Angle"
      c_xdsss = "c:xdsss", "p.u., Saturated Subtransient Reactance"
      c_xdsat = "c:xdsat", "p.u., Saturated Synchronous Reactance"
      c_satur = "c:satur", "Machine Type IEC909"
      c_rgf = "c:rgf", "p.u., Fictitious Resistance"
      c_tr2tf = "c:tr2tf", "PSU Transformer Ratio"
      e_pG = "e:pG", "%, Range of Voltage Regulation (+/-)"
      t_xdsss = "t:xdsss", "p.u., saturated value xd''sat"
      t_xtor = "t:xtor", "Ratio X/R"
      t_iuseXdk = "t:iuseXdk", "Ik instead of reactances"
      t_curk = "t:curk", "kA, 3-phase Ik3p"
      t_curk2p = "t:curk2p", "kA, 2-phase Ik2p"
      t_curk1p = "t:curk1p", "kA, 1-phase Ik1p"
      t_xdsat = "t:xdsat", "p.u., Reciprocal of short-circuit ratio (xdsat)"
      t_satur = "t:satur", "Machine type IEC909/IEC60909"
      m_Ikss_bus1_A = "m:Ikss:bus1:A", "kA, Initial Short-Circuit Current"
      m_Ikss_bus1_B = "m:Ikss:bus1:B", "kA, Initial Short-Circuit Current"
      m_Ikss_bus1_C = "m:Ikss:bus1:C", "kA, Initial Short-Circuit Current"
      m_Ikss_bus1_N = "m:Ikss:bus1:N", "kA, Initial Short-Circuit Current"
      m_Ikss_bus1n = "m:Ikss:bus1n", "kA, Initial Short-Circuit Current"
      m_ikss_bus1_A = "m:ikss:bus1:A", "p.u., Initial Short-Circuit Current, Magnitude"
      m_ikss_bus1_B = "m:ikss:bus1:B", "p.u., Initial Short-Circuit Current, Magnitude"
      m_ikss_bus1_C = "m:ikss:bus1:C", "p.u., Initial Short-Circuit Current, Magnitude"
      m_ikss_bus1_N = "m:ikss:bus1:N", "p.u., Initial Short-Circuit Current, Magnitude"
      m_ikss_bus1n = "m:ikss:bus1n", "p.u., Initial Short-Circuit Current, Magnitude"
      m_maxIkss_bus1n = "m:maxIkss:bus1n", "kA, Max. Initial Short-Circuit Current"
      m_Skss_bus1_A = "m:Skss:bus1:A", "MVA, Initial Short-Circuit Power"
      m_Skss_bus1_B = "m:Skss:bus1:B", "MVA, Initial Short-Circuit Power"
      m_Skss_bus1_C = "m:Skss:bus1:C", "MVA, Initial Short-Circuit Power"
      m_Skss_bus1_N = "m:Skss:bus1:N", "MVA, Initial Short-Circuit Power"
      m_Skss_bus1n = "m:Skss:bus1n", "MVA, Initial Short-Circuit Power"
      m_Skssx3_bus1_A = "m:Skssx3:bus1:A", "MVA, 3*Initial Short-Circuit Power"
      m_Skssx3_bus1_B = "m:Skssx3:bus1:B", "MVA, 3*Initial Short-Circuit Power"
      m_Skssx3_bus1_C = "m:Skssx3:bus1:C", "MVA, 3*Initial Short-Circuit Power"
      m_Skssx3_bus1_N = "m:Skssx3:bus1:N", "MVA, 3*Initial Short-Circuit Power"
      m_Skssx3_bus1n = "m:Skssx3:bus1n", "MVA, 3*Initial Short-Circuit Power"
      m_ip_bus1_A = "m:ip:bus1:A", "kA, Peak Short-Circuit Current"
      m_ip_bus1_B = "m:ip:bus1:B", "kA, Peak Short-Circuit Current"
      m_ip_bus1_C = "m:ip:bus1:C", "kA, Peak Short-Circuit Current"
      m_ip_bus1_N = "m:ip:bus1:N", "kA, Peak Short-Circuit Current"
      m_ip_bus1n = "m:ip:bus1n", "kA, Peak Short-Circuit Current"
      m_maxip_bus1n = "m:maxip:bus1n", "kA, Max. Peak Short-Circuit Current"
      m_maxIth_bus1n = "m:maxIth:bus1n", "kA, Max. Thermal equivalent Short-Circuit Current"
      m_Z_bus1_A = "m:Z:bus1:A", "Ohm, Fault Impedance, Magnitude"
      m_Z_bus1_B = "m:Z:bus1:B", "Ohm, Fault Impedance, Magnitude"
      m_Z_bus1_C = "m:Z:bus1:C", "Ohm, Fault Impedance, Magnitude"
      m_Z_bus1_N = "m:Z:bus1:N", "Ohm, Fault Impedance, Magnitude"
      m_Z_bus1n = "m:Z:bus1n", "Ohm, Fault Impedance, Magnitude"
      m_phiz_bus1_A = "m:phiz:bus1:A", "deg, Fault Impedance, Angle"
      m_phiz_bus1_B = "m:phiz:bus1:B", "deg, Fault Impedance, Angle"
      m_phiz_bus1_C = "m:phiz:bus1:C", "deg, Fault Impedance, Angle"
      m_phiz_bus1_N = "m:phiz:bus1:N", "deg, Fault Impedance, Angle"
      m_phiz_bus1n = "m:phiz:bus1n", "deg, Fault Impedance, Angle"
      m_Zl_bus1_A = "m:Zl:bus1:A", "Ohm, Line-Line Fault Impedance, Magnitude"
      m_Zl_bus1_B = "m:Zl:bus1:B", "Ohm, Line-Line Fault Impedance, Magnitude"
      m_Zl_bus1_C = "m:Zl:bus1:C", "Ohm, Line-Line Fault Impedance, Magnitude"
      m_Zl_bus1_N = "m:Zl:bus1:N", "Ohm, Line-Line Fault Impedance, Magnitude"
      m_Zl_bus1n = "m:Zl:bus1n", "Ohm, Line-Line Fault Impedance, Magnitude"
      m_phizl_bus1_A = "m:phizl:bus1:A", "deg, Line-Line Fault Impedance, Angle"
      m_phizl_bus1_B = "m:phizl:bus1:B", "deg, Line-Line Fault Impedance, Angle"
      m_phizl_bus1_C = "m:phizl:bus1:C", "deg, Line-Line Fault Impedance, Angle"
      m_phizl_bus1_N = "m:phizl:bus1:N", "deg, Line-Line Fault Impedance, Angle"
      m_phizl_bus1n = "m:phizl:bus1n", "deg, Line-Line Fault Impedance, Angle"
      m_Z0_bus1 = "m:Z0:bus1", "Ohm, Zero-Sequence Fault Impedance, Magnitude"
      m_Z0_bus1n = "m:Z0:bus1n", "Ohm, Zero-Sequence Fault Impedance, Magnitude"
      m_phiz0_bus1 = "m:phiz0:bus1", "deg, Zero-Sequence Fault Impedance, Angle"
      m_phiz0_bus1n = "m:phiz0:bus1n", "deg, Zero-Sequence Fault Impedance, Angle"
      m_z0Sbase_bus1 = "m:z0Sbase:bus1", "p.u., Zero-Sequence Fault Impedance, Magnitude"
      m_z0Sbase_bus1n = "m:z0Sbase:bus1n", "p.u., Zero-Sequence Fault Impedance, Magnitude"
      m_cosphiu1i1_bus1 = "m:cosphiu1i1:bus1", "Positive-Sequence cosphi"
      m_cosphiu1i1_bus1n = "m:cosphiu1i1:bus1n", "Positive-Sequence cosphi"
      m_I0x3watt_bus1 = "m:I0x3watt:bus1", "kA, Earth-Fault Residual Current"
      m_I0x3watt_bus1n = "m:I0x3watt:bus1n", "kA, Earth-Fault Residual Current"
      m_I0x3var_bus1 = "m:I0x3var:bus1", "kA, Capacitive Earth-Fault Current"
      m_I0x3var_bus1n = "m:I0x3var:bus1n", "kA, Capacitive Earth-Fault Current"
      m_Q0cap_bus1 = "m:Q0cap:bus1", "Mvar, Q0 capacitive"
      m_Q0cap_bus1n = "m:Q0cap:bus1n", "Mvar, Q0 capacitive"
      m_Zfrel_bus1 = "m:Zfrel:bus1", "Ohm, Relay Impedance, Magnitude"
      m_Zfrel_bus1n = "m:Zfrel:bus1n", "Ohm, Relay Impedance, Magnitude"
      m_phizfrel_bus1 = "m:phizfrel:bus1", "deg, Relay Impedance, Angle"
      m_phizfrel_bus1n = "m:phizfrel:bus1n", "deg, Relay Impedance, Angle"
      m_Rfrel_bus1 = "m:Rfrel:bus1", "Ohm, Relay Impedance, Real Part"
      m_Rfrel_bus1n = "m:Rfrel:bus1n", "Ohm, Relay Impedance, Real Part"
      m_Xfrel_bus1 = "m:Xfrel:bus1", "Ohm, Relay Impedance, Imaginary Part"
      m_Xfrel_bus1n = "m:Xfrel:bus1n", "Ohm, Relay Impedance, Imaginary Part"
      m_Zs_bus1n = "m:Zs:bus1n", "Ohm, Source Impedance, Magnitude"
      m_phizs_bus1n = "m:phizs:bus1n", "deg, Source Impedance, Angle"
      m_Rs_bus1n = "m:Rs:bus1n", "Ohm, Source Impedance, Real Part"
      m_Xs_bus1n = "m:Xs:bus1n", "Ohm, Source Impedance, Imaginary Part"
      m_Zf_bus1 = "m:Zf:bus1", "Ohm, Meas. Fault Impedance, Magnitude"
      m_Zf_bus1n = "m:Zf:bus1n", "Ohm, Meas. Fault Impedance, Magnitude"
      m_phizf_bus1 = "m:phizf:bus1", "deg, Meas. Fault Impedance, Angle"
      m_phizf_bus1n = "m:phizf:bus1n", "deg, Meas. Fault Impedance, Angle"
      m_BrkIpload_bus1n = "m:BrkIpload:bus1n", "%, Max. Breaker Loading (peak)"
      n_Ikss_bus1_A = "n:Ikss:bus1:A", "kA, Initial Short-Circuit Current"
      n_Ikss_bus1_B = "n:Ikss:bus1:B", "kA, Initial Short-Circuit Current"
      n_Ikss_bus1_C = "n:Ikss:bus1:C", "kA, Initial Short-Circuit Current"
      n_Ikss_bus1_N = "n:Ikss:bus1:N", "kA, Initial Short-Circuit Current"
      n_Ikss_bus1n = "n:Ikss:bus1n", "kA, Initial Short-Circuit Current"
      n_I_bus1_A = "n:I:bus1:A", "kA, Short-Circuit Current"
      n_I_bus1_B = "n:I:bus1:B", "kA, Short-Circuit Current"
      n_I_bus1_C = "n:I:bus1:C", "kA, Short-Circuit Current"
      n_I_bus1_N = "n:I:bus1:N", "kA, Short-Circuit Current"
      n_I_bus1n = "n:I:bus1n", "kA, Short-Circuit Current"
      n_phii_bus1_A = "n:phii:bus1:A", "deg, Phase Current, Angle"
      n_phii_bus1_B = "n:phii:bus1:B", "deg, Phase Current, Angle"
      n_phii_bus1_C = "n:phii:bus1:C", "deg, Phase Current, Angle"
      n_phii_bus1_N = "n:phii:bus1:N", "deg, Phase Current, Angle"
      n_phii_bus1n = "n:phii:bus1n", "deg, Phase Current, Angle"
      n_Skss_bus1_A = "n:Skss:bus1:A", "MVA, Initial Short-Circuit Power"
      n_Skss_bus1_B = "n:Skss:bus1:B", "MVA, Initial Short-Circuit Power"
      n_Skss_bus1_C = "n:Skss:bus1:C", "MVA, Initial Short-Circuit Power"
      n_Skss_bus1_N = "n:Skss:bus1:N", "MVA, Initial Short-Circuit Power"
      n_Skss_bus1n = "n:Skss:bus1n", "MVA, Initial Short-Circuit Power"
      n_Skssx3_bus1_A = "n:Skssx3:bus1:A", "MVA, 3*Initial Short-Circuit Power"
      n_Skssx3_bus1_B = "n:Skssx3:bus1:B", "MVA, 3*Initial Short-Circuit Power"
      n_Skssx3_bus1_C = "n:Skssx3:bus1:C", "MVA, 3*Initial Short-Circuit Power"
      n_Skssx3_bus1_N = "n:Skssx3:bus1:N", "MVA, 3*Initial Short-Circuit Power"
      n_Skssx3_bus1n = "n:Skssx3:bus1n", "MVA, 3*Initial Short-Circuit Power"
      n_ip_bus1_A = "n:ip:bus1:A", "kA, Peak Short-Circuit Current"
      n_ip_bus1_B = "n:ip:bus1:B", "kA, Peak Short-Circuit Current"
      n_ip_bus1_C = "n:ip:bus1:C", "kA, Peak Short-Circuit Current"
      n_ip_bus1_N = "n:ip:bus1:N", "kA, Peak Short-Circuit Current"
      n_ip_bus1n = "n:ip:bus1n", "kA, Peak Short-Circuit Current"
      n_Ib_bus1_A = "n:Ib:bus1:A", "kA, Short-Circuit Breaking Current"
      n_Ib_bus1_B = "n:Ib:bus1:B", "kA, Short-Circuit Breaking Current"
      n_Ib_bus1_C = "n:Ib:bus1:C", "kA, Short-Circuit Breaking Current"
      n_Ib_bus1_N = "n:Ib:bus1:N", "kA, Short-Circuit Breaking Current"
      n_Ib_bus1n = "n:Ib:bus1n", "kA, Short-Circuit Breaking Current"
      n_Sb_bus1_A = "n:Sb:bus1:A", "MVA, Short-Circuit Breaking Power"
      n_Sb_bus1_B = "n:Sb:bus1:B", "MVA, Short-Circuit Breaking Power"
      n_Sb_bus1_C = "n:Sb:bus1:C", "MVA, Short-Circuit Breaking Power"
      n_Sb_bus1_N = "n:Sb:bus1:N", "MVA, Short-Circuit Breaking Power"
      n_Sb_bus1n = "n:Sb:bus1n", "MVA, Short-Circuit Breaking Power"
      n_Ith_bus1_A = "n:Ith:bus1:A", "kA, Thermal equivalent Short-Circuit Current"
      n_Ith_bus1_B = "n:Ith:bus1:B", "kA, Thermal equivalent Short-Circuit Current"
      n_Ith_bus1_C = "n:Ith:bus1:C", "kA, Thermal equivalent Short-Circuit Current"
      n_Ith_bus1_N = "n:Ith:bus1:N", "kA, Thermal equivalent Short-Circuit Current"
      n_Ith_bus1n = "n:Ith:bus1n", "kA, Thermal equivalent Short-Circuit Current"
      n_Ithmax_bus1 = "n:Ithmax:bus1", "kA, Max. Thermal equivalent Short-Circuit Current"
      n_Ithmax_bus1n = "n:Ithmax:bus1n", "kA, Max. Thermal equivalent Short-Circuit Current"
      n_Ik_bus1_A = "n:Ik:bus1:A", "kA, Steady-State Short-Circuit Current"
      n_Ik_bus1_B = "n:Ik:bus1:B", "kA, Steady-State Short-Circuit Current"
      n_Ik_bus1_C = "n:Ik:bus1:C", "kA, Steady-State Short-Circuit Current"
      n_Ik_bus1_N = "n:Ik:bus1:N", "kA, Steady-State Short-Circuit Current"
      n_Ik_bus1n = "n:Ik:bus1n", "kA, Steady-State Short-Circuit Current"
      n_R0_bus1 = "n:R0:bus1", "Ohm, Zero-Sequence Impedance, Real Part"
      n_R0_bus1n = "n:R0:bus1n", "Ohm, Zero-Sequence Impedance, Real Part"
      n_X0_bus1 = "n:X0:bus1", "Ohm, Zero-Sequence Impedance, Imaginary Part"
      n_X0_bus1n = "n:X0:bus1n", "Ohm, Zero-Sequence Impedance, Imaginary Part"
      n_Z0_bus1 = "n:Z0:bus1", "Ohm, Zero-Sequence Impedance, Magnitude"
      n_Z0_bus1n = "n:Z0:bus1n", "Ohm, Zero-Sequence Impedance, Magnitude"
      n_phiz0_bus1 = "n:phiz0:bus1", "deg, Zero-Sequence Impedance, Angle"
      n_phiz0_bus1n = "n:phiz0:bus1n", "deg, Zero-Sequence Impedance, Angle"
      n_r0Sbase_bus1 = "n:r0Sbase:bus1", "p.u., Zero-Sequence Impedance, Real Part"
      n_r0Sbase_bus1n = "n:r0Sbase:bus1n", "p.u., Zero-Sequence Impedance, Real Part"
      n_x0Sbase_bus1 = "n:x0Sbase:bus1", "p.u., Zero-Sequence Impedance, Imaginary Part"
      n_x0Sbase_bus1n = "n:x0Sbase:bus1n", "p.u., Zero-Sequence Impedance, Imaginary Part"
      n_z0Sbase_bus1 = "n:z0Sbase:bus1", "p.u., Zero-Sequence Impedance, Magnitude"
      n_z0Sbase_bus1n = "n:z0Sbase:bus1n", "p.u., Zero-Sequence Impedance, Magnitude"
      n_R1_bus1 = "n:R1:bus1", "Ohm, Positive-Sequence Impedance, Real Part"
      n_R1_bus1n = "n:R1:bus1n", "Ohm, Positive-Sequence Impedance, Real Part"
      n_X1_bus1 = "n:X1:bus1", "Ohm, Positive-Sequence Impedance, Imaginary Part"
      n_X1_bus1n = "n:X1:bus1n", "Ohm, Positive-Sequence Impedance, Imaginary Part"
      n_Z1_bus1 = "n:Z1:bus1", "Ohm, Positive-Sequence Impedance, Magnitude"
      n_Z1_bus1n = "n:Z1:bus1n", "Ohm, Positive-Sequence Impedance, Magnitude"
      n_phiz1_bus1 = "n:phiz1:bus1", "deg, Positive-Sequence Impedance, Angle"
      n_phiz1_bus1n = "n:phiz1:bus1n", "deg, Positive-Sequence Impedance, Angle"
      n_r1Sbase_bus1 = "n:r1Sbase:bus1", "p.u., Positive-Sequence Impedance, Real Part"
      n_r1Sbase_bus1n = "n:r1Sbase:bus1n", "p.u., Positive-Sequence Impedance, Real Part"
      n_x1Sbase_bus1 = "n:x1Sbase:bus1", "p.u., Positive-Sequence Impedance, Imaginary Part"
      n_x1Sbase_bus1n = "n:x1Sbase:bus1n", "p.u., Positive-Sequence Impedance, Imaginary Part"
      n_z1Sbase_bus1 = "n:z1Sbase:bus1", "p.u., Positive-Sequence Impedance, Magnitude"
      n_z1Sbase_bus1n = "n:z1Sbase:bus1n", "p.u., Positive-Sequence Impedance, Magnitude"
      n_R2_bus1 = "n:R2:bus1", "Ohm, Negative-Sequence Impedance, Real Part"
      n_R2_bus1n = "n:R2:bus1n", "Ohm, Negative-Sequence Impedance, Real Part"
      n_X2_bus1 = "n:X2:bus1", "Ohm, Negative-Sequence Impedance, Imaginary Part"
      n_X2_bus1n = "n:X2:bus1n", "Ohm, Negative-Sequence Impedance, Imaginary Part"
      n_Z2_bus1 = "n:Z2:bus1", "Ohm, Negative-Sequence Impedance, Magnitude"
      n_Z2_bus1n = "n:Z2:bus1n", "Ohm, Negative-Sequence Impedance, Magnitude"
      n_phiz2_bus1 = "n:phiz2:bus1", "deg, Negative-Sequence Impedance, Angle"
      n_phiz2_bus1n = "n:phiz2:bus1n", "deg, Negative-Sequence Impedance, Angle"
      n_r2Sbase_bus1 = "n:r2Sbase:bus1", "p.u., Negative-Sequence Impedance, Real Part"
      n_r2Sbase_bus1n = "n:r2Sbase:bus1n", "p.u., Negative-Sequence Impedance, Real Part"
      n_x2Sbase_bus1 = "n:x2Sbase:bus1", "p.u., Negative-Sequence Impedance, Imaginary Part"
      n_x2Sbase_bus1n = "n:x2Sbase:bus1n", "p.u., Negative-Sequence Impedance, Imaginary Part"
      n_z2Sbase_bus1 = "n:z2Sbase:bus1", "p.u., Negative-Sequence Impedance, Magnitude"
      n_z2Sbase_bus1n = "n:z2Sbase:bus1n", "p.u., Negative-Sequence Impedance, Magnitude"
      n_R_bus1_A = "n:R:bus1:A", "Ohm, Network Resistance, Line-Ground"
      n_R_bus1_B = "n:R:bus1:B", "Ohm, Network Resistance, Line-Ground"
      n_R_bus1_C = "n:R:bus1:C", "Ohm, Network Resistance, Line-Ground"
      n_R_bus1_N = "n:R:bus1:N", "Ohm, Network Resistance, Line-Ground"
      n_R_bus1n = "n:R:bus1n", "Ohm, Network Resistance, Line-Ground"
      n_X_bus1_A = "n:X:bus1:A", "Ohm, Network Reactance, Line-Ground"
      n_X_bus1_B = "n:X:bus1:B", "Ohm, Network Reactance, Line-Ground"
      n_X_bus1_C = "n:X:bus1:C", "Ohm, Network Reactance, Line-Ground"
      n_X_bus1_N = "n:X:bus1:N", "Ohm, Network Reactance, Line-Ground"
      n_X_bus1n = "n:X:bus1n", "Ohm, Network Reactance, Line-Ground"
      n_Z_bus1_A = "n:Z:bus1:A", "Ohm, Network Impedance, Line-Ground"
      n_Z_bus1_B = "n:Z:bus1:B", "Ohm, Network Impedance, Line-Ground"
      n_Z_bus1_C = "n:Z:bus1:C", "Ohm, Network Impedance, Line-Ground"
      n_Z_bus1_N = "n:Z:bus1:N", "Ohm, Network Impedance, Line-Ground"
      n_Z_bus1n = "n:Z:bus1n", "Ohm, Network Impedance, Line-Ground"
      n_phiz_bus1_A = "n:phiz:bus1:A", "deg, Angle of Network Impedance, Line-Ground"
      n_phiz_bus1_B = "n:phiz:bus1:B", "deg, Angle of Network Impedance, Line-Ground"
      n_phiz_bus1_C = "n:phiz:bus1:C", "deg, Angle of Network Impedance, Line-Ground"
      n_phiz_bus1_N = "n:phiz:bus1:N", "deg, Angle of Network Impedance, Line-Ground"
      n_phiz_bus1n = "n:phiz:bus1n", "deg, Angle of Network Impedance, Line-Ground"
      n_Rl_bus1_A = "n:Rl:bus1:A", "Ohm, Network Resistance, Line-Line"
      n_Rl_bus1_B = "n:Rl:bus1:B", "Ohm, Network Resistance, Line-Line"
      n_Rl_bus1_C = "n:Rl:bus1:C", "Ohm, Network Resistance, Line-Line"
      n_Rl_bus1_N = "n:Rl:bus1:N", "Ohm, Network Resistance, Line-Line"
      n_Rl_bus1n = "n:Rl:bus1n", "Ohm, Network Resistance, Line-Line"
      n_Xl_bus1_A = "n:Xl:bus1:A", "Ohm, Network Reactance, Line-Line"
      n_Xl_bus1_B = "n:Xl:bus1:B", "Ohm, Network Reactance, Line-Line"
      n_Xl_bus1_C = "n:Xl:bus1:C", "Ohm, Network Reactance, Line-Line"
      n_Xl_bus1_N = "n:Xl:bus1:N", "Ohm, Network Reactance, Line-Line"
      n_Xl_bus1n = "n:Xl:bus1n", "Ohm, Network Reactance, Line-Line"
      n_Zl_bus1_A = "n:Zl:bus1:A", "Ohm, Network Impedance, Line-Line"
      n_Zl_bus1_B = "n:Zl:bus1:B", "Ohm, Network Impedance, Line-Line"
      n_Zl_bus1_C = "n:Zl:bus1:C", "Ohm, Network Impedance, Line-Line"
      n_Zl_bus1_N = "n:Zl:bus1:N", "Ohm, Network Impedance, Line-Line"
      n_Zl_bus1n = "n:Zl:bus1n", "Ohm, Network Impedance, Line-Line"
      n_phizl_bus1_A = "n:phizl:bus1:A", "deg, Angle of Network Impedance, Line-Line"
      n_phizl_bus1_B = "n:phizl:bus1:B", "deg, Angle of Network Impedance, Line-Line"
      n_phizl_bus1_C = "n:phizl:bus1:C", "deg, Angle of Network Impedance, Line-Line"
      n_phizl_bus1_N = "n:phizl:bus1:N", "deg, Angle of Network Impedance, Line-Line"
      n_phizl_bus1n = "n:phizl:bus1n", "deg, Angle of Network Impedance, Line-Line"
      n_Rln_bus1_A = "n:Rln:bus1:A", "Ohm, Network Resistance, Line-Neutral"
      n_Rln_bus1_B = "n:Rln:bus1:B", "Ohm, Network Resistance, Line-Neutral"
      n_Rln_bus1_C = "n:Rln:bus1:C", "Ohm, Network Resistance, Line-Neutral"
      n_Rln_bus1_N = "n:Rln:bus1:N", "Ohm, Network Resistance, Line-Neutral"
      n_Rln_bus1n = "n:Rln:bus1n", "Ohm, Network Resistance, Line-Neutral"
      n_Xln_bus1_A = "n:Xln:bus1:A", "Ohm, Network Reactance, Line-Neutral"
      n_Xln_bus1_B = "n:Xln:bus1:B", "Ohm, Network Reactance, Line-Neutral"
      n_Xln_bus1_C = "n:Xln:bus1:C", "Ohm, Network Reactance, Line-Neutral"
      n_Xln_bus1_N = "n:Xln:bus1:N", "Ohm, Network Reactance, Line-Neutral"
      n_Xln_bus1n = "n:Xln:bus1n", "Ohm, Network Reactance, Line-Neutral"
      n_Zln_bus1_A = "n:Zln:bus1:A", "Ohm, Network Impedance, Line-Neutral"
      n_Zln_bus1_B = "n:Zln:bus1:B", "Ohm, Network Impedance, Line-Neutral"
      n_Zln_bus1_C = "n:Zln:bus1:C", "Ohm, Network Impedance, Line-Neutral"
      n_Zln_bus1_N = "n:Zln:bus1:N", "Ohm, Network Impedance, Line-Neutral"
      n_Zln_bus1n = "n:Zln:bus1n", "Ohm, Network Impedance, Line-Neutral"
      n_phizln_bus1_A = "n:phizln:bus1:A", "deg, Angle of Network Impedance, Line-Neutral"
      n_phizln_bus1_B = "n:phizln:bus1:B", "deg, Angle of Network Impedance, Line-Neutral"
      n_phizln_bus1_C = "n:phizln:bus1:C", "deg, Angle of Network Impedance, Line-Neutral"
      n_phizln_bus1_N = "n:phizln:bus1:N", "deg, Angle of Network Impedance, Line-Neutral"
      n_phizln_bus1n = "n:phizln:bus1n", "deg, Angle of Network Impedance, Line-Neutral"
      n_Rn_bus1 = "n:Rn:bus1", "Ohm, Network Resistance, Neutral"
      n_Rn_bus1n = "n:Rn:bus1n", "Ohm, Network Resistance, Neutral"
      n_Xn_bus1 = "n:Xn:bus1", "Ohm, Network Reactance, Neutral"
      n_Xn_bus1n = "n:Xn:bus1n", "Ohm, Network Reactance, Neutral"
      n_Zn_bus1 = "n:Zn:bus1", "Ohm, Network Impedance, Neutral"
      n_Zn_bus1n = "n:Zn:bus1n", "Ohm, Network Impedance, Neutral"
      n_phizn_bus1 = "n:phizn:bus1", "deg, Angle of Network Impedance, Neutral"
      n_phizn_bus1n = "n:phizn:bus1n", "deg, Angle of Network Impedance, Neutral"
      n_eff_bus1_A = "n:eff:bus1:A", "Earth Fault Factor"
      n_eff_bus1_B = "n:eff:bus1:B", "Earth Fault Factor"
      n_eff_bus1_C = "n:eff:bus1:C", "Earth Fault Factor"
      n_eff_bus1_N = "n:eff:bus1:N", "Earth Fault Factor"
      n_eff_bus1n = "n:eff:bus1n", "Earth Fault Factor"
      n_X0toX1_bus1 = "n:X0toX1:bus1", "X0/X1"
      n_X0toX1_bus1n = "n:X0toX1:bus1n", "X0/X1"
      n_in_bus1 = "n:in:bus1", "p.u., Neutral Current, Magnitude"
      n_in_bus1n = "n:in:bus1n", "p.u., Neutral Current, Magnitude"
      n_In_bus1 = "n:In:bus1", "kA, Neutral Current, Magnitude"
      n_In_bus1n = "n:In:bus1n", "kA, Neutral Current, Magnitude"
      n_phiin_bus1 = "n:phiin:bus1", "deg, Neutral Current, Angle"
      n_phiin_bus1n = "n:phiin:bus1n", "deg, Neutral Current, Angle"
      n_i0r_bus1 = "n:i0r:bus1", "p.u., Zero-Sequence Current, Real Part"
      n_i0r_bus1n = "n:i0r:bus1n", "p.u., Zero-Sequence Current, Real Part"
      n_i0i_bus1 = "n:i0i:bus1", "p.u., Zero-Sequence Current, Imaginary Part"
      n_i0i_bus1n = "n:i0i:bus1n", "p.u., Zero-Sequence Current, Imaginary Part"
      n_i0_bus1 = "n:i0:bus1", "p.u., Zero-Sequence Current, Magnitude"
      n_i0_bus1n = "n:i0:bus1n", "p.u., Zero-Sequence Current, Magnitude"
      n_I0_bus1 = "n:I0:bus1", "kA, Zero-Sequence Current, Magnitude"
      n_I0_bus1n = "n:I0:bus1n", "kA, Zero-Sequence Current, Magnitude"
      n_I0x3_bus1 = "n:I0x3:bus1", "kA, 3*I0"
      n_I0x3_bus1n = "n:I0x3:bus1n", "kA, 3*I0"
      n_phii0_bus1 = "n:phii0:bus1", "deg, Zero-Sequence Current, Angle"
      n_phii0_bus1n = "n:phii0:bus1n", "deg, Zero-Sequence Current, Angle"
      n_i1r_bus1 = "n:i1r:bus1", "p.u., Positive-Sequence Current, Real Part"
      n_i1r_bus1n = "n:i1r:bus1n", "p.u., Positive-Sequence Current, Real Part"
      n_i1i_bus1 = "n:i1i:bus1", "p.u., Positive-Sequence Current, Imaginary Part"
      n_i1i_bus1n = "n:i1i:bus1n", "p.u., Positive-Sequence Current, Imaginary Part"
      n_i1_bus1 = "n:i1:bus1", "p.u., Positive-Sequence Current, Magnitude"
      n_i1_bus1n = "n:i1:bus1n", "p.u., Positive-Sequence Current, Magnitude"
      n_I1_bus1 = "n:I1:bus1", "kA, Positive-Sequence Current, Magnitude"
      n_I1_bus1n = "n:I1:bus1n", "kA, Positive-Sequence Current, Magnitude"
      n_phii1_bus1 = "n:phii1:bus1", "deg, Positive-Sequence Current, Angle"
      n_phii1_bus1n = "n:phii1:bus1n", "deg, Positive-Sequence Current, Angle"
      n_i2r_bus1 = "n:i2r:bus1", "p.u., Negative-Sequence Current, Real Part"
      n_i2r_bus1n = "n:i2r:bus1n", "p.u., Negative-Sequence Current, Real Part"
      n_i2i_bus1 = "n:i2i:bus1", "p.u., Negative-Sequence Current, Imaginary Part"
      n_i2i_bus1n = "n:i2i:bus1n", "p.u., Negative-Sequence Current, Imaginary Part"
      n_i2_bus1 = "n:i2:bus1", "p.u., Negative-Sequence Current, Magnitude"
      n_i2_bus1n = "n:i2:bus1n", "p.u., Negative-Sequence Current, Magnitude"
      n_I2_bus1 = "n:I2:bus1", "kA, Negative-Sequence Current, Magnitude"
      n_I2_bus1n = "n:I2:bus1n", "kA, Negative-Sequence Current, Magnitude"
      n_phii2_bus1 = "n:phii2:bus1", "deg, Negative-Sequence Current, Angle"
      n_phii2_bus1n = "n:phii2:bus1n", "deg, Negative-Sequence Current, Angle"
      n_du_bus1_A = "n:du:bus1:A", "%, Voltage Deviation"
      n_du_bus1_B = "n:du:bus1:B", "%, Voltage Deviation"
      n_du_bus1_C = "n:du:bus1:C", "%, Voltage Deviation"
      n_du_bus1_N = "n:du:bus1:N", "%, Voltage Deviation"
      n_du_bus1n = "n:du:bus1n", "%, Voltage Deviation"
      n_urpref_bus1_A = "n:urpref:bus1:A", "p.u., Pre-fault Voltage, Real Part"
      n_urpref_bus1_B = "n:urpref:bus1:B", "p.u., Pre-fault Voltage, Real Part"
      n_urpref_bus1_C = "n:urpref:bus1:C", "p.u., Pre-fault Voltage, Real Part"
      n_urpref_bus1_N = "n:urpref:bus1:N", "p.u., Pre-fault Voltage, Real Part"
      n_urpref_bus1n = "n:urpref:bus1n", "p.u., Pre-fault Voltage, Real Part"
      n_uipref_bus1_A = "n:uipref:bus1:A", "p.u., Pre-fault Voltage, Imaginary Part"
      n_uipref_bus1_B = "n:uipref:bus1:B", "p.u., Pre-fault Voltage, Imaginary Part"
      n_uipref_bus1_C = "n:uipref:bus1:C", "p.u., Pre-fault Voltage, Imaginary Part"
      n_uipref_bus1_N = "n:uipref:bus1:N", "p.u., Pre-fault Voltage, Imaginary Part"
      n_uipref_bus1n = "n:uipref:bus1n", "p.u., Pre-fault Voltage, Imaginary Part"
      n_upref_bus1_A = "n:upref:bus1:A", "p.u., Pre-fault Voltage, Magnitude"
      n_upref_bus1_B = "n:upref:bus1:B", "p.u., Pre-fault Voltage, Magnitude"
      n_upref_bus1_C = "n:upref:bus1:C", "p.u., Pre-fault Voltage, Magnitude"
      n_upref_bus1_N = "n:upref:bus1:N", "p.u., Pre-fault Voltage, Magnitude"
      n_upref_bus1n = "n:upref:bus1n", "p.u., Pre-fault Voltage, Magnitude"
      n_uprefpc_bus1_A = "n:uprefpc:bus1:A", "%, Pre-fault Voltage, Magnitude"
      n_uprefpc_bus1_B = "n:uprefpc:bus1:B", "%, Pre-fault Voltage, Magnitude"
      n_uprefpc_bus1_C = "n:uprefpc:bus1:C", "%, Pre-fault Voltage, Magnitude"
      n_uprefpc_bus1_N = "n:uprefpc:bus1:N", "%, Pre-fault Voltage, Magnitude"
      n_uprefpc_bus1n = "n:uprefpc:bus1n", "%, Pre-fault Voltage, Magnitude"
      n_Upref_bus1_A = "n:Upref:bus1:A", "kV, Line-Ground Pre-fault Voltage, Magnitude"
      n_Upref_bus1_B = "n:Upref:bus1:B", "kV, Line-Ground Pre-fault Voltage, Magnitude"
      n_Upref_bus1_C = "n:Upref:bus1:C", "kV, Line-Ground Pre-fault Voltage, Magnitude"
      n_Upref_bus1_N = "n:Upref:bus1:N", "kV, Line-Ground Pre-fault Voltage, Magnitude"
      n_Upref_bus1n = "n:Upref:bus1n", "kV, Line-Ground Pre-fault Voltage, Magnitude"
      n_Ulpref_bus1_A = "n:Ulpref:bus1:A", "kV, Line-Line Pre-fault Voltage, Magnitude"
      n_Ulpref_bus1_B = "n:Ulpref:bus1:B", "kV, Line-Line Pre-fault Voltage, Magnitude"
      n_Ulpref_bus1_C = "n:Ulpref:bus1:C", "kV, Line-Line Pre-fault Voltage, Magnitude"
      n_Ulpref_bus1_N = "n:Ulpref:bus1:N", "kV, Line-Line Pre-fault Voltage, Magnitude"
      n_Ulpref_bus1n = "n:Ulpref:bus1n", "kV, Line-Line Pre-fault Voltage, Magnitude"
      n_phiupref_bus1_A = "n:phiupref:bus1:A", "deg, Pre-fault Voltage, Angle"
      n_phiupref_bus1_B = "n:phiupref:bus1:B", "deg, Pre-fault Voltage, Angle"
      n_phiupref_bus1_C = "n:phiupref:bus1:C", "deg, Pre-fault Voltage, Angle"
      n_phiupref_bus1_N = "n:phiupref:bus1:N", "deg, Pre-fault Voltage, Angle"
      n_phiupref_bus1n = "n:phiupref:bus1n", "deg, Pre-fault Voltage, Angle"
      n_phiui_bus1_A = "n:phiui:bus1:A", "deg, Angle between Voltage and Current"
      n_phiui_bus1_B = "n:phiui:bus1:B", "deg, Angle between Voltage and Current"
      n_phiui_bus1_C = "n:phiui:bus1:C", "deg, Angle between Voltage and Current"
      n_phiui_bus1_N = "n:phiui:bus1:N", "deg, Angle between Voltage and Current"
      n_phiui_bus1n = "n:phiui:bus1n", "deg, Angle between Voltage and Current"
      n_phiu1i1_bus1 = "n:phiu1i1:bus1", "deg, Angle between Voltage and Current in positive sequence system"
      n_phiu1i1_bus1n = "n:phiu1i1:bus1n", "deg, Angle between Voltage and Current in positive sequence system"
      n_phiu2i2_bus1 = "n:phiu2i2:bus1", "deg, Angle between Voltage and Current in negative sequence system"
      n_phiu2i2_bus1n = "n:phiu2i2:bus1n", "deg, Angle between Voltage and Current in negative sequence system"
      n_phiu0i0_bus1 = "n:phiu0i0:bus1", "deg, Angle between Voltage and Current in zero-sequence system"
      n_phiu0i0_bus1n = "n:phiu0i0:bus1n", "deg, Angle between Voltage and Current in zero-sequence system"
      n_Ithload_bus1n = "n:Ithload:bus1n", "%, Loading, Thermal equivalent Current"
      n_Ipload_bus1n = "n:Ipload:bus1n", "%, Loading, Peak Short-Circuit Current"
      n_Ithrtk_bus1n = "n:Ithrtk:bus1n", "kA, Rated Short-Time Current (Tk)"
      n_cfac_bus1n = "n:cfac:bus1n", "Voltage Factor"
      n_XtoR_bus1n = "n:XtoR:bus1n", "X/R ratio (peak)"
      n_RtoX_bus1n = "n:RtoX:bus1n", "R/X ratio (peak)"
      c_z2_r = "c:z2:r", "p.u., Negative-sequence impedance, Real Part"
      c_z2_i = "c:z2:i", "p.u., Negative-sequence impedance, Imaginary Part"
      c_z2_phi = "c:z2:phi", "deg, Negative-sequence impedance, Angle"
      c_z2_mag = "c:z2:mag", "p.u., Negative-sequence impedance, Magnitude"
      c_z2_phirad = "c:z2:phirad", "rad, Negative-sequence impedance, Angle"
      c_z0_r = "c:z0:r", "p.u., Zero-sequence impedance, Real Part"
      c_z0_i = "c:z0:i", "p.u., Zero-sequence impedance, Imaginary Part"
      c_z0_phi = "c:z0:phi", "deg, Zero-sequence impedance, Angle"
      c_z0_mag = "c:z0:mag", "p.u., Zero-sequence impedance, Magnitude"
      c_z0_phirad = "c:z0:phirad", "rad, Zero-sequence impedance, Angle"
      c_ze_r = "c:ze:r", "p.u., Grounding impedance, Real Part"
      c_ze_i = "c:ze:i", "p.u., Grounding impedance, Imaginary Part"
      c_ze_phi = "c:ze:phi", "deg, Grounding impedance, Angle"
      c_ze_mag = "c:ze:mag", "p.u., Grounding impedance, Magnitude"
      c_ze_phirad = "c:ze:phirad", "rad, Grounding impedance, Angle"
      c_pG = "c:pG", "%., Range of Voltage Regulation (+/-)"

    class ElmTerm(Enum):
      _init_ = 'value __doc__'

      m_ur_A = "m:ur:A", "p.u., Line-Ground Voltage, Real Part"
      m_ur_B = "m:ur:B", "p.u., Line-Ground Voltage, Real Part"
      m_ur_C = "m:ur:C", "p.u., Line-Ground Voltage, Real Part"
      m_ui_A = "m:ui:A", "p.u., Line-Ground Voltage, Imaginary Part"
      m_ui_B = "m:ui:B", "p.u., Line-Ground Voltage, Imaginary Part"
      m_ui_C = "m:ui:C", "p.u., Line-Ground Voltage, Imaginary Part"
      m_u_A = "m:u:A", "p.u., Line-Ground Voltage, Magnitude"
      m_u_B = "m:u:B", "p.u., Line-Ground Voltage, Magnitude"
      m_u_C = "m:u:C", "p.u., Line-Ground Voltage, Magnitude"
      m_upc_A = "m:upc:A", "%, Line-Ground Voltage, Magnitude"
      m_upc_B = "m:upc:B", "%, Line-Ground Voltage, Magnitude"
      m_upc_C = "m:upc:C", "%, Line-Ground Voltage, Magnitude"
      m_U_A = "m:U:A", "kV, Line-Ground Voltage, Magnitude"
      m_U_B = "m:U:B", "kV, Line-Ground Voltage, Magnitude"
      m_U_C = "m:U:C", "kV, Line-Ground Voltage, Magnitude"
      m_phiu_A = "m:phiu:A", "deg, Line-Ground Voltage, Angle"
      m_phiu_B = "m:phiu:B", "deg, Line-Ground Voltage, Angle"
      m_phiu_C = "m:phiu:C", "deg, Line-Ground Voltage, Angle"
      m_phiurel = "m:phiurel", "deg, Voltage, Relative Angle"
      m_ul_A = "m:ul:A", "p.u., Line-Line Voltage, Magnitude"
      m_ul_B = "m:ul:B", "p.u., Line-Line Voltage, Magnitude"
      m_ul_C = "m:ul:C", "p.u., Line-Line Voltage, Magnitude"
      m_ulpc_A = "m:ulpc:A", "%, Line-Line Voltage, Magnitude"
      m_ulpc_B = "m:ulpc:B", "%, Line-Line Voltage, Magnitude"
      m_ulpc_C = "m:ulpc:C", "%, Line-Line Voltage, Magnitude"
      m_Ul_A = "m:Ul:A", "kV, Line-Line Voltage, Magnitude"
      m_Ul_B = "m:Ul:B", "kV, Line-Line Voltage, Magnitude"
      m_Ul_C = "m:Ul:C", "kV, Line-Line Voltage, Magnitude"
      m_phiul_A = "m:phiul:A", "deg, Line-Line Voltage, Angle"
      m_phiul_B = "m:phiul:B", "deg, Line-Line Voltage, Angle"
      m_phiul_C = "m:phiul:C", "deg, Line-Line Voltage, Angle"
      m_u0 = "m:u0", "p.u., Zero-Sequence Voltage, Magnitude"
      m_U0 = "m:U0", "kV, Zero-Sequence Voltage, Magnitude"
      m_U0x3 = "m:U0x3", "kV, 3*U0"
      m_phiu0 = "m:phiu0", "deg, Zero-Sequence Voltage, Angle"
      m_u1 = "m:u1", "p.u., Positive-Sequence Voltage, Magnitude"
      m_u1pc = "m:u1pc", "%, Positive-Sequence Voltage, Magnitude"
      m_u1r = "m:u1r", "p.u., Positive-Sequence Voltage, Real Part"
      m_u1i = "m:u1i", "p.u., Positive-Sequence Voltage, Imaginary Part"
      m_U1 = "m:U1", "kV, Line-Ground Positive-Sequence Voltage, Magnitude"
      m_phiu1 = "m:phiu1", "deg, Positive-Sequence Voltage, Angle"
      m_u2 = "m:u2", "p.u., Negative-Sequence Voltage, Magnitude"
      m_U2 = "m:U2", "kV, Line-Ground Negative-Sequence Voltage, Magnitude"
      m_phiu2 = "m:phiu2", "deg, Negative-Sequence Voltage, Angle"
      m_U1l = "m:U1l", "kV, Line-Line Positive-Sequence Voltage, Magnitude"
      m_U2l = "m:U2l", "kV, Line-Line Negative-Sequence Voltage, Magnitude"
      m_uln_A = "m:uln:A", "p.u., Line-Neutral Voltage, Magnitude"
      m_uln_B = "m:uln:B", "p.u., Line-Neutral Voltage, Magnitude"
      m_uln_C = "m:uln:C", "p.u., Line-Neutral Voltage, Magnitude"
      m_Uln_A = "m:Uln:A", "kV, Line-Neutral Voltage, Magnitude"
      m_Uln_B = "m:Uln:B", "kV, Line-Neutral Voltage, Magnitude"
      m_Uln_C = "m:Uln:C", "kV, Line-Neutral Voltage, Magnitude"
      m_phiuln_A = "m:phiuln:A", "deg, Line-Neutral Voltage, Angle"
      m_phiuln_B = "m:phiuln:B", "deg, Line-Neutral Voltage, Angle"
      m_phiuln_C = "m:phiuln:C", "deg, Line-Neutral Voltage, Angle"
      m_un = "m:un", "p.u., Neutral-Ground Voltage, Magnitude"
      m_Un = "m:Un", "kV, Neutral-Ground Voltage, Magnitude"
      m_phiun = "m:phiun", "deg, Neutral-Ground Voltage, Angle"
      m_upht_A = "m:upht:A", "p.u., Phase Technology dependent Voltage, Magnitude"
      m_upht_B = "m:upht:B", "p.u., Phase Technology dependent Voltage, Magnitude"
      m_upht_C = "m:upht:C", "p.u., Phase Technology dependent Voltage, Magnitude"
      m_Upht_A = "m:Upht:A", "kV, Phase Technology dependent Voltage, Magnitude"
      m_Upht_B = "m:Upht:B", "kV, Phase Technology dependent Voltage, Magnitude"
      m_Upht_C = "m:Upht:C", "kV, Phase Technology dependent Voltage, Magnitude"
      m_um = "m:um", "p.u., Average-Voltage, Magnitude"
      m_Um = "m:Um", "kV, Average-Voltage, Magnitude"
      m_uphtmin = "m:uphtmin", "p.u., Minimum of Phase Technology dependent Voltage, Magnitude"
      m_uphtmax = "m:uphtmax", "p.u., Maximum of Phase Technology dependent Voltage, Magnitude"
      m_Pgen = "m:Pgen", "MW, Generation, Active Power"
      m_Qgen = "m:Qgen", "Mvar, Generation, Reactive Power"
      m_Pmot = "m:Pmot", "MW, Motor Load, Active Power"
      m_Qmot = "m:Qmot", "Mvar, Motor Load, Reactive Power"
      m_Pload = "m:Pload", "MW, General Load, Active Power"
      m_Qload = "m:Qload", "Mvar, General Load, Reactive Power"
      m_Pcomp = "m:Pcomp", "MW, Compensation (Losses)"
      m_Qcomp = "m:Qcomp", "Mvar, Compensation"
      m_Pnet = "m:Pnet", "MW, External Networks, Active Power"
      m_Qnet = "m:Qnet", "Mvar, External Networks, Reactive Power"
      m_Pflow = "m:Pflow", "MW, Power Flow, Active Power"
      m_Qflow = "m:Qflow", "Mvar, Power Flow, Reactive Power"
      m_Pout = "m:Pout", "MW, Outgoing Flow, Active Power"
      m_Qout = "m:Qout", "Mvar, Outgoing Power, Reactive Power"
      m_Sout = "m:Sout", "MVA, Outgoing Power, Apparent Power"
      m_cosphiout = "m:cosphiout", "Outgoing Power, Power Factor"
      m_Pbalance = "m:Pbalance", "MW, Active Power Balance (=0)"
      m_Qbalance = "m:Qbalance", "Mvar, Reactive Power Balance (=0)"
      m_ubfac = "m:ubfac", "%, Unbalance Factor"
      m_LossPdown = "m:LossPdown", "MW, Losses, downstream"
      m_LossQdown = "m:LossQdown", "Mvar, Losses, downstream (Reactive Power)"
      m_LossPdownload = "m:LossPdownload", "MW, Load losses, downstream"
      m_LossQdownload = "m:LossQdownload", "Mvar, Load losses, downstream"
      m_LossPdownnoload = "m:LossPdownnoload", "MW, No load losses, downstream"
      m_LossQdownnoload = "m:LossQdownnoload", "Mvar, No load losses, downstream"
      m_Imax = "m:Imax", "kA, Max. Current"
      m_ImaxPh_A = "m:ImaxPh:A", "kA, Max. Current (per phase)"
      m_ImaxPh_B = "m:ImaxPh:B", "kA, Max. Current (per phase)"
      m_ImaxPh_C = "m:ImaxPh:C", "kA, Max. Current (per phase)"
      m_loading = "m:loading", "%, Max. Loading"
      m_loadingPh_A = "m:loadingPh:A", "%, Loading"
      m_loadingPh_B = "m:loadingPh:B", "%, Loading"
      m_loadingPh_C = "m:loadingPh:C", "%, Loading"
      e_vtarget = "e:vtarget", "p.u., Target Voltage"
      e_Vtarget = "e:Vtarget", "kV, Target Voltage"
      e_dvmax = "e:dvmax", "%, Delta V max"
      e_dvmin = "e:dvmin", "%, Delta V min"
      e_ivpriority = "e:ivpriority", "Priority"
      e_vmax = "e:vmax", "p.u., Upper Voltage Limit"
      e_vmin = "e:vmin", "p.u., Lower Voltage Limit"
      e_cpFeed = "e:cpFeed", "Feeder"
      e_ciDist = "e:ciDist", "Distance from infeed in number of buses"
      e_ciLater = "e:ciLater", "Lateral Index"
      e_ciDistRoot = "e:ciDistRoot", "Distance from first infeed in number of buses"
      e_ciDistAll = "e:ciDistAll", "Distance from infeed in number of buses including switches"
      e_ciDistAllRoot = "e:ciDistAllRoot", "Distance from first infeed in number of buses including switches"
      r_cpFeed = "r:cpFeed", "Feeder"
      m_Ikss = "m:Ikss", "kA, Initial Short-Circuit Current"
      m_I = "m:I", "kA, Short-Circuit Current"
      m_phii = "m:phii", "deg, Phase Current, Angle"
      m_Skss = "m:Skss", "MVA, Initial Short-Circuit Power"
      m_ip = "m:ip", "kA, Peak Short-Circuit Current"
      m_Ib = "m:Ib", "kA, Short-Circuit Breaking Current"
      m_Sb = "m:Sb", "MVA, Short-Circuit Breaking Power"
      m_Ik = "m:Ik", "kA, Steady-State Short-Circuit Current"
      m_Ith = "m:Ith", "kA, Thermal equivalent Short-Circuit Current"
      m_R = "m:R", "Ohm, Short-Circuit Impedance, Real Part"
      m_X = "m:X", "Ohm, Short-Circuit Impedance, Imaginary Part"
      m_Z = "m:Z", "Ohm, Short-Circuit Impedance, Magnitude"
      m_phiz = "m:phiz", "deg, Short-Circuit Impedance, Angle"
      m_rSbase = "m:rSbase", "p.u., Short-Circuit Impedance, Real Part"
      m_xSbase = "m:xSbase", "p.u., Short-Circuit Impedance, Imaginary Part"
      m_zSbase = "m:zSbase", "p.u., Short-Circuit Impedance, Magnitude"
      m_XtoR = "m:XtoR", "X/R ratio (peak)"
      m_RtoX = "m:RtoX", "R/X ratio (peak)"
      m_ur = "m:ur", "p.u., Voltage, Real Part"
      m_ui = "m:ui", "p.u., Voltage, Imaginary Part"
      m_u = "m:u", "p.u., Voltage, Magnitude"
      m_upc = "m:upc", "%, Voltage, Magnitude"
      m_U = "m:U", "kV, Line-Ground Voltage, Magnitude"
      m_Ul = "m:Ul", "kV, Line-Line Voltage, Magnitude"
      m_phiu = "m:phiu", "deg, Voltage, Angle"
      m_du = "m:du", "%, Voltage Deviation"
      m_urpref = "m:urpref", "p.u., Pre-fault Voltage, Real Part"
      m_uipref = "m:uipref", "p.u., Pre-fault Voltage, Imaginary Part"
      m_upref = "m:upref", "p.u., Pre-fault Voltage, Magnitude"
      m_uprefpc = "m:uprefpc", "%, Pre-fault Voltage, Magnitude"
      m_Upref = "m:Upref", "kV, Line-Ground Pre-fault Voltage, Magnitude"
      m_Ulpref = "m:Ulpref", "kV, Line-Line Pre-fault Voltage, Magnitude"
      m_phiupref = "m:phiupref", "deg, Pre-fault Voltage, Angle"
      m_phiui = "m:phiui", "deg, Angle between Voltage and Current"
      m_cfac = "m:cfac", "Voltage Factor"
      m_idc = "m:idc", "kA, Decaying Component"
      m_Iasy = "m:Iasy", "kA, Asymmetrical Breaking Current"
      m_Ithload = "m:Ithload", "%, Loading, Thermal equivalent Current"
      m_Ipload = "m:Ipload", "%, Loading, Peak Short-Circuit Current"
      m_Ithrtk = "m:Ithrtk", "kA, Rated Short-Time Current (Tk)"
      m_Tn = "m:Tn", "ms, Grid Time Constant"
      m_XtoR_b = "m:XtoR_b", "X/R ratio (dc)"
      m_RtoX_b = "m:RtoX_b", "R/X ratio (dc)"
      m_fctof_b = "m:fctof_b", "Equivalent Frequency fc/f (dc)"
      m_Ikss_A = "m:Ikss:A", "kA, Initial Short-Circuit Current"
      m_Ikss_B = "m:Ikss:B", "kA, Initial Short-Circuit Current"
      m_Ikss_C = "m:Ikss:C", "kA, Initial Short-Circuit Current"
      m_I_A = "m:I:A", "kA, Short-Circuit Current"
      m_I_B = "m:I:B", "kA, Short-Circuit Current"
      m_I_C = "m:I:C", "kA, Short-Circuit Current"
      m_phii_A = "m:phii:A", "deg, Phase Current, Angle"
      m_phii_B = "m:phii:B", "deg, Phase Current, Angle"
      m_phii_C = "m:phii:C", "deg, Phase Current, Angle"
      m_Skss_A = "m:Skss:A", "MVA, Initial Short-Circuit Power"
      m_Skss_B = "m:Skss:B", "MVA, Initial Short-Circuit Power"
      m_Skss_C = "m:Skss:C", "MVA, Initial Short-Circuit Power"
      m_Skssx3_A = "m:Skssx3:A", "MVA, 3*Initial Short-Circuit Power"
      m_Skssx3_B = "m:Skssx3:B", "MVA, 3*Initial Short-Circuit Power"
      m_Skssx3_C = "m:Skssx3:C", "MVA, 3*Initial Short-Circuit Power"
      m_ip_A = "m:ip:A", "kA, Peak Short-Circuit Current"
      m_ip_B = "m:ip:B", "kA, Peak Short-Circuit Current"
      m_ip_C = "m:ip:C", "kA, Peak Short-Circuit Current"
      m_Ib_A = "m:Ib:A", "kA, Short-Circuit Breaking Current"
      m_Ib_B = "m:Ib:B", "kA, Short-Circuit Breaking Current"
      m_Ib_C = "m:Ib:C", "kA, Short-Circuit Breaking Current"
      m_Sb_A = "m:Sb:A", "MVA, Short-Circuit Breaking Power"
      m_Sb_B = "m:Sb:B", "MVA, Short-Circuit Breaking Power"
      m_Sb_C = "m:Sb:C", "MVA, Short-Circuit Breaking Power"
      m_Ith_A = "m:Ith:A", "kA, Thermal equivalent Short-Circuit Current"
      m_Ith_B = "m:Ith:B", "kA, Thermal equivalent Short-Circuit Current"
      m_Ith_C = "m:Ith:C", "kA, Thermal equivalent Short-Circuit Current"
      m_Ithmax = "m:Ithmax", "kA, Max. Thermal equivalent Short-Circuit Current"
      m_Ik_A = "m:Ik:A", "kA, Steady-State Short-Circuit Current"
      m_Ik_B = "m:Ik:B", "kA, Steady-State Short-Circuit Current"
      m_Ik_C = "m:Ik:C", "kA, Steady-State Short-Circuit Current"
      m_R0 = "m:R0", "Ohm, Zero-Sequence Impedance, Real Part"
      m_X0 = "m:X0", "Ohm, Zero-Sequence Impedance, Imaginary Part"
      m_Z0 = "m:Z0", "Ohm, Zero-Sequence Impedance, Magnitude"
      m_phiz0 = "m:phiz0", "deg, Zero-Sequence Impedance, Angle"
      m_r0Sbase = "m:r0Sbase", "p.u., Zero-Sequence Impedance, Real Part"
      m_x0Sbase = "m:x0Sbase", "p.u., Zero-Sequence Impedance, Imaginary Part"
      m_z0Sbase = "m:z0Sbase", "p.u., Zero-Sequence Impedance, Magnitude"
      m_R1 = "m:R1", "Ohm, Positive-Sequence Impedance, Real Part"
      m_X1 = "m:X1", "Ohm, Positive-Sequence Impedance, Imaginary Part"
      m_Z1 = "m:Z1", "Ohm, Positive-Sequence Impedance, Magnitude"
      m_phiz1 = "m:phiz1", "deg, Positive-Sequence Impedance, Angle"
      m_r1Sbase = "m:r1Sbase", "p.u., Positive-Sequence Impedance, Real Part"
      m_x1Sbase = "m:x1Sbase", "p.u., Positive-Sequence Impedance, Imaginary Part"
      m_z1Sbase = "m:z1Sbase", "p.u., Positive-Sequence Impedance, Magnitude"
      m_R2 = "m:R2", "Ohm, Negative-Sequence Impedance, Real Part"
      m_X2 = "m:X2", "Ohm, Negative-Sequence Impedance, Imaginary Part"
      m_Z2 = "m:Z2", "Ohm, Negative-Sequence Impedance, Magnitude"
      m_phiz2 = "m:phiz2", "deg, Negative-Sequence Impedance, Angle"
      m_r2Sbase = "m:r2Sbase", "p.u., Negative-Sequence Impedance, Real Part"
      m_x2Sbase = "m:x2Sbase", "p.u., Negative-Sequence Impedance, Imaginary Part"
      m_z2Sbase = "m:z2Sbase", "p.u., Negative-Sequence Impedance, Magnitude"
      m_R_A = "m:R:A", "Ohm, Network Resistance, Line-Ground"
      m_R_B = "m:R:B", "Ohm, Network Resistance, Line-Ground"
      m_R_C = "m:R:C", "Ohm, Network Resistance, Line-Ground"
      m_X_A = "m:X:A", "Ohm, Network Reactance, Line-Ground"
      m_X_B = "m:X:B", "Ohm, Network Reactance, Line-Ground"
      m_X_C = "m:X:C", "Ohm, Network Reactance, Line-Ground"
      m_Z_A = "m:Z:A", "Ohm, Network Impedance, Line-Ground"
      m_Z_B = "m:Z:B", "Ohm, Network Impedance, Line-Ground"
      m_Z_C = "m:Z:C", "Ohm, Network Impedance, Line-Ground"
      m_phiz_A = "m:phiz:A", "deg, Angle of Network Impedance, Line-Ground"
      m_phiz_B = "m:phiz:B", "deg, Angle of Network Impedance, Line-Ground"
      m_phiz_C = "m:phiz:C", "deg, Angle of Network Impedance, Line-Ground"
      m_Rl_A = "m:Rl:A", "Ohm, Network Resistance, Line-Line"
      m_Rl_B = "m:Rl:B", "Ohm, Network Resistance, Line-Line"
      m_Rl_C = "m:Rl:C", "Ohm, Network Resistance, Line-Line"
      m_Xl_A = "m:Xl:A", "Ohm, Network Reactance, Line-Line"
      m_Xl_B = "m:Xl:B", "Ohm, Network Reactance, Line-Line"
      m_Xl_C = "m:Xl:C", "Ohm, Network Reactance, Line-Line"
      m_Zl_A = "m:Zl:A", "Ohm, Network Impedance, Line-Line"
      m_Zl_B = "m:Zl:B", "Ohm, Network Impedance, Line-Line"
      m_Zl_C = "m:Zl:C", "Ohm, Network Impedance, Line-Line"
      m_phizl_A = "m:phizl:A", "deg, Angle of Network Impedance, Line-Line"
      m_phizl_B = "m:phizl:B", "deg, Angle of Network Impedance, Line-Line"
      m_phizl_C = "m:phizl:C", "deg, Angle of Network Impedance, Line-Line"
      m_Rln_A = "m:Rln:A", "Ohm, Network Resistance, Line-Neutral"
      m_Rln_B = "m:Rln:B", "Ohm, Network Resistance, Line-Neutral"
      m_Rln_C = "m:Rln:C", "Ohm, Network Resistance, Line-Neutral"
      m_Xln_A = "m:Xln:A", "Ohm, Network Reactance, Line-Neutral"
      m_Xln_B = "m:Xln:B", "Ohm, Network Reactance, Line-Neutral"
      m_Xln_C = "m:Xln:C", "Ohm, Network Reactance, Line-Neutral"
      m_Zln_A = "m:Zln:A", "Ohm, Network Impedance, Line-Neutral"
      m_Zln_B = "m:Zln:B", "Ohm, Network Impedance, Line-Neutral"
      m_Zln_C = "m:Zln:C", "Ohm, Network Impedance, Line-Neutral"
      m_phizln_A = "m:phizln:A", "deg, Angle of Network Impedance, Line-Neutral"
      m_phizln_B = "m:phizln:B", "deg, Angle of Network Impedance, Line-Neutral"
      m_phizln_C = "m:phizln:C", "deg, Angle of Network Impedance, Line-Neutral"
      m_Rn = "m:Rn", "Ohm, Network Resistance, Neutral"
      m_Xn = "m:Xn", "Ohm, Network Reactance, Neutral"
      m_Zn = "m:Zn", "Ohm, Network Impedance, Neutral"
      m_phizn = "m:phizn", "deg, Angle of Network Impedance, Neutral"
      m_eff_A = "m:eff:A", "Earth Fault Factor"
      m_eff_B = "m:eff:B", "Earth Fault Factor"
      m_eff_C = "m:eff:C", "Earth Fault Factor"
      m_X0toX1 = "m:X0toX1", "X0/X1"
      m_in = "m:in", "p.u., Neutral Current, Magnitude"
      m_In = "m:In", "kA, Neutral Current, Magnitude"
      m_phiin = "m:phiin", "deg, Neutral Current, Angle"
      m_i0r = "m:i0r", "p.u., Zero-Sequence Current, Real Part"
      m_i0i = "m:i0i", "p.u., Zero-Sequence Current, Imaginary Part"
      m_i0 = "m:i0", "p.u., Zero-Sequence Current, Magnitude"
      m_I0 = "m:I0", "kA, Zero-Sequence Current, Magnitude"
      m_I0x3 = "m:I0x3", "kA, 3*I0"
      m_phii0 = "m:phii0", "deg, Zero-Sequence Current, Angle"
      m_i1r = "m:i1r", "p.u., Positive-Sequence Current, Real Part"
      m_i1i = "m:i1i", "p.u., Positive-Sequence Current, Imaginary Part"
      m_i1 = "m:i1", "p.u., Positive-Sequence Current, Magnitude"
      m_I1 = "m:I1", "kA, Positive-Sequence Current, Magnitude"
      m_phii1 = "m:phii1", "deg, Positive-Sequence Current, Angle"
      m_i2r = "m:i2r", "p.u., Negative-Sequence Current, Real Part"
      m_i2i = "m:i2i", "p.u., Negative-Sequence Current, Imaginary Part"
      m_i2 = "m:i2", "p.u., Negative-Sequence Current, Magnitude"
      m_I2 = "m:I2", "kA, Negative-Sequence Current, Magnitude"
      m_phii2 = "m:phii2", "deg, Negative-Sequence Current, Angle"
      m_du_A = "m:du:A", "%, Voltage Deviation"
      m_du_B = "m:du:B", "%, Voltage Deviation"
      m_du_C = "m:du:C", "%, Voltage Deviation"
      m_urpref_A = "m:urpref:A", "p.u., Pre-fault Voltage, Real Part"
      m_urpref_B = "m:urpref:B", "p.u., Pre-fault Voltage, Real Part"
      m_urpref_C = "m:urpref:C", "p.u., Pre-fault Voltage, Real Part"
      m_uipref_A = "m:uipref:A", "p.u., Pre-fault Voltage, Imaginary Part"
      m_uipref_B = "m:uipref:B", "p.u., Pre-fault Voltage, Imaginary Part"
      m_uipref_C = "m:uipref:C", "p.u., Pre-fault Voltage, Imaginary Part"
      m_upref_A = "m:upref:A", "p.u., Pre-fault Voltage, Magnitude"
      m_upref_B = "m:upref:B", "p.u., Pre-fault Voltage, Magnitude"
      m_upref_C = "m:upref:C", "p.u., Pre-fault Voltage, Magnitude"
      m_uprefpc_A = "m:uprefpc:A", "%, Pre-fault Voltage, Magnitude"
      m_uprefpc_B = "m:uprefpc:B", "%, Pre-fault Voltage, Magnitude"
      m_uprefpc_C = "m:uprefpc:C", "%, Pre-fault Voltage, Magnitude"
      m_Upref_A = "m:Upref:A", "kV, Line-Ground Pre-fault Voltage, Magnitude"
      m_Upref_B = "m:Upref:B", "kV, Line-Ground Pre-fault Voltage, Magnitude"
      m_Upref_C = "m:Upref:C", "kV, Line-Ground Pre-fault Voltage, Magnitude"
      m_Ulpref_A = "m:Ulpref:A", "kV, Line-Line Pre-fault Voltage, Magnitude"
      m_Ulpref_B = "m:Ulpref:B", "kV, Line-Line Pre-fault Voltage, Magnitude"
      m_Ulpref_C = "m:Ulpref:C", "kV, Line-Line Pre-fault Voltage, Magnitude"
      m_phiupref_A = "m:phiupref:A", "deg, Pre-fault Voltage, Angle"
      m_phiupref_B = "m:phiupref:B", "deg, Pre-fault Voltage, Angle"
      m_phiupref_C = "m:phiupref:C", "deg, Pre-fault Voltage, Angle"
      m_phiui_A = "m:phiui:A", "deg, Angle between Voltage and Current"
      m_phiui_B = "m:phiui:B", "deg, Angle between Voltage and Current"
      m_phiui_C = "m:phiui:C", "deg, Angle between Voltage and Current"
      m_phiu1i1 = "m:phiu1i1", "deg, Angle between Voltage and Current in positive sequence system"
      m_phiu2i2 = "m:phiu2i2", "deg, Angle between Voltage and Current in negative sequence system"
      m_phiu0i0 = "m:phiu0i0", "deg, Angle between Voltage and Current in zero-sequence system"

    class ElmTr2(Enum):
      _init_ = 'value __doc__'

      m_u0_bushv = "m:u0:bushv", "p.u., Zero-Sequence-Voltage, Magnitude"
      m_u0_buslv = "m:u0:buslv", "p.u., Zero-Sequence-Voltage, Magnitude"
      m_u0_bushvn = "m:u0:bushvn", "p.u., Zero-Sequence-Voltage, Magnitude"
      m_u0_buslvn = "m:u0:buslvn", "p.u., Zero-Sequence-Voltage, Magnitude"
      m_phiu0_bushv = "m:phiu0:bushv", "deg, Zero-Sequence-Voltage, Angle"
      m_phiu0_buslv = "m:phiu0:buslv", "deg, Zero-Sequence-Voltage, Angle"
      m_phiu0_bushvn = "m:phiu0:bushvn", "deg, Zero-Sequence-Voltage, Angle"
      m_phiu0_buslvn = "m:phiu0:buslvn", "deg, Zero-Sequence-Voltage, Angle"
      m_u1r_bushv = "m:u1r:bushv", "p.u., Positive-Sequence-Voltage, Real Part"
      m_u1r_buslv = "m:u1r:buslv", "p.u., Positive-Sequence-Voltage, Real Part"
      m_u1r_bushvn = "m:u1r:bushvn", "p.u., Positive-Sequence-Voltage, Real Part"
      m_u1r_buslvn = "m:u1r:buslvn", "p.u., Positive-Sequence-Voltage, Real Part"
      m_u1i_bushv = "m:u1i:bushv", "p.u., Positive-Sequence-Voltage, Imaginary Part"
      m_u1i_buslv = "m:u1i:buslv", "p.u., Positive-Sequence-Voltage, Imaginary Part"
      m_u1i_bushvn = "m:u1i:bushvn", "p.u., Positive-Sequence-Voltage, Imaginary Part"
      m_u1i_buslvn = "m:u1i:buslvn", "p.u., Positive-Sequence-Voltage, Imaginary Part"
      m_u1_bushv = "m:u1:bushv", "p.u., Positive-Sequence-Voltage, Magnitude"
      m_u1_buslv = "m:u1:buslv", "p.u., Positive-Sequence-Voltage, Magnitude"
      m_u1_bushvn = "m:u1:bushvn", "p.u., Positive-Sequence-Voltage, Magnitude"
      m_u1_buslvn = "m:u1:buslvn", "p.u., Positive-Sequence-Voltage, Magnitude"
      m_U1_bushv = "m:U1:bushv", "kV, Line-Ground Positive-Sequence-Voltage, Magnitude"
      m_U1_buslv = "m:U1:buslv", "kV, Line-Ground Positive-Sequence-Voltage, Magnitude"
      m_U1_bushvn = "m:U1:bushvn", "kV, Line-Ground Positive-Sequence-Voltage, Magnitude"
      m_U1_buslvn = "m:U1:buslvn", "kV, Line-Ground Positive-Sequence-Voltage, Magnitude"
      m_U1l_bushv = "m:U1l:bushv", "kV, Line-Line Positive-Sequence-Voltage, Magnitude"
      m_U1l_buslv = "m:U1l:buslv", "kV, Line-Line Positive-Sequence-Voltage, Magnitude"
      m_U1l_bushvn = "m:U1l:bushvn", "kV, Line-Line Positive-Sequence-Voltage, Magnitude"
      m_U1l_buslvn = "m:U1l:buslvn", "kV, Line-Line Positive-Sequence-Voltage, Magnitude"
      m_phiu1_bushv = "m:phiu1:bushv", "deg, Positive-Sequence-Voltage, Angle"
      m_phiu1_buslv = "m:phiu1:buslv", "deg, Positive-Sequence-Voltage, Angle"
      m_phiu1_bushvn = "m:phiu1:bushvn", "deg, Positive-Sequence-Voltage, Angle"
      m_phiu1_buslvn = "m:phiu1:buslvn", "deg, Positive-Sequence-Voltage, Angle"
      m_u2_bushv = "m:u2:bushv", "p.u., Negative-Sequence-Voltage, Magnitude"
      m_u2_buslv = "m:u2:buslv", "p.u., Negative-Sequence-Voltage, Magnitude"
      m_u2_bushvn = "m:u2:bushvn", "p.u., Negative-Sequence-Voltage, Magnitude"
      m_u2_buslvn = "m:u2:buslvn", "p.u., Negative-Sequence-Voltage, Magnitude"
      m_phiu2_bushv = "m:phiu2:bushv", "deg, Negative-Sequence-Voltage, Angle"
      m_phiu2_buslv = "m:phiu2:buslv", "deg, Negative-Sequence-Voltage, Angle"
      m_phiu2_bushvn = "m:phiu2:bushvn", "deg, Negative-Sequence-Voltage, Angle"
      m_phiu2_buslvn = "m:phiu2:buslvn", "deg, Negative-Sequence-Voltage, Angle"
      m_inet_bushv_A = "m:inet:bushv:A", "p.u., Phase Current, Magnitude, referred to network"
      m_inet_bushv_B = "m:inet:bushv:B", "p.u., Phase Current, Magnitude, referred to network"
      m_inet_bushv_C = "m:inet:bushv:C", "p.u., Phase Current, Magnitude, referred to network"
      m_inet_bushv_N = "m:inet:bushv:N", "p.u., Phase Current, Magnitude, referred to network"
      m_inet_buslv_A = "m:inet:buslv:A", "p.u., Phase Current, Magnitude, referred to network"
      m_inet_buslv_B = "m:inet:buslv:B", "p.u., Phase Current, Magnitude, referred to network"
      m_inet_buslv_C = "m:inet:buslv:C", "p.u., Phase Current, Magnitude, referred to network"
      m_inet_buslv_N = "m:inet:buslv:N", "p.u., Phase Current, Magnitude, referred to network"
      m_inet_bushvn = "m:inet:bushvn", "p.u., Phase Current, Magnitude, referred to network"
      m_inet_buslvn = "m:inet:buslvn", "p.u., Phase Current, Magnitude, referred to network"
      m_I_bushv_A = "m:I:bushv:A", "kA, Phase Current, Magnitude"
      m_I_bushv_B = "m:I:bushv:B", "kA, Phase Current, Magnitude"
      m_I_bushv_C = "m:I:bushv:C", "kA, Phase Current, Magnitude"
      m_I_bushv_N = "m:I:bushv:N", "kA, Phase Current, Magnitude"
      m_I_buslv_A = "m:I:buslv:A", "kA, Phase Current, Magnitude"
      m_I_buslv_B = "m:I:buslv:B", "kA, Phase Current, Magnitude"
      m_I_buslv_C = "m:I:buslv:C", "kA, Phase Current, Magnitude"
      m_I_buslv_N = "m:I:buslv:N", "kA, Phase Current, Magnitude"
      m_I_bushvn = "m:I:bushvn", "kA, Phase Current, Magnitude"
      m_I_buslvn = "m:I:buslvn", "kA, Phase Current, Magnitude"
      m_phii_bushv_A = "m:phii:bushv:A", "deg, Phase Current, Angle"
      m_phii_bushv_B = "m:phii:bushv:B", "deg, Phase Current, Angle"
      m_phii_bushv_C = "m:phii:bushv:C", "deg, Phase Current, Angle"
      m_phii_bushv_N = "m:phii:bushv:N", "deg, Phase Current, Angle"
      m_phii_buslv_A = "m:phii:buslv:A", "deg, Phase Current, Angle"
      m_phii_buslv_B = "m:phii:buslv:B", "deg, Phase Current, Angle"
      m_phii_buslv_C = "m:phii:buslv:C", "deg, Phase Current, Angle"
      m_phii_buslv_N = "m:phii:buslv:N", "deg, Phase Current, Angle"
      m_phii_bushvn = "m:phii:bushvn", "deg, Phase Current, Angle"
      m_phii_buslvn = "m:phii:buslvn", "deg, Phase Current, Angle"
      m_i0r_bushv = "m:i0r:bushv", "p.u., Zero-Sequence Current, Real Part"
      m_i0r_buslv = "m:i0r:buslv", "p.u., Zero-Sequence Current, Real Part"
      m_i0r_bushvn = "m:i0r:bushvn", "p.u., Zero-Sequence Current, Real Part"
      m_i0r_buslvn = "m:i0r:buslvn", "p.u., Zero-Sequence Current, Real Part"
      m_i0i_bushv = "m:i0i:bushv", "p.u., Zero-Sequence Current, Imaginary Part"
      m_i0i_buslv = "m:i0i:buslv", "p.u., Zero-Sequence Current, Imaginary Part"
      m_i0i_bushvn = "m:i0i:bushvn", "p.u., Zero-Sequence Current, Imaginary Part"
      m_i0i_buslvn = "m:i0i:buslvn", "p.u., Zero-Sequence Current, Imaginary Part"
      m_i0_bushv = "m:i0:bushv", "p.u., Zero-Sequence Current, Magnitude"
      m_i0_buslv = "m:i0:buslv", "p.u., Zero-Sequence Current, Magnitude"
      m_i0_bushvn = "m:i0:bushvn", "p.u., Zero-Sequence Current, Magnitude"
      m_i0_buslvn = "m:i0:buslvn", "p.u., Zero-Sequence Current, Magnitude"
      m_I0_bushv = "m:I0:bushv", "kA, Zero-Sequence Current, Magnitude"
      m_I0_buslv = "m:I0:buslv", "kA, Zero-Sequence Current, Magnitude"
      m_I0_bushvn = "m:I0:bushvn", "kA, Zero-Sequence Current, Magnitude"
      m_I0_buslvn = "m:I0:buslvn", "kA, Zero-Sequence Current, Magnitude"
      m_I0x3_bushv = "m:I0x3:bushv", "kA, 3*I0"
      m_I0x3_buslv = "m:I0x3:buslv", "kA, 3*I0"
      m_I0x3_bushvn = "m:I0x3:bushvn", "kA, 3*I0"
      m_I0x3_buslvn = "m:I0x3:buslvn", "kA, 3*I0"
      m_phii0_bushv = "m:phii0:bushv", "deg, Zero-Sequence Current, Angle"
      m_phii0_buslv = "m:phii0:buslv", "deg, Zero-Sequence Current, Angle"
      m_phii0_bushvn = "m:phii0:bushvn", "deg, Zero-Sequence Current, Angle"
      m_phii0_buslvn = "m:phii0:buslvn", "deg, Zero-Sequence Current, Angle"
      m_i1r_bushv = "m:i1r:bushv", "p.u., Positive-Sequence Current, Real Part"
      m_i1r_buslv = "m:i1r:buslv", "p.u., Positive-Sequence Current, Real Part"
      m_i1r_bushvn = "m:i1r:bushvn", "p.u., Positive-Sequence Current, Real Part"
      m_i1r_buslvn = "m:i1r:buslvn", "p.u., Positive-Sequence Current, Real Part"
      m_i1i_bushv = "m:i1i:bushv", "p.u., Positive-Sequence Current, Imaginary Part"
      m_i1i_buslv = "m:i1i:buslv", "p.u., Positive-Sequence Current, Imaginary Part"
      m_i1i_bushvn = "m:i1i:bushvn", "p.u., Positive-Sequence Current, Imaginary Part"
      m_i1i_buslvn = "m:i1i:buslvn", "p.u., Positive-Sequence Current, Imaginary Part"
      m_i1_bushv = "m:i1:bushv", "p.u., Positive-Sequence Current, Magnitude"
      m_i1_buslv = "m:i1:buslv", "p.u., Positive-Sequence Current, Magnitude"
      m_i1_bushvn = "m:i1:bushvn", "p.u., Positive-Sequence Current, Magnitude"
      m_i1_buslvn = "m:i1:buslvn", "p.u., Positive-Sequence Current, Magnitude"
      m_I1_bushv = "m:I1:bushv", "kA, Positive-Sequence Current, Magnitude"
      m_I1_buslv = "m:I1:buslv", "kA, Positive-Sequence Current, Magnitude"
      m_I1_bushvn = "m:I1:bushvn", "kA, Positive-Sequence Current, Magnitude"
      m_I1_buslvn = "m:I1:buslvn", "kA, Positive-Sequence Current, Magnitude"
      m_phii1_bushv = "m:phii1:bushv", "deg, Positive-Sequence Current, Angle"
      m_phii1_buslv = "m:phii1:buslv", "deg, Positive-Sequence Current, Angle"
      m_phii1_bushvn = "m:phii1:bushvn", "deg, Positive-Sequence Current, Angle"
      m_phii1_buslvn = "m:phii1:buslvn", "deg, Positive-Sequence Current, Angle"
      m_i2r_bushv = "m:i2r:bushv", "p.u., Negative-Sequence Current, Real Part"
      m_i2r_buslv = "m:i2r:buslv", "p.u., Negative-Sequence Current, Real Part"
      m_i2r_bushvn = "m:i2r:bushvn", "p.u., Negative-Sequence Current, Real Part"
      m_i2r_buslvn = "m:i2r:buslvn", "p.u., Negative-Sequence Current, Real Part"
      m_i2i_bushv = "m:i2i:bushv", "p.u., Negative-Sequence Current, Imaginary Part"
      m_i2i_buslv = "m:i2i:buslv", "p.u., Negative-Sequence Current, Imaginary Part"
      m_i2i_bushvn = "m:i2i:bushvn", "p.u., Negative-Sequence Current, Imaginary Part"
      m_i2i_buslvn = "m:i2i:buslvn", "p.u., Negative-Sequence Current, Imaginary Part"
      m_i2_bushv = "m:i2:bushv", "p.u., Negative-Sequence Current, Magnitude"
      m_i2_buslv = "m:i2:buslv", "p.u., Negative-Sequence Current, Magnitude"
      m_i2_bushvn = "m:i2:bushvn", "p.u., Negative-Sequence Current, Magnitude"
      m_i2_buslvn = "m:i2:buslvn", "p.u., Negative-Sequence Current, Magnitude"
      m_I2_bushv = "m:I2:bushv", "kA, Negative-Sequence Current, Magnitude"
      m_I2_buslv = "m:I2:buslv", "kA, Negative-Sequence Current, Magnitude"
      m_I2_bushvn = "m:I2:bushvn", "kA, Negative-Sequence Current, Magnitude"
      m_I2_buslvn = "m:I2:buslvn", "kA, Negative-Sequence Current, Magnitude"
      m_phii2_bushv = "m:phii2:bushv", "deg, Negative-Sequence Current, Angle"
      m_phii2_buslv = "m:phii2:buslv", "deg, Negative-Sequence Current, Angle"
      m_phii2_bushvn = "m:phii2:bushvn", "deg, Negative-Sequence Current, Angle"
      m_phii2_buslvn = "m:phii2:buslvn", "deg, Negative-Sequence Current, Angle"
      m_ubfacI_bushv = "m:ubfacI:bushv", "%, Current Unbalance Factor"
      m_ubfacI_buslv = "m:ubfacI:buslv", "%, Current Unbalance Factor"
      m_ubfacI_bushvn = "m:ubfacI:bushvn", "%, Current Unbalance Factor"
      m_ubfacI_buslvn = "m:ubfacI:buslvn", "%, Current Unbalance Factor"
      m_ubfacS_bushv = "m:ubfacS:bushv", "%, Power Unbalance Factor"
      m_ubfacS_buslv = "m:ubfacS:buslv", "%, Power Unbalance Factor"
      m_ubfacS_bushvn = "m:ubfacS:bushvn", "%, Power Unbalance Factor"
      m_ubfacS_buslvn = "m:ubfacS:buslvn", "%, Power Unbalance Factor"
      m_P_bushv_A = "m:P:bushv:A", "MW, Active Power"
      m_P_bushv_B = "m:P:bushv:B", "MW, Active Power"
      m_P_bushv_C = "m:P:bushv:C", "MW, Active Power"
      m_P_bushv_N = "m:P:bushv:N", "MW, Active Power"
      m_P_buslv_A = "m:P:buslv:A", "MW, Active Power"
      m_P_buslv_B = "m:P:buslv:B", "MW, Active Power"
      m_P_buslv_C = "m:P:buslv:C", "MW, Active Power"
      m_P_buslv_N = "m:P:buslv:N", "MW, Active Power"
      m_P_bushvn = "m:P:bushvn", "MW, Active Power"
      m_P_buslvn = "m:P:buslvn", "MW, Active Power"
      m_Q_bushv_A = "m:Q:bushv:A", "Mvar, Reactive Power"
      m_Q_bushv_B = "m:Q:bushv:B", "Mvar, Reactive Power"
      m_Q_bushv_C = "m:Q:bushv:C", "Mvar, Reactive Power"
      m_Q_bushv_N = "m:Q:bushv:N", "Mvar, Reactive Power"
      m_Q_buslv_A = "m:Q:buslv:A", "Mvar, Reactive Power"
      m_Q_buslv_B = "m:Q:buslv:B", "Mvar, Reactive Power"
      m_Q_buslv_C = "m:Q:buslv:C", "Mvar, Reactive Power"
      m_Q_buslv_N = "m:Q:buslv:N", "Mvar, Reactive Power"
      m_Q_bushvn = "m:Q:bushvn", "Mvar, Reactive Power"
      m_Q_buslvn = "m:Q:buslvn", "Mvar, Reactive Power"
      m_S_bushv_A = "m:S:bushv:A", "MVA, Apparent Power"
      m_S_bushv_B = "m:S:bushv:B", "MVA, Apparent Power"
      m_S_bushv_C = "m:S:bushv:C", "MVA, Apparent Power"
      m_S_bushv_N = "m:S:bushv:N", "MVA, Apparent Power"
      m_S_buslv_A = "m:S:buslv:A", "MVA, Apparent Power"
      m_S_buslv_B = "m:S:buslv:B", "MVA, Apparent Power"
      m_S_buslv_C = "m:S:buslv:C", "MVA, Apparent Power"
      m_S_buslv_N = "m:S:buslv:N", "MVA, Apparent Power"
      m_S_bushvn = "m:S:bushvn", "MVA, Apparent Power"
      m_S_buslvn = "m:S:buslvn", "MVA, Apparent Power"
      m_cosphi_bushv_A = "m:cosphi:bushv:A", "Power Factor"
      m_cosphi_bushv_B = "m:cosphi:bushv:B", "Power Factor"
      m_cosphi_bushv_C = "m:cosphi:bushv:C", "Power Factor"
      m_cosphi_bushv_N = "m:cosphi:bushv:N", "Power Factor"
      m_cosphi_buslv_A = "m:cosphi:buslv:A", "Power Factor"
      m_cosphi_buslv_B = "m:cosphi:buslv:B", "Power Factor"
      m_cosphi_buslv_C = "m:cosphi:buslv:C", "Power Factor"
      m_cosphi_buslv_N = "m:cosphi:buslv:N", "Power Factor"
      m_cosphi_bushvn = "m:cosphi:bushvn", "Power Factor"
      m_cosphi_buslvn = "m:cosphi:buslvn", "Power Factor"
      m_tanphi_bushv_A = "m:tanphi:bushv:A", "tan(phi)"
      m_tanphi_bushv_B = "m:tanphi:bushv:B", "tan(phi)"
      m_tanphi_bushv_C = "m:tanphi:bushv:C", "tan(phi)"
      m_tanphi_bushv_N = "m:tanphi:bushv:N", "tan(phi)"
      m_tanphi_buslv_A = "m:tanphi:buslv:A", "tan(phi)"
      m_tanphi_buslv_B = "m:tanphi:buslv:B", "tan(phi)"
      m_tanphi_buslv_C = "m:tanphi:buslv:C", "tan(phi)"
      m_tanphi_buslv_N = "m:tanphi:buslv:N", "tan(phi)"
      m_tanphi_bushvn = "m:tanphi:bushvn", "tan(phi)"
      m_tanphi_buslvn = "m:tanphi:buslvn", "tan(phi)"
      m_Psum_bushv = "m:Psum:bushv", "MW, Total Active Power"
      m_Psum_buslv = "m:Psum:buslv", "MW, Total Active Power"
      m_Psum_bushvn = "m:Psum:bushvn", "MW, Total Active Power"
      m_Psum_buslvn = "m:Psum:buslvn", "MW, Total Active Power"
      m_Qsum_bushv = "m:Qsum:bushv", "Mvar, Total Reactive Power"
      m_Qsum_buslv = "m:Qsum:buslv", "Mvar, Total Reactive Power"
      m_Qsum_bushvn = "m:Qsum:bushvn", "Mvar, Total Reactive Power"
      m_Qsum_buslvn = "m:Qsum:buslvn", "Mvar, Total Reactive Power"
      m_Ssum_bushv = "m:Ssum:bushv", "MVA, Total Apparent Power"
      m_Ssum_buslv = "m:Ssum:buslv", "MVA, Total Apparent Power"
      m_Ssum_bushvn = "m:Ssum:bushvn", "MVA, Total Apparent Power"
      m_Ssum_buslvn = "m:Ssum:buslvn", "MVA, Total Apparent Power"
      m_cosphisum_bushv = "m:cosphisum:bushv", "Total Power Factor"
      m_cosphisum_buslv = "m:cosphisum:buslv", "Total Power Factor"
      m_cosphisum_bushvn = "m:cosphisum:bushvn", "Total Power Factor"
      m_cosphisum_buslvn = "m:cosphisum:buslvn", "Total Power Factor"
      m_tanphisum_bushv = "m:tanphisum:bushv", "Total tan(phi)"
      m_tanphisum_buslv = "m:tanphisum:buslv", "Total tan(phi)"
      m_tanphisum_bushvn = "m:tanphisum:bushvn", "Total tan(phi)"
      m_tanphisum_buslvn = "m:tanphisum:buslvn", "Total tan(phi)"
      m_Pln_bushv_A = "m:Pln:bushv:A", "MW, Active Power (Line-Neutral)"
      m_Pln_bushv_B = "m:Pln:bushv:B", "MW, Active Power (Line-Neutral)"
      m_Pln_bushv_C = "m:Pln:bushv:C", "MW, Active Power (Line-Neutral)"
      m_Pln_bushv_N = "m:Pln:bushv:N", "MW, Active Power (Line-Neutral)"
      m_Pln_buslv_A = "m:Pln:buslv:A", "MW, Active Power (Line-Neutral)"
      m_Pln_buslv_B = "m:Pln:buslv:B", "MW, Active Power (Line-Neutral)"
      m_Pln_buslv_C = "m:Pln:buslv:C", "MW, Active Power (Line-Neutral)"
      m_Pln_buslv_N = "m:Pln:buslv:N", "MW, Active Power (Line-Neutral)"
      m_Pln_bushvn = "m:Pln:bushvn", "MW, Active Power (Line-Neutral)"
      m_Pln_buslvn = "m:Pln:buslvn", "MW, Active Power (Line-Neutral)"
      m_Qln_bushv_A = "m:Qln:bushv:A", "Mvar, Reactive Power (Line-Neutral)"
      m_Qln_bushv_B = "m:Qln:bushv:B", "Mvar, Reactive Power (Line-Neutral)"
      m_Qln_bushv_C = "m:Qln:bushv:C", "Mvar, Reactive Power (Line-Neutral)"
      m_Qln_bushv_N = "m:Qln:bushv:N", "Mvar, Reactive Power (Line-Neutral)"
      m_Qln_buslv_A = "m:Qln:buslv:A", "Mvar, Reactive Power (Line-Neutral)"
      m_Qln_buslv_B = "m:Qln:buslv:B", "Mvar, Reactive Power (Line-Neutral)"
      m_Qln_buslv_C = "m:Qln:buslv:C", "Mvar, Reactive Power (Line-Neutral)"
      m_Qln_buslv_N = "m:Qln:buslv:N", "Mvar, Reactive Power (Line-Neutral)"
      m_Qln_bushvn = "m:Qln:bushvn", "Mvar, Reactive Power (Line-Neutral)"
      m_Qln_buslvn = "m:Qln:buslvn", "Mvar, Reactive Power (Line-Neutral)"
      m_P1_bushv = "m:P1:bushv", "MW, Positive Sequence Active Power"
      m_P1_buslv = "m:P1:buslv", "MW, Positive Sequence Active Power"
      m_P1_bushvn = "m:P1:bushvn", "MW, Positive Sequence Active Power"
      m_P1_buslvn = "m:P1:buslvn", "MW, Positive Sequence Active Power"
      m_Q1_bushv = "m:Q1:bushv", "Mvar, Positive Sequence Reactive Power"
      m_Q1_buslv = "m:Q1:buslv", "Mvar, Positive Sequence Reactive Power"
      m_Q1_bushvn = "m:Q1:bushvn", "Mvar, Positive Sequence Reactive Power"
      m_Q1_buslvn = "m:Q1:buslvn", "Mvar, Positive Sequence Reactive Power"
      m_P2_bushv = "m:P2:bushv", "MW, Negative Sequence Active Power"
      m_P2_buslv = "m:P2:buslv", "MW, Negative Sequence Active Power"
      m_P2_bushvn = "m:P2:bushvn", "MW, Negative Sequence Active Power"
      m_P2_buslvn = "m:P2:buslvn", "MW, Negative Sequence Active Power"
      m_Q2_bushv = "m:Q2:bushv", "Mvar, Negative Sequence Reactive Power"
      m_Q2_buslv = "m:Q2:buslv", "Mvar, Negative Sequence Reactive Power"
      m_Q2_bushvn = "m:Q2:bushvn", "Mvar, Negative Sequence Reactive Power"
      m_Q2_buslvn = "m:Q2:buslvn", "Mvar, Negative Sequence Reactive Power"
      m_P0_bushv = "m:P0:bushv", "MW, Zero Sequence Active Power"
      m_P0_buslv = "m:P0:buslv", "MW, Zero Sequence Active Power"
      m_P0_bushvn = "m:P0:bushvn", "MW, Zero Sequence Active Power"
      m_P0_buslvn = "m:P0:buslvn", "MW, Zero Sequence Active Power"
      m_Q0_bushv = "m:Q0:bushv", "Mvar, Zero Sequence Reactive Power"
      m_Q0_buslv = "m:Q0:buslv", "Mvar, Zero Sequence Reactive Power"
      m_Q0_bushvn = "m:Q0:bushvn", "Mvar, Zero Sequence Reactive Power"
      m_Q0_buslvn = "m:Q0:buslvn", "Mvar, Zero Sequence Reactive Power"
      m_i1P_bushv = "m:i1P:bushv", "p.u., Positive-Sequence Active Current"
      m_i1P_buslv = "m:i1P:buslv", "p.u., Positive-Sequence Active Current"
      m_i1P_bushvn = "m:i1P:bushvn", "p.u., Positive-Sequence Active Current"
      m_i1P_buslvn = "m:i1P:buslvn", "p.u., Positive-Sequence Active Current"
      m_i1Q_bushv = "m:i1Q:bushv", "p.u., Positive-Sequence Reactive Current"
      m_i1Q_buslv = "m:i1Q:buslv", "p.u., Positive-Sequence Reactive Current"
      m_i1Q_bushvn = "m:i1Q:bushvn", "p.u., Positive-Sequence Reactive Current"
      m_i1Q_buslvn = "m:i1Q:buslvn", "p.u., Positive-Sequence Reactive Current"
      m_I1P_bushv = "m:I1P:bushv", "kA, Positive-Sequence Active Current"
      m_I1P_buslv = "m:I1P:buslv", "kA, Positive-Sequence Active Current"
      m_I1P_bushvn = "m:I1P:bushvn", "kA, Positive-Sequence Active Current"
      m_I1P_buslvn = "m:I1P:buslvn", "kA, Positive-Sequence Active Current"
      m_I1Q_bushv = "m:I1Q:bushv", "kA, Positive-Sequence Reactive Current"
      m_I1Q_buslv = "m:I1Q:buslv", "kA, Positive-Sequence Reactive Current"
      m_I1Q_bushvn = "m:I1Q:bushvn", "kA, Positive-Sequence Reactive Current"
      m_I1Q_buslvn = "m:I1Q:buslvn", "kA, Positive-Sequence Reactive Current"
      m_i2P_bushv = "m:i2P:bushv", "p.u., Negative-Sequence Active Current"
      m_i2P_buslv = "m:i2P:buslv", "p.u., Negative-Sequence Active Current"
      m_i2P_bushvn = "m:i2P:bushvn", "p.u., Negative-Sequence Active Current"
      m_i2P_buslvn = "m:i2P:buslvn", "p.u., Negative-Sequence Active Current"
      m_i2Q_bushv = "m:i2Q:bushv", "p.u., Negative-Sequence Reactive Current"
      m_i2Q_buslv = "m:i2Q:buslv", "p.u., Negative-Sequence Reactive Current"
      m_i2Q_bushvn = "m:i2Q:bushvn", "p.u., Negative-Sequence Reactive Current"
      m_i2Q_buslvn = "m:i2Q:buslvn", "p.u., Negative-Sequence Reactive Current"
      m_I2P_bushv = "m:I2P:bushv", "kA, Negative-Sequence Active Current"
      m_I2P_buslv = "m:I2P:buslv", "kA, Negative-Sequence Active Current"
      m_I2P_bushvn = "m:I2P:bushvn", "kA, Negative-Sequence Active Current"
      m_I2P_buslvn = "m:I2P:buslvn", "kA, Negative-Sequence Active Current"
      m_I2Q_bushv = "m:I2Q:bushv", "kA, Negative-Sequence Reactive Current"
      m_I2Q_buslv = "m:I2Q:buslv", "kA, Negative-Sequence Reactive Current"
      m_I2Q_bushvn = "m:I2Q:bushvn", "kA, Negative-Sequence Reactive Current"
      m_I2Q_buslvn = "m:I2Q:buslvn", "kA, Negative-Sequence Reactive Current"
      m_ur_bushv_A = "m:ur:bushv:A", "p.u., Phase Voltage, Real Part"
      m_ur_bushv_B = "m:ur:bushv:B", "p.u., Phase Voltage, Real Part"
      m_ur_bushv_C = "m:ur:bushv:C", "p.u., Phase Voltage, Real Part"
      m_ur_bushv_N = "m:ur:bushv:N", "p.u., Phase Voltage, Real Part"
      m_ur_buslv_A = "m:ur:buslv:A", "p.u., Phase Voltage, Real Part"
      m_ur_buslv_B = "m:ur:buslv:B", "p.u., Phase Voltage, Real Part"
      m_ur_buslv_C = "m:ur:buslv:C", "p.u., Phase Voltage, Real Part"
      m_ur_buslv_N = "m:ur:buslv:N", "p.u., Phase Voltage, Real Part"
      m_ur_bushvn = "m:ur:bushvn", "p.u., Phase Voltage, Real Part"
      m_ur_buslvn = "m:ur:buslvn", "p.u., Phase Voltage, Real Part"
      m_ui_bushv_A = "m:ui:bushv:A", "p.u., Phase Voltage, Imaginary Part"
      m_ui_bushv_B = "m:ui:bushv:B", "p.u., Phase Voltage, Imaginary Part"
      m_ui_bushv_C = "m:ui:bushv:C", "p.u., Phase Voltage, Imaginary Part"
      m_ui_bushv_N = "m:ui:bushv:N", "p.u., Phase Voltage, Imaginary Part"
      m_ui_buslv_A = "m:ui:buslv:A", "p.u., Phase Voltage, Imaginary Part"
      m_ui_buslv_B = "m:ui:buslv:B", "p.u., Phase Voltage, Imaginary Part"
      m_ui_buslv_C = "m:ui:buslv:C", "p.u., Phase Voltage, Imaginary Part"
      m_ui_buslv_N = "m:ui:buslv:N", "p.u., Phase Voltage, Imaginary Part"
      m_ui_bushvn = "m:ui:bushvn", "p.u., Phase Voltage, Imaginary Part"
      m_ui_buslvn = "m:ui:buslvn", "p.u., Phase Voltage, Imaginary Part"
      m_u_bushv_A = "m:u:bushv:A", "p.u., Phase Voltage, Magnitude"
      m_u_bushv_B = "m:u:bushv:B", "p.u., Phase Voltage, Magnitude"
      m_u_bushv_C = "m:u:bushv:C", "p.u., Phase Voltage, Magnitude"
      m_u_bushv_N = "m:u:bushv:N", "p.u., Phase Voltage, Magnitude"
      m_u_buslv_A = "m:u:buslv:A", "p.u., Phase Voltage, Magnitude"
      m_u_buslv_B = "m:u:buslv:B", "p.u., Phase Voltage, Magnitude"
      m_u_buslv_C = "m:u:buslv:C", "p.u., Phase Voltage, Magnitude"
      m_u_buslv_N = "m:u:buslv:N", "p.u., Phase Voltage, Magnitude"
      m_u_bushvn = "m:u:bushvn", "p.u., Phase Voltage, Magnitude"
      m_u_buslvn = "m:u:buslvn", "p.u., Phase Voltage, Magnitude"
      m_ir_bushv_A = "m:ir:bushv:A", "p.u., Phase Current, Real Part"
      m_ir_bushv_B = "m:ir:bushv:B", "p.u., Phase Current, Real Part"
      m_ir_bushv_C = "m:ir:bushv:C", "p.u., Phase Current, Real Part"
      m_ir_bushv_N = "m:ir:bushv:N", "p.u., Phase Current, Real Part"
      m_ir_buslv_A = "m:ir:buslv:A", "p.u., Phase Current, Real Part"
      m_ir_buslv_B = "m:ir:buslv:B", "p.u., Phase Current, Real Part"
      m_ir_buslv_C = "m:ir:buslv:C", "p.u., Phase Current, Real Part"
      m_ir_buslv_N = "m:ir:buslv:N", "p.u., Phase Current, Real Part"
      m_ir_bushvn = "m:ir:bushvn", "p.u., Phase Current, Real Part"
      m_ir_buslvn = "m:ir:buslvn", "p.u., Phase Current, Real Part"
      m_ii_bushv_A = "m:ii:bushv:A", "p.u., Phase Current, Imaginary Part"
      m_ii_bushv_B = "m:ii:bushv:B", "p.u., Phase Current, Imaginary Part"
      m_ii_bushv_C = "m:ii:bushv:C", "p.u., Phase Current, Imaginary Part"
      m_ii_bushv_N = "m:ii:bushv:N", "p.u., Phase Current, Imaginary Part"
      m_ii_buslv_A = "m:ii:buslv:A", "p.u., Phase Current, Imaginary Part"
      m_ii_buslv_B = "m:ii:buslv:B", "p.u., Phase Current, Imaginary Part"
      m_ii_buslv_C = "m:ii:buslv:C", "p.u., Phase Current, Imaginary Part"
      m_ii_buslv_N = "m:ii:buslv:N", "p.u., Phase Current, Imaginary Part"
      m_ii_bushvn = "m:ii:bushvn", "p.u., Phase Current, Imaginary Part"
      m_ii_buslvn = "m:ii:buslvn", "p.u., Phase Current, Imaginary Part"
      m_i_bushv_A = "m:i:bushv:A", "p.u., Phase Current, Magnitude"
      m_i_bushv_B = "m:i:bushv:B", "p.u., Phase Current, Magnitude"
      m_i_bushv_C = "m:i:bushv:C", "p.u., Phase Current, Magnitude"
      m_i_bushv_N = "m:i:bushv:N", "p.u., Phase Current, Magnitude"
      m_i_buslv_A = "m:i:buslv:A", "p.u., Phase Current, Magnitude"
      m_i_buslv_B = "m:i:buslv:B", "p.u., Phase Current, Magnitude"
      m_i_buslv_C = "m:i:buslv:C", "p.u., Phase Current, Magnitude"
      m_i_buslv_N = "m:i:buslv:N", "p.u., Phase Current, Magnitude"
      m_i_bushvn = "m:i:bushvn", "p.u., Phase Current, Magnitude"
      m_i_buslvn = "m:i:buslvn", "p.u., Phase Current, Magnitude"
      m_phiui_bushv_A = "m:phiui:bushv:A", "deg, Angle between Voltage and Current"
      m_phiui_bushv_B = "m:phiui:bushv:B", "deg, Angle between Voltage and Current"
      m_phiui_bushv_C = "m:phiui:bushv:C", "deg, Angle between Voltage and Current"
      m_phiui_bushv_N = "m:phiui:bushv:N", "deg, Angle between Voltage and Current"
      m_phiui_buslv_A = "m:phiui:buslv:A", "deg, Angle between Voltage and Current"
      m_phiui_buslv_B = "m:phiui:buslv:B", "deg, Angle between Voltage and Current"
      m_phiui_buslv_C = "m:phiui:buslv:C", "deg, Angle between Voltage and Current"
      m_phiui_buslv_N = "m:phiui:buslv:N", "deg, Angle between Voltage and Current"
      m_phiui_bushvn = "m:phiui:bushvn", "deg, Angle between Voltage and Current"
      m_phiui_buslvn = "m:phiui:buslvn", "deg, Angle between Voltage and Current"
      m_phiu0i0_bushv = "m:phiu0i0:bushv", "deg, Angle between Voltage and Current in zero-sequence system"
      m_phiu0i0_buslv = "m:phiu0i0:buslv", "deg, Angle between Voltage and Current in zero-sequence system"
      m_phiu0i0_bushvn = "m:phiu0i0:bushvn", "deg, Angle between Voltage and Current in zero-sequence system"
      m_phiu0i0_buslvn = "m:phiu0i0:buslvn", "deg, Angle between Voltage and Current in zero-sequence system"
      m_phiu1i1_bushv = "m:phiu1i1:bushv", "deg, Angle between Voltage and Current in positive sequence system"
      m_phiu1i1_buslv = "m:phiu1i1:buslv", "deg, Angle between Voltage and Current in positive sequence system"
      m_phiu1i1_bushvn = "m:phiu1i1:bushvn", "deg, Angle between Voltage and Current in positive sequence system"
      m_phiu1i1_buslvn = "m:phiu1i1:buslvn", "deg, Angle between Voltage and Current in positive sequence system"
      m_phiu2i2_bushv = "m:phiu2i2:bushv", "deg, Angle between Voltage and Current in negative sequence system"
      m_phiu2i2_buslv = "m:phiu2i2:buslv", "deg, Angle between Voltage and Current in negative sequence system"
      m_phiu2i2_bushvn = "m:phiu2i2:bushvn", "deg, Angle between Voltage and Current in negative sequence system"
      m_phiu2i2_buslvn = "m:phiu2i2:buslvn", "deg, Angle between Voltage and Current in negative sequence system"
      m_Tfct_bushv = "m:Tfct:bushv", "s, Fault Clearing Time"
      m_Tfct_buslv = "m:Tfct:buslv", "s, Fault Clearing Time"
      m_Tfct_bushvn = "m:Tfct:bushvn", "s, Fault Clearing Time"
      m_Tfct_buslvn = "m:Tfct:buslvn", "s, Fault Clearing Time"
      m_TfctPh_bushv_A = "m:TfctPh:bushv:A", "s, Fault Clearing Time"
      m_TfctPh_bushv_B = "m:TfctPh:bushv:B", "s, Fault Clearing Time"
      m_TfctPh_bushv_C = "m:TfctPh:bushv:C", "s, Fault Clearing Time"
      m_TfctPh_bushv_N = "m:TfctPh:bushv:N", "s, Fault Clearing Time"
      m_TfctPh_buslv_A = "m:TfctPh:buslv:A", "s, Fault Clearing Time"
      m_TfctPh_buslv_B = "m:TfctPh:buslv:B", "s, Fault Clearing Time"
      m_TfctPh_buslv_C = "m:TfctPh:buslv:C", "s, Fault Clearing Time"
      m_TfctPh_buslv_N = "m:TfctPh:buslv:N", "s, Fault Clearing Time"
      m_TfctPh_bushvn = "m:TfctPh:bushvn", "s, Fault Clearing Time"
      m_TfctPh_buslvn = "m:TfctPh:buslvn", "s, Fault Clearing Time"
      m_Brkload_bushv = "m:Brkload:bushv", "%, Max. Breaker Loading"
      m_Brkload_buslv = "m:Brkload:buslv", "%, Max. Breaker Loading"
      m_Brkload_bushvn = "m:Brkload:bushvn", "%, Max. Breaker Loading"
      m_Brkload_buslvn = "m:Brkload:buslvn", "%, Max. Breaker Loading"
      m_BrkloadPh_bushv_A = "m:BrkloadPh:bushv:A", "%, Breaker Loading"
      m_BrkloadPh_bushv_B = "m:BrkloadPh:bushv:B", "%, Breaker Loading"
      m_BrkloadPh_bushv_C = "m:BrkloadPh:bushv:C", "%, Breaker Loading"
      m_BrkloadPh_bushv_N = "m:BrkloadPh:bushv:N", "%, Breaker Loading"
      m_BrkloadPh_buslv_A = "m:BrkloadPh:buslv:A", "%, Breaker Loading"
      m_BrkloadPh_buslv_B = "m:BrkloadPh:buslv:B", "%, Breaker Loading"
      m_BrkloadPh_buslv_C = "m:BrkloadPh:buslv:C", "%, Breaker Loading"
      m_BrkloadPh_buslv_N = "m:BrkloadPh:buslv:N", "%, Breaker Loading"
      m_BrkloadPh_bushvn = "m:BrkloadPh:bushvn", "%, Breaker Loading"
      m_BrkloadPh_buslvn = "m:BrkloadPh:buslvn", "%, Breaker Loading"
      n_ur_bushv_A = "n:ur:bushv:A", "p.u., Line-Ground Voltage, Real Part"
      n_ur_bushv_B = "n:ur:bushv:B", "p.u., Line-Ground Voltage, Real Part"
      n_ur_bushv_C = "n:ur:bushv:C", "p.u., Line-Ground Voltage, Real Part"
      n_ur_bushv_N = "n:ur:bushv:N", "p.u., Line-Ground Voltage, Real Part"
      n_ur_buslv_A = "n:ur:buslv:A", "p.u., Line-Ground Voltage, Real Part"
      n_ur_buslv_B = "n:ur:buslv:B", "p.u., Line-Ground Voltage, Real Part"
      n_ur_buslv_C = "n:ur:buslv:C", "p.u., Line-Ground Voltage, Real Part"
      n_ur_buslv_N = "n:ur:buslv:N", "p.u., Line-Ground Voltage, Real Part"
      n_ur_bushvn = "n:ur:bushvn", "p.u., Line-Ground Voltage, Real Part"
      n_ur_buslvn = "n:ur:buslvn", "p.u., Line-Ground Voltage, Real Part"
      n_ui_bushv_A = "n:ui:bushv:A", "p.u., Line-Ground Voltage, Imaginary Part"
      n_ui_bushv_B = "n:ui:bushv:B", "p.u., Line-Ground Voltage, Imaginary Part"
      n_ui_bushv_C = "n:ui:bushv:C", "p.u., Line-Ground Voltage, Imaginary Part"
      n_ui_bushv_N = "n:ui:bushv:N", "p.u., Line-Ground Voltage, Imaginary Part"
      n_ui_buslv_A = "n:ui:buslv:A", "p.u., Line-Ground Voltage, Imaginary Part"
      n_ui_buslv_B = "n:ui:buslv:B", "p.u., Line-Ground Voltage, Imaginary Part"
      n_ui_buslv_C = "n:ui:buslv:C", "p.u., Line-Ground Voltage, Imaginary Part"
      n_ui_buslv_N = "n:ui:buslv:N", "p.u., Line-Ground Voltage, Imaginary Part"
      n_ui_bushvn = "n:ui:bushvn", "p.u., Line-Ground Voltage, Imaginary Part"
      n_ui_buslvn = "n:ui:buslvn", "p.u., Line-Ground Voltage, Imaginary Part"
      n_u_bushv_A = "n:u:bushv:A", "p.u., Line-Ground Voltage, Magnitude"
      n_u_bushv_B = "n:u:bushv:B", "p.u., Line-Ground Voltage, Magnitude"
      n_u_bushv_C = "n:u:bushv:C", "p.u., Line-Ground Voltage, Magnitude"
      n_u_bushv_N = "n:u:bushv:N", "p.u., Line-Ground Voltage, Magnitude"
      n_u_buslv_A = "n:u:buslv:A", "p.u., Line-Ground Voltage, Magnitude"
      n_u_buslv_B = "n:u:buslv:B", "p.u., Line-Ground Voltage, Magnitude"
      n_u_buslv_C = "n:u:buslv:C", "p.u., Line-Ground Voltage, Magnitude"
      n_u_buslv_N = "n:u:buslv:N", "p.u., Line-Ground Voltage, Magnitude"
      n_u_bushvn = "n:u:bushvn", "p.u., Line-Ground Voltage, Magnitude"
      n_u_buslvn = "n:u:buslvn", "p.u., Line-Ground Voltage, Magnitude"
      n_upc_bushv_A = "n:upc:bushv:A", "%, Line-Ground Voltage, Magnitude"
      n_upc_bushv_B = "n:upc:bushv:B", "%, Line-Ground Voltage, Magnitude"
      n_upc_bushv_C = "n:upc:bushv:C", "%, Line-Ground Voltage, Magnitude"
      n_upc_bushv_N = "n:upc:bushv:N", "%, Line-Ground Voltage, Magnitude"
      n_upc_buslv_A = "n:upc:buslv:A", "%, Line-Ground Voltage, Magnitude"
      n_upc_buslv_B = "n:upc:buslv:B", "%, Line-Ground Voltage, Magnitude"
      n_upc_buslv_C = "n:upc:buslv:C", "%, Line-Ground Voltage, Magnitude"
      n_upc_buslv_N = "n:upc:buslv:N", "%, Line-Ground Voltage, Magnitude"
      n_upc_bushvn = "n:upc:bushvn", "%, Line-Ground Voltage, Magnitude"
      n_upc_buslvn = "n:upc:buslvn", "%, Line-Ground Voltage, Magnitude"
      n_U_bushv_A = "n:U:bushv:A", "kV, Line-Ground Voltage, Magnitude"
      n_U_bushv_B = "n:U:bushv:B", "kV, Line-Ground Voltage, Magnitude"
      n_U_bushv_C = "n:U:bushv:C", "kV, Line-Ground Voltage, Magnitude"
      n_U_bushv_N = "n:U:bushv:N", "kV, Line-Ground Voltage, Magnitude"
      n_U_buslv_A = "n:U:buslv:A", "kV, Line-Ground Voltage, Magnitude"
      n_U_buslv_B = "n:U:buslv:B", "kV, Line-Ground Voltage, Magnitude"
      n_U_buslv_C = "n:U:buslv:C", "kV, Line-Ground Voltage, Magnitude"
      n_U_buslv_N = "n:U:buslv:N", "kV, Line-Ground Voltage, Magnitude"
      n_U_bushvn = "n:U:bushvn", "kV, Line-Ground Voltage, Magnitude"
      n_U_buslvn = "n:U:buslvn", "kV, Line-Ground Voltage, Magnitude"
      n_phiu_bushv_A = "n:phiu:bushv:A", "deg, Line-Ground Voltage, Angle"
      n_phiu_bushv_B = "n:phiu:bushv:B", "deg, Line-Ground Voltage, Angle"
      n_phiu_bushv_C = "n:phiu:bushv:C", "deg, Line-Ground Voltage, Angle"
      n_phiu_bushv_N = "n:phiu:bushv:N", "deg, Line-Ground Voltage, Angle"
      n_phiu_buslv_A = "n:phiu:buslv:A", "deg, Line-Ground Voltage, Angle"
      n_phiu_buslv_B = "n:phiu:buslv:B", "deg, Line-Ground Voltage, Angle"
      n_phiu_buslv_C = "n:phiu:buslv:C", "deg, Line-Ground Voltage, Angle"
      n_phiu_buslv_N = "n:phiu:buslv:N", "deg, Line-Ground Voltage, Angle"
      n_phiu_bushvn = "n:phiu:bushvn", "deg, Line-Ground Voltage, Angle"
      n_phiu_buslvn = "n:phiu:buslvn", "deg, Line-Ground Voltage, Angle"
      n_phiurel_bushv = "n:phiurel:bushv", "deg, Voltage, Relative Angle"
      n_phiurel_buslv = "n:phiurel:buslv", "deg, Voltage, Relative Angle"
      n_phiurel_bushvn = "n:phiurel:bushvn", "deg, Voltage, Relative Angle"
      n_phiurel_buslvn = "n:phiurel:buslvn", "deg, Voltage, Relative Angle"
      n_ul_bushv_A = "n:ul:bushv:A", "p.u., Line-Line Voltage, Magnitude"
      n_ul_bushv_B = "n:ul:bushv:B", "p.u., Line-Line Voltage, Magnitude"
      n_ul_bushv_C = "n:ul:bushv:C", "p.u., Line-Line Voltage, Magnitude"
      n_ul_bushv_N = "n:ul:bushv:N", "p.u., Line-Line Voltage, Magnitude"
      n_ul_buslv_A = "n:ul:buslv:A", "p.u., Line-Line Voltage, Magnitude"
      n_ul_buslv_B = "n:ul:buslv:B", "p.u., Line-Line Voltage, Magnitude"
      n_ul_buslv_C = "n:ul:buslv:C", "p.u., Line-Line Voltage, Magnitude"
      n_ul_buslv_N = "n:ul:buslv:N", "p.u., Line-Line Voltage, Magnitude"
      n_ul_bushvn = "n:ul:bushvn", "p.u., Line-Line Voltage, Magnitude"
      n_ul_buslvn = "n:ul:buslvn", "p.u., Line-Line Voltage, Magnitude"
      n_ulpc_bushv_A = "n:ulpc:bushv:A", "%, Line-Line Voltage, Magnitude"
      n_ulpc_bushv_B = "n:ulpc:bushv:B", "%, Line-Line Voltage, Magnitude"
      n_ulpc_bushv_C = "n:ulpc:bushv:C", "%, Line-Line Voltage, Magnitude"
      n_ulpc_bushv_N = "n:ulpc:bushv:N", "%, Line-Line Voltage, Magnitude"
      n_ulpc_buslv_A = "n:ulpc:buslv:A", "%, Line-Line Voltage, Magnitude"
      n_ulpc_buslv_B = "n:ulpc:buslv:B", "%, Line-Line Voltage, Magnitude"
      n_ulpc_buslv_C = "n:ulpc:buslv:C", "%, Line-Line Voltage, Magnitude"
      n_ulpc_buslv_N = "n:ulpc:buslv:N", "%, Line-Line Voltage, Magnitude"
      n_ulpc_bushvn = "n:ulpc:bushvn", "%, Line-Line Voltage, Magnitude"
      n_ulpc_buslvn = "n:ulpc:buslvn", "%, Line-Line Voltage, Magnitude"
      n_Ul_bushv_A = "n:Ul:bushv:A", "kV, Line-Line Voltage, Magnitude"
      n_Ul_bushv_B = "n:Ul:bushv:B", "kV, Line-Line Voltage, Magnitude"
      n_Ul_bushv_C = "n:Ul:bushv:C", "kV, Line-Line Voltage, Magnitude"
      n_Ul_bushv_N = "n:Ul:bushv:N", "kV, Line-Line Voltage, Magnitude"
      n_Ul_buslv_A = "n:Ul:buslv:A", "kV, Line-Line Voltage, Magnitude"
      n_Ul_buslv_B = "n:Ul:buslv:B", "kV, Line-Line Voltage, Magnitude"
      n_Ul_buslv_C = "n:Ul:buslv:C", "kV, Line-Line Voltage, Magnitude"
      n_Ul_buslv_N = "n:Ul:buslv:N", "kV, Line-Line Voltage, Magnitude"
      n_Ul_bushvn = "n:Ul:bushvn", "kV, Line-Line Voltage, Magnitude"
      n_Ul_buslvn = "n:Ul:buslvn", "kV, Line-Line Voltage, Magnitude"
      n_phiul_bushv_A = "n:phiul:bushv:A", "deg, Line-Line Voltage, Angle"
      n_phiul_bushv_B = "n:phiul:bushv:B", "deg, Line-Line Voltage, Angle"
      n_phiul_bushv_C = "n:phiul:bushv:C", "deg, Line-Line Voltage, Angle"
      n_phiul_bushv_N = "n:phiul:bushv:N", "deg, Line-Line Voltage, Angle"
      n_phiul_buslv_A = "n:phiul:buslv:A", "deg, Line-Line Voltage, Angle"
      n_phiul_buslv_B = "n:phiul:buslv:B", "deg, Line-Line Voltage, Angle"
      n_phiul_buslv_C = "n:phiul:buslv:C", "deg, Line-Line Voltage, Angle"
      n_phiul_buslv_N = "n:phiul:buslv:N", "deg, Line-Line Voltage, Angle"
      n_phiul_bushvn = "n:phiul:bushvn", "deg, Line-Line Voltage, Angle"
      n_phiul_buslvn = "n:phiul:buslvn", "deg, Line-Line Voltage, Angle"
      n_u0_bushv = "n:u0:bushv", "p.u., Zero-Sequence Voltage, Magnitude"
      n_u0_buslv = "n:u0:buslv", "p.u., Zero-Sequence Voltage, Magnitude"
      n_u0_bushvn = "n:u0:bushvn", "p.u., Zero-Sequence Voltage, Magnitude"
      n_u0_buslvn = "n:u0:buslvn", "p.u., Zero-Sequence Voltage, Magnitude"
      n_U0_bushv = "n:U0:bushv", "kV, Zero-Sequence Voltage, Magnitude"
      n_U0_buslv = "n:U0:buslv", "kV, Zero-Sequence Voltage, Magnitude"
      n_U0_bushvn = "n:U0:bushvn", "kV, Zero-Sequence Voltage, Magnitude"
      n_U0_buslvn = "n:U0:buslvn", "kV, Zero-Sequence Voltage, Magnitude"
      n_U0x3_bushv = "n:U0x3:bushv", "kV, 3*U0"
      n_U0x3_buslv = "n:U0x3:buslv", "kV, 3*U0"
      n_U0x3_bushvn = "n:U0x3:bushvn", "kV, 3*U0"
      n_U0x3_buslvn = "n:U0x3:buslvn", "kV, 3*U0"
      n_phiu0_bushv = "n:phiu0:bushv", "deg, Zero-Sequence Voltage, Angle"
      n_phiu0_buslv = "n:phiu0:buslv", "deg, Zero-Sequence Voltage, Angle"
      n_phiu0_bushvn = "n:phiu0:bushvn", "deg, Zero-Sequence Voltage, Angle"
      n_phiu0_buslvn = "n:phiu0:buslvn", "deg, Zero-Sequence Voltage, Angle"
      n_u1_bushv = "n:u1:bushv", "p.u., Positive-Sequence Voltage, Magnitude"
      n_u1_buslv = "n:u1:buslv", "p.u., Positive-Sequence Voltage, Magnitude"
      n_u1_bushvn = "n:u1:bushvn", "p.u., Positive-Sequence Voltage, Magnitude"
      n_u1_buslvn = "n:u1:buslvn", "p.u., Positive-Sequence Voltage, Magnitude"
      n_u1pc_bushv = "n:u1pc:bushv", "%, Positive-Sequence Voltage, Magnitude"
      n_u1pc_buslv = "n:u1pc:buslv", "%, Positive-Sequence Voltage, Magnitude"
      n_u1pc_bushvn = "n:u1pc:bushvn", "%, Positive-Sequence Voltage, Magnitude"
      n_u1pc_buslvn = "n:u1pc:buslvn", "%, Positive-Sequence Voltage, Magnitude"
      n_u1r_bushv = "n:u1r:bushv", "p.u., Positive-Sequence Voltage, Real Part"
      n_u1r_buslv = "n:u1r:buslv", "p.u., Positive-Sequence Voltage, Real Part"
      n_u1r_bushvn = "n:u1r:bushvn", "p.u., Positive-Sequence Voltage, Real Part"
      n_u1r_buslvn = "n:u1r:buslvn", "p.u., Positive-Sequence Voltage, Real Part"
      n_u1i_bushv = "n:u1i:bushv", "p.u., Positive-Sequence Voltage, Imaginary Part"
      n_u1i_buslv = "n:u1i:buslv", "p.u., Positive-Sequence Voltage, Imaginary Part"
      n_u1i_bushvn = "n:u1i:bushvn", "p.u., Positive-Sequence Voltage, Imaginary Part"
      n_u1i_buslvn = "n:u1i:buslvn", "p.u., Positive-Sequence Voltage, Imaginary Part"
      n_U1_bushv = "n:U1:bushv", "kV, Line-Ground Positive-Sequence Voltage, Magnitude"
      n_U1_buslv = "n:U1:buslv", "kV, Line-Ground Positive-Sequence Voltage, Magnitude"
      n_U1_bushvn = "n:U1:bushvn", "kV, Line-Ground Positive-Sequence Voltage, Magnitude"
      n_U1_buslvn = "n:U1:buslvn", "kV, Line-Ground Positive-Sequence Voltage, Magnitude"
      n_phiu1_bushv = "n:phiu1:bushv", "deg, Positive-Sequence Voltage, Angle"
      n_phiu1_buslv = "n:phiu1:buslv", "deg, Positive-Sequence Voltage, Angle"
      n_phiu1_bushvn = "n:phiu1:bushvn", "deg, Positive-Sequence Voltage, Angle"
      n_phiu1_buslvn = "n:phiu1:buslvn", "deg, Positive-Sequence Voltage, Angle"
      n_u2_bushv = "n:u2:bushv", "p.u., Negative-Sequence Voltage, Magnitude"
      n_u2_buslv = "n:u2:buslv", "p.u., Negative-Sequence Voltage, Magnitude"
      n_u2_bushvn = "n:u2:bushvn", "p.u., Negative-Sequence Voltage, Magnitude"
      n_u2_buslvn = "n:u2:buslvn", "p.u., Negative-Sequence Voltage, Magnitude"
      n_U2_bushv = "n:U2:bushv", "kV, Line-Ground Negative-Sequence Voltage, Magnitude"
      n_U2_buslv = "n:U2:buslv", "kV, Line-Ground Negative-Sequence Voltage, Magnitude"
      n_U2_bushvn = "n:U2:bushvn", "kV, Line-Ground Negative-Sequence Voltage, Magnitude"
      n_U2_buslvn = "n:U2:buslvn", "kV, Line-Ground Negative-Sequence Voltage, Magnitude"
      n_phiu2_bushv = "n:phiu2:bushv", "deg, Negative-Sequence Voltage, Angle"
      n_phiu2_buslv = "n:phiu2:buslv", "deg, Negative-Sequence Voltage, Angle"
      n_phiu2_bushvn = "n:phiu2:bushvn", "deg, Negative-Sequence Voltage, Angle"
      n_phiu2_buslvn = "n:phiu2:buslvn", "deg, Negative-Sequence Voltage, Angle"
      n_U1l_bushv = "n:U1l:bushv", "kV, Line-Line Positive-Sequence Voltage, Magnitude"
      n_U1l_buslv = "n:U1l:buslv", "kV, Line-Line Positive-Sequence Voltage, Magnitude"
      n_U1l_bushvn = "n:U1l:bushvn", "kV, Line-Line Positive-Sequence Voltage, Magnitude"
      n_U1l_buslvn = "n:U1l:buslvn", "kV, Line-Line Positive-Sequence Voltage, Magnitude"
      n_U2l_bushv = "n:U2l:bushv", "kV, Line-Line Negative-Sequence Voltage, Magnitude"
      n_U2l_buslv = "n:U2l:buslv", "kV, Line-Line Negative-Sequence Voltage, Magnitude"
      n_U2l_bushvn = "n:U2l:bushvn", "kV, Line-Line Negative-Sequence Voltage, Magnitude"
      n_U2l_buslvn = "n:U2l:buslvn", "kV, Line-Line Negative-Sequence Voltage, Magnitude"
      n_uln_bushv_A = "n:uln:bushv:A", "p.u., Line-Neutral Voltage, Magnitude"
      n_uln_bushv_B = "n:uln:bushv:B", "p.u., Line-Neutral Voltage, Magnitude"
      n_uln_bushv_C = "n:uln:bushv:C", "p.u., Line-Neutral Voltage, Magnitude"
      n_uln_bushv_N = "n:uln:bushv:N", "p.u., Line-Neutral Voltage, Magnitude"
      n_uln_buslv_A = "n:uln:buslv:A", "p.u., Line-Neutral Voltage, Magnitude"
      n_uln_buslv_B = "n:uln:buslv:B", "p.u., Line-Neutral Voltage, Magnitude"
      n_uln_buslv_C = "n:uln:buslv:C", "p.u., Line-Neutral Voltage, Magnitude"
      n_uln_buslv_N = "n:uln:buslv:N", "p.u., Line-Neutral Voltage, Magnitude"
      n_uln_bushvn = "n:uln:bushvn", "p.u., Line-Neutral Voltage, Magnitude"
      n_uln_buslvn = "n:uln:buslvn", "p.u., Line-Neutral Voltage, Magnitude"
      n_Uln_bushv_A = "n:Uln:bushv:A", "kV, Line-Neutral Voltage, Magnitude"
      n_Uln_bushv_B = "n:Uln:bushv:B", "kV, Line-Neutral Voltage, Magnitude"
      n_Uln_bushv_C = "n:Uln:bushv:C", "kV, Line-Neutral Voltage, Magnitude"
      n_Uln_bushv_N = "n:Uln:bushv:N", "kV, Line-Neutral Voltage, Magnitude"
      n_Uln_buslv_A = "n:Uln:buslv:A", "kV, Line-Neutral Voltage, Magnitude"
      n_Uln_buslv_B = "n:Uln:buslv:B", "kV, Line-Neutral Voltage, Magnitude"
      n_Uln_buslv_C = "n:Uln:buslv:C", "kV, Line-Neutral Voltage, Magnitude"
      n_Uln_buslv_N = "n:Uln:buslv:N", "kV, Line-Neutral Voltage, Magnitude"
      n_Uln_bushvn = "n:Uln:bushvn", "kV, Line-Neutral Voltage, Magnitude"
      n_Uln_buslvn = "n:Uln:buslvn", "kV, Line-Neutral Voltage, Magnitude"
      n_phiuln_bushv_A = "n:phiuln:bushv:A", "deg, Line-Neutral Voltage, Angle"
      n_phiuln_bushv_B = "n:phiuln:bushv:B", "deg, Line-Neutral Voltage, Angle"
      n_phiuln_bushv_C = "n:phiuln:bushv:C", "deg, Line-Neutral Voltage, Angle"
      n_phiuln_bushv_N = "n:phiuln:bushv:N", "deg, Line-Neutral Voltage, Angle"
      n_phiuln_buslv_A = "n:phiuln:buslv:A", "deg, Line-Neutral Voltage, Angle"
      n_phiuln_buslv_B = "n:phiuln:buslv:B", "deg, Line-Neutral Voltage, Angle"
      n_phiuln_buslv_C = "n:phiuln:buslv:C", "deg, Line-Neutral Voltage, Angle"
      n_phiuln_buslv_N = "n:phiuln:buslv:N", "deg, Line-Neutral Voltage, Angle"
      n_phiuln_bushvn = "n:phiuln:bushvn", "deg, Line-Neutral Voltage, Angle"
      n_phiuln_buslvn = "n:phiuln:buslvn", "deg, Line-Neutral Voltage, Angle"
      n_un_bushv = "n:un:bushv", "p.u., Neutral-Ground Voltage, Magnitude"
      n_un_buslv = "n:un:buslv", "p.u., Neutral-Ground Voltage, Magnitude"
      n_un_bushvn = "n:un:bushvn", "p.u., Neutral-Ground Voltage, Magnitude"
      n_un_buslvn = "n:un:buslvn", "p.u., Neutral-Ground Voltage, Magnitude"
      n_Un_bushv = "n:Un:bushv", "kV, Neutral-Ground Voltage, Magnitude"
      n_Un_buslv = "n:Un:buslv", "kV, Neutral-Ground Voltage, Magnitude"
      n_Un_bushvn = "n:Un:bushvn", "kV, Neutral-Ground Voltage, Magnitude"
      n_Un_buslvn = "n:Un:buslvn", "kV, Neutral-Ground Voltage, Magnitude"
      n_phiun_bushv = "n:phiun:bushv", "deg, Neutral-Ground Voltage, Angle"
      n_phiun_buslv = "n:phiun:buslv", "deg, Neutral-Ground Voltage, Angle"
      n_phiun_bushvn = "n:phiun:bushvn", "deg, Neutral-Ground Voltage, Angle"
      n_phiun_buslvn = "n:phiun:buslvn", "deg, Neutral-Ground Voltage, Angle"
      n_upht_bushv_A = "n:upht:bushv:A", "p.u., Phase Technology dependent Voltage, Magnitude"
      n_upht_bushv_B = "n:upht:bushv:B", "p.u., Phase Technology dependent Voltage, Magnitude"
      n_upht_bushv_C = "n:upht:bushv:C", "p.u., Phase Technology dependent Voltage, Magnitude"
      n_upht_bushv_N = "n:upht:bushv:N", "p.u., Phase Technology dependent Voltage, Magnitude"
      n_upht_buslv_A = "n:upht:buslv:A", "p.u., Phase Technology dependent Voltage, Magnitude"
      n_upht_buslv_B = "n:upht:buslv:B", "p.u., Phase Technology dependent Voltage, Magnitude"
      n_upht_buslv_C = "n:upht:buslv:C", "p.u., Phase Technology dependent Voltage, Magnitude"
      n_upht_buslv_N = "n:upht:buslv:N", "p.u., Phase Technology dependent Voltage, Magnitude"
      n_upht_bushvn = "n:upht:bushvn", "p.u., Phase Technology dependent Voltage, Magnitude"
      n_upht_buslvn = "n:upht:buslvn", "p.u., Phase Technology dependent Voltage, Magnitude"
      n_Upht_bushv_A = "n:Upht:bushv:A", "kV, Phase Technology dependent Voltage, Magnitude"
      n_Upht_bushv_B = "n:Upht:bushv:B", "kV, Phase Technology dependent Voltage, Magnitude"
      n_Upht_bushv_C = "n:Upht:bushv:C", "kV, Phase Technology dependent Voltage, Magnitude"
      n_Upht_bushv_N = "n:Upht:bushv:N", "kV, Phase Technology dependent Voltage, Magnitude"
      n_Upht_buslv_A = "n:Upht:buslv:A", "kV, Phase Technology dependent Voltage, Magnitude"
      n_Upht_buslv_B = "n:Upht:buslv:B", "kV, Phase Technology dependent Voltage, Magnitude"
      n_Upht_buslv_C = "n:Upht:buslv:C", "kV, Phase Technology dependent Voltage, Magnitude"
      n_Upht_buslv_N = "n:Upht:buslv:N", "kV, Phase Technology dependent Voltage, Magnitude"
      n_Upht_bushvn = "n:Upht:bushvn", "kV, Phase Technology dependent Voltage, Magnitude"
      n_Upht_buslvn = "n:Upht:buslvn", "kV, Phase Technology dependent Voltage, Magnitude"
      n_um_bushv = "n:um:bushv", "p.u., Average-Voltage, Magnitude"
      n_um_buslv = "n:um:buslv", "p.u., Average-Voltage, Magnitude"
      n_um_bushvn = "n:um:bushvn", "p.u., Average-Voltage, Magnitude"
      n_um_buslvn = "n:um:buslvn", "p.u., Average-Voltage, Magnitude"
      n_Um_bushv = "n:Um:bushv", "kV, Average-Voltage, Magnitude"
      n_Um_buslv = "n:Um:buslv", "kV, Average-Voltage, Magnitude"
      n_Um_bushvn = "n:Um:bushvn", "kV, Average-Voltage, Magnitude"
      n_Um_buslvn = "n:Um:buslvn", "kV, Average-Voltage, Magnitude"
      n_uphtmin_bushv = "n:uphtmin:bushv", "p.u., Minimum of Phase Technology dependent Voltage, Magnitude"
      n_uphtmin_buslv = "n:uphtmin:buslv", "p.u., Minimum of Phase Technology dependent Voltage, Magnitude"
      n_uphtmin_bushvn = "n:uphtmin:bushvn", "p.u., Minimum of Phase Technology dependent Voltage, Magnitude"
      n_uphtmin_buslvn = "n:uphtmin:buslvn", "p.u., Minimum of Phase Technology dependent Voltage, Magnitude"
      n_uphtmax_bushv = "n:uphtmax:bushv", "p.u., Maximum of Phase Technology dependent Voltage, Magnitude"
      n_uphtmax_buslv = "n:uphtmax:buslv", "p.u., Maximum of Phase Technology dependent Voltage, Magnitude"
      n_uphtmax_bushvn = "n:uphtmax:bushvn", "p.u., Maximum of Phase Technology dependent Voltage, Magnitude"
      n_uphtmax_buslvn = "n:uphtmax:buslvn", "p.u., Maximum of Phase Technology dependent Voltage, Magnitude"
      n_Pgen_bushv = "n:Pgen:bushv", "MW, Generation, Active Power"
      n_Pgen_buslv = "n:Pgen:buslv", "MW, Generation, Active Power"
      n_Pgen_bushvn = "n:Pgen:bushvn", "MW, Generation, Active Power"
      n_Pgen_buslvn = "n:Pgen:buslvn", "MW, Generation, Active Power"
      n_Qgen_bushv = "n:Qgen:bushv", "Mvar, Generation, Reactive Power"
      n_Qgen_buslv = "n:Qgen:buslv", "Mvar, Generation, Reactive Power"
      n_Qgen_bushvn = "n:Qgen:bushvn", "Mvar, Generation, Reactive Power"
      n_Qgen_buslvn = "n:Qgen:buslvn", "Mvar, Generation, Reactive Power"
      n_Pmot_bushv = "n:Pmot:bushv", "MW, Motor Load, Active Power"
      n_Pmot_buslv = "n:Pmot:buslv", "MW, Motor Load, Active Power"
      n_Pmot_bushvn = "n:Pmot:bushvn", "MW, Motor Load, Active Power"
      n_Pmot_buslvn = "n:Pmot:buslvn", "MW, Motor Load, Active Power"
      n_Qmot_bushv = "n:Qmot:bushv", "Mvar, Motor Load, Reactive Power"
      n_Qmot_buslv = "n:Qmot:buslv", "Mvar, Motor Load, Reactive Power"
      n_Qmot_bushvn = "n:Qmot:bushvn", "Mvar, Motor Load, Reactive Power"
      n_Qmot_buslvn = "n:Qmot:buslvn", "Mvar, Motor Load, Reactive Power"
      n_Pload_bushv = "n:Pload:bushv", "MW, General Load, Active Power"
      n_Pload_buslv = "n:Pload:buslv", "MW, General Load, Active Power"
      n_Pload_bushvn = "n:Pload:bushvn", "MW, General Load, Active Power"
      n_Pload_buslvn = "n:Pload:buslvn", "MW, General Load, Active Power"
      n_Qload_bushv = "n:Qload:bushv", "Mvar, General Load, Reactive Power"
      n_Qload_buslv = "n:Qload:buslv", "Mvar, General Load, Reactive Power"
      n_Qload_bushvn = "n:Qload:bushvn", "Mvar, General Load, Reactive Power"
      n_Qload_buslvn = "n:Qload:buslvn", "Mvar, General Load, Reactive Power"
      n_Pcomp_bushv = "n:Pcomp:bushv", "MW, Compensation (Losses)"
      n_Pcomp_buslv = "n:Pcomp:buslv", "MW, Compensation (Losses)"
      n_Pcomp_bushvn = "n:Pcomp:bushvn", "MW, Compensation (Losses)"
      n_Pcomp_buslvn = "n:Pcomp:buslvn", "MW, Compensation (Losses)"
      n_Qcomp_bushv = "n:Qcomp:bushv", "Mvar, Compensation"
      n_Qcomp_buslv = "n:Qcomp:buslv", "Mvar, Compensation"
      n_Qcomp_bushvn = "n:Qcomp:bushvn", "Mvar, Compensation"
      n_Qcomp_buslvn = "n:Qcomp:buslvn", "Mvar, Compensation"
      n_Pnet_bushv = "n:Pnet:bushv", "MW, External Networks, Active Power"
      n_Pnet_buslv = "n:Pnet:buslv", "MW, External Networks, Active Power"
      n_Pnet_bushvn = "n:Pnet:bushvn", "MW, External Networks, Active Power"
      n_Pnet_buslvn = "n:Pnet:buslvn", "MW, External Networks, Active Power"
      n_Qnet_bushv = "n:Qnet:bushv", "Mvar, External Networks, Reactive Power"
      n_Qnet_buslv = "n:Qnet:buslv", "Mvar, External Networks, Reactive Power"
      n_Qnet_bushvn = "n:Qnet:bushvn", "Mvar, External Networks, Reactive Power"
      n_Qnet_buslvn = "n:Qnet:buslvn", "Mvar, External Networks, Reactive Power"
      n_Pflow_bushv = "n:Pflow:bushv", "MW, Power Flow, Active Power"
      n_Pflow_buslv = "n:Pflow:buslv", "MW, Power Flow, Active Power"
      n_Pflow_bushvn = "n:Pflow:bushvn", "MW, Power Flow, Active Power"
      n_Pflow_buslvn = "n:Pflow:buslvn", "MW, Power Flow, Active Power"
      n_Qflow_bushv = "n:Qflow:bushv", "Mvar, Power Flow, Reactive Power"
      n_Qflow_buslv = "n:Qflow:buslv", "Mvar, Power Flow, Reactive Power"
      n_Qflow_bushvn = "n:Qflow:bushvn", "Mvar, Power Flow, Reactive Power"
      n_Qflow_buslvn = "n:Qflow:buslvn", "Mvar, Power Flow, Reactive Power"
      n_Pout_bushv = "n:Pout:bushv", "MW, Outgoing Flow, Active Power"
      n_Pout_buslv = "n:Pout:buslv", "MW, Outgoing Flow, Active Power"
      n_Pout_bushvn = "n:Pout:bushvn", "MW, Outgoing Flow, Active Power"
      n_Pout_buslvn = "n:Pout:buslvn", "MW, Outgoing Flow, Active Power"
      n_Qout_bushv = "n:Qout:bushv", "Mvar, Outgoing Power, Reactive Power"
      n_Qout_buslv = "n:Qout:buslv", "Mvar, Outgoing Power, Reactive Power"
      n_Qout_bushvn = "n:Qout:bushvn", "Mvar, Outgoing Power, Reactive Power"
      n_Qout_buslvn = "n:Qout:buslvn", "Mvar, Outgoing Power, Reactive Power"
      n_Sout_bushv = "n:Sout:bushv", "MVA, Outgoing Power, Apparent Power"
      n_Sout_buslv = "n:Sout:buslv", "MVA, Outgoing Power, Apparent Power"
      n_Sout_bushvn = "n:Sout:bushvn", "MVA, Outgoing Power, Apparent Power"
      n_Sout_buslvn = "n:Sout:buslvn", "MVA, Outgoing Power, Apparent Power"
      n_cosphiout_bushv = "n:cosphiout:bushv", "Outgoing Power, Power Factor"
      n_cosphiout_buslv = "n:cosphiout:buslv", "Outgoing Power, Power Factor"
      n_cosphiout_bushvn = "n:cosphiout:bushvn", "Outgoing Power, Power Factor"
      n_cosphiout_buslvn = "n:cosphiout:buslvn", "Outgoing Power, Power Factor"
      n_Pbalance_bushv = "n:Pbalance:bushv", "MW, Active Power Balance (=0)"
      n_Pbalance_buslv = "n:Pbalance:buslv", "MW, Active Power Balance (=0)"
      n_Pbalance_bushvn = "n:Pbalance:bushvn", "MW, Active Power Balance (=0)"
      n_Pbalance_buslvn = "n:Pbalance:buslvn", "MW, Active Power Balance (=0)"
      n_Qbalance_bushv = "n:Qbalance:bushv", "Mvar, Reactive Power Balance (=0)"
      n_Qbalance_buslv = "n:Qbalance:buslv", "Mvar, Reactive Power Balance (=0)"
      n_Qbalance_bushvn = "n:Qbalance:bushvn", "Mvar, Reactive Power Balance (=0)"
      n_Qbalance_buslvn = "n:Qbalance:buslvn", "Mvar, Reactive Power Balance (=0)"
      n_ubfac_bushv = "n:ubfac:bushv", "%, Unbalance Factor"
      n_ubfac_buslv = "n:ubfac:buslv", "%, Unbalance Factor"
      n_ubfac_bushvn = "n:ubfac:bushvn", "%, Unbalance Factor"
      n_ubfac_buslvn = "n:ubfac:buslvn", "%, Unbalance Factor"
      n_LossPdown_bushv = "n:LossPdown:bushv", "MW, Losses, downstream"
      n_LossPdown_buslv = "n:LossPdown:buslv", "MW, Losses, downstream"
      n_LossPdown_bushvn = "n:LossPdown:bushvn", "MW, Losses, downstream"
      n_LossPdown_buslvn = "n:LossPdown:buslvn", "MW, Losses, downstream"
      n_LossQdown_bushv = "n:LossQdown:bushv", "Mvar, Losses, downstream (Reactive Power)"
      n_LossQdown_buslv = "n:LossQdown:buslv", "Mvar, Losses, downstream (Reactive Power)"
      n_LossQdown_bushvn = "n:LossQdown:bushvn", "Mvar, Losses, downstream (Reactive Power)"
      n_LossQdown_buslvn = "n:LossQdown:buslvn", "Mvar, Losses, downstream (Reactive Power)"
      n_LossPdownload_bushv = "n:LossPdownload:bushv", "MW, Load losses, downstream"
      n_LossPdownload_buslv = "n:LossPdownload:buslv", "MW, Load losses, downstream"
      n_LossPdownload_bushvn = "n:LossPdownload:bushvn", "MW, Load losses, downstream"
      n_LossPdownload_buslvn = "n:LossPdownload:buslvn", "MW, Load losses, downstream"
      n_LossQdownload_bushv = "n:LossQdownload:bushv", "Mvar, Load losses, downstream"
      n_LossQdownload_buslv = "n:LossQdownload:buslv", "Mvar, Load losses, downstream"
      n_LossQdownload_bushvn = "n:LossQdownload:bushvn", "Mvar, Load losses, downstream"
      n_LossQdownload_buslvn = "n:LossQdownload:buslvn", "Mvar, Load losses, downstream"
      n_LossPdownnoload_bushv = "n:LossPdownnoload:bushv", "MW, No load losses, downstream"
      n_LossPdownnoload_buslv = "n:LossPdownnoload:buslv", "MW, No load losses, downstream"
      n_LossPdownnoload_bushvn = "n:LossPdownnoload:bushvn", "MW, No load losses, downstream"
      n_LossPdownnoload_buslvn = "n:LossPdownnoload:buslvn", "MW, No load losses, downstream"
      n_LossQdownnoload_bushv = "n:LossQdownnoload:bushv", "Mvar, No load losses, downstream"
      n_LossQdownnoload_buslv = "n:LossQdownnoload:buslv", "Mvar, No load losses, downstream"
      n_LossQdownnoload_bushvn = "n:LossQdownnoload:bushvn", "Mvar, No load losses, downstream"
      n_LossQdownnoload_buslvn = "n:LossQdownnoload:buslvn", "Mvar, No load losses, downstream"
      s_nt_in = "s:nt_in", "IN, Tap 1, Input, continuous, double"
      s_nt2_in = "s:nt2_in", "IN, Tap 2, Input, continuous, double"
      c_loading_h = "c:loading_h", "%, Loading (HV)"
      c_loading_l = "c:loading_l", "%, Loading (LV)"
      c_loading = "c:loading", "%, Loading"
      c_maxload = "c:maxload", "%, Max. Loading"
      c_nntap = "c:nntap", "Tap 1, Current Position"
      c_ntpmn = "c:ntpmn", "Tap 1, Min. Position"
      c_ntpmx = "c:ntpmx", "Tap 1, Max. Position"
      c_dut = "c:dut", "Tap 1: Voltage/Tap"
      c_phitrrd = "c:phitrrd", "rad, Tap 1:, Phase/Tap"
      c_nt2agrd = "c:nt2agrd", "rad, Phase Angle"
      c_ratfac = "c:ratfac", "Rating Factor"
      c_snom = "c:snom", "MVA, Rated Power"
      c_zshv_r = "c:zshv:r", "p.u., Leakage Impedance (HV-Side), Real Part"
      c_zshv_i = "c:zshv:i", "p.u., Leakage Impedance (HV-Side), Imaginary Part"
      c_zshv_phi = "c:zshv:phi", "deg, Leakage Impedance (HV-Side), Angle"
      c_zshv_mag = "c:zshv:mag", "p.u., Leakage Impedance (HV-Side), Magnitude"
      c_zshv_phirad = "c:zshv:phirad", "rad, Leakage Impedance (HV-Side), Angle"
      c_zslv_r = "c:zslv:r", "p.u., Leakage Impedance (LV-Side), Real Part"
      c_zslv_i = "c:zslv:i", "p.u., Leakage Impedance (LV-Side), Imaginary Part"
      c_zslv_phi = "c:zslv:phi", "deg, Leakage Impedance (LV-Side), Angle"
      c_zslv_mag = "c:zslv:mag", "p.u., Leakage Impedance (LV-Side), Magnitude"
      c_zslv_phirad = "c:zslv:phirad", "rad, Leakage Impedance (LV-Side), Angle"
      c_ym_r = "c:ym:r", "p.u., Magnetising Admittance, Real Part"
      c_ym_i = "c:ym:i", "p.u., Magnetising Admittance, Imaginary Part"
      c_ym_phi = "c:ym:phi", "deg, Magnetising Admittance, Angle"
      c_ym_mag = "c:ym:mag", "p.u., Magnetising Admittance, Magnitude"
      c_ym_phirad = "c:ym:phirad", "rad, Magnetising Admittance, Angle"
      c_thv_r = "c:thv:r", "Complex Winding Ratio HV, Real Part"
      c_thv_i = "c:thv:i", "Complex Winding Ratio HV, Imaginary Part"
      c_thv_phi = "c:thv:phi", "Complex Winding Ratio HV, Angle"
      c_thv_mag = "c:thv:mag", "Complex Winding Ratio HV, Magnitude"
      c_thv_phirad = "c:thv:phirad", "Complex Winding Ratio HV, Angle"
      c_tlv_r = "c:tlv:r", "Complex Winding Ratio LV, Real Part"
      c_tlv_i = "c:tlv:i", "Complex Winding Ratio LV, Imaginary Part"
      c_tlv_phi = "c:tlv:phi", "Complex Winding Ratio LV, Angle"
      c_tlv_mag = "c:tlv:mag", "Complex Winding Ratio LV, Magnitude"
      c_tlv_phirad = "c:tlv:phirad", "Complex Winding Ratio LV, Angle"
      c_tap_side = "c:tap_side", "0"
      c_tap_side2 = "c:tap_side2", "0"
      c_nntap2 = "c:nntap2", "Tap 2, Current Position"
      c_ntpmn2 = "c:ntpmn2", "Tap 2, Min. Position"
      c_ntpmx2 = "c:ntpmx2", "Tap 2, Max. Position"
      c_dut2 = "c:dut2", "Tap 2: Voltage/Tap"
      c_phitrrd2 = "c:phitrrd2", "rad, Tap 2:, Phase/Tap"
      c_Pmeas = "c:Pmeas", "MW, Measured active power"
      c_du_ldc = "c:du_ldc", "p.u., LDC Input"
      c_ctrl_set = "c:ctrl_set", "Setpoint"
      c_i_uplim = "c:i_uplim", "Tap 1 at Maximum"
      c_i_lowlim = "c:i_lowlim", "Tap 1 at Minimum"
      c_i_uplim2 = "c:i_uplim2", "Tap 2 at Maximum"
      c_i_lowlim2 = "c:i_lowlim2", "Tap 2 at Minimum"
      c_nCtrlIter = "c:nCtrlIter", "No. of CTRL-iterations"
      c_i_ea_h = "c:i_ea_h", "Flag: HV-Side grounded"
      c_i_ea_l = "c:i_ea_l", "Flag: LV-Side grounded"
      c_i_star_h = "c:i_star_h", "Flag: HV-Side Y"
      c_i_star_l = "c:i_star_l", "Flag: LV-Side Y"
      c_zs0hv_r = "c:zs0hv:r", "p.u., Zero sequence short circuit imp. (HV), Real Part"
      c_zs0hv_i = "c:zs0hv:i", "p.u., Zero sequence short circuit imp. (HV), Imaginary Part"
      c_zs0hv_phi = "c:zs0hv:phi", "deg, Zero sequence short circuit imp. (HV), Angle"
      c_zs0hv_mag = "c:zs0hv:mag", "p.u., Zero sequence short circuit imp. (HV), Magnitude"
      c_zs0hv_phirad = "c:zs0hv:phirad", "rad, Zero sequence short circuit imp. (HV), Angle"
      c_zs0lv_r = "c:zs0lv:r", "p.u., Zero sequence short circuit imp. (LV), Real Part"
      c_zs0lv_i = "c:zs0lv:i", "p.u., Zero sequence short circuit imp. (LV), Imaginary Part"
      c_zs0lv_phi = "c:zs0lv:phi", "deg, Zero sequence short circuit imp. (LV), Angle"
      c_zs0lv_mag = "c:zs0lv:mag", "p.u., Zero sequence short circuit imp. (LV), Magnitude"
      c_zs0lv_phirad = "c:zs0lv:phirad", "rad, Zero sequence short circuit imp. (LV), Angle"
      c_ym0_r = "c:ym0:r", "p.u., Zero Sequence Magnetising Admittance, Real Part"
      c_ym0_i = "c:ym0:i", "p.u., Zero Sequence Magnetising Admittance, Imaginary Part"
      c_ym0_phi = "c:ym0:phi", "deg, Zero Sequence Magnetising Admittance, Angle"
      c_ym0_mag = "c:ym0:mag", "p.u., Zero Sequence Magnetising Admittance, Magnitude"
      c_ym0_phirad = "c:ym0:phirad", "rad, Zero Sequence Magnetising Admittance, Angle"
      c_Ustar_h = "c:Ustar_h", "kV, Star-Point Voltage (HV)"
      c_Ustar_l = "c:Ustar_l", "kV, Star-Point Voltage (LV)"
      c_Istar_h = "c:Istar_h", "kA, Star-Point Current (HV)"
      c_Istar_l = "c:Istar_l", "kA, Star-Point Current (LV)"
      c_w_0 = "c:w_0", "Nominal winding ratio"
      c_Gmload = "c:Gmload", "kW, Iron Losses"
      c_Xmload = "c:Xmload", "kvar, Magnetising Losses"
      c_Ploss = "c:Ploss", "MW, Losses (total)"
      c_Qloss = "c:Qloss", "Mvar, Reactive-Losses (total)"
      c_Plossld = "c:Plossld", "MW, Losses (load)"
      c_Qlossld = "c:Qlossld", "Mvar, Reactive-Losses (load)"
      c_Plossnld = "c:Plossnld", "MW, Losses (no load)"
      c_Qlossnld = "c:Qlossnld", "Mvar, Reactive-Losses (no load)"
      c_Tctrl = "c:Tctrl", "s, Controller Time Constant"
      c_i_fixedtap = "c:i_fixedtap", "Flag"
      c_isat = "c:isat", "Saturated Model"
      c_loading_st = "c:loading_st", "%, Short Term Loading"
      c_Inom_HV = "c:Inom_HV", "kA, Nominal Current (HV-Side)"
      c_Inom_LV = "c:Inom_LV", "kA, Nominal Current (LV-Side)"
      c_du_A = "c:du:A", "p.u., Voltage Drop"
      c_du_B = "c:du:B", "p.u., Voltage Drop"
      c_du_C = "c:du:C", "p.u., Voltage Drop"
      c_du_N = "c:du:N", "p.u., Voltage Drop"
      c_dupc_A = "c:dupc:A", "%, Voltage Drop"
      c_dupc_B = "c:dupc:B", "%, Voltage Drop"
      c_dupc_C = "c:dupc:C", "%, Voltage Drop"
      c_dupc_N = "c:dupc:N", "%, Voltage Drop"
      c_dphiu_A = "c:dphiu:A", "deg, Voltage Drop Angle"
      c_dphiu_B = "c:dphiu:B", "deg, Voltage Drop Angle"
      c_dphiu_C = "c:dphiu:C", "deg, Voltage Drop Angle"
      c_dphiu_N = "c:dphiu:N", "deg, Voltage Drop Angle"
      c_du1 = "c:du1", "p.u., Positive Sequence Voltage Drop"
      c_du1pc = "c:du1pc", "%, Positive Sequence Voltage Drop"
      c_dphiu1 = "c:dphiu1", "deg, Positive Sequence Voltage Drop Angle"
      e_dutap = "e:dutap", "%, Additional Voltage per Tap"
      e_dphitap = "e:dphitap", "deg, Additional Angle per Tap"
      e_phitr = "e:phitr", "deg, Phase of du"
      e_nntap = "e:nntap", "Tap Position"
      e_iTaps = "e:iTaps", "According to Measurement Report"
      e_dutap2 = "e:dutap2", "%, Additional Voltage per Tap"
      e_dphitap2 = "e:dphitap2", "deg, Additional Angle per Tap"
      e_phitr2 = "e:phitr2", "deg, Phase of du"
      e_nntap2 = "e:nntap2", "Tap Position"
      e_optaplimit = "e:optaplimit", "Operational limits for tap changer"
      e_optapmin = "e:optapmin", "Minimum Position"
      e_optapmax = "e:optapmax", "Maximum Position"
      e_optap2min = "e:optap2min", "Minimum Position"
      e_optap2max = "e:optap2max", "Maximum Position"
      e_maxload = "e:maxload", "%, Max. loading"
      e_lossAssign = "e:lossAssign", "Loss assignment"
      e_c_ptapc = "e:c_ptapc", "ElmTapctrl, External Tap Controller"
      e_c_pstac = "e:c_pstac", "External Station Controller"
      e_c_plcc = "e:c_plcc", "External LCC Controller"
      e_qdslCtrl = "e:qdslCtrl", "Quasi-Dynamic Model"
      e_ntrcn = "e:ntrcn", "Automatic Tap Changing"
      e_i_cont = "e:i_cont", "Tap Changer"
      e_t2ldc = "e:t2ldc", "Controlled Node is at"
      e_ilcph = "e:ilcph", "Phase"
      e_imldc = "e:imldc", "Control Mode"
      e_uset_mode = "e:uset_mode", "Setpoint"
      e_tapctrl = "e:tapctrl", "ElmTr2*, Tap Controller"
      e_iPpart = "e:iPpart", "Active power participation"
      e_i_rem = "e:i_rem", "Remote Control"
      e_p_rem = "e:p_rem", "ElmTerm,StaBar, Controlled Node"
      e_p_cub = "e:p_cub", "StaCubic,ElmBoundary, Controlled Branch (Cubicle)"
      e_cpCtrlNode = "e:cpCtrlNode", "Target Node"
      e_Kpart = "e:Kpart", "Participation factor"
      e_Ptolerance = "e:Ptolerance", "MW, Tolerance (+/-)"
      e_p_pmeas = "e:p_pmeas", "StaCubic,ElmBoundary, P measured at"
      e_usetp = "e:usetp", "p.u., Voltage Setpoint"
      e_usp_low = "e:usp_low", "p.u., Lower Voltage Bound"
      e_usp_up = "e:usp_up", "p.u., Upper Voltage Bound"
      e_psetp = "e:psetp", "MW, Active Power Setpoint"
      e_psp_low = "e:psp_low", "MW, Lower Active Power Bound"
      e_psp_up = "e:psp_up", "MW, Upper Active Power Bound"
      e_qsetp = "e:qsetp", "Mvar, Reactive Power Setpoint"
      e_qsp_low = "e:qsp_low", "Mvar, Lower Reactive Power Bound"
      e_qsp_up = "e:qsp_up", "Mvar, Upper Reactive Power Bound"
      e_Tctrl = "e:Tctrl", "s, Controller Time Constant"
      e_Kqctrl = "e:Kqctrl", "tap/Mvar, Controller Sensitivity dtap/dQ"
      e_Kpctrl = "e:Kpctrl", "tap/MW, Controller Sensitivity dtap/dP"
      e_ildc = "e:ildc", "Compensation"
      e_pldc = "e:pldc", "StaLdc, External LDC"
      e_ldcct = "e:ldcct", "A, Current Transformer Rating"
      e_ldcpt = "e:ldcpt", "Voltage Transformer Ratio"
      e_ldcrs = "e:ldcrs", "V, Rset"
      e_ldcxs = "e:ldcxs", "V, Xset"
      e_icomp = "e:icomp", "Compounding"
      e_Vtolerance = "e:Vtolerance", "%, Tolerance (+/-)"
      e_pVccurve = "e:pVccurve", "IntVctrlcurve, V-Control-Curve"
      e_ntrcn2 = "e:ntrcn2", "Automatic Tap Changing"
      e_i_cont2 = "e:i_cont2", "Tap Changer"
      e_t2ldc2 = "e:t2ldc2", "Controlled Node is at"
      e_ilcph2 = "e:ilcph2", "Phase"
      e_imldc2 = "e:imldc2", "Control Mode"
      e_uset_mode2 = "e:uset_mode2", "Setpoint"
      e_iPpart2 = "e:iPpart2", "Active power participation"
      e_i_rem2 = "e:i_rem2", "Remote Control"
      e_p_rem2 = "e:p_rem2", "ElmTerm,StaBar, Controlled Node"
      e_p_cub2 = "e:p_cub2", "StaCubic,ElmBoundary, Controlled Branch (Cubicle)"
      e_cpCtrlNode2 = "e:cpCtrlNode2", "Target Node"
      e_Kpart2 = "e:Kpart2", "Participation factor"
      e_Ptolerance2 = "e:Ptolerance2", "MW, Tolerance (+/-)"
      e_p_pmeas2 = "e:p_pmeas2", "StaCubic,ElmBoundary, P measured at"
      e_usetp2 = "e:usetp2", "p.u., Voltage Setpoint"
      e_usp_low2 = "e:usp_low2", "p.u., Lower Voltage Bound"
      e_usp_up2 = "e:usp_up2", "p.u., Upper Voltage Bound"
      e_psetp2 = "e:psetp2", "MW, Active Power Setpoint"
      e_psp_low2 = "e:psp_low2", "MW, Lower Active Power Bound"
      e_psp_up2 = "e:psp_up2", "MW, Upper Active Power Bound"
      e_qsetp2 = "e:qsetp2", "Mvar, Reactive Power Setpoint"
      e_qsp_low2 = "e:qsp_low2", "Mvar, Lower Reactive Power Bound"
      e_qsp_up2 = "e:qsp_up2", "Mvar, Upper Reactive Power Bound"
      e_Tctrl2 = "e:Tctrl2", "s, Controller Time Constant"
      e_Kqctrl2 = "e:Kqctrl2", "tap/Mvar, Controller Sensitivity dtap/dQ"
      e_Kpctrl2 = "e:Kpctrl2", "tap/MW, Controller Sensitivity dtap/dP"
      e_ildc2 = "e:ildc2", "Compensation"
      e_pldc2 = "e:pldc2", "StaLdc, External LDC"
      e_ldcct2 = "e:ldcct2", "A, Current Transformer Rating"
      e_ldcpt2 = "e:ldcpt2", "Voltage Transformer Ratio"
      e_ldcrs2 = "e:ldcrs2", "V, Rset"
      e_ldcxs2 = "e:ldcxs2", "V, Xset"
      e_icomp2 = "e:icomp2", "Compounding"
      e_Vtolerance2 = "e:Vtolerance2", "%, Tolerance (+/-)"
      e_pVccurve2 = "e:pVccurve2", "IntVctrlcurve, V-Control-Curve"
      e_iMeasLoc = "e:iMeasLoc", "Measured at"
      e_mTaps = "e:mTaps", "Measurement Report"
      e_iopt_hf = "e:iopt_hf", "Consider Capacitances"
      e_Cg_h = "e:Cg_h", "uF, Capacitance HV-Ground"
      e_Cg_l = "e:Cg_l", "uF, Capacitance LV-Ground"
      e_Cc1_hl = "e:Cc1_hl", "uF, Capacitance HV-LV, 1-Sequence"
      e_Cc0_hl = "e:Cc0_hl", "uF, Capacitance HV-LV, 0-Sequence"
      e_cpFeed = "e:cpFeed", "Feeder"
      e_ciDist = "e:ciDist", "Distance from infeed in number of buses"
      e_ciLater = "e:ciLater", "Lateral Index"
      e_ciDistRoot = "e:ciDistRoot", "Distance from first infeed in number of buses"
      e_ciDistAll = "e:ciDistAll", "Distance from infeed in number of buses including switches"
      e_ciDistAllRoot = "e:ciDistAllRoot", "Distance from first infeed in number of buses including switches"
      t_curmg = "t:curmg", "%, No Load Current"
      t_pfe = "t:pfe", "kW, No Load Losses"
      t_zx0hl_n = "t:zx0hl_n", "Mag. Impedance/uk0"
      t_rtox0_n = "t:rtox0_n", "Mag. R/X"
      t_itrdl = "t:itrdl", "x,Pos.Seq. HV-Side"
      t_itrdl_lv = "t:itrdl_lv", "x,Pos.Seq. LV-Side"
      t_itrdr = "t:itrdr", "r,Pos.Seq. HV-Side"
      t_itrdr_lv = "t:itrdr_lv", "r,Pos.Seq. LV-Side"
      t_zx0hl_h = "t:zx0hl_h", "z, Zero Sequ. HV-Side"
      t_zx0hl_l = "t:zx0hl_l", "z, Zero Sequ. LV-Side"
      t_itapch = "t:itapch", "Tap Changer 1"
      t_tapchtype = "t:tapchtype", "Type"
      t_tap_side = "t:tap_side", "at Side"
      t_dutap = "t:dutap", "%, Additional Voltage per Tap"
      t_dphitap = "t:dphitap", "deg, Additional Angle per Tap"
      t_phitr = "t:phitr", "deg, Phase of du"
      t_nntap0 = "t:nntap0", "Neutral Position"
      t_ntpmn = "t:ntpmn", "Minimum Position"
      t_ntpmx = "t:ntpmx", "Maximum Position"
      t_itapch2 = "t:itapch2", "Tap Changer 2"
      t_tapchtype2 = "t:tapchtype2", "Type"
      t_tap_side2 = "t:tap_side2", "at Side"
      t_dutap2 = "t:dutap2", "%, Additional Voltage per Tap"
      t_dphitap2 = "t:dphitap2", "deg, Additional Angle per Tap"
      t_phitr2 = "t:phitr2", "deg, Phase of du"
      t_nntap02 = "t:nntap02", "Neutral Position"
      t_ntpmn2 = "t:ntpmn2", "Minimum Position"
      t_ntpmx2 = "t:ntpmx2", "Maximum Position"
      t_itapzdep = "t:itapzdep", "Tap dependent impedance"
      t_uktr = "t:uktr", "%, Short-Circuit Voltage uk"
      t_uktmn = "t:uktmn", "%, uk (min. tap)"
      t_uktmx = "t:uktmx", "%, uk (max. tap)"
      t_x1pu = "t:x1pu", "p.u., Reactance x1"
      t_x1putmn = "t:x1putmn", "p.u., x1 (min. tap)"
      t_x1putmx = "t:x1putmx", "p.u., x1 (max. tap)"
      t_pcutr = "t:pcutr", "kW, Copper Losses"
      t_uktrr = "t:uktrr", "%, SHC-Voltage (Re(uk)) ukr"
      t_xtor = "t:xtor", "Ratio X/R"
      t_r1pu = "t:r1pu", "p.u., Resistance r1"
      t_pcutmn = "t:pcutmn", "kW, Pcu (min. tap)"
      t_ukrtmn = "t:ukrtmn", "%, Re(uk) (min. tap)"
      t_xtortmn = "t:xtortmn", "X/R (min. tap)"
      t_r1putmn = "t:r1putmn", "p.u., r1 (min. tap)"
      t_pcutmx = "t:pcutmx", "kW, Pcu (max. tap)"
      t_ukrtmx = "t:ukrtmx", "%, Re(uk) (max. tap)"
      t_xtortmx = "t:xtortmx", "X/R (max. tap)"
      t_r1putmx = "t:r1putmx", "p.u., r1 (max. tap)"
      t_uk0tr = "t:uk0tr", "%, Short-Circuit Voltage uk0"
      t_uk0tmn = "t:uk0tmn", "%, uk0 (min. tap)"
      t_uk0tmx = "t:uk0tmx", "%, uk0 (max. tap)"
      t_x0pu = "t:x0pu", "p.u., Reactance x0"
      t_x0putmn = "t:x0putmn", "p.u., x0 (min. tap)"
      t_x0putmx = "t:x0putmx", "p.u., x0 (max. tap)"
      t_ur0tr = "t:ur0tr", "%, SHC-Voltage (Re(uk0)) uk0r"
      t_x0tor0 = "t:x0tor0", "Ratio X0/R0"
      t_r0pu = "t:r0pu", "p.u., Resistance r0"
      t_uk0rtmn = "t:uk0rtmn", "%, Re(uk0) (min. tap)"
      t_x0tor0tmn = "t:x0tor0tmn", "X0/R0 (min. tap)"
      t_r0putmn = "t:r0putmn", "p.u., r0 (min. tap)"
      t_uk0rtmx = "t:uk0rtmx", "%, Re(uk0) (max. tap)"
      t_x0tor0tmx = "t:x0tor0tmx", "X0/R0 (max. tap)"
      t_r0putmx = "t:r0putmx", "p.u., r0 (max. tap)"
      t_iTapLoading = "t:iTapLoading", "Configuration"
      t_itrldf = "t:itrldf", "Type"
      t_satcue = "t:satcue", "%, Current (RMS)"
      t_satvol = "t:satvol", "p.u., Voltage (RMS)"
      t_iIntPola = "t:iIntPola", "Interpolation"
      t_smoothf = "t:smoothf", "%, Smoothing Factor"
      t_itratioadpt = "t:itratioadpt", "Transformer Ratio Adaptation"
      r_c_ptapc = "r:c_ptapc", "ElmTapctrl, External Tap Controller"
      r_c_pstac = "r:c_pstac", "External Station Controller"
      r_c_plcc = "r:c_plcc", "External LCC Controller"
      r_qdslCtrl = "r:qdslCtrl", "Quasi-Dynamic Model"
      r_tapctrl = "r:tapctrl", "ElmTr2*, Tap Controller"
      r_p_rem = "r:p_rem", "ElmTerm,StaBar, Controlled Node"
      r_p_cub = "r:p_cub", "StaCubic,ElmBoundary, Controlled Branch (Cubicle)"
      r_cpCtrlNode = "r:cpCtrlNode", "Target Node"
      r_p_pmeas = "r:p_pmeas", "StaCubic,ElmBoundary, P measured at"
      r_pldc = "r:pldc", "StaLdc, External LDC"
      r_pVccurve = "r:pVccurve", "IntVctrlcurve, V-Control-Curve"
      r_p_rem2 = "r:p_rem2", "ElmTerm,StaBar, Controlled Node"
      r_p_cub2 = "r:p_cub2", "StaCubic,ElmBoundary, Controlled Branch (Cubicle)"
      r_cpCtrlNode2 = "r:cpCtrlNode2", "Target Node"
      r_p_pmeas2 = "r:p_pmeas2", "StaCubic,ElmBoundary, P measured at"
      r_pldc2 = "r:pldc2", "StaLdc, External LDC"
      r_pVccurve2 = "r:pVccurve2", "IntVctrlcurve, V-Control-Curve"
      r_cpFeed = "r:cpFeed", "Feeder"
      m_Ikss_bushv = "m:Ikss:bushv", "kA, Initial Short-Circuit Current"
      m_Ikss_buslv = "m:Ikss:buslv", "kA, Initial Short-Circuit Current"
      m_ikss_bushv = "m:ikss:bushv", "p.u., Initial Short-Circuit Current, Magnitude"
      m_ikss_buslv = "m:ikss:buslv", "p.u., Initial Short-Circuit Current, Magnitude"
      m_phii_bushv = "m:phii:bushv", "deg, Phase Current, Angle"
      m_phii_buslv = "m:phii:buslv", "deg, Phase Current, Angle"
      m_maxIkss_bushv = "m:maxIkss:bushv", "kA, Max. Initial Short-Circuit Current"
      m_maxIkss_buslv = "m:maxIkss:buslv", "kA, Max. Initial Short-Circuit Current"
      m_Skss_bushv = "m:Skss:bushv", "MVA, Initial Short-Circuit Power"
      m_Skss_buslv = "m:Skss:buslv", "MVA, Initial Short-Circuit Power"
      m_ip_bushv = "m:ip:bushv", "kA, Peak Short-Circuit Current"
      m_ip_buslv = "m:ip:buslv", "kA, Peak Short-Circuit Current"
      m_maxip_bushv = "m:maxip:bushv", "kA, Max. Peak Short-Circuit Current"
      m_maxip_buslv = "m:maxip:buslv", "kA, Max. Peak Short-Circuit Current"
      m_maxIb_bushv = "m:maxIb:bushv", "kA, Max. Breaking Current"
      m_maxIb_buslv = "m:maxIb:buslv", "kA, Max. Breaking Current"
      m_maxIth_bushv = "m:maxIth:bushv", "kA, Max. Thermal equivalent Short-Circuit Current"
      m_maxIth_buslv = "m:maxIth:buslv", "kA, Max. Thermal equivalent Short-Circuit Current"
      m_Ib_bushv = "m:Ib:bushv", "kA, Short-Circuit Breaking Current"
      m_Ib_buslv = "m:Ib:buslv", "kA, Short-Circuit Breaking Current"
      m_Sb_bushv = "m:Sb:bushv", "MVA, Short-Circuit Breaking Power"
      m_Sb_buslv = "m:Sb:buslv", "MVA, Short-Circuit Breaking Power"
      m_Ik_bushv = "m:Ik:bushv", "kA, Steady-State Short-Circuit Current"
      m_Ik_buslv = "m:Ik:buslv", "kA, Steady-State Short-Circuit Current"
      m_Ith_bushv = "m:Ith:bushv", "kA, Thermal equivalent Short-Circuit Current"
      m_Ith_buslv = "m:Ith:buslv", "kA, Thermal equivalent Short-Circuit Current"
      m_I_bushv = "m:I:bushv", "kA, Short-Circuit Current"
      m_I_buslv = "m:I:buslv", "kA, Short-Circuit Current"
      m_R_bushv = "m:R:bushv", "Ohm, Fault Impedance, Real Part"
      m_R_buslv = "m:R:buslv", "Ohm, Fault Impedance, Real Part"
      m_X_bushv = "m:X:bushv", "Ohm, Fault Impedance, Imaginary Part"
      m_X_buslv = "m:X:buslv", "Ohm, Fault Impedance, Imaginary Part"
      m_Z_bushv = "m:Z:bushv", "Ohm, Fault Impedance, Magnitude"
      m_Z_buslv = "m:Z:buslv", "Ohm, Fault Impedance, Magnitude"
      m_phiz_bushv = "m:phiz:bushv", "deg, Fault Impedance, Angle"
      m_phiz_buslv = "m:phiz:buslv", "deg, Fault Impedance, Angle"
      m_zSbase_bushv = "m:zSbase:bushv", "p.u., Fault Impedance, Magnitude"
      m_zSbase_buslv = "m:zSbase:buslv", "p.u., Fault Impedance, Magnitude"
      m_XtoR_bushv = "m:XtoR:bushv", "X/R ratio"
      m_XtoR_buslv = "m:XtoR:buslv", "X/R ratio"
      m_Zs_bushv = "m:Zs:bushv", "Ohm, Source Impedance, Magnitude"
      m_Zs_buslv = "m:Zs:buslv", "Ohm, Source Impedance, Magnitude"
      m_phizs_bushv = "m:phizs:bushv", "deg, Source Impedance, Angle"
      m_phizs_buslv = "m:phizs:buslv", "deg, Source Impedance, Angle"
      m_Rs_bushv = "m:Rs:bushv", "Ohm, Source Impedance, Real Part"
      m_Rs_buslv = "m:Rs:buslv", "Ohm, Source Impedance, Real Part"
      m_Xs_bushv = "m:Xs:bushv", "Ohm, Source Impedance, Imaginary Part"
      m_Xs_buslv = "m:Xs:buslv", "Ohm, Source Impedance, Imaginary Part"
      m_phiui_bushv = "m:phiui:bushv", "deg, Angle between Voltage and Current"
      m_phiui_buslv = "m:phiui:buslv", "deg, Angle between Voltage and Current"
      m_cosphiui_bushv = "m:cosphiui:bushv", "cosphi"
      m_cosphiui_buslv = "m:cosphiui:buslv", "cosphi"
      m_P_bushv = "m:P:bushv", "MW, Active Power"
      m_P_buslv = "m:P:buslv", "MW, Active Power"
      m_Q_bushv = "m:Q:bushv", "Mvar, Reactive Power"
      m_Q_buslv = "m:Q:buslv", "Mvar, Reactive Power"
      m_BrkIpload_bushv = "m:BrkIpload:bushv", "%, Breaker Loading (peak)"
      m_BrkIpload_buslv = "m:BrkIpload:buslv", "%, Breaker Loading (peak)"
      m_BrkIbload_bushv = "m:BrkIbload:bushv", "%, Breaker Loading (breaking)"
      m_BrkIbload_buslv = "m:BrkIbload:buslv", "%, Breaker Loading (breaking)"
      m_BrkmaxIb_bushv = "m:BrkmaxIb:bushv", "kA, Max. Breaking Current (breaker time)"
      m_BrkmaxIb_buslv = "m:BrkmaxIb:buslv", "kA, Max. Breaking Current (breaker time)"
      n_Ikss_bushv = "n:Ikss:bushv", "kA, Initial Short-Circuit Current"
      n_Ikss_buslv = "n:Ikss:buslv", "kA, Initial Short-Circuit Current"
      n_I_bushv = "n:I:bushv", "kA, Short-Circuit Current"
      n_I_buslv = "n:I:buslv", "kA, Short-Circuit Current"
      n_phii_bushv = "n:phii:bushv", "deg, Phase Current, Angle"
      n_phii_buslv = "n:phii:buslv", "deg, Phase Current, Angle"
      n_Skss_bushv = "n:Skss:bushv", "MVA, Initial Short-Circuit Power"
      n_Skss_buslv = "n:Skss:buslv", "MVA, Initial Short-Circuit Power"
      n_ip_bushv = "n:ip:bushv", "kA, Peak Short-Circuit Current"
      n_ip_buslv = "n:ip:buslv", "kA, Peak Short-Circuit Current"
      n_Ib_bushv = "n:Ib:bushv", "kA, Short-Circuit Breaking Current"
      n_Ib_buslv = "n:Ib:buslv", "kA, Short-Circuit Breaking Current"
      n_Sb_bushv = "n:Sb:bushv", "MVA, Short-Circuit Breaking Power"
      n_Sb_buslv = "n:Sb:buslv", "MVA, Short-Circuit Breaking Power"
      n_Ik_bushv = "n:Ik:bushv", "kA, Steady-State Short-Circuit Current"
      n_Ik_buslv = "n:Ik:buslv", "kA, Steady-State Short-Circuit Current"
      n_Ith_bushv = "n:Ith:bushv", "kA, Thermal equivalent Short-Circuit Current"
      n_Ith_buslv = "n:Ith:buslv", "kA, Thermal equivalent Short-Circuit Current"
      n_R_bushv = "n:R:bushv", "Ohm, Short-Circuit Impedance, Real Part"
      n_R_buslv = "n:R:buslv", "Ohm, Short-Circuit Impedance, Real Part"
      n_X_bushv = "n:X:bushv", "Ohm, Short-Circuit Impedance, Imaginary Part"
      n_X_buslv = "n:X:buslv", "Ohm, Short-Circuit Impedance, Imaginary Part"
      n_Z_bushv = "n:Z:bushv", "Ohm, Short-Circuit Impedance, Magnitude"
      n_Z_buslv = "n:Z:buslv", "Ohm, Short-Circuit Impedance, Magnitude"
      n_phiz_bushv = "n:phiz:bushv", "deg, Short-Circuit Impedance, Angle"
      n_phiz_buslv = "n:phiz:buslv", "deg, Short-Circuit Impedance, Angle"
      n_rSbase_bushv = "n:rSbase:bushv", "p.u., Short-Circuit Impedance, Real Part"
      n_rSbase_buslv = "n:rSbase:buslv", "p.u., Short-Circuit Impedance, Real Part"
      n_xSbase_bushv = "n:xSbase:bushv", "p.u., Short-Circuit Impedance, Imaginary Part"
      n_xSbase_buslv = "n:xSbase:buslv", "p.u., Short-Circuit Impedance, Imaginary Part"
      n_zSbase_bushv = "n:zSbase:bushv", "p.u., Short-Circuit Impedance, Magnitude"
      n_zSbase_buslv = "n:zSbase:buslv", "p.u., Short-Circuit Impedance, Magnitude"
      n_XtoR_bushv = "n:XtoR:bushv", "X/R ratio (peak)"
      n_XtoR_buslv = "n:XtoR:buslv", "X/R ratio (peak)"
      n_RtoX_bushv = "n:RtoX:bushv", "R/X ratio (peak)"
      n_RtoX_buslv = "n:RtoX:buslv", "R/X ratio (peak)"
      n_ur_bushv = "n:ur:bushv", "p.u., Voltage, Real Part"
      n_ur_buslv = "n:ur:buslv", "p.u., Voltage, Real Part"
      n_ui_bushv = "n:ui:bushv", "p.u., Voltage, Imaginary Part"
      n_ui_buslv = "n:ui:buslv", "p.u., Voltage, Imaginary Part"
      n_u_bushv = "n:u:bushv", "p.u., Voltage, Magnitude"
      n_u_buslv = "n:u:buslv", "p.u., Voltage, Magnitude"
      n_upc_bushv = "n:upc:bushv", "%, Voltage, Magnitude"
      n_upc_buslv = "n:upc:buslv", "%, Voltage, Magnitude"
      n_U_bushv = "n:U:bushv", "kV, Line-Ground Voltage, Magnitude"
      n_U_buslv = "n:U:buslv", "kV, Line-Ground Voltage, Magnitude"
      n_Ul_bushv = "n:Ul:bushv", "kV, Line-Line Voltage, Magnitude"
      n_Ul_buslv = "n:Ul:buslv", "kV, Line-Line Voltage, Magnitude"
      n_phiu_bushv = "n:phiu:bushv", "deg, Voltage, Angle"
      n_phiu_buslv = "n:phiu:buslv", "deg, Voltage, Angle"
      n_du_bushv = "n:du:bushv", "%, Voltage Deviation"
      n_du_buslv = "n:du:buslv", "%, Voltage Deviation"
      n_urpref_bushv = "n:urpref:bushv", "p.u., Pre-fault Voltage, Real Part"
      n_urpref_buslv = "n:urpref:buslv", "p.u., Pre-fault Voltage, Real Part"
      n_uipref_bushv = "n:uipref:bushv", "p.u., Pre-fault Voltage, Imaginary Part"
      n_uipref_buslv = "n:uipref:buslv", "p.u., Pre-fault Voltage, Imaginary Part"
      n_upref_bushv = "n:upref:bushv", "p.u., Pre-fault Voltage, Magnitude"
      n_upref_buslv = "n:upref:buslv", "p.u., Pre-fault Voltage, Magnitude"
      n_uprefpc_bushv = "n:uprefpc:bushv", "%, Pre-fault Voltage, Magnitude"
      n_uprefpc_buslv = "n:uprefpc:buslv", "%, Pre-fault Voltage, Magnitude"
      n_Upref_bushv = "n:Upref:bushv", "kV, Line-Ground Pre-fault Voltage, Magnitude"
      n_Upref_buslv = "n:Upref:buslv", "kV, Line-Ground Pre-fault Voltage, Magnitude"
      n_Ulpref_bushv = "n:Ulpref:bushv", "kV, Line-Line Pre-fault Voltage, Magnitude"
      n_Ulpref_buslv = "n:Ulpref:buslv", "kV, Line-Line Pre-fault Voltage, Magnitude"
      n_phiupref_bushv = "n:phiupref:bushv", "deg, Pre-fault Voltage, Angle"
      n_phiupref_buslv = "n:phiupref:buslv", "deg, Pre-fault Voltage, Angle"
      n_phiui_bushv = "n:phiui:bushv", "deg, Angle between Voltage and Current"
      n_phiui_buslv = "n:phiui:buslv", "deg, Angle between Voltage and Current"
      n_cfac_bushv = "n:cfac:bushv", "Voltage Factor"
      n_cfac_buslv = "n:cfac:buslv", "Voltage Factor"
      n_idc_bushv = "n:idc:bushv", "kA, Decaying Component"
      n_idc_buslv = "n:idc:buslv", "kA, Decaying Component"
      n_Iasy_bushv = "n:Iasy:bushv", "kA, Asymmetrical Breaking Current"
      n_Iasy_buslv = "n:Iasy:buslv", "kA, Asymmetrical Breaking Current"
      n_Ithload_bushv = "n:Ithload:bushv", "%, Loading, Thermal equivalent Current"
      n_Ithload_buslv = "n:Ithload:buslv", "%, Loading, Thermal equivalent Current"
      n_Ipload_bushv = "n:Ipload:bushv", "%, Loading, Peak Short-Circuit Current"
      n_Ipload_buslv = "n:Ipload:buslv", "%, Loading, Peak Short-Circuit Current"
      n_Ithrtk_bushv = "n:Ithrtk:bushv", "kA, Rated Short-Time Current (Tk)"
      n_Ithrtk_buslv = "n:Ithrtk:buslv", "kA, Rated Short-Time Current (Tk)"
      n_Tn_bushv = "n:Tn:bushv", "ms, Grid Time Constant"
      n_Tn_buslv = "n:Tn:buslv", "ms, Grid Time Constant"
      n_XtoR_b_bushv = "n:XtoR_b:bushv", "X/R ratio (dc)"
      n_XtoR_b_buslv = "n:XtoR_b:buslv", "X/R ratio (dc)"
      n_RtoX_b_bushv = "n:RtoX_b:bushv", "R/X ratio (dc)"
      n_RtoX_b_buslv = "n:RtoX_b:buslv", "R/X ratio (dc)"
      n_fctof_b_bushv = "n:fctof_b:bushv", "Equivalent Frequency fc/f (dc)"
      n_fctof_b_buslv = "n:fctof_b:buslv", "Equivalent Frequency fc/f (dc)"
      c_xs1 = "c:xs1", "p.u., Leakage Reactance"
      c_oltc = "c:oltc", "On-load Tap Changer"
      c_Ubqmin_hv = "c:Ubqmin_hv", "kV, Minimum Operating Voltage"
      c_ilt_op = "c:ilt_op", "Long-term operating condition before short-circuit are known"
      c_pT = "c:pT", "%, Range of Voltage Adjustment"
      e_iblock = "e:iblock", "Unit Transformer"
      e_pT = "e:pT", "Voltage Range"
      e_ilt_op = "e:ilt_op", "Long-term operating conditions before short-circuit are known"
      e_Ub_lv = "e:Ub_lv", "kV, Highest Operating Voltage"
      e_Ib_lv = "e:Ib_lv", "kA, Highest Operating Current"
      e_cosphib_lv = "e:cosphib_lv", "Power factor"
      e_Ubqmin_hv = "e:Ubqmin_hv", "kV, Minimum Operating Voltage"
      t_oltc = "t:oltc", "On-load Tap Changer"
      t_pT = "t:pT", "Voltage Range"
      m_Ikss_bushv_A = "m:Ikss:bushv:A", "kA, Initial Short-Circuit Current"
      m_Ikss_bushv_B = "m:Ikss:bushv:B", "kA, Initial Short-Circuit Current"
      m_Ikss_bushv_C = "m:Ikss:bushv:C", "kA, Initial Short-Circuit Current"
      m_Ikss_bushv_N = "m:Ikss:bushv:N", "kA, Initial Short-Circuit Current"
      m_Ikss_buslv_A = "m:Ikss:buslv:A", "kA, Initial Short-Circuit Current"
      m_Ikss_buslv_B = "m:Ikss:buslv:B", "kA, Initial Short-Circuit Current"
      m_Ikss_buslv_C = "m:Ikss:buslv:C", "kA, Initial Short-Circuit Current"
      m_Ikss_buslv_N = "m:Ikss:buslv:N", "kA, Initial Short-Circuit Current"
      m_Ikss_bushvn = "m:Ikss:bushvn", "kA, Initial Short-Circuit Current"
      m_Ikss_buslvn = "m:Ikss:buslvn", "kA, Initial Short-Circuit Current"
      m_ikss_bushv_A = "m:ikss:bushv:A", "p.u., Initial Short-Circuit Current, Magnitude"
      m_ikss_bushv_B = "m:ikss:bushv:B", "p.u., Initial Short-Circuit Current, Magnitude"
      m_ikss_bushv_C = "m:ikss:bushv:C", "p.u., Initial Short-Circuit Current, Magnitude"
      m_ikss_bushv_N = "m:ikss:bushv:N", "p.u., Initial Short-Circuit Current, Magnitude"
      m_ikss_buslv_A = "m:ikss:buslv:A", "p.u., Initial Short-Circuit Current, Magnitude"
      m_ikss_buslv_B = "m:ikss:buslv:B", "p.u., Initial Short-Circuit Current, Magnitude"
      m_ikss_buslv_C = "m:ikss:buslv:C", "p.u., Initial Short-Circuit Current, Magnitude"
      m_ikss_buslv_N = "m:ikss:buslv:N", "p.u., Initial Short-Circuit Current, Magnitude"
      m_ikss_bushvn = "m:ikss:bushvn", "p.u., Initial Short-Circuit Current, Magnitude"
      m_ikss_buslvn = "m:ikss:buslvn", "p.u., Initial Short-Circuit Current, Magnitude"
      m_maxIkss_bushvn = "m:maxIkss:bushvn", "kA, Max. Initial Short-Circuit Current"
      m_maxIkss_buslvn = "m:maxIkss:buslvn", "kA, Max. Initial Short-Circuit Current"
      m_Skss_bushv_A = "m:Skss:bushv:A", "MVA, Initial Short-Circuit Power"
      m_Skss_bushv_B = "m:Skss:bushv:B", "MVA, Initial Short-Circuit Power"
      m_Skss_bushv_C = "m:Skss:bushv:C", "MVA, Initial Short-Circuit Power"
      m_Skss_bushv_N = "m:Skss:bushv:N", "MVA, Initial Short-Circuit Power"
      m_Skss_buslv_A = "m:Skss:buslv:A", "MVA, Initial Short-Circuit Power"
      m_Skss_buslv_B = "m:Skss:buslv:B", "MVA, Initial Short-Circuit Power"
      m_Skss_buslv_C = "m:Skss:buslv:C", "MVA, Initial Short-Circuit Power"
      m_Skss_buslv_N = "m:Skss:buslv:N", "MVA, Initial Short-Circuit Power"
      m_Skss_bushvn = "m:Skss:bushvn", "MVA, Initial Short-Circuit Power"
      m_Skss_buslvn = "m:Skss:buslvn", "MVA, Initial Short-Circuit Power"
      m_Skssx3_bushv_A = "m:Skssx3:bushv:A", "MVA, 3*Initial Short-Circuit Power"
      m_Skssx3_bushv_B = "m:Skssx3:bushv:B", "MVA, 3*Initial Short-Circuit Power"
      m_Skssx3_bushv_C = "m:Skssx3:bushv:C", "MVA, 3*Initial Short-Circuit Power"
      m_Skssx3_bushv_N = "m:Skssx3:bushv:N", "MVA, 3*Initial Short-Circuit Power"
      m_Skssx3_buslv_A = "m:Skssx3:buslv:A", "MVA, 3*Initial Short-Circuit Power"
      m_Skssx3_buslv_B = "m:Skssx3:buslv:B", "MVA, 3*Initial Short-Circuit Power"
      m_Skssx3_buslv_C = "m:Skssx3:buslv:C", "MVA, 3*Initial Short-Circuit Power"
      m_Skssx3_buslv_N = "m:Skssx3:buslv:N", "MVA, 3*Initial Short-Circuit Power"
      m_Skssx3_bushvn = "m:Skssx3:bushvn", "MVA, 3*Initial Short-Circuit Power"
      m_Skssx3_buslvn = "m:Skssx3:buslvn", "MVA, 3*Initial Short-Circuit Power"
      m_ip_bushv_A = "m:ip:bushv:A", "kA, Peak Short-Circuit Current"
      m_ip_bushv_B = "m:ip:bushv:B", "kA, Peak Short-Circuit Current"
      m_ip_bushv_C = "m:ip:bushv:C", "kA, Peak Short-Circuit Current"
      m_ip_bushv_N = "m:ip:bushv:N", "kA, Peak Short-Circuit Current"
      m_ip_buslv_A = "m:ip:buslv:A", "kA, Peak Short-Circuit Current"
      m_ip_buslv_B = "m:ip:buslv:B", "kA, Peak Short-Circuit Current"
      m_ip_buslv_C = "m:ip:buslv:C", "kA, Peak Short-Circuit Current"
      m_ip_buslv_N = "m:ip:buslv:N", "kA, Peak Short-Circuit Current"
      m_ip_bushvn = "m:ip:bushvn", "kA, Peak Short-Circuit Current"
      m_ip_buslvn = "m:ip:buslvn", "kA, Peak Short-Circuit Current"
      m_maxip_bushvn = "m:maxip:bushvn", "kA, Max. Peak Short-Circuit Current"
      m_maxip_buslvn = "m:maxip:buslvn", "kA, Max. Peak Short-Circuit Current"
      m_maxIth_bushvn = "m:maxIth:bushvn", "kA, Max. Thermal equivalent Short-Circuit Current"
      m_maxIth_buslvn = "m:maxIth:buslvn", "kA, Max. Thermal equivalent Short-Circuit Current"
      m_Z_bushv_A = "m:Z:bushv:A", "Ohm, Fault Impedance, Magnitude"
      m_Z_bushv_B = "m:Z:bushv:B", "Ohm, Fault Impedance, Magnitude"
      m_Z_bushv_C = "m:Z:bushv:C", "Ohm, Fault Impedance, Magnitude"
      m_Z_bushv_N = "m:Z:bushv:N", "Ohm, Fault Impedance, Magnitude"
      m_Z_buslv_A = "m:Z:buslv:A", "Ohm, Fault Impedance, Magnitude"
      m_Z_buslv_B = "m:Z:buslv:B", "Ohm, Fault Impedance, Magnitude"
      m_Z_buslv_C = "m:Z:buslv:C", "Ohm, Fault Impedance, Magnitude"
      m_Z_buslv_N = "m:Z:buslv:N", "Ohm, Fault Impedance, Magnitude"
      m_Z_bushvn = "m:Z:bushvn", "Ohm, Fault Impedance, Magnitude"
      m_Z_buslvn = "m:Z:buslvn", "Ohm, Fault Impedance, Magnitude"
      m_phiz_bushv_A = "m:phiz:bushv:A", "deg, Fault Impedance, Angle"
      m_phiz_bushv_B = "m:phiz:bushv:B", "deg, Fault Impedance, Angle"
      m_phiz_bushv_C = "m:phiz:bushv:C", "deg, Fault Impedance, Angle"
      m_phiz_bushv_N = "m:phiz:bushv:N", "deg, Fault Impedance, Angle"
      m_phiz_buslv_A = "m:phiz:buslv:A", "deg, Fault Impedance, Angle"
      m_phiz_buslv_B = "m:phiz:buslv:B", "deg, Fault Impedance, Angle"
      m_phiz_buslv_C = "m:phiz:buslv:C", "deg, Fault Impedance, Angle"
      m_phiz_buslv_N = "m:phiz:buslv:N", "deg, Fault Impedance, Angle"
      m_phiz_bushvn = "m:phiz:bushvn", "deg, Fault Impedance, Angle"
      m_phiz_buslvn = "m:phiz:buslvn", "deg, Fault Impedance, Angle"
      m_Zl_bushv_A = "m:Zl:bushv:A", "Ohm, Line-Line Fault Impedance, Magnitude"
      m_Zl_bushv_B = "m:Zl:bushv:B", "Ohm, Line-Line Fault Impedance, Magnitude"
      m_Zl_bushv_C = "m:Zl:bushv:C", "Ohm, Line-Line Fault Impedance, Magnitude"
      m_Zl_bushv_N = "m:Zl:bushv:N", "Ohm, Line-Line Fault Impedance, Magnitude"
      m_Zl_buslv_A = "m:Zl:buslv:A", "Ohm, Line-Line Fault Impedance, Magnitude"
      m_Zl_buslv_B = "m:Zl:buslv:B", "Ohm, Line-Line Fault Impedance, Magnitude"
      m_Zl_buslv_C = "m:Zl:buslv:C", "Ohm, Line-Line Fault Impedance, Magnitude"
      m_Zl_buslv_N = "m:Zl:buslv:N", "Ohm, Line-Line Fault Impedance, Magnitude"
      m_Zl_bushvn = "m:Zl:bushvn", "Ohm, Line-Line Fault Impedance, Magnitude"
      m_Zl_buslvn = "m:Zl:buslvn", "Ohm, Line-Line Fault Impedance, Magnitude"
      m_phizl_bushv_A = "m:phizl:bushv:A", "deg, Line-Line Fault Impedance, Angle"
      m_phizl_bushv_B = "m:phizl:bushv:B", "deg, Line-Line Fault Impedance, Angle"
      m_phizl_bushv_C = "m:phizl:bushv:C", "deg, Line-Line Fault Impedance, Angle"
      m_phizl_bushv_N = "m:phizl:bushv:N", "deg, Line-Line Fault Impedance, Angle"
      m_phizl_buslv_A = "m:phizl:buslv:A", "deg, Line-Line Fault Impedance, Angle"
      m_phizl_buslv_B = "m:phizl:buslv:B", "deg, Line-Line Fault Impedance, Angle"
      m_phizl_buslv_C = "m:phizl:buslv:C", "deg, Line-Line Fault Impedance, Angle"
      m_phizl_buslv_N = "m:phizl:buslv:N", "deg, Line-Line Fault Impedance, Angle"
      m_phizl_bushvn = "m:phizl:bushvn", "deg, Line-Line Fault Impedance, Angle"
      m_phizl_buslvn = "m:phizl:buslvn", "deg, Line-Line Fault Impedance, Angle"
      m_Z0_bushv = "m:Z0:bushv", "Ohm, Zero-Sequence Fault Impedance, Magnitude"
      m_Z0_buslv = "m:Z0:buslv", "Ohm, Zero-Sequence Fault Impedance, Magnitude"
      m_Z0_bushvn = "m:Z0:bushvn", "Ohm, Zero-Sequence Fault Impedance, Magnitude"
      m_Z0_buslvn = "m:Z0:buslvn", "Ohm, Zero-Sequence Fault Impedance, Magnitude"
      m_phiz0_bushv = "m:phiz0:bushv", "deg, Zero-Sequence Fault Impedance, Angle"
      m_phiz0_buslv = "m:phiz0:buslv", "deg, Zero-Sequence Fault Impedance, Angle"
      m_phiz0_bushvn = "m:phiz0:bushvn", "deg, Zero-Sequence Fault Impedance, Angle"
      m_phiz0_buslvn = "m:phiz0:buslvn", "deg, Zero-Sequence Fault Impedance, Angle"
      m_z0Sbase_bushv = "m:z0Sbase:bushv", "p.u., Zero-Sequence Fault Impedance, Magnitude"
      m_z0Sbase_buslv = "m:z0Sbase:buslv", "p.u., Zero-Sequence Fault Impedance, Magnitude"
      m_z0Sbase_bushvn = "m:z0Sbase:bushvn", "p.u., Zero-Sequence Fault Impedance, Magnitude"
      m_z0Sbase_buslvn = "m:z0Sbase:buslvn", "p.u., Zero-Sequence Fault Impedance, Magnitude"
      m_cosphiu1i1_bushv = "m:cosphiu1i1:bushv", "Positive-Sequence cosphi"
      m_cosphiu1i1_buslv = "m:cosphiu1i1:buslv", "Positive-Sequence cosphi"
      m_cosphiu1i1_bushvn = "m:cosphiu1i1:bushvn", "Positive-Sequence cosphi"
      m_cosphiu1i1_buslvn = "m:cosphiu1i1:buslvn", "Positive-Sequence cosphi"
      m_I0x3watt_bushv = "m:I0x3watt:bushv", "kA, Earth-Fault Residual Current"
      m_I0x3watt_buslv = "m:I0x3watt:buslv", "kA, Earth-Fault Residual Current"
      m_I0x3watt_bushvn = "m:I0x3watt:bushvn", "kA, Earth-Fault Residual Current"
      m_I0x3watt_buslvn = "m:I0x3watt:buslvn", "kA, Earth-Fault Residual Current"
      m_I0x3var_bushv = "m:I0x3var:bushv", "kA, Capacitive Earth-Fault Current"
      m_I0x3var_buslv = "m:I0x3var:buslv", "kA, Capacitive Earth-Fault Current"
      m_I0x3var_bushvn = "m:I0x3var:bushvn", "kA, Capacitive Earth-Fault Current"
      m_I0x3var_buslvn = "m:I0x3var:buslvn", "kA, Capacitive Earth-Fault Current"
      m_Q0cap_bushv = "m:Q0cap:bushv", "Mvar, Q0 capacitive"
      m_Q0cap_buslv = "m:Q0cap:buslv", "Mvar, Q0 capacitive"
      m_Q0cap_bushvn = "m:Q0cap:bushvn", "Mvar, Q0 capacitive"
      m_Q0cap_buslvn = "m:Q0cap:buslvn", "Mvar, Q0 capacitive"
      m_Zfrel_bushv = "m:Zfrel:bushv", "Ohm, Relay Impedance, Magnitude"
      m_Zfrel_buslv = "m:Zfrel:buslv", "Ohm, Relay Impedance, Magnitude"
      m_Zfrel_bushvn = "m:Zfrel:bushvn", "Ohm, Relay Impedance, Magnitude"
      m_Zfrel_buslvn = "m:Zfrel:buslvn", "Ohm, Relay Impedance, Magnitude"
      m_phizfrel_bushv = "m:phizfrel:bushv", "deg, Relay Impedance, Angle"
      m_phizfrel_buslv = "m:phizfrel:buslv", "deg, Relay Impedance, Angle"
      m_phizfrel_bushvn = "m:phizfrel:bushvn", "deg, Relay Impedance, Angle"
      m_phizfrel_buslvn = "m:phizfrel:buslvn", "deg, Relay Impedance, Angle"
      m_Rfrel_bushv = "m:Rfrel:bushv", "Ohm, Relay Impedance, Real Part"
      m_Rfrel_buslv = "m:Rfrel:buslv", "Ohm, Relay Impedance, Real Part"
      m_Rfrel_bushvn = "m:Rfrel:bushvn", "Ohm, Relay Impedance, Real Part"
      m_Rfrel_buslvn = "m:Rfrel:buslvn", "Ohm, Relay Impedance, Real Part"
      m_Xfrel_bushv = "m:Xfrel:bushv", "Ohm, Relay Impedance, Imaginary Part"
      m_Xfrel_buslv = "m:Xfrel:buslv", "Ohm, Relay Impedance, Imaginary Part"
      m_Xfrel_bushvn = "m:Xfrel:bushvn", "Ohm, Relay Impedance, Imaginary Part"
      m_Xfrel_buslvn = "m:Xfrel:buslvn", "Ohm, Relay Impedance, Imaginary Part"
      m_Zs_bushvn = "m:Zs:bushvn", "Ohm, Source Impedance, Magnitude"
      m_Zs_buslvn = "m:Zs:buslvn", "Ohm, Source Impedance, Magnitude"
      m_phizs_bushvn = "m:phizs:bushvn", "deg, Source Impedance, Angle"
      m_phizs_buslvn = "m:phizs:buslvn", "deg, Source Impedance, Angle"
      m_Rs_bushvn = "m:Rs:bushvn", "Ohm, Source Impedance, Real Part"
      m_Rs_buslvn = "m:Rs:buslvn", "Ohm, Source Impedance, Real Part"
      m_Xs_bushvn = "m:Xs:bushvn", "Ohm, Source Impedance, Imaginary Part"
      m_Xs_buslvn = "m:Xs:buslvn", "Ohm, Source Impedance, Imaginary Part"
      m_Zf_bushv = "m:Zf:bushv", "Ohm, Meas. Fault Impedance, Magnitude"
      m_Zf_buslv = "m:Zf:buslv", "Ohm, Meas. Fault Impedance, Magnitude"
      m_Zf_bushvn = "m:Zf:bushvn", "Ohm, Meas. Fault Impedance, Magnitude"
      m_Zf_buslvn = "m:Zf:buslvn", "Ohm, Meas. Fault Impedance, Magnitude"
      m_phizf_bushv = "m:phizf:bushv", "deg, Meas. Fault Impedance, Angle"
      m_phizf_buslv = "m:phizf:buslv", "deg, Meas. Fault Impedance, Angle"
      m_phizf_bushvn = "m:phizf:bushvn", "deg, Meas. Fault Impedance, Angle"
      m_phizf_buslvn = "m:phizf:buslvn", "deg, Meas. Fault Impedance, Angle"
      m_BrkIpload_bushvn = "m:BrkIpload:bushvn", "%, Max. Breaker Loading (peak)"
      m_BrkIpload_buslvn = "m:BrkIpload:buslvn", "%, Max. Breaker Loading (peak)"
      n_Ikss_bushv_A = "n:Ikss:bushv:A", "kA, Initial Short-Circuit Current"
      n_Ikss_bushv_B = "n:Ikss:bushv:B", "kA, Initial Short-Circuit Current"
      n_Ikss_bushv_C = "n:Ikss:bushv:C", "kA, Initial Short-Circuit Current"
      n_Ikss_bushv_N = "n:Ikss:bushv:N", "kA, Initial Short-Circuit Current"
      n_Ikss_buslv_A = "n:Ikss:buslv:A", "kA, Initial Short-Circuit Current"
      n_Ikss_buslv_B = "n:Ikss:buslv:B", "kA, Initial Short-Circuit Current"
      n_Ikss_buslv_C = "n:Ikss:buslv:C", "kA, Initial Short-Circuit Current"
      n_Ikss_buslv_N = "n:Ikss:buslv:N", "kA, Initial Short-Circuit Current"
      n_Ikss_bushvn = "n:Ikss:bushvn", "kA, Initial Short-Circuit Current"
      n_Ikss_buslvn = "n:Ikss:buslvn", "kA, Initial Short-Circuit Current"
      n_I_bushv_A = "n:I:bushv:A", "kA, Short-Circuit Current"
      n_I_bushv_B = "n:I:bushv:B", "kA, Short-Circuit Current"
      n_I_bushv_C = "n:I:bushv:C", "kA, Short-Circuit Current"
      n_I_bushv_N = "n:I:bushv:N", "kA, Short-Circuit Current"
      n_I_buslv_A = "n:I:buslv:A", "kA, Short-Circuit Current"
      n_I_buslv_B = "n:I:buslv:B", "kA, Short-Circuit Current"
      n_I_buslv_C = "n:I:buslv:C", "kA, Short-Circuit Current"
      n_I_buslv_N = "n:I:buslv:N", "kA, Short-Circuit Current"
      n_I_bushvn = "n:I:bushvn", "kA, Short-Circuit Current"
      n_I_buslvn = "n:I:buslvn", "kA, Short-Circuit Current"
      n_phii_bushv_A = "n:phii:bushv:A", "deg, Phase Current, Angle"
      n_phii_bushv_B = "n:phii:bushv:B", "deg, Phase Current, Angle"
      n_phii_bushv_C = "n:phii:bushv:C", "deg, Phase Current, Angle"
      n_phii_bushv_N = "n:phii:bushv:N", "deg, Phase Current, Angle"
      n_phii_buslv_A = "n:phii:buslv:A", "deg, Phase Current, Angle"
      n_phii_buslv_B = "n:phii:buslv:B", "deg, Phase Current, Angle"
      n_phii_buslv_C = "n:phii:buslv:C", "deg, Phase Current, Angle"
      n_phii_buslv_N = "n:phii:buslv:N", "deg, Phase Current, Angle"
      n_phii_bushvn = "n:phii:bushvn", "deg, Phase Current, Angle"
      n_phii_buslvn = "n:phii:buslvn", "deg, Phase Current, Angle"
      n_Skss_bushv_A = "n:Skss:bushv:A", "MVA, Initial Short-Circuit Power"
      n_Skss_bushv_B = "n:Skss:bushv:B", "MVA, Initial Short-Circuit Power"
      n_Skss_bushv_C = "n:Skss:bushv:C", "MVA, Initial Short-Circuit Power"
      n_Skss_bushv_N = "n:Skss:bushv:N", "MVA, Initial Short-Circuit Power"
      n_Skss_buslv_A = "n:Skss:buslv:A", "MVA, Initial Short-Circuit Power"
      n_Skss_buslv_B = "n:Skss:buslv:B", "MVA, Initial Short-Circuit Power"
      n_Skss_buslv_C = "n:Skss:buslv:C", "MVA, Initial Short-Circuit Power"
      n_Skss_buslv_N = "n:Skss:buslv:N", "MVA, Initial Short-Circuit Power"
      n_Skss_bushvn = "n:Skss:bushvn", "MVA, Initial Short-Circuit Power"
      n_Skss_buslvn = "n:Skss:buslvn", "MVA, Initial Short-Circuit Power"
      n_Skssx3_bushv_A = "n:Skssx3:bushv:A", "MVA, 3*Initial Short-Circuit Power"
      n_Skssx3_bushv_B = "n:Skssx3:bushv:B", "MVA, 3*Initial Short-Circuit Power"
      n_Skssx3_bushv_C = "n:Skssx3:bushv:C", "MVA, 3*Initial Short-Circuit Power"
      n_Skssx3_bushv_N = "n:Skssx3:bushv:N", "MVA, 3*Initial Short-Circuit Power"
      n_Skssx3_buslv_A = "n:Skssx3:buslv:A", "MVA, 3*Initial Short-Circuit Power"
      n_Skssx3_buslv_B = "n:Skssx3:buslv:B", "MVA, 3*Initial Short-Circuit Power"
      n_Skssx3_buslv_C = "n:Skssx3:buslv:C", "MVA, 3*Initial Short-Circuit Power"
      n_Skssx3_buslv_N = "n:Skssx3:buslv:N", "MVA, 3*Initial Short-Circuit Power"
      n_Skssx3_bushvn = "n:Skssx3:bushvn", "MVA, 3*Initial Short-Circuit Power"
      n_Skssx3_buslvn = "n:Skssx3:buslvn", "MVA, 3*Initial Short-Circuit Power"
      n_ip_bushv_A = "n:ip:bushv:A", "kA, Peak Short-Circuit Current"
      n_ip_bushv_B = "n:ip:bushv:B", "kA, Peak Short-Circuit Current"
      n_ip_bushv_C = "n:ip:bushv:C", "kA, Peak Short-Circuit Current"
      n_ip_bushv_N = "n:ip:bushv:N", "kA, Peak Short-Circuit Current"
      n_ip_buslv_A = "n:ip:buslv:A", "kA, Peak Short-Circuit Current"
      n_ip_buslv_B = "n:ip:buslv:B", "kA, Peak Short-Circuit Current"
      n_ip_buslv_C = "n:ip:buslv:C", "kA, Peak Short-Circuit Current"
      n_ip_buslv_N = "n:ip:buslv:N", "kA, Peak Short-Circuit Current"
      n_ip_bushvn = "n:ip:bushvn", "kA, Peak Short-Circuit Current"
      n_ip_buslvn = "n:ip:buslvn", "kA, Peak Short-Circuit Current"
      n_Ib_bushv_A = "n:Ib:bushv:A", "kA, Short-Circuit Breaking Current"
      n_Ib_bushv_B = "n:Ib:bushv:B", "kA, Short-Circuit Breaking Current"
      n_Ib_bushv_C = "n:Ib:bushv:C", "kA, Short-Circuit Breaking Current"
      n_Ib_bushv_N = "n:Ib:bushv:N", "kA, Short-Circuit Breaking Current"
      n_Ib_buslv_A = "n:Ib:buslv:A", "kA, Short-Circuit Breaking Current"
      n_Ib_buslv_B = "n:Ib:buslv:B", "kA, Short-Circuit Breaking Current"
      n_Ib_buslv_C = "n:Ib:buslv:C", "kA, Short-Circuit Breaking Current"
      n_Ib_buslv_N = "n:Ib:buslv:N", "kA, Short-Circuit Breaking Current"
      n_Ib_bushvn = "n:Ib:bushvn", "kA, Short-Circuit Breaking Current"
      n_Ib_buslvn = "n:Ib:buslvn", "kA, Short-Circuit Breaking Current"
      n_Sb_bushv_A = "n:Sb:bushv:A", "MVA, Short-Circuit Breaking Power"
      n_Sb_bushv_B = "n:Sb:bushv:B", "MVA, Short-Circuit Breaking Power"
      n_Sb_bushv_C = "n:Sb:bushv:C", "MVA, Short-Circuit Breaking Power"
      n_Sb_bushv_N = "n:Sb:bushv:N", "MVA, Short-Circuit Breaking Power"
      n_Sb_buslv_A = "n:Sb:buslv:A", "MVA, Short-Circuit Breaking Power"
      n_Sb_buslv_B = "n:Sb:buslv:B", "MVA, Short-Circuit Breaking Power"
      n_Sb_buslv_C = "n:Sb:buslv:C", "MVA, Short-Circuit Breaking Power"
      n_Sb_buslv_N = "n:Sb:buslv:N", "MVA, Short-Circuit Breaking Power"
      n_Sb_bushvn = "n:Sb:bushvn", "MVA, Short-Circuit Breaking Power"
      n_Sb_buslvn = "n:Sb:buslvn", "MVA, Short-Circuit Breaking Power"
      n_Ith_bushv_A = "n:Ith:bushv:A", "kA, Thermal equivalent Short-Circuit Current"
      n_Ith_bushv_B = "n:Ith:bushv:B", "kA, Thermal equivalent Short-Circuit Current"
      n_Ith_bushv_C = "n:Ith:bushv:C", "kA, Thermal equivalent Short-Circuit Current"
      n_Ith_bushv_N = "n:Ith:bushv:N", "kA, Thermal equivalent Short-Circuit Current"
      n_Ith_buslv_A = "n:Ith:buslv:A", "kA, Thermal equivalent Short-Circuit Current"
      n_Ith_buslv_B = "n:Ith:buslv:B", "kA, Thermal equivalent Short-Circuit Current"
      n_Ith_buslv_C = "n:Ith:buslv:C", "kA, Thermal equivalent Short-Circuit Current"
      n_Ith_buslv_N = "n:Ith:buslv:N", "kA, Thermal equivalent Short-Circuit Current"
      n_Ith_bushvn = "n:Ith:bushvn", "kA, Thermal equivalent Short-Circuit Current"
      n_Ith_buslvn = "n:Ith:buslvn", "kA, Thermal equivalent Short-Circuit Current"
      n_Ithmax_bushv = "n:Ithmax:bushv", "kA, Max. Thermal equivalent Short-Circuit Current"
      n_Ithmax_buslv = "n:Ithmax:buslv", "kA, Max. Thermal equivalent Short-Circuit Current"
      n_Ithmax_bushvn = "n:Ithmax:bushvn", "kA, Max. Thermal equivalent Short-Circuit Current"
      n_Ithmax_buslvn = "n:Ithmax:buslvn", "kA, Max. Thermal equivalent Short-Circuit Current"
      n_Ik_bushv_A = "n:Ik:bushv:A", "kA, Steady-State Short-Circuit Current"
      n_Ik_bushv_B = "n:Ik:bushv:B", "kA, Steady-State Short-Circuit Current"
      n_Ik_bushv_C = "n:Ik:bushv:C", "kA, Steady-State Short-Circuit Current"
      n_Ik_bushv_N = "n:Ik:bushv:N", "kA, Steady-State Short-Circuit Current"
      n_Ik_buslv_A = "n:Ik:buslv:A", "kA, Steady-State Short-Circuit Current"
      n_Ik_buslv_B = "n:Ik:buslv:B", "kA, Steady-State Short-Circuit Current"
      n_Ik_buslv_C = "n:Ik:buslv:C", "kA, Steady-State Short-Circuit Current"
      n_Ik_buslv_N = "n:Ik:buslv:N", "kA, Steady-State Short-Circuit Current"
      n_Ik_bushvn = "n:Ik:bushvn", "kA, Steady-State Short-Circuit Current"
      n_Ik_buslvn = "n:Ik:buslvn", "kA, Steady-State Short-Circuit Current"
      n_R0_bushv = "n:R0:bushv", "Ohm, Zero-Sequence Impedance, Real Part"
      n_R0_buslv = "n:R0:buslv", "Ohm, Zero-Sequence Impedance, Real Part"
      n_R0_bushvn = "n:R0:bushvn", "Ohm, Zero-Sequence Impedance, Real Part"
      n_R0_buslvn = "n:R0:buslvn", "Ohm, Zero-Sequence Impedance, Real Part"
      n_X0_bushv = "n:X0:bushv", "Ohm, Zero-Sequence Impedance, Imaginary Part"
      n_X0_buslv = "n:X0:buslv", "Ohm, Zero-Sequence Impedance, Imaginary Part"
      n_X0_bushvn = "n:X0:bushvn", "Ohm, Zero-Sequence Impedance, Imaginary Part"
      n_X0_buslvn = "n:X0:buslvn", "Ohm, Zero-Sequence Impedance, Imaginary Part"
      n_Z0_bushv = "n:Z0:bushv", "Ohm, Zero-Sequence Impedance, Magnitude"
      n_Z0_buslv = "n:Z0:buslv", "Ohm, Zero-Sequence Impedance, Magnitude"
      n_Z0_bushvn = "n:Z0:bushvn", "Ohm, Zero-Sequence Impedance, Magnitude"
      n_Z0_buslvn = "n:Z0:buslvn", "Ohm, Zero-Sequence Impedance, Magnitude"
      n_phiz0_bushv = "n:phiz0:bushv", "deg, Zero-Sequence Impedance, Angle"
      n_phiz0_buslv = "n:phiz0:buslv", "deg, Zero-Sequence Impedance, Angle"
      n_phiz0_bushvn = "n:phiz0:bushvn", "deg, Zero-Sequence Impedance, Angle"
      n_phiz0_buslvn = "n:phiz0:buslvn", "deg, Zero-Sequence Impedance, Angle"
      n_r0Sbase_bushv = "n:r0Sbase:bushv", "p.u., Zero-Sequence Impedance, Real Part"
      n_r0Sbase_buslv = "n:r0Sbase:buslv", "p.u., Zero-Sequence Impedance, Real Part"
      n_r0Sbase_bushvn = "n:r0Sbase:bushvn", "p.u., Zero-Sequence Impedance, Real Part"
      n_r0Sbase_buslvn = "n:r0Sbase:buslvn", "p.u., Zero-Sequence Impedance, Real Part"
      n_x0Sbase_bushv = "n:x0Sbase:bushv", "p.u., Zero-Sequence Impedance, Imaginary Part"
      n_x0Sbase_buslv = "n:x0Sbase:buslv", "p.u., Zero-Sequence Impedance, Imaginary Part"
      n_x0Sbase_bushvn = "n:x0Sbase:bushvn", "p.u., Zero-Sequence Impedance, Imaginary Part"
      n_x0Sbase_buslvn = "n:x0Sbase:buslvn", "p.u., Zero-Sequence Impedance, Imaginary Part"
      n_z0Sbase_bushv = "n:z0Sbase:bushv", "p.u., Zero-Sequence Impedance, Magnitude"
      n_z0Sbase_buslv = "n:z0Sbase:buslv", "p.u., Zero-Sequence Impedance, Magnitude"
      n_z0Sbase_bushvn = "n:z0Sbase:bushvn", "p.u., Zero-Sequence Impedance, Magnitude"
      n_z0Sbase_buslvn = "n:z0Sbase:buslvn", "p.u., Zero-Sequence Impedance, Magnitude"
      n_R1_bushv = "n:R1:bushv", "Ohm, Positive-Sequence Impedance, Real Part"
      n_R1_buslv = "n:R1:buslv", "Ohm, Positive-Sequence Impedance, Real Part"
      n_R1_bushvn = "n:R1:bushvn", "Ohm, Positive-Sequence Impedance, Real Part"
      n_R1_buslvn = "n:R1:buslvn", "Ohm, Positive-Sequence Impedance, Real Part"
      n_X1_bushv = "n:X1:bushv", "Ohm, Positive-Sequence Impedance, Imaginary Part"
      n_X1_buslv = "n:X1:buslv", "Ohm, Positive-Sequence Impedance, Imaginary Part"
      n_X1_bushvn = "n:X1:bushvn", "Ohm, Positive-Sequence Impedance, Imaginary Part"
      n_X1_buslvn = "n:X1:buslvn", "Ohm, Positive-Sequence Impedance, Imaginary Part"
      n_Z1_bushv = "n:Z1:bushv", "Ohm, Positive-Sequence Impedance, Magnitude"
      n_Z1_buslv = "n:Z1:buslv", "Ohm, Positive-Sequence Impedance, Magnitude"
      n_Z1_bushvn = "n:Z1:bushvn", "Ohm, Positive-Sequence Impedance, Magnitude"
      n_Z1_buslvn = "n:Z1:buslvn", "Ohm, Positive-Sequence Impedance, Magnitude"
      n_phiz1_bushv = "n:phiz1:bushv", "deg, Positive-Sequence Impedance, Angle"
      n_phiz1_buslv = "n:phiz1:buslv", "deg, Positive-Sequence Impedance, Angle"
      n_phiz1_bushvn = "n:phiz1:bushvn", "deg, Positive-Sequence Impedance, Angle"
      n_phiz1_buslvn = "n:phiz1:buslvn", "deg, Positive-Sequence Impedance, Angle"
      n_r1Sbase_bushv = "n:r1Sbase:bushv", "p.u., Positive-Sequence Impedance, Real Part"
      n_r1Sbase_buslv = "n:r1Sbase:buslv", "p.u., Positive-Sequence Impedance, Real Part"
      n_r1Sbase_bushvn = "n:r1Sbase:bushvn", "p.u., Positive-Sequence Impedance, Real Part"
      n_r1Sbase_buslvn = "n:r1Sbase:buslvn", "p.u., Positive-Sequence Impedance, Real Part"
      n_x1Sbase_bushv = "n:x1Sbase:bushv", "p.u., Positive-Sequence Impedance, Imaginary Part"
      n_x1Sbase_buslv = "n:x1Sbase:buslv", "p.u., Positive-Sequence Impedance, Imaginary Part"
      n_x1Sbase_bushvn = "n:x1Sbase:bushvn", "p.u., Positive-Sequence Impedance, Imaginary Part"
      n_x1Sbase_buslvn = "n:x1Sbase:buslvn", "p.u., Positive-Sequence Impedance, Imaginary Part"
      n_z1Sbase_bushv = "n:z1Sbase:bushv", "p.u., Positive-Sequence Impedance, Magnitude"
      n_z1Sbase_buslv = "n:z1Sbase:buslv", "p.u., Positive-Sequence Impedance, Magnitude"
      n_z1Sbase_bushvn = "n:z1Sbase:bushvn", "p.u., Positive-Sequence Impedance, Magnitude"
      n_z1Sbase_buslvn = "n:z1Sbase:buslvn", "p.u., Positive-Sequence Impedance, Magnitude"
      n_R2_bushv = "n:R2:bushv", "Ohm, Negative-Sequence Impedance, Real Part"
      n_R2_buslv = "n:R2:buslv", "Ohm, Negative-Sequence Impedance, Real Part"
      n_R2_bushvn = "n:R2:bushvn", "Ohm, Negative-Sequence Impedance, Real Part"
      n_R2_buslvn = "n:R2:buslvn", "Ohm, Negative-Sequence Impedance, Real Part"
      n_X2_bushv = "n:X2:bushv", "Ohm, Negative-Sequence Impedance, Imaginary Part"
      n_X2_buslv = "n:X2:buslv", "Ohm, Negative-Sequence Impedance, Imaginary Part"
      n_X2_bushvn = "n:X2:bushvn", "Ohm, Negative-Sequence Impedance, Imaginary Part"
      n_X2_buslvn = "n:X2:buslvn", "Ohm, Negative-Sequence Impedance, Imaginary Part"
      n_Z2_bushv = "n:Z2:bushv", "Ohm, Negative-Sequence Impedance, Magnitude"
      n_Z2_buslv = "n:Z2:buslv", "Ohm, Negative-Sequence Impedance, Magnitude"
      n_Z2_bushvn = "n:Z2:bushvn", "Ohm, Negative-Sequence Impedance, Magnitude"
      n_Z2_buslvn = "n:Z2:buslvn", "Ohm, Negative-Sequence Impedance, Magnitude"
      n_phiz2_bushv = "n:phiz2:bushv", "deg, Negative-Sequence Impedance, Angle"
      n_phiz2_buslv = "n:phiz2:buslv", "deg, Negative-Sequence Impedance, Angle"
      n_phiz2_bushvn = "n:phiz2:bushvn", "deg, Negative-Sequence Impedance, Angle"
      n_phiz2_buslvn = "n:phiz2:buslvn", "deg, Negative-Sequence Impedance, Angle"
      n_r2Sbase_bushv = "n:r2Sbase:bushv", "p.u., Negative-Sequence Impedance, Real Part"
      n_r2Sbase_buslv = "n:r2Sbase:buslv", "p.u., Negative-Sequence Impedance, Real Part"
      n_r2Sbase_bushvn = "n:r2Sbase:bushvn", "p.u., Negative-Sequence Impedance, Real Part"
      n_r2Sbase_buslvn = "n:r2Sbase:buslvn", "p.u., Negative-Sequence Impedance, Real Part"
      n_x2Sbase_bushv = "n:x2Sbase:bushv", "p.u., Negative-Sequence Impedance, Imaginary Part"
      n_x2Sbase_buslv = "n:x2Sbase:buslv", "p.u., Negative-Sequence Impedance, Imaginary Part"
      n_x2Sbase_bushvn = "n:x2Sbase:bushvn", "p.u., Negative-Sequence Impedance, Imaginary Part"
      n_x2Sbase_buslvn = "n:x2Sbase:buslvn", "p.u., Negative-Sequence Impedance, Imaginary Part"
      n_z2Sbase_bushv = "n:z2Sbase:bushv", "p.u., Negative-Sequence Impedance, Magnitude"
      n_z2Sbase_buslv = "n:z2Sbase:buslv", "p.u., Negative-Sequence Impedance, Magnitude"
      n_z2Sbase_bushvn = "n:z2Sbase:bushvn", "p.u., Negative-Sequence Impedance, Magnitude"
      n_z2Sbase_buslvn = "n:z2Sbase:buslvn", "p.u., Negative-Sequence Impedance, Magnitude"
      n_R_bushv_A = "n:R:bushv:A", "Ohm, Network Resistance, Line-Ground"
      n_R_bushv_B = "n:R:bushv:B", "Ohm, Network Resistance, Line-Ground"
      n_R_bushv_C = "n:R:bushv:C", "Ohm, Network Resistance, Line-Ground"
      n_R_bushv_N = "n:R:bushv:N", "Ohm, Network Resistance, Line-Ground"
      n_R_buslv_A = "n:R:buslv:A", "Ohm, Network Resistance, Line-Ground"
      n_R_buslv_B = "n:R:buslv:B", "Ohm, Network Resistance, Line-Ground"
      n_R_buslv_C = "n:R:buslv:C", "Ohm, Network Resistance, Line-Ground"
      n_R_buslv_N = "n:R:buslv:N", "Ohm, Network Resistance, Line-Ground"
      n_R_bushvn = "n:R:bushvn", "Ohm, Network Resistance, Line-Ground"
      n_R_buslvn = "n:R:buslvn", "Ohm, Network Resistance, Line-Ground"
      n_X_bushv_A = "n:X:bushv:A", "Ohm, Network Reactance, Line-Ground"
      n_X_bushv_B = "n:X:bushv:B", "Ohm, Network Reactance, Line-Ground"
      n_X_bushv_C = "n:X:bushv:C", "Ohm, Network Reactance, Line-Ground"
      n_X_bushv_N = "n:X:bushv:N", "Ohm, Network Reactance, Line-Ground"
      n_X_buslv_A = "n:X:buslv:A", "Ohm, Network Reactance, Line-Ground"
      n_X_buslv_B = "n:X:buslv:B", "Ohm, Network Reactance, Line-Ground"
      n_X_buslv_C = "n:X:buslv:C", "Ohm, Network Reactance, Line-Ground"
      n_X_buslv_N = "n:X:buslv:N", "Ohm, Network Reactance, Line-Ground"
      n_X_bushvn = "n:X:bushvn", "Ohm, Network Reactance, Line-Ground"
      n_X_buslvn = "n:X:buslvn", "Ohm, Network Reactance, Line-Ground"
      n_Z_bushv_A = "n:Z:bushv:A", "Ohm, Network Impedance, Line-Ground"
      n_Z_bushv_B = "n:Z:bushv:B", "Ohm, Network Impedance, Line-Ground"
      n_Z_bushv_C = "n:Z:bushv:C", "Ohm, Network Impedance, Line-Ground"
      n_Z_bushv_N = "n:Z:bushv:N", "Ohm, Network Impedance, Line-Ground"
      n_Z_buslv_A = "n:Z:buslv:A", "Ohm, Network Impedance, Line-Ground"
      n_Z_buslv_B = "n:Z:buslv:B", "Ohm, Network Impedance, Line-Ground"
      n_Z_buslv_C = "n:Z:buslv:C", "Ohm, Network Impedance, Line-Ground"
      n_Z_buslv_N = "n:Z:buslv:N", "Ohm, Network Impedance, Line-Ground"
      n_Z_bushvn = "n:Z:bushvn", "Ohm, Network Impedance, Line-Ground"
      n_Z_buslvn = "n:Z:buslvn", "Ohm, Network Impedance, Line-Ground"
      n_phiz_bushv_A = "n:phiz:bushv:A", "deg, Angle of Network Impedance, Line-Ground"
      n_phiz_bushv_B = "n:phiz:bushv:B", "deg, Angle of Network Impedance, Line-Ground"
      n_phiz_bushv_C = "n:phiz:bushv:C", "deg, Angle of Network Impedance, Line-Ground"
      n_phiz_bushv_N = "n:phiz:bushv:N", "deg, Angle of Network Impedance, Line-Ground"
      n_phiz_buslv_A = "n:phiz:buslv:A", "deg, Angle of Network Impedance, Line-Ground"
      n_phiz_buslv_B = "n:phiz:buslv:B", "deg, Angle of Network Impedance, Line-Ground"
      n_phiz_buslv_C = "n:phiz:buslv:C", "deg, Angle of Network Impedance, Line-Ground"
      n_phiz_buslv_N = "n:phiz:buslv:N", "deg, Angle of Network Impedance, Line-Ground"
      n_phiz_bushvn = "n:phiz:bushvn", "deg, Angle of Network Impedance, Line-Ground"
      n_phiz_buslvn = "n:phiz:buslvn", "deg, Angle of Network Impedance, Line-Ground"
      n_Rl_bushv_A = "n:Rl:bushv:A", "Ohm, Network Resistance, Line-Line"
      n_Rl_bushv_B = "n:Rl:bushv:B", "Ohm, Network Resistance, Line-Line"
      n_Rl_bushv_C = "n:Rl:bushv:C", "Ohm, Network Resistance, Line-Line"
      n_Rl_bushv_N = "n:Rl:bushv:N", "Ohm, Network Resistance, Line-Line"
      n_Rl_buslv_A = "n:Rl:buslv:A", "Ohm, Network Resistance, Line-Line"
      n_Rl_buslv_B = "n:Rl:buslv:B", "Ohm, Network Resistance, Line-Line"
      n_Rl_buslv_C = "n:Rl:buslv:C", "Ohm, Network Resistance, Line-Line"
      n_Rl_buslv_N = "n:Rl:buslv:N", "Ohm, Network Resistance, Line-Line"
      n_Rl_bushvn = "n:Rl:bushvn", "Ohm, Network Resistance, Line-Line"
      n_Rl_buslvn = "n:Rl:buslvn", "Ohm, Network Resistance, Line-Line"
      n_Xl_bushv_A = "n:Xl:bushv:A", "Ohm, Network Reactance, Line-Line"
      n_Xl_bushv_B = "n:Xl:bushv:B", "Ohm, Network Reactance, Line-Line"
      n_Xl_bushv_C = "n:Xl:bushv:C", "Ohm, Network Reactance, Line-Line"
      n_Xl_bushv_N = "n:Xl:bushv:N", "Ohm, Network Reactance, Line-Line"
      n_Xl_buslv_A = "n:Xl:buslv:A", "Ohm, Network Reactance, Line-Line"
      n_Xl_buslv_B = "n:Xl:buslv:B", "Ohm, Network Reactance, Line-Line"
      n_Xl_buslv_C = "n:Xl:buslv:C", "Ohm, Network Reactance, Line-Line"
      n_Xl_buslv_N = "n:Xl:buslv:N", "Ohm, Network Reactance, Line-Line"
      n_Xl_bushvn = "n:Xl:bushvn", "Ohm, Network Reactance, Line-Line"
      n_Xl_buslvn = "n:Xl:buslvn", "Ohm, Network Reactance, Line-Line"
      n_Zl_bushv_A = "n:Zl:bushv:A", "Ohm, Network Impedance, Line-Line"
      n_Zl_bushv_B = "n:Zl:bushv:B", "Ohm, Network Impedance, Line-Line"
      n_Zl_bushv_C = "n:Zl:bushv:C", "Ohm, Network Impedance, Line-Line"
      n_Zl_bushv_N = "n:Zl:bushv:N", "Ohm, Network Impedance, Line-Line"
      n_Zl_buslv_A = "n:Zl:buslv:A", "Ohm, Network Impedance, Line-Line"
      n_Zl_buslv_B = "n:Zl:buslv:B", "Ohm, Network Impedance, Line-Line"
      n_Zl_buslv_C = "n:Zl:buslv:C", "Ohm, Network Impedance, Line-Line"
      n_Zl_buslv_N = "n:Zl:buslv:N", "Ohm, Network Impedance, Line-Line"
      n_Zl_bushvn = "n:Zl:bushvn", "Ohm, Network Impedance, Line-Line"
      n_Zl_buslvn = "n:Zl:buslvn", "Ohm, Network Impedance, Line-Line"
      n_phizl_bushv_A = "n:phizl:bushv:A", "deg, Angle of Network Impedance, Line-Line"
      n_phizl_bushv_B = "n:phizl:bushv:B", "deg, Angle of Network Impedance, Line-Line"
      n_phizl_bushv_C = "n:phizl:bushv:C", "deg, Angle of Network Impedance, Line-Line"
      n_phizl_bushv_N = "n:phizl:bushv:N", "deg, Angle of Network Impedance, Line-Line"
      n_phizl_buslv_A = "n:phizl:buslv:A", "deg, Angle of Network Impedance, Line-Line"
      n_phizl_buslv_B = "n:phizl:buslv:B", "deg, Angle of Network Impedance, Line-Line"
      n_phizl_buslv_C = "n:phizl:buslv:C", "deg, Angle of Network Impedance, Line-Line"
      n_phizl_buslv_N = "n:phizl:buslv:N", "deg, Angle of Network Impedance, Line-Line"
      n_phizl_bushvn = "n:phizl:bushvn", "deg, Angle of Network Impedance, Line-Line"
      n_phizl_buslvn = "n:phizl:buslvn", "deg, Angle of Network Impedance, Line-Line"
      n_Rln_bushv_A = "n:Rln:bushv:A", "Ohm, Network Resistance, Line-Neutral"
      n_Rln_bushv_B = "n:Rln:bushv:B", "Ohm, Network Resistance, Line-Neutral"
      n_Rln_bushv_C = "n:Rln:bushv:C", "Ohm, Network Resistance, Line-Neutral"
      n_Rln_bushv_N = "n:Rln:bushv:N", "Ohm, Network Resistance, Line-Neutral"
      n_Rln_buslv_A = "n:Rln:buslv:A", "Ohm, Network Resistance, Line-Neutral"
      n_Rln_buslv_B = "n:Rln:buslv:B", "Ohm, Network Resistance, Line-Neutral"
      n_Rln_buslv_C = "n:Rln:buslv:C", "Ohm, Network Resistance, Line-Neutral"
      n_Rln_buslv_N = "n:Rln:buslv:N", "Ohm, Network Resistance, Line-Neutral"
      n_Rln_bushvn = "n:Rln:bushvn", "Ohm, Network Resistance, Line-Neutral"
      n_Rln_buslvn = "n:Rln:buslvn", "Ohm, Network Resistance, Line-Neutral"
      n_Xln_bushv_A = "n:Xln:bushv:A", "Ohm, Network Reactance, Line-Neutral"
      n_Xln_bushv_B = "n:Xln:bushv:B", "Ohm, Network Reactance, Line-Neutral"
      n_Xln_bushv_C = "n:Xln:bushv:C", "Ohm, Network Reactance, Line-Neutral"
      n_Xln_bushv_N = "n:Xln:bushv:N", "Ohm, Network Reactance, Line-Neutral"
      n_Xln_buslv_A = "n:Xln:buslv:A", "Ohm, Network Reactance, Line-Neutral"
      n_Xln_buslv_B = "n:Xln:buslv:B", "Ohm, Network Reactance, Line-Neutral"
      n_Xln_buslv_C = "n:Xln:buslv:C", "Ohm, Network Reactance, Line-Neutral"
      n_Xln_buslv_N = "n:Xln:buslv:N", "Ohm, Network Reactance, Line-Neutral"
      n_Xln_bushvn = "n:Xln:bushvn", "Ohm, Network Reactance, Line-Neutral"
      n_Xln_buslvn = "n:Xln:buslvn", "Ohm, Network Reactance, Line-Neutral"
      n_Zln_bushv_A = "n:Zln:bushv:A", "Ohm, Network Impedance, Line-Neutral"
      n_Zln_bushv_B = "n:Zln:bushv:B", "Ohm, Network Impedance, Line-Neutral"
      n_Zln_bushv_C = "n:Zln:bushv:C", "Ohm, Network Impedance, Line-Neutral"
      n_Zln_bushv_N = "n:Zln:bushv:N", "Ohm, Network Impedance, Line-Neutral"
      n_Zln_buslv_A = "n:Zln:buslv:A", "Ohm, Network Impedance, Line-Neutral"
      n_Zln_buslv_B = "n:Zln:buslv:B", "Ohm, Network Impedance, Line-Neutral"
      n_Zln_buslv_C = "n:Zln:buslv:C", "Ohm, Network Impedance, Line-Neutral"
      n_Zln_buslv_N = "n:Zln:buslv:N", "Ohm, Network Impedance, Line-Neutral"
      n_Zln_bushvn = "n:Zln:bushvn", "Ohm, Network Impedance, Line-Neutral"
      n_Zln_buslvn = "n:Zln:buslvn", "Ohm, Network Impedance, Line-Neutral"
      n_phizln_bushv_A = "n:phizln:bushv:A", "deg, Angle of Network Impedance, Line-Neutral"
      n_phizln_bushv_B = "n:phizln:bushv:B", "deg, Angle of Network Impedance, Line-Neutral"
      n_phizln_bushv_C = "n:phizln:bushv:C", "deg, Angle of Network Impedance, Line-Neutral"
      n_phizln_bushv_N = "n:phizln:bushv:N", "deg, Angle of Network Impedance, Line-Neutral"
      n_phizln_buslv_A = "n:phizln:buslv:A", "deg, Angle of Network Impedance, Line-Neutral"
      n_phizln_buslv_B = "n:phizln:buslv:B", "deg, Angle of Network Impedance, Line-Neutral"
      n_phizln_buslv_C = "n:phizln:buslv:C", "deg, Angle of Network Impedance, Line-Neutral"
      n_phizln_buslv_N = "n:phizln:buslv:N", "deg, Angle of Network Impedance, Line-Neutral"
      n_phizln_bushvn = "n:phizln:bushvn", "deg, Angle of Network Impedance, Line-Neutral"
      n_phizln_buslvn = "n:phizln:buslvn", "deg, Angle of Network Impedance, Line-Neutral"
      n_Rn_bushv = "n:Rn:bushv", "Ohm, Network Resistance, Neutral"
      n_Rn_buslv = "n:Rn:buslv", "Ohm, Network Resistance, Neutral"
      n_Rn_bushvn = "n:Rn:bushvn", "Ohm, Network Resistance, Neutral"
      n_Rn_buslvn = "n:Rn:buslvn", "Ohm, Network Resistance, Neutral"
      n_Xn_bushv = "n:Xn:bushv", "Ohm, Network Reactance, Neutral"
      n_Xn_buslv = "n:Xn:buslv", "Ohm, Network Reactance, Neutral"
      n_Xn_bushvn = "n:Xn:bushvn", "Ohm, Network Reactance, Neutral"
      n_Xn_buslvn = "n:Xn:buslvn", "Ohm, Network Reactance, Neutral"
      n_Zn_bushv = "n:Zn:bushv", "Ohm, Network Impedance, Neutral"
      n_Zn_buslv = "n:Zn:buslv", "Ohm, Network Impedance, Neutral"
      n_Zn_bushvn = "n:Zn:bushvn", "Ohm, Network Impedance, Neutral"
      n_Zn_buslvn = "n:Zn:buslvn", "Ohm, Network Impedance, Neutral"
      n_phizn_bushv = "n:phizn:bushv", "deg, Angle of Network Impedance, Neutral"
      n_phizn_buslv = "n:phizn:buslv", "deg, Angle of Network Impedance, Neutral"
      n_phizn_bushvn = "n:phizn:bushvn", "deg, Angle of Network Impedance, Neutral"
      n_phizn_buslvn = "n:phizn:buslvn", "deg, Angle of Network Impedance, Neutral"
      n_eff_bushv_A = "n:eff:bushv:A", "Earth Fault Factor"
      n_eff_bushv_B = "n:eff:bushv:B", "Earth Fault Factor"
      n_eff_bushv_C = "n:eff:bushv:C", "Earth Fault Factor"
      n_eff_bushv_N = "n:eff:bushv:N", "Earth Fault Factor"
      n_eff_buslv_A = "n:eff:buslv:A", "Earth Fault Factor"
      n_eff_buslv_B = "n:eff:buslv:B", "Earth Fault Factor"
      n_eff_buslv_C = "n:eff:buslv:C", "Earth Fault Factor"
      n_eff_buslv_N = "n:eff:buslv:N", "Earth Fault Factor"
      n_eff_bushvn = "n:eff:bushvn", "Earth Fault Factor"
      n_eff_buslvn = "n:eff:buslvn", "Earth Fault Factor"
      n_X0toX1_bushv = "n:X0toX1:bushv", "X0/X1"
      n_X0toX1_buslv = "n:X0toX1:buslv", "X0/X1"
      n_X0toX1_bushvn = "n:X0toX1:bushvn", "X0/X1"
      n_X0toX1_buslvn = "n:X0toX1:buslvn", "X0/X1"
      n_in_bushv = "n:in:bushv", "p.u., Neutral Current, Magnitude"
      n_in_buslv = "n:in:buslv", "p.u., Neutral Current, Magnitude"
      n_in_bushvn = "n:in:bushvn", "p.u., Neutral Current, Magnitude"
      n_in_buslvn = "n:in:buslvn", "p.u., Neutral Current, Magnitude"
      n_In_bushv = "n:In:bushv", "kA, Neutral Current, Magnitude"
      n_In_buslv = "n:In:buslv", "kA, Neutral Current, Magnitude"
      n_In_bushvn = "n:In:bushvn", "kA, Neutral Current, Magnitude"
      n_In_buslvn = "n:In:buslvn", "kA, Neutral Current, Magnitude"
      n_phiin_bushv = "n:phiin:bushv", "deg, Neutral Current, Angle"
      n_phiin_buslv = "n:phiin:buslv", "deg, Neutral Current, Angle"
      n_phiin_bushvn = "n:phiin:bushvn", "deg, Neutral Current, Angle"
      n_phiin_buslvn = "n:phiin:buslvn", "deg, Neutral Current, Angle"
      n_i0r_bushv = "n:i0r:bushv", "p.u., Zero-Sequence Current, Real Part"
      n_i0r_buslv = "n:i0r:buslv", "p.u., Zero-Sequence Current, Real Part"
      n_i0r_bushvn = "n:i0r:bushvn", "p.u., Zero-Sequence Current, Real Part"
      n_i0r_buslvn = "n:i0r:buslvn", "p.u., Zero-Sequence Current, Real Part"
      n_i0i_bushv = "n:i0i:bushv", "p.u., Zero-Sequence Current, Imaginary Part"
      n_i0i_buslv = "n:i0i:buslv", "p.u., Zero-Sequence Current, Imaginary Part"
      n_i0i_bushvn = "n:i0i:bushvn", "p.u., Zero-Sequence Current, Imaginary Part"
      n_i0i_buslvn = "n:i0i:buslvn", "p.u., Zero-Sequence Current, Imaginary Part"
      n_i0_bushv = "n:i0:bushv", "p.u., Zero-Sequence Current, Magnitude"
      n_i0_buslv = "n:i0:buslv", "p.u., Zero-Sequence Current, Magnitude"
      n_i0_bushvn = "n:i0:bushvn", "p.u., Zero-Sequence Current, Magnitude"
      n_i0_buslvn = "n:i0:buslvn", "p.u., Zero-Sequence Current, Magnitude"
      n_I0_bushv = "n:I0:bushv", "kA, Zero-Sequence Current, Magnitude"
      n_I0_buslv = "n:I0:buslv", "kA, Zero-Sequence Current, Magnitude"
      n_I0_bushvn = "n:I0:bushvn", "kA, Zero-Sequence Current, Magnitude"
      n_I0_buslvn = "n:I0:buslvn", "kA, Zero-Sequence Current, Magnitude"
      n_I0x3_bushv = "n:I0x3:bushv", "kA, 3*I0"
      n_I0x3_buslv = "n:I0x3:buslv", "kA, 3*I0"
      n_I0x3_bushvn = "n:I0x3:bushvn", "kA, 3*I0"
      n_I0x3_buslvn = "n:I0x3:buslvn", "kA, 3*I0"
      n_phii0_bushv = "n:phii0:bushv", "deg, Zero-Sequence Current, Angle"
      n_phii0_buslv = "n:phii0:buslv", "deg, Zero-Sequence Current, Angle"
      n_phii0_bushvn = "n:phii0:bushvn", "deg, Zero-Sequence Current, Angle"
      n_phii0_buslvn = "n:phii0:buslvn", "deg, Zero-Sequence Current, Angle"
      n_i1r_bushv = "n:i1r:bushv", "p.u., Positive-Sequence Current, Real Part"
      n_i1r_buslv = "n:i1r:buslv", "p.u., Positive-Sequence Current, Real Part"
      n_i1r_bushvn = "n:i1r:bushvn", "p.u., Positive-Sequence Current, Real Part"
      n_i1r_buslvn = "n:i1r:buslvn", "p.u., Positive-Sequence Current, Real Part"
      n_i1i_bushv = "n:i1i:bushv", "p.u., Positive-Sequence Current, Imaginary Part"
      n_i1i_buslv = "n:i1i:buslv", "p.u., Positive-Sequence Current, Imaginary Part"
      n_i1i_bushvn = "n:i1i:bushvn", "p.u., Positive-Sequence Current, Imaginary Part"
      n_i1i_buslvn = "n:i1i:buslvn", "p.u., Positive-Sequence Current, Imaginary Part"
      n_i1_bushv = "n:i1:bushv", "p.u., Positive-Sequence Current, Magnitude"
      n_i1_buslv = "n:i1:buslv", "p.u., Positive-Sequence Current, Magnitude"
      n_i1_bushvn = "n:i1:bushvn", "p.u., Positive-Sequence Current, Magnitude"
      n_i1_buslvn = "n:i1:buslvn", "p.u., Positive-Sequence Current, Magnitude"
      n_I1_bushv = "n:I1:bushv", "kA, Positive-Sequence Current, Magnitude"
      n_I1_buslv = "n:I1:buslv", "kA, Positive-Sequence Current, Magnitude"
      n_I1_bushvn = "n:I1:bushvn", "kA, Positive-Sequence Current, Magnitude"
      n_I1_buslvn = "n:I1:buslvn", "kA, Positive-Sequence Current, Magnitude"
      n_phii1_bushv = "n:phii1:bushv", "deg, Positive-Sequence Current, Angle"
      n_phii1_buslv = "n:phii1:buslv", "deg, Positive-Sequence Current, Angle"
      n_phii1_bushvn = "n:phii1:bushvn", "deg, Positive-Sequence Current, Angle"
      n_phii1_buslvn = "n:phii1:buslvn", "deg, Positive-Sequence Current, Angle"
      n_i2r_bushv = "n:i2r:bushv", "p.u., Negative-Sequence Current, Real Part"
      n_i2r_buslv = "n:i2r:buslv", "p.u., Negative-Sequence Current, Real Part"
      n_i2r_bushvn = "n:i2r:bushvn", "p.u., Negative-Sequence Current, Real Part"
      n_i2r_buslvn = "n:i2r:buslvn", "p.u., Negative-Sequence Current, Real Part"
      n_i2i_bushv = "n:i2i:bushv", "p.u., Negative-Sequence Current, Imaginary Part"
      n_i2i_buslv = "n:i2i:buslv", "p.u., Negative-Sequence Current, Imaginary Part"
      n_i2i_bushvn = "n:i2i:bushvn", "p.u., Negative-Sequence Current, Imaginary Part"
      n_i2i_buslvn = "n:i2i:buslvn", "p.u., Negative-Sequence Current, Imaginary Part"
      n_i2_bushv = "n:i2:bushv", "p.u., Negative-Sequence Current, Magnitude"
      n_i2_buslv = "n:i2:buslv", "p.u., Negative-Sequence Current, Magnitude"
      n_i2_bushvn = "n:i2:bushvn", "p.u., Negative-Sequence Current, Magnitude"
      n_i2_buslvn = "n:i2:buslvn", "p.u., Negative-Sequence Current, Magnitude"
      n_I2_bushv = "n:I2:bushv", "kA, Negative-Sequence Current, Magnitude"
      n_I2_buslv = "n:I2:buslv", "kA, Negative-Sequence Current, Magnitude"
      n_I2_bushvn = "n:I2:bushvn", "kA, Negative-Sequence Current, Magnitude"
      n_I2_buslvn = "n:I2:buslvn", "kA, Negative-Sequence Current, Magnitude"
      n_phii2_bushv = "n:phii2:bushv", "deg, Negative-Sequence Current, Angle"
      n_phii2_buslv = "n:phii2:buslv", "deg, Negative-Sequence Current, Angle"
      n_phii2_bushvn = "n:phii2:bushvn", "deg, Negative-Sequence Current, Angle"
      n_phii2_buslvn = "n:phii2:buslvn", "deg, Negative-Sequence Current, Angle"
      n_du_bushv_A = "n:du:bushv:A", "%, Voltage Deviation"
      n_du_bushv_B = "n:du:bushv:B", "%, Voltage Deviation"
      n_du_bushv_C = "n:du:bushv:C", "%, Voltage Deviation"
      n_du_bushv_N = "n:du:bushv:N", "%, Voltage Deviation"
      n_du_buslv_A = "n:du:buslv:A", "%, Voltage Deviation"
      n_du_buslv_B = "n:du:buslv:B", "%, Voltage Deviation"
      n_du_buslv_C = "n:du:buslv:C", "%, Voltage Deviation"
      n_du_buslv_N = "n:du:buslv:N", "%, Voltage Deviation"
      n_du_bushvn = "n:du:bushvn", "%, Voltage Deviation"
      n_du_buslvn = "n:du:buslvn", "%, Voltage Deviation"
      n_urpref_bushv_A = "n:urpref:bushv:A", "p.u., Pre-fault Voltage, Real Part"
      n_urpref_bushv_B = "n:urpref:bushv:B", "p.u., Pre-fault Voltage, Real Part"
      n_urpref_bushv_C = "n:urpref:bushv:C", "p.u., Pre-fault Voltage, Real Part"
      n_urpref_bushv_N = "n:urpref:bushv:N", "p.u., Pre-fault Voltage, Real Part"
      n_urpref_buslv_A = "n:urpref:buslv:A", "p.u., Pre-fault Voltage, Real Part"
      n_urpref_buslv_B = "n:urpref:buslv:B", "p.u., Pre-fault Voltage, Real Part"
      n_urpref_buslv_C = "n:urpref:buslv:C", "p.u., Pre-fault Voltage, Real Part"
      n_urpref_buslv_N = "n:urpref:buslv:N", "p.u., Pre-fault Voltage, Real Part"
      n_urpref_bushvn = "n:urpref:bushvn", "p.u., Pre-fault Voltage, Real Part"
      n_urpref_buslvn = "n:urpref:buslvn", "p.u., Pre-fault Voltage, Real Part"
      n_uipref_bushv_A = "n:uipref:bushv:A", "p.u., Pre-fault Voltage, Imaginary Part"
      n_uipref_bushv_B = "n:uipref:bushv:B", "p.u., Pre-fault Voltage, Imaginary Part"
      n_uipref_bushv_C = "n:uipref:bushv:C", "p.u., Pre-fault Voltage, Imaginary Part"
      n_uipref_bushv_N = "n:uipref:bushv:N", "p.u., Pre-fault Voltage, Imaginary Part"
      n_uipref_buslv_A = "n:uipref:buslv:A", "p.u., Pre-fault Voltage, Imaginary Part"
      n_uipref_buslv_B = "n:uipref:buslv:B", "p.u., Pre-fault Voltage, Imaginary Part"
      n_uipref_buslv_C = "n:uipref:buslv:C", "p.u., Pre-fault Voltage, Imaginary Part"
      n_uipref_buslv_N = "n:uipref:buslv:N", "p.u., Pre-fault Voltage, Imaginary Part"
      n_uipref_bushvn = "n:uipref:bushvn", "p.u., Pre-fault Voltage, Imaginary Part"
      n_uipref_buslvn = "n:uipref:buslvn", "p.u., Pre-fault Voltage, Imaginary Part"
      n_upref_bushv_A = "n:upref:bushv:A", "p.u., Pre-fault Voltage, Magnitude"
      n_upref_bushv_B = "n:upref:bushv:B", "p.u., Pre-fault Voltage, Magnitude"
      n_upref_bushv_C = "n:upref:bushv:C", "p.u., Pre-fault Voltage, Magnitude"
      n_upref_bushv_N = "n:upref:bushv:N", "p.u., Pre-fault Voltage, Magnitude"
      n_upref_buslv_A = "n:upref:buslv:A", "p.u., Pre-fault Voltage, Magnitude"
      n_upref_buslv_B = "n:upref:buslv:B", "p.u., Pre-fault Voltage, Magnitude"
      n_upref_buslv_C = "n:upref:buslv:C", "p.u., Pre-fault Voltage, Magnitude"
      n_upref_buslv_N = "n:upref:buslv:N", "p.u., Pre-fault Voltage, Magnitude"
      n_upref_bushvn = "n:upref:bushvn", "p.u., Pre-fault Voltage, Magnitude"
      n_upref_buslvn = "n:upref:buslvn", "p.u., Pre-fault Voltage, Magnitude"
      n_uprefpc_bushv_A = "n:uprefpc:bushv:A", "%, Pre-fault Voltage, Magnitude"
      n_uprefpc_bushv_B = "n:uprefpc:bushv:B", "%, Pre-fault Voltage, Magnitude"
      n_uprefpc_bushv_C = "n:uprefpc:bushv:C", "%, Pre-fault Voltage, Magnitude"
      n_uprefpc_bushv_N = "n:uprefpc:bushv:N", "%, Pre-fault Voltage, Magnitude"
      n_uprefpc_buslv_A = "n:uprefpc:buslv:A", "%, Pre-fault Voltage, Magnitude"
      n_uprefpc_buslv_B = "n:uprefpc:buslv:B", "%, Pre-fault Voltage, Magnitude"
      n_uprefpc_buslv_C = "n:uprefpc:buslv:C", "%, Pre-fault Voltage, Magnitude"
      n_uprefpc_buslv_N = "n:uprefpc:buslv:N", "%, Pre-fault Voltage, Magnitude"
      n_uprefpc_bushvn = "n:uprefpc:bushvn", "%, Pre-fault Voltage, Magnitude"
      n_uprefpc_buslvn = "n:uprefpc:buslvn", "%, Pre-fault Voltage, Magnitude"
      n_Upref_bushv_A = "n:Upref:bushv:A", "kV, Line-Ground Pre-fault Voltage, Magnitude"
      n_Upref_bushv_B = "n:Upref:bushv:B", "kV, Line-Ground Pre-fault Voltage, Magnitude"
      n_Upref_bushv_C = "n:Upref:bushv:C", "kV, Line-Ground Pre-fault Voltage, Magnitude"
      n_Upref_bushv_N = "n:Upref:bushv:N", "kV, Line-Ground Pre-fault Voltage, Magnitude"
      n_Upref_buslv_A = "n:Upref:buslv:A", "kV, Line-Ground Pre-fault Voltage, Magnitude"
      n_Upref_buslv_B = "n:Upref:buslv:B", "kV, Line-Ground Pre-fault Voltage, Magnitude"
      n_Upref_buslv_C = "n:Upref:buslv:C", "kV, Line-Ground Pre-fault Voltage, Magnitude"
      n_Upref_buslv_N = "n:Upref:buslv:N", "kV, Line-Ground Pre-fault Voltage, Magnitude"
      n_Upref_bushvn = "n:Upref:bushvn", "kV, Line-Ground Pre-fault Voltage, Magnitude"
      n_Upref_buslvn = "n:Upref:buslvn", "kV, Line-Ground Pre-fault Voltage, Magnitude"
      n_Ulpref_bushv_A = "n:Ulpref:bushv:A", "kV, Line-Line Pre-fault Voltage, Magnitude"
      n_Ulpref_bushv_B = "n:Ulpref:bushv:B", "kV, Line-Line Pre-fault Voltage, Magnitude"
      n_Ulpref_bushv_C = "n:Ulpref:bushv:C", "kV, Line-Line Pre-fault Voltage, Magnitude"
      n_Ulpref_bushv_N = "n:Ulpref:bushv:N", "kV, Line-Line Pre-fault Voltage, Magnitude"
      n_Ulpref_buslv_A = "n:Ulpref:buslv:A", "kV, Line-Line Pre-fault Voltage, Magnitude"
      n_Ulpref_buslv_B = "n:Ulpref:buslv:B", "kV, Line-Line Pre-fault Voltage, Magnitude"
      n_Ulpref_buslv_C = "n:Ulpref:buslv:C", "kV, Line-Line Pre-fault Voltage, Magnitude"
      n_Ulpref_buslv_N = "n:Ulpref:buslv:N", "kV, Line-Line Pre-fault Voltage, Magnitude"
      n_Ulpref_bushvn = "n:Ulpref:bushvn", "kV, Line-Line Pre-fault Voltage, Magnitude"
      n_Ulpref_buslvn = "n:Ulpref:buslvn", "kV, Line-Line Pre-fault Voltage, Magnitude"
      n_phiupref_bushv_A = "n:phiupref:bushv:A", "deg, Pre-fault Voltage, Angle"
      n_phiupref_bushv_B = "n:phiupref:bushv:B", "deg, Pre-fault Voltage, Angle"
      n_phiupref_bushv_C = "n:phiupref:bushv:C", "deg, Pre-fault Voltage, Angle"
      n_phiupref_bushv_N = "n:phiupref:bushv:N", "deg, Pre-fault Voltage, Angle"
      n_phiupref_buslv_A = "n:phiupref:buslv:A", "deg, Pre-fault Voltage, Angle"
      n_phiupref_buslv_B = "n:phiupref:buslv:B", "deg, Pre-fault Voltage, Angle"
      n_phiupref_buslv_C = "n:phiupref:buslv:C", "deg, Pre-fault Voltage, Angle"
      n_phiupref_buslv_N = "n:phiupref:buslv:N", "deg, Pre-fault Voltage, Angle"
      n_phiupref_bushvn = "n:phiupref:bushvn", "deg, Pre-fault Voltage, Angle"
      n_phiupref_buslvn = "n:phiupref:buslvn", "deg, Pre-fault Voltage, Angle"
      n_phiui_bushv_A = "n:phiui:bushv:A", "deg, Angle between Voltage and Current"
      n_phiui_bushv_B = "n:phiui:bushv:B", "deg, Angle between Voltage and Current"
      n_phiui_bushv_C = "n:phiui:bushv:C", "deg, Angle between Voltage and Current"
      n_phiui_bushv_N = "n:phiui:bushv:N", "deg, Angle between Voltage and Current"
      n_phiui_buslv_A = "n:phiui:buslv:A", "deg, Angle between Voltage and Current"
      n_phiui_buslv_B = "n:phiui:buslv:B", "deg, Angle between Voltage and Current"
      n_phiui_buslv_C = "n:phiui:buslv:C", "deg, Angle between Voltage and Current"
      n_phiui_buslv_N = "n:phiui:buslv:N", "deg, Angle between Voltage and Current"
      n_phiui_bushvn = "n:phiui:bushvn", "deg, Angle between Voltage and Current"
      n_phiui_buslvn = "n:phiui:buslvn", "deg, Angle between Voltage and Current"
      n_phiu1i1_bushv = "n:phiu1i1:bushv", "deg, Angle between Voltage and Current in positive sequence system"
      n_phiu1i1_buslv = "n:phiu1i1:buslv", "deg, Angle between Voltage and Current in positive sequence system"
      n_phiu1i1_bushvn = "n:phiu1i1:bushvn", "deg, Angle between Voltage and Current in positive sequence system"
      n_phiu1i1_buslvn = "n:phiu1i1:buslvn", "deg, Angle between Voltage and Current in positive sequence system"
      n_phiu2i2_bushv = "n:phiu2i2:bushv", "deg, Angle between Voltage and Current in negative sequence system"
      n_phiu2i2_buslv = "n:phiu2i2:buslv", "deg, Angle between Voltage and Current in negative sequence system"
      n_phiu2i2_bushvn = "n:phiu2i2:bushvn", "deg, Angle between Voltage and Current in negative sequence system"
      n_phiu2i2_buslvn = "n:phiu2i2:buslvn", "deg, Angle between Voltage and Current in negative sequence system"
      n_phiu0i0_bushv = "n:phiu0i0:bushv", "deg, Angle between Voltage and Current in zero-sequence system"
      n_phiu0i0_buslv = "n:phiu0i0:buslv", "deg, Angle between Voltage and Current in zero-sequence system"
      n_phiu0i0_bushvn = "n:phiu0i0:bushvn", "deg, Angle between Voltage and Current in zero-sequence system"
      n_phiu0i0_buslvn = "n:phiu0i0:buslvn", "deg, Angle between Voltage and Current in zero-sequence system"
      n_Ithload_bushvn = "n:Ithload:bushvn", "%, Loading, Thermal equivalent Current"
      n_Ithload_buslvn = "n:Ithload:buslvn", "%, Loading, Thermal equivalent Current"
      n_Ipload_bushvn = "n:Ipload:bushvn", "%, Loading, Peak Short-Circuit Current"
      n_Ipload_buslvn = "n:Ipload:buslvn", "%, Loading, Peak Short-Circuit Current"
      n_Ithrtk_bushvn = "n:Ithrtk:bushvn", "kA, Rated Short-Time Current (Tk)"
      n_Ithrtk_buslvn = "n:Ithrtk:buslvn", "kA, Rated Short-Time Current (Tk)"
      n_cfac_bushvn = "n:cfac:bushvn", "Voltage Factor"
      n_cfac_buslvn = "n:cfac:buslvn", "Voltage Factor"
      n_XtoR_bushvn = "n:XtoR:bushvn", "X/R ratio (peak)"
      n_XtoR_buslvn = "n:XtoR:buslvn", "X/R ratio (peak)"
      n_RtoX_bushvn = "n:RtoX:bushvn", "R/X ratio (peak)"
      n_RtoX_buslvn = "n:RtoX:buslvn", "R/X ratio (peak)"

    class ElmTr3(Enum):
      _init_ = 'value __doc__'

      m_u0_bushv = "m:u0:bushv", "p.u., Zero-Sequence-Voltage, Magnitude"
      m_u0_busmv = "m:u0:busmv", "p.u., Zero-Sequence-Voltage, Magnitude"
      m_u0_buslv = "m:u0:buslv", "p.u., Zero-Sequence-Voltage, Magnitude"
      m_u0_bushvn = "m:u0:bushvn", "p.u., Zero-Sequence-Voltage, Magnitude"
      m_u0_busmvn = "m:u0:busmvn", "p.u., Zero-Sequence-Voltage, Magnitude"
      m_u0_buslvn = "m:u0:buslvn", "p.u., Zero-Sequence-Voltage, Magnitude"
      m_phiu0_bushv = "m:phiu0:bushv", "deg, Zero-Sequence-Voltage, Angle"
      m_phiu0_busmv = "m:phiu0:busmv", "deg, Zero-Sequence-Voltage, Angle"
      m_phiu0_buslv = "m:phiu0:buslv", "deg, Zero-Sequence-Voltage, Angle"
      m_phiu0_bushvn = "m:phiu0:bushvn", "deg, Zero-Sequence-Voltage, Angle"
      m_phiu0_busmvn = "m:phiu0:busmvn", "deg, Zero-Sequence-Voltage, Angle"
      m_phiu0_buslvn = "m:phiu0:buslvn", "deg, Zero-Sequence-Voltage, Angle"
      m_u1r_bushv = "m:u1r:bushv", "p.u., Positive-Sequence-Voltage, Real Part"
      m_u1r_busmv = "m:u1r:busmv", "p.u., Positive-Sequence-Voltage, Real Part"
      m_u1r_buslv = "m:u1r:buslv", "p.u., Positive-Sequence-Voltage, Real Part"
      m_u1r_bushvn = "m:u1r:bushvn", "p.u., Positive-Sequence-Voltage, Real Part"
      m_u1r_busmvn = "m:u1r:busmvn", "p.u., Positive-Sequence-Voltage, Real Part"
      m_u1r_buslvn = "m:u1r:buslvn", "p.u., Positive-Sequence-Voltage, Real Part"
      m_u1i_bushv = "m:u1i:bushv", "p.u., Positive-Sequence-Voltage, Imaginary Part"
      m_u1i_busmv = "m:u1i:busmv", "p.u., Positive-Sequence-Voltage, Imaginary Part"
      m_u1i_buslv = "m:u1i:buslv", "p.u., Positive-Sequence-Voltage, Imaginary Part"
      m_u1i_bushvn = "m:u1i:bushvn", "p.u., Positive-Sequence-Voltage, Imaginary Part"
      m_u1i_busmvn = "m:u1i:busmvn", "p.u., Positive-Sequence-Voltage, Imaginary Part"
      m_u1i_buslvn = "m:u1i:buslvn", "p.u., Positive-Sequence-Voltage, Imaginary Part"
      m_u1_bushv = "m:u1:bushv", "p.u., Positive-Sequence-Voltage, Magnitude"
      m_u1_busmv = "m:u1:busmv", "p.u., Positive-Sequence-Voltage, Magnitude"
      m_u1_buslv = "m:u1:buslv", "p.u., Positive-Sequence-Voltage, Magnitude"
      m_u1_bushvn = "m:u1:bushvn", "p.u., Positive-Sequence-Voltage, Magnitude"
      m_u1_busmvn = "m:u1:busmvn", "p.u., Positive-Sequence-Voltage, Magnitude"
      m_u1_buslvn = "m:u1:buslvn", "p.u., Positive-Sequence-Voltage, Magnitude"
      m_U1_bushv = "m:U1:bushv", "kV, Line-Ground Positive-Sequence-Voltage, Magnitude"
      m_U1_busmv = "m:U1:busmv", "kV, Line-Ground Positive-Sequence-Voltage, Magnitude"
      m_U1_buslv = "m:U1:buslv", "kV, Line-Ground Positive-Sequence-Voltage, Magnitude"
      m_U1_bushvn = "m:U1:bushvn", "kV, Line-Ground Positive-Sequence-Voltage, Magnitude"
      m_U1_busmvn = "m:U1:busmvn", "kV, Line-Ground Positive-Sequence-Voltage, Magnitude"
      m_U1_buslvn = "m:U1:buslvn", "kV, Line-Ground Positive-Sequence-Voltage, Magnitude"
      m_U1l_bushv = "m:U1l:bushv", "kV, Line-Line Positive-Sequence-Voltage, Magnitude"
      m_U1l_busmv = "m:U1l:busmv", "kV, Line-Line Positive-Sequence-Voltage, Magnitude"
      m_U1l_buslv = "m:U1l:buslv", "kV, Line-Line Positive-Sequence-Voltage, Magnitude"
      m_U1l_bushvn = "m:U1l:bushvn", "kV, Line-Line Positive-Sequence-Voltage, Magnitude"
      m_U1l_busmvn = "m:U1l:busmvn", "kV, Line-Line Positive-Sequence-Voltage, Magnitude"
      m_U1l_buslvn = "m:U1l:buslvn", "kV, Line-Line Positive-Sequence-Voltage, Magnitude"
      m_phiu1_bushv = "m:phiu1:bushv", "deg, Positive-Sequence-Voltage, Angle"
      m_phiu1_busmv = "m:phiu1:busmv", "deg, Positive-Sequence-Voltage, Angle"
      m_phiu1_buslv = "m:phiu1:buslv", "deg, Positive-Sequence-Voltage, Angle"
      m_phiu1_bushvn = "m:phiu1:bushvn", "deg, Positive-Sequence-Voltage, Angle"
      m_phiu1_busmvn = "m:phiu1:busmvn", "deg, Positive-Sequence-Voltage, Angle"
      m_phiu1_buslvn = "m:phiu1:buslvn", "deg, Positive-Sequence-Voltage, Angle"
      m_u2_bushv = "m:u2:bushv", "p.u., Negative-Sequence-Voltage, Magnitude"
      m_u2_busmv = "m:u2:busmv", "p.u., Negative-Sequence-Voltage, Magnitude"
      m_u2_buslv = "m:u2:buslv", "p.u., Negative-Sequence-Voltage, Magnitude"
      m_u2_bushvn = "m:u2:bushvn", "p.u., Negative-Sequence-Voltage, Magnitude"
      m_u2_busmvn = "m:u2:busmvn", "p.u., Negative-Sequence-Voltage, Magnitude"
      m_u2_buslvn = "m:u2:buslvn", "p.u., Negative-Sequence-Voltage, Magnitude"
      m_phiu2_bushv = "m:phiu2:bushv", "deg, Negative-Sequence-Voltage, Angle"
      m_phiu2_busmv = "m:phiu2:busmv", "deg, Negative-Sequence-Voltage, Angle"
      m_phiu2_buslv = "m:phiu2:buslv", "deg, Negative-Sequence-Voltage, Angle"
      m_phiu2_bushvn = "m:phiu2:bushvn", "deg, Negative-Sequence-Voltage, Angle"
      m_phiu2_busmvn = "m:phiu2:busmvn", "deg, Negative-Sequence-Voltage, Angle"
      m_phiu2_buslvn = "m:phiu2:buslvn", "deg, Negative-Sequence-Voltage, Angle"
      m_inet_bushv_A = "m:inet:bushv:A", "p.u., Phase Current, Magnitude, referred to network"
      m_inet_bushv_B = "m:inet:bushv:B", "p.u., Phase Current, Magnitude, referred to network"
      m_inet_bushv_C = "m:inet:bushv:C", "p.u., Phase Current, Magnitude, referred to network"
      m_inet_bushv_N = "m:inet:bushv:N", "p.u., Phase Current, Magnitude, referred to network"
      m_inet_busmv_A = "m:inet:busmv:A", "p.u., Phase Current, Magnitude, referred to network"
      m_inet_busmv_B = "m:inet:busmv:B", "p.u., Phase Current, Magnitude, referred to network"
      m_inet_busmv_C = "m:inet:busmv:C", "p.u., Phase Current, Magnitude, referred to network"
      m_inet_busmv_N = "m:inet:busmv:N", "p.u., Phase Current, Magnitude, referred to network"
      m_inet_buslv_A = "m:inet:buslv:A", "p.u., Phase Current, Magnitude, referred to network"
      m_inet_buslv_B = "m:inet:buslv:B", "p.u., Phase Current, Magnitude, referred to network"
      m_inet_buslv_C = "m:inet:buslv:C", "p.u., Phase Current, Magnitude, referred to network"
      m_inet_buslv_N = "m:inet:buslv:N", "p.u., Phase Current, Magnitude, referred to network"
      m_inet_bushvn = "m:inet:bushvn", "p.u., Phase Current, Magnitude, referred to network"
      m_inet_busmvn = "m:inet:busmvn", "p.u., Phase Current, Magnitude, referred to network"
      m_inet_buslvn = "m:inet:buslvn", "p.u., Phase Current, Magnitude, referred to network"
      m_I_bushv_A = "m:I:bushv:A", "kA, Phase Current, Magnitude"
      m_I_bushv_B = "m:I:bushv:B", "kA, Phase Current, Magnitude"
      m_I_bushv_C = "m:I:bushv:C", "kA, Phase Current, Magnitude"
      m_I_bushv_N = "m:I:bushv:N", "kA, Phase Current, Magnitude"
      m_I_busmv_A = "m:I:busmv:A", "kA, Phase Current, Magnitude"
      m_I_busmv_B = "m:I:busmv:B", "kA, Phase Current, Magnitude"
      m_I_busmv_C = "m:I:busmv:C", "kA, Phase Current, Magnitude"
      m_I_busmv_N = "m:I:busmv:N", "kA, Phase Current, Magnitude"
      m_I_buslv_A = "m:I:buslv:A", "kA, Phase Current, Magnitude"
      m_I_buslv_B = "m:I:buslv:B", "kA, Phase Current, Magnitude"
      m_I_buslv_C = "m:I:buslv:C", "kA, Phase Current, Magnitude"
      m_I_buslv_N = "m:I:buslv:N", "kA, Phase Current, Magnitude"
      m_I_bushvn = "m:I:bushvn", "kA, Phase Current, Magnitude"
      m_I_busmvn = "m:I:busmvn", "kA, Phase Current, Magnitude"
      m_I_buslvn = "m:I:buslvn", "kA, Phase Current, Magnitude"
      m_phii_bushv_A = "m:phii:bushv:A", "deg, Phase Current, Angle"
      m_phii_bushv_B = "m:phii:bushv:B", "deg, Phase Current, Angle"
      m_phii_bushv_C = "m:phii:bushv:C", "deg, Phase Current, Angle"
      m_phii_bushv_N = "m:phii:bushv:N", "deg, Phase Current, Angle"
      m_phii_busmv_A = "m:phii:busmv:A", "deg, Phase Current, Angle"
      m_phii_busmv_B = "m:phii:busmv:B", "deg, Phase Current, Angle"
      m_phii_busmv_C = "m:phii:busmv:C", "deg, Phase Current, Angle"
      m_phii_busmv_N = "m:phii:busmv:N", "deg, Phase Current, Angle"
      m_phii_buslv_A = "m:phii:buslv:A", "deg, Phase Current, Angle"
      m_phii_buslv_B = "m:phii:buslv:B", "deg, Phase Current, Angle"
      m_phii_buslv_C = "m:phii:buslv:C", "deg, Phase Current, Angle"
      m_phii_buslv_N = "m:phii:buslv:N", "deg, Phase Current, Angle"
      m_phii_bushvn = "m:phii:bushvn", "deg, Phase Current, Angle"
      m_phii_busmvn = "m:phii:busmvn", "deg, Phase Current, Angle"
      m_phii_buslvn = "m:phii:buslvn", "deg, Phase Current, Angle"
      m_i0r_bushv = "m:i0r:bushv", "p.u., Zero-Sequence Current, Real Part"
      m_i0r_busmv = "m:i0r:busmv", "p.u., Zero-Sequence Current, Real Part"
      m_i0r_buslv = "m:i0r:buslv", "p.u., Zero-Sequence Current, Real Part"
      m_i0r_bushvn = "m:i0r:bushvn", "p.u., Zero-Sequence Current, Real Part"
      m_i0r_busmvn = "m:i0r:busmvn", "p.u., Zero-Sequence Current, Real Part"
      m_i0r_buslvn = "m:i0r:buslvn", "p.u., Zero-Sequence Current, Real Part"
      m_i0i_bushv = "m:i0i:bushv", "p.u., Zero-Sequence Current, Imaginary Part"
      m_i0i_busmv = "m:i0i:busmv", "p.u., Zero-Sequence Current, Imaginary Part"
      m_i0i_buslv = "m:i0i:buslv", "p.u., Zero-Sequence Current, Imaginary Part"
      m_i0i_bushvn = "m:i0i:bushvn", "p.u., Zero-Sequence Current, Imaginary Part"
      m_i0i_busmvn = "m:i0i:busmvn", "p.u., Zero-Sequence Current, Imaginary Part"
      m_i0i_buslvn = "m:i0i:buslvn", "p.u., Zero-Sequence Current, Imaginary Part"
      m_i0_bushv = "m:i0:bushv", "p.u., Zero-Sequence Current, Magnitude"
      m_i0_busmv = "m:i0:busmv", "p.u., Zero-Sequence Current, Magnitude"
      m_i0_buslv = "m:i0:buslv", "p.u., Zero-Sequence Current, Magnitude"
      m_i0_bushvn = "m:i0:bushvn", "p.u., Zero-Sequence Current, Magnitude"
      m_i0_busmvn = "m:i0:busmvn", "p.u., Zero-Sequence Current, Magnitude"
      m_i0_buslvn = "m:i0:buslvn", "p.u., Zero-Sequence Current, Magnitude"
      m_I0_bushv = "m:I0:bushv", "kA, Zero-Sequence Current, Magnitude"
      m_I0_busmv = "m:I0:busmv", "kA, Zero-Sequence Current, Magnitude"
      m_I0_buslv = "m:I0:buslv", "kA, Zero-Sequence Current, Magnitude"
      m_I0_bushvn = "m:I0:bushvn", "kA, Zero-Sequence Current, Magnitude"
      m_I0_busmvn = "m:I0:busmvn", "kA, Zero-Sequence Current, Magnitude"
      m_I0_buslvn = "m:I0:buslvn", "kA, Zero-Sequence Current, Magnitude"
      m_I0x3_bushv = "m:I0x3:bushv", "kA, 3*I0"
      m_I0x3_busmv = "m:I0x3:busmv", "kA, 3*I0"
      m_I0x3_buslv = "m:I0x3:buslv", "kA, 3*I0"
      m_I0x3_bushvn = "m:I0x3:bushvn", "kA, 3*I0"
      m_I0x3_busmvn = "m:I0x3:busmvn", "kA, 3*I0"
      m_I0x3_buslvn = "m:I0x3:buslvn", "kA, 3*I0"
      m_phii0_bushv = "m:phii0:bushv", "deg, Zero-Sequence Current, Angle"
      m_phii0_busmv = "m:phii0:busmv", "deg, Zero-Sequence Current, Angle"
      m_phii0_buslv = "m:phii0:buslv", "deg, Zero-Sequence Current, Angle"
      m_phii0_bushvn = "m:phii0:bushvn", "deg, Zero-Sequence Current, Angle"
      m_phii0_busmvn = "m:phii0:busmvn", "deg, Zero-Sequence Current, Angle"
      m_phii0_buslvn = "m:phii0:buslvn", "deg, Zero-Sequence Current, Angle"
      m_i1r_bushv = "m:i1r:bushv", "p.u., Positive-Sequence Current, Real Part"
      m_i1r_busmv = "m:i1r:busmv", "p.u., Positive-Sequence Current, Real Part"
      m_i1r_buslv = "m:i1r:buslv", "p.u., Positive-Sequence Current, Real Part"
      m_i1r_bushvn = "m:i1r:bushvn", "p.u., Positive-Sequence Current, Real Part"
      m_i1r_busmvn = "m:i1r:busmvn", "p.u., Positive-Sequence Current, Real Part"
      m_i1r_buslvn = "m:i1r:buslvn", "p.u., Positive-Sequence Current, Real Part"
      m_i1i_bushv = "m:i1i:bushv", "p.u., Positive-Sequence Current, Imaginary Part"
      m_i1i_busmv = "m:i1i:busmv", "p.u., Positive-Sequence Current, Imaginary Part"
      m_i1i_buslv = "m:i1i:buslv", "p.u., Positive-Sequence Current, Imaginary Part"
      m_i1i_bushvn = "m:i1i:bushvn", "p.u., Positive-Sequence Current, Imaginary Part"
      m_i1i_busmvn = "m:i1i:busmvn", "p.u., Positive-Sequence Current, Imaginary Part"
      m_i1i_buslvn = "m:i1i:buslvn", "p.u., Positive-Sequence Current, Imaginary Part"
      m_i1_bushv = "m:i1:bushv", "p.u., Positive-Sequence Current, Magnitude"
      m_i1_busmv = "m:i1:busmv", "p.u., Positive-Sequence Current, Magnitude"
      m_i1_buslv = "m:i1:buslv", "p.u., Positive-Sequence Current, Magnitude"
      m_i1_bushvn = "m:i1:bushvn", "p.u., Positive-Sequence Current, Magnitude"
      m_i1_busmvn = "m:i1:busmvn", "p.u., Positive-Sequence Current, Magnitude"
      m_i1_buslvn = "m:i1:buslvn", "p.u., Positive-Sequence Current, Magnitude"
      m_I1_bushv = "m:I1:bushv", "kA, Positive-Sequence Current, Magnitude"
      m_I1_busmv = "m:I1:busmv", "kA, Positive-Sequence Current, Magnitude"
      m_I1_buslv = "m:I1:buslv", "kA, Positive-Sequence Current, Magnitude"
      m_I1_bushvn = "m:I1:bushvn", "kA, Positive-Sequence Current, Magnitude"
      m_I1_busmvn = "m:I1:busmvn", "kA, Positive-Sequence Current, Magnitude"
      m_I1_buslvn = "m:I1:buslvn", "kA, Positive-Sequence Current, Magnitude"
      m_phii1_bushv = "m:phii1:bushv", "deg, Positive-Sequence Current, Angle"
      m_phii1_busmv = "m:phii1:busmv", "deg, Positive-Sequence Current, Angle"
      m_phii1_buslv = "m:phii1:buslv", "deg, Positive-Sequence Current, Angle"
      m_phii1_bushvn = "m:phii1:bushvn", "deg, Positive-Sequence Current, Angle"
      m_phii1_busmvn = "m:phii1:busmvn", "deg, Positive-Sequence Current, Angle"
      m_phii1_buslvn = "m:phii1:buslvn", "deg, Positive-Sequence Current, Angle"
      m_i2r_bushv = "m:i2r:bushv", "p.u., Negative-Sequence Current, Real Part"
      m_i2r_busmv = "m:i2r:busmv", "p.u., Negative-Sequence Current, Real Part"
      m_i2r_buslv = "m:i2r:buslv", "p.u., Negative-Sequence Current, Real Part"
      m_i2r_bushvn = "m:i2r:bushvn", "p.u., Negative-Sequence Current, Real Part"
      m_i2r_busmvn = "m:i2r:busmvn", "p.u., Negative-Sequence Current, Real Part"
      m_i2r_buslvn = "m:i2r:buslvn", "p.u., Negative-Sequence Current, Real Part"
      m_i2i_bushv = "m:i2i:bushv", "p.u., Negative-Sequence Current, Imaginary Part"
      m_i2i_busmv = "m:i2i:busmv", "p.u., Negative-Sequence Current, Imaginary Part"
      m_i2i_buslv = "m:i2i:buslv", "p.u., Negative-Sequence Current, Imaginary Part"
      m_i2i_bushvn = "m:i2i:bushvn", "p.u., Negative-Sequence Current, Imaginary Part"
      m_i2i_busmvn = "m:i2i:busmvn", "p.u., Negative-Sequence Current, Imaginary Part"
      m_i2i_buslvn = "m:i2i:buslvn", "p.u., Negative-Sequence Current, Imaginary Part"
      m_i2_bushv = "m:i2:bushv", "p.u., Negative-Sequence Current, Magnitude"
      m_i2_busmv = "m:i2:busmv", "p.u., Negative-Sequence Current, Magnitude"
      m_i2_buslv = "m:i2:buslv", "p.u., Negative-Sequence Current, Magnitude"
      m_i2_bushvn = "m:i2:bushvn", "p.u., Negative-Sequence Current, Magnitude"
      m_i2_busmvn = "m:i2:busmvn", "p.u., Negative-Sequence Current, Magnitude"
      m_i2_buslvn = "m:i2:buslvn", "p.u., Negative-Sequence Current, Magnitude"
      m_I2_bushv = "m:I2:bushv", "kA, Negative-Sequence Current, Magnitude"
      m_I2_busmv = "m:I2:busmv", "kA, Negative-Sequence Current, Magnitude"
      m_I2_buslv = "m:I2:buslv", "kA, Negative-Sequence Current, Magnitude"
      m_I2_bushvn = "m:I2:bushvn", "kA, Negative-Sequence Current, Magnitude"
      m_I2_busmvn = "m:I2:busmvn", "kA, Negative-Sequence Current, Magnitude"
      m_I2_buslvn = "m:I2:buslvn", "kA, Negative-Sequence Current, Magnitude"
      m_phii2_bushv = "m:phii2:bushv", "deg, Negative-Sequence Current, Angle"
      m_phii2_busmv = "m:phii2:busmv", "deg, Negative-Sequence Current, Angle"
      m_phii2_buslv = "m:phii2:buslv", "deg, Negative-Sequence Current, Angle"
      m_phii2_bushvn = "m:phii2:bushvn", "deg, Negative-Sequence Current, Angle"
      m_phii2_busmvn = "m:phii2:busmvn", "deg, Negative-Sequence Current, Angle"
      m_phii2_buslvn = "m:phii2:buslvn", "deg, Negative-Sequence Current, Angle"
      m_ubfacI_bushv = "m:ubfacI:bushv", "%, Current Unbalance Factor"
      m_ubfacI_busmv = "m:ubfacI:busmv", "%, Current Unbalance Factor"
      m_ubfacI_buslv = "m:ubfacI:buslv", "%, Current Unbalance Factor"
      m_ubfacI_bushvn = "m:ubfacI:bushvn", "%, Current Unbalance Factor"
      m_ubfacI_busmvn = "m:ubfacI:busmvn", "%, Current Unbalance Factor"
      m_ubfacI_buslvn = "m:ubfacI:buslvn", "%, Current Unbalance Factor"
      m_ubfacS_bushv = "m:ubfacS:bushv", "%, Power Unbalance Factor"
      m_ubfacS_busmv = "m:ubfacS:busmv", "%, Power Unbalance Factor"
      m_ubfacS_buslv = "m:ubfacS:buslv", "%, Power Unbalance Factor"
      m_ubfacS_bushvn = "m:ubfacS:bushvn", "%, Power Unbalance Factor"
      m_ubfacS_busmvn = "m:ubfacS:busmvn", "%, Power Unbalance Factor"
      m_ubfacS_buslvn = "m:ubfacS:buslvn", "%, Power Unbalance Factor"
      m_P_bushv_A = "m:P:bushv:A", "MW, Active Power"
      m_P_bushv_B = "m:P:bushv:B", "MW, Active Power"
      m_P_bushv_C = "m:P:bushv:C", "MW, Active Power"
      m_P_bushv_N = "m:P:bushv:N", "MW, Active Power"
      m_P_busmv_A = "m:P:busmv:A", "MW, Active Power"
      m_P_busmv_B = "m:P:busmv:B", "MW, Active Power"
      m_P_busmv_C = "m:P:busmv:C", "MW, Active Power"
      m_P_busmv_N = "m:P:busmv:N", "MW, Active Power"
      m_P_buslv_A = "m:P:buslv:A", "MW, Active Power"
      m_P_buslv_B = "m:P:buslv:B", "MW, Active Power"
      m_P_buslv_C = "m:P:buslv:C", "MW, Active Power"
      m_P_buslv_N = "m:P:buslv:N", "MW, Active Power"
      m_P_bushvn = "m:P:bushvn", "MW, Active Power"
      m_P_busmvn = "m:P:busmvn", "MW, Active Power"
      m_P_buslvn = "m:P:buslvn", "MW, Active Power"
      m_Q_bushv_A = "m:Q:bushv:A", "Mvar, Reactive Power"
      m_Q_bushv_B = "m:Q:bushv:B", "Mvar, Reactive Power"
      m_Q_bushv_C = "m:Q:bushv:C", "Mvar, Reactive Power"
      m_Q_bushv_N = "m:Q:bushv:N", "Mvar, Reactive Power"
      m_Q_busmv_A = "m:Q:busmv:A", "Mvar, Reactive Power"
      m_Q_busmv_B = "m:Q:busmv:B", "Mvar, Reactive Power"
      m_Q_busmv_C = "m:Q:busmv:C", "Mvar, Reactive Power"
      m_Q_busmv_N = "m:Q:busmv:N", "Mvar, Reactive Power"
      m_Q_buslv_A = "m:Q:buslv:A", "Mvar, Reactive Power"
      m_Q_buslv_B = "m:Q:buslv:B", "Mvar, Reactive Power"
      m_Q_buslv_C = "m:Q:buslv:C", "Mvar, Reactive Power"
      m_Q_buslv_N = "m:Q:buslv:N", "Mvar, Reactive Power"
      m_Q_bushvn = "m:Q:bushvn", "Mvar, Reactive Power"
      m_Q_busmvn = "m:Q:busmvn", "Mvar, Reactive Power"
      m_Q_buslvn = "m:Q:buslvn", "Mvar, Reactive Power"
      m_S_bushv_A = "m:S:bushv:A", "MVA, Apparent Power"
      m_S_bushv_B = "m:S:bushv:B", "MVA, Apparent Power"
      m_S_bushv_C = "m:S:bushv:C", "MVA, Apparent Power"
      m_S_bushv_N = "m:S:bushv:N", "MVA, Apparent Power"
      m_S_busmv_A = "m:S:busmv:A", "MVA, Apparent Power"
      m_S_busmv_B = "m:S:busmv:B", "MVA, Apparent Power"
      m_S_busmv_C = "m:S:busmv:C", "MVA, Apparent Power"
      m_S_busmv_N = "m:S:busmv:N", "MVA, Apparent Power"
      m_S_buslv_A = "m:S:buslv:A", "MVA, Apparent Power"
      m_S_buslv_B = "m:S:buslv:B", "MVA, Apparent Power"
      m_S_buslv_C = "m:S:buslv:C", "MVA, Apparent Power"
      m_S_buslv_N = "m:S:buslv:N", "MVA, Apparent Power"
      m_S_bushvn = "m:S:bushvn", "MVA, Apparent Power"
      m_S_busmvn = "m:S:busmvn", "MVA, Apparent Power"
      m_S_buslvn = "m:S:buslvn", "MVA, Apparent Power"
      m_cosphi_bushv_A = "m:cosphi:bushv:A", "Power Factor"
      m_cosphi_bushv_B = "m:cosphi:bushv:B", "Power Factor"
      m_cosphi_bushv_C = "m:cosphi:bushv:C", "Power Factor"
      m_cosphi_bushv_N = "m:cosphi:bushv:N", "Power Factor"
      m_cosphi_busmv_A = "m:cosphi:busmv:A", "Power Factor"
      m_cosphi_busmv_B = "m:cosphi:busmv:B", "Power Factor"
      m_cosphi_busmv_C = "m:cosphi:busmv:C", "Power Factor"
      m_cosphi_busmv_N = "m:cosphi:busmv:N", "Power Factor"
      m_cosphi_buslv_A = "m:cosphi:buslv:A", "Power Factor"
      m_cosphi_buslv_B = "m:cosphi:buslv:B", "Power Factor"
      m_cosphi_buslv_C = "m:cosphi:buslv:C", "Power Factor"
      m_cosphi_buslv_N = "m:cosphi:buslv:N", "Power Factor"
      m_cosphi_bushvn = "m:cosphi:bushvn", "Power Factor"
      m_cosphi_busmvn = "m:cosphi:busmvn", "Power Factor"
      m_cosphi_buslvn = "m:cosphi:buslvn", "Power Factor"
      m_tanphi_bushv_A = "m:tanphi:bushv:A", "tan(phi)"
      m_tanphi_bushv_B = "m:tanphi:bushv:B", "tan(phi)"
      m_tanphi_bushv_C = "m:tanphi:bushv:C", "tan(phi)"
      m_tanphi_bushv_N = "m:tanphi:bushv:N", "tan(phi)"
      m_tanphi_busmv_A = "m:tanphi:busmv:A", "tan(phi)"
      m_tanphi_busmv_B = "m:tanphi:busmv:B", "tan(phi)"
      m_tanphi_busmv_C = "m:tanphi:busmv:C", "tan(phi)"
      m_tanphi_busmv_N = "m:tanphi:busmv:N", "tan(phi)"
      m_tanphi_buslv_A = "m:tanphi:buslv:A", "tan(phi)"
      m_tanphi_buslv_B = "m:tanphi:buslv:B", "tan(phi)"
      m_tanphi_buslv_C = "m:tanphi:buslv:C", "tan(phi)"
      m_tanphi_buslv_N = "m:tanphi:buslv:N", "tan(phi)"
      m_tanphi_bushvn = "m:tanphi:bushvn", "tan(phi)"
      m_tanphi_busmvn = "m:tanphi:busmvn", "tan(phi)"
      m_tanphi_buslvn = "m:tanphi:buslvn", "tan(phi)"
      m_Psum_bushv = "m:Psum:bushv", "MW, Total Active Power"
      m_Psum_busmv = "m:Psum:busmv", "MW, Total Active Power"
      m_Psum_buslv = "m:Psum:buslv", "MW, Total Active Power"
      m_Psum_bushvn = "m:Psum:bushvn", "MW, Total Active Power"
      m_Psum_busmvn = "m:Psum:busmvn", "MW, Total Active Power"
      m_Psum_buslvn = "m:Psum:buslvn", "MW, Total Active Power"
      m_Qsum_bushv = "m:Qsum:bushv", "Mvar, Total Reactive Power"
      m_Qsum_busmv = "m:Qsum:busmv", "Mvar, Total Reactive Power"
      m_Qsum_buslv = "m:Qsum:buslv", "Mvar, Total Reactive Power"
      m_Qsum_bushvn = "m:Qsum:bushvn", "Mvar, Total Reactive Power"
      m_Qsum_busmvn = "m:Qsum:busmvn", "Mvar, Total Reactive Power"
      m_Qsum_buslvn = "m:Qsum:buslvn", "Mvar, Total Reactive Power"
      m_Ssum_bushv = "m:Ssum:bushv", "MVA, Total Apparent Power"
      m_Ssum_busmv = "m:Ssum:busmv", "MVA, Total Apparent Power"
      m_Ssum_buslv = "m:Ssum:buslv", "MVA, Total Apparent Power"
      m_Ssum_bushvn = "m:Ssum:bushvn", "MVA, Total Apparent Power"
      m_Ssum_busmvn = "m:Ssum:busmvn", "MVA, Total Apparent Power"
      m_Ssum_buslvn = "m:Ssum:buslvn", "MVA, Total Apparent Power"
      m_cosphisum_bushv = "m:cosphisum:bushv", "Total Power Factor"
      m_cosphisum_busmv = "m:cosphisum:busmv", "Total Power Factor"
      m_cosphisum_buslv = "m:cosphisum:buslv", "Total Power Factor"
      m_cosphisum_bushvn = "m:cosphisum:bushvn", "Total Power Factor"
      m_cosphisum_busmvn = "m:cosphisum:busmvn", "Total Power Factor"
      m_cosphisum_buslvn = "m:cosphisum:buslvn", "Total Power Factor"
      m_tanphisum_bushv = "m:tanphisum:bushv", "Total tan(phi)"
      m_tanphisum_busmv = "m:tanphisum:busmv", "Total tan(phi)"
      m_tanphisum_buslv = "m:tanphisum:buslv", "Total tan(phi)"
      m_tanphisum_bushvn = "m:tanphisum:bushvn", "Total tan(phi)"
      m_tanphisum_busmvn = "m:tanphisum:busmvn", "Total tan(phi)"
      m_tanphisum_buslvn = "m:tanphisum:buslvn", "Total tan(phi)"
      m_Pln_bushv_A = "m:Pln:bushv:A", "MW, Active Power (Line-Neutral)"
      m_Pln_bushv_B = "m:Pln:bushv:B", "MW, Active Power (Line-Neutral)"
      m_Pln_bushv_C = "m:Pln:bushv:C", "MW, Active Power (Line-Neutral)"
      m_Pln_bushv_N = "m:Pln:bushv:N", "MW, Active Power (Line-Neutral)"
      m_Pln_busmv_A = "m:Pln:busmv:A", "MW, Active Power (Line-Neutral)"
      m_Pln_busmv_B = "m:Pln:busmv:B", "MW, Active Power (Line-Neutral)"
      m_Pln_busmv_C = "m:Pln:busmv:C", "MW, Active Power (Line-Neutral)"
      m_Pln_busmv_N = "m:Pln:busmv:N", "MW, Active Power (Line-Neutral)"
      m_Pln_buslv_A = "m:Pln:buslv:A", "MW, Active Power (Line-Neutral)"
      m_Pln_buslv_B = "m:Pln:buslv:B", "MW, Active Power (Line-Neutral)"
      m_Pln_buslv_C = "m:Pln:buslv:C", "MW, Active Power (Line-Neutral)"
      m_Pln_buslv_N = "m:Pln:buslv:N", "MW, Active Power (Line-Neutral)"
      m_Pln_bushvn = "m:Pln:bushvn", "MW, Active Power (Line-Neutral)"
      m_Pln_busmvn = "m:Pln:busmvn", "MW, Active Power (Line-Neutral)"
      m_Pln_buslvn = "m:Pln:buslvn", "MW, Active Power (Line-Neutral)"
      m_Qln_bushv_A = "m:Qln:bushv:A", "Mvar, Reactive Power (Line-Neutral)"
      m_Qln_bushv_B = "m:Qln:bushv:B", "Mvar, Reactive Power (Line-Neutral)"
      m_Qln_bushv_C = "m:Qln:bushv:C", "Mvar, Reactive Power (Line-Neutral)"
      m_Qln_bushv_N = "m:Qln:bushv:N", "Mvar, Reactive Power (Line-Neutral)"
      m_Qln_busmv_A = "m:Qln:busmv:A", "Mvar, Reactive Power (Line-Neutral)"
      m_Qln_busmv_B = "m:Qln:busmv:B", "Mvar, Reactive Power (Line-Neutral)"
      m_Qln_busmv_C = "m:Qln:busmv:C", "Mvar, Reactive Power (Line-Neutral)"
      m_Qln_busmv_N = "m:Qln:busmv:N", "Mvar, Reactive Power (Line-Neutral)"
      m_Qln_buslv_A = "m:Qln:buslv:A", "Mvar, Reactive Power (Line-Neutral)"
      m_Qln_buslv_B = "m:Qln:buslv:B", "Mvar, Reactive Power (Line-Neutral)"
      m_Qln_buslv_C = "m:Qln:buslv:C", "Mvar, Reactive Power (Line-Neutral)"
      m_Qln_buslv_N = "m:Qln:buslv:N", "Mvar, Reactive Power (Line-Neutral)"
      m_Qln_bushvn = "m:Qln:bushvn", "Mvar, Reactive Power (Line-Neutral)"
      m_Qln_busmvn = "m:Qln:busmvn", "Mvar, Reactive Power (Line-Neutral)"
      m_Qln_buslvn = "m:Qln:buslvn", "Mvar, Reactive Power (Line-Neutral)"
      m_P1_bushv = "m:P1:bushv", "MW, Positive Sequence Active Power"
      m_P1_busmv = "m:P1:busmv", "MW, Positive Sequence Active Power"
      m_P1_buslv = "m:P1:buslv", "MW, Positive Sequence Active Power"
      m_P1_bushvn = "m:P1:bushvn", "MW, Positive Sequence Active Power"
      m_P1_busmvn = "m:P1:busmvn", "MW, Positive Sequence Active Power"
      m_P1_buslvn = "m:P1:buslvn", "MW, Positive Sequence Active Power"
      m_Q1_bushv = "m:Q1:bushv", "Mvar, Positive Sequence Reactive Power"
      m_Q1_busmv = "m:Q1:busmv", "Mvar, Positive Sequence Reactive Power"
      m_Q1_buslv = "m:Q1:buslv", "Mvar, Positive Sequence Reactive Power"
      m_Q1_bushvn = "m:Q1:bushvn", "Mvar, Positive Sequence Reactive Power"
      m_Q1_busmvn = "m:Q1:busmvn", "Mvar, Positive Sequence Reactive Power"
      m_Q1_buslvn = "m:Q1:buslvn", "Mvar, Positive Sequence Reactive Power"
      m_P2_bushv = "m:P2:bushv", "MW, Negative Sequence Active Power"
      m_P2_busmv = "m:P2:busmv", "MW, Negative Sequence Active Power"
      m_P2_buslv = "m:P2:buslv", "MW, Negative Sequence Active Power"
      m_P2_bushvn = "m:P2:bushvn", "MW, Negative Sequence Active Power"
      m_P2_busmvn = "m:P2:busmvn", "MW, Negative Sequence Active Power"
      m_P2_buslvn = "m:P2:buslvn", "MW, Negative Sequence Active Power"
      m_Q2_bushv = "m:Q2:bushv", "Mvar, Negative Sequence Reactive Power"
      m_Q2_busmv = "m:Q2:busmv", "Mvar, Negative Sequence Reactive Power"
      m_Q2_buslv = "m:Q2:buslv", "Mvar, Negative Sequence Reactive Power"
      m_Q2_bushvn = "m:Q2:bushvn", "Mvar, Negative Sequence Reactive Power"
      m_Q2_busmvn = "m:Q2:busmvn", "Mvar, Negative Sequence Reactive Power"
      m_Q2_buslvn = "m:Q2:buslvn", "Mvar, Negative Sequence Reactive Power"
      m_P0_bushv = "m:P0:bushv", "MW, Zero Sequence Active Power"
      m_P0_busmv = "m:P0:busmv", "MW, Zero Sequence Active Power"
      m_P0_buslv = "m:P0:buslv", "MW, Zero Sequence Active Power"
      m_P0_bushvn = "m:P0:bushvn", "MW, Zero Sequence Active Power"
      m_P0_busmvn = "m:P0:busmvn", "MW, Zero Sequence Active Power"
      m_P0_buslvn = "m:P0:buslvn", "MW, Zero Sequence Active Power"
      m_Q0_bushv = "m:Q0:bushv", "Mvar, Zero Sequence Reactive Power"
      m_Q0_busmv = "m:Q0:busmv", "Mvar, Zero Sequence Reactive Power"
      m_Q0_buslv = "m:Q0:buslv", "Mvar, Zero Sequence Reactive Power"
      m_Q0_bushvn = "m:Q0:bushvn", "Mvar, Zero Sequence Reactive Power"
      m_Q0_busmvn = "m:Q0:busmvn", "Mvar, Zero Sequence Reactive Power"
      m_Q0_buslvn = "m:Q0:buslvn", "Mvar, Zero Sequence Reactive Power"
      m_i1P_bushv = "m:i1P:bushv", "p.u., Positive-Sequence Active Current"
      m_i1P_busmv = "m:i1P:busmv", "p.u., Positive-Sequence Active Current"
      m_i1P_buslv = "m:i1P:buslv", "p.u., Positive-Sequence Active Current"
      m_i1P_bushvn = "m:i1P:bushvn", "p.u., Positive-Sequence Active Current"
      m_i1P_busmvn = "m:i1P:busmvn", "p.u., Positive-Sequence Active Current"
      m_i1P_buslvn = "m:i1P:buslvn", "p.u., Positive-Sequence Active Current"
      m_i1Q_bushv = "m:i1Q:bushv", "p.u., Positive-Sequence Reactive Current"
      m_i1Q_busmv = "m:i1Q:busmv", "p.u., Positive-Sequence Reactive Current"
      m_i1Q_buslv = "m:i1Q:buslv", "p.u., Positive-Sequence Reactive Current"
      m_i1Q_bushvn = "m:i1Q:bushvn", "p.u., Positive-Sequence Reactive Current"
      m_i1Q_busmvn = "m:i1Q:busmvn", "p.u., Positive-Sequence Reactive Current"
      m_i1Q_buslvn = "m:i1Q:buslvn", "p.u., Positive-Sequence Reactive Current"
      m_I1P_bushv = "m:I1P:bushv", "kA, Positive-Sequence Active Current"
      m_I1P_busmv = "m:I1P:busmv", "kA, Positive-Sequence Active Current"
      m_I1P_buslv = "m:I1P:buslv", "kA, Positive-Sequence Active Current"
      m_I1P_bushvn = "m:I1P:bushvn", "kA, Positive-Sequence Active Current"
      m_I1P_busmvn = "m:I1P:busmvn", "kA, Positive-Sequence Active Current"
      m_I1P_buslvn = "m:I1P:buslvn", "kA, Positive-Sequence Active Current"
      m_I1Q_bushv = "m:I1Q:bushv", "kA, Positive-Sequence Reactive Current"
      m_I1Q_busmv = "m:I1Q:busmv", "kA, Positive-Sequence Reactive Current"
      m_I1Q_buslv = "m:I1Q:buslv", "kA, Positive-Sequence Reactive Current"
      m_I1Q_bushvn = "m:I1Q:bushvn", "kA, Positive-Sequence Reactive Current"
      m_I1Q_busmvn = "m:I1Q:busmvn", "kA, Positive-Sequence Reactive Current"
      m_I1Q_buslvn = "m:I1Q:buslvn", "kA, Positive-Sequence Reactive Current"
      m_i2P_bushv = "m:i2P:bushv", "p.u., Negative-Sequence Active Current"
      m_i2P_busmv = "m:i2P:busmv", "p.u., Negative-Sequence Active Current"
      m_i2P_buslv = "m:i2P:buslv", "p.u., Negative-Sequence Active Current"
      m_i2P_bushvn = "m:i2P:bushvn", "p.u., Negative-Sequence Active Current"
      m_i2P_busmvn = "m:i2P:busmvn", "p.u., Negative-Sequence Active Current"
      m_i2P_buslvn = "m:i2P:buslvn", "p.u., Negative-Sequence Active Current"
      m_i2Q_bushv = "m:i2Q:bushv", "p.u., Negative-Sequence Reactive Current"
      m_i2Q_busmv = "m:i2Q:busmv", "p.u., Negative-Sequence Reactive Current"
      m_i2Q_buslv = "m:i2Q:buslv", "p.u., Negative-Sequence Reactive Current"
      m_i2Q_bushvn = "m:i2Q:bushvn", "p.u., Negative-Sequence Reactive Current"
      m_i2Q_busmvn = "m:i2Q:busmvn", "p.u., Negative-Sequence Reactive Current"
      m_i2Q_buslvn = "m:i2Q:buslvn", "p.u., Negative-Sequence Reactive Current"
      m_I2P_bushv = "m:I2P:bushv", "kA, Negative-Sequence Active Current"
      m_I2P_busmv = "m:I2P:busmv", "kA, Negative-Sequence Active Current"
      m_I2P_buslv = "m:I2P:buslv", "kA, Negative-Sequence Active Current"
      m_I2P_bushvn = "m:I2P:bushvn", "kA, Negative-Sequence Active Current"
      m_I2P_busmvn = "m:I2P:busmvn", "kA, Negative-Sequence Active Current"
      m_I2P_buslvn = "m:I2P:buslvn", "kA, Negative-Sequence Active Current"
      m_I2Q_bushv = "m:I2Q:bushv", "kA, Negative-Sequence Reactive Current"
      m_I2Q_busmv = "m:I2Q:busmv", "kA, Negative-Sequence Reactive Current"
      m_I2Q_buslv = "m:I2Q:buslv", "kA, Negative-Sequence Reactive Current"
      m_I2Q_bushvn = "m:I2Q:bushvn", "kA, Negative-Sequence Reactive Current"
      m_I2Q_busmvn = "m:I2Q:busmvn", "kA, Negative-Sequence Reactive Current"
      m_I2Q_buslvn = "m:I2Q:buslvn", "kA, Negative-Sequence Reactive Current"
      m_ur_bushv_A = "m:ur:bushv:A", "p.u., Phase Voltage, Real Part"
      m_ur_bushv_B = "m:ur:bushv:B", "p.u., Phase Voltage, Real Part"
      m_ur_bushv_C = "m:ur:bushv:C", "p.u., Phase Voltage, Real Part"
      m_ur_bushv_N = "m:ur:bushv:N", "p.u., Phase Voltage, Real Part"
      m_ur_busmv_A = "m:ur:busmv:A", "p.u., Phase Voltage, Real Part"
      m_ur_busmv_B = "m:ur:busmv:B", "p.u., Phase Voltage, Real Part"
      m_ur_busmv_C = "m:ur:busmv:C", "p.u., Phase Voltage, Real Part"
      m_ur_busmv_N = "m:ur:busmv:N", "p.u., Phase Voltage, Real Part"
      m_ur_buslv_A = "m:ur:buslv:A", "p.u., Phase Voltage, Real Part"
      m_ur_buslv_B = "m:ur:buslv:B", "p.u., Phase Voltage, Real Part"
      m_ur_buslv_C = "m:ur:buslv:C", "p.u., Phase Voltage, Real Part"
      m_ur_buslv_N = "m:ur:buslv:N", "p.u., Phase Voltage, Real Part"
      m_ur_bushvn = "m:ur:bushvn", "p.u., Phase Voltage, Real Part"
      m_ur_busmvn = "m:ur:busmvn", "p.u., Phase Voltage, Real Part"
      m_ur_buslvn = "m:ur:buslvn", "p.u., Phase Voltage, Real Part"
      m_ui_bushv_A = "m:ui:bushv:A", "p.u., Phase Voltage, Imaginary Part"
      m_ui_bushv_B = "m:ui:bushv:B", "p.u., Phase Voltage, Imaginary Part"
      m_ui_bushv_C = "m:ui:bushv:C", "p.u., Phase Voltage, Imaginary Part"
      m_ui_bushv_N = "m:ui:bushv:N", "p.u., Phase Voltage, Imaginary Part"
      m_ui_busmv_A = "m:ui:busmv:A", "p.u., Phase Voltage, Imaginary Part"
      m_ui_busmv_B = "m:ui:busmv:B", "p.u., Phase Voltage, Imaginary Part"
      m_ui_busmv_C = "m:ui:busmv:C", "p.u., Phase Voltage, Imaginary Part"
      m_ui_busmv_N = "m:ui:busmv:N", "p.u., Phase Voltage, Imaginary Part"
      m_ui_buslv_A = "m:ui:buslv:A", "p.u., Phase Voltage, Imaginary Part"
      m_ui_buslv_B = "m:ui:buslv:B", "p.u., Phase Voltage, Imaginary Part"
      m_ui_buslv_C = "m:ui:buslv:C", "p.u., Phase Voltage, Imaginary Part"
      m_ui_buslv_N = "m:ui:buslv:N", "p.u., Phase Voltage, Imaginary Part"
      m_ui_bushvn = "m:ui:bushvn", "p.u., Phase Voltage, Imaginary Part"
      m_ui_busmvn = "m:ui:busmvn", "p.u., Phase Voltage, Imaginary Part"
      m_ui_buslvn = "m:ui:buslvn", "p.u., Phase Voltage, Imaginary Part"
      m_u_bushv_A = "m:u:bushv:A", "p.u., Phase Voltage, Magnitude"
      m_u_bushv_B = "m:u:bushv:B", "p.u., Phase Voltage, Magnitude"
      m_u_bushv_C = "m:u:bushv:C", "p.u., Phase Voltage, Magnitude"
      m_u_bushv_N = "m:u:bushv:N", "p.u., Phase Voltage, Magnitude"
      m_u_busmv_A = "m:u:busmv:A", "p.u., Phase Voltage, Magnitude"
      m_u_busmv_B = "m:u:busmv:B", "p.u., Phase Voltage, Magnitude"
      m_u_busmv_C = "m:u:busmv:C", "p.u., Phase Voltage, Magnitude"
      m_u_busmv_N = "m:u:busmv:N", "p.u., Phase Voltage, Magnitude"
      m_u_buslv_A = "m:u:buslv:A", "p.u., Phase Voltage, Magnitude"
      m_u_buslv_B = "m:u:buslv:B", "p.u., Phase Voltage, Magnitude"
      m_u_buslv_C = "m:u:buslv:C", "p.u., Phase Voltage, Magnitude"
      m_u_buslv_N = "m:u:buslv:N", "p.u., Phase Voltage, Magnitude"
      m_u_bushvn = "m:u:bushvn", "p.u., Phase Voltage, Magnitude"
      m_u_busmvn = "m:u:busmvn", "p.u., Phase Voltage, Magnitude"
      m_u_buslvn = "m:u:buslvn", "p.u., Phase Voltage, Magnitude"
      m_ir_bushv_A = "m:ir:bushv:A", "p.u., Phase Current, Real Part"
      m_ir_bushv_B = "m:ir:bushv:B", "p.u., Phase Current, Real Part"
      m_ir_bushv_C = "m:ir:bushv:C", "p.u., Phase Current, Real Part"
      m_ir_bushv_N = "m:ir:bushv:N", "p.u., Phase Current, Real Part"
      m_ir_busmv_A = "m:ir:busmv:A", "p.u., Phase Current, Real Part"
      m_ir_busmv_B = "m:ir:busmv:B", "p.u., Phase Current, Real Part"
      m_ir_busmv_C = "m:ir:busmv:C", "p.u., Phase Current, Real Part"
      m_ir_busmv_N = "m:ir:busmv:N", "p.u., Phase Current, Real Part"
      m_ir_buslv_A = "m:ir:buslv:A", "p.u., Phase Current, Real Part"
      m_ir_buslv_B = "m:ir:buslv:B", "p.u., Phase Current, Real Part"
      m_ir_buslv_C = "m:ir:buslv:C", "p.u., Phase Current, Real Part"
      m_ir_buslv_N = "m:ir:buslv:N", "p.u., Phase Current, Real Part"
      m_ir_bushvn = "m:ir:bushvn", "p.u., Phase Current, Real Part"
      m_ir_busmvn = "m:ir:busmvn", "p.u., Phase Current, Real Part"
      m_ir_buslvn = "m:ir:buslvn", "p.u., Phase Current, Real Part"
      m_ii_bushv_A = "m:ii:bushv:A", "p.u., Phase Current, Imaginary Part"
      m_ii_bushv_B = "m:ii:bushv:B", "p.u., Phase Current, Imaginary Part"
      m_ii_bushv_C = "m:ii:bushv:C", "p.u., Phase Current, Imaginary Part"
      m_ii_bushv_N = "m:ii:bushv:N", "p.u., Phase Current, Imaginary Part"
      m_ii_busmv_A = "m:ii:busmv:A", "p.u., Phase Current, Imaginary Part"
      m_ii_busmv_B = "m:ii:busmv:B", "p.u., Phase Current, Imaginary Part"
      m_ii_busmv_C = "m:ii:busmv:C", "p.u., Phase Current, Imaginary Part"
      m_ii_busmv_N = "m:ii:busmv:N", "p.u., Phase Current, Imaginary Part"
      m_ii_buslv_A = "m:ii:buslv:A", "p.u., Phase Current, Imaginary Part"
      m_ii_buslv_B = "m:ii:buslv:B", "p.u., Phase Current, Imaginary Part"
      m_ii_buslv_C = "m:ii:buslv:C", "p.u., Phase Current, Imaginary Part"
      m_ii_buslv_N = "m:ii:buslv:N", "p.u., Phase Current, Imaginary Part"
      m_ii_bushvn = "m:ii:bushvn", "p.u., Phase Current, Imaginary Part"
      m_ii_busmvn = "m:ii:busmvn", "p.u., Phase Current, Imaginary Part"
      m_ii_buslvn = "m:ii:buslvn", "p.u., Phase Current, Imaginary Part"
      m_i_bushv_A = "m:i:bushv:A", "p.u., Phase Current, Magnitude"
      m_i_bushv_B = "m:i:bushv:B", "p.u., Phase Current, Magnitude"
      m_i_bushv_C = "m:i:bushv:C", "p.u., Phase Current, Magnitude"
      m_i_bushv_N = "m:i:bushv:N", "p.u., Phase Current, Magnitude"
      m_i_busmv_A = "m:i:busmv:A", "p.u., Phase Current, Magnitude"
      m_i_busmv_B = "m:i:busmv:B", "p.u., Phase Current, Magnitude"
      m_i_busmv_C = "m:i:busmv:C", "p.u., Phase Current, Magnitude"
      m_i_busmv_N = "m:i:busmv:N", "p.u., Phase Current, Magnitude"
      m_i_buslv_A = "m:i:buslv:A", "p.u., Phase Current, Magnitude"
      m_i_buslv_B = "m:i:buslv:B", "p.u., Phase Current, Magnitude"
      m_i_buslv_C = "m:i:buslv:C", "p.u., Phase Current, Magnitude"
      m_i_buslv_N = "m:i:buslv:N", "p.u., Phase Current, Magnitude"
      m_i_bushvn = "m:i:bushvn", "p.u., Phase Current, Magnitude"
      m_i_busmvn = "m:i:busmvn", "p.u., Phase Current, Magnitude"
      m_i_buslvn = "m:i:buslvn", "p.u., Phase Current, Magnitude"
      m_phiui_bushv_A = "m:phiui:bushv:A", "deg, Angle between Voltage and Current"
      m_phiui_bushv_B = "m:phiui:bushv:B", "deg, Angle between Voltage and Current"
      m_phiui_bushv_C = "m:phiui:bushv:C", "deg, Angle between Voltage and Current"
      m_phiui_bushv_N = "m:phiui:bushv:N", "deg, Angle between Voltage and Current"
      m_phiui_busmv_A = "m:phiui:busmv:A", "deg, Angle between Voltage and Current"
      m_phiui_busmv_B = "m:phiui:busmv:B", "deg, Angle between Voltage and Current"
      m_phiui_busmv_C = "m:phiui:busmv:C", "deg, Angle between Voltage and Current"
      m_phiui_busmv_N = "m:phiui:busmv:N", "deg, Angle between Voltage and Current"
      m_phiui_buslv_A = "m:phiui:buslv:A", "deg, Angle between Voltage and Current"
      m_phiui_buslv_B = "m:phiui:buslv:B", "deg, Angle between Voltage and Current"
      m_phiui_buslv_C = "m:phiui:buslv:C", "deg, Angle between Voltage and Current"
      m_phiui_buslv_N = "m:phiui:buslv:N", "deg, Angle between Voltage and Current"
      m_phiui_bushvn = "m:phiui:bushvn", "deg, Angle between Voltage and Current"
      m_phiui_busmvn = "m:phiui:busmvn", "deg, Angle between Voltage and Current"
      m_phiui_buslvn = "m:phiui:buslvn", "deg, Angle between Voltage and Current"
      m_phiu0i0_bushv = "m:phiu0i0:bushv", "deg, Angle between Voltage and Current in zero-sequence system"
      m_phiu0i0_busmv = "m:phiu0i0:busmv", "deg, Angle between Voltage and Current in zero-sequence system"
      m_phiu0i0_buslv = "m:phiu0i0:buslv", "deg, Angle between Voltage and Current in zero-sequence system"
      m_phiu0i0_bushvn = "m:phiu0i0:bushvn", "deg, Angle between Voltage and Current in zero-sequence system"
      m_phiu0i0_busmvn = "m:phiu0i0:busmvn", "deg, Angle between Voltage and Current in zero-sequence system"
      m_phiu0i0_buslvn = "m:phiu0i0:buslvn", "deg, Angle between Voltage and Current in zero-sequence system"
      m_phiu1i1_bushv = "m:phiu1i1:bushv", "deg, Angle between Voltage and Current in positive sequence system"
      m_phiu1i1_busmv = "m:phiu1i1:busmv", "deg, Angle between Voltage and Current in positive sequence system"
      m_phiu1i1_buslv = "m:phiu1i1:buslv", "deg, Angle between Voltage and Current in positive sequence system"
      m_phiu1i1_bushvn = "m:phiu1i1:bushvn", "deg, Angle between Voltage and Current in positive sequence system"
      m_phiu1i1_busmvn = "m:phiu1i1:busmvn", "deg, Angle between Voltage and Current in positive sequence system"
      m_phiu1i1_buslvn = "m:phiu1i1:buslvn", "deg, Angle between Voltage and Current in positive sequence system"
      m_phiu2i2_bushv = "m:phiu2i2:bushv", "deg, Angle between Voltage and Current in negative sequence system"
      m_phiu2i2_busmv = "m:phiu2i2:busmv", "deg, Angle between Voltage and Current in negative sequence system"
      m_phiu2i2_buslv = "m:phiu2i2:buslv", "deg, Angle between Voltage and Current in negative sequence system"
      m_phiu2i2_bushvn = "m:phiu2i2:bushvn", "deg, Angle between Voltage and Current in negative sequence system"
      m_phiu2i2_busmvn = "m:phiu2i2:busmvn", "deg, Angle between Voltage and Current in negative sequence system"
      m_phiu2i2_buslvn = "m:phiu2i2:buslvn", "deg, Angle between Voltage and Current in negative sequence system"
      m_Tfct_bushv = "m:Tfct:bushv", "s, Fault Clearing Time"
      m_Tfct_busmv = "m:Tfct:busmv", "s, Fault Clearing Time"
      m_Tfct_buslv = "m:Tfct:buslv", "s, Fault Clearing Time"
      m_Tfct_bushvn = "m:Tfct:bushvn", "s, Fault Clearing Time"
      m_Tfct_busmvn = "m:Tfct:busmvn", "s, Fault Clearing Time"
      m_Tfct_buslvn = "m:Tfct:buslvn", "s, Fault Clearing Time"
      m_TfctPh_bushv_A = "m:TfctPh:bushv:A", "s, Fault Clearing Time"
      m_TfctPh_bushv_B = "m:TfctPh:bushv:B", "s, Fault Clearing Time"
      m_TfctPh_bushv_C = "m:TfctPh:bushv:C", "s, Fault Clearing Time"
      m_TfctPh_bushv_N = "m:TfctPh:bushv:N", "s, Fault Clearing Time"
      m_TfctPh_busmv_A = "m:TfctPh:busmv:A", "s, Fault Clearing Time"
      m_TfctPh_busmv_B = "m:TfctPh:busmv:B", "s, Fault Clearing Time"
      m_TfctPh_busmv_C = "m:TfctPh:busmv:C", "s, Fault Clearing Time"
      m_TfctPh_busmv_N = "m:TfctPh:busmv:N", "s, Fault Clearing Time"
      m_TfctPh_buslv_A = "m:TfctPh:buslv:A", "s, Fault Clearing Time"
      m_TfctPh_buslv_B = "m:TfctPh:buslv:B", "s, Fault Clearing Time"
      m_TfctPh_buslv_C = "m:TfctPh:buslv:C", "s, Fault Clearing Time"
      m_TfctPh_buslv_N = "m:TfctPh:buslv:N", "s, Fault Clearing Time"
      m_TfctPh_bushvn = "m:TfctPh:bushvn", "s, Fault Clearing Time"
      m_TfctPh_busmvn = "m:TfctPh:busmvn", "s, Fault Clearing Time"
      m_TfctPh_buslvn = "m:TfctPh:buslvn", "s, Fault Clearing Time"
      m_Brkload_bushv = "m:Brkload:bushv", "%, Max. Breaker Loading"
      m_Brkload_busmv = "m:Brkload:busmv", "%, Max. Breaker Loading"
      m_Brkload_buslv = "m:Brkload:buslv", "%, Max. Breaker Loading"
      m_Brkload_bushvn = "m:Brkload:bushvn", "%, Max. Breaker Loading"
      m_Brkload_busmvn = "m:Brkload:busmvn", "%, Max. Breaker Loading"
      m_Brkload_buslvn = "m:Brkload:buslvn", "%, Max. Breaker Loading"
      m_BrkloadPh_bushv_A = "m:BrkloadPh:bushv:A", "%, Breaker Loading"
      m_BrkloadPh_bushv_B = "m:BrkloadPh:bushv:B", "%, Breaker Loading"
      m_BrkloadPh_bushv_C = "m:BrkloadPh:bushv:C", "%, Breaker Loading"
      m_BrkloadPh_bushv_N = "m:BrkloadPh:bushv:N", "%, Breaker Loading"
      m_BrkloadPh_busmv_A = "m:BrkloadPh:busmv:A", "%, Breaker Loading"
      m_BrkloadPh_busmv_B = "m:BrkloadPh:busmv:B", "%, Breaker Loading"
      m_BrkloadPh_busmv_C = "m:BrkloadPh:busmv:C", "%, Breaker Loading"
      m_BrkloadPh_busmv_N = "m:BrkloadPh:busmv:N", "%, Breaker Loading"
      m_BrkloadPh_buslv_A = "m:BrkloadPh:buslv:A", "%, Breaker Loading"
      m_BrkloadPh_buslv_B = "m:BrkloadPh:buslv:B", "%, Breaker Loading"
      m_BrkloadPh_buslv_C = "m:BrkloadPh:buslv:C", "%, Breaker Loading"
      m_BrkloadPh_buslv_N = "m:BrkloadPh:buslv:N", "%, Breaker Loading"
      m_BrkloadPh_bushvn = "m:BrkloadPh:bushvn", "%, Breaker Loading"
      m_BrkloadPh_busmvn = "m:BrkloadPh:busmvn", "%, Breaker Loading"
      m_BrkloadPh_buslvn = "m:BrkloadPh:buslvn", "%, Breaker Loading"
      n_ur_bushv_A = "n:ur:bushv:A", "p.u., Line-Ground Voltage, Real Part"
      n_ur_bushv_B = "n:ur:bushv:B", "p.u., Line-Ground Voltage, Real Part"
      n_ur_bushv_C = "n:ur:bushv:C", "p.u., Line-Ground Voltage, Real Part"
      n_ur_bushv_N = "n:ur:bushv:N", "p.u., Line-Ground Voltage, Real Part"
      n_ur_busmv_A = "n:ur:busmv:A", "p.u., Line-Ground Voltage, Real Part"
      n_ur_busmv_B = "n:ur:busmv:B", "p.u., Line-Ground Voltage, Real Part"
      n_ur_busmv_C = "n:ur:busmv:C", "p.u., Line-Ground Voltage, Real Part"
      n_ur_busmv_N = "n:ur:busmv:N", "p.u., Line-Ground Voltage, Real Part"
      n_ur_buslv_A = "n:ur:buslv:A", "p.u., Line-Ground Voltage, Real Part"
      n_ur_buslv_B = "n:ur:buslv:B", "p.u., Line-Ground Voltage, Real Part"
      n_ur_buslv_C = "n:ur:buslv:C", "p.u., Line-Ground Voltage, Real Part"
      n_ur_buslv_N = "n:ur:buslv:N", "p.u., Line-Ground Voltage, Real Part"
      n_ur_bushvn = "n:ur:bushvn", "p.u., Line-Ground Voltage, Real Part"
      n_ur_busmvn = "n:ur:busmvn", "p.u., Line-Ground Voltage, Real Part"
      n_ur_buslvn = "n:ur:buslvn", "p.u., Line-Ground Voltage, Real Part"
      n_ui_bushv_A = "n:ui:bushv:A", "p.u., Line-Ground Voltage, Imaginary Part"
      n_ui_bushv_B = "n:ui:bushv:B", "p.u., Line-Ground Voltage, Imaginary Part"
      n_ui_bushv_C = "n:ui:bushv:C", "p.u., Line-Ground Voltage, Imaginary Part"
      n_ui_bushv_N = "n:ui:bushv:N", "p.u., Line-Ground Voltage, Imaginary Part"
      n_ui_busmv_A = "n:ui:busmv:A", "p.u., Line-Ground Voltage, Imaginary Part"
      n_ui_busmv_B = "n:ui:busmv:B", "p.u., Line-Ground Voltage, Imaginary Part"
      n_ui_busmv_C = "n:ui:busmv:C", "p.u., Line-Ground Voltage, Imaginary Part"
      n_ui_busmv_N = "n:ui:busmv:N", "p.u., Line-Ground Voltage, Imaginary Part"
      n_ui_buslv_A = "n:ui:buslv:A", "p.u., Line-Ground Voltage, Imaginary Part"
      n_ui_buslv_B = "n:ui:buslv:B", "p.u., Line-Ground Voltage, Imaginary Part"
      n_ui_buslv_C = "n:ui:buslv:C", "p.u., Line-Ground Voltage, Imaginary Part"
      n_ui_buslv_N = "n:ui:buslv:N", "p.u., Line-Ground Voltage, Imaginary Part"
      n_ui_bushvn = "n:ui:bushvn", "p.u., Line-Ground Voltage, Imaginary Part"
      n_ui_busmvn = "n:ui:busmvn", "p.u., Line-Ground Voltage, Imaginary Part"
      n_ui_buslvn = "n:ui:buslvn", "p.u., Line-Ground Voltage, Imaginary Part"
      n_u_bushv_A = "n:u:bushv:A", "p.u., Line-Ground Voltage, Magnitude"
      n_u_bushv_B = "n:u:bushv:B", "p.u., Line-Ground Voltage, Magnitude"
      n_u_bushv_C = "n:u:bushv:C", "p.u., Line-Ground Voltage, Magnitude"
      n_u_bushv_N = "n:u:bushv:N", "p.u., Line-Ground Voltage, Magnitude"
      n_u_busmv_A = "n:u:busmv:A", "p.u., Line-Ground Voltage, Magnitude"
      n_u_busmv_B = "n:u:busmv:B", "p.u., Line-Ground Voltage, Magnitude"
      n_u_busmv_C = "n:u:busmv:C", "p.u., Line-Ground Voltage, Magnitude"
      n_u_busmv_N = "n:u:busmv:N", "p.u., Line-Ground Voltage, Magnitude"
      n_u_buslv_A = "n:u:buslv:A", "p.u., Line-Ground Voltage, Magnitude"
      n_u_buslv_B = "n:u:buslv:B", "p.u., Line-Ground Voltage, Magnitude"
      n_u_buslv_C = "n:u:buslv:C", "p.u., Line-Ground Voltage, Magnitude"
      n_u_buslv_N = "n:u:buslv:N", "p.u., Line-Ground Voltage, Magnitude"
      n_u_bushvn = "n:u:bushvn", "p.u., Line-Ground Voltage, Magnitude"
      n_u_busmvn = "n:u:busmvn", "p.u., Line-Ground Voltage, Magnitude"
      n_u_buslvn = "n:u:buslvn", "p.u., Line-Ground Voltage, Magnitude"
      n_upc_bushv_A = "n:upc:bushv:A", "%, Line-Ground Voltage, Magnitude"
      n_upc_bushv_B = "n:upc:bushv:B", "%, Line-Ground Voltage, Magnitude"
      n_upc_bushv_C = "n:upc:bushv:C", "%, Line-Ground Voltage, Magnitude"
      n_upc_bushv_N = "n:upc:bushv:N", "%, Line-Ground Voltage, Magnitude"
      n_upc_busmv_A = "n:upc:busmv:A", "%, Line-Ground Voltage, Magnitude"
      n_upc_busmv_B = "n:upc:busmv:B", "%, Line-Ground Voltage, Magnitude"
      n_upc_busmv_C = "n:upc:busmv:C", "%, Line-Ground Voltage, Magnitude"
      n_upc_busmv_N = "n:upc:busmv:N", "%, Line-Ground Voltage, Magnitude"
      n_upc_buslv_A = "n:upc:buslv:A", "%, Line-Ground Voltage, Magnitude"
      n_upc_buslv_B = "n:upc:buslv:B", "%, Line-Ground Voltage, Magnitude"
      n_upc_buslv_C = "n:upc:buslv:C", "%, Line-Ground Voltage, Magnitude"
      n_upc_buslv_N = "n:upc:buslv:N", "%, Line-Ground Voltage, Magnitude"
      n_upc_bushvn = "n:upc:bushvn", "%, Line-Ground Voltage, Magnitude"
      n_upc_busmvn = "n:upc:busmvn", "%, Line-Ground Voltage, Magnitude"
      n_upc_buslvn = "n:upc:buslvn", "%, Line-Ground Voltage, Magnitude"
      n_U_bushv_A = "n:U:bushv:A", "kV, Line-Ground Voltage, Magnitude"
      n_U_bushv_B = "n:U:bushv:B", "kV, Line-Ground Voltage, Magnitude"
      n_U_bushv_C = "n:U:bushv:C", "kV, Line-Ground Voltage, Magnitude"
      n_U_bushv_N = "n:U:bushv:N", "kV, Line-Ground Voltage, Magnitude"
      n_U_busmv_A = "n:U:busmv:A", "kV, Line-Ground Voltage, Magnitude"
      n_U_busmv_B = "n:U:busmv:B", "kV, Line-Ground Voltage, Magnitude"
      n_U_busmv_C = "n:U:busmv:C", "kV, Line-Ground Voltage, Magnitude"
      n_U_busmv_N = "n:U:busmv:N", "kV, Line-Ground Voltage, Magnitude"
      n_U_buslv_A = "n:U:buslv:A", "kV, Line-Ground Voltage, Magnitude"
      n_U_buslv_B = "n:U:buslv:B", "kV, Line-Ground Voltage, Magnitude"
      n_U_buslv_C = "n:U:buslv:C", "kV, Line-Ground Voltage, Magnitude"
      n_U_buslv_N = "n:U:buslv:N", "kV, Line-Ground Voltage, Magnitude"
      n_U_bushvn = "n:U:bushvn", "kV, Line-Ground Voltage, Magnitude"
      n_U_busmvn = "n:U:busmvn", "kV, Line-Ground Voltage, Magnitude"
      n_U_buslvn = "n:U:buslvn", "kV, Line-Ground Voltage, Magnitude"
      n_phiu_bushv_A = "n:phiu:bushv:A", "deg, Line-Ground Voltage, Angle"
      n_phiu_bushv_B = "n:phiu:bushv:B", "deg, Line-Ground Voltage, Angle"
      n_phiu_bushv_C = "n:phiu:bushv:C", "deg, Line-Ground Voltage, Angle"
      n_phiu_bushv_N = "n:phiu:bushv:N", "deg, Line-Ground Voltage, Angle"
      n_phiu_busmv_A = "n:phiu:busmv:A", "deg, Line-Ground Voltage, Angle"
      n_phiu_busmv_B = "n:phiu:busmv:B", "deg, Line-Ground Voltage, Angle"
      n_phiu_busmv_C = "n:phiu:busmv:C", "deg, Line-Ground Voltage, Angle"
      n_phiu_busmv_N = "n:phiu:busmv:N", "deg, Line-Ground Voltage, Angle"
      n_phiu_buslv_A = "n:phiu:buslv:A", "deg, Line-Ground Voltage, Angle"
      n_phiu_buslv_B = "n:phiu:buslv:B", "deg, Line-Ground Voltage, Angle"
      n_phiu_buslv_C = "n:phiu:buslv:C", "deg, Line-Ground Voltage, Angle"
      n_phiu_buslv_N = "n:phiu:buslv:N", "deg, Line-Ground Voltage, Angle"
      n_phiu_bushvn = "n:phiu:bushvn", "deg, Line-Ground Voltage, Angle"
      n_phiu_busmvn = "n:phiu:busmvn", "deg, Line-Ground Voltage, Angle"
      n_phiu_buslvn = "n:phiu:buslvn", "deg, Line-Ground Voltage, Angle"
      n_phiurel_bushv = "n:phiurel:bushv", "deg, Voltage, Relative Angle"
      n_phiurel_busmv = "n:phiurel:busmv", "deg, Voltage, Relative Angle"
      n_phiurel_buslv = "n:phiurel:buslv", "deg, Voltage, Relative Angle"
      n_phiurel_bushvn = "n:phiurel:bushvn", "deg, Voltage, Relative Angle"
      n_phiurel_busmvn = "n:phiurel:busmvn", "deg, Voltage, Relative Angle"
      n_phiurel_buslvn = "n:phiurel:buslvn", "deg, Voltage, Relative Angle"
      n_ul_bushv_A = "n:ul:bushv:A", "p.u., Line-Line Voltage, Magnitude"
      n_ul_bushv_B = "n:ul:bushv:B", "p.u., Line-Line Voltage, Magnitude"
      n_ul_bushv_C = "n:ul:bushv:C", "p.u., Line-Line Voltage, Magnitude"
      n_ul_bushv_N = "n:ul:bushv:N", "p.u., Line-Line Voltage, Magnitude"
      n_ul_busmv_A = "n:ul:busmv:A", "p.u., Line-Line Voltage, Magnitude"
      n_ul_busmv_B = "n:ul:busmv:B", "p.u., Line-Line Voltage, Magnitude"
      n_ul_busmv_C = "n:ul:busmv:C", "p.u., Line-Line Voltage, Magnitude"
      n_ul_busmv_N = "n:ul:busmv:N", "p.u., Line-Line Voltage, Magnitude"
      n_ul_buslv_A = "n:ul:buslv:A", "p.u., Line-Line Voltage, Magnitude"
      n_ul_buslv_B = "n:ul:buslv:B", "p.u., Line-Line Voltage, Magnitude"
      n_ul_buslv_C = "n:ul:buslv:C", "p.u., Line-Line Voltage, Magnitude"
      n_ul_buslv_N = "n:ul:buslv:N", "p.u., Line-Line Voltage, Magnitude"
      n_ul_bushvn = "n:ul:bushvn", "p.u., Line-Line Voltage, Magnitude"
      n_ul_busmvn = "n:ul:busmvn", "p.u., Line-Line Voltage, Magnitude"
      n_ul_buslvn = "n:ul:buslvn", "p.u., Line-Line Voltage, Magnitude"
      n_ulpc_bushv_A = "n:ulpc:bushv:A", "%, Line-Line Voltage, Magnitude"
      n_ulpc_bushv_B = "n:ulpc:bushv:B", "%, Line-Line Voltage, Magnitude"
      n_ulpc_bushv_C = "n:ulpc:bushv:C", "%, Line-Line Voltage, Magnitude"
      n_ulpc_bushv_N = "n:ulpc:bushv:N", "%, Line-Line Voltage, Magnitude"
      n_ulpc_busmv_A = "n:ulpc:busmv:A", "%, Line-Line Voltage, Magnitude"
      n_ulpc_busmv_B = "n:ulpc:busmv:B", "%, Line-Line Voltage, Magnitude"
      n_ulpc_busmv_C = "n:ulpc:busmv:C", "%, Line-Line Voltage, Magnitude"
      n_ulpc_busmv_N = "n:ulpc:busmv:N", "%, Line-Line Voltage, Magnitude"
      n_ulpc_buslv_A = "n:ulpc:buslv:A", "%, Line-Line Voltage, Magnitude"
      n_ulpc_buslv_B = "n:ulpc:buslv:B", "%, Line-Line Voltage, Magnitude"
      n_ulpc_buslv_C = "n:ulpc:buslv:C", "%, Line-Line Voltage, Magnitude"
      n_ulpc_buslv_N = "n:ulpc:buslv:N", "%, Line-Line Voltage, Magnitude"
      n_ulpc_bushvn = "n:ulpc:bushvn", "%, Line-Line Voltage, Magnitude"
      n_ulpc_busmvn = "n:ulpc:busmvn", "%, Line-Line Voltage, Magnitude"
      n_ulpc_buslvn = "n:ulpc:buslvn", "%, Line-Line Voltage, Magnitude"
      n_Ul_bushv_A = "n:Ul:bushv:A", "kV, Line-Line Voltage, Magnitude"
      n_Ul_bushv_B = "n:Ul:bushv:B", "kV, Line-Line Voltage, Magnitude"
      n_Ul_bushv_C = "n:Ul:bushv:C", "kV, Line-Line Voltage, Magnitude"
      n_Ul_bushv_N = "n:Ul:bushv:N", "kV, Line-Line Voltage, Magnitude"
      n_Ul_busmv_A = "n:Ul:busmv:A", "kV, Line-Line Voltage, Magnitude"
      n_Ul_busmv_B = "n:Ul:busmv:B", "kV, Line-Line Voltage, Magnitude"
      n_Ul_busmv_C = "n:Ul:busmv:C", "kV, Line-Line Voltage, Magnitude"
      n_Ul_busmv_N = "n:Ul:busmv:N", "kV, Line-Line Voltage, Magnitude"
      n_Ul_buslv_A = "n:Ul:buslv:A", "kV, Line-Line Voltage, Magnitude"
      n_Ul_buslv_B = "n:Ul:buslv:B", "kV, Line-Line Voltage, Magnitude"
      n_Ul_buslv_C = "n:Ul:buslv:C", "kV, Line-Line Voltage, Magnitude"
      n_Ul_buslv_N = "n:Ul:buslv:N", "kV, Line-Line Voltage, Magnitude"
      n_Ul_bushvn = "n:Ul:bushvn", "kV, Line-Line Voltage, Magnitude"
      n_Ul_busmvn = "n:Ul:busmvn", "kV, Line-Line Voltage, Magnitude"
      n_Ul_buslvn = "n:Ul:buslvn", "kV, Line-Line Voltage, Magnitude"
      n_phiul_bushv_A = "n:phiul:bushv:A", "deg, Line-Line Voltage, Angle"
      n_phiul_bushv_B = "n:phiul:bushv:B", "deg, Line-Line Voltage, Angle"
      n_phiul_bushv_C = "n:phiul:bushv:C", "deg, Line-Line Voltage, Angle"
      n_phiul_bushv_N = "n:phiul:bushv:N", "deg, Line-Line Voltage, Angle"
      n_phiul_busmv_A = "n:phiul:busmv:A", "deg, Line-Line Voltage, Angle"
      n_phiul_busmv_B = "n:phiul:busmv:B", "deg, Line-Line Voltage, Angle"
      n_phiul_busmv_C = "n:phiul:busmv:C", "deg, Line-Line Voltage, Angle"
      n_phiul_busmv_N = "n:phiul:busmv:N", "deg, Line-Line Voltage, Angle"
      n_phiul_buslv_A = "n:phiul:buslv:A", "deg, Line-Line Voltage, Angle"
      n_phiul_buslv_B = "n:phiul:buslv:B", "deg, Line-Line Voltage, Angle"
      n_phiul_buslv_C = "n:phiul:buslv:C", "deg, Line-Line Voltage, Angle"
      n_phiul_buslv_N = "n:phiul:buslv:N", "deg, Line-Line Voltage, Angle"
      n_phiul_bushvn = "n:phiul:bushvn", "deg, Line-Line Voltage, Angle"
      n_phiul_busmvn = "n:phiul:busmvn", "deg, Line-Line Voltage, Angle"
      n_phiul_buslvn = "n:phiul:buslvn", "deg, Line-Line Voltage, Angle"
      n_u0_bushv = "n:u0:bushv", "p.u., Zero-Sequence Voltage, Magnitude"
      n_u0_busmv = "n:u0:busmv", "p.u., Zero-Sequence Voltage, Magnitude"
      n_u0_buslv = "n:u0:buslv", "p.u., Zero-Sequence Voltage, Magnitude"
      n_u0_bushvn = "n:u0:bushvn", "p.u., Zero-Sequence Voltage, Magnitude"
      n_u0_busmvn = "n:u0:busmvn", "p.u., Zero-Sequence Voltage, Magnitude"
      n_u0_buslvn = "n:u0:buslvn", "p.u., Zero-Sequence Voltage, Magnitude"
      n_U0_bushv = "n:U0:bushv", "kV, Zero-Sequence Voltage, Magnitude"
      n_U0_busmv = "n:U0:busmv", "kV, Zero-Sequence Voltage, Magnitude"
      n_U0_buslv = "n:U0:buslv", "kV, Zero-Sequence Voltage, Magnitude"
      n_U0_bushvn = "n:U0:bushvn", "kV, Zero-Sequence Voltage, Magnitude"
      n_U0_busmvn = "n:U0:busmvn", "kV, Zero-Sequence Voltage, Magnitude"
      n_U0_buslvn = "n:U0:buslvn", "kV, Zero-Sequence Voltage, Magnitude"
      n_U0x3_bushv = "n:U0x3:bushv", "kV, 3*U0"
      n_U0x3_busmv = "n:U0x3:busmv", "kV, 3*U0"
      n_U0x3_buslv = "n:U0x3:buslv", "kV, 3*U0"
      n_U0x3_bushvn = "n:U0x3:bushvn", "kV, 3*U0"
      n_U0x3_busmvn = "n:U0x3:busmvn", "kV, 3*U0"
      n_U0x3_buslvn = "n:U0x3:buslvn", "kV, 3*U0"
      n_phiu0_bushv = "n:phiu0:bushv", "deg, Zero-Sequence Voltage, Angle"
      n_phiu0_busmv = "n:phiu0:busmv", "deg, Zero-Sequence Voltage, Angle"
      n_phiu0_buslv = "n:phiu0:buslv", "deg, Zero-Sequence Voltage, Angle"
      n_phiu0_bushvn = "n:phiu0:bushvn", "deg, Zero-Sequence Voltage, Angle"
      n_phiu0_busmvn = "n:phiu0:busmvn", "deg, Zero-Sequence Voltage, Angle"
      n_phiu0_buslvn = "n:phiu0:buslvn", "deg, Zero-Sequence Voltage, Angle"
      n_u1_bushv = "n:u1:bushv", "p.u., Positive-Sequence Voltage, Magnitude"
      n_u1_busmv = "n:u1:busmv", "p.u., Positive-Sequence Voltage, Magnitude"
      n_u1_buslv = "n:u1:buslv", "p.u., Positive-Sequence Voltage, Magnitude"
      n_u1_bushvn = "n:u1:bushvn", "p.u., Positive-Sequence Voltage, Magnitude"
      n_u1_busmvn = "n:u1:busmvn", "p.u., Positive-Sequence Voltage, Magnitude"
      n_u1_buslvn = "n:u1:buslvn", "p.u., Positive-Sequence Voltage, Magnitude"
      n_u1pc_bushv = "n:u1pc:bushv", "%, Positive-Sequence Voltage, Magnitude"
      n_u1pc_busmv = "n:u1pc:busmv", "%, Positive-Sequence Voltage, Magnitude"
      n_u1pc_buslv = "n:u1pc:buslv", "%, Positive-Sequence Voltage, Magnitude"
      n_u1pc_bushvn = "n:u1pc:bushvn", "%, Positive-Sequence Voltage, Magnitude"
      n_u1pc_busmvn = "n:u1pc:busmvn", "%, Positive-Sequence Voltage, Magnitude"
      n_u1pc_buslvn = "n:u1pc:buslvn", "%, Positive-Sequence Voltage, Magnitude"
      n_u1r_bushv = "n:u1r:bushv", "p.u., Positive-Sequence Voltage, Real Part"
      n_u1r_busmv = "n:u1r:busmv", "p.u., Positive-Sequence Voltage, Real Part"
      n_u1r_buslv = "n:u1r:buslv", "p.u., Positive-Sequence Voltage, Real Part"
      n_u1r_bushvn = "n:u1r:bushvn", "p.u., Positive-Sequence Voltage, Real Part"
      n_u1r_busmvn = "n:u1r:busmvn", "p.u., Positive-Sequence Voltage, Real Part"
      n_u1r_buslvn = "n:u1r:buslvn", "p.u., Positive-Sequence Voltage, Real Part"
      n_u1i_bushv = "n:u1i:bushv", "p.u., Positive-Sequence Voltage, Imaginary Part"
      n_u1i_busmv = "n:u1i:busmv", "p.u., Positive-Sequence Voltage, Imaginary Part"
      n_u1i_buslv = "n:u1i:buslv", "p.u., Positive-Sequence Voltage, Imaginary Part"
      n_u1i_bushvn = "n:u1i:bushvn", "p.u., Positive-Sequence Voltage, Imaginary Part"
      n_u1i_busmvn = "n:u1i:busmvn", "p.u., Positive-Sequence Voltage, Imaginary Part"
      n_u1i_buslvn = "n:u1i:buslvn", "p.u., Positive-Sequence Voltage, Imaginary Part"
      n_U1_bushv = "n:U1:bushv", "kV, Line-Ground Positive-Sequence Voltage, Magnitude"
      n_U1_busmv = "n:U1:busmv", "kV, Line-Ground Positive-Sequence Voltage, Magnitude"
      n_U1_buslv = "n:U1:buslv", "kV, Line-Ground Positive-Sequence Voltage, Magnitude"
      n_U1_bushvn = "n:U1:bushvn", "kV, Line-Ground Positive-Sequence Voltage, Magnitude"
      n_U1_busmvn = "n:U1:busmvn", "kV, Line-Ground Positive-Sequence Voltage, Magnitude"
      n_U1_buslvn = "n:U1:buslvn", "kV, Line-Ground Positive-Sequence Voltage, Magnitude"
      n_phiu1_bushv = "n:phiu1:bushv", "deg, Positive-Sequence Voltage, Angle"
      n_phiu1_busmv = "n:phiu1:busmv", "deg, Positive-Sequence Voltage, Angle"
      n_phiu1_buslv = "n:phiu1:buslv", "deg, Positive-Sequence Voltage, Angle"
      n_phiu1_bushvn = "n:phiu1:bushvn", "deg, Positive-Sequence Voltage, Angle"
      n_phiu1_busmvn = "n:phiu1:busmvn", "deg, Positive-Sequence Voltage, Angle"
      n_phiu1_buslvn = "n:phiu1:buslvn", "deg, Positive-Sequence Voltage, Angle"
      n_u2_bushv = "n:u2:bushv", "p.u., Negative-Sequence Voltage, Magnitude"
      n_u2_busmv = "n:u2:busmv", "p.u., Negative-Sequence Voltage, Magnitude"
      n_u2_buslv = "n:u2:buslv", "p.u., Negative-Sequence Voltage, Magnitude"
      n_u2_bushvn = "n:u2:bushvn", "p.u., Negative-Sequence Voltage, Magnitude"
      n_u2_busmvn = "n:u2:busmvn", "p.u., Negative-Sequence Voltage, Magnitude"
      n_u2_buslvn = "n:u2:buslvn", "p.u., Negative-Sequence Voltage, Magnitude"
      n_U2_bushv = "n:U2:bushv", "kV, Line-Ground Negative-Sequence Voltage, Magnitude"
      n_U2_busmv = "n:U2:busmv", "kV, Line-Ground Negative-Sequence Voltage, Magnitude"
      n_U2_buslv = "n:U2:buslv", "kV, Line-Ground Negative-Sequence Voltage, Magnitude"
      n_U2_bushvn = "n:U2:bushvn", "kV, Line-Ground Negative-Sequence Voltage, Magnitude"
      n_U2_busmvn = "n:U2:busmvn", "kV, Line-Ground Negative-Sequence Voltage, Magnitude"
      n_U2_buslvn = "n:U2:buslvn", "kV, Line-Ground Negative-Sequence Voltage, Magnitude"
      n_phiu2_bushv = "n:phiu2:bushv", "deg, Negative-Sequence Voltage, Angle"
      n_phiu2_busmv = "n:phiu2:busmv", "deg, Negative-Sequence Voltage, Angle"
      n_phiu2_buslv = "n:phiu2:buslv", "deg, Negative-Sequence Voltage, Angle"
      n_phiu2_bushvn = "n:phiu2:bushvn", "deg, Negative-Sequence Voltage, Angle"
      n_phiu2_busmvn = "n:phiu2:busmvn", "deg, Negative-Sequence Voltage, Angle"
      n_phiu2_buslvn = "n:phiu2:buslvn", "deg, Negative-Sequence Voltage, Angle"
      n_U1l_bushv = "n:U1l:bushv", "kV, Line-Line Positive-Sequence Voltage, Magnitude"
      n_U1l_busmv = "n:U1l:busmv", "kV, Line-Line Positive-Sequence Voltage, Magnitude"
      n_U1l_buslv = "n:U1l:buslv", "kV, Line-Line Positive-Sequence Voltage, Magnitude"
      n_U1l_bushvn = "n:U1l:bushvn", "kV, Line-Line Positive-Sequence Voltage, Magnitude"
      n_U1l_busmvn = "n:U1l:busmvn", "kV, Line-Line Positive-Sequence Voltage, Magnitude"
      n_U1l_buslvn = "n:U1l:buslvn", "kV, Line-Line Positive-Sequence Voltage, Magnitude"
      n_U2l_bushv = "n:U2l:bushv", "kV, Line-Line Negative-Sequence Voltage, Magnitude"
      n_U2l_busmv = "n:U2l:busmv", "kV, Line-Line Negative-Sequence Voltage, Magnitude"
      n_U2l_buslv = "n:U2l:buslv", "kV, Line-Line Negative-Sequence Voltage, Magnitude"
      n_U2l_bushvn = "n:U2l:bushvn", "kV, Line-Line Negative-Sequence Voltage, Magnitude"
      n_U2l_busmvn = "n:U2l:busmvn", "kV, Line-Line Negative-Sequence Voltage, Magnitude"
      n_U2l_buslvn = "n:U2l:buslvn", "kV, Line-Line Negative-Sequence Voltage, Magnitude"
      n_uln_bushv_A = "n:uln:bushv:A", "p.u., Line-Neutral Voltage, Magnitude"
      n_uln_bushv_B = "n:uln:bushv:B", "p.u., Line-Neutral Voltage, Magnitude"
      n_uln_bushv_C = "n:uln:bushv:C", "p.u., Line-Neutral Voltage, Magnitude"
      n_uln_bushv_N = "n:uln:bushv:N", "p.u., Line-Neutral Voltage, Magnitude"
      n_uln_busmv_A = "n:uln:busmv:A", "p.u., Line-Neutral Voltage, Magnitude"
      n_uln_busmv_B = "n:uln:busmv:B", "p.u., Line-Neutral Voltage, Magnitude"
      n_uln_busmv_C = "n:uln:busmv:C", "p.u., Line-Neutral Voltage, Magnitude"
      n_uln_busmv_N = "n:uln:busmv:N", "p.u., Line-Neutral Voltage, Magnitude"
      n_uln_buslv_A = "n:uln:buslv:A", "p.u., Line-Neutral Voltage, Magnitude"
      n_uln_buslv_B = "n:uln:buslv:B", "p.u., Line-Neutral Voltage, Magnitude"
      n_uln_buslv_C = "n:uln:buslv:C", "p.u., Line-Neutral Voltage, Magnitude"
      n_uln_buslv_N = "n:uln:buslv:N", "p.u., Line-Neutral Voltage, Magnitude"
      n_uln_bushvn = "n:uln:bushvn", "p.u., Line-Neutral Voltage, Magnitude"
      n_uln_busmvn = "n:uln:busmvn", "p.u., Line-Neutral Voltage, Magnitude"
      n_uln_buslvn = "n:uln:buslvn", "p.u., Line-Neutral Voltage, Magnitude"
      n_Uln_bushv_A = "n:Uln:bushv:A", "kV, Line-Neutral Voltage, Magnitude"
      n_Uln_bushv_B = "n:Uln:bushv:B", "kV, Line-Neutral Voltage, Magnitude"
      n_Uln_bushv_C = "n:Uln:bushv:C", "kV, Line-Neutral Voltage, Magnitude"
      n_Uln_bushv_N = "n:Uln:bushv:N", "kV, Line-Neutral Voltage, Magnitude"
      n_Uln_busmv_A = "n:Uln:busmv:A", "kV, Line-Neutral Voltage, Magnitude"
      n_Uln_busmv_B = "n:Uln:busmv:B", "kV, Line-Neutral Voltage, Magnitude"
      n_Uln_busmv_C = "n:Uln:busmv:C", "kV, Line-Neutral Voltage, Magnitude"
      n_Uln_busmv_N = "n:Uln:busmv:N", "kV, Line-Neutral Voltage, Magnitude"
      n_Uln_buslv_A = "n:Uln:buslv:A", "kV, Line-Neutral Voltage, Magnitude"
      n_Uln_buslv_B = "n:Uln:buslv:B", "kV, Line-Neutral Voltage, Magnitude"
      n_Uln_buslv_C = "n:Uln:buslv:C", "kV, Line-Neutral Voltage, Magnitude"
      n_Uln_buslv_N = "n:Uln:buslv:N", "kV, Line-Neutral Voltage, Magnitude"
      n_Uln_bushvn = "n:Uln:bushvn", "kV, Line-Neutral Voltage, Magnitude"
      n_Uln_busmvn = "n:Uln:busmvn", "kV, Line-Neutral Voltage, Magnitude"
      n_Uln_buslvn = "n:Uln:buslvn", "kV, Line-Neutral Voltage, Magnitude"
      n_phiuln_bushv_A = "n:phiuln:bushv:A", "deg, Line-Neutral Voltage, Angle"
      n_phiuln_bushv_B = "n:phiuln:bushv:B", "deg, Line-Neutral Voltage, Angle"
      n_phiuln_bushv_C = "n:phiuln:bushv:C", "deg, Line-Neutral Voltage, Angle"
      n_phiuln_bushv_N = "n:phiuln:bushv:N", "deg, Line-Neutral Voltage, Angle"
      n_phiuln_busmv_A = "n:phiuln:busmv:A", "deg, Line-Neutral Voltage, Angle"
      n_phiuln_busmv_B = "n:phiuln:busmv:B", "deg, Line-Neutral Voltage, Angle"
      n_phiuln_busmv_C = "n:phiuln:busmv:C", "deg, Line-Neutral Voltage, Angle"
      n_phiuln_busmv_N = "n:phiuln:busmv:N", "deg, Line-Neutral Voltage, Angle"
      n_phiuln_buslv_A = "n:phiuln:buslv:A", "deg, Line-Neutral Voltage, Angle"
      n_phiuln_buslv_B = "n:phiuln:buslv:B", "deg, Line-Neutral Voltage, Angle"
      n_phiuln_buslv_C = "n:phiuln:buslv:C", "deg, Line-Neutral Voltage, Angle"
      n_phiuln_buslv_N = "n:phiuln:buslv:N", "deg, Line-Neutral Voltage, Angle"
      n_phiuln_bushvn = "n:phiuln:bushvn", "deg, Line-Neutral Voltage, Angle"
      n_phiuln_busmvn = "n:phiuln:busmvn", "deg, Line-Neutral Voltage, Angle"
      n_phiuln_buslvn = "n:phiuln:buslvn", "deg, Line-Neutral Voltage, Angle"
      n_un_bushv = "n:un:bushv", "p.u., Neutral-Ground Voltage, Magnitude"
      n_un_busmv = "n:un:busmv", "p.u., Neutral-Ground Voltage, Magnitude"
      n_un_buslv = "n:un:buslv", "p.u., Neutral-Ground Voltage, Magnitude"
      n_un_bushvn = "n:un:bushvn", "p.u., Neutral-Ground Voltage, Magnitude"
      n_un_busmvn = "n:un:busmvn", "p.u., Neutral-Ground Voltage, Magnitude"
      n_un_buslvn = "n:un:buslvn", "p.u., Neutral-Ground Voltage, Magnitude"
      n_Un_bushv = "n:Un:bushv", "kV, Neutral-Ground Voltage, Magnitude"
      n_Un_busmv = "n:Un:busmv", "kV, Neutral-Ground Voltage, Magnitude"
      n_Un_buslv = "n:Un:buslv", "kV, Neutral-Ground Voltage, Magnitude"
      n_Un_bushvn = "n:Un:bushvn", "kV, Neutral-Ground Voltage, Magnitude"
      n_Un_busmvn = "n:Un:busmvn", "kV, Neutral-Ground Voltage, Magnitude"
      n_Un_buslvn = "n:Un:buslvn", "kV, Neutral-Ground Voltage, Magnitude"
      n_phiun_bushv = "n:phiun:bushv", "deg, Neutral-Ground Voltage, Angle"
      n_phiun_busmv = "n:phiun:busmv", "deg, Neutral-Ground Voltage, Angle"
      n_phiun_buslv = "n:phiun:buslv", "deg, Neutral-Ground Voltage, Angle"
      n_phiun_bushvn = "n:phiun:bushvn", "deg, Neutral-Ground Voltage, Angle"
      n_phiun_busmvn = "n:phiun:busmvn", "deg, Neutral-Ground Voltage, Angle"
      n_phiun_buslvn = "n:phiun:buslvn", "deg, Neutral-Ground Voltage, Angle"
      n_upht_bushv_A = "n:upht:bushv:A", "p.u., Phase Technology dependent Voltage, Magnitude"
      n_upht_bushv_B = "n:upht:bushv:B", "p.u., Phase Technology dependent Voltage, Magnitude"
      n_upht_bushv_C = "n:upht:bushv:C", "p.u., Phase Technology dependent Voltage, Magnitude"
      n_upht_bushv_N = "n:upht:bushv:N", "p.u., Phase Technology dependent Voltage, Magnitude"
      n_upht_busmv_A = "n:upht:busmv:A", "p.u., Phase Technology dependent Voltage, Magnitude"
      n_upht_busmv_B = "n:upht:busmv:B", "p.u., Phase Technology dependent Voltage, Magnitude"
      n_upht_busmv_C = "n:upht:busmv:C", "p.u., Phase Technology dependent Voltage, Magnitude"
      n_upht_busmv_N = "n:upht:busmv:N", "p.u., Phase Technology dependent Voltage, Magnitude"
      n_upht_buslv_A = "n:upht:buslv:A", "p.u., Phase Technology dependent Voltage, Magnitude"
      n_upht_buslv_B = "n:upht:buslv:B", "p.u., Phase Technology dependent Voltage, Magnitude"
      n_upht_buslv_C = "n:upht:buslv:C", "p.u., Phase Technology dependent Voltage, Magnitude"
      n_upht_buslv_N = "n:upht:buslv:N", "p.u., Phase Technology dependent Voltage, Magnitude"
      n_upht_bushvn = "n:upht:bushvn", "p.u., Phase Technology dependent Voltage, Magnitude"
      n_upht_busmvn = "n:upht:busmvn", "p.u., Phase Technology dependent Voltage, Magnitude"
      n_upht_buslvn = "n:upht:buslvn", "p.u., Phase Technology dependent Voltage, Magnitude"
      n_Upht_bushv_A = "n:Upht:bushv:A", "kV, Phase Technology dependent Voltage, Magnitude"
      n_Upht_bushv_B = "n:Upht:bushv:B", "kV, Phase Technology dependent Voltage, Magnitude"
      n_Upht_bushv_C = "n:Upht:bushv:C", "kV, Phase Technology dependent Voltage, Magnitude"
      n_Upht_bushv_N = "n:Upht:bushv:N", "kV, Phase Technology dependent Voltage, Magnitude"
      n_Upht_busmv_A = "n:Upht:busmv:A", "kV, Phase Technology dependent Voltage, Magnitude"
      n_Upht_busmv_B = "n:Upht:busmv:B", "kV, Phase Technology dependent Voltage, Magnitude"
      n_Upht_busmv_C = "n:Upht:busmv:C", "kV, Phase Technology dependent Voltage, Magnitude"
      n_Upht_busmv_N = "n:Upht:busmv:N", "kV, Phase Technology dependent Voltage, Magnitude"
      n_Upht_buslv_A = "n:Upht:buslv:A", "kV, Phase Technology dependent Voltage, Magnitude"
      n_Upht_buslv_B = "n:Upht:buslv:B", "kV, Phase Technology dependent Voltage, Magnitude"
      n_Upht_buslv_C = "n:Upht:buslv:C", "kV, Phase Technology dependent Voltage, Magnitude"
      n_Upht_buslv_N = "n:Upht:buslv:N", "kV, Phase Technology dependent Voltage, Magnitude"
      n_Upht_bushvn = "n:Upht:bushvn", "kV, Phase Technology dependent Voltage, Magnitude"
      n_Upht_busmvn = "n:Upht:busmvn", "kV, Phase Technology dependent Voltage, Magnitude"
      n_Upht_buslvn = "n:Upht:buslvn", "kV, Phase Technology dependent Voltage, Magnitude"
      n_um_bushv = "n:um:bushv", "p.u., Average-Voltage, Magnitude"
      n_um_busmv = "n:um:busmv", "p.u., Average-Voltage, Magnitude"
      n_um_buslv = "n:um:buslv", "p.u., Average-Voltage, Magnitude"
      n_um_bushvn = "n:um:bushvn", "p.u., Average-Voltage, Magnitude"
      n_um_busmvn = "n:um:busmvn", "p.u., Average-Voltage, Magnitude"
      n_um_buslvn = "n:um:buslvn", "p.u., Average-Voltage, Magnitude"
      n_Um_bushv = "n:Um:bushv", "kV, Average-Voltage, Magnitude"
      n_Um_busmv = "n:Um:busmv", "kV, Average-Voltage, Magnitude"
      n_Um_buslv = "n:Um:buslv", "kV, Average-Voltage, Magnitude"
      n_Um_bushvn = "n:Um:bushvn", "kV, Average-Voltage, Magnitude"
      n_Um_busmvn = "n:Um:busmvn", "kV, Average-Voltage, Magnitude"
      n_Um_buslvn = "n:Um:buslvn", "kV, Average-Voltage, Magnitude"
      n_uphtmin_bushv = "n:uphtmin:bushv", "p.u., Minimum of Phase Technology dependent Voltage, Magnitude"
      n_uphtmin_busmv = "n:uphtmin:busmv", "p.u., Minimum of Phase Technology dependent Voltage, Magnitude"
      n_uphtmin_buslv = "n:uphtmin:buslv", "p.u., Minimum of Phase Technology dependent Voltage, Magnitude"
      n_uphtmin_bushvn = "n:uphtmin:bushvn", "p.u., Minimum of Phase Technology dependent Voltage, Magnitude"
      n_uphtmin_busmvn = "n:uphtmin:busmvn", "p.u., Minimum of Phase Technology dependent Voltage, Magnitude"
      n_uphtmin_buslvn = "n:uphtmin:buslvn", "p.u., Minimum of Phase Technology dependent Voltage, Magnitude"
      n_uphtmax_bushv = "n:uphtmax:bushv", "p.u., Maximum of Phase Technology dependent Voltage, Magnitude"
      n_uphtmax_busmv = "n:uphtmax:busmv", "p.u., Maximum of Phase Technology dependent Voltage, Magnitude"
      n_uphtmax_buslv = "n:uphtmax:buslv", "p.u., Maximum of Phase Technology dependent Voltage, Magnitude"
      n_uphtmax_bushvn = "n:uphtmax:bushvn", "p.u., Maximum of Phase Technology dependent Voltage, Magnitude"
      n_uphtmax_busmvn = "n:uphtmax:busmvn", "p.u., Maximum of Phase Technology dependent Voltage, Magnitude"
      n_uphtmax_buslvn = "n:uphtmax:buslvn", "p.u., Maximum of Phase Technology dependent Voltage, Magnitude"
      n_Pgen_bushv = "n:Pgen:bushv", "MW, Generation, Active Power"
      n_Pgen_busmv = "n:Pgen:busmv", "MW, Generation, Active Power"
      n_Pgen_buslv = "n:Pgen:buslv", "MW, Generation, Active Power"
      n_Pgen_bushvn = "n:Pgen:bushvn", "MW, Generation, Active Power"
      n_Pgen_busmvn = "n:Pgen:busmvn", "MW, Generation, Active Power"
      n_Pgen_buslvn = "n:Pgen:buslvn", "MW, Generation, Active Power"
      n_Qgen_bushv = "n:Qgen:bushv", "Mvar, Generation, Reactive Power"
      n_Qgen_busmv = "n:Qgen:busmv", "Mvar, Generation, Reactive Power"
      n_Qgen_buslv = "n:Qgen:buslv", "Mvar, Generation, Reactive Power"
      n_Qgen_bushvn = "n:Qgen:bushvn", "Mvar, Generation, Reactive Power"
      n_Qgen_busmvn = "n:Qgen:busmvn", "Mvar, Generation, Reactive Power"
      n_Qgen_buslvn = "n:Qgen:buslvn", "Mvar, Generation, Reactive Power"
      n_Pmot_bushv = "n:Pmot:bushv", "MW, Motor Load, Active Power"
      n_Pmot_busmv = "n:Pmot:busmv", "MW, Motor Load, Active Power"
      n_Pmot_buslv = "n:Pmot:buslv", "MW, Motor Load, Active Power"
      n_Pmot_bushvn = "n:Pmot:bushvn", "MW, Motor Load, Active Power"
      n_Pmot_busmvn = "n:Pmot:busmvn", "MW, Motor Load, Active Power"
      n_Pmot_buslvn = "n:Pmot:buslvn", "MW, Motor Load, Active Power"
      n_Qmot_bushv = "n:Qmot:bushv", "Mvar, Motor Load, Reactive Power"
      n_Qmot_busmv = "n:Qmot:busmv", "Mvar, Motor Load, Reactive Power"
      n_Qmot_buslv = "n:Qmot:buslv", "Mvar, Motor Load, Reactive Power"
      n_Qmot_bushvn = "n:Qmot:bushvn", "Mvar, Motor Load, Reactive Power"
      n_Qmot_busmvn = "n:Qmot:busmvn", "Mvar, Motor Load, Reactive Power"
      n_Qmot_buslvn = "n:Qmot:buslvn", "Mvar, Motor Load, Reactive Power"
      n_Pload_bushv = "n:Pload:bushv", "MW, General Load, Active Power"
      n_Pload_busmv = "n:Pload:busmv", "MW, General Load, Active Power"
      n_Pload_buslv = "n:Pload:buslv", "MW, General Load, Active Power"
      n_Pload_bushvn = "n:Pload:bushvn", "MW, General Load, Active Power"
      n_Pload_busmvn = "n:Pload:busmvn", "MW, General Load, Active Power"
      n_Pload_buslvn = "n:Pload:buslvn", "MW, General Load, Active Power"
      n_Qload_bushv = "n:Qload:bushv", "Mvar, General Load, Reactive Power"
      n_Qload_busmv = "n:Qload:busmv", "Mvar, General Load, Reactive Power"
      n_Qload_buslv = "n:Qload:buslv", "Mvar, General Load, Reactive Power"
      n_Qload_bushvn = "n:Qload:bushvn", "Mvar, General Load, Reactive Power"
      n_Qload_busmvn = "n:Qload:busmvn", "Mvar, General Load, Reactive Power"
      n_Qload_buslvn = "n:Qload:buslvn", "Mvar, General Load, Reactive Power"
      n_Pcomp_bushv = "n:Pcomp:bushv", "MW, Compensation (Losses)"
      n_Pcomp_busmv = "n:Pcomp:busmv", "MW, Compensation (Losses)"
      n_Pcomp_buslv = "n:Pcomp:buslv", "MW, Compensation (Losses)"
      n_Pcomp_bushvn = "n:Pcomp:bushvn", "MW, Compensation (Losses)"
      n_Pcomp_busmvn = "n:Pcomp:busmvn", "MW, Compensation (Losses)"
      n_Pcomp_buslvn = "n:Pcomp:buslvn", "MW, Compensation (Losses)"
      n_Qcomp_bushv = "n:Qcomp:bushv", "Mvar, Compensation"
      n_Qcomp_busmv = "n:Qcomp:busmv", "Mvar, Compensation"
      n_Qcomp_buslv = "n:Qcomp:buslv", "Mvar, Compensation"
      n_Qcomp_bushvn = "n:Qcomp:bushvn", "Mvar, Compensation"
      n_Qcomp_busmvn = "n:Qcomp:busmvn", "Mvar, Compensation"
      n_Qcomp_buslvn = "n:Qcomp:buslvn", "Mvar, Compensation"
      n_Pnet_bushv = "n:Pnet:bushv", "MW, External Networks, Active Power"
      n_Pnet_busmv = "n:Pnet:busmv", "MW, External Networks, Active Power"
      n_Pnet_buslv = "n:Pnet:buslv", "MW, External Networks, Active Power"
      n_Pnet_bushvn = "n:Pnet:bushvn", "MW, External Networks, Active Power"
      n_Pnet_busmvn = "n:Pnet:busmvn", "MW, External Networks, Active Power"
      n_Pnet_buslvn = "n:Pnet:buslvn", "MW, External Networks, Active Power"
      n_Qnet_bushv = "n:Qnet:bushv", "Mvar, External Networks, Reactive Power"
      n_Qnet_busmv = "n:Qnet:busmv", "Mvar, External Networks, Reactive Power"
      n_Qnet_buslv = "n:Qnet:buslv", "Mvar, External Networks, Reactive Power"
      n_Qnet_bushvn = "n:Qnet:bushvn", "Mvar, External Networks, Reactive Power"
      n_Qnet_busmvn = "n:Qnet:busmvn", "Mvar, External Networks, Reactive Power"
      n_Qnet_buslvn = "n:Qnet:buslvn", "Mvar, External Networks, Reactive Power"
      n_Pflow_bushv = "n:Pflow:bushv", "MW, Power Flow, Active Power"
      n_Pflow_busmv = "n:Pflow:busmv", "MW, Power Flow, Active Power"
      n_Pflow_buslv = "n:Pflow:buslv", "MW, Power Flow, Active Power"
      n_Pflow_bushvn = "n:Pflow:bushvn", "MW, Power Flow, Active Power"
      n_Pflow_busmvn = "n:Pflow:busmvn", "MW, Power Flow, Active Power"
      n_Pflow_buslvn = "n:Pflow:buslvn", "MW, Power Flow, Active Power"
      n_Qflow_bushv = "n:Qflow:bushv", "Mvar, Power Flow, Reactive Power"
      n_Qflow_busmv = "n:Qflow:busmv", "Mvar, Power Flow, Reactive Power"
      n_Qflow_buslv = "n:Qflow:buslv", "Mvar, Power Flow, Reactive Power"
      n_Qflow_bushvn = "n:Qflow:bushvn", "Mvar, Power Flow, Reactive Power"
      n_Qflow_busmvn = "n:Qflow:busmvn", "Mvar, Power Flow, Reactive Power"
      n_Qflow_buslvn = "n:Qflow:buslvn", "Mvar, Power Flow, Reactive Power"
      n_Pout_bushv = "n:Pout:bushv", "MW, Outgoing Flow, Active Power"
      n_Pout_busmv = "n:Pout:busmv", "MW, Outgoing Flow, Active Power"
      n_Pout_buslv = "n:Pout:buslv", "MW, Outgoing Flow, Active Power"
      n_Pout_bushvn = "n:Pout:bushvn", "MW, Outgoing Flow, Active Power"
      n_Pout_busmvn = "n:Pout:busmvn", "MW, Outgoing Flow, Active Power"
      n_Pout_buslvn = "n:Pout:buslvn", "MW, Outgoing Flow, Active Power"
      n_Qout_bushv = "n:Qout:bushv", "Mvar, Outgoing Power, Reactive Power"
      n_Qout_busmv = "n:Qout:busmv", "Mvar, Outgoing Power, Reactive Power"
      n_Qout_buslv = "n:Qout:buslv", "Mvar, Outgoing Power, Reactive Power"
      n_Qout_bushvn = "n:Qout:bushvn", "Mvar, Outgoing Power, Reactive Power"
      n_Qout_busmvn = "n:Qout:busmvn", "Mvar, Outgoing Power, Reactive Power"
      n_Qout_buslvn = "n:Qout:buslvn", "Mvar, Outgoing Power, Reactive Power"
      n_Sout_bushv = "n:Sout:bushv", "MVA, Outgoing Power, Apparent Power"
      n_Sout_busmv = "n:Sout:busmv", "MVA, Outgoing Power, Apparent Power"
      n_Sout_buslv = "n:Sout:buslv", "MVA, Outgoing Power, Apparent Power"
      n_Sout_bushvn = "n:Sout:bushvn", "MVA, Outgoing Power, Apparent Power"
      n_Sout_busmvn = "n:Sout:busmvn", "MVA, Outgoing Power, Apparent Power"
      n_Sout_buslvn = "n:Sout:buslvn", "MVA, Outgoing Power, Apparent Power"
      n_cosphiout_bushv = "n:cosphiout:bushv", "Outgoing Power, Power Factor"
      n_cosphiout_busmv = "n:cosphiout:busmv", "Outgoing Power, Power Factor"
      n_cosphiout_buslv = "n:cosphiout:buslv", "Outgoing Power, Power Factor"
      n_cosphiout_bushvn = "n:cosphiout:bushvn", "Outgoing Power, Power Factor"
      n_cosphiout_busmvn = "n:cosphiout:busmvn", "Outgoing Power, Power Factor"
      n_cosphiout_buslvn = "n:cosphiout:buslvn", "Outgoing Power, Power Factor"
      n_Pbalance_bushv = "n:Pbalance:bushv", "MW, Active Power Balance (=0)"
      n_Pbalance_busmv = "n:Pbalance:busmv", "MW, Active Power Balance (=0)"
      n_Pbalance_buslv = "n:Pbalance:buslv", "MW, Active Power Balance (=0)"
      n_Pbalance_bushvn = "n:Pbalance:bushvn", "MW, Active Power Balance (=0)"
      n_Pbalance_busmvn = "n:Pbalance:busmvn", "MW, Active Power Balance (=0)"
      n_Pbalance_buslvn = "n:Pbalance:buslvn", "MW, Active Power Balance (=0)"
      n_Qbalance_bushv = "n:Qbalance:bushv", "Mvar, Reactive Power Balance (=0)"
      n_Qbalance_busmv = "n:Qbalance:busmv", "Mvar, Reactive Power Balance (=0)"
      n_Qbalance_buslv = "n:Qbalance:buslv", "Mvar, Reactive Power Balance (=0)"
      n_Qbalance_bushvn = "n:Qbalance:bushvn", "Mvar, Reactive Power Balance (=0)"
      n_Qbalance_busmvn = "n:Qbalance:busmvn", "Mvar, Reactive Power Balance (=0)"
      n_Qbalance_buslvn = "n:Qbalance:buslvn", "Mvar, Reactive Power Balance (=0)"
      n_ubfac_bushv = "n:ubfac:bushv", "%, Unbalance Factor"
      n_ubfac_busmv = "n:ubfac:busmv", "%, Unbalance Factor"
      n_ubfac_buslv = "n:ubfac:buslv", "%, Unbalance Factor"
      n_ubfac_bushvn = "n:ubfac:bushvn", "%, Unbalance Factor"
      n_ubfac_busmvn = "n:ubfac:busmvn", "%, Unbalance Factor"
      n_ubfac_buslvn = "n:ubfac:buslvn", "%, Unbalance Factor"
      n_LossPdown_bushv = "n:LossPdown:bushv", "MW, Losses, downstream"
      n_LossPdown_busmv = "n:LossPdown:busmv", "MW, Losses, downstream"
      n_LossPdown_buslv = "n:LossPdown:buslv", "MW, Losses, downstream"
      n_LossPdown_bushvn = "n:LossPdown:bushvn", "MW, Losses, downstream"
      n_LossPdown_busmvn = "n:LossPdown:busmvn", "MW, Losses, downstream"
      n_LossPdown_buslvn = "n:LossPdown:buslvn", "MW, Losses, downstream"
      n_LossQdown_bushv = "n:LossQdown:bushv", "Mvar, Losses, downstream (Reactive Power)"
      n_LossQdown_busmv = "n:LossQdown:busmv", "Mvar, Losses, downstream (Reactive Power)"
      n_LossQdown_buslv = "n:LossQdown:buslv", "Mvar, Losses, downstream (Reactive Power)"
      n_LossQdown_bushvn = "n:LossQdown:bushvn", "Mvar, Losses, downstream (Reactive Power)"
      n_LossQdown_busmvn = "n:LossQdown:busmvn", "Mvar, Losses, downstream (Reactive Power)"
      n_LossQdown_buslvn = "n:LossQdown:buslvn", "Mvar, Losses, downstream (Reactive Power)"
      n_LossPdownload_bushv = "n:LossPdownload:bushv", "MW, Load losses, downstream"
      n_LossPdownload_busmv = "n:LossPdownload:busmv", "MW, Load losses, downstream"
      n_LossPdownload_buslv = "n:LossPdownload:buslv", "MW, Load losses, downstream"
      n_LossPdownload_bushvn = "n:LossPdownload:bushvn", "MW, Load losses, downstream"
      n_LossPdownload_busmvn = "n:LossPdownload:busmvn", "MW, Load losses, downstream"
      n_LossPdownload_buslvn = "n:LossPdownload:buslvn", "MW, Load losses, downstream"
      n_LossQdownload_bushv = "n:LossQdownload:bushv", "Mvar, Load losses, downstream"
      n_LossQdownload_busmv = "n:LossQdownload:busmv", "Mvar, Load losses, downstream"
      n_LossQdownload_buslv = "n:LossQdownload:buslv", "Mvar, Load losses, downstream"
      n_LossQdownload_bushvn = "n:LossQdownload:bushvn", "Mvar, Load losses, downstream"
      n_LossQdownload_busmvn = "n:LossQdownload:busmvn", "Mvar, Load losses, downstream"
      n_LossQdownload_buslvn = "n:LossQdownload:buslvn", "Mvar, Load losses, downstream"
      n_LossPdownnoload_bushv = "n:LossPdownnoload:bushv", "MW, No load losses, downstream"
      n_LossPdownnoload_busmv = "n:LossPdownnoload:busmv", "MW, No load losses, downstream"
      n_LossPdownnoload_buslv = "n:LossPdownnoload:buslv", "MW, No load losses, downstream"
      n_LossPdownnoload_bushvn = "n:LossPdownnoload:bushvn", "MW, No load losses, downstream"
      n_LossPdownnoload_busmvn = "n:LossPdownnoload:busmvn", "MW, No load losses, downstream"
      n_LossPdownnoload_buslvn = "n:LossPdownnoload:buslvn", "MW, No load losses, downstream"
      n_LossQdownnoload_bushv = "n:LossQdownnoload:bushv", "Mvar, No load losses, downstream"
      n_LossQdownnoload_busmv = "n:LossQdownnoload:busmv", "Mvar, No load losses, downstream"
      n_LossQdownnoload_buslv = "n:LossQdownnoload:buslv", "Mvar, No load losses, downstream"
      n_LossQdownnoload_bushvn = "n:LossQdownnoload:bushvn", "Mvar, No load losses, downstream"
      n_LossQdownnoload_busmvn = "n:LossQdownnoload:busmvn", "Mvar, No load losses, downstream"
      n_LossQdownnoload_buslvn = "n:LossQdownnoload:buslvn", "Mvar, No load losses, downstream"
      s_nt_in_h = "s:nt_in_h", "IN, Tap Position (HV), Input, continuous, double"
      s_nt_in_m = "s:nt_in_m", "IN, Tap Position (MV), Input, continuous, double"
      s_nt_in_l = "s:nt_in_l", "IN, Tap Position (LV), Input, continuous, double"
      s_du_ldc = "s:du_ldc", "IN, LDC Input, continuous, double"
      s_I0rDelta_h = "s:I0rDelta_h", "kA, OUT, Circulating current in HV-Delta-Winding, Real Part, continuous, double"
      s_I0rDelta_m = "s:I0rDelta_m", "kA, OUT, Circulating current in HV-Delta-Winding, Real Part, continuous, double"
      s_I0rDelta_l = "s:I0rDelta_l", "kA, OUT, Circulating current in LV-Delta-Winding, Real Part, continuous, double"
      s_I0iDelta_h = "s:I0iDelta_h", "kA, OUT, Circulating current in HV-Delta-Winding, Imaginary Part continuous, double"
      s_I0iDelta_m = "s:I0iDelta_m", "kA, OUT, Circulating current in MV-Delta-Winding, Imaginary Part continuous, double"
      s_I0iDelta_l = "s:I0iDelta_l", "kA, OUT, Circulating current in LV-Delta-Winding, Imaginary Part continuous, double"
      c_loading_h = "c:loading_h", "%, Loading (HV)"
      c_loading_m = "c:loading_m", "%, Loading (MV)"
      c_loading_l = "c:loading_l", "%, Loading (LV)"
      c_loading = "c:loading", "%, Maximum Loading"
      c_maxload = "c:maxload", "%, Max. Loading"
      c_n3tap_h = "c:n3tap_h", "Current Tap Position (HV)"
      c_n3tap_m = "c:n3tap_m", "Current Tap Position (MV)"
      c_n3tap_l = "c:n3tap_l", "Current Tap Position (LV)"
      c_nntap_int_h = "c:nntap_int_h", "Tap (HV), relative Position"
      c_nntap_int_m = "c:nntap_int_m", "Tap (MV), relative Position"
      c_nntap_int_l = "c:nntap_int_l", "Tap (LV), relative Position"
      c_zs_h_r = "c:zs_h:r", "p.u., Short Circuit Impedance (HV), Real Part"
      c_zs_h_i = "c:zs_h:i", "p.u., Short Circuit Impedance (HV), Imaginary Part"
      c_zs_h_phi = "c:zs_h:phi", "deg, Short Circuit Impedance (HV), Angle"
      c_zs_h_mag = "c:zs_h:mag", "p.u., Short Circuit Impedance (HV), Magnitude"
      c_zs_h_phirad = "c:zs_h:phirad", "rad, Short Circuit Impedance (HV), Angle"
      c_zs_m_r = "c:zs_m:r", "p.u., Short Circuit Impedance (MV), Real Part"
      c_zs_m_i = "c:zs_m:i", "p.u., Short Circuit Impedance (MV), Imaginary Part"
      c_zs_m_phi = "c:zs_m:phi", "deg, Short Circuit Impedance (MV), Angle"
      c_zs_m_mag = "c:zs_m:mag", "p.u., Short Circuit Impedance (MV), Magnitude"
      c_zs_m_phirad = "c:zs_m:phirad", "rad, Short Circuit Impedance (MV), Angle"
      c_zs_l_r = "c:zs_l:r", "p.u., Short Circuit Impedance (LV), Real Part"
      c_zs_l_i = "c:zs_l:i", "p.u., Short Circuit Impedance (LV), Imaginary Part"
      c_zs_l_phi = "c:zs_l:phi", "deg, Short Circuit Impedance (LV), Angle"
      c_zs_l_mag = "c:zs_l:mag", "p.u., Short Circuit Impedance (LV), Magnitude"
      c_zs_l_phirad = "c:zs_l:phirad", "rad, Short Circuit Impedance (LV), Angle"
      c_ym_r = "c:ym:r", "p.u., Magnetising Admittance, Real Part"
      c_ym_i = "c:ym:i", "p.u., Magnetising Admittance, Imaginary Part"
      c_ym_phi = "c:ym:phi", "deg, Magnetising Admittance, Angle"
      c_ym_mag = "c:ym:mag", "p.u., Magnetising Admittance, Magnitude"
      c_ym_phirad = "c:ym:phirad", "rad, Magnetising Admittance, Angle"
      c_t_h_r = "c:t_h:r", "Complex Winding Ratio (HV/S), Real Part"
      c_t_h_i = "c:t_h:i", "Complex Winding Ratio (HV/S), Imaginary Part"
      c_t_h_phi = "c:t_h:phi", "Complex Winding Ratio (HV/S), Angle"
      c_t_h_mag = "c:t_h:mag", "Complex Winding Ratio (HV/S), Magnitude"
      c_t_h_phirad = "c:t_h:phirad", "Complex Winding Ratio (HV/S), Angle"
      c_t_m_r = "c:t_m:r", "Complex Winding Ratio (MV/S), Real Part"
      c_t_m_i = "c:t_m:i", "Complex Winding Ratio (MV/S), Imaginary Part"
      c_t_m_phi = "c:t_m:phi", "Complex Winding Ratio (MV/S), Angle"
      c_t_m_mag = "c:t_m:mag", "Complex Winding Ratio (MV/S), Magnitude"
      c_t_m_phirad = "c:t_m:phirad", "Complex Winding Ratio (MV/S), Angle"
      c_t_l_r = "c:t_l:r", "Complex Winding Ratio (LV/S), Real Part"
      c_t_l_i = "c:t_l:i", "Complex Winding Ratio (LV/S), Imaginary Part"
      c_t_l_phi = "c:t_l:phi", "Complex Winding Ratio (LV/S), Angle"
      c_t_l_mag = "c:t_l:mag", "Complex Winding Ratio (LV/S), Magnitude"
      c_t_l_phirad = "c:t_l:phirad", "Complex Winding Ratio (LV/S), Angle"
      c_i_uplim = "c:i_uplim", "Tap at Maximum"
      c_i_lowlim = "c:i_lowlim", "Tap at Minimum"
      c_nCtrlIter = "c:nCtrlIter", "No. of CTRL-iterations"
      c_itrcn = "c:itrcn", "Flag f. auto. Tap"
      c_i3loc = "c:i3loc", "Position of Magn.ind."
      c_i30lc = "c:i30lc", "Position of 0-Sequ. Magn.ind."
      c_zs0_h_r = "c:zs0_h:r", "p.u., 0-Sequ. Short-Circuit Imp (HV), Real Part"
      c_zs0_h_i = "c:zs0_h:i", "p.u., 0-Sequ. Short-Circuit Imp (HV), Imaginary Part"
      c_zs0_h_phi = "c:zs0_h:phi", "deg, 0-Sequ. Short-Circuit Imp (HV), Angle"
      c_zs0_h_mag = "c:zs0_h:mag", "p.u., 0-Sequ. Short-Circuit Imp (HV), Magnitude"
      c_zs0_h_phirad = "c:zs0_h:phirad", "rad, 0-Sequ. Short-Circuit Imp (HV), Angle"
      c_zs0_m_r = "c:zs0_m:r", "p.u., 0-Sequ. Short-Circuit Imp (MV), Real Part"
      c_zs0_m_i = "c:zs0_m:i", "p.u., 0-Sequ. Short-Circuit Imp (MV), Imaginary Part"
      c_zs0_m_phi = "c:zs0_m:phi", "deg, 0-Sequ. Short-Circuit Imp (MV), Angle"
      c_zs0_m_mag = "c:zs0_m:mag", "p.u., 0-Sequ. Short-Circuit Imp (MV), Magnitude"
      c_zs0_m_phirad = "c:zs0_m:phirad", "rad, 0-Sequ. Short-Circuit Imp (MV), Angle"
      c_zs0_l_r = "c:zs0_l:r", "p.u., 0-Sequ. Short-Circuit Imp (LV), Real Part"
      c_zs0_l_i = "c:zs0_l:i", "p.u., 0-Sequ. Short-Circuit Imp (LV), Imaginary Part"
      c_zs0_l_phi = "c:zs0_l:phi", "deg, 0-Sequ. Short-Circuit Imp (LV), Angle"
      c_zs0_l_mag = "c:zs0_l:mag", "p.u., 0-Sequ. Short-Circuit Imp (LV), Magnitude"
      c_zs0_l_phirad = "c:zs0_l:phirad", "rad, 0-Sequ. Short-Circuit Imp (LV), Angle"
      c_ze0_h_r = "c:ze0_h:r", "p.u., Earth Impedance (HV), Real Part"
      c_ze0_h_i = "c:ze0_h:i", "p.u., Earth Impedance (HV), Imaginary Part"
      c_ze0_h_phi = "c:ze0_h:phi", "deg, Earth Impedance (HV), Angle"
      c_ze0_h_mag = "c:ze0_h:mag", "p.u., Earth Impedance (HV), Magnitude"
      c_ze0_h_phirad = "c:ze0_h:phirad", "rad, Earth Impedance (HV), Angle"
      c_ze0_m_r = "c:ze0_m:r", "p.u., Earth Impedance (MV), Real Part"
      c_ze0_m_i = "c:ze0_m:i", "p.u., Earth Impedance (MV), Imaginary Part"
      c_ze0_m_phi = "c:ze0_m:phi", "deg, Earth Impedance (MV), Angle"
      c_ze0_m_mag = "c:ze0_m:mag", "p.u., Earth Impedance (MV), Magnitude"
      c_ze0_m_phirad = "c:ze0_m:phirad", "rad, Earth Impedance (MV), Angle"
      c_ze0_l_r = "c:ze0_l:r", "p.u., Earth Impedance (LV), Real Part"
      c_ze0_l_i = "c:ze0_l:i", "p.u., Earth Impedance (LV), Imaginary Part"
      c_ze0_l_phi = "c:ze0_l:phi", "deg, Earth Impedance (LV), Angle"
      c_ze0_l_mag = "c:ze0_l:mag", "p.u., Earth Impedance (LV), Magnitude"
      c_ze0_l_phirad = "c:ze0_l:phirad", "rad, Earth Impedance (LV), Angle"
      c_i_star_h = "c:i_star_h", "Flag"
      c_i_star_m = "c:i_star_m", "Flag"
      c_i_star_l = "c:i_star_l", "Flag"
      c_i_earth_h = "c:i_earth_h", "Flag"
      c_i_earth_m = "c:i_earth_m", "Flag"
      c_i_earth_l = "c:i_earth_l", "Flag"
      c_ym0_r = "c:ym0:r", "p.u., 0-Sequ. Magn. Admitt., Real Part"
      c_ym0_i = "c:ym0:i", "p.u., 0-Sequ. Magn. Admitt., Imaginary Part"
      c_ym0_phi = "c:ym0:phi", "deg, 0-Sequ. Magn. Admitt., Angle"
      c_ym0_mag = "c:ym0:mag", "p.u., 0-Sequ. Magn. Admitt., Magnitude"
      c_ym0_phirad = "c:ym0:phirad", "rad, 0-Sequ. Magn. Admitt., Angle"
      c_i_stshort = "c:i_stshort", "Flag"
      c_Ustar_h = "c:Ustar_h", "kV, Star-Point Voltage (HV)"
      c_Ustar_m = "c:Ustar_m", "kV, Star-Point Voltage (MV)"
      c_Ustar_l = "c:Ustar_l", "kV, Star-Point Voltage (LV)"
      c_Istar_h = "c:Istar_h", "kA, Star-Point Current (HV)"
      c_Istar_m = "c:Istar_m", "kA, Star-Point Current (MV)"
      c_Istar_l = "c:Istar_l", "kA, Star-Point Current (LV)"
      c_um_1_r = "c:um_1:r", "p.u., Magnetising Flux (Real Part), Real Part"
      c_um_1_i = "c:um_1:i", "p.u., Magnetising Flux (Real Part), Imaginary Part"
      c_um_1_phi = "c:um_1:phi", "deg, Magnetising Flux (Real Part), Angle"
      c_um_1_mag = "c:um_1:mag", "p.u., Magnetising Flux (Real Part), Magnitude"
      c_um_1_phirad = "c:um_1:phirad", "rad, Magnetising Flux (Real Part), Angle"
      c_um_2_r = "c:um_2:r", "p.u., Magnetising Flux (Imaginary Part), Real Part"
      c_um_2_i = "c:um_2:i", "p.u., Magnetising Flux (Imaginary Part), Imaginary Part"
      c_um_2_phi = "c:um_2:phi", "deg, Magnetising Flux (Imaginary Part), Angle"
      c_um_2_mag = "c:um_2:mag", "p.u., Magnetising Flux (Imaginary Part), Magnitude"
      c_um_2_phirad = "c:um_2:phirad", "rad, Magnetising Flux (Imaginary Part), Angle"
      c_um_0_r = "c:um_0:r", "p.u., Magnetising Flux (0-Sequence), Real Part"
      c_um_0_i = "c:um_0:i", "p.u., Magnetising Flux (0-Sequence), Imaginary Part"
      c_um_0_phi = "c:um_0:phi", "deg, Magnetising Flux (0-Sequence), Angle"
      c_um_0_mag = "c:um_0:mag", "p.u., Magnetising Flux (0-Sequence), Magnitude"
      c_um_0_phirad = "c:um_0:phirad", "rad, Magnetising Flux (0-Sequence), Angle"
      c_u1_h_r = "c:u1_h:r", "Internal, Real Part"
      c_u1_h_i = "c:u1_h:i", "Internal, Imaginary Part"
      c_u1_h_phi = "c:u1_h:phi", "Internal, Angle"
      c_u1_h_mag = "c:u1_h:mag", "Internal, Magnitude"
      c_u1_h_phirad = "c:u1_h:phirad", "Internal, Angle"
      c_u1_m_r = "c:u1_m:r", "Internal, Real Part"
      c_u1_m_i = "c:u1_m:i", "Internal, Imaginary Part"
      c_u1_m_phi = "c:u1_m:phi", "Internal, Angle"
      c_u1_m_mag = "c:u1_m:mag", "Internal, Magnitude"
      c_u1_m_phirad = "c:u1_m:phirad", "Internal, Angle"
      c_u1_l_r = "c:u1_l:r", "Internal, Real Part"
      c_u1_l_i = "c:u1_l:i", "Internal, Imaginary Part"
      c_u1_l_phi = "c:u1_l:phi", "Internal, Angle"
      c_u1_l_mag = "c:u1_l:mag", "Internal, Magnitude"
      c_u1_l_phirad = "c:u1_l:phirad", "Internal, Angle"
      c_u2_h_r = "c:u2_h:r", "Internal, Real Part"
      c_u2_h_i = "c:u2_h:i", "Internal, Imaginary Part"
      c_u2_h_phi = "c:u2_h:phi", "Internal, Angle"
      c_u2_h_mag = "c:u2_h:mag", "Internal, Magnitude"
      c_u2_h_phirad = "c:u2_h:phirad", "Internal, Angle"
      c_u2_m_r = "c:u2_m:r", "Internal, Real Part"
      c_u2_m_i = "c:u2_m:i", "Internal, Imaginary Part"
      c_u2_m_phi = "c:u2_m:phi", "Internal, Angle"
      c_u2_m_mag = "c:u2_m:mag", "Internal, Magnitude"
      c_u2_m_phirad = "c:u2_m:phirad", "Internal, Angle"
      c_u2_l_r = "c:u2_l:r", "Internal, Real Part"
      c_u2_l_i = "c:u2_l:i", "Internal, Imaginary Part"
      c_u2_l_phi = "c:u2_l:phi", "Internal, Angle"
      c_u2_l_mag = "c:u2_l:mag", "Internal, Magnitude"
      c_u2_l_phirad = "c:u2_l:phirad", "Internal, Angle"
      c_w_0 = "c:w_0", "Nominal winding ratio"
      c_Gmload = "c:Gmload", "kW, Iron Losses"
      c_Xmload = "c:Xmload", "kvar, Magnetising Losses"
      c_Ploss = "c:Ploss", "MW, Losses (total)"
      c_Qloss = "c:Qloss", "Mvar, Reactive-Losses (total)"
      c_Plossld = "c:Plossld", "MW, Losses (load)"
      c_Qlossld = "c:Qlossld", "Mvar, Reactive-Losses (load)"
      c_Plossnld = "c:Plossnld", "MW, Losses (no load)"
      c_Qlossnld = "c:Qlossnld", "Mvar, Reactive-Losses (no load)"
      c_I0Delta_h = "c:I0Delta_h", "kA, Circulating current in Delta-Winding, Magnitude"
      c_I0Delta_m = "c:I0Delta_m", "kA, Circulating current in Delta-Winding, Magnitude"
      c_I0Delta_l = "c:I0Delta_l", "kA, Circulating current in Delta-Winding, Magnitude"
      c_Pmeas = "c:Pmeas", "MW, Measured active power"
      c_i_fixedtap = "c:i_fixedtap", "Flag"
      c_isat = "c:isat", "Saturated Model"
      c_loading_st = "c:loading_st", "%, Short Term Loading"
      c_dphiu_hm1 = "c:dphiu_hm1", "deg, Positive Sequence Voltage Drop Angle (HV-MV)"
      c_dphiu_hl1 = "c:dphiu_hl1", "deg, Positive Sequence Voltage Drop Angle (HV-LV)"
      c_dphiu_ml1 = "c:dphiu_ml1", "deg, Positive Sequence Voltage Drop Angle (MV-LV)"
      c_Inom_HV = "c:Inom_HV", "kA, Nominal Current (HV-Side)"
      c_Inom_MV = "c:Inom_MV", "kA, Nominal Current (MV-Side)"
      c_Inom_LV = "c:Inom_LV", "kA, Nominal Current (LV-Side)"
      e_du3tp_h = "e:du3tp_h", "%, Add. Voltage per Tap"
      e_ph3tr_h = "e:ph3tr_h", "deg, Phase of du"
      e_n3tap_h = "e:n3tap_h", "Act. Position"
      e_du3tp_m = "e:du3tp_m", "%, Add. Voltage per Tap"
      e_ph3tr_m = "e:ph3tr_m", "deg, Phase of du"
      e_n3tap_m = "e:n3tap_m", "Act. Position"
      e_du3tp_l = "e:du3tp_l", "%, Add. Voltage per Tap"
      e_ph3tr_l = "e:ph3tr_l", "deg, Phase of du"
      e_n3tap_l = "e:n3tap_l", "Act. Position"
      e_optaplimit = "e:optaplimit", "Operational limits for tap changer"
      e_optapmin_h = "e:optapmin_h", "Minimum Position"
      e_optapmax_h = "e:optapmax_h", "Maximum Position"
      e_optapmin_m = "e:optapmin_m", "Minimum Position"
      e_optapmax_m = "e:optapmax_m", "Maximum Position"
      e_optapmin_l = "e:optapmin_l", "Minimum Position"
      e_optapmax_l = "e:optapmax_l", "Maximum Position"
      e_maxload = "e:maxload", "%, Max. loading"
      e_lossAssign = "e:lossAssign", "Loss assignment"
      e_ictrlside = "e:ictrlside", "Controller for tap at"
      e_c_ptapc = "e:c_ptapc", "ElmTapctrl, External Tap Controller"
      e_c_pstac = "e:c_pstac", "External Station Controller"
      e_c_plcc = "e:c_plcc", "External LCC Controller"
      e_qdslCtrl = "e:qdslCtrl", "Quasi-Dynamic Model"
      e_ntrcn = "e:ntrcn", "Automatic Tap Changing"
      e_i_cont = "e:i_cont", "Tap Changer"
      e_t3ldc = "e:t3ldc", "Controlled Node is at"
      e_ilcph = "e:ilcph", "Phase"
      e_imldc = "e:imldc", "Control Mode"
      e_uset_mode = "e:uset_mode", "Setpoint"
      e_tapctrl = "e:tapctrl", "ElmTr3,ElmTr2, Tap Controller"
      e_iPpart = "e:iPpart", "Active power participation"
      e_i_rem = "e:i_rem", "Remote Control"
      e_p_rem = "e:p_rem", "StaBar,ElmTerm, Controlled Node"
      e_p_cub = "e:p_cub", "StaCubic,ElmBoundary, Controlled Branch (Cubicle)"
      e_cpCtrlNode = "e:cpCtrlNode", "Target Node"
      e_Kpart = "e:Kpart", "Participation factor"
      e_Ptolerance = "e:Ptolerance", "MW, Tolerance (+/-)"
      e_p_pmeas = "e:p_pmeas", "StaCubic,ElmBoundary, P measured at"
      e_usetp = "e:usetp", "p.u., Voltage Setpoint"
      e_usp_low = "e:usp_low", "p.u., Lower Voltage Bound"
      e_usp_up = "e:usp_up", "p.u., Upper Voltage Bound"
      e_psetp = "e:psetp", "MW, Active Power Setpoint"
      e_psp_low = "e:psp_low", "MW, Lower Active Power Bound"
      e_psp_up = "e:psp_up", "MW, Upper Active Power Bound"
      e_qsetp = "e:qsetp", "Mvar, Reactive Power Setpoint"
      e_qsp_low = "e:qsp_low", "Mvar, Lower Reactive Power Bound"
      e_qsp_up = "e:qsp_up", "Mvar, Upper Reactive Power Bound"
      e_Tctrl = "e:Tctrl", "s, Controller Time Constant"
      e_Kqctrl = "e:Kqctrl", "tap/Mvar, Controller Sensitivity dtap/dQ"
      e_Kpctrl = "e:Kpctrl", "tap/MW, Controller Sensitivity dtap/dP"
      e_ildc = "e:ildc", "Compensation"
      e_pldc = "e:pldc", "StaLdc, External LDC"
      e_icomp = "e:icomp", "Compounding"
      e_Vtolerance = "e:Vtolerance", "%, Tolerance (+/-)"
      e_pVccurve = "e:pVccurve", "IntVctrlcurve, V-Control-Curve"
      e_ictrlside2 = "e:ictrlside2", "Controller for tap at"
      e_ntrcn2 = "e:ntrcn2", "Automatic Tap Changing"
      e_i_cont2 = "e:i_cont2", "Tap Changer"
      e_t3ldc2 = "e:t3ldc2", "Controlled Node is at"
      e_ilcph2 = "e:ilcph2", "Phase"
      e_imldc2 = "e:imldc2", "Control Mode"
      e_uset_mode2 = "e:uset_mode2", "Setpoint"
      e_iPpart2 = "e:iPpart2", "Active power participation"
      e_i_rem2 = "e:i_rem2", "Remote Control"
      e_p_rem2 = "e:p_rem2", "StaBar,ElmTerm, Controlled Node"
      e_p_cub2 = "e:p_cub2", "StaCubic,ElmBoundary, Controlled Branch (Cubicle)"
      e_cpCtrlNode2 = "e:cpCtrlNode2", "Target Node"
      e_Kpart2 = "e:Kpart2", "Participation factor"
      e_Ptolerance2 = "e:Ptolerance2", "MW, Tolerance (+/-)"
      e_p_pmeas2 = "e:p_pmeas2", "StaCubic,ElmBoundary, P measured at"
      e_usetp2 = "e:usetp2", "p.u., Voltage Setpoint"
      e_usp_low2 = "e:usp_low2", "p.u., Lower Voltage Bound"
      e_usp_up2 = "e:usp_up2", "p.u., Upper Voltage Bound"
      e_psetp2 = "e:psetp2", "MW, Active Power Setpoint"
      e_psp_low2 = "e:psp_low2", "MW, Lower Active Power Bound"
      e_psp_up2 = "e:psp_up2", "MW, Upper Active Power Bound"
      e_qsetp2 = "e:qsetp2", "Mvar, Reactive Power Setpoint"
      e_qsp_low2 = "e:qsp_low2", "Mvar, Lower Reactive Power Bound"
      e_qsp_up2 = "e:qsp_up2", "Mvar, Upper Reactive Power Bound"
      e_Tctrl2 = "e:Tctrl2", "s, Controller Time Constant"
      e_Kqctrl2 = "e:Kqctrl2", "tap/Mvar, Controller Sensitivity dtap/dQ"
      e_Kpctrl2 = "e:Kpctrl2", "tap/MW, Controller Sensitivity dtap/dP"
      e_ildc2 = "e:ildc2", "Compensation"
      e_pldc2 = "e:pldc2", "StaLdc, External LDC"
      e_icomp2 = "e:icomp2", "Compounding"
      e_Vtolerance2 = "e:Vtolerance2", "%, Tolerance (+/-)"
      e_pVccurve2 = "e:pVccurve2", "IntVctrlcurve, V-Control-Curve"
      e_iTaps = "e:iTaps", "According to Measurement Report"
      e_iMeasTap = "e:iMeasTap", "for Tap at"
      e_iMeasLoc = "e:iMeasLoc", "Measured at"
      e_mTaps = "e:mTaps", "Measurement Report"
      e_iopt_hf = "e:iopt_hf", "Consider Capacitances"
      e_Cg_h = "e:Cg_h", "uF, HV-Ground"
      e_Cg_m = "e:Cg_m", "uF, MV-Ground"
      e_Cg_l = "e:Cg_l", "uF, LV-Ground"
      e_Cc1_hm = "e:Cc1_hm", "uF, HV-MV"
      e_Cc1_ml = "e:Cc1_ml", "uF, MV-LV"
      e_Cc1_lh = "e:Cc1_lh", "uF, LV-HV"
      e_Cc0_hm = "e:Cc0_hm", "uF, HV-MV"
      e_Cc0_ml = "e:Cc0_ml", "uF, MV-LV"
      e_Cc0_lh = "e:Cc0_lh", "uF, LV-HV"
      e_cpFeed = "e:cpFeed", "Feeder"
      e_ciDist = "e:ciDist", "Distance from infeed in number of buses"
      e_ciLater = "e:ciLater", "Lateral Index"
      e_ciDistRoot = "e:ciDistRoot", "Distance from first infeed in number of buses"
      e_ciDistAll = "e:ciDistAll", "Distance from infeed in number of buses including switches"
      e_ciDistAllRoot = "e:ciDistAllRoot", "Distance from first infeed in number of buses including switches"
      t_du3tp_h = "t:du3tp_h", "%, Add. Voltage per Tap"
      t_ph3tr_h = "t:ph3tr_h", "deg, Phase of du"
      t_n3tp0_h = "t:n3tp0_h", "Neutral Position"
      t_n3tmn_h = "t:n3tmn_h", "Min. Position"
      t_n3tmx_h = "t:n3tmx_h", "Max. Position"
      t_du3tp_m = "t:du3tp_m", "%, Add. Voltage per Tap"
      t_ph3tr_m = "t:ph3tr_m", "deg, Phase of du"
      t_n3tp0_m = "t:n3tp0_m", "Neutral Position"
      t_n3tmn_m = "t:n3tmn_m", "Min. Position"
      t_n3tmx_m = "t:n3tmx_m", "Max. Position"
      t_du3tp_l = "t:du3tp_l", "%, Add. Voltage per Tap"
      t_ph3tr_l = "t:ph3tr_l", "deg, Phase of du"
      t_n3tp0_l = "t:n3tp0_l", "Neutral Position"
      t_n3tmn_l = "t:n3tmn_l", "Min. Position"
      t_n3tmx_l = "t:n3tmx_l", "Max. Position"
      t_itapos = "t:itapos", "Tap Modelled at"
      t_i3loc = "t:i3loc", "Position"
      t_curm3 = "t:curm3", "%, No Load Current"
      t_pfe = "t:pfe", "kW, No Load Losses"
      t_i30lc = "t:i30lc", "Position"
      t_cr0m3 = "t:cr0m3", "%, No Load Current"
      t_rtox0_n = "t:rtox0_n", "Mag. R/X"
      t_iTapLoading = "t:iTapLoading", "Configuration"
      t_itapzdep = "t:itapzdep", "Tap dependent impedance"
      t_itapzside = "t:itapzside", "for Tap at"
      t_uktr3_h = "t:uktr3_h", "%, HV-MV"
      t_uktr3mn_h = "t:uktr3mn_h", "%, uk(HV-MV)(min. tap)"
      t_uktr3mx_h = "t:uktr3mx_h", "%, uk(HV-MV)(max. tap)"
      t_uktr3_m = "t:uktr3_m", "%, MV-LV"
      t_uktr3mn_m = "t:uktr3mn_m", "%, uk(MV-LV)(min. tap)"
      t_uktr3mx_m = "t:uktr3mx_m", "%, uk(MV-LV)(max. tap)"
      t_uktr3_l = "t:uktr3_l", "%, LV-HV"
      t_uktr3mn_l = "t:uktr3mn_l", "%, uk(LV-HV)(min. tap)"
      t_uktr3mx_l = "t:uktr3mx_l", "%, uk(LV-HV)(max. tap)"
      t_x1pu_h = "t:x1pu_h", "p.u., HV-MV"
      t_x1putmn_h = "t:x1putmn_h", "p.u., x1(HV-MV) (min. tap)"
      t_x1putmx_h = "t:x1putmx_h", "p.u., x1(HV-MV) (max. tap)"
      t_x1pu_m = "t:x1pu_m", "p.u., MV-LV"
      t_x1putmn_m = "t:x1putmn_m", "p.u., x1(MV-LV) (min. tap)"
      t_x1putmx_m = "t:x1putmx_m", "p.u., x1(MV-LV) (max. tap)"
      t_x1pu_l = "t:x1pu_l", "p.u., LV-HV"
      t_x1putmn_l = "t:x1putmn_l", "p.u., x1(LV-HV) (min. tap)"
      t_x1putmx_l = "t:x1putmx_l", "p.u., x1(LV-HV) (max. tap)"
      t_pcut3_h = "t:pcut3_h", "kW, HV-MV"
      t_uktrr3_h = "t:uktrr3_h", "%, HV-MV"
      t_xtor3_h = "t:xtor3_h", "HV-MV"
      t_r1pu_h = "t:r1pu_h", "p.u., HV-MV"
      t_pcut3mn_h = "t:pcut3mn_h", "kW, Pcu(HV-MV)(min. tap)"
      t_uktrr3mn_h = "t:uktrr3mn_h", "%, Re(uk)(HV-MV) (min. tap)"
      t_xtor3mn_h = "t:xtor3mn_h", "X/R (HV-MV)(min. tap)"
      t_r1putmn_h = "t:r1putmn_h", "p.u., r1(HV-MV) (min. tap)"
      t_pcut3mx_h = "t:pcut3mx_h", "kW, Pcu(HV-MV)(max. tap)"
      t_uktrr3mx_h = "t:uktrr3mx_h", "%, Re(uk)(HV-MV) (max. tap)"
      t_xtor3mx_h = "t:xtor3mx_h", "X/R (HV-MV)(max. tap)"
      t_r1putmx_h = "t:r1putmx_h", "p.u., r1(HV-MV) (max. tap)"
      t_pcut3_m = "t:pcut3_m", "kW, MV-LV"
      t_uktrr3_m = "t:uktrr3_m", "%, MV-LV"
      t_xtor3_m = "t:xtor3_m", "MV-LV"
      t_r1pu_m = "t:r1pu_m", "p.u., MV-LV"
      t_pcut3mn_m = "t:pcut3mn_m", "kW, Pcu(MV-LV)(min. tap)"
      t_uktrr3mn_m = "t:uktrr3mn_m", "%, Re(uk)(MV-LV) (min. tap)"
      t_xtor3mn_m = "t:xtor3mn_m", "X/R (MV-LV)(min. tap)"
      t_r1putmn_m = "t:r1putmn_m", "p.u., r1(MV-LV) (min. tap)"
      t_pcut3mx_m = "t:pcut3mx_m", "kW, Pcu(MV-LV)(max. tap)"
      t_uktrr3mx_m = "t:uktrr3mx_m", "%, Re(uk)(MV-LV) (max. tap)"
      t_xtor3mx_m = "t:xtor3mx_m", "X/R (MV-LV)(max. tap)"
      t_r1putmx_m = "t:r1putmx_m", "p.u., r1(MV-LV) (max. tap)"
      t_pcut3_l = "t:pcut3_l", "kW, LV-HV"
      t_uktrr3_l = "t:uktrr3_l", "%, LV-HV"
      t_xtor3_l = "t:xtor3_l", "LV-HV"
      t_r1pu_l = "t:r1pu_l", "p.u., LV-HV"
      t_pcut3mn_l = "t:pcut3mn_l", "kW, Pcu(LV-HV)(min. tap)"
      t_uktrr3mn_l = "t:uktrr3mn_l", "%, Re(uk)(LV-HV) (min. tap)"
      t_xtor3mn_l = "t:xtor3mn_l", "X/R (LV-HV)(min. tap)"
      t_r1putmn_l = "t:r1putmn_l", "p.u., r1(LV-HV) (min. tap)"
      t_pcut3mx_l = "t:pcut3mx_l", "kW, Pcu(LV-HV)(max. tap)"
      t_uktrr3mx_l = "t:uktrr3mx_l", "%, Re(uk)(LV-HV) (max. tap)"
      t_xtor3mx_l = "t:xtor3mx_l", "X/R (LV-HV)(max. tap)"
      t_r1putmx_l = "t:r1putmx_l", "p.u., r1(LV-HV) (max. tap)"
      t_uk0hm = "t:uk0hm", "%, HV-MV"
      t_uk0mnhm = "t:uk0mnhm", "%, uk0(HV-MV)(min. tap)"
      t_uk0mxhm = "t:uk0mxhm", "%, uk0(HV-MV)(max. tap)"
      t_uk0ml = "t:uk0ml", "%, MV-LV"
      t_uk0mnml = "t:uk0mnml", "%, uk0(MV-LV)(min. tap)"
      t_uk0mxml = "t:uk0mxml", "%, uk0(MV-LV)(max. tap)"
      t_uk0hl = "t:uk0hl", "%, LV-HV"
      t_uk0mnhl = "t:uk0mnhl", "%, uk0(LV-HV)(min. tap)"
      t_uk0mxhl = "t:uk0mxhl", "%, uk0(LV-HV)(max. tap)"
      t_x0puhm = "t:x0puhm", "p.u., HV-MV"
      t_x0pumnhm = "t:x0pumnhm", "p.u., x0(HV-MV) (min. tap)"
      t_x0pumxhm = "t:x0pumxhm", "p.u., x0(HV-MV) (max. tap)"
      t_x0puml = "t:x0puml", "p.u., MV-LV"
      t_x0pumnml = "t:x0pumnml", "p.u., x0(MV-LV) (min. tap)"
      t_x0pumxml = "t:x0pumxml", "p.u., x0(MV-LV) (max. tap)"
      t_x0puhl = "t:x0puhl", "p.u., LV-HV"
      t_x0pumnhl = "t:x0pumnhl", "p.u., x0(LV-HV) (min. tap)"
      t_x0pumxhl = "t:x0pumxhl", "p.u., x0(LV-HV) (max. tap)"
      t_ur0hm = "t:ur0hm", "%, HV-MV"
      t_ur0mnhm = "t:ur0mnhm", "%, Re(uk0)(HV-MV)(min. tap)"
      t_ur0mxhm = "t:ur0mxhm", "%, Re(uk0)(HV-MV)(max. tap)"
      t_ur0ml = "t:ur0ml", "%, MV-LV"
      t_ur0mnml = "t:ur0mnml", "%, Re(uk0)(MV-LV)(min. tap)"
      t_ur0mxml = "t:ur0mxml", "%, Re(uk0)(MV-LV)(max. tap)"
      t_ur0hl = "t:ur0hl", "%, LV-HV"
      t_ur0mnhl = "t:ur0mnhl", "%, Re(uk0)(LV-HV)(min. tap)"
      t_ur0mxhl = "t:ur0mxhl", "%, Re(uk0)(LV-HV)(max. tap)"
      t_xtr0hm = "t:xtr0hm", "HV-MV"
      t_xtr0mnhm = "t:xtr0mnhm", "X0/R0 (HV-MV)(min. tap)"
      t_xtr0mxhm = "t:xtr0mxhm", "X0/R0 (HV-MV)(max. tap)"
      t_xtr0ml = "t:xtr0ml", "MV-LV"
      t_xtr0mnml = "t:xtr0mnml", "X0/R0 (MV-LV)(min. tap)"
      t_xtr0mxml = "t:xtr0mxml", "X0/R0 (MV-LV)(max. tap)"
      t_xtr0hl = "t:xtr0hl", "LV-HV"
      t_xtr0mnhl = "t:xtr0mnhl", "X0/R0 (LV-HV)(min. tap)"
      t_xtr0mxhl = "t:xtr0mxhl", "X0/R0 (LV-HV)(max. tap)"
      t_r0puhm = "t:r0puhm", "p.u., HV-MV"
      t_r0pumnhm = "t:r0pumnhm", "p.u., r0(HV-MV) (min. tap)"
      t_r0pumxhm = "t:r0pumxhm", "p.u., r0(HV-MV) (max. tap)"
      t_r0puml = "t:r0puml", "p.u., MV-LV"
      t_r0pumnml = "t:r0pumnml", "p.u., r0(MV-LV) (min. tap)"
      t_r0pumxml = "t:r0pumxml", "p.u., r0(MV-LV) (max. tap)"
      t_r0puhl = "t:r0puhl", "p.u., LV-HV"
      t_r0pumnhl = "t:r0pumnhl", "p.u., r0(LV-HV) (min. tap)"
      t_r0pumxhl = "t:r0pumxhl", "p.u., r0(LV-HV) (max. tap)"
      t_itrldf = "t:itrldf", "Type"
      t_satcue = "t:satcue", "%, Current (RMS)"
      t_satvol = "t:satvol", "p.u., Voltage (RMS)"
      t_iIntPola = "t:iIntPola", "Interpolation"
      t_smoothf = "t:smoothf", "%, Smoothing Factor"
      t_itratioadpt = "t:itratioadpt", "Transformer Ratio Adaptation"
      r_c_ptapc = "r:c_ptapc", "ElmTapctrl, External Tap Controller"
      r_c_pstac = "r:c_pstac", "External Station Controller"
      r_c_plcc = "r:c_plcc", "External LCC Controller"
      r_qdslCtrl = "r:qdslCtrl", "Quasi-Dynamic Model"
      r_tapctrl = "r:tapctrl", "ElmTr3,ElmTr2, Tap Controller"
      r_p_rem = "r:p_rem", "StaBar,ElmTerm, Controlled Node"
      r_p_cub = "r:p_cub", "StaCubic,ElmBoundary, Controlled Branch (Cubicle)"
      r_cpCtrlNode = "r:cpCtrlNode", "Target Node"
      r_p_pmeas = "r:p_pmeas", "StaCubic,ElmBoundary, P measured at"
      r_pldc = "r:pldc", "StaLdc, External LDC"
      r_pVccurve = "r:pVccurve", "IntVctrlcurve, V-Control-Curve"
      r_p_rem2 = "r:p_rem2", "StaBar,ElmTerm, Controlled Node"
      r_p_cub2 = "r:p_cub2", "StaCubic,ElmBoundary, Controlled Branch (Cubicle)"
      r_cpCtrlNode2 = "r:cpCtrlNode2", "Target Node"
      r_p_pmeas2 = "r:p_pmeas2", "StaCubic,ElmBoundary, P measured at"
      r_pldc2 = "r:pldc2", "StaLdc, External LDC"
      r_pVccurve2 = "r:pVccurve2", "IntVctrlcurve, V-Control-Curve"
      r_cpFeed = "r:cpFeed", "Feeder"
      m_Ikss_bushv = "m:Ikss:bushv", "kA, Initial Short-Circuit Current"
      m_Ikss_busmv = "m:Ikss:busmv", "kA, Initial Short-Circuit Current"
      m_Ikss_buslv = "m:Ikss:buslv", "kA, Initial Short-Circuit Current"
      m_ikss_bushv = "m:ikss:bushv", "p.u., Initial Short-Circuit Current, Magnitude"
      m_ikss_busmv = "m:ikss:busmv", "p.u., Initial Short-Circuit Current, Magnitude"
      m_ikss_buslv = "m:ikss:buslv", "p.u., Initial Short-Circuit Current, Magnitude"
      m_phii_bushv = "m:phii:bushv", "deg, Phase Current, Angle"
      m_phii_busmv = "m:phii:busmv", "deg, Phase Current, Angle"
      m_phii_buslv = "m:phii:buslv", "deg, Phase Current, Angle"
      m_maxIkss_bushv = "m:maxIkss:bushv", "kA, Max. Initial Short-Circuit Current"
      m_maxIkss_busmv = "m:maxIkss:busmv", "kA, Max. Initial Short-Circuit Current"
      m_maxIkss_buslv = "m:maxIkss:buslv", "kA, Max. Initial Short-Circuit Current"
      m_Skss_bushv = "m:Skss:bushv", "MVA, Initial Short-Circuit Power"
      m_Skss_busmv = "m:Skss:busmv", "MVA, Initial Short-Circuit Power"
      m_Skss_buslv = "m:Skss:buslv", "MVA, Initial Short-Circuit Power"
      m_ip_bushv = "m:ip:bushv", "kA, Peak Short-Circuit Current"
      m_ip_busmv = "m:ip:busmv", "kA, Peak Short-Circuit Current"
      m_ip_buslv = "m:ip:buslv", "kA, Peak Short-Circuit Current"
      m_maxip_bushv = "m:maxip:bushv", "kA, Max. Peak Short-Circuit Current"
      m_maxip_busmv = "m:maxip:busmv", "kA, Max. Peak Short-Circuit Current"
      m_maxip_buslv = "m:maxip:buslv", "kA, Max. Peak Short-Circuit Current"
      m_maxIb_bushv = "m:maxIb:bushv", "kA, Max. Breaking Current"
      m_maxIb_busmv = "m:maxIb:busmv", "kA, Max. Breaking Current"
      m_maxIb_buslv = "m:maxIb:buslv", "kA, Max. Breaking Current"
      m_maxIth_bushv = "m:maxIth:bushv", "kA, Max. Thermal equivalent Short-Circuit Current"
      m_maxIth_busmv = "m:maxIth:busmv", "kA, Max. Thermal equivalent Short-Circuit Current"
      m_maxIth_buslv = "m:maxIth:buslv", "kA, Max. Thermal equivalent Short-Circuit Current"
      m_Ib_bushv = "m:Ib:bushv", "kA, Short-Circuit Breaking Current"
      m_Ib_busmv = "m:Ib:busmv", "kA, Short-Circuit Breaking Current"
      m_Ib_buslv = "m:Ib:buslv", "kA, Short-Circuit Breaking Current"
      m_Sb_bushv = "m:Sb:bushv", "MVA, Short-Circuit Breaking Power"
      m_Sb_busmv = "m:Sb:busmv", "MVA, Short-Circuit Breaking Power"
      m_Sb_buslv = "m:Sb:buslv", "MVA, Short-Circuit Breaking Power"
      m_Ik_bushv = "m:Ik:bushv", "kA, Steady-State Short-Circuit Current"
      m_Ik_busmv = "m:Ik:busmv", "kA, Steady-State Short-Circuit Current"
      m_Ik_buslv = "m:Ik:buslv", "kA, Steady-State Short-Circuit Current"
      m_Ith_bushv = "m:Ith:bushv", "kA, Thermal equivalent Short-Circuit Current"
      m_Ith_busmv = "m:Ith:busmv", "kA, Thermal equivalent Short-Circuit Current"
      m_Ith_buslv = "m:Ith:buslv", "kA, Thermal equivalent Short-Circuit Current"
      m_I_bushv = "m:I:bushv", "kA, Short-Circuit Current"
      m_I_busmv = "m:I:busmv", "kA, Short-Circuit Current"
      m_I_buslv = "m:I:buslv", "kA, Short-Circuit Current"
      m_R_bushv = "m:R:bushv", "Ohm, Fault Impedance, Real Part"
      m_R_busmv = "m:R:busmv", "Ohm, Fault Impedance, Real Part"
      m_R_buslv = "m:R:buslv", "Ohm, Fault Impedance, Real Part"
      m_X_bushv = "m:X:bushv", "Ohm, Fault Impedance, Imaginary Part"
      m_X_busmv = "m:X:busmv", "Ohm, Fault Impedance, Imaginary Part"
      m_X_buslv = "m:X:buslv", "Ohm, Fault Impedance, Imaginary Part"
      m_Z_bushv = "m:Z:bushv", "Ohm, Fault Impedance, Magnitude"
      m_Z_busmv = "m:Z:busmv", "Ohm, Fault Impedance, Magnitude"
      m_Z_buslv = "m:Z:buslv", "Ohm, Fault Impedance, Magnitude"
      m_phiz_bushv = "m:phiz:bushv", "deg, Fault Impedance, Angle"
      m_phiz_busmv = "m:phiz:busmv", "deg, Fault Impedance, Angle"
      m_phiz_buslv = "m:phiz:buslv", "deg, Fault Impedance, Angle"
      m_zSbase_bushv = "m:zSbase:bushv", "p.u., Fault Impedance, Magnitude"
      m_zSbase_busmv = "m:zSbase:busmv", "p.u., Fault Impedance, Magnitude"
      m_zSbase_buslv = "m:zSbase:buslv", "p.u., Fault Impedance, Magnitude"
      m_XtoR_bushv = "m:XtoR:bushv", "X/R ratio"
      m_XtoR_busmv = "m:XtoR:busmv", "X/R ratio"
      m_XtoR_buslv = "m:XtoR:buslv", "X/R ratio"
      m_Zs_bushv = "m:Zs:bushv", "Ohm, Source Impedance, Magnitude"
      m_Zs_busmv = "m:Zs:busmv", "Ohm, Source Impedance, Magnitude"
      m_Zs_buslv = "m:Zs:buslv", "Ohm, Source Impedance, Magnitude"
      m_phizs_bushv = "m:phizs:bushv", "deg, Source Impedance, Angle"
      m_phizs_busmv = "m:phizs:busmv", "deg, Source Impedance, Angle"
      m_phizs_buslv = "m:phizs:buslv", "deg, Source Impedance, Angle"
      m_Rs_bushv = "m:Rs:bushv", "Ohm, Source Impedance, Real Part"
      m_Rs_busmv = "m:Rs:busmv", "Ohm, Source Impedance, Real Part"
      m_Rs_buslv = "m:Rs:buslv", "Ohm, Source Impedance, Real Part"
      m_Xs_bushv = "m:Xs:bushv", "Ohm, Source Impedance, Imaginary Part"
      m_Xs_busmv = "m:Xs:busmv", "Ohm, Source Impedance, Imaginary Part"
      m_Xs_buslv = "m:Xs:buslv", "Ohm, Source Impedance, Imaginary Part"
      m_phiui_bushv = "m:phiui:bushv", "deg, Angle between Voltage and Current"
      m_phiui_busmv = "m:phiui:busmv", "deg, Angle between Voltage and Current"
      m_phiui_buslv = "m:phiui:buslv", "deg, Angle between Voltage and Current"
      m_cosphiui_bushv = "m:cosphiui:bushv", "cosphi"
      m_cosphiui_busmv = "m:cosphiui:busmv", "cosphi"
      m_cosphiui_buslv = "m:cosphiui:buslv", "cosphi"
      m_P_bushv = "m:P:bushv", "MW, Active Power"
      m_P_busmv = "m:P:busmv", "MW, Active Power"
      m_P_buslv = "m:P:buslv", "MW, Active Power"
      m_Q_bushv = "m:Q:bushv", "Mvar, Reactive Power"
      m_Q_busmv = "m:Q:busmv", "Mvar, Reactive Power"
      m_Q_buslv = "m:Q:buslv", "Mvar, Reactive Power"
      m_BrkIpload_bushv = "m:BrkIpload:bushv", "%, Breaker Loading (peak)"
      m_BrkIpload_busmv = "m:BrkIpload:busmv", "%, Breaker Loading (peak)"
      m_BrkIpload_buslv = "m:BrkIpload:buslv", "%, Breaker Loading (peak)"
      m_BrkIbload_bushv = "m:BrkIbload:bushv", "%, Breaker Loading (breaking)"
      m_BrkIbload_busmv = "m:BrkIbload:busmv", "%, Breaker Loading (breaking)"
      m_BrkIbload_buslv = "m:BrkIbload:buslv", "%, Breaker Loading (breaking)"
      m_BrkmaxIb_bushv = "m:BrkmaxIb:bushv", "kA, Max. Breaking Current (breaker time)"
      m_BrkmaxIb_busmv = "m:BrkmaxIb:busmv", "kA, Max. Breaking Current (breaker time)"
      m_BrkmaxIb_buslv = "m:BrkmaxIb:buslv", "kA, Max. Breaking Current (breaker time)"
      n_Ikss_bushv = "n:Ikss:bushv", "kA, Initial Short-Circuit Current"
      n_Ikss_busmv = "n:Ikss:busmv", "kA, Initial Short-Circuit Current"
      n_Ikss_buslv = "n:Ikss:buslv", "kA, Initial Short-Circuit Current"
      n_I_bushv = "n:I:bushv", "kA, Short-Circuit Current"
      n_I_busmv = "n:I:busmv", "kA, Short-Circuit Current"
      n_I_buslv = "n:I:buslv", "kA, Short-Circuit Current"
      n_phii_bushv = "n:phii:bushv", "deg, Phase Current, Angle"
      n_phii_busmv = "n:phii:busmv", "deg, Phase Current, Angle"
      n_phii_buslv = "n:phii:buslv", "deg, Phase Current, Angle"
      n_Skss_bushv = "n:Skss:bushv", "MVA, Initial Short-Circuit Power"
      n_Skss_busmv = "n:Skss:busmv", "MVA, Initial Short-Circuit Power"
      n_Skss_buslv = "n:Skss:buslv", "MVA, Initial Short-Circuit Power"
      n_ip_bushv = "n:ip:bushv", "kA, Peak Short-Circuit Current"
      n_ip_busmv = "n:ip:busmv", "kA, Peak Short-Circuit Current"
      n_ip_buslv = "n:ip:buslv", "kA, Peak Short-Circuit Current"
      n_Ib_bushv = "n:Ib:bushv", "kA, Short-Circuit Breaking Current"
      n_Ib_busmv = "n:Ib:busmv", "kA, Short-Circuit Breaking Current"
      n_Ib_buslv = "n:Ib:buslv", "kA, Short-Circuit Breaking Current"
      n_Sb_bushv = "n:Sb:bushv", "MVA, Short-Circuit Breaking Power"
      n_Sb_busmv = "n:Sb:busmv", "MVA, Short-Circuit Breaking Power"
      n_Sb_buslv = "n:Sb:buslv", "MVA, Short-Circuit Breaking Power"
      n_Ik_bushv = "n:Ik:bushv", "kA, Steady-State Short-Circuit Current"
      n_Ik_busmv = "n:Ik:busmv", "kA, Steady-State Short-Circuit Current"
      n_Ik_buslv = "n:Ik:buslv", "kA, Steady-State Short-Circuit Current"
      n_Ith_bushv = "n:Ith:bushv", "kA, Thermal equivalent Short-Circuit Current"
      n_Ith_busmv = "n:Ith:busmv", "kA, Thermal equivalent Short-Circuit Current"
      n_Ith_buslv = "n:Ith:buslv", "kA, Thermal equivalent Short-Circuit Current"
      n_R_bushv = "n:R:bushv", "Ohm, Short-Circuit Impedance, Real Part"
      n_R_busmv = "n:R:busmv", "Ohm, Short-Circuit Impedance, Real Part"
      n_R_buslv = "n:R:buslv", "Ohm, Short-Circuit Impedance, Real Part"
      n_X_bushv = "n:X:bushv", "Ohm, Short-Circuit Impedance, Imaginary Part"
      n_X_busmv = "n:X:busmv", "Ohm, Short-Circuit Impedance, Imaginary Part"
      n_X_buslv = "n:X:buslv", "Ohm, Short-Circuit Impedance, Imaginary Part"
      n_Z_bushv = "n:Z:bushv", "Ohm, Short-Circuit Impedance, Magnitude"
      n_Z_busmv = "n:Z:busmv", "Ohm, Short-Circuit Impedance, Magnitude"
      n_Z_buslv = "n:Z:buslv", "Ohm, Short-Circuit Impedance, Magnitude"
      n_phiz_bushv = "n:phiz:bushv", "deg, Short-Circuit Impedance, Angle"
      n_phiz_busmv = "n:phiz:busmv", "deg, Short-Circuit Impedance, Angle"
      n_phiz_buslv = "n:phiz:buslv", "deg, Short-Circuit Impedance, Angle"
      n_rSbase_bushv = "n:rSbase:bushv", "p.u., Short-Circuit Impedance, Real Part"
      n_rSbase_busmv = "n:rSbase:busmv", "p.u., Short-Circuit Impedance, Real Part"
      n_rSbase_buslv = "n:rSbase:buslv", "p.u., Short-Circuit Impedance, Real Part"
      n_xSbase_bushv = "n:xSbase:bushv", "p.u., Short-Circuit Impedance, Imaginary Part"
      n_xSbase_busmv = "n:xSbase:busmv", "p.u., Short-Circuit Impedance, Imaginary Part"
      n_xSbase_buslv = "n:xSbase:buslv", "p.u., Short-Circuit Impedance, Imaginary Part"
      n_zSbase_bushv = "n:zSbase:bushv", "p.u., Short-Circuit Impedance, Magnitude"
      n_zSbase_busmv = "n:zSbase:busmv", "p.u., Short-Circuit Impedance, Magnitude"
      n_zSbase_buslv = "n:zSbase:buslv", "p.u., Short-Circuit Impedance, Magnitude"
      n_XtoR_bushv = "n:XtoR:bushv", "X/R ratio (peak)"
      n_XtoR_busmv = "n:XtoR:busmv", "X/R ratio (peak)"
      n_XtoR_buslv = "n:XtoR:buslv", "X/R ratio (peak)"
      n_RtoX_bushv = "n:RtoX:bushv", "R/X ratio (peak)"
      n_RtoX_busmv = "n:RtoX:busmv", "R/X ratio (peak)"
      n_RtoX_buslv = "n:RtoX:buslv", "R/X ratio (peak)"
      n_ur_bushv = "n:ur:bushv", "p.u., Voltage, Real Part"
      n_ur_busmv = "n:ur:busmv", "p.u., Voltage, Real Part"
      n_ur_buslv = "n:ur:buslv", "p.u., Voltage, Real Part"
      n_ui_bushv = "n:ui:bushv", "p.u., Voltage, Imaginary Part"
      n_ui_busmv = "n:ui:busmv", "p.u., Voltage, Imaginary Part"
      n_ui_buslv = "n:ui:buslv", "p.u., Voltage, Imaginary Part"
      n_u_bushv = "n:u:bushv", "p.u., Voltage, Magnitude"
      n_u_busmv = "n:u:busmv", "p.u., Voltage, Magnitude"
      n_u_buslv = "n:u:buslv", "p.u., Voltage, Magnitude"
      n_upc_bushv = "n:upc:bushv", "%, Voltage, Magnitude"
      n_upc_busmv = "n:upc:busmv", "%, Voltage, Magnitude"
      n_upc_buslv = "n:upc:buslv", "%, Voltage, Magnitude"
      n_U_bushv = "n:U:bushv", "kV, Line-Ground Voltage, Magnitude"
      n_U_busmv = "n:U:busmv", "kV, Line-Ground Voltage, Magnitude"
      n_U_buslv = "n:U:buslv", "kV, Line-Ground Voltage, Magnitude"
      n_Ul_bushv = "n:Ul:bushv", "kV, Line-Line Voltage, Magnitude"
      n_Ul_busmv = "n:Ul:busmv", "kV, Line-Line Voltage, Magnitude"
      n_Ul_buslv = "n:Ul:buslv", "kV, Line-Line Voltage, Magnitude"
      n_phiu_bushv = "n:phiu:bushv", "deg, Voltage, Angle"
      n_phiu_busmv = "n:phiu:busmv", "deg, Voltage, Angle"
      n_phiu_buslv = "n:phiu:buslv", "deg, Voltage, Angle"
      n_du_bushv = "n:du:bushv", "%, Voltage Deviation"
      n_du_busmv = "n:du:busmv", "%, Voltage Deviation"
      n_du_buslv = "n:du:buslv", "%, Voltage Deviation"
      n_urpref_bushv = "n:urpref:bushv", "p.u., Pre-fault Voltage, Real Part"
      n_urpref_busmv = "n:urpref:busmv", "p.u., Pre-fault Voltage, Real Part"
      n_urpref_buslv = "n:urpref:buslv", "p.u., Pre-fault Voltage, Real Part"
      n_uipref_bushv = "n:uipref:bushv", "p.u., Pre-fault Voltage, Imaginary Part"
      n_uipref_busmv = "n:uipref:busmv", "p.u., Pre-fault Voltage, Imaginary Part"
      n_uipref_buslv = "n:uipref:buslv", "p.u., Pre-fault Voltage, Imaginary Part"
      n_upref_bushv = "n:upref:bushv", "p.u., Pre-fault Voltage, Magnitude"
      n_upref_busmv = "n:upref:busmv", "p.u., Pre-fault Voltage, Magnitude"
      n_upref_buslv = "n:upref:buslv", "p.u., Pre-fault Voltage, Magnitude"
      n_uprefpc_bushv = "n:uprefpc:bushv", "%, Pre-fault Voltage, Magnitude"
      n_uprefpc_busmv = "n:uprefpc:busmv", "%, Pre-fault Voltage, Magnitude"
      n_uprefpc_buslv = "n:uprefpc:buslv", "%, Pre-fault Voltage, Magnitude"
      n_Upref_bushv = "n:Upref:bushv", "kV, Line-Ground Pre-fault Voltage, Magnitude"
      n_Upref_busmv = "n:Upref:busmv", "kV, Line-Ground Pre-fault Voltage, Magnitude"
      n_Upref_buslv = "n:Upref:buslv", "kV, Line-Ground Pre-fault Voltage, Magnitude"
      n_Ulpref_bushv = "n:Ulpref:bushv", "kV, Line-Line Pre-fault Voltage, Magnitude"
      n_Ulpref_busmv = "n:Ulpref:busmv", "kV, Line-Line Pre-fault Voltage, Magnitude"
      n_Ulpref_buslv = "n:Ulpref:buslv", "kV, Line-Line Pre-fault Voltage, Magnitude"
      n_phiupref_bushv = "n:phiupref:bushv", "deg, Pre-fault Voltage, Angle"
      n_phiupref_busmv = "n:phiupref:busmv", "deg, Pre-fault Voltage, Angle"
      n_phiupref_buslv = "n:phiupref:buslv", "deg, Pre-fault Voltage, Angle"
      n_phiui_bushv = "n:phiui:bushv", "deg, Angle between Voltage and Current"
      n_phiui_busmv = "n:phiui:busmv", "deg, Angle between Voltage and Current"
      n_phiui_buslv = "n:phiui:buslv", "deg, Angle between Voltage and Current"
      n_cfac_bushv = "n:cfac:bushv", "Voltage Factor"
      n_cfac_busmv = "n:cfac:busmv", "Voltage Factor"
      n_cfac_buslv = "n:cfac:buslv", "Voltage Factor"
      n_idc_bushv = "n:idc:bushv", "kA, Decaying Component"
      n_idc_busmv = "n:idc:busmv", "kA, Decaying Component"
      n_idc_buslv = "n:idc:buslv", "kA, Decaying Component"
      n_Iasy_bushv = "n:Iasy:bushv", "kA, Asymmetrical Breaking Current"
      n_Iasy_busmv = "n:Iasy:busmv", "kA, Asymmetrical Breaking Current"
      n_Iasy_buslv = "n:Iasy:buslv", "kA, Asymmetrical Breaking Current"
      n_Ithload_bushv = "n:Ithload:bushv", "%, Loading, Thermal equivalent Current"
      n_Ithload_busmv = "n:Ithload:busmv", "%, Loading, Thermal equivalent Current"
      n_Ithload_buslv = "n:Ithload:buslv", "%, Loading, Thermal equivalent Current"
      n_Ipload_bushv = "n:Ipload:bushv", "%, Loading, Peak Short-Circuit Current"
      n_Ipload_busmv = "n:Ipload:busmv", "%, Loading, Peak Short-Circuit Current"
      n_Ipload_buslv = "n:Ipload:buslv", "%, Loading, Peak Short-Circuit Current"
      n_Ithrtk_bushv = "n:Ithrtk:bushv", "kA, Rated Short-Time Current (Tk)"
      n_Ithrtk_busmv = "n:Ithrtk:busmv", "kA, Rated Short-Time Current (Tk)"
      n_Ithrtk_buslv = "n:Ithrtk:buslv", "kA, Rated Short-Time Current (Tk)"
      n_Tn_bushv = "n:Tn:bushv", "ms, Grid Time Constant"
      n_Tn_busmv = "n:Tn:busmv", "ms, Grid Time Constant"
      n_Tn_buslv = "n:Tn:buslv", "ms, Grid Time Constant"
      n_XtoR_b_bushv = "n:XtoR_b:bushv", "X/R ratio (dc)"
      n_XtoR_b_busmv = "n:XtoR_b:busmv", "X/R ratio (dc)"
      n_XtoR_b_buslv = "n:XtoR_b:buslv", "X/R ratio (dc)"
      n_RtoX_b_bushv = "n:RtoX_b:bushv", "R/X ratio (dc)"
      n_RtoX_b_busmv = "n:RtoX_b:busmv", "R/X ratio (dc)"
      n_RtoX_b_buslv = "n:RtoX_b:buslv", "R/X ratio (dc)"
      n_fctof_b_bushv = "n:fctof_b:bushv", "Equivalent Frequency fc/f (dc)"
      n_fctof_b_busmv = "n:fctof_b:busmv", "Equivalent Frequency fc/f (dc)"
      n_fctof_b_buslv = "n:fctof_b:buslv", "Equivalent Frequency fc/f (dc)"
      c_ntpmn_A = "c:ntpmn:A", "Minimum Tap Position"
      c_ntpmn_B = "c:ntpmn:B", "Minimum Tap Position"
      c_ntpmn_C = "c:ntpmn:C", "Minimum Tap Position"
      c_ntpmn_N = "c:ntpmn:N", "Minimum Tap Position"
      c_ntpmx_A = "c:ntpmx:A", "Maximum Tap Position"
      c_ntpmx_B = "c:ntpmx:B", "Maximum Tap Position"
      c_ntpmx_C = "c:ntpmx:C", "Maximum Tap Position"
      c_ntpmx_N = "c:ntpmx:N", "Maximum Tap Position"
      c_nntap0_A = "c:nntap0:A", "Neutral Position"
      c_nntap0_B = "c:nntap0:B", "Neutral Position"
      c_nntap0_C = "c:nntap0:C", "Neutral Position"
      c_nntap0_N = "c:nntap0:N", "Neutral Position"
      c_ntagrd_A = "c:ntagrd:A", "Phase Shift"
      c_ntagrd_B = "c:ntagrd:B", "Phase Shift"
      c_ntagrd_C = "c:ntagrd:C", "Phase Shift"
      c_ntagrd_N = "c:ntagrd:N", "Phase Shift"
      c_pT_A = "c:pT:A", "%, Range of Voltage Adjustment"
      c_pT_B = "c:pT:B", "%, Range of Voltage Adjustment"
      c_pT_C = "c:pT:C", "%, Range of Voltage Adjustment"
      c_pT_N = "c:pT:N", "%, Range of Voltage Adjustment"
      c_Kt_h = "c:Kt_h", "VDE/IEC Correction Factor, HV-MV"
      c_Kt_m = "c:Kt_m", "VDE/IEC Correction Factor, MV-LV"
      c_Kt_l = "c:Kt_l", "VDE/IEC Correction Factor, LV-HV"
      e_iblock = "e:iblock", "Unit Transformer"
      e_pT_h = "e:pT_h", "Voltage Range"
      e_pT_m = "e:pT_m", "Voltage Range"
      e_pT_l = "e:pT_l", "Voltage Range"
      t_pT_h = "t:pT_h", "Voltage Range"
      t_pT_m = "t:pT_m", "Voltage Range"
      t_pT_l = "t:pT_l", "Voltage Range"
      t_oltc_h = "t:oltc_h", "HV-Side"
      t_oltc_m = "t:oltc_m", "MV-Side"
      t_oltc_l = "t:oltc_l", "LV-Side"
      m_Ikss_bushv_A = "m:Ikss:bushv:A", "kA, Initial Short-Circuit Current"
      m_Ikss_bushv_B = "m:Ikss:bushv:B", "kA, Initial Short-Circuit Current"
      m_Ikss_bushv_C = "m:Ikss:bushv:C", "kA, Initial Short-Circuit Current"
      m_Ikss_bushv_N = "m:Ikss:bushv:N", "kA, Initial Short-Circuit Current"
      m_Ikss_busmv_A = "m:Ikss:busmv:A", "kA, Initial Short-Circuit Current"
      m_Ikss_busmv_B = "m:Ikss:busmv:B", "kA, Initial Short-Circuit Current"
      m_Ikss_busmv_C = "m:Ikss:busmv:C", "kA, Initial Short-Circuit Current"
      m_Ikss_busmv_N = "m:Ikss:busmv:N", "kA, Initial Short-Circuit Current"
      m_Ikss_buslv_A = "m:Ikss:buslv:A", "kA, Initial Short-Circuit Current"
      m_Ikss_buslv_B = "m:Ikss:buslv:B", "kA, Initial Short-Circuit Current"
      m_Ikss_buslv_C = "m:Ikss:buslv:C", "kA, Initial Short-Circuit Current"
      m_Ikss_buslv_N = "m:Ikss:buslv:N", "kA, Initial Short-Circuit Current"
      m_Ikss_bushvn = "m:Ikss:bushvn", "kA, Initial Short-Circuit Current"
      m_Ikss_busmvn = "m:Ikss:busmvn", "kA, Initial Short-Circuit Current"
      m_Ikss_buslvn = "m:Ikss:buslvn", "kA, Initial Short-Circuit Current"
      m_ikss_bushv_A = "m:ikss:bushv:A", "p.u., Initial Short-Circuit Current, Magnitude"
      m_ikss_bushv_B = "m:ikss:bushv:B", "p.u., Initial Short-Circuit Current, Magnitude"
      m_ikss_bushv_C = "m:ikss:bushv:C", "p.u., Initial Short-Circuit Current, Magnitude"
      m_ikss_bushv_N = "m:ikss:bushv:N", "p.u., Initial Short-Circuit Current, Magnitude"
      m_ikss_busmv_A = "m:ikss:busmv:A", "p.u., Initial Short-Circuit Current, Magnitude"
      m_ikss_busmv_B = "m:ikss:busmv:B", "p.u., Initial Short-Circuit Current, Magnitude"
      m_ikss_busmv_C = "m:ikss:busmv:C", "p.u., Initial Short-Circuit Current, Magnitude"
      m_ikss_busmv_N = "m:ikss:busmv:N", "p.u., Initial Short-Circuit Current, Magnitude"
      m_ikss_buslv_A = "m:ikss:buslv:A", "p.u., Initial Short-Circuit Current, Magnitude"
      m_ikss_buslv_B = "m:ikss:buslv:B", "p.u., Initial Short-Circuit Current, Magnitude"
      m_ikss_buslv_C = "m:ikss:buslv:C", "p.u., Initial Short-Circuit Current, Magnitude"
      m_ikss_buslv_N = "m:ikss:buslv:N", "p.u., Initial Short-Circuit Current, Magnitude"
      m_ikss_bushvn = "m:ikss:bushvn", "p.u., Initial Short-Circuit Current, Magnitude"
      m_ikss_busmvn = "m:ikss:busmvn", "p.u., Initial Short-Circuit Current, Magnitude"
      m_ikss_buslvn = "m:ikss:buslvn", "p.u., Initial Short-Circuit Current, Magnitude"
      m_maxIkss_bushvn = "m:maxIkss:bushvn", "kA, Max. Initial Short-Circuit Current"
      m_maxIkss_busmvn = "m:maxIkss:busmvn", "kA, Max. Initial Short-Circuit Current"
      m_maxIkss_buslvn = "m:maxIkss:buslvn", "kA, Max. Initial Short-Circuit Current"
      m_Skss_bushv_A = "m:Skss:bushv:A", "MVA, Initial Short-Circuit Power"
      m_Skss_bushv_B = "m:Skss:bushv:B", "MVA, Initial Short-Circuit Power"
      m_Skss_bushv_C = "m:Skss:bushv:C", "MVA, Initial Short-Circuit Power"
      m_Skss_bushv_N = "m:Skss:bushv:N", "MVA, Initial Short-Circuit Power"
      m_Skss_busmv_A = "m:Skss:busmv:A", "MVA, Initial Short-Circuit Power"
      m_Skss_busmv_B = "m:Skss:busmv:B", "MVA, Initial Short-Circuit Power"
      m_Skss_busmv_C = "m:Skss:busmv:C", "MVA, Initial Short-Circuit Power"
      m_Skss_busmv_N = "m:Skss:busmv:N", "MVA, Initial Short-Circuit Power"
      m_Skss_buslv_A = "m:Skss:buslv:A", "MVA, Initial Short-Circuit Power"
      m_Skss_buslv_B = "m:Skss:buslv:B", "MVA, Initial Short-Circuit Power"
      m_Skss_buslv_C = "m:Skss:buslv:C", "MVA, Initial Short-Circuit Power"
      m_Skss_buslv_N = "m:Skss:buslv:N", "MVA, Initial Short-Circuit Power"
      m_Skss_bushvn = "m:Skss:bushvn", "MVA, Initial Short-Circuit Power"
      m_Skss_busmvn = "m:Skss:busmvn", "MVA, Initial Short-Circuit Power"
      m_Skss_buslvn = "m:Skss:buslvn", "MVA, Initial Short-Circuit Power"
      m_Skssx3_bushv_A = "m:Skssx3:bushv:A", "MVA, 3*Initial Short-Circuit Power"
      m_Skssx3_bushv_B = "m:Skssx3:bushv:B", "MVA, 3*Initial Short-Circuit Power"
      m_Skssx3_bushv_C = "m:Skssx3:bushv:C", "MVA, 3*Initial Short-Circuit Power"
      m_Skssx3_bushv_N = "m:Skssx3:bushv:N", "MVA, 3*Initial Short-Circuit Power"
      m_Skssx3_busmv_A = "m:Skssx3:busmv:A", "MVA, 3*Initial Short-Circuit Power"
      m_Skssx3_busmv_B = "m:Skssx3:busmv:B", "MVA, 3*Initial Short-Circuit Power"
      m_Skssx3_busmv_C = "m:Skssx3:busmv:C", "MVA, 3*Initial Short-Circuit Power"
      m_Skssx3_busmv_N = "m:Skssx3:busmv:N", "MVA, 3*Initial Short-Circuit Power"
      m_Skssx3_buslv_A = "m:Skssx3:buslv:A", "MVA, 3*Initial Short-Circuit Power"
      m_Skssx3_buslv_B = "m:Skssx3:buslv:B", "MVA, 3*Initial Short-Circuit Power"
      m_Skssx3_buslv_C = "m:Skssx3:buslv:C", "MVA, 3*Initial Short-Circuit Power"
      m_Skssx3_buslv_N = "m:Skssx3:buslv:N", "MVA, 3*Initial Short-Circuit Power"
      m_Skssx3_bushvn = "m:Skssx3:bushvn", "MVA, 3*Initial Short-Circuit Power"
      m_Skssx3_busmvn = "m:Skssx3:busmvn", "MVA, 3*Initial Short-Circuit Power"
      m_Skssx3_buslvn = "m:Skssx3:buslvn", "MVA, 3*Initial Short-Circuit Power"
      m_ip_bushv_A = "m:ip:bushv:A", "kA, Peak Short-Circuit Current"
      m_ip_bushv_B = "m:ip:bushv:B", "kA, Peak Short-Circuit Current"
      m_ip_bushv_C = "m:ip:bushv:C", "kA, Peak Short-Circuit Current"
      m_ip_bushv_N = "m:ip:bushv:N", "kA, Peak Short-Circuit Current"
      m_ip_busmv_A = "m:ip:busmv:A", "kA, Peak Short-Circuit Current"
      m_ip_busmv_B = "m:ip:busmv:B", "kA, Peak Short-Circuit Current"
      m_ip_busmv_C = "m:ip:busmv:C", "kA, Peak Short-Circuit Current"
      m_ip_busmv_N = "m:ip:busmv:N", "kA, Peak Short-Circuit Current"
      m_ip_buslv_A = "m:ip:buslv:A", "kA, Peak Short-Circuit Current"
      m_ip_buslv_B = "m:ip:buslv:B", "kA, Peak Short-Circuit Current"
      m_ip_buslv_C = "m:ip:buslv:C", "kA, Peak Short-Circuit Current"
      m_ip_buslv_N = "m:ip:buslv:N", "kA, Peak Short-Circuit Current"
      m_ip_bushvn = "m:ip:bushvn", "kA, Peak Short-Circuit Current"
      m_ip_busmvn = "m:ip:busmvn", "kA, Peak Short-Circuit Current"
      m_ip_buslvn = "m:ip:buslvn", "kA, Peak Short-Circuit Current"
      m_maxip_bushvn = "m:maxip:bushvn", "kA, Max. Peak Short-Circuit Current"
      m_maxip_busmvn = "m:maxip:busmvn", "kA, Max. Peak Short-Circuit Current"
      m_maxip_buslvn = "m:maxip:buslvn", "kA, Max. Peak Short-Circuit Current"
      m_maxIth_bushvn = "m:maxIth:bushvn", "kA, Max. Thermal equivalent Short-Circuit Current"
      m_maxIth_busmvn = "m:maxIth:busmvn", "kA, Max. Thermal equivalent Short-Circuit Current"
      m_maxIth_buslvn = "m:maxIth:buslvn", "kA, Max. Thermal equivalent Short-Circuit Current"
      m_Z_bushv_A = "m:Z:bushv:A", "Ohm, Fault Impedance, Magnitude"
      m_Z_bushv_B = "m:Z:bushv:B", "Ohm, Fault Impedance, Magnitude"
      m_Z_bushv_C = "m:Z:bushv:C", "Ohm, Fault Impedance, Magnitude"
      m_Z_bushv_N = "m:Z:bushv:N", "Ohm, Fault Impedance, Magnitude"
      m_Z_busmv_A = "m:Z:busmv:A", "Ohm, Fault Impedance, Magnitude"
      m_Z_busmv_B = "m:Z:busmv:B", "Ohm, Fault Impedance, Magnitude"
      m_Z_busmv_C = "m:Z:busmv:C", "Ohm, Fault Impedance, Magnitude"
      m_Z_busmv_N = "m:Z:busmv:N", "Ohm, Fault Impedance, Magnitude"
      m_Z_buslv_A = "m:Z:buslv:A", "Ohm, Fault Impedance, Magnitude"
      m_Z_buslv_B = "m:Z:buslv:B", "Ohm, Fault Impedance, Magnitude"
      m_Z_buslv_C = "m:Z:buslv:C", "Ohm, Fault Impedance, Magnitude"
      m_Z_buslv_N = "m:Z:buslv:N", "Ohm, Fault Impedance, Magnitude"
      m_Z_bushvn = "m:Z:bushvn", "Ohm, Fault Impedance, Magnitude"
      m_Z_busmvn = "m:Z:busmvn", "Ohm, Fault Impedance, Magnitude"
      m_Z_buslvn = "m:Z:buslvn", "Ohm, Fault Impedance, Magnitude"
      m_phiz_bushv_A = "m:phiz:bushv:A", "deg, Fault Impedance, Angle"
      m_phiz_bushv_B = "m:phiz:bushv:B", "deg, Fault Impedance, Angle"
      m_phiz_bushv_C = "m:phiz:bushv:C", "deg, Fault Impedance, Angle"
      m_phiz_bushv_N = "m:phiz:bushv:N", "deg, Fault Impedance, Angle"
      m_phiz_busmv_A = "m:phiz:busmv:A", "deg, Fault Impedance, Angle"
      m_phiz_busmv_B = "m:phiz:busmv:B", "deg, Fault Impedance, Angle"
      m_phiz_busmv_C = "m:phiz:busmv:C", "deg, Fault Impedance, Angle"
      m_phiz_busmv_N = "m:phiz:busmv:N", "deg, Fault Impedance, Angle"
      m_phiz_buslv_A = "m:phiz:buslv:A", "deg, Fault Impedance, Angle"
      m_phiz_buslv_B = "m:phiz:buslv:B", "deg, Fault Impedance, Angle"
      m_phiz_buslv_C = "m:phiz:buslv:C", "deg, Fault Impedance, Angle"
      m_phiz_buslv_N = "m:phiz:buslv:N", "deg, Fault Impedance, Angle"
      m_phiz_bushvn = "m:phiz:bushvn", "deg, Fault Impedance, Angle"
      m_phiz_busmvn = "m:phiz:busmvn", "deg, Fault Impedance, Angle"
      m_phiz_buslvn = "m:phiz:buslvn", "deg, Fault Impedance, Angle"
      m_Zl_bushv_A = "m:Zl:bushv:A", "Ohm, Line-Line Fault Impedance, Magnitude"
      m_Zl_bushv_B = "m:Zl:bushv:B", "Ohm, Line-Line Fault Impedance, Magnitude"
      m_Zl_bushv_C = "m:Zl:bushv:C", "Ohm, Line-Line Fault Impedance, Magnitude"
      m_Zl_bushv_N = "m:Zl:bushv:N", "Ohm, Line-Line Fault Impedance, Magnitude"
      m_Zl_busmv_A = "m:Zl:busmv:A", "Ohm, Line-Line Fault Impedance, Magnitude"
      m_Zl_busmv_B = "m:Zl:busmv:B", "Ohm, Line-Line Fault Impedance, Magnitude"
      m_Zl_busmv_C = "m:Zl:busmv:C", "Ohm, Line-Line Fault Impedance, Magnitude"
      m_Zl_busmv_N = "m:Zl:busmv:N", "Ohm, Line-Line Fault Impedance, Magnitude"
      m_Zl_buslv_A = "m:Zl:buslv:A", "Ohm, Line-Line Fault Impedance, Magnitude"
      m_Zl_buslv_B = "m:Zl:buslv:B", "Ohm, Line-Line Fault Impedance, Magnitude"
      m_Zl_buslv_C = "m:Zl:buslv:C", "Ohm, Line-Line Fault Impedance, Magnitude"
      m_Zl_buslv_N = "m:Zl:buslv:N", "Ohm, Line-Line Fault Impedance, Magnitude"
      m_Zl_bushvn = "m:Zl:bushvn", "Ohm, Line-Line Fault Impedance, Magnitude"
      m_Zl_busmvn = "m:Zl:busmvn", "Ohm, Line-Line Fault Impedance, Magnitude"
      m_Zl_buslvn = "m:Zl:buslvn", "Ohm, Line-Line Fault Impedance, Magnitude"
      m_phizl_bushv_A = "m:phizl:bushv:A", "deg, Line-Line Fault Impedance, Angle"
      m_phizl_bushv_B = "m:phizl:bushv:B", "deg, Line-Line Fault Impedance, Angle"
      m_phizl_bushv_C = "m:phizl:bushv:C", "deg, Line-Line Fault Impedance, Angle"
      m_phizl_bushv_N = "m:phizl:bushv:N", "deg, Line-Line Fault Impedance, Angle"
      m_phizl_busmv_A = "m:phizl:busmv:A", "deg, Line-Line Fault Impedance, Angle"
      m_phizl_busmv_B = "m:phizl:busmv:B", "deg, Line-Line Fault Impedance, Angle"
      m_phizl_busmv_C = "m:phizl:busmv:C", "deg, Line-Line Fault Impedance, Angle"
      m_phizl_busmv_N = "m:phizl:busmv:N", "deg, Line-Line Fault Impedance, Angle"
      m_phizl_buslv_A = "m:phizl:buslv:A", "deg, Line-Line Fault Impedance, Angle"
      m_phizl_buslv_B = "m:phizl:buslv:B", "deg, Line-Line Fault Impedance, Angle"
      m_phizl_buslv_C = "m:phizl:buslv:C", "deg, Line-Line Fault Impedance, Angle"
      m_phizl_buslv_N = "m:phizl:buslv:N", "deg, Line-Line Fault Impedance, Angle"
      m_phizl_bushvn = "m:phizl:bushvn", "deg, Line-Line Fault Impedance, Angle"
      m_phizl_busmvn = "m:phizl:busmvn", "deg, Line-Line Fault Impedance, Angle"
      m_phizl_buslvn = "m:phizl:buslvn", "deg, Line-Line Fault Impedance, Angle"
      m_Z0_bushv = "m:Z0:bushv", "Ohm, Zero-Sequence Fault Impedance, Magnitude"
      m_Z0_busmv = "m:Z0:busmv", "Ohm, Zero-Sequence Fault Impedance, Magnitude"
      m_Z0_buslv = "m:Z0:buslv", "Ohm, Zero-Sequence Fault Impedance, Magnitude"
      m_Z0_bushvn = "m:Z0:bushvn", "Ohm, Zero-Sequence Fault Impedance, Magnitude"
      m_Z0_busmvn = "m:Z0:busmvn", "Ohm, Zero-Sequence Fault Impedance, Magnitude"
      m_Z0_buslvn = "m:Z0:buslvn", "Ohm, Zero-Sequence Fault Impedance, Magnitude"
      m_phiz0_bushv = "m:phiz0:bushv", "deg, Zero-Sequence Fault Impedance, Angle"
      m_phiz0_busmv = "m:phiz0:busmv", "deg, Zero-Sequence Fault Impedance, Angle"
      m_phiz0_buslv = "m:phiz0:buslv", "deg, Zero-Sequence Fault Impedance, Angle"
      m_phiz0_bushvn = "m:phiz0:bushvn", "deg, Zero-Sequence Fault Impedance, Angle"
      m_phiz0_busmvn = "m:phiz0:busmvn", "deg, Zero-Sequence Fault Impedance, Angle"
      m_phiz0_buslvn = "m:phiz0:buslvn", "deg, Zero-Sequence Fault Impedance, Angle"
      m_z0Sbase_bushv = "m:z0Sbase:bushv", "p.u., Zero-Sequence Fault Impedance, Magnitude"
      m_z0Sbase_busmv = "m:z0Sbase:busmv", "p.u., Zero-Sequence Fault Impedance, Magnitude"
      m_z0Sbase_buslv = "m:z0Sbase:buslv", "p.u., Zero-Sequence Fault Impedance, Magnitude"
      m_z0Sbase_bushvn = "m:z0Sbase:bushvn", "p.u., Zero-Sequence Fault Impedance, Magnitude"
      m_z0Sbase_busmvn = "m:z0Sbase:busmvn", "p.u., Zero-Sequence Fault Impedance, Magnitude"
      m_z0Sbase_buslvn = "m:z0Sbase:buslvn", "p.u., Zero-Sequence Fault Impedance, Magnitude"
      m_cosphiu1i1_bushv = "m:cosphiu1i1:bushv", "Positive-Sequence cosphi"
      m_cosphiu1i1_busmv = "m:cosphiu1i1:busmv", "Positive-Sequence cosphi"
      m_cosphiu1i1_buslv = "m:cosphiu1i1:buslv", "Positive-Sequence cosphi"
      m_cosphiu1i1_bushvn = "m:cosphiu1i1:bushvn", "Positive-Sequence cosphi"
      m_cosphiu1i1_busmvn = "m:cosphiu1i1:busmvn", "Positive-Sequence cosphi"
      m_cosphiu1i1_buslvn = "m:cosphiu1i1:buslvn", "Positive-Sequence cosphi"
      m_I0x3watt_bushv = "m:I0x3watt:bushv", "kA, Earth-Fault Residual Current"
      m_I0x3watt_busmv = "m:I0x3watt:busmv", "kA, Earth-Fault Residual Current"
      m_I0x3watt_buslv = "m:I0x3watt:buslv", "kA, Earth-Fault Residual Current"
      m_I0x3watt_bushvn = "m:I0x3watt:bushvn", "kA, Earth-Fault Residual Current"
      m_I0x3watt_busmvn = "m:I0x3watt:busmvn", "kA, Earth-Fault Residual Current"
      m_I0x3watt_buslvn = "m:I0x3watt:buslvn", "kA, Earth-Fault Residual Current"
      m_I0x3var_bushv = "m:I0x3var:bushv", "kA, Capacitive Earth-Fault Current"
      m_I0x3var_busmv = "m:I0x3var:busmv", "kA, Capacitive Earth-Fault Current"
      m_I0x3var_buslv = "m:I0x3var:buslv", "kA, Capacitive Earth-Fault Current"
      m_I0x3var_bushvn = "m:I0x3var:bushvn", "kA, Capacitive Earth-Fault Current"
      m_I0x3var_busmvn = "m:I0x3var:busmvn", "kA, Capacitive Earth-Fault Current"
      m_I0x3var_buslvn = "m:I0x3var:buslvn", "kA, Capacitive Earth-Fault Current"
      m_Q0cap_bushv = "m:Q0cap:bushv", "Mvar, Q0 capacitive"
      m_Q0cap_busmv = "m:Q0cap:busmv", "Mvar, Q0 capacitive"
      m_Q0cap_buslv = "m:Q0cap:buslv", "Mvar, Q0 capacitive"
      m_Q0cap_bushvn = "m:Q0cap:bushvn", "Mvar, Q0 capacitive"
      m_Q0cap_busmvn = "m:Q0cap:busmvn", "Mvar, Q0 capacitive"
      m_Q0cap_buslvn = "m:Q0cap:buslvn", "Mvar, Q0 capacitive"
      m_Zfrel_bushv = "m:Zfrel:bushv", "Ohm, Relay Impedance, Magnitude"
      m_Zfrel_busmv = "m:Zfrel:busmv", "Ohm, Relay Impedance, Magnitude"
      m_Zfrel_buslv = "m:Zfrel:buslv", "Ohm, Relay Impedance, Magnitude"
      m_Zfrel_bushvn = "m:Zfrel:bushvn", "Ohm, Relay Impedance, Magnitude"
      m_Zfrel_busmvn = "m:Zfrel:busmvn", "Ohm, Relay Impedance, Magnitude"
      m_Zfrel_buslvn = "m:Zfrel:buslvn", "Ohm, Relay Impedance, Magnitude"
      m_phizfrel_bushv = "m:phizfrel:bushv", "deg, Relay Impedance, Angle"
      m_phizfrel_busmv = "m:phizfrel:busmv", "deg, Relay Impedance, Angle"
      m_phizfrel_buslv = "m:phizfrel:buslv", "deg, Relay Impedance, Angle"
      m_phizfrel_bushvn = "m:phizfrel:bushvn", "deg, Relay Impedance, Angle"
      m_phizfrel_busmvn = "m:phizfrel:busmvn", "deg, Relay Impedance, Angle"
      m_phizfrel_buslvn = "m:phizfrel:buslvn", "deg, Relay Impedance, Angle"
      m_Rfrel_bushv = "m:Rfrel:bushv", "Ohm, Relay Impedance, Real Part"
      m_Rfrel_busmv = "m:Rfrel:busmv", "Ohm, Relay Impedance, Real Part"
      m_Rfrel_buslv = "m:Rfrel:buslv", "Ohm, Relay Impedance, Real Part"
      m_Rfrel_bushvn = "m:Rfrel:bushvn", "Ohm, Relay Impedance, Real Part"
      m_Rfrel_busmvn = "m:Rfrel:busmvn", "Ohm, Relay Impedance, Real Part"
      m_Rfrel_buslvn = "m:Rfrel:buslvn", "Ohm, Relay Impedance, Real Part"
      m_Xfrel_bushv = "m:Xfrel:bushv", "Ohm, Relay Impedance, Imaginary Part"
      m_Xfrel_busmv = "m:Xfrel:busmv", "Ohm, Relay Impedance, Imaginary Part"
      m_Xfrel_buslv = "m:Xfrel:buslv", "Ohm, Relay Impedance, Imaginary Part"
      m_Xfrel_bushvn = "m:Xfrel:bushvn", "Ohm, Relay Impedance, Imaginary Part"
      m_Xfrel_busmvn = "m:Xfrel:busmvn", "Ohm, Relay Impedance, Imaginary Part"
      m_Xfrel_buslvn = "m:Xfrel:buslvn", "Ohm, Relay Impedance, Imaginary Part"
      m_Zs_bushvn = "m:Zs:bushvn", "Ohm, Source Impedance, Magnitude"
      m_Zs_busmvn = "m:Zs:busmvn", "Ohm, Source Impedance, Magnitude"
      m_Zs_buslvn = "m:Zs:buslvn", "Ohm, Source Impedance, Magnitude"
      m_phizs_bushvn = "m:phizs:bushvn", "deg, Source Impedance, Angle"
      m_phizs_busmvn = "m:phizs:busmvn", "deg, Source Impedance, Angle"
      m_phizs_buslvn = "m:phizs:buslvn", "deg, Source Impedance, Angle"
      m_Rs_bushvn = "m:Rs:bushvn", "Ohm, Source Impedance, Real Part"
      m_Rs_busmvn = "m:Rs:busmvn", "Ohm, Source Impedance, Real Part"
      m_Rs_buslvn = "m:Rs:buslvn", "Ohm, Source Impedance, Real Part"
      m_Xs_bushvn = "m:Xs:bushvn", "Ohm, Source Impedance, Imaginary Part"
      m_Xs_busmvn = "m:Xs:busmvn", "Ohm, Source Impedance, Imaginary Part"
      m_Xs_buslvn = "m:Xs:buslvn", "Ohm, Source Impedance, Imaginary Part"
      m_Zf_bushv = "m:Zf:bushv", "Ohm, Meas. Fault Impedance, Magnitude"
      m_Zf_busmv = "m:Zf:busmv", "Ohm, Meas. Fault Impedance, Magnitude"
      m_Zf_buslv = "m:Zf:buslv", "Ohm, Meas. Fault Impedance, Magnitude"
      m_Zf_bushvn = "m:Zf:bushvn", "Ohm, Meas. Fault Impedance, Magnitude"
      m_Zf_busmvn = "m:Zf:busmvn", "Ohm, Meas. Fault Impedance, Magnitude"
      m_Zf_buslvn = "m:Zf:buslvn", "Ohm, Meas. Fault Impedance, Magnitude"
      m_phizf_bushv = "m:phizf:bushv", "deg, Meas. Fault Impedance, Angle"
      m_phizf_busmv = "m:phizf:busmv", "deg, Meas. Fault Impedance, Angle"
      m_phizf_buslv = "m:phizf:buslv", "deg, Meas. Fault Impedance, Angle"
      m_phizf_bushvn = "m:phizf:bushvn", "deg, Meas. Fault Impedance, Angle"
      m_phizf_busmvn = "m:phizf:busmvn", "deg, Meas. Fault Impedance, Angle"
      m_phizf_buslvn = "m:phizf:buslvn", "deg, Meas. Fault Impedance, Angle"
      m_BrkIpload_bushvn = "m:BrkIpload:bushvn", "%, Max. Breaker Loading (peak)"
      m_BrkIpload_busmvn = "m:BrkIpload:busmvn", "%, Max. Breaker Loading (peak)"
      m_BrkIpload_buslvn = "m:BrkIpload:buslvn", "%, Max. Breaker Loading (peak)"
      n_Ikss_bushv_A = "n:Ikss:bushv:A", "kA, Initial Short-Circuit Current"
      n_Ikss_bushv_B = "n:Ikss:bushv:B", "kA, Initial Short-Circuit Current"
      n_Ikss_bushv_C = "n:Ikss:bushv:C", "kA, Initial Short-Circuit Current"
      n_Ikss_bushv_N = "n:Ikss:bushv:N", "kA, Initial Short-Circuit Current"
      n_Ikss_busmv_A = "n:Ikss:busmv:A", "kA, Initial Short-Circuit Current"
      n_Ikss_busmv_B = "n:Ikss:busmv:B", "kA, Initial Short-Circuit Current"
      n_Ikss_busmv_C = "n:Ikss:busmv:C", "kA, Initial Short-Circuit Current"
      n_Ikss_busmv_N = "n:Ikss:busmv:N", "kA, Initial Short-Circuit Current"
      n_Ikss_buslv_A = "n:Ikss:buslv:A", "kA, Initial Short-Circuit Current"
      n_Ikss_buslv_B = "n:Ikss:buslv:B", "kA, Initial Short-Circuit Current"
      n_Ikss_buslv_C = "n:Ikss:buslv:C", "kA, Initial Short-Circuit Current"
      n_Ikss_buslv_N = "n:Ikss:buslv:N", "kA, Initial Short-Circuit Current"
      n_Ikss_bushvn = "n:Ikss:bushvn", "kA, Initial Short-Circuit Current"
      n_Ikss_busmvn = "n:Ikss:busmvn", "kA, Initial Short-Circuit Current"
      n_Ikss_buslvn = "n:Ikss:buslvn", "kA, Initial Short-Circuit Current"
      n_I_bushv_A = "n:I:bushv:A", "kA, Short-Circuit Current"
      n_I_bushv_B = "n:I:bushv:B", "kA, Short-Circuit Current"
      n_I_bushv_C = "n:I:bushv:C", "kA, Short-Circuit Current"
      n_I_bushv_N = "n:I:bushv:N", "kA, Short-Circuit Current"
      n_I_busmv_A = "n:I:busmv:A", "kA, Short-Circuit Current"
      n_I_busmv_B = "n:I:busmv:B", "kA, Short-Circuit Current"
      n_I_busmv_C = "n:I:busmv:C", "kA, Short-Circuit Current"
      n_I_busmv_N = "n:I:busmv:N", "kA, Short-Circuit Current"
      n_I_buslv_A = "n:I:buslv:A", "kA, Short-Circuit Current"
      n_I_buslv_B = "n:I:buslv:B", "kA, Short-Circuit Current"
      n_I_buslv_C = "n:I:buslv:C", "kA, Short-Circuit Current"
      n_I_buslv_N = "n:I:buslv:N", "kA, Short-Circuit Current"
      n_I_bushvn = "n:I:bushvn", "kA, Short-Circuit Current"
      n_I_busmvn = "n:I:busmvn", "kA, Short-Circuit Current"
      n_I_buslvn = "n:I:buslvn", "kA, Short-Circuit Current"
      n_phii_bushv_A = "n:phii:bushv:A", "deg, Phase Current, Angle"
      n_phii_bushv_B = "n:phii:bushv:B", "deg, Phase Current, Angle"
      n_phii_bushv_C = "n:phii:bushv:C", "deg, Phase Current, Angle"
      n_phii_bushv_N = "n:phii:bushv:N", "deg, Phase Current, Angle"
      n_phii_busmv_A = "n:phii:busmv:A", "deg, Phase Current, Angle"
      n_phii_busmv_B = "n:phii:busmv:B", "deg, Phase Current, Angle"
      n_phii_busmv_C = "n:phii:busmv:C", "deg, Phase Current, Angle"
      n_phii_busmv_N = "n:phii:busmv:N", "deg, Phase Current, Angle"
      n_phii_buslv_A = "n:phii:buslv:A", "deg, Phase Current, Angle"
      n_phii_buslv_B = "n:phii:buslv:B", "deg, Phase Current, Angle"
      n_phii_buslv_C = "n:phii:buslv:C", "deg, Phase Current, Angle"
      n_phii_buslv_N = "n:phii:buslv:N", "deg, Phase Current, Angle"
      n_phii_bushvn = "n:phii:bushvn", "deg, Phase Current, Angle"
      n_phii_busmvn = "n:phii:busmvn", "deg, Phase Current, Angle"
      n_phii_buslvn = "n:phii:buslvn", "deg, Phase Current, Angle"
      n_Skss_bushv_A = "n:Skss:bushv:A", "MVA, Initial Short-Circuit Power"
      n_Skss_bushv_B = "n:Skss:bushv:B", "MVA, Initial Short-Circuit Power"
      n_Skss_bushv_C = "n:Skss:bushv:C", "MVA, Initial Short-Circuit Power"
      n_Skss_bushv_N = "n:Skss:bushv:N", "MVA, Initial Short-Circuit Power"
      n_Skss_busmv_A = "n:Skss:busmv:A", "MVA, Initial Short-Circuit Power"
      n_Skss_busmv_B = "n:Skss:busmv:B", "MVA, Initial Short-Circuit Power"
      n_Skss_busmv_C = "n:Skss:busmv:C", "MVA, Initial Short-Circuit Power"
      n_Skss_busmv_N = "n:Skss:busmv:N", "MVA, Initial Short-Circuit Power"
      n_Skss_buslv_A = "n:Skss:buslv:A", "MVA, Initial Short-Circuit Power"
      n_Skss_buslv_B = "n:Skss:buslv:B", "MVA, Initial Short-Circuit Power"
      n_Skss_buslv_C = "n:Skss:buslv:C", "MVA, Initial Short-Circuit Power"
      n_Skss_buslv_N = "n:Skss:buslv:N", "MVA, Initial Short-Circuit Power"
      n_Skss_bushvn = "n:Skss:bushvn", "MVA, Initial Short-Circuit Power"
      n_Skss_busmvn = "n:Skss:busmvn", "MVA, Initial Short-Circuit Power"
      n_Skss_buslvn = "n:Skss:buslvn", "MVA, Initial Short-Circuit Power"
      n_Skssx3_bushv_A = "n:Skssx3:bushv:A", "MVA, 3*Initial Short-Circuit Power"
      n_Skssx3_bushv_B = "n:Skssx3:bushv:B", "MVA, 3*Initial Short-Circuit Power"
      n_Skssx3_bushv_C = "n:Skssx3:bushv:C", "MVA, 3*Initial Short-Circuit Power"
      n_Skssx3_bushv_N = "n:Skssx3:bushv:N", "MVA, 3*Initial Short-Circuit Power"
      n_Skssx3_busmv_A = "n:Skssx3:busmv:A", "MVA, 3*Initial Short-Circuit Power"
      n_Skssx3_busmv_B = "n:Skssx3:busmv:B", "MVA, 3*Initial Short-Circuit Power"
      n_Skssx3_busmv_C = "n:Skssx3:busmv:C", "MVA, 3*Initial Short-Circuit Power"
      n_Skssx3_busmv_N = "n:Skssx3:busmv:N", "MVA, 3*Initial Short-Circuit Power"
      n_Skssx3_buslv_A = "n:Skssx3:buslv:A", "MVA, 3*Initial Short-Circuit Power"
      n_Skssx3_buslv_B = "n:Skssx3:buslv:B", "MVA, 3*Initial Short-Circuit Power"
      n_Skssx3_buslv_C = "n:Skssx3:buslv:C", "MVA, 3*Initial Short-Circuit Power"
      n_Skssx3_buslv_N = "n:Skssx3:buslv:N", "MVA, 3*Initial Short-Circuit Power"
      n_Skssx3_bushvn = "n:Skssx3:bushvn", "MVA, 3*Initial Short-Circuit Power"
      n_Skssx3_busmvn = "n:Skssx3:busmvn", "MVA, 3*Initial Short-Circuit Power"
      n_Skssx3_buslvn = "n:Skssx3:buslvn", "MVA, 3*Initial Short-Circuit Power"
      n_ip_bushv_A = "n:ip:bushv:A", "kA, Peak Short-Circuit Current"
      n_ip_bushv_B = "n:ip:bushv:B", "kA, Peak Short-Circuit Current"
      n_ip_bushv_C = "n:ip:bushv:C", "kA, Peak Short-Circuit Current"
      n_ip_bushv_N = "n:ip:bushv:N", "kA, Peak Short-Circuit Current"
      n_ip_busmv_A = "n:ip:busmv:A", "kA, Peak Short-Circuit Current"
      n_ip_busmv_B = "n:ip:busmv:B", "kA, Peak Short-Circuit Current"
      n_ip_busmv_C = "n:ip:busmv:C", "kA, Peak Short-Circuit Current"
      n_ip_busmv_N = "n:ip:busmv:N", "kA, Peak Short-Circuit Current"
      n_ip_buslv_A = "n:ip:buslv:A", "kA, Peak Short-Circuit Current"
      n_ip_buslv_B = "n:ip:buslv:B", "kA, Peak Short-Circuit Current"
      n_ip_buslv_C = "n:ip:buslv:C", "kA, Peak Short-Circuit Current"
      n_ip_buslv_N = "n:ip:buslv:N", "kA, Peak Short-Circuit Current"
      n_ip_bushvn = "n:ip:bushvn", "kA, Peak Short-Circuit Current"
      n_ip_busmvn = "n:ip:busmvn", "kA, Peak Short-Circuit Current"
      n_ip_buslvn = "n:ip:buslvn", "kA, Peak Short-Circuit Current"
      n_Ib_bushv_A = "n:Ib:bushv:A", "kA, Short-Circuit Breaking Current"
      n_Ib_bushv_B = "n:Ib:bushv:B", "kA, Short-Circuit Breaking Current"
      n_Ib_bushv_C = "n:Ib:bushv:C", "kA, Short-Circuit Breaking Current"
      n_Ib_bushv_N = "n:Ib:bushv:N", "kA, Short-Circuit Breaking Current"
      n_Ib_busmv_A = "n:Ib:busmv:A", "kA, Short-Circuit Breaking Current"
      n_Ib_busmv_B = "n:Ib:busmv:B", "kA, Short-Circuit Breaking Current"
      n_Ib_busmv_C = "n:Ib:busmv:C", "kA, Short-Circuit Breaking Current"
      n_Ib_busmv_N = "n:Ib:busmv:N", "kA, Short-Circuit Breaking Current"
      n_Ib_buslv_A = "n:Ib:buslv:A", "kA, Short-Circuit Breaking Current"
      n_Ib_buslv_B = "n:Ib:buslv:B", "kA, Short-Circuit Breaking Current"
      n_Ib_buslv_C = "n:Ib:buslv:C", "kA, Short-Circuit Breaking Current"
      n_Ib_buslv_N = "n:Ib:buslv:N", "kA, Short-Circuit Breaking Current"
      n_Ib_bushvn = "n:Ib:bushvn", "kA, Short-Circuit Breaking Current"
      n_Ib_busmvn = "n:Ib:busmvn", "kA, Short-Circuit Breaking Current"
      n_Ib_buslvn = "n:Ib:buslvn", "kA, Short-Circuit Breaking Current"
      n_Sb_bushv_A = "n:Sb:bushv:A", "MVA, Short-Circuit Breaking Power"
      n_Sb_bushv_B = "n:Sb:bushv:B", "MVA, Short-Circuit Breaking Power"
      n_Sb_bushv_C = "n:Sb:bushv:C", "MVA, Short-Circuit Breaking Power"
      n_Sb_bushv_N = "n:Sb:bushv:N", "MVA, Short-Circuit Breaking Power"
      n_Sb_busmv_A = "n:Sb:busmv:A", "MVA, Short-Circuit Breaking Power"
      n_Sb_busmv_B = "n:Sb:busmv:B", "MVA, Short-Circuit Breaking Power"
      n_Sb_busmv_C = "n:Sb:busmv:C", "MVA, Short-Circuit Breaking Power"
      n_Sb_busmv_N = "n:Sb:busmv:N", "MVA, Short-Circuit Breaking Power"
      n_Sb_buslv_A = "n:Sb:buslv:A", "MVA, Short-Circuit Breaking Power"
      n_Sb_buslv_B = "n:Sb:buslv:B", "MVA, Short-Circuit Breaking Power"
      n_Sb_buslv_C = "n:Sb:buslv:C", "MVA, Short-Circuit Breaking Power"
      n_Sb_buslv_N = "n:Sb:buslv:N", "MVA, Short-Circuit Breaking Power"
      n_Sb_bushvn = "n:Sb:bushvn", "MVA, Short-Circuit Breaking Power"
      n_Sb_busmvn = "n:Sb:busmvn", "MVA, Short-Circuit Breaking Power"
      n_Sb_buslvn = "n:Sb:buslvn", "MVA, Short-Circuit Breaking Power"
      n_Ith_bushv_A = "n:Ith:bushv:A", "kA, Thermal equivalent Short-Circuit Current"
      n_Ith_bushv_B = "n:Ith:bushv:B", "kA, Thermal equivalent Short-Circuit Current"
      n_Ith_bushv_C = "n:Ith:bushv:C", "kA, Thermal equivalent Short-Circuit Current"
      n_Ith_bushv_N = "n:Ith:bushv:N", "kA, Thermal equivalent Short-Circuit Current"
      n_Ith_busmv_A = "n:Ith:busmv:A", "kA, Thermal equivalent Short-Circuit Current"
      n_Ith_busmv_B = "n:Ith:busmv:B", "kA, Thermal equivalent Short-Circuit Current"
      n_Ith_busmv_C = "n:Ith:busmv:C", "kA, Thermal equivalent Short-Circuit Current"
      n_Ith_busmv_N = "n:Ith:busmv:N", "kA, Thermal equivalent Short-Circuit Current"
      n_Ith_buslv_A = "n:Ith:buslv:A", "kA, Thermal equivalent Short-Circuit Current"
      n_Ith_buslv_B = "n:Ith:buslv:B", "kA, Thermal equivalent Short-Circuit Current"
      n_Ith_buslv_C = "n:Ith:buslv:C", "kA, Thermal equivalent Short-Circuit Current"
      n_Ith_buslv_N = "n:Ith:buslv:N", "kA, Thermal equivalent Short-Circuit Current"
      n_Ith_bushvn = "n:Ith:bushvn", "kA, Thermal equivalent Short-Circuit Current"
      n_Ith_busmvn = "n:Ith:busmvn", "kA, Thermal equivalent Short-Circuit Current"
      n_Ith_buslvn = "n:Ith:buslvn", "kA, Thermal equivalent Short-Circuit Current"
      n_Ithmax_bushv = "n:Ithmax:bushv", "kA, Max. Thermal equivalent Short-Circuit Current"
      n_Ithmax_busmv = "n:Ithmax:busmv", "kA, Max. Thermal equivalent Short-Circuit Current"
      n_Ithmax_buslv = "n:Ithmax:buslv", "kA, Max. Thermal equivalent Short-Circuit Current"
      n_Ithmax_bushvn = "n:Ithmax:bushvn", "kA, Max. Thermal equivalent Short-Circuit Current"
      n_Ithmax_busmvn = "n:Ithmax:busmvn", "kA, Max. Thermal equivalent Short-Circuit Current"
      n_Ithmax_buslvn = "n:Ithmax:buslvn", "kA, Max. Thermal equivalent Short-Circuit Current"
      n_Ik_bushv_A = "n:Ik:bushv:A", "kA, Steady-State Short-Circuit Current"
      n_Ik_bushv_B = "n:Ik:bushv:B", "kA, Steady-State Short-Circuit Current"
      n_Ik_bushv_C = "n:Ik:bushv:C", "kA, Steady-State Short-Circuit Current"
      n_Ik_bushv_N = "n:Ik:bushv:N", "kA, Steady-State Short-Circuit Current"
      n_Ik_busmv_A = "n:Ik:busmv:A", "kA, Steady-State Short-Circuit Current"
      n_Ik_busmv_B = "n:Ik:busmv:B", "kA, Steady-State Short-Circuit Current"
      n_Ik_busmv_C = "n:Ik:busmv:C", "kA, Steady-State Short-Circuit Current"
      n_Ik_busmv_N = "n:Ik:busmv:N", "kA, Steady-State Short-Circuit Current"
      n_Ik_buslv_A = "n:Ik:buslv:A", "kA, Steady-State Short-Circuit Current"
      n_Ik_buslv_B = "n:Ik:buslv:B", "kA, Steady-State Short-Circuit Current"
      n_Ik_buslv_C = "n:Ik:buslv:C", "kA, Steady-State Short-Circuit Current"
      n_Ik_buslv_N = "n:Ik:buslv:N", "kA, Steady-State Short-Circuit Current"
      n_Ik_bushvn = "n:Ik:bushvn", "kA, Steady-State Short-Circuit Current"
      n_Ik_busmvn = "n:Ik:busmvn", "kA, Steady-State Short-Circuit Current"
      n_Ik_buslvn = "n:Ik:buslvn", "kA, Steady-State Short-Circuit Current"
      n_R0_bushv = "n:R0:bushv", "Ohm, Zero-Sequence Impedance, Real Part"
      n_R0_busmv = "n:R0:busmv", "Ohm, Zero-Sequence Impedance, Real Part"
      n_R0_buslv = "n:R0:buslv", "Ohm, Zero-Sequence Impedance, Real Part"
      n_R0_bushvn = "n:R0:bushvn", "Ohm, Zero-Sequence Impedance, Real Part"
      n_R0_busmvn = "n:R0:busmvn", "Ohm, Zero-Sequence Impedance, Real Part"
      n_R0_buslvn = "n:R0:buslvn", "Ohm, Zero-Sequence Impedance, Real Part"
      n_X0_bushv = "n:X0:bushv", "Ohm, Zero-Sequence Impedance, Imaginary Part"
      n_X0_busmv = "n:X0:busmv", "Ohm, Zero-Sequence Impedance, Imaginary Part"
      n_X0_buslv = "n:X0:buslv", "Ohm, Zero-Sequence Impedance, Imaginary Part"
      n_X0_bushvn = "n:X0:bushvn", "Ohm, Zero-Sequence Impedance, Imaginary Part"
      n_X0_busmvn = "n:X0:busmvn", "Ohm, Zero-Sequence Impedance, Imaginary Part"
      n_X0_buslvn = "n:X0:buslvn", "Ohm, Zero-Sequence Impedance, Imaginary Part"
      n_Z0_bushv = "n:Z0:bushv", "Ohm, Zero-Sequence Impedance, Magnitude"
      n_Z0_busmv = "n:Z0:busmv", "Ohm, Zero-Sequence Impedance, Magnitude"
      n_Z0_buslv = "n:Z0:buslv", "Ohm, Zero-Sequence Impedance, Magnitude"
      n_Z0_bushvn = "n:Z0:bushvn", "Ohm, Zero-Sequence Impedance, Magnitude"
      n_Z0_busmvn = "n:Z0:busmvn", "Ohm, Zero-Sequence Impedance, Magnitude"
      n_Z0_buslvn = "n:Z0:buslvn", "Ohm, Zero-Sequence Impedance, Magnitude"
      n_phiz0_bushv = "n:phiz0:bushv", "deg, Zero-Sequence Impedance, Angle"
      n_phiz0_busmv = "n:phiz0:busmv", "deg, Zero-Sequence Impedance, Angle"
      n_phiz0_buslv = "n:phiz0:buslv", "deg, Zero-Sequence Impedance, Angle"
      n_phiz0_bushvn = "n:phiz0:bushvn", "deg, Zero-Sequence Impedance, Angle"
      n_phiz0_busmvn = "n:phiz0:busmvn", "deg, Zero-Sequence Impedance, Angle"
      n_phiz0_buslvn = "n:phiz0:buslvn", "deg, Zero-Sequence Impedance, Angle"
      n_r0Sbase_bushv = "n:r0Sbase:bushv", "p.u., Zero-Sequence Impedance, Real Part"
      n_r0Sbase_busmv = "n:r0Sbase:busmv", "p.u., Zero-Sequence Impedance, Real Part"
      n_r0Sbase_buslv = "n:r0Sbase:buslv", "p.u., Zero-Sequence Impedance, Real Part"
      n_r0Sbase_bushvn = "n:r0Sbase:bushvn", "p.u., Zero-Sequence Impedance, Real Part"
      n_r0Sbase_busmvn = "n:r0Sbase:busmvn", "p.u., Zero-Sequence Impedance, Real Part"
      n_r0Sbase_buslvn = "n:r0Sbase:buslvn", "p.u., Zero-Sequence Impedance, Real Part"
      n_x0Sbase_bushv = "n:x0Sbase:bushv", "p.u., Zero-Sequence Impedance, Imaginary Part"
      n_x0Sbase_busmv = "n:x0Sbase:busmv", "p.u., Zero-Sequence Impedance, Imaginary Part"
      n_x0Sbase_buslv = "n:x0Sbase:buslv", "p.u., Zero-Sequence Impedance, Imaginary Part"
      n_x0Sbase_bushvn = "n:x0Sbase:bushvn", "p.u., Zero-Sequence Impedance, Imaginary Part"
      n_x0Sbase_busmvn = "n:x0Sbase:busmvn", "p.u., Zero-Sequence Impedance, Imaginary Part"
      n_x0Sbase_buslvn = "n:x0Sbase:buslvn", "p.u., Zero-Sequence Impedance, Imaginary Part"
      n_z0Sbase_bushv = "n:z0Sbase:bushv", "p.u., Zero-Sequence Impedance, Magnitude"
      n_z0Sbase_busmv = "n:z0Sbase:busmv", "p.u., Zero-Sequence Impedance, Magnitude"
      n_z0Sbase_buslv = "n:z0Sbase:buslv", "p.u., Zero-Sequence Impedance, Magnitude"
      n_z0Sbase_bushvn = "n:z0Sbase:bushvn", "p.u., Zero-Sequence Impedance, Magnitude"
      n_z0Sbase_busmvn = "n:z0Sbase:busmvn", "p.u., Zero-Sequence Impedance, Magnitude"
      n_z0Sbase_buslvn = "n:z0Sbase:buslvn", "p.u., Zero-Sequence Impedance, Magnitude"
      n_R1_bushv = "n:R1:bushv", "Ohm, Positive-Sequence Impedance, Real Part"
      n_R1_busmv = "n:R1:busmv", "Ohm, Positive-Sequence Impedance, Real Part"
      n_R1_buslv = "n:R1:buslv", "Ohm, Positive-Sequence Impedance, Real Part"
      n_R1_bushvn = "n:R1:bushvn", "Ohm, Positive-Sequence Impedance, Real Part"
      n_R1_busmvn = "n:R1:busmvn", "Ohm, Positive-Sequence Impedance, Real Part"
      n_R1_buslvn = "n:R1:buslvn", "Ohm, Positive-Sequence Impedance, Real Part"
      n_X1_bushv = "n:X1:bushv", "Ohm, Positive-Sequence Impedance, Imaginary Part"
      n_X1_busmv = "n:X1:busmv", "Ohm, Positive-Sequence Impedance, Imaginary Part"
      n_X1_buslv = "n:X1:buslv", "Ohm, Positive-Sequence Impedance, Imaginary Part"
      n_X1_bushvn = "n:X1:bushvn", "Ohm, Positive-Sequence Impedance, Imaginary Part"
      n_X1_busmvn = "n:X1:busmvn", "Ohm, Positive-Sequence Impedance, Imaginary Part"
      n_X1_buslvn = "n:X1:buslvn", "Ohm, Positive-Sequence Impedance, Imaginary Part"
      n_Z1_bushv = "n:Z1:bushv", "Ohm, Positive-Sequence Impedance, Magnitude"
      n_Z1_busmv = "n:Z1:busmv", "Ohm, Positive-Sequence Impedance, Magnitude"
      n_Z1_buslv = "n:Z1:buslv", "Ohm, Positive-Sequence Impedance, Magnitude"
      n_Z1_bushvn = "n:Z1:bushvn", "Ohm, Positive-Sequence Impedance, Magnitude"
      n_Z1_busmvn = "n:Z1:busmvn", "Ohm, Positive-Sequence Impedance, Magnitude"
      n_Z1_buslvn = "n:Z1:buslvn", "Ohm, Positive-Sequence Impedance, Magnitude"
      n_phiz1_bushv = "n:phiz1:bushv", "deg, Positive-Sequence Impedance, Angle"
      n_phiz1_busmv = "n:phiz1:busmv", "deg, Positive-Sequence Impedance, Angle"
      n_phiz1_buslv = "n:phiz1:buslv", "deg, Positive-Sequence Impedance, Angle"
      n_phiz1_bushvn = "n:phiz1:bushvn", "deg, Positive-Sequence Impedance, Angle"
      n_phiz1_busmvn = "n:phiz1:busmvn", "deg, Positive-Sequence Impedance, Angle"
      n_phiz1_buslvn = "n:phiz1:buslvn", "deg, Positive-Sequence Impedance, Angle"
      n_r1Sbase_bushv = "n:r1Sbase:bushv", "p.u., Positive-Sequence Impedance, Real Part"
      n_r1Sbase_busmv = "n:r1Sbase:busmv", "p.u., Positive-Sequence Impedance, Real Part"
      n_r1Sbase_buslv = "n:r1Sbase:buslv", "p.u., Positive-Sequence Impedance, Real Part"
      n_r1Sbase_bushvn = "n:r1Sbase:bushvn", "p.u., Positive-Sequence Impedance, Real Part"
      n_r1Sbase_busmvn = "n:r1Sbase:busmvn", "p.u., Positive-Sequence Impedance, Real Part"
      n_r1Sbase_buslvn = "n:r1Sbase:buslvn", "p.u., Positive-Sequence Impedance, Real Part"
      n_x1Sbase_bushv = "n:x1Sbase:bushv", "p.u., Positive-Sequence Impedance, Imaginary Part"
      n_x1Sbase_busmv = "n:x1Sbase:busmv", "p.u., Positive-Sequence Impedance, Imaginary Part"
      n_x1Sbase_buslv = "n:x1Sbase:buslv", "p.u., Positive-Sequence Impedance, Imaginary Part"
      n_x1Sbase_bushvn = "n:x1Sbase:bushvn", "p.u., Positive-Sequence Impedance, Imaginary Part"
      n_x1Sbase_busmvn = "n:x1Sbase:busmvn", "p.u., Positive-Sequence Impedance, Imaginary Part"
      n_x1Sbase_buslvn = "n:x1Sbase:buslvn", "p.u., Positive-Sequence Impedance, Imaginary Part"
      n_z1Sbase_bushv = "n:z1Sbase:bushv", "p.u., Positive-Sequence Impedance, Magnitude"
      n_z1Sbase_busmv = "n:z1Sbase:busmv", "p.u., Positive-Sequence Impedance, Magnitude"
      n_z1Sbase_buslv = "n:z1Sbase:buslv", "p.u., Positive-Sequence Impedance, Magnitude"
      n_z1Sbase_bushvn = "n:z1Sbase:bushvn", "p.u., Positive-Sequence Impedance, Magnitude"
      n_z1Sbase_busmvn = "n:z1Sbase:busmvn", "p.u., Positive-Sequence Impedance, Magnitude"
      n_z1Sbase_buslvn = "n:z1Sbase:buslvn", "p.u., Positive-Sequence Impedance, Magnitude"
      n_R2_bushv = "n:R2:bushv", "Ohm, Negative-Sequence Impedance, Real Part"
      n_R2_busmv = "n:R2:busmv", "Ohm, Negative-Sequence Impedance, Real Part"
      n_R2_buslv = "n:R2:buslv", "Ohm, Negative-Sequence Impedance, Real Part"
      n_R2_bushvn = "n:R2:bushvn", "Ohm, Negative-Sequence Impedance, Real Part"
      n_R2_busmvn = "n:R2:busmvn", "Ohm, Negative-Sequence Impedance, Real Part"
      n_R2_buslvn = "n:R2:buslvn", "Ohm, Negative-Sequence Impedance, Real Part"
      n_X2_bushv = "n:X2:bushv", "Ohm, Negative-Sequence Impedance, Imaginary Part"
      n_X2_busmv = "n:X2:busmv", "Ohm, Negative-Sequence Impedance, Imaginary Part"
      n_X2_buslv = "n:X2:buslv", "Ohm, Negative-Sequence Impedance, Imaginary Part"
      n_X2_bushvn = "n:X2:bushvn", "Ohm, Negative-Sequence Impedance, Imaginary Part"
      n_X2_busmvn = "n:X2:busmvn", "Ohm, Negative-Sequence Impedance, Imaginary Part"
      n_X2_buslvn = "n:X2:buslvn", "Ohm, Negative-Sequence Impedance, Imaginary Part"
      n_Z2_bushv = "n:Z2:bushv", "Ohm, Negative-Sequence Impedance, Magnitude"
      n_Z2_busmv = "n:Z2:busmv", "Ohm, Negative-Sequence Impedance, Magnitude"
      n_Z2_buslv = "n:Z2:buslv", "Ohm, Negative-Sequence Impedance, Magnitude"
      n_Z2_bushvn = "n:Z2:bushvn", "Ohm, Negative-Sequence Impedance, Magnitude"
      n_Z2_busmvn = "n:Z2:busmvn", "Ohm, Negative-Sequence Impedance, Magnitude"
      n_Z2_buslvn = "n:Z2:buslvn", "Ohm, Negative-Sequence Impedance, Magnitude"
      n_phiz2_bushv = "n:phiz2:bushv", "deg, Negative-Sequence Impedance, Angle"
      n_phiz2_busmv = "n:phiz2:busmv", "deg, Negative-Sequence Impedance, Angle"
      n_phiz2_buslv = "n:phiz2:buslv", "deg, Negative-Sequence Impedance, Angle"
      n_phiz2_bushvn = "n:phiz2:bushvn", "deg, Negative-Sequence Impedance, Angle"
      n_phiz2_busmvn = "n:phiz2:busmvn", "deg, Negative-Sequence Impedance, Angle"
      n_phiz2_buslvn = "n:phiz2:buslvn", "deg, Negative-Sequence Impedance, Angle"
      n_r2Sbase_bushv = "n:r2Sbase:bushv", "p.u., Negative-Sequence Impedance, Real Part"
      n_r2Sbase_busmv = "n:r2Sbase:busmv", "p.u., Negative-Sequence Impedance, Real Part"
      n_r2Sbase_buslv = "n:r2Sbase:buslv", "p.u., Negative-Sequence Impedance, Real Part"
      n_r2Sbase_bushvn = "n:r2Sbase:bushvn", "p.u., Negative-Sequence Impedance, Real Part"
      n_r2Sbase_busmvn = "n:r2Sbase:busmvn", "p.u., Negative-Sequence Impedance, Real Part"
      n_r2Sbase_buslvn = "n:r2Sbase:buslvn", "p.u., Negative-Sequence Impedance, Real Part"
      n_x2Sbase_bushv = "n:x2Sbase:bushv", "p.u., Negative-Sequence Impedance, Imaginary Part"
      n_x2Sbase_busmv = "n:x2Sbase:busmv", "p.u., Negative-Sequence Impedance, Imaginary Part"
      n_x2Sbase_buslv = "n:x2Sbase:buslv", "p.u., Negative-Sequence Impedance, Imaginary Part"
      n_x2Sbase_bushvn = "n:x2Sbase:bushvn", "p.u., Negative-Sequence Impedance, Imaginary Part"
      n_x2Sbase_busmvn = "n:x2Sbase:busmvn", "p.u., Negative-Sequence Impedance, Imaginary Part"
      n_x2Sbase_buslvn = "n:x2Sbase:buslvn", "p.u., Negative-Sequence Impedance, Imaginary Part"
      n_z2Sbase_bushv = "n:z2Sbase:bushv", "p.u., Negative-Sequence Impedance, Magnitude"
      n_z2Sbase_busmv = "n:z2Sbase:busmv", "p.u., Negative-Sequence Impedance, Magnitude"
      n_z2Sbase_buslv = "n:z2Sbase:buslv", "p.u., Negative-Sequence Impedance, Magnitude"
      n_z2Sbase_bushvn = "n:z2Sbase:bushvn", "p.u., Negative-Sequence Impedance, Magnitude"
      n_z2Sbase_busmvn = "n:z2Sbase:busmvn", "p.u., Negative-Sequence Impedance, Magnitude"
      n_z2Sbase_buslvn = "n:z2Sbase:buslvn", "p.u., Negative-Sequence Impedance, Magnitude"
      n_R_bushv_A = "n:R:bushv:A", "Ohm, Network Resistance, Line-Ground"
      n_R_bushv_B = "n:R:bushv:B", "Ohm, Network Resistance, Line-Ground"
      n_R_bushv_C = "n:R:bushv:C", "Ohm, Network Resistance, Line-Ground"
      n_R_bushv_N = "n:R:bushv:N", "Ohm, Network Resistance, Line-Ground"
      n_R_busmv_A = "n:R:busmv:A", "Ohm, Network Resistance, Line-Ground"
      n_R_busmv_B = "n:R:busmv:B", "Ohm, Network Resistance, Line-Ground"
      n_R_busmv_C = "n:R:busmv:C", "Ohm, Network Resistance, Line-Ground"
      n_R_busmv_N = "n:R:busmv:N", "Ohm, Network Resistance, Line-Ground"
      n_R_buslv_A = "n:R:buslv:A", "Ohm, Network Resistance, Line-Ground"
      n_R_buslv_B = "n:R:buslv:B", "Ohm, Network Resistance, Line-Ground"
      n_R_buslv_C = "n:R:buslv:C", "Ohm, Network Resistance, Line-Ground"
      n_R_buslv_N = "n:R:buslv:N", "Ohm, Network Resistance, Line-Ground"
      n_R_bushvn = "n:R:bushvn", "Ohm, Network Resistance, Line-Ground"
      n_R_busmvn = "n:R:busmvn", "Ohm, Network Resistance, Line-Ground"
      n_R_buslvn = "n:R:buslvn", "Ohm, Network Resistance, Line-Ground"
      n_X_bushv_A = "n:X:bushv:A", "Ohm, Network Reactance, Line-Ground"
      n_X_bushv_B = "n:X:bushv:B", "Ohm, Network Reactance, Line-Ground"
      n_X_bushv_C = "n:X:bushv:C", "Ohm, Network Reactance, Line-Ground"
      n_X_bushv_N = "n:X:bushv:N", "Ohm, Network Reactance, Line-Ground"
      n_X_busmv_A = "n:X:busmv:A", "Ohm, Network Reactance, Line-Ground"
      n_X_busmv_B = "n:X:busmv:B", "Ohm, Network Reactance, Line-Ground"
      n_X_busmv_C = "n:X:busmv:C", "Ohm, Network Reactance, Line-Ground"
      n_X_busmv_N = "n:X:busmv:N", "Ohm, Network Reactance, Line-Ground"
      n_X_buslv_A = "n:X:buslv:A", "Ohm, Network Reactance, Line-Ground"
      n_X_buslv_B = "n:X:buslv:B", "Ohm, Network Reactance, Line-Ground"
      n_X_buslv_C = "n:X:buslv:C", "Ohm, Network Reactance, Line-Ground"
      n_X_buslv_N = "n:X:buslv:N", "Ohm, Network Reactance, Line-Ground"
      n_X_bushvn = "n:X:bushvn", "Ohm, Network Reactance, Line-Ground"
      n_X_busmvn = "n:X:busmvn", "Ohm, Network Reactance, Line-Ground"
      n_X_buslvn = "n:X:buslvn", "Ohm, Network Reactance, Line-Ground"
      n_Z_bushv_A = "n:Z:bushv:A", "Ohm, Network Impedance, Line-Ground"
      n_Z_bushv_B = "n:Z:bushv:B", "Ohm, Network Impedance, Line-Ground"
      n_Z_bushv_C = "n:Z:bushv:C", "Ohm, Network Impedance, Line-Ground"
      n_Z_bushv_N = "n:Z:bushv:N", "Ohm, Network Impedance, Line-Ground"
      n_Z_busmv_A = "n:Z:busmv:A", "Ohm, Network Impedance, Line-Ground"
      n_Z_busmv_B = "n:Z:busmv:B", "Ohm, Network Impedance, Line-Ground"
      n_Z_busmv_C = "n:Z:busmv:C", "Ohm, Network Impedance, Line-Ground"
      n_Z_busmv_N = "n:Z:busmv:N", "Ohm, Network Impedance, Line-Ground"
      n_Z_buslv_A = "n:Z:buslv:A", "Ohm, Network Impedance, Line-Ground"
      n_Z_buslv_B = "n:Z:buslv:B", "Ohm, Network Impedance, Line-Ground"
      n_Z_buslv_C = "n:Z:buslv:C", "Ohm, Network Impedance, Line-Ground"
      n_Z_buslv_N = "n:Z:buslv:N", "Ohm, Network Impedance, Line-Ground"
      n_Z_bushvn = "n:Z:bushvn", "Ohm, Network Impedance, Line-Ground"
      n_Z_busmvn = "n:Z:busmvn", "Ohm, Network Impedance, Line-Ground"
      n_Z_buslvn = "n:Z:buslvn", "Ohm, Network Impedance, Line-Ground"
      n_phiz_bushv_A = "n:phiz:bushv:A", "deg, Angle of Network Impedance, Line-Ground"
      n_phiz_bushv_B = "n:phiz:bushv:B", "deg, Angle of Network Impedance, Line-Ground"
      n_phiz_bushv_C = "n:phiz:bushv:C", "deg, Angle of Network Impedance, Line-Ground"
      n_phiz_bushv_N = "n:phiz:bushv:N", "deg, Angle of Network Impedance, Line-Ground"
      n_phiz_busmv_A = "n:phiz:busmv:A", "deg, Angle of Network Impedance, Line-Ground"
      n_phiz_busmv_B = "n:phiz:busmv:B", "deg, Angle of Network Impedance, Line-Ground"
      n_phiz_busmv_C = "n:phiz:busmv:C", "deg, Angle of Network Impedance, Line-Ground"
      n_phiz_busmv_N = "n:phiz:busmv:N", "deg, Angle of Network Impedance, Line-Ground"
      n_phiz_buslv_A = "n:phiz:buslv:A", "deg, Angle of Network Impedance, Line-Ground"
      n_phiz_buslv_B = "n:phiz:buslv:B", "deg, Angle of Network Impedance, Line-Ground"
      n_phiz_buslv_C = "n:phiz:buslv:C", "deg, Angle of Network Impedance, Line-Ground"
      n_phiz_buslv_N = "n:phiz:buslv:N", "deg, Angle of Network Impedance, Line-Ground"
      n_phiz_bushvn = "n:phiz:bushvn", "deg, Angle of Network Impedance, Line-Ground"
      n_phiz_busmvn = "n:phiz:busmvn", "deg, Angle of Network Impedance, Line-Ground"
      n_phiz_buslvn = "n:phiz:buslvn", "deg, Angle of Network Impedance, Line-Ground"
      n_Rl_bushv_A = "n:Rl:bushv:A", "Ohm, Network Resistance, Line-Line"
      n_Rl_bushv_B = "n:Rl:bushv:B", "Ohm, Network Resistance, Line-Line"
      n_Rl_bushv_C = "n:Rl:bushv:C", "Ohm, Network Resistance, Line-Line"
      n_Rl_bushv_N = "n:Rl:bushv:N", "Ohm, Network Resistance, Line-Line"
      n_Rl_busmv_A = "n:Rl:busmv:A", "Ohm, Network Resistance, Line-Line"
      n_Rl_busmv_B = "n:Rl:busmv:B", "Ohm, Network Resistance, Line-Line"
      n_Rl_busmv_C = "n:Rl:busmv:C", "Ohm, Network Resistance, Line-Line"
      n_Rl_busmv_N = "n:Rl:busmv:N", "Ohm, Network Resistance, Line-Line"
      n_Rl_buslv_A = "n:Rl:buslv:A", "Ohm, Network Resistance, Line-Line"
      n_Rl_buslv_B = "n:Rl:buslv:B", "Ohm, Network Resistance, Line-Line"
      n_Rl_buslv_C = "n:Rl:buslv:C", "Ohm, Network Resistance, Line-Line"
      n_Rl_buslv_N = "n:Rl:buslv:N", "Ohm, Network Resistance, Line-Line"
      n_Rl_bushvn = "n:Rl:bushvn", "Ohm, Network Resistance, Line-Line"
      n_Rl_busmvn = "n:Rl:busmvn", "Ohm, Network Resistance, Line-Line"
      n_Rl_buslvn = "n:Rl:buslvn", "Ohm, Network Resistance, Line-Line"
      n_Xl_bushv_A = "n:Xl:bushv:A", "Ohm, Network Reactance, Line-Line"
      n_Xl_bushv_B = "n:Xl:bushv:B", "Ohm, Network Reactance, Line-Line"
      n_Xl_bushv_C = "n:Xl:bushv:C", "Ohm, Network Reactance, Line-Line"
      n_Xl_bushv_N = "n:Xl:bushv:N", "Ohm, Network Reactance, Line-Line"
      n_Xl_busmv_A = "n:Xl:busmv:A", "Ohm, Network Reactance, Line-Line"
      n_Xl_busmv_B = "n:Xl:busmv:B", "Ohm, Network Reactance, Line-Line"
      n_Xl_busmv_C = "n:Xl:busmv:C", "Ohm, Network Reactance, Line-Line"
      n_Xl_busmv_N = "n:Xl:busmv:N", "Ohm, Network Reactance, Line-Line"
      n_Xl_buslv_A = "n:Xl:buslv:A", "Ohm, Network Reactance, Line-Line"
      n_Xl_buslv_B = "n:Xl:buslv:B", "Ohm, Network Reactance, Line-Line"
      n_Xl_buslv_C = "n:Xl:buslv:C", "Ohm, Network Reactance, Line-Line"
      n_Xl_buslv_N = "n:Xl:buslv:N", "Ohm, Network Reactance, Line-Line"
      n_Xl_bushvn = "n:Xl:bushvn", "Ohm, Network Reactance, Line-Line"
      n_Xl_busmvn = "n:Xl:busmvn", "Ohm, Network Reactance, Line-Line"
      n_Xl_buslvn = "n:Xl:buslvn", "Ohm, Network Reactance, Line-Line"
      n_Zl_bushv_A = "n:Zl:bushv:A", "Ohm, Network Impedance, Line-Line"
      n_Zl_bushv_B = "n:Zl:bushv:B", "Ohm, Network Impedance, Line-Line"
      n_Zl_bushv_C = "n:Zl:bushv:C", "Ohm, Network Impedance, Line-Line"
      n_Zl_bushv_N = "n:Zl:bushv:N", "Ohm, Network Impedance, Line-Line"
      n_Zl_busmv_A = "n:Zl:busmv:A", "Ohm, Network Impedance, Line-Line"
      n_Zl_busmv_B = "n:Zl:busmv:B", "Ohm, Network Impedance, Line-Line"
      n_Zl_busmv_C = "n:Zl:busmv:C", "Ohm, Network Impedance, Line-Line"
      n_Zl_busmv_N = "n:Zl:busmv:N", "Ohm, Network Impedance, Line-Line"
      n_Zl_buslv_A = "n:Zl:buslv:A", "Ohm, Network Impedance, Line-Line"
      n_Zl_buslv_B = "n:Zl:buslv:B", "Ohm, Network Impedance, Line-Line"
      n_Zl_buslv_C = "n:Zl:buslv:C", "Ohm, Network Impedance, Line-Line"
      n_Zl_buslv_N = "n:Zl:buslv:N", "Ohm, Network Impedance, Line-Line"
      n_Zl_bushvn = "n:Zl:bushvn", "Ohm, Network Impedance, Line-Line"
      n_Zl_busmvn = "n:Zl:busmvn", "Ohm, Network Impedance, Line-Line"
      n_Zl_buslvn = "n:Zl:buslvn", "Ohm, Network Impedance, Line-Line"
      n_phizl_bushv_A = "n:phizl:bushv:A", "deg, Angle of Network Impedance, Line-Line"
      n_phizl_bushv_B = "n:phizl:bushv:B", "deg, Angle of Network Impedance, Line-Line"
      n_phizl_bushv_C = "n:phizl:bushv:C", "deg, Angle of Network Impedance, Line-Line"
      n_phizl_bushv_N = "n:phizl:bushv:N", "deg, Angle of Network Impedance, Line-Line"
      n_phizl_busmv_A = "n:phizl:busmv:A", "deg, Angle of Network Impedance, Line-Line"
      n_phizl_busmv_B = "n:phizl:busmv:B", "deg, Angle of Network Impedance, Line-Line"
      n_phizl_busmv_C = "n:phizl:busmv:C", "deg, Angle of Network Impedance, Line-Line"
      n_phizl_busmv_N = "n:phizl:busmv:N", "deg, Angle of Network Impedance, Line-Line"
      n_phizl_buslv_A = "n:phizl:buslv:A", "deg, Angle of Network Impedance, Line-Line"
      n_phizl_buslv_B = "n:phizl:buslv:B", "deg, Angle of Network Impedance, Line-Line"
      n_phizl_buslv_C = "n:phizl:buslv:C", "deg, Angle of Network Impedance, Line-Line"
      n_phizl_buslv_N = "n:phizl:buslv:N", "deg, Angle of Network Impedance, Line-Line"
      n_phizl_bushvn = "n:phizl:bushvn", "deg, Angle of Network Impedance, Line-Line"
      n_phizl_busmvn = "n:phizl:busmvn", "deg, Angle of Network Impedance, Line-Line"
      n_phizl_buslvn = "n:phizl:buslvn", "deg, Angle of Network Impedance, Line-Line"
      n_Rln_bushv_A = "n:Rln:bushv:A", "Ohm, Network Resistance, Line-Neutral"
      n_Rln_bushv_B = "n:Rln:bushv:B", "Ohm, Network Resistance, Line-Neutral"
      n_Rln_bushv_C = "n:Rln:bushv:C", "Ohm, Network Resistance, Line-Neutral"
      n_Rln_bushv_N = "n:Rln:bushv:N", "Ohm, Network Resistance, Line-Neutral"
      n_Rln_busmv_A = "n:Rln:busmv:A", "Ohm, Network Resistance, Line-Neutral"
      n_Rln_busmv_B = "n:Rln:busmv:B", "Ohm, Network Resistance, Line-Neutral"
      n_Rln_busmv_C = "n:Rln:busmv:C", "Ohm, Network Resistance, Line-Neutral"
      n_Rln_busmv_N = "n:Rln:busmv:N", "Ohm, Network Resistance, Line-Neutral"
      n_Rln_buslv_A = "n:Rln:buslv:A", "Ohm, Network Resistance, Line-Neutral"
      n_Rln_buslv_B = "n:Rln:buslv:B", "Ohm, Network Resistance, Line-Neutral"
      n_Rln_buslv_C = "n:Rln:buslv:C", "Ohm, Network Resistance, Line-Neutral"
      n_Rln_buslv_N = "n:Rln:buslv:N", "Ohm, Network Resistance, Line-Neutral"
      n_Rln_bushvn = "n:Rln:bushvn", "Ohm, Network Resistance, Line-Neutral"
      n_Rln_busmvn = "n:Rln:busmvn", "Ohm, Network Resistance, Line-Neutral"
      n_Rln_buslvn = "n:Rln:buslvn", "Ohm, Network Resistance, Line-Neutral"
      n_Xln_bushv_A = "n:Xln:bushv:A", "Ohm, Network Reactance, Line-Neutral"
      n_Xln_bushv_B = "n:Xln:bushv:B", "Ohm, Network Reactance, Line-Neutral"
      n_Xln_bushv_C = "n:Xln:bushv:C", "Ohm, Network Reactance, Line-Neutral"
      n_Xln_bushv_N = "n:Xln:bushv:N", "Ohm, Network Reactance, Line-Neutral"
      n_Xln_busmv_A = "n:Xln:busmv:A", "Ohm, Network Reactance, Line-Neutral"
      n_Xln_busmv_B = "n:Xln:busmv:B", "Ohm, Network Reactance, Line-Neutral"
      n_Xln_busmv_C = "n:Xln:busmv:C", "Ohm, Network Reactance, Line-Neutral"
      n_Xln_busmv_N = "n:Xln:busmv:N", "Ohm, Network Reactance, Line-Neutral"
      n_Xln_buslv_A = "n:Xln:buslv:A", "Ohm, Network Reactance, Line-Neutral"
      n_Xln_buslv_B = "n:Xln:buslv:B", "Ohm, Network Reactance, Line-Neutral"
      n_Xln_buslv_C = "n:Xln:buslv:C", "Ohm, Network Reactance, Line-Neutral"
      n_Xln_buslv_N = "n:Xln:buslv:N", "Ohm, Network Reactance, Line-Neutral"
      n_Xln_bushvn = "n:Xln:bushvn", "Ohm, Network Reactance, Line-Neutral"
      n_Xln_busmvn = "n:Xln:busmvn", "Ohm, Network Reactance, Line-Neutral"
      n_Xln_buslvn = "n:Xln:buslvn", "Ohm, Network Reactance, Line-Neutral"
      n_Zln_bushv_A = "n:Zln:bushv:A", "Ohm, Network Impedance, Line-Neutral"
      n_Zln_bushv_B = "n:Zln:bushv:B", "Ohm, Network Impedance, Line-Neutral"
      n_Zln_bushv_C = "n:Zln:bushv:C", "Ohm, Network Impedance, Line-Neutral"
      n_Zln_bushv_N = "n:Zln:bushv:N", "Ohm, Network Impedance, Line-Neutral"
      n_Zln_busmv_A = "n:Zln:busmv:A", "Ohm, Network Impedance, Line-Neutral"
      n_Zln_busmv_B = "n:Zln:busmv:B", "Ohm, Network Impedance, Line-Neutral"
      n_Zln_busmv_C = "n:Zln:busmv:C", "Ohm, Network Impedance, Line-Neutral"
      n_Zln_busmv_N = "n:Zln:busmv:N", "Ohm, Network Impedance, Line-Neutral"
      n_Zln_buslv_A = "n:Zln:buslv:A", "Ohm, Network Impedance, Line-Neutral"
      n_Zln_buslv_B = "n:Zln:buslv:B", "Ohm, Network Impedance, Line-Neutral"
      n_Zln_buslv_C = "n:Zln:buslv:C", "Ohm, Network Impedance, Line-Neutral"
      n_Zln_buslv_N = "n:Zln:buslv:N", "Ohm, Network Impedance, Line-Neutral"
      n_Zln_bushvn = "n:Zln:bushvn", "Ohm, Network Impedance, Line-Neutral"
      n_Zln_busmvn = "n:Zln:busmvn", "Ohm, Network Impedance, Line-Neutral"
      n_Zln_buslvn = "n:Zln:buslvn", "Ohm, Network Impedance, Line-Neutral"
      n_phizln_bushv_A = "n:phizln:bushv:A", "deg, Angle of Network Impedance, Line-Neutral"
      n_phizln_bushv_B = "n:phizln:bushv:B", "deg, Angle of Network Impedance, Line-Neutral"
      n_phizln_bushv_C = "n:phizln:bushv:C", "deg, Angle of Network Impedance, Line-Neutral"
      n_phizln_bushv_N = "n:phizln:bushv:N", "deg, Angle of Network Impedance, Line-Neutral"
      n_phizln_busmv_A = "n:phizln:busmv:A", "deg, Angle of Network Impedance, Line-Neutral"
      n_phizln_busmv_B = "n:phizln:busmv:B", "deg, Angle of Network Impedance, Line-Neutral"
      n_phizln_busmv_C = "n:phizln:busmv:C", "deg, Angle of Network Impedance, Line-Neutral"
      n_phizln_busmv_N = "n:phizln:busmv:N", "deg, Angle of Network Impedance, Line-Neutral"
      n_phizln_buslv_A = "n:phizln:buslv:A", "deg, Angle of Network Impedance, Line-Neutral"
      n_phizln_buslv_B = "n:phizln:buslv:B", "deg, Angle of Network Impedance, Line-Neutral"
      n_phizln_buslv_C = "n:phizln:buslv:C", "deg, Angle of Network Impedance, Line-Neutral"
      n_phizln_buslv_N = "n:phizln:buslv:N", "deg, Angle of Network Impedance, Line-Neutral"
      n_phizln_bushvn = "n:phizln:bushvn", "deg, Angle of Network Impedance, Line-Neutral"
      n_phizln_busmvn = "n:phizln:busmvn", "deg, Angle of Network Impedance, Line-Neutral"
      n_phizln_buslvn = "n:phizln:buslvn", "deg, Angle of Network Impedance, Line-Neutral"
      n_Rn_bushv = "n:Rn:bushv", "Ohm, Network Resistance, Neutral"
      n_Rn_busmv = "n:Rn:busmv", "Ohm, Network Resistance, Neutral"
      n_Rn_buslv = "n:Rn:buslv", "Ohm, Network Resistance, Neutral"
      n_Rn_bushvn = "n:Rn:bushvn", "Ohm, Network Resistance, Neutral"
      n_Rn_busmvn = "n:Rn:busmvn", "Ohm, Network Resistance, Neutral"
      n_Rn_buslvn = "n:Rn:buslvn", "Ohm, Network Resistance, Neutral"
      n_Xn_bushv = "n:Xn:bushv", "Ohm, Network Reactance, Neutral"
      n_Xn_busmv = "n:Xn:busmv", "Ohm, Network Reactance, Neutral"
      n_Xn_buslv = "n:Xn:buslv", "Ohm, Network Reactance, Neutral"
      n_Xn_bushvn = "n:Xn:bushvn", "Ohm, Network Reactance, Neutral"
      n_Xn_busmvn = "n:Xn:busmvn", "Ohm, Network Reactance, Neutral"
      n_Xn_buslvn = "n:Xn:buslvn", "Ohm, Network Reactance, Neutral"
      n_Zn_bushv = "n:Zn:bushv", "Ohm, Network Impedance, Neutral"
      n_Zn_busmv = "n:Zn:busmv", "Ohm, Network Impedance, Neutral"
      n_Zn_buslv = "n:Zn:buslv", "Ohm, Network Impedance, Neutral"
      n_Zn_bushvn = "n:Zn:bushvn", "Ohm, Network Impedance, Neutral"
      n_Zn_busmvn = "n:Zn:busmvn", "Ohm, Network Impedance, Neutral"
      n_Zn_buslvn = "n:Zn:buslvn", "Ohm, Network Impedance, Neutral"
      n_phizn_bushv = "n:phizn:bushv", "deg, Angle of Network Impedance, Neutral"
      n_phizn_busmv = "n:phizn:busmv", "deg, Angle of Network Impedance, Neutral"
      n_phizn_buslv = "n:phizn:buslv", "deg, Angle of Network Impedance, Neutral"
      n_phizn_bushvn = "n:phizn:bushvn", "deg, Angle of Network Impedance, Neutral"
      n_phizn_busmvn = "n:phizn:busmvn", "deg, Angle of Network Impedance, Neutral"
      n_phizn_buslvn = "n:phizn:buslvn", "deg, Angle of Network Impedance, Neutral"
      n_eff_bushv_A = "n:eff:bushv:A", "Earth Fault Factor"
      n_eff_bushv_B = "n:eff:bushv:B", "Earth Fault Factor"
      n_eff_bushv_C = "n:eff:bushv:C", "Earth Fault Factor"
      n_eff_bushv_N = "n:eff:bushv:N", "Earth Fault Factor"
      n_eff_busmv_A = "n:eff:busmv:A", "Earth Fault Factor"
      n_eff_busmv_B = "n:eff:busmv:B", "Earth Fault Factor"
      n_eff_busmv_C = "n:eff:busmv:C", "Earth Fault Factor"
      n_eff_busmv_N = "n:eff:busmv:N", "Earth Fault Factor"
      n_eff_buslv_A = "n:eff:buslv:A", "Earth Fault Factor"
      n_eff_buslv_B = "n:eff:buslv:B", "Earth Fault Factor"
      n_eff_buslv_C = "n:eff:buslv:C", "Earth Fault Factor"
      n_eff_buslv_N = "n:eff:buslv:N", "Earth Fault Factor"
      n_eff_bushvn = "n:eff:bushvn", "Earth Fault Factor"
      n_eff_busmvn = "n:eff:busmvn", "Earth Fault Factor"
      n_eff_buslvn = "n:eff:buslvn", "Earth Fault Factor"
      n_X0toX1_bushv = "n:X0toX1:bushv", "X0/X1"
      n_X0toX1_busmv = "n:X0toX1:busmv", "X0/X1"
      n_X0toX1_buslv = "n:X0toX1:buslv", "X0/X1"
      n_X0toX1_bushvn = "n:X0toX1:bushvn", "X0/X1"
      n_X0toX1_busmvn = "n:X0toX1:busmvn", "X0/X1"
      n_X0toX1_buslvn = "n:X0toX1:buslvn", "X0/X1"
      n_in_bushv = "n:in:bushv", "p.u., Neutral Current, Magnitude"
      n_in_busmv = "n:in:busmv", "p.u., Neutral Current, Magnitude"
      n_in_buslv = "n:in:buslv", "p.u., Neutral Current, Magnitude"
      n_in_bushvn = "n:in:bushvn", "p.u., Neutral Current, Magnitude"
      n_in_busmvn = "n:in:busmvn", "p.u., Neutral Current, Magnitude"
      n_in_buslvn = "n:in:buslvn", "p.u., Neutral Current, Magnitude"
      n_In_bushv = "n:In:bushv", "kA, Neutral Current, Magnitude"
      n_In_busmv = "n:In:busmv", "kA, Neutral Current, Magnitude"
      n_In_buslv = "n:In:buslv", "kA, Neutral Current, Magnitude"
      n_In_bushvn = "n:In:bushvn", "kA, Neutral Current, Magnitude"
      n_In_busmvn = "n:In:busmvn", "kA, Neutral Current, Magnitude"
      n_In_buslvn = "n:In:buslvn", "kA, Neutral Current, Magnitude"
      n_phiin_bushv = "n:phiin:bushv", "deg, Neutral Current, Angle"
      n_phiin_busmv = "n:phiin:busmv", "deg, Neutral Current, Angle"
      n_phiin_buslv = "n:phiin:buslv", "deg, Neutral Current, Angle"
      n_phiin_bushvn = "n:phiin:bushvn", "deg, Neutral Current, Angle"
      n_phiin_busmvn = "n:phiin:busmvn", "deg, Neutral Current, Angle"
      n_phiin_buslvn = "n:phiin:buslvn", "deg, Neutral Current, Angle"
      n_i0r_bushv = "n:i0r:bushv", "p.u., Zero-Sequence Current, Real Part"
      n_i0r_busmv = "n:i0r:busmv", "p.u., Zero-Sequence Current, Real Part"
      n_i0r_buslv = "n:i0r:buslv", "p.u., Zero-Sequence Current, Real Part"
      n_i0r_bushvn = "n:i0r:bushvn", "p.u., Zero-Sequence Current, Real Part"
      n_i0r_busmvn = "n:i0r:busmvn", "p.u., Zero-Sequence Current, Real Part"
      n_i0r_buslvn = "n:i0r:buslvn", "p.u., Zero-Sequence Current, Real Part"
      n_i0i_bushv = "n:i0i:bushv", "p.u., Zero-Sequence Current, Imaginary Part"
      n_i0i_busmv = "n:i0i:busmv", "p.u., Zero-Sequence Current, Imaginary Part"
      n_i0i_buslv = "n:i0i:buslv", "p.u., Zero-Sequence Current, Imaginary Part"
      n_i0i_bushvn = "n:i0i:bushvn", "p.u., Zero-Sequence Current, Imaginary Part"
      n_i0i_busmvn = "n:i0i:busmvn", "p.u., Zero-Sequence Current, Imaginary Part"
      n_i0i_buslvn = "n:i0i:buslvn", "p.u., Zero-Sequence Current, Imaginary Part"
      n_i0_bushv = "n:i0:bushv", "p.u., Zero-Sequence Current, Magnitude"
      n_i0_busmv = "n:i0:busmv", "p.u., Zero-Sequence Current, Magnitude"
      n_i0_buslv = "n:i0:buslv", "p.u., Zero-Sequence Current, Magnitude"
      n_i0_bushvn = "n:i0:bushvn", "p.u., Zero-Sequence Current, Magnitude"
      n_i0_busmvn = "n:i0:busmvn", "p.u., Zero-Sequence Current, Magnitude"
      n_i0_buslvn = "n:i0:buslvn", "p.u., Zero-Sequence Current, Magnitude"
      n_I0_bushv = "n:I0:bushv", "kA, Zero-Sequence Current, Magnitude"
      n_I0_busmv = "n:I0:busmv", "kA, Zero-Sequence Current, Magnitude"
      n_I0_buslv = "n:I0:buslv", "kA, Zero-Sequence Current, Magnitude"
      n_I0_bushvn = "n:I0:bushvn", "kA, Zero-Sequence Current, Magnitude"
      n_I0_busmvn = "n:I0:busmvn", "kA, Zero-Sequence Current, Magnitude"
      n_I0_buslvn = "n:I0:buslvn", "kA, Zero-Sequence Current, Magnitude"
      n_I0x3_bushv = "n:I0x3:bushv", "kA, 3*I0"
      n_I0x3_busmv = "n:I0x3:busmv", "kA, 3*I0"
      n_I0x3_buslv = "n:I0x3:buslv", "kA, 3*I0"
      n_I0x3_bushvn = "n:I0x3:bushvn", "kA, 3*I0"
      n_I0x3_busmvn = "n:I0x3:busmvn", "kA, 3*I0"
      n_I0x3_buslvn = "n:I0x3:buslvn", "kA, 3*I0"
      n_phii0_bushv = "n:phii0:bushv", "deg, Zero-Sequence Current, Angle"
      n_phii0_busmv = "n:phii0:busmv", "deg, Zero-Sequence Current, Angle"
      n_phii0_buslv = "n:phii0:buslv", "deg, Zero-Sequence Current, Angle"
      n_phii0_bushvn = "n:phii0:bushvn", "deg, Zero-Sequence Current, Angle"
      n_phii0_busmvn = "n:phii0:busmvn", "deg, Zero-Sequence Current, Angle"
      n_phii0_buslvn = "n:phii0:buslvn", "deg, Zero-Sequence Current, Angle"
      n_i1r_bushv = "n:i1r:bushv", "p.u., Positive-Sequence Current, Real Part"
      n_i1r_busmv = "n:i1r:busmv", "p.u., Positive-Sequence Current, Real Part"
      n_i1r_buslv = "n:i1r:buslv", "p.u., Positive-Sequence Current, Real Part"
      n_i1r_bushvn = "n:i1r:bushvn", "p.u., Positive-Sequence Current, Real Part"
      n_i1r_busmvn = "n:i1r:busmvn", "p.u., Positive-Sequence Current, Real Part"
      n_i1r_buslvn = "n:i1r:buslvn", "p.u., Positive-Sequence Current, Real Part"
      n_i1i_bushv = "n:i1i:bushv", "p.u., Positive-Sequence Current, Imaginary Part"
      n_i1i_busmv = "n:i1i:busmv", "p.u., Positive-Sequence Current, Imaginary Part"
      n_i1i_buslv = "n:i1i:buslv", "p.u., Positive-Sequence Current, Imaginary Part"
      n_i1i_bushvn = "n:i1i:bushvn", "p.u., Positive-Sequence Current, Imaginary Part"
      n_i1i_busmvn = "n:i1i:busmvn", "p.u., Positive-Sequence Current, Imaginary Part"
      n_i1i_buslvn = "n:i1i:buslvn", "p.u., Positive-Sequence Current, Imaginary Part"
      n_i1_bushv = "n:i1:bushv", "p.u., Positive-Sequence Current, Magnitude"
      n_i1_busmv = "n:i1:busmv", "p.u., Positive-Sequence Current, Magnitude"
      n_i1_buslv = "n:i1:buslv", "p.u., Positive-Sequence Current, Magnitude"
      n_i1_bushvn = "n:i1:bushvn", "p.u., Positive-Sequence Current, Magnitude"
      n_i1_busmvn = "n:i1:busmvn", "p.u., Positive-Sequence Current, Magnitude"
      n_i1_buslvn = "n:i1:buslvn", "p.u., Positive-Sequence Current, Magnitude"
      n_I1_bushv = "n:I1:bushv", "kA, Positive-Sequence Current, Magnitude"
      n_I1_busmv = "n:I1:busmv", "kA, Positive-Sequence Current, Magnitude"
      n_I1_buslv = "n:I1:buslv", "kA, Positive-Sequence Current, Magnitude"
      n_I1_bushvn = "n:I1:bushvn", "kA, Positive-Sequence Current, Magnitude"
      n_I1_busmvn = "n:I1:busmvn", "kA, Positive-Sequence Current, Magnitude"
      n_I1_buslvn = "n:I1:buslvn", "kA, Positive-Sequence Current, Magnitude"
      n_phii1_bushv = "n:phii1:bushv", "deg, Positive-Sequence Current, Angle"
      n_phii1_busmv = "n:phii1:busmv", "deg, Positive-Sequence Current, Angle"
      n_phii1_buslv = "n:phii1:buslv", "deg, Positive-Sequence Current, Angle"
      n_phii1_bushvn = "n:phii1:bushvn", "deg, Positive-Sequence Current, Angle"
      n_phii1_busmvn = "n:phii1:busmvn", "deg, Positive-Sequence Current, Angle"
      n_phii1_buslvn = "n:phii1:buslvn", "deg, Positive-Sequence Current, Angle"
      n_i2r_bushv = "n:i2r:bushv", "p.u., Negative-Sequence Current, Real Part"
      n_i2r_busmv = "n:i2r:busmv", "p.u., Negative-Sequence Current, Real Part"
      n_i2r_buslv = "n:i2r:buslv", "p.u., Negative-Sequence Current, Real Part"
      n_i2r_bushvn = "n:i2r:bushvn", "p.u., Negative-Sequence Current, Real Part"
      n_i2r_busmvn = "n:i2r:busmvn", "p.u., Negative-Sequence Current, Real Part"
      n_i2r_buslvn = "n:i2r:buslvn", "p.u., Negative-Sequence Current, Real Part"
      n_i2i_bushv = "n:i2i:bushv", "p.u., Negative-Sequence Current, Imaginary Part"
      n_i2i_busmv = "n:i2i:busmv", "p.u., Negative-Sequence Current, Imaginary Part"
      n_i2i_buslv = "n:i2i:buslv", "p.u., Negative-Sequence Current, Imaginary Part"
      n_i2i_bushvn = "n:i2i:bushvn", "p.u., Negative-Sequence Current, Imaginary Part"
      n_i2i_busmvn = "n:i2i:busmvn", "p.u., Negative-Sequence Current, Imaginary Part"
      n_i2i_buslvn = "n:i2i:buslvn", "p.u., Negative-Sequence Current, Imaginary Part"
      n_i2_bushv = "n:i2:bushv", "p.u., Negative-Sequence Current, Magnitude"
      n_i2_busmv = "n:i2:busmv", "p.u., Negative-Sequence Current, Magnitude"
      n_i2_buslv = "n:i2:buslv", "p.u., Negative-Sequence Current, Magnitude"
      n_i2_bushvn = "n:i2:bushvn", "p.u., Negative-Sequence Current, Magnitude"
      n_i2_busmvn = "n:i2:busmvn", "p.u., Negative-Sequence Current, Magnitude"
      n_i2_buslvn = "n:i2:buslvn", "p.u., Negative-Sequence Current, Magnitude"
      n_I2_bushv = "n:I2:bushv", "kA, Negative-Sequence Current, Magnitude"
      n_I2_busmv = "n:I2:busmv", "kA, Negative-Sequence Current, Magnitude"
      n_I2_buslv = "n:I2:buslv", "kA, Negative-Sequence Current, Magnitude"
      n_I2_bushvn = "n:I2:bushvn", "kA, Negative-Sequence Current, Magnitude"
      n_I2_busmvn = "n:I2:busmvn", "kA, Negative-Sequence Current, Magnitude"
      n_I2_buslvn = "n:I2:buslvn", "kA, Negative-Sequence Current, Magnitude"
      n_phii2_bushv = "n:phii2:bushv", "deg, Negative-Sequence Current, Angle"
      n_phii2_busmv = "n:phii2:busmv", "deg, Negative-Sequence Current, Angle"
      n_phii2_buslv = "n:phii2:buslv", "deg, Negative-Sequence Current, Angle"
      n_phii2_bushvn = "n:phii2:bushvn", "deg, Negative-Sequence Current, Angle"
      n_phii2_busmvn = "n:phii2:busmvn", "deg, Negative-Sequence Current, Angle"
      n_phii2_buslvn = "n:phii2:buslvn", "deg, Negative-Sequence Current, Angle"
      n_du_bushv_A = "n:du:bushv:A", "%, Voltage Deviation"
      n_du_bushv_B = "n:du:bushv:B", "%, Voltage Deviation"
      n_du_bushv_C = "n:du:bushv:C", "%, Voltage Deviation"
      n_du_bushv_N = "n:du:bushv:N", "%, Voltage Deviation"
      n_du_busmv_A = "n:du:busmv:A", "%, Voltage Deviation"
      n_du_busmv_B = "n:du:busmv:B", "%, Voltage Deviation"
      n_du_busmv_C = "n:du:busmv:C", "%, Voltage Deviation"
      n_du_busmv_N = "n:du:busmv:N", "%, Voltage Deviation"
      n_du_buslv_A = "n:du:buslv:A", "%, Voltage Deviation"
      n_du_buslv_B = "n:du:buslv:B", "%, Voltage Deviation"
      n_du_buslv_C = "n:du:buslv:C", "%, Voltage Deviation"
      n_du_buslv_N = "n:du:buslv:N", "%, Voltage Deviation"
      n_du_bushvn = "n:du:bushvn", "%, Voltage Deviation"
      n_du_busmvn = "n:du:busmvn", "%, Voltage Deviation"
      n_du_buslvn = "n:du:buslvn", "%, Voltage Deviation"
      n_urpref_bushv_A = "n:urpref:bushv:A", "p.u., Pre-fault Voltage, Real Part"
      n_urpref_bushv_B = "n:urpref:bushv:B", "p.u., Pre-fault Voltage, Real Part"
      n_urpref_bushv_C = "n:urpref:bushv:C", "p.u., Pre-fault Voltage, Real Part"
      n_urpref_bushv_N = "n:urpref:bushv:N", "p.u., Pre-fault Voltage, Real Part"
      n_urpref_busmv_A = "n:urpref:busmv:A", "p.u., Pre-fault Voltage, Real Part"
      n_urpref_busmv_B = "n:urpref:busmv:B", "p.u., Pre-fault Voltage, Real Part"
      n_urpref_busmv_C = "n:urpref:busmv:C", "p.u., Pre-fault Voltage, Real Part"
      n_urpref_busmv_N = "n:urpref:busmv:N", "p.u., Pre-fault Voltage, Real Part"
      n_urpref_buslv_A = "n:urpref:buslv:A", "p.u., Pre-fault Voltage, Real Part"
      n_urpref_buslv_B = "n:urpref:buslv:B", "p.u., Pre-fault Voltage, Real Part"
      n_urpref_buslv_C = "n:urpref:buslv:C", "p.u., Pre-fault Voltage, Real Part"
      n_urpref_buslv_N = "n:urpref:buslv:N", "p.u., Pre-fault Voltage, Real Part"
      n_urpref_bushvn = "n:urpref:bushvn", "p.u., Pre-fault Voltage, Real Part"
      n_urpref_busmvn = "n:urpref:busmvn", "p.u., Pre-fault Voltage, Real Part"
      n_urpref_buslvn = "n:urpref:buslvn", "p.u., Pre-fault Voltage, Real Part"
      n_uipref_bushv_A = "n:uipref:bushv:A", "p.u., Pre-fault Voltage, Imaginary Part"
      n_uipref_bushv_B = "n:uipref:bushv:B", "p.u., Pre-fault Voltage, Imaginary Part"
      n_uipref_bushv_C = "n:uipref:bushv:C", "p.u., Pre-fault Voltage, Imaginary Part"
      n_uipref_bushv_N = "n:uipref:bushv:N", "p.u., Pre-fault Voltage, Imaginary Part"
      n_uipref_busmv_A = "n:uipref:busmv:A", "p.u., Pre-fault Voltage, Imaginary Part"
      n_uipref_busmv_B = "n:uipref:busmv:B", "p.u., Pre-fault Voltage, Imaginary Part"
      n_uipref_busmv_C = "n:uipref:busmv:C", "p.u., Pre-fault Voltage, Imaginary Part"
      n_uipref_busmv_N = "n:uipref:busmv:N", "p.u., Pre-fault Voltage, Imaginary Part"
      n_uipref_buslv_A = "n:uipref:buslv:A", "p.u., Pre-fault Voltage, Imaginary Part"
      n_uipref_buslv_B = "n:uipref:buslv:B", "p.u., Pre-fault Voltage, Imaginary Part"
      n_uipref_buslv_C = "n:uipref:buslv:C", "p.u., Pre-fault Voltage, Imaginary Part"
      n_uipref_buslv_N = "n:uipref:buslv:N", "p.u., Pre-fault Voltage, Imaginary Part"
      n_uipref_bushvn = "n:uipref:bushvn", "p.u., Pre-fault Voltage, Imaginary Part"
      n_uipref_busmvn = "n:uipref:busmvn", "p.u., Pre-fault Voltage, Imaginary Part"
      n_uipref_buslvn = "n:uipref:buslvn", "p.u., Pre-fault Voltage, Imaginary Part"
      n_upref_bushv_A = "n:upref:bushv:A", "p.u., Pre-fault Voltage, Magnitude"
      n_upref_bushv_B = "n:upref:bushv:B", "p.u., Pre-fault Voltage, Magnitude"
      n_upref_bushv_C = "n:upref:bushv:C", "p.u., Pre-fault Voltage, Magnitude"
      n_upref_bushv_N = "n:upref:bushv:N", "p.u., Pre-fault Voltage, Magnitude"
      n_upref_busmv_A = "n:upref:busmv:A", "p.u., Pre-fault Voltage, Magnitude"
      n_upref_busmv_B = "n:upref:busmv:B", "p.u., Pre-fault Voltage, Magnitude"
      n_upref_busmv_C = "n:upref:busmv:C", "p.u., Pre-fault Voltage, Magnitude"
      n_upref_busmv_N = "n:upref:busmv:N", "p.u., Pre-fault Voltage, Magnitude"
      n_upref_buslv_A = "n:upref:buslv:A", "p.u., Pre-fault Voltage, Magnitude"
      n_upref_buslv_B = "n:upref:buslv:B", "p.u., Pre-fault Voltage, Magnitude"
      n_upref_buslv_C = "n:upref:buslv:C", "p.u., Pre-fault Voltage, Magnitude"
      n_upref_buslv_N = "n:upref:buslv:N", "p.u., Pre-fault Voltage, Magnitude"
      n_upref_bushvn = "n:upref:bushvn", "p.u., Pre-fault Voltage, Magnitude"
      n_upref_busmvn = "n:upref:busmvn", "p.u., Pre-fault Voltage, Magnitude"
      n_upref_buslvn = "n:upref:buslvn", "p.u., Pre-fault Voltage, Magnitude"
      n_uprefpc_bushv_A = "n:uprefpc:bushv:A", "%, Pre-fault Voltage, Magnitude"
      n_uprefpc_bushv_B = "n:uprefpc:bushv:B", "%, Pre-fault Voltage, Magnitude"
      n_uprefpc_bushv_C = "n:uprefpc:bushv:C", "%, Pre-fault Voltage, Magnitude"
      n_uprefpc_bushv_N = "n:uprefpc:bushv:N", "%, Pre-fault Voltage, Magnitude"
      n_uprefpc_busmv_A = "n:uprefpc:busmv:A", "%, Pre-fault Voltage, Magnitude"
      n_uprefpc_busmv_B = "n:uprefpc:busmv:B", "%, Pre-fault Voltage, Magnitude"
      n_uprefpc_busmv_C = "n:uprefpc:busmv:C", "%, Pre-fault Voltage, Magnitude"
      n_uprefpc_busmv_N = "n:uprefpc:busmv:N", "%, Pre-fault Voltage, Magnitude"
      n_uprefpc_buslv_A = "n:uprefpc:buslv:A", "%, Pre-fault Voltage, Magnitude"
      n_uprefpc_buslv_B = "n:uprefpc:buslv:B", "%, Pre-fault Voltage, Magnitude"
      n_uprefpc_buslv_C = "n:uprefpc:buslv:C", "%, Pre-fault Voltage, Magnitude"
      n_uprefpc_buslv_N = "n:uprefpc:buslv:N", "%, Pre-fault Voltage, Magnitude"
      n_uprefpc_bushvn = "n:uprefpc:bushvn", "%, Pre-fault Voltage, Magnitude"
      n_uprefpc_busmvn = "n:uprefpc:busmvn", "%, Pre-fault Voltage, Magnitude"
      n_uprefpc_buslvn = "n:uprefpc:buslvn", "%, Pre-fault Voltage, Magnitude"
      n_Upref_bushv_A = "n:Upref:bushv:A", "kV, Line-Ground Pre-fault Voltage, Magnitude"
      n_Upref_bushv_B = "n:Upref:bushv:B", "kV, Line-Ground Pre-fault Voltage, Magnitude"
      n_Upref_bushv_C = "n:Upref:bushv:C", "kV, Line-Ground Pre-fault Voltage, Magnitude"
      n_Upref_bushv_N = "n:Upref:bushv:N", "kV, Line-Ground Pre-fault Voltage, Magnitude"
      n_Upref_busmv_A = "n:Upref:busmv:A", "kV, Line-Ground Pre-fault Voltage, Magnitude"
      n_Upref_busmv_B = "n:Upref:busmv:B", "kV, Line-Ground Pre-fault Voltage, Magnitude"
      n_Upref_busmv_C = "n:Upref:busmv:C", "kV, Line-Ground Pre-fault Voltage, Magnitude"
      n_Upref_busmv_N = "n:Upref:busmv:N", "kV, Line-Ground Pre-fault Voltage, Magnitude"
      n_Upref_buslv_A = "n:Upref:buslv:A", "kV, Line-Ground Pre-fault Voltage, Magnitude"
      n_Upref_buslv_B = "n:Upref:buslv:B", "kV, Line-Ground Pre-fault Voltage, Magnitude"
      n_Upref_buslv_C = "n:Upref:buslv:C", "kV, Line-Ground Pre-fault Voltage, Magnitude"
      n_Upref_buslv_N = "n:Upref:buslv:N", "kV, Line-Ground Pre-fault Voltage, Magnitude"
      n_Upref_bushvn = "n:Upref:bushvn", "kV, Line-Ground Pre-fault Voltage, Magnitude"
      n_Upref_busmvn = "n:Upref:busmvn", "kV, Line-Ground Pre-fault Voltage, Magnitude"
      n_Upref_buslvn = "n:Upref:buslvn", "kV, Line-Ground Pre-fault Voltage, Magnitude"
      n_Ulpref_bushv_A = "n:Ulpref:bushv:A", "kV, Line-Line Pre-fault Voltage, Magnitude"
      n_Ulpref_bushv_B = "n:Ulpref:bushv:B", "kV, Line-Line Pre-fault Voltage, Magnitude"
      n_Ulpref_bushv_C = "n:Ulpref:bushv:C", "kV, Line-Line Pre-fault Voltage, Magnitude"
      n_Ulpref_bushv_N = "n:Ulpref:bushv:N", "kV, Line-Line Pre-fault Voltage, Magnitude"
      n_Ulpref_busmv_A = "n:Ulpref:busmv:A", "kV, Line-Line Pre-fault Voltage, Magnitude"
      n_Ulpref_busmv_B = "n:Ulpref:busmv:B", "kV, Line-Line Pre-fault Voltage, Magnitude"
      n_Ulpref_busmv_C = "n:Ulpref:busmv:C", "kV, Line-Line Pre-fault Voltage, Magnitude"
      n_Ulpref_busmv_N = "n:Ulpref:busmv:N", "kV, Line-Line Pre-fault Voltage, Magnitude"
      n_Ulpref_buslv_A = "n:Ulpref:buslv:A", "kV, Line-Line Pre-fault Voltage, Magnitude"
      n_Ulpref_buslv_B = "n:Ulpref:buslv:B", "kV, Line-Line Pre-fault Voltage, Magnitude"
      n_Ulpref_buslv_C = "n:Ulpref:buslv:C", "kV, Line-Line Pre-fault Voltage, Magnitude"
      n_Ulpref_buslv_N = "n:Ulpref:buslv:N", "kV, Line-Line Pre-fault Voltage, Magnitude"
      n_Ulpref_bushvn = "n:Ulpref:bushvn", "kV, Line-Line Pre-fault Voltage, Magnitude"
      n_Ulpref_busmvn = "n:Ulpref:busmvn", "kV, Line-Line Pre-fault Voltage, Magnitude"
      n_Ulpref_buslvn = "n:Ulpref:buslvn", "kV, Line-Line Pre-fault Voltage, Magnitude"
      n_phiupref_bushv_A = "n:phiupref:bushv:A", "deg, Pre-fault Voltage, Angle"
      n_phiupref_bushv_B = "n:phiupref:bushv:B", "deg, Pre-fault Voltage, Angle"
      n_phiupref_bushv_C = "n:phiupref:bushv:C", "deg, Pre-fault Voltage, Angle"
      n_phiupref_bushv_N = "n:phiupref:bushv:N", "deg, Pre-fault Voltage, Angle"
      n_phiupref_busmv_A = "n:phiupref:busmv:A", "deg, Pre-fault Voltage, Angle"
      n_phiupref_busmv_B = "n:phiupref:busmv:B", "deg, Pre-fault Voltage, Angle"
      n_phiupref_busmv_C = "n:phiupref:busmv:C", "deg, Pre-fault Voltage, Angle"
      n_phiupref_busmv_N = "n:phiupref:busmv:N", "deg, Pre-fault Voltage, Angle"
      n_phiupref_buslv_A = "n:phiupref:buslv:A", "deg, Pre-fault Voltage, Angle"
      n_phiupref_buslv_B = "n:phiupref:buslv:B", "deg, Pre-fault Voltage, Angle"
      n_phiupref_buslv_C = "n:phiupref:buslv:C", "deg, Pre-fault Voltage, Angle"
      n_phiupref_buslv_N = "n:phiupref:buslv:N", "deg, Pre-fault Voltage, Angle"
      n_phiupref_bushvn = "n:phiupref:bushvn", "deg, Pre-fault Voltage, Angle"
      n_phiupref_busmvn = "n:phiupref:busmvn", "deg, Pre-fault Voltage, Angle"
      n_phiupref_buslvn = "n:phiupref:buslvn", "deg, Pre-fault Voltage, Angle"
      n_phiui_bushv_A = "n:phiui:bushv:A", "deg, Angle between Voltage and Current"
      n_phiui_bushv_B = "n:phiui:bushv:B", "deg, Angle between Voltage and Current"
      n_phiui_bushv_C = "n:phiui:bushv:C", "deg, Angle between Voltage and Current"
      n_phiui_bushv_N = "n:phiui:bushv:N", "deg, Angle between Voltage and Current"
      n_phiui_busmv_A = "n:phiui:busmv:A", "deg, Angle between Voltage and Current"
      n_phiui_busmv_B = "n:phiui:busmv:B", "deg, Angle between Voltage and Current"
      n_phiui_busmv_C = "n:phiui:busmv:C", "deg, Angle between Voltage and Current"
      n_phiui_busmv_N = "n:phiui:busmv:N", "deg, Angle between Voltage and Current"
      n_phiui_buslv_A = "n:phiui:buslv:A", "deg, Angle between Voltage and Current"
      n_phiui_buslv_B = "n:phiui:buslv:B", "deg, Angle between Voltage and Current"
      n_phiui_buslv_C = "n:phiui:buslv:C", "deg, Angle between Voltage and Current"
      n_phiui_buslv_N = "n:phiui:buslv:N", "deg, Angle between Voltage and Current"
      n_phiui_bushvn = "n:phiui:bushvn", "deg, Angle between Voltage and Current"
      n_phiui_busmvn = "n:phiui:busmvn", "deg, Angle between Voltage and Current"
      n_phiui_buslvn = "n:phiui:buslvn", "deg, Angle between Voltage and Current"
      n_phiu1i1_bushv = "n:phiu1i1:bushv", "deg, Angle between Voltage and Current in positive sequence system"
      n_phiu1i1_busmv = "n:phiu1i1:busmv", "deg, Angle between Voltage and Current in positive sequence system"
      n_phiu1i1_buslv = "n:phiu1i1:buslv", "deg, Angle between Voltage and Current in positive sequence system"
      n_phiu1i1_bushvn = "n:phiu1i1:bushvn", "deg, Angle between Voltage and Current in positive sequence system"
      n_phiu1i1_busmvn = "n:phiu1i1:busmvn", "deg, Angle between Voltage and Current in positive sequence system"
      n_phiu1i1_buslvn = "n:phiu1i1:buslvn", "deg, Angle between Voltage and Current in positive sequence system"
      n_phiu2i2_bushv = "n:phiu2i2:bushv", "deg, Angle between Voltage and Current in negative sequence system"
      n_phiu2i2_busmv = "n:phiu2i2:busmv", "deg, Angle between Voltage and Current in negative sequence system"
      n_phiu2i2_buslv = "n:phiu2i2:buslv", "deg, Angle between Voltage and Current in negative sequence system"
      n_phiu2i2_bushvn = "n:phiu2i2:bushvn", "deg, Angle between Voltage and Current in negative sequence system"
      n_phiu2i2_busmvn = "n:phiu2i2:busmvn", "deg, Angle between Voltage and Current in negative sequence system"
      n_phiu2i2_buslvn = "n:phiu2i2:buslvn", "deg, Angle between Voltage and Current in negative sequence system"
      n_phiu0i0_bushv = "n:phiu0i0:bushv", "deg, Angle between Voltage and Current in zero-sequence system"
      n_phiu0i0_busmv = "n:phiu0i0:busmv", "deg, Angle between Voltage and Current in zero-sequence system"
      n_phiu0i0_buslv = "n:phiu0i0:buslv", "deg, Angle between Voltage and Current in zero-sequence system"
      n_phiu0i0_bushvn = "n:phiu0i0:bushvn", "deg, Angle between Voltage and Current in zero-sequence system"
      n_phiu0i0_busmvn = "n:phiu0i0:busmvn", "deg, Angle between Voltage and Current in zero-sequence system"
      n_phiu0i0_buslvn = "n:phiu0i0:buslvn", "deg, Angle between Voltage and Current in zero-sequence system"
      n_Ithload_bushvn = "n:Ithload:bushvn", "%, Loading, Thermal equivalent Current"
      n_Ithload_busmvn = "n:Ithload:busmvn", "%, Loading, Thermal equivalent Current"
      n_Ithload_buslvn = "n:Ithload:buslvn", "%, Loading, Thermal equivalent Current"
      n_Ipload_bushvn = "n:Ipload:bushvn", "%, Loading, Peak Short-Circuit Current"
      n_Ipload_busmvn = "n:Ipload:busmvn", "%, Loading, Peak Short-Circuit Current"
      n_Ipload_buslvn = "n:Ipload:buslvn", "%, Loading, Peak Short-Circuit Current"
      n_Ithrtk_bushvn = "n:Ithrtk:bushvn", "kA, Rated Short-Time Current (Tk)"
      n_Ithrtk_busmvn = "n:Ithrtk:busmvn", "kA, Rated Short-Time Current (Tk)"
      n_Ithrtk_buslvn = "n:Ithrtk:buslvn", "kA, Rated Short-Time Current (Tk)"
      n_cfac_bushvn = "n:cfac:bushvn", "Voltage Factor"
      n_cfac_busmvn = "n:cfac:busmvn", "Voltage Factor"
      n_cfac_buslvn = "n:cfac:buslvn", "Voltage Factor"
      n_XtoR_bushvn = "n:XtoR:bushvn", "X/R ratio (peak)"
      n_XtoR_busmvn = "n:XtoR:busmvn", "X/R ratio (peak)"
      n_XtoR_buslvn = "n:XtoR:buslvn", "X/R ratio (peak)"
      n_RtoX_bushvn = "n:RtoX:bushvn", "R/X ratio (peak)"
      n_RtoX_busmvn = "n:RtoX:busmvn", "R/X ratio (peak)"
      n_RtoX_buslvn = "n:RtoX:buslvn", "R/X ratio (peak)"

    class ElmVac(Enum):
      _init_ = 'value __doc__'

      m_u0_bus1 = "m:u0:bus1", "p.u., Zero-Sequence-Voltage, Magnitude"
      m_phiu0_bus1 = "m:phiu0:bus1", "deg, Zero-Sequence-Voltage, Angle"
      m_u1r_bus1 = "m:u1r:bus1", "p.u., Positive-Sequence-Voltage, Real Part"
      m_u1i_bus1 = "m:u1i:bus1", "p.u., Positive-Sequence-Voltage, Imaginary Part"
      m_u1_bus1 = "m:u1:bus1", "p.u., Positive-Sequence-Voltage, Magnitude"
      m_U1_bus1 = "m:U1:bus1", "kV, Line-Ground Positive-Sequence-Voltage, Magnitude"
      m_U1l_bus1 = "m:U1l:bus1", "kV, Line-Line Positive-Sequence-Voltage, Magnitude"
      m_phiu1_bus1 = "m:phiu1:bus1", "deg, Positive-Sequence-Voltage, Angle"
      m_u2_bus1 = "m:u2:bus1", "p.u., Negative-Sequence-Voltage, Magnitude"
      m_phiu2_bus1 = "m:phiu2:bus1", "deg, Negative-Sequence-Voltage, Angle"
      m_inet_bus1_A = "m:inet:bus1:A", "p.u., Phase Current, Magnitude, referred to network"
      m_inet_bus1_B = "m:inet:bus1:B", "p.u., Phase Current, Magnitude, referred to network"
      m_inet_bus1_C = "m:inet:bus1:C", "p.u., Phase Current, Magnitude, referred to network"
      m_I_bus1_A = "m:I:bus1:A", "kA, Phase Current, Magnitude"
      m_I_bus1_B = "m:I:bus1:B", "kA, Phase Current, Magnitude"
      m_I_bus1_C = "m:I:bus1:C", "kA, Phase Current, Magnitude"
      m_phii_bus1_A = "m:phii:bus1:A", "deg, Phase Current, Angle"
      m_phii_bus1_B = "m:phii:bus1:B", "deg, Phase Current, Angle"
      m_phii_bus1_C = "m:phii:bus1:C", "deg, Phase Current, Angle"
      m_i0r_bus1 = "m:i0r:bus1", "p.u., Zero-Sequence Current, Real Part"
      m_i0i_bus1 = "m:i0i:bus1", "p.u., Zero-Sequence Current, Imaginary Part"
      m_i0_bus1 = "m:i0:bus1", "p.u., Zero-Sequence Current, Magnitude"
      m_I0_bus1 = "m:I0:bus1", "kA, Zero-Sequence Current, Magnitude"
      m_I0x3_bus1 = "m:I0x3:bus1", "kA, 3*I0"
      m_phii0_bus1 = "m:phii0:bus1", "deg, Zero-Sequence Current, Angle"
      m_i1r_bus1 = "m:i1r:bus1", "p.u., Positive-Sequence Current, Real Part"
      m_i1i_bus1 = "m:i1i:bus1", "p.u., Positive-Sequence Current, Imaginary Part"
      m_i1_bus1 = "m:i1:bus1", "p.u., Positive-Sequence Current, Magnitude"
      m_I1_bus1 = "m:I1:bus1", "kA, Positive-Sequence Current, Magnitude"
      m_phii1_bus1 = "m:phii1:bus1", "deg, Positive-Sequence Current, Angle"
      m_i2r_bus1 = "m:i2r:bus1", "p.u., Negative-Sequence Current, Real Part"
      m_i2i_bus1 = "m:i2i:bus1", "p.u., Negative-Sequence Current, Imaginary Part"
      m_i2_bus1 = "m:i2:bus1", "p.u., Negative-Sequence Current, Magnitude"
      m_I2_bus1 = "m:I2:bus1", "kA, Negative-Sequence Current, Magnitude"
      m_phii2_bus1 = "m:phii2:bus1", "deg, Negative-Sequence Current, Angle"
      m_ubfacI_bus1 = "m:ubfacI:bus1", "%, Current Unbalance Factor"
      m_ubfacS_bus1 = "m:ubfacS:bus1", "%, Power Unbalance Factor"
      m_P_bus1_A = "m:P:bus1:A", "MW, Active Power"
      m_P_bus1_B = "m:P:bus1:B", "MW, Active Power"
      m_P_bus1_C = "m:P:bus1:C", "MW, Active Power"
      m_Q_bus1_A = "m:Q:bus1:A", "Mvar, Reactive Power"
      m_Q_bus1_B = "m:Q:bus1:B", "Mvar, Reactive Power"
      m_Q_bus1_C = "m:Q:bus1:C", "Mvar, Reactive Power"
      m_S_bus1_A = "m:S:bus1:A", "MVA, Apparent Power"
      m_S_bus1_B = "m:S:bus1:B", "MVA, Apparent Power"
      m_S_bus1_C = "m:S:bus1:C", "MVA, Apparent Power"
      m_cosphi_bus1_A = "m:cosphi:bus1:A", "Power Factor"
      m_cosphi_bus1_B = "m:cosphi:bus1:B", "Power Factor"
      m_cosphi_bus1_C = "m:cosphi:bus1:C", "Power Factor"
      m_tanphi_bus1_A = "m:tanphi:bus1:A", "tan(phi)"
      m_tanphi_bus1_B = "m:tanphi:bus1:B", "tan(phi)"
      m_tanphi_bus1_C = "m:tanphi:bus1:C", "tan(phi)"
      m_Psum_bus1 = "m:Psum:bus1", "MW, Total Active Power"
      m_Qsum_bus1 = "m:Qsum:bus1", "Mvar, Total Reactive Power"
      m_Ssum_bus1 = "m:Ssum:bus1", "MVA, Total Apparent Power"
      m_cosphisum_bus1 = "m:cosphisum:bus1", "Total Power Factor"
      m_tanphisum_bus1 = "m:tanphisum:bus1", "Total tan(phi)"
      m_Pln_bus1_A = "m:Pln:bus1:A", "MW, Active Power (Line-Neutral)"
      m_Pln_bus1_B = "m:Pln:bus1:B", "MW, Active Power (Line-Neutral)"
      m_Pln_bus1_C = "m:Pln:bus1:C", "MW, Active Power (Line-Neutral)"
      m_Qln_bus1_A = "m:Qln:bus1:A", "Mvar, Reactive Power (Line-Neutral)"
      m_Qln_bus1_B = "m:Qln:bus1:B", "Mvar, Reactive Power (Line-Neutral)"
      m_Qln_bus1_C = "m:Qln:bus1:C", "Mvar, Reactive Power (Line-Neutral)"
      m_P1_bus1 = "m:P1:bus1", "MW, Positive Sequence Active Power"
      m_Q1_bus1 = "m:Q1:bus1", "Mvar, Positive Sequence Reactive Power"
      m_P2_bus1 = "m:P2:bus1", "MW, Negative Sequence Active Power"
      m_Q2_bus1 = "m:Q2:bus1", "Mvar, Negative Sequence Reactive Power"
      m_P0_bus1 = "m:P0:bus1", "MW, Zero Sequence Active Power"
      m_Q0_bus1 = "m:Q0:bus1", "Mvar, Zero Sequence Reactive Power"
      m_i1P_bus1 = "m:i1P:bus1", "p.u., Positive-Sequence Active Current"
      m_i1Q_bus1 = "m:i1Q:bus1", "p.u., Positive-Sequence Reactive Current"
      m_I1P_bus1 = "m:I1P:bus1", "kA, Positive-Sequence Active Current"
      m_I1Q_bus1 = "m:I1Q:bus1", "kA, Positive-Sequence Reactive Current"
      m_i2P_bus1 = "m:i2P:bus1", "p.u., Negative-Sequence Active Current"
      m_i2Q_bus1 = "m:i2Q:bus1", "p.u., Negative-Sequence Reactive Current"
      m_I2P_bus1 = "m:I2P:bus1", "kA, Negative-Sequence Active Current"
      m_I2Q_bus1 = "m:I2Q:bus1", "kA, Negative-Sequence Reactive Current"
      m_ur_bus1_A = "m:ur:bus1:A", "p.u., Phase Voltage, Real Part"
      m_ur_bus1_B = "m:ur:bus1:B", "p.u., Phase Voltage, Real Part"
      m_ur_bus1_C = "m:ur:bus1:C", "p.u., Phase Voltage, Real Part"
      m_ui_bus1_A = "m:ui:bus1:A", "p.u., Phase Voltage, Imaginary Part"
      m_ui_bus1_B = "m:ui:bus1:B", "p.u., Phase Voltage, Imaginary Part"
      m_ui_bus1_C = "m:ui:bus1:C", "p.u., Phase Voltage, Imaginary Part"
      m_u_bus1_A = "m:u:bus1:A", "p.u., Phase Voltage, Magnitude"
      m_u_bus1_B = "m:u:bus1:B", "p.u., Phase Voltage, Magnitude"
      m_u_bus1_C = "m:u:bus1:C", "p.u., Phase Voltage, Magnitude"
      m_ir_bus1_A = "m:ir:bus1:A", "p.u., Phase Current, Real Part"
      m_ir_bus1_B = "m:ir:bus1:B", "p.u., Phase Current, Real Part"
      m_ir_bus1_C = "m:ir:bus1:C", "p.u., Phase Current, Real Part"
      m_ii_bus1_A = "m:ii:bus1:A", "p.u., Phase Current, Imaginary Part"
      m_ii_bus1_B = "m:ii:bus1:B", "p.u., Phase Current, Imaginary Part"
      m_ii_bus1_C = "m:ii:bus1:C", "p.u., Phase Current, Imaginary Part"
      m_i_bus1_A = "m:i:bus1:A", "p.u., Phase Current, Magnitude"
      m_i_bus1_B = "m:i:bus1:B", "p.u., Phase Current, Magnitude"
      m_i_bus1_C = "m:i:bus1:C", "p.u., Phase Current, Magnitude"
      m_phiui_bus1_A = "m:phiui:bus1:A", "deg, Angle between Voltage and Current"
      m_phiui_bus1_B = "m:phiui:bus1:B", "deg, Angle between Voltage and Current"
      m_phiui_bus1_C = "m:phiui:bus1:C", "deg, Angle between Voltage and Current"
      m_phiu0i0_bus1 = "m:phiu0i0:bus1", "deg, Angle between Voltage and Current in zero-sequence system"
      m_phiu1i1_bus1 = "m:phiu1i1:bus1", "deg, Angle between Voltage and Current in positive sequence system"
      m_phiu2i2_bus1 = "m:phiu2i2:bus1", "deg, Angle between Voltage and Current in negative sequence system"
      m_Tfct_bus1 = "m:Tfct:bus1", "s, Fault Clearing Time"
      m_TfctPh_bus1_A = "m:TfctPh:bus1:A", "s, Fault Clearing Time"
      m_TfctPh_bus1_B = "m:TfctPh:bus1:B", "s, Fault Clearing Time"
      m_TfctPh_bus1_C = "m:TfctPh:bus1:C", "s, Fault Clearing Time"
      m_Brkload_bus1 = "m:Brkload:bus1", "%, Max. Breaker Loading"
      m_BrkloadPh_bus1_A = "m:BrkloadPh:bus1:A", "%, Breaker Loading"
      m_BrkloadPh_bus1_B = "m:BrkloadPh:bus1:B", "%, Breaker Loading"
      m_BrkloadPh_bus1_C = "m:BrkloadPh:bus1:C", "%, Breaker Loading"
      n_ur_bus1_A = "n:ur:bus1:A", "p.u., Line-Ground Voltage, Real Part"
      n_ur_bus1_B = "n:ur:bus1:B", "p.u., Line-Ground Voltage, Real Part"
      n_ur_bus1_C = "n:ur:bus1:C", "p.u., Line-Ground Voltage, Real Part"
      n_ui_bus1_A = "n:ui:bus1:A", "p.u., Line-Ground Voltage, Imaginary Part"
      n_ui_bus1_B = "n:ui:bus1:B", "p.u., Line-Ground Voltage, Imaginary Part"
      n_ui_bus1_C = "n:ui:bus1:C", "p.u., Line-Ground Voltage, Imaginary Part"
      n_u_bus1_A = "n:u:bus1:A", "p.u., Line-Ground Voltage, Magnitude"
      n_u_bus1_B = "n:u:bus1:B", "p.u., Line-Ground Voltage, Magnitude"
      n_u_bus1_C = "n:u:bus1:C", "p.u., Line-Ground Voltage, Magnitude"
      n_upc_bus1_A = "n:upc:bus1:A", "%, Line-Ground Voltage, Magnitude"
      n_upc_bus1_B = "n:upc:bus1:B", "%, Line-Ground Voltage, Magnitude"
      n_upc_bus1_C = "n:upc:bus1:C", "%, Line-Ground Voltage, Magnitude"
      n_U_bus1_A = "n:U:bus1:A", "kV, Line-Ground Voltage, Magnitude"
      n_U_bus1_B = "n:U:bus1:B", "kV, Line-Ground Voltage, Magnitude"
      n_U_bus1_C = "n:U:bus1:C", "kV, Line-Ground Voltage, Magnitude"
      n_phiu_bus1_A = "n:phiu:bus1:A", "deg, Line-Ground Voltage, Angle"
      n_phiu_bus1_B = "n:phiu:bus1:B", "deg, Line-Ground Voltage, Angle"
      n_phiu_bus1_C = "n:phiu:bus1:C", "deg, Line-Ground Voltage, Angle"
      n_phiurel_bus1 = "n:phiurel:bus1", "deg, Voltage, Relative Angle"
      n_ul_bus1_A = "n:ul:bus1:A", "p.u., Line-Line Voltage, Magnitude"
      n_ul_bus1_B = "n:ul:bus1:B", "p.u., Line-Line Voltage, Magnitude"
      n_ul_bus1_C = "n:ul:bus1:C", "p.u., Line-Line Voltage, Magnitude"
      n_ulpc_bus1_A = "n:ulpc:bus1:A", "%, Line-Line Voltage, Magnitude"
      n_ulpc_bus1_B = "n:ulpc:bus1:B", "%, Line-Line Voltage, Magnitude"
      n_ulpc_bus1_C = "n:ulpc:bus1:C", "%, Line-Line Voltage, Magnitude"
      n_Ul_bus1_A = "n:Ul:bus1:A", "kV, Line-Line Voltage, Magnitude"
      n_Ul_bus1_B = "n:Ul:bus1:B", "kV, Line-Line Voltage, Magnitude"
      n_Ul_bus1_C = "n:Ul:bus1:C", "kV, Line-Line Voltage, Magnitude"
      n_phiul_bus1_A = "n:phiul:bus1:A", "deg, Line-Line Voltage, Angle"
      n_phiul_bus1_B = "n:phiul:bus1:B", "deg, Line-Line Voltage, Angle"
      n_phiul_bus1_C = "n:phiul:bus1:C", "deg, Line-Line Voltage, Angle"
      n_u0_bus1 = "n:u0:bus1", "p.u., Zero-Sequence Voltage, Magnitude"
      n_U0_bus1 = "n:U0:bus1", "kV, Zero-Sequence Voltage, Magnitude"
      n_U0x3_bus1 = "n:U0x3:bus1", "kV, 3*U0"
      n_phiu0_bus1 = "n:phiu0:bus1", "deg, Zero-Sequence Voltage, Angle"
      n_u1_bus1 = "n:u1:bus1", "p.u., Positive-Sequence Voltage, Magnitude"
      n_u1pc_bus1 = "n:u1pc:bus1", "%, Positive-Sequence Voltage, Magnitude"
      n_u1r_bus1 = "n:u1r:bus1", "p.u., Positive-Sequence Voltage, Real Part"
      n_u1i_bus1 = "n:u1i:bus1", "p.u., Positive-Sequence Voltage, Imaginary Part"
      n_U1_bus1 = "n:U1:bus1", "kV, Line-Ground Positive-Sequence Voltage, Magnitude"
      n_phiu1_bus1 = "n:phiu1:bus1", "deg, Positive-Sequence Voltage, Angle"
      n_u2_bus1 = "n:u2:bus1", "p.u., Negative-Sequence Voltage, Magnitude"
      n_U2_bus1 = "n:U2:bus1", "kV, Line-Ground Negative-Sequence Voltage, Magnitude"
      n_phiu2_bus1 = "n:phiu2:bus1", "deg, Negative-Sequence Voltage, Angle"
      n_U1l_bus1 = "n:U1l:bus1", "kV, Line-Line Positive-Sequence Voltage, Magnitude"
      n_U2l_bus1 = "n:U2l:bus1", "kV, Line-Line Negative-Sequence Voltage, Magnitude"
      n_uln_bus1_A = "n:uln:bus1:A", "p.u., Line-Neutral Voltage, Magnitude"
      n_uln_bus1_B = "n:uln:bus1:B", "p.u., Line-Neutral Voltage, Magnitude"
      n_uln_bus1_C = "n:uln:bus1:C", "p.u., Line-Neutral Voltage, Magnitude"
      n_Uln_bus1_A = "n:Uln:bus1:A", "kV, Line-Neutral Voltage, Magnitude"
      n_Uln_bus1_B = "n:Uln:bus1:B", "kV, Line-Neutral Voltage, Magnitude"
      n_Uln_bus1_C = "n:Uln:bus1:C", "kV, Line-Neutral Voltage, Magnitude"
      n_phiuln_bus1_A = "n:phiuln:bus1:A", "deg, Line-Neutral Voltage, Angle"
      n_phiuln_bus1_B = "n:phiuln:bus1:B", "deg, Line-Neutral Voltage, Angle"
      n_phiuln_bus1_C = "n:phiuln:bus1:C", "deg, Line-Neutral Voltage, Angle"
      n_un_bus1 = "n:un:bus1", "p.u., Neutral-Ground Voltage, Magnitude"
      n_Un_bus1 = "n:Un:bus1", "kV, Neutral-Ground Voltage, Magnitude"
      n_phiun_bus1 = "n:phiun:bus1", "deg, Neutral-Ground Voltage, Angle"
      n_upht_bus1_A = "n:upht:bus1:A", "p.u., Phase Technology dependent Voltage, Magnitude"
      n_upht_bus1_B = "n:upht:bus1:B", "p.u., Phase Technology dependent Voltage, Magnitude"
      n_upht_bus1_C = "n:upht:bus1:C", "p.u., Phase Technology dependent Voltage, Magnitude"
      n_Upht_bus1_A = "n:Upht:bus1:A", "kV, Phase Technology dependent Voltage, Magnitude"
      n_Upht_bus1_B = "n:Upht:bus1:B", "kV, Phase Technology dependent Voltage, Magnitude"
      n_Upht_bus1_C = "n:Upht:bus1:C", "kV, Phase Technology dependent Voltage, Magnitude"
      n_um_bus1 = "n:um:bus1", "p.u., Average-Voltage, Magnitude"
      n_Um_bus1 = "n:Um:bus1", "kV, Average-Voltage, Magnitude"
      n_uphtmin_bus1 = "n:uphtmin:bus1", "p.u., Minimum of Phase Technology dependent Voltage, Magnitude"
      n_uphtmax_bus1 = "n:uphtmax:bus1", "p.u., Maximum of Phase Technology dependent Voltage, Magnitude"
      n_Pgen_bus1 = "n:Pgen:bus1", "MW, Generation, Active Power"
      n_Qgen_bus1 = "n:Qgen:bus1", "Mvar, Generation, Reactive Power"
      n_Pmot_bus1 = "n:Pmot:bus1", "MW, Motor Load, Active Power"
      n_Qmot_bus1 = "n:Qmot:bus1", "Mvar, Motor Load, Reactive Power"
      n_Pload_bus1 = "n:Pload:bus1", "MW, General Load, Active Power"
      n_Qload_bus1 = "n:Qload:bus1", "Mvar, General Load, Reactive Power"
      n_Pcomp_bus1 = "n:Pcomp:bus1", "MW, Compensation (Losses)"
      n_Qcomp_bus1 = "n:Qcomp:bus1", "Mvar, Compensation"
      n_Pnet_bus1 = "n:Pnet:bus1", "MW, External Networks, Active Power"
      n_Qnet_bus1 = "n:Qnet:bus1", "Mvar, External Networks, Reactive Power"
      n_Pflow_bus1 = "n:Pflow:bus1", "MW, Power Flow, Active Power"
      n_Qflow_bus1 = "n:Qflow:bus1", "Mvar, Power Flow, Reactive Power"
      n_Pout_bus1 = "n:Pout:bus1", "MW, Outgoing Flow, Active Power"
      n_Qout_bus1 = "n:Qout:bus1", "Mvar, Outgoing Power, Reactive Power"
      n_Sout_bus1 = "n:Sout:bus1", "MVA, Outgoing Power, Apparent Power"
      n_cosphiout_bus1 = "n:cosphiout:bus1", "Outgoing Power, Power Factor"
      n_Pbalance_bus1 = "n:Pbalance:bus1", "MW, Active Power Balance (=0)"
      n_Qbalance_bus1 = "n:Qbalance:bus1", "Mvar, Reactive Power Balance (=0)"
      n_ubfac_bus1 = "n:ubfac:bus1", "%, Unbalance Factor"
      n_LossPdown_bus1 = "n:LossPdown:bus1", "MW, Losses, downstream"
      n_LossQdown_bus1 = "n:LossQdown:bus1", "Mvar, Losses, downstream (Reactive Power)"
      n_LossPdownload_bus1 = "n:LossPdownload:bus1", "MW, Load losses, downstream"
      n_LossQdownload_bus1 = "n:LossQdownload:bus1", "Mvar, Load losses, downstream"
      n_LossPdownnoload_bus1 = "n:LossPdownnoload:bus1", "MW, No load losses, downstream"
      n_LossQdownnoload_bus1 = "n:LossQdownnoload:bus1", "Mvar, No load losses, downstream"
      s_u0 = "s:u0", "p.u., IN, Voltage-Input, continuous, double"
      s_Ul0 = "s:Ul0", "kV, IN, Voltage-Input (Line-Line), continuous, double"
      s_U_A = "s:U_A", "kV, IN, Voltage, Magnitude, Phase a, continuous, double"
      s_U_B = "s:U_B", "kV, IN, Voltage, Magnitude, Phase b, continuous, double"
      s_U_C = "s:U_C", "kV, IN, Voltage, Magnitude, Phase c, continuous, double"
      s_phiu_B = "s:phiu_B", "deg, IN, Voltage, Angle, Phase b continuous, double"
      s_phiu_C = "s:phiu_C", "deg, IN, Voltage, Angle, Phase c continuous, double"
      s_dphiu = "s:dphiu", "rad, IN, Voltage Angle Input, continuous, double"
      s_dFin = "s:dFin", "Hz, IN, Frequency Deviation, continuous, double"
      s_dF = "s:dF", "Hz, OUT, Frequency Deviation Output, continuous, double"
      c_uini = "c:uini", "p.u., Voltage"
      c_phiu = "c:phiu", "rad, Voltage Angle"
      c_uini0_r = "c:uini0:r", "0-Sequ. Voltage, Real Part"
      c_uini0_i = "c:uini0:i", "0-Sequ. Voltage, Imaginary Part"
      c_uini0_phi = "c:uini0:phi", "0-Sequ. Voltage, Angle"
      c_uini0_mag = "c:uini0:mag", "0-Sequ. Voltage, Magnitude"
      c_uini0_phirad = "c:uini0:phirad", "0-Sequ. Voltage, Angle"
      c_uini2_r = "c:uini2:r", "2-Sequ. Voltage, Real Part"
      c_uini2_i = "c:uini2:i", "2-Sequ. Voltage, Imaginary Part"
      c_uini2_phi = "c:uini2:phi", "2-Sequ. Voltage, Angle"
      c_uini2_mag = "c:uini2:mag", "2-Sequ. Voltage, Magnitude"
      c_uini2_phirad = "c:uini2:phirad", "2-Sequ. Voltage, Angle"
      c_Kpf = "c:Kpf", "MW/Hz, Primary Frequency Bias"
      c_Ksf = "c:Ksf", "MW/Hz, Secondary Frequency Bias"
      c_Kp = "c:Kp", "Secondary Factor"
      c_iRefElement = "c:iRefElement", "Reference Element"
      c_Ploss = "c:Ploss", "MW, Losses (total)"
      c_Qloss = "c:Qloss", "Mvar, Reactive-Losses (total)"
      c_Plossld = "c:Plossld", "MW, Losses (load)"
      c_Qlossld = "c:Qlossld", "Mvar, Reactive-Losses (load)"
      c_Plossnld = "c:Plossnld", "MW, Losses (no load)"
      c_Qlossnld = "c:Qlossnld", "Mvar, Reactive-Losses (no load)"
      e_usetp = "e:usetp", "p.u., Voltage, Magnitude"
      e_phisetp = "e:phisetp", "deg, Voltage, Angle"
      e_contbar = "e:contbar", "ElmTerm, Controlled Node"
      e_R1 = "e:R1", "Ohm, Resistance, R1"
      e_X1 = "e:X1", "Ohm, Reactance, X1"
      e_p_uctrl = "e:p_uctrl", "ElmLdfctrl*, Voltage Control"
      e_p_phictrl = "e:p_phictrl", "ElmLdfctrl*, Angle Control"
      e_Pload = "e:Pload", "MW, Active Power"
      e_Qload = "e:Qload", "Mvar, Reactive Power"
      e_Pzload = "e:Pzload", "MW, Active Power"
      e_Qzload = "e:Qzload", "Mvar, Reactive Power"
      e_Kpf = "e:Kpf", "MW/Hz, Prim. Frequency Bias"
      e_Ksf = "e:Ksf", "MW/Hz, Sec. Frequency Bias"
      e_c_psecc = "e:c_psecc", "ElmSecctrl, Ext. Secondary Controller"
      e_Pgen = "e:Pgen", "MW, Active Power"
      e_Qgen = "e:Qgen", "Mvar, Reactive Power"
      e_Rext = "e:Rext", "Ohm, Resistance"
      e_Xext = "e:Xext", "Ohm, Reactance"
      e_usetp0 = "e:usetp0", "p.u., Voltage, Magnitude"
      e_phisetp0 = "e:phisetp0", "deg, Voltage, Angle"
      e_R0 = "e:R0", "Ohm, Resistance, R0"
      e_X0 = "e:X0", "Ohm, Reactance, X0"
      e_usetp2 = "e:usetp2", "p.u., Voltage, Magnitude"
      e_phisetp2 = "e:phisetp2", "deg, Voltage, Angle"
      e_R2 = "e:R2", "Ohm, Resistance, R2"
      e_X2 = "e:X2", "Ohm, Reactance, X2"
      e_iPossibleSlack = "e:iPossibleSlack", "Can be considered as a potential slack"
      e_qdslCtrl = "e:qdslCtrl", "Quasi-Dynamic Model"
      e_cpFeed = "e:cpFeed", "Feeder"
      e_ciDist = "e:ciDist", "Distance from infeed in number of buses"
      e_ciLater = "e:ciLater", "Lateral Index"
      e_ciDistRoot = "e:ciDistRoot", "Distance from first infeed in number of buses"
      e_ciDistAll = "e:ciDistAll", "Distance from infeed in number of buses including switches"
      e_ciDistAllRoot = "e:ciDistAllRoot", "Distance from first infeed in number of buses including switches"
      r_contbar = "r:contbar", "ElmTerm, Controlled Node"
      r_p_uctrl = "r:p_uctrl", "ElmLdfctrl*, Voltage Control"
      r_p_phictrl = "r:p_phictrl", "ElmLdfctrl*, Angle Control"
      r_c_psecc = "r:c_psecc", "ElmSecctrl, Ext. Secondary Controller"
      r_qdslCtrl = "r:qdslCtrl", "Quasi-Dynamic Model"
      r_cpFeed = "r:cpFeed", "Feeder"
      m_Ikss_bus1 = "m:Ikss:bus1", "kA, Initial Short-Circuit Current"
      m_ikss_bus1 = "m:ikss:bus1", "p.u., Initial Short-Circuit Current, Magnitude"
      m_phii_bus1 = "m:phii:bus1", "deg, Phase Current, Angle"
      m_maxIkss_bus1 = "m:maxIkss:bus1", "kA, Max. Initial Short-Circuit Current"
      m_Skss_bus1 = "m:Skss:bus1", "MVA, Initial Short-Circuit Power"
      m_ip_bus1 = "m:ip:bus1", "kA, Peak Short-Circuit Current"
      m_maxip_bus1 = "m:maxip:bus1", "kA, Max. Peak Short-Circuit Current"
      m_maxIb_bus1 = "m:maxIb:bus1", "kA, Max. Breaking Current"
      m_maxIth_bus1 = "m:maxIth:bus1", "kA, Max. Thermal equivalent Short-Circuit Current"
      m_Ib_bus1 = "m:Ib:bus1", "kA, Short-Circuit Breaking Current"
      m_Sb_bus1 = "m:Sb:bus1", "MVA, Short-Circuit Breaking Power"
      m_Ik_bus1 = "m:Ik:bus1", "kA, Steady-State Short-Circuit Current"
      m_Ith_bus1 = "m:Ith:bus1", "kA, Thermal equivalent Short-Circuit Current"
      m_I_bus1 = "m:I:bus1", "kA, Short-Circuit Current"
      m_R_bus1 = "m:R:bus1", "Ohm, Fault Impedance, Real Part"
      m_X_bus1 = "m:X:bus1", "Ohm, Fault Impedance, Imaginary Part"
      m_Z_bus1 = "m:Z:bus1", "Ohm, Fault Impedance, Magnitude"
      m_phiz_bus1 = "m:phiz:bus1", "deg, Fault Impedance, Angle"
      m_zSbase_bus1 = "m:zSbase:bus1", "p.u., Fault Impedance, Magnitude"
      m_XtoR_bus1 = "m:XtoR:bus1", "X/R ratio"
      m_Zs_bus1 = "m:Zs:bus1", "Ohm, Source Impedance, Magnitude"
      m_phizs_bus1 = "m:phizs:bus1", "deg, Source Impedance, Angle"
      m_Rs_bus1 = "m:Rs:bus1", "Ohm, Source Impedance, Real Part"
      m_Xs_bus1 = "m:Xs:bus1", "Ohm, Source Impedance, Imaginary Part"
      m_phiui_bus1 = "m:phiui:bus1", "deg, Angle between Voltage and Current"
      m_cosphiui_bus1 = "m:cosphiui:bus1", "cosphi"
      m_P_bus1 = "m:P:bus1", "MW, Active Power"
      m_Q_bus1 = "m:Q:bus1", "Mvar, Reactive Power"
      m_BrkIpload_bus1 = "m:BrkIpload:bus1", "%, Breaker Loading (peak)"
      m_BrkIbload_bus1 = "m:BrkIbload:bus1", "%, Breaker Loading (breaking)"
      m_BrkmaxIb_bus1 = "m:BrkmaxIb:bus1", "kA, Max. Breaking Current (breaker time)"
      n_Ikss_bus1 = "n:Ikss:bus1", "kA, Initial Short-Circuit Current"
      n_I_bus1 = "n:I:bus1", "kA, Short-Circuit Current"
      n_phii_bus1 = "n:phii:bus1", "deg, Phase Current, Angle"
      n_Skss_bus1 = "n:Skss:bus1", "MVA, Initial Short-Circuit Power"
      n_ip_bus1 = "n:ip:bus1", "kA, Peak Short-Circuit Current"
      n_Ib_bus1 = "n:Ib:bus1", "kA, Short-Circuit Breaking Current"
      n_Sb_bus1 = "n:Sb:bus1", "MVA, Short-Circuit Breaking Power"
      n_Ik_bus1 = "n:Ik:bus1", "kA, Steady-State Short-Circuit Current"
      n_Ith_bus1 = "n:Ith:bus1", "kA, Thermal equivalent Short-Circuit Current"
      n_R_bus1 = "n:R:bus1", "Ohm, Short-Circuit Impedance, Real Part"
      n_X_bus1 = "n:X:bus1", "Ohm, Short-Circuit Impedance, Imaginary Part"
      n_Z_bus1 = "n:Z:bus1", "Ohm, Short-Circuit Impedance, Magnitude"
      n_phiz_bus1 = "n:phiz:bus1", "deg, Short-Circuit Impedance, Angle"
      n_rSbase_bus1 = "n:rSbase:bus1", "p.u., Short-Circuit Impedance, Real Part"
      n_xSbase_bus1 = "n:xSbase:bus1", "p.u., Short-Circuit Impedance, Imaginary Part"
      n_zSbase_bus1 = "n:zSbase:bus1", "p.u., Short-Circuit Impedance, Magnitude"
      n_XtoR_bus1 = "n:XtoR:bus1", "X/R ratio (peak)"
      n_RtoX_bus1 = "n:RtoX:bus1", "R/X ratio (peak)"
      n_ur_bus1 = "n:ur:bus1", "p.u., Voltage, Real Part"
      n_ui_bus1 = "n:ui:bus1", "p.u., Voltage, Imaginary Part"
      n_u_bus1 = "n:u:bus1", "p.u., Voltage, Magnitude"
      n_upc_bus1 = "n:upc:bus1", "%, Voltage, Magnitude"
      n_U_bus1 = "n:U:bus1", "kV, Line-Ground Voltage, Magnitude"
      n_Ul_bus1 = "n:Ul:bus1", "kV, Line-Line Voltage, Magnitude"
      n_phiu_bus1 = "n:phiu:bus1", "deg, Voltage, Angle"
      n_du_bus1 = "n:du:bus1", "%, Voltage Deviation"
      n_urpref_bus1 = "n:urpref:bus1", "p.u., Pre-fault Voltage, Real Part"
      n_uipref_bus1 = "n:uipref:bus1", "p.u., Pre-fault Voltage, Imaginary Part"
      n_upref_bus1 = "n:upref:bus1", "p.u., Pre-fault Voltage, Magnitude"
      n_uprefpc_bus1 = "n:uprefpc:bus1", "%, Pre-fault Voltage, Magnitude"
      n_Upref_bus1 = "n:Upref:bus1", "kV, Line-Ground Pre-fault Voltage, Magnitude"
      n_Ulpref_bus1 = "n:Ulpref:bus1", "kV, Line-Line Pre-fault Voltage, Magnitude"
      n_phiupref_bus1 = "n:phiupref:bus1", "deg, Pre-fault Voltage, Angle"
      n_phiui_bus1 = "n:phiui:bus1", "deg, Angle between Voltage and Current"
      n_cfac_bus1 = "n:cfac:bus1", "Voltage Factor"
      n_idc_bus1 = "n:idc:bus1", "kA, Decaying Component"
      n_Iasy_bus1 = "n:Iasy:bus1", "kA, Asymmetrical Breaking Current"
      n_Ithload_bus1 = "n:Ithload:bus1", "%, Loading, Thermal equivalent Current"
      n_Ipload_bus1 = "n:Ipload:bus1", "%, Loading, Peak Short-Circuit Current"
      n_Ithrtk_bus1 = "n:Ithrtk:bus1", "kA, Rated Short-Time Current (Tk)"
      n_Tn_bus1 = "n:Tn:bus1", "ms, Grid Time Constant"
      n_XtoR_b_bus1 = "n:XtoR_b:bus1", "X/R ratio (dc)"
      n_RtoX_b_bus1 = "n:RtoX_b:bus1", "R/X ratio (dc)"
      n_fctof_b_bus1 = "n:fctof_b:bus1", "Equivalent Frequency fc/f (dc)"
      c_R = "c:R", "Ohm, Resistance"
      c_X = "c:X", "Ohm, Reactance"
      c_Z = "c:Z", "Ohm, Impedance"
      c_G = "c:G", "MW, Load Conductance"
      c_B = "c:B", "Mvar, Load Susceptance"
      c_Y = "c:Y", "MVA, Load Admittance"
      m_Ikss_bus1_A = "m:Ikss:bus1:A", "kA, Initial Short-Circuit Current"
      m_Ikss_bus1_B = "m:Ikss:bus1:B", "kA, Initial Short-Circuit Current"
      m_Ikss_bus1_C = "m:Ikss:bus1:C", "kA, Initial Short-Circuit Current"
      m_ikss_bus1_A = "m:ikss:bus1:A", "p.u., Initial Short-Circuit Current, Magnitude"
      m_ikss_bus1_B = "m:ikss:bus1:B", "p.u., Initial Short-Circuit Current, Magnitude"
      m_ikss_bus1_C = "m:ikss:bus1:C", "p.u., Initial Short-Circuit Current, Magnitude"
      m_Skss_bus1_A = "m:Skss:bus1:A", "MVA, Initial Short-Circuit Power"
      m_Skss_bus1_B = "m:Skss:bus1:B", "MVA, Initial Short-Circuit Power"
      m_Skss_bus1_C = "m:Skss:bus1:C", "MVA, Initial Short-Circuit Power"
      m_Skssx3_bus1_A = "m:Skssx3:bus1:A", "MVA, 3*Initial Short-Circuit Power"
      m_Skssx3_bus1_B = "m:Skssx3:bus1:B", "MVA, 3*Initial Short-Circuit Power"
      m_Skssx3_bus1_C = "m:Skssx3:bus1:C", "MVA, 3*Initial Short-Circuit Power"
      m_ip_bus1_A = "m:ip:bus1:A", "kA, Peak Short-Circuit Current"
      m_ip_bus1_B = "m:ip:bus1:B", "kA, Peak Short-Circuit Current"
      m_ip_bus1_C = "m:ip:bus1:C", "kA, Peak Short-Circuit Current"
      m_Z_bus1_A = "m:Z:bus1:A", "Ohm, Fault Impedance, Magnitude"
      m_Z_bus1_B = "m:Z:bus1:B", "Ohm, Fault Impedance, Magnitude"
      m_Z_bus1_C = "m:Z:bus1:C", "Ohm, Fault Impedance, Magnitude"
      m_phiz_bus1_A = "m:phiz:bus1:A", "deg, Fault Impedance, Angle"
      m_phiz_bus1_B = "m:phiz:bus1:B", "deg, Fault Impedance, Angle"
      m_phiz_bus1_C = "m:phiz:bus1:C", "deg, Fault Impedance, Angle"
      m_Zl_bus1_A = "m:Zl:bus1:A", "Ohm, Line-Line Fault Impedance, Magnitude"
      m_Zl_bus1_B = "m:Zl:bus1:B", "Ohm, Line-Line Fault Impedance, Magnitude"
      m_Zl_bus1_C = "m:Zl:bus1:C", "Ohm, Line-Line Fault Impedance, Magnitude"
      m_phizl_bus1_A = "m:phizl:bus1:A", "deg, Line-Line Fault Impedance, Angle"
      m_phizl_bus1_B = "m:phizl:bus1:B", "deg, Line-Line Fault Impedance, Angle"
      m_phizl_bus1_C = "m:phizl:bus1:C", "deg, Line-Line Fault Impedance, Angle"
      m_Z0_bus1 = "m:Z0:bus1", "Ohm, Zero-Sequence Fault Impedance, Magnitude"
      m_phiz0_bus1 = "m:phiz0:bus1", "deg, Zero-Sequence Fault Impedance, Angle"
      m_z0Sbase_bus1 = "m:z0Sbase:bus1", "p.u., Zero-Sequence Fault Impedance, Magnitude"
      m_cosphiu1i1_bus1 = "m:cosphiu1i1:bus1", "Positive-Sequence cosphi"
      m_I0x3watt_bus1 = "m:I0x3watt:bus1", "kA, Earth-Fault Residual Current"
      m_I0x3var_bus1 = "m:I0x3var:bus1", "kA, Capacitive Earth-Fault Current"
      m_Q0cap_bus1 = "m:Q0cap:bus1", "Mvar, Q0 capacitive"
      m_Zfrel_bus1 = "m:Zfrel:bus1", "Ohm, Relay Impedance, Magnitude"
      m_phizfrel_bus1 = "m:phizfrel:bus1", "deg, Relay Impedance, Angle"
      m_Rfrel_bus1 = "m:Rfrel:bus1", "Ohm, Relay Impedance, Real Part"
      m_Xfrel_bus1 = "m:Xfrel:bus1", "Ohm, Relay Impedance, Imaginary Part"
      m_Zf_bus1 = "m:Zf:bus1", "Ohm, Meas. Fault Impedance, Magnitude"
      m_phizf_bus1 = "m:phizf:bus1", "deg, Meas. Fault Impedance, Angle"
      n_Ikss_bus1_A = "n:Ikss:bus1:A", "kA, Initial Short-Circuit Current"
      n_Ikss_bus1_B = "n:Ikss:bus1:B", "kA, Initial Short-Circuit Current"
      n_Ikss_bus1_C = "n:Ikss:bus1:C", "kA, Initial Short-Circuit Current"
      n_I_bus1_A = "n:I:bus1:A", "kA, Short-Circuit Current"
      n_I_bus1_B = "n:I:bus1:B", "kA, Short-Circuit Current"
      n_I_bus1_C = "n:I:bus1:C", "kA, Short-Circuit Current"
      n_phii_bus1_A = "n:phii:bus1:A", "deg, Phase Current, Angle"
      n_phii_bus1_B = "n:phii:bus1:B", "deg, Phase Current, Angle"
      n_phii_bus1_C = "n:phii:bus1:C", "deg, Phase Current, Angle"
      n_Skss_bus1_A = "n:Skss:bus1:A", "MVA, Initial Short-Circuit Power"
      n_Skss_bus1_B = "n:Skss:bus1:B", "MVA, Initial Short-Circuit Power"
      n_Skss_bus1_C = "n:Skss:bus1:C", "MVA, Initial Short-Circuit Power"
      n_Skssx3_bus1_A = "n:Skssx3:bus1:A", "MVA, 3*Initial Short-Circuit Power"
      n_Skssx3_bus1_B = "n:Skssx3:bus1:B", "MVA, 3*Initial Short-Circuit Power"
      n_Skssx3_bus1_C = "n:Skssx3:bus1:C", "MVA, 3*Initial Short-Circuit Power"
      n_ip_bus1_A = "n:ip:bus1:A", "kA, Peak Short-Circuit Current"
      n_ip_bus1_B = "n:ip:bus1:B", "kA, Peak Short-Circuit Current"
      n_ip_bus1_C = "n:ip:bus1:C", "kA, Peak Short-Circuit Current"
      n_Ib_bus1_A = "n:Ib:bus1:A", "kA, Short-Circuit Breaking Current"
      n_Ib_bus1_B = "n:Ib:bus1:B", "kA, Short-Circuit Breaking Current"
      n_Ib_bus1_C = "n:Ib:bus1:C", "kA, Short-Circuit Breaking Current"
      n_Sb_bus1_A = "n:Sb:bus1:A", "MVA, Short-Circuit Breaking Power"
      n_Sb_bus1_B = "n:Sb:bus1:B", "MVA, Short-Circuit Breaking Power"
      n_Sb_bus1_C = "n:Sb:bus1:C", "MVA, Short-Circuit Breaking Power"
      n_Ith_bus1_A = "n:Ith:bus1:A", "kA, Thermal equivalent Short-Circuit Current"
      n_Ith_bus1_B = "n:Ith:bus1:B", "kA, Thermal equivalent Short-Circuit Current"
      n_Ith_bus1_C = "n:Ith:bus1:C", "kA, Thermal equivalent Short-Circuit Current"
      n_Ithmax_bus1 = "n:Ithmax:bus1", "kA, Max. Thermal equivalent Short-Circuit Current"
      n_Ik_bus1_A = "n:Ik:bus1:A", "kA, Steady-State Short-Circuit Current"
      n_Ik_bus1_B = "n:Ik:bus1:B", "kA, Steady-State Short-Circuit Current"
      n_Ik_bus1_C = "n:Ik:bus1:C", "kA, Steady-State Short-Circuit Current"
      n_R0_bus1 = "n:R0:bus1", "Ohm, Zero-Sequence Impedance, Real Part"
      n_X0_bus1 = "n:X0:bus1", "Ohm, Zero-Sequence Impedance, Imaginary Part"
      n_Z0_bus1 = "n:Z0:bus1", "Ohm, Zero-Sequence Impedance, Magnitude"
      n_phiz0_bus1 = "n:phiz0:bus1", "deg, Zero-Sequence Impedance, Angle"
      n_r0Sbase_bus1 = "n:r0Sbase:bus1", "p.u., Zero-Sequence Impedance, Real Part"
      n_x0Sbase_bus1 = "n:x0Sbase:bus1", "p.u., Zero-Sequence Impedance, Imaginary Part"
      n_z0Sbase_bus1 = "n:z0Sbase:bus1", "p.u., Zero-Sequence Impedance, Magnitude"
      n_R1_bus1 = "n:R1:bus1", "Ohm, Positive-Sequence Impedance, Real Part"
      n_X1_bus1 = "n:X1:bus1", "Ohm, Positive-Sequence Impedance, Imaginary Part"
      n_Z1_bus1 = "n:Z1:bus1", "Ohm, Positive-Sequence Impedance, Magnitude"
      n_phiz1_bus1 = "n:phiz1:bus1", "deg, Positive-Sequence Impedance, Angle"
      n_r1Sbase_bus1 = "n:r1Sbase:bus1", "p.u., Positive-Sequence Impedance, Real Part"
      n_x1Sbase_bus1 = "n:x1Sbase:bus1", "p.u., Positive-Sequence Impedance, Imaginary Part"
      n_z1Sbase_bus1 = "n:z1Sbase:bus1", "p.u., Positive-Sequence Impedance, Magnitude"
      n_R2_bus1 = "n:R2:bus1", "Ohm, Negative-Sequence Impedance, Real Part"
      n_X2_bus1 = "n:X2:bus1", "Ohm, Negative-Sequence Impedance, Imaginary Part"
      n_Z2_bus1 = "n:Z2:bus1", "Ohm, Negative-Sequence Impedance, Magnitude"
      n_phiz2_bus1 = "n:phiz2:bus1", "deg, Negative-Sequence Impedance, Angle"
      n_r2Sbase_bus1 = "n:r2Sbase:bus1", "p.u., Negative-Sequence Impedance, Real Part"
      n_x2Sbase_bus1 = "n:x2Sbase:bus1", "p.u., Negative-Sequence Impedance, Imaginary Part"
      n_z2Sbase_bus1 = "n:z2Sbase:bus1", "p.u., Negative-Sequence Impedance, Magnitude"
      n_R_bus1_A = "n:R:bus1:A", "Ohm, Network Resistance, Line-Ground"
      n_R_bus1_B = "n:R:bus1:B", "Ohm, Network Resistance, Line-Ground"
      n_R_bus1_C = "n:R:bus1:C", "Ohm, Network Resistance, Line-Ground"
      n_X_bus1_A = "n:X:bus1:A", "Ohm, Network Reactance, Line-Ground"
      n_X_bus1_B = "n:X:bus1:B", "Ohm, Network Reactance, Line-Ground"
      n_X_bus1_C = "n:X:bus1:C", "Ohm, Network Reactance, Line-Ground"
      n_Z_bus1_A = "n:Z:bus1:A", "Ohm, Network Impedance, Line-Ground"
      n_Z_bus1_B = "n:Z:bus1:B", "Ohm, Network Impedance, Line-Ground"
      n_Z_bus1_C = "n:Z:bus1:C", "Ohm, Network Impedance, Line-Ground"
      n_phiz_bus1_A = "n:phiz:bus1:A", "deg, Angle of Network Impedance, Line-Ground"
      n_phiz_bus1_B = "n:phiz:bus1:B", "deg, Angle of Network Impedance, Line-Ground"
      n_phiz_bus1_C = "n:phiz:bus1:C", "deg, Angle of Network Impedance, Line-Ground"
      n_Rl_bus1_A = "n:Rl:bus1:A", "Ohm, Network Resistance, Line-Line"
      n_Rl_bus1_B = "n:Rl:bus1:B", "Ohm, Network Resistance, Line-Line"
      n_Rl_bus1_C = "n:Rl:bus1:C", "Ohm, Network Resistance, Line-Line"
      n_Xl_bus1_A = "n:Xl:bus1:A", "Ohm, Network Reactance, Line-Line"
      n_Xl_bus1_B = "n:Xl:bus1:B", "Ohm, Network Reactance, Line-Line"
      n_Xl_bus1_C = "n:Xl:bus1:C", "Ohm, Network Reactance, Line-Line"
      n_Zl_bus1_A = "n:Zl:bus1:A", "Ohm, Network Impedance, Line-Line"
      n_Zl_bus1_B = "n:Zl:bus1:B", "Ohm, Network Impedance, Line-Line"
      n_Zl_bus1_C = "n:Zl:bus1:C", "Ohm, Network Impedance, Line-Line"
      n_phizl_bus1_A = "n:phizl:bus1:A", "deg, Angle of Network Impedance, Line-Line"
      n_phizl_bus1_B = "n:phizl:bus1:B", "deg, Angle of Network Impedance, Line-Line"
      n_phizl_bus1_C = "n:phizl:bus1:C", "deg, Angle of Network Impedance, Line-Line"
      n_Rln_bus1_A = "n:Rln:bus1:A", "Ohm, Network Resistance, Line-Neutral"
      n_Rln_bus1_B = "n:Rln:bus1:B", "Ohm, Network Resistance, Line-Neutral"
      n_Rln_bus1_C = "n:Rln:bus1:C", "Ohm, Network Resistance, Line-Neutral"
      n_Xln_bus1_A = "n:Xln:bus1:A", "Ohm, Network Reactance, Line-Neutral"
      n_Xln_bus1_B = "n:Xln:bus1:B", "Ohm, Network Reactance, Line-Neutral"
      n_Xln_bus1_C = "n:Xln:bus1:C", "Ohm, Network Reactance, Line-Neutral"
      n_Zln_bus1_A = "n:Zln:bus1:A", "Ohm, Network Impedance, Line-Neutral"
      n_Zln_bus1_B = "n:Zln:bus1:B", "Ohm, Network Impedance, Line-Neutral"
      n_Zln_bus1_C = "n:Zln:bus1:C", "Ohm, Network Impedance, Line-Neutral"
      n_phizln_bus1_A = "n:phizln:bus1:A", "deg, Angle of Network Impedance, Line-Neutral"
      n_phizln_bus1_B = "n:phizln:bus1:B", "deg, Angle of Network Impedance, Line-Neutral"
      n_phizln_bus1_C = "n:phizln:bus1:C", "deg, Angle of Network Impedance, Line-Neutral"
      n_Rn_bus1 = "n:Rn:bus1", "Ohm, Network Resistance, Neutral"
      n_Xn_bus1 = "n:Xn:bus1", "Ohm, Network Reactance, Neutral"
      n_Zn_bus1 = "n:Zn:bus1", "Ohm, Network Impedance, Neutral"
      n_phizn_bus1 = "n:phizn:bus1", "deg, Angle of Network Impedance, Neutral"
      n_eff_bus1_A = "n:eff:bus1:A", "Earth Fault Factor"
      n_eff_bus1_B = "n:eff:bus1:B", "Earth Fault Factor"
      n_eff_bus1_C = "n:eff:bus1:C", "Earth Fault Factor"
      n_X0toX1_bus1 = "n:X0toX1:bus1", "X0/X1"
      n_in_bus1 = "n:in:bus1", "p.u., Neutral Current, Magnitude"
      n_In_bus1 = "n:In:bus1", "kA, Neutral Current, Magnitude"
      n_phiin_bus1 = "n:phiin:bus1", "deg, Neutral Current, Angle"
      n_i0r_bus1 = "n:i0r:bus1", "p.u., Zero-Sequence Current, Real Part"
      n_i0i_bus1 = "n:i0i:bus1", "p.u., Zero-Sequence Current, Imaginary Part"
      n_i0_bus1 = "n:i0:bus1", "p.u., Zero-Sequence Current, Magnitude"
      n_I0_bus1 = "n:I0:bus1", "kA, Zero-Sequence Current, Magnitude"
      n_I0x3_bus1 = "n:I0x3:bus1", "kA, 3*I0"
      n_phii0_bus1 = "n:phii0:bus1", "deg, Zero-Sequence Current, Angle"
      n_i1r_bus1 = "n:i1r:bus1", "p.u., Positive-Sequence Current, Real Part"
      n_i1i_bus1 = "n:i1i:bus1", "p.u., Positive-Sequence Current, Imaginary Part"
      n_i1_bus1 = "n:i1:bus1", "p.u., Positive-Sequence Current, Magnitude"
      n_I1_bus1 = "n:I1:bus1", "kA, Positive-Sequence Current, Magnitude"
      n_phii1_bus1 = "n:phii1:bus1", "deg, Positive-Sequence Current, Angle"
      n_i2r_bus1 = "n:i2r:bus1", "p.u., Negative-Sequence Current, Real Part"
      n_i2i_bus1 = "n:i2i:bus1", "p.u., Negative-Sequence Current, Imaginary Part"
      n_i2_bus1 = "n:i2:bus1", "p.u., Negative-Sequence Current, Magnitude"
      n_I2_bus1 = "n:I2:bus1", "kA, Negative-Sequence Current, Magnitude"
      n_phii2_bus1 = "n:phii2:bus1", "deg, Negative-Sequence Current, Angle"
      n_du_bus1_A = "n:du:bus1:A", "%, Voltage Deviation"
      n_du_bus1_B = "n:du:bus1:B", "%, Voltage Deviation"
      n_du_bus1_C = "n:du:bus1:C", "%, Voltage Deviation"
      n_urpref_bus1_A = "n:urpref:bus1:A", "p.u., Pre-fault Voltage, Real Part"
      n_urpref_bus1_B = "n:urpref:bus1:B", "p.u., Pre-fault Voltage, Real Part"
      n_urpref_bus1_C = "n:urpref:bus1:C", "p.u., Pre-fault Voltage, Real Part"
      n_uipref_bus1_A = "n:uipref:bus1:A", "p.u., Pre-fault Voltage, Imaginary Part"
      n_uipref_bus1_B = "n:uipref:bus1:B", "p.u., Pre-fault Voltage, Imaginary Part"
      n_uipref_bus1_C = "n:uipref:bus1:C", "p.u., Pre-fault Voltage, Imaginary Part"
      n_upref_bus1_A = "n:upref:bus1:A", "p.u., Pre-fault Voltage, Magnitude"
      n_upref_bus1_B = "n:upref:bus1:B", "p.u., Pre-fault Voltage, Magnitude"
      n_upref_bus1_C = "n:upref:bus1:C", "p.u., Pre-fault Voltage, Magnitude"
      n_uprefpc_bus1_A = "n:uprefpc:bus1:A", "%, Pre-fault Voltage, Magnitude"
      n_uprefpc_bus1_B = "n:uprefpc:bus1:B", "%, Pre-fault Voltage, Magnitude"
      n_uprefpc_bus1_C = "n:uprefpc:bus1:C", "%, Pre-fault Voltage, Magnitude"
      n_Upref_bus1_A = "n:Upref:bus1:A", "kV, Line-Ground Pre-fault Voltage, Magnitude"
      n_Upref_bus1_B = "n:Upref:bus1:B", "kV, Line-Ground Pre-fault Voltage, Magnitude"
      n_Upref_bus1_C = "n:Upref:bus1:C", "kV, Line-Ground Pre-fault Voltage, Magnitude"
      n_Ulpref_bus1_A = "n:Ulpref:bus1:A", "kV, Line-Line Pre-fault Voltage, Magnitude"
      n_Ulpref_bus1_B = "n:Ulpref:bus1:B", "kV, Line-Line Pre-fault Voltage, Magnitude"
      n_Ulpref_bus1_C = "n:Ulpref:bus1:C", "kV, Line-Line Pre-fault Voltage, Magnitude"
      n_phiupref_bus1_A = "n:phiupref:bus1:A", "deg, Pre-fault Voltage, Angle"
      n_phiupref_bus1_B = "n:phiupref:bus1:B", "deg, Pre-fault Voltage, Angle"
      n_phiupref_bus1_C = "n:phiupref:bus1:C", "deg, Pre-fault Voltage, Angle"
      n_phiui_bus1_A = "n:phiui:bus1:A", "deg, Angle between Voltage and Current"
      n_phiui_bus1_B = "n:phiui:bus1:B", "deg, Angle between Voltage and Current"
      n_phiui_bus1_C = "n:phiui:bus1:C", "deg, Angle between Voltage and Current"
      n_phiu1i1_bus1 = "n:phiu1i1:bus1", "deg, Angle between Voltage and Current in positive sequence system"
      n_phiu2i2_bus1 = "n:phiu2i2:bus1", "deg, Angle between Voltage and Current in negative sequence system"
      n_phiu0i0_bus1 = "n:phiu0i0:bus1", "deg, Angle between Voltage and Current in zero-sequence system"
      c_R1 = "c:R1", "Ohm, Resistance"
      c_X1 = "c:X1", "Ohm, Reactance"
      c_Z1 = "c:Z1", "Ohm, Impedance"
      c_G1 = "c:G1", "MW, Load Conductance"
      c_B1 = "c:B1", "Mvar, Load Susceptance"
      c_Y1 = "c:Y1", "MVA, Load Admittance"

    class ElmVsc(Enum):
      _init_ = 'value __doc__'

      m_u0_busac = "m:u0:busac", "p.u., Zero-Sequence-Voltage, Magnitude"
      m_u0_busdp = "m:u0:busdp", "p.u., Zero-Sequence-Voltage, Magnitude"
      m_u0_busdm = "m:u0:busdm", "p.u., Zero-Sequence-Voltage, Magnitude"
      m_phiu0_busac = "m:phiu0:busac", "deg, Zero-Sequence-Voltage, Angle"
      m_phiu0_busdp = "m:phiu0:busdp", "deg, Zero-Sequence-Voltage, Angle"
      m_phiu0_busdm = "m:phiu0:busdm", "deg, Zero-Sequence-Voltage, Angle"
      m_u1r_busac = "m:u1r:busac", "p.u., Positive-Sequence-Voltage, Real Part"
      m_u1r_busdp = "m:u1r:busdp", "p.u., Positive-Sequence-Voltage, Real Part"
      m_u1r_busdm = "m:u1r:busdm", "p.u., Positive-Sequence-Voltage, Real Part"
      m_u1i_busac = "m:u1i:busac", "p.u., Positive-Sequence-Voltage, Imaginary Part"
      m_u1i_busdp = "m:u1i:busdp", "p.u., Positive-Sequence-Voltage, Imaginary Part"
      m_u1i_busdm = "m:u1i:busdm", "p.u., Positive-Sequence-Voltage, Imaginary Part"
      m_u1_busac = "m:u1:busac", "p.u., Positive-Sequence-Voltage, Magnitude"
      m_u1_busdp = "m:u1:busdp", "p.u., Positive-Sequence-Voltage, Magnitude"
      m_u1_busdm = "m:u1:busdm", "p.u., Positive-Sequence-Voltage, Magnitude"
      m_U1_busac = "m:U1:busac", "kV, Line-Ground Positive-Sequence-Voltage, Magnitude"
      m_U1_busdp = "m:U1:busdp", "kV, Line-Ground Positive-Sequence-Voltage, Magnitude"
      m_U1_busdm = "m:U1:busdm", "kV, Line-Ground Positive-Sequence-Voltage, Magnitude"
      m_U1l_busac = "m:U1l:busac", "kV, Line-Line Positive-Sequence-Voltage, Magnitude"
      m_U1l_busdp = "m:U1l:busdp", "kV, Line-Line Positive-Sequence-Voltage, Magnitude"
      m_U1l_busdm = "m:U1l:busdm", "kV, Line-Line Positive-Sequence-Voltage, Magnitude"
      m_phiu1_busac = "m:phiu1:busac", "deg, Positive-Sequence-Voltage, Angle"
      m_phiu1_busdp = "m:phiu1:busdp", "deg, Positive-Sequence-Voltage, Angle"
      m_phiu1_busdm = "m:phiu1:busdm", "deg, Positive-Sequence-Voltage, Angle"
      m_u2_busac = "m:u2:busac", "p.u., Negative-Sequence-Voltage, Magnitude"
      m_u2_busdp = "m:u2:busdp", "p.u., Negative-Sequence-Voltage, Magnitude"
      m_u2_busdm = "m:u2:busdm", "p.u., Negative-Sequence-Voltage, Magnitude"
      m_phiu2_busac = "m:phiu2:busac", "deg, Negative-Sequence-Voltage, Angle"
      m_phiu2_busdp = "m:phiu2:busdp", "deg, Negative-Sequence-Voltage, Angle"
      m_phiu2_busdm = "m:phiu2:busdm", "deg, Negative-Sequence-Voltage, Angle"
      m_inet_busac_A = "m:inet:busac:A", "p.u., Phase Current, Magnitude, referred to network"
      m_inet_busac_B = "m:inet:busac:B", "p.u., Phase Current, Magnitude, referred to network"
      m_inet_busac_C = "m:inet:busac:C", "p.u., Phase Current, Magnitude, referred to network"
      m_inet_busdp = "m:inet:busdp", "p.u., Phase Current, Magnitude, referred to network"
      m_inet_busdm = "m:inet:busdm", "p.u., Phase Current, Magnitude, referred to network"
      m_I_busac_A = "m:I:busac:A", "kA, Phase Current, Magnitude"
      m_I_busac_B = "m:I:busac:B", "kA, Phase Current, Magnitude"
      m_I_busac_C = "m:I:busac:C", "kA, Phase Current, Magnitude"
      m_I_busdp = "m:I:busdp", "kA, Phase Current, Magnitude"
      m_I_busdm = "m:I:busdm", "kA, Phase Current, Magnitude"
      m_phii_busac_A = "m:phii:busac:A", "deg, Phase Current, Angle"
      m_phii_busac_B = "m:phii:busac:B", "deg, Phase Current, Angle"
      m_phii_busac_C = "m:phii:busac:C", "deg, Phase Current, Angle"
      m_phii_busdp = "m:phii:busdp", "deg, Phase Current, Angle"
      m_phii_busdm = "m:phii:busdm", "deg, Phase Current, Angle"
      m_i0r_busac = "m:i0r:busac", "p.u., Zero-Sequence Current, Real Part"
      m_i0r_busdp = "m:i0r:busdp", "p.u., Zero-Sequence Current, Real Part"
      m_i0r_busdm = "m:i0r:busdm", "p.u., Zero-Sequence Current, Real Part"
      m_i0i_busac = "m:i0i:busac", "p.u., Zero-Sequence Current, Imaginary Part"
      m_i0i_busdp = "m:i0i:busdp", "p.u., Zero-Sequence Current, Imaginary Part"
      m_i0i_busdm = "m:i0i:busdm", "p.u., Zero-Sequence Current, Imaginary Part"
      m_i0_busac = "m:i0:busac", "p.u., Zero-Sequence Current, Magnitude"
      m_i0_busdp = "m:i0:busdp", "p.u., Zero-Sequence Current, Magnitude"
      m_i0_busdm = "m:i0:busdm", "p.u., Zero-Sequence Current, Magnitude"
      m_I0_busac = "m:I0:busac", "kA, Zero-Sequence Current, Magnitude"
      m_I0_busdp = "m:I0:busdp", "kA, Zero-Sequence Current, Magnitude"
      m_I0_busdm = "m:I0:busdm", "kA, Zero-Sequence Current, Magnitude"
      m_I0x3_busac = "m:I0x3:busac", "kA, 3*I0"
      m_I0x3_busdp = "m:I0x3:busdp", "kA, 3*I0"
      m_I0x3_busdm = "m:I0x3:busdm", "kA, 3*I0"
      m_phii0_busac = "m:phii0:busac", "deg, Zero-Sequence Current, Angle"
      m_phii0_busdp = "m:phii0:busdp", "deg, Zero-Sequence Current, Angle"
      m_phii0_busdm = "m:phii0:busdm", "deg, Zero-Sequence Current, Angle"
      m_i1r_busac = "m:i1r:busac", "p.u., Positive-Sequence Current, Real Part"
      m_i1r_busdp = "m:i1r:busdp", "p.u., Positive-Sequence Current, Real Part"
      m_i1r_busdm = "m:i1r:busdm", "p.u., Positive-Sequence Current, Real Part"
      m_i1i_busac = "m:i1i:busac", "p.u., Positive-Sequence Current, Imaginary Part"
      m_i1i_busdp = "m:i1i:busdp", "p.u., Positive-Sequence Current, Imaginary Part"
      m_i1i_busdm = "m:i1i:busdm", "p.u., Positive-Sequence Current, Imaginary Part"
      m_i1_busac = "m:i1:busac", "p.u., Positive-Sequence Current, Magnitude"
      m_i1_busdp = "m:i1:busdp", "p.u., Positive-Sequence Current, Magnitude"
      m_i1_busdm = "m:i1:busdm", "p.u., Positive-Sequence Current, Magnitude"
      m_I1_busac = "m:I1:busac", "kA, Positive-Sequence Current, Magnitude"
      m_I1_busdp = "m:I1:busdp", "kA, Positive-Sequence Current, Magnitude"
      m_I1_busdm = "m:I1:busdm", "kA, Positive-Sequence Current, Magnitude"
      m_phii1_busac = "m:phii1:busac", "deg, Positive-Sequence Current, Angle"
      m_phii1_busdp = "m:phii1:busdp", "deg, Positive-Sequence Current, Angle"
      m_phii1_busdm = "m:phii1:busdm", "deg, Positive-Sequence Current, Angle"
      m_i2r_busac = "m:i2r:busac", "p.u., Negative-Sequence Current, Real Part"
      m_i2r_busdp = "m:i2r:busdp", "p.u., Negative-Sequence Current, Real Part"
      m_i2r_busdm = "m:i2r:busdm", "p.u., Negative-Sequence Current, Real Part"
      m_i2i_busac = "m:i2i:busac", "p.u., Negative-Sequence Current, Imaginary Part"
      m_i2i_busdp = "m:i2i:busdp", "p.u., Negative-Sequence Current, Imaginary Part"
      m_i2i_busdm = "m:i2i:busdm", "p.u., Negative-Sequence Current, Imaginary Part"
      m_i2_busac = "m:i2:busac", "p.u., Negative-Sequence Current, Magnitude"
      m_i2_busdp = "m:i2:busdp", "p.u., Negative-Sequence Current, Magnitude"
      m_i2_busdm = "m:i2:busdm", "p.u., Negative-Sequence Current, Magnitude"
      m_I2_busac = "m:I2:busac", "kA, Negative-Sequence Current, Magnitude"
      m_I2_busdp = "m:I2:busdp", "kA, Negative-Sequence Current, Magnitude"
      m_I2_busdm = "m:I2:busdm", "kA, Negative-Sequence Current, Magnitude"
      m_phii2_busac = "m:phii2:busac", "deg, Negative-Sequence Current, Angle"
      m_phii2_busdp = "m:phii2:busdp", "deg, Negative-Sequence Current, Angle"
      m_phii2_busdm = "m:phii2:busdm", "deg, Negative-Sequence Current, Angle"
      m_ubfacI_busac = "m:ubfacI:busac", "%, Current Unbalance Factor"
      m_ubfacI_busdp = "m:ubfacI:busdp", "%, Current Unbalance Factor"
      m_ubfacI_busdm = "m:ubfacI:busdm", "%, Current Unbalance Factor"
      m_ubfacS_busac = "m:ubfacS:busac", "%, Power Unbalance Factor"
      m_ubfacS_busdp = "m:ubfacS:busdp", "%, Power Unbalance Factor"
      m_ubfacS_busdm = "m:ubfacS:busdm", "%, Power Unbalance Factor"
      m_P_busac_A = "m:P:busac:A", "MW, Active Power"
      m_P_busac_B = "m:P:busac:B", "MW, Active Power"
      m_P_busac_C = "m:P:busac:C", "MW, Active Power"
      m_P_busdp = "m:P:busdp", "MW, Active Power"
      m_P_busdm = "m:P:busdm", "MW, Active Power"
      m_Q_busac_A = "m:Q:busac:A", "Mvar, Reactive Power"
      m_Q_busac_B = "m:Q:busac:B", "Mvar, Reactive Power"
      m_Q_busac_C = "m:Q:busac:C", "Mvar, Reactive Power"
      m_Q_busdp = "m:Q:busdp", "Mvar, Reactive Power"
      m_Q_busdm = "m:Q:busdm", "Mvar, Reactive Power"
      m_S_busac_A = "m:S:busac:A", "MVA, Apparent Power"
      m_S_busac_B = "m:S:busac:B", "MVA, Apparent Power"
      m_S_busac_C = "m:S:busac:C", "MVA, Apparent Power"
      m_S_busdp = "m:S:busdp", "MVA, Apparent Power"
      m_S_busdm = "m:S:busdm", "MVA, Apparent Power"
      m_cosphi_busac_A = "m:cosphi:busac:A", "Power Factor"
      m_cosphi_busac_B = "m:cosphi:busac:B", "Power Factor"
      m_cosphi_busac_C = "m:cosphi:busac:C", "Power Factor"
      m_cosphi_busdp = "m:cosphi:busdp", "Power Factor"
      m_cosphi_busdm = "m:cosphi:busdm", "Power Factor"
      m_tanphi_busac_A = "m:tanphi:busac:A", "tan(phi)"
      m_tanphi_busac_B = "m:tanphi:busac:B", "tan(phi)"
      m_tanphi_busac_C = "m:tanphi:busac:C", "tan(phi)"
      m_tanphi_busdp = "m:tanphi:busdp", "tan(phi)"
      m_tanphi_busdm = "m:tanphi:busdm", "tan(phi)"
      m_Psum_busac = "m:Psum:busac", "MW, Total Active Power"
      m_Psum_busdp = "m:Psum:busdp", "MW, Total Active Power"
      m_Psum_busdm = "m:Psum:busdm", "MW, Total Active Power"
      m_Qsum_busac = "m:Qsum:busac", "Mvar, Total Reactive Power"
      m_Qsum_busdp = "m:Qsum:busdp", "Mvar, Total Reactive Power"
      m_Qsum_busdm = "m:Qsum:busdm", "Mvar, Total Reactive Power"
      m_Ssum_busac = "m:Ssum:busac", "MVA, Total Apparent Power"
      m_Ssum_busdp = "m:Ssum:busdp", "MVA, Total Apparent Power"
      m_Ssum_busdm = "m:Ssum:busdm", "MVA, Total Apparent Power"
      m_cosphisum_busac = "m:cosphisum:busac", "Total Power Factor"
      m_cosphisum_busdp = "m:cosphisum:busdp", "Total Power Factor"
      m_cosphisum_busdm = "m:cosphisum:busdm", "Total Power Factor"
      m_tanphisum_busac = "m:tanphisum:busac", "Total tan(phi)"
      m_tanphisum_busdp = "m:tanphisum:busdp", "Total tan(phi)"
      m_tanphisum_busdm = "m:tanphisum:busdm", "Total tan(phi)"
      m_Pln_busac_A = "m:Pln:busac:A", "MW, Active Power (Line-Neutral)"
      m_Pln_busac_B = "m:Pln:busac:B", "MW, Active Power (Line-Neutral)"
      m_Pln_busac_C = "m:Pln:busac:C", "MW, Active Power (Line-Neutral)"
      m_Pln_busdp = "m:Pln:busdp", "MW, Active Power (Line-Neutral)"
      m_Pln_busdm = "m:Pln:busdm", "MW, Active Power (Line-Neutral)"
      m_Qln_busac_A = "m:Qln:busac:A", "Mvar, Reactive Power (Line-Neutral)"
      m_Qln_busac_B = "m:Qln:busac:B", "Mvar, Reactive Power (Line-Neutral)"
      m_Qln_busac_C = "m:Qln:busac:C", "Mvar, Reactive Power (Line-Neutral)"
      m_Qln_busdp = "m:Qln:busdp", "Mvar, Reactive Power (Line-Neutral)"
      m_Qln_busdm = "m:Qln:busdm", "Mvar, Reactive Power (Line-Neutral)"
      m_P1_busac = "m:P1:busac", "MW, Positive Sequence Active Power"
      m_P1_busdp = "m:P1:busdp", "MW, Positive Sequence Active Power"
      m_P1_busdm = "m:P1:busdm", "MW, Positive Sequence Active Power"
      m_Q1_busac = "m:Q1:busac", "Mvar, Positive Sequence Reactive Power"
      m_Q1_busdp = "m:Q1:busdp", "Mvar, Positive Sequence Reactive Power"
      m_Q1_busdm = "m:Q1:busdm", "Mvar, Positive Sequence Reactive Power"
      m_P2_busac = "m:P2:busac", "MW, Negative Sequence Active Power"
      m_P2_busdp = "m:P2:busdp", "MW, Negative Sequence Active Power"
      m_P2_busdm = "m:P2:busdm", "MW, Negative Sequence Active Power"
      m_Q2_busac = "m:Q2:busac", "Mvar, Negative Sequence Reactive Power"
      m_Q2_busdp = "m:Q2:busdp", "Mvar, Negative Sequence Reactive Power"
      m_Q2_busdm = "m:Q2:busdm", "Mvar, Negative Sequence Reactive Power"
      m_P0_busac = "m:P0:busac", "MW, Zero Sequence Active Power"
      m_P0_busdp = "m:P0:busdp", "MW, Zero Sequence Active Power"
      m_P0_busdm = "m:P0:busdm", "MW, Zero Sequence Active Power"
      m_Q0_busac = "m:Q0:busac", "Mvar, Zero Sequence Reactive Power"
      m_Q0_busdp = "m:Q0:busdp", "Mvar, Zero Sequence Reactive Power"
      m_Q0_busdm = "m:Q0:busdm", "Mvar, Zero Sequence Reactive Power"
      m_i1P_busac = "m:i1P:busac", "p.u., Positive-Sequence Active Current"
      m_i1P_busdp = "m:i1P:busdp", "p.u., Positive-Sequence Active Current"
      m_i1P_busdm = "m:i1P:busdm", "p.u., Positive-Sequence Active Current"
      m_i1Q_busac = "m:i1Q:busac", "p.u., Positive-Sequence Reactive Current"
      m_i1Q_busdp = "m:i1Q:busdp", "p.u., Positive-Sequence Reactive Current"
      m_i1Q_busdm = "m:i1Q:busdm", "p.u., Positive-Sequence Reactive Current"
      m_I1P_busac = "m:I1P:busac", "kA, Positive-Sequence Active Current"
      m_I1P_busdp = "m:I1P:busdp", "kA, Positive-Sequence Active Current"
      m_I1P_busdm = "m:I1P:busdm", "kA, Positive-Sequence Active Current"
      m_I1Q_busac = "m:I1Q:busac", "kA, Positive-Sequence Reactive Current"
      m_I1Q_busdp = "m:I1Q:busdp", "kA, Positive-Sequence Reactive Current"
      m_I1Q_busdm = "m:I1Q:busdm", "kA, Positive-Sequence Reactive Current"
      m_i2P_busac = "m:i2P:busac", "p.u., Negative-Sequence Active Current"
      m_i2P_busdp = "m:i2P:busdp", "p.u., Negative-Sequence Active Current"
      m_i2P_busdm = "m:i2P:busdm", "p.u., Negative-Sequence Active Current"
      m_i2Q_busac = "m:i2Q:busac", "p.u., Negative-Sequence Reactive Current"
      m_i2Q_busdp = "m:i2Q:busdp", "p.u., Negative-Sequence Reactive Current"
      m_i2Q_busdm = "m:i2Q:busdm", "p.u., Negative-Sequence Reactive Current"
      m_I2P_busac = "m:I2P:busac", "kA, Negative-Sequence Active Current"
      m_I2P_busdp = "m:I2P:busdp", "kA, Negative-Sequence Active Current"
      m_I2P_busdm = "m:I2P:busdm", "kA, Negative-Sequence Active Current"
      m_I2Q_busac = "m:I2Q:busac", "kA, Negative-Sequence Reactive Current"
      m_I2Q_busdp = "m:I2Q:busdp", "kA, Negative-Sequence Reactive Current"
      m_I2Q_busdm = "m:I2Q:busdm", "kA, Negative-Sequence Reactive Current"
      m_ur_busac_A = "m:ur:busac:A", "p.u., Phase Voltage, Real Part"
      m_ur_busac_B = "m:ur:busac:B", "p.u., Phase Voltage, Real Part"
      m_ur_busac_C = "m:ur:busac:C", "p.u., Phase Voltage, Real Part"
      m_ur_busdp = "m:ur:busdp", "p.u., Phase Voltage, Real Part"
      m_ur_busdm = "m:ur:busdm", "p.u., Phase Voltage, Real Part"
      m_ui_busac_A = "m:ui:busac:A", "p.u., Phase Voltage, Imaginary Part"
      m_ui_busac_B = "m:ui:busac:B", "p.u., Phase Voltage, Imaginary Part"
      m_ui_busac_C = "m:ui:busac:C", "p.u., Phase Voltage, Imaginary Part"
      m_ui_busdp = "m:ui:busdp", "p.u., Phase Voltage, Imaginary Part"
      m_ui_busdm = "m:ui:busdm", "p.u., Phase Voltage, Imaginary Part"
      m_u_busac_A = "m:u:busac:A", "p.u., Phase Voltage, Magnitude"
      m_u_busac_B = "m:u:busac:B", "p.u., Phase Voltage, Magnitude"
      m_u_busac_C = "m:u:busac:C", "p.u., Phase Voltage, Magnitude"
      m_u_busdp = "m:u:busdp", "p.u., Phase Voltage, Magnitude"
      m_u_busdm = "m:u:busdm", "p.u., Phase Voltage, Magnitude"
      m_ir_busac_A = "m:ir:busac:A", "p.u., Phase Current, Real Part"
      m_ir_busac_B = "m:ir:busac:B", "p.u., Phase Current, Real Part"
      m_ir_busac_C = "m:ir:busac:C", "p.u., Phase Current, Real Part"
      m_ir_busdp = "m:ir:busdp", "p.u., Phase Current, Real Part"
      m_ir_busdm = "m:ir:busdm", "p.u., Phase Current, Real Part"
      m_ii_busac_A = "m:ii:busac:A", "p.u., Phase Current, Imaginary Part"
      m_ii_busac_B = "m:ii:busac:B", "p.u., Phase Current, Imaginary Part"
      m_ii_busac_C = "m:ii:busac:C", "p.u., Phase Current, Imaginary Part"
      m_ii_busdp = "m:ii:busdp", "p.u., Phase Current, Imaginary Part"
      m_ii_busdm = "m:ii:busdm", "p.u., Phase Current, Imaginary Part"
      m_i_busac_A = "m:i:busac:A", "p.u., Phase Current, Magnitude"
      m_i_busac_B = "m:i:busac:B", "p.u., Phase Current, Magnitude"
      m_i_busac_C = "m:i:busac:C", "p.u., Phase Current, Magnitude"
      m_i_busdp = "m:i:busdp", "p.u., Phase Current, Magnitude"
      m_i_busdm = "m:i:busdm", "p.u., Phase Current, Magnitude"
      m_phiui_busac_A = "m:phiui:busac:A", "deg, Angle between Voltage and Current"
      m_phiui_busac_B = "m:phiui:busac:B", "deg, Angle between Voltage and Current"
      m_phiui_busac_C = "m:phiui:busac:C", "deg, Angle between Voltage and Current"
      m_phiui_busdp = "m:phiui:busdp", "deg, Angle between Voltage and Current"
      m_phiui_busdm = "m:phiui:busdm", "deg, Angle between Voltage and Current"
      m_phiu0i0_busac = "m:phiu0i0:busac", "deg, Angle between Voltage and Current in zero-sequence system"
      m_phiu0i0_busdp = "m:phiu0i0:busdp", "deg, Angle between Voltage and Current in zero-sequence system"
      m_phiu0i0_busdm = "m:phiu0i0:busdm", "deg, Angle between Voltage and Current in zero-sequence system"
      m_phiu1i1_busac = "m:phiu1i1:busac", "deg, Angle between Voltage and Current in positive sequence system"
      m_phiu1i1_busdp = "m:phiu1i1:busdp", "deg, Angle between Voltage and Current in positive sequence system"
      m_phiu1i1_busdm = "m:phiu1i1:busdm", "deg, Angle between Voltage and Current in positive sequence system"
      m_phiu2i2_busac = "m:phiu2i2:busac", "deg, Angle between Voltage and Current in negative sequence system"
      m_phiu2i2_busdp = "m:phiu2i2:busdp", "deg, Angle between Voltage and Current in negative sequence system"
      m_phiu2i2_busdm = "m:phiu2i2:busdm", "deg, Angle between Voltage and Current in negative sequence system"
      m_Tfct_busac = "m:Tfct:busac", "s, Fault Clearing Time"
      m_Tfct_busdp = "m:Tfct:busdp", "s, Fault Clearing Time"
      m_Tfct_busdm = "m:Tfct:busdm", "s, Fault Clearing Time"
      m_TfctPh_busac_A = "m:TfctPh:busac:A", "s, Fault Clearing Time"
      m_TfctPh_busac_B = "m:TfctPh:busac:B", "s, Fault Clearing Time"
      m_TfctPh_busac_C = "m:TfctPh:busac:C", "s, Fault Clearing Time"
      m_TfctPh_busdp = "m:TfctPh:busdp", "s, Fault Clearing Time"
      m_TfctPh_busdm = "m:TfctPh:busdm", "s, Fault Clearing Time"
      m_Brkload_busac = "m:Brkload:busac", "%, Max. Breaker Loading"
      m_Brkload_busdp = "m:Brkload:busdp", "%, Max. Breaker Loading"
      m_Brkload_busdm = "m:Brkload:busdm", "%, Max. Breaker Loading"
      m_BrkloadPh_busac_A = "m:BrkloadPh:busac:A", "%, Breaker Loading"
      m_BrkloadPh_busac_B = "m:BrkloadPh:busac:B", "%, Breaker Loading"
      m_BrkloadPh_busac_C = "m:BrkloadPh:busac:C", "%, Breaker Loading"
      m_BrkloadPh_busdp = "m:BrkloadPh:busdp", "%, Breaker Loading"
      m_BrkloadPh_busdm = "m:BrkloadPh:busdm", "%, Breaker Loading"
      n_ur_busac_A = "n:ur:busac:A", "p.u., Line-Ground Voltage, Real Part"
      n_ur_busac_B = "n:ur:busac:B", "p.u., Line-Ground Voltage, Real Part"
      n_ur_busac_C = "n:ur:busac:C", "p.u., Line-Ground Voltage, Real Part"
      n_ur_busdp = "n:ur:busdp", "p.u., Line-Ground Voltage, Real Part"
      n_ur_busdm = "n:ur:busdm", "p.u., Line-Ground Voltage, Real Part"
      n_ui_busac_A = "n:ui:busac:A", "p.u., Line-Ground Voltage, Imaginary Part"
      n_ui_busac_B = "n:ui:busac:B", "p.u., Line-Ground Voltage, Imaginary Part"
      n_ui_busac_C = "n:ui:busac:C", "p.u., Line-Ground Voltage, Imaginary Part"
      n_ui_busdp = "n:ui:busdp", "p.u., Line-Ground Voltage, Imaginary Part"
      n_ui_busdm = "n:ui:busdm", "p.u., Line-Ground Voltage, Imaginary Part"
      n_u_busac_A = "n:u:busac:A", "p.u., Line-Ground Voltage, Magnitude"
      n_u_busac_B = "n:u:busac:B", "p.u., Line-Ground Voltage, Magnitude"
      n_u_busac_C = "n:u:busac:C", "p.u., Line-Ground Voltage, Magnitude"
      n_u_busdp = "n:u:busdp", "p.u., Line-Ground Voltage, Magnitude"
      n_u_busdm = "n:u:busdm", "p.u., Line-Ground Voltage, Magnitude"
      n_upc_busac_A = "n:upc:busac:A", "%, Line-Ground Voltage, Magnitude"
      n_upc_busac_B = "n:upc:busac:B", "%, Line-Ground Voltage, Magnitude"
      n_upc_busac_C = "n:upc:busac:C", "%, Line-Ground Voltage, Magnitude"
      n_upc_busdp = "n:upc:busdp", "%, Line-Ground Voltage, Magnitude"
      n_upc_busdm = "n:upc:busdm", "%, Line-Ground Voltage, Magnitude"
      n_U_busac_A = "n:U:busac:A", "kV, Line-Ground Voltage, Magnitude"
      n_U_busac_B = "n:U:busac:B", "kV, Line-Ground Voltage, Magnitude"
      n_U_busac_C = "n:U:busac:C", "kV, Line-Ground Voltage, Magnitude"
      n_U_busdp = "n:U:busdp", "kV, Line-Ground Voltage, Magnitude"
      n_U_busdm = "n:U:busdm", "kV, Line-Ground Voltage, Magnitude"
      n_phiu_busac_A = "n:phiu:busac:A", "deg, Line-Ground Voltage, Angle"
      n_phiu_busac_B = "n:phiu:busac:B", "deg, Line-Ground Voltage, Angle"
      n_phiu_busac_C = "n:phiu:busac:C", "deg, Line-Ground Voltage, Angle"
      n_phiu_busdp = "n:phiu:busdp", "deg, Line-Ground Voltage, Angle"
      n_phiu_busdm = "n:phiu:busdm", "deg, Line-Ground Voltage, Angle"
      n_phiurel_busac = "n:phiurel:busac", "deg, Voltage, Relative Angle"
      n_phiurel_busdp = "n:phiurel:busdp", "deg, Voltage, Relative Angle"
      n_phiurel_busdm = "n:phiurel:busdm", "deg, Voltage, Relative Angle"
      n_ul_busac_A = "n:ul:busac:A", "p.u., Line-Line Voltage, Magnitude"
      n_ul_busac_B = "n:ul:busac:B", "p.u., Line-Line Voltage, Magnitude"
      n_ul_busac_C = "n:ul:busac:C", "p.u., Line-Line Voltage, Magnitude"
      n_ul_busdp = "n:ul:busdp", "p.u., Line-Line Voltage, Magnitude"
      n_ul_busdm = "n:ul:busdm", "p.u., Line-Line Voltage, Magnitude"
      n_ulpc_busac_A = "n:ulpc:busac:A", "%, Line-Line Voltage, Magnitude"
      n_ulpc_busac_B = "n:ulpc:busac:B", "%, Line-Line Voltage, Magnitude"
      n_ulpc_busac_C = "n:ulpc:busac:C", "%, Line-Line Voltage, Magnitude"
      n_ulpc_busdp = "n:ulpc:busdp", "%, Line-Line Voltage, Magnitude"
      n_ulpc_busdm = "n:ulpc:busdm", "%, Line-Line Voltage, Magnitude"
      n_Ul_busac_A = "n:Ul:busac:A", "kV, Line-Line Voltage, Magnitude"
      n_Ul_busac_B = "n:Ul:busac:B", "kV, Line-Line Voltage, Magnitude"
      n_Ul_busac_C = "n:Ul:busac:C", "kV, Line-Line Voltage, Magnitude"
      n_Ul_busdp = "n:Ul:busdp", "kV, Line-Line Voltage, Magnitude"
      n_Ul_busdm = "n:Ul:busdm", "kV, Line-Line Voltage, Magnitude"
      n_phiul_busac_A = "n:phiul:busac:A", "deg, Line-Line Voltage, Angle"
      n_phiul_busac_B = "n:phiul:busac:B", "deg, Line-Line Voltage, Angle"
      n_phiul_busac_C = "n:phiul:busac:C", "deg, Line-Line Voltage, Angle"
      n_phiul_busdp = "n:phiul:busdp", "deg, Line-Line Voltage, Angle"
      n_phiul_busdm = "n:phiul:busdm", "deg, Line-Line Voltage, Angle"
      n_u0_busac = "n:u0:busac", "p.u., Zero-Sequence Voltage, Magnitude"
      n_u0_busdp = "n:u0:busdp", "p.u., Zero-Sequence Voltage, Magnitude"
      n_u0_busdm = "n:u0:busdm", "p.u., Zero-Sequence Voltage, Magnitude"
      n_U0_busac = "n:U0:busac", "kV, Zero-Sequence Voltage, Magnitude"
      n_U0_busdp = "n:U0:busdp", "kV, Zero-Sequence Voltage, Magnitude"
      n_U0_busdm = "n:U0:busdm", "kV, Zero-Sequence Voltage, Magnitude"
      n_U0x3_busac = "n:U0x3:busac", "kV, 3*U0"
      n_U0x3_busdp = "n:U0x3:busdp", "kV, 3*U0"
      n_U0x3_busdm = "n:U0x3:busdm", "kV, 3*U0"
      n_phiu0_busac = "n:phiu0:busac", "deg, Zero-Sequence Voltage, Angle"
      n_phiu0_busdp = "n:phiu0:busdp", "deg, Zero-Sequence Voltage, Angle"
      n_phiu0_busdm = "n:phiu0:busdm", "deg, Zero-Sequence Voltage, Angle"
      n_u1_busac = "n:u1:busac", "p.u., Positive-Sequence Voltage, Magnitude"
      n_u1_busdp = "n:u1:busdp", "p.u., Positive-Sequence Voltage, Magnitude"
      n_u1_busdm = "n:u1:busdm", "p.u., Positive-Sequence Voltage, Magnitude"
      n_u1pc_busac = "n:u1pc:busac", "%, Positive-Sequence Voltage, Magnitude"
      n_u1pc_busdp = "n:u1pc:busdp", "%, Positive-Sequence Voltage, Magnitude"
      n_u1pc_busdm = "n:u1pc:busdm", "%, Positive-Sequence Voltage, Magnitude"
      n_u1r_busac = "n:u1r:busac", "p.u., Positive-Sequence Voltage, Real Part"
      n_u1r_busdp = "n:u1r:busdp", "p.u., Positive-Sequence Voltage, Real Part"
      n_u1r_busdm = "n:u1r:busdm", "p.u., Positive-Sequence Voltage, Real Part"
      n_u1i_busac = "n:u1i:busac", "p.u., Positive-Sequence Voltage, Imaginary Part"
      n_u1i_busdp = "n:u1i:busdp", "p.u., Positive-Sequence Voltage, Imaginary Part"
      n_u1i_busdm = "n:u1i:busdm", "p.u., Positive-Sequence Voltage, Imaginary Part"
      n_U1_busac = "n:U1:busac", "kV, Line-Ground Positive-Sequence Voltage, Magnitude"
      n_U1_busdp = "n:U1:busdp", "kV, Line-Ground Positive-Sequence Voltage, Magnitude"
      n_U1_busdm = "n:U1:busdm", "kV, Line-Ground Positive-Sequence Voltage, Magnitude"
      n_phiu1_busac = "n:phiu1:busac", "deg, Positive-Sequence Voltage, Angle"
      n_phiu1_busdp = "n:phiu1:busdp", "deg, Positive-Sequence Voltage, Angle"
      n_phiu1_busdm = "n:phiu1:busdm", "deg, Positive-Sequence Voltage, Angle"
      n_u2_busac = "n:u2:busac", "p.u., Negative-Sequence Voltage, Magnitude"
      n_u2_busdp = "n:u2:busdp", "p.u., Negative-Sequence Voltage, Magnitude"
      n_u2_busdm = "n:u2:busdm", "p.u., Negative-Sequence Voltage, Magnitude"
      n_U2_busac = "n:U2:busac", "kV, Line-Ground Negative-Sequence Voltage, Magnitude"
      n_U2_busdp = "n:U2:busdp", "kV, Line-Ground Negative-Sequence Voltage, Magnitude"
      n_U2_busdm = "n:U2:busdm", "kV, Line-Ground Negative-Sequence Voltage, Magnitude"
      n_phiu2_busac = "n:phiu2:busac", "deg, Negative-Sequence Voltage, Angle"
      n_phiu2_busdp = "n:phiu2:busdp", "deg, Negative-Sequence Voltage, Angle"
      n_phiu2_busdm = "n:phiu2:busdm", "deg, Negative-Sequence Voltage, Angle"
      n_U1l_busac = "n:U1l:busac", "kV, Line-Line Positive-Sequence Voltage, Magnitude"
      n_U1l_busdp = "n:U1l:busdp", "kV, Line-Line Positive-Sequence Voltage, Magnitude"
      n_U1l_busdm = "n:U1l:busdm", "kV, Line-Line Positive-Sequence Voltage, Magnitude"
      n_U2l_busac = "n:U2l:busac", "kV, Line-Line Negative-Sequence Voltage, Magnitude"
      n_U2l_busdp = "n:U2l:busdp", "kV, Line-Line Negative-Sequence Voltage, Magnitude"
      n_U2l_busdm = "n:U2l:busdm", "kV, Line-Line Negative-Sequence Voltage, Magnitude"
      n_uln_busac_A = "n:uln:busac:A", "p.u., Line-Neutral Voltage, Magnitude"
      n_uln_busac_B = "n:uln:busac:B", "p.u., Line-Neutral Voltage, Magnitude"
      n_uln_busac_C = "n:uln:busac:C", "p.u., Line-Neutral Voltage, Magnitude"
      n_uln_busdp = "n:uln:busdp", "p.u., Line-Neutral Voltage, Magnitude"
      n_uln_busdm = "n:uln:busdm", "p.u., Line-Neutral Voltage, Magnitude"
      n_Uln_busac_A = "n:Uln:busac:A", "kV, Line-Neutral Voltage, Magnitude"
      n_Uln_busac_B = "n:Uln:busac:B", "kV, Line-Neutral Voltage, Magnitude"
      n_Uln_busac_C = "n:Uln:busac:C", "kV, Line-Neutral Voltage, Magnitude"
      n_Uln_busdp = "n:Uln:busdp", "kV, Line-Neutral Voltage, Magnitude"
      n_Uln_busdm = "n:Uln:busdm", "kV, Line-Neutral Voltage, Magnitude"
      n_phiuln_busac_A = "n:phiuln:busac:A", "deg, Line-Neutral Voltage, Angle"
      n_phiuln_busac_B = "n:phiuln:busac:B", "deg, Line-Neutral Voltage, Angle"
      n_phiuln_busac_C = "n:phiuln:busac:C", "deg, Line-Neutral Voltage, Angle"
      n_phiuln_busdp = "n:phiuln:busdp", "deg, Line-Neutral Voltage, Angle"
      n_phiuln_busdm = "n:phiuln:busdm", "deg, Line-Neutral Voltage, Angle"
      n_un_busac = "n:un:busac", "p.u., Neutral-Ground Voltage, Magnitude"
      n_un_busdp = "n:un:busdp", "p.u., Neutral-Ground Voltage, Magnitude"
      n_un_busdm = "n:un:busdm", "p.u., Neutral-Ground Voltage, Magnitude"
      n_Un_busac = "n:Un:busac", "kV, Neutral-Ground Voltage, Magnitude"
      n_Un_busdp = "n:Un:busdp", "kV, Neutral-Ground Voltage, Magnitude"
      n_Un_busdm = "n:Un:busdm", "kV, Neutral-Ground Voltage, Magnitude"
      n_phiun_busac = "n:phiun:busac", "deg, Neutral-Ground Voltage, Angle"
      n_phiun_busdp = "n:phiun:busdp", "deg, Neutral-Ground Voltage, Angle"
      n_phiun_busdm = "n:phiun:busdm", "deg, Neutral-Ground Voltage, Angle"
      n_upht_busac_A = "n:upht:busac:A", "p.u., Phase Technology dependent Voltage, Magnitude"
      n_upht_busac_B = "n:upht:busac:B", "p.u., Phase Technology dependent Voltage, Magnitude"
      n_upht_busac_C = "n:upht:busac:C", "p.u., Phase Technology dependent Voltage, Magnitude"
      n_upht_busdp = "n:upht:busdp", "p.u., Phase Technology dependent Voltage, Magnitude"
      n_upht_busdm = "n:upht:busdm", "p.u., Phase Technology dependent Voltage, Magnitude"
      n_Upht_busac_A = "n:Upht:busac:A", "kV, Phase Technology dependent Voltage, Magnitude"
      n_Upht_busac_B = "n:Upht:busac:B", "kV, Phase Technology dependent Voltage, Magnitude"
      n_Upht_busac_C = "n:Upht:busac:C", "kV, Phase Technology dependent Voltage, Magnitude"
      n_Upht_busdp = "n:Upht:busdp", "kV, Phase Technology dependent Voltage, Magnitude"
      n_Upht_busdm = "n:Upht:busdm", "kV, Phase Technology dependent Voltage, Magnitude"
      n_um_busac = "n:um:busac", "p.u., Average-Voltage, Magnitude"
      n_um_busdp = "n:um:busdp", "p.u., Average-Voltage, Magnitude"
      n_um_busdm = "n:um:busdm", "p.u., Average-Voltage, Magnitude"
      n_Um_busac = "n:Um:busac", "kV, Average-Voltage, Magnitude"
      n_Um_busdp = "n:Um:busdp", "kV, Average-Voltage, Magnitude"
      n_Um_busdm = "n:Um:busdm", "kV, Average-Voltage, Magnitude"
      n_uphtmin_busac = "n:uphtmin:busac", "p.u., Minimum of Phase Technology dependent Voltage, Magnitude"
      n_uphtmin_busdp = "n:uphtmin:busdp", "p.u., Minimum of Phase Technology dependent Voltage, Magnitude"
      n_uphtmin_busdm = "n:uphtmin:busdm", "p.u., Minimum of Phase Technology dependent Voltage, Magnitude"
      n_uphtmax_busac = "n:uphtmax:busac", "p.u., Maximum of Phase Technology dependent Voltage, Magnitude"
      n_uphtmax_busdp = "n:uphtmax:busdp", "p.u., Maximum of Phase Technology dependent Voltage, Magnitude"
      n_uphtmax_busdm = "n:uphtmax:busdm", "p.u., Maximum of Phase Technology dependent Voltage, Magnitude"
      n_Pgen_busac = "n:Pgen:busac", "MW, Generation, Active Power"
      n_Pgen_busdp = "n:Pgen:busdp", "MW, Generation, Active Power"
      n_Pgen_busdm = "n:Pgen:busdm", "MW, Generation, Active Power"
      n_Qgen_busac = "n:Qgen:busac", "Mvar, Generation, Reactive Power"
      n_Qgen_busdp = "n:Qgen:busdp", "Mvar, Generation, Reactive Power"
      n_Qgen_busdm = "n:Qgen:busdm", "Mvar, Generation, Reactive Power"
      n_Pmot_busac = "n:Pmot:busac", "MW, Motor Load, Active Power"
      n_Pmot_busdp = "n:Pmot:busdp", "MW, Motor Load, Active Power"
      n_Pmot_busdm = "n:Pmot:busdm", "MW, Motor Load, Active Power"
      n_Qmot_busac = "n:Qmot:busac", "Mvar, Motor Load, Reactive Power"
      n_Qmot_busdp = "n:Qmot:busdp", "Mvar, Motor Load, Reactive Power"
      n_Qmot_busdm = "n:Qmot:busdm", "Mvar, Motor Load, Reactive Power"
      n_Pload_busac = "n:Pload:busac", "MW, General Load, Active Power"
      n_Pload_busdp = "n:Pload:busdp", "MW, General Load, Active Power"
      n_Pload_busdm = "n:Pload:busdm", "MW, General Load, Active Power"
      n_Qload_busac = "n:Qload:busac", "Mvar, General Load, Reactive Power"
      n_Qload_busdp = "n:Qload:busdp", "Mvar, General Load, Reactive Power"
      n_Qload_busdm = "n:Qload:busdm", "Mvar, General Load, Reactive Power"
      n_Pcomp_busac = "n:Pcomp:busac", "MW, Compensation (Losses)"
      n_Pcomp_busdp = "n:Pcomp:busdp", "MW, Compensation (Losses)"
      n_Pcomp_busdm = "n:Pcomp:busdm", "MW, Compensation (Losses)"
      n_Qcomp_busac = "n:Qcomp:busac", "Mvar, Compensation"
      n_Qcomp_busdp = "n:Qcomp:busdp", "Mvar, Compensation"
      n_Qcomp_busdm = "n:Qcomp:busdm", "Mvar, Compensation"
      n_Pnet_busac = "n:Pnet:busac", "MW, External Networks, Active Power"
      n_Pnet_busdp = "n:Pnet:busdp", "MW, External Networks, Active Power"
      n_Pnet_busdm = "n:Pnet:busdm", "MW, External Networks, Active Power"
      n_Qnet_busac = "n:Qnet:busac", "Mvar, External Networks, Reactive Power"
      n_Qnet_busdp = "n:Qnet:busdp", "Mvar, External Networks, Reactive Power"
      n_Qnet_busdm = "n:Qnet:busdm", "Mvar, External Networks, Reactive Power"
      n_Pflow_busac = "n:Pflow:busac", "MW, Power Flow, Active Power"
      n_Pflow_busdp = "n:Pflow:busdp", "MW, Power Flow, Active Power"
      n_Pflow_busdm = "n:Pflow:busdm", "MW, Power Flow, Active Power"
      n_Qflow_busac = "n:Qflow:busac", "Mvar, Power Flow, Reactive Power"
      n_Qflow_busdp = "n:Qflow:busdp", "Mvar, Power Flow, Reactive Power"
      n_Qflow_busdm = "n:Qflow:busdm", "Mvar, Power Flow, Reactive Power"
      n_Pout_busac = "n:Pout:busac", "MW, Outgoing Flow, Active Power"
      n_Pout_busdp = "n:Pout:busdp", "MW, Outgoing Flow, Active Power"
      n_Pout_busdm = "n:Pout:busdm", "MW, Outgoing Flow, Active Power"
      n_Qout_busac = "n:Qout:busac", "Mvar, Outgoing Power, Reactive Power"
      n_Qout_busdp = "n:Qout:busdp", "Mvar, Outgoing Power, Reactive Power"
      n_Qout_busdm = "n:Qout:busdm", "Mvar, Outgoing Power, Reactive Power"
      n_Sout_busac = "n:Sout:busac", "MVA, Outgoing Power, Apparent Power"
      n_Sout_busdp = "n:Sout:busdp", "MVA, Outgoing Power, Apparent Power"
      n_Sout_busdm = "n:Sout:busdm", "MVA, Outgoing Power, Apparent Power"
      n_cosphiout_busac = "n:cosphiout:busac", "Outgoing Power, Power Factor"
      n_cosphiout_busdp = "n:cosphiout:busdp", "Outgoing Power, Power Factor"
      n_cosphiout_busdm = "n:cosphiout:busdm", "Outgoing Power, Power Factor"
      n_Pbalance_busac = "n:Pbalance:busac", "MW, Active Power Balance (=0)"
      n_Pbalance_busdp = "n:Pbalance:busdp", "MW, Active Power Balance (=0)"
      n_Pbalance_busdm = "n:Pbalance:busdm", "MW, Active Power Balance (=0)"
      n_Qbalance_busac = "n:Qbalance:busac", "Mvar, Reactive Power Balance (=0)"
      n_Qbalance_busdp = "n:Qbalance:busdp", "Mvar, Reactive Power Balance (=0)"
      n_Qbalance_busdm = "n:Qbalance:busdm", "Mvar, Reactive Power Balance (=0)"
      n_ubfac_busac = "n:ubfac:busac", "%, Unbalance Factor"
      n_ubfac_busdp = "n:ubfac:busdp", "%, Unbalance Factor"
      n_ubfac_busdm = "n:ubfac:busdm", "%, Unbalance Factor"
      n_LossPdown_busac = "n:LossPdown:busac", "MW, Losses, downstream"
      n_LossPdown_busdp = "n:LossPdown:busdp", "MW, Losses, downstream"
      n_LossPdown_busdm = "n:LossPdown:busdm", "MW, Losses, downstream"
      n_LossQdown_busac = "n:LossQdown:busac", "Mvar, Losses, downstream (Reactive Power)"
      n_LossQdown_busdp = "n:LossQdown:busdp", "Mvar, Losses, downstream (Reactive Power)"
      n_LossQdown_busdm = "n:LossQdown:busdm", "Mvar, Losses, downstream (Reactive Power)"
      n_LossPdownload_busac = "n:LossPdownload:busac", "MW, Load losses, downstream"
      n_LossPdownload_busdp = "n:LossPdownload:busdp", "MW, Load losses, downstream"
      n_LossPdownload_busdm = "n:LossPdownload:busdm", "MW, Load losses, downstream"
      n_LossQdownload_busac = "n:LossQdownload:busac", "Mvar, Load losses, downstream"
      n_LossQdownload_busdp = "n:LossQdownload:busdp", "Mvar, Load losses, downstream"
      n_LossQdownload_busdm = "n:LossQdownload:busdm", "Mvar, Load losses, downstream"
      n_LossPdownnoload_busac = "n:LossPdownnoload:busac", "MW, No load losses, downstream"
      n_LossPdownnoload_busdp = "n:LossPdownnoload:busdp", "MW, No load losses, downstream"
      n_LossPdownnoload_busdm = "n:LossPdownnoload:busdm", "MW, No load losses, downstream"
      n_LossQdownnoload_busac = "n:LossQdownnoload:busac", "Mvar, No load losses, downstream"
      n_LossQdownnoload_busdp = "n:LossQdownnoload:busdp", "Mvar, No load losses, downstream"
      n_LossQdownnoload_busdm = "n:LossQdownnoload:busdm", "Mvar, No load losses, downstream"
      s_pset = "s:pset", "MW, IN, Active power setpoint, continuous, double"
      s_qset = "s:qset", "Mvar, IN, Reactive power setpoint continuous, double"
      s_udcset = "s:udcset", "p.u., IN, DC Voltage setpoint, continuous, double"
      s_dFin = "s:dFin", "Hz, IN, Frequency Deviation, continuous, double"
      s_i2r_set = "s:i2r_set", "p.u., IN, Negative sequence current setpoint, Real Part, continuous, double"
      s_i2i_set = "s:i2i_set", "p.u., IN, Negative sequence current setpoint, Imaginary Part, continuous, double"
      c_phiu = "c:phiu", "p.u., Voltage angle"
      c_uini = "c:uini", "p.u., Voltage setpoint"
      c_snomn = "c:snomn", "MVA, Total rated apparent power (nparnum*Snom)"
      c_uinidc = "c:uinidc", "p.u., Voltage setpoint (DC)"
      c_pmsetp = "c:pmsetp", "PWM factor"
      c_i_mod = "c:i_mod", "Modulation"
      c_K0 = "c:K0", "Vac/Vdc at PWM=1"
      c_Pm = "c:Pm", "Pulse-modulation factor"
      c_uDC = "c:uDC", "p.u., DC-voltage"
      c_pini = "c:pini", "Active power setpoint"
      c_qini = "c:qini", "Reactive power setpoint"
      c_DCPoleLosses = "c:DCPoleLosses", "MW, DC pole losses"
      c_Qmax = "c:Qmax", "Mvar, Upper limit of reactive power"
      c_Qmin = "c:Qmin", "Mvar, Lower limit of reactive power"
      c_Pmin = "c:Pmin", "MW, Lower limit of active power"
      c_Pmax = "c:Pmax", "MW, Upper limit of active power"
      c_i_qlim = "c:i_qlim", "Reactive power limited"
      c_i_plim = "c:i_plim", "Active power limited"
      c_Kpf = "c:Kpf", "MW/Hz, Frequency bias"
      c_Kp = "c:Kp", "p.u., Active power participation"
      c_zonewscale = "c:zonewscale", "Wind generation zone scaling factor"
      c_iRefElement = "c:iRefElement", "Reference element"
      c_Kq = "c:Kq", "p.u., Reactive power participation"
      c_iPowerOn = "c:iPowerOn", "Flag"
      c_i_vlim = "c:i_vlim", "Voltage setpoint limit reached"
      c_dphi = "c:dphi", "deg, Measured angle difference"
      c_Pmeas = "c:Pmeas", "MW, Measured active power"
      c_nCtrlIter = "c:nCtrlIter", "No. of CTRL-iterations"
      c_Ploss = "c:Ploss", "MW, Losses (total)"
      c_Qloss = "c:Qloss", "Mvar, Reactive-Losses (total)"
      c_Plossld = "c:Plossld", "MW, Losses (load)"
      c_Qlossld = "c:Qlossld", "Mvar, Reactive-Losses (load)"
      c_Plossnld = "c:Plossnld", "MW, Losses (no load)"
      c_Qlossnld = "c:Qlossnld", "Mvar, Reactive-Losses (no load)"
      e_i_acdc = "e:i_acdc", "Control mode"
      e_ivacdroop = "e:ivacdroop", "AC-Voltage Droop"
      e_usetp = "e:usetp", "p.u., AC voltage setpoint"
      e_pmsetp = "e:pmsetp", "p.u., PWM factor"
      e_p_uctrl = "e:p_uctrl", "ElmTerm*,StaBar*, Controlled node (AC)"
      e_usetpdc = "e:usetpdc", "p.u., DC voltage setpoint"
      e_phisetp = "e:phisetp", "deg, Phase setpoint"
      e_psetp = "e:psetp", "MW, Active power setpoint"
      e_Kpf = "e:Kpf", "MW/Hz, Prim. frequency bias"
      e_qsetp = "e:qsetp", "Mvar, Reactive power setpoint"
      e_iQorient = "e:iQorient", "Orientation"
      e_pfsetp = "e:pfsetp", "Power factor"
      e_pf_recap = "e:pf_recap", "Power factor"
      e_p_pctrl = "e:p_pctrl", "StaCubic,ElmBoundary, Controlled flow"
      e_c_pstac = "e:c_pstac", "ElmStactrl, External station controller"
      e_c_psecc = "e:c_psecc", "ElmSecctrl, Ext. secondary controller"
      e_iVacMax = "e:iVacMax", "Consider modulation index limit"
      e_Pmmax = "e:Pmmax", "Max. PWM factor"
      e_Pnold = "e:Pnold", "kW, No-load losses"
      e_swtLossFactor = "e:swtLossFactor", "kW/A, Switching loss factor"
      e_resLossFactor = "e:resLossFactor", "Ohm, Resistive loss factor"
      e_ddroop = "e:ddroop", "%, Droop"
      e_usp_min = "e:usp_min", "p.u., Min. voltage setpoint"
      e_usp_max = "e:usp_max", "p.u., Max. voltage setpoint"
      e_idroop = "e:idroop", "DC-voltage dependent P-droop"
      e_K = "e:K", "p.u./MW, K"
      e_iPphidrp = "e:iPphidrp", "Angle difference dependent P-droop"
      e_Kpphi = "e:Kpphi", "MW/deg, Kpphi"
      e_p_b1phiu = "e:p_b1phiu", "ElmTerm*, Remote AC busbar"
      e_p_b2phiu = "e:p_b2phiu", "ElmTerm*, Local AC busbar"
      e_considerPset = "e:considerPset", "Consider active power setpoint"
      e_iPpart = "e:iPpart", "Active power participation"
      e_Kpart = "e:Kpart", "Participation factor"
      e_p_pmeas = "e:p_pmeas", "StaCubic*,ElmBoundary, P(AC) measured at"
      e_pQlimType = "e:pQlimType", "IntQlim, Capability curve"
      e_q_min = "e:q_min", "p.u., Min."
      e_q_max = "e:q_max", "p.u., Max."
      e_cQ_min = "e:cQ_min", "Mvar, Min."
      e_cQ_max = "e:cQ_max", "Mvar, Max."
      e_scaleQmin = "e:scaleQmin", "%, Scaling factor (min.)"
      e_scaleQmax = "e:scaleQmax", "%, Scaling factor (max.)"
      e_Pmin_uc = "e:Pmin_uc", "MW, Min."
      e_Pmax_uc = "e:Pmax_uc", "MW, Max."
      e_Pnom = "e:Pnom", "MW, Pr(rated)"
      e_Qmin_a = "e:Qmin_a", "Mvar, Qmin(act.)"
      e_Qmax_a = "e:Qmax_a", "Mvar, Qmax(act.)"
      e_P_max = "e:P_max", "MW, Max."
      e_r2 = "e:r2", "p.u., Resistance, r2"
      e_x2 = "e:x2", "p.u., Reactance, x2"
      e_priority = "e:priority", "Merit Order"
      e_cpFeed = "e:cpFeed", "Feeder"
      e_ciDist = "e:ciDist", "Distance from infeed in number of buses"
      e_ciLater = "e:ciLater", "Lateral Index"
      e_ciDistRoot = "e:ciDistRoot", "Distance from first infeed in number of buses"
      e_ciDistAll = "e:ciDistAll", "Distance from infeed in number of buses including switches"
      e_ciDistAllRoot = "e:ciDistAllRoot", "Distance from first infeed in number of buses including switches"
      r_p_uctrl = "r:p_uctrl", "ElmTerm*,StaBar*, Controlled node (AC)"
      r_p_pctrl = "r:p_pctrl", "StaCubic,ElmBoundary, Controlled flow"
      r_c_pstac = "r:c_pstac", "ElmStactrl, External station controller"
      r_c_psecc = "r:c_psecc", "ElmSecctrl, Ext. secondary controller"
      r_p_b1phiu = "r:p_b1phiu", "ElmTerm*, Remote AC busbar"
      r_p_b2phiu = "r:p_b2phiu", "ElmTerm*, Local AC busbar"
      r_p_pmeas = "r:p_pmeas", "StaCubic*,ElmBoundary, P(AC) measured at"
      r_pQlimType = "r:pQlimType", "IntQlim, Capability curve"
      r_cpFeed = "r:cpFeed", "Feeder"
      m_Ikss_busac = "m:Ikss:busac", "kA, Initial Short-Circuit Current"
      m_Ikss_busdp = "m:Ikss:busdp", "kA, Initial Short-Circuit Current"
      m_Ikss_busdm = "m:Ikss:busdm", "kA, Initial Short-Circuit Current"
      m_ikss_busac = "m:ikss:busac", "p.u., Initial Short-Circuit Current, Magnitude"
      m_ikss_busdp = "m:ikss:busdp", "p.u., Initial Short-Circuit Current, Magnitude"
      m_ikss_busdm = "m:ikss:busdm", "p.u., Initial Short-Circuit Current, Magnitude"
      m_phii_busac = "m:phii:busac", "deg, Phase Current, Angle"
      m_maxIkss_busac = "m:maxIkss:busac", "kA, Max. Initial Short-Circuit Current"
      m_maxIkss_busdp = "m:maxIkss:busdp", "kA, Max. Initial Short-Circuit Current"
      m_maxIkss_busdm = "m:maxIkss:busdm", "kA, Max. Initial Short-Circuit Current"
      m_Skss_busac = "m:Skss:busac", "MVA, Initial Short-Circuit Power"
      m_Skss_busdp = "m:Skss:busdp", "MVA, Initial Short-Circuit Power"
      m_Skss_busdm = "m:Skss:busdm", "MVA, Initial Short-Circuit Power"
      m_ip_busac = "m:ip:busac", "kA, Peak Short-Circuit Current"
      m_ip_busdp = "m:ip:busdp", "kA, Peak Short-Circuit Current"
      m_ip_busdm = "m:ip:busdm", "kA, Peak Short-Circuit Current"
      m_maxip_busac = "m:maxip:busac", "kA, Max. Peak Short-Circuit Current"
      m_maxip_busdp = "m:maxip:busdp", "kA, Max. Peak Short-Circuit Current"
      m_maxip_busdm = "m:maxip:busdm", "kA, Max. Peak Short-Circuit Current"
      m_maxIb_busac = "m:maxIb:busac", "kA, Max. Breaking Current"
      m_maxIb_busdp = "m:maxIb:busdp", "kA, Max. Breaking Current"
      m_maxIb_busdm = "m:maxIb:busdm", "kA, Max. Breaking Current"
      m_maxIth_busac = "m:maxIth:busac", "kA, Max. Thermal equivalent Short-Circuit Current"
      m_maxIth_busdp = "m:maxIth:busdp", "kA, Max. Thermal equivalent Short-Circuit Current"
      m_maxIth_busdm = "m:maxIth:busdm", "kA, Max. Thermal equivalent Short-Circuit Current"
      m_Ib_busac = "m:Ib:busac", "kA, Short-Circuit Breaking Current"
      m_Ib_busdp = "m:Ib:busdp", "kA, Short-Circuit Breaking Current"
      m_Ib_busdm = "m:Ib:busdm", "kA, Short-Circuit Breaking Current"
      m_Sb_busac = "m:Sb:busac", "MVA, Short-Circuit Breaking Power"
      m_Sb_busdp = "m:Sb:busdp", "MVA, Short-Circuit Breaking Power"
      m_Sb_busdm = "m:Sb:busdm", "MVA, Short-Circuit Breaking Power"
      m_Ik_busac = "m:Ik:busac", "kA, Steady-State Short-Circuit Current"
      m_Ik_busdp = "m:Ik:busdp", "kA, Steady-State Short-Circuit Current"
      m_Ik_busdm = "m:Ik:busdm", "kA, Steady-State Short-Circuit Current"
      m_Ith_busac = "m:Ith:busac", "kA, Thermal equivalent Short-Circuit Current"
      m_Ith_busdp = "m:Ith:busdp", "kA, Thermal equivalent Short-Circuit Current"
      m_Ith_busdm = "m:Ith:busdm", "kA, Thermal equivalent Short-Circuit Current"
      m_I_busac = "m:I:busac", "kA, Short-Circuit Current"
      m_R_busac = "m:R:busac", "Ohm, Fault Impedance, Real Part"
      m_R_busdp = "m:R:busdp", "Ohm, Fault Impedance, Real Part"
      m_R_busdm = "m:R:busdm", "Ohm, Fault Impedance, Real Part"
      m_X_busac = "m:X:busac", "Ohm, Fault Impedance, Imaginary Part"
      m_X_busdp = "m:X:busdp", "Ohm, Fault Impedance, Imaginary Part"
      m_X_busdm = "m:X:busdm", "Ohm, Fault Impedance, Imaginary Part"
      m_Z_busac = "m:Z:busac", "Ohm, Fault Impedance, Magnitude"
      m_Z_busdp = "m:Z:busdp", "Ohm, Fault Impedance, Magnitude"
      m_Z_busdm = "m:Z:busdm", "Ohm, Fault Impedance, Magnitude"
      m_phiz_busac = "m:phiz:busac", "deg, Fault Impedance, Angle"
      m_phiz_busdp = "m:phiz:busdp", "deg, Fault Impedance, Angle"
      m_phiz_busdm = "m:phiz:busdm", "deg, Fault Impedance, Angle"
      m_zSbase_busac = "m:zSbase:busac", "p.u., Fault Impedance, Magnitude"
      m_zSbase_busdp = "m:zSbase:busdp", "p.u., Fault Impedance, Magnitude"
      m_zSbase_busdm = "m:zSbase:busdm", "p.u., Fault Impedance, Magnitude"
      m_XtoR_busac = "m:XtoR:busac", "X/R ratio"
      m_XtoR_busdp = "m:XtoR:busdp", "X/R ratio"
      m_XtoR_busdm = "m:XtoR:busdm", "X/R ratio"
      m_Zs_busac = "m:Zs:busac", "Ohm, Source Impedance, Magnitude"
      m_Zs_busdp = "m:Zs:busdp", "Ohm, Source Impedance, Magnitude"
      m_Zs_busdm = "m:Zs:busdm", "Ohm, Source Impedance, Magnitude"
      m_phizs_busac = "m:phizs:busac", "deg, Source Impedance, Angle"
      m_phizs_busdp = "m:phizs:busdp", "deg, Source Impedance, Angle"
      m_phizs_busdm = "m:phizs:busdm", "deg, Source Impedance, Angle"
      m_Rs_busac = "m:Rs:busac", "Ohm, Source Impedance, Real Part"
      m_Rs_busdp = "m:Rs:busdp", "Ohm, Source Impedance, Real Part"
      m_Rs_busdm = "m:Rs:busdm", "Ohm, Source Impedance, Real Part"
      m_Xs_busac = "m:Xs:busac", "Ohm, Source Impedance, Imaginary Part"
      m_Xs_busdp = "m:Xs:busdp", "Ohm, Source Impedance, Imaginary Part"
      m_Xs_busdm = "m:Xs:busdm", "Ohm, Source Impedance, Imaginary Part"
      m_phiui_busac = "m:phiui:busac", "deg, Angle between Voltage and Current"
      m_cosphiui_busac = "m:cosphiui:busac", "cosphi"
      m_cosphiui_busdp = "m:cosphiui:busdp", "cosphi"
      m_cosphiui_busdm = "m:cosphiui:busdm", "cosphi"
      m_P_busac = "m:P:busac", "MW, Active Power"
      m_Q_busac = "m:Q:busac", "Mvar, Reactive Power"
      m_BrkIpload_busac = "m:BrkIpload:busac", "%, Breaker Loading (peak)"
      m_BrkIpload_busdp = "m:BrkIpload:busdp", "%, Breaker Loading (peak)"
      m_BrkIpload_busdm = "m:BrkIpload:busdm", "%, Breaker Loading (peak)"
      m_BrkIbload_busac = "m:BrkIbload:busac", "%, Breaker Loading (breaking)"
      m_BrkIbload_busdp = "m:BrkIbload:busdp", "%, Breaker Loading (breaking)"
      m_BrkIbload_busdm = "m:BrkIbload:busdm", "%, Breaker Loading (breaking)"
      m_BrkmaxIb_busac = "m:BrkmaxIb:busac", "kA, Max. Breaking Current (breaker time)"
      m_BrkmaxIb_busdp = "m:BrkmaxIb:busdp", "kA, Max. Breaking Current (breaker time)"
      m_BrkmaxIb_busdm = "m:BrkmaxIb:busdm", "kA, Max. Breaking Current (breaker time)"
      n_Ikss_busac = "n:Ikss:busac", "kA, Initial Short-Circuit Current"
      n_Ikss_busdp = "n:Ikss:busdp", "kA, Initial Short-Circuit Current"
      n_Ikss_busdm = "n:Ikss:busdm", "kA, Initial Short-Circuit Current"
      n_I_busac = "n:I:busac", "kA, Short-Circuit Current"
      n_I_busdp = "n:I:busdp", "kA, Short-Circuit Current"
      n_I_busdm = "n:I:busdm", "kA, Short-Circuit Current"
      n_phii_busac = "n:phii:busac", "deg, Phase Current, Angle"
      n_phii_busdp = "n:phii:busdp", "deg, Phase Current, Angle"
      n_phii_busdm = "n:phii:busdm", "deg, Phase Current, Angle"
      n_Skss_busac = "n:Skss:busac", "MVA, Initial Short-Circuit Power"
      n_Skss_busdp = "n:Skss:busdp", "MVA, Initial Short-Circuit Power"
      n_Skss_busdm = "n:Skss:busdm", "MVA, Initial Short-Circuit Power"
      n_ip_busac = "n:ip:busac", "kA, Peak Short-Circuit Current"
      n_ip_busdp = "n:ip:busdp", "kA, Peak Short-Circuit Current"
      n_ip_busdm = "n:ip:busdm", "kA, Peak Short-Circuit Current"
      n_Ib_busac = "n:Ib:busac", "kA, Short-Circuit Breaking Current"
      n_Ib_busdp = "n:Ib:busdp", "kA, Short-Circuit Breaking Current"
      n_Ib_busdm = "n:Ib:busdm", "kA, Short-Circuit Breaking Current"
      n_Sb_busac = "n:Sb:busac", "MVA, Short-Circuit Breaking Power"
      n_Sb_busdp = "n:Sb:busdp", "MVA, Short-Circuit Breaking Power"
      n_Sb_busdm = "n:Sb:busdm", "MVA, Short-Circuit Breaking Power"
      n_Ik_busac = "n:Ik:busac", "kA, Steady-State Short-Circuit Current"
      n_Ik_busdp = "n:Ik:busdp", "kA, Steady-State Short-Circuit Current"
      n_Ik_busdm = "n:Ik:busdm", "kA, Steady-State Short-Circuit Current"
      n_Ith_busac = "n:Ith:busac", "kA, Thermal equivalent Short-Circuit Current"
      n_Ith_busdp = "n:Ith:busdp", "kA, Thermal equivalent Short-Circuit Current"
      n_Ith_busdm = "n:Ith:busdm", "kA, Thermal equivalent Short-Circuit Current"
      n_R_busac = "n:R:busac", "Ohm, Short-Circuit Impedance, Real Part"
      n_R_busdp = "n:R:busdp", "Ohm, Short-Circuit Impedance, Real Part"
      n_R_busdm = "n:R:busdm", "Ohm, Short-Circuit Impedance, Real Part"
      n_X_busac = "n:X:busac", "Ohm, Short-Circuit Impedance, Imaginary Part"
      n_X_busdp = "n:X:busdp", "Ohm, Short-Circuit Impedance, Imaginary Part"
      n_X_busdm = "n:X:busdm", "Ohm, Short-Circuit Impedance, Imaginary Part"
      n_Z_busac = "n:Z:busac", "Ohm, Short-Circuit Impedance, Magnitude"
      n_Z_busdp = "n:Z:busdp", "Ohm, Short-Circuit Impedance, Magnitude"
      n_Z_busdm = "n:Z:busdm", "Ohm, Short-Circuit Impedance, Magnitude"
      n_phiz_busac = "n:phiz:busac", "deg, Short-Circuit Impedance, Angle"
      n_phiz_busdp = "n:phiz:busdp", "deg, Short-Circuit Impedance, Angle"
      n_phiz_busdm = "n:phiz:busdm", "deg, Short-Circuit Impedance, Angle"
      n_rSbase_busac = "n:rSbase:busac", "p.u., Short-Circuit Impedance, Real Part"
      n_rSbase_busdp = "n:rSbase:busdp", "p.u., Short-Circuit Impedance, Real Part"
      n_rSbase_busdm = "n:rSbase:busdm", "p.u., Short-Circuit Impedance, Real Part"
      n_xSbase_busac = "n:xSbase:busac", "p.u., Short-Circuit Impedance, Imaginary Part"
      n_xSbase_busdp = "n:xSbase:busdp", "p.u., Short-Circuit Impedance, Imaginary Part"
      n_xSbase_busdm = "n:xSbase:busdm", "p.u., Short-Circuit Impedance, Imaginary Part"
      n_zSbase_busac = "n:zSbase:busac", "p.u., Short-Circuit Impedance, Magnitude"
      n_zSbase_busdp = "n:zSbase:busdp", "p.u., Short-Circuit Impedance, Magnitude"
      n_zSbase_busdm = "n:zSbase:busdm", "p.u., Short-Circuit Impedance, Magnitude"
      n_XtoR_busac = "n:XtoR:busac", "X/R ratio (peak)"
      n_XtoR_busdp = "n:XtoR:busdp", "X/R ratio (peak)"
      n_XtoR_busdm = "n:XtoR:busdm", "X/R ratio (peak)"
      n_RtoX_busac = "n:RtoX:busac", "R/X ratio (peak)"
      n_RtoX_busdp = "n:RtoX:busdp", "R/X ratio (peak)"
      n_RtoX_busdm = "n:RtoX:busdm", "R/X ratio (peak)"
      n_ur_busac = "n:ur:busac", "p.u., Voltage, Real Part"
      n_ui_busac = "n:ui:busac", "p.u., Voltage, Imaginary Part"
      n_u_busac = "n:u:busac", "p.u., Voltage, Magnitude"
      n_upc_busac = "n:upc:busac", "%, Voltage, Magnitude"
      n_U_busac = "n:U:busac", "kV, Line-Ground Voltage, Magnitude"
      n_Ul_busac = "n:Ul:busac", "kV, Line-Line Voltage, Magnitude"
      n_phiu_busac = "n:phiu:busac", "deg, Voltage, Angle"
      n_du_busac = "n:du:busac", "%, Voltage Deviation"
      n_du_busdp = "n:du:busdp", "%, Voltage Deviation"
      n_du_busdm = "n:du:busdm", "%, Voltage Deviation"
      n_urpref_busac = "n:urpref:busac", "p.u., Pre-fault Voltage, Real Part"
      n_urpref_busdp = "n:urpref:busdp", "p.u., Pre-fault Voltage, Real Part"
      n_urpref_busdm = "n:urpref:busdm", "p.u., Pre-fault Voltage, Real Part"
      n_uipref_busac = "n:uipref:busac", "p.u., Pre-fault Voltage, Imaginary Part"
      n_uipref_busdp = "n:uipref:busdp", "p.u., Pre-fault Voltage, Imaginary Part"
      n_uipref_busdm = "n:uipref:busdm", "p.u., Pre-fault Voltage, Imaginary Part"
      n_upref_busac = "n:upref:busac", "p.u., Pre-fault Voltage, Magnitude"
      n_upref_busdp = "n:upref:busdp", "p.u., Pre-fault Voltage, Magnitude"
      n_upref_busdm = "n:upref:busdm", "p.u., Pre-fault Voltage, Magnitude"
      n_uprefpc_busac = "n:uprefpc:busac", "%, Pre-fault Voltage, Magnitude"
      n_uprefpc_busdp = "n:uprefpc:busdp", "%, Pre-fault Voltage, Magnitude"
      n_uprefpc_busdm = "n:uprefpc:busdm", "%, Pre-fault Voltage, Magnitude"
      n_Upref_busac = "n:Upref:busac", "kV, Line-Ground Pre-fault Voltage, Magnitude"
      n_Upref_busdp = "n:Upref:busdp", "kV, Line-Ground Pre-fault Voltage, Magnitude"
      n_Upref_busdm = "n:Upref:busdm", "kV, Line-Ground Pre-fault Voltage, Magnitude"
      n_Ulpref_busac = "n:Ulpref:busac", "kV, Line-Line Pre-fault Voltage, Magnitude"
      n_Ulpref_busdp = "n:Ulpref:busdp", "kV, Line-Line Pre-fault Voltage, Magnitude"
      n_Ulpref_busdm = "n:Ulpref:busdm", "kV, Line-Line Pre-fault Voltage, Magnitude"
      n_phiupref_busac = "n:phiupref:busac", "deg, Pre-fault Voltage, Angle"
      n_phiupref_busdp = "n:phiupref:busdp", "deg, Pre-fault Voltage, Angle"
      n_phiupref_busdm = "n:phiupref:busdm", "deg, Pre-fault Voltage, Angle"
      n_phiui_busac = "n:phiui:busac", "deg, Angle between Voltage and Current"
      n_phiui_busdp = "n:phiui:busdp", "deg, Angle between Voltage and Current"
      n_phiui_busdm = "n:phiui:busdm", "deg, Angle between Voltage and Current"
      n_cfac_busac = "n:cfac:busac", "Voltage Factor"
      n_cfac_busdp = "n:cfac:busdp", "Voltage Factor"
      n_cfac_busdm = "n:cfac:busdm", "Voltage Factor"
      n_idc_busac = "n:idc:busac", "kA, Decaying Component"
      n_idc_busdp = "n:idc:busdp", "kA, Decaying Component"
      n_idc_busdm = "n:idc:busdm", "kA, Decaying Component"
      n_Iasy_busac = "n:Iasy:busac", "kA, Asymmetrical Breaking Current"
      n_Iasy_busdp = "n:Iasy:busdp", "kA, Asymmetrical Breaking Current"
      n_Iasy_busdm = "n:Iasy:busdm", "kA, Asymmetrical Breaking Current"
      n_Ithload_busac = "n:Ithload:busac", "%, Loading, Thermal equivalent Current"
      n_Ithload_busdp = "n:Ithload:busdp", "%, Loading, Thermal equivalent Current"
      n_Ithload_busdm = "n:Ithload:busdm", "%, Loading, Thermal equivalent Current"
      n_Ipload_busac = "n:Ipload:busac", "%, Loading, Peak Short-Circuit Current"
      n_Ipload_busdp = "n:Ipload:busdp", "%, Loading, Peak Short-Circuit Current"
      n_Ipload_busdm = "n:Ipload:busdm", "%, Loading, Peak Short-Circuit Current"
      n_Ithrtk_busac = "n:Ithrtk:busac", "kA, Rated Short-Time Current (Tk)"
      n_Ithrtk_busdp = "n:Ithrtk:busdp", "kA, Rated Short-Time Current (Tk)"
      n_Ithrtk_busdm = "n:Ithrtk:busdm", "kA, Rated Short-Time Current (Tk)"
      n_Tn_busac = "n:Tn:busac", "ms, Grid Time Constant"
      n_Tn_busdp = "n:Tn:busdp", "ms, Grid Time Constant"
      n_Tn_busdm = "n:Tn:busdm", "ms, Grid Time Constant"
      n_XtoR_b_busac = "n:XtoR_b:busac", "X/R ratio (dc)"
      n_XtoR_b_busdp = "n:XtoR_b:busdp", "X/R ratio (dc)"
      n_XtoR_b_busdm = "n:XtoR_b:busdm", "X/R ratio (dc)"
      n_RtoX_b_busac = "n:RtoX_b:busac", "R/X ratio (dc)"
      n_RtoX_b_busdp = "n:RtoX_b:busdp", "R/X ratio (dc)"
      n_RtoX_b_busdm = "n:RtoX_b:busdm", "R/X ratio (dc)"
      n_fctof_b_busac = "n:fctof_b:busac", "Equivalent Frequency fc/f (dc)"
      n_fctof_b_busdp = "n:fctof_b:busdp", "Equivalent Frequency fc/f (dc)"
      n_fctof_b_busdm = "n:fctof_b:busdm", "Equivalent Frequency fc/f (dc)"
      c_imax = "c:imax", "p.u., Max. current"
      e_iNoShcContr = "e:iNoShcContr", "No short-circuit contribution"
      e_iconfed = "e:iconfed", "Static converter-fed drive"
      e_psutype = "e:psutype", "Power station unit type"
      e_searchBlockTrf = "e:searchBlockTrf", "Externally modelled unit transformer"
      e_pblocktrf = "e:pblocktrf", "Unit transformer"
      e_Ikss3PF = "e:Ikss3PF", "kA, Three-phase faults, Ik''3PF"
      e_Ikss2PF = "e:Ikss2PF", "kA, Two-phase faults, Ik''2PF"
      e_Ikss1PF = "e:Ikss1PF", "kA, Single-phase faults, Ik''1PF"
      e_IkPFmax = "e:IkPFmax", "kA, Maximum current"
      e_IkPFmin = "e:IkPFmin", "kA, Minimum current"
      e_r2iec = "e:r2iec", "p.u., Resistance, r2"
      e_x2iec = "e:x2iec", "p.u., Reactance, x2"
      e_r0iec = "e:r0iec", "p.u., Resistance, r0"
      e_x0iec = "e:x0iec", "p.u., Reactance, x0"
      e_Ikss = "e:Ikss", "kA, Subtransient short-circuit current"
      e_Skss = "e:Skss", "MVA, Subtransient short-circuit level"
      e_rtox = "e:rtox", "R to X'' ratio"
      e_xtor = "e:xtor", "X'' to R ratio"
      e_r2shc = "e:r2shc", "p.u., Resistance, r2"
      e_x2shc = "e:x2shc", "p.u., Reactance, x2"
      r_pblocktrf = "r:pblocktrf", "Unit transformer"
      m_Ikss_busac_A = "m:Ikss:busac:A", "kA, Initial Short-Circuit Current"
      m_Ikss_busac_B = "m:Ikss:busac:B", "kA, Initial Short-Circuit Current"
      m_Ikss_busac_C = "m:Ikss:busac:C", "kA, Initial Short-Circuit Current"
      m_ikss_busac_A = "m:ikss:busac:A", "p.u., Initial Short-Circuit Current, Magnitude"
      m_ikss_busac_B = "m:ikss:busac:B", "p.u., Initial Short-Circuit Current, Magnitude"
      m_ikss_busac_C = "m:ikss:busac:C", "p.u., Initial Short-Circuit Current, Magnitude"
      m_Skss_busac_A = "m:Skss:busac:A", "MVA, Initial Short-Circuit Power"
      m_Skss_busac_B = "m:Skss:busac:B", "MVA, Initial Short-Circuit Power"
      m_Skss_busac_C = "m:Skss:busac:C", "MVA, Initial Short-Circuit Power"
      m_Skssx3_busac_A = "m:Skssx3:busac:A", "MVA, 3*Initial Short-Circuit Power"
      m_Skssx3_busac_B = "m:Skssx3:busac:B", "MVA, 3*Initial Short-Circuit Power"
      m_Skssx3_busac_C = "m:Skssx3:busac:C", "MVA, 3*Initial Short-Circuit Power"
      m_Skssx3_busdp = "m:Skssx3:busdp", "MVA, 3*Initial Short-Circuit Power"
      m_Skssx3_busdm = "m:Skssx3:busdm", "MVA, 3*Initial Short-Circuit Power"
      m_ip_busac_A = "m:ip:busac:A", "kA, Peak Short-Circuit Current"
      m_ip_busac_B = "m:ip:busac:B", "kA, Peak Short-Circuit Current"
      m_ip_busac_C = "m:ip:busac:C", "kA, Peak Short-Circuit Current"
      m_Z_busac_A = "m:Z:busac:A", "Ohm, Fault Impedance, Magnitude"
      m_Z_busac_B = "m:Z:busac:B", "Ohm, Fault Impedance, Magnitude"
      m_Z_busac_C = "m:Z:busac:C", "Ohm, Fault Impedance, Magnitude"
      m_phiz_busac_A = "m:phiz:busac:A", "deg, Fault Impedance, Angle"
      m_phiz_busac_B = "m:phiz:busac:B", "deg, Fault Impedance, Angle"
      m_phiz_busac_C = "m:phiz:busac:C", "deg, Fault Impedance, Angle"
      m_Zl_busac_A = "m:Zl:busac:A", "Ohm, Line-Line Fault Impedance, Magnitude"
      m_Zl_busac_B = "m:Zl:busac:B", "Ohm, Line-Line Fault Impedance, Magnitude"
      m_Zl_busac_C = "m:Zl:busac:C", "Ohm, Line-Line Fault Impedance, Magnitude"
      m_Zl_busdp = "m:Zl:busdp", "Ohm, Line-Line Fault Impedance, Magnitude"
      m_Zl_busdm = "m:Zl:busdm", "Ohm, Line-Line Fault Impedance, Magnitude"
      m_phizl_busac_A = "m:phizl:busac:A", "deg, Line-Line Fault Impedance, Angle"
      m_phizl_busac_B = "m:phizl:busac:B", "deg, Line-Line Fault Impedance, Angle"
      m_phizl_busac_C = "m:phizl:busac:C", "deg, Line-Line Fault Impedance, Angle"
      m_phizl_busdp = "m:phizl:busdp", "deg, Line-Line Fault Impedance, Angle"
      m_phizl_busdm = "m:phizl:busdm", "deg, Line-Line Fault Impedance, Angle"
      m_Z0_busac = "m:Z0:busac", "Ohm, Zero-Sequence Fault Impedance, Magnitude"
      m_Z0_busdp = "m:Z0:busdp", "Ohm, Zero-Sequence Fault Impedance, Magnitude"
      m_Z0_busdm = "m:Z0:busdm", "Ohm, Zero-Sequence Fault Impedance, Magnitude"
      m_phiz0_busac = "m:phiz0:busac", "deg, Zero-Sequence Fault Impedance, Angle"
      m_phiz0_busdp = "m:phiz0:busdp", "deg, Zero-Sequence Fault Impedance, Angle"
      m_phiz0_busdm = "m:phiz0:busdm", "deg, Zero-Sequence Fault Impedance, Angle"
      m_z0Sbase_busac = "m:z0Sbase:busac", "p.u., Zero-Sequence Fault Impedance, Magnitude"
      m_z0Sbase_busdp = "m:z0Sbase:busdp", "p.u., Zero-Sequence Fault Impedance, Magnitude"
      m_z0Sbase_busdm = "m:z0Sbase:busdm", "p.u., Zero-Sequence Fault Impedance, Magnitude"
      m_cosphiu1i1_busac = "m:cosphiu1i1:busac", "Positive-Sequence cosphi"
      m_cosphiu1i1_busdp = "m:cosphiu1i1:busdp", "Positive-Sequence cosphi"
      m_cosphiu1i1_busdm = "m:cosphiu1i1:busdm", "Positive-Sequence cosphi"
      m_I0x3watt_busac = "m:I0x3watt:busac", "kA, Earth-Fault Residual Current"
      m_I0x3watt_busdp = "m:I0x3watt:busdp", "kA, Earth-Fault Residual Current"
      m_I0x3watt_busdm = "m:I0x3watt:busdm", "kA, Earth-Fault Residual Current"
      m_I0x3var_busac = "m:I0x3var:busac", "kA, Capacitive Earth-Fault Current"
      m_I0x3var_busdp = "m:I0x3var:busdp", "kA, Capacitive Earth-Fault Current"
      m_I0x3var_busdm = "m:I0x3var:busdm", "kA, Capacitive Earth-Fault Current"
      m_Q0cap_busac = "m:Q0cap:busac", "Mvar, Q0 capacitive"
      m_Q0cap_busdp = "m:Q0cap:busdp", "Mvar, Q0 capacitive"
      m_Q0cap_busdm = "m:Q0cap:busdm", "Mvar, Q0 capacitive"
      m_Zfrel_busac = "m:Zfrel:busac", "Ohm, Relay Impedance, Magnitude"
      m_Zfrel_busdp = "m:Zfrel:busdp", "Ohm, Relay Impedance, Magnitude"
      m_Zfrel_busdm = "m:Zfrel:busdm", "Ohm, Relay Impedance, Magnitude"
      m_phizfrel_busac = "m:phizfrel:busac", "deg, Relay Impedance, Angle"
      m_phizfrel_busdp = "m:phizfrel:busdp", "deg, Relay Impedance, Angle"
      m_phizfrel_busdm = "m:phizfrel:busdm", "deg, Relay Impedance, Angle"
      m_Rfrel_busac = "m:Rfrel:busac", "Ohm, Relay Impedance, Real Part"
      m_Rfrel_busdp = "m:Rfrel:busdp", "Ohm, Relay Impedance, Real Part"
      m_Rfrel_busdm = "m:Rfrel:busdm", "Ohm, Relay Impedance, Real Part"
      m_Xfrel_busac = "m:Xfrel:busac", "Ohm, Relay Impedance, Imaginary Part"
      m_Xfrel_busdp = "m:Xfrel:busdp", "Ohm, Relay Impedance, Imaginary Part"
      m_Xfrel_busdm = "m:Xfrel:busdm", "Ohm, Relay Impedance, Imaginary Part"
      m_Zf_busac = "m:Zf:busac", "Ohm, Meas. Fault Impedance, Magnitude"
      m_Zf_busdp = "m:Zf:busdp", "Ohm, Meas. Fault Impedance, Magnitude"
      m_Zf_busdm = "m:Zf:busdm", "Ohm, Meas. Fault Impedance, Magnitude"
      m_phizf_busac = "m:phizf:busac", "deg, Meas. Fault Impedance, Angle"
      m_phizf_busdp = "m:phizf:busdp", "deg, Meas. Fault Impedance, Angle"
      m_phizf_busdm = "m:phizf:busdm", "deg, Meas. Fault Impedance, Angle"
      n_Ikss_busac_A = "n:Ikss:busac:A", "kA, Initial Short-Circuit Current"
      n_Ikss_busac_B = "n:Ikss:busac:B", "kA, Initial Short-Circuit Current"
      n_Ikss_busac_C = "n:Ikss:busac:C", "kA, Initial Short-Circuit Current"
      n_I_busac_A = "n:I:busac:A", "kA, Short-Circuit Current"
      n_I_busac_B = "n:I:busac:B", "kA, Short-Circuit Current"
      n_I_busac_C = "n:I:busac:C", "kA, Short-Circuit Current"
      n_phii_busac_A = "n:phii:busac:A", "deg, Phase Current, Angle"
      n_phii_busac_B = "n:phii:busac:B", "deg, Phase Current, Angle"
      n_phii_busac_C = "n:phii:busac:C", "deg, Phase Current, Angle"
      n_Skss_busac_A = "n:Skss:busac:A", "MVA, Initial Short-Circuit Power"
      n_Skss_busac_B = "n:Skss:busac:B", "MVA, Initial Short-Circuit Power"
      n_Skss_busac_C = "n:Skss:busac:C", "MVA, Initial Short-Circuit Power"
      n_Skssx3_busac_A = "n:Skssx3:busac:A", "MVA, 3*Initial Short-Circuit Power"
      n_Skssx3_busac_B = "n:Skssx3:busac:B", "MVA, 3*Initial Short-Circuit Power"
      n_Skssx3_busac_C = "n:Skssx3:busac:C", "MVA, 3*Initial Short-Circuit Power"
      n_Skssx3_busdp = "n:Skssx3:busdp", "MVA, 3*Initial Short-Circuit Power"
      n_Skssx3_busdm = "n:Skssx3:busdm", "MVA, 3*Initial Short-Circuit Power"
      n_ip_busac_A = "n:ip:busac:A", "kA, Peak Short-Circuit Current"
      n_ip_busac_B = "n:ip:busac:B", "kA, Peak Short-Circuit Current"
      n_ip_busac_C = "n:ip:busac:C", "kA, Peak Short-Circuit Current"
      n_Ib_busac_A = "n:Ib:busac:A", "kA, Short-Circuit Breaking Current"
      n_Ib_busac_B = "n:Ib:busac:B", "kA, Short-Circuit Breaking Current"
      n_Ib_busac_C = "n:Ib:busac:C", "kA, Short-Circuit Breaking Current"
      n_Sb_busac_A = "n:Sb:busac:A", "MVA, Short-Circuit Breaking Power"
      n_Sb_busac_B = "n:Sb:busac:B", "MVA, Short-Circuit Breaking Power"
      n_Sb_busac_C = "n:Sb:busac:C", "MVA, Short-Circuit Breaking Power"
      n_Ith_busac_A = "n:Ith:busac:A", "kA, Thermal equivalent Short-Circuit Current"
      n_Ith_busac_B = "n:Ith:busac:B", "kA, Thermal equivalent Short-Circuit Current"
      n_Ith_busac_C = "n:Ith:busac:C", "kA, Thermal equivalent Short-Circuit Current"
      n_Ithmax_busac = "n:Ithmax:busac", "kA, Max. Thermal equivalent Short-Circuit Current"
      n_Ithmax_busdp = "n:Ithmax:busdp", "kA, Max. Thermal equivalent Short-Circuit Current"
      n_Ithmax_busdm = "n:Ithmax:busdm", "kA, Max. Thermal equivalent Short-Circuit Current"
      n_Ik_busac_A = "n:Ik:busac:A", "kA, Steady-State Short-Circuit Current"
      n_Ik_busac_B = "n:Ik:busac:B", "kA, Steady-State Short-Circuit Current"
      n_Ik_busac_C = "n:Ik:busac:C", "kA, Steady-State Short-Circuit Current"
      n_R0_busac = "n:R0:busac", "Ohm, Zero-Sequence Impedance, Real Part"
      n_R0_busdp = "n:R0:busdp", "Ohm, Zero-Sequence Impedance, Real Part"
      n_R0_busdm = "n:R0:busdm", "Ohm, Zero-Sequence Impedance, Real Part"
      n_X0_busac = "n:X0:busac", "Ohm, Zero-Sequence Impedance, Imaginary Part"
      n_X0_busdp = "n:X0:busdp", "Ohm, Zero-Sequence Impedance, Imaginary Part"
      n_X0_busdm = "n:X0:busdm", "Ohm, Zero-Sequence Impedance, Imaginary Part"
      n_Z0_busac = "n:Z0:busac", "Ohm, Zero-Sequence Impedance, Magnitude"
      n_Z0_busdp = "n:Z0:busdp", "Ohm, Zero-Sequence Impedance, Magnitude"
      n_Z0_busdm = "n:Z0:busdm", "Ohm, Zero-Sequence Impedance, Magnitude"
      n_phiz0_busac = "n:phiz0:busac", "deg, Zero-Sequence Impedance, Angle"
      n_phiz0_busdp = "n:phiz0:busdp", "deg, Zero-Sequence Impedance, Angle"
      n_phiz0_busdm = "n:phiz0:busdm", "deg, Zero-Sequence Impedance, Angle"
      n_r0Sbase_busac = "n:r0Sbase:busac", "p.u., Zero-Sequence Impedance, Real Part"
      n_r0Sbase_busdp = "n:r0Sbase:busdp", "p.u., Zero-Sequence Impedance, Real Part"
      n_r0Sbase_busdm = "n:r0Sbase:busdm", "p.u., Zero-Sequence Impedance, Real Part"
      n_x0Sbase_busac = "n:x0Sbase:busac", "p.u., Zero-Sequence Impedance, Imaginary Part"
      n_x0Sbase_busdp = "n:x0Sbase:busdp", "p.u., Zero-Sequence Impedance, Imaginary Part"
      n_x0Sbase_busdm = "n:x0Sbase:busdm", "p.u., Zero-Sequence Impedance, Imaginary Part"
      n_z0Sbase_busac = "n:z0Sbase:busac", "p.u., Zero-Sequence Impedance, Magnitude"
      n_z0Sbase_busdp = "n:z0Sbase:busdp", "p.u., Zero-Sequence Impedance, Magnitude"
      n_z0Sbase_busdm = "n:z0Sbase:busdm", "p.u., Zero-Sequence Impedance, Magnitude"
      n_R1_busac = "n:R1:busac", "Ohm, Positive-Sequence Impedance, Real Part"
      n_R1_busdp = "n:R1:busdp", "Ohm, Positive-Sequence Impedance, Real Part"
      n_R1_busdm = "n:R1:busdm", "Ohm, Positive-Sequence Impedance, Real Part"
      n_X1_busac = "n:X1:busac", "Ohm, Positive-Sequence Impedance, Imaginary Part"
      n_X1_busdp = "n:X1:busdp", "Ohm, Positive-Sequence Impedance, Imaginary Part"
      n_X1_busdm = "n:X1:busdm", "Ohm, Positive-Sequence Impedance, Imaginary Part"
      n_Z1_busac = "n:Z1:busac", "Ohm, Positive-Sequence Impedance, Magnitude"
      n_Z1_busdp = "n:Z1:busdp", "Ohm, Positive-Sequence Impedance, Magnitude"
      n_Z1_busdm = "n:Z1:busdm", "Ohm, Positive-Sequence Impedance, Magnitude"
      n_phiz1_busac = "n:phiz1:busac", "deg, Positive-Sequence Impedance, Angle"
      n_phiz1_busdp = "n:phiz1:busdp", "deg, Positive-Sequence Impedance, Angle"
      n_phiz1_busdm = "n:phiz1:busdm", "deg, Positive-Sequence Impedance, Angle"
      n_r1Sbase_busac = "n:r1Sbase:busac", "p.u., Positive-Sequence Impedance, Real Part"
      n_r1Sbase_busdp = "n:r1Sbase:busdp", "p.u., Positive-Sequence Impedance, Real Part"
      n_r1Sbase_busdm = "n:r1Sbase:busdm", "p.u., Positive-Sequence Impedance, Real Part"
      n_x1Sbase_busac = "n:x1Sbase:busac", "p.u., Positive-Sequence Impedance, Imaginary Part"
      n_x1Sbase_busdp = "n:x1Sbase:busdp", "p.u., Positive-Sequence Impedance, Imaginary Part"
      n_x1Sbase_busdm = "n:x1Sbase:busdm", "p.u., Positive-Sequence Impedance, Imaginary Part"
      n_z1Sbase_busac = "n:z1Sbase:busac", "p.u., Positive-Sequence Impedance, Magnitude"
      n_z1Sbase_busdp = "n:z1Sbase:busdp", "p.u., Positive-Sequence Impedance, Magnitude"
      n_z1Sbase_busdm = "n:z1Sbase:busdm", "p.u., Positive-Sequence Impedance, Magnitude"
      n_R2_busac = "n:R2:busac", "Ohm, Negative-Sequence Impedance, Real Part"
      n_R2_busdp = "n:R2:busdp", "Ohm, Negative-Sequence Impedance, Real Part"
      n_R2_busdm = "n:R2:busdm", "Ohm, Negative-Sequence Impedance, Real Part"
      n_X2_busac = "n:X2:busac", "Ohm, Negative-Sequence Impedance, Imaginary Part"
      n_X2_busdp = "n:X2:busdp", "Ohm, Negative-Sequence Impedance, Imaginary Part"
      n_X2_busdm = "n:X2:busdm", "Ohm, Negative-Sequence Impedance, Imaginary Part"
      n_Z2_busac = "n:Z2:busac", "Ohm, Negative-Sequence Impedance, Magnitude"
      n_Z2_busdp = "n:Z2:busdp", "Ohm, Negative-Sequence Impedance, Magnitude"
      n_Z2_busdm = "n:Z2:busdm", "Ohm, Negative-Sequence Impedance, Magnitude"
      n_phiz2_busac = "n:phiz2:busac", "deg, Negative-Sequence Impedance, Angle"
      n_phiz2_busdp = "n:phiz2:busdp", "deg, Negative-Sequence Impedance, Angle"
      n_phiz2_busdm = "n:phiz2:busdm", "deg, Negative-Sequence Impedance, Angle"
      n_r2Sbase_busac = "n:r2Sbase:busac", "p.u., Negative-Sequence Impedance, Real Part"
      n_r2Sbase_busdp = "n:r2Sbase:busdp", "p.u., Negative-Sequence Impedance, Real Part"
      n_r2Sbase_busdm = "n:r2Sbase:busdm", "p.u., Negative-Sequence Impedance, Real Part"
      n_x2Sbase_busac = "n:x2Sbase:busac", "p.u., Negative-Sequence Impedance, Imaginary Part"
      n_x2Sbase_busdp = "n:x2Sbase:busdp", "p.u., Negative-Sequence Impedance, Imaginary Part"
      n_x2Sbase_busdm = "n:x2Sbase:busdm", "p.u., Negative-Sequence Impedance, Imaginary Part"
      n_z2Sbase_busac = "n:z2Sbase:busac", "p.u., Negative-Sequence Impedance, Magnitude"
      n_z2Sbase_busdp = "n:z2Sbase:busdp", "p.u., Negative-Sequence Impedance, Magnitude"
      n_z2Sbase_busdm = "n:z2Sbase:busdm", "p.u., Negative-Sequence Impedance, Magnitude"
      n_R_busac_A = "n:R:busac:A", "Ohm, Network Resistance, Line-Ground"
      n_R_busac_B = "n:R:busac:B", "Ohm, Network Resistance, Line-Ground"
      n_R_busac_C = "n:R:busac:C", "Ohm, Network Resistance, Line-Ground"
      n_X_busac_A = "n:X:busac:A", "Ohm, Network Reactance, Line-Ground"
      n_X_busac_B = "n:X:busac:B", "Ohm, Network Reactance, Line-Ground"
      n_X_busac_C = "n:X:busac:C", "Ohm, Network Reactance, Line-Ground"
      n_Z_busac_A = "n:Z:busac:A", "Ohm, Network Impedance, Line-Ground"
      n_Z_busac_B = "n:Z:busac:B", "Ohm, Network Impedance, Line-Ground"
      n_Z_busac_C = "n:Z:busac:C", "Ohm, Network Impedance, Line-Ground"
      n_phiz_busac_A = "n:phiz:busac:A", "deg, Angle of Network Impedance, Line-Ground"
      n_phiz_busac_B = "n:phiz:busac:B", "deg, Angle of Network Impedance, Line-Ground"
      n_phiz_busac_C = "n:phiz:busac:C", "deg, Angle of Network Impedance, Line-Ground"
      n_Rl_busac_A = "n:Rl:busac:A", "Ohm, Network Resistance, Line-Line"
      n_Rl_busac_B = "n:Rl:busac:B", "Ohm, Network Resistance, Line-Line"
      n_Rl_busac_C = "n:Rl:busac:C", "Ohm, Network Resistance, Line-Line"
      n_Rl_busdp = "n:Rl:busdp", "Ohm, Network Resistance, Line-Line"
      n_Rl_busdm = "n:Rl:busdm", "Ohm, Network Resistance, Line-Line"
      n_Xl_busac_A = "n:Xl:busac:A", "Ohm, Network Reactance, Line-Line"
      n_Xl_busac_B = "n:Xl:busac:B", "Ohm, Network Reactance, Line-Line"
      n_Xl_busac_C = "n:Xl:busac:C", "Ohm, Network Reactance, Line-Line"
      n_Xl_busdp = "n:Xl:busdp", "Ohm, Network Reactance, Line-Line"
      n_Xl_busdm = "n:Xl:busdm", "Ohm, Network Reactance, Line-Line"
      n_Zl_busac_A = "n:Zl:busac:A", "Ohm, Network Impedance, Line-Line"
      n_Zl_busac_B = "n:Zl:busac:B", "Ohm, Network Impedance, Line-Line"
      n_Zl_busac_C = "n:Zl:busac:C", "Ohm, Network Impedance, Line-Line"
      n_Zl_busdp = "n:Zl:busdp", "Ohm, Network Impedance, Line-Line"
      n_Zl_busdm = "n:Zl:busdm", "Ohm, Network Impedance, Line-Line"
      n_phizl_busac_A = "n:phizl:busac:A", "deg, Angle of Network Impedance, Line-Line"
      n_phizl_busac_B = "n:phizl:busac:B", "deg, Angle of Network Impedance, Line-Line"
      n_phizl_busac_C = "n:phizl:busac:C", "deg, Angle of Network Impedance, Line-Line"
      n_phizl_busdp = "n:phizl:busdp", "deg, Angle of Network Impedance, Line-Line"
      n_phizl_busdm = "n:phizl:busdm", "deg, Angle of Network Impedance, Line-Line"
      n_Rln_busac_A = "n:Rln:busac:A", "Ohm, Network Resistance, Line-Neutral"
      n_Rln_busac_B = "n:Rln:busac:B", "Ohm, Network Resistance, Line-Neutral"
      n_Rln_busac_C = "n:Rln:busac:C", "Ohm, Network Resistance, Line-Neutral"
      n_Rln_busdp = "n:Rln:busdp", "Ohm, Network Resistance, Line-Neutral"
      n_Rln_busdm = "n:Rln:busdm", "Ohm, Network Resistance, Line-Neutral"
      n_Xln_busac_A = "n:Xln:busac:A", "Ohm, Network Reactance, Line-Neutral"
      n_Xln_busac_B = "n:Xln:busac:B", "Ohm, Network Reactance, Line-Neutral"
      n_Xln_busac_C = "n:Xln:busac:C", "Ohm, Network Reactance, Line-Neutral"
      n_Xln_busdp = "n:Xln:busdp", "Ohm, Network Reactance, Line-Neutral"
      n_Xln_busdm = "n:Xln:busdm", "Ohm, Network Reactance, Line-Neutral"
      n_Zln_busac_A = "n:Zln:busac:A", "Ohm, Network Impedance, Line-Neutral"
      n_Zln_busac_B = "n:Zln:busac:B", "Ohm, Network Impedance, Line-Neutral"
      n_Zln_busac_C = "n:Zln:busac:C", "Ohm, Network Impedance, Line-Neutral"
      n_Zln_busdp = "n:Zln:busdp", "Ohm, Network Impedance, Line-Neutral"
      n_Zln_busdm = "n:Zln:busdm", "Ohm, Network Impedance, Line-Neutral"
      n_phizln_busac_A = "n:phizln:busac:A", "deg, Angle of Network Impedance, Line-Neutral"
      n_phizln_busac_B = "n:phizln:busac:B", "deg, Angle of Network Impedance, Line-Neutral"
      n_phizln_busac_C = "n:phizln:busac:C", "deg, Angle of Network Impedance, Line-Neutral"
      n_phizln_busdp = "n:phizln:busdp", "deg, Angle of Network Impedance, Line-Neutral"
      n_phizln_busdm = "n:phizln:busdm", "deg, Angle of Network Impedance, Line-Neutral"
      n_Rn_busac = "n:Rn:busac", "Ohm, Network Resistance, Neutral"
      n_Rn_busdp = "n:Rn:busdp", "Ohm, Network Resistance, Neutral"
      n_Rn_busdm = "n:Rn:busdm", "Ohm, Network Resistance, Neutral"
      n_Xn_busac = "n:Xn:busac", "Ohm, Network Reactance, Neutral"
      n_Xn_busdp = "n:Xn:busdp", "Ohm, Network Reactance, Neutral"
      n_Xn_busdm = "n:Xn:busdm", "Ohm, Network Reactance, Neutral"
      n_Zn_busac = "n:Zn:busac", "Ohm, Network Impedance, Neutral"
      n_Zn_busdp = "n:Zn:busdp", "Ohm, Network Impedance, Neutral"
      n_Zn_busdm = "n:Zn:busdm", "Ohm, Network Impedance, Neutral"
      n_phizn_busac = "n:phizn:busac", "deg, Angle of Network Impedance, Neutral"
      n_phizn_busdp = "n:phizn:busdp", "deg, Angle of Network Impedance, Neutral"
      n_phizn_busdm = "n:phizn:busdm", "deg, Angle of Network Impedance, Neutral"
      n_eff_busac_A = "n:eff:busac:A", "Earth Fault Factor"
      n_eff_busac_B = "n:eff:busac:B", "Earth Fault Factor"
      n_eff_busac_C = "n:eff:busac:C", "Earth Fault Factor"
      n_eff_busdp = "n:eff:busdp", "Earth Fault Factor"
      n_eff_busdm = "n:eff:busdm", "Earth Fault Factor"
      n_X0toX1_busac = "n:X0toX1:busac", "X0/X1"
      n_X0toX1_busdp = "n:X0toX1:busdp", "X0/X1"
      n_X0toX1_busdm = "n:X0toX1:busdm", "X0/X1"
      n_in_busac = "n:in:busac", "p.u., Neutral Current, Magnitude"
      n_in_busdp = "n:in:busdp", "p.u., Neutral Current, Magnitude"
      n_in_busdm = "n:in:busdm", "p.u., Neutral Current, Magnitude"
      n_In_busac = "n:In:busac", "kA, Neutral Current, Magnitude"
      n_In_busdp = "n:In:busdp", "kA, Neutral Current, Magnitude"
      n_In_busdm = "n:In:busdm", "kA, Neutral Current, Magnitude"
      n_phiin_busac = "n:phiin:busac", "deg, Neutral Current, Angle"
      n_phiin_busdp = "n:phiin:busdp", "deg, Neutral Current, Angle"
      n_phiin_busdm = "n:phiin:busdm", "deg, Neutral Current, Angle"
      n_i0r_busac = "n:i0r:busac", "p.u., Zero-Sequence Current, Real Part"
      n_i0r_busdp = "n:i0r:busdp", "p.u., Zero-Sequence Current, Real Part"
      n_i0r_busdm = "n:i0r:busdm", "p.u., Zero-Sequence Current, Real Part"
      n_i0i_busac = "n:i0i:busac", "p.u., Zero-Sequence Current, Imaginary Part"
      n_i0i_busdp = "n:i0i:busdp", "p.u., Zero-Sequence Current, Imaginary Part"
      n_i0i_busdm = "n:i0i:busdm", "p.u., Zero-Sequence Current, Imaginary Part"
      n_i0_busac = "n:i0:busac", "p.u., Zero-Sequence Current, Magnitude"
      n_i0_busdp = "n:i0:busdp", "p.u., Zero-Sequence Current, Magnitude"
      n_i0_busdm = "n:i0:busdm", "p.u., Zero-Sequence Current, Magnitude"
      n_I0_busac = "n:I0:busac", "kA, Zero-Sequence Current, Magnitude"
      n_I0_busdp = "n:I0:busdp", "kA, Zero-Sequence Current, Magnitude"
      n_I0_busdm = "n:I0:busdm", "kA, Zero-Sequence Current, Magnitude"
      n_I0x3_busac = "n:I0x3:busac", "kA, 3*I0"
      n_I0x3_busdp = "n:I0x3:busdp", "kA, 3*I0"
      n_I0x3_busdm = "n:I0x3:busdm", "kA, 3*I0"
      n_phii0_busac = "n:phii0:busac", "deg, Zero-Sequence Current, Angle"
      n_phii0_busdp = "n:phii0:busdp", "deg, Zero-Sequence Current, Angle"
      n_phii0_busdm = "n:phii0:busdm", "deg, Zero-Sequence Current, Angle"
      n_i1r_busac = "n:i1r:busac", "p.u., Positive-Sequence Current, Real Part"
      n_i1r_busdp = "n:i1r:busdp", "p.u., Positive-Sequence Current, Real Part"
      n_i1r_busdm = "n:i1r:busdm", "p.u., Positive-Sequence Current, Real Part"
      n_i1i_busac = "n:i1i:busac", "p.u., Positive-Sequence Current, Imaginary Part"
      n_i1i_busdp = "n:i1i:busdp", "p.u., Positive-Sequence Current, Imaginary Part"
      n_i1i_busdm = "n:i1i:busdm", "p.u., Positive-Sequence Current, Imaginary Part"
      n_i1_busac = "n:i1:busac", "p.u., Positive-Sequence Current, Magnitude"
      n_i1_busdp = "n:i1:busdp", "p.u., Positive-Sequence Current, Magnitude"
      n_i1_busdm = "n:i1:busdm", "p.u., Positive-Sequence Current, Magnitude"
      n_I1_busac = "n:I1:busac", "kA, Positive-Sequence Current, Magnitude"
      n_I1_busdp = "n:I1:busdp", "kA, Positive-Sequence Current, Magnitude"
      n_I1_busdm = "n:I1:busdm", "kA, Positive-Sequence Current, Magnitude"
      n_phii1_busac = "n:phii1:busac", "deg, Positive-Sequence Current, Angle"
      n_phii1_busdp = "n:phii1:busdp", "deg, Positive-Sequence Current, Angle"
      n_phii1_busdm = "n:phii1:busdm", "deg, Positive-Sequence Current, Angle"
      n_i2r_busac = "n:i2r:busac", "p.u., Negative-Sequence Current, Real Part"
      n_i2r_busdp = "n:i2r:busdp", "p.u., Negative-Sequence Current, Real Part"
      n_i2r_busdm = "n:i2r:busdm", "p.u., Negative-Sequence Current, Real Part"
      n_i2i_busac = "n:i2i:busac", "p.u., Negative-Sequence Current, Imaginary Part"
      n_i2i_busdp = "n:i2i:busdp", "p.u., Negative-Sequence Current, Imaginary Part"
      n_i2i_busdm = "n:i2i:busdm", "p.u., Negative-Sequence Current, Imaginary Part"
      n_i2_busac = "n:i2:busac", "p.u., Negative-Sequence Current, Magnitude"
      n_i2_busdp = "n:i2:busdp", "p.u., Negative-Sequence Current, Magnitude"
      n_i2_busdm = "n:i2:busdm", "p.u., Negative-Sequence Current, Magnitude"
      n_I2_busac = "n:I2:busac", "kA, Negative-Sequence Current, Magnitude"
      n_I2_busdp = "n:I2:busdp", "kA, Negative-Sequence Current, Magnitude"
      n_I2_busdm = "n:I2:busdm", "kA, Negative-Sequence Current, Magnitude"
      n_phii2_busac = "n:phii2:busac", "deg, Negative-Sequence Current, Angle"
      n_phii2_busdp = "n:phii2:busdp", "deg, Negative-Sequence Current, Angle"
      n_phii2_busdm = "n:phii2:busdm", "deg, Negative-Sequence Current, Angle"
      n_du_busac_A = "n:du:busac:A", "%, Voltage Deviation"
      n_du_busac_B = "n:du:busac:B", "%, Voltage Deviation"
      n_du_busac_C = "n:du:busac:C", "%, Voltage Deviation"
      n_urpref_busac_A = "n:urpref:busac:A", "p.u., Pre-fault Voltage, Real Part"
      n_urpref_busac_B = "n:urpref:busac:B", "p.u., Pre-fault Voltage, Real Part"
      n_urpref_busac_C = "n:urpref:busac:C", "p.u., Pre-fault Voltage, Real Part"
      n_uipref_busac_A = "n:uipref:busac:A", "p.u., Pre-fault Voltage, Imaginary Part"
      n_uipref_busac_B = "n:uipref:busac:B", "p.u., Pre-fault Voltage, Imaginary Part"
      n_uipref_busac_C = "n:uipref:busac:C", "p.u., Pre-fault Voltage, Imaginary Part"
      n_upref_busac_A = "n:upref:busac:A", "p.u., Pre-fault Voltage, Magnitude"
      n_upref_busac_B = "n:upref:busac:B", "p.u., Pre-fault Voltage, Magnitude"
      n_upref_busac_C = "n:upref:busac:C", "p.u., Pre-fault Voltage, Magnitude"
      n_uprefpc_busac_A = "n:uprefpc:busac:A", "%, Pre-fault Voltage, Magnitude"
      n_uprefpc_busac_B = "n:uprefpc:busac:B", "%, Pre-fault Voltage, Magnitude"
      n_uprefpc_busac_C = "n:uprefpc:busac:C", "%, Pre-fault Voltage, Magnitude"
      n_Upref_busac_A = "n:Upref:busac:A", "kV, Line-Ground Pre-fault Voltage, Magnitude"
      n_Upref_busac_B = "n:Upref:busac:B", "kV, Line-Ground Pre-fault Voltage, Magnitude"
      n_Upref_busac_C = "n:Upref:busac:C", "kV, Line-Ground Pre-fault Voltage, Magnitude"
      n_Ulpref_busac_A = "n:Ulpref:busac:A", "kV, Line-Line Pre-fault Voltage, Magnitude"
      n_Ulpref_busac_B = "n:Ulpref:busac:B", "kV, Line-Line Pre-fault Voltage, Magnitude"
      n_Ulpref_busac_C = "n:Ulpref:busac:C", "kV, Line-Line Pre-fault Voltage, Magnitude"
      n_phiupref_busac_A = "n:phiupref:busac:A", "deg, Pre-fault Voltage, Angle"
      n_phiupref_busac_B = "n:phiupref:busac:B", "deg, Pre-fault Voltage, Angle"
      n_phiupref_busac_C = "n:phiupref:busac:C", "deg, Pre-fault Voltage, Angle"
      n_phiui_busac_A = "n:phiui:busac:A", "deg, Angle between Voltage and Current"
      n_phiui_busac_B = "n:phiui:busac:B", "deg, Angle between Voltage and Current"
      n_phiui_busac_C = "n:phiui:busac:C", "deg, Angle between Voltage and Current"
      n_phiu1i1_busac = "n:phiu1i1:busac", "deg, Angle between Voltage and Current in positive sequence system"
      n_phiu1i1_busdp = "n:phiu1i1:busdp", "deg, Angle between Voltage and Current in positive sequence system"
      n_phiu1i1_busdm = "n:phiu1i1:busdm", "deg, Angle between Voltage and Current in positive sequence system"
      n_phiu2i2_busac = "n:phiu2i2:busac", "deg, Angle between Voltage and Current in negative sequence system"
      n_phiu2i2_busdp = "n:phiu2i2:busdp", "deg, Angle between Voltage and Current in negative sequence system"
      n_phiu2i2_busdm = "n:phiu2i2:busdm", "deg, Angle between Voltage and Current in negative sequence system"
      n_phiu0i0_busac = "n:phiu0i0:busac", "deg, Angle between Voltage and Current in zero-sequence system"
      n_phiu0i0_busdp = "n:phiu0i0:busdp", "deg, Angle between Voltage and Current in zero-sequence system"
      n_phiu0i0_busdm = "n:phiu0i0:busdm", "deg, Angle between Voltage and Current in zero-sequence system"
      c_z1_r = "c:z1:r", "p.u., Subtransient impedance, Real Part"
      c_z1_i = "c:z1:i", "p.u., Subtransient impedance, Imaginary Part"
      c_z1_phi = "c:z1:phi", "deg, Subtransient impedance, Angle"
      c_z1_mag = "c:z1:mag", "p.u., Subtransient impedance, Magnitude"
      c_z1_phirad = "c:z1:phirad", "rad, Subtransient impedance, Angle"
      c_z2_r = "c:z2:r", "p.u., Negative sequence impedance, Real Part"
      c_z2_i = "c:z2:i", "p.u., Negative sequence impedance, Imaginary Part"
      c_z2_phi = "c:z2:phi", "deg, Negative sequence impedance, Angle"
      c_z2_mag = "c:z2:mag", "p.u., Negative sequence impedance, Magnitude"
      c_z2_phirad = "c:z2:phirad", "rad, Negative sequence impedance, Angle"
      c_z0_r = "c:z0:r", "p.u., Zero sequence impedance, Real Part"
      c_z0_i = "c:z0:i", "p.u., Zero sequence impedance, Imaginary Part"
      c_z0_phi = "c:z0:phi", "deg, Zero sequence impedance, Angle"
      c_z0_mag = "c:z0:mag", "p.u., Zero sequence impedance, Magnitude"
      c_z0_phirad = "c:z0:phirad", "rad, Zero sequence impedance, Angle"

    class ElmVscmono(Enum):
      _init_ = 'value __doc__'

      m_u0_busac = "m:u0:busac", "p.u., Zero-Sequence-Voltage, Magnitude"
      m_u0_busdc = "m:u0:busdc", "p.u., Zero-Sequence-Voltage, Magnitude"
      m_phiu0_busac = "m:phiu0:busac", "deg, Zero-Sequence-Voltage, Angle"
      m_phiu0_busdc = "m:phiu0:busdc", "deg, Zero-Sequence-Voltage, Angle"
      m_u1r_busac = "m:u1r:busac", "p.u., Positive-Sequence-Voltage, Real Part"
      m_u1r_busdc = "m:u1r:busdc", "p.u., Positive-Sequence-Voltage, Real Part"
      m_u1i_busac = "m:u1i:busac", "p.u., Positive-Sequence-Voltage, Imaginary Part"
      m_u1i_busdc = "m:u1i:busdc", "p.u., Positive-Sequence-Voltage, Imaginary Part"
      m_u1_busac = "m:u1:busac", "p.u., Positive-Sequence-Voltage, Magnitude"
      m_u1_busdc = "m:u1:busdc", "p.u., Positive-Sequence-Voltage, Magnitude"
      m_U1_busac = "m:U1:busac", "kV, Line-Ground Positive-Sequence-Voltage, Magnitude"
      m_U1_busdc = "m:U1:busdc", "kV, Line-Ground Positive-Sequence-Voltage, Magnitude"
      m_U1l_busac = "m:U1l:busac", "kV, Line-Line Positive-Sequence-Voltage, Magnitude"
      m_U1l_busdc = "m:U1l:busdc", "kV, Line-Line Positive-Sequence-Voltage, Magnitude"
      m_phiu1_busac = "m:phiu1:busac", "deg, Positive-Sequence-Voltage, Angle"
      m_phiu1_busdc = "m:phiu1:busdc", "deg, Positive-Sequence-Voltage, Angle"
      m_u2_busac = "m:u2:busac", "p.u., Negative-Sequence-Voltage, Magnitude"
      m_u2_busdc = "m:u2:busdc", "p.u., Negative-Sequence-Voltage, Magnitude"
      m_phiu2_busac = "m:phiu2:busac", "deg, Negative-Sequence-Voltage, Angle"
      m_phiu2_busdc = "m:phiu2:busdc", "deg, Negative-Sequence-Voltage, Angle"
      m_inet_busac_A = "m:inet:busac:A", "p.u., Phase Current, Magnitude, referred to network"
      m_inet_busac_B = "m:inet:busac:B", "p.u., Phase Current, Magnitude, referred to network"
      m_inet_busac_C = "m:inet:busac:C", "p.u., Phase Current, Magnitude, referred to network"
      m_inet_busdc = "m:inet:busdc", "p.u., Phase Current, Magnitude, referred to network"
      m_I_busac_A = "m:I:busac:A", "kA, Phase Current, Magnitude"
      m_I_busac_B = "m:I:busac:B", "kA, Phase Current, Magnitude"
      m_I_busac_C = "m:I:busac:C", "kA, Phase Current, Magnitude"
      m_I_busdc = "m:I:busdc", "kA, Phase Current, Magnitude"
      m_phii_busac_A = "m:phii:busac:A", "deg, Phase Current, Angle"
      m_phii_busac_B = "m:phii:busac:B", "deg, Phase Current, Angle"
      m_phii_busac_C = "m:phii:busac:C", "deg, Phase Current, Angle"
      m_phii_busdc = "m:phii:busdc", "deg, Phase Current, Angle"
      m_i0r_busac = "m:i0r:busac", "p.u., Zero-Sequence Current, Real Part"
      m_i0r_busdc = "m:i0r:busdc", "p.u., Zero-Sequence Current, Real Part"
      m_i0i_busac = "m:i0i:busac", "p.u., Zero-Sequence Current, Imaginary Part"
      m_i0i_busdc = "m:i0i:busdc", "p.u., Zero-Sequence Current, Imaginary Part"
      m_i0_busac = "m:i0:busac", "p.u., Zero-Sequence Current, Magnitude"
      m_i0_busdc = "m:i0:busdc", "p.u., Zero-Sequence Current, Magnitude"
      m_I0_busac = "m:I0:busac", "kA, Zero-Sequence Current, Magnitude"
      m_I0_busdc = "m:I0:busdc", "kA, Zero-Sequence Current, Magnitude"
      m_I0x3_busac = "m:I0x3:busac", "kA, 3*I0"
      m_I0x3_busdc = "m:I0x3:busdc", "kA, 3*I0"
      m_phii0_busac = "m:phii0:busac", "deg, Zero-Sequence Current, Angle"
      m_phii0_busdc = "m:phii0:busdc", "deg, Zero-Sequence Current, Angle"
      m_i1r_busac = "m:i1r:busac", "p.u., Positive-Sequence Current, Real Part"
      m_i1r_busdc = "m:i1r:busdc", "p.u., Positive-Sequence Current, Real Part"
      m_i1i_busac = "m:i1i:busac", "p.u., Positive-Sequence Current, Imaginary Part"
      m_i1i_busdc = "m:i1i:busdc", "p.u., Positive-Sequence Current, Imaginary Part"
      m_i1_busac = "m:i1:busac", "p.u., Positive-Sequence Current, Magnitude"
      m_i1_busdc = "m:i1:busdc", "p.u., Positive-Sequence Current, Magnitude"
      m_I1_busac = "m:I1:busac", "kA, Positive-Sequence Current, Magnitude"
      m_I1_busdc = "m:I1:busdc", "kA, Positive-Sequence Current, Magnitude"
      m_phii1_busac = "m:phii1:busac", "deg, Positive-Sequence Current, Angle"
      m_phii1_busdc = "m:phii1:busdc", "deg, Positive-Sequence Current, Angle"
      m_i2r_busac = "m:i2r:busac", "p.u., Negative-Sequence Current, Real Part"
      m_i2r_busdc = "m:i2r:busdc", "p.u., Negative-Sequence Current, Real Part"
      m_i2i_busac = "m:i2i:busac", "p.u., Negative-Sequence Current, Imaginary Part"
      m_i2i_busdc = "m:i2i:busdc", "p.u., Negative-Sequence Current, Imaginary Part"
      m_i2_busac = "m:i2:busac", "p.u., Negative-Sequence Current, Magnitude"
      m_i2_busdc = "m:i2:busdc", "p.u., Negative-Sequence Current, Magnitude"
      m_I2_busac = "m:I2:busac", "kA, Negative-Sequence Current, Magnitude"
      m_I2_busdc = "m:I2:busdc", "kA, Negative-Sequence Current, Magnitude"
      m_phii2_busac = "m:phii2:busac", "deg, Negative-Sequence Current, Angle"
      m_phii2_busdc = "m:phii2:busdc", "deg, Negative-Sequence Current, Angle"
      m_ubfacI_busac = "m:ubfacI:busac", "%, Current Unbalance Factor"
      m_ubfacI_busdc = "m:ubfacI:busdc", "%, Current Unbalance Factor"
      m_ubfacS_busac = "m:ubfacS:busac", "%, Power Unbalance Factor"
      m_ubfacS_busdc = "m:ubfacS:busdc", "%, Power Unbalance Factor"
      m_P_busac_A = "m:P:busac:A", "MW, Active Power"
      m_P_busac_B = "m:P:busac:B", "MW, Active Power"
      m_P_busac_C = "m:P:busac:C", "MW, Active Power"
      m_P_busdc = "m:P:busdc", "MW, Active Power"
      m_Q_busac_A = "m:Q:busac:A", "Mvar, Reactive Power"
      m_Q_busac_B = "m:Q:busac:B", "Mvar, Reactive Power"
      m_Q_busac_C = "m:Q:busac:C", "Mvar, Reactive Power"
      m_Q_busdc = "m:Q:busdc", "Mvar, Reactive Power"
      m_S_busac_A = "m:S:busac:A", "MVA, Apparent Power"
      m_S_busac_B = "m:S:busac:B", "MVA, Apparent Power"
      m_S_busac_C = "m:S:busac:C", "MVA, Apparent Power"
      m_S_busdc = "m:S:busdc", "MVA, Apparent Power"
      m_cosphi_busac_A = "m:cosphi:busac:A", "Power Factor"
      m_cosphi_busac_B = "m:cosphi:busac:B", "Power Factor"
      m_cosphi_busac_C = "m:cosphi:busac:C", "Power Factor"
      m_cosphi_busdc = "m:cosphi:busdc", "Power Factor"
      m_tanphi_busac_A = "m:tanphi:busac:A", "tan(phi)"
      m_tanphi_busac_B = "m:tanphi:busac:B", "tan(phi)"
      m_tanphi_busac_C = "m:tanphi:busac:C", "tan(phi)"
      m_tanphi_busdc = "m:tanphi:busdc", "tan(phi)"
      m_Psum_busac = "m:Psum:busac", "MW, Total Active Power"
      m_Psum_busdc = "m:Psum:busdc", "MW, Total Active Power"
      m_Qsum_busac = "m:Qsum:busac", "Mvar, Total Reactive Power"
      m_Qsum_busdc = "m:Qsum:busdc", "Mvar, Total Reactive Power"
      m_Ssum_busac = "m:Ssum:busac", "MVA, Total Apparent Power"
      m_Ssum_busdc = "m:Ssum:busdc", "MVA, Total Apparent Power"
      m_cosphisum_busac = "m:cosphisum:busac", "Total Power Factor"
      m_cosphisum_busdc = "m:cosphisum:busdc", "Total Power Factor"
      m_tanphisum_busac = "m:tanphisum:busac", "Total tan(phi)"
      m_tanphisum_busdc = "m:tanphisum:busdc", "Total tan(phi)"
      m_Pln_busac_A = "m:Pln:busac:A", "MW, Active Power (Line-Neutral)"
      m_Pln_busac_B = "m:Pln:busac:B", "MW, Active Power (Line-Neutral)"
      m_Pln_busac_C = "m:Pln:busac:C", "MW, Active Power (Line-Neutral)"
      m_Pln_busdc = "m:Pln:busdc", "MW, Active Power (Line-Neutral)"
      m_Qln_busac_A = "m:Qln:busac:A", "Mvar, Reactive Power (Line-Neutral)"
      m_Qln_busac_B = "m:Qln:busac:B", "Mvar, Reactive Power (Line-Neutral)"
      m_Qln_busac_C = "m:Qln:busac:C", "Mvar, Reactive Power (Line-Neutral)"
      m_Qln_busdc = "m:Qln:busdc", "Mvar, Reactive Power (Line-Neutral)"
      m_P1_busac = "m:P1:busac", "MW, Positive Sequence Active Power"
      m_P1_busdc = "m:P1:busdc", "MW, Positive Sequence Active Power"
      m_Q1_busac = "m:Q1:busac", "Mvar, Positive Sequence Reactive Power"
      m_Q1_busdc = "m:Q1:busdc", "Mvar, Positive Sequence Reactive Power"
      m_P2_busac = "m:P2:busac", "MW, Negative Sequence Active Power"
      m_P2_busdc = "m:P2:busdc", "MW, Negative Sequence Active Power"
      m_Q2_busac = "m:Q2:busac", "Mvar, Negative Sequence Reactive Power"
      m_Q2_busdc = "m:Q2:busdc", "Mvar, Negative Sequence Reactive Power"
      m_P0_busac = "m:P0:busac", "MW, Zero Sequence Active Power"
      m_P0_busdc = "m:P0:busdc", "MW, Zero Sequence Active Power"
      m_Q0_busac = "m:Q0:busac", "Mvar, Zero Sequence Reactive Power"
      m_Q0_busdc = "m:Q0:busdc", "Mvar, Zero Sequence Reactive Power"
      m_i1P_busac = "m:i1P:busac", "p.u., Positive-Sequence Active Current"
      m_i1P_busdc = "m:i1P:busdc", "p.u., Positive-Sequence Active Current"
      m_i1Q_busac = "m:i1Q:busac", "p.u., Positive-Sequence Reactive Current"
      m_i1Q_busdc = "m:i1Q:busdc", "p.u., Positive-Sequence Reactive Current"
      m_I1P_busac = "m:I1P:busac", "kA, Positive-Sequence Active Current"
      m_I1P_busdc = "m:I1P:busdc", "kA, Positive-Sequence Active Current"
      m_I1Q_busac = "m:I1Q:busac", "kA, Positive-Sequence Reactive Current"
      m_I1Q_busdc = "m:I1Q:busdc", "kA, Positive-Sequence Reactive Current"
      m_i2P_busac = "m:i2P:busac", "p.u., Negative-Sequence Active Current"
      m_i2P_busdc = "m:i2P:busdc", "p.u., Negative-Sequence Active Current"
      m_i2Q_busac = "m:i2Q:busac", "p.u., Negative-Sequence Reactive Current"
      m_i2Q_busdc = "m:i2Q:busdc", "p.u., Negative-Sequence Reactive Current"
      m_I2P_busac = "m:I2P:busac", "kA, Negative-Sequence Active Current"
      m_I2P_busdc = "m:I2P:busdc", "kA, Negative-Sequence Active Current"
      m_I2Q_busac = "m:I2Q:busac", "kA, Negative-Sequence Reactive Current"
      m_I2Q_busdc = "m:I2Q:busdc", "kA, Negative-Sequence Reactive Current"
      m_ur_busac_A = "m:ur:busac:A", "p.u., Phase Voltage, Real Part"
      m_ur_busac_B = "m:ur:busac:B", "p.u., Phase Voltage, Real Part"
      m_ur_busac_C = "m:ur:busac:C", "p.u., Phase Voltage, Real Part"
      m_ur_busdc = "m:ur:busdc", "p.u., Phase Voltage, Real Part"
      m_ui_busac_A = "m:ui:busac:A", "p.u., Phase Voltage, Imaginary Part"
      m_ui_busac_B = "m:ui:busac:B", "p.u., Phase Voltage, Imaginary Part"
      m_ui_busac_C = "m:ui:busac:C", "p.u., Phase Voltage, Imaginary Part"
      m_ui_busdc = "m:ui:busdc", "p.u., Phase Voltage, Imaginary Part"
      m_u_busac_A = "m:u:busac:A", "p.u., Phase Voltage, Magnitude"
      m_u_busac_B = "m:u:busac:B", "p.u., Phase Voltage, Magnitude"
      m_u_busac_C = "m:u:busac:C", "p.u., Phase Voltage, Magnitude"
      m_u_busdc = "m:u:busdc", "p.u., Phase Voltage, Magnitude"
      m_ir_busac_A = "m:ir:busac:A", "p.u., Phase Current, Real Part"
      m_ir_busac_B = "m:ir:busac:B", "p.u., Phase Current, Real Part"
      m_ir_busac_C = "m:ir:busac:C", "p.u., Phase Current, Real Part"
      m_ir_busdc = "m:ir:busdc", "p.u., Phase Current, Real Part"
      m_ii_busac_A = "m:ii:busac:A", "p.u., Phase Current, Imaginary Part"
      m_ii_busac_B = "m:ii:busac:B", "p.u., Phase Current, Imaginary Part"
      m_ii_busac_C = "m:ii:busac:C", "p.u., Phase Current, Imaginary Part"
      m_ii_busdc = "m:ii:busdc", "p.u., Phase Current, Imaginary Part"
      m_i_busac_A = "m:i:busac:A", "p.u., Phase Current, Magnitude"
      m_i_busac_B = "m:i:busac:B", "p.u., Phase Current, Magnitude"
      m_i_busac_C = "m:i:busac:C", "p.u., Phase Current, Magnitude"
      m_i_busdc = "m:i:busdc", "p.u., Phase Current, Magnitude"
      m_phiui_busac_A = "m:phiui:busac:A", "deg, Angle between Voltage and Current"
      m_phiui_busac_B = "m:phiui:busac:B", "deg, Angle between Voltage and Current"
      m_phiui_busac_C = "m:phiui:busac:C", "deg, Angle between Voltage and Current"
      m_phiui_busdc = "m:phiui:busdc", "deg, Angle between Voltage and Current"
      m_phiu0i0_busac = "m:phiu0i0:busac", "deg, Angle between Voltage and Current in zero-sequence system"
      m_phiu0i0_busdc = "m:phiu0i0:busdc", "deg, Angle between Voltage and Current in zero-sequence system"
      m_phiu1i1_busac = "m:phiu1i1:busac", "deg, Angle between Voltage and Current in positive sequence system"
      m_phiu1i1_busdc = "m:phiu1i1:busdc", "deg, Angle between Voltage and Current in positive sequence system"
      m_phiu2i2_busac = "m:phiu2i2:busac", "deg, Angle between Voltage and Current in negative sequence system"
      m_phiu2i2_busdc = "m:phiu2i2:busdc", "deg, Angle between Voltage and Current in negative sequence system"
      m_Tfct_busac = "m:Tfct:busac", "s, Fault Clearing Time"
      m_Tfct_busdc = "m:Tfct:busdc", "s, Fault Clearing Time"
      m_TfctPh_busac_A = "m:TfctPh:busac:A", "s, Fault Clearing Time"
      m_TfctPh_busac_B = "m:TfctPh:busac:B", "s, Fault Clearing Time"
      m_TfctPh_busac_C = "m:TfctPh:busac:C", "s, Fault Clearing Time"
      m_TfctPh_busdc = "m:TfctPh:busdc", "s, Fault Clearing Time"
      m_Brkload_busac = "m:Brkload:busac", "%, Max. Breaker Loading"
      m_Brkload_busdc = "m:Brkload:busdc", "%, Max. Breaker Loading"
      m_BrkloadPh_busac_A = "m:BrkloadPh:busac:A", "%, Breaker Loading"
      m_BrkloadPh_busac_B = "m:BrkloadPh:busac:B", "%, Breaker Loading"
      m_BrkloadPh_busac_C = "m:BrkloadPh:busac:C", "%, Breaker Loading"
      m_BrkloadPh_busdc = "m:BrkloadPh:busdc", "%, Breaker Loading"
      n_ur_busac_A = "n:ur:busac:A", "p.u., Line-Ground Voltage, Real Part"
      n_ur_busac_B = "n:ur:busac:B", "p.u., Line-Ground Voltage, Real Part"
      n_ur_busac_C = "n:ur:busac:C", "p.u., Line-Ground Voltage, Real Part"
      n_ur_busdc = "n:ur:busdc", "p.u., Line-Ground Voltage, Real Part"
      n_ui_busac_A = "n:ui:busac:A", "p.u., Line-Ground Voltage, Imaginary Part"
      n_ui_busac_B = "n:ui:busac:B", "p.u., Line-Ground Voltage, Imaginary Part"
      n_ui_busac_C = "n:ui:busac:C", "p.u., Line-Ground Voltage, Imaginary Part"
      n_ui_busdc = "n:ui:busdc", "p.u., Line-Ground Voltage, Imaginary Part"
      n_u_busac_A = "n:u:busac:A", "p.u., Line-Ground Voltage, Magnitude"
      n_u_busac_B = "n:u:busac:B", "p.u., Line-Ground Voltage, Magnitude"
      n_u_busac_C = "n:u:busac:C", "p.u., Line-Ground Voltage, Magnitude"
      n_u_busdc = "n:u:busdc", "p.u., Line-Ground Voltage, Magnitude"
      n_upc_busac_A = "n:upc:busac:A", "%, Line-Ground Voltage, Magnitude"
      n_upc_busac_B = "n:upc:busac:B", "%, Line-Ground Voltage, Magnitude"
      n_upc_busac_C = "n:upc:busac:C", "%, Line-Ground Voltage, Magnitude"
      n_upc_busdc = "n:upc:busdc", "%, Line-Ground Voltage, Magnitude"
      n_U_busac_A = "n:U:busac:A", "kV, Line-Ground Voltage, Magnitude"
      n_U_busac_B = "n:U:busac:B", "kV, Line-Ground Voltage, Magnitude"
      n_U_busac_C = "n:U:busac:C", "kV, Line-Ground Voltage, Magnitude"
      n_U_busdc = "n:U:busdc", "kV, Line-Ground Voltage, Magnitude"
      n_phiu_busac_A = "n:phiu:busac:A", "deg, Line-Ground Voltage, Angle"
      n_phiu_busac_B = "n:phiu:busac:B", "deg, Line-Ground Voltage, Angle"
      n_phiu_busac_C = "n:phiu:busac:C", "deg, Line-Ground Voltage, Angle"
      n_phiu_busdc = "n:phiu:busdc", "deg, Line-Ground Voltage, Angle"
      n_phiurel_busac = "n:phiurel:busac", "deg, Voltage, Relative Angle"
      n_phiurel_busdc = "n:phiurel:busdc", "deg, Voltage, Relative Angle"
      n_ul_busac_A = "n:ul:busac:A", "p.u., Line-Line Voltage, Magnitude"
      n_ul_busac_B = "n:ul:busac:B", "p.u., Line-Line Voltage, Magnitude"
      n_ul_busac_C = "n:ul:busac:C", "p.u., Line-Line Voltage, Magnitude"
      n_ul_busdc = "n:ul:busdc", "p.u., Line-Line Voltage, Magnitude"
      n_ulpc_busac_A = "n:ulpc:busac:A", "%, Line-Line Voltage, Magnitude"
      n_ulpc_busac_B = "n:ulpc:busac:B", "%, Line-Line Voltage, Magnitude"
      n_ulpc_busac_C = "n:ulpc:busac:C", "%, Line-Line Voltage, Magnitude"
      n_ulpc_busdc = "n:ulpc:busdc", "%, Line-Line Voltage, Magnitude"
      n_Ul_busac_A = "n:Ul:busac:A", "kV, Line-Line Voltage, Magnitude"
      n_Ul_busac_B = "n:Ul:busac:B", "kV, Line-Line Voltage, Magnitude"
      n_Ul_busac_C = "n:Ul:busac:C", "kV, Line-Line Voltage, Magnitude"
      n_Ul_busdc = "n:Ul:busdc", "kV, Line-Line Voltage, Magnitude"
      n_phiul_busac_A = "n:phiul:busac:A", "deg, Line-Line Voltage, Angle"
      n_phiul_busac_B = "n:phiul:busac:B", "deg, Line-Line Voltage, Angle"
      n_phiul_busac_C = "n:phiul:busac:C", "deg, Line-Line Voltage, Angle"
      n_phiul_busdc = "n:phiul:busdc", "deg, Line-Line Voltage, Angle"
      n_u0_busac = "n:u0:busac", "p.u., Zero-Sequence Voltage, Magnitude"
      n_u0_busdc = "n:u0:busdc", "p.u., Zero-Sequence Voltage, Magnitude"
      n_U0_busac = "n:U0:busac", "kV, Zero-Sequence Voltage, Magnitude"
      n_U0_busdc = "n:U0:busdc", "kV, Zero-Sequence Voltage, Magnitude"
      n_U0x3_busac = "n:U0x3:busac", "kV, 3*U0"
      n_U0x3_busdc = "n:U0x3:busdc", "kV, 3*U0"
      n_phiu0_busac = "n:phiu0:busac", "deg, Zero-Sequence Voltage, Angle"
      n_phiu0_busdc = "n:phiu0:busdc", "deg, Zero-Sequence Voltage, Angle"
      n_u1_busac = "n:u1:busac", "p.u., Positive-Sequence Voltage, Magnitude"
      n_u1_busdc = "n:u1:busdc", "p.u., Positive-Sequence Voltage, Magnitude"
      n_u1pc_busac = "n:u1pc:busac", "%, Positive-Sequence Voltage, Magnitude"
      n_u1pc_busdc = "n:u1pc:busdc", "%, Positive-Sequence Voltage, Magnitude"
      n_u1r_busac = "n:u1r:busac", "p.u., Positive-Sequence Voltage, Real Part"
      n_u1r_busdc = "n:u1r:busdc", "p.u., Positive-Sequence Voltage, Real Part"
      n_u1i_busac = "n:u1i:busac", "p.u., Positive-Sequence Voltage, Imaginary Part"
      n_u1i_busdc = "n:u1i:busdc", "p.u., Positive-Sequence Voltage, Imaginary Part"
      n_U1_busac = "n:U1:busac", "kV, Line-Ground Positive-Sequence Voltage, Magnitude"
      n_U1_busdc = "n:U1:busdc", "kV, Line-Ground Positive-Sequence Voltage, Magnitude"
      n_phiu1_busac = "n:phiu1:busac", "deg, Positive-Sequence Voltage, Angle"
      n_phiu1_busdc = "n:phiu1:busdc", "deg, Positive-Sequence Voltage, Angle"
      n_u2_busac = "n:u2:busac", "p.u., Negative-Sequence Voltage, Magnitude"
      n_u2_busdc = "n:u2:busdc", "p.u., Negative-Sequence Voltage, Magnitude"
      n_U2_busac = "n:U2:busac", "kV, Line-Ground Negative-Sequence Voltage, Magnitude"
      n_U2_busdc = "n:U2:busdc", "kV, Line-Ground Negative-Sequence Voltage, Magnitude"
      n_phiu2_busac = "n:phiu2:busac", "deg, Negative-Sequence Voltage, Angle"
      n_phiu2_busdc = "n:phiu2:busdc", "deg, Negative-Sequence Voltage, Angle"
      n_U1l_busac = "n:U1l:busac", "kV, Line-Line Positive-Sequence Voltage, Magnitude"
      n_U1l_busdc = "n:U1l:busdc", "kV, Line-Line Positive-Sequence Voltage, Magnitude"
      n_U2l_busac = "n:U2l:busac", "kV, Line-Line Negative-Sequence Voltage, Magnitude"
      n_U2l_busdc = "n:U2l:busdc", "kV, Line-Line Negative-Sequence Voltage, Magnitude"
      n_uln_busac_A = "n:uln:busac:A", "p.u., Line-Neutral Voltage, Magnitude"
      n_uln_busac_B = "n:uln:busac:B", "p.u., Line-Neutral Voltage, Magnitude"
      n_uln_busac_C = "n:uln:busac:C", "p.u., Line-Neutral Voltage, Magnitude"
      n_uln_busdc = "n:uln:busdc", "p.u., Line-Neutral Voltage, Magnitude"
      n_Uln_busac_A = "n:Uln:busac:A", "kV, Line-Neutral Voltage, Magnitude"
      n_Uln_busac_B = "n:Uln:busac:B", "kV, Line-Neutral Voltage, Magnitude"
      n_Uln_busac_C = "n:Uln:busac:C", "kV, Line-Neutral Voltage, Magnitude"
      n_Uln_busdc = "n:Uln:busdc", "kV, Line-Neutral Voltage, Magnitude"
      n_phiuln_busac_A = "n:phiuln:busac:A", "deg, Line-Neutral Voltage, Angle"
      n_phiuln_busac_B = "n:phiuln:busac:B", "deg, Line-Neutral Voltage, Angle"
      n_phiuln_busac_C = "n:phiuln:busac:C", "deg, Line-Neutral Voltage, Angle"
      n_phiuln_busdc = "n:phiuln:busdc", "deg, Line-Neutral Voltage, Angle"
      n_un_busac = "n:un:busac", "p.u., Neutral-Ground Voltage, Magnitude"
      n_un_busdc = "n:un:busdc", "p.u., Neutral-Ground Voltage, Magnitude"
      n_Un_busac = "n:Un:busac", "kV, Neutral-Ground Voltage, Magnitude"
      n_Un_busdc = "n:Un:busdc", "kV, Neutral-Ground Voltage, Magnitude"
      n_phiun_busac = "n:phiun:busac", "deg, Neutral-Ground Voltage, Angle"
      n_phiun_busdc = "n:phiun:busdc", "deg, Neutral-Ground Voltage, Angle"
      n_upht_busac_A = "n:upht:busac:A", "p.u., Phase Technology dependent Voltage, Magnitude"
      n_upht_busac_B = "n:upht:busac:B", "p.u., Phase Technology dependent Voltage, Magnitude"
      n_upht_busac_C = "n:upht:busac:C", "p.u., Phase Technology dependent Voltage, Magnitude"
      n_upht_busdc = "n:upht:busdc", "p.u., Phase Technology dependent Voltage, Magnitude"
      n_Upht_busac_A = "n:Upht:busac:A", "kV, Phase Technology dependent Voltage, Magnitude"
      n_Upht_busac_B = "n:Upht:busac:B", "kV, Phase Technology dependent Voltage, Magnitude"
      n_Upht_busac_C = "n:Upht:busac:C", "kV, Phase Technology dependent Voltage, Magnitude"
      n_Upht_busdc = "n:Upht:busdc", "kV, Phase Technology dependent Voltage, Magnitude"
      n_um_busac = "n:um:busac", "p.u., Average-Voltage, Magnitude"
      n_um_busdc = "n:um:busdc", "p.u., Average-Voltage, Magnitude"
      n_Um_busac = "n:Um:busac", "kV, Average-Voltage, Magnitude"
      n_Um_busdc = "n:Um:busdc", "kV, Average-Voltage, Magnitude"
      n_uphtmin_busac = "n:uphtmin:busac", "p.u., Minimum of Phase Technology dependent Voltage, Magnitude"
      n_uphtmin_busdc = "n:uphtmin:busdc", "p.u., Minimum of Phase Technology dependent Voltage, Magnitude"
      n_uphtmax_busac = "n:uphtmax:busac", "p.u., Maximum of Phase Technology dependent Voltage, Magnitude"
      n_uphtmax_busdc = "n:uphtmax:busdc", "p.u., Maximum of Phase Technology dependent Voltage, Magnitude"
      n_Pgen_busac = "n:Pgen:busac", "MW, Generation, Active Power"
      n_Pgen_busdc = "n:Pgen:busdc", "MW, Generation, Active Power"
      n_Qgen_busac = "n:Qgen:busac", "Mvar, Generation, Reactive Power"
      n_Qgen_busdc = "n:Qgen:busdc", "Mvar, Generation, Reactive Power"
      n_Pmot_busac = "n:Pmot:busac", "MW, Motor Load, Active Power"
      n_Pmot_busdc = "n:Pmot:busdc", "MW, Motor Load, Active Power"
      n_Qmot_busac = "n:Qmot:busac", "Mvar, Motor Load, Reactive Power"
      n_Qmot_busdc = "n:Qmot:busdc", "Mvar, Motor Load, Reactive Power"
      n_Pload_busac = "n:Pload:busac", "MW, General Load, Active Power"
      n_Pload_busdc = "n:Pload:busdc", "MW, General Load, Active Power"
      n_Qload_busac = "n:Qload:busac", "Mvar, General Load, Reactive Power"
      n_Qload_busdc = "n:Qload:busdc", "Mvar, General Load, Reactive Power"
      n_Pcomp_busac = "n:Pcomp:busac", "MW, Compensation (Losses)"
      n_Pcomp_busdc = "n:Pcomp:busdc", "MW, Compensation (Losses)"
      n_Qcomp_busac = "n:Qcomp:busac", "Mvar, Compensation"
      n_Qcomp_busdc = "n:Qcomp:busdc", "Mvar, Compensation"
      n_Pnet_busac = "n:Pnet:busac", "MW, External Networks, Active Power"
      n_Pnet_busdc = "n:Pnet:busdc", "MW, External Networks, Active Power"
      n_Qnet_busac = "n:Qnet:busac", "Mvar, External Networks, Reactive Power"
      n_Qnet_busdc = "n:Qnet:busdc", "Mvar, External Networks, Reactive Power"
      n_Pflow_busac = "n:Pflow:busac", "MW, Power Flow, Active Power"
      n_Pflow_busdc = "n:Pflow:busdc", "MW, Power Flow, Active Power"
      n_Qflow_busac = "n:Qflow:busac", "Mvar, Power Flow, Reactive Power"
      n_Qflow_busdc = "n:Qflow:busdc", "Mvar, Power Flow, Reactive Power"
      n_Pout_busac = "n:Pout:busac", "MW, Outgoing Flow, Active Power"
      n_Pout_busdc = "n:Pout:busdc", "MW, Outgoing Flow, Active Power"
      n_Qout_busac = "n:Qout:busac", "Mvar, Outgoing Power, Reactive Power"
      n_Qout_busdc = "n:Qout:busdc", "Mvar, Outgoing Power, Reactive Power"
      n_Sout_busac = "n:Sout:busac", "MVA, Outgoing Power, Apparent Power"
      n_Sout_busdc = "n:Sout:busdc", "MVA, Outgoing Power, Apparent Power"
      n_cosphiout_busac = "n:cosphiout:busac", "Outgoing Power, Power Factor"
      n_cosphiout_busdc = "n:cosphiout:busdc", "Outgoing Power, Power Factor"
      n_Pbalance_busac = "n:Pbalance:busac", "MW, Active Power Balance (=0)"
      n_Pbalance_busdc = "n:Pbalance:busdc", "MW, Active Power Balance (=0)"
      n_Qbalance_busac = "n:Qbalance:busac", "Mvar, Reactive Power Balance (=0)"
      n_Qbalance_busdc = "n:Qbalance:busdc", "Mvar, Reactive Power Balance (=0)"
      n_ubfac_busac = "n:ubfac:busac", "%, Unbalance Factor"
      n_ubfac_busdc = "n:ubfac:busdc", "%, Unbalance Factor"
      n_LossPdown_busac = "n:LossPdown:busac", "MW, Losses, downstream"
      n_LossPdown_busdc = "n:LossPdown:busdc", "MW, Losses, downstream"
      n_LossQdown_busac = "n:LossQdown:busac", "Mvar, Losses, downstream (Reactive Power)"
      n_LossQdown_busdc = "n:LossQdown:busdc", "Mvar, Losses, downstream (Reactive Power)"
      n_LossPdownload_busac = "n:LossPdownload:busac", "MW, Load losses, downstream"
      n_LossPdownload_busdc = "n:LossPdownload:busdc", "MW, Load losses, downstream"
      n_LossQdownload_busac = "n:LossQdownload:busac", "Mvar, Load losses, downstream"
      n_LossQdownload_busdc = "n:LossQdownload:busdc", "Mvar, Load losses, downstream"
      n_LossPdownnoload_busac = "n:LossPdownnoload:busac", "MW, No load losses, downstream"
      n_LossPdownnoload_busdc = "n:LossPdownnoload:busdc", "MW, No load losses, downstream"
      n_LossQdownnoload_busac = "n:LossQdownnoload:busac", "Mvar, No load losses, downstream"
      n_LossQdownnoload_busdc = "n:LossQdownnoload:busdc", "Mvar, No load losses, downstream"
      s_pset = "s:pset", "MW, IN, Active power setpoint, continuous, double"
      s_qset = "s:qset", "Mvar, IN, Reactive power setpoint continuous, double"
      s_udcset = "s:udcset", "p.u., IN, DC Voltage setpoint, continuous, double"
      s_dFin = "s:dFin", "Hz, IN, Frequency Deviation, continuous, double"
      s_i2r_set = "s:i2r_set", "p.u., IN, Negative sequence current setpoint, Real Part, continuous, double"
      s_i2i_set = "s:i2i_set", "p.u., IN, Negative sequence current setpoint, Imaginary Part, continuous, double"
      c_phiu = "c:phiu", "p.u., Voltage angle"
      c_uini = "c:uini", "p.u., Voltage setpoint"
      c_snomn = "c:snomn", "MVA, Total rated apparent power (nparnum*Snom)"
      c_uinidc = "c:uinidc", "p.u., Voltage setpoint (DC)"
      c_pmsetp = "c:pmsetp", "PWM factor"
      c_i_mod = "c:i_mod", "Modulation"
      c_K0 = "c:K0", "Vac/Vdc at PWM=1"
      c_Pm = "c:Pm", "Pulse-modulation factor"
      c_uDC = "c:uDC", "p.u., DC-voltage"
      c_pini = "c:pini", "Active power setpoint"
      c_qini = "c:qini", "Reactive power setpoint"
      c_DCPoleLosses = "c:DCPoleLosses", "MW, DC pole losses"
      c_Qmax = "c:Qmax", "Mvar, Upper limit of reactive power"
      c_Qmin = "c:Qmin", "Mvar, Lower limit of reactive power"
      c_Pmin = "c:Pmin", "MW, Lower limit of active power"
      c_Pmax = "c:Pmax", "MW, Upper limit of active power"
      c_i_qlim = "c:i_qlim", "Reactive power limited"
      c_i_plim = "c:i_plim", "Active power limited"
      c_Kpf = "c:Kpf", "MW/Hz, Frequency bias"
      c_Kp = "c:Kp", "p.u., Active power participation"
      c_zonewscale = "c:zonewscale", "Wind generation zone scaling factor"
      c_iRefElement = "c:iRefElement", "Reference element"
      c_Kq = "c:Kq", "p.u., Reactive power participation"
      c_iPowerOn = "c:iPowerOn", "Flag"
      c_i_vlim = "c:i_vlim", "Voltage setpoint limit reached"
      c_dphi = "c:dphi", "deg, Measured angle difference"
      c_Pmeas = "c:Pmeas", "MW, Measured active power"
      c_nCtrlIter = "c:nCtrlIter", "No. of CTRL-iterations"
      c_Ploss = "c:Ploss", "MW, Losses (total)"
      c_Qloss = "c:Qloss", "Mvar, Reactive-Losses (total)"
      c_Plossld = "c:Plossld", "MW, Losses (load)"
      c_Qlossld = "c:Qlossld", "Mvar, Reactive-Losses (load)"
      c_Plossnld = "c:Plossnld", "MW, Losses (no load)"
      c_Qlossnld = "c:Qlossnld", "Mvar, Reactive-Losses (no load)"
      e_i_acdc = "e:i_acdc", "Control mode"
      e_ivacdroop = "e:ivacdroop", "AC-Voltage Droop"
      e_usetp = "e:usetp", "p.u., AC voltage setpoint"
      e_pmsetp = "e:pmsetp", "p.u., PWM factor"
      e_p_uctrl = "e:p_uctrl", "ElmTerm*,StaBar*, Controlled node (AC)"
      e_usetpdc = "e:usetpdc", "p.u., DC voltage setpoint"
      e_p_uctrldc = "e:p_uctrldc", "ElmTerm*,StaBar*, Controlled node (DC)"
      e_phisetp = "e:phisetp", "deg, Phase setpoint"
      e_psetp = "e:psetp", "MW, Active power setpoint"
      e_Kpf = "e:Kpf", "MW/Hz, Prim. frequency bias"
      e_qsetp = "e:qsetp", "Mvar, Reactive power setpoint"
      e_iQorient = "e:iQorient", "Orientation"
      e_pfsetp = "e:pfsetp", "Power factor"
      e_pf_recap = "e:pf_recap", "Power factor"
      e_p_pctrl = "e:p_pctrl", "StaCubic,ElmBoundary, Controlled flow"
      e_c_pstac = "e:c_pstac", "ElmStactrl, External station controller"
      e_c_psecc = "e:c_psecc", "ElmSecctrl, Ext. secondary controller"
      e_iVacMax = "e:iVacMax", "Consider modulation index limit"
      e_Pmmax = "e:Pmmax", "Max. PWM factor"
      e_Pnold = "e:Pnold", "kW, No-load losses"
      e_swtLossFactor = "e:swtLossFactor", "kW/A, Switching loss factor"
      e_resLossFactor = "e:resLossFactor", "Ohm, Resistive loss factor"
      e_ddroop = "e:ddroop", "%, Droop"
      e_usp_min = "e:usp_min", "p.u., Min. voltage setpoint"
      e_usp_max = "e:usp_max", "p.u., Max. voltage setpoint"
      e_idroop = "e:idroop", "DC-voltage dependent P-droop"
      e_K = "e:K", "p.u./MW, K"
      e_iPphidrp = "e:iPphidrp", "Angle difference dependent P-droop"
      e_Kpphi = "e:Kpphi", "MW/deg, Kpphi"
      e_p_b1phiu = "e:p_b1phiu", "ElmTerm*, Remote AC busbar"
      e_p_b2phiu = "e:p_b2phiu", "ElmTerm*, Local AC busbar"
      e_considerPset = "e:considerPset", "Consider active power setpoint"
      e_iPpart = "e:iPpart", "Active power participation"
      e_Kpart = "e:Kpart", "Participation factor"
      e_p_pmeas = "e:p_pmeas", "StaCubic*,ElmBoundary, P(AC) measured at"
      e_pQlimType = "e:pQlimType", "IntQlim, Capability curve"
      e_q_min = "e:q_min", "p.u., Min."
      e_q_max = "e:q_max", "p.u., Max."
      e_cQ_min = "e:cQ_min", "Mvar, Min."
      e_cQ_max = "e:cQ_max", "Mvar, Max."
      e_scaleQmin = "e:scaleQmin", "%, Scaling factor (min.)"
      e_scaleQmax = "e:scaleQmax", "%, Scaling factor (max.)"
      e_Pmin_uc = "e:Pmin_uc", "MW, Min."
      e_Pmax_uc = "e:Pmax_uc", "MW, Max."
      e_Pnom = "e:Pnom", "MW, Pr(rated)"
      e_Qmin_a = "e:Qmin_a", "Mvar, Qmin(act.)"
      e_Qmax_a = "e:Qmax_a", "Mvar, Qmax(act.)"
      e_P_max = "e:P_max", "MW, Max."
      e_r2 = "e:r2", "p.u., Resistance, r2"
      e_x2 = "e:x2", "p.u., Reactance, x2"
      e_priority = "e:priority", "Merit Order"
      e_cpFeed = "e:cpFeed", "Feeder"
      e_ciDist = "e:ciDist", "Distance from infeed in number of buses"
      e_ciLater = "e:ciLater", "Lateral Index"
      e_ciDistRoot = "e:ciDistRoot", "Distance from first infeed in number of buses"
      e_ciDistAll = "e:ciDistAll", "Distance from infeed in number of buses including switches"
      e_ciDistAllRoot = "e:ciDistAllRoot", "Distance from first infeed in number of buses including switches"
      r_p_uctrl = "r:p_uctrl", "ElmTerm*,StaBar*, Controlled node (AC)"
      r_p_uctrldc = "r:p_uctrldc", "ElmTerm*,StaBar*, Controlled node (DC)"
      r_p_pctrl = "r:p_pctrl", "StaCubic,ElmBoundary, Controlled flow"
      r_c_pstac = "r:c_pstac", "ElmStactrl, External station controller"
      r_c_psecc = "r:c_psecc", "ElmSecctrl, Ext. secondary controller"
      r_p_b1phiu = "r:p_b1phiu", "ElmTerm*, Remote AC busbar"
      r_p_b2phiu = "r:p_b2phiu", "ElmTerm*, Local AC busbar"
      r_p_pmeas = "r:p_pmeas", "StaCubic*,ElmBoundary, P(AC) measured at"
      r_pQlimType = "r:pQlimType", "IntQlim, Capability curve"
      r_cpFeed = "r:cpFeed", "Feeder"
      m_Ikss_busac = "m:Ikss:busac", "kA, Initial Short-Circuit Current"
      m_Ikss_busdc = "m:Ikss:busdc", "kA, Initial Short-Circuit Current"
      m_ikss_busac = "m:ikss:busac", "p.u., Initial Short-Circuit Current, Magnitude"
      m_ikss_busdc = "m:ikss:busdc", "p.u., Initial Short-Circuit Current, Magnitude"
      m_phii_busac = "m:phii:busac", "deg, Phase Current, Angle"
      m_maxIkss_busac = "m:maxIkss:busac", "kA, Max. Initial Short-Circuit Current"
      m_maxIkss_busdc = "m:maxIkss:busdc", "kA, Max. Initial Short-Circuit Current"
      m_Skss_busac = "m:Skss:busac", "MVA, Initial Short-Circuit Power"
      m_Skss_busdc = "m:Skss:busdc", "MVA, Initial Short-Circuit Power"
      m_ip_busac = "m:ip:busac", "kA, Peak Short-Circuit Current"
      m_ip_busdc = "m:ip:busdc", "kA, Peak Short-Circuit Current"
      m_maxip_busac = "m:maxip:busac", "kA, Max. Peak Short-Circuit Current"
      m_maxip_busdc = "m:maxip:busdc", "kA, Max. Peak Short-Circuit Current"
      m_maxIb_busac = "m:maxIb:busac", "kA, Max. Breaking Current"
      m_maxIb_busdc = "m:maxIb:busdc", "kA, Max. Breaking Current"
      m_maxIth_busac = "m:maxIth:busac", "kA, Max. Thermal equivalent Short-Circuit Current"
      m_maxIth_busdc = "m:maxIth:busdc", "kA, Max. Thermal equivalent Short-Circuit Current"
      m_Ib_busac = "m:Ib:busac", "kA, Short-Circuit Breaking Current"
      m_Ib_busdc = "m:Ib:busdc", "kA, Short-Circuit Breaking Current"
      m_Sb_busac = "m:Sb:busac", "MVA, Short-Circuit Breaking Power"
      m_Sb_busdc = "m:Sb:busdc", "MVA, Short-Circuit Breaking Power"
      m_Ik_busac = "m:Ik:busac", "kA, Steady-State Short-Circuit Current"
      m_Ik_busdc = "m:Ik:busdc", "kA, Steady-State Short-Circuit Current"
      m_Ith_busac = "m:Ith:busac", "kA, Thermal equivalent Short-Circuit Current"
      m_Ith_busdc = "m:Ith:busdc", "kA, Thermal equivalent Short-Circuit Current"
      m_I_busac = "m:I:busac", "kA, Short-Circuit Current"
      m_R_busac = "m:R:busac", "Ohm, Fault Impedance, Real Part"
      m_R_busdc = "m:R:busdc", "Ohm, Fault Impedance, Real Part"
      m_X_busac = "m:X:busac", "Ohm, Fault Impedance, Imaginary Part"
      m_X_busdc = "m:X:busdc", "Ohm, Fault Impedance, Imaginary Part"
      m_Z_busac = "m:Z:busac", "Ohm, Fault Impedance, Magnitude"
      m_Z_busdc = "m:Z:busdc", "Ohm, Fault Impedance, Magnitude"
      m_phiz_busac = "m:phiz:busac", "deg, Fault Impedance, Angle"
      m_phiz_busdc = "m:phiz:busdc", "deg, Fault Impedance, Angle"
      m_zSbase_busac = "m:zSbase:busac", "p.u., Fault Impedance, Magnitude"
      m_zSbase_busdc = "m:zSbase:busdc", "p.u., Fault Impedance, Magnitude"
      m_XtoR_busac = "m:XtoR:busac", "X/R ratio"
      m_XtoR_busdc = "m:XtoR:busdc", "X/R ratio"
      m_Zs_busac = "m:Zs:busac", "Ohm, Source Impedance, Magnitude"
      m_Zs_busdc = "m:Zs:busdc", "Ohm, Source Impedance, Magnitude"
      m_phizs_busac = "m:phizs:busac", "deg, Source Impedance, Angle"
      m_phizs_busdc = "m:phizs:busdc", "deg, Source Impedance, Angle"
      m_Rs_busac = "m:Rs:busac", "Ohm, Source Impedance, Real Part"
      m_Rs_busdc = "m:Rs:busdc", "Ohm, Source Impedance, Real Part"
      m_Xs_busac = "m:Xs:busac", "Ohm, Source Impedance, Imaginary Part"
      m_Xs_busdc = "m:Xs:busdc", "Ohm, Source Impedance, Imaginary Part"
      m_phiui_busac = "m:phiui:busac", "deg, Angle between Voltage and Current"
      m_cosphiui_busac = "m:cosphiui:busac", "cosphi"
      m_cosphiui_busdc = "m:cosphiui:busdc", "cosphi"
      m_P_busac = "m:P:busac", "MW, Active Power"
      m_Q_busac = "m:Q:busac", "Mvar, Reactive Power"
      m_BrkIpload_busac = "m:BrkIpload:busac", "%, Breaker Loading (peak)"
      m_BrkIpload_busdc = "m:BrkIpload:busdc", "%, Breaker Loading (peak)"
      m_BrkIbload_busac = "m:BrkIbload:busac", "%, Breaker Loading (breaking)"
      m_BrkIbload_busdc = "m:BrkIbload:busdc", "%, Breaker Loading (breaking)"
      m_BrkmaxIb_busac = "m:BrkmaxIb:busac", "kA, Max. Breaking Current (breaker time)"
      m_BrkmaxIb_busdc = "m:BrkmaxIb:busdc", "kA, Max. Breaking Current (breaker time)"
      n_Ikss_busac = "n:Ikss:busac", "kA, Initial Short-Circuit Current"
      n_Ikss_busdc = "n:Ikss:busdc", "kA, Initial Short-Circuit Current"
      n_I_busac = "n:I:busac", "kA, Short-Circuit Current"
      n_I_busdc = "n:I:busdc", "kA, Short-Circuit Current"
      n_phii_busac = "n:phii:busac", "deg, Phase Current, Angle"
      n_phii_busdc = "n:phii:busdc", "deg, Phase Current, Angle"
      n_Skss_busac = "n:Skss:busac", "MVA, Initial Short-Circuit Power"
      n_Skss_busdc = "n:Skss:busdc", "MVA, Initial Short-Circuit Power"
      n_ip_busac = "n:ip:busac", "kA, Peak Short-Circuit Current"
      n_ip_busdc = "n:ip:busdc", "kA, Peak Short-Circuit Current"
      n_Ib_busac = "n:Ib:busac", "kA, Short-Circuit Breaking Current"
      n_Ib_busdc = "n:Ib:busdc", "kA, Short-Circuit Breaking Current"
      n_Sb_busac = "n:Sb:busac", "MVA, Short-Circuit Breaking Power"
      n_Sb_busdc = "n:Sb:busdc", "MVA, Short-Circuit Breaking Power"
      n_Ik_busac = "n:Ik:busac", "kA, Steady-State Short-Circuit Current"
      n_Ik_busdc = "n:Ik:busdc", "kA, Steady-State Short-Circuit Current"
      n_Ith_busac = "n:Ith:busac", "kA, Thermal equivalent Short-Circuit Current"
      n_Ith_busdc = "n:Ith:busdc", "kA, Thermal equivalent Short-Circuit Current"
      n_R_busac = "n:R:busac", "Ohm, Short-Circuit Impedance, Real Part"
      n_R_busdc = "n:R:busdc", "Ohm, Short-Circuit Impedance, Real Part"
      n_X_busac = "n:X:busac", "Ohm, Short-Circuit Impedance, Imaginary Part"
      n_X_busdc = "n:X:busdc", "Ohm, Short-Circuit Impedance, Imaginary Part"
      n_Z_busac = "n:Z:busac", "Ohm, Short-Circuit Impedance, Magnitude"
      n_Z_busdc = "n:Z:busdc", "Ohm, Short-Circuit Impedance, Magnitude"
      n_phiz_busac = "n:phiz:busac", "deg, Short-Circuit Impedance, Angle"
      n_phiz_busdc = "n:phiz:busdc", "deg, Short-Circuit Impedance, Angle"
      n_rSbase_busac = "n:rSbase:busac", "p.u., Short-Circuit Impedance, Real Part"
      n_rSbase_busdc = "n:rSbase:busdc", "p.u., Short-Circuit Impedance, Real Part"
      n_xSbase_busac = "n:xSbase:busac", "p.u., Short-Circuit Impedance, Imaginary Part"
      n_xSbase_busdc = "n:xSbase:busdc", "p.u., Short-Circuit Impedance, Imaginary Part"
      n_zSbase_busac = "n:zSbase:busac", "p.u., Short-Circuit Impedance, Magnitude"
      n_zSbase_busdc = "n:zSbase:busdc", "p.u., Short-Circuit Impedance, Magnitude"
      n_XtoR_busac = "n:XtoR:busac", "X/R ratio (peak)"
      n_XtoR_busdc = "n:XtoR:busdc", "X/R ratio (peak)"
      n_RtoX_busac = "n:RtoX:busac", "R/X ratio (peak)"
      n_RtoX_busdc = "n:RtoX:busdc", "R/X ratio (peak)"
      n_ur_busac = "n:ur:busac", "p.u., Voltage, Real Part"
      n_ui_busac = "n:ui:busac", "p.u., Voltage, Imaginary Part"
      n_u_busac = "n:u:busac", "p.u., Voltage, Magnitude"
      n_upc_busac = "n:upc:busac", "%, Voltage, Magnitude"
      n_U_busac = "n:U:busac", "kV, Line-Ground Voltage, Magnitude"
      n_Ul_busac = "n:Ul:busac", "kV, Line-Line Voltage, Magnitude"
      n_phiu_busac = "n:phiu:busac", "deg, Voltage, Angle"
      n_du_busac = "n:du:busac", "%, Voltage Deviation"
      n_du_busdc = "n:du:busdc", "%, Voltage Deviation"
      n_urpref_busac = "n:urpref:busac", "p.u., Pre-fault Voltage, Real Part"
      n_urpref_busdc = "n:urpref:busdc", "p.u., Pre-fault Voltage, Real Part"
      n_uipref_busac = "n:uipref:busac", "p.u., Pre-fault Voltage, Imaginary Part"
      n_uipref_busdc = "n:uipref:busdc", "p.u., Pre-fault Voltage, Imaginary Part"
      n_upref_busac = "n:upref:busac", "p.u., Pre-fault Voltage, Magnitude"
      n_upref_busdc = "n:upref:busdc", "p.u., Pre-fault Voltage, Magnitude"
      n_uprefpc_busac = "n:uprefpc:busac", "%, Pre-fault Voltage, Magnitude"
      n_uprefpc_busdc = "n:uprefpc:busdc", "%, Pre-fault Voltage, Magnitude"
      n_Upref_busac = "n:Upref:busac", "kV, Line-Ground Pre-fault Voltage, Magnitude"
      n_Upref_busdc = "n:Upref:busdc", "kV, Line-Ground Pre-fault Voltage, Magnitude"
      n_Ulpref_busac = "n:Ulpref:busac", "kV, Line-Line Pre-fault Voltage, Magnitude"
      n_Ulpref_busdc = "n:Ulpref:busdc", "kV, Line-Line Pre-fault Voltage, Magnitude"
      n_phiupref_busac = "n:phiupref:busac", "deg, Pre-fault Voltage, Angle"
      n_phiupref_busdc = "n:phiupref:busdc", "deg, Pre-fault Voltage, Angle"
      n_phiui_busac = "n:phiui:busac", "deg, Angle between Voltage and Current"
      n_phiui_busdc = "n:phiui:busdc", "deg, Angle between Voltage and Current"
      n_cfac_busac = "n:cfac:busac", "Voltage Factor"
      n_cfac_busdc = "n:cfac:busdc", "Voltage Factor"
      n_idc_busac = "n:idc:busac", "kA, Decaying Component"
      n_idc_busdc = "n:idc:busdc", "kA, Decaying Component"
      n_Iasy_busac = "n:Iasy:busac", "kA, Asymmetrical Breaking Current"
      n_Iasy_busdc = "n:Iasy:busdc", "kA, Asymmetrical Breaking Current"
      n_Ithload_busac = "n:Ithload:busac", "%, Loading, Thermal equivalent Current"
      n_Ithload_busdc = "n:Ithload:busdc", "%, Loading, Thermal equivalent Current"
      n_Ipload_busac = "n:Ipload:busac", "%, Loading, Peak Short-Circuit Current"
      n_Ipload_busdc = "n:Ipload:busdc", "%, Loading, Peak Short-Circuit Current"
      n_Ithrtk_busac = "n:Ithrtk:busac", "kA, Rated Short-Time Current (Tk)"
      n_Ithrtk_busdc = "n:Ithrtk:busdc", "kA, Rated Short-Time Current (Tk)"
      n_Tn_busac = "n:Tn:busac", "ms, Grid Time Constant"
      n_Tn_busdc = "n:Tn:busdc", "ms, Grid Time Constant"
      n_XtoR_b_busac = "n:XtoR_b:busac", "X/R ratio (dc)"
      n_XtoR_b_busdc = "n:XtoR_b:busdc", "X/R ratio (dc)"
      n_RtoX_b_busac = "n:RtoX_b:busac", "R/X ratio (dc)"
      n_RtoX_b_busdc = "n:RtoX_b:busdc", "R/X ratio (dc)"
      n_fctof_b_busac = "n:fctof_b:busac", "Equivalent Frequency fc/f (dc)"
      n_fctof_b_busdc = "n:fctof_b:busdc", "Equivalent Frequency fc/f (dc)"
      c_imax = "c:imax", "p.u., Max. current"
      e_iNoShcContr = "e:iNoShcContr", "No short-circuit contribution"
      e_iconfed = "e:iconfed", "Static converter-fed drive"
      e_psutype = "e:psutype", "Power station unit type"
      e_searchBlockTrf = "e:searchBlockTrf", "Externally modelled unit transformer"
      e_pblocktrf = "e:pblocktrf", "Unit transformer"
      e_Ikss3PF = "e:Ikss3PF", "kA, Three-phase faults, Ik''3PF"
      e_Ikss2PF = "e:Ikss2PF", "kA, Two-phase faults, Ik''2PF"
      e_Ikss1PF = "e:Ikss1PF", "kA, Single-phase faults, Ik''1PF"
      e_IkPFmax = "e:IkPFmax", "kA, Maximum current"
      e_IkPFmin = "e:IkPFmin", "kA, Minimum current"
      e_r2iec = "e:r2iec", "p.u., Resistance, r2"
      e_x2iec = "e:x2iec", "p.u., Reactance, x2"
      e_r0iec = "e:r0iec", "p.u., Resistance, r0"
      e_x0iec = "e:x0iec", "p.u., Reactance, x0"
      e_Ikss = "e:Ikss", "kA, Subtransient short-circuit current"
      e_Skss = "e:Skss", "MVA, Subtransient short-circuit level"
      e_rtox = "e:rtox", "R to X'' ratio"
      e_xtor = "e:xtor", "X'' to R ratio"
      e_r2shc = "e:r2shc", "p.u., Resistance, r2"
      e_x2shc = "e:x2shc", "p.u., Reactance, x2"
      r_pblocktrf = "r:pblocktrf", "Unit transformer"
      m_Ikss_busac_A = "m:Ikss:busac:A", "kA, Initial Short-Circuit Current"
      m_Ikss_busac_B = "m:Ikss:busac:B", "kA, Initial Short-Circuit Current"
      m_Ikss_busac_C = "m:Ikss:busac:C", "kA, Initial Short-Circuit Current"
      m_ikss_busac_A = "m:ikss:busac:A", "p.u., Initial Short-Circuit Current, Magnitude"
      m_ikss_busac_B = "m:ikss:busac:B", "p.u., Initial Short-Circuit Current, Magnitude"
      m_ikss_busac_C = "m:ikss:busac:C", "p.u., Initial Short-Circuit Current, Magnitude"
      m_Skss_busac_A = "m:Skss:busac:A", "MVA, Initial Short-Circuit Power"
      m_Skss_busac_B = "m:Skss:busac:B", "MVA, Initial Short-Circuit Power"
      m_Skss_busac_C = "m:Skss:busac:C", "MVA, Initial Short-Circuit Power"
      m_Skssx3_busac_A = "m:Skssx3:busac:A", "MVA, 3*Initial Short-Circuit Power"
      m_Skssx3_busac_B = "m:Skssx3:busac:B", "MVA, 3*Initial Short-Circuit Power"
      m_Skssx3_busac_C = "m:Skssx3:busac:C", "MVA, 3*Initial Short-Circuit Power"
      m_Skssx3_busdc = "m:Skssx3:busdc", "MVA, 3*Initial Short-Circuit Power"
      m_ip_busac_A = "m:ip:busac:A", "kA, Peak Short-Circuit Current"
      m_ip_busac_B = "m:ip:busac:B", "kA, Peak Short-Circuit Current"
      m_ip_busac_C = "m:ip:busac:C", "kA, Peak Short-Circuit Current"
      m_Z_busac_A = "m:Z:busac:A", "Ohm, Fault Impedance, Magnitude"
      m_Z_busac_B = "m:Z:busac:B", "Ohm, Fault Impedance, Magnitude"
      m_Z_busac_C = "m:Z:busac:C", "Ohm, Fault Impedance, Magnitude"
      m_phiz_busac_A = "m:phiz:busac:A", "deg, Fault Impedance, Angle"
      m_phiz_busac_B = "m:phiz:busac:B", "deg, Fault Impedance, Angle"
      m_phiz_busac_C = "m:phiz:busac:C", "deg, Fault Impedance, Angle"
      m_Zl_busac_A = "m:Zl:busac:A", "Ohm, Line-Line Fault Impedance, Magnitude"
      m_Zl_busac_B = "m:Zl:busac:B", "Ohm, Line-Line Fault Impedance, Magnitude"
      m_Zl_busac_C = "m:Zl:busac:C", "Ohm, Line-Line Fault Impedance, Magnitude"
      m_Zl_busdc = "m:Zl:busdc", "Ohm, Line-Line Fault Impedance, Magnitude"
      m_phizl_busac_A = "m:phizl:busac:A", "deg, Line-Line Fault Impedance, Angle"
      m_phizl_busac_B = "m:phizl:busac:B", "deg, Line-Line Fault Impedance, Angle"
      m_phizl_busac_C = "m:phizl:busac:C", "deg, Line-Line Fault Impedance, Angle"
      m_phizl_busdc = "m:phizl:busdc", "deg, Line-Line Fault Impedance, Angle"
      m_Z0_busac = "m:Z0:busac", "Ohm, Zero-Sequence Fault Impedance, Magnitude"
      m_Z0_busdc = "m:Z0:busdc", "Ohm, Zero-Sequence Fault Impedance, Magnitude"
      m_phiz0_busac = "m:phiz0:busac", "deg, Zero-Sequence Fault Impedance, Angle"
      m_phiz0_busdc = "m:phiz0:busdc", "deg, Zero-Sequence Fault Impedance, Angle"
      m_z0Sbase_busac = "m:z0Sbase:busac", "p.u., Zero-Sequence Fault Impedance, Magnitude"
      m_z0Sbase_busdc = "m:z0Sbase:busdc", "p.u., Zero-Sequence Fault Impedance, Magnitude"
      m_cosphiu1i1_busac = "m:cosphiu1i1:busac", "Positive-Sequence cosphi"
      m_cosphiu1i1_busdc = "m:cosphiu1i1:busdc", "Positive-Sequence cosphi"
      m_I0x3watt_busac = "m:I0x3watt:busac", "kA, Earth-Fault Residual Current"
      m_I0x3watt_busdc = "m:I0x3watt:busdc", "kA, Earth-Fault Residual Current"
      m_I0x3var_busac = "m:I0x3var:busac", "kA, Capacitive Earth-Fault Current"
      m_I0x3var_busdc = "m:I0x3var:busdc", "kA, Capacitive Earth-Fault Current"
      m_Q0cap_busac = "m:Q0cap:busac", "Mvar, Q0 capacitive"
      m_Q0cap_busdc = "m:Q0cap:busdc", "Mvar, Q0 capacitive"
      m_Zfrel_busac = "m:Zfrel:busac", "Ohm, Relay Impedance, Magnitude"
      m_Zfrel_busdc = "m:Zfrel:busdc", "Ohm, Relay Impedance, Magnitude"
      m_phizfrel_busac = "m:phizfrel:busac", "deg, Relay Impedance, Angle"
      m_phizfrel_busdc = "m:phizfrel:busdc", "deg, Relay Impedance, Angle"
      m_Rfrel_busac = "m:Rfrel:busac", "Ohm, Relay Impedance, Real Part"
      m_Rfrel_busdc = "m:Rfrel:busdc", "Ohm, Relay Impedance, Real Part"
      m_Xfrel_busac = "m:Xfrel:busac", "Ohm, Relay Impedance, Imaginary Part"
      m_Xfrel_busdc = "m:Xfrel:busdc", "Ohm, Relay Impedance, Imaginary Part"
      m_Zf_busac = "m:Zf:busac", "Ohm, Meas. Fault Impedance, Magnitude"
      m_Zf_busdc = "m:Zf:busdc", "Ohm, Meas. Fault Impedance, Magnitude"
      m_phizf_busac = "m:phizf:busac", "deg, Meas. Fault Impedance, Angle"
      m_phizf_busdc = "m:phizf:busdc", "deg, Meas. Fault Impedance, Angle"
      n_Ikss_busac_A = "n:Ikss:busac:A", "kA, Initial Short-Circuit Current"
      n_Ikss_busac_B = "n:Ikss:busac:B", "kA, Initial Short-Circuit Current"
      n_Ikss_busac_C = "n:Ikss:busac:C", "kA, Initial Short-Circuit Current"
      n_I_busac_A = "n:I:busac:A", "kA, Short-Circuit Current"
      n_I_busac_B = "n:I:busac:B", "kA, Short-Circuit Current"
      n_I_busac_C = "n:I:busac:C", "kA, Short-Circuit Current"
      n_phii_busac_A = "n:phii:busac:A", "deg, Phase Current, Angle"
      n_phii_busac_B = "n:phii:busac:B", "deg, Phase Current, Angle"
      n_phii_busac_C = "n:phii:busac:C", "deg, Phase Current, Angle"
      n_Skss_busac_A = "n:Skss:busac:A", "MVA, Initial Short-Circuit Power"
      n_Skss_busac_B = "n:Skss:busac:B", "MVA, Initial Short-Circuit Power"
      n_Skss_busac_C = "n:Skss:busac:C", "MVA, Initial Short-Circuit Power"
      n_Skssx3_busac_A = "n:Skssx3:busac:A", "MVA, 3*Initial Short-Circuit Power"
      n_Skssx3_busac_B = "n:Skssx3:busac:B", "MVA, 3*Initial Short-Circuit Power"
      n_Skssx3_busac_C = "n:Skssx3:busac:C", "MVA, 3*Initial Short-Circuit Power"
      n_Skssx3_busdc = "n:Skssx3:busdc", "MVA, 3*Initial Short-Circuit Power"
      n_ip_busac_A = "n:ip:busac:A", "kA, Peak Short-Circuit Current"
      n_ip_busac_B = "n:ip:busac:B", "kA, Peak Short-Circuit Current"
      n_ip_busac_C = "n:ip:busac:C", "kA, Peak Short-Circuit Current"
      n_Ib_busac_A = "n:Ib:busac:A", "kA, Short-Circuit Breaking Current"
      n_Ib_busac_B = "n:Ib:busac:B", "kA, Short-Circuit Breaking Current"
      n_Ib_busac_C = "n:Ib:busac:C", "kA, Short-Circuit Breaking Current"
      n_Sb_busac_A = "n:Sb:busac:A", "MVA, Short-Circuit Breaking Power"
      n_Sb_busac_B = "n:Sb:busac:B", "MVA, Short-Circuit Breaking Power"
      n_Sb_busac_C = "n:Sb:busac:C", "MVA, Short-Circuit Breaking Power"
      n_Ith_busac_A = "n:Ith:busac:A", "kA, Thermal equivalent Short-Circuit Current"
      n_Ith_busac_B = "n:Ith:busac:B", "kA, Thermal equivalent Short-Circuit Current"
      n_Ith_busac_C = "n:Ith:busac:C", "kA, Thermal equivalent Short-Circuit Current"
      n_Ithmax_busac = "n:Ithmax:busac", "kA, Max. Thermal equivalent Short-Circuit Current"
      n_Ithmax_busdc = "n:Ithmax:busdc", "kA, Max. Thermal equivalent Short-Circuit Current"
      n_Ik_busac_A = "n:Ik:busac:A", "kA, Steady-State Short-Circuit Current"
      n_Ik_busac_B = "n:Ik:busac:B", "kA, Steady-State Short-Circuit Current"
      n_Ik_busac_C = "n:Ik:busac:C", "kA, Steady-State Short-Circuit Current"
      n_R0_busac = "n:R0:busac", "Ohm, Zero-Sequence Impedance, Real Part"
      n_R0_busdc = "n:R0:busdc", "Ohm, Zero-Sequence Impedance, Real Part"
      n_X0_busac = "n:X0:busac", "Ohm, Zero-Sequence Impedance, Imaginary Part"
      n_X0_busdc = "n:X0:busdc", "Ohm, Zero-Sequence Impedance, Imaginary Part"
      n_Z0_busac = "n:Z0:busac", "Ohm, Zero-Sequence Impedance, Magnitude"
      n_Z0_busdc = "n:Z0:busdc", "Ohm, Zero-Sequence Impedance, Magnitude"
      n_phiz0_busac = "n:phiz0:busac", "deg, Zero-Sequence Impedance, Angle"
      n_phiz0_busdc = "n:phiz0:busdc", "deg, Zero-Sequence Impedance, Angle"
      n_r0Sbase_busac = "n:r0Sbase:busac", "p.u., Zero-Sequence Impedance, Real Part"
      n_r0Sbase_busdc = "n:r0Sbase:busdc", "p.u., Zero-Sequence Impedance, Real Part"
      n_x0Sbase_busac = "n:x0Sbase:busac", "p.u., Zero-Sequence Impedance, Imaginary Part"
      n_x0Sbase_busdc = "n:x0Sbase:busdc", "p.u., Zero-Sequence Impedance, Imaginary Part"
      n_z0Sbase_busac = "n:z0Sbase:busac", "p.u., Zero-Sequence Impedance, Magnitude"
      n_z0Sbase_busdc = "n:z0Sbase:busdc", "p.u., Zero-Sequence Impedance, Magnitude"
      n_R1_busac = "n:R1:busac", "Ohm, Positive-Sequence Impedance, Real Part"
      n_R1_busdc = "n:R1:busdc", "Ohm, Positive-Sequence Impedance, Real Part"
      n_X1_busac = "n:X1:busac", "Ohm, Positive-Sequence Impedance, Imaginary Part"
      n_X1_busdc = "n:X1:busdc", "Ohm, Positive-Sequence Impedance, Imaginary Part"
      n_Z1_busac = "n:Z1:busac", "Ohm, Positive-Sequence Impedance, Magnitude"
      n_Z1_busdc = "n:Z1:busdc", "Ohm, Positive-Sequence Impedance, Magnitude"
      n_phiz1_busac = "n:phiz1:busac", "deg, Positive-Sequence Impedance, Angle"
      n_phiz1_busdc = "n:phiz1:busdc", "deg, Positive-Sequence Impedance, Angle"
      n_r1Sbase_busac = "n:r1Sbase:busac", "p.u., Positive-Sequence Impedance, Real Part"
      n_r1Sbase_busdc = "n:r1Sbase:busdc", "p.u., Positive-Sequence Impedance, Real Part"
      n_x1Sbase_busac = "n:x1Sbase:busac", "p.u., Positive-Sequence Impedance, Imaginary Part"
      n_x1Sbase_busdc = "n:x1Sbase:busdc", "p.u., Positive-Sequence Impedance, Imaginary Part"
      n_z1Sbase_busac = "n:z1Sbase:busac", "p.u., Positive-Sequence Impedance, Magnitude"
      n_z1Sbase_busdc = "n:z1Sbase:busdc", "p.u., Positive-Sequence Impedance, Magnitude"
      n_R2_busac = "n:R2:busac", "Ohm, Negative-Sequence Impedance, Real Part"
      n_R2_busdc = "n:R2:busdc", "Ohm, Negative-Sequence Impedance, Real Part"
      n_X2_busac = "n:X2:busac", "Ohm, Negative-Sequence Impedance, Imaginary Part"
      n_X2_busdc = "n:X2:busdc", "Ohm, Negative-Sequence Impedance, Imaginary Part"
      n_Z2_busac = "n:Z2:busac", "Ohm, Negative-Sequence Impedance, Magnitude"
      n_Z2_busdc = "n:Z2:busdc", "Ohm, Negative-Sequence Impedance, Magnitude"
      n_phiz2_busac = "n:phiz2:busac", "deg, Negative-Sequence Impedance, Angle"
      n_phiz2_busdc = "n:phiz2:busdc", "deg, Negative-Sequence Impedance, Angle"
      n_r2Sbase_busac = "n:r2Sbase:busac", "p.u., Negative-Sequence Impedance, Real Part"
      n_r2Sbase_busdc = "n:r2Sbase:busdc", "p.u., Negative-Sequence Impedance, Real Part"
      n_x2Sbase_busac = "n:x2Sbase:busac", "p.u., Negative-Sequence Impedance, Imaginary Part"
      n_x2Sbase_busdc = "n:x2Sbase:busdc", "p.u., Negative-Sequence Impedance, Imaginary Part"
      n_z2Sbase_busac = "n:z2Sbase:busac", "p.u., Negative-Sequence Impedance, Magnitude"
      n_z2Sbase_busdc = "n:z2Sbase:busdc", "p.u., Negative-Sequence Impedance, Magnitude"
      n_R_busac_A = "n:R:busac:A", "Ohm, Network Resistance, Line-Ground"
      n_R_busac_B = "n:R:busac:B", "Ohm, Network Resistance, Line-Ground"
      n_R_busac_C = "n:R:busac:C", "Ohm, Network Resistance, Line-Ground"
      n_X_busac_A = "n:X:busac:A", "Ohm, Network Reactance, Line-Ground"
      n_X_busac_B = "n:X:busac:B", "Ohm, Network Reactance, Line-Ground"
      n_X_busac_C = "n:X:busac:C", "Ohm, Network Reactance, Line-Ground"
      n_Z_busac_A = "n:Z:busac:A", "Ohm, Network Impedance, Line-Ground"
      n_Z_busac_B = "n:Z:busac:B", "Ohm, Network Impedance, Line-Ground"
      n_Z_busac_C = "n:Z:busac:C", "Ohm, Network Impedance, Line-Ground"
      n_phiz_busac_A = "n:phiz:busac:A", "deg, Angle of Network Impedance, Line-Ground"
      n_phiz_busac_B = "n:phiz:busac:B", "deg, Angle of Network Impedance, Line-Ground"
      n_phiz_busac_C = "n:phiz:busac:C", "deg, Angle of Network Impedance, Line-Ground"
      n_Rl_busac_A = "n:Rl:busac:A", "Ohm, Network Resistance, Line-Line"
      n_Rl_busac_B = "n:Rl:busac:B", "Ohm, Network Resistance, Line-Line"
      n_Rl_busac_C = "n:Rl:busac:C", "Ohm, Network Resistance, Line-Line"
      n_Rl_busdc = "n:Rl:busdc", "Ohm, Network Resistance, Line-Line"
      n_Xl_busac_A = "n:Xl:busac:A", "Ohm, Network Reactance, Line-Line"
      n_Xl_busac_B = "n:Xl:busac:B", "Ohm, Network Reactance, Line-Line"
      n_Xl_busac_C = "n:Xl:busac:C", "Ohm, Network Reactance, Line-Line"
      n_Xl_busdc = "n:Xl:busdc", "Ohm, Network Reactance, Line-Line"
      n_Zl_busac_A = "n:Zl:busac:A", "Ohm, Network Impedance, Line-Line"
      n_Zl_busac_B = "n:Zl:busac:B", "Ohm, Network Impedance, Line-Line"
      n_Zl_busac_C = "n:Zl:busac:C", "Ohm, Network Impedance, Line-Line"
      n_Zl_busdc = "n:Zl:busdc", "Ohm, Network Impedance, Line-Line"
      n_phizl_busac_A = "n:phizl:busac:A", "deg, Angle of Network Impedance, Line-Line"
      n_phizl_busac_B = "n:phizl:busac:B", "deg, Angle of Network Impedance, Line-Line"
      n_phizl_busac_C = "n:phizl:busac:C", "deg, Angle of Network Impedance, Line-Line"
      n_phizl_busdc = "n:phizl:busdc", "deg, Angle of Network Impedance, Line-Line"
      n_Rln_busac_A = "n:Rln:busac:A", "Ohm, Network Resistance, Line-Neutral"
      n_Rln_busac_B = "n:Rln:busac:B", "Ohm, Network Resistance, Line-Neutral"
      n_Rln_busac_C = "n:Rln:busac:C", "Ohm, Network Resistance, Line-Neutral"
      n_Rln_busdc = "n:Rln:busdc", "Ohm, Network Resistance, Line-Neutral"
      n_Xln_busac_A = "n:Xln:busac:A", "Ohm, Network Reactance, Line-Neutral"
      n_Xln_busac_B = "n:Xln:busac:B", "Ohm, Network Reactance, Line-Neutral"
      n_Xln_busac_C = "n:Xln:busac:C", "Ohm, Network Reactance, Line-Neutral"
      n_Xln_busdc = "n:Xln:busdc", "Ohm, Network Reactance, Line-Neutral"
      n_Zln_busac_A = "n:Zln:busac:A", "Ohm, Network Impedance, Line-Neutral"
      n_Zln_busac_B = "n:Zln:busac:B", "Ohm, Network Impedance, Line-Neutral"
      n_Zln_busac_C = "n:Zln:busac:C", "Ohm, Network Impedance, Line-Neutral"
      n_Zln_busdc = "n:Zln:busdc", "Ohm, Network Impedance, Line-Neutral"
      n_phizln_busac_A = "n:phizln:busac:A", "deg, Angle of Network Impedance, Line-Neutral"
      n_phizln_busac_B = "n:phizln:busac:B", "deg, Angle of Network Impedance, Line-Neutral"
      n_phizln_busac_C = "n:phizln:busac:C", "deg, Angle of Network Impedance, Line-Neutral"
      n_phizln_busdc = "n:phizln:busdc", "deg, Angle of Network Impedance, Line-Neutral"
      n_Rn_busac = "n:Rn:busac", "Ohm, Network Resistance, Neutral"
      n_Rn_busdc = "n:Rn:busdc", "Ohm, Network Resistance, Neutral"
      n_Xn_busac = "n:Xn:busac", "Ohm, Network Reactance, Neutral"
      n_Xn_busdc = "n:Xn:busdc", "Ohm, Network Reactance, Neutral"
      n_Zn_busac = "n:Zn:busac", "Ohm, Network Impedance, Neutral"
      n_Zn_busdc = "n:Zn:busdc", "Ohm, Network Impedance, Neutral"
      n_phizn_busac = "n:phizn:busac", "deg, Angle of Network Impedance, Neutral"
      n_phizn_busdc = "n:phizn:busdc", "deg, Angle of Network Impedance, Neutral"
      n_eff_busac_A = "n:eff:busac:A", "Earth Fault Factor"
      n_eff_busac_B = "n:eff:busac:B", "Earth Fault Factor"
      n_eff_busac_C = "n:eff:busac:C", "Earth Fault Factor"
      n_eff_busdc = "n:eff:busdc", "Earth Fault Factor"
      n_X0toX1_busac = "n:X0toX1:busac", "X0/X1"
      n_X0toX1_busdc = "n:X0toX1:busdc", "X0/X1"
      n_in_busac = "n:in:busac", "p.u., Neutral Current, Magnitude"
      n_in_busdc = "n:in:busdc", "p.u., Neutral Current, Magnitude"
      n_In_busac = "n:In:busac", "kA, Neutral Current, Magnitude"
      n_In_busdc = "n:In:busdc", "kA, Neutral Current, Magnitude"
      n_phiin_busac = "n:phiin:busac", "deg, Neutral Current, Angle"
      n_phiin_busdc = "n:phiin:busdc", "deg, Neutral Current, Angle"
      n_i0r_busac = "n:i0r:busac", "p.u., Zero-Sequence Current, Real Part"
      n_i0r_busdc = "n:i0r:busdc", "p.u., Zero-Sequence Current, Real Part"
      n_i0i_busac = "n:i0i:busac", "p.u., Zero-Sequence Current, Imaginary Part"
      n_i0i_busdc = "n:i0i:busdc", "p.u., Zero-Sequence Current, Imaginary Part"
      n_i0_busac = "n:i0:busac", "p.u., Zero-Sequence Current, Magnitude"
      n_i0_busdc = "n:i0:busdc", "p.u., Zero-Sequence Current, Magnitude"
      n_I0_busac = "n:I0:busac", "kA, Zero-Sequence Current, Magnitude"
      n_I0_busdc = "n:I0:busdc", "kA, Zero-Sequence Current, Magnitude"
      n_I0x3_busac = "n:I0x3:busac", "kA, 3*I0"
      n_I0x3_busdc = "n:I0x3:busdc", "kA, 3*I0"
      n_phii0_busac = "n:phii0:busac", "deg, Zero-Sequence Current, Angle"
      n_phii0_busdc = "n:phii0:busdc", "deg, Zero-Sequence Current, Angle"
      n_i1r_busac = "n:i1r:busac", "p.u., Positive-Sequence Current, Real Part"
      n_i1r_busdc = "n:i1r:busdc", "p.u., Positive-Sequence Current, Real Part"
      n_i1i_busac = "n:i1i:busac", "p.u., Positive-Sequence Current, Imaginary Part"
      n_i1i_busdc = "n:i1i:busdc", "p.u., Positive-Sequence Current, Imaginary Part"
      n_i1_busac = "n:i1:busac", "p.u., Positive-Sequence Current, Magnitude"
      n_i1_busdc = "n:i1:busdc", "p.u., Positive-Sequence Current, Magnitude"
      n_I1_busac = "n:I1:busac", "kA, Positive-Sequence Current, Magnitude"
      n_I1_busdc = "n:I1:busdc", "kA, Positive-Sequence Current, Magnitude"
      n_phii1_busac = "n:phii1:busac", "deg, Positive-Sequence Current, Angle"
      n_phii1_busdc = "n:phii1:busdc", "deg, Positive-Sequence Current, Angle"
      n_i2r_busac = "n:i2r:busac", "p.u., Negative-Sequence Current, Real Part"
      n_i2r_busdc = "n:i2r:busdc", "p.u., Negative-Sequence Current, Real Part"
      n_i2i_busac = "n:i2i:busac", "p.u., Negative-Sequence Current, Imaginary Part"
      n_i2i_busdc = "n:i2i:busdc", "p.u., Negative-Sequence Current, Imaginary Part"
      n_i2_busac = "n:i2:busac", "p.u., Negative-Sequence Current, Magnitude"
      n_i2_busdc = "n:i2:busdc", "p.u., Negative-Sequence Current, Magnitude"
      n_I2_busac = "n:I2:busac", "kA, Negative-Sequence Current, Magnitude"
      n_I2_busdc = "n:I2:busdc", "kA, Negative-Sequence Current, Magnitude"
      n_phii2_busac = "n:phii2:busac", "deg, Negative-Sequence Current, Angle"
      n_phii2_busdc = "n:phii2:busdc", "deg, Negative-Sequence Current, Angle"
      n_du_busac_A = "n:du:busac:A", "%, Voltage Deviation"
      n_du_busac_B = "n:du:busac:B", "%, Voltage Deviation"
      n_du_busac_C = "n:du:busac:C", "%, Voltage Deviation"
      n_urpref_busac_A = "n:urpref:busac:A", "p.u., Pre-fault Voltage, Real Part"
      n_urpref_busac_B = "n:urpref:busac:B", "p.u., Pre-fault Voltage, Real Part"
      n_urpref_busac_C = "n:urpref:busac:C", "p.u., Pre-fault Voltage, Real Part"
      n_uipref_busac_A = "n:uipref:busac:A", "p.u., Pre-fault Voltage, Imaginary Part"
      n_uipref_busac_B = "n:uipref:busac:B", "p.u., Pre-fault Voltage, Imaginary Part"
      n_uipref_busac_C = "n:uipref:busac:C", "p.u., Pre-fault Voltage, Imaginary Part"
      n_upref_busac_A = "n:upref:busac:A", "p.u., Pre-fault Voltage, Magnitude"
      n_upref_busac_B = "n:upref:busac:B", "p.u., Pre-fault Voltage, Magnitude"
      n_upref_busac_C = "n:upref:busac:C", "p.u., Pre-fault Voltage, Magnitude"
      n_uprefpc_busac_A = "n:uprefpc:busac:A", "%, Pre-fault Voltage, Magnitude"
      n_uprefpc_busac_B = "n:uprefpc:busac:B", "%, Pre-fault Voltage, Magnitude"
      n_uprefpc_busac_C = "n:uprefpc:busac:C", "%, Pre-fault Voltage, Magnitude"
      n_Upref_busac_A = "n:Upref:busac:A", "kV, Line-Ground Pre-fault Voltage, Magnitude"
      n_Upref_busac_B = "n:Upref:busac:B", "kV, Line-Ground Pre-fault Voltage, Magnitude"
      n_Upref_busac_C = "n:Upref:busac:C", "kV, Line-Ground Pre-fault Voltage, Magnitude"
      n_Ulpref_busac_A = "n:Ulpref:busac:A", "kV, Line-Line Pre-fault Voltage, Magnitude"
      n_Ulpref_busac_B = "n:Ulpref:busac:B", "kV, Line-Line Pre-fault Voltage, Magnitude"
      n_Ulpref_busac_C = "n:Ulpref:busac:C", "kV, Line-Line Pre-fault Voltage, Magnitude"
      n_phiupref_busac_A = "n:phiupref:busac:A", "deg, Pre-fault Voltage, Angle"
      n_phiupref_busac_B = "n:phiupref:busac:B", "deg, Pre-fault Voltage, Angle"
      n_phiupref_busac_C = "n:phiupref:busac:C", "deg, Pre-fault Voltage, Angle"
      n_phiui_busac_A = "n:phiui:busac:A", "deg, Angle between Voltage and Current"
      n_phiui_busac_B = "n:phiui:busac:B", "deg, Angle between Voltage and Current"
      n_phiui_busac_C = "n:phiui:busac:C", "deg, Angle between Voltage and Current"
      n_phiu1i1_busac = "n:phiu1i1:busac", "deg, Angle between Voltage and Current in positive sequence system"
      n_phiu1i1_busdc = "n:phiu1i1:busdc", "deg, Angle between Voltage and Current in positive sequence system"
      n_phiu2i2_busac = "n:phiu2i2:busac", "deg, Angle between Voltage and Current in negative sequence system"
      n_phiu2i2_busdc = "n:phiu2i2:busdc", "deg, Angle between Voltage and Current in negative sequence system"
      n_phiu0i0_busac = "n:phiu0i0:busac", "deg, Angle between Voltage and Current in zero-sequence system"
      n_phiu0i0_busdc = "n:phiu0i0:busdc", "deg, Angle between Voltage and Current in zero-sequence system"
      c_z1_r = "c:z1:r", "p.u., Subtransient impedance, Real Part"
      c_z1_i = "c:z1:i", "p.u., Subtransient impedance, Imaginary Part"
      c_z1_phi = "c:z1:phi", "deg, Subtransient impedance, Angle"
      c_z1_mag = "c:z1:mag", "p.u., Subtransient impedance, Magnitude"
      c_z1_phirad = "c:z1:phirad", "rad, Subtransient impedance, Angle"
      c_z2_r = "c:z2:r", "p.u., Negative sequence impedance, Real Part"
      c_z2_i = "c:z2:i", "p.u., Negative sequence impedance, Imaginary Part"
      c_z2_phi = "c:z2:phi", "deg, Negative sequence impedance, Angle"
      c_z2_mag = "c:z2:mag", "p.u., Negative sequence impedance, Magnitude"
      c_z2_phirad = "c:z2:phirad", "rad, Negative sequence impedance, Angle"
      c_z0_r = "c:z0:r", "p.u., Zero sequence impedance, Real Part"
      c_z0_i = "c:z0:i", "p.u., Zero sequence impedance, Imaginary Part"
      c_z0_phi = "c:z0:phi", "deg, Zero sequence impedance, Angle"
      c_z0_mag = "c:z0:mag", "p.u., Zero sequence impedance, Magnitude"
      c_z0_phirad = "c:z0:phirad", "rad, Zero sequence impedance, Angle"

    class ElmXnet(Enum):
      _init_ = 'value __doc__'

      m_u0_bus1 = "m:u0:bus1", "p.u., Zero-Sequence-Voltage, Magnitude"
      m_phiu0_bus1 = "m:phiu0:bus1", "deg, Zero-Sequence-Voltage, Angle"
      m_u1r_bus1 = "m:u1r:bus1", "p.u., Positive-Sequence-Voltage, Real Part"
      m_u1i_bus1 = "m:u1i:bus1", "p.u., Positive-Sequence-Voltage, Imaginary Part"
      m_u1_bus1 = "m:u1:bus1", "p.u., Positive-Sequence-Voltage, Magnitude"
      m_U1_bus1 = "m:U1:bus1", "kV, Line-Ground Positive-Sequence-Voltage, Magnitude"
      m_U1l_bus1 = "m:U1l:bus1", "kV, Line-Line Positive-Sequence-Voltage, Magnitude"
      m_phiu1_bus1 = "m:phiu1:bus1", "deg, Positive-Sequence-Voltage, Angle"
      m_u2_bus1 = "m:u2:bus1", "p.u., Negative-Sequence-Voltage, Magnitude"
      m_phiu2_bus1 = "m:phiu2:bus1", "deg, Negative-Sequence-Voltage, Angle"
      m_inet_bus1_A = "m:inet:bus1:A", "p.u., Phase Current, Magnitude, referred to network"
      m_inet_bus1_B = "m:inet:bus1:B", "p.u., Phase Current, Magnitude, referred to network"
      m_inet_bus1_C = "m:inet:bus1:C", "p.u., Phase Current, Magnitude, referred to network"
      m_I_bus1_A = "m:I:bus1:A", "kA, Phase Current, Magnitude"
      m_I_bus1_B = "m:I:bus1:B", "kA, Phase Current, Magnitude"
      m_I_bus1_C = "m:I:bus1:C", "kA, Phase Current, Magnitude"
      m_phii_bus1_A = "m:phii:bus1:A", "deg, Phase Current, Angle"
      m_phii_bus1_B = "m:phii:bus1:B", "deg, Phase Current, Angle"
      m_phii_bus1_C = "m:phii:bus1:C", "deg, Phase Current, Angle"
      m_i0r_bus1 = "m:i0r:bus1", "p.u., Zero-Sequence Current, Real Part"
      m_i0i_bus1 = "m:i0i:bus1", "p.u., Zero-Sequence Current, Imaginary Part"
      m_i0_bus1 = "m:i0:bus1", "p.u., Zero-Sequence Current, Magnitude"
      m_I0_bus1 = "m:I0:bus1", "kA, Zero-Sequence Current, Magnitude"
      m_I0x3_bus1 = "m:I0x3:bus1", "kA, 3*I0"
      m_phii0_bus1 = "m:phii0:bus1", "deg, Zero-Sequence Current, Angle"
      m_i1r_bus1 = "m:i1r:bus1", "p.u., Positive-Sequence Current, Real Part"
      m_i1i_bus1 = "m:i1i:bus1", "p.u., Positive-Sequence Current, Imaginary Part"
      m_i1_bus1 = "m:i1:bus1", "p.u., Positive-Sequence Current, Magnitude"
      m_I1_bus1 = "m:I1:bus1", "kA, Positive-Sequence Current, Magnitude"
      m_phii1_bus1 = "m:phii1:bus1", "deg, Positive-Sequence Current, Angle"
      m_i2r_bus1 = "m:i2r:bus1", "p.u., Negative-Sequence Current, Real Part"
      m_i2i_bus1 = "m:i2i:bus1", "p.u., Negative-Sequence Current, Imaginary Part"
      m_i2_bus1 = "m:i2:bus1", "p.u., Negative-Sequence Current, Magnitude"
      m_I2_bus1 = "m:I2:bus1", "kA, Negative-Sequence Current, Magnitude"
      m_phii2_bus1 = "m:phii2:bus1", "deg, Negative-Sequence Current, Angle"
      m_ubfacI_bus1 = "m:ubfacI:bus1", "%, Current Unbalance Factor"
      m_ubfacS_bus1 = "m:ubfacS:bus1", "%, Power Unbalance Factor"
      m_P_bus1_A = "m:P:bus1:A", "MW, Active Power"
      m_P_bus1_B = "m:P:bus1:B", "MW, Active Power"
      m_P_bus1_C = "m:P:bus1:C", "MW, Active Power"
      m_Q_bus1_A = "m:Q:bus1:A", "Mvar, Reactive Power"
      m_Q_bus1_B = "m:Q:bus1:B", "Mvar, Reactive Power"
      m_Q_bus1_C = "m:Q:bus1:C", "Mvar, Reactive Power"
      m_S_bus1_A = "m:S:bus1:A", "MVA, Apparent Power"
      m_S_bus1_B = "m:S:bus1:B", "MVA, Apparent Power"
      m_S_bus1_C = "m:S:bus1:C", "MVA, Apparent Power"
      m_cosphi_bus1_A = "m:cosphi:bus1:A", "Power Factor"
      m_cosphi_bus1_B = "m:cosphi:bus1:B", "Power Factor"
      m_cosphi_bus1_C = "m:cosphi:bus1:C", "Power Factor"
      m_tanphi_bus1_A = "m:tanphi:bus1:A", "tan(phi)"
      m_tanphi_bus1_B = "m:tanphi:bus1:B", "tan(phi)"
      m_tanphi_bus1_C = "m:tanphi:bus1:C", "tan(phi)"
      m_Psum_bus1 = "m:Psum:bus1", "MW, Total Active Power"
      m_Qsum_bus1 = "m:Qsum:bus1", "Mvar, Total Reactive Power"
      m_Ssum_bus1 = "m:Ssum:bus1", "MVA, Total Apparent Power"
      m_cosphisum_bus1 = "m:cosphisum:bus1", "Total Power Factor"
      m_tanphisum_bus1 = "m:tanphisum:bus1", "Total tan(phi)"
      m_Pln_bus1_A = "m:Pln:bus1:A", "MW, Active Power (Line-Neutral)"
      m_Pln_bus1_B = "m:Pln:bus1:B", "MW, Active Power (Line-Neutral)"
      m_Pln_bus1_C = "m:Pln:bus1:C", "MW, Active Power (Line-Neutral)"
      m_Qln_bus1_A = "m:Qln:bus1:A", "Mvar, Reactive Power (Line-Neutral)"
      m_Qln_bus1_B = "m:Qln:bus1:B", "Mvar, Reactive Power (Line-Neutral)"
      m_Qln_bus1_C = "m:Qln:bus1:C", "Mvar, Reactive Power (Line-Neutral)"
      m_P1_bus1 = "m:P1:bus1", "MW, Positive Sequence Active Power"
      m_Q1_bus1 = "m:Q1:bus1", "Mvar, Positive Sequence Reactive Power"
      m_P2_bus1 = "m:P2:bus1", "MW, Negative Sequence Active Power"
      m_Q2_bus1 = "m:Q2:bus1", "Mvar, Negative Sequence Reactive Power"
      m_P0_bus1 = "m:P0:bus1", "MW, Zero Sequence Active Power"
      m_Q0_bus1 = "m:Q0:bus1", "Mvar, Zero Sequence Reactive Power"
      m_i1P_bus1 = "m:i1P:bus1", "p.u., Positive-Sequence Active Current"
      m_i1Q_bus1 = "m:i1Q:bus1", "p.u., Positive-Sequence Reactive Current"
      m_I1P_bus1 = "m:I1P:bus1", "kA, Positive-Sequence Active Current"
      m_I1Q_bus1 = "m:I1Q:bus1", "kA, Positive-Sequence Reactive Current"
      m_i2P_bus1 = "m:i2P:bus1", "p.u., Negative-Sequence Active Current"
      m_i2Q_bus1 = "m:i2Q:bus1", "p.u., Negative-Sequence Reactive Current"
      m_I2P_bus1 = "m:I2P:bus1", "kA, Negative-Sequence Active Current"
      m_I2Q_bus1 = "m:I2Q:bus1", "kA, Negative-Sequence Reactive Current"
      m_ur_bus1_A = "m:ur:bus1:A", "p.u., Phase Voltage, Real Part"
      m_ur_bus1_B = "m:ur:bus1:B", "p.u., Phase Voltage, Real Part"
      m_ur_bus1_C = "m:ur:bus1:C", "p.u., Phase Voltage, Real Part"
      m_ui_bus1_A = "m:ui:bus1:A", "p.u., Phase Voltage, Imaginary Part"
      m_ui_bus1_B = "m:ui:bus1:B", "p.u., Phase Voltage, Imaginary Part"
      m_ui_bus1_C = "m:ui:bus1:C", "p.u., Phase Voltage, Imaginary Part"
      m_u_bus1_A = "m:u:bus1:A", "p.u., Phase Voltage, Magnitude"
      m_u_bus1_B = "m:u:bus1:B", "p.u., Phase Voltage, Magnitude"
      m_u_bus1_C = "m:u:bus1:C", "p.u., Phase Voltage, Magnitude"
      m_ir_bus1_A = "m:ir:bus1:A", "p.u., Phase Current, Real Part"
      m_ir_bus1_B = "m:ir:bus1:B", "p.u., Phase Current, Real Part"
      m_ir_bus1_C = "m:ir:bus1:C", "p.u., Phase Current, Real Part"
      m_ii_bus1_A = "m:ii:bus1:A", "p.u., Phase Current, Imaginary Part"
      m_ii_bus1_B = "m:ii:bus1:B", "p.u., Phase Current, Imaginary Part"
      m_ii_bus1_C = "m:ii:bus1:C", "p.u., Phase Current, Imaginary Part"
      m_i_bus1_A = "m:i:bus1:A", "p.u., Phase Current, Magnitude"
      m_i_bus1_B = "m:i:bus1:B", "p.u., Phase Current, Magnitude"
      m_i_bus1_C = "m:i:bus1:C", "p.u., Phase Current, Magnitude"
      m_phiui_bus1_A = "m:phiui:bus1:A", "deg, Angle between Voltage and Current"
      m_phiui_bus1_B = "m:phiui:bus1:B", "deg, Angle between Voltage and Current"
      m_phiui_bus1_C = "m:phiui:bus1:C", "deg, Angle between Voltage and Current"
      m_phiu0i0_bus1 = "m:phiu0i0:bus1", "deg, Angle between Voltage and Current in zero-sequence system"
      m_phiu1i1_bus1 = "m:phiu1i1:bus1", "deg, Angle between Voltage and Current in positive sequence system"
      m_phiu2i2_bus1 = "m:phiu2i2:bus1", "deg, Angle between Voltage and Current in negative sequence system"
      m_Tfct_bus1 = "m:Tfct:bus1", "s, Fault Clearing Time"
      m_TfctPh_bus1_A = "m:TfctPh:bus1:A", "s, Fault Clearing Time"
      m_TfctPh_bus1_B = "m:TfctPh:bus1:B", "s, Fault Clearing Time"
      m_TfctPh_bus1_C = "m:TfctPh:bus1:C", "s, Fault Clearing Time"
      m_Brkload_bus1 = "m:Brkload:bus1", "%, Max. Breaker Loading"
      m_BrkloadPh_bus1_A = "m:BrkloadPh:bus1:A", "%, Breaker Loading"
      m_BrkloadPh_bus1_B = "m:BrkloadPh:bus1:B", "%, Breaker Loading"
      m_BrkloadPh_bus1_C = "m:BrkloadPh:bus1:C", "%, Breaker Loading"
      n_ur_bus1_A = "n:ur:bus1:A", "p.u., Line-Ground Voltage, Real Part"
      n_ur_bus1_B = "n:ur:bus1:B", "p.u., Line-Ground Voltage, Real Part"
      n_ur_bus1_C = "n:ur:bus1:C", "p.u., Line-Ground Voltage, Real Part"
      n_ui_bus1_A = "n:ui:bus1:A", "p.u., Line-Ground Voltage, Imaginary Part"
      n_ui_bus1_B = "n:ui:bus1:B", "p.u., Line-Ground Voltage, Imaginary Part"
      n_ui_bus1_C = "n:ui:bus1:C", "p.u., Line-Ground Voltage, Imaginary Part"
      n_u_bus1_A = "n:u:bus1:A", "p.u., Line-Ground Voltage, Magnitude"
      n_u_bus1_B = "n:u:bus1:B", "p.u., Line-Ground Voltage, Magnitude"
      n_u_bus1_C = "n:u:bus1:C", "p.u., Line-Ground Voltage, Magnitude"
      n_upc_bus1_A = "n:upc:bus1:A", "%, Line-Ground Voltage, Magnitude"
      n_upc_bus1_B = "n:upc:bus1:B", "%, Line-Ground Voltage, Magnitude"
      n_upc_bus1_C = "n:upc:bus1:C", "%, Line-Ground Voltage, Magnitude"
      n_U_bus1_A = "n:U:bus1:A", "kV, Line-Ground Voltage, Magnitude"
      n_U_bus1_B = "n:U:bus1:B", "kV, Line-Ground Voltage, Magnitude"
      n_U_bus1_C = "n:U:bus1:C", "kV, Line-Ground Voltage, Magnitude"
      n_phiu_bus1_A = "n:phiu:bus1:A", "deg, Line-Ground Voltage, Angle"
      n_phiu_bus1_B = "n:phiu:bus1:B", "deg, Line-Ground Voltage, Angle"
      n_phiu_bus1_C = "n:phiu:bus1:C", "deg, Line-Ground Voltage, Angle"
      n_phiurel_bus1 = "n:phiurel:bus1", "deg, Voltage, Relative Angle"
      n_ul_bus1_A = "n:ul:bus1:A", "p.u., Line-Line Voltage, Magnitude"
      n_ul_bus1_B = "n:ul:bus1:B", "p.u., Line-Line Voltage, Magnitude"
      n_ul_bus1_C = "n:ul:bus1:C", "p.u., Line-Line Voltage, Magnitude"
      n_ulpc_bus1_A = "n:ulpc:bus1:A", "%, Line-Line Voltage, Magnitude"
      n_ulpc_bus1_B = "n:ulpc:bus1:B", "%, Line-Line Voltage, Magnitude"
      n_ulpc_bus1_C = "n:ulpc:bus1:C", "%, Line-Line Voltage, Magnitude"
      n_Ul_bus1_A = "n:Ul:bus1:A", "kV, Line-Line Voltage, Magnitude"
      n_Ul_bus1_B = "n:Ul:bus1:B", "kV, Line-Line Voltage, Magnitude"
      n_Ul_bus1_C = "n:Ul:bus1:C", "kV, Line-Line Voltage, Magnitude"
      n_phiul_bus1_A = "n:phiul:bus1:A", "deg, Line-Line Voltage, Angle"
      n_phiul_bus1_B = "n:phiul:bus1:B", "deg, Line-Line Voltage, Angle"
      n_phiul_bus1_C = "n:phiul:bus1:C", "deg, Line-Line Voltage, Angle"
      n_u0_bus1 = "n:u0:bus1", "p.u., Zero-Sequence Voltage, Magnitude"
      n_U0_bus1 = "n:U0:bus1", "kV, Zero-Sequence Voltage, Magnitude"
      n_U0x3_bus1 = "n:U0x3:bus1", "kV, 3*U0"
      n_phiu0_bus1 = "n:phiu0:bus1", "deg, Zero-Sequence Voltage, Angle"
      n_u1_bus1 = "n:u1:bus1", "p.u., Positive-Sequence Voltage, Magnitude"
      n_u1pc_bus1 = "n:u1pc:bus1", "%, Positive-Sequence Voltage, Magnitude"
      n_u1r_bus1 = "n:u1r:bus1", "p.u., Positive-Sequence Voltage, Real Part"
      n_u1i_bus1 = "n:u1i:bus1", "p.u., Positive-Sequence Voltage, Imaginary Part"
      n_U1_bus1 = "n:U1:bus1", "kV, Line-Ground Positive-Sequence Voltage, Magnitude"
      n_phiu1_bus1 = "n:phiu1:bus1", "deg, Positive-Sequence Voltage, Angle"
      n_u2_bus1 = "n:u2:bus1", "p.u., Negative-Sequence Voltage, Magnitude"
      n_U2_bus1 = "n:U2:bus1", "kV, Line-Ground Negative-Sequence Voltage, Magnitude"
      n_phiu2_bus1 = "n:phiu2:bus1", "deg, Negative-Sequence Voltage, Angle"
      n_U1l_bus1 = "n:U1l:bus1", "kV, Line-Line Positive-Sequence Voltage, Magnitude"
      n_U2l_bus1 = "n:U2l:bus1", "kV, Line-Line Negative-Sequence Voltage, Magnitude"
      n_uln_bus1_A = "n:uln:bus1:A", "p.u., Line-Neutral Voltage, Magnitude"
      n_uln_bus1_B = "n:uln:bus1:B", "p.u., Line-Neutral Voltage, Magnitude"
      n_uln_bus1_C = "n:uln:bus1:C", "p.u., Line-Neutral Voltage, Magnitude"
      n_Uln_bus1_A = "n:Uln:bus1:A", "kV, Line-Neutral Voltage, Magnitude"
      n_Uln_bus1_B = "n:Uln:bus1:B", "kV, Line-Neutral Voltage, Magnitude"
      n_Uln_bus1_C = "n:Uln:bus1:C", "kV, Line-Neutral Voltage, Magnitude"
      n_phiuln_bus1_A = "n:phiuln:bus1:A", "deg, Line-Neutral Voltage, Angle"
      n_phiuln_bus1_B = "n:phiuln:bus1:B", "deg, Line-Neutral Voltage, Angle"
      n_phiuln_bus1_C = "n:phiuln:bus1:C", "deg, Line-Neutral Voltage, Angle"
      n_un_bus1 = "n:un:bus1", "p.u., Neutral-Ground Voltage, Magnitude"
      n_Un_bus1 = "n:Un:bus1", "kV, Neutral-Ground Voltage, Magnitude"
      n_phiun_bus1 = "n:phiun:bus1", "deg, Neutral-Ground Voltage, Angle"
      n_upht_bus1_A = "n:upht:bus1:A", "p.u., Phase Technology dependent Voltage, Magnitude"
      n_upht_bus1_B = "n:upht:bus1:B", "p.u., Phase Technology dependent Voltage, Magnitude"
      n_upht_bus1_C = "n:upht:bus1:C", "p.u., Phase Technology dependent Voltage, Magnitude"
      n_Upht_bus1_A = "n:Upht:bus1:A", "kV, Phase Technology dependent Voltage, Magnitude"
      n_Upht_bus1_B = "n:Upht:bus1:B", "kV, Phase Technology dependent Voltage, Magnitude"
      n_Upht_bus1_C = "n:Upht:bus1:C", "kV, Phase Technology dependent Voltage, Magnitude"
      n_um_bus1 = "n:um:bus1", "p.u., Average-Voltage, Magnitude"
      n_Um_bus1 = "n:Um:bus1", "kV, Average-Voltage, Magnitude"
      n_uphtmin_bus1 = "n:uphtmin:bus1", "p.u., Minimum of Phase Technology dependent Voltage, Magnitude"
      n_uphtmax_bus1 = "n:uphtmax:bus1", "p.u., Maximum of Phase Technology dependent Voltage, Magnitude"
      n_Pgen_bus1 = "n:Pgen:bus1", "MW, Generation, Active Power"
      n_Qgen_bus1 = "n:Qgen:bus1", "Mvar, Generation, Reactive Power"
      n_Pmot_bus1 = "n:Pmot:bus1", "MW, Motor Load, Active Power"
      n_Qmot_bus1 = "n:Qmot:bus1", "Mvar, Motor Load, Reactive Power"
      n_Pload_bus1 = "n:Pload:bus1", "MW, General Load, Active Power"
      n_Qload_bus1 = "n:Qload:bus1", "Mvar, General Load, Reactive Power"
      n_Pcomp_bus1 = "n:Pcomp:bus1", "MW, Compensation (Losses)"
      n_Qcomp_bus1 = "n:Qcomp:bus1", "Mvar, Compensation"
      n_Pnet_bus1 = "n:Pnet:bus1", "MW, External Networks, Active Power"
      n_Qnet_bus1 = "n:Qnet:bus1", "Mvar, External Networks, Reactive Power"
      n_Pflow_bus1 = "n:Pflow:bus1", "MW, Power Flow, Active Power"
      n_Qflow_bus1 = "n:Qflow:bus1", "Mvar, Power Flow, Reactive Power"
      n_Pout_bus1 = "n:Pout:bus1", "MW, Outgoing Flow, Active Power"
      n_Qout_bus1 = "n:Qout:bus1", "Mvar, Outgoing Power, Reactive Power"
      n_Sout_bus1 = "n:Sout:bus1", "MVA, Outgoing Power, Apparent Power"
      n_cosphiout_bus1 = "n:cosphiout:bus1", "Outgoing Power, Power Factor"
      n_Pbalance_bus1 = "n:Pbalance:bus1", "MW, Active Power Balance (=0)"
      n_Qbalance_bus1 = "n:Qbalance:bus1", "Mvar, Reactive Power Balance (=0)"
      n_ubfac_bus1 = "n:ubfac:bus1", "%, Unbalance Factor"
      n_LossPdown_bus1 = "n:LossPdown:bus1", "MW, Losses, downstream"
      n_LossQdown_bus1 = "n:LossQdown:bus1", "Mvar, Losses, downstream (Reactive Power)"
      n_LossPdownload_bus1 = "n:LossPdownload:bus1", "MW, Load losses, downstream"
      n_LossQdownload_bus1 = "n:LossQdownload:bus1", "Mvar, Load losses, downstream"
      n_LossPdownnoload_bus1 = "n:LossPdownnoload:bus1", "MW, No load losses, downstream"
      n_LossQdownnoload_bus1 = "n:LossQdownnoload:bus1", "Mvar, No load losses, downstream"
      s_pset = "s:pset", "MW, IN, Active Power Setpoint, continuous, double"
      s_qset = "s:qset", "Mvar, IN, Reactive Power Setpoint continuous, double"
      s_dFin = "s:dFin", "Hz, IN, Frequency Deviation, continuous, double"
      s_dF = "s:dF", "Hz, OUT, Frequency Deviation Output, continuous, double"
      c_p = "c:p", "MW, Total Active Power"
      c_q = "c:q", "Mvar, Total Reactive Power"
      c_sgnn = "c:sgnn", "MVA, Total rated apparent power (ngnum*sgn)"
      c_cosn = "c:cosn", "Nominal-Power factor"
      c_c2 = "c:c2", "Voltage Coefficient (uknom/ugn)"
      c_uini_r = "c:uini:r", "p.u., Voltage Setpoint for PV and Slack, Real Part"
      c_uini_i = "c:uini:i", "p.u., Voltage Setpoint for PV and Slack, Imaginary Part"
      c_uini_phi = "c:uini:phi", "deg, Voltage Setpoint for PV and Slack, Angle"
      c_uini_mag = "c:uini:mag", "p.u., Voltage Setpoint for PV and Slack, Magnitude"
      c_uini_phirad = "c:uini:phirad", "rad, Voltage Setpoint for PV and Slack, Angle"
      c_iPowerOn = "c:iPowerOn", "Flag"
      c_xdq = "c:xdq", "p.u., Synchronous Reactance"
      c_rs = "c:rs", "p.u., Resistance"
      c_z2_r = "c:z2:r", "p.u., Negative Sequence Impedance, Real Part"
      c_z2_i = "c:z2:i", "p.u., Negative Sequence Impedance, Imaginary Part"
      c_z2_phi = "c:z2:phi", "deg, Negative Sequence Impedance, Angle"
      c_z2_mag = "c:z2:mag", "p.u., Negative Sequence Impedance, Magnitude"
      c_z2_phirad = "c:z2:phirad", "rad, Negative Sequence Impedance, Angle"
      c_z0_r = "c:z0:r", "p.u., Zero Sequence Impedance, Real Part"
      c_z0_i = "c:z0:i", "p.u., Zero Sequence Impedance, Imaginary Part"
      c_z0_phi = "c:z0:phi", "deg, Zero Sequence Impedance, Angle"
      c_z0_mag = "c:z0:mag", "p.u., Zero Sequence Impedance, Magnitude"
      c_z0_phirad = "c:z0:phirad", "rad, Zero Sequence Impedance, Angle"
      c_ze_r = "c:ze:r", "p.u., Grounding Impedance, Real Part"
      c_ze_i = "c:ze:i", "p.u., Grounding Impedance, Imaginary Part"
      c_ze_phi = "c:ze:phi", "deg, Grounding Impedance, Angle"
      c_ze_mag = "c:ze:mag", "p.u., Grounding Impedance, Magnitude"
      c_ze_phirad = "c:ze:phirad", "rad, Grounding Impedance, Angle"
      c_i_qlim = "c:i_qlim", "Reactive power limit reached"
      c_Qmin = "c:Qmin", "Mvar, Lower limit of reactive power"
      c_Qmax = "c:Qmax", "Mvar, Upper limit of reactive power"
      c_phiini = "c:phiini", "Initial Angle"
      c_MaxS = "c:MaxS", "MVA, Max. Apparent Power"
      c_Pdisp = "c:Pdisp", "MW, Power (Eco-Dispatch)"
      c_cst_disp = "c:cst_disp", "USD/h, Operating Costs"
      c_Kf = "c:Kf", "MW/Hz, Frequency Bias"
      c_Kp = "c:Kp", "p.u., Active Power Participation"
      c_iRefElement = "c:iRefElement", "Reference Element"
      c_nCtrlIter = "c:nCtrlIter", "No. of CTRL-iterations"
      c_Ploss = "c:Ploss", "MW, Losses (total)"
      c_Qloss = "c:Qloss", "Mvar, Reactive-Losses (total)"
      c_Plossld = "c:Plossld", "MW, Losses (load)"
      c_Qlossld = "c:Qlossld", "Mvar, Reactive-Losses (load)"
      c_Plossnld = "c:Plossnld", "MW, Losses (no load)"
      c_Qlossnld = "c:Qlossnld", "Mvar, Reactive-Losses (no load)"
      e_c_pstac = "e:c_pstac", "ElmStactrl, External Station Controller"
      e_c_psecc = "e:c_psecc", "ElmSecctrl, External Secondary Controller"
      e_bustp = "e:bustp", "Bus Type"
      e_uset_mode = "e:uset_mode", "Setpoint"
      e_outServPzero = "e:outServPzero", "Out of service when active power is zero"
      e_mode_inp = "e:mode_inp", "Input Mode"
      e_pgini = "e:pgini", "MW, Active Power"
      e_qgini = "e:qgini", "Mvar, Reactive Power"
      e_sgini = "e:sgini", "MVA, Apparent Power"
      e_cosgini = "e:cosgini", "Power Factor"
      e_pf_recap = "e:pf_recap", "Power Factor"
      e_p_direc = "e:p_direc", "Power Direction"
      e_usetp = "e:usetp", "p.u., Voltage Setpoint"
      e_phiini = "e:phiini", "deg, Angle"
      e_p_uctrl = "e:p_uctrl", "ElmTerm*,StaBar*,ElmLdfctrl*, Reference Busbar"
      e_cpCtrlNode = "e:cpCtrlNode", "Target Node"
      e_Kpf = "e:Kpf", "MW/Hz, Primary Frequency Bias"
      e_K = "e:K", "MW/Hz, Secondary Frequency Bias"
      e_pQlimType = "e:pQlimType", "IntQlim, Capability Curve"
      e_cQ_min = "e:cQ_min", "Mvar, Min."
      e_cQ_max = "e:cQ_max", "Mvar, Max."
      e_scaleQmin = "e:scaleQmin", "%, Scaling Factor (min.)"
      e_scaleQmax = "e:scaleQmax", "%, Scaling Factor (max.)"
      e_qdslCtrl = "e:qdslCtrl", "Quasi-Dynamic Model"
      e_cused = "e:cused", "Use for calculation"
      e_snss = "e:snss", "MVA, Short-Circuit Power Sk''max"
      e_ikss = "e:ikss", "kA, Short-Circuit Current Ik''max"
      e_cmax = "e:cmax", "c-Factor (max.)"
      e_rntxn = "e:rntxn", "R/X Ratio (max.)"
      e_xntrn = "e:xntrn", "X/R Ratio (max.)"
      e_z2tz1 = "e:z2tz1", "Z2/Z1 max."
      e_x0tx1 = "e:x0tx1", "X0/X1 max."
      e_r0tx0 = "e:r0tx0", "R0/X0 max."
      e_snssmin = "e:snssmin", "MVA, Short-Circuit Power Sk''min"
      e_ikssmin = "e:ikssmin", "kA, Short-Circuit Current Ik''min"
      e_cmin = "e:cmin", "c-Factor (min.)"
      e_rntxnmin = "e:rntxnmin", "R/X Ratio (min.)"
      e_xntrnmin = "e:xntrnmin", "X/R Ratio (min.)"
      e_z2tz1min = "e:z2tz1min", "Z2/Z1 min."
      e_x0tx1min = "e:x0tx1min", "X0/X1 min."
      e_r0tx0min = "e:r0tx0min", "R0/X0 min."
      e_is4SpinReserve = "e:is4SpinReserve", "Consider for region spinning reserve"
      e_genShiftKey = "e:genShiftKey", "%, Generation shift key"
      e_pBMU = "e:pBMU", "ElmBmu, Virtual Power Plant"
      e_dispatch = "e:dispatch", "Generator Dispatch"
      e_Pmin_uc = "e:Pmin_uc", "MW, Min."
      e_Pmin_ucPU = "e:Pmin_ucPU", "p.u., Min."
      e_MaxS = "e:MaxS", "MW, Max."
      e_Pmax_ucPU = "e:Pmax_ucPU", "p.u., Max."
      e_priority = "e:priority", "Merit Order"
      e_fixed = "e:fixed", "Must run"
      e_cpFeed = "e:cpFeed", "Feeder"
      e_ciDist = "e:ciDist", "Distance from infeed in number of buses"
      e_ciLater = "e:ciLater", "Lateral Index"
      e_ciDistRoot = "e:ciDistRoot", "Distance from first infeed in number of buses"
      e_ciDistAll = "e:ciDistAll", "Distance from infeed in number of buses including switches"
      e_ciDistAllRoot = "e:ciDistAllRoot", "Distance from first infeed in number of buses including switches"
      r_c_pstac = "r:c_pstac", "ElmStactrl, External Station Controller"
      r_c_psecc = "r:c_psecc", "ElmSecctrl, External Secondary Controller"
      r_p_uctrl = "r:p_uctrl", "ElmTerm*,StaBar*,ElmLdfctrl*, Reference Busbar"
      r_cpCtrlNode = "r:cpCtrlNode", "Target Node"
      r_pQlimType = "r:pQlimType", "IntQlim, Capability Curve"
      r_qdslCtrl = "r:qdslCtrl", "Quasi-Dynamic Model"
      r_pBMU = "r:pBMU", "ElmBmu, Virtual Power Plant"
      r_cpFeed = "r:cpFeed", "Feeder"
      m_Ikss_bus1 = "m:Ikss:bus1", "kA, Initial Short-Circuit Current"
      m_ikss_bus1 = "m:ikss:bus1", "p.u., Initial Short-Circuit Current, Magnitude"
      m_phii_bus1 = "m:phii:bus1", "deg, Phase Current, Angle"
      m_maxIkss_bus1 = "m:maxIkss:bus1", "kA, Max. Initial Short-Circuit Current"
      m_Skss_bus1 = "m:Skss:bus1", "MVA, Initial Short-Circuit Power"
      m_ip_bus1 = "m:ip:bus1", "kA, Peak Short-Circuit Current"
      m_maxip_bus1 = "m:maxip:bus1", "kA, Max. Peak Short-Circuit Current"
      m_maxIb_bus1 = "m:maxIb:bus1", "kA, Max. Breaking Current"
      m_maxIth_bus1 = "m:maxIth:bus1", "kA, Max. Thermal equivalent Short-Circuit Current"
      m_Ib_bus1 = "m:Ib:bus1", "kA, Short-Circuit Breaking Current"
      m_Sb_bus1 = "m:Sb:bus1", "MVA, Short-Circuit Breaking Power"
      m_Ik_bus1 = "m:Ik:bus1", "kA, Steady-State Short-Circuit Current"
      m_Ith_bus1 = "m:Ith:bus1", "kA, Thermal equivalent Short-Circuit Current"
      m_I_bus1 = "m:I:bus1", "kA, Short-Circuit Current"
      m_R_bus1 = "m:R:bus1", "Ohm, Fault Impedance, Real Part"
      m_X_bus1 = "m:X:bus1", "Ohm, Fault Impedance, Imaginary Part"
      m_Z_bus1 = "m:Z:bus1", "Ohm, Fault Impedance, Magnitude"
      m_phiz_bus1 = "m:phiz:bus1", "deg, Fault Impedance, Angle"
      m_zSbase_bus1 = "m:zSbase:bus1", "p.u., Fault Impedance, Magnitude"
      m_XtoR_bus1 = "m:XtoR:bus1", "X/R ratio"
      m_Zs_bus1 = "m:Zs:bus1", "Ohm, Source Impedance, Magnitude"
      m_phizs_bus1 = "m:phizs:bus1", "deg, Source Impedance, Angle"
      m_Rs_bus1 = "m:Rs:bus1", "Ohm, Source Impedance, Real Part"
      m_Xs_bus1 = "m:Xs:bus1", "Ohm, Source Impedance, Imaginary Part"
      m_phiui_bus1 = "m:phiui:bus1", "deg, Angle between Voltage and Current"
      m_cosphiui_bus1 = "m:cosphiui:bus1", "cosphi"
      m_P_bus1 = "m:P:bus1", "MW, Active Power"
      m_Q_bus1 = "m:Q:bus1", "Mvar, Reactive Power"
      m_BrkIpload_bus1 = "m:BrkIpload:bus1", "%, Breaker Loading (peak)"
      m_BrkIbload_bus1 = "m:BrkIbload:bus1", "%, Breaker Loading (breaking)"
      m_BrkmaxIb_bus1 = "m:BrkmaxIb:bus1", "kA, Max. Breaking Current (breaker time)"
      n_Ikss_bus1 = "n:Ikss:bus1", "kA, Initial Short-Circuit Current"
      n_I_bus1 = "n:I:bus1", "kA, Short-Circuit Current"
      n_phii_bus1 = "n:phii:bus1", "deg, Phase Current, Angle"
      n_Skss_bus1 = "n:Skss:bus1", "MVA, Initial Short-Circuit Power"
      n_ip_bus1 = "n:ip:bus1", "kA, Peak Short-Circuit Current"
      n_Ib_bus1 = "n:Ib:bus1", "kA, Short-Circuit Breaking Current"
      n_Sb_bus1 = "n:Sb:bus1", "MVA, Short-Circuit Breaking Power"
      n_Ik_bus1 = "n:Ik:bus1", "kA, Steady-State Short-Circuit Current"
      n_Ith_bus1 = "n:Ith:bus1", "kA, Thermal equivalent Short-Circuit Current"
      n_R_bus1 = "n:R:bus1", "Ohm, Short-Circuit Impedance, Real Part"
      n_X_bus1 = "n:X:bus1", "Ohm, Short-Circuit Impedance, Imaginary Part"
      n_Z_bus1 = "n:Z:bus1", "Ohm, Short-Circuit Impedance, Magnitude"
      n_phiz_bus1 = "n:phiz:bus1", "deg, Short-Circuit Impedance, Angle"
      n_rSbase_bus1 = "n:rSbase:bus1", "p.u., Short-Circuit Impedance, Real Part"
      n_xSbase_bus1 = "n:xSbase:bus1", "p.u., Short-Circuit Impedance, Imaginary Part"
      n_zSbase_bus1 = "n:zSbase:bus1", "p.u., Short-Circuit Impedance, Magnitude"
      n_XtoR_bus1 = "n:XtoR:bus1", "X/R ratio (peak)"
      n_RtoX_bus1 = "n:RtoX:bus1", "R/X ratio (peak)"
      n_ur_bus1 = "n:ur:bus1", "p.u., Voltage, Real Part"
      n_ui_bus1 = "n:ui:bus1", "p.u., Voltage, Imaginary Part"
      n_u_bus1 = "n:u:bus1", "p.u., Voltage, Magnitude"
      n_upc_bus1 = "n:upc:bus1", "%, Voltage, Magnitude"
      n_U_bus1 = "n:U:bus1", "kV, Line-Ground Voltage, Magnitude"
      n_Ul_bus1 = "n:Ul:bus1", "kV, Line-Line Voltage, Magnitude"
      n_phiu_bus1 = "n:phiu:bus1", "deg, Voltage, Angle"
      n_du_bus1 = "n:du:bus1", "%, Voltage Deviation"
      n_urpref_bus1 = "n:urpref:bus1", "p.u., Pre-fault Voltage, Real Part"
      n_uipref_bus1 = "n:uipref:bus1", "p.u., Pre-fault Voltage, Imaginary Part"
      n_upref_bus1 = "n:upref:bus1", "p.u., Pre-fault Voltage, Magnitude"
      n_uprefpc_bus1 = "n:uprefpc:bus1", "%, Pre-fault Voltage, Magnitude"
      n_Upref_bus1 = "n:Upref:bus1", "kV, Line-Ground Pre-fault Voltage, Magnitude"
      n_Ulpref_bus1 = "n:Ulpref:bus1", "kV, Line-Line Pre-fault Voltage, Magnitude"
      n_phiupref_bus1 = "n:phiupref:bus1", "deg, Pre-fault Voltage, Angle"
      n_phiui_bus1 = "n:phiui:bus1", "deg, Angle between Voltage and Current"
      n_cfac_bus1 = "n:cfac:bus1", "Voltage Factor"
      n_idc_bus1 = "n:idc:bus1", "kA, Decaying Component"
      n_Iasy_bus1 = "n:Iasy:bus1", "kA, Asymmetrical Breaking Current"
      n_Ithload_bus1 = "n:Ithload:bus1", "%, Loading, Thermal equivalent Current"
      n_Ipload_bus1 = "n:Ipload:bus1", "%, Loading, Peak Short-Circuit Current"
      n_Ithrtk_bus1 = "n:Ithrtk:bus1", "kA, Rated Short-Time Current (Tk)"
      n_Tn_bus1 = "n:Tn:bus1", "ms, Grid Time Constant"
      n_XtoR_b_bus1 = "n:XtoR_b:bus1", "X/R ratio (dc)"
      n_RtoX_b_bus1 = "n:RtoX_b:bus1", "R/X ratio (dc)"
      n_fctof_b_bus1 = "n:fctof_b:bus1", "Equivalent Frequency fc/f (dc)"
      c_z1_r = "c:z1:r", "p.u., Positive-sequence impedance, Real Part"
      c_z1_i = "c:z1:i", "p.u., Positive-sequence impedance, Imaginary Part"
      c_z1_phi = "c:z1:phi", "deg, Positive-sequence impedance, Angle"
      c_z1_mag = "c:z1:mag", "p.u., Positive-sequence impedance, Magnitude"
      c_z1_phirad = "c:z1:phirad", "rad, Positive-sequence impedance, Angle"
      c_uini1_r = "c:uini1:r", "p.u., Internal positive-sequence voltage, Real Part"
      c_uini1_i = "c:uini1:i", "p.u., Internal positive-sequence voltage, Imaginary Part"
      c_uini1_phi = "c:uini1:phi", "deg, Internal positive-sequence voltage, Angle"
      c_uini1_mag = "c:uini1:mag", "p.u., Internal positive-sequence voltage, Magnitude"
      c_uini1_phirad = "c:uini1:phirad", "rad, Internal positive-sequence voltage, Angle"
      m_Ikss_bus1_A = "m:Ikss:bus1:A", "kA, Initial Short-Circuit Current"
      m_Ikss_bus1_B = "m:Ikss:bus1:B", "kA, Initial Short-Circuit Current"
      m_Ikss_bus1_C = "m:Ikss:bus1:C", "kA, Initial Short-Circuit Current"
      m_ikss_bus1_A = "m:ikss:bus1:A", "p.u., Initial Short-Circuit Current, Magnitude"
      m_ikss_bus1_B = "m:ikss:bus1:B", "p.u., Initial Short-Circuit Current, Magnitude"
      m_ikss_bus1_C = "m:ikss:bus1:C", "p.u., Initial Short-Circuit Current, Magnitude"
      m_Skss_bus1_A = "m:Skss:bus1:A", "MVA, Initial Short-Circuit Power"
      m_Skss_bus1_B = "m:Skss:bus1:B", "MVA, Initial Short-Circuit Power"
      m_Skss_bus1_C = "m:Skss:bus1:C", "MVA, Initial Short-Circuit Power"
      m_Skssx3_bus1_A = "m:Skssx3:bus1:A", "MVA, 3*Initial Short-Circuit Power"
      m_Skssx3_bus1_B = "m:Skssx3:bus1:B", "MVA, 3*Initial Short-Circuit Power"
      m_Skssx3_bus1_C = "m:Skssx3:bus1:C", "MVA, 3*Initial Short-Circuit Power"
      m_ip_bus1_A = "m:ip:bus1:A", "kA, Peak Short-Circuit Current"
      m_ip_bus1_B = "m:ip:bus1:B", "kA, Peak Short-Circuit Current"
      m_ip_bus1_C = "m:ip:bus1:C", "kA, Peak Short-Circuit Current"
      m_Z_bus1_A = "m:Z:bus1:A", "Ohm, Fault Impedance, Magnitude"
      m_Z_bus1_B = "m:Z:bus1:B", "Ohm, Fault Impedance, Magnitude"
      m_Z_bus1_C = "m:Z:bus1:C", "Ohm, Fault Impedance, Magnitude"
      m_phiz_bus1_A = "m:phiz:bus1:A", "deg, Fault Impedance, Angle"
      m_phiz_bus1_B = "m:phiz:bus1:B", "deg, Fault Impedance, Angle"
      m_phiz_bus1_C = "m:phiz:bus1:C", "deg, Fault Impedance, Angle"
      m_Zl_bus1_A = "m:Zl:bus1:A", "Ohm, Line-Line Fault Impedance, Magnitude"
      m_Zl_bus1_B = "m:Zl:bus1:B", "Ohm, Line-Line Fault Impedance, Magnitude"
      m_Zl_bus1_C = "m:Zl:bus1:C", "Ohm, Line-Line Fault Impedance, Magnitude"
      m_phizl_bus1_A = "m:phizl:bus1:A", "deg, Line-Line Fault Impedance, Angle"
      m_phizl_bus1_B = "m:phizl:bus1:B", "deg, Line-Line Fault Impedance, Angle"
      m_phizl_bus1_C = "m:phizl:bus1:C", "deg, Line-Line Fault Impedance, Angle"
      m_Z0_bus1 = "m:Z0:bus1", "Ohm, Zero-Sequence Fault Impedance, Magnitude"
      m_phiz0_bus1 = "m:phiz0:bus1", "deg, Zero-Sequence Fault Impedance, Angle"
      m_z0Sbase_bus1 = "m:z0Sbase:bus1", "p.u., Zero-Sequence Fault Impedance, Magnitude"
      m_cosphiu1i1_bus1 = "m:cosphiu1i1:bus1", "Positive-Sequence cosphi"
      m_I0x3watt_bus1 = "m:I0x3watt:bus1", "kA, Earth-Fault Residual Current"
      m_I0x3var_bus1 = "m:I0x3var:bus1", "kA, Capacitive Earth-Fault Current"
      m_Q0cap_bus1 = "m:Q0cap:bus1", "Mvar, Q0 capacitive"
      m_Zfrel_bus1 = "m:Zfrel:bus1", "Ohm, Relay Impedance, Magnitude"
      m_phizfrel_bus1 = "m:phizfrel:bus1", "deg, Relay Impedance, Angle"
      m_Rfrel_bus1 = "m:Rfrel:bus1", "Ohm, Relay Impedance, Real Part"
      m_Xfrel_bus1 = "m:Xfrel:bus1", "Ohm, Relay Impedance, Imaginary Part"
      m_Zf_bus1 = "m:Zf:bus1", "Ohm, Meas. Fault Impedance, Magnitude"
      m_phizf_bus1 = "m:phizf:bus1", "deg, Meas. Fault Impedance, Angle"
      n_Ikss_bus1_A = "n:Ikss:bus1:A", "kA, Initial Short-Circuit Current"
      n_Ikss_bus1_B = "n:Ikss:bus1:B", "kA, Initial Short-Circuit Current"
      n_Ikss_bus1_C = "n:Ikss:bus1:C", "kA, Initial Short-Circuit Current"
      n_I_bus1_A = "n:I:bus1:A", "kA, Short-Circuit Current"
      n_I_bus1_B = "n:I:bus1:B", "kA, Short-Circuit Current"
      n_I_bus1_C = "n:I:bus1:C", "kA, Short-Circuit Current"
      n_phii_bus1_A = "n:phii:bus1:A", "deg, Phase Current, Angle"
      n_phii_bus1_B = "n:phii:bus1:B", "deg, Phase Current, Angle"
      n_phii_bus1_C = "n:phii:bus1:C", "deg, Phase Current, Angle"
      n_Skss_bus1_A = "n:Skss:bus1:A", "MVA, Initial Short-Circuit Power"
      n_Skss_bus1_B = "n:Skss:bus1:B", "MVA, Initial Short-Circuit Power"
      n_Skss_bus1_C = "n:Skss:bus1:C", "MVA, Initial Short-Circuit Power"
      n_Skssx3_bus1_A = "n:Skssx3:bus1:A", "MVA, 3*Initial Short-Circuit Power"
      n_Skssx3_bus1_B = "n:Skssx3:bus1:B", "MVA, 3*Initial Short-Circuit Power"
      n_Skssx3_bus1_C = "n:Skssx3:bus1:C", "MVA, 3*Initial Short-Circuit Power"
      n_ip_bus1_A = "n:ip:bus1:A", "kA, Peak Short-Circuit Current"
      n_ip_bus1_B = "n:ip:bus1:B", "kA, Peak Short-Circuit Current"
      n_ip_bus1_C = "n:ip:bus1:C", "kA, Peak Short-Circuit Current"
      n_Ib_bus1_A = "n:Ib:bus1:A", "kA, Short-Circuit Breaking Current"
      n_Ib_bus1_B = "n:Ib:bus1:B", "kA, Short-Circuit Breaking Current"
      n_Ib_bus1_C = "n:Ib:bus1:C", "kA, Short-Circuit Breaking Current"
      n_Sb_bus1_A = "n:Sb:bus1:A", "MVA, Short-Circuit Breaking Power"
      n_Sb_bus1_B = "n:Sb:bus1:B", "MVA, Short-Circuit Breaking Power"
      n_Sb_bus1_C = "n:Sb:bus1:C", "MVA, Short-Circuit Breaking Power"
      n_Ith_bus1_A = "n:Ith:bus1:A", "kA, Thermal equivalent Short-Circuit Current"
      n_Ith_bus1_B = "n:Ith:bus1:B", "kA, Thermal equivalent Short-Circuit Current"
      n_Ith_bus1_C = "n:Ith:bus1:C", "kA, Thermal equivalent Short-Circuit Current"
      n_Ithmax_bus1 = "n:Ithmax:bus1", "kA, Max. Thermal equivalent Short-Circuit Current"
      n_Ik_bus1_A = "n:Ik:bus1:A", "kA, Steady-State Short-Circuit Current"
      n_Ik_bus1_B = "n:Ik:bus1:B", "kA, Steady-State Short-Circuit Current"
      n_Ik_bus1_C = "n:Ik:bus1:C", "kA, Steady-State Short-Circuit Current"
      n_R0_bus1 = "n:R0:bus1", "Ohm, Zero-Sequence Impedance, Real Part"
      n_X0_bus1 = "n:X0:bus1", "Ohm, Zero-Sequence Impedance, Imaginary Part"
      n_Z0_bus1 = "n:Z0:bus1", "Ohm, Zero-Sequence Impedance, Magnitude"
      n_phiz0_bus1 = "n:phiz0:bus1", "deg, Zero-Sequence Impedance, Angle"
      n_r0Sbase_bus1 = "n:r0Sbase:bus1", "p.u., Zero-Sequence Impedance, Real Part"
      n_x0Sbase_bus1 = "n:x0Sbase:bus1", "p.u., Zero-Sequence Impedance, Imaginary Part"
      n_z0Sbase_bus1 = "n:z0Sbase:bus1", "p.u., Zero-Sequence Impedance, Magnitude"
      n_R1_bus1 = "n:R1:bus1", "Ohm, Positive-Sequence Impedance, Real Part"
      n_X1_bus1 = "n:X1:bus1", "Ohm, Positive-Sequence Impedance, Imaginary Part"
      n_Z1_bus1 = "n:Z1:bus1", "Ohm, Positive-Sequence Impedance, Magnitude"
      n_phiz1_bus1 = "n:phiz1:bus1", "deg, Positive-Sequence Impedance, Angle"
      n_r1Sbase_bus1 = "n:r1Sbase:bus1", "p.u., Positive-Sequence Impedance, Real Part"
      n_x1Sbase_bus1 = "n:x1Sbase:bus1", "p.u., Positive-Sequence Impedance, Imaginary Part"
      n_z1Sbase_bus1 = "n:z1Sbase:bus1", "p.u., Positive-Sequence Impedance, Magnitude"
      n_R2_bus1 = "n:R2:bus1", "Ohm, Negative-Sequence Impedance, Real Part"
      n_X2_bus1 = "n:X2:bus1", "Ohm, Negative-Sequence Impedance, Imaginary Part"
      n_Z2_bus1 = "n:Z2:bus1", "Ohm, Negative-Sequence Impedance, Magnitude"
      n_phiz2_bus1 = "n:phiz2:bus1", "deg, Negative-Sequence Impedance, Angle"
      n_r2Sbase_bus1 = "n:r2Sbase:bus1", "p.u., Negative-Sequence Impedance, Real Part"
      n_x2Sbase_bus1 = "n:x2Sbase:bus1", "p.u., Negative-Sequence Impedance, Imaginary Part"
      n_z2Sbase_bus1 = "n:z2Sbase:bus1", "p.u., Negative-Sequence Impedance, Magnitude"
      n_R_bus1_A = "n:R:bus1:A", "Ohm, Network Resistance, Line-Ground"
      n_R_bus1_B = "n:R:bus1:B", "Ohm, Network Resistance, Line-Ground"
      n_R_bus1_C = "n:R:bus1:C", "Ohm, Network Resistance, Line-Ground"
      n_X_bus1_A = "n:X:bus1:A", "Ohm, Network Reactance, Line-Ground"
      n_X_bus1_B = "n:X:bus1:B", "Ohm, Network Reactance, Line-Ground"
      n_X_bus1_C = "n:X:bus1:C", "Ohm, Network Reactance, Line-Ground"
      n_Z_bus1_A = "n:Z:bus1:A", "Ohm, Network Impedance, Line-Ground"
      n_Z_bus1_B = "n:Z:bus1:B", "Ohm, Network Impedance, Line-Ground"
      n_Z_bus1_C = "n:Z:bus1:C", "Ohm, Network Impedance, Line-Ground"
      n_phiz_bus1_A = "n:phiz:bus1:A", "deg, Angle of Network Impedance, Line-Ground"
      n_phiz_bus1_B = "n:phiz:bus1:B", "deg, Angle of Network Impedance, Line-Ground"
      n_phiz_bus1_C = "n:phiz:bus1:C", "deg, Angle of Network Impedance, Line-Ground"
      n_Rl_bus1_A = "n:Rl:bus1:A", "Ohm, Network Resistance, Line-Line"
      n_Rl_bus1_B = "n:Rl:bus1:B", "Ohm, Network Resistance, Line-Line"
      n_Rl_bus1_C = "n:Rl:bus1:C", "Ohm, Network Resistance, Line-Line"
      n_Xl_bus1_A = "n:Xl:bus1:A", "Ohm, Network Reactance, Line-Line"
      n_Xl_bus1_B = "n:Xl:bus1:B", "Ohm, Network Reactance, Line-Line"
      n_Xl_bus1_C = "n:Xl:bus1:C", "Ohm, Network Reactance, Line-Line"
      n_Zl_bus1_A = "n:Zl:bus1:A", "Ohm, Network Impedance, Line-Line"
      n_Zl_bus1_B = "n:Zl:bus1:B", "Ohm, Network Impedance, Line-Line"
      n_Zl_bus1_C = "n:Zl:bus1:C", "Ohm, Network Impedance, Line-Line"
      n_phizl_bus1_A = "n:phizl:bus1:A", "deg, Angle of Network Impedance, Line-Line"
      n_phizl_bus1_B = "n:phizl:bus1:B", "deg, Angle of Network Impedance, Line-Line"
      n_phizl_bus1_C = "n:phizl:bus1:C", "deg, Angle of Network Impedance, Line-Line"
      n_Rln_bus1_A = "n:Rln:bus1:A", "Ohm, Network Resistance, Line-Neutral"
      n_Rln_bus1_B = "n:Rln:bus1:B", "Ohm, Network Resistance, Line-Neutral"
      n_Rln_bus1_C = "n:Rln:bus1:C", "Ohm, Network Resistance, Line-Neutral"
      n_Xln_bus1_A = "n:Xln:bus1:A", "Ohm, Network Reactance, Line-Neutral"
      n_Xln_bus1_B = "n:Xln:bus1:B", "Ohm, Network Reactance, Line-Neutral"
      n_Xln_bus1_C = "n:Xln:bus1:C", "Ohm, Network Reactance, Line-Neutral"
      n_Zln_bus1_A = "n:Zln:bus1:A", "Ohm, Network Impedance, Line-Neutral"
      n_Zln_bus1_B = "n:Zln:bus1:B", "Ohm, Network Impedance, Line-Neutral"
      n_Zln_bus1_C = "n:Zln:bus1:C", "Ohm, Network Impedance, Line-Neutral"
      n_phizln_bus1_A = "n:phizln:bus1:A", "deg, Angle of Network Impedance, Line-Neutral"
      n_phizln_bus1_B = "n:phizln:bus1:B", "deg, Angle of Network Impedance, Line-Neutral"
      n_phizln_bus1_C = "n:phizln:bus1:C", "deg, Angle of Network Impedance, Line-Neutral"
      n_Rn_bus1 = "n:Rn:bus1", "Ohm, Network Resistance, Neutral"
      n_Xn_bus1 = "n:Xn:bus1", "Ohm, Network Reactance, Neutral"
      n_Zn_bus1 = "n:Zn:bus1", "Ohm, Network Impedance, Neutral"
      n_phizn_bus1 = "n:phizn:bus1", "deg, Angle of Network Impedance, Neutral"
      n_eff_bus1_A = "n:eff:bus1:A", "Earth Fault Factor"
      n_eff_bus1_B = "n:eff:bus1:B", "Earth Fault Factor"
      n_eff_bus1_C = "n:eff:bus1:C", "Earth Fault Factor"
      n_X0toX1_bus1 = "n:X0toX1:bus1", "X0/X1"
      n_in_bus1 = "n:in:bus1", "p.u., Neutral Current, Magnitude"
      n_In_bus1 = "n:In:bus1", "kA, Neutral Current, Magnitude"
      n_phiin_bus1 = "n:phiin:bus1", "deg, Neutral Current, Angle"
      n_i0r_bus1 = "n:i0r:bus1", "p.u., Zero-Sequence Current, Real Part"
      n_i0i_bus1 = "n:i0i:bus1", "p.u., Zero-Sequence Current, Imaginary Part"
      n_i0_bus1 = "n:i0:bus1", "p.u., Zero-Sequence Current, Magnitude"
      n_I0_bus1 = "n:I0:bus1", "kA, Zero-Sequence Current, Magnitude"
      n_I0x3_bus1 = "n:I0x3:bus1", "kA, 3*I0"
      n_phii0_bus1 = "n:phii0:bus1", "deg, Zero-Sequence Current, Angle"
      n_i1r_bus1 = "n:i1r:bus1", "p.u., Positive-Sequence Current, Real Part"
      n_i1i_bus1 = "n:i1i:bus1", "p.u., Positive-Sequence Current, Imaginary Part"
      n_i1_bus1 = "n:i1:bus1", "p.u., Positive-Sequence Current, Magnitude"
      n_I1_bus1 = "n:I1:bus1", "kA, Positive-Sequence Current, Magnitude"
      n_phii1_bus1 = "n:phii1:bus1", "deg, Positive-Sequence Current, Angle"
      n_i2r_bus1 = "n:i2r:bus1", "p.u., Negative-Sequence Current, Real Part"
      n_i2i_bus1 = "n:i2i:bus1", "p.u., Negative-Sequence Current, Imaginary Part"
      n_i2_bus1 = "n:i2:bus1", "p.u., Negative-Sequence Current, Magnitude"
      n_I2_bus1 = "n:I2:bus1", "kA, Negative-Sequence Current, Magnitude"
      n_phii2_bus1 = "n:phii2:bus1", "deg, Negative-Sequence Current, Angle"
      n_du_bus1_A = "n:du:bus1:A", "%, Voltage Deviation"
      n_du_bus1_B = "n:du:bus1:B", "%, Voltage Deviation"
      n_du_bus1_C = "n:du:bus1:C", "%, Voltage Deviation"
      n_urpref_bus1_A = "n:urpref:bus1:A", "p.u., Pre-fault Voltage, Real Part"
      n_urpref_bus1_B = "n:urpref:bus1:B", "p.u., Pre-fault Voltage, Real Part"
      n_urpref_bus1_C = "n:urpref:bus1:C", "p.u., Pre-fault Voltage, Real Part"
      n_uipref_bus1_A = "n:uipref:bus1:A", "p.u., Pre-fault Voltage, Imaginary Part"
      n_uipref_bus1_B = "n:uipref:bus1:B", "p.u., Pre-fault Voltage, Imaginary Part"
      n_uipref_bus1_C = "n:uipref:bus1:C", "p.u., Pre-fault Voltage, Imaginary Part"
      n_upref_bus1_A = "n:upref:bus1:A", "p.u., Pre-fault Voltage, Magnitude"
      n_upref_bus1_B = "n:upref:bus1:B", "p.u., Pre-fault Voltage, Magnitude"
      n_upref_bus1_C = "n:upref:bus1:C", "p.u., Pre-fault Voltage, Magnitude"
      n_uprefpc_bus1_A = "n:uprefpc:bus1:A", "%, Pre-fault Voltage, Magnitude"
      n_uprefpc_bus1_B = "n:uprefpc:bus1:B", "%, Pre-fault Voltage, Magnitude"
      n_uprefpc_bus1_C = "n:uprefpc:bus1:C", "%, Pre-fault Voltage, Magnitude"
      n_Upref_bus1_A = "n:Upref:bus1:A", "kV, Line-Ground Pre-fault Voltage, Magnitude"
      n_Upref_bus1_B = "n:Upref:bus1:B", "kV, Line-Ground Pre-fault Voltage, Magnitude"
      n_Upref_bus1_C = "n:Upref:bus1:C", "kV, Line-Ground Pre-fault Voltage, Magnitude"
      n_Ulpref_bus1_A = "n:Ulpref:bus1:A", "kV, Line-Line Pre-fault Voltage, Magnitude"
      n_Ulpref_bus1_B = "n:Ulpref:bus1:B", "kV, Line-Line Pre-fault Voltage, Magnitude"
      n_Ulpref_bus1_C = "n:Ulpref:bus1:C", "kV, Line-Line Pre-fault Voltage, Magnitude"
      n_phiupref_bus1_A = "n:phiupref:bus1:A", "deg, Pre-fault Voltage, Angle"
      n_phiupref_bus1_B = "n:phiupref:bus1:B", "deg, Pre-fault Voltage, Angle"
      n_phiupref_bus1_C = "n:phiupref:bus1:C", "deg, Pre-fault Voltage, Angle"
      n_phiui_bus1_A = "n:phiui:bus1:A", "deg, Angle between Voltage and Current"
      n_phiui_bus1_B = "n:phiui:bus1:B", "deg, Angle between Voltage and Current"
      n_phiui_bus1_C = "n:phiui:bus1:C", "deg, Angle between Voltage and Current"
      n_phiu1i1_bus1 = "n:phiu1i1:bus1", "deg, Angle between Voltage and Current in positive sequence system"
      n_phiu2i2_bus1 = "n:phiu2i2:bus1", "deg, Angle between Voltage and Current in negative sequence system"
      n_phiu0i0_bus1 = "n:phiu0i0:bus1", "deg, Angle between Voltage and Current in zero-sequence system"

    class ElmZone(Enum):
      _init_ = 'value __doc__'

      c_n_ele = "c:n_ele", "Number of Elements"
      c_n_zones = "c:n_zones", "Number of Connected Zones"
      c_n_volt = "c:n_volt", "Number of Voltage Levels"
      c_n_bar = "c:n_bar", "No. of Busbars"
      c_n_term = "c:n_term", "No. of Terminals"
      c_n_lne = "c:n_lne", "No. of Lines"
      c_n_sym = "c:n_sym", "No. of syn. Machines"
      c_n_asm = "c:n_asm", "No. of asyn. Machines"
      c_n_stg = "c:n_stg", "No. of Static Generators"
      c_n_pvs = "c:n_pvs", "No. of PV Systems"
      c_n_lod = "c:n_lod", "No. of Loads"
      c_n_cust = "c:n_cust", "No. of Customers"
      c_TotgenP = "c:TotgenP", "MW, Total Generation, Active Power"
      c_TotgenQ = "c:TotgenQ", "Mvar, Total Generation, Reactive Power"
      c_TotconsP = "c:TotconsP", "MW, Total Consumption, Active Power"
      c_TotconsQ = "c:TotconsQ", "Mvar, Total Consumption, Reactive Power"
      c_GenP = "c:GenP", "MW, Generators, Active Power"
      c_GenQ = "c:GenQ", "Mvar, Generators, Reactive Power"
      c_GenS = "c:GenS", "MVA, Generators, Apparent Power"
      c_Gencos = "c:Gencos", "Generators, Power Factor"
      c_GenPnom = "c:GenPnom", "MW, Generators, Nominal Active Power"
      c_GenQnom = "c:GenQnom", "Mvar, Generators, Nominal Reactive Power"
      c_GenSnom = "c:GenSnom", "MVA, Generators, Nominal Apparent Power"
      c_GenPdiff = "c:GenPdiff", "MW, Generators, difference between nominal and actual active power"
      c_GenQdiff = "c:GenQdiff", "Mvar, Generators, difference between nominal and actual reactive power"
      c_Gencosnom = "c:Gencosnom", "Generators, Nominal Power Factor"
      c_GenPdisp = "c:GenPdisp", "MW, Generators, Dispatch Active Power"
      c_GenPmism = "c:GenPmism", "MW, Generators, difference between actual and dispatched active power"
      c_GenPspinres = "c:GenPspinres", "MW, Generators, difference between maximum and actual active power"
      c_GenQspinres = "c:GenQspinres", "Mvar, Generators, difference between maximum and actual reactive power"
      c_MotP = "c:MotP", "MW, Motor Loads, Active Power"
      c_MotQ = "c:MotQ", "Mvar, Motor Loads, Reactive Power"
      c_MotS = "c:MotS", "MVA, Motor Loads, Apparent Power"
      c_Motcos = "c:Motcos", "Motor Loads, Power Factor"
      c_LoadP = "c:LoadP", "MW, Loads, Active Power"
      c_LoadQ = "c:LoadQ", "Mvar, Loads, Reactive Power"
      c_LoadS = "c:LoadS", "MVA, Loads, Apparent Power"
      c_Loadcos = "c:Loadcos", "Loads, Power Factor"
      c_LoadPnom = "c:LoadPnom", "MW, Loads, Nominal Active Power"
      c_LoadQnom = "c:LoadQnom", "Mvar, Loads, Nominal Reactive Power"
      c_LoadSnom = "c:LoadSnom", "MVA, Loads, Nominal Apparent Power"
      c_Loadcosnom = "c:Loadcosnom", "Loads, Nominal Power Factor"
      c_LoadPdiff = "c:LoadPdiff", "MW, Loads, difference between nominal and actual active power"
      c_LoadQdiff = "c:LoadQdiff", "Mvar, Loads, difference between nominal and actual reactive power"
      c_LoadPdem = "c:LoadPdem", "MW, Loads, Active Power Demand"
      c_LoadPmism = "c:LoadPmism", "MW, Loads, difference between actual active power and demand"
      c_CompP = "c:CompP", "MW, Compensation (Losses)"
      c_CompQ = "c:CompQ", "Mvar, Compensation, C"
      c_ComprQ = "c:ComprQ", "Mvar, Compensation, L"
      c_NetP = "c:NetP", "MW, External Networks, Active Power"
      c_NetQ = "c:NetQ", "Mvar, External Networks, Reactive Power"
      c_NetS = "c:NetS", "MVA, External Networks, Apparent Power"
      c_Netcos = "c:Netcos", "External Networks, Power Factor"
      c_InterP = "c:InterP", "MW, Interchange Flow, Active Power"
      c_InterQ = "c:InterQ", "Mvar, Interchange Flow, Reactive Power"
      c_LossP = "c:LossP", "MW, Losses, Active Power"
      c_LossQ = "c:LossQ", "Mvar, Losses, Reactive Power"
      c_LossPld = "c:LossPld", "MW, Losses, Active Power (load)"
      c_LossQld = "c:LossQld", "Mvar, Losses, Reactive Power (load)"
      c_LossPnld = "c:LossPnld", "MW, Losses, Active Power (no load)"
      c_LossQnld = "c:LossQnld", "Mvar, Losses, Reactive Power (no load)"
      c_LossPlne = "c:LossPlne", "MW, Line Losses, Active Power"
      c_LossQlne = "c:LossQlne", "Mvar, Line Losses, Reactive Power"
      c_LossPldlne = "c:LossPldlne", "MW, Line Losses, Active Power (load)"
      c_LossQldlne = "c:LossQldlne", "Mvar, Line Losses, Reactive Power (load)"
      c_LossPnldlne = "c:LossPnldlne", "MW, Line Losses, Active Power (no load)"
      c_LossQnldlne = "c:LossQnldlne", "Mvar, Line Losses, Reactive Power (no load)"
      c_LossPtrf = "c:LossPtrf", "MW, Transformer Losses, Active Power"
      c_LossQtrf = "c:LossQtrf", "Mvar, Transformer Losses, Reactive Power"
      c_LossPldtrf = "c:LossPldtrf", "MW, Transformer Losses, Active Power (load)"
      c_LossQldtrf = "c:LossQldtrf", "Mvar, Transformer Losses, Reactive Power (load)"
      c_LossPnldtrf = "c:LossPnldtrf", "MW, Transformer Losses, Active Power (no load)"
      c_LossQnldtrf = "c:LossQnldtrf", "Mvar, Transformer Losses, Reactive Power (no load)"
      c_TotLoadP = "c:TotLoadP", "MW, Total Load, Active Power"
      c_TotLoadQ = "c:TotLoadQ", "Mvar, Total Load, Reactive Power"
      c_TotLoadS = "c:TotLoadS", "MVA, Total Load, Apparent Power"
      c_TotLoadcos = "c:TotLoadcos", "Total Load, Power Factor"
      c_TotP = "c:TotP", "MW, Total Demand"
      c_UnsupP = "c:UnsupP", "MW, Unsupplied Demand"
      c_SupP = "c:SupP", "MW, Supplied Demand"
      c_ExportP = "c:ExportP", "MW, Export, Active Power"
      c_ExportQ = "c:ExportQ", "Mvar, Export, Reactive Power"
      c_ImportP = "c:ImportP", "MW, Import, Active Power"
      c_ImportQ = "c:ImportQ", "Mvar, Import, Reactive Power"
      c_cst_disp = "c:cst_disp", "USD/h, Total Production Costs"
      c_RUC_Lead = "c:RUC_Lead", "Mvar, Used Reactive Capacity Lead"
      c_RUC_Lag = "c:RUC_Lag", "Mvar, Used Reactive Capacity Lag"
      c_RSC_Lead = "c:RSC_Lead", "Mvar, Spare Reactive Capacity Lead"
      c_RSC_Lag = "c:RSC_Lag", "Mvar, Spare Reactive Capacity Lag"
      c_Q_shgain = "c:Q_shgain", "Mvar, Reactive Power Shunt Gain"
      c_ConnMvar_max = "c:ConnMvar_max", "Mvar, Connected MVAR Capacity (max)"
      c_ConnMvar_min = "c:ConnMvar_min", "Mvar, Connected MVAR Capacity (min)"
      c_Maxloading = "c:Maxloading", "%, Maximum Loading"
      c_uphtmin = "c:uphtmin", "p.u., Minimum voltage of all terminals"
      c_uphtmax = "c:uphtmax", "p.u., Maximum voltage of all terminals"
      e_iInterChg = "e:iInterChg", "Consider Interchange Schedule"
      e_InterPset = "e:InterPset", "MW, Scheduled Active Power Interchange"

    class ElmZpu(Enum):
      _init_ = 'value __doc__'

      m_u0_bus1 = "m:u0:bus1", "p.u., Zero-Sequence-Voltage, Magnitude"
      m_u0_bus2 = "m:u0:bus2", "p.u., Zero-Sequence-Voltage, Magnitude"
      m_phiu0_bus1 = "m:phiu0:bus1", "deg, Zero-Sequence-Voltage, Angle"
      m_phiu0_bus2 = "m:phiu0:bus2", "deg, Zero-Sequence-Voltage, Angle"
      m_u1r_bus1 = "m:u1r:bus1", "p.u., Positive-Sequence-Voltage, Real Part"
      m_u1r_bus2 = "m:u1r:bus2", "p.u., Positive-Sequence-Voltage, Real Part"
      m_u1i_bus1 = "m:u1i:bus1", "p.u., Positive-Sequence-Voltage, Imaginary Part"
      m_u1i_bus2 = "m:u1i:bus2", "p.u., Positive-Sequence-Voltage, Imaginary Part"
      m_u1_bus1 = "m:u1:bus1", "p.u., Positive-Sequence-Voltage, Magnitude"
      m_u1_bus2 = "m:u1:bus2", "p.u., Positive-Sequence-Voltage, Magnitude"
      m_U1_bus1 = "m:U1:bus1", "kV, Line-Ground Positive-Sequence-Voltage, Magnitude"
      m_U1_bus2 = "m:U1:bus2", "kV, Line-Ground Positive-Sequence-Voltage, Magnitude"
      m_U1l_bus1 = "m:U1l:bus1", "kV, Line-Line Positive-Sequence-Voltage, Magnitude"
      m_U1l_bus2 = "m:U1l:bus2", "kV, Line-Line Positive-Sequence-Voltage, Magnitude"
      m_phiu1_bus1 = "m:phiu1:bus1", "deg, Positive-Sequence-Voltage, Angle"
      m_phiu1_bus2 = "m:phiu1:bus2", "deg, Positive-Sequence-Voltage, Angle"
      m_u2_bus1 = "m:u2:bus1", "p.u., Negative-Sequence-Voltage, Magnitude"
      m_u2_bus2 = "m:u2:bus2", "p.u., Negative-Sequence-Voltage, Magnitude"
      m_phiu2_bus1 = "m:phiu2:bus1", "deg, Negative-Sequence-Voltage, Angle"
      m_phiu2_bus2 = "m:phiu2:bus2", "deg, Negative-Sequence-Voltage, Angle"
      m_inet_bus1_A = "m:inet:bus1:A", "p.u., Phase Current, Magnitude, referred to network"
      m_inet_bus1_B = "m:inet:bus1:B", "p.u., Phase Current, Magnitude, referred to network"
      m_inet_bus1_C = "m:inet:bus1:C", "p.u., Phase Current, Magnitude, referred to network"
      m_inet_bus2_A = "m:inet:bus2:A", "p.u., Phase Current, Magnitude, referred to network"
      m_inet_bus2_B = "m:inet:bus2:B", "p.u., Phase Current, Magnitude, referred to network"
      m_inet_bus2_C = "m:inet:bus2:C", "p.u., Phase Current, Magnitude, referred to network"
      m_I_bus1_A = "m:I:bus1:A", "kA, Phase Current, Magnitude"
      m_I_bus1_B = "m:I:bus1:B", "kA, Phase Current, Magnitude"
      m_I_bus1_C = "m:I:bus1:C", "kA, Phase Current, Magnitude"
      m_I_bus2_A = "m:I:bus2:A", "kA, Phase Current, Magnitude"
      m_I_bus2_B = "m:I:bus2:B", "kA, Phase Current, Magnitude"
      m_I_bus2_C = "m:I:bus2:C", "kA, Phase Current, Magnitude"
      m_phii_bus1_A = "m:phii:bus1:A", "deg, Phase Current, Angle"
      m_phii_bus1_B = "m:phii:bus1:B", "deg, Phase Current, Angle"
      m_phii_bus1_C = "m:phii:bus1:C", "deg, Phase Current, Angle"
      m_phii_bus2_A = "m:phii:bus2:A", "deg, Phase Current, Angle"
      m_phii_bus2_B = "m:phii:bus2:B", "deg, Phase Current, Angle"
      m_phii_bus2_C = "m:phii:bus2:C", "deg, Phase Current, Angle"
      m_i0r_bus1 = "m:i0r:bus1", "p.u., Zero-Sequence Current, Real Part"
      m_i0r_bus2 = "m:i0r:bus2", "p.u., Zero-Sequence Current, Real Part"
      m_i0i_bus1 = "m:i0i:bus1", "p.u., Zero-Sequence Current, Imaginary Part"
      m_i0i_bus2 = "m:i0i:bus2", "p.u., Zero-Sequence Current, Imaginary Part"
      m_i0_bus1 = "m:i0:bus1", "p.u., Zero-Sequence Current, Magnitude"
      m_i0_bus2 = "m:i0:bus2", "p.u., Zero-Sequence Current, Magnitude"
      m_I0_bus1 = "m:I0:bus1", "kA, Zero-Sequence Current, Magnitude"
      m_I0_bus2 = "m:I0:bus2", "kA, Zero-Sequence Current, Magnitude"
      m_I0x3_bus1 = "m:I0x3:bus1", "kA, 3*I0"
      m_I0x3_bus2 = "m:I0x3:bus2", "kA, 3*I0"
      m_phii0_bus1 = "m:phii0:bus1", "deg, Zero-Sequence Current, Angle"
      m_phii0_bus2 = "m:phii0:bus2", "deg, Zero-Sequence Current, Angle"
      m_i1r_bus1 = "m:i1r:bus1", "p.u., Positive-Sequence Current, Real Part"
      m_i1r_bus2 = "m:i1r:bus2", "p.u., Positive-Sequence Current, Real Part"
      m_i1i_bus1 = "m:i1i:bus1", "p.u., Positive-Sequence Current, Imaginary Part"
      m_i1i_bus2 = "m:i1i:bus2", "p.u., Positive-Sequence Current, Imaginary Part"
      m_i1_bus1 = "m:i1:bus1", "p.u., Positive-Sequence Current, Magnitude"
      m_i1_bus2 = "m:i1:bus2", "p.u., Positive-Sequence Current, Magnitude"
      m_I1_bus1 = "m:I1:bus1", "kA, Positive-Sequence Current, Magnitude"
      m_I1_bus2 = "m:I1:bus2", "kA, Positive-Sequence Current, Magnitude"
      m_phii1_bus1 = "m:phii1:bus1", "deg, Positive-Sequence Current, Angle"
      m_phii1_bus2 = "m:phii1:bus2", "deg, Positive-Sequence Current, Angle"
      m_i2r_bus1 = "m:i2r:bus1", "p.u., Negative-Sequence Current, Real Part"
      m_i2r_bus2 = "m:i2r:bus2", "p.u., Negative-Sequence Current, Real Part"
      m_i2i_bus1 = "m:i2i:bus1", "p.u., Negative-Sequence Current, Imaginary Part"
      m_i2i_bus2 = "m:i2i:bus2", "p.u., Negative-Sequence Current, Imaginary Part"
      m_i2_bus1 = "m:i2:bus1", "p.u., Negative-Sequence Current, Magnitude"
      m_i2_bus2 = "m:i2:bus2", "p.u., Negative-Sequence Current, Magnitude"
      m_I2_bus1 = "m:I2:bus1", "kA, Negative-Sequence Current, Magnitude"
      m_I2_bus2 = "m:I2:bus2", "kA, Negative-Sequence Current, Magnitude"
      m_phii2_bus1 = "m:phii2:bus1", "deg, Negative-Sequence Current, Angle"
      m_phii2_bus2 = "m:phii2:bus2", "deg, Negative-Sequence Current, Angle"
      m_ubfacI_bus1 = "m:ubfacI:bus1", "%, Current Unbalance Factor"
      m_ubfacI_bus2 = "m:ubfacI:bus2", "%, Current Unbalance Factor"
      m_ubfacS_bus1 = "m:ubfacS:bus1", "%, Power Unbalance Factor"
      m_ubfacS_bus2 = "m:ubfacS:bus2", "%, Power Unbalance Factor"
      m_P_bus1_A = "m:P:bus1:A", "MW, Active Power"
      m_P_bus1_B = "m:P:bus1:B", "MW, Active Power"
      m_P_bus1_C = "m:P:bus1:C", "MW, Active Power"
      m_P_bus2_A = "m:P:bus2:A", "MW, Active Power"
      m_P_bus2_B = "m:P:bus2:B", "MW, Active Power"
      m_P_bus2_C = "m:P:bus2:C", "MW, Active Power"
      m_Q_bus1_A = "m:Q:bus1:A", "Mvar, Reactive Power"
      m_Q_bus1_B = "m:Q:bus1:B", "Mvar, Reactive Power"
      m_Q_bus1_C = "m:Q:bus1:C", "Mvar, Reactive Power"
      m_Q_bus2_A = "m:Q:bus2:A", "Mvar, Reactive Power"
      m_Q_bus2_B = "m:Q:bus2:B", "Mvar, Reactive Power"
      m_Q_bus2_C = "m:Q:bus2:C", "Mvar, Reactive Power"
      m_S_bus1_A = "m:S:bus1:A", "MVA, Apparent Power"
      m_S_bus1_B = "m:S:bus1:B", "MVA, Apparent Power"
      m_S_bus1_C = "m:S:bus1:C", "MVA, Apparent Power"
      m_S_bus2_A = "m:S:bus2:A", "MVA, Apparent Power"
      m_S_bus2_B = "m:S:bus2:B", "MVA, Apparent Power"
      m_S_bus2_C = "m:S:bus2:C", "MVA, Apparent Power"
      m_cosphi_bus1_A = "m:cosphi:bus1:A", "Power Factor"
      m_cosphi_bus1_B = "m:cosphi:bus1:B", "Power Factor"
      m_cosphi_bus1_C = "m:cosphi:bus1:C", "Power Factor"
      m_cosphi_bus2_A = "m:cosphi:bus2:A", "Power Factor"
      m_cosphi_bus2_B = "m:cosphi:bus2:B", "Power Factor"
      m_cosphi_bus2_C = "m:cosphi:bus2:C", "Power Factor"
      m_tanphi_bus1_A = "m:tanphi:bus1:A", "tan(phi)"
      m_tanphi_bus1_B = "m:tanphi:bus1:B", "tan(phi)"
      m_tanphi_bus1_C = "m:tanphi:bus1:C", "tan(phi)"
      m_tanphi_bus2_A = "m:tanphi:bus2:A", "tan(phi)"
      m_tanphi_bus2_B = "m:tanphi:bus2:B", "tan(phi)"
      m_tanphi_bus2_C = "m:tanphi:bus2:C", "tan(phi)"
      m_Psum_bus1 = "m:Psum:bus1", "MW, Total Active Power"
      m_Psum_bus2 = "m:Psum:bus2", "MW, Total Active Power"
      m_Qsum_bus1 = "m:Qsum:bus1", "Mvar, Total Reactive Power"
      m_Qsum_bus2 = "m:Qsum:bus2", "Mvar, Total Reactive Power"
      m_Ssum_bus1 = "m:Ssum:bus1", "MVA, Total Apparent Power"
      m_Ssum_bus2 = "m:Ssum:bus2", "MVA, Total Apparent Power"
      m_cosphisum_bus1 = "m:cosphisum:bus1", "Total Power Factor"
      m_cosphisum_bus2 = "m:cosphisum:bus2", "Total Power Factor"
      m_tanphisum_bus1 = "m:tanphisum:bus1", "Total tan(phi)"
      m_tanphisum_bus2 = "m:tanphisum:bus2", "Total tan(phi)"
      m_Pln_bus1_A = "m:Pln:bus1:A", "MW, Active Power (Line-Neutral)"
      m_Pln_bus1_B = "m:Pln:bus1:B", "MW, Active Power (Line-Neutral)"
      m_Pln_bus1_C = "m:Pln:bus1:C", "MW, Active Power (Line-Neutral)"
      m_Pln_bus2_A = "m:Pln:bus2:A", "MW, Active Power (Line-Neutral)"
      m_Pln_bus2_B = "m:Pln:bus2:B", "MW, Active Power (Line-Neutral)"
      m_Pln_bus2_C = "m:Pln:bus2:C", "MW, Active Power (Line-Neutral)"
      m_Qln_bus1_A = "m:Qln:bus1:A", "Mvar, Reactive Power (Line-Neutral)"
      m_Qln_bus1_B = "m:Qln:bus1:B", "Mvar, Reactive Power (Line-Neutral)"
      m_Qln_bus1_C = "m:Qln:bus1:C", "Mvar, Reactive Power (Line-Neutral)"
      m_Qln_bus2_A = "m:Qln:bus2:A", "Mvar, Reactive Power (Line-Neutral)"
      m_Qln_bus2_B = "m:Qln:bus2:B", "Mvar, Reactive Power (Line-Neutral)"
      m_Qln_bus2_C = "m:Qln:bus2:C", "Mvar, Reactive Power (Line-Neutral)"
      m_P1_bus1 = "m:P1:bus1", "MW, Positive Sequence Active Power"
      m_P1_bus2 = "m:P1:bus2", "MW, Positive Sequence Active Power"
      m_Q1_bus1 = "m:Q1:bus1", "Mvar, Positive Sequence Reactive Power"
      m_Q1_bus2 = "m:Q1:bus2", "Mvar, Positive Sequence Reactive Power"
      m_P2_bus1 = "m:P2:bus1", "MW, Negative Sequence Active Power"
      m_P2_bus2 = "m:P2:bus2", "MW, Negative Sequence Active Power"
      m_Q2_bus1 = "m:Q2:bus1", "Mvar, Negative Sequence Reactive Power"
      m_Q2_bus2 = "m:Q2:bus2", "Mvar, Negative Sequence Reactive Power"
      m_P0_bus1 = "m:P0:bus1", "MW, Zero Sequence Active Power"
      m_P0_bus2 = "m:P0:bus2", "MW, Zero Sequence Active Power"
      m_Q0_bus1 = "m:Q0:bus1", "Mvar, Zero Sequence Reactive Power"
      m_Q0_bus2 = "m:Q0:bus2", "Mvar, Zero Sequence Reactive Power"
      m_i1P_bus1 = "m:i1P:bus1", "p.u., Positive-Sequence Active Current"
      m_i1P_bus2 = "m:i1P:bus2", "p.u., Positive-Sequence Active Current"
      m_i1Q_bus1 = "m:i1Q:bus1", "p.u., Positive-Sequence Reactive Current"
      m_i1Q_bus2 = "m:i1Q:bus2", "p.u., Positive-Sequence Reactive Current"
      m_I1P_bus1 = "m:I1P:bus1", "kA, Positive-Sequence Active Current"
      m_I1P_bus2 = "m:I1P:bus2", "kA, Positive-Sequence Active Current"
      m_I1Q_bus1 = "m:I1Q:bus1", "kA, Positive-Sequence Reactive Current"
      m_I1Q_bus2 = "m:I1Q:bus2", "kA, Positive-Sequence Reactive Current"
      m_i2P_bus1 = "m:i2P:bus1", "p.u., Negative-Sequence Active Current"
      m_i2P_bus2 = "m:i2P:bus2", "p.u., Negative-Sequence Active Current"
      m_i2Q_bus1 = "m:i2Q:bus1", "p.u., Negative-Sequence Reactive Current"
      m_i2Q_bus2 = "m:i2Q:bus2", "p.u., Negative-Sequence Reactive Current"
      m_I2P_bus1 = "m:I2P:bus1", "kA, Negative-Sequence Active Current"
      m_I2P_bus2 = "m:I2P:bus2", "kA, Negative-Sequence Active Current"
      m_I2Q_bus1 = "m:I2Q:bus1", "kA, Negative-Sequence Reactive Current"
      m_I2Q_bus2 = "m:I2Q:bus2", "kA, Negative-Sequence Reactive Current"
      m_ur_bus1_A = "m:ur:bus1:A", "p.u., Phase Voltage, Real Part"
      m_ur_bus1_B = "m:ur:bus1:B", "p.u., Phase Voltage, Real Part"
      m_ur_bus1_C = "m:ur:bus1:C", "p.u., Phase Voltage, Real Part"
      m_ur_bus2_A = "m:ur:bus2:A", "p.u., Phase Voltage, Real Part"
      m_ur_bus2_B = "m:ur:bus2:B", "p.u., Phase Voltage, Real Part"
      m_ur_bus2_C = "m:ur:bus2:C", "p.u., Phase Voltage, Real Part"
      m_ui_bus1_A = "m:ui:bus1:A", "p.u., Phase Voltage, Imaginary Part"
      m_ui_bus1_B = "m:ui:bus1:B", "p.u., Phase Voltage, Imaginary Part"
      m_ui_bus1_C = "m:ui:bus1:C", "p.u., Phase Voltage, Imaginary Part"
      m_ui_bus2_A = "m:ui:bus2:A", "p.u., Phase Voltage, Imaginary Part"
      m_ui_bus2_B = "m:ui:bus2:B", "p.u., Phase Voltage, Imaginary Part"
      m_ui_bus2_C = "m:ui:bus2:C", "p.u., Phase Voltage, Imaginary Part"
      m_u_bus1_A = "m:u:bus1:A", "p.u., Phase Voltage, Magnitude"
      m_u_bus1_B = "m:u:bus1:B", "p.u., Phase Voltage, Magnitude"
      m_u_bus1_C = "m:u:bus1:C", "p.u., Phase Voltage, Magnitude"
      m_u_bus2_A = "m:u:bus2:A", "p.u., Phase Voltage, Magnitude"
      m_u_bus2_B = "m:u:bus2:B", "p.u., Phase Voltage, Magnitude"
      m_u_bus2_C = "m:u:bus2:C", "p.u., Phase Voltage, Magnitude"
      m_ir_bus1_A = "m:ir:bus1:A", "p.u., Phase Current, Real Part"
      m_ir_bus1_B = "m:ir:bus1:B", "p.u., Phase Current, Real Part"
      m_ir_bus1_C = "m:ir:bus1:C", "p.u., Phase Current, Real Part"
      m_ir_bus2_A = "m:ir:bus2:A", "p.u., Phase Current, Real Part"
      m_ir_bus2_B = "m:ir:bus2:B", "p.u., Phase Current, Real Part"
      m_ir_bus2_C = "m:ir:bus2:C", "p.u., Phase Current, Real Part"
      m_ii_bus1_A = "m:ii:bus1:A", "p.u., Phase Current, Imaginary Part"
      m_ii_bus1_B = "m:ii:bus1:B", "p.u., Phase Current, Imaginary Part"
      m_ii_bus1_C = "m:ii:bus1:C", "p.u., Phase Current, Imaginary Part"
      m_ii_bus2_A = "m:ii:bus2:A", "p.u., Phase Current, Imaginary Part"
      m_ii_bus2_B = "m:ii:bus2:B", "p.u., Phase Current, Imaginary Part"
      m_ii_bus2_C = "m:ii:bus2:C", "p.u., Phase Current, Imaginary Part"
      m_i_bus1_A = "m:i:bus1:A", "p.u., Phase Current, Magnitude"
      m_i_bus1_B = "m:i:bus1:B", "p.u., Phase Current, Magnitude"
      m_i_bus1_C = "m:i:bus1:C", "p.u., Phase Current, Magnitude"
      m_i_bus2_A = "m:i:bus2:A", "p.u., Phase Current, Magnitude"
      m_i_bus2_B = "m:i:bus2:B", "p.u., Phase Current, Magnitude"
      m_i_bus2_C = "m:i:bus2:C", "p.u., Phase Current, Magnitude"
      m_phiui_bus1_A = "m:phiui:bus1:A", "deg, Angle between Voltage and Current"
      m_phiui_bus1_B = "m:phiui:bus1:B", "deg, Angle between Voltage and Current"
      m_phiui_bus1_C = "m:phiui:bus1:C", "deg, Angle between Voltage and Current"
      m_phiui_bus2_A = "m:phiui:bus2:A", "deg, Angle between Voltage and Current"
      m_phiui_bus2_B = "m:phiui:bus2:B", "deg, Angle between Voltage and Current"
      m_phiui_bus2_C = "m:phiui:bus2:C", "deg, Angle between Voltage and Current"
      m_phiu0i0_bus1 = "m:phiu0i0:bus1", "deg, Angle between Voltage and Current in zero-sequence system"
      m_phiu0i0_bus2 = "m:phiu0i0:bus2", "deg, Angle between Voltage and Current in zero-sequence system"
      m_phiu1i1_bus1 = "m:phiu1i1:bus1", "deg, Angle between Voltage and Current in positive sequence system"
      m_phiu1i1_bus2 = "m:phiu1i1:bus2", "deg, Angle between Voltage and Current in positive sequence system"
      m_phiu2i2_bus1 = "m:phiu2i2:bus1", "deg, Angle between Voltage and Current in negative sequence system"
      m_phiu2i2_bus2 = "m:phiu2i2:bus2", "deg, Angle between Voltage and Current in negative sequence system"
      m_Tfct_bus1 = "m:Tfct:bus1", "s, Fault Clearing Time"
      m_Tfct_bus2 = "m:Tfct:bus2", "s, Fault Clearing Time"
      m_TfctPh_bus1_A = "m:TfctPh:bus1:A", "s, Fault Clearing Time"
      m_TfctPh_bus1_B = "m:TfctPh:bus1:B", "s, Fault Clearing Time"
      m_TfctPh_bus1_C = "m:TfctPh:bus1:C", "s, Fault Clearing Time"
      m_TfctPh_bus2_A = "m:TfctPh:bus2:A", "s, Fault Clearing Time"
      m_TfctPh_bus2_B = "m:TfctPh:bus2:B", "s, Fault Clearing Time"
      m_TfctPh_bus2_C = "m:TfctPh:bus2:C", "s, Fault Clearing Time"
      m_Brkload_bus1 = "m:Brkload:bus1", "%, Max. Breaker Loading"
      m_Brkload_bus2 = "m:Brkload:bus2", "%, Max. Breaker Loading"
      m_BrkloadPh_bus1_A = "m:BrkloadPh:bus1:A", "%, Breaker Loading"
      m_BrkloadPh_bus1_B = "m:BrkloadPh:bus1:B", "%, Breaker Loading"
      m_BrkloadPh_bus1_C = "m:BrkloadPh:bus1:C", "%, Breaker Loading"
      m_BrkloadPh_bus2_A = "m:BrkloadPh:bus2:A", "%, Breaker Loading"
      m_BrkloadPh_bus2_B = "m:BrkloadPh:bus2:B", "%, Breaker Loading"
      m_BrkloadPh_bus2_C = "m:BrkloadPh:bus2:C", "%, Breaker Loading"
      n_ur_bus1_A = "n:ur:bus1:A", "p.u., Line-Ground Voltage, Real Part"
      n_ur_bus1_B = "n:ur:bus1:B", "p.u., Line-Ground Voltage, Real Part"
      n_ur_bus1_C = "n:ur:bus1:C", "p.u., Line-Ground Voltage, Real Part"
      n_ur_bus2_A = "n:ur:bus2:A", "p.u., Line-Ground Voltage, Real Part"
      n_ur_bus2_B = "n:ur:bus2:B", "p.u., Line-Ground Voltage, Real Part"
      n_ur_bus2_C = "n:ur:bus2:C", "p.u., Line-Ground Voltage, Real Part"
      n_ui_bus1_A = "n:ui:bus1:A", "p.u., Line-Ground Voltage, Imaginary Part"
      n_ui_bus1_B = "n:ui:bus1:B", "p.u., Line-Ground Voltage, Imaginary Part"
      n_ui_bus1_C = "n:ui:bus1:C", "p.u., Line-Ground Voltage, Imaginary Part"
      n_ui_bus2_A = "n:ui:bus2:A", "p.u., Line-Ground Voltage, Imaginary Part"
      n_ui_bus2_B = "n:ui:bus2:B", "p.u., Line-Ground Voltage, Imaginary Part"
      n_ui_bus2_C = "n:ui:bus2:C", "p.u., Line-Ground Voltage, Imaginary Part"
      n_u_bus1_A = "n:u:bus1:A", "p.u., Line-Ground Voltage, Magnitude"
      n_u_bus1_B = "n:u:bus1:B", "p.u., Line-Ground Voltage, Magnitude"
      n_u_bus1_C = "n:u:bus1:C", "p.u., Line-Ground Voltage, Magnitude"
      n_u_bus2_A = "n:u:bus2:A", "p.u., Line-Ground Voltage, Magnitude"
      n_u_bus2_B = "n:u:bus2:B", "p.u., Line-Ground Voltage, Magnitude"
      n_u_bus2_C = "n:u:bus2:C", "p.u., Line-Ground Voltage, Magnitude"
      n_upc_bus1_A = "n:upc:bus1:A", "%, Line-Ground Voltage, Magnitude"
      n_upc_bus1_B = "n:upc:bus1:B", "%, Line-Ground Voltage, Magnitude"
      n_upc_bus1_C = "n:upc:bus1:C", "%, Line-Ground Voltage, Magnitude"
      n_upc_bus2_A = "n:upc:bus2:A", "%, Line-Ground Voltage, Magnitude"
      n_upc_bus2_B = "n:upc:bus2:B", "%, Line-Ground Voltage, Magnitude"
      n_upc_bus2_C = "n:upc:bus2:C", "%, Line-Ground Voltage, Magnitude"
      n_U_bus1_A = "n:U:bus1:A", "kV, Line-Ground Voltage, Magnitude"
      n_U_bus1_B = "n:U:bus1:B", "kV, Line-Ground Voltage, Magnitude"
      n_U_bus1_C = "n:U:bus1:C", "kV, Line-Ground Voltage, Magnitude"
      n_U_bus2_A = "n:U:bus2:A", "kV, Line-Ground Voltage, Magnitude"
      n_U_bus2_B = "n:U:bus2:B", "kV, Line-Ground Voltage, Magnitude"
      n_U_bus2_C = "n:U:bus2:C", "kV, Line-Ground Voltage, Magnitude"
      n_phiu_bus1_A = "n:phiu:bus1:A", "deg, Line-Ground Voltage, Angle"
      n_phiu_bus1_B = "n:phiu:bus1:B", "deg, Line-Ground Voltage, Angle"
      n_phiu_bus1_C = "n:phiu:bus1:C", "deg, Line-Ground Voltage, Angle"
      n_phiu_bus2_A = "n:phiu:bus2:A", "deg, Line-Ground Voltage, Angle"
      n_phiu_bus2_B = "n:phiu:bus2:B", "deg, Line-Ground Voltage, Angle"
      n_phiu_bus2_C = "n:phiu:bus2:C", "deg, Line-Ground Voltage, Angle"
      n_phiurel_bus1 = "n:phiurel:bus1", "deg, Voltage, Relative Angle"
      n_phiurel_bus2 = "n:phiurel:bus2", "deg, Voltage, Relative Angle"
      n_ul_bus1_A = "n:ul:bus1:A", "p.u., Line-Line Voltage, Magnitude"
      n_ul_bus1_B = "n:ul:bus1:B", "p.u., Line-Line Voltage, Magnitude"
      n_ul_bus1_C = "n:ul:bus1:C", "p.u., Line-Line Voltage, Magnitude"
      n_ul_bus2_A = "n:ul:bus2:A", "p.u., Line-Line Voltage, Magnitude"
      n_ul_bus2_B = "n:ul:bus2:B", "p.u., Line-Line Voltage, Magnitude"
      n_ul_bus2_C = "n:ul:bus2:C", "p.u., Line-Line Voltage, Magnitude"
      n_ulpc_bus1_A = "n:ulpc:bus1:A", "%, Line-Line Voltage, Magnitude"
      n_ulpc_bus1_B = "n:ulpc:bus1:B", "%, Line-Line Voltage, Magnitude"
      n_ulpc_bus1_C = "n:ulpc:bus1:C", "%, Line-Line Voltage, Magnitude"
      n_ulpc_bus2_A = "n:ulpc:bus2:A", "%, Line-Line Voltage, Magnitude"
      n_ulpc_bus2_B = "n:ulpc:bus2:B", "%, Line-Line Voltage, Magnitude"
      n_ulpc_bus2_C = "n:ulpc:bus2:C", "%, Line-Line Voltage, Magnitude"
      n_Ul_bus1_A = "n:Ul:bus1:A", "kV, Line-Line Voltage, Magnitude"
      n_Ul_bus1_B = "n:Ul:bus1:B", "kV, Line-Line Voltage, Magnitude"
      n_Ul_bus1_C = "n:Ul:bus1:C", "kV, Line-Line Voltage, Magnitude"
      n_Ul_bus2_A = "n:Ul:bus2:A", "kV, Line-Line Voltage, Magnitude"
      n_Ul_bus2_B = "n:Ul:bus2:B", "kV, Line-Line Voltage, Magnitude"
      n_Ul_bus2_C = "n:Ul:bus2:C", "kV, Line-Line Voltage, Magnitude"
      n_phiul_bus1_A = "n:phiul:bus1:A", "deg, Line-Line Voltage, Angle"
      n_phiul_bus1_B = "n:phiul:bus1:B", "deg, Line-Line Voltage, Angle"
      n_phiul_bus1_C = "n:phiul:bus1:C", "deg, Line-Line Voltage, Angle"
      n_phiul_bus2_A = "n:phiul:bus2:A", "deg, Line-Line Voltage, Angle"
      n_phiul_bus2_B = "n:phiul:bus2:B", "deg, Line-Line Voltage, Angle"
      n_phiul_bus2_C = "n:phiul:bus2:C", "deg, Line-Line Voltage, Angle"
      n_u0_bus1 = "n:u0:bus1", "p.u., Zero-Sequence Voltage, Magnitude"
      n_u0_bus2 = "n:u0:bus2", "p.u., Zero-Sequence Voltage, Magnitude"
      n_U0_bus1 = "n:U0:bus1", "kV, Zero-Sequence Voltage, Magnitude"
      n_U0_bus2 = "n:U0:bus2", "kV, Zero-Sequence Voltage, Magnitude"
      n_U0x3_bus1 = "n:U0x3:bus1", "kV, 3*U0"
      n_U0x3_bus2 = "n:U0x3:bus2", "kV, 3*U0"
      n_phiu0_bus1 = "n:phiu0:bus1", "deg, Zero-Sequence Voltage, Angle"
      n_phiu0_bus2 = "n:phiu0:bus2", "deg, Zero-Sequence Voltage, Angle"
      n_u1_bus1 = "n:u1:bus1", "p.u., Positive-Sequence Voltage, Magnitude"
      n_u1_bus2 = "n:u1:bus2", "p.u., Positive-Sequence Voltage, Magnitude"
      n_u1pc_bus1 = "n:u1pc:bus1", "%, Positive-Sequence Voltage, Magnitude"
      n_u1pc_bus2 = "n:u1pc:bus2", "%, Positive-Sequence Voltage, Magnitude"
      n_u1r_bus1 = "n:u1r:bus1", "p.u., Positive-Sequence Voltage, Real Part"
      n_u1r_bus2 = "n:u1r:bus2", "p.u., Positive-Sequence Voltage, Real Part"
      n_u1i_bus1 = "n:u1i:bus1", "p.u., Positive-Sequence Voltage, Imaginary Part"
      n_u1i_bus2 = "n:u1i:bus2", "p.u., Positive-Sequence Voltage, Imaginary Part"
      n_U1_bus1 = "n:U1:bus1", "kV, Line-Ground Positive-Sequence Voltage, Magnitude"
      n_U1_bus2 = "n:U1:bus2", "kV, Line-Ground Positive-Sequence Voltage, Magnitude"
      n_phiu1_bus1 = "n:phiu1:bus1", "deg, Positive-Sequence Voltage, Angle"
      n_phiu1_bus2 = "n:phiu1:bus2", "deg, Positive-Sequence Voltage, Angle"
      n_u2_bus1 = "n:u2:bus1", "p.u., Negative-Sequence Voltage, Magnitude"
      n_u2_bus2 = "n:u2:bus2", "p.u., Negative-Sequence Voltage, Magnitude"
      n_U2_bus1 = "n:U2:bus1", "kV, Line-Ground Negative-Sequence Voltage, Magnitude"
      n_U2_bus2 = "n:U2:bus2", "kV, Line-Ground Negative-Sequence Voltage, Magnitude"
      n_phiu2_bus1 = "n:phiu2:bus1", "deg, Negative-Sequence Voltage, Angle"
      n_phiu2_bus2 = "n:phiu2:bus2", "deg, Negative-Sequence Voltage, Angle"
      n_U1l_bus1 = "n:U1l:bus1", "kV, Line-Line Positive-Sequence Voltage, Magnitude"
      n_U1l_bus2 = "n:U1l:bus2", "kV, Line-Line Positive-Sequence Voltage, Magnitude"
      n_U2l_bus1 = "n:U2l:bus1", "kV, Line-Line Negative-Sequence Voltage, Magnitude"
      n_U2l_bus2 = "n:U2l:bus2", "kV, Line-Line Negative-Sequence Voltage, Magnitude"
      n_uln_bus1_A = "n:uln:bus1:A", "p.u., Line-Neutral Voltage, Magnitude"
      n_uln_bus1_B = "n:uln:bus1:B", "p.u., Line-Neutral Voltage, Magnitude"
      n_uln_bus1_C = "n:uln:bus1:C", "p.u., Line-Neutral Voltage, Magnitude"
      n_uln_bus2_A = "n:uln:bus2:A", "p.u., Line-Neutral Voltage, Magnitude"
      n_uln_bus2_B = "n:uln:bus2:B", "p.u., Line-Neutral Voltage, Magnitude"
      n_uln_bus2_C = "n:uln:bus2:C", "p.u., Line-Neutral Voltage, Magnitude"
      n_Uln_bus1_A = "n:Uln:bus1:A", "kV, Line-Neutral Voltage, Magnitude"
      n_Uln_bus1_B = "n:Uln:bus1:B", "kV, Line-Neutral Voltage, Magnitude"
      n_Uln_bus1_C = "n:Uln:bus1:C", "kV, Line-Neutral Voltage, Magnitude"
      n_Uln_bus2_A = "n:Uln:bus2:A", "kV, Line-Neutral Voltage, Magnitude"
      n_Uln_bus2_B = "n:Uln:bus2:B", "kV, Line-Neutral Voltage, Magnitude"
      n_Uln_bus2_C = "n:Uln:bus2:C", "kV, Line-Neutral Voltage, Magnitude"
      n_phiuln_bus1_A = "n:phiuln:bus1:A", "deg, Line-Neutral Voltage, Angle"
      n_phiuln_bus1_B = "n:phiuln:bus1:B", "deg, Line-Neutral Voltage, Angle"
      n_phiuln_bus1_C = "n:phiuln:bus1:C", "deg, Line-Neutral Voltage, Angle"
      n_phiuln_bus2_A = "n:phiuln:bus2:A", "deg, Line-Neutral Voltage, Angle"
      n_phiuln_bus2_B = "n:phiuln:bus2:B", "deg, Line-Neutral Voltage, Angle"
      n_phiuln_bus2_C = "n:phiuln:bus2:C", "deg, Line-Neutral Voltage, Angle"
      n_un_bus1 = "n:un:bus1", "p.u., Neutral-Ground Voltage, Magnitude"
      n_un_bus2 = "n:un:bus2", "p.u., Neutral-Ground Voltage, Magnitude"
      n_Un_bus1 = "n:Un:bus1", "kV, Neutral-Ground Voltage, Magnitude"
      n_Un_bus2 = "n:Un:bus2", "kV, Neutral-Ground Voltage, Magnitude"
      n_phiun_bus1 = "n:phiun:bus1", "deg, Neutral-Ground Voltage, Angle"
      n_phiun_bus2 = "n:phiun:bus2", "deg, Neutral-Ground Voltage, Angle"
      n_upht_bus1_A = "n:upht:bus1:A", "p.u., Phase Technology dependent Voltage, Magnitude"
      n_upht_bus1_B = "n:upht:bus1:B", "p.u., Phase Technology dependent Voltage, Magnitude"
      n_upht_bus1_C = "n:upht:bus1:C", "p.u., Phase Technology dependent Voltage, Magnitude"
      n_upht_bus2_A = "n:upht:bus2:A", "p.u., Phase Technology dependent Voltage, Magnitude"
      n_upht_bus2_B = "n:upht:bus2:B", "p.u., Phase Technology dependent Voltage, Magnitude"
      n_upht_bus2_C = "n:upht:bus2:C", "p.u., Phase Technology dependent Voltage, Magnitude"
      n_Upht_bus1_A = "n:Upht:bus1:A", "kV, Phase Technology dependent Voltage, Magnitude"
      n_Upht_bus1_B = "n:Upht:bus1:B", "kV, Phase Technology dependent Voltage, Magnitude"
      n_Upht_bus1_C = "n:Upht:bus1:C", "kV, Phase Technology dependent Voltage, Magnitude"
      n_Upht_bus2_A = "n:Upht:bus2:A", "kV, Phase Technology dependent Voltage, Magnitude"
      n_Upht_bus2_B = "n:Upht:bus2:B", "kV, Phase Technology dependent Voltage, Magnitude"
      n_Upht_bus2_C = "n:Upht:bus2:C", "kV, Phase Technology dependent Voltage, Magnitude"
      n_um_bus1 = "n:um:bus1", "p.u., Average-Voltage, Magnitude"
      n_um_bus2 = "n:um:bus2", "p.u., Average-Voltage, Magnitude"
      n_Um_bus1 = "n:Um:bus1", "kV, Average-Voltage, Magnitude"
      n_Um_bus2 = "n:Um:bus2", "kV, Average-Voltage, Magnitude"
      n_uphtmin_bus1 = "n:uphtmin:bus1", "p.u., Minimum of Phase Technology dependent Voltage, Magnitude"
      n_uphtmin_bus2 = "n:uphtmin:bus2", "p.u., Minimum of Phase Technology dependent Voltage, Magnitude"
      n_uphtmax_bus1 = "n:uphtmax:bus1", "p.u., Maximum of Phase Technology dependent Voltage, Magnitude"
      n_uphtmax_bus2 = "n:uphtmax:bus2", "p.u., Maximum of Phase Technology dependent Voltage, Magnitude"
      n_Pgen_bus1 = "n:Pgen:bus1", "MW, Generation, Active Power"
      n_Pgen_bus2 = "n:Pgen:bus2", "MW, Generation, Active Power"
      n_Qgen_bus1 = "n:Qgen:bus1", "Mvar, Generation, Reactive Power"
      n_Qgen_bus2 = "n:Qgen:bus2", "Mvar, Generation, Reactive Power"
      n_Pmot_bus1 = "n:Pmot:bus1", "MW, Motor Load, Active Power"
      n_Pmot_bus2 = "n:Pmot:bus2", "MW, Motor Load, Active Power"
      n_Qmot_bus1 = "n:Qmot:bus1", "Mvar, Motor Load, Reactive Power"
      n_Qmot_bus2 = "n:Qmot:bus2", "Mvar, Motor Load, Reactive Power"
      n_Pload_bus1 = "n:Pload:bus1", "MW, General Load, Active Power"
      n_Pload_bus2 = "n:Pload:bus2", "MW, General Load, Active Power"
      n_Qload_bus1 = "n:Qload:bus1", "Mvar, General Load, Reactive Power"
      n_Qload_bus2 = "n:Qload:bus2", "Mvar, General Load, Reactive Power"
      n_Pcomp_bus1 = "n:Pcomp:bus1", "MW, Compensation (Losses)"
      n_Pcomp_bus2 = "n:Pcomp:bus2", "MW, Compensation (Losses)"
      n_Qcomp_bus1 = "n:Qcomp:bus1", "Mvar, Compensation"
      n_Qcomp_bus2 = "n:Qcomp:bus2", "Mvar, Compensation"
      n_Pnet_bus1 = "n:Pnet:bus1", "MW, External Networks, Active Power"
      n_Pnet_bus2 = "n:Pnet:bus2", "MW, External Networks, Active Power"
      n_Qnet_bus1 = "n:Qnet:bus1", "Mvar, External Networks, Reactive Power"
      n_Qnet_bus2 = "n:Qnet:bus2", "Mvar, External Networks, Reactive Power"
      n_Pflow_bus1 = "n:Pflow:bus1", "MW, Power Flow, Active Power"
      n_Pflow_bus2 = "n:Pflow:bus2", "MW, Power Flow, Active Power"
      n_Qflow_bus1 = "n:Qflow:bus1", "Mvar, Power Flow, Reactive Power"
      n_Qflow_bus2 = "n:Qflow:bus2", "Mvar, Power Flow, Reactive Power"
      n_Pout_bus1 = "n:Pout:bus1", "MW, Outgoing Flow, Active Power"
      n_Pout_bus2 = "n:Pout:bus2", "MW, Outgoing Flow, Active Power"
      n_Qout_bus1 = "n:Qout:bus1", "Mvar, Outgoing Power, Reactive Power"
      n_Qout_bus2 = "n:Qout:bus2", "Mvar, Outgoing Power, Reactive Power"
      n_Sout_bus1 = "n:Sout:bus1", "MVA, Outgoing Power, Apparent Power"
      n_Sout_bus2 = "n:Sout:bus2", "MVA, Outgoing Power, Apparent Power"
      n_cosphiout_bus1 = "n:cosphiout:bus1", "Outgoing Power, Power Factor"
      n_cosphiout_bus2 = "n:cosphiout:bus2", "Outgoing Power, Power Factor"
      n_Pbalance_bus1 = "n:Pbalance:bus1", "MW, Active Power Balance (=0)"
      n_Pbalance_bus2 = "n:Pbalance:bus2", "MW, Active Power Balance (=0)"
      n_Qbalance_bus1 = "n:Qbalance:bus1", "Mvar, Reactive Power Balance (=0)"
      n_Qbalance_bus2 = "n:Qbalance:bus2", "Mvar, Reactive Power Balance (=0)"
      n_ubfac_bus1 = "n:ubfac:bus1", "%, Unbalance Factor"
      n_ubfac_bus2 = "n:ubfac:bus2", "%, Unbalance Factor"
      n_LossPdown_bus1 = "n:LossPdown:bus1", "MW, Losses, downstream"
      n_LossPdown_bus2 = "n:LossPdown:bus2", "MW, Losses, downstream"
      n_LossQdown_bus1 = "n:LossQdown:bus1", "Mvar, Losses, downstream (Reactive Power)"
      n_LossQdown_bus2 = "n:LossQdown:bus2", "Mvar, Losses, downstream (Reactive Power)"
      n_LossPdownload_bus1 = "n:LossPdownload:bus1", "MW, Load losses, downstream"
      n_LossPdownload_bus2 = "n:LossPdownload:bus2", "MW, Load losses, downstream"
      n_LossQdownload_bus1 = "n:LossQdownload:bus1", "Mvar, Load losses, downstream"
      n_LossQdownload_bus2 = "n:LossQdownload:bus2", "Mvar, Load losses, downstream"
      n_LossPdownnoload_bus1 = "n:LossPdownnoload:bus1", "MW, No load losses, downstream"
      n_LossPdownnoload_bus2 = "n:LossPdownnoload:bus2", "MW, No load losses, downstream"
      n_LossQdownnoload_bus1 = "n:LossQdownnoload:bus1", "Mvar, No load losses, downstream"
      n_LossQdownnoload_bus2 = "n:LossQdownnoload:bus2", "Mvar, No load losses, downstream"
      c_z1_ij_r = "c:z1_ij:r", "p.u., Positive-Sequence Impedance i-j, Real Part"
      c_z1_ij_i = "c:z1_ij:i", "p.u., Positive-Sequence Impedance i-j, Imaginary Part"
      c_z1_ij_phi = "c:z1_ij:phi", "deg, Positive-Sequence Impedance i-j, Angle"
      c_z1_ij_mag = "c:z1_ij:mag", "p.u., Positive-Sequence Impedance i-j, Magnitude"
      c_z1_ij_phirad = "c:z1_ij:phirad", "rad, Positive-Sequence Impedance i-j, Angle"
      c_z1_ji_r = "c:z1_ji:r", "p.u., Positive-Sequence Impedance j-i, Real Part"
      c_z1_ji_i = "c:z1_ji:i", "p.u., Positive-Sequence Impedance j-i, Imaginary Part"
      c_z1_ji_phi = "c:z1_ji:phi", "deg, Positive-Sequence Impedance j-i, Angle"
      c_z1_ji_mag = "c:z1_ji:mag", "p.u., Positive-Sequence Impedance j-i, Magnitude"
      c_z1_ji_phirad = "c:z1_ji:phirad", "rad, Positive-Sequence Impedance j-i, Angle"
      c_z0_ij_r = "c:z0_ij:r", "p.u., Zero-Sequence Impedance i-j, Real Part"
      c_z0_ij_i = "c:z0_ij:i", "p.u., Zero-Sequence Impedance i-j, Imaginary Part"
      c_z0_ij_phi = "c:z0_ij:phi", "deg, Zero-Sequence Impedance i-j, Angle"
      c_z0_ij_mag = "c:z0_ij:mag", "p.u., Zero-Sequence Impedance i-j, Magnitude"
      c_z0_ij_phirad = "c:z0_ij:phirad", "rad, Zero-Sequence Impedance i-j, Angle"
      c_z0_ji_r = "c:z0_ji:r", "p.u., Zero-Sequence Impedance j-i, Real Part"
      c_z0_ji_i = "c:z0_ji:i", "p.u., Zero-Sequence Impedance j-i, Imaginary Part"
      c_z0_ji_phi = "c:z0_ji:phi", "deg, Zero-Sequence Impedance j-i, Angle"
      c_z0_ji_mag = "c:z0_ji:mag", "p.u., Zero-Sequence Impedance j-i, Magnitude"
      c_z0_ji_phirad = "c:z0_ji:phirad", "rad, Zero-Sequence Impedance j-i, Angle"
      c_z2_ij_r = "c:z2_ij:r", "p.u., Negative-Sequence Impedance i-j, Real Part"
      c_z2_ij_i = "c:z2_ij:i", "p.u., Negative-Sequence Impedance i-j, Imaginary Part"
      c_z2_ij_phi = "c:z2_ij:phi", "deg, Negative-Sequence Impedance i-j, Angle"
      c_z2_ij_mag = "c:z2_ij:mag", "p.u., Negative-Sequence Impedance i-j, Magnitude"
      c_z2_ij_phirad = "c:z2_ij:phirad", "rad, Negative-Sequence Impedance i-j, Angle"
      c_z2_ji_r = "c:z2_ji:r", "p.u., Negative-Sequence Impedance j-i, Real Part"
      c_z2_ji_i = "c:z2_ji:i", "p.u., Negative-Sequence Impedance j-i, Imaginary Part"
      c_z2_ji_phi = "c:z2_ji:phi", "deg, Negative-Sequence Impedance j-i, Angle"
      c_z2_ji_mag = "c:z2_ji:mag", "p.u., Negative-Sequence Impedance j-i, Magnitude"
      c_z2_ji_phirad = "c:z2_ji:phirad", "rad, Negative-Sequence Impedance j-i, Angle"
      c_y1_ij_r = "c:y1_ij:r", "p.u., Positive-Sequence Admittance i-j, Real Part"
      c_y1_ij_i = "c:y1_ij:i", "p.u., Positive-Sequence Admittance i-j, Imaginary Part"
      c_y1_ij_phi = "c:y1_ij:phi", "deg, Positive-Sequence Admittance i-j, Angle"
      c_y1_ij_mag = "c:y1_ij:mag", "p.u., Positive-Sequence Admittance i-j, Magnitude"
      c_y1_ij_phirad = "c:y1_ij:phirad", "rad, Positive-Sequence Admittance i-j, Angle"
      c_y1_ji_r = "c:y1_ji:r", "p.u., Positive-Sequence Admittance j-i, Real Part"
      c_y1_ji_i = "c:y1_ji:i", "p.u., Positive-Sequence Admittance j-i, Imaginary Part"
      c_y1_ji_phi = "c:y1_ji:phi", "deg, Positive-Sequence Admittance j-i, Angle"
      c_y1_ji_mag = "c:y1_ji:mag", "p.u., Positive-Sequence Admittance j-i, Magnitude"
      c_y1_ji_phirad = "c:y1_ji:phirad", "rad, Positive-Sequence Admittance j-i, Angle"
      c_y0_ij_r = "c:y0_ij:r", "p.u., Zero-Sequence Admittance i-j, Real Part"
      c_y0_ij_i = "c:y0_ij:i", "p.u., Zero-Sequence Admittance i-j, Imaginary Part"
      c_y0_ij_phi = "c:y0_ij:phi", "deg, Zero-Sequence Admittance i-j, Angle"
      c_y0_ij_mag = "c:y0_ij:mag", "p.u., Zero-Sequence Admittance i-j, Magnitude"
      c_y0_ij_phirad = "c:y0_ij:phirad", "rad, Zero-Sequence Admittance i-j, Angle"
      c_y0_ji_r = "c:y0_ji:r", "p.u., Zero-Sequence Admittance j-i, Real Part"
      c_y0_ji_i = "c:y0_ji:i", "p.u., Zero-Sequence Admittance j-i, Imaginary Part"
      c_y0_ji_phi = "c:y0_ji:phi", "deg, Zero-Sequence Admittance j-i, Angle"
      c_y0_ji_mag = "c:y0_ji:mag", "p.u., Zero-Sequence Admittance j-i, Magnitude"
      c_y0_ji_phirad = "c:y0_ji:phirad", "rad, Zero-Sequence Admittance j-i, Angle"
      c_y2_ij_r = "c:y2_ij:r", "p.u., Negative-Sequence Admittance i-j, Real Part"
      c_y2_ij_i = "c:y2_ij:i", "p.u., Negative-Sequence Admittance i-j, Imaginary Part"
      c_y2_ij_phi = "c:y2_ij:phi", "deg, Negative-Sequence Admittance i-j, Angle"
      c_y2_ij_mag = "c:y2_ij:mag", "p.u., Negative-Sequence Admittance i-j, Magnitude"
      c_y2_ij_phirad = "c:y2_ij:phirad", "rad, Negative-Sequence Admittance i-j, Angle"
      c_y2_ji_r = "c:y2_ji:r", "p.u., Negative-Sequence Admittance j-i, Real Part"
      c_y2_ji_i = "c:y2_ji:i", "p.u., Negative-Sequence Admittance j-i, Imaginary Part"
      c_y2_ji_phi = "c:y2_ji:phi", "deg, Negative-Sequence Admittance j-i, Angle"
      c_y2_ji_mag = "c:y2_ji:mag", "p.u., Negative-Sequence Admittance j-i, Magnitude"
      c_y2_ji_phirad = "c:y2_ji:phirad", "rad, Negative-Sequence Admittance j-i, Angle"
      c_uratio = "c:uratio", "Rating Factor"
      c_du_A = "c:du:A", "p.u., Voltage Drop"
      c_du_B = "c:du:B", "p.u., Voltage Drop"
      c_du_C = "c:du:C", "p.u., Voltage Drop"
      c_du_N = "c:du:N", "p.u., Voltage Drop"
      c_dupc_A = "c:dupc:A", "%, Voltage Drop"
      c_dupc_B = "c:dupc:B", "%, Voltage Drop"
      c_dupc_C = "c:dupc:C", "%, Voltage Drop"
      c_dupc_N = "c:dupc:N", "%, Voltage Drop"
      c_dphiu_A = "c:dphiu:A", "deg, Voltage Drop Angle"
      c_dphiu_B = "c:dphiu:B", "deg, Voltage Drop Angle"
      c_dphiu_C = "c:dphiu:C", "deg, Voltage Drop Angle"
      c_dphiu_N = "c:dphiu:N", "deg, Voltage Drop Angle"
      c_du1 = "c:du1", "p.u., Positive Sequence Voltage Drop"
      c_du1pc = "c:du1pc", "%, Positive Sequence Voltage Drop"
      c_dphiu1 = "c:dphiu1", "deg, Positive Sequence Voltage Drop Angle"
      c_Ploss = "c:Ploss", "MW, Losses (total)"
      c_Qloss = "c:Qloss", "Mvar, Reactive-Losses (total)"
      c_Plossld = "c:Plossld", "MW, Losses (load)"
      c_Qlossld = "c:Qlossld", "Mvar, Reactive-Losses (load)"
      c_Plossnld = "c:Plossnld", "MW, Losses (no load)"
      c_Qlossnld = "c:Qlossnld", "Mvar, Reactive-Losses (no load)"
      e_iequalz = "e:iequalz", "Use equal Impedances (zij = zji)"
      e_r_pu = "e:r_pu", "p.u., Real Part"
      e_x_pu = "e:x_pu", "p.u., Imaginary Part"
      e_r_pu_ji = "e:r_pu_ji", "p.u., Real Part"
      e_x_pu_ji = "e:x_pu_ji", "p.u., Imaginary Part"
      e_r0_pu = "e:r0_pu", "p.u., Real Part"
      e_x0_pu = "e:x0_pu", "p.u., Imaginary Part"
      e_r0_pu_ji = "e:r0_pu_ji", "p.u., Real Part"
      e_x0_pu_ji = "e:x0_pu_ji", "p.u., Imaginary Part"
      e_iz2eqz1 = "e:iz2eqz1", "Use Impedance z2 = z1"
      e_r2_pu = "e:r2_pu", "p.u., Real Part"
      e_x2_pu = "e:x2_pu", "p.u., Imaginary Part"
      e_r2_pu_ji = "e:r2_pu_ji", "p.u., Real Part"
      e_x2_pu_ji = "e:x2_pu_ji", "p.u., Imaginary Part"
      e_gi_pu = "e:gi_pu", "p.u., Conductance i"
      e_bi_pu = "e:bi_pu", "p.u., Susceptance i"
      e_gj_pu = "e:gj_pu", "p.u., Conductance j"
      e_bj_pu = "e:bj_pu", "p.u., Susceptance j"
      e_gi0_pu = "e:gi0_pu", "p.u., Conductance i"
      e_bi0_pu = "e:bi0_pu", "p.u., Susceptance i"
      e_gj0_pu = "e:gj0_pu", "p.u., Conductance j"
      e_bj0_pu = "e:bj0_pu", "p.u., Susceptance j"
      e_iy2eqy1 = "e:iy2eqy1", "Use Admittance y2 = y1"
      e_gi2_pu = "e:gi2_pu", "p.u., Conductance i"
      e_bi2_pu = "e:bi2_pu", "p.u., Susceptance i"
      e_gj2_pu = "e:gj2_pu", "p.u., Conductance j"
      e_bj2_pu = "e:bj2_pu", "p.u., Susceptance j"
      e_cpFeed = "e:cpFeed", "Feeder"
      e_ciDist = "e:ciDist", "Distance from infeed in number of buses"
      e_ciLater = "e:ciLater", "Lateral Index"
      e_ciDistRoot = "e:ciDistRoot", "Distance from first infeed in number of buses"
      e_ciDistAll = "e:ciDistAll", "Distance from infeed in number of buses including switches"
      e_ciDistAllRoot = "e:ciDistAllRoot", "Distance from first infeed in number of buses including switches"
      r_cpFeed = "r:cpFeed", "Feeder"
      m_Ikss_bus1 = "m:Ikss:bus1", "kA, Initial Short-Circuit Current"
      m_Ikss_bus2 = "m:Ikss:bus2", "kA, Initial Short-Circuit Current"
      m_ikss_bus1 = "m:ikss:bus1", "p.u., Initial Short-Circuit Current, Magnitude"
      m_ikss_bus2 = "m:ikss:bus2", "p.u., Initial Short-Circuit Current, Magnitude"
      m_phii_bus1 = "m:phii:bus1", "deg, Phase Current, Angle"
      m_phii_bus2 = "m:phii:bus2", "deg, Phase Current, Angle"
      m_maxIkss_bus1 = "m:maxIkss:bus1", "kA, Max. Initial Short-Circuit Current"
      m_maxIkss_bus2 = "m:maxIkss:bus2", "kA, Max. Initial Short-Circuit Current"
      m_Skss_bus1 = "m:Skss:bus1", "MVA, Initial Short-Circuit Power"
      m_Skss_bus2 = "m:Skss:bus2", "MVA, Initial Short-Circuit Power"
      m_ip_bus1 = "m:ip:bus1", "kA, Peak Short-Circuit Current"
      m_ip_bus2 = "m:ip:bus2", "kA, Peak Short-Circuit Current"
      m_maxip_bus1 = "m:maxip:bus1", "kA, Max. Peak Short-Circuit Current"
      m_maxip_bus2 = "m:maxip:bus2", "kA, Max. Peak Short-Circuit Current"
      m_maxIb_bus1 = "m:maxIb:bus1", "kA, Max. Breaking Current"
      m_maxIb_bus2 = "m:maxIb:bus2", "kA, Max. Breaking Current"
      m_maxIth_bus1 = "m:maxIth:bus1", "kA, Max. Thermal equivalent Short-Circuit Current"
      m_maxIth_bus2 = "m:maxIth:bus2", "kA, Max. Thermal equivalent Short-Circuit Current"
      m_Ib_bus1 = "m:Ib:bus1", "kA, Short-Circuit Breaking Current"
      m_Ib_bus2 = "m:Ib:bus2", "kA, Short-Circuit Breaking Current"
      m_Sb_bus1 = "m:Sb:bus1", "MVA, Short-Circuit Breaking Power"
      m_Sb_bus2 = "m:Sb:bus2", "MVA, Short-Circuit Breaking Power"
      m_Ik_bus1 = "m:Ik:bus1", "kA, Steady-State Short-Circuit Current"
      m_Ik_bus2 = "m:Ik:bus2", "kA, Steady-State Short-Circuit Current"
      m_Ith_bus1 = "m:Ith:bus1", "kA, Thermal equivalent Short-Circuit Current"
      m_Ith_bus2 = "m:Ith:bus2", "kA, Thermal equivalent Short-Circuit Current"
      m_I_bus1 = "m:I:bus1", "kA, Short-Circuit Current"
      m_I_bus2 = "m:I:bus2", "kA, Short-Circuit Current"
      m_R_bus1 = "m:R:bus1", "Ohm, Fault Impedance, Real Part"
      m_R_bus2 = "m:R:bus2", "Ohm, Fault Impedance, Real Part"
      m_X_bus1 = "m:X:bus1", "Ohm, Fault Impedance, Imaginary Part"
      m_X_bus2 = "m:X:bus2", "Ohm, Fault Impedance, Imaginary Part"
      m_Z_bus1 = "m:Z:bus1", "Ohm, Fault Impedance, Magnitude"
      m_Z_bus2 = "m:Z:bus2", "Ohm, Fault Impedance, Magnitude"
      m_phiz_bus1 = "m:phiz:bus1", "deg, Fault Impedance, Angle"
      m_phiz_bus2 = "m:phiz:bus2", "deg, Fault Impedance, Angle"
      m_zSbase_bus1 = "m:zSbase:bus1", "p.u., Fault Impedance, Magnitude"
      m_zSbase_bus2 = "m:zSbase:bus2", "p.u., Fault Impedance, Magnitude"
      m_XtoR_bus1 = "m:XtoR:bus1", "X/R ratio"
      m_XtoR_bus2 = "m:XtoR:bus2", "X/R ratio"
      m_Zs_bus1 = "m:Zs:bus1", "Ohm, Source Impedance, Magnitude"
      m_Zs_bus2 = "m:Zs:bus2", "Ohm, Source Impedance, Magnitude"
      m_phizs_bus1 = "m:phizs:bus1", "deg, Source Impedance, Angle"
      m_phizs_bus2 = "m:phizs:bus2", "deg, Source Impedance, Angle"
      m_Rs_bus1 = "m:Rs:bus1", "Ohm, Source Impedance, Real Part"
      m_Rs_bus2 = "m:Rs:bus2", "Ohm, Source Impedance, Real Part"
      m_Xs_bus1 = "m:Xs:bus1", "Ohm, Source Impedance, Imaginary Part"
      m_Xs_bus2 = "m:Xs:bus2", "Ohm, Source Impedance, Imaginary Part"
      m_phiui_bus1 = "m:phiui:bus1", "deg, Angle between Voltage and Current"
      m_phiui_bus2 = "m:phiui:bus2", "deg, Angle between Voltage and Current"
      m_cosphiui_bus1 = "m:cosphiui:bus1", "cosphi"
      m_cosphiui_bus2 = "m:cosphiui:bus2", "cosphi"
      m_P_bus1 = "m:P:bus1", "MW, Active Power"
      m_P_bus2 = "m:P:bus2", "MW, Active Power"
      m_Q_bus1 = "m:Q:bus1", "Mvar, Reactive Power"
      m_Q_bus2 = "m:Q:bus2", "Mvar, Reactive Power"
      m_BrkIpload_bus1 = "m:BrkIpload:bus1", "%, Breaker Loading (peak)"
      m_BrkIpload_bus2 = "m:BrkIpload:bus2", "%, Breaker Loading (peak)"
      m_BrkIbload_bus1 = "m:BrkIbload:bus1", "%, Breaker Loading (breaking)"
      m_BrkIbload_bus2 = "m:BrkIbload:bus2", "%, Breaker Loading (breaking)"
      m_BrkmaxIb_bus1 = "m:BrkmaxIb:bus1", "kA, Max. Breaking Current (breaker time)"
      m_BrkmaxIb_bus2 = "m:BrkmaxIb:bus2", "kA, Max. Breaking Current (breaker time)"
      n_Ikss_bus1 = "n:Ikss:bus1", "kA, Initial Short-Circuit Current"
      n_Ikss_bus2 = "n:Ikss:bus2", "kA, Initial Short-Circuit Current"
      n_I_bus1 = "n:I:bus1", "kA, Short-Circuit Current"
      n_I_bus2 = "n:I:bus2", "kA, Short-Circuit Current"
      n_phii_bus1 = "n:phii:bus1", "deg, Phase Current, Angle"
      n_phii_bus2 = "n:phii:bus2", "deg, Phase Current, Angle"
      n_Skss_bus1 = "n:Skss:bus1", "MVA, Initial Short-Circuit Power"
      n_Skss_bus2 = "n:Skss:bus2", "MVA, Initial Short-Circuit Power"
      n_ip_bus1 = "n:ip:bus1", "kA, Peak Short-Circuit Current"
      n_ip_bus2 = "n:ip:bus2", "kA, Peak Short-Circuit Current"
      n_Ib_bus1 = "n:Ib:bus1", "kA, Short-Circuit Breaking Current"
      n_Ib_bus2 = "n:Ib:bus2", "kA, Short-Circuit Breaking Current"
      n_Sb_bus1 = "n:Sb:bus1", "MVA, Short-Circuit Breaking Power"
      n_Sb_bus2 = "n:Sb:bus2", "MVA, Short-Circuit Breaking Power"
      n_Ik_bus1 = "n:Ik:bus1", "kA, Steady-State Short-Circuit Current"
      n_Ik_bus2 = "n:Ik:bus2", "kA, Steady-State Short-Circuit Current"
      n_Ith_bus1 = "n:Ith:bus1", "kA, Thermal equivalent Short-Circuit Current"
      n_Ith_bus2 = "n:Ith:bus2", "kA, Thermal equivalent Short-Circuit Current"
      n_R_bus1 = "n:R:bus1", "Ohm, Short-Circuit Impedance, Real Part"
      n_R_bus2 = "n:R:bus2", "Ohm, Short-Circuit Impedance, Real Part"
      n_X_bus1 = "n:X:bus1", "Ohm, Short-Circuit Impedance, Imaginary Part"
      n_X_bus2 = "n:X:bus2", "Ohm, Short-Circuit Impedance, Imaginary Part"
      n_Z_bus1 = "n:Z:bus1", "Ohm, Short-Circuit Impedance, Magnitude"
      n_Z_bus2 = "n:Z:bus2", "Ohm, Short-Circuit Impedance, Magnitude"
      n_phiz_bus1 = "n:phiz:bus1", "deg, Short-Circuit Impedance, Angle"
      n_phiz_bus2 = "n:phiz:bus2", "deg, Short-Circuit Impedance, Angle"
      n_rSbase_bus1 = "n:rSbase:bus1", "p.u., Short-Circuit Impedance, Real Part"
      n_rSbase_bus2 = "n:rSbase:bus2", "p.u., Short-Circuit Impedance, Real Part"
      n_xSbase_bus1 = "n:xSbase:bus1", "p.u., Short-Circuit Impedance, Imaginary Part"
      n_xSbase_bus2 = "n:xSbase:bus2", "p.u., Short-Circuit Impedance, Imaginary Part"
      n_zSbase_bus1 = "n:zSbase:bus1", "p.u., Short-Circuit Impedance, Magnitude"
      n_zSbase_bus2 = "n:zSbase:bus2", "p.u., Short-Circuit Impedance, Magnitude"
      n_XtoR_bus1 = "n:XtoR:bus1", "X/R ratio (peak)"
      n_XtoR_bus2 = "n:XtoR:bus2", "X/R ratio (peak)"
      n_RtoX_bus1 = "n:RtoX:bus1", "R/X ratio (peak)"
      n_RtoX_bus2 = "n:RtoX:bus2", "R/X ratio (peak)"
      n_ur_bus1 = "n:ur:bus1", "p.u., Voltage, Real Part"
      n_ur_bus2 = "n:ur:bus2", "p.u., Voltage, Real Part"
      n_ui_bus1 = "n:ui:bus1", "p.u., Voltage, Imaginary Part"
      n_ui_bus2 = "n:ui:bus2", "p.u., Voltage, Imaginary Part"
      n_u_bus1 = "n:u:bus1", "p.u., Voltage, Magnitude"
      n_u_bus2 = "n:u:bus2", "p.u., Voltage, Magnitude"
      n_upc_bus1 = "n:upc:bus1", "%, Voltage, Magnitude"
      n_upc_bus2 = "n:upc:bus2", "%, Voltage, Magnitude"
      n_U_bus1 = "n:U:bus1", "kV, Line-Ground Voltage, Magnitude"
      n_U_bus2 = "n:U:bus2", "kV, Line-Ground Voltage, Magnitude"
      n_Ul_bus1 = "n:Ul:bus1", "kV, Line-Line Voltage, Magnitude"
      n_Ul_bus2 = "n:Ul:bus2", "kV, Line-Line Voltage, Magnitude"
      n_phiu_bus1 = "n:phiu:bus1", "deg, Voltage, Angle"
      n_phiu_bus2 = "n:phiu:bus2", "deg, Voltage, Angle"
      n_du_bus1 = "n:du:bus1", "%, Voltage Deviation"
      n_du_bus2 = "n:du:bus2", "%, Voltage Deviation"
      n_urpref_bus1 = "n:urpref:bus1", "p.u., Pre-fault Voltage, Real Part"
      n_urpref_bus2 = "n:urpref:bus2", "p.u., Pre-fault Voltage, Real Part"
      n_uipref_bus1 = "n:uipref:bus1", "p.u., Pre-fault Voltage, Imaginary Part"
      n_uipref_bus2 = "n:uipref:bus2", "p.u., Pre-fault Voltage, Imaginary Part"
      n_upref_bus1 = "n:upref:bus1", "p.u., Pre-fault Voltage, Magnitude"
      n_upref_bus2 = "n:upref:bus2", "p.u., Pre-fault Voltage, Magnitude"
      n_uprefpc_bus1 = "n:uprefpc:bus1", "%, Pre-fault Voltage, Magnitude"
      n_uprefpc_bus2 = "n:uprefpc:bus2", "%, Pre-fault Voltage, Magnitude"
      n_Upref_bus1 = "n:Upref:bus1", "kV, Line-Ground Pre-fault Voltage, Magnitude"
      n_Upref_bus2 = "n:Upref:bus2", "kV, Line-Ground Pre-fault Voltage, Magnitude"
      n_Ulpref_bus1 = "n:Ulpref:bus1", "kV, Line-Line Pre-fault Voltage, Magnitude"
      n_Ulpref_bus2 = "n:Ulpref:bus2", "kV, Line-Line Pre-fault Voltage, Magnitude"
      n_phiupref_bus1 = "n:phiupref:bus1", "deg, Pre-fault Voltage, Angle"
      n_phiupref_bus2 = "n:phiupref:bus2", "deg, Pre-fault Voltage, Angle"
      n_phiui_bus1 = "n:phiui:bus1", "deg, Angle between Voltage and Current"
      n_phiui_bus2 = "n:phiui:bus2", "deg, Angle between Voltage and Current"
      n_cfac_bus1 = "n:cfac:bus1", "Voltage Factor"
      n_cfac_bus2 = "n:cfac:bus2", "Voltage Factor"
      n_idc_bus1 = "n:idc:bus1", "kA, Decaying Component"
      n_idc_bus2 = "n:idc:bus2", "kA, Decaying Component"
      n_Iasy_bus1 = "n:Iasy:bus1", "kA, Asymmetrical Breaking Current"
      n_Iasy_bus2 = "n:Iasy:bus2", "kA, Asymmetrical Breaking Current"
      n_Ithload_bus1 = "n:Ithload:bus1", "%, Loading, Thermal equivalent Current"
      n_Ithload_bus2 = "n:Ithload:bus2", "%, Loading, Thermal equivalent Current"
      n_Ipload_bus1 = "n:Ipload:bus1", "%, Loading, Peak Short-Circuit Current"
      n_Ipload_bus2 = "n:Ipload:bus2", "%, Loading, Peak Short-Circuit Current"
      n_Ithrtk_bus1 = "n:Ithrtk:bus1", "kA, Rated Short-Time Current (Tk)"
      n_Ithrtk_bus2 = "n:Ithrtk:bus2", "kA, Rated Short-Time Current (Tk)"
      n_Tn_bus1 = "n:Tn:bus1", "ms, Grid Time Constant"
      n_Tn_bus2 = "n:Tn:bus2", "ms, Grid Time Constant"
      n_XtoR_b_bus1 = "n:XtoR_b:bus1", "X/R ratio (dc)"
      n_XtoR_b_bus2 = "n:XtoR_b:bus2", "X/R ratio (dc)"
      n_RtoX_b_bus1 = "n:RtoX_b:bus1", "R/X ratio (dc)"
      n_RtoX_b_bus2 = "n:RtoX_b:bus2", "R/X ratio (dc)"
      n_fctof_b_bus1 = "n:fctof_b:bus1", "Equivalent Frequency fc/f (dc)"
      n_fctof_b_bus2 = "n:fctof_b:bus2", "Equivalent Frequency fc/f (dc)"
      e_iZshc = "e:iZshc", "Use same impedance as for load flow"
      e_rs_pu = "e:rs_pu", "p.u., Real Part"
      e_xs_pu = "e:xs_pu", "p.u., Imaginary Part"
      e_rs_pu_ji = "e:rs_pu_ji", "p.u., Real Part"
      e_xs_pu_ji = "e:xs_pu_ji", "p.u., Imaginary Part"
      e_r0s_pu = "e:r0s_pu", "p.u., Real Part"
      e_x0s_pu = "e:x0s_pu", "p.u., Imaginary Part"
      e_r0s_pu_ji = "e:r0s_pu_ji", "p.u., Real Part"
      e_x0s_pu_ji = "e:x0s_pu_ji", "p.u., Imaginary Part"
      e_r2s_pu = "e:r2s_pu", "p.u., Real Part"
      e_x2s_pu = "e:x2s_pu", "p.u., Imaginary Part"
      e_r2s_pu_ji = "e:r2s_pu_ji", "p.u., Real Part"
      e_x2s_pu_ji = "e:x2s_pu_ji", "p.u., Imaginary Part"
      m_Ikss_bus1_A = "m:Ikss:bus1:A", "kA, Initial Short-Circuit Current"
      m_Ikss_bus1_B = "m:Ikss:bus1:B", "kA, Initial Short-Circuit Current"
      m_Ikss_bus1_C = "m:Ikss:bus1:C", "kA, Initial Short-Circuit Current"
      m_Ikss_bus2_A = "m:Ikss:bus2:A", "kA, Initial Short-Circuit Current"
      m_Ikss_bus2_B = "m:Ikss:bus2:B", "kA, Initial Short-Circuit Current"
      m_Ikss_bus2_C = "m:Ikss:bus2:C", "kA, Initial Short-Circuit Current"
      m_ikss_bus1_A = "m:ikss:bus1:A", "p.u., Initial Short-Circuit Current, Magnitude"
      m_ikss_bus1_B = "m:ikss:bus1:B", "p.u., Initial Short-Circuit Current, Magnitude"
      m_ikss_bus1_C = "m:ikss:bus1:C", "p.u., Initial Short-Circuit Current, Magnitude"
      m_ikss_bus2_A = "m:ikss:bus2:A", "p.u., Initial Short-Circuit Current, Magnitude"
      m_ikss_bus2_B = "m:ikss:bus2:B", "p.u., Initial Short-Circuit Current, Magnitude"
      m_ikss_bus2_C = "m:ikss:bus2:C", "p.u., Initial Short-Circuit Current, Magnitude"
      m_Skss_bus1_A = "m:Skss:bus1:A", "MVA, Initial Short-Circuit Power"
      m_Skss_bus1_B = "m:Skss:bus1:B", "MVA, Initial Short-Circuit Power"
      m_Skss_bus1_C = "m:Skss:bus1:C", "MVA, Initial Short-Circuit Power"
      m_Skss_bus2_A = "m:Skss:bus2:A", "MVA, Initial Short-Circuit Power"
      m_Skss_bus2_B = "m:Skss:bus2:B", "MVA, Initial Short-Circuit Power"
      m_Skss_bus2_C = "m:Skss:bus2:C", "MVA, Initial Short-Circuit Power"
      m_Skssx3_bus1_A = "m:Skssx3:bus1:A", "MVA, 3*Initial Short-Circuit Power"
      m_Skssx3_bus1_B = "m:Skssx3:bus1:B", "MVA, 3*Initial Short-Circuit Power"
      m_Skssx3_bus1_C = "m:Skssx3:bus1:C", "MVA, 3*Initial Short-Circuit Power"
      m_Skssx3_bus2_A = "m:Skssx3:bus2:A", "MVA, 3*Initial Short-Circuit Power"
      m_Skssx3_bus2_B = "m:Skssx3:bus2:B", "MVA, 3*Initial Short-Circuit Power"
      m_Skssx3_bus2_C = "m:Skssx3:bus2:C", "MVA, 3*Initial Short-Circuit Power"
      m_ip_bus1_A = "m:ip:bus1:A", "kA, Peak Short-Circuit Current"
      m_ip_bus1_B = "m:ip:bus1:B", "kA, Peak Short-Circuit Current"
      m_ip_bus1_C = "m:ip:bus1:C", "kA, Peak Short-Circuit Current"
      m_ip_bus2_A = "m:ip:bus2:A", "kA, Peak Short-Circuit Current"
      m_ip_bus2_B = "m:ip:bus2:B", "kA, Peak Short-Circuit Current"
      m_ip_bus2_C = "m:ip:bus2:C", "kA, Peak Short-Circuit Current"
      m_Z_bus1_A = "m:Z:bus1:A", "Ohm, Fault Impedance, Magnitude"
      m_Z_bus1_B = "m:Z:bus1:B", "Ohm, Fault Impedance, Magnitude"
      m_Z_bus1_C = "m:Z:bus1:C", "Ohm, Fault Impedance, Magnitude"
      m_Z_bus2_A = "m:Z:bus2:A", "Ohm, Fault Impedance, Magnitude"
      m_Z_bus2_B = "m:Z:bus2:B", "Ohm, Fault Impedance, Magnitude"
      m_Z_bus2_C = "m:Z:bus2:C", "Ohm, Fault Impedance, Magnitude"
      m_phiz_bus1_A = "m:phiz:bus1:A", "deg, Fault Impedance, Angle"
      m_phiz_bus1_B = "m:phiz:bus1:B", "deg, Fault Impedance, Angle"
      m_phiz_bus1_C = "m:phiz:bus1:C", "deg, Fault Impedance, Angle"
      m_phiz_bus2_A = "m:phiz:bus2:A", "deg, Fault Impedance, Angle"
      m_phiz_bus2_B = "m:phiz:bus2:B", "deg, Fault Impedance, Angle"
      m_phiz_bus2_C = "m:phiz:bus2:C", "deg, Fault Impedance, Angle"
      m_Zl_bus1_A = "m:Zl:bus1:A", "Ohm, Line-Line Fault Impedance, Magnitude"
      m_Zl_bus1_B = "m:Zl:bus1:B", "Ohm, Line-Line Fault Impedance, Magnitude"
      m_Zl_bus1_C = "m:Zl:bus1:C", "Ohm, Line-Line Fault Impedance, Magnitude"
      m_Zl_bus2_A = "m:Zl:bus2:A", "Ohm, Line-Line Fault Impedance, Magnitude"
      m_Zl_bus2_B = "m:Zl:bus2:B", "Ohm, Line-Line Fault Impedance, Magnitude"
      m_Zl_bus2_C = "m:Zl:bus2:C", "Ohm, Line-Line Fault Impedance, Magnitude"
      m_phizl_bus1_A = "m:phizl:bus1:A", "deg, Line-Line Fault Impedance, Angle"
      m_phizl_bus1_B = "m:phizl:bus1:B", "deg, Line-Line Fault Impedance, Angle"
      m_phizl_bus1_C = "m:phizl:bus1:C", "deg, Line-Line Fault Impedance, Angle"
      m_phizl_bus2_A = "m:phizl:bus2:A", "deg, Line-Line Fault Impedance, Angle"
      m_phizl_bus2_B = "m:phizl:bus2:B", "deg, Line-Line Fault Impedance, Angle"
      m_phizl_bus2_C = "m:phizl:bus2:C", "deg, Line-Line Fault Impedance, Angle"
      m_Z0_bus1 = "m:Z0:bus1", "Ohm, Zero-Sequence Fault Impedance, Magnitude"
      m_Z0_bus2 = "m:Z0:bus2", "Ohm, Zero-Sequence Fault Impedance, Magnitude"
      m_phiz0_bus1 = "m:phiz0:bus1", "deg, Zero-Sequence Fault Impedance, Angle"
      m_phiz0_bus2 = "m:phiz0:bus2", "deg, Zero-Sequence Fault Impedance, Angle"
      m_z0Sbase_bus1 = "m:z0Sbase:bus1", "p.u., Zero-Sequence Fault Impedance, Magnitude"
      m_z0Sbase_bus2 = "m:z0Sbase:bus2", "p.u., Zero-Sequence Fault Impedance, Magnitude"
      m_cosphiu1i1_bus1 = "m:cosphiu1i1:bus1", "Positive-Sequence cosphi"
      m_cosphiu1i1_bus2 = "m:cosphiu1i1:bus2", "Positive-Sequence cosphi"
      m_I0x3watt_bus1 = "m:I0x3watt:bus1", "kA, Earth-Fault Residual Current"
      m_I0x3watt_bus2 = "m:I0x3watt:bus2", "kA, Earth-Fault Residual Current"
      m_I0x3var_bus1 = "m:I0x3var:bus1", "kA, Capacitive Earth-Fault Current"
      m_I0x3var_bus2 = "m:I0x3var:bus2", "kA, Capacitive Earth-Fault Current"
      m_Q0cap_bus1 = "m:Q0cap:bus1", "Mvar, Q0 capacitive"
      m_Q0cap_bus2 = "m:Q0cap:bus2", "Mvar, Q0 capacitive"
      m_Zfrel_bus1 = "m:Zfrel:bus1", "Ohm, Relay Impedance, Magnitude"
      m_Zfrel_bus2 = "m:Zfrel:bus2", "Ohm, Relay Impedance, Magnitude"
      m_phizfrel_bus1 = "m:phizfrel:bus1", "deg, Relay Impedance, Angle"
      m_phizfrel_bus2 = "m:phizfrel:bus2", "deg, Relay Impedance, Angle"
      m_Rfrel_bus1 = "m:Rfrel:bus1", "Ohm, Relay Impedance, Real Part"
      m_Rfrel_bus2 = "m:Rfrel:bus2", "Ohm, Relay Impedance, Real Part"
      m_Xfrel_bus1 = "m:Xfrel:bus1", "Ohm, Relay Impedance, Imaginary Part"
      m_Xfrel_bus2 = "m:Xfrel:bus2", "Ohm, Relay Impedance, Imaginary Part"
      m_Zf_bus1 = "m:Zf:bus1", "Ohm, Meas. Fault Impedance, Magnitude"
      m_Zf_bus2 = "m:Zf:bus2", "Ohm, Meas. Fault Impedance, Magnitude"
      m_phizf_bus1 = "m:phizf:bus1", "deg, Meas. Fault Impedance, Angle"
      m_phizf_bus2 = "m:phizf:bus2", "deg, Meas. Fault Impedance, Angle"
      n_Ikss_bus1_A = "n:Ikss:bus1:A", "kA, Initial Short-Circuit Current"
      n_Ikss_bus1_B = "n:Ikss:bus1:B", "kA, Initial Short-Circuit Current"
      n_Ikss_bus1_C = "n:Ikss:bus1:C", "kA, Initial Short-Circuit Current"
      n_Ikss_bus2_A = "n:Ikss:bus2:A", "kA, Initial Short-Circuit Current"
      n_Ikss_bus2_B = "n:Ikss:bus2:B", "kA, Initial Short-Circuit Current"
      n_Ikss_bus2_C = "n:Ikss:bus2:C", "kA, Initial Short-Circuit Current"
      n_I_bus1_A = "n:I:bus1:A", "kA, Short-Circuit Current"
      n_I_bus1_B = "n:I:bus1:B", "kA, Short-Circuit Current"
      n_I_bus1_C = "n:I:bus1:C", "kA, Short-Circuit Current"
      n_I_bus2_A = "n:I:bus2:A", "kA, Short-Circuit Current"
      n_I_bus2_B = "n:I:bus2:B", "kA, Short-Circuit Current"
      n_I_bus2_C = "n:I:bus2:C", "kA, Short-Circuit Current"
      n_phii_bus1_A = "n:phii:bus1:A", "deg, Phase Current, Angle"
      n_phii_bus1_B = "n:phii:bus1:B", "deg, Phase Current, Angle"
      n_phii_bus1_C = "n:phii:bus1:C", "deg, Phase Current, Angle"
      n_phii_bus2_A = "n:phii:bus2:A", "deg, Phase Current, Angle"
      n_phii_bus2_B = "n:phii:bus2:B", "deg, Phase Current, Angle"
      n_phii_bus2_C = "n:phii:bus2:C", "deg, Phase Current, Angle"
      n_Skss_bus1_A = "n:Skss:bus1:A", "MVA, Initial Short-Circuit Power"
      n_Skss_bus1_B = "n:Skss:bus1:B", "MVA, Initial Short-Circuit Power"
      n_Skss_bus1_C = "n:Skss:bus1:C", "MVA, Initial Short-Circuit Power"
      n_Skss_bus2_A = "n:Skss:bus2:A", "MVA, Initial Short-Circuit Power"
      n_Skss_bus2_B = "n:Skss:bus2:B", "MVA, Initial Short-Circuit Power"
      n_Skss_bus2_C = "n:Skss:bus2:C", "MVA, Initial Short-Circuit Power"
      n_Skssx3_bus1_A = "n:Skssx3:bus1:A", "MVA, 3*Initial Short-Circuit Power"
      n_Skssx3_bus1_B = "n:Skssx3:bus1:B", "MVA, 3*Initial Short-Circuit Power"
      n_Skssx3_bus1_C = "n:Skssx3:bus1:C", "MVA, 3*Initial Short-Circuit Power"
      n_Skssx3_bus2_A = "n:Skssx3:bus2:A", "MVA, 3*Initial Short-Circuit Power"
      n_Skssx3_bus2_B = "n:Skssx3:bus2:B", "MVA, 3*Initial Short-Circuit Power"
      n_Skssx3_bus2_C = "n:Skssx3:bus2:C", "MVA, 3*Initial Short-Circuit Power"
      n_ip_bus1_A = "n:ip:bus1:A", "kA, Peak Short-Circuit Current"
      n_ip_bus1_B = "n:ip:bus1:B", "kA, Peak Short-Circuit Current"
      n_ip_bus1_C = "n:ip:bus1:C", "kA, Peak Short-Circuit Current"
      n_ip_bus2_A = "n:ip:bus2:A", "kA, Peak Short-Circuit Current"
      n_ip_bus2_B = "n:ip:bus2:B", "kA, Peak Short-Circuit Current"
      n_ip_bus2_C = "n:ip:bus2:C", "kA, Peak Short-Circuit Current"
      n_Ib_bus1_A = "n:Ib:bus1:A", "kA, Short-Circuit Breaking Current"
      n_Ib_bus1_B = "n:Ib:bus1:B", "kA, Short-Circuit Breaking Current"
      n_Ib_bus1_C = "n:Ib:bus1:C", "kA, Short-Circuit Breaking Current"
      n_Ib_bus2_A = "n:Ib:bus2:A", "kA, Short-Circuit Breaking Current"
      n_Ib_bus2_B = "n:Ib:bus2:B", "kA, Short-Circuit Breaking Current"
      n_Ib_bus2_C = "n:Ib:bus2:C", "kA, Short-Circuit Breaking Current"
      n_Sb_bus1_A = "n:Sb:bus1:A", "MVA, Short-Circuit Breaking Power"
      n_Sb_bus1_B = "n:Sb:bus1:B", "MVA, Short-Circuit Breaking Power"
      n_Sb_bus1_C = "n:Sb:bus1:C", "MVA, Short-Circuit Breaking Power"
      n_Sb_bus2_A = "n:Sb:bus2:A", "MVA, Short-Circuit Breaking Power"
      n_Sb_bus2_B = "n:Sb:bus2:B", "MVA, Short-Circuit Breaking Power"
      n_Sb_bus2_C = "n:Sb:bus2:C", "MVA, Short-Circuit Breaking Power"
      n_Ith_bus1_A = "n:Ith:bus1:A", "kA, Thermal equivalent Short-Circuit Current"
      n_Ith_bus1_B = "n:Ith:bus1:B", "kA, Thermal equivalent Short-Circuit Current"
      n_Ith_bus1_C = "n:Ith:bus1:C", "kA, Thermal equivalent Short-Circuit Current"
      n_Ith_bus2_A = "n:Ith:bus2:A", "kA, Thermal equivalent Short-Circuit Current"
      n_Ith_bus2_B = "n:Ith:bus2:B", "kA, Thermal equivalent Short-Circuit Current"
      n_Ith_bus2_C = "n:Ith:bus2:C", "kA, Thermal equivalent Short-Circuit Current"
      n_Ithmax_bus1 = "n:Ithmax:bus1", "kA, Max. Thermal equivalent Short-Circuit Current"
      n_Ithmax_bus2 = "n:Ithmax:bus2", "kA, Max. Thermal equivalent Short-Circuit Current"
      n_Ik_bus1_A = "n:Ik:bus1:A", "kA, Steady-State Short-Circuit Current"
      n_Ik_bus1_B = "n:Ik:bus1:B", "kA, Steady-State Short-Circuit Current"
      n_Ik_bus1_C = "n:Ik:bus1:C", "kA, Steady-State Short-Circuit Current"
      n_Ik_bus2_A = "n:Ik:bus2:A", "kA, Steady-State Short-Circuit Current"
      n_Ik_bus2_B = "n:Ik:bus2:B", "kA, Steady-State Short-Circuit Current"
      n_Ik_bus2_C = "n:Ik:bus2:C", "kA, Steady-State Short-Circuit Current"
      n_R0_bus1 = "n:R0:bus1", "Ohm, Zero-Sequence Impedance, Real Part"
      n_R0_bus2 = "n:R0:bus2", "Ohm, Zero-Sequence Impedance, Real Part"
      n_X0_bus1 = "n:X0:bus1", "Ohm, Zero-Sequence Impedance, Imaginary Part"
      n_X0_bus2 = "n:X0:bus2", "Ohm, Zero-Sequence Impedance, Imaginary Part"
      n_Z0_bus1 = "n:Z0:bus1", "Ohm, Zero-Sequence Impedance, Magnitude"
      n_Z0_bus2 = "n:Z0:bus2", "Ohm, Zero-Sequence Impedance, Magnitude"
      n_phiz0_bus1 = "n:phiz0:bus1", "deg, Zero-Sequence Impedance, Angle"
      n_phiz0_bus2 = "n:phiz0:bus2", "deg, Zero-Sequence Impedance, Angle"
      n_r0Sbase_bus1 = "n:r0Sbase:bus1", "p.u., Zero-Sequence Impedance, Real Part"
      n_r0Sbase_bus2 = "n:r0Sbase:bus2", "p.u., Zero-Sequence Impedance, Real Part"
      n_x0Sbase_bus1 = "n:x0Sbase:bus1", "p.u., Zero-Sequence Impedance, Imaginary Part"
      n_x0Sbase_bus2 = "n:x0Sbase:bus2", "p.u., Zero-Sequence Impedance, Imaginary Part"
      n_z0Sbase_bus1 = "n:z0Sbase:bus1", "p.u., Zero-Sequence Impedance, Magnitude"
      n_z0Sbase_bus2 = "n:z0Sbase:bus2", "p.u., Zero-Sequence Impedance, Magnitude"
      n_R1_bus1 = "n:R1:bus1", "Ohm, Positive-Sequence Impedance, Real Part"
      n_R1_bus2 = "n:R1:bus2", "Ohm, Positive-Sequence Impedance, Real Part"
      n_X1_bus1 = "n:X1:bus1", "Ohm, Positive-Sequence Impedance, Imaginary Part"
      n_X1_bus2 = "n:X1:bus2", "Ohm, Positive-Sequence Impedance, Imaginary Part"
      n_Z1_bus1 = "n:Z1:bus1", "Ohm, Positive-Sequence Impedance, Magnitude"
      n_Z1_bus2 = "n:Z1:bus2", "Ohm, Positive-Sequence Impedance, Magnitude"
      n_phiz1_bus1 = "n:phiz1:bus1", "deg, Positive-Sequence Impedance, Angle"
      n_phiz1_bus2 = "n:phiz1:bus2", "deg, Positive-Sequence Impedance, Angle"
      n_r1Sbase_bus1 = "n:r1Sbase:bus1", "p.u., Positive-Sequence Impedance, Real Part"
      n_r1Sbase_bus2 = "n:r1Sbase:bus2", "p.u., Positive-Sequence Impedance, Real Part"
      n_x1Sbase_bus1 = "n:x1Sbase:bus1", "p.u., Positive-Sequence Impedance, Imaginary Part"
      n_x1Sbase_bus2 = "n:x1Sbase:bus2", "p.u., Positive-Sequence Impedance, Imaginary Part"
      n_z1Sbase_bus1 = "n:z1Sbase:bus1", "p.u., Positive-Sequence Impedance, Magnitude"
      n_z1Sbase_bus2 = "n:z1Sbase:bus2", "p.u., Positive-Sequence Impedance, Magnitude"
      n_R2_bus1 = "n:R2:bus1", "Ohm, Negative-Sequence Impedance, Real Part"
      n_R2_bus2 = "n:R2:bus2", "Ohm, Negative-Sequence Impedance, Real Part"
      n_X2_bus1 = "n:X2:bus1", "Ohm, Negative-Sequence Impedance, Imaginary Part"
      n_X2_bus2 = "n:X2:bus2", "Ohm, Negative-Sequence Impedance, Imaginary Part"
      n_Z2_bus1 = "n:Z2:bus1", "Ohm, Negative-Sequence Impedance, Magnitude"
      n_Z2_bus2 = "n:Z2:bus2", "Ohm, Negative-Sequence Impedance, Magnitude"
      n_phiz2_bus1 = "n:phiz2:bus1", "deg, Negative-Sequence Impedance, Angle"
      n_phiz2_bus2 = "n:phiz2:bus2", "deg, Negative-Sequence Impedance, Angle"
      n_r2Sbase_bus1 = "n:r2Sbase:bus1", "p.u., Negative-Sequence Impedance, Real Part"
      n_r2Sbase_bus2 = "n:r2Sbase:bus2", "p.u., Negative-Sequence Impedance, Real Part"
      n_x2Sbase_bus1 = "n:x2Sbase:bus1", "p.u., Negative-Sequence Impedance, Imaginary Part"
      n_x2Sbase_bus2 = "n:x2Sbase:bus2", "p.u., Negative-Sequence Impedance, Imaginary Part"
      n_z2Sbase_bus1 = "n:z2Sbase:bus1", "p.u., Negative-Sequence Impedance, Magnitude"
      n_z2Sbase_bus2 = "n:z2Sbase:bus2", "p.u., Negative-Sequence Impedance, Magnitude"
      n_R_bus1_A = "n:R:bus1:A", "Ohm, Network Resistance, Line-Ground"
      n_R_bus1_B = "n:R:bus1:B", "Ohm, Network Resistance, Line-Ground"
      n_R_bus1_C = "n:R:bus1:C", "Ohm, Network Resistance, Line-Ground"
      n_R_bus2_A = "n:R:bus2:A", "Ohm, Network Resistance, Line-Ground"
      n_R_bus2_B = "n:R:bus2:B", "Ohm, Network Resistance, Line-Ground"
      n_R_bus2_C = "n:R:bus2:C", "Ohm, Network Resistance, Line-Ground"
      n_X_bus1_A = "n:X:bus1:A", "Ohm, Network Reactance, Line-Ground"
      n_X_bus1_B = "n:X:bus1:B", "Ohm, Network Reactance, Line-Ground"
      n_X_bus1_C = "n:X:bus1:C", "Ohm, Network Reactance, Line-Ground"
      n_X_bus2_A = "n:X:bus2:A", "Ohm, Network Reactance, Line-Ground"
      n_X_bus2_B = "n:X:bus2:B", "Ohm, Network Reactance, Line-Ground"
      n_X_bus2_C = "n:X:bus2:C", "Ohm, Network Reactance, Line-Ground"
      n_Z_bus1_A = "n:Z:bus1:A", "Ohm, Network Impedance, Line-Ground"
      n_Z_bus1_B = "n:Z:bus1:B", "Ohm, Network Impedance, Line-Ground"
      n_Z_bus1_C = "n:Z:bus1:C", "Ohm, Network Impedance, Line-Ground"
      n_Z_bus2_A = "n:Z:bus2:A", "Ohm, Network Impedance, Line-Ground"
      n_Z_bus2_B = "n:Z:bus2:B", "Ohm, Network Impedance, Line-Ground"
      n_Z_bus2_C = "n:Z:bus2:C", "Ohm, Network Impedance, Line-Ground"
      n_phiz_bus1_A = "n:phiz:bus1:A", "deg, Angle of Network Impedance, Line-Ground"
      n_phiz_bus1_B = "n:phiz:bus1:B", "deg, Angle of Network Impedance, Line-Ground"
      n_phiz_bus1_C = "n:phiz:bus1:C", "deg, Angle of Network Impedance, Line-Ground"
      n_phiz_bus2_A = "n:phiz:bus2:A", "deg, Angle of Network Impedance, Line-Ground"
      n_phiz_bus2_B = "n:phiz:bus2:B", "deg, Angle of Network Impedance, Line-Ground"
      n_phiz_bus2_C = "n:phiz:bus2:C", "deg, Angle of Network Impedance, Line-Ground"
      n_Rl_bus1_A = "n:Rl:bus1:A", "Ohm, Network Resistance, Line-Line"
      n_Rl_bus1_B = "n:Rl:bus1:B", "Ohm, Network Resistance, Line-Line"
      n_Rl_bus1_C = "n:Rl:bus1:C", "Ohm, Network Resistance, Line-Line"
      n_Rl_bus2_A = "n:Rl:bus2:A", "Ohm, Network Resistance, Line-Line"
      n_Rl_bus2_B = "n:Rl:bus2:B", "Ohm, Network Resistance, Line-Line"
      n_Rl_bus2_C = "n:Rl:bus2:C", "Ohm, Network Resistance, Line-Line"
      n_Xl_bus1_A = "n:Xl:bus1:A", "Ohm, Network Reactance, Line-Line"
      n_Xl_bus1_B = "n:Xl:bus1:B", "Ohm, Network Reactance, Line-Line"
      n_Xl_bus1_C = "n:Xl:bus1:C", "Ohm, Network Reactance, Line-Line"
      n_Xl_bus2_A = "n:Xl:bus2:A", "Ohm, Network Reactance, Line-Line"
      n_Xl_bus2_B = "n:Xl:bus2:B", "Ohm, Network Reactance, Line-Line"
      n_Xl_bus2_C = "n:Xl:bus2:C", "Ohm, Network Reactance, Line-Line"
      n_Zl_bus1_A = "n:Zl:bus1:A", "Ohm, Network Impedance, Line-Line"
      n_Zl_bus1_B = "n:Zl:bus1:B", "Ohm, Network Impedance, Line-Line"
      n_Zl_bus1_C = "n:Zl:bus1:C", "Ohm, Network Impedance, Line-Line"
      n_Zl_bus2_A = "n:Zl:bus2:A", "Ohm, Network Impedance, Line-Line"
      n_Zl_bus2_B = "n:Zl:bus2:B", "Ohm, Network Impedance, Line-Line"
      n_Zl_bus2_C = "n:Zl:bus2:C", "Ohm, Network Impedance, Line-Line"
      n_phizl_bus1_A = "n:phizl:bus1:A", "deg, Angle of Network Impedance, Line-Line"
      n_phizl_bus1_B = "n:phizl:bus1:B", "deg, Angle of Network Impedance, Line-Line"
      n_phizl_bus1_C = "n:phizl:bus1:C", "deg, Angle of Network Impedance, Line-Line"
      n_phizl_bus2_A = "n:phizl:bus2:A", "deg, Angle of Network Impedance, Line-Line"
      n_phizl_bus2_B = "n:phizl:bus2:B", "deg, Angle of Network Impedance, Line-Line"
      n_phizl_bus2_C = "n:phizl:bus2:C", "deg, Angle of Network Impedance, Line-Line"
      n_Rln_bus1_A = "n:Rln:bus1:A", "Ohm, Network Resistance, Line-Neutral"
      n_Rln_bus1_B = "n:Rln:bus1:B", "Ohm, Network Resistance, Line-Neutral"
      n_Rln_bus1_C = "n:Rln:bus1:C", "Ohm, Network Resistance, Line-Neutral"
      n_Rln_bus2_A = "n:Rln:bus2:A", "Ohm, Network Resistance, Line-Neutral"
      n_Rln_bus2_B = "n:Rln:bus2:B", "Ohm, Network Resistance, Line-Neutral"
      n_Rln_bus2_C = "n:Rln:bus2:C", "Ohm, Network Resistance, Line-Neutral"
      n_Xln_bus1_A = "n:Xln:bus1:A", "Ohm, Network Reactance, Line-Neutral"
      n_Xln_bus1_B = "n:Xln:bus1:B", "Ohm, Network Reactance, Line-Neutral"
      n_Xln_bus1_C = "n:Xln:bus1:C", "Ohm, Network Reactance, Line-Neutral"
      n_Xln_bus2_A = "n:Xln:bus2:A", "Ohm, Network Reactance, Line-Neutral"
      n_Xln_bus2_B = "n:Xln:bus2:B", "Ohm, Network Reactance, Line-Neutral"
      n_Xln_bus2_C = "n:Xln:bus2:C", "Ohm, Network Reactance, Line-Neutral"
      n_Zln_bus1_A = "n:Zln:bus1:A", "Ohm, Network Impedance, Line-Neutral"
      n_Zln_bus1_B = "n:Zln:bus1:B", "Ohm, Network Impedance, Line-Neutral"
      n_Zln_bus1_C = "n:Zln:bus1:C", "Ohm, Network Impedance, Line-Neutral"
      n_Zln_bus2_A = "n:Zln:bus2:A", "Ohm, Network Impedance, Line-Neutral"
      n_Zln_bus2_B = "n:Zln:bus2:B", "Ohm, Network Impedance, Line-Neutral"
      n_Zln_bus2_C = "n:Zln:bus2:C", "Ohm, Network Impedance, Line-Neutral"
      n_phizln_bus1_A = "n:phizln:bus1:A", "deg, Angle of Network Impedance, Line-Neutral"
      n_phizln_bus1_B = "n:phizln:bus1:B", "deg, Angle of Network Impedance, Line-Neutral"
      n_phizln_bus1_C = "n:phizln:bus1:C", "deg, Angle of Network Impedance, Line-Neutral"
      n_phizln_bus2_A = "n:phizln:bus2:A", "deg, Angle of Network Impedance, Line-Neutral"
      n_phizln_bus2_B = "n:phizln:bus2:B", "deg, Angle of Network Impedance, Line-Neutral"
      n_phizln_bus2_C = "n:phizln:bus2:C", "deg, Angle of Network Impedance, Line-Neutral"
      n_Rn_bus1 = "n:Rn:bus1", "Ohm, Network Resistance, Neutral"
      n_Rn_bus2 = "n:Rn:bus2", "Ohm, Network Resistance, Neutral"
      n_Xn_bus1 = "n:Xn:bus1", "Ohm, Network Reactance, Neutral"
      n_Xn_bus2 = "n:Xn:bus2", "Ohm, Network Reactance, Neutral"
      n_Zn_bus1 = "n:Zn:bus1", "Ohm, Network Impedance, Neutral"
      n_Zn_bus2 = "n:Zn:bus2", "Ohm, Network Impedance, Neutral"
      n_phizn_bus1 = "n:phizn:bus1", "deg, Angle of Network Impedance, Neutral"
      n_phizn_bus2 = "n:phizn:bus2", "deg, Angle of Network Impedance, Neutral"
      n_eff_bus1_A = "n:eff:bus1:A", "Earth Fault Factor"
      n_eff_bus1_B = "n:eff:bus1:B", "Earth Fault Factor"
      n_eff_bus1_C = "n:eff:bus1:C", "Earth Fault Factor"
      n_eff_bus2_A = "n:eff:bus2:A", "Earth Fault Factor"
      n_eff_bus2_B = "n:eff:bus2:B", "Earth Fault Factor"
      n_eff_bus2_C = "n:eff:bus2:C", "Earth Fault Factor"
      n_X0toX1_bus1 = "n:X0toX1:bus1", "X0/X1"
      n_X0toX1_bus2 = "n:X0toX1:bus2", "X0/X1"
      n_in_bus1 = "n:in:bus1", "p.u., Neutral Current, Magnitude"
      n_in_bus2 = "n:in:bus2", "p.u., Neutral Current, Magnitude"
      n_In_bus1 = "n:In:bus1", "kA, Neutral Current, Magnitude"
      n_In_bus2 = "n:In:bus2", "kA, Neutral Current, Magnitude"
      n_phiin_bus1 = "n:phiin:bus1", "deg, Neutral Current, Angle"
      n_phiin_bus2 = "n:phiin:bus2", "deg, Neutral Current, Angle"
      n_i0r_bus1 = "n:i0r:bus1", "p.u., Zero-Sequence Current, Real Part"
      n_i0r_bus2 = "n:i0r:bus2", "p.u., Zero-Sequence Current, Real Part"
      n_i0i_bus1 = "n:i0i:bus1", "p.u., Zero-Sequence Current, Imaginary Part"
      n_i0i_bus2 = "n:i0i:bus2", "p.u., Zero-Sequence Current, Imaginary Part"
      n_i0_bus1 = "n:i0:bus1", "p.u., Zero-Sequence Current, Magnitude"
      n_i0_bus2 = "n:i0:bus2", "p.u., Zero-Sequence Current, Magnitude"
      n_I0_bus1 = "n:I0:bus1", "kA, Zero-Sequence Current, Magnitude"
      n_I0_bus2 = "n:I0:bus2", "kA, Zero-Sequence Current, Magnitude"
      n_I0x3_bus1 = "n:I0x3:bus1", "kA, 3*I0"
      n_I0x3_bus2 = "n:I0x3:bus2", "kA, 3*I0"
      n_phii0_bus1 = "n:phii0:bus1", "deg, Zero-Sequence Current, Angle"
      n_phii0_bus2 = "n:phii0:bus2", "deg, Zero-Sequence Current, Angle"
      n_i1r_bus1 = "n:i1r:bus1", "p.u., Positive-Sequence Current, Real Part"
      n_i1r_bus2 = "n:i1r:bus2", "p.u., Positive-Sequence Current, Real Part"
      n_i1i_bus1 = "n:i1i:bus1", "p.u., Positive-Sequence Current, Imaginary Part"
      n_i1i_bus2 = "n:i1i:bus2", "p.u., Positive-Sequence Current, Imaginary Part"
      n_i1_bus1 = "n:i1:bus1", "p.u., Positive-Sequence Current, Magnitude"
      n_i1_bus2 = "n:i1:bus2", "p.u., Positive-Sequence Current, Magnitude"
      n_I1_bus1 = "n:I1:bus1", "kA, Positive-Sequence Current, Magnitude"
      n_I1_bus2 = "n:I1:bus2", "kA, Positive-Sequence Current, Magnitude"
      n_phii1_bus1 = "n:phii1:bus1", "deg, Positive-Sequence Current, Angle"
      n_phii1_bus2 = "n:phii1:bus2", "deg, Positive-Sequence Current, Angle"
      n_i2r_bus1 = "n:i2r:bus1", "p.u., Negative-Sequence Current, Real Part"
      n_i2r_bus2 = "n:i2r:bus2", "p.u., Negative-Sequence Current, Real Part"
      n_i2i_bus1 = "n:i2i:bus1", "p.u., Negative-Sequence Current, Imaginary Part"
      n_i2i_bus2 = "n:i2i:bus2", "p.u., Negative-Sequence Current, Imaginary Part"
      n_i2_bus1 = "n:i2:bus1", "p.u., Negative-Sequence Current, Magnitude"
      n_i2_bus2 = "n:i2:bus2", "p.u., Negative-Sequence Current, Magnitude"
      n_I2_bus1 = "n:I2:bus1", "kA, Negative-Sequence Current, Magnitude"
      n_I2_bus2 = "n:I2:bus2", "kA, Negative-Sequence Current, Magnitude"
      n_phii2_bus1 = "n:phii2:bus1", "deg, Negative-Sequence Current, Angle"
      n_phii2_bus2 = "n:phii2:bus2", "deg, Negative-Sequence Current, Angle"
      n_du_bus1_A = "n:du:bus1:A", "%, Voltage Deviation"
      n_du_bus1_B = "n:du:bus1:B", "%, Voltage Deviation"
      n_du_bus1_C = "n:du:bus1:C", "%, Voltage Deviation"
      n_du_bus2_A = "n:du:bus2:A", "%, Voltage Deviation"
      n_du_bus2_B = "n:du:bus2:B", "%, Voltage Deviation"
      n_du_bus2_C = "n:du:bus2:C", "%, Voltage Deviation"
      n_urpref_bus1_A = "n:urpref:bus1:A", "p.u., Pre-fault Voltage, Real Part"
      n_urpref_bus1_B = "n:urpref:bus1:B", "p.u., Pre-fault Voltage, Real Part"
      n_urpref_bus1_C = "n:urpref:bus1:C", "p.u., Pre-fault Voltage, Real Part"
      n_urpref_bus2_A = "n:urpref:bus2:A", "p.u., Pre-fault Voltage, Real Part"
      n_urpref_bus2_B = "n:urpref:bus2:B", "p.u., Pre-fault Voltage, Real Part"
      n_urpref_bus2_C = "n:urpref:bus2:C", "p.u., Pre-fault Voltage, Real Part"
      n_uipref_bus1_A = "n:uipref:bus1:A", "p.u., Pre-fault Voltage, Imaginary Part"
      n_uipref_bus1_B = "n:uipref:bus1:B", "p.u., Pre-fault Voltage, Imaginary Part"
      n_uipref_bus1_C = "n:uipref:bus1:C", "p.u., Pre-fault Voltage, Imaginary Part"
      n_uipref_bus2_A = "n:uipref:bus2:A", "p.u., Pre-fault Voltage, Imaginary Part"
      n_uipref_bus2_B = "n:uipref:bus2:B", "p.u., Pre-fault Voltage, Imaginary Part"
      n_uipref_bus2_C = "n:uipref:bus2:C", "p.u., Pre-fault Voltage, Imaginary Part"
      n_upref_bus1_A = "n:upref:bus1:A", "p.u., Pre-fault Voltage, Magnitude"
      n_upref_bus1_B = "n:upref:bus1:B", "p.u., Pre-fault Voltage, Magnitude"
      n_upref_bus1_C = "n:upref:bus1:C", "p.u., Pre-fault Voltage, Magnitude"
      n_upref_bus2_A = "n:upref:bus2:A", "p.u., Pre-fault Voltage, Magnitude"
      n_upref_bus2_B = "n:upref:bus2:B", "p.u., Pre-fault Voltage, Magnitude"
      n_upref_bus2_C = "n:upref:bus2:C", "p.u., Pre-fault Voltage, Magnitude"
      n_uprefpc_bus1_A = "n:uprefpc:bus1:A", "%, Pre-fault Voltage, Magnitude"
      n_uprefpc_bus1_B = "n:uprefpc:bus1:B", "%, Pre-fault Voltage, Magnitude"
      n_uprefpc_bus1_C = "n:uprefpc:bus1:C", "%, Pre-fault Voltage, Magnitude"
      n_uprefpc_bus2_A = "n:uprefpc:bus2:A", "%, Pre-fault Voltage, Magnitude"
      n_uprefpc_bus2_B = "n:uprefpc:bus2:B", "%, Pre-fault Voltage, Magnitude"
      n_uprefpc_bus2_C = "n:uprefpc:bus2:C", "%, Pre-fault Voltage, Magnitude"
      n_Upref_bus1_A = "n:Upref:bus1:A", "kV, Line-Ground Pre-fault Voltage, Magnitude"
      n_Upref_bus1_B = "n:Upref:bus1:B", "kV, Line-Ground Pre-fault Voltage, Magnitude"
      n_Upref_bus1_C = "n:Upref:bus1:C", "kV, Line-Ground Pre-fault Voltage, Magnitude"
      n_Upref_bus2_A = "n:Upref:bus2:A", "kV, Line-Ground Pre-fault Voltage, Magnitude"
      n_Upref_bus2_B = "n:Upref:bus2:B", "kV, Line-Ground Pre-fault Voltage, Magnitude"
      n_Upref_bus2_C = "n:Upref:bus2:C", "kV, Line-Ground Pre-fault Voltage, Magnitude"
      n_Ulpref_bus1_A = "n:Ulpref:bus1:A", "kV, Line-Line Pre-fault Voltage, Magnitude"
      n_Ulpref_bus1_B = "n:Ulpref:bus1:B", "kV, Line-Line Pre-fault Voltage, Magnitude"
      n_Ulpref_bus1_C = "n:Ulpref:bus1:C", "kV, Line-Line Pre-fault Voltage, Magnitude"
      n_Ulpref_bus2_A = "n:Ulpref:bus2:A", "kV, Line-Line Pre-fault Voltage, Magnitude"
      n_Ulpref_bus2_B = "n:Ulpref:bus2:B", "kV, Line-Line Pre-fault Voltage, Magnitude"
      n_Ulpref_bus2_C = "n:Ulpref:bus2:C", "kV, Line-Line Pre-fault Voltage, Magnitude"
      n_phiupref_bus1_A = "n:phiupref:bus1:A", "deg, Pre-fault Voltage, Angle"
      n_phiupref_bus1_B = "n:phiupref:bus1:B", "deg, Pre-fault Voltage, Angle"
      n_phiupref_bus1_C = "n:phiupref:bus1:C", "deg, Pre-fault Voltage, Angle"
      n_phiupref_bus2_A = "n:phiupref:bus2:A", "deg, Pre-fault Voltage, Angle"
      n_phiupref_bus2_B = "n:phiupref:bus2:B", "deg, Pre-fault Voltage, Angle"
      n_phiupref_bus2_C = "n:phiupref:bus2:C", "deg, Pre-fault Voltage, Angle"
      n_phiui_bus1_A = "n:phiui:bus1:A", "deg, Angle between Voltage and Current"
      n_phiui_bus1_B = "n:phiui:bus1:B", "deg, Angle between Voltage and Current"
      n_phiui_bus1_C = "n:phiui:bus1:C", "deg, Angle between Voltage and Current"
      n_phiui_bus2_A = "n:phiui:bus2:A", "deg, Angle between Voltage and Current"
      n_phiui_bus2_B = "n:phiui:bus2:B", "deg, Angle between Voltage and Current"
      n_phiui_bus2_C = "n:phiui:bus2:C", "deg, Angle between Voltage and Current"
      n_phiu1i1_bus1 = "n:phiu1i1:bus1", "deg, Angle between Voltage and Current in positive sequence system"
      n_phiu1i1_bus2 = "n:phiu1i1:bus2", "deg, Angle between Voltage and Current in positive sequence system"
      n_phiu2i2_bus1 = "n:phiu2i2:bus1", "deg, Angle between Voltage and Current in negative sequence system"
      n_phiu2i2_bus2 = "n:phiu2i2:bus2", "deg, Angle between Voltage and Current in negative sequence system"
      n_phiu0i0_bus1 = "n:phiu0i0:bus1", "deg, Angle between Voltage and Current in zero-sequence system"
      n_phiu0i0_bus2 = "n:phiu0i0:bus2", "deg, Angle between Voltage and Current in zero-sequence system"


  class RMS_Bal:

    class ElmArea(Enum):
      _init_ = 'value __doc__'

      c_Pgen = "c:Pgen", "MW, Generators, Active Power"
      c_Qgen = "c:Qgen", "Mvar, Generators, Reactive Power"
      c_Sgen = "c:Sgen", "MVA, Generators, Apparent Power"
      c_cosgen = "c:cosgen", "Generators, Power Factor"
      c_Pmot = "c:Pmot", "MW, Motor Loads, Active Power"
      c_Qmot = "c:Qmot", "Mvar, Motor Loads, Reactive Power"
      c_Smot = "c:Smot", "MVA, Motor Loads, Apparent Power"
      c_cosmot = "c:cosmot", "Motor Loads, Power Factor"
      c_Pload = "c:Pload", "MW, Loads, Active Power"
      c_Qload = "c:Qload", "Mvar, Loads, Reactive Power"
      c_Sload = "c:Sload", "MVA, Loads, Apparent Power"
      c_cosload = "c:cosload", "Loads, Power Factor"
      c_Pcomp = "c:Pcomp", "MW, Compensation (Losses)"
      c_Qcomp = "c:Qcomp", "Mvar, Compensation, C"
      c_Qcompr = "c:Qcompr", "Mvar, Compensation, L"
      c_Pnet = "c:Pnet", "MW, External Networks, Active Power"
      c_Qnet = "c:Qnet", "Mvar, External Networks, Reactive Power"
      c_Snet = "c:Snet", "MVA, External Networks, Apparent Power"
      c_cosnet = "c:cosnet", "External Networks, Power Factor"
      c_Pinter = "c:Pinter", "MW, Interchange Flow, Active Power"
      c_Qinter = "c:Qinter", "Mvar, Interchange Flow, Reactive Power"
      c_Ploss = "c:Ploss", "MW, Losses, Active Power"
      c_Qloss = "c:Qloss", "Mvar, Losses, Reactive Power"
      c_Pt = "c:Pt", "MW, Turbine Power"
      c_Pres = "c:Pres", "MW, Available-Reserve"

    class ElmAsm(Enum):
      _init_ = 'value __doc__'

      m_u1r_bus1 = "m:u1r:bus1", "p.u., Positive-Sequence-Voltage, Real Part"
      m_u1i_bus1 = "m:u1i:bus1", "p.u., Positive-Sequence-Voltage, Imaginary Part"
      m_u1_bus1 = "m:u1:bus1", "p.u., Positive-Sequence-Voltage, Magnitude"
      m_U1_bus1 = "m:U1:bus1", "kV, Line-Ground Positive-Sequence-Voltage, Magnitude"
      m_U1l_bus1 = "m:U1l:bus1", "kV, Line-Line Positive-Sequence-Voltage, Magnitude"
      m_phiu1_bus1 = "m:phiu1:bus1", "deg, Positive-Sequence-Voltage, Angle"
      m_inet_bus1 = "m:inet:bus1", "p.u., Current, Magnitude, referred to network"
      m_I_bus1 = "m:I:bus1", "kA, Current, Magnitude"
      m_phii_bus1 = "m:phii:bus1", "deg, Current, Angle"
      m_i1r_bus1 = "m:i1r:bus1", "p.u., Positive-Sequence Current, Real Part"
      m_i1i_bus1 = "m:i1i:bus1", "p.u., Positive-Sequence Current, Imaginary Part"
      m_i1_bus1 = "m:i1:bus1", "p.u., Positive-Sequence Current, Magnitude"
      m_I1_bus1 = "m:I1:bus1", "kA, Positive-Sequence Current, Magnitude"
      m_phii1_bus1 = "m:phii1:bus1", "deg, Positive-Sequence Current, Angle"
      m_P_bus1 = "m:P:bus1", "MW, Active Power"
      m_Q_bus1 = "m:Q:bus1", "Mvar, Reactive Power"
      m_S_bus1 = "m:S:bus1", "MVA, Apparent Power"
      m_cosphi_bus1 = "m:cosphi:bus1", "Power Factor"
      m_tanphi_bus1 = "m:tanphi:bus1", "tan(phi)"
      m_Psum_bus1 = "m:Psum:bus1", "MW, Total Active Power"
      m_Qsum_bus1 = "m:Qsum:bus1", "Mvar, Total Reactive Power"
      m_Ssum_bus1 = "m:Ssum:bus1", "MVA, Total Apparent Power"
      m_cosphisum_bus1 = "m:cosphisum:bus1", "Total Power Factor"
      m_tanphisum_bus1 = "m:tanphisum:bus1", "Total tan(phi)"
      m_Spu_bus1 = "m:Spu:bus1", "MVA/p.u., Apparent Power per p.u. Voltage"
      m_i1P_bus1 = "m:i1P:bus1", "p.u., Positive-Sequence Active Current"
      m_i1Q_bus1 = "m:i1Q:bus1", "p.u., Positive-Sequence Reactive Current"
      m_I1P_bus1 = "m:I1P:bus1", "kA, Positive-Sequence Active Current"
      m_I1Q_bus1 = "m:I1Q:bus1", "kA, Positive-Sequence Reactive Current"
      m_ur_bus1 = "m:ur:bus1", "p.u., Voltage, Real Part"
      m_ui_bus1 = "m:ui:bus1", "p.u., Voltage, Imaginary Part"
      m_u_bus1 = "m:u:bus1", "p.u., Voltage, Magnitude"
      m_ir_bus1 = "m:ir:bus1", "p.u., Current, Real Part"
      m_ii_bus1 = "m:ii:bus1", "p.u., Current, Imaginary Part"
      m_i_bus1 = "m:i:bus1", "p.u., Current, Magnitude"
      m_phiui_bus1 = "m:phiui:bus1", "deg, Angle between Voltage and Current"
      m_phiu1i1_bus1 = "m:phiu1i1:bus1", "deg, Angle between Voltage and Current in positive sequence system"
      m_Tfct_bus1 = "m:Tfct:bus1", "s, Fault Clearing Time"
      m_Brkload_bus1 = "m:Brkload:bus1", "%, Breaker Loading"
      n_ur_bus1 = "n:ur:bus1", "p.u., Voltage, Real Part"
      n_ui_bus1 = "n:ui:bus1", "p.u., Voltage, Imaginary Part"
      n_u_bus1 = "n:u:bus1", "p.u., Voltage, Magnitude"
      n_upc_bus1 = "n:upc:bus1", "%, Voltage, Magnitude"
      n_u1_bus1 = "n:u1:bus1", "p.u., Voltage, Magnitude"
      n_u1pc_bus1 = "n:u1pc:bus1", "%, Voltage, Magnitude"
      n_u1r_bus1 = "n:u1r:bus1", "p.u., Positive-Sequence Voltage, Real Part"
      n_u1i_bus1 = "n:u1i:bus1", "p.u., Positive-Sequence Voltage, Imaginary Part"
      n_U_bus1 = "n:U:bus1", "kV, Line-Ground Voltage, Magnitude"
      n_Ul_bus1 = "n:Ul:bus1", "kV, Line-Line Voltage, Magnitude"
      n_phiu_bus1 = "n:phiu:bus1", "deg, Voltage, Angle"
      n_phiurel_bus1 = "n:phiurel:bus1", "deg, Voltage, Relative Angle"
      n_du_bus1 = "n:du:bus1", "%, Voltage Deviation"
      n_ishcr_bus1 = "n:ishcr:bus1", "p.u., Short-Circuit Current, real"
      n_ishci_bus1 = "n:ishci:bus1", "p.u., Short-Circuit Current, imag"
      n_ishc_bus1 = "n:ishc:bus1", "p.u., Short-Circuit Current, magnitude"
      n_Ishc_bus1 = "n:Ishc:bus1", "kA, Short-Circuit Current, magnitude"
      n_fe_bus1 = "n:fe:bus1", "p.u., Electrical Frequency"
      n_frdev_bus1 = "n:frdev:bus1", "Hz, average frequency"
      n_dfehz_bus1 = "n:dfehz:bus1", "Hz, Deviation of the El. Frequency"
      n_dfedt_bus1 = "n:dfedt:bus1", "p.u./s, Derivative of El. Frequency"
      n_fehz_bus1 = "n:fehz:bus1", "Hz, Electrical Frequency"
      n_frnom_bus1 = "n:frnom:bus1", "Hz, Nominal Frequency"
      s_pt = "s:pt", "p.u., IN, Turbine Power, continuous, double"
      s_xmdm = "s:xmdm", "p.u., IN, MDM-Torque, continuous, double"
      s_rradd = "s:rradd", "p.u., IN, Additional Rotor Resistance, continuous, double"
      s_bcomp = "s:bcomp", "p.u., IN, Shunt for Compensation, continuous, double"
      s_xspeed = "s:xspeed", "p.u., OUT, Speed, continuous, double"
      s_pgt = "s:pgt", "p.u., OUT, Active power, continuous, double"
      s_xme = "s:xme", "p.u., OUT, Electrical Torque, continuous, double"
      s_irot = "s:irot", "p.u., OUT, Rotor Current, continuous, double"
      s_speed = "s:speed", "p.u., STATE, Speed, continuous, double"
      s_psiA1_r = "s:psiA1_r", "p.u., STATE, Flux, Loop A1, Real Part, continuous, double"
      s_psiA1_i = "s:psiA1_i", "p.u., STATE, Flux, Loop A1, Imaginary Part, continuous, double"
      s_psiA2_r = "s:psiA2_r", "p.u., STATE, Flux, Loop A2, Real Part, continuous, double"
      s_psiA2_i = "s:psiA2_i", "p.u., STATE, Flux, Loop A2, Imaginary Part, continuous, double"
      s_psiB_r = "s:psiB_r", "p.u., STATE, Flux, Loop B, Real Part continuous, double"
      s_psiB_i = "s:psiB_i", "p.u., STATE, Flux, Loop B, Imaginary Part, continuous, double"
      s_speed_dt = "s:speed:dt", "p.u./s, d/dt, Speed, derivative, continuous, double"
      s_psiA1_r_dt = "s:psiA1_r:dt", "p.u./s, d/dt, Flux, Loop A1, Real Part, derivative, continuous, double"
      s_psiA1_i_dt = "s:psiA1_i:dt", "p.u./s, d/dt, Flux, Loop A1, Imaginary Part, derivative, continuous, double"
      s_psiA2_r_dt = "s:psiA2_r:dt", "p.u./s, d/dt, Flux, Loop A2, Real Part, derivative, continuous, double"
      s_psiA2_i_dt = "s:psiA2_i:dt", "p.u./s, d/dt, Flux, Loop A2, Imaginary Part, derivative, continuous, double"
      s_psiB_r_dt = "s:psiB_r:dt", "p.u./s, d/dt, Flux, Loop B, Real Part, derivative, continuous, double"
      s_psiB_i_dt = "s:psiB_i:dt", "p.u./s, d/dt, Flux, Loop B, Imaginary Part, derivative, continuous, double"
      c_cosn = "c:cosn", "Rated Power Factor"
      c_rstr = "c:rstr", "p.u., Stator Resistance"
      c_xstr = "c:xstr", "p.u., Stator Reactance"
      c_rrtrA = "c:rrtrA", "p.u., Rotor Resistance A"
      c_rrtrB = "c:rrtrB", "p.u., Rotor Resistance B"
      c_xrtrA = "c:xrtrA", "p.u., Rotor Reactance A"
      c_xrtrB = "c:xrtrB", "p.u., Rotor Reactance B"
      c_xm = "c:xm", "p.u., Magnetising Reactance"
      c_xmrtr = "c:xmrtr", "p.u., Rotor Leakage Reactance"
      c_frnom = "c:frnom", "Hz, Nominal Frequency"
      c_i_mot = "c:i_mot", "i_mot=0"
      c_xmem = "c:xmem", "p.u., Electrical Torque"
      c_xmt = "c:xmt", "p.u., Mechanical Torque"
      c_slip = "c:slip", "%, Slip"
      c_tag = "c:tag", "s, Acceleration Time Constant"
      c_i_star = "c:i_star", "Star/Delta"
      c_i_auto = "c:i_auto", "Auto Transformer"
      c_xss = "c:xss", "p.u., Subtransient Reactance"
      c_addmt = "c:addmt", "p.u., Additional Torque"
      c_fe = "c:fe", "Electrical Frequency"
      c_xradd = "c:xradd", "p.u., Additional Rotor Reactance"
      c_xstadd = "c:xstadd", "p.u., Additional Stator Reactance"
      c_psiM = "c:psiM", "p.u., Magnetising Flux"
      c_ksat = "c:ksat", "Saturation Factor"
      c_xmsat = "c:xmsat", "p.u., Saturated Magnetising Reactance"
      c_iTrigg = "c:iTrigg", "Triggered by"
      c_Tyd = "c:Tyd", "s, Y - D switching time"
      c_speedyd = "c:speedyd", "s, Y - D switching speed"
      c_xreac = "c:xreac", "p.u., Reactor Reactance"
      c_Tbypass = "c:Tbypass", "p.u., Bypass time"
      c_speedbyp = "c:speedbyp", "p.u., Bypass speed"
      c_autotap = "c:autotap", "p.u., Auto Transformer Tap"
      c_Tstaroff = "c:Tstaroff", "p.u., Star contactor off time"
      c_speedsoff = "c:speedsoff", "p.u., Star contactor off speed"
      e_iAstabint = "e:iAstabint", "A-stable integration algorithm"
      e_c_pmdm = "e:c_pmdm", "Mdm"
      e_mdmlp = "e:mdmlp", "p.u., Proportional factor"
      e_mdmex = "e:mdmex", "Exponent"
      e_Jme = "e:Jme", "kgm^2, Moment of Inertia"
      e_gratio = "e:gratio", "Gear ratio"
      e_cJ = "e:cJ", "kgm^2, Moment of Inertia from Type"
      e_cJtot = "e:cJtot", "kgm^2, Total Moment of Inertia"
      e_ctagtot = "e:ctagtot", "s, Total Acceleration Time Const."
      e_iUseStart = "e:iUseStart", "Use motor starting method"
      e_iStartMethod = "e:iStartMethod", "Motor starting method"
      e_iTrigg = "e:iTrigg", "Triggered by..."
      e_Tyd = "e:Tyd", "s, Switch to 'D' after"
      e_speedyd = "e:speedyd", "p.u., Switch to 'D' at speed >="
      e_ctime = "e:ctime", "s, Time"
      e_crotor = "e:crotor", "p.u., Additional rotor resistance"
      e_cspeed = "e:cspeed", "p.u., Speed"
      e_ratedStr = "e:ratedStr", "kVA, Rated apparent power"
      e_xrea = "e:xrea", "%, Reactance"
      e_Tbypass = "e:Tbypass", "s, Bypass after"
      e_speedbyp = "e:speedbyp", "p.u., Bypass at speed >="
      e_autotap = "e:autotap", "%, Tap"
      e_Tstaroff = "e:Tstaroff", "s, Release star contactor after"
      e_speedsoff = "e:speedsoff", "p.u., Release star contactor at speed >="
      e_speed1ph = "e:speed1ph", "p.u., Switch at speed >="
      e_T1ph = "e:T1ph", "s, Switch after"
      r_c_pmdm = "r:c_pmdm", "Mdm"

    class ElmAsmsc(Enum):
      _init_ = 'value __doc__'

      m_u1r_bus1 = "m:u1r:bus1", "p.u., Positive-Sequence-Voltage, Real Part"
      m_u1r_bus2 = "m:u1r:bus2", "p.u., Positive-Sequence-Voltage, Real Part"
      m_u1i_bus1 = "m:u1i:bus1", "p.u., Positive-Sequence-Voltage, Imaginary Part"
      m_u1i_bus2 = "m:u1i:bus2", "p.u., Positive-Sequence-Voltage, Imaginary Part"
      m_u1_bus1 = "m:u1:bus1", "p.u., Positive-Sequence-Voltage, Magnitude"
      m_u1_bus2 = "m:u1:bus2", "p.u., Positive-Sequence-Voltage, Magnitude"
      m_U1_bus1 = "m:U1:bus1", "kV, Line-Ground Positive-Sequence-Voltage, Magnitude"
      m_U1_bus2 = "m:U1:bus2", "kV, Line-Ground Positive-Sequence-Voltage, Magnitude"
      m_U1l_bus1 = "m:U1l:bus1", "kV, Line-Line Positive-Sequence-Voltage, Magnitude"
      m_U1l_bus2 = "m:U1l:bus2", "kV, Line-Line Positive-Sequence-Voltage, Magnitude"
      m_phiu1_bus1 = "m:phiu1:bus1", "deg, Positive-Sequence-Voltage, Angle"
      m_phiu1_bus2 = "m:phiu1:bus2", "deg, Positive-Sequence-Voltage, Angle"
      m_inet_bus1 = "m:inet:bus1", "p.u., Current, Magnitude, referred to network"
      m_inet_bus2 = "m:inet:bus2", "p.u., Current, Magnitude, referred to network"
      m_I_bus1 = "m:I:bus1", "kA, Current, Magnitude"
      m_I_bus2 = "m:I:bus2", "kA, Current, Magnitude"
      m_phii_bus1 = "m:phii:bus1", "deg, Current, Angle"
      m_phii_bus2 = "m:phii:bus2", "deg, Current, Angle"
      m_i1r_bus1 = "m:i1r:bus1", "p.u., Positive-Sequence Current, Real Part"
      m_i1r_bus2 = "m:i1r:bus2", "p.u., Positive-Sequence Current, Real Part"
      m_i1i_bus1 = "m:i1i:bus1", "p.u., Positive-Sequence Current, Imaginary Part"
      m_i1i_bus2 = "m:i1i:bus2", "p.u., Positive-Sequence Current, Imaginary Part"
      m_i1_bus1 = "m:i1:bus1", "p.u., Positive-Sequence Current, Magnitude"
      m_i1_bus2 = "m:i1:bus2", "p.u., Positive-Sequence Current, Magnitude"
      m_I1_bus1 = "m:I1:bus1", "kA, Positive-Sequence Current, Magnitude"
      m_I1_bus2 = "m:I1:bus2", "kA, Positive-Sequence Current, Magnitude"
      m_phii1_bus1 = "m:phii1:bus1", "deg, Positive-Sequence Current, Angle"
      m_phii1_bus2 = "m:phii1:bus2", "deg, Positive-Sequence Current, Angle"
      m_P_bus1 = "m:P:bus1", "MW, Active Power"
      m_P_bus2 = "m:P:bus2", "MW, Active Power"
      m_Q_bus1 = "m:Q:bus1", "Mvar, Reactive Power"
      m_Q_bus2 = "m:Q:bus2", "Mvar, Reactive Power"
      m_S_bus1 = "m:S:bus1", "MVA, Apparent Power"
      m_S_bus2 = "m:S:bus2", "MVA, Apparent Power"
      m_cosphi_bus1 = "m:cosphi:bus1", "Power Factor"
      m_cosphi_bus2 = "m:cosphi:bus2", "Power Factor"
      m_tanphi_bus1 = "m:tanphi:bus1", "tan(phi)"
      m_tanphi_bus2 = "m:tanphi:bus2", "tan(phi)"
      m_Psum_bus1 = "m:Psum:bus1", "MW, Total Active Power"
      m_Psum_bus2 = "m:Psum:bus2", "MW, Total Active Power"
      m_Qsum_bus1 = "m:Qsum:bus1", "Mvar, Total Reactive Power"
      m_Qsum_bus2 = "m:Qsum:bus2", "Mvar, Total Reactive Power"
      m_Ssum_bus1 = "m:Ssum:bus1", "MVA, Total Apparent Power"
      m_Ssum_bus2 = "m:Ssum:bus2", "MVA, Total Apparent Power"
      m_cosphisum_bus1 = "m:cosphisum:bus1", "Total Power Factor"
      m_cosphisum_bus2 = "m:cosphisum:bus2", "Total Power Factor"
      m_tanphisum_bus1 = "m:tanphisum:bus1", "Total tan(phi)"
      m_tanphisum_bus2 = "m:tanphisum:bus2", "Total tan(phi)"
      m_Spu_bus1 = "m:Spu:bus1", "MVA/p.u., Apparent Power per p.u. Voltage"
      m_Spu_bus2 = "m:Spu:bus2", "MVA/p.u., Apparent Power per p.u. Voltage"
      m_i1P_bus1 = "m:i1P:bus1", "p.u., Positive-Sequence Active Current"
      m_i1P_bus2 = "m:i1P:bus2", "p.u., Positive-Sequence Active Current"
      m_i1Q_bus1 = "m:i1Q:bus1", "p.u., Positive-Sequence Reactive Current"
      m_i1Q_bus2 = "m:i1Q:bus2", "p.u., Positive-Sequence Reactive Current"
      m_I1P_bus1 = "m:I1P:bus1", "kA, Positive-Sequence Active Current"
      m_I1P_bus2 = "m:I1P:bus2", "kA, Positive-Sequence Active Current"
      m_I1Q_bus1 = "m:I1Q:bus1", "kA, Positive-Sequence Reactive Current"
      m_I1Q_bus2 = "m:I1Q:bus2", "kA, Positive-Sequence Reactive Current"
      m_ur_bus1 = "m:ur:bus1", "p.u., Voltage, Real Part"
      m_ur_bus2 = "m:ur:bus2", "p.u., Voltage, Real Part"
      m_ui_bus1 = "m:ui:bus1", "p.u., Voltage, Imaginary Part"
      m_ui_bus2 = "m:ui:bus2", "p.u., Voltage, Imaginary Part"
      m_u_bus1 = "m:u:bus1", "p.u., Voltage, Magnitude"
      m_u_bus2 = "m:u:bus2", "p.u., Voltage, Magnitude"
      m_ir_bus1 = "m:ir:bus1", "p.u., Current, Real Part"
      m_ir_bus2 = "m:ir:bus2", "p.u., Current, Real Part"
      m_ii_bus1 = "m:ii:bus1", "p.u., Current, Imaginary Part"
      m_ii_bus2 = "m:ii:bus2", "p.u., Current, Imaginary Part"
      m_i_bus1 = "m:i:bus1", "p.u., Current, Magnitude"
      m_i_bus2 = "m:i:bus2", "p.u., Current, Magnitude"
      m_phiui_bus1 = "m:phiui:bus1", "deg, Angle between Voltage and Current"
      m_phiui_bus2 = "m:phiui:bus2", "deg, Angle between Voltage and Current"
      m_phiu1i1_bus1 = "m:phiu1i1:bus1", "deg, Angle between Voltage and Current in positive sequence system"
      m_phiu1i1_bus2 = "m:phiu1i1:bus2", "deg, Angle between Voltage and Current in positive sequence system"
      m_Tfct_bus1 = "m:Tfct:bus1", "s, Fault Clearing Time"
      m_Tfct_bus2 = "m:Tfct:bus2", "s, Fault Clearing Time"
      m_Brkload_bus1 = "m:Brkload:bus1", "%, Breaker Loading"
      m_Brkload_bus2 = "m:Brkload:bus2", "%, Breaker Loading"
      n_ur_bus1 = "n:ur:bus1", "p.u., Voltage, Real Part"
      n_ur_bus2 = "n:ur:bus2", "p.u., Voltage, Real Part"
      n_ui_bus1 = "n:ui:bus1", "p.u., Voltage, Imaginary Part"
      n_ui_bus2 = "n:ui:bus2", "p.u., Voltage, Imaginary Part"
      n_u_bus1 = "n:u:bus1", "p.u., Voltage, Magnitude"
      n_u_bus2 = "n:u:bus2", "p.u., Voltage, Magnitude"
      n_upc_bus1 = "n:upc:bus1", "%, Voltage, Magnitude"
      n_upc_bus2 = "n:upc:bus2", "%, Voltage, Magnitude"
      n_u1_bus1 = "n:u1:bus1", "p.u., Voltage, Magnitude"
      n_u1_bus2 = "n:u1:bus2", "p.u., Voltage, Magnitude"
      n_u1pc_bus1 = "n:u1pc:bus1", "%, Voltage, Magnitude"
      n_u1pc_bus2 = "n:u1pc:bus2", "%, Voltage, Magnitude"
      n_u1r_bus1 = "n:u1r:bus1", "p.u., Positive-Sequence Voltage, Real Part"
      n_u1r_bus2 = "n:u1r:bus2", "p.u., Positive-Sequence Voltage, Real Part"
      n_u1i_bus1 = "n:u1i:bus1", "p.u., Positive-Sequence Voltage, Imaginary Part"
      n_u1i_bus2 = "n:u1i:bus2", "p.u., Positive-Sequence Voltage, Imaginary Part"
      n_U_bus1 = "n:U:bus1", "kV, Line-Ground Voltage, Magnitude"
      n_U_bus2 = "n:U:bus2", "kV, Line-Ground Voltage, Magnitude"
      n_Ul_bus1 = "n:Ul:bus1", "kV, Line-Line Voltage, Magnitude"
      n_Ul_bus2 = "n:Ul:bus2", "kV, Line-Line Voltage, Magnitude"
      n_phiu_bus1 = "n:phiu:bus1", "deg, Voltage, Angle"
      n_phiu_bus2 = "n:phiu:bus2", "deg, Voltage, Angle"
      n_phiurel_bus1 = "n:phiurel:bus1", "deg, Voltage, Relative Angle"
      n_phiurel_bus2 = "n:phiurel:bus2", "deg, Voltage, Relative Angle"
      n_du_bus1 = "n:du:bus1", "%, Voltage Deviation"
      n_du_bus2 = "n:du:bus2", "%, Voltage Deviation"
      n_ishcr_bus1 = "n:ishcr:bus1", "p.u., Short-Circuit Current, real"
      n_ishcr_bus2 = "n:ishcr:bus2", "p.u., Short-Circuit Current, real"
      n_ishci_bus1 = "n:ishci:bus1", "p.u., Short-Circuit Current, imag"
      n_ishci_bus2 = "n:ishci:bus2", "p.u., Short-Circuit Current, imag"
      n_ishc_bus1 = "n:ishc:bus1", "p.u., Short-Circuit Current, magnitude"
      n_ishc_bus2 = "n:ishc:bus2", "p.u., Short-Circuit Current, magnitude"
      n_Ishc_bus1 = "n:Ishc:bus1", "kA, Short-Circuit Current, magnitude"
      n_Ishc_bus2 = "n:Ishc:bus2", "kA, Short-Circuit Current, magnitude"
      n_fe_bus1 = "n:fe:bus1", "p.u., Electrical Frequency"
      n_fe_bus2 = "n:fe:bus2", "p.u., Electrical Frequency"
      n_frdev_bus1 = "n:frdev:bus1", "Hz, average frequency"
      n_frdev_bus2 = "n:frdev:bus2", "Hz, average frequency"
      n_dfehz_bus1 = "n:dfehz:bus1", "Hz, Deviation of the El. Frequency"
      n_dfehz_bus2 = "n:dfehz:bus2", "Hz, Deviation of the El. Frequency"
      n_dfedt_bus1 = "n:dfedt:bus1", "p.u./s, Derivative of El. Frequency"
      n_dfedt_bus2 = "n:dfedt:bus2", "p.u./s, Derivative of El. Frequency"
      n_fehz_bus1 = "n:fehz:bus1", "Hz, Electrical Frequency"
      n_fehz_bus2 = "n:fehz:bus2", "Hz, Electrical Frequency"
      n_frnom_bus1 = "n:frnom:bus1", "Hz, Nominal Frequency"
      n_frnom_bus2 = "n:frnom:bus2", "Hz, Nominal Frequency"
      s_pt = "s:pt", "p.u., IN, Turbine Power, continuous, double"
      s_xmdm = "s:xmdm", "p.u., IN, MDM-Torque, continuous, double"
      s_rradd = "s:rradd", "p.u., IN, Additional Rotor Resistance, continuous, double"
      s_Pmd = "s:Pmd", "IN, PWM Amplification, Real Part, continuous, double"
      s_Pmq = "s:Pmq", "IN, PWM Amplification, Imaginary Part, continuous, double"
      s_xspeed = "s:xspeed", "p.u., OUT, Speed, continuous, double"
      s_pgt = "s:pgt", "p.u., OUT, Active power, continuous, double"
      s_xme = "s:xme", "p.u., OUT, Electrical Torque, continuous, double"
      s_ird = "s:ird", "p.u., OUT, Rotor Current, d-Axis, continuous, double"
      s_irq = "s:irq", "p.u., OUT, Rotor Current, q-Axis, continuous, double"
      s_xphim = "s:xphim", "rad, OUT, Rotor Angle, continuous, double"
      s_psis_r = "s:psis_r", "p.u., OUT, Stator Flux, Real Part, continuous, double"
      s_psis_i = "s:psis_i", "p.u., OUT, Stator Flux, Imaginary Part, continuous, double"
      s_cosphim = "s:cosphim", "OUT, cos(phim), continuous, double"
      s_sinphim = "s:sinphim", "OUT, sin(phim), continuous, double"
      s_psir_r = "s:psir_r", "p.u., OUT, Rotor Flux, Real Part, continuous, double"
      s_psir_i = "s:psir_i", "p.u., OUT, Rotor Flux, Imaginary Part, continuous, double"
      s_speed = "s:speed", "STATE, Speed, continuous, double"
      s_psiA1_r = "s:psiA1_r", "STATE, Flux, Loop A, Real Part continuous, double"
      s_psiA1_i = "s:psiA1_i", "STATE, Flux, Loop A, Imaginary Part, continuous, double"
      s_psiA2_r = "s:psiA2_r", "STATE, Flux, Loop A, Real Part continuous, double"
      s_psiA2_i = "s:psiA2_i", "STATE, Flux, Loop A, Imaginary Part, continuous, double"
      s_psiB_r = "s:psiB_r", "STATE, Flux, Loop B, Real Part continuous, double"
      s_psiB_i = "s:psiB_i", "STATE, Flux, Loop B, Imaginary Part, continuous, double"
      s_speed_dt = "s:speed:dt", "/s, d/dt, Speed, derivative, continuous, double"
      s_psiA1_r_dt = "s:psiA1_r:dt", "/s, d/dt, Flux, Loop A, Real Part, derivative, continuous, double"
      s_psiA1_i_dt = "s:psiA1_i:dt", "/s, d/dt, Flux, Loop A, Imaginary Part, derivative, continuous, double"
      s_psiA2_r_dt = "s:psiA2_r:dt", "/s, d/dt, Flux, Loop A, Real Part, derivative, continuous, double"
      s_psiA2_i_dt = "s:psiA2_i:dt", "/s, d/dt, Flux, Loop A, Imaginary Part, derivative, continuous, double"
      s_psiB_r_dt = "s:psiB_r:dt", "/s, d/dt, Flux, Loop B, Real Part, derivative, continuous, double"
      s_psiB_i_dt = "s:psiB_i:dt", "/s, d/dt, Flux, Loop B, Imaginary Part, derivative, continuous, double"
      c_cosn = "c:cosn", "Rated Power Factor"
      c_p = "c:p", "MW, Active Power (Electrical)"
      c_rstr = "c:rstr", "p.u., Stator Resistance"
      c_xstr = "c:xstr", "p.u., Stator Reactance"
      c_rrtrA = "c:rrtrA", "p.u., Rotor Resistance A"
      c_rrtrB = "c:rrtrB", "p.u., Rotor Resistance B"
      c_xrtrA = "c:xrtrA", "p.u., Rotor Reactance A"
      c_xrtrB = "c:xrtrB", "p.u., Rotor Reactance B"
      c_xm = "c:xm", "p.u., Magnetising Reactance"
      c_xmrtr = "c:xmrtr", "p.u., Rotor Leakage Reactance"
      c_frnom = "c:frnom", "Hz, Nominal Frequency"
      c_i_mot = "c:i_mot", "i_mot=0"
      c_xmem = "c:xmem", "p.u., Electrical Torque"
      c_xmt = "c:xmt", "p.u., Mechanical Torque"
      c_slip = "c:slip", "%, Slip"
      c_tag = "c:tag", "s, Acceleration Time Constant"
      c_i_star = "c:i_star", "Star/Delta"
      c_xss = "c:xss", "p.u., Subtransient Reactance"
      c_addmt = "c:addmt", "p.u., Additional Torque"
      c_fe = "c:fe", "Electrical Frequency"
      c_Pmr = "c:Pmr", "Amplification, Real Part"
      c_Pmi = "c:Pmi", "Amplification, Imaginary Part"
      c_usr = "c:usr", "p.u., Rotor Voltage, Real Part"
      c_usi = "c:usi", "p.u., Rotor Voltage, Imaginary Part"
      c_Unrot = "c:Unrot", "kV, Rated Slip Ring Voltage"
      c_urd = "c:urd", "p.u., Rotor Voltage, d-Axis"
      c_urq = "c:urq", "p.u., Rotor Voltage, q-Axis"
      c_ura = "c:ura", "p.u., Rotor Voltage, Phase a"
      c_urb = "c:urb", "p.u., Rotor Voltage, Phase b"
      c_urc = "c:urc", "p.u., Rotor Voltage, Phase c"
      c_Ird = "c:Ird", "kA, Rotor Current, d-Axis"
      c_Irq = "c:Irq", "kA, Rotor Current, q-Axis"
      c_Ira = "c:Ira", "kA, Rotor Current, Phase a"
      c_Irb = "c:Irb", "kA, Rotor Current, Phase b"
      c_Irc = "c:Irc", "kA, Rotor Current, Phase c"
      c_xradd = "c:xradd", "p.u., Additional Rotor Reactance"
      c_xstadd = "c:xstadd", "p.u., Additional Stator Reactance"
      c_Irot = "c:Irot", "kA, Rotor Current, Magnitude of Phasor"
      c_rcrow = "c:rcrow", "Crow-Bar Resistance"
      c_xcrow = "c:xcrow", "Crow-Bar Reactance"
      c_set_bypass = "c:set_bypass", "Set Rotor-Bypass"
      c_reset_bypass = "c:reset_bypass", "Reset Rotor-Bypass"
      c_psiM = "c:psiM", "p.u., Magnetising flux"
      c_ksat = "c:ksat", "Saturation factor"
      c_xmsat = "c:xmsat", "p.u., Saturated magnetising reactance"
      e_iAstabint = "e:iAstabint", "A-stable integration algorithm"
      e_rcrow = "e:rcrow", "p.u., Crow-Bar Resistance"
      e_xcrow = "e:xcrow", "p.u., Crow-Bar Reactance"
      e_mdmlp = "e:mdmlp", "p.u., Proportional Factor"
      e_mdmex = "e:mdmex", "Exponent"

    class ElmBoundary(Enum):
      _init_ = 'value __doc__'

      s_Pinter = "s:Pinter", "MW, OUT, Active Power Interchange, continuous, double"
      s_Qinter = "s:Qinter", "Mvar, OUT, Reactive Power Interchange, continuous, double"

    class ElmCoup(Enum):
      _init_ = 'value __doc__'

      m_u1r_bus1 = "m:u1r:bus1", "p.u., Positive-Sequence-Voltage, Real Part"
      m_u1r_bus2 = "m:u1r:bus2", "p.u., Positive-Sequence-Voltage, Real Part"
      m_u1i_bus1 = "m:u1i:bus1", "p.u., Positive-Sequence-Voltage, Imaginary Part"
      m_u1i_bus2 = "m:u1i:bus2", "p.u., Positive-Sequence-Voltage, Imaginary Part"
      m_u1_bus1 = "m:u1:bus1", "p.u., Positive-Sequence-Voltage, Magnitude"
      m_u1_bus2 = "m:u1:bus2", "p.u., Positive-Sequence-Voltage, Magnitude"
      m_U1_bus1 = "m:U1:bus1", "kV, Line-Ground Positive-Sequence-Voltage, Magnitude"
      m_U1_bus2 = "m:U1:bus2", "kV, Line-Ground Positive-Sequence-Voltage, Magnitude"
      m_U1l_bus1 = "m:U1l:bus1", "kV, Line-Line Positive-Sequence-Voltage, Magnitude"
      m_U1l_bus2 = "m:U1l:bus2", "kV, Line-Line Positive-Sequence-Voltage, Magnitude"
      m_phiu1_bus1 = "m:phiu1:bus1", "deg, Positive-Sequence-Voltage, Angle"
      m_phiu1_bus2 = "m:phiu1:bus2", "deg, Positive-Sequence-Voltage, Angle"
      m_inet_bus1 = "m:inet:bus1", "p.u., Current, Magnitude, referred to network"
      m_inet_bus2 = "m:inet:bus2", "p.u., Current, Magnitude, referred to network"
      m_I_bus1 = "m:I:bus1", "kA, Current, Magnitude"
      m_I_bus2 = "m:I:bus2", "kA, Current, Magnitude"
      m_phii_bus1 = "m:phii:bus1", "deg, Current, Angle"
      m_phii_bus2 = "m:phii:bus2", "deg, Current, Angle"
      m_i1r_bus1 = "m:i1r:bus1", "p.u., Positive-Sequence Current, Real Part"
      m_i1r_bus2 = "m:i1r:bus2", "p.u., Positive-Sequence Current, Real Part"
      m_i1i_bus1 = "m:i1i:bus1", "p.u., Positive-Sequence Current, Imaginary Part"
      m_i1i_bus2 = "m:i1i:bus2", "p.u., Positive-Sequence Current, Imaginary Part"
      m_i1_bus1 = "m:i1:bus1", "p.u., Positive-Sequence Current, Magnitude"
      m_i1_bus2 = "m:i1:bus2", "p.u., Positive-Sequence Current, Magnitude"
      m_I1_bus1 = "m:I1:bus1", "kA, Positive-Sequence Current, Magnitude"
      m_I1_bus2 = "m:I1:bus2", "kA, Positive-Sequence Current, Magnitude"
      m_phii1_bus1 = "m:phii1:bus1", "deg, Positive-Sequence Current, Angle"
      m_phii1_bus2 = "m:phii1:bus2", "deg, Positive-Sequence Current, Angle"
      m_P_bus1 = "m:P:bus1", "MW, Active Power"
      m_P_bus2 = "m:P:bus2", "MW, Active Power"
      m_Q_bus1 = "m:Q:bus1", "Mvar, Reactive Power"
      m_Q_bus2 = "m:Q:bus2", "Mvar, Reactive Power"
      m_S_bus1 = "m:S:bus1", "MVA, Apparent Power"
      m_S_bus2 = "m:S:bus2", "MVA, Apparent Power"
      m_cosphi_bus1 = "m:cosphi:bus1", "Power Factor"
      m_cosphi_bus2 = "m:cosphi:bus2", "Power Factor"
      m_tanphi_bus1 = "m:tanphi:bus1", "tan(phi)"
      m_tanphi_bus2 = "m:tanphi:bus2", "tan(phi)"
      m_Psum_bus1 = "m:Psum:bus1", "MW, Total Active Power"
      m_Psum_bus2 = "m:Psum:bus2", "MW, Total Active Power"
      m_Qsum_bus1 = "m:Qsum:bus1", "Mvar, Total Reactive Power"
      m_Qsum_bus2 = "m:Qsum:bus2", "Mvar, Total Reactive Power"
      m_Ssum_bus1 = "m:Ssum:bus1", "MVA, Total Apparent Power"
      m_Ssum_bus2 = "m:Ssum:bus2", "MVA, Total Apparent Power"
      m_cosphisum_bus1 = "m:cosphisum:bus1", "Total Power Factor"
      m_cosphisum_bus2 = "m:cosphisum:bus2", "Total Power Factor"
      m_tanphisum_bus1 = "m:tanphisum:bus1", "Total tan(phi)"
      m_tanphisum_bus2 = "m:tanphisum:bus2", "Total tan(phi)"
      m_Spu_bus1 = "m:Spu:bus1", "MVA/p.u., Apparent Power per p.u. Voltage"
      m_Spu_bus2 = "m:Spu:bus2", "MVA/p.u., Apparent Power per p.u. Voltage"
      m_i1P_bus1 = "m:i1P:bus1", "p.u., Positive-Sequence Active Current"
      m_i1P_bus2 = "m:i1P:bus2", "p.u., Positive-Sequence Active Current"
      m_i1Q_bus1 = "m:i1Q:bus1", "p.u., Positive-Sequence Reactive Current"
      m_i1Q_bus2 = "m:i1Q:bus2", "p.u., Positive-Sequence Reactive Current"
      m_I1P_bus1 = "m:I1P:bus1", "kA, Positive-Sequence Active Current"
      m_I1P_bus2 = "m:I1P:bus2", "kA, Positive-Sequence Active Current"
      m_I1Q_bus1 = "m:I1Q:bus1", "kA, Positive-Sequence Reactive Current"
      m_I1Q_bus2 = "m:I1Q:bus2", "kA, Positive-Sequence Reactive Current"
      m_ur_bus1 = "m:ur:bus1", "p.u., Voltage, Real Part"
      m_ur_bus2 = "m:ur:bus2", "p.u., Voltage, Real Part"
      m_ui_bus1 = "m:ui:bus1", "p.u., Voltage, Imaginary Part"
      m_ui_bus2 = "m:ui:bus2", "p.u., Voltage, Imaginary Part"
      m_u_bus1 = "m:u:bus1", "p.u., Voltage, Magnitude"
      m_u_bus2 = "m:u:bus2", "p.u., Voltage, Magnitude"
      m_ir_bus1 = "m:ir:bus1", "p.u., Current, Real Part"
      m_ir_bus2 = "m:ir:bus2", "p.u., Current, Real Part"
      m_ii_bus1 = "m:ii:bus1", "p.u., Current, Imaginary Part"
      m_ii_bus2 = "m:ii:bus2", "p.u., Current, Imaginary Part"
      m_i_bus1 = "m:i:bus1", "p.u., Current, Magnitude"
      m_i_bus2 = "m:i:bus2", "p.u., Current, Magnitude"
      m_phiui_bus1 = "m:phiui:bus1", "deg, Angle between Voltage and Current"
      m_phiui_bus2 = "m:phiui:bus2", "deg, Angle between Voltage and Current"
      m_phiu1i1_bus1 = "m:phiu1i1:bus1", "deg, Angle between Voltage and Current in positive sequence system"
      m_phiu1i1_bus2 = "m:phiu1i1:bus2", "deg, Angle between Voltage and Current in positive sequence system"
      m_Tfct_bus1 = "m:Tfct:bus1", "s, Fault Clearing Time"
      m_Tfct_bus2 = "m:Tfct:bus2", "s, Fault Clearing Time"
      m_Brkload_bus1 = "m:Brkload:bus1", "%, Breaker Loading"
      m_Brkload_bus2 = "m:Brkload:bus2", "%, Breaker Loading"
      n_ur_bus1 = "n:ur:bus1", "p.u., Voltage, Real Part"
      n_ur_bus2 = "n:ur:bus2", "p.u., Voltage, Real Part"
      n_ui_bus1 = "n:ui:bus1", "p.u., Voltage, Imaginary Part"
      n_ui_bus2 = "n:ui:bus2", "p.u., Voltage, Imaginary Part"
      n_u_bus1 = "n:u:bus1", "p.u., Voltage, Magnitude"
      n_u_bus2 = "n:u:bus2", "p.u., Voltage, Magnitude"
      n_upc_bus1 = "n:upc:bus1", "%, Voltage, Magnitude"
      n_upc_bus2 = "n:upc:bus2", "%, Voltage, Magnitude"
      n_u1_bus1 = "n:u1:bus1", "p.u., Voltage, Magnitude"
      n_u1_bus2 = "n:u1:bus2", "p.u., Voltage, Magnitude"
      n_u1pc_bus1 = "n:u1pc:bus1", "%, Voltage, Magnitude"
      n_u1pc_bus2 = "n:u1pc:bus2", "%, Voltage, Magnitude"
      n_u1r_bus1 = "n:u1r:bus1", "p.u., Positive-Sequence Voltage, Real Part"
      n_u1r_bus2 = "n:u1r:bus2", "p.u., Positive-Sequence Voltage, Real Part"
      n_u1i_bus1 = "n:u1i:bus1", "p.u., Positive-Sequence Voltage, Imaginary Part"
      n_u1i_bus2 = "n:u1i:bus2", "p.u., Positive-Sequence Voltage, Imaginary Part"
      n_U_bus1 = "n:U:bus1", "kV, Line-Ground Voltage, Magnitude"
      n_U_bus2 = "n:U:bus2", "kV, Line-Ground Voltage, Magnitude"
      n_Ul_bus1 = "n:Ul:bus1", "kV, Line-Line Voltage, Magnitude"
      n_Ul_bus2 = "n:Ul:bus2", "kV, Line-Line Voltage, Magnitude"
      n_phiu_bus1 = "n:phiu:bus1", "deg, Voltage, Angle"
      n_phiu_bus2 = "n:phiu:bus2", "deg, Voltage, Angle"
      n_phiurel_bus1 = "n:phiurel:bus1", "deg, Voltage, Relative Angle"
      n_phiurel_bus2 = "n:phiurel:bus2", "deg, Voltage, Relative Angle"
      n_du_bus1 = "n:du:bus1", "%, Voltage Deviation"
      n_du_bus2 = "n:du:bus2", "%, Voltage Deviation"
      n_ishcr_bus1 = "n:ishcr:bus1", "p.u., Short-Circuit Current, real"
      n_ishcr_bus2 = "n:ishcr:bus2", "p.u., Short-Circuit Current, real"
      n_ishci_bus1 = "n:ishci:bus1", "p.u., Short-Circuit Current, imag"
      n_ishci_bus2 = "n:ishci:bus2", "p.u., Short-Circuit Current, imag"
      n_ishc_bus1 = "n:ishc:bus1", "p.u., Short-Circuit Current, magnitude"
      n_ishc_bus2 = "n:ishc:bus2", "p.u., Short-Circuit Current, magnitude"
      n_Ishc_bus1 = "n:Ishc:bus1", "kA, Short-Circuit Current, magnitude"
      n_Ishc_bus2 = "n:Ishc:bus2", "kA, Short-Circuit Current, magnitude"
      n_fe_bus1 = "n:fe:bus1", "p.u., Electrical Frequency"
      n_fe_bus2 = "n:fe:bus2", "p.u., Electrical Frequency"
      n_frdev_bus1 = "n:frdev:bus1", "Hz, average frequency"
      n_frdev_bus2 = "n:frdev:bus2", "Hz, average frequency"
      n_dfehz_bus1 = "n:dfehz:bus1", "Hz, Deviation of the El. Frequency"
      n_dfehz_bus2 = "n:dfehz:bus2", "Hz, Deviation of the El. Frequency"
      n_dfedt_bus1 = "n:dfedt:bus1", "p.u./s, Derivative of El. Frequency"
      n_dfedt_bus2 = "n:dfedt:bus2", "p.u./s, Derivative of El. Frequency"
      n_fehz_bus1 = "n:fehz:bus1", "Hz, Electrical Frequency"
      n_fehz_bus2 = "n:fehz:bus2", "Hz, Electrical Frequency"
      n_frnom_bus1 = "n:frnom:bus1", "Hz, Nominal Frequency"
      n_frnom_bus2 = "n:frnom:bus2", "Hz, Nominal Frequency"
      c_input = "c:input", "Control Input"
      c_R_on = "c:R_on", "Ohm, On Resistance"
      c_X_on = "c:X_on", "Ohm, On Reactance"
      c_t_open = "c:t_open", "s, Opening Time"
      c_i_ctrl = "c:i_ctrl", "Internal"
      c_Inom = "c:Inom", "kA, Rated Current"
      c_loading = "c:loading", "%, Loading"
      c_closed = "c:closed", "Breaker Closed"
      c_locked = "c:locked", "Locked"
      c_IdFailure = "c:IdFailure", "Indication Failure"
      c_Indication = "c:Indication", "Indication"
      c_TCM = "c:TCM", "Trip Coil Monitor"

    class ElmGenstat(Enum):
      _init_ = 'value __doc__'

      m_u1r_bus1 = "m:u1r:bus1", "p.u., Positive-Sequence-Voltage, Real Part"
      m_u1i_bus1 = "m:u1i:bus1", "p.u., Positive-Sequence-Voltage, Imaginary Part"
      m_u1_bus1 = "m:u1:bus1", "p.u., Positive-Sequence-Voltage, Magnitude"
      m_U1_bus1 = "m:U1:bus1", "kV, Line-Ground Positive-Sequence-Voltage, Magnitude"
      m_U1l_bus1 = "m:U1l:bus1", "kV, Line-Line Positive-Sequence-Voltage, Magnitude"
      m_phiu1_bus1 = "m:phiu1:bus1", "deg, Positive-Sequence-Voltage, Angle"
      m_inet_bus1 = "m:inet:bus1", "p.u., Current, Magnitude, referred to network"
      m_I_bus1 = "m:I:bus1", "kA, Current, Magnitude"
      m_phii_bus1 = "m:phii:bus1", "deg, Current, Angle"
      m_i1r_bus1 = "m:i1r:bus1", "p.u., Positive-Sequence Current, Real Part"
      m_i1i_bus1 = "m:i1i:bus1", "p.u., Positive-Sequence Current, Imaginary Part"
      m_i1_bus1 = "m:i1:bus1", "p.u., Positive-Sequence Current, Magnitude"
      m_I1_bus1 = "m:I1:bus1", "kA, Positive-Sequence Current, Magnitude"
      m_phii1_bus1 = "m:phii1:bus1", "deg, Positive-Sequence Current, Angle"
      m_P_bus1 = "m:P:bus1", "MW, Active Power"
      m_Q_bus1 = "m:Q:bus1", "Mvar, Reactive Power"
      m_S_bus1 = "m:S:bus1", "MVA, Apparent Power"
      m_cosphi_bus1 = "m:cosphi:bus1", "Power Factor"
      m_tanphi_bus1 = "m:tanphi:bus1", "tan(phi)"
      m_Psum_bus1 = "m:Psum:bus1", "MW, Total Active Power"
      m_Qsum_bus1 = "m:Qsum:bus1", "Mvar, Total Reactive Power"
      m_Ssum_bus1 = "m:Ssum:bus1", "MVA, Total Apparent Power"
      m_cosphisum_bus1 = "m:cosphisum:bus1", "Total Power Factor"
      m_tanphisum_bus1 = "m:tanphisum:bus1", "Total tan(phi)"
      m_Spu_bus1 = "m:Spu:bus1", "MVA/p.u., Apparent Power per p.u. Voltage"
      m_i1P_bus1 = "m:i1P:bus1", "p.u., Positive-Sequence Active Current"
      m_i1Q_bus1 = "m:i1Q:bus1", "p.u., Positive-Sequence Reactive Current"
      m_I1P_bus1 = "m:I1P:bus1", "kA, Positive-Sequence Active Current"
      m_I1Q_bus1 = "m:I1Q:bus1", "kA, Positive-Sequence Reactive Current"
      m_ur_bus1 = "m:ur:bus1", "p.u., Voltage, Real Part"
      m_ui_bus1 = "m:ui:bus1", "p.u., Voltage, Imaginary Part"
      m_u_bus1 = "m:u:bus1", "p.u., Voltage, Magnitude"
      m_ir_bus1 = "m:ir:bus1", "p.u., Current, Real Part"
      m_ii_bus1 = "m:ii:bus1", "p.u., Current, Imaginary Part"
      m_i_bus1 = "m:i:bus1", "p.u., Current, Magnitude"
      m_phiui_bus1 = "m:phiui:bus1", "deg, Angle between Voltage and Current"
      m_phiu1i1_bus1 = "m:phiu1i1:bus1", "deg, Angle between Voltage and Current in positive sequence system"
      m_Tfct_bus1 = "m:Tfct:bus1", "s, Fault Clearing Time"
      m_Brkload_bus1 = "m:Brkload:bus1", "%, Breaker Loading"
      n_ur_bus1 = "n:ur:bus1", "p.u., Voltage, Real Part"
      n_ui_bus1 = "n:ui:bus1", "p.u., Voltage, Imaginary Part"
      n_u_bus1 = "n:u:bus1", "p.u., Voltage, Magnitude"
      n_upc_bus1 = "n:upc:bus1", "%, Voltage, Magnitude"
      n_u1_bus1 = "n:u1:bus1", "p.u., Voltage, Magnitude"
      n_u1pc_bus1 = "n:u1pc:bus1", "%, Voltage, Magnitude"
      n_u1r_bus1 = "n:u1r:bus1", "p.u., Positive-Sequence Voltage, Real Part"
      n_u1i_bus1 = "n:u1i:bus1", "p.u., Positive-Sequence Voltage, Imaginary Part"
      n_U_bus1 = "n:U:bus1", "kV, Line-Ground Voltage, Magnitude"
      n_Ul_bus1 = "n:Ul:bus1", "kV, Line-Line Voltage, Magnitude"
      n_phiu_bus1 = "n:phiu:bus1", "deg, Voltage, Angle"
      n_phiurel_bus1 = "n:phiurel:bus1", "deg, Voltage, Relative Angle"
      n_du_bus1 = "n:du:bus1", "%, Voltage Deviation"
      n_ishcr_bus1 = "n:ishcr:bus1", "p.u., Short-Circuit Current, real"
      n_ishci_bus1 = "n:ishci:bus1", "p.u., Short-Circuit Current, imag"
      n_ishc_bus1 = "n:ishc:bus1", "p.u., Short-Circuit Current, magnitude"
      n_Ishc_bus1 = "n:Ishc:bus1", "kA, Short-Circuit Current, magnitude"
      n_fe_bus1 = "n:fe:bus1", "p.u., Electrical Frequency"
      n_frdev_bus1 = "n:frdev:bus1", "Hz, average frequency"
      n_dfehz_bus1 = "n:dfehz:bus1", "Hz, Deviation of the El. Frequency"
      n_dfedt_bus1 = "n:dfedt:bus1", "p.u./s, Derivative of El. Frequency"
      n_fehz_bus1 = "n:fehz:bus1", "Hz, Electrical Frequency"
      n_frnom_bus1 = "n:frnom:bus1", "Hz, Nominal Frequency"
      s_f0 = "s:f0", "p.u., IN, Frequency, input, continuous, double"
      s_F0Hz = "s:F0Hz", "Hz, IN, Frequency input (Hz), continuous, double"
      s_fref = "s:fref", "p.u., IN, Reference frequency, continuous, double"
      s_id_ref = "s:id_ref", "p.u., IN, d-axis current reference, continuous, double"
      s_iq_ref = "s:iq_ref", "p.u., IN, q-axis current reference, continuous, double"
      s_u1r_in = "s:u1r_in", "p.u., IN, Voltage input, real part, continuous, double"
      s_u1i_in = "s:u1i_in", "p.u., IN, Voltage input, imaginary part, continuous, double"
      s_cosref = "s:cosref", "IN, cos(dq-reference angle) continuous, double"
      s_sinref = "s:sinref", "IN, sin(dq-reference angle) continuous, double"
      s_xspeed = "s:xspeed", "p.u., OUT, Frequency, continuous, double"
      s_id = "s:id", "p.u., OUT, Current, d-Axis continuous, double"
      s_iq = "s:iq", "p.u., OUT, Current, q-Axis continuous, double"
      c_sgnn = "c:sgnn", "MVA, Total rated apparent power (ngnum*sgn)"
      c_cosn = "c:cosn", "Nominal power factor"
      c_hpi = "c:hpi", "rad/s, Nominal angular frequency"
      c_umin = "c:umin", "p.u., Min. operation voltage"
      c_phiu = "c:phiu", "rad, Voltage angle"
      e_iSimModel = "e:iSimModel", "Model"
      e_iAstabint = "e:iAstabint", "A-stable integration algorithm"
      e_umin = "e:umin", "p.u., Switch-off threshold"
      e_uonthr = "e:uonthr", "p.u., Switch-on threshold"
      e_Tondelay = "e:Tondelay", "s, Switch-on delay"
      e_uk = "e:uk", "%, Short circuit impedance"
      e_Pcu = "e:Pcu", "kW, Copper losses"
      e_R0toR1 = "e:R0toR1", "R0/R1 ratio"
      e_X0toX1 = "e:X0toX1", "X0/X1 ratio"
      e_pinput = "e:pinput", "Active Power signal"
      e_qinput = "e:qinput", "Reactive Power signal"
      e_iDelay = "e:iDelay", "dq-reference angle delay (if dq-reference signals not connected)"
      e_Tdelay = "e:Tdelay", "s, Delay time constant"
      e_iUseNegSeqAngle = "e:iUseNegSeqAngle", "Current source model: Use negative sequence angle for negative sequence current source"
      e_iUseZsr = "e:iUseZsr", "Use series reactor impedance, if negative sequence voltage signals are connected"
      e_r2 = "e:r2", "p.u., Resistance r2"
      e_x2 = "e:x2", "p.u., Reactance x2"

    class ElmIac(Enum):
      _init_ = 'value __doc__'

      m_u1r_bus1 = "m:u1r:bus1", "p.u., Positive-Sequence-Voltage, Real Part"
      m_u1i_bus1 = "m:u1i:bus1", "p.u., Positive-Sequence-Voltage, Imaginary Part"
      m_u1_bus1 = "m:u1:bus1", "p.u., Positive-Sequence-Voltage, Magnitude"
      m_U1_bus1 = "m:U1:bus1", "kV, Line-Ground Positive-Sequence-Voltage, Magnitude"
      m_U1l_bus1 = "m:U1l:bus1", "kV, Line-Line Positive-Sequence-Voltage, Magnitude"
      m_phiu1_bus1 = "m:phiu1:bus1", "deg, Positive-Sequence-Voltage, Angle"
      m_inet_bus1 = "m:inet:bus1", "p.u., Current, Magnitude, referred to network"
      m_I_bus1 = "m:I:bus1", "kA, Current, Magnitude"
      m_phii_bus1 = "m:phii:bus1", "deg, Current, Angle"
      m_i1r_bus1 = "m:i1r:bus1", "p.u., Positive-Sequence Current, Real Part"
      m_i1i_bus1 = "m:i1i:bus1", "p.u., Positive-Sequence Current, Imaginary Part"
      m_i1_bus1 = "m:i1:bus1", "p.u., Positive-Sequence Current, Magnitude"
      m_I1_bus1 = "m:I1:bus1", "kA, Positive-Sequence Current, Magnitude"
      m_phii1_bus1 = "m:phii1:bus1", "deg, Positive-Sequence Current, Angle"
      m_P_bus1 = "m:P:bus1", "MW, Active Power"
      m_Q_bus1 = "m:Q:bus1", "Mvar, Reactive Power"
      m_S_bus1 = "m:S:bus1", "MVA, Apparent Power"
      m_cosphi_bus1 = "m:cosphi:bus1", "Power Factor"
      m_tanphi_bus1 = "m:tanphi:bus1", "tan(phi)"
      m_Psum_bus1 = "m:Psum:bus1", "MW, Total Active Power"
      m_Qsum_bus1 = "m:Qsum:bus1", "Mvar, Total Reactive Power"
      m_Ssum_bus1 = "m:Ssum:bus1", "MVA, Total Apparent Power"
      m_cosphisum_bus1 = "m:cosphisum:bus1", "Total Power Factor"
      m_tanphisum_bus1 = "m:tanphisum:bus1", "Total tan(phi)"
      m_Spu_bus1 = "m:Spu:bus1", "MVA/p.u., Apparent Power per p.u. Voltage"
      m_i1P_bus1 = "m:i1P:bus1", "p.u., Positive-Sequence Active Current"
      m_i1Q_bus1 = "m:i1Q:bus1", "p.u., Positive-Sequence Reactive Current"
      m_I1P_bus1 = "m:I1P:bus1", "kA, Positive-Sequence Active Current"
      m_I1Q_bus1 = "m:I1Q:bus1", "kA, Positive-Sequence Reactive Current"
      m_ur_bus1 = "m:ur:bus1", "p.u., Voltage, Real Part"
      m_ui_bus1 = "m:ui:bus1", "p.u., Voltage, Imaginary Part"
      m_u_bus1 = "m:u:bus1", "p.u., Voltage, Magnitude"
      m_ir_bus1 = "m:ir:bus1", "p.u., Current, Real Part"
      m_ii_bus1 = "m:ii:bus1", "p.u., Current, Imaginary Part"
      m_i_bus1 = "m:i:bus1", "p.u., Current, Magnitude"
      m_phiui_bus1 = "m:phiui:bus1", "deg, Angle between Voltage and Current"
      m_phiu1i1_bus1 = "m:phiu1i1:bus1", "deg, Angle between Voltage and Current in positive sequence system"
      m_Tfct_bus1 = "m:Tfct:bus1", "s, Fault Clearing Time"
      m_Brkload_bus1 = "m:Brkload:bus1", "%, Breaker Loading"
      n_ur_bus1 = "n:ur:bus1", "p.u., Voltage, Real Part"
      n_ui_bus1 = "n:ui:bus1", "p.u., Voltage, Imaginary Part"
      n_u_bus1 = "n:u:bus1", "p.u., Voltage, Magnitude"
      n_upc_bus1 = "n:upc:bus1", "%, Voltage, Magnitude"
      n_u1_bus1 = "n:u1:bus1", "p.u., Voltage, Magnitude"
      n_u1pc_bus1 = "n:u1pc:bus1", "%, Voltage, Magnitude"
      n_u1r_bus1 = "n:u1r:bus1", "p.u., Positive-Sequence Voltage, Real Part"
      n_u1i_bus1 = "n:u1i:bus1", "p.u., Positive-Sequence Voltage, Imaginary Part"
      n_U_bus1 = "n:U:bus1", "kV, Line-Ground Voltage, Magnitude"
      n_Ul_bus1 = "n:Ul:bus1", "kV, Line-Line Voltage, Magnitude"
      n_phiu_bus1 = "n:phiu:bus1", "deg, Voltage, Angle"
      n_phiurel_bus1 = "n:phiurel:bus1", "deg, Voltage, Relative Angle"
      n_du_bus1 = "n:du:bus1", "%, Voltage Deviation"
      n_ishcr_bus1 = "n:ishcr:bus1", "p.u., Short-Circuit Current, real"
      n_ishci_bus1 = "n:ishci:bus1", "p.u., Short-Circuit Current, imag"
      n_ishc_bus1 = "n:ishc:bus1", "p.u., Short-Circuit Current, magnitude"
      n_Ishc_bus1 = "n:Ishc:bus1", "kA, Short-Circuit Current, magnitude"
      n_fe_bus1 = "n:fe:bus1", "p.u., Electrical Frequency"
      n_frdev_bus1 = "n:frdev:bus1", "Hz, average frequency"
      n_dfehz_bus1 = "n:dfehz:bus1", "Hz, Deviation of the El. Frequency"
      n_dfedt_bus1 = "n:dfedt:bus1", "p.u./s, Derivative of El. Frequency"
      n_fehz_bus1 = "n:fehz:bus1", "Hz, Electrical Frequency"
      n_frnom_bus1 = "n:frnom:bus1", "Hz, Nominal Frequency"
      s_i0 = "s:i0", "p.u., IN, Current-Input, continuous, double"
      s_I0 = "s:I0", "kA, IN, Current-Input, continuous, double"
      s_f0 = "s:f0", "p.u., IN, Frequency-Input continuous, double"
      s_F0Hz = "s:F0Hz", "Hz, IN, Frequency-Input continuous, double"
      s_I_A = "s:I_A", "kA, IN, Current, Magnitude, continuous, double"
      s_I_B = "s:I_B", "kA, IN, Current, Magnitude, continuous, double"
      s_I_C = "s:I_C", "kA, IN, Current, Magnitude, continuous, double"
      s_phii_B = "s:phii_B", "deg, IN, Current, Angle, Phase b continuous, double"
      s_phii_C = "s:phii_C", "deg, IN, Current, Angle, Phase c continuous, double"
      s_dphii = "s:dphii", "rad, IN, Additional Angle, continuous, double"
      s_fref = "s:fref", "p.u., IN, Reference Frequency, continuous, double"
      s_xspeed = "s:xspeed", "p.u., OUT, Frequency, continuous, double"
      s_phii = "s:phii", "rad, STATE, Current-Angle, continuous, double"
      s_phii_dt = "s:phii:dt", "rad/s, d/dt, Current-Angle, derivative, continuous, double"
      c_Ir = "c:Ir", "A, Nominal Current"
      c_isetp = "c:isetp", "p.u., Current Setpoint"
      c_cosini = "c:cosini", "Power Factor"
      c_hpi = "c:hpi", "rad/s, Nominal-Angular-Frequency"
      c_G1 = "c:G1", "S, Positive Sequence Conductance"
      c_C1 = "c:C1", "F, Positive Sequence Capacitance"
      e_iAstabint = "e:iAstabint", "A-stable integration algorithm"
      e_G1 = "e:G1", "S, Internal Conductance, G1"
      e_B1 = "e:B1", "S, Internal Susceptance, B1"
      e_G2 = "e:G2", "S, Internal Conductance, G2"
      e_B2 = "e:B2", "S, Internal Susceptance, B2"
      e_G0 = "e:G0", "S, Internal Conductance, G0"
      e_B0 = "e:B0", "S, Internal Susceptance, B0"
      e_leadIinput = "e:leadIinput", "Current input"
      e_leadFinput = "e:leadFinput", "Frequency input"

    class ElmLne(Enum):
      _init_ = 'value __doc__'

      m_u1r_bus1 = "m:u1r:bus1", "p.u., Positive-Sequence-Voltage, Real Part"
      m_u1r_bus2 = "m:u1r:bus2", "p.u., Positive-Sequence-Voltage, Real Part"
      m_u1i_bus1 = "m:u1i:bus1", "p.u., Positive-Sequence-Voltage, Imaginary Part"
      m_u1i_bus2 = "m:u1i:bus2", "p.u., Positive-Sequence-Voltage, Imaginary Part"
      m_u1_bus1 = "m:u1:bus1", "p.u., Positive-Sequence-Voltage, Magnitude"
      m_u1_bus2 = "m:u1:bus2", "p.u., Positive-Sequence-Voltage, Magnitude"
      m_U1_bus1 = "m:U1:bus1", "kV, Line-Ground Positive-Sequence-Voltage, Magnitude"
      m_U1_bus2 = "m:U1:bus2", "kV, Line-Ground Positive-Sequence-Voltage, Magnitude"
      m_U1l_bus1 = "m:U1l:bus1", "kV, Line-Line Positive-Sequence-Voltage, Magnitude"
      m_U1l_bus2 = "m:U1l:bus2", "kV, Line-Line Positive-Sequence-Voltage, Magnitude"
      m_phiu1_bus1 = "m:phiu1:bus1", "deg, Positive-Sequence-Voltage, Angle"
      m_phiu1_bus2 = "m:phiu1:bus2", "deg, Positive-Sequence-Voltage, Angle"
      m_inet_bus1 = "m:inet:bus1", "p.u., Current, Magnitude, referred to network"
      m_inet_bus2 = "m:inet:bus2", "p.u., Current, Magnitude, referred to network"
      m_I_bus1 = "m:I:bus1", "kA, Current, Magnitude"
      m_I_bus2 = "m:I:bus2", "kA, Current, Magnitude"
      m_phii_bus1 = "m:phii:bus1", "deg, Current, Angle"
      m_phii_bus2 = "m:phii:bus2", "deg, Current, Angle"
      m_i1r_bus1 = "m:i1r:bus1", "p.u., Positive-Sequence Current, Real Part"
      m_i1r_bus2 = "m:i1r:bus2", "p.u., Positive-Sequence Current, Real Part"
      m_i1i_bus1 = "m:i1i:bus1", "p.u., Positive-Sequence Current, Imaginary Part"
      m_i1i_bus2 = "m:i1i:bus2", "p.u., Positive-Sequence Current, Imaginary Part"
      m_i1_bus1 = "m:i1:bus1", "p.u., Positive-Sequence Current, Magnitude"
      m_i1_bus2 = "m:i1:bus2", "p.u., Positive-Sequence Current, Magnitude"
      m_I1_bus1 = "m:I1:bus1", "kA, Positive-Sequence Current, Magnitude"
      m_I1_bus2 = "m:I1:bus2", "kA, Positive-Sequence Current, Magnitude"
      m_phii1_bus1 = "m:phii1:bus1", "deg, Positive-Sequence Current, Angle"
      m_phii1_bus2 = "m:phii1:bus2", "deg, Positive-Sequence Current, Angle"
      m_P_bus1 = "m:P:bus1", "MW, Active Power"
      m_P_bus2 = "m:P:bus2", "MW, Active Power"
      m_Q_bus1 = "m:Q:bus1", "Mvar, Reactive Power"
      m_Q_bus2 = "m:Q:bus2", "Mvar, Reactive Power"
      m_S_bus1 = "m:S:bus1", "MVA, Apparent Power"
      m_S_bus2 = "m:S:bus2", "MVA, Apparent Power"
      m_cosphi_bus1 = "m:cosphi:bus1", "Power Factor"
      m_cosphi_bus2 = "m:cosphi:bus2", "Power Factor"
      m_tanphi_bus1 = "m:tanphi:bus1", "tan(phi)"
      m_tanphi_bus2 = "m:tanphi:bus2", "tan(phi)"
      m_Psum_bus1 = "m:Psum:bus1", "MW, Total Active Power"
      m_Psum_bus2 = "m:Psum:bus2", "MW, Total Active Power"
      m_Qsum_bus1 = "m:Qsum:bus1", "Mvar, Total Reactive Power"
      m_Qsum_bus2 = "m:Qsum:bus2", "Mvar, Total Reactive Power"
      m_Ssum_bus1 = "m:Ssum:bus1", "MVA, Total Apparent Power"
      m_Ssum_bus2 = "m:Ssum:bus2", "MVA, Total Apparent Power"
      m_cosphisum_bus1 = "m:cosphisum:bus1", "Total Power Factor"
      m_cosphisum_bus2 = "m:cosphisum:bus2", "Total Power Factor"
      m_tanphisum_bus1 = "m:tanphisum:bus1", "Total tan(phi)"
      m_tanphisum_bus2 = "m:tanphisum:bus2", "Total tan(phi)"
      m_Spu_bus1 = "m:Spu:bus1", "MVA/p.u., Apparent Power per p.u. Voltage"
      m_Spu_bus2 = "m:Spu:bus2", "MVA/p.u., Apparent Power per p.u. Voltage"
      m_i1P_bus1 = "m:i1P:bus1", "p.u., Positive-Sequence Active Current"
      m_i1P_bus2 = "m:i1P:bus2", "p.u., Positive-Sequence Active Current"
      m_i1Q_bus1 = "m:i1Q:bus1", "p.u., Positive-Sequence Reactive Current"
      m_i1Q_bus2 = "m:i1Q:bus2", "p.u., Positive-Sequence Reactive Current"
      m_I1P_bus1 = "m:I1P:bus1", "kA, Positive-Sequence Active Current"
      m_I1P_bus2 = "m:I1P:bus2", "kA, Positive-Sequence Active Current"
      m_I1Q_bus1 = "m:I1Q:bus1", "kA, Positive-Sequence Reactive Current"
      m_I1Q_bus2 = "m:I1Q:bus2", "kA, Positive-Sequence Reactive Current"
      m_ur_bus1 = "m:ur:bus1", "p.u., Voltage, Real Part"
      m_ur_bus2 = "m:ur:bus2", "p.u., Voltage, Real Part"
      m_ui_bus1 = "m:ui:bus1", "p.u., Voltage, Imaginary Part"
      m_ui_bus2 = "m:ui:bus2", "p.u., Voltage, Imaginary Part"
      m_u_bus1 = "m:u:bus1", "p.u., Voltage, Magnitude"
      m_u_bus2 = "m:u:bus2", "p.u., Voltage, Magnitude"
      m_ir_bus1 = "m:ir:bus1", "p.u., Current, Real Part"
      m_ir_bus2 = "m:ir:bus2", "p.u., Current, Real Part"
      m_ii_bus1 = "m:ii:bus1", "p.u., Current, Imaginary Part"
      m_ii_bus2 = "m:ii:bus2", "p.u., Current, Imaginary Part"
      m_i_bus1 = "m:i:bus1", "p.u., Current, Magnitude"
      m_i_bus2 = "m:i:bus2", "p.u., Current, Magnitude"
      m_phiui_bus1 = "m:phiui:bus1", "deg, Angle between Voltage and Current"
      m_phiui_bus2 = "m:phiui:bus2", "deg, Angle between Voltage and Current"
      m_phiu1i1_bus1 = "m:phiu1i1:bus1", "deg, Angle between Voltage and Current in positive sequence system"
      m_phiu1i1_bus2 = "m:phiu1i1:bus2", "deg, Angle between Voltage and Current in positive sequence system"
      m_Tfct_bus1 = "m:Tfct:bus1", "s, Fault Clearing Time"
      m_Tfct_bus2 = "m:Tfct:bus2", "s, Fault Clearing Time"
      m_Brkload_bus1 = "m:Brkload:bus1", "%, Breaker Loading"
      m_Brkload_bus2 = "m:Brkload:bus2", "%, Breaker Loading"
      n_ur_bus1 = "n:ur:bus1", "p.u., Voltage, Real Part"
      n_ur_bus2 = "n:ur:bus2", "p.u., Voltage, Real Part"
      n_ui_bus1 = "n:ui:bus1", "p.u., Voltage, Imaginary Part"
      n_ui_bus2 = "n:ui:bus2", "p.u., Voltage, Imaginary Part"
      n_u_bus1 = "n:u:bus1", "p.u., Voltage, Magnitude"
      n_u_bus2 = "n:u:bus2", "p.u., Voltage, Magnitude"
      n_upc_bus1 = "n:upc:bus1", "%, Voltage, Magnitude"
      n_upc_bus2 = "n:upc:bus2", "%, Voltage, Magnitude"
      n_u1_bus1 = "n:u1:bus1", "p.u., Voltage, Magnitude"
      n_u1_bus2 = "n:u1:bus2", "p.u., Voltage, Magnitude"
      n_u1pc_bus1 = "n:u1pc:bus1", "%, Voltage, Magnitude"
      n_u1pc_bus2 = "n:u1pc:bus2", "%, Voltage, Magnitude"
      n_u1r_bus1 = "n:u1r:bus1", "p.u., Positive-Sequence Voltage, Real Part"
      n_u1r_bus2 = "n:u1r:bus2", "p.u., Positive-Sequence Voltage, Real Part"
      n_u1i_bus1 = "n:u1i:bus1", "p.u., Positive-Sequence Voltage, Imaginary Part"
      n_u1i_bus2 = "n:u1i:bus2", "p.u., Positive-Sequence Voltage, Imaginary Part"
      n_U_bus1 = "n:U:bus1", "kV, Line-Ground Voltage, Magnitude"
      n_U_bus2 = "n:U:bus2", "kV, Line-Ground Voltage, Magnitude"
      n_Ul_bus1 = "n:Ul:bus1", "kV, Line-Line Voltage, Magnitude"
      n_Ul_bus2 = "n:Ul:bus2", "kV, Line-Line Voltage, Magnitude"
      n_phiu_bus1 = "n:phiu:bus1", "deg, Voltage, Angle"
      n_phiu_bus2 = "n:phiu:bus2", "deg, Voltage, Angle"
      n_phiurel_bus1 = "n:phiurel:bus1", "deg, Voltage, Relative Angle"
      n_phiurel_bus2 = "n:phiurel:bus2", "deg, Voltage, Relative Angle"
      n_du_bus1 = "n:du:bus1", "%, Voltage Deviation"
      n_du_bus2 = "n:du:bus2", "%, Voltage Deviation"
      n_ishcr_bus1 = "n:ishcr:bus1", "p.u., Short-Circuit Current, real"
      n_ishcr_bus2 = "n:ishcr:bus2", "p.u., Short-Circuit Current, real"
      n_ishci_bus1 = "n:ishci:bus1", "p.u., Short-Circuit Current, imag"
      n_ishci_bus2 = "n:ishci:bus2", "p.u., Short-Circuit Current, imag"
      n_ishc_bus1 = "n:ishc:bus1", "p.u., Short-Circuit Current, magnitude"
      n_ishc_bus2 = "n:ishc:bus2", "p.u., Short-Circuit Current, magnitude"
      n_Ishc_bus1 = "n:Ishc:bus1", "kA, Short-Circuit Current, magnitude"
      n_Ishc_bus2 = "n:Ishc:bus2", "kA, Short-Circuit Current, magnitude"
      n_fe_bus1 = "n:fe:bus1", "p.u., Electrical Frequency"
      n_fe_bus2 = "n:fe:bus2", "p.u., Electrical Frequency"
      n_frdev_bus1 = "n:frdev:bus1", "Hz, average frequency"
      n_frdev_bus2 = "n:frdev:bus2", "Hz, average frequency"
      n_dfehz_bus1 = "n:dfehz:bus1", "Hz, Deviation of the El. Frequency"
      n_dfehz_bus2 = "n:dfehz:bus2", "Hz, Deviation of the El. Frequency"
      n_dfedt_bus1 = "n:dfedt:bus1", "p.u./s, Derivative of El. Frequency"
      n_dfedt_bus2 = "n:dfedt:bus2", "p.u./s, Derivative of El. Frequency"
      n_fehz_bus1 = "n:fehz:bus1", "Hz, Electrical Frequency"
      n_fehz_bus2 = "n:fehz:bus2", "Hz, Electrical Frequency"
      n_frnom_bus1 = "n:frnom:bus1", "Hz, Nominal Frequency"
      n_frnom_bus2 = "n:frnom:bus2", "Hz, Nominal Frequency"
      c_zline_r = "c:zline:r", "p.u., Impedance, Real Part"
      c_zline_i = "c:zline:i", "p.u., Impedance, Imaginary Part"
      c_zline_phi = "c:zline:phi", "deg, Impedance, Angle"
      c_zline_mag = "c:zline:mag", "p.u., Impedance, Magnitude"
      c_zline_phirad = "c:zline:phirad", "rad, Impedance, Angle"
      c_yline_r = "c:yline:r", "p.u., Admittance, Real Part"
      c_yline_i = "c:yline:i", "p.u., Admittance, Imaginary Part"
      c_yline_phi = "c:yline:phi", "deg, Admittance, Angle"
      c_yline_mag = "c:yline:mag", "p.u., Admittance, Magnitude"
      c_yline_phirad = "c:yline:phirad", "rad, Admittance, Angle"
      c_frnom = "c:frnom", "Hz, Nominal frequency"
      c_hpi = "c:hpi", "rad/s, Nominal angular frequency"
      c_loading = "c:loading", "%, Loading"
      c_Cload = "c:Cload", "Mvar, Capacitive Losses"
      c_Gload = "c:Gload", "kW, Active no load losses"
      c_Losses = "c:Losses", "kW, Losses"
      c_curnom = "c:curnom", "kA, Nominal Current"
      c_rtfac = "c:rtfac", "Temperature Factor"
      c_Inom = "c:Inom", "kA, Nominal Current"
      e_i_const_rms = "e:i_const_rms", "Travelling wave"
      e_lossModel_rms = "e:lossModel_rms", "Loss model:R/2 at both ends"
      e_iopt_tem_rms = "e:iopt_tem_rms", "Temperature dependency"
      e_ckz0_rms = "e:ckz0_rms", "Ohm, Surge impedance, HF, mode 0."
      e_ca0dc_rms = "e:ca0dc_rms", "p.u., Wave propagation, DC, mode 0"
      e_cTa0_rms = "e:cTa0_rms", "s, Travel time, mode 0"
      e_ckz1_rms = "e:ckz1_rms", "Ohm, Surge Impedance, HF, mode 1."
      e_ca1dc_rms = "e:ca1dc_rms", "p.u., Wave propagation, DC, mode 1"
      e_cTa1_rms = "e:cTa1_rms", "s, Travel time, mode 1"
      e_ckz2_rms = "e:ckz2_rms", "Ohm, Surge Impedance, HF, mode 2."
      e_ca2dc_rms = "e:ca2dc_rms", "p.u., Wave propagation, DC, mode 2"
      e_cTa2_rms = "e:cTa2_rms", "s, Travel time, mode 2"
      t_tmax = "t:tmax", "degC, Max. Operating Temperature"
      t_rline = "t:rline", "Ohm/km, AC-Resistance R'(20°C)"
      t_rline_tmax = "t:rline_tmax", "Ohm/km, AC-Resistance R' at max. operating temperature"
      t_alpha = "t:alpha", "1/K, Temperature Coefficient"
      t_mlei = "t:mlei", "Conductor Material"
      t_bline = "t:bline", "uS/km, Susceptance B'"
      t_cline = "t:cline", "uF/km, Capacitance C'"
      t_tline = "t:tline", "Ins. Factor"
      t_gline = "t:gline", "uS/km, Conductance G'"
      t_bline0 = "t:bline0", "uS/km, Susceptance B0'"
      t_cline0 = "t:cline0", "uF/km, Capacitance C0'"
      t_Ices = "t:Ices", "A/km, Earth-Fault Current"
      t_tline0 = "t:tline0", "Ins. Factor"
      t_gline0 = "t:gline0", "uS/km, Conductance G0'"
      t_bnline = "t:bnline", "uS/km, Susceptance Bn'"
      t_cnline = "t:cnline", "uF/km, Capacitance Cn'"
      t_bpnline = "t:bpnline", "uS/km, Susceptance Bpn'"
      t_cpnline = "t:cpnline", "uF/km, Capacitance Cpn'"

    class ElmLod(Enum):
      _init_ = 'value __doc__'

      m_u1r_bus1 = "m:u1r:bus1", "p.u., Positive-Sequence-Voltage, Real Part"
      m_u1i_bus1 = "m:u1i:bus1", "p.u., Positive-Sequence-Voltage, Imaginary Part"
      m_u1_bus1 = "m:u1:bus1", "p.u., Positive-Sequence-Voltage, Magnitude"
      m_U1_bus1 = "m:U1:bus1", "kV, Line-Ground Positive-Sequence-Voltage, Magnitude"
      m_U1l_bus1 = "m:U1l:bus1", "kV, Line-Line Positive-Sequence-Voltage, Magnitude"
      m_phiu1_bus1 = "m:phiu1:bus1", "deg, Positive-Sequence-Voltage, Angle"
      m_inet_bus1 = "m:inet:bus1", "p.u., Current, Magnitude, referred to network"
      m_I_bus1 = "m:I:bus1", "kA, Current, Magnitude"
      m_phii_bus1 = "m:phii:bus1", "deg, Current, Angle"
      m_i1r_bus1 = "m:i1r:bus1", "p.u., Positive-Sequence Current, Real Part"
      m_i1i_bus1 = "m:i1i:bus1", "p.u., Positive-Sequence Current, Imaginary Part"
      m_i1_bus1 = "m:i1:bus1", "p.u., Positive-Sequence Current, Magnitude"
      m_I1_bus1 = "m:I1:bus1", "kA, Positive-Sequence Current, Magnitude"
      m_phii1_bus1 = "m:phii1:bus1", "deg, Positive-Sequence Current, Angle"
      m_P_bus1 = "m:P:bus1", "MW, Active Power"
      m_Q_bus1 = "m:Q:bus1", "Mvar, Reactive Power"
      m_S_bus1 = "m:S:bus1", "MVA, Apparent Power"
      m_cosphi_bus1 = "m:cosphi:bus1", "Power Factor"
      m_tanphi_bus1 = "m:tanphi:bus1", "tan(phi)"
      m_Psum_bus1 = "m:Psum:bus1", "MW, Total Active Power"
      m_Qsum_bus1 = "m:Qsum:bus1", "Mvar, Total Reactive Power"
      m_Ssum_bus1 = "m:Ssum:bus1", "MVA, Total Apparent Power"
      m_cosphisum_bus1 = "m:cosphisum:bus1", "Total Power Factor"
      m_tanphisum_bus1 = "m:tanphisum:bus1", "Total tan(phi)"
      m_Spu_bus1 = "m:Spu:bus1", "MVA/p.u., Apparent Power per p.u. Voltage"
      m_i1P_bus1 = "m:i1P:bus1", "p.u., Positive-Sequence Active Current"
      m_i1Q_bus1 = "m:i1Q:bus1", "p.u., Positive-Sequence Reactive Current"
      m_I1P_bus1 = "m:I1P:bus1", "kA, Positive-Sequence Active Current"
      m_I1Q_bus1 = "m:I1Q:bus1", "kA, Positive-Sequence Reactive Current"
      m_ur_bus1 = "m:ur:bus1", "p.u., Voltage, Real Part"
      m_ui_bus1 = "m:ui:bus1", "p.u., Voltage, Imaginary Part"
      m_u_bus1 = "m:u:bus1", "p.u., Voltage, Magnitude"
      m_ir_bus1 = "m:ir:bus1", "p.u., Current, Real Part"
      m_ii_bus1 = "m:ii:bus1", "p.u., Current, Imaginary Part"
      m_i_bus1 = "m:i:bus1", "p.u., Current, Magnitude"
      m_phiui_bus1 = "m:phiui:bus1", "deg, Angle between Voltage and Current"
      m_phiu1i1_bus1 = "m:phiu1i1:bus1", "deg, Angle between Voltage and Current in positive sequence system"
      m_Tfct_bus1 = "m:Tfct:bus1", "s, Fault Clearing Time"
      m_Brkload_bus1 = "m:Brkload:bus1", "%, Breaker Loading"
      n_ur_bus1 = "n:ur:bus1", "p.u., Voltage, Real Part"
      n_ui_bus1 = "n:ui:bus1", "p.u., Voltage, Imaginary Part"
      n_u_bus1 = "n:u:bus1", "p.u., Voltage, Magnitude"
      n_upc_bus1 = "n:upc:bus1", "%, Voltage, Magnitude"
      n_u1_bus1 = "n:u1:bus1", "p.u., Voltage, Magnitude"
      n_u1pc_bus1 = "n:u1pc:bus1", "%, Voltage, Magnitude"
      n_u1r_bus1 = "n:u1r:bus1", "p.u., Positive-Sequence Voltage, Real Part"
      n_u1i_bus1 = "n:u1i:bus1", "p.u., Positive-Sequence Voltage, Imaginary Part"
      n_U_bus1 = "n:U:bus1", "kV, Line-Ground Voltage, Magnitude"
      n_Ul_bus1 = "n:Ul:bus1", "kV, Line-Line Voltage, Magnitude"
      n_phiu_bus1 = "n:phiu:bus1", "deg, Voltage, Angle"
      n_phiurel_bus1 = "n:phiurel:bus1", "deg, Voltage, Relative Angle"
      n_du_bus1 = "n:du:bus1", "%, Voltage Deviation"
      n_ishcr_bus1 = "n:ishcr:bus1", "p.u., Short-Circuit Current, real"
      n_ishci_bus1 = "n:ishci:bus1", "p.u., Short-Circuit Current, imag"
      n_ishc_bus1 = "n:ishc:bus1", "p.u., Short-Circuit Current, magnitude"
      n_Ishc_bus1 = "n:Ishc:bus1", "kA, Short-Circuit Current, magnitude"
      n_fe_bus1 = "n:fe:bus1", "p.u., Electrical Frequency"
      n_frdev_bus1 = "n:frdev:bus1", "Hz, average frequency"
      n_dfehz_bus1 = "n:dfehz:bus1", "Hz, Deviation of the El. Frequency"
      n_dfedt_bus1 = "n:dfedt:bus1", "p.u./s, Derivative of El. Frequency"
      n_fehz_bus1 = "n:fehz:bus1", "Hz, Electrical Frequency"
      n_frnom_bus1 = "n:frnom:bus1", "Hz, Nominal Frequency"
      s_Pext = "s:Pext", "MW, IN, Active Power Input, continuous, double"
      s_Qext = "s:Qext", "Mvar, IN, Reactive Power Input, continuous, double"
      s_fref = "s:fref", "p.u., IN, Reference Frequency, continuous, double"
      s_xu = "s:xu", "p.u., STATE, State Variable, continuous, double"
      s_xf = "s:xf", "p.u., STATE, State Variable, continuous, double"
      s_xu_dt = "s:xu:dt", "p.u./s, d/dt, State Variable, derivative, continuous, double"
      s_xf_dt = "s:xf:dt", "p.u./s, d/dt, State Variable, derivative, continuous, double"
      c_lodst_l = "c:lodst_l", "Static"
      c_loddy_l = "c:loddy_l", "Dynamic"
      c_kpf = "c:kpf", "Static Frequency Dependence"
      c_tpf = "c:tpf", "s, Transient Frequency Dependence"
      c_tpu = "c:tpu", "Transient Voltage Dependence"
      c_kqf = "c:kqf", "Static Frequency Dependence"
      c_tqf = "c:tqf", "s, Transient Frequency Dependence"
      c_tqu = "c:tqu", "s, Transient Voltage Dependence"
      c_t1 = "c:t1", "s, Dynamic Load Time Constant"
      c_sd0_r = "c:sd0:r", "MW, Nominal power of dynamic load, Real Part"
      c_sd0_i = "c:sd0:i", "Mvar, Nominal power of dynamic load, Imaginary Part"
      c_sd0_phi = "c:sd0:phi", "deg, Nominal power of dynamic load, Angle"
      c_sd0_mag = "c:sd0:mag", "MVA, Nominal power of dynamic load, Magnitude"
      c_sd0_phirad = "c:sd0:phirad", "rad, Nominal power of dynamic load, Angle"
      c_sd00_r = "c:sd00:r", "MW, Nominal power of dynamic load, Real Part"
      c_sd00_i = "c:sd00:i", "Mvar, Nominal power of dynamic load, Imaginary Part"
      c_sd00_phi = "c:sd00:phi", "deg, Nominal power of dynamic load, Angle"
      c_sd00_mag = "c:sd00:mag", "MVA, Nominal power of dynamic load, Magnitude"
      c_sd00_phirad = "c:sd00:phirad", "rad, Nominal power of dynamic load, Angle"
      c_u0 = "c:u0", "p.u., Nominal voltage"
      c_udmax = "c:udmax", "p.u., Upper Voltage Limit"
      c_udmin = "c:udmin", "p.u., Lower Voltage Limit"
      c_i_nln = "c:i_nln", "Internal Flag"
      c_fe = "c:fe", "Electrical Frequency"
      c_kpu0 = "c:kpu0", "Volt. Dependence I on P"
      c_kqu0 = "c:kqu0", "Volt. Dependence I on Q"
      c_kpu1 = "c:kpu1", "Volt. Dependence II on P"
      c_kqu1 = "c:kqu1", "Volt. Dependence II on Q"
      c_kpu = "c:kpu", "Volt. Dependence III on P"
      c_kqu = "c:kqu", "Volt. Dependence III on Q"
      c_aP = "c:aP", "Percent of Volt. Dep. I on P"
      c_bP = "c:bP", "Percent of Volt. Dep. II on P"
      c_aQ = "c:aQ", "Percent of Volt. Dep. I on Q"
      c_bQ = "c:bQ", "Percent of Volt. Dep. II on Q"
      e_mode_inp = "e:mode_inp", "Input Mode"
      e_i_sym = "e:i_sym", "Balanced/Unbalanced"
      e_plini = "e:plini", "MW, Active Power"
      e_qlini = "e:qlini", "Mvar, Reactive Power"
      e_slini = "e:slini", "MVA, Apparent Power"
      e_ilini = "e:ilini", "kA, Current"
      e_coslini = "e:coslini", "Power Factor"
      e_pf_recap = "e:pf_recap", "Power Factor"
      e_p_direc = "e:p_direc", "Power Direction"
      e_u0 = "e:u0", "p.u., Voltage"
      e_scale0 = "e:scale0", "Scaling Factor"
      e_ramp_type = "e:ramp_type", "Allow Load-Ramp Event"
      e_i_normPQext = "e:i_normPQext", "Pext, Qext input signals normalised to load flow voltage"
      e_plinir = "e:plinir", "MW, Active Power"
      e_qlinir = "e:qlinir", "Mvar, Reactive Power"
      e_slinir = "e:slinir", "MVA, Apparent Power"
      e_ilinir = "e:ilinir", "kA, Current"
      e_coslinir = "e:coslinir", "Power Factor"
      e_pf_recapr = "e:pf_recapr", "Power Factor"
      e_p_direcr = "e:p_direcr", "Power Direction"
      e_plinis = "e:plinis", "MW, Active Power"
      e_qlinis = "e:qlinis", "Mvar, Reactive Power"
      e_slinis = "e:slinis", "MVA, Apparent Power"
      e_ilinis = "e:ilinis", "kA, Current"
      e_coslinis = "e:coslinis", "Power Factor"
      e_pf_recaps = "e:pf_recaps", "Power Factor"
      e_p_direcs = "e:p_direcs", "Power Direction"
      e_plinit = "e:plinit", "MW, Active Power"
      e_qlinit = "e:qlinit", "Mvar, Reactive Power"
      e_slinit = "e:slinit", "MVA, Apparent Power"
      e_ilinit = "e:ilinit", "kA, Current"
      e_coslinit = "e:coslinit", "Power Factor"
      e_pf_recapt = "e:pf_recapt", "Power Factor"
      e_p_direct = "e:p_direct", "Power Direction"
      e_iAstabint = "e:iAstabint", "A-stable integration algorithm"

    class ElmLodlv(Enum):
      _init_ = 'value __doc__'

      m_u1r_bus1 = "m:u1r:bus1", "p.u., Positive-Sequence-Voltage, Real Part"
      m_u1i_bus1 = "m:u1i:bus1", "p.u., Positive-Sequence-Voltage, Imaginary Part"
      m_u1_bus1 = "m:u1:bus1", "p.u., Positive-Sequence-Voltage, Magnitude"
      m_U1_bus1 = "m:U1:bus1", "kV, Line-Ground Positive-Sequence-Voltage, Magnitude"
      m_U1l_bus1 = "m:U1l:bus1", "kV, Line-Line Positive-Sequence-Voltage, Magnitude"
      m_phiu1_bus1 = "m:phiu1:bus1", "deg, Positive-Sequence-Voltage, Angle"
      m_inet_bus1 = "m:inet:bus1", "p.u., Current, Magnitude, referred to network"
      m_I_bus1 = "m:I:bus1", "kA, Current, Magnitude"
      m_phii_bus1 = "m:phii:bus1", "deg, Current, Angle"
      m_i1r_bus1 = "m:i1r:bus1", "p.u., Positive-Sequence Current, Real Part"
      m_i1i_bus1 = "m:i1i:bus1", "p.u., Positive-Sequence Current, Imaginary Part"
      m_i1_bus1 = "m:i1:bus1", "p.u., Positive-Sequence Current, Magnitude"
      m_I1_bus1 = "m:I1:bus1", "kA, Positive-Sequence Current, Magnitude"
      m_phii1_bus1 = "m:phii1:bus1", "deg, Positive-Sequence Current, Angle"
      m_P_bus1 = "m:P:bus1", "MW, Active Power"
      m_Q_bus1 = "m:Q:bus1", "Mvar, Reactive Power"
      m_S_bus1 = "m:S:bus1", "MVA, Apparent Power"
      m_cosphi_bus1 = "m:cosphi:bus1", "Power Factor"
      m_tanphi_bus1 = "m:tanphi:bus1", "tan(phi)"
      m_Psum_bus1 = "m:Psum:bus1", "MW, Total Active Power"
      m_Qsum_bus1 = "m:Qsum:bus1", "Mvar, Total Reactive Power"
      m_Ssum_bus1 = "m:Ssum:bus1", "MVA, Total Apparent Power"
      m_cosphisum_bus1 = "m:cosphisum:bus1", "Total Power Factor"
      m_tanphisum_bus1 = "m:tanphisum:bus1", "Total tan(phi)"
      m_Spu_bus1 = "m:Spu:bus1", "MVA/p.u., Apparent Power per p.u. Voltage"
      m_i1P_bus1 = "m:i1P:bus1", "p.u., Positive-Sequence Active Current"
      m_i1Q_bus1 = "m:i1Q:bus1", "p.u., Positive-Sequence Reactive Current"
      m_I1P_bus1 = "m:I1P:bus1", "kA, Positive-Sequence Active Current"
      m_I1Q_bus1 = "m:I1Q:bus1", "kA, Positive-Sequence Reactive Current"
      m_ur_bus1 = "m:ur:bus1", "p.u., Voltage, Real Part"
      m_ui_bus1 = "m:ui:bus1", "p.u., Voltage, Imaginary Part"
      m_u_bus1 = "m:u:bus1", "p.u., Voltage, Magnitude"
      m_ir_bus1 = "m:ir:bus1", "p.u., Current, Real Part"
      m_ii_bus1 = "m:ii:bus1", "p.u., Current, Imaginary Part"
      m_i_bus1 = "m:i:bus1", "p.u., Current, Magnitude"
      m_phiui_bus1 = "m:phiui:bus1", "deg, Angle between Voltage and Current"
      m_phiu1i1_bus1 = "m:phiu1i1:bus1", "deg, Angle between Voltage and Current in positive sequence system"
      m_Tfct_bus1 = "m:Tfct:bus1", "s, Fault Clearing Time"
      m_Brkload_bus1 = "m:Brkload:bus1", "%, Breaker Loading"
      n_ur_bus1 = "n:ur:bus1", "p.u., Voltage, Real Part"
      n_ui_bus1 = "n:ui:bus1", "p.u., Voltage, Imaginary Part"
      n_u_bus1 = "n:u:bus1", "p.u., Voltage, Magnitude"
      n_upc_bus1 = "n:upc:bus1", "%, Voltage, Magnitude"
      n_u1_bus1 = "n:u1:bus1", "p.u., Voltage, Magnitude"
      n_u1pc_bus1 = "n:u1pc:bus1", "%, Voltage, Magnitude"
      n_u1r_bus1 = "n:u1r:bus1", "p.u., Positive-Sequence Voltage, Real Part"
      n_u1i_bus1 = "n:u1i:bus1", "p.u., Positive-Sequence Voltage, Imaginary Part"
      n_U_bus1 = "n:U:bus1", "kV, Line-Ground Voltage, Magnitude"
      n_Ul_bus1 = "n:Ul:bus1", "kV, Line-Line Voltage, Magnitude"
      n_phiu_bus1 = "n:phiu:bus1", "deg, Voltage, Angle"
      n_phiurel_bus1 = "n:phiurel:bus1", "deg, Voltage, Relative Angle"
      n_du_bus1 = "n:du:bus1", "%, Voltage Deviation"
      n_ishcr_bus1 = "n:ishcr:bus1", "p.u., Short-Circuit Current, real"
      n_ishci_bus1 = "n:ishci:bus1", "p.u., Short-Circuit Current, imag"
      n_ishc_bus1 = "n:ishc:bus1", "p.u., Short-Circuit Current, magnitude"
      n_Ishc_bus1 = "n:Ishc:bus1", "kA, Short-Circuit Current, magnitude"
      n_fe_bus1 = "n:fe:bus1", "p.u., Electrical Frequency"
      n_frdev_bus1 = "n:frdev:bus1", "Hz, average frequency"
      n_dfehz_bus1 = "n:dfehz:bus1", "Hz, Deviation of the El. Frequency"
      n_dfedt_bus1 = "n:dfedt:bus1", "p.u./s, Derivative of El. Frequency"
      n_fehz_bus1 = "n:fehz:bus1", "Hz, Electrical Frequency"
      n_frnom_bus1 = "n:frnom:bus1", "Hz, Nominal Frequency"
      e_iopt_type = "e:iopt_type", "Allow Load-Ramp Event"

    class ElmLodmv(Enum):
      _init_ = 'value __doc__'

      m_u1r_bus1 = "m:u1r:bus1", "p.u., Positive-Sequence-Voltage, Real Part"
      m_u1i_bus1 = "m:u1i:bus1", "p.u., Positive-Sequence-Voltage, Imaginary Part"
      m_u1_bus1 = "m:u1:bus1", "p.u., Positive-Sequence-Voltage, Magnitude"
      m_U1_bus1 = "m:U1:bus1", "kV, Line-Ground Positive-Sequence-Voltage, Magnitude"
      m_U1l_bus1 = "m:U1l:bus1", "kV, Line-Line Positive-Sequence-Voltage, Magnitude"
      m_phiu1_bus1 = "m:phiu1:bus1", "deg, Positive-Sequence-Voltage, Angle"
      m_inet_bus1 = "m:inet:bus1", "p.u., Current, Magnitude, referred to network"
      m_I_bus1 = "m:I:bus1", "kA, Current, Magnitude"
      m_phii_bus1 = "m:phii:bus1", "deg, Current, Angle"
      m_i1r_bus1 = "m:i1r:bus1", "p.u., Positive-Sequence Current, Real Part"
      m_i1i_bus1 = "m:i1i:bus1", "p.u., Positive-Sequence Current, Imaginary Part"
      m_i1_bus1 = "m:i1:bus1", "p.u., Positive-Sequence Current, Magnitude"
      m_I1_bus1 = "m:I1:bus1", "kA, Positive-Sequence Current, Magnitude"
      m_phii1_bus1 = "m:phii1:bus1", "deg, Positive-Sequence Current, Angle"
      m_P_bus1 = "m:P:bus1", "MW, Active Power"
      m_Q_bus1 = "m:Q:bus1", "Mvar, Reactive Power"
      m_S_bus1 = "m:S:bus1", "MVA, Apparent Power"
      m_cosphi_bus1 = "m:cosphi:bus1", "Power Factor"
      m_tanphi_bus1 = "m:tanphi:bus1", "tan(phi)"
      m_Psum_bus1 = "m:Psum:bus1", "MW, Total Active Power"
      m_Qsum_bus1 = "m:Qsum:bus1", "Mvar, Total Reactive Power"
      m_Ssum_bus1 = "m:Ssum:bus1", "MVA, Total Apparent Power"
      m_cosphisum_bus1 = "m:cosphisum:bus1", "Total Power Factor"
      m_tanphisum_bus1 = "m:tanphisum:bus1", "Total tan(phi)"
      m_Spu_bus1 = "m:Spu:bus1", "MVA/p.u., Apparent Power per p.u. Voltage"
      m_i1P_bus1 = "m:i1P:bus1", "p.u., Positive-Sequence Active Current"
      m_i1Q_bus1 = "m:i1Q:bus1", "p.u., Positive-Sequence Reactive Current"
      m_I1P_bus1 = "m:I1P:bus1", "kA, Positive-Sequence Active Current"
      m_I1Q_bus1 = "m:I1Q:bus1", "kA, Positive-Sequence Reactive Current"
      m_ur_bus1 = "m:ur:bus1", "p.u., Voltage, Real Part"
      m_ui_bus1 = "m:ui:bus1", "p.u., Voltage, Imaginary Part"
      m_u_bus1 = "m:u:bus1", "p.u., Voltage, Magnitude"
      m_ir_bus1 = "m:ir:bus1", "p.u., Current, Real Part"
      m_ii_bus1 = "m:ii:bus1", "p.u., Current, Imaginary Part"
      m_i_bus1 = "m:i:bus1", "p.u., Current, Magnitude"
      m_phiui_bus1 = "m:phiui:bus1", "deg, Angle between Voltage and Current"
      m_phiu1i1_bus1 = "m:phiu1i1:bus1", "deg, Angle between Voltage and Current in positive sequence system"
      m_Tfct_bus1 = "m:Tfct:bus1", "s, Fault Clearing Time"
      m_Brkload_bus1 = "m:Brkload:bus1", "%, Breaker Loading"
      n_ur_bus1 = "n:ur:bus1", "p.u., Voltage, Real Part"
      n_ui_bus1 = "n:ui:bus1", "p.u., Voltage, Imaginary Part"
      n_u_bus1 = "n:u:bus1", "p.u., Voltage, Magnitude"
      n_upc_bus1 = "n:upc:bus1", "%, Voltage, Magnitude"
      n_u1_bus1 = "n:u1:bus1", "p.u., Voltage, Magnitude"
      n_u1pc_bus1 = "n:u1pc:bus1", "%, Voltage, Magnitude"
      n_u1r_bus1 = "n:u1r:bus1", "p.u., Positive-Sequence Voltage, Real Part"
      n_u1i_bus1 = "n:u1i:bus1", "p.u., Positive-Sequence Voltage, Imaginary Part"
      n_U_bus1 = "n:U:bus1", "kV, Line-Ground Voltage, Magnitude"
      n_Ul_bus1 = "n:Ul:bus1", "kV, Line-Line Voltage, Magnitude"
      n_phiu_bus1 = "n:phiu:bus1", "deg, Voltage, Angle"
      n_phiurel_bus1 = "n:phiurel:bus1", "deg, Voltage, Relative Angle"
      n_du_bus1 = "n:du:bus1", "%, Voltage Deviation"
      n_ishcr_bus1 = "n:ishcr:bus1", "p.u., Short-Circuit Current, real"
      n_ishci_bus1 = "n:ishci:bus1", "p.u., Short-Circuit Current, imag"
      n_ishc_bus1 = "n:ishc:bus1", "p.u., Short-Circuit Current, magnitude"
      n_Ishc_bus1 = "n:Ishc:bus1", "kA, Short-Circuit Current, magnitude"
      n_fe_bus1 = "n:fe:bus1", "p.u., Electrical Frequency"
      n_frdev_bus1 = "n:frdev:bus1", "Hz, average frequency"
      n_dfehz_bus1 = "n:dfehz:bus1", "Hz, Deviation of the El. Frequency"
      n_dfedt_bus1 = "n:dfedt:bus1", "p.u./s, Derivative of El. Frequency"
      n_fehz_bus1 = "n:fehz:bus1", "Hz, Electrical Frequency"
      n_frnom_bus1 = "n:frnom:bus1", "Hz, Nominal Frequency"
      s_Pext = "s:Pext", "MW, IN, Load: Active Power, continuous, double"
      s_Qext = "s:Qext", "Mvar, IN, Load: Reactive Power, continuous, double"
      s_igen = "s:igen", "p.u., IN, Generation: Current, continuous, double"
      s_phiuigen = "s:phiuigen", "rad, IN, Generation: Angle u,i, continuous, double"
      e_iopt_type = "e:iopt_type", "Allow Load-Ramp Event"
      e_uk = "e:uk", "%, Short Circuit Impedance"
      e_Pcu = "e:Pcu", "kW, Copper Losses"
      e_r2 = "e:r2", "p.u., Resistance r2"
      e_x2 = "e:x2", "p.u., Reactance x2"

    class ElmPvsys(Enum):
      _init_ = 'value __doc__'

      m_u1r_bus1 = "m:u1r:bus1", "p.u., Positive-Sequence-Voltage, Real Part"
      m_u1i_bus1 = "m:u1i:bus1", "p.u., Positive-Sequence-Voltage, Imaginary Part"
      m_u1_bus1 = "m:u1:bus1", "p.u., Positive-Sequence-Voltage, Magnitude"
      m_U1_bus1 = "m:U1:bus1", "kV, Line-Ground Positive-Sequence-Voltage, Magnitude"
      m_U1l_bus1 = "m:U1l:bus1", "kV, Line-Line Positive-Sequence-Voltage, Magnitude"
      m_phiu1_bus1 = "m:phiu1:bus1", "deg, Positive-Sequence-Voltage, Angle"
      m_inet_bus1 = "m:inet:bus1", "p.u., Current, Magnitude, referred to network"
      m_I_bus1 = "m:I:bus1", "kA, Current, Magnitude"
      m_phii_bus1 = "m:phii:bus1", "deg, Current, Angle"
      m_i1r_bus1 = "m:i1r:bus1", "p.u., Positive-Sequence Current, Real Part"
      m_i1i_bus1 = "m:i1i:bus1", "p.u., Positive-Sequence Current, Imaginary Part"
      m_i1_bus1 = "m:i1:bus1", "p.u., Positive-Sequence Current, Magnitude"
      m_I1_bus1 = "m:I1:bus1", "kA, Positive-Sequence Current, Magnitude"
      m_phii1_bus1 = "m:phii1:bus1", "deg, Positive-Sequence Current, Angle"
      m_P_bus1 = "m:P:bus1", "MW, Active Power"
      m_Q_bus1 = "m:Q:bus1", "Mvar, Reactive Power"
      m_S_bus1 = "m:S:bus1", "MVA, Apparent Power"
      m_cosphi_bus1 = "m:cosphi:bus1", "Power Factor"
      m_tanphi_bus1 = "m:tanphi:bus1", "tan(phi)"
      m_Psum_bus1 = "m:Psum:bus1", "MW, Total Active Power"
      m_Qsum_bus1 = "m:Qsum:bus1", "Mvar, Total Reactive Power"
      m_Ssum_bus1 = "m:Ssum:bus1", "MVA, Total Apparent Power"
      m_cosphisum_bus1 = "m:cosphisum:bus1", "Total Power Factor"
      m_tanphisum_bus1 = "m:tanphisum:bus1", "Total tan(phi)"
      m_Spu_bus1 = "m:Spu:bus1", "MVA/p.u., Apparent Power per p.u. Voltage"
      m_i1P_bus1 = "m:i1P:bus1", "p.u., Positive-Sequence Active Current"
      m_i1Q_bus1 = "m:i1Q:bus1", "p.u., Positive-Sequence Reactive Current"
      m_I1P_bus1 = "m:I1P:bus1", "kA, Positive-Sequence Active Current"
      m_I1Q_bus1 = "m:I1Q:bus1", "kA, Positive-Sequence Reactive Current"
      m_ur_bus1 = "m:ur:bus1", "p.u., Voltage, Real Part"
      m_ui_bus1 = "m:ui:bus1", "p.u., Voltage, Imaginary Part"
      m_u_bus1 = "m:u:bus1", "p.u., Voltage, Magnitude"
      m_ir_bus1 = "m:ir:bus1", "p.u., Current, Real Part"
      m_ii_bus1 = "m:ii:bus1", "p.u., Current, Imaginary Part"
      m_i_bus1 = "m:i:bus1", "p.u., Current, Magnitude"
      m_phiui_bus1 = "m:phiui:bus1", "deg, Angle between Voltage and Current"
      m_phiu1i1_bus1 = "m:phiu1i1:bus1", "deg, Angle between Voltage and Current in positive sequence system"
      m_Tfct_bus1 = "m:Tfct:bus1", "s, Fault Clearing Time"
      m_Brkload_bus1 = "m:Brkload:bus1", "%, Breaker Loading"
      n_ur_bus1 = "n:ur:bus1", "p.u., Voltage, Real Part"
      n_ui_bus1 = "n:ui:bus1", "p.u., Voltage, Imaginary Part"
      n_u_bus1 = "n:u:bus1", "p.u., Voltage, Magnitude"
      n_upc_bus1 = "n:upc:bus1", "%, Voltage, Magnitude"
      n_u1_bus1 = "n:u1:bus1", "p.u., Voltage, Magnitude"
      n_u1pc_bus1 = "n:u1pc:bus1", "%, Voltage, Magnitude"
      n_u1r_bus1 = "n:u1r:bus1", "p.u., Positive-Sequence Voltage, Real Part"
      n_u1i_bus1 = "n:u1i:bus1", "p.u., Positive-Sequence Voltage, Imaginary Part"
      n_U_bus1 = "n:U:bus1", "kV, Line-Ground Voltage, Magnitude"
      n_Ul_bus1 = "n:Ul:bus1", "kV, Line-Line Voltage, Magnitude"
      n_phiu_bus1 = "n:phiu:bus1", "deg, Voltage, Angle"
      n_phiurel_bus1 = "n:phiurel:bus1", "deg, Voltage, Relative Angle"
      n_du_bus1 = "n:du:bus1", "%, Voltage Deviation"
      n_ishcr_bus1 = "n:ishcr:bus1", "p.u., Short-Circuit Current, real"
      n_ishci_bus1 = "n:ishci:bus1", "p.u., Short-Circuit Current, imag"
      n_ishc_bus1 = "n:ishc:bus1", "p.u., Short-Circuit Current, magnitude"
      n_Ishc_bus1 = "n:Ishc:bus1", "kA, Short-Circuit Current, magnitude"
      n_fe_bus1 = "n:fe:bus1", "p.u., Electrical Frequency"
      n_frdev_bus1 = "n:frdev:bus1", "Hz, average frequency"
      n_dfehz_bus1 = "n:dfehz:bus1", "Hz, Deviation of the El. Frequency"
      n_dfedt_bus1 = "n:dfedt:bus1", "p.u./s, Derivative of El. Frequency"
      n_fehz_bus1 = "n:fehz:bus1", "Hz, Electrical Frequency"
      n_frnom_bus1 = "n:frnom:bus1", "Hz, Nominal Frequency"
      s_f0 = "s:f0", "p.u., IN, Frequency, input, continuous, double"
      s_F0Hz = "s:F0Hz", "Hz, IN, Frequency input (Hz), continuous, double"
      s_fref = "s:fref", "p.u., IN, Reference frequency, continuous, double"
      s_id_ref = "s:id_ref", "p.u., IN, d-axis current reference, continuous, double"
      s_iq_ref = "s:iq_ref", "p.u., IN, q-axis current reference, continuous, double"
      s_u1r_in = "s:u1r_in", "p.u., IN, Voltage input, real part, continuous, double"
      s_u1i_in = "s:u1i_in", "p.u., IN, Voltage input, imaginary part, continuous, double"
      s_cosref = "s:cosref", "IN, cos(dq-reference angle) continuous, double"
      s_sinref = "s:sinref", "IN, sin(dq-reference angle) continuous, double"
      s_xspeed = "s:xspeed", "p.u., OUT, Frequency, continuous, double"
      s_id = "s:id", "p.u., OUT, Current, d-Axis continuous, double"
      s_iq = "s:iq", "p.u., OUT, Current, q-Axis continuous, double"
      c_sgnn = "c:sgnn", "MVA, Total rated apparent power (ngnum*sgn)"
      c_cosn = "c:cosn", "Nominal power factor"
      c_hpi = "c:hpi", "rad/s, Nominal angular frequency"
      c_umin = "c:umin", "p.u., Min. operation voltage"
      c_phiu = "c:phiu", "rad, Voltage angle"
      e_iSimModel = "e:iSimModel", "Model"
      e_iAstabint = "e:iAstabint", "A-stable integration algorithm"
      e_umin = "e:umin", "p.u., Switch-off threshold"
      e_uonthr = "e:uonthr", "p.u., Switch-on threshold"
      e_Tondelay = "e:Tondelay", "s, Switch-on delay"
      e_uk = "e:uk", "%, Short circuit impedance"
      e_Pcu = "e:Pcu", "kW, Copper losses"
      e_R0toR1 = "e:R0toR1", "R0/R1 ratio"
      e_X0toX1 = "e:X0toX1", "X0/X1 ratio"
      e_iDelay = "e:iDelay", "dq-reference angle delay (if dq-reference signals not connected)"
      e_Tdelay = "e:Tdelay", "s, Delay time constant"
      e_iUseNegSeqAngle = "e:iUseNegSeqAngle", "Current source model: Use negative sequence angle for negative sequence current source"
      e_iUseZsr = "e:iUseZsr", "Use series reactor impedance, if negative sequence voltage signals are connected"
      e_r2 = "e:r2", "p.u., Resistance r2"
      e_x2 = "e:x2", "p.u., Reactance x2"
      e_pinput = "e:pinput", "Active Power signal"
      e_qinput = "e:qinput", "Reactive Power signal"

    class ElmSecctrl(Enum):
      _init_ = 'value __doc__'


    class ElmShnt(Enum):
      _init_ = 'value __doc__'

      m_u1r_bus1 = "m:u1r:bus1", "p.u., Positive-Sequence-Voltage, Real Part"
      m_u1i_bus1 = "m:u1i:bus1", "p.u., Positive-Sequence-Voltage, Imaginary Part"
      m_u1_bus1 = "m:u1:bus1", "p.u., Positive-Sequence-Voltage, Magnitude"
      m_U1_bus1 = "m:U1:bus1", "kV, Line-Ground Positive-Sequence-Voltage, Magnitude"
      m_U1l_bus1 = "m:U1l:bus1", "kV, Line-Line Positive-Sequence-Voltage, Magnitude"
      m_phiu1_bus1 = "m:phiu1:bus1", "deg, Positive-Sequence-Voltage, Angle"
      m_inet_bus1 = "m:inet:bus1", "p.u., Current, Magnitude, referred to network"
      m_I_bus1 = "m:I:bus1", "kA, Current, Magnitude"
      m_phii_bus1 = "m:phii:bus1", "deg, Current, Angle"
      m_i1r_bus1 = "m:i1r:bus1", "p.u., Positive-Sequence Current, Real Part"
      m_i1i_bus1 = "m:i1i:bus1", "p.u., Positive-Sequence Current, Imaginary Part"
      m_i1_bus1 = "m:i1:bus1", "p.u., Positive-Sequence Current, Magnitude"
      m_I1_bus1 = "m:I1:bus1", "kA, Positive-Sequence Current, Magnitude"
      m_phii1_bus1 = "m:phii1:bus1", "deg, Positive-Sequence Current, Angle"
      m_P_bus1 = "m:P:bus1", "MW, Active Power"
      m_Q_bus1 = "m:Q:bus1", "Mvar, Reactive Power"
      m_S_bus1 = "m:S:bus1", "MVA, Apparent Power"
      m_cosphi_bus1 = "m:cosphi:bus1", "Power Factor"
      m_tanphi_bus1 = "m:tanphi:bus1", "tan(phi)"
      m_Psum_bus1 = "m:Psum:bus1", "MW, Total Active Power"
      m_Qsum_bus1 = "m:Qsum:bus1", "Mvar, Total Reactive Power"
      m_Ssum_bus1 = "m:Ssum:bus1", "MVA, Total Apparent Power"
      m_cosphisum_bus1 = "m:cosphisum:bus1", "Total Power Factor"
      m_tanphisum_bus1 = "m:tanphisum:bus1", "Total tan(phi)"
      m_Spu_bus1 = "m:Spu:bus1", "MVA/p.u., Apparent Power per p.u. Voltage"
      m_i1P_bus1 = "m:i1P:bus1", "p.u., Positive-Sequence Active Current"
      m_i1Q_bus1 = "m:i1Q:bus1", "p.u., Positive-Sequence Reactive Current"
      m_I1P_bus1 = "m:I1P:bus1", "kA, Positive-Sequence Active Current"
      m_I1Q_bus1 = "m:I1Q:bus1", "kA, Positive-Sequence Reactive Current"
      m_ur_bus1 = "m:ur:bus1", "p.u., Voltage, Real Part"
      m_ui_bus1 = "m:ui:bus1", "p.u., Voltage, Imaginary Part"
      m_u_bus1 = "m:u:bus1", "p.u., Voltage, Magnitude"
      m_ir_bus1 = "m:ir:bus1", "p.u., Current, Real Part"
      m_ii_bus1 = "m:ii:bus1", "p.u., Current, Imaginary Part"
      m_i_bus1 = "m:i:bus1", "p.u., Current, Magnitude"
      m_phiui_bus1 = "m:phiui:bus1", "deg, Angle between Voltage and Current"
      m_phiu1i1_bus1 = "m:phiu1i1:bus1", "deg, Angle between Voltage and Current in positive sequence system"
      m_Tfct_bus1 = "m:Tfct:bus1", "s, Fault Clearing Time"
      m_Brkload_bus1 = "m:Brkload:bus1", "%, Breaker Loading"
      n_ur_bus1 = "n:ur:bus1", "p.u., Voltage, Real Part"
      n_ui_bus1 = "n:ui:bus1", "p.u., Voltage, Imaginary Part"
      n_u_bus1 = "n:u:bus1", "p.u., Voltage, Magnitude"
      n_upc_bus1 = "n:upc:bus1", "%, Voltage, Magnitude"
      n_u1_bus1 = "n:u1:bus1", "p.u., Voltage, Magnitude"
      n_u1pc_bus1 = "n:u1pc:bus1", "%, Voltage, Magnitude"
      n_u1r_bus1 = "n:u1r:bus1", "p.u., Positive-Sequence Voltage, Real Part"
      n_u1i_bus1 = "n:u1i:bus1", "p.u., Positive-Sequence Voltage, Imaginary Part"
      n_U_bus1 = "n:U:bus1", "kV, Line-Ground Voltage, Magnitude"
      n_Ul_bus1 = "n:Ul:bus1", "kV, Line-Line Voltage, Magnitude"
      n_phiu_bus1 = "n:phiu:bus1", "deg, Voltage, Angle"
      n_phiurel_bus1 = "n:phiurel:bus1", "deg, Voltage, Relative Angle"
      n_du_bus1 = "n:du:bus1", "%, Voltage Deviation"
      n_ishcr_bus1 = "n:ishcr:bus1", "p.u., Short-Circuit Current, real"
      n_ishci_bus1 = "n:ishci:bus1", "p.u., Short-Circuit Current, imag"
      n_ishc_bus1 = "n:ishc:bus1", "p.u., Short-Circuit Current, magnitude"
      n_Ishc_bus1 = "n:Ishc:bus1", "kA, Short-Circuit Current, magnitude"
      n_fe_bus1 = "n:fe:bus1", "p.u., Electrical Frequency"
      n_frdev_bus1 = "n:frdev:bus1", "Hz, average frequency"
      n_dfehz_bus1 = "n:dfehz:bus1", "Hz, Deviation of the El. Frequency"
      n_dfedt_bus1 = "n:dfedt:bus1", "p.u./s, Derivative of El. Frequency"
      n_fehz_bus1 = "n:fehz:bus1", "Hz, Electrical Frequency"
      n_frnom_bus1 = "n:frnom:bus1", "Hz, Nominal Frequency"
      s_u_r = "s:u_r", "p.u., IN, Voltage, Real Part, continuous, double"
      s_u_i = "s:u_i", "p.u., IN, Voltage, Imaginary Part continuous, double"
      s_i_r = "s:i_r", "p.u., IN, Current, Real Part, continuous, double"
      s_i_i = "s:i_i", "p.u., IN, Current, Imaginary Part continuous, double"
      c_z_r = "c:z:r", "p.u., Shunt-Impedance, Real Part"
      c_z_i = "c:z:i", "p.u., Shunt-Impedance, Imaginary Part"
      c_z_phi = "c:z:phi", "deg, Shunt-Impedance, Angle"
      c_z_mag = "c:z:mag", "p.u., Shunt-Impedance, Magnitude"
      c_z_phirad = "c:z:phirad", "rad, Shunt-Impedance, Angle"
      c_cn = "c:cn", "Connection Mode"
      c_frnom = "c:frnom", "Hz, Nominal Frequency"
      c_hpi = "c:hpi", "rad/s, Nominal Angular Frequency"
      c_b0 = "c:b0", "p.u., Susceptance to Ground"
      c_ncapa = "c:ncapa", "Number of Stages in Parallel"
      c_ncapx = "c:ncapx", "Max. No. of Steps"
      c_Qmax = "c:Qmax", "Mvar, Max. Rated Reactive Power"
      e_iIntTapCtrl = "e:iIntTapCtrl", "Use Integrated Tap Controller"

    class ElmSind(Enum):
      _init_ = 'value __doc__'

      m_u1r_bus1 = "m:u1r:bus1", "p.u., Positive-Sequence-Voltage, Real Part"
      m_u1r_bus2 = "m:u1r:bus2", "p.u., Positive-Sequence-Voltage, Real Part"
      m_u1i_bus1 = "m:u1i:bus1", "p.u., Positive-Sequence-Voltage, Imaginary Part"
      m_u1i_bus2 = "m:u1i:bus2", "p.u., Positive-Sequence-Voltage, Imaginary Part"
      m_u1_bus1 = "m:u1:bus1", "p.u., Positive-Sequence-Voltage, Magnitude"
      m_u1_bus2 = "m:u1:bus2", "p.u., Positive-Sequence-Voltage, Magnitude"
      m_U1_bus1 = "m:U1:bus1", "kV, Line-Ground Positive-Sequence-Voltage, Magnitude"
      m_U1_bus2 = "m:U1:bus2", "kV, Line-Ground Positive-Sequence-Voltage, Magnitude"
      m_U1l_bus1 = "m:U1l:bus1", "kV, Line-Line Positive-Sequence-Voltage, Magnitude"
      m_U1l_bus2 = "m:U1l:bus2", "kV, Line-Line Positive-Sequence-Voltage, Magnitude"
      m_phiu1_bus1 = "m:phiu1:bus1", "deg, Positive-Sequence-Voltage, Angle"
      m_phiu1_bus2 = "m:phiu1:bus2", "deg, Positive-Sequence-Voltage, Angle"
      m_inet_bus1 = "m:inet:bus1", "p.u., Current, Magnitude, referred to network"
      m_inet_bus2 = "m:inet:bus2", "p.u., Current, Magnitude, referred to network"
      m_I_bus1 = "m:I:bus1", "kA, Current, Magnitude"
      m_I_bus2 = "m:I:bus2", "kA, Current, Magnitude"
      m_phii_bus1 = "m:phii:bus1", "deg, Current, Angle"
      m_phii_bus2 = "m:phii:bus2", "deg, Current, Angle"
      m_i1r_bus1 = "m:i1r:bus1", "p.u., Positive-Sequence Current, Real Part"
      m_i1r_bus2 = "m:i1r:bus2", "p.u., Positive-Sequence Current, Real Part"
      m_i1i_bus1 = "m:i1i:bus1", "p.u., Positive-Sequence Current, Imaginary Part"
      m_i1i_bus2 = "m:i1i:bus2", "p.u., Positive-Sequence Current, Imaginary Part"
      m_i1_bus1 = "m:i1:bus1", "p.u., Positive-Sequence Current, Magnitude"
      m_i1_bus2 = "m:i1:bus2", "p.u., Positive-Sequence Current, Magnitude"
      m_I1_bus1 = "m:I1:bus1", "kA, Positive-Sequence Current, Magnitude"
      m_I1_bus2 = "m:I1:bus2", "kA, Positive-Sequence Current, Magnitude"
      m_phii1_bus1 = "m:phii1:bus1", "deg, Positive-Sequence Current, Angle"
      m_phii1_bus2 = "m:phii1:bus2", "deg, Positive-Sequence Current, Angle"
      m_P_bus1 = "m:P:bus1", "MW, Active Power"
      m_P_bus2 = "m:P:bus2", "MW, Active Power"
      m_Q_bus1 = "m:Q:bus1", "Mvar, Reactive Power"
      m_Q_bus2 = "m:Q:bus2", "Mvar, Reactive Power"
      m_S_bus1 = "m:S:bus1", "MVA, Apparent Power"
      m_S_bus2 = "m:S:bus2", "MVA, Apparent Power"
      m_cosphi_bus1 = "m:cosphi:bus1", "Power Factor"
      m_cosphi_bus2 = "m:cosphi:bus2", "Power Factor"
      m_tanphi_bus1 = "m:tanphi:bus1", "tan(phi)"
      m_tanphi_bus2 = "m:tanphi:bus2", "tan(phi)"
      m_Psum_bus1 = "m:Psum:bus1", "MW, Total Active Power"
      m_Psum_bus2 = "m:Psum:bus2", "MW, Total Active Power"
      m_Qsum_bus1 = "m:Qsum:bus1", "Mvar, Total Reactive Power"
      m_Qsum_bus2 = "m:Qsum:bus2", "Mvar, Total Reactive Power"
      m_Ssum_bus1 = "m:Ssum:bus1", "MVA, Total Apparent Power"
      m_Ssum_bus2 = "m:Ssum:bus2", "MVA, Total Apparent Power"
      m_cosphisum_bus1 = "m:cosphisum:bus1", "Total Power Factor"
      m_cosphisum_bus2 = "m:cosphisum:bus2", "Total Power Factor"
      m_tanphisum_bus1 = "m:tanphisum:bus1", "Total tan(phi)"
      m_tanphisum_bus2 = "m:tanphisum:bus2", "Total tan(phi)"
      m_Spu_bus1 = "m:Spu:bus1", "MVA/p.u., Apparent Power per p.u. Voltage"
      m_Spu_bus2 = "m:Spu:bus2", "MVA/p.u., Apparent Power per p.u. Voltage"
      m_i1P_bus1 = "m:i1P:bus1", "p.u., Positive-Sequence Active Current"
      m_i1P_bus2 = "m:i1P:bus2", "p.u., Positive-Sequence Active Current"
      m_i1Q_bus1 = "m:i1Q:bus1", "p.u., Positive-Sequence Reactive Current"
      m_i1Q_bus2 = "m:i1Q:bus2", "p.u., Positive-Sequence Reactive Current"
      m_I1P_bus1 = "m:I1P:bus1", "kA, Positive-Sequence Active Current"
      m_I1P_bus2 = "m:I1P:bus2", "kA, Positive-Sequence Active Current"
      m_I1Q_bus1 = "m:I1Q:bus1", "kA, Positive-Sequence Reactive Current"
      m_I1Q_bus2 = "m:I1Q:bus2", "kA, Positive-Sequence Reactive Current"
      m_ur_bus1 = "m:ur:bus1", "p.u., Voltage, Real Part"
      m_ur_bus2 = "m:ur:bus2", "p.u., Voltage, Real Part"
      m_ui_bus1 = "m:ui:bus1", "p.u., Voltage, Imaginary Part"
      m_ui_bus2 = "m:ui:bus2", "p.u., Voltage, Imaginary Part"
      m_u_bus1 = "m:u:bus1", "p.u., Voltage, Magnitude"
      m_u_bus2 = "m:u:bus2", "p.u., Voltage, Magnitude"
      m_ir_bus1 = "m:ir:bus1", "p.u., Current, Real Part"
      m_ir_bus2 = "m:ir:bus2", "p.u., Current, Real Part"
      m_ii_bus1 = "m:ii:bus1", "p.u., Current, Imaginary Part"
      m_ii_bus2 = "m:ii:bus2", "p.u., Current, Imaginary Part"
      m_i_bus1 = "m:i:bus1", "p.u., Current, Magnitude"
      m_i_bus2 = "m:i:bus2", "p.u., Current, Magnitude"
      m_phiui_bus1 = "m:phiui:bus1", "deg, Angle between Voltage and Current"
      m_phiui_bus2 = "m:phiui:bus2", "deg, Angle between Voltage and Current"
      m_phiu1i1_bus1 = "m:phiu1i1:bus1", "deg, Angle between Voltage and Current in positive sequence system"
      m_phiu1i1_bus2 = "m:phiu1i1:bus2", "deg, Angle between Voltage and Current in positive sequence system"
      m_Tfct_bus1 = "m:Tfct:bus1", "s, Fault Clearing Time"
      m_Tfct_bus2 = "m:Tfct:bus2", "s, Fault Clearing Time"
      m_Brkload_bus1 = "m:Brkload:bus1", "%, Breaker Loading"
      m_Brkload_bus2 = "m:Brkload:bus2", "%, Breaker Loading"
      n_ur_bus1 = "n:ur:bus1", "p.u., Voltage, Real Part"
      n_ur_bus2 = "n:ur:bus2", "p.u., Voltage, Real Part"
      n_ui_bus1 = "n:ui:bus1", "p.u., Voltage, Imaginary Part"
      n_ui_bus2 = "n:ui:bus2", "p.u., Voltage, Imaginary Part"
      n_u_bus1 = "n:u:bus1", "p.u., Voltage, Magnitude"
      n_u_bus2 = "n:u:bus2", "p.u., Voltage, Magnitude"
      n_upc_bus1 = "n:upc:bus1", "%, Voltage, Magnitude"
      n_upc_bus2 = "n:upc:bus2", "%, Voltage, Magnitude"
      n_u1_bus1 = "n:u1:bus1", "p.u., Voltage, Magnitude"
      n_u1_bus2 = "n:u1:bus2", "p.u., Voltage, Magnitude"
      n_u1pc_bus1 = "n:u1pc:bus1", "%, Voltage, Magnitude"
      n_u1pc_bus2 = "n:u1pc:bus2", "%, Voltage, Magnitude"
      n_u1r_bus1 = "n:u1r:bus1", "p.u., Positive-Sequence Voltage, Real Part"
      n_u1r_bus2 = "n:u1r:bus2", "p.u., Positive-Sequence Voltage, Real Part"
      n_u1i_bus1 = "n:u1i:bus1", "p.u., Positive-Sequence Voltage, Imaginary Part"
      n_u1i_bus2 = "n:u1i:bus2", "p.u., Positive-Sequence Voltage, Imaginary Part"
      n_U_bus1 = "n:U:bus1", "kV, Line-Ground Voltage, Magnitude"
      n_U_bus2 = "n:U:bus2", "kV, Line-Ground Voltage, Magnitude"
      n_Ul_bus1 = "n:Ul:bus1", "kV, Line-Line Voltage, Magnitude"
      n_Ul_bus2 = "n:Ul:bus2", "kV, Line-Line Voltage, Magnitude"
      n_phiu_bus1 = "n:phiu:bus1", "deg, Voltage, Angle"
      n_phiu_bus2 = "n:phiu:bus2", "deg, Voltage, Angle"
      n_phiurel_bus1 = "n:phiurel:bus1", "deg, Voltage, Relative Angle"
      n_phiurel_bus2 = "n:phiurel:bus2", "deg, Voltage, Relative Angle"
      n_du_bus1 = "n:du:bus1", "%, Voltage Deviation"
      n_du_bus2 = "n:du:bus2", "%, Voltage Deviation"
      n_ishcr_bus1 = "n:ishcr:bus1", "p.u., Short-Circuit Current, real"
      n_ishcr_bus2 = "n:ishcr:bus2", "p.u., Short-Circuit Current, real"
      n_ishci_bus1 = "n:ishci:bus1", "p.u., Short-Circuit Current, imag"
      n_ishci_bus2 = "n:ishci:bus2", "p.u., Short-Circuit Current, imag"
      n_ishc_bus1 = "n:ishc:bus1", "p.u., Short-Circuit Current, magnitude"
      n_ishc_bus2 = "n:ishc:bus2", "p.u., Short-Circuit Current, magnitude"
      n_Ishc_bus1 = "n:Ishc:bus1", "kA, Short-Circuit Current, magnitude"
      n_Ishc_bus2 = "n:Ishc:bus2", "kA, Short-Circuit Current, magnitude"
      n_fe_bus1 = "n:fe:bus1", "p.u., Electrical Frequency"
      n_fe_bus2 = "n:fe:bus2", "p.u., Electrical Frequency"
      n_frdev_bus1 = "n:frdev:bus1", "Hz, average frequency"
      n_frdev_bus2 = "n:frdev:bus2", "Hz, average frequency"
      n_dfehz_bus1 = "n:dfehz:bus1", "Hz, Deviation of the El. Frequency"
      n_dfehz_bus2 = "n:dfehz:bus2", "Hz, Deviation of the El. Frequency"
      n_dfedt_bus1 = "n:dfedt:bus1", "p.u./s, Derivative of El. Frequency"
      n_dfedt_bus2 = "n:dfedt:bus2", "p.u./s, Derivative of El. Frequency"
      n_fehz_bus1 = "n:fehz:bus1", "Hz, Electrical Frequency"
      n_fehz_bus2 = "n:fehz:bus2", "Hz, Electrical Frequency"
      n_frnom_bus1 = "n:frnom:bus1", "Hz, Nominal Frequency"
      n_frnom_bus2 = "n:frnom:bus2", "Hz, Nominal Frequency"
      s_Rin = "s:Rin", "Ohm, IN, Resistance (Input), continuous, double"
      s_Xin = "s:Xin", "Ohm, IN, Reactance (Input), continuous, double"
      c_Zind_1_r = "c:Zind_1:r", "Ohm, Impedance, Real Part"
      c_Zind_1_i = "c:Zind_1:i", "Ohm, Impedance, Imaginary Part"
      c_Zind_1_phi = "c:Zind_1:phi", "deg, Impedance, Angle"
      c_Zind_1_mag = "c:Zind_1:mag", "Ohm, Impedance, Magnitude"
      c_Zind_1_phirad = "c:Zind_1:phirad", "rad, Impedance, Angle"
      c_loading = "c:loading", "%, Loading"
      c_Inom = "c:Inom", "kA, Nominal Current"
      e_iAstabint = "e:iAstabint", "A-stable integration algorithm"
      e_emtLinSig = "e:emtLinSig", "Variable inductance/reactance via input signals"
      e_uk = "e:uk", "%, Short-Circuit Voltage uk"
      e_Zd = "e:Zd", "Ohm, Impedance (absolute) Zd"
      e_lrea = "e:lrea", "mH, Inductance, L"
      e_xrea = "e:xrea", "Ohm, Reactance, X"
      e_Pcu = "e:Pcu", "kW, Copper Losses"
      e_ukr = "e:ukr", "%, Short-Circuit Voltage (Re(uk)) ukr"
      e_rrea = "e:rrea", "Ohm, Resistance, R"
      e_R0toR1 = "e:R0toR1", "R0/R1 ratio"
      e_X0toX1 = "e:X0toX1", "X0/X1 ratio"

    class ElmStactrl(Enum):
      _init_ = 'value __doc__'


    class ElmSym(Enum):
      _init_ = 'value __doc__'

      m_u1r_bus1 = "m:u1r:bus1", "p.u., Positive-Sequence-Voltage, Real Part"
      m_u1i_bus1 = "m:u1i:bus1", "p.u., Positive-Sequence-Voltage, Imaginary Part"
      m_u1_bus1 = "m:u1:bus1", "p.u., Positive-Sequence-Voltage, Magnitude"
      m_U1_bus1 = "m:U1:bus1", "kV, Line-Ground Positive-Sequence-Voltage, Magnitude"
      m_U1l_bus1 = "m:U1l:bus1", "kV, Line-Line Positive-Sequence-Voltage, Magnitude"
      m_phiu1_bus1 = "m:phiu1:bus1", "deg, Positive-Sequence-Voltage, Angle"
      m_inet_bus1 = "m:inet:bus1", "p.u., Current, Magnitude, referred to network"
      m_I_bus1 = "m:I:bus1", "kA, Current, Magnitude"
      m_phii_bus1 = "m:phii:bus1", "deg, Current, Angle"
      m_i1r_bus1 = "m:i1r:bus1", "p.u., Positive-Sequence Current, Real Part"
      m_i1i_bus1 = "m:i1i:bus1", "p.u., Positive-Sequence Current, Imaginary Part"
      m_i1_bus1 = "m:i1:bus1", "p.u., Positive-Sequence Current, Magnitude"
      m_I1_bus1 = "m:I1:bus1", "kA, Positive-Sequence Current, Magnitude"
      m_phii1_bus1 = "m:phii1:bus1", "deg, Positive-Sequence Current, Angle"
      m_P_bus1 = "m:P:bus1", "MW, Active Power"
      m_Q_bus1 = "m:Q:bus1", "Mvar, Reactive Power"
      m_S_bus1 = "m:S:bus1", "MVA, Apparent Power"
      m_cosphi_bus1 = "m:cosphi:bus1", "Power Factor"
      m_tanphi_bus1 = "m:tanphi:bus1", "tan(phi)"
      m_Psum_bus1 = "m:Psum:bus1", "MW, Total Active Power"
      m_Qsum_bus1 = "m:Qsum:bus1", "Mvar, Total Reactive Power"
      m_Ssum_bus1 = "m:Ssum:bus1", "MVA, Total Apparent Power"
      m_cosphisum_bus1 = "m:cosphisum:bus1", "Total Power Factor"
      m_tanphisum_bus1 = "m:tanphisum:bus1", "Total tan(phi)"
      m_Spu_bus1 = "m:Spu:bus1", "MVA/p.u., Apparent Power per p.u. Voltage"
      m_i1P_bus1 = "m:i1P:bus1", "p.u., Positive-Sequence Active Current"
      m_i1Q_bus1 = "m:i1Q:bus1", "p.u., Positive-Sequence Reactive Current"
      m_I1P_bus1 = "m:I1P:bus1", "kA, Positive-Sequence Active Current"
      m_I1Q_bus1 = "m:I1Q:bus1", "kA, Positive-Sequence Reactive Current"
      m_ur_bus1 = "m:ur:bus1", "p.u., Voltage, Real Part"
      m_ui_bus1 = "m:ui:bus1", "p.u., Voltage, Imaginary Part"
      m_u_bus1 = "m:u:bus1", "p.u., Voltage, Magnitude"
      m_ir_bus1 = "m:ir:bus1", "p.u., Current, Real Part"
      m_ii_bus1 = "m:ii:bus1", "p.u., Current, Imaginary Part"
      m_i_bus1 = "m:i:bus1", "p.u., Current, Magnitude"
      m_phiui_bus1 = "m:phiui:bus1", "deg, Angle between Voltage and Current"
      m_phiu1i1_bus1 = "m:phiu1i1:bus1", "deg, Angle between Voltage and Current in positive sequence system"
      m_Tfct_bus1 = "m:Tfct:bus1", "s, Fault Clearing Time"
      m_Brkload_bus1 = "m:Brkload:bus1", "%, Breaker Loading"
      n_ur_bus1 = "n:ur:bus1", "p.u., Voltage, Real Part"
      n_ui_bus1 = "n:ui:bus1", "p.u., Voltage, Imaginary Part"
      n_u_bus1 = "n:u:bus1", "p.u., Voltage, Magnitude"
      n_upc_bus1 = "n:upc:bus1", "%, Voltage, Magnitude"
      n_u1_bus1 = "n:u1:bus1", "p.u., Voltage, Magnitude"
      n_u1pc_bus1 = "n:u1pc:bus1", "%, Voltage, Magnitude"
      n_u1r_bus1 = "n:u1r:bus1", "p.u., Positive-Sequence Voltage, Real Part"
      n_u1i_bus1 = "n:u1i:bus1", "p.u., Positive-Sequence Voltage, Imaginary Part"
      n_U_bus1 = "n:U:bus1", "kV, Line-Ground Voltage, Magnitude"
      n_Ul_bus1 = "n:Ul:bus1", "kV, Line-Line Voltage, Magnitude"
      n_phiu_bus1 = "n:phiu:bus1", "deg, Voltage, Angle"
      n_phiurel_bus1 = "n:phiurel:bus1", "deg, Voltage, Relative Angle"
      n_du_bus1 = "n:du:bus1", "%, Voltage Deviation"
      n_ishcr_bus1 = "n:ishcr:bus1", "p.u., Short-Circuit Current, real"
      n_ishci_bus1 = "n:ishci:bus1", "p.u., Short-Circuit Current, imag"
      n_ishc_bus1 = "n:ishc:bus1", "p.u., Short-Circuit Current, magnitude"
      n_Ishc_bus1 = "n:Ishc:bus1", "kA, Short-Circuit Current, magnitude"
      n_fe_bus1 = "n:fe:bus1", "p.u., Electrical Frequency"
      n_frdev_bus1 = "n:frdev:bus1", "Hz, average frequency"
      n_dfehz_bus1 = "n:dfehz:bus1", "Hz, Deviation of the El. Frequency"
      n_dfedt_bus1 = "n:dfedt:bus1", "p.u./s, Derivative of El. Frequency"
      n_fehz_bus1 = "n:fehz:bus1", "Hz, Electrical Frequency"
      n_frnom_bus1 = "n:frnom:bus1", "Hz, Nominal Frequency"
      s_ve = "s:ve", "p.u., IN, Excitation Voltage, continuous, double"
      s_pt = "s:pt", "p.u., IN, Turbine Power, continuous, double"
      s_xmdm = "s:xmdm", "p.u., IN, Torque Input, continuous, double"
      s_dw = "s:dw", "p.u., IN, dw, continuous, double"
      s_fref = "s:fref", "p.u., IN, Reference Frequency, continuous, double"
      s_freflocal = "s:freflocal", "p.u., IN, Local Reference Frequency, continuous, double"
      s_psid = "s:psid", "OUT, Stator Flux, d-axis, continuous, double"
      s_psiq = "s:psiq", "p.u., OUT, Stator Flux, q-axis, continuous, double"
      s_outofstep = "s:outofstep", "OUT, Out of Step (pole slip) continuous, double"
      s_xme = "s:xme", "p.u., OUT, Electrical Torque, continuous, double"
      s_xmt = "s:xmt", "p.u., OUT, Mechanical Torque, continuous, double"
      s_xspeed = "s:xspeed", "p.u., OUT, Speed, continuous, double"
      s_ie = "s:ie", "p.u., OUT, Excitation Current (non-reciprocal p.u. system) continuous, double"
      s_pgt = "s:pgt", "p.u., OUT, Electrical Power, continuous, double"
      s_xphi = "s:xphi", "rad, OUT, Rotor position d-axis referred to the reference bus voltage, continuous, double"
      s_fe = "s:fe", "p.u., OUT, Frequency Output, continuous, double"
      s_ut = "s:ut", "p.u., OUT, Terminal voltage, continuous, double"
      s_utr = "s:utr", "p.u., OUT, Terminal voltage, real part, continuous, double"
      s_uti = "s:uti", "p.u., OUT, Terminal voltage, imaginary part, continuous, double"
      s_cur1 = "s:cur1", "p.u., OUT, Positive-sequence current, magnitude, continuous, double"
      s_cur1r = "s:cur1r", "p.u., OUT, Positive-sequence current, real part, continuous, double"
      s_cur1i = "s:cur1i", "p.u., OUT, Positive-sequence current, imaginary part, continuous, double"
      s_P1 = "s:P1", "MW, OUT, Positive-sequence, active power continuous, double"
      s_Q1 = "s:Q1", "Mvar, OUT, Positive-sequence, reactive power, continuous, double"
      s_fipol = "s:fipol", "deg, OUT, Internal rotor angle, continuous, double"
      s_psi1d = "s:psi1d", "p.u., STATE, Flux in 1d-damper winding, continuous, double"
      s_psifd = "s:psifd", "p.u., STATE, Excitation Flux continuous, double"
      s_psi1q = "s:psi1q", "p.u., STATE, Flux in 1q-damper winding, continuous, double"
      s_psi2q = "s:psi2q", "p.u., STATE, Flux in 2q-damper winding, continuous, double"
      s_speed = "s:speed", "p.u., STATE, Speed, continuous, double"
      s_phi = "s:phi", "rad, STATE, Rotor position d-axis referred to the reference bus voltage, continuous, double"
      s_psi1d_dt = "s:psi1d:dt", "p.u./s, d/dt, Flux in 1d-damper winding, derivative, continuous, double"
      s_psifd_dt = "s:psifd:dt", "p.u./s, d/dt, Excitation Flux, derivative, continuous, double"
      s_psi1q_dt = "s:psi1q:dt", "p.u./s, d/dt, Flux in 1q-damper winding, derivative, continuous, double"
      s_psi2q_dt = "s:psi2q:dt", "p.u./s, d/dt, Flux in 2q-damper winding, derivative, continuous, double"
      s_speed_dt = "s:speed:dt", "p.u./s, d/dt, Speed, derivative, continuous, double"
      s_phi_dt = "s:phi:dt", "rad/s, d/dt, Rotor position d-axis referred to the reference bus voltage, derivative continuous, double"
      c_ud = "c:ud", "p.u., Voltage, d-axis"
      c_uq = "c:uq", "p.u., Voltage, q-axis"
      c_id = "c:id", "p.u., Stator Current, d-axis"
      c_iq = "c:iq", "p.u., Stator Current, q-axis"
      c_psim = "c:psim", "p.u., Magnetisation Flux"
      c_psidss = "c:psidss", "p.u., Subtransient Flux, d-axis"
      c_psiqss = "c:psiqss", "p.u., Subtransient Flux, q-axis"
      c_ussd = "c:ussd", "p.u., Subtransient Voltage, d-axis"
      c_ussq = "c:ussq", "p.u., Subtransient Voltage, q-axis"
      c_xmem = "c:xmem", "p.u., Electrical Torque"
      c_addmt = "c:addmt", "p.u., Additional Torque"
      c_xmdpe = "c:xmdpe", "p.u., Damping Torque based on power"
      c_xmdkd = "c:xmdkd", "p.u., Damping Torque"
      c_ifd = "c:ifd", "p.u., Current in excitation winding"
      c_i1d = "c:i1d", "p.u., Current in 1d-damper winding"
      c_i1q = "c:i1q", "p.u., Current in 1q-damper winding"
      c_i2q = "c:i2q", "p.u., Current in 2q-damper winding"
      c_iD = "c:iD", "p.u., Current in 1d-damper winding (non-reciprocal p.u. system)"
      c_ix = "c:ix", "p.u., Current in 2q-damper winding (non-reciprocal p.u. system)"
      c_iQ = "c:iQ", "p.u., Current in 1q-damper winding (non-reciprocal p.u. system)"
      c_psie = "c:psie", "p.u., Excitation Flux (non-reciprocal p.u. system)"
      c_psiD = "c:psiD", "p.u., Flux in 1d-damper winding (non-reciprocal p.u. system)"
      c_psiQ = "c:psiQ", "p.u., Flux in 1q-damper winding (non-reciprocal p.u. system)"
      c_psix = "c:psix", "p.u., Flux in 2q-damper winding (non-reciprocal p.u. system)"
      c_fi = "c:fi", "deg, Rotor position d-axis referred to the reference bus voltage"
      c_firot = "c:firot", "deg, External rotor angle"
      c_firel = "c:firel", "deg, Rotor angle referred to the reference machine rotor angle"
      c_dfrotx = "c:dfrotx", "deg, Maximum rotor angle difference"
      c_dfrotxM1 = "c:dfrotxM1", "Machine 1 used to obtain dfrotx"
      c_dfrotxM2 = "c:dfrotxM2", "Machine 2 used to obtain dfrotx"
      c_dorhz = "c:dorhz", "Hz, Speed Deviation"
      c_Ptspin = "c:Ptspin", "MW, Total Turbine Power of Area"
      c_Pnomspin = "c:Pnomspin", "MW, Total Nominal Power of Spinning Machines in Area"
      c_Presspin = "c:Presspin", "MW, Spinning Reserve of Area"
      c_Tag = "c:Tag", "s, Acceleration Time Constant"
      c_Tagav = "c:Tagav", "s, System Inertia of Area"
      c_speedinit = "c:speedinit", "p.u., Initial speed"
      c_xadu = "c:xadu", "p.u., Unsaturated mutual reactance, d-axis"
      c_xaqu = "c:xaqu", "p.u., Unsaturated mutual reactance, q-axis"
      c_xad = "c:xad", "p.u., Saturated mutual reactance, d-axis"
      c_xaq = "c:xaq", "p.u., Saturated mutual reactance q-axis"
      c_satd = "c:satd", "p.u., Saturation factor, d-axis"
      c_satq = "c:satq", "p.u., Saturation factor, q-axis"
      c_rfd = "c:rfd", "p.u., Resistance of excitation winding"
      c_xfd = "c:xfd", "p.u., Reactance of excitation winding"
      c_r1d = "c:r1d", "p.u., Resistance of 1d-damper winding"
      c_x1d = "c:x1d", "p.u., Reactance of 1d-damper winding"
      c_r1q = "c:r1q", "p.u., Resistance of 1q-damper winding"
      c_x1q = "c:x1q", "p.u., Reactance of 1q-damper winding"
      c_r2q = "c:r2q", "p.u., Resistance of 2q-damper winding"
      c_x2q = "c:x2q", "p.u., Reactance of 2q-damper winding"
      c_xd = "c:xd", "p.u., Synchronous Reactance, d-axis"
      c_xq = "c:xq", "p.u., Synchronous Reactance, q-axis"
      c_xdss = "c:xdss", "p.u., Subtransient Reactance, d-axis"
      c_xqss = "c:xqss", "p.u., Subtransient Reactance, q-axis"
      c_sgnn = "c:sgnn", "MVA, Total rated apparent power (ngnum*sgn)"
      c_c2 = "c:c2", "Voltage Coefficient (uknom/ugn)"
      c_cosn = "c:cosn", "Nominal-Power factor"
      c_hpi = "c:hpi", "rad/s, Nominal-Angular-Frequency"
      c_Qmin = "c:Qmin", "Mvar, Lower limit of reactive power"
      c_Qmax = "c:Qmax", "Mvar, Upper limit of reactive power"
      c_p_mag = "c:p_mag", "Participation, Magnitude"
      c_p_phi = "c:p_phi", "deg, Participation, Angle"
      c_p_mags = "c:p_mags", "Participation, Magnitude (signed)"
      c_lEVec_mag = "c:lEVec_mag", "Controllability, Magnitude"
      c_lEVec_phi = "c:lEVec_phi", "deg, Controllability, Angle"
      c_lEVec_mags = "c:lEVec_mags", "Controllability, Magnitude (signed)"
      c_rEVec_mag = "c:rEVec_mag", "Observability, Magnitude"
      c_rEVec_phi = "c:rEVec_phi", "deg, Observability, Angle"
      c_rEVec_mags = "c:rEVec_mags", "Observability, Magnitude (signed)"
      c_fe_bus = "c:fe_bus", "El. frequency"
      c_loading = "c:loading", "%, Loading"
      c_slipreset = "c:slipreset", "Reset out of step"
      c_Gm = "c:Gm", "Conversion Factor (=xadu/rfd)"
      c_ve_rated = "c:ve_rated", "p.u., Rated Excitation Voltage"
      c_Pmaxspin = "c:Pmaxspin", "MW, Total Maximum Power of Spinning Machines in Area"
      c_Pmax = "c:Pmax", "MW, Upper limit of active power"
      c_Td0s = "c:Td0s", "s, Td0'"
      c_mdmlp = "c:mdmlp", "p.u., Proportional Factor"
      c_mdmex = "c:mdmex", "Exponent"
      e_iAstabint = "e:iAstabint", "A-stable integration algorithm"
      e_efdBaseRatio = "e:efdBaseRatio", "Efd base ratio"
      e_ifdBaseType = "e:ifdBaseType", "Excitation base mode"
      e_c_pmdm = "e:c_pmdm", "Mdm"
      e_mdmlp = "e:mdmlp", "p.u., Proportional Factor"
      e_mdmex = "e:mdmex", "Exponent"
      e_Jme = "e:Jme", "kgm^2, Moment of inertia"
      e_gratio = "e:gratio", "Gear ratio"
      e_ctag = "e:ctag", "s, Acceleration time const. from type"
      e_ctagtot = "e:ctagtot", "s, Total acceleration time const."
      e_i_avr = "e:i_avr", "Use integrated AVR for motor starting"
      e_rf_st = "e:rf_st", "p.u., Starting field resistance"
      e_speed_th = "e:speed_th", "p.u., Trigger excitation at speed"
      e_ve_const = "e:ve_const", "p.u., Constant field voltage"
      e_ve_rated = "e:ve_rated", "p.u., Rated field voltage"
      t_i_exactConv = "t:i_exactConv", "Exact conversion of time constants"
      t_i_xme2 = "t:i_xme2", "Consider negative sequence torque"
      t_i_speedVar = "t:i_speedVar", "Effect of speed variation"
      t_i_extModel = "t:i_extModel", "Model"
      t_model_inp = "t:model_inp", "Model"
      t_iopt_data = "t:iopt_data", "Input parameters"
      t_iner_inp = "t:iner_inp", "Input mode"
      t_tag = "t:tag", "s, Acceleration time const. Tag (rated to Pgn)"
      t_tags = "t:tags", "s, Acceleration time const. Tag (rated to Sgn)"
      t_h = "t:h", "s, Inertia constant H (rated to Sgn)"
      t_hpn = "t:hpn", "s, Inertia constant H (rated to Pgn)"
      t_J = "t:J", "kgm^2, Moment of inertia J"
      t_polepairs = "t:polepairs", "No. of pole pairs"
      t_frequ = "t:frequ", "Hz, Nominal frequency"
      t_rstr = "t:rstr", "p.u., rstr"
      t_xl = "t:xl", "p.u., xl"
      t_i_xl = "t:i_xl", "xl is current dependent"
      t_iturbo = "t:iturbo", "Rotor type"
      t_tds = "t:tds", "s, Td'"
      t_tqs = "t:tqs", "s, Tq'"
      t_tds0 = "t:tds0", "s, Td0'"
      t_tqs0 = "t:tqs0", "s, Tq0'"
      t_tdss = "t:tdss", "s, Td''"
      t_tqss = "t:tqss", "s, Tq''"
      t_tdss0 = "t:tdss0", "s, Td0''"
      t_tqss0 = "t:tqss0", "s, Tq0''"
      t_xd = "t:xd", "p.u., xd"
      t_xq = "t:xq", "p.u., xq"
      t_xrl = "t:xrl", "p.u., xrld"
      t_xrlq = "t:xrlq", "p.u., xrlq"
      t_xds = "t:xds", "p.u., xd'"
      t_xqs = "t:xqs", "p.u., xq'"
      t_xdss = "t:xdss", "p.u., xd''"
      t_xqss = "t:xqss", "p.u., xq''"
      t_xstr = "t:xstr", "p.u., xstr"
      t_x1d = "t:x1d", "p.u., x1d"
      t_r1d = "t:r1d", "p.u., r1d"
      t_xfd = "t:xfd", "p.u., xfd"
      t_rfd = "t:rfd", "p.u., rfd"
      t_x1q = "t:x1q", "p.u., x1q"
      t_r1q = "t:r1q", "p.u., r1q"
      t_x2q = "t:x2q", "p.u., x2q"
      t_r2q = "t:r2q", "p.u., r2q"
      t_xrl2d = "t:xrl2d", "p.u., xrl2d"
      t_x2d = "t:x2d", "p.u., x2d"
      t_r2d = "t:r2d", "p.u., r2d"
      t_t1d = "t:t1d", "s, t1d"
      t_t2d = "t:t2d", "s, t2d"
      t_t3d = "t:t3d", "s, t3d"
      t_t4d = "t:t4d", "s, t4d"
      t_t5d = "t:t5d", "s, t5d"
      t_t6d = "t:t6d", "s, t6d"
      t_t7d = "t:t7d", "s, t7d"
      t_t8d = "t:t8d", "s, t8d"
      t_xad = "t:xad", "p.u., xad"
      t_xaq = "t:xaq", "p.u., xaq"
      t_x3q = "t:x3q", "p.u., x3q"
      t_r3q = "t:r3q", "p.u., r3q"
      t_t1q = "t:t1q", "s, t1q"
      t_t2q = "t:t2q", "s, t2q"
      t_t3q = "t:t3q", "s, t3q"
      t_t4q = "t:t4q", "s, t4q"
      t_t5q = "t:t5q", "s, t5q"
      t_t6q = "t:t6q", "s, t6q"
      t_x1dc = "t:x1dc", "p.u., x1dc"
      t_x1de = "t:x1de", "p.u., x1de"
      t_r1de = "t:r1de", "p.u., r1de"
      t_xfde = "t:xfde", "p.u., xfde"
      t_rfde = "t:rfde", "p.u., rfde"
      t_x0sy = "t:x0sy", "p.u., Reactance x0"
      t_r0sy = "t:r0sy", "p.u., Resistance r0"
      t_x1qc = "t:x1qc", "p.u., x1qc"
      t_x1qe = "t:x1qe", "p.u., x1qe"
      t_r1qe = "t:r1qe", "p.u., r1qe"
      t_psim = "t:psim", "p.u., Rotor flux"
      t_slipmin = "t:slipmin", "%, Minimum slip"
      t_curTab = "t:curTab", "p.u., Current"
      t_xlTab = "t:xlTab", "p.u., xl"
      t_depSmoothFac = "t:depSmoothFac", "%, Dependency curve smoothing factor"
      t_isat = "t:isat", "Main flux saturation"
      t_isatSalient = "t:isatSalient", "Saturation"
      t_sg10 = "t:sg10", "p.u., SG10"
      t_sg12 = "t:sg12", "p.u., SG12"
      t_sg10q = "t:sg10q", "p.u., SG10 q-axis"
      t_sg12q = "t:sg12q", "p.u., SG12 q-axis"
      t_satv = "t:satv", "p.u., Term. voltage"
      t_satse = "t:satse", "p.u., SG(u)"
      t_ifd = "t:ifd", "%, ie"
      t_satvq = "t:satvq", "p.u., Term. voltage"
      t_satseq = "t:satseq", "p.u., SG(u)"
      t_ifdq = "t:ifdq", "%, ie"
      t_smoothFac = "t:smoothFac", "%, Smoothing factor"
      t_dpu = "t:dpu", "Turbine shaft friction torque coefficient"
      t_dkd = "t:dkd", "Damping torque coefficient"
      t_dpe = "t:dpe", "Damping torque coefficient based on power"
      t_x2sy = "t:x2sy", "p.u., Reactance x2"
      t_r2sy = "t:r2sy", "p.u., Resistance r2"
      t_i_speedD = "t:i_speedD", "Speed deviation based on"
      r_c_pmdm = "r:c_pmdm", "Mdm"

    class ElmTerm(Enum):
      _init_ = 'value __doc__'

      m_ur = "m:ur", "p.u., Voltage, Real Part"
      m_ui = "m:ui", "p.u., Voltage, Imaginary Part"
      m_u = "m:u", "p.u., Voltage, Magnitude"
      m_upc = "m:upc", "%, Voltage, Magnitude"
      m_u1 = "m:u1", "p.u., Voltage, Magnitude"
      m_u1pc = "m:u1pc", "%, Voltage, Magnitude"
      m_u1r = "m:u1r", "p.u., Positive-Sequence Voltage, Real Part"
      m_u1i = "m:u1i", "p.u., Positive-Sequence Voltage, Imaginary Part"
      m_U = "m:U", "kV, Line-Ground Voltage, Magnitude"
      m_Ul = "m:Ul", "kV, Line-Line Voltage, Magnitude"
      m_phiu = "m:phiu", "deg, Voltage, Angle"
      m_phiurel = "m:phiurel", "deg, Voltage, Relative Angle"
      m_du = "m:du", "%, Voltage Deviation"
      m_ishcr = "m:ishcr", "p.u., Short-Circuit Current, real"
      m_ishci = "m:ishci", "p.u., Short-Circuit Current, imag"
      m_ishc = "m:ishc", "p.u., Short-Circuit Current, magnitude"
      m_Ishc = "m:Ishc", "kA, Short-Circuit Current, magnitude"
      m_fe = "m:fe", "p.u., Electrical Frequency"
      m_frdev = "m:frdev", "Hz, average frequency"
      m_dfehz = "m:dfehz", "Hz, Deviation of the El. Frequency"
      m_dfedt = "m:dfedt", "p.u./s, Derivative of El. Frequency"
      m_fehz = "m:fehz", "Hz, Electrical Frequency"
      m_frnom = "m:frnom", "Hz, Nominal Frequency"

    class ElmTr2(Enum):
      _init_ = 'value __doc__'

      m_u1r_bushv = "m:u1r:bushv", "p.u., Positive-Sequence-Voltage, Real Part"
      m_u1r_buslv = "m:u1r:buslv", "p.u., Positive-Sequence-Voltage, Real Part"
      m_u1i_bushv = "m:u1i:bushv", "p.u., Positive-Sequence-Voltage, Imaginary Part"
      m_u1i_buslv = "m:u1i:buslv", "p.u., Positive-Sequence-Voltage, Imaginary Part"
      m_u1_bushv = "m:u1:bushv", "p.u., Positive-Sequence-Voltage, Magnitude"
      m_u1_buslv = "m:u1:buslv", "p.u., Positive-Sequence-Voltage, Magnitude"
      m_U1_bushv = "m:U1:bushv", "kV, Line-Ground Positive-Sequence-Voltage, Magnitude"
      m_U1_buslv = "m:U1:buslv", "kV, Line-Ground Positive-Sequence-Voltage, Magnitude"
      m_U1l_bushv = "m:U1l:bushv", "kV, Line-Line Positive-Sequence-Voltage, Magnitude"
      m_U1l_buslv = "m:U1l:buslv", "kV, Line-Line Positive-Sequence-Voltage, Magnitude"
      m_phiu1_bushv = "m:phiu1:bushv", "deg, Positive-Sequence-Voltage, Angle"
      m_phiu1_buslv = "m:phiu1:buslv", "deg, Positive-Sequence-Voltage, Angle"
      m_inet_bushv = "m:inet:bushv", "p.u., Current, Magnitude, referred to network"
      m_inet_buslv = "m:inet:buslv", "p.u., Current, Magnitude, referred to network"
      m_I_bushv = "m:I:bushv", "kA, Current, Magnitude"
      m_I_buslv = "m:I:buslv", "kA, Current, Magnitude"
      m_phii_bushv = "m:phii:bushv", "deg, Current, Angle"
      m_phii_buslv = "m:phii:buslv", "deg, Current, Angle"
      m_i1r_bushv = "m:i1r:bushv", "p.u., Positive-Sequence Current, Real Part"
      m_i1r_buslv = "m:i1r:buslv", "p.u., Positive-Sequence Current, Real Part"
      m_i1i_bushv = "m:i1i:bushv", "p.u., Positive-Sequence Current, Imaginary Part"
      m_i1i_buslv = "m:i1i:buslv", "p.u., Positive-Sequence Current, Imaginary Part"
      m_i1_bushv = "m:i1:bushv", "p.u., Positive-Sequence Current, Magnitude"
      m_i1_buslv = "m:i1:buslv", "p.u., Positive-Sequence Current, Magnitude"
      m_I1_bushv = "m:I1:bushv", "kA, Positive-Sequence Current, Magnitude"
      m_I1_buslv = "m:I1:buslv", "kA, Positive-Sequence Current, Magnitude"
      m_phii1_bushv = "m:phii1:bushv", "deg, Positive-Sequence Current, Angle"
      m_phii1_buslv = "m:phii1:buslv", "deg, Positive-Sequence Current, Angle"
      m_P_bushv = "m:P:bushv", "MW, Active Power"
      m_P_buslv = "m:P:buslv", "MW, Active Power"
      m_Q_bushv = "m:Q:bushv", "Mvar, Reactive Power"
      m_Q_buslv = "m:Q:buslv", "Mvar, Reactive Power"
      m_S_bushv = "m:S:bushv", "MVA, Apparent Power"
      m_S_buslv = "m:S:buslv", "MVA, Apparent Power"
      m_cosphi_bushv = "m:cosphi:bushv", "Power Factor"
      m_cosphi_buslv = "m:cosphi:buslv", "Power Factor"
      m_tanphi_bushv = "m:tanphi:bushv", "tan(phi)"
      m_tanphi_buslv = "m:tanphi:buslv", "tan(phi)"
      m_Psum_bushv = "m:Psum:bushv", "MW, Total Active Power"
      m_Psum_buslv = "m:Psum:buslv", "MW, Total Active Power"
      m_Qsum_bushv = "m:Qsum:bushv", "Mvar, Total Reactive Power"
      m_Qsum_buslv = "m:Qsum:buslv", "Mvar, Total Reactive Power"
      m_Ssum_bushv = "m:Ssum:bushv", "MVA, Total Apparent Power"
      m_Ssum_buslv = "m:Ssum:buslv", "MVA, Total Apparent Power"
      m_cosphisum_bushv = "m:cosphisum:bushv", "Total Power Factor"
      m_cosphisum_buslv = "m:cosphisum:buslv", "Total Power Factor"
      m_tanphisum_bushv = "m:tanphisum:bushv", "Total tan(phi)"
      m_tanphisum_buslv = "m:tanphisum:buslv", "Total tan(phi)"
      m_Spu_bushv = "m:Spu:bushv", "MVA/p.u., Apparent Power per p.u. Voltage"
      m_Spu_buslv = "m:Spu:buslv", "MVA/p.u., Apparent Power per p.u. Voltage"
      m_i1P_bushv = "m:i1P:bushv", "p.u., Positive-Sequence Active Current"
      m_i1P_buslv = "m:i1P:buslv", "p.u., Positive-Sequence Active Current"
      m_i1Q_bushv = "m:i1Q:bushv", "p.u., Positive-Sequence Reactive Current"
      m_i1Q_buslv = "m:i1Q:buslv", "p.u., Positive-Sequence Reactive Current"
      m_I1P_bushv = "m:I1P:bushv", "kA, Positive-Sequence Active Current"
      m_I1P_buslv = "m:I1P:buslv", "kA, Positive-Sequence Active Current"
      m_I1Q_bushv = "m:I1Q:bushv", "kA, Positive-Sequence Reactive Current"
      m_I1Q_buslv = "m:I1Q:buslv", "kA, Positive-Sequence Reactive Current"
      m_ur_bushv = "m:ur:bushv", "p.u., Voltage, Real Part"
      m_ur_buslv = "m:ur:buslv", "p.u., Voltage, Real Part"
      m_ui_bushv = "m:ui:bushv", "p.u., Voltage, Imaginary Part"
      m_ui_buslv = "m:ui:buslv", "p.u., Voltage, Imaginary Part"
      m_u_bushv = "m:u:bushv", "p.u., Voltage, Magnitude"
      m_u_buslv = "m:u:buslv", "p.u., Voltage, Magnitude"
      m_ir_bushv = "m:ir:bushv", "p.u., Current, Real Part"
      m_ir_buslv = "m:ir:buslv", "p.u., Current, Real Part"
      m_ii_bushv = "m:ii:bushv", "p.u., Current, Imaginary Part"
      m_ii_buslv = "m:ii:buslv", "p.u., Current, Imaginary Part"
      m_i_bushv = "m:i:bushv", "p.u., Current, Magnitude"
      m_i_buslv = "m:i:buslv", "p.u., Current, Magnitude"
      m_phiui_bushv = "m:phiui:bushv", "deg, Angle between Voltage and Current"
      m_phiui_buslv = "m:phiui:buslv", "deg, Angle between Voltage and Current"
      m_phiu1i1_bushv = "m:phiu1i1:bushv", "deg, Angle between Voltage and Current in positive sequence system"
      m_phiu1i1_buslv = "m:phiu1i1:buslv", "deg, Angle between Voltage and Current in positive sequence system"
      m_Tfct_bushv = "m:Tfct:bushv", "s, Fault Clearing Time"
      m_Tfct_buslv = "m:Tfct:buslv", "s, Fault Clearing Time"
      m_Brkload_bushv = "m:Brkload:bushv", "%, Breaker Loading"
      m_Brkload_buslv = "m:Brkload:buslv", "%, Breaker Loading"
      n_ur_bushv = "n:ur:bushv", "p.u., Voltage, Real Part"
      n_ur_buslv = "n:ur:buslv", "p.u., Voltage, Real Part"
      n_ui_bushv = "n:ui:bushv", "p.u., Voltage, Imaginary Part"
      n_ui_buslv = "n:ui:buslv", "p.u., Voltage, Imaginary Part"
      n_u_bushv = "n:u:bushv", "p.u., Voltage, Magnitude"
      n_u_buslv = "n:u:buslv", "p.u., Voltage, Magnitude"
      n_upc_bushv = "n:upc:bushv", "%, Voltage, Magnitude"
      n_upc_buslv = "n:upc:buslv", "%, Voltage, Magnitude"
      n_u1_bushv = "n:u1:bushv", "p.u., Voltage, Magnitude"
      n_u1_buslv = "n:u1:buslv", "p.u., Voltage, Magnitude"
      n_u1pc_bushv = "n:u1pc:bushv", "%, Voltage, Magnitude"
      n_u1pc_buslv = "n:u1pc:buslv", "%, Voltage, Magnitude"
      n_u1r_bushv = "n:u1r:bushv", "p.u., Positive-Sequence Voltage, Real Part"
      n_u1r_buslv = "n:u1r:buslv", "p.u., Positive-Sequence Voltage, Real Part"
      n_u1i_bushv = "n:u1i:bushv", "p.u., Positive-Sequence Voltage, Imaginary Part"
      n_u1i_buslv = "n:u1i:buslv", "p.u., Positive-Sequence Voltage, Imaginary Part"
      n_U_bushv = "n:U:bushv", "kV, Line-Ground Voltage, Magnitude"
      n_U_buslv = "n:U:buslv", "kV, Line-Ground Voltage, Magnitude"
      n_Ul_bushv = "n:Ul:bushv", "kV, Line-Line Voltage, Magnitude"
      n_Ul_buslv = "n:Ul:buslv", "kV, Line-Line Voltage, Magnitude"
      n_phiu_bushv = "n:phiu:bushv", "deg, Voltage, Angle"
      n_phiu_buslv = "n:phiu:buslv", "deg, Voltage, Angle"
      n_phiurel_bushv = "n:phiurel:bushv", "deg, Voltage, Relative Angle"
      n_phiurel_buslv = "n:phiurel:buslv", "deg, Voltage, Relative Angle"
      n_du_bushv = "n:du:bushv", "%, Voltage Deviation"
      n_du_buslv = "n:du:buslv", "%, Voltage Deviation"
      n_ishcr_bushv = "n:ishcr:bushv", "p.u., Short-Circuit Current, real"
      n_ishcr_buslv = "n:ishcr:buslv", "p.u., Short-Circuit Current, real"
      n_ishci_bushv = "n:ishci:bushv", "p.u., Short-Circuit Current, imag"
      n_ishci_buslv = "n:ishci:buslv", "p.u., Short-Circuit Current, imag"
      n_ishc_bushv = "n:ishc:bushv", "p.u., Short-Circuit Current, magnitude"
      n_ishc_buslv = "n:ishc:buslv", "p.u., Short-Circuit Current, magnitude"
      n_Ishc_bushv = "n:Ishc:bushv", "kA, Short-Circuit Current, magnitude"
      n_Ishc_buslv = "n:Ishc:buslv", "kA, Short-Circuit Current, magnitude"
      n_fe_bushv = "n:fe:bushv", "p.u., Electrical Frequency"
      n_fe_buslv = "n:fe:buslv", "p.u., Electrical Frequency"
      n_frdev_bushv = "n:frdev:bushv", "Hz, average frequency"
      n_frdev_buslv = "n:frdev:buslv", "Hz, average frequency"
      n_dfehz_bushv = "n:dfehz:bushv", "Hz, Deviation of the El. Frequency"
      n_dfehz_buslv = "n:dfehz:buslv", "Hz, Deviation of the El. Frequency"
      n_dfedt_bushv = "n:dfedt:bushv", "p.u./s, Derivative of El. Frequency"
      n_dfedt_buslv = "n:dfedt:buslv", "p.u./s, Derivative of El. Frequency"
      n_fehz_bushv = "n:fehz:bushv", "Hz, Electrical Frequency"
      n_fehz_buslv = "n:fehz:buslv", "Hz, Electrical Frequency"
      n_frnom_bushv = "n:frnom:bushv", "Hz, Nominal Frequency"
      n_frnom_buslv = "n:frnom:buslv", "Hz, Nominal Frequency"
      s_nntapin = "s:nntapin", "IN, Tap 1, Controller Input continuous, double"
      s_nntapin2 = "s:nntapin2", "IN, Tap 2, Controller Input continuous, double"
      c_nntap = "c:nntap", "Tap 1, Current Position"
      c_nntap_int = "c:nntap_int", "Tap 1, relative Position"
      c_ntpmn = "c:ntpmn", "Tap 1, Min. Position"
      c_ntpmx = "c:ntpmx", "Tap 1, Max. Position"
      c_dut = "c:dut", "Tap 1: Voltage/Tap"
      c_phitrrd = "c:phitrrd", "rad, Tap 1:, Phase/Tap"
      c_nt2agrd = "c:nt2agrd", "rad, Phase Angle"
      c_tap_side = "c:tap_side", "0"
      c_nntap2 = "c:nntap2", "Tap 2, Current Position"
      c_ntpmn2 = "c:ntpmn2", "Tap 2, Min. Position"
      c_ntpmx2 = "c:ntpmx2", "Tap 2, Max. Position"
      c_dut2 = "c:dut2", "Tap 2: Voltage/Tap"
      c_phitrrd2 = "c:phitrrd2", "rad, Tap 2:, Phase/Tap"
      c_tap_side2 = "c:tap_side2", "0"
      c_ratfac = "c:ratfac", "Rating Factor"
      c_snom = "c:snom", "MVA, Rated Power"
      c_zshv_r = "c:zshv:r", "p.u., Leakage Impedance (HV-Side), Real Part"
      c_zshv_i = "c:zshv:i", "p.u., Leakage Impedance (HV-Side), Imaginary Part"
      c_zshv_phi = "c:zshv:phi", "deg, Leakage Impedance (HV-Side), Angle"
      c_zshv_mag = "c:zshv:mag", "p.u., Leakage Impedance (HV-Side), Magnitude"
      c_zshv_phirad = "c:zshv:phirad", "rad, Leakage Impedance (HV-Side), Angle"
      c_zslv_r = "c:zslv:r", "p.u., Leakage Impedance (LV-Side), Real Part"
      c_zslv_i = "c:zslv:i", "p.u., Leakage Impedance (LV-Side), Imaginary Part"
      c_zslv_phi = "c:zslv:phi", "deg, Leakage Impedance (LV-Side), Angle"
      c_zslv_mag = "c:zslv:mag", "p.u., Leakage Impedance (LV-Side), Magnitude"
      c_zslv_phirad = "c:zslv:phirad", "rad, Leakage Impedance (LV-Side), Angle"
      c_ym_r = "c:ym:r", "p.u., Magnetising Admittance, Real Part"
      c_ym_i = "c:ym:i", "p.u., Magnetising Admittance, Imaginary Part"
      c_ym_phi = "c:ym:phi", "deg, Magnetising Admittance, Angle"
      c_ym_mag = "c:ym:mag", "p.u., Magnetising Admittance, Magnitude"
      c_ym_phirad = "c:ym:phirad", "rad, Magnetising Admittance, Angle"
      c_thv_r = "c:thv:r", "Complex Winding Ratio HV, Real Part"
      c_thv_i = "c:thv:i", "Complex Winding Ratio HV, Imaginary Part"
      c_thv_phi = "c:thv:phi", "Complex Winding Ratio HV, Angle"
      c_thv_mag = "c:thv:mag", "Complex Winding Ratio HV, Magnitude"
      c_thv_phirad = "c:thv:phirad", "Complex Winding Ratio HV, Angle"
      c_tlv_r = "c:tlv:r", "Complex Winding Ratio LV, Real Part"
      c_tlv_i = "c:tlv:i", "Complex Winding Ratio LV, Imaginary Part"
      c_tlv_phi = "c:tlv:phi", "Complex Winding Ratio LV, Angle"
      c_tlv_mag = "c:tlv:mag", "Complex Winding Ratio LV, Magnitude"
      c_tlv_phirad = "c:tlv:phirad", "Complex Winding Ratio LV, Angle"
      c_loading_h = "c:loading_h", "%, Loading (HV)"
      c_loading_l = "c:loading_l", "%, Loading (LV)"
      c_loading = "c:loading", "%, Loading"
      c_w_0 = "c:w_0", "Nominal winding ratio"
      c_iAutoTap = "c:iAutoTap", "Controller Mode: 0 = Manual, 1=Auto"
      c_Pmeas = "c:Pmeas", "MW, Measured active power"
      c_du_ldc = "c:du_ldc", "p.u., LDC Input"
      c_ctrl_set = "c:ctrl_set", "Setpoint"
      c_Inom_HV = "c:Inom_HV", "kA, Nominal Current (HV-Side)"
      c_Inom_LV = "c:Inom_LV", "kA, Nominal Current (LV-Side)"
      c_isat = "c:isat", "Saturated Model"
      e_dutap = "e:dutap", "%, Additional Voltage per Tap"
      e_dphitap = "e:dphitap", "deg, Additional Angle per Tap"
      e_phitr = "e:phitr", "deg, Phase of du"
      e_nntap = "e:nntap", "Tap Position"
      e_iTaps = "e:iTaps", "According to Measurement Report"
      e_dutap2 = "e:dutap2", "%, Additional Voltage per Tap"
      e_dphitap2 = "e:dphitap2", "deg, Additional Angle per Tap"
      e_phitr2 = "e:phitr2", "deg, Phase of du"
      e_nntap2 = "e:nntap2", "Tap Position"
      e_iMeasLoc = "e:iMeasLoc", "Measured at"
      e_mTaps = "e:mTaps", "Measurement Report"
      e_iIntTapCtrl = "e:iIntTapCtrl", "Use Integrated Tap Controller"
      e_iopt_hf = "e:iopt_hf", "Consider Capacitances"
      e_Cg_h = "e:Cg_h", "uF, Capacitance HV-Ground"
      e_Cg_l = "e:Cg_l", "uF, Capacitance LV-Ground"
      e_Cc1_hl = "e:Cc1_hl", "uF, Capacitance HV-LV, 1-Sequence"
      e_Cc0_hl = "e:Cc0_hl", "uF, Capacitance HV-LV, 0-Sequence"
      e_iAstabint = "e:iAstabint", "A-stable integration algorithm"
      t_curmg = "t:curmg", "%, No Load Current"
      t_pfe = "t:pfe", "kW, No Load Losses"
      t_zx0hl_n = "t:zx0hl_n", "Mag. Impedance/uk0"
      t_rtox0_n = "t:rtox0_n", "Mag. R/X"
      t_itrdl = "t:itrdl", "x,Pos.Seq. HV-Side"
      t_itrdl_lv = "t:itrdl_lv", "x,Pos.Seq. LV-Side"
      t_itrdr = "t:itrdr", "r,Pos.Seq. HV-Side"
      t_itrdr_lv = "t:itrdr_lv", "r,Pos.Seq. LV-Side"
      t_zx0hl_h = "t:zx0hl_h", "z, Zero Sequ. HV-Side"
      t_zx0hl_l = "t:zx0hl_l", "z, Zero Sequ. LV-Side"
      t_itapch = "t:itapch", "Tap Changer 1"
      t_tapchtype = "t:tapchtype", "Type"
      t_tap_side = "t:tap_side", "at Side"
      t_dutap = "t:dutap", "%, Additional Voltage per Tap"
      t_dphitap = "t:dphitap", "deg, Additional Angle per Tap"
      t_phitr = "t:phitr", "deg, Phase of du"
      t_nntap0 = "t:nntap0", "Neutral Position"
      t_ntpmn = "t:ntpmn", "Minimum Position"
      t_ntpmx = "t:ntpmx", "Maximum Position"
      t_itapch2 = "t:itapch2", "Tap Changer 2"
      t_tapchtype2 = "t:tapchtype2", "Type"
      t_tap_side2 = "t:tap_side2", "at Side"
      t_dutap2 = "t:dutap2", "%, Additional Voltage per Tap"
      t_dphitap2 = "t:dphitap2", "deg, Additional Angle per Tap"
      t_phitr2 = "t:phitr2", "deg, Phase of du"
      t_nntap02 = "t:nntap02", "Neutral Position"
      t_ntpmn2 = "t:ntpmn2", "Minimum Position"
      t_ntpmx2 = "t:ntpmx2", "Maximum Position"
      t_itapzdep = "t:itapzdep", "Tap dependent impedance"
      t_uktr = "t:uktr", "%, Short-Circuit Voltage uk"
      t_uktmn = "t:uktmn", "%, uk (min. tap)"
      t_uktmx = "t:uktmx", "%, uk (max. tap)"
      t_x1pu = "t:x1pu", "p.u., Reactance x1"
      t_x1putmn = "t:x1putmn", "p.u., x1 (min. tap)"
      t_x1putmx = "t:x1putmx", "p.u., x1 (max. tap)"
      t_pcutr = "t:pcutr", "kW, Copper Losses"
      t_uktrr = "t:uktrr", "%, SHC-Voltage (Re(uk)) ukr"
      t_xtor = "t:xtor", "Ratio X/R"
      t_r1pu = "t:r1pu", "p.u., Resistance r1"
      t_pcutmn = "t:pcutmn", "kW, Pcu (min. tap)"
      t_ukrtmn = "t:ukrtmn", "%, Re(uk) (min. tap)"
      t_xtortmn = "t:xtortmn", "X/R (min. tap)"
      t_r1putmn = "t:r1putmn", "p.u., r1 (min. tap)"
      t_pcutmx = "t:pcutmx", "kW, Pcu (max. tap)"
      t_ukrtmx = "t:ukrtmx", "%, Re(uk) (max. tap)"
      t_xtortmx = "t:xtortmx", "X/R (max. tap)"
      t_r1putmx = "t:r1putmx", "p.u., r1 (max. tap)"
      t_uk0tr = "t:uk0tr", "%, Short-Circuit Voltage uk0"
      t_uk0tmn = "t:uk0tmn", "%, uk0 (min. tap)"
      t_uk0tmx = "t:uk0tmx", "%, uk0 (max. tap)"
      t_x0pu = "t:x0pu", "p.u., Reactance x0"
      t_x0putmn = "t:x0putmn", "p.u., x0 (min. tap)"
      t_x0putmx = "t:x0putmx", "p.u., x0 (max. tap)"
      t_ur0tr = "t:ur0tr", "%, SHC-Voltage (Re(uk0)) uk0r"
      t_x0tor0 = "t:x0tor0", "Ratio X0/R0"
      t_r0pu = "t:r0pu", "p.u., Resistance r0"
      t_uk0rtmn = "t:uk0rtmn", "%, Re(uk0) (min. tap)"
      t_x0tor0tmn = "t:x0tor0tmn", "X0/R0 (min. tap)"
      t_r0putmn = "t:r0putmn", "p.u., r0 (min. tap)"
      t_uk0rtmx = "t:uk0rtmx", "%, Re(uk0) (max. tap)"
      t_x0tor0tmx = "t:x0tor0tmx", "X0/R0 (max. tap)"
      t_r0putmx = "t:r0putmx", "p.u., r0 (max. tap)"
      t_iTapLoading = "t:iTapLoading", "Configuration"
      t_itrldf = "t:itrldf", "Type"
      t_satcue = "t:satcue", "%, Current (RMS)"
      t_satvol = "t:satvol", "p.u., Voltage (RMS)"
      t_iIntPola = "t:iIntPola", "Interpolation"
      t_smoothf = "t:smoothf", "%, Smoothing Factor"

    class ElmTr3(Enum):
      _init_ = 'value __doc__'

      m_u1r_bushv = "m:u1r:bushv", "p.u., Positive-Sequence-Voltage, Real Part"
      m_u1r_busmv = "m:u1r:busmv", "p.u., Positive-Sequence-Voltage, Real Part"
      m_u1r_buslv = "m:u1r:buslv", "p.u., Positive-Sequence-Voltage, Real Part"
      m_u1i_bushv = "m:u1i:bushv", "p.u., Positive-Sequence-Voltage, Imaginary Part"
      m_u1i_busmv = "m:u1i:busmv", "p.u., Positive-Sequence-Voltage, Imaginary Part"
      m_u1i_buslv = "m:u1i:buslv", "p.u., Positive-Sequence-Voltage, Imaginary Part"
      m_u1_bushv = "m:u1:bushv", "p.u., Positive-Sequence-Voltage, Magnitude"
      m_u1_busmv = "m:u1:busmv", "p.u., Positive-Sequence-Voltage, Magnitude"
      m_u1_buslv = "m:u1:buslv", "p.u., Positive-Sequence-Voltage, Magnitude"
      m_U1_bushv = "m:U1:bushv", "kV, Line-Ground Positive-Sequence-Voltage, Magnitude"
      m_U1_busmv = "m:U1:busmv", "kV, Line-Ground Positive-Sequence-Voltage, Magnitude"
      m_U1_buslv = "m:U1:buslv", "kV, Line-Ground Positive-Sequence-Voltage, Magnitude"
      m_U1l_bushv = "m:U1l:bushv", "kV, Line-Line Positive-Sequence-Voltage, Magnitude"
      m_U1l_busmv = "m:U1l:busmv", "kV, Line-Line Positive-Sequence-Voltage, Magnitude"
      m_U1l_buslv = "m:U1l:buslv", "kV, Line-Line Positive-Sequence-Voltage, Magnitude"
      m_phiu1_bushv = "m:phiu1:bushv", "deg, Positive-Sequence-Voltage, Angle"
      m_phiu1_busmv = "m:phiu1:busmv", "deg, Positive-Sequence-Voltage, Angle"
      m_phiu1_buslv = "m:phiu1:buslv", "deg, Positive-Sequence-Voltage, Angle"
      m_inet_bushv = "m:inet:bushv", "p.u., Current, Magnitude, referred to network"
      m_inet_busmv = "m:inet:busmv", "p.u., Current, Magnitude, referred to network"
      m_inet_buslv = "m:inet:buslv", "p.u., Current, Magnitude, referred to network"
      m_I_bushv = "m:I:bushv", "kA, Current, Magnitude"
      m_I_busmv = "m:I:busmv", "kA, Current, Magnitude"
      m_I_buslv = "m:I:buslv", "kA, Current, Magnitude"
      m_phii_bushv = "m:phii:bushv", "deg, Current, Angle"
      m_phii_busmv = "m:phii:busmv", "deg, Current, Angle"
      m_phii_buslv = "m:phii:buslv", "deg, Current, Angle"
      m_i1r_bushv = "m:i1r:bushv", "p.u., Positive-Sequence Current, Real Part"
      m_i1r_busmv = "m:i1r:busmv", "p.u., Positive-Sequence Current, Real Part"
      m_i1r_buslv = "m:i1r:buslv", "p.u., Positive-Sequence Current, Real Part"
      m_i1i_bushv = "m:i1i:bushv", "p.u., Positive-Sequence Current, Imaginary Part"
      m_i1i_busmv = "m:i1i:busmv", "p.u., Positive-Sequence Current, Imaginary Part"
      m_i1i_buslv = "m:i1i:buslv", "p.u., Positive-Sequence Current, Imaginary Part"
      m_i1_bushv = "m:i1:bushv", "p.u., Positive-Sequence Current, Magnitude"
      m_i1_busmv = "m:i1:busmv", "p.u., Positive-Sequence Current, Magnitude"
      m_i1_buslv = "m:i1:buslv", "p.u., Positive-Sequence Current, Magnitude"
      m_I1_bushv = "m:I1:bushv", "kA, Positive-Sequence Current, Magnitude"
      m_I1_busmv = "m:I1:busmv", "kA, Positive-Sequence Current, Magnitude"
      m_I1_buslv = "m:I1:buslv", "kA, Positive-Sequence Current, Magnitude"
      m_phii1_bushv = "m:phii1:bushv", "deg, Positive-Sequence Current, Angle"
      m_phii1_busmv = "m:phii1:busmv", "deg, Positive-Sequence Current, Angle"
      m_phii1_buslv = "m:phii1:buslv", "deg, Positive-Sequence Current, Angle"
      m_P_bushv = "m:P:bushv", "MW, Active Power"
      m_P_busmv = "m:P:busmv", "MW, Active Power"
      m_P_buslv = "m:P:buslv", "MW, Active Power"
      m_Q_bushv = "m:Q:bushv", "Mvar, Reactive Power"
      m_Q_busmv = "m:Q:busmv", "Mvar, Reactive Power"
      m_Q_buslv = "m:Q:buslv", "Mvar, Reactive Power"
      m_S_bushv = "m:S:bushv", "MVA, Apparent Power"
      m_S_busmv = "m:S:busmv", "MVA, Apparent Power"
      m_S_buslv = "m:S:buslv", "MVA, Apparent Power"
      m_cosphi_bushv = "m:cosphi:bushv", "Power Factor"
      m_cosphi_busmv = "m:cosphi:busmv", "Power Factor"
      m_cosphi_buslv = "m:cosphi:buslv", "Power Factor"
      m_tanphi_bushv = "m:tanphi:bushv", "tan(phi)"
      m_tanphi_busmv = "m:tanphi:busmv", "tan(phi)"
      m_tanphi_buslv = "m:tanphi:buslv", "tan(phi)"
      m_Psum_bushv = "m:Psum:bushv", "MW, Total Active Power"
      m_Psum_busmv = "m:Psum:busmv", "MW, Total Active Power"
      m_Psum_buslv = "m:Psum:buslv", "MW, Total Active Power"
      m_Qsum_bushv = "m:Qsum:bushv", "Mvar, Total Reactive Power"
      m_Qsum_busmv = "m:Qsum:busmv", "Mvar, Total Reactive Power"
      m_Qsum_buslv = "m:Qsum:buslv", "Mvar, Total Reactive Power"
      m_Ssum_bushv = "m:Ssum:bushv", "MVA, Total Apparent Power"
      m_Ssum_busmv = "m:Ssum:busmv", "MVA, Total Apparent Power"
      m_Ssum_buslv = "m:Ssum:buslv", "MVA, Total Apparent Power"
      m_cosphisum_bushv = "m:cosphisum:bushv", "Total Power Factor"
      m_cosphisum_busmv = "m:cosphisum:busmv", "Total Power Factor"
      m_cosphisum_buslv = "m:cosphisum:buslv", "Total Power Factor"
      m_tanphisum_bushv = "m:tanphisum:bushv", "Total tan(phi)"
      m_tanphisum_busmv = "m:tanphisum:busmv", "Total tan(phi)"
      m_tanphisum_buslv = "m:tanphisum:buslv", "Total tan(phi)"
      m_Spu_bushv = "m:Spu:bushv", "MVA/p.u., Apparent Power per p.u. Voltage"
      m_Spu_busmv = "m:Spu:busmv", "MVA/p.u., Apparent Power per p.u. Voltage"
      m_Spu_buslv = "m:Spu:buslv", "MVA/p.u., Apparent Power per p.u. Voltage"
      m_i1P_bushv = "m:i1P:bushv", "p.u., Positive-Sequence Active Current"
      m_i1P_busmv = "m:i1P:busmv", "p.u., Positive-Sequence Active Current"
      m_i1P_buslv = "m:i1P:buslv", "p.u., Positive-Sequence Active Current"
      m_i1Q_bushv = "m:i1Q:bushv", "p.u., Positive-Sequence Reactive Current"
      m_i1Q_busmv = "m:i1Q:busmv", "p.u., Positive-Sequence Reactive Current"
      m_i1Q_buslv = "m:i1Q:buslv", "p.u., Positive-Sequence Reactive Current"
      m_I1P_bushv = "m:I1P:bushv", "kA, Positive-Sequence Active Current"
      m_I1P_busmv = "m:I1P:busmv", "kA, Positive-Sequence Active Current"
      m_I1P_buslv = "m:I1P:buslv", "kA, Positive-Sequence Active Current"
      m_I1Q_bushv = "m:I1Q:bushv", "kA, Positive-Sequence Reactive Current"
      m_I1Q_busmv = "m:I1Q:busmv", "kA, Positive-Sequence Reactive Current"
      m_I1Q_buslv = "m:I1Q:buslv", "kA, Positive-Sequence Reactive Current"
      m_ur_bushv = "m:ur:bushv", "p.u., Voltage, Real Part"
      m_ur_busmv = "m:ur:busmv", "p.u., Voltage, Real Part"
      m_ur_buslv = "m:ur:buslv", "p.u., Voltage, Real Part"
      m_ui_bushv = "m:ui:bushv", "p.u., Voltage, Imaginary Part"
      m_ui_busmv = "m:ui:busmv", "p.u., Voltage, Imaginary Part"
      m_ui_buslv = "m:ui:buslv", "p.u., Voltage, Imaginary Part"
      m_u_bushv = "m:u:bushv", "p.u., Voltage, Magnitude"
      m_u_busmv = "m:u:busmv", "p.u., Voltage, Magnitude"
      m_u_buslv = "m:u:buslv", "p.u., Voltage, Magnitude"
      m_ir_bushv = "m:ir:bushv", "p.u., Current, Real Part"
      m_ir_busmv = "m:ir:busmv", "p.u., Current, Real Part"
      m_ir_buslv = "m:ir:buslv", "p.u., Current, Real Part"
      m_ii_bushv = "m:ii:bushv", "p.u., Current, Imaginary Part"
      m_ii_busmv = "m:ii:busmv", "p.u., Current, Imaginary Part"
      m_ii_buslv = "m:ii:buslv", "p.u., Current, Imaginary Part"
      m_i_bushv = "m:i:bushv", "p.u., Current, Magnitude"
      m_i_busmv = "m:i:busmv", "p.u., Current, Magnitude"
      m_i_buslv = "m:i:buslv", "p.u., Current, Magnitude"
      m_phiui_bushv = "m:phiui:bushv", "deg, Angle between Voltage and Current"
      m_phiui_busmv = "m:phiui:busmv", "deg, Angle between Voltage and Current"
      m_phiui_buslv = "m:phiui:buslv", "deg, Angle between Voltage and Current"
      m_phiu1i1_bushv = "m:phiu1i1:bushv", "deg, Angle between Voltage and Current in positive sequence system"
      m_phiu1i1_busmv = "m:phiu1i1:busmv", "deg, Angle between Voltage and Current in positive sequence system"
      m_phiu1i1_buslv = "m:phiu1i1:buslv", "deg, Angle between Voltage and Current in positive sequence system"
      m_Tfct_bushv = "m:Tfct:bushv", "s, Fault Clearing Time"
      m_Tfct_busmv = "m:Tfct:busmv", "s, Fault Clearing Time"
      m_Tfct_buslv = "m:Tfct:buslv", "s, Fault Clearing Time"
      m_Brkload_bushv = "m:Brkload:bushv", "%, Breaker Loading"
      m_Brkload_busmv = "m:Brkload:busmv", "%, Breaker Loading"
      m_Brkload_buslv = "m:Brkload:buslv", "%, Breaker Loading"
      n_ur_bushv = "n:ur:bushv", "p.u., Voltage, Real Part"
      n_ur_busmv = "n:ur:busmv", "p.u., Voltage, Real Part"
      n_ur_buslv = "n:ur:buslv", "p.u., Voltage, Real Part"
      n_ui_bushv = "n:ui:bushv", "p.u., Voltage, Imaginary Part"
      n_ui_busmv = "n:ui:busmv", "p.u., Voltage, Imaginary Part"
      n_ui_buslv = "n:ui:buslv", "p.u., Voltage, Imaginary Part"
      n_u_bushv = "n:u:bushv", "p.u., Voltage, Magnitude"
      n_u_busmv = "n:u:busmv", "p.u., Voltage, Magnitude"
      n_u_buslv = "n:u:buslv", "p.u., Voltage, Magnitude"
      n_upc_bushv = "n:upc:bushv", "%, Voltage, Magnitude"
      n_upc_busmv = "n:upc:busmv", "%, Voltage, Magnitude"
      n_upc_buslv = "n:upc:buslv", "%, Voltage, Magnitude"
      n_u1_bushv = "n:u1:bushv", "p.u., Voltage, Magnitude"
      n_u1_busmv = "n:u1:busmv", "p.u., Voltage, Magnitude"
      n_u1_buslv = "n:u1:buslv", "p.u., Voltage, Magnitude"
      n_u1pc_bushv = "n:u1pc:bushv", "%, Voltage, Magnitude"
      n_u1pc_busmv = "n:u1pc:busmv", "%, Voltage, Magnitude"
      n_u1pc_buslv = "n:u1pc:buslv", "%, Voltage, Magnitude"
      n_u1r_bushv = "n:u1r:bushv", "p.u., Positive-Sequence Voltage, Real Part"
      n_u1r_busmv = "n:u1r:busmv", "p.u., Positive-Sequence Voltage, Real Part"
      n_u1r_buslv = "n:u1r:buslv", "p.u., Positive-Sequence Voltage, Real Part"
      n_u1i_bushv = "n:u1i:bushv", "p.u., Positive-Sequence Voltage, Imaginary Part"
      n_u1i_busmv = "n:u1i:busmv", "p.u., Positive-Sequence Voltage, Imaginary Part"
      n_u1i_buslv = "n:u1i:buslv", "p.u., Positive-Sequence Voltage, Imaginary Part"
      n_U_bushv = "n:U:bushv", "kV, Line-Ground Voltage, Magnitude"
      n_U_busmv = "n:U:busmv", "kV, Line-Ground Voltage, Magnitude"
      n_U_buslv = "n:U:buslv", "kV, Line-Ground Voltage, Magnitude"
      n_Ul_bushv = "n:Ul:bushv", "kV, Line-Line Voltage, Magnitude"
      n_Ul_busmv = "n:Ul:busmv", "kV, Line-Line Voltage, Magnitude"
      n_Ul_buslv = "n:Ul:buslv", "kV, Line-Line Voltage, Magnitude"
      n_phiu_bushv = "n:phiu:bushv", "deg, Voltage, Angle"
      n_phiu_busmv = "n:phiu:busmv", "deg, Voltage, Angle"
      n_phiu_buslv = "n:phiu:buslv", "deg, Voltage, Angle"
      n_phiurel_bushv = "n:phiurel:bushv", "deg, Voltage, Relative Angle"
      n_phiurel_busmv = "n:phiurel:busmv", "deg, Voltage, Relative Angle"
      n_phiurel_buslv = "n:phiurel:buslv", "deg, Voltage, Relative Angle"
      n_du_bushv = "n:du:bushv", "%, Voltage Deviation"
      n_du_busmv = "n:du:busmv", "%, Voltage Deviation"
      n_du_buslv = "n:du:buslv", "%, Voltage Deviation"
      n_ishcr_bushv = "n:ishcr:bushv", "p.u., Short-Circuit Current, real"
      n_ishcr_busmv = "n:ishcr:busmv", "p.u., Short-Circuit Current, real"
      n_ishcr_buslv = "n:ishcr:buslv", "p.u., Short-Circuit Current, real"
      n_ishci_bushv = "n:ishci:bushv", "p.u., Short-Circuit Current, imag"
      n_ishci_busmv = "n:ishci:busmv", "p.u., Short-Circuit Current, imag"
      n_ishci_buslv = "n:ishci:buslv", "p.u., Short-Circuit Current, imag"
      n_ishc_bushv = "n:ishc:bushv", "p.u., Short-Circuit Current, magnitude"
      n_ishc_busmv = "n:ishc:busmv", "p.u., Short-Circuit Current, magnitude"
      n_ishc_buslv = "n:ishc:buslv", "p.u., Short-Circuit Current, magnitude"
      n_Ishc_bushv = "n:Ishc:bushv", "kA, Short-Circuit Current, magnitude"
      n_Ishc_busmv = "n:Ishc:busmv", "kA, Short-Circuit Current, magnitude"
      n_Ishc_buslv = "n:Ishc:buslv", "kA, Short-Circuit Current, magnitude"
      n_fe_bushv = "n:fe:bushv", "p.u., Electrical Frequency"
      n_fe_busmv = "n:fe:busmv", "p.u., Electrical Frequency"
      n_fe_buslv = "n:fe:buslv", "p.u., Electrical Frequency"
      n_frdev_bushv = "n:frdev:bushv", "Hz, average frequency"
      n_frdev_busmv = "n:frdev:busmv", "Hz, average frequency"
      n_frdev_buslv = "n:frdev:buslv", "Hz, average frequency"
      n_dfehz_bushv = "n:dfehz:bushv", "Hz, Deviation of the El. Frequency"
      n_dfehz_busmv = "n:dfehz:busmv", "Hz, Deviation of the El. Frequency"
      n_dfehz_buslv = "n:dfehz:buslv", "Hz, Deviation of the El. Frequency"
      n_dfedt_bushv = "n:dfedt:bushv", "p.u./s, Derivative of El. Frequency"
      n_dfedt_busmv = "n:dfedt:busmv", "p.u./s, Derivative of El. Frequency"
      n_dfedt_buslv = "n:dfedt:buslv", "p.u./s, Derivative of El. Frequency"
      n_fehz_bushv = "n:fehz:bushv", "Hz, Electrical Frequency"
      n_fehz_busmv = "n:fehz:busmv", "Hz, Electrical Frequency"
      n_fehz_buslv = "n:fehz:buslv", "Hz, Electrical Frequency"
      n_frnom_bushv = "n:frnom:bushv", "Hz, Nominal Frequency"
      n_frnom_busmv = "n:frnom:busmv", "Hz, Nominal Frequency"
      n_frnom_buslv = "n:frnom:buslv", "Hz, Nominal Frequency"
      s_nntapin_h = "s:nntapin_h", "IN, Tap (HV), Controller Input, continuous, double"
      s_nntapin_m = "s:nntapin_m", "IN, Tap (MV), Controller Input, continuous, double"
      s_nntapin_l = "s:nntapin_l", "IN, Tap (LV), Controller Input, continuous, double"
      c_n3tap_h = "c:n3tap_h", "Current Tap Position (HV)"
      c_n3tap_m = "c:n3tap_m", "Current Tap Position (MV)"
      c_n3tap_l = "c:n3tap_l", "Current Tap Position (LV)"
      c_nntap_int_h = "c:nntap_int_h", "Tap (HV), relative Position"
      c_nntap_int_m = "c:nntap_int_m", "Tap (MV), relative Position"
      c_nntap_int_l = "c:nntap_int_l", "Tap (LV), relative Position"
      c_zs_h_r = "c:zs_h:r", "p.u., Short Circuit Impedance (HV), Real Part"
      c_zs_h_i = "c:zs_h:i", "p.u., Short Circuit Impedance (HV), Imaginary Part"
      c_zs_h_phi = "c:zs_h:phi", "deg, Short Circuit Impedance (HV), Angle"
      c_zs_h_mag = "c:zs_h:mag", "p.u., Short Circuit Impedance (HV), Magnitude"
      c_zs_h_phirad = "c:zs_h:phirad", "rad, Short Circuit Impedance (HV), Angle"
      c_zs_m_r = "c:zs_m:r", "p.u., Short Circuit Impedance (MV), Real Part"
      c_zs_m_i = "c:zs_m:i", "p.u., Short Circuit Impedance (MV), Imaginary Part"
      c_zs_m_phi = "c:zs_m:phi", "deg, Short Circuit Impedance (MV), Angle"
      c_zs_m_mag = "c:zs_m:mag", "p.u., Short Circuit Impedance (MV), Magnitude"
      c_zs_m_phirad = "c:zs_m:phirad", "rad, Short Circuit Impedance (MV), Angle"
      c_zs_l_r = "c:zs_l:r", "p.u., Short Circuit Impedance (LV), Real Part"
      c_zs_l_i = "c:zs_l:i", "p.u., Short Circuit Impedance (LV), Imaginary Part"
      c_zs_l_phi = "c:zs_l:phi", "deg, Short Circuit Impedance (LV), Angle"
      c_zs_l_mag = "c:zs_l:mag", "p.u., Short Circuit Impedance (LV), Magnitude"
      c_zs_l_phirad = "c:zs_l:phirad", "rad, Short Circuit Impedance (LV), Angle"
      c_ym_r = "c:ym:r", "p.u., Magnetising Admittance, Real Part"
      c_ym_i = "c:ym:i", "p.u., Magnetising Admittance, Imaginary Part"
      c_ym_phi = "c:ym:phi", "deg, Magnetising Admittance, Angle"
      c_ym_mag = "c:ym:mag", "p.u., Magnetising Admittance, Magnitude"
      c_ym_phirad = "c:ym:phirad", "rad, Magnetising Admittance, Angle"
      c_t_h_r = "c:t_h:r", "Complex Winding Ratio (HV/S), Real Part"
      c_t_h_i = "c:t_h:i", "Complex Winding Ratio (HV/S), Imaginary Part"
      c_t_h_phi = "c:t_h:phi", "Complex Winding Ratio (HV/S), Angle"
      c_t_h_mag = "c:t_h:mag", "Complex Winding Ratio (HV/S), Magnitude"
      c_t_h_phirad = "c:t_h:phirad", "Complex Winding Ratio (HV/S), Angle"
      c_t_m_r = "c:t_m:r", "Complex Winding Ratio (MV/S), Real Part"
      c_t_m_i = "c:t_m:i", "Complex Winding Ratio (MV/S), Imaginary Part"
      c_t_m_phi = "c:t_m:phi", "Complex Winding Ratio (MV/S), Angle"
      c_t_m_mag = "c:t_m:mag", "Complex Winding Ratio (MV/S), Magnitude"
      c_t_m_phirad = "c:t_m:phirad", "Complex Winding Ratio (MV/S), Angle"
      c_t_l_r = "c:t_l:r", "Complex Winding Ratio (LV/S), Real Part"
      c_t_l_i = "c:t_l:i", "Complex Winding Ratio (LV/S), Imaginary Part"
      c_t_l_phi = "c:t_l:phi", "Complex Winding Ratio (LV/S), Angle"
      c_t_l_mag = "c:t_l:mag", "Complex Winding Ratio (LV/S), Magnitude"
      c_t_l_phirad = "c:t_l:phirad", "Complex Winding Ratio (LV/S), Angle"
      c_loading_h = "c:loading_h", "%, Loading (HV)"
      c_loading_m = "c:loading_m", "%, Loading (MV)"
      c_loading_l = "c:loading_l", "%, Loading (LV)"
      c_loading = "c:loading", "%, Maximum Loading"
      c_i3loc = "c:i3loc", "Position of Magn.ind."
      c_w_0 = "c:w_0", "Nominal winding ratio"
      c_iAutoTap = "c:iAutoTap", "Controller Mode: 0 = Manual, 1=Auto"
      c_Inom_HV = "c:Inom_HV", "kA, Nominal Current (HV-Side)"
      c_Inom_MV = "c:Inom_MV", "kA, Nominal Current (MV-Side)"
      c_Inom_LV = "c:Inom_LV", "kA, Nominal Current (LV-Side)"
      c_isat = "c:isat", "Saturated Model"
      e_du3tp_h = "e:du3tp_h", "%, Add. Voltage per Tap"
      e_ph3tr_h = "e:ph3tr_h", "deg, Phase of du"
      e_n3tap_h = "e:n3tap_h", "Act. Position"
      e_du3tp_m = "e:du3tp_m", "%, Add. Voltage per Tap"
      e_ph3tr_m = "e:ph3tr_m", "deg, Phase of du"
      e_n3tap_m = "e:n3tap_m", "Act. Position"
      e_du3tp_l = "e:du3tp_l", "%, Add. Voltage per Tap"
      e_ph3tr_l = "e:ph3tr_l", "deg, Phase of du"
      e_n3tap_l = "e:n3tap_l", "Act. Position"
      e_iTaps = "e:iTaps", "According to Measurement Report"
      e_iMeasTap = "e:iMeasTap", "for Tap at"
      e_iMeasLoc = "e:iMeasLoc", "Measured at"
      e_mTaps = "e:mTaps", "Measurement Report"
      e_iIntTapCtrl = "e:iIntTapCtrl", "Use Integrated Tap Controller"
      e_iopt_hf = "e:iopt_hf", "Consider Capacitances"
      e_Cg_h = "e:Cg_h", "uF, HV-Ground"
      e_Cg_m = "e:Cg_m", "uF, MV-Ground"
      e_Cg_l = "e:Cg_l", "uF, LV-Ground"
      e_Cc1_hm = "e:Cc1_hm", "uF, HV-MV"
      e_Cc1_ml = "e:Cc1_ml", "uF, MV-LV"
      e_Cc1_lh = "e:Cc1_lh", "uF, LV-HV"
      e_Cc0_hm = "e:Cc0_hm", "uF, HV-MV"
      e_Cc0_ml = "e:Cc0_ml", "uF, MV-LV"
      e_Cc0_lh = "e:Cc0_lh", "uF, LV-HV"
      e_iAstabint = "e:iAstabint", "A-stable integration algorithm"
      t_du3tp_h = "t:du3tp_h", "%, Add. Voltage per Tap"
      t_ph3tr_h = "t:ph3tr_h", "deg, Phase of du"
      t_n3tp0_h = "t:n3tp0_h", "Neutral Position"
      t_n3tmn_h = "t:n3tmn_h", "Min. Position"
      t_n3tmx_h = "t:n3tmx_h", "Max. Position"
      t_du3tp_m = "t:du3tp_m", "%, Add. Voltage per Tap"
      t_ph3tr_m = "t:ph3tr_m", "deg, Phase of du"
      t_n3tp0_m = "t:n3tp0_m", "Neutral Position"
      t_n3tmn_m = "t:n3tmn_m", "Min. Position"
      t_n3tmx_m = "t:n3tmx_m", "Max. Position"
      t_du3tp_l = "t:du3tp_l", "%, Add. Voltage per Tap"
      t_ph3tr_l = "t:ph3tr_l", "deg, Phase of du"
      t_n3tp0_l = "t:n3tp0_l", "Neutral Position"
      t_n3tmn_l = "t:n3tmn_l", "Min. Position"
      t_n3tmx_l = "t:n3tmx_l", "Max. Position"
      t_itapos = "t:itapos", "Tap Modelled at"
      t_i3loc = "t:i3loc", "Position"
      t_curm3 = "t:curm3", "%, No Load Current"
      t_pfe = "t:pfe", "kW, No Load Losses"
      t_i30lc = "t:i30lc", "Position"
      t_cr0m3 = "t:cr0m3", "%, No Load Current"
      t_rtox0_n = "t:rtox0_n", "Mag. R/X"
      t_iTapLoading = "t:iTapLoading", "Configuration"
      t_itrldf = "t:itrldf", "Type"
      t_satcue = "t:satcue", "%, Current (RMS)"
      t_satvol = "t:satvol", "p.u., Voltage (RMS)"
      t_iIntPola = "t:iIntPola", "Interpolation"
      t_smoothf = "t:smoothf", "%, Smoothing Factor"

    class ElmVac(Enum):
      _init_ = 'value __doc__'

      m_u1r_bus1 = "m:u1r:bus1", "p.u., Positive-Sequence-Voltage, Real Part"
      m_u1i_bus1 = "m:u1i:bus1", "p.u., Positive-Sequence-Voltage, Imaginary Part"
      m_u1_bus1 = "m:u1:bus1", "p.u., Positive-Sequence-Voltage, Magnitude"
      m_U1_bus1 = "m:U1:bus1", "kV, Line-Ground Positive-Sequence-Voltage, Magnitude"
      m_U1l_bus1 = "m:U1l:bus1", "kV, Line-Line Positive-Sequence-Voltage, Magnitude"
      m_phiu1_bus1 = "m:phiu1:bus1", "deg, Positive-Sequence-Voltage, Angle"
      m_inet_bus1 = "m:inet:bus1", "p.u., Current, Magnitude, referred to network"
      m_I_bus1 = "m:I:bus1", "kA, Current, Magnitude"
      m_phii_bus1 = "m:phii:bus1", "deg, Current, Angle"
      m_i1r_bus1 = "m:i1r:bus1", "p.u., Positive-Sequence Current, Real Part"
      m_i1i_bus1 = "m:i1i:bus1", "p.u., Positive-Sequence Current, Imaginary Part"
      m_i1_bus1 = "m:i1:bus1", "p.u., Positive-Sequence Current, Magnitude"
      m_I1_bus1 = "m:I1:bus1", "kA, Positive-Sequence Current, Magnitude"
      m_phii1_bus1 = "m:phii1:bus1", "deg, Positive-Sequence Current, Angle"
      m_P_bus1 = "m:P:bus1", "MW, Active Power"
      m_Q_bus1 = "m:Q:bus1", "Mvar, Reactive Power"
      m_S_bus1 = "m:S:bus1", "MVA, Apparent Power"
      m_cosphi_bus1 = "m:cosphi:bus1", "Power Factor"
      m_tanphi_bus1 = "m:tanphi:bus1", "tan(phi)"
      m_Psum_bus1 = "m:Psum:bus1", "MW, Total Active Power"
      m_Qsum_bus1 = "m:Qsum:bus1", "Mvar, Total Reactive Power"
      m_Ssum_bus1 = "m:Ssum:bus1", "MVA, Total Apparent Power"
      m_cosphisum_bus1 = "m:cosphisum:bus1", "Total Power Factor"
      m_tanphisum_bus1 = "m:tanphisum:bus1", "Total tan(phi)"
      m_Spu_bus1 = "m:Spu:bus1", "MVA/p.u., Apparent Power per p.u. Voltage"
      m_i1P_bus1 = "m:i1P:bus1", "p.u., Positive-Sequence Active Current"
      m_i1Q_bus1 = "m:i1Q:bus1", "p.u., Positive-Sequence Reactive Current"
      m_I1P_bus1 = "m:I1P:bus1", "kA, Positive-Sequence Active Current"
      m_I1Q_bus1 = "m:I1Q:bus1", "kA, Positive-Sequence Reactive Current"
      m_ur_bus1 = "m:ur:bus1", "p.u., Voltage, Real Part"
      m_ui_bus1 = "m:ui:bus1", "p.u., Voltage, Imaginary Part"
      m_u_bus1 = "m:u:bus1", "p.u., Voltage, Magnitude"
      m_ir_bus1 = "m:ir:bus1", "p.u., Current, Real Part"
      m_ii_bus1 = "m:ii:bus1", "p.u., Current, Imaginary Part"
      m_i_bus1 = "m:i:bus1", "p.u., Current, Magnitude"
      m_phiui_bus1 = "m:phiui:bus1", "deg, Angle between Voltage and Current"
      m_phiu1i1_bus1 = "m:phiu1i1:bus1", "deg, Angle between Voltage and Current in positive sequence system"
      m_Tfct_bus1 = "m:Tfct:bus1", "s, Fault Clearing Time"
      m_Brkload_bus1 = "m:Brkload:bus1", "%, Breaker Loading"
      n_ur_bus1 = "n:ur:bus1", "p.u., Voltage, Real Part"
      n_ui_bus1 = "n:ui:bus1", "p.u., Voltage, Imaginary Part"
      n_u_bus1 = "n:u:bus1", "p.u., Voltage, Magnitude"
      n_upc_bus1 = "n:upc:bus1", "%, Voltage, Magnitude"
      n_u1_bus1 = "n:u1:bus1", "p.u., Voltage, Magnitude"
      n_u1pc_bus1 = "n:u1pc:bus1", "%, Voltage, Magnitude"
      n_u1r_bus1 = "n:u1r:bus1", "p.u., Positive-Sequence Voltage, Real Part"
      n_u1i_bus1 = "n:u1i:bus1", "p.u., Positive-Sequence Voltage, Imaginary Part"
      n_U_bus1 = "n:U:bus1", "kV, Line-Ground Voltage, Magnitude"
      n_Ul_bus1 = "n:Ul:bus1", "kV, Line-Line Voltage, Magnitude"
      n_phiu_bus1 = "n:phiu:bus1", "deg, Voltage, Angle"
      n_phiurel_bus1 = "n:phiurel:bus1", "deg, Voltage, Relative Angle"
      n_du_bus1 = "n:du:bus1", "%, Voltage Deviation"
      n_ishcr_bus1 = "n:ishcr:bus1", "p.u., Short-Circuit Current, real"
      n_ishci_bus1 = "n:ishci:bus1", "p.u., Short-Circuit Current, imag"
      n_ishc_bus1 = "n:ishc:bus1", "p.u., Short-Circuit Current, magnitude"
      n_Ishc_bus1 = "n:Ishc:bus1", "kA, Short-Circuit Current, magnitude"
      n_fe_bus1 = "n:fe:bus1", "p.u., Electrical Frequency"
      n_frdev_bus1 = "n:frdev:bus1", "Hz, average frequency"
      n_dfehz_bus1 = "n:dfehz:bus1", "Hz, Deviation of the El. Frequency"
      n_dfedt_bus1 = "n:dfedt:bus1", "p.u./s, Derivative of El. Frequency"
      n_fehz_bus1 = "n:fehz:bus1", "Hz, Electrical Frequency"
      n_frnom_bus1 = "n:frnom:bus1", "Hz, Nominal Frequency"
      s_u0 = "s:u0", "p.u., IN, Voltage-Input, continuous, double"
      s_Ul0 = "s:Ul0", "kV, IN, Voltage-Input (Line-Line), continuous, double"
      s_f0 = "s:f0", "p.u., IN, Frequency-Input continuous, double"
      s_F0Hz = "s:F0Hz", "Hz, IN, Frequency-Input continuous, double"
      s_U_A = "s:U_A", "kV, IN, Voltage, Magnitude, Phase a, continuous, double"
      s_U_B = "s:U_B", "kV, IN, Voltage, Magnitude, Phase b, continuous, double"
      s_U_C = "s:U_C", "kV, IN, Voltage, Magnitude, Phase c, continuous, double"
      s_phiu_B = "s:phiu_B", "deg, IN, Voltage, Angle, Phase b continuous, double"
      s_phiu_C = "s:phiu_C", "deg, IN, Voltage, Angle, Phase c continuous, double"
      s_dphiu = "s:dphiu", "rad, IN, Additional Angle, continuous, double"
      s_fref = "s:fref", "p.u., IN, Reference Frequency, continuous, double"
      s_xspeed = "s:xspeed", "p.u., OUT, Frequency, continuous, double"
      s_phiu = "s:phiu", "rad, STATE, Voltage-Angle, continuous, double"
      s_phiu_dt = "s:phiu:dt", "rad/s, d/dt, Voltage-Angle, derivative, continuous, double"
      c_hpi = "c:hpi", "rad/s, Nominal-Angular-Frequency"
      e_iAstabint = "e:iAstabint", "A-stable integration algorithm"
      e_R1 = "e:R1", "Ohm, Resistance, R1"
      e_X1 = "e:X1", "Ohm, Reactance, X1"
      e_R0 = "e:R0", "Ohm, Resistance, R0"
      e_X0 = "e:X0", "Ohm, Reactance, X0"
      e_R2 = "e:R2", "Ohm, Resistance, R2"
      e_X2 = "e:X2", "Ohm, Reactance, X2"
      e_leadUinput = "e:leadUinput", "Voltage input"
      e_leadFinput = "e:leadFinput", "Frequency input"
      e_cosimModel = "e:cosimModel", "Used for Co-simulation"
      e_phiSlackRef = "e:phiSlackRef", "rad, Initial rotor angle"
      e_phiiniRef = "e:phiiniRef", "rad, Initial angle of bus voltage"

    class ElmVsc(Enum):
      _init_ = 'value __doc__'

      m_u1r_busac = "m:u1r:busac", "p.u., Positive-Sequence-Voltage, Real Part"
      m_u1r_busdp = "m:u1r:busdp", "p.u., Positive-Sequence-Voltage, Real Part"
      m_u1r_busdm = "m:u1r:busdm", "p.u., Positive-Sequence-Voltage, Real Part"
      m_u1i_busac = "m:u1i:busac", "p.u., Positive-Sequence-Voltage, Imaginary Part"
      m_u1i_busdp = "m:u1i:busdp", "p.u., Positive-Sequence-Voltage, Imaginary Part"
      m_u1i_busdm = "m:u1i:busdm", "p.u., Positive-Sequence-Voltage, Imaginary Part"
      m_u1_busac = "m:u1:busac", "p.u., Positive-Sequence-Voltage, Magnitude"
      m_u1_busdp = "m:u1:busdp", "p.u., Positive-Sequence-Voltage, Magnitude"
      m_u1_busdm = "m:u1:busdm", "p.u., Positive-Sequence-Voltage, Magnitude"
      m_U1_busac = "m:U1:busac", "kV, Line-Ground Positive-Sequence-Voltage, Magnitude"
      m_U1_busdp = "m:U1:busdp", "kV, Line-Ground Positive-Sequence-Voltage, Magnitude"
      m_U1_busdm = "m:U1:busdm", "kV, Line-Ground Positive-Sequence-Voltage, Magnitude"
      m_U1l_busac = "m:U1l:busac", "kV, Line-Line Positive-Sequence-Voltage, Magnitude"
      m_U1l_busdp = "m:U1l:busdp", "kV, Line-Line Positive-Sequence-Voltage, Magnitude"
      m_U1l_busdm = "m:U1l:busdm", "kV, Line-Line Positive-Sequence-Voltage, Magnitude"
      m_phiu1_busac = "m:phiu1:busac", "deg, Positive-Sequence-Voltage, Angle"
      m_phiu1_busdp = "m:phiu1:busdp", "deg, Positive-Sequence-Voltage, Angle"
      m_phiu1_busdm = "m:phiu1:busdm", "deg, Positive-Sequence-Voltage, Angle"
      m_inet_busac = "m:inet:busac", "p.u., Current, Magnitude, referred to network"
      m_inet_busdp = "m:inet:busdp", "p.u., Current, Magnitude, referred to network"
      m_inet_busdm = "m:inet:busdm", "p.u., Current, Magnitude, referred to network"
      m_I_busac = "m:I:busac", "kA, Current, Magnitude"
      m_I_busdp = "m:I:busdp", "kA, Current, Magnitude"
      m_I_busdm = "m:I:busdm", "kA, Current, Magnitude"
      m_phii_busac = "m:phii:busac", "deg, Current, Angle"
      m_phii_busdp = "m:phii:busdp", "deg, Current, Angle"
      m_phii_busdm = "m:phii:busdm", "deg, Current, Angle"
      m_i1r_busac = "m:i1r:busac", "p.u., Positive-Sequence Current, Real Part"
      m_i1r_busdp = "m:i1r:busdp", "p.u., Positive-Sequence Current, Real Part"
      m_i1r_busdm = "m:i1r:busdm", "p.u., Positive-Sequence Current, Real Part"
      m_i1i_busac = "m:i1i:busac", "p.u., Positive-Sequence Current, Imaginary Part"
      m_i1i_busdp = "m:i1i:busdp", "p.u., Positive-Sequence Current, Imaginary Part"
      m_i1i_busdm = "m:i1i:busdm", "p.u., Positive-Sequence Current, Imaginary Part"
      m_i1_busac = "m:i1:busac", "p.u., Positive-Sequence Current, Magnitude"
      m_i1_busdp = "m:i1:busdp", "p.u., Positive-Sequence Current, Magnitude"
      m_i1_busdm = "m:i1:busdm", "p.u., Positive-Sequence Current, Magnitude"
      m_I1_busac = "m:I1:busac", "kA, Positive-Sequence Current, Magnitude"
      m_I1_busdp = "m:I1:busdp", "kA, Positive-Sequence Current, Magnitude"
      m_I1_busdm = "m:I1:busdm", "kA, Positive-Sequence Current, Magnitude"
      m_phii1_busac = "m:phii1:busac", "deg, Positive-Sequence Current, Angle"
      m_phii1_busdp = "m:phii1:busdp", "deg, Positive-Sequence Current, Angle"
      m_phii1_busdm = "m:phii1:busdm", "deg, Positive-Sequence Current, Angle"
      m_P_busac = "m:P:busac", "MW, Active Power"
      m_P_busdp = "m:P:busdp", "MW, Active Power"
      m_P_busdm = "m:P:busdm", "MW, Active Power"
      m_Q_busac = "m:Q:busac", "Mvar, Reactive Power"
      m_Q_busdp = "m:Q:busdp", "Mvar, Reactive Power"
      m_Q_busdm = "m:Q:busdm", "Mvar, Reactive Power"
      m_S_busac = "m:S:busac", "MVA, Apparent Power"
      m_S_busdp = "m:S:busdp", "MVA, Apparent Power"
      m_S_busdm = "m:S:busdm", "MVA, Apparent Power"
      m_cosphi_busac = "m:cosphi:busac", "Power Factor"
      m_cosphi_busdp = "m:cosphi:busdp", "Power Factor"
      m_cosphi_busdm = "m:cosphi:busdm", "Power Factor"
      m_tanphi_busac = "m:tanphi:busac", "tan(phi)"
      m_tanphi_busdp = "m:tanphi:busdp", "tan(phi)"
      m_tanphi_busdm = "m:tanphi:busdm", "tan(phi)"
      m_Psum_busac = "m:Psum:busac", "MW, Total Active Power"
      m_Psum_busdp = "m:Psum:busdp", "MW, Total Active Power"
      m_Psum_busdm = "m:Psum:busdm", "MW, Total Active Power"
      m_Qsum_busac = "m:Qsum:busac", "Mvar, Total Reactive Power"
      m_Qsum_busdp = "m:Qsum:busdp", "Mvar, Total Reactive Power"
      m_Qsum_busdm = "m:Qsum:busdm", "Mvar, Total Reactive Power"
      m_Ssum_busac = "m:Ssum:busac", "MVA, Total Apparent Power"
      m_Ssum_busdp = "m:Ssum:busdp", "MVA, Total Apparent Power"
      m_Ssum_busdm = "m:Ssum:busdm", "MVA, Total Apparent Power"
      m_cosphisum_busac = "m:cosphisum:busac", "Total Power Factor"
      m_cosphisum_busdp = "m:cosphisum:busdp", "Total Power Factor"
      m_cosphisum_busdm = "m:cosphisum:busdm", "Total Power Factor"
      m_tanphisum_busac = "m:tanphisum:busac", "Total tan(phi)"
      m_tanphisum_busdp = "m:tanphisum:busdp", "Total tan(phi)"
      m_tanphisum_busdm = "m:tanphisum:busdm", "Total tan(phi)"
      m_Spu_busac = "m:Spu:busac", "MVA/p.u., Apparent Power per p.u. Voltage"
      m_Spu_busdp = "m:Spu:busdp", "MVA/p.u., Apparent Power per p.u. Voltage"
      m_Spu_busdm = "m:Spu:busdm", "MVA/p.u., Apparent Power per p.u. Voltage"
      m_i1P_busac = "m:i1P:busac", "p.u., Positive-Sequence Active Current"
      m_i1P_busdp = "m:i1P:busdp", "p.u., Positive-Sequence Active Current"
      m_i1P_busdm = "m:i1P:busdm", "p.u., Positive-Sequence Active Current"
      m_i1Q_busac = "m:i1Q:busac", "p.u., Positive-Sequence Reactive Current"
      m_i1Q_busdp = "m:i1Q:busdp", "p.u., Positive-Sequence Reactive Current"
      m_i1Q_busdm = "m:i1Q:busdm", "p.u., Positive-Sequence Reactive Current"
      m_I1P_busac = "m:I1P:busac", "kA, Positive-Sequence Active Current"
      m_I1P_busdp = "m:I1P:busdp", "kA, Positive-Sequence Active Current"
      m_I1P_busdm = "m:I1P:busdm", "kA, Positive-Sequence Active Current"
      m_I1Q_busac = "m:I1Q:busac", "kA, Positive-Sequence Reactive Current"
      m_I1Q_busdp = "m:I1Q:busdp", "kA, Positive-Sequence Reactive Current"
      m_I1Q_busdm = "m:I1Q:busdm", "kA, Positive-Sequence Reactive Current"
      m_ur_busac = "m:ur:busac", "p.u., Voltage, Real Part"
      m_ur_busdp = "m:ur:busdp", "p.u., Voltage, Real Part"
      m_ur_busdm = "m:ur:busdm", "p.u., Voltage, Real Part"
      m_ui_busac = "m:ui:busac", "p.u., Voltage, Imaginary Part"
      m_ui_busdp = "m:ui:busdp", "p.u., Voltage, Imaginary Part"
      m_ui_busdm = "m:ui:busdm", "p.u., Voltage, Imaginary Part"
      m_u_busac = "m:u:busac", "p.u., Voltage, Magnitude"
      m_u_busdp = "m:u:busdp", "p.u., Voltage, Magnitude"
      m_u_busdm = "m:u:busdm", "p.u., Voltage, Magnitude"
      m_ir_busac = "m:ir:busac", "p.u., Current, Real Part"
      m_ir_busdp = "m:ir:busdp", "p.u., Current, Real Part"
      m_ir_busdm = "m:ir:busdm", "p.u., Current, Real Part"
      m_ii_busac = "m:ii:busac", "p.u., Current, Imaginary Part"
      m_ii_busdp = "m:ii:busdp", "p.u., Current, Imaginary Part"
      m_ii_busdm = "m:ii:busdm", "p.u., Current, Imaginary Part"
      m_i_busac = "m:i:busac", "p.u., Current, Magnitude"
      m_i_busdp = "m:i:busdp", "p.u., Current, Magnitude"
      m_i_busdm = "m:i:busdm", "p.u., Current, Magnitude"
      m_phiui_busac = "m:phiui:busac", "deg, Angle between Voltage and Current"
      m_phiui_busdp = "m:phiui:busdp", "deg, Angle between Voltage and Current"
      m_phiui_busdm = "m:phiui:busdm", "deg, Angle between Voltage and Current"
      m_phiu1i1_busac = "m:phiu1i1:busac", "deg, Angle between Voltage and Current in positive sequence system"
      m_phiu1i1_busdp = "m:phiu1i1:busdp", "deg, Angle between Voltage and Current in positive sequence system"
      m_phiu1i1_busdm = "m:phiu1i1:busdm", "deg, Angle between Voltage and Current in positive sequence system"
      m_Tfct_busac = "m:Tfct:busac", "s, Fault Clearing Time"
      m_Tfct_busdp = "m:Tfct:busdp", "s, Fault Clearing Time"
      m_Tfct_busdm = "m:Tfct:busdm", "s, Fault Clearing Time"
      m_Brkload_busac = "m:Brkload:busac", "%, Breaker Loading"
      m_Brkload_busdp = "m:Brkload:busdp", "%, Breaker Loading"
      m_Brkload_busdm = "m:Brkload:busdm", "%, Breaker Loading"
      n_ur_busac = "n:ur:busac", "p.u., Voltage, Real Part"
      n_ur_busdp = "n:ur:busdp", "p.u., Voltage, Real Part"
      n_ur_busdm = "n:ur:busdm", "p.u., Voltage, Real Part"
      n_ui_busac = "n:ui:busac", "p.u., Voltage, Imaginary Part"
      n_ui_busdp = "n:ui:busdp", "p.u., Voltage, Imaginary Part"
      n_ui_busdm = "n:ui:busdm", "p.u., Voltage, Imaginary Part"
      n_u_busac = "n:u:busac", "p.u., Voltage, Magnitude"
      n_u_busdp = "n:u:busdp", "p.u., Voltage, Magnitude"
      n_u_busdm = "n:u:busdm", "p.u., Voltage, Magnitude"
      n_upc_busac = "n:upc:busac", "%, Voltage, Magnitude"
      n_upc_busdp = "n:upc:busdp", "%, Voltage, Magnitude"
      n_upc_busdm = "n:upc:busdm", "%, Voltage, Magnitude"
      n_u1_busac = "n:u1:busac", "p.u., Voltage, Magnitude"
      n_u1_busdp = "n:u1:busdp", "p.u., Voltage, Magnitude"
      n_u1_busdm = "n:u1:busdm", "p.u., Voltage, Magnitude"
      n_u1pc_busac = "n:u1pc:busac", "%, Voltage, Magnitude"
      n_u1pc_busdp = "n:u1pc:busdp", "%, Voltage, Magnitude"
      n_u1pc_busdm = "n:u1pc:busdm", "%, Voltage, Magnitude"
      n_u1r_busac = "n:u1r:busac", "p.u., Positive-Sequence Voltage, Real Part"
      n_u1r_busdp = "n:u1r:busdp", "p.u., Positive-Sequence Voltage, Real Part"
      n_u1r_busdm = "n:u1r:busdm", "p.u., Positive-Sequence Voltage, Real Part"
      n_u1i_busac = "n:u1i:busac", "p.u., Positive-Sequence Voltage, Imaginary Part"
      n_u1i_busdp = "n:u1i:busdp", "p.u., Positive-Sequence Voltage, Imaginary Part"
      n_u1i_busdm = "n:u1i:busdm", "p.u., Positive-Sequence Voltage, Imaginary Part"
      n_U_busac = "n:U:busac", "kV, Line-Ground Voltage, Magnitude"
      n_U_busdp = "n:U:busdp", "kV, Line-Ground Voltage, Magnitude"
      n_U_busdm = "n:U:busdm", "kV, Line-Ground Voltage, Magnitude"
      n_Ul_busac = "n:Ul:busac", "kV, Line-Line Voltage, Magnitude"
      n_Ul_busdp = "n:Ul:busdp", "kV, Line-Line Voltage, Magnitude"
      n_Ul_busdm = "n:Ul:busdm", "kV, Line-Line Voltage, Magnitude"
      n_phiu_busac = "n:phiu:busac", "deg, Voltage, Angle"
      n_phiu_busdp = "n:phiu:busdp", "deg, Voltage, Angle"
      n_phiu_busdm = "n:phiu:busdm", "deg, Voltage, Angle"
      n_phiurel_busac = "n:phiurel:busac", "deg, Voltage, Relative Angle"
      n_phiurel_busdp = "n:phiurel:busdp", "deg, Voltage, Relative Angle"
      n_phiurel_busdm = "n:phiurel:busdm", "deg, Voltage, Relative Angle"
      n_du_busac = "n:du:busac", "%, Voltage Deviation"
      n_du_busdp = "n:du:busdp", "%, Voltage Deviation"
      n_du_busdm = "n:du:busdm", "%, Voltage Deviation"
      n_ishcr_busac = "n:ishcr:busac", "p.u., Short-Circuit Current, real"
      n_ishcr_busdp = "n:ishcr:busdp", "p.u., Short-Circuit Current, real"
      n_ishcr_busdm = "n:ishcr:busdm", "p.u., Short-Circuit Current, real"
      n_ishci_busac = "n:ishci:busac", "p.u., Short-Circuit Current, imag"
      n_ishci_busdp = "n:ishci:busdp", "p.u., Short-Circuit Current, imag"
      n_ishci_busdm = "n:ishci:busdm", "p.u., Short-Circuit Current, imag"
      n_ishc_busac = "n:ishc:busac", "p.u., Short-Circuit Current, magnitude"
      n_ishc_busdp = "n:ishc:busdp", "p.u., Short-Circuit Current, magnitude"
      n_ishc_busdm = "n:ishc:busdm", "p.u., Short-Circuit Current, magnitude"
      n_Ishc_busac = "n:Ishc:busac", "kA, Short-Circuit Current, magnitude"
      n_Ishc_busdp = "n:Ishc:busdp", "kA, Short-Circuit Current, magnitude"
      n_Ishc_busdm = "n:Ishc:busdm", "kA, Short-Circuit Current, magnitude"
      n_fe_busac = "n:fe:busac", "p.u., Electrical Frequency"
      n_fe_busdp = "n:fe:busdp", "p.u., Electrical Frequency"
      n_fe_busdm = "n:fe:busdm", "p.u., Electrical Frequency"
      n_frdev_busac = "n:frdev:busac", "Hz, average frequency"
      n_frdev_busdp = "n:frdev:busdp", "Hz, average frequency"
      n_frdev_busdm = "n:frdev:busdm", "Hz, average frequency"
      n_dfehz_busac = "n:dfehz:busac", "Hz, Deviation of the El. Frequency"
      n_dfehz_busdp = "n:dfehz:busdp", "Hz, Deviation of the El. Frequency"
      n_dfehz_busdm = "n:dfehz:busdm", "Hz, Deviation of the El. Frequency"
      n_dfedt_busac = "n:dfedt:busac", "p.u./s, Derivative of El. Frequency"
      n_dfedt_busdp = "n:dfedt:busdp", "p.u./s, Derivative of El. Frequency"
      n_dfedt_busdm = "n:dfedt:busdm", "p.u./s, Derivative of El. Frequency"
      n_fehz_busac = "n:fehz:busac", "Hz, Electrical Frequency"
      n_fehz_busdp = "n:fehz:busdp", "Hz, Electrical Frequency"
      n_fehz_busdm = "n:fehz:busdm", "Hz, Electrical Frequency"
      n_frnom_busac = "n:frnom:busac", "Hz, Nominal Frequency"
      n_frnom_busdp = "n:frnom:busdp", "Hz, Nominal Frequency"
      n_frnom_busdm = "n:frnom:busdm", "Hz, Nominal Frequency"
      s_Pm_in = "s:Pm_in", "p.u., IN, Pulse-modulation factor (input) continuous, double"
      s_f0 = "s:f0", "p.u., IN, Frequency-input continuous, double"
      s_F0Hz = "s:F0Hz", "Hz, IN, Frequency-input continuous, double"
      s_dphiu = "s:dphiu", "rad, IN, Voltage-angle (input), continuous, double"
      s_fref = "s:fref", "p.u., IN, Reference frequency, continuous, double"
      s_Pmr = "s:Pmr", "IN, Pulse-modulation factor (real part), continuous, double"
      s_Pmi = "s:Pmi", "IN, Pulse-modulation factor (imaginary part), continuous, double"
      s_cosref = "s:cosref", "IN, Cos(dq-reference-angle) continuous, double"
      s_sinref = "s:sinref", "IN, Sin(dq-reference-angle) continuous, double"
      s_Pmd = "s:Pmd", "IN, Pulse-modulation factor (d-axis), continuous, double"
      s_Pmq = "s:Pmq", "IN, Pulse-modulation factor (q-axis), continuous, double"
      s_id_ref = "s:id_ref", "p.u., IN, d-axis current reference, continuous, double"
      s_iq_ref = "s:iq_ref", "p.u., IN, q-axis current reference, continuous, double"
      s_block = "s:block", "p.u., IN, Blocking, continuous, double"
      s_mdc = "s:mdc", "p.u., IN, Insertion index, DC-bias, continuous, double"
      s_xspeed = "s:xspeed", "p.u., OUT, Frequency, continuous, double"
      s_iDC = "s:iDC", "p.u., OUT, DC-current, continuous, double"
      s_uDC = "s:uDC", "p.u., OUT, DC-voltage, continuous, double"
      s_iAC_r = "s:iAC_r", "p.u., OUT, AC-current, real part, continuous, double"
      s_iAC_i = "s:iAC_i", "p.u., OUT, AC-current, imaginary part, continuous, double"
      s_uAC_r = "s:uAC_r", "p.u., OUT, AC-voltage, real part, continuous, double"
      s_uAC_i = "s:uAC_i", "p.u., OUT, AC-voltage, imaginary part, continuous, double"
      s_id = "s:id", "p.u., OUT, Current, d-axis continuous, double"
      s_iq = "s:iq", "p.u., OUT, Current, q-axis continuous, double"
      s_yUcell = "s:yUcell", "OUT, Voltage across cell capacitor, output, continuous, double"
      s_phiu = "s:phiu", "rad, STATE, Voltage-angle, continuous, double"
      s_xd = "s:xd", "p.u., STATE, State variable, current controller d-axis, continuous, double"
      s_xq = "s:xq", "p.u., STATE, State variable, current controller q-axis, continuous, double"
      s_Ucell = "s:Ucell", "kV, STATE, Voltage across cell capacitor, continuous, double"
      s_phiu_dt = "s:phiu:dt", "rad/s, d/dt, Voltage-angle, derivative, continuous, double"
      s_xd_dt = "s:xd:dt", "p.u./s, d/dt, State variable, current controller d-axis, derivative, continuous, double"
      s_xq_dt = "s:xq:dt", "p.u./s, d/dt, State variable, current controller q-axis, derivative, continuous, double"
      s_Ucell_dt = "s:Ucell:dt", "kV/s, d/dt, Voltage across cell capacitor, derivative, continuous, double"
      c_hpi = "c:hpi", "rad/s, Nominal-angular-frequency"
      c_Pm = "c:Pm", "Pulse-modulation factor"
      c_snomn = "c:snomn", "MVA, Total rated apparent power (nparnum*Snom)"
      c_i_mod = "c:i_mod", "Modulation"
      c_K0 = "c:K0", "Vac/Vdc at PWM=1"
      c_DCPoleLosses = "c:DCPoleLosses", "MW, DC pole losses"
      c_Pmd_ctrl = "c:Pmd_ctrl", "Pulse-modulation factor (d-axis)"
      c_Pmq_ctrl = "c:Pmq_ctrl", "Pulse-modulation factor (q-axis)"
      c_Ccell = "c:Ccell", "F, Cell capacitor"
      c_i_acdc = "c:i_acdc", "Control mode"
      e_iAstabint = "e:iAstabint", "A-stable integration algorithm"
      e_iVIsource = "e:iVIsource", "Independent AC and DC voltage/current source model"
      e_Rac = "e:Rac", "Ohm, AC resistance"
      e_Lac = "e:Lac", "mH, AC inductance"
      e_R0toR1ac = "e:R0toR1ac", "R0/R1 ratio"
      e_X0toX1ac = "e:X0toX1ac", "X0/X1 ratio"
      e_Rdc = "e:Rdc", "Ohm, DC resistance"
      e_Ldc = "e:Ldc", "mH, DC inductance"
      e_Pnold = "e:Pnold", "kW, No-load losses"
      e_swtLossFactor = "e:swtLossFactor", "kW/A, Switching loss factor"
      e_resLossFactor = "e:resLossFactor", "Ohm, Resistive loss factor"
      e_Xd = "e:Xd", "Ohm, Commutation reactance"
      e_Cdc = "e:Cdc", "uF, Cell capacitor"
      e_mmcCmod = "e:mmcCmod", "uF, Submodule capacitance"
      e_NmmcSM = "e:NmmcSM", "Number of submodules per arm"
      e_r2 = "e:r2", "p.u., Resistance, r2"
      e_x2 = "e:x2", "p.u., Reactance, x2"
      e_i_ctrl = "e:i_ctrl", "Use integrated current controller"
      e_Kd = "e:Kd", "Kd: d-axis, proportional gain"
      e_Td = "e:Td", "s, Td: d-axis, integration time constant"
      e_Kq = "e:Kq", "Kq: q-axis, proportional gain"
      e_Tq = "e:Tq", "s, Tq: q-axis, integration time constant"

    class ElmVscmono(Enum):
      _init_ = 'value __doc__'

      m_u1r_busac = "m:u1r:busac", "p.u., Positive-Sequence-Voltage, Real Part"
      m_u1r_busdc = "m:u1r:busdc", "p.u., Positive-Sequence-Voltage, Real Part"
      m_u1i_busac = "m:u1i:busac", "p.u., Positive-Sequence-Voltage, Imaginary Part"
      m_u1i_busdc = "m:u1i:busdc", "p.u., Positive-Sequence-Voltage, Imaginary Part"
      m_u1_busac = "m:u1:busac", "p.u., Positive-Sequence-Voltage, Magnitude"
      m_u1_busdc = "m:u1:busdc", "p.u., Positive-Sequence-Voltage, Magnitude"
      m_U1_busac = "m:U1:busac", "kV, Line-Ground Positive-Sequence-Voltage, Magnitude"
      m_U1_busdc = "m:U1:busdc", "kV, Line-Ground Positive-Sequence-Voltage, Magnitude"
      m_U1l_busac = "m:U1l:busac", "kV, Line-Line Positive-Sequence-Voltage, Magnitude"
      m_U1l_busdc = "m:U1l:busdc", "kV, Line-Line Positive-Sequence-Voltage, Magnitude"
      m_phiu1_busac = "m:phiu1:busac", "deg, Positive-Sequence-Voltage, Angle"
      m_phiu1_busdc = "m:phiu1:busdc", "deg, Positive-Sequence-Voltage, Angle"
      m_inet_busac = "m:inet:busac", "p.u., Current, Magnitude, referred to network"
      m_inet_busdc = "m:inet:busdc", "p.u., Current, Magnitude, referred to network"
      m_I_busac = "m:I:busac", "kA, Current, Magnitude"
      m_I_busdc = "m:I:busdc", "kA, Current, Magnitude"
      m_phii_busac = "m:phii:busac", "deg, Current, Angle"
      m_phii_busdc = "m:phii:busdc", "deg, Current, Angle"
      m_i1r_busac = "m:i1r:busac", "p.u., Positive-Sequence Current, Real Part"
      m_i1r_busdc = "m:i1r:busdc", "p.u., Positive-Sequence Current, Real Part"
      m_i1i_busac = "m:i1i:busac", "p.u., Positive-Sequence Current, Imaginary Part"
      m_i1i_busdc = "m:i1i:busdc", "p.u., Positive-Sequence Current, Imaginary Part"
      m_i1_busac = "m:i1:busac", "p.u., Positive-Sequence Current, Magnitude"
      m_i1_busdc = "m:i1:busdc", "p.u., Positive-Sequence Current, Magnitude"
      m_I1_busac = "m:I1:busac", "kA, Positive-Sequence Current, Magnitude"
      m_I1_busdc = "m:I1:busdc", "kA, Positive-Sequence Current, Magnitude"
      m_phii1_busac = "m:phii1:busac", "deg, Positive-Sequence Current, Angle"
      m_phii1_busdc = "m:phii1:busdc", "deg, Positive-Sequence Current, Angle"
      m_P_busac = "m:P:busac", "MW, Active Power"
      m_P_busdc = "m:P:busdc", "MW, Active Power"
      m_Q_busac = "m:Q:busac", "Mvar, Reactive Power"
      m_Q_busdc = "m:Q:busdc", "Mvar, Reactive Power"
      m_S_busac = "m:S:busac", "MVA, Apparent Power"
      m_S_busdc = "m:S:busdc", "MVA, Apparent Power"
      m_cosphi_busac = "m:cosphi:busac", "Power Factor"
      m_cosphi_busdc = "m:cosphi:busdc", "Power Factor"
      m_tanphi_busac = "m:tanphi:busac", "tan(phi)"
      m_tanphi_busdc = "m:tanphi:busdc", "tan(phi)"
      m_Psum_busac = "m:Psum:busac", "MW, Total Active Power"
      m_Psum_busdc = "m:Psum:busdc", "MW, Total Active Power"
      m_Qsum_busac = "m:Qsum:busac", "Mvar, Total Reactive Power"
      m_Qsum_busdc = "m:Qsum:busdc", "Mvar, Total Reactive Power"
      m_Ssum_busac = "m:Ssum:busac", "MVA, Total Apparent Power"
      m_Ssum_busdc = "m:Ssum:busdc", "MVA, Total Apparent Power"
      m_cosphisum_busac = "m:cosphisum:busac", "Total Power Factor"
      m_cosphisum_busdc = "m:cosphisum:busdc", "Total Power Factor"
      m_tanphisum_busac = "m:tanphisum:busac", "Total tan(phi)"
      m_tanphisum_busdc = "m:tanphisum:busdc", "Total tan(phi)"
      m_Spu_busac = "m:Spu:busac", "MVA/p.u., Apparent Power per p.u. Voltage"
      m_Spu_busdc = "m:Spu:busdc", "MVA/p.u., Apparent Power per p.u. Voltage"
      m_i1P_busac = "m:i1P:busac", "p.u., Positive-Sequence Active Current"
      m_i1P_busdc = "m:i1P:busdc", "p.u., Positive-Sequence Active Current"
      m_i1Q_busac = "m:i1Q:busac", "p.u., Positive-Sequence Reactive Current"
      m_i1Q_busdc = "m:i1Q:busdc", "p.u., Positive-Sequence Reactive Current"
      m_I1P_busac = "m:I1P:busac", "kA, Positive-Sequence Active Current"
      m_I1P_busdc = "m:I1P:busdc", "kA, Positive-Sequence Active Current"
      m_I1Q_busac = "m:I1Q:busac", "kA, Positive-Sequence Reactive Current"
      m_I1Q_busdc = "m:I1Q:busdc", "kA, Positive-Sequence Reactive Current"
      m_ur_busac = "m:ur:busac", "p.u., Voltage, Real Part"
      m_ur_busdc = "m:ur:busdc", "p.u., Voltage, Real Part"
      m_ui_busac = "m:ui:busac", "p.u., Voltage, Imaginary Part"
      m_ui_busdc = "m:ui:busdc", "p.u., Voltage, Imaginary Part"
      m_u_busac = "m:u:busac", "p.u., Voltage, Magnitude"
      m_u_busdc = "m:u:busdc", "p.u., Voltage, Magnitude"
      m_ir_busac = "m:ir:busac", "p.u., Current, Real Part"
      m_ir_busdc = "m:ir:busdc", "p.u., Current, Real Part"
      m_ii_busac = "m:ii:busac", "p.u., Current, Imaginary Part"
      m_ii_busdc = "m:ii:busdc", "p.u., Current, Imaginary Part"
      m_i_busac = "m:i:busac", "p.u., Current, Magnitude"
      m_i_busdc = "m:i:busdc", "p.u., Current, Magnitude"
      m_phiui_busac = "m:phiui:busac", "deg, Angle between Voltage and Current"
      m_phiui_busdc = "m:phiui:busdc", "deg, Angle between Voltage and Current"
      m_phiu1i1_busac = "m:phiu1i1:busac", "deg, Angle between Voltage and Current in positive sequence system"
      m_phiu1i1_busdc = "m:phiu1i1:busdc", "deg, Angle between Voltage and Current in positive sequence system"
      m_Tfct_busac = "m:Tfct:busac", "s, Fault Clearing Time"
      m_Tfct_busdc = "m:Tfct:busdc", "s, Fault Clearing Time"
      m_Brkload_busac = "m:Brkload:busac", "%, Breaker Loading"
      m_Brkload_busdc = "m:Brkload:busdc", "%, Breaker Loading"
      n_ur_busac = "n:ur:busac", "p.u., Voltage, Real Part"
      n_ur_busdc = "n:ur:busdc", "p.u., Voltage, Real Part"
      n_ui_busac = "n:ui:busac", "p.u., Voltage, Imaginary Part"
      n_ui_busdc = "n:ui:busdc", "p.u., Voltage, Imaginary Part"
      n_u_busac = "n:u:busac", "p.u., Voltage, Magnitude"
      n_u_busdc = "n:u:busdc", "p.u., Voltage, Magnitude"
      n_upc_busac = "n:upc:busac", "%, Voltage, Magnitude"
      n_upc_busdc = "n:upc:busdc", "%, Voltage, Magnitude"
      n_u1_busac = "n:u1:busac", "p.u., Voltage, Magnitude"
      n_u1_busdc = "n:u1:busdc", "p.u., Voltage, Magnitude"
      n_u1pc_busac = "n:u1pc:busac", "%, Voltage, Magnitude"
      n_u1pc_busdc = "n:u1pc:busdc", "%, Voltage, Magnitude"
      n_u1r_busac = "n:u1r:busac", "p.u., Positive-Sequence Voltage, Real Part"
      n_u1r_busdc = "n:u1r:busdc", "p.u., Positive-Sequence Voltage, Real Part"
      n_u1i_busac = "n:u1i:busac", "p.u., Positive-Sequence Voltage, Imaginary Part"
      n_u1i_busdc = "n:u1i:busdc", "p.u., Positive-Sequence Voltage, Imaginary Part"
      n_U_busac = "n:U:busac", "kV, Line-Ground Voltage, Magnitude"
      n_U_busdc = "n:U:busdc", "kV, Line-Ground Voltage, Magnitude"
      n_Ul_busac = "n:Ul:busac", "kV, Line-Line Voltage, Magnitude"
      n_Ul_busdc = "n:Ul:busdc", "kV, Line-Line Voltage, Magnitude"
      n_phiu_busac = "n:phiu:busac", "deg, Voltage, Angle"
      n_phiu_busdc = "n:phiu:busdc", "deg, Voltage, Angle"
      n_phiurel_busac = "n:phiurel:busac", "deg, Voltage, Relative Angle"
      n_phiurel_busdc = "n:phiurel:busdc", "deg, Voltage, Relative Angle"
      n_du_busac = "n:du:busac", "%, Voltage Deviation"
      n_du_busdc = "n:du:busdc", "%, Voltage Deviation"
      n_ishcr_busac = "n:ishcr:busac", "p.u., Short-Circuit Current, real"
      n_ishcr_busdc = "n:ishcr:busdc", "p.u., Short-Circuit Current, real"
      n_ishci_busac = "n:ishci:busac", "p.u., Short-Circuit Current, imag"
      n_ishci_busdc = "n:ishci:busdc", "p.u., Short-Circuit Current, imag"
      n_ishc_busac = "n:ishc:busac", "p.u., Short-Circuit Current, magnitude"
      n_ishc_busdc = "n:ishc:busdc", "p.u., Short-Circuit Current, magnitude"
      n_Ishc_busac = "n:Ishc:busac", "kA, Short-Circuit Current, magnitude"
      n_Ishc_busdc = "n:Ishc:busdc", "kA, Short-Circuit Current, magnitude"
      n_fe_busac = "n:fe:busac", "p.u., Electrical Frequency"
      n_fe_busdc = "n:fe:busdc", "p.u., Electrical Frequency"
      n_frdev_busac = "n:frdev:busac", "Hz, average frequency"
      n_frdev_busdc = "n:frdev:busdc", "Hz, average frequency"
      n_dfehz_busac = "n:dfehz:busac", "Hz, Deviation of the El. Frequency"
      n_dfehz_busdc = "n:dfehz:busdc", "Hz, Deviation of the El. Frequency"
      n_dfedt_busac = "n:dfedt:busac", "p.u./s, Derivative of El. Frequency"
      n_dfedt_busdc = "n:dfedt:busdc", "p.u./s, Derivative of El. Frequency"
      n_fehz_busac = "n:fehz:busac", "Hz, Electrical Frequency"
      n_fehz_busdc = "n:fehz:busdc", "Hz, Electrical Frequency"
      n_frnom_busac = "n:frnom:busac", "Hz, Nominal Frequency"
      n_frnom_busdc = "n:frnom:busdc", "Hz, Nominal Frequency"
      s_Pm_in = "s:Pm_in", "p.u., IN, Pulse-modulation factor (input) continuous, double"
      s_f0 = "s:f0", "p.u., IN, Frequency-input continuous, double"
      s_F0Hz = "s:F0Hz", "Hz, IN, Frequency-input continuous, double"
      s_dphiu = "s:dphiu", "rad, IN, Voltage-angle (input), continuous, double"
      s_fref = "s:fref", "p.u., IN, Reference frequency, continuous, double"
      s_Pmr = "s:Pmr", "IN, Pulse-modulation factor (real part), continuous, double"
      s_Pmi = "s:Pmi", "IN, Pulse-modulation factor (imaginary part), continuous, double"
      s_cosref = "s:cosref", "IN, Cos(dq-reference-angle) continuous, double"
      s_sinref = "s:sinref", "IN, Sin(dq-reference-angle) continuous, double"
      s_Pmd = "s:Pmd", "IN, Pulse-modulation factor (d-axis), continuous, double"
      s_Pmq = "s:Pmq", "IN, Pulse-modulation factor (q-axis), continuous, double"
      s_id_ref = "s:id_ref", "p.u., IN, d-axis current reference, continuous, double"
      s_iq_ref = "s:iq_ref", "p.u., IN, q-axis current reference, continuous, double"
      s_block = "s:block", "p.u., IN, Blocking, continuous, double"
      s_mdc = "s:mdc", "p.u., IN, Insertion index, DC-bias, continuous, double"
      s_xspeed = "s:xspeed", "p.u., OUT, Frequency, continuous, double"
      s_iDC = "s:iDC", "p.u., OUT, DC-current, continuous, double"
      s_uDC = "s:uDC", "p.u., OUT, DC-voltage, continuous, double"
      s_iAC_r = "s:iAC_r", "p.u., OUT, AC-current, real part, continuous, double"
      s_iAC_i = "s:iAC_i", "p.u., OUT, AC-current, imaginary part, continuous, double"
      s_uAC_r = "s:uAC_r", "p.u., OUT, AC-voltage, real part, continuous, double"
      s_uAC_i = "s:uAC_i", "p.u., OUT, AC-voltage, imaginary part, continuous, double"
      s_id = "s:id", "p.u., OUT, Current, d-axis continuous, double"
      s_iq = "s:iq", "p.u., OUT, Current, q-axis continuous, double"
      s_yUcell = "s:yUcell", "OUT, Voltage across cell capacitor, output, continuous, double"
      s_phiu = "s:phiu", "rad, STATE, Voltage-angle, continuous, double"
      s_xd = "s:xd", "p.u., STATE, State variable, current controller d-axis, continuous, double"
      s_xq = "s:xq", "p.u., STATE, State variable, current controller q-axis, continuous, double"
      s_Ucell = "s:Ucell", "kV, STATE, Voltage across cell capacitor, continuous, double"
      s_phiu_dt = "s:phiu:dt", "rad/s, d/dt, Voltage-angle, derivative, continuous, double"
      s_xd_dt = "s:xd:dt", "p.u./s, d/dt, State variable, current controller d-axis, derivative, continuous, double"
      s_xq_dt = "s:xq:dt", "p.u./s, d/dt, State variable, current controller q-axis, derivative, continuous, double"
      s_Ucell_dt = "s:Ucell:dt", "kV/s, d/dt, Voltage across cell capacitor, derivative, continuous, double"
      c_hpi = "c:hpi", "rad/s, Nominal-angular-frequency"
      c_Pm = "c:Pm", "Pulse-modulation factor"
      c_snomn = "c:snomn", "MVA, Total rated apparent power (nparnum*Snom)"
      c_i_mod = "c:i_mod", "Modulation"
      c_K0 = "c:K0", "Vac/Vdc at PWM=1"
      c_DCPoleLosses = "c:DCPoleLosses", "MW, DC pole losses"
      c_Pmd_ctrl = "c:Pmd_ctrl", "Pulse-modulation factor (d-axis)"
      c_Pmq_ctrl = "c:Pmq_ctrl", "Pulse-modulation factor (q-axis)"
      c_Ccell = "c:Ccell", "F, Cell capacitor"
      c_i_acdc = "c:i_acdc", "Control mode"
      e_iAstabint = "e:iAstabint", "A-stable integration algorithm"
      e_iVIsource = "e:iVIsource", "Independent AC and DC voltage/current source model"
      e_Rac = "e:Rac", "Ohm, AC resistance"
      e_Lac = "e:Lac", "mH, AC inductance"
      e_R0toR1ac = "e:R0toR1ac", "R0/R1 ratio"
      e_X0toX1ac = "e:X0toX1ac", "X0/X1 ratio"
      e_Rdc = "e:Rdc", "Ohm, DC resistance"
      e_Ldc = "e:Ldc", "mH, DC inductance"
      e_Pnold = "e:Pnold", "kW, No-load losses"
      e_swtLossFactor = "e:swtLossFactor", "kW/A, Switching loss factor"
      e_resLossFactor = "e:resLossFactor", "Ohm, Resistive loss factor"
      e_Xd = "e:Xd", "Ohm, Commutation reactance"
      e_Cdc = "e:Cdc", "uF, Cell capacitor"
      e_mmcCmod = "e:mmcCmod", "uF, Submodule capacitance"
      e_NmmcSM = "e:NmmcSM", "Number of submodules per arm"
      e_r2 = "e:r2", "p.u., Resistance, r2"
      e_x2 = "e:x2", "p.u., Reactance, x2"
      e_i_ctrl = "e:i_ctrl", "Use integrated current controller"
      e_Kd = "e:Kd", "Kd: d-axis, proportional gain"
      e_Td = "e:Td", "s, Td: d-axis, integration time constant"
      e_Kq = "e:Kq", "Kq: q-axis, proportional gain"
      e_Tq = "e:Tq", "s, Tq: q-axis, integration time constant"

    class ElmXnet(Enum):
      _init_ = 'value __doc__'

      m_u1r_bus1 = "m:u1r:bus1", "p.u., Positive-Sequence-Voltage, Real Part"
      m_u1i_bus1 = "m:u1i:bus1", "p.u., Positive-Sequence-Voltage, Imaginary Part"
      m_u1_bus1 = "m:u1:bus1", "p.u., Positive-Sequence-Voltage, Magnitude"
      m_U1_bus1 = "m:U1:bus1", "kV, Line-Ground Positive-Sequence-Voltage, Magnitude"
      m_U1l_bus1 = "m:U1l:bus1", "kV, Line-Line Positive-Sequence-Voltage, Magnitude"
      m_phiu1_bus1 = "m:phiu1:bus1", "deg, Positive-Sequence-Voltage, Angle"
      m_inet_bus1 = "m:inet:bus1", "p.u., Current, Magnitude, referred to network"
      m_I_bus1 = "m:I:bus1", "kA, Current, Magnitude"
      m_phii_bus1 = "m:phii:bus1", "deg, Current, Angle"
      m_i1r_bus1 = "m:i1r:bus1", "p.u., Positive-Sequence Current, Real Part"
      m_i1i_bus1 = "m:i1i:bus1", "p.u., Positive-Sequence Current, Imaginary Part"
      m_i1_bus1 = "m:i1:bus1", "p.u., Positive-Sequence Current, Magnitude"
      m_I1_bus1 = "m:I1:bus1", "kA, Positive-Sequence Current, Magnitude"
      m_phii1_bus1 = "m:phii1:bus1", "deg, Positive-Sequence Current, Angle"
      m_P_bus1 = "m:P:bus1", "MW, Active Power"
      m_Q_bus1 = "m:Q:bus1", "Mvar, Reactive Power"
      m_S_bus1 = "m:S:bus1", "MVA, Apparent Power"
      m_cosphi_bus1 = "m:cosphi:bus1", "Power Factor"
      m_tanphi_bus1 = "m:tanphi:bus1", "tan(phi)"
      m_Psum_bus1 = "m:Psum:bus1", "MW, Total Active Power"
      m_Qsum_bus1 = "m:Qsum:bus1", "Mvar, Total Reactive Power"
      m_Ssum_bus1 = "m:Ssum:bus1", "MVA, Total Apparent Power"
      m_cosphisum_bus1 = "m:cosphisum:bus1", "Total Power Factor"
      m_tanphisum_bus1 = "m:tanphisum:bus1", "Total tan(phi)"
      m_Spu_bus1 = "m:Spu:bus1", "MVA/p.u., Apparent Power per p.u. Voltage"
      m_i1P_bus1 = "m:i1P:bus1", "p.u., Positive-Sequence Active Current"
      m_i1Q_bus1 = "m:i1Q:bus1", "p.u., Positive-Sequence Reactive Current"
      m_I1P_bus1 = "m:I1P:bus1", "kA, Positive-Sequence Active Current"
      m_I1Q_bus1 = "m:I1Q:bus1", "kA, Positive-Sequence Reactive Current"
      m_ur_bus1 = "m:ur:bus1", "p.u., Voltage, Real Part"
      m_ui_bus1 = "m:ui:bus1", "p.u., Voltage, Imaginary Part"
      m_u_bus1 = "m:u:bus1", "p.u., Voltage, Magnitude"
      m_ir_bus1 = "m:ir:bus1", "p.u., Current, Real Part"
      m_ii_bus1 = "m:ii:bus1", "p.u., Current, Imaginary Part"
      m_i_bus1 = "m:i:bus1", "p.u., Current, Magnitude"
      m_phiui_bus1 = "m:phiui:bus1", "deg, Angle between Voltage and Current"
      m_phiu1i1_bus1 = "m:phiu1i1:bus1", "deg, Angle between Voltage and Current in positive sequence system"
      m_Tfct_bus1 = "m:Tfct:bus1", "s, Fault Clearing Time"
      m_Brkload_bus1 = "m:Brkload:bus1", "%, Breaker Loading"
      n_ur_bus1 = "n:ur:bus1", "p.u., Voltage, Real Part"
      n_ui_bus1 = "n:ui:bus1", "p.u., Voltage, Imaginary Part"
      n_u_bus1 = "n:u:bus1", "p.u., Voltage, Magnitude"
      n_upc_bus1 = "n:upc:bus1", "%, Voltage, Magnitude"
      n_u1_bus1 = "n:u1:bus1", "p.u., Voltage, Magnitude"
      n_u1pc_bus1 = "n:u1pc:bus1", "%, Voltage, Magnitude"
      n_u1r_bus1 = "n:u1r:bus1", "p.u., Positive-Sequence Voltage, Real Part"
      n_u1i_bus1 = "n:u1i:bus1", "p.u., Positive-Sequence Voltage, Imaginary Part"
      n_U_bus1 = "n:U:bus1", "kV, Line-Ground Voltage, Magnitude"
      n_Ul_bus1 = "n:Ul:bus1", "kV, Line-Line Voltage, Magnitude"
      n_phiu_bus1 = "n:phiu:bus1", "deg, Voltage, Angle"
      n_phiurel_bus1 = "n:phiurel:bus1", "deg, Voltage, Relative Angle"
      n_du_bus1 = "n:du:bus1", "%, Voltage Deviation"
      n_ishcr_bus1 = "n:ishcr:bus1", "p.u., Short-Circuit Current, real"
      n_ishci_bus1 = "n:ishci:bus1", "p.u., Short-Circuit Current, imag"
      n_ishc_bus1 = "n:ishc:bus1", "p.u., Short-Circuit Current, magnitude"
      n_Ishc_bus1 = "n:Ishc:bus1", "kA, Short-Circuit Current, magnitude"
      n_fe_bus1 = "n:fe:bus1", "p.u., Electrical Frequency"
      n_frdev_bus1 = "n:frdev:bus1", "Hz, average frequency"
      n_dfehz_bus1 = "n:dfehz:bus1", "Hz, Deviation of the El. Frequency"
      n_dfedt_bus1 = "n:dfedt:bus1", "p.u./s, Derivative of El. Frequency"
      n_fehz_bus1 = "n:fehz:bus1", "Hz, Electrical Frequency"
      n_frnom_bus1 = "n:frnom:bus1", "Hz, Nominal Frequency"
      s_ve = "s:ve", "p.u., IN, Excitation voltage, continuous, double"
      s_pt = "s:pt", "p.u., IN, Turbine Power, continuous, double"
      s_xmdm = "s:xmdm", "p.u., IN, Mechanical Torque, continuous, double"
      s_fref = "s:fref", "p.u., IN, Reference Frequency, continuous, double"
      s_uout = "s:uout", "p.u., OUT, Voltage continuous, double"
      s_xspeed = "s:xspeed", "p.u., OUT, Speed, continuous, double"
      s_psisr = "s:psisr", "p.u., STATE, Stator Flux, Real Part, continuous, double"
      s_psisi = "s:psisi", "p.u., STATE, Stator Flux, Imaginary Part, continuous, double"
      s_psie = "s:psie", "p.u., STATE, Excitation-Flux continuous, double"
      s_psiD = "s:psiD", "p.u., STATE, Flux in D-winding, continuous, double"
      s_psix = "s:psix", "p.u., STATE, Flux in x-winding, continuous, double"
      s_psiQ = "s:psiQ", "p.u., STATE, Flux in Q-winding, continuous, double"
      s_speed = "s:speed", "p.u., STATE, Speed, continuous, double"
      s_phi = "s:phi", "p.u., STATE, Rotor-angle, continuous, double"
      s_psisr_dt = "s:psisr:dt", "p.u./s, d/dt, Stator Flux, Real Part, derivative, continuous, double"
      s_psisi_dt = "s:psisi:dt", "p.u./s, d/dt, Stator Flux, Imaginary Part, derivative continuous, double"
      s_psie_dt = "s:psie:dt", "p.u./s, d/dt, Excitation-Flux, derivative, continuous, double"
      s_psiD_dt = "s:psiD:dt", "p.u./s, d/dt, Flux in D-winding, derivative, continuous, double"
      s_psix_dt = "s:psix:dt", "p.u./s, d/dt, Flux in x-winding, derivative, continuous, double"
      s_psiQ_dt = "s:psiQ:dt", "p.u./s, d/dt, Flux in Q-winding, derivative, continuous, double"
      s_speed_dt = "s:speed:dt", "p.u./s, d/dt, Speed, derivative, continuous, double"
      s_phi_dt = "s:phi:dt", "p.u./s, d/dt, Rotor-angle, derivative continuous, double"
      c_sgnn = "c:sgnn", "MVA, Total rated apparent power (ngnum*sgn)"
      c_c2 = "c:c2", "Voltage Coefficient (uknom/ugn)"
      c_Te = "c:Te", "s, Excitation-Time constant"
      c_TD = "c:TD", "s, Time constant of D-winding"
      c_Tx = "c:Tx", "s, Time Constant of x-winding"
      c_TQ = "c:TQ", "s, Time Constant of Q-winding"
      c_Tag = "c:Tag", "s, Acceleration Time"
      c_xme = "c:xme", "p.u., Electrical Torque"
      c_Yd = "c:Yd", "p.u., Inverted reactance matrix"
      c_Yq = "c:Yq", "p.u., Inverted reactance matrix"
      c_frnom = "c:frnom", "Hz, Nominal frequency"
      c_hpi = "c:hpi", "rad/s, Nominal angular frequency"
      c_loading = "c:loading", "%, Loading"
      c_id = "c:id", "Direct Axis Current"
      c_iq = "c:iq", "Quadrature Axis Current"
      c_ie = "c:ie", "Excitation Current"
      c_addmt = "c:addmt", "Additional Torque"
      c_xmt = "c:xmt", "Mechanical Torque"
      c_psid = "c:psid", "Flux in d-axis"
      c_psiq = "c:psiq", "Flux in q-axis"
      c_pgt = "c:pgt", "p.u., Active power"
      c_fe = "c:fe", "El. frequency"
      c_xdss = "c:xdss", "p.u., Subtransient reactance (d-axis)"
      c_xqss = "c:xqss", "p.u., Subtransient reactance (q-axis)"
      c_Ptspin = "c:Ptspin", "MW, Total Turbine Power of Area"
      c_Pnomspin = "c:Pnomspin", "MW, Total Nominal Power of Spinning Machines in Area"
      c_Presspin = "c:Presspin", "MW, Spinning Reserve of Area"
      c_Tagav = "c:Tagav", "s, System Inertia of Area"
      c_p_mag = "c:p_mag", "Participation, Magnitude"
      c_p_phi = "c:p_phi", "deg, Participation, Angle"
      c_p_mags = "c:p_mags", "Participation, Magnitude (signed)"
      c_lEVec_mag = "c:lEVec_mag", "Controllability, Magnitude"
      c_lEVec_phi = "c:lEVec_phi", "deg, Controllability, Angle"
      c_lEVec_mags = "c:lEVec_mags", "Controllability, Magnitude (signed)"
      c_rEVec_mag = "c:rEVec_mag", "Observability, Magnitude"
      c_rEVec_phi = "c:rEVec_phi", "deg, Observability, Angle"
      c_rEVec_mags = "c:rEVec_mags", "Observability, Magnitude (signed)"
      c_droop = "c:droop", "System droop"
      c_dp = "c:dp", "p.u., Output of internal Speed Controller"
      c_psidss = "c:psidss", "p.u., Subtransient Flux (d-axis)"
      c_psiqss = "c:psiqss", "p.u., Subtransient Flux (q-axis)"
      e_iAstabint = "e:iAstabint", "A-stable integration algorithm"
      e_tag = "e:tag", "s, Acceleration Time Constant"
      e_K = "e:K", "MW/Hz, Secondary Frequency Bias"
      e_cused = "e:cused", "Use for calculation"
      e_snss = "e:snss", "MVA, Short-Circuit Power Sk''max"
      e_ikss = "e:ikss", "kA, Short-Circuit Current Ik''max"
      e_cmax = "e:cmax", "c-Factor (max.)"
      e_rntxn = "e:rntxn", "R/X Ratio (max.)"
      e_xntrn = "e:xntrn", "X/R Ratio (max.)"
      e_x0tx1 = "e:x0tx1", "X0/X1 max."
      e_r0tx0 = "e:r0tx0", "R0/X0 max."
      e_snssmin = "e:snssmin", "MVA, Short-Circuit Power Sk''min"
      e_ikssmin = "e:ikssmin", "kA, Short-Circuit Current Ik''min"
      e_cmin = "e:cmin", "c-Factor (min.)"
      e_rntxnmin = "e:rntxnmin", "R/X Ratio (min.)"
      e_xntrnmin = "e:xntrnmin", "X/R Ratio (min.)"
      e_x0tx1min = "e:x0tx1min", "X0/X1 min."
      e_r0tx0min = "e:r0tx0min", "R0/X0 min."
      e_tds = "e:tds", "s, Td'"
      e_tqs = "e:tqs", "s, Tq'"
      e_tdss = "e:tdss", "s, Td''"
      e_tqss = "e:tqss", "s, Tq''"
      e_cosn = "e:cosn", "Power Factor"
      e_snom = "e:snom", "MVA, Nominal Apparent Power"
      e_xd = "e:xd", "p.u., xd"
      e_xq = "e:xq", "p.u., xq"
      e_xds = "e:xds", "p.u., xd'"
      e_xqs = "e:xqs", "p.u., xq'"

    class ElmZone(Enum):
      _init_ = 'value __doc__'

      c_Pgen = "c:Pgen", "MW, Generators, Active Power"
      c_Qgen = "c:Qgen", "Mvar, Generators, Reactive Power"
      c_Sgen = "c:Sgen", "MVA, Generators, Apparent Power"
      c_cosgen = "c:cosgen", "Generators, Power Factor"
      c_Pmot = "c:Pmot", "MW, Motor Loads, Active Power"
      c_Qmot = "c:Qmot", "Mvar, Motor Loads, Reactive Power"
      c_Smot = "c:Smot", "MVA, Motor Loads, Apparent Power"
      c_cosmot = "c:cosmot", "Motor Loads, Power Factor"
      c_Pload = "c:Pload", "MW, Loads, Active Power"
      c_Qload = "c:Qload", "Mvar, Loads, Reactive Power"
      c_Sload = "c:Sload", "MVA, Loads, Apparent Power"
      c_cosload = "c:cosload", "Loads, Power Factor"
      c_Pcomp = "c:Pcomp", "MW, Compensation (Losses)"
      c_Qcomp = "c:Qcomp", "Mvar, Compensation, C"
      c_Qcompr = "c:Qcompr", "Mvar, Compensation, L"
      c_Pnet = "c:Pnet", "MW, External Networks, Active Power"
      c_Qnet = "c:Qnet", "Mvar, External Networks, Reactive Power"
      c_Snet = "c:Snet", "MVA, External Networks, Apparent Power"
      c_cosnet = "c:cosnet", "External Networks, Power Factor"
      c_Pinter = "c:Pinter", "MW, Interchange Flow, Active Power"
      c_Qinter = "c:Qinter", "Mvar, Interchange Flow, Reactive Power"
      c_Ploss = "c:Ploss", "MW, Losses, Active Power"
      c_Qloss = "c:Qloss", "Mvar, Losses, Reactive Power"
      c_Pt = "c:Pt", "MW, Turbine Power"
      c_Pres = "c:Pres", "MW, Available-Reserve"

    class ElmZpu(Enum):
      _init_ = 'value __doc__'

      m_u1r_bus1 = "m:u1r:bus1", "p.u., Positive-Sequence-Voltage, Real Part"
      m_u1r_bus2 = "m:u1r:bus2", "p.u., Positive-Sequence-Voltage, Real Part"
      m_u1i_bus1 = "m:u1i:bus1", "p.u., Positive-Sequence-Voltage, Imaginary Part"
      m_u1i_bus2 = "m:u1i:bus2", "p.u., Positive-Sequence-Voltage, Imaginary Part"
      m_u1_bus1 = "m:u1:bus1", "p.u., Positive-Sequence-Voltage, Magnitude"
      m_u1_bus2 = "m:u1:bus2", "p.u., Positive-Sequence-Voltage, Magnitude"
      m_U1_bus1 = "m:U1:bus1", "kV, Line-Ground Positive-Sequence-Voltage, Magnitude"
      m_U1_bus2 = "m:U1:bus2", "kV, Line-Ground Positive-Sequence-Voltage, Magnitude"
      m_U1l_bus1 = "m:U1l:bus1", "kV, Line-Line Positive-Sequence-Voltage, Magnitude"
      m_U1l_bus2 = "m:U1l:bus2", "kV, Line-Line Positive-Sequence-Voltage, Magnitude"
      m_phiu1_bus1 = "m:phiu1:bus1", "deg, Positive-Sequence-Voltage, Angle"
      m_phiu1_bus2 = "m:phiu1:bus2", "deg, Positive-Sequence-Voltage, Angle"
      m_inet_bus1 = "m:inet:bus1", "p.u., Current, Magnitude, referred to network"
      m_inet_bus2 = "m:inet:bus2", "p.u., Current, Magnitude, referred to network"
      m_I_bus1 = "m:I:bus1", "kA, Current, Magnitude"
      m_I_bus2 = "m:I:bus2", "kA, Current, Magnitude"
      m_phii_bus1 = "m:phii:bus1", "deg, Current, Angle"
      m_phii_bus2 = "m:phii:bus2", "deg, Current, Angle"
      m_i1r_bus1 = "m:i1r:bus1", "p.u., Positive-Sequence Current, Real Part"
      m_i1r_bus2 = "m:i1r:bus2", "p.u., Positive-Sequence Current, Real Part"
      m_i1i_bus1 = "m:i1i:bus1", "p.u., Positive-Sequence Current, Imaginary Part"
      m_i1i_bus2 = "m:i1i:bus2", "p.u., Positive-Sequence Current, Imaginary Part"
      m_i1_bus1 = "m:i1:bus1", "p.u., Positive-Sequence Current, Magnitude"
      m_i1_bus2 = "m:i1:bus2", "p.u., Positive-Sequence Current, Magnitude"
      m_I1_bus1 = "m:I1:bus1", "kA, Positive-Sequence Current, Magnitude"
      m_I1_bus2 = "m:I1:bus2", "kA, Positive-Sequence Current, Magnitude"
      m_phii1_bus1 = "m:phii1:bus1", "deg, Positive-Sequence Current, Angle"
      m_phii1_bus2 = "m:phii1:bus2", "deg, Positive-Sequence Current, Angle"
      m_P_bus1 = "m:P:bus1", "MW, Active Power"
      m_P_bus2 = "m:P:bus2", "MW, Active Power"
      m_Q_bus1 = "m:Q:bus1", "Mvar, Reactive Power"
      m_Q_bus2 = "m:Q:bus2", "Mvar, Reactive Power"
      m_S_bus1 = "m:S:bus1", "MVA, Apparent Power"
      m_S_bus2 = "m:S:bus2", "MVA, Apparent Power"
      m_cosphi_bus1 = "m:cosphi:bus1", "Power Factor"
      m_cosphi_bus2 = "m:cosphi:bus2", "Power Factor"
      m_tanphi_bus1 = "m:tanphi:bus1", "tan(phi)"
      m_tanphi_bus2 = "m:tanphi:bus2", "tan(phi)"
      m_Psum_bus1 = "m:Psum:bus1", "MW, Total Active Power"
      m_Psum_bus2 = "m:Psum:bus2", "MW, Total Active Power"
      m_Qsum_bus1 = "m:Qsum:bus1", "Mvar, Total Reactive Power"
      m_Qsum_bus2 = "m:Qsum:bus2", "Mvar, Total Reactive Power"
      m_Ssum_bus1 = "m:Ssum:bus1", "MVA, Total Apparent Power"
      m_Ssum_bus2 = "m:Ssum:bus2", "MVA, Total Apparent Power"
      m_cosphisum_bus1 = "m:cosphisum:bus1", "Total Power Factor"
      m_cosphisum_bus2 = "m:cosphisum:bus2", "Total Power Factor"
      m_tanphisum_bus1 = "m:tanphisum:bus1", "Total tan(phi)"
      m_tanphisum_bus2 = "m:tanphisum:bus2", "Total tan(phi)"
      m_Spu_bus1 = "m:Spu:bus1", "MVA/p.u., Apparent Power per p.u. Voltage"
      m_Spu_bus2 = "m:Spu:bus2", "MVA/p.u., Apparent Power per p.u. Voltage"
      m_i1P_bus1 = "m:i1P:bus1", "p.u., Positive-Sequence Active Current"
      m_i1P_bus2 = "m:i1P:bus2", "p.u., Positive-Sequence Active Current"
      m_i1Q_bus1 = "m:i1Q:bus1", "p.u., Positive-Sequence Reactive Current"
      m_i1Q_bus2 = "m:i1Q:bus2", "p.u., Positive-Sequence Reactive Current"
      m_I1P_bus1 = "m:I1P:bus1", "kA, Positive-Sequence Active Current"
      m_I1P_bus2 = "m:I1P:bus2", "kA, Positive-Sequence Active Current"
      m_I1Q_bus1 = "m:I1Q:bus1", "kA, Positive-Sequence Reactive Current"
      m_I1Q_bus2 = "m:I1Q:bus2", "kA, Positive-Sequence Reactive Current"
      m_ur_bus1 = "m:ur:bus1", "p.u., Voltage, Real Part"
      m_ur_bus2 = "m:ur:bus2", "p.u., Voltage, Real Part"
      m_ui_bus1 = "m:ui:bus1", "p.u., Voltage, Imaginary Part"
      m_ui_bus2 = "m:ui:bus2", "p.u., Voltage, Imaginary Part"
      m_u_bus1 = "m:u:bus1", "p.u., Voltage, Magnitude"
      m_u_bus2 = "m:u:bus2", "p.u., Voltage, Magnitude"
      m_ir_bus1 = "m:ir:bus1", "p.u., Current, Real Part"
      m_ir_bus2 = "m:ir:bus2", "p.u., Current, Real Part"
      m_ii_bus1 = "m:ii:bus1", "p.u., Current, Imaginary Part"
      m_ii_bus2 = "m:ii:bus2", "p.u., Current, Imaginary Part"
      m_i_bus1 = "m:i:bus1", "p.u., Current, Magnitude"
      m_i_bus2 = "m:i:bus2", "p.u., Current, Magnitude"
      m_phiui_bus1 = "m:phiui:bus1", "deg, Angle between Voltage and Current"
      m_phiui_bus2 = "m:phiui:bus2", "deg, Angle between Voltage and Current"
      m_phiu1i1_bus1 = "m:phiu1i1:bus1", "deg, Angle between Voltage and Current in positive sequence system"
      m_phiu1i1_bus2 = "m:phiu1i1:bus2", "deg, Angle between Voltage and Current in positive sequence system"
      m_Tfct_bus1 = "m:Tfct:bus1", "s, Fault Clearing Time"
      m_Tfct_bus2 = "m:Tfct:bus2", "s, Fault Clearing Time"
      m_Brkload_bus1 = "m:Brkload:bus1", "%, Breaker Loading"
      m_Brkload_bus2 = "m:Brkload:bus2", "%, Breaker Loading"
      n_ur_bus1 = "n:ur:bus1", "p.u., Voltage, Real Part"
      n_ur_bus2 = "n:ur:bus2", "p.u., Voltage, Real Part"
      n_ui_bus1 = "n:ui:bus1", "p.u., Voltage, Imaginary Part"
      n_ui_bus2 = "n:ui:bus2", "p.u., Voltage, Imaginary Part"
      n_u_bus1 = "n:u:bus1", "p.u., Voltage, Magnitude"
      n_u_bus2 = "n:u:bus2", "p.u., Voltage, Magnitude"
      n_upc_bus1 = "n:upc:bus1", "%, Voltage, Magnitude"
      n_upc_bus2 = "n:upc:bus2", "%, Voltage, Magnitude"
      n_u1_bus1 = "n:u1:bus1", "p.u., Voltage, Magnitude"
      n_u1_bus2 = "n:u1:bus2", "p.u., Voltage, Magnitude"
      n_u1pc_bus1 = "n:u1pc:bus1", "%, Voltage, Magnitude"
      n_u1pc_bus2 = "n:u1pc:bus2", "%, Voltage, Magnitude"
      n_u1r_bus1 = "n:u1r:bus1", "p.u., Positive-Sequence Voltage, Real Part"
      n_u1r_bus2 = "n:u1r:bus2", "p.u., Positive-Sequence Voltage, Real Part"
      n_u1i_bus1 = "n:u1i:bus1", "p.u., Positive-Sequence Voltage, Imaginary Part"
      n_u1i_bus2 = "n:u1i:bus2", "p.u., Positive-Sequence Voltage, Imaginary Part"
      n_U_bus1 = "n:U:bus1", "kV, Line-Ground Voltage, Magnitude"
      n_U_bus2 = "n:U:bus2", "kV, Line-Ground Voltage, Magnitude"
      n_Ul_bus1 = "n:Ul:bus1", "kV, Line-Line Voltage, Magnitude"
      n_Ul_bus2 = "n:Ul:bus2", "kV, Line-Line Voltage, Magnitude"
      n_phiu_bus1 = "n:phiu:bus1", "deg, Voltage, Angle"
      n_phiu_bus2 = "n:phiu:bus2", "deg, Voltage, Angle"
      n_phiurel_bus1 = "n:phiurel:bus1", "deg, Voltage, Relative Angle"
      n_phiurel_bus2 = "n:phiurel:bus2", "deg, Voltage, Relative Angle"
      n_du_bus1 = "n:du:bus1", "%, Voltage Deviation"
      n_du_bus2 = "n:du:bus2", "%, Voltage Deviation"
      n_ishcr_bus1 = "n:ishcr:bus1", "p.u., Short-Circuit Current, real"
      n_ishcr_bus2 = "n:ishcr:bus2", "p.u., Short-Circuit Current, real"
      n_ishci_bus1 = "n:ishci:bus1", "p.u., Short-Circuit Current, imag"
      n_ishci_bus2 = "n:ishci:bus2", "p.u., Short-Circuit Current, imag"
      n_ishc_bus1 = "n:ishc:bus1", "p.u., Short-Circuit Current, magnitude"
      n_ishc_bus2 = "n:ishc:bus2", "p.u., Short-Circuit Current, magnitude"
      n_Ishc_bus1 = "n:Ishc:bus1", "kA, Short-Circuit Current, magnitude"
      n_Ishc_bus2 = "n:Ishc:bus2", "kA, Short-Circuit Current, magnitude"
      n_fe_bus1 = "n:fe:bus1", "p.u., Electrical Frequency"
      n_fe_bus2 = "n:fe:bus2", "p.u., Electrical Frequency"
      n_frdev_bus1 = "n:frdev:bus1", "Hz, average frequency"
      n_frdev_bus2 = "n:frdev:bus2", "Hz, average frequency"
      n_dfehz_bus1 = "n:dfehz:bus1", "Hz, Deviation of the El. Frequency"
      n_dfehz_bus2 = "n:dfehz:bus2", "Hz, Deviation of the El. Frequency"
      n_dfedt_bus1 = "n:dfedt:bus1", "p.u./s, Derivative of El. Frequency"
      n_dfedt_bus2 = "n:dfedt:bus2", "p.u./s, Derivative of El. Frequency"
      n_fehz_bus1 = "n:fehz:bus1", "Hz, Electrical Frequency"
      n_fehz_bus2 = "n:fehz:bus2", "Hz, Electrical Frequency"
      n_frnom_bus1 = "n:frnom:bus1", "Hz, Nominal Frequency"
      n_frnom_bus2 = "n:frnom:bus2", "Hz, Nominal Frequency"
      c_z1_ij_r = "c:z1_ij:r", "p.u., Impedance i-j, Real Part"
      c_z1_ij_i = "c:z1_ij:i", "p.u., Impedance i-j, Imaginary Part"
      c_z1_ij_phi = "c:z1_ij:phi", "deg, Impedance i-j, Angle"
      c_z1_ij_mag = "c:z1_ij:mag", "p.u., Impedance i-j, Magnitude"
      c_z1_ij_phirad = "c:z1_ij:phirad", "rad, Impedance i-j, Angle"
      c_z1_ji_r = "c:z1_ji:r", "p.u., Impedance j-i, Real Part"
      c_z1_ji_i = "c:z1_ji:i", "p.u., Impedance j-i, Imaginary Part"
      c_z1_ji_phi = "c:z1_ji:phi", "deg, Impedance j-i, Angle"
      c_z1_ji_mag = "c:z1_ji:mag", "p.u., Impedance j-i, Magnitude"
      c_z1_ji_phirad = "c:z1_ji:phirad", "rad, Impedance j-i, Angle"
      c_y1_ij_r = "c:y1_ij:r", "p.u., Admittance i-j, Real Part"
      c_y1_ij_i = "c:y1_ij:i", "p.u., Admittance i-j, Imaginary Part"
      c_y1_ij_phi = "c:y1_ij:phi", "deg, Admittance i-j, Angle"
      c_y1_ij_mag = "c:y1_ij:mag", "p.u., Admittance i-j, Magnitude"
      c_y1_ij_phirad = "c:y1_ij:phirad", "rad, Admittance i-j, Angle"
      c_y1_ji_r = "c:y1_ji:r", "p.u., Admittance j-i, Real Part"
      c_y1_ji_i = "c:y1_ji:i", "p.u., Admittance j-i, Imaginary Part"
      c_y1_ji_phi = "c:y1_ji:phi", "deg, Admittance j-i, Angle"
      c_y1_ji_mag = "c:y1_ji:mag", "p.u., Admittance j-i, Magnitude"
      c_y1_ji_phirad = "c:y1_ji:phirad", "rad, Admittance j-i, Angle"
      e_iequalz = "e:iequalz", "Use equal Impedances (zij = zji)"
      e_r_pu = "e:r_pu", "p.u., Real Part"
      e_x_pu = "e:x_pu", "p.u., Imaginary Part"
      e_r_pu_ji = "e:r_pu_ji", "p.u., Real Part"
      e_x_pu_ji = "e:x_pu_ji", "p.u., Imaginary Part"
      e_r0_pu = "e:r0_pu", "p.u., Real Part"
      e_x0_pu = "e:x0_pu", "p.u., Imaginary Part"
      e_r0_pu_ji = "e:r0_pu_ji", "p.u., Real Part"
      e_x0_pu_ji = "e:x0_pu_ji", "p.u., Imaginary Part"
      e_iz2eqz1 = "e:iz2eqz1", "Use Impedance z2 = z1"
      e_r2_pu = "e:r2_pu", "p.u., Real Part"
      e_x2_pu = "e:x2_pu", "p.u., Imaginary Part"
      e_r2_pu_ji = "e:r2_pu_ji", "p.u., Real Part"
      e_x2_pu_ji = "e:x2_pu_ji", "p.u., Imaginary Part"
      e_gi_pu = "e:gi_pu", "p.u., Conductance i"
      e_bi_pu = "e:bi_pu", "p.u., Susceptance i"
      e_gj_pu = "e:gj_pu", "p.u., Conductance j"
      e_bj_pu = "e:bj_pu", "p.u., Susceptance j"
      e_gi0_pu = "e:gi0_pu", "p.u., Conductance i"
      e_bi0_pu = "e:bi0_pu", "p.u., Susceptance i"
      e_gj0_pu = "e:gj0_pu", "p.u., Conductance j"
      e_bj0_pu = "e:bj0_pu", "p.u., Susceptance j"
      e_iy2eqy1 = "e:iy2eqy1", "Use Admittance y2 = y1"
      e_gi2_pu = "e:gi2_pu", "p.u., Conductance i"
      e_bi2_pu = "e:bi2_pu", "p.u., Susceptance i"
      e_gj2_pu = "e:gj2_pu", "p.u., Conductance j"
      e_bj2_pu = "e:bj2_pu", "p.u., Susceptance j"


  class RMS_Unbal:

    class ElmArea(Enum):
      _init_ = 'value __doc__'

      c_Pgen = "c:Pgen", "MW, Generators, Active Power"
      c_Qgen = "c:Qgen", "Mvar, Generators, Reactive Power"
      c_Sgen = "c:Sgen", "MVA, Generators, Apparent Power"
      c_cosgen = "c:cosgen", "Generators, Power Factor"
      c_Pmot = "c:Pmot", "MW, Motor Loads, Active Power"
      c_Qmot = "c:Qmot", "Mvar, Motor Loads, Reactive Power"
      c_Smot = "c:Smot", "MVA, Motor Loads, Apparent Power"
      c_cosmot = "c:cosmot", "Motor Loads, Power Factor"
      c_Pload = "c:Pload", "MW, Loads, Active Power"
      c_Qload = "c:Qload", "Mvar, Loads, Reactive Power"
      c_Sload = "c:Sload", "MVA, Loads, Apparent Power"
      c_cosload = "c:cosload", "Loads, Power Factor"
      c_Pcomp = "c:Pcomp", "MW, Compensation (Losses)"
      c_Qcomp = "c:Qcomp", "Mvar, Compensation, C"
      c_Qcompr = "c:Qcompr", "Mvar, Compensation, L"
      c_Pnet = "c:Pnet", "MW, External Networks, Active Power"
      c_Qnet = "c:Qnet", "Mvar, External Networks, Reactive Power"
      c_Snet = "c:Snet", "MVA, External Networks, Apparent Power"
      c_cosnet = "c:cosnet", "External Networks, Power Factor"
      c_Pinter = "c:Pinter", "MW, Interchange Flow, Active Power"
      c_Qinter = "c:Qinter", "Mvar, Interchange Flow, Reactive Power"
      c_Ploss = "c:Ploss", "MW, Losses, Active Power"
      c_Qloss = "c:Qloss", "Mvar, Losses, Reactive Power"
      c_Pt = "c:Pt", "MW, Turbine Power"
      c_Pres = "c:Pres", "MW, Available Reserve"

    class ElmAsm(Enum):
      _init_ = 'value __doc__'

      m_u0_bus1 = "m:u0:bus1", "p.u., Zero-Sequence-Voltage, Magnitude"
      m_u0_bus1n = "m:u0:bus1n", "p.u., Zero-Sequence-Voltage, Magnitude"
      m_phiu0_bus1 = "m:phiu0:bus1", "deg, Zero-Sequence-Voltage, Angle"
      m_phiu0_bus1n = "m:phiu0:bus1n", "deg, Zero-Sequence-Voltage, Angle"
      m_u1r_bus1 = "m:u1r:bus1", "p.u., Positive-Sequence-Voltage, Real Part"
      m_u1r_bus1n = "m:u1r:bus1n", "p.u., Positive-Sequence-Voltage, Real Part"
      m_u1i_bus1 = "m:u1i:bus1", "p.u., Positive-Sequence-Voltage, Imaginary Part"
      m_u1i_bus1n = "m:u1i:bus1n", "p.u., Positive-Sequence-Voltage, Imaginary Part"
      m_u1_bus1 = "m:u1:bus1", "p.u., Positive-Sequence-Voltage, Magnitude"
      m_u1_bus1n = "m:u1:bus1n", "p.u., Positive-Sequence-Voltage, Magnitude"
      m_U1_bus1 = "m:U1:bus1", "kV, Line-Ground Positive-Sequence-Voltage, Magnitude"
      m_U1_bus1n = "m:U1:bus1n", "kV, Line-Ground Positive-Sequence-Voltage, Magnitude"
      m_U1l_bus1 = "m:U1l:bus1", "kV, Line-Line Positive-Sequence-Voltage, Magnitude"
      m_U1l_bus1n = "m:U1l:bus1n", "kV, Line-Line Positive-Sequence-Voltage, Magnitude"
      m_phiu1_bus1 = "m:phiu1:bus1", "deg, Positive-Sequence-Voltage, Angle"
      m_phiu1_bus1n = "m:phiu1:bus1n", "deg, Positive-Sequence-Voltage, Angle"
      m_u2_bus1 = "m:u2:bus1", "p.u., Negative-Sequence-Voltage, Magnitude"
      m_u2_bus1n = "m:u2:bus1n", "p.u., Negative-Sequence-Voltage, Magnitude"
      m_phiu2_bus1 = "m:phiu2:bus1", "deg, Negative-Sequence-Voltage, Angle"
      m_phiu2_bus1n = "m:phiu2:bus1n", "deg, Negative-Sequence-Voltage, Angle"
      m_inet_bus1_A = "m:inet:bus1:A", "p.u., Phase Current, Magnitude"
      m_inet_bus1_B = "m:inet:bus1:B", "p.u., Phase Current, Magnitude"
      m_inet_bus1_C = "m:inet:bus1:C", "p.u., Phase Current, Magnitude"
      m_inet_bus1_N = "m:inet:bus1:N", "p.u., Phase Current, Magnitude"
      m_inet_bus1n = "m:inet:bus1n", "p.u., Phase Current, Magnitude"
      m_I_bus1_A = "m:I:bus1:A", "kA, Phase Current, Magnitude"
      m_I_bus1_B = "m:I:bus1:B", "kA, Phase Current, Magnitude"
      m_I_bus1_C = "m:I:bus1:C", "kA, Phase Current, Magnitude"
      m_I_bus1_N = "m:I:bus1:N", "kA, Phase Current, Magnitude"
      m_I_bus1n = "m:I:bus1n", "kA, Phase Current, Magnitude"
      m_phii_bus1_A = "m:phii:bus1:A", "deg, Phase Current, Angle"
      m_phii_bus1_B = "m:phii:bus1:B", "deg, Phase Current, Angle"
      m_phii_bus1_C = "m:phii:bus1:C", "deg, Phase Current, Angle"
      m_phii_bus1_N = "m:phii:bus1:N", "deg, Phase Current, Angle"
      m_phii_bus1n = "m:phii:bus1n", "deg, Phase Current, Angle"
      m_i0r_bus1 = "m:i0r:bus1", "p.u., Zero-Sequence Current, Real Part"
      m_i0r_bus1n = "m:i0r:bus1n", "p.u., Zero-Sequence Current, Real Part"
      m_i0i_bus1 = "m:i0i:bus1", "p.u., Zero-Sequence Current, Imaginary Part"
      m_i0i_bus1n = "m:i0i:bus1n", "p.u., Zero-Sequence Current, Imaginary Part"
      m_i0_bus1 = "m:i0:bus1", "p.u., Zero-Sequence Current, Magnitude"
      m_i0_bus1n = "m:i0:bus1n", "p.u., Zero-Sequence Current, Magnitude"
      m_I0_bus1 = "m:I0:bus1", "kA, Zero-Sequence Current, Magnitude"
      m_I0_bus1n = "m:I0:bus1n", "kA, Zero-Sequence Current, Magnitude"
      m_I0x3_bus1 = "m:I0x3:bus1", "kA, 3*I0"
      m_I0x3_bus1n = "m:I0x3:bus1n", "kA, 3*I0"
      m_phii0_bus1 = "m:phii0:bus1", "deg, Zero-Sequence Current, Angle"
      m_phii0_bus1n = "m:phii0:bus1n", "deg, Zero-Sequence Current, Angle"
      m_i1r_bus1 = "m:i1r:bus1", "p.u., Positive-Sequence Current, Real Part"
      m_i1r_bus1n = "m:i1r:bus1n", "p.u., Positive-Sequence Current, Real Part"
      m_i1i_bus1 = "m:i1i:bus1", "p.u., Positive-Sequence Current, Imaginary Part"
      m_i1i_bus1n = "m:i1i:bus1n", "p.u., Positive-Sequence Current, Imaginary Part"
      m_i1_bus1 = "m:i1:bus1", "p.u., Positive-Sequence Current, Magnitude"
      m_i1_bus1n = "m:i1:bus1n", "p.u., Positive-Sequence Current, Magnitude"
      m_I1_bus1 = "m:I1:bus1", "kA, Positive-Sequence Current, Magnitude"
      m_I1_bus1n = "m:I1:bus1n", "kA, Positive-Sequence Current, Magnitude"
      m_phii1_bus1 = "m:phii1:bus1", "deg, Positive-Sequence Current, Angle"
      m_phii1_bus1n = "m:phii1:bus1n", "deg, Positive-Sequence Current, Angle"
      m_i2r_bus1 = "m:i2r:bus1", "p.u., Negative-Sequence Current, Real Part"
      m_i2r_bus1n = "m:i2r:bus1n", "p.u., Negative-Sequence Current, Real Part"
      m_i2i_bus1 = "m:i2i:bus1", "p.u., Negative-Sequence Current, Imaginary Part"
      m_i2i_bus1n = "m:i2i:bus1n", "p.u., Negative-Sequence Current, Imaginary Part"
      m_i2_bus1 = "m:i2:bus1", "p.u., Negative-Sequence Current, Magnitude"
      m_i2_bus1n = "m:i2:bus1n", "p.u., Negative-Sequence Current, Magnitude"
      m_I2_bus1 = "m:I2:bus1", "kA, Negative-Sequence Current, Magnitude"
      m_I2_bus1n = "m:I2:bus1n", "kA, Negative-Sequence Current, Magnitude"
      m_phii2_bus1 = "m:phii2:bus1", "deg, Negative-Sequence Current, Angle"
      m_phii2_bus1n = "m:phii2:bus1n", "deg, Negative-Sequence Current, Angle"
      m_P_bus1_A = "m:P:bus1:A", "MW, Active Power"
      m_P_bus1_B = "m:P:bus1:B", "MW, Active Power"
      m_P_bus1_C = "m:P:bus1:C", "MW, Active Power"
      m_P_bus1_N = "m:P:bus1:N", "MW, Active Power"
      m_P_bus1n = "m:P:bus1n", "MW, Active Power"
      m_Q_bus1_A = "m:Q:bus1:A", "Mvar, Reactive Power"
      m_Q_bus1_B = "m:Q:bus1:B", "Mvar, Reactive Power"
      m_Q_bus1_C = "m:Q:bus1:C", "Mvar, Reactive Power"
      m_Q_bus1_N = "m:Q:bus1:N", "Mvar, Reactive Power"
      m_Q_bus1n = "m:Q:bus1n", "Mvar, Reactive Power"
      m_S_bus1_A = "m:S:bus1:A", "MVA, Apparent Power"
      m_S_bus1_B = "m:S:bus1:B", "MVA, Apparent Power"
      m_S_bus1_C = "m:S:bus1:C", "MVA, Apparent Power"
      m_S_bus1_N = "m:S:bus1:N", "MVA, Apparent Power"
      m_S_bus1n = "m:S:bus1n", "MVA, Apparent Power"
      m_cosphi_bus1_A = "m:cosphi:bus1:A", "Power Factor"
      m_cosphi_bus1_B = "m:cosphi:bus1:B", "Power Factor"
      m_cosphi_bus1_C = "m:cosphi:bus1:C", "Power Factor"
      m_cosphi_bus1_N = "m:cosphi:bus1:N", "Power Factor"
      m_cosphi_bus1n = "m:cosphi:bus1n", "Power Factor"
      m_tanphi_bus1_A = "m:tanphi:bus1:A", "tan(phi)"
      m_tanphi_bus1_B = "m:tanphi:bus1:B", "tan(phi)"
      m_tanphi_bus1_C = "m:tanphi:bus1:C", "tan(phi)"
      m_tanphi_bus1_N = "m:tanphi:bus1:N", "tan(phi)"
      m_tanphi_bus1n = "m:tanphi:bus1n", "tan(phi)"
      m_Psum_bus1 = "m:Psum:bus1", "MW, Total Active Power"
      m_Psum_bus1n = "m:Psum:bus1n", "MW, Total Active Power"
      m_Qsum_bus1 = "m:Qsum:bus1", "Mvar, Total Reactive Power"
      m_Qsum_bus1n = "m:Qsum:bus1n", "Mvar, Total Reactive Power"
      m_Ssum_bus1 = "m:Ssum:bus1", "MVA, Total Apparent Power"
      m_Ssum_bus1n = "m:Ssum:bus1n", "MVA, Total Apparent Power"
      m_cosphisum_bus1 = "m:cosphisum:bus1", "Total Power Factor"
      m_cosphisum_bus1n = "m:cosphisum:bus1n", "Total Power Factor"
      m_tanphisum_bus1 = "m:tanphisum:bus1", "Total tan(phi)"
      m_tanphisum_bus1n = "m:tanphisum:bus1n", "Total tan(phi)"
      m_Pln_bus1_A = "m:Pln:bus1:A", "MW, Active Power (Line-Neutral)"
      m_Pln_bus1_B = "m:Pln:bus1:B", "MW, Active Power (Line-Neutral)"
      m_Pln_bus1_C = "m:Pln:bus1:C", "MW, Active Power (Line-Neutral)"
      m_Pln_bus1_N = "m:Pln:bus1:N", "MW, Active Power (Line-Neutral)"
      m_Pln_bus1n = "m:Pln:bus1n", "MW, Active Power (Line-Neutral)"
      m_Qln_bus1_A = "m:Qln:bus1:A", "Mvar, Reactive Power (Line-Neutral)"
      m_Qln_bus1_B = "m:Qln:bus1:B", "Mvar, Reactive Power (Line-Neutral)"
      m_Qln_bus1_C = "m:Qln:bus1:C", "Mvar, Reactive Power (Line-Neutral)"
      m_Qln_bus1_N = "m:Qln:bus1:N", "Mvar, Reactive Power (Line-Neutral)"
      m_Qln_bus1n = "m:Qln:bus1n", "Mvar, Reactive Power (Line-Neutral)"
      m_P1_bus1 = "m:P1:bus1", "MW, Positive Sequence Active Power"
      m_P1_bus1n = "m:P1:bus1n", "MW, Positive Sequence Active Power"
      m_Q1_bus1 = "m:Q1:bus1", "Mvar, Positive Sequence Reactive Power"
      m_Q1_bus1n = "m:Q1:bus1n", "Mvar, Positive Sequence Reactive Power"
      m_P2_bus1 = "m:P2:bus1", "MW, Negative Sequence Active Power"
      m_P2_bus1n = "m:P2:bus1n", "MW, Negative Sequence Active Power"
      m_Q2_bus1 = "m:Q2:bus1", "Mvar, Negative Sequence Reactive Power"
      m_Q2_bus1n = "m:Q2:bus1n", "Mvar, Negative Sequence Reactive Power"
      m_P0_bus1 = "m:P0:bus1", "MW, Zero Sequence Active Power"
      m_P0_bus1n = "m:P0:bus1n", "MW, Zero Sequence Active Power"
      m_Q0_bus1 = "m:Q0:bus1", "Mvar, Zero Sequence Reactive Power"
      m_Q0_bus1n = "m:Q0:bus1n", "Mvar, Zero Sequence Reactive Power"
      m_i1P_bus1 = "m:i1P:bus1", "p.u., Positive-Sequence Active Current"
      m_i1P_bus1n = "m:i1P:bus1n", "p.u., Positive-Sequence Active Current"
      m_i1Q_bus1 = "m:i1Q:bus1", "p.u., Positive-Sequence Reactive Current"
      m_i1Q_bus1n = "m:i1Q:bus1n", "p.u., Positive-Sequence Reactive Current"
      m_I1P_bus1 = "m:I1P:bus1", "kA, Positive-Sequence Active Current"
      m_I1P_bus1n = "m:I1P:bus1n", "kA, Positive-Sequence Active Current"
      m_I1Q_bus1 = "m:I1Q:bus1", "kA, Positive-Sequence Reactive Current"
      m_I1Q_bus1n = "m:I1Q:bus1n", "kA, Positive-Sequence Reactive Current"
      m_i2P_bus1 = "m:i2P:bus1", "p.u., Negative-Sequence Active Current"
      m_i2P_bus1n = "m:i2P:bus1n", "p.u., Negative-Sequence Active Current"
      m_i2Q_bus1 = "m:i2Q:bus1", "p.u., Negative-Sequence Reactive Current"
      m_i2Q_bus1n = "m:i2Q:bus1n", "p.u., Negative-Sequence Reactive Current"
      m_I2P_bus1 = "m:I2P:bus1", "kA, Negative-Sequence Active Current"
      m_I2P_bus1n = "m:I2P:bus1n", "kA, Negative-Sequence Active Current"
      m_I2Q_bus1 = "m:I2Q:bus1", "kA, Negative-Sequence Reactive Current"
      m_I2Q_bus1n = "m:I2Q:bus1n", "kA, Negative-Sequence Reactive Current"
      m_ur_bus1_A = "m:ur:bus1:A", "p.u., Phase Voltage, Real Part"
      m_ur_bus1_B = "m:ur:bus1:B", "p.u., Phase Voltage, Real Part"
      m_ur_bus1_C = "m:ur:bus1:C", "p.u., Phase Voltage, Real Part"
      m_ur_bus1_N = "m:ur:bus1:N", "p.u., Phase Voltage, Real Part"
      m_ur_bus1n = "m:ur:bus1n", "p.u., Phase Voltage, Real Part"
      m_ui_bus1_A = "m:ui:bus1:A", "p.u., Phase Voltage, Imaginary Part"
      m_ui_bus1_B = "m:ui:bus1:B", "p.u., Phase Voltage, Imaginary Part"
      m_ui_bus1_C = "m:ui:bus1:C", "p.u., Phase Voltage, Imaginary Part"
      m_ui_bus1_N = "m:ui:bus1:N", "p.u., Phase Voltage, Imaginary Part"
      m_ui_bus1n = "m:ui:bus1n", "p.u., Phase Voltage, Imaginary Part"
      m_u_bus1_A = "m:u:bus1:A", "p.u., Phase Voltage, Magnitude"
      m_u_bus1_B = "m:u:bus1:B", "p.u., Phase Voltage, Magnitude"
      m_u_bus1_C = "m:u:bus1:C", "p.u., Phase Voltage, Magnitude"
      m_u_bus1_N = "m:u:bus1:N", "p.u., Phase Voltage, Magnitude"
      m_u_bus1n = "m:u:bus1n", "p.u., Phase Voltage, Magnitude"
      m_ir_bus1_A = "m:ir:bus1:A", "p.u., Phase Current, Real Part"
      m_ir_bus1_B = "m:ir:bus1:B", "p.u., Phase Current, Real Part"
      m_ir_bus1_C = "m:ir:bus1:C", "p.u., Phase Current, Real Part"
      m_ir_bus1_N = "m:ir:bus1:N", "p.u., Phase Current, Real Part"
      m_ir_bus1n = "m:ir:bus1n", "p.u., Phase Current, Real Part"
      m_ii_bus1_A = "m:ii:bus1:A", "p.u., Phase Current, Imaginary Part"
      m_ii_bus1_B = "m:ii:bus1:B", "p.u., Phase Current, Imaginary Part"
      m_ii_bus1_C = "m:ii:bus1:C", "p.u., Phase Current, Imaginary Part"
      m_ii_bus1_N = "m:ii:bus1:N", "p.u., Phase Current, Imaginary Part"
      m_ii_bus1n = "m:ii:bus1n", "p.u., Phase Current, Imaginary Part"
      m_i_bus1_A = "m:i:bus1:A", "p.u., Phase Current, Magnitude"
      m_i_bus1_B = "m:i:bus1:B", "p.u., Phase Current, Magnitude"
      m_i_bus1_C = "m:i:bus1:C", "p.u., Phase Current, Magnitude"
      m_i_bus1_N = "m:i:bus1:N", "p.u., Phase Current, Magnitude"
      m_i_bus1n = "m:i:bus1n", "p.u., Phase Current, Magnitude"
      m_phiui_bus1_A = "m:phiui:bus1:A", "deg, Angle between Voltage and Current"
      m_phiui_bus1_B = "m:phiui:bus1:B", "deg, Angle between Voltage and Current"
      m_phiui_bus1_C = "m:phiui:bus1:C", "deg, Angle between Voltage and Current"
      m_phiui_bus1_N = "m:phiui:bus1:N", "deg, Angle between Voltage and Current"
      m_phiui_bus1n = "m:phiui:bus1n", "deg, Angle between Voltage and Current"
      m_phiu0i0_bus1 = "m:phiu0i0:bus1", "deg, Angle between Voltage and Current in zero-sequence system"
      m_phiu0i0_bus1n = "m:phiu0i0:bus1n", "deg, Angle between Voltage and Current in zero-sequence system"
      m_phiu1i1_bus1 = "m:phiu1i1:bus1", "deg, Angle between Voltage and Current in positive sequence system"
      m_phiu1i1_bus1n = "m:phiu1i1:bus1n", "deg, Angle between Voltage and Current in positive sequence system"
      m_phiu2i2_bus1 = "m:phiu2i2:bus1", "deg, Angle between Voltage and Current in negative sequence system"
      m_phiu2i2_bus1n = "m:phiu2i2:bus1n", "deg, Angle between Voltage and Current in negative sequence system"
      m_Tfct_bus1 = "m:Tfct:bus1", "s, Fault Clearing Time"
      m_Tfct_bus1n = "m:Tfct:bus1n", "s, Fault Clearing Time"
      m_TfctPh_bus1_A = "m:TfctPh:bus1:A", "s, Fault Clearing Time"
      m_TfctPh_bus1_B = "m:TfctPh:bus1:B", "s, Fault Clearing Time"
      m_TfctPh_bus1_C = "m:TfctPh:bus1:C", "s, Fault Clearing Time"
      m_TfctPh_bus1_N = "m:TfctPh:bus1:N", "s, Fault Clearing Time"
      m_TfctPh_bus1n = "m:TfctPh:bus1n", "s, Fault Clearing Time"
      m_Brkload_bus1 = "m:Brkload:bus1", "%, Max. Breaker Loading"
      m_Brkload_bus1n = "m:Brkload:bus1n", "%, Max. Breaker Loading"
      m_BrkloadPh_bus1_A = "m:BrkloadPh:bus1:A", "%, Breaker Loading"
      m_BrkloadPh_bus1_B = "m:BrkloadPh:bus1:B", "%, Breaker Loading"
      m_BrkloadPh_bus1_C = "m:BrkloadPh:bus1:C", "%, Breaker Loading"
      m_BrkloadPh_bus1_N = "m:BrkloadPh:bus1:N", "%, Breaker Loading"
      m_BrkloadPh_bus1n = "m:BrkloadPh:bus1n", "%, Breaker Loading"
      n_ur_bus1_A = "n:ur:bus1:A", "p.u., Line-Ground Voltage, Real Part"
      n_ur_bus1_B = "n:ur:bus1:B", "p.u., Line-Ground Voltage, Real Part"
      n_ur_bus1_C = "n:ur:bus1:C", "p.u., Line-Ground Voltage, Real Part"
      n_ur_bus1_N = "n:ur:bus1:N", "p.u., Line-Ground Voltage, Real Part"
      n_ur_bus1n = "n:ur:bus1n", "p.u., Line-Ground Voltage, Real Part"
      n_ui_bus1_A = "n:ui:bus1:A", "p.u., Line-Ground Voltage, Imaginary Part"
      n_ui_bus1_B = "n:ui:bus1:B", "p.u., Line-Ground Voltage, Imaginary Part"
      n_ui_bus1_C = "n:ui:bus1:C", "p.u., Line-Ground Voltage, Imaginary Part"
      n_ui_bus1_N = "n:ui:bus1:N", "p.u., Line-Ground Voltage, Imaginary Part"
      n_ui_bus1n = "n:ui:bus1n", "p.u., Line-Ground Voltage, Imaginary Part"
      n_u_bus1_A = "n:u:bus1:A", "p.u., Line-Ground Voltage, Magnitude"
      n_u_bus1_B = "n:u:bus1:B", "p.u., Line-Ground Voltage, Magnitude"
      n_u_bus1_C = "n:u:bus1:C", "p.u., Line-Ground Voltage, Magnitude"
      n_u_bus1_N = "n:u:bus1:N", "p.u., Line-Ground Voltage, Magnitude"
      n_u_bus1n = "n:u:bus1n", "p.u., Line-Ground Voltage, Magnitude"
      n_upc_bus1_A = "n:upc:bus1:A", "%, Line-Ground Voltage, Magnitude"
      n_upc_bus1_B = "n:upc:bus1:B", "%, Line-Ground Voltage, Magnitude"
      n_upc_bus1_C = "n:upc:bus1:C", "%, Line-Ground Voltage, Magnitude"
      n_upc_bus1_N = "n:upc:bus1:N", "%, Line-Ground Voltage, Magnitude"
      n_upc_bus1n = "n:upc:bus1n", "%, Line-Ground Voltage, Magnitude"
      n_U_bus1_A = "n:U:bus1:A", "kV, Line-Ground Voltage, Magnitude"
      n_U_bus1_B = "n:U:bus1:B", "kV, Line-Ground Voltage, Magnitude"
      n_U_bus1_C = "n:U:bus1:C", "kV, Line-Ground Voltage, Magnitude"
      n_U_bus1_N = "n:U:bus1:N", "kV, Line-Ground Voltage, Magnitude"
      n_U_bus1n = "n:U:bus1n", "kV, Line-Ground Voltage, Magnitude"
      n_phiu_bus1_A = "n:phiu:bus1:A", "deg, Line-Ground Voltage, Angle"
      n_phiu_bus1_B = "n:phiu:bus1:B", "deg, Line-Ground Voltage, Angle"
      n_phiu_bus1_C = "n:phiu:bus1:C", "deg, Line-Ground Voltage, Angle"
      n_phiu_bus1_N = "n:phiu:bus1:N", "deg, Line-Ground Voltage, Angle"
      n_phiu_bus1n = "n:phiu:bus1n", "deg, Line-Ground Voltage, Angle"
      n_phiurel_bus1 = "n:phiurel:bus1", "deg, Voltage, Relative Angle"
      n_phiurel_bus1n = "n:phiurel:bus1n", "deg, Voltage, Relative Angle"
      n_ul_bus1_A = "n:ul:bus1:A", "p.u., Line-Line Voltage, Magnitude"
      n_ul_bus1_B = "n:ul:bus1:B", "p.u., Line-Line Voltage, Magnitude"
      n_ul_bus1_C = "n:ul:bus1:C", "p.u., Line-Line Voltage, Magnitude"
      n_ul_bus1_N = "n:ul:bus1:N", "p.u., Line-Line Voltage, Magnitude"
      n_ul_bus1n = "n:ul:bus1n", "p.u., Line-Line Voltage, Magnitude"
      n_ulpc_bus1_A = "n:ulpc:bus1:A", "%, Line-Line Voltage, Magnitude"
      n_ulpc_bus1_B = "n:ulpc:bus1:B", "%, Line-Line Voltage, Magnitude"
      n_ulpc_bus1_C = "n:ulpc:bus1:C", "%, Line-Line Voltage, Magnitude"
      n_ulpc_bus1_N = "n:ulpc:bus1:N", "%, Line-Line Voltage, Magnitude"
      n_ulpc_bus1n = "n:ulpc:bus1n", "%, Line-Line Voltage, Magnitude"
      n_Ul_bus1_A = "n:Ul:bus1:A", "kV, Line-Line Voltage, Magnitude"
      n_Ul_bus1_B = "n:Ul:bus1:B", "kV, Line-Line Voltage, Magnitude"
      n_Ul_bus1_C = "n:Ul:bus1:C", "kV, Line-Line Voltage, Magnitude"
      n_Ul_bus1_N = "n:Ul:bus1:N", "kV, Line-Line Voltage, Magnitude"
      n_Ul_bus1n = "n:Ul:bus1n", "kV, Line-Line Voltage, Magnitude"
      n_phiul_bus1_A = "n:phiul:bus1:A", "deg, Line-Line Voltage, Angle"
      n_phiul_bus1_B = "n:phiul:bus1:B", "deg, Line-Line Voltage, Angle"
      n_phiul_bus1_C = "n:phiul:bus1:C", "deg, Line-Line Voltage, Angle"
      n_phiul_bus1_N = "n:phiul:bus1:N", "deg, Line-Line Voltage, Angle"
      n_phiul_bus1n = "n:phiul:bus1n", "deg, Line-Line Voltage, Angle"
      n_u0_bus1 = "n:u0:bus1", "p.u., Zero-Sequence Voltage, Magnitude"
      n_u0_bus1n = "n:u0:bus1n", "p.u., Zero-Sequence Voltage, Magnitude"
      n_U0_bus1 = "n:U0:bus1", "kV, Zero-Sequence Voltage, Magnitude"
      n_U0_bus1n = "n:U0:bus1n", "kV, Zero-Sequence Voltage, Magnitude"
      n_U0x3_bus1 = "n:U0x3:bus1", "kV, 3*U0"
      n_U0x3_bus1n = "n:U0x3:bus1n", "kV, 3*U0"
      n_phiu0_bus1 = "n:phiu0:bus1", "deg, Zero-Sequence Voltage, Angle"
      n_phiu0_bus1n = "n:phiu0:bus1n", "deg, Zero-Sequence Voltage, Angle"
      n_u1_bus1 = "n:u1:bus1", "p.u., Positive-Sequence Voltage, Magnitude"
      n_u1_bus1n = "n:u1:bus1n", "p.u., Positive-Sequence Voltage, Magnitude"
      n_u1pc_bus1 = "n:u1pc:bus1", "%, Positive-Sequence Voltage, Magnitude"
      n_u1pc_bus1n = "n:u1pc:bus1n", "%, Positive-Sequence Voltage, Magnitude"
      n_u1r_bus1 = "n:u1r:bus1", "p.u., Positive-Sequence Voltage, Real Part"
      n_u1r_bus1n = "n:u1r:bus1n", "p.u., Positive-Sequence Voltage, Real Part"
      n_u1i_bus1 = "n:u1i:bus1", "p.u., Positive-Sequence Voltage, Imaginary Part"
      n_u1i_bus1n = "n:u1i:bus1n", "p.u., Positive-Sequence Voltage, Imaginary Part"
      n_U1_bus1 = "n:U1:bus1", "kV, Line-Ground Positive-Sequence Voltage, Magnitude"
      n_U1_bus1n = "n:U1:bus1n", "kV, Line-Ground Positive-Sequence Voltage, Magnitude"
      n_phiu1_bus1 = "n:phiu1:bus1", "deg, Positive-Sequence Voltage, Angle"
      n_phiu1_bus1n = "n:phiu1:bus1n", "deg, Positive-Sequence Voltage, Angle"
      n_u2_bus1 = "n:u2:bus1", "p.u., Negative-Sequence Voltage, Magnitude"
      n_u2_bus1n = "n:u2:bus1n", "p.u., Negative-Sequence Voltage, Magnitude"
      n_U2_bus1 = "n:U2:bus1", "kV, Line-Ground Negative-Sequence Voltage, Magnitude"
      n_U2_bus1n = "n:U2:bus1n", "kV, Line-Ground Negative-Sequence Voltage, Magnitude"
      n_phiu2_bus1 = "n:phiu2:bus1", "deg, Negative-Sequence Voltage, Angle"
      n_phiu2_bus1n = "n:phiu2:bus1n", "deg, Negative-Sequence Voltage, Angle"
      n_U1l_bus1 = "n:U1l:bus1", "kV, Line-Line Positive-Sequence Voltage, Magnitude"
      n_U1l_bus1n = "n:U1l:bus1n", "kV, Line-Line Positive-Sequence Voltage, Magnitude"
      n_U2l_bus1 = "n:U2l:bus1", "kV, Line-Line Negative-Sequence Voltage, Magnitude"
      n_U2l_bus1n = "n:U2l:bus1n", "kV, Line-Line Negative-Sequence Voltage, Magnitude"
      n_uln_bus1_A = "n:uln:bus1:A", "p.u., Line-Neutral Voltage, Magnitude"
      n_uln_bus1_B = "n:uln:bus1:B", "p.u., Line-Neutral Voltage, Magnitude"
      n_uln_bus1_C = "n:uln:bus1:C", "p.u., Line-Neutral Voltage, Magnitude"
      n_uln_bus1_N = "n:uln:bus1:N", "p.u., Line-Neutral Voltage, Magnitude"
      n_uln_bus1n = "n:uln:bus1n", "p.u., Line-Neutral Voltage, Magnitude"
      n_Uln_bus1_A = "n:Uln:bus1:A", "kV, Line-Neutral Voltage, Magnitude"
      n_Uln_bus1_B = "n:Uln:bus1:B", "kV, Line-Neutral Voltage, Magnitude"
      n_Uln_bus1_C = "n:Uln:bus1:C", "kV, Line-Neutral Voltage, Magnitude"
      n_Uln_bus1_N = "n:Uln:bus1:N", "kV, Line-Neutral Voltage, Magnitude"
      n_Uln_bus1n = "n:Uln:bus1n", "kV, Line-Neutral Voltage, Magnitude"
      n_phiuln_bus1_A = "n:phiuln:bus1:A", "deg, Line-Neutral Voltage, Angle"
      n_phiuln_bus1_B = "n:phiuln:bus1:B", "deg, Line-Neutral Voltage, Angle"
      n_phiuln_bus1_C = "n:phiuln:bus1:C", "deg, Line-Neutral Voltage, Angle"
      n_phiuln_bus1_N = "n:phiuln:bus1:N", "deg, Line-Neutral Voltage, Angle"
      n_phiuln_bus1n = "n:phiuln:bus1n", "deg, Line-Neutral Voltage, Angle"
      n_un_bus1 = "n:un:bus1", "p.u., Neutral-Ground Voltage, Magnitude"
      n_un_bus1n = "n:un:bus1n", "p.u., Neutral-Ground Voltage, Magnitude"
      n_Un_bus1 = "n:Un:bus1", "kV, Neutral-Ground Voltage, Magnitude"
      n_Un_bus1n = "n:Un:bus1n", "kV, Neutral-Ground Voltage, Magnitude"
      n_phiun_bus1 = "n:phiun:bus1", "deg, Neutral-Ground Voltage, Angle"
      n_phiun_bus1n = "n:phiun:bus1n", "deg, Neutral-Ground Voltage, Angle"
      n_upht_bus1_A = "n:upht:bus1:A", "p.u., Phase Technology dependent Voltage, Magnitude"
      n_upht_bus1_B = "n:upht:bus1:B", "p.u., Phase Technology dependent Voltage, Magnitude"
      n_upht_bus1_C = "n:upht:bus1:C", "p.u., Phase Technology dependent Voltage, Magnitude"
      n_upht_bus1_N = "n:upht:bus1:N", "p.u., Phase Technology dependent Voltage, Magnitude"
      n_upht_bus1n = "n:upht:bus1n", "p.u., Phase Technology dependent Voltage, Magnitude"
      n_Upht_bus1_A = "n:Upht:bus1:A", "kV, Phase Technology dependent Voltage, Magnitude"
      n_Upht_bus1_B = "n:Upht:bus1:B", "kV, Phase Technology dependent Voltage, Magnitude"
      n_Upht_bus1_C = "n:Upht:bus1:C", "kV, Phase Technology dependent Voltage, Magnitude"
      n_Upht_bus1_N = "n:Upht:bus1:N", "kV, Phase Technology dependent Voltage, Magnitude"
      n_Upht_bus1n = "n:Upht:bus1n", "kV, Phase Technology dependent Voltage, Magnitude"
      n_um_bus1 = "n:um:bus1", "p.u., Average-Voltage, Magnitude"
      n_um_bus1n = "n:um:bus1n", "p.u., Average-Voltage, Magnitude"
      n_Um_bus1 = "n:Um:bus1", "kV, Average-Voltage, Magnitude"
      n_Um_bus1n = "n:Um:bus1n", "kV, Average-Voltage, Magnitude"
      n_ishcr_bus1_A = "n:ishcr:bus1:A", "p.u., Short-Circuit Current, real"
      n_ishcr_bus1_B = "n:ishcr:bus1:B", "p.u., Short-Circuit Current, real"
      n_ishcr_bus1_C = "n:ishcr:bus1:C", "p.u., Short-Circuit Current, real"
      n_ishcr_bus1_N = "n:ishcr:bus1:N", "p.u., Short-Circuit Current, real"
      n_ishcr_bus1n = "n:ishcr:bus1n", "p.u., Short-Circuit Current, real"
      n_ishci_bus1_A = "n:ishci:bus1:A", "p.u., Short-Circuit Current, imag"
      n_ishci_bus1_B = "n:ishci:bus1:B", "p.u., Short-Circuit Current, imag"
      n_ishci_bus1_C = "n:ishci:bus1:C", "p.u., Short-Circuit Current, imag"
      n_ishci_bus1_N = "n:ishci:bus1:N", "p.u., Short-Circuit Current, imag"
      n_ishci_bus1n = "n:ishci:bus1n", "p.u., Short-Circuit Current, imag"
      n_ishc_bus1_A = "n:ishc:bus1:A", "p.u., Short-Circuit Current, magnitude"
      n_ishc_bus1_B = "n:ishc:bus1:B", "p.u., Short-Circuit Current, magnitude"
      n_ishc_bus1_C = "n:ishc:bus1:C", "p.u., Short-Circuit Current, magnitude"
      n_ishc_bus1_N = "n:ishc:bus1:N", "p.u., Short-Circuit Current, magnitude"
      n_ishc_bus1n = "n:ishc:bus1n", "p.u., Short-Circuit Current, magnitude"
      n_Ishc_bus1_A = "n:Ishc:bus1:A", "kA, Short-Circuit Current, magnitude"
      n_Ishc_bus1_B = "n:Ishc:bus1:B", "kA, Short-Circuit Current, magnitude"
      n_Ishc_bus1_C = "n:Ishc:bus1:C", "kA, Short-Circuit Current, magnitude"
      n_Ishc_bus1_N = "n:Ishc:bus1:N", "kA, Short-Circuit Current, magnitude"
      n_Ishc_bus1n = "n:Ishc:bus1n", "kA, Short-Circuit Current, magnitude"
      n_inshc_bus1 = "n:inshc:bus1", "p.u., Neutral Short-Circuit Current, magnitude"
      n_inshc_bus1n = "n:inshc:bus1n", "p.u., Neutral Short-Circuit Current, magnitude"
      n_Inshc_bus1 = "n:Inshc:bus1", "kA, Neutral Short-Circuit Current, magnitude"
      n_Inshc_bus1n = "n:Inshc:bus1n", "kA, Neutral Short-Circuit Current, magnitude"
      n_fe_bus1 = "n:fe:bus1", "p.u., Electrical Frequency"
      n_fe_bus1n = "n:fe:bus1n", "p.u., Electrical Frequency"
      n_frdev_bus1 = "n:frdev:bus1", "Hz, average frequency"
      n_frdev_bus1n = "n:frdev:bus1n", "Hz, average frequency"
      n_dfehz_bus1 = "n:dfehz:bus1", "Hz, Deviation of the El. Frequency"
      n_dfehz_bus1n = "n:dfehz:bus1n", "Hz, Deviation of the El. Frequency"
      n_dfedt_bus1 = "n:dfedt:bus1", "p.u./s, Derivative of El. Frequency"
      n_dfedt_bus1n = "n:dfedt:bus1n", "p.u./s, Derivative of El. Frequency"
      n_fehz_bus1 = "n:fehz:bus1", "Hz, Electrical Frequency"
      n_fehz_bus1n = "n:fehz:bus1n", "Hz, Electrical Frequency"
      n_frnom_bus1 = "n:frnom:bus1", "Hz, Nominal Frequency"
      n_frnom_bus1n = "n:frnom:bus1n", "Hz, Nominal Frequency"
      s_pt = "s:pt", "p.u., IN, Turbine Power, continuous, double"
      s_xmdm = "s:xmdm", "p.u., IN, MDM-Torque, continuous, double"
      s_rradd = "s:rradd", "p.u., IN, Additional Rotor Resistance, continuous, double"
      s_bcomp = "s:bcomp", "p.u., IN, Shunt for Compensation, continuous, double"
      s_xspeed = "s:xspeed", "p.u., OUT, Speed, continuous, double"
      s_pgt = "s:pgt", "p.u., OUT, Active power, continuous, double"
      s_xme = "s:xme", "p.u., OUT, Electrical Torque, continuous, double"
      s_irot = "s:irot", "p.u., OUT, Rotor Current, continuous, double"
      s_speed = "s:speed", "STATE, Speed, continuous, double"
      s_psiA1_r = "s:psiA1_r", "STATE, Flux, Loop A, Real Part continuous, double"
      s_psiA1_i = "s:psiA1_i", "STATE, Flux, Loop A, Imaginary Part, continuous, double"
      s_psiA2_r = "s:psiA2_r", "STATE, Flux, Loop A, Real Part continuous, double"
      s_psiA2_i = "s:psiA2_i", "STATE, Flux, Loop A, Imaginary Part, continuous, double"
      s_psiB_r = "s:psiB_r", "STATE, Flux, Loop B, Real Part continuous, double"
      s_psiB_i = "s:psiB_i", "STATE, Flux, Loop B, Imaginary Part, continuous, double"
      s_speed_dt = "s:speed:dt", "/s, d/dt, Speed, derivative, continuous, double"
      s_psiA1_r_dt = "s:psiA1_r:dt", "/s, d/dt, Flux, Loop A, Real Part, derivative, continuous, double"
      s_psiA1_i_dt = "s:psiA1_i:dt", "/s, d/dt, Flux, Loop A, Imaginary Part, derivative, continuous, double"
      s_psiA2_r_dt = "s:psiA2_r:dt", "/s, d/dt, Flux, Loop A, Real Part, derivative, continuous, double"
      s_psiA2_i_dt = "s:psiA2_i:dt", "/s, d/dt, Flux, Loop A, Imaginary Part, derivative, continuous, double"
      s_psiB_r_dt = "s:psiB_r:dt", "/s, d/dt, Flux, Loop B, Real Part, derivative, continuous, double"
      s_psiB_i_dt = "s:psiB_i:dt", "/s, d/dt, Flux, Loop B, Imaginary Part, derivative, continuous, double"
      c_cosn = "c:cosn", "Rated Power Factor"
      c_rstr = "c:rstr", "p.u., Stator Resistance"
      c_xstr = "c:xstr", "p.u., Stator Reactance"
      c_rrtrA = "c:rrtrA", "p.u., Rotor Resistance A"
      c_rrtrB = "c:rrtrB", "p.u., Rotor Resistance B"
      c_xrtrA = "c:xrtrA", "p.u., Rotor Reactance A"
      c_xrtrB = "c:xrtrB", "p.u., Rotor Reactance B"
      c_xm = "c:xm", "p.u., Magnetising Reactance"
      c_xmrtr = "c:xmrtr", "p.u., Rotor Leakage Reactance"
      c_frnom = "c:frnom", "Hz, Nominal Frequency"
      c_i_mot = "c:i_mot", "i_mot=0"
      c_xmem = "c:xmem", "p.u., Electrical Torque"
      c_xmt = "c:xmt", "p.u., Mechanical Torque"
      c_slip = "c:slip", "%, Slip"
      c_tag = "c:tag", "s, Acceleration Time Constant"
      c_i_star = "c:i_star", "Star/Delta"
      c_i_auto = "c:i_auto", "Auto Transformer"
      c_xss = "c:xss", "p.u., Subtransient Reactance"
      c_addmt = "c:addmt", "p.u., Additional Torque"
      c_fe = "c:fe", "Electrical Frequency"
      c_z0_r = "c:z0:r", "p.u., Zero Sequence Impedance, Real Part"
      c_z0_i = "c:z0:i", "p.u., Zero Sequence Impedance, Imaginary Part"
      c_z0_phi = "c:z0:phi", "deg, Zero Sequence Impedance, Angle"
      c_z0_mag = "c:z0:mag", "p.u., Zero Sequence Impedance, Magnitude"
      c_z0_phirad = "c:z0:phirad", "rad, Zero Sequence Impedance, Angle"
      c_ze_r = "c:ze:r", "p.u., Grounding Impedance, Real Part"
      c_ze_i = "c:ze:i", "p.u., Grounding Impedance, Imaginary Part"
      c_ze_phi = "c:ze:phi", "deg, Grounding Impedance, Angle"
      c_ze_mag = "c:ze:mag", "p.u., Grounding Impedance, Magnitude"
      c_ze_phirad = "c:ze:phirad", "rad, Grounding Impedance, Angle"
      c_z2_r = "c:z2:r", "p.u., Negative Sequence Impedance, Real Part"
      c_z2_i = "c:z2:i", "p.u., Negative Sequence Impedance, Imaginary Part"
      c_z2_phi = "c:z2:phi", "deg, Negative Sequence Impedance, Angle"
      c_z2_mag = "c:z2:mag", "p.u., Negative Sequence Impedance, Magnitude"
      c_z2_phirad = "c:z2:phirad", "rad, Negative Sequence Impedance, Angle"
      c_xradd = "c:xradd", "p.u., Additional Rotor Reactance"
      c_xstadd = "c:xstadd", "p.u., Additional Stator Reactance"
      c_psiM = "c:psiM", "p.u., Magnetising Flux"
      c_ksat = "c:ksat", "Saturation Factor"
      c_xmsat = "c:xmsat", "p.u., Saturated Magnetising Reactance"
      c_iTrigg = "c:iTrigg", "Triggered by"
      c_Tyd = "c:Tyd", "s, Y - D switching time"
      c_speedyd = "c:speedyd", "s, Y - D switching speed"
      c_xreac = "c:xreac", "p.u., Reactor Reactance"
      c_Tbypass = "c:Tbypass", "p.u., Bypass time"
      c_speedbyp = "c:speedbyp", "p.u., Bypass speed"
      c_autotap = "c:autotap", "p.u., Auto Transformer Tap"
      c_Tstaroff = "c:Tstaroff", "p.u., Star contactor off time"
      c_speedsoff = "c:speedsoff", "p.u., Star contactor off speed"
      e_iAstabint = "e:iAstabint", "A-stable integration algorithm"
      e_c_pmdm = "e:c_pmdm", "Mdm"
      e_mdmlp = "e:mdmlp", "p.u., Proportional factor"
      e_mdmex = "e:mdmex", "Exponent"
      e_Jme = "e:Jme", "kgm^2, Moment of Inertia"
      e_gratio = "e:gratio", "Gear ratio"
      e_cJ = "e:cJ", "kgm^2, Moment of Inertia from Type"
      e_cJtot = "e:cJtot", "kgm^2, Total Moment of Inertia"
      e_ctagtot = "e:ctagtot", "s, Total Acceleration Time Const."
      e_iUseStart = "e:iUseStart", "Use motor starting method"
      e_iStartMethod = "e:iStartMethod", "Motor starting method"
      e_iTrigg = "e:iTrigg", "Triggered by..."
      e_Tyd = "e:Tyd", "s, Switch to 'D' after"
      e_speedyd = "e:speedyd", "p.u., Switch to 'D' at speed >="
      e_ctime = "e:ctime", "s, Time"
      e_crotor = "e:crotor", "p.u., Additional rotor resistance"
      e_cspeed = "e:cspeed", "p.u., Speed"
      e_ratedStr = "e:ratedStr", "kVA, Rated apparent power"
      e_xrea = "e:xrea", "%, Reactance"
      e_Tbypass = "e:Tbypass", "s, Bypass after"
      e_speedbyp = "e:speedbyp", "p.u., Bypass at speed >="
      e_autotap = "e:autotap", "%, Tap"
      e_Tstaroff = "e:Tstaroff", "s, Release star contactor after"
      e_speedsoff = "e:speedsoff", "p.u., Release star contactor at speed >="
      e_speed1ph = "e:speed1ph", "p.u., Switch at speed >="
      e_T1ph = "e:T1ph", "s, Switch after"
      r_c_pmdm = "r:c_pmdm", "Mdm"

    class ElmAsmsc(Enum):
      _init_ = 'value __doc__'

      m_u0_bus1 = "m:u0:bus1", "p.u., Zero-Sequence-Voltage, Magnitude"
      m_u0_bus2 = "m:u0:bus2", "p.u., Zero-Sequence-Voltage, Magnitude"
      m_phiu0_bus1 = "m:phiu0:bus1", "deg, Zero-Sequence-Voltage, Angle"
      m_phiu0_bus2 = "m:phiu0:bus2", "deg, Zero-Sequence-Voltage, Angle"
      m_u1r_bus1 = "m:u1r:bus1", "p.u., Positive-Sequence-Voltage, Real Part"
      m_u1r_bus2 = "m:u1r:bus2", "p.u., Positive-Sequence-Voltage, Real Part"
      m_u1i_bus1 = "m:u1i:bus1", "p.u., Positive-Sequence-Voltage, Imaginary Part"
      m_u1i_bus2 = "m:u1i:bus2", "p.u., Positive-Sequence-Voltage, Imaginary Part"
      m_u1_bus1 = "m:u1:bus1", "p.u., Positive-Sequence-Voltage, Magnitude"
      m_u1_bus2 = "m:u1:bus2", "p.u., Positive-Sequence-Voltage, Magnitude"
      m_U1_bus1 = "m:U1:bus1", "kV, Line-Ground Positive-Sequence-Voltage, Magnitude"
      m_U1_bus2 = "m:U1:bus2", "kV, Line-Ground Positive-Sequence-Voltage, Magnitude"
      m_U1l_bus1 = "m:U1l:bus1", "kV, Line-Line Positive-Sequence-Voltage, Magnitude"
      m_U1l_bus2 = "m:U1l:bus2", "kV, Line-Line Positive-Sequence-Voltage, Magnitude"
      m_phiu1_bus1 = "m:phiu1:bus1", "deg, Positive-Sequence-Voltage, Angle"
      m_phiu1_bus2 = "m:phiu1:bus2", "deg, Positive-Sequence-Voltage, Angle"
      m_u2_bus1 = "m:u2:bus1", "p.u., Negative-Sequence-Voltage, Magnitude"
      m_u2_bus2 = "m:u2:bus2", "p.u., Negative-Sequence-Voltage, Magnitude"
      m_phiu2_bus1 = "m:phiu2:bus1", "deg, Negative-Sequence-Voltage, Angle"
      m_phiu2_bus2 = "m:phiu2:bus2", "deg, Negative-Sequence-Voltage, Angle"
      m_inet_bus1_A = "m:inet:bus1:A", "p.u., Phase Current, Magnitude"
      m_inet_bus1_B = "m:inet:bus1:B", "p.u., Phase Current, Magnitude"
      m_inet_bus1_C = "m:inet:bus1:C", "p.u., Phase Current, Magnitude"
      m_inet_bus2 = "m:inet:bus2", "p.u., Phase Current, Magnitude"
      m_I_bus1_A = "m:I:bus1:A", "kA, Phase Current, Magnitude"
      m_I_bus1_B = "m:I:bus1:B", "kA, Phase Current, Magnitude"
      m_I_bus1_C = "m:I:bus1:C", "kA, Phase Current, Magnitude"
      m_I_bus2 = "m:I:bus2", "kA, Phase Current, Magnitude"
      m_phii_bus1_A = "m:phii:bus1:A", "deg, Phase Current, Angle"
      m_phii_bus1_B = "m:phii:bus1:B", "deg, Phase Current, Angle"
      m_phii_bus1_C = "m:phii:bus1:C", "deg, Phase Current, Angle"
      m_phii_bus2 = "m:phii:bus2", "deg, Phase Current, Angle"
      m_i0r_bus1 = "m:i0r:bus1", "p.u., Zero-Sequence Current, Real Part"
      m_i0r_bus2 = "m:i0r:bus2", "p.u., Zero-Sequence Current, Real Part"
      m_i0i_bus1 = "m:i0i:bus1", "p.u., Zero-Sequence Current, Imaginary Part"
      m_i0i_bus2 = "m:i0i:bus2", "p.u., Zero-Sequence Current, Imaginary Part"
      m_i0_bus1 = "m:i0:bus1", "p.u., Zero-Sequence Current, Magnitude"
      m_i0_bus2 = "m:i0:bus2", "p.u., Zero-Sequence Current, Magnitude"
      m_I0_bus1 = "m:I0:bus1", "kA, Zero-Sequence Current, Magnitude"
      m_I0_bus2 = "m:I0:bus2", "kA, Zero-Sequence Current, Magnitude"
      m_I0x3_bus1 = "m:I0x3:bus1", "kA, 3*I0"
      m_I0x3_bus2 = "m:I0x3:bus2", "kA, 3*I0"
      m_phii0_bus1 = "m:phii0:bus1", "deg, Zero-Sequence Current, Angle"
      m_phii0_bus2 = "m:phii0:bus2", "deg, Zero-Sequence Current, Angle"
      m_i1r_bus1 = "m:i1r:bus1", "p.u., Positive-Sequence Current, Real Part"
      m_i1r_bus2 = "m:i1r:bus2", "p.u., Positive-Sequence Current, Real Part"
      m_i1i_bus1 = "m:i1i:bus1", "p.u., Positive-Sequence Current, Imaginary Part"
      m_i1i_bus2 = "m:i1i:bus2", "p.u., Positive-Sequence Current, Imaginary Part"
      m_i1_bus1 = "m:i1:bus1", "p.u., Positive-Sequence Current, Magnitude"
      m_i1_bus2 = "m:i1:bus2", "p.u., Positive-Sequence Current, Magnitude"
      m_I1_bus1 = "m:I1:bus1", "kA, Positive-Sequence Current, Magnitude"
      m_I1_bus2 = "m:I1:bus2", "kA, Positive-Sequence Current, Magnitude"
      m_phii1_bus1 = "m:phii1:bus1", "deg, Positive-Sequence Current, Angle"
      m_phii1_bus2 = "m:phii1:bus2", "deg, Positive-Sequence Current, Angle"
      m_i2r_bus1 = "m:i2r:bus1", "p.u., Negative-Sequence Current, Real Part"
      m_i2r_bus2 = "m:i2r:bus2", "p.u., Negative-Sequence Current, Real Part"
      m_i2i_bus1 = "m:i2i:bus1", "p.u., Negative-Sequence Current, Imaginary Part"
      m_i2i_bus2 = "m:i2i:bus2", "p.u., Negative-Sequence Current, Imaginary Part"
      m_i2_bus1 = "m:i2:bus1", "p.u., Negative-Sequence Current, Magnitude"
      m_i2_bus2 = "m:i2:bus2", "p.u., Negative-Sequence Current, Magnitude"
      m_I2_bus1 = "m:I2:bus1", "kA, Negative-Sequence Current, Magnitude"
      m_I2_bus2 = "m:I2:bus2", "kA, Negative-Sequence Current, Magnitude"
      m_phii2_bus1 = "m:phii2:bus1", "deg, Negative-Sequence Current, Angle"
      m_phii2_bus2 = "m:phii2:bus2", "deg, Negative-Sequence Current, Angle"
      m_P_bus1_A = "m:P:bus1:A", "MW, Active Power"
      m_P_bus1_B = "m:P:bus1:B", "MW, Active Power"
      m_P_bus1_C = "m:P:bus1:C", "MW, Active Power"
      m_P_bus2 = "m:P:bus2", "MW, Active Power"
      m_Q_bus1_A = "m:Q:bus1:A", "Mvar, Reactive Power"
      m_Q_bus1_B = "m:Q:bus1:B", "Mvar, Reactive Power"
      m_Q_bus1_C = "m:Q:bus1:C", "Mvar, Reactive Power"
      m_Q_bus2 = "m:Q:bus2", "Mvar, Reactive Power"
      m_S_bus1_A = "m:S:bus1:A", "MVA, Apparent Power"
      m_S_bus1_B = "m:S:bus1:B", "MVA, Apparent Power"
      m_S_bus1_C = "m:S:bus1:C", "MVA, Apparent Power"
      m_S_bus2 = "m:S:bus2", "MVA, Apparent Power"
      m_cosphi_bus1_A = "m:cosphi:bus1:A", "Power Factor"
      m_cosphi_bus1_B = "m:cosphi:bus1:B", "Power Factor"
      m_cosphi_bus1_C = "m:cosphi:bus1:C", "Power Factor"
      m_cosphi_bus2 = "m:cosphi:bus2", "Power Factor"
      m_tanphi_bus1_A = "m:tanphi:bus1:A", "tan(phi)"
      m_tanphi_bus1_B = "m:tanphi:bus1:B", "tan(phi)"
      m_tanphi_bus1_C = "m:tanphi:bus1:C", "tan(phi)"
      m_tanphi_bus2 = "m:tanphi:bus2", "tan(phi)"
      m_Psum_bus1 = "m:Psum:bus1", "MW, Total Active Power"
      m_Psum_bus2 = "m:Psum:bus2", "MW, Total Active Power"
      m_Qsum_bus1 = "m:Qsum:bus1", "Mvar, Total Reactive Power"
      m_Qsum_bus2 = "m:Qsum:bus2", "Mvar, Total Reactive Power"
      m_Ssum_bus1 = "m:Ssum:bus1", "MVA, Total Apparent Power"
      m_Ssum_bus2 = "m:Ssum:bus2", "MVA, Total Apparent Power"
      m_cosphisum_bus1 = "m:cosphisum:bus1", "Total Power Factor"
      m_cosphisum_bus2 = "m:cosphisum:bus2", "Total Power Factor"
      m_tanphisum_bus1 = "m:tanphisum:bus1", "Total tan(phi)"
      m_tanphisum_bus2 = "m:tanphisum:bus2", "Total tan(phi)"
      m_Pln_bus1_A = "m:Pln:bus1:A", "MW, Active Power (Line-Neutral)"
      m_Pln_bus1_B = "m:Pln:bus1:B", "MW, Active Power (Line-Neutral)"
      m_Pln_bus1_C = "m:Pln:bus1:C", "MW, Active Power (Line-Neutral)"
      m_Pln_bus2 = "m:Pln:bus2", "MW, Active Power (Line-Neutral)"
      m_Qln_bus1_A = "m:Qln:bus1:A", "Mvar, Reactive Power (Line-Neutral)"
      m_Qln_bus1_B = "m:Qln:bus1:B", "Mvar, Reactive Power (Line-Neutral)"
      m_Qln_bus1_C = "m:Qln:bus1:C", "Mvar, Reactive Power (Line-Neutral)"
      m_Qln_bus2 = "m:Qln:bus2", "Mvar, Reactive Power (Line-Neutral)"
      m_P1_bus1 = "m:P1:bus1", "MW, Positive Sequence Active Power"
      m_P1_bus2 = "m:P1:bus2", "MW, Positive Sequence Active Power"
      m_Q1_bus1 = "m:Q1:bus1", "Mvar, Positive Sequence Reactive Power"
      m_Q1_bus2 = "m:Q1:bus2", "Mvar, Positive Sequence Reactive Power"
      m_P2_bus1 = "m:P2:bus1", "MW, Negative Sequence Active Power"
      m_P2_bus2 = "m:P2:bus2", "MW, Negative Sequence Active Power"
      m_Q2_bus1 = "m:Q2:bus1", "Mvar, Negative Sequence Reactive Power"
      m_Q2_bus2 = "m:Q2:bus2", "Mvar, Negative Sequence Reactive Power"
      m_P0_bus1 = "m:P0:bus1", "MW, Zero Sequence Active Power"
      m_P0_bus2 = "m:P0:bus2", "MW, Zero Sequence Active Power"
      m_Q0_bus1 = "m:Q0:bus1", "Mvar, Zero Sequence Reactive Power"
      m_Q0_bus2 = "m:Q0:bus2", "Mvar, Zero Sequence Reactive Power"
      m_i1P_bus1 = "m:i1P:bus1", "p.u., Positive-Sequence Active Current"
      m_i1P_bus2 = "m:i1P:bus2", "p.u., Positive-Sequence Active Current"
      m_i1Q_bus1 = "m:i1Q:bus1", "p.u., Positive-Sequence Reactive Current"
      m_i1Q_bus2 = "m:i1Q:bus2", "p.u., Positive-Sequence Reactive Current"
      m_I1P_bus1 = "m:I1P:bus1", "kA, Positive-Sequence Active Current"
      m_I1P_bus2 = "m:I1P:bus2", "kA, Positive-Sequence Active Current"
      m_I1Q_bus1 = "m:I1Q:bus1", "kA, Positive-Sequence Reactive Current"
      m_I1Q_bus2 = "m:I1Q:bus2", "kA, Positive-Sequence Reactive Current"
      m_i2P_bus1 = "m:i2P:bus1", "p.u., Negative-Sequence Active Current"
      m_i2P_bus2 = "m:i2P:bus2", "p.u., Negative-Sequence Active Current"
      m_i2Q_bus1 = "m:i2Q:bus1", "p.u., Negative-Sequence Reactive Current"
      m_i2Q_bus2 = "m:i2Q:bus2", "p.u., Negative-Sequence Reactive Current"
      m_I2P_bus1 = "m:I2P:bus1", "kA, Negative-Sequence Active Current"
      m_I2P_bus2 = "m:I2P:bus2", "kA, Negative-Sequence Active Current"
      m_I2Q_bus1 = "m:I2Q:bus1", "kA, Negative-Sequence Reactive Current"
      m_I2Q_bus2 = "m:I2Q:bus2", "kA, Negative-Sequence Reactive Current"
      m_ur_bus1_A = "m:ur:bus1:A", "p.u., Phase Voltage, Real Part"
      m_ur_bus1_B = "m:ur:bus1:B", "p.u., Phase Voltage, Real Part"
      m_ur_bus1_C = "m:ur:bus1:C", "p.u., Phase Voltage, Real Part"
      m_ur_bus2 = "m:ur:bus2", "p.u., Phase Voltage, Real Part"
      m_ui_bus1_A = "m:ui:bus1:A", "p.u., Phase Voltage, Imaginary Part"
      m_ui_bus1_B = "m:ui:bus1:B", "p.u., Phase Voltage, Imaginary Part"
      m_ui_bus1_C = "m:ui:bus1:C", "p.u., Phase Voltage, Imaginary Part"
      m_ui_bus2 = "m:ui:bus2", "p.u., Phase Voltage, Imaginary Part"
      m_u_bus1_A = "m:u:bus1:A", "p.u., Phase Voltage, Magnitude"
      m_u_bus1_B = "m:u:bus1:B", "p.u., Phase Voltage, Magnitude"
      m_u_bus1_C = "m:u:bus1:C", "p.u., Phase Voltage, Magnitude"
      m_u_bus2 = "m:u:bus2", "p.u., Phase Voltage, Magnitude"
      m_ir_bus1_A = "m:ir:bus1:A", "p.u., Phase Current, Real Part"
      m_ir_bus1_B = "m:ir:bus1:B", "p.u., Phase Current, Real Part"
      m_ir_bus1_C = "m:ir:bus1:C", "p.u., Phase Current, Real Part"
      m_ir_bus2 = "m:ir:bus2", "p.u., Phase Current, Real Part"
      m_ii_bus1_A = "m:ii:bus1:A", "p.u., Phase Current, Imaginary Part"
      m_ii_bus1_B = "m:ii:bus1:B", "p.u., Phase Current, Imaginary Part"
      m_ii_bus1_C = "m:ii:bus1:C", "p.u., Phase Current, Imaginary Part"
      m_ii_bus2 = "m:ii:bus2", "p.u., Phase Current, Imaginary Part"
      m_i_bus1_A = "m:i:bus1:A", "p.u., Phase Current, Magnitude"
      m_i_bus1_B = "m:i:bus1:B", "p.u., Phase Current, Magnitude"
      m_i_bus1_C = "m:i:bus1:C", "p.u., Phase Current, Magnitude"
      m_i_bus2 = "m:i:bus2", "p.u., Phase Current, Magnitude"
      m_phiui_bus1_A = "m:phiui:bus1:A", "deg, Angle between Voltage and Current"
      m_phiui_bus1_B = "m:phiui:bus1:B", "deg, Angle between Voltage and Current"
      m_phiui_bus1_C = "m:phiui:bus1:C", "deg, Angle between Voltage and Current"
      m_phiui_bus2 = "m:phiui:bus2", "deg, Angle between Voltage and Current"
      m_phiu0i0_bus1 = "m:phiu0i0:bus1", "deg, Angle between Voltage and Current in zero-sequence system"
      m_phiu0i0_bus2 = "m:phiu0i0:bus2", "deg, Angle between Voltage and Current in zero-sequence system"
      m_phiu1i1_bus1 = "m:phiu1i1:bus1", "deg, Angle between Voltage and Current in positive sequence system"
      m_phiu1i1_bus2 = "m:phiu1i1:bus2", "deg, Angle between Voltage and Current in positive sequence system"
      m_phiu2i2_bus1 = "m:phiu2i2:bus1", "deg, Angle between Voltage and Current in negative sequence system"
      m_phiu2i2_bus2 = "m:phiu2i2:bus2", "deg, Angle between Voltage and Current in negative sequence system"
      m_Tfct_bus1 = "m:Tfct:bus1", "s, Fault Clearing Time"
      m_Tfct_bus2 = "m:Tfct:bus2", "s, Fault Clearing Time"
      m_TfctPh_bus1_A = "m:TfctPh:bus1:A", "s, Fault Clearing Time"
      m_TfctPh_bus1_B = "m:TfctPh:bus1:B", "s, Fault Clearing Time"
      m_TfctPh_bus1_C = "m:TfctPh:bus1:C", "s, Fault Clearing Time"
      m_TfctPh_bus2 = "m:TfctPh:bus2", "s, Fault Clearing Time"
      m_Brkload_bus1 = "m:Brkload:bus1", "%, Max. Breaker Loading"
      m_Brkload_bus2 = "m:Brkload:bus2", "%, Max. Breaker Loading"
      m_BrkloadPh_bus1_A = "m:BrkloadPh:bus1:A", "%, Breaker Loading"
      m_BrkloadPh_bus1_B = "m:BrkloadPh:bus1:B", "%, Breaker Loading"
      m_BrkloadPh_bus1_C = "m:BrkloadPh:bus1:C", "%, Breaker Loading"
      m_BrkloadPh_bus2 = "m:BrkloadPh:bus2", "%, Breaker Loading"
      n_ur_bus1_A = "n:ur:bus1:A", "p.u., Line-Ground Voltage, Real Part"
      n_ur_bus1_B = "n:ur:bus1:B", "p.u., Line-Ground Voltage, Real Part"
      n_ur_bus1_C = "n:ur:bus1:C", "p.u., Line-Ground Voltage, Real Part"
      n_ur_bus2 = "n:ur:bus2", "p.u., Line-Ground Voltage, Real Part"
      n_ui_bus1_A = "n:ui:bus1:A", "p.u., Line-Ground Voltage, Imaginary Part"
      n_ui_bus1_B = "n:ui:bus1:B", "p.u., Line-Ground Voltage, Imaginary Part"
      n_ui_bus1_C = "n:ui:bus1:C", "p.u., Line-Ground Voltage, Imaginary Part"
      n_ui_bus2 = "n:ui:bus2", "p.u., Line-Ground Voltage, Imaginary Part"
      n_u_bus1_A = "n:u:bus1:A", "p.u., Line-Ground Voltage, Magnitude"
      n_u_bus1_B = "n:u:bus1:B", "p.u., Line-Ground Voltage, Magnitude"
      n_u_bus1_C = "n:u:bus1:C", "p.u., Line-Ground Voltage, Magnitude"
      n_u_bus2 = "n:u:bus2", "p.u., Line-Ground Voltage, Magnitude"
      n_upc_bus1_A = "n:upc:bus1:A", "%, Line-Ground Voltage, Magnitude"
      n_upc_bus1_B = "n:upc:bus1:B", "%, Line-Ground Voltage, Magnitude"
      n_upc_bus1_C = "n:upc:bus1:C", "%, Line-Ground Voltage, Magnitude"
      n_upc_bus2 = "n:upc:bus2", "%, Line-Ground Voltage, Magnitude"
      n_U_bus1_A = "n:U:bus1:A", "kV, Line-Ground Voltage, Magnitude"
      n_U_bus1_B = "n:U:bus1:B", "kV, Line-Ground Voltage, Magnitude"
      n_U_bus1_C = "n:U:bus1:C", "kV, Line-Ground Voltage, Magnitude"
      n_U_bus2 = "n:U:bus2", "kV, Line-Ground Voltage, Magnitude"
      n_phiu_bus1_A = "n:phiu:bus1:A", "deg, Line-Ground Voltage, Angle"
      n_phiu_bus1_B = "n:phiu:bus1:B", "deg, Line-Ground Voltage, Angle"
      n_phiu_bus1_C = "n:phiu:bus1:C", "deg, Line-Ground Voltage, Angle"
      n_phiu_bus2 = "n:phiu:bus2", "deg, Line-Ground Voltage, Angle"
      n_phiurel_bus1 = "n:phiurel:bus1", "deg, Voltage, Relative Angle"
      n_phiurel_bus2 = "n:phiurel:bus2", "deg, Voltage, Relative Angle"
      n_ul_bus1_A = "n:ul:bus1:A", "p.u., Line-Line Voltage, Magnitude"
      n_ul_bus1_B = "n:ul:bus1:B", "p.u., Line-Line Voltage, Magnitude"
      n_ul_bus1_C = "n:ul:bus1:C", "p.u., Line-Line Voltage, Magnitude"
      n_ul_bus2 = "n:ul:bus2", "p.u., Line-Line Voltage, Magnitude"
      n_ulpc_bus1_A = "n:ulpc:bus1:A", "%, Line-Line Voltage, Magnitude"
      n_ulpc_bus1_B = "n:ulpc:bus1:B", "%, Line-Line Voltage, Magnitude"
      n_ulpc_bus1_C = "n:ulpc:bus1:C", "%, Line-Line Voltage, Magnitude"
      n_ulpc_bus2 = "n:ulpc:bus2", "%, Line-Line Voltage, Magnitude"
      n_Ul_bus1_A = "n:Ul:bus1:A", "kV, Line-Line Voltage, Magnitude"
      n_Ul_bus1_B = "n:Ul:bus1:B", "kV, Line-Line Voltage, Magnitude"
      n_Ul_bus1_C = "n:Ul:bus1:C", "kV, Line-Line Voltage, Magnitude"
      n_Ul_bus2 = "n:Ul:bus2", "kV, Line-Line Voltage, Magnitude"
      n_phiul_bus1_A = "n:phiul:bus1:A", "deg, Line-Line Voltage, Angle"
      n_phiul_bus1_B = "n:phiul:bus1:B", "deg, Line-Line Voltage, Angle"
      n_phiul_bus1_C = "n:phiul:bus1:C", "deg, Line-Line Voltage, Angle"
      n_phiul_bus2 = "n:phiul:bus2", "deg, Line-Line Voltage, Angle"
      n_u0_bus1 = "n:u0:bus1", "p.u., Zero-Sequence Voltage, Magnitude"
      n_u0_bus2 = "n:u0:bus2", "p.u., Zero-Sequence Voltage, Magnitude"
      n_U0_bus1 = "n:U0:bus1", "kV, Zero-Sequence Voltage, Magnitude"
      n_U0_bus2 = "n:U0:bus2", "kV, Zero-Sequence Voltage, Magnitude"
      n_U0x3_bus1 = "n:U0x3:bus1", "kV, 3*U0"
      n_U0x3_bus2 = "n:U0x3:bus2", "kV, 3*U0"
      n_phiu0_bus1 = "n:phiu0:bus1", "deg, Zero-Sequence Voltage, Angle"
      n_phiu0_bus2 = "n:phiu0:bus2", "deg, Zero-Sequence Voltage, Angle"
      n_u1_bus1 = "n:u1:bus1", "p.u., Positive-Sequence Voltage, Magnitude"
      n_u1_bus2 = "n:u1:bus2", "p.u., Positive-Sequence Voltage, Magnitude"
      n_u1pc_bus1 = "n:u1pc:bus1", "%, Positive-Sequence Voltage, Magnitude"
      n_u1pc_bus2 = "n:u1pc:bus2", "%, Positive-Sequence Voltage, Magnitude"
      n_u1r_bus1 = "n:u1r:bus1", "p.u., Positive-Sequence Voltage, Real Part"
      n_u1r_bus2 = "n:u1r:bus2", "p.u., Positive-Sequence Voltage, Real Part"
      n_u1i_bus1 = "n:u1i:bus1", "p.u., Positive-Sequence Voltage, Imaginary Part"
      n_u1i_bus2 = "n:u1i:bus2", "p.u., Positive-Sequence Voltage, Imaginary Part"
      n_U1_bus1 = "n:U1:bus1", "kV, Line-Ground Positive-Sequence Voltage, Magnitude"
      n_U1_bus2 = "n:U1:bus2", "kV, Line-Ground Positive-Sequence Voltage, Magnitude"
      n_phiu1_bus1 = "n:phiu1:bus1", "deg, Positive-Sequence Voltage, Angle"
      n_phiu1_bus2 = "n:phiu1:bus2", "deg, Positive-Sequence Voltage, Angle"
      n_u2_bus1 = "n:u2:bus1", "p.u., Negative-Sequence Voltage, Magnitude"
      n_u2_bus2 = "n:u2:bus2", "p.u., Negative-Sequence Voltage, Magnitude"
      n_U2_bus1 = "n:U2:bus1", "kV, Line-Ground Negative-Sequence Voltage, Magnitude"
      n_U2_bus2 = "n:U2:bus2", "kV, Line-Ground Negative-Sequence Voltage, Magnitude"
      n_phiu2_bus1 = "n:phiu2:bus1", "deg, Negative-Sequence Voltage, Angle"
      n_phiu2_bus2 = "n:phiu2:bus2", "deg, Negative-Sequence Voltage, Angle"
      n_U1l_bus1 = "n:U1l:bus1", "kV, Line-Line Positive-Sequence Voltage, Magnitude"
      n_U1l_bus2 = "n:U1l:bus2", "kV, Line-Line Positive-Sequence Voltage, Magnitude"
      n_U2l_bus1 = "n:U2l:bus1", "kV, Line-Line Negative-Sequence Voltage, Magnitude"
      n_U2l_bus2 = "n:U2l:bus2", "kV, Line-Line Negative-Sequence Voltage, Magnitude"
      n_uln_bus1_A = "n:uln:bus1:A", "p.u., Line-Neutral Voltage, Magnitude"
      n_uln_bus1_B = "n:uln:bus1:B", "p.u., Line-Neutral Voltage, Magnitude"
      n_uln_bus1_C = "n:uln:bus1:C", "p.u., Line-Neutral Voltage, Magnitude"
      n_uln_bus2 = "n:uln:bus2", "p.u., Line-Neutral Voltage, Magnitude"
      n_Uln_bus1_A = "n:Uln:bus1:A", "kV, Line-Neutral Voltage, Magnitude"
      n_Uln_bus1_B = "n:Uln:bus1:B", "kV, Line-Neutral Voltage, Magnitude"
      n_Uln_bus1_C = "n:Uln:bus1:C", "kV, Line-Neutral Voltage, Magnitude"
      n_Uln_bus2 = "n:Uln:bus2", "kV, Line-Neutral Voltage, Magnitude"
      n_phiuln_bus1_A = "n:phiuln:bus1:A", "deg, Line-Neutral Voltage, Angle"
      n_phiuln_bus1_B = "n:phiuln:bus1:B", "deg, Line-Neutral Voltage, Angle"
      n_phiuln_bus1_C = "n:phiuln:bus1:C", "deg, Line-Neutral Voltage, Angle"
      n_phiuln_bus2 = "n:phiuln:bus2", "deg, Line-Neutral Voltage, Angle"
      n_un_bus1 = "n:un:bus1", "p.u., Neutral-Ground Voltage, Magnitude"
      n_un_bus2 = "n:un:bus2", "p.u., Neutral-Ground Voltage, Magnitude"
      n_Un_bus1 = "n:Un:bus1", "kV, Neutral-Ground Voltage, Magnitude"
      n_Un_bus2 = "n:Un:bus2", "kV, Neutral-Ground Voltage, Magnitude"
      n_phiun_bus1 = "n:phiun:bus1", "deg, Neutral-Ground Voltage, Angle"
      n_phiun_bus2 = "n:phiun:bus2", "deg, Neutral-Ground Voltage, Angle"
      n_upht_bus1_A = "n:upht:bus1:A", "p.u., Phase Technology dependent Voltage, Magnitude"
      n_upht_bus1_B = "n:upht:bus1:B", "p.u., Phase Technology dependent Voltage, Magnitude"
      n_upht_bus1_C = "n:upht:bus1:C", "p.u., Phase Technology dependent Voltage, Magnitude"
      n_upht_bus2 = "n:upht:bus2", "p.u., Phase Technology dependent Voltage, Magnitude"
      n_Upht_bus1_A = "n:Upht:bus1:A", "kV, Phase Technology dependent Voltage, Magnitude"
      n_Upht_bus1_B = "n:Upht:bus1:B", "kV, Phase Technology dependent Voltage, Magnitude"
      n_Upht_bus1_C = "n:Upht:bus1:C", "kV, Phase Technology dependent Voltage, Magnitude"
      n_Upht_bus2 = "n:Upht:bus2", "kV, Phase Technology dependent Voltage, Magnitude"
      n_um_bus1 = "n:um:bus1", "p.u., Average-Voltage, Magnitude"
      n_um_bus2 = "n:um:bus2", "p.u., Average-Voltage, Magnitude"
      n_Um_bus1 = "n:Um:bus1", "kV, Average-Voltage, Magnitude"
      n_Um_bus2 = "n:Um:bus2", "kV, Average-Voltage, Magnitude"
      n_ishcr_bus1_A = "n:ishcr:bus1:A", "p.u., Short-Circuit Current, real"
      n_ishcr_bus1_B = "n:ishcr:bus1:B", "p.u., Short-Circuit Current, real"
      n_ishcr_bus1_C = "n:ishcr:bus1:C", "p.u., Short-Circuit Current, real"
      n_ishcr_bus2 = "n:ishcr:bus2", "p.u., Short-Circuit Current, real"
      n_ishci_bus1_A = "n:ishci:bus1:A", "p.u., Short-Circuit Current, imag"
      n_ishci_bus1_B = "n:ishci:bus1:B", "p.u., Short-Circuit Current, imag"
      n_ishci_bus1_C = "n:ishci:bus1:C", "p.u., Short-Circuit Current, imag"
      n_ishci_bus2 = "n:ishci:bus2", "p.u., Short-Circuit Current, imag"
      n_ishc_bus1_A = "n:ishc:bus1:A", "p.u., Short-Circuit Current, magnitude"
      n_ishc_bus1_B = "n:ishc:bus1:B", "p.u., Short-Circuit Current, magnitude"
      n_ishc_bus1_C = "n:ishc:bus1:C", "p.u., Short-Circuit Current, magnitude"
      n_ishc_bus2 = "n:ishc:bus2", "p.u., Short-Circuit Current, magnitude"
      n_Ishc_bus1_A = "n:Ishc:bus1:A", "kA, Short-Circuit Current, magnitude"
      n_Ishc_bus1_B = "n:Ishc:bus1:B", "kA, Short-Circuit Current, magnitude"
      n_Ishc_bus1_C = "n:Ishc:bus1:C", "kA, Short-Circuit Current, magnitude"
      n_Ishc_bus2 = "n:Ishc:bus2", "kA, Short-Circuit Current, magnitude"
      n_inshc_bus1 = "n:inshc:bus1", "p.u., Neutral Short-Circuit Current, magnitude"
      n_inshc_bus2 = "n:inshc:bus2", "p.u., Neutral Short-Circuit Current, magnitude"
      n_Inshc_bus1 = "n:Inshc:bus1", "kA, Neutral Short-Circuit Current, magnitude"
      n_Inshc_bus2 = "n:Inshc:bus2", "kA, Neutral Short-Circuit Current, magnitude"
      n_fe_bus1 = "n:fe:bus1", "p.u., Electrical Frequency"
      n_fe_bus2 = "n:fe:bus2", "p.u., Electrical Frequency"
      n_frdev_bus1 = "n:frdev:bus1", "Hz, average frequency"
      n_frdev_bus2 = "n:frdev:bus2", "Hz, average frequency"
      n_dfehz_bus1 = "n:dfehz:bus1", "Hz, Deviation of the El. Frequency"
      n_dfehz_bus2 = "n:dfehz:bus2", "Hz, Deviation of the El. Frequency"
      n_dfedt_bus1 = "n:dfedt:bus1", "p.u./s, Derivative of El. Frequency"
      n_dfedt_bus2 = "n:dfedt:bus2", "p.u./s, Derivative of El. Frequency"
      n_fehz_bus1 = "n:fehz:bus1", "Hz, Electrical Frequency"
      n_fehz_bus2 = "n:fehz:bus2", "Hz, Electrical Frequency"
      n_frnom_bus1 = "n:frnom:bus1", "Hz, Nominal Frequency"
      n_frnom_bus2 = "n:frnom:bus2", "Hz, Nominal Frequency"
      s_pt = "s:pt", "p.u., IN, Turbine Power, continuous, double"
      s_xmdm = "s:xmdm", "p.u., IN, MDM-Torque, continuous, double"
      s_rradd = "s:rradd", "p.u., IN, Additional Rotor Resistance, continuous, double"
      s_Pmd = "s:Pmd", "IN, PWM Amplification, Real Part, continuous, double"
      s_Pmq = "s:Pmq", "IN, PWM Amplification, Imaginary Part, continuous, double"
      s_xspeed = "s:xspeed", "p.u., OUT, Speed, continuous, double"
      s_pgt = "s:pgt", "p.u., OUT, Active power, continuous, double"
      s_xme = "s:xme", "p.u., OUT, Electrical Torque, continuous, double"
      s_ird = "s:ird", "p.u., OUT, Rotor Current, d-Axis, continuous, double"
      s_irq = "s:irq", "p.u., OUT, Rotor Current, q-Axis, continuous, double"
      s_xphim = "s:xphim", "rad, OUT, Rotor Angle, continuous, double"
      s_psis_r = "s:psis_r", "p.u., OUT, Stator Flux, Real Part, continuous, double"
      s_psis_i = "s:psis_i", "p.u., OUT, Stator Flux, Imaginary Part, continuous, double"
      s_cosphim = "s:cosphim", "OUT, cos(phim), continuous, double"
      s_sinphim = "s:sinphim", "OUT, sin(phim), continuous, double"
      s_psir_r = "s:psir_r", "p.u., OUT, Rotor Flux, Real Part, continuous, double"
      s_psir_i = "s:psir_i", "p.u., OUT, Rotor Flux, Imaginary Part, continuous, double"
      s_speed = "s:speed", "STATE, Speed, continuous, double"
      s_psiA1_r = "s:psiA1_r", "STATE, Flux, Loop A1, Real Part, continuous, double"
      s_psiA1_i = "s:psiA1_i", "STATE, Flux, Loop A1, Imaginary Part, continuous, double"
      s_psiA2_r = "s:psiA2_r", "STATE, Flux, Loop A2, Real Part, continuous, double"
      s_psiA2_i = "s:psiA2_i", "STATE, Flux, Loop A2, Imaginary Part, continuous, double"
      s_psiB_r = "s:psiB_r", "STATE, Flux, Loop B, Real Part continuous, double"
      s_psiB_i = "s:psiB_i", "STATE, Flux, Loop B, Imaginary Part, continuous, double"
      s_speed_dt = "s:speed:dt", "/s, d/dt, Speed, derivative, continuous, double"
      s_psiA1_r_dt = "s:psiA1_r:dt", "/s, d/dt, Flux, Loop A1, Real Part, derivative, continuous, double"
      s_psiA1_i_dt = "s:psiA1_i:dt", "/s, d/dt, Flux, Loop A1, Imaginary Part, derivative, continuous, double"
      s_psiA2_r_dt = "s:psiA2_r:dt", "/s, d/dt, Flux, Loop A2, Real Part, derivative, continuous, double"
      s_psiA2_i_dt = "s:psiA2_i:dt", "/s, d/dt, Flux, Loop A2, Imaginary Part, derivative, continuous, double"
      s_psiB_r_dt = "s:psiB_r:dt", "/s, d/dt, Flux, Loop B, Real Part, derivative, continuous, double"
      s_psiB_i_dt = "s:psiB_i:dt", "/s, d/dt, Flux, Loop B, Imaginary Part, derivative, continuous, double"
      c_cosn = "c:cosn", "Rated Power Factor"
      c_p = "c:p", "MW, Active Power (Electrical)"
      c_rstr = "c:rstr", "p.u., Stator Resistance"
      c_xstr = "c:xstr", "p.u., Stator Reactance"
      c_rrtrA = "c:rrtrA", "p.u., Rotor Resistance A"
      c_rrtrB = "c:rrtrB", "p.u., Rotor Resistance B"
      c_xrtrA = "c:xrtrA", "p.u., Rotor Reactance A"
      c_xrtrB = "c:xrtrB", "p.u., Rotor Reactance B"
      c_xm = "c:xm", "p.u., Magnetising Reactance"
      c_xmrtr = "c:xmrtr", "p.u., Rotor Leakage Reactance"
      c_frnom = "c:frnom", "Hz, Nominal Frequency"
      c_i_mot = "c:i_mot", "i_mot=0"
      c_xmem = "c:xmem", "p.u., Electrical Torque"
      c_xmt = "c:xmt", "p.u., Mechanical Torque"
      c_slip = "c:slip", "%, Slip"
      c_tag = "c:tag", "s, Acceleration Time Constant"
      c_i_star = "c:i_star", "Star/Delta"
      c_xss = "c:xss", "p.u., Subtransient Reactance"
      c_addmt = "c:addmt", "p.u., Additional Torque"
      c_fe = "c:fe", "Electrical Frequency"
      c_z0_r = "c:z0:r", "p.u., Zero Sequence Impedance, Real Part"
      c_z0_i = "c:z0:i", "p.u., Zero Sequence Impedance, Imaginary Part"
      c_z0_phi = "c:z0:phi", "deg, Zero Sequence Impedance, Angle"
      c_z0_mag = "c:z0:mag", "p.u., Zero Sequence Impedance, Magnitude"
      c_z0_phirad = "c:z0:phirad", "rad, Zero Sequence Impedance, Angle"
      c_ze_r = "c:ze:r", "p.u., Grounding Impedance, Real Part"
      c_ze_i = "c:ze:i", "p.u., Grounding Impedance, Imaginary Part"
      c_ze_phi = "c:ze:phi", "deg, Grounding Impedance, Angle"
      c_ze_mag = "c:ze:mag", "p.u., Grounding Impedance, Magnitude"
      c_ze_phirad = "c:ze:phirad", "rad, Grounding Impedance, Angle"
      c_z2_r = "c:z2:r", "p.u., Negative Sequence Impedance, Real Part"
      c_z2_i = "c:z2:i", "p.u., Negative Sequence Impedance, Imaginary Part"
      c_z2_phi = "c:z2:phi", "deg, Negative Sequence Impedance, Angle"
      c_z2_mag = "c:z2:mag", "p.u., Negative Sequence Impedance, Magnitude"
      c_z2_phirad = "c:z2:phirad", "rad, Negative Sequence Impedance, Angle"
      c_Pmr = "c:Pmr", "Amplification, Real Part"
      c_Pmi = "c:Pmi", "Amplification, Imaginary Part"
      c_Unrot = "c:Unrot", "kV, Rated Slip Ring Voltage"
      c_usr = "c:usr", "p.u., Rotor Voltage, Real Part"
      c_usi = "c:usi", "p.u., Rotor Voltage, Imaginary Part"
      c_urd = "c:urd", "p.u., Rotor Voltage, d-Axis"
      c_urq = "c:urq", "p.u., Rotor Voltage, q-Axis"
      c_ura = "c:ura", "p.u., Rotor Voltage, Phase a"
      c_urb = "c:urb", "p.u., Rotor Voltage, Phase b"
      c_urc = "c:urc", "p.u., Rotor Voltage, Phase c"
      c_Ird = "c:Ird", "kA, Rotor Current, d-Axis"
      c_Irq = "c:Irq", "kA, Rotor Current, q-Axis"
      c_Ira = "c:Ira", "kA, Rotor Current, Phase a"
      c_Irb = "c:Irb", "kA, Rotor Current, Phase b"
      c_Irc = "c:Irc", "kA, Rotor Current, Phase c"
      c_xradd = "c:xradd", "Additional Rotor Reactance"
      c_xstadd = "c:xstadd", "p.u., Additional Stator Reactance"
      c_Irot = "c:Irot", "kA, Rotor Current, Magnitude of Phasor"
      c_rcrow = "c:rcrow", "Crow-Bar Resistance"
      c_xcrow = "c:xcrow", "Crow-Bar Reactance"
      c_set_bypass = "c:set_bypass", "Set Rotor-Bypass"
      c_reset_bypass = "c:reset_bypass", "Reset Rotor-Bypass"
      c_psiM = "c:psiM", "p.u., Magnetising flux"
      c_ksat = "c:ksat", "Saturation factor"
      c_xmsat = "c:xmsat", "p.u., Saturated magnetising reactance"
      e_iAstabint = "e:iAstabint", "A-stable integration algorithm"
      e_rcrow = "e:rcrow", "p.u., Crow-Bar Resistance"
      e_xcrow = "e:xcrow", "p.u., Crow-Bar Reactance"
      e_mdmlp = "e:mdmlp", "p.u., Proportional Factor"
      e_mdmex = "e:mdmex", "Exponent"

    class ElmBoundary(Enum):
      _init_ = 'value __doc__'

      s_Pinter = "s:Pinter", "MW, OUT, Active Power Interchange, continuous, double"
      s_Qinter = "s:Qinter", "Mvar, OUT, Reactive Power Interchange, continuous, double"

    class ElmCoup(Enum):
      _init_ = 'value __doc__'

      m_u0_bus1 = "m:u0:bus1", "p.u., Zero-Sequence-Voltage, Magnitude"
      m_u0_bus2 = "m:u0:bus2", "p.u., Zero-Sequence-Voltage, Magnitude"
      m_phiu0_bus1 = "m:phiu0:bus1", "deg, Zero-Sequence-Voltage, Angle"
      m_phiu0_bus2 = "m:phiu0:bus2", "deg, Zero-Sequence-Voltage, Angle"
      m_u1r_bus1 = "m:u1r:bus1", "p.u., Positive-Sequence-Voltage, Real Part"
      m_u1r_bus2 = "m:u1r:bus2", "p.u., Positive-Sequence-Voltage, Real Part"
      m_u1i_bus1 = "m:u1i:bus1", "p.u., Positive-Sequence-Voltage, Imaginary Part"
      m_u1i_bus2 = "m:u1i:bus2", "p.u., Positive-Sequence-Voltage, Imaginary Part"
      m_u1_bus1 = "m:u1:bus1", "p.u., Positive-Sequence-Voltage, Magnitude"
      m_u1_bus2 = "m:u1:bus2", "p.u., Positive-Sequence-Voltage, Magnitude"
      m_U1_bus1 = "m:U1:bus1", "kV, Line-Ground Positive-Sequence-Voltage, Magnitude"
      m_U1_bus2 = "m:U1:bus2", "kV, Line-Ground Positive-Sequence-Voltage, Magnitude"
      m_U1l_bus1 = "m:U1l:bus1", "kV, Line-Line Positive-Sequence-Voltage, Magnitude"
      m_U1l_bus2 = "m:U1l:bus2", "kV, Line-Line Positive-Sequence-Voltage, Magnitude"
      m_phiu1_bus1 = "m:phiu1:bus1", "deg, Positive-Sequence-Voltage, Angle"
      m_phiu1_bus2 = "m:phiu1:bus2", "deg, Positive-Sequence-Voltage, Angle"
      m_u2_bus1 = "m:u2:bus1", "p.u., Negative-Sequence-Voltage, Magnitude"
      m_u2_bus2 = "m:u2:bus2", "p.u., Negative-Sequence-Voltage, Magnitude"
      m_phiu2_bus1 = "m:phiu2:bus1", "deg, Negative-Sequence-Voltage, Angle"
      m_phiu2_bus2 = "m:phiu2:bus2", "deg, Negative-Sequence-Voltage, Angle"
      m_inet_bus1_A = "m:inet:bus1:A", "p.u., Phase Current, Magnitude"
      m_inet_bus1_B = "m:inet:bus1:B", "p.u., Phase Current, Magnitude"
      m_inet_bus1_C = "m:inet:bus1:C", "p.u., Phase Current, Magnitude"
      m_inet_bus1_N = "m:inet:bus1:N", "p.u., Phase Current, Magnitude"
      m_inet_bus2_A = "m:inet:bus2:A", "p.u., Phase Current, Magnitude"
      m_inet_bus2_B = "m:inet:bus2:B", "p.u., Phase Current, Magnitude"
      m_inet_bus2_C = "m:inet:bus2:C", "p.u., Phase Current, Magnitude"
      m_inet_bus2_N = "m:inet:bus2:N", "p.u., Phase Current, Magnitude"
      m_I_bus1_A = "m:I:bus1:A", "kA, Phase Current, Magnitude"
      m_I_bus1_B = "m:I:bus1:B", "kA, Phase Current, Magnitude"
      m_I_bus1_C = "m:I:bus1:C", "kA, Phase Current, Magnitude"
      m_I_bus1_N = "m:I:bus1:N", "kA, Phase Current, Magnitude"
      m_I_bus2_A = "m:I:bus2:A", "kA, Phase Current, Magnitude"
      m_I_bus2_B = "m:I:bus2:B", "kA, Phase Current, Magnitude"
      m_I_bus2_C = "m:I:bus2:C", "kA, Phase Current, Magnitude"
      m_I_bus2_N = "m:I:bus2:N", "kA, Phase Current, Magnitude"
      m_phii_bus1_A = "m:phii:bus1:A", "deg, Phase Current, Angle"
      m_phii_bus1_B = "m:phii:bus1:B", "deg, Phase Current, Angle"
      m_phii_bus1_C = "m:phii:bus1:C", "deg, Phase Current, Angle"
      m_phii_bus1_N = "m:phii:bus1:N", "deg, Phase Current, Angle"
      m_phii_bus2_A = "m:phii:bus2:A", "deg, Phase Current, Angle"
      m_phii_bus2_B = "m:phii:bus2:B", "deg, Phase Current, Angle"
      m_phii_bus2_C = "m:phii:bus2:C", "deg, Phase Current, Angle"
      m_phii_bus2_N = "m:phii:bus2:N", "deg, Phase Current, Angle"
      m_i0r_bus1 = "m:i0r:bus1", "p.u., Zero-Sequence Current, Real Part"
      m_i0r_bus2 = "m:i0r:bus2", "p.u., Zero-Sequence Current, Real Part"
      m_i0i_bus1 = "m:i0i:bus1", "p.u., Zero-Sequence Current, Imaginary Part"
      m_i0i_bus2 = "m:i0i:bus2", "p.u., Zero-Sequence Current, Imaginary Part"
      m_i0_bus1 = "m:i0:bus1", "p.u., Zero-Sequence Current, Magnitude"
      m_i0_bus2 = "m:i0:bus2", "p.u., Zero-Sequence Current, Magnitude"
      m_I0_bus1 = "m:I0:bus1", "kA, Zero-Sequence Current, Magnitude"
      m_I0_bus2 = "m:I0:bus2", "kA, Zero-Sequence Current, Magnitude"
      m_I0x3_bus1 = "m:I0x3:bus1", "kA, 3*I0"
      m_I0x3_bus2 = "m:I0x3:bus2", "kA, 3*I0"
      m_phii0_bus1 = "m:phii0:bus1", "deg, Zero-Sequence Current, Angle"
      m_phii0_bus2 = "m:phii0:bus2", "deg, Zero-Sequence Current, Angle"
      m_i1r_bus1 = "m:i1r:bus1", "p.u., Positive-Sequence Current, Real Part"
      m_i1r_bus2 = "m:i1r:bus2", "p.u., Positive-Sequence Current, Real Part"
      m_i1i_bus1 = "m:i1i:bus1", "p.u., Positive-Sequence Current, Imaginary Part"
      m_i1i_bus2 = "m:i1i:bus2", "p.u., Positive-Sequence Current, Imaginary Part"
      m_i1_bus1 = "m:i1:bus1", "p.u., Positive-Sequence Current, Magnitude"
      m_i1_bus2 = "m:i1:bus2", "p.u., Positive-Sequence Current, Magnitude"
      m_I1_bus1 = "m:I1:bus1", "kA, Positive-Sequence Current, Magnitude"
      m_I1_bus2 = "m:I1:bus2", "kA, Positive-Sequence Current, Magnitude"
      m_phii1_bus1 = "m:phii1:bus1", "deg, Positive-Sequence Current, Angle"
      m_phii1_bus2 = "m:phii1:bus2", "deg, Positive-Sequence Current, Angle"
      m_i2r_bus1 = "m:i2r:bus1", "p.u., Negative-Sequence Current, Real Part"
      m_i2r_bus2 = "m:i2r:bus2", "p.u., Negative-Sequence Current, Real Part"
      m_i2i_bus1 = "m:i2i:bus1", "p.u., Negative-Sequence Current, Imaginary Part"
      m_i2i_bus2 = "m:i2i:bus2", "p.u., Negative-Sequence Current, Imaginary Part"
      m_i2_bus1 = "m:i2:bus1", "p.u., Negative-Sequence Current, Magnitude"
      m_i2_bus2 = "m:i2:bus2", "p.u., Negative-Sequence Current, Magnitude"
      m_I2_bus1 = "m:I2:bus1", "kA, Negative-Sequence Current, Magnitude"
      m_I2_bus2 = "m:I2:bus2", "kA, Negative-Sequence Current, Magnitude"
      m_phii2_bus1 = "m:phii2:bus1", "deg, Negative-Sequence Current, Angle"
      m_phii2_bus2 = "m:phii2:bus2", "deg, Negative-Sequence Current, Angle"
      m_P_bus1_A = "m:P:bus1:A", "MW, Active Power"
      m_P_bus1_B = "m:P:bus1:B", "MW, Active Power"
      m_P_bus1_C = "m:P:bus1:C", "MW, Active Power"
      m_P_bus1_N = "m:P:bus1:N", "MW, Active Power"
      m_P_bus2_A = "m:P:bus2:A", "MW, Active Power"
      m_P_bus2_B = "m:P:bus2:B", "MW, Active Power"
      m_P_bus2_C = "m:P:bus2:C", "MW, Active Power"
      m_P_bus2_N = "m:P:bus2:N", "MW, Active Power"
      m_Q_bus1_A = "m:Q:bus1:A", "Mvar, Reactive Power"
      m_Q_bus1_B = "m:Q:bus1:B", "Mvar, Reactive Power"
      m_Q_bus1_C = "m:Q:bus1:C", "Mvar, Reactive Power"
      m_Q_bus1_N = "m:Q:bus1:N", "Mvar, Reactive Power"
      m_Q_bus2_A = "m:Q:bus2:A", "Mvar, Reactive Power"
      m_Q_bus2_B = "m:Q:bus2:B", "Mvar, Reactive Power"
      m_Q_bus2_C = "m:Q:bus2:C", "Mvar, Reactive Power"
      m_Q_bus2_N = "m:Q:bus2:N", "Mvar, Reactive Power"
      m_S_bus1_A = "m:S:bus1:A", "MVA, Apparent Power"
      m_S_bus1_B = "m:S:bus1:B", "MVA, Apparent Power"
      m_S_bus1_C = "m:S:bus1:C", "MVA, Apparent Power"
      m_S_bus1_N = "m:S:bus1:N", "MVA, Apparent Power"
      m_S_bus2_A = "m:S:bus2:A", "MVA, Apparent Power"
      m_S_bus2_B = "m:S:bus2:B", "MVA, Apparent Power"
      m_S_bus2_C = "m:S:bus2:C", "MVA, Apparent Power"
      m_S_bus2_N = "m:S:bus2:N", "MVA, Apparent Power"
      m_cosphi_bus1_A = "m:cosphi:bus1:A", "Power Factor"
      m_cosphi_bus1_B = "m:cosphi:bus1:B", "Power Factor"
      m_cosphi_bus1_C = "m:cosphi:bus1:C", "Power Factor"
      m_cosphi_bus1_N = "m:cosphi:bus1:N", "Power Factor"
      m_cosphi_bus2_A = "m:cosphi:bus2:A", "Power Factor"
      m_cosphi_bus2_B = "m:cosphi:bus2:B", "Power Factor"
      m_cosphi_bus2_C = "m:cosphi:bus2:C", "Power Factor"
      m_cosphi_bus2_N = "m:cosphi:bus2:N", "Power Factor"
      m_tanphi_bus1_A = "m:tanphi:bus1:A", "tan(phi)"
      m_tanphi_bus1_B = "m:tanphi:bus1:B", "tan(phi)"
      m_tanphi_bus1_C = "m:tanphi:bus1:C", "tan(phi)"
      m_tanphi_bus1_N = "m:tanphi:bus1:N", "tan(phi)"
      m_tanphi_bus2_A = "m:tanphi:bus2:A", "tan(phi)"
      m_tanphi_bus2_B = "m:tanphi:bus2:B", "tan(phi)"
      m_tanphi_bus2_C = "m:tanphi:bus2:C", "tan(phi)"
      m_tanphi_bus2_N = "m:tanphi:bus2:N", "tan(phi)"
      m_Psum_bus1 = "m:Psum:bus1", "MW, Total Active Power"
      m_Psum_bus2 = "m:Psum:bus2", "MW, Total Active Power"
      m_Qsum_bus1 = "m:Qsum:bus1", "Mvar, Total Reactive Power"
      m_Qsum_bus2 = "m:Qsum:bus2", "Mvar, Total Reactive Power"
      m_Ssum_bus1 = "m:Ssum:bus1", "MVA, Total Apparent Power"
      m_Ssum_bus2 = "m:Ssum:bus2", "MVA, Total Apparent Power"
      m_cosphisum_bus1 = "m:cosphisum:bus1", "Total Power Factor"
      m_cosphisum_bus2 = "m:cosphisum:bus2", "Total Power Factor"
      m_tanphisum_bus1 = "m:tanphisum:bus1", "Total tan(phi)"
      m_tanphisum_bus2 = "m:tanphisum:bus2", "Total tan(phi)"
      m_Pln_bus1_A = "m:Pln:bus1:A", "MW, Active Power (Line-Neutral)"
      m_Pln_bus1_B = "m:Pln:bus1:B", "MW, Active Power (Line-Neutral)"
      m_Pln_bus1_C = "m:Pln:bus1:C", "MW, Active Power (Line-Neutral)"
      m_Pln_bus1_N = "m:Pln:bus1:N", "MW, Active Power (Line-Neutral)"
      m_Pln_bus2_A = "m:Pln:bus2:A", "MW, Active Power (Line-Neutral)"
      m_Pln_bus2_B = "m:Pln:bus2:B", "MW, Active Power (Line-Neutral)"
      m_Pln_bus2_C = "m:Pln:bus2:C", "MW, Active Power (Line-Neutral)"
      m_Pln_bus2_N = "m:Pln:bus2:N", "MW, Active Power (Line-Neutral)"
      m_Qln_bus1_A = "m:Qln:bus1:A", "Mvar, Reactive Power (Line-Neutral)"
      m_Qln_bus1_B = "m:Qln:bus1:B", "Mvar, Reactive Power (Line-Neutral)"
      m_Qln_bus1_C = "m:Qln:bus1:C", "Mvar, Reactive Power (Line-Neutral)"
      m_Qln_bus1_N = "m:Qln:bus1:N", "Mvar, Reactive Power (Line-Neutral)"
      m_Qln_bus2_A = "m:Qln:bus2:A", "Mvar, Reactive Power (Line-Neutral)"
      m_Qln_bus2_B = "m:Qln:bus2:B", "Mvar, Reactive Power (Line-Neutral)"
      m_Qln_bus2_C = "m:Qln:bus2:C", "Mvar, Reactive Power (Line-Neutral)"
      m_Qln_bus2_N = "m:Qln:bus2:N", "Mvar, Reactive Power (Line-Neutral)"
      m_P1_bus1 = "m:P1:bus1", "MW, Positive Sequence Active Power"
      m_P1_bus2 = "m:P1:bus2", "MW, Positive Sequence Active Power"
      m_Q1_bus1 = "m:Q1:bus1", "Mvar, Positive Sequence Reactive Power"
      m_Q1_bus2 = "m:Q1:bus2", "Mvar, Positive Sequence Reactive Power"
      m_P2_bus1 = "m:P2:bus1", "MW, Negative Sequence Active Power"
      m_P2_bus2 = "m:P2:bus2", "MW, Negative Sequence Active Power"
      m_Q2_bus1 = "m:Q2:bus1", "Mvar, Negative Sequence Reactive Power"
      m_Q2_bus2 = "m:Q2:bus2", "Mvar, Negative Sequence Reactive Power"
      m_P0_bus1 = "m:P0:bus1", "MW, Zero Sequence Active Power"
      m_P0_bus2 = "m:P0:bus2", "MW, Zero Sequence Active Power"
      m_Q0_bus1 = "m:Q0:bus1", "Mvar, Zero Sequence Reactive Power"
      m_Q0_bus2 = "m:Q0:bus2", "Mvar, Zero Sequence Reactive Power"
      m_i1P_bus1 = "m:i1P:bus1", "p.u., Positive-Sequence Active Current"
      m_i1P_bus2 = "m:i1P:bus2", "p.u., Positive-Sequence Active Current"
      m_i1Q_bus1 = "m:i1Q:bus1", "p.u., Positive-Sequence Reactive Current"
      m_i1Q_bus2 = "m:i1Q:bus2", "p.u., Positive-Sequence Reactive Current"
      m_I1P_bus1 = "m:I1P:bus1", "kA, Positive-Sequence Active Current"
      m_I1P_bus2 = "m:I1P:bus2", "kA, Positive-Sequence Active Current"
      m_I1Q_bus1 = "m:I1Q:bus1", "kA, Positive-Sequence Reactive Current"
      m_I1Q_bus2 = "m:I1Q:bus2", "kA, Positive-Sequence Reactive Current"
      m_i2P_bus1 = "m:i2P:bus1", "p.u., Negative-Sequence Active Current"
      m_i2P_bus2 = "m:i2P:bus2", "p.u., Negative-Sequence Active Current"
      m_i2Q_bus1 = "m:i2Q:bus1", "p.u., Negative-Sequence Reactive Current"
      m_i2Q_bus2 = "m:i2Q:bus2", "p.u., Negative-Sequence Reactive Current"
      m_I2P_bus1 = "m:I2P:bus1", "kA, Negative-Sequence Active Current"
      m_I2P_bus2 = "m:I2P:bus2", "kA, Negative-Sequence Active Current"
      m_I2Q_bus1 = "m:I2Q:bus1", "kA, Negative-Sequence Reactive Current"
      m_I2Q_bus2 = "m:I2Q:bus2", "kA, Negative-Sequence Reactive Current"
      m_ur_bus1_A = "m:ur:bus1:A", "p.u., Phase Voltage, Real Part"
      m_ur_bus1_B = "m:ur:bus1:B", "p.u., Phase Voltage, Real Part"
      m_ur_bus1_C = "m:ur:bus1:C", "p.u., Phase Voltage, Real Part"
      m_ur_bus1_N = "m:ur:bus1:N", "p.u., Phase Voltage, Real Part"
      m_ur_bus2_A = "m:ur:bus2:A", "p.u., Phase Voltage, Real Part"
      m_ur_bus2_B = "m:ur:bus2:B", "p.u., Phase Voltage, Real Part"
      m_ur_bus2_C = "m:ur:bus2:C", "p.u., Phase Voltage, Real Part"
      m_ur_bus2_N = "m:ur:bus2:N", "p.u., Phase Voltage, Real Part"
      m_ui_bus1_A = "m:ui:bus1:A", "p.u., Phase Voltage, Imaginary Part"
      m_ui_bus1_B = "m:ui:bus1:B", "p.u., Phase Voltage, Imaginary Part"
      m_ui_bus1_C = "m:ui:bus1:C", "p.u., Phase Voltage, Imaginary Part"
      m_ui_bus1_N = "m:ui:bus1:N", "p.u., Phase Voltage, Imaginary Part"
      m_ui_bus2_A = "m:ui:bus2:A", "p.u., Phase Voltage, Imaginary Part"
      m_ui_bus2_B = "m:ui:bus2:B", "p.u., Phase Voltage, Imaginary Part"
      m_ui_bus2_C = "m:ui:bus2:C", "p.u., Phase Voltage, Imaginary Part"
      m_ui_bus2_N = "m:ui:bus2:N", "p.u., Phase Voltage, Imaginary Part"
      m_u_bus1_A = "m:u:bus1:A", "p.u., Phase Voltage, Magnitude"
      m_u_bus1_B = "m:u:bus1:B", "p.u., Phase Voltage, Magnitude"
      m_u_bus1_C = "m:u:bus1:C", "p.u., Phase Voltage, Magnitude"
      m_u_bus1_N = "m:u:bus1:N", "p.u., Phase Voltage, Magnitude"
      m_u_bus2_A = "m:u:bus2:A", "p.u., Phase Voltage, Magnitude"
      m_u_bus2_B = "m:u:bus2:B", "p.u., Phase Voltage, Magnitude"
      m_u_bus2_C = "m:u:bus2:C", "p.u., Phase Voltage, Magnitude"
      m_u_bus2_N = "m:u:bus2:N", "p.u., Phase Voltage, Magnitude"
      m_ir_bus1_A = "m:ir:bus1:A", "p.u., Phase Current, Real Part"
      m_ir_bus1_B = "m:ir:bus1:B", "p.u., Phase Current, Real Part"
      m_ir_bus1_C = "m:ir:bus1:C", "p.u., Phase Current, Real Part"
      m_ir_bus1_N = "m:ir:bus1:N", "p.u., Phase Current, Real Part"
      m_ir_bus2_A = "m:ir:bus2:A", "p.u., Phase Current, Real Part"
      m_ir_bus2_B = "m:ir:bus2:B", "p.u., Phase Current, Real Part"
      m_ir_bus2_C = "m:ir:bus2:C", "p.u., Phase Current, Real Part"
      m_ir_bus2_N = "m:ir:bus2:N", "p.u., Phase Current, Real Part"
      m_ii_bus1_A = "m:ii:bus1:A", "p.u., Phase Current, Imaginary Part"
      m_ii_bus1_B = "m:ii:bus1:B", "p.u., Phase Current, Imaginary Part"
      m_ii_bus1_C = "m:ii:bus1:C", "p.u., Phase Current, Imaginary Part"
      m_ii_bus1_N = "m:ii:bus1:N", "p.u., Phase Current, Imaginary Part"
      m_ii_bus2_A = "m:ii:bus2:A", "p.u., Phase Current, Imaginary Part"
      m_ii_bus2_B = "m:ii:bus2:B", "p.u., Phase Current, Imaginary Part"
      m_ii_bus2_C = "m:ii:bus2:C", "p.u., Phase Current, Imaginary Part"
      m_ii_bus2_N = "m:ii:bus2:N", "p.u., Phase Current, Imaginary Part"
      m_i_bus1_A = "m:i:bus1:A", "p.u., Phase Current, Magnitude"
      m_i_bus1_B = "m:i:bus1:B", "p.u., Phase Current, Magnitude"
      m_i_bus1_C = "m:i:bus1:C", "p.u., Phase Current, Magnitude"
      m_i_bus1_N = "m:i:bus1:N", "p.u., Phase Current, Magnitude"
      m_i_bus2_A = "m:i:bus2:A", "p.u., Phase Current, Magnitude"
      m_i_bus2_B = "m:i:bus2:B", "p.u., Phase Current, Magnitude"
      m_i_bus2_C = "m:i:bus2:C", "p.u., Phase Current, Magnitude"
      m_i_bus2_N = "m:i:bus2:N", "p.u., Phase Current, Magnitude"
      m_phiui_bus1_A = "m:phiui:bus1:A", "deg, Angle between Voltage and Current"
      m_phiui_bus1_B = "m:phiui:bus1:B", "deg, Angle between Voltage and Current"
      m_phiui_bus1_C = "m:phiui:bus1:C", "deg, Angle between Voltage and Current"
      m_phiui_bus1_N = "m:phiui:bus1:N", "deg, Angle between Voltage and Current"
      m_phiui_bus2_A = "m:phiui:bus2:A", "deg, Angle between Voltage and Current"
      m_phiui_bus2_B = "m:phiui:bus2:B", "deg, Angle between Voltage and Current"
      m_phiui_bus2_C = "m:phiui:bus2:C", "deg, Angle between Voltage and Current"
      m_phiui_bus2_N = "m:phiui:bus2:N", "deg, Angle between Voltage and Current"
      m_phiu0i0_bus1 = "m:phiu0i0:bus1", "deg, Angle between Voltage and Current in zero-sequence system"
      m_phiu0i0_bus2 = "m:phiu0i0:bus2", "deg, Angle between Voltage and Current in zero-sequence system"
      m_phiu1i1_bus1 = "m:phiu1i1:bus1", "deg, Angle between Voltage and Current in positive sequence system"
      m_phiu1i1_bus2 = "m:phiu1i1:bus2", "deg, Angle between Voltage and Current in positive sequence system"
      m_phiu2i2_bus1 = "m:phiu2i2:bus1", "deg, Angle between Voltage and Current in negative sequence system"
      m_phiu2i2_bus2 = "m:phiu2i2:bus2", "deg, Angle between Voltage and Current in negative sequence system"
      m_Tfct_bus1 = "m:Tfct:bus1", "s, Fault Clearing Time"
      m_Tfct_bus2 = "m:Tfct:bus2", "s, Fault Clearing Time"
      m_TfctPh_bus1_A = "m:TfctPh:bus1:A", "s, Fault Clearing Time"
      m_TfctPh_bus1_B = "m:TfctPh:bus1:B", "s, Fault Clearing Time"
      m_TfctPh_bus1_C = "m:TfctPh:bus1:C", "s, Fault Clearing Time"
      m_TfctPh_bus1_N = "m:TfctPh:bus1:N", "s, Fault Clearing Time"
      m_TfctPh_bus2_A = "m:TfctPh:bus2:A", "s, Fault Clearing Time"
      m_TfctPh_bus2_B = "m:TfctPh:bus2:B", "s, Fault Clearing Time"
      m_TfctPh_bus2_C = "m:TfctPh:bus2:C", "s, Fault Clearing Time"
      m_TfctPh_bus2_N = "m:TfctPh:bus2:N", "s, Fault Clearing Time"
      m_Brkload_bus1 = "m:Brkload:bus1", "%, Max. Breaker Loading"
      m_Brkload_bus2 = "m:Brkload:bus2", "%, Max. Breaker Loading"
      m_BrkloadPh_bus1_A = "m:BrkloadPh:bus1:A", "%, Breaker Loading"
      m_BrkloadPh_bus1_B = "m:BrkloadPh:bus1:B", "%, Breaker Loading"
      m_BrkloadPh_bus1_C = "m:BrkloadPh:bus1:C", "%, Breaker Loading"
      m_BrkloadPh_bus1_N = "m:BrkloadPh:bus1:N", "%, Breaker Loading"
      m_BrkloadPh_bus2_A = "m:BrkloadPh:bus2:A", "%, Breaker Loading"
      m_BrkloadPh_bus2_B = "m:BrkloadPh:bus2:B", "%, Breaker Loading"
      m_BrkloadPh_bus2_C = "m:BrkloadPh:bus2:C", "%, Breaker Loading"
      m_BrkloadPh_bus2_N = "m:BrkloadPh:bus2:N", "%, Breaker Loading"
      n_ur_bus1_A = "n:ur:bus1:A", "p.u., Line-Ground Voltage, Real Part"
      n_ur_bus1_B = "n:ur:bus1:B", "p.u., Line-Ground Voltage, Real Part"
      n_ur_bus1_C = "n:ur:bus1:C", "p.u., Line-Ground Voltage, Real Part"
      n_ur_bus1_N = "n:ur:bus1:N", "p.u., Line-Ground Voltage, Real Part"
      n_ur_bus2_A = "n:ur:bus2:A", "p.u., Line-Ground Voltage, Real Part"
      n_ur_bus2_B = "n:ur:bus2:B", "p.u., Line-Ground Voltage, Real Part"
      n_ur_bus2_C = "n:ur:bus2:C", "p.u., Line-Ground Voltage, Real Part"
      n_ur_bus2_N = "n:ur:bus2:N", "p.u., Line-Ground Voltage, Real Part"
      n_ui_bus1_A = "n:ui:bus1:A", "p.u., Line-Ground Voltage, Imaginary Part"
      n_ui_bus1_B = "n:ui:bus1:B", "p.u., Line-Ground Voltage, Imaginary Part"
      n_ui_bus1_C = "n:ui:bus1:C", "p.u., Line-Ground Voltage, Imaginary Part"
      n_ui_bus1_N = "n:ui:bus1:N", "p.u., Line-Ground Voltage, Imaginary Part"
      n_ui_bus2_A = "n:ui:bus2:A", "p.u., Line-Ground Voltage, Imaginary Part"
      n_ui_bus2_B = "n:ui:bus2:B", "p.u., Line-Ground Voltage, Imaginary Part"
      n_ui_bus2_C = "n:ui:bus2:C", "p.u., Line-Ground Voltage, Imaginary Part"
      n_ui_bus2_N = "n:ui:bus2:N", "p.u., Line-Ground Voltage, Imaginary Part"
      n_u_bus1_A = "n:u:bus1:A", "p.u., Line-Ground Voltage, Magnitude"
      n_u_bus1_B = "n:u:bus1:B", "p.u., Line-Ground Voltage, Magnitude"
      n_u_bus1_C = "n:u:bus1:C", "p.u., Line-Ground Voltage, Magnitude"
      n_u_bus1_N = "n:u:bus1:N", "p.u., Line-Ground Voltage, Magnitude"
      n_u_bus2_A = "n:u:bus2:A", "p.u., Line-Ground Voltage, Magnitude"
      n_u_bus2_B = "n:u:bus2:B", "p.u., Line-Ground Voltage, Magnitude"
      n_u_bus2_C = "n:u:bus2:C", "p.u., Line-Ground Voltage, Magnitude"
      n_u_bus2_N = "n:u:bus2:N", "p.u., Line-Ground Voltage, Magnitude"
      n_upc_bus1_A = "n:upc:bus1:A", "%, Line-Ground Voltage, Magnitude"
      n_upc_bus1_B = "n:upc:bus1:B", "%, Line-Ground Voltage, Magnitude"
      n_upc_bus1_C = "n:upc:bus1:C", "%, Line-Ground Voltage, Magnitude"
      n_upc_bus1_N = "n:upc:bus1:N", "%, Line-Ground Voltage, Magnitude"
      n_upc_bus2_A = "n:upc:bus2:A", "%, Line-Ground Voltage, Magnitude"
      n_upc_bus2_B = "n:upc:bus2:B", "%, Line-Ground Voltage, Magnitude"
      n_upc_bus2_C = "n:upc:bus2:C", "%, Line-Ground Voltage, Magnitude"
      n_upc_bus2_N = "n:upc:bus2:N", "%, Line-Ground Voltage, Magnitude"
      n_U_bus1_A = "n:U:bus1:A", "kV, Line-Ground Voltage, Magnitude"
      n_U_bus1_B = "n:U:bus1:B", "kV, Line-Ground Voltage, Magnitude"
      n_U_bus1_C = "n:U:bus1:C", "kV, Line-Ground Voltage, Magnitude"
      n_U_bus1_N = "n:U:bus1:N", "kV, Line-Ground Voltage, Magnitude"
      n_U_bus2_A = "n:U:bus2:A", "kV, Line-Ground Voltage, Magnitude"
      n_U_bus2_B = "n:U:bus2:B", "kV, Line-Ground Voltage, Magnitude"
      n_U_bus2_C = "n:U:bus2:C", "kV, Line-Ground Voltage, Magnitude"
      n_U_bus2_N = "n:U:bus2:N", "kV, Line-Ground Voltage, Magnitude"
      n_phiu_bus1_A = "n:phiu:bus1:A", "deg, Line-Ground Voltage, Angle"
      n_phiu_bus1_B = "n:phiu:bus1:B", "deg, Line-Ground Voltage, Angle"
      n_phiu_bus1_C = "n:phiu:bus1:C", "deg, Line-Ground Voltage, Angle"
      n_phiu_bus1_N = "n:phiu:bus1:N", "deg, Line-Ground Voltage, Angle"
      n_phiu_bus2_A = "n:phiu:bus2:A", "deg, Line-Ground Voltage, Angle"
      n_phiu_bus2_B = "n:phiu:bus2:B", "deg, Line-Ground Voltage, Angle"
      n_phiu_bus2_C = "n:phiu:bus2:C", "deg, Line-Ground Voltage, Angle"
      n_phiu_bus2_N = "n:phiu:bus2:N", "deg, Line-Ground Voltage, Angle"
      n_phiurel_bus1 = "n:phiurel:bus1", "deg, Voltage, Relative Angle"
      n_phiurel_bus2 = "n:phiurel:bus2", "deg, Voltage, Relative Angle"
      n_ul_bus1_A = "n:ul:bus1:A", "p.u., Line-Line Voltage, Magnitude"
      n_ul_bus1_B = "n:ul:bus1:B", "p.u., Line-Line Voltage, Magnitude"
      n_ul_bus1_C = "n:ul:bus1:C", "p.u., Line-Line Voltage, Magnitude"
      n_ul_bus1_N = "n:ul:bus1:N", "p.u., Line-Line Voltage, Magnitude"
      n_ul_bus2_A = "n:ul:bus2:A", "p.u., Line-Line Voltage, Magnitude"
      n_ul_bus2_B = "n:ul:bus2:B", "p.u., Line-Line Voltage, Magnitude"
      n_ul_bus2_C = "n:ul:bus2:C", "p.u., Line-Line Voltage, Magnitude"
      n_ul_bus2_N = "n:ul:bus2:N", "p.u., Line-Line Voltage, Magnitude"
      n_ulpc_bus1_A = "n:ulpc:bus1:A", "%, Line-Line Voltage, Magnitude"
      n_ulpc_bus1_B = "n:ulpc:bus1:B", "%, Line-Line Voltage, Magnitude"
      n_ulpc_bus1_C = "n:ulpc:bus1:C", "%, Line-Line Voltage, Magnitude"
      n_ulpc_bus1_N = "n:ulpc:bus1:N", "%, Line-Line Voltage, Magnitude"
      n_ulpc_bus2_A = "n:ulpc:bus2:A", "%, Line-Line Voltage, Magnitude"
      n_ulpc_bus2_B = "n:ulpc:bus2:B", "%, Line-Line Voltage, Magnitude"
      n_ulpc_bus2_C = "n:ulpc:bus2:C", "%, Line-Line Voltage, Magnitude"
      n_ulpc_bus2_N = "n:ulpc:bus2:N", "%, Line-Line Voltage, Magnitude"
      n_Ul_bus1_A = "n:Ul:bus1:A", "kV, Line-Line Voltage, Magnitude"
      n_Ul_bus1_B = "n:Ul:bus1:B", "kV, Line-Line Voltage, Magnitude"
      n_Ul_bus1_C = "n:Ul:bus1:C", "kV, Line-Line Voltage, Magnitude"
      n_Ul_bus1_N = "n:Ul:bus1:N", "kV, Line-Line Voltage, Magnitude"
      n_Ul_bus2_A = "n:Ul:bus2:A", "kV, Line-Line Voltage, Magnitude"
      n_Ul_bus2_B = "n:Ul:bus2:B", "kV, Line-Line Voltage, Magnitude"
      n_Ul_bus2_C = "n:Ul:bus2:C", "kV, Line-Line Voltage, Magnitude"
      n_Ul_bus2_N = "n:Ul:bus2:N", "kV, Line-Line Voltage, Magnitude"
      n_phiul_bus1_A = "n:phiul:bus1:A", "deg, Line-Line Voltage, Angle"
      n_phiul_bus1_B = "n:phiul:bus1:B", "deg, Line-Line Voltage, Angle"
      n_phiul_bus1_C = "n:phiul:bus1:C", "deg, Line-Line Voltage, Angle"
      n_phiul_bus1_N = "n:phiul:bus1:N", "deg, Line-Line Voltage, Angle"
      n_phiul_bus2_A = "n:phiul:bus2:A", "deg, Line-Line Voltage, Angle"
      n_phiul_bus2_B = "n:phiul:bus2:B", "deg, Line-Line Voltage, Angle"
      n_phiul_bus2_C = "n:phiul:bus2:C", "deg, Line-Line Voltage, Angle"
      n_phiul_bus2_N = "n:phiul:bus2:N", "deg, Line-Line Voltage, Angle"
      n_u0_bus1 = "n:u0:bus1", "p.u., Zero-Sequence Voltage, Magnitude"
      n_u0_bus2 = "n:u0:bus2", "p.u., Zero-Sequence Voltage, Magnitude"
      n_U0_bus1 = "n:U0:bus1", "kV, Zero-Sequence Voltage, Magnitude"
      n_U0_bus2 = "n:U0:bus2", "kV, Zero-Sequence Voltage, Magnitude"
      n_U0x3_bus1 = "n:U0x3:bus1", "kV, 3*U0"
      n_U0x3_bus2 = "n:U0x3:bus2", "kV, 3*U0"
      n_phiu0_bus1 = "n:phiu0:bus1", "deg, Zero-Sequence Voltage, Angle"
      n_phiu0_bus2 = "n:phiu0:bus2", "deg, Zero-Sequence Voltage, Angle"
      n_u1_bus1 = "n:u1:bus1", "p.u., Positive-Sequence Voltage, Magnitude"
      n_u1_bus2 = "n:u1:bus2", "p.u., Positive-Sequence Voltage, Magnitude"
      n_u1pc_bus1 = "n:u1pc:bus1", "%, Positive-Sequence Voltage, Magnitude"
      n_u1pc_bus2 = "n:u1pc:bus2", "%, Positive-Sequence Voltage, Magnitude"
      n_u1r_bus1 = "n:u1r:bus1", "p.u., Positive-Sequence Voltage, Real Part"
      n_u1r_bus2 = "n:u1r:bus2", "p.u., Positive-Sequence Voltage, Real Part"
      n_u1i_bus1 = "n:u1i:bus1", "p.u., Positive-Sequence Voltage, Imaginary Part"
      n_u1i_bus2 = "n:u1i:bus2", "p.u., Positive-Sequence Voltage, Imaginary Part"
      n_U1_bus1 = "n:U1:bus1", "kV, Line-Ground Positive-Sequence Voltage, Magnitude"
      n_U1_bus2 = "n:U1:bus2", "kV, Line-Ground Positive-Sequence Voltage, Magnitude"
      n_phiu1_bus1 = "n:phiu1:bus1", "deg, Positive-Sequence Voltage, Angle"
      n_phiu1_bus2 = "n:phiu1:bus2", "deg, Positive-Sequence Voltage, Angle"
      n_u2_bus1 = "n:u2:bus1", "p.u., Negative-Sequence Voltage, Magnitude"
      n_u2_bus2 = "n:u2:bus2", "p.u., Negative-Sequence Voltage, Magnitude"
      n_U2_bus1 = "n:U2:bus1", "kV, Line-Ground Negative-Sequence Voltage, Magnitude"
      n_U2_bus2 = "n:U2:bus2", "kV, Line-Ground Negative-Sequence Voltage, Magnitude"
      n_phiu2_bus1 = "n:phiu2:bus1", "deg, Negative-Sequence Voltage, Angle"
      n_phiu2_bus2 = "n:phiu2:bus2", "deg, Negative-Sequence Voltage, Angle"
      n_U1l_bus1 = "n:U1l:bus1", "kV, Line-Line Positive-Sequence Voltage, Magnitude"
      n_U1l_bus2 = "n:U1l:bus2", "kV, Line-Line Positive-Sequence Voltage, Magnitude"
      n_U2l_bus1 = "n:U2l:bus1", "kV, Line-Line Negative-Sequence Voltage, Magnitude"
      n_U2l_bus2 = "n:U2l:bus2", "kV, Line-Line Negative-Sequence Voltage, Magnitude"
      n_uln_bus1_A = "n:uln:bus1:A", "p.u., Line-Neutral Voltage, Magnitude"
      n_uln_bus1_B = "n:uln:bus1:B", "p.u., Line-Neutral Voltage, Magnitude"
      n_uln_bus1_C = "n:uln:bus1:C", "p.u., Line-Neutral Voltage, Magnitude"
      n_uln_bus1_N = "n:uln:bus1:N", "p.u., Line-Neutral Voltage, Magnitude"
      n_uln_bus2_A = "n:uln:bus2:A", "p.u., Line-Neutral Voltage, Magnitude"
      n_uln_bus2_B = "n:uln:bus2:B", "p.u., Line-Neutral Voltage, Magnitude"
      n_uln_bus2_C = "n:uln:bus2:C", "p.u., Line-Neutral Voltage, Magnitude"
      n_uln_bus2_N = "n:uln:bus2:N", "p.u., Line-Neutral Voltage, Magnitude"
      n_Uln_bus1_A = "n:Uln:bus1:A", "kV, Line-Neutral Voltage, Magnitude"
      n_Uln_bus1_B = "n:Uln:bus1:B", "kV, Line-Neutral Voltage, Magnitude"
      n_Uln_bus1_C = "n:Uln:bus1:C", "kV, Line-Neutral Voltage, Magnitude"
      n_Uln_bus1_N = "n:Uln:bus1:N", "kV, Line-Neutral Voltage, Magnitude"
      n_Uln_bus2_A = "n:Uln:bus2:A", "kV, Line-Neutral Voltage, Magnitude"
      n_Uln_bus2_B = "n:Uln:bus2:B", "kV, Line-Neutral Voltage, Magnitude"
      n_Uln_bus2_C = "n:Uln:bus2:C", "kV, Line-Neutral Voltage, Magnitude"
      n_Uln_bus2_N = "n:Uln:bus2:N", "kV, Line-Neutral Voltage, Magnitude"
      n_phiuln_bus1_A = "n:phiuln:bus1:A", "deg, Line-Neutral Voltage, Angle"
      n_phiuln_bus1_B = "n:phiuln:bus1:B", "deg, Line-Neutral Voltage, Angle"
      n_phiuln_bus1_C = "n:phiuln:bus1:C", "deg, Line-Neutral Voltage, Angle"
      n_phiuln_bus1_N = "n:phiuln:bus1:N", "deg, Line-Neutral Voltage, Angle"
      n_phiuln_bus2_A = "n:phiuln:bus2:A", "deg, Line-Neutral Voltage, Angle"
      n_phiuln_bus2_B = "n:phiuln:bus2:B", "deg, Line-Neutral Voltage, Angle"
      n_phiuln_bus2_C = "n:phiuln:bus2:C", "deg, Line-Neutral Voltage, Angle"
      n_phiuln_bus2_N = "n:phiuln:bus2:N", "deg, Line-Neutral Voltage, Angle"
      n_un_bus1 = "n:un:bus1", "p.u., Neutral-Ground Voltage, Magnitude"
      n_un_bus2 = "n:un:bus2", "p.u., Neutral-Ground Voltage, Magnitude"
      n_Un_bus1 = "n:Un:bus1", "kV, Neutral-Ground Voltage, Magnitude"
      n_Un_bus2 = "n:Un:bus2", "kV, Neutral-Ground Voltage, Magnitude"
      n_phiun_bus1 = "n:phiun:bus1", "deg, Neutral-Ground Voltage, Angle"
      n_phiun_bus2 = "n:phiun:bus2", "deg, Neutral-Ground Voltage, Angle"
      n_upht_bus1_A = "n:upht:bus1:A", "p.u., Phase Technology dependent Voltage, Magnitude"
      n_upht_bus1_B = "n:upht:bus1:B", "p.u., Phase Technology dependent Voltage, Magnitude"
      n_upht_bus1_C = "n:upht:bus1:C", "p.u., Phase Technology dependent Voltage, Magnitude"
      n_upht_bus1_N = "n:upht:bus1:N", "p.u., Phase Technology dependent Voltage, Magnitude"
      n_upht_bus2_A = "n:upht:bus2:A", "p.u., Phase Technology dependent Voltage, Magnitude"
      n_upht_bus2_B = "n:upht:bus2:B", "p.u., Phase Technology dependent Voltage, Magnitude"
      n_upht_bus2_C = "n:upht:bus2:C", "p.u., Phase Technology dependent Voltage, Magnitude"
      n_upht_bus2_N = "n:upht:bus2:N", "p.u., Phase Technology dependent Voltage, Magnitude"
      n_Upht_bus1_A = "n:Upht:bus1:A", "kV, Phase Technology dependent Voltage, Magnitude"
      n_Upht_bus1_B = "n:Upht:bus1:B", "kV, Phase Technology dependent Voltage, Magnitude"
      n_Upht_bus1_C = "n:Upht:bus1:C", "kV, Phase Technology dependent Voltage, Magnitude"
      n_Upht_bus1_N = "n:Upht:bus1:N", "kV, Phase Technology dependent Voltage, Magnitude"
      n_Upht_bus2_A = "n:Upht:bus2:A", "kV, Phase Technology dependent Voltage, Magnitude"
      n_Upht_bus2_B = "n:Upht:bus2:B", "kV, Phase Technology dependent Voltage, Magnitude"
      n_Upht_bus2_C = "n:Upht:bus2:C", "kV, Phase Technology dependent Voltage, Magnitude"
      n_Upht_bus2_N = "n:Upht:bus2:N", "kV, Phase Technology dependent Voltage, Magnitude"
      n_um_bus1 = "n:um:bus1", "p.u., Average-Voltage, Magnitude"
      n_um_bus2 = "n:um:bus2", "p.u., Average-Voltage, Magnitude"
      n_Um_bus1 = "n:Um:bus1", "kV, Average-Voltage, Magnitude"
      n_Um_bus2 = "n:Um:bus2", "kV, Average-Voltage, Magnitude"
      n_ishcr_bus1_A = "n:ishcr:bus1:A", "p.u., Short-Circuit Current, real"
      n_ishcr_bus1_B = "n:ishcr:bus1:B", "p.u., Short-Circuit Current, real"
      n_ishcr_bus1_C = "n:ishcr:bus1:C", "p.u., Short-Circuit Current, real"
      n_ishcr_bus1_N = "n:ishcr:bus1:N", "p.u., Short-Circuit Current, real"
      n_ishcr_bus2_A = "n:ishcr:bus2:A", "p.u., Short-Circuit Current, real"
      n_ishcr_bus2_B = "n:ishcr:bus2:B", "p.u., Short-Circuit Current, real"
      n_ishcr_bus2_C = "n:ishcr:bus2:C", "p.u., Short-Circuit Current, real"
      n_ishcr_bus2_N = "n:ishcr:bus2:N", "p.u., Short-Circuit Current, real"
      n_ishci_bus1_A = "n:ishci:bus1:A", "p.u., Short-Circuit Current, imag"
      n_ishci_bus1_B = "n:ishci:bus1:B", "p.u., Short-Circuit Current, imag"
      n_ishci_bus1_C = "n:ishci:bus1:C", "p.u., Short-Circuit Current, imag"
      n_ishci_bus1_N = "n:ishci:bus1:N", "p.u., Short-Circuit Current, imag"
      n_ishci_bus2_A = "n:ishci:bus2:A", "p.u., Short-Circuit Current, imag"
      n_ishci_bus2_B = "n:ishci:bus2:B", "p.u., Short-Circuit Current, imag"
      n_ishci_bus2_C = "n:ishci:bus2:C", "p.u., Short-Circuit Current, imag"
      n_ishci_bus2_N = "n:ishci:bus2:N", "p.u., Short-Circuit Current, imag"
      n_ishc_bus1_A = "n:ishc:bus1:A", "p.u., Short-Circuit Current, magnitude"
      n_ishc_bus1_B = "n:ishc:bus1:B", "p.u., Short-Circuit Current, magnitude"
      n_ishc_bus1_C = "n:ishc:bus1:C", "p.u., Short-Circuit Current, magnitude"
      n_ishc_bus1_N = "n:ishc:bus1:N", "p.u., Short-Circuit Current, magnitude"
      n_ishc_bus2_A = "n:ishc:bus2:A", "p.u., Short-Circuit Current, magnitude"
      n_ishc_bus2_B = "n:ishc:bus2:B", "p.u., Short-Circuit Current, magnitude"
      n_ishc_bus2_C = "n:ishc:bus2:C", "p.u., Short-Circuit Current, magnitude"
      n_ishc_bus2_N = "n:ishc:bus2:N", "p.u., Short-Circuit Current, magnitude"
      n_Ishc_bus1_A = "n:Ishc:bus1:A", "kA, Short-Circuit Current, magnitude"
      n_Ishc_bus1_B = "n:Ishc:bus1:B", "kA, Short-Circuit Current, magnitude"
      n_Ishc_bus1_C = "n:Ishc:bus1:C", "kA, Short-Circuit Current, magnitude"
      n_Ishc_bus1_N = "n:Ishc:bus1:N", "kA, Short-Circuit Current, magnitude"
      n_Ishc_bus2_A = "n:Ishc:bus2:A", "kA, Short-Circuit Current, magnitude"
      n_Ishc_bus2_B = "n:Ishc:bus2:B", "kA, Short-Circuit Current, magnitude"
      n_Ishc_bus2_C = "n:Ishc:bus2:C", "kA, Short-Circuit Current, magnitude"
      n_Ishc_bus2_N = "n:Ishc:bus2:N", "kA, Short-Circuit Current, magnitude"
      n_inshc_bus1 = "n:inshc:bus1", "p.u., Neutral Short-Circuit Current, magnitude"
      n_inshc_bus2 = "n:inshc:bus2", "p.u., Neutral Short-Circuit Current, magnitude"
      n_Inshc_bus1 = "n:Inshc:bus1", "kA, Neutral Short-Circuit Current, magnitude"
      n_Inshc_bus2 = "n:Inshc:bus2", "kA, Neutral Short-Circuit Current, magnitude"
      n_fe_bus1 = "n:fe:bus1", "p.u., Electrical Frequency"
      n_fe_bus2 = "n:fe:bus2", "p.u., Electrical Frequency"
      n_frdev_bus1 = "n:frdev:bus1", "Hz, average frequency"
      n_frdev_bus2 = "n:frdev:bus2", "Hz, average frequency"
      n_dfehz_bus1 = "n:dfehz:bus1", "Hz, Deviation of the El. Frequency"
      n_dfehz_bus2 = "n:dfehz:bus2", "Hz, Deviation of the El. Frequency"
      n_dfedt_bus1 = "n:dfedt:bus1", "p.u./s, Derivative of El. Frequency"
      n_dfedt_bus2 = "n:dfedt:bus2", "p.u./s, Derivative of El. Frequency"
      n_fehz_bus1 = "n:fehz:bus1", "Hz, Electrical Frequency"
      n_fehz_bus2 = "n:fehz:bus2", "Hz, Electrical Frequency"
      n_frnom_bus1 = "n:frnom:bus1", "Hz, Nominal Frequency"
      n_frnom_bus2 = "n:frnom:bus2", "Hz, Nominal Frequency"
      c_input = "c:input", "Control Input"
      c_R_on = "c:R_on", "Ohm, On Resistance"
      c_X_on = "c:X_on", "Ohm, On Reactance"
      c_t_open = "c:t_open", "s, Opening Time"
      c_i_ctrl = "c:i_ctrl", "Internal"
      c_Inom = "c:Inom", "kA, Rated Current"
      c_loading = "c:loading", "%, Max. Loading"
      c_loadingPh_A = "c:loadingPh:A", "%, Loading"
      c_loadingPh_B = "c:loadingPh:B", "%, Loading"
      c_loadingPh_C = "c:loadingPh:C", "%, Loading"
      c_loadingPh_N = "c:loadingPh:N", "%, Loading"
      c_closed = "c:closed", "Breaker Closed"
      c_closedPh_A = "c:closedPh:A", "Breaker Closed"
      c_closedPh_B = "c:closedPh:B", "Breaker Closed"
      c_closedPh_C = "c:closedPh:C", "Breaker Closed"
      c_closedPh_N = "c:closedPh:N", "Breaker Closed"
      c_locked = "c:locked", "Locked"
      c_IdFailure = "c:IdFailure", "Indication Failure"
      c_Indication = "c:Indication", "Indication"
      c_TCM = "c:TCM", "Trip Coil Monitor"

    class ElmGenstat(Enum):
      _init_ = 'value __doc__'

      m_u0_bus1 = "m:u0:bus1", "p.u., Zero-Sequence-Voltage, Magnitude"
      m_u0_bus1n = "m:u0:bus1n", "p.u., Zero-Sequence-Voltage, Magnitude"
      m_phiu0_bus1 = "m:phiu0:bus1", "deg, Zero-Sequence-Voltage, Angle"
      m_phiu0_bus1n = "m:phiu0:bus1n", "deg, Zero-Sequence-Voltage, Angle"
      m_u1r_bus1 = "m:u1r:bus1", "p.u., Positive-Sequence-Voltage, Real Part"
      m_u1r_bus1n = "m:u1r:bus1n", "p.u., Positive-Sequence-Voltage, Real Part"
      m_u1i_bus1 = "m:u1i:bus1", "p.u., Positive-Sequence-Voltage, Imaginary Part"
      m_u1i_bus1n = "m:u1i:bus1n", "p.u., Positive-Sequence-Voltage, Imaginary Part"
      m_u1_bus1 = "m:u1:bus1", "p.u., Positive-Sequence-Voltage, Magnitude"
      m_u1_bus1n = "m:u1:bus1n", "p.u., Positive-Sequence-Voltage, Magnitude"
      m_U1_bus1 = "m:U1:bus1", "kV, Line-Ground Positive-Sequence-Voltage, Magnitude"
      m_U1_bus1n = "m:U1:bus1n", "kV, Line-Ground Positive-Sequence-Voltage, Magnitude"
      m_U1l_bus1 = "m:U1l:bus1", "kV, Line-Line Positive-Sequence-Voltage, Magnitude"
      m_U1l_bus1n = "m:U1l:bus1n", "kV, Line-Line Positive-Sequence-Voltage, Magnitude"
      m_phiu1_bus1 = "m:phiu1:bus1", "deg, Positive-Sequence-Voltage, Angle"
      m_phiu1_bus1n = "m:phiu1:bus1n", "deg, Positive-Sequence-Voltage, Angle"
      m_u2_bus1 = "m:u2:bus1", "p.u., Negative-Sequence-Voltage, Magnitude"
      m_u2_bus1n = "m:u2:bus1n", "p.u., Negative-Sequence-Voltage, Magnitude"
      m_phiu2_bus1 = "m:phiu2:bus1", "deg, Negative-Sequence-Voltage, Angle"
      m_phiu2_bus1n = "m:phiu2:bus1n", "deg, Negative-Sequence-Voltage, Angle"
      m_inet_bus1_A = "m:inet:bus1:A", "p.u., Phase Current, Magnitude"
      m_inet_bus1_B = "m:inet:bus1:B", "p.u., Phase Current, Magnitude"
      m_inet_bus1_C = "m:inet:bus1:C", "p.u., Phase Current, Magnitude"
      m_inet_bus1_N = "m:inet:bus1:N", "p.u., Phase Current, Magnitude"
      m_inet_bus1n = "m:inet:bus1n", "p.u., Phase Current, Magnitude"
      m_I_bus1_A = "m:I:bus1:A", "kA, Phase Current, Magnitude"
      m_I_bus1_B = "m:I:bus1:B", "kA, Phase Current, Magnitude"
      m_I_bus1_C = "m:I:bus1:C", "kA, Phase Current, Magnitude"
      m_I_bus1_N = "m:I:bus1:N", "kA, Phase Current, Magnitude"
      m_I_bus1n = "m:I:bus1n", "kA, Phase Current, Magnitude"
      m_phii_bus1_A = "m:phii:bus1:A", "deg, Phase Current, Angle"
      m_phii_bus1_B = "m:phii:bus1:B", "deg, Phase Current, Angle"
      m_phii_bus1_C = "m:phii:bus1:C", "deg, Phase Current, Angle"
      m_phii_bus1_N = "m:phii:bus1:N", "deg, Phase Current, Angle"
      m_phii_bus1n = "m:phii:bus1n", "deg, Phase Current, Angle"
      m_i0r_bus1 = "m:i0r:bus1", "p.u., Zero-Sequence Current, Real Part"
      m_i0r_bus1n = "m:i0r:bus1n", "p.u., Zero-Sequence Current, Real Part"
      m_i0i_bus1 = "m:i0i:bus1", "p.u., Zero-Sequence Current, Imaginary Part"
      m_i0i_bus1n = "m:i0i:bus1n", "p.u., Zero-Sequence Current, Imaginary Part"
      m_i0_bus1 = "m:i0:bus1", "p.u., Zero-Sequence Current, Magnitude"
      m_i0_bus1n = "m:i0:bus1n", "p.u., Zero-Sequence Current, Magnitude"
      m_I0_bus1 = "m:I0:bus1", "kA, Zero-Sequence Current, Magnitude"
      m_I0_bus1n = "m:I0:bus1n", "kA, Zero-Sequence Current, Magnitude"
      m_I0x3_bus1 = "m:I0x3:bus1", "kA, 3*I0"
      m_I0x3_bus1n = "m:I0x3:bus1n", "kA, 3*I0"
      m_phii0_bus1 = "m:phii0:bus1", "deg, Zero-Sequence Current, Angle"
      m_phii0_bus1n = "m:phii0:bus1n", "deg, Zero-Sequence Current, Angle"
      m_i1r_bus1 = "m:i1r:bus1", "p.u., Positive-Sequence Current, Real Part"
      m_i1r_bus1n = "m:i1r:bus1n", "p.u., Positive-Sequence Current, Real Part"
      m_i1i_bus1 = "m:i1i:bus1", "p.u., Positive-Sequence Current, Imaginary Part"
      m_i1i_bus1n = "m:i1i:bus1n", "p.u., Positive-Sequence Current, Imaginary Part"
      m_i1_bus1 = "m:i1:bus1", "p.u., Positive-Sequence Current, Magnitude"
      m_i1_bus1n = "m:i1:bus1n", "p.u., Positive-Sequence Current, Magnitude"
      m_I1_bus1 = "m:I1:bus1", "kA, Positive-Sequence Current, Magnitude"
      m_I1_bus1n = "m:I1:bus1n", "kA, Positive-Sequence Current, Magnitude"
      m_phii1_bus1 = "m:phii1:bus1", "deg, Positive-Sequence Current, Angle"
      m_phii1_bus1n = "m:phii1:bus1n", "deg, Positive-Sequence Current, Angle"
      m_i2r_bus1 = "m:i2r:bus1", "p.u., Negative-Sequence Current, Real Part"
      m_i2r_bus1n = "m:i2r:bus1n", "p.u., Negative-Sequence Current, Real Part"
      m_i2i_bus1 = "m:i2i:bus1", "p.u., Negative-Sequence Current, Imaginary Part"
      m_i2i_bus1n = "m:i2i:bus1n", "p.u., Negative-Sequence Current, Imaginary Part"
      m_i2_bus1 = "m:i2:bus1", "p.u., Negative-Sequence Current, Magnitude"
      m_i2_bus1n = "m:i2:bus1n", "p.u., Negative-Sequence Current, Magnitude"
      m_I2_bus1 = "m:I2:bus1", "kA, Negative-Sequence Current, Magnitude"
      m_I2_bus1n = "m:I2:bus1n", "kA, Negative-Sequence Current, Magnitude"
      m_phii2_bus1 = "m:phii2:bus1", "deg, Negative-Sequence Current, Angle"
      m_phii2_bus1n = "m:phii2:bus1n", "deg, Negative-Sequence Current, Angle"
      m_P_bus1_A = "m:P:bus1:A", "MW, Active Power"
      m_P_bus1_B = "m:P:bus1:B", "MW, Active Power"
      m_P_bus1_C = "m:P:bus1:C", "MW, Active Power"
      m_P_bus1_N = "m:P:bus1:N", "MW, Active Power"
      m_P_bus1n = "m:P:bus1n", "MW, Active Power"
      m_Q_bus1_A = "m:Q:bus1:A", "Mvar, Reactive Power"
      m_Q_bus1_B = "m:Q:bus1:B", "Mvar, Reactive Power"
      m_Q_bus1_C = "m:Q:bus1:C", "Mvar, Reactive Power"
      m_Q_bus1_N = "m:Q:bus1:N", "Mvar, Reactive Power"
      m_Q_bus1n = "m:Q:bus1n", "Mvar, Reactive Power"
      m_S_bus1_A = "m:S:bus1:A", "MVA, Apparent Power"
      m_S_bus1_B = "m:S:bus1:B", "MVA, Apparent Power"
      m_S_bus1_C = "m:S:bus1:C", "MVA, Apparent Power"
      m_S_bus1_N = "m:S:bus1:N", "MVA, Apparent Power"
      m_S_bus1n = "m:S:bus1n", "MVA, Apparent Power"
      m_cosphi_bus1_A = "m:cosphi:bus1:A", "Power Factor"
      m_cosphi_bus1_B = "m:cosphi:bus1:B", "Power Factor"
      m_cosphi_bus1_C = "m:cosphi:bus1:C", "Power Factor"
      m_cosphi_bus1_N = "m:cosphi:bus1:N", "Power Factor"
      m_cosphi_bus1n = "m:cosphi:bus1n", "Power Factor"
      m_tanphi_bus1_A = "m:tanphi:bus1:A", "tan(phi)"
      m_tanphi_bus1_B = "m:tanphi:bus1:B", "tan(phi)"
      m_tanphi_bus1_C = "m:tanphi:bus1:C", "tan(phi)"
      m_tanphi_bus1_N = "m:tanphi:bus1:N", "tan(phi)"
      m_tanphi_bus1n = "m:tanphi:bus1n", "tan(phi)"
      m_Psum_bus1 = "m:Psum:bus1", "MW, Total Active Power"
      m_Psum_bus1n = "m:Psum:bus1n", "MW, Total Active Power"
      m_Qsum_bus1 = "m:Qsum:bus1", "Mvar, Total Reactive Power"
      m_Qsum_bus1n = "m:Qsum:bus1n", "Mvar, Total Reactive Power"
      m_Ssum_bus1 = "m:Ssum:bus1", "MVA, Total Apparent Power"
      m_Ssum_bus1n = "m:Ssum:bus1n", "MVA, Total Apparent Power"
      m_cosphisum_bus1 = "m:cosphisum:bus1", "Total Power Factor"
      m_cosphisum_bus1n = "m:cosphisum:bus1n", "Total Power Factor"
      m_tanphisum_bus1 = "m:tanphisum:bus1", "Total tan(phi)"
      m_tanphisum_bus1n = "m:tanphisum:bus1n", "Total tan(phi)"
      m_Pln_bus1_A = "m:Pln:bus1:A", "MW, Active Power (Line-Neutral)"
      m_Pln_bus1_B = "m:Pln:bus1:B", "MW, Active Power (Line-Neutral)"
      m_Pln_bus1_C = "m:Pln:bus1:C", "MW, Active Power (Line-Neutral)"
      m_Pln_bus1_N = "m:Pln:bus1:N", "MW, Active Power (Line-Neutral)"
      m_Pln_bus1n = "m:Pln:bus1n", "MW, Active Power (Line-Neutral)"
      m_Qln_bus1_A = "m:Qln:bus1:A", "Mvar, Reactive Power (Line-Neutral)"
      m_Qln_bus1_B = "m:Qln:bus1:B", "Mvar, Reactive Power (Line-Neutral)"
      m_Qln_bus1_C = "m:Qln:bus1:C", "Mvar, Reactive Power (Line-Neutral)"
      m_Qln_bus1_N = "m:Qln:bus1:N", "Mvar, Reactive Power (Line-Neutral)"
      m_Qln_bus1n = "m:Qln:bus1n", "Mvar, Reactive Power (Line-Neutral)"
      m_P1_bus1 = "m:P1:bus1", "MW, Positive Sequence Active Power"
      m_P1_bus1n = "m:P1:bus1n", "MW, Positive Sequence Active Power"
      m_Q1_bus1 = "m:Q1:bus1", "Mvar, Positive Sequence Reactive Power"
      m_Q1_bus1n = "m:Q1:bus1n", "Mvar, Positive Sequence Reactive Power"
      m_P2_bus1 = "m:P2:bus1", "MW, Negative Sequence Active Power"
      m_P2_bus1n = "m:P2:bus1n", "MW, Negative Sequence Active Power"
      m_Q2_bus1 = "m:Q2:bus1", "Mvar, Negative Sequence Reactive Power"
      m_Q2_bus1n = "m:Q2:bus1n", "Mvar, Negative Sequence Reactive Power"
      m_P0_bus1 = "m:P0:bus1", "MW, Zero Sequence Active Power"
      m_P0_bus1n = "m:P0:bus1n", "MW, Zero Sequence Active Power"
      m_Q0_bus1 = "m:Q0:bus1", "Mvar, Zero Sequence Reactive Power"
      m_Q0_bus1n = "m:Q0:bus1n", "Mvar, Zero Sequence Reactive Power"
      m_i1P_bus1 = "m:i1P:bus1", "p.u., Positive-Sequence Active Current"
      m_i1P_bus1n = "m:i1P:bus1n", "p.u., Positive-Sequence Active Current"
      m_i1Q_bus1 = "m:i1Q:bus1", "p.u., Positive-Sequence Reactive Current"
      m_i1Q_bus1n = "m:i1Q:bus1n", "p.u., Positive-Sequence Reactive Current"
      m_I1P_bus1 = "m:I1P:bus1", "kA, Positive-Sequence Active Current"
      m_I1P_bus1n = "m:I1P:bus1n", "kA, Positive-Sequence Active Current"
      m_I1Q_bus1 = "m:I1Q:bus1", "kA, Positive-Sequence Reactive Current"
      m_I1Q_bus1n = "m:I1Q:bus1n", "kA, Positive-Sequence Reactive Current"
      m_i2P_bus1 = "m:i2P:bus1", "p.u., Negative-Sequence Active Current"
      m_i2P_bus1n = "m:i2P:bus1n", "p.u., Negative-Sequence Active Current"
      m_i2Q_bus1 = "m:i2Q:bus1", "p.u., Negative-Sequence Reactive Current"
      m_i2Q_bus1n = "m:i2Q:bus1n", "p.u., Negative-Sequence Reactive Current"
      m_I2P_bus1 = "m:I2P:bus1", "kA, Negative-Sequence Active Current"
      m_I2P_bus1n = "m:I2P:bus1n", "kA, Negative-Sequence Active Current"
      m_I2Q_bus1 = "m:I2Q:bus1", "kA, Negative-Sequence Reactive Current"
      m_I2Q_bus1n = "m:I2Q:bus1n", "kA, Negative-Sequence Reactive Current"
      m_ur_bus1_A = "m:ur:bus1:A", "p.u., Phase Voltage, Real Part"
      m_ur_bus1_B = "m:ur:bus1:B", "p.u., Phase Voltage, Real Part"
      m_ur_bus1_C = "m:ur:bus1:C", "p.u., Phase Voltage, Real Part"
      m_ur_bus1_N = "m:ur:bus1:N", "p.u., Phase Voltage, Real Part"
      m_ur_bus1n = "m:ur:bus1n", "p.u., Phase Voltage, Real Part"
      m_ui_bus1_A = "m:ui:bus1:A", "p.u., Phase Voltage, Imaginary Part"
      m_ui_bus1_B = "m:ui:bus1:B", "p.u., Phase Voltage, Imaginary Part"
      m_ui_bus1_C = "m:ui:bus1:C", "p.u., Phase Voltage, Imaginary Part"
      m_ui_bus1_N = "m:ui:bus1:N", "p.u., Phase Voltage, Imaginary Part"
      m_ui_bus1n = "m:ui:bus1n", "p.u., Phase Voltage, Imaginary Part"
      m_u_bus1_A = "m:u:bus1:A", "p.u., Phase Voltage, Magnitude"
      m_u_bus1_B = "m:u:bus1:B", "p.u., Phase Voltage, Magnitude"
      m_u_bus1_C = "m:u:bus1:C", "p.u., Phase Voltage, Magnitude"
      m_u_bus1_N = "m:u:bus1:N", "p.u., Phase Voltage, Magnitude"
      m_u_bus1n = "m:u:bus1n", "p.u., Phase Voltage, Magnitude"
      m_ir_bus1_A = "m:ir:bus1:A", "p.u., Phase Current, Real Part"
      m_ir_bus1_B = "m:ir:bus1:B", "p.u., Phase Current, Real Part"
      m_ir_bus1_C = "m:ir:bus1:C", "p.u., Phase Current, Real Part"
      m_ir_bus1_N = "m:ir:bus1:N", "p.u., Phase Current, Real Part"
      m_ir_bus1n = "m:ir:bus1n", "p.u., Phase Current, Real Part"
      m_ii_bus1_A = "m:ii:bus1:A", "p.u., Phase Current, Imaginary Part"
      m_ii_bus1_B = "m:ii:bus1:B", "p.u., Phase Current, Imaginary Part"
      m_ii_bus1_C = "m:ii:bus1:C", "p.u., Phase Current, Imaginary Part"
      m_ii_bus1_N = "m:ii:bus1:N", "p.u., Phase Current, Imaginary Part"
      m_ii_bus1n = "m:ii:bus1n", "p.u., Phase Current, Imaginary Part"
      m_i_bus1_A = "m:i:bus1:A", "p.u., Phase Current, Magnitude"
      m_i_bus1_B = "m:i:bus1:B", "p.u., Phase Current, Magnitude"
      m_i_bus1_C = "m:i:bus1:C", "p.u., Phase Current, Magnitude"
      m_i_bus1_N = "m:i:bus1:N", "p.u., Phase Current, Magnitude"
      m_i_bus1n = "m:i:bus1n", "p.u., Phase Current, Magnitude"
      m_phiui_bus1_A = "m:phiui:bus1:A", "deg, Angle between Voltage and Current"
      m_phiui_bus1_B = "m:phiui:bus1:B", "deg, Angle between Voltage and Current"
      m_phiui_bus1_C = "m:phiui:bus1:C", "deg, Angle between Voltage and Current"
      m_phiui_bus1_N = "m:phiui:bus1:N", "deg, Angle between Voltage and Current"
      m_phiui_bus1n = "m:phiui:bus1n", "deg, Angle between Voltage and Current"
      m_phiu0i0_bus1 = "m:phiu0i0:bus1", "deg, Angle between Voltage and Current in zero-sequence system"
      m_phiu0i0_bus1n = "m:phiu0i0:bus1n", "deg, Angle between Voltage and Current in zero-sequence system"
      m_phiu1i1_bus1 = "m:phiu1i1:bus1", "deg, Angle between Voltage and Current in positive sequence system"
      m_phiu1i1_bus1n = "m:phiu1i1:bus1n", "deg, Angle between Voltage and Current in positive sequence system"
      m_phiu2i2_bus1 = "m:phiu2i2:bus1", "deg, Angle between Voltage and Current in negative sequence system"
      m_phiu2i2_bus1n = "m:phiu2i2:bus1n", "deg, Angle between Voltage and Current in negative sequence system"
      m_Tfct_bus1 = "m:Tfct:bus1", "s, Fault Clearing Time"
      m_Tfct_bus1n = "m:Tfct:bus1n", "s, Fault Clearing Time"
      m_TfctPh_bus1_A = "m:TfctPh:bus1:A", "s, Fault Clearing Time"
      m_TfctPh_bus1_B = "m:TfctPh:bus1:B", "s, Fault Clearing Time"
      m_TfctPh_bus1_C = "m:TfctPh:bus1:C", "s, Fault Clearing Time"
      m_TfctPh_bus1_N = "m:TfctPh:bus1:N", "s, Fault Clearing Time"
      m_TfctPh_bus1n = "m:TfctPh:bus1n", "s, Fault Clearing Time"
      m_Brkload_bus1 = "m:Brkload:bus1", "%, Max. Breaker Loading"
      m_Brkload_bus1n = "m:Brkload:bus1n", "%, Max. Breaker Loading"
      m_BrkloadPh_bus1_A = "m:BrkloadPh:bus1:A", "%, Breaker Loading"
      m_BrkloadPh_bus1_B = "m:BrkloadPh:bus1:B", "%, Breaker Loading"
      m_BrkloadPh_bus1_C = "m:BrkloadPh:bus1:C", "%, Breaker Loading"
      m_BrkloadPh_bus1_N = "m:BrkloadPh:bus1:N", "%, Breaker Loading"
      m_BrkloadPh_bus1n = "m:BrkloadPh:bus1n", "%, Breaker Loading"
      n_ur_bus1_A = "n:ur:bus1:A", "p.u., Line-Ground Voltage, Real Part"
      n_ur_bus1_B = "n:ur:bus1:B", "p.u., Line-Ground Voltage, Real Part"
      n_ur_bus1_C = "n:ur:bus1:C", "p.u., Line-Ground Voltage, Real Part"
      n_ur_bus1_N = "n:ur:bus1:N", "p.u., Line-Ground Voltage, Real Part"
      n_ur_bus1n = "n:ur:bus1n", "p.u., Line-Ground Voltage, Real Part"
      n_ui_bus1_A = "n:ui:bus1:A", "p.u., Line-Ground Voltage, Imaginary Part"
      n_ui_bus1_B = "n:ui:bus1:B", "p.u., Line-Ground Voltage, Imaginary Part"
      n_ui_bus1_C = "n:ui:bus1:C", "p.u., Line-Ground Voltage, Imaginary Part"
      n_ui_bus1_N = "n:ui:bus1:N", "p.u., Line-Ground Voltage, Imaginary Part"
      n_ui_bus1n = "n:ui:bus1n", "p.u., Line-Ground Voltage, Imaginary Part"
      n_u_bus1_A = "n:u:bus1:A", "p.u., Line-Ground Voltage, Magnitude"
      n_u_bus1_B = "n:u:bus1:B", "p.u., Line-Ground Voltage, Magnitude"
      n_u_bus1_C = "n:u:bus1:C", "p.u., Line-Ground Voltage, Magnitude"
      n_u_bus1_N = "n:u:bus1:N", "p.u., Line-Ground Voltage, Magnitude"
      n_u_bus1n = "n:u:bus1n", "p.u., Line-Ground Voltage, Magnitude"
      n_upc_bus1_A = "n:upc:bus1:A", "%, Line-Ground Voltage, Magnitude"
      n_upc_bus1_B = "n:upc:bus1:B", "%, Line-Ground Voltage, Magnitude"
      n_upc_bus1_C = "n:upc:bus1:C", "%, Line-Ground Voltage, Magnitude"
      n_upc_bus1_N = "n:upc:bus1:N", "%, Line-Ground Voltage, Magnitude"
      n_upc_bus1n = "n:upc:bus1n", "%, Line-Ground Voltage, Magnitude"
      n_U_bus1_A = "n:U:bus1:A", "kV, Line-Ground Voltage, Magnitude"
      n_U_bus1_B = "n:U:bus1:B", "kV, Line-Ground Voltage, Magnitude"
      n_U_bus1_C = "n:U:bus1:C", "kV, Line-Ground Voltage, Magnitude"
      n_U_bus1_N = "n:U:bus1:N", "kV, Line-Ground Voltage, Magnitude"
      n_U_bus1n = "n:U:bus1n", "kV, Line-Ground Voltage, Magnitude"
      n_phiu_bus1_A = "n:phiu:bus1:A", "deg, Line-Ground Voltage, Angle"
      n_phiu_bus1_B = "n:phiu:bus1:B", "deg, Line-Ground Voltage, Angle"
      n_phiu_bus1_C = "n:phiu:bus1:C", "deg, Line-Ground Voltage, Angle"
      n_phiu_bus1_N = "n:phiu:bus1:N", "deg, Line-Ground Voltage, Angle"
      n_phiu_bus1n = "n:phiu:bus1n", "deg, Line-Ground Voltage, Angle"
      n_phiurel_bus1 = "n:phiurel:bus1", "deg, Voltage, Relative Angle"
      n_phiurel_bus1n = "n:phiurel:bus1n", "deg, Voltage, Relative Angle"
      n_ul_bus1_A = "n:ul:bus1:A", "p.u., Line-Line Voltage, Magnitude"
      n_ul_bus1_B = "n:ul:bus1:B", "p.u., Line-Line Voltage, Magnitude"
      n_ul_bus1_C = "n:ul:bus1:C", "p.u., Line-Line Voltage, Magnitude"
      n_ul_bus1_N = "n:ul:bus1:N", "p.u., Line-Line Voltage, Magnitude"
      n_ul_bus1n = "n:ul:bus1n", "p.u., Line-Line Voltage, Magnitude"
      n_ulpc_bus1_A = "n:ulpc:bus1:A", "%, Line-Line Voltage, Magnitude"
      n_ulpc_bus1_B = "n:ulpc:bus1:B", "%, Line-Line Voltage, Magnitude"
      n_ulpc_bus1_C = "n:ulpc:bus1:C", "%, Line-Line Voltage, Magnitude"
      n_ulpc_bus1_N = "n:ulpc:bus1:N", "%, Line-Line Voltage, Magnitude"
      n_ulpc_bus1n = "n:ulpc:bus1n", "%, Line-Line Voltage, Magnitude"
      n_Ul_bus1_A = "n:Ul:bus1:A", "kV, Line-Line Voltage, Magnitude"
      n_Ul_bus1_B = "n:Ul:bus1:B", "kV, Line-Line Voltage, Magnitude"
      n_Ul_bus1_C = "n:Ul:bus1:C", "kV, Line-Line Voltage, Magnitude"
      n_Ul_bus1_N = "n:Ul:bus1:N", "kV, Line-Line Voltage, Magnitude"
      n_Ul_bus1n = "n:Ul:bus1n", "kV, Line-Line Voltage, Magnitude"
      n_phiul_bus1_A = "n:phiul:bus1:A", "deg, Line-Line Voltage, Angle"
      n_phiul_bus1_B = "n:phiul:bus1:B", "deg, Line-Line Voltage, Angle"
      n_phiul_bus1_C = "n:phiul:bus1:C", "deg, Line-Line Voltage, Angle"
      n_phiul_bus1_N = "n:phiul:bus1:N", "deg, Line-Line Voltage, Angle"
      n_phiul_bus1n = "n:phiul:bus1n", "deg, Line-Line Voltage, Angle"
      n_u0_bus1 = "n:u0:bus1", "p.u., Zero-Sequence Voltage, Magnitude"
      n_u0_bus1n = "n:u0:bus1n", "p.u., Zero-Sequence Voltage, Magnitude"
      n_U0_bus1 = "n:U0:bus1", "kV, Zero-Sequence Voltage, Magnitude"
      n_U0_bus1n = "n:U0:bus1n", "kV, Zero-Sequence Voltage, Magnitude"
      n_U0x3_bus1 = "n:U0x3:bus1", "kV, 3*U0"
      n_U0x3_bus1n = "n:U0x3:bus1n", "kV, 3*U0"
      n_phiu0_bus1 = "n:phiu0:bus1", "deg, Zero-Sequence Voltage, Angle"
      n_phiu0_bus1n = "n:phiu0:bus1n", "deg, Zero-Sequence Voltage, Angle"
      n_u1_bus1 = "n:u1:bus1", "p.u., Positive-Sequence Voltage, Magnitude"
      n_u1_bus1n = "n:u1:bus1n", "p.u., Positive-Sequence Voltage, Magnitude"
      n_u1pc_bus1 = "n:u1pc:bus1", "%, Positive-Sequence Voltage, Magnitude"
      n_u1pc_bus1n = "n:u1pc:bus1n", "%, Positive-Sequence Voltage, Magnitude"
      n_u1r_bus1 = "n:u1r:bus1", "p.u., Positive-Sequence Voltage, Real Part"
      n_u1r_bus1n = "n:u1r:bus1n", "p.u., Positive-Sequence Voltage, Real Part"
      n_u1i_bus1 = "n:u1i:bus1", "p.u., Positive-Sequence Voltage, Imaginary Part"
      n_u1i_bus1n = "n:u1i:bus1n", "p.u., Positive-Sequence Voltage, Imaginary Part"
      n_U1_bus1 = "n:U1:bus1", "kV, Line-Ground Positive-Sequence Voltage, Magnitude"
      n_U1_bus1n = "n:U1:bus1n", "kV, Line-Ground Positive-Sequence Voltage, Magnitude"
      n_phiu1_bus1 = "n:phiu1:bus1", "deg, Positive-Sequence Voltage, Angle"
      n_phiu1_bus1n = "n:phiu1:bus1n", "deg, Positive-Sequence Voltage, Angle"
      n_u2_bus1 = "n:u2:bus1", "p.u., Negative-Sequence Voltage, Magnitude"
      n_u2_bus1n = "n:u2:bus1n", "p.u., Negative-Sequence Voltage, Magnitude"
      n_U2_bus1 = "n:U2:bus1", "kV, Line-Ground Negative-Sequence Voltage, Magnitude"
      n_U2_bus1n = "n:U2:bus1n", "kV, Line-Ground Negative-Sequence Voltage, Magnitude"
      n_phiu2_bus1 = "n:phiu2:bus1", "deg, Negative-Sequence Voltage, Angle"
      n_phiu2_bus1n = "n:phiu2:bus1n", "deg, Negative-Sequence Voltage, Angle"
      n_U1l_bus1 = "n:U1l:bus1", "kV, Line-Line Positive-Sequence Voltage, Magnitude"
      n_U1l_bus1n = "n:U1l:bus1n", "kV, Line-Line Positive-Sequence Voltage, Magnitude"
      n_U2l_bus1 = "n:U2l:bus1", "kV, Line-Line Negative-Sequence Voltage, Magnitude"
      n_U2l_bus1n = "n:U2l:bus1n", "kV, Line-Line Negative-Sequence Voltage, Magnitude"
      n_uln_bus1_A = "n:uln:bus1:A", "p.u., Line-Neutral Voltage, Magnitude"
      n_uln_bus1_B = "n:uln:bus1:B", "p.u., Line-Neutral Voltage, Magnitude"
      n_uln_bus1_C = "n:uln:bus1:C", "p.u., Line-Neutral Voltage, Magnitude"
      n_uln_bus1_N = "n:uln:bus1:N", "p.u., Line-Neutral Voltage, Magnitude"
      n_uln_bus1n = "n:uln:bus1n", "p.u., Line-Neutral Voltage, Magnitude"
      n_Uln_bus1_A = "n:Uln:bus1:A", "kV, Line-Neutral Voltage, Magnitude"
      n_Uln_bus1_B = "n:Uln:bus1:B", "kV, Line-Neutral Voltage, Magnitude"
      n_Uln_bus1_C = "n:Uln:bus1:C", "kV, Line-Neutral Voltage, Magnitude"
      n_Uln_bus1_N = "n:Uln:bus1:N", "kV, Line-Neutral Voltage, Magnitude"
      n_Uln_bus1n = "n:Uln:bus1n", "kV, Line-Neutral Voltage, Magnitude"
      n_phiuln_bus1_A = "n:phiuln:bus1:A", "deg, Line-Neutral Voltage, Angle"
      n_phiuln_bus1_B = "n:phiuln:bus1:B", "deg, Line-Neutral Voltage, Angle"
      n_phiuln_bus1_C = "n:phiuln:bus1:C", "deg, Line-Neutral Voltage, Angle"
      n_phiuln_bus1_N = "n:phiuln:bus1:N", "deg, Line-Neutral Voltage, Angle"
      n_phiuln_bus1n = "n:phiuln:bus1n", "deg, Line-Neutral Voltage, Angle"
      n_un_bus1 = "n:un:bus1", "p.u., Neutral-Ground Voltage, Magnitude"
      n_un_bus1n = "n:un:bus1n", "p.u., Neutral-Ground Voltage, Magnitude"
      n_Un_bus1 = "n:Un:bus1", "kV, Neutral-Ground Voltage, Magnitude"
      n_Un_bus1n = "n:Un:bus1n", "kV, Neutral-Ground Voltage, Magnitude"
      n_phiun_bus1 = "n:phiun:bus1", "deg, Neutral-Ground Voltage, Angle"
      n_phiun_bus1n = "n:phiun:bus1n", "deg, Neutral-Ground Voltage, Angle"
      n_upht_bus1_A = "n:upht:bus1:A", "p.u., Phase Technology dependent Voltage, Magnitude"
      n_upht_bus1_B = "n:upht:bus1:B", "p.u., Phase Technology dependent Voltage, Magnitude"
      n_upht_bus1_C = "n:upht:bus1:C", "p.u., Phase Technology dependent Voltage, Magnitude"
      n_upht_bus1_N = "n:upht:bus1:N", "p.u., Phase Technology dependent Voltage, Magnitude"
      n_upht_bus1n = "n:upht:bus1n", "p.u., Phase Technology dependent Voltage, Magnitude"
      n_Upht_bus1_A = "n:Upht:bus1:A", "kV, Phase Technology dependent Voltage, Magnitude"
      n_Upht_bus1_B = "n:Upht:bus1:B", "kV, Phase Technology dependent Voltage, Magnitude"
      n_Upht_bus1_C = "n:Upht:bus1:C", "kV, Phase Technology dependent Voltage, Magnitude"
      n_Upht_bus1_N = "n:Upht:bus1:N", "kV, Phase Technology dependent Voltage, Magnitude"
      n_Upht_bus1n = "n:Upht:bus1n", "kV, Phase Technology dependent Voltage, Magnitude"
      n_um_bus1 = "n:um:bus1", "p.u., Average-Voltage, Magnitude"
      n_um_bus1n = "n:um:bus1n", "p.u., Average-Voltage, Magnitude"
      n_Um_bus1 = "n:Um:bus1", "kV, Average-Voltage, Magnitude"
      n_Um_bus1n = "n:Um:bus1n", "kV, Average-Voltage, Magnitude"
      n_ishcr_bus1_A = "n:ishcr:bus1:A", "p.u., Short-Circuit Current, real"
      n_ishcr_bus1_B = "n:ishcr:bus1:B", "p.u., Short-Circuit Current, real"
      n_ishcr_bus1_C = "n:ishcr:bus1:C", "p.u., Short-Circuit Current, real"
      n_ishcr_bus1_N = "n:ishcr:bus1:N", "p.u., Short-Circuit Current, real"
      n_ishcr_bus1n = "n:ishcr:bus1n", "p.u., Short-Circuit Current, real"
      n_ishci_bus1_A = "n:ishci:bus1:A", "p.u., Short-Circuit Current, imag"
      n_ishci_bus1_B = "n:ishci:bus1:B", "p.u., Short-Circuit Current, imag"
      n_ishci_bus1_C = "n:ishci:bus1:C", "p.u., Short-Circuit Current, imag"
      n_ishci_bus1_N = "n:ishci:bus1:N", "p.u., Short-Circuit Current, imag"
      n_ishci_bus1n = "n:ishci:bus1n", "p.u., Short-Circuit Current, imag"
      n_ishc_bus1_A = "n:ishc:bus1:A", "p.u., Short-Circuit Current, magnitude"
      n_ishc_bus1_B = "n:ishc:bus1:B", "p.u., Short-Circuit Current, magnitude"
      n_ishc_bus1_C = "n:ishc:bus1:C", "p.u., Short-Circuit Current, magnitude"
      n_ishc_bus1_N = "n:ishc:bus1:N", "p.u., Short-Circuit Current, magnitude"
      n_ishc_bus1n = "n:ishc:bus1n", "p.u., Short-Circuit Current, magnitude"
      n_Ishc_bus1_A = "n:Ishc:bus1:A", "kA, Short-Circuit Current, magnitude"
      n_Ishc_bus1_B = "n:Ishc:bus1:B", "kA, Short-Circuit Current, magnitude"
      n_Ishc_bus1_C = "n:Ishc:bus1:C", "kA, Short-Circuit Current, magnitude"
      n_Ishc_bus1_N = "n:Ishc:bus1:N", "kA, Short-Circuit Current, magnitude"
      n_Ishc_bus1n = "n:Ishc:bus1n", "kA, Short-Circuit Current, magnitude"
      n_inshc_bus1 = "n:inshc:bus1", "p.u., Neutral Short-Circuit Current, magnitude"
      n_inshc_bus1n = "n:inshc:bus1n", "p.u., Neutral Short-Circuit Current, magnitude"
      n_Inshc_bus1 = "n:Inshc:bus1", "kA, Neutral Short-Circuit Current, magnitude"
      n_Inshc_bus1n = "n:Inshc:bus1n", "kA, Neutral Short-Circuit Current, magnitude"
      n_fe_bus1 = "n:fe:bus1", "p.u., Electrical Frequency"
      n_fe_bus1n = "n:fe:bus1n", "p.u., Electrical Frequency"
      n_frdev_bus1 = "n:frdev:bus1", "Hz, average frequency"
      n_frdev_bus1n = "n:frdev:bus1n", "Hz, average frequency"
      n_dfehz_bus1 = "n:dfehz:bus1", "Hz, Deviation of the El. Frequency"
      n_dfehz_bus1n = "n:dfehz:bus1n", "Hz, Deviation of the El. Frequency"
      n_dfedt_bus1 = "n:dfedt:bus1", "p.u./s, Derivative of El. Frequency"
      n_dfedt_bus1n = "n:dfedt:bus1n", "p.u./s, Derivative of El. Frequency"
      n_fehz_bus1 = "n:fehz:bus1", "Hz, Electrical Frequency"
      n_fehz_bus1n = "n:fehz:bus1n", "Hz, Electrical Frequency"
      n_frnom_bus1 = "n:frnom:bus1", "Hz, Nominal Frequency"
      n_frnom_bus1n = "n:frnom:bus1n", "Hz, Nominal Frequency"
      s_f0 = "s:f0", "p.u., IN, Frequency input continuous, double"
      s_F0Hz = "s:F0Hz", "Hz, IN, Frequency, input (Hz), continuous, double"
      s_fref = "s:fref", "p.u., IN, Reference frequency, continuous, double"
      s_id_ref = "s:id_ref", "p.u., IN, d-axis current reference, continuous, double"
      s_iq_ref = "s:iq_ref", "p.u., IN, q-axis current reference, continuous, double"
      s_i2d_ref = "s:i2d_ref", "p.u., IN, Negative sequence current, d-axis reference, continuous, double"
      s_i2q_ref = "s:i2q_ref", "p.u., IN, Negative sequence current, q-axis reference, continuous, double"
      s_u1r_in = "s:u1r_in", "p.u., IN, Positive sequence voltage input, real part, continuous, double"
      s_u1i_in = "s:u1i_in", "p.u., IN, Positive sequence voltage input, imaginary part continuous, double"
      s_u2r_in = "s:u2r_in", "p.u., IN, Negative sequence voltage input, real part, continuous, double"
      s_u2i_in = "s:u2i_in", "p.u., IN, Negative sequence voltage input, imaginary part continuous, double"
      s_cosref = "s:cosref", "IN, cos(positive sequence dq-reference angle), continuous, double"
      s_sinref = "s:sinref", "IN, sin(positive sequence dq-reference angle), continuous, double"
      s_cos2ref = "s:cos2ref", "IN, cos(negative sequence dq-reference angle), continuous, double"
      s_sin2ref = "s:sin2ref", "IN, sin(negative sequence dq-reference angle), continuous, double"
      s_u0r_in = "s:u0r_in", "p.u., IN, Zero sequence voltage input, real part, continuous, double"
      s_u0i_in = "s:u0i_in", "p.u., IN, Zero sequence voltage input, imaginary part, continuous, double"
      s_xspeed = "s:xspeed", "p.u., OUT, Frequency, continuous, double"
      s_id = "s:id", "p.u., OUT, Current, d-Axis continuous, double"
      s_iq = "s:iq", "p.u., OUT, Current, q-Axis continuous, double"
      c_sgnn = "c:sgnn", "MVA, Total rated apparent power (ngnum*sgn)"
      c_cosn = "c:cosn", "Nominal power factor"
      c_hpi = "c:hpi", "rad/s, Nominal angular frequency"
      c_umin = "c:umin", "p.u., Min. operation voltage"
      c_phiu = "c:phiu", "rad, Voltage angle"
      e_iSimModel = "e:iSimModel", "Model"
      e_iAstabint = "e:iAstabint", "A-stable integration algorithm"
      e_umin = "e:umin", "p.u., Switch-off threshold"
      e_uonthr = "e:uonthr", "p.u., Switch-on threshold"
      e_Tondelay = "e:Tondelay", "s, Switch-on delay"
      e_uk = "e:uk", "%, Short circuit impedance"
      e_Pcu = "e:Pcu", "kW, Copper losses"
      e_R0toR1 = "e:R0toR1", "R0/R1 ratio"
      e_X0toX1 = "e:X0toX1", "X0/X1 ratio"
      e_pinput = "e:pinput", "Active Power signal"
      e_qinput = "e:qinput", "Reactive Power signal"
      e_iDelay = "e:iDelay", "dq-reference angle delay (if dq-reference signals not connected)"
      e_Tdelay = "e:Tdelay", "s, Delay time constant"
      e_iUseNegSeqAngle = "e:iUseNegSeqAngle", "Current source model: Use negative sequence angle for negative sequence current source"
      e_iUseZsr = "e:iUseZsr", "Use series reactor impedance, if negative sequence voltage signals are connected"
      e_r2 = "e:r2", "p.u., Resistance r2"
      e_x2 = "e:x2", "p.u., Reactance x2"

    class ElmIac(Enum):
      _init_ = 'value __doc__'

      m_u0_bus1 = "m:u0:bus1", "p.u., Zero-Sequence-Voltage, Magnitude"
      m_phiu0_bus1 = "m:phiu0:bus1", "deg, Zero-Sequence-Voltage, Angle"
      m_u1r_bus1 = "m:u1r:bus1", "p.u., Positive-Sequence-Voltage, Real Part"
      m_u1i_bus1 = "m:u1i:bus1", "p.u., Positive-Sequence-Voltage, Imaginary Part"
      m_u1_bus1 = "m:u1:bus1", "p.u., Positive-Sequence-Voltage, Magnitude"
      m_U1_bus1 = "m:U1:bus1", "kV, Line-Ground Positive-Sequence-Voltage, Magnitude"
      m_U1l_bus1 = "m:U1l:bus1", "kV, Line-Line Positive-Sequence-Voltage, Magnitude"
      m_phiu1_bus1 = "m:phiu1:bus1", "deg, Positive-Sequence-Voltage, Angle"
      m_u2_bus1 = "m:u2:bus1", "p.u., Negative-Sequence-Voltage, Magnitude"
      m_phiu2_bus1 = "m:phiu2:bus1", "deg, Negative-Sequence-Voltage, Angle"
      m_inet_bus1_A = "m:inet:bus1:A", "p.u., Phase Current, Magnitude"
      m_inet_bus1_B = "m:inet:bus1:B", "p.u., Phase Current, Magnitude"
      m_inet_bus1_C = "m:inet:bus1:C", "p.u., Phase Current, Magnitude"
      m_I_bus1_A = "m:I:bus1:A", "kA, Phase Current, Magnitude"
      m_I_bus1_B = "m:I:bus1:B", "kA, Phase Current, Magnitude"
      m_I_bus1_C = "m:I:bus1:C", "kA, Phase Current, Magnitude"
      m_phii_bus1_A = "m:phii:bus1:A", "deg, Phase Current, Angle"
      m_phii_bus1_B = "m:phii:bus1:B", "deg, Phase Current, Angle"
      m_phii_bus1_C = "m:phii:bus1:C", "deg, Phase Current, Angle"
      m_i0r_bus1 = "m:i0r:bus1", "p.u., Zero-Sequence Current, Real Part"
      m_i0i_bus1 = "m:i0i:bus1", "p.u., Zero-Sequence Current, Imaginary Part"
      m_i0_bus1 = "m:i0:bus1", "p.u., Zero-Sequence Current, Magnitude"
      m_I0_bus1 = "m:I0:bus1", "kA, Zero-Sequence Current, Magnitude"
      m_I0x3_bus1 = "m:I0x3:bus1", "kA, 3*I0"
      m_phii0_bus1 = "m:phii0:bus1", "deg, Zero-Sequence Current, Angle"
      m_i1r_bus1 = "m:i1r:bus1", "p.u., Positive-Sequence Current, Real Part"
      m_i1i_bus1 = "m:i1i:bus1", "p.u., Positive-Sequence Current, Imaginary Part"
      m_i1_bus1 = "m:i1:bus1", "p.u., Positive-Sequence Current, Magnitude"
      m_I1_bus1 = "m:I1:bus1", "kA, Positive-Sequence Current, Magnitude"
      m_phii1_bus1 = "m:phii1:bus1", "deg, Positive-Sequence Current, Angle"
      m_i2r_bus1 = "m:i2r:bus1", "p.u., Negative-Sequence Current, Real Part"
      m_i2i_bus1 = "m:i2i:bus1", "p.u., Negative-Sequence Current, Imaginary Part"
      m_i2_bus1 = "m:i2:bus1", "p.u., Negative-Sequence Current, Magnitude"
      m_I2_bus1 = "m:I2:bus1", "kA, Negative-Sequence Current, Magnitude"
      m_phii2_bus1 = "m:phii2:bus1", "deg, Negative-Sequence Current, Angle"
      m_P_bus1_A = "m:P:bus1:A", "MW, Active Power"
      m_P_bus1_B = "m:P:bus1:B", "MW, Active Power"
      m_P_bus1_C = "m:P:bus1:C", "MW, Active Power"
      m_Q_bus1_A = "m:Q:bus1:A", "Mvar, Reactive Power"
      m_Q_bus1_B = "m:Q:bus1:B", "Mvar, Reactive Power"
      m_Q_bus1_C = "m:Q:bus1:C", "Mvar, Reactive Power"
      m_S_bus1_A = "m:S:bus1:A", "MVA, Apparent Power"
      m_S_bus1_B = "m:S:bus1:B", "MVA, Apparent Power"
      m_S_bus1_C = "m:S:bus1:C", "MVA, Apparent Power"
      m_cosphi_bus1_A = "m:cosphi:bus1:A", "Power Factor"
      m_cosphi_bus1_B = "m:cosphi:bus1:B", "Power Factor"
      m_cosphi_bus1_C = "m:cosphi:bus1:C", "Power Factor"
      m_tanphi_bus1_A = "m:tanphi:bus1:A", "tan(phi)"
      m_tanphi_bus1_B = "m:tanphi:bus1:B", "tan(phi)"
      m_tanphi_bus1_C = "m:tanphi:bus1:C", "tan(phi)"
      m_Psum_bus1 = "m:Psum:bus1", "MW, Total Active Power"
      m_Qsum_bus1 = "m:Qsum:bus1", "Mvar, Total Reactive Power"
      m_Ssum_bus1 = "m:Ssum:bus1", "MVA, Total Apparent Power"
      m_cosphisum_bus1 = "m:cosphisum:bus1", "Total Power Factor"
      m_tanphisum_bus1 = "m:tanphisum:bus1", "Total tan(phi)"
      m_Pln_bus1_A = "m:Pln:bus1:A", "MW, Active Power (Line-Neutral)"
      m_Pln_bus1_B = "m:Pln:bus1:B", "MW, Active Power (Line-Neutral)"
      m_Pln_bus1_C = "m:Pln:bus1:C", "MW, Active Power (Line-Neutral)"
      m_Qln_bus1_A = "m:Qln:bus1:A", "Mvar, Reactive Power (Line-Neutral)"
      m_Qln_bus1_B = "m:Qln:bus1:B", "Mvar, Reactive Power (Line-Neutral)"
      m_Qln_bus1_C = "m:Qln:bus1:C", "Mvar, Reactive Power (Line-Neutral)"
      m_P1_bus1 = "m:P1:bus1", "MW, Positive Sequence Active Power"
      m_Q1_bus1 = "m:Q1:bus1", "Mvar, Positive Sequence Reactive Power"
      m_P2_bus1 = "m:P2:bus1", "MW, Negative Sequence Active Power"
      m_Q2_bus1 = "m:Q2:bus1", "Mvar, Negative Sequence Reactive Power"
      m_P0_bus1 = "m:P0:bus1", "MW, Zero Sequence Active Power"
      m_Q0_bus1 = "m:Q0:bus1", "Mvar, Zero Sequence Reactive Power"
      m_i1P_bus1 = "m:i1P:bus1", "p.u., Positive-Sequence Active Current"
      m_i1Q_bus1 = "m:i1Q:bus1", "p.u., Positive-Sequence Reactive Current"
      m_I1P_bus1 = "m:I1P:bus1", "kA, Positive-Sequence Active Current"
      m_I1Q_bus1 = "m:I1Q:bus1", "kA, Positive-Sequence Reactive Current"
      m_i2P_bus1 = "m:i2P:bus1", "p.u., Negative-Sequence Active Current"
      m_i2Q_bus1 = "m:i2Q:bus1", "p.u., Negative-Sequence Reactive Current"
      m_I2P_bus1 = "m:I2P:bus1", "kA, Negative-Sequence Active Current"
      m_I2Q_bus1 = "m:I2Q:bus1", "kA, Negative-Sequence Reactive Current"
      m_ur_bus1_A = "m:ur:bus1:A", "p.u., Phase Voltage, Real Part"
      m_ur_bus1_B = "m:ur:bus1:B", "p.u., Phase Voltage, Real Part"
      m_ur_bus1_C = "m:ur:bus1:C", "p.u., Phase Voltage, Real Part"
      m_ui_bus1_A = "m:ui:bus1:A", "p.u., Phase Voltage, Imaginary Part"
      m_ui_bus1_B = "m:ui:bus1:B", "p.u., Phase Voltage, Imaginary Part"
      m_ui_bus1_C = "m:ui:bus1:C", "p.u., Phase Voltage, Imaginary Part"
      m_u_bus1_A = "m:u:bus1:A", "p.u., Phase Voltage, Magnitude"
      m_u_bus1_B = "m:u:bus1:B", "p.u., Phase Voltage, Magnitude"
      m_u_bus1_C = "m:u:bus1:C", "p.u., Phase Voltage, Magnitude"
      m_ir_bus1_A = "m:ir:bus1:A", "p.u., Phase Current, Real Part"
      m_ir_bus1_B = "m:ir:bus1:B", "p.u., Phase Current, Real Part"
      m_ir_bus1_C = "m:ir:bus1:C", "p.u., Phase Current, Real Part"
      m_ii_bus1_A = "m:ii:bus1:A", "p.u., Phase Current, Imaginary Part"
      m_ii_bus1_B = "m:ii:bus1:B", "p.u., Phase Current, Imaginary Part"
      m_ii_bus1_C = "m:ii:bus1:C", "p.u., Phase Current, Imaginary Part"
      m_i_bus1_A = "m:i:bus1:A", "p.u., Phase Current, Magnitude"
      m_i_bus1_B = "m:i:bus1:B", "p.u., Phase Current, Magnitude"
      m_i_bus1_C = "m:i:bus1:C", "p.u., Phase Current, Magnitude"
      m_phiui_bus1_A = "m:phiui:bus1:A", "deg, Angle between Voltage and Current"
      m_phiui_bus1_B = "m:phiui:bus1:B", "deg, Angle between Voltage and Current"
      m_phiui_bus1_C = "m:phiui:bus1:C", "deg, Angle between Voltage and Current"
      m_phiu0i0_bus1 = "m:phiu0i0:bus1", "deg, Angle between Voltage and Current in zero-sequence system"
      m_phiu1i1_bus1 = "m:phiu1i1:bus1", "deg, Angle between Voltage and Current in positive sequence system"
      m_phiu2i2_bus1 = "m:phiu2i2:bus1", "deg, Angle between Voltage and Current in negative sequence system"
      m_Tfct_bus1 = "m:Tfct:bus1", "s, Fault Clearing Time"
      m_TfctPh_bus1_A = "m:TfctPh:bus1:A", "s, Fault Clearing Time"
      m_TfctPh_bus1_B = "m:TfctPh:bus1:B", "s, Fault Clearing Time"
      m_TfctPh_bus1_C = "m:TfctPh:bus1:C", "s, Fault Clearing Time"
      m_Brkload_bus1 = "m:Brkload:bus1", "%, Max. Breaker Loading"
      m_BrkloadPh_bus1_A = "m:BrkloadPh:bus1:A", "%, Breaker Loading"
      m_BrkloadPh_bus1_B = "m:BrkloadPh:bus1:B", "%, Breaker Loading"
      m_BrkloadPh_bus1_C = "m:BrkloadPh:bus1:C", "%, Breaker Loading"
      n_ur_bus1_A = "n:ur:bus1:A", "p.u., Line-Ground Voltage, Real Part"
      n_ur_bus1_B = "n:ur:bus1:B", "p.u., Line-Ground Voltage, Real Part"
      n_ur_bus1_C = "n:ur:bus1:C", "p.u., Line-Ground Voltage, Real Part"
      n_ui_bus1_A = "n:ui:bus1:A", "p.u., Line-Ground Voltage, Imaginary Part"
      n_ui_bus1_B = "n:ui:bus1:B", "p.u., Line-Ground Voltage, Imaginary Part"
      n_ui_bus1_C = "n:ui:bus1:C", "p.u., Line-Ground Voltage, Imaginary Part"
      n_u_bus1_A = "n:u:bus1:A", "p.u., Line-Ground Voltage, Magnitude"
      n_u_bus1_B = "n:u:bus1:B", "p.u., Line-Ground Voltage, Magnitude"
      n_u_bus1_C = "n:u:bus1:C", "p.u., Line-Ground Voltage, Magnitude"
      n_upc_bus1_A = "n:upc:bus1:A", "%, Line-Ground Voltage, Magnitude"
      n_upc_bus1_B = "n:upc:bus1:B", "%, Line-Ground Voltage, Magnitude"
      n_upc_bus1_C = "n:upc:bus1:C", "%, Line-Ground Voltage, Magnitude"
      n_U_bus1_A = "n:U:bus1:A", "kV, Line-Ground Voltage, Magnitude"
      n_U_bus1_B = "n:U:bus1:B", "kV, Line-Ground Voltage, Magnitude"
      n_U_bus1_C = "n:U:bus1:C", "kV, Line-Ground Voltage, Magnitude"
      n_phiu_bus1_A = "n:phiu:bus1:A", "deg, Line-Ground Voltage, Angle"
      n_phiu_bus1_B = "n:phiu:bus1:B", "deg, Line-Ground Voltage, Angle"
      n_phiu_bus1_C = "n:phiu:bus1:C", "deg, Line-Ground Voltage, Angle"
      n_phiurel_bus1 = "n:phiurel:bus1", "deg, Voltage, Relative Angle"
      n_ul_bus1_A = "n:ul:bus1:A", "p.u., Line-Line Voltage, Magnitude"
      n_ul_bus1_B = "n:ul:bus1:B", "p.u., Line-Line Voltage, Magnitude"
      n_ul_bus1_C = "n:ul:bus1:C", "p.u., Line-Line Voltage, Magnitude"
      n_ulpc_bus1_A = "n:ulpc:bus1:A", "%, Line-Line Voltage, Magnitude"
      n_ulpc_bus1_B = "n:ulpc:bus1:B", "%, Line-Line Voltage, Magnitude"
      n_ulpc_bus1_C = "n:ulpc:bus1:C", "%, Line-Line Voltage, Magnitude"
      n_Ul_bus1_A = "n:Ul:bus1:A", "kV, Line-Line Voltage, Magnitude"
      n_Ul_bus1_B = "n:Ul:bus1:B", "kV, Line-Line Voltage, Magnitude"
      n_Ul_bus1_C = "n:Ul:bus1:C", "kV, Line-Line Voltage, Magnitude"
      n_phiul_bus1_A = "n:phiul:bus1:A", "deg, Line-Line Voltage, Angle"
      n_phiul_bus1_B = "n:phiul:bus1:B", "deg, Line-Line Voltage, Angle"
      n_phiul_bus1_C = "n:phiul:bus1:C", "deg, Line-Line Voltage, Angle"
      n_u0_bus1 = "n:u0:bus1", "p.u., Zero-Sequence Voltage, Magnitude"
      n_U0_bus1 = "n:U0:bus1", "kV, Zero-Sequence Voltage, Magnitude"
      n_U0x3_bus1 = "n:U0x3:bus1", "kV, 3*U0"
      n_phiu0_bus1 = "n:phiu0:bus1", "deg, Zero-Sequence Voltage, Angle"
      n_u1_bus1 = "n:u1:bus1", "p.u., Positive-Sequence Voltage, Magnitude"
      n_u1pc_bus1 = "n:u1pc:bus1", "%, Positive-Sequence Voltage, Magnitude"
      n_u1r_bus1 = "n:u1r:bus1", "p.u., Positive-Sequence Voltage, Real Part"
      n_u1i_bus1 = "n:u1i:bus1", "p.u., Positive-Sequence Voltage, Imaginary Part"
      n_U1_bus1 = "n:U1:bus1", "kV, Line-Ground Positive-Sequence Voltage, Magnitude"
      n_phiu1_bus1 = "n:phiu1:bus1", "deg, Positive-Sequence Voltage, Angle"
      n_u2_bus1 = "n:u2:bus1", "p.u., Negative-Sequence Voltage, Magnitude"
      n_U2_bus1 = "n:U2:bus1", "kV, Line-Ground Negative-Sequence Voltage, Magnitude"
      n_phiu2_bus1 = "n:phiu2:bus1", "deg, Negative-Sequence Voltage, Angle"
      n_U1l_bus1 = "n:U1l:bus1", "kV, Line-Line Positive-Sequence Voltage, Magnitude"
      n_U2l_bus1 = "n:U2l:bus1", "kV, Line-Line Negative-Sequence Voltage, Magnitude"
      n_uln_bus1_A = "n:uln:bus1:A", "p.u., Line-Neutral Voltage, Magnitude"
      n_uln_bus1_B = "n:uln:bus1:B", "p.u., Line-Neutral Voltage, Magnitude"
      n_uln_bus1_C = "n:uln:bus1:C", "p.u., Line-Neutral Voltage, Magnitude"
      n_Uln_bus1_A = "n:Uln:bus1:A", "kV, Line-Neutral Voltage, Magnitude"
      n_Uln_bus1_B = "n:Uln:bus1:B", "kV, Line-Neutral Voltage, Magnitude"
      n_Uln_bus1_C = "n:Uln:bus1:C", "kV, Line-Neutral Voltage, Magnitude"
      n_phiuln_bus1_A = "n:phiuln:bus1:A", "deg, Line-Neutral Voltage, Angle"
      n_phiuln_bus1_B = "n:phiuln:bus1:B", "deg, Line-Neutral Voltage, Angle"
      n_phiuln_bus1_C = "n:phiuln:bus1:C", "deg, Line-Neutral Voltage, Angle"
      n_un_bus1 = "n:un:bus1", "p.u., Neutral-Ground Voltage, Magnitude"
      n_Un_bus1 = "n:Un:bus1", "kV, Neutral-Ground Voltage, Magnitude"
      n_phiun_bus1 = "n:phiun:bus1", "deg, Neutral-Ground Voltage, Angle"
      n_upht_bus1_A = "n:upht:bus1:A", "p.u., Phase Technology dependent Voltage, Magnitude"
      n_upht_bus1_B = "n:upht:bus1:B", "p.u., Phase Technology dependent Voltage, Magnitude"
      n_upht_bus1_C = "n:upht:bus1:C", "p.u., Phase Technology dependent Voltage, Magnitude"
      n_Upht_bus1_A = "n:Upht:bus1:A", "kV, Phase Technology dependent Voltage, Magnitude"
      n_Upht_bus1_B = "n:Upht:bus1:B", "kV, Phase Technology dependent Voltage, Magnitude"
      n_Upht_bus1_C = "n:Upht:bus1:C", "kV, Phase Technology dependent Voltage, Magnitude"
      n_um_bus1 = "n:um:bus1", "p.u., Average-Voltage, Magnitude"
      n_Um_bus1 = "n:Um:bus1", "kV, Average-Voltage, Magnitude"
      n_ishcr_bus1_A = "n:ishcr:bus1:A", "p.u., Short-Circuit Current, real"
      n_ishcr_bus1_B = "n:ishcr:bus1:B", "p.u., Short-Circuit Current, real"
      n_ishcr_bus1_C = "n:ishcr:bus1:C", "p.u., Short-Circuit Current, real"
      n_ishci_bus1_A = "n:ishci:bus1:A", "p.u., Short-Circuit Current, imag"
      n_ishci_bus1_B = "n:ishci:bus1:B", "p.u., Short-Circuit Current, imag"
      n_ishci_bus1_C = "n:ishci:bus1:C", "p.u., Short-Circuit Current, imag"
      n_ishc_bus1_A = "n:ishc:bus1:A", "p.u., Short-Circuit Current, magnitude"
      n_ishc_bus1_B = "n:ishc:bus1:B", "p.u., Short-Circuit Current, magnitude"
      n_ishc_bus1_C = "n:ishc:bus1:C", "p.u., Short-Circuit Current, magnitude"
      n_Ishc_bus1_A = "n:Ishc:bus1:A", "kA, Short-Circuit Current, magnitude"
      n_Ishc_bus1_B = "n:Ishc:bus1:B", "kA, Short-Circuit Current, magnitude"
      n_Ishc_bus1_C = "n:Ishc:bus1:C", "kA, Short-Circuit Current, magnitude"
      n_inshc_bus1 = "n:inshc:bus1", "p.u., Neutral Short-Circuit Current, magnitude"
      n_Inshc_bus1 = "n:Inshc:bus1", "kA, Neutral Short-Circuit Current, magnitude"
      n_fe_bus1 = "n:fe:bus1", "p.u., Electrical Frequency"
      n_frdev_bus1 = "n:frdev:bus1", "Hz, average frequency"
      n_dfehz_bus1 = "n:dfehz:bus1", "Hz, Deviation of the El. Frequency"
      n_dfedt_bus1 = "n:dfedt:bus1", "p.u./s, Derivative of El. Frequency"
      n_fehz_bus1 = "n:fehz:bus1", "Hz, Electrical Frequency"
      n_frnom_bus1 = "n:frnom:bus1", "Hz, Nominal Frequency"
      s_i0 = "s:i0", "p.u., IN, Current-Input, continuous, double"
      s_I0 = "s:I0", "kA, IN, Current-Input, continuous, double"
      s_f0 = "s:f0", "p.u., IN, Frequency-Input continuous, double"
      s_F0Hz = "s:F0Hz", "Hz, IN, Frequency-Input continuous, double"
      s_I_A = "s:I_A", "kA, IN, Current, Magnitude, continuous, double"
      s_I_B = "s:I_B", "kA, IN, Current, Magnitude, continuous, double"
      s_I_C = "s:I_C", "kA, IN, Current, Magnitude, continuous, double"
      s_phii_B = "s:phii_B", "deg, IN, Current, Angle, Phase b continuous, double"
      s_phii_C = "s:phii_C", "deg, IN, Current, Angle, Phase c continuous, double"
      s_dphii = "s:dphii", "rad, IN, Additional Angle, continuous, double"
      s_fref = "s:fref", "p.u., IN, Reference Frequency, continuous, double"
      s_xspeed = "s:xspeed", "p.u., OUT, Frequency, continuous, double"
      s_phii = "s:phii", "rad, STATE, Current-Angle, continuous, double"
      s_phii_dt = "s:phii:dt", "rad/s, d/dt, Current-Angle, derivative, continuous, double"
      c_Ir = "c:Ir", "A, Nominal Current"
      c_isetp = "c:isetp", "p.u., Current Setpoint"
      c_cosini = "c:cosini", "Power Factor"
      c_hpi = "c:hpi", "rad/s, Nominal-Angular-Frequency"
      c_G1 = "c:G1", "S, Positive Sequence Conductance"
      c_C1 = "c:C1", "F, Positive Sequence Capacitance"
      c_G2 = "c:G2", "S, Negative Sequence Conductance"
      c_C2 = "c:C2", "F, Negative Sequence Capacitance"
      c_G0 = "c:G0", "S, Zero Sequence Conductance"
      c_C0 = "c:C0", "F, Zero Sequence Capacitance"
      e_iAstabint = "e:iAstabint", "A-stable integration algorithm"
      e_G1 = "e:G1", "S, Internal Conductance, G1"
      e_B1 = "e:B1", "S, Internal Susceptance, B1"
      e_G2 = "e:G2", "S, Internal Conductance, G2"
      e_B2 = "e:B2", "S, Internal Susceptance, B2"
      e_G0 = "e:G0", "S, Internal Conductance, G0"
      e_B0 = "e:B0", "S, Internal Susceptance, B0"
      e_leadIinput = "e:leadIinput", "Current input"
      e_leadFinput = "e:leadFinput", "Frequency input"

    class ElmLne(Enum):
      _init_ = 'value __doc__'

      m_u0_bus1 = "m:u0:bus1", "p.u., Zero-Sequence-Voltage, Magnitude"
      m_u0_bus2 = "m:u0:bus2", "p.u., Zero-Sequence-Voltage, Magnitude"
      m_phiu0_bus1 = "m:phiu0:bus1", "deg, Zero-Sequence-Voltage, Angle"
      m_phiu0_bus2 = "m:phiu0:bus2", "deg, Zero-Sequence-Voltage, Angle"
      m_u1r_bus1 = "m:u1r:bus1", "p.u., Positive-Sequence-Voltage, Real Part"
      m_u1r_bus2 = "m:u1r:bus2", "p.u., Positive-Sequence-Voltage, Real Part"
      m_u1i_bus1 = "m:u1i:bus1", "p.u., Positive-Sequence-Voltage, Imaginary Part"
      m_u1i_bus2 = "m:u1i:bus2", "p.u., Positive-Sequence-Voltage, Imaginary Part"
      m_u1_bus1 = "m:u1:bus1", "p.u., Positive-Sequence-Voltage, Magnitude"
      m_u1_bus2 = "m:u1:bus2", "p.u., Positive-Sequence-Voltage, Magnitude"
      m_U1_bus1 = "m:U1:bus1", "kV, Line-Ground Positive-Sequence-Voltage, Magnitude"
      m_U1_bus2 = "m:U1:bus2", "kV, Line-Ground Positive-Sequence-Voltage, Magnitude"
      m_U1l_bus1 = "m:U1l:bus1", "kV, Line-Line Positive-Sequence-Voltage, Magnitude"
      m_U1l_bus2 = "m:U1l:bus2", "kV, Line-Line Positive-Sequence-Voltage, Magnitude"
      m_phiu1_bus1 = "m:phiu1:bus1", "deg, Positive-Sequence-Voltage, Angle"
      m_phiu1_bus2 = "m:phiu1:bus2", "deg, Positive-Sequence-Voltage, Angle"
      m_u2_bus1 = "m:u2:bus1", "p.u., Negative-Sequence-Voltage, Magnitude"
      m_u2_bus2 = "m:u2:bus2", "p.u., Negative-Sequence-Voltage, Magnitude"
      m_phiu2_bus1 = "m:phiu2:bus1", "deg, Negative-Sequence-Voltage, Angle"
      m_phiu2_bus2 = "m:phiu2:bus2", "deg, Negative-Sequence-Voltage, Angle"
      m_inet_bus1_A = "m:inet:bus1:A", "p.u., Phase Current, Magnitude"
      m_inet_bus1_B = "m:inet:bus1:B", "p.u., Phase Current, Magnitude"
      m_inet_bus1_C = "m:inet:bus1:C", "p.u., Phase Current, Magnitude"
      m_inet_bus1_N = "m:inet:bus1:N", "p.u., Phase Current, Magnitude"
      m_inet_bus2_A = "m:inet:bus2:A", "p.u., Phase Current, Magnitude"
      m_inet_bus2_B = "m:inet:bus2:B", "p.u., Phase Current, Magnitude"
      m_inet_bus2_C = "m:inet:bus2:C", "p.u., Phase Current, Magnitude"
      m_inet_bus2_N = "m:inet:bus2:N", "p.u., Phase Current, Magnitude"
      m_I_bus1_A = "m:I:bus1:A", "kA, Phase Current, Magnitude"
      m_I_bus1_B = "m:I:bus1:B", "kA, Phase Current, Magnitude"
      m_I_bus1_C = "m:I:bus1:C", "kA, Phase Current, Magnitude"
      m_I_bus1_N = "m:I:bus1:N", "kA, Phase Current, Magnitude"
      m_I_bus2_A = "m:I:bus2:A", "kA, Phase Current, Magnitude"
      m_I_bus2_B = "m:I:bus2:B", "kA, Phase Current, Magnitude"
      m_I_bus2_C = "m:I:bus2:C", "kA, Phase Current, Magnitude"
      m_I_bus2_N = "m:I:bus2:N", "kA, Phase Current, Magnitude"
      m_phii_bus1_A = "m:phii:bus1:A", "deg, Phase Current, Angle"
      m_phii_bus1_B = "m:phii:bus1:B", "deg, Phase Current, Angle"
      m_phii_bus1_C = "m:phii:bus1:C", "deg, Phase Current, Angle"
      m_phii_bus1_N = "m:phii:bus1:N", "deg, Phase Current, Angle"
      m_phii_bus2_A = "m:phii:bus2:A", "deg, Phase Current, Angle"
      m_phii_bus2_B = "m:phii:bus2:B", "deg, Phase Current, Angle"
      m_phii_bus2_C = "m:phii:bus2:C", "deg, Phase Current, Angle"
      m_phii_bus2_N = "m:phii:bus2:N", "deg, Phase Current, Angle"
      m_i0r_bus1 = "m:i0r:bus1", "p.u., Zero-Sequence Current, Real Part"
      m_i0r_bus2 = "m:i0r:bus2", "p.u., Zero-Sequence Current, Real Part"
      m_i0i_bus1 = "m:i0i:bus1", "p.u., Zero-Sequence Current, Imaginary Part"
      m_i0i_bus2 = "m:i0i:bus2", "p.u., Zero-Sequence Current, Imaginary Part"
      m_i0_bus1 = "m:i0:bus1", "p.u., Zero-Sequence Current, Magnitude"
      m_i0_bus2 = "m:i0:bus2", "p.u., Zero-Sequence Current, Magnitude"
      m_I0_bus1 = "m:I0:bus1", "kA, Zero-Sequence Current, Magnitude"
      m_I0_bus2 = "m:I0:bus2", "kA, Zero-Sequence Current, Magnitude"
      m_I0x3_bus1 = "m:I0x3:bus1", "kA, 3*I0"
      m_I0x3_bus2 = "m:I0x3:bus2", "kA, 3*I0"
      m_phii0_bus1 = "m:phii0:bus1", "deg, Zero-Sequence Current, Angle"
      m_phii0_bus2 = "m:phii0:bus2", "deg, Zero-Sequence Current, Angle"
      m_i1r_bus1 = "m:i1r:bus1", "p.u., Positive-Sequence Current, Real Part"
      m_i1r_bus2 = "m:i1r:bus2", "p.u., Positive-Sequence Current, Real Part"
      m_i1i_bus1 = "m:i1i:bus1", "p.u., Positive-Sequence Current, Imaginary Part"
      m_i1i_bus2 = "m:i1i:bus2", "p.u., Positive-Sequence Current, Imaginary Part"
      m_i1_bus1 = "m:i1:bus1", "p.u., Positive-Sequence Current, Magnitude"
      m_i1_bus2 = "m:i1:bus2", "p.u., Positive-Sequence Current, Magnitude"
      m_I1_bus1 = "m:I1:bus1", "kA, Positive-Sequence Current, Magnitude"
      m_I1_bus2 = "m:I1:bus2", "kA, Positive-Sequence Current, Magnitude"
      m_phii1_bus1 = "m:phii1:bus1", "deg, Positive-Sequence Current, Angle"
      m_phii1_bus2 = "m:phii1:bus2", "deg, Positive-Sequence Current, Angle"
      m_i2r_bus1 = "m:i2r:bus1", "p.u., Negative-Sequence Current, Real Part"
      m_i2r_bus2 = "m:i2r:bus2", "p.u., Negative-Sequence Current, Real Part"
      m_i2i_bus1 = "m:i2i:bus1", "p.u., Negative-Sequence Current, Imaginary Part"
      m_i2i_bus2 = "m:i2i:bus2", "p.u., Negative-Sequence Current, Imaginary Part"
      m_i2_bus1 = "m:i2:bus1", "p.u., Negative-Sequence Current, Magnitude"
      m_i2_bus2 = "m:i2:bus2", "p.u., Negative-Sequence Current, Magnitude"
      m_I2_bus1 = "m:I2:bus1", "kA, Negative-Sequence Current, Magnitude"
      m_I2_bus2 = "m:I2:bus2", "kA, Negative-Sequence Current, Magnitude"
      m_phii2_bus1 = "m:phii2:bus1", "deg, Negative-Sequence Current, Angle"
      m_phii2_bus2 = "m:phii2:bus2", "deg, Negative-Sequence Current, Angle"
      m_P_bus1_A = "m:P:bus1:A", "MW, Active Power"
      m_P_bus1_B = "m:P:bus1:B", "MW, Active Power"
      m_P_bus1_C = "m:P:bus1:C", "MW, Active Power"
      m_P_bus1_N = "m:P:bus1:N", "MW, Active Power"
      m_P_bus2_A = "m:P:bus2:A", "MW, Active Power"
      m_P_bus2_B = "m:P:bus2:B", "MW, Active Power"
      m_P_bus2_C = "m:P:bus2:C", "MW, Active Power"
      m_P_bus2_N = "m:P:bus2:N", "MW, Active Power"
      m_Q_bus1_A = "m:Q:bus1:A", "Mvar, Reactive Power"
      m_Q_bus1_B = "m:Q:bus1:B", "Mvar, Reactive Power"
      m_Q_bus1_C = "m:Q:bus1:C", "Mvar, Reactive Power"
      m_Q_bus1_N = "m:Q:bus1:N", "Mvar, Reactive Power"
      m_Q_bus2_A = "m:Q:bus2:A", "Mvar, Reactive Power"
      m_Q_bus2_B = "m:Q:bus2:B", "Mvar, Reactive Power"
      m_Q_bus2_C = "m:Q:bus2:C", "Mvar, Reactive Power"
      m_Q_bus2_N = "m:Q:bus2:N", "Mvar, Reactive Power"
      m_S_bus1_A = "m:S:bus1:A", "MVA, Apparent Power"
      m_S_bus1_B = "m:S:bus1:B", "MVA, Apparent Power"
      m_S_bus1_C = "m:S:bus1:C", "MVA, Apparent Power"
      m_S_bus1_N = "m:S:bus1:N", "MVA, Apparent Power"
      m_S_bus2_A = "m:S:bus2:A", "MVA, Apparent Power"
      m_S_bus2_B = "m:S:bus2:B", "MVA, Apparent Power"
      m_S_bus2_C = "m:S:bus2:C", "MVA, Apparent Power"
      m_S_bus2_N = "m:S:bus2:N", "MVA, Apparent Power"
      m_cosphi_bus1_A = "m:cosphi:bus1:A", "Power Factor"
      m_cosphi_bus1_B = "m:cosphi:bus1:B", "Power Factor"
      m_cosphi_bus1_C = "m:cosphi:bus1:C", "Power Factor"
      m_cosphi_bus1_N = "m:cosphi:bus1:N", "Power Factor"
      m_cosphi_bus2_A = "m:cosphi:bus2:A", "Power Factor"
      m_cosphi_bus2_B = "m:cosphi:bus2:B", "Power Factor"
      m_cosphi_bus2_C = "m:cosphi:bus2:C", "Power Factor"
      m_cosphi_bus2_N = "m:cosphi:bus2:N", "Power Factor"
      m_tanphi_bus1_A = "m:tanphi:bus1:A", "tan(phi)"
      m_tanphi_bus1_B = "m:tanphi:bus1:B", "tan(phi)"
      m_tanphi_bus1_C = "m:tanphi:bus1:C", "tan(phi)"
      m_tanphi_bus1_N = "m:tanphi:bus1:N", "tan(phi)"
      m_tanphi_bus2_A = "m:tanphi:bus2:A", "tan(phi)"
      m_tanphi_bus2_B = "m:tanphi:bus2:B", "tan(phi)"
      m_tanphi_bus2_C = "m:tanphi:bus2:C", "tan(phi)"
      m_tanphi_bus2_N = "m:tanphi:bus2:N", "tan(phi)"
      m_Psum_bus1 = "m:Psum:bus1", "MW, Total Active Power"
      m_Psum_bus2 = "m:Psum:bus2", "MW, Total Active Power"
      m_Qsum_bus1 = "m:Qsum:bus1", "Mvar, Total Reactive Power"
      m_Qsum_bus2 = "m:Qsum:bus2", "Mvar, Total Reactive Power"
      m_Ssum_bus1 = "m:Ssum:bus1", "MVA, Total Apparent Power"
      m_Ssum_bus2 = "m:Ssum:bus2", "MVA, Total Apparent Power"
      m_cosphisum_bus1 = "m:cosphisum:bus1", "Total Power Factor"
      m_cosphisum_bus2 = "m:cosphisum:bus2", "Total Power Factor"
      m_tanphisum_bus1 = "m:tanphisum:bus1", "Total tan(phi)"
      m_tanphisum_bus2 = "m:tanphisum:bus2", "Total tan(phi)"
      m_Pln_bus1_A = "m:Pln:bus1:A", "MW, Active Power (Line-Neutral)"
      m_Pln_bus1_B = "m:Pln:bus1:B", "MW, Active Power (Line-Neutral)"
      m_Pln_bus1_C = "m:Pln:bus1:C", "MW, Active Power (Line-Neutral)"
      m_Pln_bus1_N = "m:Pln:bus1:N", "MW, Active Power (Line-Neutral)"
      m_Pln_bus2_A = "m:Pln:bus2:A", "MW, Active Power (Line-Neutral)"
      m_Pln_bus2_B = "m:Pln:bus2:B", "MW, Active Power (Line-Neutral)"
      m_Pln_bus2_C = "m:Pln:bus2:C", "MW, Active Power (Line-Neutral)"
      m_Pln_bus2_N = "m:Pln:bus2:N", "MW, Active Power (Line-Neutral)"
      m_Qln_bus1_A = "m:Qln:bus1:A", "Mvar, Reactive Power (Line-Neutral)"
      m_Qln_bus1_B = "m:Qln:bus1:B", "Mvar, Reactive Power (Line-Neutral)"
      m_Qln_bus1_C = "m:Qln:bus1:C", "Mvar, Reactive Power (Line-Neutral)"
      m_Qln_bus1_N = "m:Qln:bus1:N", "Mvar, Reactive Power (Line-Neutral)"
      m_Qln_bus2_A = "m:Qln:bus2:A", "Mvar, Reactive Power (Line-Neutral)"
      m_Qln_bus2_B = "m:Qln:bus2:B", "Mvar, Reactive Power (Line-Neutral)"
      m_Qln_bus2_C = "m:Qln:bus2:C", "Mvar, Reactive Power (Line-Neutral)"
      m_Qln_bus2_N = "m:Qln:bus2:N", "Mvar, Reactive Power (Line-Neutral)"
      m_P1_bus1 = "m:P1:bus1", "MW, Positive Sequence Active Power"
      m_P1_bus2 = "m:P1:bus2", "MW, Positive Sequence Active Power"
      m_Q1_bus1 = "m:Q1:bus1", "Mvar, Positive Sequence Reactive Power"
      m_Q1_bus2 = "m:Q1:bus2", "Mvar, Positive Sequence Reactive Power"
      m_P2_bus1 = "m:P2:bus1", "MW, Negative Sequence Active Power"
      m_P2_bus2 = "m:P2:bus2", "MW, Negative Sequence Active Power"
      m_Q2_bus1 = "m:Q2:bus1", "Mvar, Negative Sequence Reactive Power"
      m_Q2_bus2 = "m:Q2:bus2", "Mvar, Negative Sequence Reactive Power"
      m_P0_bus1 = "m:P0:bus1", "MW, Zero Sequence Active Power"
      m_P0_bus2 = "m:P0:bus2", "MW, Zero Sequence Active Power"
      m_Q0_bus1 = "m:Q0:bus1", "Mvar, Zero Sequence Reactive Power"
      m_Q0_bus2 = "m:Q0:bus2", "Mvar, Zero Sequence Reactive Power"
      m_i1P_bus1 = "m:i1P:bus1", "p.u., Positive-Sequence Active Current"
      m_i1P_bus2 = "m:i1P:bus2", "p.u., Positive-Sequence Active Current"
      m_i1Q_bus1 = "m:i1Q:bus1", "p.u., Positive-Sequence Reactive Current"
      m_i1Q_bus2 = "m:i1Q:bus2", "p.u., Positive-Sequence Reactive Current"
      m_I1P_bus1 = "m:I1P:bus1", "kA, Positive-Sequence Active Current"
      m_I1P_bus2 = "m:I1P:bus2", "kA, Positive-Sequence Active Current"
      m_I1Q_bus1 = "m:I1Q:bus1", "kA, Positive-Sequence Reactive Current"
      m_I1Q_bus2 = "m:I1Q:bus2", "kA, Positive-Sequence Reactive Current"
      m_i2P_bus1 = "m:i2P:bus1", "p.u., Negative-Sequence Active Current"
      m_i2P_bus2 = "m:i2P:bus2", "p.u., Negative-Sequence Active Current"
      m_i2Q_bus1 = "m:i2Q:bus1", "p.u., Negative-Sequence Reactive Current"
      m_i2Q_bus2 = "m:i2Q:bus2", "p.u., Negative-Sequence Reactive Current"
      m_I2P_bus1 = "m:I2P:bus1", "kA, Negative-Sequence Active Current"
      m_I2P_bus2 = "m:I2P:bus2", "kA, Negative-Sequence Active Current"
      m_I2Q_bus1 = "m:I2Q:bus1", "kA, Negative-Sequence Reactive Current"
      m_I2Q_bus2 = "m:I2Q:bus2", "kA, Negative-Sequence Reactive Current"
      m_ur_bus1_A = "m:ur:bus1:A", "p.u., Phase Voltage, Real Part"
      m_ur_bus1_B = "m:ur:bus1:B", "p.u., Phase Voltage, Real Part"
      m_ur_bus1_C = "m:ur:bus1:C", "p.u., Phase Voltage, Real Part"
      m_ur_bus1_N = "m:ur:bus1:N", "p.u., Phase Voltage, Real Part"
      m_ur_bus2_A = "m:ur:bus2:A", "p.u., Phase Voltage, Real Part"
      m_ur_bus2_B = "m:ur:bus2:B", "p.u., Phase Voltage, Real Part"
      m_ur_bus2_C = "m:ur:bus2:C", "p.u., Phase Voltage, Real Part"
      m_ur_bus2_N = "m:ur:bus2:N", "p.u., Phase Voltage, Real Part"
      m_ui_bus1_A = "m:ui:bus1:A", "p.u., Phase Voltage, Imaginary Part"
      m_ui_bus1_B = "m:ui:bus1:B", "p.u., Phase Voltage, Imaginary Part"
      m_ui_bus1_C = "m:ui:bus1:C", "p.u., Phase Voltage, Imaginary Part"
      m_ui_bus1_N = "m:ui:bus1:N", "p.u., Phase Voltage, Imaginary Part"
      m_ui_bus2_A = "m:ui:bus2:A", "p.u., Phase Voltage, Imaginary Part"
      m_ui_bus2_B = "m:ui:bus2:B", "p.u., Phase Voltage, Imaginary Part"
      m_ui_bus2_C = "m:ui:bus2:C", "p.u., Phase Voltage, Imaginary Part"
      m_ui_bus2_N = "m:ui:bus2:N", "p.u., Phase Voltage, Imaginary Part"
      m_u_bus1_A = "m:u:bus1:A", "p.u., Phase Voltage, Magnitude"
      m_u_bus1_B = "m:u:bus1:B", "p.u., Phase Voltage, Magnitude"
      m_u_bus1_C = "m:u:bus1:C", "p.u., Phase Voltage, Magnitude"
      m_u_bus1_N = "m:u:bus1:N", "p.u., Phase Voltage, Magnitude"
      m_u_bus2_A = "m:u:bus2:A", "p.u., Phase Voltage, Magnitude"
      m_u_bus2_B = "m:u:bus2:B", "p.u., Phase Voltage, Magnitude"
      m_u_bus2_C = "m:u:bus2:C", "p.u., Phase Voltage, Magnitude"
      m_u_bus2_N = "m:u:bus2:N", "p.u., Phase Voltage, Magnitude"
      m_ir_bus1_A = "m:ir:bus1:A", "p.u., Phase Current, Real Part"
      m_ir_bus1_B = "m:ir:bus1:B", "p.u., Phase Current, Real Part"
      m_ir_bus1_C = "m:ir:bus1:C", "p.u., Phase Current, Real Part"
      m_ir_bus1_N = "m:ir:bus1:N", "p.u., Phase Current, Real Part"
      m_ir_bus2_A = "m:ir:bus2:A", "p.u., Phase Current, Real Part"
      m_ir_bus2_B = "m:ir:bus2:B", "p.u., Phase Current, Real Part"
      m_ir_bus2_C = "m:ir:bus2:C", "p.u., Phase Current, Real Part"
      m_ir_bus2_N = "m:ir:bus2:N", "p.u., Phase Current, Real Part"
      m_ii_bus1_A = "m:ii:bus1:A", "p.u., Phase Current, Imaginary Part"
      m_ii_bus1_B = "m:ii:bus1:B", "p.u., Phase Current, Imaginary Part"
      m_ii_bus1_C = "m:ii:bus1:C", "p.u., Phase Current, Imaginary Part"
      m_ii_bus1_N = "m:ii:bus1:N", "p.u., Phase Current, Imaginary Part"
      m_ii_bus2_A = "m:ii:bus2:A", "p.u., Phase Current, Imaginary Part"
      m_ii_bus2_B = "m:ii:bus2:B", "p.u., Phase Current, Imaginary Part"
      m_ii_bus2_C = "m:ii:bus2:C", "p.u., Phase Current, Imaginary Part"
      m_ii_bus2_N = "m:ii:bus2:N", "p.u., Phase Current, Imaginary Part"
      m_i_bus1_A = "m:i:bus1:A", "p.u., Phase Current, Magnitude"
      m_i_bus1_B = "m:i:bus1:B", "p.u., Phase Current, Magnitude"
      m_i_bus1_C = "m:i:bus1:C", "p.u., Phase Current, Magnitude"
      m_i_bus1_N = "m:i:bus1:N", "p.u., Phase Current, Magnitude"
      m_i_bus2_A = "m:i:bus2:A", "p.u., Phase Current, Magnitude"
      m_i_bus2_B = "m:i:bus2:B", "p.u., Phase Current, Magnitude"
      m_i_bus2_C = "m:i:bus2:C", "p.u., Phase Current, Magnitude"
      m_i_bus2_N = "m:i:bus2:N", "p.u., Phase Current, Magnitude"
      m_phiui_bus1_A = "m:phiui:bus1:A", "deg, Angle between Voltage and Current"
      m_phiui_bus1_B = "m:phiui:bus1:B", "deg, Angle between Voltage and Current"
      m_phiui_bus1_C = "m:phiui:bus1:C", "deg, Angle between Voltage and Current"
      m_phiui_bus1_N = "m:phiui:bus1:N", "deg, Angle between Voltage and Current"
      m_phiui_bus2_A = "m:phiui:bus2:A", "deg, Angle between Voltage and Current"
      m_phiui_bus2_B = "m:phiui:bus2:B", "deg, Angle between Voltage and Current"
      m_phiui_bus2_C = "m:phiui:bus2:C", "deg, Angle between Voltage and Current"
      m_phiui_bus2_N = "m:phiui:bus2:N", "deg, Angle between Voltage and Current"
      m_phiu0i0_bus1 = "m:phiu0i0:bus1", "deg, Angle between Voltage and Current in zero-sequence system"
      m_phiu0i0_bus2 = "m:phiu0i0:bus2", "deg, Angle between Voltage and Current in zero-sequence system"
      m_phiu1i1_bus1 = "m:phiu1i1:bus1", "deg, Angle between Voltage and Current in positive sequence system"
      m_phiu1i1_bus2 = "m:phiu1i1:bus2", "deg, Angle between Voltage and Current in positive sequence system"
      m_phiu2i2_bus1 = "m:phiu2i2:bus1", "deg, Angle between Voltage and Current in negative sequence system"
      m_phiu2i2_bus2 = "m:phiu2i2:bus2", "deg, Angle between Voltage and Current in negative sequence system"
      m_Tfct_bus1 = "m:Tfct:bus1", "s, Fault Clearing Time"
      m_Tfct_bus2 = "m:Tfct:bus2", "s, Fault Clearing Time"
      m_TfctPh_bus1_A = "m:TfctPh:bus1:A", "s, Fault Clearing Time"
      m_TfctPh_bus1_B = "m:TfctPh:bus1:B", "s, Fault Clearing Time"
      m_TfctPh_bus1_C = "m:TfctPh:bus1:C", "s, Fault Clearing Time"
      m_TfctPh_bus1_N = "m:TfctPh:bus1:N", "s, Fault Clearing Time"
      m_TfctPh_bus2_A = "m:TfctPh:bus2:A", "s, Fault Clearing Time"
      m_TfctPh_bus2_B = "m:TfctPh:bus2:B", "s, Fault Clearing Time"
      m_TfctPh_bus2_C = "m:TfctPh:bus2:C", "s, Fault Clearing Time"
      m_TfctPh_bus2_N = "m:TfctPh:bus2:N", "s, Fault Clearing Time"
      m_Brkload_bus1 = "m:Brkload:bus1", "%, Max. Breaker Loading"
      m_Brkload_bus2 = "m:Brkload:bus2", "%, Max. Breaker Loading"
      m_BrkloadPh_bus1_A = "m:BrkloadPh:bus1:A", "%, Breaker Loading"
      m_BrkloadPh_bus1_B = "m:BrkloadPh:bus1:B", "%, Breaker Loading"
      m_BrkloadPh_bus1_C = "m:BrkloadPh:bus1:C", "%, Breaker Loading"
      m_BrkloadPh_bus1_N = "m:BrkloadPh:bus1:N", "%, Breaker Loading"
      m_BrkloadPh_bus2_A = "m:BrkloadPh:bus2:A", "%, Breaker Loading"
      m_BrkloadPh_bus2_B = "m:BrkloadPh:bus2:B", "%, Breaker Loading"
      m_BrkloadPh_bus2_C = "m:BrkloadPh:bus2:C", "%, Breaker Loading"
      m_BrkloadPh_bus2_N = "m:BrkloadPh:bus2:N", "%, Breaker Loading"
      n_ur_bus1_A = "n:ur:bus1:A", "p.u., Line-Ground Voltage, Real Part"
      n_ur_bus1_B = "n:ur:bus1:B", "p.u., Line-Ground Voltage, Real Part"
      n_ur_bus1_C = "n:ur:bus1:C", "p.u., Line-Ground Voltage, Real Part"
      n_ur_bus1_N = "n:ur:bus1:N", "p.u., Line-Ground Voltage, Real Part"
      n_ur_bus2_A = "n:ur:bus2:A", "p.u., Line-Ground Voltage, Real Part"
      n_ur_bus2_B = "n:ur:bus2:B", "p.u., Line-Ground Voltage, Real Part"
      n_ur_bus2_C = "n:ur:bus2:C", "p.u., Line-Ground Voltage, Real Part"
      n_ur_bus2_N = "n:ur:bus2:N", "p.u., Line-Ground Voltage, Real Part"
      n_ui_bus1_A = "n:ui:bus1:A", "p.u., Line-Ground Voltage, Imaginary Part"
      n_ui_bus1_B = "n:ui:bus1:B", "p.u., Line-Ground Voltage, Imaginary Part"
      n_ui_bus1_C = "n:ui:bus1:C", "p.u., Line-Ground Voltage, Imaginary Part"
      n_ui_bus1_N = "n:ui:bus1:N", "p.u., Line-Ground Voltage, Imaginary Part"
      n_ui_bus2_A = "n:ui:bus2:A", "p.u., Line-Ground Voltage, Imaginary Part"
      n_ui_bus2_B = "n:ui:bus2:B", "p.u., Line-Ground Voltage, Imaginary Part"
      n_ui_bus2_C = "n:ui:bus2:C", "p.u., Line-Ground Voltage, Imaginary Part"
      n_ui_bus2_N = "n:ui:bus2:N", "p.u., Line-Ground Voltage, Imaginary Part"
      n_u_bus1_A = "n:u:bus1:A", "p.u., Line-Ground Voltage, Magnitude"
      n_u_bus1_B = "n:u:bus1:B", "p.u., Line-Ground Voltage, Magnitude"
      n_u_bus1_C = "n:u:bus1:C", "p.u., Line-Ground Voltage, Magnitude"
      n_u_bus1_N = "n:u:bus1:N", "p.u., Line-Ground Voltage, Magnitude"
      n_u_bus2_A = "n:u:bus2:A", "p.u., Line-Ground Voltage, Magnitude"
      n_u_bus2_B = "n:u:bus2:B", "p.u., Line-Ground Voltage, Magnitude"
      n_u_bus2_C = "n:u:bus2:C", "p.u., Line-Ground Voltage, Magnitude"
      n_u_bus2_N = "n:u:bus2:N", "p.u., Line-Ground Voltage, Magnitude"
      n_upc_bus1_A = "n:upc:bus1:A", "%, Line-Ground Voltage, Magnitude"
      n_upc_bus1_B = "n:upc:bus1:B", "%, Line-Ground Voltage, Magnitude"
      n_upc_bus1_C = "n:upc:bus1:C", "%, Line-Ground Voltage, Magnitude"
      n_upc_bus1_N = "n:upc:bus1:N", "%, Line-Ground Voltage, Magnitude"
      n_upc_bus2_A = "n:upc:bus2:A", "%, Line-Ground Voltage, Magnitude"
      n_upc_bus2_B = "n:upc:bus2:B", "%, Line-Ground Voltage, Magnitude"
      n_upc_bus2_C = "n:upc:bus2:C", "%, Line-Ground Voltage, Magnitude"
      n_upc_bus2_N = "n:upc:bus2:N", "%, Line-Ground Voltage, Magnitude"
      n_U_bus1_A = "n:U:bus1:A", "kV, Line-Ground Voltage, Magnitude"
      n_U_bus1_B = "n:U:bus1:B", "kV, Line-Ground Voltage, Magnitude"
      n_U_bus1_C = "n:U:bus1:C", "kV, Line-Ground Voltage, Magnitude"
      n_U_bus1_N = "n:U:bus1:N", "kV, Line-Ground Voltage, Magnitude"
      n_U_bus2_A = "n:U:bus2:A", "kV, Line-Ground Voltage, Magnitude"
      n_U_bus2_B = "n:U:bus2:B", "kV, Line-Ground Voltage, Magnitude"
      n_U_bus2_C = "n:U:bus2:C", "kV, Line-Ground Voltage, Magnitude"
      n_U_bus2_N = "n:U:bus2:N", "kV, Line-Ground Voltage, Magnitude"
      n_phiu_bus1_A = "n:phiu:bus1:A", "deg, Line-Ground Voltage, Angle"
      n_phiu_bus1_B = "n:phiu:bus1:B", "deg, Line-Ground Voltage, Angle"
      n_phiu_bus1_C = "n:phiu:bus1:C", "deg, Line-Ground Voltage, Angle"
      n_phiu_bus1_N = "n:phiu:bus1:N", "deg, Line-Ground Voltage, Angle"
      n_phiu_bus2_A = "n:phiu:bus2:A", "deg, Line-Ground Voltage, Angle"
      n_phiu_bus2_B = "n:phiu:bus2:B", "deg, Line-Ground Voltage, Angle"
      n_phiu_bus2_C = "n:phiu:bus2:C", "deg, Line-Ground Voltage, Angle"
      n_phiu_bus2_N = "n:phiu:bus2:N", "deg, Line-Ground Voltage, Angle"
      n_phiurel_bus1 = "n:phiurel:bus1", "deg, Voltage, Relative Angle"
      n_phiurel_bus2 = "n:phiurel:bus2", "deg, Voltage, Relative Angle"
      n_ul_bus1_A = "n:ul:bus1:A", "p.u., Line-Line Voltage, Magnitude"
      n_ul_bus1_B = "n:ul:bus1:B", "p.u., Line-Line Voltage, Magnitude"
      n_ul_bus1_C = "n:ul:bus1:C", "p.u., Line-Line Voltage, Magnitude"
      n_ul_bus1_N = "n:ul:bus1:N", "p.u., Line-Line Voltage, Magnitude"
      n_ul_bus2_A = "n:ul:bus2:A", "p.u., Line-Line Voltage, Magnitude"
      n_ul_bus2_B = "n:ul:bus2:B", "p.u., Line-Line Voltage, Magnitude"
      n_ul_bus2_C = "n:ul:bus2:C", "p.u., Line-Line Voltage, Magnitude"
      n_ul_bus2_N = "n:ul:bus2:N", "p.u., Line-Line Voltage, Magnitude"
      n_ulpc_bus1_A = "n:ulpc:bus1:A", "%, Line-Line Voltage, Magnitude"
      n_ulpc_bus1_B = "n:ulpc:bus1:B", "%, Line-Line Voltage, Magnitude"
      n_ulpc_bus1_C = "n:ulpc:bus1:C", "%, Line-Line Voltage, Magnitude"
      n_ulpc_bus1_N = "n:ulpc:bus1:N", "%, Line-Line Voltage, Magnitude"
      n_ulpc_bus2_A = "n:ulpc:bus2:A", "%, Line-Line Voltage, Magnitude"
      n_ulpc_bus2_B = "n:ulpc:bus2:B", "%, Line-Line Voltage, Magnitude"
      n_ulpc_bus2_C = "n:ulpc:bus2:C", "%, Line-Line Voltage, Magnitude"
      n_ulpc_bus2_N = "n:ulpc:bus2:N", "%, Line-Line Voltage, Magnitude"
      n_Ul_bus1_A = "n:Ul:bus1:A", "kV, Line-Line Voltage, Magnitude"
      n_Ul_bus1_B = "n:Ul:bus1:B", "kV, Line-Line Voltage, Magnitude"
      n_Ul_bus1_C = "n:Ul:bus1:C", "kV, Line-Line Voltage, Magnitude"
      n_Ul_bus1_N = "n:Ul:bus1:N", "kV, Line-Line Voltage, Magnitude"
      n_Ul_bus2_A = "n:Ul:bus2:A", "kV, Line-Line Voltage, Magnitude"
      n_Ul_bus2_B = "n:Ul:bus2:B", "kV, Line-Line Voltage, Magnitude"
      n_Ul_bus2_C = "n:Ul:bus2:C", "kV, Line-Line Voltage, Magnitude"
      n_Ul_bus2_N = "n:Ul:bus2:N", "kV, Line-Line Voltage, Magnitude"
      n_phiul_bus1_A = "n:phiul:bus1:A", "deg, Line-Line Voltage, Angle"
      n_phiul_bus1_B = "n:phiul:bus1:B", "deg, Line-Line Voltage, Angle"
      n_phiul_bus1_C = "n:phiul:bus1:C", "deg, Line-Line Voltage, Angle"
      n_phiul_bus1_N = "n:phiul:bus1:N", "deg, Line-Line Voltage, Angle"
      n_phiul_bus2_A = "n:phiul:bus2:A", "deg, Line-Line Voltage, Angle"
      n_phiul_bus2_B = "n:phiul:bus2:B", "deg, Line-Line Voltage, Angle"
      n_phiul_bus2_C = "n:phiul:bus2:C", "deg, Line-Line Voltage, Angle"
      n_phiul_bus2_N = "n:phiul:bus2:N", "deg, Line-Line Voltage, Angle"
      n_u0_bus1 = "n:u0:bus1", "p.u., Zero-Sequence Voltage, Magnitude"
      n_u0_bus2 = "n:u0:bus2", "p.u., Zero-Sequence Voltage, Magnitude"
      n_U0_bus1 = "n:U0:bus1", "kV, Zero-Sequence Voltage, Magnitude"
      n_U0_bus2 = "n:U0:bus2", "kV, Zero-Sequence Voltage, Magnitude"
      n_U0x3_bus1 = "n:U0x3:bus1", "kV, 3*U0"
      n_U0x3_bus2 = "n:U0x3:bus2", "kV, 3*U0"
      n_phiu0_bus1 = "n:phiu0:bus1", "deg, Zero-Sequence Voltage, Angle"
      n_phiu0_bus2 = "n:phiu0:bus2", "deg, Zero-Sequence Voltage, Angle"
      n_u1_bus1 = "n:u1:bus1", "p.u., Positive-Sequence Voltage, Magnitude"
      n_u1_bus2 = "n:u1:bus2", "p.u., Positive-Sequence Voltage, Magnitude"
      n_u1pc_bus1 = "n:u1pc:bus1", "%, Positive-Sequence Voltage, Magnitude"
      n_u1pc_bus2 = "n:u1pc:bus2", "%, Positive-Sequence Voltage, Magnitude"
      n_u1r_bus1 = "n:u1r:bus1", "p.u., Positive-Sequence Voltage, Real Part"
      n_u1r_bus2 = "n:u1r:bus2", "p.u., Positive-Sequence Voltage, Real Part"
      n_u1i_bus1 = "n:u1i:bus1", "p.u., Positive-Sequence Voltage, Imaginary Part"
      n_u1i_bus2 = "n:u1i:bus2", "p.u., Positive-Sequence Voltage, Imaginary Part"
      n_U1_bus1 = "n:U1:bus1", "kV, Line-Ground Positive-Sequence Voltage, Magnitude"
      n_U1_bus2 = "n:U1:bus2", "kV, Line-Ground Positive-Sequence Voltage, Magnitude"
      n_phiu1_bus1 = "n:phiu1:bus1", "deg, Positive-Sequence Voltage, Angle"
      n_phiu1_bus2 = "n:phiu1:bus2", "deg, Positive-Sequence Voltage, Angle"
      n_u2_bus1 = "n:u2:bus1", "p.u., Negative-Sequence Voltage, Magnitude"
      n_u2_bus2 = "n:u2:bus2", "p.u., Negative-Sequence Voltage, Magnitude"
      n_U2_bus1 = "n:U2:bus1", "kV, Line-Ground Negative-Sequence Voltage, Magnitude"
      n_U2_bus2 = "n:U2:bus2", "kV, Line-Ground Negative-Sequence Voltage, Magnitude"
      n_phiu2_bus1 = "n:phiu2:bus1", "deg, Negative-Sequence Voltage, Angle"
      n_phiu2_bus2 = "n:phiu2:bus2", "deg, Negative-Sequence Voltage, Angle"
      n_U1l_bus1 = "n:U1l:bus1", "kV, Line-Line Positive-Sequence Voltage, Magnitude"
      n_U1l_bus2 = "n:U1l:bus2", "kV, Line-Line Positive-Sequence Voltage, Magnitude"
      n_U2l_bus1 = "n:U2l:bus1", "kV, Line-Line Negative-Sequence Voltage, Magnitude"
      n_U2l_bus2 = "n:U2l:bus2", "kV, Line-Line Negative-Sequence Voltage, Magnitude"
      n_uln_bus1_A = "n:uln:bus1:A", "p.u., Line-Neutral Voltage, Magnitude"
      n_uln_bus1_B = "n:uln:bus1:B", "p.u., Line-Neutral Voltage, Magnitude"
      n_uln_bus1_C = "n:uln:bus1:C", "p.u., Line-Neutral Voltage, Magnitude"
      n_uln_bus1_N = "n:uln:bus1:N", "p.u., Line-Neutral Voltage, Magnitude"
      n_uln_bus2_A = "n:uln:bus2:A", "p.u., Line-Neutral Voltage, Magnitude"
      n_uln_bus2_B = "n:uln:bus2:B", "p.u., Line-Neutral Voltage, Magnitude"
      n_uln_bus2_C = "n:uln:bus2:C", "p.u., Line-Neutral Voltage, Magnitude"
      n_uln_bus2_N = "n:uln:bus2:N", "p.u., Line-Neutral Voltage, Magnitude"
      n_Uln_bus1_A = "n:Uln:bus1:A", "kV, Line-Neutral Voltage, Magnitude"
      n_Uln_bus1_B = "n:Uln:bus1:B", "kV, Line-Neutral Voltage, Magnitude"
      n_Uln_bus1_C = "n:Uln:bus1:C", "kV, Line-Neutral Voltage, Magnitude"
      n_Uln_bus1_N = "n:Uln:bus1:N", "kV, Line-Neutral Voltage, Magnitude"
      n_Uln_bus2_A = "n:Uln:bus2:A", "kV, Line-Neutral Voltage, Magnitude"
      n_Uln_bus2_B = "n:Uln:bus2:B", "kV, Line-Neutral Voltage, Magnitude"
      n_Uln_bus2_C = "n:Uln:bus2:C", "kV, Line-Neutral Voltage, Magnitude"
      n_Uln_bus2_N = "n:Uln:bus2:N", "kV, Line-Neutral Voltage, Magnitude"
      n_phiuln_bus1_A = "n:phiuln:bus1:A", "deg, Line-Neutral Voltage, Angle"
      n_phiuln_bus1_B = "n:phiuln:bus1:B", "deg, Line-Neutral Voltage, Angle"
      n_phiuln_bus1_C = "n:phiuln:bus1:C", "deg, Line-Neutral Voltage, Angle"
      n_phiuln_bus1_N = "n:phiuln:bus1:N", "deg, Line-Neutral Voltage, Angle"
      n_phiuln_bus2_A = "n:phiuln:bus2:A", "deg, Line-Neutral Voltage, Angle"
      n_phiuln_bus2_B = "n:phiuln:bus2:B", "deg, Line-Neutral Voltage, Angle"
      n_phiuln_bus2_C = "n:phiuln:bus2:C", "deg, Line-Neutral Voltage, Angle"
      n_phiuln_bus2_N = "n:phiuln:bus2:N", "deg, Line-Neutral Voltage, Angle"
      n_un_bus1 = "n:un:bus1", "p.u., Neutral-Ground Voltage, Magnitude"
      n_un_bus2 = "n:un:bus2", "p.u., Neutral-Ground Voltage, Magnitude"
      n_Un_bus1 = "n:Un:bus1", "kV, Neutral-Ground Voltage, Magnitude"
      n_Un_bus2 = "n:Un:bus2", "kV, Neutral-Ground Voltage, Magnitude"
      n_phiun_bus1 = "n:phiun:bus1", "deg, Neutral-Ground Voltage, Angle"
      n_phiun_bus2 = "n:phiun:bus2", "deg, Neutral-Ground Voltage, Angle"
      n_upht_bus1_A = "n:upht:bus1:A", "p.u., Phase Technology dependent Voltage, Magnitude"
      n_upht_bus1_B = "n:upht:bus1:B", "p.u., Phase Technology dependent Voltage, Magnitude"
      n_upht_bus1_C = "n:upht:bus1:C", "p.u., Phase Technology dependent Voltage, Magnitude"
      n_upht_bus1_N = "n:upht:bus1:N", "p.u., Phase Technology dependent Voltage, Magnitude"
      n_upht_bus2_A = "n:upht:bus2:A", "p.u., Phase Technology dependent Voltage, Magnitude"
      n_upht_bus2_B = "n:upht:bus2:B", "p.u., Phase Technology dependent Voltage, Magnitude"
      n_upht_bus2_C = "n:upht:bus2:C", "p.u., Phase Technology dependent Voltage, Magnitude"
      n_upht_bus2_N = "n:upht:bus2:N", "p.u., Phase Technology dependent Voltage, Magnitude"
      n_Upht_bus1_A = "n:Upht:bus1:A", "kV, Phase Technology dependent Voltage, Magnitude"
      n_Upht_bus1_B = "n:Upht:bus1:B", "kV, Phase Technology dependent Voltage, Magnitude"
      n_Upht_bus1_C = "n:Upht:bus1:C", "kV, Phase Technology dependent Voltage, Magnitude"
      n_Upht_bus1_N = "n:Upht:bus1:N", "kV, Phase Technology dependent Voltage, Magnitude"
      n_Upht_bus2_A = "n:Upht:bus2:A", "kV, Phase Technology dependent Voltage, Magnitude"
      n_Upht_bus2_B = "n:Upht:bus2:B", "kV, Phase Technology dependent Voltage, Magnitude"
      n_Upht_bus2_C = "n:Upht:bus2:C", "kV, Phase Technology dependent Voltage, Magnitude"
      n_Upht_bus2_N = "n:Upht:bus2:N", "kV, Phase Technology dependent Voltage, Magnitude"
      n_um_bus1 = "n:um:bus1", "p.u., Average-Voltage, Magnitude"
      n_um_bus2 = "n:um:bus2", "p.u., Average-Voltage, Magnitude"
      n_Um_bus1 = "n:Um:bus1", "kV, Average-Voltage, Magnitude"
      n_Um_bus2 = "n:Um:bus2", "kV, Average-Voltage, Magnitude"
      n_ishcr_bus1_A = "n:ishcr:bus1:A", "p.u., Short-Circuit Current, real"
      n_ishcr_bus1_B = "n:ishcr:bus1:B", "p.u., Short-Circuit Current, real"
      n_ishcr_bus1_C = "n:ishcr:bus1:C", "p.u., Short-Circuit Current, real"
      n_ishcr_bus1_N = "n:ishcr:bus1:N", "p.u., Short-Circuit Current, real"
      n_ishcr_bus2_A = "n:ishcr:bus2:A", "p.u., Short-Circuit Current, real"
      n_ishcr_bus2_B = "n:ishcr:bus2:B", "p.u., Short-Circuit Current, real"
      n_ishcr_bus2_C = "n:ishcr:bus2:C", "p.u., Short-Circuit Current, real"
      n_ishcr_bus2_N = "n:ishcr:bus2:N", "p.u., Short-Circuit Current, real"
      n_ishci_bus1_A = "n:ishci:bus1:A", "p.u., Short-Circuit Current, imag"
      n_ishci_bus1_B = "n:ishci:bus1:B", "p.u., Short-Circuit Current, imag"
      n_ishci_bus1_C = "n:ishci:bus1:C", "p.u., Short-Circuit Current, imag"
      n_ishci_bus1_N = "n:ishci:bus1:N", "p.u., Short-Circuit Current, imag"
      n_ishci_bus2_A = "n:ishci:bus2:A", "p.u., Short-Circuit Current, imag"
      n_ishci_bus2_B = "n:ishci:bus2:B", "p.u., Short-Circuit Current, imag"
      n_ishci_bus2_C = "n:ishci:bus2:C", "p.u., Short-Circuit Current, imag"
      n_ishci_bus2_N = "n:ishci:bus2:N", "p.u., Short-Circuit Current, imag"
      n_ishc_bus1_A = "n:ishc:bus1:A", "p.u., Short-Circuit Current, magnitude"
      n_ishc_bus1_B = "n:ishc:bus1:B", "p.u., Short-Circuit Current, magnitude"
      n_ishc_bus1_C = "n:ishc:bus1:C", "p.u., Short-Circuit Current, magnitude"
      n_ishc_bus1_N = "n:ishc:bus1:N", "p.u., Short-Circuit Current, magnitude"
      n_ishc_bus2_A = "n:ishc:bus2:A", "p.u., Short-Circuit Current, magnitude"
      n_ishc_bus2_B = "n:ishc:bus2:B", "p.u., Short-Circuit Current, magnitude"
      n_ishc_bus2_C = "n:ishc:bus2:C", "p.u., Short-Circuit Current, magnitude"
      n_ishc_bus2_N = "n:ishc:bus2:N", "p.u., Short-Circuit Current, magnitude"
      n_Ishc_bus1_A = "n:Ishc:bus1:A", "kA, Short-Circuit Current, magnitude"
      n_Ishc_bus1_B = "n:Ishc:bus1:B", "kA, Short-Circuit Current, magnitude"
      n_Ishc_bus1_C = "n:Ishc:bus1:C", "kA, Short-Circuit Current, magnitude"
      n_Ishc_bus1_N = "n:Ishc:bus1:N", "kA, Short-Circuit Current, magnitude"
      n_Ishc_bus2_A = "n:Ishc:bus2:A", "kA, Short-Circuit Current, magnitude"
      n_Ishc_bus2_B = "n:Ishc:bus2:B", "kA, Short-Circuit Current, magnitude"
      n_Ishc_bus2_C = "n:Ishc:bus2:C", "kA, Short-Circuit Current, magnitude"
      n_Ishc_bus2_N = "n:Ishc:bus2:N", "kA, Short-Circuit Current, magnitude"
      n_inshc_bus1 = "n:inshc:bus1", "p.u., Neutral Short-Circuit Current, magnitude"
      n_inshc_bus2 = "n:inshc:bus2", "p.u., Neutral Short-Circuit Current, magnitude"
      n_Inshc_bus1 = "n:Inshc:bus1", "kA, Neutral Short-Circuit Current, magnitude"
      n_Inshc_bus2 = "n:Inshc:bus2", "kA, Neutral Short-Circuit Current, magnitude"
      n_fe_bus1 = "n:fe:bus1", "p.u., Electrical Frequency"
      n_fe_bus2 = "n:fe:bus2", "p.u., Electrical Frequency"
      n_frdev_bus1 = "n:frdev:bus1", "Hz, average frequency"
      n_frdev_bus2 = "n:frdev:bus2", "Hz, average frequency"
      n_dfehz_bus1 = "n:dfehz:bus1", "Hz, Deviation of the El. Frequency"
      n_dfehz_bus2 = "n:dfehz:bus2", "Hz, Deviation of the El. Frequency"
      n_dfedt_bus1 = "n:dfedt:bus1", "p.u./s, Derivative of El. Frequency"
      n_dfedt_bus2 = "n:dfedt:bus2", "p.u./s, Derivative of El. Frequency"
      n_fehz_bus1 = "n:fehz:bus1", "Hz, Electrical Frequency"
      n_fehz_bus2 = "n:fehz:bus2", "Hz, Electrical Frequency"
      n_frnom_bus1 = "n:frnom:bus1", "Hz, Nominal Frequency"
      n_frnom_bus2 = "n:frnom:bus2", "Hz, Nominal Frequency"
      c_zline_r = "c:zline:r", "p.u., Impedance matrix, Real Part"
      c_zline_i = "c:zline:i", "p.u., Impedance matrix, Imaginary Part"
      c_zline_phi = "c:zline:phi", "deg, Impedance matrix, Angle"
      c_zline_mag = "c:zline:mag", "p.u., Impedance matrix, Magnitude"
      c_zline_phirad = "c:zline:phirad", "rad, Impedance matrix, Angle"
      c_yline_r = "c:yline:r", "p.u., Admittance matrix, Real Part"
      c_yline_i = "c:yline:i", "p.u., Admittance matrix, Imaginary Part"
      c_yline_phi = "c:yline:phi", "deg, Admittance matrix, Angle"
      c_yline_mag = "c:yline:mag", "p.u., Admittance matrix, Magnitude"
      c_yline_phirad = "c:yline:phirad", "rad, Admittance matrix, Angle"
      c_frnom = "c:frnom", "Hz, Nominal frequency"
      c_hpi = "c:hpi", "rad/s, Nominal angular frequency"
      c_loadingPh_A = "c:loadingPh:A", "%, Loading"
      c_loadingPh_B = "c:loadingPh:B", "%, Loading"
      c_loadingPh_C = "c:loadingPh:C", "%, Loading"
      c_loadingPh_N = "c:loadingPh:N", "%, Loading"
      c_loading = "c:loading", "%, Loading, max."
      c_CloadPh_A = "c:CloadPh:A", "Mvar, Capacitive Losses"
      c_CloadPh_B = "c:CloadPh:B", "Mvar, Capacitive Losses"
      c_CloadPh_C = "c:CloadPh:C", "Mvar, Capacitive Losses"
      c_CloadPh_N = "c:CloadPh:N", "Mvar, Capacitive Losses"
      c_Cload = "c:Cload", "Mvar, Capacitive Losses, total"
      c_GloadPh_A = "c:GloadPh:A", "kW, Active no load losses"
      c_GloadPh_B = "c:GloadPh:B", "kW, Active no load losses"
      c_GloadPh_C = "c:GloadPh:C", "kW, Active no load losses"
      c_GloadPh_N = "c:GloadPh:N", "kW, Active no load losses"
      c_Gload = "c:Gload", "kW, Active no load losses, total"
      c_LossesPh_A = "c:LossesPh:A", "kW, Losses"
      c_LossesPh_B = "c:LossesPh:B", "kW, Losses"
      c_LossesPh_C = "c:LossesPh:C", "kW, Losses"
      c_LossesPh_N = "c:LossesPh:N", "kW, Losses"
      c_Losses = "c:Losses", "kW, Losses, total"
      c_curnom = "c:curnom", "kA, Nominal Current"
      c_rtfac = "c:rtfac", "Temperature Factor"
      c_Inom = "c:Inom", "kA, Nominal Current"
      e_i_const_rms = "e:i_const_rms", "Travelling wave"
      e_lossModel_rms = "e:lossModel_rms", "Loss model:R/2 at both ends"
      e_iopt_tem_rms = "e:iopt_tem_rms", "Temperature dependency"
      e_ckz0_rms = "e:ckz0_rms", "Ohm, Surge impedance, HF, mode 0."
      e_ca0dc_rms = "e:ca0dc_rms", "p.u., Wave propagation, DC, mode 0"
      e_cTa0_rms = "e:cTa0_rms", "s, Travel time, mode 0"
      e_ckz1_rms = "e:ckz1_rms", "Ohm, Surge Impedance, HF, mode 1."
      e_ca1dc_rms = "e:ca1dc_rms", "p.u., Wave propagation, DC, mode 1"
      e_cTa1_rms = "e:cTa1_rms", "s, Travel time, mode 1"
      e_ckz2_rms = "e:ckz2_rms", "Ohm, Surge Impedance, HF, mode 2."
      e_ca2dc_rms = "e:ca2dc_rms", "p.u., Wave propagation, DC, mode 2"
      e_cTa2_rms = "e:cTa2_rms", "s, Travel time, mode 2"
      t_tmax = "t:tmax", "degC, Max. Operating Temperature"
      t_rline = "t:rline", "Ohm/km, AC-Resistance R'(20°C)"
      t_rline_tmax = "t:rline_tmax", "Ohm/km, AC-Resistance R' at max. operating temperature"
      t_alpha = "t:alpha", "1/K, Temperature Coefficient"
      t_mlei = "t:mlei", "Conductor Material"
      t_bline = "t:bline", "uS/km, Susceptance B'"
      t_cline = "t:cline", "uF/km, Capacitance C'"
      t_tline = "t:tline", "Ins. Factor"
      t_gline = "t:gline", "uS/km, Conductance G'"
      t_bline0 = "t:bline0", "uS/km, Susceptance B0'"
      t_cline0 = "t:cline0", "uF/km, Capacitance C0'"
      t_Ices = "t:Ices", "A/km, Earth-Fault Current"
      t_tline0 = "t:tline0", "Ins. Factor"
      t_gline0 = "t:gline0", "uS/km, Conductance G0'"
      t_bnline = "t:bnline", "uS/km, Susceptance Bn'"
      t_cnline = "t:cnline", "uF/km, Capacitance Cn'"
      t_bpnline = "t:bpnline", "uS/km, Susceptance Bpn'"
      t_cpnline = "t:cpnline", "uF/km, Capacitance Cpn'"

    class ElmLod(Enum):
      _init_ = 'value __doc__'

      m_u0_bus1 = "m:u0:bus1", "p.u., Zero-Sequence-Voltage, Magnitude"
      m_phiu0_bus1 = "m:phiu0:bus1", "deg, Zero-Sequence-Voltage, Angle"
      m_u1r_bus1 = "m:u1r:bus1", "p.u., Positive-Sequence-Voltage, Real Part"
      m_u1i_bus1 = "m:u1i:bus1", "p.u., Positive-Sequence-Voltage, Imaginary Part"
      m_u1_bus1 = "m:u1:bus1", "p.u., Positive-Sequence-Voltage, Magnitude"
      m_U1_bus1 = "m:U1:bus1", "kV, Line-Ground Positive-Sequence-Voltage, Magnitude"
      m_U1l_bus1 = "m:U1l:bus1", "kV, Line-Line Positive-Sequence-Voltage, Magnitude"
      m_phiu1_bus1 = "m:phiu1:bus1", "deg, Positive-Sequence-Voltage, Angle"
      m_u2_bus1 = "m:u2:bus1", "p.u., Negative-Sequence-Voltage, Magnitude"
      m_phiu2_bus1 = "m:phiu2:bus1", "deg, Negative-Sequence-Voltage, Angle"
      m_inet_bus1_A = "m:inet:bus1:A", "p.u., Phase Current, Magnitude"
      m_inet_bus1_B = "m:inet:bus1:B", "p.u., Phase Current, Magnitude"
      m_inet_bus1_C = "m:inet:bus1:C", "p.u., Phase Current, Magnitude"
      m_inet_bus1_N = "m:inet:bus1:N", "p.u., Phase Current, Magnitude"
      m_I_bus1_A = "m:I:bus1:A", "kA, Phase Current, Magnitude"
      m_I_bus1_B = "m:I:bus1:B", "kA, Phase Current, Magnitude"
      m_I_bus1_C = "m:I:bus1:C", "kA, Phase Current, Magnitude"
      m_I_bus1_N = "m:I:bus1:N", "kA, Phase Current, Magnitude"
      m_phii_bus1_A = "m:phii:bus1:A", "deg, Phase Current, Angle"
      m_phii_bus1_B = "m:phii:bus1:B", "deg, Phase Current, Angle"
      m_phii_bus1_C = "m:phii:bus1:C", "deg, Phase Current, Angle"
      m_phii_bus1_N = "m:phii:bus1:N", "deg, Phase Current, Angle"
      m_i0r_bus1 = "m:i0r:bus1", "p.u., Zero-Sequence Current, Real Part"
      m_i0i_bus1 = "m:i0i:bus1", "p.u., Zero-Sequence Current, Imaginary Part"
      m_i0_bus1 = "m:i0:bus1", "p.u., Zero-Sequence Current, Magnitude"
      m_I0_bus1 = "m:I0:bus1", "kA, Zero-Sequence Current, Magnitude"
      m_I0x3_bus1 = "m:I0x3:bus1", "kA, 3*I0"
      m_phii0_bus1 = "m:phii0:bus1", "deg, Zero-Sequence Current, Angle"
      m_i1r_bus1 = "m:i1r:bus1", "p.u., Positive-Sequence Current, Real Part"
      m_i1i_bus1 = "m:i1i:bus1", "p.u., Positive-Sequence Current, Imaginary Part"
      m_i1_bus1 = "m:i1:bus1", "p.u., Positive-Sequence Current, Magnitude"
      m_I1_bus1 = "m:I1:bus1", "kA, Positive-Sequence Current, Magnitude"
      m_phii1_bus1 = "m:phii1:bus1", "deg, Positive-Sequence Current, Angle"
      m_i2r_bus1 = "m:i2r:bus1", "p.u., Negative-Sequence Current, Real Part"
      m_i2i_bus1 = "m:i2i:bus1", "p.u., Negative-Sequence Current, Imaginary Part"
      m_i2_bus1 = "m:i2:bus1", "p.u., Negative-Sequence Current, Magnitude"
      m_I2_bus1 = "m:I2:bus1", "kA, Negative-Sequence Current, Magnitude"
      m_phii2_bus1 = "m:phii2:bus1", "deg, Negative-Sequence Current, Angle"
      m_P_bus1_A = "m:P:bus1:A", "MW, Active Power"
      m_P_bus1_B = "m:P:bus1:B", "MW, Active Power"
      m_P_bus1_C = "m:P:bus1:C", "MW, Active Power"
      m_P_bus1_N = "m:P:bus1:N", "MW, Active Power"
      m_Q_bus1_A = "m:Q:bus1:A", "Mvar, Reactive Power"
      m_Q_bus1_B = "m:Q:bus1:B", "Mvar, Reactive Power"
      m_Q_bus1_C = "m:Q:bus1:C", "Mvar, Reactive Power"
      m_Q_bus1_N = "m:Q:bus1:N", "Mvar, Reactive Power"
      m_S_bus1_A = "m:S:bus1:A", "MVA, Apparent Power"
      m_S_bus1_B = "m:S:bus1:B", "MVA, Apparent Power"
      m_S_bus1_C = "m:S:bus1:C", "MVA, Apparent Power"
      m_S_bus1_N = "m:S:bus1:N", "MVA, Apparent Power"
      m_cosphi_bus1_A = "m:cosphi:bus1:A", "Power Factor"
      m_cosphi_bus1_B = "m:cosphi:bus1:B", "Power Factor"
      m_cosphi_bus1_C = "m:cosphi:bus1:C", "Power Factor"
      m_cosphi_bus1_N = "m:cosphi:bus1:N", "Power Factor"
      m_tanphi_bus1_A = "m:tanphi:bus1:A", "tan(phi)"
      m_tanphi_bus1_B = "m:tanphi:bus1:B", "tan(phi)"
      m_tanphi_bus1_C = "m:tanphi:bus1:C", "tan(phi)"
      m_tanphi_bus1_N = "m:tanphi:bus1:N", "tan(phi)"
      m_Psum_bus1 = "m:Psum:bus1", "MW, Total Active Power"
      m_Qsum_bus1 = "m:Qsum:bus1", "Mvar, Total Reactive Power"
      m_Ssum_bus1 = "m:Ssum:bus1", "MVA, Total Apparent Power"
      m_cosphisum_bus1 = "m:cosphisum:bus1", "Total Power Factor"
      m_tanphisum_bus1 = "m:tanphisum:bus1", "Total tan(phi)"
      m_Pln_bus1_A = "m:Pln:bus1:A", "MW, Active Power (Line-Neutral)"
      m_Pln_bus1_B = "m:Pln:bus1:B", "MW, Active Power (Line-Neutral)"
      m_Pln_bus1_C = "m:Pln:bus1:C", "MW, Active Power (Line-Neutral)"
      m_Pln_bus1_N = "m:Pln:bus1:N", "MW, Active Power (Line-Neutral)"
      m_Qln_bus1_A = "m:Qln:bus1:A", "Mvar, Reactive Power (Line-Neutral)"
      m_Qln_bus1_B = "m:Qln:bus1:B", "Mvar, Reactive Power (Line-Neutral)"
      m_Qln_bus1_C = "m:Qln:bus1:C", "Mvar, Reactive Power (Line-Neutral)"
      m_Qln_bus1_N = "m:Qln:bus1:N", "Mvar, Reactive Power (Line-Neutral)"
      m_P1_bus1 = "m:P1:bus1", "MW, Positive Sequence Active Power"
      m_Q1_bus1 = "m:Q1:bus1", "Mvar, Positive Sequence Reactive Power"
      m_P2_bus1 = "m:P2:bus1", "MW, Negative Sequence Active Power"
      m_Q2_bus1 = "m:Q2:bus1", "Mvar, Negative Sequence Reactive Power"
      m_P0_bus1 = "m:P0:bus1", "MW, Zero Sequence Active Power"
      m_Q0_bus1 = "m:Q0:bus1", "Mvar, Zero Sequence Reactive Power"
      m_i1P_bus1 = "m:i1P:bus1", "p.u., Positive-Sequence Active Current"
      m_i1Q_bus1 = "m:i1Q:bus1", "p.u., Positive-Sequence Reactive Current"
      m_I1P_bus1 = "m:I1P:bus1", "kA, Positive-Sequence Active Current"
      m_I1Q_bus1 = "m:I1Q:bus1", "kA, Positive-Sequence Reactive Current"
      m_i2P_bus1 = "m:i2P:bus1", "p.u., Negative-Sequence Active Current"
      m_i2Q_bus1 = "m:i2Q:bus1", "p.u., Negative-Sequence Reactive Current"
      m_I2P_bus1 = "m:I2P:bus1", "kA, Negative-Sequence Active Current"
      m_I2Q_bus1 = "m:I2Q:bus1", "kA, Negative-Sequence Reactive Current"
      m_ur_bus1_A = "m:ur:bus1:A", "p.u., Phase Voltage, Real Part"
      m_ur_bus1_B = "m:ur:bus1:B", "p.u., Phase Voltage, Real Part"
      m_ur_bus1_C = "m:ur:bus1:C", "p.u., Phase Voltage, Real Part"
      m_ur_bus1_N = "m:ur:bus1:N", "p.u., Phase Voltage, Real Part"
      m_ui_bus1_A = "m:ui:bus1:A", "p.u., Phase Voltage, Imaginary Part"
      m_ui_bus1_B = "m:ui:bus1:B", "p.u., Phase Voltage, Imaginary Part"
      m_ui_bus1_C = "m:ui:bus1:C", "p.u., Phase Voltage, Imaginary Part"
      m_ui_bus1_N = "m:ui:bus1:N", "p.u., Phase Voltage, Imaginary Part"
      m_u_bus1_A = "m:u:bus1:A", "p.u., Phase Voltage, Magnitude"
      m_u_bus1_B = "m:u:bus1:B", "p.u., Phase Voltage, Magnitude"
      m_u_bus1_C = "m:u:bus1:C", "p.u., Phase Voltage, Magnitude"
      m_u_bus1_N = "m:u:bus1:N", "p.u., Phase Voltage, Magnitude"
      m_ir_bus1_A = "m:ir:bus1:A", "p.u., Phase Current, Real Part"
      m_ir_bus1_B = "m:ir:bus1:B", "p.u., Phase Current, Real Part"
      m_ir_bus1_C = "m:ir:bus1:C", "p.u., Phase Current, Real Part"
      m_ir_bus1_N = "m:ir:bus1:N", "p.u., Phase Current, Real Part"
      m_ii_bus1_A = "m:ii:bus1:A", "p.u., Phase Current, Imaginary Part"
      m_ii_bus1_B = "m:ii:bus1:B", "p.u., Phase Current, Imaginary Part"
      m_ii_bus1_C = "m:ii:bus1:C", "p.u., Phase Current, Imaginary Part"
      m_ii_bus1_N = "m:ii:bus1:N", "p.u., Phase Current, Imaginary Part"
      m_i_bus1_A = "m:i:bus1:A", "p.u., Phase Current, Magnitude"
      m_i_bus1_B = "m:i:bus1:B", "p.u., Phase Current, Magnitude"
      m_i_bus1_C = "m:i:bus1:C", "p.u., Phase Current, Magnitude"
      m_i_bus1_N = "m:i:bus1:N", "p.u., Phase Current, Magnitude"
      m_phiui_bus1_A = "m:phiui:bus1:A", "deg, Angle between Voltage and Current"
      m_phiui_bus1_B = "m:phiui:bus1:B", "deg, Angle between Voltage and Current"
      m_phiui_bus1_C = "m:phiui:bus1:C", "deg, Angle between Voltage and Current"
      m_phiui_bus1_N = "m:phiui:bus1:N", "deg, Angle between Voltage and Current"
      m_phiu0i0_bus1 = "m:phiu0i0:bus1", "deg, Angle between Voltage and Current in zero-sequence system"
      m_phiu1i1_bus1 = "m:phiu1i1:bus1", "deg, Angle between Voltage and Current in positive sequence system"
      m_phiu2i2_bus1 = "m:phiu2i2:bus1", "deg, Angle between Voltage and Current in negative sequence system"
      m_Tfct_bus1 = "m:Tfct:bus1", "s, Fault Clearing Time"
      m_TfctPh_bus1_A = "m:TfctPh:bus1:A", "s, Fault Clearing Time"
      m_TfctPh_bus1_B = "m:TfctPh:bus1:B", "s, Fault Clearing Time"
      m_TfctPh_bus1_C = "m:TfctPh:bus1:C", "s, Fault Clearing Time"
      m_TfctPh_bus1_N = "m:TfctPh:bus1:N", "s, Fault Clearing Time"
      m_Brkload_bus1 = "m:Brkload:bus1", "%, Max. Breaker Loading"
      m_BrkloadPh_bus1_A = "m:BrkloadPh:bus1:A", "%, Breaker Loading"
      m_BrkloadPh_bus1_B = "m:BrkloadPh:bus1:B", "%, Breaker Loading"
      m_BrkloadPh_bus1_C = "m:BrkloadPh:bus1:C", "%, Breaker Loading"
      m_BrkloadPh_bus1_N = "m:BrkloadPh:bus1:N", "%, Breaker Loading"
      n_ur_bus1_A = "n:ur:bus1:A", "p.u., Line-Ground Voltage, Real Part"
      n_ur_bus1_B = "n:ur:bus1:B", "p.u., Line-Ground Voltage, Real Part"
      n_ur_bus1_C = "n:ur:bus1:C", "p.u., Line-Ground Voltage, Real Part"
      n_ur_bus1_N = "n:ur:bus1:N", "p.u., Line-Ground Voltage, Real Part"
      n_ui_bus1_A = "n:ui:bus1:A", "p.u., Line-Ground Voltage, Imaginary Part"
      n_ui_bus1_B = "n:ui:bus1:B", "p.u., Line-Ground Voltage, Imaginary Part"
      n_ui_bus1_C = "n:ui:bus1:C", "p.u., Line-Ground Voltage, Imaginary Part"
      n_ui_bus1_N = "n:ui:bus1:N", "p.u., Line-Ground Voltage, Imaginary Part"
      n_u_bus1_A = "n:u:bus1:A", "p.u., Line-Ground Voltage, Magnitude"
      n_u_bus1_B = "n:u:bus1:B", "p.u., Line-Ground Voltage, Magnitude"
      n_u_bus1_C = "n:u:bus1:C", "p.u., Line-Ground Voltage, Magnitude"
      n_u_bus1_N = "n:u:bus1:N", "p.u., Line-Ground Voltage, Magnitude"
      n_upc_bus1_A = "n:upc:bus1:A", "%, Line-Ground Voltage, Magnitude"
      n_upc_bus1_B = "n:upc:bus1:B", "%, Line-Ground Voltage, Magnitude"
      n_upc_bus1_C = "n:upc:bus1:C", "%, Line-Ground Voltage, Magnitude"
      n_upc_bus1_N = "n:upc:bus1:N", "%, Line-Ground Voltage, Magnitude"
      n_U_bus1_A = "n:U:bus1:A", "kV, Line-Ground Voltage, Magnitude"
      n_U_bus1_B = "n:U:bus1:B", "kV, Line-Ground Voltage, Magnitude"
      n_U_bus1_C = "n:U:bus1:C", "kV, Line-Ground Voltage, Magnitude"
      n_U_bus1_N = "n:U:bus1:N", "kV, Line-Ground Voltage, Magnitude"
      n_phiu_bus1_A = "n:phiu:bus1:A", "deg, Line-Ground Voltage, Angle"
      n_phiu_bus1_B = "n:phiu:bus1:B", "deg, Line-Ground Voltage, Angle"
      n_phiu_bus1_C = "n:phiu:bus1:C", "deg, Line-Ground Voltage, Angle"
      n_phiu_bus1_N = "n:phiu:bus1:N", "deg, Line-Ground Voltage, Angle"
      n_phiurel_bus1 = "n:phiurel:bus1", "deg, Voltage, Relative Angle"
      n_ul_bus1_A = "n:ul:bus1:A", "p.u., Line-Line Voltage, Magnitude"
      n_ul_bus1_B = "n:ul:bus1:B", "p.u., Line-Line Voltage, Magnitude"
      n_ul_bus1_C = "n:ul:bus1:C", "p.u., Line-Line Voltage, Magnitude"
      n_ul_bus1_N = "n:ul:bus1:N", "p.u., Line-Line Voltage, Magnitude"
      n_ulpc_bus1_A = "n:ulpc:bus1:A", "%, Line-Line Voltage, Magnitude"
      n_ulpc_bus1_B = "n:ulpc:bus1:B", "%, Line-Line Voltage, Magnitude"
      n_ulpc_bus1_C = "n:ulpc:bus1:C", "%, Line-Line Voltage, Magnitude"
      n_ulpc_bus1_N = "n:ulpc:bus1:N", "%, Line-Line Voltage, Magnitude"
      n_Ul_bus1_A = "n:Ul:bus1:A", "kV, Line-Line Voltage, Magnitude"
      n_Ul_bus1_B = "n:Ul:bus1:B", "kV, Line-Line Voltage, Magnitude"
      n_Ul_bus1_C = "n:Ul:bus1:C", "kV, Line-Line Voltage, Magnitude"
      n_Ul_bus1_N = "n:Ul:bus1:N", "kV, Line-Line Voltage, Magnitude"
      n_phiul_bus1_A = "n:phiul:bus1:A", "deg, Line-Line Voltage, Angle"
      n_phiul_bus1_B = "n:phiul:bus1:B", "deg, Line-Line Voltage, Angle"
      n_phiul_bus1_C = "n:phiul:bus1:C", "deg, Line-Line Voltage, Angle"
      n_phiul_bus1_N = "n:phiul:bus1:N", "deg, Line-Line Voltage, Angle"
      n_u0_bus1 = "n:u0:bus1", "p.u., Zero-Sequence Voltage, Magnitude"
      n_U0_bus1 = "n:U0:bus1", "kV, Zero-Sequence Voltage, Magnitude"
      n_U0x3_bus1 = "n:U0x3:bus1", "kV, 3*U0"
      n_phiu0_bus1 = "n:phiu0:bus1", "deg, Zero-Sequence Voltage, Angle"
      n_u1_bus1 = "n:u1:bus1", "p.u., Positive-Sequence Voltage, Magnitude"
      n_u1pc_bus1 = "n:u1pc:bus1", "%, Positive-Sequence Voltage, Magnitude"
      n_u1r_bus1 = "n:u1r:bus1", "p.u., Positive-Sequence Voltage, Real Part"
      n_u1i_bus1 = "n:u1i:bus1", "p.u., Positive-Sequence Voltage, Imaginary Part"
      n_U1_bus1 = "n:U1:bus1", "kV, Line-Ground Positive-Sequence Voltage, Magnitude"
      n_phiu1_bus1 = "n:phiu1:bus1", "deg, Positive-Sequence Voltage, Angle"
      n_u2_bus1 = "n:u2:bus1", "p.u., Negative-Sequence Voltage, Magnitude"
      n_U2_bus1 = "n:U2:bus1", "kV, Line-Ground Negative-Sequence Voltage, Magnitude"
      n_phiu2_bus1 = "n:phiu2:bus1", "deg, Negative-Sequence Voltage, Angle"
      n_U1l_bus1 = "n:U1l:bus1", "kV, Line-Line Positive-Sequence Voltage, Magnitude"
      n_U2l_bus1 = "n:U2l:bus1", "kV, Line-Line Negative-Sequence Voltage, Magnitude"
      n_uln_bus1_A = "n:uln:bus1:A", "p.u., Line-Neutral Voltage, Magnitude"
      n_uln_bus1_B = "n:uln:bus1:B", "p.u., Line-Neutral Voltage, Magnitude"
      n_uln_bus1_C = "n:uln:bus1:C", "p.u., Line-Neutral Voltage, Magnitude"
      n_uln_bus1_N = "n:uln:bus1:N", "p.u., Line-Neutral Voltage, Magnitude"
      n_Uln_bus1_A = "n:Uln:bus1:A", "kV, Line-Neutral Voltage, Magnitude"
      n_Uln_bus1_B = "n:Uln:bus1:B", "kV, Line-Neutral Voltage, Magnitude"
      n_Uln_bus1_C = "n:Uln:bus1:C", "kV, Line-Neutral Voltage, Magnitude"
      n_Uln_bus1_N = "n:Uln:bus1:N", "kV, Line-Neutral Voltage, Magnitude"
      n_phiuln_bus1_A = "n:phiuln:bus1:A", "deg, Line-Neutral Voltage, Angle"
      n_phiuln_bus1_B = "n:phiuln:bus1:B", "deg, Line-Neutral Voltage, Angle"
      n_phiuln_bus1_C = "n:phiuln:bus1:C", "deg, Line-Neutral Voltage, Angle"
      n_phiuln_bus1_N = "n:phiuln:bus1:N", "deg, Line-Neutral Voltage, Angle"
      n_un_bus1 = "n:un:bus1", "p.u., Neutral-Ground Voltage, Magnitude"
      n_Un_bus1 = "n:Un:bus1", "kV, Neutral-Ground Voltage, Magnitude"
      n_phiun_bus1 = "n:phiun:bus1", "deg, Neutral-Ground Voltage, Angle"
      n_upht_bus1_A = "n:upht:bus1:A", "p.u., Phase Technology dependent Voltage, Magnitude"
      n_upht_bus1_B = "n:upht:bus1:B", "p.u., Phase Technology dependent Voltage, Magnitude"
      n_upht_bus1_C = "n:upht:bus1:C", "p.u., Phase Technology dependent Voltage, Magnitude"
      n_upht_bus1_N = "n:upht:bus1:N", "p.u., Phase Technology dependent Voltage, Magnitude"
      n_Upht_bus1_A = "n:Upht:bus1:A", "kV, Phase Technology dependent Voltage, Magnitude"
      n_Upht_bus1_B = "n:Upht:bus1:B", "kV, Phase Technology dependent Voltage, Magnitude"
      n_Upht_bus1_C = "n:Upht:bus1:C", "kV, Phase Technology dependent Voltage, Magnitude"
      n_Upht_bus1_N = "n:Upht:bus1:N", "kV, Phase Technology dependent Voltage, Magnitude"
      n_um_bus1 = "n:um:bus1", "p.u., Average-Voltage, Magnitude"
      n_Um_bus1 = "n:Um:bus1", "kV, Average-Voltage, Magnitude"
      n_ishcr_bus1_A = "n:ishcr:bus1:A", "p.u., Short-Circuit Current, real"
      n_ishcr_bus1_B = "n:ishcr:bus1:B", "p.u., Short-Circuit Current, real"
      n_ishcr_bus1_C = "n:ishcr:bus1:C", "p.u., Short-Circuit Current, real"
      n_ishcr_bus1_N = "n:ishcr:bus1:N", "p.u., Short-Circuit Current, real"
      n_ishci_bus1_A = "n:ishci:bus1:A", "p.u., Short-Circuit Current, imag"
      n_ishci_bus1_B = "n:ishci:bus1:B", "p.u., Short-Circuit Current, imag"
      n_ishci_bus1_C = "n:ishci:bus1:C", "p.u., Short-Circuit Current, imag"
      n_ishci_bus1_N = "n:ishci:bus1:N", "p.u., Short-Circuit Current, imag"
      n_ishc_bus1_A = "n:ishc:bus1:A", "p.u., Short-Circuit Current, magnitude"
      n_ishc_bus1_B = "n:ishc:bus1:B", "p.u., Short-Circuit Current, magnitude"
      n_ishc_bus1_C = "n:ishc:bus1:C", "p.u., Short-Circuit Current, magnitude"
      n_ishc_bus1_N = "n:ishc:bus1:N", "p.u., Short-Circuit Current, magnitude"
      n_Ishc_bus1_A = "n:Ishc:bus1:A", "kA, Short-Circuit Current, magnitude"
      n_Ishc_bus1_B = "n:Ishc:bus1:B", "kA, Short-Circuit Current, magnitude"
      n_Ishc_bus1_C = "n:Ishc:bus1:C", "kA, Short-Circuit Current, magnitude"
      n_Ishc_bus1_N = "n:Ishc:bus1:N", "kA, Short-Circuit Current, magnitude"
      n_inshc_bus1 = "n:inshc:bus1", "p.u., Neutral Short-Circuit Current, magnitude"
      n_Inshc_bus1 = "n:Inshc:bus1", "kA, Neutral Short-Circuit Current, magnitude"
      n_fe_bus1 = "n:fe:bus1", "p.u., Electrical Frequency"
      n_frdev_bus1 = "n:frdev:bus1", "Hz, average frequency"
      n_dfehz_bus1 = "n:dfehz:bus1", "Hz, Deviation of the El. Frequency"
      n_dfedt_bus1 = "n:dfedt:bus1", "p.u./s, Derivative of El. Frequency"
      n_fehz_bus1 = "n:fehz:bus1", "Hz, Electrical Frequency"
      n_frnom_bus1 = "n:frnom:bus1", "Hz, Nominal Frequency"
      s_Pext = "s:Pext", "MW, IN, Active Power Input, continuous, double"
      s_Qext = "s:Qext", "Mvar, IN, Reactive Power Input, continuous, double"
      s_fref = "s:fref", "p.u., IN, Reference Frequency, continuous, double"
      s_xu = "s:xu", "p.u., STATE, State Variable, continuous, double"
      s_xf = "s:xf", "p.u., STATE, State Variable, continuous, double"
      s_xu_dt = "s:xu:dt", "p.u./s, d/dt, State Variable, derivative, continuous, double"
      s_xf_dt = "s:xf:dt", "p.u./s, d/dt, State Variable, derivative, continuous, double"
      c_sr_r = "c:sr:r", "MW, Complex Power (P+jQ), Phase a, Real Part"
      c_sr_i = "c:sr:i", "MW, Complex Power (P+jQ), Phase a, Imaginary Part"
      c_sr_phi = "c:sr:phi", "deg, Complex Power (P+jQ), Phase a, Angle"
      c_sr_mag = "c:sr:mag", "MW, Complex Power (P+jQ), Phase a, Magnitude"
      c_sr_phirad = "c:sr:phirad", "rad, Complex Power (P+jQ), Phase a, Angle"
      c_ss_r = "c:ss:r", "MW, Complex Power (P+jQ), Phase b, Real Part"
      c_ss_i = "c:ss:i", "MW, Complex Power (P+jQ), Phase b, Imaginary Part"
      c_ss_phi = "c:ss:phi", "deg, Complex Power (P+jQ), Phase b, Angle"
      c_ss_mag = "c:ss:mag", "MW, Complex Power (P+jQ), Phase b, Magnitude"
      c_ss_phirad = "c:ss:phirad", "rad, Complex Power (P+jQ), Phase b, Angle"
      c_st_r = "c:st:r", "MW, Complex Power (P+jQ), Phase c, Real Part"
      c_st_i = "c:st:i", "MW, Complex Power (P+jQ), Phase c, Imaginary Part"
      c_st_phi = "c:st:phi", "deg, Complex Power (P+jQ), Phase c, Angle"
      c_st_mag = "c:st:mag", "MW, Complex Power (P+jQ), Phase c, Magnitude"
      c_st_phirad = "c:st:phirad", "rad, Complex Power (P+jQ), Phase c, Angle"
      c_yr_r = "c:yr:r", "p.u., Load Admittance, Phase a, Real Part"
      c_yr_i = "c:yr:i", "p.u., Load Admittance, Phase a, Imaginary Part"
      c_yr_phi = "c:yr:phi", "deg, Load Admittance, Phase a, Angle"
      c_yr_mag = "c:yr:mag", "p.u., Load Admittance, Phase a, Magnitude"
      c_yr_phirad = "c:yr:phirad", "rad, Load Admittance, Phase a, Angle"
      c_ys_r = "c:ys:r", "p.u., Load Admittance, Phase b, Real Part"
      c_ys_i = "c:ys:i", "p.u., Load Admittance, Phase b, Imaginary Part"
      c_ys_phi = "c:ys:phi", "deg, Load Admittance, Phase b, Angle"
      c_ys_mag = "c:ys:mag", "p.u., Load Admittance, Phase b, Magnitude"
      c_ys_phirad = "c:ys:phirad", "rad, Load Admittance, Phase b, Angle"
      c_yt_r = "c:yt:r", "p.u., Load Admittance, Phase c, Real Part"
      c_yt_i = "c:yt:i", "p.u., Load Admittance, Phase c, Imaginary Part"
      c_yt_phi = "c:yt:phi", "deg, Load Admittance, Phase c, Angle"
      c_yt_mag = "c:yt:mag", "p.u., Load Admittance, Phase c, Magnitude"
      c_yt_phirad = "c:yt:phirad", "rad, Load Admittance, Phase c, Angle"
      c_yr0_r = "c:yr0:r", "p.u., Load Admittance, Phase a, Real Part"
      c_yr0_i = "c:yr0:i", "p.u., Load Admittance, Phase a, Imaginary Part"
      c_yr0_phi = "c:yr0:phi", "deg, Load Admittance, Phase a, Angle"
      c_yr0_mag = "c:yr0:mag", "p.u., Load Admittance, Phase a, Magnitude"
      c_yr0_phirad = "c:yr0:phirad", "rad, Load Admittance, Phase a, Angle"
      c_ys0_r = "c:ys0:r", "p.u., Load Admittance, Phase b, Real Part"
      c_ys0_i = "c:ys0:i", "p.u., Load Admittance, Phase b, Imaginary Part"
      c_ys0_phi = "c:ys0:phi", "deg, Load Admittance, Phase b, Angle"
      c_ys0_mag = "c:ys0:mag", "p.u., Load Admittance, Phase b, Magnitude"
      c_ys0_phirad = "c:ys0:phirad", "rad, Load Admittance, Phase b, Angle"
      c_yt0_r = "c:yt0:r", "p.u., Load Admittance, Phase c, Real Part"
      c_yt0_i = "c:yt0:i", "p.u., Load Admittance, Phase c, Imaginary Part"
      c_yt0_phi = "c:yt0:phi", "deg, Load Admittance, Phase c, Angle"
      c_yt0_mag = "c:yt0:mag", "p.u., Load Admittance, Phase c, Magnitude"
      c_yt0_phirad = "c:yt0:phirad", "rad, Load Admittance, Phase c, Angle"
      c_cn = "c:cn", "Connection Mode"
      c_lodst_l = "c:lodst_l", "Static"
      c_loddy_l = "c:loddy_l", "Dynamic"
      c_kpf = "c:kpf", "Static Frequency Dependence"
      c_tpf = "c:tpf", "s, Transient Frequency Dependence"
      c_tpu = "c:tpu", "Transient Voltage Dependence"
      c_kqf = "c:kqf", "Static Frequency Dependence"
      c_tqf = "c:tqf", "s, Transient Frequency Dependence"
      c_tqu = "c:tqu", "s, Transient Voltage Dependence"
      c_t1 = "c:t1", "s, Dynamic Load Time Constant"
      c_sd0_r = "c:sd0:r", "MW, Nominal power of dynamic load, Real Part"
      c_sd0_i = "c:sd0:i", "Mvar, Nominal power of dynamic load, Imaginary Part"
      c_sd0_phi = "c:sd0:phi", "deg, Nominal power of dynamic load, Angle"
      c_sd0_mag = "c:sd0:mag", "MVA, Nominal power of dynamic load, Magnitude"
      c_sd0_phirad = "c:sd0:phirad", "rad, Nominal power of dynamic load, Angle"
      c_sd00_r = "c:sd00:r", "MW, Nominal power of dynamic load, Real Part"
      c_sd00_i = "c:sd00:i", "Mvar, Nominal power of dynamic load, Imaginary Part"
      c_sd00_phi = "c:sd00:phi", "deg, Nominal power of dynamic load, Angle"
      c_sd00_mag = "c:sd00:mag", "MVA, Nominal power of dynamic load, Magnitude"
      c_sd00_phirad = "c:sd00:phirad", "rad, Nominal power of dynamic load, Angle"
      c_u0 = "c:u0", "p.u., Nominal voltage"
      c_udmax = "c:udmax", "p.u., Upper Voltage Limit"
      c_udmin = "c:udmin", "p.u., Lower Voltage Limit"
      c_i_nln = "c:i_nln", "Internal Flag"
      c_fe = "c:fe", "Electrical Frequency"
      c_kpu0 = "c:kpu0", "Volt. Dependence I on P"
      c_kqu0 = "c:kqu0", "Volt. Dependence I on Q"
      c_kpu1 = "c:kpu1", "Volt. Dependence II on P"
      c_kqu1 = "c:kqu1", "Volt. Dependence II on Q"
      c_kpu = "c:kpu", "Volt. Dependence III on P"
      c_kqu = "c:kqu", "Volt. Dependence III on Q"
      c_aP = "c:aP", "Percent of Volt. Dep. I on P"
      c_bP = "c:bP", "Percent of Volt. Dep. II on P"
      c_aQ = "c:aQ", "Percent of Volt. Dep. I on Q"
      c_bQ = "c:bQ", "Percent of Volt. Dep. II on Q"
      c_z0_r = "c:z0:r", "p.u., z0, Real Part"
      c_z0_i = "c:z0:i", "p.u., z0, Imaginary Part"
      c_z0_phi = "c:z0:phi", "deg, z0, Angle"
      c_z0_mag = "c:z0:mag", "p.u., z0, Magnitude"
      c_z0_phirad = "c:z0:phirad", "rad, z0, Angle"
      c_iZzero = "c:iZzero", "..."
      e_mode_inp = "e:mode_inp", "Input Mode"
      e_i_sym = "e:i_sym", "Balanced/Unbalanced"
      e_plini = "e:plini", "MW, Active Power"
      e_qlini = "e:qlini", "Mvar, Reactive Power"
      e_slini = "e:slini", "MVA, Apparent Power"
      e_ilini = "e:ilini", "kA, Current"
      e_coslini = "e:coslini", "Power Factor"
      e_pf_recap = "e:pf_recap", "Power Factor"
      e_p_direc = "e:p_direc", "Power Direction"
      e_u0 = "e:u0", "p.u., Voltage"
      e_scale0 = "e:scale0", "Scaling Factor"
      e_ramp_type = "e:ramp_type", "Allow Load-Ramp Event"
      e_i_normPQext = "e:i_normPQext", "Pext, Qext input signals normalised to load flow voltage"
      e_plinir = "e:plinir", "MW, Active Power"
      e_qlinir = "e:qlinir", "Mvar, Reactive Power"
      e_slinir = "e:slinir", "MVA, Apparent Power"
      e_ilinir = "e:ilinir", "kA, Current"
      e_coslinir = "e:coslinir", "Power Factor"
      e_pf_recapr = "e:pf_recapr", "Power Factor"
      e_p_direcr = "e:p_direcr", "Power Direction"
      e_plinis = "e:plinis", "MW, Active Power"
      e_qlinis = "e:qlinis", "Mvar, Reactive Power"
      e_slinis = "e:slinis", "MVA, Apparent Power"
      e_ilinis = "e:ilinis", "kA, Current"
      e_coslinis = "e:coslinis", "Power Factor"
      e_pf_recaps = "e:pf_recaps", "Power Factor"
      e_p_direcs = "e:p_direcs", "Power Direction"
      e_plinit = "e:plinit", "MW, Active Power"
      e_qlinit = "e:qlinit", "Mvar, Reactive Power"
      e_slinit = "e:slinit", "MVA, Apparent Power"
      e_ilinit = "e:ilinit", "kA, Current"
      e_coslinit = "e:coslinit", "Power Factor"
      e_pf_recapt = "e:pf_recapt", "Power Factor"
      e_p_direct = "e:p_direct", "Power Direction"
      e_iAstabint = "e:iAstabint", "A-stable integration algorithm"

    class ElmLodlv(Enum):
      _init_ = 'value __doc__'

      m_u0_bus1 = "m:u0:bus1", "p.u., Zero-Sequence-Voltage, Magnitude"
      m_phiu0_bus1 = "m:phiu0:bus1", "deg, Zero-Sequence-Voltage, Angle"
      m_u1r_bus1 = "m:u1r:bus1", "p.u., Positive-Sequence-Voltage, Real Part"
      m_u1i_bus1 = "m:u1i:bus1", "p.u., Positive-Sequence-Voltage, Imaginary Part"
      m_u1_bus1 = "m:u1:bus1", "p.u., Positive-Sequence-Voltage, Magnitude"
      m_U1_bus1 = "m:U1:bus1", "kV, Line-Ground Positive-Sequence-Voltage, Magnitude"
      m_U1l_bus1 = "m:U1l:bus1", "kV, Line-Line Positive-Sequence-Voltage, Magnitude"
      m_phiu1_bus1 = "m:phiu1:bus1", "deg, Positive-Sequence-Voltage, Angle"
      m_u2_bus1 = "m:u2:bus1", "p.u., Negative-Sequence-Voltage, Magnitude"
      m_phiu2_bus1 = "m:phiu2:bus1", "deg, Negative-Sequence-Voltage, Angle"
      m_inet_bus1_A = "m:inet:bus1:A", "p.u., Phase Current, Magnitude"
      m_inet_bus1_B = "m:inet:bus1:B", "p.u., Phase Current, Magnitude"
      m_inet_bus1_C = "m:inet:bus1:C", "p.u., Phase Current, Magnitude"
      m_inet_bus1_N = "m:inet:bus1:N", "p.u., Phase Current, Magnitude"
      m_I_bus1_A = "m:I:bus1:A", "kA, Phase Current, Magnitude"
      m_I_bus1_B = "m:I:bus1:B", "kA, Phase Current, Magnitude"
      m_I_bus1_C = "m:I:bus1:C", "kA, Phase Current, Magnitude"
      m_I_bus1_N = "m:I:bus1:N", "kA, Phase Current, Magnitude"
      m_phii_bus1_A = "m:phii:bus1:A", "deg, Phase Current, Angle"
      m_phii_bus1_B = "m:phii:bus1:B", "deg, Phase Current, Angle"
      m_phii_bus1_C = "m:phii:bus1:C", "deg, Phase Current, Angle"
      m_phii_bus1_N = "m:phii:bus1:N", "deg, Phase Current, Angle"
      m_i0r_bus1 = "m:i0r:bus1", "p.u., Zero-Sequence Current, Real Part"
      m_i0i_bus1 = "m:i0i:bus1", "p.u., Zero-Sequence Current, Imaginary Part"
      m_i0_bus1 = "m:i0:bus1", "p.u., Zero-Sequence Current, Magnitude"
      m_I0_bus1 = "m:I0:bus1", "kA, Zero-Sequence Current, Magnitude"
      m_I0x3_bus1 = "m:I0x3:bus1", "kA, 3*I0"
      m_phii0_bus1 = "m:phii0:bus1", "deg, Zero-Sequence Current, Angle"
      m_i1r_bus1 = "m:i1r:bus1", "p.u., Positive-Sequence Current, Real Part"
      m_i1i_bus1 = "m:i1i:bus1", "p.u., Positive-Sequence Current, Imaginary Part"
      m_i1_bus1 = "m:i1:bus1", "p.u., Positive-Sequence Current, Magnitude"
      m_I1_bus1 = "m:I1:bus1", "kA, Positive-Sequence Current, Magnitude"
      m_phii1_bus1 = "m:phii1:bus1", "deg, Positive-Sequence Current, Angle"
      m_i2r_bus1 = "m:i2r:bus1", "p.u., Negative-Sequence Current, Real Part"
      m_i2i_bus1 = "m:i2i:bus1", "p.u., Negative-Sequence Current, Imaginary Part"
      m_i2_bus1 = "m:i2:bus1", "p.u., Negative-Sequence Current, Magnitude"
      m_I2_bus1 = "m:I2:bus1", "kA, Negative-Sequence Current, Magnitude"
      m_phii2_bus1 = "m:phii2:bus1", "deg, Negative-Sequence Current, Angle"
      m_P_bus1_A = "m:P:bus1:A", "MW, Active Power"
      m_P_bus1_B = "m:P:bus1:B", "MW, Active Power"
      m_P_bus1_C = "m:P:bus1:C", "MW, Active Power"
      m_P_bus1_N = "m:P:bus1:N", "MW, Active Power"
      m_Q_bus1_A = "m:Q:bus1:A", "Mvar, Reactive Power"
      m_Q_bus1_B = "m:Q:bus1:B", "Mvar, Reactive Power"
      m_Q_bus1_C = "m:Q:bus1:C", "Mvar, Reactive Power"
      m_Q_bus1_N = "m:Q:bus1:N", "Mvar, Reactive Power"
      m_S_bus1_A = "m:S:bus1:A", "MVA, Apparent Power"
      m_S_bus1_B = "m:S:bus1:B", "MVA, Apparent Power"
      m_S_bus1_C = "m:S:bus1:C", "MVA, Apparent Power"
      m_S_bus1_N = "m:S:bus1:N", "MVA, Apparent Power"
      m_cosphi_bus1_A = "m:cosphi:bus1:A", "Power Factor"
      m_cosphi_bus1_B = "m:cosphi:bus1:B", "Power Factor"
      m_cosphi_bus1_C = "m:cosphi:bus1:C", "Power Factor"
      m_cosphi_bus1_N = "m:cosphi:bus1:N", "Power Factor"
      m_tanphi_bus1_A = "m:tanphi:bus1:A", "tan(phi)"
      m_tanphi_bus1_B = "m:tanphi:bus1:B", "tan(phi)"
      m_tanphi_bus1_C = "m:tanphi:bus1:C", "tan(phi)"
      m_tanphi_bus1_N = "m:tanphi:bus1:N", "tan(phi)"
      m_Psum_bus1 = "m:Psum:bus1", "MW, Total Active Power"
      m_Qsum_bus1 = "m:Qsum:bus1", "Mvar, Total Reactive Power"
      m_Ssum_bus1 = "m:Ssum:bus1", "MVA, Total Apparent Power"
      m_cosphisum_bus1 = "m:cosphisum:bus1", "Total Power Factor"
      m_tanphisum_bus1 = "m:tanphisum:bus1", "Total tan(phi)"
      m_Pln_bus1_A = "m:Pln:bus1:A", "MW, Active Power (Line-Neutral)"
      m_Pln_bus1_B = "m:Pln:bus1:B", "MW, Active Power (Line-Neutral)"
      m_Pln_bus1_C = "m:Pln:bus1:C", "MW, Active Power (Line-Neutral)"
      m_Pln_bus1_N = "m:Pln:bus1:N", "MW, Active Power (Line-Neutral)"
      m_Qln_bus1_A = "m:Qln:bus1:A", "Mvar, Reactive Power (Line-Neutral)"
      m_Qln_bus1_B = "m:Qln:bus1:B", "Mvar, Reactive Power (Line-Neutral)"
      m_Qln_bus1_C = "m:Qln:bus1:C", "Mvar, Reactive Power (Line-Neutral)"
      m_Qln_bus1_N = "m:Qln:bus1:N", "Mvar, Reactive Power (Line-Neutral)"
      m_P1_bus1 = "m:P1:bus1", "MW, Positive Sequence Active Power"
      m_Q1_bus1 = "m:Q1:bus1", "Mvar, Positive Sequence Reactive Power"
      m_P2_bus1 = "m:P2:bus1", "MW, Negative Sequence Active Power"
      m_Q2_bus1 = "m:Q2:bus1", "Mvar, Negative Sequence Reactive Power"
      m_P0_bus1 = "m:P0:bus1", "MW, Zero Sequence Active Power"
      m_Q0_bus1 = "m:Q0:bus1", "Mvar, Zero Sequence Reactive Power"
      m_i1P_bus1 = "m:i1P:bus1", "p.u., Positive-Sequence Active Current"
      m_i1Q_bus1 = "m:i1Q:bus1", "p.u., Positive-Sequence Reactive Current"
      m_I1P_bus1 = "m:I1P:bus1", "kA, Positive-Sequence Active Current"
      m_I1Q_bus1 = "m:I1Q:bus1", "kA, Positive-Sequence Reactive Current"
      m_i2P_bus1 = "m:i2P:bus1", "p.u., Negative-Sequence Active Current"
      m_i2Q_bus1 = "m:i2Q:bus1", "p.u., Negative-Sequence Reactive Current"
      m_I2P_bus1 = "m:I2P:bus1", "kA, Negative-Sequence Active Current"
      m_I2Q_bus1 = "m:I2Q:bus1", "kA, Negative-Sequence Reactive Current"
      m_ur_bus1_A = "m:ur:bus1:A", "p.u., Phase Voltage, Real Part"
      m_ur_bus1_B = "m:ur:bus1:B", "p.u., Phase Voltage, Real Part"
      m_ur_bus1_C = "m:ur:bus1:C", "p.u., Phase Voltage, Real Part"
      m_ur_bus1_N = "m:ur:bus1:N", "p.u., Phase Voltage, Real Part"
      m_ui_bus1_A = "m:ui:bus1:A", "p.u., Phase Voltage, Imaginary Part"
      m_ui_bus1_B = "m:ui:bus1:B", "p.u., Phase Voltage, Imaginary Part"
      m_ui_bus1_C = "m:ui:bus1:C", "p.u., Phase Voltage, Imaginary Part"
      m_ui_bus1_N = "m:ui:bus1:N", "p.u., Phase Voltage, Imaginary Part"
      m_u_bus1_A = "m:u:bus1:A", "p.u., Phase Voltage, Magnitude"
      m_u_bus1_B = "m:u:bus1:B", "p.u., Phase Voltage, Magnitude"
      m_u_bus1_C = "m:u:bus1:C", "p.u., Phase Voltage, Magnitude"
      m_u_bus1_N = "m:u:bus1:N", "p.u., Phase Voltage, Magnitude"
      m_ir_bus1_A = "m:ir:bus1:A", "p.u., Phase Current, Real Part"
      m_ir_bus1_B = "m:ir:bus1:B", "p.u., Phase Current, Real Part"
      m_ir_bus1_C = "m:ir:bus1:C", "p.u., Phase Current, Real Part"
      m_ir_bus1_N = "m:ir:bus1:N", "p.u., Phase Current, Real Part"
      m_ii_bus1_A = "m:ii:bus1:A", "p.u., Phase Current, Imaginary Part"
      m_ii_bus1_B = "m:ii:bus1:B", "p.u., Phase Current, Imaginary Part"
      m_ii_bus1_C = "m:ii:bus1:C", "p.u., Phase Current, Imaginary Part"
      m_ii_bus1_N = "m:ii:bus1:N", "p.u., Phase Current, Imaginary Part"
      m_i_bus1_A = "m:i:bus1:A", "p.u., Phase Current, Magnitude"
      m_i_bus1_B = "m:i:bus1:B", "p.u., Phase Current, Magnitude"
      m_i_bus1_C = "m:i:bus1:C", "p.u., Phase Current, Magnitude"
      m_i_bus1_N = "m:i:bus1:N", "p.u., Phase Current, Magnitude"
      m_phiui_bus1_A = "m:phiui:bus1:A", "deg, Angle between Voltage and Current"
      m_phiui_bus1_B = "m:phiui:bus1:B", "deg, Angle between Voltage and Current"
      m_phiui_bus1_C = "m:phiui:bus1:C", "deg, Angle between Voltage and Current"
      m_phiui_bus1_N = "m:phiui:bus1:N", "deg, Angle between Voltage and Current"
      m_phiu0i0_bus1 = "m:phiu0i0:bus1", "deg, Angle between Voltage and Current in zero-sequence system"
      m_phiu1i1_bus1 = "m:phiu1i1:bus1", "deg, Angle between Voltage and Current in positive sequence system"
      m_phiu2i2_bus1 = "m:phiu2i2:bus1", "deg, Angle between Voltage and Current in negative sequence system"
      m_Tfct_bus1 = "m:Tfct:bus1", "s, Fault Clearing Time"
      m_TfctPh_bus1_A = "m:TfctPh:bus1:A", "s, Fault Clearing Time"
      m_TfctPh_bus1_B = "m:TfctPh:bus1:B", "s, Fault Clearing Time"
      m_TfctPh_bus1_C = "m:TfctPh:bus1:C", "s, Fault Clearing Time"
      m_TfctPh_bus1_N = "m:TfctPh:bus1:N", "s, Fault Clearing Time"
      m_Brkload_bus1 = "m:Brkload:bus1", "%, Max. Breaker Loading"
      m_BrkloadPh_bus1_A = "m:BrkloadPh:bus1:A", "%, Breaker Loading"
      m_BrkloadPh_bus1_B = "m:BrkloadPh:bus1:B", "%, Breaker Loading"
      m_BrkloadPh_bus1_C = "m:BrkloadPh:bus1:C", "%, Breaker Loading"
      m_BrkloadPh_bus1_N = "m:BrkloadPh:bus1:N", "%, Breaker Loading"
      n_ur_bus1_A = "n:ur:bus1:A", "p.u., Line-Ground Voltage, Real Part"
      n_ur_bus1_B = "n:ur:bus1:B", "p.u., Line-Ground Voltage, Real Part"
      n_ur_bus1_C = "n:ur:bus1:C", "p.u., Line-Ground Voltage, Real Part"
      n_ur_bus1_N = "n:ur:bus1:N", "p.u., Line-Ground Voltage, Real Part"
      n_ui_bus1_A = "n:ui:bus1:A", "p.u., Line-Ground Voltage, Imaginary Part"
      n_ui_bus1_B = "n:ui:bus1:B", "p.u., Line-Ground Voltage, Imaginary Part"
      n_ui_bus1_C = "n:ui:bus1:C", "p.u., Line-Ground Voltage, Imaginary Part"
      n_ui_bus1_N = "n:ui:bus1:N", "p.u., Line-Ground Voltage, Imaginary Part"
      n_u_bus1_A = "n:u:bus1:A", "p.u., Line-Ground Voltage, Magnitude"
      n_u_bus1_B = "n:u:bus1:B", "p.u., Line-Ground Voltage, Magnitude"
      n_u_bus1_C = "n:u:bus1:C", "p.u., Line-Ground Voltage, Magnitude"
      n_u_bus1_N = "n:u:bus1:N", "p.u., Line-Ground Voltage, Magnitude"
      n_upc_bus1_A = "n:upc:bus1:A", "%, Line-Ground Voltage, Magnitude"
      n_upc_bus1_B = "n:upc:bus1:B", "%, Line-Ground Voltage, Magnitude"
      n_upc_bus1_C = "n:upc:bus1:C", "%, Line-Ground Voltage, Magnitude"
      n_upc_bus1_N = "n:upc:bus1:N", "%, Line-Ground Voltage, Magnitude"
      n_U_bus1_A = "n:U:bus1:A", "kV, Line-Ground Voltage, Magnitude"
      n_U_bus1_B = "n:U:bus1:B", "kV, Line-Ground Voltage, Magnitude"
      n_U_bus1_C = "n:U:bus1:C", "kV, Line-Ground Voltage, Magnitude"
      n_U_bus1_N = "n:U:bus1:N", "kV, Line-Ground Voltage, Magnitude"
      n_phiu_bus1_A = "n:phiu:bus1:A", "deg, Line-Ground Voltage, Angle"
      n_phiu_bus1_B = "n:phiu:bus1:B", "deg, Line-Ground Voltage, Angle"
      n_phiu_bus1_C = "n:phiu:bus1:C", "deg, Line-Ground Voltage, Angle"
      n_phiu_bus1_N = "n:phiu:bus1:N", "deg, Line-Ground Voltage, Angle"
      n_phiurel_bus1 = "n:phiurel:bus1", "deg, Voltage, Relative Angle"
      n_ul_bus1_A = "n:ul:bus1:A", "p.u., Line-Line Voltage, Magnitude"
      n_ul_bus1_B = "n:ul:bus1:B", "p.u., Line-Line Voltage, Magnitude"
      n_ul_bus1_C = "n:ul:bus1:C", "p.u., Line-Line Voltage, Magnitude"
      n_ul_bus1_N = "n:ul:bus1:N", "p.u., Line-Line Voltage, Magnitude"
      n_ulpc_bus1_A = "n:ulpc:bus1:A", "%, Line-Line Voltage, Magnitude"
      n_ulpc_bus1_B = "n:ulpc:bus1:B", "%, Line-Line Voltage, Magnitude"
      n_ulpc_bus1_C = "n:ulpc:bus1:C", "%, Line-Line Voltage, Magnitude"
      n_ulpc_bus1_N = "n:ulpc:bus1:N", "%, Line-Line Voltage, Magnitude"
      n_Ul_bus1_A = "n:Ul:bus1:A", "kV, Line-Line Voltage, Magnitude"
      n_Ul_bus1_B = "n:Ul:bus1:B", "kV, Line-Line Voltage, Magnitude"
      n_Ul_bus1_C = "n:Ul:bus1:C", "kV, Line-Line Voltage, Magnitude"
      n_Ul_bus1_N = "n:Ul:bus1:N", "kV, Line-Line Voltage, Magnitude"
      n_phiul_bus1_A = "n:phiul:bus1:A", "deg, Line-Line Voltage, Angle"
      n_phiul_bus1_B = "n:phiul:bus1:B", "deg, Line-Line Voltage, Angle"
      n_phiul_bus1_C = "n:phiul:bus1:C", "deg, Line-Line Voltage, Angle"
      n_phiul_bus1_N = "n:phiul:bus1:N", "deg, Line-Line Voltage, Angle"
      n_u0_bus1 = "n:u0:bus1", "p.u., Zero-Sequence Voltage, Magnitude"
      n_U0_bus1 = "n:U0:bus1", "kV, Zero-Sequence Voltage, Magnitude"
      n_U0x3_bus1 = "n:U0x3:bus1", "kV, 3*U0"
      n_phiu0_bus1 = "n:phiu0:bus1", "deg, Zero-Sequence Voltage, Angle"
      n_u1_bus1 = "n:u1:bus1", "p.u., Positive-Sequence Voltage, Magnitude"
      n_u1pc_bus1 = "n:u1pc:bus1", "%, Positive-Sequence Voltage, Magnitude"
      n_u1r_bus1 = "n:u1r:bus1", "p.u., Positive-Sequence Voltage, Real Part"
      n_u1i_bus1 = "n:u1i:bus1", "p.u., Positive-Sequence Voltage, Imaginary Part"
      n_U1_bus1 = "n:U1:bus1", "kV, Line-Ground Positive-Sequence Voltage, Magnitude"
      n_phiu1_bus1 = "n:phiu1:bus1", "deg, Positive-Sequence Voltage, Angle"
      n_u2_bus1 = "n:u2:bus1", "p.u., Negative-Sequence Voltage, Magnitude"
      n_U2_bus1 = "n:U2:bus1", "kV, Line-Ground Negative-Sequence Voltage, Magnitude"
      n_phiu2_bus1 = "n:phiu2:bus1", "deg, Negative-Sequence Voltage, Angle"
      n_U1l_bus1 = "n:U1l:bus1", "kV, Line-Line Positive-Sequence Voltage, Magnitude"
      n_U2l_bus1 = "n:U2l:bus1", "kV, Line-Line Negative-Sequence Voltage, Magnitude"
      n_uln_bus1_A = "n:uln:bus1:A", "p.u., Line-Neutral Voltage, Magnitude"
      n_uln_bus1_B = "n:uln:bus1:B", "p.u., Line-Neutral Voltage, Magnitude"
      n_uln_bus1_C = "n:uln:bus1:C", "p.u., Line-Neutral Voltage, Magnitude"
      n_uln_bus1_N = "n:uln:bus1:N", "p.u., Line-Neutral Voltage, Magnitude"
      n_Uln_bus1_A = "n:Uln:bus1:A", "kV, Line-Neutral Voltage, Magnitude"
      n_Uln_bus1_B = "n:Uln:bus1:B", "kV, Line-Neutral Voltage, Magnitude"
      n_Uln_bus1_C = "n:Uln:bus1:C", "kV, Line-Neutral Voltage, Magnitude"
      n_Uln_bus1_N = "n:Uln:bus1:N", "kV, Line-Neutral Voltage, Magnitude"
      n_phiuln_bus1_A = "n:phiuln:bus1:A", "deg, Line-Neutral Voltage, Angle"
      n_phiuln_bus1_B = "n:phiuln:bus1:B", "deg, Line-Neutral Voltage, Angle"
      n_phiuln_bus1_C = "n:phiuln:bus1:C", "deg, Line-Neutral Voltage, Angle"
      n_phiuln_bus1_N = "n:phiuln:bus1:N", "deg, Line-Neutral Voltage, Angle"
      n_un_bus1 = "n:un:bus1", "p.u., Neutral-Ground Voltage, Magnitude"
      n_Un_bus1 = "n:Un:bus1", "kV, Neutral-Ground Voltage, Magnitude"
      n_phiun_bus1 = "n:phiun:bus1", "deg, Neutral-Ground Voltage, Angle"
      n_upht_bus1_A = "n:upht:bus1:A", "p.u., Phase Technology dependent Voltage, Magnitude"
      n_upht_bus1_B = "n:upht:bus1:B", "p.u., Phase Technology dependent Voltage, Magnitude"
      n_upht_bus1_C = "n:upht:bus1:C", "p.u., Phase Technology dependent Voltage, Magnitude"
      n_upht_bus1_N = "n:upht:bus1:N", "p.u., Phase Technology dependent Voltage, Magnitude"
      n_Upht_bus1_A = "n:Upht:bus1:A", "kV, Phase Technology dependent Voltage, Magnitude"
      n_Upht_bus1_B = "n:Upht:bus1:B", "kV, Phase Technology dependent Voltage, Magnitude"
      n_Upht_bus1_C = "n:Upht:bus1:C", "kV, Phase Technology dependent Voltage, Magnitude"
      n_Upht_bus1_N = "n:Upht:bus1:N", "kV, Phase Technology dependent Voltage, Magnitude"
      n_um_bus1 = "n:um:bus1", "p.u., Average-Voltage, Magnitude"
      n_Um_bus1 = "n:Um:bus1", "kV, Average-Voltage, Magnitude"
      n_ishcr_bus1_A = "n:ishcr:bus1:A", "p.u., Short-Circuit Current, real"
      n_ishcr_bus1_B = "n:ishcr:bus1:B", "p.u., Short-Circuit Current, real"
      n_ishcr_bus1_C = "n:ishcr:bus1:C", "p.u., Short-Circuit Current, real"
      n_ishcr_bus1_N = "n:ishcr:bus1:N", "p.u., Short-Circuit Current, real"
      n_ishci_bus1_A = "n:ishci:bus1:A", "p.u., Short-Circuit Current, imag"
      n_ishci_bus1_B = "n:ishci:bus1:B", "p.u., Short-Circuit Current, imag"
      n_ishci_bus1_C = "n:ishci:bus1:C", "p.u., Short-Circuit Current, imag"
      n_ishci_bus1_N = "n:ishci:bus1:N", "p.u., Short-Circuit Current, imag"
      n_ishc_bus1_A = "n:ishc:bus1:A", "p.u., Short-Circuit Current, magnitude"
      n_ishc_bus1_B = "n:ishc:bus1:B", "p.u., Short-Circuit Current, magnitude"
      n_ishc_bus1_C = "n:ishc:bus1:C", "p.u., Short-Circuit Current, magnitude"
      n_ishc_bus1_N = "n:ishc:bus1:N", "p.u., Short-Circuit Current, magnitude"
      n_Ishc_bus1_A = "n:Ishc:bus1:A", "kA, Short-Circuit Current, magnitude"
      n_Ishc_bus1_B = "n:Ishc:bus1:B", "kA, Short-Circuit Current, magnitude"
      n_Ishc_bus1_C = "n:Ishc:bus1:C", "kA, Short-Circuit Current, magnitude"
      n_Ishc_bus1_N = "n:Ishc:bus1:N", "kA, Short-Circuit Current, magnitude"
      n_inshc_bus1 = "n:inshc:bus1", "p.u., Neutral Short-Circuit Current, magnitude"
      n_Inshc_bus1 = "n:Inshc:bus1", "kA, Neutral Short-Circuit Current, magnitude"
      n_fe_bus1 = "n:fe:bus1", "p.u., Electrical Frequency"
      n_frdev_bus1 = "n:frdev:bus1", "Hz, average frequency"
      n_dfehz_bus1 = "n:dfehz:bus1", "Hz, Deviation of the El. Frequency"
      n_dfedt_bus1 = "n:dfedt:bus1", "p.u./s, Derivative of El. Frequency"
      n_fehz_bus1 = "n:fehz:bus1", "Hz, Electrical Frequency"
      n_frnom_bus1 = "n:frnom:bus1", "Hz, Nominal Frequency"
      c_sr_r = "c:sr:r", "MW, Complex Power (P+jQ), Phase a, Real Part"
      c_sr_i = "c:sr:i", "MW, Complex Power (P+jQ), Phase a, Imaginary Part"
      c_sr_phi = "c:sr:phi", "deg, Complex Power (P+jQ), Phase a, Angle"
      c_sr_mag = "c:sr:mag", "MW, Complex Power (P+jQ), Phase a, Magnitude"
      c_sr_phirad = "c:sr:phirad", "rad, Complex Power (P+jQ), Phase a, Angle"
      c_ss_r = "c:ss:r", "MW, Complex Power (P+jQ), Phase b, Real Part"
      c_ss_i = "c:ss:i", "MW, Complex Power (P+jQ), Phase b, Imaginary Part"
      c_ss_phi = "c:ss:phi", "deg, Complex Power (P+jQ), Phase b, Angle"
      c_ss_mag = "c:ss:mag", "MW, Complex Power (P+jQ), Phase b, Magnitude"
      c_ss_phirad = "c:ss:phirad", "rad, Complex Power (P+jQ), Phase b, Angle"
      c_st_r = "c:st:r", "MW, Complex Power (P+jQ), Phase c, Real Part"
      c_st_i = "c:st:i", "MW, Complex Power (P+jQ), Phase c, Imaginary Part"
      c_st_phi = "c:st:phi", "deg, Complex Power (P+jQ), Phase c, Angle"
      c_st_mag = "c:st:mag", "MW, Complex Power (P+jQ), Phase c, Magnitude"
      c_st_phirad = "c:st:phirad", "rad, Complex Power (P+jQ), Phase c, Angle"
      c_yr_r = "c:yr:r", "p.u., Load Admittance, Phase a, Real Part"
      c_yr_i = "c:yr:i", "p.u., Load Admittance, Phase a, Imaginary Part"
      c_yr_phi = "c:yr:phi", "deg, Load Admittance, Phase a, Angle"
      c_yr_mag = "c:yr:mag", "p.u., Load Admittance, Phase a, Magnitude"
      c_yr_phirad = "c:yr:phirad", "rad, Load Admittance, Phase a, Angle"
      c_ys_r = "c:ys:r", "p.u., Load Admittance, Phase b, Real Part"
      c_ys_i = "c:ys:i", "p.u., Load Admittance, Phase b, Imaginary Part"
      c_ys_phi = "c:ys:phi", "deg, Load Admittance, Phase b, Angle"
      c_ys_mag = "c:ys:mag", "p.u., Load Admittance, Phase b, Magnitude"
      c_ys_phirad = "c:ys:phirad", "rad, Load Admittance, Phase b, Angle"
      c_yt_r = "c:yt:r", "p.u., Load Admittance, Phase c, Real Part"
      c_yt_i = "c:yt:i", "p.u., Load Admittance, Phase c, Imaginary Part"
      c_yt_phi = "c:yt:phi", "deg, Load Admittance, Phase c, Angle"
      c_yt_mag = "c:yt:mag", "p.u., Load Admittance, Phase c, Magnitude"
      c_yt_phirad = "c:yt:phirad", "rad, Load Admittance, Phase c, Angle"
      c_yr0_r = "c:yr0:r", "p.u., Load Admittance, Phase a, Real Part"
      c_yr0_i = "c:yr0:i", "p.u., Load Admittance, Phase a, Imaginary Part"
      c_yr0_phi = "c:yr0:phi", "deg, Load Admittance, Phase a, Angle"
      c_yr0_mag = "c:yr0:mag", "p.u., Load Admittance, Phase a, Magnitude"
      c_yr0_phirad = "c:yr0:phirad", "rad, Load Admittance, Phase a, Angle"
      c_ys0_r = "c:ys0:r", "p.u., Load Admittance, Phase b, Real Part"
      c_ys0_i = "c:ys0:i", "p.u., Load Admittance, Phase b, Imaginary Part"
      c_ys0_phi = "c:ys0:phi", "deg, Load Admittance, Phase b, Angle"
      c_ys0_mag = "c:ys0:mag", "p.u., Load Admittance, Phase b, Magnitude"
      c_ys0_phirad = "c:ys0:phirad", "rad, Load Admittance, Phase b, Angle"
      c_yt0_r = "c:yt0:r", "p.u., Load Admittance, Phase c, Real Part"
      c_yt0_i = "c:yt0:i", "p.u., Load Admittance, Phase c, Imaginary Part"
      c_yt0_phi = "c:yt0:phi", "deg, Load Admittance, Phase c, Angle"
      c_yt0_mag = "c:yt0:mag", "p.u., Load Admittance, Phase c, Magnitude"
      c_yt0_phirad = "c:yt0:phirad", "rad, Load Admittance, Phase c, Angle"
      c_cn = "c:cn", "Connection Mode"
      c_ncust = "c:ncust", "Number of Connected Customers"
      e_iopt_type = "e:iopt_type", "Allow Load-Ramp Event"

    class ElmLodmv(Enum):
      _init_ = 'value __doc__'

      m_u0_bus1 = "m:u0:bus1", "p.u., Zero-Sequence-Voltage, Magnitude"
      m_phiu0_bus1 = "m:phiu0:bus1", "deg, Zero-Sequence-Voltage, Angle"
      m_u1r_bus1 = "m:u1r:bus1", "p.u., Positive-Sequence-Voltage, Real Part"
      m_u1i_bus1 = "m:u1i:bus1", "p.u., Positive-Sequence-Voltage, Imaginary Part"
      m_u1_bus1 = "m:u1:bus1", "p.u., Positive-Sequence-Voltage, Magnitude"
      m_U1_bus1 = "m:U1:bus1", "kV, Line-Ground Positive-Sequence-Voltage, Magnitude"
      m_U1l_bus1 = "m:U1l:bus1", "kV, Line-Line Positive-Sequence-Voltage, Magnitude"
      m_phiu1_bus1 = "m:phiu1:bus1", "deg, Positive-Sequence-Voltage, Angle"
      m_u2_bus1 = "m:u2:bus1", "p.u., Negative-Sequence-Voltage, Magnitude"
      m_phiu2_bus1 = "m:phiu2:bus1", "deg, Negative-Sequence-Voltage, Angle"
      m_inet_bus1_A = "m:inet:bus1:A", "p.u., Phase Current, Magnitude"
      m_inet_bus1_B = "m:inet:bus1:B", "p.u., Phase Current, Magnitude"
      m_inet_bus1_C = "m:inet:bus1:C", "p.u., Phase Current, Magnitude"
      m_inet_bus1_N = "m:inet:bus1:N", "p.u., Phase Current, Magnitude"
      m_I_bus1_A = "m:I:bus1:A", "kA, Phase Current, Magnitude"
      m_I_bus1_B = "m:I:bus1:B", "kA, Phase Current, Magnitude"
      m_I_bus1_C = "m:I:bus1:C", "kA, Phase Current, Magnitude"
      m_I_bus1_N = "m:I:bus1:N", "kA, Phase Current, Magnitude"
      m_phii_bus1_A = "m:phii:bus1:A", "deg, Phase Current, Angle"
      m_phii_bus1_B = "m:phii:bus1:B", "deg, Phase Current, Angle"
      m_phii_bus1_C = "m:phii:bus1:C", "deg, Phase Current, Angle"
      m_phii_bus1_N = "m:phii:bus1:N", "deg, Phase Current, Angle"
      m_i0r_bus1 = "m:i0r:bus1", "p.u., Zero-Sequence Current, Real Part"
      m_i0i_bus1 = "m:i0i:bus1", "p.u., Zero-Sequence Current, Imaginary Part"
      m_i0_bus1 = "m:i0:bus1", "p.u., Zero-Sequence Current, Magnitude"
      m_I0_bus1 = "m:I0:bus1", "kA, Zero-Sequence Current, Magnitude"
      m_I0x3_bus1 = "m:I0x3:bus1", "kA, 3*I0"
      m_phii0_bus1 = "m:phii0:bus1", "deg, Zero-Sequence Current, Angle"
      m_i1r_bus1 = "m:i1r:bus1", "p.u., Positive-Sequence Current, Real Part"
      m_i1i_bus1 = "m:i1i:bus1", "p.u., Positive-Sequence Current, Imaginary Part"
      m_i1_bus1 = "m:i1:bus1", "p.u., Positive-Sequence Current, Magnitude"
      m_I1_bus1 = "m:I1:bus1", "kA, Positive-Sequence Current, Magnitude"
      m_phii1_bus1 = "m:phii1:bus1", "deg, Positive-Sequence Current, Angle"
      m_i2r_bus1 = "m:i2r:bus1", "p.u., Negative-Sequence Current, Real Part"
      m_i2i_bus1 = "m:i2i:bus1", "p.u., Negative-Sequence Current, Imaginary Part"
      m_i2_bus1 = "m:i2:bus1", "p.u., Negative-Sequence Current, Magnitude"
      m_I2_bus1 = "m:I2:bus1", "kA, Negative-Sequence Current, Magnitude"
      m_phii2_bus1 = "m:phii2:bus1", "deg, Negative-Sequence Current, Angle"
      m_P_bus1_A = "m:P:bus1:A", "MW, Active Power"
      m_P_bus1_B = "m:P:bus1:B", "MW, Active Power"
      m_P_bus1_C = "m:P:bus1:C", "MW, Active Power"
      m_P_bus1_N = "m:P:bus1:N", "MW, Active Power"
      m_Q_bus1_A = "m:Q:bus1:A", "Mvar, Reactive Power"
      m_Q_bus1_B = "m:Q:bus1:B", "Mvar, Reactive Power"
      m_Q_bus1_C = "m:Q:bus1:C", "Mvar, Reactive Power"
      m_Q_bus1_N = "m:Q:bus1:N", "Mvar, Reactive Power"
      m_S_bus1_A = "m:S:bus1:A", "MVA, Apparent Power"
      m_S_bus1_B = "m:S:bus1:B", "MVA, Apparent Power"
      m_S_bus1_C = "m:S:bus1:C", "MVA, Apparent Power"
      m_S_bus1_N = "m:S:bus1:N", "MVA, Apparent Power"
      m_cosphi_bus1_A = "m:cosphi:bus1:A", "Power Factor"
      m_cosphi_bus1_B = "m:cosphi:bus1:B", "Power Factor"
      m_cosphi_bus1_C = "m:cosphi:bus1:C", "Power Factor"
      m_cosphi_bus1_N = "m:cosphi:bus1:N", "Power Factor"
      m_tanphi_bus1_A = "m:tanphi:bus1:A", "tan(phi)"
      m_tanphi_bus1_B = "m:tanphi:bus1:B", "tan(phi)"
      m_tanphi_bus1_C = "m:tanphi:bus1:C", "tan(phi)"
      m_tanphi_bus1_N = "m:tanphi:bus1:N", "tan(phi)"
      m_Psum_bus1 = "m:Psum:bus1", "MW, Total Active Power"
      m_Qsum_bus1 = "m:Qsum:bus1", "Mvar, Total Reactive Power"
      m_Ssum_bus1 = "m:Ssum:bus1", "MVA, Total Apparent Power"
      m_cosphisum_bus1 = "m:cosphisum:bus1", "Total Power Factor"
      m_tanphisum_bus1 = "m:tanphisum:bus1", "Total tan(phi)"
      m_Pln_bus1_A = "m:Pln:bus1:A", "MW, Active Power (Line-Neutral)"
      m_Pln_bus1_B = "m:Pln:bus1:B", "MW, Active Power (Line-Neutral)"
      m_Pln_bus1_C = "m:Pln:bus1:C", "MW, Active Power (Line-Neutral)"
      m_Pln_bus1_N = "m:Pln:bus1:N", "MW, Active Power (Line-Neutral)"
      m_Qln_bus1_A = "m:Qln:bus1:A", "Mvar, Reactive Power (Line-Neutral)"
      m_Qln_bus1_B = "m:Qln:bus1:B", "Mvar, Reactive Power (Line-Neutral)"
      m_Qln_bus1_C = "m:Qln:bus1:C", "Mvar, Reactive Power (Line-Neutral)"
      m_Qln_bus1_N = "m:Qln:bus1:N", "Mvar, Reactive Power (Line-Neutral)"
      m_P1_bus1 = "m:P1:bus1", "MW, Positive Sequence Active Power"
      m_Q1_bus1 = "m:Q1:bus1", "Mvar, Positive Sequence Reactive Power"
      m_P2_bus1 = "m:P2:bus1", "MW, Negative Sequence Active Power"
      m_Q2_bus1 = "m:Q2:bus1", "Mvar, Negative Sequence Reactive Power"
      m_P0_bus1 = "m:P0:bus1", "MW, Zero Sequence Active Power"
      m_Q0_bus1 = "m:Q0:bus1", "Mvar, Zero Sequence Reactive Power"
      m_i1P_bus1 = "m:i1P:bus1", "p.u., Positive-Sequence Active Current"
      m_i1Q_bus1 = "m:i1Q:bus1", "p.u., Positive-Sequence Reactive Current"
      m_I1P_bus1 = "m:I1P:bus1", "kA, Positive-Sequence Active Current"
      m_I1Q_bus1 = "m:I1Q:bus1", "kA, Positive-Sequence Reactive Current"
      m_i2P_bus1 = "m:i2P:bus1", "p.u., Negative-Sequence Active Current"
      m_i2Q_bus1 = "m:i2Q:bus1", "p.u., Negative-Sequence Reactive Current"
      m_I2P_bus1 = "m:I2P:bus1", "kA, Negative-Sequence Active Current"
      m_I2Q_bus1 = "m:I2Q:bus1", "kA, Negative-Sequence Reactive Current"
      m_ur_bus1_A = "m:ur:bus1:A", "p.u., Phase Voltage, Real Part"
      m_ur_bus1_B = "m:ur:bus1:B", "p.u., Phase Voltage, Real Part"
      m_ur_bus1_C = "m:ur:bus1:C", "p.u., Phase Voltage, Real Part"
      m_ur_bus1_N = "m:ur:bus1:N", "p.u., Phase Voltage, Real Part"
      m_ui_bus1_A = "m:ui:bus1:A", "p.u., Phase Voltage, Imaginary Part"
      m_ui_bus1_B = "m:ui:bus1:B", "p.u., Phase Voltage, Imaginary Part"
      m_ui_bus1_C = "m:ui:bus1:C", "p.u., Phase Voltage, Imaginary Part"
      m_ui_bus1_N = "m:ui:bus1:N", "p.u., Phase Voltage, Imaginary Part"
      m_u_bus1_A = "m:u:bus1:A", "p.u., Phase Voltage, Magnitude"
      m_u_bus1_B = "m:u:bus1:B", "p.u., Phase Voltage, Magnitude"
      m_u_bus1_C = "m:u:bus1:C", "p.u., Phase Voltage, Magnitude"
      m_u_bus1_N = "m:u:bus1:N", "p.u., Phase Voltage, Magnitude"
      m_ir_bus1_A = "m:ir:bus1:A", "p.u., Phase Current, Real Part"
      m_ir_bus1_B = "m:ir:bus1:B", "p.u., Phase Current, Real Part"
      m_ir_bus1_C = "m:ir:bus1:C", "p.u., Phase Current, Real Part"
      m_ir_bus1_N = "m:ir:bus1:N", "p.u., Phase Current, Real Part"
      m_ii_bus1_A = "m:ii:bus1:A", "p.u., Phase Current, Imaginary Part"
      m_ii_bus1_B = "m:ii:bus1:B", "p.u., Phase Current, Imaginary Part"
      m_ii_bus1_C = "m:ii:bus1:C", "p.u., Phase Current, Imaginary Part"
      m_ii_bus1_N = "m:ii:bus1:N", "p.u., Phase Current, Imaginary Part"
      m_i_bus1_A = "m:i:bus1:A", "p.u., Phase Current, Magnitude"
      m_i_bus1_B = "m:i:bus1:B", "p.u., Phase Current, Magnitude"
      m_i_bus1_C = "m:i:bus1:C", "p.u., Phase Current, Magnitude"
      m_i_bus1_N = "m:i:bus1:N", "p.u., Phase Current, Magnitude"
      m_phiui_bus1_A = "m:phiui:bus1:A", "deg, Angle between Voltage and Current"
      m_phiui_bus1_B = "m:phiui:bus1:B", "deg, Angle between Voltage and Current"
      m_phiui_bus1_C = "m:phiui:bus1:C", "deg, Angle between Voltage and Current"
      m_phiui_bus1_N = "m:phiui:bus1:N", "deg, Angle between Voltage and Current"
      m_phiu0i0_bus1 = "m:phiu0i0:bus1", "deg, Angle between Voltage and Current in zero-sequence system"
      m_phiu1i1_bus1 = "m:phiu1i1:bus1", "deg, Angle between Voltage and Current in positive sequence system"
      m_phiu2i2_bus1 = "m:phiu2i2:bus1", "deg, Angle between Voltage and Current in negative sequence system"
      m_Tfct_bus1 = "m:Tfct:bus1", "s, Fault Clearing Time"
      m_TfctPh_bus1_A = "m:TfctPh:bus1:A", "s, Fault Clearing Time"
      m_TfctPh_bus1_B = "m:TfctPh:bus1:B", "s, Fault Clearing Time"
      m_TfctPh_bus1_C = "m:TfctPh:bus1:C", "s, Fault Clearing Time"
      m_TfctPh_bus1_N = "m:TfctPh:bus1:N", "s, Fault Clearing Time"
      m_Brkload_bus1 = "m:Brkload:bus1", "%, Max. Breaker Loading"
      m_BrkloadPh_bus1_A = "m:BrkloadPh:bus1:A", "%, Breaker Loading"
      m_BrkloadPh_bus1_B = "m:BrkloadPh:bus1:B", "%, Breaker Loading"
      m_BrkloadPh_bus1_C = "m:BrkloadPh:bus1:C", "%, Breaker Loading"
      m_BrkloadPh_bus1_N = "m:BrkloadPh:bus1:N", "%, Breaker Loading"
      n_ur_bus1_A = "n:ur:bus1:A", "p.u., Line-Ground Voltage, Real Part"
      n_ur_bus1_B = "n:ur:bus1:B", "p.u., Line-Ground Voltage, Real Part"
      n_ur_bus1_C = "n:ur:bus1:C", "p.u., Line-Ground Voltage, Real Part"
      n_ur_bus1_N = "n:ur:bus1:N", "p.u., Line-Ground Voltage, Real Part"
      n_ui_bus1_A = "n:ui:bus1:A", "p.u., Line-Ground Voltage, Imaginary Part"
      n_ui_bus1_B = "n:ui:bus1:B", "p.u., Line-Ground Voltage, Imaginary Part"
      n_ui_bus1_C = "n:ui:bus1:C", "p.u., Line-Ground Voltage, Imaginary Part"
      n_ui_bus1_N = "n:ui:bus1:N", "p.u., Line-Ground Voltage, Imaginary Part"
      n_u_bus1_A = "n:u:bus1:A", "p.u., Line-Ground Voltage, Magnitude"
      n_u_bus1_B = "n:u:bus1:B", "p.u., Line-Ground Voltage, Magnitude"
      n_u_bus1_C = "n:u:bus1:C", "p.u., Line-Ground Voltage, Magnitude"
      n_u_bus1_N = "n:u:bus1:N", "p.u., Line-Ground Voltage, Magnitude"
      n_upc_bus1_A = "n:upc:bus1:A", "%, Line-Ground Voltage, Magnitude"
      n_upc_bus1_B = "n:upc:bus1:B", "%, Line-Ground Voltage, Magnitude"
      n_upc_bus1_C = "n:upc:bus1:C", "%, Line-Ground Voltage, Magnitude"
      n_upc_bus1_N = "n:upc:bus1:N", "%, Line-Ground Voltage, Magnitude"
      n_U_bus1_A = "n:U:bus1:A", "kV, Line-Ground Voltage, Magnitude"
      n_U_bus1_B = "n:U:bus1:B", "kV, Line-Ground Voltage, Magnitude"
      n_U_bus1_C = "n:U:bus1:C", "kV, Line-Ground Voltage, Magnitude"
      n_U_bus1_N = "n:U:bus1:N", "kV, Line-Ground Voltage, Magnitude"
      n_phiu_bus1_A = "n:phiu:bus1:A", "deg, Line-Ground Voltage, Angle"
      n_phiu_bus1_B = "n:phiu:bus1:B", "deg, Line-Ground Voltage, Angle"
      n_phiu_bus1_C = "n:phiu:bus1:C", "deg, Line-Ground Voltage, Angle"
      n_phiu_bus1_N = "n:phiu:bus1:N", "deg, Line-Ground Voltage, Angle"
      n_phiurel_bus1 = "n:phiurel:bus1", "deg, Voltage, Relative Angle"
      n_ul_bus1_A = "n:ul:bus1:A", "p.u., Line-Line Voltage, Magnitude"
      n_ul_bus1_B = "n:ul:bus1:B", "p.u., Line-Line Voltage, Magnitude"
      n_ul_bus1_C = "n:ul:bus1:C", "p.u., Line-Line Voltage, Magnitude"
      n_ul_bus1_N = "n:ul:bus1:N", "p.u., Line-Line Voltage, Magnitude"
      n_ulpc_bus1_A = "n:ulpc:bus1:A", "%, Line-Line Voltage, Magnitude"
      n_ulpc_bus1_B = "n:ulpc:bus1:B", "%, Line-Line Voltage, Magnitude"
      n_ulpc_bus1_C = "n:ulpc:bus1:C", "%, Line-Line Voltage, Magnitude"
      n_ulpc_bus1_N = "n:ulpc:bus1:N", "%, Line-Line Voltage, Magnitude"
      n_Ul_bus1_A = "n:Ul:bus1:A", "kV, Line-Line Voltage, Magnitude"
      n_Ul_bus1_B = "n:Ul:bus1:B", "kV, Line-Line Voltage, Magnitude"
      n_Ul_bus1_C = "n:Ul:bus1:C", "kV, Line-Line Voltage, Magnitude"
      n_Ul_bus1_N = "n:Ul:bus1:N", "kV, Line-Line Voltage, Magnitude"
      n_phiul_bus1_A = "n:phiul:bus1:A", "deg, Line-Line Voltage, Angle"
      n_phiul_bus1_B = "n:phiul:bus1:B", "deg, Line-Line Voltage, Angle"
      n_phiul_bus1_C = "n:phiul:bus1:C", "deg, Line-Line Voltage, Angle"
      n_phiul_bus1_N = "n:phiul:bus1:N", "deg, Line-Line Voltage, Angle"
      n_u0_bus1 = "n:u0:bus1", "p.u., Zero-Sequence Voltage, Magnitude"
      n_U0_bus1 = "n:U0:bus1", "kV, Zero-Sequence Voltage, Magnitude"
      n_U0x3_bus1 = "n:U0x3:bus1", "kV, 3*U0"
      n_phiu0_bus1 = "n:phiu0:bus1", "deg, Zero-Sequence Voltage, Angle"
      n_u1_bus1 = "n:u1:bus1", "p.u., Positive-Sequence Voltage, Magnitude"
      n_u1pc_bus1 = "n:u1pc:bus1", "%, Positive-Sequence Voltage, Magnitude"
      n_u1r_bus1 = "n:u1r:bus1", "p.u., Positive-Sequence Voltage, Real Part"
      n_u1i_bus1 = "n:u1i:bus1", "p.u., Positive-Sequence Voltage, Imaginary Part"
      n_U1_bus1 = "n:U1:bus1", "kV, Line-Ground Positive-Sequence Voltage, Magnitude"
      n_phiu1_bus1 = "n:phiu1:bus1", "deg, Positive-Sequence Voltage, Angle"
      n_u2_bus1 = "n:u2:bus1", "p.u., Negative-Sequence Voltage, Magnitude"
      n_U2_bus1 = "n:U2:bus1", "kV, Line-Ground Negative-Sequence Voltage, Magnitude"
      n_phiu2_bus1 = "n:phiu2:bus1", "deg, Negative-Sequence Voltage, Angle"
      n_U1l_bus1 = "n:U1l:bus1", "kV, Line-Line Positive-Sequence Voltage, Magnitude"
      n_U2l_bus1 = "n:U2l:bus1", "kV, Line-Line Negative-Sequence Voltage, Magnitude"
      n_uln_bus1_A = "n:uln:bus1:A", "p.u., Line-Neutral Voltage, Magnitude"
      n_uln_bus1_B = "n:uln:bus1:B", "p.u., Line-Neutral Voltage, Magnitude"
      n_uln_bus1_C = "n:uln:bus1:C", "p.u., Line-Neutral Voltage, Magnitude"
      n_uln_bus1_N = "n:uln:bus1:N", "p.u., Line-Neutral Voltage, Magnitude"
      n_Uln_bus1_A = "n:Uln:bus1:A", "kV, Line-Neutral Voltage, Magnitude"
      n_Uln_bus1_B = "n:Uln:bus1:B", "kV, Line-Neutral Voltage, Magnitude"
      n_Uln_bus1_C = "n:Uln:bus1:C", "kV, Line-Neutral Voltage, Magnitude"
      n_Uln_bus1_N = "n:Uln:bus1:N", "kV, Line-Neutral Voltage, Magnitude"
      n_phiuln_bus1_A = "n:phiuln:bus1:A", "deg, Line-Neutral Voltage, Angle"
      n_phiuln_bus1_B = "n:phiuln:bus1:B", "deg, Line-Neutral Voltage, Angle"
      n_phiuln_bus1_C = "n:phiuln:bus1:C", "deg, Line-Neutral Voltage, Angle"
      n_phiuln_bus1_N = "n:phiuln:bus1:N", "deg, Line-Neutral Voltage, Angle"
      n_un_bus1 = "n:un:bus1", "p.u., Neutral-Ground Voltage, Magnitude"
      n_Un_bus1 = "n:Un:bus1", "kV, Neutral-Ground Voltage, Magnitude"
      n_phiun_bus1 = "n:phiun:bus1", "deg, Neutral-Ground Voltage, Angle"
      n_upht_bus1_A = "n:upht:bus1:A", "p.u., Phase Technology dependent Voltage, Magnitude"
      n_upht_bus1_B = "n:upht:bus1:B", "p.u., Phase Technology dependent Voltage, Magnitude"
      n_upht_bus1_C = "n:upht:bus1:C", "p.u., Phase Technology dependent Voltage, Magnitude"
      n_upht_bus1_N = "n:upht:bus1:N", "p.u., Phase Technology dependent Voltage, Magnitude"
      n_Upht_bus1_A = "n:Upht:bus1:A", "kV, Phase Technology dependent Voltage, Magnitude"
      n_Upht_bus1_B = "n:Upht:bus1:B", "kV, Phase Technology dependent Voltage, Magnitude"
      n_Upht_bus1_C = "n:Upht:bus1:C", "kV, Phase Technology dependent Voltage, Magnitude"
      n_Upht_bus1_N = "n:Upht:bus1:N", "kV, Phase Technology dependent Voltage, Magnitude"
      n_um_bus1 = "n:um:bus1", "p.u., Average-Voltage, Magnitude"
      n_Um_bus1 = "n:Um:bus1", "kV, Average-Voltage, Magnitude"
      n_ishcr_bus1_A = "n:ishcr:bus1:A", "p.u., Short-Circuit Current, real"
      n_ishcr_bus1_B = "n:ishcr:bus1:B", "p.u., Short-Circuit Current, real"
      n_ishcr_bus1_C = "n:ishcr:bus1:C", "p.u., Short-Circuit Current, real"
      n_ishcr_bus1_N = "n:ishcr:bus1:N", "p.u., Short-Circuit Current, real"
      n_ishci_bus1_A = "n:ishci:bus1:A", "p.u., Short-Circuit Current, imag"
      n_ishci_bus1_B = "n:ishci:bus1:B", "p.u., Short-Circuit Current, imag"
      n_ishci_bus1_C = "n:ishci:bus1:C", "p.u., Short-Circuit Current, imag"
      n_ishci_bus1_N = "n:ishci:bus1:N", "p.u., Short-Circuit Current, imag"
      n_ishc_bus1_A = "n:ishc:bus1:A", "p.u., Short-Circuit Current, magnitude"
      n_ishc_bus1_B = "n:ishc:bus1:B", "p.u., Short-Circuit Current, magnitude"
      n_ishc_bus1_C = "n:ishc:bus1:C", "p.u., Short-Circuit Current, magnitude"
      n_ishc_bus1_N = "n:ishc:bus1:N", "p.u., Short-Circuit Current, magnitude"
      n_Ishc_bus1_A = "n:Ishc:bus1:A", "kA, Short-Circuit Current, magnitude"
      n_Ishc_bus1_B = "n:Ishc:bus1:B", "kA, Short-Circuit Current, magnitude"
      n_Ishc_bus1_C = "n:Ishc:bus1:C", "kA, Short-Circuit Current, magnitude"
      n_Ishc_bus1_N = "n:Ishc:bus1:N", "kA, Short-Circuit Current, magnitude"
      n_inshc_bus1 = "n:inshc:bus1", "p.u., Neutral Short-Circuit Current, magnitude"
      n_Inshc_bus1 = "n:Inshc:bus1", "kA, Neutral Short-Circuit Current, magnitude"
      n_fe_bus1 = "n:fe:bus1", "p.u., Electrical Frequency"
      n_frdev_bus1 = "n:frdev:bus1", "Hz, average frequency"
      n_dfehz_bus1 = "n:dfehz:bus1", "Hz, Deviation of the El. Frequency"
      n_dfedt_bus1 = "n:dfedt:bus1", "p.u./s, Derivative of El. Frequency"
      n_fehz_bus1 = "n:fehz:bus1", "Hz, Electrical Frequency"
      n_frnom_bus1 = "n:frnom:bus1", "Hz, Nominal Frequency"
      s_Pext = "s:Pext", "MW, IN, Load: Active Power, continuous, double"
      s_Qext = "s:Qext", "Mvar, IN, Load: Reactive Power, continuous, double"
      s_igena = "s:igena", "p.u., IN, Generation: Current, phase A, continuous, double"
      s_igenb = "s:igenb", "p.u., IN, Generation: Current, phase B, continuous, double"
      s_igenc = "s:igenc", "p.u., IN, Generation: Current, phase C, continuous, double"
      s_phiuigena = "s:phiuigena", "rad, IN, Generation: Angle u,i; phase A, continuous, double"
      s_phiuigenb = "s:phiuigenb", "rad, IN, Generation: Angle u,i; phase B, continuous, double"
      s_phiuigenc = "s:phiuigenc", "rad, IN, Generation: Angle u,i; phase C, continuous, double"
      c_sr_r = "c:sr:r", "MW, Complex Power (P+jQ), Phase a, Real Part"
      c_sr_i = "c:sr:i", "MW, Complex Power (P+jQ), Phase a, Imaginary Part"
      c_sr_phi = "c:sr:phi", "deg, Complex Power (P+jQ), Phase a, Angle"
      c_sr_mag = "c:sr:mag", "MW, Complex Power (P+jQ), Phase a, Magnitude"
      c_sr_phirad = "c:sr:phirad", "rad, Complex Power (P+jQ), Phase a, Angle"
      c_ss_r = "c:ss:r", "MW, Complex Power (P+jQ), Phase b, Real Part"
      c_ss_i = "c:ss:i", "MW, Complex Power (P+jQ), Phase b, Imaginary Part"
      c_ss_phi = "c:ss:phi", "deg, Complex Power (P+jQ), Phase b, Angle"
      c_ss_mag = "c:ss:mag", "MW, Complex Power (P+jQ), Phase b, Magnitude"
      c_ss_phirad = "c:ss:phirad", "rad, Complex Power (P+jQ), Phase b, Angle"
      c_st_r = "c:st:r", "MW, Complex Power (P+jQ), Phase c, Real Part"
      c_st_i = "c:st:i", "MW, Complex Power (P+jQ), Phase c, Imaginary Part"
      c_st_phi = "c:st:phi", "deg, Complex Power (P+jQ), Phase c, Angle"
      c_st_mag = "c:st:mag", "MW, Complex Power (P+jQ), Phase c, Magnitude"
      c_st_phirad = "c:st:phirad", "rad, Complex Power (P+jQ), Phase c, Angle"
      c_yr_r = "c:yr:r", "p.u., Load Admittance, Phase a, Real Part"
      c_yr_i = "c:yr:i", "p.u., Load Admittance, Phase a, Imaginary Part"
      c_yr_phi = "c:yr:phi", "deg, Load Admittance, Phase a, Angle"
      c_yr_mag = "c:yr:mag", "p.u., Load Admittance, Phase a, Magnitude"
      c_yr_phirad = "c:yr:phirad", "rad, Load Admittance, Phase a, Angle"
      c_ys_r = "c:ys:r", "p.u., Load Admittance, Phase b, Real Part"
      c_ys_i = "c:ys:i", "p.u., Load Admittance, Phase b, Imaginary Part"
      c_ys_phi = "c:ys:phi", "deg, Load Admittance, Phase b, Angle"
      c_ys_mag = "c:ys:mag", "p.u., Load Admittance, Phase b, Magnitude"
      c_ys_phirad = "c:ys:phirad", "rad, Load Admittance, Phase b, Angle"
      c_yt_r = "c:yt:r", "p.u., Load Admittance, Phase c, Real Part"
      c_yt_i = "c:yt:i", "p.u., Load Admittance, Phase c, Imaginary Part"
      c_yt_phi = "c:yt:phi", "deg, Load Admittance, Phase c, Angle"
      c_yt_mag = "c:yt:mag", "p.u., Load Admittance, Phase c, Magnitude"
      c_yt_phirad = "c:yt:phirad", "rad, Load Admittance, Phase c, Angle"
      c_yr0_r = "c:yr0:r", "p.u., Load Admittance, Phase a, Real Part"
      c_yr0_i = "c:yr0:i", "p.u., Load Admittance, Phase a, Imaginary Part"
      c_yr0_phi = "c:yr0:phi", "deg, Load Admittance, Phase a, Angle"
      c_yr0_mag = "c:yr0:mag", "p.u., Load Admittance, Phase a, Magnitude"
      c_yr0_phirad = "c:yr0:phirad", "rad, Load Admittance, Phase a, Angle"
      c_ys0_r = "c:ys0:r", "p.u., Load Admittance, Phase b, Real Part"
      c_ys0_i = "c:ys0:i", "p.u., Load Admittance, Phase b, Imaginary Part"
      c_ys0_phi = "c:ys0:phi", "deg, Load Admittance, Phase b, Angle"
      c_ys0_mag = "c:ys0:mag", "p.u., Load Admittance, Phase b, Magnitude"
      c_ys0_phirad = "c:ys0:phirad", "rad, Load Admittance, Phase b, Angle"
      c_yt0_r = "c:yt0:r", "p.u., Load Admittance, Phase c, Real Part"
      c_yt0_i = "c:yt0:i", "p.u., Load Admittance, Phase c, Imaginary Part"
      c_yt0_phi = "c:yt0:phi", "deg, Load Admittance, Phase c, Angle"
      c_yt0_mag = "c:yt0:mag", "p.u., Load Admittance, Phase c, Magnitude"
      c_yt0_phirad = "c:yt0:phirad", "rad, Load Admittance, Phase c, Angle"
      c_cn = "c:cn", "Connection Mode"
      c_z0_r = "c:z0:r", "p.u., z0, Real Part"
      c_z0_i = "c:z0:i", "p.u., z0, Imaginary Part"
      c_z0_phi = "c:z0:phi", "deg, z0, Angle"
      c_z0_mag = "c:z0:mag", "p.u., z0, Magnitude"
      c_z0_phirad = "c:z0:phirad", "rad, z0, Angle"
      e_iopt_type = "e:iopt_type", "Allow Load-Ramp Event"
      e_uk = "e:uk", "%, Short Circuit Impedance"
      e_Pcu = "e:Pcu", "kW, Copper Losses"
      e_r2 = "e:r2", "p.u., Resistance r2"
      e_x2 = "e:x2", "p.u., Reactance x2"

    class ElmPvsys(Enum):
      _init_ = 'value __doc__'

      m_u0_bus1 = "m:u0:bus1", "p.u., Zero-Sequence-Voltage, Magnitude"
      m_u0_bus1n = "m:u0:bus1n", "p.u., Zero-Sequence-Voltage, Magnitude"
      m_phiu0_bus1 = "m:phiu0:bus1", "deg, Zero-Sequence-Voltage, Angle"
      m_phiu0_bus1n = "m:phiu0:bus1n", "deg, Zero-Sequence-Voltage, Angle"
      m_u1r_bus1 = "m:u1r:bus1", "p.u., Positive-Sequence-Voltage, Real Part"
      m_u1r_bus1n = "m:u1r:bus1n", "p.u., Positive-Sequence-Voltage, Real Part"
      m_u1i_bus1 = "m:u1i:bus1", "p.u., Positive-Sequence-Voltage, Imaginary Part"
      m_u1i_bus1n = "m:u1i:bus1n", "p.u., Positive-Sequence-Voltage, Imaginary Part"
      m_u1_bus1 = "m:u1:bus1", "p.u., Positive-Sequence-Voltage, Magnitude"
      m_u1_bus1n = "m:u1:bus1n", "p.u., Positive-Sequence-Voltage, Magnitude"
      m_U1_bus1 = "m:U1:bus1", "kV, Line-Ground Positive-Sequence-Voltage, Magnitude"
      m_U1_bus1n = "m:U1:bus1n", "kV, Line-Ground Positive-Sequence-Voltage, Magnitude"
      m_U1l_bus1 = "m:U1l:bus1", "kV, Line-Line Positive-Sequence-Voltage, Magnitude"
      m_U1l_bus1n = "m:U1l:bus1n", "kV, Line-Line Positive-Sequence-Voltage, Magnitude"
      m_phiu1_bus1 = "m:phiu1:bus1", "deg, Positive-Sequence-Voltage, Angle"
      m_phiu1_bus1n = "m:phiu1:bus1n", "deg, Positive-Sequence-Voltage, Angle"
      m_u2_bus1 = "m:u2:bus1", "p.u., Negative-Sequence-Voltage, Magnitude"
      m_u2_bus1n = "m:u2:bus1n", "p.u., Negative-Sequence-Voltage, Magnitude"
      m_phiu2_bus1 = "m:phiu2:bus1", "deg, Negative-Sequence-Voltage, Angle"
      m_phiu2_bus1n = "m:phiu2:bus1n", "deg, Negative-Sequence-Voltage, Angle"
      m_inet_bus1_A = "m:inet:bus1:A", "p.u., Phase Current, Magnitude"
      m_inet_bus1_B = "m:inet:bus1:B", "p.u., Phase Current, Magnitude"
      m_inet_bus1_C = "m:inet:bus1:C", "p.u., Phase Current, Magnitude"
      m_inet_bus1_N = "m:inet:bus1:N", "p.u., Phase Current, Magnitude"
      m_inet_bus1n = "m:inet:bus1n", "p.u., Phase Current, Magnitude"
      m_I_bus1_A = "m:I:bus1:A", "kA, Phase Current, Magnitude"
      m_I_bus1_B = "m:I:bus1:B", "kA, Phase Current, Magnitude"
      m_I_bus1_C = "m:I:bus1:C", "kA, Phase Current, Magnitude"
      m_I_bus1_N = "m:I:bus1:N", "kA, Phase Current, Magnitude"
      m_I_bus1n = "m:I:bus1n", "kA, Phase Current, Magnitude"
      m_phii_bus1_A = "m:phii:bus1:A", "deg, Phase Current, Angle"
      m_phii_bus1_B = "m:phii:bus1:B", "deg, Phase Current, Angle"
      m_phii_bus1_C = "m:phii:bus1:C", "deg, Phase Current, Angle"
      m_phii_bus1_N = "m:phii:bus1:N", "deg, Phase Current, Angle"
      m_phii_bus1n = "m:phii:bus1n", "deg, Phase Current, Angle"
      m_i0r_bus1 = "m:i0r:bus1", "p.u., Zero-Sequence Current, Real Part"
      m_i0r_bus1n = "m:i0r:bus1n", "p.u., Zero-Sequence Current, Real Part"
      m_i0i_bus1 = "m:i0i:bus1", "p.u., Zero-Sequence Current, Imaginary Part"
      m_i0i_bus1n = "m:i0i:bus1n", "p.u., Zero-Sequence Current, Imaginary Part"
      m_i0_bus1 = "m:i0:bus1", "p.u., Zero-Sequence Current, Magnitude"
      m_i0_bus1n = "m:i0:bus1n", "p.u., Zero-Sequence Current, Magnitude"
      m_I0_bus1 = "m:I0:bus1", "kA, Zero-Sequence Current, Magnitude"
      m_I0_bus1n = "m:I0:bus1n", "kA, Zero-Sequence Current, Magnitude"
      m_I0x3_bus1 = "m:I0x3:bus1", "kA, 3*I0"
      m_I0x3_bus1n = "m:I0x3:bus1n", "kA, 3*I0"
      m_phii0_bus1 = "m:phii0:bus1", "deg, Zero-Sequence Current, Angle"
      m_phii0_bus1n = "m:phii0:bus1n", "deg, Zero-Sequence Current, Angle"
      m_i1r_bus1 = "m:i1r:bus1", "p.u., Positive-Sequence Current, Real Part"
      m_i1r_bus1n = "m:i1r:bus1n", "p.u., Positive-Sequence Current, Real Part"
      m_i1i_bus1 = "m:i1i:bus1", "p.u., Positive-Sequence Current, Imaginary Part"
      m_i1i_bus1n = "m:i1i:bus1n", "p.u., Positive-Sequence Current, Imaginary Part"
      m_i1_bus1 = "m:i1:bus1", "p.u., Positive-Sequence Current, Magnitude"
      m_i1_bus1n = "m:i1:bus1n", "p.u., Positive-Sequence Current, Magnitude"
      m_I1_bus1 = "m:I1:bus1", "kA, Positive-Sequence Current, Magnitude"
      m_I1_bus1n = "m:I1:bus1n", "kA, Positive-Sequence Current, Magnitude"
      m_phii1_bus1 = "m:phii1:bus1", "deg, Positive-Sequence Current, Angle"
      m_phii1_bus1n = "m:phii1:bus1n", "deg, Positive-Sequence Current, Angle"
      m_i2r_bus1 = "m:i2r:bus1", "p.u., Negative-Sequence Current, Real Part"
      m_i2r_bus1n = "m:i2r:bus1n", "p.u., Negative-Sequence Current, Real Part"
      m_i2i_bus1 = "m:i2i:bus1", "p.u., Negative-Sequence Current, Imaginary Part"
      m_i2i_bus1n = "m:i2i:bus1n", "p.u., Negative-Sequence Current, Imaginary Part"
      m_i2_bus1 = "m:i2:bus1", "p.u., Negative-Sequence Current, Magnitude"
      m_i2_bus1n = "m:i2:bus1n", "p.u., Negative-Sequence Current, Magnitude"
      m_I2_bus1 = "m:I2:bus1", "kA, Negative-Sequence Current, Magnitude"
      m_I2_bus1n = "m:I2:bus1n", "kA, Negative-Sequence Current, Magnitude"
      m_phii2_bus1 = "m:phii2:bus1", "deg, Negative-Sequence Current, Angle"
      m_phii2_bus1n = "m:phii2:bus1n", "deg, Negative-Sequence Current, Angle"
      m_P_bus1_A = "m:P:bus1:A", "MW, Active Power"
      m_P_bus1_B = "m:P:bus1:B", "MW, Active Power"
      m_P_bus1_C = "m:P:bus1:C", "MW, Active Power"
      m_P_bus1_N = "m:P:bus1:N", "MW, Active Power"
      m_P_bus1n = "m:P:bus1n", "MW, Active Power"
      m_Q_bus1_A = "m:Q:bus1:A", "Mvar, Reactive Power"
      m_Q_bus1_B = "m:Q:bus1:B", "Mvar, Reactive Power"
      m_Q_bus1_C = "m:Q:bus1:C", "Mvar, Reactive Power"
      m_Q_bus1_N = "m:Q:bus1:N", "Mvar, Reactive Power"
      m_Q_bus1n = "m:Q:bus1n", "Mvar, Reactive Power"
      m_S_bus1_A = "m:S:bus1:A", "MVA, Apparent Power"
      m_S_bus1_B = "m:S:bus1:B", "MVA, Apparent Power"
      m_S_bus1_C = "m:S:bus1:C", "MVA, Apparent Power"
      m_S_bus1_N = "m:S:bus1:N", "MVA, Apparent Power"
      m_S_bus1n = "m:S:bus1n", "MVA, Apparent Power"
      m_cosphi_bus1_A = "m:cosphi:bus1:A", "Power Factor"
      m_cosphi_bus1_B = "m:cosphi:bus1:B", "Power Factor"
      m_cosphi_bus1_C = "m:cosphi:bus1:C", "Power Factor"
      m_cosphi_bus1_N = "m:cosphi:bus1:N", "Power Factor"
      m_cosphi_bus1n = "m:cosphi:bus1n", "Power Factor"
      m_tanphi_bus1_A = "m:tanphi:bus1:A", "tan(phi)"
      m_tanphi_bus1_B = "m:tanphi:bus1:B", "tan(phi)"
      m_tanphi_bus1_C = "m:tanphi:bus1:C", "tan(phi)"
      m_tanphi_bus1_N = "m:tanphi:bus1:N", "tan(phi)"
      m_tanphi_bus1n = "m:tanphi:bus1n", "tan(phi)"
      m_Psum_bus1 = "m:Psum:bus1", "MW, Total Active Power"
      m_Psum_bus1n = "m:Psum:bus1n", "MW, Total Active Power"
      m_Qsum_bus1 = "m:Qsum:bus1", "Mvar, Total Reactive Power"
      m_Qsum_bus1n = "m:Qsum:bus1n", "Mvar, Total Reactive Power"
      m_Ssum_bus1 = "m:Ssum:bus1", "MVA, Total Apparent Power"
      m_Ssum_bus1n = "m:Ssum:bus1n", "MVA, Total Apparent Power"
      m_cosphisum_bus1 = "m:cosphisum:bus1", "Total Power Factor"
      m_cosphisum_bus1n = "m:cosphisum:bus1n", "Total Power Factor"
      m_tanphisum_bus1 = "m:tanphisum:bus1", "Total tan(phi)"
      m_tanphisum_bus1n = "m:tanphisum:bus1n", "Total tan(phi)"
      m_Pln_bus1_A = "m:Pln:bus1:A", "MW, Active Power (Line-Neutral)"
      m_Pln_bus1_B = "m:Pln:bus1:B", "MW, Active Power (Line-Neutral)"
      m_Pln_bus1_C = "m:Pln:bus1:C", "MW, Active Power (Line-Neutral)"
      m_Pln_bus1_N = "m:Pln:bus1:N", "MW, Active Power (Line-Neutral)"
      m_Pln_bus1n = "m:Pln:bus1n", "MW, Active Power (Line-Neutral)"
      m_Qln_bus1_A = "m:Qln:bus1:A", "Mvar, Reactive Power (Line-Neutral)"
      m_Qln_bus1_B = "m:Qln:bus1:B", "Mvar, Reactive Power (Line-Neutral)"
      m_Qln_bus1_C = "m:Qln:bus1:C", "Mvar, Reactive Power (Line-Neutral)"
      m_Qln_bus1_N = "m:Qln:bus1:N", "Mvar, Reactive Power (Line-Neutral)"
      m_Qln_bus1n = "m:Qln:bus1n", "Mvar, Reactive Power (Line-Neutral)"
      m_P1_bus1 = "m:P1:bus1", "MW, Positive Sequence Active Power"
      m_P1_bus1n = "m:P1:bus1n", "MW, Positive Sequence Active Power"
      m_Q1_bus1 = "m:Q1:bus1", "Mvar, Positive Sequence Reactive Power"
      m_Q1_bus1n = "m:Q1:bus1n", "Mvar, Positive Sequence Reactive Power"
      m_P2_bus1 = "m:P2:bus1", "MW, Negative Sequence Active Power"
      m_P2_bus1n = "m:P2:bus1n", "MW, Negative Sequence Active Power"
      m_Q2_bus1 = "m:Q2:bus1", "Mvar, Negative Sequence Reactive Power"
      m_Q2_bus1n = "m:Q2:bus1n", "Mvar, Negative Sequence Reactive Power"
      m_P0_bus1 = "m:P0:bus1", "MW, Zero Sequence Active Power"
      m_P0_bus1n = "m:P0:bus1n", "MW, Zero Sequence Active Power"
      m_Q0_bus1 = "m:Q0:bus1", "Mvar, Zero Sequence Reactive Power"
      m_Q0_bus1n = "m:Q0:bus1n", "Mvar, Zero Sequence Reactive Power"
      m_i1P_bus1 = "m:i1P:bus1", "p.u., Positive-Sequence Active Current"
      m_i1P_bus1n = "m:i1P:bus1n", "p.u., Positive-Sequence Active Current"
      m_i1Q_bus1 = "m:i1Q:bus1", "p.u., Positive-Sequence Reactive Current"
      m_i1Q_bus1n = "m:i1Q:bus1n", "p.u., Positive-Sequence Reactive Current"
      m_I1P_bus1 = "m:I1P:bus1", "kA, Positive-Sequence Active Current"
      m_I1P_bus1n = "m:I1P:bus1n", "kA, Positive-Sequence Active Current"
      m_I1Q_bus1 = "m:I1Q:bus1", "kA, Positive-Sequence Reactive Current"
      m_I1Q_bus1n = "m:I1Q:bus1n", "kA, Positive-Sequence Reactive Current"
      m_i2P_bus1 = "m:i2P:bus1", "p.u., Negative-Sequence Active Current"
      m_i2P_bus1n = "m:i2P:bus1n", "p.u., Negative-Sequence Active Current"
      m_i2Q_bus1 = "m:i2Q:bus1", "p.u., Negative-Sequence Reactive Current"
      m_i2Q_bus1n = "m:i2Q:bus1n", "p.u., Negative-Sequence Reactive Current"
      m_I2P_bus1 = "m:I2P:bus1", "kA, Negative-Sequence Active Current"
      m_I2P_bus1n = "m:I2P:bus1n", "kA, Negative-Sequence Active Current"
      m_I2Q_bus1 = "m:I2Q:bus1", "kA, Negative-Sequence Reactive Current"
      m_I2Q_bus1n = "m:I2Q:bus1n", "kA, Negative-Sequence Reactive Current"
      m_ur_bus1_A = "m:ur:bus1:A", "p.u., Phase Voltage, Real Part"
      m_ur_bus1_B = "m:ur:bus1:B", "p.u., Phase Voltage, Real Part"
      m_ur_bus1_C = "m:ur:bus1:C", "p.u., Phase Voltage, Real Part"
      m_ur_bus1_N = "m:ur:bus1:N", "p.u., Phase Voltage, Real Part"
      m_ur_bus1n = "m:ur:bus1n", "p.u., Phase Voltage, Real Part"
      m_ui_bus1_A = "m:ui:bus1:A", "p.u., Phase Voltage, Imaginary Part"
      m_ui_bus1_B = "m:ui:bus1:B", "p.u., Phase Voltage, Imaginary Part"
      m_ui_bus1_C = "m:ui:bus1:C", "p.u., Phase Voltage, Imaginary Part"
      m_ui_bus1_N = "m:ui:bus1:N", "p.u., Phase Voltage, Imaginary Part"
      m_ui_bus1n = "m:ui:bus1n", "p.u., Phase Voltage, Imaginary Part"
      m_u_bus1_A = "m:u:bus1:A", "p.u., Phase Voltage, Magnitude"
      m_u_bus1_B = "m:u:bus1:B", "p.u., Phase Voltage, Magnitude"
      m_u_bus1_C = "m:u:bus1:C", "p.u., Phase Voltage, Magnitude"
      m_u_bus1_N = "m:u:bus1:N", "p.u., Phase Voltage, Magnitude"
      m_u_bus1n = "m:u:bus1n", "p.u., Phase Voltage, Magnitude"
      m_ir_bus1_A = "m:ir:bus1:A", "p.u., Phase Current, Real Part"
      m_ir_bus1_B = "m:ir:bus1:B", "p.u., Phase Current, Real Part"
      m_ir_bus1_C = "m:ir:bus1:C", "p.u., Phase Current, Real Part"
      m_ir_bus1_N = "m:ir:bus1:N", "p.u., Phase Current, Real Part"
      m_ir_bus1n = "m:ir:bus1n", "p.u., Phase Current, Real Part"
      m_ii_bus1_A = "m:ii:bus1:A", "p.u., Phase Current, Imaginary Part"
      m_ii_bus1_B = "m:ii:bus1:B", "p.u., Phase Current, Imaginary Part"
      m_ii_bus1_C = "m:ii:bus1:C", "p.u., Phase Current, Imaginary Part"
      m_ii_bus1_N = "m:ii:bus1:N", "p.u., Phase Current, Imaginary Part"
      m_ii_bus1n = "m:ii:bus1n", "p.u., Phase Current, Imaginary Part"
      m_i_bus1_A = "m:i:bus1:A", "p.u., Phase Current, Magnitude"
      m_i_bus1_B = "m:i:bus1:B", "p.u., Phase Current, Magnitude"
      m_i_bus1_C = "m:i:bus1:C", "p.u., Phase Current, Magnitude"
      m_i_bus1_N = "m:i:bus1:N", "p.u., Phase Current, Magnitude"
      m_i_bus1n = "m:i:bus1n", "p.u., Phase Current, Magnitude"
      m_phiui_bus1_A = "m:phiui:bus1:A", "deg, Angle between Voltage and Current"
      m_phiui_bus1_B = "m:phiui:bus1:B", "deg, Angle between Voltage and Current"
      m_phiui_bus1_C = "m:phiui:bus1:C", "deg, Angle between Voltage and Current"
      m_phiui_bus1_N = "m:phiui:bus1:N", "deg, Angle between Voltage and Current"
      m_phiui_bus1n = "m:phiui:bus1n", "deg, Angle between Voltage and Current"
      m_phiu0i0_bus1 = "m:phiu0i0:bus1", "deg, Angle between Voltage and Current in zero-sequence system"
      m_phiu0i0_bus1n = "m:phiu0i0:bus1n", "deg, Angle between Voltage and Current in zero-sequence system"
      m_phiu1i1_bus1 = "m:phiu1i1:bus1", "deg, Angle between Voltage and Current in positive sequence system"
      m_phiu1i1_bus1n = "m:phiu1i1:bus1n", "deg, Angle between Voltage and Current in positive sequence system"
      m_phiu2i2_bus1 = "m:phiu2i2:bus1", "deg, Angle between Voltage and Current in negative sequence system"
      m_phiu2i2_bus1n = "m:phiu2i2:bus1n", "deg, Angle between Voltage and Current in negative sequence system"
      m_Tfct_bus1 = "m:Tfct:bus1", "s, Fault Clearing Time"
      m_Tfct_bus1n = "m:Tfct:bus1n", "s, Fault Clearing Time"
      m_TfctPh_bus1_A = "m:TfctPh:bus1:A", "s, Fault Clearing Time"
      m_TfctPh_bus1_B = "m:TfctPh:bus1:B", "s, Fault Clearing Time"
      m_TfctPh_bus1_C = "m:TfctPh:bus1:C", "s, Fault Clearing Time"
      m_TfctPh_bus1_N = "m:TfctPh:bus1:N", "s, Fault Clearing Time"
      m_TfctPh_bus1n = "m:TfctPh:bus1n", "s, Fault Clearing Time"
      m_Brkload_bus1 = "m:Brkload:bus1", "%, Max. Breaker Loading"
      m_Brkload_bus1n = "m:Brkload:bus1n", "%, Max. Breaker Loading"
      m_BrkloadPh_bus1_A = "m:BrkloadPh:bus1:A", "%, Breaker Loading"
      m_BrkloadPh_bus1_B = "m:BrkloadPh:bus1:B", "%, Breaker Loading"
      m_BrkloadPh_bus1_C = "m:BrkloadPh:bus1:C", "%, Breaker Loading"
      m_BrkloadPh_bus1_N = "m:BrkloadPh:bus1:N", "%, Breaker Loading"
      m_BrkloadPh_bus1n = "m:BrkloadPh:bus1n", "%, Breaker Loading"
      n_ur_bus1_A = "n:ur:bus1:A", "p.u., Line-Ground Voltage, Real Part"
      n_ur_bus1_B = "n:ur:bus1:B", "p.u., Line-Ground Voltage, Real Part"
      n_ur_bus1_C = "n:ur:bus1:C", "p.u., Line-Ground Voltage, Real Part"
      n_ur_bus1_N = "n:ur:bus1:N", "p.u., Line-Ground Voltage, Real Part"
      n_ur_bus1n = "n:ur:bus1n", "p.u., Line-Ground Voltage, Real Part"
      n_ui_bus1_A = "n:ui:bus1:A", "p.u., Line-Ground Voltage, Imaginary Part"
      n_ui_bus1_B = "n:ui:bus1:B", "p.u., Line-Ground Voltage, Imaginary Part"
      n_ui_bus1_C = "n:ui:bus1:C", "p.u., Line-Ground Voltage, Imaginary Part"
      n_ui_bus1_N = "n:ui:bus1:N", "p.u., Line-Ground Voltage, Imaginary Part"
      n_ui_bus1n = "n:ui:bus1n", "p.u., Line-Ground Voltage, Imaginary Part"
      n_u_bus1_A = "n:u:bus1:A", "p.u., Line-Ground Voltage, Magnitude"
      n_u_bus1_B = "n:u:bus1:B", "p.u., Line-Ground Voltage, Magnitude"
      n_u_bus1_C = "n:u:bus1:C", "p.u., Line-Ground Voltage, Magnitude"
      n_u_bus1_N = "n:u:bus1:N", "p.u., Line-Ground Voltage, Magnitude"
      n_u_bus1n = "n:u:bus1n", "p.u., Line-Ground Voltage, Magnitude"
      n_upc_bus1_A = "n:upc:bus1:A", "%, Line-Ground Voltage, Magnitude"
      n_upc_bus1_B = "n:upc:bus1:B", "%, Line-Ground Voltage, Magnitude"
      n_upc_bus1_C = "n:upc:bus1:C", "%, Line-Ground Voltage, Magnitude"
      n_upc_bus1_N = "n:upc:bus1:N", "%, Line-Ground Voltage, Magnitude"
      n_upc_bus1n = "n:upc:bus1n", "%, Line-Ground Voltage, Magnitude"
      n_U_bus1_A = "n:U:bus1:A", "kV, Line-Ground Voltage, Magnitude"
      n_U_bus1_B = "n:U:bus1:B", "kV, Line-Ground Voltage, Magnitude"
      n_U_bus1_C = "n:U:bus1:C", "kV, Line-Ground Voltage, Magnitude"
      n_U_bus1_N = "n:U:bus1:N", "kV, Line-Ground Voltage, Magnitude"
      n_U_bus1n = "n:U:bus1n", "kV, Line-Ground Voltage, Magnitude"
      n_phiu_bus1_A = "n:phiu:bus1:A", "deg, Line-Ground Voltage, Angle"
      n_phiu_bus1_B = "n:phiu:bus1:B", "deg, Line-Ground Voltage, Angle"
      n_phiu_bus1_C = "n:phiu:bus1:C", "deg, Line-Ground Voltage, Angle"
      n_phiu_bus1_N = "n:phiu:bus1:N", "deg, Line-Ground Voltage, Angle"
      n_phiu_bus1n = "n:phiu:bus1n", "deg, Line-Ground Voltage, Angle"
      n_phiurel_bus1 = "n:phiurel:bus1", "deg, Voltage, Relative Angle"
      n_phiurel_bus1n = "n:phiurel:bus1n", "deg, Voltage, Relative Angle"
      n_ul_bus1_A = "n:ul:bus1:A", "p.u., Line-Line Voltage, Magnitude"
      n_ul_bus1_B = "n:ul:bus1:B", "p.u., Line-Line Voltage, Magnitude"
      n_ul_bus1_C = "n:ul:bus1:C", "p.u., Line-Line Voltage, Magnitude"
      n_ul_bus1_N = "n:ul:bus1:N", "p.u., Line-Line Voltage, Magnitude"
      n_ul_bus1n = "n:ul:bus1n", "p.u., Line-Line Voltage, Magnitude"
      n_ulpc_bus1_A = "n:ulpc:bus1:A", "%, Line-Line Voltage, Magnitude"
      n_ulpc_bus1_B = "n:ulpc:bus1:B", "%, Line-Line Voltage, Magnitude"
      n_ulpc_bus1_C = "n:ulpc:bus1:C", "%, Line-Line Voltage, Magnitude"
      n_ulpc_bus1_N = "n:ulpc:bus1:N", "%, Line-Line Voltage, Magnitude"
      n_ulpc_bus1n = "n:ulpc:bus1n", "%, Line-Line Voltage, Magnitude"
      n_Ul_bus1_A = "n:Ul:bus1:A", "kV, Line-Line Voltage, Magnitude"
      n_Ul_bus1_B = "n:Ul:bus1:B", "kV, Line-Line Voltage, Magnitude"
      n_Ul_bus1_C = "n:Ul:bus1:C", "kV, Line-Line Voltage, Magnitude"
      n_Ul_bus1_N = "n:Ul:bus1:N", "kV, Line-Line Voltage, Magnitude"
      n_Ul_bus1n = "n:Ul:bus1n", "kV, Line-Line Voltage, Magnitude"
      n_phiul_bus1_A = "n:phiul:bus1:A", "deg, Line-Line Voltage, Angle"
      n_phiul_bus1_B = "n:phiul:bus1:B", "deg, Line-Line Voltage, Angle"
      n_phiul_bus1_C = "n:phiul:bus1:C", "deg, Line-Line Voltage, Angle"
      n_phiul_bus1_N = "n:phiul:bus1:N", "deg, Line-Line Voltage, Angle"
      n_phiul_bus1n = "n:phiul:bus1n", "deg, Line-Line Voltage, Angle"
      n_u0_bus1 = "n:u0:bus1", "p.u., Zero-Sequence Voltage, Magnitude"
      n_u0_bus1n = "n:u0:bus1n", "p.u., Zero-Sequence Voltage, Magnitude"
      n_U0_bus1 = "n:U0:bus1", "kV, Zero-Sequence Voltage, Magnitude"
      n_U0_bus1n = "n:U0:bus1n", "kV, Zero-Sequence Voltage, Magnitude"
      n_U0x3_bus1 = "n:U0x3:bus1", "kV, 3*U0"
      n_U0x3_bus1n = "n:U0x3:bus1n", "kV, 3*U0"
      n_phiu0_bus1 = "n:phiu0:bus1", "deg, Zero-Sequence Voltage, Angle"
      n_phiu0_bus1n = "n:phiu0:bus1n", "deg, Zero-Sequence Voltage, Angle"
      n_u1_bus1 = "n:u1:bus1", "p.u., Positive-Sequence Voltage, Magnitude"
      n_u1_bus1n = "n:u1:bus1n", "p.u., Positive-Sequence Voltage, Magnitude"
      n_u1pc_bus1 = "n:u1pc:bus1", "%, Positive-Sequence Voltage, Magnitude"
      n_u1pc_bus1n = "n:u1pc:bus1n", "%, Positive-Sequence Voltage, Magnitude"
      n_u1r_bus1 = "n:u1r:bus1", "p.u., Positive-Sequence Voltage, Real Part"
      n_u1r_bus1n = "n:u1r:bus1n", "p.u., Positive-Sequence Voltage, Real Part"
      n_u1i_bus1 = "n:u1i:bus1", "p.u., Positive-Sequence Voltage, Imaginary Part"
      n_u1i_bus1n = "n:u1i:bus1n", "p.u., Positive-Sequence Voltage, Imaginary Part"
      n_U1_bus1 = "n:U1:bus1", "kV, Line-Ground Positive-Sequence Voltage, Magnitude"
      n_U1_bus1n = "n:U1:bus1n", "kV, Line-Ground Positive-Sequence Voltage, Magnitude"
      n_phiu1_bus1 = "n:phiu1:bus1", "deg, Positive-Sequence Voltage, Angle"
      n_phiu1_bus1n = "n:phiu1:bus1n", "deg, Positive-Sequence Voltage, Angle"
      n_u2_bus1 = "n:u2:bus1", "p.u., Negative-Sequence Voltage, Magnitude"
      n_u2_bus1n = "n:u2:bus1n", "p.u., Negative-Sequence Voltage, Magnitude"
      n_U2_bus1 = "n:U2:bus1", "kV, Line-Ground Negative-Sequence Voltage, Magnitude"
      n_U2_bus1n = "n:U2:bus1n", "kV, Line-Ground Negative-Sequence Voltage, Magnitude"
      n_phiu2_bus1 = "n:phiu2:bus1", "deg, Negative-Sequence Voltage, Angle"
      n_phiu2_bus1n = "n:phiu2:bus1n", "deg, Negative-Sequence Voltage, Angle"
      n_U1l_bus1 = "n:U1l:bus1", "kV, Line-Line Positive-Sequence Voltage, Magnitude"
      n_U1l_bus1n = "n:U1l:bus1n", "kV, Line-Line Positive-Sequence Voltage, Magnitude"
      n_U2l_bus1 = "n:U2l:bus1", "kV, Line-Line Negative-Sequence Voltage, Magnitude"
      n_U2l_bus1n = "n:U2l:bus1n", "kV, Line-Line Negative-Sequence Voltage, Magnitude"
      n_uln_bus1_A = "n:uln:bus1:A", "p.u., Line-Neutral Voltage, Magnitude"
      n_uln_bus1_B = "n:uln:bus1:B", "p.u., Line-Neutral Voltage, Magnitude"
      n_uln_bus1_C = "n:uln:bus1:C", "p.u., Line-Neutral Voltage, Magnitude"
      n_uln_bus1_N = "n:uln:bus1:N", "p.u., Line-Neutral Voltage, Magnitude"
      n_uln_bus1n = "n:uln:bus1n", "p.u., Line-Neutral Voltage, Magnitude"
      n_Uln_bus1_A = "n:Uln:bus1:A", "kV, Line-Neutral Voltage, Magnitude"
      n_Uln_bus1_B = "n:Uln:bus1:B", "kV, Line-Neutral Voltage, Magnitude"
      n_Uln_bus1_C = "n:Uln:bus1:C", "kV, Line-Neutral Voltage, Magnitude"
      n_Uln_bus1_N = "n:Uln:bus1:N", "kV, Line-Neutral Voltage, Magnitude"
      n_Uln_bus1n = "n:Uln:bus1n", "kV, Line-Neutral Voltage, Magnitude"
      n_phiuln_bus1_A = "n:phiuln:bus1:A", "deg, Line-Neutral Voltage, Angle"
      n_phiuln_bus1_B = "n:phiuln:bus1:B", "deg, Line-Neutral Voltage, Angle"
      n_phiuln_bus1_C = "n:phiuln:bus1:C", "deg, Line-Neutral Voltage, Angle"
      n_phiuln_bus1_N = "n:phiuln:bus1:N", "deg, Line-Neutral Voltage, Angle"
      n_phiuln_bus1n = "n:phiuln:bus1n", "deg, Line-Neutral Voltage, Angle"
      n_un_bus1 = "n:un:bus1", "p.u., Neutral-Ground Voltage, Magnitude"
      n_un_bus1n = "n:un:bus1n", "p.u., Neutral-Ground Voltage, Magnitude"
      n_Un_bus1 = "n:Un:bus1", "kV, Neutral-Ground Voltage, Magnitude"
      n_Un_bus1n = "n:Un:bus1n", "kV, Neutral-Ground Voltage, Magnitude"
      n_phiun_bus1 = "n:phiun:bus1", "deg, Neutral-Ground Voltage, Angle"
      n_phiun_bus1n = "n:phiun:bus1n", "deg, Neutral-Ground Voltage, Angle"
      n_upht_bus1_A = "n:upht:bus1:A", "p.u., Phase Technology dependent Voltage, Magnitude"
      n_upht_bus1_B = "n:upht:bus1:B", "p.u., Phase Technology dependent Voltage, Magnitude"
      n_upht_bus1_C = "n:upht:bus1:C", "p.u., Phase Technology dependent Voltage, Magnitude"
      n_upht_bus1_N = "n:upht:bus1:N", "p.u., Phase Technology dependent Voltage, Magnitude"
      n_upht_bus1n = "n:upht:bus1n", "p.u., Phase Technology dependent Voltage, Magnitude"
      n_Upht_bus1_A = "n:Upht:bus1:A", "kV, Phase Technology dependent Voltage, Magnitude"
      n_Upht_bus1_B = "n:Upht:bus1:B", "kV, Phase Technology dependent Voltage, Magnitude"
      n_Upht_bus1_C = "n:Upht:bus1:C", "kV, Phase Technology dependent Voltage, Magnitude"
      n_Upht_bus1_N = "n:Upht:bus1:N", "kV, Phase Technology dependent Voltage, Magnitude"
      n_Upht_bus1n = "n:Upht:bus1n", "kV, Phase Technology dependent Voltage, Magnitude"
      n_um_bus1 = "n:um:bus1", "p.u., Average-Voltage, Magnitude"
      n_um_bus1n = "n:um:bus1n", "p.u., Average-Voltage, Magnitude"
      n_Um_bus1 = "n:Um:bus1", "kV, Average-Voltage, Magnitude"
      n_Um_bus1n = "n:Um:bus1n", "kV, Average-Voltage, Magnitude"
      n_ishcr_bus1_A = "n:ishcr:bus1:A", "p.u., Short-Circuit Current, real"
      n_ishcr_bus1_B = "n:ishcr:bus1:B", "p.u., Short-Circuit Current, real"
      n_ishcr_bus1_C = "n:ishcr:bus1:C", "p.u., Short-Circuit Current, real"
      n_ishcr_bus1_N = "n:ishcr:bus1:N", "p.u., Short-Circuit Current, real"
      n_ishcr_bus1n = "n:ishcr:bus1n", "p.u., Short-Circuit Current, real"
      n_ishci_bus1_A = "n:ishci:bus1:A", "p.u., Short-Circuit Current, imag"
      n_ishci_bus1_B = "n:ishci:bus1:B", "p.u., Short-Circuit Current, imag"
      n_ishci_bus1_C = "n:ishci:bus1:C", "p.u., Short-Circuit Current, imag"
      n_ishci_bus1_N = "n:ishci:bus1:N", "p.u., Short-Circuit Current, imag"
      n_ishci_bus1n = "n:ishci:bus1n", "p.u., Short-Circuit Current, imag"
      n_ishc_bus1_A = "n:ishc:bus1:A", "p.u., Short-Circuit Current, magnitude"
      n_ishc_bus1_B = "n:ishc:bus1:B", "p.u., Short-Circuit Current, magnitude"
      n_ishc_bus1_C = "n:ishc:bus1:C", "p.u., Short-Circuit Current, magnitude"
      n_ishc_bus1_N = "n:ishc:bus1:N", "p.u., Short-Circuit Current, magnitude"
      n_ishc_bus1n = "n:ishc:bus1n", "p.u., Short-Circuit Current, magnitude"
      n_Ishc_bus1_A = "n:Ishc:bus1:A", "kA, Short-Circuit Current, magnitude"
      n_Ishc_bus1_B = "n:Ishc:bus1:B", "kA, Short-Circuit Current, magnitude"
      n_Ishc_bus1_C = "n:Ishc:bus1:C", "kA, Short-Circuit Current, magnitude"
      n_Ishc_bus1_N = "n:Ishc:bus1:N", "kA, Short-Circuit Current, magnitude"
      n_Ishc_bus1n = "n:Ishc:bus1n", "kA, Short-Circuit Current, magnitude"
      n_inshc_bus1 = "n:inshc:bus1", "p.u., Neutral Short-Circuit Current, magnitude"
      n_inshc_bus1n = "n:inshc:bus1n", "p.u., Neutral Short-Circuit Current, magnitude"
      n_Inshc_bus1 = "n:Inshc:bus1", "kA, Neutral Short-Circuit Current, magnitude"
      n_Inshc_bus1n = "n:Inshc:bus1n", "kA, Neutral Short-Circuit Current, magnitude"
      n_fe_bus1 = "n:fe:bus1", "p.u., Electrical Frequency"
      n_fe_bus1n = "n:fe:bus1n", "p.u., Electrical Frequency"
      n_frdev_bus1 = "n:frdev:bus1", "Hz, average frequency"
      n_frdev_bus1n = "n:frdev:bus1n", "Hz, average frequency"
      n_dfehz_bus1 = "n:dfehz:bus1", "Hz, Deviation of the El. Frequency"
      n_dfehz_bus1n = "n:dfehz:bus1n", "Hz, Deviation of the El. Frequency"
      n_dfedt_bus1 = "n:dfedt:bus1", "p.u./s, Derivative of El. Frequency"
      n_dfedt_bus1n = "n:dfedt:bus1n", "p.u./s, Derivative of El. Frequency"
      n_fehz_bus1 = "n:fehz:bus1", "Hz, Electrical Frequency"
      n_fehz_bus1n = "n:fehz:bus1n", "Hz, Electrical Frequency"
      n_frnom_bus1 = "n:frnom:bus1", "Hz, Nominal Frequency"
      n_frnom_bus1n = "n:frnom:bus1n", "Hz, Nominal Frequency"
      s_f0 = "s:f0", "p.u., IN, Frequency input continuous, double"
      s_F0Hz = "s:F0Hz", "Hz, IN, Frequency, input (Hz), continuous, double"
      s_fref = "s:fref", "p.u., IN, Reference frequency, continuous, double"
      s_id_ref = "s:id_ref", "p.u., IN, d-axis current reference, continuous, double"
      s_iq_ref = "s:iq_ref", "p.u., IN, q-axis current reference, continuous, double"
      s_i2d_ref = "s:i2d_ref", "p.u., IN, Negative sequence current, d-axis reference, continuous, double"
      s_i2q_ref = "s:i2q_ref", "p.u., IN, Negative sequence current, q-axis reference, continuous, double"
      s_u1r_in = "s:u1r_in", "p.u., IN, Positive sequence voltage input, real part, continuous, double"
      s_u1i_in = "s:u1i_in", "p.u., IN, Positive sequence voltage input, imaginary part continuous, double"
      s_u2r_in = "s:u2r_in", "p.u., IN, Negative sequence voltage input, real part, continuous, double"
      s_u2i_in = "s:u2i_in", "p.u., IN, Negative sequence voltage input, imaginary part continuous, double"
      s_cosref = "s:cosref", "IN, cos(positive sequence dq-reference angle), continuous, double"
      s_sinref = "s:sinref", "IN, sin(positive sequence dq-reference angle), continuous, double"
      s_cos2ref = "s:cos2ref", "IN, cos(negative sequence dq-reference angle), continuous, double"
      s_sin2ref = "s:sin2ref", "IN, sin(negative sequence dq-reference angle), continuous, double"
      s_u0r_in = "s:u0r_in", "p.u., IN, Zero sequence voltage input, real part, continuous, double"
      s_u0i_in = "s:u0i_in", "p.u., IN, Zero sequence voltage input, imaginary part, continuous, double"
      s_xspeed = "s:xspeed", "p.u., OUT, Frequency, continuous, double"
      s_id = "s:id", "p.u., OUT, Current, d-Axis continuous, double"
      s_iq = "s:iq", "p.u., OUT, Current, q-Axis continuous, double"
      c_sgnn = "c:sgnn", "MVA, Total rated apparent power (ngnum*sgn)"
      c_cosn = "c:cosn", "Nominal power factor"
      c_hpi = "c:hpi", "rad/s, Nominal angular frequency"
      c_umin = "c:umin", "p.u., Min. operation voltage"
      c_phiu = "c:phiu", "rad, Voltage angle"
      e_iSimModel = "e:iSimModel", "Model"
      e_iAstabint = "e:iAstabint", "A-stable integration algorithm"
      e_umin = "e:umin", "p.u., Switch-off threshold"
      e_uonthr = "e:uonthr", "p.u., Switch-on threshold"
      e_Tondelay = "e:Tondelay", "s, Switch-on delay"
      e_uk = "e:uk", "%, Short circuit impedance"
      e_Pcu = "e:Pcu", "kW, Copper losses"
      e_R0toR1 = "e:R0toR1", "R0/R1 ratio"
      e_X0toX1 = "e:X0toX1", "X0/X1 ratio"
      e_iDelay = "e:iDelay", "dq-reference angle delay (if dq-reference signals not connected)"
      e_Tdelay = "e:Tdelay", "s, Delay time constant"
      e_iUseNegSeqAngle = "e:iUseNegSeqAngle", "Current source model: Use negative sequence angle for negative sequence current source"
      e_iUseZsr = "e:iUseZsr", "Use series reactor impedance, if negative sequence voltage signals are connected"
      e_r2 = "e:r2", "p.u., Resistance r2"
      e_x2 = "e:x2", "p.u., Reactance x2"
      e_pinput = "e:pinput", "Active Power signal"
      e_qinput = "e:qinput", "Reactive Power signal"

    class ElmSecctrl(Enum):
      _init_ = 'value __doc__'


    class ElmShnt(Enum):
      _init_ = 'value __doc__'

      m_u0_bus1 = "m:u0:bus1", "p.u., Zero-Sequence-Voltage, Magnitude"
      m_u0_bus1n = "m:u0:bus1n", "p.u., Zero-Sequence-Voltage, Magnitude"
      m_phiu0_bus1 = "m:phiu0:bus1", "deg, Zero-Sequence-Voltage, Angle"
      m_phiu0_bus1n = "m:phiu0:bus1n", "deg, Zero-Sequence-Voltage, Angle"
      m_u1r_bus1 = "m:u1r:bus1", "p.u., Positive-Sequence-Voltage, Real Part"
      m_u1r_bus1n = "m:u1r:bus1n", "p.u., Positive-Sequence-Voltage, Real Part"
      m_u1i_bus1 = "m:u1i:bus1", "p.u., Positive-Sequence-Voltage, Imaginary Part"
      m_u1i_bus1n = "m:u1i:bus1n", "p.u., Positive-Sequence-Voltage, Imaginary Part"
      m_u1_bus1 = "m:u1:bus1", "p.u., Positive-Sequence-Voltage, Magnitude"
      m_u1_bus1n = "m:u1:bus1n", "p.u., Positive-Sequence-Voltage, Magnitude"
      m_U1_bus1 = "m:U1:bus1", "kV, Line-Ground Positive-Sequence-Voltage, Magnitude"
      m_U1_bus1n = "m:U1:bus1n", "kV, Line-Ground Positive-Sequence-Voltage, Magnitude"
      m_U1l_bus1 = "m:U1l:bus1", "kV, Line-Line Positive-Sequence-Voltage, Magnitude"
      m_U1l_bus1n = "m:U1l:bus1n", "kV, Line-Line Positive-Sequence-Voltage, Magnitude"
      m_phiu1_bus1 = "m:phiu1:bus1", "deg, Positive-Sequence-Voltage, Angle"
      m_phiu1_bus1n = "m:phiu1:bus1n", "deg, Positive-Sequence-Voltage, Angle"
      m_u2_bus1 = "m:u2:bus1", "p.u., Negative-Sequence-Voltage, Magnitude"
      m_u2_bus1n = "m:u2:bus1n", "p.u., Negative-Sequence-Voltage, Magnitude"
      m_phiu2_bus1 = "m:phiu2:bus1", "deg, Negative-Sequence-Voltage, Angle"
      m_phiu2_bus1n = "m:phiu2:bus1n", "deg, Negative-Sequence-Voltage, Angle"
      m_inet_bus1_A = "m:inet:bus1:A", "p.u., Phase Current, Magnitude"
      m_inet_bus1_B = "m:inet:bus1:B", "p.u., Phase Current, Magnitude"
      m_inet_bus1_C = "m:inet:bus1:C", "p.u., Phase Current, Magnitude"
      m_inet_bus1_N = "m:inet:bus1:N", "p.u., Phase Current, Magnitude"
      m_inet_bus1n = "m:inet:bus1n", "p.u., Phase Current, Magnitude"
      m_I_bus1_A = "m:I:bus1:A", "kA, Phase Current, Magnitude"
      m_I_bus1_B = "m:I:bus1:B", "kA, Phase Current, Magnitude"
      m_I_bus1_C = "m:I:bus1:C", "kA, Phase Current, Magnitude"
      m_I_bus1_N = "m:I:bus1:N", "kA, Phase Current, Magnitude"
      m_I_bus1n = "m:I:bus1n", "kA, Phase Current, Magnitude"
      m_phii_bus1_A = "m:phii:bus1:A", "deg, Phase Current, Angle"
      m_phii_bus1_B = "m:phii:bus1:B", "deg, Phase Current, Angle"
      m_phii_bus1_C = "m:phii:bus1:C", "deg, Phase Current, Angle"
      m_phii_bus1_N = "m:phii:bus1:N", "deg, Phase Current, Angle"
      m_phii_bus1n = "m:phii:bus1n", "deg, Phase Current, Angle"
      m_i0r_bus1 = "m:i0r:bus1", "p.u., Zero-Sequence Current, Real Part"
      m_i0r_bus1n = "m:i0r:bus1n", "p.u., Zero-Sequence Current, Real Part"
      m_i0i_bus1 = "m:i0i:bus1", "p.u., Zero-Sequence Current, Imaginary Part"
      m_i0i_bus1n = "m:i0i:bus1n", "p.u., Zero-Sequence Current, Imaginary Part"
      m_i0_bus1 = "m:i0:bus1", "p.u., Zero-Sequence Current, Magnitude"
      m_i0_bus1n = "m:i0:bus1n", "p.u., Zero-Sequence Current, Magnitude"
      m_I0_bus1 = "m:I0:bus1", "kA, Zero-Sequence Current, Magnitude"
      m_I0_bus1n = "m:I0:bus1n", "kA, Zero-Sequence Current, Magnitude"
      m_I0x3_bus1 = "m:I0x3:bus1", "kA, 3*I0"
      m_I0x3_bus1n = "m:I0x3:bus1n", "kA, 3*I0"
      m_phii0_bus1 = "m:phii0:bus1", "deg, Zero-Sequence Current, Angle"
      m_phii0_bus1n = "m:phii0:bus1n", "deg, Zero-Sequence Current, Angle"
      m_i1r_bus1 = "m:i1r:bus1", "p.u., Positive-Sequence Current, Real Part"
      m_i1r_bus1n = "m:i1r:bus1n", "p.u., Positive-Sequence Current, Real Part"
      m_i1i_bus1 = "m:i1i:bus1", "p.u., Positive-Sequence Current, Imaginary Part"
      m_i1i_bus1n = "m:i1i:bus1n", "p.u., Positive-Sequence Current, Imaginary Part"
      m_i1_bus1 = "m:i1:bus1", "p.u., Positive-Sequence Current, Magnitude"
      m_i1_bus1n = "m:i1:bus1n", "p.u., Positive-Sequence Current, Magnitude"
      m_I1_bus1 = "m:I1:bus1", "kA, Positive-Sequence Current, Magnitude"
      m_I1_bus1n = "m:I1:bus1n", "kA, Positive-Sequence Current, Magnitude"
      m_phii1_bus1 = "m:phii1:bus1", "deg, Positive-Sequence Current, Angle"
      m_phii1_bus1n = "m:phii1:bus1n", "deg, Positive-Sequence Current, Angle"
      m_i2r_bus1 = "m:i2r:bus1", "p.u., Negative-Sequence Current, Real Part"
      m_i2r_bus1n = "m:i2r:bus1n", "p.u., Negative-Sequence Current, Real Part"
      m_i2i_bus1 = "m:i2i:bus1", "p.u., Negative-Sequence Current, Imaginary Part"
      m_i2i_bus1n = "m:i2i:bus1n", "p.u., Negative-Sequence Current, Imaginary Part"
      m_i2_bus1 = "m:i2:bus1", "p.u., Negative-Sequence Current, Magnitude"
      m_i2_bus1n = "m:i2:bus1n", "p.u., Negative-Sequence Current, Magnitude"
      m_I2_bus1 = "m:I2:bus1", "kA, Negative-Sequence Current, Magnitude"
      m_I2_bus1n = "m:I2:bus1n", "kA, Negative-Sequence Current, Magnitude"
      m_phii2_bus1 = "m:phii2:bus1", "deg, Negative-Sequence Current, Angle"
      m_phii2_bus1n = "m:phii2:bus1n", "deg, Negative-Sequence Current, Angle"
      m_P_bus1_A = "m:P:bus1:A", "MW, Active Power"
      m_P_bus1_B = "m:P:bus1:B", "MW, Active Power"
      m_P_bus1_C = "m:P:bus1:C", "MW, Active Power"
      m_P_bus1_N = "m:P:bus1:N", "MW, Active Power"
      m_P_bus1n = "m:P:bus1n", "MW, Active Power"
      m_Q_bus1_A = "m:Q:bus1:A", "Mvar, Reactive Power"
      m_Q_bus1_B = "m:Q:bus1:B", "Mvar, Reactive Power"
      m_Q_bus1_C = "m:Q:bus1:C", "Mvar, Reactive Power"
      m_Q_bus1_N = "m:Q:bus1:N", "Mvar, Reactive Power"
      m_Q_bus1n = "m:Q:bus1n", "Mvar, Reactive Power"
      m_S_bus1_A = "m:S:bus1:A", "MVA, Apparent Power"
      m_S_bus1_B = "m:S:bus1:B", "MVA, Apparent Power"
      m_S_bus1_C = "m:S:bus1:C", "MVA, Apparent Power"
      m_S_bus1_N = "m:S:bus1:N", "MVA, Apparent Power"
      m_S_bus1n = "m:S:bus1n", "MVA, Apparent Power"
      m_cosphi_bus1_A = "m:cosphi:bus1:A", "Power Factor"
      m_cosphi_bus1_B = "m:cosphi:bus1:B", "Power Factor"
      m_cosphi_bus1_C = "m:cosphi:bus1:C", "Power Factor"
      m_cosphi_bus1_N = "m:cosphi:bus1:N", "Power Factor"
      m_cosphi_bus1n = "m:cosphi:bus1n", "Power Factor"
      m_tanphi_bus1_A = "m:tanphi:bus1:A", "tan(phi)"
      m_tanphi_bus1_B = "m:tanphi:bus1:B", "tan(phi)"
      m_tanphi_bus1_C = "m:tanphi:bus1:C", "tan(phi)"
      m_tanphi_bus1_N = "m:tanphi:bus1:N", "tan(phi)"
      m_tanphi_bus1n = "m:tanphi:bus1n", "tan(phi)"
      m_Psum_bus1 = "m:Psum:bus1", "MW, Total Active Power"
      m_Psum_bus1n = "m:Psum:bus1n", "MW, Total Active Power"
      m_Qsum_bus1 = "m:Qsum:bus1", "Mvar, Total Reactive Power"
      m_Qsum_bus1n = "m:Qsum:bus1n", "Mvar, Total Reactive Power"
      m_Ssum_bus1 = "m:Ssum:bus1", "MVA, Total Apparent Power"
      m_Ssum_bus1n = "m:Ssum:bus1n", "MVA, Total Apparent Power"
      m_cosphisum_bus1 = "m:cosphisum:bus1", "Total Power Factor"
      m_cosphisum_bus1n = "m:cosphisum:bus1n", "Total Power Factor"
      m_tanphisum_bus1 = "m:tanphisum:bus1", "Total tan(phi)"
      m_tanphisum_bus1n = "m:tanphisum:bus1n", "Total tan(phi)"
      m_Pln_bus1_A = "m:Pln:bus1:A", "MW, Active Power (Line-Neutral)"
      m_Pln_bus1_B = "m:Pln:bus1:B", "MW, Active Power (Line-Neutral)"
      m_Pln_bus1_C = "m:Pln:bus1:C", "MW, Active Power (Line-Neutral)"
      m_Pln_bus1_N = "m:Pln:bus1:N", "MW, Active Power (Line-Neutral)"
      m_Pln_bus1n = "m:Pln:bus1n", "MW, Active Power (Line-Neutral)"
      m_Qln_bus1_A = "m:Qln:bus1:A", "Mvar, Reactive Power (Line-Neutral)"
      m_Qln_bus1_B = "m:Qln:bus1:B", "Mvar, Reactive Power (Line-Neutral)"
      m_Qln_bus1_C = "m:Qln:bus1:C", "Mvar, Reactive Power (Line-Neutral)"
      m_Qln_bus1_N = "m:Qln:bus1:N", "Mvar, Reactive Power (Line-Neutral)"
      m_Qln_bus1n = "m:Qln:bus1n", "Mvar, Reactive Power (Line-Neutral)"
      m_P1_bus1 = "m:P1:bus1", "MW, Positive Sequence Active Power"
      m_P1_bus1n = "m:P1:bus1n", "MW, Positive Sequence Active Power"
      m_Q1_bus1 = "m:Q1:bus1", "Mvar, Positive Sequence Reactive Power"
      m_Q1_bus1n = "m:Q1:bus1n", "Mvar, Positive Sequence Reactive Power"
      m_P2_bus1 = "m:P2:bus1", "MW, Negative Sequence Active Power"
      m_P2_bus1n = "m:P2:bus1n", "MW, Negative Sequence Active Power"
      m_Q2_bus1 = "m:Q2:bus1", "Mvar, Negative Sequence Reactive Power"
      m_Q2_bus1n = "m:Q2:bus1n", "Mvar, Negative Sequence Reactive Power"
      m_P0_bus1 = "m:P0:bus1", "MW, Zero Sequence Active Power"
      m_P0_bus1n = "m:P0:bus1n", "MW, Zero Sequence Active Power"
      m_Q0_bus1 = "m:Q0:bus1", "Mvar, Zero Sequence Reactive Power"
      m_Q0_bus1n = "m:Q0:bus1n", "Mvar, Zero Sequence Reactive Power"
      m_i1P_bus1 = "m:i1P:bus1", "p.u., Positive-Sequence Active Current"
      m_i1P_bus1n = "m:i1P:bus1n", "p.u., Positive-Sequence Active Current"
      m_i1Q_bus1 = "m:i1Q:bus1", "p.u., Positive-Sequence Reactive Current"
      m_i1Q_bus1n = "m:i1Q:bus1n", "p.u., Positive-Sequence Reactive Current"
      m_I1P_bus1 = "m:I1P:bus1", "kA, Positive-Sequence Active Current"
      m_I1P_bus1n = "m:I1P:bus1n", "kA, Positive-Sequence Active Current"
      m_I1Q_bus1 = "m:I1Q:bus1", "kA, Positive-Sequence Reactive Current"
      m_I1Q_bus1n = "m:I1Q:bus1n", "kA, Positive-Sequence Reactive Current"
      m_i2P_bus1 = "m:i2P:bus1", "p.u., Negative-Sequence Active Current"
      m_i2P_bus1n = "m:i2P:bus1n", "p.u., Negative-Sequence Active Current"
      m_i2Q_bus1 = "m:i2Q:bus1", "p.u., Negative-Sequence Reactive Current"
      m_i2Q_bus1n = "m:i2Q:bus1n", "p.u., Negative-Sequence Reactive Current"
      m_I2P_bus1 = "m:I2P:bus1", "kA, Negative-Sequence Active Current"
      m_I2P_bus1n = "m:I2P:bus1n", "kA, Negative-Sequence Active Current"
      m_I2Q_bus1 = "m:I2Q:bus1", "kA, Negative-Sequence Reactive Current"
      m_I2Q_bus1n = "m:I2Q:bus1n", "kA, Negative-Sequence Reactive Current"
      m_ur_bus1_A = "m:ur:bus1:A", "p.u., Phase Voltage, Real Part"
      m_ur_bus1_B = "m:ur:bus1:B", "p.u., Phase Voltage, Real Part"
      m_ur_bus1_C = "m:ur:bus1:C", "p.u., Phase Voltage, Real Part"
      m_ur_bus1_N = "m:ur:bus1:N", "p.u., Phase Voltage, Real Part"
      m_ur_bus1n = "m:ur:bus1n", "p.u., Phase Voltage, Real Part"
      m_ui_bus1_A = "m:ui:bus1:A", "p.u., Phase Voltage, Imaginary Part"
      m_ui_bus1_B = "m:ui:bus1:B", "p.u., Phase Voltage, Imaginary Part"
      m_ui_bus1_C = "m:ui:bus1:C", "p.u., Phase Voltage, Imaginary Part"
      m_ui_bus1_N = "m:ui:bus1:N", "p.u., Phase Voltage, Imaginary Part"
      m_ui_bus1n = "m:ui:bus1n", "p.u., Phase Voltage, Imaginary Part"
      m_u_bus1_A = "m:u:bus1:A", "p.u., Phase Voltage, Magnitude"
      m_u_bus1_B = "m:u:bus1:B", "p.u., Phase Voltage, Magnitude"
      m_u_bus1_C = "m:u:bus1:C", "p.u., Phase Voltage, Magnitude"
      m_u_bus1_N = "m:u:bus1:N", "p.u., Phase Voltage, Magnitude"
      m_u_bus1n = "m:u:bus1n", "p.u., Phase Voltage, Magnitude"
      m_ir_bus1_A = "m:ir:bus1:A", "p.u., Phase Current, Real Part"
      m_ir_bus1_B = "m:ir:bus1:B", "p.u., Phase Current, Real Part"
      m_ir_bus1_C = "m:ir:bus1:C", "p.u., Phase Current, Real Part"
      m_ir_bus1_N = "m:ir:bus1:N", "p.u., Phase Current, Real Part"
      m_ir_bus1n = "m:ir:bus1n", "p.u., Phase Current, Real Part"
      m_ii_bus1_A = "m:ii:bus1:A", "p.u., Phase Current, Imaginary Part"
      m_ii_bus1_B = "m:ii:bus1:B", "p.u., Phase Current, Imaginary Part"
      m_ii_bus1_C = "m:ii:bus1:C", "p.u., Phase Current, Imaginary Part"
      m_ii_bus1_N = "m:ii:bus1:N", "p.u., Phase Current, Imaginary Part"
      m_ii_bus1n = "m:ii:bus1n", "p.u., Phase Current, Imaginary Part"
      m_i_bus1_A = "m:i:bus1:A", "p.u., Phase Current, Magnitude"
      m_i_bus1_B = "m:i:bus1:B", "p.u., Phase Current, Magnitude"
      m_i_bus1_C = "m:i:bus1:C", "p.u., Phase Current, Magnitude"
      m_i_bus1_N = "m:i:bus1:N", "p.u., Phase Current, Magnitude"
      m_i_bus1n = "m:i:bus1n", "p.u., Phase Current, Magnitude"
      m_phiui_bus1_A = "m:phiui:bus1:A", "deg, Angle between Voltage and Current"
      m_phiui_bus1_B = "m:phiui:bus1:B", "deg, Angle between Voltage and Current"
      m_phiui_bus1_C = "m:phiui:bus1:C", "deg, Angle between Voltage and Current"
      m_phiui_bus1_N = "m:phiui:bus1:N", "deg, Angle between Voltage and Current"
      m_phiui_bus1n = "m:phiui:bus1n", "deg, Angle between Voltage and Current"
      m_phiu0i0_bus1 = "m:phiu0i0:bus1", "deg, Angle between Voltage and Current in zero-sequence system"
      m_phiu0i0_bus1n = "m:phiu0i0:bus1n", "deg, Angle between Voltage and Current in zero-sequence system"
      m_phiu1i1_bus1 = "m:phiu1i1:bus1", "deg, Angle between Voltage and Current in positive sequence system"
      m_phiu1i1_bus1n = "m:phiu1i1:bus1n", "deg, Angle between Voltage and Current in positive sequence system"
      m_phiu2i2_bus1 = "m:phiu2i2:bus1", "deg, Angle between Voltage and Current in negative sequence system"
      m_phiu2i2_bus1n = "m:phiu2i2:bus1n", "deg, Angle between Voltage and Current in negative sequence system"
      m_Tfct_bus1 = "m:Tfct:bus1", "s, Fault Clearing Time"
      m_Tfct_bus1n = "m:Tfct:bus1n", "s, Fault Clearing Time"
      m_TfctPh_bus1_A = "m:TfctPh:bus1:A", "s, Fault Clearing Time"
      m_TfctPh_bus1_B = "m:TfctPh:bus1:B", "s, Fault Clearing Time"
      m_TfctPh_bus1_C = "m:TfctPh:bus1:C", "s, Fault Clearing Time"
      m_TfctPh_bus1_N = "m:TfctPh:bus1:N", "s, Fault Clearing Time"
      m_TfctPh_bus1n = "m:TfctPh:bus1n", "s, Fault Clearing Time"
      m_Brkload_bus1 = "m:Brkload:bus1", "%, Max. Breaker Loading"
      m_Brkload_bus1n = "m:Brkload:bus1n", "%, Max. Breaker Loading"
      m_BrkloadPh_bus1_A = "m:BrkloadPh:bus1:A", "%, Breaker Loading"
      m_BrkloadPh_bus1_B = "m:BrkloadPh:bus1:B", "%, Breaker Loading"
      m_BrkloadPh_bus1_C = "m:BrkloadPh:bus1:C", "%, Breaker Loading"
      m_BrkloadPh_bus1_N = "m:BrkloadPh:bus1:N", "%, Breaker Loading"
      m_BrkloadPh_bus1n = "m:BrkloadPh:bus1n", "%, Breaker Loading"
      n_ur_bus1_A = "n:ur:bus1:A", "p.u., Line-Ground Voltage, Real Part"
      n_ur_bus1_B = "n:ur:bus1:B", "p.u., Line-Ground Voltage, Real Part"
      n_ur_bus1_C = "n:ur:bus1:C", "p.u., Line-Ground Voltage, Real Part"
      n_ur_bus1_N = "n:ur:bus1:N", "p.u., Line-Ground Voltage, Real Part"
      n_ur_bus1n = "n:ur:bus1n", "p.u., Line-Ground Voltage, Real Part"
      n_ui_bus1_A = "n:ui:bus1:A", "p.u., Line-Ground Voltage, Imaginary Part"
      n_ui_bus1_B = "n:ui:bus1:B", "p.u., Line-Ground Voltage, Imaginary Part"
      n_ui_bus1_C = "n:ui:bus1:C", "p.u., Line-Ground Voltage, Imaginary Part"
      n_ui_bus1_N = "n:ui:bus1:N", "p.u., Line-Ground Voltage, Imaginary Part"
      n_ui_bus1n = "n:ui:bus1n", "p.u., Line-Ground Voltage, Imaginary Part"
      n_u_bus1_A = "n:u:bus1:A", "p.u., Line-Ground Voltage, Magnitude"
      n_u_bus1_B = "n:u:bus1:B", "p.u., Line-Ground Voltage, Magnitude"
      n_u_bus1_C = "n:u:bus1:C", "p.u., Line-Ground Voltage, Magnitude"
      n_u_bus1_N = "n:u:bus1:N", "p.u., Line-Ground Voltage, Magnitude"
      n_u_bus1n = "n:u:bus1n", "p.u., Line-Ground Voltage, Magnitude"
      n_upc_bus1_A = "n:upc:bus1:A", "%, Line-Ground Voltage, Magnitude"
      n_upc_bus1_B = "n:upc:bus1:B", "%, Line-Ground Voltage, Magnitude"
      n_upc_bus1_C = "n:upc:bus1:C", "%, Line-Ground Voltage, Magnitude"
      n_upc_bus1_N = "n:upc:bus1:N", "%, Line-Ground Voltage, Magnitude"
      n_upc_bus1n = "n:upc:bus1n", "%, Line-Ground Voltage, Magnitude"
      n_U_bus1_A = "n:U:bus1:A", "kV, Line-Ground Voltage, Magnitude"
      n_U_bus1_B = "n:U:bus1:B", "kV, Line-Ground Voltage, Magnitude"
      n_U_bus1_C = "n:U:bus1:C", "kV, Line-Ground Voltage, Magnitude"
      n_U_bus1_N = "n:U:bus1:N", "kV, Line-Ground Voltage, Magnitude"
      n_U_bus1n = "n:U:bus1n", "kV, Line-Ground Voltage, Magnitude"
      n_phiu_bus1_A = "n:phiu:bus1:A", "deg, Line-Ground Voltage, Angle"
      n_phiu_bus1_B = "n:phiu:bus1:B", "deg, Line-Ground Voltage, Angle"
      n_phiu_bus1_C = "n:phiu:bus1:C", "deg, Line-Ground Voltage, Angle"
      n_phiu_bus1_N = "n:phiu:bus1:N", "deg, Line-Ground Voltage, Angle"
      n_phiu_bus1n = "n:phiu:bus1n", "deg, Line-Ground Voltage, Angle"
      n_phiurel_bus1 = "n:phiurel:bus1", "deg, Voltage, Relative Angle"
      n_phiurel_bus1n = "n:phiurel:bus1n", "deg, Voltage, Relative Angle"
      n_ul_bus1_A = "n:ul:bus1:A", "p.u., Line-Line Voltage, Magnitude"
      n_ul_bus1_B = "n:ul:bus1:B", "p.u., Line-Line Voltage, Magnitude"
      n_ul_bus1_C = "n:ul:bus1:C", "p.u., Line-Line Voltage, Magnitude"
      n_ul_bus1_N = "n:ul:bus1:N", "p.u., Line-Line Voltage, Magnitude"
      n_ul_bus1n = "n:ul:bus1n", "p.u., Line-Line Voltage, Magnitude"
      n_ulpc_bus1_A = "n:ulpc:bus1:A", "%, Line-Line Voltage, Magnitude"
      n_ulpc_bus1_B = "n:ulpc:bus1:B", "%, Line-Line Voltage, Magnitude"
      n_ulpc_bus1_C = "n:ulpc:bus1:C", "%, Line-Line Voltage, Magnitude"
      n_ulpc_bus1_N = "n:ulpc:bus1:N", "%, Line-Line Voltage, Magnitude"
      n_ulpc_bus1n = "n:ulpc:bus1n", "%, Line-Line Voltage, Magnitude"
      n_Ul_bus1_A = "n:Ul:bus1:A", "kV, Line-Line Voltage, Magnitude"
      n_Ul_bus1_B = "n:Ul:bus1:B", "kV, Line-Line Voltage, Magnitude"
      n_Ul_bus1_C = "n:Ul:bus1:C", "kV, Line-Line Voltage, Magnitude"
      n_Ul_bus1_N = "n:Ul:bus1:N", "kV, Line-Line Voltage, Magnitude"
      n_Ul_bus1n = "n:Ul:bus1n", "kV, Line-Line Voltage, Magnitude"
      n_phiul_bus1_A = "n:phiul:bus1:A", "deg, Line-Line Voltage, Angle"
      n_phiul_bus1_B = "n:phiul:bus1:B", "deg, Line-Line Voltage, Angle"
      n_phiul_bus1_C = "n:phiul:bus1:C", "deg, Line-Line Voltage, Angle"
      n_phiul_bus1_N = "n:phiul:bus1:N", "deg, Line-Line Voltage, Angle"
      n_phiul_bus1n = "n:phiul:bus1n", "deg, Line-Line Voltage, Angle"
      n_u0_bus1 = "n:u0:bus1", "p.u., Zero-Sequence Voltage, Magnitude"
      n_u0_bus1n = "n:u0:bus1n", "p.u., Zero-Sequence Voltage, Magnitude"
      n_U0_bus1 = "n:U0:bus1", "kV, Zero-Sequence Voltage, Magnitude"
      n_U0_bus1n = "n:U0:bus1n", "kV, Zero-Sequence Voltage, Magnitude"
      n_U0x3_bus1 = "n:U0x3:bus1", "kV, 3*U0"
      n_U0x3_bus1n = "n:U0x3:bus1n", "kV, 3*U0"
      n_phiu0_bus1 = "n:phiu0:bus1", "deg, Zero-Sequence Voltage, Angle"
      n_phiu0_bus1n = "n:phiu0:bus1n", "deg, Zero-Sequence Voltage, Angle"
      n_u1_bus1 = "n:u1:bus1", "p.u., Positive-Sequence Voltage, Magnitude"
      n_u1_bus1n = "n:u1:bus1n", "p.u., Positive-Sequence Voltage, Magnitude"
      n_u1pc_bus1 = "n:u1pc:bus1", "%, Positive-Sequence Voltage, Magnitude"
      n_u1pc_bus1n = "n:u1pc:bus1n", "%, Positive-Sequence Voltage, Magnitude"
      n_u1r_bus1 = "n:u1r:bus1", "p.u., Positive-Sequence Voltage, Real Part"
      n_u1r_bus1n = "n:u1r:bus1n", "p.u., Positive-Sequence Voltage, Real Part"
      n_u1i_bus1 = "n:u1i:bus1", "p.u., Positive-Sequence Voltage, Imaginary Part"
      n_u1i_bus1n = "n:u1i:bus1n", "p.u., Positive-Sequence Voltage, Imaginary Part"
      n_U1_bus1 = "n:U1:bus1", "kV, Line-Ground Positive-Sequence Voltage, Magnitude"
      n_U1_bus1n = "n:U1:bus1n", "kV, Line-Ground Positive-Sequence Voltage, Magnitude"
      n_phiu1_bus1 = "n:phiu1:bus1", "deg, Positive-Sequence Voltage, Angle"
      n_phiu1_bus1n = "n:phiu1:bus1n", "deg, Positive-Sequence Voltage, Angle"
      n_u2_bus1 = "n:u2:bus1", "p.u., Negative-Sequence Voltage, Magnitude"
      n_u2_bus1n = "n:u2:bus1n", "p.u., Negative-Sequence Voltage, Magnitude"
      n_U2_bus1 = "n:U2:bus1", "kV, Line-Ground Negative-Sequence Voltage, Magnitude"
      n_U2_bus1n = "n:U2:bus1n", "kV, Line-Ground Negative-Sequence Voltage, Magnitude"
      n_phiu2_bus1 = "n:phiu2:bus1", "deg, Negative-Sequence Voltage, Angle"
      n_phiu2_bus1n = "n:phiu2:bus1n", "deg, Negative-Sequence Voltage, Angle"
      n_U1l_bus1 = "n:U1l:bus1", "kV, Line-Line Positive-Sequence Voltage, Magnitude"
      n_U1l_bus1n = "n:U1l:bus1n", "kV, Line-Line Positive-Sequence Voltage, Magnitude"
      n_U2l_bus1 = "n:U2l:bus1", "kV, Line-Line Negative-Sequence Voltage, Magnitude"
      n_U2l_bus1n = "n:U2l:bus1n", "kV, Line-Line Negative-Sequence Voltage, Magnitude"
      n_uln_bus1_A = "n:uln:bus1:A", "p.u., Line-Neutral Voltage, Magnitude"
      n_uln_bus1_B = "n:uln:bus1:B", "p.u., Line-Neutral Voltage, Magnitude"
      n_uln_bus1_C = "n:uln:bus1:C", "p.u., Line-Neutral Voltage, Magnitude"
      n_uln_bus1_N = "n:uln:bus1:N", "p.u., Line-Neutral Voltage, Magnitude"
      n_uln_bus1n = "n:uln:bus1n", "p.u., Line-Neutral Voltage, Magnitude"
      n_Uln_bus1_A = "n:Uln:bus1:A", "kV, Line-Neutral Voltage, Magnitude"
      n_Uln_bus1_B = "n:Uln:bus1:B", "kV, Line-Neutral Voltage, Magnitude"
      n_Uln_bus1_C = "n:Uln:bus1:C", "kV, Line-Neutral Voltage, Magnitude"
      n_Uln_bus1_N = "n:Uln:bus1:N", "kV, Line-Neutral Voltage, Magnitude"
      n_Uln_bus1n = "n:Uln:bus1n", "kV, Line-Neutral Voltage, Magnitude"
      n_phiuln_bus1_A = "n:phiuln:bus1:A", "deg, Line-Neutral Voltage, Angle"
      n_phiuln_bus1_B = "n:phiuln:bus1:B", "deg, Line-Neutral Voltage, Angle"
      n_phiuln_bus1_C = "n:phiuln:bus1:C", "deg, Line-Neutral Voltage, Angle"
      n_phiuln_bus1_N = "n:phiuln:bus1:N", "deg, Line-Neutral Voltage, Angle"
      n_phiuln_bus1n = "n:phiuln:bus1n", "deg, Line-Neutral Voltage, Angle"
      n_un_bus1 = "n:un:bus1", "p.u., Neutral-Ground Voltage, Magnitude"
      n_un_bus1n = "n:un:bus1n", "p.u., Neutral-Ground Voltage, Magnitude"
      n_Un_bus1 = "n:Un:bus1", "kV, Neutral-Ground Voltage, Magnitude"
      n_Un_bus1n = "n:Un:bus1n", "kV, Neutral-Ground Voltage, Magnitude"
      n_phiun_bus1 = "n:phiun:bus1", "deg, Neutral-Ground Voltage, Angle"
      n_phiun_bus1n = "n:phiun:bus1n", "deg, Neutral-Ground Voltage, Angle"
      n_upht_bus1_A = "n:upht:bus1:A", "p.u., Phase Technology dependent Voltage, Magnitude"
      n_upht_bus1_B = "n:upht:bus1:B", "p.u., Phase Technology dependent Voltage, Magnitude"
      n_upht_bus1_C = "n:upht:bus1:C", "p.u., Phase Technology dependent Voltage, Magnitude"
      n_upht_bus1_N = "n:upht:bus1:N", "p.u., Phase Technology dependent Voltage, Magnitude"
      n_upht_bus1n = "n:upht:bus1n", "p.u., Phase Technology dependent Voltage, Magnitude"
      n_Upht_bus1_A = "n:Upht:bus1:A", "kV, Phase Technology dependent Voltage, Magnitude"
      n_Upht_bus1_B = "n:Upht:bus1:B", "kV, Phase Technology dependent Voltage, Magnitude"
      n_Upht_bus1_C = "n:Upht:bus1:C", "kV, Phase Technology dependent Voltage, Magnitude"
      n_Upht_bus1_N = "n:Upht:bus1:N", "kV, Phase Technology dependent Voltage, Magnitude"
      n_Upht_bus1n = "n:Upht:bus1n", "kV, Phase Technology dependent Voltage, Magnitude"
      n_um_bus1 = "n:um:bus1", "p.u., Average-Voltage, Magnitude"
      n_um_bus1n = "n:um:bus1n", "p.u., Average-Voltage, Magnitude"
      n_Um_bus1 = "n:Um:bus1", "kV, Average-Voltage, Magnitude"
      n_Um_bus1n = "n:Um:bus1n", "kV, Average-Voltage, Magnitude"
      n_ishcr_bus1_A = "n:ishcr:bus1:A", "p.u., Short-Circuit Current, real"
      n_ishcr_bus1_B = "n:ishcr:bus1:B", "p.u., Short-Circuit Current, real"
      n_ishcr_bus1_C = "n:ishcr:bus1:C", "p.u., Short-Circuit Current, real"
      n_ishcr_bus1_N = "n:ishcr:bus1:N", "p.u., Short-Circuit Current, real"
      n_ishcr_bus1n = "n:ishcr:bus1n", "p.u., Short-Circuit Current, real"
      n_ishci_bus1_A = "n:ishci:bus1:A", "p.u., Short-Circuit Current, imag"
      n_ishci_bus1_B = "n:ishci:bus1:B", "p.u., Short-Circuit Current, imag"
      n_ishci_bus1_C = "n:ishci:bus1:C", "p.u., Short-Circuit Current, imag"
      n_ishci_bus1_N = "n:ishci:bus1:N", "p.u., Short-Circuit Current, imag"
      n_ishci_bus1n = "n:ishci:bus1n", "p.u., Short-Circuit Current, imag"
      n_ishc_bus1_A = "n:ishc:bus1:A", "p.u., Short-Circuit Current, magnitude"
      n_ishc_bus1_B = "n:ishc:bus1:B", "p.u., Short-Circuit Current, magnitude"
      n_ishc_bus1_C = "n:ishc:bus1:C", "p.u., Short-Circuit Current, magnitude"
      n_ishc_bus1_N = "n:ishc:bus1:N", "p.u., Short-Circuit Current, magnitude"
      n_ishc_bus1n = "n:ishc:bus1n", "p.u., Short-Circuit Current, magnitude"
      n_Ishc_bus1_A = "n:Ishc:bus1:A", "kA, Short-Circuit Current, magnitude"
      n_Ishc_bus1_B = "n:Ishc:bus1:B", "kA, Short-Circuit Current, magnitude"
      n_Ishc_bus1_C = "n:Ishc:bus1:C", "kA, Short-Circuit Current, magnitude"
      n_Ishc_bus1_N = "n:Ishc:bus1:N", "kA, Short-Circuit Current, magnitude"
      n_Ishc_bus1n = "n:Ishc:bus1n", "kA, Short-Circuit Current, magnitude"
      n_inshc_bus1 = "n:inshc:bus1", "p.u., Neutral Short-Circuit Current, magnitude"
      n_inshc_bus1n = "n:inshc:bus1n", "p.u., Neutral Short-Circuit Current, magnitude"
      n_Inshc_bus1 = "n:Inshc:bus1", "kA, Neutral Short-Circuit Current, magnitude"
      n_Inshc_bus1n = "n:Inshc:bus1n", "kA, Neutral Short-Circuit Current, magnitude"
      n_fe_bus1 = "n:fe:bus1", "p.u., Electrical Frequency"
      n_fe_bus1n = "n:fe:bus1n", "p.u., Electrical Frequency"
      n_frdev_bus1 = "n:frdev:bus1", "Hz, average frequency"
      n_frdev_bus1n = "n:frdev:bus1n", "Hz, average frequency"
      n_dfehz_bus1 = "n:dfehz:bus1", "Hz, Deviation of the El. Frequency"
      n_dfehz_bus1n = "n:dfehz:bus1n", "Hz, Deviation of the El. Frequency"
      n_dfedt_bus1 = "n:dfedt:bus1", "p.u./s, Derivative of El. Frequency"
      n_dfedt_bus1n = "n:dfedt:bus1n", "p.u./s, Derivative of El. Frequency"
      n_fehz_bus1 = "n:fehz:bus1", "Hz, Electrical Frequency"
      n_fehz_bus1n = "n:fehz:bus1n", "Hz, Electrical Frequency"
      n_frnom_bus1 = "n:frnom:bus1", "Hz, Nominal Frequency"
      n_frnom_bus1n = "n:frnom:bus1n", "Hz, Nominal Frequency"
      c_z_r = "c:z:r", "p.u., Shunt-Impedance, Real Part"
      c_z_i = "c:z:i", "p.u., Shunt-Impedance, Imaginary Part"
      c_z_phi = "c:z:phi", "deg, Shunt-Impedance, Angle"
      c_z_mag = "c:z:mag", "p.u., Shunt-Impedance, Magnitude"
      c_z_phirad = "c:z:phirad", "rad, Shunt-Impedance, Angle"
      c_cn = "c:cn", "Connection Mode"
      c_frnom = "c:frnom", "Hz, Nominal Frequency"
      c_hpi = "c:hpi", "rad/s, Nominal Angular Frequency"
      c_i_earth = "c:i_earth", "Internal"
      c_y0_r = "c:y0:r", "p.u., Zero Sequence Admittance, Real Part"
      c_y0_i = "c:y0:i", "p.u., Zero Sequence Admittance, Imaginary Part"
      c_y0_phi = "c:y0:phi", "deg, Zero Sequence Admittance, Angle"
      c_y0_mag = "c:y0:mag", "p.u., Zero Sequence Admittance, Magnitude"
      c_y0_phirad = "c:y0:phirad", "rad, Zero Sequence Admittance, Angle"
      c_ze_r = "c:ze:r", "p.u., Grounding Impedance, Real Part"
      c_ze_i = "c:ze:i", "p.u., Grounding Impedance, Imaginary Part"
      c_ze_phi = "c:ze:phi", "deg, Grounding Impedance, Angle"
      c_ze_mag = "c:ze:mag", "p.u., Grounding Impedance, Magnitude"
      c_ze_phirad = "c:ze:phirad", "rad, Grounding Impedance, Angle"
      c_z0_r = "c:z0:r", "p.u., Zero Sequence Impedance, Real Part"
      c_z0_i = "c:z0:i", "p.u., Zero Sequence Impedance, Imaginary Part"
      c_z0_phi = "c:z0:phi", "deg, Zero Sequence Impedance, Angle"
      c_z0_mag = "c:z0:mag", "p.u., Zero Sequence Impedance, Magnitude"
      c_z0_phirad = "c:z0:phirad", "rad, Zero Sequence Impedance, Angle"
      c_b0 = "c:b0", "p.u., Susceptance to Ground"
      c_ncapa = "c:ncapa", "Number of Stages in Parallel"
      c_ncapx = "c:ncapx", "Max. No. of Steps"
      c_Ustar = "c:Ustar", "kV, Star-Point Voltage"
      c_Qmax = "c:Qmax", "Mvar, Max. Rated Reactive Power"
      e_iIntTapCtrl = "e:iIntTapCtrl", "Use Integrated Tap Controller"

    class ElmSind(Enum):
      _init_ = 'value __doc__'

      m_u0_bus1 = "m:u0:bus1", "p.u., Zero-Sequence-Voltage, Magnitude"
      m_u0_bus2 = "m:u0:bus2", "p.u., Zero-Sequence-Voltage, Magnitude"
      m_phiu0_bus1 = "m:phiu0:bus1", "deg, Zero-Sequence-Voltage, Angle"
      m_phiu0_bus2 = "m:phiu0:bus2", "deg, Zero-Sequence-Voltage, Angle"
      m_u1r_bus1 = "m:u1r:bus1", "p.u., Positive-Sequence-Voltage, Real Part"
      m_u1r_bus2 = "m:u1r:bus2", "p.u., Positive-Sequence-Voltage, Real Part"
      m_u1i_bus1 = "m:u1i:bus1", "p.u., Positive-Sequence-Voltage, Imaginary Part"
      m_u1i_bus2 = "m:u1i:bus2", "p.u., Positive-Sequence-Voltage, Imaginary Part"
      m_u1_bus1 = "m:u1:bus1", "p.u., Positive-Sequence-Voltage, Magnitude"
      m_u1_bus2 = "m:u1:bus2", "p.u., Positive-Sequence-Voltage, Magnitude"
      m_U1_bus1 = "m:U1:bus1", "kV, Line-Ground Positive-Sequence-Voltage, Magnitude"
      m_U1_bus2 = "m:U1:bus2", "kV, Line-Ground Positive-Sequence-Voltage, Magnitude"
      m_U1l_bus1 = "m:U1l:bus1", "kV, Line-Line Positive-Sequence-Voltage, Magnitude"
      m_U1l_bus2 = "m:U1l:bus2", "kV, Line-Line Positive-Sequence-Voltage, Magnitude"
      m_phiu1_bus1 = "m:phiu1:bus1", "deg, Positive-Sequence-Voltage, Angle"
      m_phiu1_bus2 = "m:phiu1:bus2", "deg, Positive-Sequence-Voltage, Angle"
      m_u2_bus1 = "m:u2:bus1", "p.u., Negative-Sequence-Voltage, Magnitude"
      m_u2_bus2 = "m:u2:bus2", "p.u., Negative-Sequence-Voltage, Magnitude"
      m_phiu2_bus1 = "m:phiu2:bus1", "deg, Negative-Sequence-Voltage, Angle"
      m_phiu2_bus2 = "m:phiu2:bus2", "deg, Negative-Sequence-Voltage, Angle"
      m_inet_bus1_A = "m:inet:bus1:A", "p.u., Phase Current, Magnitude"
      m_inet_bus1_B = "m:inet:bus1:B", "p.u., Phase Current, Magnitude"
      m_inet_bus1_C = "m:inet:bus1:C", "p.u., Phase Current, Magnitude"
      m_inet_bus2_A = "m:inet:bus2:A", "p.u., Phase Current, Magnitude"
      m_inet_bus2_B = "m:inet:bus2:B", "p.u., Phase Current, Magnitude"
      m_inet_bus2_C = "m:inet:bus2:C", "p.u., Phase Current, Magnitude"
      m_I_bus1_A = "m:I:bus1:A", "kA, Phase Current, Magnitude"
      m_I_bus1_B = "m:I:bus1:B", "kA, Phase Current, Magnitude"
      m_I_bus1_C = "m:I:bus1:C", "kA, Phase Current, Magnitude"
      m_I_bus2_A = "m:I:bus2:A", "kA, Phase Current, Magnitude"
      m_I_bus2_B = "m:I:bus2:B", "kA, Phase Current, Magnitude"
      m_I_bus2_C = "m:I:bus2:C", "kA, Phase Current, Magnitude"
      m_phii_bus1_A = "m:phii:bus1:A", "deg, Phase Current, Angle"
      m_phii_bus1_B = "m:phii:bus1:B", "deg, Phase Current, Angle"
      m_phii_bus1_C = "m:phii:bus1:C", "deg, Phase Current, Angle"
      m_phii_bus2_A = "m:phii:bus2:A", "deg, Phase Current, Angle"
      m_phii_bus2_B = "m:phii:bus2:B", "deg, Phase Current, Angle"
      m_phii_bus2_C = "m:phii:bus2:C", "deg, Phase Current, Angle"
      m_i0r_bus1 = "m:i0r:bus1", "p.u., Zero-Sequence Current, Real Part"
      m_i0r_bus2 = "m:i0r:bus2", "p.u., Zero-Sequence Current, Real Part"
      m_i0i_bus1 = "m:i0i:bus1", "p.u., Zero-Sequence Current, Imaginary Part"
      m_i0i_bus2 = "m:i0i:bus2", "p.u., Zero-Sequence Current, Imaginary Part"
      m_i0_bus1 = "m:i0:bus1", "p.u., Zero-Sequence Current, Magnitude"
      m_i0_bus2 = "m:i0:bus2", "p.u., Zero-Sequence Current, Magnitude"
      m_I0_bus1 = "m:I0:bus1", "kA, Zero-Sequence Current, Magnitude"
      m_I0_bus2 = "m:I0:bus2", "kA, Zero-Sequence Current, Magnitude"
      m_I0x3_bus1 = "m:I0x3:bus1", "kA, 3*I0"
      m_I0x3_bus2 = "m:I0x3:bus2", "kA, 3*I0"
      m_phii0_bus1 = "m:phii0:bus1", "deg, Zero-Sequence Current, Angle"
      m_phii0_bus2 = "m:phii0:bus2", "deg, Zero-Sequence Current, Angle"
      m_i1r_bus1 = "m:i1r:bus1", "p.u., Positive-Sequence Current, Real Part"
      m_i1r_bus2 = "m:i1r:bus2", "p.u., Positive-Sequence Current, Real Part"
      m_i1i_bus1 = "m:i1i:bus1", "p.u., Positive-Sequence Current, Imaginary Part"
      m_i1i_bus2 = "m:i1i:bus2", "p.u., Positive-Sequence Current, Imaginary Part"
      m_i1_bus1 = "m:i1:bus1", "p.u., Positive-Sequence Current, Magnitude"
      m_i1_bus2 = "m:i1:bus2", "p.u., Positive-Sequence Current, Magnitude"
      m_I1_bus1 = "m:I1:bus1", "kA, Positive-Sequence Current, Magnitude"
      m_I1_bus2 = "m:I1:bus2", "kA, Positive-Sequence Current, Magnitude"
      m_phii1_bus1 = "m:phii1:bus1", "deg, Positive-Sequence Current, Angle"
      m_phii1_bus2 = "m:phii1:bus2", "deg, Positive-Sequence Current, Angle"
      m_i2r_bus1 = "m:i2r:bus1", "p.u., Negative-Sequence Current, Real Part"
      m_i2r_bus2 = "m:i2r:bus2", "p.u., Negative-Sequence Current, Real Part"
      m_i2i_bus1 = "m:i2i:bus1", "p.u., Negative-Sequence Current, Imaginary Part"
      m_i2i_bus2 = "m:i2i:bus2", "p.u., Negative-Sequence Current, Imaginary Part"
      m_i2_bus1 = "m:i2:bus1", "p.u., Negative-Sequence Current, Magnitude"
      m_i2_bus2 = "m:i2:bus2", "p.u., Negative-Sequence Current, Magnitude"
      m_I2_bus1 = "m:I2:bus1", "kA, Negative-Sequence Current, Magnitude"
      m_I2_bus2 = "m:I2:bus2", "kA, Negative-Sequence Current, Magnitude"
      m_phii2_bus1 = "m:phii2:bus1", "deg, Negative-Sequence Current, Angle"
      m_phii2_bus2 = "m:phii2:bus2", "deg, Negative-Sequence Current, Angle"
      m_P_bus1_A = "m:P:bus1:A", "MW, Active Power"
      m_P_bus1_B = "m:P:bus1:B", "MW, Active Power"
      m_P_bus1_C = "m:P:bus1:C", "MW, Active Power"
      m_P_bus2_A = "m:P:bus2:A", "MW, Active Power"
      m_P_bus2_B = "m:P:bus2:B", "MW, Active Power"
      m_P_bus2_C = "m:P:bus2:C", "MW, Active Power"
      m_Q_bus1_A = "m:Q:bus1:A", "Mvar, Reactive Power"
      m_Q_bus1_B = "m:Q:bus1:B", "Mvar, Reactive Power"
      m_Q_bus1_C = "m:Q:bus1:C", "Mvar, Reactive Power"
      m_Q_bus2_A = "m:Q:bus2:A", "Mvar, Reactive Power"
      m_Q_bus2_B = "m:Q:bus2:B", "Mvar, Reactive Power"
      m_Q_bus2_C = "m:Q:bus2:C", "Mvar, Reactive Power"
      m_S_bus1_A = "m:S:bus1:A", "MVA, Apparent Power"
      m_S_bus1_B = "m:S:bus1:B", "MVA, Apparent Power"
      m_S_bus1_C = "m:S:bus1:C", "MVA, Apparent Power"
      m_S_bus2_A = "m:S:bus2:A", "MVA, Apparent Power"
      m_S_bus2_B = "m:S:bus2:B", "MVA, Apparent Power"
      m_S_bus2_C = "m:S:bus2:C", "MVA, Apparent Power"
      m_cosphi_bus1_A = "m:cosphi:bus1:A", "Power Factor"
      m_cosphi_bus1_B = "m:cosphi:bus1:B", "Power Factor"
      m_cosphi_bus1_C = "m:cosphi:bus1:C", "Power Factor"
      m_cosphi_bus2_A = "m:cosphi:bus2:A", "Power Factor"
      m_cosphi_bus2_B = "m:cosphi:bus2:B", "Power Factor"
      m_cosphi_bus2_C = "m:cosphi:bus2:C", "Power Factor"
      m_tanphi_bus1_A = "m:tanphi:bus1:A", "tan(phi)"
      m_tanphi_bus1_B = "m:tanphi:bus1:B", "tan(phi)"
      m_tanphi_bus1_C = "m:tanphi:bus1:C", "tan(phi)"
      m_tanphi_bus2_A = "m:tanphi:bus2:A", "tan(phi)"
      m_tanphi_bus2_B = "m:tanphi:bus2:B", "tan(phi)"
      m_tanphi_bus2_C = "m:tanphi:bus2:C", "tan(phi)"
      m_Psum_bus1 = "m:Psum:bus1", "MW, Total Active Power"
      m_Psum_bus2 = "m:Psum:bus2", "MW, Total Active Power"
      m_Qsum_bus1 = "m:Qsum:bus1", "Mvar, Total Reactive Power"
      m_Qsum_bus2 = "m:Qsum:bus2", "Mvar, Total Reactive Power"
      m_Ssum_bus1 = "m:Ssum:bus1", "MVA, Total Apparent Power"
      m_Ssum_bus2 = "m:Ssum:bus2", "MVA, Total Apparent Power"
      m_cosphisum_bus1 = "m:cosphisum:bus1", "Total Power Factor"
      m_cosphisum_bus2 = "m:cosphisum:bus2", "Total Power Factor"
      m_tanphisum_bus1 = "m:tanphisum:bus1", "Total tan(phi)"
      m_tanphisum_bus2 = "m:tanphisum:bus2", "Total tan(phi)"
      m_Pln_bus1_A = "m:Pln:bus1:A", "MW, Active Power (Line-Neutral)"
      m_Pln_bus1_B = "m:Pln:bus1:B", "MW, Active Power (Line-Neutral)"
      m_Pln_bus1_C = "m:Pln:bus1:C", "MW, Active Power (Line-Neutral)"
      m_Pln_bus2_A = "m:Pln:bus2:A", "MW, Active Power (Line-Neutral)"
      m_Pln_bus2_B = "m:Pln:bus2:B", "MW, Active Power (Line-Neutral)"
      m_Pln_bus2_C = "m:Pln:bus2:C", "MW, Active Power (Line-Neutral)"
      m_Qln_bus1_A = "m:Qln:bus1:A", "Mvar, Reactive Power (Line-Neutral)"
      m_Qln_bus1_B = "m:Qln:bus1:B", "Mvar, Reactive Power (Line-Neutral)"
      m_Qln_bus1_C = "m:Qln:bus1:C", "Mvar, Reactive Power (Line-Neutral)"
      m_Qln_bus2_A = "m:Qln:bus2:A", "Mvar, Reactive Power (Line-Neutral)"
      m_Qln_bus2_B = "m:Qln:bus2:B", "Mvar, Reactive Power (Line-Neutral)"
      m_Qln_bus2_C = "m:Qln:bus2:C", "Mvar, Reactive Power (Line-Neutral)"
      m_P1_bus1 = "m:P1:bus1", "MW, Positive Sequence Active Power"
      m_P1_bus2 = "m:P1:bus2", "MW, Positive Sequence Active Power"
      m_Q1_bus1 = "m:Q1:bus1", "Mvar, Positive Sequence Reactive Power"
      m_Q1_bus2 = "m:Q1:bus2", "Mvar, Positive Sequence Reactive Power"
      m_P2_bus1 = "m:P2:bus1", "MW, Negative Sequence Active Power"
      m_P2_bus2 = "m:P2:bus2", "MW, Negative Sequence Active Power"
      m_Q2_bus1 = "m:Q2:bus1", "Mvar, Negative Sequence Reactive Power"
      m_Q2_bus2 = "m:Q2:bus2", "Mvar, Negative Sequence Reactive Power"
      m_P0_bus1 = "m:P0:bus1", "MW, Zero Sequence Active Power"
      m_P0_bus2 = "m:P0:bus2", "MW, Zero Sequence Active Power"
      m_Q0_bus1 = "m:Q0:bus1", "Mvar, Zero Sequence Reactive Power"
      m_Q0_bus2 = "m:Q0:bus2", "Mvar, Zero Sequence Reactive Power"
      m_i1P_bus1 = "m:i1P:bus1", "p.u., Positive-Sequence Active Current"
      m_i1P_bus2 = "m:i1P:bus2", "p.u., Positive-Sequence Active Current"
      m_i1Q_bus1 = "m:i1Q:bus1", "p.u., Positive-Sequence Reactive Current"
      m_i1Q_bus2 = "m:i1Q:bus2", "p.u., Positive-Sequence Reactive Current"
      m_I1P_bus1 = "m:I1P:bus1", "kA, Positive-Sequence Active Current"
      m_I1P_bus2 = "m:I1P:bus2", "kA, Positive-Sequence Active Current"
      m_I1Q_bus1 = "m:I1Q:bus1", "kA, Positive-Sequence Reactive Current"
      m_I1Q_bus2 = "m:I1Q:bus2", "kA, Positive-Sequence Reactive Current"
      m_i2P_bus1 = "m:i2P:bus1", "p.u., Negative-Sequence Active Current"
      m_i2P_bus2 = "m:i2P:bus2", "p.u., Negative-Sequence Active Current"
      m_i2Q_bus1 = "m:i2Q:bus1", "p.u., Negative-Sequence Reactive Current"
      m_i2Q_bus2 = "m:i2Q:bus2", "p.u., Negative-Sequence Reactive Current"
      m_I2P_bus1 = "m:I2P:bus1", "kA, Negative-Sequence Active Current"
      m_I2P_bus2 = "m:I2P:bus2", "kA, Negative-Sequence Active Current"
      m_I2Q_bus1 = "m:I2Q:bus1", "kA, Negative-Sequence Reactive Current"
      m_I2Q_bus2 = "m:I2Q:bus2", "kA, Negative-Sequence Reactive Current"
      m_ur_bus1_A = "m:ur:bus1:A", "p.u., Phase Voltage, Real Part"
      m_ur_bus1_B = "m:ur:bus1:B", "p.u., Phase Voltage, Real Part"
      m_ur_bus1_C = "m:ur:bus1:C", "p.u., Phase Voltage, Real Part"
      m_ur_bus2_A = "m:ur:bus2:A", "p.u., Phase Voltage, Real Part"
      m_ur_bus2_B = "m:ur:bus2:B", "p.u., Phase Voltage, Real Part"
      m_ur_bus2_C = "m:ur:bus2:C", "p.u., Phase Voltage, Real Part"
      m_ui_bus1_A = "m:ui:bus1:A", "p.u., Phase Voltage, Imaginary Part"
      m_ui_bus1_B = "m:ui:bus1:B", "p.u., Phase Voltage, Imaginary Part"
      m_ui_bus1_C = "m:ui:bus1:C", "p.u., Phase Voltage, Imaginary Part"
      m_ui_bus2_A = "m:ui:bus2:A", "p.u., Phase Voltage, Imaginary Part"
      m_ui_bus2_B = "m:ui:bus2:B", "p.u., Phase Voltage, Imaginary Part"
      m_ui_bus2_C = "m:ui:bus2:C", "p.u., Phase Voltage, Imaginary Part"
      m_u_bus1_A = "m:u:bus1:A", "p.u., Phase Voltage, Magnitude"
      m_u_bus1_B = "m:u:bus1:B", "p.u., Phase Voltage, Magnitude"
      m_u_bus1_C = "m:u:bus1:C", "p.u., Phase Voltage, Magnitude"
      m_u_bus2_A = "m:u:bus2:A", "p.u., Phase Voltage, Magnitude"
      m_u_bus2_B = "m:u:bus2:B", "p.u., Phase Voltage, Magnitude"
      m_u_bus2_C = "m:u:bus2:C", "p.u., Phase Voltage, Magnitude"
      m_ir_bus1_A = "m:ir:bus1:A", "p.u., Phase Current, Real Part"
      m_ir_bus1_B = "m:ir:bus1:B", "p.u., Phase Current, Real Part"
      m_ir_bus1_C = "m:ir:bus1:C", "p.u., Phase Current, Real Part"
      m_ir_bus2_A = "m:ir:bus2:A", "p.u., Phase Current, Real Part"
      m_ir_bus2_B = "m:ir:bus2:B", "p.u., Phase Current, Real Part"
      m_ir_bus2_C = "m:ir:bus2:C", "p.u., Phase Current, Real Part"
      m_ii_bus1_A = "m:ii:bus1:A", "p.u., Phase Current, Imaginary Part"
      m_ii_bus1_B = "m:ii:bus1:B", "p.u., Phase Current, Imaginary Part"
      m_ii_bus1_C = "m:ii:bus1:C", "p.u., Phase Current, Imaginary Part"
      m_ii_bus2_A = "m:ii:bus2:A", "p.u., Phase Current, Imaginary Part"
      m_ii_bus2_B = "m:ii:bus2:B", "p.u., Phase Current, Imaginary Part"
      m_ii_bus2_C = "m:ii:bus2:C", "p.u., Phase Current, Imaginary Part"
      m_i_bus1_A = "m:i:bus1:A", "p.u., Phase Current, Magnitude"
      m_i_bus1_B = "m:i:bus1:B", "p.u., Phase Current, Magnitude"
      m_i_bus1_C = "m:i:bus1:C", "p.u., Phase Current, Magnitude"
      m_i_bus2_A = "m:i:bus2:A", "p.u., Phase Current, Magnitude"
      m_i_bus2_B = "m:i:bus2:B", "p.u., Phase Current, Magnitude"
      m_i_bus2_C = "m:i:bus2:C", "p.u., Phase Current, Magnitude"
      m_phiui_bus1_A = "m:phiui:bus1:A", "deg, Angle between Voltage and Current"
      m_phiui_bus1_B = "m:phiui:bus1:B", "deg, Angle between Voltage and Current"
      m_phiui_bus1_C = "m:phiui:bus1:C", "deg, Angle between Voltage and Current"
      m_phiui_bus2_A = "m:phiui:bus2:A", "deg, Angle between Voltage and Current"
      m_phiui_bus2_B = "m:phiui:bus2:B", "deg, Angle between Voltage and Current"
      m_phiui_bus2_C = "m:phiui:bus2:C", "deg, Angle between Voltage and Current"
      m_phiu0i0_bus1 = "m:phiu0i0:bus1", "deg, Angle between Voltage and Current in zero-sequence system"
      m_phiu0i0_bus2 = "m:phiu0i0:bus2", "deg, Angle between Voltage and Current in zero-sequence system"
      m_phiu1i1_bus1 = "m:phiu1i1:bus1", "deg, Angle between Voltage and Current in positive sequence system"
      m_phiu1i1_bus2 = "m:phiu1i1:bus2", "deg, Angle between Voltage and Current in positive sequence system"
      m_phiu2i2_bus1 = "m:phiu2i2:bus1", "deg, Angle between Voltage and Current in negative sequence system"
      m_phiu2i2_bus2 = "m:phiu2i2:bus2", "deg, Angle between Voltage and Current in negative sequence system"
      m_Tfct_bus1 = "m:Tfct:bus1", "s, Fault Clearing Time"
      m_Tfct_bus2 = "m:Tfct:bus2", "s, Fault Clearing Time"
      m_TfctPh_bus1_A = "m:TfctPh:bus1:A", "s, Fault Clearing Time"
      m_TfctPh_bus1_B = "m:TfctPh:bus1:B", "s, Fault Clearing Time"
      m_TfctPh_bus1_C = "m:TfctPh:bus1:C", "s, Fault Clearing Time"
      m_TfctPh_bus2_A = "m:TfctPh:bus2:A", "s, Fault Clearing Time"
      m_TfctPh_bus2_B = "m:TfctPh:bus2:B", "s, Fault Clearing Time"
      m_TfctPh_bus2_C = "m:TfctPh:bus2:C", "s, Fault Clearing Time"
      m_Brkload_bus1 = "m:Brkload:bus1", "%, Max. Breaker Loading"
      m_Brkload_bus2 = "m:Brkload:bus2", "%, Max. Breaker Loading"
      m_BrkloadPh_bus1_A = "m:BrkloadPh:bus1:A", "%, Breaker Loading"
      m_BrkloadPh_bus1_B = "m:BrkloadPh:bus1:B", "%, Breaker Loading"
      m_BrkloadPh_bus1_C = "m:BrkloadPh:bus1:C", "%, Breaker Loading"
      m_BrkloadPh_bus2_A = "m:BrkloadPh:bus2:A", "%, Breaker Loading"
      m_BrkloadPh_bus2_B = "m:BrkloadPh:bus2:B", "%, Breaker Loading"
      m_BrkloadPh_bus2_C = "m:BrkloadPh:bus2:C", "%, Breaker Loading"
      n_ur_bus1_A = "n:ur:bus1:A", "p.u., Line-Ground Voltage, Real Part"
      n_ur_bus1_B = "n:ur:bus1:B", "p.u., Line-Ground Voltage, Real Part"
      n_ur_bus1_C = "n:ur:bus1:C", "p.u., Line-Ground Voltage, Real Part"
      n_ur_bus2_A = "n:ur:bus2:A", "p.u., Line-Ground Voltage, Real Part"
      n_ur_bus2_B = "n:ur:bus2:B", "p.u., Line-Ground Voltage, Real Part"
      n_ur_bus2_C = "n:ur:bus2:C", "p.u., Line-Ground Voltage, Real Part"
      n_ui_bus1_A = "n:ui:bus1:A", "p.u., Line-Ground Voltage, Imaginary Part"
      n_ui_bus1_B = "n:ui:bus1:B", "p.u., Line-Ground Voltage, Imaginary Part"
      n_ui_bus1_C = "n:ui:bus1:C", "p.u., Line-Ground Voltage, Imaginary Part"
      n_ui_bus2_A = "n:ui:bus2:A", "p.u., Line-Ground Voltage, Imaginary Part"
      n_ui_bus2_B = "n:ui:bus2:B", "p.u., Line-Ground Voltage, Imaginary Part"
      n_ui_bus2_C = "n:ui:bus2:C", "p.u., Line-Ground Voltage, Imaginary Part"
      n_u_bus1_A = "n:u:bus1:A", "p.u., Line-Ground Voltage, Magnitude"
      n_u_bus1_B = "n:u:bus1:B", "p.u., Line-Ground Voltage, Magnitude"
      n_u_bus1_C = "n:u:bus1:C", "p.u., Line-Ground Voltage, Magnitude"
      n_u_bus2_A = "n:u:bus2:A", "p.u., Line-Ground Voltage, Magnitude"
      n_u_bus2_B = "n:u:bus2:B", "p.u., Line-Ground Voltage, Magnitude"
      n_u_bus2_C = "n:u:bus2:C", "p.u., Line-Ground Voltage, Magnitude"
      n_upc_bus1_A = "n:upc:bus1:A", "%, Line-Ground Voltage, Magnitude"
      n_upc_bus1_B = "n:upc:bus1:B", "%, Line-Ground Voltage, Magnitude"
      n_upc_bus1_C = "n:upc:bus1:C", "%, Line-Ground Voltage, Magnitude"
      n_upc_bus2_A = "n:upc:bus2:A", "%, Line-Ground Voltage, Magnitude"
      n_upc_bus2_B = "n:upc:bus2:B", "%, Line-Ground Voltage, Magnitude"
      n_upc_bus2_C = "n:upc:bus2:C", "%, Line-Ground Voltage, Magnitude"
      n_U_bus1_A = "n:U:bus1:A", "kV, Line-Ground Voltage, Magnitude"
      n_U_bus1_B = "n:U:bus1:B", "kV, Line-Ground Voltage, Magnitude"
      n_U_bus1_C = "n:U:bus1:C", "kV, Line-Ground Voltage, Magnitude"
      n_U_bus2_A = "n:U:bus2:A", "kV, Line-Ground Voltage, Magnitude"
      n_U_bus2_B = "n:U:bus2:B", "kV, Line-Ground Voltage, Magnitude"
      n_U_bus2_C = "n:U:bus2:C", "kV, Line-Ground Voltage, Magnitude"
      n_phiu_bus1_A = "n:phiu:bus1:A", "deg, Line-Ground Voltage, Angle"
      n_phiu_bus1_B = "n:phiu:bus1:B", "deg, Line-Ground Voltage, Angle"
      n_phiu_bus1_C = "n:phiu:bus1:C", "deg, Line-Ground Voltage, Angle"
      n_phiu_bus2_A = "n:phiu:bus2:A", "deg, Line-Ground Voltage, Angle"
      n_phiu_bus2_B = "n:phiu:bus2:B", "deg, Line-Ground Voltage, Angle"
      n_phiu_bus2_C = "n:phiu:bus2:C", "deg, Line-Ground Voltage, Angle"
      n_phiurel_bus1 = "n:phiurel:bus1", "deg, Voltage, Relative Angle"
      n_phiurel_bus2 = "n:phiurel:bus2", "deg, Voltage, Relative Angle"
      n_ul_bus1_A = "n:ul:bus1:A", "p.u., Line-Line Voltage, Magnitude"
      n_ul_bus1_B = "n:ul:bus1:B", "p.u., Line-Line Voltage, Magnitude"
      n_ul_bus1_C = "n:ul:bus1:C", "p.u., Line-Line Voltage, Magnitude"
      n_ul_bus2_A = "n:ul:bus2:A", "p.u., Line-Line Voltage, Magnitude"
      n_ul_bus2_B = "n:ul:bus2:B", "p.u., Line-Line Voltage, Magnitude"
      n_ul_bus2_C = "n:ul:bus2:C", "p.u., Line-Line Voltage, Magnitude"
      n_ulpc_bus1_A = "n:ulpc:bus1:A", "%, Line-Line Voltage, Magnitude"
      n_ulpc_bus1_B = "n:ulpc:bus1:B", "%, Line-Line Voltage, Magnitude"
      n_ulpc_bus1_C = "n:ulpc:bus1:C", "%, Line-Line Voltage, Magnitude"
      n_ulpc_bus2_A = "n:ulpc:bus2:A", "%, Line-Line Voltage, Magnitude"
      n_ulpc_bus2_B = "n:ulpc:bus2:B", "%, Line-Line Voltage, Magnitude"
      n_ulpc_bus2_C = "n:ulpc:bus2:C", "%, Line-Line Voltage, Magnitude"
      n_Ul_bus1_A = "n:Ul:bus1:A", "kV, Line-Line Voltage, Magnitude"
      n_Ul_bus1_B = "n:Ul:bus1:B", "kV, Line-Line Voltage, Magnitude"
      n_Ul_bus1_C = "n:Ul:bus1:C", "kV, Line-Line Voltage, Magnitude"
      n_Ul_bus2_A = "n:Ul:bus2:A", "kV, Line-Line Voltage, Magnitude"
      n_Ul_bus2_B = "n:Ul:bus2:B", "kV, Line-Line Voltage, Magnitude"
      n_Ul_bus2_C = "n:Ul:bus2:C", "kV, Line-Line Voltage, Magnitude"
      n_phiul_bus1_A = "n:phiul:bus1:A", "deg, Line-Line Voltage, Angle"
      n_phiul_bus1_B = "n:phiul:bus1:B", "deg, Line-Line Voltage, Angle"
      n_phiul_bus1_C = "n:phiul:bus1:C", "deg, Line-Line Voltage, Angle"
      n_phiul_bus2_A = "n:phiul:bus2:A", "deg, Line-Line Voltage, Angle"
      n_phiul_bus2_B = "n:phiul:bus2:B", "deg, Line-Line Voltage, Angle"
      n_phiul_bus2_C = "n:phiul:bus2:C", "deg, Line-Line Voltage, Angle"
      n_u0_bus1 = "n:u0:bus1", "p.u., Zero-Sequence Voltage, Magnitude"
      n_u0_bus2 = "n:u0:bus2", "p.u., Zero-Sequence Voltage, Magnitude"
      n_U0_bus1 = "n:U0:bus1", "kV, Zero-Sequence Voltage, Magnitude"
      n_U0_bus2 = "n:U0:bus2", "kV, Zero-Sequence Voltage, Magnitude"
      n_U0x3_bus1 = "n:U0x3:bus1", "kV, 3*U0"
      n_U0x3_bus2 = "n:U0x3:bus2", "kV, 3*U0"
      n_phiu0_bus1 = "n:phiu0:bus1", "deg, Zero-Sequence Voltage, Angle"
      n_phiu0_bus2 = "n:phiu0:bus2", "deg, Zero-Sequence Voltage, Angle"
      n_u1_bus1 = "n:u1:bus1", "p.u., Positive-Sequence Voltage, Magnitude"
      n_u1_bus2 = "n:u1:bus2", "p.u., Positive-Sequence Voltage, Magnitude"
      n_u1pc_bus1 = "n:u1pc:bus1", "%, Positive-Sequence Voltage, Magnitude"
      n_u1pc_bus2 = "n:u1pc:bus2", "%, Positive-Sequence Voltage, Magnitude"
      n_u1r_bus1 = "n:u1r:bus1", "p.u., Positive-Sequence Voltage, Real Part"
      n_u1r_bus2 = "n:u1r:bus2", "p.u., Positive-Sequence Voltage, Real Part"
      n_u1i_bus1 = "n:u1i:bus1", "p.u., Positive-Sequence Voltage, Imaginary Part"
      n_u1i_bus2 = "n:u1i:bus2", "p.u., Positive-Sequence Voltage, Imaginary Part"
      n_U1_bus1 = "n:U1:bus1", "kV, Line-Ground Positive-Sequence Voltage, Magnitude"
      n_U1_bus2 = "n:U1:bus2", "kV, Line-Ground Positive-Sequence Voltage, Magnitude"
      n_phiu1_bus1 = "n:phiu1:bus1", "deg, Positive-Sequence Voltage, Angle"
      n_phiu1_bus2 = "n:phiu1:bus2", "deg, Positive-Sequence Voltage, Angle"
      n_u2_bus1 = "n:u2:bus1", "p.u., Negative-Sequence Voltage, Magnitude"
      n_u2_bus2 = "n:u2:bus2", "p.u., Negative-Sequence Voltage, Magnitude"
      n_U2_bus1 = "n:U2:bus1", "kV, Line-Ground Negative-Sequence Voltage, Magnitude"
      n_U2_bus2 = "n:U2:bus2", "kV, Line-Ground Negative-Sequence Voltage, Magnitude"
      n_phiu2_bus1 = "n:phiu2:bus1", "deg, Negative-Sequence Voltage, Angle"
      n_phiu2_bus2 = "n:phiu2:bus2", "deg, Negative-Sequence Voltage, Angle"
      n_U1l_bus1 = "n:U1l:bus1", "kV, Line-Line Positive-Sequence Voltage, Magnitude"
      n_U1l_bus2 = "n:U1l:bus2", "kV, Line-Line Positive-Sequence Voltage, Magnitude"
      n_U2l_bus1 = "n:U2l:bus1", "kV, Line-Line Negative-Sequence Voltage, Magnitude"
      n_U2l_bus2 = "n:U2l:bus2", "kV, Line-Line Negative-Sequence Voltage, Magnitude"
      n_uln_bus1_A = "n:uln:bus1:A", "p.u., Line-Neutral Voltage, Magnitude"
      n_uln_bus1_B = "n:uln:bus1:B", "p.u., Line-Neutral Voltage, Magnitude"
      n_uln_bus1_C = "n:uln:bus1:C", "p.u., Line-Neutral Voltage, Magnitude"
      n_uln_bus2_A = "n:uln:bus2:A", "p.u., Line-Neutral Voltage, Magnitude"
      n_uln_bus2_B = "n:uln:bus2:B", "p.u., Line-Neutral Voltage, Magnitude"
      n_uln_bus2_C = "n:uln:bus2:C", "p.u., Line-Neutral Voltage, Magnitude"
      n_Uln_bus1_A = "n:Uln:bus1:A", "kV, Line-Neutral Voltage, Magnitude"
      n_Uln_bus1_B = "n:Uln:bus1:B", "kV, Line-Neutral Voltage, Magnitude"
      n_Uln_bus1_C = "n:Uln:bus1:C", "kV, Line-Neutral Voltage, Magnitude"
      n_Uln_bus2_A = "n:Uln:bus2:A", "kV, Line-Neutral Voltage, Magnitude"
      n_Uln_bus2_B = "n:Uln:bus2:B", "kV, Line-Neutral Voltage, Magnitude"
      n_Uln_bus2_C = "n:Uln:bus2:C", "kV, Line-Neutral Voltage, Magnitude"
      n_phiuln_bus1_A = "n:phiuln:bus1:A", "deg, Line-Neutral Voltage, Angle"
      n_phiuln_bus1_B = "n:phiuln:bus1:B", "deg, Line-Neutral Voltage, Angle"
      n_phiuln_bus1_C = "n:phiuln:bus1:C", "deg, Line-Neutral Voltage, Angle"
      n_phiuln_bus2_A = "n:phiuln:bus2:A", "deg, Line-Neutral Voltage, Angle"
      n_phiuln_bus2_B = "n:phiuln:bus2:B", "deg, Line-Neutral Voltage, Angle"
      n_phiuln_bus2_C = "n:phiuln:bus2:C", "deg, Line-Neutral Voltage, Angle"
      n_un_bus1 = "n:un:bus1", "p.u., Neutral-Ground Voltage, Magnitude"
      n_un_bus2 = "n:un:bus2", "p.u., Neutral-Ground Voltage, Magnitude"
      n_Un_bus1 = "n:Un:bus1", "kV, Neutral-Ground Voltage, Magnitude"
      n_Un_bus2 = "n:Un:bus2", "kV, Neutral-Ground Voltage, Magnitude"
      n_phiun_bus1 = "n:phiun:bus1", "deg, Neutral-Ground Voltage, Angle"
      n_phiun_bus2 = "n:phiun:bus2", "deg, Neutral-Ground Voltage, Angle"
      n_upht_bus1_A = "n:upht:bus1:A", "p.u., Phase Technology dependent Voltage, Magnitude"
      n_upht_bus1_B = "n:upht:bus1:B", "p.u., Phase Technology dependent Voltage, Magnitude"
      n_upht_bus1_C = "n:upht:bus1:C", "p.u., Phase Technology dependent Voltage, Magnitude"
      n_upht_bus2_A = "n:upht:bus2:A", "p.u., Phase Technology dependent Voltage, Magnitude"
      n_upht_bus2_B = "n:upht:bus2:B", "p.u., Phase Technology dependent Voltage, Magnitude"
      n_upht_bus2_C = "n:upht:bus2:C", "p.u., Phase Technology dependent Voltage, Magnitude"
      n_Upht_bus1_A = "n:Upht:bus1:A", "kV, Phase Technology dependent Voltage, Magnitude"
      n_Upht_bus1_B = "n:Upht:bus1:B", "kV, Phase Technology dependent Voltage, Magnitude"
      n_Upht_bus1_C = "n:Upht:bus1:C", "kV, Phase Technology dependent Voltage, Magnitude"
      n_Upht_bus2_A = "n:Upht:bus2:A", "kV, Phase Technology dependent Voltage, Magnitude"
      n_Upht_bus2_B = "n:Upht:bus2:B", "kV, Phase Technology dependent Voltage, Magnitude"
      n_Upht_bus2_C = "n:Upht:bus2:C", "kV, Phase Technology dependent Voltage, Magnitude"
      n_um_bus1 = "n:um:bus1", "p.u., Average-Voltage, Magnitude"
      n_um_bus2 = "n:um:bus2", "p.u., Average-Voltage, Magnitude"
      n_Um_bus1 = "n:Um:bus1", "kV, Average-Voltage, Magnitude"
      n_Um_bus2 = "n:Um:bus2", "kV, Average-Voltage, Magnitude"
      n_ishcr_bus1_A = "n:ishcr:bus1:A", "p.u., Short-Circuit Current, real"
      n_ishcr_bus1_B = "n:ishcr:bus1:B", "p.u., Short-Circuit Current, real"
      n_ishcr_bus1_C = "n:ishcr:bus1:C", "p.u., Short-Circuit Current, real"
      n_ishcr_bus2_A = "n:ishcr:bus2:A", "p.u., Short-Circuit Current, real"
      n_ishcr_bus2_B = "n:ishcr:bus2:B", "p.u., Short-Circuit Current, real"
      n_ishcr_bus2_C = "n:ishcr:bus2:C", "p.u., Short-Circuit Current, real"
      n_ishci_bus1_A = "n:ishci:bus1:A", "p.u., Short-Circuit Current, imag"
      n_ishci_bus1_B = "n:ishci:bus1:B", "p.u., Short-Circuit Current, imag"
      n_ishci_bus1_C = "n:ishci:bus1:C", "p.u., Short-Circuit Current, imag"
      n_ishci_bus2_A = "n:ishci:bus2:A", "p.u., Short-Circuit Current, imag"
      n_ishci_bus2_B = "n:ishci:bus2:B", "p.u., Short-Circuit Current, imag"
      n_ishci_bus2_C = "n:ishci:bus2:C", "p.u., Short-Circuit Current, imag"
      n_ishc_bus1_A = "n:ishc:bus1:A", "p.u., Short-Circuit Current, magnitude"
      n_ishc_bus1_B = "n:ishc:bus1:B", "p.u., Short-Circuit Current, magnitude"
      n_ishc_bus1_C = "n:ishc:bus1:C", "p.u., Short-Circuit Current, magnitude"
      n_ishc_bus2_A = "n:ishc:bus2:A", "p.u., Short-Circuit Current, magnitude"
      n_ishc_bus2_B = "n:ishc:bus2:B", "p.u., Short-Circuit Current, magnitude"
      n_ishc_bus2_C = "n:ishc:bus2:C", "p.u., Short-Circuit Current, magnitude"
      n_Ishc_bus1_A = "n:Ishc:bus1:A", "kA, Short-Circuit Current, magnitude"
      n_Ishc_bus1_B = "n:Ishc:bus1:B", "kA, Short-Circuit Current, magnitude"
      n_Ishc_bus1_C = "n:Ishc:bus1:C", "kA, Short-Circuit Current, magnitude"
      n_Ishc_bus2_A = "n:Ishc:bus2:A", "kA, Short-Circuit Current, magnitude"
      n_Ishc_bus2_B = "n:Ishc:bus2:B", "kA, Short-Circuit Current, magnitude"
      n_Ishc_bus2_C = "n:Ishc:bus2:C", "kA, Short-Circuit Current, magnitude"
      n_inshc_bus1 = "n:inshc:bus1", "p.u., Neutral Short-Circuit Current, magnitude"
      n_inshc_bus2 = "n:inshc:bus2", "p.u., Neutral Short-Circuit Current, magnitude"
      n_Inshc_bus1 = "n:Inshc:bus1", "kA, Neutral Short-Circuit Current, magnitude"
      n_Inshc_bus2 = "n:Inshc:bus2", "kA, Neutral Short-Circuit Current, magnitude"
      n_fe_bus1 = "n:fe:bus1", "p.u., Electrical Frequency"
      n_fe_bus2 = "n:fe:bus2", "p.u., Electrical Frequency"
      n_frdev_bus1 = "n:frdev:bus1", "Hz, average frequency"
      n_frdev_bus2 = "n:frdev:bus2", "Hz, average frequency"
      n_dfehz_bus1 = "n:dfehz:bus1", "Hz, Deviation of the El. Frequency"
      n_dfehz_bus2 = "n:dfehz:bus2", "Hz, Deviation of the El. Frequency"
      n_dfedt_bus1 = "n:dfedt:bus1", "p.u./s, Derivative of El. Frequency"
      n_dfedt_bus2 = "n:dfedt:bus2", "p.u./s, Derivative of El. Frequency"
      n_fehz_bus1 = "n:fehz:bus1", "Hz, Electrical Frequency"
      n_fehz_bus2 = "n:fehz:bus2", "Hz, Electrical Frequency"
      n_frnom_bus1 = "n:frnom:bus1", "Hz, Nominal Frequency"
      n_frnom_bus2 = "n:frnom:bus2", "Hz, Nominal Frequency"
      s_Rin = "s:Rin", "Ohm, IN, Resistance (Input), continuous, double"
      s_Xin = "s:Xin", "Ohm, IN, Reactance (Input), continuous, double"
      c_Zind_r_r = "c:Zind_r:r", "Ohm, Impedance, Phase a, Real Part"
      c_Zind_r_i = "c:Zind_r:i", "Ohm, Impedance, Phase a, Imaginary Part"
      c_Zind_r_phi = "c:Zind_r:phi", "deg, Impedance, Phase a, Angle"
      c_Zind_r_mag = "c:Zind_r:mag", "Ohm, Impedance, Phase a, Magnitude"
      c_Zind_r_phirad = "c:Zind_r:phirad", "rad, Impedance, Phase a, Angle"
      c_Zind_s_r = "c:Zind_s:r", "Ohm, Impedance, Phase b, Real Part"
      c_Zind_s_i = "c:Zind_s:i", "Ohm, Impedance, Phase b, Imaginary Part"
      c_Zind_s_phi = "c:Zind_s:phi", "deg, Impedance, Phase b, Angle"
      c_Zind_s_mag = "c:Zind_s:mag", "Ohm, Impedance, Phase b, Magnitude"
      c_Zind_s_phirad = "c:Zind_s:phirad", "rad, Impedance, Phase b, Angle"
      c_Zind_t_r = "c:Zind_t:r", "Ohm, Impedance, Phase c, Real Part"
      c_Zind_t_i = "c:Zind_t:i", "Ohm, Impedance, Phase c, Imaginary Part"
      c_Zind_t_phi = "c:Zind_t:phi", "deg, Impedance, Phase c, Angle"
      c_Zind_t_mag = "c:Zind_t:mag", "Ohm, Impedance, Phase c, Magnitude"
      c_Zind_t_phirad = "c:Zind_t:phirad", "rad, Impedance, Phase c, Angle"
      c_Z0_r = "c:Z0:r", "Ohm, Zero Sequence Impedance, Real Part"
      c_Z0_i = "c:Z0:i", "Ohm, Zero Sequence Impedance, Imaginary Part"
      c_Z0_phi = "c:Z0:phi", "deg, Zero Sequence Impedance, Angle"
      c_Z0_mag = "c:Z0:mag", "Ohm, Zero Sequence Impedance, Magnitude"
      c_Z0_phirad = "c:Z0:phirad", "rad, Zero Sequence Impedance, Angle"
      c_loading = "c:loading", "%, Loading"
      c_Inom = "c:Inom", "kA, Nominal Current"
      e_iAstabint = "e:iAstabint", "A-stable integration algorithm"
      e_emtLinSig = "e:emtLinSig", "Variable inductance/reactance via input signals"
      e_uk = "e:uk", "%, Short-Circuit Voltage uk"
      e_Zd = "e:Zd", "Ohm, Impedance (absolute) Zd"
      e_lrea = "e:lrea", "mH, Inductance, L"
      e_xrea = "e:xrea", "Ohm, Reactance, X"
      e_Pcu = "e:Pcu", "kW, Copper Losses"
      e_ukr = "e:ukr", "%, Short-Circuit Voltage (Re(uk)) ukr"
      e_rrea = "e:rrea", "Ohm, Resistance, R"
      e_R0toR1 = "e:R0toR1", "R0/R1 ratio"
      e_X0toX1 = "e:X0toX1", "X0/X1 ratio"

    class ElmStactrl(Enum):
      _init_ = 'value __doc__'


    class ElmSym(Enum):
      _init_ = 'value __doc__'

      m_u0_bus1 = "m:u0:bus1", "p.u., Zero-Sequence-Voltage, Magnitude"
      m_u0_bus1n = "m:u0:bus1n", "p.u., Zero-Sequence-Voltage, Magnitude"
      m_phiu0_bus1 = "m:phiu0:bus1", "deg, Zero-Sequence-Voltage, Angle"
      m_phiu0_bus1n = "m:phiu0:bus1n", "deg, Zero-Sequence-Voltage, Angle"
      m_u1r_bus1 = "m:u1r:bus1", "p.u., Positive-Sequence-Voltage, Real Part"
      m_u1r_bus1n = "m:u1r:bus1n", "p.u., Positive-Sequence-Voltage, Real Part"
      m_u1i_bus1 = "m:u1i:bus1", "p.u., Positive-Sequence-Voltage, Imaginary Part"
      m_u1i_bus1n = "m:u1i:bus1n", "p.u., Positive-Sequence-Voltage, Imaginary Part"
      m_u1_bus1 = "m:u1:bus1", "p.u., Positive-Sequence-Voltage, Magnitude"
      m_u1_bus1n = "m:u1:bus1n", "p.u., Positive-Sequence-Voltage, Magnitude"
      m_U1_bus1 = "m:U1:bus1", "kV, Line-Ground Positive-Sequence-Voltage, Magnitude"
      m_U1_bus1n = "m:U1:bus1n", "kV, Line-Ground Positive-Sequence-Voltage, Magnitude"
      m_U1l_bus1 = "m:U1l:bus1", "kV, Line-Line Positive-Sequence-Voltage, Magnitude"
      m_U1l_bus1n = "m:U1l:bus1n", "kV, Line-Line Positive-Sequence-Voltage, Magnitude"
      m_phiu1_bus1 = "m:phiu1:bus1", "deg, Positive-Sequence-Voltage, Angle"
      m_phiu1_bus1n = "m:phiu1:bus1n", "deg, Positive-Sequence-Voltage, Angle"
      m_u2_bus1 = "m:u2:bus1", "p.u., Negative-Sequence-Voltage, Magnitude"
      m_u2_bus1n = "m:u2:bus1n", "p.u., Negative-Sequence-Voltage, Magnitude"
      m_phiu2_bus1 = "m:phiu2:bus1", "deg, Negative-Sequence-Voltage, Angle"
      m_phiu2_bus1n = "m:phiu2:bus1n", "deg, Negative-Sequence-Voltage, Angle"
      m_inet_bus1_A = "m:inet:bus1:A", "p.u., Phase Current, Magnitude"
      m_inet_bus1_B = "m:inet:bus1:B", "p.u., Phase Current, Magnitude"
      m_inet_bus1_C = "m:inet:bus1:C", "p.u., Phase Current, Magnitude"
      m_inet_bus1_N = "m:inet:bus1:N", "p.u., Phase Current, Magnitude"
      m_inet_bus1n = "m:inet:bus1n", "p.u., Phase Current, Magnitude"
      m_I_bus1_A = "m:I:bus1:A", "kA, Phase Current, Magnitude"
      m_I_bus1_B = "m:I:bus1:B", "kA, Phase Current, Magnitude"
      m_I_bus1_C = "m:I:bus1:C", "kA, Phase Current, Magnitude"
      m_I_bus1_N = "m:I:bus1:N", "kA, Phase Current, Magnitude"
      m_I_bus1n = "m:I:bus1n", "kA, Phase Current, Magnitude"
      m_phii_bus1_A = "m:phii:bus1:A", "deg, Phase Current, Angle"
      m_phii_bus1_B = "m:phii:bus1:B", "deg, Phase Current, Angle"
      m_phii_bus1_C = "m:phii:bus1:C", "deg, Phase Current, Angle"
      m_phii_bus1_N = "m:phii:bus1:N", "deg, Phase Current, Angle"
      m_phii_bus1n = "m:phii:bus1n", "deg, Phase Current, Angle"
      m_i0r_bus1 = "m:i0r:bus1", "p.u., Zero-Sequence Current, Real Part"
      m_i0r_bus1n = "m:i0r:bus1n", "p.u., Zero-Sequence Current, Real Part"
      m_i0i_bus1 = "m:i0i:bus1", "p.u., Zero-Sequence Current, Imaginary Part"
      m_i0i_bus1n = "m:i0i:bus1n", "p.u., Zero-Sequence Current, Imaginary Part"
      m_i0_bus1 = "m:i0:bus1", "p.u., Zero-Sequence Current, Magnitude"
      m_i0_bus1n = "m:i0:bus1n", "p.u., Zero-Sequence Current, Magnitude"
      m_I0_bus1 = "m:I0:bus1", "kA, Zero-Sequence Current, Magnitude"
      m_I0_bus1n = "m:I0:bus1n", "kA, Zero-Sequence Current, Magnitude"
      m_I0x3_bus1 = "m:I0x3:bus1", "kA, 3*I0"
      m_I0x3_bus1n = "m:I0x3:bus1n", "kA, 3*I0"
      m_phii0_bus1 = "m:phii0:bus1", "deg, Zero-Sequence Current, Angle"
      m_phii0_bus1n = "m:phii0:bus1n", "deg, Zero-Sequence Current, Angle"
      m_i1r_bus1 = "m:i1r:bus1", "p.u., Positive-Sequence Current, Real Part"
      m_i1r_bus1n = "m:i1r:bus1n", "p.u., Positive-Sequence Current, Real Part"
      m_i1i_bus1 = "m:i1i:bus1", "p.u., Positive-Sequence Current, Imaginary Part"
      m_i1i_bus1n = "m:i1i:bus1n", "p.u., Positive-Sequence Current, Imaginary Part"
      m_i1_bus1 = "m:i1:bus1", "p.u., Positive-Sequence Current, Magnitude"
      m_i1_bus1n = "m:i1:bus1n", "p.u., Positive-Sequence Current, Magnitude"
      m_I1_bus1 = "m:I1:bus1", "kA, Positive-Sequence Current, Magnitude"
      m_I1_bus1n = "m:I1:bus1n", "kA, Positive-Sequence Current, Magnitude"
      m_phii1_bus1 = "m:phii1:bus1", "deg, Positive-Sequence Current, Angle"
      m_phii1_bus1n = "m:phii1:bus1n", "deg, Positive-Sequence Current, Angle"
      m_i2r_bus1 = "m:i2r:bus1", "p.u., Negative-Sequence Current, Real Part"
      m_i2r_bus1n = "m:i2r:bus1n", "p.u., Negative-Sequence Current, Real Part"
      m_i2i_bus1 = "m:i2i:bus1", "p.u., Negative-Sequence Current, Imaginary Part"
      m_i2i_bus1n = "m:i2i:bus1n", "p.u., Negative-Sequence Current, Imaginary Part"
      m_i2_bus1 = "m:i2:bus1", "p.u., Negative-Sequence Current, Magnitude"
      m_i2_bus1n = "m:i2:bus1n", "p.u., Negative-Sequence Current, Magnitude"
      m_I2_bus1 = "m:I2:bus1", "kA, Negative-Sequence Current, Magnitude"
      m_I2_bus1n = "m:I2:bus1n", "kA, Negative-Sequence Current, Magnitude"
      m_phii2_bus1 = "m:phii2:bus1", "deg, Negative-Sequence Current, Angle"
      m_phii2_bus1n = "m:phii2:bus1n", "deg, Negative-Sequence Current, Angle"
      m_P_bus1_A = "m:P:bus1:A", "MW, Active Power"
      m_P_bus1_B = "m:P:bus1:B", "MW, Active Power"
      m_P_bus1_C = "m:P:bus1:C", "MW, Active Power"
      m_P_bus1_N = "m:P:bus1:N", "MW, Active Power"
      m_P_bus1n = "m:P:bus1n", "MW, Active Power"
      m_Q_bus1_A = "m:Q:bus1:A", "Mvar, Reactive Power"
      m_Q_bus1_B = "m:Q:bus1:B", "Mvar, Reactive Power"
      m_Q_bus1_C = "m:Q:bus1:C", "Mvar, Reactive Power"
      m_Q_bus1_N = "m:Q:bus1:N", "Mvar, Reactive Power"
      m_Q_bus1n = "m:Q:bus1n", "Mvar, Reactive Power"
      m_S_bus1_A = "m:S:bus1:A", "MVA, Apparent Power"
      m_S_bus1_B = "m:S:bus1:B", "MVA, Apparent Power"
      m_S_bus1_C = "m:S:bus1:C", "MVA, Apparent Power"
      m_S_bus1_N = "m:S:bus1:N", "MVA, Apparent Power"
      m_S_bus1n = "m:S:bus1n", "MVA, Apparent Power"
      m_cosphi_bus1_A = "m:cosphi:bus1:A", "Power Factor"
      m_cosphi_bus1_B = "m:cosphi:bus1:B", "Power Factor"
      m_cosphi_bus1_C = "m:cosphi:bus1:C", "Power Factor"
      m_cosphi_bus1_N = "m:cosphi:bus1:N", "Power Factor"
      m_cosphi_bus1n = "m:cosphi:bus1n", "Power Factor"
      m_tanphi_bus1_A = "m:tanphi:bus1:A", "tan(phi)"
      m_tanphi_bus1_B = "m:tanphi:bus1:B", "tan(phi)"
      m_tanphi_bus1_C = "m:tanphi:bus1:C", "tan(phi)"
      m_tanphi_bus1_N = "m:tanphi:bus1:N", "tan(phi)"
      m_tanphi_bus1n = "m:tanphi:bus1n", "tan(phi)"
      m_Psum_bus1 = "m:Psum:bus1", "MW, Total Active Power"
      m_Psum_bus1n = "m:Psum:bus1n", "MW, Total Active Power"
      m_Qsum_bus1 = "m:Qsum:bus1", "Mvar, Total Reactive Power"
      m_Qsum_bus1n = "m:Qsum:bus1n", "Mvar, Total Reactive Power"
      m_Ssum_bus1 = "m:Ssum:bus1", "MVA, Total Apparent Power"
      m_Ssum_bus1n = "m:Ssum:bus1n", "MVA, Total Apparent Power"
      m_cosphisum_bus1 = "m:cosphisum:bus1", "Total Power Factor"
      m_cosphisum_bus1n = "m:cosphisum:bus1n", "Total Power Factor"
      m_tanphisum_bus1 = "m:tanphisum:bus1", "Total tan(phi)"
      m_tanphisum_bus1n = "m:tanphisum:bus1n", "Total tan(phi)"
      m_Pln_bus1_A = "m:Pln:bus1:A", "MW, Active Power (Line-Neutral)"
      m_Pln_bus1_B = "m:Pln:bus1:B", "MW, Active Power (Line-Neutral)"
      m_Pln_bus1_C = "m:Pln:bus1:C", "MW, Active Power (Line-Neutral)"
      m_Pln_bus1_N = "m:Pln:bus1:N", "MW, Active Power (Line-Neutral)"
      m_Pln_bus1n = "m:Pln:bus1n", "MW, Active Power (Line-Neutral)"
      m_Qln_bus1_A = "m:Qln:bus1:A", "Mvar, Reactive Power (Line-Neutral)"
      m_Qln_bus1_B = "m:Qln:bus1:B", "Mvar, Reactive Power (Line-Neutral)"
      m_Qln_bus1_C = "m:Qln:bus1:C", "Mvar, Reactive Power (Line-Neutral)"
      m_Qln_bus1_N = "m:Qln:bus1:N", "Mvar, Reactive Power (Line-Neutral)"
      m_Qln_bus1n = "m:Qln:bus1n", "Mvar, Reactive Power (Line-Neutral)"
      m_P1_bus1 = "m:P1:bus1", "MW, Positive Sequence Active Power"
      m_P1_bus1n = "m:P1:bus1n", "MW, Positive Sequence Active Power"
      m_Q1_bus1 = "m:Q1:bus1", "Mvar, Positive Sequence Reactive Power"
      m_Q1_bus1n = "m:Q1:bus1n", "Mvar, Positive Sequence Reactive Power"
      m_P2_bus1 = "m:P2:bus1", "MW, Negative Sequence Active Power"
      m_P2_bus1n = "m:P2:bus1n", "MW, Negative Sequence Active Power"
      m_Q2_bus1 = "m:Q2:bus1", "Mvar, Negative Sequence Reactive Power"
      m_Q2_bus1n = "m:Q2:bus1n", "Mvar, Negative Sequence Reactive Power"
      m_P0_bus1 = "m:P0:bus1", "MW, Zero Sequence Active Power"
      m_P0_bus1n = "m:P0:bus1n", "MW, Zero Sequence Active Power"
      m_Q0_bus1 = "m:Q0:bus1", "Mvar, Zero Sequence Reactive Power"
      m_Q0_bus1n = "m:Q0:bus1n", "Mvar, Zero Sequence Reactive Power"
      m_i1P_bus1 = "m:i1P:bus1", "p.u., Positive-Sequence Active Current"
      m_i1P_bus1n = "m:i1P:bus1n", "p.u., Positive-Sequence Active Current"
      m_i1Q_bus1 = "m:i1Q:bus1", "p.u., Positive-Sequence Reactive Current"
      m_i1Q_bus1n = "m:i1Q:bus1n", "p.u., Positive-Sequence Reactive Current"
      m_I1P_bus1 = "m:I1P:bus1", "kA, Positive-Sequence Active Current"
      m_I1P_bus1n = "m:I1P:bus1n", "kA, Positive-Sequence Active Current"
      m_I1Q_bus1 = "m:I1Q:bus1", "kA, Positive-Sequence Reactive Current"
      m_I1Q_bus1n = "m:I1Q:bus1n", "kA, Positive-Sequence Reactive Current"
      m_i2P_bus1 = "m:i2P:bus1", "p.u., Negative-Sequence Active Current"
      m_i2P_bus1n = "m:i2P:bus1n", "p.u., Negative-Sequence Active Current"
      m_i2Q_bus1 = "m:i2Q:bus1", "p.u., Negative-Sequence Reactive Current"
      m_i2Q_bus1n = "m:i2Q:bus1n", "p.u., Negative-Sequence Reactive Current"
      m_I2P_bus1 = "m:I2P:bus1", "kA, Negative-Sequence Active Current"
      m_I2P_bus1n = "m:I2P:bus1n", "kA, Negative-Sequence Active Current"
      m_I2Q_bus1 = "m:I2Q:bus1", "kA, Negative-Sequence Reactive Current"
      m_I2Q_bus1n = "m:I2Q:bus1n", "kA, Negative-Sequence Reactive Current"
      m_ur_bus1_A = "m:ur:bus1:A", "p.u., Phase Voltage, Real Part"
      m_ur_bus1_B = "m:ur:bus1:B", "p.u., Phase Voltage, Real Part"
      m_ur_bus1_C = "m:ur:bus1:C", "p.u., Phase Voltage, Real Part"
      m_ur_bus1_N = "m:ur:bus1:N", "p.u., Phase Voltage, Real Part"
      m_ur_bus1n = "m:ur:bus1n", "p.u., Phase Voltage, Real Part"
      m_ui_bus1_A = "m:ui:bus1:A", "p.u., Phase Voltage, Imaginary Part"
      m_ui_bus1_B = "m:ui:bus1:B", "p.u., Phase Voltage, Imaginary Part"
      m_ui_bus1_C = "m:ui:bus1:C", "p.u., Phase Voltage, Imaginary Part"
      m_ui_bus1_N = "m:ui:bus1:N", "p.u., Phase Voltage, Imaginary Part"
      m_ui_bus1n = "m:ui:bus1n", "p.u., Phase Voltage, Imaginary Part"
      m_u_bus1_A = "m:u:bus1:A", "p.u., Phase Voltage, Magnitude"
      m_u_bus1_B = "m:u:bus1:B", "p.u., Phase Voltage, Magnitude"
      m_u_bus1_C = "m:u:bus1:C", "p.u., Phase Voltage, Magnitude"
      m_u_bus1_N = "m:u:bus1:N", "p.u., Phase Voltage, Magnitude"
      m_u_bus1n = "m:u:bus1n", "p.u., Phase Voltage, Magnitude"
      m_ir_bus1_A = "m:ir:bus1:A", "p.u., Phase Current, Real Part"
      m_ir_bus1_B = "m:ir:bus1:B", "p.u., Phase Current, Real Part"
      m_ir_bus1_C = "m:ir:bus1:C", "p.u., Phase Current, Real Part"
      m_ir_bus1_N = "m:ir:bus1:N", "p.u., Phase Current, Real Part"
      m_ir_bus1n = "m:ir:bus1n", "p.u., Phase Current, Real Part"
      m_ii_bus1_A = "m:ii:bus1:A", "p.u., Phase Current, Imaginary Part"
      m_ii_bus1_B = "m:ii:bus1:B", "p.u., Phase Current, Imaginary Part"
      m_ii_bus1_C = "m:ii:bus1:C", "p.u., Phase Current, Imaginary Part"
      m_ii_bus1_N = "m:ii:bus1:N", "p.u., Phase Current, Imaginary Part"
      m_ii_bus1n = "m:ii:bus1n", "p.u., Phase Current, Imaginary Part"
      m_i_bus1_A = "m:i:bus1:A", "p.u., Phase Current, Magnitude"
      m_i_bus1_B = "m:i:bus1:B", "p.u., Phase Current, Magnitude"
      m_i_bus1_C = "m:i:bus1:C", "p.u., Phase Current, Magnitude"
      m_i_bus1_N = "m:i:bus1:N", "p.u., Phase Current, Magnitude"
      m_i_bus1n = "m:i:bus1n", "p.u., Phase Current, Magnitude"
      m_phiui_bus1_A = "m:phiui:bus1:A", "deg, Angle between Voltage and Current"
      m_phiui_bus1_B = "m:phiui:bus1:B", "deg, Angle between Voltage and Current"
      m_phiui_bus1_C = "m:phiui:bus1:C", "deg, Angle between Voltage and Current"
      m_phiui_bus1_N = "m:phiui:bus1:N", "deg, Angle between Voltage and Current"
      m_phiui_bus1n = "m:phiui:bus1n", "deg, Angle between Voltage and Current"
      m_phiu0i0_bus1 = "m:phiu0i0:bus1", "deg, Angle between Voltage and Current in zero-sequence system"
      m_phiu0i0_bus1n = "m:phiu0i0:bus1n", "deg, Angle between Voltage and Current in zero-sequence system"
      m_phiu1i1_bus1 = "m:phiu1i1:bus1", "deg, Angle between Voltage and Current in positive sequence system"
      m_phiu1i1_bus1n = "m:phiu1i1:bus1n", "deg, Angle between Voltage and Current in positive sequence system"
      m_phiu2i2_bus1 = "m:phiu2i2:bus1", "deg, Angle between Voltage and Current in negative sequence system"
      m_phiu2i2_bus1n = "m:phiu2i2:bus1n", "deg, Angle between Voltage and Current in negative sequence system"
      m_Tfct_bus1 = "m:Tfct:bus1", "s, Fault Clearing Time"
      m_Tfct_bus1n = "m:Tfct:bus1n", "s, Fault Clearing Time"
      m_TfctPh_bus1_A = "m:TfctPh:bus1:A", "s, Fault Clearing Time"
      m_TfctPh_bus1_B = "m:TfctPh:bus1:B", "s, Fault Clearing Time"
      m_TfctPh_bus1_C = "m:TfctPh:bus1:C", "s, Fault Clearing Time"
      m_TfctPh_bus1_N = "m:TfctPh:bus1:N", "s, Fault Clearing Time"
      m_TfctPh_bus1n = "m:TfctPh:bus1n", "s, Fault Clearing Time"
      m_Brkload_bus1 = "m:Brkload:bus1", "%, Max. Breaker Loading"
      m_Brkload_bus1n = "m:Brkload:bus1n", "%, Max. Breaker Loading"
      m_BrkloadPh_bus1_A = "m:BrkloadPh:bus1:A", "%, Breaker Loading"
      m_BrkloadPh_bus1_B = "m:BrkloadPh:bus1:B", "%, Breaker Loading"
      m_BrkloadPh_bus1_C = "m:BrkloadPh:bus1:C", "%, Breaker Loading"
      m_BrkloadPh_bus1_N = "m:BrkloadPh:bus1:N", "%, Breaker Loading"
      m_BrkloadPh_bus1n = "m:BrkloadPh:bus1n", "%, Breaker Loading"
      n_ur_bus1_A = "n:ur:bus1:A", "p.u., Line-Ground Voltage, Real Part"
      n_ur_bus1_B = "n:ur:bus1:B", "p.u., Line-Ground Voltage, Real Part"
      n_ur_bus1_C = "n:ur:bus1:C", "p.u., Line-Ground Voltage, Real Part"
      n_ur_bus1_N = "n:ur:bus1:N", "p.u., Line-Ground Voltage, Real Part"
      n_ur_bus1n = "n:ur:bus1n", "p.u., Line-Ground Voltage, Real Part"
      n_ui_bus1_A = "n:ui:bus1:A", "p.u., Line-Ground Voltage, Imaginary Part"
      n_ui_bus1_B = "n:ui:bus1:B", "p.u., Line-Ground Voltage, Imaginary Part"
      n_ui_bus1_C = "n:ui:bus1:C", "p.u., Line-Ground Voltage, Imaginary Part"
      n_ui_bus1_N = "n:ui:bus1:N", "p.u., Line-Ground Voltage, Imaginary Part"
      n_ui_bus1n = "n:ui:bus1n", "p.u., Line-Ground Voltage, Imaginary Part"
      n_u_bus1_A = "n:u:bus1:A", "p.u., Line-Ground Voltage, Magnitude"
      n_u_bus1_B = "n:u:bus1:B", "p.u., Line-Ground Voltage, Magnitude"
      n_u_bus1_C = "n:u:bus1:C", "p.u., Line-Ground Voltage, Magnitude"
      n_u_bus1_N = "n:u:bus1:N", "p.u., Line-Ground Voltage, Magnitude"
      n_u_bus1n = "n:u:bus1n", "p.u., Line-Ground Voltage, Magnitude"
      n_upc_bus1_A = "n:upc:bus1:A", "%, Line-Ground Voltage, Magnitude"
      n_upc_bus1_B = "n:upc:bus1:B", "%, Line-Ground Voltage, Magnitude"
      n_upc_bus1_C = "n:upc:bus1:C", "%, Line-Ground Voltage, Magnitude"
      n_upc_bus1_N = "n:upc:bus1:N", "%, Line-Ground Voltage, Magnitude"
      n_upc_bus1n = "n:upc:bus1n", "%, Line-Ground Voltage, Magnitude"
      n_U_bus1_A = "n:U:bus1:A", "kV, Line-Ground Voltage, Magnitude"
      n_U_bus1_B = "n:U:bus1:B", "kV, Line-Ground Voltage, Magnitude"
      n_U_bus1_C = "n:U:bus1:C", "kV, Line-Ground Voltage, Magnitude"
      n_U_bus1_N = "n:U:bus1:N", "kV, Line-Ground Voltage, Magnitude"
      n_U_bus1n = "n:U:bus1n", "kV, Line-Ground Voltage, Magnitude"
      n_phiu_bus1_A = "n:phiu:bus1:A", "deg, Line-Ground Voltage, Angle"
      n_phiu_bus1_B = "n:phiu:bus1:B", "deg, Line-Ground Voltage, Angle"
      n_phiu_bus1_C = "n:phiu:bus1:C", "deg, Line-Ground Voltage, Angle"
      n_phiu_bus1_N = "n:phiu:bus1:N", "deg, Line-Ground Voltage, Angle"
      n_phiu_bus1n = "n:phiu:bus1n", "deg, Line-Ground Voltage, Angle"
      n_phiurel_bus1 = "n:phiurel:bus1", "deg, Voltage, Relative Angle"
      n_phiurel_bus1n = "n:phiurel:bus1n", "deg, Voltage, Relative Angle"
      n_ul_bus1_A = "n:ul:bus1:A", "p.u., Line-Line Voltage, Magnitude"
      n_ul_bus1_B = "n:ul:bus1:B", "p.u., Line-Line Voltage, Magnitude"
      n_ul_bus1_C = "n:ul:bus1:C", "p.u., Line-Line Voltage, Magnitude"
      n_ul_bus1_N = "n:ul:bus1:N", "p.u., Line-Line Voltage, Magnitude"
      n_ul_bus1n = "n:ul:bus1n", "p.u., Line-Line Voltage, Magnitude"
      n_ulpc_bus1_A = "n:ulpc:bus1:A", "%, Line-Line Voltage, Magnitude"
      n_ulpc_bus1_B = "n:ulpc:bus1:B", "%, Line-Line Voltage, Magnitude"
      n_ulpc_bus1_C = "n:ulpc:bus1:C", "%, Line-Line Voltage, Magnitude"
      n_ulpc_bus1_N = "n:ulpc:bus1:N", "%, Line-Line Voltage, Magnitude"
      n_ulpc_bus1n = "n:ulpc:bus1n", "%, Line-Line Voltage, Magnitude"
      n_Ul_bus1_A = "n:Ul:bus1:A", "kV, Line-Line Voltage, Magnitude"
      n_Ul_bus1_B = "n:Ul:bus1:B", "kV, Line-Line Voltage, Magnitude"
      n_Ul_bus1_C = "n:Ul:bus1:C", "kV, Line-Line Voltage, Magnitude"
      n_Ul_bus1_N = "n:Ul:bus1:N", "kV, Line-Line Voltage, Magnitude"
      n_Ul_bus1n = "n:Ul:bus1n", "kV, Line-Line Voltage, Magnitude"
      n_phiul_bus1_A = "n:phiul:bus1:A", "deg, Line-Line Voltage, Angle"
      n_phiul_bus1_B = "n:phiul:bus1:B", "deg, Line-Line Voltage, Angle"
      n_phiul_bus1_C = "n:phiul:bus1:C", "deg, Line-Line Voltage, Angle"
      n_phiul_bus1_N = "n:phiul:bus1:N", "deg, Line-Line Voltage, Angle"
      n_phiul_bus1n = "n:phiul:bus1n", "deg, Line-Line Voltage, Angle"
      n_u0_bus1 = "n:u0:bus1", "p.u., Zero-Sequence Voltage, Magnitude"
      n_u0_bus1n = "n:u0:bus1n", "p.u., Zero-Sequence Voltage, Magnitude"
      n_U0_bus1 = "n:U0:bus1", "kV, Zero-Sequence Voltage, Magnitude"
      n_U0_bus1n = "n:U0:bus1n", "kV, Zero-Sequence Voltage, Magnitude"
      n_U0x3_bus1 = "n:U0x3:bus1", "kV, 3*U0"
      n_U0x3_bus1n = "n:U0x3:bus1n", "kV, 3*U0"
      n_phiu0_bus1 = "n:phiu0:bus1", "deg, Zero-Sequence Voltage, Angle"
      n_phiu0_bus1n = "n:phiu0:bus1n", "deg, Zero-Sequence Voltage, Angle"
      n_u1_bus1 = "n:u1:bus1", "p.u., Positive-Sequence Voltage, Magnitude"
      n_u1_bus1n = "n:u1:bus1n", "p.u., Positive-Sequence Voltage, Magnitude"
      n_u1pc_bus1 = "n:u1pc:bus1", "%, Positive-Sequence Voltage, Magnitude"
      n_u1pc_bus1n = "n:u1pc:bus1n", "%, Positive-Sequence Voltage, Magnitude"
      n_u1r_bus1 = "n:u1r:bus1", "p.u., Positive-Sequence Voltage, Real Part"
      n_u1r_bus1n = "n:u1r:bus1n", "p.u., Positive-Sequence Voltage, Real Part"
      n_u1i_bus1 = "n:u1i:bus1", "p.u., Positive-Sequence Voltage, Imaginary Part"
      n_u1i_bus1n = "n:u1i:bus1n", "p.u., Positive-Sequence Voltage, Imaginary Part"
      n_U1_bus1 = "n:U1:bus1", "kV, Line-Ground Positive-Sequence Voltage, Magnitude"
      n_U1_bus1n = "n:U1:bus1n", "kV, Line-Ground Positive-Sequence Voltage, Magnitude"
      n_phiu1_bus1 = "n:phiu1:bus1", "deg, Positive-Sequence Voltage, Angle"
      n_phiu1_bus1n = "n:phiu1:bus1n", "deg, Positive-Sequence Voltage, Angle"
      n_u2_bus1 = "n:u2:bus1", "p.u., Negative-Sequence Voltage, Magnitude"
      n_u2_bus1n = "n:u2:bus1n", "p.u., Negative-Sequence Voltage, Magnitude"
      n_U2_bus1 = "n:U2:bus1", "kV, Line-Ground Negative-Sequence Voltage, Magnitude"
      n_U2_bus1n = "n:U2:bus1n", "kV, Line-Ground Negative-Sequence Voltage, Magnitude"
      n_phiu2_bus1 = "n:phiu2:bus1", "deg, Negative-Sequence Voltage, Angle"
      n_phiu2_bus1n = "n:phiu2:bus1n", "deg, Negative-Sequence Voltage, Angle"
      n_U1l_bus1 = "n:U1l:bus1", "kV, Line-Line Positive-Sequence Voltage, Magnitude"
      n_U1l_bus1n = "n:U1l:bus1n", "kV, Line-Line Positive-Sequence Voltage, Magnitude"
      n_U2l_bus1 = "n:U2l:bus1", "kV, Line-Line Negative-Sequence Voltage, Magnitude"
      n_U2l_bus1n = "n:U2l:bus1n", "kV, Line-Line Negative-Sequence Voltage, Magnitude"
      n_uln_bus1_A = "n:uln:bus1:A", "p.u., Line-Neutral Voltage, Magnitude"
      n_uln_bus1_B = "n:uln:bus1:B", "p.u., Line-Neutral Voltage, Magnitude"
      n_uln_bus1_C = "n:uln:bus1:C", "p.u., Line-Neutral Voltage, Magnitude"
      n_uln_bus1_N = "n:uln:bus1:N", "p.u., Line-Neutral Voltage, Magnitude"
      n_uln_bus1n = "n:uln:bus1n", "p.u., Line-Neutral Voltage, Magnitude"
      n_Uln_bus1_A = "n:Uln:bus1:A", "kV, Line-Neutral Voltage, Magnitude"
      n_Uln_bus1_B = "n:Uln:bus1:B", "kV, Line-Neutral Voltage, Magnitude"
      n_Uln_bus1_C = "n:Uln:bus1:C", "kV, Line-Neutral Voltage, Magnitude"
      n_Uln_bus1_N = "n:Uln:bus1:N", "kV, Line-Neutral Voltage, Magnitude"
      n_Uln_bus1n = "n:Uln:bus1n", "kV, Line-Neutral Voltage, Magnitude"
      n_phiuln_bus1_A = "n:phiuln:bus1:A", "deg, Line-Neutral Voltage, Angle"
      n_phiuln_bus1_B = "n:phiuln:bus1:B", "deg, Line-Neutral Voltage, Angle"
      n_phiuln_bus1_C = "n:phiuln:bus1:C", "deg, Line-Neutral Voltage, Angle"
      n_phiuln_bus1_N = "n:phiuln:bus1:N", "deg, Line-Neutral Voltage, Angle"
      n_phiuln_bus1n = "n:phiuln:bus1n", "deg, Line-Neutral Voltage, Angle"
      n_un_bus1 = "n:un:bus1", "p.u., Neutral-Ground Voltage, Magnitude"
      n_un_bus1n = "n:un:bus1n", "p.u., Neutral-Ground Voltage, Magnitude"
      n_Un_bus1 = "n:Un:bus1", "kV, Neutral-Ground Voltage, Magnitude"
      n_Un_bus1n = "n:Un:bus1n", "kV, Neutral-Ground Voltage, Magnitude"
      n_phiun_bus1 = "n:phiun:bus1", "deg, Neutral-Ground Voltage, Angle"
      n_phiun_bus1n = "n:phiun:bus1n", "deg, Neutral-Ground Voltage, Angle"
      n_upht_bus1_A = "n:upht:bus1:A", "p.u., Phase Technology dependent Voltage, Magnitude"
      n_upht_bus1_B = "n:upht:bus1:B", "p.u., Phase Technology dependent Voltage, Magnitude"
      n_upht_bus1_C = "n:upht:bus1:C", "p.u., Phase Technology dependent Voltage, Magnitude"
      n_upht_bus1_N = "n:upht:bus1:N", "p.u., Phase Technology dependent Voltage, Magnitude"
      n_upht_bus1n = "n:upht:bus1n", "p.u., Phase Technology dependent Voltage, Magnitude"
      n_Upht_bus1_A = "n:Upht:bus1:A", "kV, Phase Technology dependent Voltage, Magnitude"
      n_Upht_bus1_B = "n:Upht:bus1:B", "kV, Phase Technology dependent Voltage, Magnitude"
      n_Upht_bus1_C = "n:Upht:bus1:C", "kV, Phase Technology dependent Voltage, Magnitude"
      n_Upht_bus1_N = "n:Upht:bus1:N", "kV, Phase Technology dependent Voltage, Magnitude"
      n_Upht_bus1n = "n:Upht:bus1n", "kV, Phase Technology dependent Voltage, Magnitude"
      n_um_bus1 = "n:um:bus1", "p.u., Average-Voltage, Magnitude"
      n_um_bus1n = "n:um:bus1n", "p.u., Average-Voltage, Magnitude"
      n_Um_bus1 = "n:Um:bus1", "kV, Average-Voltage, Magnitude"
      n_Um_bus1n = "n:Um:bus1n", "kV, Average-Voltage, Magnitude"
      n_ishcr_bus1_A = "n:ishcr:bus1:A", "p.u., Short-Circuit Current, real"
      n_ishcr_bus1_B = "n:ishcr:bus1:B", "p.u., Short-Circuit Current, real"
      n_ishcr_bus1_C = "n:ishcr:bus1:C", "p.u., Short-Circuit Current, real"
      n_ishcr_bus1_N = "n:ishcr:bus1:N", "p.u., Short-Circuit Current, real"
      n_ishcr_bus1n = "n:ishcr:bus1n", "p.u., Short-Circuit Current, real"
      n_ishci_bus1_A = "n:ishci:bus1:A", "p.u., Short-Circuit Current, imag"
      n_ishci_bus1_B = "n:ishci:bus1:B", "p.u., Short-Circuit Current, imag"
      n_ishci_bus1_C = "n:ishci:bus1:C", "p.u., Short-Circuit Current, imag"
      n_ishci_bus1_N = "n:ishci:bus1:N", "p.u., Short-Circuit Current, imag"
      n_ishci_bus1n = "n:ishci:bus1n", "p.u., Short-Circuit Current, imag"
      n_ishc_bus1_A = "n:ishc:bus1:A", "p.u., Short-Circuit Current, magnitude"
      n_ishc_bus1_B = "n:ishc:bus1:B", "p.u., Short-Circuit Current, magnitude"
      n_ishc_bus1_C = "n:ishc:bus1:C", "p.u., Short-Circuit Current, magnitude"
      n_ishc_bus1_N = "n:ishc:bus1:N", "p.u., Short-Circuit Current, magnitude"
      n_ishc_bus1n = "n:ishc:bus1n", "p.u., Short-Circuit Current, magnitude"
      n_Ishc_bus1_A = "n:Ishc:bus1:A", "kA, Short-Circuit Current, magnitude"
      n_Ishc_bus1_B = "n:Ishc:bus1:B", "kA, Short-Circuit Current, magnitude"
      n_Ishc_bus1_C = "n:Ishc:bus1:C", "kA, Short-Circuit Current, magnitude"
      n_Ishc_bus1_N = "n:Ishc:bus1:N", "kA, Short-Circuit Current, magnitude"
      n_Ishc_bus1n = "n:Ishc:bus1n", "kA, Short-Circuit Current, magnitude"
      n_inshc_bus1 = "n:inshc:bus1", "p.u., Neutral Short-Circuit Current, magnitude"
      n_inshc_bus1n = "n:inshc:bus1n", "p.u., Neutral Short-Circuit Current, magnitude"
      n_Inshc_bus1 = "n:Inshc:bus1", "kA, Neutral Short-Circuit Current, magnitude"
      n_Inshc_bus1n = "n:Inshc:bus1n", "kA, Neutral Short-Circuit Current, magnitude"
      n_fe_bus1 = "n:fe:bus1", "p.u., Electrical Frequency"
      n_fe_bus1n = "n:fe:bus1n", "p.u., Electrical Frequency"
      n_frdev_bus1 = "n:frdev:bus1", "Hz, average frequency"
      n_frdev_bus1n = "n:frdev:bus1n", "Hz, average frequency"
      n_dfehz_bus1 = "n:dfehz:bus1", "Hz, Deviation of the El. Frequency"
      n_dfehz_bus1n = "n:dfehz:bus1n", "Hz, Deviation of the El. Frequency"
      n_dfedt_bus1 = "n:dfedt:bus1", "p.u./s, Derivative of El. Frequency"
      n_dfedt_bus1n = "n:dfedt:bus1n", "p.u./s, Derivative of El. Frequency"
      n_fehz_bus1 = "n:fehz:bus1", "Hz, Electrical Frequency"
      n_fehz_bus1n = "n:fehz:bus1n", "Hz, Electrical Frequency"
      n_frnom_bus1 = "n:frnom:bus1", "Hz, Nominal Frequency"
      n_frnom_bus1n = "n:frnom:bus1n", "Hz, Nominal Frequency"
      s_ve = "s:ve", "p.u., IN, Excitation Voltage, continuous, double"
      s_pt = "s:pt", "p.u., IN, Turbine Power, continuous, double"
      s_xmdm = "s:xmdm", "p.u., IN, Torque Input, continuous, double"
      s_dw = "s:dw", "p.u., IN, dw, continuous, double"
      s_fref = "s:fref", "p.u., IN, Reference Frequency, continuous, double"
      s_freflocal = "s:freflocal", "p.u., IN, Local Reference Frequency, continuous, double"
      s_psid = "s:psid", "OUT, Stator Flux, d-xis, continuous, double"
      s_psiq = "s:psiq", "p.u., OUT, Stator Flux, q-axis, continuous, double"
      s_outofstep = "s:outofstep", "OUT, Out of Step (pole slip) continuous, double"
      s_xme = "s:xme", "p.u., OUT, Electrical Torque, continuous, double"
      s_xmt = "s:xmt", "p.u., OUT, Mechanical Torque, continuous, double"
      s_xspeed = "s:xspeed", "p.u., OUT, Speed, continuous, double"
      s_ie = "s:ie", "p.u., OUT, Excitation Current (non-reciprocal p.u. system) continuous, double"
      s_pgt = "s:pgt", "p.u., OUT, Electrical Power, continuous, double"
      s_xphi = "s:xphi", "rad, OUT, Rotor position d-axis referred to the reference bus voltage, continuous, double"
      s_fe = "s:fe", "p.u., OUT, Frequency Output, continuous, double"
      s_ut = "s:ut", "p.u., OUT, Terminal voltage, continuous, double"
      s_utr = "s:utr", "p.u., OUT, Terminal voltage, real part, continuous, double"
      s_uti = "s:uti", "p.u., OUT, Terminal voltage, imaginary part, continuous, double"
      s_cur1 = "s:cur1", "p.u., OUT, Positive-sequence current, magnitude, continuous, double"
      s_cur1r = "s:cur1r", "p.u., OUT, Positive-sequence current, real part, continuous, double"
      s_cur1i = "s:cur1i", "p.u., OUT, Positive-sequence current, imaginary part, continuous, double"
      s_P1 = "s:P1", "MW, OUT, Positive-sequence, active power continuous, double"
      s_Q1 = "s:Q1", "Mvar, OUT, Positive-sequence, reactive power, continuous, double"
      s_fipol = "s:fipol", "deg, OUT, Internal rotor angle, continuous, double"
      s_psi1d = "s:psi1d", "p.u., STATE, Flux in 1d-damper winding, continuous, double"
      s_psifd = "s:psifd", "p.u., STATE, Excitation Flux continuous, double"
      s_psi1q = "s:psi1q", "p.u., STATE, Flux in 1q-damper winding, continuous, double"
      s_psi2q = "s:psi2q", "p.u., STATE, Flux in 2q-damper winding, continuous, double"
      s_speed = "s:speed", "p.u., STATE, Speed, continuous, double"
      s_phi = "s:phi", "rad, STATE, Rotor position d-axis referred to the reference bus voltage, continuous, double"
      s_psi1d_dt = "s:psi1d:dt", "p.u./s, d/dt, Flux in 1d-damper winding, derivative, continuous, double"
      s_psifd_dt = "s:psifd:dt", "p.u./s, d/dt, Excitation Flux, derivative, continuous, double"
      s_psi1q_dt = "s:psi1q:dt", "p.u./s, d/dt, Flux in 1q-damper winding, derivative, continuous, double"
      s_psi2q_dt = "s:psi2q:dt", "p.u./s, d/dt, Flux in 2q-damper winding, derivative, continuous, double"
      s_speed_dt = "s:speed:dt", "p.u./s, d/dt, Speed, derivative, continuous, double"
      s_phi_dt = "s:phi:dt", "rad/s, d/dt, Rotor position d-axis referred to the reference bus voltage, derivative continuous, double"
      c_ud = "c:ud", "p.u., Voltage, d-axis"
      c_uq = "c:uq", "p.u., Voltage, q-axis"
      c_id = "c:id", "p.u., Stator Current, d-axis"
      c_iq = "c:iq", "p.u., Stator Current, q-axis"
      c_psim = "c:psim", "p.u., Magnetisation Flux"
      c_psidss = "c:psidss", "p.u., Subtransient Flux, d-axis"
      c_psiqss = "c:psiqss", "p.u., Subtransient Flux, q-axis"
      c_ussd = "c:ussd", "p.u., Subtransient Voltage, d-axis"
      c_ussq = "c:ussq", "p.u., Subtransient Voltage, q-axis"
      c_xmem = "c:xmem", "p.u., Electrical Torque"
      c_xme1 = "c:xme1", "p.u., Electrical Torque, Positive Sequence"
      c_xme2 = "c:xme2", "p.u., Electrical Torque, Negative Sequence"
      c_addmt = "c:addmt", "p.u., Additional Torque"
      c_xmdpe = "c:xmdpe", "p.u., Damping Torque based on power"
      c_xmdkd = "c:xmdkd", "p.u., Damping Torque"
      c_ifd = "c:ifd", "p.u., Current in excitation winding"
      c_i1d = "c:i1d", "p.u., Current in 1d-damper winding"
      c_i1q = "c:i1q", "p.u., Current in 1q-damper winding"
      c_i2q = "c:i2q", "p.u., Current in 2q-damper winding"
      c_iD = "c:iD", "p.u., Current in 1d-damper winding (non-reciprocal p.u. system)"
      c_ix = "c:ix", "p.u., Current in 2q-damper winding (non-reciprocal p.u. system)"
      c_iQ = "c:iQ", "p.u., Current in 1q-damper winding (non-reciprocal p.u. system)"
      c_psie = "c:psie", "p.u., Excitation Flux (non-reciprocal p.u. system)"
      c_psiD = "c:psiD", "p.u., Flux in 1d-damper winding (non-reciprocal p.u. system)"
      c_psiQ = "c:psiQ", "p.u., Flux in 1q-damper winding (non-reciprocal p.u. system)"
      c_psix = "c:psix", "p.u., Flux in 2q-damper winding (non-reciprocal p.u. system)"
      c_fi = "c:fi", "deg, Rotor position d-axis referred to the reference bus voltage"
      c_firot = "c:firot", "deg, External rotor angle"
      c_firel = "c:firel", "deg, Rotor angle referred to the reference machine rotor angle"
      c_dfrotx = "c:dfrotx", "deg, Maximum rotor angle difference"
      c_dfrotxM1 = "c:dfrotxM1", "Machine 1 used to obtain dfrotx"
      c_dfrotxM2 = "c:dfrotxM2", "Machine 2 used to obtain dfrotx"
      c_dorhz = "c:dorhz", "Hz, Speed Deviation"
      c_Ptspin = "c:Ptspin", "MW, Total Turbine Power of Area"
      c_Pnomspin = "c:Pnomspin", "MW, Total Nominal Power of Spinning Machines in Area"
      c_Presspin = "c:Presspin", "MW, Spinning Reserve of Area"
      c_Tagav = "c:Tagav", "s, System Inertia of Area"
      c_Tag = "c:Tag", "s, Acceleration Time Constant"
      c_speedinit = "c:speedinit", "p.u., Initial speed"
      c_xadu = "c:xadu", "p.u., Unsaturated mutual reactance, d-axis"
      c_xaqu = "c:xaqu", "p.u., Unsaturated mutual reactance, q-axis"
      c_xad = "c:xad", "p.u., Saturated mutual reactance, d-axis"
      c_xaq = "c:xaq", "p.u., Saturated mutual reactance q-axis"
      c_satd = "c:satd", "p.u., Saturation factor, d-axis"
      c_satq = "c:satq", "p.u., Saturation factor, q-axis"
      c_rfd = "c:rfd", "p.u., Resistance of excitation winding"
      c_xfd = "c:xfd", "p.u., Reactance of excitation winding"
      c_r1d = "c:r1d", "p.u., Resistance of 1d-damper winding"
      c_x1d = "c:x1d", "p.u., Reactance of 1d-damper winding"
      c_r1q = "c:r1q", "p.u., Resistance of 1q-damper winding"
      c_x1q = "c:x1q", "p.u., Reactance of 1q-damper winding"
      c_r2q = "c:r2q", "p.u., Resistance of 2q-damper winding"
      c_x2q = "c:x2q", "p.u., Reactance of 2q-damper winding"
      c_xd = "c:xd", "p.u., Synchronous Reactance, d-axis"
      c_xq = "c:xq", "p.u., Synchronous Reactance, q-axis"
      c_xdss = "c:xdss", "p.u., Subtransient Reactance, d-axis"
      c_xqss = "c:xqss", "p.u., Subtransient Reactance, q-axis"
      c_r0 = "c:r0", "p.u., Zero Sequence Resistance"
      c_x0 = "c:x0", "p.u., Zero Sequence Reactance"
      c_rearth = "c:rearth", "p.u., Grounding Resistance"
      c_xearth = "c:xearth", "p.u., Grounding Reactance"
      c_r2 = "c:r2", "p.u., Negative Sequence Resistance"
      c_x2 = "c:x2", "p.u., Negative Sequence Reactance"
      c_sgnn = "c:sgnn", "MVA, Total rated apparent power (ngnum*sgn)"
      c_c2 = "c:c2", "Voltage Coefficient (uknom/ugn)"
      c_cosn = "c:cosn", "Nominal-Power factor"
      c_hpi = "c:hpi", "rad/s, Nominal-Angular-Frequency"
      c_Qmin = "c:Qmin", "Mvar, Lower limit of reactive power"
      c_Qmax = "c:Qmax", "Mvar, Upper limit of reactive power"
      c_p_mag = "c:p_mag", "Participation, Magnitude"
      c_p_phi = "c:p_phi", "deg, Participation, Angle"
      c_p_mags = "c:p_mags", "Participation, Magnitude (signed)"
      c_lEVec_mag = "c:lEVec_mag", "Controllability, Magnitude"
      c_lEVec_phi = "c:lEVec_phi", "deg, Controllability, Angle"
      c_lEVec_mags = "c:lEVec_mags", "Controllability, Magnitude (signed)"
      c_rEVec_mag = "c:rEVec_mag", "Observability, Magnitude"
      c_rEVec_phi = "c:rEVec_phi", "deg, Observability, Angle"
      c_rEVec_mags = "c:rEVec_mags", "Observability, Magnitude (signed)"
      c_fe_bus = "c:fe_bus", "El. frequency"
      c_loading = "c:loading", "%, Loading"
      c_slipreset = "c:slipreset", "Reset out of step"
      c_Gm = "c:Gm", "Conversion Factor (=xadu/rfd)"
      c_ve_rated = "c:ve_rated", "p.u., Rated Excitation Voltage"
      c_Pmaxspin = "c:Pmaxspin", "MW, Total Maximum Power of Spinning Machines in Area"
      c_Pmax = "c:Pmax", "MW, Upper limit of active power"
      c_Td0s = "c:Td0s", "s, Td0'"
      c_mdmlp = "c:mdmlp", "p.u., Proportional Factor"
      c_mdmex = "c:mdmex", "Exponent"
      e_iAstabint = "e:iAstabint", "A-stable integration algorithm"
      e_efdBaseRatio = "e:efdBaseRatio", "Efd base ratio"
      e_ifdBaseType = "e:ifdBaseType", "Excitation base mode"
      e_c_pmdm = "e:c_pmdm", "Mdm"
      e_mdmlp = "e:mdmlp", "p.u., Proportional Factor"
      e_mdmex = "e:mdmex", "Exponent"
      e_Jme = "e:Jme", "kgm^2, Moment of inertia"
      e_gratio = "e:gratio", "Gear ratio"
      e_ctag = "e:ctag", "s, Acceleration time const. from type"
      e_ctagtot = "e:ctagtot", "s, Total acceleration time const."
      e_i_avr = "e:i_avr", "Use integrated AVR for motor starting"
      e_rf_st = "e:rf_st", "p.u., Starting field resistance"
      e_speed_th = "e:speed_th", "p.u., Trigger excitation at speed"
      e_ve_const = "e:ve_const", "p.u., Constant field voltage"
      e_ve_rated = "e:ve_rated", "p.u., Rated field voltage"
      t_i_exactConv = "t:i_exactConv", "Exact conversion of time constants"
      t_i_xme2 = "t:i_xme2", "Consider negative sequence torque"
      t_i_speedVar = "t:i_speedVar", "Effect of speed variation"
      t_i_extModel = "t:i_extModel", "Model"
      t_model_inp = "t:model_inp", "Model"
      t_iopt_data = "t:iopt_data", "Input parameters"
      t_iner_inp = "t:iner_inp", "Input mode"
      t_tag = "t:tag", "s, Acceleration time const. Tag (rated to Pgn)"
      t_tags = "t:tags", "s, Acceleration time const. Tag (rated to Sgn)"
      t_h = "t:h", "s, Inertia constant H (rated to Sgn)"
      t_hpn = "t:hpn", "s, Inertia constant H (rated to Pgn)"
      t_J = "t:J", "kgm^2, Moment of inertia J"
      t_polepairs = "t:polepairs", "No. of pole pairs"
      t_frequ = "t:frequ", "Hz, Nominal frequency"
      t_rstr = "t:rstr", "p.u., rstr"
      t_xl = "t:xl", "p.u., xl"
      t_i_xl = "t:i_xl", "xl is current dependent"
      t_iturbo = "t:iturbo", "Rotor type"
      t_tds = "t:tds", "s, Td'"
      t_tqs = "t:tqs", "s, Tq'"
      t_tds0 = "t:tds0", "s, Td0'"
      t_tqs0 = "t:tqs0", "s, Tq0'"
      t_tdss = "t:tdss", "s, Td''"
      t_tqss = "t:tqss", "s, Tq''"
      t_tdss0 = "t:tdss0", "s, Td0''"
      t_tqss0 = "t:tqss0", "s, Tq0''"
      t_xd = "t:xd", "p.u., xd"
      t_xq = "t:xq", "p.u., xq"
      t_xrl = "t:xrl", "p.u., xrld"
      t_xrlq = "t:xrlq", "p.u., xrlq"
      t_xds = "t:xds", "p.u., xd'"
      t_xqs = "t:xqs", "p.u., xq'"
      t_xdss = "t:xdss", "p.u., xd''"
      t_xqss = "t:xqss", "p.u., xq''"
      t_xstr = "t:xstr", "p.u., xstr"
      t_x1d = "t:x1d", "p.u., x1d"
      t_r1d = "t:r1d", "p.u., r1d"
      t_xfd = "t:xfd", "p.u., xfd"
      t_rfd = "t:rfd", "p.u., rfd"
      t_x1q = "t:x1q", "p.u., x1q"
      t_r1q = "t:r1q", "p.u., r1q"
      t_x2q = "t:x2q", "p.u., x2q"
      t_r2q = "t:r2q", "p.u., r2q"
      t_xrl2d = "t:xrl2d", "p.u., xrl2d"
      t_x2d = "t:x2d", "p.u., x2d"
      t_r2d = "t:r2d", "p.u., r2d"
      t_t1d = "t:t1d", "s, t1d"
      t_t2d = "t:t2d", "s, t2d"
      t_t3d = "t:t3d", "s, t3d"
      t_t4d = "t:t4d", "s, t4d"
      t_t5d = "t:t5d", "s, t5d"
      t_t6d = "t:t6d", "s, t6d"
      t_t7d = "t:t7d", "s, t7d"
      t_t8d = "t:t8d", "s, t8d"
      t_xad = "t:xad", "p.u., xad"
      t_xaq = "t:xaq", "p.u., xaq"
      t_x3q = "t:x3q", "p.u., x3q"
      t_r3q = "t:r3q", "p.u., r3q"
      t_t1q = "t:t1q", "s, t1q"
      t_t2q = "t:t2q", "s, t2q"
      t_t3q = "t:t3q", "s, t3q"
      t_t4q = "t:t4q", "s, t4q"
      t_t5q = "t:t5q", "s, t5q"
      t_t6q = "t:t6q", "s, t6q"
      t_x1dc = "t:x1dc", "p.u., x1dc"
      t_x1de = "t:x1de", "p.u., x1de"
      t_r1de = "t:r1de", "p.u., r1de"
      t_xfde = "t:xfde", "p.u., xfde"
      t_rfde = "t:rfde", "p.u., rfde"
      t_x0sy = "t:x0sy", "p.u., Reactance x0"
      t_r0sy = "t:r0sy", "p.u., Resistance r0"
      t_x1qc = "t:x1qc", "p.u., x1qc"
      t_x1qe = "t:x1qe", "p.u., x1qe"
      t_r1qe = "t:r1qe", "p.u., r1qe"
      t_psim = "t:psim", "p.u., Rotor flux"
      t_slipmin = "t:slipmin", "%, Minimum slip"
      t_curTab = "t:curTab", "p.u., Current"
      t_xlTab = "t:xlTab", "p.u., xl"
      t_depSmoothFac = "t:depSmoothFac", "%, Dependency curve smoothing factor"
      t_isat = "t:isat", "Main flux saturation"
      t_isatSalient = "t:isatSalient", "Saturation"
      t_sg10 = "t:sg10", "p.u., SG10"
      t_sg12 = "t:sg12", "p.u., SG12"
      t_sg10q = "t:sg10q", "p.u., SG10 q-axis"
      t_sg12q = "t:sg12q", "p.u., SG12 q-axis"
      t_satv = "t:satv", "p.u., Term. voltage"
      t_satse = "t:satse", "p.u., SG(u)"
      t_ifd = "t:ifd", "%, ie"
      t_satvq = "t:satvq", "p.u., Term. voltage"
      t_satseq = "t:satseq", "p.u., SG(u)"
      t_ifdq = "t:ifdq", "%, ie"
      t_smoothFac = "t:smoothFac", "%, Smoothing factor"
      t_dpu = "t:dpu", "Turbine shaft friction torque coefficient"
      t_dkd = "t:dkd", "Damping torque coefficient"
      t_dpe = "t:dpe", "Damping torque coefficient based on power"
      t_x2sy = "t:x2sy", "p.u., Reactance x2"
      t_r2sy = "t:r2sy", "p.u., Resistance r2"
      t_i_speedD = "t:i_speedD", "Speed deviation based on"
      r_c_pmdm = "r:c_pmdm", "Mdm"

    class ElmTerm(Enum):
      _init_ = 'value __doc__'

      m_ur_A = "m:ur:A", "p.u., Line-Ground Voltage, Real Part"
      m_ur_B = "m:ur:B", "p.u., Line-Ground Voltage, Real Part"
      m_ur_C = "m:ur:C", "p.u., Line-Ground Voltage, Real Part"
      m_ui_A = "m:ui:A", "p.u., Line-Ground Voltage, Imaginary Part"
      m_ui_B = "m:ui:B", "p.u., Line-Ground Voltage, Imaginary Part"
      m_ui_C = "m:ui:C", "p.u., Line-Ground Voltage, Imaginary Part"
      m_u_A = "m:u:A", "p.u., Line-Ground Voltage, Magnitude"
      m_u_B = "m:u:B", "p.u., Line-Ground Voltage, Magnitude"
      m_u_C = "m:u:C", "p.u., Line-Ground Voltage, Magnitude"
      m_upc_A = "m:upc:A", "%, Line-Ground Voltage, Magnitude"
      m_upc_B = "m:upc:B", "%, Line-Ground Voltage, Magnitude"
      m_upc_C = "m:upc:C", "%, Line-Ground Voltage, Magnitude"
      m_U_A = "m:U:A", "kV, Line-Ground Voltage, Magnitude"
      m_U_B = "m:U:B", "kV, Line-Ground Voltage, Magnitude"
      m_U_C = "m:U:C", "kV, Line-Ground Voltage, Magnitude"
      m_phiu_A = "m:phiu:A", "deg, Line-Ground Voltage, Angle"
      m_phiu_B = "m:phiu:B", "deg, Line-Ground Voltage, Angle"
      m_phiu_C = "m:phiu:C", "deg, Line-Ground Voltage, Angle"
      m_phiurel = "m:phiurel", "deg, Voltage, Relative Angle"
      m_ul_A = "m:ul:A", "p.u., Line-Line Voltage, Magnitude"
      m_ul_B = "m:ul:B", "p.u., Line-Line Voltage, Magnitude"
      m_ul_C = "m:ul:C", "p.u., Line-Line Voltage, Magnitude"
      m_ulpc_A = "m:ulpc:A", "%, Line-Line Voltage, Magnitude"
      m_ulpc_B = "m:ulpc:B", "%, Line-Line Voltage, Magnitude"
      m_ulpc_C = "m:ulpc:C", "%, Line-Line Voltage, Magnitude"
      m_Ul_A = "m:Ul:A", "kV, Line-Line Voltage, Magnitude"
      m_Ul_B = "m:Ul:B", "kV, Line-Line Voltage, Magnitude"
      m_Ul_C = "m:Ul:C", "kV, Line-Line Voltage, Magnitude"
      m_phiul_A = "m:phiul:A", "deg, Line-Line Voltage, Angle"
      m_phiul_B = "m:phiul:B", "deg, Line-Line Voltage, Angle"
      m_phiul_C = "m:phiul:C", "deg, Line-Line Voltage, Angle"
      m_u0 = "m:u0", "p.u., Zero-Sequence Voltage, Magnitude"
      m_U0 = "m:U0", "kV, Zero-Sequence Voltage, Magnitude"
      m_U0x3 = "m:U0x3", "kV, 3*U0"
      m_phiu0 = "m:phiu0", "deg, Zero-Sequence Voltage, Angle"
      m_u1 = "m:u1", "p.u., Positive-Sequence Voltage, Magnitude"
      m_u1pc = "m:u1pc", "%, Positive-Sequence Voltage, Magnitude"
      m_u1r = "m:u1r", "p.u., Positive-Sequence Voltage, Real Part"
      m_u1i = "m:u1i", "p.u., Positive-Sequence Voltage, Imaginary Part"
      m_U1 = "m:U1", "kV, Line-Ground Positive-Sequence Voltage, Magnitude"
      m_phiu1 = "m:phiu1", "deg, Positive-Sequence Voltage, Angle"
      m_u2 = "m:u2", "p.u., Negative-Sequence Voltage, Magnitude"
      m_U2 = "m:U2", "kV, Line-Ground Negative-Sequence Voltage, Magnitude"
      m_phiu2 = "m:phiu2", "deg, Negative-Sequence Voltage, Angle"
      m_U1l = "m:U1l", "kV, Line-Line Positive-Sequence Voltage, Magnitude"
      m_U2l = "m:U2l", "kV, Line-Line Negative-Sequence Voltage, Magnitude"
      m_uln_A = "m:uln:A", "p.u., Line-Neutral Voltage, Magnitude"
      m_uln_B = "m:uln:B", "p.u., Line-Neutral Voltage, Magnitude"
      m_uln_C = "m:uln:C", "p.u., Line-Neutral Voltage, Magnitude"
      m_Uln_A = "m:Uln:A", "kV, Line-Neutral Voltage, Magnitude"
      m_Uln_B = "m:Uln:B", "kV, Line-Neutral Voltage, Magnitude"
      m_Uln_C = "m:Uln:C", "kV, Line-Neutral Voltage, Magnitude"
      m_phiuln_A = "m:phiuln:A", "deg, Line-Neutral Voltage, Angle"
      m_phiuln_B = "m:phiuln:B", "deg, Line-Neutral Voltage, Angle"
      m_phiuln_C = "m:phiuln:C", "deg, Line-Neutral Voltage, Angle"
      m_un = "m:un", "p.u., Neutral-Ground Voltage, Magnitude"
      m_Un = "m:Un", "kV, Neutral-Ground Voltage, Magnitude"
      m_phiun = "m:phiun", "deg, Neutral-Ground Voltage, Angle"
      m_upht_A = "m:upht:A", "p.u., Phase Technology dependent Voltage, Magnitude"
      m_upht_B = "m:upht:B", "p.u., Phase Technology dependent Voltage, Magnitude"
      m_upht_C = "m:upht:C", "p.u., Phase Technology dependent Voltage, Magnitude"
      m_Upht_A = "m:Upht:A", "kV, Phase Technology dependent Voltage, Magnitude"
      m_Upht_B = "m:Upht:B", "kV, Phase Technology dependent Voltage, Magnitude"
      m_Upht_C = "m:Upht:C", "kV, Phase Technology dependent Voltage, Magnitude"
      m_um = "m:um", "p.u., Average-Voltage, Magnitude"
      m_Um = "m:Um", "kV, Average-Voltage, Magnitude"
      m_ishcr_A = "m:ishcr:A", "p.u., Short-Circuit Current, real"
      m_ishcr_B = "m:ishcr:B", "p.u., Short-Circuit Current, real"
      m_ishcr_C = "m:ishcr:C", "p.u., Short-Circuit Current, real"
      m_ishci_A = "m:ishci:A", "p.u., Short-Circuit Current, imag"
      m_ishci_B = "m:ishci:B", "p.u., Short-Circuit Current, imag"
      m_ishci_C = "m:ishci:C", "p.u., Short-Circuit Current, imag"
      m_ishc_A = "m:ishc:A", "p.u., Short-Circuit Current, magnitude"
      m_ishc_B = "m:ishc:B", "p.u., Short-Circuit Current, magnitude"
      m_ishc_C = "m:ishc:C", "p.u., Short-Circuit Current, magnitude"
      m_Ishc_A = "m:Ishc:A", "kA, Short-Circuit Current, magnitude"
      m_Ishc_B = "m:Ishc:B", "kA, Short-Circuit Current, magnitude"
      m_Ishc_C = "m:Ishc:C", "kA, Short-Circuit Current, magnitude"
      m_inshc = "m:inshc", "p.u., Neutral Short-Circuit Current, magnitude"
      m_Inshc = "m:Inshc", "kA, Neutral Short-Circuit Current, magnitude"
      m_fe = "m:fe", "p.u., Electrical Frequency"
      m_frdev = "m:frdev", "Hz, average frequency"
      m_dfehz = "m:dfehz", "Hz, Deviation of the El. Frequency"
      m_dfedt = "m:dfedt", "p.u./s, Derivative of El. Frequency"
      m_fehz = "m:fehz", "Hz, Electrical Frequency"
      m_frnom = "m:frnom", "Hz, Nominal Frequency"

    class ElmTr2(Enum):
      _init_ = 'value __doc__'

      m_u0_bushv = "m:u0:bushv", "p.u., Zero-Sequence-Voltage, Magnitude"
      m_u0_buslv = "m:u0:buslv", "p.u., Zero-Sequence-Voltage, Magnitude"
      m_u0_bushvn = "m:u0:bushvn", "p.u., Zero-Sequence-Voltage, Magnitude"
      m_u0_buslvn = "m:u0:buslvn", "p.u., Zero-Sequence-Voltage, Magnitude"
      m_phiu0_bushv = "m:phiu0:bushv", "deg, Zero-Sequence-Voltage, Angle"
      m_phiu0_buslv = "m:phiu0:buslv", "deg, Zero-Sequence-Voltage, Angle"
      m_phiu0_bushvn = "m:phiu0:bushvn", "deg, Zero-Sequence-Voltage, Angle"
      m_phiu0_buslvn = "m:phiu0:buslvn", "deg, Zero-Sequence-Voltage, Angle"
      m_u1r_bushv = "m:u1r:bushv", "p.u., Positive-Sequence-Voltage, Real Part"
      m_u1r_buslv = "m:u1r:buslv", "p.u., Positive-Sequence-Voltage, Real Part"
      m_u1r_bushvn = "m:u1r:bushvn", "p.u., Positive-Sequence-Voltage, Real Part"
      m_u1r_buslvn = "m:u1r:buslvn", "p.u., Positive-Sequence-Voltage, Real Part"
      m_u1i_bushv = "m:u1i:bushv", "p.u., Positive-Sequence-Voltage, Imaginary Part"
      m_u1i_buslv = "m:u1i:buslv", "p.u., Positive-Sequence-Voltage, Imaginary Part"
      m_u1i_bushvn = "m:u1i:bushvn", "p.u., Positive-Sequence-Voltage, Imaginary Part"
      m_u1i_buslvn = "m:u1i:buslvn", "p.u., Positive-Sequence-Voltage, Imaginary Part"
      m_u1_bushv = "m:u1:bushv", "p.u., Positive-Sequence-Voltage, Magnitude"
      m_u1_buslv = "m:u1:buslv", "p.u., Positive-Sequence-Voltage, Magnitude"
      m_u1_bushvn = "m:u1:bushvn", "p.u., Positive-Sequence-Voltage, Magnitude"
      m_u1_buslvn = "m:u1:buslvn", "p.u., Positive-Sequence-Voltage, Magnitude"
      m_U1_bushv = "m:U1:bushv", "kV, Line-Ground Positive-Sequence-Voltage, Magnitude"
      m_U1_buslv = "m:U1:buslv", "kV, Line-Ground Positive-Sequence-Voltage, Magnitude"
      m_U1_bushvn = "m:U1:bushvn", "kV, Line-Ground Positive-Sequence-Voltage, Magnitude"
      m_U1_buslvn = "m:U1:buslvn", "kV, Line-Ground Positive-Sequence-Voltage, Magnitude"
      m_U1l_bushv = "m:U1l:bushv", "kV, Line-Line Positive-Sequence-Voltage, Magnitude"
      m_U1l_buslv = "m:U1l:buslv", "kV, Line-Line Positive-Sequence-Voltage, Magnitude"
      m_U1l_bushvn = "m:U1l:bushvn", "kV, Line-Line Positive-Sequence-Voltage, Magnitude"
      m_U1l_buslvn = "m:U1l:buslvn", "kV, Line-Line Positive-Sequence-Voltage, Magnitude"
      m_phiu1_bushv = "m:phiu1:bushv", "deg, Positive-Sequence-Voltage, Angle"
      m_phiu1_buslv = "m:phiu1:buslv", "deg, Positive-Sequence-Voltage, Angle"
      m_phiu1_bushvn = "m:phiu1:bushvn", "deg, Positive-Sequence-Voltage, Angle"
      m_phiu1_buslvn = "m:phiu1:buslvn", "deg, Positive-Sequence-Voltage, Angle"
      m_u2_bushv = "m:u2:bushv", "p.u., Negative-Sequence-Voltage, Magnitude"
      m_u2_buslv = "m:u2:buslv", "p.u., Negative-Sequence-Voltage, Magnitude"
      m_u2_bushvn = "m:u2:bushvn", "p.u., Negative-Sequence-Voltage, Magnitude"
      m_u2_buslvn = "m:u2:buslvn", "p.u., Negative-Sequence-Voltage, Magnitude"
      m_phiu2_bushv = "m:phiu2:bushv", "deg, Negative-Sequence-Voltage, Angle"
      m_phiu2_buslv = "m:phiu2:buslv", "deg, Negative-Sequence-Voltage, Angle"
      m_phiu2_bushvn = "m:phiu2:bushvn", "deg, Negative-Sequence-Voltage, Angle"
      m_phiu2_buslvn = "m:phiu2:buslvn", "deg, Negative-Sequence-Voltage, Angle"
      m_inet_bushv_A = "m:inet:bushv:A", "p.u., Phase Current, Magnitude"
      m_inet_bushv_B = "m:inet:bushv:B", "p.u., Phase Current, Magnitude"
      m_inet_bushv_C = "m:inet:bushv:C", "p.u., Phase Current, Magnitude"
      m_inet_bushv_N = "m:inet:bushv:N", "p.u., Phase Current, Magnitude"
      m_inet_buslv_A = "m:inet:buslv:A", "p.u., Phase Current, Magnitude"
      m_inet_buslv_B = "m:inet:buslv:B", "p.u., Phase Current, Magnitude"
      m_inet_buslv_C = "m:inet:buslv:C", "p.u., Phase Current, Magnitude"
      m_inet_buslv_N = "m:inet:buslv:N", "p.u., Phase Current, Magnitude"
      m_inet_bushvn = "m:inet:bushvn", "p.u., Phase Current, Magnitude"
      m_inet_buslvn = "m:inet:buslvn", "p.u., Phase Current, Magnitude"
      m_I_bushv_A = "m:I:bushv:A", "kA, Phase Current, Magnitude"
      m_I_bushv_B = "m:I:bushv:B", "kA, Phase Current, Magnitude"
      m_I_bushv_C = "m:I:bushv:C", "kA, Phase Current, Magnitude"
      m_I_bushv_N = "m:I:bushv:N", "kA, Phase Current, Magnitude"
      m_I_buslv_A = "m:I:buslv:A", "kA, Phase Current, Magnitude"
      m_I_buslv_B = "m:I:buslv:B", "kA, Phase Current, Magnitude"
      m_I_buslv_C = "m:I:buslv:C", "kA, Phase Current, Magnitude"
      m_I_buslv_N = "m:I:buslv:N", "kA, Phase Current, Magnitude"
      m_I_bushvn = "m:I:bushvn", "kA, Phase Current, Magnitude"
      m_I_buslvn = "m:I:buslvn", "kA, Phase Current, Magnitude"
      m_phii_bushv_A = "m:phii:bushv:A", "deg, Phase Current, Angle"
      m_phii_bushv_B = "m:phii:bushv:B", "deg, Phase Current, Angle"
      m_phii_bushv_C = "m:phii:bushv:C", "deg, Phase Current, Angle"
      m_phii_bushv_N = "m:phii:bushv:N", "deg, Phase Current, Angle"
      m_phii_buslv_A = "m:phii:buslv:A", "deg, Phase Current, Angle"
      m_phii_buslv_B = "m:phii:buslv:B", "deg, Phase Current, Angle"
      m_phii_buslv_C = "m:phii:buslv:C", "deg, Phase Current, Angle"
      m_phii_buslv_N = "m:phii:buslv:N", "deg, Phase Current, Angle"
      m_phii_bushvn = "m:phii:bushvn", "deg, Phase Current, Angle"
      m_phii_buslvn = "m:phii:buslvn", "deg, Phase Current, Angle"
      m_i0r_bushv = "m:i0r:bushv", "p.u., Zero-Sequence Current, Real Part"
      m_i0r_buslv = "m:i0r:buslv", "p.u., Zero-Sequence Current, Real Part"
      m_i0r_bushvn = "m:i0r:bushvn", "p.u., Zero-Sequence Current, Real Part"
      m_i0r_buslvn = "m:i0r:buslvn", "p.u., Zero-Sequence Current, Real Part"
      m_i0i_bushv = "m:i0i:bushv", "p.u., Zero-Sequence Current, Imaginary Part"
      m_i0i_buslv = "m:i0i:buslv", "p.u., Zero-Sequence Current, Imaginary Part"
      m_i0i_bushvn = "m:i0i:bushvn", "p.u., Zero-Sequence Current, Imaginary Part"
      m_i0i_buslvn = "m:i0i:buslvn", "p.u., Zero-Sequence Current, Imaginary Part"
      m_i0_bushv = "m:i0:bushv", "p.u., Zero-Sequence Current, Magnitude"
      m_i0_buslv = "m:i0:buslv", "p.u., Zero-Sequence Current, Magnitude"
      m_i0_bushvn = "m:i0:bushvn", "p.u., Zero-Sequence Current, Magnitude"
      m_i0_buslvn = "m:i0:buslvn", "p.u., Zero-Sequence Current, Magnitude"
      m_I0_bushv = "m:I0:bushv", "kA, Zero-Sequence Current, Magnitude"
      m_I0_buslv = "m:I0:buslv", "kA, Zero-Sequence Current, Magnitude"
      m_I0_bushvn = "m:I0:bushvn", "kA, Zero-Sequence Current, Magnitude"
      m_I0_buslvn = "m:I0:buslvn", "kA, Zero-Sequence Current, Magnitude"
      m_I0x3_bushv = "m:I0x3:bushv", "kA, 3*I0"
      m_I0x3_buslv = "m:I0x3:buslv", "kA, 3*I0"
      m_I0x3_bushvn = "m:I0x3:bushvn", "kA, 3*I0"
      m_I0x3_buslvn = "m:I0x3:buslvn", "kA, 3*I0"
      m_phii0_bushv = "m:phii0:bushv", "deg, Zero-Sequence Current, Angle"
      m_phii0_buslv = "m:phii0:buslv", "deg, Zero-Sequence Current, Angle"
      m_phii0_bushvn = "m:phii0:bushvn", "deg, Zero-Sequence Current, Angle"
      m_phii0_buslvn = "m:phii0:buslvn", "deg, Zero-Sequence Current, Angle"
      m_i1r_bushv = "m:i1r:bushv", "p.u., Positive-Sequence Current, Real Part"
      m_i1r_buslv = "m:i1r:buslv", "p.u., Positive-Sequence Current, Real Part"
      m_i1r_bushvn = "m:i1r:bushvn", "p.u., Positive-Sequence Current, Real Part"
      m_i1r_buslvn = "m:i1r:buslvn", "p.u., Positive-Sequence Current, Real Part"
      m_i1i_bushv = "m:i1i:bushv", "p.u., Positive-Sequence Current, Imaginary Part"
      m_i1i_buslv = "m:i1i:buslv", "p.u., Positive-Sequence Current, Imaginary Part"
      m_i1i_bushvn = "m:i1i:bushvn", "p.u., Positive-Sequence Current, Imaginary Part"
      m_i1i_buslvn = "m:i1i:buslvn", "p.u., Positive-Sequence Current, Imaginary Part"
      m_i1_bushv = "m:i1:bushv", "p.u., Positive-Sequence Current, Magnitude"
      m_i1_buslv = "m:i1:buslv", "p.u., Positive-Sequence Current, Magnitude"
      m_i1_bushvn = "m:i1:bushvn", "p.u., Positive-Sequence Current, Magnitude"
      m_i1_buslvn = "m:i1:buslvn", "p.u., Positive-Sequence Current, Magnitude"
      m_I1_bushv = "m:I1:bushv", "kA, Positive-Sequence Current, Magnitude"
      m_I1_buslv = "m:I1:buslv", "kA, Positive-Sequence Current, Magnitude"
      m_I1_bushvn = "m:I1:bushvn", "kA, Positive-Sequence Current, Magnitude"
      m_I1_buslvn = "m:I1:buslvn", "kA, Positive-Sequence Current, Magnitude"
      m_phii1_bushv = "m:phii1:bushv", "deg, Positive-Sequence Current, Angle"
      m_phii1_buslv = "m:phii1:buslv", "deg, Positive-Sequence Current, Angle"
      m_phii1_bushvn = "m:phii1:bushvn", "deg, Positive-Sequence Current, Angle"
      m_phii1_buslvn = "m:phii1:buslvn", "deg, Positive-Sequence Current, Angle"
      m_i2r_bushv = "m:i2r:bushv", "p.u., Negative-Sequence Current, Real Part"
      m_i2r_buslv = "m:i2r:buslv", "p.u., Negative-Sequence Current, Real Part"
      m_i2r_bushvn = "m:i2r:bushvn", "p.u., Negative-Sequence Current, Real Part"
      m_i2r_buslvn = "m:i2r:buslvn", "p.u., Negative-Sequence Current, Real Part"
      m_i2i_bushv = "m:i2i:bushv", "p.u., Negative-Sequence Current, Imaginary Part"
      m_i2i_buslv = "m:i2i:buslv", "p.u., Negative-Sequence Current, Imaginary Part"
      m_i2i_bushvn = "m:i2i:bushvn", "p.u., Negative-Sequence Current, Imaginary Part"
      m_i2i_buslvn = "m:i2i:buslvn", "p.u., Negative-Sequence Current, Imaginary Part"
      m_i2_bushv = "m:i2:bushv", "p.u., Negative-Sequence Current, Magnitude"
      m_i2_buslv = "m:i2:buslv", "p.u., Negative-Sequence Current, Magnitude"
      m_i2_bushvn = "m:i2:bushvn", "p.u., Negative-Sequence Current, Magnitude"
      m_i2_buslvn = "m:i2:buslvn", "p.u., Negative-Sequence Current, Magnitude"
      m_I2_bushv = "m:I2:bushv", "kA, Negative-Sequence Current, Magnitude"
      m_I2_buslv = "m:I2:buslv", "kA, Negative-Sequence Current, Magnitude"
      m_I2_bushvn = "m:I2:bushvn", "kA, Negative-Sequence Current, Magnitude"
      m_I2_buslvn = "m:I2:buslvn", "kA, Negative-Sequence Current, Magnitude"
      m_phii2_bushv = "m:phii2:bushv", "deg, Negative-Sequence Current, Angle"
      m_phii2_buslv = "m:phii2:buslv", "deg, Negative-Sequence Current, Angle"
      m_phii2_bushvn = "m:phii2:bushvn", "deg, Negative-Sequence Current, Angle"
      m_phii2_buslvn = "m:phii2:buslvn", "deg, Negative-Sequence Current, Angle"
      m_P_bushv_A = "m:P:bushv:A", "MW, Active Power"
      m_P_bushv_B = "m:P:bushv:B", "MW, Active Power"
      m_P_bushv_C = "m:P:bushv:C", "MW, Active Power"
      m_P_bushv_N = "m:P:bushv:N", "MW, Active Power"
      m_P_buslv_A = "m:P:buslv:A", "MW, Active Power"
      m_P_buslv_B = "m:P:buslv:B", "MW, Active Power"
      m_P_buslv_C = "m:P:buslv:C", "MW, Active Power"
      m_P_buslv_N = "m:P:buslv:N", "MW, Active Power"
      m_P_bushvn = "m:P:bushvn", "MW, Active Power"
      m_P_buslvn = "m:P:buslvn", "MW, Active Power"
      m_Q_bushv_A = "m:Q:bushv:A", "Mvar, Reactive Power"
      m_Q_bushv_B = "m:Q:bushv:B", "Mvar, Reactive Power"
      m_Q_bushv_C = "m:Q:bushv:C", "Mvar, Reactive Power"
      m_Q_bushv_N = "m:Q:bushv:N", "Mvar, Reactive Power"
      m_Q_buslv_A = "m:Q:buslv:A", "Mvar, Reactive Power"
      m_Q_buslv_B = "m:Q:buslv:B", "Mvar, Reactive Power"
      m_Q_buslv_C = "m:Q:buslv:C", "Mvar, Reactive Power"
      m_Q_buslv_N = "m:Q:buslv:N", "Mvar, Reactive Power"
      m_Q_bushvn = "m:Q:bushvn", "Mvar, Reactive Power"
      m_Q_buslvn = "m:Q:buslvn", "Mvar, Reactive Power"
      m_S_bushv_A = "m:S:bushv:A", "MVA, Apparent Power"
      m_S_bushv_B = "m:S:bushv:B", "MVA, Apparent Power"
      m_S_bushv_C = "m:S:bushv:C", "MVA, Apparent Power"
      m_S_bushv_N = "m:S:bushv:N", "MVA, Apparent Power"
      m_S_buslv_A = "m:S:buslv:A", "MVA, Apparent Power"
      m_S_buslv_B = "m:S:buslv:B", "MVA, Apparent Power"
      m_S_buslv_C = "m:S:buslv:C", "MVA, Apparent Power"
      m_S_buslv_N = "m:S:buslv:N", "MVA, Apparent Power"
      m_S_bushvn = "m:S:bushvn", "MVA, Apparent Power"
      m_S_buslvn = "m:S:buslvn", "MVA, Apparent Power"
      m_cosphi_bushv_A = "m:cosphi:bushv:A", "Power Factor"
      m_cosphi_bushv_B = "m:cosphi:bushv:B", "Power Factor"
      m_cosphi_bushv_C = "m:cosphi:bushv:C", "Power Factor"
      m_cosphi_bushv_N = "m:cosphi:bushv:N", "Power Factor"
      m_cosphi_buslv_A = "m:cosphi:buslv:A", "Power Factor"
      m_cosphi_buslv_B = "m:cosphi:buslv:B", "Power Factor"
      m_cosphi_buslv_C = "m:cosphi:buslv:C", "Power Factor"
      m_cosphi_buslv_N = "m:cosphi:buslv:N", "Power Factor"
      m_cosphi_bushvn = "m:cosphi:bushvn", "Power Factor"
      m_cosphi_buslvn = "m:cosphi:buslvn", "Power Factor"
      m_tanphi_bushv_A = "m:tanphi:bushv:A", "tan(phi)"
      m_tanphi_bushv_B = "m:tanphi:bushv:B", "tan(phi)"
      m_tanphi_bushv_C = "m:tanphi:bushv:C", "tan(phi)"
      m_tanphi_bushv_N = "m:tanphi:bushv:N", "tan(phi)"
      m_tanphi_buslv_A = "m:tanphi:buslv:A", "tan(phi)"
      m_tanphi_buslv_B = "m:tanphi:buslv:B", "tan(phi)"
      m_tanphi_buslv_C = "m:tanphi:buslv:C", "tan(phi)"
      m_tanphi_buslv_N = "m:tanphi:buslv:N", "tan(phi)"
      m_tanphi_bushvn = "m:tanphi:bushvn", "tan(phi)"
      m_tanphi_buslvn = "m:tanphi:buslvn", "tan(phi)"
      m_Psum_bushv = "m:Psum:bushv", "MW, Total Active Power"
      m_Psum_buslv = "m:Psum:buslv", "MW, Total Active Power"
      m_Psum_bushvn = "m:Psum:bushvn", "MW, Total Active Power"
      m_Psum_buslvn = "m:Psum:buslvn", "MW, Total Active Power"
      m_Qsum_bushv = "m:Qsum:bushv", "Mvar, Total Reactive Power"
      m_Qsum_buslv = "m:Qsum:buslv", "Mvar, Total Reactive Power"
      m_Qsum_bushvn = "m:Qsum:bushvn", "Mvar, Total Reactive Power"
      m_Qsum_buslvn = "m:Qsum:buslvn", "Mvar, Total Reactive Power"
      m_Ssum_bushv = "m:Ssum:bushv", "MVA, Total Apparent Power"
      m_Ssum_buslv = "m:Ssum:buslv", "MVA, Total Apparent Power"
      m_Ssum_bushvn = "m:Ssum:bushvn", "MVA, Total Apparent Power"
      m_Ssum_buslvn = "m:Ssum:buslvn", "MVA, Total Apparent Power"
      m_cosphisum_bushv = "m:cosphisum:bushv", "Total Power Factor"
      m_cosphisum_buslv = "m:cosphisum:buslv", "Total Power Factor"
      m_cosphisum_bushvn = "m:cosphisum:bushvn", "Total Power Factor"
      m_cosphisum_buslvn = "m:cosphisum:buslvn", "Total Power Factor"
      m_tanphisum_bushv = "m:tanphisum:bushv", "Total tan(phi)"
      m_tanphisum_buslv = "m:tanphisum:buslv", "Total tan(phi)"
      m_tanphisum_bushvn = "m:tanphisum:bushvn", "Total tan(phi)"
      m_tanphisum_buslvn = "m:tanphisum:buslvn", "Total tan(phi)"
      m_Pln_bushv_A = "m:Pln:bushv:A", "MW, Active Power (Line-Neutral)"
      m_Pln_bushv_B = "m:Pln:bushv:B", "MW, Active Power (Line-Neutral)"
      m_Pln_bushv_C = "m:Pln:bushv:C", "MW, Active Power (Line-Neutral)"
      m_Pln_bushv_N = "m:Pln:bushv:N", "MW, Active Power (Line-Neutral)"
      m_Pln_buslv_A = "m:Pln:buslv:A", "MW, Active Power (Line-Neutral)"
      m_Pln_buslv_B = "m:Pln:buslv:B", "MW, Active Power (Line-Neutral)"
      m_Pln_buslv_C = "m:Pln:buslv:C", "MW, Active Power (Line-Neutral)"
      m_Pln_buslv_N = "m:Pln:buslv:N", "MW, Active Power (Line-Neutral)"
      m_Pln_bushvn = "m:Pln:bushvn", "MW, Active Power (Line-Neutral)"
      m_Pln_buslvn = "m:Pln:buslvn", "MW, Active Power (Line-Neutral)"
      m_Qln_bushv_A = "m:Qln:bushv:A", "Mvar, Reactive Power (Line-Neutral)"
      m_Qln_bushv_B = "m:Qln:bushv:B", "Mvar, Reactive Power (Line-Neutral)"
      m_Qln_bushv_C = "m:Qln:bushv:C", "Mvar, Reactive Power (Line-Neutral)"
      m_Qln_bushv_N = "m:Qln:bushv:N", "Mvar, Reactive Power (Line-Neutral)"
      m_Qln_buslv_A = "m:Qln:buslv:A", "Mvar, Reactive Power (Line-Neutral)"
      m_Qln_buslv_B = "m:Qln:buslv:B", "Mvar, Reactive Power (Line-Neutral)"
      m_Qln_buslv_C = "m:Qln:buslv:C", "Mvar, Reactive Power (Line-Neutral)"
      m_Qln_buslv_N = "m:Qln:buslv:N", "Mvar, Reactive Power (Line-Neutral)"
      m_Qln_bushvn = "m:Qln:bushvn", "Mvar, Reactive Power (Line-Neutral)"
      m_Qln_buslvn = "m:Qln:buslvn", "Mvar, Reactive Power (Line-Neutral)"
      m_P1_bushv = "m:P1:bushv", "MW, Positive Sequence Active Power"
      m_P1_buslv = "m:P1:buslv", "MW, Positive Sequence Active Power"
      m_P1_bushvn = "m:P1:bushvn", "MW, Positive Sequence Active Power"
      m_P1_buslvn = "m:P1:buslvn", "MW, Positive Sequence Active Power"
      m_Q1_bushv = "m:Q1:bushv", "Mvar, Positive Sequence Reactive Power"
      m_Q1_buslv = "m:Q1:buslv", "Mvar, Positive Sequence Reactive Power"
      m_Q1_bushvn = "m:Q1:bushvn", "Mvar, Positive Sequence Reactive Power"
      m_Q1_buslvn = "m:Q1:buslvn", "Mvar, Positive Sequence Reactive Power"
      m_P2_bushv = "m:P2:bushv", "MW, Negative Sequence Active Power"
      m_P2_buslv = "m:P2:buslv", "MW, Negative Sequence Active Power"
      m_P2_bushvn = "m:P2:bushvn", "MW, Negative Sequence Active Power"
      m_P2_buslvn = "m:P2:buslvn", "MW, Negative Sequence Active Power"
      m_Q2_bushv = "m:Q2:bushv", "Mvar, Negative Sequence Reactive Power"
      m_Q2_buslv = "m:Q2:buslv", "Mvar, Negative Sequence Reactive Power"
      m_Q2_bushvn = "m:Q2:bushvn", "Mvar, Negative Sequence Reactive Power"
      m_Q2_buslvn = "m:Q2:buslvn", "Mvar, Negative Sequence Reactive Power"
      m_P0_bushv = "m:P0:bushv", "MW, Zero Sequence Active Power"
      m_P0_buslv = "m:P0:buslv", "MW, Zero Sequence Active Power"
      m_P0_bushvn = "m:P0:bushvn", "MW, Zero Sequence Active Power"
      m_P0_buslvn = "m:P0:buslvn", "MW, Zero Sequence Active Power"
      m_Q0_bushv = "m:Q0:bushv", "Mvar, Zero Sequence Reactive Power"
      m_Q0_buslv = "m:Q0:buslv", "Mvar, Zero Sequence Reactive Power"
      m_Q0_bushvn = "m:Q0:bushvn", "Mvar, Zero Sequence Reactive Power"
      m_Q0_buslvn = "m:Q0:buslvn", "Mvar, Zero Sequence Reactive Power"
      m_i1P_bushv = "m:i1P:bushv", "p.u., Positive-Sequence Active Current"
      m_i1P_buslv = "m:i1P:buslv", "p.u., Positive-Sequence Active Current"
      m_i1P_bushvn = "m:i1P:bushvn", "p.u., Positive-Sequence Active Current"
      m_i1P_buslvn = "m:i1P:buslvn", "p.u., Positive-Sequence Active Current"
      m_i1Q_bushv = "m:i1Q:bushv", "p.u., Positive-Sequence Reactive Current"
      m_i1Q_buslv = "m:i1Q:buslv", "p.u., Positive-Sequence Reactive Current"
      m_i1Q_bushvn = "m:i1Q:bushvn", "p.u., Positive-Sequence Reactive Current"
      m_i1Q_buslvn = "m:i1Q:buslvn", "p.u., Positive-Sequence Reactive Current"
      m_I1P_bushv = "m:I1P:bushv", "kA, Positive-Sequence Active Current"
      m_I1P_buslv = "m:I1P:buslv", "kA, Positive-Sequence Active Current"
      m_I1P_bushvn = "m:I1P:bushvn", "kA, Positive-Sequence Active Current"
      m_I1P_buslvn = "m:I1P:buslvn", "kA, Positive-Sequence Active Current"
      m_I1Q_bushv = "m:I1Q:bushv", "kA, Positive-Sequence Reactive Current"
      m_I1Q_buslv = "m:I1Q:buslv", "kA, Positive-Sequence Reactive Current"
      m_I1Q_bushvn = "m:I1Q:bushvn", "kA, Positive-Sequence Reactive Current"
      m_I1Q_buslvn = "m:I1Q:buslvn", "kA, Positive-Sequence Reactive Current"
      m_i2P_bushv = "m:i2P:bushv", "p.u., Negative-Sequence Active Current"
      m_i2P_buslv = "m:i2P:buslv", "p.u., Negative-Sequence Active Current"
      m_i2P_bushvn = "m:i2P:bushvn", "p.u., Negative-Sequence Active Current"
      m_i2P_buslvn = "m:i2P:buslvn", "p.u., Negative-Sequence Active Current"
      m_i2Q_bushv = "m:i2Q:bushv", "p.u., Negative-Sequence Reactive Current"
      m_i2Q_buslv = "m:i2Q:buslv", "p.u., Negative-Sequence Reactive Current"
      m_i2Q_bushvn = "m:i2Q:bushvn", "p.u., Negative-Sequence Reactive Current"
      m_i2Q_buslvn = "m:i2Q:buslvn", "p.u., Negative-Sequence Reactive Current"
      m_I2P_bushv = "m:I2P:bushv", "kA, Negative-Sequence Active Current"
      m_I2P_buslv = "m:I2P:buslv", "kA, Negative-Sequence Active Current"
      m_I2P_bushvn = "m:I2P:bushvn", "kA, Negative-Sequence Active Current"
      m_I2P_buslvn = "m:I2P:buslvn", "kA, Negative-Sequence Active Current"
      m_I2Q_bushv = "m:I2Q:bushv", "kA, Negative-Sequence Reactive Current"
      m_I2Q_buslv = "m:I2Q:buslv", "kA, Negative-Sequence Reactive Current"
      m_I2Q_bushvn = "m:I2Q:bushvn", "kA, Negative-Sequence Reactive Current"
      m_I2Q_buslvn = "m:I2Q:buslvn", "kA, Negative-Sequence Reactive Current"
      m_ur_bushv_A = "m:ur:bushv:A", "p.u., Phase Voltage, Real Part"
      m_ur_bushv_B = "m:ur:bushv:B", "p.u., Phase Voltage, Real Part"
      m_ur_bushv_C = "m:ur:bushv:C", "p.u., Phase Voltage, Real Part"
      m_ur_bushv_N = "m:ur:bushv:N", "p.u., Phase Voltage, Real Part"
      m_ur_buslv_A = "m:ur:buslv:A", "p.u., Phase Voltage, Real Part"
      m_ur_buslv_B = "m:ur:buslv:B", "p.u., Phase Voltage, Real Part"
      m_ur_buslv_C = "m:ur:buslv:C", "p.u., Phase Voltage, Real Part"
      m_ur_buslv_N = "m:ur:buslv:N", "p.u., Phase Voltage, Real Part"
      m_ur_bushvn = "m:ur:bushvn", "p.u., Phase Voltage, Real Part"
      m_ur_buslvn = "m:ur:buslvn", "p.u., Phase Voltage, Real Part"
      m_ui_bushv_A = "m:ui:bushv:A", "p.u., Phase Voltage, Imaginary Part"
      m_ui_bushv_B = "m:ui:bushv:B", "p.u., Phase Voltage, Imaginary Part"
      m_ui_bushv_C = "m:ui:bushv:C", "p.u., Phase Voltage, Imaginary Part"
      m_ui_bushv_N = "m:ui:bushv:N", "p.u., Phase Voltage, Imaginary Part"
      m_ui_buslv_A = "m:ui:buslv:A", "p.u., Phase Voltage, Imaginary Part"
      m_ui_buslv_B = "m:ui:buslv:B", "p.u., Phase Voltage, Imaginary Part"
      m_ui_buslv_C = "m:ui:buslv:C", "p.u., Phase Voltage, Imaginary Part"
      m_ui_buslv_N = "m:ui:buslv:N", "p.u., Phase Voltage, Imaginary Part"
      m_ui_bushvn = "m:ui:bushvn", "p.u., Phase Voltage, Imaginary Part"
      m_ui_buslvn = "m:ui:buslvn", "p.u., Phase Voltage, Imaginary Part"
      m_u_bushv_A = "m:u:bushv:A", "p.u., Phase Voltage, Magnitude"
      m_u_bushv_B = "m:u:bushv:B", "p.u., Phase Voltage, Magnitude"
      m_u_bushv_C = "m:u:bushv:C", "p.u., Phase Voltage, Magnitude"
      m_u_bushv_N = "m:u:bushv:N", "p.u., Phase Voltage, Magnitude"
      m_u_buslv_A = "m:u:buslv:A", "p.u., Phase Voltage, Magnitude"
      m_u_buslv_B = "m:u:buslv:B", "p.u., Phase Voltage, Magnitude"
      m_u_buslv_C = "m:u:buslv:C", "p.u., Phase Voltage, Magnitude"
      m_u_buslv_N = "m:u:buslv:N", "p.u., Phase Voltage, Magnitude"
      m_u_bushvn = "m:u:bushvn", "p.u., Phase Voltage, Magnitude"
      m_u_buslvn = "m:u:buslvn", "p.u., Phase Voltage, Magnitude"
      m_ir_bushv_A = "m:ir:bushv:A", "p.u., Phase Current, Real Part"
      m_ir_bushv_B = "m:ir:bushv:B", "p.u., Phase Current, Real Part"
      m_ir_bushv_C = "m:ir:bushv:C", "p.u., Phase Current, Real Part"
      m_ir_bushv_N = "m:ir:bushv:N", "p.u., Phase Current, Real Part"
      m_ir_buslv_A = "m:ir:buslv:A", "p.u., Phase Current, Real Part"
      m_ir_buslv_B = "m:ir:buslv:B", "p.u., Phase Current, Real Part"
      m_ir_buslv_C = "m:ir:buslv:C", "p.u., Phase Current, Real Part"
      m_ir_buslv_N = "m:ir:buslv:N", "p.u., Phase Current, Real Part"
      m_ir_bushvn = "m:ir:bushvn", "p.u., Phase Current, Real Part"
      m_ir_buslvn = "m:ir:buslvn", "p.u., Phase Current, Real Part"
      m_ii_bushv_A = "m:ii:bushv:A", "p.u., Phase Current, Imaginary Part"
      m_ii_bushv_B = "m:ii:bushv:B", "p.u., Phase Current, Imaginary Part"
      m_ii_bushv_C = "m:ii:bushv:C", "p.u., Phase Current, Imaginary Part"
      m_ii_bushv_N = "m:ii:bushv:N", "p.u., Phase Current, Imaginary Part"
      m_ii_buslv_A = "m:ii:buslv:A", "p.u., Phase Current, Imaginary Part"
      m_ii_buslv_B = "m:ii:buslv:B", "p.u., Phase Current, Imaginary Part"
      m_ii_buslv_C = "m:ii:buslv:C", "p.u., Phase Current, Imaginary Part"
      m_ii_buslv_N = "m:ii:buslv:N", "p.u., Phase Current, Imaginary Part"
      m_ii_bushvn = "m:ii:bushvn", "p.u., Phase Current, Imaginary Part"
      m_ii_buslvn = "m:ii:buslvn", "p.u., Phase Current, Imaginary Part"
      m_i_bushv_A = "m:i:bushv:A", "p.u., Phase Current, Magnitude"
      m_i_bushv_B = "m:i:bushv:B", "p.u., Phase Current, Magnitude"
      m_i_bushv_C = "m:i:bushv:C", "p.u., Phase Current, Magnitude"
      m_i_bushv_N = "m:i:bushv:N", "p.u., Phase Current, Magnitude"
      m_i_buslv_A = "m:i:buslv:A", "p.u., Phase Current, Magnitude"
      m_i_buslv_B = "m:i:buslv:B", "p.u., Phase Current, Magnitude"
      m_i_buslv_C = "m:i:buslv:C", "p.u., Phase Current, Magnitude"
      m_i_buslv_N = "m:i:buslv:N", "p.u., Phase Current, Magnitude"
      m_i_bushvn = "m:i:bushvn", "p.u., Phase Current, Magnitude"
      m_i_buslvn = "m:i:buslvn", "p.u., Phase Current, Magnitude"
      m_phiui_bushv_A = "m:phiui:bushv:A", "deg, Angle between Voltage and Current"
      m_phiui_bushv_B = "m:phiui:bushv:B", "deg, Angle between Voltage and Current"
      m_phiui_bushv_C = "m:phiui:bushv:C", "deg, Angle between Voltage and Current"
      m_phiui_bushv_N = "m:phiui:bushv:N", "deg, Angle between Voltage and Current"
      m_phiui_buslv_A = "m:phiui:buslv:A", "deg, Angle between Voltage and Current"
      m_phiui_buslv_B = "m:phiui:buslv:B", "deg, Angle between Voltage and Current"
      m_phiui_buslv_C = "m:phiui:buslv:C", "deg, Angle between Voltage and Current"
      m_phiui_buslv_N = "m:phiui:buslv:N", "deg, Angle between Voltage and Current"
      m_phiui_bushvn = "m:phiui:bushvn", "deg, Angle between Voltage and Current"
      m_phiui_buslvn = "m:phiui:buslvn", "deg, Angle between Voltage and Current"
      m_phiu0i0_bushv = "m:phiu0i0:bushv", "deg, Angle between Voltage and Current in zero-sequence system"
      m_phiu0i0_buslv = "m:phiu0i0:buslv", "deg, Angle between Voltage and Current in zero-sequence system"
      m_phiu0i0_bushvn = "m:phiu0i0:bushvn", "deg, Angle between Voltage and Current in zero-sequence system"
      m_phiu0i0_buslvn = "m:phiu0i0:buslvn", "deg, Angle between Voltage and Current in zero-sequence system"
      m_phiu1i1_bushv = "m:phiu1i1:bushv", "deg, Angle between Voltage and Current in positive sequence system"
      m_phiu1i1_buslv = "m:phiu1i1:buslv", "deg, Angle between Voltage and Current in positive sequence system"
      m_phiu1i1_bushvn = "m:phiu1i1:bushvn", "deg, Angle between Voltage and Current in positive sequence system"
      m_phiu1i1_buslvn = "m:phiu1i1:buslvn", "deg, Angle between Voltage and Current in positive sequence system"
      m_phiu2i2_bushv = "m:phiu2i2:bushv", "deg, Angle between Voltage and Current in negative sequence system"
      m_phiu2i2_buslv = "m:phiu2i2:buslv", "deg, Angle between Voltage and Current in negative sequence system"
      m_phiu2i2_bushvn = "m:phiu2i2:bushvn", "deg, Angle between Voltage and Current in negative sequence system"
      m_phiu2i2_buslvn = "m:phiu2i2:buslvn", "deg, Angle between Voltage and Current in negative sequence system"
      m_Tfct_bushv = "m:Tfct:bushv", "s, Fault Clearing Time"
      m_Tfct_buslv = "m:Tfct:buslv", "s, Fault Clearing Time"
      m_Tfct_bushvn = "m:Tfct:bushvn", "s, Fault Clearing Time"
      m_Tfct_buslvn = "m:Tfct:buslvn", "s, Fault Clearing Time"
      m_TfctPh_bushv_A = "m:TfctPh:bushv:A", "s, Fault Clearing Time"
      m_TfctPh_bushv_B = "m:TfctPh:bushv:B", "s, Fault Clearing Time"
      m_TfctPh_bushv_C = "m:TfctPh:bushv:C", "s, Fault Clearing Time"
      m_TfctPh_bushv_N = "m:TfctPh:bushv:N", "s, Fault Clearing Time"
      m_TfctPh_buslv_A = "m:TfctPh:buslv:A", "s, Fault Clearing Time"
      m_TfctPh_buslv_B = "m:TfctPh:buslv:B", "s, Fault Clearing Time"
      m_TfctPh_buslv_C = "m:TfctPh:buslv:C", "s, Fault Clearing Time"
      m_TfctPh_buslv_N = "m:TfctPh:buslv:N", "s, Fault Clearing Time"
      m_TfctPh_bushvn = "m:TfctPh:bushvn", "s, Fault Clearing Time"
      m_TfctPh_buslvn = "m:TfctPh:buslvn", "s, Fault Clearing Time"
      m_Brkload_bushv = "m:Brkload:bushv", "%, Max. Breaker Loading"
      m_Brkload_buslv = "m:Brkload:buslv", "%, Max. Breaker Loading"
      m_Brkload_bushvn = "m:Brkload:bushvn", "%, Max. Breaker Loading"
      m_Brkload_buslvn = "m:Brkload:buslvn", "%, Max. Breaker Loading"
      m_BrkloadPh_bushv_A = "m:BrkloadPh:bushv:A", "%, Breaker Loading"
      m_BrkloadPh_bushv_B = "m:BrkloadPh:bushv:B", "%, Breaker Loading"
      m_BrkloadPh_bushv_C = "m:BrkloadPh:bushv:C", "%, Breaker Loading"
      m_BrkloadPh_bushv_N = "m:BrkloadPh:bushv:N", "%, Breaker Loading"
      m_BrkloadPh_buslv_A = "m:BrkloadPh:buslv:A", "%, Breaker Loading"
      m_BrkloadPh_buslv_B = "m:BrkloadPh:buslv:B", "%, Breaker Loading"
      m_BrkloadPh_buslv_C = "m:BrkloadPh:buslv:C", "%, Breaker Loading"
      m_BrkloadPh_buslv_N = "m:BrkloadPh:buslv:N", "%, Breaker Loading"
      m_BrkloadPh_bushvn = "m:BrkloadPh:bushvn", "%, Breaker Loading"
      m_BrkloadPh_buslvn = "m:BrkloadPh:buslvn", "%, Breaker Loading"
      n_ur_bushv_A = "n:ur:bushv:A", "p.u., Line-Ground Voltage, Real Part"
      n_ur_bushv_B = "n:ur:bushv:B", "p.u., Line-Ground Voltage, Real Part"
      n_ur_bushv_C = "n:ur:bushv:C", "p.u., Line-Ground Voltage, Real Part"
      n_ur_bushv_N = "n:ur:bushv:N", "p.u., Line-Ground Voltage, Real Part"
      n_ur_buslv_A = "n:ur:buslv:A", "p.u., Line-Ground Voltage, Real Part"
      n_ur_buslv_B = "n:ur:buslv:B", "p.u., Line-Ground Voltage, Real Part"
      n_ur_buslv_C = "n:ur:buslv:C", "p.u., Line-Ground Voltage, Real Part"
      n_ur_buslv_N = "n:ur:buslv:N", "p.u., Line-Ground Voltage, Real Part"
      n_ur_bushvn = "n:ur:bushvn", "p.u., Line-Ground Voltage, Real Part"
      n_ur_buslvn = "n:ur:buslvn", "p.u., Line-Ground Voltage, Real Part"
      n_ui_bushv_A = "n:ui:bushv:A", "p.u., Line-Ground Voltage, Imaginary Part"
      n_ui_bushv_B = "n:ui:bushv:B", "p.u., Line-Ground Voltage, Imaginary Part"
      n_ui_bushv_C = "n:ui:bushv:C", "p.u., Line-Ground Voltage, Imaginary Part"
      n_ui_bushv_N = "n:ui:bushv:N", "p.u., Line-Ground Voltage, Imaginary Part"
      n_ui_buslv_A = "n:ui:buslv:A", "p.u., Line-Ground Voltage, Imaginary Part"
      n_ui_buslv_B = "n:ui:buslv:B", "p.u., Line-Ground Voltage, Imaginary Part"
      n_ui_buslv_C = "n:ui:buslv:C", "p.u., Line-Ground Voltage, Imaginary Part"
      n_ui_buslv_N = "n:ui:buslv:N", "p.u., Line-Ground Voltage, Imaginary Part"
      n_ui_bushvn = "n:ui:bushvn", "p.u., Line-Ground Voltage, Imaginary Part"
      n_ui_buslvn = "n:ui:buslvn", "p.u., Line-Ground Voltage, Imaginary Part"
      n_u_bushv_A = "n:u:bushv:A", "p.u., Line-Ground Voltage, Magnitude"
      n_u_bushv_B = "n:u:bushv:B", "p.u., Line-Ground Voltage, Magnitude"
      n_u_bushv_C = "n:u:bushv:C", "p.u., Line-Ground Voltage, Magnitude"
      n_u_bushv_N = "n:u:bushv:N", "p.u., Line-Ground Voltage, Magnitude"
      n_u_buslv_A = "n:u:buslv:A", "p.u., Line-Ground Voltage, Magnitude"
      n_u_buslv_B = "n:u:buslv:B", "p.u., Line-Ground Voltage, Magnitude"
      n_u_buslv_C = "n:u:buslv:C", "p.u., Line-Ground Voltage, Magnitude"
      n_u_buslv_N = "n:u:buslv:N", "p.u., Line-Ground Voltage, Magnitude"
      n_u_bushvn = "n:u:bushvn", "p.u., Line-Ground Voltage, Magnitude"
      n_u_buslvn = "n:u:buslvn", "p.u., Line-Ground Voltage, Magnitude"
      n_upc_bushv_A = "n:upc:bushv:A", "%, Line-Ground Voltage, Magnitude"
      n_upc_bushv_B = "n:upc:bushv:B", "%, Line-Ground Voltage, Magnitude"
      n_upc_bushv_C = "n:upc:bushv:C", "%, Line-Ground Voltage, Magnitude"
      n_upc_bushv_N = "n:upc:bushv:N", "%, Line-Ground Voltage, Magnitude"
      n_upc_buslv_A = "n:upc:buslv:A", "%, Line-Ground Voltage, Magnitude"
      n_upc_buslv_B = "n:upc:buslv:B", "%, Line-Ground Voltage, Magnitude"
      n_upc_buslv_C = "n:upc:buslv:C", "%, Line-Ground Voltage, Magnitude"
      n_upc_buslv_N = "n:upc:buslv:N", "%, Line-Ground Voltage, Magnitude"
      n_upc_bushvn = "n:upc:bushvn", "%, Line-Ground Voltage, Magnitude"
      n_upc_buslvn = "n:upc:buslvn", "%, Line-Ground Voltage, Magnitude"
      n_U_bushv_A = "n:U:bushv:A", "kV, Line-Ground Voltage, Magnitude"
      n_U_bushv_B = "n:U:bushv:B", "kV, Line-Ground Voltage, Magnitude"
      n_U_bushv_C = "n:U:bushv:C", "kV, Line-Ground Voltage, Magnitude"
      n_U_bushv_N = "n:U:bushv:N", "kV, Line-Ground Voltage, Magnitude"
      n_U_buslv_A = "n:U:buslv:A", "kV, Line-Ground Voltage, Magnitude"
      n_U_buslv_B = "n:U:buslv:B", "kV, Line-Ground Voltage, Magnitude"
      n_U_buslv_C = "n:U:buslv:C", "kV, Line-Ground Voltage, Magnitude"
      n_U_buslv_N = "n:U:buslv:N", "kV, Line-Ground Voltage, Magnitude"
      n_U_bushvn = "n:U:bushvn", "kV, Line-Ground Voltage, Magnitude"
      n_U_buslvn = "n:U:buslvn", "kV, Line-Ground Voltage, Magnitude"
      n_phiu_bushv_A = "n:phiu:bushv:A", "deg, Line-Ground Voltage, Angle"
      n_phiu_bushv_B = "n:phiu:bushv:B", "deg, Line-Ground Voltage, Angle"
      n_phiu_bushv_C = "n:phiu:bushv:C", "deg, Line-Ground Voltage, Angle"
      n_phiu_bushv_N = "n:phiu:bushv:N", "deg, Line-Ground Voltage, Angle"
      n_phiu_buslv_A = "n:phiu:buslv:A", "deg, Line-Ground Voltage, Angle"
      n_phiu_buslv_B = "n:phiu:buslv:B", "deg, Line-Ground Voltage, Angle"
      n_phiu_buslv_C = "n:phiu:buslv:C", "deg, Line-Ground Voltage, Angle"
      n_phiu_buslv_N = "n:phiu:buslv:N", "deg, Line-Ground Voltage, Angle"
      n_phiu_bushvn = "n:phiu:bushvn", "deg, Line-Ground Voltage, Angle"
      n_phiu_buslvn = "n:phiu:buslvn", "deg, Line-Ground Voltage, Angle"
      n_phiurel_bushv = "n:phiurel:bushv", "deg, Voltage, Relative Angle"
      n_phiurel_buslv = "n:phiurel:buslv", "deg, Voltage, Relative Angle"
      n_phiurel_bushvn = "n:phiurel:bushvn", "deg, Voltage, Relative Angle"
      n_phiurel_buslvn = "n:phiurel:buslvn", "deg, Voltage, Relative Angle"
      n_ul_bushv_A = "n:ul:bushv:A", "p.u., Line-Line Voltage, Magnitude"
      n_ul_bushv_B = "n:ul:bushv:B", "p.u., Line-Line Voltage, Magnitude"
      n_ul_bushv_C = "n:ul:bushv:C", "p.u., Line-Line Voltage, Magnitude"
      n_ul_bushv_N = "n:ul:bushv:N", "p.u., Line-Line Voltage, Magnitude"
      n_ul_buslv_A = "n:ul:buslv:A", "p.u., Line-Line Voltage, Magnitude"
      n_ul_buslv_B = "n:ul:buslv:B", "p.u., Line-Line Voltage, Magnitude"
      n_ul_buslv_C = "n:ul:buslv:C", "p.u., Line-Line Voltage, Magnitude"
      n_ul_buslv_N = "n:ul:buslv:N", "p.u., Line-Line Voltage, Magnitude"
      n_ul_bushvn = "n:ul:bushvn", "p.u., Line-Line Voltage, Magnitude"
      n_ul_buslvn = "n:ul:buslvn", "p.u., Line-Line Voltage, Magnitude"
      n_ulpc_bushv_A = "n:ulpc:bushv:A", "%, Line-Line Voltage, Magnitude"
      n_ulpc_bushv_B = "n:ulpc:bushv:B", "%, Line-Line Voltage, Magnitude"
      n_ulpc_bushv_C = "n:ulpc:bushv:C", "%, Line-Line Voltage, Magnitude"
      n_ulpc_bushv_N = "n:ulpc:bushv:N", "%, Line-Line Voltage, Magnitude"
      n_ulpc_buslv_A = "n:ulpc:buslv:A", "%, Line-Line Voltage, Magnitude"
      n_ulpc_buslv_B = "n:ulpc:buslv:B", "%, Line-Line Voltage, Magnitude"
      n_ulpc_buslv_C = "n:ulpc:buslv:C", "%, Line-Line Voltage, Magnitude"
      n_ulpc_buslv_N = "n:ulpc:buslv:N", "%, Line-Line Voltage, Magnitude"
      n_ulpc_bushvn = "n:ulpc:bushvn", "%, Line-Line Voltage, Magnitude"
      n_ulpc_buslvn = "n:ulpc:buslvn", "%, Line-Line Voltage, Magnitude"
      n_Ul_bushv_A = "n:Ul:bushv:A", "kV, Line-Line Voltage, Magnitude"
      n_Ul_bushv_B = "n:Ul:bushv:B", "kV, Line-Line Voltage, Magnitude"
      n_Ul_bushv_C = "n:Ul:bushv:C", "kV, Line-Line Voltage, Magnitude"
      n_Ul_bushv_N = "n:Ul:bushv:N", "kV, Line-Line Voltage, Magnitude"
      n_Ul_buslv_A = "n:Ul:buslv:A", "kV, Line-Line Voltage, Magnitude"
      n_Ul_buslv_B = "n:Ul:buslv:B", "kV, Line-Line Voltage, Magnitude"
      n_Ul_buslv_C = "n:Ul:buslv:C", "kV, Line-Line Voltage, Magnitude"
      n_Ul_buslv_N = "n:Ul:buslv:N", "kV, Line-Line Voltage, Magnitude"
      n_Ul_bushvn = "n:Ul:bushvn", "kV, Line-Line Voltage, Magnitude"
      n_Ul_buslvn = "n:Ul:buslvn", "kV, Line-Line Voltage, Magnitude"
      n_phiul_bushv_A = "n:phiul:bushv:A", "deg, Line-Line Voltage, Angle"
      n_phiul_bushv_B = "n:phiul:bushv:B", "deg, Line-Line Voltage, Angle"
      n_phiul_bushv_C = "n:phiul:bushv:C", "deg, Line-Line Voltage, Angle"
      n_phiul_bushv_N = "n:phiul:bushv:N", "deg, Line-Line Voltage, Angle"
      n_phiul_buslv_A = "n:phiul:buslv:A", "deg, Line-Line Voltage, Angle"
      n_phiul_buslv_B = "n:phiul:buslv:B", "deg, Line-Line Voltage, Angle"
      n_phiul_buslv_C = "n:phiul:buslv:C", "deg, Line-Line Voltage, Angle"
      n_phiul_buslv_N = "n:phiul:buslv:N", "deg, Line-Line Voltage, Angle"
      n_phiul_bushvn = "n:phiul:bushvn", "deg, Line-Line Voltage, Angle"
      n_phiul_buslvn = "n:phiul:buslvn", "deg, Line-Line Voltage, Angle"
      n_u0_bushv = "n:u0:bushv", "p.u., Zero-Sequence Voltage, Magnitude"
      n_u0_buslv = "n:u0:buslv", "p.u., Zero-Sequence Voltage, Magnitude"
      n_u0_bushvn = "n:u0:bushvn", "p.u., Zero-Sequence Voltage, Magnitude"
      n_u0_buslvn = "n:u0:buslvn", "p.u., Zero-Sequence Voltage, Magnitude"
      n_U0_bushv = "n:U0:bushv", "kV, Zero-Sequence Voltage, Magnitude"
      n_U0_buslv = "n:U0:buslv", "kV, Zero-Sequence Voltage, Magnitude"
      n_U0_bushvn = "n:U0:bushvn", "kV, Zero-Sequence Voltage, Magnitude"
      n_U0_buslvn = "n:U0:buslvn", "kV, Zero-Sequence Voltage, Magnitude"
      n_U0x3_bushv = "n:U0x3:bushv", "kV, 3*U0"
      n_U0x3_buslv = "n:U0x3:buslv", "kV, 3*U0"
      n_U0x3_bushvn = "n:U0x3:bushvn", "kV, 3*U0"
      n_U0x3_buslvn = "n:U0x3:buslvn", "kV, 3*U0"
      n_phiu0_bushv = "n:phiu0:bushv", "deg, Zero-Sequence Voltage, Angle"
      n_phiu0_buslv = "n:phiu0:buslv", "deg, Zero-Sequence Voltage, Angle"
      n_phiu0_bushvn = "n:phiu0:bushvn", "deg, Zero-Sequence Voltage, Angle"
      n_phiu0_buslvn = "n:phiu0:buslvn", "deg, Zero-Sequence Voltage, Angle"
      n_u1_bushv = "n:u1:bushv", "p.u., Positive-Sequence Voltage, Magnitude"
      n_u1_buslv = "n:u1:buslv", "p.u., Positive-Sequence Voltage, Magnitude"
      n_u1_bushvn = "n:u1:bushvn", "p.u., Positive-Sequence Voltage, Magnitude"
      n_u1_buslvn = "n:u1:buslvn", "p.u., Positive-Sequence Voltage, Magnitude"
      n_u1pc_bushv = "n:u1pc:bushv", "%, Positive-Sequence Voltage, Magnitude"
      n_u1pc_buslv = "n:u1pc:buslv", "%, Positive-Sequence Voltage, Magnitude"
      n_u1pc_bushvn = "n:u1pc:bushvn", "%, Positive-Sequence Voltage, Magnitude"
      n_u1pc_buslvn = "n:u1pc:buslvn", "%, Positive-Sequence Voltage, Magnitude"
      n_u1r_bushv = "n:u1r:bushv", "p.u., Positive-Sequence Voltage, Real Part"
      n_u1r_buslv = "n:u1r:buslv", "p.u., Positive-Sequence Voltage, Real Part"
      n_u1r_bushvn = "n:u1r:bushvn", "p.u., Positive-Sequence Voltage, Real Part"
      n_u1r_buslvn = "n:u1r:buslvn", "p.u., Positive-Sequence Voltage, Real Part"
      n_u1i_bushv = "n:u1i:bushv", "p.u., Positive-Sequence Voltage, Imaginary Part"
      n_u1i_buslv = "n:u1i:buslv", "p.u., Positive-Sequence Voltage, Imaginary Part"
      n_u1i_bushvn = "n:u1i:bushvn", "p.u., Positive-Sequence Voltage, Imaginary Part"
      n_u1i_buslvn = "n:u1i:buslvn", "p.u., Positive-Sequence Voltage, Imaginary Part"
      n_U1_bushv = "n:U1:bushv", "kV, Line-Ground Positive-Sequence Voltage, Magnitude"
      n_U1_buslv = "n:U1:buslv", "kV, Line-Ground Positive-Sequence Voltage, Magnitude"
      n_U1_bushvn = "n:U1:bushvn", "kV, Line-Ground Positive-Sequence Voltage, Magnitude"
      n_U1_buslvn = "n:U1:buslvn", "kV, Line-Ground Positive-Sequence Voltage, Magnitude"
      n_phiu1_bushv = "n:phiu1:bushv", "deg, Positive-Sequence Voltage, Angle"
      n_phiu1_buslv = "n:phiu1:buslv", "deg, Positive-Sequence Voltage, Angle"
      n_phiu1_bushvn = "n:phiu1:bushvn", "deg, Positive-Sequence Voltage, Angle"
      n_phiu1_buslvn = "n:phiu1:buslvn", "deg, Positive-Sequence Voltage, Angle"
      n_u2_bushv = "n:u2:bushv", "p.u., Negative-Sequence Voltage, Magnitude"
      n_u2_buslv = "n:u2:buslv", "p.u., Negative-Sequence Voltage, Magnitude"
      n_u2_bushvn = "n:u2:bushvn", "p.u., Negative-Sequence Voltage, Magnitude"
      n_u2_buslvn = "n:u2:buslvn", "p.u., Negative-Sequence Voltage, Magnitude"
      n_U2_bushv = "n:U2:bushv", "kV, Line-Ground Negative-Sequence Voltage, Magnitude"
      n_U2_buslv = "n:U2:buslv", "kV, Line-Ground Negative-Sequence Voltage, Magnitude"
      n_U2_bushvn = "n:U2:bushvn", "kV, Line-Ground Negative-Sequence Voltage, Magnitude"
      n_U2_buslvn = "n:U2:buslvn", "kV, Line-Ground Negative-Sequence Voltage, Magnitude"
      n_phiu2_bushv = "n:phiu2:bushv", "deg, Negative-Sequence Voltage, Angle"
      n_phiu2_buslv = "n:phiu2:buslv", "deg, Negative-Sequence Voltage, Angle"
      n_phiu2_bushvn = "n:phiu2:bushvn", "deg, Negative-Sequence Voltage, Angle"
      n_phiu2_buslvn = "n:phiu2:buslvn", "deg, Negative-Sequence Voltage, Angle"
      n_U1l_bushv = "n:U1l:bushv", "kV, Line-Line Positive-Sequence Voltage, Magnitude"
      n_U1l_buslv = "n:U1l:buslv", "kV, Line-Line Positive-Sequence Voltage, Magnitude"
      n_U1l_bushvn = "n:U1l:bushvn", "kV, Line-Line Positive-Sequence Voltage, Magnitude"
      n_U1l_buslvn = "n:U1l:buslvn", "kV, Line-Line Positive-Sequence Voltage, Magnitude"
      n_U2l_bushv = "n:U2l:bushv", "kV, Line-Line Negative-Sequence Voltage, Magnitude"
      n_U2l_buslv = "n:U2l:buslv", "kV, Line-Line Negative-Sequence Voltage, Magnitude"
      n_U2l_bushvn = "n:U2l:bushvn", "kV, Line-Line Negative-Sequence Voltage, Magnitude"
      n_U2l_buslvn = "n:U2l:buslvn", "kV, Line-Line Negative-Sequence Voltage, Magnitude"
      n_uln_bushv_A = "n:uln:bushv:A", "p.u., Line-Neutral Voltage, Magnitude"
      n_uln_bushv_B = "n:uln:bushv:B", "p.u., Line-Neutral Voltage, Magnitude"
      n_uln_bushv_C = "n:uln:bushv:C", "p.u., Line-Neutral Voltage, Magnitude"
      n_uln_bushv_N = "n:uln:bushv:N", "p.u., Line-Neutral Voltage, Magnitude"
      n_uln_buslv_A = "n:uln:buslv:A", "p.u., Line-Neutral Voltage, Magnitude"
      n_uln_buslv_B = "n:uln:buslv:B", "p.u., Line-Neutral Voltage, Magnitude"
      n_uln_buslv_C = "n:uln:buslv:C", "p.u., Line-Neutral Voltage, Magnitude"
      n_uln_buslv_N = "n:uln:buslv:N", "p.u., Line-Neutral Voltage, Magnitude"
      n_uln_bushvn = "n:uln:bushvn", "p.u., Line-Neutral Voltage, Magnitude"
      n_uln_buslvn = "n:uln:buslvn", "p.u., Line-Neutral Voltage, Magnitude"
      n_Uln_bushv_A = "n:Uln:bushv:A", "kV, Line-Neutral Voltage, Magnitude"
      n_Uln_bushv_B = "n:Uln:bushv:B", "kV, Line-Neutral Voltage, Magnitude"
      n_Uln_bushv_C = "n:Uln:bushv:C", "kV, Line-Neutral Voltage, Magnitude"
      n_Uln_bushv_N = "n:Uln:bushv:N", "kV, Line-Neutral Voltage, Magnitude"
      n_Uln_buslv_A = "n:Uln:buslv:A", "kV, Line-Neutral Voltage, Magnitude"
      n_Uln_buslv_B = "n:Uln:buslv:B", "kV, Line-Neutral Voltage, Magnitude"
      n_Uln_buslv_C = "n:Uln:buslv:C", "kV, Line-Neutral Voltage, Magnitude"
      n_Uln_buslv_N = "n:Uln:buslv:N", "kV, Line-Neutral Voltage, Magnitude"
      n_Uln_bushvn = "n:Uln:bushvn", "kV, Line-Neutral Voltage, Magnitude"
      n_Uln_buslvn = "n:Uln:buslvn", "kV, Line-Neutral Voltage, Magnitude"
      n_phiuln_bushv_A = "n:phiuln:bushv:A", "deg, Line-Neutral Voltage, Angle"
      n_phiuln_bushv_B = "n:phiuln:bushv:B", "deg, Line-Neutral Voltage, Angle"
      n_phiuln_bushv_C = "n:phiuln:bushv:C", "deg, Line-Neutral Voltage, Angle"
      n_phiuln_bushv_N = "n:phiuln:bushv:N", "deg, Line-Neutral Voltage, Angle"
      n_phiuln_buslv_A = "n:phiuln:buslv:A", "deg, Line-Neutral Voltage, Angle"
      n_phiuln_buslv_B = "n:phiuln:buslv:B", "deg, Line-Neutral Voltage, Angle"
      n_phiuln_buslv_C = "n:phiuln:buslv:C", "deg, Line-Neutral Voltage, Angle"
      n_phiuln_buslv_N = "n:phiuln:buslv:N", "deg, Line-Neutral Voltage, Angle"
      n_phiuln_bushvn = "n:phiuln:bushvn", "deg, Line-Neutral Voltage, Angle"
      n_phiuln_buslvn = "n:phiuln:buslvn", "deg, Line-Neutral Voltage, Angle"
      n_un_bushv = "n:un:bushv", "p.u., Neutral-Ground Voltage, Magnitude"
      n_un_buslv = "n:un:buslv", "p.u., Neutral-Ground Voltage, Magnitude"
      n_un_bushvn = "n:un:bushvn", "p.u., Neutral-Ground Voltage, Magnitude"
      n_un_buslvn = "n:un:buslvn", "p.u., Neutral-Ground Voltage, Magnitude"
      n_Un_bushv = "n:Un:bushv", "kV, Neutral-Ground Voltage, Magnitude"
      n_Un_buslv = "n:Un:buslv", "kV, Neutral-Ground Voltage, Magnitude"
      n_Un_bushvn = "n:Un:bushvn", "kV, Neutral-Ground Voltage, Magnitude"
      n_Un_buslvn = "n:Un:buslvn", "kV, Neutral-Ground Voltage, Magnitude"
      n_phiun_bushv = "n:phiun:bushv", "deg, Neutral-Ground Voltage, Angle"
      n_phiun_buslv = "n:phiun:buslv", "deg, Neutral-Ground Voltage, Angle"
      n_phiun_bushvn = "n:phiun:bushvn", "deg, Neutral-Ground Voltage, Angle"
      n_phiun_buslvn = "n:phiun:buslvn", "deg, Neutral-Ground Voltage, Angle"
      n_upht_bushv_A = "n:upht:bushv:A", "p.u., Phase Technology dependent Voltage, Magnitude"
      n_upht_bushv_B = "n:upht:bushv:B", "p.u., Phase Technology dependent Voltage, Magnitude"
      n_upht_bushv_C = "n:upht:bushv:C", "p.u., Phase Technology dependent Voltage, Magnitude"
      n_upht_bushv_N = "n:upht:bushv:N", "p.u., Phase Technology dependent Voltage, Magnitude"
      n_upht_buslv_A = "n:upht:buslv:A", "p.u., Phase Technology dependent Voltage, Magnitude"
      n_upht_buslv_B = "n:upht:buslv:B", "p.u., Phase Technology dependent Voltage, Magnitude"
      n_upht_buslv_C = "n:upht:buslv:C", "p.u., Phase Technology dependent Voltage, Magnitude"
      n_upht_buslv_N = "n:upht:buslv:N", "p.u., Phase Technology dependent Voltage, Magnitude"
      n_upht_bushvn = "n:upht:bushvn", "p.u., Phase Technology dependent Voltage, Magnitude"
      n_upht_buslvn = "n:upht:buslvn", "p.u., Phase Technology dependent Voltage, Magnitude"
      n_Upht_bushv_A = "n:Upht:bushv:A", "kV, Phase Technology dependent Voltage, Magnitude"
      n_Upht_bushv_B = "n:Upht:bushv:B", "kV, Phase Technology dependent Voltage, Magnitude"
      n_Upht_bushv_C = "n:Upht:bushv:C", "kV, Phase Technology dependent Voltage, Magnitude"
      n_Upht_bushv_N = "n:Upht:bushv:N", "kV, Phase Technology dependent Voltage, Magnitude"
      n_Upht_buslv_A = "n:Upht:buslv:A", "kV, Phase Technology dependent Voltage, Magnitude"
      n_Upht_buslv_B = "n:Upht:buslv:B", "kV, Phase Technology dependent Voltage, Magnitude"
      n_Upht_buslv_C = "n:Upht:buslv:C", "kV, Phase Technology dependent Voltage, Magnitude"
      n_Upht_buslv_N = "n:Upht:buslv:N", "kV, Phase Technology dependent Voltage, Magnitude"
      n_Upht_bushvn = "n:Upht:bushvn", "kV, Phase Technology dependent Voltage, Magnitude"
      n_Upht_buslvn = "n:Upht:buslvn", "kV, Phase Technology dependent Voltage, Magnitude"
      n_um_bushv = "n:um:bushv", "p.u., Average-Voltage, Magnitude"
      n_um_buslv = "n:um:buslv", "p.u., Average-Voltage, Magnitude"
      n_um_bushvn = "n:um:bushvn", "p.u., Average-Voltage, Magnitude"
      n_um_buslvn = "n:um:buslvn", "p.u., Average-Voltage, Magnitude"
      n_Um_bushv = "n:Um:bushv", "kV, Average-Voltage, Magnitude"
      n_Um_buslv = "n:Um:buslv", "kV, Average-Voltage, Magnitude"
      n_Um_bushvn = "n:Um:bushvn", "kV, Average-Voltage, Magnitude"
      n_Um_buslvn = "n:Um:buslvn", "kV, Average-Voltage, Magnitude"
      n_ishcr_bushv_A = "n:ishcr:bushv:A", "p.u., Short-Circuit Current, real"
      n_ishcr_bushv_B = "n:ishcr:bushv:B", "p.u., Short-Circuit Current, real"
      n_ishcr_bushv_C = "n:ishcr:bushv:C", "p.u., Short-Circuit Current, real"
      n_ishcr_bushv_N = "n:ishcr:bushv:N", "p.u., Short-Circuit Current, real"
      n_ishcr_buslv_A = "n:ishcr:buslv:A", "p.u., Short-Circuit Current, real"
      n_ishcr_buslv_B = "n:ishcr:buslv:B", "p.u., Short-Circuit Current, real"
      n_ishcr_buslv_C = "n:ishcr:buslv:C", "p.u., Short-Circuit Current, real"
      n_ishcr_buslv_N = "n:ishcr:buslv:N", "p.u., Short-Circuit Current, real"
      n_ishcr_bushvn = "n:ishcr:bushvn", "p.u., Short-Circuit Current, real"
      n_ishcr_buslvn = "n:ishcr:buslvn", "p.u., Short-Circuit Current, real"
      n_ishci_bushv_A = "n:ishci:bushv:A", "p.u., Short-Circuit Current, imag"
      n_ishci_bushv_B = "n:ishci:bushv:B", "p.u., Short-Circuit Current, imag"
      n_ishci_bushv_C = "n:ishci:bushv:C", "p.u., Short-Circuit Current, imag"
      n_ishci_bushv_N = "n:ishci:bushv:N", "p.u., Short-Circuit Current, imag"
      n_ishci_buslv_A = "n:ishci:buslv:A", "p.u., Short-Circuit Current, imag"
      n_ishci_buslv_B = "n:ishci:buslv:B", "p.u., Short-Circuit Current, imag"
      n_ishci_buslv_C = "n:ishci:buslv:C", "p.u., Short-Circuit Current, imag"
      n_ishci_buslv_N = "n:ishci:buslv:N", "p.u., Short-Circuit Current, imag"
      n_ishci_bushvn = "n:ishci:bushvn", "p.u., Short-Circuit Current, imag"
      n_ishci_buslvn = "n:ishci:buslvn", "p.u., Short-Circuit Current, imag"
      n_ishc_bushv_A = "n:ishc:bushv:A", "p.u., Short-Circuit Current, magnitude"
      n_ishc_bushv_B = "n:ishc:bushv:B", "p.u., Short-Circuit Current, magnitude"
      n_ishc_bushv_C = "n:ishc:bushv:C", "p.u., Short-Circuit Current, magnitude"
      n_ishc_bushv_N = "n:ishc:bushv:N", "p.u., Short-Circuit Current, magnitude"
      n_ishc_buslv_A = "n:ishc:buslv:A", "p.u., Short-Circuit Current, magnitude"
      n_ishc_buslv_B = "n:ishc:buslv:B", "p.u., Short-Circuit Current, magnitude"
      n_ishc_buslv_C = "n:ishc:buslv:C", "p.u., Short-Circuit Current, magnitude"
      n_ishc_buslv_N = "n:ishc:buslv:N", "p.u., Short-Circuit Current, magnitude"
      n_ishc_bushvn = "n:ishc:bushvn", "p.u., Short-Circuit Current, magnitude"
      n_ishc_buslvn = "n:ishc:buslvn", "p.u., Short-Circuit Current, magnitude"
      n_Ishc_bushv_A = "n:Ishc:bushv:A", "kA, Short-Circuit Current, magnitude"
      n_Ishc_bushv_B = "n:Ishc:bushv:B", "kA, Short-Circuit Current, magnitude"
      n_Ishc_bushv_C = "n:Ishc:bushv:C", "kA, Short-Circuit Current, magnitude"
      n_Ishc_bushv_N = "n:Ishc:bushv:N", "kA, Short-Circuit Current, magnitude"
      n_Ishc_buslv_A = "n:Ishc:buslv:A", "kA, Short-Circuit Current, magnitude"
      n_Ishc_buslv_B = "n:Ishc:buslv:B", "kA, Short-Circuit Current, magnitude"
      n_Ishc_buslv_C = "n:Ishc:buslv:C", "kA, Short-Circuit Current, magnitude"
      n_Ishc_buslv_N = "n:Ishc:buslv:N", "kA, Short-Circuit Current, magnitude"
      n_Ishc_bushvn = "n:Ishc:bushvn", "kA, Short-Circuit Current, magnitude"
      n_Ishc_buslvn = "n:Ishc:buslvn", "kA, Short-Circuit Current, magnitude"
      n_inshc_bushv = "n:inshc:bushv", "p.u., Neutral Short-Circuit Current, magnitude"
      n_inshc_buslv = "n:inshc:buslv", "p.u., Neutral Short-Circuit Current, magnitude"
      n_inshc_bushvn = "n:inshc:bushvn", "p.u., Neutral Short-Circuit Current, magnitude"
      n_inshc_buslvn = "n:inshc:buslvn", "p.u., Neutral Short-Circuit Current, magnitude"
      n_Inshc_bushv = "n:Inshc:bushv", "kA, Neutral Short-Circuit Current, magnitude"
      n_Inshc_buslv = "n:Inshc:buslv", "kA, Neutral Short-Circuit Current, magnitude"
      n_Inshc_bushvn = "n:Inshc:bushvn", "kA, Neutral Short-Circuit Current, magnitude"
      n_Inshc_buslvn = "n:Inshc:buslvn", "kA, Neutral Short-Circuit Current, magnitude"
      n_fe_bushv = "n:fe:bushv", "p.u., Electrical Frequency"
      n_fe_buslv = "n:fe:buslv", "p.u., Electrical Frequency"
      n_fe_bushvn = "n:fe:bushvn", "p.u., Electrical Frequency"
      n_fe_buslvn = "n:fe:buslvn", "p.u., Electrical Frequency"
      n_frdev_bushv = "n:frdev:bushv", "Hz, average frequency"
      n_frdev_buslv = "n:frdev:buslv", "Hz, average frequency"
      n_frdev_bushvn = "n:frdev:bushvn", "Hz, average frequency"
      n_frdev_buslvn = "n:frdev:buslvn", "Hz, average frequency"
      n_dfehz_bushv = "n:dfehz:bushv", "Hz, Deviation of the El. Frequency"
      n_dfehz_buslv = "n:dfehz:buslv", "Hz, Deviation of the El. Frequency"
      n_dfehz_bushvn = "n:dfehz:bushvn", "Hz, Deviation of the El. Frequency"
      n_dfehz_buslvn = "n:dfehz:buslvn", "Hz, Deviation of the El. Frequency"
      n_dfedt_bushv = "n:dfedt:bushv", "p.u./s, Derivative of El. Frequency"
      n_dfedt_buslv = "n:dfedt:buslv", "p.u./s, Derivative of El. Frequency"
      n_dfedt_bushvn = "n:dfedt:bushvn", "p.u./s, Derivative of El. Frequency"
      n_dfedt_buslvn = "n:dfedt:buslvn", "p.u./s, Derivative of El. Frequency"
      n_fehz_bushv = "n:fehz:bushv", "Hz, Electrical Frequency"
      n_fehz_buslv = "n:fehz:buslv", "Hz, Electrical Frequency"
      n_fehz_bushvn = "n:fehz:bushvn", "Hz, Electrical Frequency"
      n_fehz_buslvn = "n:fehz:buslvn", "Hz, Electrical Frequency"
      n_frnom_bushv = "n:frnom:bushv", "Hz, Nominal Frequency"
      n_frnom_buslv = "n:frnom:buslv", "Hz, Nominal Frequency"
      n_frnom_bushvn = "n:frnom:bushvn", "Hz, Nominal Frequency"
      n_frnom_buslvn = "n:frnom:buslvn", "Hz, Nominal Frequency"
      s_nntapin = "s:nntapin", "IN, Tap 1, Controller Input continuous, double"
      s_nntapin2 = "s:nntapin2", "IN, Tap 2, Controller Input continuous, double"
      c_nntap = "c:nntap", "Tap 1, Current Position"
      c_nntap_int = "c:nntap_int", "Tap 1, relative Position"
      c_ntpmn = "c:ntpmn", "Tap 1, Min. Position"
      c_ntpmx = "c:ntpmx", "Tap 1, Max. Position"
      c_dut = "c:dut", "Tap 1: Voltage/Tap"
      c_phitrrd = "c:phitrrd", "rad, Tap 1:, Phase/Tap"
      c_nt2agrd = "c:nt2agrd", "rad, Phase Angle"
      c_tap_side = "c:tap_side", "0"
      c_nntap2 = "c:nntap2", "Tap 2, Current Position"
      c_ntpmn2 = "c:ntpmn2", "Tap 2, Min. Position"
      c_ntpmx2 = "c:ntpmx2", "Tap 2, Max. Position"
      c_dut2 = "c:dut2", "Tap 2: Voltage/Tap"
      c_phitrrd2 = "c:phitrrd2", "rad, Tap 2:, Phase/Tap"
      c_tap_side2 = "c:tap_side2", "0"
      c_ratfac = "c:ratfac", "Rating Factor"
      c_snom = "c:snom", "MVA, Rated Power"
      c_zshv_r = "c:zshv:r", "p.u., Leakage Impedance (HV-Side), Real Part"
      c_zshv_i = "c:zshv:i", "p.u., Leakage Impedance (HV-Side), Imaginary Part"
      c_zshv_phi = "c:zshv:phi", "deg, Leakage Impedance (HV-Side), Angle"
      c_zshv_mag = "c:zshv:mag", "p.u., Leakage Impedance (HV-Side), Magnitude"
      c_zshv_phirad = "c:zshv:phirad", "rad, Leakage Impedance (HV-Side), Angle"
      c_zslv_r = "c:zslv:r", "p.u., Leakage Impedance (LV-Side), Real Part"
      c_zslv_i = "c:zslv:i", "p.u., Leakage Impedance (LV-Side), Imaginary Part"
      c_zslv_phi = "c:zslv:phi", "deg, Leakage Impedance (LV-Side), Angle"
      c_zslv_mag = "c:zslv:mag", "p.u., Leakage Impedance (LV-Side), Magnitude"
      c_zslv_phirad = "c:zslv:phirad", "rad, Leakage Impedance (LV-Side), Angle"
      c_ym_r = "c:ym:r", "p.u., Magnetising Admittance, Real Part"
      c_ym_i = "c:ym:i", "p.u., Magnetising Admittance, Imaginary Part"
      c_ym_phi = "c:ym:phi", "deg, Magnetising Admittance, Angle"
      c_ym_mag = "c:ym:mag", "p.u., Magnetising Admittance, Magnitude"
      c_ym_phirad = "c:ym:phirad", "rad, Magnetising Admittance, Angle"
      c_thv_r = "c:thv:r", "Complex Winding Ratio HV, Real Part"
      c_thv_i = "c:thv:i", "Complex Winding Ratio HV, Imaginary Part"
      c_thv_phi = "c:thv:phi", "Complex Winding Ratio HV, Angle"
      c_thv_mag = "c:thv:mag", "Complex Winding Ratio HV, Magnitude"
      c_thv_phirad = "c:thv:phirad", "Complex Winding Ratio HV, Angle"
      c_tlv_r = "c:tlv:r", "Complex Winding Ratio LV, Real Part"
      c_tlv_i = "c:tlv:i", "Complex Winding Ratio LV, Imaginary Part"
      c_tlv_phi = "c:tlv:phi", "Complex Winding Ratio LV, Angle"
      c_tlv_mag = "c:tlv:mag", "Complex Winding Ratio LV, Magnitude"
      c_tlv_phirad = "c:tlv:phirad", "Complex Winding Ratio LV, Angle"
      c_loading_h = "c:loading_h", "%, Loading (HV)"
      c_loading_l = "c:loading_l", "%, Loading (LV)"
      c_loading = "c:loading", "%, Loading"
      c_i_ea_h = "c:i_ea_h", "Flag: HV-Side grounded"
      c_i_ea_l = "c:i_ea_l", "Flag: LV-Side grounded"
      c_i_star_h = "c:i_star_h", "Flag: HV-Side Y"
      c_i_star_l = "c:i_star_l", "Flag: LV-Side Y"
      c_zs0hv_r = "c:zs0hv:r", "p.u., Zero sequence short circuit imp. (HV), Real Part"
      c_zs0hv_i = "c:zs0hv:i", "p.u., Zero sequence short circuit imp. (HV), Imaginary Part"
      c_zs0hv_phi = "c:zs0hv:phi", "deg, Zero sequence short circuit imp. (HV), Angle"
      c_zs0hv_mag = "c:zs0hv:mag", "p.u., Zero sequence short circuit imp. (HV), Magnitude"
      c_zs0hv_phirad = "c:zs0hv:phirad", "rad, Zero sequence short circuit imp. (HV), Angle"
      c_zs0lv_r = "c:zs0lv:r", "p.u., Zero sequence short circuit imp. (LV), Real Part"
      c_zs0lv_i = "c:zs0lv:i", "p.u., Zero sequence short circuit imp. (LV), Imaginary Part"
      c_zs0lv_phi = "c:zs0lv:phi", "deg, Zero sequence short circuit imp. (LV), Angle"
      c_zs0lv_mag = "c:zs0lv:mag", "p.u., Zero sequence short circuit imp. (LV), Magnitude"
      c_zs0lv_phirad = "c:zs0lv:phirad", "rad, Zero sequence short circuit imp. (LV), Angle"
      c_ym0_r = "c:ym0:r", "p.u., Zero Sequence Magnetising Admittance, Real Part"
      c_ym0_i = "c:ym0:i", "p.u., Zero Sequence Magnetising Admittance, Imaginary Part"
      c_ym0_phi = "c:ym0:phi", "deg, Zero Sequence Magnetising Admittance, Angle"
      c_ym0_mag = "c:ym0:mag", "p.u., Zero Sequence Magnetising Admittance, Magnitude"
      c_ym0_phirad = "c:ym0:phirad", "rad, Zero Sequence Magnetising Admittance, Angle"
      c_w_0 = "c:w_0", "Nominal winding ratio"
      c_Ustar_h = "c:Ustar_h", "kV, Star-Point Voltage (HV)"
      c_Ustar_l = "c:Ustar_l", "kV, Star-Point Voltage (LV)"
      c_Istar_h = "c:Istar_h", "kA, Star-Point Current (HV)"
      c_Istar_l = "c:Istar_l", "kA, Star-Point Current (LV)"
      c_iAutoTap = "c:iAutoTap", "Controller Mode: 0 = Manual, 1=Auto"
      c_Pmeas = "c:Pmeas", "MW, Measured active power"
      c_du_ldc = "c:du_ldc", "p.u., LDC Input"
      c_ctrl_set = "c:ctrl_set", "Setpoint"
      c_Inom_HV = "c:Inom_HV", "kA, Nominal Current (HV-Side)"
      c_Inom_LV = "c:Inom_LV", "kA, Nominal Current (LV-Side)"
      c_isat = "c:isat", "Saturated Model"
      e_dutap = "e:dutap", "%, Additional Voltage per Tap"
      e_dphitap = "e:dphitap", "deg, Additional Angle per Tap"
      e_phitr = "e:phitr", "deg, Phase of du"
      e_nntap = "e:nntap", "Tap Position"
      e_iTaps = "e:iTaps", "According to Measurement Report"
      e_dutap2 = "e:dutap2", "%, Additional Voltage per Tap"
      e_dphitap2 = "e:dphitap2", "deg, Additional Angle per Tap"
      e_phitr2 = "e:phitr2", "deg, Phase of du"
      e_nntap2 = "e:nntap2", "Tap Position"
      e_iMeasLoc = "e:iMeasLoc", "Measured at"
      e_mTaps = "e:mTaps", "Measurement Report"
      e_iIntTapCtrl = "e:iIntTapCtrl", "Use Integrated Tap Controller"
      e_iopt_hf = "e:iopt_hf", "Consider Capacitances"
      e_Cg_h = "e:Cg_h", "uF, Capacitance HV-Ground"
      e_Cg_l = "e:Cg_l", "uF, Capacitance LV-Ground"
      e_Cc1_hl = "e:Cc1_hl", "uF, Capacitance HV-LV, 1-Sequence"
      e_Cc0_hl = "e:Cc0_hl", "uF, Capacitance HV-LV, 0-Sequence"
      e_iAstabint = "e:iAstabint", "A-stable integration algorithm"
      t_curmg = "t:curmg", "%, No Load Current"
      t_pfe = "t:pfe", "kW, No Load Losses"
      t_zx0hl_n = "t:zx0hl_n", "Mag. Impedance/uk0"
      t_rtox0_n = "t:rtox0_n", "Mag. R/X"
      t_itrdl = "t:itrdl", "x,Pos.Seq. HV-Side"
      t_itrdl_lv = "t:itrdl_lv", "x,Pos.Seq. LV-Side"
      t_itrdr = "t:itrdr", "r,Pos.Seq. HV-Side"
      t_itrdr_lv = "t:itrdr_lv", "r,Pos.Seq. LV-Side"
      t_zx0hl_h = "t:zx0hl_h", "z, Zero Sequ. HV-Side"
      t_zx0hl_l = "t:zx0hl_l", "z, Zero Sequ. LV-Side"
      t_itapch = "t:itapch", "Tap Changer 1"
      t_tapchtype = "t:tapchtype", "Type"
      t_tap_side = "t:tap_side", "at Side"
      t_dutap = "t:dutap", "%, Additional Voltage per Tap"
      t_dphitap = "t:dphitap", "deg, Additional Angle per Tap"
      t_phitr = "t:phitr", "deg, Phase of du"
      t_nntap0 = "t:nntap0", "Neutral Position"
      t_ntpmn = "t:ntpmn", "Minimum Position"
      t_ntpmx = "t:ntpmx", "Maximum Position"
      t_itapch2 = "t:itapch2", "Tap Changer 2"
      t_tapchtype2 = "t:tapchtype2", "Type"
      t_tap_side2 = "t:tap_side2", "at Side"
      t_dutap2 = "t:dutap2", "%, Additional Voltage per Tap"
      t_dphitap2 = "t:dphitap2", "deg, Additional Angle per Tap"
      t_phitr2 = "t:phitr2", "deg, Phase of du"
      t_nntap02 = "t:nntap02", "Neutral Position"
      t_ntpmn2 = "t:ntpmn2", "Minimum Position"
      t_ntpmx2 = "t:ntpmx2", "Maximum Position"
      t_itapzdep = "t:itapzdep", "Tap dependent impedance"
      t_uktr = "t:uktr", "%, Short-Circuit Voltage uk"
      t_uktmn = "t:uktmn", "%, uk (min. tap)"
      t_uktmx = "t:uktmx", "%, uk (max. tap)"
      t_x1pu = "t:x1pu", "p.u., Reactance x1"
      t_x1putmn = "t:x1putmn", "p.u., x1 (min. tap)"
      t_x1putmx = "t:x1putmx", "p.u., x1 (max. tap)"
      t_pcutr = "t:pcutr", "kW, Copper Losses"
      t_uktrr = "t:uktrr", "%, SHC-Voltage (Re(uk)) ukr"
      t_xtor = "t:xtor", "Ratio X/R"
      t_r1pu = "t:r1pu", "p.u., Resistance r1"
      t_pcutmn = "t:pcutmn", "kW, Pcu (min. tap)"
      t_ukrtmn = "t:ukrtmn", "%, Re(uk) (min. tap)"
      t_xtortmn = "t:xtortmn", "X/R (min. tap)"
      t_r1putmn = "t:r1putmn", "p.u., r1 (min. tap)"
      t_pcutmx = "t:pcutmx", "kW, Pcu (max. tap)"
      t_ukrtmx = "t:ukrtmx", "%, Re(uk) (max. tap)"
      t_xtortmx = "t:xtortmx", "X/R (max. tap)"
      t_r1putmx = "t:r1putmx", "p.u., r1 (max. tap)"
      t_uk0tr = "t:uk0tr", "%, Short-Circuit Voltage uk0"
      t_uk0tmn = "t:uk0tmn", "%, uk0 (min. tap)"
      t_uk0tmx = "t:uk0tmx", "%, uk0 (max. tap)"
      t_x0pu = "t:x0pu", "p.u., Reactance x0"
      t_x0putmn = "t:x0putmn", "p.u., x0 (min. tap)"
      t_x0putmx = "t:x0putmx", "p.u., x0 (max. tap)"
      t_ur0tr = "t:ur0tr", "%, SHC-Voltage (Re(uk0)) uk0r"
      t_x0tor0 = "t:x0tor0", "Ratio X0/R0"
      t_r0pu = "t:r0pu", "p.u., Resistance r0"
      t_uk0rtmn = "t:uk0rtmn", "%, Re(uk0) (min. tap)"
      t_x0tor0tmn = "t:x0tor0tmn", "X0/R0 (min. tap)"
      t_r0putmn = "t:r0putmn", "p.u., r0 (min. tap)"
      t_uk0rtmx = "t:uk0rtmx", "%, Re(uk0) (max. tap)"
      t_x0tor0tmx = "t:x0tor0tmx", "X0/R0 (max. tap)"
      t_r0putmx = "t:r0putmx", "p.u., r0 (max. tap)"
      t_iTapLoading = "t:iTapLoading", "Configuration"
      t_itrldf = "t:itrldf", "Type"
      t_satcue = "t:satcue", "%, Current (RMS)"
      t_satvol = "t:satvol", "p.u., Voltage (RMS)"
      t_iIntPola = "t:iIntPola", "Interpolation"
      t_smoothf = "t:smoothf", "%, Smoothing Factor"

    class ElmTr3(Enum):
      _init_ = 'value __doc__'

      m_u0_bushv = "m:u0:bushv", "p.u., Zero-Sequence-Voltage, Magnitude"
      m_u0_busmv = "m:u0:busmv", "p.u., Zero-Sequence-Voltage, Magnitude"
      m_u0_buslv = "m:u0:buslv", "p.u., Zero-Sequence-Voltage, Magnitude"
      m_u0_bushvn = "m:u0:bushvn", "p.u., Zero-Sequence-Voltage, Magnitude"
      m_u0_busmvn = "m:u0:busmvn", "p.u., Zero-Sequence-Voltage, Magnitude"
      m_u0_buslvn = "m:u0:buslvn", "p.u., Zero-Sequence-Voltage, Magnitude"
      m_phiu0_bushv = "m:phiu0:bushv", "deg, Zero-Sequence-Voltage, Angle"
      m_phiu0_busmv = "m:phiu0:busmv", "deg, Zero-Sequence-Voltage, Angle"
      m_phiu0_buslv = "m:phiu0:buslv", "deg, Zero-Sequence-Voltage, Angle"
      m_phiu0_bushvn = "m:phiu0:bushvn", "deg, Zero-Sequence-Voltage, Angle"
      m_phiu0_busmvn = "m:phiu0:busmvn", "deg, Zero-Sequence-Voltage, Angle"
      m_phiu0_buslvn = "m:phiu0:buslvn", "deg, Zero-Sequence-Voltage, Angle"
      m_u1r_bushv = "m:u1r:bushv", "p.u., Positive-Sequence-Voltage, Real Part"
      m_u1r_busmv = "m:u1r:busmv", "p.u., Positive-Sequence-Voltage, Real Part"
      m_u1r_buslv = "m:u1r:buslv", "p.u., Positive-Sequence-Voltage, Real Part"
      m_u1r_bushvn = "m:u1r:bushvn", "p.u., Positive-Sequence-Voltage, Real Part"
      m_u1r_busmvn = "m:u1r:busmvn", "p.u., Positive-Sequence-Voltage, Real Part"
      m_u1r_buslvn = "m:u1r:buslvn", "p.u., Positive-Sequence-Voltage, Real Part"
      m_u1i_bushv = "m:u1i:bushv", "p.u., Positive-Sequence-Voltage, Imaginary Part"
      m_u1i_busmv = "m:u1i:busmv", "p.u., Positive-Sequence-Voltage, Imaginary Part"
      m_u1i_buslv = "m:u1i:buslv", "p.u., Positive-Sequence-Voltage, Imaginary Part"
      m_u1i_bushvn = "m:u1i:bushvn", "p.u., Positive-Sequence-Voltage, Imaginary Part"
      m_u1i_busmvn = "m:u1i:busmvn", "p.u., Positive-Sequence-Voltage, Imaginary Part"
      m_u1i_buslvn = "m:u1i:buslvn", "p.u., Positive-Sequence-Voltage, Imaginary Part"
      m_u1_bushv = "m:u1:bushv", "p.u., Positive-Sequence-Voltage, Magnitude"
      m_u1_busmv = "m:u1:busmv", "p.u., Positive-Sequence-Voltage, Magnitude"
      m_u1_buslv = "m:u1:buslv", "p.u., Positive-Sequence-Voltage, Magnitude"
      m_u1_bushvn = "m:u1:bushvn", "p.u., Positive-Sequence-Voltage, Magnitude"
      m_u1_busmvn = "m:u1:busmvn", "p.u., Positive-Sequence-Voltage, Magnitude"
      m_u1_buslvn = "m:u1:buslvn", "p.u., Positive-Sequence-Voltage, Magnitude"
      m_U1_bushv = "m:U1:bushv", "kV, Line-Ground Positive-Sequence-Voltage, Magnitude"
      m_U1_busmv = "m:U1:busmv", "kV, Line-Ground Positive-Sequence-Voltage, Magnitude"
      m_U1_buslv = "m:U1:buslv", "kV, Line-Ground Positive-Sequence-Voltage, Magnitude"
      m_U1_bushvn = "m:U1:bushvn", "kV, Line-Ground Positive-Sequence-Voltage, Magnitude"
      m_U1_busmvn = "m:U1:busmvn", "kV, Line-Ground Positive-Sequence-Voltage, Magnitude"
      m_U1_buslvn = "m:U1:buslvn", "kV, Line-Ground Positive-Sequence-Voltage, Magnitude"
      m_U1l_bushv = "m:U1l:bushv", "kV, Line-Line Positive-Sequence-Voltage, Magnitude"
      m_U1l_busmv = "m:U1l:busmv", "kV, Line-Line Positive-Sequence-Voltage, Magnitude"
      m_U1l_buslv = "m:U1l:buslv", "kV, Line-Line Positive-Sequence-Voltage, Magnitude"
      m_U1l_bushvn = "m:U1l:bushvn", "kV, Line-Line Positive-Sequence-Voltage, Magnitude"
      m_U1l_busmvn = "m:U1l:busmvn", "kV, Line-Line Positive-Sequence-Voltage, Magnitude"
      m_U1l_buslvn = "m:U1l:buslvn", "kV, Line-Line Positive-Sequence-Voltage, Magnitude"
      m_phiu1_bushv = "m:phiu1:bushv", "deg, Positive-Sequence-Voltage, Angle"
      m_phiu1_busmv = "m:phiu1:busmv", "deg, Positive-Sequence-Voltage, Angle"
      m_phiu1_buslv = "m:phiu1:buslv", "deg, Positive-Sequence-Voltage, Angle"
      m_phiu1_bushvn = "m:phiu1:bushvn", "deg, Positive-Sequence-Voltage, Angle"
      m_phiu1_busmvn = "m:phiu1:busmvn", "deg, Positive-Sequence-Voltage, Angle"
      m_phiu1_buslvn = "m:phiu1:buslvn", "deg, Positive-Sequence-Voltage, Angle"
      m_u2_bushv = "m:u2:bushv", "p.u., Negative-Sequence-Voltage, Magnitude"
      m_u2_busmv = "m:u2:busmv", "p.u., Negative-Sequence-Voltage, Magnitude"
      m_u2_buslv = "m:u2:buslv", "p.u., Negative-Sequence-Voltage, Magnitude"
      m_u2_bushvn = "m:u2:bushvn", "p.u., Negative-Sequence-Voltage, Magnitude"
      m_u2_busmvn = "m:u2:busmvn", "p.u., Negative-Sequence-Voltage, Magnitude"
      m_u2_buslvn = "m:u2:buslvn", "p.u., Negative-Sequence-Voltage, Magnitude"
      m_phiu2_bushv = "m:phiu2:bushv", "deg, Negative-Sequence-Voltage, Angle"
      m_phiu2_busmv = "m:phiu2:busmv", "deg, Negative-Sequence-Voltage, Angle"
      m_phiu2_buslv = "m:phiu2:buslv", "deg, Negative-Sequence-Voltage, Angle"
      m_phiu2_bushvn = "m:phiu2:bushvn", "deg, Negative-Sequence-Voltage, Angle"
      m_phiu2_busmvn = "m:phiu2:busmvn", "deg, Negative-Sequence-Voltage, Angle"
      m_phiu2_buslvn = "m:phiu2:buslvn", "deg, Negative-Sequence-Voltage, Angle"
      m_inet_bushv_A = "m:inet:bushv:A", "p.u., Phase Current, Magnitude"
      m_inet_bushv_B = "m:inet:bushv:B", "p.u., Phase Current, Magnitude"
      m_inet_bushv_C = "m:inet:bushv:C", "p.u., Phase Current, Magnitude"
      m_inet_bushv_N = "m:inet:bushv:N", "p.u., Phase Current, Magnitude"
      m_inet_busmv_A = "m:inet:busmv:A", "p.u., Phase Current, Magnitude"
      m_inet_busmv_B = "m:inet:busmv:B", "p.u., Phase Current, Magnitude"
      m_inet_busmv_C = "m:inet:busmv:C", "p.u., Phase Current, Magnitude"
      m_inet_busmv_N = "m:inet:busmv:N", "p.u., Phase Current, Magnitude"
      m_inet_buslv_A = "m:inet:buslv:A", "p.u., Phase Current, Magnitude"
      m_inet_buslv_B = "m:inet:buslv:B", "p.u., Phase Current, Magnitude"
      m_inet_buslv_C = "m:inet:buslv:C", "p.u., Phase Current, Magnitude"
      m_inet_buslv_N = "m:inet:buslv:N", "p.u., Phase Current, Magnitude"
      m_inet_bushvn = "m:inet:bushvn", "p.u., Phase Current, Magnitude"
      m_inet_busmvn = "m:inet:busmvn", "p.u., Phase Current, Magnitude"
      m_inet_buslvn = "m:inet:buslvn", "p.u., Phase Current, Magnitude"
      m_I_bushv_A = "m:I:bushv:A", "kA, Phase Current, Magnitude"
      m_I_bushv_B = "m:I:bushv:B", "kA, Phase Current, Magnitude"
      m_I_bushv_C = "m:I:bushv:C", "kA, Phase Current, Magnitude"
      m_I_bushv_N = "m:I:bushv:N", "kA, Phase Current, Magnitude"
      m_I_busmv_A = "m:I:busmv:A", "kA, Phase Current, Magnitude"
      m_I_busmv_B = "m:I:busmv:B", "kA, Phase Current, Magnitude"
      m_I_busmv_C = "m:I:busmv:C", "kA, Phase Current, Magnitude"
      m_I_busmv_N = "m:I:busmv:N", "kA, Phase Current, Magnitude"
      m_I_buslv_A = "m:I:buslv:A", "kA, Phase Current, Magnitude"
      m_I_buslv_B = "m:I:buslv:B", "kA, Phase Current, Magnitude"
      m_I_buslv_C = "m:I:buslv:C", "kA, Phase Current, Magnitude"
      m_I_buslv_N = "m:I:buslv:N", "kA, Phase Current, Magnitude"
      m_I_bushvn = "m:I:bushvn", "kA, Phase Current, Magnitude"
      m_I_busmvn = "m:I:busmvn", "kA, Phase Current, Magnitude"
      m_I_buslvn = "m:I:buslvn", "kA, Phase Current, Magnitude"
      m_phii_bushv_A = "m:phii:bushv:A", "deg, Phase Current, Angle"
      m_phii_bushv_B = "m:phii:bushv:B", "deg, Phase Current, Angle"
      m_phii_bushv_C = "m:phii:bushv:C", "deg, Phase Current, Angle"
      m_phii_bushv_N = "m:phii:bushv:N", "deg, Phase Current, Angle"
      m_phii_busmv_A = "m:phii:busmv:A", "deg, Phase Current, Angle"
      m_phii_busmv_B = "m:phii:busmv:B", "deg, Phase Current, Angle"
      m_phii_busmv_C = "m:phii:busmv:C", "deg, Phase Current, Angle"
      m_phii_busmv_N = "m:phii:busmv:N", "deg, Phase Current, Angle"
      m_phii_buslv_A = "m:phii:buslv:A", "deg, Phase Current, Angle"
      m_phii_buslv_B = "m:phii:buslv:B", "deg, Phase Current, Angle"
      m_phii_buslv_C = "m:phii:buslv:C", "deg, Phase Current, Angle"
      m_phii_buslv_N = "m:phii:buslv:N", "deg, Phase Current, Angle"
      m_phii_bushvn = "m:phii:bushvn", "deg, Phase Current, Angle"
      m_phii_busmvn = "m:phii:busmvn", "deg, Phase Current, Angle"
      m_phii_buslvn = "m:phii:buslvn", "deg, Phase Current, Angle"
      m_i0r_bushv = "m:i0r:bushv", "p.u., Zero-Sequence Current, Real Part"
      m_i0r_busmv = "m:i0r:busmv", "p.u., Zero-Sequence Current, Real Part"
      m_i0r_buslv = "m:i0r:buslv", "p.u., Zero-Sequence Current, Real Part"
      m_i0r_bushvn = "m:i0r:bushvn", "p.u., Zero-Sequence Current, Real Part"
      m_i0r_busmvn = "m:i0r:busmvn", "p.u., Zero-Sequence Current, Real Part"
      m_i0r_buslvn = "m:i0r:buslvn", "p.u., Zero-Sequence Current, Real Part"
      m_i0i_bushv = "m:i0i:bushv", "p.u., Zero-Sequence Current, Imaginary Part"
      m_i0i_busmv = "m:i0i:busmv", "p.u., Zero-Sequence Current, Imaginary Part"
      m_i0i_buslv = "m:i0i:buslv", "p.u., Zero-Sequence Current, Imaginary Part"
      m_i0i_bushvn = "m:i0i:bushvn", "p.u., Zero-Sequence Current, Imaginary Part"
      m_i0i_busmvn = "m:i0i:busmvn", "p.u., Zero-Sequence Current, Imaginary Part"
      m_i0i_buslvn = "m:i0i:buslvn", "p.u., Zero-Sequence Current, Imaginary Part"
      m_i0_bushv = "m:i0:bushv", "p.u., Zero-Sequence Current, Magnitude"
      m_i0_busmv = "m:i0:busmv", "p.u., Zero-Sequence Current, Magnitude"
      m_i0_buslv = "m:i0:buslv", "p.u., Zero-Sequence Current, Magnitude"
      m_i0_bushvn = "m:i0:bushvn", "p.u., Zero-Sequence Current, Magnitude"
      m_i0_busmvn = "m:i0:busmvn", "p.u., Zero-Sequence Current, Magnitude"
      m_i0_buslvn = "m:i0:buslvn", "p.u., Zero-Sequence Current, Magnitude"
      m_I0_bushv = "m:I0:bushv", "kA, Zero-Sequence Current, Magnitude"
      m_I0_busmv = "m:I0:busmv", "kA, Zero-Sequence Current, Magnitude"
      m_I0_buslv = "m:I0:buslv", "kA, Zero-Sequence Current, Magnitude"
      m_I0_bushvn = "m:I0:bushvn", "kA, Zero-Sequence Current, Magnitude"
      m_I0_busmvn = "m:I0:busmvn", "kA, Zero-Sequence Current, Magnitude"
      m_I0_buslvn = "m:I0:buslvn", "kA, Zero-Sequence Current, Magnitude"
      m_I0x3_bushv = "m:I0x3:bushv", "kA, 3*I0"
      m_I0x3_busmv = "m:I0x3:busmv", "kA, 3*I0"
      m_I0x3_buslv = "m:I0x3:buslv", "kA, 3*I0"
      m_I0x3_bushvn = "m:I0x3:bushvn", "kA, 3*I0"
      m_I0x3_busmvn = "m:I0x3:busmvn", "kA, 3*I0"
      m_I0x3_buslvn = "m:I0x3:buslvn", "kA, 3*I0"
      m_phii0_bushv = "m:phii0:bushv", "deg, Zero-Sequence Current, Angle"
      m_phii0_busmv = "m:phii0:busmv", "deg, Zero-Sequence Current, Angle"
      m_phii0_buslv = "m:phii0:buslv", "deg, Zero-Sequence Current, Angle"
      m_phii0_bushvn = "m:phii0:bushvn", "deg, Zero-Sequence Current, Angle"
      m_phii0_busmvn = "m:phii0:busmvn", "deg, Zero-Sequence Current, Angle"
      m_phii0_buslvn = "m:phii0:buslvn", "deg, Zero-Sequence Current, Angle"
      m_i1r_bushv = "m:i1r:bushv", "p.u., Positive-Sequence Current, Real Part"
      m_i1r_busmv = "m:i1r:busmv", "p.u., Positive-Sequence Current, Real Part"
      m_i1r_buslv = "m:i1r:buslv", "p.u., Positive-Sequence Current, Real Part"
      m_i1r_bushvn = "m:i1r:bushvn", "p.u., Positive-Sequence Current, Real Part"
      m_i1r_busmvn = "m:i1r:busmvn", "p.u., Positive-Sequence Current, Real Part"
      m_i1r_buslvn = "m:i1r:buslvn", "p.u., Positive-Sequence Current, Real Part"
      m_i1i_bushv = "m:i1i:bushv", "p.u., Positive-Sequence Current, Imaginary Part"
      m_i1i_busmv = "m:i1i:busmv", "p.u., Positive-Sequence Current, Imaginary Part"
      m_i1i_buslv = "m:i1i:buslv", "p.u., Positive-Sequence Current, Imaginary Part"
      m_i1i_bushvn = "m:i1i:bushvn", "p.u., Positive-Sequence Current, Imaginary Part"
      m_i1i_busmvn = "m:i1i:busmvn", "p.u., Positive-Sequence Current, Imaginary Part"
      m_i1i_buslvn = "m:i1i:buslvn", "p.u., Positive-Sequence Current, Imaginary Part"
      m_i1_bushv = "m:i1:bushv", "p.u., Positive-Sequence Current, Magnitude"
      m_i1_busmv = "m:i1:busmv", "p.u., Positive-Sequence Current, Magnitude"
      m_i1_buslv = "m:i1:buslv", "p.u., Positive-Sequence Current, Magnitude"
      m_i1_bushvn = "m:i1:bushvn", "p.u., Positive-Sequence Current, Magnitude"
      m_i1_busmvn = "m:i1:busmvn", "p.u., Positive-Sequence Current, Magnitude"
      m_i1_buslvn = "m:i1:buslvn", "p.u., Positive-Sequence Current, Magnitude"
      m_I1_bushv = "m:I1:bushv", "kA, Positive-Sequence Current, Magnitude"
      m_I1_busmv = "m:I1:busmv", "kA, Positive-Sequence Current, Magnitude"
      m_I1_buslv = "m:I1:buslv", "kA, Positive-Sequence Current, Magnitude"
      m_I1_bushvn = "m:I1:bushvn", "kA, Positive-Sequence Current, Magnitude"
      m_I1_busmvn = "m:I1:busmvn", "kA, Positive-Sequence Current, Magnitude"
      m_I1_buslvn = "m:I1:buslvn", "kA, Positive-Sequence Current, Magnitude"
      m_phii1_bushv = "m:phii1:bushv", "deg, Positive-Sequence Current, Angle"
      m_phii1_busmv = "m:phii1:busmv", "deg, Positive-Sequence Current, Angle"
      m_phii1_buslv = "m:phii1:buslv", "deg, Positive-Sequence Current, Angle"
      m_phii1_bushvn = "m:phii1:bushvn", "deg, Positive-Sequence Current, Angle"
      m_phii1_busmvn = "m:phii1:busmvn", "deg, Positive-Sequence Current, Angle"
      m_phii1_buslvn = "m:phii1:buslvn", "deg, Positive-Sequence Current, Angle"
      m_i2r_bushv = "m:i2r:bushv", "p.u., Negative-Sequence Current, Real Part"
      m_i2r_busmv = "m:i2r:busmv", "p.u., Negative-Sequence Current, Real Part"
      m_i2r_buslv = "m:i2r:buslv", "p.u., Negative-Sequence Current, Real Part"
      m_i2r_bushvn = "m:i2r:bushvn", "p.u., Negative-Sequence Current, Real Part"
      m_i2r_busmvn = "m:i2r:busmvn", "p.u., Negative-Sequence Current, Real Part"
      m_i2r_buslvn = "m:i2r:buslvn", "p.u., Negative-Sequence Current, Real Part"
      m_i2i_bushv = "m:i2i:bushv", "p.u., Negative-Sequence Current, Imaginary Part"
      m_i2i_busmv = "m:i2i:busmv", "p.u., Negative-Sequence Current, Imaginary Part"
      m_i2i_buslv = "m:i2i:buslv", "p.u., Negative-Sequence Current, Imaginary Part"
      m_i2i_bushvn = "m:i2i:bushvn", "p.u., Negative-Sequence Current, Imaginary Part"
      m_i2i_busmvn = "m:i2i:busmvn", "p.u., Negative-Sequence Current, Imaginary Part"
      m_i2i_buslvn = "m:i2i:buslvn", "p.u., Negative-Sequence Current, Imaginary Part"
      m_i2_bushv = "m:i2:bushv", "p.u., Negative-Sequence Current, Magnitude"
      m_i2_busmv = "m:i2:busmv", "p.u., Negative-Sequence Current, Magnitude"
      m_i2_buslv = "m:i2:buslv", "p.u., Negative-Sequence Current, Magnitude"
      m_i2_bushvn = "m:i2:bushvn", "p.u., Negative-Sequence Current, Magnitude"
      m_i2_busmvn = "m:i2:busmvn", "p.u., Negative-Sequence Current, Magnitude"
      m_i2_buslvn = "m:i2:buslvn", "p.u., Negative-Sequence Current, Magnitude"
      m_I2_bushv = "m:I2:bushv", "kA, Negative-Sequence Current, Magnitude"
      m_I2_busmv = "m:I2:busmv", "kA, Negative-Sequence Current, Magnitude"
      m_I2_buslv = "m:I2:buslv", "kA, Negative-Sequence Current, Magnitude"
      m_I2_bushvn = "m:I2:bushvn", "kA, Negative-Sequence Current, Magnitude"
      m_I2_busmvn = "m:I2:busmvn", "kA, Negative-Sequence Current, Magnitude"
      m_I2_buslvn = "m:I2:buslvn", "kA, Negative-Sequence Current, Magnitude"
      m_phii2_bushv = "m:phii2:bushv", "deg, Negative-Sequence Current, Angle"
      m_phii2_busmv = "m:phii2:busmv", "deg, Negative-Sequence Current, Angle"
      m_phii2_buslv = "m:phii2:buslv", "deg, Negative-Sequence Current, Angle"
      m_phii2_bushvn = "m:phii2:bushvn", "deg, Negative-Sequence Current, Angle"
      m_phii2_busmvn = "m:phii2:busmvn", "deg, Negative-Sequence Current, Angle"
      m_phii2_buslvn = "m:phii2:buslvn", "deg, Negative-Sequence Current, Angle"
      m_P_bushv_A = "m:P:bushv:A", "MW, Active Power"
      m_P_bushv_B = "m:P:bushv:B", "MW, Active Power"
      m_P_bushv_C = "m:P:bushv:C", "MW, Active Power"
      m_P_bushv_N = "m:P:bushv:N", "MW, Active Power"
      m_P_busmv_A = "m:P:busmv:A", "MW, Active Power"
      m_P_busmv_B = "m:P:busmv:B", "MW, Active Power"
      m_P_busmv_C = "m:P:busmv:C", "MW, Active Power"
      m_P_busmv_N = "m:P:busmv:N", "MW, Active Power"
      m_P_buslv_A = "m:P:buslv:A", "MW, Active Power"
      m_P_buslv_B = "m:P:buslv:B", "MW, Active Power"
      m_P_buslv_C = "m:P:buslv:C", "MW, Active Power"
      m_P_buslv_N = "m:P:buslv:N", "MW, Active Power"
      m_P_bushvn = "m:P:bushvn", "MW, Active Power"
      m_P_busmvn = "m:P:busmvn", "MW, Active Power"
      m_P_buslvn = "m:P:buslvn", "MW, Active Power"
      m_Q_bushv_A = "m:Q:bushv:A", "Mvar, Reactive Power"
      m_Q_bushv_B = "m:Q:bushv:B", "Mvar, Reactive Power"
      m_Q_bushv_C = "m:Q:bushv:C", "Mvar, Reactive Power"
      m_Q_bushv_N = "m:Q:bushv:N", "Mvar, Reactive Power"
      m_Q_busmv_A = "m:Q:busmv:A", "Mvar, Reactive Power"
      m_Q_busmv_B = "m:Q:busmv:B", "Mvar, Reactive Power"
      m_Q_busmv_C = "m:Q:busmv:C", "Mvar, Reactive Power"
      m_Q_busmv_N = "m:Q:busmv:N", "Mvar, Reactive Power"
      m_Q_buslv_A = "m:Q:buslv:A", "Mvar, Reactive Power"
      m_Q_buslv_B = "m:Q:buslv:B", "Mvar, Reactive Power"
      m_Q_buslv_C = "m:Q:buslv:C", "Mvar, Reactive Power"
      m_Q_buslv_N = "m:Q:buslv:N", "Mvar, Reactive Power"
      m_Q_bushvn = "m:Q:bushvn", "Mvar, Reactive Power"
      m_Q_busmvn = "m:Q:busmvn", "Mvar, Reactive Power"
      m_Q_buslvn = "m:Q:buslvn", "Mvar, Reactive Power"
      m_S_bushv_A = "m:S:bushv:A", "MVA, Apparent Power"
      m_S_bushv_B = "m:S:bushv:B", "MVA, Apparent Power"
      m_S_bushv_C = "m:S:bushv:C", "MVA, Apparent Power"
      m_S_bushv_N = "m:S:bushv:N", "MVA, Apparent Power"
      m_S_busmv_A = "m:S:busmv:A", "MVA, Apparent Power"
      m_S_busmv_B = "m:S:busmv:B", "MVA, Apparent Power"
      m_S_busmv_C = "m:S:busmv:C", "MVA, Apparent Power"
      m_S_busmv_N = "m:S:busmv:N", "MVA, Apparent Power"
      m_S_buslv_A = "m:S:buslv:A", "MVA, Apparent Power"
      m_S_buslv_B = "m:S:buslv:B", "MVA, Apparent Power"
      m_S_buslv_C = "m:S:buslv:C", "MVA, Apparent Power"
      m_S_buslv_N = "m:S:buslv:N", "MVA, Apparent Power"
      m_S_bushvn = "m:S:bushvn", "MVA, Apparent Power"
      m_S_busmvn = "m:S:busmvn", "MVA, Apparent Power"
      m_S_buslvn = "m:S:buslvn", "MVA, Apparent Power"
      m_cosphi_bushv_A = "m:cosphi:bushv:A", "Power Factor"
      m_cosphi_bushv_B = "m:cosphi:bushv:B", "Power Factor"
      m_cosphi_bushv_C = "m:cosphi:bushv:C", "Power Factor"
      m_cosphi_bushv_N = "m:cosphi:bushv:N", "Power Factor"
      m_cosphi_busmv_A = "m:cosphi:busmv:A", "Power Factor"
      m_cosphi_busmv_B = "m:cosphi:busmv:B", "Power Factor"
      m_cosphi_busmv_C = "m:cosphi:busmv:C", "Power Factor"
      m_cosphi_busmv_N = "m:cosphi:busmv:N", "Power Factor"
      m_cosphi_buslv_A = "m:cosphi:buslv:A", "Power Factor"
      m_cosphi_buslv_B = "m:cosphi:buslv:B", "Power Factor"
      m_cosphi_buslv_C = "m:cosphi:buslv:C", "Power Factor"
      m_cosphi_buslv_N = "m:cosphi:buslv:N", "Power Factor"
      m_cosphi_bushvn = "m:cosphi:bushvn", "Power Factor"
      m_cosphi_busmvn = "m:cosphi:busmvn", "Power Factor"
      m_cosphi_buslvn = "m:cosphi:buslvn", "Power Factor"
      m_tanphi_bushv_A = "m:tanphi:bushv:A", "tan(phi)"
      m_tanphi_bushv_B = "m:tanphi:bushv:B", "tan(phi)"
      m_tanphi_bushv_C = "m:tanphi:bushv:C", "tan(phi)"
      m_tanphi_bushv_N = "m:tanphi:bushv:N", "tan(phi)"
      m_tanphi_busmv_A = "m:tanphi:busmv:A", "tan(phi)"
      m_tanphi_busmv_B = "m:tanphi:busmv:B", "tan(phi)"
      m_tanphi_busmv_C = "m:tanphi:busmv:C", "tan(phi)"
      m_tanphi_busmv_N = "m:tanphi:busmv:N", "tan(phi)"
      m_tanphi_buslv_A = "m:tanphi:buslv:A", "tan(phi)"
      m_tanphi_buslv_B = "m:tanphi:buslv:B", "tan(phi)"
      m_tanphi_buslv_C = "m:tanphi:buslv:C", "tan(phi)"
      m_tanphi_buslv_N = "m:tanphi:buslv:N", "tan(phi)"
      m_tanphi_bushvn = "m:tanphi:bushvn", "tan(phi)"
      m_tanphi_busmvn = "m:tanphi:busmvn", "tan(phi)"
      m_tanphi_buslvn = "m:tanphi:buslvn", "tan(phi)"
      m_Psum_bushv = "m:Psum:bushv", "MW, Total Active Power"
      m_Psum_busmv = "m:Psum:busmv", "MW, Total Active Power"
      m_Psum_buslv = "m:Psum:buslv", "MW, Total Active Power"
      m_Psum_bushvn = "m:Psum:bushvn", "MW, Total Active Power"
      m_Psum_busmvn = "m:Psum:busmvn", "MW, Total Active Power"
      m_Psum_buslvn = "m:Psum:buslvn", "MW, Total Active Power"
      m_Qsum_bushv = "m:Qsum:bushv", "Mvar, Total Reactive Power"
      m_Qsum_busmv = "m:Qsum:busmv", "Mvar, Total Reactive Power"
      m_Qsum_buslv = "m:Qsum:buslv", "Mvar, Total Reactive Power"
      m_Qsum_bushvn = "m:Qsum:bushvn", "Mvar, Total Reactive Power"
      m_Qsum_busmvn = "m:Qsum:busmvn", "Mvar, Total Reactive Power"
      m_Qsum_buslvn = "m:Qsum:buslvn", "Mvar, Total Reactive Power"
      m_Ssum_bushv = "m:Ssum:bushv", "MVA, Total Apparent Power"
      m_Ssum_busmv = "m:Ssum:busmv", "MVA, Total Apparent Power"
      m_Ssum_buslv = "m:Ssum:buslv", "MVA, Total Apparent Power"
      m_Ssum_bushvn = "m:Ssum:bushvn", "MVA, Total Apparent Power"
      m_Ssum_busmvn = "m:Ssum:busmvn", "MVA, Total Apparent Power"
      m_Ssum_buslvn = "m:Ssum:buslvn", "MVA, Total Apparent Power"
      m_cosphisum_bushv = "m:cosphisum:bushv", "Total Power Factor"
      m_cosphisum_busmv = "m:cosphisum:busmv", "Total Power Factor"
      m_cosphisum_buslv = "m:cosphisum:buslv", "Total Power Factor"
      m_cosphisum_bushvn = "m:cosphisum:bushvn", "Total Power Factor"
      m_cosphisum_busmvn = "m:cosphisum:busmvn", "Total Power Factor"
      m_cosphisum_buslvn = "m:cosphisum:buslvn", "Total Power Factor"
      m_tanphisum_bushv = "m:tanphisum:bushv", "Total tan(phi)"
      m_tanphisum_busmv = "m:tanphisum:busmv", "Total tan(phi)"
      m_tanphisum_buslv = "m:tanphisum:buslv", "Total tan(phi)"
      m_tanphisum_bushvn = "m:tanphisum:bushvn", "Total tan(phi)"
      m_tanphisum_busmvn = "m:tanphisum:busmvn", "Total tan(phi)"
      m_tanphisum_buslvn = "m:tanphisum:buslvn", "Total tan(phi)"
      m_Pln_bushv_A = "m:Pln:bushv:A", "MW, Active Power (Line-Neutral)"
      m_Pln_bushv_B = "m:Pln:bushv:B", "MW, Active Power (Line-Neutral)"
      m_Pln_bushv_C = "m:Pln:bushv:C", "MW, Active Power (Line-Neutral)"
      m_Pln_bushv_N = "m:Pln:bushv:N", "MW, Active Power (Line-Neutral)"
      m_Pln_busmv_A = "m:Pln:busmv:A", "MW, Active Power (Line-Neutral)"
      m_Pln_busmv_B = "m:Pln:busmv:B", "MW, Active Power (Line-Neutral)"
      m_Pln_busmv_C = "m:Pln:busmv:C", "MW, Active Power (Line-Neutral)"
      m_Pln_busmv_N = "m:Pln:busmv:N", "MW, Active Power (Line-Neutral)"
      m_Pln_buslv_A = "m:Pln:buslv:A", "MW, Active Power (Line-Neutral)"
      m_Pln_buslv_B = "m:Pln:buslv:B", "MW, Active Power (Line-Neutral)"
      m_Pln_buslv_C = "m:Pln:buslv:C", "MW, Active Power (Line-Neutral)"
      m_Pln_buslv_N = "m:Pln:buslv:N", "MW, Active Power (Line-Neutral)"
      m_Pln_bushvn = "m:Pln:bushvn", "MW, Active Power (Line-Neutral)"
      m_Pln_busmvn = "m:Pln:busmvn", "MW, Active Power (Line-Neutral)"
      m_Pln_buslvn = "m:Pln:buslvn", "MW, Active Power (Line-Neutral)"
      m_Qln_bushv_A = "m:Qln:bushv:A", "Mvar, Reactive Power (Line-Neutral)"
      m_Qln_bushv_B = "m:Qln:bushv:B", "Mvar, Reactive Power (Line-Neutral)"
      m_Qln_bushv_C = "m:Qln:bushv:C", "Mvar, Reactive Power (Line-Neutral)"
      m_Qln_bushv_N = "m:Qln:bushv:N", "Mvar, Reactive Power (Line-Neutral)"
      m_Qln_busmv_A = "m:Qln:busmv:A", "Mvar, Reactive Power (Line-Neutral)"
      m_Qln_busmv_B = "m:Qln:busmv:B", "Mvar, Reactive Power (Line-Neutral)"
      m_Qln_busmv_C = "m:Qln:busmv:C", "Mvar, Reactive Power (Line-Neutral)"
      m_Qln_busmv_N = "m:Qln:busmv:N", "Mvar, Reactive Power (Line-Neutral)"
      m_Qln_buslv_A = "m:Qln:buslv:A", "Mvar, Reactive Power (Line-Neutral)"
      m_Qln_buslv_B = "m:Qln:buslv:B", "Mvar, Reactive Power (Line-Neutral)"
      m_Qln_buslv_C = "m:Qln:buslv:C", "Mvar, Reactive Power (Line-Neutral)"
      m_Qln_buslv_N = "m:Qln:buslv:N", "Mvar, Reactive Power (Line-Neutral)"
      m_Qln_bushvn = "m:Qln:bushvn", "Mvar, Reactive Power (Line-Neutral)"
      m_Qln_busmvn = "m:Qln:busmvn", "Mvar, Reactive Power (Line-Neutral)"
      m_Qln_buslvn = "m:Qln:buslvn", "Mvar, Reactive Power (Line-Neutral)"
      m_P1_bushv = "m:P1:bushv", "MW, Positive Sequence Active Power"
      m_P1_busmv = "m:P1:busmv", "MW, Positive Sequence Active Power"
      m_P1_buslv = "m:P1:buslv", "MW, Positive Sequence Active Power"
      m_P1_bushvn = "m:P1:bushvn", "MW, Positive Sequence Active Power"
      m_P1_busmvn = "m:P1:busmvn", "MW, Positive Sequence Active Power"
      m_P1_buslvn = "m:P1:buslvn", "MW, Positive Sequence Active Power"
      m_Q1_bushv = "m:Q1:bushv", "Mvar, Positive Sequence Reactive Power"
      m_Q1_busmv = "m:Q1:busmv", "Mvar, Positive Sequence Reactive Power"
      m_Q1_buslv = "m:Q1:buslv", "Mvar, Positive Sequence Reactive Power"
      m_Q1_bushvn = "m:Q1:bushvn", "Mvar, Positive Sequence Reactive Power"
      m_Q1_busmvn = "m:Q1:busmvn", "Mvar, Positive Sequence Reactive Power"
      m_Q1_buslvn = "m:Q1:buslvn", "Mvar, Positive Sequence Reactive Power"
      m_P2_bushv = "m:P2:bushv", "MW, Negative Sequence Active Power"
      m_P2_busmv = "m:P2:busmv", "MW, Negative Sequence Active Power"
      m_P2_buslv = "m:P2:buslv", "MW, Negative Sequence Active Power"
      m_P2_bushvn = "m:P2:bushvn", "MW, Negative Sequence Active Power"
      m_P2_busmvn = "m:P2:busmvn", "MW, Negative Sequence Active Power"
      m_P2_buslvn = "m:P2:buslvn", "MW, Negative Sequence Active Power"
      m_Q2_bushv = "m:Q2:bushv", "Mvar, Negative Sequence Reactive Power"
      m_Q2_busmv = "m:Q2:busmv", "Mvar, Negative Sequence Reactive Power"
      m_Q2_buslv = "m:Q2:buslv", "Mvar, Negative Sequence Reactive Power"
      m_Q2_bushvn = "m:Q2:bushvn", "Mvar, Negative Sequence Reactive Power"
      m_Q2_busmvn = "m:Q2:busmvn", "Mvar, Negative Sequence Reactive Power"
      m_Q2_buslvn = "m:Q2:buslvn", "Mvar, Negative Sequence Reactive Power"
      m_P0_bushv = "m:P0:bushv", "MW, Zero Sequence Active Power"
      m_P0_busmv = "m:P0:busmv", "MW, Zero Sequence Active Power"
      m_P0_buslv = "m:P0:buslv", "MW, Zero Sequence Active Power"
      m_P0_bushvn = "m:P0:bushvn", "MW, Zero Sequence Active Power"
      m_P0_busmvn = "m:P0:busmvn", "MW, Zero Sequence Active Power"
      m_P0_buslvn = "m:P0:buslvn", "MW, Zero Sequence Active Power"
      m_Q0_bushv = "m:Q0:bushv", "Mvar, Zero Sequence Reactive Power"
      m_Q0_busmv = "m:Q0:busmv", "Mvar, Zero Sequence Reactive Power"
      m_Q0_buslv = "m:Q0:buslv", "Mvar, Zero Sequence Reactive Power"
      m_Q0_bushvn = "m:Q0:bushvn", "Mvar, Zero Sequence Reactive Power"
      m_Q0_busmvn = "m:Q0:busmvn", "Mvar, Zero Sequence Reactive Power"
      m_Q0_buslvn = "m:Q0:buslvn", "Mvar, Zero Sequence Reactive Power"
      m_i1P_bushv = "m:i1P:bushv", "p.u., Positive-Sequence Active Current"
      m_i1P_busmv = "m:i1P:busmv", "p.u., Positive-Sequence Active Current"
      m_i1P_buslv = "m:i1P:buslv", "p.u., Positive-Sequence Active Current"
      m_i1P_bushvn = "m:i1P:bushvn", "p.u., Positive-Sequence Active Current"
      m_i1P_busmvn = "m:i1P:busmvn", "p.u., Positive-Sequence Active Current"
      m_i1P_buslvn = "m:i1P:buslvn", "p.u., Positive-Sequence Active Current"
      m_i1Q_bushv = "m:i1Q:bushv", "p.u., Positive-Sequence Reactive Current"
      m_i1Q_busmv = "m:i1Q:busmv", "p.u., Positive-Sequence Reactive Current"
      m_i1Q_buslv = "m:i1Q:buslv", "p.u., Positive-Sequence Reactive Current"
      m_i1Q_bushvn = "m:i1Q:bushvn", "p.u., Positive-Sequence Reactive Current"
      m_i1Q_busmvn = "m:i1Q:busmvn", "p.u., Positive-Sequence Reactive Current"
      m_i1Q_buslvn = "m:i1Q:buslvn", "p.u., Positive-Sequence Reactive Current"
      m_I1P_bushv = "m:I1P:bushv", "kA, Positive-Sequence Active Current"
      m_I1P_busmv = "m:I1P:busmv", "kA, Positive-Sequence Active Current"
      m_I1P_buslv = "m:I1P:buslv", "kA, Positive-Sequence Active Current"
      m_I1P_bushvn = "m:I1P:bushvn", "kA, Positive-Sequence Active Current"
      m_I1P_busmvn = "m:I1P:busmvn", "kA, Positive-Sequence Active Current"
      m_I1P_buslvn = "m:I1P:buslvn", "kA, Positive-Sequence Active Current"
      m_I1Q_bushv = "m:I1Q:bushv", "kA, Positive-Sequence Reactive Current"
      m_I1Q_busmv = "m:I1Q:busmv", "kA, Positive-Sequence Reactive Current"
      m_I1Q_buslv = "m:I1Q:buslv", "kA, Positive-Sequence Reactive Current"
      m_I1Q_bushvn = "m:I1Q:bushvn", "kA, Positive-Sequence Reactive Current"
      m_I1Q_busmvn = "m:I1Q:busmvn", "kA, Positive-Sequence Reactive Current"
      m_I1Q_buslvn = "m:I1Q:buslvn", "kA, Positive-Sequence Reactive Current"
      m_i2P_bushv = "m:i2P:bushv", "p.u., Negative-Sequence Active Current"
      m_i2P_busmv = "m:i2P:busmv", "p.u., Negative-Sequence Active Current"
      m_i2P_buslv = "m:i2P:buslv", "p.u., Negative-Sequence Active Current"
      m_i2P_bushvn = "m:i2P:bushvn", "p.u., Negative-Sequence Active Current"
      m_i2P_busmvn = "m:i2P:busmvn", "p.u., Negative-Sequence Active Current"
      m_i2P_buslvn = "m:i2P:buslvn", "p.u., Negative-Sequence Active Current"
      m_i2Q_bushv = "m:i2Q:bushv", "p.u., Negative-Sequence Reactive Current"
      m_i2Q_busmv = "m:i2Q:busmv", "p.u., Negative-Sequence Reactive Current"
      m_i2Q_buslv = "m:i2Q:buslv", "p.u., Negative-Sequence Reactive Current"
      m_i2Q_bushvn = "m:i2Q:bushvn", "p.u., Negative-Sequence Reactive Current"
      m_i2Q_busmvn = "m:i2Q:busmvn", "p.u., Negative-Sequence Reactive Current"
      m_i2Q_buslvn = "m:i2Q:buslvn", "p.u., Negative-Sequence Reactive Current"
      m_I2P_bushv = "m:I2P:bushv", "kA, Negative-Sequence Active Current"
      m_I2P_busmv = "m:I2P:busmv", "kA, Negative-Sequence Active Current"
      m_I2P_buslv = "m:I2P:buslv", "kA, Negative-Sequence Active Current"
      m_I2P_bushvn = "m:I2P:bushvn", "kA, Negative-Sequence Active Current"
      m_I2P_busmvn = "m:I2P:busmvn", "kA, Negative-Sequence Active Current"
      m_I2P_buslvn = "m:I2P:buslvn", "kA, Negative-Sequence Active Current"
      m_I2Q_bushv = "m:I2Q:bushv", "kA, Negative-Sequence Reactive Current"
      m_I2Q_busmv = "m:I2Q:busmv", "kA, Negative-Sequence Reactive Current"
      m_I2Q_buslv = "m:I2Q:buslv", "kA, Negative-Sequence Reactive Current"
      m_I2Q_bushvn = "m:I2Q:bushvn", "kA, Negative-Sequence Reactive Current"
      m_I2Q_busmvn = "m:I2Q:busmvn", "kA, Negative-Sequence Reactive Current"
      m_I2Q_buslvn = "m:I2Q:buslvn", "kA, Negative-Sequence Reactive Current"
      m_ur_bushv_A = "m:ur:bushv:A", "p.u., Phase Voltage, Real Part"
      m_ur_bushv_B = "m:ur:bushv:B", "p.u., Phase Voltage, Real Part"
      m_ur_bushv_C = "m:ur:bushv:C", "p.u., Phase Voltage, Real Part"
      m_ur_bushv_N = "m:ur:bushv:N", "p.u., Phase Voltage, Real Part"
      m_ur_busmv_A = "m:ur:busmv:A", "p.u., Phase Voltage, Real Part"
      m_ur_busmv_B = "m:ur:busmv:B", "p.u., Phase Voltage, Real Part"
      m_ur_busmv_C = "m:ur:busmv:C", "p.u., Phase Voltage, Real Part"
      m_ur_busmv_N = "m:ur:busmv:N", "p.u., Phase Voltage, Real Part"
      m_ur_buslv_A = "m:ur:buslv:A", "p.u., Phase Voltage, Real Part"
      m_ur_buslv_B = "m:ur:buslv:B", "p.u., Phase Voltage, Real Part"
      m_ur_buslv_C = "m:ur:buslv:C", "p.u., Phase Voltage, Real Part"
      m_ur_buslv_N = "m:ur:buslv:N", "p.u., Phase Voltage, Real Part"
      m_ur_bushvn = "m:ur:bushvn", "p.u., Phase Voltage, Real Part"
      m_ur_busmvn = "m:ur:busmvn", "p.u., Phase Voltage, Real Part"
      m_ur_buslvn = "m:ur:buslvn", "p.u., Phase Voltage, Real Part"
      m_ui_bushv_A = "m:ui:bushv:A", "p.u., Phase Voltage, Imaginary Part"
      m_ui_bushv_B = "m:ui:bushv:B", "p.u., Phase Voltage, Imaginary Part"
      m_ui_bushv_C = "m:ui:bushv:C", "p.u., Phase Voltage, Imaginary Part"
      m_ui_bushv_N = "m:ui:bushv:N", "p.u., Phase Voltage, Imaginary Part"
      m_ui_busmv_A = "m:ui:busmv:A", "p.u., Phase Voltage, Imaginary Part"
      m_ui_busmv_B = "m:ui:busmv:B", "p.u., Phase Voltage, Imaginary Part"
      m_ui_busmv_C = "m:ui:busmv:C", "p.u., Phase Voltage, Imaginary Part"
      m_ui_busmv_N = "m:ui:busmv:N", "p.u., Phase Voltage, Imaginary Part"
      m_ui_buslv_A = "m:ui:buslv:A", "p.u., Phase Voltage, Imaginary Part"
      m_ui_buslv_B = "m:ui:buslv:B", "p.u., Phase Voltage, Imaginary Part"
      m_ui_buslv_C = "m:ui:buslv:C", "p.u., Phase Voltage, Imaginary Part"
      m_ui_buslv_N = "m:ui:buslv:N", "p.u., Phase Voltage, Imaginary Part"
      m_ui_bushvn = "m:ui:bushvn", "p.u., Phase Voltage, Imaginary Part"
      m_ui_busmvn = "m:ui:busmvn", "p.u., Phase Voltage, Imaginary Part"
      m_ui_buslvn = "m:ui:buslvn", "p.u., Phase Voltage, Imaginary Part"
      m_u_bushv_A = "m:u:bushv:A", "p.u., Phase Voltage, Magnitude"
      m_u_bushv_B = "m:u:bushv:B", "p.u., Phase Voltage, Magnitude"
      m_u_bushv_C = "m:u:bushv:C", "p.u., Phase Voltage, Magnitude"
      m_u_bushv_N = "m:u:bushv:N", "p.u., Phase Voltage, Magnitude"
      m_u_busmv_A = "m:u:busmv:A", "p.u., Phase Voltage, Magnitude"
      m_u_busmv_B = "m:u:busmv:B", "p.u., Phase Voltage, Magnitude"
      m_u_busmv_C = "m:u:busmv:C", "p.u., Phase Voltage, Magnitude"
      m_u_busmv_N = "m:u:busmv:N", "p.u., Phase Voltage, Magnitude"
      m_u_buslv_A = "m:u:buslv:A", "p.u., Phase Voltage, Magnitude"
      m_u_buslv_B = "m:u:buslv:B", "p.u., Phase Voltage, Magnitude"
      m_u_buslv_C = "m:u:buslv:C", "p.u., Phase Voltage, Magnitude"
      m_u_buslv_N = "m:u:buslv:N", "p.u., Phase Voltage, Magnitude"
      m_u_bushvn = "m:u:bushvn", "p.u., Phase Voltage, Magnitude"
      m_u_busmvn = "m:u:busmvn", "p.u., Phase Voltage, Magnitude"
      m_u_buslvn = "m:u:buslvn", "p.u., Phase Voltage, Magnitude"
      m_ir_bushv_A = "m:ir:bushv:A", "p.u., Phase Current, Real Part"
      m_ir_bushv_B = "m:ir:bushv:B", "p.u., Phase Current, Real Part"
      m_ir_bushv_C = "m:ir:bushv:C", "p.u., Phase Current, Real Part"
      m_ir_bushv_N = "m:ir:bushv:N", "p.u., Phase Current, Real Part"
      m_ir_busmv_A = "m:ir:busmv:A", "p.u., Phase Current, Real Part"
      m_ir_busmv_B = "m:ir:busmv:B", "p.u., Phase Current, Real Part"
      m_ir_busmv_C = "m:ir:busmv:C", "p.u., Phase Current, Real Part"
      m_ir_busmv_N = "m:ir:busmv:N", "p.u., Phase Current, Real Part"
      m_ir_buslv_A = "m:ir:buslv:A", "p.u., Phase Current, Real Part"
      m_ir_buslv_B = "m:ir:buslv:B", "p.u., Phase Current, Real Part"
      m_ir_buslv_C = "m:ir:buslv:C", "p.u., Phase Current, Real Part"
      m_ir_buslv_N = "m:ir:buslv:N", "p.u., Phase Current, Real Part"
      m_ir_bushvn = "m:ir:bushvn", "p.u., Phase Current, Real Part"
      m_ir_busmvn = "m:ir:busmvn", "p.u., Phase Current, Real Part"
      m_ir_buslvn = "m:ir:buslvn", "p.u., Phase Current, Real Part"
      m_ii_bushv_A = "m:ii:bushv:A", "p.u., Phase Current, Imaginary Part"
      m_ii_bushv_B = "m:ii:bushv:B", "p.u., Phase Current, Imaginary Part"
      m_ii_bushv_C = "m:ii:bushv:C", "p.u., Phase Current, Imaginary Part"
      m_ii_bushv_N = "m:ii:bushv:N", "p.u., Phase Current, Imaginary Part"
      m_ii_busmv_A = "m:ii:busmv:A", "p.u., Phase Current, Imaginary Part"
      m_ii_busmv_B = "m:ii:busmv:B", "p.u., Phase Current, Imaginary Part"
      m_ii_busmv_C = "m:ii:busmv:C", "p.u., Phase Current, Imaginary Part"
      m_ii_busmv_N = "m:ii:busmv:N", "p.u., Phase Current, Imaginary Part"
      m_ii_buslv_A = "m:ii:buslv:A", "p.u., Phase Current, Imaginary Part"
      m_ii_buslv_B = "m:ii:buslv:B", "p.u., Phase Current, Imaginary Part"
      m_ii_buslv_C = "m:ii:buslv:C", "p.u., Phase Current, Imaginary Part"
      m_ii_buslv_N = "m:ii:buslv:N", "p.u., Phase Current, Imaginary Part"
      m_ii_bushvn = "m:ii:bushvn", "p.u., Phase Current, Imaginary Part"
      m_ii_busmvn = "m:ii:busmvn", "p.u., Phase Current, Imaginary Part"
      m_ii_buslvn = "m:ii:buslvn", "p.u., Phase Current, Imaginary Part"
      m_i_bushv_A = "m:i:bushv:A", "p.u., Phase Current, Magnitude"
      m_i_bushv_B = "m:i:bushv:B", "p.u., Phase Current, Magnitude"
      m_i_bushv_C = "m:i:bushv:C", "p.u., Phase Current, Magnitude"
      m_i_bushv_N = "m:i:bushv:N", "p.u., Phase Current, Magnitude"
      m_i_busmv_A = "m:i:busmv:A", "p.u., Phase Current, Magnitude"
      m_i_busmv_B = "m:i:busmv:B", "p.u., Phase Current, Magnitude"
      m_i_busmv_C = "m:i:busmv:C", "p.u., Phase Current, Magnitude"
      m_i_busmv_N = "m:i:busmv:N", "p.u., Phase Current, Magnitude"
      m_i_buslv_A = "m:i:buslv:A", "p.u., Phase Current, Magnitude"
      m_i_buslv_B = "m:i:buslv:B", "p.u., Phase Current, Magnitude"
      m_i_buslv_C = "m:i:buslv:C", "p.u., Phase Current, Magnitude"
      m_i_buslv_N = "m:i:buslv:N", "p.u., Phase Current, Magnitude"
      m_i_bushvn = "m:i:bushvn", "p.u., Phase Current, Magnitude"
      m_i_busmvn = "m:i:busmvn", "p.u., Phase Current, Magnitude"
      m_i_buslvn = "m:i:buslvn", "p.u., Phase Current, Magnitude"
      m_phiui_bushv_A = "m:phiui:bushv:A", "deg, Angle between Voltage and Current"
      m_phiui_bushv_B = "m:phiui:bushv:B", "deg, Angle between Voltage and Current"
      m_phiui_bushv_C = "m:phiui:bushv:C", "deg, Angle between Voltage and Current"
      m_phiui_bushv_N = "m:phiui:bushv:N", "deg, Angle between Voltage and Current"
      m_phiui_busmv_A = "m:phiui:busmv:A", "deg, Angle between Voltage and Current"
      m_phiui_busmv_B = "m:phiui:busmv:B", "deg, Angle between Voltage and Current"
      m_phiui_busmv_C = "m:phiui:busmv:C", "deg, Angle between Voltage and Current"
      m_phiui_busmv_N = "m:phiui:busmv:N", "deg, Angle between Voltage and Current"
      m_phiui_buslv_A = "m:phiui:buslv:A", "deg, Angle between Voltage and Current"
      m_phiui_buslv_B = "m:phiui:buslv:B", "deg, Angle between Voltage and Current"
      m_phiui_buslv_C = "m:phiui:buslv:C", "deg, Angle between Voltage and Current"
      m_phiui_buslv_N = "m:phiui:buslv:N", "deg, Angle between Voltage and Current"
      m_phiui_bushvn = "m:phiui:bushvn", "deg, Angle between Voltage and Current"
      m_phiui_busmvn = "m:phiui:busmvn", "deg, Angle between Voltage and Current"
      m_phiui_buslvn = "m:phiui:buslvn", "deg, Angle between Voltage and Current"
      m_phiu0i0_bushv = "m:phiu0i0:bushv", "deg, Angle between Voltage and Current in zero-sequence system"
      m_phiu0i0_busmv = "m:phiu0i0:busmv", "deg, Angle between Voltage and Current in zero-sequence system"
      m_phiu0i0_buslv = "m:phiu0i0:buslv", "deg, Angle between Voltage and Current in zero-sequence system"
      m_phiu0i0_bushvn = "m:phiu0i0:bushvn", "deg, Angle between Voltage and Current in zero-sequence system"
      m_phiu0i0_busmvn = "m:phiu0i0:busmvn", "deg, Angle between Voltage and Current in zero-sequence system"
      m_phiu0i0_buslvn = "m:phiu0i0:buslvn", "deg, Angle between Voltage and Current in zero-sequence system"
      m_phiu1i1_bushv = "m:phiu1i1:bushv", "deg, Angle between Voltage and Current in positive sequence system"
      m_phiu1i1_busmv = "m:phiu1i1:busmv", "deg, Angle between Voltage and Current in positive sequence system"
      m_phiu1i1_buslv = "m:phiu1i1:buslv", "deg, Angle between Voltage and Current in positive sequence system"
      m_phiu1i1_bushvn = "m:phiu1i1:bushvn", "deg, Angle between Voltage and Current in positive sequence system"
      m_phiu1i1_busmvn = "m:phiu1i1:busmvn", "deg, Angle between Voltage and Current in positive sequence system"
      m_phiu1i1_buslvn = "m:phiu1i1:buslvn", "deg, Angle between Voltage and Current in positive sequence system"
      m_phiu2i2_bushv = "m:phiu2i2:bushv", "deg, Angle between Voltage and Current in negative sequence system"
      m_phiu2i2_busmv = "m:phiu2i2:busmv", "deg, Angle between Voltage and Current in negative sequence system"
      m_phiu2i2_buslv = "m:phiu2i2:buslv", "deg, Angle between Voltage and Current in negative sequence system"
      m_phiu2i2_bushvn = "m:phiu2i2:bushvn", "deg, Angle between Voltage and Current in negative sequence system"
      m_phiu2i2_busmvn = "m:phiu2i2:busmvn", "deg, Angle between Voltage and Current in negative sequence system"
      m_phiu2i2_buslvn = "m:phiu2i2:buslvn", "deg, Angle between Voltage and Current in negative sequence system"
      m_Tfct_bushv = "m:Tfct:bushv", "s, Fault Clearing Time"
      m_Tfct_busmv = "m:Tfct:busmv", "s, Fault Clearing Time"
      m_Tfct_buslv = "m:Tfct:buslv", "s, Fault Clearing Time"
      m_Tfct_bushvn = "m:Tfct:bushvn", "s, Fault Clearing Time"
      m_Tfct_busmvn = "m:Tfct:busmvn", "s, Fault Clearing Time"
      m_Tfct_buslvn = "m:Tfct:buslvn", "s, Fault Clearing Time"
      m_TfctPh_bushv_A = "m:TfctPh:bushv:A", "s, Fault Clearing Time"
      m_TfctPh_bushv_B = "m:TfctPh:bushv:B", "s, Fault Clearing Time"
      m_TfctPh_bushv_C = "m:TfctPh:bushv:C", "s, Fault Clearing Time"
      m_TfctPh_bushv_N = "m:TfctPh:bushv:N", "s, Fault Clearing Time"
      m_TfctPh_busmv_A = "m:TfctPh:busmv:A", "s, Fault Clearing Time"
      m_TfctPh_busmv_B = "m:TfctPh:busmv:B", "s, Fault Clearing Time"
      m_TfctPh_busmv_C = "m:TfctPh:busmv:C", "s, Fault Clearing Time"
      m_TfctPh_busmv_N = "m:TfctPh:busmv:N", "s, Fault Clearing Time"
      m_TfctPh_buslv_A = "m:TfctPh:buslv:A", "s, Fault Clearing Time"
      m_TfctPh_buslv_B = "m:TfctPh:buslv:B", "s, Fault Clearing Time"
      m_TfctPh_buslv_C = "m:TfctPh:buslv:C", "s, Fault Clearing Time"
      m_TfctPh_buslv_N = "m:TfctPh:buslv:N", "s, Fault Clearing Time"
      m_TfctPh_bushvn = "m:TfctPh:bushvn", "s, Fault Clearing Time"
      m_TfctPh_busmvn = "m:TfctPh:busmvn", "s, Fault Clearing Time"
      m_TfctPh_buslvn = "m:TfctPh:buslvn", "s, Fault Clearing Time"
      m_Brkload_bushv = "m:Brkload:bushv", "%, Max. Breaker Loading"
      m_Brkload_busmv = "m:Brkload:busmv", "%, Max. Breaker Loading"
      m_Brkload_buslv = "m:Brkload:buslv", "%, Max. Breaker Loading"
      m_Brkload_bushvn = "m:Brkload:bushvn", "%, Max. Breaker Loading"
      m_Brkload_busmvn = "m:Brkload:busmvn", "%, Max. Breaker Loading"
      m_Brkload_buslvn = "m:Brkload:buslvn", "%, Max. Breaker Loading"
      m_BrkloadPh_bushv_A = "m:BrkloadPh:bushv:A", "%, Breaker Loading"
      m_BrkloadPh_bushv_B = "m:BrkloadPh:bushv:B", "%, Breaker Loading"
      m_BrkloadPh_bushv_C = "m:BrkloadPh:bushv:C", "%, Breaker Loading"
      m_BrkloadPh_bushv_N = "m:BrkloadPh:bushv:N", "%, Breaker Loading"
      m_BrkloadPh_busmv_A = "m:BrkloadPh:busmv:A", "%, Breaker Loading"
      m_BrkloadPh_busmv_B = "m:BrkloadPh:busmv:B", "%, Breaker Loading"
      m_BrkloadPh_busmv_C = "m:BrkloadPh:busmv:C", "%, Breaker Loading"
      m_BrkloadPh_busmv_N = "m:BrkloadPh:busmv:N", "%, Breaker Loading"
      m_BrkloadPh_buslv_A = "m:BrkloadPh:buslv:A", "%, Breaker Loading"
      m_BrkloadPh_buslv_B = "m:BrkloadPh:buslv:B", "%, Breaker Loading"
      m_BrkloadPh_buslv_C = "m:BrkloadPh:buslv:C", "%, Breaker Loading"
      m_BrkloadPh_buslv_N = "m:BrkloadPh:buslv:N", "%, Breaker Loading"
      m_BrkloadPh_bushvn = "m:BrkloadPh:bushvn", "%, Breaker Loading"
      m_BrkloadPh_busmvn = "m:BrkloadPh:busmvn", "%, Breaker Loading"
      m_BrkloadPh_buslvn = "m:BrkloadPh:buslvn", "%, Breaker Loading"
      n_ur_bushv_A = "n:ur:bushv:A", "p.u., Line-Ground Voltage, Real Part"
      n_ur_bushv_B = "n:ur:bushv:B", "p.u., Line-Ground Voltage, Real Part"
      n_ur_bushv_C = "n:ur:bushv:C", "p.u., Line-Ground Voltage, Real Part"
      n_ur_bushv_N = "n:ur:bushv:N", "p.u., Line-Ground Voltage, Real Part"
      n_ur_busmv_A = "n:ur:busmv:A", "p.u., Line-Ground Voltage, Real Part"
      n_ur_busmv_B = "n:ur:busmv:B", "p.u., Line-Ground Voltage, Real Part"
      n_ur_busmv_C = "n:ur:busmv:C", "p.u., Line-Ground Voltage, Real Part"
      n_ur_busmv_N = "n:ur:busmv:N", "p.u., Line-Ground Voltage, Real Part"
      n_ur_buslv_A = "n:ur:buslv:A", "p.u., Line-Ground Voltage, Real Part"
      n_ur_buslv_B = "n:ur:buslv:B", "p.u., Line-Ground Voltage, Real Part"
      n_ur_buslv_C = "n:ur:buslv:C", "p.u., Line-Ground Voltage, Real Part"
      n_ur_buslv_N = "n:ur:buslv:N", "p.u., Line-Ground Voltage, Real Part"
      n_ur_bushvn = "n:ur:bushvn", "p.u., Line-Ground Voltage, Real Part"
      n_ur_busmvn = "n:ur:busmvn", "p.u., Line-Ground Voltage, Real Part"
      n_ur_buslvn = "n:ur:buslvn", "p.u., Line-Ground Voltage, Real Part"
      n_ui_bushv_A = "n:ui:bushv:A", "p.u., Line-Ground Voltage, Imaginary Part"
      n_ui_bushv_B = "n:ui:bushv:B", "p.u., Line-Ground Voltage, Imaginary Part"
      n_ui_bushv_C = "n:ui:bushv:C", "p.u., Line-Ground Voltage, Imaginary Part"
      n_ui_bushv_N = "n:ui:bushv:N", "p.u., Line-Ground Voltage, Imaginary Part"
      n_ui_busmv_A = "n:ui:busmv:A", "p.u., Line-Ground Voltage, Imaginary Part"
      n_ui_busmv_B = "n:ui:busmv:B", "p.u., Line-Ground Voltage, Imaginary Part"
      n_ui_busmv_C = "n:ui:busmv:C", "p.u., Line-Ground Voltage, Imaginary Part"
      n_ui_busmv_N = "n:ui:busmv:N", "p.u., Line-Ground Voltage, Imaginary Part"
      n_ui_buslv_A = "n:ui:buslv:A", "p.u., Line-Ground Voltage, Imaginary Part"
      n_ui_buslv_B = "n:ui:buslv:B", "p.u., Line-Ground Voltage, Imaginary Part"
      n_ui_buslv_C = "n:ui:buslv:C", "p.u., Line-Ground Voltage, Imaginary Part"
      n_ui_buslv_N = "n:ui:buslv:N", "p.u., Line-Ground Voltage, Imaginary Part"
      n_ui_bushvn = "n:ui:bushvn", "p.u., Line-Ground Voltage, Imaginary Part"
      n_ui_busmvn = "n:ui:busmvn", "p.u., Line-Ground Voltage, Imaginary Part"
      n_ui_buslvn = "n:ui:buslvn", "p.u., Line-Ground Voltage, Imaginary Part"
      n_u_bushv_A = "n:u:bushv:A", "p.u., Line-Ground Voltage, Magnitude"
      n_u_bushv_B = "n:u:bushv:B", "p.u., Line-Ground Voltage, Magnitude"
      n_u_bushv_C = "n:u:bushv:C", "p.u., Line-Ground Voltage, Magnitude"
      n_u_bushv_N = "n:u:bushv:N", "p.u., Line-Ground Voltage, Magnitude"
      n_u_busmv_A = "n:u:busmv:A", "p.u., Line-Ground Voltage, Magnitude"
      n_u_busmv_B = "n:u:busmv:B", "p.u., Line-Ground Voltage, Magnitude"
      n_u_busmv_C = "n:u:busmv:C", "p.u., Line-Ground Voltage, Magnitude"
      n_u_busmv_N = "n:u:busmv:N", "p.u., Line-Ground Voltage, Magnitude"
      n_u_buslv_A = "n:u:buslv:A", "p.u., Line-Ground Voltage, Magnitude"
      n_u_buslv_B = "n:u:buslv:B", "p.u., Line-Ground Voltage, Magnitude"
      n_u_buslv_C = "n:u:buslv:C", "p.u., Line-Ground Voltage, Magnitude"
      n_u_buslv_N = "n:u:buslv:N", "p.u., Line-Ground Voltage, Magnitude"
      n_u_bushvn = "n:u:bushvn", "p.u., Line-Ground Voltage, Magnitude"
      n_u_busmvn = "n:u:busmvn", "p.u., Line-Ground Voltage, Magnitude"
      n_u_buslvn = "n:u:buslvn", "p.u., Line-Ground Voltage, Magnitude"
      n_upc_bushv_A = "n:upc:bushv:A", "%, Line-Ground Voltage, Magnitude"
      n_upc_bushv_B = "n:upc:bushv:B", "%, Line-Ground Voltage, Magnitude"
      n_upc_bushv_C = "n:upc:bushv:C", "%, Line-Ground Voltage, Magnitude"
      n_upc_bushv_N = "n:upc:bushv:N", "%, Line-Ground Voltage, Magnitude"
      n_upc_busmv_A = "n:upc:busmv:A", "%, Line-Ground Voltage, Magnitude"
      n_upc_busmv_B = "n:upc:busmv:B", "%, Line-Ground Voltage, Magnitude"
      n_upc_busmv_C = "n:upc:busmv:C", "%, Line-Ground Voltage, Magnitude"
      n_upc_busmv_N = "n:upc:busmv:N", "%, Line-Ground Voltage, Magnitude"
      n_upc_buslv_A = "n:upc:buslv:A", "%, Line-Ground Voltage, Magnitude"
      n_upc_buslv_B = "n:upc:buslv:B", "%, Line-Ground Voltage, Magnitude"
      n_upc_buslv_C = "n:upc:buslv:C", "%, Line-Ground Voltage, Magnitude"
      n_upc_buslv_N = "n:upc:buslv:N", "%, Line-Ground Voltage, Magnitude"
      n_upc_bushvn = "n:upc:bushvn", "%, Line-Ground Voltage, Magnitude"
      n_upc_busmvn = "n:upc:busmvn", "%, Line-Ground Voltage, Magnitude"
      n_upc_buslvn = "n:upc:buslvn", "%, Line-Ground Voltage, Magnitude"
      n_U_bushv_A = "n:U:bushv:A", "kV, Line-Ground Voltage, Magnitude"
      n_U_bushv_B = "n:U:bushv:B", "kV, Line-Ground Voltage, Magnitude"
      n_U_bushv_C = "n:U:bushv:C", "kV, Line-Ground Voltage, Magnitude"
      n_U_bushv_N = "n:U:bushv:N", "kV, Line-Ground Voltage, Magnitude"
      n_U_busmv_A = "n:U:busmv:A", "kV, Line-Ground Voltage, Magnitude"
      n_U_busmv_B = "n:U:busmv:B", "kV, Line-Ground Voltage, Magnitude"
      n_U_busmv_C = "n:U:busmv:C", "kV, Line-Ground Voltage, Magnitude"
      n_U_busmv_N = "n:U:busmv:N", "kV, Line-Ground Voltage, Magnitude"
      n_U_buslv_A = "n:U:buslv:A", "kV, Line-Ground Voltage, Magnitude"
      n_U_buslv_B = "n:U:buslv:B", "kV, Line-Ground Voltage, Magnitude"
      n_U_buslv_C = "n:U:buslv:C", "kV, Line-Ground Voltage, Magnitude"
      n_U_buslv_N = "n:U:buslv:N", "kV, Line-Ground Voltage, Magnitude"
      n_U_bushvn = "n:U:bushvn", "kV, Line-Ground Voltage, Magnitude"
      n_U_busmvn = "n:U:busmvn", "kV, Line-Ground Voltage, Magnitude"
      n_U_buslvn = "n:U:buslvn", "kV, Line-Ground Voltage, Magnitude"
      n_phiu_bushv_A = "n:phiu:bushv:A", "deg, Line-Ground Voltage, Angle"
      n_phiu_bushv_B = "n:phiu:bushv:B", "deg, Line-Ground Voltage, Angle"
      n_phiu_bushv_C = "n:phiu:bushv:C", "deg, Line-Ground Voltage, Angle"
      n_phiu_bushv_N = "n:phiu:bushv:N", "deg, Line-Ground Voltage, Angle"
      n_phiu_busmv_A = "n:phiu:busmv:A", "deg, Line-Ground Voltage, Angle"
      n_phiu_busmv_B = "n:phiu:busmv:B", "deg, Line-Ground Voltage, Angle"
      n_phiu_busmv_C = "n:phiu:busmv:C", "deg, Line-Ground Voltage, Angle"
      n_phiu_busmv_N = "n:phiu:busmv:N", "deg, Line-Ground Voltage, Angle"
      n_phiu_buslv_A = "n:phiu:buslv:A", "deg, Line-Ground Voltage, Angle"
      n_phiu_buslv_B = "n:phiu:buslv:B", "deg, Line-Ground Voltage, Angle"
      n_phiu_buslv_C = "n:phiu:buslv:C", "deg, Line-Ground Voltage, Angle"
      n_phiu_buslv_N = "n:phiu:buslv:N", "deg, Line-Ground Voltage, Angle"
      n_phiu_bushvn = "n:phiu:bushvn", "deg, Line-Ground Voltage, Angle"
      n_phiu_busmvn = "n:phiu:busmvn", "deg, Line-Ground Voltage, Angle"
      n_phiu_buslvn = "n:phiu:buslvn", "deg, Line-Ground Voltage, Angle"
      n_phiurel_bushv = "n:phiurel:bushv", "deg, Voltage, Relative Angle"
      n_phiurel_busmv = "n:phiurel:busmv", "deg, Voltage, Relative Angle"
      n_phiurel_buslv = "n:phiurel:buslv", "deg, Voltage, Relative Angle"
      n_phiurel_bushvn = "n:phiurel:bushvn", "deg, Voltage, Relative Angle"
      n_phiurel_busmvn = "n:phiurel:busmvn", "deg, Voltage, Relative Angle"
      n_phiurel_buslvn = "n:phiurel:buslvn", "deg, Voltage, Relative Angle"
      n_ul_bushv_A = "n:ul:bushv:A", "p.u., Line-Line Voltage, Magnitude"
      n_ul_bushv_B = "n:ul:bushv:B", "p.u., Line-Line Voltage, Magnitude"
      n_ul_bushv_C = "n:ul:bushv:C", "p.u., Line-Line Voltage, Magnitude"
      n_ul_bushv_N = "n:ul:bushv:N", "p.u., Line-Line Voltage, Magnitude"
      n_ul_busmv_A = "n:ul:busmv:A", "p.u., Line-Line Voltage, Magnitude"
      n_ul_busmv_B = "n:ul:busmv:B", "p.u., Line-Line Voltage, Magnitude"
      n_ul_busmv_C = "n:ul:busmv:C", "p.u., Line-Line Voltage, Magnitude"
      n_ul_busmv_N = "n:ul:busmv:N", "p.u., Line-Line Voltage, Magnitude"
      n_ul_buslv_A = "n:ul:buslv:A", "p.u., Line-Line Voltage, Magnitude"
      n_ul_buslv_B = "n:ul:buslv:B", "p.u., Line-Line Voltage, Magnitude"
      n_ul_buslv_C = "n:ul:buslv:C", "p.u., Line-Line Voltage, Magnitude"
      n_ul_buslv_N = "n:ul:buslv:N", "p.u., Line-Line Voltage, Magnitude"
      n_ul_bushvn = "n:ul:bushvn", "p.u., Line-Line Voltage, Magnitude"
      n_ul_busmvn = "n:ul:busmvn", "p.u., Line-Line Voltage, Magnitude"
      n_ul_buslvn = "n:ul:buslvn", "p.u., Line-Line Voltage, Magnitude"
      n_ulpc_bushv_A = "n:ulpc:bushv:A", "%, Line-Line Voltage, Magnitude"
      n_ulpc_bushv_B = "n:ulpc:bushv:B", "%, Line-Line Voltage, Magnitude"
      n_ulpc_bushv_C = "n:ulpc:bushv:C", "%, Line-Line Voltage, Magnitude"
      n_ulpc_bushv_N = "n:ulpc:bushv:N", "%, Line-Line Voltage, Magnitude"
      n_ulpc_busmv_A = "n:ulpc:busmv:A", "%, Line-Line Voltage, Magnitude"
      n_ulpc_busmv_B = "n:ulpc:busmv:B", "%, Line-Line Voltage, Magnitude"
      n_ulpc_busmv_C = "n:ulpc:busmv:C", "%, Line-Line Voltage, Magnitude"
      n_ulpc_busmv_N = "n:ulpc:busmv:N", "%, Line-Line Voltage, Magnitude"
      n_ulpc_buslv_A = "n:ulpc:buslv:A", "%, Line-Line Voltage, Magnitude"
      n_ulpc_buslv_B = "n:ulpc:buslv:B", "%, Line-Line Voltage, Magnitude"
      n_ulpc_buslv_C = "n:ulpc:buslv:C", "%, Line-Line Voltage, Magnitude"
      n_ulpc_buslv_N = "n:ulpc:buslv:N", "%, Line-Line Voltage, Magnitude"
      n_ulpc_bushvn = "n:ulpc:bushvn", "%, Line-Line Voltage, Magnitude"
      n_ulpc_busmvn = "n:ulpc:busmvn", "%, Line-Line Voltage, Magnitude"
      n_ulpc_buslvn = "n:ulpc:buslvn", "%, Line-Line Voltage, Magnitude"
      n_Ul_bushv_A = "n:Ul:bushv:A", "kV, Line-Line Voltage, Magnitude"
      n_Ul_bushv_B = "n:Ul:bushv:B", "kV, Line-Line Voltage, Magnitude"
      n_Ul_bushv_C = "n:Ul:bushv:C", "kV, Line-Line Voltage, Magnitude"
      n_Ul_bushv_N = "n:Ul:bushv:N", "kV, Line-Line Voltage, Magnitude"
      n_Ul_busmv_A = "n:Ul:busmv:A", "kV, Line-Line Voltage, Magnitude"
      n_Ul_busmv_B = "n:Ul:busmv:B", "kV, Line-Line Voltage, Magnitude"
      n_Ul_busmv_C = "n:Ul:busmv:C", "kV, Line-Line Voltage, Magnitude"
      n_Ul_busmv_N = "n:Ul:busmv:N", "kV, Line-Line Voltage, Magnitude"
      n_Ul_buslv_A = "n:Ul:buslv:A", "kV, Line-Line Voltage, Magnitude"
      n_Ul_buslv_B = "n:Ul:buslv:B", "kV, Line-Line Voltage, Magnitude"
      n_Ul_buslv_C = "n:Ul:buslv:C", "kV, Line-Line Voltage, Magnitude"
      n_Ul_buslv_N = "n:Ul:buslv:N", "kV, Line-Line Voltage, Magnitude"
      n_Ul_bushvn = "n:Ul:bushvn", "kV, Line-Line Voltage, Magnitude"
      n_Ul_busmvn = "n:Ul:busmvn", "kV, Line-Line Voltage, Magnitude"
      n_Ul_buslvn = "n:Ul:buslvn", "kV, Line-Line Voltage, Magnitude"
      n_phiul_bushv_A = "n:phiul:bushv:A", "deg, Line-Line Voltage, Angle"
      n_phiul_bushv_B = "n:phiul:bushv:B", "deg, Line-Line Voltage, Angle"
      n_phiul_bushv_C = "n:phiul:bushv:C", "deg, Line-Line Voltage, Angle"
      n_phiul_bushv_N = "n:phiul:bushv:N", "deg, Line-Line Voltage, Angle"
      n_phiul_busmv_A = "n:phiul:busmv:A", "deg, Line-Line Voltage, Angle"
      n_phiul_busmv_B = "n:phiul:busmv:B", "deg, Line-Line Voltage, Angle"
      n_phiul_busmv_C = "n:phiul:busmv:C", "deg, Line-Line Voltage, Angle"
      n_phiul_busmv_N = "n:phiul:busmv:N", "deg, Line-Line Voltage, Angle"
      n_phiul_buslv_A = "n:phiul:buslv:A", "deg, Line-Line Voltage, Angle"
      n_phiul_buslv_B = "n:phiul:buslv:B", "deg, Line-Line Voltage, Angle"
      n_phiul_buslv_C = "n:phiul:buslv:C", "deg, Line-Line Voltage, Angle"
      n_phiul_buslv_N = "n:phiul:buslv:N", "deg, Line-Line Voltage, Angle"
      n_phiul_bushvn = "n:phiul:bushvn", "deg, Line-Line Voltage, Angle"
      n_phiul_busmvn = "n:phiul:busmvn", "deg, Line-Line Voltage, Angle"
      n_phiul_buslvn = "n:phiul:buslvn", "deg, Line-Line Voltage, Angle"
      n_u0_bushv = "n:u0:bushv", "p.u., Zero-Sequence Voltage, Magnitude"
      n_u0_busmv = "n:u0:busmv", "p.u., Zero-Sequence Voltage, Magnitude"
      n_u0_buslv = "n:u0:buslv", "p.u., Zero-Sequence Voltage, Magnitude"
      n_u0_bushvn = "n:u0:bushvn", "p.u., Zero-Sequence Voltage, Magnitude"
      n_u0_busmvn = "n:u0:busmvn", "p.u., Zero-Sequence Voltage, Magnitude"
      n_u0_buslvn = "n:u0:buslvn", "p.u., Zero-Sequence Voltage, Magnitude"
      n_U0_bushv = "n:U0:bushv", "kV, Zero-Sequence Voltage, Magnitude"
      n_U0_busmv = "n:U0:busmv", "kV, Zero-Sequence Voltage, Magnitude"
      n_U0_buslv = "n:U0:buslv", "kV, Zero-Sequence Voltage, Magnitude"
      n_U0_bushvn = "n:U0:bushvn", "kV, Zero-Sequence Voltage, Magnitude"
      n_U0_busmvn = "n:U0:busmvn", "kV, Zero-Sequence Voltage, Magnitude"
      n_U0_buslvn = "n:U0:buslvn", "kV, Zero-Sequence Voltage, Magnitude"
      n_U0x3_bushv = "n:U0x3:bushv", "kV, 3*U0"
      n_U0x3_busmv = "n:U0x3:busmv", "kV, 3*U0"
      n_U0x3_buslv = "n:U0x3:buslv", "kV, 3*U0"
      n_U0x3_bushvn = "n:U0x3:bushvn", "kV, 3*U0"
      n_U0x3_busmvn = "n:U0x3:busmvn", "kV, 3*U0"
      n_U0x3_buslvn = "n:U0x3:buslvn", "kV, 3*U0"
      n_phiu0_bushv = "n:phiu0:bushv", "deg, Zero-Sequence Voltage, Angle"
      n_phiu0_busmv = "n:phiu0:busmv", "deg, Zero-Sequence Voltage, Angle"
      n_phiu0_buslv = "n:phiu0:buslv", "deg, Zero-Sequence Voltage, Angle"
      n_phiu0_bushvn = "n:phiu0:bushvn", "deg, Zero-Sequence Voltage, Angle"
      n_phiu0_busmvn = "n:phiu0:busmvn", "deg, Zero-Sequence Voltage, Angle"
      n_phiu0_buslvn = "n:phiu0:buslvn", "deg, Zero-Sequence Voltage, Angle"
      n_u1_bushv = "n:u1:bushv", "p.u., Positive-Sequence Voltage, Magnitude"
      n_u1_busmv = "n:u1:busmv", "p.u., Positive-Sequence Voltage, Magnitude"
      n_u1_buslv = "n:u1:buslv", "p.u., Positive-Sequence Voltage, Magnitude"
      n_u1_bushvn = "n:u1:bushvn", "p.u., Positive-Sequence Voltage, Magnitude"
      n_u1_busmvn = "n:u1:busmvn", "p.u., Positive-Sequence Voltage, Magnitude"
      n_u1_buslvn = "n:u1:buslvn", "p.u., Positive-Sequence Voltage, Magnitude"
      n_u1pc_bushv = "n:u1pc:bushv", "%, Positive-Sequence Voltage, Magnitude"
      n_u1pc_busmv = "n:u1pc:busmv", "%, Positive-Sequence Voltage, Magnitude"
      n_u1pc_buslv = "n:u1pc:buslv", "%, Positive-Sequence Voltage, Magnitude"
      n_u1pc_bushvn = "n:u1pc:bushvn", "%, Positive-Sequence Voltage, Magnitude"
      n_u1pc_busmvn = "n:u1pc:busmvn", "%, Positive-Sequence Voltage, Magnitude"
      n_u1pc_buslvn = "n:u1pc:buslvn", "%, Positive-Sequence Voltage, Magnitude"
      n_u1r_bushv = "n:u1r:bushv", "p.u., Positive-Sequence Voltage, Real Part"
      n_u1r_busmv = "n:u1r:busmv", "p.u., Positive-Sequence Voltage, Real Part"
      n_u1r_buslv = "n:u1r:buslv", "p.u., Positive-Sequence Voltage, Real Part"
      n_u1r_bushvn = "n:u1r:bushvn", "p.u., Positive-Sequence Voltage, Real Part"
      n_u1r_busmvn = "n:u1r:busmvn", "p.u., Positive-Sequence Voltage, Real Part"
      n_u1r_buslvn = "n:u1r:buslvn", "p.u., Positive-Sequence Voltage, Real Part"
      n_u1i_bushv = "n:u1i:bushv", "p.u., Positive-Sequence Voltage, Imaginary Part"
      n_u1i_busmv = "n:u1i:busmv", "p.u., Positive-Sequence Voltage, Imaginary Part"
      n_u1i_buslv = "n:u1i:buslv", "p.u., Positive-Sequence Voltage, Imaginary Part"
      n_u1i_bushvn = "n:u1i:bushvn", "p.u., Positive-Sequence Voltage, Imaginary Part"
      n_u1i_busmvn = "n:u1i:busmvn", "p.u., Positive-Sequence Voltage, Imaginary Part"
      n_u1i_buslvn = "n:u1i:buslvn", "p.u., Positive-Sequence Voltage, Imaginary Part"
      n_U1_bushv = "n:U1:bushv", "kV, Line-Ground Positive-Sequence Voltage, Magnitude"
      n_U1_busmv = "n:U1:busmv", "kV, Line-Ground Positive-Sequence Voltage, Magnitude"
      n_U1_buslv = "n:U1:buslv", "kV, Line-Ground Positive-Sequence Voltage, Magnitude"
      n_U1_bushvn = "n:U1:bushvn", "kV, Line-Ground Positive-Sequence Voltage, Magnitude"
      n_U1_busmvn = "n:U1:busmvn", "kV, Line-Ground Positive-Sequence Voltage, Magnitude"
      n_U1_buslvn = "n:U1:buslvn", "kV, Line-Ground Positive-Sequence Voltage, Magnitude"
      n_phiu1_bushv = "n:phiu1:bushv", "deg, Positive-Sequence Voltage, Angle"
      n_phiu1_busmv = "n:phiu1:busmv", "deg, Positive-Sequence Voltage, Angle"
      n_phiu1_buslv = "n:phiu1:buslv", "deg, Positive-Sequence Voltage, Angle"
      n_phiu1_bushvn = "n:phiu1:bushvn", "deg, Positive-Sequence Voltage, Angle"
      n_phiu1_busmvn = "n:phiu1:busmvn", "deg, Positive-Sequence Voltage, Angle"
      n_phiu1_buslvn = "n:phiu1:buslvn", "deg, Positive-Sequence Voltage, Angle"
      n_u2_bushv = "n:u2:bushv", "p.u., Negative-Sequence Voltage, Magnitude"
      n_u2_busmv = "n:u2:busmv", "p.u., Negative-Sequence Voltage, Magnitude"
      n_u2_buslv = "n:u2:buslv", "p.u., Negative-Sequence Voltage, Magnitude"
      n_u2_bushvn = "n:u2:bushvn", "p.u., Negative-Sequence Voltage, Magnitude"
      n_u2_busmvn = "n:u2:busmvn", "p.u., Negative-Sequence Voltage, Magnitude"
      n_u2_buslvn = "n:u2:buslvn", "p.u., Negative-Sequence Voltage, Magnitude"
      n_U2_bushv = "n:U2:bushv", "kV, Line-Ground Negative-Sequence Voltage, Magnitude"
      n_U2_busmv = "n:U2:busmv", "kV, Line-Ground Negative-Sequence Voltage, Magnitude"
      n_U2_buslv = "n:U2:buslv", "kV, Line-Ground Negative-Sequence Voltage, Magnitude"
      n_U2_bushvn = "n:U2:bushvn", "kV, Line-Ground Negative-Sequence Voltage, Magnitude"
      n_U2_busmvn = "n:U2:busmvn", "kV, Line-Ground Negative-Sequence Voltage, Magnitude"
      n_U2_buslvn = "n:U2:buslvn", "kV, Line-Ground Negative-Sequence Voltage, Magnitude"
      n_phiu2_bushv = "n:phiu2:bushv", "deg, Negative-Sequence Voltage, Angle"
      n_phiu2_busmv = "n:phiu2:busmv", "deg, Negative-Sequence Voltage, Angle"
      n_phiu2_buslv = "n:phiu2:buslv", "deg, Negative-Sequence Voltage, Angle"
      n_phiu2_bushvn = "n:phiu2:bushvn", "deg, Negative-Sequence Voltage, Angle"
      n_phiu2_busmvn = "n:phiu2:busmvn", "deg, Negative-Sequence Voltage, Angle"
      n_phiu2_buslvn = "n:phiu2:buslvn", "deg, Negative-Sequence Voltage, Angle"
      n_U1l_bushv = "n:U1l:bushv", "kV, Line-Line Positive-Sequence Voltage, Magnitude"
      n_U1l_busmv = "n:U1l:busmv", "kV, Line-Line Positive-Sequence Voltage, Magnitude"
      n_U1l_buslv = "n:U1l:buslv", "kV, Line-Line Positive-Sequence Voltage, Magnitude"
      n_U1l_bushvn = "n:U1l:bushvn", "kV, Line-Line Positive-Sequence Voltage, Magnitude"
      n_U1l_busmvn = "n:U1l:busmvn", "kV, Line-Line Positive-Sequence Voltage, Magnitude"
      n_U1l_buslvn = "n:U1l:buslvn", "kV, Line-Line Positive-Sequence Voltage, Magnitude"
      n_U2l_bushv = "n:U2l:bushv", "kV, Line-Line Negative-Sequence Voltage, Magnitude"
      n_U2l_busmv = "n:U2l:busmv", "kV, Line-Line Negative-Sequence Voltage, Magnitude"
      n_U2l_buslv = "n:U2l:buslv", "kV, Line-Line Negative-Sequence Voltage, Magnitude"
      n_U2l_bushvn = "n:U2l:bushvn", "kV, Line-Line Negative-Sequence Voltage, Magnitude"
      n_U2l_busmvn = "n:U2l:busmvn", "kV, Line-Line Negative-Sequence Voltage, Magnitude"
      n_U2l_buslvn = "n:U2l:buslvn", "kV, Line-Line Negative-Sequence Voltage, Magnitude"
      n_uln_bushv_A = "n:uln:bushv:A", "p.u., Line-Neutral Voltage, Magnitude"
      n_uln_bushv_B = "n:uln:bushv:B", "p.u., Line-Neutral Voltage, Magnitude"
      n_uln_bushv_C = "n:uln:bushv:C", "p.u., Line-Neutral Voltage, Magnitude"
      n_uln_bushv_N = "n:uln:bushv:N", "p.u., Line-Neutral Voltage, Magnitude"
      n_uln_busmv_A = "n:uln:busmv:A", "p.u., Line-Neutral Voltage, Magnitude"
      n_uln_busmv_B = "n:uln:busmv:B", "p.u., Line-Neutral Voltage, Magnitude"
      n_uln_busmv_C = "n:uln:busmv:C", "p.u., Line-Neutral Voltage, Magnitude"
      n_uln_busmv_N = "n:uln:busmv:N", "p.u., Line-Neutral Voltage, Magnitude"
      n_uln_buslv_A = "n:uln:buslv:A", "p.u., Line-Neutral Voltage, Magnitude"
      n_uln_buslv_B = "n:uln:buslv:B", "p.u., Line-Neutral Voltage, Magnitude"
      n_uln_buslv_C = "n:uln:buslv:C", "p.u., Line-Neutral Voltage, Magnitude"
      n_uln_buslv_N = "n:uln:buslv:N", "p.u., Line-Neutral Voltage, Magnitude"
      n_uln_bushvn = "n:uln:bushvn", "p.u., Line-Neutral Voltage, Magnitude"
      n_uln_busmvn = "n:uln:busmvn", "p.u., Line-Neutral Voltage, Magnitude"
      n_uln_buslvn = "n:uln:buslvn", "p.u., Line-Neutral Voltage, Magnitude"
      n_Uln_bushv_A = "n:Uln:bushv:A", "kV, Line-Neutral Voltage, Magnitude"
      n_Uln_bushv_B = "n:Uln:bushv:B", "kV, Line-Neutral Voltage, Magnitude"
      n_Uln_bushv_C = "n:Uln:bushv:C", "kV, Line-Neutral Voltage, Magnitude"
      n_Uln_bushv_N = "n:Uln:bushv:N", "kV, Line-Neutral Voltage, Magnitude"
      n_Uln_busmv_A = "n:Uln:busmv:A", "kV, Line-Neutral Voltage, Magnitude"
      n_Uln_busmv_B = "n:Uln:busmv:B", "kV, Line-Neutral Voltage, Magnitude"
      n_Uln_busmv_C = "n:Uln:busmv:C", "kV, Line-Neutral Voltage, Magnitude"
      n_Uln_busmv_N = "n:Uln:busmv:N", "kV, Line-Neutral Voltage, Magnitude"
      n_Uln_buslv_A = "n:Uln:buslv:A", "kV, Line-Neutral Voltage, Magnitude"
      n_Uln_buslv_B = "n:Uln:buslv:B", "kV, Line-Neutral Voltage, Magnitude"
      n_Uln_buslv_C = "n:Uln:buslv:C", "kV, Line-Neutral Voltage, Magnitude"
      n_Uln_buslv_N = "n:Uln:buslv:N", "kV, Line-Neutral Voltage, Magnitude"
      n_Uln_bushvn = "n:Uln:bushvn", "kV, Line-Neutral Voltage, Magnitude"
      n_Uln_busmvn = "n:Uln:busmvn", "kV, Line-Neutral Voltage, Magnitude"
      n_Uln_buslvn = "n:Uln:buslvn", "kV, Line-Neutral Voltage, Magnitude"
      n_phiuln_bushv_A = "n:phiuln:bushv:A", "deg, Line-Neutral Voltage, Angle"
      n_phiuln_bushv_B = "n:phiuln:bushv:B", "deg, Line-Neutral Voltage, Angle"
      n_phiuln_bushv_C = "n:phiuln:bushv:C", "deg, Line-Neutral Voltage, Angle"
      n_phiuln_bushv_N = "n:phiuln:bushv:N", "deg, Line-Neutral Voltage, Angle"
      n_phiuln_busmv_A = "n:phiuln:busmv:A", "deg, Line-Neutral Voltage, Angle"
      n_phiuln_busmv_B = "n:phiuln:busmv:B", "deg, Line-Neutral Voltage, Angle"
      n_phiuln_busmv_C = "n:phiuln:busmv:C", "deg, Line-Neutral Voltage, Angle"
      n_phiuln_busmv_N = "n:phiuln:busmv:N", "deg, Line-Neutral Voltage, Angle"
      n_phiuln_buslv_A = "n:phiuln:buslv:A", "deg, Line-Neutral Voltage, Angle"
      n_phiuln_buslv_B = "n:phiuln:buslv:B", "deg, Line-Neutral Voltage, Angle"
      n_phiuln_buslv_C = "n:phiuln:buslv:C", "deg, Line-Neutral Voltage, Angle"
      n_phiuln_buslv_N = "n:phiuln:buslv:N", "deg, Line-Neutral Voltage, Angle"
      n_phiuln_bushvn = "n:phiuln:bushvn", "deg, Line-Neutral Voltage, Angle"
      n_phiuln_busmvn = "n:phiuln:busmvn", "deg, Line-Neutral Voltage, Angle"
      n_phiuln_buslvn = "n:phiuln:buslvn", "deg, Line-Neutral Voltage, Angle"
      n_un_bushv = "n:un:bushv", "p.u., Neutral-Ground Voltage, Magnitude"
      n_un_busmv = "n:un:busmv", "p.u., Neutral-Ground Voltage, Magnitude"
      n_un_buslv = "n:un:buslv", "p.u., Neutral-Ground Voltage, Magnitude"
      n_un_bushvn = "n:un:bushvn", "p.u., Neutral-Ground Voltage, Magnitude"
      n_un_busmvn = "n:un:busmvn", "p.u., Neutral-Ground Voltage, Magnitude"
      n_un_buslvn = "n:un:buslvn", "p.u., Neutral-Ground Voltage, Magnitude"
      n_Un_bushv = "n:Un:bushv", "kV, Neutral-Ground Voltage, Magnitude"
      n_Un_busmv = "n:Un:busmv", "kV, Neutral-Ground Voltage, Magnitude"
      n_Un_buslv = "n:Un:buslv", "kV, Neutral-Ground Voltage, Magnitude"
      n_Un_bushvn = "n:Un:bushvn", "kV, Neutral-Ground Voltage, Magnitude"
      n_Un_busmvn = "n:Un:busmvn", "kV, Neutral-Ground Voltage, Magnitude"
      n_Un_buslvn = "n:Un:buslvn", "kV, Neutral-Ground Voltage, Magnitude"
      n_phiun_bushv = "n:phiun:bushv", "deg, Neutral-Ground Voltage, Angle"
      n_phiun_busmv = "n:phiun:busmv", "deg, Neutral-Ground Voltage, Angle"
      n_phiun_buslv = "n:phiun:buslv", "deg, Neutral-Ground Voltage, Angle"
      n_phiun_bushvn = "n:phiun:bushvn", "deg, Neutral-Ground Voltage, Angle"
      n_phiun_busmvn = "n:phiun:busmvn", "deg, Neutral-Ground Voltage, Angle"
      n_phiun_buslvn = "n:phiun:buslvn", "deg, Neutral-Ground Voltage, Angle"
      n_upht_bushv_A = "n:upht:bushv:A", "p.u., Phase Technology dependent Voltage, Magnitude"
      n_upht_bushv_B = "n:upht:bushv:B", "p.u., Phase Technology dependent Voltage, Magnitude"
      n_upht_bushv_C = "n:upht:bushv:C", "p.u., Phase Technology dependent Voltage, Magnitude"
      n_upht_bushv_N = "n:upht:bushv:N", "p.u., Phase Technology dependent Voltage, Magnitude"
      n_upht_busmv_A = "n:upht:busmv:A", "p.u., Phase Technology dependent Voltage, Magnitude"
      n_upht_busmv_B = "n:upht:busmv:B", "p.u., Phase Technology dependent Voltage, Magnitude"
      n_upht_busmv_C = "n:upht:busmv:C", "p.u., Phase Technology dependent Voltage, Magnitude"
      n_upht_busmv_N = "n:upht:busmv:N", "p.u., Phase Technology dependent Voltage, Magnitude"
      n_upht_buslv_A = "n:upht:buslv:A", "p.u., Phase Technology dependent Voltage, Magnitude"
      n_upht_buslv_B = "n:upht:buslv:B", "p.u., Phase Technology dependent Voltage, Magnitude"
      n_upht_buslv_C = "n:upht:buslv:C", "p.u., Phase Technology dependent Voltage, Magnitude"
      n_upht_buslv_N = "n:upht:buslv:N", "p.u., Phase Technology dependent Voltage, Magnitude"
      n_upht_bushvn = "n:upht:bushvn", "p.u., Phase Technology dependent Voltage, Magnitude"
      n_upht_busmvn = "n:upht:busmvn", "p.u., Phase Technology dependent Voltage, Magnitude"
      n_upht_buslvn = "n:upht:buslvn", "p.u., Phase Technology dependent Voltage, Magnitude"
      n_Upht_bushv_A = "n:Upht:bushv:A", "kV, Phase Technology dependent Voltage, Magnitude"
      n_Upht_bushv_B = "n:Upht:bushv:B", "kV, Phase Technology dependent Voltage, Magnitude"
      n_Upht_bushv_C = "n:Upht:bushv:C", "kV, Phase Technology dependent Voltage, Magnitude"
      n_Upht_bushv_N = "n:Upht:bushv:N", "kV, Phase Technology dependent Voltage, Magnitude"
      n_Upht_busmv_A = "n:Upht:busmv:A", "kV, Phase Technology dependent Voltage, Magnitude"
      n_Upht_busmv_B = "n:Upht:busmv:B", "kV, Phase Technology dependent Voltage, Magnitude"
      n_Upht_busmv_C = "n:Upht:busmv:C", "kV, Phase Technology dependent Voltage, Magnitude"
      n_Upht_busmv_N = "n:Upht:busmv:N", "kV, Phase Technology dependent Voltage, Magnitude"
      n_Upht_buslv_A = "n:Upht:buslv:A", "kV, Phase Technology dependent Voltage, Magnitude"
      n_Upht_buslv_B = "n:Upht:buslv:B", "kV, Phase Technology dependent Voltage, Magnitude"
      n_Upht_buslv_C = "n:Upht:buslv:C", "kV, Phase Technology dependent Voltage, Magnitude"
      n_Upht_buslv_N = "n:Upht:buslv:N", "kV, Phase Technology dependent Voltage, Magnitude"
      n_Upht_bushvn = "n:Upht:bushvn", "kV, Phase Technology dependent Voltage, Magnitude"
      n_Upht_busmvn = "n:Upht:busmvn", "kV, Phase Technology dependent Voltage, Magnitude"
      n_Upht_buslvn = "n:Upht:buslvn", "kV, Phase Technology dependent Voltage, Magnitude"
      n_um_bushv = "n:um:bushv", "p.u., Average-Voltage, Magnitude"
      n_um_busmv = "n:um:busmv", "p.u., Average-Voltage, Magnitude"
      n_um_buslv = "n:um:buslv", "p.u., Average-Voltage, Magnitude"
      n_um_bushvn = "n:um:bushvn", "p.u., Average-Voltage, Magnitude"
      n_um_busmvn = "n:um:busmvn", "p.u., Average-Voltage, Magnitude"
      n_um_buslvn = "n:um:buslvn", "p.u., Average-Voltage, Magnitude"
      n_Um_bushv = "n:Um:bushv", "kV, Average-Voltage, Magnitude"
      n_Um_busmv = "n:Um:busmv", "kV, Average-Voltage, Magnitude"
      n_Um_buslv = "n:Um:buslv", "kV, Average-Voltage, Magnitude"
      n_Um_bushvn = "n:Um:bushvn", "kV, Average-Voltage, Magnitude"
      n_Um_busmvn = "n:Um:busmvn", "kV, Average-Voltage, Magnitude"
      n_Um_buslvn = "n:Um:buslvn", "kV, Average-Voltage, Magnitude"
      n_ishcr_bushv_A = "n:ishcr:bushv:A", "p.u., Short-Circuit Current, real"
      n_ishcr_bushv_B = "n:ishcr:bushv:B", "p.u., Short-Circuit Current, real"
      n_ishcr_bushv_C = "n:ishcr:bushv:C", "p.u., Short-Circuit Current, real"
      n_ishcr_bushv_N = "n:ishcr:bushv:N", "p.u., Short-Circuit Current, real"
      n_ishcr_busmv_A = "n:ishcr:busmv:A", "p.u., Short-Circuit Current, real"
      n_ishcr_busmv_B = "n:ishcr:busmv:B", "p.u., Short-Circuit Current, real"
      n_ishcr_busmv_C = "n:ishcr:busmv:C", "p.u., Short-Circuit Current, real"
      n_ishcr_busmv_N = "n:ishcr:busmv:N", "p.u., Short-Circuit Current, real"
      n_ishcr_buslv_A = "n:ishcr:buslv:A", "p.u., Short-Circuit Current, real"
      n_ishcr_buslv_B = "n:ishcr:buslv:B", "p.u., Short-Circuit Current, real"
      n_ishcr_buslv_C = "n:ishcr:buslv:C", "p.u., Short-Circuit Current, real"
      n_ishcr_buslv_N = "n:ishcr:buslv:N", "p.u., Short-Circuit Current, real"
      n_ishcr_bushvn = "n:ishcr:bushvn", "p.u., Short-Circuit Current, real"
      n_ishcr_busmvn = "n:ishcr:busmvn", "p.u., Short-Circuit Current, real"
      n_ishcr_buslvn = "n:ishcr:buslvn", "p.u., Short-Circuit Current, real"
      n_ishci_bushv_A = "n:ishci:bushv:A", "p.u., Short-Circuit Current, imag"
      n_ishci_bushv_B = "n:ishci:bushv:B", "p.u., Short-Circuit Current, imag"
      n_ishci_bushv_C = "n:ishci:bushv:C", "p.u., Short-Circuit Current, imag"
      n_ishci_bushv_N = "n:ishci:bushv:N", "p.u., Short-Circuit Current, imag"
      n_ishci_busmv_A = "n:ishci:busmv:A", "p.u., Short-Circuit Current, imag"
      n_ishci_busmv_B = "n:ishci:busmv:B", "p.u., Short-Circuit Current, imag"
      n_ishci_busmv_C = "n:ishci:busmv:C", "p.u., Short-Circuit Current, imag"
      n_ishci_busmv_N = "n:ishci:busmv:N", "p.u., Short-Circuit Current, imag"
      n_ishci_buslv_A = "n:ishci:buslv:A", "p.u., Short-Circuit Current, imag"
      n_ishci_buslv_B = "n:ishci:buslv:B", "p.u., Short-Circuit Current, imag"
      n_ishci_buslv_C = "n:ishci:buslv:C", "p.u., Short-Circuit Current, imag"
      n_ishci_buslv_N = "n:ishci:buslv:N", "p.u., Short-Circuit Current, imag"
      n_ishci_bushvn = "n:ishci:bushvn", "p.u., Short-Circuit Current, imag"
      n_ishci_busmvn = "n:ishci:busmvn", "p.u., Short-Circuit Current, imag"
      n_ishci_buslvn = "n:ishci:buslvn", "p.u., Short-Circuit Current, imag"
      n_ishc_bushv_A = "n:ishc:bushv:A", "p.u., Short-Circuit Current, magnitude"
      n_ishc_bushv_B = "n:ishc:bushv:B", "p.u., Short-Circuit Current, magnitude"
      n_ishc_bushv_C = "n:ishc:bushv:C", "p.u., Short-Circuit Current, magnitude"
      n_ishc_bushv_N = "n:ishc:bushv:N", "p.u., Short-Circuit Current, magnitude"
      n_ishc_busmv_A = "n:ishc:busmv:A", "p.u., Short-Circuit Current, magnitude"
      n_ishc_busmv_B = "n:ishc:busmv:B", "p.u., Short-Circuit Current, magnitude"
      n_ishc_busmv_C = "n:ishc:busmv:C", "p.u., Short-Circuit Current, magnitude"
      n_ishc_busmv_N = "n:ishc:busmv:N", "p.u., Short-Circuit Current, magnitude"
      n_ishc_buslv_A = "n:ishc:buslv:A", "p.u., Short-Circuit Current, magnitude"
      n_ishc_buslv_B = "n:ishc:buslv:B", "p.u., Short-Circuit Current, magnitude"
      n_ishc_buslv_C = "n:ishc:buslv:C", "p.u., Short-Circuit Current, magnitude"
      n_ishc_buslv_N = "n:ishc:buslv:N", "p.u., Short-Circuit Current, magnitude"
      n_ishc_bushvn = "n:ishc:bushvn", "p.u., Short-Circuit Current, magnitude"
      n_ishc_busmvn = "n:ishc:busmvn", "p.u., Short-Circuit Current, magnitude"
      n_ishc_buslvn = "n:ishc:buslvn", "p.u., Short-Circuit Current, magnitude"
      n_Ishc_bushv_A = "n:Ishc:bushv:A", "kA, Short-Circuit Current, magnitude"
      n_Ishc_bushv_B = "n:Ishc:bushv:B", "kA, Short-Circuit Current, magnitude"
      n_Ishc_bushv_C = "n:Ishc:bushv:C", "kA, Short-Circuit Current, magnitude"
      n_Ishc_bushv_N = "n:Ishc:bushv:N", "kA, Short-Circuit Current, magnitude"
      n_Ishc_busmv_A = "n:Ishc:busmv:A", "kA, Short-Circuit Current, magnitude"
      n_Ishc_busmv_B = "n:Ishc:busmv:B", "kA, Short-Circuit Current, magnitude"
      n_Ishc_busmv_C = "n:Ishc:busmv:C", "kA, Short-Circuit Current, magnitude"
      n_Ishc_busmv_N = "n:Ishc:busmv:N", "kA, Short-Circuit Current, magnitude"
      n_Ishc_buslv_A = "n:Ishc:buslv:A", "kA, Short-Circuit Current, magnitude"
      n_Ishc_buslv_B = "n:Ishc:buslv:B", "kA, Short-Circuit Current, magnitude"
      n_Ishc_buslv_C = "n:Ishc:buslv:C", "kA, Short-Circuit Current, magnitude"
      n_Ishc_buslv_N = "n:Ishc:buslv:N", "kA, Short-Circuit Current, magnitude"
      n_Ishc_bushvn = "n:Ishc:bushvn", "kA, Short-Circuit Current, magnitude"
      n_Ishc_busmvn = "n:Ishc:busmvn", "kA, Short-Circuit Current, magnitude"
      n_Ishc_buslvn = "n:Ishc:buslvn", "kA, Short-Circuit Current, magnitude"
      n_inshc_bushv = "n:inshc:bushv", "p.u., Neutral Short-Circuit Current, magnitude"
      n_inshc_busmv = "n:inshc:busmv", "p.u., Neutral Short-Circuit Current, magnitude"
      n_inshc_buslv = "n:inshc:buslv", "p.u., Neutral Short-Circuit Current, magnitude"
      n_inshc_bushvn = "n:inshc:bushvn", "p.u., Neutral Short-Circuit Current, magnitude"
      n_inshc_busmvn = "n:inshc:busmvn", "p.u., Neutral Short-Circuit Current, magnitude"
      n_inshc_buslvn = "n:inshc:buslvn", "p.u., Neutral Short-Circuit Current, magnitude"
      n_Inshc_bushv = "n:Inshc:bushv", "kA, Neutral Short-Circuit Current, magnitude"
      n_Inshc_busmv = "n:Inshc:busmv", "kA, Neutral Short-Circuit Current, magnitude"
      n_Inshc_buslv = "n:Inshc:buslv", "kA, Neutral Short-Circuit Current, magnitude"
      n_Inshc_bushvn = "n:Inshc:bushvn", "kA, Neutral Short-Circuit Current, magnitude"
      n_Inshc_busmvn = "n:Inshc:busmvn", "kA, Neutral Short-Circuit Current, magnitude"
      n_Inshc_buslvn = "n:Inshc:buslvn", "kA, Neutral Short-Circuit Current, magnitude"
      n_fe_bushv = "n:fe:bushv", "p.u., Electrical Frequency"
      n_fe_busmv = "n:fe:busmv", "p.u., Electrical Frequency"
      n_fe_buslv = "n:fe:buslv", "p.u., Electrical Frequency"
      n_fe_bushvn = "n:fe:bushvn", "p.u., Electrical Frequency"
      n_fe_busmvn = "n:fe:busmvn", "p.u., Electrical Frequency"
      n_fe_buslvn = "n:fe:buslvn", "p.u., Electrical Frequency"
      n_frdev_bushv = "n:frdev:bushv", "Hz, average frequency"
      n_frdev_busmv = "n:frdev:busmv", "Hz, average frequency"
      n_frdev_buslv = "n:frdev:buslv", "Hz, average frequency"
      n_frdev_bushvn = "n:frdev:bushvn", "Hz, average frequency"
      n_frdev_busmvn = "n:frdev:busmvn", "Hz, average frequency"
      n_frdev_buslvn = "n:frdev:buslvn", "Hz, average frequency"
      n_dfehz_bushv = "n:dfehz:bushv", "Hz, Deviation of the El. Frequency"
      n_dfehz_busmv = "n:dfehz:busmv", "Hz, Deviation of the El. Frequency"
      n_dfehz_buslv = "n:dfehz:buslv", "Hz, Deviation of the El. Frequency"
      n_dfehz_bushvn = "n:dfehz:bushvn", "Hz, Deviation of the El. Frequency"
      n_dfehz_busmvn = "n:dfehz:busmvn", "Hz, Deviation of the El. Frequency"
      n_dfehz_buslvn = "n:dfehz:buslvn", "Hz, Deviation of the El. Frequency"
      n_dfedt_bushv = "n:dfedt:bushv", "p.u./s, Derivative of El. Frequency"
      n_dfedt_busmv = "n:dfedt:busmv", "p.u./s, Derivative of El. Frequency"
      n_dfedt_buslv = "n:dfedt:buslv", "p.u./s, Derivative of El. Frequency"
      n_dfedt_bushvn = "n:dfedt:bushvn", "p.u./s, Derivative of El. Frequency"
      n_dfedt_busmvn = "n:dfedt:busmvn", "p.u./s, Derivative of El. Frequency"
      n_dfedt_buslvn = "n:dfedt:buslvn", "p.u./s, Derivative of El. Frequency"
      n_fehz_bushv = "n:fehz:bushv", "Hz, Electrical Frequency"
      n_fehz_busmv = "n:fehz:busmv", "Hz, Electrical Frequency"
      n_fehz_buslv = "n:fehz:buslv", "Hz, Electrical Frequency"
      n_fehz_bushvn = "n:fehz:bushvn", "Hz, Electrical Frequency"
      n_fehz_busmvn = "n:fehz:busmvn", "Hz, Electrical Frequency"
      n_fehz_buslvn = "n:fehz:buslvn", "Hz, Electrical Frequency"
      n_frnom_bushv = "n:frnom:bushv", "Hz, Nominal Frequency"
      n_frnom_busmv = "n:frnom:busmv", "Hz, Nominal Frequency"
      n_frnom_buslv = "n:frnom:buslv", "Hz, Nominal Frequency"
      n_frnom_bushvn = "n:frnom:bushvn", "Hz, Nominal Frequency"
      n_frnom_busmvn = "n:frnom:busmvn", "Hz, Nominal Frequency"
      n_frnom_buslvn = "n:frnom:buslvn", "Hz, Nominal Frequency"
      s_nntapin_h = "s:nntapin_h", "IN, Tap (HV), Controller Input, continuous, double"
      s_nntapin_m = "s:nntapin_m", "IN, Tap (MV), Controller Input, continuous, double"
      s_nntapin_l = "s:nntapin_l", "IN, Tap (LV), Controller Input, continuous, double"
      s_I0rDelta_h = "s:I0rDelta_h", "kA, OUT, Circulating current in HV-Delta-Winding, Real Part, continuous, double"
      s_I0rDelta_m = "s:I0rDelta_m", "kA, OUT, Circulating current in HV-Delta-Winding, Real Part, continuous, double"
      s_I0rDelta_l = "s:I0rDelta_l", "kA, OUT, Circulating current in LV-Delta-Winding, Real Part, continuous, double"
      s_I0iDelta_h = "s:I0iDelta_h", "kA, OUT, Circulating current in HV-Delta-Winding, Imaginary Part continuous, double"
      s_I0iDelta_m = "s:I0iDelta_m", "kA, OUT, Circulating current in MV-Delta-Winding, Imaginary Part continuous, double"
      s_I0iDelta_l = "s:I0iDelta_l", "kA, OUT, Circulating current in LV-Delta-Winding, Imaginary Part continuous, double"
      c_n3tap_h = "c:n3tap_h", "Current Tap Position (HV)"
      c_n3tap_m = "c:n3tap_m", "Current Tap Position (MV)"
      c_n3tap_l = "c:n3tap_l", "Current Tap Position (LV)"
      c_nntap_int_h = "c:nntap_int_h", "Tap (HV), relative Position"
      c_nntap_int_m = "c:nntap_int_m", "Tap (MV), relative Position"
      c_nntap_int_l = "c:nntap_int_l", "Tap (LV), relative Position"
      c_zs_h_r = "c:zs_h:r", "p.u., Short Circuit Impedance (HV), Real Part"
      c_zs_h_i = "c:zs_h:i", "p.u., Short Circuit Impedance (HV), Imaginary Part"
      c_zs_h_phi = "c:zs_h:phi", "deg, Short Circuit Impedance (HV), Angle"
      c_zs_h_mag = "c:zs_h:mag", "p.u., Short Circuit Impedance (HV), Magnitude"
      c_zs_h_phirad = "c:zs_h:phirad", "rad, Short Circuit Impedance (HV), Angle"
      c_zs_m_r = "c:zs_m:r", "p.u., Short Circuit Impedance (MV), Real Part"
      c_zs_m_i = "c:zs_m:i", "p.u., Short Circuit Impedance (MV), Imaginary Part"
      c_zs_m_phi = "c:zs_m:phi", "deg, Short Circuit Impedance (MV), Angle"
      c_zs_m_mag = "c:zs_m:mag", "p.u., Short Circuit Impedance (MV), Magnitude"
      c_zs_m_phirad = "c:zs_m:phirad", "rad, Short Circuit Impedance (MV), Angle"
      c_zs_l_r = "c:zs_l:r", "p.u., Short Circuit Impedance (LV), Real Part"
      c_zs_l_i = "c:zs_l:i", "p.u., Short Circuit Impedance (LV), Imaginary Part"
      c_zs_l_phi = "c:zs_l:phi", "deg, Short Circuit Impedance (LV), Angle"
      c_zs_l_mag = "c:zs_l:mag", "p.u., Short Circuit Impedance (LV), Magnitude"
      c_zs_l_phirad = "c:zs_l:phirad", "rad, Short Circuit Impedance (LV), Angle"
      c_ym_r = "c:ym:r", "p.u., Magnetising Admittance, Real Part"
      c_ym_i = "c:ym:i", "p.u., Magnetising Admittance, Imaginary Part"
      c_ym_phi = "c:ym:phi", "deg, Magnetising Admittance, Angle"
      c_ym_mag = "c:ym:mag", "p.u., Magnetising Admittance, Magnitude"
      c_ym_phirad = "c:ym:phirad", "rad, Magnetising Admittance, Angle"
      c_t_h_r = "c:t_h:r", "Complex Winding Ratio (HV/S), Real Part"
      c_t_h_i = "c:t_h:i", "Complex Winding Ratio (HV/S), Imaginary Part"
      c_t_h_phi = "c:t_h:phi", "Complex Winding Ratio (HV/S), Angle"
      c_t_h_mag = "c:t_h:mag", "Complex Winding Ratio (HV/S), Magnitude"
      c_t_h_phirad = "c:t_h:phirad", "Complex Winding Ratio (HV/S), Angle"
      c_t_m_r = "c:t_m:r", "Complex Winding Ratio (MV/S), Real Part"
      c_t_m_i = "c:t_m:i", "Complex Winding Ratio (MV/S), Imaginary Part"
      c_t_m_phi = "c:t_m:phi", "Complex Winding Ratio (MV/S), Angle"
      c_t_m_mag = "c:t_m:mag", "Complex Winding Ratio (MV/S), Magnitude"
      c_t_m_phirad = "c:t_m:phirad", "Complex Winding Ratio (MV/S), Angle"
      c_t_l_r = "c:t_l:r", "Complex Winding Ratio (LV/S), Real Part"
      c_t_l_i = "c:t_l:i", "Complex Winding Ratio (LV/S), Imaginary Part"
      c_t_l_phi = "c:t_l:phi", "Complex Winding Ratio (LV/S), Angle"
      c_t_l_mag = "c:t_l:mag", "Complex Winding Ratio (LV/S), Magnitude"
      c_t_l_phirad = "c:t_l:phirad", "Complex Winding Ratio (LV/S), Angle"
      c_loading_h = "c:loading_h", "%, Loading (HV)"
      c_loading_m = "c:loading_m", "%, Loading (MV)"
      c_loading_l = "c:loading_l", "%, Loading (LV)"
      c_loading = "c:loading", "%, Maximum Loading"
      c_i3loc = "c:i3loc", "Position of Magn.ind."
      c_i30lc = "c:i30lc", "Position of 0-Sequ. Magn.ind."
      c_zs0_h_r = "c:zs0_h:r", "p.u., 0-Sequ. Short-Circuit Imp (HV), Real Part"
      c_zs0_h_i = "c:zs0_h:i", "p.u., 0-Sequ. Short-Circuit Imp (HV), Imaginary Part"
      c_zs0_h_phi = "c:zs0_h:phi", "deg, 0-Sequ. Short-Circuit Imp (HV), Angle"
      c_zs0_h_mag = "c:zs0_h:mag", "p.u., 0-Sequ. Short-Circuit Imp (HV), Magnitude"
      c_zs0_h_phirad = "c:zs0_h:phirad", "rad, 0-Sequ. Short-Circuit Imp (HV), Angle"
      c_zs0_m_r = "c:zs0_m:r", "p.u., 0-Sequ. Short-Circuit Imp (MV), Real Part"
      c_zs0_m_i = "c:zs0_m:i", "p.u., 0-Sequ. Short-Circuit Imp (MV), Imaginary Part"
      c_zs0_m_phi = "c:zs0_m:phi", "deg, 0-Sequ. Short-Circuit Imp (MV), Angle"
      c_zs0_m_mag = "c:zs0_m:mag", "p.u., 0-Sequ. Short-Circuit Imp (MV), Magnitude"
      c_zs0_m_phirad = "c:zs0_m:phirad", "rad, 0-Sequ. Short-Circuit Imp (MV), Angle"
      c_zs0_l_r = "c:zs0_l:r", "p.u., 0-Sequ. Short-Circuit Imp (LV), Real Part"
      c_zs0_l_i = "c:zs0_l:i", "p.u., 0-Sequ. Short-Circuit Imp (LV), Imaginary Part"
      c_zs0_l_phi = "c:zs0_l:phi", "deg, 0-Sequ. Short-Circuit Imp (LV), Angle"
      c_zs0_l_mag = "c:zs0_l:mag", "p.u., 0-Sequ. Short-Circuit Imp (LV), Magnitude"
      c_zs0_l_phirad = "c:zs0_l:phirad", "rad, 0-Sequ. Short-Circuit Imp (LV), Angle"
      c_ze0_h_r = "c:ze0_h:r", "p.u., Earth Impedance (HV), Real Part"
      c_ze0_h_i = "c:ze0_h:i", "p.u., Earth Impedance (HV), Imaginary Part"
      c_ze0_h_phi = "c:ze0_h:phi", "deg, Earth Impedance (HV), Angle"
      c_ze0_h_mag = "c:ze0_h:mag", "p.u., Earth Impedance (HV), Magnitude"
      c_ze0_h_phirad = "c:ze0_h:phirad", "rad, Earth Impedance (HV), Angle"
      c_ze0_m_r = "c:ze0_m:r", "p.u., Earth Impedance (MV), Real Part"
      c_ze0_m_i = "c:ze0_m:i", "p.u., Earth Impedance (MV), Imaginary Part"
      c_ze0_m_phi = "c:ze0_m:phi", "deg, Earth Impedance (MV), Angle"
      c_ze0_m_mag = "c:ze0_m:mag", "p.u., Earth Impedance (MV), Magnitude"
      c_ze0_m_phirad = "c:ze0_m:phirad", "rad, Earth Impedance (MV), Angle"
      c_ze0_l_r = "c:ze0_l:r", "p.u., Earth Impedance (LV), Real Part"
      c_ze0_l_i = "c:ze0_l:i", "p.u., Earth Impedance (LV), Imaginary Part"
      c_ze0_l_phi = "c:ze0_l:phi", "deg, Earth Impedance (LV), Angle"
      c_ze0_l_mag = "c:ze0_l:mag", "p.u., Earth Impedance (LV), Magnitude"
      c_ze0_l_phirad = "c:ze0_l:phirad", "rad, Earth Impedance (LV), Angle"
      c_i_star_h = "c:i_star_h", "Flag"
      c_i_star_m = "c:i_star_m", "Flag"
      c_i_star_l = "c:i_star_l", "Flag"
      c_i_earth_h = "c:i_earth_h", "Flag"
      c_i_earth_m = "c:i_earth_m", "Flag"
      c_i_earth_l = "c:i_earth_l", "Flag"
      c_ym0_r = "c:ym0:r", "p.u., 0-Sequ. Magn. Admitt., Real Part"
      c_ym0_i = "c:ym0:i", "p.u., 0-Sequ. Magn. Admitt., Imaginary Part"
      c_ym0_phi = "c:ym0:phi", "deg, 0-Sequ. Magn. Admitt., Angle"
      c_ym0_mag = "c:ym0:mag", "p.u., 0-Sequ. Magn. Admitt., Magnitude"
      c_ym0_phirad = "c:ym0:phirad", "rad, 0-Sequ. Magn. Admitt., Angle"
      c_i_stshort = "c:i_stshort", "Flag"
      c_Ustar_h = "c:Ustar_h", "kV, Star-Point Voltage (HV)"
      c_Ustar_m = "c:Ustar_m", "kV, Star-Point Voltage (MV)"
      c_Ustar_l = "c:Ustar_l", "kV, Star-Point Voltage (LV)"
      c_Istar_h = "c:Istar_h", "kA, Star-Point Current (HV)"
      c_Istar_m = "c:Istar_m", "kA, Star-Point Current (MV)"
      c_Istar_l = "c:Istar_l", "kA, Star-Point Current (LV)"
      c_w_0 = "c:w_0", "Nominal winding ratio"
      c_I0Delta_h = "c:I0Delta_h", "kA, Circulating current in HV-D-Winding"
      c_I0Delta_m = "c:I0Delta_m", "kA, Circulating current in MV-D-Winding"
      c_I0Delta_l = "c:I0Delta_l", "kA, Circulating current in LV-D-Winding"
      c_iAutoTap = "c:iAutoTap", "Controller Mode: 0 = Manual, 1=Auto"
      c_Inom_HV = "c:Inom_HV", "kA, Nominal Current (HV-Side)"
      c_Inom_MV = "c:Inom_MV", "kA, Nominal Current (MV-Side)"
      c_Inom_LV = "c:Inom_LV", "kA, Nominal Current (LV-Side)"
      c_isat = "c:isat", "Saturated Model"
      e_du3tp_h = "e:du3tp_h", "%, Add. Voltage per Tap"
      e_ph3tr_h = "e:ph3tr_h", "deg, Phase of du"
      e_n3tap_h = "e:n3tap_h", "Act. Position"
      e_du3tp_m = "e:du3tp_m", "%, Add. Voltage per Tap"
      e_ph3tr_m = "e:ph3tr_m", "deg, Phase of du"
      e_n3tap_m = "e:n3tap_m", "Act. Position"
      e_du3tp_l = "e:du3tp_l", "%, Add. Voltage per Tap"
      e_ph3tr_l = "e:ph3tr_l", "deg, Phase of du"
      e_n3tap_l = "e:n3tap_l", "Act. Position"
      e_iTaps = "e:iTaps", "According to Measurement Report"
      e_iMeasTap = "e:iMeasTap", "for Tap at"
      e_iMeasLoc = "e:iMeasLoc", "Measured at"
      e_mTaps = "e:mTaps", "Measurement Report"
      e_iIntTapCtrl = "e:iIntTapCtrl", "Use Integrated Tap Controller"
      e_iopt_hf = "e:iopt_hf", "Consider Capacitances"
      e_Cg_h = "e:Cg_h", "uF, HV-Ground"
      e_Cg_m = "e:Cg_m", "uF, MV-Ground"
      e_Cg_l = "e:Cg_l", "uF, LV-Ground"
      e_Cc1_hm = "e:Cc1_hm", "uF, HV-MV"
      e_Cc1_ml = "e:Cc1_ml", "uF, MV-LV"
      e_Cc1_lh = "e:Cc1_lh", "uF, LV-HV"
      e_Cc0_hm = "e:Cc0_hm", "uF, HV-MV"
      e_Cc0_ml = "e:Cc0_ml", "uF, MV-LV"
      e_Cc0_lh = "e:Cc0_lh", "uF, LV-HV"
      e_iAstabint = "e:iAstabint", "A-stable integration algorithm"
      t_du3tp_h = "t:du3tp_h", "%, Add. Voltage per Tap"
      t_ph3tr_h = "t:ph3tr_h", "deg, Phase of du"
      t_n3tp0_h = "t:n3tp0_h", "Neutral Position"
      t_n3tmn_h = "t:n3tmn_h", "Min. Position"
      t_n3tmx_h = "t:n3tmx_h", "Max. Position"
      t_du3tp_m = "t:du3tp_m", "%, Add. Voltage per Tap"
      t_ph3tr_m = "t:ph3tr_m", "deg, Phase of du"
      t_n3tp0_m = "t:n3tp0_m", "Neutral Position"
      t_n3tmn_m = "t:n3tmn_m", "Min. Position"
      t_n3tmx_m = "t:n3tmx_m", "Max. Position"
      t_du3tp_l = "t:du3tp_l", "%, Add. Voltage per Tap"
      t_ph3tr_l = "t:ph3tr_l", "deg, Phase of du"
      t_n3tp0_l = "t:n3tp0_l", "Neutral Position"
      t_n3tmn_l = "t:n3tmn_l", "Min. Position"
      t_n3tmx_l = "t:n3tmx_l", "Max. Position"
      t_itapos = "t:itapos", "Tap Modelled at"
      t_i3loc = "t:i3loc", "Position"
      t_curm3 = "t:curm3", "%, No Load Current"
      t_pfe = "t:pfe", "kW, No Load Losses"
      t_i30lc = "t:i30lc", "Position"
      t_cr0m3 = "t:cr0m3", "%, No Load Current"
      t_rtox0_n = "t:rtox0_n", "Mag. R/X"
      t_iTapLoading = "t:iTapLoading", "Configuration"
      t_itrldf = "t:itrldf", "Type"
      t_satcue = "t:satcue", "%, Current (RMS)"
      t_satvol = "t:satvol", "p.u., Voltage (RMS)"
      t_iIntPola = "t:iIntPola", "Interpolation"
      t_smoothf = "t:smoothf", "%, Smoothing Factor"

    class ElmVac(Enum):
      _init_ = 'value __doc__'

      m_u0_bus1 = "m:u0:bus1", "p.u., Zero-Sequence-Voltage, Magnitude"
      m_phiu0_bus1 = "m:phiu0:bus1", "deg, Zero-Sequence-Voltage, Angle"
      m_u1r_bus1 = "m:u1r:bus1", "p.u., Positive-Sequence-Voltage, Real Part"
      m_u1i_bus1 = "m:u1i:bus1", "p.u., Positive-Sequence-Voltage, Imaginary Part"
      m_u1_bus1 = "m:u1:bus1", "p.u., Positive-Sequence-Voltage, Magnitude"
      m_U1_bus1 = "m:U1:bus1", "kV, Line-Ground Positive-Sequence-Voltage, Magnitude"
      m_U1l_bus1 = "m:U1l:bus1", "kV, Line-Line Positive-Sequence-Voltage, Magnitude"
      m_phiu1_bus1 = "m:phiu1:bus1", "deg, Positive-Sequence-Voltage, Angle"
      m_u2_bus1 = "m:u2:bus1", "p.u., Negative-Sequence-Voltage, Magnitude"
      m_phiu2_bus1 = "m:phiu2:bus1", "deg, Negative-Sequence-Voltage, Angle"
      m_inet_bus1_A = "m:inet:bus1:A", "p.u., Phase Current, Magnitude"
      m_inet_bus1_B = "m:inet:bus1:B", "p.u., Phase Current, Magnitude"
      m_inet_bus1_C = "m:inet:bus1:C", "p.u., Phase Current, Magnitude"
      m_I_bus1_A = "m:I:bus1:A", "kA, Phase Current, Magnitude"
      m_I_bus1_B = "m:I:bus1:B", "kA, Phase Current, Magnitude"
      m_I_bus1_C = "m:I:bus1:C", "kA, Phase Current, Magnitude"
      m_phii_bus1_A = "m:phii:bus1:A", "deg, Phase Current, Angle"
      m_phii_bus1_B = "m:phii:bus1:B", "deg, Phase Current, Angle"
      m_phii_bus1_C = "m:phii:bus1:C", "deg, Phase Current, Angle"
      m_i0r_bus1 = "m:i0r:bus1", "p.u., Zero-Sequence Current, Real Part"
      m_i0i_bus1 = "m:i0i:bus1", "p.u., Zero-Sequence Current, Imaginary Part"
      m_i0_bus1 = "m:i0:bus1", "p.u., Zero-Sequence Current, Magnitude"
      m_I0_bus1 = "m:I0:bus1", "kA, Zero-Sequence Current, Magnitude"
      m_I0x3_bus1 = "m:I0x3:bus1", "kA, 3*I0"
      m_phii0_bus1 = "m:phii0:bus1", "deg, Zero-Sequence Current, Angle"
      m_i1r_bus1 = "m:i1r:bus1", "p.u., Positive-Sequence Current, Real Part"
      m_i1i_bus1 = "m:i1i:bus1", "p.u., Positive-Sequence Current, Imaginary Part"
      m_i1_bus1 = "m:i1:bus1", "p.u., Positive-Sequence Current, Magnitude"
      m_I1_bus1 = "m:I1:bus1", "kA, Positive-Sequence Current, Magnitude"
      m_phii1_bus1 = "m:phii1:bus1", "deg, Positive-Sequence Current, Angle"
      m_i2r_bus1 = "m:i2r:bus1", "p.u., Negative-Sequence Current, Real Part"
      m_i2i_bus1 = "m:i2i:bus1", "p.u., Negative-Sequence Current, Imaginary Part"
      m_i2_bus1 = "m:i2:bus1", "p.u., Negative-Sequence Current, Magnitude"
      m_I2_bus1 = "m:I2:bus1", "kA, Negative-Sequence Current, Magnitude"
      m_phii2_bus1 = "m:phii2:bus1", "deg, Negative-Sequence Current, Angle"
      m_P_bus1_A = "m:P:bus1:A", "MW, Active Power"
      m_P_bus1_B = "m:P:bus1:B", "MW, Active Power"
      m_P_bus1_C = "m:P:bus1:C", "MW, Active Power"
      m_Q_bus1_A = "m:Q:bus1:A", "Mvar, Reactive Power"
      m_Q_bus1_B = "m:Q:bus1:B", "Mvar, Reactive Power"
      m_Q_bus1_C = "m:Q:bus1:C", "Mvar, Reactive Power"
      m_S_bus1_A = "m:S:bus1:A", "MVA, Apparent Power"
      m_S_bus1_B = "m:S:bus1:B", "MVA, Apparent Power"
      m_S_bus1_C = "m:S:bus1:C", "MVA, Apparent Power"
      m_cosphi_bus1_A = "m:cosphi:bus1:A", "Power Factor"
      m_cosphi_bus1_B = "m:cosphi:bus1:B", "Power Factor"
      m_cosphi_bus1_C = "m:cosphi:bus1:C", "Power Factor"
      m_tanphi_bus1_A = "m:tanphi:bus1:A", "tan(phi)"
      m_tanphi_bus1_B = "m:tanphi:bus1:B", "tan(phi)"
      m_tanphi_bus1_C = "m:tanphi:bus1:C", "tan(phi)"
      m_Psum_bus1 = "m:Psum:bus1", "MW, Total Active Power"
      m_Qsum_bus1 = "m:Qsum:bus1", "Mvar, Total Reactive Power"
      m_Ssum_bus1 = "m:Ssum:bus1", "MVA, Total Apparent Power"
      m_cosphisum_bus1 = "m:cosphisum:bus1", "Total Power Factor"
      m_tanphisum_bus1 = "m:tanphisum:bus1", "Total tan(phi)"
      m_Pln_bus1_A = "m:Pln:bus1:A", "MW, Active Power (Line-Neutral)"
      m_Pln_bus1_B = "m:Pln:bus1:B", "MW, Active Power (Line-Neutral)"
      m_Pln_bus1_C = "m:Pln:bus1:C", "MW, Active Power (Line-Neutral)"
      m_Qln_bus1_A = "m:Qln:bus1:A", "Mvar, Reactive Power (Line-Neutral)"
      m_Qln_bus1_B = "m:Qln:bus1:B", "Mvar, Reactive Power (Line-Neutral)"
      m_Qln_bus1_C = "m:Qln:bus1:C", "Mvar, Reactive Power (Line-Neutral)"
      m_P1_bus1 = "m:P1:bus1", "MW, Positive Sequence Active Power"
      m_Q1_bus1 = "m:Q1:bus1", "Mvar, Positive Sequence Reactive Power"
      m_P2_bus1 = "m:P2:bus1", "MW, Negative Sequence Active Power"
      m_Q2_bus1 = "m:Q2:bus1", "Mvar, Negative Sequence Reactive Power"
      m_P0_bus1 = "m:P0:bus1", "MW, Zero Sequence Active Power"
      m_Q0_bus1 = "m:Q0:bus1", "Mvar, Zero Sequence Reactive Power"
      m_i1P_bus1 = "m:i1P:bus1", "p.u., Positive-Sequence Active Current"
      m_i1Q_bus1 = "m:i1Q:bus1", "p.u., Positive-Sequence Reactive Current"
      m_I1P_bus1 = "m:I1P:bus1", "kA, Positive-Sequence Active Current"
      m_I1Q_bus1 = "m:I1Q:bus1", "kA, Positive-Sequence Reactive Current"
      m_i2P_bus1 = "m:i2P:bus1", "p.u., Negative-Sequence Active Current"
      m_i2Q_bus1 = "m:i2Q:bus1", "p.u., Negative-Sequence Reactive Current"
      m_I2P_bus1 = "m:I2P:bus1", "kA, Negative-Sequence Active Current"
      m_I2Q_bus1 = "m:I2Q:bus1", "kA, Negative-Sequence Reactive Current"
      m_ur_bus1_A = "m:ur:bus1:A", "p.u., Phase Voltage, Real Part"
      m_ur_bus1_B = "m:ur:bus1:B", "p.u., Phase Voltage, Real Part"
      m_ur_bus1_C = "m:ur:bus1:C", "p.u., Phase Voltage, Real Part"
      m_ui_bus1_A = "m:ui:bus1:A", "p.u., Phase Voltage, Imaginary Part"
      m_ui_bus1_B = "m:ui:bus1:B", "p.u., Phase Voltage, Imaginary Part"
      m_ui_bus1_C = "m:ui:bus1:C", "p.u., Phase Voltage, Imaginary Part"
      m_u_bus1_A = "m:u:bus1:A", "p.u., Phase Voltage, Magnitude"
      m_u_bus1_B = "m:u:bus1:B", "p.u., Phase Voltage, Magnitude"
      m_u_bus1_C = "m:u:bus1:C", "p.u., Phase Voltage, Magnitude"
      m_ir_bus1_A = "m:ir:bus1:A", "p.u., Phase Current, Real Part"
      m_ir_bus1_B = "m:ir:bus1:B", "p.u., Phase Current, Real Part"
      m_ir_bus1_C = "m:ir:bus1:C", "p.u., Phase Current, Real Part"
      m_ii_bus1_A = "m:ii:bus1:A", "p.u., Phase Current, Imaginary Part"
      m_ii_bus1_B = "m:ii:bus1:B", "p.u., Phase Current, Imaginary Part"
      m_ii_bus1_C = "m:ii:bus1:C", "p.u., Phase Current, Imaginary Part"
      m_i_bus1_A = "m:i:bus1:A", "p.u., Phase Current, Magnitude"
      m_i_bus1_B = "m:i:bus1:B", "p.u., Phase Current, Magnitude"
      m_i_bus1_C = "m:i:bus1:C", "p.u., Phase Current, Magnitude"
      m_phiui_bus1_A = "m:phiui:bus1:A", "deg, Angle between Voltage and Current"
      m_phiui_bus1_B = "m:phiui:bus1:B", "deg, Angle between Voltage and Current"
      m_phiui_bus1_C = "m:phiui:bus1:C", "deg, Angle between Voltage and Current"
      m_phiu0i0_bus1 = "m:phiu0i0:bus1", "deg, Angle between Voltage and Current in zero-sequence system"
      m_phiu1i1_bus1 = "m:phiu1i1:bus1", "deg, Angle between Voltage and Current in positive sequence system"
      m_phiu2i2_bus1 = "m:phiu2i2:bus1", "deg, Angle between Voltage and Current in negative sequence system"
      m_Tfct_bus1 = "m:Tfct:bus1", "s, Fault Clearing Time"
      m_TfctPh_bus1_A = "m:TfctPh:bus1:A", "s, Fault Clearing Time"
      m_TfctPh_bus1_B = "m:TfctPh:bus1:B", "s, Fault Clearing Time"
      m_TfctPh_bus1_C = "m:TfctPh:bus1:C", "s, Fault Clearing Time"
      m_Brkload_bus1 = "m:Brkload:bus1", "%, Max. Breaker Loading"
      m_BrkloadPh_bus1_A = "m:BrkloadPh:bus1:A", "%, Breaker Loading"
      m_BrkloadPh_bus1_B = "m:BrkloadPh:bus1:B", "%, Breaker Loading"
      m_BrkloadPh_bus1_C = "m:BrkloadPh:bus1:C", "%, Breaker Loading"
      n_ur_bus1_A = "n:ur:bus1:A", "p.u., Line-Ground Voltage, Real Part"
      n_ur_bus1_B = "n:ur:bus1:B", "p.u., Line-Ground Voltage, Real Part"
      n_ur_bus1_C = "n:ur:bus1:C", "p.u., Line-Ground Voltage, Real Part"
      n_ui_bus1_A = "n:ui:bus1:A", "p.u., Line-Ground Voltage, Imaginary Part"
      n_ui_bus1_B = "n:ui:bus1:B", "p.u., Line-Ground Voltage, Imaginary Part"
      n_ui_bus1_C = "n:ui:bus1:C", "p.u., Line-Ground Voltage, Imaginary Part"
      n_u_bus1_A = "n:u:bus1:A", "p.u., Line-Ground Voltage, Magnitude"
      n_u_bus1_B = "n:u:bus1:B", "p.u., Line-Ground Voltage, Magnitude"
      n_u_bus1_C = "n:u:bus1:C", "p.u., Line-Ground Voltage, Magnitude"
      n_upc_bus1_A = "n:upc:bus1:A", "%, Line-Ground Voltage, Magnitude"
      n_upc_bus1_B = "n:upc:bus1:B", "%, Line-Ground Voltage, Magnitude"
      n_upc_bus1_C = "n:upc:bus1:C", "%, Line-Ground Voltage, Magnitude"
      n_U_bus1_A = "n:U:bus1:A", "kV, Line-Ground Voltage, Magnitude"
      n_U_bus1_B = "n:U:bus1:B", "kV, Line-Ground Voltage, Magnitude"
      n_U_bus1_C = "n:U:bus1:C", "kV, Line-Ground Voltage, Magnitude"
      n_phiu_bus1_A = "n:phiu:bus1:A", "deg, Line-Ground Voltage, Angle"
      n_phiu_bus1_B = "n:phiu:bus1:B", "deg, Line-Ground Voltage, Angle"
      n_phiu_bus1_C = "n:phiu:bus1:C", "deg, Line-Ground Voltage, Angle"
      n_phiurel_bus1 = "n:phiurel:bus1", "deg, Voltage, Relative Angle"
      n_ul_bus1_A = "n:ul:bus1:A", "p.u., Line-Line Voltage, Magnitude"
      n_ul_bus1_B = "n:ul:bus1:B", "p.u., Line-Line Voltage, Magnitude"
      n_ul_bus1_C = "n:ul:bus1:C", "p.u., Line-Line Voltage, Magnitude"
      n_ulpc_bus1_A = "n:ulpc:bus1:A", "%, Line-Line Voltage, Magnitude"
      n_ulpc_bus1_B = "n:ulpc:bus1:B", "%, Line-Line Voltage, Magnitude"
      n_ulpc_bus1_C = "n:ulpc:bus1:C", "%, Line-Line Voltage, Magnitude"
      n_Ul_bus1_A = "n:Ul:bus1:A", "kV, Line-Line Voltage, Magnitude"
      n_Ul_bus1_B = "n:Ul:bus1:B", "kV, Line-Line Voltage, Magnitude"
      n_Ul_bus1_C = "n:Ul:bus1:C", "kV, Line-Line Voltage, Magnitude"
      n_phiul_bus1_A = "n:phiul:bus1:A", "deg, Line-Line Voltage, Angle"
      n_phiul_bus1_B = "n:phiul:bus1:B", "deg, Line-Line Voltage, Angle"
      n_phiul_bus1_C = "n:phiul:bus1:C", "deg, Line-Line Voltage, Angle"
      n_u0_bus1 = "n:u0:bus1", "p.u., Zero-Sequence Voltage, Magnitude"
      n_U0_bus1 = "n:U0:bus1", "kV, Zero-Sequence Voltage, Magnitude"
      n_U0x3_bus1 = "n:U0x3:bus1", "kV, 3*U0"
      n_phiu0_bus1 = "n:phiu0:bus1", "deg, Zero-Sequence Voltage, Angle"
      n_u1_bus1 = "n:u1:bus1", "p.u., Positive-Sequence Voltage, Magnitude"
      n_u1pc_bus1 = "n:u1pc:bus1", "%, Positive-Sequence Voltage, Magnitude"
      n_u1r_bus1 = "n:u1r:bus1", "p.u., Positive-Sequence Voltage, Real Part"
      n_u1i_bus1 = "n:u1i:bus1", "p.u., Positive-Sequence Voltage, Imaginary Part"
      n_U1_bus1 = "n:U1:bus1", "kV, Line-Ground Positive-Sequence Voltage, Magnitude"
      n_phiu1_bus1 = "n:phiu1:bus1", "deg, Positive-Sequence Voltage, Angle"
      n_u2_bus1 = "n:u2:bus1", "p.u., Negative-Sequence Voltage, Magnitude"
      n_U2_bus1 = "n:U2:bus1", "kV, Line-Ground Negative-Sequence Voltage, Magnitude"
      n_phiu2_bus1 = "n:phiu2:bus1", "deg, Negative-Sequence Voltage, Angle"
      n_U1l_bus1 = "n:U1l:bus1", "kV, Line-Line Positive-Sequence Voltage, Magnitude"
      n_U2l_bus1 = "n:U2l:bus1", "kV, Line-Line Negative-Sequence Voltage, Magnitude"
      n_uln_bus1_A = "n:uln:bus1:A", "p.u., Line-Neutral Voltage, Magnitude"
      n_uln_bus1_B = "n:uln:bus1:B", "p.u., Line-Neutral Voltage, Magnitude"
      n_uln_bus1_C = "n:uln:bus1:C", "p.u., Line-Neutral Voltage, Magnitude"
      n_Uln_bus1_A = "n:Uln:bus1:A", "kV, Line-Neutral Voltage, Magnitude"
      n_Uln_bus1_B = "n:Uln:bus1:B", "kV, Line-Neutral Voltage, Magnitude"
      n_Uln_bus1_C = "n:Uln:bus1:C", "kV, Line-Neutral Voltage, Magnitude"
      n_phiuln_bus1_A = "n:phiuln:bus1:A", "deg, Line-Neutral Voltage, Angle"
      n_phiuln_bus1_B = "n:phiuln:bus1:B", "deg, Line-Neutral Voltage, Angle"
      n_phiuln_bus1_C = "n:phiuln:bus1:C", "deg, Line-Neutral Voltage, Angle"
      n_un_bus1 = "n:un:bus1", "p.u., Neutral-Ground Voltage, Magnitude"
      n_Un_bus1 = "n:Un:bus1", "kV, Neutral-Ground Voltage, Magnitude"
      n_phiun_bus1 = "n:phiun:bus1", "deg, Neutral-Ground Voltage, Angle"
      n_upht_bus1_A = "n:upht:bus1:A", "p.u., Phase Technology dependent Voltage, Magnitude"
      n_upht_bus1_B = "n:upht:bus1:B", "p.u., Phase Technology dependent Voltage, Magnitude"
      n_upht_bus1_C = "n:upht:bus1:C", "p.u., Phase Technology dependent Voltage, Magnitude"
      n_Upht_bus1_A = "n:Upht:bus1:A", "kV, Phase Technology dependent Voltage, Magnitude"
      n_Upht_bus1_B = "n:Upht:bus1:B", "kV, Phase Technology dependent Voltage, Magnitude"
      n_Upht_bus1_C = "n:Upht:bus1:C", "kV, Phase Technology dependent Voltage, Magnitude"
      n_um_bus1 = "n:um:bus1", "p.u., Average-Voltage, Magnitude"
      n_Um_bus1 = "n:Um:bus1", "kV, Average-Voltage, Magnitude"
      n_ishcr_bus1_A = "n:ishcr:bus1:A", "p.u., Short-Circuit Current, real"
      n_ishcr_bus1_B = "n:ishcr:bus1:B", "p.u., Short-Circuit Current, real"
      n_ishcr_bus1_C = "n:ishcr:bus1:C", "p.u., Short-Circuit Current, real"
      n_ishci_bus1_A = "n:ishci:bus1:A", "p.u., Short-Circuit Current, imag"
      n_ishci_bus1_B = "n:ishci:bus1:B", "p.u., Short-Circuit Current, imag"
      n_ishci_bus1_C = "n:ishci:bus1:C", "p.u., Short-Circuit Current, imag"
      n_ishc_bus1_A = "n:ishc:bus1:A", "p.u., Short-Circuit Current, magnitude"
      n_ishc_bus1_B = "n:ishc:bus1:B", "p.u., Short-Circuit Current, magnitude"
      n_ishc_bus1_C = "n:ishc:bus1:C", "p.u., Short-Circuit Current, magnitude"
      n_Ishc_bus1_A = "n:Ishc:bus1:A", "kA, Short-Circuit Current, magnitude"
      n_Ishc_bus1_B = "n:Ishc:bus1:B", "kA, Short-Circuit Current, magnitude"
      n_Ishc_bus1_C = "n:Ishc:bus1:C", "kA, Short-Circuit Current, magnitude"
      n_inshc_bus1 = "n:inshc:bus1", "p.u., Neutral Short-Circuit Current, magnitude"
      n_Inshc_bus1 = "n:Inshc:bus1", "kA, Neutral Short-Circuit Current, magnitude"
      n_fe_bus1 = "n:fe:bus1", "p.u., Electrical Frequency"
      n_frdev_bus1 = "n:frdev:bus1", "Hz, average frequency"
      n_dfehz_bus1 = "n:dfehz:bus1", "Hz, Deviation of the El. Frequency"
      n_dfedt_bus1 = "n:dfedt:bus1", "p.u./s, Derivative of El. Frequency"
      n_fehz_bus1 = "n:fehz:bus1", "Hz, Electrical Frequency"
      n_frnom_bus1 = "n:frnom:bus1", "Hz, Nominal Frequency"
      s_u0 = "s:u0", "p.u., IN, Voltage-Input, continuous, double"
      s_Ul0 = "s:Ul0", "kV, IN, Voltage-Input (Line-Line), continuous, double"
      s_f0 = "s:f0", "p.u., IN, Frequency-Input continuous, double"
      s_F0Hz = "s:F0Hz", "Hz, IN, Frequency-Input continuous, double"
      s_U_A = "s:U_A", "kV, IN, Voltage, Magnitude, Phase a, continuous, double"
      s_U_B = "s:U_B", "kV, IN, Voltage, Magnitude, Phase b, continuous, double"
      s_U_C = "s:U_C", "kV, IN, Voltage, Magnitude, Phase c, continuous, double"
      s_phiu_B = "s:phiu_B", "deg, IN, Voltage, Angle, Phase b continuous, double"
      s_phiu_C = "s:phiu_C", "deg, IN, Voltage, Angle, Phase c continuous, double"
      s_dphiu = "s:dphiu", "rad, IN, Additional Angle, continuous, double"
      s_fref = "s:fref", "p.u., IN, Reference Frequency, continuous, double"
      s_xspeed = "s:xspeed", "p.u., OUT, Frequency, continuous, double"
      s_phiu = "s:phiu", "rad, STATE, Voltage-Angle, continuous, double"
      s_phiu_dt = "s:phiu:dt", "rad/s, d/dt, Voltage-Angle, derivative, continuous, double"
      c_hpi = "c:hpi", "rad/s, Nominal-Angular-Frequency"
      c_uini0_r = "c:uini0:r", "0-Sequ. Voltage, Real Part"
      c_uini0_i = "c:uini0:i", "0-Sequ. Voltage, Imaginary Part"
      c_uini0_phi = "c:uini0:phi", "0-Sequ. Voltage, Angle"
      c_uini0_mag = "c:uini0:mag", "0-Sequ. Voltage, Magnitude"
      c_uini0_phirad = "c:uini0:phirad", "0-Sequ. Voltage, Angle"
      c_uini2_r = "c:uini2:r", "2-Sequ. Voltage, Real Part"
      c_uini2_i = "c:uini2:i", "2-Sequ. Voltage, Imaginary Part"
      c_uini2_phi = "c:uini2:phi", "2-Sequ. Voltage, Angle"
      c_uini2_mag = "c:uini2:mag", "2-Sequ. Voltage, Magnitude"
      c_uini2_phirad = "c:uini2:phirad", "2-Sequ. Voltage, Angle"
      e_iAstabint = "e:iAstabint", "A-stable integration algorithm"
      e_R1 = "e:R1", "Ohm, Resistance, R1"
      e_X1 = "e:X1", "Ohm, Reactance, X1"
      e_R0 = "e:R0", "Ohm, Resistance, R0"
      e_X0 = "e:X0", "Ohm, Reactance, X0"
      e_R2 = "e:R2", "Ohm, Resistance, R2"
      e_X2 = "e:X2", "Ohm, Reactance, X2"
      e_leadUinput = "e:leadUinput", "Voltage input"
      e_leadFinput = "e:leadFinput", "Frequency input"
      e_cosimModel = "e:cosimModel", "Used for Co-simulation"
      e_phiSlackRef = "e:phiSlackRef", "rad, Initial rotor angle"
      e_phiiniRef = "e:phiiniRef", "rad, Initial angle of bus voltage"

    class ElmVsc(Enum):
      _init_ = 'value __doc__'

      m_u0_busac = "m:u0:busac", "p.u., Zero-Sequence-Voltage, Magnitude"
      m_u0_busdp = "m:u0:busdp", "p.u., Zero-Sequence-Voltage, Magnitude"
      m_u0_busdm = "m:u0:busdm", "p.u., Zero-Sequence-Voltage, Magnitude"
      m_phiu0_busac = "m:phiu0:busac", "deg, Zero-Sequence-Voltage, Angle"
      m_phiu0_busdp = "m:phiu0:busdp", "deg, Zero-Sequence-Voltage, Angle"
      m_phiu0_busdm = "m:phiu0:busdm", "deg, Zero-Sequence-Voltage, Angle"
      m_u1r_busac = "m:u1r:busac", "p.u., Positive-Sequence-Voltage, Real Part"
      m_u1r_busdp = "m:u1r:busdp", "p.u., Positive-Sequence-Voltage, Real Part"
      m_u1r_busdm = "m:u1r:busdm", "p.u., Positive-Sequence-Voltage, Real Part"
      m_u1i_busac = "m:u1i:busac", "p.u., Positive-Sequence-Voltage, Imaginary Part"
      m_u1i_busdp = "m:u1i:busdp", "p.u., Positive-Sequence-Voltage, Imaginary Part"
      m_u1i_busdm = "m:u1i:busdm", "p.u., Positive-Sequence-Voltage, Imaginary Part"
      m_u1_busac = "m:u1:busac", "p.u., Positive-Sequence-Voltage, Magnitude"
      m_u1_busdp = "m:u1:busdp", "p.u., Positive-Sequence-Voltage, Magnitude"
      m_u1_busdm = "m:u1:busdm", "p.u., Positive-Sequence-Voltage, Magnitude"
      m_U1_busac = "m:U1:busac", "kV, Line-Ground Positive-Sequence-Voltage, Magnitude"
      m_U1_busdp = "m:U1:busdp", "kV, Line-Ground Positive-Sequence-Voltage, Magnitude"
      m_U1_busdm = "m:U1:busdm", "kV, Line-Ground Positive-Sequence-Voltage, Magnitude"
      m_U1l_busac = "m:U1l:busac", "kV, Line-Line Positive-Sequence-Voltage, Magnitude"
      m_U1l_busdp = "m:U1l:busdp", "kV, Line-Line Positive-Sequence-Voltage, Magnitude"
      m_U1l_busdm = "m:U1l:busdm", "kV, Line-Line Positive-Sequence-Voltage, Magnitude"
      m_phiu1_busac = "m:phiu1:busac", "deg, Positive-Sequence-Voltage, Angle"
      m_phiu1_busdp = "m:phiu1:busdp", "deg, Positive-Sequence-Voltage, Angle"
      m_phiu1_busdm = "m:phiu1:busdm", "deg, Positive-Sequence-Voltage, Angle"
      m_u2_busac = "m:u2:busac", "p.u., Negative-Sequence-Voltage, Magnitude"
      m_u2_busdp = "m:u2:busdp", "p.u., Negative-Sequence-Voltage, Magnitude"
      m_u2_busdm = "m:u2:busdm", "p.u., Negative-Sequence-Voltage, Magnitude"
      m_phiu2_busac = "m:phiu2:busac", "deg, Negative-Sequence-Voltage, Angle"
      m_phiu2_busdp = "m:phiu2:busdp", "deg, Negative-Sequence-Voltage, Angle"
      m_phiu2_busdm = "m:phiu2:busdm", "deg, Negative-Sequence-Voltage, Angle"
      m_inet_busac_A = "m:inet:busac:A", "p.u., Phase Current, Magnitude"
      m_inet_busac_B = "m:inet:busac:B", "p.u., Phase Current, Magnitude"
      m_inet_busac_C = "m:inet:busac:C", "p.u., Phase Current, Magnitude"
      m_inet_busdp = "m:inet:busdp", "p.u., Phase Current, Magnitude"
      m_inet_busdm = "m:inet:busdm", "p.u., Phase Current, Magnitude"
      m_I_busac_A = "m:I:busac:A", "kA, Phase Current, Magnitude"
      m_I_busac_B = "m:I:busac:B", "kA, Phase Current, Magnitude"
      m_I_busac_C = "m:I:busac:C", "kA, Phase Current, Magnitude"
      m_I_busdp = "m:I:busdp", "kA, Phase Current, Magnitude"
      m_I_busdm = "m:I:busdm", "kA, Phase Current, Magnitude"
      m_phii_busac_A = "m:phii:busac:A", "deg, Phase Current, Angle"
      m_phii_busac_B = "m:phii:busac:B", "deg, Phase Current, Angle"
      m_phii_busac_C = "m:phii:busac:C", "deg, Phase Current, Angle"
      m_phii_busdp = "m:phii:busdp", "deg, Phase Current, Angle"
      m_phii_busdm = "m:phii:busdm", "deg, Phase Current, Angle"
      m_i0r_busac = "m:i0r:busac", "p.u., Zero-Sequence Current, Real Part"
      m_i0r_busdp = "m:i0r:busdp", "p.u., Zero-Sequence Current, Real Part"
      m_i0r_busdm = "m:i0r:busdm", "p.u., Zero-Sequence Current, Real Part"
      m_i0i_busac = "m:i0i:busac", "p.u., Zero-Sequence Current, Imaginary Part"
      m_i0i_busdp = "m:i0i:busdp", "p.u., Zero-Sequence Current, Imaginary Part"
      m_i0i_busdm = "m:i0i:busdm", "p.u., Zero-Sequence Current, Imaginary Part"
      m_i0_busac = "m:i0:busac", "p.u., Zero-Sequence Current, Magnitude"
      m_i0_busdp = "m:i0:busdp", "p.u., Zero-Sequence Current, Magnitude"
      m_i0_busdm = "m:i0:busdm", "p.u., Zero-Sequence Current, Magnitude"
      m_I0_busac = "m:I0:busac", "kA, Zero-Sequence Current, Magnitude"
      m_I0_busdp = "m:I0:busdp", "kA, Zero-Sequence Current, Magnitude"
      m_I0_busdm = "m:I0:busdm", "kA, Zero-Sequence Current, Magnitude"
      m_I0x3_busac = "m:I0x3:busac", "kA, 3*I0"
      m_I0x3_busdp = "m:I0x3:busdp", "kA, 3*I0"
      m_I0x3_busdm = "m:I0x3:busdm", "kA, 3*I0"
      m_phii0_busac = "m:phii0:busac", "deg, Zero-Sequence Current, Angle"
      m_phii0_busdp = "m:phii0:busdp", "deg, Zero-Sequence Current, Angle"
      m_phii0_busdm = "m:phii0:busdm", "deg, Zero-Sequence Current, Angle"
      m_i1r_busac = "m:i1r:busac", "p.u., Positive-Sequence Current, Real Part"
      m_i1r_busdp = "m:i1r:busdp", "p.u., Positive-Sequence Current, Real Part"
      m_i1r_busdm = "m:i1r:busdm", "p.u., Positive-Sequence Current, Real Part"
      m_i1i_busac = "m:i1i:busac", "p.u., Positive-Sequence Current, Imaginary Part"
      m_i1i_busdp = "m:i1i:busdp", "p.u., Positive-Sequence Current, Imaginary Part"
      m_i1i_busdm = "m:i1i:busdm", "p.u., Positive-Sequence Current, Imaginary Part"
      m_i1_busac = "m:i1:busac", "p.u., Positive-Sequence Current, Magnitude"
      m_i1_busdp = "m:i1:busdp", "p.u., Positive-Sequence Current, Magnitude"
      m_i1_busdm = "m:i1:busdm", "p.u., Positive-Sequence Current, Magnitude"
      m_I1_busac = "m:I1:busac", "kA, Positive-Sequence Current, Magnitude"
      m_I1_busdp = "m:I1:busdp", "kA, Positive-Sequence Current, Magnitude"
      m_I1_busdm = "m:I1:busdm", "kA, Positive-Sequence Current, Magnitude"
      m_phii1_busac = "m:phii1:busac", "deg, Positive-Sequence Current, Angle"
      m_phii1_busdp = "m:phii1:busdp", "deg, Positive-Sequence Current, Angle"
      m_phii1_busdm = "m:phii1:busdm", "deg, Positive-Sequence Current, Angle"
      m_i2r_busac = "m:i2r:busac", "p.u., Negative-Sequence Current, Real Part"
      m_i2r_busdp = "m:i2r:busdp", "p.u., Negative-Sequence Current, Real Part"
      m_i2r_busdm = "m:i2r:busdm", "p.u., Negative-Sequence Current, Real Part"
      m_i2i_busac = "m:i2i:busac", "p.u., Negative-Sequence Current, Imaginary Part"
      m_i2i_busdp = "m:i2i:busdp", "p.u., Negative-Sequence Current, Imaginary Part"
      m_i2i_busdm = "m:i2i:busdm", "p.u., Negative-Sequence Current, Imaginary Part"
      m_i2_busac = "m:i2:busac", "p.u., Negative-Sequence Current, Magnitude"
      m_i2_busdp = "m:i2:busdp", "p.u., Negative-Sequence Current, Magnitude"
      m_i2_busdm = "m:i2:busdm", "p.u., Negative-Sequence Current, Magnitude"
      m_I2_busac = "m:I2:busac", "kA, Negative-Sequence Current, Magnitude"
      m_I2_busdp = "m:I2:busdp", "kA, Negative-Sequence Current, Magnitude"
      m_I2_busdm = "m:I2:busdm", "kA, Negative-Sequence Current, Magnitude"
      m_phii2_busac = "m:phii2:busac", "deg, Negative-Sequence Current, Angle"
      m_phii2_busdp = "m:phii2:busdp", "deg, Negative-Sequence Current, Angle"
      m_phii2_busdm = "m:phii2:busdm", "deg, Negative-Sequence Current, Angle"
      m_P_busac_A = "m:P:busac:A", "MW, Active Power"
      m_P_busac_B = "m:P:busac:B", "MW, Active Power"
      m_P_busac_C = "m:P:busac:C", "MW, Active Power"
      m_P_busdp = "m:P:busdp", "MW, Active Power"
      m_P_busdm = "m:P:busdm", "MW, Active Power"
      m_Q_busac_A = "m:Q:busac:A", "Mvar, Reactive Power"
      m_Q_busac_B = "m:Q:busac:B", "Mvar, Reactive Power"
      m_Q_busac_C = "m:Q:busac:C", "Mvar, Reactive Power"
      m_Q_busdp = "m:Q:busdp", "Mvar, Reactive Power"
      m_Q_busdm = "m:Q:busdm", "Mvar, Reactive Power"
      m_S_busac_A = "m:S:busac:A", "MVA, Apparent Power"
      m_S_busac_B = "m:S:busac:B", "MVA, Apparent Power"
      m_S_busac_C = "m:S:busac:C", "MVA, Apparent Power"
      m_S_busdp = "m:S:busdp", "MVA, Apparent Power"
      m_S_busdm = "m:S:busdm", "MVA, Apparent Power"
      m_cosphi_busac_A = "m:cosphi:busac:A", "Power Factor"
      m_cosphi_busac_B = "m:cosphi:busac:B", "Power Factor"
      m_cosphi_busac_C = "m:cosphi:busac:C", "Power Factor"
      m_cosphi_busdp = "m:cosphi:busdp", "Power Factor"
      m_cosphi_busdm = "m:cosphi:busdm", "Power Factor"
      m_tanphi_busac_A = "m:tanphi:busac:A", "tan(phi)"
      m_tanphi_busac_B = "m:tanphi:busac:B", "tan(phi)"
      m_tanphi_busac_C = "m:tanphi:busac:C", "tan(phi)"
      m_tanphi_busdp = "m:tanphi:busdp", "tan(phi)"
      m_tanphi_busdm = "m:tanphi:busdm", "tan(phi)"
      m_Psum_busac = "m:Psum:busac", "MW, Total Active Power"
      m_Psum_busdp = "m:Psum:busdp", "MW, Total Active Power"
      m_Psum_busdm = "m:Psum:busdm", "MW, Total Active Power"
      m_Qsum_busac = "m:Qsum:busac", "Mvar, Total Reactive Power"
      m_Qsum_busdp = "m:Qsum:busdp", "Mvar, Total Reactive Power"
      m_Qsum_busdm = "m:Qsum:busdm", "Mvar, Total Reactive Power"
      m_Ssum_busac = "m:Ssum:busac", "MVA, Total Apparent Power"
      m_Ssum_busdp = "m:Ssum:busdp", "MVA, Total Apparent Power"
      m_Ssum_busdm = "m:Ssum:busdm", "MVA, Total Apparent Power"
      m_cosphisum_busac = "m:cosphisum:busac", "Total Power Factor"
      m_cosphisum_busdp = "m:cosphisum:busdp", "Total Power Factor"
      m_cosphisum_busdm = "m:cosphisum:busdm", "Total Power Factor"
      m_tanphisum_busac = "m:tanphisum:busac", "Total tan(phi)"
      m_tanphisum_busdp = "m:tanphisum:busdp", "Total tan(phi)"
      m_tanphisum_busdm = "m:tanphisum:busdm", "Total tan(phi)"
      m_Pln_busac_A = "m:Pln:busac:A", "MW, Active Power (Line-Neutral)"
      m_Pln_busac_B = "m:Pln:busac:B", "MW, Active Power (Line-Neutral)"
      m_Pln_busac_C = "m:Pln:busac:C", "MW, Active Power (Line-Neutral)"
      m_Pln_busdp = "m:Pln:busdp", "MW, Active Power (Line-Neutral)"
      m_Pln_busdm = "m:Pln:busdm", "MW, Active Power (Line-Neutral)"
      m_Qln_busac_A = "m:Qln:busac:A", "Mvar, Reactive Power (Line-Neutral)"
      m_Qln_busac_B = "m:Qln:busac:B", "Mvar, Reactive Power (Line-Neutral)"
      m_Qln_busac_C = "m:Qln:busac:C", "Mvar, Reactive Power (Line-Neutral)"
      m_Qln_busdp = "m:Qln:busdp", "Mvar, Reactive Power (Line-Neutral)"
      m_Qln_busdm = "m:Qln:busdm", "Mvar, Reactive Power (Line-Neutral)"
      m_P1_busac = "m:P1:busac", "MW, Positive Sequence Active Power"
      m_P1_busdp = "m:P1:busdp", "MW, Positive Sequence Active Power"
      m_P1_busdm = "m:P1:busdm", "MW, Positive Sequence Active Power"
      m_Q1_busac = "m:Q1:busac", "Mvar, Positive Sequence Reactive Power"
      m_Q1_busdp = "m:Q1:busdp", "Mvar, Positive Sequence Reactive Power"
      m_Q1_busdm = "m:Q1:busdm", "Mvar, Positive Sequence Reactive Power"
      m_P2_busac = "m:P2:busac", "MW, Negative Sequence Active Power"
      m_P2_busdp = "m:P2:busdp", "MW, Negative Sequence Active Power"
      m_P2_busdm = "m:P2:busdm", "MW, Negative Sequence Active Power"
      m_Q2_busac = "m:Q2:busac", "Mvar, Negative Sequence Reactive Power"
      m_Q2_busdp = "m:Q2:busdp", "Mvar, Negative Sequence Reactive Power"
      m_Q2_busdm = "m:Q2:busdm", "Mvar, Negative Sequence Reactive Power"
      m_P0_busac = "m:P0:busac", "MW, Zero Sequence Active Power"
      m_P0_busdp = "m:P0:busdp", "MW, Zero Sequence Active Power"
      m_P0_busdm = "m:P0:busdm", "MW, Zero Sequence Active Power"
      m_Q0_busac = "m:Q0:busac", "Mvar, Zero Sequence Reactive Power"
      m_Q0_busdp = "m:Q0:busdp", "Mvar, Zero Sequence Reactive Power"
      m_Q0_busdm = "m:Q0:busdm", "Mvar, Zero Sequence Reactive Power"
      m_i1P_busac = "m:i1P:busac", "p.u., Positive-Sequence Active Current"
      m_i1P_busdp = "m:i1P:busdp", "p.u., Positive-Sequence Active Current"
      m_i1P_busdm = "m:i1P:busdm", "p.u., Positive-Sequence Active Current"
      m_i1Q_busac = "m:i1Q:busac", "p.u., Positive-Sequence Reactive Current"
      m_i1Q_busdp = "m:i1Q:busdp", "p.u., Positive-Sequence Reactive Current"
      m_i1Q_busdm = "m:i1Q:busdm", "p.u., Positive-Sequence Reactive Current"
      m_I1P_busac = "m:I1P:busac", "kA, Positive-Sequence Active Current"
      m_I1P_busdp = "m:I1P:busdp", "kA, Positive-Sequence Active Current"
      m_I1P_busdm = "m:I1P:busdm", "kA, Positive-Sequence Active Current"
      m_I1Q_busac = "m:I1Q:busac", "kA, Positive-Sequence Reactive Current"
      m_I1Q_busdp = "m:I1Q:busdp", "kA, Positive-Sequence Reactive Current"
      m_I1Q_busdm = "m:I1Q:busdm", "kA, Positive-Sequence Reactive Current"
      m_i2P_busac = "m:i2P:busac", "p.u., Negative-Sequence Active Current"
      m_i2P_busdp = "m:i2P:busdp", "p.u., Negative-Sequence Active Current"
      m_i2P_busdm = "m:i2P:busdm", "p.u., Negative-Sequence Active Current"
      m_i2Q_busac = "m:i2Q:busac", "p.u., Negative-Sequence Reactive Current"
      m_i2Q_busdp = "m:i2Q:busdp", "p.u., Negative-Sequence Reactive Current"
      m_i2Q_busdm = "m:i2Q:busdm", "p.u., Negative-Sequence Reactive Current"
      m_I2P_busac = "m:I2P:busac", "kA, Negative-Sequence Active Current"
      m_I2P_busdp = "m:I2P:busdp", "kA, Negative-Sequence Active Current"
      m_I2P_busdm = "m:I2P:busdm", "kA, Negative-Sequence Active Current"
      m_I2Q_busac = "m:I2Q:busac", "kA, Negative-Sequence Reactive Current"
      m_I2Q_busdp = "m:I2Q:busdp", "kA, Negative-Sequence Reactive Current"
      m_I2Q_busdm = "m:I2Q:busdm", "kA, Negative-Sequence Reactive Current"
      m_ur_busac_A = "m:ur:busac:A", "p.u., Phase Voltage, Real Part"
      m_ur_busac_B = "m:ur:busac:B", "p.u., Phase Voltage, Real Part"
      m_ur_busac_C = "m:ur:busac:C", "p.u., Phase Voltage, Real Part"
      m_ur_busdp = "m:ur:busdp", "p.u., Phase Voltage, Real Part"
      m_ur_busdm = "m:ur:busdm", "p.u., Phase Voltage, Real Part"
      m_ui_busac_A = "m:ui:busac:A", "p.u., Phase Voltage, Imaginary Part"
      m_ui_busac_B = "m:ui:busac:B", "p.u., Phase Voltage, Imaginary Part"
      m_ui_busac_C = "m:ui:busac:C", "p.u., Phase Voltage, Imaginary Part"
      m_ui_busdp = "m:ui:busdp", "p.u., Phase Voltage, Imaginary Part"
      m_ui_busdm = "m:ui:busdm", "p.u., Phase Voltage, Imaginary Part"
      m_u_busac_A = "m:u:busac:A", "p.u., Phase Voltage, Magnitude"
      m_u_busac_B = "m:u:busac:B", "p.u., Phase Voltage, Magnitude"
      m_u_busac_C = "m:u:busac:C", "p.u., Phase Voltage, Magnitude"
      m_u_busdp = "m:u:busdp", "p.u., Phase Voltage, Magnitude"
      m_u_busdm = "m:u:busdm", "p.u., Phase Voltage, Magnitude"
      m_ir_busac_A = "m:ir:busac:A", "p.u., Phase Current, Real Part"
      m_ir_busac_B = "m:ir:busac:B", "p.u., Phase Current, Real Part"
      m_ir_busac_C = "m:ir:busac:C", "p.u., Phase Current, Real Part"
      m_ir_busdp = "m:ir:busdp", "p.u., Phase Current, Real Part"
      m_ir_busdm = "m:ir:busdm", "p.u., Phase Current, Real Part"
      m_ii_busac_A = "m:ii:busac:A", "p.u., Phase Current, Imaginary Part"
      m_ii_busac_B = "m:ii:busac:B", "p.u., Phase Current, Imaginary Part"
      m_ii_busac_C = "m:ii:busac:C", "p.u., Phase Current, Imaginary Part"
      m_ii_busdp = "m:ii:busdp", "p.u., Phase Current, Imaginary Part"
      m_ii_busdm = "m:ii:busdm", "p.u., Phase Current, Imaginary Part"
      m_i_busac_A = "m:i:busac:A", "p.u., Phase Current, Magnitude"
      m_i_busac_B = "m:i:busac:B", "p.u., Phase Current, Magnitude"
      m_i_busac_C = "m:i:busac:C", "p.u., Phase Current, Magnitude"
      m_i_busdp = "m:i:busdp", "p.u., Phase Current, Magnitude"
      m_i_busdm = "m:i:busdm", "p.u., Phase Current, Magnitude"
      m_phiui_busac_A = "m:phiui:busac:A", "deg, Angle between Voltage and Current"
      m_phiui_busac_B = "m:phiui:busac:B", "deg, Angle between Voltage and Current"
      m_phiui_busac_C = "m:phiui:busac:C", "deg, Angle between Voltage and Current"
      m_phiui_busdp = "m:phiui:busdp", "deg, Angle between Voltage and Current"
      m_phiui_busdm = "m:phiui:busdm", "deg, Angle between Voltage and Current"
      m_phiu0i0_busac = "m:phiu0i0:busac", "deg, Angle between Voltage and Current in zero-sequence system"
      m_phiu0i0_busdp = "m:phiu0i0:busdp", "deg, Angle between Voltage and Current in zero-sequence system"
      m_phiu0i0_busdm = "m:phiu0i0:busdm", "deg, Angle between Voltage and Current in zero-sequence system"
      m_phiu1i1_busac = "m:phiu1i1:busac", "deg, Angle between Voltage and Current in positive sequence system"
      m_phiu1i1_busdp = "m:phiu1i1:busdp", "deg, Angle between Voltage and Current in positive sequence system"
      m_phiu1i1_busdm = "m:phiu1i1:busdm", "deg, Angle between Voltage and Current in positive sequence system"
      m_phiu2i2_busac = "m:phiu2i2:busac", "deg, Angle between Voltage and Current in negative sequence system"
      m_phiu2i2_busdp = "m:phiu2i2:busdp", "deg, Angle between Voltage and Current in negative sequence system"
      m_phiu2i2_busdm = "m:phiu2i2:busdm", "deg, Angle between Voltage and Current in negative sequence system"
      m_Tfct_busac = "m:Tfct:busac", "s, Fault Clearing Time"
      m_Tfct_busdp = "m:Tfct:busdp", "s, Fault Clearing Time"
      m_Tfct_busdm = "m:Tfct:busdm", "s, Fault Clearing Time"
      m_TfctPh_busac_A = "m:TfctPh:busac:A", "s, Fault Clearing Time"
      m_TfctPh_busac_B = "m:TfctPh:busac:B", "s, Fault Clearing Time"
      m_TfctPh_busac_C = "m:TfctPh:busac:C", "s, Fault Clearing Time"
      m_TfctPh_busdp = "m:TfctPh:busdp", "s, Fault Clearing Time"
      m_TfctPh_busdm = "m:TfctPh:busdm", "s, Fault Clearing Time"
      m_Brkload_busac = "m:Brkload:busac", "%, Max. Breaker Loading"
      m_Brkload_busdp = "m:Brkload:busdp", "%, Max. Breaker Loading"
      m_Brkload_busdm = "m:Brkload:busdm", "%, Max. Breaker Loading"
      m_BrkloadPh_busac_A = "m:BrkloadPh:busac:A", "%, Breaker Loading"
      m_BrkloadPh_busac_B = "m:BrkloadPh:busac:B", "%, Breaker Loading"
      m_BrkloadPh_busac_C = "m:BrkloadPh:busac:C", "%, Breaker Loading"
      m_BrkloadPh_busdp = "m:BrkloadPh:busdp", "%, Breaker Loading"
      m_BrkloadPh_busdm = "m:BrkloadPh:busdm", "%, Breaker Loading"
      n_ur_busac_A = "n:ur:busac:A", "p.u., Line-Ground Voltage, Real Part"
      n_ur_busac_B = "n:ur:busac:B", "p.u., Line-Ground Voltage, Real Part"
      n_ur_busac_C = "n:ur:busac:C", "p.u., Line-Ground Voltage, Real Part"
      n_ur_busdp = "n:ur:busdp", "p.u., Line-Ground Voltage, Real Part"
      n_ur_busdm = "n:ur:busdm", "p.u., Line-Ground Voltage, Real Part"
      n_ui_busac_A = "n:ui:busac:A", "p.u., Line-Ground Voltage, Imaginary Part"
      n_ui_busac_B = "n:ui:busac:B", "p.u., Line-Ground Voltage, Imaginary Part"
      n_ui_busac_C = "n:ui:busac:C", "p.u., Line-Ground Voltage, Imaginary Part"
      n_ui_busdp = "n:ui:busdp", "p.u., Line-Ground Voltage, Imaginary Part"
      n_ui_busdm = "n:ui:busdm", "p.u., Line-Ground Voltage, Imaginary Part"
      n_u_busac_A = "n:u:busac:A", "p.u., Line-Ground Voltage, Magnitude"
      n_u_busac_B = "n:u:busac:B", "p.u., Line-Ground Voltage, Magnitude"
      n_u_busac_C = "n:u:busac:C", "p.u., Line-Ground Voltage, Magnitude"
      n_u_busdp = "n:u:busdp", "p.u., Line-Ground Voltage, Magnitude"
      n_u_busdm = "n:u:busdm", "p.u., Line-Ground Voltage, Magnitude"
      n_upc_busac_A = "n:upc:busac:A", "%, Line-Ground Voltage, Magnitude"
      n_upc_busac_B = "n:upc:busac:B", "%, Line-Ground Voltage, Magnitude"
      n_upc_busac_C = "n:upc:busac:C", "%, Line-Ground Voltage, Magnitude"
      n_upc_busdp = "n:upc:busdp", "%, Line-Ground Voltage, Magnitude"
      n_upc_busdm = "n:upc:busdm", "%, Line-Ground Voltage, Magnitude"
      n_U_busac_A = "n:U:busac:A", "kV, Line-Ground Voltage, Magnitude"
      n_U_busac_B = "n:U:busac:B", "kV, Line-Ground Voltage, Magnitude"
      n_U_busac_C = "n:U:busac:C", "kV, Line-Ground Voltage, Magnitude"
      n_U_busdp = "n:U:busdp", "kV, Line-Ground Voltage, Magnitude"
      n_U_busdm = "n:U:busdm", "kV, Line-Ground Voltage, Magnitude"
      n_phiu_busac_A = "n:phiu:busac:A", "deg, Line-Ground Voltage, Angle"
      n_phiu_busac_B = "n:phiu:busac:B", "deg, Line-Ground Voltage, Angle"
      n_phiu_busac_C = "n:phiu:busac:C", "deg, Line-Ground Voltage, Angle"
      n_phiu_busdp = "n:phiu:busdp", "deg, Line-Ground Voltage, Angle"
      n_phiu_busdm = "n:phiu:busdm", "deg, Line-Ground Voltage, Angle"
      n_phiurel_busac = "n:phiurel:busac", "deg, Voltage, Relative Angle"
      n_phiurel_busdp = "n:phiurel:busdp", "deg, Voltage, Relative Angle"
      n_phiurel_busdm = "n:phiurel:busdm", "deg, Voltage, Relative Angle"
      n_ul_busac_A = "n:ul:busac:A", "p.u., Line-Line Voltage, Magnitude"
      n_ul_busac_B = "n:ul:busac:B", "p.u., Line-Line Voltage, Magnitude"
      n_ul_busac_C = "n:ul:busac:C", "p.u., Line-Line Voltage, Magnitude"
      n_ul_busdp = "n:ul:busdp", "p.u., Line-Line Voltage, Magnitude"
      n_ul_busdm = "n:ul:busdm", "p.u., Line-Line Voltage, Magnitude"
      n_ulpc_busac_A = "n:ulpc:busac:A", "%, Line-Line Voltage, Magnitude"
      n_ulpc_busac_B = "n:ulpc:busac:B", "%, Line-Line Voltage, Magnitude"
      n_ulpc_busac_C = "n:ulpc:busac:C", "%, Line-Line Voltage, Magnitude"
      n_ulpc_busdp = "n:ulpc:busdp", "%, Line-Line Voltage, Magnitude"
      n_ulpc_busdm = "n:ulpc:busdm", "%, Line-Line Voltage, Magnitude"
      n_Ul_busac_A = "n:Ul:busac:A", "kV, Line-Line Voltage, Magnitude"
      n_Ul_busac_B = "n:Ul:busac:B", "kV, Line-Line Voltage, Magnitude"
      n_Ul_busac_C = "n:Ul:busac:C", "kV, Line-Line Voltage, Magnitude"
      n_Ul_busdp = "n:Ul:busdp", "kV, Line-Line Voltage, Magnitude"
      n_Ul_busdm = "n:Ul:busdm", "kV, Line-Line Voltage, Magnitude"
      n_phiul_busac_A = "n:phiul:busac:A", "deg, Line-Line Voltage, Angle"
      n_phiul_busac_B = "n:phiul:busac:B", "deg, Line-Line Voltage, Angle"
      n_phiul_busac_C = "n:phiul:busac:C", "deg, Line-Line Voltage, Angle"
      n_phiul_busdp = "n:phiul:busdp", "deg, Line-Line Voltage, Angle"
      n_phiul_busdm = "n:phiul:busdm", "deg, Line-Line Voltage, Angle"
      n_u0_busac = "n:u0:busac", "p.u., Zero-Sequence Voltage, Magnitude"
      n_u0_busdp = "n:u0:busdp", "p.u., Zero-Sequence Voltage, Magnitude"
      n_u0_busdm = "n:u0:busdm", "p.u., Zero-Sequence Voltage, Magnitude"
      n_U0_busac = "n:U0:busac", "kV, Zero-Sequence Voltage, Magnitude"
      n_U0_busdp = "n:U0:busdp", "kV, Zero-Sequence Voltage, Magnitude"
      n_U0_busdm = "n:U0:busdm", "kV, Zero-Sequence Voltage, Magnitude"
      n_U0x3_busac = "n:U0x3:busac", "kV, 3*U0"
      n_U0x3_busdp = "n:U0x3:busdp", "kV, 3*U0"
      n_U0x3_busdm = "n:U0x3:busdm", "kV, 3*U0"
      n_phiu0_busac = "n:phiu0:busac", "deg, Zero-Sequence Voltage, Angle"
      n_phiu0_busdp = "n:phiu0:busdp", "deg, Zero-Sequence Voltage, Angle"
      n_phiu0_busdm = "n:phiu0:busdm", "deg, Zero-Sequence Voltage, Angle"
      n_u1_busac = "n:u1:busac", "p.u., Positive-Sequence Voltage, Magnitude"
      n_u1_busdp = "n:u1:busdp", "p.u., Positive-Sequence Voltage, Magnitude"
      n_u1_busdm = "n:u1:busdm", "p.u., Positive-Sequence Voltage, Magnitude"
      n_u1pc_busac = "n:u1pc:busac", "%, Positive-Sequence Voltage, Magnitude"
      n_u1pc_busdp = "n:u1pc:busdp", "%, Positive-Sequence Voltage, Magnitude"
      n_u1pc_busdm = "n:u1pc:busdm", "%, Positive-Sequence Voltage, Magnitude"
      n_u1r_busac = "n:u1r:busac", "p.u., Positive-Sequence Voltage, Real Part"
      n_u1r_busdp = "n:u1r:busdp", "p.u., Positive-Sequence Voltage, Real Part"
      n_u1r_busdm = "n:u1r:busdm", "p.u., Positive-Sequence Voltage, Real Part"
      n_u1i_busac = "n:u1i:busac", "p.u., Positive-Sequence Voltage, Imaginary Part"
      n_u1i_busdp = "n:u1i:busdp", "p.u., Positive-Sequence Voltage, Imaginary Part"
      n_u1i_busdm = "n:u1i:busdm", "p.u., Positive-Sequence Voltage, Imaginary Part"
      n_U1_busac = "n:U1:busac", "kV, Line-Ground Positive-Sequence Voltage, Magnitude"
      n_U1_busdp = "n:U1:busdp", "kV, Line-Ground Positive-Sequence Voltage, Magnitude"
      n_U1_busdm = "n:U1:busdm", "kV, Line-Ground Positive-Sequence Voltage, Magnitude"
      n_phiu1_busac = "n:phiu1:busac", "deg, Positive-Sequence Voltage, Angle"
      n_phiu1_busdp = "n:phiu1:busdp", "deg, Positive-Sequence Voltage, Angle"
      n_phiu1_busdm = "n:phiu1:busdm", "deg, Positive-Sequence Voltage, Angle"
      n_u2_busac = "n:u2:busac", "p.u., Negative-Sequence Voltage, Magnitude"
      n_u2_busdp = "n:u2:busdp", "p.u., Negative-Sequence Voltage, Magnitude"
      n_u2_busdm = "n:u2:busdm", "p.u., Negative-Sequence Voltage, Magnitude"
      n_U2_busac = "n:U2:busac", "kV, Line-Ground Negative-Sequence Voltage, Magnitude"
      n_U2_busdp = "n:U2:busdp", "kV, Line-Ground Negative-Sequence Voltage, Magnitude"
      n_U2_busdm = "n:U2:busdm", "kV, Line-Ground Negative-Sequence Voltage, Magnitude"
      n_phiu2_busac = "n:phiu2:busac", "deg, Negative-Sequence Voltage, Angle"
      n_phiu2_busdp = "n:phiu2:busdp", "deg, Negative-Sequence Voltage, Angle"
      n_phiu2_busdm = "n:phiu2:busdm", "deg, Negative-Sequence Voltage, Angle"
      n_U1l_busac = "n:U1l:busac", "kV, Line-Line Positive-Sequence Voltage, Magnitude"
      n_U1l_busdp = "n:U1l:busdp", "kV, Line-Line Positive-Sequence Voltage, Magnitude"
      n_U1l_busdm = "n:U1l:busdm", "kV, Line-Line Positive-Sequence Voltage, Magnitude"
      n_U2l_busac = "n:U2l:busac", "kV, Line-Line Negative-Sequence Voltage, Magnitude"
      n_U2l_busdp = "n:U2l:busdp", "kV, Line-Line Negative-Sequence Voltage, Magnitude"
      n_U2l_busdm = "n:U2l:busdm", "kV, Line-Line Negative-Sequence Voltage, Magnitude"
      n_uln_busac_A = "n:uln:busac:A", "p.u., Line-Neutral Voltage, Magnitude"
      n_uln_busac_B = "n:uln:busac:B", "p.u., Line-Neutral Voltage, Magnitude"
      n_uln_busac_C = "n:uln:busac:C", "p.u., Line-Neutral Voltage, Magnitude"
      n_uln_busdp = "n:uln:busdp", "p.u., Line-Neutral Voltage, Magnitude"
      n_uln_busdm = "n:uln:busdm", "p.u., Line-Neutral Voltage, Magnitude"
      n_Uln_busac_A = "n:Uln:busac:A", "kV, Line-Neutral Voltage, Magnitude"
      n_Uln_busac_B = "n:Uln:busac:B", "kV, Line-Neutral Voltage, Magnitude"
      n_Uln_busac_C = "n:Uln:busac:C", "kV, Line-Neutral Voltage, Magnitude"
      n_Uln_busdp = "n:Uln:busdp", "kV, Line-Neutral Voltage, Magnitude"
      n_Uln_busdm = "n:Uln:busdm", "kV, Line-Neutral Voltage, Magnitude"
      n_phiuln_busac_A = "n:phiuln:busac:A", "deg, Line-Neutral Voltage, Angle"
      n_phiuln_busac_B = "n:phiuln:busac:B", "deg, Line-Neutral Voltage, Angle"
      n_phiuln_busac_C = "n:phiuln:busac:C", "deg, Line-Neutral Voltage, Angle"
      n_phiuln_busdp = "n:phiuln:busdp", "deg, Line-Neutral Voltage, Angle"
      n_phiuln_busdm = "n:phiuln:busdm", "deg, Line-Neutral Voltage, Angle"
      n_un_busac = "n:un:busac", "p.u., Neutral-Ground Voltage, Magnitude"
      n_un_busdp = "n:un:busdp", "p.u., Neutral-Ground Voltage, Magnitude"
      n_un_busdm = "n:un:busdm", "p.u., Neutral-Ground Voltage, Magnitude"
      n_Un_busac = "n:Un:busac", "kV, Neutral-Ground Voltage, Magnitude"
      n_Un_busdp = "n:Un:busdp", "kV, Neutral-Ground Voltage, Magnitude"
      n_Un_busdm = "n:Un:busdm", "kV, Neutral-Ground Voltage, Magnitude"
      n_phiun_busac = "n:phiun:busac", "deg, Neutral-Ground Voltage, Angle"
      n_phiun_busdp = "n:phiun:busdp", "deg, Neutral-Ground Voltage, Angle"
      n_phiun_busdm = "n:phiun:busdm", "deg, Neutral-Ground Voltage, Angle"
      n_upht_busac_A = "n:upht:busac:A", "p.u., Phase Technology dependent Voltage, Magnitude"
      n_upht_busac_B = "n:upht:busac:B", "p.u., Phase Technology dependent Voltage, Magnitude"
      n_upht_busac_C = "n:upht:busac:C", "p.u., Phase Technology dependent Voltage, Magnitude"
      n_upht_busdp = "n:upht:busdp", "p.u., Phase Technology dependent Voltage, Magnitude"
      n_upht_busdm = "n:upht:busdm", "p.u., Phase Technology dependent Voltage, Magnitude"
      n_Upht_busac_A = "n:Upht:busac:A", "kV, Phase Technology dependent Voltage, Magnitude"
      n_Upht_busac_B = "n:Upht:busac:B", "kV, Phase Technology dependent Voltage, Magnitude"
      n_Upht_busac_C = "n:Upht:busac:C", "kV, Phase Technology dependent Voltage, Magnitude"
      n_Upht_busdp = "n:Upht:busdp", "kV, Phase Technology dependent Voltage, Magnitude"
      n_Upht_busdm = "n:Upht:busdm", "kV, Phase Technology dependent Voltage, Magnitude"
      n_um_busac = "n:um:busac", "p.u., Average-Voltage, Magnitude"
      n_um_busdp = "n:um:busdp", "p.u., Average-Voltage, Magnitude"
      n_um_busdm = "n:um:busdm", "p.u., Average-Voltage, Magnitude"
      n_Um_busac = "n:Um:busac", "kV, Average-Voltage, Magnitude"
      n_Um_busdp = "n:Um:busdp", "kV, Average-Voltage, Magnitude"
      n_Um_busdm = "n:Um:busdm", "kV, Average-Voltage, Magnitude"
      n_ishcr_busac_A = "n:ishcr:busac:A", "p.u., Short-Circuit Current, real"
      n_ishcr_busac_B = "n:ishcr:busac:B", "p.u., Short-Circuit Current, real"
      n_ishcr_busac_C = "n:ishcr:busac:C", "p.u., Short-Circuit Current, real"
      n_ishcr_busdp = "n:ishcr:busdp", "p.u., Short-Circuit Current, real"
      n_ishcr_busdm = "n:ishcr:busdm", "p.u., Short-Circuit Current, real"
      n_ishci_busac_A = "n:ishci:busac:A", "p.u., Short-Circuit Current, imag"
      n_ishci_busac_B = "n:ishci:busac:B", "p.u., Short-Circuit Current, imag"
      n_ishci_busac_C = "n:ishci:busac:C", "p.u., Short-Circuit Current, imag"
      n_ishci_busdp = "n:ishci:busdp", "p.u., Short-Circuit Current, imag"
      n_ishci_busdm = "n:ishci:busdm", "p.u., Short-Circuit Current, imag"
      n_ishc_busac_A = "n:ishc:busac:A", "p.u., Short-Circuit Current, magnitude"
      n_ishc_busac_B = "n:ishc:busac:B", "p.u., Short-Circuit Current, magnitude"
      n_ishc_busac_C = "n:ishc:busac:C", "p.u., Short-Circuit Current, magnitude"
      n_ishc_busdp = "n:ishc:busdp", "p.u., Short-Circuit Current, magnitude"
      n_ishc_busdm = "n:ishc:busdm", "p.u., Short-Circuit Current, magnitude"
      n_Ishc_busac_A = "n:Ishc:busac:A", "kA, Short-Circuit Current, magnitude"
      n_Ishc_busac_B = "n:Ishc:busac:B", "kA, Short-Circuit Current, magnitude"
      n_Ishc_busac_C = "n:Ishc:busac:C", "kA, Short-Circuit Current, magnitude"
      n_Ishc_busdp = "n:Ishc:busdp", "kA, Short-Circuit Current, magnitude"
      n_Ishc_busdm = "n:Ishc:busdm", "kA, Short-Circuit Current, magnitude"
      n_inshc_busac = "n:inshc:busac", "p.u., Neutral Short-Circuit Current, magnitude"
      n_inshc_busdp = "n:inshc:busdp", "p.u., Neutral Short-Circuit Current, magnitude"
      n_inshc_busdm = "n:inshc:busdm", "p.u., Neutral Short-Circuit Current, magnitude"
      n_Inshc_busac = "n:Inshc:busac", "kA, Neutral Short-Circuit Current, magnitude"
      n_Inshc_busdp = "n:Inshc:busdp", "kA, Neutral Short-Circuit Current, magnitude"
      n_Inshc_busdm = "n:Inshc:busdm", "kA, Neutral Short-Circuit Current, magnitude"
      n_fe_busac = "n:fe:busac", "p.u., Electrical Frequency"
      n_fe_busdp = "n:fe:busdp", "p.u., Electrical Frequency"
      n_fe_busdm = "n:fe:busdm", "p.u., Electrical Frequency"
      n_frdev_busac = "n:frdev:busac", "Hz, average frequency"
      n_frdev_busdp = "n:frdev:busdp", "Hz, average frequency"
      n_frdev_busdm = "n:frdev:busdm", "Hz, average frequency"
      n_dfehz_busac = "n:dfehz:busac", "Hz, Deviation of the El. Frequency"
      n_dfehz_busdp = "n:dfehz:busdp", "Hz, Deviation of the El. Frequency"
      n_dfehz_busdm = "n:dfehz:busdm", "Hz, Deviation of the El. Frequency"
      n_dfedt_busac = "n:dfedt:busac", "p.u./s, Derivative of El. Frequency"
      n_dfedt_busdp = "n:dfedt:busdp", "p.u./s, Derivative of El. Frequency"
      n_dfedt_busdm = "n:dfedt:busdm", "p.u./s, Derivative of El. Frequency"
      n_fehz_busac = "n:fehz:busac", "Hz, Electrical Frequency"
      n_fehz_busdp = "n:fehz:busdp", "Hz, Electrical Frequency"
      n_fehz_busdm = "n:fehz:busdm", "Hz, Electrical Frequency"
      n_frnom_busac = "n:frnom:busac", "Hz, Nominal Frequency"
      n_frnom_busdp = "n:frnom:busdp", "Hz, Nominal Frequency"
      n_frnom_busdm = "n:frnom:busdm", "Hz, Nominal Frequency"
      s_Pm_in = "s:Pm_in", "p.u., IN, Pulse-modulation factor (input) continuous, double"
      s_f0 = "s:f0", "p.u., IN, Frequency-input continuous, double"
      s_F0Hz = "s:F0Hz", "Hz, IN, Frequency-input continuous, double"
      s_dphiu = "s:dphiu", "rad, IN, Voltage-angle (input), continuous, double"
      s_fref = "s:fref", "p.u., IN, Reference frequency, continuous, double"
      s_Pmr = "s:Pmr", "IN, Pulse-modulation factor (real part), continuous, double"
      s_Pmi = "s:Pmi", "IN, Pulse-modulation factor (imaginary part), continuous, double"
      s_cosref = "s:cosref", "IN, Cos(dq-reference-angle) continuous, double"
      s_sinref = "s:sinref", "IN, Sin(dq-reference-angle) continuous, double"
      s_Pmd = "s:Pmd", "IN, Pulse-modulation factor (d-axis), continuous, double"
      s_Pmq = "s:Pmq", "IN, Pulse-modulation factor (q-axis), continuous, double"
      s_id_ref = "s:id_ref", "p.u., IN, d-axis current reference, continuous, double"
      s_iq_ref = "s:iq_ref", "p.u., IN, q-axis current reference, continuous, double"
      s_i2d_ref = "s:i2d_ref", "p.u., IN, Negative sequence current, d-axis reference, continuous, double"
      s_i2q_ref = "s:i2q_ref", "p.u., IN, Negative sequence current, q-axis reference, continuous, double"
      s_u2r_in = "s:u2r_in", "p.u., IN, Negative Sequence Voltage Input, Real Part, continuous, double"
      s_u2i_in = "s:u2i_in", "p.u., IN, Negative Sequence Voltage Input, Imaginary Part continuous, double"
      s_block = "s:block", "p.u., IN, Blocking, continuous, double"
      s_mdc = "s:mdc", "p.u., IN, Insertion index, DC-bias, continuous, double"
      s_u0r_in = "s:u0r_in", "p.u., IN, Zero Sequence Voltage Input, Real Part, continuous, double"
      s_u0i_in = "s:u0i_in", "p.u., IN, Zero Sequence Voltage Input, Imaginary Part, continuous, double"
      s_xspeed = "s:xspeed", "p.u., OUT, Frequency, continuous, double"
      s_iDC = "s:iDC", "p.u., OUT, DC-current, continuous, double"
      s_uDC = "s:uDC", "p.u., OUT, DC-voltage, continuous, double"
      s_iAC_r = "s:iAC_r", "p.u., OUT, AC-current, real part, continuous, double"
      s_iAC_i = "s:iAC_i", "p.u., OUT, AC-current, imaginary part, continuous, double"
      s_uAC_r = "s:uAC_r", "p.u., OUT, AC-voltage, real part, continuous, double"
      s_uAC_i = "s:uAC_i", "p.u., OUT, AC-voltage, imaginary part, continuous, double"
      s_id = "s:id", "p.u., OUT, Current, d-axis continuous, double"
      s_iq = "s:iq", "p.u., OUT, Current, q-axis continuous, double"
      s_yUcell = "s:yUcell", "OUT, Voltage across cell capacitor, output, continuous, double"
      s_phiu = "s:phiu", "rad, STATE, Voltage-angle, continuous, double"
      s_xd = "s:xd", "p.u., STATE, State variable, current controller d-axis, continuous, double"
      s_xq = "s:xq", "p.u., STATE, State variable, current controller q-axis, continuous, double"
      s_Ucell = "s:Ucell", "kV, STATE, Voltage across cell capacitor, continuous, double"
      s_phiu_dt = "s:phiu:dt", "rad/s, d/dt, Voltage-angle, derivative, continuous, double"
      s_xd_dt = "s:xd:dt", "p.u./s, d/dt, State variable, current controller d-axis, derivative, continuous, double"
      s_xq_dt = "s:xq:dt", "p.u./s, d/dt, State variable, current controller q-axis, derivative, continuous, double"
      s_Ucell_dt = "s:Ucell:dt", "kV/s, d/dt, Voltage across cell capacitor, derivative, continuous, double"
      c_hpi = "c:hpi", "rad/s, Nominal-angular-frequency"
      c_snomn = "c:snomn", "MVA, Total rated apparent power (nparnum*Snom)"
      c_Pm = "c:Pm", "Pulse-modulation factor"
      c_i_mod = "c:i_mod", "Modulation"
      c_K0 = "c:K0", "Vac/Vdc at PWM=1"
      c_DCPoleLosses = "c:DCPoleLosses", "MW, DC pole losses"
      c_Pmd_ctrl = "c:Pmd_ctrl", "Pulse-modulation factor (d-axis)"
      c_Pmq_ctrl = "c:Pmq_ctrl", "Pulse-modulation factor (q-axis)"
      c_Ccell = "c:Ccell", "F, Cell capacitor"
      c_i_acdc = "c:i_acdc", "Control mode"
      e_iAstabint = "e:iAstabint", "A-stable integration algorithm"
      e_iVIsource = "e:iVIsource", "Independent AC and DC voltage/current source model"
      e_Rac = "e:Rac", "Ohm, AC resistance"
      e_Lac = "e:Lac", "mH, AC inductance"
      e_R0toR1ac = "e:R0toR1ac", "R0/R1 ratio"
      e_X0toX1ac = "e:X0toX1ac", "X0/X1 ratio"
      e_Rdc = "e:Rdc", "Ohm, DC resistance"
      e_Ldc = "e:Ldc", "mH, DC inductance"
      e_Pnold = "e:Pnold", "kW, No-load losses"
      e_swtLossFactor = "e:swtLossFactor", "kW/A, Switching loss factor"
      e_resLossFactor = "e:resLossFactor", "Ohm, Resistive loss factor"
      e_Xd = "e:Xd", "Ohm, Commutation reactance"
      e_Cdc = "e:Cdc", "uF, Cell capacitor"
      e_mmcCmod = "e:mmcCmod", "uF, Submodule capacitance"
      e_NmmcSM = "e:NmmcSM", "Number of submodules per arm"
      e_r2 = "e:r2", "p.u., Resistance, r2"
      e_x2 = "e:x2", "p.u., Reactance, x2"
      e_i_ctrl = "e:i_ctrl", "Use integrated current controller"
      e_Kd = "e:Kd", "Kd: d-axis, proportional gain"
      e_Td = "e:Td", "s, Td: d-axis, integration time constant"
      e_Kq = "e:Kq", "Kq: q-axis, proportional gain"
      e_Tq = "e:Tq", "s, Tq: q-axis, integration time constant"

    class ElmVscmono(Enum):
      _init_ = 'value __doc__'

      m_u0_busac = "m:u0:busac", "p.u., Zero-Sequence-Voltage, Magnitude"
      m_u0_busdc = "m:u0:busdc", "p.u., Zero-Sequence-Voltage, Magnitude"
      m_phiu0_busac = "m:phiu0:busac", "deg, Zero-Sequence-Voltage, Angle"
      m_phiu0_busdc = "m:phiu0:busdc", "deg, Zero-Sequence-Voltage, Angle"
      m_u1r_busac = "m:u1r:busac", "p.u., Positive-Sequence-Voltage, Real Part"
      m_u1r_busdc = "m:u1r:busdc", "p.u., Positive-Sequence-Voltage, Real Part"
      m_u1i_busac = "m:u1i:busac", "p.u., Positive-Sequence-Voltage, Imaginary Part"
      m_u1i_busdc = "m:u1i:busdc", "p.u., Positive-Sequence-Voltage, Imaginary Part"
      m_u1_busac = "m:u1:busac", "p.u., Positive-Sequence-Voltage, Magnitude"
      m_u1_busdc = "m:u1:busdc", "p.u., Positive-Sequence-Voltage, Magnitude"
      m_U1_busac = "m:U1:busac", "kV, Line-Ground Positive-Sequence-Voltage, Magnitude"
      m_U1_busdc = "m:U1:busdc", "kV, Line-Ground Positive-Sequence-Voltage, Magnitude"
      m_U1l_busac = "m:U1l:busac", "kV, Line-Line Positive-Sequence-Voltage, Magnitude"
      m_U1l_busdc = "m:U1l:busdc", "kV, Line-Line Positive-Sequence-Voltage, Magnitude"
      m_phiu1_busac = "m:phiu1:busac", "deg, Positive-Sequence-Voltage, Angle"
      m_phiu1_busdc = "m:phiu1:busdc", "deg, Positive-Sequence-Voltage, Angle"
      m_u2_busac = "m:u2:busac", "p.u., Negative-Sequence-Voltage, Magnitude"
      m_u2_busdc = "m:u2:busdc", "p.u., Negative-Sequence-Voltage, Magnitude"
      m_phiu2_busac = "m:phiu2:busac", "deg, Negative-Sequence-Voltage, Angle"
      m_phiu2_busdc = "m:phiu2:busdc", "deg, Negative-Sequence-Voltage, Angle"
      m_inet_busac_A = "m:inet:busac:A", "p.u., Phase Current, Magnitude"
      m_inet_busac_B = "m:inet:busac:B", "p.u., Phase Current, Magnitude"
      m_inet_busac_C = "m:inet:busac:C", "p.u., Phase Current, Magnitude"
      m_inet_busdc = "m:inet:busdc", "p.u., Phase Current, Magnitude"
      m_I_busac_A = "m:I:busac:A", "kA, Phase Current, Magnitude"
      m_I_busac_B = "m:I:busac:B", "kA, Phase Current, Magnitude"
      m_I_busac_C = "m:I:busac:C", "kA, Phase Current, Magnitude"
      m_I_busdc = "m:I:busdc", "kA, Phase Current, Magnitude"
      m_phii_busac_A = "m:phii:busac:A", "deg, Phase Current, Angle"
      m_phii_busac_B = "m:phii:busac:B", "deg, Phase Current, Angle"
      m_phii_busac_C = "m:phii:busac:C", "deg, Phase Current, Angle"
      m_phii_busdc = "m:phii:busdc", "deg, Phase Current, Angle"
      m_i0r_busac = "m:i0r:busac", "p.u., Zero-Sequence Current, Real Part"
      m_i0r_busdc = "m:i0r:busdc", "p.u., Zero-Sequence Current, Real Part"
      m_i0i_busac = "m:i0i:busac", "p.u., Zero-Sequence Current, Imaginary Part"
      m_i0i_busdc = "m:i0i:busdc", "p.u., Zero-Sequence Current, Imaginary Part"
      m_i0_busac = "m:i0:busac", "p.u., Zero-Sequence Current, Magnitude"
      m_i0_busdc = "m:i0:busdc", "p.u., Zero-Sequence Current, Magnitude"
      m_I0_busac = "m:I0:busac", "kA, Zero-Sequence Current, Magnitude"
      m_I0_busdc = "m:I0:busdc", "kA, Zero-Sequence Current, Magnitude"
      m_I0x3_busac = "m:I0x3:busac", "kA, 3*I0"
      m_I0x3_busdc = "m:I0x3:busdc", "kA, 3*I0"
      m_phii0_busac = "m:phii0:busac", "deg, Zero-Sequence Current, Angle"
      m_phii0_busdc = "m:phii0:busdc", "deg, Zero-Sequence Current, Angle"
      m_i1r_busac = "m:i1r:busac", "p.u., Positive-Sequence Current, Real Part"
      m_i1r_busdc = "m:i1r:busdc", "p.u., Positive-Sequence Current, Real Part"
      m_i1i_busac = "m:i1i:busac", "p.u., Positive-Sequence Current, Imaginary Part"
      m_i1i_busdc = "m:i1i:busdc", "p.u., Positive-Sequence Current, Imaginary Part"
      m_i1_busac = "m:i1:busac", "p.u., Positive-Sequence Current, Magnitude"
      m_i1_busdc = "m:i1:busdc", "p.u., Positive-Sequence Current, Magnitude"
      m_I1_busac = "m:I1:busac", "kA, Positive-Sequence Current, Magnitude"
      m_I1_busdc = "m:I1:busdc", "kA, Positive-Sequence Current, Magnitude"
      m_phii1_busac = "m:phii1:busac", "deg, Positive-Sequence Current, Angle"
      m_phii1_busdc = "m:phii1:busdc", "deg, Positive-Sequence Current, Angle"
      m_i2r_busac = "m:i2r:busac", "p.u., Negative-Sequence Current, Real Part"
      m_i2r_busdc = "m:i2r:busdc", "p.u., Negative-Sequence Current, Real Part"
      m_i2i_busac = "m:i2i:busac", "p.u., Negative-Sequence Current, Imaginary Part"
      m_i2i_busdc = "m:i2i:busdc", "p.u., Negative-Sequence Current, Imaginary Part"
      m_i2_busac = "m:i2:busac", "p.u., Negative-Sequence Current, Magnitude"
      m_i2_busdc = "m:i2:busdc", "p.u., Negative-Sequence Current, Magnitude"
      m_I2_busac = "m:I2:busac", "kA, Negative-Sequence Current, Magnitude"
      m_I2_busdc = "m:I2:busdc", "kA, Negative-Sequence Current, Magnitude"
      m_phii2_busac = "m:phii2:busac", "deg, Negative-Sequence Current, Angle"
      m_phii2_busdc = "m:phii2:busdc", "deg, Negative-Sequence Current, Angle"
      m_P_busac_A = "m:P:busac:A", "MW, Active Power"
      m_P_busac_B = "m:P:busac:B", "MW, Active Power"
      m_P_busac_C = "m:P:busac:C", "MW, Active Power"
      m_P_busdc = "m:P:busdc", "MW, Active Power"
      m_Q_busac_A = "m:Q:busac:A", "Mvar, Reactive Power"
      m_Q_busac_B = "m:Q:busac:B", "Mvar, Reactive Power"
      m_Q_busac_C = "m:Q:busac:C", "Mvar, Reactive Power"
      m_Q_busdc = "m:Q:busdc", "Mvar, Reactive Power"
      m_S_busac_A = "m:S:busac:A", "MVA, Apparent Power"
      m_S_busac_B = "m:S:busac:B", "MVA, Apparent Power"
      m_S_busac_C = "m:S:busac:C", "MVA, Apparent Power"
      m_S_busdc = "m:S:busdc", "MVA, Apparent Power"
      m_cosphi_busac_A = "m:cosphi:busac:A", "Power Factor"
      m_cosphi_busac_B = "m:cosphi:busac:B", "Power Factor"
      m_cosphi_busac_C = "m:cosphi:busac:C", "Power Factor"
      m_cosphi_busdc = "m:cosphi:busdc", "Power Factor"
      m_tanphi_busac_A = "m:tanphi:busac:A", "tan(phi)"
      m_tanphi_busac_B = "m:tanphi:busac:B", "tan(phi)"
      m_tanphi_busac_C = "m:tanphi:busac:C", "tan(phi)"
      m_tanphi_busdc = "m:tanphi:busdc", "tan(phi)"
      m_Psum_busac = "m:Psum:busac", "MW, Total Active Power"
      m_Psum_busdc = "m:Psum:busdc", "MW, Total Active Power"
      m_Qsum_busac = "m:Qsum:busac", "Mvar, Total Reactive Power"
      m_Qsum_busdc = "m:Qsum:busdc", "Mvar, Total Reactive Power"
      m_Ssum_busac = "m:Ssum:busac", "MVA, Total Apparent Power"
      m_Ssum_busdc = "m:Ssum:busdc", "MVA, Total Apparent Power"
      m_cosphisum_busac = "m:cosphisum:busac", "Total Power Factor"
      m_cosphisum_busdc = "m:cosphisum:busdc", "Total Power Factor"
      m_tanphisum_busac = "m:tanphisum:busac", "Total tan(phi)"
      m_tanphisum_busdc = "m:tanphisum:busdc", "Total tan(phi)"
      m_Pln_busac_A = "m:Pln:busac:A", "MW, Active Power (Line-Neutral)"
      m_Pln_busac_B = "m:Pln:busac:B", "MW, Active Power (Line-Neutral)"
      m_Pln_busac_C = "m:Pln:busac:C", "MW, Active Power (Line-Neutral)"
      m_Pln_busdc = "m:Pln:busdc", "MW, Active Power (Line-Neutral)"
      m_Qln_busac_A = "m:Qln:busac:A", "Mvar, Reactive Power (Line-Neutral)"
      m_Qln_busac_B = "m:Qln:busac:B", "Mvar, Reactive Power (Line-Neutral)"
      m_Qln_busac_C = "m:Qln:busac:C", "Mvar, Reactive Power (Line-Neutral)"
      m_Qln_busdc = "m:Qln:busdc", "Mvar, Reactive Power (Line-Neutral)"
      m_P1_busac = "m:P1:busac", "MW, Positive Sequence Active Power"
      m_P1_busdc = "m:P1:busdc", "MW, Positive Sequence Active Power"
      m_Q1_busac = "m:Q1:busac", "Mvar, Positive Sequence Reactive Power"
      m_Q1_busdc = "m:Q1:busdc", "Mvar, Positive Sequence Reactive Power"
      m_P2_busac = "m:P2:busac", "MW, Negative Sequence Active Power"
      m_P2_busdc = "m:P2:busdc", "MW, Negative Sequence Active Power"
      m_Q2_busac = "m:Q2:busac", "Mvar, Negative Sequence Reactive Power"
      m_Q2_busdc = "m:Q2:busdc", "Mvar, Negative Sequence Reactive Power"
      m_P0_busac = "m:P0:busac", "MW, Zero Sequence Active Power"
      m_P0_busdc = "m:P0:busdc", "MW, Zero Sequence Active Power"
      m_Q0_busac = "m:Q0:busac", "Mvar, Zero Sequence Reactive Power"
      m_Q0_busdc = "m:Q0:busdc", "Mvar, Zero Sequence Reactive Power"
      m_i1P_busac = "m:i1P:busac", "p.u., Positive-Sequence Active Current"
      m_i1P_busdc = "m:i1P:busdc", "p.u., Positive-Sequence Active Current"
      m_i1Q_busac = "m:i1Q:busac", "p.u., Positive-Sequence Reactive Current"
      m_i1Q_busdc = "m:i1Q:busdc", "p.u., Positive-Sequence Reactive Current"
      m_I1P_busac = "m:I1P:busac", "kA, Positive-Sequence Active Current"
      m_I1P_busdc = "m:I1P:busdc", "kA, Positive-Sequence Active Current"
      m_I1Q_busac = "m:I1Q:busac", "kA, Positive-Sequence Reactive Current"
      m_I1Q_busdc = "m:I1Q:busdc", "kA, Positive-Sequence Reactive Current"
      m_i2P_busac = "m:i2P:busac", "p.u., Negative-Sequence Active Current"
      m_i2P_busdc = "m:i2P:busdc", "p.u., Negative-Sequence Active Current"
      m_i2Q_busac = "m:i2Q:busac", "p.u., Negative-Sequence Reactive Current"
      m_i2Q_busdc = "m:i2Q:busdc", "p.u., Negative-Sequence Reactive Current"
      m_I2P_busac = "m:I2P:busac", "kA, Negative-Sequence Active Current"
      m_I2P_busdc = "m:I2P:busdc", "kA, Negative-Sequence Active Current"
      m_I2Q_busac = "m:I2Q:busac", "kA, Negative-Sequence Reactive Current"
      m_I2Q_busdc = "m:I2Q:busdc", "kA, Negative-Sequence Reactive Current"
      m_ur_busac_A = "m:ur:busac:A", "p.u., Phase Voltage, Real Part"
      m_ur_busac_B = "m:ur:busac:B", "p.u., Phase Voltage, Real Part"
      m_ur_busac_C = "m:ur:busac:C", "p.u., Phase Voltage, Real Part"
      m_ur_busdc = "m:ur:busdc", "p.u., Phase Voltage, Real Part"
      m_ui_busac_A = "m:ui:busac:A", "p.u., Phase Voltage, Imaginary Part"
      m_ui_busac_B = "m:ui:busac:B", "p.u., Phase Voltage, Imaginary Part"
      m_ui_busac_C = "m:ui:busac:C", "p.u., Phase Voltage, Imaginary Part"
      m_ui_busdc = "m:ui:busdc", "p.u., Phase Voltage, Imaginary Part"
      m_u_busac_A = "m:u:busac:A", "p.u., Phase Voltage, Magnitude"
      m_u_busac_B = "m:u:busac:B", "p.u., Phase Voltage, Magnitude"
      m_u_busac_C = "m:u:busac:C", "p.u., Phase Voltage, Magnitude"
      m_u_busdc = "m:u:busdc", "p.u., Phase Voltage, Magnitude"
      m_ir_busac_A = "m:ir:busac:A", "p.u., Phase Current, Real Part"
      m_ir_busac_B = "m:ir:busac:B", "p.u., Phase Current, Real Part"
      m_ir_busac_C = "m:ir:busac:C", "p.u., Phase Current, Real Part"
      m_ir_busdc = "m:ir:busdc", "p.u., Phase Current, Real Part"
      m_ii_busac_A = "m:ii:busac:A", "p.u., Phase Current, Imaginary Part"
      m_ii_busac_B = "m:ii:busac:B", "p.u., Phase Current, Imaginary Part"
      m_ii_busac_C = "m:ii:busac:C", "p.u., Phase Current, Imaginary Part"
      m_ii_busdc = "m:ii:busdc", "p.u., Phase Current, Imaginary Part"
      m_i_busac_A = "m:i:busac:A", "p.u., Phase Current, Magnitude"
      m_i_busac_B = "m:i:busac:B", "p.u., Phase Current, Magnitude"
      m_i_busac_C = "m:i:busac:C", "p.u., Phase Current, Magnitude"
      m_i_busdc = "m:i:busdc", "p.u., Phase Current, Magnitude"
      m_phiui_busac_A = "m:phiui:busac:A", "deg, Angle between Voltage and Current"
      m_phiui_busac_B = "m:phiui:busac:B", "deg, Angle between Voltage and Current"
      m_phiui_busac_C = "m:phiui:busac:C", "deg, Angle between Voltage and Current"
      m_phiui_busdc = "m:phiui:busdc", "deg, Angle between Voltage and Current"
      m_phiu0i0_busac = "m:phiu0i0:busac", "deg, Angle between Voltage and Current in zero-sequence system"
      m_phiu0i0_busdc = "m:phiu0i0:busdc", "deg, Angle between Voltage and Current in zero-sequence system"
      m_phiu1i1_busac = "m:phiu1i1:busac", "deg, Angle between Voltage and Current in positive sequence system"
      m_phiu1i1_busdc = "m:phiu1i1:busdc", "deg, Angle between Voltage and Current in positive sequence system"
      m_phiu2i2_busac = "m:phiu2i2:busac", "deg, Angle between Voltage and Current in negative sequence system"
      m_phiu2i2_busdc = "m:phiu2i2:busdc", "deg, Angle between Voltage and Current in negative sequence system"
      m_Tfct_busac = "m:Tfct:busac", "s, Fault Clearing Time"
      m_Tfct_busdc = "m:Tfct:busdc", "s, Fault Clearing Time"
      m_TfctPh_busac_A = "m:TfctPh:busac:A", "s, Fault Clearing Time"
      m_TfctPh_busac_B = "m:TfctPh:busac:B", "s, Fault Clearing Time"
      m_TfctPh_busac_C = "m:TfctPh:busac:C", "s, Fault Clearing Time"
      m_TfctPh_busdc = "m:TfctPh:busdc", "s, Fault Clearing Time"
      m_Brkload_busac = "m:Brkload:busac", "%, Max. Breaker Loading"
      m_Brkload_busdc = "m:Brkload:busdc", "%, Max. Breaker Loading"
      m_BrkloadPh_busac_A = "m:BrkloadPh:busac:A", "%, Breaker Loading"
      m_BrkloadPh_busac_B = "m:BrkloadPh:busac:B", "%, Breaker Loading"
      m_BrkloadPh_busac_C = "m:BrkloadPh:busac:C", "%, Breaker Loading"
      m_BrkloadPh_busdc = "m:BrkloadPh:busdc", "%, Breaker Loading"
      n_ur_busac_A = "n:ur:busac:A", "p.u., Line-Ground Voltage, Real Part"
      n_ur_busac_B = "n:ur:busac:B", "p.u., Line-Ground Voltage, Real Part"
      n_ur_busac_C = "n:ur:busac:C", "p.u., Line-Ground Voltage, Real Part"
      n_ur_busdc = "n:ur:busdc", "p.u., Line-Ground Voltage, Real Part"
      n_ui_busac_A = "n:ui:busac:A", "p.u., Line-Ground Voltage, Imaginary Part"
      n_ui_busac_B = "n:ui:busac:B", "p.u., Line-Ground Voltage, Imaginary Part"
      n_ui_busac_C = "n:ui:busac:C", "p.u., Line-Ground Voltage, Imaginary Part"
      n_ui_busdc = "n:ui:busdc", "p.u., Line-Ground Voltage, Imaginary Part"
      n_u_busac_A = "n:u:busac:A", "p.u., Line-Ground Voltage, Magnitude"
      n_u_busac_B = "n:u:busac:B", "p.u., Line-Ground Voltage, Magnitude"
      n_u_busac_C = "n:u:busac:C", "p.u., Line-Ground Voltage, Magnitude"
      n_u_busdc = "n:u:busdc", "p.u., Line-Ground Voltage, Magnitude"
      n_upc_busac_A = "n:upc:busac:A", "%, Line-Ground Voltage, Magnitude"
      n_upc_busac_B = "n:upc:busac:B", "%, Line-Ground Voltage, Magnitude"
      n_upc_busac_C = "n:upc:busac:C", "%, Line-Ground Voltage, Magnitude"
      n_upc_busdc = "n:upc:busdc", "%, Line-Ground Voltage, Magnitude"
      n_U_busac_A = "n:U:busac:A", "kV, Line-Ground Voltage, Magnitude"
      n_U_busac_B = "n:U:busac:B", "kV, Line-Ground Voltage, Magnitude"
      n_U_busac_C = "n:U:busac:C", "kV, Line-Ground Voltage, Magnitude"
      n_U_busdc = "n:U:busdc", "kV, Line-Ground Voltage, Magnitude"
      n_phiu_busac_A = "n:phiu:busac:A", "deg, Line-Ground Voltage, Angle"
      n_phiu_busac_B = "n:phiu:busac:B", "deg, Line-Ground Voltage, Angle"
      n_phiu_busac_C = "n:phiu:busac:C", "deg, Line-Ground Voltage, Angle"
      n_phiu_busdc = "n:phiu:busdc", "deg, Line-Ground Voltage, Angle"
      n_phiurel_busac = "n:phiurel:busac", "deg, Voltage, Relative Angle"
      n_phiurel_busdc = "n:phiurel:busdc", "deg, Voltage, Relative Angle"
      n_ul_busac_A = "n:ul:busac:A", "p.u., Line-Line Voltage, Magnitude"
      n_ul_busac_B = "n:ul:busac:B", "p.u., Line-Line Voltage, Magnitude"
      n_ul_busac_C = "n:ul:busac:C", "p.u., Line-Line Voltage, Magnitude"
      n_ul_busdc = "n:ul:busdc", "p.u., Line-Line Voltage, Magnitude"
      n_ulpc_busac_A = "n:ulpc:busac:A", "%, Line-Line Voltage, Magnitude"
      n_ulpc_busac_B = "n:ulpc:busac:B", "%, Line-Line Voltage, Magnitude"
      n_ulpc_busac_C = "n:ulpc:busac:C", "%, Line-Line Voltage, Magnitude"
      n_ulpc_busdc = "n:ulpc:busdc", "%, Line-Line Voltage, Magnitude"
      n_Ul_busac_A = "n:Ul:busac:A", "kV, Line-Line Voltage, Magnitude"
      n_Ul_busac_B = "n:Ul:busac:B", "kV, Line-Line Voltage, Magnitude"
      n_Ul_busac_C = "n:Ul:busac:C", "kV, Line-Line Voltage, Magnitude"
      n_Ul_busdc = "n:Ul:busdc", "kV, Line-Line Voltage, Magnitude"
      n_phiul_busac_A = "n:phiul:busac:A", "deg, Line-Line Voltage, Angle"
      n_phiul_busac_B = "n:phiul:busac:B", "deg, Line-Line Voltage, Angle"
      n_phiul_busac_C = "n:phiul:busac:C", "deg, Line-Line Voltage, Angle"
      n_phiul_busdc = "n:phiul:busdc", "deg, Line-Line Voltage, Angle"
      n_u0_busac = "n:u0:busac", "p.u., Zero-Sequence Voltage, Magnitude"
      n_u0_busdc = "n:u0:busdc", "p.u., Zero-Sequence Voltage, Magnitude"
      n_U0_busac = "n:U0:busac", "kV, Zero-Sequence Voltage, Magnitude"
      n_U0_busdc = "n:U0:busdc", "kV, Zero-Sequence Voltage, Magnitude"
      n_U0x3_busac = "n:U0x3:busac", "kV, 3*U0"
      n_U0x3_busdc = "n:U0x3:busdc", "kV, 3*U0"
      n_phiu0_busac = "n:phiu0:busac", "deg, Zero-Sequence Voltage, Angle"
      n_phiu0_busdc = "n:phiu0:busdc", "deg, Zero-Sequence Voltage, Angle"
      n_u1_busac = "n:u1:busac", "p.u., Positive-Sequence Voltage, Magnitude"
      n_u1_busdc = "n:u1:busdc", "p.u., Positive-Sequence Voltage, Magnitude"
      n_u1pc_busac = "n:u1pc:busac", "%, Positive-Sequence Voltage, Magnitude"
      n_u1pc_busdc = "n:u1pc:busdc", "%, Positive-Sequence Voltage, Magnitude"
      n_u1r_busac = "n:u1r:busac", "p.u., Positive-Sequence Voltage, Real Part"
      n_u1r_busdc = "n:u1r:busdc", "p.u., Positive-Sequence Voltage, Real Part"
      n_u1i_busac = "n:u1i:busac", "p.u., Positive-Sequence Voltage, Imaginary Part"
      n_u1i_busdc = "n:u1i:busdc", "p.u., Positive-Sequence Voltage, Imaginary Part"
      n_U1_busac = "n:U1:busac", "kV, Line-Ground Positive-Sequence Voltage, Magnitude"
      n_U1_busdc = "n:U1:busdc", "kV, Line-Ground Positive-Sequence Voltage, Magnitude"
      n_phiu1_busac = "n:phiu1:busac", "deg, Positive-Sequence Voltage, Angle"
      n_phiu1_busdc = "n:phiu1:busdc", "deg, Positive-Sequence Voltage, Angle"
      n_u2_busac = "n:u2:busac", "p.u., Negative-Sequence Voltage, Magnitude"
      n_u2_busdc = "n:u2:busdc", "p.u., Negative-Sequence Voltage, Magnitude"
      n_U2_busac = "n:U2:busac", "kV, Line-Ground Negative-Sequence Voltage, Magnitude"
      n_U2_busdc = "n:U2:busdc", "kV, Line-Ground Negative-Sequence Voltage, Magnitude"
      n_phiu2_busac = "n:phiu2:busac", "deg, Negative-Sequence Voltage, Angle"
      n_phiu2_busdc = "n:phiu2:busdc", "deg, Negative-Sequence Voltage, Angle"
      n_U1l_busac = "n:U1l:busac", "kV, Line-Line Positive-Sequence Voltage, Magnitude"
      n_U1l_busdc = "n:U1l:busdc", "kV, Line-Line Positive-Sequence Voltage, Magnitude"
      n_U2l_busac = "n:U2l:busac", "kV, Line-Line Negative-Sequence Voltage, Magnitude"
      n_U2l_busdc = "n:U2l:busdc", "kV, Line-Line Negative-Sequence Voltage, Magnitude"
      n_uln_busac_A = "n:uln:busac:A", "p.u., Line-Neutral Voltage, Magnitude"
      n_uln_busac_B = "n:uln:busac:B", "p.u., Line-Neutral Voltage, Magnitude"
      n_uln_busac_C = "n:uln:busac:C", "p.u., Line-Neutral Voltage, Magnitude"
      n_uln_busdc = "n:uln:busdc", "p.u., Line-Neutral Voltage, Magnitude"
      n_Uln_busac_A = "n:Uln:busac:A", "kV, Line-Neutral Voltage, Magnitude"
      n_Uln_busac_B = "n:Uln:busac:B", "kV, Line-Neutral Voltage, Magnitude"
      n_Uln_busac_C = "n:Uln:busac:C", "kV, Line-Neutral Voltage, Magnitude"
      n_Uln_busdc = "n:Uln:busdc", "kV, Line-Neutral Voltage, Magnitude"
      n_phiuln_busac_A = "n:phiuln:busac:A", "deg, Line-Neutral Voltage, Angle"
      n_phiuln_busac_B = "n:phiuln:busac:B", "deg, Line-Neutral Voltage, Angle"
      n_phiuln_busac_C = "n:phiuln:busac:C", "deg, Line-Neutral Voltage, Angle"
      n_phiuln_busdc = "n:phiuln:busdc", "deg, Line-Neutral Voltage, Angle"
      n_un_busac = "n:un:busac", "p.u., Neutral-Ground Voltage, Magnitude"
      n_un_busdc = "n:un:busdc", "p.u., Neutral-Ground Voltage, Magnitude"
      n_Un_busac = "n:Un:busac", "kV, Neutral-Ground Voltage, Magnitude"
      n_Un_busdc = "n:Un:busdc", "kV, Neutral-Ground Voltage, Magnitude"
      n_phiun_busac = "n:phiun:busac", "deg, Neutral-Ground Voltage, Angle"
      n_phiun_busdc = "n:phiun:busdc", "deg, Neutral-Ground Voltage, Angle"
      n_upht_busac_A = "n:upht:busac:A", "p.u., Phase Technology dependent Voltage, Magnitude"
      n_upht_busac_B = "n:upht:busac:B", "p.u., Phase Technology dependent Voltage, Magnitude"
      n_upht_busac_C = "n:upht:busac:C", "p.u., Phase Technology dependent Voltage, Magnitude"
      n_upht_busdc = "n:upht:busdc", "p.u., Phase Technology dependent Voltage, Magnitude"
      n_Upht_busac_A = "n:Upht:busac:A", "kV, Phase Technology dependent Voltage, Magnitude"
      n_Upht_busac_B = "n:Upht:busac:B", "kV, Phase Technology dependent Voltage, Magnitude"
      n_Upht_busac_C = "n:Upht:busac:C", "kV, Phase Technology dependent Voltage, Magnitude"
      n_Upht_busdc = "n:Upht:busdc", "kV, Phase Technology dependent Voltage, Magnitude"
      n_um_busac = "n:um:busac", "p.u., Average-Voltage, Magnitude"
      n_um_busdc = "n:um:busdc", "p.u., Average-Voltage, Magnitude"
      n_Um_busac = "n:Um:busac", "kV, Average-Voltage, Magnitude"
      n_Um_busdc = "n:Um:busdc", "kV, Average-Voltage, Magnitude"
      n_ishcr_busac_A = "n:ishcr:busac:A", "p.u., Short-Circuit Current, real"
      n_ishcr_busac_B = "n:ishcr:busac:B", "p.u., Short-Circuit Current, real"
      n_ishcr_busac_C = "n:ishcr:busac:C", "p.u., Short-Circuit Current, real"
      n_ishcr_busdc = "n:ishcr:busdc", "p.u., Short-Circuit Current, real"
      n_ishci_busac_A = "n:ishci:busac:A", "p.u., Short-Circuit Current, imag"
      n_ishci_busac_B = "n:ishci:busac:B", "p.u., Short-Circuit Current, imag"
      n_ishci_busac_C = "n:ishci:busac:C", "p.u., Short-Circuit Current, imag"
      n_ishci_busdc = "n:ishci:busdc", "p.u., Short-Circuit Current, imag"
      n_ishc_busac_A = "n:ishc:busac:A", "p.u., Short-Circuit Current, magnitude"
      n_ishc_busac_B = "n:ishc:busac:B", "p.u., Short-Circuit Current, magnitude"
      n_ishc_busac_C = "n:ishc:busac:C", "p.u., Short-Circuit Current, magnitude"
      n_ishc_busdc = "n:ishc:busdc", "p.u., Short-Circuit Current, magnitude"
      n_Ishc_busac_A = "n:Ishc:busac:A", "kA, Short-Circuit Current, magnitude"
      n_Ishc_busac_B = "n:Ishc:busac:B", "kA, Short-Circuit Current, magnitude"
      n_Ishc_busac_C = "n:Ishc:busac:C", "kA, Short-Circuit Current, magnitude"
      n_Ishc_busdc = "n:Ishc:busdc", "kA, Short-Circuit Current, magnitude"
      n_inshc_busac = "n:inshc:busac", "p.u., Neutral Short-Circuit Current, magnitude"
      n_inshc_busdc = "n:inshc:busdc", "p.u., Neutral Short-Circuit Current, magnitude"
      n_Inshc_busac = "n:Inshc:busac", "kA, Neutral Short-Circuit Current, magnitude"
      n_Inshc_busdc = "n:Inshc:busdc", "kA, Neutral Short-Circuit Current, magnitude"
      n_fe_busac = "n:fe:busac", "p.u., Electrical Frequency"
      n_fe_busdc = "n:fe:busdc", "p.u., Electrical Frequency"
      n_frdev_busac = "n:frdev:busac", "Hz, average frequency"
      n_frdev_busdc = "n:frdev:busdc", "Hz, average frequency"
      n_dfehz_busac = "n:dfehz:busac", "Hz, Deviation of the El. Frequency"
      n_dfehz_busdc = "n:dfehz:busdc", "Hz, Deviation of the El. Frequency"
      n_dfedt_busac = "n:dfedt:busac", "p.u./s, Derivative of El. Frequency"
      n_dfedt_busdc = "n:dfedt:busdc", "p.u./s, Derivative of El. Frequency"
      n_fehz_busac = "n:fehz:busac", "Hz, Electrical Frequency"
      n_fehz_busdc = "n:fehz:busdc", "Hz, Electrical Frequency"
      n_frnom_busac = "n:frnom:busac", "Hz, Nominal Frequency"
      n_frnom_busdc = "n:frnom:busdc", "Hz, Nominal Frequency"
      s_Pm_in = "s:Pm_in", "p.u., IN, Pulse-modulation factor (input) continuous, double"
      s_f0 = "s:f0", "p.u., IN, Frequency-input continuous, double"
      s_F0Hz = "s:F0Hz", "Hz, IN, Frequency-input continuous, double"
      s_dphiu = "s:dphiu", "rad, IN, Voltage-angle (input), continuous, double"
      s_fref = "s:fref", "p.u., IN, Reference frequency, continuous, double"
      s_Pmr = "s:Pmr", "IN, Pulse-modulation factor (real part), continuous, double"
      s_Pmi = "s:Pmi", "IN, Pulse-modulation factor (imaginary part), continuous, double"
      s_cosref = "s:cosref", "IN, Cos(dq-reference-angle) continuous, double"
      s_sinref = "s:sinref", "IN, Sin(dq-reference-angle) continuous, double"
      s_Pmd = "s:Pmd", "IN, Pulse-modulation factor (d-axis), continuous, double"
      s_Pmq = "s:Pmq", "IN, Pulse-modulation factor (q-axis), continuous, double"
      s_id_ref = "s:id_ref", "p.u., IN, d-axis current reference, continuous, double"
      s_iq_ref = "s:iq_ref", "p.u., IN, q-axis current reference, continuous, double"
      s_i2d_ref = "s:i2d_ref", "p.u., IN, Negative sequence current, d-axis reference, continuous, double"
      s_i2q_ref = "s:i2q_ref", "p.u., IN, Negative sequence current, q-axis reference, continuous, double"
      s_u2r_in = "s:u2r_in", "p.u., IN, Negative Sequence Voltage Input, Real Part, continuous, double"
      s_u2i_in = "s:u2i_in", "p.u., IN, Negative Sequence Voltage Input, Imaginary Part continuous, double"
      s_block = "s:block", "p.u., IN, Blocking, continuous, double"
      s_mdc = "s:mdc", "p.u., IN, Insertion index, DC-bias, continuous, double"
      s_u0r_in = "s:u0r_in", "p.u., IN, Zero Sequence Voltage Input, Real Part, continuous, double"
      s_u0i_in = "s:u0i_in", "p.u., IN, Zero Sequence Voltage Input, Imaginary Part, continuous, double"
      s_xspeed = "s:xspeed", "p.u., OUT, Frequency, continuous, double"
      s_iDC = "s:iDC", "p.u., OUT, DC-current, continuous, double"
      s_uDC = "s:uDC", "p.u., OUT, DC-voltage, continuous, double"
      s_iAC_r = "s:iAC_r", "p.u., OUT, AC-current, real part, continuous, double"
      s_iAC_i = "s:iAC_i", "p.u., OUT, AC-current, imaginary part, continuous, double"
      s_uAC_r = "s:uAC_r", "p.u., OUT, AC-voltage, real part, continuous, double"
      s_uAC_i = "s:uAC_i", "p.u., OUT, AC-voltage, imaginary part, continuous, double"
      s_id = "s:id", "p.u., OUT, Current, d-axis continuous, double"
      s_iq = "s:iq", "p.u., OUT, Current, q-axis continuous, double"
      s_yUcell = "s:yUcell", "OUT, Voltage across cell capacitor, output, continuous, double"
      s_phiu = "s:phiu", "rad, STATE, Voltage-angle, continuous, double"
      s_xd = "s:xd", "p.u., STATE, State variable, current controller d-axis, continuous, double"
      s_xq = "s:xq", "p.u., STATE, State variable, current controller q-axis, continuous, double"
      s_Ucell = "s:Ucell", "kV, STATE, Voltage across cell capacitor, continuous, double"
      s_phiu_dt = "s:phiu:dt", "rad/s, d/dt, Voltage-angle, derivative, continuous, double"
      s_xd_dt = "s:xd:dt", "p.u./s, d/dt, State variable, current controller d-axis, derivative, continuous, double"
      s_xq_dt = "s:xq:dt", "p.u./s, d/dt, State variable, current controller q-axis, derivative, continuous, double"
      s_Ucell_dt = "s:Ucell:dt", "kV/s, d/dt, Voltage across cell capacitor, derivative, continuous, double"
      c_hpi = "c:hpi", "rad/s, Nominal-angular-frequency"
      c_snomn = "c:snomn", "MVA, Total rated apparent power (nparnum*Snom)"
      c_Pm = "c:Pm", "Pulse-modulation factor"
      c_i_mod = "c:i_mod", "Modulation"
      c_K0 = "c:K0", "Vac/Vdc at PWM=1"
      c_DCPoleLosses = "c:DCPoleLosses", "MW, DC pole losses"
      c_Pmd_ctrl = "c:Pmd_ctrl", "Pulse-modulation factor (d-axis)"
      c_Pmq_ctrl = "c:Pmq_ctrl", "Pulse-modulation factor (q-axis)"
      c_Ccell = "c:Ccell", "F, Cell capacitor"
      c_i_acdc = "c:i_acdc", "Control mode"
      e_iAstabint = "e:iAstabint", "A-stable integration algorithm"
      e_iVIsource = "e:iVIsource", "Independent AC and DC voltage/current source model"
      e_Rac = "e:Rac", "Ohm, AC resistance"
      e_Lac = "e:Lac", "mH, AC inductance"
      e_R0toR1ac = "e:R0toR1ac", "R0/R1 ratio"
      e_X0toX1ac = "e:X0toX1ac", "X0/X1 ratio"
      e_Rdc = "e:Rdc", "Ohm, DC resistance"
      e_Ldc = "e:Ldc", "mH, DC inductance"
      e_Pnold = "e:Pnold", "kW, No-load losses"
      e_swtLossFactor = "e:swtLossFactor", "kW/A, Switching loss factor"
      e_resLossFactor = "e:resLossFactor", "Ohm, Resistive loss factor"
      e_Xd = "e:Xd", "Ohm, Commutation reactance"
      e_Cdc = "e:Cdc", "uF, Cell capacitor"
      e_mmcCmod = "e:mmcCmod", "uF, Submodule capacitance"
      e_NmmcSM = "e:NmmcSM", "Number of submodules per arm"
      e_r2 = "e:r2", "p.u., Resistance, r2"
      e_x2 = "e:x2", "p.u., Reactance, x2"
      e_i_ctrl = "e:i_ctrl", "Use integrated current controller"
      e_Kd = "e:Kd", "Kd: d-axis, proportional gain"
      e_Td = "e:Td", "s, Td: d-axis, integration time constant"
      e_Kq = "e:Kq", "Kq: q-axis, proportional gain"
      e_Tq = "e:Tq", "s, Tq: q-axis, integration time constant"

    class ElmXnet(Enum):
      _init_ = 'value __doc__'

      m_u0_bus1 = "m:u0:bus1", "p.u., Zero-Sequence-Voltage, Magnitude"
      m_phiu0_bus1 = "m:phiu0:bus1", "deg, Zero-Sequence-Voltage, Angle"
      m_u1r_bus1 = "m:u1r:bus1", "p.u., Positive-Sequence-Voltage, Real Part"
      m_u1i_bus1 = "m:u1i:bus1", "p.u., Positive-Sequence-Voltage, Imaginary Part"
      m_u1_bus1 = "m:u1:bus1", "p.u., Positive-Sequence-Voltage, Magnitude"
      m_U1_bus1 = "m:U1:bus1", "kV, Line-Ground Positive-Sequence-Voltage, Magnitude"
      m_U1l_bus1 = "m:U1l:bus1", "kV, Line-Line Positive-Sequence-Voltage, Magnitude"
      m_phiu1_bus1 = "m:phiu1:bus1", "deg, Positive-Sequence-Voltage, Angle"
      m_u2_bus1 = "m:u2:bus1", "p.u., Negative-Sequence-Voltage, Magnitude"
      m_phiu2_bus1 = "m:phiu2:bus1", "deg, Negative-Sequence-Voltage, Angle"
      m_inet_bus1_A = "m:inet:bus1:A", "p.u., Phase Current, Magnitude"
      m_inet_bus1_B = "m:inet:bus1:B", "p.u., Phase Current, Magnitude"
      m_inet_bus1_C = "m:inet:bus1:C", "p.u., Phase Current, Magnitude"
      m_I_bus1_A = "m:I:bus1:A", "kA, Phase Current, Magnitude"
      m_I_bus1_B = "m:I:bus1:B", "kA, Phase Current, Magnitude"
      m_I_bus1_C = "m:I:bus1:C", "kA, Phase Current, Magnitude"
      m_phii_bus1_A = "m:phii:bus1:A", "deg, Phase Current, Angle"
      m_phii_bus1_B = "m:phii:bus1:B", "deg, Phase Current, Angle"
      m_phii_bus1_C = "m:phii:bus1:C", "deg, Phase Current, Angle"
      m_i0r_bus1 = "m:i0r:bus1", "p.u., Zero-Sequence Current, Real Part"
      m_i0i_bus1 = "m:i0i:bus1", "p.u., Zero-Sequence Current, Imaginary Part"
      m_i0_bus1 = "m:i0:bus1", "p.u., Zero-Sequence Current, Magnitude"
      m_I0_bus1 = "m:I0:bus1", "kA, Zero-Sequence Current, Magnitude"
      m_I0x3_bus1 = "m:I0x3:bus1", "kA, 3*I0"
      m_phii0_bus1 = "m:phii0:bus1", "deg, Zero-Sequence Current, Angle"
      m_i1r_bus1 = "m:i1r:bus1", "p.u., Positive-Sequence Current, Real Part"
      m_i1i_bus1 = "m:i1i:bus1", "p.u., Positive-Sequence Current, Imaginary Part"
      m_i1_bus1 = "m:i1:bus1", "p.u., Positive-Sequence Current, Magnitude"
      m_I1_bus1 = "m:I1:bus1", "kA, Positive-Sequence Current, Magnitude"
      m_phii1_bus1 = "m:phii1:bus1", "deg, Positive-Sequence Current, Angle"
      m_i2r_bus1 = "m:i2r:bus1", "p.u., Negative-Sequence Current, Real Part"
      m_i2i_bus1 = "m:i2i:bus1", "p.u., Negative-Sequence Current, Imaginary Part"
      m_i2_bus1 = "m:i2:bus1", "p.u., Negative-Sequence Current, Magnitude"
      m_I2_bus1 = "m:I2:bus1", "kA, Negative-Sequence Current, Magnitude"
      m_phii2_bus1 = "m:phii2:bus1", "deg, Negative-Sequence Current, Angle"
      m_P_bus1_A = "m:P:bus1:A", "MW, Active Power"
      m_P_bus1_B = "m:P:bus1:B", "MW, Active Power"
      m_P_bus1_C = "m:P:bus1:C", "MW, Active Power"
      m_Q_bus1_A = "m:Q:bus1:A", "Mvar, Reactive Power"
      m_Q_bus1_B = "m:Q:bus1:B", "Mvar, Reactive Power"
      m_Q_bus1_C = "m:Q:bus1:C", "Mvar, Reactive Power"
      m_S_bus1_A = "m:S:bus1:A", "MVA, Apparent Power"
      m_S_bus1_B = "m:S:bus1:B", "MVA, Apparent Power"
      m_S_bus1_C = "m:S:bus1:C", "MVA, Apparent Power"
      m_cosphi_bus1_A = "m:cosphi:bus1:A", "Power Factor"
      m_cosphi_bus1_B = "m:cosphi:bus1:B", "Power Factor"
      m_cosphi_bus1_C = "m:cosphi:bus1:C", "Power Factor"
      m_tanphi_bus1_A = "m:tanphi:bus1:A", "tan(phi)"
      m_tanphi_bus1_B = "m:tanphi:bus1:B", "tan(phi)"
      m_tanphi_bus1_C = "m:tanphi:bus1:C", "tan(phi)"
      m_Psum_bus1 = "m:Psum:bus1", "MW, Total Active Power"
      m_Qsum_bus1 = "m:Qsum:bus1", "Mvar, Total Reactive Power"
      m_Ssum_bus1 = "m:Ssum:bus1", "MVA, Total Apparent Power"
      m_cosphisum_bus1 = "m:cosphisum:bus1", "Total Power Factor"
      m_tanphisum_bus1 = "m:tanphisum:bus1", "Total tan(phi)"
      m_Pln_bus1_A = "m:Pln:bus1:A", "MW, Active Power (Line-Neutral)"
      m_Pln_bus1_B = "m:Pln:bus1:B", "MW, Active Power (Line-Neutral)"
      m_Pln_bus1_C = "m:Pln:bus1:C", "MW, Active Power (Line-Neutral)"
      m_Qln_bus1_A = "m:Qln:bus1:A", "Mvar, Reactive Power (Line-Neutral)"
      m_Qln_bus1_B = "m:Qln:bus1:B", "Mvar, Reactive Power (Line-Neutral)"
      m_Qln_bus1_C = "m:Qln:bus1:C", "Mvar, Reactive Power (Line-Neutral)"
      m_P1_bus1 = "m:P1:bus1", "MW, Positive Sequence Active Power"
      m_Q1_bus1 = "m:Q1:bus1", "Mvar, Positive Sequence Reactive Power"
      m_P2_bus1 = "m:P2:bus1", "MW, Negative Sequence Active Power"
      m_Q2_bus1 = "m:Q2:bus1", "Mvar, Negative Sequence Reactive Power"
      m_P0_bus1 = "m:P0:bus1", "MW, Zero Sequence Active Power"
      m_Q0_bus1 = "m:Q0:bus1", "Mvar, Zero Sequence Reactive Power"
      m_i1P_bus1 = "m:i1P:bus1", "p.u., Positive-Sequence Active Current"
      m_i1Q_bus1 = "m:i1Q:bus1", "p.u., Positive-Sequence Reactive Current"
      m_I1P_bus1 = "m:I1P:bus1", "kA, Positive-Sequence Active Current"
      m_I1Q_bus1 = "m:I1Q:bus1", "kA, Positive-Sequence Reactive Current"
      m_i2P_bus1 = "m:i2P:bus1", "p.u., Negative-Sequence Active Current"
      m_i2Q_bus1 = "m:i2Q:bus1", "p.u., Negative-Sequence Reactive Current"
      m_I2P_bus1 = "m:I2P:bus1", "kA, Negative-Sequence Active Current"
      m_I2Q_bus1 = "m:I2Q:bus1", "kA, Negative-Sequence Reactive Current"
      m_ur_bus1_A = "m:ur:bus1:A", "p.u., Phase Voltage, Real Part"
      m_ur_bus1_B = "m:ur:bus1:B", "p.u., Phase Voltage, Real Part"
      m_ur_bus1_C = "m:ur:bus1:C", "p.u., Phase Voltage, Real Part"
      m_ui_bus1_A = "m:ui:bus1:A", "p.u., Phase Voltage, Imaginary Part"
      m_ui_bus1_B = "m:ui:bus1:B", "p.u., Phase Voltage, Imaginary Part"
      m_ui_bus1_C = "m:ui:bus1:C", "p.u., Phase Voltage, Imaginary Part"
      m_u_bus1_A = "m:u:bus1:A", "p.u., Phase Voltage, Magnitude"
      m_u_bus1_B = "m:u:bus1:B", "p.u., Phase Voltage, Magnitude"
      m_u_bus1_C = "m:u:bus1:C", "p.u., Phase Voltage, Magnitude"
      m_ir_bus1_A = "m:ir:bus1:A", "p.u., Phase Current, Real Part"
      m_ir_bus1_B = "m:ir:bus1:B", "p.u., Phase Current, Real Part"
      m_ir_bus1_C = "m:ir:bus1:C", "p.u., Phase Current, Real Part"
      m_ii_bus1_A = "m:ii:bus1:A", "p.u., Phase Current, Imaginary Part"
      m_ii_bus1_B = "m:ii:bus1:B", "p.u., Phase Current, Imaginary Part"
      m_ii_bus1_C = "m:ii:bus1:C", "p.u., Phase Current, Imaginary Part"
      m_i_bus1_A = "m:i:bus1:A", "p.u., Phase Current, Magnitude"
      m_i_bus1_B = "m:i:bus1:B", "p.u., Phase Current, Magnitude"
      m_i_bus1_C = "m:i:bus1:C", "p.u., Phase Current, Magnitude"
      m_phiui_bus1_A = "m:phiui:bus1:A", "deg, Angle between Voltage and Current"
      m_phiui_bus1_B = "m:phiui:bus1:B", "deg, Angle between Voltage and Current"
      m_phiui_bus1_C = "m:phiui:bus1:C", "deg, Angle between Voltage and Current"
      m_phiu0i0_bus1 = "m:phiu0i0:bus1", "deg, Angle between Voltage and Current in zero-sequence system"
      m_phiu1i1_bus1 = "m:phiu1i1:bus1", "deg, Angle between Voltage and Current in positive sequence system"
      m_phiu2i2_bus1 = "m:phiu2i2:bus1", "deg, Angle between Voltage and Current in negative sequence system"
      m_Tfct_bus1 = "m:Tfct:bus1", "s, Fault Clearing Time"
      m_TfctPh_bus1_A = "m:TfctPh:bus1:A", "s, Fault Clearing Time"
      m_TfctPh_bus1_B = "m:TfctPh:bus1:B", "s, Fault Clearing Time"
      m_TfctPh_bus1_C = "m:TfctPh:bus1:C", "s, Fault Clearing Time"
      m_Brkload_bus1 = "m:Brkload:bus1", "%, Max. Breaker Loading"
      m_BrkloadPh_bus1_A = "m:BrkloadPh:bus1:A", "%, Breaker Loading"
      m_BrkloadPh_bus1_B = "m:BrkloadPh:bus1:B", "%, Breaker Loading"
      m_BrkloadPh_bus1_C = "m:BrkloadPh:bus1:C", "%, Breaker Loading"
      n_ur_bus1_A = "n:ur:bus1:A", "p.u., Line-Ground Voltage, Real Part"
      n_ur_bus1_B = "n:ur:bus1:B", "p.u., Line-Ground Voltage, Real Part"
      n_ur_bus1_C = "n:ur:bus1:C", "p.u., Line-Ground Voltage, Real Part"
      n_ui_bus1_A = "n:ui:bus1:A", "p.u., Line-Ground Voltage, Imaginary Part"
      n_ui_bus1_B = "n:ui:bus1:B", "p.u., Line-Ground Voltage, Imaginary Part"
      n_ui_bus1_C = "n:ui:bus1:C", "p.u., Line-Ground Voltage, Imaginary Part"
      n_u_bus1_A = "n:u:bus1:A", "p.u., Line-Ground Voltage, Magnitude"
      n_u_bus1_B = "n:u:bus1:B", "p.u., Line-Ground Voltage, Magnitude"
      n_u_bus1_C = "n:u:bus1:C", "p.u., Line-Ground Voltage, Magnitude"
      n_upc_bus1_A = "n:upc:bus1:A", "%, Line-Ground Voltage, Magnitude"
      n_upc_bus1_B = "n:upc:bus1:B", "%, Line-Ground Voltage, Magnitude"
      n_upc_bus1_C = "n:upc:bus1:C", "%, Line-Ground Voltage, Magnitude"
      n_U_bus1_A = "n:U:bus1:A", "kV, Line-Ground Voltage, Magnitude"
      n_U_bus1_B = "n:U:bus1:B", "kV, Line-Ground Voltage, Magnitude"
      n_U_bus1_C = "n:U:bus1:C", "kV, Line-Ground Voltage, Magnitude"
      n_phiu_bus1_A = "n:phiu:bus1:A", "deg, Line-Ground Voltage, Angle"
      n_phiu_bus1_B = "n:phiu:bus1:B", "deg, Line-Ground Voltage, Angle"
      n_phiu_bus1_C = "n:phiu:bus1:C", "deg, Line-Ground Voltage, Angle"
      n_phiurel_bus1 = "n:phiurel:bus1", "deg, Voltage, Relative Angle"
      n_ul_bus1_A = "n:ul:bus1:A", "p.u., Line-Line Voltage, Magnitude"
      n_ul_bus1_B = "n:ul:bus1:B", "p.u., Line-Line Voltage, Magnitude"
      n_ul_bus1_C = "n:ul:bus1:C", "p.u., Line-Line Voltage, Magnitude"
      n_ulpc_bus1_A = "n:ulpc:bus1:A", "%, Line-Line Voltage, Magnitude"
      n_ulpc_bus1_B = "n:ulpc:bus1:B", "%, Line-Line Voltage, Magnitude"
      n_ulpc_bus1_C = "n:ulpc:bus1:C", "%, Line-Line Voltage, Magnitude"
      n_Ul_bus1_A = "n:Ul:bus1:A", "kV, Line-Line Voltage, Magnitude"
      n_Ul_bus1_B = "n:Ul:bus1:B", "kV, Line-Line Voltage, Magnitude"
      n_Ul_bus1_C = "n:Ul:bus1:C", "kV, Line-Line Voltage, Magnitude"
      n_phiul_bus1_A = "n:phiul:bus1:A", "deg, Line-Line Voltage, Angle"
      n_phiul_bus1_B = "n:phiul:bus1:B", "deg, Line-Line Voltage, Angle"
      n_phiul_bus1_C = "n:phiul:bus1:C", "deg, Line-Line Voltage, Angle"
      n_u0_bus1 = "n:u0:bus1", "p.u., Zero-Sequence Voltage, Magnitude"
      n_U0_bus1 = "n:U0:bus1", "kV, Zero-Sequence Voltage, Magnitude"
      n_U0x3_bus1 = "n:U0x3:bus1", "kV, 3*U0"
      n_phiu0_bus1 = "n:phiu0:bus1", "deg, Zero-Sequence Voltage, Angle"
      n_u1_bus1 = "n:u1:bus1", "p.u., Positive-Sequence Voltage, Magnitude"
      n_u1pc_bus1 = "n:u1pc:bus1", "%, Positive-Sequence Voltage, Magnitude"
      n_u1r_bus1 = "n:u1r:bus1", "p.u., Positive-Sequence Voltage, Real Part"
      n_u1i_bus1 = "n:u1i:bus1", "p.u., Positive-Sequence Voltage, Imaginary Part"
      n_U1_bus1 = "n:U1:bus1", "kV, Line-Ground Positive-Sequence Voltage, Magnitude"
      n_phiu1_bus1 = "n:phiu1:bus1", "deg, Positive-Sequence Voltage, Angle"
      n_u2_bus1 = "n:u2:bus1", "p.u., Negative-Sequence Voltage, Magnitude"
      n_U2_bus1 = "n:U2:bus1", "kV, Line-Ground Negative-Sequence Voltage, Magnitude"
      n_phiu2_bus1 = "n:phiu2:bus1", "deg, Negative-Sequence Voltage, Angle"
      n_U1l_bus1 = "n:U1l:bus1", "kV, Line-Line Positive-Sequence Voltage, Magnitude"
      n_U2l_bus1 = "n:U2l:bus1", "kV, Line-Line Negative-Sequence Voltage, Magnitude"
      n_uln_bus1_A = "n:uln:bus1:A", "p.u., Line-Neutral Voltage, Magnitude"
      n_uln_bus1_B = "n:uln:bus1:B", "p.u., Line-Neutral Voltage, Magnitude"
      n_uln_bus1_C = "n:uln:bus1:C", "p.u., Line-Neutral Voltage, Magnitude"
      n_Uln_bus1_A = "n:Uln:bus1:A", "kV, Line-Neutral Voltage, Magnitude"
      n_Uln_bus1_B = "n:Uln:bus1:B", "kV, Line-Neutral Voltage, Magnitude"
      n_Uln_bus1_C = "n:Uln:bus1:C", "kV, Line-Neutral Voltage, Magnitude"
      n_phiuln_bus1_A = "n:phiuln:bus1:A", "deg, Line-Neutral Voltage, Angle"
      n_phiuln_bus1_B = "n:phiuln:bus1:B", "deg, Line-Neutral Voltage, Angle"
      n_phiuln_bus1_C = "n:phiuln:bus1:C", "deg, Line-Neutral Voltage, Angle"
      n_un_bus1 = "n:un:bus1", "p.u., Neutral-Ground Voltage, Magnitude"
      n_Un_bus1 = "n:Un:bus1", "kV, Neutral-Ground Voltage, Magnitude"
      n_phiun_bus1 = "n:phiun:bus1", "deg, Neutral-Ground Voltage, Angle"
      n_upht_bus1_A = "n:upht:bus1:A", "p.u., Phase Technology dependent Voltage, Magnitude"
      n_upht_bus1_B = "n:upht:bus1:B", "p.u., Phase Technology dependent Voltage, Magnitude"
      n_upht_bus1_C = "n:upht:bus1:C", "p.u., Phase Technology dependent Voltage, Magnitude"
      n_Upht_bus1_A = "n:Upht:bus1:A", "kV, Phase Technology dependent Voltage, Magnitude"
      n_Upht_bus1_B = "n:Upht:bus1:B", "kV, Phase Technology dependent Voltage, Magnitude"
      n_Upht_bus1_C = "n:Upht:bus1:C", "kV, Phase Technology dependent Voltage, Magnitude"
      n_um_bus1 = "n:um:bus1", "p.u., Average-Voltage, Magnitude"
      n_Um_bus1 = "n:Um:bus1", "kV, Average-Voltage, Magnitude"
      n_ishcr_bus1_A = "n:ishcr:bus1:A", "p.u., Short-Circuit Current, real"
      n_ishcr_bus1_B = "n:ishcr:bus1:B", "p.u., Short-Circuit Current, real"
      n_ishcr_bus1_C = "n:ishcr:bus1:C", "p.u., Short-Circuit Current, real"
      n_ishci_bus1_A = "n:ishci:bus1:A", "p.u., Short-Circuit Current, imag"
      n_ishci_bus1_B = "n:ishci:bus1:B", "p.u., Short-Circuit Current, imag"
      n_ishci_bus1_C = "n:ishci:bus1:C", "p.u., Short-Circuit Current, imag"
      n_ishc_bus1_A = "n:ishc:bus1:A", "p.u., Short-Circuit Current, magnitude"
      n_ishc_bus1_B = "n:ishc:bus1:B", "p.u., Short-Circuit Current, magnitude"
      n_ishc_bus1_C = "n:ishc:bus1:C", "p.u., Short-Circuit Current, magnitude"
      n_Ishc_bus1_A = "n:Ishc:bus1:A", "kA, Short-Circuit Current, magnitude"
      n_Ishc_bus1_B = "n:Ishc:bus1:B", "kA, Short-Circuit Current, magnitude"
      n_Ishc_bus1_C = "n:Ishc:bus1:C", "kA, Short-Circuit Current, magnitude"
      n_inshc_bus1 = "n:inshc:bus1", "p.u., Neutral Short-Circuit Current, magnitude"
      n_Inshc_bus1 = "n:Inshc:bus1", "kA, Neutral Short-Circuit Current, magnitude"
      n_fe_bus1 = "n:fe:bus1", "p.u., Electrical Frequency"
      n_frdev_bus1 = "n:frdev:bus1", "Hz, average frequency"
      n_dfehz_bus1 = "n:dfehz:bus1", "Hz, Deviation of the El. Frequency"
      n_dfedt_bus1 = "n:dfedt:bus1", "p.u./s, Derivative of El. Frequency"
      n_fehz_bus1 = "n:fehz:bus1", "Hz, Electrical Frequency"
      n_frnom_bus1 = "n:frnom:bus1", "Hz, Nominal Frequency"
      s_ve = "s:ve", "p.u., IN, Excitation voltage, continuous, double"
      s_pt = "s:pt", "p.u., IN, Turbine Power, continuous, double"
      s_xmdm = "s:xmdm", "p.u., IN, Mechanical Torque, continuous, double"
      s_fref = "s:fref", "p.u., IN, Reference Frequency, continuous, double"
      s_uout = "s:uout", "p.u., OUT, Voltage continuous, double"
      s_xspeed = "s:xspeed", "p.u., OUT, Speed, continuous, double"
      s_psisr = "s:psisr", "p.u., STATE, Stator Flux, Real Part, continuous, double"
      s_psisi = "s:psisi", "p.u., STATE, Stator Flux, Imaginary Part, continuous, double"
      s_psie = "s:psie", "p.u., STATE, Excitation-Flux continuous, double"
      s_psiD = "s:psiD", "p.u., STATE, Flux in D-winding, continuous, double"
      s_psix = "s:psix", "p.u., STATE, Flux in x-winding, continuous, double"
      s_psiQ = "s:psiQ", "p.u., STATE, Flux in Q-winding, continuous, double"
      s_speed = "s:speed", "p.u., STATE, Speed, continuous, double"
      s_phi = "s:phi", "p.u., STATE, Rotor-angle, continuous, double"
      s_psisr_dt = "s:psisr:dt", "p.u./s, d/dt, Stator Flux, Real Part, derivative, continuous, double"
      s_psisi_dt = "s:psisi:dt", "p.u./s, d/dt, Stator Flux, Imaginary Part, derivative continuous, double"
      s_psie_dt = "s:psie:dt", "p.u./s, d/dt, Excitation-Flux, derivative, continuous, double"
      s_psiD_dt = "s:psiD:dt", "p.u./s, d/dt, Flux in D-winding, derivative, continuous, double"
      s_psix_dt = "s:psix:dt", "p.u./s, d/dt, Flux in x-winding, derivative, continuous, double"
      s_psiQ_dt = "s:psiQ:dt", "p.u./s, d/dt, Flux in Q-winding, derivative, continuous, double"
      s_speed_dt = "s:speed:dt", "p.u./s, d/dt, Speed, derivative, continuous, double"
      s_phi_dt = "s:phi:dt", "p.u./s, d/dt, Rotor-angle, derivative continuous, double"
      c_sgnn = "c:sgnn", "MVA, Total rated apparent power (ngnum*sgn)"
      c_c2 = "c:c2", "Voltage Coefficient (uknom/ugn)"
      c_Te = "c:Te", "s, Excitation-Time constant"
      c_TD = "c:TD", "s, Time constant of D-winding"
      c_Tx = "c:Tx", "s, Time Constant of x-winding"
      c_TQ = "c:TQ", "s, Time Constant of Q-winding"
      c_Tag = "c:Tag", "s, Acceleration Time"
      c_xme = "c:xme", "p.u., Electrical Torque"
      c_Yd = "c:Yd", "p.u., Inverted reactance matrix"
      c_Yq = "c:Yq", "p.u., Inverted reactance matrix"
      c_frnom = "c:frnom", "Hz, Nominal frequency"
      c_hpi = "c:hpi", "rad/s, Nominal angular frequency"
      c_loading = "c:loading", "%, Loading"
      c_id = "c:id", "Direct Axis Current"
      c_iq = "c:iq", "Quadrature Axis Current"
      c_ie = "c:ie", "Excitation Current"
      c_xmt = "c:xmt", "Mechanical Torque"
      c_r0 = "c:r0", "Zero Sequence Resistance"
      c_x0 = "c:x0", "Zero Sequence Reactance"
      c_r2 = "c:r2", "Negative Sequence Resistance"
      c_x2 = "c:x2", "Negative Sequence Reactance"
      c_rearth = "c:rearth", "Grounding Resistance"
      c_xearth = "c:xearth", "Grounding Reactance"
      c_addmt = "c:addmt", "Additional Torque"
      c_psid = "c:psid", "Flux in d-axis"
      c_psiq = "c:psiq", "Flux in q-axis"
      c_pgt = "c:pgt", "p.u., Active power"
      c_fe = "c:fe", "El. frequency"
      c_xdss = "c:xdss", "p.u., Subtransient reactance (d-axis)"
      c_xqss = "c:xqss", "p.u., Subtransient reactance (q-axis)"
      c_Ptspin = "c:Ptspin", "MW, Total Turbine Power of Area"
      c_Pnomspin = "c:Pnomspin", "MW, Total Nominal Power of Spinning Machines in Area"
      c_Presspin = "c:Presspin", "MW, Spinning Reserve of Area"
      c_Tagav = "c:Tagav", "s, System Inertia of Area"
      c_p_mag = "c:p_mag", "Participation, Magnitude"
      c_p_phi = "c:p_phi", "deg, Participation, Angle"
      c_p_mags = "c:p_mags", "Participation, Magnitude (signed)"
      c_lEVec_mag = "c:lEVec_mag", "Controllability, Magnitude"
      c_lEVec_phi = "c:lEVec_phi", "deg, Controllability, Angle"
      c_lEVec_mags = "c:lEVec_mags", "Controllability, Magnitude (signed)"
      c_rEVec_mag = "c:rEVec_mag", "Observability, Magnitude"
      c_rEVec_phi = "c:rEVec_phi", "deg, Observability, Angle"
      c_rEVec_mags = "c:rEVec_mags", "Observability, Magnitude (signed)"
      c_droop = "c:droop", "System droop"
      c_dp = "c:dp", "p.u., Output of internal Speed Controller"
      c_psidss = "c:psidss", "p.u., Subtransient Flux (d-axis)"
      c_psiqss = "c:psiqss", "p.u., Subtransient Flux (q-axis)"
      e_iAstabint = "e:iAstabint", "A-stable integration algorithm"
      e_tag = "e:tag", "s, Acceleration Time Constant"
      e_K = "e:K", "MW/Hz, Secondary Frequency Bias"
      e_cused = "e:cused", "Use for calculation"
      e_snss = "e:snss", "MVA, Short-Circuit Power Sk''max"
      e_ikss = "e:ikss", "kA, Short-Circuit Current Ik''max"
      e_cmax = "e:cmax", "c-Factor (max.)"
      e_rntxn = "e:rntxn", "R/X Ratio (max.)"
      e_xntrn = "e:xntrn", "X/R Ratio (max.)"
      e_x0tx1 = "e:x0tx1", "X0/X1 max."
      e_r0tx0 = "e:r0tx0", "R0/X0 max."
      e_snssmin = "e:snssmin", "MVA, Short-Circuit Power Sk''min"
      e_ikssmin = "e:ikssmin", "kA, Short-Circuit Current Ik''min"
      e_cmin = "e:cmin", "c-Factor (min.)"
      e_rntxnmin = "e:rntxnmin", "R/X Ratio (min.)"
      e_xntrnmin = "e:xntrnmin", "X/R Ratio (min.)"
      e_x0tx1min = "e:x0tx1min", "X0/X1 min."
      e_r0tx0min = "e:r0tx0min", "R0/X0 min."
      e_tds = "e:tds", "s, Td'"
      e_tqs = "e:tqs", "s, Tq'"
      e_tdss = "e:tdss", "s, Td''"
      e_tqss = "e:tqss", "s, Tq''"
      e_cosn = "e:cosn", "Power Factor"
      e_snom = "e:snom", "MVA, Nominal Apparent Power"
      e_xd = "e:xd", "p.u., xd"
      e_xq = "e:xq", "p.u., xq"
      e_xds = "e:xds", "p.u., xd'"
      e_xqs = "e:xqs", "p.u., xq'"

    class ElmZone(Enum):
      _init_ = 'value __doc__'

      c_Pgen = "c:Pgen", "MW, Generators, Active Power"
      c_Qgen = "c:Qgen", "Mvar, Generators, Reactive Power"
      c_Sgen = "c:Sgen", "MVA, Generators, Apparent Power"
      c_cosgen = "c:cosgen", "Generators, Power Factor"
      c_Pmot = "c:Pmot", "MW, Motor Loads, Active Power"
      c_Qmot = "c:Qmot", "Mvar, Motor Loads, Reactive Power"
      c_Smot = "c:Smot", "MVA, Motor Loads, Apparent Power"
      c_cosmot = "c:cosmot", "Motor Loads, Power Factor"
      c_Pload = "c:Pload", "MW, Loads, Active Power"
      c_Qload = "c:Qload", "Mvar, Loads, Reactive Power"
      c_Sload = "c:Sload", "MVA, Loads, Apparent Power"
      c_cosload = "c:cosload", "Loads, Power Factor"
      c_Pcomp = "c:Pcomp", "MW, Compensation (Losses)"
      c_Qcomp = "c:Qcomp", "Mvar, Compensation, C"
      c_Qcompr = "c:Qcompr", "Mvar, Compensation, L"
      c_Pnet = "c:Pnet", "MW, External Networks, Active Power"
      c_Qnet = "c:Qnet", "Mvar, External Networks, Reactive Power"
      c_Snet = "c:Snet", "MVA, External Networks, Apparent Power"
      c_cosnet = "c:cosnet", "External Networks, Power Factor"
      c_Pinter = "c:Pinter", "MW, Interchange Flow, Active Power"
      c_Qinter = "c:Qinter", "Mvar, Interchange Flow, Reactive Power"
      c_Ploss = "c:Ploss", "MW, Losses, Active Power"
      c_Qloss = "c:Qloss", "Mvar, Losses, Reactive Power"
      c_Pt = "c:Pt", "MW, Turbine Power"
      c_Pres = "c:Pres", "MW, Available Reserve"

    class ElmZpu(Enum):
      _init_ = 'value __doc__'

      m_u0_bus1 = "m:u0:bus1", "p.u., Zero-Sequence-Voltage, Magnitude"
      m_u0_bus2 = "m:u0:bus2", "p.u., Zero-Sequence-Voltage, Magnitude"
      m_phiu0_bus1 = "m:phiu0:bus1", "deg, Zero-Sequence-Voltage, Angle"
      m_phiu0_bus2 = "m:phiu0:bus2", "deg, Zero-Sequence-Voltage, Angle"
      m_u1r_bus1 = "m:u1r:bus1", "p.u., Positive-Sequence-Voltage, Real Part"
      m_u1r_bus2 = "m:u1r:bus2", "p.u., Positive-Sequence-Voltage, Real Part"
      m_u1i_bus1 = "m:u1i:bus1", "p.u., Positive-Sequence-Voltage, Imaginary Part"
      m_u1i_bus2 = "m:u1i:bus2", "p.u., Positive-Sequence-Voltage, Imaginary Part"
      m_u1_bus1 = "m:u1:bus1", "p.u., Positive-Sequence-Voltage, Magnitude"
      m_u1_bus2 = "m:u1:bus2", "p.u., Positive-Sequence-Voltage, Magnitude"
      m_U1_bus1 = "m:U1:bus1", "kV, Line-Ground Positive-Sequence-Voltage, Magnitude"
      m_U1_bus2 = "m:U1:bus2", "kV, Line-Ground Positive-Sequence-Voltage, Magnitude"
      m_U1l_bus1 = "m:U1l:bus1", "kV, Line-Line Positive-Sequence-Voltage, Magnitude"
      m_U1l_bus2 = "m:U1l:bus2", "kV, Line-Line Positive-Sequence-Voltage, Magnitude"
      m_phiu1_bus1 = "m:phiu1:bus1", "deg, Positive-Sequence-Voltage, Angle"
      m_phiu1_bus2 = "m:phiu1:bus2", "deg, Positive-Sequence-Voltage, Angle"
      m_u2_bus1 = "m:u2:bus1", "p.u., Negative-Sequence-Voltage, Magnitude"
      m_u2_bus2 = "m:u2:bus2", "p.u., Negative-Sequence-Voltage, Magnitude"
      m_phiu2_bus1 = "m:phiu2:bus1", "deg, Negative-Sequence-Voltage, Angle"
      m_phiu2_bus2 = "m:phiu2:bus2", "deg, Negative-Sequence-Voltage, Angle"
      m_inet_bus1_A = "m:inet:bus1:A", "p.u., Phase Current, Magnitude"
      m_inet_bus1_B = "m:inet:bus1:B", "p.u., Phase Current, Magnitude"
      m_inet_bus1_C = "m:inet:bus1:C", "p.u., Phase Current, Magnitude"
      m_inet_bus2_A = "m:inet:bus2:A", "p.u., Phase Current, Magnitude"
      m_inet_bus2_B = "m:inet:bus2:B", "p.u., Phase Current, Magnitude"
      m_inet_bus2_C = "m:inet:bus2:C", "p.u., Phase Current, Magnitude"
      m_I_bus1_A = "m:I:bus1:A", "kA, Phase Current, Magnitude"
      m_I_bus1_B = "m:I:bus1:B", "kA, Phase Current, Magnitude"
      m_I_bus1_C = "m:I:bus1:C", "kA, Phase Current, Magnitude"
      m_I_bus2_A = "m:I:bus2:A", "kA, Phase Current, Magnitude"
      m_I_bus2_B = "m:I:bus2:B", "kA, Phase Current, Magnitude"
      m_I_bus2_C = "m:I:bus2:C", "kA, Phase Current, Magnitude"
      m_phii_bus1_A = "m:phii:bus1:A", "deg, Phase Current, Angle"
      m_phii_bus1_B = "m:phii:bus1:B", "deg, Phase Current, Angle"
      m_phii_bus1_C = "m:phii:bus1:C", "deg, Phase Current, Angle"
      m_phii_bus2_A = "m:phii:bus2:A", "deg, Phase Current, Angle"
      m_phii_bus2_B = "m:phii:bus2:B", "deg, Phase Current, Angle"
      m_phii_bus2_C = "m:phii:bus2:C", "deg, Phase Current, Angle"
      m_i0r_bus1 = "m:i0r:bus1", "p.u., Zero-Sequence Current, Real Part"
      m_i0r_bus2 = "m:i0r:bus2", "p.u., Zero-Sequence Current, Real Part"
      m_i0i_bus1 = "m:i0i:bus1", "p.u., Zero-Sequence Current, Imaginary Part"
      m_i0i_bus2 = "m:i0i:bus2", "p.u., Zero-Sequence Current, Imaginary Part"
      m_i0_bus1 = "m:i0:bus1", "p.u., Zero-Sequence Current, Magnitude"
      m_i0_bus2 = "m:i0:bus2", "p.u., Zero-Sequence Current, Magnitude"
      m_I0_bus1 = "m:I0:bus1", "kA, Zero-Sequence Current, Magnitude"
      m_I0_bus2 = "m:I0:bus2", "kA, Zero-Sequence Current, Magnitude"
      m_I0x3_bus1 = "m:I0x3:bus1", "kA, 3*I0"
      m_I0x3_bus2 = "m:I0x3:bus2", "kA, 3*I0"
      m_phii0_bus1 = "m:phii0:bus1", "deg, Zero-Sequence Current, Angle"
      m_phii0_bus2 = "m:phii0:bus2", "deg, Zero-Sequence Current, Angle"
      m_i1r_bus1 = "m:i1r:bus1", "p.u., Positive-Sequence Current, Real Part"
      m_i1r_bus2 = "m:i1r:bus2", "p.u., Positive-Sequence Current, Real Part"
      m_i1i_bus1 = "m:i1i:bus1", "p.u., Positive-Sequence Current, Imaginary Part"
      m_i1i_bus2 = "m:i1i:bus2", "p.u., Positive-Sequence Current, Imaginary Part"
      m_i1_bus1 = "m:i1:bus1", "p.u., Positive-Sequence Current, Magnitude"
      m_i1_bus2 = "m:i1:bus2", "p.u., Positive-Sequence Current, Magnitude"
      m_I1_bus1 = "m:I1:bus1", "kA, Positive-Sequence Current, Magnitude"
      m_I1_bus2 = "m:I1:bus2", "kA, Positive-Sequence Current, Magnitude"
      m_phii1_bus1 = "m:phii1:bus1", "deg, Positive-Sequence Current, Angle"
      m_phii1_bus2 = "m:phii1:bus2", "deg, Positive-Sequence Current, Angle"
      m_i2r_bus1 = "m:i2r:bus1", "p.u., Negative-Sequence Current, Real Part"
      m_i2r_bus2 = "m:i2r:bus2", "p.u., Negative-Sequence Current, Real Part"
      m_i2i_bus1 = "m:i2i:bus1", "p.u., Negative-Sequence Current, Imaginary Part"
      m_i2i_bus2 = "m:i2i:bus2", "p.u., Negative-Sequence Current, Imaginary Part"
      m_i2_bus1 = "m:i2:bus1", "p.u., Negative-Sequence Current, Magnitude"
      m_i2_bus2 = "m:i2:bus2", "p.u., Negative-Sequence Current, Magnitude"
      m_I2_bus1 = "m:I2:bus1", "kA, Negative-Sequence Current, Magnitude"
      m_I2_bus2 = "m:I2:bus2", "kA, Negative-Sequence Current, Magnitude"
      m_phii2_bus1 = "m:phii2:bus1", "deg, Negative-Sequence Current, Angle"
      m_phii2_bus2 = "m:phii2:bus2", "deg, Negative-Sequence Current, Angle"
      m_P_bus1_A = "m:P:bus1:A", "MW, Active Power"
      m_P_bus1_B = "m:P:bus1:B", "MW, Active Power"
      m_P_bus1_C = "m:P:bus1:C", "MW, Active Power"
      m_P_bus2_A = "m:P:bus2:A", "MW, Active Power"
      m_P_bus2_B = "m:P:bus2:B", "MW, Active Power"
      m_P_bus2_C = "m:P:bus2:C", "MW, Active Power"
      m_Q_bus1_A = "m:Q:bus1:A", "Mvar, Reactive Power"
      m_Q_bus1_B = "m:Q:bus1:B", "Mvar, Reactive Power"
      m_Q_bus1_C = "m:Q:bus1:C", "Mvar, Reactive Power"
      m_Q_bus2_A = "m:Q:bus2:A", "Mvar, Reactive Power"
      m_Q_bus2_B = "m:Q:bus2:B", "Mvar, Reactive Power"
      m_Q_bus2_C = "m:Q:bus2:C", "Mvar, Reactive Power"
      m_S_bus1_A = "m:S:bus1:A", "MVA, Apparent Power"
      m_S_bus1_B = "m:S:bus1:B", "MVA, Apparent Power"
      m_S_bus1_C = "m:S:bus1:C", "MVA, Apparent Power"
      m_S_bus2_A = "m:S:bus2:A", "MVA, Apparent Power"
      m_S_bus2_B = "m:S:bus2:B", "MVA, Apparent Power"
      m_S_bus2_C = "m:S:bus2:C", "MVA, Apparent Power"
      m_cosphi_bus1_A = "m:cosphi:bus1:A", "Power Factor"
      m_cosphi_bus1_B = "m:cosphi:bus1:B", "Power Factor"
      m_cosphi_bus1_C = "m:cosphi:bus1:C", "Power Factor"
      m_cosphi_bus2_A = "m:cosphi:bus2:A", "Power Factor"
      m_cosphi_bus2_B = "m:cosphi:bus2:B", "Power Factor"
      m_cosphi_bus2_C = "m:cosphi:bus2:C", "Power Factor"
      m_tanphi_bus1_A = "m:tanphi:bus1:A", "tan(phi)"
      m_tanphi_bus1_B = "m:tanphi:bus1:B", "tan(phi)"
      m_tanphi_bus1_C = "m:tanphi:bus1:C", "tan(phi)"
      m_tanphi_bus2_A = "m:tanphi:bus2:A", "tan(phi)"
      m_tanphi_bus2_B = "m:tanphi:bus2:B", "tan(phi)"
      m_tanphi_bus2_C = "m:tanphi:bus2:C", "tan(phi)"
      m_Psum_bus1 = "m:Psum:bus1", "MW, Total Active Power"
      m_Psum_bus2 = "m:Psum:bus2", "MW, Total Active Power"
      m_Qsum_bus1 = "m:Qsum:bus1", "Mvar, Total Reactive Power"
      m_Qsum_bus2 = "m:Qsum:bus2", "Mvar, Total Reactive Power"
      m_Ssum_bus1 = "m:Ssum:bus1", "MVA, Total Apparent Power"
      m_Ssum_bus2 = "m:Ssum:bus2", "MVA, Total Apparent Power"
      m_cosphisum_bus1 = "m:cosphisum:bus1", "Total Power Factor"
      m_cosphisum_bus2 = "m:cosphisum:bus2", "Total Power Factor"
      m_tanphisum_bus1 = "m:tanphisum:bus1", "Total tan(phi)"
      m_tanphisum_bus2 = "m:tanphisum:bus2", "Total tan(phi)"
      m_Pln_bus1_A = "m:Pln:bus1:A", "MW, Active Power (Line-Neutral)"
      m_Pln_bus1_B = "m:Pln:bus1:B", "MW, Active Power (Line-Neutral)"
      m_Pln_bus1_C = "m:Pln:bus1:C", "MW, Active Power (Line-Neutral)"
      m_Pln_bus2_A = "m:Pln:bus2:A", "MW, Active Power (Line-Neutral)"
      m_Pln_bus2_B = "m:Pln:bus2:B", "MW, Active Power (Line-Neutral)"
      m_Pln_bus2_C = "m:Pln:bus2:C", "MW, Active Power (Line-Neutral)"
      m_Qln_bus1_A = "m:Qln:bus1:A", "Mvar, Reactive Power (Line-Neutral)"
      m_Qln_bus1_B = "m:Qln:bus1:B", "Mvar, Reactive Power (Line-Neutral)"
      m_Qln_bus1_C = "m:Qln:bus1:C", "Mvar, Reactive Power (Line-Neutral)"
      m_Qln_bus2_A = "m:Qln:bus2:A", "Mvar, Reactive Power (Line-Neutral)"
      m_Qln_bus2_B = "m:Qln:bus2:B", "Mvar, Reactive Power (Line-Neutral)"
      m_Qln_bus2_C = "m:Qln:bus2:C", "Mvar, Reactive Power (Line-Neutral)"
      m_P1_bus1 = "m:P1:bus1", "MW, Positive Sequence Active Power"
      m_P1_bus2 = "m:P1:bus2", "MW, Positive Sequence Active Power"
      m_Q1_bus1 = "m:Q1:bus1", "Mvar, Positive Sequence Reactive Power"
      m_Q1_bus2 = "m:Q1:bus2", "Mvar, Positive Sequence Reactive Power"
      m_P2_bus1 = "m:P2:bus1", "MW, Negative Sequence Active Power"
      m_P2_bus2 = "m:P2:bus2", "MW, Negative Sequence Active Power"
      m_Q2_bus1 = "m:Q2:bus1", "Mvar, Negative Sequence Reactive Power"
      m_Q2_bus2 = "m:Q2:bus2", "Mvar, Negative Sequence Reactive Power"
      m_P0_bus1 = "m:P0:bus1", "MW, Zero Sequence Active Power"
      m_P0_bus2 = "m:P0:bus2", "MW, Zero Sequence Active Power"
      m_Q0_bus1 = "m:Q0:bus1", "Mvar, Zero Sequence Reactive Power"
      m_Q0_bus2 = "m:Q0:bus2", "Mvar, Zero Sequence Reactive Power"
      m_i1P_bus1 = "m:i1P:bus1", "p.u., Positive-Sequence Active Current"
      m_i1P_bus2 = "m:i1P:bus2", "p.u., Positive-Sequence Active Current"
      m_i1Q_bus1 = "m:i1Q:bus1", "p.u., Positive-Sequence Reactive Current"
      m_i1Q_bus2 = "m:i1Q:bus2", "p.u., Positive-Sequence Reactive Current"
      m_I1P_bus1 = "m:I1P:bus1", "kA, Positive-Sequence Active Current"
      m_I1P_bus2 = "m:I1P:bus2", "kA, Positive-Sequence Active Current"
      m_I1Q_bus1 = "m:I1Q:bus1", "kA, Positive-Sequence Reactive Current"
      m_I1Q_bus2 = "m:I1Q:bus2", "kA, Positive-Sequence Reactive Current"
      m_i2P_bus1 = "m:i2P:bus1", "p.u., Negative-Sequence Active Current"
      m_i2P_bus2 = "m:i2P:bus2", "p.u., Negative-Sequence Active Current"
      m_i2Q_bus1 = "m:i2Q:bus1", "p.u., Negative-Sequence Reactive Current"
      m_i2Q_bus2 = "m:i2Q:bus2", "p.u., Negative-Sequence Reactive Current"
      m_I2P_bus1 = "m:I2P:bus1", "kA, Negative-Sequence Active Current"
      m_I2P_bus2 = "m:I2P:bus2", "kA, Negative-Sequence Active Current"
      m_I2Q_bus1 = "m:I2Q:bus1", "kA, Negative-Sequence Reactive Current"
      m_I2Q_bus2 = "m:I2Q:bus2", "kA, Negative-Sequence Reactive Current"
      m_ur_bus1_A = "m:ur:bus1:A", "p.u., Phase Voltage, Real Part"
      m_ur_bus1_B = "m:ur:bus1:B", "p.u., Phase Voltage, Real Part"
      m_ur_bus1_C = "m:ur:bus1:C", "p.u., Phase Voltage, Real Part"
      m_ur_bus2_A = "m:ur:bus2:A", "p.u., Phase Voltage, Real Part"
      m_ur_bus2_B = "m:ur:bus2:B", "p.u., Phase Voltage, Real Part"
      m_ur_bus2_C = "m:ur:bus2:C", "p.u., Phase Voltage, Real Part"
      m_ui_bus1_A = "m:ui:bus1:A", "p.u., Phase Voltage, Imaginary Part"
      m_ui_bus1_B = "m:ui:bus1:B", "p.u., Phase Voltage, Imaginary Part"
      m_ui_bus1_C = "m:ui:bus1:C", "p.u., Phase Voltage, Imaginary Part"
      m_ui_bus2_A = "m:ui:bus2:A", "p.u., Phase Voltage, Imaginary Part"
      m_ui_bus2_B = "m:ui:bus2:B", "p.u., Phase Voltage, Imaginary Part"
      m_ui_bus2_C = "m:ui:bus2:C", "p.u., Phase Voltage, Imaginary Part"
      m_u_bus1_A = "m:u:bus1:A", "p.u., Phase Voltage, Magnitude"
      m_u_bus1_B = "m:u:bus1:B", "p.u., Phase Voltage, Magnitude"
      m_u_bus1_C = "m:u:bus1:C", "p.u., Phase Voltage, Magnitude"
      m_u_bus2_A = "m:u:bus2:A", "p.u., Phase Voltage, Magnitude"
      m_u_bus2_B = "m:u:bus2:B", "p.u., Phase Voltage, Magnitude"
      m_u_bus2_C = "m:u:bus2:C", "p.u., Phase Voltage, Magnitude"
      m_ir_bus1_A = "m:ir:bus1:A", "p.u., Phase Current, Real Part"
      m_ir_bus1_B = "m:ir:bus1:B", "p.u., Phase Current, Real Part"
      m_ir_bus1_C = "m:ir:bus1:C", "p.u., Phase Current, Real Part"
      m_ir_bus2_A = "m:ir:bus2:A", "p.u., Phase Current, Real Part"
      m_ir_bus2_B = "m:ir:bus2:B", "p.u., Phase Current, Real Part"
      m_ir_bus2_C = "m:ir:bus2:C", "p.u., Phase Current, Real Part"
      m_ii_bus1_A = "m:ii:bus1:A", "p.u., Phase Current, Imaginary Part"
      m_ii_bus1_B = "m:ii:bus1:B", "p.u., Phase Current, Imaginary Part"
      m_ii_bus1_C = "m:ii:bus1:C", "p.u., Phase Current, Imaginary Part"
      m_ii_bus2_A = "m:ii:bus2:A", "p.u., Phase Current, Imaginary Part"
      m_ii_bus2_B = "m:ii:bus2:B", "p.u., Phase Current, Imaginary Part"
      m_ii_bus2_C = "m:ii:bus2:C", "p.u., Phase Current, Imaginary Part"
      m_i_bus1_A = "m:i:bus1:A", "p.u., Phase Current, Magnitude"
      m_i_bus1_B = "m:i:bus1:B", "p.u., Phase Current, Magnitude"
      m_i_bus1_C = "m:i:bus1:C", "p.u., Phase Current, Magnitude"
      m_i_bus2_A = "m:i:bus2:A", "p.u., Phase Current, Magnitude"
      m_i_bus2_B = "m:i:bus2:B", "p.u., Phase Current, Magnitude"
      m_i_bus2_C = "m:i:bus2:C", "p.u., Phase Current, Magnitude"
      m_phiui_bus1_A = "m:phiui:bus1:A", "deg, Angle between Voltage and Current"
      m_phiui_bus1_B = "m:phiui:bus1:B", "deg, Angle between Voltage and Current"
      m_phiui_bus1_C = "m:phiui:bus1:C", "deg, Angle between Voltage and Current"
      m_phiui_bus2_A = "m:phiui:bus2:A", "deg, Angle between Voltage and Current"
      m_phiui_bus2_B = "m:phiui:bus2:B", "deg, Angle between Voltage and Current"
      m_phiui_bus2_C = "m:phiui:bus2:C", "deg, Angle between Voltage and Current"
      m_phiu0i0_bus1 = "m:phiu0i0:bus1", "deg, Angle between Voltage and Current in zero-sequence system"
      m_phiu0i0_bus2 = "m:phiu0i0:bus2", "deg, Angle between Voltage and Current in zero-sequence system"
      m_phiu1i1_bus1 = "m:phiu1i1:bus1", "deg, Angle between Voltage and Current in positive sequence system"
      m_phiu1i1_bus2 = "m:phiu1i1:bus2", "deg, Angle between Voltage and Current in positive sequence system"
      m_phiu2i2_bus1 = "m:phiu2i2:bus1", "deg, Angle between Voltage and Current in negative sequence system"
      m_phiu2i2_bus2 = "m:phiu2i2:bus2", "deg, Angle between Voltage and Current in negative sequence system"
      m_Tfct_bus1 = "m:Tfct:bus1", "s, Fault Clearing Time"
      m_Tfct_bus2 = "m:Tfct:bus2", "s, Fault Clearing Time"
      m_TfctPh_bus1_A = "m:TfctPh:bus1:A", "s, Fault Clearing Time"
      m_TfctPh_bus1_B = "m:TfctPh:bus1:B", "s, Fault Clearing Time"
      m_TfctPh_bus1_C = "m:TfctPh:bus1:C", "s, Fault Clearing Time"
      m_TfctPh_bus2_A = "m:TfctPh:bus2:A", "s, Fault Clearing Time"
      m_TfctPh_bus2_B = "m:TfctPh:bus2:B", "s, Fault Clearing Time"
      m_TfctPh_bus2_C = "m:TfctPh:bus2:C", "s, Fault Clearing Time"
      m_Brkload_bus1 = "m:Brkload:bus1", "%, Max. Breaker Loading"
      m_Brkload_bus2 = "m:Brkload:bus2", "%, Max. Breaker Loading"
      m_BrkloadPh_bus1_A = "m:BrkloadPh:bus1:A", "%, Breaker Loading"
      m_BrkloadPh_bus1_B = "m:BrkloadPh:bus1:B", "%, Breaker Loading"
      m_BrkloadPh_bus1_C = "m:BrkloadPh:bus1:C", "%, Breaker Loading"
      m_BrkloadPh_bus2_A = "m:BrkloadPh:bus2:A", "%, Breaker Loading"
      m_BrkloadPh_bus2_B = "m:BrkloadPh:bus2:B", "%, Breaker Loading"
      m_BrkloadPh_bus2_C = "m:BrkloadPh:bus2:C", "%, Breaker Loading"
      n_ur_bus1_A = "n:ur:bus1:A", "p.u., Line-Ground Voltage, Real Part"
      n_ur_bus1_B = "n:ur:bus1:B", "p.u., Line-Ground Voltage, Real Part"
      n_ur_bus1_C = "n:ur:bus1:C", "p.u., Line-Ground Voltage, Real Part"
      n_ur_bus2_A = "n:ur:bus2:A", "p.u., Line-Ground Voltage, Real Part"
      n_ur_bus2_B = "n:ur:bus2:B", "p.u., Line-Ground Voltage, Real Part"
      n_ur_bus2_C = "n:ur:bus2:C", "p.u., Line-Ground Voltage, Real Part"
      n_ui_bus1_A = "n:ui:bus1:A", "p.u., Line-Ground Voltage, Imaginary Part"
      n_ui_bus1_B = "n:ui:bus1:B", "p.u., Line-Ground Voltage, Imaginary Part"
      n_ui_bus1_C = "n:ui:bus1:C", "p.u., Line-Ground Voltage, Imaginary Part"
      n_ui_bus2_A = "n:ui:bus2:A", "p.u., Line-Ground Voltage, Imaginary Part"
      n_ui_bus2_B = "n:ui:bus2:B", "p.u., Line-Ground Voltage, Imaginary Part"
      n_ui_bus2_C = "n:ui:bus2:C", "p.u., Line-Ground Voltage, Imaginary Part"
      n_u_bus1_A = "n:u:bus1:A", "p.u., Line-Ground Voltage, Magnitude"
      n_u_bus1_B = "n:u:bus1:B", "p.u., Line-Ground Voltage, Magnitude"
      n_u_bus1_C = "n:u:bus1:C", "p.u., Line-Ground Voltage, Magnitude"
      n_u_bus2_A = "n:u:bus2:A", "p.u., Line-Ground Voltage, Magnitude"
      n_u_bus2_B = "n:u:bus2:B", "p.u., Line-Ground Voltage, Magnitude"
      n_u_bus2_C = "n:u:bus2:C", "p.u., Line-Ground Voltage, Magnitude"
      n_upc_bus1_A = "n:upc:bus1:A", "%, Line-Ground Voltage, Magnitude"
      n_upc_bus1_B = "n:upc:bus1:B", "%, Line-Ground Voltage, Magnitude"
      n_upc_bus1_C = "n:upc:bus1:C", "%, Line-Ground Voltage, Magnitude"
      n_upc_bus2_A = "n:upc:bus2:A", "%, Line-Ground Voltage, Magnitude"
      n_upc_bus2_B = "n:upc:bus2:B", "%, Line-Ground Voltage, Magnitude"
      n_upc_bus2_C = "n:upc:bus2:C", "%, Line-Ground Voltage, Magnitude"
      n_U_bus1_A = "n:U:bus1:A", "kV, Line-Ground Voltage, Magnitude"
      n_U_bus1_B = "n:U:bus1:B", "kV, Line-Ground Voltage, Magnitude"
      n_U_bus1_C = "n:U:bus1:C", "kV, Line-Ground Voltage, Magnitude"
      n_U_bus2_A = "n:U:bus2:A", "kV, Line-Ground Voltage, Magnitude"
      n_U_bus2_B = "n:U:bus2:B", "kV, Line-Ground Voltage, Magnitude"
      n_U_bus2_C = "n:U:bus2:C", "kV, Line-Ground Voltage, Magnitude"
      n_phiu_bus1_A = "n:phiu:bus1:A", "deg, Line-Ground Voltage, Angle"
      n_phiu_bus1_B = "n:phiu:bus1:B", "deg, Line-Ground Voltage, Angle"
      n_phiu_bus1_C = "n:phiu:bus1:C", "deg, Line-Ground Voltage, Angle"
      n_phiu_bus2_A = "n:phiu:bus2:A", "deg, Line-Ground Voltage, Angle"
      n_phiu_bus2_B = "n:phiu:bus2:B", "deg, Line-Ground Voltage, Angle"
      n_phiu_bus2_C = "n:phiu:bus2:C", "deg, Line-Ground Voltage, Angle"
      n_phiurel_bus1 = "n:phiurel:bus1", "deg, Voltage, Relative Angle"
      n_phiurel_bus2 = "n:phiurel:bus2", "deg, Voltage, Relative Angle"
      n_ul_bus1_A = "n:ul:bus1:A", "p.u., Line-Line Voltage, Magnitude"
      n_ul_bus1_B = "n:ul:bus1:B", "p.u., Line-Line Voltage, Magnitude"
      n_ul_bus1_C = "n:ul:bus1:C", "p.u., Line-Line Voltage, Magnitude"
      n_ul_bus2_A = "n:ul:bus2:A", "p.u., Line-Line Voltage, Magnitude"
      n_ul_bus2_B = "n:ul:bus2:B", "p.u., Line-Line Voltage, Magnitude"
      n_ul_bus2_C = "n:ul:bus2:C", "p.u., Line-Line Voltage, Magnitude"
      n_ulpc_bus1_A = "n:ulpc:bus1:A", "%, Line-Line Voltage, Magnitude"
      n_ulpc_bus1_B = "n:ulpc:bus1:B", "%, Line-Line Voltage, Magnitude"
      n_ulpc_bus1_C = "n:ulpc:bus1:C", "%, Line-Line Voltage, Magnitude"
      n_ulpc_bus2_A = "n:ulpc:bus2:A", "%, Line-Line Voltage, Magnitude"
      n_ulpc_bus2_B = "n:ulpc:bus2:B", "%, Line-Line Voltage, Magnitude"
      n_ulpc_bus2_C = "n:ulpc:bus2:C", "%, Line-Line Voltage, Magnitude"
      n_Ul_bus1_A = "n:Ul:bus1:A", "kV, Line-Line Voltage, Magnitude"
      n_Ul_bus1_B = "n:Ul:bus1:B", "kV, Line-Line Voltage, Magnitude"
      n_Ul_bus1_C = "n:Ul:bus1:C", "kV, Line-Line Voltage, Magnitude"
      n_Ul_bus2_A = "n:Ul:bus2:A", "kV, Line-Line Voltage, Magnitude"
      n_Ul_bus2_B = "n:Ul:bus2:B", "kV, Line-Line Voltage, Magnitude"
      n_Ul_bus2_C = "n:Ul:bus2:C", "kV, Line-Line Voltage, Magnitude"
      n_phiul_bus1_A = "n:phiul:bus1:A", "deg, Line-Line Voltage, Angle"
      n_phiul_bus1_B = "n:phiul:bus1:B", "deg, Line-Line Voltage, Angle"
      n_phiul_bus1_C = "n:phiul:bus1:C", "deg, Line-Line Voltage, Angle"
      n_phiul_bus2_A = "n:phiul:bus2:A", "deg, Line-Line Voltage, Angle"
      n_phiul_bus2_B = "n:phiul:bus2:B", "deg, Line-Line Voltage, Angle"
      n_phiul_bus2_C = "n:phiul:bus2:C", "deg, Line-Line Voltage, Angle"
      n_u0_bus1 = "n:u0:bus1", "p.u., Zero-Sequence Voltage, Magnitude"
      n_u0_bus2 = "n:u0:bus2", "p.u., Zero-Sequence Voltage, Magnitude"
      n_U0_bus1 = "n:U0:bus1", "kV, Zero-Sequence Voltage, Magnitude"
      n_U0_bus2 = "n:U0:bus2", "kV, Zero-Sequence Voltage, Magnitude"
      n_U0x3_bus1 = "n:U0x3:bus1", "kV, 3*U0"
      n_U0x3_bus2 = "n:U0x3:bus2", "kV, 3*U0"
      n_phiu0_bus1 = "n:phiu0:bus1", "deg, Zero-Sequence Voltage, Angle"
      n_phiu0_bus2 = "n:phiu0:bus2", "deg, Zero-Sequence Voltage, Angle"
      n_u1_bus1 = "n:u1:bus1", "p.u., Positive-Sequence Voltage, Magnitude"
      n_u1_bus2 = "n:u1:bus2", "p.u., Positive-Sequence Voltage, Magnitude"
      n_u1pc_bus1 = "n:u1pc:bus1", "%, Positive-Sequence Voltage, Magnitude"
      n_u1pc_bus2 = "n:u1pc:bus2", "%, Positive-Sequence Voltage, Magnitude"
      n_u1r_bus1 = "n:u1r:bus1", "p.u., Positive-Sequence Voltage, Real Part"
      n_u1r_bus2 = "n:u1r:bus2", "p.u., Positive-Sequence Voltage, Real Part"
      n_u1i_bus1 = "n:u1i:bus1", "p.u., Positive-Sequence Voltage, Imaginary Part"
      n_u1i_bus2 = "n:u1i:bus2", "p.u., Positive-Sequence Voltage, Imaginary Part"
      n_U1_bus1 = "n:U1:bus1", "kV, Line-Ground Positive-Sequence Voltage, Magnitude"
      n_U1_bus2 = "n:U1:bus2", "kV, Line-Ground Positive-Sequence Voltage, Magnitude"
      n_phiu1_bus1 = "n:phiu1:bus1", "deg, Positive-Sequence Voltage, Angle"
      n_phiu1_bus2 = "n:phiu1:bus2", "deg, Positive-Sequence Voltage, Angle"
      n_u2_bus1 = "n:u2:bus1", "p.u., Negative-Sequence Voltage, Magnitude"
      n_u2_bus2 = "n:u2:bus2", "p.u., Negative-Sequence Voltage, Magnitude"
      n_U2_bus1 = "n:U2:bus1", "kV, Line-Ground Negative-Sequence Voltage, Magnitude"
      n_U2_bus2 = "n:U2:bus2", "kV, Line-Ground Negative-Sequence Voltage, Magnitude"
      n_phiu2_bus1 = "n:phiu2:bus1", "deg, Negative-Sequence Voltage, Angle"
      n_phiu2_bus2 = "n:phiu2:bus2", "deg, Negative-Sequence Voltage, Angle"
      n_U1l_bus1 = "n:U1l:bus1", "kV, Line-Line Positive-Sequence Voltage, Magnitude"
      n_U1l_bus2 = "n:U1l:bus2", "kV, Line-Line Positive-Sequence Voltage, Magnitude"
      n_U2l_bus1 = "n:U2l:bus1", "kV, Line-Line Negative-Sequence Voltage, Magnitude"
      n_U2l_bus2 = "n:U2l:bus2", "kV, Line-Line Negative-Sequence Voltage, Magnitude"
      n_uln_bus1_A = "n:uln:bus1:A", "p.u., Line-Neutral Voltage, Magnitude"
      n_uln_bus1_B = "n:uln:bus1:B", "p.u., Line-Neutral Voltage, Magnitude"
      n_uln_bus1_C = "n:uln:bus1:C", "p.u., Line-Neutral Voltage, Magnitude"
      n_uln_bus2_A = "n:uln:bus2:A", "p.u., Line-Neutral Voltage, Magnitude"
      n_uln_bus2_B = "n:uln:bus2:B", "p.u., Line-Neutral Voltage, Magnitude"
      n_uln_bus2_C = "n:uln:bus2:C", "p.u., Line-Neutral Voltage, Magnitude"
      n_Uln_bus1_A = "n:Uln:bus1:A", "kV, Line-Neutral Voltage, Magnitude"
      n_Uln_bus1_B = "n:Uln:bus1:B", "kV, Line-Neutral Voltage, Magnitude"
      n_Uln_bus1_C = "n:Uln:bus1:C", "kV, Line-Neutral Voltage, Magnitude"
      n_Uln_bus2_A = "n:Uln:bus2:A", "kV, Line-Neutral Voltage, Magnitude"
      n_Uln_bus2_B = "n:Uln:bus2:B", "kV, Line-Neutral Voltage, Magnitude"
      n_Uln_bus2_C = "n:Uln:bus2:C", "kV, Line-Neutral Voltage, Magnitude"
      n_phiuln_bus1_A = "n:phiuln:bus1:A", "deg, Line-Neutral Voltage, Angle"
      n_phiuln_bus1_B = "n:phiuln:bus1:B", "deg, Line-Neutral Voltage, Angle"
      n_phiuln_bus1_C = "n:phiuln:bus1:C", "deg, Line-Neutral Voltage, Angle"
      n_phiuln_bus2_A = "n:phiuln:bus2:A", "deg, Line-Neutral Voltage, Angle"
      n_phiuln_bus2_B = "n:phiuln:bus2:B", "deg, Line-Neutral Voltage, Angle"
      n_phiuln_bus2_C = "n:phiuln:bus2:C", "deg, Line-Neutral Voltage, Angle"
      n_un_bus1 = "n:un:bus1", "p.u., Neutral-Ground Voltage, Magnitude"
      n_un_bus2 = "n:un:bus2", "p.u., Neutral-Ground Voltage, Magnitude"
      n_Un_bus1 = "n:Un:bus1", "kV, Neutral-Ground Voltage, Magnitude"
      n_Un_bus2 = "n:Un:bus2", "kV, Neutral-Ground Voltage, Magnitude"
      n_phiun_bus1 = "n:phiun:bus1", "deg, Neutral-Ground Voltage, Angle"
      n_phiun_bus2 = "n:phiun:bus2", "deg, Neutral-Ground Voltage, Angle"
      n_upht_bus1_A = "n:upht:bus1:A", "p.u., Phase Technology dependent Voltage, Magnitude"
      n_upht_bus1_B = "n:upht:bus1:B", "p.u., Phase Technology dependent Voltage, Magnitude"
      n_upht_bus1_C = "n:upht:bus1:C", "p.u., Phase Technology dependent Voltage, Magnitude"
      n_upht_bus2_A = "n:upht:bus2:A", "p.u., Phase Technology dependent Voltage, Magnitude"
      n_upht_bus2_B = "n:upht:bus2:B", "p.u., Phase Technology dependent Voltage, Magnitude"
      n_upht_bus2_C = "n:upht:bus2:C", "p.u., Phase Technology dependent Voltage, Magnitude"
      n_Upht_bus1_A = "n:Upht:bus1:A", "kV, Phase Technology dependent Voltage, Magnitude"
      n_Upht_bus1_B = "n:Upht:bus1:B", "kV, Phase Technology dependent Voltage, Magnitude"
      n_Upht_bus1_C = "n:Upht:bus1:C", "kV, Phase Technology dependent Voltage, Magnitude"
      n_Upht_bus2_A = "n:Upht:bus2:A", "kV, Phase Technology dependent Voltage, Magnitude"
      n_Upht_bus2_B = "n:Upht:bus2:B", "kV, Phase Technology dependent Voltage, Magnitude"
      n_Upht_bus2_C = "n:Upht:bus2:C", "kV, Phase Technology dependent Voltage, Magnitude"
      n_um_bus1 = "n:um:bus1", "p.u., Average-Voltage, Magnitude"
      n_um_bus2 = "n:um:bus2", "p.u., Average-Voltage, Magnitude"
      n_Um_bus1 = "n:Um:bus1", "kV, Average-Voltage, Magnitude"
      n_Um_bus2 = "n:Um:bus2", "kV, Average-Voltage, Magnitude"
      n_ishcr_bus1_A = "n:ishcr:bus1:A", "p.u., Short-Circuit Current, real"
      n_ishcr_bus1_B = "n:ishcr:bus1:B", "p.u., Short-Circuit Current, real"
      n_ishcr_bus1_C = "n:ishcr:bus1:C", "p.u., Short-Circuit Current, real"
      n_ishcr_bus2_A = "n:ishcr:bus2:A", "p.u., Short-Circuit Current, real"
      n_ishcr_bus2_B = "n:ishcr:bus2:B", "p.u., Short-Circuit Current, real"
      n_ishcr_bus2_C = "n:ishcr:bus2:C", "p.u., Short-Circuit Current, real"
      n_ishci_bus1_A = "n:ishci:bus1:A", "p.u., Short-Circuit Current, imag"
      n_ishci_bus1_B = "n:ishci:bus1:B", "p.u., Short-Circuit Current, imag"
      n_ishci_bus1_C = "n:ishci:bus1:C", "p.u., Short-Circuit Current, imag"
      n_ishci_bus2_A = "n:ishci:bus2:A", "p.u., Short-Circuit Current, imag"
      n_ishci_bus2_B = "n:ishci:bus2:B", "p.u., Short-Circuit Current, imag"
      n_ishci_bus2_C = "n:ishci:bus2:C", "p.u., Short-Circuit Current, imag"
      n_ishc_bus1_A = "n:ishc:bus1:A", "p.u., Short-Circuit Current, magnitude"
      n_ishc_bus1_B = "n:ishc:bus1:B", "p.u., Short-Circuit Current, magnitude"
      n_ishc_bus1_C = "n:ishc:bus1:C", "p.u., Short-Circuit Current, magnitude"
      n_ishc_bus2_A = "n:ishc:bus2:A", "p.u., Short-Circuit Current, magnitude"
      n_ishc_bus2_B = "n:ishc:bus2:B", "p.u., Short-Circuit Current, magnitude"
      n_ishc_bus2_C = "n:ishc:bus2:C", "p.u., Short-Circuit Current, magnitude"
      n_Ishc_bus1_A = "n:Ishc:bus1:A", "kA, Short-Circuit Current, magnitude"
      n_Ishc_bus1_B = "n:Ishc:bus1:B", "kA, Short-Circuit Current, magnitude"
      n_Ishc_bus1_C = "n:Ishc:bus1:C", "kA, Short-Circuit Current, magnitude"
      n_Ishc_bus2_A = "n:Ishc:bus2:A", "kA, Short-Circuit Current, magnitude"
      n_Ishc_bus2_B = "n:Ishc:bus2:B", "kA, Short-Circuit Current, magnitude"
      n_Ishc_bus2_C = "n:Ishc:bus2:C", "kA, Short-Circuit Current, magnitude"
      n_inshc_bus1 = "n:inshc:bus1", "p.u., Neutral Short-Circuit Current, magnitude"
      n_inshc_bus2 = "n:inshc:bus2", "p.u., Neutral Short-Circuit Current, magnitude"
      n_Inshc_bus1 = "n:Inshc:bus1", "kA, Neutral Short-Circuit Current, magnitude"
      n_Inshc_bus2 = "n:Inshc:bus2", "kA, Neutral Short-Circuit Current, magnitude"
      n_fe_bus1 = "n:fe:bus1", "p.u., Electrical Frequency"
      n_fe_bus2 = "n:fe:bus2", "p.u., Electrical Frequency"
      n_frdev_bus1 = "n:frdev:bus1", "Hz, average frequency"
      n_frdev_bus2 = "n:frdev:bus2", "Hz, average frequency"
      n_dfehz_bus1 = "n:dfehz:bus1", "Hz, Deviation of the El. Frequency"
      n_dfehz_bus2 = "n:dfehz:bus2", "Hz, Deviation of the El. Frequency"
      n_dfedt_bus1 = "n:dfedt:bus1", "p.u./s, Derivative of El. Frequency"
      n_dfedt_bus2 = "n:dfedt:bus2", "p.u./s, Derivative of El. Frequency"
      n_fehz_bus1 = "n:fehz:bus1", "Hz, Electrical Frequency"
      n_fehz_bus2 = "n:fehz:bus2", "Hz, Electrical Frequency"
      n_frnom_bus1 = "n:frnom:bus1", "Hz, Nominal Frequency"
      n_frnom_bus2 = "n:frnom:bus2", "Hz, Nominal Frequency"
      c_z1_ij_r = "c:z1_ij:r", "p.u., Positive-Sequence Impedance i-j, Real Part"
      c_z1_ij_i = "c:z1_ij:i", "p.u., Positive-Sequence Impedance i-j, Imaginary Part"
      c_z1_ij_phi = "c:z1_ij:phi", "deg, Positive-Sequence Impedance i-j, Angle"
      c_z1_ij_mag = "c:z1_ij:mag", "p.u., Positive-Sequence Impedance i-j, Magnitude"
      c_z1_ij_phirad = "c:z1_ij:phirad", "rad, Positive-Sequence Impedance i-j, Angle"
      c_z1_ji_r = "c:z1_ji:r", "p.u., Positive-Sequence Impedance j-i, Real Part"
      c_z1_ji_i = "c:z1_ji:i", "p.u., Positive-Sequence Impedance j-i, Imaginary Part"
      c_z1_ji_phi = "c:z1_ji:phi", "deg, Positive-Sequence Impedance j-i, Angle"
      c_z1_ji_mag = "c:z1_ji:mag", "p.u., Positive-Sequence Impedance j-i, Magnitude"
      c_z1_ji_phirad = "c:z1_ji:phirad", "rad, Positive-Sequence Impedance j-i, Angle"
      c_z0_ij_r = "c:z0_ij:r", "p.u., Zero-Sequence Impedance i-j, Real Part"
      c_z0_ij_i = "c:z0_ij:i", "p.u., Zero-Sequence Impedance i-j, Imaginary Part"
      c_z0_ij_phi = "c:z0_ij:phi", "deg, Zero-Sequence Impedance i-j, Angle"
      c_z0_ij_mag = "c:z0_ij:mag", "p.u., Zero-Sequence Impedance i-j, Magnitude"
      c_z0_ij_phirad = "c:z0_ij:phirad", "rad, Zero-Sequence Impedance i-j, Angle"
      c_z0_ji_r = "c:z0_ji:r", "p.u., Zero-Sequence Impedance j-i, Real Part"
      c_z0_ji_i = "c:z0_ji:i", "p.u., Zero-Sequence Impedance j-i, Imaginary Part"
      c_z0_ji_phi = "c:z0_ji:phi", "deg, Zero-Sequence Impedance j-i, Angle"
      c_z0_ji_mag = "c:z0_ji:mag", "p.u., Zero-Sequence Impedance j-i, Magnitude"
      c_z0_ji_phirad = "c:z0_ji:phirad", "rad, Zero-Sequence Impedance j-i, Angle"
      c_z2_ij_r = "c:z2_ij:r", "p.u., Negative-Sequence Impedance i-j, Real Part"
      c_z2_ij_i = "c:z2_ij:i", "p.u., Negative-Sequence Impedance i-j, Imaginary Part"
      c_z2_ij_phi = "c:z2_ij:phi", "deg, Negative-Sequence Impedance i-j, Angle"
      c_z2_ij_mag = "c:z2_ij:mag", "p.u., Negative-Sequence Impedance i-j, Magnitude"
      c_z2_ij_phirad = "c:z2_ij:phirad", "rad, Negative-Sequence Impedance i-j, Angle"
      c_z2_ji_r = "c:z2_ji:r", "p.u., Negative-Sequence Impedance j-i, Real Part"
      c_z2_ji_i = "c:z2_ji:i", "p.u., Negative-Sequence Impedance j-i, Imaginary Part"
      c_z2_ji_phi = "c:z2_ji:phi", "deg, Negative-Sequence Impedance j-i, Angle"
      c_z2_ji_mag = "c:z2_ji:mag", "p.u., Negative-Sequence Impedance j-i, Magnitude"
      c_z2_ji_phirad = "c:z2_ji:phirad", "rad, Negative-Sequence Impedance j-i, Angle"
      c_y1_ij_r = "c:y1_ij:r", "p.u., Positive-Sequence Admittance i-j, Real Part"
      c_y1_ij_i = "c:y1_ij:i", "p.u., Positive-Sequence Admittance i-j, Imaginary Part"
      c_y1_ij_phi = "c:y1_ij:phi", "deg, Positive-Sequence Admittance i-j, Angle"
      c_y1_ij_mag = "c:y1_ij:mag", "p.u., Positive-Sequence Admittance i-j, Magnitude"
      c_y1_ij_phirad = "c:y1_ij:phirad", "rad, Positive-Sequence Admittance i-j, Angle"
      c_y1_ji_r = "c:y1_ji:r", "p.u., Positive-Sequence Admittance j-i, Real Part"
      c_y1_ji_i = "c:y1_ji:i", "p.u., Positive-Sequence Admittance j-i, Imaginary Part"
      c_y1_ji_phi = "c:y1_ji:phi", "deg, Positive-Sequence Admittance j-i, Angle"
      c_y1_ji_mag = "c:y1_ji:mag", "p.u., Positive-Sequence Admittance j-i, Magnitude"
      c_y1_ji_phirad = "c:y1_ji:phirad", "rad, Positive-Sequence Admittance j-i, Angle"
      c_y0_ij_r = "c:y0_ij:r", "p.u., Zero-Sequence Admittance i-j, Real Part"
      c_y0_ij_i = "c:y0_ij:i", "p.u., Zero-Sequence Admittance i-j, Imaginary Part"
      c_y0_ij_phi = "c:y0_ij:phi", "deg, Zero-Sequence Admittance i-j, Angle"
      c_y0_ij_mag = "c:y0_ij:mag", "p.u., Zero-Sequence Admittance i-j, Magnitude"
      c_y0_ij_phirad = "c:y0_ij:phirad", "rad, Zero-Sequence Admittance i-j, Angle"
      c_y0_ji_r = "c:y0_ji:r", "p.u., Zero-Sequence Admittance j-i, Real Part"
      c_y0_ji_i = "c:y0_ji:i", "p.u., Zero-Sequence Admittance j-i, Imaginary Part"
      c_y0_ji_phi = "c:y0_ji:phi", "deg, Zero-Sequence Admittance j-i, Angle"
      c_y0_ji_mag = "c:y0_ji:mag", "p.u., Zero-Sequence Admittance j-i, Magnitude"
      c_y0_ji_phirad = "c:y0_ji:phirad", "rad, Zero-Sequence Admittance j-i, Angle"
      c_y2_ij_r = "c:y2_ij:r", "p.u., Negative-Sequence Admittance i-j, Real Part"
      c_y2_ij_i = "c:y2_ij:i", "p.u., Negative-Sequence Admittance i-j, Imaginary Part"
      c_y2_ij_phi = "c:y2_ij:phi", "deg, Negative-Sequence Admittance i-j, Angle"
      c_y2_ij_mag = "c:y2_ij:mag", "p.u., Negative-Sequence Admittance i-j, Magnitude"
      c_y2_ij_phirad = "c:y2_ij:phirad", "rad, Negative-Sequence Admittance i-j, Angle"
      c_y2_ji_r = "c:y2_ji:r", "p.u., Negative-Sequence Admittance j-i, Real Part"
      c_y2_ji_i = "c:y2_ji:i", "p.u., Negative-Sequence Admittance j-i, Imaginary Part"
      c_y2_ji_phi = "c:y2_ji:phi", "deg, Negative-Sequence Admittance j-i, Angle"
      c_y2_ji_mag = "c:y2_ji:mag", "p.u., Negative-Sequence Admittance j-i, Magnitude"
      c_y2_ji_phirad = "c:y2_ji:phirad", "rad, Negative-Sequence Admittance j-i, Angle"
      c_uratio = "c:uratio", "Rating Factor"
      e_iequalz = "e:iequalz", "Use equal Impedances (zij = zji)"
      e_r_pu = "e:r_pu", "p.u., Real Part"
      e_x_pu = "e:x_pu", "p.u., Imaginary Part"
      e_r_pu_ji = "e:r_pu_ji", "p.u., Real Part"
      e_x_pu_ji = "e:x_pu_ji", "p.u., Imaginary Part"
      e_r0_pu = "e:r0_pu", "p.u., Real Part"
      e_x0_pu = "e:x0_pu", "p.u., Imaginary Part"
      e_r0_pu_ji = "e:r0_pu_ji", "p.u., Real Part"
      e_x0_pu_ji = "e:x0_pu_ji", "p.u., Imaginary Part"
      e_iz2eqz1 = "e:iz2eqz1", "Use Impedance z2 = z1"
      e_r2_pu = "e:r2_pu", "p.u., Real Part"
      e_x2_pu = "e:x2_pu", "p.u., Imaginary Part"
      e_r2_pu_ji = "e:r2_pu_ji", "p.u., Real Part"
      e_x2_pu_ji = "e:x2_pu_ji", "p.u., Imaginary Part"
      e_gi_pu = "e:gi_pu", "p.u., Conductance i"
      e_bi_pu = "e:bi_pu", "p.u., Susceptance i"
      e_gj_pu = "e:gj_pu", "p.u., Conductance j"
      e_bj_pu = "e:bj_pu", "p.u., Susceptance j"
      e_gi0_pu = "e:gi0_pu", "p.u., Conductance i"
      e_bi0_pu = "e:bi0_pu", "p.u., Susceptance i"
      e_gj0_pu = "e:gj0_pu", "p.u., Conductance j"
      e_bj0_pu = "e:bj0_pu", "p.u., Susceptance j"
      e_iy2eqy1 = "e:iy2eqy1", "Use Admittance y2 = y1"
      e_gi2_pu = "e:gi2_pu", "p.u., Conductance i"
      e_bi2_pu = "e:bi2_pu", "p.u., Susceptance i"
      e_gj2_pu = "e:gj2_pu", "p.u., Conductance j"
      e_bj2_pu = "e:bj2_pu", "p.u., Susceptance j"


  class EMT:

    class ElmArea(Enum):
      _init_ = 'value __doc__'

      c_Pgen = "c:Pgen", "MW, Generators, Active Power"
      c_Qgen = "c:Qgen", "Mvar, Generators, Reactive Power"
      c_Sgen = "c:Sgen", "MVA, Generators, Apparent Power"
      c_cosgen = "c:cosgen", "Generators, Power Factor"
      c_Pmot = "c:Pmot", "MW, Motor Loads, Active Power"
      c_Qmot = "c:Qmot", "Mvar, Motor Loads, Reactive Power"
      c_Smot = "c:Smot", "MVA, Motor Loads, Apparent Power"
      c_cosmot = "c:cosmot", "Motor Loads, Power Factor"
      c_Pload = "c:Pload", "MW, Loads, Active Power"
      c_Qload = "c:Qload", "Mvar, Loads, Reactive Power"
      c_Sload = "c:Sload", "MVA, Loads, Apparent Power"
      c_cosload = "c:cosload", "Loads, Power Factor"
      c_Pcomp = "c:Pcomp", "MW, Compensation (Losses)"
      c_Qcomp = "c:Qcomp", "Mvar, Compensation, C"
      c_Qcompr = "c:Qcompr", "Mvar, Compensation, L"
      c_Pnet = "c:Pnet", "MW, External Networks, Active Power"
      c_Qnet = "c:Qnet", "Mvar, External Networks, Reactive Power"
      c_Snet = "c:Snet", "MVA, External Networks, Apparent Power"
      c_cosnet = "c:cosnet", "External Networks, Power Factor"
      c_Pinter = "c:Pinter", "MW, Interchange Flow, Active Power"
      c_Qinter = "c:Qinter", "Mvar, Interchange Flow, Reactive Power"
      c_Ploss = "c:Ploss", "MW, Losses, Active Power"
      c_Qloss = "c:Qloss", "Mvar, Losses, Reactive Power"
      c_Pt = "c:Pt", "MW, Turbine Power"
      c_Pres = "c:Pres", "MW, Available Reserve"

    class ElmAsm(Enum):
      _init_ = 'value __doc__'

      m_u0_bus1 = "m:u0:bus1", "p.u., Zero-Sequence-Voltage"
      m_u0_bus1n = "m:u0:bus1n", "p.u., Zero-Sequence-Voltage"
      m_u1r_bus1 = "m:u1r:bus1", "p.u., Voltage Phasor, Real Part"
      m_u1r_bus1n = "m:u1r:bus1n", "p.u., Voltage Phasor, Real Part"
      m_u1i_bus1 = "m:u1i:bus1", "p.u., Voltage Phasor, Imaginary Part"
      m_u1i_bus1n = "m:u1i:bus1n", "p.u., Voltage Phasor, Imaginary Part"
      m_u1_bus1 = "m:u1:bus1", "p.u., Voltage Phasor, Magnitude"
      m_u1_bus1n = "m:u1:bus1n", "p.u., Voltage Phasor, Magnitude"
      m_U1_bus1 = "m:U1:bus1", "kV, Line-Ground Voltage Phasor, Magnitude"
      m_U1_bus1n = "m:U1:bus1n", "kV, Line-Ground Voltage Phasor, Magnitude"
      m_U1l_bus1 = "m:U1l:bus1", "kV, Line-Line Voltage Phasor, Magnitude"
      m_U1l_bus1n = "m:U1l:bus1n", "kV, Line-Line Voltage Phasor, Magnitude"
      m_phiu1_bus1 = "m:phiu1:bus1", "deg, Voltage Phasor, Angle"
      m_phiu1_bus1n = "m:phiu1:bus1n", "deg, Voltage Phasor, Angle"
      m_I_bus1_A = "m:I:bus1:A", "kA, Phase Current"
      m_I_bus1_B = "m:I:bus1:B", "kA, Phase Current"
      m_I_bus1_C = "m:I:bus1:C", "kA, Phase Current"
      m_I_bus1_N = "m:I:bus1:N", "kA, Phase Current"
      m_I_bus1n = "m:I:bus1n", "kA, Phase Current"
      m_i0_bus1 = "m:i0:bus1", "p.u., Zero-Sequence Current"
      m_i0_bus1n = "m:i0:bus1n", "p.u., Zero-Sequence Current"
      m_I0_bus1 = "m:I0:bus1", "kA, Zero-Sequence Current"
      m_I0_bus1n = "m:I0:bus1n", "kA, Zero-Sequence Current"
      m_I0x3_bus1 = "m:I0x3:bus1", "kA, 3*I0"
      m_I0x3_bus1n = "m:I0x3:bus1n", "kA, 3*I0"
      m_i1r_bus1 = "m:i1r:bus1", "p.u., Current Phasor, Real Part"
      m_i1r_bus1n = "m:i1r:bus1n", "p.u., Current Phasor, Real Part"
      m_i1i_bus1 = "m:i1i:bus1", "p.u., Current Phasor, Imaginary Part"
      m_i1i_bus1n = "m:i1i:bus1n", "p.u., Current Phasor, Imaginary Part"
      m_i1_bus1 = "m:i1:bus1", "p.u., Current Phasor, Magnitude"
      m_i1_bus1n = "m:i1:bus1n", "p.u., Current Phasor, Magnitude"
      m_I1_bus1 = "m:I1:bus1", "kA, Current Phasor, Magnitude"
      m_I1_bus1n = "m:I1:bus1n", "kA, Current Phasor, Magnitude"
      m_phii1_bus1 = "m:phii1:bus1", "deg, Current Phasor, Angle"
      m_phii1_bus1n = "m:phii1:bus1n", "deg, Current Phasor, Angle"
      m_P_bus1_A = "m:P:bus1:A", "MW, Active Power"
      m_P_bus1_B = "m:P:bus1:B", "MW, Active Power"
      m_P_bus1_C = "m:P:bus1:C", "MW, Active Power"
      m_P_bus1_N = "m:P:bus1:N", "MW, Active Power"
      m_P_bus1n = "m:P:bus1n", "MW, Active Power"
      m_Psum_bus1 = "m:Psum:bus1", "MW, Power-Phasor, Active Power"
      m_Psum_bus1n = "m:Psum:bus1n", "MW, Power-Phasor, Active Power"
      m_Qsum_bus1 = "m:Qsum:bus1", "Mvar, Power Phasor, Reactive Part"
      m_Qsum_bus1n = "m:Qsum:bus1n", "Mvar, Power Phasor, Reactive Part"
      m_Ssum_bus1 = "m:Ssum:bus1", "MVA, Power Phasor, Apparent Power"
      m_Ssum_bus1n = "m:Ssum:bus1n", "MVA, Power Phasor, Apparent Power"
      m_cosphi1_bus1 = "m:cosphi1:bus1", "Power-Phasor, Power-Factor"
      m_cosphi1_bus1n = "m:cosphi1:bus1n", "Power-Phasor, Power-Factor"
      m_i1P_bus1 = "m:i1P:bus1", "p.u., Current Phasor, Active Current"
      m_i1P_bus1n = "m:i1P:bus1n", "p.u., Current Phasor, Active Current"
      m_i1Q_bus1 = "m:i1Q:bus1", "p.u., Current Phasor, Reactive Current"
      m_i1Q_bus1n = "m:i1Q:bus1n", "p.u., Current Phasor, Reactive Current"
      m_I1P_bus1 = "m:I1P:bus1", "kA, Current Phasor, Active Current"
      m_I1P_bus1n = "m:I1P:bus1n", "kA, Current Phasor, Active Current"
      m_I1Q_bus1 = "m:I1Q:bus1", "kA, Current Phasor, Reactive Current"
      m_I1Q_bus1n = "m:I1Q:bus1n", "kA, Current Phasor, Reactive Current"
      m_u_bus1_A = "m:u:bus1:A", "p.u., Phase Voltage"
      m_u_bus1_B = "m:u:bus1:B", "p.u., Phase Voltage"
      m_u_bus1_C = "m:u:bus1:C", "p.u., Phase Voltage"
      m_u_bus1_N = "m:u:bus1:N", "p.u., Phase Voltage"
      m_u_bus1n = "m:u:bus1n", "p.u., Phase Voltage"
      m_i_bus1_A = "m:i:bus1:A", "p.u., Phase Current"
      m_i_bus1_B = "m:i:bus1:B", "p.u., Phase Current"
      m_i_bus1_C = "m:i:bus1:C", "p.u., Phase Current"
      m_i_bus1_N = "m:i:bus1:N", "p.u., Phase Current"
      m_i_bus1n = "m:i:bus1n", "p.u., Phase Current"
      m_phiu1i1_bus1 = "m:phiu1i1:bus1", "deg, Angle between Voltage and Current Phasor"
      m_phiu1i1_bus1n = "m:phiu1i1:bus1n", "deg, Angle between Voltage and Current Phasor"
      m_Tfct_bus1 = "m:Tfct:bus1", "s, Fault Clearing Time"
      m_Tfct_bus1n = "m:Tfct:bus1n", "s, Fault Clearing Time"
      m_TfctPh_bus1_A = "m:TfctPh:bus1:A", "s, Fault Clearing Time"
      m_TfctPh_bus1_B = "m:TfctPh:bus1:B", "s, Fault Clearing Time"
      m_TfctPh_bus1_C = "m:TfctPh:bus1:C", "s, Fault Clearing Time"
      m_TfctPh_bus1_N = "m:TfctPh:bus1:N", "s, Fault Clearing Time"
      m_TfctPh_bus1n = "m:TfctPh:bus1n", "s, Fault Clearing Time"
      n_u_bus1_A = "n:u:bus1:A", "p.u., Phase Voltage"
      n_u_bus1_B = "n:u:bus1:B", "p.u., Phase Voltage"
      n_u_bus1_C = "n:u:bus1:C", "p.u., Phase Voltage"
      n_u_bus1_N = "n:u:bus1:N", "p.u., Phase Voltage"
      n_u_bus1n = "n:u:bus1n", "p.u., Phase Voltage"
      n_upc_bus1_A = "n:upc:bus1:A", "%, Phase Voltage"
      n_upc_bus1_B = "n:upc:bus1:B", "%, Phase Voltage"
      n_upc_bus1_C = "n:upc:bus1:C", "%, Phase Voltage"
      n_upc_bus1_N = "n:upc:bus1:N", "%, Phase Voltage"
      n_upc_bus1n = "n:upc:bus1n", "%, Phase Voltage"
      n_U_bus1_A = "n:U:bus1:A", "kV, Phase Voltage"
      n_U_bus1_B = "n:U:bus1:B", "kV, Phase Voltage"
      n_U_bus1_C = "n:U:bus1:C", "kV, Phase Voltage"
      n_U_bus1_N = "n:U:bus1:N", "kV, Phase Voltage"
      n_U_bus1n = "n:U:bus1n", "kV, Phase Voltage"
      n_ul_bus1_A = "n:ul:bus1:A", "p.u., Line-Line Voltage"
      n_ul_bus1_B = "n:ul:bus1:B", "p.u., Line-Line Voltage"
      n_ul_bus1_C = "n:ul:bus1:C", "p.u., Line-Line Voltage"
      n_ul_bus1_N = "n:ul:bus1:N", "p.u., Line-Line Voltage"
      n_ul_bus1n = "n:ul:bus1n", "p.u., Line-Line Voltage"
      n_ulpc_bus1_A = "n:ulpc:bus1:A", "%, Line-Line Voltage"
      n_ulpc_bus1_B = "n:ulpc:bus1:B", "%, Line-Line Voltage"
      n_ulpc_bus1_C = "n:ulpc:bus1:C", "%, Line-Line Voltage"
      n_ulpc_bus1_N = "n:ulpc:bus1:N", "%, Line-Line Voltage"
      n_ulpc_bus1n = "n:ulpc:bus1n", "%, Line-Line Voltage"
      n_Ul_bus1_A = "n:Ul:bus1:A", "kV, Line-Line Voltage"
      n_Ul_bus1_B = "n:Ul:bus1:B", "kV, Line-Line Voltage"
      n_Ul_bus1_C = "n:Ul:bus1:C", "kV, Line-Line Voltage"
      n_Ul_bus1_N = "n:Ul:bus1:N", "kV, Line-Line Voltage"
      n_Ul_bus1n = "n:Ul:bus1n", "kV, Line-Line Voltage"
      n_u0_bus1 = "n:u0:bus1", "p.u., Zero-Sequence-Voltage"
      n_u0_bus1n = "n:u0:bus1n", "p.u., Zero-Sequence-Voltage"
      n_U0_bus1 = "n:U0:bus1", "kV, Zero-Sequence-Voltage"
      n_U0_bus1n = "n:U0:bus1n", "kV, Zero-Sequence-Voltage"
      n_U0x3_bus1 = "n:U0x3:bus1", "kV, 3*U0"
      n_U0x3_bus1n = "n:U0x3:bus1n", "kV, 3*U0"
      n_u1r_bus1 = "n:u1r:bus1", "p.u., Voltage Phasor, Real Part"
      n_u1r_bus1n = "n:u1r:bus1n", "p.u., Voltage Phasor, Real Part"
      n_u1i_bus1 = "n:u1i:bus1", "p.u., Voltage Phasor, Imaginary Part"
      n_u1i_bus1n = "n:u1i:bus1n", "p.u., Voltage Phasor, Imaginary Part"
      n_u1_bus1 = "n:u1:bus1", "p.u., Voltage Phasor, Magnitude"
      n_u1_bus1n = "n:u1:bus1n", "p.u., Voltage Phasor, Magnitude"
      n_u1pc_bus1 = "n:u1pc:bus1", "%, Voltage Phasor, Magnitude"
      n_u1pc_bus1n = "n:u1pc:bus1n", "%, Voltage Phasor, Magnitude"
      n_U1_bus1 = "n:U1:bus1", "kV, Voltage Phasor, Magnitude"
      n_U1_bus1n = "n:U1:bus1n", "kV, Voltage Phasor, Magnitude"
      n_phiu1_bus1 = "n:phiu1:bus1", "deg, Voltage Phasor, Angle"
      n_phiu1_bus1n = "n:phiu1:bus1n", "deg, Voltage Phasor, Angle"
      n_U1l_bus1 = "n:U1l:bus1", "kV, Line-Line Voltage Phasor, Magnitude"
      n_U1l_bus1n = "n:U1l:bus1n", "kV, Line-Line Voltage Phasor, Magnitude"
      n_uln_bus1_A = "n:uln:bus1:A", "p.u., Line-Neutral Voltage"
      n_uln_bus1_B = "n:uln:bus1:B", "p.u., Line-Neutral Voltage"
      n_uln_bus1_C = "n:uln:bus1:C", "p.u., Line-Neutral Voltage"
      n_uln_bus1_N = "n:uln:bus1:N", "p.u., Line-Neutral Voltage"
      n_uln_bus1n = "n:uln:bus1n", "p.u., Line-Neutral Voltage"
      n_Uln_bus1_A = "n:Uln:bus1:A", "kV, Line-Neutral Voltage"
      n_Uln_bus1_B = "n:Uln:bus1:B", "kV, Line-Neutral Voltage"
      n_Uln_bus1_C = "n:Uln:bus1:C", "kV, Line-Neutral Voltage"
      n_Uln_bus1_N = "n:Uln:bus1:N", "kV, Line-Neutral Voltage"
      n_Uln_bus1n = "n:Uln:bus1n", "kV, Line-Neutral Voltage"
      n_un_bus1 = "n:un:bus1", "p.u., Neutral-Ground Voltage"
      n_un_bus1n = "n:un:bus1n", "p.u., Neutral-Ground Voltage"
      n_Un_bus1 = "n:Un:bus1", "kV, Neutral-Ground Voltage"
      n_Un_bus1n = "n:Un:bus1n", "kV, Neutral-Ground Voltage"
      n_ishc_bus1_A = "n:ishc:bus1:A", "p.u., Short-Circuit Current"
      n_ishc_bus1_B = "n:ishc:bus1:B", "p.u., Short-Circuit Current"
      n_ishc_bus1_C = "n:ishc:bus1:C", "p.u., Short-Circuit Current"
      n_ishc_bus1_N = "n:ishc:bus1:N", "p.u., Short-Circuit Current"
      n_ishc_bus1n = "n:ishc:bus1n", "p.u., Short-Circuit Current"
      n_Ishc_bus1_A = "n:Ishc:bus1:A", "kA, Short-Circuit Current"
      n_Ishc_bus1_B = "n:Ishc:bus1:B", "kA, Short-Circuit Current"
      n_Ishc_bus1_C = "n:Ishc:bus1:C", "kA, Short-Circuit Current"
      n_Ishc_bus1_N = "n:Ishc:bus1:N", "kA, Short-Circuit Current"
      n_Ishc_bus1n = "n:Ishc:bus1n", "kA, Short-Circuit Current"
      n_inshc_bus1 = "n:inshc:bus1", "p.u., Neutral Short-Circuit Current, magnitude"
      n_inshc_bus1n = "n:inshc:bus1n", "p.u., Neutral Short-Circuit Current, magnitude"
      n_Inshc_bus1 = "n:Inshc:bus1", "kA, Neutral Short-Circuit Current, magnitude"
      n_Inshc_bus1n = "n:Inshc:bus1n", "kA, Neutral Short-Circuit Current, magnitude"
      n_fe_bus1 = "n:fe:bus1", "p.u., Electrical Frequency"
      n_fe_bus1n = "n:fe:bus1n", "p.u., Electrical Frequency"
      n_frdev_bus1 = "n:frdev:bus1", "Hz, average frequency"
      n_frdev_bus1n = "n:frdev:bus1n", "Hz, average frequency"
      n_dfehz_bus1 = "n:dfehz:bus1", "Hz, Deviation of the El. Frequency"
      n_dfehz_bus1n = "n:dfehz:bus1n", "Hz, Deviation of the El. Frequency"
      n_fehz_bus1 = "n:fehz:bus1", "Hz, Electrical Frequency"
      n_fehz_bus1n = "n:fehz:bus1n", "Hz, Electrical Frequency"
      n_frnom_bus1 = "n:frnom:bus1", "Hz, Nominal Frequency"
      n_frnom_bus1n = "n:frnom:bus1n", "Hz, Nominal Frequency"
      s_pt = "s:pt", "p.u., IN, Turbine Power, continuous, double"
      s_xmdm = "s:xmdm", "p.u., IN, MDM-Torque, continuous, double"
      s_rradd = "s:rradd", "p.u., IN, Additional Rotor Resistance, continuous, double"
      s_phi = "s:phi", "rad, OUT, Angle, continuous, double"
      s_xspeed = "s:xspeed", "p.u., OUT, Speed, continuous, double"
      s_pgt = "s:pgt", "p.u., OUT, Active power, continuous, double"
      s_xme = "s:xme", "p.u., OUT, Electrical Torque, continuous, double"
      s_irot = "s:irot", "p.u., OUT, Rotor Current, continuous, double"
      s_speed = "s:speed", "p.u., STATE, Speed, continuous, double"
      s_psiA1_r = "s:psiA1_r", "p.u., STATE, Flux, Loop A1, Real Part, continuous, double"
      s_psiA1_i = "s:psiA1_i", "p.u., STATE, Flux, Loop A1, Imaginary Part, continuous, double"
      s_psiA2_r = "s:psiA2_r", "p.u., STATE, Flux, Loop A2, Real Part, continuous, double"
      s_psiA2_i = "s:psiA2_i", "p.u., STATE, Flux, Loop A2, Imaginary Part, continuous, double"
      s_psiB_r = "s:psiB_r", "p.u., STATE, Flux, Loop B, Real Part continuous, double"
      s_psiB_i = "s:psiB_i", "p.u., STATE, Flux, Loop B, Imaginary Part, continuous, double"
      s_speed_dt = "s:speed:dt", "p.u./s, d/dt, Speed, derivative, continuous, double"
      s_psiA1_r_dt = "s:psiA1_r:dt", "p.u./s, d/dt, Flux, Loop A1, Real Part, derivative, continuous, double"
      s_psiA1_i_dt = "s:psiA1_i:dt", "p.u./s, d/dt, Flux, Loop A1, Imaginary Part, derivative, continuous, double"
      s_psiA2_r_dt = "s:psiA2_r:dt", "p.u./s, d/dt, Flux, Loop A2, Real Part, derivative, continuous, double"
      s_psiA2_i_dt = "s:psiA2_i:dt", "p.u./s, d/dt, Flux, Loop A2, Imaginary Part, derivative, continuous, double"
      s_psiB_r_dt = "s:psiB_r:dt", "p.u./s, d/dt, Flux, Loop B, Real Part, derivative, continuous, double"
      s_psiB_i_dt = "s:psiB_i:dt", "p.u./s, d/dt, Flux, Loop B, Imaginary Part, derivative, continuous, double"
      c_cosn = "c:cosn", "Rated Power Factor"
      c_p = "c:p", "MW, Active Power (Electrical)"
      c_rstr = "c:rstr", "p.u., Stator Resistance"
      c_xstr = "c:xstr", "p.u., Stator Reactance"
      c_rrtrA = "c:rrtrA", "p.u., Rotor Resistance A"
      c_rrtrB = "c:rrtrB", "p.u., Rotor Resistance B"
      c_xrtrA = "c:xrtrA", "p.u., Rotor Reactance A"
      c_xrtrB = "c:xrtrB", "p.u., Rotor Reactance B"
      c_xm = "c:xm", "p.u., Magnetising Reactance"
      c_xmrtr = "c:xmrtr", "p.u., Rotor Leakage Reactance"
      c_frnom = "c:frnom", "Hz, Nominal Frequency"
      c_i_mot = "c:i_mot", "i_mot=0"
      c_xmem = "c:xmem", "p.u., Electrical Torque"
      c_xmt = "c:xmt", "Mechanical Torque"
      c_slip = "c:slip", "%, Slip"
      c_tag = "c:tag", "s, Acceleration Time Constant"
      c_i_star = "c:i_star", "Star/Delta"
      c_i_auto = "c:i_auto", "Auto Transformer"
      c_xss = "c:xss", "p.u., Subtransient Reactance"
      c_addmt = "c:addmt", "p.u., Additional Torque"
      c_ccomp = "c:ccomp", "p.u., Capacitor for Compensation"
      c_xradd = "c:xradd", "p.u., Additional Rotor Reactance"
      c_xstadd = "c:xstadd", "p.u., Additional Stator Reactance"
      c_iTrigg = "c:iTrigg", "Triggered by"
      c_Tyd = "c:Tyd", "s, Y - D switching time"
      c_speedyd = "c:speedyd", "p.u., Y - D switching speed"
      c_xreac = "c:xreac", "p.u., Reactor Reactance"
      c_Tbypass = "c:Tbypass", "p.u., Bypass time"
      c_speedbyp = "c:speedbyp", "p.u., Bypass speed"
      c_autotap = "c:autotap", "p.u., Auto Transformer Tap"
      c_Tstaroff = "c:Tstaroff", "p.u., Star contactor off time"
      c_speedsoff = "c:speedsoff", "p.u., Star contactor off speed"
      c_psiM = "c:psiM", "p.u., Magnetising flux"
      c_ksat = "c:ksat", "Saturation factor"
      c_xmsat = "c:xmsat", "p.u., Saturated magnetising reactance"
      e_c_pmdm = "e:c_pmdm", "Mdm"
      e_mdmlp = "e:mdmlp", "p.u., Proportional factor"
      e_mdmex = "e:mdmex", "Exponent"
      e_Jme = "e:Jme", "kgm^2, Moment of Inertia"
      e_gratio = "e:gratio", "Gear ratio"
      e_cJ = "e:cJ", "kgm^2, Moment of Inertia from Type"
      e_cJtot = "e:cJtot", "kgm^2, Total Moment of Inertia"
      e_ctagtot = "e:ctagtot", "s, Total Acceleration Time Const."
      e_iUseStart = "e:iUseStart", "Use motor starting method"
      e_iStartMethod = "e:iStartMethod", "Motor starting method"
      e_iTrigg = "e:iTrigg", "Triggered by..."
      e_Tyd = "e:Tyd", "s, Switch to 'D' after"
      e_speedyd = "e:speedyd", "p.u., Switch to 'D' at speed >="
      e_ctime = "e:ctime", "s, Time"
      e_crotor = "e:crotor", "p.u., Additional rotor resistance"
      e_cspeed = "e:cspeed", "p.u., Speed"
      e_ratedStr = "e:ratedStr", "kVA, Rated apparent power"
      e_xrea = "e:xrea", "%, Reactance"
      e_Tbypass = "e:Tbypass", "s, Bypass after"
      e_speedbyp = "e:speedbyp", "p.u., Bypass at speed >="
      e_autotap = "e:autotap", "%, Tap"
      e_Tstaroff = "e:Tstaroff", "s, Release star contactor after"
      e_speedsoff = "e:speedsoff", "p.u., Release star contactor at speed >="
      e_speed1ph = "e:speed1ph", "p.u., Switch at speed >="
      e_T1ph = "e:T1ph", "s, Switch after"
      r_c_pmdm = "r:c_pmdm", "Mdm"

    class ElmAsmsc(Enum):
      _init_ = 'value __doc__'

      m_u0_bus1 = "m:u0:bus1", "p.u., Zero-Sequence-Voltage"
      m_u0_bus2 = "m:u0:bus2", "p.u., Zero-Sequence-Voltage"
      m_u1r_bus1 = "m:u1r:bus1", "p.u., Voltage Phasor, Real Part"
      m_u1r_bus2 = "m:u1r:bus2", "p.u., Voltage Phasor, Real Part"
      m_u1i_bus1 = "m:u1i:bus1", "p.u., Voltage Phasor, Imaginary Part"
      m_u1i_bus2 = "m:u1i:bus2", "p.u., Voltage Phasor, Imaginary Part"
      m_u1_bus1 = "m:u1:bus1", "p.u., Voltage Phasor, Magnitude"
      m_u1_bus2 = "m:u1:bus2", "p.u., Voltage Phasor, Magnitude"
      m_U1_bus1 = "m:U1:bus1", "kV, Line-Ground Voltage Phasor, Magnitude"
      m_U1_bus2 = "m:U1:bus2", "kV, Line-Ground Voltage Phasor, Magnitude"
      m_U1l_bus1 = "m:U1l:bus1", "kV, Line-Line Voltage Phasor, Magnitude"
      m_U1l_bus2 = "m:U1l:bus2", "kV, Line-Line Voltage Phasor, Magnitude"
      m_phiu1_bus1 = "m:phiu1:bus1", "deg, Voltage Phasor, Angle"
      m_phiu1_bus2 = "m:phiu1:bus2", "deg, Voltage Phasor, Angle"
      m_I_bus1_A = "m:I:bus1:A", "kA, Phase Current"
      m_I_bus1_B = "m:I:bus1:B", "kA, Phase Current"
      m_I_bus1_C = "m:I:bus1:C", "kA, Phase Current"
      m_I_bus2 = "m:I:bus2", "kA, Phase Current"
      m_i0_bus1 = "m:i0:bus1", "p.u., Zero-Sequence Current"
      m_i0_bus2 = "m:i0:bus2", "p.u., Zero-Sequence Current"
      m_I0_bus1 = "m:I0:bus1", "kA, Zero-Sequence Current"
      m_I0_bus2 = "m:I0:bus2", "kA, Zero-Sequence Current"
      m_I0x3_bus1 = "m:I0x3:bus1", "kA, 3*I0"
      m_I0x3_bus2 = "m:I0x3:bus2", "kA, 3*I0"
      m_i1r_bus1 = "m:i1r:bus1", "p.u., Current Phasor, Real Part"
      m_i1r_bus2 = "m:i1r:bus2", "p.u., Current Phasor, Real Part"
      m_i1i_bus1 = "m:i1i:bus1", "p.u., Current Phasor, Imaginary Part"
      m_i1i_bus2 = "m:i1i:bus2", "p.u., Current Phasor, Imaginary Part"
      m_i1_bus1 = "m:i1:bus1", "p.u., Current Phasor, Magnitude"
      m_i1_bus2 = "m:i1:bus2", "p.u., Current Phasor, Magnitude"
      m_I1_bus1 = "m:I1:bus1", "kA, Current Phasor, Magnitude"
      m_I1_bus2 = "m:I1:bus2", "kA, Current Phasor, Magnitude"
      m_phii1_bus1 = "m:phii1:bus1", "deg, Current Phasor, Angle"
      m_phii1_bus2 = "m:phii1:bus2", "deg, Current Phasor, Angle"
      m_P_bus1_A = "m:P:bus1:A", "MW, Active Power"
      m_P_bus1_B = "m:P:bus1:B", "MW, Active Power"
      m_P_bus1_C = "m:P:bus1:C", "MW, Active Power"
      m_P_bus2 = "m:P:bus2", "MW, Active Power"
      m_Psum_bus1 = "m:Psum:bus1", "MW, Power-Phasor, Active Power"
      m_Psum_bus2 = "m:Psum:bus2", "MW, Power-Phasor, Active Power"
      m_Qsum_bus1 = "m:Qsum:bus1", "Mvar, Power Phasor, Reactive Part"
      m_Qsum_bus2 = "m:Qsum:bus2", "Mvar, Power Phasor, Reactive Part"
      m_Ssum_bus1 = "m:Ssum:bus1", "MVA, Power Phasor, Apparent Power"
      m_Ssum_bus2 = "m:Ssum:bus2", "MVA, Power Phasor, Apparent Power"
      m_cosphi1_bus1 = "m:cosphi1:bus1", "Power-Phasor, Power-Factor"
      m_cosphi1_bus2 = "m:cosphi1:bus2", "Power-Phasor, Power-Factor"
      m_i1P_bus1 = "m:i1P:bus1", "p.u., Current Phasor, Active Current"
      m_i1P_bus2 = "m:i1P:bus2", "p.u., Current Phasor, Active Current"
      m_i1Q_bus1 = "m:i1Q:bus1", "p.u., Current Phasor, Reactive Current"
      m_i1Q_bus2 = "m:i1Q:bus2", "p.u., Current Phasor, Reactive Current"
      m_I1P_bus1 = "m:I1P:bus1", "kA, Current Phasor, Active Current"
      m_I1P_bus2 = "m:I1P:bus2", "kA, Current Phasor, Active Current"
      m_I1Q_bus1 = "m:I1Q:bus1", "kA, Current Phasor, Reactive Current"
      m_I1Q_bus2 = "m:I1Q:bus2", "kA, Current Phasor, Reactive Current"
      m_u_bus1_A = "m:u:bus1:A", "p.u., Phase Voltage"
      m_u_bus1_B = "m:u:bus1:B", "p.u., Phase Voltage"
      m_u_bus1_C = "m:u:bus1:C", "p.u., Phase Voltage"
      m_u_bus2 = "m:u:bus2", "p.u., Phase Voltage"
      m_i_bus1_A = "m:i:bus1:A", "p.u., Phase Current"
      m_i_bus1_B = "m:i:bus1:B", "p.u., Phase Current"
      m_i_bus1_C = "m:i:bus1:C", "p.u., Phase Current"
      m_i_bus2 = "m:i:bus2", "p.u., Phase Current"
      m_phiu1i1_bus1 = "m:phiu1i1:bus1", "deg, Angle between Voltage and Current Phasor"
      m_phiu1i1_bus2 = "m:phiu1i1:bus2", "deg, Angle between Voltage and Current Phasor"
      m_Tfct_bus1 = "m:Tfct:bus1", "s, Fault Clearing Time"
      m_Tfct_bus2 = "m:Tfct:bus2", "s, Fault Clearing Time"
      m_TfctPh_bus1_A = "m:TfctPh:bus1:A", "s, Fault Clearing Time"
      m_TfctPh_bus1_B = "m:TfctPh:bus1:B", "s, Fault Clearing Time"
      m_TfctPh_bus1_C = "m:TfctPh:bus1:C", "s, Fault Clearing Time"
      m_TfctPh_bus2 = "m:TfctPh:bus2", "s, Fault Clearing Time"
      n_u_bus1_A = "n:u:bus1:A", "p.u., Phase Voltage"
      n_u_bus1_B = "n:u:bus1:B", "p.u., Phase Voltage"
      n_u_bus1_C = "n:u:bus1:C", "p.u., Phase Voltage"
      n_u_bus2 = "n:u:bus2", "p.u., Phase Voltage"
      n_upc_bus1_A = "n:upc:bus1:A", "%, Phase Voltage"
      n_upc_bus1_B = "n:upc:bus1:B", "%, Phase Voltage"
      n_upc_bus1_C = "n:upc:bus1:C", "%, Phase Voltage"
      n_upc_bus2 = "n:upc:bus2", "%, Phase Voltage"
      n_U_bus1_A = "n:U:bus1:A", "kV, Phase Voltage"
      n_U_bus1_B = "n:U:bus1:B", "kV, Phase Voltage"
      n_U_bus1_C = "n:U:bus1:C", "kV, Phase Voltage"
      n_U_bus2 = "n:U:bus2", "kV, Phase Voltage"
      n_ul_bus1_A = "n:ul:bus1:A", "p.u., Line-Line Voltage"
      n_ul_bus1_B = "n:ul:bus1:B", "p.u., Line-Line Voltage"
      n_ul_bus1_C = "n:ul:bus1:C", "p.u., Line-Line Voltage"
      n_ul_bus2 = "n:ul:bus2", "p.u., Line-Line Voltage"
      n_ulpc_bus1_A = "n:ulpc:bus1:A", "%, Line-Line Voltage"
      n_ulpc_bus1_B = "n:ulpc:bus1:B", "%, Line-Line Voltage"
      n_ulpc_bus1_C = "n:ulpc:bus1:C", "%, Line-Line Voltage"
      n_ulpc_bus2 = "n:ulpc:bus2", "%, Line-Line Voltage"
      n_Ul_bus1_A = "n:Ul:bus1:A", "kV, Line-Line Voltage"
      n_Ul_bus1_B = "n:Ul:bus1:B", "kV, Line-Line Voltage"
      n_Ul_bus1_C = "n:Ul:bus1:C", "kV, Line-Line Voltage"
      n_Ul_bus2 = "n:Ul:bus2", "kV, Line-Line Voltage"
      n_u0_bus1 = "n:u0:bus1", "p.u., Zero-Sequence-Voltage"
      n_u0_bus2 = "n:u0:bus2", "p.u., Zero-Sequence-Voltage"
      n_U0_bus1 = "n:U0:bus1", "kV, Zero-Sequence-Voltage"
      n_U0_bus2 = "n:U0:bus2", "kV, Zero-Sequence-Voltage"
      n_U0x3_bus1 = "n:U0x3:bus1", "kV, 3*U0"
      n_U0x3_bus2 = "n:U0x3:bus2", "kV, 3*U0"
      n_u1r_bus1 = "n:u1r:bus1", "p.u., Voltage Phasor, Real Part"
      n_u1r_bus2 = "n:u1r:bus2", "p.u., Voltage Phasor, Real Part"
      n_u1i_bus1 = "n:u1i:bus1", "p.u., Voltage Phasor, Imaginary Part"
      n_u1i_bus2 = "n:u1i:bus2", "p.u., Voltage Phasor, Imaginary Part"
      n_u1_bus1 = "n:u1:bus1", "p.u., Voltage Phasor, Magnitude"
      n_u1_bus2 = "n:u1:bus2", "p.u., Voltage Phasor, Magnitude"
      n_u1pc_bus1 = "n:u1pc:bus1", "%, Voltage Phasor, Magnitude"
      n_u1pc_bus2 = "n:u1pc:bus2", "%, Voltage Phasor, Magnitude"
      n_U1_bus1 = "n:U1:bus1", "kV, Voltage Phasor, Magnitude"
      n_U1_bus2 = "n:U1:bus2", "kV, Voltage Phasor, Magnitude"
      n_phiu1_bus1 = "n:phiu1:bus1", "deg, Voltage Phasor, Angle"
      n_phiu1_bus2 = "n:phiu1:bus2", "deg, Voltage Phasor, Angle"
      n_U1l_bus1 = "n:U1l:bus1", "kV, Line-Line Voltage Phasor, Magnitude"
      n_U1l_bus2 = "n:U1l:bus2", "kV, Line-Line Voltage Phasor, Magnitude"
      n_uln_bus1_A = "n:uln:bus1:A", "p.u., Line-Neutral Voltage"
      n_uln_bus1_B = "n:uln:bus1:B", "p.u., Line-Neutral Voltage"
      n_uln_bus1_C = "n:uln:bus1:C", "p.u., Line-Neutral Voltage"
      n_uln_bus2 = "n:uln:bus2", "p.u., Line-Neutral Voltage"
      n_Uln_bus1_A = "n:Uln:bus1:A", "kV, Line-Neutral Voltage"
      n_Uln_bus1_B = "n:Uln:bus1:B", "kV, Line-Neutral Voltage"
      n_Uln_bus1_C = "n:Uln:bus1:C", "kV, Line-Neutral Voltage"
      n_Uln_bus2 = "n:Uln:bus2", "kV, Line-Neutral Voltage"
      n_un_bus1 = "n:un:bus1", "p.u., Neutral-Ground Voltage"
      n_un_bus2 = "n:un:bus2", "p.u., Neutral-Ground Voltage"
      n_Un_bus1 = "n:Un:bus1", "kV, Neutral-Ground Voltage"
      n_Un_bus2 = "n:Un:bus2", "kV, Neutral-Ground Voltage"
      n_ishc_bus1_A = "n:ishc:bus1:A", "p.u., Short-Circuit Current"
      n_ishc_bus1_B = "n:ishc:bus1:B", "p.u., Short-Circuit Current"
      n_ishc_bus1_C = "n:ishc:bus1:C", "p.u., Short-Circuit Current"
      n_ishc_bus2 = "n:ishc:bus2", "p.u., Short-Circuit Current"
      n_Ishc_bus1_A = "n:Ishc:bus1:A", "kA, Short-Circuit Current"
      n_Ishc_bus1_B = "n:Ishc:bus1:B", "kA, Short-Circuit Current"
      n_Ishc_bus1_C = "n:Ishc:bus1:C", "kA, Short-Circuit Current"
      n_Ishc_bus2 = "n:Ishc:bus2", "kA, Short-Circuit Current"
      n_inshc_bus1 = "n:inshc:bus1", "p.u., Neutral Short-Circuit Current, magnitude"
      n_inshc_bus2 = "n:inshc:bus2", "p.u., Neutral Short-Circuit Current, magnitude"
      n_Inshc_bus1 = "n:Inshc:bus1", "kA, Neutral Short-Circuit Current, magnitude"
      n_Inshc_bus2 = "n:Inshc:bus2", "kA, Neutral Short-Circuit Current, magnitude"
      n_fe_bus1 = "n:fe:bus1", "p.u., Electrical Frequency"
      n_fe_bus2 = "n:fe:bus2", "p.u., Electrical Frequency"
      n_frdev_bus1 = "n:frdev:bus1", "Hz, average frequency"
      n_frdev_bus2 = "n:frdev:bus2", "Hz, average frequency"
      n_dfehz_bus1 = "n:dfehz:bus1", "Hz, Deviation of the El. Frequency"
      n_dfehz_bus2 = "n:dfehz:bus2", "Hz, Deviation of the El. Frequency"
      n_fehz_bus1 = "n:fehz:bus1", "Hz, Electrical Frequency"
      n_fehz_bus2 = "n:fehz:bus2", "Hz, Electrical Frequency"
      n_frnom_bus1 = "n:frnom:bus1", "Hz, Nominal Frequency"
      n_frnom_bus2 = "n:frnom:bus2", "Hz, Nominal Frequency"
      s_pt = "s:pt", "p.u., IN, Turbine Power, continuous, double"
      s_xmdm = "s:xmdm", "p.u., IN, MDM-Torque, continuous, double"
      s_rradd = "s:rradd", "p.u., IN, Additional Rotor Resistance, continuous, double"
      s_Pmd = "s:Pmd", "IN, PWM Amplification, Real Part, continuous, double"
      s_Pmq = "s:Pmq", "IN, PWM Amplification, Imaginary Part, continuous, double"
      s_phi = "s:phi", "rad, OUT, Angle, continuous, double"
      s_xspeed = "s:xspeed", "p.u., OUT, Speed, continuous, double"
      s_pgt = "s:pgt", "p.u., OUT, Active power, continuous, double"
      s_xme = "s:xme", "p.u., OUT, Electrical Torque, continuous, double"
      s_ird = "s:ird", "p.u., OUT, Rotor Current, d-Axis, continuous, double"
      s_irq = "s:irq", "p.u., OUT, Rotor Current, q-Axis, continuous, double"
      s_xphim = "s:xphim", "rad, OUT, Rotor Angle, continuous, double"
      s_psis_r = "s:psis_r", "p.u., OUT, Stator Flux, Real Part, continuous, double"
      s_psis_i = "s:psis_i", "p.u., OUT, Stator Flux, Imaginary Part, continuous, double"
      s_cosphim = "s:cosphim", "OUT, cos(phim), continuous, double"
      s_sinphim = "s:sinphim", "OUT, sin(phim), continuous, double"
      s_cosphi = "s:cosphi", "OUT, cos(phi), continuous, double"
      s_sinphi = "s:sinphi", "OUT, sin(phi), continuous, double"
      s_psir_r = "s:psir_r", "OUT, Rotor-Flux, Real Part, continuous, double"
      s_psir_i = "s:psir_i", "OUT, Rotor-Flux, Imaginary Part, continuous, double"
      s_speed = "s:speed", "STATE, Speed, continuous, double"
      s_psiA1_r = "s:psiA1_r", "STATE, Flux, Loop A, Real Part continuous, double"
      s_psiA1_i = "s:psiA1_i", "STATE, Flux, Loop A, Imaginary Part, continuous, double"
      s_psiA2_r = "s:psiA2_r", "STATE, Flux, Loop A, Real Part continuous, double"
      s_psiA2_i = "s:psiA2_i", "STATE, Flux, Loop A, Imaginary Part, continuous, double"
      s_psiB_r = "s:psiB_r", "STATE, Flux, Loop B, Real Part continuous, double"
      s_psiB_i = "s:psiB_i", "STATE, Flux, Loop B, Imaginary Part, continuous, double"
      s_phim = "s:phim", "rad, STATE, Mechanical Angle (with reference to synchronous reference frame), continuous, double"
      s_speed_dt = "s:speed:dt", "/s, d/dt, Speed, derivative, continuous, double"
      s_psiA1_r_dt = "s:psiA1_r:dt", "/s, d/dt, Flux, Loop A, Real Part, derivative, continuous, double"
      s_psiA1_i_dt = "s:psiA1_i:dt", "/s, d/dt, Flux, Loop A, Imaginary Part, derivative, continuous, double"
      s_psiA2_r_dt = "s:psiA2_r:dt", "/s, d/dt, Flux, Loop A, Real Part, derivative, continuous, double"
      s_psiA2_i_dt = "s:psiA2_i:dt", "/s, d/dt, Flux, Loop A, Imaginary Part, derivative, continuous, double"
      s_psiB_r_dt = "s:psiB_r:dt", "/s, d/dt, Flux, Loop B, Real Part, derivative, continuous, double"
      s_psiB_i_dt = "s:psiB_i:dt", "/s, d/dt, Flux, Loop B, Imaginary Part, derivative, continuous, double"
      s_phim_dt = "s:phim:dt", "rad/s, d/dt, Mechanical Angle (with reference to synchronous reference frame), derivative, continuous, double"
      c_cosn = "c:cosn", "Rated Power Factor"
      c_p = "c:p", "MW, Active Power (Electrical)"
      c_rstr = "c:rstr", "p.u., Stator Resistance"
      c_xstr = "c:xstr", "p.u., Stator Reactance"
      c_rrtrA = "c:rrtrA", "p.u., Rotor Resistance A"
      c_rrtrB = "c:rrtrB", "p.u., Rotor Resistance B"
      c_xrtrA = "c:xrtrA", "p.u., Rotor Reactance A"
      c_xrtrB = "c:xrtrB", "p.u., Rotor Reactance B"
      c_xm = "c:xm", "p.u., Magnetising Reactance"
      c_xmrtr = "c:xmrtr", "p.u., Rotor Leakage Reactance"
      c_frnom = "c:frnom", "Hz, Nominal Frequency"
      c_i_mot = "c:i_mot", "i_mot=0"
      c_xmem = "c:xmem", "p.u., Electrical Torque"
      c_xmt = "c:xmt", "Mechanical Torque"
      c_slip = "c:slip", "%, Slip"
      c_tag = "c:tag", "s, Acceleration Time Constant"
      c_i_star = "c:i_star", "Star/Delta"
      c_xss = "c:xss", "p.u., Subtransient Reactance"
      c_addmt = "c:addmt", "p.u., Additional Torque"
      c_ccomp = "c:ccomp", "p.u., Capacitor for Compensation"
      c_Pmr = "c:Pmr", "Amplification, Real Part"
      c_Pmi = "c:Pmi", "Amplification, Imaginary Part"
      c_Unrot = "c:Unrot", "kV, Rated Slip Ring Voltage"
      c_urd = "c:urd", "p.u., Rotor Voltage, d-Axis"
      c_urq = "c:urq", "p.u., Rotor Voltage, q-Axis"
      c_ura = "c:ura", "p.u., Rotor Voltage, Phase a"
      c_urb = "c:urb", "p.u., Rotor Voltage, Phase b"
      c_urc = "c:urc", "p.u., Rotor Voltage, Phase c"
      c_Ird = "c:Ird", "kA, Rotor Current, d-Axis"
      c_Irq = "c:Irq", "kA, Rotor Current, q-Axis"
      c_Ira = "c:Ira", "kA, Rotor Current, Phase a"
      c_Irb = "c:Irb", "kA, Rotor Current, Phase b"
      c_Irc = "c:Irc", "kA, Rotor Current, Phase c"
      c_xradd = "c:xradd", "p.u., Additional Rotor Reactance"
      c_xstadd = "c:xstadd", "p.u., Additional Stator Reactance"
      c_Irot = "c:Irot", "kA, Rotor Current, Magnitude of Phasor"
      c_rcrow = "c:rcrow", "Crow-Bar Resistance"
      c_xcrow = "c:xcrow", "Crow-Bar Reactance"
      c_set_bypass = "c:set_bypass", "Set Rotor-Bypass"
      c_reset_bypass = "c:reset_bypass", "Reset Rotor-Bypass"
      c_psiM = "c:psiM", "p.u., Magnetising flux"
      c_ksat = "c:ksat", "Saturation factor"
      c_xmsat = "c:xmsat", "p.u., Saturated magnetising reactance"
      e_rcrow = "e:rcrow", "p.u., Crow-Bar Resistance"
      e_xcrow = "e:xcrow", "p.u., Crow-Bar Reactance"
      e_mdmlp = "e:mdmlp", "p.u., Proportional Factor"
      e_mdmex = "e:mdmex", "Exponent"

    class ElmBoundary(Enum):
      _init_ = 'value __doc__'


    class ElmCoup(Enum):
      _init_ = 'value __doc__'

      m_u0_bus1 = "m:u0:bus1", "p.u., Zero-Sequence-Voltage"
      m_u0_bus2 = "m:u0:bus2", "p.u., Zero-Sequence-Voltage"
      m_u1r_bus1 = "m:u1r:bus1", "p.u., Voltage Phasor, Real Part"
      m_u1r_bus2 = "m:u1r:bus2", "p.u., Voltage Phasor, Real Part"
      m_u1i_bus1 = "m:u1i:bus1", "p.u., Voltage Phasor, Imaginary Part"
      m_u1i_bus2 = "m:u1i:bus2", "p.u., Voltage Phasor, Imaginary Part"
      m_u1_bus1 = "m:u1:bus1", "p.u., Voltage Phasor, Magnitude"
      m_u1_bus2 = "m:u1:bus2", "p.u., Voltage Phasor, Magnitude"
      m_U1_bus1 = "m:U1:bus1", "kV, Line-Ground Voltage Phasor, Magnitude"
      m_U1_bus2 = "m:U1:bus2", "kV, Line-Ground Voltage Phasor, Magnitude"
      m_U1l_bus1 = "m:U1l:bus1", "kV, Line-Line Voltage Phasor, Magnitude"
      m_U1l_bus2 = "m:U1l:bus2", "kV, Line-Line Voltage Phasor, Magnitude"
      m_phiu1_bus1 = "m:phiu1:bus1", "deg, Voltage Phasor, Angle"
      m_phiu1_bus2 = "m:phiu1:bus2", "deg, Voltage Phasor, Angle"
      m_I_bus1_A = "m:I:bus1:A", "kA, Phase Current"
      m_I_bus1_B = "m:I:bus1:B", "kA, Phase Current"
      m_I_bus1_C = "m:I:bus1:C", "kA, Phase Current"
      m_I_bus1_N = "m:I:bus1:N", "kA, Phase Current"
      m_I_bus2_A = "m:I:bus2:A", "kA, Phase Current"
      m_I_bus2_B = "m:I:bus2:B", "kA, Phase Current"
      m_I_bus2_C = "m:I:bus2:C", "kA, Phase Current"
      m_I_bus2_N = "m:I:bus2:N", "kA, Phase Current"
      m_i0_bus1 = "m:i0:bus1", "p.u., Zero-Sequence Current"
      m_i0_bus2 = "m:i0:bus2", "p.u., Zero-Sequence Current"
      m_I0_bus1 = "m:I0:bus1", "kA, Zero-Sequence Current"
      m_I0_bus2 = "m:I0:bus2", "kA, Zero-Sequence Current"
      m_I0x3_bus1 = "m:I0x3:bus1", "kA, 3*I0"
      m_I0x3_bus2 = "m:I0x3:bus2", "kA, 3*I0"
      m_i1r_bus1 = "m:i1r:bus1", "p.u., Current Phasor, Real Part"
      m_i1r_bus2 = "m:i1r:bus2", "p.u., Current Phasor, Real Part"
      m_i1i_bus1 = "m:i1i:bus1", "p.u., Current Phasor, Imaginary Part"
      m_i1i_bus2 = "m:i1i:bus2", "p.u., Current Phasor, Imaginary Part"
      m_i1_bus1 = "m:i1:bus1", "p.u., Current Phasor, Magnitude"
      m_i1_bus2 = "m:i1:bus2", "p.u., Current Phasor, Magnitude"
      m_I1_bus1 = "m:I1:bus1", "kA, Current Phasor, Magnitude"
      m_I1_bus2 = "m:I1:bus2", "kA, Current Phasor, Magnitude"
      m_phii1_bus1 = "m:phii1:bus1", "deg, Current Phasor, Angle"
      m_phii1_bus2 = "m:phii1:bus2", "deg, Current Phasor, Angle"
      m_P_bus1_A = "m:P:bus1:A", "MW, Active Power"
      m_P_bus1_B = "m:P:bus1:B", "MW, Active Power"
      m_P_bus1_C = "m:P:bus1:C", "MW, Active Power"
      m_P_bus1_N = "m:P:bus1:N", "MW, Active Power"
      m_P_bus2_A = "m:P:bus2:A", "MW, Active Power"
      m_P_bus2_B = "m:P:bus2:B", "MW, Active Power"
      m_P_bus2_C = "m:P:bus2:C", "MW, Active Power"
      m_P_bus2_N = "m:P:bus2:N", "MW, Active Power"
      m_Psum_bus1 = "m:Psum:bus1", "MW, Power-Phasor, Active Power"
      m_Psum_bus2 = "m:Psum:bus2", "MW, Power-Phasor, Active Power"
      m_Qsum_bus1 = "m:Qsum:bus1", "Mvar, Power Phasor, Reactive Part"
      m_Qsum_bus2 = "m:Qsum:bus2", "Mvar, Power Phasor, Reactive Part"
      m_Ssum_bus1 = "m:Ssum:bus1", "MVA, Power Phasor, Apparent Power"
      m_Ssum_bus2 = "m:Ssum:bus2", "MVA, Power Phasor, Apparent Power"
      m_cosphi1_bus1 = "m:cosphi1:bus1", "Power-Phasor, Power-Factor"
      m_cosphi1_bus2 = "m:cosphi1:bus2", "Power-Phasor, Power-Factor"
      m_i1P_bus1 = "m:i1P:bus1", "p.u., Current Phasor, Active Current"
      m_i1P_bus2 = "m:i1P:bus2", "p.u., Current Phasor, Active Current"
      m_i1Q_bus1 = "m:i1Q:bus1", "p.u., Current Phasor, Reactive Current"
      m_i1Q_bus2 = "m:i1Q:bus2", "p.u., Current Phasor, Reactive Current"
      m_I1P_bus1 = "m:I1P:bus1", "kA, Current Phasor, Active Current"
      m_I1P_bus2 = "m:I1P:bus2", "kA, Current Phasor, Active Current"
      m_I1Q_bus1 = "m:I1Q:bus1", "kA, Current Phasor, Reactive Current"
      m_I1Q_bus2 = "m:I1Q:bus2", "kA, Current Phasor, Reactive Current"
      m_u_bus1_A = "m:u:bus1:A", "p.u., Phase Voltage"
      m_u_bus1_B = "m:u:bus1:B", "p.u., Phase Voltage"
      m_u_bus1_C = "m:u:bus1:C", "p.u., Phase Voltage"
      m_u_bus1_N = "m:u:bus1:N", "p.u., Phase Voltage"
      m_u_bus2_A = "m:u:bus2:A", "p.u., Phase Voltage"
      m_u_bus2_B = "m:u:bus2:B", "p.u., Phase Voltage"
      m_u_bus2_C = "m:u:bus2:C", "p.u., Phase Voltage"
      m_u_bus2_N = "m:u:bus2:N", "p.u., Phase Voltage"
      m_i_bus1_A = "m:i:bus1:A", "p.u., Phase Current"
      m_i_bus1_B = "m:i:bus1:B", "p.u., Phase Current"
      m_i_bus1_C = "m:i:bus1:C", "p.u., Phase Current"
      m_i_bus1_N = "m:i:bus1:N", "p.u., Phase Current"
      m_i_bus2_A = "m:i:bus2:A", "p.u., Phase Current"
      m_i_bus2_B = "m:i:bus2:B", "p.u., Phase Current"
      m_i_bus2_C = "m:i:bus2:C", "p.u., Phase Current"
      m_i_bus2_N = "m:i:bus2:N", "p.u., Phase Current"
      m_phiu1i1_bus1 = "m:phiu1i1:bus1", "deg, Angle between Voltage and Current Phasor"
      m_phiu1i1_bus2 = "m:phiu1i1:bus2", "deg, Angle between Voltage and Current Phasor"
      m_Tfct_bus1 = "m:Tfct:bus1", "s, Fault Clearing Time"
      m_Tfct_bus2 = "m:Tfct:bus2", "s, Fault Clearing Time"
      m_TfctPh_bus1_A = "m:TfctPh:bus1:A", "s, Fault Clearing Time"
      m_TfctPh_bus1_B = "m:TfctPh:bus1:B", "s, Fault Clearing Time"
      m_TfctPh_bus1_C = "m:TfctPh:bus1:C", "s, Fault Clearing Time"
      m_TfctPh_bus1_N = "m:TfctPh:bus1:N", "s, Fault Clearing Time"
      m_TfctPh_bus2_A = "m:TfctPh:bus2:A", "s, Fault Clearing Time"
      m_TfctPh_bus2_B = "m:TfctPh:bus2:B", "s, Fault Clearing Time"
      m_TfctPh_bus2_C = "m:TfctPh:bus2:C", "s, Fault Clearing Time"
      m_TfctPh_bus2_N = "m:TfctPh:bus2:N", "s, Fault Clearing Time"
      n_u_bus1_A = "n:u:bus1:A", "p.u., Phase Voltage"
      n_u_bus1_B = "n:u:bus1:B", "p.u., Phase Voltage"
      n_u_bus1_C = "n:u:bus1:C", "p.u., Phase Voltage"
      n_u_bus1_N = "n:u:bus1:N", "p.u., Phase Voltage"
      n_u_bus2_A = "n:u:bus2:A", "p.u., Phase Voltage"
      n_u_bus2_B = "n:u:bus2:B", "p.u., Phase Voltage"
      n_u_bus2_C = "n:u:bus2:C", "p.u., Phase Voltage"
      n_u_bus2_N = "n:u:bus2:N", "p.u., Phase Voltage"
      n_upc_bus1_A = "n:upc:bus1:A", "%, Phase Voltage"
      n_upc_bus1_B = "n:upc:bus1:B", "%, Phase Voltage"
      n_upc_bus1_C = "n:upc:bus1:C", "%, Phase Voltage"
      n_upc_bus1_N = "n:upc:bus1:N", "%, Phase Voltage"
      n_upc_bus2_A = "n:upc:bus2:A", "%, Phase Voltage"
      n_upc_bus2_B = "n:upc:bus2:B", "%, Phase Voltage"
      n_upc_bus2_C = "n:upc:bus2:C", "%, Phase Voltage"
      n_upc_bus2_N = "n:upc:bus2:N", "%, Phase Voltage"
      n_U_bus1_A = "n:U:bus1:A", "kV, Phase Voltage"
      n_U_bus1_B = "n:U:bus1:B", "kV, Phase Voltage"
      n_U_bus1_C = "n:U:bus1:C", "kV, Phase Voltage"
      n_U_bus1_N = "n:U:bus1:N", "kV, Phase Voltage"
      n_U_bus2_A = "n:U:bus2:A", "kV, Phase Voltage"
      n_U_bus2_B = "n:U:bus2:B", "kV, Phase Voltage"
      n_U_bus2_C = "n:U:bus2:C", "kV, Phase Voltage"
      n_U_bus2_N = "n:U:bus2:N", "kV, Phase Voltage"
      n_ul_bus1_A = "n:ul:bus1:A", "p.u., Line-Line Voltage"
      n_ul_bus1_B = "n:ul:bus1:B", "p.u., Line-Line Voltage"
      n_ul_bus1_C = "n:ul:bus1:C", "p.u., Line-Line Voltage"
      n_ul_bus1_N = "n:ul:bus1:N", "p.u., Line-Line Voltage"
      n_ul_bus2_A = "n:ul:bus2:A", "p.u., Line-Line Voltage"
      n_ul_bus2_B = "n:ul:bus2:B", "p.u., Line-Line Voltage"
      n_ul_bus2_C = "n:ul:bus2:C", "p.u., Line-Line Voltage"
      n_ul_bus2_N = "n:ul:bus2:N", "p.u., Line-Line Voltage"
      n_ulpc_bus1_A = "n:ulpc:bus1:A", "%, Line-Line Voltage"
      n_ulpc_bus1_B = "n:ulpc:bus1:B", "%, Line-Line Voltage"
      n_ulpc_bus1_C = "n:ulpc:bus1:C", "%, Line-Line Voltage"
      n_ulpc_bus1_N = "n:ulpc:bus1:N", "%, Line-Line Voltage"
      n_ulpc_bus2_A = "n:ulpc:bus2:A", "%, Line-Line Voltage"
      n_ulpc_bus2_B = "n:ulpc:bus2:B", "%, Line-Line Voltage"
      n_ulpc_bus2_C = "n:ulpc:bus2:C", "%, Line-Line Voltage"
      n_ulpc_bus2_N = "n:ulpc:bus2:N", "%, Line-Line Voltage"
      n_Ul_bus1_A = "n:Ul:bus1:A", "kV, Line-Line Voltage"
      n_Ul_bus1_B = "n:Ul:bus1:B", "kV, Line-Line Voltage"
      n_Ul_bus1_C = "n:Ul:bus1:C", "kV, Line-Line Voltage"
      n_Ul_bus1_N = "n:Ul:bus1:N", "kV, Line-Line Voltage"
      n_Ul_bus2_A = "n:Ul:bus2:A", "kV, Line-Line Voltage"
      n_Ul_bus2_B = "n:Ul:bus2:B", "kV, Line-Line Voltage"
      n_Ul_bus2_C = "n:Ul:bus2:C", "kV, Line-Line Voltage"
      n_Ul_bus2_N = "n:Ul:bus2:N", "kV, Line-Line Voltage"
      n_u0_bus1 = "n:u0:bus1", "p.u., Zero-Sequence-Voltage"
      n_u0_bus2 = "n:u0:bus2", "p.u., Zero-Sequence-Voltage"
      n_U0_bus1 = "n:U0:bus1", "kV, Zero-Sequence-Voltage"
      n_U0_bus2 = "n:U0:bus2", "kV, Zero-Sequence-Voltage"
      n_U0x3_bus1 = "n:U0x3:bus1", "kV, 3*U0"
      n_U0x3_bus2 = "n:U0x3:bus2", "kV, 3*U0"
      n_u1r_bus1 = "n:u1r:bus1", "p.u., Voltage Phasor, Real Part"
      n_u1r_bus2 = "n:u1r:bus2", "p.u., Voltage Phasor, Real Part"
      n_u1i_bus1 = "n:u1i:bus1", "p.u., Voltage Phasor, Imaginary Part"
      n_u1i_bus2 = "n:u1i:bus2", "p.u., Voltage Phasor, Imaginary Part"
      n_u1_bus1 = "n:u1:bus1", "p.u., Voltage Phasor, Magnitude"
      n_u1_bus2 = "n:u1:bus2", "p.u., Voltage Phasor, Magnitude"
      n_u1pc_bus1 = "n:u1pc:bus1", "%, Voltage Phasor, Magnitude"
      n_u1pc_bus2 = "n:u1pc:bus2", "%, Voltage Phasor, Magnitude"
      n_U1_bus1 = "n:U1:bus1", "kV, Voltage Phasor, Magnitude"
      n_U1_bus2 = "n:U1:bus2", "kV, Voltage Phasor, Magnitude"
      n_phiu1_bus1 = "n:phiu1:bus1", "deg, Voltage Phasor, Angle"
      n_phiu1_bus2 = "n:phiu1:bus2", "deg, Voltage Phasor, Angle"
      n_U1l_bus1 = "n:U1l:bus1", "kV, Line-Line Voltage Phasor, Magnitude"
      n_U1l_bus2 = "n:U1l:bus2", "kV, Line-Line Voltage Phasor, Magnitude"
      n_uln_bus1_A = "n:uln:bus1:A", "p.u., Line-Neutral Voltage"
      n_uln_bus1_B = "n:uln:bus1:B", "p.u., Line-Neutral Voltage"
      n_uln_bus1_C = "n:uln:bus1:C", "p.u., Line-Neutral Voltage"
      n_uln_bus1_N = "n:uln:bus1:N", "p.u., Line-Neutral Voltage"
      n_uln_bus2_A = "n:uln:bus2:A", "p.u., Line-Neutral Voltage"
      n_uln_bus2_B = "n:uln:bus2:B", "p.u., Line-Neutral Voltage"
      n_uln_bus2_C = "n:uln:bus2:C", "p.u., Line-Neutral Voltage"
      n_uln_bus2_N = "n:uln:bus2:N", "p.u., Line-Neutral Voltage"
      n_Uln_bus1_A = "n:Uln:bus1:A", "kV, Line-Neutral Voltage"
      n_Uln_bus1_B = "n:Uln:bus1:B", "kV, Line-Neutral Voltage"
      n_Uln_bus1_C = "n:Uln:bus1:C", "kV, Line-Neutral Voltage"
      n_Uln_bus1_N = "n:Uln:bus1:N", "kV, Line-Neutral Voltage"
      n_Uln_bus2_A = "n:Uln:bus2:A", "kV, Line-Neutral Voltage"
      n_Uln_bus2_B = "n:Uln:bus2:B", "kV, Line-Neutral Voltage"
      n_Uln_bus2_C = "n:Uln:bus2:C", "kV, Line-Neutral Voltage"
      n_Uln_bus2_N = "n:Uln:bus2:N", "kV, Line-Neutral Voltage"
      n_un_bus1 = "n:un:bus1", "p.u., Neutral-Ground Voltage"
      n_un_bus2 = "n:un:bus2", "p.u., Neutral-Ground Voltage"
      n_Un_bus1 = "n:Un:bus1", "kV, Neutral-Ground Voltage"
      n_Un_bus2 = "n:Un:bus2", "kV, Neutral-Ground Voltage"
      n_ishc_bus1_A = "n:ishc:bus1:A", "p.u., Short-Circuit Current"
      n_ishc_bus1_B = "n:ishc:bus1:B", "p.u., Short-Circuit Current"
      n_ishc_bus1_C = "n:ishc:bus1:C", "p.u., Short-Circuit Current"
      n_ishc_bus1_N = "n:ishc:bus1:N", "p.u., Short-Circuit Current"
      n_ishc_bus2_A = "n:ishc:bus2:A", "p.u., Short-Circuit Current"
      n_ishc_bus2_B = "n:ishc:bus2:B", "p.u., Short-Circuit Current"
      n_ishc_bus2_C = "n:ishc:bus2:C", "p.u., Short-Circuit Current"
      n_ishc_bus2_N = "n:ishc:bus2:N", "p.u., Short-Circuit Current"
      n_Ishc_bus1_A = "n:Ishc:bus1:A", "kA, Short-Circuit Current"
      n_Ishc_bus1_B = "n:Ishc:bus1:B", "kA, Short-Circuit Current"
      n_Ishc_bus1_C = "n:Ishc:bus1:C", "kA, Short-Circuit Current"
      n_Ishc_bus1_N = "n:Ishc:bus1:N", "kA, Short-Circuit Current"
      n_Ishc_bus2_A = "n:Ishc:bus2:A", "kA, Short-Circuit Current"
      n_Ishc_bus2_B = "n:Ishc:bus2:B", "kA, Short-Circuit Current"
      n_Ishc_bus2_C = "n:Ishc:bus2:C", "kA, Short-Circuit Current"
      n_Ishc_bus2_N = "n:Ishc:bus2:N", "kA, Short-Circuit Current"
      n_inshc_bus1 = "n:inshc:bus1", "p.u., Neutral Short-Circuit Current, magnitude"
      n_inshc_bus2 = "n:inshc:bus2", "p.u., Neutral Short-Circuit Current, magnitude"
      n_Inshc_bus1 = "n:Inshc:bus1", "kA, Neutral Short-Circuit Current, magnitude"
      n_Inshc_bus2 = "n:Inshc:bus2", "kA, Neutral Short-Circuit Current, magnitude"
      n_fe_bus1 = "n:fe:bus1", "p.u., Electrical Frequency"
      n_fe_bus2 = "n:fe:bus2", "p.u., Electrical Frequency"
      n_frdev_bus1 = "n:frdev:bus1", "Hz, average frequency"
      n_frdev_bus2 = "n:frdev:bus2", "Hz, average frequency"
      n_dfehz_bus1 = "n:dfehz:bus1", "Hz, Deviation of the El. Frequency"
      n_dfehz_bus2 = "n:dfehz:bus2", "Hz, Deviation of the El. Frequency"
      n_fehz_bus1 = "n:fehz:bus1", "Hz, Electrical Frequency"
      n_fehz_bus2 = "n:fehz:bus2", "Hz, Electrical Frequency"
      n_frnom_bus1 = "n:frnom:bus1", "Hz, Nominal Frequency"
      n_frnom_bus2 = "n:frnom:bus2", "Hz, Nominal Frequency"
      s_dU_A = "s:dU_A", "kV, OUT, Voltage Difference, continuous, double"
      s_dU_B = "s:dU_B", "kV, OUT, Voltage Difference, continuous, double"
      s_dU_C = "s:dU_C", "kV, OUT, Voltage Difference, continuous, double"
      c_input = "c:input", "Control Input"
      c_i_inst = "c:i_inst", "Instant of Closing"
      c_t_delElm = "c:t_delElm", "s, Delay Time from network element and type"
      c_t_delType = "c:t_delType", "s, Delay Time from network element and type"
      c_t_delAcc = "c:t_delAcc", "s, Accumulated delay time"
      c_v_lim = "c:v_lim", "p.u., Voltage Limit"
      c_i_scheme = "c:i_scheme", "Scheme"
      c_i_larlow = "c:i_larlow", "Larger or lower than voltage threshold"
      c_R_on = "c:R_on", "Ohm, On Resistance"
      c_L_on = "c:L_on", "Ohm, On Inductance"
      c_i_ctrl = "c:i_ctrl", "Internal"
      c_i_trigPh = "c:i_trigPh", "Phase that triggers"
      c_closed = "c:closed", "Breaker Closed"
      c_closedPh_A = "c:closedPh:A", "Breaker Closed"
      c_closedPh_B = "c:closedPh:B", "Breaker Closed"
      c_closedPh_C = "c:closedPh:C", "Breaker Closed"
      c_closedPh_N = "c:closedPh:N", "Breaker Closed"
      c_i_open = "c:i_open", "Open at executions time or zero crossing"
      c_iNeutInter = "c:iNeutInter", "Switch interrupts neutral wire"
      c_TRV_pos_A = "c:TRV_pos:A", "CB TRV envelope positive"
      c_TRV_pos_B = "c:TRV_pos:B", "CB TRV envelope positive"
      c_TRV_pos_C = "c:TRV_pos:C", "CB TRV envelope positive"
      c_TRV_pos_N = "c:TRV_pos:N", "CB TRV envelope positive"
      c_TRV_neg_A = "c:TRV_neg:A", "CB TRV envelope negative"
      c_TRV_neg_B = "c:TRV_neg:B", "CB TRV envelope negative"
      c_TRV_neg_C = "c:TRV_neg:C", "CB TRV envelope negative"
      c_TRV_neg_N = "c:TRV_neg:N", "CB TRV envelope negative"
      c_locked = "c:locked", "Locked"
      c_IdFailure = "c:IdFailure", "Indication Failure"
      c_Indication = "c:Indication", "Indication"
      c_TCM = "c:TCM", "Trip Coil Monitor"
      e_t_del_a = "e:t_del_a", "ms, Phase a"
      e_t_del_b = "e:t_del_b", "ms, Phase b"
      e_t_del_c = "e:t_del_c", "ms, Phase c"
      e_t_del_n = "e:t_del_n", "ms, Neutral"
      e_logEnvCurve = "e:logEnvCurve", "Log TRV envelope"

    class ElmGenstat(Enum):
      _init_ = 'value __doc__'

      m_u0_bus1 = "m:u0:bus1", "p.u., Zero-Sequence-Voltage"
      m_u0_bus1n = "m:u0:bus1n", "p.u., Zero-Sequence-Voltage"
      m_u1r_bus1 = "m:u1r:bus1", "p.u., Voltage Phasor, Real Part"
      m_u1r_bus1n = "m:u1r:bus1n", "p.u., Voltage Phasor, Real Part"
      m_u1i_bus1 = "m:u1i:bus1", "p.u., Voltage Phasor, Imaginary Part"
      m_u1i_bus1n = "m:u1i:bus1n", "p.u., Voltage Phasor, Imaginary Part"
      m_u1_bus1 = "m:u1:bus1", "p.u., Voltage Phasor, Magnitude"
      m_u1_bus1n = "m:u1:bus1n", "p.u., Voltage Phasor, Magnitude"
      m_U1_bus1 = "m:U1:bus1", "kV, Line-Ground Voltage Phasor, Magnitude"
      m_U1_bus1n = "m:U1:bus1n", "kV, Line-Ground Voltage Phasor, Magnitude"
      m_U1l_bus1 = "m:U1l:bus1", "kV, Line-Line Voltage Phasor, Magnitude"
      m_U1l_bus1n = "m:U1l:bus1n", "kV, Line-Line Voltage Phasor, Magnitude"
      m_phiu1_bus1 = "m:phiu1:bus1", "deg, Voltage Phasor, Angle"
      m_phiu1_bus1n = "m:phiu1:bus1n", "deg, Voltage Phasor, Angle"
      m_I_bus1_A = "m:I:bus1:A", "kA, Phase Current"
      m_I_bus1_B = "m:I:bus1:B", "kA, Phase Current"
      m_I_bus1_C = "m:I:bus1:C", "kA, Phase Current"
      m_I_bus1_N = "m:I:bus1:N", "kA, Phase Current"
      m_I_bus1n = "m:I:bus1n", "kA, Phase Current"
      m_i0_bus1 = "m:i0:bus1", "p.u., Zero-Sequence Current"
      m_i0_bus1n = "m:i0:bus1n", "p.u., Zero-Sequence Current"
      m_I0_bus1 = "m:I0:bus1", "kA, Zero-Sequence Current"
      m_I0_bus1n = "m:I0:bus1n", "kA, Zero-Sequence Current"
      m_I0x3_bus1 = "m:I0x3:bus1", "kA, 3*I0"
      m_I0x3_bus1n = "m:I0x3:bus1n", "kA, 3*I0"
      m_i1r_bus1 = "m:i1r:bus1", "p.u., Current Phasor, Real Part"
      m_i1r_bus1n = "m:i1r:bus1n", "p.u., Current Phasor, Real Part"
      m_i1i_bus1 = "m:i1i:bus1", "p.u., Current Phasor, Imaginary Part"
      m_i1i_bus1n = "m:i1i:bus1n", "p.u., Current Phasor, Imaginary Part"
      m_i1_bus1 = "m:i1:bus1", "p.u., Current Phasor, Magnitude"
      m_i1_bus1n = "m:i1:bus1n", "p.u., Current Phasor, Magnitude"
      m_I1_bus1 = "m:I1:bus1", "kA, Current Phasor, Magnitude"
      m_I1_bus1n = "m:I1:bus1n", "kA, Current Phasor, Magnitude"
      m_phii1_bus1 = "m:phii1:bus1", "deg, Current Phasor, Angle"
      m_phii1_bus1n = "m:phii1:bus1n", "deg, Current Phasor, Angle"
      m_P_bus1_A = "m:P:bus1:A", "MW, Active Power"
      m_P_bus1_B = "m:P:bus1:B", "MW, Active Power"
      m_P_bus1_C = "m:P:bus1:C", "MW, Active Power"
      m_P_bus1_N = "m:P:bus1:N", "MW, Active Power"
      m_P_bus1n = "m:P:bus1n", "MW, Active Power"
      m_Psum_bus1 = "m:Psum:bus1", "MW, Power-Phasor, Active Power"
      m_Psum_bus1n = "m:Psum:bus1n", "MW, Power-Phasor, Active Power"
      m_Qsum_bus1 = "m:Qsum:bus1", "Mvar, Power Phasor, Reactive Part"
      m_Qsum_bus1n = "m:Qsum:bus1n", "Mvar, Power Phasor, Reactive Part"
      m_Ssum_bus1 = "m:Ssum:bus1", "MVA, Power Phasor, Apparent Power"
      m_Ssum_bus1n = "m:Ssum:bus1n", "MVA, Power Phasor, Apparent Power"
      m_cosphi1_bus1 = "m:cosphi1:bus1", "Power-Phasor, Power-Factor"
      m_cosphi1_bus1n = "m:cosphi1:bus1n", "Power-Phasor, Power-Factor"
      m_i1P_bus1 = "m:i1P:bus1", "p.u., Current Phasor, Active Current"
      m_i1P_bus1n = "m:i1P:bus1n", "p.u., Current Phasor, Active Current"
      m_i1Q_bus1 = "m:i1Q:bus1", "p.u., Current Phasor, Reactive Current"
      m_i1Q_bus1n = "m:i1Q:bus1n", "p.u., Current Phasor, Reactive Current"
      m_I1P_bus1 = "m:I1P:bus1", "kA, Current Phasor, Active Current"
      m_I1P_bus1n = "m:I1P:bus1n", "kA, Current Phasor, Active Current"
      m_I1Q_bus1 = "m:I1Q:bus1", "kA, Current Phasor, Reactive Current"
      m_I1Q_bus1n = "m:I1Q:bus1n", "kA, Current Phasor, Reactive Current"
      m_u_bus1_A = "m:u:bus1:A", "p.u., Phase Voltage"
      m_u_bus1_B = "m:u:bus1:B", "p.u., Phase Voltage"
      m_u_bus1_C = "m:u:bus1:C", "p.u., Phase Voltage"
      m_u_bus1_N = "m:u:bus1:N", "p.u., Phase Voltage"
      m_u_bus1n = "m:u:bus1n", "p.u., Phase Voltage"
      m_i_bus1_A = "m:i:bus1:A", "p.u., Phase Current"
      m_i_bus1_B = "m:i:bus1:B", "p.u., Phase Current"
      m_i_bus1_C = "m:i:bus1:C", "p.u., Phase Current"
      m_i_bus1_N = "m:i:bus1:N", "p.u., Phase Current"
      m_i_bus1n = "m:i:bus1n", "p.u., Phase Current"
      m_phiu1i1_bus1 = "m:phiu1i1:bus1", "deg, Angle between Voltage and Current Phasor"
      m_phiu1i1_bus1n = "m:phiu1i1:bus1n", "deg, Angle between Voltage and Current Phasor"
      m_Tfct_bus1 = "m:Tfct:bus1", "s, Fault Clearing Time"
      m_Tfct_bus1n = "m:Tfct:bus1n", "s, Fault Clearing Time"
      m_TfctPh_bus1_A = "m:TfctPh:bus1:A", "s, Fault Clearing Time"
      m_TfctPh_bus1_B = "m:TfctPh:bus1:B", "s, Fault Clearing Time"
      m_TfctPh_bus1_C = "m:TfctPh:bus1:C", "s, Fault Clearing Time"
      m_TfctPh_bus1_N = "m:TfctPh:bus1:N", "s, Fault Clearing Time"
      m_TfctPh_bus1n = "m:TfctPh:bus1n", "s, Fault Clearing Time"
      n_u_bus1_A = "n:u:bus1:A", "p.u., Phase Voltage"
      n_u_bus1_B = "n:u:bus1:B", "p.u., Phase Voltage"
      n_u_bus1_C = "n:u:bus1:C", "p.u., Phase Voltage"
      n_u_bus1_N = "n:u:bus1:N", "p.u., Phase Voltage"
      n_u_bus1n = "n:u:bus1n", "p.u., Phase Voltage"
      n_upc_bus1_A = "n:upc:bus1:A", "%, Phase Voltage"
      n_upc_bus1_B = "n:upc:bus1:B", "%, Phase Voltage"
      n_upc_bus1_C = "n:upc:bus1:C", "%, Phase Voltage"
      n_upc_bus1_N = "n:upc:bus1:N", "%, Phase Voltage"
      n_upc_bus1n = "n:upc:bus1n", "%, Phase Voltage"
      n_U_bus1_A = "n:U:bus1:A", "kV, Phase Voltage"
      n_U_bus1_B = "n:U:bus1:B", "kV, Phase Voltage"
      n_U_bus1_C = "n:U:bus1:C", "kV, Phase Voltage"
      n_U_bus1_N = "n:U:bus1:N", "kV, Phase Voltage"
      n_U_bus1n = "n:U:bus1n", "kV, Phase Voltage"
      n_ul_bus1_A = "n:ul:bus1:A", "p.u., Line-Line Voltage"
      n_ul_bus1_B = "n:ul:bus1:B", "p.u., Line-Line Voltage"
      n_ul_bus1_C = "n:ul:bus1:C", "p.u., Line-Line Voltage"
      n_ul_bus1_N = "n:ul:bus1:N", "p.u., Line-Line Voltage"
      n_ul_bus1n = "n:ul:bus1n", "p.u., Line-Line Voltage"
      n_ulpc_bus1_A = "n:ulpc:bus1:A", "%, Line-Line Voltage"
      n_ulpc_bus1_B = "n:ulpc:bus1:B", "%, Line-Line Voltage"
      n_ulpc_bus1_C = "n:ulpc:bus1:C", "%, Line-Line Voltage"
      n_ulpc_bus1_N = "n:ulpc:bus1:N", "%, Line-Line Voltage"
      n_ulpc_bus1n = "n:ulpc:bus1n", "%, Line-Line Voltage"
      n_Ul_bus1_A = "n:Ul:bus1:A", "kV, Line-Line Voltage"
      n_Ul_bus1_B = "n:Ul:bus1:B", "kV, Line-Line Voltage"
      n_Ul_bus1_C = "n:Ul:bus1:C", "kV, Line-Line Voltage"
      n_Ul_bus1_N = "n:Ul:bus1:N", "kV, Line-Line Voltage"
      n_Ul_bus1n = "n:Ul:bus1n", "kV, Line-Line Voltage"
      n_u0_bus1 = "n:u0:bus1", "p.u., Zero-Sequence-Voltage"
      n_u0_bus1n = "n:u0:bus1n", "p.u., Zero-Sequence-Voltage"
      n_U0_bus1 = "n:U0:bus1", "kV, Zero-Sequence-Voltage"
      n_U0_bus1n = "n:U0:bus1n", "kV, Zero-Sequence-Voltage"
      n_U0x3_bus1 = "n:U0x3:bus1", "kV, 3*U0"
      n_U0x3_bus1n = "n:U0x3:bus1n", "kV, 3*U0"
      n_u1r_bus1 = "n:u1r:bus1", "p.u., Voltage Phasor, Real Part"
      n_u1r_bus1n = "n:u1r:bus1n", "p.u., Voltage Phasor, Real Part"
      n_u1i_bus1 = "n:u1i:bus1", "p.u., Voltage Phasor, Imaginary Part"
      n_u1i_bus1n = "n:u1i:bus1n", "p.u., Voltage Phasor, Imaginary Part"
      n_u1_bus1 = "n:u1:bus1", "p.u., Voltage Phasor, Magnitude"
      n_u1_bus1n = "n:u1:bus1n", "p.u., Voltage Phasor, Magnitude"
      n_u1pc_bus1 = "n:u1pc:bus1", "%, Voltage Phasor, Magnitude"
      n_u1pc_bus1n = "n:u1pc:bus1n", "%, Voltage Phasor, Magnitude"
      n_U1_bus1 = "n:U1:bus1", "kV, Voltage Phasor, Magnitude"
      n_U1_bus1n = "n:U1:bus1n", "kV, Voltage Phasor, Magnitude"
      n_phiu1_bus1 = "n:phiu1:bus1", "deg, Voltage Phasor, Angle"
      n_phiu1_bus1n = "n:phiu1:bus1n", "deg, Voltage Phasor, Angle"
      n_U1l_bus1 = "n:U1l:bus1", "kV, Line-Line Voltage Phasor, Magnitude"
      n_U1l_bus1n = "n:U1l:bus1n", "kV, Line-Line Voltage Phasor, Magnitude"
      n_uln_bus1_A = "n:uln:bus1:A", "p.u., Line-Neutral Voltage"
      n_uln_bus1_B = "n:uln:bus1:B", "p.u., Line-Neutral Voltage"
      n_uln_bus1_C = "n:uln:bus1:C", "p.u., Line-Neutral Voltage"
      n_uln_bus1_N = "n:uln:bus1:N", "p.u., Line-Neutral Voltage"
      n_uln_bus1n = "n:uln:bus1n", "p.u., Line-Neutral Voltage"
      n_Uln_bus1_A = "n:Uln:bus1:A", "kV, Line-Neutral Voltage"
      n_Uln_bus1_B = "n:Uln:bus1:B", "kV, Line-Neutral Voltage"
      n_Uln_bus1_C = "n:Uln:bus1:C", "kV, Line-Neutral Voltage"
      n_Uln_bus1_N = "n:Uln:bus1:N", "kV, Line-Neutral Voltage"
      n_Uln_bus1n = "n:Uln:bus1n", "kV, Line-Neutral Voltage"
      n_un_bus1 = "n:un:bus1", "p.u., Neutral-Ground Voltage"
      n_un_bus1n = "n:un:bus1n", "p.u., Neutral-Ground Voltage"
      n_Un_bus1 = "n:Un:bus1", "kV, Neutral-Ground Voltage"
      n_Un_bus1n = "n:Un:bus1n", "kV, Neutral-Ground Voltage"
      n_ishc_bus1_A = "n:ishc:bus1:A", "p.u., Short-Circuit Current"
      n_ishc_bus1_B = "n:ishc:bus1:B", "p.u., Short-Circuit Current"
      n_ishc_bus1_C = "n:ishc:bus1:C", "p.u., Short-Circuit Current"
      n_ishc_bus1_N = "n:ishc:bus1:N", "p.u., Short-Circuit Current"
      n_ishc_bus1n = "n:ishc:bus1n", "p.u., Short-Circuit Current"
      n_Ishc_bus1_A = "n:Ishc:bus1:A", "kA, Short-Circuit Current"
      n_Ishc_bus1_B = "n:Ishc:bus1:B", "kA, Short-Circuit Current"
      n_Ishc_bus1_C = "n:Ishc:bus1:C", "kA, Short-Circuit Current"
      n_Ishc_bus1_N = "n:Ishc:bus1:N", "kA, Short-Circuit Current"
      n_Ishc_bus1n = "n:Ishc:bus1n", "kA, Short-Circuit Current"
      n_inshc_bus1 = "n:inshc:bus1", "p.u., Neutral Short-Circuit Current, magnitude"
      n_inshc_bus1n = "n:inshc:bus1n", "p.u., Neutral Short-Circuit Current, magnitude"
      n_Inshc_bus1 = "n:Inshc:bus1", "kA, Neutral Short-Circuit Current, magnitude"
      n_Inshc_bus1n = "n:Inshc:bus1n", "kA, Neutral Short-Circuit Current, magnitude"
      n_fe_bus1 = "n:fe:bus1", "p.u., Electrical Frequency"
      n_fe_bus1n = "n:fe:bus1n", "p.u., Electrical Frequency"
      n_frdev_bus1 = "n:frdev:bus1", "Hz, average frequency"
      n_frdev_bus1n = "n:frdev:bus1n", "Hz, average frequency"
      n_dfehz_bus1 = "n:dfehz:bus1", "Hz, Deviation of the El. Frequency"
      n_dfehz_bus1n = "n:dfehz:bus1n", "Hz, Deviation of the El. Frequency"
      n_fehz_bus1 = "n:fehz:bus1", "Hz, Electrical Frequency"
      n_fehz_bus1n = "n:fehz:bus1n", "Hz, Electrical Frequency"
      n_frnom_bus1 = "n:frnom:bus1", "Hz, Nominal Frequency"
      n_frnom_bus1n = "n:frnom:bus1n", "Hz, Nominal Frequency"
      s_f0 = "s:f0", "p.u., IN, Frequency input continuous, double"
      s_F0Hz = "s:F0Hz", "Hz, IN, Frequency input (Hz), continuous, double"
      s_fref = "s:fref", "p.u., IN, Reference frequency, continuous, double"
      s_id_ref = "s:id_ref", "p.u., IN, d-axis current reference, continuous, double"
      s_iq_ref = "s:iq_ref", "p.u., IN, q-axis current reference, continuous, double"
      s_i2d_ref = "s:i2d_ref", "p.u., IN, Negative sequence current, d-axis reference, continuous, double"
      s_i2q_ref = "s:i2q_ref", "p.u., IN, Negative sequence current, q-axis reference, continuous, double"
      s_u1r_in = "s:u1r_in", "p.u., IN, Voltage input, real part, continuous, double"
      s_u1i_in = "s:u1i_in", "p.u., IN, Voltage input, imaginary part, continuous, double"
      s_cosref = "s:cosref", "IN, cos(dq-reference angle) continuous, double"
      s_sinref = "s:sinref", "IN, sin(dq-reference angle) continuous, double"
      s_iin_a = "s:iin_a", "p.u., IN, Current, phase A, continuous, double"
      s_iin_b = "s:iin_b", "p.u., IN, Current, phase B, continuous, double"
      s_iin_c = "s:iin_c", "p.u., IN, Current, phase C, continuous, double"
      s_u0_in = "s:u0_in", "p.u., IN, Voltage input, zero sequence component, continuous, double"
      s_id = "s:id", "p.u., OUT, Current, d-axis continuous, double"
      s_iq = "s:iq", "p.u., OUT, Current, q-axis continuous, double"
      s_xd = "s:xd", "p.u., OUT, State variable, current controller d-axis, continuous, double"
      s_xq = "s:xq", "p.u., OUT, State variable, current controller q-axis, continuous, double"
      s_xd_dt = "s:xd:dt", "p.u./s, OUT, State variable, current controller d-axis, derivative, continuous, double"
      s_xq_dt = "s:xq:dt", "p.u./s, OUT, State variable, current controller q-axis, derivative, continuous, double"
      s_phiu = "s:phiu", "rad, STATE, Voltage angle, continuous, double"
      s_phiu_dt = "s:phiu:dt", "rad/s, d/dt, Voltage angle, derivative, continuous, double"
      c_sgnn = "c:sgnn", "MVA, Total rated apparent power (ngnum*sgn)"
      c_cosn = "c:cosn", "Nominal power factor"
      c_hpi = "c:hpi", "rad/s, Nominal angular frequency"
      e_pSubModel = "e:pSubModel", "ElmSubmodel, Submodel"
      e_umin = "e:umin", "p.u., Switch-off threshold"
      e_uonthr = "e:uonthr", "p.u., Switch-on threshold"
      e_Tondelay = "e:Tondelay", "s, Switch-on delay"
      e_uk = "e:uk", "%, Short circuit impedance"
      e_Pcu = "e:Pcu", "kW, Copper losses"
      e_R0toR1 = "e:R0toR1", "R0/R1 ratio"
      e_X0toX1 = "e:X0toX1", "X0/X1 ratio"
      e_Kd = "e:Kd", "Kd: d-Axis, proportional gain"
      e_Td = "e:Td", "s, Td: d-Axis, integration time constant"
      e_Kq = "e:Kq", "Kq: q-Axis, proportional gain"
      e_Tq = "e:Tq", "s, Tq: q-Axis, integration time constant"
      e_Kp_pllu = "e:Kp_pllu", "Proportional Gain"
      e_Ki_pllu = "e:Ki_pllu", "Integration Gain"
      e_T1_pllu = "e:T1_pllu", "ms, Low-Pass Filter Time Constant"
      e_Kp_plli = "e:Kp_plli", "Proportional Gain"
      e_Ki_plli = "e:Ki_plli", "Integration Gain"
      e_T1_plli = "e:T1_plli", "ms, Low-Pass Filter Time Constant"
      r_pSubModel = "r:pSubModel", "ElmSubmodel, Submodel"

    class ElmIac(Enum):
      _init_ = 'value __doc__'

      m_u0_bus1 = "m:u0:bus1", "p.u., Zero-Sequence-Voltage"
      m_u1r_bus1 = "m:u1r:bus1", "p.u., Voltage Phasor, Real Part"
      m_u1i_bus1 = "m:u1i:bus1", "p.u., Voltage Phasor, Imaginary Part"
      m_u1_bus1 = "m:u1:bus1", "p.u., Voltage Phasor, Magnitude"
      m_U1_bus1 = "m:U1:bus1", "kV, Line-Ground Voltage Phasor, Magnitude"
      m_U1l_bus1 = "m:U1l:bus1", "kV, Line-Line Voltage Phasor, Magnitude"
      m_phiu1_bus1 = "m:phiu1:bus1", "deg, Voltage Phasor, Angle"
      m_I_bus1_A = "m:I:bus1:A", "kA, Phase Current"
      m_I_bus1_B = "m:I:bus1:B", "kA, Phase Current"
      m_I_bus1_C = "m:I:bus1:C", "kA, Phase Current"
      m_i0_bus1 = "m:i0:bus1", "p.u., Zero-Sequence Current"
      m_I0_bus1 = "m:I0:bus1", "kA, Zero-Sequence Current"
      m_I0x3_bus1 = "m:I0x3:bus1", "kA, 3*I0"
      m_i1r_bus1 = "m:i1r:bus1", "p.u., Current Phasor, Real Part"
      m_i1i_bus1 = "m:i1i:bus1", "p.u., Current Phasor, Imaginary Part"
      m_i1_bus1 = "m:i1:bus1", "p.u., Current Phasor, Magnitude"
      m_I1_bus1 = "m:I1:bus1", "kA, Current Phasor, Magnitude"
      m_phii1_bus1 = "m:phii1:bus1", "deg, Current Phasor, Angle"
      m_P_bus1_A = "m:P:bus1:A", "MW, Active Power"
      m_P_bus1_B = "m:P:bus1:B", "MW, Active Power"
      m_P_bus1_C = "m:P:bus1:C", "MW, Active Power"
      m_Psum_bus1 = "m:Psum:bus1", "MW, Power-Phasor, Active Power"
      m_Qsum_bus1 = "m:Qsum:bus1", "Mvar, Power Phasor, Reactive Part"
      m_Ssum_bus1 = "m:Ssum:bus1", "MVA, Power Phasor, Apparent Power"
      m_cosphi1_bus1 = "m:cosphi1:bus1", "Power-Phasor, Power-Factor"
      m_i1P_bus1 = "m:i1P:bus1", "p.u., Current Phasor, Active Current"
      m_i1Q_bus1 = "m:i1Q:bus1", "p.u., Current Phasor, Reactive Current"
      m_I1P_bus1 = "m:I1P:bus1", "kA, Current Phasor, Active Current"
      m_I1Q_bus1 = "m:I1Q:bus1", "kA, Current Phasor, Reactive Current"
      m_u_bus1_A = "m:u:bus1:A", "p.u., Phase Voltage"
      m_u_bus1_B = "m:u:bus1:B", "p.u., Phase Voltage"
      m_u_bus1_C = "m:u:bus1:C", "p.u., Phase Voltage"
      m_i_bus1_A = "m:i:bus1:A", "p.u., Phase Current"
      m_i_bus1_B = "m:i:bus1:B", "p.u., Phase Current"
      m_i_bus1_C = "m:i:bus1:C", "p.u., Phase Current"
      m_phiu1i1_bus1 = "m:phiu1i1:bus1", "deg, Angle between Voltage and Current Phasor"
      m_Tfct_bus1 = "m:Tfct:bus1", "s, Fault Clearing Time"
      m_TfctPh_bus1_A = "m:TfctPh:bus1:A", "s, Fault Clearing Time"
      m_TfctPh_bus1_B = "m:TfctPh:bus1:B", "s, Fault Clearing Time"
      m_TfctPh_bus1_C = "m:TfctPh:bus1:C", "s, Fault Clearing Time"
      n_u_bus1_A = "n:u:bus1:A", "p.u., Phase Voltage"
      n_u_bus1_B = "n:u:bus1:B", "p.u., Phase Voltage"
      n_u_bus1_C = "n:u:bus1:C", "p.u., Phase Voltage"
      n_upc_bus1_A = "n:upc:bus1:A", "%, Phase Voltage"
      n_upc_bus1_B = "n:upc:bus1:B", "%, Phase Voltage"
      n_upc_bus1_C = "n:upc:bus1:C", "%, Phase Voltage"
      n_U_bus1_A = "n:U:bus1:A", "kV, Phase Voltage"
      n_U_bus1_B = "n:U:bus1:B", "kV, Phase Voltage"
      n_U_bus1_C = "n:U:bus1:C", "kV, Phase Voltage"
      n_ul_bus1_A = "n:ul:bus1:A", "p.u., Line-Line Voltage"
      n_ul_bus1_B = "n:ul:bus1:B", "p.u., Line-Line Voltage"
      n_ul_bus1_C = "n:ul:bus1:C", "p.u., Line-Line Voltage"
      n_ulpc_bus1_A = "n:ulpc:bus1:A", "%, Line-Line Voltage"
      n_ulpc_bus1_B = "n:ulpc:bus1:B", "%, Line-Line Voltage"
      n_ulpc_bus1_C = "n:ulpc:bus1:C", "%, Line-Line Voltage"
      n_Ul_bus1_A = "n:Ul:bus1:A", "kV, Line-Line Voltage"
      n_Ul_bus1_B = "n:Ul:bus1:B", "kV, Line-Line Voltage"
      n_Ul_bus1_C = "n:Ul:bus1:C", "kV, Line-Line Voltage"
      n_u0_bus1 = "n:u0:bus1", "p.u., Zero-Sequence-Voltage"
      n_U0_bus1 = "n:U0:bus1", "kV, Zero-Sequence-Voltage"
      n_U0x3_bus1 = "n:U0x3:bus1", "kV, 3*U0"
      n_u1r_bus1 = "n:u1r:bus1", "p.u., Voltage Phasor, Real Part"
      n_u1i_bus1 = "n:u1i:bus1", "p.u., Voltage Phasor, Imaginary Part"
      n_u1_bus1 = "n:u1:bus1", "p.u., Voltage Phasor, Magnitude"
      n_u1pc_bus1 = "n:u1pc:bus1", "%, Voltage Phasor, Magnitude"
      n_U1_bus1 = "n:U1:bus1", "kV, Voltage Phasor, Magnitude"
      n_phiu1_bus1 = "n:phiu1:bus1", "deg, Voltage Phasor, Angle"
      n_U1l_bus1 = "n:U1l:bus1", "kV, Line-Line Voltage Phasor, Magnitude"
      n_uln_bus1_A = "n:uln:bus1:A", "p.u., Line-Neutral Voltage"
      n_uln_bus1_B = "n:uln:bus1:B", "p.u., Line-Neutral Voltage"
      n_uln_bus1_C = "n:uln:bus1:C", "p.u., Line-Neutral Voltage"
      n_Uln_bus1_A = "n:Uln:bus1:A", "kV, Line-Neutral Voltage"
      n_Uln_bus1_B = "n:Uln:bus1:B", "kV, Line-Neutral Voltage"
      n_Uln_bus1_C = "n:Uln:bus1:C", "kV, Line-Neutral Voltage"
      n_un_bus1 = "n:un:bus1", "p.u., Neutral-Ground Voltage"
      n_Un_bus1 = "n:Un:bus1", "kV, Neutral-Ground Voltage"
      n_ishc_bus1_A = "n:ishc:bus1:A", "p.u., Short-Circuit Current"
      n_ishc_bus1_B = "n:ishc:bus1:B", "p.u., Short-Circuit Current"
      n_ishc_bus1_C = "n:ishc:bus1:C", "p.u., Short-Circuit Current"
      n_Ishc_bus1_A = "n:Ishc:bus1:A", "kA, Short-Circuit Current"
      n_Ishc_bus1_B = "n:Ishc:bus1:B", "kA, Short-Circuit Current"
      n_Ishc_bus1_C = "n:Ishc:bus1:C", "kA, Short-Circuit Current"
      n_inshc_bus1 = "n:inshc:bus1", "p.u., Neutral Short-Circuit Current, magnitude"
      n_Inshc_bus1 = "n:Inshc:bus1", "kA, Neutral Short-Circuit Current, magnitude"
      n_fe_bus1 = "n:fe:bus1", "p.u., Electrical Frequency"
      n_frdev_bus1 = "n:frdev:bus1", "Hz, average frequency"
      n_dfehz_bus1 = "n:dfehz:bus1", "Hz, Deviation of the El. Frequency"
      n_fehz_bus1 = "n:fehz:bus1", "Hz, Electrical Frequency"
      n_frnom_bus1 = "n:frnom:bus1", "Hz, Nominal Frequency"
      s_i0 = "s:i0", "p.u., IN, Current-Input, continuous, double"
      s_I0 = "s:I0", "kA, IN, Current-Input, continuous, double"
      s_f0 = "s:f0", "p.u., IN, Frequency-Input continuous, double"
      s_F0Hz = "s:F0Hz", "Hz, IN, Frequency-Input continuous, double"
      s_I_A = "s:I_A", "kA, IN, Current, Magnitude, continuous, double"
      s_I_B = "s:I_B", "kA, IN, Current, Magnitude, continuous, double"
      s_I_C = "s:I_C", "kA, IN, Current, Magnitude, continuous, double"
      s_dphii = "s:dphii", "rad, IN, Additional Angle, continuous, double"
      s_phii = "s:phii", "rad, STATE, Current-Angle, continuous, double"
      s_phii_dt = "s:phii:dt", "rad/s, d/dt, Current-Angle, derivative, continuous, double"
      c_Ir = "c:Ir", "A, Nominal Current"
      c_isetp = "c:isetp", "p.u., Current Setpoint"
      c_cosini = "c:cosini", "Power Factor"
      c_G1 = "c:G1", "S, Positive Sequence Conductance"
      c_C1 = "c:C1", "F, Positive Sequence Capacitance"
      c_G0 = "c:G0", "S, Zero Sequence Conductance"
      c_C0 = "c:C0", "F, Zero Sequence Capacitance"
      c_hpi = "c:hpi", "rad/s, Nominal-Angular-Frequency"
      e_G1 = "e:G1", "S, Internal Conductance, G1"
      e_B1 = "e:B1", "S, Internal Susceptance, B1"
      e_G0 = "e:G0", "S, Internal Conductance, G0"
      e_B0 = "e:B0", "S, Internal Susceptance, B0"

    class ElmLne(Enum):
      _init_ = 'value __doc__'

      m_u0_bus1 = "m:u0:bus1", "p.u., Zero-Sequence-Voltage"
      m_u0_bus2 = "m:u0:bus2", "p.u., Zero-Sequence-Voltage"
      m_u1r_bus1 = "m:u1r:bus1", "p.u., Voltage Phasor, Real Part"
      m_u1r_bus2 = "m:u1r:bus2", "p.u., Voltage Phasor, Real Part"
      m_u1i_bus1 = "m:u1i:bus1", "p.u., Voltage Phasor, Imaginary Part"
      m_u1i_bus2 = "m:u1i:bus2", "p.u., Voltage Phasor, Imaginary Part"
      m_u1_bus1 = "m:u1:bus1", "p.u., Voltage Phasor, Magnitude"
      m_u1_bus2 = "m:u1:bus2", "p.u., Voltage Phasor, Magnitude"
      m_U1_bus1 = "m:U1:bus1", "kV, Line-Ground Voltage Phasor, Magnitude"
      m_U1_bus2 = "m:U1:bus2", "kV, Line-Ground Voltage Phasor, Magnitude"
      m_U1l_bus1 = "m:U1l:bus1", "kV, Line-Line Voltage Phasor, Magnitude"
      m_U1l_bus2 = "m:U1l:bus2", "kV, Line-Line Voltage Phasor, Magnitude"
      m_phiu1_bus1 = "m:phiu1:bus1", "deg, Voltage Phasor, Angle"
      m_phiu1_bus2 = "m:phiu1:bus2", "deg, Voltage Phasor, Angle"
      m_I_bus1_A = "m:I:bus1:A", "kA, Phase Current"
      m_I_bus1_B = "m:I:bus1:B", "kA, Phase Current"
      m_I_bus1_C = "m:I:bus1:C", "kA, Phase Current"
      m_I_bus1_N = "m:I:bus1:N", "kA, Phase Current"
      m_I_bus2_A = "m:I:bus2:A", "kA, Phase Current"
      m_I_bus2_B = "m:I:bus2:B", "kA, Phase Current"
      m_I_bus2_C = "m:I:bus2:C", "kA, Phase Current"
      m_I_bus2_N = "m:I:bus2:N", "kA, Phase Current"
      m_i0_bus1 = "m:i0:bus1", "p.u., Zero-Sequence Current"
      m_i0_bus2 = "m:i0:bus2", "p.u., Zero-Sequence Current"
      m_I0_bus1 = "m:I0:bus1", "kA, Zero-Sequence Current"
      m_I0_bus2 = "m:I0:bus2", "kA, Zero-Sequence Current"
      m_I0x3_bus1 = "m:I0x3:bus1", "kA, 3*I0"
      m_I0x3_bus2 = "m:I0x3:bus2", "kA, 3*I0"
      m_i1r_bus1 = "m:i1r:bus1", "p.u., Current Phasor, Real Part"
      m_i1r_bus2 = "m:i1r:bus2", "p.u., Current Phasor, Real Part"
      m_i1i_bus1 = "m:i1i:bus1", "p.u., Current Phasor, Imaginary Part"
      m_i1i_bus2 = "m:i1i:bus2", "p.u., Current Phasor, Imaginary Part"
      m_i1_bus1 = "m:i1:bus1", "p.u., Current Phasor, Magnitude"
      m_i1_bus2 = "m:i1:bus2", "p.u., Current Phasor, Magnitude"
      m_I1_bus1 = "m:I1:bus1", "kA, Current Phasor, Magnitude"
      m_I1_bus2 = "m:I1:bus2", "kA, Current Phasor, Magnitude"
      m_phii1_bus1 = "m:phii1:bus1", "deg, Current Phasor, Angle"
      m_phii1_bus2 = "m:phii1:bus2", "deg, Current Phasor, Angle"
      m_P_bus1_A = "m:P:bus1:A", "MW, Active Power"
      m_P_bus1_B = "m:P:bus1:B", "MW, Active Power"
      m_P_bus1_C = "m:P:bus1:C", "MW, Active Power"
      m_P_bus1_N = "m:P:bus1:N", "MW, Active Power"
      m_P_bus2_A = "m:P:bus2:A", "MW, Active Power"
      m_P_bus2_B = "m:P:bus2:B", "MW, Active Power"
      m_P_bus2_C = "m:P:bus2:C", "MW, Active Power"
      m_P_bus2_N = "m:P:bus2:N", "MW, Active Power"
      m_Psum_bus1 = "m:Psum:bus1", "MW, Power-Phasor, Active Power"
      m_Psum_bus2 = "m:Psum:bus2", "MW, Power-Phasor, Active Power"
      m_Qsum_bus1 = "m:Qsum:bus1", "Mvar, Power Phasor, Reactive Part"
      m_Qsum_bus2 = "m:Qsum:bus2", "Mvar, Power Phasor, Reactive Part"
      m_Ssum_bus1 = "m:Ssum:bus1", "MVA, Power Phasor, Apparent Power"
      m_Ssum_bus2 = "m:Ssum:bus2", "MVA, Power Phasor, Apparent Power"
      m_cosphi1_bus1 = "m:cosphi1:bus1", "Power-Phasor, Power-Factor"
      m_cosphi1_bus2 = "m:cosphi1:bus2", "Power-Phasor, Power-Factor"
      m_i1P_bus1 = "m:i1P:bus1", "p.u., Current Phasor, Active Current"
      m_i1P_bus2 = "m:i1P:bus2", "p.u., Current Phasor, Active Current"
      m_i1Q_bus1 = "m:i1Q:bus1", "p.u., Current Phasor, Reactive Current"
      m_i1Q_bus2 = "m:i1Q:bus2", "p.u., Current Phasor, Reactive Current"
      m_I1P_bus1 = "m:I1P:bus1", "kA, Current Phasor, Active Current"
      m_I1P_bus2 = "m:I1P:bus2", "kA, Current Phasor, Active Current"
      m_I1Q_bus1 = "m:I1Q:bus1", "kA, Current Phasor, Reactive Current"
      m_I1Q_bus2 = "m:I1Q:bus2", "kA, Current Phasor, Reactive Current"
      m_u_bus1_A = "m:u:bus1:A", "p.u., Phase Voltage"
      m_u_bus1_B = "m:u:bus1:B", "p.u., Phase Voltage"
      m_u_bus1_C = "m:u:bus1:C", "p.u., Phase Voltage"
      m_u_bus1_N = "m:u:bus1:N", "p.u., Phase Voltage"
      m_u_bus2_A = "m:u:bus2:A", "p.u., Phase Voltage"
      m_u_bus2_B = "m:u:bus2:B", "p.u., Phase Voltage"
      m_u_bus2_C = "m:u:bus2:C", "p.u., Phase Voltage"
      m_u_bus2_N = "m:u:bus2:N", "p.u., Phase Voltage"
      m_i_bus1_A = "m:i:bus1:A", "p.u., Phase Current"
      m_i_bus1_B = "m:i:bus1:B", "p.u., Phase Current"
      m_i_bus1_C = "m:i:bus1:C", "p.u., Phase Current"
      m_i_bus1_N = "m:i:bus1:N", "p.u., Phase Current"
      m_i_bus2_A = "m:i:bus2:A", "p.u., Phase Current"
      m_i_bus2_B = "m:i:bus2:B", "p.u., Phase Current"
      m_i_bus2_C = "m:i:bus2:C", "p.u., Phase Current"
      m_i_bus2_N = "m:i:bus2:N", "p.u., Phase Current"
      m_phiu1i1_bus1 = "m:phiu1i1:bus1", "deg, Angle between Voltage and Current Phasor"
      m_phiu1i1_bus2 = "m:phiu1i1:bus2", "deg, Angle between Voltage and Current Phasor"
      m_Tfct_bus1 = "m:Tfct:bus1", "s, Fault Clearing Time"
      m_Tfct_bus2 = "m:Tfct:bus2", "s, Fault Clearing Time"
      m_TfctPh_bus1_A = "m:TfctPh:bus1:A", "s, Fault Clearing Time"
      m_TfctPh_bus1_B = "m:TfctPh:bus1:B", "s, Fault Clearing Time"
      m_TfctPh_bus1_C = "m:TfctPh:bus1:C", "s, Fault Clearing Time"
      m_TfctPh_bus1_N = "m:TfctPh:bus1:N", "s, Fault Clearing Time"
      m_TfctPh_bus2_A = "m:TfctPh:bus2:A", "s, Fault Clearing Time"
      m_TfctPh_bus2_B = "m:TfctPh:bus2:B", "s, Fault Clearing Time"
      m_TfctPh_bus2_C = "m:TfctPh:bus2:C", "s, Fault Clearing Time"
      m_TfctPh_bus2_N = "m:TfctPh:bus2:N", "s, Fault Clearing Time"
      n_u_bus1_A = "n:u:bus1:A", "p.u., Phase Voltage"
      n_u_bus1_B = "n:u:bus1:B", "p.u., Phase Voltage"
      n_u_bus1_C = "n:u:bus1:C", "p.u., Phase Voltage"
      n_u_bus1_N = "n:u:bus1:N", "p.u., Phase Voltage"
      n_u_bus2_A = "n:u:bus2:A", "p.u., Phase Voltage"
      n_u_bus2_B = "n:u:bus2:B", "p.u., Phase Voltage"
      n_u_bus2_C = "n:u:bus2:C", "p.u., Phase Voltage"
      n_u_bus2_N = "n:u:bus2:N", "p.u., Phase Voltage"
      n_upc_bus1_A = "n:upc:bus1:A", "%, Phase Voltage"
      n_upc_bus1_B = "n:upc:bus1:B", "%, Phase Voltage"
      n_upc_bus1_C = "n:upc:bus1:C", "%, Phase Voltage"
      n_upc_bus1_N = "n:upc:bus1:N", "%, Phase Voltage"
      n_upc_bus2_A = "n:upc:bus2:A", "%, Phase Voltage"
      n_upc_bus2_B = "n:upc:bus2:B", "%, Phase Voltage"
      n_upc_bus2_C = "n:upc:bus2:C", "%, Phase Voltage"
      n_upc_bus2_N = "n:upc:bus2:N", "%, Phase Voltage"
      n_U_bus1_A = "n:U:bus1:A", "kV, Phase Voltage"
      n_U_bus1_B = "n:U:bus1:B", "kV, Phase Voltage"
      n_U_bus1_C = "n:U:bus1:C", "kV, Phase Voltage"
      n_U_bus1_N = "n:U:bus1:N", "kV, Phase Voltage"
      n_U_bus2_A = "n:U:bus2:A", "kV, Phase Voltage"
      n_U_bus2_B = "n:U:bus2:B", "kV, Phase Voltage"
      n_U_bus2_C = "n:U:bus2:C", "kV, Phase Voltage"
      n_U_bus2_N = "n:U:bus2:N", "kV, Phase Voltage"
      n_ul_bus1_A = "n:ul:bus1:A", "p.u., Line-Line Voltage"
      n_ul_bus1_B = "n:ul:bus1:B", "p.u., Line-Line Voltage"
      n_ul_bus1_C = "n:ul:bus1:C", "p.u., Line-Line Voltage"
      n_ul_bus1_N = "n:ul:bus1:N", "p.u., Line-Line Voltage"
      n_ul_bus2_A = "n:ul:bus2:A", "p.u., Line-Line Voltage"
      n_ul_bus2_B = "n:ul:bus2:B", "p.u., Line-Line Voltage"
      n_ul_bus2_C = "n:ul:bus2:C", "p.u., Line-Line Voltage"
      n_ul_bus2_N = "n:ul:bus2:N", "p.u., Line-Line Voltage"
      n_ulpc_bus1_A = "n:ulpc:bus1:A", "%, Line-Line Voltage"
      n_ulpc_bus1_B = "n:ulpc:bus1:B", "%, Line-Line Voltage"
      n_ulpc_bus1_C = "n:ulpc:bus1:C", "%, Line-Line Voltage"
      n_ulpc_bus1_N = "n:ulpc:bus1:N", "%, Line-Line Voltage"
      n_ulpc_bus2_A = "n:ulpc:bus2:A", "%, Line-Line Voltage"
      n_ulpc_bus2_B = "n:ulpc:bus2:B", "%, Line-Line Voltage"
      n_ulpc_bus2_C = "n:ulpc:bus2:C", "%, Line-Line Voltage"
      n_ulpc_bus2_N = "n:ulpc:bus2:N", "%, Line-Line Voltage"
      n_Ul_bus1_A = "n:Ul:bus1:A", "kV, Line-Line Voltage"
      n_Ul_bus1_B = "n:Ul:bus1:B", "kV, Line-Line Voltage"
      n_Ul_bus1_C = "n:Ul:bus1:C", "kV, Line-Line Voltage"
      n_Ul_bus1_N = "n:Ul:bus1:N", "kV, Line-Line Voltage"
      n_Ul_bus2_A = "n:Ul:bus2:A", "kV, Line-Line Voltage"
      n_Ul_bus2_B = "n:Ul:bus2:B", "kV, Line-Line Voltage"
      n_Ul_bus2_C = "n:Ul:bus2:C", "kV, Line-Line Voltage"
      n_Ul_bus2_N = "n:Ul:bus2:N", "kV, Line-Line Voltage"
      n_u0_bus1 = "n:u0:bus1", "p.u., Zero-Sequence-Voltage"
      n_u0_bus2 = "n:u0:bus2", "p.u., Zero-Sequence-Voltage"
      n_U0_bus1 = "n:U0:bus1", "kV, Zero-Sequence-Voltage"
      n_U0_bus2 = "n:U0:bus2", "kV, Zero-Sequence-Voltage"
      n_U0x3_bus1 = "n:U0x3:bus1", "kV, 3*U0"
      n_U0x3_bus2 = "n:U0x3:bus2", "kV, 3*U0"
      n_u1r_bus1 = "n:u1r:bus1", "p.u., Voltage Phasor, Real Part"
      n_u1r_bus2 = "n:u1r:bus2", "p.u., Voltage Phasor, Real Part"
      n_u1i_bus1 = "n:u1i:bus1", "p.u., Voltage Phasor, Imaginary Part"
      n_u1i_bus2 = "n:u1i:bus2", "p.u., Voltage Phasor, Imaginary Part"
      n_u1_bus1 = "n:u1:bus1", "p.u., Voltage Phasor, Magnitude"
      n_u1_bus2 = "n:u1:bus2", "p.u., Voltage Phasor, Magnitude"
      n_u1pc_bus1 = "n:u1pc:bus1", "%, Voltage Phasor, Magnitude"
      n_u1pc_bus2 = "n:u1pc:bus2", "%, Voltage Phasor, Magnitude"
      n_U1_bus1 = "n:U1:bus1", "kV, Voltage Phasor, Magnitude"
      n_U1_bus2 = "n:U1:bus2", "kV, Voltage Phasor, Magnitude"
      n_phiu1_bus1 = "n:phiu1:bus1", "deg, Voltage Phasor, Angle"
      n_phiu1_bus2 = "n:phiu1:bus2", "deg, Voltage Phasor, Angle"
      n_U1l_bus1 = "n:U1l:bus1", "kV, Line-Line Voltage Phasor, Magnitude"
      n_U1l_bus2 = "n:U1l:bus2", "kV, Line-Line Voltage Phasor, Magnitude"
      n_uln_bus1_A = "n:uln:bus1:A", "p.u., Line-Neutral Voltage"
      n_uln_bus1_B = "n:uln:bus1:B", "p.u., Line-Neutral Voltage"
      n_uln_bus1_C = "n:uln:bus1:C", "p.u., Line-Neutral Voltage"
      n_uln_bus1_N = "n:uln:bus1:N", "p.u., Line-Neutral Voltage"
      n_uln_bus2_A = "n:uln:bus2:A", "p.u., Line-Neutral Voltage"
      n_uln_bus2_B = "n:uln:bus2:B", "p.u., Line-Neutral Voltage"
      n_uln_bus2_C = "n:uln:bus2:C", "p.u., Line-Neutral Voltage"
      n_uln_bus2_N = "n:uln:bus2:N", "p.u., Line-Neutral Voltage"
      n_Uln_bus1_A = "n:Uln:bus1:A", "kV, Line-Neutral Voltage"
      n_Uln_bus1_B = "n:Uln:bus1:B", "kV, Line-Neutral Voltage"
      n_Uln_bus1_C = "n:Uln:bus1:C", "kV, Line-Neutral Voltage"
      n_Uln_bus1_N = "n:Uln:bus1:N", "kV, Line-Neutral Voltage"
      n_Uln_bus2_A = "n:Uln:bus2:A", "kV, Line-Neutral Voltage"
      n_Uln_bus2_B = "n:Uln:bus2:B", "kV, Line-Neutral Voltage"
      n_Uln_bus2_C = "n:Uln:bus2:C", "kV, Line-Neutral Voltage"
      n_Uln_bus2_N = "n:Uln:bus2:N", "kV, Line-Neutral Voltage"
      n_un_bus1 = "n:un:bus1", "p.u., Neutral-Ground Voltage"
      n_un_bus2 = "n:un:bus2", "p.u., Neutral-Ground Voltage"
      n_Un_bus1 = "n:Un:bus1", "kV, Neutral-Ground Voltage"
      n_Un_bus2 = "n:Un:bus2", "kV, Neutral-Ground Voltage"
      n_ishc_bus1_A = "n:ishc:bus1:A", "p.u., Short-Circuit Current"
      n_ishc_bus1_B = "n:ishc:bus1:B", "p.u., Short-Circuit Current"
      n_ishc_bus1_C = "n:ishc:bus1:C", "p.u., Short-Circuit Current"
      n_ishc_bus1_N = "n:ishc:bus1:N", "p.u., Short-Circuit Current"
      n_ishc_bus2_A = "n:ishc:bus2:A", "p.u., Short-Circuit Current"
      n_ishc_bus2_B = "n:ishc:bus2:B", "p.u., Short-Circuit Current"
      n_ishc_bus2_C = "n:ishc:bus2:C", "p.u., Short-Circuit Current"
      n_ishc_bus2_N = "n:ishc:bus2:N", "p.u., Short-Circuit Current"
      n_Ishc_bus1_A = "n:Ishc:bus1:A", "kA, Short-Circuit Current"
      n_Ishc_bus1_B = "n:Ishc:bus1:B", "kA, Short-Circuit Current"
      n_Ishc_bus1_C = "n:Ishc:bus1:C", "kA, Short-Circuit Current"
      n_Ishc_bus1_N = "n:Ishc:bus1:N", "kA, Short-Circuit Current"
      n_Ishc_bus2_A = "n:Ishc:bus2:A", "kA, Short-Circuit Current"
      n_Ishc_bus2_B = "n:Ishc:bus2:B", "kA, Short-Circuit Current"
      n_Ishc_bus2_C = "n:Ishc:bus2:C", "kA, Short-Circuit Current"
      n_Ishc_bus2_N = "n:Ishc:bus2:N", "kA, Short-Circuit Current"
      n_inshc_bus1 = "n:inshc:bus1", "p.u., Neutral Short-Circuit Current, magnitude"
      n_inshc_bus2 = "n:inshc:bus2", "p.u., Neutral Short-Circuit Current, magnitude"
      n_Inshc_bus1 = "n:Inshc:bus1", "kA, Neutral Short-Circuit Current, magnitude"
      n_Inshc_bus2 = "n:Inshc:bus2", "kA, Neutral Short-Circuit Current, magnitude"
      n_fe_bus1 = "n:fe:bus1", "p.u., Electrical Frequency"
      n_fe_bus2 = "n:fe:bus2", "p.u., Electrical Frequency"
      n_frdev_bus1 = "n:frdev:bus1", "Hz, average frequency"
      n_frdev_bus2 = "n:frdev:bus2", "Hz, average frequency"
      n_dfehz_bus1 = "n:dfehz:bus1", "Hz, Deviation of the El. Frequency"
      n_dfehz_bus2 = "n:dfehz:bus2", "Hz, Deviation of the El. Frequency"
      n_fehz_bus1 = "n:fehz:bus1", "Hz, Electrical Frequency"
      n_fehz_bus2 = "n:fehz:bus2", "Hz, Electrical Frequency"
      n_frnom_bus1 = "n:frnom:bus1", "Hz, Nominal Frequency"
      n_frnom_bus2 = "n:frnom:bus2", "Hz, Nominal Frequency"
      s_iLA = "s:iLA", "p.u., OUT, Current through inductance, Phase A, continuous, double"
      s_iLB = "s:iLB", "p.u., OUT, Current through inductance, Phase B, continuous, double"
      s_iLC = "s:iLC", "p.u., OUT, Current through inductance, Phase C, continuous, double"
      s_iLA_dt = "s:iLA:dt", "p.u./s, OUT, Current through inductance, Phase A, derivative continuous, double"
      s_iLB_dt = "s:iLB:dt", "p.u./s, OUT, Current through inductance, Phase B, derivative continuous, double"
      s_iLC_dt = "s:iLC:dt", "p.u./s, OUT, Current through inductance, Phase C, derivative continuous, double"
      c_rline_l = "c:rline_l", "p.u., Resistance matrix"
      c_lline_l = "c:lline_l", "p.u., Inductance matrix"
      c_gline_l = "c:gline_l", "p.u., Conductance matrix"
      c_cline_l = "c:cline_l", "p.u., Capacitance matrix"
      c_frnom = "c:frnom", "Hz, Nominal frequency"
      c_hpi = "c:hpi", "rad/s, Nominal angular frequency"
      c_rtfac = "c:rtfac", "Temperature Factor"
      e_i_model = "e:i_model", "Line Model"
      e_ci_model = "e:ci_model", "Line Model"
      e_lossModel_emt = "e:lossModel_emt", "Loss model"
      e_iopt_tem = "e:iopt_tem", "Temperature Dependency"
      e_ftau = "e:ftau", "Hz, Frequency for Travel-Time Estimation"
      e_fmin = "e:fmin", "Hz, Min. Frequency of Parameter Fitting"
      e_fmax = "e:fmax", "Hz, Max. Frequency of Parameter Fitting"
      e_tolBode = "e:tolBode", "%, Tolerance for Bode Approximation"
      e_fd_model = "e:fd_model", "Line Model"
      e_kz0 = "e:kz0", "Ohm, Surge Impedance, HF, Mode 0."
      e_a0dc = "e:a0dc", "p.u., Wave Propagation, DC, Mode 0"
      e_c_Ta0 = "e:c_Ta0", "us, Travel Time, Mode 0"
      e_kz1 = "e:kz1", "Ohm, Surge Impedance, HF, Mode 1."
      e_a1dc = "e:a1dc", "p.u., Wave Propagation, DC, Mode 1"
      e_c_Ta1 = "e:c_Ta1", "us, Travel Time, Mode 1"
      e_kz2 = "e:kz2", "Ohm, Surge Impedance, HF, Mode 2."
      e_a2dc = "e:a2dc", "p.u., Wave Propagation, DC, Mode 2"
      e_c_Ta2 = "e:c_Ta2", "us, Travel Time, Mode 2"
      e_c_pz0 = "e:c_pz0", "Z0, N° Poles/Zeros"
      e_c_pa0 = "e:c_pa0", "A0, N° Poles"
      e_c_za0 = "e:c_za0", "A0, N° Zeros"
      e_c_pz1 = "e:c_pz1", "Z1, N° Poles/Zeros"
      e_c_pa1 = "e:c_pa1", "A1, N° Poles"
      e_c_za1 = "e:c_za1", "A1, N° Zeros"
      e_c_pz2 = "e:c_pz2", "Z2, N° Poles/Zeros"
      e_c_pa2 = "e:c_pa2", "A2, N° Poles"
      e_c_za2 = "e:c_za2", "A2, N° Zeros"
      e_maxErrVFY = "e:maxErrVFY", "S, RMS error"
      e_maxErrVFA = "e:maxErrVFA", "RMS error"
      e_iterVFY = "e:iterVFY", "Max. number of iterations"
      e_iterVFA = "e:iterVFA", "Max. number of iterations"
      e_polesVFY = "e:polesVFY", "Max. number of poles"
      e_polesVFA = "e:polesVFA", "Max. number of poles"
      e_methodVFY = "e:methodVFY", "Number of poles"
      e_methodVFA = "e:methodVFA", "Number of poles"
      e_stMarginY = "e:stMarginY", "Stability margin"
      e_stMarginA = "e:stMarginA", "Stability margin"
      e_tuneDelays = "e:tuneDelays", "Fine-tune delays"
      e_dcCorr = "e:dcCorr", "Apply DC correction"
      e_outputPoles = "e:outputPoles", "Output poles"
      e_cdisp_col = "e:cdisp_col", "Display data for phase matrix index"
      e_cdisp_row = "e:cdisp_row", "Display data for phase matrix row"
      e_c_paULM = "e:c_paULM", "A, Number of poles:"
      e_c_aAtDC = "e:c_aAtDC", "p.u., Wave propagation, DC"
      e_c_pTadULM = "e:c_pTadULM", "us, Travel time"
      t_tmax = "t:tmax", "degC, Max. Operating Temperature"
      t_rline = "t:rline", "Ohm/km, AC-Resistance R'(20°C)"
      t_rline_tmax = "t:rline_tmax", "Ohm/km, AC-Resistance R' at max. operating temperature"
      t_alpha = "t:alpha", "1/K, Temperature Coefficient"
      t_mlei = "t:mlei", "Conductor Material"
      t_bline = "t:bline", "uS/km, Susceptance B'"
      t_cline = "t:cline", "uF/km, Capacitance C'"
      t_tline = "t:tline", "Ins. Factor"
      t_gline = "t:gline", "uS/km, Conductance G'"
      t_bline0 = "t:bline0", "uS/km, Susceptance B0'"
      t_cline0 = "t:cline0", "uF/km, Capacitance C0'"
      t_Ices = "t:Ices", "A/km, Earth-Fault Current"
      t_tline0 = "t:tline0", "Ins. Factor"
      t_gline0 = "t:gline0", "uS/km, Conductance G0'"
      t_bnline = "t:bnline", "uS/km, Susceptance Bn'"
      t_cnline = "t:cnline", "uF/km, Capacitance Cn'"
      t_bpnline = "t:bpnline", "uS/km, Susceptance Bpn'"
      t_cpnline = "t:cpnline", "uF/km, Capacitance Cpn'"
      t_fcharR1 = "t:fcharR1", "ChaPol,ChaVec,ChaMat, R1'(AC)(f)"
      t_lline = "t:lline", "mH/km, Inductance L'"
      t_fcharL1 = "t:fcharL1", "ChaPol,ChaVec,ChaMat, L1'(f)"
      t_rline0 = "t:rline0", "Ohm/km, AC-Resistance R0'"
      t_fcharR0 = "t:fcharR0", "ChaPol,ChaVec,ChaMat, R0'(AC)(f)"
      t_lline0 = "t:lline0", "mH/km, Inductance L0'"
      t_fcharL0 = "t:fcharL0", "ChaPol,ChaVec,ChaMat, L0'(f)"
      t_fcharC1 = "t:fcharC1", "ChaPol,ChaVec,ChaMat, C1'(f)"
      t_fcharC0 = "t:fcharC0", "ChaPol,ChaVec,ChaMat, C0'(f)"

    class ElmLod(Enum):
      _init_ = 'value __doc__'

      m_u0_bus1 = "m:u0:bus1", "p.u., Zero-Sequence-Voltage"
      m_u1r_bus1 = "m:u1r:bus1", "p.u., Voltage Phasor, Real Part"
      m_u1i_bus1 = "m:u1i:bus1", "p.u., Voltage Phasor, Imaginary Part"
      m_u1_bus1 = "m:u1:bus1", "p.u., Voltage Phasor, Magnitude"
      m_U1_bus1 = "m:U1:bus1", "kV, Line-Ground Voltage Phasor, Magnitude"
      m_U1l_bus1 = "m:U1l:bus1", "kV, Line-Line Voltage Phasor, Magnitude"
      m_phiu1_bus1 = "m:phiu1:bus1", "deg, Voltage Phasor, Angle"
      m_I_bus1_A = "m:I:bus1:A", "kA, Phase Current"
      m_I_bus1_B = "m:I:bus1:B", "kA, Phase Current"
      m_I_bus1_C = "m:I:bus1:C", "kA, Phase Current"
      m_I_bus1_N = "m:I:bus1:N", "kA, Phase Current"
      m_i0_bus1 = "m:i0:bus1", "p.u., Zero-Sequence Current"
      m_I0_bus1 = "m:I0:bus1", "kA, Zero-Sequence Current"
      m_I0x3_bus1 = "m:I0x3:bus1", "kA, 3*I0"
      m_i1r_bus1 = "m:i1r:bus1", "p.u., Current Phasor, Real Part"
      m_i1i_bus1 = "m:i1i:bus1", "p.u., Current Phasor, Imaginary Part"
      m_i1_bus1 = "m:i1:bus1", "p.u., Current Phasor, Magnitude"
      m_I1_bus1 = "m:I1:bus1", "kA, Current Phasor, Magnitude"
      m_phii1_bus1 = "m:phii1:bus1", "deg, Current Phasor, Angle"
      m_P_bus1_A = "m:P:bus1:A", "MW, Active Power"
      m_P_bus1_B = "m:P:bus1:B", "MW, Active Power"
      m_P_bus1_C = "m:P:bus1:C", "MW, Active Power"
      m_P_bus1_N = "m:P:bus1:N", "MW, Active Power"
      m_Psum_bus1 = "m:Psum:bus1", "MW, Power-Phasor, Active Power"
      m_Qsum_bus1 = "m:Qsum:bus1", "Mvar, Power Phasor, Reactive Part"
      m_Ssum_bus1 = "m:Ssum:bus1", "MVA, Power Phasor, Apparent Power"
      m_cosphi1_bus1 = "m:cosphi1:bus1", "Power-Phasor, Power-Factor"
      m_i1P_bus1 = "m:i1P:bus1", "p.u., Current Phasor, Active Current"
      m_i1Q_bus1 = "m:i1Q:bus1", "p.u., Current Phasor, Reactive Current"
      m_I1P_bus1 = "m:I1P:bus1", "kA, Current Phasor, Active Current"
      m_I1Q_bus1 = "m:I1Q:bus1", "kA, Current Phasor, Reactive Current"
      m_u_bus1_A = "m:u:bus1:A", "p.u., Phase Voltage"
      m_u_bus1_B = "m:u:bus1:B", "p.u., Phase Voltage"
      m_u_bus1_C = "m:u:bus1:C", "p.u., Phase Voltage"
      m_u_bus1_N = "m:u:bus1:N", "p.u., Phase Voltage"
      m_i_bus1_A = "m:i:bus1:A", "p.u., Phase Current"
      m_i_bus1_B = "m:i:bus1:B", "p.u., Phase Current"
      m_i_bus1_C = "m:i:bus1:C", "p.u., Phase Current"
      m_i_bus1_N = "m:i:bus1:N", "p.u., Phase Current"
      m_phiu1i1_bus1 = "m:phiu1i1:bus1", "deg, Angle between Voltage and Current Phasor"
      m_Tfct_bus1 = "m:Tfct:bus1", "s, Fault Clearing Time"
      m_TfctPh_bus1_A = "m:TfctPh:bus1:A", "s, Fault Clearing Time"
      m_TfctPh_bus1_B = "m:TfctPh:bus1:B", "s, Fault Clearing Time"
      m_TfctPh_bus1_C = "m:TfctPh:bus1:C", "s, Fault Clearing Time"
      m_TfctPh_bus1_N = "m:TfctPh:bus1:N", "s, Fault Clearing Time"
      n_u_bus1_A = "n:u:bus1:A", "p.u., Phase Voltage"
      n_u_bus1_B = "n:u:bus1:B", "p.u., Phase Voltage"
      n_u_bus1_C = "n:u:bus1:C", "p.u., Phase Voltage"
      n_u_bus1_N = "n:u:bus1:N", "p.u., Phase Voltage"
      n_upc_bus1_A = "n:upc:bus1:A", "%, Phase Voltage"
      n_upc_bus1_B = "n:upc:bus1:B", "%, Phase Voltage"
      n_upc_bus1_C = "n:upc:bus1:C", "%, Phase Voltage"
      n_upc_bus1_N = "n:upc:bus1:N", "%, Phase Voltage"
      n_U_bus1_A = "n:U:bus1:A", "kV, Phase Voltage"
      n_U_bus1_B = "n:U:bus1:B", "kV, Phase Voltage"
      n_U_bus1_C = "n:U:bus1:C", "kV, Phase Voltage"
      n_U_bus1_N = "n:U:bus1:N", "kV, Phase Voltage"
      n_ul_bus1_A = "n:ul:bus1:A", "p.u., Line-Line Voltage"
      n_ul_bus1_B = "n:ul:bus1:B", "p.u., Line-Line Voltage"
      n_ul_bus1_C = "n:ul:bus1:C", "p.u., Line-Line Voltage"
      n_ul_bus1_N = "n:ul:bus1:N", "p.u., Line-Line Voltage"
      n_ulpc_bus1_A = "n:ulpc:bus1:A", "%, Line-Line Voltage"
      n_ulpc_bus1_B = "n:ulpc:bus1:B", "%, Line-Line Voltage"
      n_ulpc_bus1_C = "n:ulpc:bus1:C", "%, Line-Line Voltage"
      n_ulpc_bus1_N = "n:ulpc:bus1:N", "%, Line-Line Voltage"
      n_Ul_bus1_A = "n:Ul:bus1:A", "kV, Line-Line Voltage"
      n_Ul_bus1_B = "n:Ul:bus1:B", "kV, Line-Line Voltage"
      n_Ul_bus1_C = "n:Ul:bus1:C", "kV, Line-Line Voltage"
      n_Ul_bus1_N = "n:Ul:bus1:N", "kV, Line-Line Voltage"
      n_u0_bus1 = "n:u0:bus1", "p.u., Zero-Sequence-Voltage"
      n_U0_bus1 = "n:U0:bus1", "kV, Zero-Sequence-Voltage"
      n_U0x3_bus1 = "n:U0x3:bus1", "kV, 3*U0"
      n_u1r_bus1 = "n:u1r:bus1", "p.u., Voltage Phasor, Real Part"
      n_u1i_bus1 = "n:u1i:bus1", "p.u., Voltage Phasor, Imaginary Part"
      n_u1_bus1 = "n:u1:bus1", "p.u., Voltage Phasor, Magnitude"
      n_u1pc_bus1 = "n:u1pc:bus1", "%, Voltage Phasor, Magnitude"
      n_U1_bus1 = "n:U1:bus1", "kV, Voltage Phasor, Magnitude"
      n_phiu1_bus1 = "n:phiu1:bus1", "deg, Voltage Phasor, Angle"
      n_U1l_bus1 = "n:U1l:bus1", "kV, Line-Line Voltage Phasor, Magnitude"
      n_uln_bus1_A = "n:uln:bus1:A", "p.u., Line-Neutral Voltage"
      n_uln_bus1_B = "n:uln:bus1:B", "p.u., Line-Neutral Voltage"
      n_uln_bus1_C = "n:uln:bus1:C", "p.u., Line-Neutral Voltage"
      n_uln_bus1_N = "n:uln:bus1:N", "p.u., Line-Neutral Voltage"
      n_Uln_bus1_A = "n:Uln:bus1:A", "kV, Line-Neutral Voltage"
      n_Uln_bus1_B = "n:Uln:bus1:B", "kV, Line-Neutral Voltage"
      n_Uln_bus1_C = "n:Uln:bus1:C", "kV, Line-Neutral Voltage"
      n_Uln_bus1_N = "n:Uln:bus1:N", "kV, Line-Neutral Voltage"
      n_un_bus1 = "n:un:bus1", "p.u., Neutral-Ground Voltage"
      n_Un_bus1 = "n:Un:bus1", "kV, Neutral-Ground Voltage"
      n_ishc_bus1_A = "n:ishc:bus1:A", "p.u., Short-Circuit Current"
      n_ishc_bus1_B = "n:ishc:bus1:B", "p.u., Short-Circuit Current"
      n_ishc_bus1_C = "n:ishc:bus1:C", "p.u., Short-Circuit Current"
      n_ishc_bus1_N = "n:ishc:bus1:N", "p.u., Short-Circuit Current"
      n_Ishc_bus1_A = "n:Ishc:bus1:A", "kA, Short-Circuit Current"
      n_Ishc_bus1_B = "n:Ishc:bus1:B", "kA, Short-Circuit Current"
      n_Ishc_bus1_C = "n:Ishc:bus1:C", "kA, Short-Circuit Current"
      n_Ishc_bus1_N = "n:Ishc:bus1:N", "kA, Short-Circuit Current"
      n_inshc_bus1 = "n:inshc:bus1", "p.u., Neutral Short-Circuit Current, magnitude"
      n_Inshc_bus1 = "n:Inshc:bus1", "kA, Neutral Short-Circuit Current, magnitude"
      n_fe_bus1 = "n:fe:bus1", "p.u., Electrical Frequency"
      n_frdev_bus1 = "n:frdev:bus1", "Hz, average frequency"
      n_dfehz_bus1 = "n:dfehz:bus1", "Hz, Deviation of the El. Frequency"
      n_fehz_bus1 = "n:fehz:bus1", "Hz, Electrical Frequency"
      n_frnom_bus1 = "n:frnom:bus1", "Hz, Nominal Frequency"
      c_frnom = "c:frnom", "Hz, Nominal Frequency"
      c_hpi = "c:hpi", "rad/s, Nominal angular frequency"
      e_mode_inp = "e:mode_inp", "Input Mode"
      e_i_sym = "e:i_sym", "Balanced/Unbalanced"
      e_plini = "e:plini", "MW, Active Power"
      e_qlini = "e:qlini", "Mvar, Reactive Power"
      e_slini = "e:slini", "MVA, Apparent Power"
      e_ilini = "e:ilini", "kA, Current"
      e_coslini = "e:coslini", "Power Factor"
      e_pf_recap = "e:pf_recap", "Power Factor"
      e_p_direc = "e:p_direc", "Power Direction"
      e_u0 = "e:u0", "p.u., Voltage"
      e_scale0 = "e:scale0", "Scaling Factor"
      e_plinir = "e:plinir", "MW, Active Power"
      e_qlinir = "e:qlinir", "Mvar, Reactive Power"
      e_slinir = "e:slinir", "MVA, Apparent Power"
      e_ilinir = "e:ilinir", "kA, Current"
      e_coslinir = "e:coslinir", "Power Factor"
      e_pf_recapr = "e:pf_recapr", "Power Factor"
      e_p_direcr = "e:p_direcr", "Power Direction"
      e_plinis = "e:plinis", "MW, Active Power"
      e_qlinis = "e:qlinis", "Mvar, Reactive Power"
      e_slinis = "e:slinis", "MVA, Apparent Power"
      e_ilinis = "e:ilinis", "kA, Current"
      e_coslinis = "e:coslinis", "Power Factor"
      e_pf_recaps = "e:pf_recaps", "Power Factor"
      e_p_direcs = "e:p_direcs", "Power Direction"
      e_plinit = "e:plinit", "MW, Active Power"
      e_qlinit = "e:qlinit", "Mvar, Reactive Power"
      e_slinit = "e:slinit", "MVA, Apparent Power"
      e_ilinit = "e:ilinit", "kA, Current"
      e_coslinit = "e:coslinit", "Power Factor"
      e_pf_recapt = "e:pf_recapt", "Power Factor"
      e_p_direct = "e:p_direct", "Power Direction"

    class ElmLodlv(Enum):
      _init_ = 'value __doc__'

      m_u0_bus1 = "m:u0:bus1", "p.u., Zero-Sequence-Voltage"
      m_u1r_bus1 = "m:u1r:bus1", "p.u., Voltage Phasor, Real Part"
      m_u1i_bus1 = "m:u1i:bus1", "p.u., Voltage Phasor, Imaginary Part"
      m_u1_bus1 = "m:u1:bus1", "p.u., Voltage Phasor, Magnitude"
      m_U1_bus1 = "m:U1:bus1", "kV, Line-Ground Voltage Phasor, Magnitude"
      m_U1l_bus1 = "m:U1l:bus1", "kV, Line-Line Voltage Phasor, Magnitude"
      m_phiu1_bus1 = "m:phiu1:bus1", "deg, Voltage Phasor, Angle"
      m_I_bus1_A = "m:I:bus1:A", "kA, Phase Current"
      m_I_bus1_B = "m:I:bus1:B", "kA, Phase Current"
      m_I_bus1_C = "m:I:bus1:C", "kA, Phase Current"
      m_I_bus1_N = "m:I:bus1:N", "kA, Phase Current"
      m_i0_bus1 = "m:i0:bus1", "p.u., Zero-Sequence Current"
      m_I0_bus1 = "m:I0:bus1", "kA, Zero-Sequence Current"
      m_I0x3_bus1 = "m:I0x3:bus1", "kA, 3*I0"
      m_i1r_bus1 = "m:i1r:bus1", "p.u., Current Phasor, Real Part"
      m_i1i_bus1 = "m:i1i:bus1", "p.u., Current Phasor, Imaginary Part"
      m_i1_bus1 = "m:i1:bus1", "p.u., Current Phasor, Magnitude"
      m_I1_bus1 = "m:I1:bus1", "kA, Current Phasor, Magnitude"
      m_phii1_bus1 = "m:phii1:bus1", "deg, Current Phasor, Angle"
      m_P_bus1_A = "m:P:bus1:A", "MW, Active Power"
      m_P_bus1_B = "m:P:bus1:B", "MW, Active Power"
      m_P_bus1_C = "m:P:bus1:C", "MW, Active Power"
      m_P_bus1_N = "m:P:bus1:N", "MW, Active Power"
      m_Psum_bus1 = "m:Psum:bus1", "MW, Power-Phasor, Active Power"
      m_Qsum_bus1 = "m:Qsum:bus1", "Mvar, Power Phasor, Reactive Part"
      m_Ssum_bus1 = "m:Ssum:bus1", "MVA, Power Phasor, Apparent Power"
      m_cosphi1_bus1 = "m:cosphi1:bus1", "Power-Phasor, Power-Factor"
      m_i1P_bus1 = "m:i1P:bus1", "p.u., Current Phasor, Active Current"
      m_i1Q_bus1 = "m:i1Q:bus1", "p.u., Current Phasor, Reactive Current"
      m_I1P_bus1 = "m:I1P:bus1", "kA, Current Phasor, Active Current"
      m_I1Q_bus1 = "m:I1Q:bus1", "kA, Current Phasor, Reactive Current"
      m_u_bus1_A = "m:u:bus1:A", "p.u., Phase Voltage"
      m_u_bus1_B = "m:u:bus1:B", "p.u., Phase Voltage"
      m_u_bus1_C = "m:u:bus1:C", "p.u., Phase Voltage"
      m_u_bus1_N = "m:u:bus1:N", "p.u., Phase Voltage"
      m_i_bus1_A = "m:i:bus1:A", "p.u., Phase Current"
      m_i_bus1_B = "m:i:bus1:B", "p.u., Phase Current"
      m_i_bus1_C = "m:i:bus1:C", "p.u., Phase Current"
      m_i_bus1_N = "m:i:bus1:N", "p.u., Phase Current"
      m_phiu1i1_bus1 = "m:phiu1i1:bus1", "deg, Angle between Voltage and Current Phasor"
      m_Tfct_bus1 = "m:Tfct:bus1", "s, Fault Clearing Time"
      m_TfctPh_bus1_A = "m:TfctPh:bus1:A", "s, Fault Clearing Time"
      m_TfctPh_bus1_B = "m:TfctPh:bus1:B", "s, Fault Clearing Time"
      m_TfctPh_bus1_C = "m:TfctPh:bus1:C", "s, Fault Clearing Time"
      m_TfctPh_bus1_N = "m:TfctPh:bus1:N", "s, Fault Clearing Time"
      n_u_bus1_A = "n:u:bus1:A", "p.u., Phase Voltage"
      n_u_bus1_B = "n:u:bus1:B", "p.u., Phase Voltage"
      n_u_bus1_C = "n:u:bus1:C", "p.u., Phase Voltage"
      n_u_bus1_N = "n:u:bus1:N", "p.u., Phase Voltage"
      n_upc_bus1_A = "n:upc:bus1:A", "%, Phase Voltage"
      n_upc_bus1_B = "n:upc:bus1:B", "%, Phase Voltage"
      n_upc_bus1_C = "n:upc:bus1:C", "%, Phase Voltage"
      n_upc_bus1_N = "n:upc:bus1:N", "%, Phase Voltage"
      n_U_bus1_A = "n:U:bus1:A", "kV, Phase Voltage"
      n_U_bus1_B = "n:U:bus1:B", "kV, Phase Voltage"
      n_U_bus1_C = "n:U:bus1:C", "kV, Phase Voltage"
      n_U_bus1_N = "n:U:bus1:N", "kV, Phase Voltage"
      n_ul_bus1_A = "n:ul:bus1:A", "p.u., Line-Line Voltage"
      n_ul_bus1_B = "n:ul:bus1:B", "p.u., Line-Line Voltage"
      n_ul_bus1_C = "n:ul:bus1:C", "p.u., Line-Line Voltage"
      n_ul_bus1_N = "n:ul:bus1:N", "p.u., Line-Line Voltage"
      n_ulpc_bus1_A = "n:ulpc:bus1:A", "%, Line-Line Voltage"
      n_ulpc_bus1_B = "n:ulpc:bus1:B", "%, Line-Line Voltage"
      n_ulpc_bus1_C = "n:ulpc:bus1:C", "%, Line-Line Voltage"
      n_ulpc_bus1_N = "n:ulpc:bus1:N", "%, Line-Line Voltage"
      n_Ul_bus1_A = "n:Ul:bus1:A", "kV, Line-Line Voltage"
      n_Ul_bus1_B = "n:Ul:bus1:B", "kV, Line-Line Voltage"
      n_Ul_bus1_C = "n:Ul:bus1:C", "kV, Line-Line Voltage"
      n_Ul_bus1_N = "n:Ul:bus1:N", "kV, Line-Line Voltage"
      n_u0_bus1 = "n:u0:bus1", "p.u., Zero-Sequence-Voltage"
      n_U0_bus1 = "n:U0:bus1", "kV, Zero-Sequence-Voltage"
      n_U0x3_bus1 = "n:U0x3:bus1", "kV, 3*U0"
      n_u1r_bus1 = "n:u1r:bus1", "p.u., Voltage Phasor, Real Part"
      n_u1i_bus1 = "n:u1i:bus1", "p.u., Voltage Phasor, Imaginary Part"
      n_u1_bus1 = "n:u1:bus1", "p.u., Voltage Phasor, Magnitude"
      n_u1pc_bus1 = "n:u1pc:bus1", "%, Voltage Phasor, Magnitude"
      n_U1_bus1 = "n:U1:bus1", "kV, Voltage Phasor, Magnitude"
      n_phiu1_bus1 = "n:phiu1:bus1", "deg, Voltage Phasor, Angle"
      n_U1l_bus1 = "n:U1l:bus1", "kV, Line-Line Voltage Phasor, Magnitude"
      n_uln_bus1_A = "n:uln:bus1:A", "p.u., Line-Neutral Voltage"
      n_uln_bus1_B = "n:uln:bus1:B", "p.u., Line-Neutral Voltage"
      n_uln_bus1_C = "n:uln:bus1:C", "p.u., Line-Neutral Voltage"
      n_uln_bus1_N = "n:uln:bus1:N", "p.u., Line-Neutral Voltage"
      n_Uln_bus1_A = "n:Uln:bus1:A", "kV, Line-Neutral Voltage"
      n_Uln_bus1_B = "n:Uln:bus1:B", "kV, Line-Neutral Voltage"
      n_Uln_bus1_C = "n:Uln:bus1:C", "kV, Line-Neutral Voltage"
      n_Uln_bus1_N = "n:Uln:bus1:N", "kV, Line-Neutral Voltage"
      n_un_bus1 = "n:un:bus1", "p.u., Neutral-Ground Voltage"
      n_Un_bus1 = "n:Un:bus1", "kV, Neutral-Ground Voltage"
      n_ishc_bus1_A = "n:ishc:bus1:A", "p.u., Short-Circuit Current"
      n_ishc_bus1_B = "n:ishc:bus1:B", "p.u., Short-Circuit Current"
      n_ishc_bus1_C = "n:ishc:bus1:C", "p.u., Short-Circuit Current"
      n_ishc_bus1_N = "n:ishc:bus1:N", "p.u., Short-Circuit Current"
      n_Ishc_bus1_A = "n:Ishc:bus1:A", "kA, Short-Circuit Current"
      n_Ishc_bus1_B = "n:Ishc:bus1:B", "kA, Short-Circuit Current"
      n_Ishc_bus1_C = "n:Ishc:bus1:C", "kA, Short-Circuit Current"
      n_Ishc_bus1_N = "n:Ishc:bus1:N", "kA, Short-Circuit Current"
      n_inshc_bus1 = "n:inshc:bus1", "p.u., Neutral Short-Circuit Current, magnitude"
      n_Inshc_bus1 = "n:Inshc:bus1", "kA, Neutral Short-Circuit Current, magnitude"
      n_fe_bus1 = "n:fe:bus1", "p.u., Electrical Frequency"
      n_frdev_bus1 = "n:frdev:bus1", "Hz, average frequency"
      n_dfehz_bus1 = "n:dfehz:bus1", "Hz, Deviation of the El. Frequency"
      n_fehz_bus1 = "n:fehz:bus1", "Hz, Electrical Frequency"
      n_frnom_bus1 = "n:frnom:bus1", "Hz, Nominal Frequency"
      c_frnom = "c:frnom", "Hz, Nominal Frequency"
      c_hpi = "c:hpi", "rad/s, Nominal angular frequency"
      c_ncust = "c:ncust", "Number of Connected Customers"

    class ElmLodmv(Enum):
      _init_ = 'value __doc__'

      m_u0_bus1 = "m:u0:bus1", "p.u., Zero-Sequence-Voltage"
      m_u1r_bus1 = "m:u1r:bus1", "p.u., Voltage Phasor, Real Part"
      m_u1i_bus1 = "m:u1i:bus1", "p.u., Voltage Phasor, Imaginary Part"
      m_u1_bus1 = "m:u1:bus1", "p.u., Voltage Phasor, Magnitude"
      m_U1_bus1 = "m:U1:bus1", "kV, Line-Ground Voltage Phasor, Magnitude"
      m_U1l_bus1 = "m:U1l:bus1", "kV, Line-Line Voltage Phasor, Magnitude"
      m_phiu1_bus1 = "m:phiu1:bus1", "deg, Voltage Phasor, Angle"
      m_I_bus1_A = "m:I:bus1:A", "kA, Phase Current"
      m_I_bus1_B = "m:I:bus1:B", "kA, Phase Current"
      m_I_bus1_C = "m:I:bus1:C", "kA, Phase Current"
      m_I_bus1_N = "m:I:bus1:N", "kA, Phase Current"
      m_i0_bus1 = "m:i0:bus1", "p.u., Zero-Sequence Current"
      m_I0_bus1 = "m:I0:bus1", "kA, Zero-Sequence Current"
      m_I0x3_bus1 = "m:I0x3:bus1", "kA, 3*I0"
      m_i1r_bus1 = "m:i1r:bus1", "p.u., Current Phasor, Real Part"
      m_i1i_bus1 = "m:i1i:bus1", "p.u., Current Phasor, Imaginary Part"
      m_i1_bus1 = "m:i1:bus1", "p.u., Current Phasor, Magnitude"
      m_I1_bus1 = "m:I1:bus1", "kA, Current Phasor, Magnitude"
      m_phii1_bus1 = "m:phii1:bus1", "deg, Current Phasor, Angle"
      m_P_bus1_A = "m:P:bus1:A", "MW, Active Power"
      m_P_bus1_B = "m:P:bus1:B", "MW, Active Power"
      m_P_bus1_C = "m:P:bus1:C", "MW, Active Power"
      m_P_bus1_N = "m:P:bus1:N", "MW, Active Power"
      m_Psum_bus1 = "m:Psum:bus1", "MW, Power-Phasor, Active Power"
      m_Qsum_bus1 = "m:Qsum:bus1", "Mvar, Power Phasor, Reactive Part"
      m_Ssum_bus1 = "m:Ssum:bus1", "MVA, Power Phasor, Apparent Power"
      m_cosphi1_bus1 = "m:cosphi1:bus1", "Power-Phasor, Power-Factor"
      m_i1P_bus1 = "m:i1P:bus1", "p.u., Current Phasor, Active Current"
      m_i1Q_bus1 = "m:i1Q:bus1", "p.u., Current Phasor, Reactive Current"
      m_I1P_bus1 = "m:I1P:bus1", "kA, Current Phasor, Active Current"
      m_I1Q_bus1 = "m:I1Q:bus1", "kA, Current Phasor, Reactive Current"
      m_u_bus1_A = "m:u:bus1:A", "p.u., Phase Voltage"
      m_u_bus1_B = "m:u:bus1:B", "p.u., Phase Voltage"
      m_u_bus1_C = "m:u:bus1:C", "p.u., Phase Voltage"
      m_u_bus1_N = "m:u:bus1:N", "p.u., Phase Voltage"
      m_i_bus1_A = "m:i:bus1:A", "p.u., Phase Current"
      m_i_bus1_B = "m:i:bus1:B", "p.u., Phase Current"
      m_i_bus1_C = "m:i:bus1:C", "p.u., Phase Current"
      m_i_bus1_N = "m:i:bus1:N", "p.u., Phase Current"
      m_phiu1i1_bus1 = "m:phiu1i1:bus1", "deg, Angle between Voltage and Current Phasor"
      m_Tfct_bus1 = "m:Tfct:bus1", "s, Fault Clearing Time"
      m_TfctPh_bus1_A = "m:TfctPh:bus1:A", "s, Fault Clearing Time"
      m_TfctPh_bus1_B = "m:TfctPh:bus1:B", "s, Fault Clearing Time"
      m_TfctPh_bus1_C = "m:TfctPh:bus1:C", "s, Fault Clearing Time"
      m_TfctPh_bus1_N = "m:TfctPh:bus1:N", "s, Fault Clearing Time"
      n_u_bus1_A = "n:u:bus1:A", "p.u., Phase Voltage"
      n_u_bus1_B = "n:u:bus1:B", "p.u., Phase Voltage"
      n_u_bus1_C = "n:u:bus1:C", "p.u., Phase Voltage"
      n_u_bus1_N = "n:u:bus1:N", "p.u., Phase Voltage"
      n_upc_bus1_A = "n:upc:bus1:A", "%, Phase Voltage"
      n_upc_bus1_B = "n:upc:bus1:B", "%, Phase Voltage"
      n_upc_bus1_C = "n:upc:bus1:C", "%, Phase Voltage"
      n_upc_bus1_N = "n:upc:bus1:N", "%, Phase Voltage"
      n_U_bus1_A = "n:U:bus1:A", "kV, Phase Voltage"
      n_U_bus1_B = "n:U:bus1:B", "kV, Phase Voltage"
      n_U_bus1_C = "n:U:bus1:C", "kV, Phase Voltage"
      n_U_bus1_N = "n:U:bus1:N", "kV, Phase Voltage"
      n_ul_bus1_A = "n:ul:bus1:A", "p.u., Line-Line Voltage"
      n_ul_bus1_B = "n:ul:bus1:B", "p.u., Line-Line Voltage"
      n_ul_bus1_C = "n:ul:bus1:C", "p.u., Line-Line Voltage"
      n_ul_bus1_N = "n:ul:bus1:N", "p.u., Line-Line Voltage"
      n_ulpc_bus1_A = "n:ulpc:bus1:A", "%, Line-Line Voltage"
      n_ulpc_bus1_B = "n:ulpc:bus1:B", "%, Line-Line Voltage"
      n_ulpc_bus1_C = "n:ulpc:bus1:C", "%, Line-Line Voltage"
      n_ulpc_bus1_N = "n:ulpc:bus1:N", "%, Line-Line Voltage"
      n_Ul_bus1_A = "n:Ul:bus1:A", "kV, Line-Line Voltage"
      n_Ul_bus1_B = "n:Ul:bus1:B", "kV, Line-Line Voltage"
      n_Ul_bus1_C = "n:Ul:bus1:C", "kV, Line-Line Voltage"
      n_Ul_bus1_N = "n:Ul:bus1:N", "kV, Line-Line Voltage"
      n_u0_bus1 = "n:u0:bus1", "p.u., Zero-Sequence-Voltage"
      n_U0_bus1 = "n:U0:bus1", "kV, Zero-Sequence-Voltage"
      n_U0x3_bus1 = "n:U0x3:bus1", "kV, 3*U0"
      n_u1r_bus1 = "n:u1r:bus1", "p.u., Voltage Phasor, Real Part"
      n_u1i_bus1 = "n:u1i:bus1", "p.u., Voltage Phasor, Imaginary Part"
      n_u1_bus1 = "n:u1:bus1", "p.u., Voltage Phasor, Magnitude"
      n_u1pc_bus1 = "n:u1pc:bus1", "%, Voltage Phasor, Magnitude"
      n_U1_bus1 = "n:U1:bus1", "kV, Voltage Phasor, Magnitude"
      n_phiu1_bus1 = "n:phiu1:bus1", "deg, Voltage Phasor, Angle"
      n_U1l_bus1 = "n:U1l:bus1", "kV, Line-Line Voltage Phasor, Magnitude"
      n_uln_bus1_A = "n:uln:bus1:A", "p.u., Line-Neutral Voltage"
      n_uln_bus1_B = "n:uln:bus1:B", "p.u., Line-Neutral Voltage"
      n_uln_bus1_C = "n:uln:bus1:C", "p.u., Line-Neutral Voltage"
      n_uln_bus1_N = "n:uln:bus1:N", "p.u., Line-Neutral Voltage"
      n_Uln_bus1_A = "n:Uln:bus1:A", "kV, Line-Neutral Voltage"
      n_Uln_bus1_B = "n:Uln:bus1:B", "kV, Line-Neutral Voltage"
      n_Uln_bus1_C = "n:Uln:bus1:C", "kV, Line-Neutral Voltage"
      n_Uln_bus1_N = "n:Uln:bus1:N", "kV, Line-Neutral Voltage"
      n_un_bus1 = "n:un:bus1", "p.u., Neutral-Ground Voltage"
      n_Un_bus1 = "n:Un:bus1", "kV, Neutral-Ground Voltage"
      n_ishc_bus1_A = "n:ishc:bus1:A", "p.u., Short-Circuit Current"
      n_ishc_bus1_B = "n:ishc:bus1:B", "p.u., Short-Circuit Current"
      n_ishc_bus1_C = "n:ishc:bus1:C", "p.u., Short-Circuit Current"
      n_ishc_bus1_N = "n:ishc:bus1:N", "p.u., Short-Circuit Current"
      n_Ishc_bus1_A = "n:Ishc:bus1:A", "kA, Short-Circuit Current"
      n_Ishc_bus1_B = "n:Ishc:bus1:B", "kA, Short-Circuit Current"
      n_Ishc_bus1_C = "n:Ishc:bus1:C", "kA, Short-Circuit Current"
      n_Ishc_bus1_N = "n:Ishc:bus1:N", "kA, Short-Circuit Current"
      n_inshc_bus1 = "n:inshc:bus1", "p.u., Neutral Short-Circuit Current, magnitude"
      n_Inshc_bus1 = "n:Inshc:bus1", "kA, Neutral Short-Circuit Current, magnitude"
      n_fe_bus1 = "n:fe:bus1", "p.u., Electrical Frequency"
      n_frdev_bus1 = "n:frdev:bus1", "Hz, average frequency"
      n_dfehz_bus1 = "n:dfehz:bus1", "Hz, Deviation of the El. Frequency"
      n_fehz_bus1 = "n:fehz:bus1", "Hz, Electrical Frequency"
      n_frnom_bus1 = "n:frnom:bus1", "Hz, Nominal Frequency"
      c_Lr = "c:Lr", "p.u., Inductance, Phase a"
      c_Ls = "c:Ls", "p.u., Inductance, Phase b"
      c_Lt = "c:Lt", "p.u., Inductance, Phase c"
      c_Rr = "c:Rr", "p.u., Resistance, Phase a"
      c_Rs = "c:Rs", "p.u., Resistance, Phase b"
      c_Rt = "c:Rt", "p.u., Resistance, Phase c"
      c_Cr = "c:Cr", "p.u., Capacitance, Phase a"
      c_Cs = "c:Cs", "p.u., Capacitance, Phase b"
      c_Ct = "c:Ct", "p.u., Capacitance, Phase c"
      c_frnom = "c:frnom", "Hz, Nominal Frequency"
      c_hpi = "c:hpi", "rad/s, Nominal angular frequency"
      c_i_pure = "c:i_pure", "Purely inductive/capacitive"

    class ElmPvsys(Enum):
      _init_ = 'value __doc__'

      m_u0_bus1 = "m:u0:bus1", "p.u., Zero-Sequence-Voltage"
      m_u0_bus1n = "m:u0:bus1n", "p.u., Zero-Sequence-Voltage"
      m_u1r_bus1 = "m:u1r:bus1", "p.u., Voltage Phasor, Real Part"
      m_u1r_bus1n = "m:u1r:bus1n", "p.u., Voltage Phasor, Real Part"
      m_u1i_bus1 = "m:u1i:bus1", "p.u., Voltage Phasor, Imaginary Part"
      m_u1i_bus1n = "m:u1i:bus1n", "p.u., Voltage Phasor, Imaginary Part"
      m_u1_bus1 = "m:u1:bus1", "p.u., Voltage Phasor, Magnitude"
      m_u1_bus1n = "m:u1:bus1n", "p.u., Voltage Phasor, Magnitude"
      m_U1_bus1 = "m:U1:bus1", "kV, Line-Ground Voltage Phasor, Magnitude"
      m_U1_bus1n = "m:U1:bus1n", "kV, Line-Ground Voltage Phasor, Magnitude"
      m_U1l_bus1 = "m:U1l:bus1", "kV, Line-Line Voltage Phasor, Magnitude"
      m_U1l_bus1n = "m:U1l:bus1n", "kV, Line-Line Voltage Phasor, Magnitude"
      m_phiu1_bus1 = "m:phiu1:bus1", "deg, Voltage Phasor, Angle"
      m_phiu1_bus1n = "m:phiu1:bus1n", "deg, Voltage Phasor, Angle"
      m_I_bus1_A = "m:I:bus1:A", "kA, Phase Current"
      m_I_bus1_B = "m:I:bus1:B", "kA, Phase Current"
      m_I_bus1_C = "m:I:bus1:C", "kA, Phase Current"
      m_I_bus1_N = "m:I:bus1:N", "kA, Phase Current"
      m_I_bus1n = "m:I:bus1n", "kA, Phase Current"
      m_i0_bus1 = "m:i0:bus1", "p.u., Zero-Sequence Current"
      m_i0_bus1n = "m:i0:bus1n", "p.u., Zero-Sequence Current"
      m_I0_bus1 = "m:I0:bus1", "kA, Zero-Sequence Current"
      m_I0_bus1n = "m:I0:bus1n", "kA, Zero-Sequence Current"
      m_I0x3_bus1 = "m:I0x3:bus1", "kA, 3*I0"
      m_I0x3_bus1n = "m:I0x3:bus1n", "kA, 3*I0"
      m_i1r_bus1 = "m:i1r:bus1", "p.u., Current Phasor, Real Part"
      m_i1r_bus1n = "m:i1r:bus1n", "p.u., Current Phasor, Real Part"
      m_i1i_bus1 = "m:i1i:bus1", "p.u., Current Phasor, Imaginary Part"
      m_i1i_bus1n = "m:i1i:bus1n", "p.u., Current Phasor, Imaginary Part"
      m_i1_bus1 = "m:i1:bus1", "p.u., Current Phasor, Magnitude"
      m_i1_bus1n = "m:i1:bus1n", "p.u., Current Phasor, Magnitude"
      m_I1_bus1 = "m:I1:bus1", "kA, Current Phasor, Magnitude"
      m_I1_bus1n = "m:I1:bus1n", "kA, Current Phasor, Magnitude"
      m_phii1_bus1 = "m:phii1:bus1", "deg, Current Phasor, Angle"
      m_phii1_bus1n = "m:phii1:bus1n", "deg, Current Phasor, Angle"
      m_P_bus1_A = "m:P:bus1:A", "MW, Active Power"
      m_P_bus1_B = "m:P:bus1:B", "MW, Active Power"
      m_P_bus1_C = "m:P:bus1:C", "MW, Active Power"
      m_P_bus1_N = "m:P:bus1:N", "MW, Active Power"
      m_P_bus1n = "m:P:bus1n", "MW, Active Power"
      m_Psum_bus1 = "m:Psum:bus1", "MW, Power-Phasor, Active Power"
      m_Psum_bus1n = "m:Psum:bus1n", "MW, Power-Phasor, Active Power"
      m_Qsum_bus1 = "m:Qsum:bus1", "Mvar, Power Phasor, Reactive Part"
      m_Qsum_bus1n = "m:Qsum:bus1n", "Mvar, Power Phasor, Reactive Part"
      m_Ssum_bus1 = "m:Ssum:bus1", "MVA, Power Phasor, Apparent Power"
      m_Ssum_bus1n = "m:Ssum:bus1n", "MVA, Power Phasor, Apparent Power"
      m_cosphi1_bus1 = "m:cosphi1:bus1", "Power-Phasor, Power-Factor"
      m_cosphi1_bus1n = "m:cosphi1:bus1n", "Power-Phasor, Power-Factor"
      m_i1P_bus1 = "m:i1P:bus1", "p.u., Current Phasor, Active Current"
      m_i1P_bus1n = "m:i1P:bus1n", "p.u., Current Phasor, Active Current"
      m_i1Q_bus1 = "m:i1Q:bus1", "p.u., Current Phasor, Reactive Current"
      m_i1Q_bus1n = "m:i1Q:bus1n", "p.u., Current Phasor, Reactive Current"
      m_I1P_bus1 = "m:I1P:bus1", "kA, Current Phasor, Active Current"
      m_I1P_bus1n = "m:I1P:bus1n", "kA, Current Phasor, Active Current"
      m_I1Q_bus1 = "m:I1Q:bus1", "kA, Current Phasor, Reactive Current"
      m_I1Q_bus1n = "m:I1Q:bus1n", "kA, Current Phasor, Reactive Current"
      m_u_bus1_A = "m:u:bus1:A", "p.u., Phase Voltage"
      m_u_bus1_B = "m:u:bus1:B", "p.u., Phase Voltage"
      m_u_bus1_C = "m:u:bus1:C", "p.u., Phase Voltage"
      m_u_bus1_N = "m:u:bus1:N", "p.u., Phase Voltage"
      m_u_bus1n = "m:u:bus1n", "p.u., Phase Voltage"
      m_i_bus1_A = "m:i:bus1:A", "p.u., Phase Current"
      m_i_bus1_B = "m:i:bus1:B", "p.u., Phase Current"
      m_i_bus1_C = "m:i:bus1:C", "p.u., Phase Current"
      m_i_bus1_N = "m:i:bus1:N", "p.u., Phase Current"
      m_i_bus1n = "m:i:bus1n", "p.u., Phase Current"
      m_phiu1i1_bus1 = "m:phiu1i1:bus1", "deg, Angle between Voltage and Current Phasor"
      m_phiu1i1_bus1n = "m:phiu1i1:bus1n", "deg, Angle between Voltage and Current Phasor"
      m_Tfct_bus1 = "m:Tfct:bus1", "s, Fault Clearing Time"
      m_Tfct_bus1n = "m:Tfct:bus1n", "s, Fault Clearing Time"
      m_TfctPh_bus1_A = "m:TfctPh:bus1:A", "s, Fault Clearing Time"
      m_TfctPh_bus1_B = "m:TfctPh:bus1:B", "s, Fault Clearing Time"
      m_TfctPh_bus1_C = "m:TfctPh:bus1:C", "s, Fault Clearing Time"
      m_TfctPh_bus1_N = "m:TfctPh:bus1:N", "s, Fault Clearing Time"
      m_TfctPh_bus1n = "m:TfctPh:bus1n", "s, Fault Clearing Time"
      n_u_bus1_A = "n:u:bus1:A", "p.u., Phase Voltage"
      n_u_bus1_B = "n:u:bus1:B", "p.u., Phase Voltage"
      n_u_bus1_C = "n:u:bus1:C", "p.u., Phase Voltage"
      n_u_bus1_N = "n:u:bus1:N", "p.u., Phase Voltage"
      n_u_bus1n = "n:u:bus1n", "p.u., Phase Voltage"
      n_upc_bus1_A = "n:upc:bus1:A", "%, Phase Voltage"
      n_upc_bus1_B = "n:upc:bus1:B", "%, Phase Voltage"
      n_upc_bus1_C = "n:upc:bus1:C", "%, Phase Voltage"
      n_upc_bus1_N = "n:upc:bus1:N", "%, Phase Voltage"
      n_upc_bus1n = "n:upc:bus1n", "%, Phase Voltage"
      n_U_bus1_A = "n:U:bus1:A", "kV, Phase Voltage"
      n_U_bus1_B = "n:U:bus1:B", "kV, Phase Voltage"
      n_U_bus1_C = "n:U:bus1:C", "kV, Phase Voltage"
      n_U_bus1_N = "n:U:bus1:N", "kV, Phase Voltage"
      n_U_bus1n = "n:U:bus1n", "kV, Phase Voltage"
      n_ul_bus1_A = "n:ul:bus1:A", "p.u., Line-Line Voltage"
      n_ul_bus1_B = "n:ul:bus1:B", "p.u., Line-Line Voltage"
      n_ul_bus1_C = "n:ul:bus1:C", "p.u., Line-Line Voltage"
      n_ul_bus1_N = "n:ul:bus1:N", "p.u., Line-Line Voltage"
      n_ul_bus1n = "n:ul:bus1n", "p.u., Line-Line Voltage"
      n_ulpc_bus1_A = "n:ulpc:bus1:A", "%, Line-Line Voltage"
      n_ulpc_bus1_B = "n:ulpc:bus1:B", "%, Line-Line Voltage"
      n_ulpc_bus1_C = "n:ulpc:bus1:C", "%, Line-Line Voltage"
      n_ulpc_bus1_N = "n:ulpc:bus1:N", "%, Line-Line Voltage"
      n_ulpc_bus1n = "n:ulpc:bus1n", "%, Line-Line Voltage"
      n_Ul_bus1_A = "n:Ul:bus1:A", "kV, Line-Line Voltage"
      n_Ul_bus1_B = "n:Ul:bus1:B", "kV, Line-Line Voltage"
      n_Ul_bus1_C = "n:Ul:bus1:C", "kV, Line-Line Voltage"
      n_Ul_bus1_N = "n:Ul:bus1:N", "kV, Line-Line Voltage"
      n_Ul_bus1n = "n:Ul:bus1n", "kV, Line-Line Voltage"
      n_u0_bus1 = "n:u0:bus1", "p.u., Zero-Sequence-Voltage"
      n_u0_bus1n = "n:u0:bus1n", "p.u., Zero-Sequence-Voltage"
      n_U0_bus1 = "n:U0:bus1", "kV, Zero-Sequence-Voltage"
      n_U0_bus1n = "n:U0:bus1n", "kV, Zero-Sequence-Voltage"
      n_U0x3_bus1 = "n:U0x3:bus1", "kV, 3*U0"
      n_U0x3_bus1n = "n:U0x3:bus1n", "kV, 3*U0"
      n_u1r_bus1 = "n:u1r:bus1", "p.u., Voltage Phasor, Real Part"
      n_u1r_bus1n = "n:u1r:bus1n", "p.u., Voltage Phasor, Real Part"
      n_u1i_bus1 = "n:u1i:bus1", "p.u., Voltage Phasor, Imaginary Part"
      n_u1i_bus1n = "n:u1i:bus1n", "p.u., Voltage Phasor, Imaginary Part"
      n_u1_bus1 = "n:u1:bus1", "p.u., Voltage Phasor, Magnitude"
      n_u1_bus1n = "n:u1:bus1n", "p.u., Voltage Phasor, Magnitude"
      n_u1pc_bus1 = "n:u1pc:bus1", "%, Voltage Phasor, Magnitude"
      n_u1pc_bus1n = "n:u1pc:bus1n", "%, Voltage Phasor, Magnitude"
      n_U1_bus1 = "n:U1:bus1", "kV, Voltage Phasor, Magnitude"
      n_U1_bus1n = "n:U1:bus1n", "kV, Voltage Phasor, Magnitude"
      n_phiu1_bus1 = "n:phiu1:bus1", "deg, Voltage Phasor, Angle"
      n_phiu1_bus1n = "n:phiu1:bus1n", "deg, Voltage Phasor, Angle"
      n_U1l_bus1 = "n:U1l:bus1", "kV, Line-Line Voltage Phasor, Magnitude"
      n_U1l_bus1n = "n:U1l:bus1n", "kV, Line-Line Voltage Phasor, Magnitude"
      n_uln_bus1_A = "n:uln:bus1:A", "p.u., Line-Neutral Voltage"
      n_uln_bus1_B = "n:uln:bus1:B", "p.u., Line-Neutral Voltage"
      n_uln_bus1_C = "n:uln:bus1:C", "p.u., Line-Neutral Voltage"
      n_uln_bus1_N = "n:uln:bus1:N", "p.u., Line-Neutral Voltage"
      n_uln_bus1n = "n:uln:bus1n", "p.u., Line-Neutral Voltage"
      n_Uln_bus1_A = "n:Uln:bus1:A", "kV, Line-Neutral Voltage"
      n_Uln_bus1_B = "n:Uln:bus1:B", "kV, Line-Neutral Voltage"
      n_Uln_bus1_C = "n:Uln:bus1:C", "kV, Line-Neutral Voltage"
      n_Uln_bus1_N = "n:Uln:bus1:N", "kV, Line-Neutral Voltage"
      n_Uln_bus1n = "n:Uln:bus1n", "kV, Line-Neutral Voltage"
      n_un_bus1 = "n:un:bus1", "p.u., Neutral-Ground Voltage"
      n_un_bus1n = "n:un:bus1n", "p.u., Neutral-Ground Voltage"
      n_Un_bus1 = "n:Un:bus1", "kV, Neutral-Ground Voltage"
      n_Un_bus1n = "n:Un:bus1n", "kV, Neutral-Ground Voltage"
      n_ishc_bus1_A = "n:ishc:bus1:A", "p.u., Short-Circuit Current"
      n_ishc_bus1_B = "n:ishc:bus1:B", "p.u., Short-Circuit Current"
      n_ishc_bus1_C = "n:ishc:bus1:C", "p.u., Short-Circuit Current"
      n_ishc_bus1_N = "n:ishc:bus1:N", "p.u., Short-Circuit Current"
      n_ishc_bus1n = "n:ishc:bus1n", "p.u., Short-Circuit Current"
      n_Ishc_bus1_A = "n:Ishc:bus1:A", "kA, Short-Circuit Current"
      n_Ishc_bus1_B = "n:Ishc:bus1:B", "kA, Short-Circuit Current"
      n_Ishc_bus1_C = "n:Ishc:bus1:C", "kA, Short-Circuit Current"
      n_Ishc_bus1_N = "n:Ishc:bus1:N", "kA, Short-Circuit Current"
      n_Ishc_bus1n = "n:Ishc:bus1n", "kA, Short-Circuit Current"
      n_inshc_bus1 = "n:inshc:bus1", "p.u., Neutral Short-Circuit Current, magnitude"
      n_inshc_bus1n = "n:inshc:bus1n", "p.u., Neutral Short-Circuit Current, magnitude"
      n_Inshc_bus1 = "n:Inshc:bus1", "kA, Neutral Short-Circuit Current, magnitude"
      n_Inshc_bus1n = "n:Inshc:bus1n", "kA, Neutral Short-Circuit Current, magnitude"
      n_fe_bus1 = "n:fe:bus1", "p.u., Electrical Frequency"
      n_fe_bus1n = "n:fe:bus1n", "p.u., Electrical Frequency"
      n_frdev_bus1 = "n:frdev:bus1", "Hz, average frequency"
      n_frdev_bus1n = "n:frdev:bus1n", "Hz, average frequency"
      n_dfehz_bus1 = "n:dfehz:bus1", "Hz, Deviation of the El. Frequency"
      n_dfehz_bus1n = "n:dfehz:bus1n", "Hz, Deviation of the El. Frequency"
      n_fehz_bus1 = "n:fehz:bus1", "Hz, Electrical Frequency"
      n_fehz_bus1n = "n:fehz:bus1n", "Hz, Electrical Frequency"
      n_frnom_bus1 = "n:frnom:bus1", "Hz, Nominal Frequency"
      n_frnom_bus1n = "n:frnom:bus1n", "Hz, Nominal Frequency"
      s_f0 = "s:f0", "p.u., IN, Frequency input continuous, double"
      s_F0Hz = "s:F0Hz", "Hz, IN, Frequency input (Hz), continuous, double"
      s_fref = "s:fref", "p.u., IN, Reference frequency, continuous, double"
      s_id_ref = "s:id_ref", "p.u., IN, d-axis current reference, continuous, double"
      s_iq_ref = "s:iq_ref", "p.u., IN, q-axis current reference, continuous, double"
      s_i2d_ref = "s:i2d_ref", "p.u., IN, Negative sequence current, d-axis reference, continuous, double"
      s_i2q_ref = "s:i2q_ref", "p.u., IN, Negative sequence current, q-axis reference, continuous, double"
      s_u1r_in = "s:u1r_in", "p.u., IN, Voltage input, real part, continuous, double"
      s_u1i_in = "s:u1i_in", "p.u., IN, Voltage input, imaginary part, continuous, double"
      s_cosref = "s:cosref", "IN, cos(dq-reference angle) continuous, double"
      s_sinref = "s:sinref", "IN, sin(dq-reference angle) continuous, double"
      s_iin_a = "s:iin_a", "p.u., IN, Current, phase A, continuous, double"
      s_iin_b = "s:iin_b", "p.u., IN, Current, phase B, continuous, double"
      s_iin_c = "s:iin_c", "p.u., IN, Current, phase C, continuous, double"
      s_u0_in = "s:u0_in", "p.u., IN, Voltage input, zero sequence component, continuous, double"
      s_id = "s:id", "p.u., OUT, Current, d-axis continuous, double"
      s_iq = "s:iq", "p.u., OUT, Current, q-axis continuous, double"
      s_xd = "s:xd", "p.u., OUT, State variable, current controller d-axis, continuous, double"
      s_xq = "s:xq", "p.u., OUT, State variable, current controller q-axis, continuous, double"
      s_xd_dt = "s:xd:dt", "p.u./s, OUT, State variable, current controller d-axis, derivative, continuous, double"
      s_xq_dt = "s:xq:dt", "p.u./s, OUT, State variable, current controller q-axis, derivative, continuous, double"
      s_phiu = "s:phiu", "rad, STATE, Voltage angle, continuous, double"
      s_phiu_dt = "s:phiu:dt", "rad/s, d/dt, Voltage angle, derivative, continuous, double"
      c_sgnn = "c:sgnn", "MVA, Total rated apparent power (ngnum*sgn)"
      c_cosn = "c:cosn", "Nominal power factor"
      c_hpi = "c:hpi", "rad/s, Nominal angular frequency"
      e_pSubModel = "e:pSubModel", "ElmSubmodel, Submodel"
      e_umin = "e:umin", "p.u., Switch-off threshold"
      e_uonthr = "e:uonthr", "p.u., Switch-on threshold"
      e_Tondelay = "e:Tondelay", "s, Switch-on delay"
      e_uk = "e:uk", "%, Short circuit impedance"
      e_Pcu = "e:Pcu", "kW, Copper losses"
      e_R0toR1 = "e:R0toR1", "R0/R1 ratio"
      e_X0toX1 = "e:X0toX1", "X0/X1 ratio"
      e_Kd = "e:Kd", "Kd: d-Axis, proportional gain"
      e_Td = "e:Td", "s, Td: d-Axis, integration time constant"
      e_Kq = "e:Kq", "Kq: q-Axis, proportional gain"
      e_Tq = "e:Tq", "s, Tq: q-Axis, integration time constant"
      e_Kp_pllu = "e:Kp_pllu", "Proportional Gain"
      e_Ki_pllu = "e:Ki_pllu", "Integration Gain"
      e_T1_pllu = "e:T1_pllu", "ms, Low-Pass Filter Time Constant"
      e_Kp_plli = "e:Kp_plli", "Proportional Gain"
      e_Ki_plli = "e:Ki_plli", "Integration Gain"
      e_T1_plli = "e:T1_plli", "ms, Low-Pass Filter Time Constant"
      r_pSubModel = "r:pSubModel", "ElmSubmodel, Submodel"

    class ElmSecctrl(Enum):
      _init_ = 'value __doc__'


    class ElmShnt(Enum):
      _init_ = 'value __doc__'

      m_u0_bus1 = "m:u0:bus1", "p.u., Zero-Sequence-Voltage"
      m_u0_bus1n = "m:u0:bus1n", "p.u., Zero-Sequence-Voltage"
      m_u1r_bus1 = "m:u1r:bus1", "p.u., Voltage Phasor, Real Part"
      m_u1r_bus1n = "m:u1r:bus1n", "p.u., Voltage Phasor, Real Part"
      m_u1i_bus1 = "m:u1i:bus1", "p.u., Voltage Phasor, Imaginary Part"
      m_u1i_bus1n = "m:u1i:bus1n", "p.u., Voltage Phasor, Imaginary Part"
      m_u1_bus1 = "m:u1:bus1", "p.u., Voltage Phasor, Magnitude"
      m_u1_bus1n = "m:u1:bus1n", "p.u., Voltage Phasor, Magnitude"
      m_U1_bus1 = "m:U1:bus1", "kV, Line-Ground Voltage Phasor, Magnitude"
      m_U1_bus1n = "m:U1:bus1n", "kV, Line-Ground Voltage Phasor, Magnitude"
      m_U1l_bus1 = "m:U1l:bus1", "kV, Line-Line Voltage Phasor, Magnitude"
      m_U1l_bus1n = "m:U1l:bus1n", "kV, Line-Line Voltage Phasor, Magnitude"
      m_phiu1_bus1 = "m:phiu1:bus1", "deg, Voltage Phasor, Angle"
      m_phiu1_bus1n = "m:phiu1:bus1n", "deg, Voltage Phasor, Angle"
      m_I_bus1_A = "m:I:bus1:A", "kA, Phase Current"
      m_I_bus1_B = "m:I:bus1:B", "kA, Phase Current"
      m_I_bus1_C = "m:I:bus1:C", "kA, Phase Current"
      m_I_bus1_N = "m:I:bus1:N", "kA, Phase Current"
      m_I_bus1n = "m:I:bus1n", "kA, Phase Current"
      m_i0_bus1 = "m:i0:bus1", "p.u., Zero-Sequence Current"
      m_i0_bus1n = "m:i0:bus1n", "p.u., Zero-Sequence Current"
      m_I0_bus1 = "m:I0:bus1", "kA, Zero-Sequence Current"
      m_I0_bus1n = "m:I0:bus1n", "kA, Zero-Sequence Current"
      m_I0x3_bus1 = "m:I0x3:bus1", "kA, 3*I0"
      m_I0x3_bus1n = "m:I0x3:bus1n", "kA, 3*I0"
      m_i1r_bus1 = "m:i1r:bus1", "p.u., Current Phasor, Real Part"
      m_i1r_bus1n = "m:i1r:bus1n", "p.u., Current Phasor, Real Part"
      m_i1i_bus1 = "m:i1i:bus1", "p.u., Current Phasor, Imaginary Part"
      m_i1i_bus1n = "m:i1i:bus1n", "p.u., Current Phasor, Imaginary Part"
      m_i1_bus1 = "m:i1:bus1", "p.u., Current Phasor, Magnitude"
      m_i1_bus1n = "m:i1:bus1n", "p.u., Current Phasor, Magnitude"
      m_I1_bus1 = "m:I1:bus1", "kA, Current Phasor, Magnitude"
      m_I1_bus1n = "m:I1:bus1n", "kA, Current Phasor, Magnitude"
      m_phii1_bus1 = "m:phii1:bus1", "deg, Current Phasor, Angle"
      m_phii1_bus1n = "m:phii1:bus1n", "deg, Current Phasor, Angle"
      m_P_bus1_A = "m:P:bus1:A", "MW, Active Power"
      m_P_bus1_B = "m:P:bus1:B", "MW, Active Power"
      m_P_bus1_C = "m:P:bus1:C", "MW, Active Power"
      m_P_bus1_N = "m:P:bus1:N", "MW, Active Power"
      m_P_bus1n = "m:P:bus1n", "MW, Active Power"
      m_Psum_bus1 = "m:Psum:bus1", "MW, Power-Phasor, Active Power"
      m_Psum_bus1n = "m:Psum:bus1n", "MW, Power-Phasor, Active Power"
      m_Qsum_bus1 = "m:Qsum:bus1", "Mvar, Power Phasor, Reactive Part"
      m_Qsum_bus1n = "m:Qsum:bus1n", "Mvar, Power Phasor, Reactive Part"
      m_Ssum_bus1 = "m:Ssum:bus1", "MVA, Power Phasor, Apparent Power"
      m_Ssum_bus1n = "m:Ssum:bus1n", "MVA, Power Phasor, Apparent Power"
      m_cosphi1_bus1 = "m:cosphi1:bus1", "Power-Phasor, Power-Factor"
      m_cosphi1_bus1n = "m:cosphi1:bus1n", "Power-Phasor, Power-Factor"
      m_i1P_bus1 = "m:i1P:bus1", "p.u., Current Phasor, Active Current"
      m_i1P_bus1n = "m:i1P:bus1n", "p.u., Current Phasor, Active Current"
      m_i1Q_bus1 = "m:i1Q:bus1", "p.u., Current Phasor, Reactive Current"
      m_i1Q_bus1n = "m:i1Q:bus1n", "p.u., Current Phasor, Reactive Current"
      m_I1P_bus1 = "m:I1P:bus1", "kA, Current Phasor, Active Current"
      m_I1P_bus1n = "m:I1P:bus1n", "kA, Current Phasor, Active Current"
      m_I1Q_bus1 = "m:I1Q:bus1", "kA, Current Phasor, Reactive Current"
      m_I1Q_bus1n = "m:I1Q:bus1n", "kA, Current Phasor, Reactive Current"
      m_u_bus1_A = "m:u:bus1:A", "p.u., Phase Voltage"
      m_u_bus1_B = "m:u:bus1:B", "p.u., Phase Voltage"
      m_u_bus1_C = "m:u:bus1:C", "p.u., Phase Voltage"
      m_u_bus1_N = "m:u:bus1:N", "p.u., Phase Voltage"
      m_u_bus1n = "m:u:bus1n", "p.u., Phase Voltage"
      m_i_bus1_A = "m:i:bus1:A", "p.u., Phase Current"
      m_i_bus1_B = "m:i:bus1:B", "p.u., Phase Current"
      m_i_bus1_C = "m:i:bus1:C", "p.u., Phase Current"
      m_i_bus1_N = "m:i:bus1:N", "p.u., Phase Current"
      m_i_bus1n = "m:i:bus1n", "p.u., Phase Current"
      m_phiu1i1_bus1 = "m:phiu1i1:bus1", "deg, Angle between Voltage and Current Phasor"
      m_phiu1i1_bus1n = "m:phiu1i1:bus1n", "deg, Angle between Voltage and Current Phasor"
      m_Tfct_bus1 = "m:Tfct:bus1", "s, Fault Clearing Time"
      m_Tfct_bus1n = "m:Tfct:bus1n", "s, Fault Clearing Time"
      m_TfctPh_bus1_A = "m:TfctPh:bus1:A", "s, Fault Clearing Time"
      m_TfctPh_bus1_B = "m:TfctPh:bus1:B", "s, Fault Clearing Time"
      m_TfctPh_bus1_C = "m:TfctPh:bus1:C", "s, Fault Clearing Time"
      m_TfctPh_bus1_N = "m:TfctPh:bus1:N", "s, Fault Clearing Time"
      m_TfctPh_bus1n = "m:TfctPh:bus1n", "s, Fault Clearing Time"
      n_u_bus1_A = "n:u:bus1:A", "p.u., Phase Voltage"
      n_u_bus1_B = "n:u:bus1:B", "p.u., Phase Voltage"
      n_u_bus1_C = "n:u:bus1:C", "p.u., Phase Voltage"
      n_u_bus1_N = "n:u:bus1:N", "p.u., Phase Voltage"
      n_u_bus1n = "n:u:bus1n", "p.u., Phase Voltage"
      n_upc_bus1_A = "n:upc:bus1:A", "%, Phase Voltage"
      n_upc_bus1_B = "n:upc:bus1:B", "%, Phase Voltage"
      n_upc_bus1_C = "n:upc:bus1:C", "%, Phase Voltage"
      n_upc_bus1_N = "n:upc:bus1:N", "%, Phase Voltage"
      n_upc_bus1n = "n:upc:bus1n", "%, Phase Voltage"
      n_U_bus1_A = "n:U:bus1:A", "kV, Phase Voltage"
      n_U_bus1_B = "n:U:bus1:B", "kV, Phase Voltage"
      n_U_bus1_C = "n:U:bus1:C", "kV, Phase Voltage"
      n_U_bus1_N = "n:U:bus1:N", "kV, Phase Voltage"
      n_U_bus1n = "n:U:bus1n", "kV, Phase Voltage"
      n_ul_bus1_A = "n:ul:bus1:A", "p.u., Line-Line Voltage"
      n_ul_bus1_B = "n:ul:bus1:B", "p.u., Line-Line Voltage"
      n_ul_bus1_C = "n:ul:bus1:C", "p.u., Line-Line Voltage"
      n_ul_bus1_N = "n:ul:bus1:N", "p.u., Line-Line Voltage"
      n_ul_bus1n = "n:ul:bus1n", "p.u., Line-Line Voltage"
      n_ulpc_bus1_A = "n:ulpc:bus1:A", "%, Line-Line Voltage"
      n_ulpc_bus1_B = "n:ulpc:bus1:B", "%, Line-Line Voltage"
      n_ulpc_bus1_C = "n:ulpc:bus1:C", "%, Line-Line Voltage"
      n_ulpc_bus1_N = "n:ulpc:bus1:N", "%, Line-Line Voltage"
      n_ulpc_bus1n = "n:ulpc:bus1n", "%, Line-Line Voltage"
      n_Ul_bus1_A = "n:Ul:bus1:A", "kV, Line-Line Voltage"
      n_Ul_bus1_B = "n:Ul:bus1:B", "kV, Line-Line Voltage"
      n_Ul_bus1_C = "n:Ul:bus1:C", "kV, Line-Line Voltage"
      n_Ul_bus1_N = "n:Ul:bus1:N", "kV, Line-Line Voltage"
      n_Ul_bus1n = "n:Ul:bus1n", "kV, Line-Line Voltage"
      n_u0_bus1 = "n:u0:bus1", "p.u., Zero-Sequence-Voltage"
      n_u0_bus1n = "n:u0:bus1n", "p.u., Zero-Sequence-Voltage"
      n_U0_bus1 = "n:U0:bus1", "kV, Zero-Sequence-Voltage"
      n_U0_bus1n = "n:U0:bus1n", "kV, Zero-Sequence-Voltage"
      n_U0x3_bus1 = "n:U0x3:bus1", "kV, 3*U0"
      n_U0x3_bus1n = "n:U0x3:bus1n", "kV, 3*U0"
      n_u1r_bus1 = "n:u1r:bus1", "p.u., Voltage Phasor, Real Part"
      n_u1r_bus1n = "n:u1r:bus1n", "p.u., Voltage Phasor, Real Part"
      n_u1i_bus1 = "n:u1i:bus1", "p.u., Voltage Phasor, Imaginary Part"
      n_u1i_bus1n = "n:u1i:bus1n", "p.u., Voltage Phasor, Imaginary Part"
      n_u1_bus1 = "n:u1:bus1", "p.u., Voltage Phasor, Magnitude"
      n_u1_bus1n = "n:u1:bus1n", "p.u., Voltage Phasor, Magnitude"
      n_u1pc_bus1 = "n:u1pc:bus1", "%, Voltage Phasor, Magnitude"
      n_u1pc_bus1n = "n:u1pc:bus1n", "%, Voltage Phasor, Magnitude"
      n_U1_bus1 = "n:U1:bus1", "kV, Voltage Phasor, Magnitude"
      n_U1_bus1n = "n:U1:bus1n", "kV, Voltage Phasor, Magnitude"
      n_phiu1_bus1 = "n:phiu1:bus1", "deg, Voltage Phasor, Angle"
      n_phiu1_bus1n = "n:phiu1:bus1n", "deg, Voltage Phasor, Angle"
      n_U1l_bus1 = "n:U1l:bus1", "kV, Line-Line Voltage Phasor, Magnitude"
      n_U1l_bus1n = "n:U1l:bus1n", "kV, Line-Line Voltage Phasor, Magnitude"
      n_uln_bus1_A = "n:uln:bus1:A", "p.u., Line-Neutral Voltage"
      n_uln_bus1_B = "n:uln:bus1:B", "p.u., Line-Neutral Voltage"
      n_uln_bus1_C = "n:uln:bus1:C", "p.u., Line-Neutral Voltage"
      n_uln_bus1_N = "n:uln:bus1:N", "p.u., Line-Neutral Voltage"
      n_uln_bus1n = "n:uln:bus1n", "p.u., Line-Neutral Voltage"
      n_Uln_bus1_A = "n:Uln:bus1:A", "kV, Line-Neutral Voltage"
      n_Uln_bus1_B = "n:Uln:bus1:B", "kV, Line-Neutral Voltage"
      n_Uln_bus1_C = "n:Uln:bus1:C", "kV, Line-Neutral Voltage"
      n_Uln_bus1_N = "n:Uln:bus1:N", "kV, Line-Neutral Voltage"
      n_Uln_bus1n = "n:Uln:bus1n", "kV, Line-Neutral Voltage"
      n_un_bus1 = "n:un:bus1", "p.u., Neutral-Ground Voltage"
      n_un_bus1n = "n:un:bus1n", "p.u., Neutral-Ground Voltage"
      n_Un_bus1 = "n:Un:bus1", "kV, Neutral-Ground Voltage"
      n_Un_bus1n = "n:Un:bus1n", "kV, Neutral-Ground Voltage"
      n_ishc_bus1_A = "n:ishc:bus1:A", "p.u., Short-Circuit Current"
      n_ishc_bus1_B = "n:ishc:bus1:B", "p.u., Short-Circuit Current"
      n_ishc_bus1_C = "n:ishc:bus1:C", "p.u., Short-Circuit Current"
      n_ishc_bus1_N = "n:ishc:bus1:N", "p.u., Short-Circuit Current"
      n_ishc_bus1n = "n:ishc:bus1n", "p.u., Short-Circuit Current"
      n_Ishc_bus1_A = "n:Ishc:bus1:A", "kA, Short-Circuit Current"
      n_Ishc_bus1_B = "n:Ishc:bus1:B", "kA, Short-Circuit Current"
      n_Ishc_bus1_C = "n:Ishc:bus1:C", "kA, Short-Circuit Current"
      n_Ishc_bus1_N = "n:Ishc:bus1:N", "kA, Short-Circuit Current"
      n_Ishc_bus1n = "n:Ishc:bus1n", "kA, Short-Circuit Current"
      n_inshc_bus1 = "n:inshc:bus1", "p.u., Neutral Short-Circuit Current, magnitude"
      n_inshc_bus1n = "n:inshc:bus1n", "p.u., Neutral Short-Circuit Current, magnitude"
      n_Inshc_bus1 = "n:Inshc:bus1", "kA, Neutral Short-Circuit Current, magnitude"
      n_Inshc_bus1n = "n:Inshc:bus1n", "kA, Neutral Short-Circuit Current, magnitude"
      n_fe_bus1 = "n:fe:bus1", "p.u., Electrical Frequency"
      n_fe_bus1n = "n:fe:bus1n", "p.u., Electrical Frequency"
      n_frdev_bus1 = "n:frdev:bus1", "Hz, average frequency"
      n_frdev_bus1n = "n:frdev:bus1n", "Hz, average frequency"
      n_dfehz_bus1 = "n:dfehz:bus1", "Hz, Deviation of the El. Frequency"
      n_dfehz_bus1n = "n:dfehz:bus1n", "Hz, Deviation of the El. Frequency"
      n_fehz_bus1 = "n:fehz:bus1", "Hz, Electrical Frequency"
      n_fehz_bus1n = "n:fehz:bus1n", "Hz, Electrical Frequency"
      n_frnom_bus1 = "n:frnom:bus1", "Hz, Nominal Frequency"
      n_frnom_bus1n = "n:frnom:bus1n", "Hz, Nominal Frequency"
      s_psil_d = "s:psil_d", "p.u., STATE, Flux, d-axis, continuous, double"
      s_psil_q = "s:psil_q", "p.u., STATE, Flux, q-axis, continuous, double"
      s_psil_0 = "s:psil_0", "p.u., STATE, Flux, 0-Sequence, continuous, double"
      s_uC_a = "s:uC_a", "p.u., STATE, Voltage at Capacitance, Phase a continuous, double"
      s_uC_b = "s:uC_b", "p.u., STATE, Voltage at Capacitance, Phase b continuous, double"
      s_uC_c = "s:uC_c", "p.u., STATE, Voltage at Capacitance, Phase c continuous, double"
      s_psil_d_dt = "s:psil_d:dt", "p.u./s, d/dt, Flux, d-axis, derivative, continuous, double"
      s_psil_q_dt = "s:psil_q:dt", "p.u./s, d/dt, Flux, q-axis, derivative, continuous, double"
      s_psil_0_dt = "s:psil_0:dt", "p.u./s, d/dt, Flux, 0-Sequence, derivative, continuous, double"
      s_uC_a_dt = "s:uC_a:dt", "p.u./s, d/dt, Voltage at Capacitance, Phase a, derivative, continuous, double"
      s_uC_b_dt = "s:uC_b:dt", "p.u./s, d/dt, Voltage at Capacitance, Phase b, derivative, continuous, double"
      s_uC_c_dt = "s:uC_c:dt", "p.u./s, d/dt, Voltage at Capacitance, Phase c, derivative, continuous, double"
      c_rs = "c:rs", "p.u., Series-Resistance"
      c_gp = "c:gp", "p.u., Parallel-Resistance"
      c_l = "c:l", "p.u., Inductance"
      c_c = "c:c", "p.u., Capacitance"
      c_rs0 = "c:rs0", "p.u., Zero Sequence Series-Resistance"
      c_l0 = "c:l0", "p.u., Zero Sequence Inductance"
      c_le = "c:le", "p.u., Grounding Inductance"
      c_re = "c:re", "p.u., Grounding Resistance"
      c_c0 = "c:c0", "p.u., Parasitic Capacitance"
      c_Unom = "c:Unom", "kV, Nominal Voltage"
      c_Snom = "c:Snom", "MVA, Nominal Power"
      c_Ustar = "c:Ustar", "kV, Star-Point Voltage"
      c_ILph_A = "c:ILph:A", "A, Current through Inductor L (Phase Value)"
      c_ILph_B = "c:ILph:B", "A, Current through Inductor L (Phase Value)"
      c_ILph_C = "c:ILph:C", "A, Current through Inductor L (Phase Value)"
      c_ILph_N = "c:ILph:N", "A, Current through Inductor L (Phase Value)"
      c_ICph_A = "c:ICph:A", "A, Current through Capacitor C (Phase Value)"
      c_ICph_B = "c:ICph:B", "A, Current through Capacitor C (Phase Value)"
      c_ICph_C = "c:ICph:C", "A, Current through Capacitor C (Phase Value)"
      c_ICph_N = "c:ICph:N", "A, Current through Capacitor C (Phase Value)"
      c_IRpph_A = "c:IRpph:A", "A, Current through Rp (Phase Value)"
      c_IRpph_B = "c:IRpph:B", "A, Current through Rp (Phase Value)"
      c_IRpph_C = "c:IRpph:C", "A, Current through Rp (Phase Value)"
      c_IRpph_N = "c:IRpph:N", "A, Current through Rp (Phase Value)"
      c_ucph_A = "c:ucph:A", "p.u., Voltage across Capacitor (Phase Value)"
      c_ucph_B = "c:ucph:B", "p.u., Voltage across Capacitor (Phase Value)"
      c_ucph_C = "c:ucph:C", "p.u., Voltage across Capacitor (Phase Value)"
      c_ucph_N = "c:ucph:N", "p.u., Voltage across Capacitor (Phase Value)"
      c_urlph_A = "c:urlph:A", "p.u., Voltage across Inductor (Phase Value)"
      c_urlph_B = "c:urlph:B", "p.u., Voltage across Inductor (Phase Value)"
      c_urlph_C = "c:urlph:C", "p.u., Voltage across Inductor (Phase Value)"
      c_urlph_N = "c:urlph:N", "p.u., Voltage across Inductor (Phase Value)"
      c_Ucph_A = "c:Ucph:A", "kV, Voltage across Capacitor (Phase Value)"
      c_Ucph_B = "c:Ucph:B", "kV, Voltage across Capacitor (Phase Value)"
      c_Ucph_C = "c:Ucph:C", "kV, Voltage across Capacitor (Phase Value)"
      c_Ucph_N = "c:Ucph:N", "kV, Voltage across Capacitor (Phase Value)"
      c_Urlph_A = "c:Urlph:A", "kV, Voltage across Inductor (Phase Value)"
      c_Urlph_B = "c:Urlph:B", "kV, Voltage across Inductor (Phase Value)"
      c_Urlph_C = "c:Urlph:C", "kV, Voltage across Inductor (Phase Value)"
      c_Urlph_N = "c:Urlph:N", "kV, Voltage across Inductor (Phase Value)"
      c_frnom = "c:frnom", "Hz, Nominal Frequency"
      c_hpi = "c:hpi", "rad/s, Nominal Angular Frequency"
      c_cn = "c:cn", "Connection Mode"
      c_shtype = "c:shtype", "Type"
      c_i_earth = "c:i_earth", "Internal"
      c_ncapa = "c:ncapa", "Number of Stages in Parallel"
      c_ncapx = "c:ncapx", "Max. No. of Steps"
      c_Qmax = "c:Qmax", "Mvar, Max. Rated Reactive Power"
      c_iRL_A = "c:iRL:A", "p.u., Current through Inductance"
      c_iRL_B = "c:iRL:B", "p.u., Current through Inductance"
      c_iRL_C = "c:iRL:C", "p.u., Current through Inductance"
      c_iRL_N = "c:iRL:N", "p.u., Current through Inductance"
      e_itrmt = "e:itrmt", "Type"
      e_psi0 = "e:psi0", "p.u., Knee Flux"
      e_xreapu = "e:xreapu", "p.u., Linear Reactance"
      e_xmair = "e:xmair", "p.u., Saturated Reactance"
      e_ksat = "e:ksat", "Saturation Exponent"
      e_satcur = "e:satcur", "p.u., Current (peak)"
      e_satflux = "e:satflux", "p.u., Flux (peak)"
      e_satcue = "e:satcue", "%, Current (RMS)"
      e_satvol = "e:satvol", "p.u., Voltage (RMS)"
      e_iInterPol = "e:iInterPol", "Interpolation"
      e_iFit = "e:iFit", "Data fitting"
      e_smoothfac = "e:smoothfac", "%, Smoothing Factor"
      e_iFinalSlope = "e:iFinalSlope", "Set final slope (peak values)"
      e_iHyster = "e:iHyster", "Model"
      e_Lwidth = "e:Lwidth", "%, Loop width"
      e_iLimb = "e:iLimb", "Core"
      e_iResFlux = "e:iResFlux", "Residual flux"
      e_PsiresA = "e:PsiresA", "p.u., Residual flux, ph. A"
      e_PsiresB = "e:PsiresB", "p.u., Residual flux, ph. B"
      e_PsiresC = "e:PsiresC", "p.u., Residual flux, ph. C"
      e_cPsiresC = "e:cPsiresC", "p.u., Residual flux, ph. C"
      e_aFrolich = "e:aFrolich", "Frolich equation coefficient a"
      e_bFrolich = "e:bFrolich", "Frolich equation coefficient b"
      e_cFrolich = "e:cFrolich", "Frolich equation coefficient c"
      e_xsatFrolich = "e:xsatFrolich", "Frolich equation saturated reactance (p.u.)"
      e_mseFrolich = "e:mseFrolich", "Frolich equation, mean squared error"

    class ElmSind(Enum):
      _init_ = 'value __doc__'

      m_u0_bus1 = "m:u0:bus1", "p.u., Zero-Sequence-Voltage"
      m_u0_bus2 = "m:u0:bus2", "p.u., Zero-Sequence-Voltage"
      m_u1r_bus1 = "m:u1r:bus1", "p.u., Voltage Phasor, Real Part"
      m_u1r_bus2 = "m:u1r:bus2", "p.u., Voltage Phasor, Real Part"
      m_u1i_bus1 = "m:u1i:bus1", "p.u., Voltage Phasor, Imaginary Part"
      m_u1i_bus2 = "m:u1i:bus2", "p.u., Voltage Phasor, Imaginary Part"
      m_u1_bus1 = "m:u1:bus1", "p.u., Voltage Phasor, Magnitude"
      m_u1_bus2 = "m:u1:bus2", "p.u., Voltage Phasor, Magnitude"
      m_U1_bus1 = "m:U1:bus1", "kV, Line-Ground Voltage Phasor, Magnitude"
      m_U1_bus2 = "m:U1:bus2", "kV, Line-Ground Voltage Phasor, Magnitude"
      m_U1l_bus1 = "m:U1l:bus1", "kV, Line-Line Voltage Phasor, Magnitude"
      m_U1l_bus2 = "m:U1l:bus2", "kV, Line-Line Voltage Phasor, Magnitude"
      m_phiu1_bus1 = "m:phiu1:bus1", "deg, Voltage Phasor, Angle"
      m_phiu1_bus2 = "m:phiu1:bus2", "deg, Voltage Phasor, Angle"
      m_I_bus1_A = "m:I:bus1:A", "kA, Phase Current"
      m_I_bus1_B = "m:I:bus1:B", "kA, Phase Current"
      m_I_bus1_C = "m:I:bus1:C", "kA, Phase Current"
      m_I_bus2_A = "m:I:bus2:A", "kA, Phase Current"
      m_I_bus2_B = "m:I:bus2:B", "kA, Phase Current"
      m_I_bus2_C = "m:I:bus2:C", "kA, Phase Current"
      m_i0_bus1 = "m:i0:bus1", "p.u., Zero-Sequence Current"
      m_i0_bus2 = "m:i0:bus2", "p.u., Zero-Sequence Current"
      m_I0_bus1 = "m:I0:bus1", "kA, Zero-Sequence Current"
      m_I0_bus2 = "m:I0:bus2", "kA, Zero-Sequence Current"
      m_I0x3_bus1 = "m:I0x3:bus1", "kA, 3*I0"
      m_I0x3_bus2 = "m:I0x3:bus2", "kA, 3*I0"
      m_i1r_bus1 = "m:i1r:bus1", "p.u., Current Phasor, Real Part"
      m_i1r_bus2 = "m:i1r:bus2", "p.u., Current Phasor, Real Part"
      m_i1i_bus1 = "m:i1i:bus1", "p.u., Current Phasor, Imaginary Part"
      m_i1i_bus2 = "m:i1i:bus2", "p.u., Current Phasor, Imaginary Part"
      m_i1_bus1 = "m:i1:bus1", "p.u., Current Phasor, Magnitude"
      m_i1_bus2 = "m:i1:bus2", "p.u., Current Phasor, Magnitude"
      m_I1_bus1 = "m:I1:bus1", "kA, Current Phasor, Magnitude"
      m_I1_bus2 = "m:I1:bus2", "kA, Current Phasor, Magnitude"
      m_phii1_bus1 = "m:phii1:bus1", "deg, Current Phasor, Angle"
      m_phii1_bus2 = "m:phii1:bus2", "deg, Current Phasor, Angle"
      m_P_bus1_A = "m:P:bus1:A", "MW, Active Power"
      m_P_bus1_B = "m:P:bus1:B", "MW, Active Power"
      m_P_bus1_C = "m:P:bus1:C", "MW, Active Power"
      m_P_bus2_A = "m:P:bus2:A", "MW, Active Power"
      m_P_bus2_B = "m:P:bus2:B", "MW, Active Power"
      m_P_bus2_C = "m:P:bus2:C", "MW, Active Power"
      m_Psum_bus1 = "m:Psum:bus1", "MW, Power-Phasor, Active Power"
      m_Psum_bus2 = "m:Psum:bus2", "MW, Power-Phasor, Active Power"
      m_Qsum_bus1 = "m:Qsum:bus1", "Mvar, Power Phasor, Reactive Part"
      m_Qsum_bus2 = "m:Qsum:bus2", "Mvar, Power Phasor, Reactive Part"
      m_Ssum_bus1 = "m:Ssum:bus1", "MVA, Power Phasor, Apparent Power"
      m_Ssum_bus2 = "m:Ssum:bus2", "MVA, Power Phasor, Apparent Power"
      m_cosphi1_bus1 = "m:cosphi1:bus1", "Power-Phasor, Power-Factor"
      m_cosphi1_bus2 = "m:cosphi1:bus2", "Power-Phasor, Power-Factor"
      m_i1P_bus1 = "m:i1P:bus1", "p.u., Current Phasor, Active Current"
      m_i1P_bus2 = "m:i1P:bus2", "p.u., Current Phasor, Active Current"
      m_i1Q_bus1 = "m:i1Q:bus1", "p.u., Current Phasor, Reactive Current"
      m_i1Q_bus2 = "m:i1Q:bus2", "p.u., Current Phasor, Reactive Current"
      m_I1P_bus1 = "m:I1P:bus1", "kA, Current Phasor, Active Current"
      m_I1P_bus2 = "m:I1P:bus2", "kA, Current Phasor, Active Current"
      m_I1Q_bus1 = "m:I1Q:bus1", "kA, Current Phasor, Reactive Current"
      m_I1Q_bus2 = "m:I1Q:bus2", "kA, Current Phasor, Reactive Current"
      m_u_bus1_A = "m:u:bus1:A", "p.u., Phase Voltage"
      m_u_bus1_B = "m:u:bus1:B", "p.u., Phase Voltage"
      m_u_bus1_C = "m:u:bus1:C", "p.u., Phase Voltage"
      m_u_bus2_A = "m:u:bus2:A", "p.u., Phase Voltage"
      m_u_bus2_B = "m:u:bus2:B", "p.u., Phase Voltage"
      m_u_bus2_C = "m:u:bus2:C", "p.u., Phase Voltage"
      m_i_bus1_A = "m:i:bus1:A", "p.u., Phase Current"
      m_i_bus1_B = "m:i:bus1:B", "p.u., Phase Current"
      m_i_bus1_C = "m:i:bus1:C", "p.u., Phase Current"
      m_i_bus2_A = "m:i:bus2:A", "p.u., Phase Current"
      m_i_bus2_B = "m:i:bus2:B", "p.u., Phase Current"
      m_i_bus2_C = "m:i:bus2:C", "p.u., Phase Current"
      m_phiu1i1_bus1 = "m:phiu1i1:bus1", "deg, Angle between Voltage and Current Phasor"
      m_phiu1i1_bus2 = "m:phiu1i1:bus2", "deg, Angle between Voltage and Current Phasor"
      m_Tfct_bus1 = "m:Tfct:bus1", "s, Fault Clearing Time"
      m_Tfct_bus2 = "m:Tfct:bus2", "s, Fault Clearing Time"
      m_TfctPh_bus1_A = "m:TfctPh:bus1:A", "s, Fault Clearing Time"
      m_TfctPh_bus1_B = "m:TfctPh:bus1:B", "s, Fault Clearing Time"
      m_TfctPh_bus1_C = "m:TfctPh:bus1:C", "s, Fault Clearing Time"
      m_TfctPh_bus2_A = "m:TfctPh:bus2:A", "s, Fault Clearing Time"
      m_TfctPh_bus2_B = "m:TfctPh:bus2:B", "s, Fault Clearing Time"
      m_TfctPh_bus2_C = "m:TfctPh:bus2:C", "s, Fault Clearing Time"
      n_u_bus1_A = "n:u:bus1:A", "p.u., Phase Voltage"
      n_u_bus1_B = "n:u:bus1:B", "p.u., Phase Voltage"
      n_u_bus1_C = "n:u:bus1:C", "p.u., Phase Voltage"
      n_u_bus2_A = "n:u:bus2:A", "p.u., Phase Voltage"
      n_u_bus2_B = "n:u:bus2:B", "p.u., Phase Voltage"
      n_u_bus2_C = "n:u:bus2:C", "p.u., Phase Voltage"
      n_upc_bus1_A = "n:upc:bus1:A", "%, Phase Voltage"
      n_upc_bus1_B = "n:upc:bus1:B", "%, Phase Voltage"
      n_upc_bus1_C = "n:upc:bus1:C", "%, Phase Voltage"
      n_upc_bus2_A = "n:upc:bus2:A", "%, Phase Voltage"
      n_upc_bus2_B = "n:upc:bus2:B", "%, Phase Voltage"
      n_upc_bus2_C = "n:upc:bus2:C", "%, Phase Voltage"
      n_U_bus1_A = "n:U:bus1:A", "kV, Phase Voltage"
      n_U_bus1_B = "n:U:bus1:B", "kV, Phase Voltage"
      n_U_bus1_C = "n:U:bus1:C", "kV, Phase Voltage"
      n_U_bus2_A = "n:U:bus2:A", "kV, Phase Voltage"
      n_U_bus2_B = "n:U:bus2:B", "kV, Phase Voltage"
      n_U_bus2_C = "n:U:bus2:C", "kV, Phase Voltage"
      n_ul_bus1_A = "n:ul:bus1:A", "p.u., Line-Line Voltage"
      n_ul_bus1_B = "n:ul:bus1:B", "p.u., Line-Line Voltage"
      n_ul_bus1_C = "n:ul:bus1:C", "p.u., Line-Line Voltage"
      n_ul_bus2_A = "n:ul:bus2:A", "p.u., Line-Line Voltage"
      n_ul_bus2_B = "n:ul:bus2:B", "p.u., Line-Line Voltage"
      n_ul_bus2_C = "n:ul:bus2:C", "p.u., Line-Line Voltage"
      n_ulpc_bus1_A = "n:ulpc:bus1:A", "%, Line-Line Voltage"
      n_ulpc_bus1_B = "n:ulpc:bus1:B", "%, Line-Line Voltage"
      n_ulpc_bus1_C = "n:ulpc:bus1:C", "%, Line-Line Voltage"
      n_ulpc_bus2_A = "n:ulpc:bus2:A", "%, Line-Line Voltage"
      n_ulpc_bus2_B = "n:ulpc:bus2:B", "%, Line-Line Voltage"
      n_ulpc_bus2_C = "n:ulpc:bus2:C", "%, Line-Line Voltage"
      n_Ul_bus1_A = "n:Ul:bus1:A", "kV, Line-Line Voltage"
      n_Ul_bus1_B = "n:Ul:bus1:B", "kV, Line-Line Voltage"
      n_Ul_bus1_C = "n:Ul:bus1:C", "kV, Line-Line Voltage"
      n_Ul_bus2_A = "n:Ul:bus2:A", "kV, Line-Line Voltage"
      n_Ul_bus2_B = "n:Ul:bus2:B", "kV, Line-Line Voltage"
      n_Ul_bus2_C = "n:Ul:bus2:C", "kV, Line-Line Voltage"
      n_u0_bus1 = "n:u0:bus1", "p.u., Zero-Sequence-Voltage"
      n_u0_bus2 = "n:u0:bus2", "p.u., Zero-Sequence-Voltage"
      n_U0_bus1 = "n:U0:bus1", "kV, Zero-Sequence-Voltage"
      n_U0_bus2 = "n:U0:bus2", "kV, Zero-Sequence-Voltage"
      n_U0x3_bus1 = "n:U0x3:bus1", "kV, 3*U0"
      n_U0x3_bus2 = "n:U0x3:bus2", "kV, 3*U0"
      n_u1r_bus1 = "n:u1r:bus1", "p.u., Voltage Phasor, Real Part"
      n_u1r_bus2 = "n:u1r:bus2", "p.u., Voltage Phasor, Real Part"
      n_u1i_bus1 = "n:u1i:bus1", "p.u., Voltage Phasor, Imaginary Part"
      n_u1i_bus2 = "n:u1i:bus2", "p.u., Voltage Phasor, Imaginary Part"
      n_u1_bus1 = "n:u1:bus1", "p.u., Voltage Phasor, Magnitude"
      n_u1_bus2 = "n:u1:bus2", "p.u., Voltage Phasor, Magnitude"
      n_u1pc_bus1 = "n:u1pc:bus1", "%, Voltage Phasor, Magnitude"
      n_u1pc_bus2 = "n:u1pc:bus2", "%, Voltage Phasor, Magnitude"
      n_U1_bus1 = "n:U1:bus1", "kV, Voltage Phasor, Magnitude"
      n_U1_bus2 = "n:U1:bus2", "kV, Voltage Phasor, Magnitude"
      n_phiu1_bus1 = "n:phiu1:bus1", "deg, Voltage Phasor, Angle"
      n_phiu1_bus2 = "n:phiu1:bus2", "deg, Voltage Phasor, Angle"
      n_U1l_bus1 = "n:U1l:bus1", "kV, Line-Line Voltage Phasor, Magnitude"
      n_U1l_bus2 = "n:U1l:bus2", "kV, Line-Line Voltage Phasor, Magnitude"
      n_uln_bus1_A = "n:uln:bus1:A", "p.u., Line-Neutral Voltage"
      n_uln_bus1_B = "n:uln:bus1:B", "p.u., Line-Neutral Voltage"
      n_uln_bus1_C = "n:uln:bus1:C", "p.u., Line-Neutral Voltage"
      n_uln_bus2_A = "n:uln:bus2:A", "p.u., Line-Neutral Voltage"
      n_uln_bus2_B = "n:uln:bus2:B", "p.u., Line-Neutral Voltage"
      n_uln_bus2_C = "n:uln:bus2:C", "p.u., Line-Neutral Voltage"
      n_Uln_bus1_A = "n:Uln:bus1:A", "kV, Line-Neutral Voltage"
      n_Uln_bus1_B = "n:Uln:bus1:B", "kV, Line-Neutral Voltage"
      n_Uln_bus1_C = "n:Uln:bus1:C", "kV, Line-Neutral Voltage"
      n_Uln_bus2_A = "n:Uln:bus2:A", "kV, Line-Neutral Voltage"
      n_Uln_bus2_B = "n:Uln:bus2:B", "kV, Line-Neutral Voltage"
      n_Uln_bus2_C = "n:Uln:bus2:C", "kV, Line-Neutral Voltage"
      n_un_bus1 = "n:un:bus1", "p.u., Neutral-Ground Voltage"
      n_un_bus2 = "n:un:bus2", "p.u., Neutral-Ground Voltage"
      n_Un_bus1 = "n:Un:bus1", "kV, Neutral-Ground Voltage"
      n_Un_bus2 = "n:Un:bus2", "kV, Neutral-Ground Voltage"
      n_ishc_bus1_A = "n:ishc:bus1:A", "p.u., Short-Circuit Current"
      n_ishc_bus1_B = "n:ishc:bus1:B", "p.u., Short-Circuit Current"
      n_ishc_bus1_C = "n:ishc:bus1:C", "p.u., Short-Circuit Current"
      n_ishc_bus2_A = "n:ishc:bus2:A", "p.u., Short-Circuit Current"
      n_ishc_bus2_B = "n:ishc:bus2:B", "p.u., Short-Circuit Current"
      n_ishc_bus2_C = "n:ishc:bus2:C", "p.u., Short-Circuit Current"
      n_Ishc_bus1_A = "n:Ishc:bus1:A", "kA, Short-Circuit Current"
      n_Ishc_bus1_B = "n:Ishc:bus1:B", "kA, Short-Circuit Current"
      n_Ishc_bus1_C = "n:Ishc:bus1:C", "kA, Short-Circuit Current"
      n_Ishc_bus2_A = "n:Ishc:bus2:A", "kA, Short-Circuit Current"
      n_Ishc_bus2_B = "n:Ishc:bus2:B", "kA, Short-Circuit Current"
      n_Ishc_bus2_C = "n:Ishc:bus2:C", "kA, Short-Circuit Current"
      n_inshc_bus1 = "n:inshc:bus1", "p.u., Neutral Short-Circuit Current, magnitude"
      n_inshc_bus2 = "n:inshc:bus2", "p.u., Neutral Short-Circuit Current, magnitude"
      n_Inshc_bus1 = "n:Inshc:bus1", "kA, Neutral Short-Circuit Current, magnitude"
      n_Inshc_bus2 = "n:Inshc:bus2", "kA, Neutral Short-Circuit Current, magnitude"
      n_fe_bus1 = "n:fe:bus1", "p.u., Electrical Frequency"
      n_fe_bus2 = "n:fe:bus2", "p.u., Electrical Frequency"
      n_frdev_bus1 = "n:frdev:bus1", "Hz, average frequency"
      n_frdev_bus2 = "n:frdev:bus2", "Hz, average frequency"
      n_dfehz_bus1 = "n:dfehz:bus1", "Hz, Deviation of the El. Frequency"
      n_dfehz_bus2 = "n:dfehz:bus2", "Hz, Deviation of the El. Frequency"
      n_fehz_bus1 = "n:fehz:bus1", "Hz, Electrical Frequency"
      n_fehz_bus2 = "n:fehz:bus2", "Hz, Electrical Frequency"
      n_frnom_bus1 = "n:frnom:bus1", "Hz, Nominal Frequency"
      n_frnom_bus2 = "n:frnom:bus2", "Hz, Nominal Frequency"
      s_Rin = "s:Rin", "Ohm, IN, Resistance (Input), continuous, double"
      s_psil_d = "s:psil_d", "p.u., STATE, Flux, d-axis, continuous, double"
      s_psil_q = "s:psil_q", "p.u., STATE, Flux, q-axis, continuous, double"
      s_psil_0 = "s:psil_0", "p.u., STATE, Flux, 0-Sequence, continuous, double"
      s_psil_d_dt = "s:psil_d:dt", "p.u./s, d/dt, Flux, d-axis, derivative, continuous, double"
      s_psil_q_dt = "s:psil_q:dt", "p.u./s, d/dt, Flux, q-axis, derivative, continuous, double"
      s_psil_0_dt = "s:psil_0:dt", "p.u./s, d/dt, Flux, 0-Sequence, derivative, continuous, double"
      c_Ind_r = "c:Ind_r", "H, Inductance, Phase a"
      c_Ind_s = "c:Ind_s", "H, Inductance, Phase b"
      c_Ind_t = "c:Ind_t", "H, Inductance, Phase c"
      c_Rrea_r = "c:Rrea_r", "Ohm, Resistance, Phase a"
      c_Rrea_s = "c:Rrea_s", "Ohm, Resistance, Phase b"
      c_Rrea_t = "c:Rrea_t", "Ohm, Resistance, Phase c"
      e_emtLinSig = "e:emtLinSig", "Variable inductance/reactance via input signals"
      e_uk = "e:uk", "%, Short-Circuit Voltage uk"
      e_Zd = "e:Zd", "Ohm, Impedance (absolute) Zd"
      e_lrea = "e:lrea", "mH, Inductance, L"
      e_xrea = "e:xrea", "Ohm, Reactance, X"
      e_Pcu = "e:Pcu", "kW, Copper Losses"
      e_ukr = "e:ukr", "%, Short-Circuit Voltage (Re(uk)) ukr"
      e_rrea = "e:rrea", "Ohm, Resistance, R"
      e_R0toR1 = "e:R0toR1", "R0/R1 ratio"
      e_X0toX1 = "e:X0toX1", "X0/X1 ratio"
      e_itrmt = "e:itrmt", "Type"
      e_psi0 = "e:psi0", "p.u., Knee Flux"
      e_xreapu = "e:xreapu", "p.u., Linear Reactance"
      e_xmair = "e:xmair", "p.u., Saturated Reactance"
      e_ksat = "e:ksat", "Saturation Exponent"
      e_satcur = "e:satcur", "p.u., Current (peak)"
      e_satflux = "e:satflux", "p.u., Flux (peak)"
      e_satcue = "e:satcue", "%, Current (RMS)"
      e_satvol = "e:satvol", "p.u., Voltage (RMS)"
      e_iInterPol = "e:iInterPol", "Interpolation"
      e_iFit = "e:iFit", "Data fitting"
      e_smoothfac = "e:smoothfac", "%, Smoothing Factor"
      e_iFinalSlope = "e:iFinalSlope", "Set final slope (peak values)"
      e_iHyster = "e:iHyster", "Model"
      e_Lwidth = "e:Lwidth", "%, Loop width"
      e_iLimb = "e:iLimb", "Core"
      e_iResFlux = "e:iResFlux", "Residual flux"
      e_PsiresA = "e:PsiresA", "p.u., Residual flux, ph. A"
      e_PsiresB = "e:PsiresB", "p.u., Residual flux, ph. B"
      e_PsiresC = "e:PsiresC", "p.u., Residual flux, ph. C"
      e_cPsiresC = "e:cPsiresC", "p.u., Residual flux, ph. C"
      e_aFrolich = "e:aFrolich", "Frolich equation coefficient a"
      e_bFrolich = "e:bFrolich", "Frolich equation coefficient b"
      e_cFrolich = "e:cFrolich", "Frolich equation coefficient c"
      e_xsatFrolich = "e:xsatFrolich", "Frolich equation saturated reactance (p.u.)"
      e_mseFrolich = "e:mseFrolich", "Frolich equation, mean squared error"

    class ElmStactrl(Enum):
      _init_ = 'value __doc__'


    class ElmSym(Enum):
      _init_ = 'value __doc__'

      m_u0_bus1 = "m:u0:bus1", "p.u., Zero-Sequence-Voltage"
      m_u0_bus1n = "m:u0:bus1n", "p.u., Zero-Sequence-Voltage"
      m_u1r_bus1 = "m:u1r:bus1", "p.u., Voltage Phasor, Real Part"
      m_u1r_bus1n = "m:u1r:bus1n", "p.u., Voltage Phasor, Real Part"
      m_u1i_bus1 = "m:u1i:bus1", "p.u., Voltage Phasor, Imaginary Part"
      m_u1i_bus1n = "m:u1i:bus1n", "p.u., Voltage Phasor, Imaginary Part"
      m_u1_bus1 = "m:u1:bus1", "p.u., Voltage Phasor, Magnitude"
      m_u1_bus1n = "m:u1:bus1n", "p.u., Voltage Phasor, Magnitude"
      m_U1_bus1 = "m:U1:bus1", "kV, Line-Ground Voltage Phasor, Magnitude"
      m_U1_bus1n = "m:U1:bus1n", "kV, Line-Ground Voltage Phasor, Magnitude"
      m_U1l_bus1 = "m:U1l:bus1", "kV, Line-Line Voltage Phasor, Magnitude"
      m_U1l_bus1n = "m:U1l:bus1n", "kV, Line-Line Voltage Phasor, Magnitude"
      m_phiu1_bus1 = "m:phiu1:bus1", "deg, Voltage Phasor, Angle"
      m_phiu1_bus1n = "m:phiu1:bus1n", "deg, Voltage Phasor, Angle"
      m_I_bus1_A = "m:I:bus1:A", "kA, Phase Current"
      m_I_bus1_B = "m:I:bus1:B", "kA, Phase Current"
      m_I_bus1_C = "m:I:bus1:C", "kA, Phase Current"
      m_I_bus1_N = "m:I:bus1:N", "kA, Phase Current"
      m_I_bus1n = "m:I:bus1n", "kA, Phase Current"
      m_i0_bus1 = "m:i0:bus1", "p.u., Zero-Sequence Current"
      m_i0_bus1n = "m:i0:bus1n", "p.u., Zero-Sequence Current"
      m_I0_bus1 = "m:I0:bus1", "kA, Zero-Sequence Current"
      m_I0_bus1n = "m:I0:bus1n", "kA, Zero-Sequence Current"
      m_I0x3_bus1 = "m:I0x3:bus1", "kA, 3*I0"
      m_I0x3_bus1n = "m:I0x3:bus1n", "kA, 3*I0"
      m_i1r_bus1 = "m:i1r:bus1", "p.u., Current Phasor, Real Part"
      m_i1r_bus1n = "m:i1r:bus1n", "p.u., Current Phasor, Real Part"
      m_i1i_bus1 = "m:i1i:bus1", "p.u., Current Phasor, Imaginary Part"
      m_i1i_bus1n = "m:i1i:bus1n", "p.u., Current Phasor, Imaginary Part"
      m_i1_bus1 = "m:i1:bus1", "p.u., Current Phasor, Magnitude"
      m_i1_bus1n = "m:i1:bus1n", "p.u., Current Phasor, Magnitude"
      m_I1_bus1 = "m:I1:bus1", "kA, Current Phasor, Magnitude"
      m_I1_bus1n = "m:I1:bus1n", "kA, Current Phasor, Magnitude"
      m_phii1_bus1 = "m:phii1:bus1", "deg, Current Phasor, Angle"
      m_phii1_bus1n = "m:phii1:bus1n", "deg, Current Phasor, Angle"
      m_P_bus1_A = "m:P:bus1:A", "MW, Active Power"
      m_P_bus1_B = "m:P:bus1:B", "MW, Active Power"
      m_P_bus1_C = "m:P:bus1:C", "MW, Active Power"
      m_P_bus1_N = "m:P:bus1:N", "MW, Active Power"
      m_P_bus1n = "m:P:bus1n", "MW, Active Power"
      m_Psum_bus1 = "m:Psum:bus1", "MW, Power-Phasor, Active Power"
      m_Psum_bus1n = "m:Psum:bus1n", "MW, Power-Phasor, Active Power"
      m_Qsum_bus1 = "m:Qsum:bus1", "Mvar, Power Phasor, Reactive Part"
      m_Qsum_bus1n = "m:Qsum:bus1n", "Mvar, Power Phasor, Reactive Part"
      m_Ssum_bus1 = "m:Ssum:bus1", "MVA, Power Phasor, Apparent Power"
      m_Ssum_bus1n = "m:Ssum:bus1n", "MVA, Power Phasor, Apparent Power"
      m_cosphi1_bus1 = "m:cosphi1:bus1", "Power-Phasor, Power-Factor"
      m_cosphi1_bus1n = "m:cosphi1:bus1n", "Power-Phasor, Power-Factor"
      m_i1P_bus1 = "m:i1P:bus1", "p.u., Current Phasor, Active Current"
      m_i1P_bus1n = "m:i1P:bus1n", "p.u., Current Phasor, Active Current"
      m_i1Q_bus1 = "m:i1Q:bus1", "p.u., Current Phasor, Reactive Current"
      m_i1Q_bus1n = "m:i1Q:bus1n", "p.u., Current Phasor, Reactive Current"
      m_I1P_bus1 = "m:I1P:bus1", "kA, Current Phasor, Active Current"
      m_I1P_bus1n = "m:I1P:bus1n", "kA, Current Phasor, Active Current"
      m_I1Q_bus1 = "m:I1Q:bus1", "kA, Current Phasor, Reactive Current"
      m_I1Q_bus1n = "m:I1Q:bus1n", "kA, Current Phasor, Reactive Current"
      m_u_bus1_A = "m:u:bus1:A", "p.u., Phase Voltage"
      m_u_bus1_B = "m:u:bus1:B", "p.u., Phase Voltage"
      m_u_bus1_C = "m:u:bus1:C", "p.u., Phase Voltage"
      m_u_bus1_N = "m:u:bus1:N", "p.u., Phase Voltage"
      m_u_bus1n = "m:u:bus1n", "p.u., Phase Voltage"
      m_i_bus1_A = "m:i:bus1:A", "p.u., Phase Current"
      m_i_bus1_B = "m:i:bus1:B", "p.u., Phase Current"
      m_i_bus1_C = "m:i:bus1:C", "p.u., Phase Current"
      m_i_bus1_N = "m:i:bus1:N", "p.u., Phase Current"
      m_i_bus1n = "m:i:bus1n", "p.u., Phase Current"
      m_phiu1i1_bus1 = "m:phiu1i1:bus1", "deg, Angle between Voltage and Current Phasor"
      m_phiu1i1_bus1n = "m:phiu1i1:bus1n", "deg, Angle between Voltage and Current Phasor"
      m_Tfct_bus1 = "m:Tfct:bus1", "s, Fault Clearing Time"
      m_Tfct_bus1n = "m:Tfct:bus1n", "s, Fault Clearing Time"
      m_TfctPh_bus1_A = "m:TfctPh:bus1:A", "s, Fault Clearing Time"
      m_TfctPh_bus1_B = "m:TfctPh:bus1:B", "s, Fault Clearing Time"
      m_TfctPh_bus1_C = "m:TfctPh:bus1:C", "s, Fault Clearing Time"
      m_TfctPh_bus1_N = "m:TfctPh:bus1:N", "s, Fault Clearing Time"
      m_TfctPh_bus1n = "m:TfctPh:bus1n", "s, Fault Clearing Time"
      n_u_bus1_A = "n:u:bus1:A", "p.u., Phase Voltage"
      n_u_bus1_B = "n:u:bus1:B", "p.u., Phase Voltage"
      n_u_bus1_C = "n:u:bus1:C", "p.u., Phase Voltage"
      n_u_bus1_N = "n:u:bus1:N", "p.u., Phase Voltage"
      n_u_bus1n = "n:u:bus1n", "p.u., Phase Voltage"
      n_upc_bus1_A = "n:upc:bus1:A", "%, Phase Voltage"
      n_upc_bus1_B = "n:upc:bus1:B", "%, Phase Voltage"
      n_upc_bus1_C = "n:upc:bus1:C", "%, Phase Voltage"
      n_upc_bus1_N = "n:upc:bus1:N", "%, Phase Voltage"
      n_upc_bus1n = "n:upc:bus1n", "%, Phase Voltage"
      n_U_bus1_A = "n:U:bus1:A", "kV, Phase Voltage"
      n_U_bus1_B = "n:U:bus1:B", "kV, Phase Voltage"
      n_U_bus1_C = "n:U:bus1:C", "kV, Phase Voltage"
      n_U_bus1_N = "n:U:bus1:N", "kV, Phase Voltage"
      n_U_bus1n = "n:U:bus1n", "kV, Phase Voltage"
      n_ul_bus1_A = "n:ul:bus1:A", "p.u., Line-Line Voltage"
      n_ul_bus1_B = "n:ul:bus1:B", "p.u., Line-Line Voltage"
      n_ul_bus1_C = "n:ul:bus1:C", "p.u., Line-Line Voltage"
      n_ul_bus1_N = "n:ul:bus1:N", "p.u., Line-Line Voltage"
      n_ul_bus1n = "n:ul:bus1n", "p.u., Line-Line Voltage"
      n_ulpc_bus1_A = "n:ulpc:bus1:A", "%, Line-Line Voltage"
      n_ulpc_bus1_B = "n:ulpc:bus1:B", "%, Line-Line Voltage"
      n_ulpc_bus1_C = "n:ulpc:bus1:C", "%, Line-Line Voltage"
      n_ulpc_bus1_N = "n:ulpc:bus1:N", "%, Line-Line Voltage"
      n_ulpc_bus1n = "n:ulpc:bus1n", "%, Line-Line Voltage"
      n_Ul_bus1_A = "n:Ul:bus1:A", "kV, Line-Line Voltage"
      n_Ul_bus1_B = "n:Ul:bus1:B", "kV, Line-Line Voltage"
      n_Ul_bus1_C = "n:Ul:bus1:C", "kV, Line-Line Voltage"
      n_Ul_bus1_N = "n:Ul:bus1:N", "kV, Line-Line Voltage"
      n_Ul_bus1n = "n:Ul:bus1n", "kV, Line-Line Voltage"
      n_u0_bus1 = "n:u0:bus1", "p.u., Zero-Sequence-Voltage"
      n_u0_bus1n = "n:u0:bus1n", "p.u., Zero-Sequence-Voltage"
      n_U0_bus1 = "n:U0:bus1", "kV, Zero-Sequence-Voltage"
      n_U0_bus1n = "n:U0:bus1n", "kV, Zero-Sequence-Voltage"
      n_U0x3_bus1 = "n:U0x3:bus1", "kV, 3*U0"
      n_U0x3_bus1n = "n:U0x3:bus1n", "kV, 3*U0"
      n_u1r_bus1 = "n:u1r:bus1", "p.u., Voltage Phasor, Real Part"
      n_u1r_bus1n = "n:u1r:bus1n", "p.u., Voltage Phasor, Real Part"
      n_u1i_bus1 = "n:u1i:bus1", "p.u., Voltage Phasor, Imaginary Part"
      n_u1i_bus1n = "n:u1i:bus1n", "p.u., Voltage Phasor, Imaginary Part"
      n_u1_bus1 = "n:u1:bus1", "p.u., Voltage Phasor, Magnitude"
      n_u1_bus1n = "n:u1:bus1n", "p.u., Voltage Phasor, Magnitude"
      n_u1pc_bus1 = "n:u1pc:bus1", "%, Voltage Phasor, Magnitude"
      n_u1pc_bus1n = "n:u1pc:bus1n", "%, Voltage Phasor, Magnitude"
      n_U1_bus1 = "n:U1:bus1", "kV, Voltage Phasor, Magnitude"
      n_U1_bus1n = "n:U1:bus1n", "kV, Voltage Phasor, Magnitude"
      n_phiu1_bus1 = "n:phiu1:bus1", "deg, Voltage Phasor, Angle"
      n_phiu1_bus1n = "n:phiu1:bus1n", "deg, Voltage Phasor, Angle"
      n_U1l_bus1 = "n:U1l:bus1", "kV, Line-Line Voltage Phasor, Magnitude"
      n_U1l_bus1n = "n:U1l:bus1n", "kV, Line-Line Voltage Phasor, Magnitude"
      n_uln_bus1_A = "n:uln:bus1:A", "p.u., Line-Neutral Voltage"
      n_uln_bus1_B = "n:uln:bus1:B", "p.u., Line-Neutral Voltage"
      n_uln_bus1_C = "n:uln:bus1:C", "p.u., Line-Neutral Voltage"
      n_uln_bus1_N = "n:uln:bus1:N", "p.u., Line-Neutral Voltage"
      n_uln_bus1n = "n:uln:bus1n", "p.u., Line-Neutral Voltage"
      n_Uln_bus1_A = "n:Uln:bus1:A", "kV, Line-Neutral Voltage"
      n_Uln_bus1_B = "n:Uln:bus1:B", "kV, Line-Neutral Voltage"
      n_Uln_bus1_C = "n:Uln:bus1:C", "kV, Line-Neutral Voltage"
      n_Uln_bus1_N = "n:Uln:bus1:N", "kV, Line-Neutral Voltage"
      n_Uln_bus1n = "n:Uln:bus1n", "kV, Line-Neutral Voltage"
      n_un_bus1 = "n:un:bus1", "p.u., Neutral-Ground Voltage"
      n_un_bus1n = "n:un:bus1n", "p.u., Neutral-Ground Voltage"
      n_Un_bus1 = "n:Un:bus1", "kV, Neutral-Ground Voltage"
      n_Un_bus1n = "n:Un:bus1n", "kV, Neutral-Ground Voltage"
      n_ishc_bus1_A = "n:ishc:bus1:A", "p.u., Short-Circuit Current"
      n_ishc_bus1_B = "n:ishc:bus1:B", "p.u., Short-Circuit Current"
      n_ishc_bus1_C = "n:ishc:bus1:C", "p.u., Short-Circuit Current"
      n_ishc_bus1_N = "n:ishc:bus1:N", "p.u., Short-Circuit Current"
      n_ishc_bus1n = "n:ishc:bus1n", "p.u., Short-Circuit Current"
      n_Ishc_bus1_A = "n:Ishc:bus1:A", "kA, Short-Circuit Current"
      n_Ishc_bus1_B = "n:Ishc:bus1:B", "kA, Short-Circuit Current"
      n_Ishc_bus1_C = "n:Ishc:bus1:C", "kA, Short-Circuit Current"
      n_Ishc_bus1_N = "n:Ishc:bus1:N", "kA, Short-Circuit Current"
      n_Ishc_bus1n = "n:Ishc:bus1n", "kA, Short-Circuit Current"
      n_inshc_bus1 = "n:inshc:bus1", "p.u., Neutral Short-Circuit Current, magnitude"
      n_inshc_bus1n = "n:inshc:bus1n", "p.u., Neutral Short-Circuit Current, magnitude"
      n_Inshc_bus1 = "n:Inshc:bus1", "kA, Neutral Short-Circuit Current, magnitude"
      n_Inshc_bus1n = "n:Inshc:bus1n", "kA, Neutral Short-Circuit Current, magnitude"
      n_fe_bus1 = "n:fe:bus1", "p.u., Electrical Frequency"
      n_fe_bus1n = "n:fe:bus1n", "p.u., Electrical Frequency"
      n_frdev_bus1 = "n:frdev:bus1", "Hz, average frequency"
      n_frdev_bus1n = "n:frdev:bus1n", "Hz, average frequency"
      n_dfehz_bus1 = "n:dfehz:bus1", "Hz, Deviation of the El. Frequency"
      n_dfehz_bus1n = "n:dfehz:bus1n", "Hz, Deviation of the El. Frequency"
      n_fehz_bus1 = "n:fehz:bus1", "Hz, Electrical Frequency"
      n_fehz_bus1n = "n:fehz:bus1n", "Hz, Electrical Frequency"
      n_frnom_bus1 = "n:frnom:bus1", "Hz, Nominal Frequency"
      n_frnom_bus1n = "n:frnom:bus1n", "Hz, Nominal Frequency"
      s_ve = "s:ve", "p.u., IN, Excitation Voltage, continuous, double"
      s_pt = "s:pt", "p.u., IN, Turbine Power, continuous, double"
      s_xmdm = "s:xmdm", "p.u., IN, Torque Input, continuous, double"
      s_dw = "s:dw", "p.u., IN, dw, continuous, double"
      s_freflocal = "s:freflocal", "p.u., IN, Local Reference Frequency, continuous, double"
      s_xme = "s:xme", "p.u., OUT, Electrical Torque, continuous, double"
      s_xmt = "s:xmt", "p.u., OUT, Mechanical Torque, continuous, double"
      s_xspeed = "s:xspeed", "p.u., OUT, Speed, continuous, double"
      s_ie = "s:ie", "p.u., OUT, Excitation Current (non-reciprocal p.u. system) continuous, double"
      s_pgt = "s:pgt", "p.u., OUT, Electrical Power, continuous, double"
      s_xphi = "s:xphi", "rad, OUT, Rotor position d-axis referred to the reference bus voltage, continuous, double"
      s_fe = "s:fe", "p.u., OUT, Frequency Output, continuous, double"
      s_ut = "s:ut", "p.u., OUT, Terminal voltage, continuous, double"
      s_utr = "s:utr", "p.u., OUT, Terminal voltage, real part, continuous, double"
      s_uti = "s:uti", "p.u., OUT, Terminal voltage, imaginary part, continuous, double"
      s_cur1 = "s:cur1", "p.u., OUT, Positive-sequence current, magnitude, continuous, double"
      s_cur1r = "s:cur1r", "p.u., OUT, Positive-sequence current, real part, continuous, double"
      s_cur1i = "s:cur1i", "p.u., OUT, Positive-sequence current, imaginary part, continuous, double"
      s_P1 = "s:P1", "MW, OUT, Positive-sequence, active power continuous, double"
      s_Q1 = "s:Q1", "Mvar, OUT, Positive-sequence, reactive power, continuous, double"
      s_ussd = "s:ussd", "p.u., OUT, Subtransient Voltage, d-axis, continuous, double"
      s_ussq = "s:ussq", "p.u., OUT, Subtransient Voltage, q-axis, continuous, double"
      s_fipol = "s:fipol", "deg, OUT, Internal rotor angle, continuous, double"
      s_psi1d = "s:psi1d", "p.u., STATE, Flux in 1d-damper winding, continuous, double"
      s_psifd = "s:psifd", "p.u., STATE, Excitation Flux continuous, double"
      s_psi1q = "s:psi1q", "p.u., STATE, Flux in 1q-damper winding, continuous, double"
      s_psi2q = "s:psi2q", "p.u., STATE, Flux in 2q-damper winding, continuous, double"
      s_speed = "s:speed", "p.u., STATE, Speed, continuous, double"
      s_phi = "s:phi", "rad, STATE, Rotor position d-axis referred to the reference bus voltage, continuous, double"
      s_psi1d_dt = "s:psi1d:dt", "p.u./s, d/dt, Flux in 1d-damper winding, derivative, continuous, double"
      s_psifd_dt = "s:psifd:dt", "p.u./s, d/dt, Excitation Flux, derivative, continuous, double"
      s_psi1q_dt = "s:psi1q:dt", "p.u./s, d/dt, Flux in 1q-damper winding, derivative, continuous, double"
      s_psi2q_dt = "s:psi2q:dt", "p.u./s, d/dt, Flux in 2q-damper winding, derivative, continuous, double"
      s_speed_dt = "s:speed:dt", "p.u./s, d/dt, Speed, derivative, continuous, double"
      s_phi_dt = "s:phi:dt", "rad/s, d/dt, Rotor position d-axis referred to the reference bus voltage, derivative continuous, double"
      c_ud = "c:ud", "p.u., Voltage, d-axis"
      c_uq = "c:uq", "p.u., Voltage, q-axis"
      c_id = "c:id", "p.u., Stator Current, d-axis"
      c_iq = "c:iq", "p.u., Stator Current, q-axis"
      c_psid = "c:psid", "p.u., Stator Flux, d-axis"
      c_psiq = "c:psiq", "p.u., Stator Flux, q-axis"
      c_psim = "c:psim", "p.u., Magnetisation Flux"
      c_psidss = "c:psidss", "p.u., Subtransient Flux, d-axis"
      c_psiqss = "c:psiqss", "p.u., Subtransient Flux, q-axis"
      c_xmem = "c:xmem", "p.u., Electrical Torque"
      c_addmt = "c:addmt", "p.u., Additional Torque"
      c_xmdpe = "c:xmdpe", "p.u., Damping Torque based on power"
      c_xmdkd = "c:xmdkd", "p.u., Damping Torque"
      c_ifd = "c:ifd", "p.u., Current in excitation winding"
      c_i1d = "c:i1d", "p.u., Current in 1d-damper winding"
      c_i1q = "c:i1q", "p.u., Current in 1q-damper winding"
      c_i2q = "c:i2q", "p.u., Current in 2q-damper winding"
      c_iD = "c:iD", "p.u., Current in 1d-damper winding (non-reciprocal p.u. system)"
      c_ix = "c:ix", "p.u., Current in 2q-damper winding (non-reciprocal p.u. system)"
      c_iQ = "c:iQ", "p.u., Current in 1q-damper winding (non-reciprocal p.u. system)"
      c_psie = "c:psie", "p.u., Excitation Flux (non-reciprocal p.u. system)"
      c_psiD = "c:psiD", "p.u., Flux in 1d-damper winding (non-reciprocal p.u. system)"
      c_psiQ = "c:psiQ", "p.u., Flux in 1q-damper winding (non-reciprocal p.u. system)"
      c_psix = "c:psix", "p.u., Flux in 2q-damper winding (non-reciprocal p.u. system)"
      c_fi = "c:fi", "deg, Rotor position d-axis referred to the reference bus voltage"
      c_firot = "c:firot", "deg, External rotor angle"
      c_firel = "c:firel", "deg, Rotor angle referred to the reference machine rotor angle"
      c_dfrotx = "c:dfrotx", "deg, Maximum rotor angle difference"
      c_dfrotxM1 = "c:dfrotxM1", "Machine 1 used to obtain dfrotx"
      c_dfrotxM2 = "c:dfrotxM2", "Machine 2 used to obtain dfrotx"
      c_dorhz = "c:dorhz", "Hz, Speed Deviation"
      c_Ptspin = "c:Ptspin", "MW, Total Turbine Power of Area"
      c_Pnomspin = "c:Pnomspin", "MW, Total Nominal Power of Spinning Machines in Area"
      c_Presspin = "c:Presspin", "MW, Spinning Reserve of Area"
      c_Tag = "c:Tag", "s, Acceleration Time Constant"
      c_Tagav = "c:Tagav", "s, System Inertia of Area"
      c_xadu = "c:xadu", "p.u., Unsaturated mutual reactance, d-axis"
      c_xaqu = "c:xaqu", "p.u., Unsaturated mutual reactance, q-axis"
      c_satd = "c:satd", "p.u., Saturation factor, d-axis"
      c_satq = "c:satq", "p.u., Saturation factor, q-axis"
      c_xad = "c:xad", "p.u., Saturated mutual reactance, d-axis"
      c_xaq = "c:xaq", "p.u., Saturated mutual reactance, q-axis"
      c_rfd = "c:rfd", "p.u., Resistance of excitation winding"
      c_xfd = "c:xfd", "p.u., Reactance of excitation winding"
      c_r1d = "c:r1d", "p.u., Resistance of 1d-damper winding"
      c_x1d = "c:x1d", "p.u., Reactance of 1d-damper winding"
      c_r1q = "c:r1q", "p.u., Resistance of 1q-damper winding"
      c_x1q = "c:x1q", "p.u., Reactance of 1q-damper winding"
      c_r2q = "c:r2q", "p.u., Resistance of 2q-damper winding"
      c_x2q = "c:x2q", "p.u., Reactance of 2q-damper winding"
      c_xd = "c:xd", "p.u., Synchronous Reactance, d-axis"
      c_xq = "c:xq", "p.u., Synchronous Reactance, q-axis"
      c_xdss = "c:xdss", "p.u., Subtransient Reactance, d-axis"
      c_xqss = "c:xqss", "p.u., Subtransient Reactance, q-axis"
      c_r0 = "c:r0", "p.u., Zero Sequence Resistance"
      c_x0 = "c:x0", "p.u., Zero Sequence Reactance"
      c_rearth = "c:rearth", "p.u., Grounding Resistance"
      c_xearth = "c:xearth", "p.u., Grounding Reactance"
      c_sgnn = "c:sgnn", "MVA, Total rated apparent power (ngnum*sgn)"
      c_c2 = "c:c2", "Voltage Coefficient (uknom/ugn)"
      c_cosn = "c:cosn", "Nominal-Power Factor"
      c_hpi = "c:hpi", "rad/s, Nominal-Angular-Frequency"
      c_xss = "c:xss", "p.u., (xdss+xqss)/2"
      c_dxss = "c:dxss", "p.u., (xdss-xqss)/2"
      c_Gm = "c:Gm", "Conversion Factor (=xadu/rfd)"
      c_ve_rated = "c:ve_rated", "p.u., Rated Excitation Voltage"
      c_Pmaxspin = "c:Pmaxspin", "MW, Total Maximum Power of Spinning Machines in Area"
      c_Pmax = "c:Pmax", "MW, Upper limit of active power"
      c_Td0s = "c:Td0s", "s, Td0'"
      c_fe_bus = "c:fe_bus", "El. frequency"
      c_mdmlp = "c:mdmlp", "p.u., Proportional Factor"
      c_mdmex = "c:mdmex", "Exponent"
      c_p_mag = "c:p_mag", "Participation, Magnitude"
      c_p_phi = "c:p_phi", "deg, Participation, Angle"
      c_p_mags = "c:p_mags", "Participation, Magnitude (signed)"
      c_lEVec_mag = "c:lEVec_mag", "Controllability, Magnitude"
      c_lEVec_phi = "c:lEVec_phi", "deg, Controllability, Angle"
      c_lEVec_mags = "c:lEVec_mags", "Controllability, Magnitude (signed)"
      c_rEVec_mag = "c:rEVec_mag", "Observability, Magnitude"
      c_rEVec_phi = "c:rEVec_phi", "deg, Observability, Angle"
      c_rEVec_mags = "c:rEVec_mags", "Observability, Magnitude (signed)"
      e_efdBaseRatio = "e:efdBaseRatio", "Efd base ratio"
      e_ifdBaseType = "e:ifdBaseType", "Excitation base mode"
      e_c_pmdm = "e:c_pmdm", "Mdm"
      e_mdmlp = "e:mdmlp", "p.u., Proportional Factor"
      e_mdmex = "e:mdmex", "Exponent"
      e_Jme = "e:Jme", "kgm^2, Moment of inertia"
      e_gratio = "e:gratio", "Gear ratio"
      e_ctag = "e:ctag", "s, Acceleration time const. from type"
      e_ctagtot = "e:ctagtot", "s, Total acceleration time const."
      e_i_avr = "e:i_avr", "Use integrated AVR for motor starting"
      e_rf_st = "e:rf_st", "p.u., Starting field resistance"
      e_speed_th = "e:speed_th", "p.u., Trigger excitation at speed"
      e_ve_const = "e:ve_const", "p.u., Constant field voltage"
      e_ve_rated = "e:ve_rated", "p.u., Rated field voltage"
      t_model_inp = "t:model_inp", "Model"
      t_iopt_data = "t:iopt_data", "Input parameters"
      t_iner_inp = "t:iner_inp", "Input mode"
      t_tag = "t:tag", "s, Acceleration time const. Tag (rated to Pgn)"
      t_tags = "t:tags", "s, Acceleration time const. Tag (rated to Sgn)"
      t_h = "t:h", "s, Inertia constant H (rated to Sgn)"
      t_hpn = "t:hpn", "s, Inertia constant H (rated to Pgn)"
      t_J = "t:J", "kgm^2, Moment of inertia J"
      t_polepairs = "t:polepairs", "No. of pole pairs"
      t_frequ = "t:frequ", "Hz, Nominal frequency"
      t_rstr = "t:rstr", "p.u., rstr"
      t_xl = "t:xl", "p.u., xl"
      t_i_xl = "t:i_xl", "xl is current dependent"
      t_iturbo = "t:iturbo", "Rotor type"
      t_tds = "t:tds", "s, Td'"
      t_tqs = "t:tqs", "s, Tq'"
      t_tds0 = "t:tds0", "s, Td0'"
      t_tqs0 = "t:tqs0", "s, Tq0'"
      t_tdss = "t:tdss", "s, Td''"
      t_tqss = "t:tqss", "s, Tq''"
      t_tdss0 = "t:tdss0", "s, Td0''"
      t_tqss0 = "t:tqss0", "s, Tq0''"
      t_xd = "t:xd", "p.u., xd"
      t_xq = "t:xq", "p.u., xq"
      t_xrl = "t:xrl", "p.u., xrld"
      t_xrlq = "t:xrlq", "p.u., xrlq"
      t_xds = "t:xds", "p.u., xd'"
      t_xqs = "t:xqs", "p.u., xq'"
      t_xdss = "t:xdss", "p.u., xd''"
      t_xqss = "t:xqss", "p.u., xq''"
      t_xstr = "t:xstr", "p.u., xstr"
      t_x1d = "t:x1d", "p.u., x1d"
      t_r1d = "t:r1d", "p.u., r1d"
      t_xfd = "t:xfd", "p.u., xfd"
      t_rfd = "t:rfd", "p.u., rfd"
      t_x1q = "t:x1q", "p.u., x1q"
      t_r1q = "t:r1q", "p.u., r1q"
      t_x2q = "t:x2q", "p.u., x2q"
      t_r2q = "t:r2q", "p.u., r2q"
      t_xrl2d = "t:xrl2d", "p.u., xrl2d"
      t_x2d = "t:x2d", "p.u., x2d"
      t_r2d = "t:r2d", "p.u., r2d"
      t_t1d = "t:t1d", "s, t1d"
      t_t2d = "t:t2d", "s, t2d"
      t_t3d = "t:t3d", "s, t3d"
      t_t4d = "t:t4d", "s, t4d"
      t_t5d = "t:t5d", "s, t5d"
      t_t6d = "t:t6d", "s, t6d"
      t_t7d = "t:t7d", "s, t7d"
      t_t8d = "t:t8d", "s, t8d"
      t_xad = "t:xad", "p.u., xad"
      t_xaq = "t:xaq", "p.u., xaq"
      t_x3q = "t:x3q", "p.u., x3q"
      t_r3q = "t:r3q", "p.u., r3q"
      t_t1q = "t:t1q", "s, t1q"
      t_t2q = "t:t2q", "s, t2q"
      t_t3q = "t:t3q", "s, t3q"
      t_t4q = "t:t4q", "s, t4q"
      t_t5q = "t:t5q", "s, t5q"
      t_t6q = "t:t6q", "s, t6q"
      t_x1dc = "t:x1dc", "p.u., x1dc"
      t_x1de = "t:x1de", "p.u., x1de"
      t_r1de = "t:r1de", "p.u., r1de"
      t_xfde = "t:xfde", "p.u., xfde"
      t_rfde = "t:rfde", "p.u., rfde"
      t_x0sy = "t:x0sy", "p.u., Reactance x0"
      t_r0sy = "t:r0sy", "p.u., Resistance r0"
      t_x1qc = "t:x1qc", "p.u., x1qc"
      t_x1qe = "t:x1qe", "p.u., x1qe"
      t_r1qe = "t:r1qe", "p.u., r1qe"
      t_psim = "t:psim", "p.u., Rotor flux"
      t_slipmin = "t:slipmin", "%, Minimum slip"
      t_curTab = "t:curTab", "p.u., Current"
      t_xlTab = "t:xlTab", "p.u., xl"
      t_depSmoothFac = "t:depSmoothFac", "%, Dependency curve smoothing factor"
      t_isat = "t:isat", "Main flux saturation"
      t_isatSalient = "t:isatSalient", "Saturation"
      t_sg10 = "t:sg10", "p.u., SG10"
      t_sg12 = "t:sg12", "p.u., SG12"
      t_sg10q = "t:sg10q", "p.u., SG10 q-axis"
      t_sg12q = "t:sg12q", "p.u., SG12 q-axis"
      t_satv = "t:satv", "p.u., Term. voltage"
      t_satse = "t:satse", "p.u., SG(u)"
      t_ifd = "t:ifd", "%, ie"
      t_satvq = "t:satvq", "p.u., Term. voltage"
      t_satseq = "t:satseq", "p.u., SG(u)"
      t_ifdq = "t:ifdq", "%, ie"
      t_smoothFac = "t:smoothFac", "%, Smoothing factor"
      t_dpu = "t:dpu", "Turbine shaft friction torque coefficient"
      t_dkd = "t:dkd", "Damping torque coefficient"
      t_dpe = "t:dpe", "Damping torque coefficient based on power"
      t_i_speedD = "t:i_speedD", "Speed deviation based on"
      t_i_exactConv = "t:i_exactConv", "Exact conversion of time constants"
      r_c_pmdm = "r:c_pmdm", "Mdm"

    class ElmTerm(Enum):
      _init_ = 'value __doc__'

      m_u_A = "m:u:A", "p.u., Phase Voltage"
      m_u_B = "m:u:B", "p.u., Phase Voltage"
      m_u_C = "m:u:C", "p.u., Phase Voltage"
      m_upc_A = "m:upc:A", "%, Phase Voltage"
      m_upc_B = "m:upc:B", "%, Phase Voltage"
      m_upc_C = "m:upc:C", "%, Phase Voltage"
      m_U_A = "m:U:A", "kV, Phase Voltage"
      m_U_B = "m:U:B", "kV, Phase Voltage"
      m_U_C = "m:U:C", "kV, Phase Voltage"
      m_ul_A = "m:ul:A", "p.u., Line-Line Voltage"
      m_ul_B = "m:ul:B", "p.u., Line-Line Voltage"
      m_ul_C = "m:ul:C", "p.u., Line-Line Voltage"
      m_ulpc_A = "m:ulpc:A", "%, Line-Line Voltage"
      m_ulpc_B = "m:ulpc:B", "%, Line-Line Voltage"
      m_ulpc_C = "m:ulpc:C", "%, Line-Line Voltage"
      m_Ul_A = "m:Ul:A", "kV, Line-Line Voltage"
      m_Ul_B = "m:Ul:B", "kV, Line-Line Voltage"
      m_Ul_C = "m:Ul:C", "kV, Line-Line Voltage"
      m_u0 = "m:u0", "p.u., Zero-Sequence-Voltage"
      m_U0 = "m:U0", "kV, Zero-Sequence-Voltage"
      m_U0x3 = "m:U0x3", "kV, 3*U0"
      m_u1r = "m:u1r", "p.u., Voltage Phasor, Real Part"
      m_u1i = "m:u1i", "p.u., Voltage Phasor, Imaginary Part"
      m_u1 = "m:u1", "p.u., Voltage Phasor, Magnitude"
      m_u1pc = "m:u1pc", "%, Voltage Phasor, Magnitude"
      m_U1 = "m:U1", "kV, Voltage Phasor, Magnitude"
      m_phiu1 = "m:phiu1", "deg, Voltage Phasor, Angle"
      m_U1l = "m:U1l", "kV, Line-Line Voltage Phasor, Magnitude"
      m_uln_A = "m:uln:A", "p.u., Line-Neutral Voltage"
      m_uln_B = "m:uln:B", "p.u., Line-Neutral Voltage"
      m_uln_C = "m:uln:C", "p.u., Line-Neutral Voltage"
      m_Uln_A = "m:Uln:A", "kV, Line-Neutral Voltage"
      m_Uln_B = "m:Uln:B", "kV, Line-Neutral Voltage"
      m_Uln_C = "m:Uln:C", "kV, Line-Neutral Voltage"
      m_un = "m:un", "p.u., Neutral-Ground Voltage"
      m_Un = "m:Un", "kV, Neutral-Ground Voltage"
      m_ishc_A = "m:ishc:A", "p.u., Short-Circuit Current"
      m_ishc_B = "m:ishc:B", "p.u., Short-Circuit Current"
      m_ishc_C = "m:ishc:C", "p.u., Short-Circuit Current"
      m_Ishc_A = "m:Ishc:A", "kA, Short-Circuit Current"
      m_Ishc_B = "m:Ishc:B", "kA, Short-Circuit Current"
      m_Ishc_C = "m:Ishc:C", "kA, Short-Circuit Current"
      m_inshc = "m:inshc", "p.u., Neutral Short-Circuit Current, magnitude"
      m_Inshc = "m:Inshc", "kA, Neutral Short-Circuit Current, magnitude"
      m_fe = "m:fe", "p.u., Electrical Frequency"
      m_frdev = "m:frdev", "Hz, average frequency"
      m_dfehz = "m:dfehz", "Hz, Deviation of the El. Frequency"
      m_fehz = "m:fehz", "Hz, Electrical Frequency"
      m_frnom = "m:frnom", "Hz, Nominal Frequency"

    class ElmTr2(Enum):
      _init_ = 'value __doc__'

      m_u0_bushv = "m:u0:bushv", "p.u., Zero-Sequence-Voltage"
      m_u0_buslv = "m:u0:buslv", "p.u., Zero-Sequence-Voltage"
      m_u0_bushvn = "m:u0:bushvn", "p.u., Zero-Sequence-Voltage"
      m_u0_buslvn = "m:u0:buslvn", "p.u., Zero-Sequence-Voltage"
      m_u1r_bushv = "m:u1r:bushv", "p.u., Voltage Phasor, Real Part"
      m_u1r_buslv = "m:u1r:buslv", "p.u., Voltage Phasor, Real Part"
      m_u1r_bushvn = "m:u1r:bushvn", "p.u., Voltage Phasor, Real Part"
      m_u1r_buslvn = "m:u1r:buslvn", "p.u., Voltage Phasor, Real Part"
      m_u1i_bushv = "m:u1i:bushv", "p.u., Voltage Phasor, Imaginary Part"
      m_u1i_buslv = "m:u1i:buslv", "p.u., Voltage Phasor, Imaginary Part"
      m_u1i_bushvn = "m:u1i:bushvn", "p.u., Voltage Phasor, Imaginary Part"
      m_u1i_buslvn = "m:u1i:buslvn", "p.u., Voltage Phasor, Imaginary Part"
      m_u1_bushv = "m:u1:bushv", "p.u., Voltage Phasor, Magnitude"
      m_u1_buslv = "m:u1:buslv", "p.u., Voltage Phasor, Magnitude"
      m_u1_bushvn = "m:u1:bushvn", "p.u., Voltage Phasor, Magnitude"
      m_u1_buslvn = "m:u1:buslvn", "p.u., Voltage Phasor, Magnitude"
      m_U1_bushv = "m:U1:bushv", "kV, Line-Ground Voltage Phasor, Magnitude"
      m_U1_buslv = "m:U1:buslv", "kV, Line-Ground Voltage Phasor, Magnitude"
      m_U1_bushvn = "m:U1:bushvn", "kV, Line-Ground Voltage Phasor, Magnitude"
      m_U1_buslvn = "m:U1:buslvn", "kV, Line-Ground Voltage Phasor, Magnitude"
      m_U1l_bushv = "m:U1l:bushv", "kV, Line-Line Voltage Phasor, Magnitude"
      m_U1l_buslv = "m:U1l:buslv", "kV, Line-Line Voltage Phasor, Magnitude"
      m_U1l_bushvn = "m:U1l:bushvn", "kV, Line-Line Voltage Phasor, Magnitude"
      m_U1l_buslvn = "m:U1l:buslvn", "kV, Line-Line Voltage Phasor, Magnitude"
      m_phiu1_bushv = "m:phiu1:bushv", "deg, Voltage Phasor, Angle"
      m_phiu1_buslv = "m:phiu1:buslv", "deg, Voltage Phasor, Angle"
      m_phiu1_bushvn = "m:phiu1:bushvn", "deg, Voltage Phasor, Angle"
      m_phiu1_buslvn = "m:phiu1:buslvn", "deg, Voltage Phasor, Angle"
      m_I_bushv_A = "m:I:bushv:A", "kA, Phase Current"
      m_I_bushv_B = "m:I:bushv:B", "kA, Phase Current"
      m_I_bushv_C = "m:I:bushv:C", "kA, Phase Current"
      m_I_bushv_N = "m:I:bushv:N", "kA, Phase Current"
      m_I_buslv_A = "m:I:buslv:A", "kA, Phase Current"
      m_I_buslv_B = "m:I:buslv:B", "kA, Phase Current"
      m_I_buslv_C = "m:I:buslv:C", "kA, Phase Current"
      m_I_buslv_N = "m:I:buslv:N", "kA, Phase Current"
      m_I_bushvn = "m:I:bushvn", "kA, Phase Current"
      m_I_buslvn = "m:I:buslvn", "kA, Phase Current"
      m_i0_bushv = "m:i0:bushv", "p.u., Zero-Sequence Current"
      m_i0_buslv = "m:i0:buslv", "p.u., Zero-Sequence Current"
      m_i0_bushvn = "m:i0:bushvn", "p.u., Zero-Sequence Current"
      m_i0_buslvn = "m:i0:buslvn", "p.u., Zero-Sequence Current"
      m_I0_bushv = "m:I0:bushv", "kA, Zero-Sequence Current"
      m_I0_buslv = "m:I0:buslv", "kA, Zero-Sequence Current"
      m_I0_bushvn = "m:I0:bushvn", "kA, Zero-Sequence Current"
      m_I0_buslvn = "m:I0:buslvn", "kA, Zero-Sequence Current"
      m_I0x3_bushv = "m:I0x3:bushv", "kA, 3*I0"
      m_I0x3_buslv = "m:I0x3:buslv", "kA, 3*I0"
      m_I0x3_bushvn = "m:I0x3:bushvn", "kA, 3*I0"
      m_I0x3_buslvn = "m:I0x3:buslvn", "kA, 3*I0"
      m_i1r_bushv = "m:i1r:bushv", "p.u., Current Phasor, Real Part"
      m_i1r_buslv = "m:i1r:buslv", "p.u., Current Phasor, Real Part"
      m_i1r_bushvn = "m:i1r:bushvn", "p.u., Current Phasor, Real Part"
      m_i1r_buslvn = "m:i1r:buslvn", "p.u., Current Phasor, Real Part"
      m_i1i_bushv = "m:i1i:bushv", "p.u., Current Phasor, Imaginary Part"
      m_i1i_buslv = "m:i1i:buslv", "p.u., Current Phasor, Imaginary Part"
      m_i1i_bushvn = "m:i1i:bushvn", "p.u., Current Phasor, Imaginary Part"
      m_i1i_buslvn = "m:i1i:buslvn", "p.u., Current Phasor, Imaginary Part"
      m_i1_bushv = "m:i1:bushv", "p.u., Current Phasor, Magnitude"
      m_i1_buslv = "m:i1:buslv", "p.u., Current Phasor, Magnitude"
      m_i1_bushvn = "m:i1:bushvn", "p.u., Current Phasor, Magnitude"
      m_i1_buslvn = "m:i1:buslvn", "p.u., Current Phasor, Magnitude"
      m_I1_bushv = "m:I1:bushv", "kA, Current Phasor, Magnitude"
      m_I1_buslv = "m:I1:buslv", "kA, Current Phasor, Magnitude"
      m_I1_bushvn = "m:I1:bushvn", "kA, Current Phasor, Magnitude"
      m_I1_buslvn = "m:I1:buslvn", "kA, Current Phasor, Magnitude"
      m_phii1_bushv = "m:phii1:bushv", "deg, Current Phasor, Angle"
      m_phii1_buslv = "m:phii1:buslv", "deg, Current Phasor, Angle"
      m_phii1_bushvn = "m:phii1:bushvn", "deg, Current Phasor, Angle"
      m_phii1_buslvn = "m:phii1:buslvn", "deg, Current Phasor, Angle"
      m_P_bushv_A = "m:P:bushv:A", "MW, Active Power"
      m_P_bushv_B = "m:P:bushv:B", "MW, Active Power"
      m_P_bushv_C = "m:P:bushv:C", "MW, Active Power"
      m_P_bushv_N = "m:P:bushv:N", "MW, Active Power"
      m_P_buslv_A = "m:P:buslv:A", "MW, Active Power"
      m_P_buslv_B = "m:P:buslv:B", "MW, Active Power"
      m_P_buslv_C = "m:P:buslv:C", "MW, Active Power"
      m_P_buslv_N = "m:P:buslv:N", "MW, Active Power"
      m_P_bushvn = "m:P:bushvn", "MW, Active Power"
      m_P_buslvn = "m:P:buslvn", "MW, Active Power"
      m_Psum_bushv = "m:Psum:bushv", "MW, Power-Phasor, Active Power"
      m_Psum_buslv = "m:Psum:buslv", "MW, Power-Phasor, Active Power"
      m_Psum_bushvn = "m:Psum:bushvn", "MW, Power-Phasor, Active Power"
      m_Psum_buslvn = "m:Psum:buslvn", "MW, Power-Phasor, Active Power"
      m_Qsum_bushv = "m:Qsum:bushv", "Mvar, Power Phasor, Reactive Part"
      m_Qsum_buslv = "m:Qsum:buslv", "Mvar, Power Phasor, Reactive Part"
      m_Qsum_bushvn = "m:Qsum:bushvn", "Mvar, Power Phasor, Reactive Part"
      m_Qsum_buslvn = "m:Qsum:buslvn", "Mvar, Power Phasor, Reactive Part"
      m_Ssum_bushv = "m:Ssum:bushv", "MVA, Power Phasor, Apparent Power"
      m_Ssum_buslv = "m:Ssum:buslv", "MVA, Power Phasor, Apparent Power"
      m_Ssum_bushvn = "m:Ssum:bushvn", "MVA, Power Phasor, Apparent Power"
      m_Ssum_buslvn = "m:Ssum:buslvn", "MVA, Power Phasor, Apparent Power"
      m_cosphi1_bushv = "m:cosphi1:bushv", "Power-Phasor, Power-Factor"
      m_cosphi1_buslv = "m:cosphi1:buslv", "Power-Phasor, Power-Factor"
      m_cosphi1_bushvn = "m:cosphi1:bushvn", "Power-Phasor, Power-Factor"
      m_cosphi1_buslvn = "m:cosphi1:buslvn", "Power-Phasor, Power-Factor"
      m_i1P_bushv = "m:i1P:bushv", "p.u., Current Phasor, Active Current"
      m_i1P_buslv = "m:i1P:buslv", "p.u., Current Phasor, Active Current"
      m_i1P_bushvn = "m:i1P:bushvn", "p.u., Current Phasor, Active Current"
      m_i1P_buslvn = "m:i1P:buslvn", "p.u., Current Phasor, Active Current"
      m_i1Q_bushv = "m:i1Q:bushv", "p.u., Current Phasor, Reactive Current"
      m_i1Q_buslv = "m:i1Q:buslv", "p.u., Current Phasor, Reactive Current"
      m_i1Q_bushvn = "m:i1Q:bushvn", "p.u., Current Phasor, Reactive Current"
      m_i1Q_buslvn = "m:i1Q:buslvn", "p.u., Current Phasor, Reactive Current"
      m_I1P_bushv = "m:I1P:bushv", "kA, Current Phasor, Active Current"
      m_I1P_buslv = "m:I1P:buslv", "kA, Current Phasor, Active Current"
      m_I1P_bushvn = "m:I1P:bushvn", "kA, Current Phasor, Active Current"
      m_I1P_buslvn = "m:I1P:buslvn", "kA, Current Phasor, Active Current"
      m_I1Q_bushv = "m:I1Q:bushv", "kA, Current Phasor, Reactive Current"
      m_I1Q_buslv = "m:I1Q:buslv", "kA, Current Phasor, Reactive Current"
      m_I1Q_bushvn = "m:I1Q:bushvn", "kA, Current Phasor, Reactive Current"
      m_I1Q_buslvn = "m:I1Q:buslvn", "kA, Current Phasor, Reactive Current"
      m_u_bushv_A = "m:u:bushv:A", "p.u., Phase Voltage"
      m_u_bushv_B = "m:u:bushv:B", "p.u., Phase Voltage"
      m_u_bushv_C = "m:u:bushv:C", "p.u., Phase Voltage"
      m_u_bushv_N = "m:u:bushv:N", "p.u., Phase Voltage"
      m_u_buslv_A = "m:u:buslv:A", "p.u., Phase Voltage"
      m_u_buslv_B = "m:u:buslv:B", "p.u., Phase Voltage"
      m_u_buslv_C = "m:u:buslv:C", "p.u., Phase Voltage"
      m_u_buslv_N = "m:u:buslv:N", "p.u., Phase Voltage"
      m_u_bushvn = "m:u:bushvn", "p.u., Phase Voltage"
      m_u_buslvn = "m:u:buslvn", "p.u., Phase Voltage"
      m_i_bushv_A = "m:i:bushv:A", "p.u., Phase Current"
      m_i_bushv_B = "m:i:bushv:B", "p.u., Phase Current"
      m_i_bushv_C = "m:i:bushv:C", "p.u., Phase Current"
      m_i_bushv_N = "m:i:bushv:N", "p.u., Phase Current"
      m_i_buslv_A = "m:i:buslv:A", "p.u., Phase Current"
      m_i_buslv_B = "m:i:buslv:B", "p.u., Phase Current"
      m_i_buslv_C = "m:i:buslv:C", "p.u., Phase Current"
      m_i_buslv_N = "m:i:buslv:N", "p.u., Phase Current"
      m_i_bushvn = "m:i:bushvn", "p.u., Phase Current"
      m_i_buslvn = "m:i:buslvn", "p.u., Phase Current"
      m_phiu1i1_bushv = "m:phiu1i1:bushv", "deg, Angle between Voltage and Current Phasor"
      m_phiu1i1_buslv = "m:phiu1i1:buslv", "deg, Angle between Voltage and Current Phasor"
      m_phiu1i1_bushvn = "m:phiu1i1:bushvn", "deg, Angle between Voltage and Current Phasor"
      m_phiu1i1_buslvn = "m:phiu1i1:buslvn", "deg, Angle between Voltage and Current Phasor"
      m_Tfct_bushv = "m:Tfct:bushv", "s, Fault Clearing Time"
      m_Tfct_buslv = "m:Tfct:buslv", "s, Fault Clearing Time"
      m_Tfct_bushvn = "m:Tfct:bushvn", "s, Fault Clearing Time"
      m_Tfct_buslvn = "m:Tfct:buslvn", "s, Fault Clearing Time"
      m_TfctPh_bushv_A = "m:TfctPh:bushv:A", "s, Fault Clearing Time"
      m_TfctPh_bushv_B = "m:TfctPh:bushv:B", "s, Fault Clearing Time"
      m_TfctPh_bushv_C = "m:TfctPh:bushv:C", "s, Fault Clearing Time"
      m_TfctPh_bushv_N = "m:TfctPh:bushv:N", "s, Fault Clearing Time"
      m_TfctPh_buslv_A = "m:TfctPh:buslv:A", "s, Fault Clearing Time"
      m_TfctPh_buslv_B = "m:TfctPh:buslv:B", "s, Fault Clearing Time"
      m_TfctPh_buslv_C = "m:TfctPh:buslv:C", "s, Fault Clearing Time"
      m_TfctPh_buslv_N = "m:TfctPh:buslv:N", "s, Fault Clearing Time"
      m_TfctPh_bushvn = "m:TfctPh:bushvn", "s, Fault Clearing Time"
      m_TfctPh_buslvn = "m:TfctPh:buslvn", "s, Fault Clearing Time"
      n_u_bushv_A = "n:u:bushv:A", "p.u., Phase Voltage"
      n_u_bushv_B = "n:u:bushv:B", "p.u., Phase Voltage"
      n_u_bushv_C = "n:u:bushv:C", "p.u., Phase Voltage"
      n_u_bushv_N = "n:u:bushv:N", "p.u., Phase Voltage"
      n_u_buslv_A = "n:u:buslv:A", "p.u., Phase Voltage"
      n_u_buslv_B = "n:u:buslv:B", "p.u., Phase Voltage"
      n_u_buslv_C = "n:u:buslv:C", "p.u., Phase Voltage"
      n_u_buslv_N = "n:u:buslv:N", "p.u., Phase Voltage"
      n_u_bushvn = "n:u:bushvn", "p.u., Phase Voltage"
      n_u_buslvn = "n:u:buslvn", "p.u., Phase Voltage"
      n_upc_bushv_A = "n:upc:bushv:A", "%, Phase Voltage"
      n_upc_bushv_B = "n:upc:bushv:B", "%, Phase Voltage"
      n_upc_bushv_C = "n:upc:bushv:C", "%, Phase Voltage"
      n_upc_bushv_N = "n:upc:bushv:N", "%, Phase Voltage"
      n_upc_buslv_A = "n:upc:buslv:A", "%, Phase Voltage"
      n_upc_buslv_B = "n:upc:buslv:B", "%, Phase Voltage"
      n_upc_buslv_C = "n:upc:buslv:C", "%, Phase Voltage"
      n_upc_buslv_N = "n:upc:buslv:N", "%, Phase Voltage"
      n_upc_bushvn = "n:upc:bushvn", "%, Phase Voltage"
      n_upc_buslvn = "n:upc:buslvn", "%, Phase Voltage"
      n_U_bushv_A = "n:U:bushv:A", "kV, Phase Voltage"
      n_U_bushv_B = "n:U:bushv:B", "kV, Phase Voltage"
      n_U_bushv_C = "n:U:bushv:C", "kV, Phase Voltage"
      n_U_bushv_N = "n:U:bushv:N", "kV, Phase Voltage"
      n_U_buslv_A = "n:U:buslv:A", "kV, Phase Voltage"
      n_U_buslv_B = "n:U:buslv:B", "kV, Phase Voltage"
      n_U_buslv_C = "n:U:buslv:C", "kV, Phase Voltage"
      n_U_buslv_N = "n:U:buslv:N", "kV, Phase Voltage"
      n_U_bushvn = "n:U:bushvn", "kV, Phase Voltage"
      n_U_buslvn = "n:U:buslvn", "kV, Phase Voltage"
      n_ul_bushv_A = "n:ul:bushv:A", "p.u., Line-Line Voltage"
      n_ul_bushv_B = "n:ul:bushv:B", "p.u., Line-Line Voltage"
      n_ul_bushv_C = "n:ul:bushv:C", "p.u., Line-Line Voltage"
      n_ul_bushv_N = "n:ul:bushv:N", "p.u., Line-Line Voltage"
      n_ul_buslv_A = "n:ul:buslv:A", "p.u., Line-Line Voltage"
      n_ul_buslv_B = "n:ul:buslv:B", "p.u., Line-Line Voltage"
      n_ul_buslv_C = "n:ul:buslv:C", "p.u., Line-Line Voltage"
      n_ul_buslv_N = "n:ul:buslv:N", "p.u., Line-Line Voltage"
      n_ul_bushvn = "n:ul:bushvn", "p.u., Line-Line Voltage"
      n_ul_buslvn = "n:ul:buslvn", "p.u., Line-Line Voltage"
      n_ulpc_bushv_A = "n:ulpc:bushv:A", "%, Line-Line Voltage"
      n_ulpc_bushv_B = "n:ulpc:bushv:B", "%, Line-Line Voltage"
      n_ulpc_bushv_C = "n:ulpc:bushv:C", "%, Line-Line Voltage"
      n_ulpc_bushv_N = "n:ulpc:bushv:N", "%, Line-Line Voltage"
      n_ulpc_buslv_A = "n:ulpc:buslv:A", "%, Line-Line Voltage"
      n_ulpc_buslv_B = "n:ulpc:buslv:B", "%, Line-Line Voltage"
      n_ulpc_buslv_C = "n:ulpc:buslv:C", "%, Line-Line Voltage"
      n_ulpc_buslv_N = "n:ulpc:buslv:N", "%, Line-Line Voltage"
      n_ulpc_bushvn = "n:ulpc:bushvn", "%, Line-Line Voltage"
      n_ulpc_buslvn = "n:ulpc:buslvn", "%, Line-Line Voltage"
      n_Ul_bushv_A = "n:Ul:bushv:A", "kV, Line-Line Voltage"
      n_Ul_bushv_B = "n:Ul:bushv:B", "kV, Line-Line Voltage"
      n_Ul_bushv_C = "n:Ul:bushv:C", "kV, Line-Line Voltage"
      n_Ul_bushv_N = "n:Ul:bushv:N", "kV, Line-Line Voltage"
      n_Ul_buslv_A = "n:Ul:buslv:A", "kV, Line-Line Voltage"
      n_Ul_buslv_B = "n:Ul:buslv:B", "kV, Line-Line Voltage"
      n_Ul_buslv_C = "n:Ul:buslv:C", "kV, Line-Line Voltage"
      n_Ul_buslv_N = "n:Ul:buslv:N", "kV, Line-Line Voltage"
      n_Ul_bushvn = "n:Ul:bushvn", "kV, Line-Line Voltage"
      n_Ul_buslvn = "n:Ul:buslvn", "kV, Line-Line Voltage"
      n_u0_bushv = "n:u0:bushv", "p.u., Zero-Sequence-Voltage"
      n_u0_buslv = "n:u0:buslv", "p.u., Zero-Sequence-Voltage"
      n_u0_bushvn = "n:u0:bushvn", "p.u., Zero-Sequence-Voltage"
      n_u0_buslvn = "n:u0:buslvn", "p.u., Zero-Sequence-Voltage"
      n_U0_bushv = "n:U0:bushv", "kV, Zero-Sequence-Voltage"
      n_U0_buslv = "n:U0:buslv", "kV, Zero-Sequence-Voltage"
      n_U0_bushvn = "n:U0:bushvn", "kV, Zero-Sequence-Voltage"
      n_U0_buslvn = "n:U0:buslvn", "kV, Zero-Sequence-Voltage"
      n_U0x3_bushv = "n:U0x3:bushv", "kV, 3*U0"
      n_U0x3_buslv = "n:U0x3:buslv", "kV, 3*U0"
      n_U0x3_bushvn = "n:U0x3:bushvn", "kV, 3*U0"
      n_U0x3_buslvn = "n:U0x3:buslvn", "kV, 3*U0"
      n_u1r_bushv = "n:u1r:bushv", "p.u., Voltage Phasor, Real Part"
      n_u1r_buslv = "n:u1r:buslv", "p.u., Voltage Phasor, Real Part"
      n_u1r_bushvn = "n:u1r:bushvn", "p.u., Voltage Phasor, Real Part"
      n_u1r_buslvn = "n:u1r:buslvn", "p.u., Voltage Phasor, Real Part"
      n_u1i_bushv = "n:u1i:bushv", "p.u., Voltage Phasor, Imaginary Part"
      n_u1i_buslv = "n:u1i:buslv", "p.u., Voltage Phasor, Imaginary Part"
      n_u1i_bushvn = "n:u1i:bushvn", "p.u., Voltage Phasor, Imaginary Part"
      n_u1i_buslvn = "n:u1i:buslvn", "p.u., Voltage Phasor, Imaginary Part"
      n_u1_bushv = "n:u1:bushv", "p.u., Voltage Phasor, Magnitude"
      n_u1_buslv = "n:u1:buslv", "p.u., Voltage Phasor, Magnitude"
      n_u1_bushvn = "n:u1:bushvn", "p.u., Voltage Phasor, Magnitude"
      n_u1_buslvn = "n:u1:buslvn", "p.u., Voltage Phasor, Magnitude"
      n_u1pc_bushv = "n:u1pc:bushv", "%, Voltage Phasor, Magnitude"
      n_u1pc_buslv = "n:u1pc:buslv", "%, Voltage Phasor, Magnitude"
      n_u1pc_bushvn = "n:u1pc:bushvn", "%, Voltage Phasor, Magnitude"
      n_u1pc_buslvn = "n:u1pc:buslvn", "%, Voltage Phasor, Magnitude"
      n_U1_bushv = "n:U1:bushv", "kV, Voltage Phasor, Magnitude"
      n_U1_buslv = "n:U1:buslv", "kV, Voltage Phasor, Magnitude"
      n_U1_bushvn = "n:U1:bushvn", "kV, Voltage Phasor, Magnitude"
      n_U1_buslvn = "n:U1:buslvn", "kV, Voltage Phasor, Magnitude"
      n_phiu1_bushv = "n:phiu1:bushv", "deg, Voltage Phasor, Angle"
      n_phiu1_buslv = "n:phiu1:buslv", "deg, Voltage Phasor, Angle"
      n_phiu1_bushvn = "n:phiu1:bushvn", "deg, Voltage Phasor, Angle"
      n_phiu1_buslvn = "n:phiu1:buslvn", "deg, Voltage Phasor, Angle"
      n_U1l_bushv = "n:U1l:bushv", "kV, Line-Line Voltage Phasor, Magnitude"
      n_U1l_buslv = "n:U1l:buslv", "kV, Line-Line Voltage Phasor, Magnitude"
      n_U1l_bushvn = "n:U1l:bushvn", "kV, Line-Line Voltage Phasor, Magnitude"
      n_U1l_buslvn = "n:U1l:buslvn", "kV, Line-Line Voltage Phasor, Magnitude"
      n_uln_bushv_A = "n:uln:bushv:A", "p.u., Line-Neutral Voltage"
      n_uln_bushv_B = "n:uln:bushv:B", "p.u., Line-Neutral Voltage"
      n_uln_bushv_C = "n:uln:bushv:C", "p.u., Line-Neutral Voltage"
      n_uln_bushv_N = "n:uln:bushv:N", "p.u., Line-Neutral Voltage"
      n_uln_buslv_A = "n:uln:buslv:A", "p.u., Line-Neutral Voltage"
      n_uln_buslv_B = "n:uln:buslv:B", "p.u., Line-Neutral Voltage"
      n_uln_buslv_C = "n:uln:buslv:C", "p.u., Line-Neutral Voltage"
      n_uln_buslv_N = "n:uln:buslv:N", "p.u., Line-Neutral Voltage"
      n_uln_bushvn = "n:uln:bushvn", "p.u., Line-Neutral Voltage"
      n_uln_buslvn = "n:uln:buslvn", "p.u., Line-Neutral Voltage"
      n_Uln_bushv_A = "n:Uln:bushv:A", "kV, Line-Neutral Voltage"
      n_Uln_bushv_B = "n:Uln:bushv:B", "kV, Line-Neutral Voltage"
      n_Uln_bushv_C = "n:Uln:bushv:C", "kV, Line-Neutral Voltage"
      n_Uln_bushv_N = "n:Uln:bushv:N", "kV, Line-Neutral Voltage"
      n_Uln_buslv_A = "n:Uln:buslv:A", "kV, Line-Neutral Voltage"
      n_Uln_buslv_B = "n:Uln:buslv:B", "kV, Line-Neutral Voltage"
      n_Uln_buslv_C = "n:Uln:buslv:C", "kV, Line-Neutral Voltage"
      n_Uln_buslv_N = "n:Uln:buslv:N", "kV, Line-Neutral Voltage"
      n_Uln_bushvn = "n:Uln:bushvn", "kV, Line-Neutral Voltage"
      n_Uln_buslvn = "n:Uln:buslvn", "kV, Line-Neutral Voltage"
      n_un_bushv = "n:un:bushv", "p.u., Neutral-Ground Voltage"
      n_un_buslv = "n:un:buslv", "p.u., Neutral-Ground Voltage"
      n_un_bushvn = "n:un:bushvn", "p.u., Neutral-Ground Voltage"
      n_un_buslvn = "n:un:buslvn", "p.u., Neutral-Ground Voltage"
      n_Un_bushv = "n:Un:bushv", "kV, Neutral-Ground Voltage"
      n_Un_buslv = "n:Un:buslv", "kV, Neutral-Ground Voltage"
      n_Un_bushvn = "n:Un:bushvn", "kV, Neutral-Ground Voltage"
      n_Un_buslvn = "n:Un:buslvn", "kV, Neutral-Ground Voltage"
      n_ishc_bushv_A = "n:ishc:bushv:A", "p.u., Short-Circuit Current"
      n_ishc_bushv_B = "n:ishc:bushv:B", "p.u., Short-Circuit Current"
      n_ishc_bushv_C = "n:ishc:bushv:C", "p.u., Short-Circuit Current"
      n_ishc_bushv_N = "n:ishc:bushv:N", "p.u., Short-Circuit Current"
      n_ishc_buslv_A = "n:ishc:buslv:A", "p.u., Short-Circuit Current"
      n_ishc_buslv_B = "n:ishc:buslv:B", "p.u., Short-Circuit Current"
      n_ishc_buslv_C = "n:ishc:buslv:C", "p.u., Short-Circuit Current"
      n_ishc_buslv_N = "n:ishc:buslv:N", "p.u., Short-Circuit Current"
      n_ishc_bushvn = "n:ishc:bushvn", "p.u., Short-Circuit Current"
      n_ishc_buslvn = "n:ishc:buslvn", "p.u., Short-Circuit Current"
      n_Ishc_bushv_A = "n:Ishc:bushv:A", "kA, Short-Circuit Current"
      n_Ishc_bushv_B = "n:Ishc:bushv:B", "kA, Short-Circuit Current"
      n_Ishc_bushv_C = "n:Ishc:bushv:C", "kA, Short-Circuit Current"
      n_Ishc_bushv_N = "n:Ishc:bushv:N", "kA, Short-Circuit Current"
      n_Ishc_buslv_A = "n:Ishc:buslv:A", "kA, Short-Circuit Current"
      n_Ishc_buslv_B = "n:Ishc:buslv:B", "kA, Short-Circuit Current"
      n_Ishc_buslv_C = "n:Ishc:buslv:C", "kA, Short-Circuit Current"
      n_Ishc_buslv_N = "n:Ishc:buslv:N", "kA, Short-Circuit Current"
      n_Ishc_bushvn = "n:Ishc:bushvn", "kA, Short-Circuit Current"
      n_Ishc_buslvn = "n:Ishc:buslvn", "kA, Short-Circuit Current"
      n_inshc_bushv = "n:inshc:bushv", "p.u., Neutral Short-Circuit Current, magnitude"
      n_inshc_buslv = "n:inshc:buslv", "p.u., Neutral Short-Circuit Current, magnitude"
      n_inshc_bushvn = "n:inshc:bushvn", "p.u., Neutral Short-Circuit Current, magnitude"
      n_inshc_buslvn = "n:inshc:buslvn", "p.u., Neutral Short-Circuit Current, magnitude"
      n_Inshc_bushv = "n:Inshc:bushv", "kA, Neutral Short-Circuit Current, magnitude"
      n_Inshc_buslv = "n:Inshc:buslv", "kA, Neutral Short-Circuit Current, magnitude"
      n_Inshc_bushvn = "n:Inshc:bushvn", "kA, Neutral Short-Circuit Current, magnitude"
      n_Inshc_buslvn = "n:Inshc:buslvn", "kA, Neutral Short-Circuit Current, magnitude"
      n_fe_bushv = "n:fe:bushv", "p.u., Electrical Frequency"
      n_fe_buslv = "n:fe:buslv", "p.u., Electrical Frequency"
      n_fe_bushvn = "n:fe:bushvn", "p.u., Electrical Frequency"
      n_fe_buslvn = "n:fe:buslvn", "p.u., Electrical Frequency"
      n_frdev_bushv = "n:frdev:bushv", "Hz, average frequency"
      n_frdev_buslv = "n:frdev:buslv", "Hz, average frequency"
      n_frdev_bushvn = "n:frdev:bushvn", "Hz, average frequency"
      n_frdev_buslvn = "n:frdev:buslvn", "Hz, average frequency"
      n_dfehz_bushv = "n:dfehz:bushv", "Hz, Deviation of the El. Frequency"
      n_dfehz_buslv = "n:dfehz:buslv", "Hz, Deviation of the El. Frequency"
      n_dfehz_bushvn = "n:dfehz:bushvn", "Hz, Deviation of the El. Frequency"
      n_dfehz_buslvn = "n:dfehz:buslvn", "Hz, Deviation of the El. Frequency"
      n_fehz_bushv = "n:fehz:bushv", "Hz, Electrical Frequency"
      n_fehz_buslv = "n:fehz:buslv", "Hz, Electrical Frequency"
      n_fehz_bushvn = "n:fehz:bushvn", "Hz, Electrical Frequency"
      n_fehz_buslvn = "n:fehz:buslvn", "Hz, Electrical Frequency"
      n_frnom_bushv = "n:frnom:bushv", "Hz, Nominal Frequency"
      n_frnom_buslv = "n:frnom:buslv", "Hz, Nominal Frequency"
      n_frnom_bushvn = "n:frnom:bushvn", "Hz, Nominal Frequency"
      n_frnom_buslvn = "n:frnom:buslvn", "Hz, Nominal Frequency"
      s_gm_a = "s:gm_a", "p.u., IN, Magnetising Conductance continuous, double"
      s_gm_b = "s:gm_b", "p.u., IN, Magnetising Conductance continuous, double"
      s_gm_c = "s:gm_c", "p.u., IN, Magnetising Conductance continuous, double"
      s_nntapin = "s:nntapin", "IN, Tap 1, Controller Input continuous, double"
      s_psim_a = "s:psim_a", "p.u., OUT, Magnetising Flux, continuous, double"
      s_psim_b = "s:psim_b", "p.u., OUT, Magnetising Flux, continuous, double"
      s_psim_c = "s:psim_c", "p.u., OUT, Magnetising Flux, continuous, double"
      s_psimd = "s:psimd", "p.u., STATE, Magnetising Flux, alpha-component, continuous, double"
      s_psimq = "s:psimq", "p.u., STATE, Magnetising Flux, beta-component, continuous, double"
      s_psim0 = "s:psim0", "p.u., STATE, Magnetising Flux, gamma-component, continuous, double"
      s_psimd_dt = "s:psimd:dt", "p.u./s, d/dt, Magnetising Flux, alpha-component, derivative, continuous, double"
      s_psimq_dt = "s:psimq:dt", "p.u./s, d/dt, Magnetising Flux, beta-component, derivative, continuous, double"
      s_psim0_dt = "s:psim0:dt", "p.u./s, d/dt, Magnetising Flux, gamma-component, derivative, continuous, double"
      c_nntap = "c:nntap", "Tap 1, Current Position"
      c_nntap_int = "c:nntap_int", "Tap 1, relative Position"
      c_ntpmn = "c:ntpmn", "Tap 1, Min. Position"
      c_ntpmx = "c:ntpmx", "Tap 1, Max. Position"
      c_dut = "c:dut", "Tap 1: Voltage/Tap"
      c_phitrrd = "c:phitrrd", "rad, Tap 1:, Phase/Tap"
      c_nt2agrd = "c:nt2agrd", "rad, Phase Angle"
      c_tap_side = "c:tap_side", "0"
      c_nntap2 = "c:nntap2", "Tap 2, Current Position"
      c_ntpmn2 = "c:ntpmn2", "Tap 2, Min. Position"
      c_ntpmx2 = "c:ntpmx2", "Tap 2, Max. Position"
      c_dut2 = "c:dut2", "Tap 2: Voltage/Tap"
      c_phitrrd2 = "c:phitrrd2", "rad, Tap 2:, Phase/Tap"
      c_tap_side2 = "c:tap_side2", "0"
      c_rshv = "c:rshv", "p.u., Leakage Resistance (HV-Side)"
      c_lshv = "c:lshv", "p.u., Leakage Inductance (HV-Side)"
      c_rslv = "c:rslv", "p.u., Leakage Resistance (LV-Side)"
      c_lslv = "c:lslv", "p.u., Leakage Inductance (LV-Side)"
      c_bm = "c:bm", "p.u., Magnetising Susceptance"
      c_gm = "c:gm", "p.u., Magnetising Conductance"
      c_thv_r = "c:thv:r", "Complex Winding Ratio HV, Real Part"
      c_thv_i = "c:thv:i", "Complex Winding Ratio HV, Imaginary Part"
      c_thv_phi = "c:thv:phi", "Complex Winding Ratio HV, Angle"
      c_thv_mag = "c:thv:mag", "Complex Winding Ratio HV, Magnitude"
      c_thv_phirad = "c:thv:phirad", "Complex Winding Ratio HV, Angle"
      c_tlv_r = "c:tlv:r", "Complex Winding Ratio LV, Real Part"
      c_tlv_i = "c:tlv:i", "Complex Winding Ratio LV, Imaginary Part"
      c_tlv_phi = "c:tlv:phi", "Complex Winding Ratio LV, Angle"
      c_tlv_mag = "c:tlv:mag", "Complex Winding Ratio LV, Magnitude"
      c_tlv_phirad = "c:tlv:phirad", "Complex Winding Ratio LV, Angle"
      c_i_ea_h = "c:i_ea_h", "Flag: HV-Side grounded"
      c_i_ea_l = "c:i_ea_l", "Flag: LV-Side grounded"
      c_i_star_h = "c:i_star_h", "Flag: HV-Side Y"
      c_i_star_l = "c:i_star_l", "Flag: LV-Side Y"
      c_rs0hv = "c:rs0hv", "p.u., Leakage Resistance (HV-Side)"
      c_ls0hv = "c:ls0hv", "p.u., Leakage Inductance (HV-Side)"
      c_rs0lv = "c:rs0lv", "p.u., Leakage Resistance (LV-Side)"
      c_ls0lv = "c:ls0lv", "p.u., Leakage Inductance (LV-Side)"
      c_Ustar_h = "c:Ustar_h", "kV, Star-Point Voltage (HV)"
      c_Ustar_l = "c:Ustar_l", "kV, Star-Point Voltage (LV)"
      c_Istar_h = "c:Istar_h", "kA, Star-Point Current (HV)"
      c_Istar_l = "c:Istar_l", "kA, Star-Point Current (LV)"
      c_bm0 = "c:bm0", "p.u., Zero Sequence Magnetising Admittance"
      c_w_0 = "c:w_0", "Nominal winding ratio"
      c_uh_r = "c:uh:r", "p.u., Aux.-value, Real Part"
      c_uh_i = "c:uh:i", "p.u., Aux.-value, Imaginary Part"
      c_uh_phi = "c:uh:phi", "deg, Aux.-value, Angle"
      c_uh_mag = "c:uh:mag", "p.u., Aux.-value, Magnitude"
      c_uh_phirad = "c:uh:phirad", "rad, Aux.-value, Angle"
      c_hpi = "c:hpi", "rad/s, Angular frequency"
      c_psimknee = "c:psimknee", "p.u., Knee Flux"
      c_bmsat = "c:bmsat", "p.u., Saturated Magnetising Susceptance"
      c_i_sat = "c:i_sat", "Saturated Model"
      c_im_a = "c:im_a", "p.u., Magnetising Current, Phase a"
      c_im_b = "c:im_b", "p.u., Magnetising Current, Phase b"
      c_im_c = "c:im_c", "p.u., Magnetising Current, Phase c"
      c_ksat = "c:ksat", "Saturation Exponent"
      c_i_five = "c:i_five", "Five Limb Design"
      c_physt = "c:physt", "kW, Hysteresis losses"
      e_dutap = "e:dutap", "%, Additional Voltage per Tap"
      e_dphitap = "e:dphitap", "deg, Additional Angle per Tap"
      e_phitr = "e:phitr", "deg, Phase of du"
      e_nntap = "e:nntap", "Tap Position"
      e_iTaps = "e:iTaps", "According to Measurement Report"
      e_dutap2 = "e:dutap2", "%, Additional Voltage per Tap"
      e_dphitap2 = "e:dphitap2", "deg, Additional Angle per Tap"
      e_phitr2 = "e:phitr2", "deg, Phase of du"
      e_nntap2 = "e:nntap2", "Tap Position"
      e_iMeasLoc = "e:iMeasLoc", "Measured at"
      e_mTaps = "e:mTaps", "Measurement Report"
      e_iopt_hf = "e:iopt_hf", "Consider Capacitances"
      e_Cg_h = "e:Cg_h", "uF, Capacitance HV-Ground"
      e_Cg_l = "e:Cg_l", "uF, Capacitance LV-Ground"
      e_Cc1_hl = "e:Cc1_hl", "uF, Capacitance HV-LV, 1-Sequence"
      e_Cc0_hl = "e:Cc0_hl", "uF, Capacitance HV-LV, 0-Sequence"
      e_iResFlux = "e:iResFlux", "Residual flux"
      e_PsiresA = "e:PsiresA", "p.u., Phase A"
      e_PsiresB = "e:PsiresB", "p.u., Phase B"
      e_PsiresC = "e:PsiresC", "p.u., Phase C"
      e_cPsiresC = "e:cPsiresC", "p.u., Phase C"
      t_curmg = "t:curmg", "%, No Load Current"
      t_pfe = "t:pfe", "kW, No Load Losses"
      t_zx0hl_n = "t:zx0hl_n", "Mag. Impedance/uk0"
      t_rtox0_n = "t:rtox0_n", "Mag. R/X"
      t_itrdl = "t:itrdl", "x,Pos.Seq. HV-Side"
      t_itrdl_lv = "t:itrdl_lv", "x,Pos.Seq. LV-Side"
      t_itrdr = "t:itrdr", "r,Pos.Seq. HV-Side"
      t_itrdr_lv = "t:itrdr_lv", "r,Pos.Seq. LV-Side"
      t_zx0hl_h = "t:zx0hl_h", "z, Zero Sequ. HV-Side"
      t_zx0hl_l = "t:zx0hl_l", "z, Zero Sequ. LV-Side"
      t_itapch = "t:itapch", "Tap Changer 1"
      t_tapchtype = "t:tapchtype", "Type"
      t_tap_side = "t:tap_side", "at Side"
      t_dutap = "t:dutap", "%, Additional Voltage per Tap"
      t_dphitap = "t:dphitap", "deg, Additional Angle per Tap"
      t_phitr = "t:phitr", "deg, Phase of du"
      t_nntap0 = "t:nntap0", "Neutral Position"
      t_ntpmn = "t:ntpmn", "Minimum Position"
      t_ntpmx = "t:ntpmx", "Maximum Position"
      t_itapch2 = "t:itapch2", "Tap Changer 2"
      t_tapchtype2 = "t:tapchtype2", "Type"
      t_tap_side2 = "t:tap_side2", "at Side"
      t_dutap2 = "t:dutap2", "%, Additional Voltage per Tap"
      t_dphitap2 = "t:dphitap2", "deg, Additional Angle per Tap"
      t_phitr2 = "t:phitr2", "deg, Phase of du"
      t_nntap02 = "t:nntap02", "Neutral Position"
      t_ntpmn2 = "t:ntpmn2", "Minimum Position"
      t_ntpmx2 = "t:ntpmx2", "Maximum Position"
      t_itapzdep = "t:itapzdep", "Tap dependent impedance"
      t_uktr = "t:uktr", "%, Short-Circuit Voltage uk"
      t_uktmn = "t:uktmn", "%, uk (min. tap)"
      t_uktmx = "t:uktmx", "%, uk (max. tap)"
      t_x1pu = "t:x1pu", "p.u., Reactance x1"
      t_x1putmn = "t:x1putmn", "p.u., x1 (min. tap)"
      t_x1putmx = "t:x1putmx", "p.u., x1 (max. tap)"
      t_pcutr = "t:pcutr", "kW, Copper Losses"
      t_uktrr = "t:uktrr", "%, SHC-Voltage (Re(uk)) ukr"
      t_xtor = "t:xtor", "Ratio X/R"
      t_r1pu = "t:r1pu", "p.u., Resistance r1"
      t_pcutmn = "t:pcutmn", "kW, Pcu (min. tap)"
      t_ukrtmn = "t:ukrtmn", "%, Re(uk) (min. tap)"
      t_xtortmn = "t:xtortmn", "X/R (min. tap)"
      t_r1putmn = "t:r1putmn", "p.u., r1 (min. tap)"
      t_pcutmx = "t:pcutmx", "kW, Pcu (max. tap)"
      t_ukrtmx = "t:ukrtmx", "%, Re(uk) (max. tap)"
      t_xtortmx = "t:xtortmx", "X/R (max. tap)"
      t_r1putmx = "t:r1putmx", "p.u., r1 (max. tap)"
      t_uk0tr = "t:uk0tr", "%, Short-Circuit Voltage uk0"
      t_uk0tmn = "t:uk0tmn", "%, uk0 (min. tap)"
      t_uk0tmx = "t:uk0tmx", "%, uk0 (max. tap)"
      t_x0pu = "t:x0pu", "p.u., Reactance x0"
      t_x0putmn = "t:x0putmn", "p.u., x0 (min. tap)"
      t_x0putmx = "t:x0putmx", "p.u., x0 (max. tap)"
      t_ur0tr = "t:ur0tr", "%, SHC-Voltage (Re(uk0)) uk0r"
      t_x0tor0 = "t:x0tor0", "Ratio X0/R0"
      t_r0pu = "t:r0pu", "p.u., Resistance r0"
      t_uk0rtmn = "t:uk0rtmn", "%, Re(uk0) (min. tap)"
      t_x0tor0tmn = "t:x0tor0tmn", "X0/R0 (min. tap)"
      t_r0putmn = "t:r0putmn", "p.u., r0 (min. tap)"
      t_uk0rtmx = "t:uk0rtmx", "%, Re(uk0) (max. tap)"
      t_x0tor0tmx = "t:x0tor0tmx", "X0/R0 (max. tap)"
      t_r0putmx = "t:r0putmx", "p.u., r0 (max. tap)"
      t_itrmt = "t:itrmt", "Type"
      t_psi0 = "t:psi0", "p.u., Knee Flux"
      t_xmlin = "t:xmlin", "p.u., Linear Reactance"
      t_xmair = "t:xmair", "p.u., Saturated Reactance"
      t_ksat = "t:ksat", "Saturation Exponent"
      t_satcur = "t:satcur", "p.u., Current (peak)"
      t_satflux = "t:satflux", "p.u., Flux (peak)"
      t_satcue = "t:satcue", "%, Current (RMS)"
      t_satvol = "t:satvol", "p.u., Voltage (RMS)"
      t_iInterPol = "t:iInterPol", "Interpolation"
      t_iFit = "t:iFit", "Data fitting"
      t_smoothfac = "t:smoothfac", "%, Smoothing Factor"
      t_iFinalSlope = "t:iFinalSlope", "Set final slope (peak values)"
      t_iHyster = "t:iHyster", "Model"
      t_Lwidth = "t:Lwidth", "%, Loop width"
      t_iLimb = "t:iLimb", "Core"
      t_aFrolich = "t:aFrolich", "Frolich equation coefficient a"
      t_bFrolich = "t:bFrolich", "Frolich equation coefficient b"
      t_cFrolich = "t:cFrolich", "Frolich equation coefficient c"
      t_xsatFrolich = "t:xsatFrolich", "Frolich equation saturated reactance (p.u.)"
      t_mseFrolich = "t:mseFrolich", "Frolich equation, mean squared error"

    class ElmTr3(Enum):
      _init_ = 'value __doc__'

      m_u0_bushv = "m:u0:bushv", "p.u., Zero-Sequence-Voltage"
      m_u0_busmv = "m:u0:busmv", "p.u., Zero-Sequence-Voltage"
      m_u0_buslv = "m:u0:buslv", "p.u., Zero-Sequence-Voltage"
      m_u0_bushvn = "m:u0:bushvn", "p.u., Zero-Sequence-Voltage"
      m_u0_busmvn = "m:u0:busmvn", "p.u., Zero-Sequence-Voltage"
      m_u0_buslvn = "m:u0:buslvn", "p.u., Zero-Sequence-Voltage"
      m_u1r_bushv = "m:u1r:bushv", "p.u., Voltage Phasor, Real Part"
      m_u1r_busmv = "m:u1r:busmv", "p.u., Voltage Phasor, Real Part"
      m_u1r_buslv = "m:u1r:buslv", "p.u., Voltage Phasor, Real Part"
      m_u1r_bushvn = "m:u1r:bushvn", "p.u., Voltage Phasor, Real Part"
      m_u1r_busmvn = "m:u1r:busmvn", "p.u., Voltage Phasor, Real Part"
      m_u1r_buslvn = "m:u1r:buslvn", "p.u., Voltage Phasor, Real Part"
      m_u1i_bushv = "m:u1i:bushv", "p.u., Voltage Phasor, Imaginary Part"
      m_u1i_busmv = "m:u1i:busmv", "p.u., Voltage Phasor, Imaginary Part"
      m_u1i_buslv = "m:u1i:buslv", "p.u., Voltage Phasor, Imaginary Part"
      m_u1i_bushvn = "m:u1i:bushvn", "p.u., Voltage Phasor, Imaginary Part"
      m_u1i_busmvn = "m:u1i:busmvn", "p.u., Voltage Phasor, Imaginary Part"
      m_u1i_buslvn = "m:u1i:buslvn", "p.u., Voltage Phasor, Imaginary Part"
      m_u1_bushv = "m:u1:bushv", "p.u., Voltage Phasor, Magnitude"
      m_u1_busmv = "m:u1:busmv", "p.u., Voltage Phasor, Magnitude"
      m_u1_buslv = "m:u1:buslv", "p.u., Voltage Phasor, Magnitude"
      m_u1_bushvn = "m:u1:bushvn", "p.u., Voltage Phasor, Magnitude"
      m_u1_busmvn = "m:u1:busmvn", "p.u., Voltage Phasor, Magnitude"
      m_u1_buslvn = "m:u1:buslvn", "p.u., Voltage Phasor, Magnitude"
      m_U1_bushv = "m:U1:bushv", "kV, Line-Ground Voltage Phasor, Magnitude"
      m_U1_busmv = "m:U1:busmv", "kV, Line-Ground Voltage Phasor, Magnitude"
      m_U1_buslv = "m:U1:buslv", "kV, Line-Ground Voltage Phasor, Magnitude"
      m_U1_bushvn = "m:U1:bushvn", "kV, Line-Ground Voltage Phasor, Magnitude"
      m_U1_busmvn = "m:U1:busmvn", "kV, Line-Ground Voltage Phasor, Magnitude"
      m_U1_buslvn = "m:U1:buslvn", "kV, Line-Ground Voltage Phasor, Magnitude"
      m_U1l_bushv = "m:U1l:bushv", "kV, Line-Line Voltage Phasor, Magnitude"
      m_U1l_busmv = "m:U1l:busmv", "kV, Line-Line Voltage Phasor, Magnitude"
      m_U1l_buslv = "m:U1l:buslv", "kV, Line-Line Voltage Phasor, Magnitude"
      m_U1l_bushvn = "m:U1l:bushvn", "kV, Line-Line Voltage Phasor, Magnitude"
      m_U1l_busmvn = "m:U1l:busmvn", "kV, Line-Line Voltage Phasor, Magnitude"
      m_U1l_buslvn = "m:U1l:buslvn", "kV, Line-Line Voltage Phasor, Magnitude"
      m_phiu1_bushv = "m:phiu1:bushv", "deg, Voltage Phasor, Angle"
      m_phiu1_busmv = "m:phiu1:busmv", "deg, Voltage Phasor, Angle"
      m_phiu1_buslv = "m:phiu1:buslv", "deg, Voltage Phasor, Angle"
      m_phiu1_bushvn = "m:phiu1:bushvn", "deg, Voltage Phasor, Angle"
      m_phiu1_busmvn = "m:phiu1:busmvn", "deg, Voltage Phasor, Angle"
      m_phiu1_buslvn = "m:phiu1:buslvn", "deg, Voltage Phasor, Angle"
      m_I_bushv_A = "m:I:bushv:A", "kA, Phase Current"
      m_I_bushv_B = "m:I:bushv:B", "kA, Phase Current"
      m_I_bushv_C = "m:I:bushv:C", "kA, Phase Current"
      m_I_bushv_N = "m:I:bushv:N", "kA, Phase Current"
      m_I_busmv_A = "m:I:busmv:A", "kA, Phase Current"
      m_I_busmv_B = "m:I:busmv:B", "kA, Phase Current"
      m_I_busmv_C = "m:I:busmv:C", "kA, Phase Current"
      m_I_busmv_N = "m:I:busmv:N", "kA, Phase Current"
      m_I_buslv_A = "m:I:buslv:A", "kA, Phase Current"
      m_I_buslv_B = "m:I:buslv:B", "kA, Phase Current"
      m_I_buslv_C = "m:I:buslv:C", "kA, Phase Current"
      m_I_buslv_N = "m:I:buslv:N", "kA, Phase Current"
      m_I_bushvn = "m:I:bushvn", "kA, Phase Current"
      m_I_busmvn = "m:I:busmvn", "kA, Phase Current"
      m_I_buslvn = "m:I:buslvn", "kA, Phase Current"
      m_i0_bushv = "m:i0:bushv", "p.u., Zero-Sequence Current"
      m_i0_busmv = "m:i0:busmv", "p.u., Zero-Sequence Current"
      m_i0_buslv = "m:i0:buslv", "p.u., Zero-Sequence Current"
      m_i0_bushvn = "m:i0:bushvn", "p.u., Zero-Sequence Current"
      m_i0_busmvn = "m:i0:busmvn", "p.u., Zero-Sequence Current"
      m_i0_buslvn = "m:i0:buslvn", "p.u., Zero-Sequence Current"
      m_I0_bushv = "m:I0:bushv", "kA, Zero-Sequence Current"
      m_I0_busmv = "m:I0:busmv", "kA, Zero-Sequence Current"
      m_I0_buslv = "m:I0:buslv", "kA, Zero-Sequence Current"
      m_I0_bushvn = "m:I0:bushvn", "kA, Zero-Sequence Current"
      m_I0_busmvn = "m:I0:busmvn", "kA, Zero-Sequence Current"
      m_I0_buslvn = "m:I0:buslvn", "kA, Zero-Sequence Current"
      m_I0x3_bushv = "m:I0x3:bushv", "kA, 3*I0"
      m_I0x3_busmv = "m:I0x3:busmv", "kA, 3*I0"
      m_I0x3_buslv = "m:I0x3:buslv", "kA, 3*I0"
      m_I0x3_bushvn = "m:I0x3:bushvn", "kA, 3*I0"
      m_I0x3_busmvn = "m:I0x3:busmvn", "kA, 3*I0"
      m_I0x3_buslvn = "m:I0x3:buslvn", "kA, 3*I0"
      m_i1r_bushv = "m:i1r:bushv", "p.u., Current Phasor, Real Part"
      m_i1r_busmv = "m:i1r:busmv", "p.u., Current Phasor, Real Part"
      m_i1r_buslv = "m:i1r:buslv", "p.u., Current Phasor, Real Part"
      m_i1r_bushvn = "m:i1r:bushvn", "p.u., Current Phasor, Real Part"
      m_i1r_busmvn = "m:i1r:busmvn", "p.u., Current Phasor, Real Part"
      m_i1r_buslvn = "m:i1r:buslvn", "p.u., Current Phasor, Real Part"
      m_i1i_bushv = "m:i1i:bushv", "p.u., Current Phasor, Imaginary Part"
      m_i1i_busmv = "m:i1i:busmv", "p.u., Current Phasor, Imaginary Part"
      m_i1i_buslv = "m:i1i:buslv", "p.u., Current Phasor, Imaginary Part"
      m_i1i_bushvn = "m:i1i:bushvn", "p.u., Current Phasor, Imaginary Part"
      m_i1i_busmvn = "m:i1i:busmvn", "p.u., Current Phasor, Imaginary Part"
      m_i1i_buslvn = "m:i1i:buslvn", "p.u., Current Phasor, Imaginary Part"
      m_i1_bushv = "m:i1:bushv", "p.u., Current Phasor, Magnitude"
      m_i1_busmv = "m:i1:busmv", "p.u., Current Phasor, Magnitude"
      m_i1_buslv = "m:i1:buslv", "p.u., Current Phasor, Magnitude"
      m_i1_bushvn = "m:i1:bushvn", "p.u., Current Phasor, Magnitude"
      m_i1_busmvn = "m:i1:busmvn", "p.u., Current Phasor, Magnitude"
      m_i1_buslvn = "m:i1:buslvn", "p.u., Current Phasor, Magnitude"
      m_I1_bushv = "m:I1:bushv", "kA, Current Phasor, Magnitude"
      m_I1_busmv = "m:I1:busmv", "kA, Current Phasor, Magnitude"
      m_I1_buslv = "m:I1:buslv", "kA, Current Phasor, Magnitude"
      m_I1_bushvn = "m:I1:bushvn", "kA, Current Phasor, Magnitude"
      m_I1_busmvn = "m:I1:busmvn", "kA, Current Phasor, Magnitude"
      m_I1_buslvn = "m:I1:buslvn", "kA, Current Phasor, Magnitude"
      m_phii1_bushv = "m:phii1:bushv", "deg, Current Phasor, Angle"
      m_phii1_busmv = "m:phii1:busmv", "deg, Current Phasor, Angle"
      m_phii1_buslv = "m:phii1:buslv", "deg, Current Phasor, Angle"
      m_phii1_bushvn = "m:phii1:bushvn", "deg, Current Phasor, Angle"
      m_phii1_busmvn = "m:phii1:busmvn", "deg, Current Phasor, Angle"
      m_phii1_buslvn = "m:phii1:buslvn", "deg, Current Phasor, Angle"
      m_P_bushv_A = "m:P:bushv:A", "MW, Active Power"
      m_P_bushv_B = "m:P:bushv:B", "MW, Active Power"
      m_P_bushv_C = "m:P:bushv:C", "MW, Active Power"
      m_P_bushv_N = "m:P:bushv:N", "MW, Active Power"
      m_P_busmv_A = "m:P:busmv:A", "MW, Active Power"
      m_P_busmv_B = "m:P:busmv:B", "MW, Active Power"
      m_P_busmv_C = "m:P:busmv:C", "MW, Active Power"
      m_P_busmv_N = "m:P:busmv:N", "MW, Active Power"
      m_P_buslv_A = "m:P:buslv:A", "MW, Active Power"
      m_P_buslv_B = "m:P:buslv:B", "MW, Active Power"
      m_P_buslv_C = "m:P:buslv:C", "MW, Active Power"
      m_P_buslv_N = "m:P:buslv:N", "MW, Active Power"
      m_P_bushvn = "m:P:bushvn", "MW, Active Power"
      m_P_busmvn = "m:P:busmvn", "MW, Active Power"
      m_P_buslvn = "m:P:buslvn", "MW, Active Power"
      m_Psum_bushv = "m:Psum:bushv", "MW, Power-Phasor, Active Power"
      m_Psum_busmv = "m:Psum:busmv", "MW, Power-Phasor, Active Power"
      m_Psum_buslv = "m:Psum:buslv", "MW, Power-Phasor, Active Power"
      m_Psum_bushvn = "m:Psum:bushvn", "MW, Power-Phasor, Active Power"
      m_Psum_busmvn = "m:Psum:busmvn", "MW, Power-Phasor, Active Power"
      m_Psum_buslvn = "m:Psum:buslvn", "MW, Power-Phasor, Active Power"
      m_Qsum_bushv = "m:Qsum:bushv", "Mvar, Power Phasor, Reactive Part"
      m_Qsum_busmv = "m:Qsum:busmv", "Mvar, Power Phasor, Reactive Part"
      m_Qsum_buslv = "m:Qsum:buslv", "Mvar, Power Phasor, Reactive Part"
      m_Qsum_bushvn = "m:Qsum:bushvn", "Mvar, Power Phasor, Reactive Part"
      m_Qsum_busmvn = "m:Qsum:busmvn", "Mvar, Power Phasor, Reactive Part"
      m_Qsum_buslvn = "m:Qsum:buslvn", "Mvar, Power Phasor, Reactive Part"
      m_Ssum_bushv = "m:Ssum:bushv", "MVA, Power Phasor, Apparent Power"
      m_Ssum_busmv = "m:Ssum:busmv", "MVA, Power Phasor, Apparent Power"
      m_Ssum_buslv = "m:Ssum:buslv", "MVA, Power Phasor, Apparent Power"
      m_Ssum_bushvn = "m:Ssum:bushvn", "MVA, Power Phasor, Apparent Power"
      m_Ssum_busmvn = "m:Ssum:busmvn", "MVA, Power Phasor, Apparent Power"
      m_Ssum_buslvn = "m:Ssum:buslvn", "MVA, Power Phasor, Apparent Power"
      m_cosphi1_bushv = "m:cosphi1:bushv", "Power-Phasor, Power-Factor"
      m_cosphi1_busmv = "m:cosphi1:busmv", "Power-Phasor, Power-Factor"
      m_cosphi1_buslv = "m:cosphi1:buslv", "Power-Phasor, Power-Factor"
      m_cosphi1_bushvn = "m:cosphi1:bushvn", "Power-Phasor, Power-Factor"
      m_cosphi1_busmvn = "m:cosphi1:busmvn", "Power-Phasor, Power-Factor"
      m_cosphi1_buslvn = "m:cosphi1:buslvn", "Power-Phasor, Power-Factor"
      m_i1P_bushv = "m:i1P:bushv", "p.u., Current Phasor, Active Current"
      m_i1P_busmv = "m:i1P:busmv", "p.u., Current Phasor, Active Current"
      m_i1P_buslv = "m:i1P:buslv", "p.u., Current Phasor, Active Current"
      m_i1P_bushvn = "m:i1P:bushvn", "p.u., Current Phasor, Active Current"
      m_i1P_busmvn = "m:i1P:busmvn", "p.u., Current Phasor, Active Current"
      m_i1P_buslvn = "m:i1P:buslvn", "p.u., Current Phasor, Active Current"
      m_i1Q_bushv = "m:i1Q:bushv", "p.u., Current Phasor, Reactive Current"
      m_i1Q_busmv = "m:i1Q:busmv", "p.u., Current Phasor, Reactive Current"
      m_i1Q_buslv = "m:i1Q:buslv", "p.u., Current Phasor, Reactive Current"
      m_i1Q_bushvn = "m:i1Q:bushvn", "p.u., Current Phasor, Reactive Current"
      m_i1Q_busmvn = "m:i1Q:busmvn", "p.u., Current Phasor, Reactive Current"
      m_i1Q_buslvn = "m:i1Q:buslvn", "p.u., Current Phasor, Reactive Current"
      m_I1P_bushv = "m:I1P:bushv", "kA, Current Phasor, Active Current"
      m_I1P_busmv = "m:I1P:busmv", "kA, Current Phasor, Active Current"
      m_I1P_buslv = "m:I1P:buslv", "kA, Current Phasor, Active Current"
      m_I1P_bushvn = "m:I1P:bushvn", "kA, Current Phasor, Active Current"
      m_I1P_busmvn = "m:I1P:busmvn", "kA, Current Phasor, Active Current"
      m_I1P_buslvn = "m:I1P:buslvn", "kA, Current Phasor, Active Current"
      m_I1Q_bushv = "m:I1Q:bushv", "kA, Current Phasor, Reactive Current"
      m_I1Q_busmv = "m:I1Q:busmv", "kA, Current Phasor, Reactive Current"
      m_I1Q_buslv = "m:I1Q:buslv", "kA, Current Phasor, Reactive Current"
      m_I1Q_bushvn = "m:I1Q:bushvn", "kA, Current Phasor, Reactive Current"
      m_I1Q_busmvn = "m:I1Q:busmvn", "kA, Current Phasor, Reactive Current"
      m_I1Q_buslvn = "m:I1Q:buslvn", "kA, Current Phasor, Reactive Current"
      m_u_bushv_A = "m:u:bushv:A", "p.u., Phase Voltage"
      m_u_bushv_B = "m:u:bushv:B", "p.u., Phase Voltage"
      m_u_bushv_C = "m:u:bushv:C", "p.u., Phase Voltage"
      m_u_bushv_N = "m:u:bushv:N", "p.u., Phase Voltage"
      m_u_busmv_A = "m:u:busmv:A", "p.u., Phase Voltage"
      m_u_busmv_B = "m:u:busmv:B", "p.u., Phase Voltage"
      m_u_busmv_C = "m:u:busmv:C", "p.u., Phase Voltage"
      m_u_busmv_N = "m:u:busmv:N", "p.u., Phase Voltage"
      m_u_buslv_A = "m:u:buslv:A", "p.u., Phase Voltage"
      m_u_buslv_B = "m:u:buslv:B", "p.u., Phase Voltage"
      m_u_buslv_C = "m:u:buslv:C", "p.u., Phase Voltage"
      m_u_buslv_N = "m:u:buslv:N", "p.u., Phase Voltage"
      m_u_bushvn = "m:u:bushvn", "p.u., Phase Voltage"
      m_u_busmvn = "m:u:busmvn", "p.u., Phase Voltage"
      m_u_buslvn = "m:u:buslvn", "p.u., Phase Voltage"
      m_i_bushv_A = "m:i:bushv:A", "p.u., Phase Current"
      m_i_bushv_B = "m:i:bushv:B", "p.u., Phase Current"
      m_i_bushv_C = "m:i:bushv:C", "p.u., Phase Current"
      m_i_bushv_N = "m:i:bushv:N", "p.u., Phase Current"
      m_i_busmv_A = "m:i:busmv:A", "p.u., Phase Current"
      m_i_busmv_B = "m:i:busmv:B", "p.u., Phase Current"
      m_i_busmv_C = "m:i:busmv:C", "p.u., Phase Current"
      m_i_busmv_N = "m:i:busmv:N", "p.u., Phase Current"
      m_i_buslv_A = "m:i:buslv:A", "p.u., Phase Current"
      m_i_buslv_B = "m:i:buslv:B", "p.u., Phase Current"
      m_i_buslv_C = "m:i:buslv:C", "p.u., Phase Current"
      m_i_buslv_N = "m:i:buslv:N", "p.u., Phase Current"
      m_i_bushvn = "m:i:bushvn", "p.u., Phase Current"
      m_i_busmvn = "m:i:busmvn", "p.u., Phase Current"
      m_i_buslvn = "m:i:buslvn", "p.u., Phase Current"
      m_phiu1i1_bushv = "m:phiu1i1:bushv", "deg, Angle between Voltage and Current Phasor"
      m_phiu1i1_busmv = "m:phiu1i1:busmv", "deg, Angle between Voltage and Current Phasor"
      m_phiu1i1_buslv = "m:phiu1i1:buslv", "deg, Angle between Voltage and Current Phasor"
      m_phiu1i1_bushvn = "m:phiu1i1:bushvn", "deg, Angle between Voltage and Current Phasor"
      m_phiu1i1_busmvn = "m:phiu1i1:busmvn", "deg, Angle between Voltage and Current Phasor"
      m_phiu1i1_buslvn = "m:phiu1i1:buslvn", "deg, Angle between Voltage and Current Phasor"
      m_Tfct_bushv = "m:Tfct:bushv", "s, Fault Clearing Time"
      m_Tfct_busmv = "m:Tfct:busmv", "s, Fault Clearing Time"
      m_Tfct_buslv = "m:Tfct:buslv", "s, Fault Clearing Time"
      m_Tfct_bushvn = "m:Tfct:bushvn", "s, Fault Clearing Time"
      m_Tfct_busmvn = "m:Tfct:busmvn", "s, Fault Clearing Time"
      m_Tfct_buslvn = "m:Tfct:buslvn", "s, Fault Clearing Time"
      m_TfctPh_bushv_A = "m:TfctPh:bushv:A", "s, Fault Clearing Time"
      m_TfctPh_bushv_B = "m:TfctPh:bushv:B", "s, Fault Clearing Time"
      m_TfctPh_bushv_C = "m:TfctPh:bushv:C", "s, Fault Clearing Time"
      m_TfctPh_bushv_N = "m:TfctPh:bushv:N", "s, Fault Clearing Time"
      m_TfctPh_busmv_A = "m:TfctPh:busmv:A", "s, Fault Clearing Time"
      m_TfctPh_busmv_B = "m:TfctPh:busmv:B", "s, Fault Clearing Time"
      m_TfctPh_busmv_C = "m:TfctPh:busmv:C", "s, Fault Clearing Time"
      m_TfctPh_busmv_N = "m:TfctPh:busmv:N", "s, Fault Clearing Time"
      m_TfctPh_buslv_A = "m:TfctPh:buslv:A", "s, Fault Clearing Time"
      m_TfctPh_buslv_B = "m:TfctPh:buslv:B", "s, Fault Clearing Time"
      m_TfctPh_buslv_C = "m:TfctPh:buslv:C", "s, Fault Clearing Time"
      m_TfctPh_buslv_N = "m:TfctPh:buslv:N", "s, Fault Clearing Time"
      m_TfctPh_bushvn = "m:TfctPh:bushvn", "s, Fault Clearing Time"
      m_TfctPh_busmvn = "m:TfctPh:busmvn", "s, Fault Clearing Time"
      m_TfctPh_buslvn = "m:TfctPh:buslvn", "s, Fault Clearing Time"
      n_u_bushv_A = "n:u:bushv:A", "p.u., Phase Voltage"
      n_u_bushv_B = "n:u:bushv:B", "p.u., Phase Voltage"
      n_u_bushv_C = "n:u:bushv:C", "p.u., Phase Voltage"
      n_u_bushv_N = "n:u:bushv:N", "p.u., Phase Voltage"
      n_u_busmv_A = "n:u:busmv:A", "p.u., Phase Voltage"
      n_u_busmv_B = "n:u:busmv:B", "p.u., Phase Voltage"
      n_u_busmv_C = "n:u:busmv:C", "p.u., Phase Voltage"
      n_u_busmv_N = "n:u:busmv:N", "p.u., Phase Voltage"
      n_u_buslv_A = "n:u:buslv:A", "p.u., Phase Voltage"
      n_u_buslv_B = "n:u:buslv:B", "p.u., Phase Voltage"
      n_u_buslv_C = "n:u:buslv:C", "p.u., Phase Voltage"
      n_u_buslv_N = "n:u:buslv:N", "p.u., Phase Voltage"
      n_u_bushvn = "n:u:bushvn", "p.u., Phase Voltage"
      n_u_busmvn = "n:u:busmvn", "p.u., Phase Voltage"
      n_u_buslvn = "n:u:buslvn", "p.u., Phase Voltage"
      n_upc_bushv_A = "n:upc:bushv:A", "%, Phase Voltage"
      n_upc_bushv_B = "n:upc:bushv:B", "%, Phase Voltage"
      n_upc_bushv_C = "n:upc:bushv:C", "%, Phase Voltage"
      n_upc_bushv_N = "n:upc:bushv:N", "%, Phase Voltage"
      n_upc_busmv_A = "n:upc:busmv:A", "%, Phase Voltage"
      n_upc_busmv_B = "n:upc:busmv:B", "%, Phase Voltage"
      n_upc_busmv_C = "n:upc:busmv:C", "%, Phase Voltage"
      n_upc_busmv_N = "n:upc:busmv:N", "%, Phase Voltage"
      n_upc_buslv_A = "n:upc:buslv:A", "%, Phase Voltage"
      n_upc_buslv_B = "n:upc:buslv:B", "%, Phase Voltage"
      n_upc_buslv_C = "n:upc:buslv:C", "%, Phase Voltage"
      n_upc_buslv_N = "n:upc:buslv:N", "%, Phase Voltage"
      n_upc_bushvn = "n:upc:bushvn", "%, Phase Voltage"
      n_upc_busmvn = "n:upc:busmvn", "%, Phase Voltage"
      n_upc_buslvn = "n:upc:buslvn", "%, Phase Voltage"
      n_U_bushv_A = "n:U:bushv:A", "kV, Phase Voltage"
      n_U_bushv_B = "n:U:bushv:B", "kV, Phase Voltage"
      n_U_bushv_C = "n:U:bushv:C", "kV, Phase Voltage"
      n_U_bushv_N = "n:U:bushv:N", "kV, Phase Voltage"
      n_U_busmv_A = "n:U:busmv:A", "kV, Phase Voltage"
      n_U_busmv_B = "n:U:busmv:B", "kV, Phase Voltage"
      n_U_busmv_C = "n:U:busmv:C", "kV, Phase Voltage"
      n_U_busmv_N = "n:U:busmv:N", "kV, Phase Voltage"
      n_U_buslv_A = "n:U:buslv:A", "kV, Phase Voltage"
      n_U_buslv_B = "n:U:buslv:B", "kV, Phase Voltage"
      n_U_buslv_C = "n:U:buslv:C", "kV, Phase Voltage"
      n_U_buslv_N = "n:U:buslv:N", "kV, Phase Voltage"
      n_U_bushvn = "n:U:bushvn", "kV, Phase Voltage"
      n_U_busmvn = "n:U:busmvn", "kV, Phase Voltage"
      n_U_buslvn = "n:U:buslvn", "kV, Phase Voltage"
      n_ul_bushv_A = "n:ul:bushv:A", "p.u., Line-Line Voltage"
      n_ul_bushv_B = "n:ul:bushv:B", "p.u., Line-Line Voltage"
      n_ul_bushv_C = "n:ul:bushv:C", "p.u., Line-Line Voltage"
      n_ul_bushv_N = "n:ul:bushv:N", "p.u., Line-Line Voltage"
      n_ul_busmv_A = "n:ul:busmv:A", "p.u., Line-Line Voltage"
      n_ul_busmv_B = "n:ul:busmv:B", "p.u., Line-Line Voltage"
      n_ul_busmv_C = "n:ul:busmv:C", "p.u., Line-Line Voltage"
      n_ul_busmv_N = "n:ul:busmv:N", "p.u., Line-Line Voltage"
      n_ul_buslv_A = "n:ul:buslv:A", "p.u., Line-Line Voltage"
      n_ul_buslv_B = "n:ul:buslv:B", "p.u., Line-Line Voltage"
      n_ul_buslv_C = "n:ul:buslv:C", "p.u., Line-Line Voltage"
      n_ul_buslv_N = "n:ul:buslv:N", "p.u., Line-Line Voltage"
      n_ul_bushvn = "n:ul:bushvn", "p.u., Line-Line Voltage"
      n_ul_busmvn = "n:ul:busmvn", "p.u., Line-Line Voltage"
      n_ul_buslvn = "n:ul:buslvn", "p.u., Line-Line Voltage"
      n_ulpc_bushv_A = "n:ulpc:bushv:A", "%, Line-Line Voltage"
      n_ulpc_bushv_B = "n:ulpc:bushv:B", "%, Line-Line Voltage"
      n_ulpc_bushv_C = "n:ulpc:bushv:C", "%, Line-Line Voltage"
      n_ulpc_bushv_N = "n:ulpc:bushv:N", "%, Line-Line Voltage"
      n_ulpc_busmv_A = "n:ulpc:busmv:A", "%, Line-Line Voltage"
      n_ulpc_busmv_B = "n:ulpc:busmv:B", "%, Line-Line Voltage"
      n_ulpc_busmv_C = "n:ulpc:busmv:C", "%, Line-Line Voltage"
      n_ulpc_busmv_N = "n:ulpc:busmv:N", "%, Line-Line Voltage"
      n_ulpc_buslv_A = "n:ulpc:buslv:A", "%, Line-Line Voltage"
      n_ulpc_buslv_B = "n:ulpc:buslv:B", "%, Line-Line Voltage"
      n_ulpc_buslv_C = "n:ulpc:buslv:C", "%, Line-Line Voltage"
      n_ulpc_buslv_N = "n:ulpc:buslv:N", "%, Line-Line Voltage"
      n_ulpc_bushvn = "n:ulpc:bushvn", "%, Line-Line Voltage"
      n_ulpc_busmvn = "n:ulpc:busmvn", "%, Line-Line Voltage"
      n_ulpc_buslvn = "n:ulpc:buslvn", "%, Line-Line Voltage"
      n_Ul_bushv_A = "n:Ul:bushv:A", "kV, Line-Line Voltage"
      n_Ul_bushv_B = "n:Ul:bushv:B", "kV, Line-Line Voltage"
      n_Ul_bushv_C = "n:Ul:bushv:C", "kV, Line-Line Voltage"
      n_Ul_bushv_N = "n:Ul:bushv:N", "kV, Line-Line Voltage"
      n_Ul_busmv_A = "n:Ul:busmv:A", "kV, Line-Line Voltage"
      n_Ul_busmv_B = "n:Ul:busmv:B", "kV, Line-Line Voltage"
      n_Ul_busmv_C = "n:Ul:busmv:C", "kV, Line-Line Voltage"
      n_Ul_busmv_N = "n:Ul:busmv:N", "kV, Line-Line Voltage"
      n_Ul_buslv_A = "n:Ul:buslv:A", "kV, Line-Line Voltage"
      n_Ul_buslv_B = "n:Ul:buslv:B", "kV, Line-Line Voltage"
      n_Ul_buslv_C = "n:Ul:buslv:C", "kV, Line-Line Voltage"
      n_Ul_buslv_N = "n:Ul:buslv:N", "kV, Line-Line Voltage"
      n_Ul_bushvn = "n:Ul:bushvn", "kV, Line-Line Voltage"
      n_Ul_busmvn = "n:Ul:busmvn", "kV, Line-Line Voltage"
      n_Ul_buslvn = "n:Ul:buslvn", "kV, Line-Line Voltage"
      n_u0_bushv = "n:u0:bushv", "p.u., Zero-Sequence-Voltage"
      n_u0_busmv = "n:u0:busmv", "p.u., Zero-Sequence-Voltage"
      n_u0_buslv = "n:u0:buslv", "p.u., Zero-Sequence-Voltage"
      n_u0_bushvn = "n:u0:bushvn", "p.u., Zero-Sequence-Voltage"
      n_u0_busmvn = "n:u0:busmvn", "p.u., Zero-Sequence-Voltage"
      n_u0_buslvn = "n:u0:buslvn", "p.u., Zero-Sequence-Voltage"
      n_U0_bushv = "n:U0:bushv", "kV, Zero-Sequence-Voltage"
      n_U0_busmv = "n:U0:busmv", "kV, Zero-Sequence-Voltage"
      n_U0_buslv = "n:U0:buslv", "kV, Zero-Sequence-Voltage"
      n_U0_bushvn = "n:U0:bushvn", "kV, Zero-Sequence-Voltage"
      n_U0_busmvn = "n:U0:busmvn", "kV, Zero-Sequence-Voltage"
      n_U0_buslvn = "n:U0:buslvn", "kV, Zero-Sequence-Voltage"
      n_U0x3_bushv = "n:U0x3:bushv", "kV, 3*U0"
      n_U0x3_busmv = "n:U0x3:busmv", "kV, 3*U0"
      n_U0x3_buslv = "n:U0x3:buslv", "kV, 3*U0"
      n_U0x3_bushvn = "n:U0x3:bushvn", "kV, 3*U0"
      n_U0x3_busmvn = "n:U0x3:busmvn", "kV, 3*U0"
      n_U0x3_buslvn = "n:U0x3:buslvn", "kV, 3*U0"
      n_u1r_bushv = "n:u1r:bushv", "p.u., Voltage Phasor, Real Part"
      n_u1r_busmv = "n:u1r:busmv", "p.u., Voltage Phasor, Real Part"
      n_u1r_buslv = "n:u1r:buslv", "p.u., Voltage Phasor, Real Part"
      n_u1r_bushvn = "n:u1r:bushvn", "p.u., Voltage Phasor, Real Part"
      n_u1r_busmvn = "n:u1r:busmvn", "p.u., Voltage Phasor, Real Part"
      n_u1r_buslvn = "n:u1r:buslvn", "p.u., Voltage Phasor, Real Part"
      n_u1i_bushv = "n:u1i:bushv", "p.u., Voltage Phasor, Imaginary Part"
      n_u1i_busmv = "n:u1i:busmv", "p.u., Voltage Phasor, Imaginary Part"
      n_u1i_buslv = "n:u1i:buslv", "p.u., Voltage Phasor, Imaginary Part"
      n_u1i_bushvn = "n:u1i:bushvn", "p.u., Voltage Phasor, Imaginary Part"
      n_u1i_busmvn = "n:u1i:busmvn", "p.u., Voltage Phasor, Imaginary Part"
      n_u1i_buslvn = "n:u1i:buslvn", "p.u., Voltage Phasor, Imaginary Part"
      n_u1_bushv = "n:u1:bushv", "p.u., Voltage Phasor, Magnitude"
      n_u1_busmv = "n:u1:busmv", "p.u., Voltage Phasor, Magnitude"
      n_u1_buslv = "n:u1:buslv", "p.u., Voltage Phasor, Magnitude"
      n_u1_bushvn = "n:u1:bushvn", "p.u., Voltage Phasor, Magnitude"
      n_u1_busmvn = "n:u1:busmvn", "p.u., Voltage Phasor, Magnitude"
      n_u1_buslvn = "n:u1:buslvn", "p.u., Voltage Phasor, Magnitude"
      n_u1pc_bushv = "n:u1pc:bushv", "%, Voltage Phasor, Magnitude"
      n_u1pc_busmv = "n:u1pc:busmv", "%, Voltage Phasor, Magnitude"
      n_u1pc_buslv = "n:u1pc:buslv", "%, Voltage Phasor, Magnitude"
      n_u1pc_bushvn = "n:u1pc:bushvn", "%, Voltage Phasor, Magnitude"
      n_u1pc_busmvn = "n:u1pc:busmvn", "%, Voltage Phasor, Magnitude"
      n_u1pc_buslvn = "n:u1pc:buslvn", "%, Voltage Phasor, Magnitude"
      n_U1_bushv = "n:U1:bushv", "kV, Voltage Phasor, Magnitude"
      n_U1_busmv = "n:U1:busmv", "kV, Voltage Phasor, Magnitude"
      n_U1_buslv = "n:U1:buslv", "kV, Voltage Phasor, Magnitude"
      n_U1_bushvn = "n:U1:bushvn", "kV, Voltage Phasor, Magnitude"
      n_U1_busmvn = "n:U1:busmvn", "kV, Voltage Phasor, Magnitude"
      n_U1_buslvn = "n:U1:buslvn", "kV, Voltage Phasor, Magnitude"
      n_phiu1_bushv = "n:phiu1:bushv", "deg, Voltage Phasor, Angle"
      n_phiu1_busmv = "n:phiu1:busmv", "deg, Voltage Phasor, Angle"
      n_phiu1_buslv = "n:phiu1:buslv", "deg, Voltage Phasor, Angle"
      n_phiu1_bushvn = "n:phiu1:bushvn", "deg, Voltage Phasor, Angle"
      n_phiu1_busmvn = "n:phiu1:busmvn", "deg, Voltage Phasor, Angle"
      n_phiu1_buslvn = "n:phiu1:buslvn", "deg, Voltage Phasor, Angle"
      n_U1l_bushv = "n:U1l:bushv", "kV, Line-Line Voltage Phasor, Magnitude"
      n_U1l_busmv = "n:U1l:busmv", "kV, Line-Line Voltage Phasor, Magnitude"
      n_U1l_buslv = "n:U1l:buslv", "kV, Line-Line Voltage Phasor, Magnitude"
      n_U1l_bushvn = "n:U1l:bushvn", "kV, Line-Line Voltage Phasor, Magnitude"
      n_U1l_busmvn = "n:U1l:busmvn", "kV, Line-Line Voltage Phasor, Magnitude"
      n_U1l_buslvn = "n:U1l:buslvn", "kV, Line-Line Voltage Phasor, Magnitude"
      n_uln_bushv_A = "n:uln:bushv:A", "p.u., Line-Neutral Voltage"
      n_uln_bushv_B = "n:uln:bushv:B", "p.u., Line-Neutral Voltage"
      n_uln_bushv_C = "n:uln:bushv:C", "p.u., Line-Neutral Voltage"
      n_uln_bushv_N = "n:uln:bushv:N", "p.u., Line-Neutral Voltage"
      n_uln_busmv_A = "n:uln:busmv:A", "p.u., Line-Neutral Voltage"
      n_uln_busmv_B = "n:uln:busmv:B", "p.u., Line-Neutral Voltage"
      n_uln_busmv_C = "n:uln:busmv:C", "p.u., Line-Neutral Voltage"
      n_uln_busmv_N = "n:uln:busmv:N", "p.u., Line-Neutral Voltage"
      n_uln_buslv_A = "n:uln:buslv:A", "p.u., Line-Neutral Voltage"
      n_uln_buslv_B = "n:uln:buslv:B", "p.u., Line-Neutral Voltage"
      n_uln_buslv_C = "n:uln:buslv:C", "p.u., Line-Neutral Voltage"
      n_uln_buslv_N = "n:uln:buslv:N", "p.u., Line-Neutral Voltage"
      n_uln_bushvn = "n:uln:bushvn", "p.u., Line-Neutral Voltage"
      n_uln_busmvn = "n:uln:busmvn", "p.u., Line-Neutral Voltage"
      n_uln_buslvn = "n:uln:buslvn", "p.u., Line-Neutral Voltage"
      n_Uln_bushv_A = "n:Uln:bushv:A", "kV, Line-Neutral Voltage"
      n_Uln_bushv_B = "n:Uln:bushv:B", "kV, Line-Neutral Voltage"
      n_Uln_bushv_C = "n:Uln:bushv:C", "kV, Line-Neutral Voltage"
      n_Uln_bushv_N = "n:Uln:bushv:N", "kV, Line-Neutral Voltage"
      n_Uln_busmv_A = "n:Uln:busmv:A", "kV, Line-Neutral Voltage"
      n_Uln_busmv_B = "n:Uln:busmv:B", "kV, Line-Neutral Voltage"
      n_Uln_busmv_C = "n:Uln:busmv:C", "kV, Line-Neutral Voltage"
      n_Uln_busmv_N = "n:Uln:busmv:N", "kV, Line-Neutral Voltage"
      n_Uln_buslv_A = "n:Uln:buslv:A", "kV, Line-Neutral Voltage"
      n_Uln_buslv_B = "n:Uln:buslv:B", "kV, Line-Neutral Voltage"
      n_Uln_buslv_C = "n:Uln:buslv:C", "kV, Line-Neutral Voltage"
      n_Uln_buslv_N = "n:Uln:buslv:N", "kV, Line-Neutral Voltage"
      n_Uln_bushvn = "n:Uln:bushvn", "kV, Line-Neutral Voltage"
      n_Uln_busmvn = "n:Uln:busmvn", "kV, Line-Neutral Voltage"
      n_Uln_buslvn = "n:Uln:buslvn", "kV, Line-Neutral Voltage"
      n_un_bushv = "n:un:bushv", "p.u., Neutral-Ground Voltage"
      n_un_busmv = "n:un:busmv", "p.u., Neutral-Ground Voltage"
      n_un_buslv = "n:un:buslv", "p.u., Neutral-Ground Voltage"
      n_un_bushvn = "n:un:bushvn", "p.u., Neutral-Ground Voltage"
      n_un_busmvn = "n:un:busmvn", "p.u., Neutral-Ground Voltage"
      n_un_buslvn = "n:un:buslvn", "p.u., Neutral-Ground Voltage"
      n_Un_bushv = "n:Un:bushv", "kV, Neutral-Ground Voltage"
      n_Un_busmv = "n:Un:busmv", "kV, Neutral-Ground Voltage"
      n_Un_buslv = "n:Un:buslv", "kV, Neutral-Ground Voltage"
      n_Un_bushvn = "n:Un:bushvn", "kV, Neutral-Ground Voltage"
      n_Un_busmvn = "n:Un:busmvn", "kV, Neutral-Ground Voltage"
      n_Un_buslvn = "n:Un:buslvn", "kV, Neutral-Ground Voltage"
      n_ishc_bushv_A = "n:ishc:bushv:A", "p.u., Short-Circuit Current"
      n_ishc_bushv_B = "n:ishc:bushv:B", "p.u., Short-Circuit Current"
      n_ishc_bushv_C = "n:ishc:bushv:C", "p.u., Short-Circuit Current"
      n_ishc_bushv_N = "n:ishc:bushv:N", "p.u., Short-Circuit Current"
      n_ishc_busmv_A = "n:ishc:busmv:A", "p.u., Short-Circuit Current"
      n_ishc_busmv_B = "n:ishc:busmv:B", "p.u., Short-Circuit Current"
      n_ishc_busmv_C = "n:ishc:busmv:C", "p.u., Short-Circuit Current"
      n_ishc_busmv_N = "n:ishc:busmv:N", "p.u., Short-Circuit Current"
      n_ishc_buslv_A = "n:ishc:buslv:A", "p.u., Short-Circuit Current"
      n_ishc_buslv_B = "n:ishc:buslv:B", "p.u., Short-Circuit Current"
      n_ishc_buslv_C = "n:ishc:buslv:C", "p.u., Short-Circuit Current"
      n_ishc_buslv_N = "n:ishc:buslv:N", "p.u., Short-Circuit Current"
      n_ishc_bushvn = "n:ishc:bushvn", "p.u., Short-Circuit Current"
      n_ishc_busmvn = "n:ishc:busmvn", "p.u., Short-Circuit Current"
      n_ishc_buslvn = "n:ishc:buslvn", "p.u., Short-Circuit Current"
      n_Ishc_bushv_A = "n:Ishc:bushv:A", "kA, Short-Circuit Current"
      n_Ishc_bushv_B = "n:Ishc:bushv:B", "kA, Short-Circuit Current"
      n_Ishc_bushv_C = "n:Ishc:bushv:C", "kA, Short-Circuit Current"
      n_Ishc_bushv_N = "n:Ishc:bushv:N", "kA, Short-Circuit Current"
      n_Ishc_busmv_A = "n:Ishc:busmv:A", "kA, Short-Circuit Current"
      n_Ishc_busmv_B = "n:Ishc:busmv:B", "kA, Short-Circuit Current"
      n_Ishc_busmv_C = "n:Ishc:busmv:C", "kA, Short-Circuit Current"
      n_Ishc_busmv_N = "n:Ishc:busmv:N", "kA, Short-Circuit Current"
      n_Ishc_buslv_A = "n:Ishc:buslv:A", "kA, Short-Circuit Current"
      n_Ishc_buslv_B = "n:Ishc:buslv:B", "kA, Short-Circuit Current"
      n_Ishc_buslv_C = "n:Ishc:buslv:C", "kA, Short-Circuit Current"
      n_Ishc_buslv_N = "n:Ishc:buslv:N", "kA, Short-Circuit Current"
      n_Ishc_bushvn = "n:Ishc:bushvn", "kA, Short-Circuit Current"
      n_Ishc_busmvn = "n:Ishc:busmvn", "kA, Short-Circuit Current"
      n_Ishc_buslvn = "n:Ishc:buslvn", "kA, Short-Circuit Current"
      n_inshc_bushv = "n:inshc:bushv", "p.u., Neutral Short-Circuit Current, magnitude"
      n_inshc_busmv = "n:inshc:busmv", "p.u., Neutral Short-Circuit Current, magnitude"
      n_inshc_buslv = "n:inshc:buslv", "p.u., Neutral Short-Circuit Current, magnitude"
      n_inshc_bushvn = "n:inshc:bushvn", "p.u., Neutral Short-Circuit Current, magnitude"
      n_inshc_busmvn = "n:inshc:busmvn", "p.u., Neutral Short-Circuit Current, magnitude"
      n_inshc_buslvn = "n:inshc:buslvn", "p.u., Neutral Short-Circuit Current, magnitude"
      n_Inshc_bushv = "n:Inshc:bushv", "kA, Neutral Short-Circuit Current, magnitude"
      n_Inshc_busmv = "n:Inshc:busmv", "kA, Neutral Short-Circuit Current, magnitude"
      n_Inshc_buslv = "n:Inshc:buslv", "kA, Neutral Short-Circuit Current, magnitude"
      n_Inshc_bushvn = "n:Inshc:bushvn", "kA, Neutral Short-Circuit Current, magnitude"
      n_Inshc_busmvn = "n:Inshc:busmvn", "kA, Neutral Short-Circuit Current, magnitude"
      n_Inshc_buslvn = "n:Inshc:buslvn", "kA, Neutral Short-Circuit Current, magnitude"
      n_fe_bushv = "n:fe:bushv", "p.u., Electrical Frequency"
      n_fe_busmv = "n:fe:busmv", "p.u., Electrical Frequency"
      n_fe_buslv = "n:fe:buslv", "p.u., Electrical Frequency"
      n_fe_bushvn = "n:fe:bushvn", "p.u., Electrical Frequency"
      n_fe_busmvn = "n:fe:busmvn", "p.u., Electrical Frequency"
      n_fe_buslvn = "n:fe:buslvn", "p.u., Electrical Frequency"
      n_frdev_bushv = "n:frdev:bushv", "Hz, average frequency"
      n_frdev_busmv = "n:frdev:busmv", "Hz, average frequency"
      n_frdev_buslv = "n:frdev:buslv", "Hz, average frequency"
      n_frdev_bushvn = "n:frdev:bushvn", "Hz, average frequency"
      n_frdev_busmvn = "n:frdev:busmvn", "Hz, average frequency"
      n_frdev_buslvn = "n:frdev:buslvn", "Hz, average frequency"
      n_dfehz_bushv = "n:dfehz:bushv", "Hz, Deviation of the El. Frequency"
      n_dfehz_busmv = "n:dfehz:busmv", "Hz, Deviation of the El. Frequency"
      n_dfehz_buslv = "n:dfehz:buslv", "Hz, Deviation of the El. Frequency"
      n_dfehz_bushvn = "n:dfehz:bushvn", "Hz, Deviation of the El. Frequency"
      n_dfehz_busmvn = "n:dfehz:busmvn", "Hz, Deviation of the El. Frequency"
      n_dfehz_buslvn = "n:dfehz:buslvn", "Hz, Deviation of the El. Frequency"
      n_fehz_bushv = "n:fehz:bushv", "Hz, Electrical Frequency"
      n_fehz_busmv = "n:fehz:busmv", "Hz, Electrical Frequency"
      n_fehz_buslv = "n:fehz:buslv", "Hz, Electrical Frequency"
      n_fehz_bushvn = "n:fehz:bushvn", "Hz, Electrical Frequency"
      n_fehz_busmvn = "n:fehz:busmvn", "Hz, Electrical Frequency"
      n_fehz_buslvn = "n:fehz:buslvn", "Hz, Electrical Frequency"
      n_frnom_bushv = "n:frnom:bushv", "Hz, Nominal Frequency"
      n_frnom_busmv = "n:frnom:busmv", "Hz, Nominal Frequency"
      n_frnom_buslv = "n:frnom:buslv", "Hz, Nominal Frequency"
      n_frnom_bushvn = "n:frnom:bushvn", "Hz, Nominal Frequency"
      n_frnom_busmvn = "n:frnom:busmvn", "Hz, Nominal Frequency"
      n_frnom_buslvn = "n:frnom:buslvn", "Hz, Nominal Frequency"
      s_nntapin_h = "s:nntapin_h", "IN, Tap (HV), Controller Input, continuous, double"
      s_nntapin_m = "s:nntapin_m", "IN, Tap (MV), Controller Input, continuous, double"
      s_nntapin_l = "s:nntapin_l", "IN, Tap (LV), Controller Input, continuous, double"
      s_I0Delta_h = "s:I0Delta_h", "kA, OUT, Circulating current in HV-D-Winding, continuous, double"
      s_I0Delta_m = "s:I0Delta_m", "kA, OUT, Circulating current in MV-D-Winding, continuous, double"
      s_I0Delta_l = "s:I0Delta_l", "kA, OUT, Circulating current in LV-D-Winding, continuous, double"
      s_psim_a = "s:psim_a", "p.u., OUT, Magnetising Flux, continuous, double"
      s_psim_b = "s:psim_b", "p.u., OUT, Magnetising Flux, continuous, double"
      s_psim_c = "s:psim_c", "p.u., OUT, Magnetising Flux, continuous, double"
      s_i0_h = "s:i0_h", "p.u., STATE, 0-Current in Delta-Winding (HV) continuous, double"
      s_i0_m = "s:i0_m", "p.u., STATE, 0-Current in Delta-Winding (MV) continuous, double"
      s_i0_l = "s:i0_l", "p.u., STATE, 0-Current in Delta-Winding (LV) continuous, double"
      s_psim_r = "s:psim_r", "p.u., STATE, Magnetising Flux (Real Part), continuous, double"
      s_psim_i = "s:psim_i", "p.u., STATE, Magnetising Flux (Imaginary Part), continuous, double"
      s_psim_0 = "s:psim_0", "p.u., STATE, Magnetising Flux (0-Sequence), continuous, double"
      s_i0_h_dt = "s:i0_h:dt", "p.u./s, d/dt, 0-Current in Delta-Winding (HV), derivative, continuous, double"
      s_i0_m_dt = "s:i0_m:dt", "p.u./s, d/dt, 0-Current in Delta-Winding (MV), derivative, continuous, double"
      s_i0_l_dt = "s:i0_l:dt", "p.u./s, d/dt, 0-Current in Delta-Winding (LV), derivative, continuous, double"
      s_psim_r_dt = "s:psim_r:dt", "p.u./s, d/dt, Magnetising Flux (Real Part), derivative, continuous, double"
      s_psim_i_dt = "s:psim_i:dt", "p.u./s, d/dt, Magnetising Flux (Imaginary Part), derivative, continuous, double"
      s_psim_0_dt = "s:psim_0:dt", "p.u./s, d/dt, Magnetising Flux (0-Sequence), derivative, continuous, double"
      c_n3tap_h = "c:n3tap_h", "Current Tap Position (HV)"
      c_n3tap_m = "c:n3tap_m", "Current Tap Position (MV)"
      c_n3tap_l = "c:n3tap_l", "Current Tap Position (LV)"
      c_nntap_int_h = "c:nntap_int_h", "Tap (HV), relative Position"
      c_nntap_int_m = "c:nntap_int_m", "Tap (MV), relative Position"
      c_nntap_int_l = "c:nntap_int_l", "Tap (LV), relative Position"
      c_rs_h = "c:rs_h", "p.u., Short Circuit Resistance (HV)"
      c_rs_m = "c:rs_m", "p.u., Short Circuit Resistance (MV)"
      c_rs_l = "c:rs_l", "p.u., Short Circuit Resistance (LV)"
      c_ls_h = "c:ls_h", "p.u., Short Circuit Inductance (HV)"
      c_ls_m = "c:ls_m", "p.u., Short Circuit Inductance (MV)"
      c_ls_l = "c:ls_l", "p.u., Short Circuit Inductance (LV)"
      c_bm = "c:bm", "p.u., Magnetising Inductance"
      c_gm = "c:gm", "p.u., Magnetising Conductance"
      c_t_h_r = "c:t_h:r", "Complex Winding Ratio (HV/S), Real Part"
      c_t_h_i = "c:t_h:i", "Complex Winding Ratio (HV/S), Imaginary Part"
      c_t_h_phi = "c:t_h:phi", "Complex Winding Ratio (HV/S), Angle"
      c_t_h_mag = "c:t_h:mag", "Complex Winding Ratio (HV/S), Magnitude"
      c_t_h_phirad = "c:t_h:phirad", "Complex Winding Ratio (HV/S), Angle"
      c_t_m_r = "c:t_m:r", "Complex Winding Ratio (MV/S), Real Part"
      c_t_m_i = "c:t_m:i", "Complex Winding Ratio (MV/S), Imaginary Part"
      c_t_m_phi = "c:t_m:phi", "Complex Winding Ratio (MV/S), Angle"
      c_t_m_mag = "c:t_m:mag", "Complex Winding Ratio (MV/S), Magnitude"
      c_t_m_phirad = "c:t_m:phirad", "Complex Winding Ratio (MV/S), Angle"
      c_t_l_r = "c:t_l:r", "Complex Winding Ratio (LV/S), Real Part"
      c_t_l_i = "c:t_l:i", "Complex Winding Ratio (LV/S), Imaginary Part"
      c_t_l_phi = "c:t_l:phi", "Complex Winding Ratio (LV/S), Angle"
      c_t_l_mag = "c:t_l:mag", "Complex Winding Ratio (LV/S), Magnitude"
      c_t_l_phirad = "c:t_l:phirad", "Complex Winding Ratio (LV/S), Angle"
      c_i3loc = "c:i3loc", "Position of Magn.ind."
      c_i30lc = "c:i30lc", "Position of 0-Sequ. Magn.ind."
      c_rs0_h = "c:rs0_h", "p.u., 0-Sequ. Short-Circuit Imp (HV)"
      c_rs0_m = "c:rs0_m", "p.u., 0-Sequ. Short-Circuit Imp (MV)"
      c_rs0_l = "c:rs0_l", "p.u., 0-Sequ. Short-Circuit Imp (LV)"
      c_re0_h = "c:re0_h", "p.u., Earth Impedance (HV)"
      c_re0_m = "c:re0_m", "p.u., Earth Impedance (MV)"
      c_re0_l = "c:re0_l", "p.u., Earth Impedance (LV)"
      c_ls0_h = "c:ls0_h", "p.u., 0-Sequ. Short-Circuit Imp (HV)"
      c_ls0_m = "c:ls0_m", "p.u., 0-Sequ. Short-Circuit Imp (MV)"
      c_ls0_l = "c:ls0_l", "p.u., 0-Sequ. Short-Circuit Imp (LV)"
      c_le0_h = "c:le0_h", "p.u., Earth Impedance (HV)"
      c_le0_m = "c:le0_m", "p.u., Earth Impedance (MV)"
      c_le0_l = "c:le0_l", "p.u., Earth Impedance (LV)"
      c_i_star_h = "c:i_star_h", "Flag"
      c_i_star_m = "c:i_star_m", "Flag"
      c_i_star_l = "c:i_star_l", "Flag"
      c_i_earth_h = "c:i_earth_h", "Flag"
      c_i_earth_m = "c:i_earth_m", "Flag"
      c_i_earth_l = "c:i_earth_l", "Flag"
      c_bm0 = "c:bm0", "p.u., 0-Sequ. Magn. Inductance"
      c_rm0 = "c:rm0", "p.u., 0-Sequ. Magn. Resistance"
      c_hpi = "c:hpi", "rad/s, Angular frequency"
      c_i_stshort = "c:i_stshort", "Flag"
      c_Ustar_h = "c:Ustar_h", "kV, Star-Point Voltage (HV)"
      c_Ustar_m = "c:Ustar_m", "kV, Star-Point Voltage (MV)"
      c_Ustar_l = "c:Ustar_l", "kV, Star-Point Voltage (LV)"
      c_Istar_h = "c:Istar_h", "kA, Star-Point Current (HV)"
      c_Istar_m = "c:Istar_m", "kA, Star-Point Current (MV)"
      c_Istar_l = "c:Istar_l", "kA, Star-Point Current (LV)"
      c_w_0 = "c:w_0", "Nominal winding ratio"
      c_i_sat = "c:i_sat", "Saturated Model"
      c_im_a = "c:im_a", "p.u., Magnetising Current, Phase a"
      c_im_b = "c:im_b", "p.u., Magnetising Current, Phase b"
      c_im_c = "c:im_c", "p.u., Magnetising Current, Phase c"
      c_physt = "c:physt", "kW, Hysteresis losses"
      e_du3tp_h = "e:du3tp_h", "%, Add. Voltage per Tap"
      e_ph3tr_h = "e:ph3tr_h", "deg, Phase of du"
      e_n3tap_h = "e:n3tap_h", "Act. Position"
      e_du3tp_m = "e:du3tp_m", "%, Add. Voltage per Tap"
      e_ph3tr_m = "e:ph3tr_m", "deg, Phase of du"
      e_n3tap_m = "e:n3tap_m", "Act. Position"
      e_du3tp_l = "e:du3tp_l", "%, Add. Voltage per Tap"
      e_ph3tr_l = "e:ph3tr_l", "deg, Phase of du"
      e_n3tap_l = "e:n3tap_l", "Act. Position"
      e_iTaps = "e:iTaps", "According to Measurement Report"
      e_iMeasTap = "e:iMeasTap", "for Tap at"
      e_iMeasLoc = "e:iMeasLoc", "Measured at"
      e_mTaps = "e:mTaps", "Measurement Report"
      e_iopt_hf = "e:iopt_hf", "Consider Capacitances"
      e_Cg_h = "e:Cg_h", "uF, HV-Ground"
      e_Cg_m = "e:Cg_m", "uF, MV-Ground"
      e_Cg_l = "e:Cg_l", "uF, LV-Ground"
      e_Cc1_hm = "e:Cc1_hm", "uF, HV-MV"
      e_Cc1_ml = "e:Cc1_ml", "uF, MV-LV"
      e_Cc1_lh = "e:Cc1_lh", "uF, LV-HV"
      e_Cc0_hm = "e:Cc0_hm", "uF, HV-MV"
      e_Cc0_ml = "e:Cc0_ml", "uF, MV-LV"
      e_Cc0_lh = "e:Cc0_lh", "uF, LV-HV"
      e_iResFlux = "e:iResFlux", "Residual flux"
      e_PsiresA = "e:PsiresA", "p.u., Phase A"
      e_PsiresB = "e:PsiresB", "p.u., Phase B"
      e_PsiresC = "e:PsiresC", "p.u., Phase C"
      e_cPsiresC = "e:cPsiresC", "p.u., Phase C"
      t_du3tp_h = "t:du3tp_h", "%, Add. Voltage per Tap"
      t_ph3tr_h = "t:ph3tr_h", "deg, Phase of du"
      t_n3tp0_h = "t:n3tp0_h", "Neutral Position"
      t_n3tmn_h = "t:n3tmn_h", "Min. Position"
      t_n3tmx_h = "t:n3tmx_h", "Max. Position"
      t_du3tp_m = "t:du3tp_m", "%, Add. Voltage per Tap"
      t_ph3tr_m = "t:ph3tr_m", "deg, Phase of du"
      t_n3tp0_m = "t:n3tp0_m", "Neutral Position"
      t_n3tmn_m = "t:n3tmn_m", "Min. Position"
      t_n3tmx_m = "t:n3tmx_m", "Max. Position"
      t_du3tp_l = "t:du3tp_l", "%, Add. Voltage per Tap"
      t_ph3tr_l = "t:ph3tr_l", "deg, Phase of du"
      t_n3tp0_l = "t:n3tp0_l", "Neutral Position"
      t_n3tmn_l = "t:n3tmn_l", "Min. Position"
      t_n3tmx_l = "t:n3tmx_l", "Max. Position"
      t_itapos = "t:itapos", "Tap Modelled at"
      t_i3loc = "t:i3loc", "Position"
      t_curm3 = "t:curm3", "%, No Load Current"
      t_pfe = "t:pfe", "kW, No Load Losses"
      t_i30lc = "t:i30lc", "Position"
      t_cr0m3 = "t:cr0m3", "%, No Load Current"
      t_rtox0_n = "t:rtox0_n", "Mag. R/X"
      t_itrmt = "t:itrmt", "Type"
      t_psi0 = "t:psi0", "p.u., Knee Flux"
      t_x3lin = "t:x3lin", "p.u., Linear Reactance"
      t_x3air = "t:x3air", "p.u., Saturated Reactance"
      t_ksat = "t:ksat", "Saturation Exponent"
      t_satcur = "t:satcur", "p.u., Current (peak)"
      t_satflux = "t:satflux", "p.u., Flux (peak)"
      t_satcue = "t:satcue", "%, Current (RMS)"
      t_satvol = "t:satvol", "p.u., Voltage (RMS)"
      t_iInterPol = "t:iInterPol", "Interpolation"
      t_iFit = "t:iFit", "Data fitting"
      t_smoothfac = "t:smoothfac", "%, Smoothing Factor"
      t_iFinalSlope = "t:iFinalSlope", "Set final slope (peak values)"
      t_iHyster = "t:iHyster", "Model"
      t_Lwidth = "t:Lwidth", "%, Loop width"
      t_iLimb = "t:iLimb", "Core"
      t_aFrolich = "t:aFrolich", "Frolich equation coefficient a"
      t_bFrolich = "t:bFrolich", "Frolich equation coefficient b"
      t_cFrolich = "t:cFrolich", "Frolich equation coefficient c"
      t_xsatFrolich = "t:xsatFrolich", "Frolich equation saturated reactance (p.u.)"
      t_mseFrolich = "t:mseFrolich", "Frolich equation, mean squared error"

    class ElmVac(Enum):
      _init_ = 'value __doc__'

      m_u0_bus1 = "m:u0:bus1", "p.u., Zero-Sequence-Voltage"
      m_u1r_bus1 = "m:u1r:bus1", "p.u., Voltage Phasor, Real Part"
      m_u1i_bus1 = "m:u1i:bus1", "p.u., Voltage Phasor, Imaginary Part"
      m_u1_bus1 = "m:u1:bus1", "p.u., Voltage Phasor, Magnitude"
      m_U1_bus1 = "m:U1:bus1", "kV, Line-Ground Voltage Phasor, Magnitude"
      m_U1l_bus1 = "m:U1l:bus1", "kV, Line-Line Voltage Phasor, Magnitude"
      m_phiu1_bus1 = "m:phiu1:bus1", "deg, Voltage Phasor, Angle"
      m_I_bus1_A = "m:I:bus1:A", "kA, Phase Current"
      m_I_bus1_B = "m:I:bus1:B", "kA, Phase Current"
      m_I_bus1_C = "m:I:bus1:C", "kA, Phase Current"
      m_i0_bus1 = "m:i0:bus1", "p.u., Zero-Sequence Current"
      m_I0_bus1 = "m:I0:bus1", "kA, Zero-Sequence Current"
      m_I0x3_bus1 = "m:I0x3:bus1", "kA, 3*I0"
      m_i1r_bus1 = "m:i1r:bus1", "p.u., Current Phasor, Real Part"
      m_i1i_bus1 = "m:i1i:bus1", "p.u., Current Phasor, Imaginary Part"
      m_i1_bus1 = "m:i1:bus1", "p.u., Current Phasor, Magnitude"
      m_I1_bus1 = "m:I1:bus1", "kA, Current Phasor, Magnitude"
      m_phii1_bus1 = "m:phii1:bus1", "deg, Current Phasor, Angle"
      m_P_bus1_A = "m:P:bus1:A", "MW, Active Power"
      m_P_bus1_B = "m:P:bus1:B", "MW, Active Power"
      m_P_bus1_C = "m:P:bus1:C", "MW, Active Power"
      m_Psum_bus1 = "m:Psum:bus1", "MW, Power-Phasor, Active Power"
      m_Qsum_bus1 = "m:Qsum:bus1", "Mvar, Power Phasor, Reactive Part"
      m_Ssum_bus1 = "m:Ssum:bus1", "MVA, Power Phasor, Apparent Power"
      m_cosphi1_bus1 = "m:cosphi1:bus1", "Power-Phasor, Power-Factor"
      m_i1P_bus1 = "m:i1P:bus1", "p.u., Current Phasor, Active Current"
      m_i1Q_bus1 = "m:i1Q:bus1", "p.u., Current Phasor, Reactive Current"
      m_I1P_bus1 = "m:I1P:bus1", "kA, Current Phasor, Active Current"
      m_I1Q_bus1 = "m:I1Q:bus1", "kA, Current Phasor, Reactive Current"
      m_u_bus1_A = "m:u:bus1:A", "p.u., Phase Voltage"
      m_u_bus1_B = "m:u:bus1:B", "p.u., Phase Voltage"
      m_u_bus1_C = "m:u:bus1:C", "p.u., Phase Voltage"
      m_i_bus1_A = "m:i:bus1:A", "p.u., Phase Current"
      m_i_bus1_B = "m:i:bus1:B", "p.u., Phase Current"
      m_i_bus1_C = "m:i:bus1:C", "p.u., Phase Current"
      m_phiu1i1_bus1 = "m:phiu1i1:bus1", "deg, Angle between Voltage and Current Phasor"
      m_Tfct_bus1 = "m:Tfct:bus1", "s, Fault Clearing Time"
      m_TfctPh_bus1_A = "m:TfctPh:bus1:A", "s, Fault Clearing Time"
      m_TfctPh_bus1_B = "m:TfctPh:bus1:B", "s, Fault Clearing Time"
      m_TfctPh_bus1_C = "m:TfctPh:bus1:C", "s, Fault Clearing Time"
      n_u_bus1_A = "n:u:bus1:A", "p.u., Phase Voltage"
      n_u_bus1_B = "n:u:bus1:B", "p.u., Phase Voltage"
      n_u_bus1_C = "n:u:bus1:C", "p.u., Phase Voltage"
      n_upc_bus1_A = "n:upc:bus1:A", "%, Phase Voltage"
      n_upc_bus1_B = "n:upc:bus1:B", "%, Phase Voltage"
      n_upc_bus1_C = "n:upc:bus1:C", "%, Phase Voltage"
      n_U_bus1_A = "n:U:bus1:A", "kV, Phase Voltage"
      n_U_bus1_B = "n:U:bus1:B", "kV, Phase Voltage"
      n_U_bus1_C = "n:U:bus1:C", "kV, Phase Voltage"
      n_ul_bus1_A = "n:ul:bus1:A", "p.u., Line-Line Voltage"
      n_ul_bus1_B = "n:ul:bus1:B", "p.u., Line-Line Voltage"
      n_ul_bus1_C = "n:ul:bus1:C", "p.u., Line-Line Voltage"
      n_ulpc_bus1_A = "n:ulpc:bus1:A", "%, Line-Line Voltage"
      n_ulpc_bus1_B = "n:ulpc:bus1:B", "%, Line-Line Voltage"
      n_ulpc_bus1_C = "n:ulpc:bus1:C", "%, Line-Line Voltage"
      n_Ul_bus1_A = "n:Ul:bus1:A", "kV, Line-Line Voltage"
      n_Ul_bus1_B = "n:Ul:bus1:B", "kV, Line-Line Voltage"
      n_Ul_bus1_C = "n:Ul:bus1:C", "kV, Line-Line Voltage"
      n_u0_bus1 = "n:u0:bus1", "p.u., Zero-Sequence-Voltage"
      n_U0_bus1 = "n:U0:bus1", "kV, Zero-Sequence-Voltage"
      n_U0x3_bus1 = "n:U0x3:bus1", "kV, 3*U0"
      n_u1r_bus1 = "n:u1r:bus1", "p.u., Voltage Phasor, Real Part"
      n_u1i_bus1 = "n:u1i:bus1", "p.u., Voltage Phasor, Imaginary Part"
      n_u1_bus1 = "n:u1:bus1", "p.u., Voltage Phasor, Magnitude"
      n_u1pc_bus1 = "n:u1pc:bus1", "%, Voltage Phasor, Magnitude"
      n_U1_bus1 = "n:U1:bus1", "kV, Voltage Phasor, Magnitude"
      n_phiu1_bus1 = "n:phiu1:bus1", "deg, Voltage Phasor, Angle"
      n_U1l_bus1 = "n:U1l:bus1", "kV, Line-Line Voltage Phasor, Magnitude"
      n_uln_bus1_A = "n:uln:bus1:A", "p.u., Line-Neutral Voltage"
      n_uln_bus1_B = "n:uln:bus1:B", "p.u., Line-Neutral Voltage"
      n_uln_bus1_C = "n:uln:bus1:C", "p.u., Line-Neutral Voltage"
      n_Uln_bus1_A = "n:Uln:bus1:A", "kV, Line-Neutral Voltage"
      n_Uln_bus1_B = "n:Uln:bus1:B", "kV, Line-Neutral Voltage"
      n_Uln_bus1_C = "n:Uln:bus1:C", "kV, Line-Neutral Voltage"
      n_un_bus1 = "n:un:bus1", "p.u., Neutral-Ground Voltage"
      n_Un_bus1 = "n:Un:bus1", "kV, Neutral-Ground Voltage"
      n_ishc_bus1_A = "n:ishc:bus1:A", "p.u., Short-Circuit Current"
      n_ishc_bus1_B = "n:ishc:bus1:B", "p.u., Short-Circuit Current"
      n_ishc_bus1_C = "n:ishc:bus1:C", "p.u., Short-Circuit Current"
      n_Ishc_bus1_A = "n:Ishc:bus1:A", "kA, Short-Circuit Current"
      n_Ishc_bus1_B = "n:Ishc:bus1:B", "kA, Short-Circuit Current"
      n_Ishc_bus1_C = "n:Ishc:bus1:C", "kA, Short-Circuit Current"
      n_inshc_bus1 = "n:inshc:bus1", "p.u., Neutral Short-Circuit Current, magnitude"
      n_Inshc_bus1 = "n:Inshc:bus1", "kA, Neutral Short-Circuit Current, magnitude"
      n_fe_bus1 = "n:fe:bus1", "p.u., Electrical Frequency"
      n_frdev_bus1 = "n:frdev:bus1", "Hz, average frequency"
      n_dfehz_bus1 = "n:dfehz:bus1", "Hz, Deviation of the El. Frequency"
      n_fehz_bus1 = "n:fehz:bus1", "Hz, Electrical Frequency"
      n_frnom_bus1 = "n:frnom:bus1", "Hz, Nominal Frequency"
      s_u0 = "s:u0", "p.u., IN, Voltage-Input, continuous, double"
      s_Ul0 = "s:Ul0", "kV, IN, Voltage-Input (Line-Line), continuous, double"
      s_f0 = "s:f0", "p.u., IN, Frequency-Input continuous, double"
      s_F0Hz = "s:F0Hz", "Hz, IN, Frequency-Input continuous, double"
      s_U_A = "s:U_A", "kV, IN, Voltage, Magnitude, Phase a, continuous, double"
      s_U_B = "s:U_B", "kV, IN, Voltage, Magnitude, Phase b, continuous, double"
      s_U_C = "s:U_C", "kV, IN, Voltage, Magnitude, Phase c, continuous, double"
      s_dphiu = "s:dphiu", "rad, IN, Additional Angle, continuous, double"
      s_phiu = "s:phiu", "rad, STATE, Voltage-Angle, continuous, double"
      s_phiu_dt = "s:phiu:dt", "rad/s, d/dt, Voltage-Angle, derivative, continuous, double"
      c_hpi = "c:hpi", "rad/s, Nominal-Angular-Frequency"
      c_uini0 = "c:uini0", "0-Sequ. Voltage"
      c_uini2 = "c:uini2", "2-Sequ. Voltage"
      c_phiini0 = "c:phiini0", "rad, 0-Sequ. Voltage, Angle"
      c_phiini2 = "c:phiini2", "rad, 2-Sequ. Voltage, Angle"
      e_R1 = "e:R1", "Ohm, Resistance, R1"
      e_X1 = "e:X1", "Ohm, Reactance, X1"
      e_R0 = "e:R0", "Ohm, Resistance, R0"
      e_X0 = "e:X0", "Ohm, Reactance, X0"
      e_leadUinput = "e:leadUinput", "Voltage input"
      e_leadFinput = "e:leadFinput", "Frequency input"
      e_cosimModel = "e:cosimModel", "Used for Co-simulation"
      e_phiSlackRef = "e:phiSlackRef", "rad, Initial rotor angle"
      e_phiiniRef = "e:phiiniRef", "rad, Initial angle of bus voltage"

    class ElmVsc(Enum):
      _init_ = 'value __doc__'

      m_u0_busac = "m:u0:busac", "p.u., Zero-Sequence-Voltage"
      m_u0_busdp = "m:u0:busdp", "p.u., Zero-Sequence-Voltage"
      m_u0_busdm = "m:u0:busdm", "p.u., Zero-Sequence-Voltage"
      m_u1r_busac = "m:u1r:busac", "p.u., Voltage Phasor, Real Part"
      m_u1r_busdp = "m:u1r:busdp", "p.u., Voltage Phasor, Real Part"
      m_u1r_busdm = "m:u1r:busdm", "p.u., Voltage Phasor, Real Part"
      m_u1i_busac = "m:u1i:busac", "p.u., Voltage Phasor, Imaginary Part"
      m_u1i_busdp = "m:u1i:busdp", "p.u., Voltage Phasor, Imaginary Part"
      m_u1i_busdm = "m:u1i:busdm", "p.u., Voltage Phasor, Imaginary Part"
      m_u1_busac = "m:u1:busac", "p.u., Voltage Phasor, Magnitude"
      m_u1_busdp = "m:u1:busdp", "p.u., Voltage Phasor, Magnitude"
      m_u1_busdm = "m:u1:busdm", "p.u., Voltage Phasor, Magnitude"
      m_U1_busac = "m:U1:busac", "kV, Line-Ground Voltage Phasor, Magnitude"
      m_U1_busdp = "m:U1:busdp", "kV, Line-Ground Voltage Phasor, Magnitude"
      m_U1_busdm = "m:U1:busdm", "kV, Line-Ground Voltage Phasor, Magnitude"
      m_U1l_busac = "m:U1l:busac", "kV, Line-Line Voltage Phasor, Magnitude"
      m_U1l_busdp = "m:U1l:busdp", "kV, Line-Line Voltage Phasor, Magnitude"
      m_U1l_busdm = "m:U1l:busdm", "kV, Line-Line Voltage Phasor, Magnitude"
      m_phiu1_busac = "m:phiu1:busac", "deg, Voltage Phasor, Angle"
      m_phiu1_busdp = "m:phiu1:busdp", "deg, Voltage Phasor, Angle"
      m_phiu1_busdm = "m:phiu1:busdm", "deg, Voltage Phasor, Angle"
      m_I_busac_A = "m:I:busac:A", "kA, Phase Current"
      m_I_busac_B = "m:I:busac:B", "kA, Phase Current"
      m_I_busac_C = "m:I:busac:C", "kA, Phase Current"
      m_I_busdp = "m:I:busdp", "kA, Phase Current"
      m_I_busdm = "m:I:busdm", "kA, Phase Current"
      m_i0_busac = "m:i0:busac", "p.u., Zero-Sequence Current"
      m_i0_busdp = "m:i0:busdp", "p.u., Zero-Sequence Current"
      m_i0_busdm = "m:i0:busdm", "p.u., Zero-Sequence Current"
      m_I0_busac = "m:I0:busac", "kA, Zero-Sequence Current"
      m_I0_busdp = "m:I0:busdp", "kA, Zero-Sequence Current"
      m_I0_busdm = "m:I0:busdm", "kA, Zero-Sequence Current"
      m_I0x3_busac = "m:I0x3:busac", "kA, 3*I0"
      m_I0x3_busdp = "m:I0x3:busdp", "kA, 3*I0"
      m_I0x3_busdm = "m:I0x3:busdm", "kA, 3*I0"
      m_i1r_busac = "m:i1r:busac", "p.u., Current Phasor, Real Part"
      m_i1r_busdp = "m:i1r:busdp", "p.u., Current Phasor, Real Part"
      m_i1r_busdm = "m:i1r:busdm", "p.u., Current Phasor, Real Part"
      m_i1i_busac = "m:i1i:busac", "p.u., Current Phasor, Imaginary Part"
      m_i1i_busdp = "m:i1i:busdp", "p.u., Current Phasor, Imaginary Part"
      m_i1i_busdm = "m:i1i:busdm", "p.u., Current Phasor, Imaginary Part"
      m_i1_busac = "m:i1:busac", "p.u., Current Phasor, Magnitude"
      m_i1_busdp = "m:i1:busdp", "p.u., Current Phasor, Magnitude"
      m_i1_busdm = "m:i1:busdm", "p.u., Current Phasor, Magnitude"
      m_I1_busac = "m:I1:busac", "kA, Current Phasor, Magnitude"
      m_I1_busdp = "m:I1:busdp", "kA, Current Phasor, Magnitude"
      m_I1_busdm = "m:I1:busdm", "kA, Current Phasor, Magnitude"
      m_phii1_busac = "m:phii1:busac", "deg, Current Phasor, Angle"
      m_phii1_busdp = "m:phii1:busdp", "deg, Current Phasor, Angle"
      m_phii1_busdm = "m:phii1:busdm", "deg, Current Phasor, Angle"
      m_P_busac_A = "m:P:busac:A", "MW, Active Power"
      m_P_busac_B = "m:P:busac:B", "MW, Active Power"
      m_P_busac_C = "m:P:busac:C", "MW, Active Power"
      m_P_busdp = "m:P:busdp", "MW, Active Power"
      m_P_busdm = "m:P:busdm", "MW, Active Power"
      m_Psum_busac = "m:Psum:busac", "MW, Power-Phasor, Active Power"
      m_Psum_busdp = "m:Psum:busdp", "MW, Power-Phasor, Active Power"
      m_Psum_busdm = "m:Psum:busdm", "MW, Power-Phasor, Active Power"
      m_Qsum_busac = "m:Qsum:busac", "Mvar, Power Phasor, Reactive Part"
      m_Qsum_busdp = "m:Qsum:busdp", "Mvar, Power Phasor, Reactive Part"
      m_Qsum_busdm = "m:Qsum:busdm", "Mvar, Power Phasor, Reactive Part"
      m_Ssum_busac = "m:Ssum:busac", "MVA, Power Phasor, Apparent Power"
      m_Ssum_busdp = "m:Ssum:busdp", "MVA, Power Phasor, Apparent Power"
      m_Ssum_busdm = "m:Ssum:busdm", "MVA, Power Phasor, Apparent Power"
      m_cosphi1_busac = "m:cosphi1:busac", "Power-Phasor, Power-Factor"
      m_cosphi1_busdp = "m:cosphi1:busdp", "Power-Phasor, Power-Factor"
      m_cosphi1_busdm = "m:cosphi1:busdm", "Power-Phasor, Power-Factor"
      m_i1P_busac = "m:i1P:busac", "p.u., Current Phasor, Active Current"
      m_i1P_busdp = "m:i1P:busdp", "p.u., Current Phasor, Active Current"
      m_i1P_busdm = "m:i1P:busdm", "p.u., Current Phasor, Active Current"
      m_i1Q_busac = "m:i1Q:busac", "p.u., Current Phasor, Reactive Current"
      m_i1Q_busdp = "m:i1Q:busdp", "p.u., Current Phasor, Reactive Current"
      m_i1Q_busdm = "m:i1Q:busdm", "p.u., Current Phasor, Reactive Current"
      m_I1P_busac = "m:I1P:busac", "kA, Current Phasor, Active Current"
      m_I1P_busdp = "m:I1P:busdp", "kA, Current Phasor, Active Current"
      m_I1P_busdm = "m:I1P:busdm", "kA, Current Phasor, Active Current"
      m_I1Q_busac = "m:I1Q:busac", "kA, Current Phasor, Reactive Current"
      m_I1Q_busdp = "m:I1Q:busdp", "kA, Current Phasor, Reactive Current"
      m_I1Q_busdm = "m:I1Q:busdm", "kA, Current Phasor, Reactive Current"
      m_u_busac_A = "m:u:busac:A", "p.u., Phase Voltage"
      m_u_busac_B = "m:u:busac:B", "p.u., Phase Voltage"
      m_u_busac_C = "m:u:busac:C", "p.u., Phase Voltage"
      m_u_busdp = "m:u:busdp", "p.u., Phase Voltage"
      m_u_busdm = "m:u:busdm", "p.u., Phase Voltage"
      m_i_busac_A = "m:i:busac:A", "p.u., Phase Current"
      m_i_busac_B = "m:i:busac:B", "p.u., Phase Current"
      m_i_busac_C = "m:i:busac:C", "p.u., Phase Current"
      m_i_busdp = "m:i:busdp", "p.u., Phase Current"
      m_i_busdm = "m:i:busdm", "p.u., Phase Current"
      m_phiu1i1_busac = "m:phiu1i1:busac", "deg, Angle between Voltage and Current Phasor"
      m_phiu1i1_busdp = "m:phiu1i1:busdp", "deg, Angle between Voltage and Current Phasor"
      m_phiu1i1_busdm = "m:phiu1i1:busdm", "deg, Angle between Voltage and Current Phasor"
      m_Tfct_busac = "m:Tfct:busac", "s, Fault Clearing Time"
      m_Tfct_busdp = "m:Tfct:busdp", "s, Fault Clearing Time"
      m_Tfct_busdm = "m:Tfct:busdm", "s, Fault Clearing Time"
      m_TfctPh_busac_A = "m:TfctPh:busac:A", "s, Fault Clearing Time"
      m_TfctPh_busac_B = "m:TfctPh:busac:B", "s, Fault Clearing Time"
      m_TfctPh_busac_C = "m:TfctPh:busac:C", "s, Fault Clearing Time"
      m_TfctPh_busdp = "m:TfctPh:busdp", "s, Fault Clearing Time"
      m_TfctPh_busdm = "m:TfctPh:busdm", "s, Fault Clearing Time"
      n_u_busac_A = "n:u:busac:A", "p.u., Phase Voltage"
      n_u_busac_B = "n:u:busac:B", "p.u., Phase Voltage"
      n_u_busac_C = "n:u:busac:C", "p.u., Phase Voltage"
      n_u_busdp = "n:u:busdp", "p.u., Phase Voltage"
      n_u_busdm = "n:u:busdm", "p.u., Phase Voltage"
      n_upc_busac_A = "n:upc:busac:A", "%, Phase Voltage"
      n_upc_busac_B = "n:upc:busac:B", "%, Phase Voltage"
      n_upc_busac_C = "n:upc:busac:C", "%, Phase Voltage"
      n_upc_busdp = "n:upc:busdp", "%, Phase Voltage"
      n_upc_busdm = "n:upc:busdm", "%, Phase Voltage"
      n_U_busac_A = "n:U:busac:A", "kV, Phase Voltage"
      n_U_busac_B = "n:U:busac:B", "kV, Phase Voltage"
      n_U_busac_C = "n:U:busac:C", "kV, Phase Voltage"
      n_U_busdp = "n:U:busdp", "kV, Phase Voltage"
      n_U_busdm = "n:U:busdm", "kV, Phase Voltage"
      n_ul_busac_A = "n:ul:busac:A", "p.u., Line-Line Voltage"
      n_ul_busac_B = "n:ul:busac:B", "p.u., Line-Line Voltage"
      n_ul_busac_C = "n:ul:busac:C", "p.u., Line-Line Voltage"
      n_ul_busdp = "n:ul:busdp", "p.u., Line-Line Voltage"
      n_ul_busdm = "n:ul:busdm", "p.u., Line-Line Voltage"
      n_ulpc_busac_A = "n:ulpc:busac:A", "%, Line-Line Voltage"
      n_ulpc_busac_B = "n:ulpc:busac:B", "%, Line-Line Voltage"
      n_ulpc_busac_C = "n:ulpc:busac:C", "%, Line-Line Voltage"
      n_ulpc_busdp = "n:ulpc:busdp", "%, Line-Line Voltage"
      n_ulpc_busdm = "n:ulpc:busdm", "%, Line-Line Voltage"
      n_Ul_busac_A = "n:Ul:busac:A", "kV, Line-Line Voltage"
      n_Ul_busac_B = "n:Ul:busac:B", "kV, Line-Line Voltage"
      n_Ul_busac_C = "n:Ul:busac:C", "kV, Line-Line Voltage"
      n_Ul_busdp = "n:Ul:busdp", "kV, Line-Line Voltage"
      n_Ul_busdm = "n:Ul:busdm", "kV, Line-Line Voltage"
      n_u0_busac = "n:u0:busac", "p.u., Zero-Sequence-Voltage"
      n_u0_busdp = "n:u0:busdp", "p.u., Zero-Sequence-Voltage"
      n_u0_busdm = "n:u0:busdm", "p.u., Zero-Sequence-Voltage"
      n_U0_busac = "n:U0:busac", "kV, Zero-Sequence-Voltage"
      n_U0_busdp = "n:U0:busdp", "kV, Zero-Sequence-Voltage"
      n_U0_busdm = "n:U0:busdm", "kV, Zero-Sequence-Voltage"
      n_U0x3_busac = "n:U0x3:busac", "kV, 3*U0"
      n_U0x3_busdp = "n:U0x3:busdp", "kV, 3*U0"
      n_U0x3_busdm = "n:U0x3:busdm", "kV, 3*U0"
      n_u1r_busac = "n:u1r:busac", "p.u., Voltage Phasor, Real Part"
      n_u1r_busdp = "n:u1r:busdp", "p.u., Voltage Phasor, Real Part"
      n_u1r_busdm = "n:u1r:busdm", "p.u., Voltage Phasor, Real Part"
      n_u1i_busac = "n:u1i:busac", "p.u., Voltage Phasor, Imaginary Part"
      n_u1i_busdp = "n:u1i:busdp", "p.u., Voltage Phasor, Imaginary Part"
      n_u1i_busdm = "n:u1i:busdm", "p.u., Voltage Phasor, Imaginary Part"
      n_u1_busac = "n:u1:busac", "p.u., Voltage Phasor, Magnitude"
      n_u1_busdp = "n:u1:busdp", "p.u., Voltage Phasor, Magnitude"
      n_u1_busdm = "n:u1:busdm", "p.u., Voltage Phasor, Magnitude"
      n_u1pc_busac = "n:u1pc:busac", "%, Voltage Phasor, Magnitude"
      n_u1pc_busdp = "n:u1pc:busdp", "%, Voltage Phasor, Magnitude"
      n_u1pc_busdm = "n:u1pc:busdm", "%, Voltage Phasor, Magnitude"
      n_U1_busac = "n:U1:busac", "kV, Voltage Phasor, Magnitude"
      n_U1_busdp = "n:U1:busdp", "kV, Voltage Phasor, Magnitude"
      n_U1_busdm = "n:U1:busdm", "kV, Voltage Phasor, Magnitude"
      n_phiu1_busac = "n:phiu1:busac", "deg, Voltage Phasor, Angle"
      n_phiu1_busdp = "n:phiu1:busdp", "deg, Voltage Phasor, Angle"
      n_phiu1_busdm = "n:phiu1:busdm", "deg, Voltage Phasor, Angle"
      n_U1l_busac = "n:U1l:busac", "kV, Line-Line Voltage Phasor, Magnitude"
      n_U1l_busdp = "n:U1l:busdp", "kV, Line-Line Voltage Phasor, Magnitude"
      n_U1l_busdm = "n:U1l:busdm", "kV, Line-Line Voltage Phasor, Magnitude"
      n_uln_busac_A = "n:uln:busac:A", "p.u., Line-Neutral Voltage"
      n_uln_busac_B = "n:uln:busac:B", "p.u., Line-Neutral Voltage"
      n_uln_busac_C = "n:uln:busac:C", "p.u., Line-Neutral Voltage"
      n_uln_busdp = "n:uln:busdp", "p.u., Line-Neutral Voltage"
      n_uln_busdm = "n:uln:busdm", "p.u., Line-Neutral Voltage"
      n_Uln_busac_A = "n:Uln:busac:A", "kV, Line-Neutral Voltage"
      n_Uln_busac_B = "n:Uln:busac:B", "kV, Line-Neutral Voltage"
      n_Uln_busac_C = "n:Uln:busac:C", "kV, Line-Neutral Voltage"
      n_Uln_busdp = "n:Uln:busdp", "kV, Line-Neutral Voltage"
      n_Uln_busdm = "n:Uln:busdm", "kV, Line-Neutral Voltage"
      n_un_busac = "n:un:busac", "p.u., Neutral-Ground Voltage"
      n_un_busdp = "n:un:busdp", "p.u., Neutral-Ground Voltage"
      n_un_busdm = "n:un:busdm", "p.u., Neutral-Ground Voltage"
      n_Un_busac = "n:Un:busac", "kV, Neutral-Ground Voltage"
      n_Un_busdp = "n:Un:busdp", "kV, Neutral-Ground Voltage"
      n_Un_busdm = "n:Un:busdm", "kV, Neutral-Ground Voltage"
      n_ishc_busac_A = "n:ishc:busac:A", "p.u., Short-Circuit Current"
      n_ishc_busac_B = "n:ishc:busac:B", "p.u., Short-Circuit Current"
      n_ishc_busac_C = "n:ishc:busac:C", "p.u., Short-Circuit Current"
      n_ishc_busdp = "n:ishc:busdp", "p.u., Short-Circuit Current"
      n_ishc_busdm = "n:ishc:busdm", "p.u., Short-Circuit Current"
      n_Ishc_busac_A = "n:Ishc:busac:A", "kA, Short-Circuit Current"
      n_Ishc_busac_B = "n:Ishc:busac:B", "kA, Short-Circuit Current"
      n_Ishc_busac_C = "n:Ishc:busac:C", "kA, Short-Circuit Current"
      n_Ishc_busdp = "n:Ishc:busdp", "kA, Short-Circuit Current"
      n_Ishc_busdm = "n:Ishc:busdm", "kA, Short-Circuit Current"
      n_inshc_busac = "n:inshc:busac", "p.u., Neutral Short-Circuit Current, magnitude"
      n_inshc_busdp = "n:inshc:busdp", "p.u., Neutral Short-Circuit Current, magnitude"
      n_inshc_busdm = "n:inshc:busdm", "p.u., Neutral Short-Circuit Current, magnitude"
      n_Inshc_busac = "n:Inshc:busac", "kA, Neutral Short-Circuit Current, magnitude"
      n_Inshc_busdp = "n:Inshc:busdp", "kA, Neutral Short-Circuit Current, magnitude"
      n_Inshc_busdm = "n:Inshc:busdm", "kA, Neutral Short-Circuit Current, magnitude"
      n_fe_busac = "n:fe:busac", "p.u., Electrical Frequency"
      n_fe_busdp = "n:fe:busdp", "p.u., Electrical Frequency"
      n_fe_busdm = "n:fe:busdm", "p.u., Electrical Frequency"
      n_frdev_busac = "n:frdev:busac", "Hz, average frequency"
      n_frdev_busdp = "n:frdev:busdp", "Hz, average frequency"
      n_frdev_busdm = "n:frdev:busdm", "Hz, average frequency"
      n_dfehz_busac = "n:dfehz:busac", "Hz, Deviation of the El. Frequency"
      n_dfehz_busdp = "n:dfehz:busdp", "Hz, Deviation of the El. Frequency"
      n_dfehz_busdm = "n:dfehz:busdm", "Hz, Deviation of the El. Frequency"
      n_fehz_busac = "n:fehz:busac", "Hz, Electrical Frequency"
      n_fehz_busdp = "n:fehz:busdp", "Hz, Electrical Frequency"
      n_fehz_busdm = "n:fehz:busdm", "Hz, Electrical Frequency"
      n_frnom_busac = "n:frnom:busac", "Hz, Nominal Frequency"
      n_frnom_busdp = "n:frnom:busdp", "Hz, Nominal Frequency"
      n_frnom_busdm = "n:frnom:busdm", "Hz, Nominal Frequency"
      s_Pm_in = "s:Pm_in", "p.u., IN, Pulse-modulation factor (input) continuous, double"
      s_f0 = "s:f0", "p.u., IN, Frequency-input continuous, double"
      s_F0Hz = "s:F0Hz", "Hz, IN, Frequency-input continuous, double"
      s_dphiu = "s:dphiu", "rad, IN, Voltage-angle (input), continuous, double"
      s_u_a = "s:u_a", "p.u., IN, Firing signal, phase A, continuous, double"
      s_u_b = "s:u_b", "p.u., IN, Firing signal, phase B, continuous, double"
      s_u_c = "s:u_c", "p.u., IN, Firing signal, phase C, continuous, double"
      s_Pmr = "s:Pmr", "IN, Pulse-modulation factor (real part), continuous, double"
      s_Pmi = "s:Pmi", "IN, Pulse-modulation factor (imaginary part), continuous, double"
      s_cosref = "s:cosref", "IN, Cos(dq-reference-angle) continuous, double"
      s_sinref = "s:sinref", "IN, Sin(dq-reference-angle) continuous, double"
      s_Pmd = "s:Pmd", "IN, Pulse-modulation factor (d-axis), continuous, double"
      s_Pmq = "s:Pmq", "IN, Pulse-modulation factor (q-axis), continuous, double"
      s_id_ref = "s:id_ref", "p.u., IN, d-axis current reference, continuous, double"
      s_iq_ref = "s:iq_ref", "p.u., IN, q-axis current reference, continuous, double"
      s_i2d_ref = "s:i2d_ref", "p.u., IN, Negative sequence current, d-axis reference, continuous, double"
      s_i2q_ref = "s:i2q_ref", "p.u., IN, Negative sequence current, q-axis reference, continuous, double"
      s_block = "s:block", "p.u., IN, Blocking, continuous, double"
      s_urefA = "s:urefA", "p.u., IN, Reference voltage, phase A, continuous, double"
      s_urefB = "s:urefB", "p.u., IN, Reference voltage, phase B, continuous, double"
      s_urefC = "s:urefC", "p.u., IN, Reference voltage, phase C, continuous, double"
      s_Pm0 = "s:Pm0", "p.u., IN, Pulse-modulation factor (zero sequence) continuous, double"
      s_g1 = "s:g1", "IN, Firing signal, T1, continuous, double"
      s_g2 = "s:g2", "IN, Firing signal, T2, continuous, double"
      s_g3 = "s:g3", "IN, Firing signal, T3, continuous, double"
      s_g4 = "s:g4", "IN, Firing signal, T4, continuous, double"
      s_g5 = "s:g5", "IN, Firing signal, T5, continuous, double"
      s_g6 = "s:g6", "IN, Firing signal, T6, continuous, double"
      s_iDC = "s:iDC", "p.u., OUT, DC-current, continuous, double"
      s_uDC = "s:uDC", "p.u., OUT, DC-voltage, continuous, double"
      s_iAC_r = "s:iAC_r", "p.u., OUT, AC-current, real part, continuous, double"
      s_iAC_i = "s:iAC_i", "p.u., OUT, AC-current, imaginary part, continuous, double"
      s_uAC_r = "s:uAC_r", "p.u., OUT, AC-voltage, real part, continuous, double"
      s_uAC_i = "s:uAC_i", "p.u., OUT, AC-voltage, imaginary part, continuous, double"
      s_xd = "s:xd", "p.u., OUT, State variable, current controller d-axis, continuous, double"
      s_xq = "s:xq", "p.u., OUT, State variable, current controller q-axis, continuous, double"
      s_Upc_A = "s:Upc_A", "kV, OUT, Capacitive voltage, TpA continuous, double"
      s_Upc_B = "s:Upc_B", "kV, OUT, Capacitive voltage, TpB continuous, double"
      s_Upc_C = "s:Upc_C", "kV, OUT, Capacitive voltage, TpC continuous, double"
      s_Umc_A = "s:Umc_A", "kV, OUT, Capacitive voltage, TmA continuous, double"
      s_Umc_B = "s:Umc_B", "kV, OUT, Capacitive voltage, TmB continuous, double"
      s_Umc_C = "s:Umc_C", "kV, OUT, Capacitive voltage, TmC continuous, double"
      s_id = "s:id", "p.u., OUT, Current, d-axis continuous, double"
      s_iq = "s:iq", "p.u., OUT, Current, q-axis continuous, double"
      s_Ip_A = "s:Ip_A", "kA, OUT, T1 current, continuous, double"
      s_Ip_B = "s:Ip_B", "kA, OUT, T2 current, continuous, double"
      s_Ip_C = "s:Ip_C", "kA, OUT, T3 current, continuous, double"
      s_Im_A = "s:Im_A", "kA, OUT, T4 current, continuous, double"
      s_Im_B = "s:Im_B", "kA, OUT, T5 current, continuous, double"
      s_Im_C = "s:Im_C", "kA, OUT, T6 current, continuous, double"
      s_Idp_A = "s:Idp_A", "kA, OUT, D1 current, continuous, double"
      s_Idp_B = "s:Idp_B", "kA, OUT, D2 current, continuous, double"
      s_Idp_C = "s:Idp_C", "kA, OUT, D3 current, continuous, double"
      s_Idm_A = "s:Idm_A", "kA, OUT, D4 current, continuous, double"
      s_Idm_B = "s:Idm_B", "kA, OUT, D5 current, continuous, double"
      s_Idm_C = "s:Idm_C", "kA, OUT, D6 current, continuous, double"
      s_xd_dt = "s:xd:dt", "p.u./s, OUT, State variable, current controller d-axis, derivative, continuous, double"
      s_xq_dt = "s:xq:dt", "p.u./s, OUT, State variable, current controller q-axis, derivative, continuous, double"
      s_Upc_A_dt = "s:Upc_A:dt", "kV/s, OUT, Capacitive voltage, TpA, derivative, continuous, double"
      s_Upc_B_dt = "s:Upc_B:dt", "kV/s, OUT, Capacitive voltage, TpB, derivative, continuous, double"
      s_Upc_C_dt = "s:Upc_C:dt", "kV/s, OUT, Capacitive voltage, TpC, derivative, continuous, double"
      s_Umc_A_dt = "s:Umc_A:dt", "kV/s, OUT, Capacitive voltage, TmA, derivative, continuous, double"
      s_Umc_B_dt = "s:Umc_B:dt", "kV/s, OUT, Capacitive voltage, TmB, derivative, continuous, double"
      s_Umc_C_dt = "s:Umc_C:dt", "kV/s, OUT, Capacitive voltage, TmC, derivative, continuous, double"
      s_phiu = "s:phiu", "rad, STATE, Voltage-angle, continuous, double"
      s_phiu_dt = "s:phiu:dt", "rad/s, d/dt, Voltage-angle, derivative, continuous, double"
      c_hpi = "c:hpi", "rad/s, Nominal-angular-frequency"
      c_Pm = "c:Pm", "Pulse-modulation factor"
      c_snomn = "c:snomn", "MVA, Total rated apparent power (nparnum*Snom)"
      c_fmod = "c:fmod", "Hz, Modulation-frequency"
      c_i_mod = "c:i_mod", "Modulation"
      c_K0 = "c:K0", "Vac/Vdc at PWM=1"
      c_Losses = "c:Losses", "kW, Total converter losses"
      c_Pmd_ctrl = "c:Pmd_ctrl", "Pulse-modulation factor (d-axis)"
      c_Pmq_ctrl = "c:Pmq_ctrl", "Pulse-modulation factor (q-axis)"
      c_i_acdc = "c:i_acdc", "Control mode"
      e_pSubModel = "e:pSubModel", "ElmSubmodel, Submodel"
      e_iVIsource = "e:iVIsource", "Independent AC and DC voltage/current source model"
      e_Rac = "e:Rac", "Ohm, AC resistance"
      e_Lac = "e:Lac", "mH, AC inductance"
      e_R0toR1ac = "e:R0toR1ac", "R0/R1 ratio"
      e_X0toX1ac = "e:X0toX1ac", "X0/X1 ratio"
      e_Rdc = "e:Rdc", "Ohm, DC resistance"
      e_Ldc = "e:Ldc", "mH, DC inductance"
      e_i_det = "e:i_det", "Model"
      e_mmcModel = "e:mmcModel", "MMC model"
      e_Pnold = "e:Pnold", "kW, No-load losses"
      e_swtLossFactor = "e:swtLossFactor", "kW/A, Switching loss factor"
      e_resLossFactor = "e:resLossFactor", "Ohm, Resistive loss factor"
      e_fmod = "e:fmod", "Hz, Modulation frequency"
      e_Tdeadtime = "e:Tdeadtime", "us, Dead time"
      e_Ron = "e:Ron", "Ohm, On-resistance"
      e_Goff = "e:Goff", "uS, Off-conductance"
      e_Cvalve = "e:Cvalve", "uF, Snubber capacitance"
      e_Gvalve = "e:Gvalve", "S, Snubber conductance"
      e_NmmcSM = "e:NmmcSM", "Number of submodules per arm"
      e_mmcCmod = "e:mmcCmod", "uF, Submodule capacitance"
      e_mmcGcap = "e:mmcGcap", "uS, Capacitor parallel conductance"
      e_mmcModType = "e:mmcModType", "Modulation"
      e_fmodMmc = "e:fmodMmc", "Hz, Modulation frequency"
      e_Rdon = "e:Rdon", "Ohm, On-resistance"
      e_Gdoff = "e:Gdoff", "uS, Off-conductance"
      e_i_ctrl = "e:i_ctrl", "Use integrated current controller"
      e_Kd = "e:Kd", "Kd: d-axis, proportional gain"
      e_Td = "e:Td", "s, Td: d-axis, integration time constant"
      e_Kq = "e:Kq", "Kq: q-axis, proportional gain"
      e_Tq = "e:Tq", "s, Tq: q-axis, integration time constant"
      e_iArmFilt = "e:iArmFilt", "2nd-Harmonic Filter"
      e_cLarm1 = "e:cLarm1", "mH, Inductance, Larm1"
      e_Lfilt = "e:Lfilt", "mH, Inductance, Lfilt"
      e_Cfilt = "e:Cfilt", "uF, Capacitance, Cfilt"
      r_pSubModel = "r:pSubModel", "ElmSubmodel, Submodel"

    class ElmVscmono(Enum):
      _init_ = 'value __doc__'

      m_u0_busac = "m:u0:busac", "p.u., Zero-Sequence-Voltage"
      m_u0_busdc = "m:u0:busdc", "p.u., Zero-Sequence-Voltage"
      m_u1r_busac = "m:u1r:busac", "p.u., Voltage Phasor, Real Part"
      m_u1r_busdc = "m:u1r:busdc", "p.u., Voltage Phasor, Real Part"
      m_u1i_busac = "m:u1i:busac", "p.u., Voltage Phasor, Imaginary Part"
      m_u1i_busdc = "m:u1i:busdc", "p.u., Voltage Phasor, Imaginary Part"
      m_u1_busac = "m:u1:busac", "p.u., Voltage Phasor, Magnitude"
      m_u1_busdc = "m:u1:busdc", "p.u., Voltage Phasor, Magnitude"
      m_U1_busac = "m:U1:busac", "kV, Line-Ground Voltage Phasor, Magnitude"
      m_U1_busdc = "m:U1:busdc", "kV, Line-Ground Voltage Phasor, Magnitude"
      m_U1l_busac = "m:U1l:busac", "kV, Line-Line Voltage Phasor, Magnitude"
      m_U1l_busdc = "m:U1l:busdc", "kV, Line-Line Voltage Phasor, Magnitude"
      m_phiu1_busac = "m:phiu1:busac", "deg, Voltage Phasor, Angle"
      m_phiu1_busdc = "m:phiu1:busdc", "deg, Voltage Phasor, Angle"
      m_I_busac_A = "m:I:busac:A", "kA, Phase Current"
      m_I_busac_B = "m:I:busac:B", "kA, Phase Current"
      m_I_busac_C = "m:I:busac:C", "kA, Phase Current"
      m_I_busdc = "m:I:busdc", "kA, Phase Current"
      m_i0_busac = "m:i0:busac", "p.u., Zero-Sequence Current"
      m_i0_busdc = "m:i0:busdc", "p.u., Zero-Sequence Current"
      m_I0_busac = "m:I0:busac", "kA, Zero-Sequence Current"
      m_I0_busdc = "m:I0:busdc", "kA, Zero-Sequence Current"
      m_I0x3_busac = "m:I0x3:busac", "kA, 3*I0"
      m_I0x3_busdc = "m:I0x3:busdc", "kA, 3*I0"
      m_i1r_busac = "m:i1r:busac", "p.u., Current Phasor, Real Part"
      m_i1r_busdc = "m:i1r:busdc", "p.u., Current Phasor, Real Part"
      m_i1i_busac = "m:i1i:busac", "p.u., Current Phasor, Imaginary Part"
      m_i1i_busdc = "m:i1i:busdc", "p.u., Current Phasor, Imaginary Part"
      m_i1_busac = "m:i1:busac", "p.u., Current Phasor, Magnitude"
      m_i1_busdc = "m:i1:busdc", "p.u., Current Phasor, Magnitude"
      m_I1_busac = "m:I1:busac", "kA, Current Phasor, Magnitude"
      m_I1_busdc = "m:I1:busdc", "kA, Current Phasor, Magnitude"
      m_phii1_busac = "m:phii1:busac", "deg, Current Phasor, Angle"
      m_phii1_busdc = "m:phii1:busdc", "deg, Current Phasor, Angle"
      m_P_busac_A = "m:P:busac:A", "MW, Active Power"
      m_P_busac_B = "m:P:busac:B", "MW, Active Power"
      m_P_busac_C = "m:P:busac:C", "MW, Active Power"
      m_P_busdc = "m:P:busdc", "MW, Active Power"
      m_Psum_busac = "m:Psum:busac", "MW, Power-Phasor, Active Power"
      m_Psum_busdc = "m:Psum:busdc", "MW, Power-Phasor, Active Power"
      m_Qsum_busac = "m:Qsum:busac", "Mvar, Power Phasor, Reactive Part"
      m_Qsum_busdc = "m:Qsum:busdc", "Mvar, Power Phasor, Reactive Part"
      m_Ssum_busac = "m:Ssum:busac", "MVA, Power Phasor, Apparent Power"
      m_Ssum_busdc = "m:Ssum:busdc", "MVA, Power Phasor, Apparent Power"
      m_cosphi1_busac = "m:cosphi1:busac", "Power-Phasor, Power-Factor"
      m_cosphi1_busdc = "m:cosphi1:busdc", "Power-Phasor, Power-Factor"
      m_i1P_busac = "m:i1P:busac", "p.u., Current Phasor, Active Current"
      m_i1P_busdc = "m:i1P:busdc", "p.u., Current Phasor, Active Current"
      m_i1Q_busac = "m:i1Q:busac", "p.u., Current Phasor, Reactive Current"
      m_i1Q_busdc = "m:i1Q:busdc", "p.u., Current Phasor, Reactive Current"
      m_I1P_busac = "m:I1P:busac", "kA, Current Phasor, Active Current"
      m_I1P_busdc = "m:I1P:busdc", "kA, Current Phasor, Active Current"
      m_I1Q_busac = "m:I1Q:busac", "kA, Current Phasor, Reactive Current"
      m_I1Q_busdc = "m:I1Q:busdc", "kA, Current Phasor, Reactive Current"
      m_u_busac_A = "m:u:busac:A", "p.u., Phase Voltage"
      m_u_busac_B = "m:u:busac:B", "p.u., Phase Voltage"
      m_u_busac_C = "m:u:busac:C", "p.u., Phase Voltage"
      m_u_busdc = "m:u:busdc", "p.u., Phase Voltage"
      m_i_busac_A = "m:i:busac:A", "p.u., Phase Current"
      m_i_busac_B = "m:i:busac:B", "p.u., Phase Current"
      m_i_busac_C = "m:i:busac:C", "p.u., Phase Current"
      m_i_busdc = "m:i:busdc", "p.u., Phase Current"
      m_phiu1i1_busac = "m:phiu1i1:busac", "deg, Angle between Voltage and Current Phasor"
      m_phiu1i1_busdc = "m:phiu1i1:busdc", "deg, Angle between Voltage and Current Phasor"
      m_Tfct_busac = "m:Tfct:busac", "s, Fault Clearing Time"
      m_Tfct_busdc = "m:Tfct:busdc", "s, Fault Clearing Time"
      m_TfctPh_busac_A = "m:TfctPh:busac:A", "s, Fault Clearing Time"
      m_TfctPh_busac_B = "m:TfctPh:busac:B", "s, Fault Clearing Time"
      m_TfctPh_busac_C = "m:TfctPh:busac:C", "s, Fault Clearing Time"
      m_TfctPh_busdc = "m:TfctPh:busdc", "s, Fault Clearing Time"
      n_u_busac_A = "n:u:busac:A", "p.u., Phase Voltage"
      n_u_busac_B = "n:u:busac:B", "p.u., Phase Voltage"
      n_u_busac_C = "n:u:busac:C", "p.u., Phase Voltage"
      n_u_busdc = "n:u:busdc", "p.u., Phase Voltage"
      n_upc_busac_A = "n:upc:busac:A", "%, Phase Voltage"
      n_upc_busac_B = "n:upc:busac:B", "%, Phase Voltage"
      n_upc_busac_C = "n:upc:busac:C", "%, Phase Voltage"
      n_upc_busdc = "n:upc:busdc", "%, Phase Voltage"
      n_U_busac_A = "n:U:busac:A", "kV, Phase Voltage"
      n_U_busac_B = "n:U:busac:B", "kV, Phase Voltage"
      n_U_busac_C = "n:U:busac:C", "kV, Phase Voltage"
      n_U_busdc = "n:U:busdc", "kV, Phase Voltage"
      n_ul_busac_A = "n:ul:busac:A", "p.u., Line-Line Voltage"
      n_ul_busac_B = "n:ul:busac:B", "p.u., Line-Line Voltage"
      n_ul_busac_C = "n:ul:busac:C", "p.u., Line-Line Voltage"
      n_ul_busdc = "n:ul:busdc", "p.u., Line-Line Voltage"
      n_ulpc_busac_A = "n:ulpc:busac:A", "%, Line-Line Voltage"
      n_ulpc_busac_B = "n:ulpc:busac:B", "%, Line-Line Voltage"
      n_ulpc_busac_C = "n:ulpc:busac:C", "%, Line-Line Voltage"
      n_ulpc_busdc = "n:ulpc:busdc", "%, Line-Line Voltage"
      n_Ul_busac_A = "n:Ul:busac:A", "kV, Line-Line Voltage"
      n_Ul_busac_B = "n:Ul:busac:B", "kV, Line-Line Voltage"
      n_Ul_busac_C = "n:Ul:busac:C", "kV, Line-Line Voltage"
      n_Ul_busdc = "n:Ul:busdc", "kV, Line-Line Voltage"
      n_u0_busac = "n:u0:busac", "p.u., Zero-Sequence-Voltage"
      n_u0_busdc = "n:u0:busdc", "p.u., Zero-Sequence-Voltage"
      n_U0_busac = "n:U0:busac", "kV, Zero-Sequence-Voltage"
      n_U0_busdc = "n:U0:busdc", "kV, Zero-Sequence-Voltage"
      n_U0x3_busac = "n:U0x3:busac", "kV, 3*U0"
      n_U0x3_busdc = "n:U0x3:busdc", "kV, 3*U0"
      n_u1r_busac = "n:u1r:busac", "p.u., Voltage Phasor, Real Part"
      n_u1r_busdc = "n:u1r:busdc", "p.u., Voltage Phasor, Real Part"
      n_u1i_busac = "n:u1i:busac", "p.u., Voltage Phasor, Imaginary Part"
      n_u1i_busdc = "n:u1i:busdc", "p.u., Voltage Phasor, Imaginary Part"
      n_u1_busac = "n:u1:busac", "p.u., Voltage Phasor, Magnitude"
      n_u1_busdc = "n:u1:busdc", "p.u., Voltage Phasor, Magnitude"
      n_u1pc_busac = "n:u1pc:busac", "%, Voltage Phasor, Magnitude"
      n_u1pc_busdc = "n:u1pc:busdc", "%, Voltage Phasor, Magnitude"
      n_U1_busac = "n:U1:busac", "kV, Voltage Phasor, Magnitude"
      n_U1_busdc = "n:U1:busdc", "kV, Voltage Phasor, Magnitude"
      n_phiu1_busac = "n:phiu1:busac", "deg, Voltage Phasor, Angle"
      n_phiu1_busdc = "n:phiu1:busdc", "deg, Voltage Phasor, Angle"
      n_U1l_busac = "n:U1l:busac", "kV, Line-Line Voltage Phasor, Magnitude"
      n_U1l_busdc = "n:U1l:busdc", "kV, Line-Line Voltage Phasor, Magnitude"
      n_uln_busac_A = "n:uln:busac:A", "p.u., Line-Neutral Voltage"
      n_uln_busac_B = "n:uln:busac:B", "p.u., Line-Neutral Voltage"
      n_uln_busac_C = "n:uln:busac:C", "p.u., Line-Neutral Voltage"
      n_uln_busdc = "n:uln:busdc", "p.u., Line-Neutral Voltage"
      n_Uln_busac_A = "n:Uln:busac:A", "kV, Line-Neutral Voltage"
      n_Uln_busac_B = "n:Uln:busac:B", "kV, Line-Neutral Voltage"
      n_Uln_busac_C = "n:Uln:busac:C", "kV, Line-Neutral Voltage"
      n_Uln_busdc = "n:Uln:busdc", "kV, Line-Neutral Voltage"
      n_un_busac = "n:un:busac", "p.u., Neutral-Ground Voltage"
      n_un_busdc = "n:un:busdc", "p.u., Neutral-Ground Voltage"
      n_Un_busac = "n:Un:busac", "kV, Neutral-Ground Voltage"
      n_Un_busdc = "n:Un:busdc", "kV, Neutral-Ground Voltage"
      n_ishc_busac_A = "n:ishc:busac:A", "p.u., Short-Circuit Current"
      n_ishc_busac_B = "n:ishc:busac:B", "p.u., Short-Circuit Current"
      n_ishc_busac_C = "n:ishc:busac:C", "p.u., Short-Circuit Current"
      n_ishc_busdc = "n:ishc:busdc", "p.u., Short-Circuit Current"
      n_Ishc_busac_A = "n:Ishc:busac:A", "kA, Short-Circuit Current"
      n_Ishc_busac_B = "n:Ishc:busac:B", "kA, Short-Circuit Current"
      n_Ishc_busac_C = "n:Ishc:busac:C", "kA, Short-Circuit Current"
      n_Ishc_busdc = "n:Ishc:busdc", "kA, Short-Circuit Current"
      n_inshc_busac = "n:inshc:busac", "p.u., Neutral Short-Circuit Current, magnitude"
      n_inshc_busdc = "n:inshc:busdc", "p.u., Neutral Short-Circuit Current, magnitude"
      n_Inshc_busac = "n:Inshc:busac", "kA, Neutral Short-Circuit Current, magnitude"
      n_Inshc_busdc = "n:Inshc:busdc", "kA, Neutral Short-Circuit Current, magnitude"
      n_fe_busac = "n:fe:busac", "p.u., Electrical Frequency"
      n_fe_busdc = "n:fe:busdc", "p.u., Electrical Frequency"
      n_frdev_busac = "n:frdev:busac", "Hz, average frequency"
      n_frdev_busdc = "n:frdev:busdc", "Hz, average frequency"
      n_dfehz_busac = "n:dfehz:busac", "Hz, Deviation of the El. Frequency"
      n_dfehz_busdc = "n:dfehz:busdc", "Hz, Deviation of the El. Frequency"
      n_fehz_busac = "n:fehz:busac", "Hz, Electrical Frequency"
      n_fehz_busdc = "n:fehz:busdc", "Hz, Electrical Frequency"
      n_frnom_busac = "n:frnom:busac", "Hz, Nominal Frequency"
      n_frnom_busdc = "n:frnom:busdc", "Hz, Nominal Frequency"
      s_Pm_in = "s:Pm_in", "p.u., IN, Pulse-modulation factor (input) continuous, double"
      s_f0 = "s:f0", "p.u., IN, Frequency-input continuous, double"
      s_F0Hz = "s:F0Hz", "Hz, IN, Frequency-input continuous, double"
      s_dphiu = "s:dphiu", "rad, IN, Voltage-angle (input), continuous, double"
      s_u_a = "s:u_a", "p.u., IN, Firing signal, phase A, continuous, double"
      s_u_b = "s:u_b", "p.u., IN, Firing signal, phase B, continuous, double"
      s_u_c = "s:u_c", "p.u., IN, Firing signal, phase C, continuous, double"
      s_Pmr = "s:Pmr", "IN, Pulse-modulation factor (real part), continuous, double"
      s_Pmi = "s:Pmi", "IN, Pulse-modulation factor (imaginary part), continuous, double"
      s_cosref = "s:cosref", "IN, Cos(dq-reference-angle) continuous, double"
      s_sinref = "s:sinref", "IN, Sin(dq-reference-angle) continuous, double"
      s_Pmd = "s:Pmd", "IN, Pulse-modulation factor (d-axis), continuous, double"
      s_Pmq = "s:Pmq", "IN, Pulse-modulation factor (q-axis), continuous, double"
      s_id_ref = "s:id_ref", "p.u., IN, d-axis current reference, continuous, double"
      s_iq_ref = "s:iq_ref", "p.u., IN, q-axis current reference, continuous, double"
      s_i2d_ref = "s:i2d_ref", "p.u., IN, Negative sequence current, d-axis reference, continuous, double"
      s_i2q_ref = "s:i2q_ref", "p.u., IN, Negative sequence current, q-axis reference, continuous, double"
      s_block = "s:block", "p.u., IN, Blocking, continuous, double"
      s_urefA = "s:urefA", "p.u., IN, Reference voltage, phase A, continuous, double"
      s_urefB = "s:urefB", "p.u., IN, Reference voltage, phase B, continuous, double"
      s_urefC = "s:urefC", "p.u., IN, Reference voltage, phase C, continuous, double"
      s_Pm0 = "s:Pm0", "p.u., IN, Pulse-modulation factor (zero sequence) continuous, double"
      s_g1 = "s:g1", "IN, Firing signal, T1, continuous, double"
      s_g2 = "s:g2", "IN, Firing signal, T2, continuous, double"
      s_g3 = "s:g3", "IN, Firing signal, T3, continuous, double"
      s_g4 = "s:g4", "IN, Firing signal, T4, continuous, double"
      s_g5 = "s:g5", "IN, Firing signal, T5, continuous, double"
      s_g6 = "s:g6", "IN, Firing signal, T6, continuous, double"
      s_iDC = "s:iDC", "p.u., OUT, DC-current, continuous, double"
      s_uDC = "s:uDC", "p.u., OUT, DC-voltage, continuous, double"
      s_iAC_r = "s:iAC_r", "p.u., OUT, AC-current, real part, continuous, double"
      s_iAC_i = "s:iAC_i", "p.u., OUT, AC-current, imaginary part, continuous, double"
      s_uAC_r = "s:uAC_r", "p.u., OUT, AC-voltage, real part, continuous, double"
      s_uAC_i = "s:uAC_i", "p.u., OUT, AC-voltage, imaginary part, continuous, double"
      s_xd = "s:xd", "p.u., OUT, State variable, current controller d-axis, continuous, double"
      s_xq = "s:xq", "p.u., OUT, State variable, current controller q-axis, continuous, double"
      s_Upc_A = "s:Upc_A", "kV, OUT, Capacitive voltage, TpA continuous, double"
      s_Upc_B = "s:Upc_B", "kV, OUT, Capacitive voltage, TpB continuous, double"
      s_Upc_C = "s:Upc_C", "kV, OUT, Capacitive voltage, TpC continuous, double"
      s_Umc_A = "s:Umc_A", "kV, OUT, Capacitive voltage, TmA continuous, double"
      s_Umc_B = "s:Umc_B", "kV, OUT, Capacitive voltage, TmB continuous, double"
      s_Umc_C = "s:Umc_C", "kV, OUT, Capacitive voltage, TmC continuous, double"
      s_id = "s:id", "p.u., OUT, Current, d-axis continuous, double"
      s_iq = "s:iq", "p.u., OUT, Current, q-axis continuous, double"
      s_Ip_A = "s:Ip_A", "kA, OUT, T1 current, continuous, double"
      s_Ip_B = "s:Ip_B", "kA, OUT, T2 current, continuous, double"
      s_Ip_C = "s:Ip_C", "kA, OUT, T3 current, continuous, double"
      s_Im_A = "s:Im_A", "kA, OUT, T4 current, continuous, double"
      s_Im_B = "s:Im_B", "kA, OUT, T5 current, continuous, double"
      s_Im_C = "s:Im_C", "kA, OUT, T6 current, continuous, double"
      s_Idp_A = "s:Idp_A", "kA, OUT, D1 current, continuous, double"
      s_Idp_B = "s:Idp_B", "kA, OUT, D2 current, continuous, double"
      s_Idp_C = "s:Idp_C", "kA, OUT, D3 current, continuous, double"
      s_Idm_A = "s:Idm_A", "kA, OUT, D4 current, continuous, double"
      s_Idm_B = "s:Idm_B", "kA, OUT, D5 current, continuous, double"
      s_Idm_C = "s:Idm_C", "kA, OUT, D6 current, continuous, double"
      s_xd_dt = "s:xd:dt", "p.u./s, OUT, State variable, current controller d-axis, derivative, continuous, double"
      s_xq_dt = "s:xq:dt", "p.u./s, OUT, State variable, current controller q-axis, derivative, continuous, double"
      s_Upc_A_dt = "s:Upc_A:dt", "kV/s, OUT, Capacitive voltage, TpA, derivative, continuous, double"
      s_Upc_B_dt = "s:Upc_B:dt", "kV/s, OUT, Capacitive voltage, TpB, derivative, continuous, double"
      s_Upc_C_dt = "s:Upc_C:dt", "kV/s, OUT, Capacitive voltage, TpC, derivative, continuous, double"
      s_Umc_A_dt = "s:Umc_A:dt", "kV/s, OUT, Capacitive voltage, TmA, derivative, continuous, double"
      s_Umc_B_dt = "s:Umc_B:dt", "kV/s, OUT, Capacitive voltage, TmB, derivative, continuous, double"
      s_Umc_C_dt = "s:Umc_C:dt", "kV/s, OUT, Capacitive voltage, TmC, derivative, continuous, double"
      s_phiu = "s:phiu", "rad, STATE, Voltage-angle, continuous, double"
      s_phiu_dt = "s:phiu:dt", "rad/s, d/dt, Voltage-angle, derivative, continuous, double"
      c_hpi = "c:hpi", "rad/s, Nominal-angular-frequency"
      c_Pm = "c:Pm", "Pulse-modulation factor"
      c_snomn = "c:snomn", "MVA, Total rated apparent power (nparnum*Snom)"
      c_fmod = "c:fmod", "Hz, Modulation-frequency"
      c_i_mod = "c:i_mod", "Modulation"
      c_K0 = "c:K0", "Vac/Vdc at PWM=1"
      c_Losses = "c:Losses", "kW, Total converter losses"
      c_Pmd_ctrl = "c:Pmd_ctrl", "Pulse-modulation factor (d-axis)"
      c_Pmq_ctrl = "c:Pmq_ctrl", "Pulse-modulation factor (q-axis)"
      c_i_acdc = "c:i_acdc", "Control mode"
      e_pSubModel = "e:pSubModel", "ElmSubmodel, Submodel"
      e_iVIsource = "e:iVIsource", "Independent AC and DC voltage/current source model"
      e_Rac = "e:Rac", "Ohm, AC resistance"
      e_Lac = "e:Lac", "mH, AC inductance"
      e_R0toR1ac = "e:R0toR1ac", "R0/R1 ratio"
      e_X0toX1ac = "e:X0toX1ac", "X0/X1 ratio"
      e_Rdc = "e:Rdc", "Ohm, DC resistance"
      e_Ldc = "e:Ldc", "mH, DC inductance"
      e_i_det = "e:i_det", "Model"
      e_mmcModel = "e:mmcModel", "MMC model"
      e_Pnold = "e:Pnold", "kW, No-load losses"
      e_swtLossFactor = "e:swtLossFactor", "kW/A, Switching loss factor"
      e_resLossFactor = "e:resLossFactor", "Ohm, Resistive loss factor"
      e_fmod = "e:fmod", "Hz, Modulation frequency"
      e_Tdeadtime = "e:Tdeadtime", "us, Dead time"
      e_Ron = "e:Ron", "Ohm, On-resistance"
      e_Goff = "e:Goff", "uS, Off-conductance"
      e_Cvalve = "e:Cvalve", "uF, Snubber capacitance"
      e_Gvalve = "e:Gvalve", "S, Snubber conductance"
      e_NmmcSM = "e:NmmcSM", "Number of submodules per arm"
      e_mmcCmod = "e:mmcCmod", "uF, Submodule capacitance"
      e_mmcGcap = "e:mmcGcap", "uS, Capacitor parallel conductance"
      e_mmcModType = "e:mmcModType", "Modulation"
      e_fmodMmc = "e:fmodMmc", "Hz, Modulation frequency"
      e_Rdon = "e:Rdon", "Ohm, On-resistance"
      e_Gdoff = "e:Gdoff", "uS, Off-conductance"
      e_i_ctrl = "e:i_ctrl", "Use integrated current controller"
      e_Kd = "e:Kd", "Kd: d-axis, proportional gain"
      e_Td = "e:Td", "s, Td: d-axis, integration time constant"
      e_Kq = "e:Kq", "Kq: q-axis, proportional gain"
      e_Tq = "e:Tq", "s, Tq: q-axis, integration time constant"
      e_iArmFilt = "e:iArmFilt", "2nd-Harmonic Filter"
      e_cLarm1 = "e:cLarm1", "mH, Inductance, Larm1"
      e_Lfilt = "e:Lfilt", "mH, Inductance, Lfilt"
      e_Cfilt = "e:Cfilt", "uF, Capacitance, Cfilt"
      r_pSubModel = "r:pSubModel", "ElmSubmodel, Submodel"

    class ElmXnet(Enum):
      _init_ = 'value __doc__'

      m_u0_bus1 = "m:u0:bus1", "p.u., Zero-Sequence-Voltage"
      m_u1r_bus1 = "m:u1r:bus1", "p.u., Voltage Phasor, Real Part"
      m_u1i_bus1 = "m:u1i:bus1", "p.u., Voltage Phasor, Imaginary Part"
      m_u1_bus1 = "m:u1:bus1", "p.u., Voltage Phasor, Magnitude"
      m_U1_bus1 = "m:U1:bus1", "kV, Line-Ground Voltage Phasor, Magnitude"
      m_U1l_bus1 = "m:U1l:bus1", "kV, Line-Line Voltage Phasor, Magnitude"
      m_phiu1_bus1 = "m:phiu1:bus1", "deg, Voltage Phasor, Angle"
      m_I_bus1_A = "m:I:bus1:A", "kA, Phase Current"
      m_I_bus1_B = "m:I:bus1:B", "kA, Phase Current"
      m_I_bus1_C = "m:I:bus1:C", "kA, Phase Current"
      m_i0_bus1 = "m:i0:bus1", "p.u., Zero-Sequence Current"
      m_I0_bus1 = "m:I0:bus1", "kA, Zero-Sequence Current"
      m_I0x3_bus1 = "m:I0x3:bus1", "kA, 3*I0"
      m_i1r_bus1 = "m:i1r:bus1", "p.u., Current Phasor, Real Part"
      m_i1i_bus1 = "m:i1i:bus1", "p.u., Current Phasor, Imaginary Part"
      m_i1_bus1 = "m:i1:bus1", "p.u., Current Phasor, Magnitude"
      m_I1_bus1 = "m:I1:bus1", "kA, Current Phasor, Magnitude"
      m_phii1_bus1 = "m:phii1:bus1", "deg, Current Phasor, Angle"
      m_P_bus1_A = "m:P:bus1:A", "MW, Active Power"
      m_P_bus1_B = "m:P:bus1:B", "MW, Active Power"
      m_P_bus1_C = "m:P:bus1:C", "MW, Active Power"
      m_Psum_bus1 = "m:Psum:bus1", "MW, Power-Phasor, Active Power"
      m_Qsum_bus1 = "m:Qsum:bus1", "Mvar, Power Phasor, Reactive Part"
      m_Ssum_bus1 = "m:Ssum:bus1", "MVA, Power Phasor, Apparent Power"
      m_cosphi1_bus1 = "m:cosphi1:bus1", "Power-Phasor, Power-Factor"
      m_i1P_bus1 = "m:i1P:bus1", "p.u., Current Phasor, Active Current"
      m_i1Q_bus1 = "m:i1Q:bus1", "p.u., Current Phasor, Reactive Current"
      m_I1P_bus1 = "m:I1P:bus1", "kA, Current Phasor, Active Current"
      m_I1Q_bus1 = "m:I1Q:bus1", "kA, Current Phasor, Reactive Current"
      m_u_bus1_A = "m:u:bus1:A", "p.u., Phase Voltage"
      m_u_bus1_B = "m:u:bus1:B", "p.u., Phase Voltage"
      m_u_bus1_C = "m:u:bus1:C", "p.u., Phase Voltage"
      m_i_bus1_A = "m:i:bus1:A", "p.u., Phase Current"
      m_i_bus1_B = "m:i:bus1:B", "p.u., Phase Current"
      m_i_bus1_C = "m:i:bus1:C", "p.u., Phase Current"
      m_phiu1i1_bus1 = "m:phiu1i1:bus1", "deg, Angle between Voltage and Current Phasor"
      m_Tfct_bus1 = "m:Tfct:bus1", "s, Fault Clearing Time"
      m_TfctPh_bus1_A = "m:TfctPh:bus1:A", "s, Fault Clearing Time"
      m_TfctPh_bus1_B = "m:TfctPh:bus1:B", "s, Fault Clearing Time"
      m_TfctPh_bus1_C = "m:TfctPh:bus1:C", "s, Fault Clearing Time"
      n_u_bus1_A = "n:u:bus1:A", "p.u., Phase Voltage"
      n_u_bus1_B = "n:u:bus1:B", "p.u., Phase Voltage"
      n_u_bus1_C = "n:u:bus1:C", "p.u., Phase Voltage"
      n_upc_bus1_A = "n:upc:bus1:A", "%, Phase Voltage"
      n_upc_bus1_B = "n:upc:bus1:B", "%, Phase Voltage"
      n_upc_bus1_C = "n:upc:bus1:C", "%, Phase Voltage"
      n_U_bus1_A = "n:U:bus1:A", "kV, Phase Voltage"
      n_U_bus1_B = "n:U:bus1:B", "kV, Phase Voltage"
      n_U_bus1_C = "n:U:bus1:C", "kV, Phase Voltage"
      n_ul_bus1_A = "n:ul:bus1:A", "p.u., Line-Line Voltage"
      n_ul_bus1_B = "n:ul:bus1:B", "p.u., Line-Line Voltage"
      n_ul_bus1_C = "n:ul:bus1:C", "p.u., Line-Line Voltage"
      n_ulpc_bus1_A = "n:ulpc:bus1:A", "%, Line-Line Voltage"
      n_ulpc_bus1_B = "n:ulpc:bus1:B", "%, Line-Line Voltage"
      n_ulpc_bus1_C = "n:ulpc:bus1:C", "%, Line-Line Voltage"
      n_Ul_bus1_A = "n:Ul:bus1:A", "kV, Line-Line Voltage"
      n_Ul_bus1_B = "n:Ul:bus1:B", "kV, Line-Line Voltage"
      n_Ul_bus1_C = "n:Ul:bus1:C", "kV, Line-Line Voltage"
      n_u0_bus1 = "n:u0:bus1", "p.u., Zero-Sequence-Voltage"
      n_U0_bus1 = "n:U0:bus1", "kV, Zero-Sequence-Voltage"
      n_U0x3_bus1 = "n:U0x3:bus1", "kV, 3*U0"
      n_u1r_bus1 = "n:u1r:bus1", "p.u., Voltage Phasor, Real Part"
      n_u1i_bus1 = "n:u1i:bus1", "p.u., Voltage Phasor, Imaginary Part"
      n_u1_bus1 = "n:u1:bus1", "p.u., Voltage Phasor, Magnitude"
      n_u1pc_bus1 = "n:u1pc:bus1", "%, Voltage Phasor, Magnitude"
      n_U1_bus1 = "n:U1:bus1", "kV, Voltage Phasor, Magnitude"
      n_phiu1_bus1 = "n:phiu1:bus1", "deg, Voltage Phasor, Angle"
      n_U1l_bus1 = "n:U1l:bus1", "kV, Line-Line Voltage Phasor, Magnitude"
      n_uln_bus1_A = "n:uln:bus1:A", "p.u., Line-Neutral Voltage"
      n_uln_bus1_B = "n:uln:bus1:B", "p.u., Line-Neutral Voltage"
      n_uln_bus1_C = "n:uln:bus1:C", "p.u., Line-Neutral Voltage"
      n_Uln_bus1_A = "n:Uln:bus1:A", "kV, Line-Neutral Voltage"
      n_Uln_bus1_B = "n:Uln:bus1:B", "kV, Line-Neutral Voltage"
      n_Uln_bus1_C = "n:Uln:bus1:C", "kV, Line-Neutral Voltage"
      n_un_bus1 = "n:un:bus1", "p.u., Neutral-Ground Voltage"
      n_Un_bus1 = "n:Un:bus1", "kV, Neutral-Ground Voltage"
      n_ishc_bus1_A = "n:ishc:bus1:A", "p.u., Short-Circuit Current"
      n_ishc_bus1_B = "n:ishc:bus1:B", "p.u., Short-Circuit Current"
      n_ishc_bus1_C = "n:ishc:bus1:C", "p.u., Short-Circuit Current"
      n_Ishc_bus1_A = "n:Ishc:bus1:A", "kA, Short-Circuit Current"
      n_Ishc_bus1_B = "n:Ishc:bus1:B", "kA, Short-Circuit Current"
      n_Ishc_bus1_C = "n:Ishc:bus1:C", "kA, Short-Circuit Current"
      n_inshc_bus1 = "n:inshc:bus1", "p.u., Neutral Short-Circuit Current, magnitude"
      n_Inshc_bus1 = "n:Inshc:bus1", "kA, Neutral Short-Circuit Current, magnitude"
      n_fe_bus1 = "n:fe:bus1", "p.u., Electrical Frequency"
      n_frdev_bus1 = "n:frdev:bus1", "Hz, average frequency"
      n_dfehz_bus1 = "n:dfehz:bus1", "Hz, Deviation of the El. Frequency"
      n_fehz_bus1 = "n:fehz:bus1", "Hz, Electrical Frequency"
      n_frnom_bus1 = "n:frnom:bus1", "Hz, Nominal Frequency"
      s_pt = "s:pt", "p.u., IN, Turbine Power, continuous, double"
      s_xmdm = "s:xmdm", "p.u., IN, Mechanical Torque, continuous, double"
      s_usd = "s:usd", "p.u., OUT, Transient Voltage (d-axis), continuous, double"
      s_usq = "s:usq", "p.u., OUT, Transient Voltage (q-axis), continuous, double"
      s_uout = "s:uout", "p.u., OUT, Voltage continuous, double"
      s_xspeed = "s:xspeed", "p.u., OUT, Speed, continuous, double"
      s_psie = "s:psie", "p.u., STATE, Excitation-Flux continuous, double"
      s_psix = "s:psix", "p.u., STATE, Flux in x-winding, continuous, double"
      s_speed = "s:speed", "p.u., STATE, Speed, continuous, double"
      s_phi = "s:phi", "p.u., STATE, Rotor-angle, continuous, double"
      s_psiD = "s:psiD", "p.u., STATE, Flux in D-winding, continuous, double"
      s_psiQ = "s:psiQ", "p.u., STATE, Flux in Q-winding, continuous, double"
      s_psie_dt = "s:psie:dt", "p.u./s, d/dt, Excitation-Flux, derivative, continuous, double"
      s_psix_dt = "s:psix:dt", "p.u./s, d/dt, Flux in x-winding, derivative, continuous, double"
      s_speed_dt = "s:speed:dt", "p.u./s, d/dt, Speed, derivative, continuous, double"
      s_phi_dt = "s:phi:dt", "p.u./s, d/dt, Rotor-angle, derivative continuous, double"
      s_psiD_dt = "s:psiD:dt", "p.u./s, d/dt, Flux in D-winding, derivative, continuous, double"
      s_psiQ_dt = "s:psiQ:dt", "p.u./s, d/dt, Flux in Q-winding, derivative, continuous, double"
      c_sgnn = "c:sgnn", "MVA, Total rated apparent power (ngnum*sgn)"
      c_c2 = "c:c2", "Voltage Coefficient (uknom/ugn)"
      c_r0 = "c:r0", "p.u., Zero Sequence Resistance"
      c_x0 = "c:x0", "p.u., Zero Sequence Reactance"
      c_rearth = "c:rearth", "p.u., Grounding Resistance"
      c_xearth = "c:xearth", "p.u., Grounding Reactance"
      c_Te = "c:Te", "s, Excitation-Time constant"
      c_TD = "c:TD", "s, Time constant of D-winding"
      c_Tx = "c:Tx", "s, Time Constant of x-winding"
      c_TQ = "c:TQ", "s, Time Constant of Q-winding"
      c_frnom = "c:frnom", "Hz, Nominal frequency"
      c_Tag = "c:Tag", "s, Acceleration Time"
      c_xmt = "c:xmt", "p.u., Mechanical Torque"
      c_xme = "c:xme", "p.u., Electrical Torque"
      c_igen = "c:igen", "Index"
      c_Yd = "c:Yd", "p.u., Inverted reactance matrix"
      c_Yq = "c:Yq", "p.u., Inverted reactance matrix"
      c_id = "c:id", "p.u., Direct Axis current"
      c_iq = "c:iq", "p.u., Quadrature Axis current"
      c_hpi = "c:hpi", "rad/s, Nominal angular frequency"
      c_ie = "c:ie", "Excitation Current"
      c_addmt = "c:addmt", "Additional Torque"
      c_pgt = "c:pgt", "p.u., Active power"
      c_fe = "c:fe", "El. frequency"
      c_psid = "c:psid", "p.u., Flux in d-axis"
      c_psiq = "c:psiq", "p.u., Flux in q-axis"
      c_uss_r = "c:uss:r", "p.u., Subtransient Voltage, Real Part"
      c_uss_i = "c:uss:i", "p.u., Subtransient Voltage, Imaginary Part"
      c_uss_phi = "c:uss:phi", "deg, Subtransient Voltage, Angle"
      c_uss_mag = "c:uss:mag", "p.u., Subtransient Voltage, Magnitude"
      c_uss_phirad = "c:uss:phirad", "rad, Subtransient Voltage, Angle"
      c_xss = "c:xss", "p.u., Subtransient Reactance"
      c_xdss = "c:xdss", "p.u., Subtransient reactance (d-axis)"
      c_xqss = "c:xqss", "p.u., Subtransient reactance (q-axis)"
      c_Ptspin = "c:Ptspin", "MW, Total Turbine Power of Area"
      c_Pnomspin = "c:Pnomspin", "MW, Total Nominal Power of Spinning Machines in Area"
      c_Presspin = "c:Presspin", "MW, Spinning Reserve of Area"
      c_Tagav = "c:Tagav", "s, System Inertia of Area"
      c_droop = "c:droop", "System droop"
      c_dp = "c:dp", "p.u., Output of internal Speed Controller"
      c_p_mag = "c:p_mag", "Participation, Magnitude"
      c_p_phi = "c:p_phi", "deg, Participation, Angle"
      c_p_mags = "c:p_mags", "Participation, Magnitude (signed)"
      c_lEVec_mag = "c:lEVec_mag", "Controllability, Magnitude"
      c_lEVec_phi = "c:lEVec_phi", "deg, Controllability, Angle"
      c_lEVec_mags = "c:lEVec_mags", "Controllability, Magnitude (signed)"
      c_rEVec_mag = "c:rEVec_mag", "Observability, Magnitude"
      c_rEVec_phi = "c:rEVec_phi", "deg, Observability, Angle"
      c_rEVec_mags = "c:rEVec_mags", "Observability, Magnitude (signed)"
      e_tag = "e:tag", "s, Acceleration Time Constant"
      e_K = "e:K", "MW/Hz, Secondary Frequency Bias"
      e_cused = "e:cused", "Use for calculation"
      e_snss = "e:snss", "MVA, Short-Circuit Power Sk''max"
      e_ikss = "e:ikss", "kA, Short-Circuit Current Ik''max"
      e_cmax = "e:cmax", "c-Factor (max.)"
      e_rntxn = "e:rntxn", "R/X Ratio (max.)"
      e_xntrn = "e:xntrn", "X/R Ratio (max.)"
      e_x0tx1 = "e:x0tx1", "X0/X1 max."
      e_r0tx0 = "e:r0tx0", "R0/X0 max."
      e_snssmin = "e:snssmin", "MVA, Short-Circuit Power Sk''min"
      e_ikssmin = "e:ikssmin", "kA, Short-Circuit Current Ik''min"
      e_cmin = "e:cmin", "c-Factor (min.)"
      e_rntxnmin = "e:rntxnmin", "R/X Ratio (min.)"
      e_xntrnmin = "e:xntrnmin", "X/R Ratio (min.)"
      e_x0tx1min = "e:x0tx1min", "X0/X1 min."
      e_r0tx0min = "e:r0tx0min", "R0/X0 min."
      e_tds = "e:tds", "s, Td'"
      e_tqs = "e:tqs", "s, Tq'"
      e_tdss = "e:tdss", "s, Td''"
      e_tqss = "e:tqss", "s, Tq''"
      e_cosn = "e:cosn", "Power Factor"
      e_snom = "e:snom", "MVA, Nominal Apparent Power"
      e_xd = "e:xd", "p.u., xd"
      e_xq = "e:xq", "p.u., xq"
      e_xds = "e:xds", "p.u., xd'"
      e_xqs = "e:xqs", "p.u., xq'"

    class ElmZone(Enum):
      _init_ = 'value __doc__'

      c_Pgen = "c:Pgen", "MW, Generators, Active Power"
      c_Qgen = "c:Qgen", "Mvar, Generators, Reactive Power"
      c_Sgen = "c:Sgen", "MVA, Generators, Apparent Power"
      c_cosgen = "c:cosgen", "Generators, Power Factor"
      c_Pmot = "c:Pmot", "MW, Motor Loads, Active Power"
      c_Qmot = "c:Qmot", "Mvar, Motor Loads, Reactive Power"
      c_Smot = "c:Smot", "MVA, Motor Loads, Apparent Power"
      c_cosmot = "c:cosmot", "Motor Loads, Power Factor"
      c_Pload = "c:Pload", "MW, Loads, Active Power"
      c_Qload = "c:Qload", "Mvar, Loads, Reactive Power"
      c_Sload = "c:Sload", "MVA, Loads, Apparent Power"
      c_cosload = "c:cosload", "Loads, Power Factor"
      c_Pcomp = "c:Pcomp", "MW, Compensation (Losses)"
      c_Qcomp = "c:Qcomp", "Mvar, Compensation, C"
      c_Qcompr = "c:Qcompr", "Mvar, Compensation, L"
      c_Pnet = "c:Pnet", "MW, External Networks, Active Power"
      c_Qnet = "c:Qnet", "Mvar, External Networks, Reactive Power"
      c_Snet = "c:Snet", "MVA, External Networks, Apparent Power"
      c_cosnet = "c:cosnet", "External Networks, Power Factor"
      c_Pinter = "c:Pinter", "MW, Interchange Flow, Active Power"
      c_Qinter = "c:Qinter", "Mvar, Interchange Flow, Reactive Power"
      c_Ploss = "c:Ploss", "MW, Losses, Active Power"
      c_Qloss = "c:Qloss", "Mvar, Losses, Reactive Power"
      c_Pt = "c:Pt", "MW, Turbine Power"
      c_Pres = "c:Pres", "MW, Available Reserve"

    class ElmZpu(Enum):
      _init_ = 'value __doc__'

      m_u0_bus1 = "m:u0:bus1", "p.u., Zero-Sequence-Voltage"
      m_u0_bus2 = "m:u0:bus2", "p.u., Zero-Sequence-Voltage"
      m_u1r_bus1 = "m:u1r:bus1", "p.u., Voltage Phasor, Real Part"
      m_u1r_bus2 = "m:u1r:bus2", "p.u., Voltage Phasor, Real Part"
      m_u1i_bus1 = "m:u1i:bus1", "p.u., Voltage Phasor, Imaginary Part"
      m_u1i_bus2 = "m:u1i:bus2", "p.u., Voltage Phasor, Imaginary Part"
      m_u1_bus1 = "m:u1:bus1", "p.u., Voltage Phasor, Magnitude"
      m_u1_bus2 = "m:u1:bus2", "p.u., Voltage Phasor, Magnitude"
      m_U1_bus1 = "m:U1:bus1", "kV, Line-Ground Voltage Phasor, Magnitude"
      m_U1_bus2 = "m:U1:bus2", "kV, Line-Ground Voltage Phasor, Magnitude"
      m_U1l_bus1 = "m:U1l:bus1", "kV, Line-Line Voltage Phasor, Magnitude"
      m_U1l_bus2 = "m:U1l:bus2", "kV, Line-Line Voltage Phasor, Magnitude"
      m_phiu1_bus1 = "m:phiu1:bus1", "deg, Voltage Phasor, Angle"
      m_phiu1_bus2 = "m:phiu1:bus2", "deg, Voltage Phasor, Angle"
      m_I_bus1_A = "m:I:bus1:A", "kA, Phase Current"
      m_I_bus1_B = "m:I:bus1:B", "kA, Phase Current"
      m_I_bus1_C = "m:I:bus1:C", "kA, Phase Current"
      m_I_bus2_A = "m:I:bus2:A", "kA, Phase Current"
      m_I_bus2_B = "m:I:bus2:B", "kA, Phase Current"
      m_I_bus2_C = "m:I:bus2:C", "kA, Phase Current"
      m_i0_bus1 = "m:i0:bus1", "p.u., Zero-Sequence Current"
      m_i0_bus2 = "m:i0:bus2", "p.u., Zero-Sequence Current"
      m_I0_bus1 = "m:I0:bus1", "kA, Zero-Sequence Current"
      m_I0_bus2 = "m:I0:bus2", "kA, Zero-Sequence Current"
      m_I0x3_bus1 = "m:I0x3:bus1", "kA, 3*I0"
      m_I0x3_bus2 = "m:I0x3:bus2", "kA, 3*I0"
      m_i1r_bus1 = "m:i1r:bus1", "p.u., Current Phasor, Real Part"
      m_i1r_bus2 = "m:i1r:bus2", "p.u., Current Phasor, Real Part"
      m_i1i_bus1 = "m:i1i:bus1", "p.u., Current Phasor, Imaginary Part"
      m_i1i_bus2 = "m:i1i:bus2", "p.u., Current Phasor, Imaginary Part"
      m_i1_bus1 = "m:i1:bus1", "p.u., Current Phasor, Magnitude"
      m_i1_bus2 = "m:i1:bus2", "p.u., Current Phasor, Magnitude"
      m_I1_bus1 = "m:I1:bus1", "kA, Current Phasor, Magnitude"
      m_I1_bus2 = "m:I1:bus2", "kA, Current Phasor, Magnitude"
      m_phii1_bus1 = "m:phii1:bus1", "deg, Current Phasor, Angle"
      m_phii1_bus2 = "m:phii1:bus2", "deg, Current Phasor, Angle"
      m_P_bus1_A = "m:P:bus1:A", "MW, Active Power"
      m_P_bus1_B = "m:P:bus1:B", "MW, Active Power"
      m_P_bus1_C = "m:P:bus1:C", "MW, Active Power"
      m_P_bus2_A = "m:P:bus2:A", "MW, Active Power"
      m_P_bus2_B = "m:P:bus2:B", "MW, Active Power"
      m_P_bus2_C = "m:P:bus2:C", "MW, Active Power"
      m_Psum_bus1 = "m:Psum:bus1", "MW, Power-Phasor, Active Power"
      m_Psum_bus2 = "m:Psum:bus2", "MW, Power-Phasor, Active Power"
      m_Qsum_bus1 = "m:Qsum:bus1", "Mvar, Power Phasor, Reactive Part"
      m_Qsum_bus2 = "m:Qsum:bus2", "Mvar, Power Phasor, Reactive Part"
      m_Ssum_bus1 = "m:Ssum:bus1", "MVA, Power Phasor, Apparent Power"
      m_Ssum_bus2 = "m:Ssum:bus2", "MVA, Power Phasor, Apparent Power"
      m_cosphi1_bus1 = "m:cosphi1:bus1", "Power-Phasor, Power-Factor"
      m_cosphi1_bus2 = "m:cosphi1:bus2", "Power-Phasor, Power-Factor"
      m_i1P_bus1 = "m:i1P:bus1", "p.u., Current Phasor, Active Current"
      m_i1P_bus2 = "m:i1P:bus2", "p.u., Current Phasor, Active Current"
      m_i1Q_bus1 = "m:i1Q:bus1", "p.u., Current Phasor, Reactive Current"
      m_i1Q_bus2 = "m:i1Q:bus2", "p.u., Current Phasor, Reactive Current"
      m_I1P_bus1 = "m:I1P:bus1", "kA, Current Phasor, Active Current"
      m_I1P_bus2 = "m:I1P:bus2", "kA, Current Phasor, Active Current"
      m_I1Q_bus1 = "m:I1Q:bus1", "kA, Current Phasor, Reactive Current"
      m_I1Q_bus2 = "m:I1Q:bus2", "kA, Current Phasor, Reactive Current"
      m_u_bus1_A = "m:u:bus1:A", "p.u., Phase Voltage"
      m_u_bus1_B = "m:u:bus1:B", "p.u., Phase Voltage"
      m_u_bus1_C = "m:u:bus1:C", "p.u., Phase Voltage"
      m_u_bus2_A = "m:u:bus2:A", "p.u., Phase Voltage"
      m_u_bus2_B = "m:u:bus2:B", "p.u., Phase Voltage"
      m_u_bus2_C = "m:u:bus2:C", "p.u., Phase Voltage"
      m_i_bus1_A = "m:i:bus1:A", "p.u., Phase Current"
      m_i_bus1_B = "m:i:bus1:B", "p.u., Phase Current"
      m_i_bus1_C = "m:i:bus1:C", "p.u., Phase Current"
      m_i_bus2_A = "m:i:bus2:A", "p.u., Phase Current"
      m_i_bus2_B = "m:i:bus2:B", "p.u., Phase Current"
      m_i_bus2_C = "m:i:bus2:C", "p.u., Phase Current"
      m_phiu1i1_bus1 = "m:phiu1i1:bus1", "deg, Angle between Voltage and Current Phasor"
      m_phiu1i1_bus2 = "m:phiu1i1:bus2", "deg, Angle between Voltage and Current Phasor"
      m_Tfct_bus1 = "m:Tfct:bus1", "s, Fault Clearing Time"
      m_Tfct_bus2 = "m:Tfct:bus2", "s, Fault Clearing Time"
      m_TfctPh_bus1_A = "m:TfctPh:bus1:A", "s, Fault Clearing Time"
      m_TfctPh_bus1_B = "m:TfctPh:bus1:B", "s, Fault Clearing Time"
      m_TfctPh_bus1_C = "m:TfctPh:bus1:C", "s, Fault Clearing Time"
      m_TfctPh_bus2_A = "m:TfctPh:bus2:A", "s, Fault Clearing Time"
      m_TfctPh_bus2_B = "m:TfctPh:bus2:B", "s, Fault Clearing Time"
      m_TfctPh_bus2_C = "m:TfctPh:bus2:C", "s, Fault Clearing Time"
      n_u_bus1_A = "n:u:bus1:A", "p.u., Phase Voltage"
      n_u_bus1_B = "n:u:bus1:B", "p.u., Phase Voltage"
      n_u_bus1_C = "n:u:bus1:C", "p.u., Phase Voltage"
      n_u_bus2_A = "n:u:bus2:A", "p.u., Phase Voltage"
      n_u_bus2_B = "n:u:bus2:B", "p.u., Phase Voltage"
      n_u_bus2_C = "n:u:bus2:C", "p.u., Phase Voltage"
      n_upc_bus1_A = "n:upc:bus1:A", "%, Phase Voltage"
      n_upc_bus1_B = "n:upc:bus1:B", "%, Phase Voltage"
      n_upc_bus1_C = "n:upc:bus1:C", "%, Phase Voltage"
      n_upc_bus2_A = "n:upc:bus2:A", "%, Phase Voltage"
      n_upc_bus2_B = "n:upc:bus2:B", "%, Phase Voltage"
      n_upc_bus2_C = "n:upc:bus2:C", "%, Phase Voltage"
      n_U_bus1_A = "n:U:bus1:A", "kV, Phase Voltage"
      n_U_bus1_B = "n:U:bus1:B", "kV, Phase Voltage"
      n_U_bus1_C = "n:U:bus1:C", "kV, Phase Voltage"
      n_U_bus2_A = "n:U:bus2:A", "kV, Phase Voltage"
      n_U_bus2_B = "n:U:bus2:B", "kV, Phase Voltage"
      n_U_bus2_C = "n:U:bus2:C", "kV, Phase Voltage"
      n_ul_bus1_A = "n:ul:bus1:A", "p.u., Line-Line Voltage"
      n_ul_bus1_B = "n:ul:bus1:B", "p.u., Line-Line Voltage"
      n_ul_bus1_C = "n:ul:bus1:C", "p.u., Line-Line Voltage"
      n_ul_bus2_A = "n:ul:bus2:A", "p.u., Line-Line Voltage"
      n_ul_bus2_B = "n:ul:bus2:B", "p.u., Line-Line Voltage"
      n_ul_bus2_C = "n:ul:bus2:C", "p.u., Line-Line Voltage"
      n_ulpc_bus1_A = "n:ulpc:bus1:A", "%, Line-Line Voltage"
      n_ulpc_bus1_B = "n:ulpc:bus1:B", "%, Line-Line Voltage"
      n_ulpc_bus1_C = "n:ulpc:bus1:C", "%, Line-Line Voltage"
      n_ulpc_bus2_A = "n:ulpc:bus2:A", "%, Line-Line Voltage"
      n_ulpc_bus2_B = "n:ulpc:bus2:B", "%, Line-Line Voltage"
      n_ulpc_bus2_C = "n:ulpc:bus2:C", "%, Line-Line Voltage"
      n_Ul_bus1_A = "n:Ul:bus1:A", "kV, Line-Line Voltage"
      n_Ul_bus1_B = "n:Ul:bus1:B", "kV, Line-Line Voltage"
      n_Ul_bus1_C = "n:Ul:bus1:C", "kV, Line-Line Voltage"
      n_Ul_bus2_A = "n:Ul:bus2:A", "kV, Line-Line Voltage"
      n_Ul_bus2_B = "n:Ul:bus2:B", "kV, Line-Line Voltage"
      n_Ul_bus2_C = "n:Ul:bus2:C", "kV, Line-Line Voltage"
      n_u0_bus1 = "n:u0:bus1", "p.u., Zero-Sequence-Voltage"
      n_u0_bus2 = "n:u0:bus2", "p.u., Zero-Sequence-Voltage"
      n_U0_bus1 = "n:U0:bus1", "kV, Zero-Sequence-Voltage"
      n_U0_bus2 = "n:U0:bus2", "kV, Zero-Sequence-Voltage"
      n_U0x3_bus1 = "n:U0x3:bus1", "kV, 3*U0"
      n_U0x3_bus2 = "n:U0x3:bus2", "kV, 3*U0"
      n_u1r_bus1 = "n:u1r:bus1", "p.u., Voltage Phasor, Real Part"
      n_u1r_bus2 = "n:u1r:bus2", "p.u., Voltage Phasor, Real Part"
      n_u1i_bus1 = "n:u1i:bus1", "p.u., Voltage Phasor, Imaginary Part"
      n_u1i_bus2 = "n:u1i:bus2", "p.u., Voltage Phasor, Imaginary Part"
      n_u1_bus1 = "n:u1:bus1", "p.u., Voltage Phasor, Magnitude"
      n_u1_bus2 = "n:u1:bus2", "p.u., Voltage Phasor, Magnitude"
      n_u1pc_bus1 = "n:u1pc:bus1", "%, Voltage Phasor, Magnitude"
      n_u1pc_bus2 = "n:u1pc:bus2", "%, Voltage Phasor, Magnitude"
      n_U1_bus1 = "n:U1:bus1", "kV, Voltage Phasor, Magnitude"
      n_U1_bus2 = "n:U1:bus2", "kV, Voltage Phasor, Magnitude"
      n_phiu1_bus1 = "n:phiu1:bus1", "deg, Voltage Phasor, Angle"
      n_phiu1_bus2 = "n:phiu1:bus2", "deg, Voltage Phasor, Angle"
      n_U1l_bus1 = "n:U1l:bus1", "kV, Line-Line Voltage Phasor, Magnitude"
      n_U1l_bus2 = "n:U1l:bus2", "kV, Line-Line Voltage Phasor, Magnitude"
      n_uln_bus1_A = "n:uln:bus1:A", "p.u., Line-Neutral Voltage"
      n_uln_bus1_B = "n:uln:bus1:B", "p.u., Line-Neutral Voltage"
      n_uln_bus1_C = "n:uln:bus1:C", "p.u., Line-Neutral Voltage"
      n_uln_bus2_A = "n:uln:bus2:A", "p.u., Line-Neutral Voltage"
      n_uln_bus2_B = "n:uln:bus2:B", "p.u., Line-Neutral Voltage"
      n_uln_bus2_C = "n:uln:bus2:C", "p.u., Line-Neutral Voltage"
      n_Uln_bus1_A = "n:Uln:bus1:A", "kV, Line-Neutral Voltage"
      n_Uln_bus1_B = "n:Uln:bus1:B", "kV, Line-Neutral Voltage"
      n_Uln_bus1_C = "n:Uln:bus1:C", "kV, Line-Neutral Voltage"
      n_Uln_bus2_A = "n:Uln:bus2:A", "kV, Line-Neutral Voltage"
      n_Uln_bus2_B = "n:Uln:bus2:B", "kV, Line-Neutral Voltage"
      n_Uln_bus2_C = "n:Uln:bus2:C", "kV, Line-Neutral Voltage"
      n_un_bus1 = "n:un:bus1", "p.u., Neutral-Ground Voltage"
      n_un_bus2 = "n:un:bus2", "p.u., Neutral-Ground Voltage"
      n_Un_bus1 = "n:Un:bus1", "kV, Neutral-Ground Voltage"
      n_Un_bus2 = "n:Un:bus2", "kV, Neutral-Ground Voltage"
      n_ishc_bus1_A = "n:ishc:bus1:A", "p.u., Short-Circuit Current"
      n_ishc_bus1_B = "n:ishc:bus1:B", "p.u., Short-Circuit Current"
      n_ishc_bus1_C = "n:ishc:bus1:C", "p.u., Short-Circuit Current"
      n_ishc_bus2_A = "n:ishc:bus2:A", "p.u., Short-Circuit Current"
      n_ishc_bus2_B = "n:ishc:bus2:B", "p.u., Short-Circuit Current"
      n_ishc_bus2_C = "n:ishc:bus2:C", "p.u., Short-Circuit Current"
      n_Ishc_bus1_A = "n:Ishc:bus1:A", "kA, Short-Circuit Current"
      n_Ishc_bus1_B = "n:Ishc:bus1:B", "kA, Short-Circuit Current"
      n_Ishc_bus1_C = "n:Ishc:bus1:C", "kA, Short-Circuit Current"
      n_Ishc_bus2_A = "n:Ishc:bus2:A", "kA, Short-Circuit Current"
      n_Ishc_bus2_B = "n:Ishc:bus2:B", "kA, Short-Circuit Current"
      n_Ishc_bus2_C = "n:Ishc:bus2:C", "kA, Short-Circuit Current"
      n_inshc_bus1 = "n:inshc:bus1", "p.u., Neutral Short-Circuit Current, magnitude"
      n_inshc_bus2 = "n:inshc:bus2", "p.u., Neutral Short-Circuit Current, magnitude"
      n_Inshc_bus1 = "n:Inshc:bus1", "kA, Neutral Short-Circuit Current, magnitude"
      n_Inshc_bus2 = "n:Inshc:bus2", "kA, Neutral Short-Circuit Current, magnitude"
      n_fe_bus1 = "n:fe:bus1", "p.u., Electrical Frequency"
      n_fe_bus2 = "n:fe:bus2", "p.u., Electrical Frequency"
      n_frdev_bus1 = "n:frdev:bus1", "Hz, average frequency"
      n_frdev_bus2 = "n:frdev:bus2", "Hz, average frequency"
      n_dfehz_bus1 = "n:dfehz:bus1", "Hz, Deviation of the El. Frequency"
      n_dfehz_bus2 = "n:dfehz:bus2", "Hz, Deviation of the El. Frequency"
      n_fehz_bus1 = "n:fehz:bus1", "Hz, Electrical Frequency"
      n_fehz_bus2 = "n:fehz:bus2", "Hz, Electrical Frequency"
      n_frnom_bus1 = "n:frnom:bus1", "Hz, Nominal Frequency"
      n_frnom_bus2 = "n:frnom:bus2", "Hz, Nominal Frequency"
      c_z1_ij_r = "c:z1_ij:r", "p.u., Positive-Sequence Impedance i-j, Real Part"
      c_z1_ij_i = "c:z1_ij:i", "p.u., Positive-Sequence Impedance i-j, Imaginary Part"
      c_z1_ij_phi = "c:z1_ij:phi", "deg, Positive-Sequence Impedance i-j, Angle"
      c_z1_ij_mag = "c:z1_ij:mag", "p.u., Positive-Sequence Impedance i-j, Magnitude"
      c_z1_ij_phirad = "c:z1_ij:phirad", "rad, Positive-Sequence Impedance i-j, Angle"
      c_z1_ji_r = "c:z1_ji:r", "p.u., Positive-Sequence Impedance j-i, Real Part"
      c_z1_ji_i = "c:z1_ji:i", "p.u., Positive-Sequence Impedance j-i, Imaginary Part"
      c_z1_ji_phi = "c:z1_ji:phi", "deg, Positive-Sequence Impedance j-i, Angle"
      c_z1_ji_mag = "c:z1_ji:mag", "p.u., Positive-Sequence Impedance j-i, Magnitude"
      c_z1_ji_phirad = "c:z1_ji:phirad", "rad, Positive-Sequence Impedance j-i, Angle"
      c_z0_ij_r = "c:z0_ij:r", "p.u., Zero-Sequence Impedance i-j, Real Part"
      c_z0_ij_i = "c:z0_ij:i", "p.u., Zero-Sequence Impedance i-j, Imaginary Part"
      c_z0_ij_phi = "c:z0_ij:phi", "deg, Zero-Sequence Impedance i-j, Angle"
      c_z0_ij_mag = "c:z0_ij:mag", "p.u., Zero-Sequence Impedance i-j, Magnitude"
      c_z0_ij_phirad = "c:z0_ij:phirad", "rad, Zero-Sequence Impedance i-j, Angle"
      c_z0_ji_r = "c:z0_ji:r", "p.u., Zero-Sequence Impedance j-i, Real Part"
      c_z0_ji_i = "c:z0_ji:i", "p.u., Zero-Sequence Impedance j-i, Imaginary Part"
      c_z0_ji_phi = "c:z0_ji:phi", "deg, Zero-Sequence Impedance j-i, Angle"
      c_z0_ji_mag = "c:z0_ji:mag", "p.u., Zero-Sequence Impedance j-i, Magnitude"
      c_z0_ji_phirad = "c:z0_ji:phirad", "rad, Zero-Sequence Impedance j-i, Angle"
      c_z2_ij_r = "c:z2_ij:r", "p.u., Negative-Sequence Impedance i-j, Real Part"
      c_z2_ij_i = "c:z2_ij:i", "p.u., Negative-Sequence Impedance i-j, Imaginary Part"
      c_z2_ij_phi = "c:z2_ij:phi", "deg, Negative-Sequence Impedance i-j, Angle"
      c_z2_ij_mag = "c:z2_ij:mag", "p.u., Negative-Sequence Impedance i-j, Magnitude"
      c_z2_ij_phirad = "c:z2_ij:phirad", "rad, Negative-Sequence Impedance i-j, Angle"
      c_z2_ji_r = "c:z2_ji:r", "p.u., Negative-Sequence Impedance j-i, Real Part"
      c_z2_ji_i = "c:z2_ji:i", "p.u., Negative-Sequence Impedance j-i, Imaginary Part"
      c_z2_ji_phi = "c:z2_ji:phi", "deg, Negative-Sequence Impedance j-i, Angle"
      c_z2_ji_mag = "c:z2_ji:mag", "p.u., Negative-Sequence Impedance j-i, Magnitude"
      c_z2_ji_phirad = "c:z2_ji:phirad", "rad, Negative-Sequence Impedance j-i, Angle"
      c_y1_ij_r = "c:y1_ij:r", "p.u., Positive-Sequence Admittance i-j, Real Part"
      c_y1_ij_i = "c:y1_ij:i", "p.u., Positive-Sequence Admittance i-j, Imaginary Part"
      c_y1_ij_phi = "c:y1_ij:phi", "deg, Positive-Sequence Admittance i-j, Angle"
      c_y1_ij_mag = "c:y1_ij:mag", "p.u., Positive-Sequence Admittance i-j, Magnitude"
      c_y1_ij_phirad = "c:y1_ij:phirad", "rad, Positive-Sequence Admittance i-j, Angle"
      c_y1_ji_r = "c:y1_ji:r", "p.u., Positive-Sequence Admittance j-i, Real Part"
      c_y1_ji_i = "c:y1_ji:i", "p.u., Positive-Sequence Admittance j-i, Imaginary Part"
      c_y1_ji_phi = "c:y1_ji:phi", "deg, Positive-Sequence Admittance j-i, Angle"
      c_y1_ji_mag = "c:y1_ji:mag", "p.u., Positive-Sequence Admittance j-i, Magnitude"
      c_y1_ji_phirad = "c:y1_ji:phirad", "rad, Positive-Sequence Admittance j-i, Angle"
      c_y0_ij_r = "c:y0_ij:r", "p.u., Zero-Sequence Admittance i-j, Real Part"
      c_y0_ij_i = "c:y0_ij:i", "p.u., Zero-Sequence Admittance i-j, Imaginary Part"
      c_y0_ij_phi = "c:y0_ij:phi", "deg, Zero-Sequence Admittance i-j, Angle"
      c_y0_ij_mag = "c:y0_ij:mag", "p.u., Zero-Sequence Admittance i-j, Magnitude"
      c_y0_ij_phirad = "c:y0_ij:phirad", "rad, Zero-Sequence Admittance i-j, Angle"
      c_y0_ji_r = "c:y0_ji:r", "p.u., Zero-Sequence Admittance j-i, Real Part"
      c_y0_ji_i = "c:y0_ji:i", "p.u., Zero-Sequence Admittance j-i, Imaginary Part"
      c_y0_ji_phi = "c:y0_ji:phi", "deg, Zero-Sequence Admittance j-i, Angle"
      c_y0_ji_mag = "c:y0_ji:mag", "p.u., Zero-Sequence Admittance j-i, Magnitude"
      c_y0_ji_phirad = "c:y0_ji:phirad", "rad, Zero-Sequence Admittance j-i, Angle"
      c_y2_ij_r = "c:y2_ij:r", "p.u., Negative-Sequence Admittance i-j, Real Part"
      c_y2_ij_i = "c:y2_ij:i", "p.u., Negative-Sequence Admittance i-j, Imaginary Part"
      c_y2_ij_phi = "c:y2_ij:phi", "deg, Negative-Sequence Admittance i-j, Angle"
      c_y2_ij_mag = "c:y2_ij:mag", "p.u., Negative-Sequence Admittance i-j, Magnitude"
      c_y2_ij_phirad = "c:y2_ij:phirad", "rad, Negative-Sequence Admittance i-j, Angle"
      c_y2_ji_r = "c:y2_ji:r", "p.u., Negative-Sequence Admittance j-i, Real Part"
      c_y2_ji_i = "c:y2_ji:i", "p.u., Negative-Sequence Admittance j-i, Imaginary Part"
      c_y2_ji_phi = "c:y2_ji:phi", "deg, Negative-Sequence Admittance j-i, Angle"
      c_y2_ji_mag = "c:y2_ji:mag", "p.u., Negative-Sequence Admittance j-i, Magnitude"
      c_y2_ji_phirad = "c:y2_ji:phirad", "rad, Negative-Sequence Admittance j-i, Angle"
      c_uratio = "c:uratio", "Rating Factor"
      e_gi_pu = "e:gi_pu", "p.u., Conductance i"
      e_bi_pu = "e:bi_pu", "p.u., Susceptance i"
      e_gj_pu = "e:gj_pu", "p.u., Conductance j"
      e_bj_pu = "e:bj_pu", "p.u., Susceptance j"
      e_gi0_pu = "e:gi0_pu", "p.u., Conductance i"
      e_bi0_pu = "e:bi0_pu", "p.u., Susceptance i"
      e_gj0_pu = "e:gj0_pu", "p.u., Conductance j"
      e_bj0_pu = "e:bj0_pu", "p.u., Susceptance j"
      e_iequalz = "e:iequalz", "Use equal Impedances (zij = zji)"
      e_iCapParallel = "e:iCapParallel", "Consider resistive and capacitive parts in parallel"
      e_r_pu = "e:r_pu", "p.u., Real Part"
      e_x_pu = "e:x_pu", "p.u., Imaginary Part"
      e_r_pu_ji = "e:r_pu_ji", "p.u., Real Part"
      e_x_pu_ji = "e:x_pu_ji", "p.u., Imaginary Part"
      e_r0_pu = "e:r0_pu", "p.u., Real Part"
      e_x0_pu = "e:x0_pu", "p.u., Imaginary Part"
      e_r0_pu_ji = "e:r0_pu_ji", "p.u., Real Part"
      e_x0_pu_ji = "e:x0_pu_ji", "p.u., Imaginary Part"


  class Sensitivities_Bal:

    class ElmArea(Enum):
      _init_ = 'value __doc__'

      c_n_ele = "c:n_ele", "Number of Elements"
      c_n_areas = "c:n_areas", "Number of Connected Areas"
      c_n_volt = "c:n_volt", "Number of Voltage Levels"
      c_n_bar = "c:n_bar", "No. of Busbars"
      c_n_term = "c:n_term", "No. of Terminals"
      c_n_lne = "c:n_lne", "No. of Lines"
      c_n_sym = "c:n_sym", "No. of syn. Machines"
      c_n_asm = "c:n_asm", "No. of asyn. Machines"
      c_n_stg = "c:n_stg", "No. of Static Generators"
      c_n_pvs = "c:n_pvs", "No. of PV Systems"
      c_n_lod = "c:n_lod", "No. of Loads"
      c_n_cust = "c:n_cust", "No. of Customers"
      c_TotgenP = "c:TotgenP", "MW, Total Generation, Active Power"
      c_TotgenQ = "c:TotgenQ", "Mvar, Total Generation, Reactive Power"
      c_TotconsP = "c:TotconsP", "MW, Total Consumption, Active Power"
      c_TotconsQ = "c:TotconsQ", "Mvar, Total Consumption, Reactive Power"
      c_GenP = "c:GenP", "MW, Generators, Active Power"
      c_GenQ = "c:GenQ", "Mvar, Generators, Reactive Power"
      c_GenS = "c:GenS", "MVA, Generators, Apparent Power"
      c_Gencos = "c:Gencos", "Generators, Power Factor"
      c_GenPnom = "c:GenPnom", "MW, Generators, Nominal Active Power"
      c_GenQnom = "c:GenQnom", "Mvar, Generators, Nominal Reactive Power"
      c_GenSnom = "c:GenSnom", "MVA, Generators, Nominal Apparent Power"
      c_GenPdiff = "c:GenPdiff", "MW, Generators, difference between nominal and actual active power"
      c_GenQdiff = "c:GenQdiff", "Mvar, Generators, difference between nominal and actual reactive power"
      c_Gencosnom = "c:Gencosnom", "Generators, Nominal Power Factor"
      c_GenPdisp = "c:GenPdisp", "MW, Generators, Dispatch Active Power"
      c_GenPmism = "c:GenPmism", "MW, Generators, difference between actual and dispatched active power"
      c_GenPspinres = "c:GenPspinres", "MW, Generators, difference between maximum and actual active power"
      c_GenQspinres = "c:GenQspinres", "Mvar, Generators, difference between maximum and actual reactive power"
      c_MotP = "c:MotP", "MW, Motor Loads, Active Power"
      c_MotQ = "c:MotQ", "Mvar, Motor Loads, Reactive Power"
      c_MotS = "c:MotS", "MVA, Motor Loads, Apparent Power"
      c_Motcos = "c:Motcos", "Motor Loads, Power Factor"
      c_LoadP = "c:LoadP", "MW, Loads, Active Power"
      c_LoadQ = "c:LoadQ", "Mvar, Loads, Reactive Power"
      c_LoadS = "c:LoadS", "MVA, Loads, Apparent Power"
      c_Loadcos = "c:Loadcos", "Loads, Power Factor"
      c_LoadPnom = "c:LoadPnom", "MW, Loads, Nominal Active Power"
      c_LoadQnom = "c:LoadQnom", "Mvar, Loads, Nominal Reactive Power"
      c_LoadSnom = "c:LoadSnom", "MVA, Loads, Nominal Apparent Power"
      c_Loadcosnom = "c:Loadcosnom", "Loads, Nominal Power Factor"
      c_LoadPdiff = "c:LoadPdiff", "MW, Loads, difference between nominal and actual active power"
      c_LoadQdiff = "c:LoadQdiff", "Mvar, Loads, difference between nominal and actual reactive power"
      c_LoadPdem = "c:LoadPdem", "MW, Loads, Active Power Demand"
      c_LoadPmism = "c:LoadPmism", "MW, Loads, difference between actual active power and demand"
      c_CompP = "c:CompP", "MW, Compensation (Losses)"
      c_CompQ = "c:CompQ", "Mvar, Compensation, C"
      c_ComprQ = "c:ComprQ", "Mvar, Compensation, L"
      c_NetP = "c:NetP", "MW, External Networks, Active Power"
      c_NetQ = "c:NetQ", "Mvar, External Networks, Reactive Power"
      c_NetS = "c:NetS", "MVA, External Networks, Apparent Power"
      c_Netcos = "c:Netcos", "External Networks, Power Factor"
      c_InterP = "c:InterP", "MW, Interchange Flow, Active Power"
      c_InterQ = "c:InterQ", "Mvar, Interchange Flow, Reactive Power"
      c_LossP = "c:LossP", "MW, Losses, Active Power"
      c_LossQ = "c:LossQ", "Mvar, Losses, Reactive Power"
      c_LossPld = "c:LossPld", "MW, Losses, Active Power (load)"
      c_LossQld = "c:LossQld", "Mvar, Losses, Reactive Power (load)"
      c_LossPnld = "c:LossPnld", "MW, Losses, Active Power (no load)"
      c_LossQnld = "c:LossQnld", "Mvar, Losses, Reactive Power (no load)"
      c_LossPlne = "c:LossPlne", "MW, Line Losses, Active Power"
      c_LossQlne = "c:LossQlne", "Mvar, Line Losses, Reactive Power"
      c_LossPldlne = "c:LossPldlne", "MW, Line Losses, Active Power (load)"
      c_LossQldlne = "c:LossQldlne", "Mvar, Line Losses, Reactive Power (load)"
      c_LossPnldlne = "c:LossPnldlne", "MW, Line Losses, Active Power (no load)"
      c_LossQnldlne = "c:LossQnldlne", "Mvar, Line Losses, Reactive Power (no load)"
      c_LossPtrf = "c:LossPtrf", "MW, Transformer Losses, Active Power"
      c_LossQtrf = "c:LossQtrf", "Mvar, Transformer Losses, Reactive Power"
      c_LossPldtrf = "c:LossPldtrf", "MW, Transformer Losses, Active Power (load)"
      c_LossQldtrf = "c:LossQldtrf", "Mvar, Transformer Losses, Reactive Power (load)"
      c_LossPnldtrf = "c:LossPnldtrf", "MW, Transformer Losses, Active Power (no load)"
      c_LossQnldtrf = "c:LossQnldtrf", "Mvar, Transformer Losses, Reactive Power (no load)"
      c_TotLoadP = "c:TotLoadP", "MW, Total Load, Active Power"
      c_TotLoadQ = "c:TotLoadQ", "Mvar, Total Load, Reactive Power"
      c_TotLoadS = "c:TotLoadS", "MVA, Total Load, Apparent Power"
      c_TotLoadcos = "c:TotLoadcos", "Total Load, Power Factor"
      c_TotP = "c:TotP", "MW, Total Demand"
      c_UnsupP = "c:UnsupP", "MW, Unsupplied Demand"
      c_SupP = "c:SupP", "MW, Supplied Demand"
      c_ExportP = "c:ExportP", "MW, Export, Active Power"
      c_ExportQ = "c:ExportQ", "Mvar, Export, Reactive Power"
      c_ImportP = "c:ImportP", "MW, Import, Active Power"
      c_ImportQ = "c:ImportQ", "Mvar, Import, Reactive Power"
      c_cst_disp = "c:cst_disp", "USD/h, Total Production Costs"
      c_NrOvlBranch = "c:NrOvlBranch", "Number of overloaded branch components"
      c_NrGenQMax = "c:NrGenQMax", "Number of generators above their maximum reactive power limit"
      c_NrGenQMin = "c:NrGenQMin", "Number of generators below their minimum reactive power limit"
      c_NrTermMin = "c:NrTermMin", "Number of terminals below their minimum voltage limit"
      c_NrTermMax = "c:NrTermMax", "Number of terminals above their maximum voltage limit"
      c_RUC_Lead = "c:RUC_Lead", "Mvar, Used Reactive Capacity Lead"
      c_RUC_Lag = "c:RUC_Lag", "Mvar, Used Reactive Capacity Lag"
      c_RSC_Lead = "c:RSC_Lead", "Mvar, Spare Reactive Capacity Lead"
      c_RSC_Lag = "c:RSC_Lag", "Mvar, Spare Reactive Capacity Lag"
      c_Q_shgain = "c:Q_shgain", "Mvar, Reactive Power Shunt Gain"
      c_ConnMvar_max = "c:ConnMvar_max", "Mvar, Connected MVAR Capacity (max)"
      c_ConnMvar_min = "c:ConnMvar_min", "Mvar, Connected MVAR Capacity (min)"
      c_Maxloading = "c:Maxloading", "%, Maximum Loading"
      c_umin = "c:umin", "p.u., Minimum voltage of all terminals"
      c_umax = "c:umax", "p.u., Maximum voltage of all terminals"

    class ElmAsm(Enum):
      _init_ = 'value __doc__'

      m_dPdQ_bus1 = "m:dPdQ:bus1", "MW/Mvar, Branch Sensitivity dPbranch/dQbus"
      m_dQdQ_bus1 = "m:dQdQ:bus1", "Mvar/Mvar, Branch Sensitivity dQbranch/dQbus"
      m_dPdP_bus1 = "m:dPdP:bus1", "MW/MW, Branch Sensitivity dPbranch/dPbus"
      m_dQdP_bus1 = "m:dQdP:bus1", "Mvar/MW, Branch Sensitivity dQbranch/dPbus"
      m_dPdtap_bus1 = "m:dPdtap:bus1", "MW/tap, Branch Sensitivity dPbranch/dtap"
      m_dQdtap_bus1 = "m:dQdtap:bus1", "Mvar/tap, Branch Sensitivity dQbranch/dtap"
      m_LODF_bus1 = "m:LODF:bus1", "%, Line Outage Distribution Factor LODF"
      m_u1r_bus1 = "m:u1r:bus1", "p.u., Positive-Sequence-Voltage, Real Part"
      m_u1i_bus1 = "m:u1i:bus1", "p.u., Positive-Sequence-Voltage, Imaginary Part"
      m_u1_bus1 = "m:u1:bus1", "p.u., Positive-Sequence-Voltage, Magnitude"
      m_U1_bus1 = "m:U1:bus1", "kV, Line-Ground Positive-Sequence-Voltage, Magnitude"
      m_U1l_bus1 = "m:U1l:bus1", "kV, Line-Line Positive-Sequence-Voltage, Magnitude"
      m_phiu1_bus1 = "m:phiu1:bus1", "deg, Positive-Sequence-Voltage, Angle"
      m_inet_bus1 = "m:inet:bus1", "p.u., Current, Magnitude, referred to network"
      m_I_bus1 = "m:I:bus1", "kA, Current, Magnitude"
      m_phii_bus1 = "m:phii:bus1", "deg, Current, Angle"
      m_i1r_bus1 = "m:i1r:bus1", "p.u., Positive-Sequence Current, Real Part"
      m_i1i_bus1 = "m:i1i:bus1", "p.u., Positive-Sequence Current, Imaginary Part"
      m_i1_bus1 = "m:i1:bus1", "p.u., Positive-Sequence Current, Magnitude"
      m_I1_bus1 = "m:I1:bus1", "kA, Positive-Sequence Current, Magnitude"
      m_phii1_bus1 = "m:phii1:bus1", "deg, Positive-Sequence Current, Angle"
      m_P_bus1 = "m:P:bus1", "MW, Active Power"
      m_Q_bus1 = "m:Q:bus1", "Mvar, Reactive Power"
      m_S_bus1 = "m:S:bus1", "MVA, Apparent Power"
      m_cosphi_bus1 = "m:cosphi:bus1", "Power Factor"
      m_tanphi_bus1 = "m:tanphi:bus1", "tan(phi)"
      m_Psum_bus1 = "m:Psum:bus1", "MW, Total Active Power"
      m_Qsum_bus1 = "m:Qsum:bus1", "Mvar, Total Reactive Power"
      m_Ssum_bus1 = "m:Ssum:bus1", "MVA, Total Apparent Power"
      m_cosphisum_bus1 = "m:cosphisum:bus1", "Total Power Factor"
      m_tanphisum_bus1 = "m:tanphisum:bus1", "Total tan(phi)"
      m_Spu_bus1 = "m:Spu:bus1", "MVA/p.u., Apparent Power per p.u. Voltage"
      m_i1P_bus1 = "m:i1P:bus1", "p.u., Positive-Sequence Active Current"
      m_i1Q_bus1 = "m:i1Q:bus1", "p.u., Positive-Sequence Reactive Current"
      m_I1P_bus1 = "m:I1P:bus1", "kA, Positive-Sequence Active Current"
      m_I1Q_bus1 = "m:I1Q:bus1", "kA, Positive-Sequence Reactive Current"
      m_ur_bus1 = "m:ur:bus1", "p.u., Voltage, Real Part"
      m_ui_bus1 = "m:ui:bus1", "p.u., Voltage, Imaginary Part"
      m_u_bus1 = "m:u:bus1", "p.u., Voltage, Magnitude"
      m_ir_bus1 = "m:ir:bus1", "p.u., Current, Real Part"
      m_ii_bus1 = "m:ii:bus1", "p.u., Current, Imaginary Part"
      m_i_bus1 = "m:i:bus1", "p.u., Current, Magnitude"
      m_phiui_bus1 = "m:phiui:bus1", "deg, Angle between Voltage and Current"
      m_phiu1i1_bus1 = "m:phiu1i1:bus1", "deg, Angle between Voltage and Current in positive sequence system"
      m_Tfct_bus1 = "m:Tfct:bus1", "s, Fault Clearing Time"
      m_Brkload_bus1 = "m:Brkload:bus1", "%, Breaker Loading"
      n_dphidP_bus1 = "n:dphidP:bus1", "deg/MW, dphi/dP Sensitivity"
      n_dphidQ_bus1 = "n:dphidQ:bus1", "deg/Mvar, dphi/dQ Sensitivity"
      n_dphidtap_bus1 = "n:dphidtap:bus1", "deg/tap, dphi/dtap Sensitivity"
      n_dvdP_bus1 = "n:dvdP:bus1", "p.u./MW, dv/dP Sensitivity"
      n_dvdQ_bus1 = "n:dvdQ:bus1", "p.u./Mvar, dv/dQ Sensitivity"
      n_dvdtap_bus1 = "n:dvdtap:bus1", "p.u./tap, dv/dtap Sensitivity"
      n_levecQ_bus1 = "n:levecQ:bus1", "Left Eigenvector of selected Q-Mode"
      n_revecQ_bus1 = "n:revecQ:bus1", "Right Eigenvector of selected Q-Mode"
      n_partQ_bus1 = "n:partQ:bus1", "Participation Factor of selected Q-Mode"
      n_ur_bus1 = "n:ur:bus1", "p.u., Voltage, Real Part"
      n_ui_bus1 = "n:ui:bus1", "p.u., Voltage, Imaginary Part"
      n_u_bus1 = "n:u:bus1", "p.u., Voltage, Magnitude"
      n_upc_bus1 = "n:upc:bus1", "%, Voltage, Magnitude"
      n_u1_bus1 = "n:u1:bus1", "p.u., Voltage, Magnitude"
      n_u1pc_bus1 = "n:u1pc:bus1", "%, Voltage, Magnitude"
      n_u1r_bus1 = "n:u1r:bus1", "p.u., Positive-Sequence Voltage, Real Part"
      n_u1i_bus1 = "n:u1i:bus1", "p.u., Positive-Sequence Voltage, Imaginary Part"
      n_U_bus1 = "n:U:bus1", "kV, Line-Ground Voltage, Magnitude"
      n_Ul_bus1 = "n:Ul:bus1", "kV, Line-Line Voltage, Magnitude"
      n_phiu_bus1 = "n:phiu:bus1", "deg, Voltage, Angle"
      n_phiurel_bus1 = "n:phiurel:bus1", "deg, Voltage, Relative Angle"
      n_du_bus1 = "n:du:bus1", "%, Voltage Deviation"
      n_Pgen_bus1 = "n:Pgen:bus1", "MW, Generation, Active Power"
      n_Qgen_bus1 = "n:Qgen:bus1", "Mvar, Generation, Reactive Power"
      n_Pmot_bus1 = "n:Pmot:bus1", "MW, Motor Load, Active Power"
      n_Qmot_bus1 = "n:Qmot:bus1", "Mvar, Motor Load, Reactive Power"
      n_Pload_bus1 = "n:Pload:bus1", "MW, General Load, Active Power"
      n_Qload_bus1 = "n:Qload:bus1", "Mvar, General Load, Reactive Power"
      n_Pcomp_bus1 = "n:Pcomp:bus1", "MW, Compensation (Losses)"
      n_Qcomp_bus1 = "n:Qcomp:bus1", "Mvar, Compensation"
      n_Pnet_bus1 = "n:Pnet:bus1", "MW, External Networks, Active Power"
      n_Qnet_bus1 = "n:Qnet:bus1", "Mvar, External Networks, Reactive Power"
      n_Pflow_bus1 = "n:Pflow:bus1", "MW, Power Flow, Active Power"
      n_Qflow_bus1 = "n:Qflow:bus1", "Mvar, Power Flow, Reactive Power"
      n_Pout_bus1 = "n:Pout:bus1", "MW, Outgoing Flow, Active Power"
      n_Qout_bus1 = "n:Qout:bus1", "Mvar, Outgoing Power, Reactive Power"
      n_Sout_bus1 = "n:Sout:bus1", "MVA, Outgoing Power, Apparent Power"
      n_cosphiout_bus1 = "n:cosphiout:bus1", "Outgoing Power, Power Factor"
      n_Pbalance_bus1 = "n:Pbalance:bus1", "MW, Active Power Balance (=0)"
      n_Qbalance_bus1 = "n:Qbalance:bus1", "Mvar, Reactive Power Balance (=0)"
      n_LossPdown_bus1 = "n:LossPdown:bus1", "MW, Losses, downstream"
      n_LossQdown_bus1 = "n:LossQdown:bus1", "Mvar, Losses, downstream (Reactive Power)"
      n_LossPdownload_bus1 = "n:LossPdownload:bus1", "MW, Load losses, downstream"
      n_LossQdownload_bus1 = "n:LossQdownload:bus1", "Mvar, Load losses, downstream"
      n_LossPdownnoload_bus1 = "n:LossPdownnoload:bus1", "MW, No load losses, downstream"
      n_LossQdownnoload_bus1 = "n:LossQdownnoload:bus1", "Mvar, No load losses, downstream"
      n_du_feed_bus1 = "n:du_feed:bus1", "%, Voltage difference relative to feeder begin"
      s_pset = "s:pset", "MW, IN, Active Power Setpoint, continuous, double"
      s_slip = "s:slip", "%, OUT, Slip, continuous, double"
      s_rradd = "s:rradd", "p.u., OUT, Additional Rotor Resistance, continuous, double"
      c_cosn = "c:cosn", "Rated Power Factor"
      c_sgnn = "c:sgnn", "MVA, Total Nominal Apparent power (ngnum*sgn)"
      c_Pnom = "c:Pnom", "MW, Total Nominal Active power"
      c_p = "c:p", "MW, Active Power (Electrical)"
      c_q = "c:q", "MW, Reactive Power"
      c_qcomp = "c:qcomp", "Mvar, Reactive Power for Compensation"
      c_pm = "c:pm", "MW, Mechanical Power"
      c_rstr = "c:rstr", "p.u., Stator Resistance"
      c_xstr = "c:xstr", "p.u., Stator Reactance"
      c_xrtrA = "c:xrtrA", "p.u., Rotor Reactance A"
      c_xrtrB = "c:xrtrB", "p.u., Rotor Reactance B"
      c_xm = "c:xm", "p.u., Magnetising Reactance"
      c_xmrtr = "c:xmrtr", "p.u., Rotor Leakage Reactance"
      c_frnom = "c:frnom", "Hz, Nominal Frequency"
      c_loading = "c:loading", "%, Loading"
      c_i_con = "c:i_con", "flag"
      c_i_slip = "c:i_slip", "flag"
      c_slipn = "c:slipn", "%, Nominal Slip"
      c_slipset = "c:slipset", "%, Slip Setpoint"
      c_i_mot = "c:i_mot", "i_mot=0"
      c_mk = "c:mk", "Stalling Torque"
      c_xme = "c:xme", "p.u., Electrical Torque"
      c_pmset = "c:pmset", "MW, Mechanical Power Setpoint"
      c_xmtset = "c:xmtset", "p.u., Mechanical Torque Setpoint"
      c_rrtrA0 = "c:rrtrA0", "p.u., RrA0"
      c_xrtrA0 = "c:xrtrA0", "p.u., XrA0"
      c_rr0 = "c:rr0", "p.u., Rr0"
      c_xr0 = "c:xr0", "p.u., Xr0"
      c_rr1 = "c:rr1", "p.u., Rr1"
      c_xr1 = "c:xr1", "p.u., Xr1"
      c_zonewscale = "c:zonewscale", "Wind generation zone scaling factor"
      c_i_rem = "c:i_rem", "Remote Control"
      c_psiM = "c:psiM", "p.u., Magnetising Flux"
      c_ksat = "c:ksat", "Saturation Factor"
      c_xmsat = "c:xmsat", "p.u., Saturated Magnetising Reactance"
      c_Ploss = "c:Ploss", "MW, Losses (total)"
      c_Qloss = "c:Qloss", "Mvar, Reactive-Losses (total)"
      c_Plossld = "c:Plossld", "MW, Losses (load)"
      c_Qlossld = "c:Qlossld", "Mvar, Reactive-Losses (load)"
      c_Plossnld = "c:Plossnld", "MW, Losses (no load)"
      c_Qlossnld = "c:Qlossnld", "Mvar, Reactive-Losses (no load)"

    class ElmAsmsc(Enum):
      _init_ = 'value __doc__'

      m_dPdQ_bus1 = "m:dPdQ:bus1", "MW/Mvar, Branch Sensitivity dPbranch/dQbus"
      m_dPdQ_bus2 = "m:dPdQ:bus2", "MW/Mvar, Branch Sensitivity dPbranch/dQbus"
      m_dQdQ_bus1 = "m:dQdQ:bus1", "Mvar/Mvar, Branch Sensitivity dQbranch/dQbus"
      m_dQdQ_bus2 = "m:dQdQ:bus2", "Mvar/Mvar, Branch Sensitivity dQbranch/dQbus"
      m_dPdP_bus1 = "m:dPdP:bus1", "MW/MW, Branch Sensitivity dPbranch/dPbus"
      m_dPdP_bus2 = "m:dPdP:bus2", "MW/MW, Branch Sensitivity dPbranch/dPbus"
      m_dQdP_bus1 = "m:dQdP:bus1", "Mvar/MW, Branch Sensitivity dQbranch/dPbus"
      m_dQdP_bus2 = "m:dQdP:bus2", "Mvar/MW, Branch Sensitivity dQbranch/dPbus"
      m_dPdtap_bus1 = "m:dPdtap:bus1", "MW/tap, Branch Sensitivity dPbranch/dtap"
      m_dPdtap_bus2 = "m:dPdtap:bus2", "MW/tap, Branch Sensitivity dPbranch/dtap"
      m_dQdtap_bus1 = "m:dQdtap:bus1", "Mvar/tap, Branch Sensitivity dQbranch/dtap"
      m_dQdtap_bus2 = "m:dQdtap:bus2", "Mvar/tap, Branch Sensitivity dQbranch/dtap"
      m_LODF_bus1 = "m:LODF:bus1", "%, Line Outage Distribution Factor LODF"
      m_LODF_bus2 = "m:LODF:bus2", "%, Line Outage Distribution Factor LODF"
      m_u1r_bus1 = "m:u1r:bus1", "p.u., Positive-Sequence-Voltage, Real Part"
      m_u1r_bus2 = "m:u1r:bus2", "p.u., Positive-Sequence-Voltage, Real Part"
      m_u1i_bus1 = "m:u1i:bus1", "p.u., Positive-Sequence-Voltage, Imaginary Part"
      m_u1i_bus2 = "m:u1i:bus2", "p.u., Positive-Sequence-Voltage, Imaginary Part"
      m_u1_bus1 = "m:u1:bus1", "p.u., Positive-Sequence-Voltage, Magnitude"
      m_u1_bus2 = "m:u1:bus2", "p.u., Positive-Sequence-Voltage, Magnitude"
      m_U1_bus1 = "m:U1:bus1", "kV, Line-Ground Positive-Sequence-Voltage, Magnitude"
      m_U1_bus2 = "m:U1:bus2", "kV, Line-Ground Positive-Sequence-Voltage, Magnitude"
      m_U1l_bus1 = "m:U1l:bus1", "kV, Line-Line Positive-Sequence-Voltage, Magnitude"
      m_U1l_bus2 = "m:U1l:bus2", "kV, Line-Line Positive-Sequence-Voltage, Magnitude"
      m_phiu1_bus1 = "m:phiu1:bus1", "deg, Positive-Sequence-Voltage, Angle"
      m_phiu1_bus2 = "m:phiu1:bus2", "deg, Positive-Sequence-Voltage, Angle"
      m_inet_bus1 = "m:inet:bus1", "p.u., Current, Magnitude, referred to network"
      m_inet_bus2 = "m:inet:bus2", "p.u., Current, Magnitude, referred to network"
      m_I_bus1 = "m:I:bus1", "kA, Current, Magnitude"
      m_I_bus2 = "m:I:bus2", "kA, Current, Magnitude"
      m_phii_bus1 = "m:phii:bus1", "deg, Current, Angle"
      m_phii_bus2 = "m:phii:bus2", "deg, Current, Angle"
      m_i1r_bus1 = "m:i1r:bus1", "p.u., Positive-Sequence Current, Real Part"
      m_i1r_bus2 = "m:i1r:bus2", "p.u., Positive-Sequence Current, Real Part"
      m_i1i_bus1 = "m:i1i:bus1", "p.u., Positive-Sequence Current, Imaginary Part"
      m_i1i_bus2 = "m:i1i:bus2", "p.u., Positive-Sequence Current, Imaginary Part"
      m_i1_bus1 = "m:i1:bus1", "p.u., Positive-Sequence Current, Magnitude"
      m_i1_bus2 = "m:i1:bus2", "p.u., Positive-Sequence Current, Magnitude"
      m_I1_bus1 = "m:I1:bus1", "kA, Positive-Sequence Current, Magnitude"
      m_I1_bus2 = "m:I1:bus2", "kA, Positive-Sequence Current, Magnitude"
      m_phii1_bus1 = "m:phii1:bus1", "deg, Positive-Sequence Current, Angle"
      m_phii1_bus2 = "m:phii1:bus2", "deg, Positive-Sequence Current, Angle"
      m_P_bus1 = "m:P:bus1", "MW, Active Power"
      m_P_bus2 = "m:P:bus2", "MW, Active Power"
      m_Q_bus1 = "m:Q:bus1", "Mvar, Reactive Power"
      m_Q_bus2 = "m:Q:bus2", "Mvar, Reactive Power"
      m_S_bus1 = "m:S:bus1", "MVA, Apparent Power"
      m_S_bus2 = "m:S:bus2", "MVA, Apparent Power"
      m_cosphi_bus1 = "m:cosphi:bus1", "Power Factor"
      m_cosphi_bus2 = "m:cosphi:bus2", "Power Factor"
      m_tanphi_bus1 = "m:tanphi:bus1", "tan(phi)"
      m_tanphi_bus2 = "m:tanphi:bus2", "tan(phi)"
      m_Psum_bus1 = "m:Psum:bus1", "MW, Total Active Power"
      m_Psum_bus2 = "m:Psum:bus2", "MW, Total Active Power"
      m_Qsum_bus1 = "m:Qsum:bus1", "Mvar, Total Reactive Power"
      m_Qsum_bus2 = "m:Qsum:bus2", "Mvar, Total Reactive Power"
      m_Ssum_bus1 = "m:Ssum:bus1", "MVA, Total Apparent Power"
      m_Ssum_bus2 = "m:Ssum:bus2", "MVA, Total Apparent Power"
      m_cosphisum_bus1 = "m:cosphisum:bus1", "Total Power Factor"
      m_cosphisum_bus2 = "m:cosphisum:bus2", "Total Power Factor"
      m_tanphisum_bus1 = "m:tanphisum:bus1", "Total tan(phi)"
      m_tanphisum_bus2 = "m:tanphisum:bus2", "Total tan(phi)"
      m_Spu_bus1 = "m:Spu:bus1", "MVA/p.u., Apparent Power per p.u. Voltage"
      m_Spu_bus2 = "m:Spu:bus2", "MVA/p.u., Apparent Power per p.u. Voltage"
      m_i1P_bus1 = "m:i1P:bus1", "p.u., Positive-Sequence Active Current"
      m_i1P_bus2 = "m:i1P:bus2", "p.u., Positive-Sequence Active Current"
      m_i1Q_bus1 = "m:i1Q:bus1", "p.u., Positive-Sequence Reactive Current"
      m_i1Q_bus2 = "m:i1Q:bus2", "p.u., Positive-Sequence Reactive Current"
      m_I1P_bus1 = "m:I1P:bus1", "kA, Positive-Sequence Active Current"
      m_I1P_bus2 = "m:I1P:bus2", "kA, Positive-Sequence Active Current"
      m_I1Q_bus1 = "m:I1Q:bus1", "kA, Positive-Sequence Reactive Current"
      m_I1Q_bus2 = "m:I1Q:bus2", "kA, Positive-Sequence Reactive Current"
      m_ur_bus1 = "m:ur:bus1", "p.u., Voltage, Real Part"
      m_ur_bus2 = "m:ur:bus2", "p.u., Voltage, Real Part"
      m_ui_bus1 = "m:ui:bus1", "p.u., Voltage, Imaginary Part"
      m_ui_bus2 = "m:ui:bus2", "p.u., Voltage, Imaginary Part"
      m_u_bus1 = "m:u:bus1", "p.u., Voltage, Magnitude"
      m_u_bus2 = "m:u:bus2", "p.u., Voltage, Magnitude"
      m_ir_bus1 = "m:ir:bus1", "p.u., Current, Real Part"
      m_ir_bus2 = "m:ir:bus2", "p.u., Current, Real Part"
      m_ii_bus1 = "m:ii:bus1", "p.u., Current, Imaginary Part"
      m_ii_bus2 = "m:ii:bus2", "p.u., Current, Imaginary Part"
      m_i_bus1 = "m:i:bus1", "p.u., Current, Magnitude"
      m_i_bus2 = "m:i:bus2", "p.u., Current, Magnitude"
      m_phiui_bus1 = "m:phiui:bus1", "deg, Angle between Voltage and Current"
      m_phiui_bus2 = "m:phiui:bus2", "deg, Angle between Voltage and Current"
      m_phiu1i1_bus1 = "m:phiu1i1:bus1", "deg, Angle between Voltage and Current in positive sequence system"
      m_phiu1i1_bus2 = "m:phiu1i1:bus2", "deg, Angle between Voltage and Current in positive sequence system"
      m_Tfct_bus1 = "m:Tfct:bus1", "s, Fault Clearing Time"
      m_Tfct_bus2 = "m:Tfct:bus2", "s, Fault Clearing Time"
      m_Brkload_bus1 = "m:Brkload:bus1", "%, Breaker Loading"
      m_Brkload_bus2 = "m:Brkload:bus2", "%, Breaker Loading"
      n_dphidP_bus1 = "n:dphidP:bus1", "deg/MW, dphi/dP Sensitivity"
      n_dphidP_bus2 = "n:dphidP:bus2", "deg/MW, dphi/dP Sensitivity"
      n_dphidQ_bus1 = "n:dphidQ:bus1", "deg/Mvar, dphi/dQ Sensitivity"
      n_dphidQ_bus2 = "n:dphidQ:bus2", "deg/Mvar, dphi/dQ Sensitivity"
      n_dphidtap_bus1 = "n:dphidtap:bus1", "deg/tap, dphi/dtap Sensitivity"
      n_dphidtap_bus2 = "n:dphidtap:bus2", "deg/tap, dphi/dtap Sensitivity"
      n_dvdP_bus1 = "n:dvdP:bus1", "p.u./MW, dv/dP Sensitivity"
      n_dvdP_bus2 = "n:dvdP:bus2", "p.u./MW, dv/dP Sensitivity"
      n_dvdQ_bus1 = "n:dvdQ:bus1", "p.u./Mvar, dv/dQ Sensitivity"
      n_dvdQ_bus2 = "n:dvdQ:bus2", "p.u./Mvar, dv/dQ Sensitivity"
      n_dvdtap_bus1 = "n:dvdtap:bus1", "p.u./tap, dv/dtap Sensitivity"
      n_dvdtap_bus2 = "n:dvdtap:bus2", "p.u./tap, dv/dtap Sensitivity"
      n_levecQ_bus1 = "n:levecQ:bus1", "Left Eigenvector of selected Q-Mode"
      n_levecQ_bus2 = "n:levecQ:bus2", "Left Eigenvector of selected Q-Mode"
      n_revecQ_bus1 = "n:revecQ:bus1", "Right Eigenvector of selected Q-Mode"
      n_revecQ_bus2 = "n:revecQ:bus2", "Right Eigenvector of selected Q-Mode"
      n_partQ_bus1 = "n:partQ:bus1", "Participation Factor of selected Q-Mode"
      n_partQ_bus2 = "n:partQ:bus2", "Participation Factor of selected Q-Mode"
      n_ur_bus1 = "n:ur:bus1", "p.u., Voltage, Real Part"
      n_ur_bus2 = "n:ur:bus2", "p.u., Voltage, Real Part"
      n_ui_bus1 = "n:ui:bus1", "p.u., Voltage, Imaginary Part"
      n_ui_bus2 = "n:ui:bus2", "p.u., Voltage, Imaginary Part"
      n_u_bus1 = "n:u:bus1", "p.u., Voltage, Magnitude"
      n_u_bus2 = "n:u:bus2", "p.u., Voltage, Magnitude"
      n_upc_bus1 = "n:upc:bus1", "%, Voltage, Magnitude"
      n_upc_bus2 = "n:upc:bus2", "%, Voltage, Magnitude"
      n_u1_bus1 = "n:u1:bus1", "p.u., Voltage, Magnitude"
      n_u1_bus2 = "n:u1:bus2", "p.u., Voltage, Magnitude"
      n_u1pc_bus1 = "n:u1pc:bus1", "%, Voltage, Magnitude"
      n_u1pc_bus2 = "n:u1pc:bus2", "%, Voltage, Magnitude"
      n_u1r_bus1 = "n:u1r:bus1", "p.u., Positive-Sequence Voltage, Real Part"
      n_u1r_bus2 = "n:u1r:bus2", "p.u., Positive-Sequence Voltage, Real Part"
      n_u1i_bus1 = "n:u1i:bus1", "p.u., Positive-Sequence Voltage, Imaginary Part"
      n_u1i_bus2 = "n:u1i:bus2", "p.u., Positive-Sequence Voltage, Imaginary Part"
      n_U_bus1 = "n:U:bus1", "kV, Line-Ground Voltage, Magnitude"
      n_U_bus2 = "n:U:bus2", "kV, Line-Ground Voltage, Magnitude"
      n_Ul_bus1 = "n:Ul:bus1", "kV, Line-Line Voltage, Magnitude"
      n_Ul_bus2 = "n:Ul:bus2", "kV, Line-Line Voltage, Magnitude"
      n_phiu_bus1 = "n:phiu:bus1", "deg, Voltage, Angle"
      n_phiu_bus2 = "n:phiu:bus2", "deg, Voltage, Angle"
      n_phiurel_bus1 = "n:phiurel:bus1", "deg, Voltage, Relative Angle"
      n_phiurel_bus2 = "n:phiurel:bus2", "deg, Voltage, Relative Angle"
      n_du_bus1 = "n:du:bus1", "%, Voltage Deviation"
      n_du_bus2 = "n:du:bus2", "%, Voltage Deviation"
      n_Pgen_bus1 = "n:Pgen:bus1", "MW, Generation, Active Power"
      n_Pgen_bus2 = "n:Pgen:bus2", "MW, Generation, Active Power"
      n_Qgen_bus1 = "n:Qgen:bus1", "Mvar, Generation, Reactive Power"
      n_Qgen_bus2 = "n:Qgen:bus2", "Mvar, Generation, Reactive Power"
      n_Pmot_bus1 = "n:Pmot:bus1", "MW, Motor Load, Active Power"
      n_Pmot_bus2 = "n:Pmot:bus2", "MW, Motor Load, Active Power"
      n_Qmot_bus1 = "n:Qmot:bus1", "Mvar, Motor Load, Reactive Power"
      n_Qmot_bus2 = "n:Qmot:bus2", "Mvar, Motor Load, Reactive Power"
      n_Pload_bus1 = "n:Pload:bus1", "MW, General Load, Active Power"
      n_Pload_bus2 = "n:Pload:bus2", "MW, General Load, Active Power"
      n_Qload_bus1 = "n:Qload:bus1", "Mvar, General Load, Reactive Power"
      n_Qload_bus2 = "n:Qload:bus2", "Mvar, General Load, Reactive Power"
      n_Pcomp_bus1 = "n:Pcomp:bus1", "MW, Compensation (Losses)"
      n_Pcomp_bus2 = "n:Pcomp:bus2", "MW, Compensation (Losses)"
      n_Qcomp_bus1 = "n:Qcomp:bus1", "Mvar, Compensation"
      n_Qcomp_bus2 = "n:Qcomp:bus2", "Mvar, Compensation"
      n_Pnet_bus1 = "n:Pnet:bus1", "MW, External Networks, Active Power"
      n_Pnet_bus2 = "n:Pnet:bus2", "MW, External Networks, Active Power"
      n_Qnet_bus1 = "n:Qnet:bus1", "Mvar, External Networks, Reactive Power"
      n_Qnet_bus2 = "n:Qnet:bus2", "Mvar, External Networks, Reactive Power"
      n_Pflow_bus1 = "n:Pflow:bus1", "MW, Power Flow, Active Power"
      n_Pflow_bus2 = "n:Pflow:bus2", "MW, Power Flow, Active Power"
      n_Qflow_bus1 = "n:Qflow:bus1", "Mvar, Power Flow, Reactive Power"
      n_Qflow_bus2 = "n:Qflow:bus2", "Mvar, Power Flow, Reactive Power"
      n_Pout_bus1 = "n:Pout:bus1", "MW, Outgoing Flow, Active Power"
      n_Pout_bus2 = "n:Pout:bus2", "MW, Outgoing Flow, Active Power"
      n_Qout_bus1 = "n:Qout:bus1", "Mvar, Outgoing Power, Reactive Power"
      n_Qout_bus2 = "n:Qout:bus2", "Mvar, Outgoing Power, Reactive Power"
      n_Sout_bus1 = "n:Sout:bus1", "MVA, Outgoing Power, Apparent Power"
      n_Sout_bus2 = "n:Sout:bus2", "MVA, Outgoing Power, Apparent Power"
      n_cosphiout_bus1 = "n:cosphiout:bus1", "Outgoing Power, Power Factor"
      n_cosphiout_bus2 = "n:cosphiout:bus2", "Outgoing Power, Power Factor"
      n_Pbalance_bus1 = "n:Pbalance:bus1", "MW, Active Power Balance (=0)"
      n_Pbalance_bus2 = "n:Pbalance:bus2", "MW, Active Power Balance (=0)"
      n_Qbalance_bus1 = "n:Qbalance:bus1", "Mvar, Reactive Power Balance (=0)"
      n_Qbalance_bus2 = "n:Qbalance:bus2", "Mvar, Reactive Power Balance (=0)"
      n_LossPdown_bus1 = "n:LossPdown:bus1", "MW, Losses, downstream"
      n_LossPdown_bus2 = "n:LossPdown:bus2", "MW, Losses, downstream"
      n_LossQdown_bus1 = "n:LossQdown:bus1", "Mvar, Losses, downstream (Reactive Power)"
      n_LossQdown_bus2 = "n:LossQdown:bus2", "Mvar, Losses, downstream (Reactive Power)"
      n_LossPdownload_bus1 = "n:LossPdownload:bus1", "MW, Load losses, downstream"
      n_LossPdownload_bus2 = "n:LossPdownload:bus2", "MW, Load losses, downstream"
      n_LossQdownload_bus1 = "n:LossQdownload:bus1", "Mvar, Load losses, downstream"
      n_LossQdownload_bus2 = "n:LossQdownload:bus2", "Mvar, Load losses, downstream"
      n_LossPdownnoload_bus1 = "n:LossPdownnoload:bus1", "MW, No load losses, downstream"
      n_LossPdownnoload_bus2 = "n:LossPdownnoload:bus2", "MW, No load losses, downstream"
      n_LossQdownnoload_bus1 = "n:LossQdownnoload:bus1", "Mvar, No load losses, downstream"
      n_LossQdownnoload_bus2 = "n:LossQdownnoload:bus2", "Mvar, No load losses, downstream"
      n_du_feed_bus1 = "n:du_feed:bus1", "%, Voltage difference relative to feeder begin"
      n_du_feed_bus2 = "n:du_feed:bus2", "%, Voltage difference relative to feeder begin"
      s_usr = "s:usr", "p.u., STATE, Slip-ring Voltage, Real Part, continuous, double"
      s_usi = "s:usi", "p.u., STATE, Slip-ring Voltage, Imaginary Part, continuous, double"
      c_cosn = "c:cosn", "Rated Power Factor"
      c_sgnn = "c:sgnn", "MVA, Total rated apparent power (ngnum*sgn)"
      c_Pnom = "c:Pnom", "MW, Total Nominal Active power"
      c_p = "c:p", "MW, Active Power (Electrical)"
      c_q = "c:q", "Mvar, Reactive Power"
      c_rstr = "c:rstr", "p.u., Stator Resistance"
      c_xstr = "c:xstr", "p.u., Stator Reactance"
      c_xrtrA = "c:xrtrA", "p.u., Rotor Reactance A"
      c_xrtrB = "c:xrtrB", "p.u., Rotor Reactance B"
      c_xm = "c:xm", "p.u., Magnetising Reactance"
      c_xmrtr = "c:xmrtr", "p.u., Rotor Leakage Reactance"
      c_frnom = "c:frnom", "Hz, Nominal Frequency"
      c_loading = "c:loading", "%, Loading"
      c_i_con = "c:i_con", "flag"
      c_i_slip = "c:i_slip", "flag"
      c_slipn = "c:slipn", "%, Nominal Slip"
      c_i_mot = "c:i_mot", "i_mot=0"
      c_mk = "c:mk", "Stalling Torque"
      c_xme = "c:xme", "p.u., Electrical Torque"
      c_i_cdisp = "c:i_cdisp", "Current Displacement"
      c_rrtrA0 = "c:rrtrA0", "p.u., RrA0"
      c_xrtrA0 = "c:xrtrA0", "p.u., XrA0"
      c_rr0 = "c:rr0", "p.u., Rr0"
      c_xr0 = "c:xr0", "p.u., Xr0"
      c_rr1 = "c:rr1", "p.u., Rr1"
      c_xr1 = "c:xr1", "p.u., Xr1"
      c_Pmr = "c:Pmr", "Amplification, Real Part"
      c_Pmi = "c:Pmi", "Amplification, Imaginary Part"
      c_slip = "c:slip", "%, Slip"
      c_Unrot = "c:Unrot", "kV, Rated Slip Ring Voltage"
      c_zonewscale = "c:zonewscale", "Wind generation zone scaling factor"
      c_psiM = "c:psiM", "p.u., Magnetising Flux"
      c_ksat = "c:ksat", "Saturation Factor"
      c_xmsat = "c:xmsat", "p.u., Saturated Magnetising Reactance"
      c_Ploss = "c:Ploss", "MW, Losses (total)"
      c_Qloss = "c:Qloss", "Mvar, Reactive-Losses (total)"
      c_Plossld = "c:Plossld", "MW, Losses (load)"
      c_Qlossld = "c:Qlossld", "Mvar, Reactive-Losses (load)"
      c_Plossnld = "c:Plossnld", "MW, Losses (no load)"
      c_Qlossnld = "c:Qlossnld", "Mvar, Reactive-Losses (no load)"

    class ElmBoundary(Enum):
      _init_ = 'value __doc__'

      c_dPdP = "c:dPdP", "MW/MW, Sensitivity dPdP"
      c_dPdQ = "c:dPdQ", "MW/Mvar, Sensitivity dPdQ"
      c_dQdP = "c:dQdP", "Mvar/MW, Sensitivity dQdP"
      c_dQdQ = "c:dQdQ", "Mvar/Mvar, Sensitivity dQdQ"
      c_dPdtap = "c:dPdtap", "MW/tap, Sensitivity dPdtap"
      c_dQdtap = "c:dQdtap", "Mvar/tap, Sensitivity dQdtap"
      c_Pinter = "c:Pinter", "MW, Active Power Interchange"
      c_Qinter = "c:Qinter", "Mvar, Reactive Power Interchange"
      c_ExportP = "c:ExportP", "MW, Export, Active Power"
      c_ExportQ = "c:ExportQ", "Mvar, Export, Reactive Power"
      c_ImportP = "c:ImportP", "MW, Import, Active Power"
      c_ImportQ = "c:ImportQ", "Mvar, Import, Reactive Power"

    class ElmCoup(Enum):
      _init_ = 'value __doc__'

      m_dPdQ_bus1 = "m:dPdQ:bus1", "MW/Mvar, Branch Sensitivity dPbranch/dQbus"
      m_dPdQ_bus2 = "m:dPdQ:bus2", "MW/Mvar, Branch Sensitivity dPbranch/dQbus"
      m_dQdQ_bus1 = "m:dQdQ:bus1", "Mvar/Mvar, Branch Sensitivity dQbranch/dQbus"
      m_dQdQ_bus2 = "m:dQdQ:bus2", "Mvar/Mvar, Branch Sensitivity dQbranch/dQbus"
      m_dPdP_bus1 = "m:dPdP:bus1", "MW/MW, Branch Sensitivity dPbranch/dPbus"
      m_dPdP_bus2 = "m:dPdP:bus2", "MW/MW, Branch Sensitivity dPbranch/dPbus"
      m_dQdP_bus1 = "m:dQdP:bus1", "Mvar/MW, Branch Sensitivity dQbranch/dPbus"
      m_dQdP_bus2 = "m:dQdP:bus2", "Mvar/MW, Branch Sensitivity dQbranch/dPbus"
      m_dPdtap_bus1 = "m:dPdtap:bus1", "MW/tap, Branch Sensitivity dPbranch/dtap"
      m_dPdtap_bus2 = "m:dPdtap:bus2", "MW/tap, Branch Sensitivity dPbranch/dtap"
      m_dQdtap_bus1 = "m:dQdtap:bus1", "Mvar/tap, Branch Sensitivity dQbranch/dtap"
      m_dQdtap_bus2 = "m:dQdtap:bus2", "Mvar/tap, Branch Sensitivity dQbranch/dtap"
      m_LODF_bus1 = "m:LODF:bus1", "%, Line Outage Distribution Factor LODF"
      m_LODF_bus2 = "m:LODF:bus2", "%, Line Outage Distribution Factor LODF"
      m_u1r_bus1 = "m:u1r:bus1", "p.u., Positive-Sequence-Voltage, Real Part"
      m_u1r_bus2 = "m:u1r:bus2", "p.u., Positive-Sequence-Voltage, Real Part"
      m_u1i_bus1 = "m:u1i:bus1", "p.u., Positive-Sequence-Voltage, Imaginary Part"
      m_u1i_bus2 = "m:u1i:bus2", "p.u., Positive-Sequence-Voltage, Imaginary Part"
      m_u1_bus1 = "m:u1:bus1", "p.u., Positive-Sequence-Voltage, Magnitude"
      m_u1_bus2 = "m:u1:bus2", "p.u., Positive-Sequence-Voltage, Magnitude"
      m_U1_bus1 = "m:U1:bus1", "kV, Line-Ground Positive-Sequence-Voltage, Magnitude"
      m_U1_bus2 = "m:U1:bus2", "kV, Line-Ground Positive-Sequence-Voltage, Magnitude"
      m_U1l_bus1 = "m:U1l:bus1", "kV, Line-Line Positive-Sequence-Voltage, Magnitude"
      m_U1l_bus2 = "m:U1l:bus2", "kV, Line-Line Positive-Sequence-Voltage, Magnitude"
      m_phiu1_bus1 = "m:phiu1:bus1", "deg, Positive-Sequence-Voltage, Angle"
      m_phiu1_bus2 = "m:phiu1:bus2", "deg, Positive-Sequence-Voltage, Angle"
      m_inet_bus1 = "m:inet:bus1", "p.u., Current, Magnitude, referred to network"
      m_inet_bus2 = "m:inet:bus2", "p.u., Current, Magnitude, referred to network"
      m_I_bus1 = "m:I:bus1", "kA, Current, Magnitude"
      m_I_bus2 = "m:I:bus2", "kA, Current, Magnitude"
      m_phii_bus1 = "m:phii:bus1", "deg, Current, Angle"
      m_phii_bus2 = "m:phii:bus2", "deg, Current, Angle"
      m_i1r_bus1 = "m:i1r:bus1", "p.u., Positive-Sequence Current, Real Part"
      m_i1r_bus2 = "m:i1r:bus2", "p.u., Positive-Sequence Current, Real Part"
      m_i1i_bus1 = "m:i1i:bus1", "p.u., Positive-Sequence Current, Imaginary Part"
      m_i1i_bus2 = "m:i1i:bus2", "p.u., Positive-Sequence Current, Imaginary Part"
      m_i1_bus1 = "m:i1:bus1", "p.u., Positive-Sequence Current, Magnitude"
      m_i1_bus2 = "m:i1:bus2", "p.u., Positive-Sequence Current, Magnitude"
      m_I1_bus1 = "m:I1:bus1", "kA, Positive-Sequence Current, Magnitude"
      m_I1_bus2 = "m:I1:bus2", "kA, Positive-Sequence Current, Magnitude"
      m_phii1_bus1 = "m:phii1:bus1", "deg, Positive-Sequence Current, Angle"
      m_phii1_bus2 = "m:phii1:bus2", "deg, Positive-Sequence Current, Angle"
      m_P_bus1 = "m:P:bus1", "MW, Active Power"
      m_P_bus2 = "m:P:bus2", "MW, Active Power"
      m_Q_bus1 = "m:Q:bus1", "Mvar, Reactive Power"
      m_Q_bus2 = "m:Q:bus2", "Mvar, Reactive Power"
      m_S_bus1 = "m:S:bus1", "MVA, Apparent Power"
      m_S_bus2 = "m:S:bus2", "MVA, Apparent Power"
      m_cosphi_bus1 = "m:cosphi:bus1", "Power Factor"
      m_cosphi_bus2 = "m:cosphi:bus2", "Power Factor"
      m_tanphi_bus1 = "m:tanphi:bus1", "tan(phi)"
      m_tanphi_bus2 = "m:tanphi:bus2", "tan(phi)"
      m_Psum_bus1 = "m:Psum:bus1", "MW, Total Active Power"
      m_Psum_bus2 = "m:Psum:bus2", "MW, Total Active Power"
      m_Qsum_bus1 = "m:Qsum:bus1", "Mvar, Total Reactive Power"
      m_Qsum_bus2 = "m:Qsum:bus2", "Mvar, Total Reactive Power"
      m_Ssum_bus1 = "m:Ssum:bus1", "MVA, Total Apparent Power"
      m_Ssum_bus2 = "m:Ssum:bus2", "MVA, Total Apparent Power"
      m_cosphisum_bus1 = "m:cosphisum:bus1", "Total Power Factor"
      m_cosphisum_bus2 = "m:cosphisum:bus2", "Total Power Factor"
      m_tanphisum_bus1 = "m:tanphisum:bus1", "Total tan(phi)"
      m_tanphisum_bus2 = "m:tanphisum:bus2", "Total tan(phi)"
      m_Spu_bus1 = "m:Spu:bus1", "MVA/p.u., Apparent Power per p.u. Voltage"
      m_Spu_bus2 = "m:Spu:bus2", "MVA/p.u., Apparent Power per p.u. Voltage"
      m_i1P_bus1 = "m:i1P:bus1", "p.u., Positive-Sequence Active Current"
      m_i1P_bus2 = "m:i1P:bus2", "p.u., Positive-Sequence Active Current"
      m_i1Q_bus1 = "m:i1Q:bus1", "p.u., Positive-Sequence Reactive Current"
      m_i1Q_bus2 = "m:i1Q:bus2", "p.u., Positive-Sequence Reactive Current"
      m_I1P_bus1 = "m:I1P:bus1", "kA, Positive-Sequence Active Current"
      m_I1P_bus2 = "m:I1P:bus2", "kA, Positive-Sequence Active Current"
      m_I1Q_bus1 = "m:I1Q:bus1", "kA, Positive-Sequence Reactive Current"
      m_I1Q_bus2 = "m:I1Q:bus2", "kA, Positive-Sequence Reactive Current"
      m_ur_bus1 = "m:ur:bus1", "p.u., Voltage, Real Part"
      m_ur_bus2 = "m:ur:bus2", "p.u., Voltage, Real Part"
      m_ui_bus1 = "m:ui:bus1", "p.u., Voltage, Imaginary Part"
      m_ui_bus2 = "m:ui:bus2", "p.u., Voltage, Imaginary Part"
      m_u_bus1 = "m:u:bus1", "p.u., Voltage, Magnitude"
      m_u_bus2 = "m:u:bus2", "p.u., Voltage, Magnitude"
      m_ir_bus1 = "m:ir:bus1", "p.u., Current, Real Part"
      m_ir_bus2 = "m:ir:bus2", "p.u., Current, Real Part"
      m_ii_bus1 = "m:ii:bus1", "p.u., Current, Imaginary Part"
      m_ii_bus2 = "m:ii:bus2", "p.u., Current, Imaginary Part"
      m_i_bus1 = "m:i:bus1", "p.u., Current, Magnitude"
      m_i_bus2 = "m:i:bus2", "p.u., Current, Magnitude"
      m_phiui_bus1 = "m:phiui:bus1", "deg, Angle between Voltage and Current"
      m_phiui_bus2 = "m:phiui:bus2", "deg, Angle between Voltage and Current"
      m_phiu1i1_bus1 = "m:phiu1i1:bus1", "deg, Angle between Voltage and Current in positive sequence system"
      m_phiu1i1_bus2 = "m:phiu1i1:bus2", "deg, Angle between Voltage and Current in positive sequence system"
      m_Tfct_bus1 = "m:Tfct:bus1", "s, Fault Clearing Time"
      m_Tfct_bus2 = "m:Tfct:bus2", "s, Fault Clearing Time"
      m_Brkload_bus1 = "m:Brkload:bus1", "%, Breaker Loading"
      m_Brkload_bus2 = "m:Brkload:bus2", "%, Breaker Loading"
      n_dphidP_bus1 = "n:dphidP:bus1", "deg/MW, dphi/dP Sensitivity"
      n_dphidP_bus2 = "n:dphidP:bus2", "deg/MW, dphi/dP Sensitivity"
      n_dphidQ_bus1 = "n:dphidQ:bus1", "deg/Mvar, dphi/dQ Sensitivity"
      n_dphidQ_bus2 = "n:dphidQ:bus2", "deg/Mvar, dphi/dQ Sensitivity"
      n_dphidtap_bus1 = "n:dphidtap:bus1", "deg/tap, dphi/dtap Sensitivity"
      n_dphidtap_bus2 = "n:dphidtap:bus2", "deg/tap, dphi/dtap Sensitivity"
      n_dvdP_bus1 = "n:dvdP:bus1", "p.u./MW, dv/dP Sensitivity"
      n_dvdP_bus2 = "n:dvdP:bus2", "p.u./MW, dv/dP Sensitivity"
      n_dvdQ_bus1 = "n:dvdQ:bus1", "p.u./Mvar, dv/dQ Sensitivity"
      n_dvdQ_bus2 = "n:dvdQ:bus2", "p.u./Mvar, dv/dQ Sensitivity"
      n_dvdtap_bus1 = "n:dvdtap:bus1", "p.u./tap, dv/dtap Sensitivity"
      n_dvdtap_bus2 = "n:dvdtap:bus2", "p.u./tap, dv/dtap Sensitivity"
      n_levecQ_bus1 = "n:levecQ:bus1", "Left Eigenvector of selected Q-Mode"
      n_levecQ_bus2 = "n:levecQ:bus2", "Left Eigenvector of selected Q-Mode"
      n_revecQ_bus1 = "n:revecQ:bus1", "Right Eigenvector of selected Q-Mode"
      n_revecQ_bus2 = "n:revecQ:bus2", "Right Eigenvector of selected Q-Mode"
      n_partQ_bus1 = "n:partQ:bus1", "Participation Factor of selected Q-Mode"
      n_partQ_bus2 = "n:partQ:bus2", "Participation Factor of selected Q-Mode"
      n_ur_bus1 = "n:ur:bus1", "p.u., Voltage, Real Part"
      n_ur_bus2 = "n:ur:bus2", "p.u., Voltage, Real Part"
      n_ui_bus1 = "n:ui:bus1", "p.u., Voltage, Imaginary Part"
      n_ui_bus2 = "n:ui:bus2", "p.u., Voltage, Imaginary Part"
      n_u_bus1 = "n:u:bus1", "p.u., Voltage, Magnitude"
      n_u_bus2 = "n:u:bus2", "p.u., Voltage, Magnitude"
      n_upc_bus1 = "n:upc:bus1", "%, Voltage, Magnitude"
      n_upc_bus2 = "n:upc:bus2", "%, Voltage, Magnitude"
      n_u1_bus1 = "n:u1:bus1", "p.u., Voltage, Magnitude"
      n_u1_bus2 = "n:u1:bus2", "p.u., Voltage, Magnitude"
      n_u1pc_bus1 = "n:u1pc:bus1", "%, Voltage, Magnitude"
      n_u1pc_bus2 = "n:u1pc:bus2", "%, Voltage, Magnitude"
      n_u1r_bus1 = "n:u1r:bus1", "p.u., Positive-Sequence Voltage, Real Part"
      n_u1r_bus2 = "n:u1r:bus2", "p.u., Positive-Sequence Voltage, Real Part"
      n_u1i_bus1 = "n:u1i:bus1", "p.u., Positive-Sequence Voltage, Imaginary Part"
      n_u1i_bus2 = "n:u1i:bus2", "p.u., Positive-Sequence Voltage, Imaginary Part"
      n_U_bus1 = "n:U:bus1", "kV, Line-Ground Voltage, Magnitude"
      n_U_bus2 = "n:U:bus2", "kV, Line-Ground Voltage, Magnitude"
      n_Ul_bus1 = "n:Ul:bus1", "kV, Line-Line Voltage, Magnitude"
      n_Ul_bus2 = "n:Ul:bus2", "kV, Line-Line Voltage, Magnitude"
      n_phiu_bus1 = "n:phiu:bus1", "deg, Voltage, Angle"
      n_phiu_bus2 = "n:phiu:bus2", "deg, Voltage, Angle"
      n_phiurel_bus1 = "n:phiurel:bus1", "deg, Voltage, Relative Angle"
      n_phiurel_bus2 = "n:phiurel:bus2", "deg, Voltage, Relative Angle"
      n_du_bus1 = "n:du:bus1", "%, Voltage Deviation"
      n_du_bus2 = "n:du:bus2", "%, Voltage Deviation"
      n_Pgen_bus1 = "n:Pgen:bus1", "MW, Generation, Active Power"
      n_Pgen_bus2 = "n:Pgen:bus2", "MW, Generation, Active Power"
      n_Qgen_bus1 = "n:Qgen:bus1", "Mvar, Generation, Reactive Power"
      n_Qgen_bus2 = "n:Qgen:bus2", "Mvar, Generation, Reactive Power"
      n_Pmot_bus1 = "n:Pmot:bus1", "MW, Motor Load, Active Power"
      n_Pmot_bus2 = "n:Pmot:bus2", "MW, Motor Load, Active Power"
      n_Qmot_bus1 = "n:Qmot:bus1", "Mvar, Motor Load, Reactive Power"
      n_Qmot_bus2 = "n:Qmot:bus2", "Mvar, Motor Load, Reactive Power"
      n_Pload_bus1 = "n:Pload:bus1", "MW, General Load, Active Power"
      n_Pload_bus2 = "n:Pload:bus2", "MW, General Load, Active Power"
      n_Qload_bus1 = "n:Qload:bus1", "Mvar, General Load, Reactive Power"
      n_Qload_bus2 = "n:Qload:bus2", "Mvar, General Load, Reactive Power"
      n_Pcomp_bus1 = "n:Pcomp:bus1", "MW, Compensation (Losses)"
      n_Pcomp_bus2 = "n:Pcomp:bus2", "MW, Compensation (Losses)"
      n_Qcomp_bus1 = "n:Qcomp:bus1", "Mvar, Compensation"
      n_Qcomp_bus2 = "n:Qcomp:bus2", "Mvar, Compensation"
      n_Pnet_bus1 = "n:Pnet:bus1", "MW, External Networks, Active Power"
      n_Pnet_bus2 = "n:Pnet:bus2", "MW, External Networks, Active Power"
      n_Qnet_bus1 = "n:Qnet:bus1", "Mvar, External Networks, Reactive Power"
      n_Qnet_bus2 = "n:Qnet:bus2", "Mvar, External Networks, Reactive Power"
      n_Pflow_bus1 = "n:Pflow:bus1", "MW, Power Flow, Active Power"
      n_Pflow_bus2 = "n:Pflow:bus2", "MW, Power Flow, Active Power"
      n_Qflow_bus1 = "n:Qflow:bus1", "Mvar, Power Flow, Reactive Power"
      n_Qflow_bus2 = "n:Qflow:bus2", "Mvar, Power Flow, Reactive Power"
      n_Pout_bus1 = "n:Pout:bus1", "MW, Outgoing Flow, Active Power"
      n_Pout_bus2 = "n:Pout:bus2", "MW, Outgoing Flow, Active Power"
      n_Qout_bus1 = "n:Qout:bus1", "Mvar, Outgoing Power, Reactive Power"
      n_Qout_bus2 = "n:Qout:bus2", "Mvar, Outgoing Power, Reactive Power"
      n_Sout_bus1 = "n:Sout:bus1", "MVA, Outgoing Power, Apparent Power"
      n_Sout_bus2 = "n:Sout:bus2", "MVA, Outgoing Power, Apparent Power"
      n_cosphiout_bus1 = "n:cosphiout:bus1", "Outgoing Power, Power Factor"
      n_cosphiout_bus2 = "n:cosphiout:bus2", "Outgoing Power, Power Factor"
      n_Pbalance_bus1 = "n:Pbalance:bus1", "MW, Active Power Balance (=0)"
      n_Pbalance_bus2 = "n:Pbalance:bus2", "MW, Active Power Balance (=0)"
      n_Qbalance_bus1 = "n:Qbalance:bus1", "Mvar, Reactive Power Balance (=0)"
      n_Qbalance_bus2 = "n:Qbalance:bus2", "Mvar, Reactive Power Balance (=0)"
      n_LossPdown_bus1 = "n:LossPdown:bus1", "MW, Losses, downstream"
      n_LossPdown_bus2 = "n:LossPdown:bus2", "MW, Losses, downstream"
      n_LossQdown_bus1 = "n:LossQdown:bus1", "Mvar, Losses, downstream (Reactive Power)"
      n_LossQdown_bus2 = "n:LossQdown:bus2", "Mvar, Losses, downstream (Reactive Power)"
      n_LossPdownload_bus1 = "n:LossPdownload:bus1", "MW, Load losses, downstream"
      n_LossPdownload_bus2 = "n:LossPdownload:bus2", "MW, Load losses, downstream"
      n_LossQdownload_bus1 = "n:LossQdownload:bus1", "Mvar, Load losses, downstream"
      n_LossQdownload_bus2 = "n:LossQdownload:bus2", "Mvar, Load losses, downstream"
      n_LossPdownnoload_bus1 = "n:LossPdownnoload:bus1", "MW, No load losses, downstream"
      n_LossPdownnoload_bus2 = "n:LossPdownnoload:bus2", "MW, No load losses, downstream"
      n_LossQdownnoload_bus1 = "n:LossQdownnoload:bus1", "Mvar, No load losses, downstream"
      n_LossQdownnoload_bus2 = "n:LossQdownnoload:bus2", "Mvar, No load losses, downstream"
      n_du_feed_bus1 = "n:du_feed:bus1", "%, Voltage difference relative to feeder begin"
      n_du_feed_bus2 = "n:du_feed:bus2", "%, Voltage difference relative to feeder begin"
      c_R_on = "c:R_on", "Ohm, On Resistance"
      c_X_on = "c:X_on", "Ohm, On Reactance"
      c_on_off = "c:on_off", "Flag Closed"
      c_Inom = "c:Inom", "kA, Rated Current"
      c_loading = "c:loading", "%, Loading"
      c_maxload = "c:maxload", "%, Max. Loading"
      c_Ploss = "c:Ploss", "MW, Losses (total)"
      c_Qloss = "c:Qloss", "Mvar, Reactive-Losses (total)"
      c_Plossld = "c:Plossld", "MW, Losses (load)"
      c_Qlossld = "c:Qlossld", "Mvar, Reactive-Losses (load)"
      c_Plossnld = "c:Plossnld", "MW, Losses (no load)"
      c_Qlossnld = "c:Qlossnld", "Mvar, Reactive-Losses (no load)"

    class ElmGenstat(Enum):
      _init_ = 'value __doc__'

      m_dPdQ_bus1 = "m:dPdQ:bus1", "MW/Mvar, Branch Sensitivity dPbranch/dQbus"
      m_dQdQ_bus1 = "m:dQdQ:bus1", "Mvar/Mvar, Branch Sensitivity dQbranch/dQbus"
      m_dPdP_bus1 = "m:dPdP:bus1", "MW/MW, Branch Sensitivity dPbranch/dPbus"
      m_dQdP_bus1 = "m:dQdP:bus1", "Mvar/MW, Branch Sensitivity dQbranch/dPbus"
      m_dPdtap_bus1 = "m:dPdtap:bus1", "MW/tap, Branch Sensitivity dPbranch/dtap"
      m_dQdtap_bus1 = "m:dQdtap:bus1", "Mvar/tap, Branch Sensitivity dQbranch/dtap"
      m_LODF_bus1 = "m:LODF:bus1", "%, Line Outage Distribution Factor LODF"
      m_u1r_bus1 = "m:u1r:bus1", "p.u., Positive-Sequence-Voltage, Real Part"
      m_u1i_bus1 = "m:u1i:bus1", "p.u., Positive-Sequence-Voltage, Imaginary Part"
      m_u1_bus1 = "m:u1:bus1", "p.u., Positive-Sequence-Voltage, Magnitude"
      m_U1_bus1 = "m:U1:bus1", "kV, Line-Ground Positive-Sequence-Voltage, Magnitude"
      m_U1l_bus1 = "m:U1l:bus1", "kV, Line-Line Positive-Sequence-Voltage, Magnitude"
      m_phiu1_bus1 = "m:phiu1:bus1", "deg, Positive-Sequence-Voltage, Angle"
      m_inet_bus1 = "m:inet:bus1", "p.u., Current, Magnitude, referred to network"
      m_I_bus1 = "m:I:bus1", "kA, Current, Magnitude"
      m_phii_bus1 = "m:phii:bus1", "deg, Current, Angle"
      m_i1r_bus1 = "m:i1r:bus1", "p.u., Positive-Sequence Current, Real Part"
      m_i1i_bus1 = "m:i1i:bus1", "p.u., Positive-Sequence Current, Imaginary Part"
      m_i1_bus1 = "m:i1:bus1", "p.u., Positive-Sequence Current, Magnitude"
      m_I1_bus1 = "m:I1:bus1", "kA, Positive-Sequence Current, Magnitude"
      m_phii1_bus1 = "m:phii1:bus1", "deg, Positive-Sequence Current, Angle"
      m_P_bus1 = "m:P:bus1", "MW, Active Power"
      m_Q_bus1 = "m:Q:bus1", "Mvar, Reactive Power"
      m_S_bus1 = "m:S:bus1", "MVA, Apparent Power"
      m_cosphi_bus1 = "m:cosphi:bus1", "Power Factor"
      m_tanphi_bus1 = "m:tanphi:bus1", "tan(phi)"
      m_Psum_bus1 = "m:Psum:bus1", "MW, Total Active Power"
      m_Qsum_bus1 = "m:Qsum:bus1", "Mvar, Total Reactive Power"
      m_Ssum_bus1 = "m:Ssum:bus1", "MVA, Total Apparent Power"
      m_cosphisum_bus1 = "m:cosphisum:bus1", "Total Power Factor"
      m_tanphisum_bus1 = "m:tanphisum:bus1", "Total tan(phi)"
      m_Spu_bus1 = "m:Spu:bus1", "MVA/p.u., Apparent Power per p.u. Voltage"
      m_i1P_bus1 = "m:i1P:bus1", "p.u., Positive-Sequence Active Current"
      m_i1Q_bus1 = "m:i1Q:bus1", "p.u., Positive-Sequence Reactive Current"
      m_I1P_bus1 = "m:I1P:bus1", "kA, Positive-Sequence Active Current"
      m_I1Q_bus1 = "m:I1Q:bus1", "kA, Positive-Sequence Reactive Current"
      m_ur_bus1 = "m:ur:bus1", "p.u., Voltage, Real Part"
      m_ui_bus1 = "m:ui:bus1", "p.u., Voltage, Imaginary Part"
      m_u_bus1 = "m:u:bus1", "p.u., Voltage, Magnitude"
      m_ir_bus1 = "m:ir:bus1", "p.u., Current, Real Part"
      m_ii_bus1 = "m:ii:bus1", "p.u., Current, Imaginary Part"
      m_i_bus1 = "m:i:bus1", "p.u., Current, Magnitude"
      m_phiui_bus1 = "m:phiui:bus1", "deg, Angle between Voltage and Current"
      m_phiu1i1_bus1 = "m:phiu1i1:bus1", "deg, Angle between Voltage and Current in positive sequence system"
      m_Tfct_bus1 = "m:Tfct:bus1", "s, Fault Clearing Time"
      m_Brkload_bus1 = "m:Brkload:bus1", "%, Breaker Loading"
      n_dphidP_bus1 = "n:dphidP:bus1", "deg/MW, dphi/dP Sensitivity"
      n_dphidQ_bus1 = "n:dphidQ:bus1", "deg/Mvar, dphi/dQ Sensitivity"
      n_dphidtap_bus1 = "n:dphidtap:bus1", "deg/tap, dphi/dtap Sensitivity"
      n_dvdP_bus1 = "n:dvdP:bus1", "p.u./MW, dv/dP Sensitivity"
      n_dvdQ_bus1 = "n:dvdQ:bus1", "p.u./Mvar, dv/dQ Sensitivity"
      n_dvdtap_bus1 = "n:dvdtap:bus1", "p.u./tap, dv/dtap Sensitivity"
      n_levecQ_bus1 = "n:levecQ:bus1", "Left Eigenvector of selected Q-Mode"
      n_revecQ_bus1 = "n:revecQ:bus1", "Right Eigenvector of selected Q-Mode"
      n_partQ_bus1 = "n:partQ:bus1", "Participation Factor of selected Q-Mode"
      n_ur_bus1 = "n:ur:bus1", "p.u., Voltage, Real Part"
      n_ui_bus1 = "n:ui:bus1", "p.u., Voltage, Imaginary Part"
      n_u_bus1 = "n:u:bus1", "p.u., Voltage, Magnitude"
      n_upc_bus1 = "n:upc:bus1", "%, Voltage, Magnitude"
      n_u1_bus1 = "n:u1:bus1", "p.u., Voltage, Magnitude"
      n_u1pc_bus1 = "n:u1pc:bus1", "%, Voltage, Magnitude"
      n_u1r_bus1 = "n:u1r:bus1", "p.u., Positive-Sequence Voltage, Real Part"
      n_u1i_bus1 = "n:u1i:bus1", "p.u., Positive-Sequence Voltage, Imaginary Part"
      n_U_bus1 = "n:U:bus1", "kV, Line-Ground Voltage, Magnitude"
      n_Ul_bus1 = "n:Ul:bus1", "kV, Line-Line Voltage, Magnitude"
      n_phiu_bus1 = "n:phiu:bus1", "deg, Voltage, Angle"
      n_phiurel_bus1 = "n:phiurel:bus1", "deg, Voltage, Relative Angle"
      n_du_bus1 = "n:du:bus1", "%, Voltage Deviation"
      n_Pgen_bus1 = "n:Pgen:bus1", "MW, Generation, Active Power"
      n_Qgen_bus1 = "n:Qgen:bus1", "Mvar, Generation, Reactive Power"
      n_Pmot_bus1 = "n:Pmot:bus1", "MW, Motor Load, Active Power"
      n_Qmot_bus1 = "n:Qmot:bus1", "Mvar, Motor Load, Reactive Power"
      n_Pload_bus1 = "n:Pload:bus1", "MW, General Load, Active Power"
      n_Qload_bus1 = "n:Qload:bus1", "Mvar, General Load, Reactive Power"
      n_Pcomp_bus1 = "n:Pcomp:bus1", "MW, Compensation (Losses)"
      n_Qcomp_bus1 = "n:Qcomp:bus1", "Mvar, Compensation"
      n_Pnet_bus1 = "n:Pnet:bus1", "MW, External Networks, Active Power"
      n_Qnet_bus1 = "n:Qnet:bus1", "Mvar, External Networks, Reactive Power"
      n_Pflow_bus1 = "n:Pflow:bus1", "MW, Power Flow, Active Power"
      n_Qflow_bus1 = "n:Qflow:bus1", "Mvar, Power Flow, Reactive Power"
      n_Pout_bus1 = "n:Pout:bus1", "MW, Outgoing Flow, Active Power"
      n_Qout_bus1 = "n:Qout:bus1", "Mvar, Outgoing Power, Reactive Power"
      n_Sout_bus1 = "n:Sout:bus1", "MVA, Outgoing Power, Apparent Power"
      n_cosphiout_bus1 = "n:cosphiout:bus1", "Outgoing Power, Power Factor"
      n_Pbalance_bus1 = "n:Pbalance:bus1", "MW, Active Power Balance (=0)"
      n_Qbalance_bus1 = "n:Qbalance:bus1", "Mvar, Reactive Power Balance (=0)"
      n_LossPdown_bus1 = "n:LossPdown:bus1", "MW, Losses, downstream"
      n_LossQdown_bus1 = "n:LossQdown:bus1", "Mvar, Losses, downstream (Reactive Power)"
      n_LossPdownload_bus1 = "n:LossPdownload:bus1", "MW, Load losses, downstream"
      n_LossQdownload_bus1 = "n:LossQdownload:bus1", "Mvar, Load losses, downstream"
      n_LossPdownnoload_bus1 = "n:LossPdownnoload:bus1", "MW, No load losses, downstream"
      n_LossQdownnoload_bus1 = "n:LossQdownnoload:bus1", "Mvar, No load losses, downstream"
      n_du_feed_bus1 = "n:du_feed:bus1", "%, Voltage difference relative to feeder begin"
      s_pset = "s:pset", "MW, IN, Active Power Setpoint, continuous, double"
      s_qset = "s:qset", "Mvar, IN, Reactive Power Setpoint continuous, double"
      s_dFin = "s:dFin", "Hz, IN, Frequency Deviation, continuous, double"
      c_p = "c:p", "MW, Total Active Power"
      c_q = "c:q", "Mvar, Total Reactive Power"
      c_sgnn = "c:sgnn", "MVA, Total rated apparent power (ngnum*sgn)"
      c_cosn = "c:cosn", "Nominal-Power factor"
      c_Pnom = "c:Pnom", "MW, Total Nominal Active power"
      c_c2 = "c:c2", "Voltage Coefficient (uknom/ugn)"
      c_uini_r = "c:uini:r", "p.u., Voltage Setpoint for PV and Slack, Real Part"
      c_uini_i = "c:uini:i", "p.u., Voltage Setpoint for PV and Slack, Imaginary Part"
      c_uini_phi = "c:uini:phi", "deg, Voltage Setpoint for PV and Slack, Angle"
      c_uini_mag = "c:uini:mag", "p.u., Voltage Setpoint for PV and Slack, Magnitude"
      c_uini_phirad = "c:uini:phirad", "rad, Voltage Setpoint for PV and Slack, Angle"
      c_phiini = "c:phiini", "Initial Angle"
      c_loading = "c:loading", "%, Loading"
      c_iPowerOn = "c:iPowerOn", "Flag"
      c_Qmin = "c:Qmin", "Mvar, Lower limit of reactive power"
      c_Qmax = "c:Qmax", "Mvar, Upper limit of reactive power"
      c_Qsparelead = "c:Qsparelead", "Mvar, Q Spare Lead"
      c_Qsparelag = "c:Qsparelag", "Mvar, Q Spare Lag"
      c_i_qlim = "c:i_qlim", "Reactive power limit reached"
      c_i_vlim = "c:i_vlim", "Voltage setpoint limit reached"
      c_Pmin = "c:Pmin", "MW, Lower limit of active power"
      c_Pmax = "c:Pmax", "MW, Upper limit of active power"
      c_i_plim = "c:i_plim", "Active power limit reached"
      c_sm_r = "c:sm:r", "MW, Complex Power, Real Part"
      c_sm_i = "c:sm:i", "Mvar, Complex Power, Imaginary Part"
      c_sm_phi = "c:sm:phi", "deg, Complex Power, Angle"
      c_sm_mag = "c:sm:mag", "MVA, Complex Power, Magnitude"
      c_sm_phirad = "c:sm:phirad", "rad, Complex Power, Angle"
      c_u1_r = "c:u1:r", "p.u., Internal, Real Part"
      c_u1_i = "c:u1:i", "p.u., Internal, Imaginary Part"
      c_u1_phi = "c:u1:phi", "deg, Internal, Angle"
      c_u1_mag = "c:u1:mag", "p.u., Internal, Magnitude"
      c_u1_phirad = "c:u1:phirad", "rad, Internal, Angle"
      c_iRefElement = "c:iRefElement", "Reference Element"
      c_Snom = "c:Snom", "MVA, Nom. Apparent Power (1 mach.)"
      c_Nnom = "c:Nnom", "Number of parallel units"
      c_Kp = "c:Kp", "p.u., Active Power Participation"
      c_Kq = "c:Kq", "p.u., Reactive Power Participation"
      c_Kpf = "c:Kpf", "MW/Hz, Frequency Bias"
      c_zonewscale = "c:zonewscale", "Wind generation zone scaling factor"
      c_Pmism = "c:Pmism", "MW, Active Power Mismatch"
      c_nCtrlIter = "c:nCtrlIter", "No. of CTRL-iterations"
      c_Ploss = "c:Ploss", "MW, Losses (total)"
      c_Qloss = "c:Qloss", "Mvar, Reactive-Losses (total)"
      c_Plossld = "c:Plossld", "MW, Losses (load)"
      c_Qlossld = "c:Qlossld", "Mvar, Reactive-Losses (load)"
      c_Plossnld = "c:Plossnld", "MW, Losses (no load)"
      c_Qlossnld = "c:Qlossnld", "Mvar, Reactive-Losses (no load)"

    class ElmIac(Enum):
      _init_ = 'value __doc__'

      m_dPdQ_bus1 = "m:dPdQ:bus1", "MW/Mvar, Branch Sensitivity dPbranch/dQbus"
      m_dQdQ_bus1 = "m:dQdQ:bus1", "Mvar/Mvar, Branch Sensitivity dQbranch/dQbus"
      m_dPdP_bus1 = "m:dPdP:bus1", "MW/MW, Branch Sensitivity dPbranch/dPbus"
      m_dQdP_bus1 = "m:dQdP:bus1", "Mvar/MW, Branch Sensitivity dQbranch/dPbus"
      m_dPdtap_bus1 = "m:dPdtap:bus1", "MW/tap, Branch Sensitivity dPbranch/dtap"
      m_dQdtap_bus1 = "m:dQdtap:bus1", "Mvar/tap, Branch Sensitivity dQbranch/dtap"
      m_LODF_bus1 = "m:LODF:bus1", "%, Line Outage Distribution Factor LODF"
      m_u1r_bus1 = "m:u1r:bus1", "p.u., Positive-Sequence-Voltage, Real Part"
      m_u1i_bus1 = "m:u1i:bus1", "p.u., Positive-Sequence-Voltage, Imaginary Part"
      m_u1_bus1 = "m:u1:bus1", "p.u., Positive-Sequence-Voltage, Magnitude"
      m_U1_bus1 = "m:U1:bus1", "kV, Line-Ground Positive-Sequence-Voltage, Magnitude"
      m_U1l_bus1 = "m:U1l:bus1", "kV, Line-Line Positive-Sequence-Voltage, Magnitude"
      m_phiu1_bus1 = "m:phiu1:bus1", "deg, Positive-Sequence-Voltage, Angle"
      m_inet_bus1 = "m:inet:bus1", "p.u., Current, Magnitude, referred to network"
      m_I_bus1 = "m:I:bus1", "kA, Current, Magnitude"
      m_phii_bus1 = "m:phii:bus1", "deg, Current, Angle"
      m_i1r_bus1 = "m:i1r:bus1", "p.u., Positive-Sequence Current, Real Part"
      m_i1i_bus1 = "m:i1i:bus1", "p.u., Positive-Sequence Current, Imaginary Part"
      m_i1_bus1 = "m:i1:bus1", "p.u., Positive-Sequence Current, Magnitude"
      m_I1_bus1 = "m:I1:bus1", "kA, Positive-Sequence Current, Magnitude"
      m_phii1_bus1 = "m:phii1:bus1", "deg, Positive-Sequence Current, Angle"
      m_P_bus1 = "m:P:bus1", "MW, Active Power"
      m_Q_bus1 = "m:Q:bus1", "Mvar, Reactive Power"
      m_S_bus1 = "m:S:bus1", "MVA, Apparent Power"
      m_cosphi_bus1 = "m:cosphi:bus1", "Power Factor"
      m_tanphi_bus1 = "m:tanphi:bus1", "tan(phi)"
      m_Psum_bus1 = "m:Psum:bus1", "MW, Total Active Power"
      m_Qsum_bus1 = "m:Qsum:bus1", "Mvar, Total Reactive Power"
      m_Ssum_bus1 = "m:Ssum:bus1", "MVA, Total Apparent Power"
      m_cosphisum_bus1 = "m:cosphisum:bus1", "Total Power Factor"
      m_tanphisum_bus1 = "m:tanphisum:bus1", "Total tan(phi)"
      m_Spu_bus1 = "m:Spu:bus1", "MVA/p.u., Apparent Power per p.u. Voltage"
      m_i1P_bus1 = "m:i1P:bus1", "p.u., Positive-Sequence Active Current"
      m_i1Q_bus1 = "m:i1Q:bus1", "p.u., Positive-Sequence Reactive Current"
      m_I1P_bus1 = "m:I1P:bus1", "kA, Positive-Sequence Active Current"
      m_I1Q_bus1 = "m:I1Q:bus1", "kA, Positive-Sequence Reactive Current"
      m_ur_bus1 = "m:ur:bus1", "p.u., Voltage, Real Part"
      m_ui_bus1 = "m:ui:bus1", "p.u., Voltage, Imaginary Part"
      m_u_bus1 = "m:u:bus1", "p.u., Voltage, Magnitude"
      m_ir_bus1 = "m:ir:bus1", "p.u., Current, Real Part"
      m_ii_bus1 = "m:ii:bus1", "p.u., Current, Imaginary Part"
      m_i_bus1 = "m:i:bus1", "p.u., Current, Magnitude"
      m_phiui_bus1 = "m:phiui:bus1", "deg, Angle between Voltage and Current"
      m_phiu1i1_bus1 = "m:phiu1i1:bus1", "deg, Angle between Voltage and Current in positive sequence system"
      m_Tfct_bus1 = "m:Tfct:bus1", "s, Fault Clearing Time"
      m_Brkload_bus1 = "m:Brkload:bus1", "%, Breaker Loading"
      n_dphidP_bus1 = "n:dphidP:bus1", "deg/MW, dphi/dP Sensitivity"
      n_dphidQ_bus1 = "n:dphidQ:bus1", "deg/Mvar, dphi/dQ Sensitivity"
      n_dphidtap_bus1 = "n:dphidtap:bus1", "deg/tap, dphi/dtap Sensitivity"
      n_dvdP_bus1 = "n:dvdP:bus1", "p.u./MW, dv/dP Sensitivity"
      n_dvdQ_bus1 = "n:dvdQ:bus1", "p.u./Mvar, dv/dQ Sensitivity"
      n_dvdtap_bus1 = "n:dvdtap:bus1", "p.u./tap, dv/dtap Sensitivity"
      n_levecQ_bus1 = "n:levecQ:bus1", "Left Eigenvector of selected Q-Mode"
      n_revecQ_bus1 = "n:revecQ:bus1", "Right Eigenvector of selected Q-Mode"
      n_partQ_bus1 = "n:partQ:bus1", "Participation Factor of selected Q-Mode"
      n_ur_bus1 = "n:ur:bus1", "p.u., Voltage, Real Part"
      n_ui_bus1 = "n:ui:bus1", "p.u., Voltage, Imaginary Part"
      n_u_bus1 = "n:u:bus1", "p.u., Voltage, Magnitude"
      n_upc_bus1 = "n:upc:bus1", "%, Voltage, Magnitude"
      n_u1_bus1 = "n:u1:bus1", "p.u., Voltage, Magnitude"
      n_u1pc_bus1 = "n:u1pc:bus1", "%, Voltage, Magnitude"
      n_u1r_bus1 = "n:u1r:bus1", "p.u., Positive-Sequence Voltage, Real Part"
      n_u1i_bus1 = "n:u1i:bus1", "p.u., Positive-Sequence Voltage, Imaginary Part"
      n_U_bus1 = "n:U:bus1", "kV, Line-Ground Voltage, Magnitude"
      n_Ul_bus1 = "n:Ul:bus1", "kV, Line-Line Voltage, Magnitude"
      n_phiu_bus1 = "n:phiu:bus1", "deg, Voltage, Angle"
      n_phiurel_bus1 = "n:phiurel:bus1", "deg, Voltage, Relative Angle"
      n_du_bus1 = "n:du:bus1", "%, Voltage Deviation"
      n_Pgen_bus1 = "n:Pgen:bus1", "MW, Generation, Active Power"
      n_Qgen_bus1 = "n:Qgen:bus1", "Mvar, Generation, Reactive Power"
      n_Pmot_bus1 = "n:Pmot:bus1", "MW, Motor Load, Active Power"
      n_Qmot_bus1 = "n:Qmot:bus1", "Mvar, Motor Load, Reactive Power"
      n_Pload_bus1 = "n:Pload:bus1", "MW, General Load, Active Power"
      n_Qload_bus1 = "n:Qload:bus1", "Mvar, General Load, Reactive Power"
      n_Pcomp_bus1 = "n:Pcomp:bus1", "MW, Compensation (Losses)"
      n_Qcomp_bus1 = "n:Qcomp:bus1", "Mvar, Compensation"
      n_Pnet_bus1 = "n:Pnet:bus1", "MW, External Networks, Active Power"
      n_Qnet_bus1 = "n:Qnet:bus1", "Mvar, External Networks, Reactive Power"
      n_Pflow_bus1 = "n:Pflow:bus1", "MW, Power Flow, Active Power"
      n_Qflow_bus1 = "n:Qflow:bus1", "Mvar, Power Flow, Reactive Power"
      n_Pout_bus1 = "n:Pout:bus1", "MW, Outgoing Flow, Active Power"
      n_Qout_bus1 = "n:Qout:bus1", "Mvar, Outgoing Power, Reactive Power"
      n_Sout_bus1 = "n:Sout:bus1", "MVA, Outgoing Power, Apparent Power"
      n_cosphiout_bus1 = "n:cosphiout:bus1", "Outgoing Power, Power Factor"
      n_Pbalance_bus1 = "n:Pbalance:bus1", "MW, Active Power Balance (=0)"
      n_Qbalance_bus1 = "n:Qbalance:bus1", "Mvar, Reactive Power Balance (=0)"
      n_LossPdown_bus1 = "n:LossPdown:bus1", "MW, Losses, downstream"
      n_LossQdown_bus1 = "n:LossQdown:bus1", "Mvar, Losses, downstream (Reactive Power)"
      n_LossPdownload_bus1 = "n:LossPdownload:bus1", "MW, Load losses, downstream"
      n_LossQdownload_bus1 = "n:LossQdownload:bus1", "Mvar, Load losses, downstream"
      n_LossPdownnoload_bus1 = "n:LossPdownnoload:bus1", "MW, No load losses, downstream"
      n_LossQdownnoload_bus1 = "n:LossQdownnoload:bus1", "Mvar, No load losses, downstream"
      n_du_feed_bus1 = "n:du_feed:bus1", "%, Voltage difference relative to feeder begin"
      s_i0 = "s:i0", "p.u., IN, Current-Input, continuous, double"
      s_phiui = "s:phiui", "rad, IN, Angle-Input (u,i), continuous, double"
      c_Ir = "c:Ir", "A, Nominal Current"
      c_isetp = "c:isetp", "p.u., Current Setpoint"
      c_cosini = "c:cosini", "Power Factor"
      c_Y1_r = "c:Y1:r", "S, Internal Admittance, Real Part"
      c_Y1_i = "c:Y1:i", "S, Internal Admittance, Imaginary Part"
      c_Y1_phi = "c:Y1:phi", "deg, Internal Admittance, Angle"
      c_Y1_mag = "c:Y1:mag", "S, Internal Admittance, Magnitude"
      c_Y1_phirad = "c:Y1:phirad", "rad, Internal Admittance, Angle"
      c_Ploss = "c:Ploss", "MW, Losses (total)"
      c_Qloss = "c:Qloss", "Mvar, Reactive-Losses (total)"
      c_Plossld = "c:Plossld", "MW, Losses (load)"
      c_Qlossld = "c:Qlossld", "Mvar, Reactive-Losses (load)"
      c_Plossnld = "c:Plossnld", "MW, Losses (no load)"
      c_Qlossnld = "c:Qlossnld", "Mvar, Reactive-Losses (no load)"

    class ElmLne(Enum):
      _init_ = 'value __doc__'

      m_dPdQ_bus1 = "m:dPdQ:bus1", "MW/Mvar, Branch Sensitivity dPbranch/dQbus"
      m_dPdQ_bus2 = "m:dPdQ:bus2", "MW/Mvar, Branch Sensitivity dPbranch/dQbus"
      m_dQdQ_bus1 = "m:dQdQ:bus1", "Mvar/Mvar, Branch Sensitivity dQbranch/dQbus"
      m_dQdQ_bus2 = "m:dQdQ:bus2", "Mvar/Mvar, Branch Sensitivity dQbranch/dQbus"
      m_dPdP_bus1 = "m:dPdP:bus1", "MW/MW, Branch Sensitivity dPbranch/dPbus"
      m_dPdP_bus2 = "m:dPdP:bus2", "MW/MW, Branch Sensitivity dPbranch/dPbus"
      m_dQdP_bus1 = "m:dQdP:bus1", "Mvar/MW, Branch Sensitivity dQbranch/dPbus"
      m_dQdP_bus2 = "m:dQdP:bus2", "Mvar/MW, Branch Sensitivity dQbranch/dPbus"
      m_dPdtap_bus1 = "m:dPdtap:bus1", "MW/tap, Branch Sensitivity dPbranch/dtap"
      m_dPdtap_bus2 = "m:dPdtap:bus2", "MW/tap, Branch Sensitivity dPbranch/dtap"
      m_dQdtap_bus1 = "m:dQdtap:bus1", "Mvar/tap, Branch Sensitivity dQbranch/dtap"
      m_dQdtap_bus2 = "m:dQdtap:bus2", "Mvar/tap, Branch Sensitivity dQbranch/dtap"
      m_LODF_bus1 = "m:LODF:bus1", "%, Line Outage Distribution Factor LODF"
      m_LODF_bus2 = "m:LODF:bus2", "%, Line Outage Distribution Factor LODF"
      m_u1r_bus1 = "m:u1r:bus1", "p.u., Positive-Sequence-Voltage, Real Part"
      m_u1r_bus2 = "m:u1r:bus2", "p.u., Positive-Sequence-Voltage, Real Part"
      m_u1i_bus1 = "m:u1i:bus1", "p.u., Positive-Sequence-Voltage, Imaginary Part"
      m_u1i_bus2 = "m:u1i:bus2", "p.u., Positive-Sequence-Voltage, Imaginary Part"
      m_u1_bus1 = "m:u1:bus1", "p.u., Positive-Sequence-Voltage, Magnitude"
      m_u1_bus2 = "m:u1:bus2", "p.u., Positive-Sequence-Voltage, Magnitude"
      m_U1_bus1 = "m:U1:bus1", "kV, Line-Ground Positive-Sequence-Voltage, Magnitude"
      m_U1_bus2 = "m:U1:bus2", "kV, Line-Ground Positive-Sequence-Voltage, Magnitude"
      m_U1l_bus1 = "m:U1l:bus1", "kV, Line-Line Positive-Sequence-Voltage, Magnitude"
      m_U1l_bus2 = "m:U1l:bus2", "kV, Line-Line Positive-Sequence-Voltage, Magnitude"
      m_phiu1_bus1 = "m:phiu1:bus1", "deg, Positive-Sequence-Voltage, Angle"
      m_phiu1_bus2 = "m:phiu1:bus2", "deg, Positive-Sequence-Voltage, Angle"
      m_inet_bus1 = "m:inet:bus1", "p.u., Current, Magnitude, referred to network"
      m_inet_bus2 = "m:inet:bus2", "p.u., Current, Magnitude, referred to network"
      m_I_bus1 = "m:I:bus1", "kA, Current, Magnitude"
      m_I_bus2 = "m:I:bus2", "kA, Current, Magnitude"
      m_phii_bus1 = "m:phii:bus1", "deg, Current, Angle"
      m_phii_bus2 = "m:phii:bus2", "deg, Current, Angle"
      m_i1r_bus1 = "m:i1r:bus1", "p.u., Positive-Sequence Current, Real Part"
      m_i1r_bus2 = "m:i1r:bus2", "p.u., Positive-Sequence Current, Real Part"
      m_i1i_bus1 = "m:i1i:bus1", "p.u., Positive-Sequence Current, Imaginary Part"
      m_i1i_bus2 = "m:i1i:bus2", "p.u., Positive-Sequence Current, Imaginary Part"
      m_i1_bus1 = "m:i1:bus1", "p.u., Positive-Sequence Current, Magnitude"
      m_i1_bus2 = "m:i1:bus2", "p.u., Positive-Sequence Current, Magnitude"
      m_I1_bus1 = "m:I1:bus1", "kA, Positive-Sequence Current, Magnitude"
      m_I1_bus2 = "m:I1:bus2", "kA, Positive-Sequence Current, Magnitude"
      m_phii1_bus1 = "m:phii1:bus1", "deg, Positive-Sequence Current, Angle"
      m_phii1_bus2 = "m:phii1:bus2", "deg, Positive-Sequence Current, Angle"
      m_P_bus1 = "m:P:bus1", "MW, Active Power"
      m_P_bus2 = "m:P:bus2", "MW, Active Power"
      m_Q_bus1 = "m:Q:bus1", "Mvar, Reactive Power"
      m_Q_bus2 = "m:Q:bus2", "Mvar, Reactive Power"
      m_S_bus1 = "m:S:bus1", "MVA, Apparent Power"
      m_S_bus2 = "m:S:bus2", "MVA, Apparent Power"
      m_cosphi_bus1 = "m:cosphi:bus1", "Power Factor"
      m_cosphi_bus2 = "m:cosphi:bus2", "Power Factor"
      m_tanphi_bus1 = "m:tanphi:bus1", "tan(phi)"
      m_tanphi_bus2 = "m:tanphi:bus2", "tan(phi)"
      m_Psum_bus1 = "m:Psum:bus1", "MW, Total Active Power"
      m_Psum_bus2 = "m:Psum:bus2", "MW, Total Active Power"
      m_Qsum_bus1 = "m:Qsum:bus1", "Mvar, Total Reactive Power"
      m_Qsum_bus2 = "m:Qsum:bus2", "Mvar, Total Reactive Power"
      m_Ssum_bus1 = "m:Ssum:bus1", "MVA, Total Apparent Power"
      m_Ssum_bus2 = "m:Ssum:bus2", "MVA, Total Apparent Power"
      m_cosphisum_bus1 = "m:cosphisum:bus1", "Total Power Factor"
      m_cosphisum_bus2 = "m:cosphisum:bus2", "Total Power Factor"
      m_tanphisum_bus1 = "m:tanphisum:bus1", "Total tan(phi)"
      m_tanphisum_bus2 = "m:tanphisum:bus2", "Total tan(phi)"
      m_Spu_bus1 = "m:Spu:bus1", "MVA/p.u., Apparent Power per p.u. Voltage"
      m_Spu_bus2 = "m:Spu:bus2", "MVA/p.u., Apparent Power per p.u. Voltage"
      m_i1P_bus1 = "m:i1P:bus1", "p.u., Positive-Sequence Active Current"
      m_i1P_bus2 = "m:i1P:bus2", "p.u., Positive-Sequence Active Current"
      m_i1Q_bus1 = "m:i1Q:bus1", "p.u., Positive-Sequence Reactive Current"
      m_i1Q_bus2 = "m:i1Q:bus2", "p.u., Positive-Sequence Reactive Current"
      m_I1P_bus1 = "m:I1P:bus1", "kA, Positive-Sequence Active Current"
      m_I1P_bus2 = "m:I1P:bus2", "kA, Positive-Sequence Active Current"
      m_I1Q_bus1 = "m:I1Q:bus1", "kA, Positive-Sequence Reactive Current"
      m_I1Q_bus2 = "m:I1Q:bus2", "kA, Positive-Sequence Reactive Current"
      m_ur_bus1 = "m:ur:bus1", "p.u., Voltage, Real Part"
      m_ur_bus2 = "m:ur:bus2", "p.u., Voltage, Real Part"
      m_ui_bus1 = "m:ui:bus1", "p.u., Voltage, Imaginary Part"
      m_ui_bus2 = "m:ui:bus2", "p.u., Voltage, Imaginary Part"
      m_u_bus1 = "m:u:bus1", "p.u., Voltage, Magnitude"
      m_u_bus2 = "m:u:bus2", "p.u., Voltage, Magnitude"
      m_ir_bus1 = "m:ir:bus1", "p.u., Current, Real Part"
      m_ir_bus2 = "m:ir:bus2", "p.u., Current, Real Part"
      m_ii_bus1 = "m:ii:bus1", "p.u., Current, Imaginary Part"
      m_ii_bus2 = "m:ii:bus2", "p.u., Current, Imaginary Part"
      m_i_bus1 = "m:i:bus1", "p.u., Current, Magnitude"
      m_i_bus2 = "m:i:bus2", "p.u., Current, Magnitude"
      m_phiui_bus1 = "m:phiui:bus1", "deg, Angle between Voltage and Current"
      m_phiui_bus2 = "m:phiui:bus2", "deg, Angle between Voltage and Current"
      m_phiu1i1_bus1 = "m:phiu1i1:bus1", "deg, Angle between Voltage and Current in positive sequence system"
      m_phiu1i1_bus2 = "m:phiu1i1:bus2", "deg, Angle between Voltage and Current in positive sequence system"
      m_Tfct_bus1 = "m:Tfct:bus1", "s, Fault Clearing Time"
      m_Tfct_bus2 = "m:Tfct:bus2", "s, Fault Clearing Time"
      m_Brkload_bus1 = "m:Brkload:bus1", "%, Breaker Loading"
      m_Brkload_bus2 = "m:Brkload:bus2", "%, Breaker Loading"
      n_dphidP_bus1 = "n:dphidP:bus1", "deg/MW, dphi/dP Sensitivity"
      n_dphidP_bus2 = "n:dphidP:bus2", "deg/MW, dphi/dP Sensitivity"
      n_dphidQ_bus1 = "n:dphidQ:bus1", "deg/Mvar, dphi/dQ Sensitivity"
      n_dphidQ_bus2 = "n:dphidQ:bus2", "deg/Mvar, dphi/dQ Sensitivity"
      n_dphidtap_bus1 = "n:dphidtap:bus1", "deg/tap, dphi/dtap Sensitivity"
      n_dphidtap_bus2 = "n:dphidtap:bus2", "deg/tap, dphi/dtap Sensitivity"
      n_dvdP_bus1 = "n:dvdP:bus1", "p.u./MW, dv/dP Sensitivity"
      n_dvdP_bus2 = "n:dvdP:bus2", "p.u./MW, dv/dP Sensitivity"
      n_dvdQ_bus1 = "n:dvdQ:bus1", "p.u./Mvar, dv/dQ Sensitivity"
      n_dvdQ_bus2 = "n:dvdQ:bus2", "p.u./Mvar, dv/dQ Sensitivity"
      n_dvdtap_bus1 = "n:dvdtap:bus1", "p.u./tap, dv/dtap Sensitivity"
      n_dvdtap_bus2 = "n:dvdtap:bus2", "p.u./tap, dv/dtap Sensitivity"
      n_levecQ_bus1 = "n:levecQ:bus1", "Left Eigenvector of selected Q-Mode"
      n_levecQ_bus2 = "n:levecQ:bus2", "Left Eigenvector of selected Q-Mode"
      n_revecQ_bus1 = "n:revecQ:bus1", "Right Eigenvector of selected Q-Mode"
      n_revecQ_bus2 = "n:revecQ:bus2", "Right Eigenvector of selected Q-Mode"
      n_partQ_bus1 = "n:partQ:bus1", "Participation Factor of selected Q-Mode"
      n_partQ_bus2 = "n:partQ:bus2", "Participation Factor of selected Q-Mode"
      n_ur_bus1 = "n:ur:bus1", "p.u., Voltage, Real Part"
      n_ur_bus2 = "n:ur:bus2", "p.u., Voltage, Real Part"
      n_ui_bus1 = "n:ui:bus1", "p.u., Voltage, Imaginary Part"
      n_ui_bus2 = "n:ui:bus2", "p.u., Voltage, Imaginary Part"
      n_u_bus1 = "n:u:bus1", "p.u., Voltage, Magnitude"
      n_u_bus2 = "n:u:bus2", "p.u., Voltage, Magnitude"
      n_upc_bus1 = "n:upc:bus1", "%, Voltage, Magnitude"
      n_upc_bus2 = "n:upc:bus2", "%, Voltage, Magnitude"
      n_u1_bus1 = "n:u1:bus1", "p.u., Voltage, Magnitude"
      n_u1_bus2 = "n:u1:bus2", "p.u., Voltage, Magnitude"
      n_u1pc_bus1 = "n:u1pc:bus1", "%, Voltage, Magnitude"
      n_u1pc_bus2 = "n:u1pc:bus2", "%, Voltage, Magnitude"
      n_u1r_bus1 = "n:u1r:bus1", "p.u., Positive-Sequence Voltage, Real Part"
      n_u1r_bus2 = "n:u1r:bus2", "p.u., Positive-Sequence Voltage, Real Part"
      n_u1i_bus1 = "n:u1i:bus1", "p.u., Positive-Sequence Voltage, Imaginary Part"
      n_u1i_bus2 = "n:u1i:bus2", "p.u., Positive-Sequence Voltage, Imaginary Part"
      n_U_bus1 = "n:U:bus1", "kV, Line-Ground Voltage, Magnitude"
      n_U_bus2 = "n:U:bus2", "kV, Line-Ground Voltage, Magnitude"
      n_Ul_bus1 = "n:Ul:bus1", "kV, Line-Line Voltage, Magnitude"
      n_Ul_bus2 = "n:Ul:bus2", "kV, Line-Line Voltage, Magnitude"
      n_phiu_bus1 = "n:phiu:bus1", "deg, Voltage, Angle"
      n_phiu_bus2 = "n:phiu:bus2", "deg, Voltage, Angle"
      n_phiurel_bus1 = "n:phiurel:bus1", "deg, Voltage, Relative Angle"
      n_phiurel_bus2 = "n:phiurel:bus2", "deg, Voltage, Relative Angle"
      n_du_bus1 = "n:du:bus1", "%, Voltage Deviation"
      n_du_bus2 = "n:du:bus2", "%, Voltage Deviation"
      n_Pgen_bus1 = "n:Pgen:bus1", "MW, Generation, Active Power"
      n_Pgen_bus2 = "n:Pgen:bus2", "MW, Generation, Active Power"
      n_Qgen_bus1 = "n:Qgen:bus1", "Mvar, Generation, Reactive Power"
      n_Qgen_bus2 = "n:Qgen:bus2", "Mvar, Generation, Reactive Power"
      n_Pmot_bus1 = "n:Pmot:bus1", "MW, Motor Load, Active Power"
      n_Pmot_bus2 = "n:Pmot:bus2", "MW, Motor Load, Active Power"
      n_Qmot_bus1 = "n:Qmot:bus1", "Mvar, Motor Load, Reactive Power"
      n_Qmot_bus2 = "n:Qmot:bus2", "Mvar, Motor Load, Reactive Power"
      n_Pload_bus1 = "n:Pload:bus1", "MW, General Load, Active Power"
      n_Pload_bus2 = "n:Pload:bus2", "MW, General Load, Active Power"
      n_Qload_bus1 = "n:Qload:bus1", "Mvar, General Load, Reactive Power"
      n_Qload_bus2 = "n:Qload:bus2", "Mvar, General Load, Reactive Power"
      n_Pcomp_bus1 = "n:Pcomp:bus1", "MW, Compensation (Losses)"
      n_Pcomp_bus2 = "n:Pcomp:bus2", "MW, Compensation (Losses)"
      n_Qcomp_bus1 = "n:Qcomp:bus1", "Mvar, Compensation"
      n_Qcomp_bus2 = "n:Qcomp:bus2", "Mvar, Compensation"
      n_Pnet_bus1 = "n:Pnet:bus1", "MW, External Networks, Active Power"
      n_Pnet_bus2 = "n:Pnet:bus2", "MW, External Networks, Active Power"
      n_Qnet_bus1 = "n:Qnet:bus1", "Mvar, External Networks, Reactive Power"
      n_Qnet_bus2 = "n:Qnet:bus2", "Mvar, External Networks, Reactive Power"
      n_Pflow_bus1 = "n:Pflow:bus1", "MW, Power Flow, Active Power"
      n_Pflow_bus2 = "n:Pflow:bus2", "MW, Power Flow, Active Power"
      n_Qflow_bus1 = "n:Qflow:bus1", "Mvar, Power Flow, Reactive Power"
      n_Qflow_bus2 = "n:Qflow:bus2", "Mvar, Power Flow, Reactive Power"
      n_Pout_bus1 = "n:Pout:bus1", "MW, Outgoing Flow, Active Power"
      n_Pout_bus2 = "n:Pout:bus2", "MW, Outgoing Flow, Active Power"
      n_Qout_bus1 = "n:Qout:bus1", "Mvar, Outgoing Power, Reactive Power"
      n_Qout_bus2 = "n:Qout:bus2", "Mvar, Outgoing Power, Reactive Power"
      n_Sout_bus1 = "n:Sout:bus1", "MVA, Outgoing Power, Apparent Power"
      n_Sout_bus2 = "n:Sout:bus2", "MVA, Outgoing Power, Apparent Power"
      n_cosphiout_bus1 = "n:cosphiout:bus1", "Outgoing Power, Power Factor"
      n_cosphiout_bus2 = "n:cosphiout:bus2", "Outgoing Power, Power Factor"
      n_Pbalance_bus1 = "n:Pbalance:bus1", "MW, Active Power Balance (=0)"
      n_Pbalance_bus2 = "n:Pbalance:bus2", "MW, Active Power Balance (=0)"
      n_Qbalance_bus1 = "n:Qbalance:bus1", "Mvar, Reactive Power Balance (=0)"
      n_Qbalance_bus2 = "n:Qbalance:bus2", "Mvar, Reactive Power Balance (=0)"
      n_LossPdown_bus1 = "n:LossPdown:bus1", "MW, Losses, downstream"
      n_LossPdown_bus2 = "n:LossPdown:bus2", "MW, Losses, downstream"
      n_LossQdown_bus1 = "n:LossQdown:bus1", "Mvar, Losses, downstream (Reactive Power)"
      n_LossQdown_bus2 = "n:LossQdown:bus2", "Mvar, Losses, downstream (Reactive Power)"
      n_LossPdownload_bus1 = "n:LossPdownload:bus1", "MW, Load losses, downstream"
      n_LossPdownload_bus2 = "n:LossPdownload:bus2", "MW, Load losses, downstream"
      n_LossQdownload_bus1 = "n:LossQdownload:bus1", "Mvar, Load losses, downstream"
      n_LossQdownload_bus2 = "n:LossQdownload:bus2", "Mvar, Load losses, downstream"
      n_LossPdownnoload_bus1 = "n:LossPdownnoload:bus1", "MW, No load losses, downstream"
      n_LossPdownnoload_bus2 = "n:LossPdownnoload:bus2", "MW, No load losses, downstream"
      n_LossQdownnoload_bus1 = "n:LossQdownnoload:bus1", "Mvar, No load losses, downstream"
      n_LossQdownnoload_bus2 = "n:LossQdownnoload:bus2", "Mvar, No load losses, downstream"
      n_du_feed_bus1 = "n:du_feed:bus1", "%, Voltage difference relative to feeder begin"
      n_du_feed_bus2 = "n:du_feed:bus2", "%, Voltage difference relative to feeder begin"
      c_dLoadingdP = "c:dLoadingdP", "%/MW, Branch Loading Sensitivity: dLbranch/dPbus"
      c_dLoadingdtap = "c:dLoadingdtap", "%/tap, Branch Loading Sensitivity: dLbranch/dtap"
      c_dPlossdP = "c:dPlossdP", "kW/MW, Active Power Losses Sensitivity: dPloss/dP"
      c_dPlossdQ = "c:dPlossdQ", "kW/Mvar, Active Power Losses Sensitivity: dPloss/dQ"
      c_dQlossdP = "c:dQlossdP", "kvar/MW, Reactive Power Losses Sensitivity: dQloss/dP"
      c_dQlossdQ = "c:dQlossdQ", "kvar/Mvar, Reactive Power Losses Sensitivity: dQloss/dQ"
      c_dPlossdtap = "c:dPlossdtap", "kW/tap, Active Power Losses Sensitivity: dPloss/dtap"
      c_dQlossdtap = "c:dQlossdtap", "kvar/tap, Reactive Power Losses Sensitivity: dQloss/dtap"
      c_zline_r = "c:zline:r", "p.u., Impedance, Real Part"
      c_zline_i = "c:zline:i", "p.u., Impedance, Imaginary Part"
      c_zline_phi = "c:zline:phi", "deg, Impedance, Angle"
      c_zline_mag = "c:zline:mag", "p.u., Impedance, Magnitude"
      c_zline_phirad = "c:zline:phirad", "rad, Impedance, Angle"
      c_yline_r = "c:yline:r", "p.u., Admittance, Real Part"
      c_yline_i = "c:yline:i", "p.u., Admittance, Imaginary Part"
      c_yline_phi = "c:yline:phi", "deg, Admittance, Angle"
      c_yline_mag = "c:yline:mag", "p.u., Admittance, Magnitude"
      c_yline_phirad = "c:yline:phirad", "rad, Admittance, Angle"
      c_frnom = "c:frnom", "Hz, Nominal frequency"
      c_hpi = "c:hpi", "rad/s, Nominal angular frequency"
      c_loading = "c:loading", "%, Loading"
      c_curnom = "c:curnom", "kA, Nominal Current"
      c_rtfac = "c:rtfac", "Temperature Factor"
      c_maxloadpu = "c:maxloadpu", "p.u., Max. Loading"
      c_maxload = "c:maxload", "%, Max. Loading"
      c_du = "c:du", "p.u., Voltage Drop"
      c_dupc = "c:dupc", "%, Voltage Drop"
      c_dphiu = "c:dphiu", "deg, Voltage Drop Angle"
      c_du1 = "c:du1", "p.u., Positive Sequence Voltage Drop"
      c_du1pc = "c:du1pc", "%, Positive Sequence Voltage Drop"
      c_dphiu1 = "c:dphiu1", "deg, Positive Sequence Voltage Drop Angle"
      c_Cload = "c:Cload", "Mvar, Capacitive Losses"
      c_Gload = "c:Gload", "kW, Active no load losses"
      c_Losses = "c:Losses", "kW, Losses"
      c_Imax = "c:Imax", "kA, Maximum Current"
      c_Ploss = "c:Ploss", "MW, Losses (total)"
      c_Qloss = "c:Qloss", "Mvar, Reactive-Losses (total)"
      c_Plossld = "c:Plossld", "MW, Losses (load)"
      c_Qlossld = "c:Qlossld", "Mvar, Reactive-Losses (load)"
      c_Plossnld = "c:Plossnld", "MW, Losses (no load)"
      c_Qlossnld = "c:Qlossnld", "Mvar, Reactive-Losses (no load)"
      c_ncust = "c:ncust", "Number of Customers"
      c_Sl_r = "c:Sl:r", "MW, Load, variable Part, Apparent Power, Real Part"
      c_Sl_i = "c:Sl:i", "Mvar, Load, variable Part, Apparent Power, Imaginary Part"
      c_Sl_phi = "c:Sl:phi", "deg, Load, variable Part, Apparent Power, Angle"
      c_Sl_mag = "c:Sl:mag", "MVA, Load, variable Part, Apparent Power, Magnitude"
      c_Sl_phirad = "c:Sl:phirad", "rad, Load, variable Part, Apparent Power, Angle"
      c_ldcosphi = "c:ldcosphi", "Load, Variable Part, Power Factor"
      c_posld = "c:posld", "Weighted Load Position"
      c_ildlv = "c:ildlv", "Line Loads Present"
      c_Sldlv = "c:Sldlv", "kVA, Load, Apparent Power"
      c_Pldlv = "c:Pldlv", "kW, Load, Active Power"
      c_Qldlv = "c:Qldlv", "kvar, Load, Reactive Power"
      c_Ildlv = "c:Ildlv", "A, Load, Current"
      c_cosphildlv = "c:cosphildlv", "k, Load, Power Factor"

    class ElmLod(Enum):
      _init_ = 'value __doc__'

      m_dPdQ_bus1 = "m:dPdQ:bus1", "MW/Mvar, Branch Sensitivity dPbranch/dQbus"
      m_dQdQ_bus1 = "m:dQdQ:bus1", "Mvar/Mvar, Branch Sensitivity dQbranch/dQbus"
      m_dPdP_bus1 = "m:dPdP:bus1", "MW/MW, Branch Sensitivity dPbranch/dPbus"
      m_dQdP_bus1 = "m:dQdP:bus1", "Mvar/MW, Branch Sensitivity dQbranch/dPbus"
      m_dPdtap_bus1 = "m:dPdtap:bus1", "MW/tap, Branch Sensitivity dPbranch/dtap"
      m_dQdtap_bus1 = "m:dQdtap:bus1", "Mvar/tap, Branch Sensitivity dQbranch/dtap"
      m_LODF_bus1 = "m:LODF:bus1", "%, Line Outage Distribution Factor LODF"
      m_u1r_bus1 = "m:u1r:bus1", "p.u., Positive-Sequence-Voltage, Real Part"
      m_u1i_bus1 = "m:u1i:bus1", "p.u., Positive-Sequence-Voltage, Imaginary Part"
      m_u1_bus1 = "m:u1:bus1", "p.u., Positive-Sequence-Voltage, Magnitude"
      m_U1_bus1 = "m:U1:bus1", "kV, Line-Ground Positive-Sequence-Voltage, Magnitude"
      m_U1l_bus1 = "m:U1l:bus1", "kV, Line-Line Positive-Sequence-Voltage, Magnitude"
      m_phiu1_bus1 = "m:phiu1:bus1", "deg, Positive-Sequence-Voltage, Angle"
      m_inet_bus1 = "m:inet:bus1", "p.u., Current, Magnitude, referred to network"
      m_I_bus1 = "m:I:bus1", "kA, Current, Magnitude"
      m_phii_bus1 = "m:phii:bus1", "deg, Current, Angle"
      m_i1r_bus1 = "m:i1r:bus1", "p.u., Positive-Sequence Current, Real Part"
      m_i1i_bus1 = "m:i1i:bus1", "p.u., Positive-Sequence Current, Imaginary Part"
      m_i1_bus1 = "m:i1:bus1", "p.u., Positive-Sequence Current, Magnitude"
      m_I1_bus1 = "m:I1:bus1", "kA, Positive-Sequence Current, Magnitude"
      m_phii1_bus1 = "m:phii1:bus1", "deg, Positive-Sequence Current, Angle"
      m_P_bus1 = "m:P:bus1", "MW, Active Power"
      m_Q_bus1 = "m:Q:bus1", "Mvar, Reactive Power"
      m_S_bus1 = "m:S:bus1", "MVA, Apparent Power"
      m_cosphi_bus1 = "m:cosphi:bus1", "Power Factor"
      m_tanphi_bus1 = "m:tanphi:bus1", "tan(phi)"
      m_Psum_bus1 = "m:Psum:bus1", "MW, Total Active Power"
      m_Qsum_bus1 = "m:Qsum:bus1", "Mvar, Total Reactive Power"
      m_Ssum_bus1 = "m:Ssum:bus1", "MVA, Total Apparent Power"
      m_cosphisum_bus1 = "m:cosphisum:bus1", "Total Power Factor"
      m_tanphisum_bus1 = "m:tanphisum:bus1", "Total tan(phi)"
      m_Spu_bus1 = "m:Spu:bus1", "MVA/p.u., Apparent Power per p.u. Voltage"
      m_i1P_bus1 = "m:i1P:bus1", "p.u., Positive-Sequence Active Current"
      m_i1Q_bus1 = "m:i1Q:bus1", "p.u., Positive-Sequence Reactive Current"
      m_I1P_bus1 = "m:I1P:bus1", "kA, Positive-Sequence Active Current"
      m_I1Q_bus1 = "m:I1Q:bus1", "kA, Positive-Sequence Reactive Current"
      m_ur_bus1 = "m:ur:bus1", "p.u., Voltage, Real Part"
      m_ui_bus1 = "m:ui:bus1", "p.u., Voltage, Imaginary Part"
      m_u_bus1 = "m:u:bus1", "p.u., Voltage, Magnitude"
      m_ir_bus1 = "m:ir:bus1", "p.u., Current, Real Part"
      m_ii_bus1 = "m:ii:bus1", "p.u., Current, Imaginary Part"
      m_i_bus1 = "m:i:bus1", "p.u., Current, Magnitude"
      m_phiui_bus1 = "m:phiui:bus1", "deg, Angle between Voltage and Current"
      m_phiu1i1_bus1 = "m:phiu1i1:bus1", "deg, Angle between Voltage and Current in positive sequence system"
      m_Tfct_bus1 = "m:Tfct:bus1", "s, Fault Clearing Time"
      m_Brkload_bus1 = "m:Brkload:bus1", "%, Breaker Loading"
      n_dphidP_bus1 = "n:dphidP:bus1", "deg/MW, dphi/dP Sensitivity"
      n_dphidQ_bus1 = "n:dphidQ:bus1", "deg/Mvar, dphi/dQ Sensitivity"
      n_dphidtap_bus1 = "n:dphidtap:bus1", "deg/tap, dphi/dtap Sensitivity"
      n_dvdP_bus1 = "n:dvdP:bus1", "p.u./MW, dv/dP Sensitivity"
      n_dvdQ_bus1 = "n:dvdQ:bus1", "p.u./Mvar, dv/dQ Sensitivity"
      n_dvdtap_bus1 = "n:dvdtap:bus1", "p.u./tap, dv/dtap Sensitivity"
      n_levecQ_bus1 = "n:levecQ:bus1", "Left Eigenvector of selected Q-Mode"
      n_revecQ_bus1 = "n:revecQ:bus1", "Right Eigenvector of selected Q-Mode"
      n_partQ_bus1 = "n:partQ:bus1", "Participation Factor of selected Q-Mode"
      n_ur_bus1 = "n:ur:bus1", "p.u., Voltage, Real Part"
      n_ui_bus1 = "n:ui:bus1", "p.u., Voltage, Imaginary Part"
      n_u_bus1 = "n:u:bus1", "p.u., Voltage, Magnitude"
      n_upc_bus1 = "n:upc:bus1", "%, Voltage, Magnitude"
      n_u1_bus1 = "n:u1:bus1", "p.u., Voltage, Magnitude"
      n_u1pc_bus1 = "n:u1pc:bus1", "%, Voltage, Magnitude"
      n_u1r_bus1 = "n:u1r:bus1", "p.u., Positive-Sequence Voltage, Real Part"
      n_u1i_bus1 = "n:u1i:bus1", "p.u., Positive-Sequence Voltage, Imaginary Part"
      n_U_bus1 = "n:U:bus1", "kV, Line-Ground Voltage, Magnitude"
      n_Ul_bus1 = "n:Ul:bus1", "kV, Line-Line Voltage, Magnitude"
      n_phiu_bus1 = "n:phiu:bus1", "deg, Voltage, Angle"
      n_phiurel_bus1 = "n:phiurel:bus1", "deg, Voltage, Relative Angle"
      n_du_bus1 = "n:du:bus1", "%, Voltage Deviation"
      n_Pgen_bus1 = "n:Pgen:bus1", "MW, Generation, Active Power"
      n_Qgen_bus1 = "n:Qgen:bus1", "Mvar, Generation, Reactive Power"
      n_Pmot_bus1 = "n:Pmot:bus1", "MW, Motor Load, Active Power"
      n_Qmot_bus1 = "n:Qmot:bus1", "Mvar, Motor Load, Reactive Power"
      n_Pload_bus1 = "n:Pload:bus1", "MW, General Load, Active Power"
      n_Qload_bus1 = "n:Qload:bus1", "Mvar, General Load, Reactive Power"
      n_Pcomp_bus1 = "n:Pcomp:bus1", "MW, Compensation (Losses)"
      n_Qcomp_bus1 = "n:Qcomp:bus1", "Mvar, Compensation"
      n_Pnet_bus1 = "n:Pnet:bus1", "MW, External Networks, Active Power"
      n_Qnet_bus1 = "n:Qnet:bus1", "Mvar, External Networks, Reactive Power"
      n_Pflow_bus1 = "n:Pflow:bus1", "MW, Power Flow, Active Power"
      n_Qflow_bus1 = "n:Qflow:bus1", "Mvar, Power Flow, Reactive Power"
      n_Pout_bus1 = "n:Pout:bus1", "MW, Outgoing Flow, Active Power"
      n_Qout_bus1 = "n:Qout:bus1", "Mvar, Outgoing Power, Reactive Power"
      n_Sout_bus1 = "n:Sout:bus1", "MVA, Outgoing Power, Apparent Power"
      n_cosphiout_bus1 = "n:cosphiout:bus1", "Outgoing Power, Power Factor"
      n_Pbalance_bus1 = "n:Pbalance:bus1", "MW, Active Power Balance (=0)"
      n_Qbalance_bus1 = "n:Qbalance:bus1", "Mvar, Reactive Power Balance (=0)"
      n_LossPdown_bus1 = "n:LossPdown:bus1", "MW, Losses, downstream"
      n_LossQdown_bus1 = "n:LossQdown:bus1", "Mvar, Losses, downstream (Reactive Power)"
      n_LossPdownload_bus1 = "n:LossPdownload:bus1", "MW, Load losses, downstream"
      n_LossQdownload_bus1 = "n:LossQdownload:bus1", "Mvar, Load losses, downstream"
      n_LossPdownnoload_bus1 = "n:LossPdownnoload:bus1", "MW, No load losses, downstream"
      n_LossQdownnoload_bus1 = "n:LossQdownnoload:bus1", "Mvar, No load losses, downstream"
      n_du_feed_bus1 = "n:du_feed:bus1", "%, Voltage difference relative to feeder begin"
      s_scale = "s:scale", "IN, Scale Factor, continuous, double"
      s_scalephi = "s:scalephi", "rad, IN, Load Scaling: Power Angle, continuous, double"
      s_pini_set = "s:pini_set", "IN, Nominal Active Power, continuous, double"
      s_qini_set = "s:qini_set", "IN, Nominal Reactive Power, continuous, double"
      c_pini = "c:pini", "MW, Nominal Active Power"
      c_qini = "c:qini", "Mvar, Nominal Reactive Power"
      c_s_r = "c:s:r", "Complex Power, Real Part"
      c_s_i = "c:s:i", "Complex Power, Imaginary Part"
      c_s_phi = "c:s:phi", "Complex Power, Angle"
      c_s_mag = "c:s:mag", "Complex Power, Magnitude"
      c_s_phirad = "c:s:phirad", "Complex Power, Angle"
      c_frnom = "c:frnom", "Hz, Nominal Frequency"
      c_hpi = "c:hpi", "rad/s, Nominal angular frequency"
      c_u0 = "c:u0", "p.u., Nominal Voltage"
      c_ncust = "c:ncust", "Number of Connected Customers"
      c_scale0 = "c:scale0", "Initial Scale Factor"
      c_i_rem = "c:i_rem", "Remote Control"
      c_zonescale = "c:zonescale", "Zone Scale Factor"
      c_Pnom = "c:Pnom", "MW, Nominal Active Power"
      c_Qnom = "c:Qnom", "Mvar, Nominal Reactive Power"
      c_kpu0 = "c:kpu0", "Volt. Dependence I on P"
      c_kqu0 = "c:kqu0", "Volt. Dependence I on Q"
      c_kpu1 = "c:kpu1", "Volt. Dependence II on P"
      c_kqu1 = "c:kqu1", "Volt. Dependence II on Q"
      c_kpu = "c:kpu", "Volt. Dependence III on P"
      c_kqu = "c:kqu", "Volt. Dependence III on Q"
      c_aP = "c:aP", "Percent of Volt. Dep. I on P"
      c_bP = "c:bP", "Percent of Volt. Dep. II on P"
      c_aQ = "c:aQ", "Percent of Volt. Dep. I on Q"
      c_bQ = "c:bQ", "Percent of Volt. Dep. II on Q"
      c_iRefElement = "c:iRefElement", "Reference Element"
      c_Pmism = "c:Pmism", "MW, Active Power Mismatch"
      c_Ppeak = "c:Ppeak", "MW, Peak Active Power"
      c_Qpeak = "c:Qpeak", "Mvar, Peak Reactive Power"
      c_Ploss = "c:Ploss", "MW, Losses (total)"
      c_Qloss = "c:Qloss", "Mvar, Reactive-Losses (total)"
      c_Plossld = "c:Plossld", "MW, Losses (load)"
      c_Qlossld = "c:Qlossld", "Mvar, Reactive-Losses (load)"
      c_Plossnld = "c:Plossnld", "MW, Losses (no load)"
      c_Qlossnld = "c:Qlossnld", "Mvar, Reactive-Losses (no load)"

    class ElmLodlv(Enum):
      _init_ = 'value __doc__'

      m_dPdQ_bus1 = "m:dPdQ:bus1", "MW/Mvar, Branch Sensitivity dPbranch/dQbus"
      m_dQdQ_bus1 = "m:dQdQ:bus1", "Mvar/Mvar, Branch Sensitivity dQbranch/dQbus"
      m_dPdP_bus1 = "m:dPdP:bus1", "MW/MW, Branch Sensitivity dPbranch/dPbus"
      m_dQdP_bus1 = "m:dQdP:bus1", "Mvar/MW, Branch Sensitivity dQbranch/dPbus"
      m_dPdtap_bus1 = "m:dPdtap:bus1", "MW/tap, Branch Sensitivity dPbranch/dtap"
      m_dQdtap_bus1 = "m:dQdtap:bus1", "Mvar/tap, Branch Sensitivity dQbranch/dtap"
      m_LODF_bus1 = "m:LODF:bus1", "%, Line Outage Distribution Factor LODF"
      m_u1r_bus1 = "m:u1r:bus1", "p.u., Positive-Sequence-Voltage, Real Part"
      m_u1i_bus1 = "m:u1i:bus1", "p.u., Positive-Sequence-Voltage, Imaginary Part"
      m_u1_bus1 = "m:u1:bus1", "p.u., Positive-Sequence-Voltage, Magnitude"
      m_U1_bus1 = "m:U1:bus1", "kV, Line-Ground Positive-Sequence-Voltage, Magnitude"
      m_U1l_bus1 = "m:U1l:bus1", "kV, Line-Line Positive-Sequence-Voltage, Magnitude"
      m_phiu1_bus1 = "m:phiu1:bus1", "deg, Positive-Sequence-Voltage, Angle"
      m_inet_bus1 = "m:inet:bus1", "p.u., Current, Magnitude, referred to network"
      m_I_bus1 = "m:I:bus1", "kA, Current, Magnitude"
      m_phii_bus1 = "m:phii:bus1", "deg, Current, Angle"
      m_i1r_bus1 = "m:i1r:bus1", "p.u., Positive-Sequence Current, Real Part"
      m_i1i_bus1 = "m:i1i:bus1", "p.u., Positive-Sequence Current, Imaginary Part"
      m_i1_bus1 = "m:i1:bus1", "p.u., Positive-Sequence Current, Magnitude"
      m_I1_bus1 = "m:I1:bus1", "kA, Positive-Sequence Current, Magnitude"
      m_phii1_bus1 = "m:phii1:bus1", "deg, Positive-Sequence Current, Angle"
      m_P_bus1 = "m:P:bus1", "MW, Active Power"
      m_Q_bus1 = "m:Q:bus1", "Mvar, Reactive Power"
      m_S_bus1 = "m:S:bus1", "MVA, Apparent Power"
      m_cosphi_bus1 = "m:cosphi:bus1", "Power Factor"
      m_tanphi_bus1 = "m:tanphi:bus1", "tan(phi)"
      m_Psum_bus1 = "m:Psum:bus1", "MW, Total Active Power"
      m_Qsum_bus1 = "m:Qsum:bus1", "Mvar, Total Reactive Power"
      m_Ssum_bus1 = "m:Ssum:bus1", "MVA, Total Apparent Power"
      m_cosphisum_bus1 = "m:cosphisum:bus1", "Total Power Factor"
      m_tanphisum_bus1 = "m:tanphisum:bus1", "Total tan(phi)"
      m_Spu_bus1 = "m:Spu:bus1", "MVA/p.u., Apparent Power per p.u. Voltage"
      m_i1P_bus1 = "m:i1P:bus1", "p.u., Positive-Sequence Active Current"
      m_i1Q_bus1 = "m:i1Q:bus1", "p.u., Positive-Sequence Reactive Current"
      m_I1P_bus1 = "m:I1P:bus1", "kA, Positive-Sequence Active Current"
      m_I1Q_bus1 = "m:I1Q:bus1", "kA, Positive-Sequence Reactive Current"
      m_ur_bus1 = "m:ur:bus1", "p.u., Voltage, Real Part"
      m_ui_bus1 = "m:ui:bus1", "p.u., Voltage, Imaginary Part"
      m_u_bus1 = "m:u:bus1", "p.u., Voltage, Magnitude"
      m_ir_bus1 = "m:ir:bus1", "p.u., Current, Real Part"
      m_ii_bus1 = "m:ii:bus1", "p.u., Current, Imaginary Part"
      m_i_bus1 = "m:i:bus1", "p.u., Current, Magnitude"
      m_phiui_bus1 = "m:phiui:bus1", "deg, Angle between Voltage and Current"
      m_phiu1i1_bus1 = "m:phiu1i1:bus1", "deg, Angle between Voltage and Current in positive sequence system"
      m_Tfct_bus1 = "m:Tfct:bus1", "s, Fault Clearing Time"
      m_Brkload_bus1 = "m:Brkload:bus1", "%, Breaker Loading"
      n_dphidP_bus1 = "n:dphidP:bus1", "deg/MW, dphi/dP Sensitivity"
      n_dphidQ_bus1 = "n:dphidQ:bus1", "deg/Mvar, dphi/dQ Sensitivity"
      n_dphidtap_bus1 = "n:dphidtap:bus1", "deg/tap, dphi/dtap Sensitivity"
      n_dvdP_bus1 = "n:dvdP:bus1", "p.u./MW, dv/dP Sensitivity"
      n_dvdQ_bus1 = "n:dvdQ:bus1", "p.u./Mvar, dv/dQ Sensitivity"
      n_dvdtap_bus1 = "n:dvdtap:bus1", "p.u./tap, dv/dtap Sensitivity"
      n_levecQ_bus1 = "n:levecQ:bus1", "Left Eigenvector of selected Q-Mode"
      n_revecQ_bus1 = "n:revecQ:bus1", "Right Eigenvector of selected Q-Mode"
      n_partQ_bus1 = "n:partQ:bus1", "Participation Factor of selected Q-Mode"
      n_ur_bus1 = "n:ur:bus1", "p.u., Voltage, Real Part"
      n_ui_bus1 = "n:ui:bus1", "p.u., Voltage, Imaginary Part"
      n_u_bus1 = "n:u:bus1", "p.u., Voltage, Magnitude"
      n_upc_bus1 = "n:upc:bus1", "%, Voltage, Magnitude"
      n_u1_bus1 = "n:u1:bus1", "p.u., Voltage, Magnitude"
      n_u1pc_bus1 = "n:u1pc:bus1", "%, Voltage, Magnitude"
      n_u1r_bus1 = "n:u1r:bus1", "p.u., Positive-Sequence Voltage, Real Part"
      n_u1i_bus1 = "n:u1i:bus1", "p.u., Positive-Sequence Voltage, Imaginary Part"
      n_U_bus1 = "n:U:bus1", "kV, Line-Ground Voltage, Magnitude"
      n_Ul_bus1 = "n:Ul:bus1", "kV, Line-Line Voltage, Magnitude"
      n_phiu_bus1 = "n:phiu:bus1", "deg, Voltage, Angle"
      n_phiurel_bus1 = "n:phiurel:bus1", "deg, Voltage, Relative Angle"
      n_du_bus1 = "n:du:bus1", "%, Voltage Deviation"
      n_Pgen_bus1 = "n:Pgen:bus1", "MW, Generation, Active Power"
      n_Qgen_bus1 = "n:Qgen:bus1", "Mvar, Generation, Reactive Power"
      n_Pmot_bus1 = "n:Pmot:bus1", "MW, Motor Load, Active Power"
      n_Qmot_bus1 = "n:Qmot:bus1", "Mvar, Motor Load, Reactive Power"
      n_Pload_bus1 = "n:Pload:bus1", "MW, General Load, Active Power"
      n_Qload_bus1 = "n:Qload:bus1", "Mvar, General Load, Reactive Power"
      n_Pcomp_bus1 = "n:Pcomp:bus1", "MW, Compensation (Losses)"
      n_Qcomp_bus1 = "n:Qcomp:bus1", "Mvar, Compensation"
      n_Pnet_bus1 = "n:Pnet:bus1", "MW, External Networks, Active Power"
      n_Qnet_bus1 = "n:Qnet:bus1", "Mvar, External Networks, Reactive Power"
      n_Pflow_bus1 = "n:Pflow:bus1", "MW, Power Flow, Active Power"
      n_Qflow_bus1 = "n:Qflow:bus1", "Mvar, Power Flow, Reactive Power"
      n_Pout_bus1 = "n:Pout:bus1", "MW, Outgoing Flow, Active Power"
      n_Qout_bus1 = "n:Qout:bus1", "Mvar, Outgoing Power, Reactive Power"
      n_Sout_bus1 = "n:Sout:bus1", "MVA, Outgoing Power, Apparent Power"
      n_cosphiout_bus1 = "n:cosphiout:bus1", "Outgoing Power, Power Factor"
      n_Pbalance_bus1 = "n:Pbalance:bus1", "MW, Active Power Balance (=0)"
      n_Qbalance_bus1 = "n:Qbalance:bus1", "Mvar, Reactive Power Balance (=0)"
      n_LossPdown_bus1 = "n:LossPdown:bus1", "MW, Losses, downstream"
      n_LossQdown_bus1 = "n:LossQdown:bus1", "Mvar, Losses, downstream (Reactive Power)"
      n_LossPdownload_bus1 = "n:LossPdownload:bus1", "MW, Load losses, downstream"
      n_LossQdownload_bus1 = "n:LossQdownload:bus1", "Mvar, Load losses, downstream"
      n_LossPdownnoload_bus1 = "n:LossPdownnoload:bus1", "MW, No load losses, downstream"
      n_LossQdownnoload_bus1 = "n:LossQdownnoload:bus1", "Mvar, No load losses, downstream"
      n_du_feed_bus1 = "n:du_feed:bus1", "%, Voltage difference relative to feeder begin"
      s_scale = "s:scale", "IN, Scale Factor, continuous, double"
      s_scalephi = "s:scalephi", "rad, IN, Load Scaling: Power Angle, continuous, double"
      s_pini_set = "s:pini_set", "IN, Nominal Active Power, continuous, double"
      s_qini_set = "s:qini_set", "IN, Nominal Reactive Power, continuous, double"
      c_pini = "c:pini", "MW, Nominal Active Power"
      c_qini = "c:qini", "Mvar, Nominal Reactive Power"
      c_s_r = "c:s:r", "Complex Power, Real Part"
      c_s_i = "c:s:i", "Complex Power, Imaginary Part"
      c_s_phi = "c:s:phi", "Complex Power, Angle"
      c_s_mag = "c:s:mag", "Complex Power, Magnitude"
      c_s_phirad = "c:s:phirad", "Complex Power, Angle"
      c_frnom = "c:frnom", "Hz, Nominal Frequency"
      c_hpi = "c:hpi", "rad/s, Nominal angular frequency"
      c_u0 = "c:u0", "p.u., Nominal Voltage"
      c_ncust = "c:ncust", "Number of Connected Customers"
      c_scale0 = "c:scale0", "Initial Scale Factor"
      c_zonescale = "c:zonescale", "Zone Scale Factor"
      c_Pnom = "c:Pnom", "MW, Nominal Active Power"
      c_Qnom = "c:Qnom", "Mvar, Nominal Reactive Power"
      c_kpu0 = "c:kpu0", "Volt. Dependence I on P"
      c_kqu0 = "c:kqu0", "Volt. Dependence I on Q"
      c_kpu1 = "c:kpu1", "Volt. Dependence II on P"
      c_kqu1 = "c:kqu1", "Volt. Dependence II on Q"
      c_kpu = "c:kpu", "Volt. Dependence III on P"
      c_kqu = "c:kqu", "Volt. Dependence III on Q"
      c_aP = "c:aP", "Percent of Volt. Dep. I on P"
      c_bP = "c:bP", "Percent of Volt. Dep. II on P"
      c_aQ = "c:aQ", "Percent of Volt. Dep. I on Q"
      c_bQ = "c:bQ", "Percent of Volt. Dep. II on Q"
      c_Pmism = "c:Pmism", "MW, Active Power Mismatch"
      c_Ppeak = "c:Ppeak", "MW, Peak Active Power"
      c_Qpeak = "c:Qpeak", "Mvar, Peak Reactive Power"
      c_Ploss = "c:Ploss", "MW, Losses (total)"
      c_Qloss = "c:Qloss", "Mvar, Reactive-Losses (total)"
      c_Plossld = "c:Plossld", "MW, Losses (load)"
      c_Qlossld = "c:Qlossld", "Mvar, Reactive-Losses (load)"
      c_Plossnld = "c:Plossnld", "MW, Losses (no load)"
      c_Qlossnld = "c:Qlossnld", "Mvar, Reactive-Losses (no load)"

    class ElmLodmv(Enum):
      _init_ = 'value __doc__'

      m_dPdQ_bus1 = "m:dPdQ:bus1", "MW/Mvar, Branch Sensitivity dPbranch/dQbus"
      m_dQdQ_bus1 = "m:dQdQ:bus1", "Mvar/Mvar, Branch Sensitivity dQbranch/dQbus"
      m_dPdP_bus1 = "m:dPdP:bus1", "MW/MW, Branch Sensitivity dPbranch/dPbus"
      m_dQdP_bus1 = "m:dQdP:bus1", "Mvar/MW, Branch Sensitivity dQbranch/dPbus"
      m_dPdtap_bus1 = "m:dPdtap:bus1", "MW/tap, Branch Sensitivity dPbranch/dtap"
      m_dQdtap_bus1 = "m:dQdtap:bus1", "Mvar/tap, Branch Sensitivity dQbranch/dtap"
      m_LODF_bus1 = "m:LODF:bus1", "%, Line Outage Distribution Factor LODF"
      m_u1r_bus1 = "m:u1r:bus1", "p.u., Positive-Sequence-Voltage, Real Part"
      m_u1i_bus1 = "m:u1i:bus1", "p.u., Positive-Sequence-Voltage, Imaginary Part"
      m_u1_bus1 = "m:u1:bus1", "p.u., Positive-Sequence-Voltage, Magnitude"
      m_U1_bus1 = "m:U1:bus1", "kV, Line-Ground Positive-Sequence-Voltage, Magnitude"
      m_U1l_bus1 = "m:U1l:bus1", "kV, Line-Line Positive-Sequence-Voltage, Magnitude"
      m_phiu1_bus1 = "m:phiu1:bus1", "deg, Positive-Sequence-Voltage, Angle"
      m_inet_bus1 = "m:inet:bus1", "p.u., Current, Magnitude, referred to network"
      m_I_bus1 = "m:I:bus1", "kA, Current, Magnitude"
      m_phii_bus1 = "m:phii:bus1", "deg, Current, Angle"
      m_i1r_bus1 = "m:i1r:bus1", "p.u., Positive-Sequence Current, Real Part"
      m_i1i_bus1 = "m:i1i:bus1", "p.u., Positive-Sequence Current, Imaginary Part"
      m_i1_bus1 = "m:i1:bus1", "p.u., Positive-Sequence Current, Magnitude"
      m_I1_bus1 = "m:I1:bus1", "kA, Positive-Sequence Current, Magnitude"
      m_phii1_bus1 = "m:phii1:bus1", "deg, Positive-Sequence Current, Angle"
      m_P_bus1 = "m:P:bus1", "MW, Active Power"
      m_Q_bus1 = "m:Q:bus1", "Mvar, Reactive Power"
      m_S_bus1 = "m:S:bus1", "MVA, Apparent Power"
      m_cosphi_bus1 = "m:cosphi:bus1", "Power Factor"
      m_tanphi_bus1 = "m:tanphi:bus1", "tan(phi)"
      m_Psum_bus1 = "m:Psum:bus1", "MW, Total Active Power"
      m_Qsum_bus1 = "m:Qsum:bus1", "Mvar, Total Reactive Power"
      m_Ssum_bus1 = "m:Ssum:bus1", "MVA, Total Apparent Power"
      m_cosphisum_bus1 = "m:cosphisum:bus1", "Total Power Factor"
      m_tanphisum_bus1 = "m:tanphisum:bus1", "Total tan(phi)"
      m_Spu_bus1 = "m:Spu:bus1", "MVA/p.u., Apparent Power per p.u. Voltage"
      m_i1P_bus1 = "m:i1P:bus1", "p.u., Positive-Sequence Active Current"
      m_i1Q_bus1 = "m:i1Q:bus1", "p.u., Positive-Sequence Reactive Current"
      m_I1P_bus1 = "m:I1P:bus1", "kA, Positive-Sequence Active Current"
      m_I1Q_bus1 = "m:I1Q:bus1", "kA, Positive-Sequence Reactive Current"
      m_ur_bus1 = "m:ur:bus1", "p.u., Voltage, Real Part"
      m_ui_bus1 = "m:ui:bus1", "p.u., Voltage, Imaginary Part"
      m_u_bus1 = "m:u:bus1", "p.u., Voltage, Magnitude"
      m_ir_bus1 = "m:ir:bus1", "p.u., Current, Real Part"
      m_ii_bus1 = "m:ii:bus1", "p.u., Current, Imaginary Part"
      m_i_bus1 = "m:i:bus1", "p.u., Current, Magnitude"
      m_phiui_bus1 = "m:phiui:bus1", "deg, Angle between Voltage and Current"
      m_phiu1i1_bus1 = "m:phiu1i1:bus1", "deg, Angle between Voltage and Current in positive sequence system"
      m_Tfct_bus1 = "m:Tfct:bus1", "s, Fault Clearing Time"
      m_Brkload_bus1 = "m:Brkload:bus1", "%, Breaker Loading"
      n_dphidP_bus1 = "n:dphidP:bus1", "deg/MW, dphi/dP Sensitivity"
      n_dphidQ_bus1 = "n:dphidQ:bus1", "deg/Mvar, dphi/dQ Sensitivity"
      n_dphidtap_bus1 = "n:dphidtap:bus1", "deg/tap, dphi/dtap Sensitivity"
      n_dvdP_bus1 = "n:dvdP:bus1", "p.u./MW, dv/dP Sensitivity"
      n_dvdQ_bus1 = "n:dvdQ:bus1", "p.u./Mvar, dv/dQ Sensitivity"
      n_dvdtap_bus1 = "n:dvdtap:bus1", "p.u./tap, dv/dtap Sensitivity"
      n_levecQ_bus1 = "n:levecQ:bus1", "Left Eigenvector of selected Q-Mode"
      n_revecQ_bus1 = "n:revecQ:bus1", "Right Eigenvector of selected Q-Mode"
      n_partQ_bus1 = "n:partQ:bus1", "Participation Factor of selected Q-Mode"
      n_ur_bus1 = "n:ur:bus1", "p.u., Voltage, Real Part"
      n_ui_bus1 = "n:ui:bus1", "p.u., Voltage, Imaginary Part"
      n_u_bus1 = "n:u:bus1", "p.u., Voltage, Magnitude"
      n_upc_bus1 = "n:upc:bus1", "%, Voltage, Magnitude"
      n_u1_bus1 = "n:u1:bus1", "p.u., Voltage, Magnitude"
      n_u1pc_bus1 = "n:u1pc:bus1", "%, Voltage, Magnitude"
      n_u1r_bus1 = "n:u1r:bus1", "p.u., Positive-Sequence Voltage, Real Part"
      n_u1i_bus1 = "n:u1i:bus1", "p.u., Positive-Sequence Voltage, Imaginary Part"
      n_U_bus1 = "n:U:bus1", "kV, Line-Ground Voltage, Magnitude"
      n_Ul_bus1 = "n:Ul:bus1", "kV, Line-Line Voltage, Magnitude"
      n_phiu_bus1 = "n:phiu:bus1", "deg, Voltage, Angle"
      n_phiurel_bus1 = "n:phiurel:bus1", "deg, Voltage, Relative Angle"
      n_du_bus1 = "n:du:bus1", "%, Voltage Deviation"
      n_Pgen_bus1 = "n:Pgen:bus1", "MW, Generation, Active Power"
      n_Qgen_bus1 = "n:Qgen:bus1", "Mvar, Generation, Reactive Power"
      n_Pmot_bus1 = "n:Pmot:bus1", "MW, Motor Load, Active Power"
      n_Qmot_bus1 = "n:Qmot:bus1", "Mvar, Motor Load, Reactive Power"
      n_Pload_bus1 = "n:Pload:bus1", "MW, General Load, Active Power"
      n_Qload_bus1 = "n:Qload:bus1", "Mvar, General Load, Reactive Power"
      n_Pcomp_bus1 = "n:Pcomp:bus1", "MW, Compensation (Losses)"
      n_Qcomp_bus1 = "n:Qcomp:bus1", "Mvar, Compensation"
      n_Pnet_bus1 = "n:Pnet:bus1", "MW, External Networks, Active Power"
      n_Qnet_bus1 = "n:Qnet:bus1", "Mvar, External Networks, Reactive Power"
      n_Pflow_bus1 = "n:Pflow:bus1", "MW, Power Flow, Active Power"
      n_Qflow_bus1 = "n:Qflow:bus1", "Mvar, Power Flow, Reactive Power"
      n_Pout_bus1 = "n:Pout:bus1", "MW, Outgoing Flow, Active Power"
      n_Qout_bus1 = "n:Qout:bus1", "Mvar, Outgoing Power, Reactive Power"
      n_Sout_bus1 = "n:Sout:bus1", "MVA, Outgoing Power, Apparent Power"
      n_cosphiout_bus1 = "n:cosphiout:bus1", "Outgoing Power, Power Factor"
      n_Pbalance_bus1 = "n:Pbalance:bus1", "MW, Active Power Balance (=0)"
      n_Qbalance_bus1 = "n:Qbalance:bus1", "Mvar, Reactive Power Balance (=0)"
      n_LossPdown_bus1 = "n:LossPdown:bus1", "MW, Losses, downstream"
      n_LossQdown_bus1 = "n:LossQdown:bus1", "Mvar, Losses, downstream (Reactive Power)"
      n_LossPdownload_bus1 = "n:LossPdownload:bus1", "MW, Load losses, downstream"
      n_LossQdownload_bus1 = "n:LossQdownload:bus1", "Mvar, Load losses, downstream"
      n_LossPdownnoload_bus1 = "n:LossPdownnoload:bus1", "MW, No load losses, downstream"
      n_LossQdownnoload_bus1 = "n:LossQdownnoload:bus1", "Mvar, No load losses, downstream"
      n_du_feed_bus1 = "n:du_feed:bus1", "%, Voltage difference relative to feeder begin"
      s_scale = "s:scale", "IN, Scale Factor, continuous, double"
      s_scalephi = "s:scalephi", "rad, IN, Load Scaling: Power Angle, continuous, double"
      s_pini_set = "s:pini_set", "IN, Nominal Active Power, continuous, double"
      s_qini_set = "s:qini_set", "IN, Nominal Reactive Power, continuous, double"
      c_pini = "c:pini", "MW, Nominal Active Power"
      c_qini = "c:qini", "Mvar, Nominal Reactive Power"
      c_s_r = "c:s:r", "Complex Power, Real Part"
      c_s_i = "c:s:i", "Complex Power, Imaginary Part"
      c_s_phi = "c:s:phi", "Complex Power, Angle"
      c_s_mag = "c:s:mag", "Complex Power, Magnitude"
      c_s_phirad = "c:s:phirad", "Complex Power, Angle"
      c_frnom = "c:frnom", "Hz, Nominal Frequency"
      c_hpi = "c:hpi", "rad/s, Nominal angular frequency"
      c_u0 = "c:u0", "p.u., Nominal Voltage"
      c_nntap = "c:nntap", "Current Tap Position"
      c_ulv = "c:ulv", "p.u., LV Voltage"
      c_Gmload = "c:Gmload", "kW, Iron Losses"
      c_dudropTrf = "c:dudropTrf", "%, max. Voltage drop across transformer"
      c_duriseTrf = "c:duriseTrf", "%, max. Voltage rise across transformer"
      c_ncust = "c:ncust", "Number of Connected Customers"
      c_scale0 = "c:scale0", "Initial Scale Factor"
      c_zonescale = "c:zonescale", "Zone Scale Factor"
      c_gscale = "c:gscale", "Generation Scale Factor"
      c_tempscFac = "c:tempscFac", "Temperature dependent load scaling factor"
      c_Pnom = "c:Pnom", "MW, Nominal Active Power"
      c_Qnom = "c:Qnom", "Mvar, Nominal Reactive Power"
      c_kpu0 = "c:kpu0", "Volt. Dependence I on P"
      c_kqu0 = "c:kqu0", "Volt. Dependence I on Q"
      c_kpu1 = "c:kpu1", "Volt. Dependence II on P"
      c_kqu1 = "c:kqu1", "Volt. Dependence II on Q"
      c_kpu = "c:kpu", "Volt. Dependence III on P"
      c_kqu = "c:kqu", "Volt. Dependence III on Q"
      c_aP = "c:aP", "Percent of Volt. Dep. I on P"
      c_bP = "c:bP", "Percent of Volt. Dep. II on P"
      c_aQ = "c:aQ", "Percent of Volt. Dep. I on Q"
      c_bQ = "c:bQ", "Percent of Volt. Dep. II on Q"
      c_iRefElement = "c:iRefElement", "Reference Element"
      c_Ppeak = "c:Ppeak", "MW, Peak Active Power"
      c_Qpeak = "c:Qpeak", "Mvar, Peak Reactive Power"
      c_Ploss = "c:Ploss", "MW, Losses (total)"
      c_Qloss = "c:Qloss", "Mvar, Reactive-Losses (total)"
      c_Plossld = "c:Plossld", "MW, Losses (load)"
      c_Qlossld = "c:Qlossld", "Mvar, Reactive-Losses (load)"
      c_Plossnld = "c:Plossnld", "MW, Losses (no load)"
      c_Qlossnld = "c:Qlossnld", "Mvar, Reactive-Losses (no load)"

    class ElmPvsys(Enum):
      _init_ = 'value __doc__'

      m_dPdQ_bus1 = "m:dPdQ:bus1", "MW/Mvar, Branch Sensitivity dPbranch/dQbus"
      m_dQdQ_bus1 = "m:dQdQ:bus1", "Mvar/Mvar, Branch Sensitivity dQbranch/dQbus"
      m_dPdP_bus1 = "m:dPdP:bus1", "MW/MW, Branch Sensitivity dPbranch/dPbus"
      m_dQdP_bus1 = "m:dQdP:bus1", "Mvar/MW, Branch Sensitivity dQbranch/dPbus"
      m_dPdtap_bus1 = "m:dPdtap:bus1", "MW/tap, Branch Sensitivity dPbranch/dtap"
      m_dQdtap_bus1 = "m:dQdtap:bus1", "Mvar/tap, Branch Sensitivity dQbranch/dtap"
      m_LODF_bus1 = "m:LODF:bus1", "%, Line Outage Distribution Factor LODF"
      m_u1r_bus1 = "m:u1r:bus1", "p.u., Positive-Sequence-Voltage, Real Part"
      m_u1i_bus1 = "m:u1i:bus1", "p.u., Positive-Sequence-Voltage, Imaginary Part"
      m_u1_bus1 = "m:u1:bus1", "p.u., Positive-Sequence-Voltage, Magnitude"
      m_U1_bus1 = "m:U1:bus1", "kV, Line-Ground Positive-Sequence-Voltage, Magnitude"
      m_U1l_bus1 = "m:U1l:bus1", "kV, Line-Line Positive-Sequence-Voltage, Magnitude"
      m_phiu1_bus1 = "m:phiu1:bus1", "deg, Positive-Sequence-Voltage, Angle"
      m_inet_bus1 = "m:inet:bus1", "p.u., Current, Magnitude, referred to network"
      m_I_bus1 = "m:I:bus1", "kA, Current, Magnitude"
      m_phii_bus1 = "m:phii:bus1", "deg, Current, Angle"
      m_i1r_bus1 = "m:i1r:bus1", "p.u., Positive-Sequence Current, Real Part"
      m_i1i_bus1 = "m:i1i:bus1", "p.u., Positive-Sequence Current, Imaginary Part"
      m_i1_bus1 = "m:i1:bus1", "p.u., Positive-Sequence Current, Magnitude"
      m_I1_bus1 = "m:I1:bus1", "kA, Positive-Sequence Current, Magnitude"
      m_phii1_bus1 = "m:phii1:bus1", "deg, Positive-Sequence Current, Angle"
      m_P_bus1 = "m:P:bus1", "MW, Active Power"
      m_Q_bus1 = "m:Q:bus1", "Mvar, Reactive Power"
      m_S_bus1 = "m:S:bus1", "MVA, Apparent Power"
      m_cosphi_bus1 = "m:cosphi:bus1", "Power Factor"
      m_tanphi_bus1 = "m:tanphi:bus1", "tan(phi)"
      m_Psum_bus1 = "m:Psum:bus1", "MW, Total Active Power"
      m_Qsum_bus1 = "m:Qsum:bus1", "Mvar, Total Reactive Power"
      m_Ssum_bus1 = "m:Ssum:bus1", "MVA, Total Apparent Power"
      m_cosphisum_bus1 = "m:cosphisum:bus1", "Total Power Factor"
      m_tanphisum_bus1 = "m:tanphisum:bus1", "Total tan(phi)"
      m_Spu_bus1 = "m:Spu:bus1", "MVA/p.u., Apparent Power per p.u. Voltage"
      m_i1P_bus1 = "m:i1P:bus1", "p.u., Positive-Sequence Active Current"
      m_i1Q_bus1 = "m:i1Q:bus1", "p.u., Positive-Sequence Reactive Current"
      m_I1P_bus1 = "m:I1P:bus1", "kA, Positive-Sequence Active Current"
      m_I1Q_bus1 = "m:I1Q:bus1", "kA, Positive-Sequence Reactive Current"
      m_ur_bus1 = "m:ur:bus1", "p.u., Voltage, Real Part"
      m_ui_bus1 = "m:ui:bus1", "p.u., Voltage, Imaginary Part"
      m_u_bus1 = "m:u:bus1", "p.u., Voltage, Magnitude"
      m_ir_bus1 = "m:ir:bus1", "p.u., Current, Real Part"
      m_ii_bus1 = "m:ii:bus1", "p.u., Current, Imaginary Part"
      m_i_bus1 = "m:i:bus1", "p.u., Current, Magnitude"
      m_phiui_bus1 = "m:phiui:bus1", "deg, Angle between Voltage and Current"
      m_phiu1i1_bus1 = "m:phiu1i1:bus1", "deg, Angle between Voltage and Current in positive sequence system"
      m_Tfct_bus1 = "m:Tfct:bus1", "s, Fault Clearing Time"
      m_Brkload_bus1 = "m:Brkload:bus1", "%, Breaker Loading"
      n_dphidP_bus1 = "n:dphidP:bus1", "deg/MW, dphi/dP Sensitivity"
      n_dphidQ_bus1 = "n:dphidQ:bus1", "deg/Mvar, dphi/dQ Sensitivity"
      n_dphidtap_bus1 = "n:dphidtap:bus1", "deg/tap, dphi/dtap Sensitivity"
      n_dvdP_bus1 = "n:dvdP:bus1", "p.u./MW, dv/dP Sensitivity"
      n_dvdQ_bus1 = "n:dvdQ:bus1", "p.u./Mvar, dv/dQ Sensitivity"
      n_dvdtap_bus1 = "n:dvdtap:bus1", "p.u./tap, dv/dtap Sensitivity"
      n_levecQ_bus1 = "n:levecQ:bus1", "Left Eigenvector of selected Q-Mode"
      n_revecQ_bus1 = "n:revecQ:bus1", "Right Eigenvector of selected Q-Mode"
      n_partQ_bus1 = "n:partQ:bus1", "Participation Factor of selected Q-Mode"
      n_ur_bus1 = "n:ur:bus1", "p.u., Voltage, Real Part"
      n_ui_bus1 = "n:ui:bus1", "p.u., Voltage, Imaginary Part"
      n_u_bus1 = "n:u:bus1", "p.u., Voltage, Magnitude"
      n_upc_bus1 = "n:upc:bus1", "%, Voltage, Magnitude"
      n_u1_bus1 = "n:u1:bus1", "p.u., Voltage, Magnitude"
      n_u1pc_bus1 = "n:u1pc:bus1", "%, Voltage, Magnitude"
      n_u1r_bus1 = "n:u1r:bus1", "p.u., Positive-Sequence Voltage, Real Part"
      n_u1i_bus1 = "n:u1i:bus1", "p.u., Positive-Sequence Voltage, Imaginary Part"
      n_U_bus1 = "n:U:bus1", "kV, Line-Ground Voltage, Magnitude"
      n_Ul_bus1 = "n:Ul:bus1", "kV, Line-Line Voltage, Magnitude"
      n_phiu_bus1 = "n:phiu:bus1", "deg, Voltage, Angle"
      n_phiurel_bus1 = "n:phiurel:bus1", "deg, Voltage, Relative Angle"
      n_du_bus1 = "n:du:bus1", "%, Voltage Deviation"
      n_Pgen_bus1 = "n:Pgen:bus1", "MW, Generation, Active Power"
      n_Qgen_bus1 = "n:Qgen:bus1", "Mvar, Generation, Reactive Power"
      n_Pmot_bus1 = "n:Pmot:bus1", "MW, Motor Load, Active Power"
      n_Qmot_bus1 = "n:Qmot:bus1", "Mvar, Motor Load, Reactive Power"
      n_Pload_bus1 = "n:Pload:bus1", "MW, General Load, Active Power"
      n_Qload_bus1 = "n:Qload:bus1", "Mvar, General Load, Reactive Power"
      n_Pcomp_bus1 = "n:Pcomp:bus1", "MW, Compensation (Losses)"
      n_Qcomp_bus1 = "n:Qcomp:bus1", "Mvar, Compensation"
      n_Pnet_bus1 = "n:Pnet:bus1", "MW, External Networks, Active Power"
      n_Qnet_bus1 = "n:Qnet:bus1", "Mvar, External Networks, Reactive Power"
      n_Pflow_bus1 = "n:Pflow:bus1", "MW, Power Flow, Active Power"
      n_Qflow_bus1 = "n:Qflow:bus1", "Mvar, Power Flow, Reactive Power"
      n_Pout_bus1 = "n:Pout:bus1", "MW, Outgoing Flow, Active Power"
      n_Qout_bus1 = "n:Qout:bus1", "Mvar, Outgoing Power, Reactive Power"
      n_Sout_bus1 = "n:Sout:bus1", "MVA, Outgoing Power, Apparent Power"
      n_cosphiout_bus1 = "n:cosphiout:bus1", "Outgoing Power, Power Factor"
      n_Pbalance_bus1 = "n:Pbalance:bus1", "MW, Active Power Balance (=0)"
      n_Qbalance_bus1 = "n:Qbalance:bus1", "Mvar, Reactive Power Balance (=0)"
      n_LossPdown_bus1 = "n:LossPdown:bus1", "MW, Losses, downstream"
      n_LossQdown_bus1 = "n:LossQdown:bus1", "Mvar, Losses, downstream (Reactive Power)"
      n_LossPdownload_bus1 = "n:LossPdownload:bus1", "MW, Load losses, downstream"
      n_LossQdownload_bus1 = "n:LossQdownload:bus1", "Mvar, Load losses, downstream"
      n_LossPdownnoload_bus1 = "n:LossPdownnoload:bus1", "MW, No load losses, downstream"
      n_LossQdownnoload_bus1 = "n:LossQdownnoload:bus1", "Mvar, No load losses, downstream"
      n_du_feed_bus1 = "n:du_feed:bus1", "%, Voltage difference relative to feeder begin"
      s_pset = "s:pset", "MW, IN, Active Power Setpoint, continuous, double"
      s_qset = "s:qset", "Mvar, IN, Reactive Power Setpoint continuous, double"
      s_dFin = "s:dFin", "Hz, IN, Frequency Deviation, continuous, double"
      c_p = "c:p", "MW, Total Active Power"
      c_q = "c:q", "Mvar, Total Reactive Power"
      c_sgnn = "c:sgnn", "MVA, Total rated apparent power (ngnum*sgn)"
      c_cosn = "c:cosn", "Nominal-Power factor"
      c_Pnom = "c:Pnom", "MW, Total Nominal Active power"
      c_c2 = "c:c2", "Voltage Coefficient (uknom/ugn)"
      c_uini_r = "c:uini:r", "p.u., Voltage Setpoint for PV and Slack, Real Part"
      c_uini_i = "c:uini:i", "p.u., Voltage Setpoint for PV and Slack, Imaginary Part"
      c_uini_phi = "c:uini:phi", "deg, Voltage Setpoint for PV and Slack, Angle"
      c_uini_mag = "c:uini:mag", "p.u., Voltage Setpoint for PV and Slack, Magnitude"
      c_uini_phirad = "c:uini:phirad", "rad, Voltage Setpoint for PV and Slack, Angle"
      c_phiini = "c:phiini", "Initial Angle"
      c_loading = "c:loading", "%, Loading"
      c_iPowerOn = "c:iPowerOn", "Flag"
      c_Qmin = "c:Qmin", "Mvar, Lower limit of reactive power"
      c_Qmax = "c:Qmax", "Mvar, Upper limit of reactive power"
      c_Qsparelead = "c:Qsparelead", "Mvar, Q Spare Lead"
      c_Qsparelag = "c:Qsparelag", "Mvar, Q Spare Lag"
      c_i_qlim = "c:i_qlim", "Reactive power limit reached"
      c_i_vlim = "c:i_vlim", "Voltage setpoint limit reached"
      c_Pmin = "c:Pmin", "MW, Lower limit of active power"
      c_Pmax = "c:Pmax", "MW, Upper limit of active power"
      c_i_plim = "c:i_plim", "Active power limit reached"
      c_sm_r = "c:sm:r", "MW, Complex Power, Real Part"
      c_sm_i = "c:sm:i", "Mvar, Complex Power, Imaginary Part"
      c_sm_phi = "c:sm:phi", "deg, Complex Power, Angle"
      c_sm_mag = "c:sm:mag", "MVA, Complex Power, Magnitude"
      c_sm_phirad = "c:sm:phirad", "rad, Complex Power, Angle"
      c_u1_r = "c:u1:r", "p.u., Internal, Real Part"
      c_u1_i = "c:u1:i", "p.u., Internal, Imaginary Part"
      c_u1_phi = "c:u1:phi", "deg, Internal, Angle"
      c_u1_mag = "c:u1:mag", "p.u., Internal, Magnitude"
      c_u1_phirad = "c:u1:phirad", "rad, Internal, Angle"
      c_iRefElement = "c:iRefElement", "Reference Element"
      c_Snom = "c:Snom", "MVA, Nom. Apparent Power (1 mach.)"
      c_Nnom = "c:Nnom", "Number of parallel units"
      c_Kp = "c:Kp", "p.u., Active Power Participation"
      c_Kq = "c:Kq", "p.u., Reactive Power Participation"
      c_Kpf = "c:Kpf", "MW/Hz, Frequency Bias"
      c_zonewscale = "c:zonewscale", "Wind generation zone scaling factor"
      c_Pmism = "c:Pmism", "MW, Active Power Mismatch"
      c_nCtrlIter = "c:nCtrlIter", "No. of CTRL-iterations"
      c_Ploss = "c:Ploss", "MW, Losses (total)"
      c_Qloss = "c:Qloss", "Mvar, Reactive-Losses (total)"
      c_Plossld = "c:Plossld", "MW, Losses (load)"
      c_Qlossld = "c:Qlossld", "Mvar, Reactive-Losses (load)"
      c_Plossnld = "c:Plossnld", "MW, Losses (no load)"
      c_Qlossnld = "c:Qlossnld", "Mvar, Reactive-Losses (no load)"

    class ElmSecctrl(Enum):
      _init_ = 'value __doc__'

      s_pset = "s:pset", "MW, IN, Active Power Setpoint, continuous, double"
      s_dpsco = "s:dpsco", "MW, OUT, Controller Output, continuous, double"
      c_rembar = "c:rembar", "Controlled Busbar"
      c_pini = "c:pini", "MW, Active Power Setpoint"
      c_imode = "c:imode", "Control Mode"
      c_i_modus = "c:i_modus", "Control Mode 0=phi 1=p"
      c_phiu = "c:phiu", "rad, Phase Angle"
      c_PExch = "c:PExch", "MW, Power Exchange"
      c_PctrlLim = "c:PctrlLim", "MW, Controller Output Limit"

    class ElmShnt(Enum):
      _init_ = 'value __doc__'

      m_dPdQ_bus1 = "m:dPdQ:bus1", "MW/Mvar, Branch Sensitivity dPbranch/dQbus"
      m_dQdQ_bus1 = "m:dQdQ:bus1", "Mvar/Mvar, Branch Sensitivity dQbranch/dQbus"
      m_dPdP_bus1 = "m:dPdP:bus1", "MW/MW, Branch Sensitivity dPbranch/dPbus"
      m_dQdP_bus1 = "m:dQdP:bus1", "Mvar/MW, Branch Sensitivity dQbranch/dPbus"
      m_dPdtap_bus1 = "m:dPdtap:bus1", "MW/tap, Branch Sensitivity dPbranch/dtap"
      m_dQdtap_bus1 = "m:dQdtap:bus1", "Mvar/tap, Branch Sensitivity dQbranch/dtap"
      m_LODF_bus1 = "m:LODF:bus1", "%, Line Outage Distribution Factor LODF"
      m_u1r_bus1 = "m:u1r:bus1", "p.u., Positive-Sequence-Voltage, Real Part"
      m_u1i_bus1 = "m:u1i:bus1", "p.u., Positive-Sequence-Voltage, Imaginary Part"
      m_u1_bus1 = "m:u1:bus1", "p.u., Positive-Sequence-Voltage, Magnitude"
      m_U1_bus1 = "m:U1:bus1", "kV, Line-Ground Positive-Sequence-Voltage, Magnitude"
      m_U1l_bus1 = "m:U1l:bus1", "kV, Line-Line Positive-Sequence-Voltage, Magnitude"
      m_phiu1_bus1 = "m:phiu1:bus1", "deg, Positive-Sequence-Voltage, Angle"
      m_inet_bus1 = "m:inet:bus1", "p.u., Current, Magnitude, referred to network"
      m_I_bus1 = "m:I:bus1", "kA, Current, Magnitude"
      m_phii_bus1 = "m:phii:bus1", "deg, Current, Angle"
      m_i1r_bus1 = "m:i1r:bus1", "p.u., Positive-Sequence Current, Real Part"
      m_i1i_bus1 = "m:i1i:bus1", "p.u., Positive-Sequence Current, Imaginary Part"
      m_i1_bus1 = "m:i1:bus1", "p.u., Positive-Sequence Current, Magnitude"
      m_I1_bus1 = "m:I1:bus1", "kA, Positive-Sequence Current, Magnitude"
      m_phii1_bus1 = "m:phii1:bus1", "deg, Positive-Sequence Current, Angle"
      m_P_bus1 = "m:P:bus1", "MW, Active Power"
      m_Q_bus1 = "m:Q:bus1", "Mvar, Reactive Power"
      m_S_bus1 = "m:S:bus1", "MVA, Apparent Power"
      m_cosphi_bus1 = "m:cosphi:bus1", "Power Factor"
      m_tanphi_bus1 = "m:tanphi:bus1", "tan(phi)"
      m_Psum_bus1 = "m:Psum:bus1", "MW, Total Active Power"
      m_Qsum_bus1 = "m:Qsum:bus1", "Mvar, Total Reactive Power"
      m_Ssum_bus1 = "m:Ssum:bus1", "MVA, Total Apparent Power"
      m_cosphisum_bus1 = "m:cosphisum:bus1", "Total Power Factor"
      m_tanphisum_bus1 = "m:tanphisum:bus1", "Total tan(phi)"
      m_Spu_bus1 = "m:Spu:bus1", "MVA/p.u., Apparent Power per p.u. Voltage"
      m_i1P_bus1 = "m:i1P:bus1", "p.u., Positive-Sequence Active Current"
      m_i1Q_bus1 = "m:i1Q:bus1", "p.u., Positive-Sequence Reactive Current"
      m_I1P_bus1 = "m:I1P:bus1", "kA, Positive-Sequence Active Current"
      m_I1Q_bus1 = "m:I1Q:bus1", "kA, Positive-Sequence Reactive Current"
      m_ur_bus1 = "m:ur:bus1", "p.u., Voltage, Real Part"
      m_ui_bus1 = "m:ui:bus1", "p.u., Voltage, Imaginary Part"
      m_u_bus1 = "m:u:bus1", "p.u., Voltage, Magnitude"
      m_ir_bus1 = "m:ir:bus1", "p.u., Current, Real Part"
      m_ii_bus1 = "m:ii:bus1", "p.u., Current, Imaginary Part"
      m_i_bus1 = "m:i:bus1", "p.u., Current, Magnitude"
      m_phiui_bus1 = "m:phiui:bus1", "deg, Angle between Voltage and Current"
      m_phiu1i1_bus1 = "m:phiu1i1:bus1", "deg, Angle between Voltage and Current in positive sequence system"
      m_Tfct_bus1 = "m:Tfct:bus1", "s, Fault Clearing Time"
      m_Brkload_bus1 = "m:Brkload:bus1", "%, Breaker Loading"
      n_dphidP_bus1 = "n:dphidP:bus1", "deg/MW, dphi/dP Sensitivity"
      n_dphidQ_bus1 = "n:dphidQ:bus1", "deg/Mvar, dphi/dQ Sensitivity"
      n_dphidtap_bus1 = "n:dphidtap:bus1", "deg/tap, dphi/dtap Sensitivity"
      n_dvdP_bus1 = "n:dvdP:bus1", "p.u./MW, dv/dP Sensitivity"
      n_dvdQ_bus1 = "n:dvdQ:bus1", "p.u./Mvar, dv/dQ Sensitivity"
      n_dvdtap_bus1 = "n:dvdtap:bus1", "p.u./tap, dv/dtap Sensitivity"
      n_levecQ_bus1 = "n:levecQ:bus1", "Left Eigenvector of selected Q-Mode"
      n_revecQ_bus1 = "n:revecQ:bus1", "Right Eigenvector of selected Q-Mode"
      n_partQ_bus1 = "n:partQ:bus1", "Participation Factor of selected Q-Mode"
      n_ur_bus1 = "n:ur:bus1", "p.u., Voltage, Real Part"
      n_ui_bus1 = "n:ui:bus1", "p.u., Voltage, Imaginary Part"
      n_u_bus1 = "n:u:bus1", "p.u., Voltage, Magnitude"
      n_upc_bus1 = "n:upc:bus1", "%, Voltage, Magnitude"
      n_u1_bus1 = "n:u1:bus1", "p.u., Voltage, Magnitude"
      n_u1pc_bus1 = "n:u1pc:bus1", "%, Voltage, Magnitude"
      n_u1r_bus1 = "n:u1r:bus1", "p.u., Positive-Sequence Voltage, Real Part"
      n_u1i_bus1 = "n:u1i:bus1", "p.u., Positive-Sequence Voltage, Imaginary Part"
      n_U_bus1 = "n:U:bus1", "kV, Line-Ground Voltage, Magnitude"
      n_Ul_bus1 = "n:Ul:bus1", "kV, Line-Line Voltage, Magnitude"
      n_phiu_bus1 = "n:phiu:bus1", "deg, Voltage, Angle"
      n_phiurel_bus1 = "n:phiurel:bus1", "deg, Voltage, Relative Angle"
      n_du_bus1 = "n:du:bus1", "%, Voltage Deviation"
      n_Pgen_bus1 = "n:Pgen:bus1", "MW, Generation, Active Power"
      n_Qgen_bus1 = "n:Qgen:bus1", "Mvar, Generation, Reactive Power"
      n_Pmot_bus1 = "n:Pmot:bus1", "MW, Motor Load, Active Power"
      n_Qmot_bus1 = "n:Qmot:bus1", "Mvar, Motor Load, Reactive Power"
      n_Pload_bus1 = "n:Pload:bus1", "MW, General Load, Active Power"
      n_Qload_bus1 = "n:Qload:bus1", "Mvar, General Load, Reactive Power"
      n_Pcomp_bus1 = "n:Pcomp:bus1", "MW, Compensation (Losses)"
      n_Qcomp_bus1 = "n:Qcomp:bus1", "Mvar, Compensation"
      n_Pnet_bus1 = "n:Pnet:bus1", "MW, External Networks, Active Power"
      n_Qnet_bus1 = "n:Qnet:bus1", "Mvar, External Networks, Reactive Power"
      n_Pflow_bus1 = "n:Pflow:bus1", "MW, Power Flow, Active Power"
      n_Qflow_bus1 = "n:Qflow:bus1", "Mvar, Power Flow, Reactive Power"
      n_Pout_bus1 = "n:Pout:bus1", "MW, Outgoing Flow, Active Power"
      n_Qout_bus1 = "n:Qout:bus1", "Mvar, Outgoing Power, Reactive Power"
      n_Sout_bus1 = "n:Sout:bus1", "MVA, Outgoing Power, Apparent Power"
      n_cosphiout_bus1 = "n:cosphiout:bus1", "Outgoing Power, Power Factor"
      n_Pbalance_bus1 = "n:Pbalance:bus1", "MW, Active Power Balance (=0)"
      n_Qbalance_bus1 = "n:Qbalance:bus1", "Mvar, Reactive Power Balance (=0)"
      n_LossPdown_bus1 = "n:LossPdown:bus1", "MW, Losses, downstream"
      n_LossQdown_bus1 = "n:LossQdown:bus1", "Mvar, Losses, downstream (Reactive Power)"
      n_LossPdownload_bus1 = "n:LossPdownload:bus1", "MW, Load losses, downstream"
      n_LossQdownload_bus1 = "n:LossQdownload:bus1", "Mvar, Load losses, downstream"
      n_LossPdownnoload_bus1 = "n:LossPdownnoload:bus1", "MW, No load losses, downstream"
      n_LossQdownnoload_bus1 = "n:LossQdownnoload:bus1", "Mvar, No load losses, downstream"
      n_du_feed_bus1 = "n:du_feed:bus1", "%, Voltage difference relative to feeder begin"
      s_ncapain = "s:ncapain", "IN, Number of Stages in Parallel, input, continuous, double"
      c_z_r = "c:z:r", "p.u., Shunt-Impedance, Real Part"
      c_z_i = "c:z:i", "p.u., Shunt-Impedance, Imaginary Part"
      c_z_phi = "c:z:phi", "deg, Shunt-Impedance, Angle"
      c_z_mag = "c:z:mag", "p.u., Shunt-Impedance, Magnitude"
      c_z_phirad = "c:z:phirad", "rad, Shunt-Impedance, Angle"
      c_cn = "c:cn", "Connection Mode"
      c_frnom = "c:frnom", "Hz, Nominal Frequency"
      c_hpi = "c:hpi", "rad/s, Nominal Angular Frequency"
      c_rs = "c:rs", "p.u., Rs"
      c_gp = "c:gp", "p.u., Gp"
      c_xl = "c:xl", "p.u., Xl"
      c_bc = "c:bc", "p.u., Bc"
      c_b0 = "c:b0", "p.u., Susceptance to Ground"
      c_gpc = "c:gpc", "p.u., Gp(C)"
      c_uc = "c:uc", "p.u., Voltage across Capacitor"
      c_IL = "c:IL", "A, Current through Inductor L"
      c_IC = "c:IC", "A, Current through Capacitor C"
      c_IRp = "c:IRp", "A, Current through Rp"
      c_PRp = "c:PRp", "kW, Losses in Rp"
      c_PL = "c:PL", "kW, Losses in Inductor (Rs)"
      c_url = "c:url", "p.u., Voltage across Inductor (Rs-L)"
      c_Uc = "c:Uc", "kV, Voltage across Capacitor"
      c_Url = "c:Url", "kV, Voltage across Inductor (Rs-L)"
      c_ncapa = "c:ncapa", "Number of Stages in Parallel"
      c_acost = "c:acost", "Annual Cost"
      c_ncapx = "c:ncapx", "Max. No. of Steps"
      c_nCtrlIter = "c:nCtrlIter", "No. of CTRL-iterations"
      c_Tctrl = "c:Tctrl", "Controller Time Constant"
      c_Kctrl = "c:Kctrl", "p.u./%, Controller Sensitivity dq/dv"
      c_Ploss = "c:Ploss", "MW, Losses (total)"
      c_Qloss = "c:Qloss", "Mvar, Reactive-Losses (total)"
      c_Plossld = "c:Plossld", "MW, Losses (load)"
      c_Qlossld = "c:Qlossld", "Mvar, Reactive-Losses (load)"
      c_Plossnld = "c:Plossnld", "MW, Losses (no load)"
      c_Qlossnld = "c:Qlossnld", "Mvar, Reactive-Losses (no load)"

    class ElmSind(Enum):
      _init_ = 'value __doc__'

      m_dPdQ_bus1 = "m:dPdQ:bus1", "MW/Mvar, Branch Sensitivity dPbranch/dQbus"
      m_dPdQ_bus2 = "m:dPdQ:bus2", "MW/Mvar, Branch Sensitivity dPbranch/dQbus"
      m_dQdQ_bus1 = "m:dQdQ:bus1", "Mvar/Mvar, Branch Sensitivity dQbranch/dQbus"
      m_dQdQ_bus2 = "m:dQdQ:bus2", "Mvar/Mvar, Branch Sensitivity dQbranch/dQbus"
      m_dPdP_bus1 = "m:dPdP:bus1", "MW/MW, Branch Sensitivity dPbranch/dPbus"
      m_dPdP_bus2 = "m:dPdP:bus2", "MW/MW, Branch Sensitivity dPbranch/dPbus"
      m_dQdP_bus1 = "m:dQdP:bus1", "Mvar/MW, Branch Sensitivity dQbranch/dPbus"
      m_dQdP_bus2 = "m:dQdP:bus2", "Mvar/MW, Branch Sensitivity dQbranch/dPbus"
      m_dPdtap_bus1 = "m:dPdtap:bus1", "MW/tap, Branch Sensitivity dPbranch/dtap"
      m_dPdtap_bus2 = "m:dPdtap:bus2", "MW/tap, Branch Sensitivity dPbranch/dtap"
      m_dQdtap_bus1 = "m:dQdtap:bus1", "Mvar/tap, Branch Sensitivity dQbranch/dtap"
      m_dQdtap_bus2 = "m:dQdtap:bus2", "Mvar/tap, Branch Sensitivity dQbranch/dtap"
      m_LODF_bus1 = "m:LODF:bus1", "%, Line Outage Distribution Factor LODF"
      m_LODF_bus2 = "m:LODF:bus2", "%, Line Outage Distribution Factor LODF"
      m_u1r_bus1 = "m:u1r:bus1", "p.u., Positive-Sequence-Voltage, Real Part"
      m_u1r_bus2 = "m:u1r:bus2", "p.u., Positive-Sequence-Voltage, Real Part"
      m_u1i_bus1 = "m:u1i:bus1", "p.u., Positive-Sequence-Voltage, Imaginary Part"
      m_u1i_bus2 = "m:u1i:bus2", "p.u., Positive-Sequence-Voltage, Imaginary Part"
      m_u1_bus1 = "m:u1:bus1", "p.u., Positive-Sequence-Voltage, Magnitude"
      m_u1_bus2 = "m:u1:bus2", "p.u., Positive-Sequence-Voltage, Magnitude"
      m_U1_bus1 = "m:U1:bus1", "kV, Line-Ground Positive-Sequence-Voltage, Magnitude"
      m_U1_bus2 = "m:U1:bus2", "kV, Line-Ground Positive-Sequence-Voltage, Magnitude"
      m_U1l_bus1 = "m:U1l:bus1", "kV, Line-Line Positive-Sequence-Voltage, Magnitude"
      m_U1l_bus2 = "m:U1l:bus2", "kV, Line-Line Positive-Sequence-Voltage, Magnitude"
      m_phiu1_bus1 = "m:phiu1:bus1", "deg, Positive-Sequence-Voltage, Angle"
      m_phiu1_bus2 = "m:phiu1:bus2", "deg, Positive-Sequence-Voltage, Angle"
      m_inet_bus1 = "m:inet:bus1", "p.u., Current, Magnitude, referred to network"
      m_inet_bus2 = "m:inet:bus2", "p.u., Current, Magnitude, referred to network"
      m_I_bus1 = "m:I:bus1", "kA, Current, Magnitude"
      m_I_bus2 = "m:I:bus2", "kA, Current, Magnitude"
      m_phii_bus1 = "m:phii:bus1", "deg, Current, Angle"
      m_phii_bus2 = "m:phii:bus2", "deg, Current, Angle"
      m_i1r_bus1 = "m:i1r:bus1", "p.u., Positive-Sequence Current, Real Part"
      m_i1r_bus2 = "m:i1r:bus2", "p.u., Positive-Sequence Current, Real Part"
      m_i1i_bus1 = "m:i1i:bus1", "p.u., Positive-Sequence Current, Imaginary Part"
      m_i1i_bus2 = "m:i1i:bus2", "p.u., Positive-Sequence Current, Imaginary Part"
      m_i1_bus1 = "m:i1:bus1", "p.u., Positive-Sequence Current, Magnitude"
      m_i1_bus2 = "m:i1:bus2", "p.u., Positive-Sequence Current, Magnitude"
      m_I1_bus1 = "m:I1:bus1", "kA, Positive-Sequence Current, Magnitude"
      m_I1_bus2 = "m:I1:bus2", "kA, Positive-Sequence Current, Magnitude"
      m_phii1_bus1 = "m:phii1:bus1", "deg, Positive-Sequence Current, Angle"
      m_phii1_bus2 = "m:phii1:bus2", "deg, Positive-Sequence Current, Angle"
      m_P_bus1 = "m:P:bus1", "MW, Active Power"
      m_P_bus2 = "m:P:bus2", "MW, Active Power"
      m_Q_bus1 = "m:Q:bus1", "Mvar, Reactive Power"
      m_Q_bus2 = "m:Q:bus2", "Mvar, Reactive Power"
      m_S_bus1 = "m:S:bus1", "MVA, Apparent Power"
      m_S_bus2 = "m:S:bus2", "MVA, Apparent Power"
      m_cosphi_bus1 = "m:cosphi:bus1", "Power Factor"
      m_cosphi_bus2 = "m:cosphi:bus2", "Power Factor"
      m_tanphi_bus1 = "m:tanphi:bus1", "tan(phi)"
      m_tanphi_bus2 = "m:tanphi:bus2", "tan(phi)"
      m_Psum_bus1 = "m:Psum:bus1", "MW, Total Active Power"
      m_Psum_bus2 = "m:Psum:bus2", "MW, Total Active Power"
      m_Qsum_bus1 = "m:Qsum:bus1", "Mvar, Total Reactive Power"
      m_Qsum_bus2 = "m:Qsum:bus2", "Mvar, Total Reactive Power"
      m_Ssum_bus1 = "m:Ssum:bus1", "MVA, Total Apparent Power"
      m_Ssum_bus2 = "m:Ssum:bus2", "MVA, Total Apparent Power"
      m_cosphisum_bus1 = "m:cosphisum:bus1", "Total Power Factor"
      m_cosphisum_bus2 = "m:cosphisum:bus2", "Total Power Factor"
      m_tanphisum_bus1 = "m:tanphisum:bus1", "Total tan(phi)"
      m_tanphisum_bus2 = "m:tanphisum:bus2", "Total tan(phi)"
      m_Spu_bus1 = "m:Spu:bus1", "MVA/p.u., Apparent Power per p.u. Voltage"
      m_Spu_bus2 = "m:Spu:bus2", "MVA/p.u., Apparent Power per p.u. Voltage"
      m_i1P_bus1 = "m:i1P:bus1", "p.u., Positive-Sequence Active Current"
      m_i1P_bus2 = "m:i1P:bus2", "p.u., Positive-Sequence Active Current"
      m_i1Q_bus1 = "m:i1Q:bus1", "p.u., Positive-Sequence Reactive Current"
      m_i1Q_bus2 = "m:i1Q:bus2", "p.u., Positive-Sequence Reactive Current"
      m_I1P_bus1 = "m:I1P:bus1", "kA, Positive-Sequence Active Current"
      m_I1P_bus2 = "m:I1P:bus2", "kA, Positive-Sequence Active Current"
      m_I1Q_bus1 = "m:I1Q:bus1", "kA, Positive-Sequence Reactive Current"
      m_I1Q_bus2 = "m:I1Q:bus2", "kA, Positive-Sequence Reactive Current"
      m_ur_bus1 = "m:ur:bus1", "p.u., Voltage, Real Part"
      m_ur_bus2 = "m:ur:bus2", "p.u., Voltage, Real Part"
      m_ui_bus1 = "m:ui:bus1", "p.u., Voltage, Imaginary Part"
      m_ui_bus2 = "m:ui:bus2", "p.u., Voltage, Imaginary Part"
      m_u_bus1 = "m:u:bus1", "p.u., Voltage, Magnitude"
      m_u_bus2 = "m:u:bus2", "p.u., Voltage, Magnitude"
      m_ir_bus1 = "m:ir:bus1", "p.u., Current, Real Part"
      m_ir_bus2 = "m:ir:bus2", "p.u., Current, Real Part"
      m_ii_bus1 = "m:ii:bus1", "p.u., Current, Imaginary Part"
      m_ii_bus2 = "m:ii:bus2", "p.u., Current, Imaginary Part"
      m_i_bus1 = "m:i:bus1", "p.u., Current, Magnitude"
      m_i_bus2 = "m:i:bus2", "p.u., Current, Magnitude"
      m_phiui_bus1 = "m:phiui:bus1", "deg, Angle between Voltage and Current"
      m_phiui_bus2 = "m:phiui:bus2", "deg, Angle between Voltage and Current"
      m_phiu1i1_bus1 = "m:phiu1i1:bus1", "deg, Angle between Voltage and Current in positive sequence system"
      m_phiu1i1_bus2 = "m:phiu1i1:bus2", "deg, Angle between Voltage and Current in positive sequence system"
      m_Tfct_bus1 = "m:Tfct:bus1", "s, Fault Clearing Time"
      m_Tfct_bus2 = "m:Tfct:bus2", "s, Fault Clearing Time"
      m_Brkload_bus1 = "m:Brkload:bus1", "%, Breaker Loading"
      m_Brkload_bus2 = "m:Brkload:bus2", "%, Breaker Loading"
      n_dphidP_bus1 = "n:dphidP:bus1", "deg/MW, dphi/dP Sensitivity"
      n_dphidP_bus2 = "n:dphidP:bus2", "deg/MW, dphi/dP Sensitivity"
      n_dphidQ_bus1 = "n:dphidQ:bus1", "deg/Mvar, dphi/dQ Sensitivity"
      n_dphidQ_bus2 = "n:dphidQ:bus2", "deg/Mvar, dphi/dQ Sensitivity"
      n_dphidtap_bus1 = "n:dphidtap:bus1", "deg/tap, dphi/dtap Sensitivity"
      n_dphidtap_bus2 = "n:dphidtap:bus2", "deg/tap, dphi/dtap Sensitivity"
      n_dvdP_bus1 = "n:dvdP:bus1", "p.u./MW, dv/dP Sensitivity"
      n_dvdP_bus2 = "n:dvdP:bus2", "p.u./MW, dv/dP Sensitivity"
      n_dvdQ_bus1 = "n:dvdQ:bus1", "p.u./Mvar, dv/dQ Sensitivity"
      n_dvdQ_bus2 = "n:dvdQ:bus2", "p.u./Mvar, dv/dQ Sensitivity"
      n_dvdtap_bus1 = "n:dvdtap:bus1", "p.u./tap, dv/dtap Sensitivity"
      n_dvdtap_bus2 = "n:dvdtap:bus2", "p.u./tap, dv/dtap Sensitivity"
      n_levecQ_bus1 = "n:levecQ:bus1", "Left Eigenvector of selected Q-Mode"
      n_levecQ_bus2 = "n:levecQ:bus2", "Left Eigenvector of selected Q-Mode"
      n_revecQ_bus1 = "n:revecQ:bus1", "Right Eigenvector of selected Q-Mode"
      n_revecQ_bus2 = "n:revecQ:bus2", "Right Eigenvector of selected Q-Mode"
      n_partQ_bus1 = "n:partQ:bus1", "Participation Factor of selected Q-Mode"
      n_partQ_bus2 = "n:partQ:bus2", "Participation Factor of selected Q-Mode"
      n_ur_bus1 = "n:ur:bus1", "p.u., Voltage, Real Part"
      n_ur_bus2 = "n:ur:bus2", "p.u., Voltage, Real Part"
      n_ui_bus1 = "n:ui:bus1", "p.u., Voltage, Imaginary Part"
      n_ui_bus2 = "n:ui:bus2", "p.u., Voltage, Imaginary Part"
      n_u_bus1 = "n:u:bus1", "p.u., Voltage, Magnitude"
      n_u_bus2 = "n:u:bus2", "p.u., Voltage, Magnitude"
      n_upc_bus1 = "n:upc:bus1", "%, Voltage, Magnitude"
      n_upc_bus2 = "n:upc:bus2", "%, Voltage, Magnitude"
      n_u1_bus1 = "n:u1:bus1", "p.u., Voltage, Magnitude"
      n_u1_bus2 = "n:u1:bus2", "p.u., Voltage, Magnitude"
      n_u1pc_bus1 = "n:u1pc:bus1", "%, Voltage, Magnitude"
      n_u1pc_bus2 = "n:u1pc:bus2", "%, Voltage, Magnitude"
      n_u1r_bus1 = "n:u1r:bus1", "p.u., Positive-Sequence Voltage, Real Part"
      n_u1r_bus2 = "n:u1r:bus2", "p.u., Positive-Sequence Voltage, Real Part"
      n_u1i_bus1 = "n:u1i:bus1", "p.u., Positive-Sequence Voltage, Imaginary Part"
      n_u1i_bus2 = "n:u1i:bus2", "p.u., Positive-Sequence Voltage, Imaginary Part"
      n_U_bus1 = "n:U:bus1", "kV, Line-Ground Voltage, Magnitude"
      n_U_bus2 = "n:U:bus2", "kV, Line-Ground Voltage, Magnitude"
      n_Ul_bus1 = "n:Ul:bus1", "kV, Line-Line Voltage, Magnitude"
      n_Ul_bus2 = "n:Ul:bus2", "kV, Line-Line Voltage, Magnitude"
      n_phiu_bus1 = "n:phiu:bus1", "deg, Voltage, Angle"
      n_phiu_bus2 = "n:phiu:bus2", "deg, Voltage, Angle"
      n_phiurel_bus1 = "n:phiurel:bus1", "deg, Voltage, Relative Angle"
      n_phiurel_bus2 = "n:phiurel:bus2", "deg, Voltage, Relative Angle"
      n_du_bus1 = "n:du:bus1", "%, Voltage Deviation"
      n_du_bus2 = "n:du:bus2", "%, Voltage Deviation"
      n_Pgen_bus1 = "n:Pgen:bus1", "MW, Generation, Active Power"
      n_Pgen_bus2 = "n:Pgen:bus2", "MW, Generation, Active Power"
      n_Qgen_bus1 = "n:Qgen:bus1", "Mvar, Generation, Reactive Power"
      n_Qgen_bus2 = "n:Qgen:bus2", "Mvar, Generation, Reactive Power"
      n_Pmot_bus1 = "n:Pmot:bus1", "MW, Motor Load, Active Power"
      n_Pmot_bus2 = "n:Pmot:bus2", "MW, Motor Load, Active Power"
      n_Qmot_bus1 = "n:Qmot:bus1", "Mvar, Motor Load, Reactive Power"
      n_Qmot_bus2 = "n:Qmot:bus2", "Mvar, Motor Load, Reactive Power"
      n_Pload_bus1 = "n:Pload:bus1", "MW, General Load, Active Power"
      n_Pload_bus2 = "n:Pload:bus2", "MW, General Load, Active Power"
      n_Qload_bus1 = "n:Qload:bus1", "Mvar, General Load, Reactive Power"
      n_Qload_bus2 = "n:Qload:bus2", "Mvar, General Load, Reactive Power"
      n_Pcomp_bus1 = "n:Pcomp:bus1", "MW, Compensation (Losses)"
      n_Pcomp_bus2 = "n:Pcomp:bus2", "MW, Compensation (Losses)"
      n_Qcomp_bus1 = "n:Qcomp:bus1", "Mvar, Compensation"
      n_Qcomp_bus2 = "n:Qcomp:bus2", "Mvar, Compensation"
      n_Pnet_bus1 = "n:Pnet:bus1", "MW, External Networks, Active Power"
      n_Pnet_bus2 = "n:Pnet:bus2", "MW, External Networks, Active Power"
      n_Qnet_bus1 = "n:Qnet:bus1", "Mvar, External Networks, Reactive Power"
      n_Qnet_bus2 = "n:Qnet:bus2", "Mvar, External Networks, Reactive Power"
      n_Pflow_bus1 = "n:Pflow:bus1", "MW, Power Flow, Active Power"
      n_Pflow_bus2 = "n:Pflow:bus2", "MW, Power Flow, Active Power"
      n_Qflow_bus1 = "n:Qflow:bus1", "Mvar, Power Flow, Reactive Power"
      n_Qflow_bus2 = "n:Qflow:bus2", "Mvar, Power Flow, Reactive Power"
      n_Pout_bus1 = "n:Pout:bus1", "MW, Outgoing Flow, Active Power"
      n_Pout_bus2 = "n:Pout:bus2", "MW, Outgoing Flow, Active Power"
      n_Qout_bus1 = "n:Qout:bus1", "Mvar, Outgoing Power, Reactive Power"
      n_Qout_bus2 = "n:Qout:bus2", "Mvar, Outgoing Power, Reactive Power"
      n_Sout_bus1 = "n:Sout:bus1", "MVA, Outgoing Power, Apparent Power"
      n_Sout_bus2 = "n:Sout:bus2", "MVA, Outgoing Power, Apparent Power"
      n_cosphiout_bus1 = "n:cosphiout:bus1", "Outgoing Power, Power Factor"
      n_cosphiout_bus2 = "n:cosphiout:bus2", "Outgoing Power, Power Factor"
      n_Pbalance_bus1 = "n:Pbalance:bus1", "MW, Active Power Balance (=0)"
      n_Pbalance_bus2 = "n:Pbalance:bus2", "MW, Active Power Balance (=0)"
      n_Qbalance_bus1 = "n:Qbalance:bus1", "Mvar, Reactive Power Balance (=0)"
      n_Qbalance_bus2 = "n:Qbalance:bus2", "Mvar, Reactive Power Balance (=0)"
      n_LossPdown_bus1 = "n:LossPdown:bus1", "MW, Losses, downstream"
      n_LossPdown_bus2 = "n:LossPdown:bus2", "MW, Losses, downstream"
      n_LossQdown_bus1 = "n:LossQdown:bus1", "Mvar, Losses, downstream (Reactive Power)"
      n_LossQdown_bus2 = "n:LossQdown:bus2", "Mvar, Losses, downstream (Reactive Power)"
      n_LossPdownload_bus1 = "n:LossPdownload:bus1", "MW, Load losses, downstream"
      n_LossPdownload_bus2 = "n:LossPdownload:bus2", "MW, Load losses, downstream"
      n_LossQdownload_bus1 = "n:LossQdownload:bus1", "Mvar, Load losses, downstream"
      n_LossQdownload_bus2 = "n:LossQdownload:bus2", "Mvar, Load losses, downstream"
      n_LossPdownnoload_bus1 = "n:LossPdownnoload:bus1", "MW, No load losses, downstream"
      n_LossPdownnoload_bus2 = "n:LossPdownnoload:bus2", "MW, No load losses, downstream"
      n_LossQdownnoload_bus1 = "n:LossQdownnoload:bus1", "Mvar, No load losses, downstream"
      n_LossQdownnoload_bus2 = "n:LossQdownnoload:bus2", "Mvar, No load losses, downstream"
      n_du_feed_bus1 = "n:du_feed:bus1", "%, Voltage difference relative to feeder begin"
      n_du_feed_bus2 = "n:du_feed:bus2", "%, Voltage difference relative to feeder begin"
      s_Rin = "s:Rin", "Ohm, IN, Resistance (Input), continuous, double"
      s_Xin = "s:Xin", "Ohm, IN, Reactance (Input), continuous, double"
      c_dLoadingdP = "c:dLoadingdP", "%/MW, Branch Loading Sensitivity: dLbranch/dPbus"
      c_dLoadingdtap = "c:dLoadingdtap", "%/tap, Branch Loading Sensitivity: dLbranch/dtap"
      c_dPlossdP = "c:dPlossdP", "kW/MW, Active Power Losses Sensitivity: dPloss/dP"
      c_dPlossdQ = "c:dPlossdQ", "kW/Mvar, Active Power Losses Sensitivity: dPloss/dQ"
      c_dQlossdP = "c:dQlossdP", "kvar/MW, Reactive Power Losses Sensitivity: dQloss/dP"
      c_dQlossdQ = "c:dQlossdQ", "kvar/Mvar, Reactive Power Losses Sensitivity: dQloss/dQ"
      c_dPlossdtap = "c:dPlossdtap", "kW/tap, Active Power Losses Sensitivity: dPloss/dtap"
      c_dQlossdtap = "c:dQlossdtap", "kvar/tap, Reactive Power Losses Sensitivity: dQloss/dtap"
      c_Zind_1_r = "c:Zind_1:r", "Ohm, Impedance, Real Part"
      c_Zind_1_i = "c:Zind_1:i", "Ohm, Impedance, Imaginary Part"
      c_Zind_1_phi = "c:Zind_1:phi", "deg, Impedance, Angle"
      c_Zind_1_mag = "c:Zind_1:mag", "Ohm, Impedance, Magnitude"
      c_Zind_1_phirad = "c:Zind_1:phirad", "rad, Impedance, Angle"
      c_loading = "c:loading", "%, Loading"
      c_maxload = "c:maxload", "%, Max. Loading"
      c_du = "c:du", "p.u., Voltage Drop"
      c_dupc = "c:dupc", "%, Voltage Drop"
      c_dphiu = "c:dphiu", "deg, Voltage Drop Angle"
      c_du1 = "c:du1", "p.u., Positive Sequence Voltage Drop"
      c_du1pc = "c:du1pc", "%, Positive Sequence Voltage Drop"
      c_dphiu1 = "c:dphiu1", "deg, Positive Sequence Voltage Drop Angle"
      c_Ploss = "c:Ploss", "MW, Losses (total)"
      c_Qloss = "c:Qloss", "Mvar, Reactive-Losses (total)"
      c_Plossld = "c:Plossld", "MW, Losses (load)"
      c_Qlossld = "c:Qlossld", "Mvar, Reactive-Losses (load)"
      c_Plossnld = "c:Plossnld", "MW, Losses (no load)"
      c_Qlossnld = "c:Qlossnld", "Mvar, Reactive-Losses (no load)"

    class ElmStactrl(Enum):
      _init_ = 'value __doc__'

      c_rembar = "c:rembar", "Controlled Busbar"
      c_uini_r = "c:uini:r", "p.u., Voltage Setpoint, Real Part"
      c_uini_i = "c:uini:i", "p.u., Voltage Setpoint, Imaginary Part"
      c_uini_phi = "c:uini:phi", "deg, Voltage Setpoint, Angle"
      c_uini_mag = "c:uini:mag", "p.u., Voltage Setpoint, Magnitude"
      c_uini_phirad = "c:uini:phirad", "rad, Voltage Setpoint, Angle"
      c_qset = "c:qset", "Reactive Power Percentage"
      c_imode = "c:imode", "Control Mode"
      c_i_ctrl = "c:i_ctrl", "Load Flow Control"
      c_qini = "c:qini", "Mvar, Reactive Power Setpoint"
      c_qres = "c:qres", "Mvar, Reactive Power Reserve"
      c_Qdmeas = "c:Qdmeas", "Mvar, Q(measured) for Droop"
      c_Qdroop = "c:Qdroop", "Mvar/p.u., Droop"
      c_umin = "c:umin", "p.u., Lower Deadband Limit"
      c_umax = "c:umax", "p.u., Upper Deadband Limit"
      c_Qmin = "c:Qmin", "Mvar, Lower limit of reactive power"
      c_Qmax = "c:Qmax", "Mvar, Upper limit of reactive power"
      c_i_qlim = "c:i_qlim", "Reactive power limit reached"
      c_nCtrlIter = "c:nCtrlIter", "No. of CTRL-iterations"

    class ElmSym(Enum):
      _init_ = 'value __doc__'

      m_dPdQ_bus1 = "m:dPdQ:bus1", "MW/Mvar, Branch Sensitivity dPbranch/dQbus"
      m_dQdQ_bus1 = "m:dQdQ:bus1", "Mvar/Mvar, Branch Sensitivity dQbranch/dQbus"
      m_dPdP_bus1 = "m:dPdP:bus1", "MW/MW, Branch Sensitivity dPbranch/dPbus"
      m_dQdP_bus1 = "m:dQdP:bus1", "Mvar/MW, Branch Sensitivity dQbranch/dPbus"
      m_dPdtap_bus1 = "m:dPdtap:bus1", "MW/tap, Branch Sensitivity dPbranch/dtap"
      m_dQdtap_bus1 = "m:dQdtap:bus1", "Mvar/tap, Branch Sensitivity dQbranch/dtap"
      m_LODF_bus1 = "m:LODF:bus1", "%, Line Outage Distribution Factor LODF"
      m_u1r_bus1 = "m:u1r:bus1", "p.u., Positive-Sequence-Voltage, Real Part"
      m_u1i_bus1 = "m:u1i:bus1", "p.u., Positive-Sequence-Voltage, Imaginary Part"
      m_u1_bus1 = "m:u1:bus1", "p.u., Positive-Sequence-Voltage, Magnitude"
      m_U1_bus1 = "m:U1:bus1", "kV, Line-Ground Positive-Sequence-Voltage, Magnitude"
      m_U1l_bus1 = "m:U1l:bus1", "kV, Line-Line Positive-Sequence-Voltage, Magnitude"
      m_phiu1_bus1 = "m:phiu1:bus1", "deg, Positive-Sequence-Voltage, Angle"
      m_inet_bus1 = "m:inet:bus1", "p.u., Current, Magnitude, referred to network"
      m_I_bus1 = "m:I:bus1", "kA, Current, Magnitude"
      m_phii_bus1 = "m:phii:bus1", "deg, Current, Angle"
      m_i1r_bus1 = "m:i1r:bus1", "p.u., Positive-Sequence Current, Real Part"
      m_i1i_bus1 = "m:i1i:bus1", "p.u., Positive-Sequence Current, Imaginary Part"
      m_i1_bus1 = "m:i1:bus1", "p.u., Positive-Sequence Current, Magnitude"
      m_I1_bus1 = "m:I1:bus1", "kA, Positive-Sequence Current, Magnitude"
      m_phii1_bus1 = "m:phii1:bus1", "deg, Positive-Sequence Current, Angle"
      m_P_bus1 = "m:P:bus1", "MW, Active Power"
      m_Q_bus1 = "m:Q:bus1", "Mvar, Reactive Power"
      m_S_bus1 = "m:S:bus1", "MVA, Apparent Power"
      m_cosphi_bus1 = "m:cosphi:bus1", "Power Factor"
      m_tanphi_bus1 = "m:tanphi:bus1", "tan(phi)"
      m_Psum_bus1 = "m:Psum:bus1", "MW, Total Active Power"
      m_Qsum_bus1 = "m:Qsum:bus1", "Mvar, Total Reactive Power"
      m_Ssum_bus1 = "m:Ssum:bus1", "MVA, Total Apparent Power"
      m_cosphisum_bus1 = "m:cosphisum:bus1", "Total Power Factor"
      m_tanphisum_bus1 = "m:tanphisum:bus1", "Total tan(phi)"
      m_Spu_bus1 = "m:Spu:bus1", "MVA/p.u., Apparent Power per p.u. Voltage"
      m_i1P_bus1 = "m:i1P:bus1", "p.u., Positive-Sequence Active Current"
      m_i1Q_bus1 = "m:i1Q:bus1", "p.u., Positive-Sequence Reactive Current"
      m_I1P_bus1 = "m:I1P:bus1", "kA, Positive-Sequence Active Current"
      m_I1Q_bus1 = "m:I1Q:bus1", "kA, Positive-Sequence Reactive Current"
      m_ur_bus1 = "m:ur:bus1", "p.u., Voltage, Real Part"
      m_ui_bus1 = "m:ui:bus1", "p.u., Voltage, Imaginary Part"
      m_u_bus1 = "m:u:bus1", "p.u., Voltage, Magnitude"
      m_ir_bus1 = "m:ir:bus1", "p.u., Current, Real Part"
      m_ii_bus1 = "m:ii:bus1", "p.u., Current, Imaginary Part"
      m_i_bus1 = "m:i:bus1", "p.u., Current, Magnitude"
      m_phiui_bus1 = "m:phiui:bus1", "deg, Angle between Voltage and Current"
      m_phiu1i1_bus1 = "m:phiu1i1:bus1", "deg, Angle between Voltage and Current in positive sequence system"
      m_Tfct_bus1 = "m:Tfct:bus1", "s, Fault Clearing Time"
      m_Brkload_bus1 = "m:Brkload:bus1", "%, Breaker Loading"
      n_dphidP_bus1 = "n:dphidP:bus1", "deg/MW, dphi/dP Sensitivity"
      n_dphidQ_bus1 = "n:dphidQ:bus1", "deg/Mvar, dphi/dQ Sensitivity"
      n_dphidtap_bus1 = "n:dphidtap:bus1", "deg/tap, dphi/dtap Sensitivity"
      n_dvdP_bus1 = "n:dvdP:bus1", "p.u./MW, dv/dP Sensitivity"
      n_dvdQ_bus1 = "n:dvdQ:bus1", "p.u./Mvar, dv/dQ Sensitivity"
      n_dvdtap_bus1 = "n:dvdtap:bus1", "p.u./tap, dv/dtap Sensitivity"
      n_levecQ_bus1 = "n:levecQ:bus1", "Left Eigenvector of selected Q-Mode"
      n_revecQ_bus1 = "n:revecQ:bus1", "Right Eigenvector of selected Q-Mode"
      n_partQ_bus1 = "n:partQ:bus1", "Participation Factor of selected Q-Mode"
      n_ur_bus1 = "n:ur:bus1", "p.u., Voltage, Real Part"
      n_ui_bus1 = "n:ui:bus1", "p.u., Voltage, Imaginary Part"
      n_u_bus1 = "n:u:bus1", "p.u., Voltage, Magnitude"
      n_upc_bus1 = "n:upc:bus1", "%, Voltage, Magnitude"
      n_u1_bus1 = "n:u1:bus1", "p.u., Voltage, Magnitude"
      n_u1pc_bus1 = "n:u1pc:bus1", "%, Voltage, Magnitude"
      n_u1r_bus1 = "n:u1r:bus1", "p.u., Positive-Sequence Voltage, Real Part"
      n_u1i_bus1 = "n:u1i:bus1", "p.u., Positive-Sequence Voltage, Imaginary Part"
      n_U_bus1 = "n:U:bus1", "kV, Line-Ground Voltage, Magnitude"
      n_Ul_bus1 = "n:Ul:bus1", "kV, Line-Line Voltage, Magnitude"
      n_phiu_bus1 = "n:phiu:bus1", "deg, Voltage, Angle"
      n_phiurel_bus1 = "n:phiurel:bus1", "deg, Voltage, Relative Angle"
      n_du_bus1 = "n:du:bus1", "%, Voltage Deviation"
      n_Pgen_bus1 = "n:Pgen:bus1", "MW, Generation, Active Power"
      n_Qgen_bus1 = "n:Qgen:bus1", "Mvar, Generation, Reactive Power"
      n_Pmot_bus1 = "n:Pmot:bus1", "MW, Motor Load, Active Power"
      n_Qmot_bus1 = "n:Qmot:bus1", "Mvar, Motor Load, Reactive Power"
      n_Pload_bus1 = "n:Pload:bus1", "MW, General Load, Active Power"
      n_Qload_bus1 = "n:Qload:bus1", "Mvar, General Load, Reactive Power"
      n_Pcomp_bus1 = "n:Pcomp:bus1", "MW, Compensation (Losses)"
      n_Qcomp_bus1 = "n:Qcomp:bus1", "Mvar, Compensation"
      n_Pnet_bus1 = "n:Pnet:bus1", "MW, External Networks, Active Power"
      n_Qnet_bus1 = "n:Qnet:bus1", "Mvar, External Networks, Reactive Power"
      n_Pflow_bus1 = "n:Pflow:bus1", "MW, Power Flow, Active Power"
      n_Qflow_bus1 = "n:Qflow:bus1", "Mvar, Power Flow, Reactive Power"
      n_Pout_bus1 = "n:Pout:bus1", "MW, Outgoing Flow, Active Power"
      n_Qout_bus1 = "n:Qout:bus1", "Mvar, Outgoing Power, Reactive Power"
      n_Sout_bus1 = "n:Sout:bus1", "MVA, Outgoing Power, Apparent Power"
      n_cosphiout_bus1 = "n:cosphiout:bus1", "Outgoing Power, Power Factor"
      n_Pbalance_bus1 = "n:Pbalance:bus1", "MW, Active Power Balance (=0)"
      n_Qbalance_bus1 = "n:Qbalance:bus1", "Mvar, Reactive Power Balance (=0)"
      n_LossPdown_bus1 = "n:LossPdown:bus1", "MW, Losses, downstream"
      n_LossQdown_bus1 = "n:LossQdown:bus1", "Mvar, Losses, downstream (Reactive Power)"
      n_LossPdownload_bus1 = "n:LossPdownload:bus1", "MW, Load losses, downstream"
      n_LossQdownload_bus1 = "n:LossQdownload:bus1", "Mvar, Load losses, downstream"
      n_LossPdownnoload_bus1 = "n:LossPdownnoload:bus1", "MW, No load losses, downstream"
      n_LossQdownnoload_bus1 = "n:LossQdownnoload:bus1", "Mvar, No load losses, downstream"
      n_du_feed_bus1 = "n:du_feed:bus1", "%, Voltage difference relative to feeder begin"
      s_pset = "s:pset", "MW, IN, Active Power Setpoint, continuous, double"
      s_qset = "s:qset", "Mvar, IN, Reactive Power Setpoint continuous, double"
      s_dFin = "s:dFin", "Hz, IN, Frequency Deviation, continuous, double"
      s_dF = "s:dF", "Hz, OUT, Frequency Deviation Output, continuous, double"
      c_p = "c:p", "MW, Total Active Power"
      c_q = "c:q", "Mvar, Total Reactive Power"
      c_sgnn = "c:sgnn", "MVA, Total rated apparent power (ngnum*sgn)"
      c_cosn = "c:cosn", "Nominal-Power factor"
      c_Pnom = "c:Pnom", "MW, Total Nominal Active power"
      c_c2 = "c:c2", "Voltage Coefficient (uknom/ugn)"
      c_uini_r = "c:uini:r", "p.u., Voltage Setpoint for PV and Slack, Real Part"
      c_uini_i = "c:uini:i", "p.u., Voltage Setpoint for PV and Slack, Imaginary Part"
      c_uini_phi = "c:uini:phi", "deg, Voltage Setpoint for PV and Slack, Angle"
      c_uini_mag = "c:uini:mag", "p.u., Voltage Setpoint for PV and Slack, Magnitude"
      c_uini_phirad = "c:uini:phirad", "rad, Voltage Setpoint for PV and Slack, Angle"
      c_phiini = "c:phiini", "Initial Angle"
      c_loading = "c:loading", "%, Loading"
      c_iPowerOn = "c:iPowerOn", "Flag"
      c_xdq = "c:xdq", "p.u., Synchronous Reactance"
      c_Qmin = "c:Qmin", "Mvar, Lower limit of reactive power"
      c_Qmax = "c:Qmax", "Mvar, Upper limit of reactive power"
      c_Qsparelead = "c:Qsparelead", "Mvar, Q spare lead"
      c_Qsparelag = "c:Qsparelag", "Mvar, Q spare lag"
      c_i_qlim = "c:i_qlim", "Reactive power limit reached"
      c_i_vlim = "c:i_vlim", "Voltage setpoint limit reached"
      c_Pmin = "c:Pmin", "MW, Lower limit of active power"
      c_Pmax = "c:Pmax", "MW, Upper limit of active power"
      c_i_plim = "c:i_plim", "Active power limit reached"
      c_sm_r = "c:sm:r", "MW, Complex Power, Real Part"
      c_sm_i = "c:sm:i", "Mvar, Complex Power, Imaginary Part"
      c_sm_phi = "c:sm:phi", "deg, Complex Power, Angle"
      c_sm_mag = "c:sm:mag", "MVA, Complex Power, Magnitude"
      c_sm_phirad = "c:sm:phirad", "rad, Complex Power, Angle"
      c_u1_r = "c:u1:r", "p.u., Internal, Real Part"
      c_u1_i = "c:u1:i", "p.u., Internal, Imaginary Part"
      c_u1_phi = "c:u1:phi", "deg, Internal, Angle"
      c_u1_mag = "c:u1:mag", "p.u., Internal, Magnitude"
      c_u1_phirad = "c:u1:phirad", "rad, Internal, Angle"
      c_i_spin = "c:i_spin", "Flag"
      c_iRefElement = "c:iRefElement", "Reference Element"
      c_Snom = "c:Snom", "MVA, Nom. Apparent Power (1 mach.)"
      c_Nnom = "c:Nnom", "Number of Parallel Machines"
      c_Kp = "c:Kp", "p.u., Active Power Participation"
      c_Kq = "c:Kq", "p.u., Reactive Power Participation"
      c_Kpf = "c:Kpf", "MW/Hz, Frequency Bias"
      c_Pdisp = "c:Pdisp", "MW, Power (Eco-Dispatch)"
      c_cst_disp = "c:cst_disp", "USD/h, Operating Costs"
      c_zonewscale = "c:zonewscale", "Wind generation zone scaling factor"
      c_Pmism = "c:Pmism", "MW, Active Power Mismatch"
      c_nCtrlIter = "c:nCtrlIter", "No. of CTRL-iterations"
      c_Ploss = "c:Ploss", "MW, Losses (total)"
      c_Qloss = "c:Qloss", "Mvar, Reactive-Losses (total)"
      c_Plossld = "c:Plossld", "MW, Losses (load)"
      c_Qlossld = "c:Qlossld", "Mvar, Reactive-Losses (load)"
      c_Plossnld = "c:Plossnld", "MW, Losses (no load)"
      c_Qlossnld = "c:Qlossnld", "Mvar, Reactive-Losses (no load)"

    class ElmTerm(Enum):
      _init_ = 'value __doc__'

      m_dphidP = "m:dphidP", "deg/MW, dphi/dP Sensitivity"
      m_dphidQ = "m:dphidQ", "deg/Mvar, dphi/dQ Sensitivity"
      m_dphidtap = "m:dphidtap", "deg/tap, dphi/dtap Sensitivity"
      m_dvdP = "m:dvdP", "p.u./MW, dv/dP Sensitivity"
      m_dvdQ = "m:dvdQ", "p.u./Mvar, dv/dQ Sensitivity"
      m_dvdtap = "m:dvdtap", "p.u./tap, dv/dtap Sensitivity"
      m_levecQ = "m:levecQ", "Left Eigenvector of selected Q-Mode"
      m_revecQ = "m:revecQ", "Right Eigenvector of selected Q-Mode"
      m_partQ = "m:partQ", "Participation Factor of selected Q-Mode"
      m_ur = "m:ur", "p.u., Voltage, Real Part"
      m_ui = "m:ui", "p.u., Voltage, Imaginary Part"
      m_u = "m:u", "p.u., Voltage, Magnitude"
      m_upc = "m:upc", "%, Voltage, Magnitude"
      m_u1 = "m:u1", "p.u., Voltage, Magnitude"
      m_u1pc = "m:u1pc", "%, Voltage, Magnitude"
      m_u1r = "m:u1r", "p.u., Positive-Sequence Voltage, Real Part"
      m_u1i = "m:u1i", "p.u., Positive-Sequence Voltage, Imaginary Part"
      m_U = "m:U", "kV, Line-Ground Voltage, Magnitude"
      m_Ul = "m:Ul", "kV, Line-Line Voltage, Magnitude"
      m_phiu = "m:phiu", "deg, Voltage, Angle"
      m_phiurel = "m:phiurel", "deg, Voltage, Relative Angle"
      m_du = "m:du", "%, Voltage Deviation"
      m_Pgen = "m:Pgen", "MW, Generation, Active Power"
      m_Qgen = "m:Qgen", "Mvar, Generation, Reactive Power"
      m_Pmot = "m:Pmot", "MW, Motor Load, Active Power"
      m_Qmot = "m:Qmot", "Mvar, Motor Load, Reactive Power"
      m_Pload = "m:Pload", "MW, General Load, Active Power"
      m_Qload = "m:Qload", "Mvar, General Load, Reactive Power"
      m_Pcomp = "m:Pcomp", "MW, Compensation (Losses)"
      m_Qcomp = "m:Qcomp", "Mvar, Compensation"
      m_Pnet = "m:Pnet", "MW, External Networks, Active Power"
      m_Qnet = "m:Qnet", "Mvar, External Networks, Reactive Power"
      m_Pflow = "m:Pflow", "MW, Power Flow, Active Power"
      m_Qflow = "m:Qflow", "Mvar, Power Flow, Reactive Power"
      m_Pout = "m:Pout", "MW, Outgoing Flow, Active Power"
      m_Qout = "m:Qout", "Mvar, Outgoing Power, Reactive Power"
      m_Sout = "m:Sout", "MVA, Outgoing Power, Apparent Power"
      m_cosphiout = "m:cosphiout", "Outgoing Power, Power Factor"
      m_Pbalance = "m:Pbalance", "MW, Active Power Balance (=0)"
      m_Qbalance = "m:Qbalance", "Mvar, Reactive Power Balance (=0)"
      m_LossPdown = "m:LossPdown", "MW, Losses, downstream"
      m_LossQdown = "m:LossQdown", "Mvar, Losses, downstream (Reactive Power)"
      m_LossPdownload = "m:LossPdownload", "MW, Load losses, downstream"
      m_LossQdownload = "m:LossQdownload", "Mvar, Load losses, downstream"
      m_LossPdownnoload = "m:LossPdownnoload", "MW, No load losses, downstream"
      m_LossQdownnoload = "m:LossQdownnoload", "Mvar, No load losses, downstream"
      m_du_feed = "m:du_feed", "%, Voltage difference relative to feeder begin"
      m_Imax = "m:Imax", "kA, Max. Current"
      m_loading = "m:loading", "%, Loading"

    class ElmTr2(Enum):
      _init_ = 'value __doc__'

      m_dPdQ_bushv = "m:dPdQ:bushv", "MW/Mvar, Branch Sensitivity dPbranch/dQbus"
      m_dPdQ_buslv = "m:dPdQ:buslv", "MW/Mvar, Branch Sensitivity dPbranch/dQbus"
      m_dQdQ_bushv = "m:dQdQ:bushv", "Mvar/Mvar, Branch Sensitivity dQbranch/dQbus"
      m_dQdQ_buslv = "m:dQdQ:buslv", "Mvar/Mvar, Branch Sensitivity dQbranch/dQbus"
      m_dPdP_bushv = "m:dPdP:bushv", "MW/MW, Branch Sensitivity dPbranch/dPbus"
      m_dPdP_buslv = "m:dPdP:buslv", "MW/MW, Branch Sensitivity dPbranch/dPbus"
      m_dQdP_bushv = "m:dQdP:bushv", "Mvar/MW, Branch Sensitivity dQbranch/dPbus"
      m_dQdP_buslv = "m:dQdP:buslv", "Mvar/MW, Branch Sensitivity dQbranch/dPbus"
      m_dPdtap_bushv = "m:dPdtap:bushv", "MW/tap, Branch Sensitivity dPbranch/dtap"
      m_dPdtap_buslv = "m:dPdtap:buslv", "MW/tap, Branch Sensitivity dPbranch/dtap"
      m_dQdtap_bushv = "m:dQdtap:bushv", "Mvar/tap, Branch Sensitivity dQbranch/dtap"
      m_dQdtap_buslv = "m:dQdtap:buslv", "Mvar/tap, Branch Sensitivity dQbranch/dtap"
      m_LODF_bushv = "m:LODF:bushv", "%, Line Outage Distribution Factor LODF"
      m_LODF_buslv = "m:LODF:buslv", "%, Line Outage Distribution Factor LODF"
      m_u1r_bushv = "m:u1r:bushv", "p.u., Positive-Sequence-Voltage, Real Part"
      m_u1r_buslv = "m:u1r:buslv", "p.u., Positive-Sequence-Voltage, Real Part"
      m_u1i_bushv = "m:u1i:bushv", "p.u., Positive-Sequence-Voltage, Imaginary Part"
      m_u1i_buslv = "m:u1i:buslv", "p.u., Positive-Sequence-Voltage, Imaginary Part"
      m_u1_bushv = "m:u1:bushv", "p.u., Positive-Sequence-Voltage, Magnitude"
      m_u1_buslv = "m:u1:buslv", "p.u., Positive-Sequence-Voltage, Magnitude"
      m_U1_bushv = "m:U1:bushv", "kV, Line-Ground Positive-Sequence-Voltage, Magnitude"
      m_U1_buslv = "m:U1:buslv", "kV, Line-Ground Positive-Sequence-Voltage, Magnitude"
      m_U1l_bushv = "m:U1l:bushv", "kV, Line-Line Positive-Sequence-Voltage, Magnitude"
      m_U1l_buslv = "m:U1l:buslv", "kV, Line-Line Positive-Sequence-Voltage, Magnitude"
      m_phiu1_bushv = "m:phiu1:bushv", "deg, Positive-Sequence-Voltage, Angle"
      m_phiu1_buslv = "m:phiu1:buslv", "deg, Positive-Sequence-Voltage, Angle"
      m_inet_bushv = "m:inet:bushv", "p.u., Current, Magnitude, referred to network"
      m_inet_buslv = "m:inet:buslv", "p.u., Current, Magnitude, referred to network"
      m_I_bushv = "m:I:bushv", "kA, Current, Magnitude"
      m_I_buslv = "m:I:buslv", "kA, Current, Magnitude"
      m_phii_bushv = "m:phii:bushv", "deg, Current, Angle"
      m_phii_buslv = "m:phii:buslv", "deg, Current, Angle"
      m_i1r_bushv = "m:i1r:bushv", "p.u., Positive-Sequence Current, Real Part"
      m_i1r_buslv = "m:i1r:buslv", "p.u., Positive-Sequence Current, Real Part"
      m_i1i_bushv = "m:i1i:bushv", "p.u., Positive-Sequence Current, Imaginary Part"
      m_i1i_buslv = "m:i1i:buslv", "p.u., Positive-Sequence Current, Imaginary Part"
      m_i1_bushv = "m:i1:bushv", "p.u., Positive-Sequence Current, Magnitude"
      m_i1_buslv = "m:i1:buslv", "p.u., Positive-Sequence Current, Magnitude"
      m_I1_bushv = "m:I1:bushv", "kA, Positive-Sequence Current, Magnitude"
      m_I1_buslv = "m:I1:buslv", "kA, Positive-Sequence Current, Magnitude"
      m_phii1_bushv = "m:phii1:bushv", "deg, Positive-Sequence Current, Angle"
      m_phii1_buslv = "m:phii1:buslv", "deg, Positive-Sequence Current, Angle"
      m_P_bushv = "m:P:bushv", "MW, Active Power"
      m_P_buslv = "m:P:buslv", "MW, Active Power"
      m_Q_bushv = "m:Q:bushv", "Mvar, Reactive Power"
      m_Q_buslv = "m:Q:buslv", "Mvar, Reactive Power"
      m_S_bushv = "m:S:bushv", "MVA, Apparent Power"
      m_S_buslv = "m:S:buslv", "MVA, Apparent Power"
      m_cosphi_bushv = "m:cosphi:bushv", "Power Factor"
      m_cosphi_buslv = "m:cosphi:buslv", "Power Factor"
      m_tanphi_bushv = "m:tanphi:bushv", "tan(phi)"
      m_tanphi_buslv = "m:tanphi:buslv", "tan(phi)"
      m_Psum_bushv = "m:Psum:bushv", "MW, Total Active Power"
      m_Psum_buslv = "m:Psum:buslv", "MW, Total Active Power"
      m_Qsum_bushv = "m:Qsum:bushv", "Mvar, Total Reactive Power"
      m_Qsum_buslv = "m:Qsum:buslv", "Mvar, Total Reactive Power"
      m_Ssum_bushv = "m:Ssum:bushv", "MVA, Total Apparent Power"
      m_Ssum_buslv = "m:Ssum:buslv", "MVA, Total Apparent Power"
      m_cosphisum_bushv = "m:cosphisum:bushv", "Total Power Factor"
      m_cosphisum_buslv = "m:cosphisum:buslv", "Total Power Factor"
      m_tanphisum_bushv = "m:tanphisum:bushv", "Total tan(phi)"
      m_tanphisum_buslv = "m:tanphisum:buslv", "Total tan(phi)"
      m_Spu_bushv = "m:Spu:bushv", "MVA/p.u., Apparent Power per p.u. Voltage"
      m_Spu_buslv = "m:Spu:buslv", "MVA/p.u., Apparent Power per p.u. Voltage"
      m_i1P_bushv = "m:i1P:bushv", "p.u., Positive-Sequence Active Current"
      m_i1P_buslv = "m:i1P:buslv", "p.u., Positive-Sequence Active Current"
      m_i1Q_bushv = "m:i1Q:bushv", "p.u., Positive-Sequence Reactive Current"
      m_i1Q_buslv = "m:i1Q:buslv", "p.u., Positive-Sequence Reactive Current"
      m_I1P_bushv = "m:I1P:bushv", "kA, Positive-Sequence Active Current"
      m_I1P_buslv = "m:I1P:buslv", "kA, Positive-Sequence Active Current"
      m_I1Q_bushv = "m:I1Q:bushv", "kA, Positive-Sequence Reactive Current"
      m_I1Q_buslv = "m:I1Q:buslv", "kA, Positive-Sequence Reactive Current"
      m_ur_bushv = "m:ur:bushv", "p.u., Voltage, Real Part"
      m_ur_buslv = "m:ur:buslv", "p.u., Voltage, Real Part"
      m_ui_bushv = "m:ui:bushv", "p.u., Voltage, Imaginary Part"
      m_ui_buslv = "m:ui:buslv", "p.u., Voltage, Imaginary Part"
      m_u_bushv = "m:u:bushv", "p.u., Voltage, Magnitude"
      m_u_buslv = "m:u:buslv", "p.u., Voltage, Magnitude"
      m_ir_bushv = "m:ir:bushv", "p.u., Current, Real Part"
      m_ir_buslv = "m:ir:buslv", "p.u., Current, Real Part"
      m_ii_bushv = "m:ii:bushv", "p.u., Current, Imaginary Part"
      m_ii_buslv = "m:ii:buslv", "p.u., Current, Imaginary Part"
      m_i_bushv = "m:i:bushv", "p.u., Current, Magnitude"
      m_i_buslv = "m:i:buslv", "p.u., Current, Magnitude"
      m_phiui_bushv = "m:phiui:bushv", "deg, Angle between Voltage and Current"
      m_phiui_buslv = "m:phiui:buslv", "deg, Angle between Voltage and Current"
      m_phiu1i1_bushv = "m:phiu1i1:bushv", "deg, Angle between Voltage and Current in positive sequence system"
      m_phiu1i1_buslv = "m:phiu1i1:buslv", "deg, Angle between Voltage and Current in positive sequence system"
      m_Tfct_bushv = "m:Tfct:bushv", "s, Fault Clearing Time"
      m_Tfct_buslv = "m:Tfct:buslv", "s, Fault Clearing Time"
      m_Brkload_bushv = "m:Brkload:bushv", "%, Breaker Loading"
      m_Brkload_buslv = "m:Brkload:buslv", "%, Breaker Loading"
      n_dphidP_bushv = "n:dphidP:bushv", "deg/MW, dphi/dP Sensitivity"
      n_dphidP_buslv = "n:dphidP:buslv", "deg/MW, dphi/dP Sensitivity"
      n_dphidQ_bushv = "n:dphidQ:bushv", "deg/Mvar, dphi/dQ Sensitivity"
      n_dphidQ_buslv = "n:dphidQ:buslv", "deg/Mvar, dphi/dQ Sensitivity"
      n_dphidtap_bushv = "n:dphidtap:bushv", "deg/tap, dphi/dtap Sensitivity"
      n_dphidtap_buslv = "n:dphidtap:buslv", "deg/tap, dphi/dtap Sensitivity"
      n_dvdP_bushv = "n:dvdP:bushv", "p.u./MW, dv/dP Sensitivity"
      n_dvdP_buslv = "n:dvdP:buslv", "p.u./MW, dv/dP Sensitivity"
      n_dvdQ_bushv = "n:dvdQ:bushv", "p.u./Mvar, dv/dQ Sensitivity"
      n_dvdQ_buslv = "n:dvdQ:buslv", "p.u./Mvar, dv/dQ Sensitivity"
      n_dvdtap_bushv = "n:dvdtap:bushv", "p.u./tap, dv/dtap Sensitivity"
      n_dvdtap_buslv = "n:dvdtap:buslv", "p.u./tap, dv/dtap Sensitivity"
      n_levecQ_bushv = "n:levecQ:bushv", "Left Eigenvector of selected Q-Mode"
      n_levecQ_buslv = "n:levecQ:buslv", "Left Eigenvector of selected Q-Mode"
      n_revecQ_bushv = "n:revecQ:bushv", "Right Eigenvector of selected Q-Mode"
      n_revecQ_buslv = "n:revecQ:buslv", "Right Eigenvector of selected Q-Mode"
      n_partQ_bushv = "n:partQ:bushv", "Participation Factor of selected Q-Mode"
      n_partQ_buslv = "n:partQ:buslv", "Participation Factor of selected Q-Mode"
      n_ur_bushv = "n:ur:bushv", "p.u., Voltage, Real Part"
      n_ur_buslv = "n:ur:buslv", "p.u., Voltage, Real Part"
      n_ui_bushv = "n:ui:bushv", "p.u., Voltage, Imaginary Part"
      n_ui_buslv = "n:ui:buslv", "p.u., Voltage, Imaginary Part"
      n_u_bushv = "n:u:bushv", "p.u., Voltage, Magnitude"
      n_u_buslv = "n:u:buslv", "p.u., Voltage, Magnitude"
      n_upc_bushv = "n:upc:bushv", "%, Voltage, Magnitude"
      n_upc_buslv = "n:upc:buslv", "%, Voltage, Magnitude"
      n_u1_bushv = "n:u1:bushv", "p.u., Voltage, Magnitude"
      n_u1_buslv = "n:u1:buslv", "p.u., Voltage, Magnitude"
      n_u1pc_bushv = "n:u1pc:bushv", "%, Voltage, Magnitude"
      n_u1pc_buslv = "n:u1pc:buslv", "%, Voltage, Magnitude"
      n_u1r_bushv = "n:u1r:bushv", "p.u., Positive-Sequence Voltage, Real Part"
      n_u1r_buslv = "n:u1r:buslv", "p.u., Positive-Sequence Voltage, Real Part"
      n_u1i_bushv = "n:u1i:bushv", "p.u., Positive-Sequence Voltage, Imaginary Part"
      n_u1i_buslv = "n:u1i:buslv", "p.u., Positive-Sequence Voltage, Imaginary Part"
      n_U_bushv = "n:U:bushv", "kV, Line-Ground Voltage, Magnitude"
      n_U_buslv = "n:U:buslv", "kV, Line-Ground Voltage, Magnitude"
      n_Ul_bushv = "n:Ul:bushv", "kV, Line-Line Voltage, Magnitude"
      n_Ul_buslv = "n:Ul:buslv", "kV, Line-Line Voltage, Magnitude"
      n_phiu_bushv = "n:phiu:bushv", "deg, Voltage, Angle"
      n_phiu_buslv = "n:phiu:buslv", "deg, Voltage, Angle"
      n_phiurel_bushv = "n:phiurel:bushv", "deg, Voltage, Relative Angle"
      n_phiurel_buslv = "n:phiurel:buslv", "deg, Voltage, Relative Angle"
      n_du_bushv = "n:du:bushv", "%, Voltage Deviation"
      n_du_buslv = "n:du:buslv", "%, Voltage Deviation"
      n_Pgen_bushv = "n:Pgen:bushv", "MW, Generation, Active Power"
      n_Pgen_buslv = "n:Pgen:buslv", "MW, Generation, Active Power"
      n_Qgen_bushv = "n:Qgen:bushv", "Mvar, Generation, Reactive Power"
      n_Qgen_buslv = "n:Qgen:buslv", "Mvar, Generation, Reactive Power"
      n_Pmot_bushv = "n:Pmot:bushv", "MW, Motor Load, Active Power"
      n_Pmot_buslv = "n:Pmot:buslv", "MW, Motor Load, Active Power"
      n_Qmot_bushv = "n:Qmot:bushv", "Mvar, Motor Load, Reactive Power"
      n_Qmot_buslv = "n:Qmot:buslv", "Mvar, Motor Load, Reactive Power"
      n_Pload_bushv = "n:Pload:bushv", "MW, General Load, Active Power"
      n_Pload_buslv = "n:Pload:buslv", "MW, General Load, Active Power"
      n_Qload_bushv = "n:Qload:bushv", "Mvar, General Load, Reactive Power"
      n_Qload_buslv = "n:Qload:buslv", "Mvar, General Load, Reactive Power"
      n_Pcomp_bushv = "n:Pcomp:bushv", "MW, Compensation (Losses)"
      n_Pcomp_buslv = "n:Pcomp:buslv", "MW, Compensation (Losses)"
      n_Qcomp_bushv = "n:Qcomp:bushv", "Mvar, Compensation"
      n_Qcomp_buslv = "n:Qcomp:buslv", "Mvar, Compensation"
      n_Pnet_bushv = "n:Pnet:bushv", "MW, External Networks, Active Power"
      n_Pnet_buslv = "n:Pnet:buslv", "MW, External Networks, Active Power"
      n_Qnet_bushv = "n:Qnet:bushv", "Mvar, External Networks, Reactive Power"
      n_Qnet_buslv = "n:Qnet:buslv", "Mvar, External Networks, Reactive Power"
      n_Pflow_bushv = "n:Pflow:bushv", "MW, Power Flow, Active Power"
      n_Pflow_buslv = "n:Pflow:buslv", "MW, Power Flow, Active Power"
      n_Qflow_bushv = "n:Qflow:bushv", "Mvar, Power Flow, Reactive Power"
      n_Qflow_buslv = "n:Qflow:buslv", "Mvar, Power Flow, Reactive Power"
      n_Pout_bushv = "n:Pout:bushv", "MW, Outgoing Flow, Active Power"
      n_Pout_buslv = "n:Pout:buslv", "MW, Outgoing Flow, Active Power"
      n_Qout_bushv = "n:Qout:bushv", "Mvar, Outgoing Power, Reactive Power"
      n_Qout_buslv = "n:Qout:buslv", "Mvar, Outgoing Power, Reactive Power"
      n_Sout_bushv = "n:Sout:bushv", "MVA, Outgoing Power, Apparent Power"
      n_Sout_buslv = "n:Sout:buslv", "MVA, Outgoing Power, Apparent Power"
      n_cosphiout_bushv = "n:cosphiout:bushv", "Outgoing Power, Power Factor"
      n_cosphiout_buslv = "n:cosphiout:buslv", "Outgoing Power, Power Factor"
      n_Pbalance_bushv = "n:Pbalance:bushv", "MW, Active Power Balance (=0)"
      n_Pbalance_buslv = "n:Pbalance:buslv", "MW, Active Power Balance (=0)"
      n_Qbalance_bushv = "n:Qbalance:bushv", "Mvar, Reactive Power Balance (=0)"
      n_Qbalance_buslv = "n:Qbalance:buslv", "Mvar, Reactive Power Balance (=0)"
      n_LossPdown_bushv = "n:LossPdown:bushv", "MW, Losses, downstream"
      n_LossPdown_buslv = "n:LossPdown:buslv", "MW, Losses, downstream"
      n_LossQdown_bushv = "n:LossQdown:bushv", "Mvar, Losses, downstream (Reactive Power)"
      n_LossQdown_buslv = "n:LossQdown:buslv", "Mvar, Losses, downstream (Reactive Power)"
      n_LossPdownload_bushv = "n:LossPdownload:bushv", "MW, Load losses, downstream"
      n_LossPdownload_buslv = "n:LossPdownload:buslv", "MW, Load losses, downstream"
      n_LossQdownload_bushv = "n:LossQdownload:bushv", "Mvar, Load losses, downstream"
      n_LossQdownload_buslv = "n:LossQdownload:buslv", "Mvar, Load losses, downstream"
      n_LossPdownnoload_bushv = "n:LossPdownnoload:bushv", "MW, No load losses, downstream"
      n_LossPdownnoload_buslv = "n:LossPdownnoload:buslv", "MW, No load losses, downstream"
      n_LossQdownnoload_bushv = "n:LossQdownnoload:bushv", "Mvar, No load losses, downstream"
      n_LossQdownnoload_buslv = "n:LossQdownnoload:buslv", "Mvar, No load losses, downstream"
      n_du_feed_bushv = "n:du_feed:bushv", "%, Voltage difference relative to feeder begin"
      n_du_feed_buslv = "n:du_feed:buslv", "%, Voltage difference relative to feeder begin"
      s_nt_in = "s:nt_in", "IN, Tap 1, Input, continuous, double"
      s_nt2_in = "s:nt2_in", "IN, Tap 2, Input, continuous, double"
      c_dLoadingdP = "c:dLoadingdP", "%/MW, Branch Loading Sensitivity: dLbranch/dPbus"
      c_dLoadingdtap = "c:dLoadingdtap", "%/tap, Branch Loading Sensitivity: dLbranch/dtap"
      c_dPlossdP = "c:dPlossdP", "kW/MW, Active Power Losses Sensitivity: dPloss/dP"
      c_dPlossdQ = "c:dPlossdQ", "kW/Mvar, Active Power Losses Sensitivity: dPloss/dQ"
      c_dQlossdP = "c:dQlossdP", "kvar/MW, Reactive Power Losses Sensitivity: dQloss/dP"
      c_dQlossdQ = "c:dQlossdQ", "kvar/Mvar, Reactive Power Losses Sensitivity: dQloss/dQ"
      c_dPlossdtap = "c:dPlossdtap", "kW/tap, Active Power Losses Sensitivity: dPloss/dtap"
      c_dQlossdtap = "c:dQlossdtap", "kvar/tap, Reactive Power Losses Sensitivity: dQloss/dtap"
      c_loading_h = "c:loading_h", "%, Loading (HV)"
      c_loading_l = "c:loading_l", "%, Loading (LV)"
      c_loading = "c:loading", "%, Loading"
      c_maxload = "c:maxload", "%, Max. Loading"
      c_maxloadpu = "c:maxloadpu", "p.u., Max. Loading"
      c_nntap = "c:nntap", "Tap 1, Current Position"
      c_ntpmn = "c:ntpmn", "Tap 1, Min. Position"
      c_ntpmx = "c:ntpmx", "Tap 1, Max. Position"
      c_dut = "c:dut", "Tap 1: Voltage/Tap"
      c_phitrrd = "c:phitrrd", "rad, Tap 1:, Phase/Tap"
      c_nt2agrd = "c:nt2agrd", "rad, Phase Angle"
      c_ratfac = "c:ratfac", "Rating Factor"
      c_snom = "c:snom", "MVA, Rated Power"
      c_zshv_r = "c:zshv:r", "p.u., Leakage Impedance (HV-Side), Real Part"
      c_zshv_i = "c:zshv:i", "p.u., Leakage Impedance (HV-Side), Imaginary Part"
      c_zshv_phi = "c:zshv:phi", "deg, Leakage Impedance (HV-Side), Angle"
      c_zshv_mag = "c:zshv:mag", "p.u., Leakage Impedance (HV-Side), Magnitude"
      c_zshv_phirad = "c:zshv:phirad", "rad, Leakage Impedance (HV-Side), Angle"
      c_zslv_r = "c:zslv:r", "p.u., Leakage Impedance (LV-Side), Real Part"
      c_zslv_i = "c:zslv:i", "p.u., Leakage Impedance (LV-Side), Imaginary Part"
      c_zslv_phi = "c:zslv:phi", "deg, Leakage Impedance (LV-Side), Angle"
      c_zslv_mag = "c:zslv:mag", "p.u., Leakage Impedance (LV-Side), Magnitude"
      c_zslv_phirad = "c:zslv:phirad", "rad, Leakage Impedance (LV-Side), Angle"
      c_ym_r = "c:ym:r", "p.u., Magnetising Admittance, Real Part"
      c_ym_i = "c:ym:i", "p.u., Magnetising Admittance, Imaginary Part"
      c_ym_phi = "c:ym:phi", "deg, Magnetising Admittance, Angle"
      c_ym_mag = "c:ym:mag", "p.u., Magnetising Admittance, Magnitude"
      c_ym_phirad = "c:ym:phirad", "rad, Magnetising Admittance, Angle"
      c_thv_r = "c:thv:r", "Complex Winding Ratio HV, Real Part"
      c_thv_i = "c:thv:i", "Complex Winding Ratio HV, Imaginary Part"
      c_thv_phi = "c:thv:phi", "Complex Winding Ratio HV, Angle"
      c_thv_mag = "c:thv:mag", "Complex Winding Ratio HV, Magnitude"
      c_thv_phirad = "c:thv:phirad", "Complex Winding Ratio HV, Angle"
      c_tlv_r = "c:tlv:r", "Complex Winding Ratio LV, Real Part"
      c_tlv_i = "c:tlv:i", "Complex Winding Ratio LV, Imaginary Part"
      c_tlv_phi = "c:tlv:phi", "Complex Winding Ratio LV, Angle"
      c_tlv_mag = "c:tlv:mag", "Complex Winding Ratio LV, Magnitude"
      c_tlv_phirad = "c:tlv:phirad", "Complex Winding Ratio LV, Angle"
      c_tap_side = "c:tap_side", "0"
      c_tap_side2 = "c:tap_side2", "0"
      c_nntap2 = "c:nntap2", "Tap 2, Current Position"
      c_ntpmn2 = "c:ntpmn2", "Tap 2, Min. Position"
      c_ntpmx2 = "c:ntpmx2", "Tap 2, Max. Position"
      c_dut2 = "c:dut2", "Tap 2: Voltage/Tap"
      c_phitrrd2 = "c:phitrrd2", "rad, Tap 2:, Phase/Tap"
      c_Inom_HV = "c:Inom_HV", "kA, Nominal Current (HV-Side)"
      c_Inom_LV = "c:Inom_LV", "kA, Nominal Current (LV-Side)"
      c_du = "c:du", "p.u., Voltage Drop"
      c_dupc = "c:dupc", "%, Voltage Drop"
      c_dphiu = "c:dphiu", "deg, Voltage Drop Angle"
      c_du1 = "c:du1", "p.u., Positive Sequence Voltage Drop"
      c_du1pc = "c:du1pc", "%, Positive Sequence Voltage Drop"
      c_dphiu1 = "c:dphiu1", "deg, Positive Sequence Voltage Drop Angle"
      c_Pmeas = "c:Pmeas", "MW, Measured active power"
      c_du_ldc = "c:du_ldc", "p.u., LDC Input"
      c_ctrl_set = "c:ctrl_set", "Setpoint"
      c_i_uplim = "c:i_uplim", "Tap 1 at Maximum"
      c_i_lowlim = "c:i_lowlim", "Tap 1 at Minimum"
      c_i_uplim2 = "c:i_uplim2", "Tap 2 at Maximum"
      c_i_lowlim2 = "c:i_lowlim2", "Tap 2 at Minimum"
      c_nCtrlIter = "c:nCtrlIter", "No. of CTRL-iterations"
      c_Tctrl = "c:Tctrl", "s, Controller Time Constant"
      c_i_fixedtap = "c:i_fixedtap", "Flag"
      c_w_0 = "c:w_0", "Nominal winding ratio"
      c_Gmload = "c:Gmload", "kW, Iron Losses"
      c_Xmload = "c:Xmload", "kvar, Magnetising Losses"
      c_Ploss = "c:Ploss", "MW, Losses (total)"
      c_Qloss = "c:Qloss", "Mvar, Reactive-Losses (total)"
      c_Plossld = "c:Plossld", "MW, Losses (load)"
      c_Qlossld = "c:Qlossld", "Mvar, Reactive-Losses (load)"
      c_Plossnld = "c:Plossnld", "MW, Losses (no load)"
      c_Qlossnld = "c:Qlossnld", "Mvar, Reactive-Losses (no load)"
      c_isat = "c:isat", "Saturated Model"

    class ElmTr3(Enum):
      _init_ = 'value __doc__'

      m_dPdQ_bushv = "m:dPdQ:bushv", "MW/Mvar, Branch Sensitivity dPbranch/dQbus"
      m_dPdQ_busmv = "m:dPdQ:busmv", "MW/Mvar, Branch Sensitivity dPbranch/dQbus"
      m_dPdQ_buslv = "m:dPdQ:buslv", "MW/Mvar, Branch Sensitivity dPbranch/dQbus"
      m_dQdQ_bushv = "m:dQdQ:bushv", "Mvar/Mvar, Branch Sensitivity dQbranch/dQbus"
      m_dQdQ_busmv = "m:dQdQ:busmv", "Mvar/Mvar, Branch Sensitivity dQbranch/dQbus"
      m_dQdQ_buslv = "m:dQdQ:buslv", "Mvar/Mvar, Branch Sensitivity dQbranch/dQbus"
      m_dPdP_bushv = "m:dPdP:bushv", "MW/MW, Branch Sensitivity dPbranch/dPbus"
      m_dPdP_busmv = "m:dPdP:busmv", "MW/MW, Branch Sensitivity dPbranch/dPbus"
      m_dPdP_buslv = "m:dPdP:buslv", "MW/MW, Branch Sensitivity dPbranch/dPbus"
      m_dQdP_bushv = "m:dQdP:bushv", "Mvar/MW, Branch Sensitivity dQbranch/dPbus"
      m_dQdP_busmv = "m:dQdP:busmv", "Mvar/MW, Branch Sensitivity dQbranch/dPbus"
      m_dQdP_buslv = "m:dQdP:buslv", "Mvar/MW, Branch Sensitivity dQbranch/dPbus"
      m_dPdtap_bushv = "m:dPdtap:bushv", "MW/tap, Branch Sensitivity dPbranch/dtap"
      m_dPdtap_busmv = "m:dPdtap:busmv", "MW/tap, Branch Sensitivity dPbranch/dtap"
      m_dPdtap_buslv = "m:dPdtap:buslv", "MW/tap, Branch Sensitivity dPbranch/dtap"
      m_dQdtap_bushv = "m:dQdtap:bushv", "Mvar/tap, Branch Sensitivity dQbranch/dtap"
      m_dQdtap_busmv = "m:dQdtap:busmv", "Mvar/tap, Branch Sensitivity dQbranch/dtap"
      m_dQdtap_buslv = "m:dQdtap:buslv", "Mvar/tap, Branch Sensitivity dQbranch/dtap"
      m_LODF_bushv = "m:LODF:bushv", "%, Line Outage Distribution Factor LODF"
      m_LODF_busmv = "m:LODF:busmv", "%, Line Outage Distribution Factor LODF"
      m_LODF_buslv = "m:LODF:buslv", "%, Line Outage Distribution Factor LODF"
      m_u1r_bushv = "m:u1r:bushv", "p.u., Positive-Sequence-Voltage, Real Part"
      m_u1r_busmv = "m:u1r:busmv", "p.u., Positive-Sequence-Voltage, Real Part"
      m_u1r_buslv = "m:u1r:buslv", "p.u., Positive-Sequence-Voltage, Real Part"
      m_u1i_bushv = "m:u1i:bushv", "p.u., Positive-Sequence-Voltage, Imaginary Part"
      m_u1i_busmv = "m:u1i:busmv", "p.u., Positive-Sequence-Voltage, Imaginary Part"
      m_u1i_buslv = "m:u1i:buslv", "p.u., Positive-Sequence-Voltage, Imaginary Part"
      m_u1_bushv = "m:u1:bushv", "p.u., Positive-Sequence-Voltage, Magnitude"
      m_u1_busmv = "m:u1:busmv", "p.u., Positive-Sequence-Voltage, Magnitude"
      m_u1_buslv = "m:u1:buslv", "p.u., Positive-Sequence-Voltage, Magnitude"
      m_U1_bushv = "m:U1:bushv", "kV, Line-Ground Positive-Sequence-Voltage, Magnitude"
      m_U1_busmv = "m:U1:busmv", "kV, Line-Ground Positive-Sequence-Voltage, Magnitude"
      m_U1_buslv = "m:U1:buslv", "kV, Line-Ground Positive-Sequence-Voltage, Magnitude"
      m_U1l_bushv = "m:U1l:bushv", "kV, Line-Line Positive-Sequence-Voltage, Magnitude"
      m_U1l_busmv = "m:U1l:busmv", "kV, Line-Line Positive-Sequence-Voltage, Magnitude"
      m_U1l_buslv = "m:U1l:buslv", "kV, Line-Line Positive-Sequence-Voltage, Magnitude"
      m_phiu1_bushv = "m:phiu1:bushv", "deg, Positive-Sequence-Voltage, Angle"
      m_phiu1_busmv = "m:phiu1:busmv", "deg, Positive-Sequence-Voltage, Angle"
      m_phiu1_buslv = "m:phiu1:buslv", "deg, Positive-Sequence-Voltage, Angle"
      m_inet_bushv = "m:inet:bushv", "p.u., Current, Magnitude, referred to network"
      m_inet_busmv = "m:inet:busmv", "p.u., Current, Magnitude, referred to network"
      m_inet_buslv = "m:inet:buslv", "p.u., Current, Magnitude, referred to network"
      m_I_bushv = "m:I:bushv", "kA, Current, Magnitude"
      m_I_busmv = "m:I:busmv", "kA, Current, Magnitude"
      m_I_buslv = "m:I:buslv", "kA, Current, Magnitude"
      m_phii_bushv = "m:phii:bushv", "deg, Current, Angle"
      m_phii_busmv = "m:phii:busmv", "deg, Current, Angle"
      m_phii_buslv = "m:phii:buslv", "deg, Current, Angle"
      m_i1r_bushv = "m:i1r:bushv", "p.u., Positive-Sequence Current, Real Part"
      m_i1r_busmv = "m:i1r:busmv", "p.u., Positive-Sequence Current, Real Part"
      m_i1r_buslv = "m:i1r:buslv", "p.u., Positive-Sequence Current, Real Part"
      m_i1i_bushv = "m:i1i:bushv", "p.u., Positive-Sequence Current, Imaginary Part"
      m_i1i_busmv = "m:i1i:busmv", "p.u., Positive-Sequence Current, Imaginary Part"
      m_i1i_buslv = "m:i1i:buslv", "p.u., Positive-Sequence Current, Imaginary Part"
      m_i1_bushv = "m:i1:bushv", "p.u., Positive-Sequence Current, Magnitude"
      m_i1_busmv = "m:i1:busmv", "p.u., Positive-Sequence Current, Magnitude"
      m_i1_buslv = "m:i1:buslv", "p.u., Positive-Sequence Current, Magnitude"
      m_I1_bushv = "m:I1:bushv", "kA, Positive-Sequence Current, Magnitude"
      m_I1_busmv = "m:I1:busmv", "kA, Positive-Sequence Current, Magnitude"
      m_I1_buslv = "m:I1:buslv", "kA, Positive-Sequence Current, Magnitude"
      m_phii1_bushv = "m:phii1:bushv", "deg, Positive-Sequence Current, Angle"
      m_phii1_busmv = "m:phii1:busmv", "deg, Positive-Sequence Current, Angle"
      m_phii1_buslv = "m:phii1:buslv", "deg, Positive-Sequence Current, Angle"
      m_P_bushv = "m:P:bushv", "MW, Active Power"
      m_P_busmv = "m:P:busmv", "MW, Active Power"
      m_P_buslv = "m:P:buslv", "MW, Active Power"
      m_Q_bushv = "m:Q:bushv", "Mvar, Reactive Power"
      m_Q_busmv = "m:Q:busmv", "Mvar, Reactive Power"
      m_Q_buslv = "m:Q:buslv", "Mvar, Reactive Power"
      m_S_bushv = "m:S:bushv", "MVA, Apparent Power"
      m_S_busmv = "m:S:busmv", "MVA, Apparent Power"
      m_S_buslv = "m:S:buslv", "MVA, Apparent Power"
      m_cosphi_bushv = "m:cosphi:bushv", "Power Factor"
      m_cosphi_busmv = "m:cosphi:busmv", "Power Factor"
      m_cosphi_buslv = "m:cosphi:buslv", "Power Factor"
      m_tanphi_bushv = "m:tanphi:bushv", "tan(phi)"
      m_tanphi_busmv = "m:tanphi:busmv", "tan(phi)"
      m_tanphi_buslv = "m:tanphi:buslv", "tan(phi)"
      m_Psum_bushv = "m:Psum:bushv", "MW, Total Active Power"
      m_Psum_busmv = "m:Psum:busmv", "MW, Total Active Power"
      m_Psum_buslv = "m:Psum:buslv", "MW, Total Active Power"
      m_Qsum_bushv = "m:Qsum:bushv", "Mvar, Total Reactive Power"
      m_Qsum_busmv = "m:Qsum:busmv", "Mvar, Total Reactive Power"
      m_Qsum_buslv = "m:Qsum:buslv", "Mvar, Total Reactive Power"
      m_Ssum_bushv = "m:Ssum:bushv", "MVA, Total Apparent Power"
      m_Ssum_busmv = "m:Ssum:busmv", "MVA, Total Apparent Power"
      m_Ssum_buslv = "m:Ssum:buslv", "MVA, Total Apparent Power"
      m_cosphisum_bushv = "m:cosphisum:bushv", "Total Power Factor"
      m_cosphisum_busmv = "m:cosphisum:busmv", "Total Power Factor"
      m_cosphisum_buslv = "m:cosphisum:buslv", "Total Power Factor"
      m_tanphisum_bushv = "m:tanphisum:bushv", "Total tan(phi)"
      m_tanphisum_busmv = "m:tanphisum:busmv", "Total tan(phi)"
      m_tanphisum_buslv = "m:tanphisum:buslv", "Total tan(phi)"
      m_Spu_bushv = "m:Spu:bushv", "MVA/p.u., Apparent Power per p.u. Voltage"
      m_Spu_busmv = "m:Spu:busmv", "MVA/p.u., Apparent Power per p.u. Voltage"
      m_Spu_buslv = "m:Spu:buslv", "MVA/p.u., Apparent Power per p.u. Voltage"
      m_i1P_bushv = "m:i1P:bushv", "p.u., Positive-Sequence Active Current"
      m_i1P_busmv = "m:i1P:busmv", "p.u., Positive-Sequence Active Current"
      m_i1P_buslv = "m:i1P:buslv", "p.u., Positive-Sequence Active Current"
      m_i1Q_bushv = "m:i1Q:bushv", "p.u., Positive-Sequence Reactive Current"
      m_i1Q_busmv = "m:i1Q:busmv", "p.u., Positive-Sequence Reactive Current"
      m_i1Q_buslv = "m:i1Q:buslv", "p.u., Positive-Sequence Reactive Current"
      m_I1P_bushv = "m:I1P:bushv", "kA, Positive-Sequence Active Current"
      m_I1P_busmv = "m:I1P:busmv", "kA, Positive-Sequence Active Current"
      m_I1P_buslv = "m:I1P:buslv", "kA, Positive-Sequence Active Current"
      m_I1Q_bushv = "m:I1Q:bushv", "kA, Positive-Sequence Reactive Current"
      m_I1Q_busmv = "m:I1Q:busmv", "kA, Positive-Sequence Reactive Current"
      m_I1Q_buslv = "m:I1Q:buslv", "kA, Positive-Sequence Reactive Current"
      m_ur_bushv = "m:ur:bushv", "p.u., Voltage, Real Part"
      m_ur_busmv = "m:ur:busmv", "p.u., Voltage, Real Part"
      m_ur_buslv = "m:ur:buslv", "p.u., Voltage, Real Part"
      m_ui_bushv = "m:ui:bushv", "p.u., Voltage, Imaginary Part"
      m_ui_busmv = "m:ui:busmv", "p.u., Voltage, Imaginary Part"
      m_ui_buslv = "m:ui:buslv", "p.u., Voltage, Imaginary Part"
      m_u_bushv = "m:u:bushv", "p.u., Voltage, Magnitude"
      m_u_busmv = "m:u:busmv", "p.u., Voltage, Magnitude"
      m_u_buslv = "m:u:buslv", "p.u., Voltage, Magnitude"
      m_ir_bushv = "m:ir:bushv", "p.u., Current, Real Part"
      m_ir_busmv = "m:ir:busmv", "p.u., Current, Real Part"
      m_ir_buslv = "m:ir:buslv", "p.u., Current, Real Part"
      m_ii_bushv = "m:ii:bushv", "p.u., Current, Imaginary Part"
      m_ii_busmv = "m:ii:busmv", "p.u., Current, Imaginary Part"
      m_ii_buslv = "m:ii:buslv", "p.u., Current, Imaginary Part"
      m_i_bushv = "m:i:bushv", "p.u., Current, Magnitude"
      m_i_busmv = "m:i:busmv", "p.u., Current, Magnitude"
      m_i_buslv = "m:i:buslv", "p.u., Current, Magnitude"
      m_phiui_bushv = "m:phiui:bushv", "deg, Angle between Voltage and Current"
      m_phiui_busmv = "m:phiui:busmv", "deg, Angle between Voltage and Current"
      m_phiui_buslv = "m:phiui:buslv", "deg, Angle between Voltage and Current"
      m_phiu1i1_bushv = "m:phiu1i1:bushv", "deg, Angle between Voltage and Current in positive sequence system"
      m_phiu1i1_busmv = "m:phiu1i1:busmv", "deg, Angle between Voltage and Current in positive sequence system"
      m_phiu1i1_buslv = "m:phiu1i1:buslv", "deg, Angle between Voltage and Current in positive sequence system"
      m_Tfct_bushv = "m:Tfct:bushv", "s, Fault Clearing Time"
      m_Tfct_busmv = "m:Tfct:busmv", "s, Fault Clearing Time"
      m_Tfct_buslv = "m:Tfct:buslv", "s, Fault Clearing Time"
      m_Brkload_bushv = "m:Brkload:bushv", "%, Breaker Loading"
      m_Brkload_busmv = "m:Brkload:busmv", "%, Breaker Loading"
      m_Brkload_buslv = "m:Brkload:buslv", "%, Breaker Loading"
      n_dphidP_bushv = "n:dphidP:bushv", "deg/MW, dphi/dP Sensitivity"
      n_dphidP_busmv = "n:dphidP:busmv", "deg/MW, dphi/dP Sensitivity"
      n_dphidP_buslv = "n:dphidP:buslv", "deg/MW, dphi/dP Sensitivity"
      n_dphidQ_bushv = "n:dphidQ:bushv", "deg/Mvar, dphi/dQ Sensitivity"
      n_dphidQ_busmv = "n:dphidQ:busmv", "deg/Mvar, dphi/dQ Sensitivity"
      n_dphidQ_buslv = "n:dphidQ:buslv", "deg/Mvar, dphi/dQ Sensitivity"
      n_dphidtap_bushv = "n:dphidtap:bushv", "deg/tap, dphi/dtap Sensitivity"
      n_dphidtap_busmv = "n:dphidtap:busmv", "deg/tap, dphi/dtap Sensitivity"
      n_dphidtap_buslv = "n:dphidtap:buslv", "deg/tap, dphi/dtap Sensitivity"
      n_dvdP_bushv = "n:dvdP:bushv", "p.u./MW, dv/dP Sensitivity"
      n_dvdP_busmv = "n:dvdP:busmv", "p.u./MW, dv/dP Sensitivity"
      n_dvdP_buslv = "n:dvdP:buslv", "p.u./MW, dv/dP Sensitivity"
      n_dvdQ_bushv = "n:dvdQ:bushv", "p.u./Mvar, dv/dQ Sensitivity"
      n_dvdQ_busmv = "n:dvdQ:busmv", "p.u./Mvar, dv/dQ Sensitivity"
      n_dvdQ_buslv = "n:dvdQ:buslv", "p.u./Mvar, dv/dQ Sensitivity"
      n_dvdtap_bushv = "n:dvdtap:bushv", "p.u./tap, dv/dtap Sensitivity"
      n_dvdtap_busmv = "n:dvdtap:busmv", "p.u./tap, dv/dtap Sensitivity"
      n_dvdtap_buslv = "n:dvdtap:buslv", "p.u./tap, dv/dtap Sensitivity"
      n_levecQ_bushv = "n:levecQ:bushv", "Left Eigenvector of selected Q-Mode"
      n_levecQ_busmv = "n:levecQ:busmv", "Left Eigenvector of selected Q-Mode"
      n_levecQ_buslv = "n:levecQ:buslv", "Left Eigenvector of selected Q-Mode"
      n_revecQ_bushv = "n:revecQ:bushv", "Right Eigenvector of selected Q-Mode"
      n_revecQ_busmv = "n:revecQ:busmv", "Right Eigenvector of selected Q-Mode"
      n_revecQ_buslv = "n:revecQ:buslv", "Right Eigenvector of selected Q-Mode"
      n_partQ_bushv = "n:partQ:bushv", "Participation Factor of selected Q-Mode"
      n_partQ_busmv = "n:partQ:busmv", "Participation Factor of selected Q-Mode"
      n_partQ_buslv = "n:partQ:buslv", "Participation Factor of selected Q-Mode"
      n_ur_bushv = "n:ur:bushv", "p.u., Voltage, Real Part"
      n_ur_busmv = "n:ur:busmv", "p.u., Voltage, Real Part"
      n_ur_buslv = "n:ur:buslv", "p.u., Voltage, Real Part"
      n_ui_bushv = "n:ui:bushv", "p.u., Voltage, Imaginary Part"
      n_ui_busmv = "n:ui:busmv", "p.u., Voltage, Imaginary Part"
      n_ui_buslv = "n:ui:buslv", "p.u., Voltage, Imaginary Part"
      n_u_bushv = "n:u:bushv", "p.u., Voltage, Magnitude"
      n_u_busmv = "n:u:busmv", "p.u., Voltage, Magnitude"
      n_u_buslv = "n:u:buslv", "p.u., Voltage, Magnitude"
      n_upc_bushv = "n:upc:bushv", "%, Voltage, Magnitude"
      n_upc_busmv = "n:upc:busmv", "%, Voltage, Magnitude"
      n_upc_buslv = "n:upc:buslv", "%, Voltage, Magnitude"
      n_u1_bushv = "n:u1:bushv", "p.u., Voltage, Magnitude"
      n_u1_busmv = "n:u1:busmv", "p.u., Voltage, Magnitude"
      n_u1_buslv = "n:u1:buslv", "p.u., Voltage, Magnitude"
      n_u1pc_bushv = "n:u1pc:bushv", "%, Voltage, Magnitude"
      n_u1pc_busmv = "n:u1pc:busmv", "%, Voltage, Magnitude"
      n_u1pc_buslv = "n:u1pc:buslv", "%, Voltage, Magnitude"
      n_u1r_bushv = "n:u1r:bushv", "p.u., Positive-Sequence Voltage, Real Part"
      n_u1r_busmv = "n:u1r:busmv", "p.u., Positive-Sequence Voltage, Real Part"
      n_u1r_buslv = "n:u1r:buslv", "p.u., Positive-Sequence Voltage, Real Part"
      n_u1i_bushv = "n:u1i:bushv", "p.u., Positive-Sequence Voltage, Imaginary Part"
      n_u1i_busmv = "n:u1i:busmv", "p.u., Positive-Sequence Voltage, Imaginary Part"
      n_u1i_buslv = "n:u1i:buslv", "p.u., Positive-Sequence Voltage, Imaginary Part"
      n_U_bushv = "n:U:bushv", "kV, Line-Ground Voltage, Magnitude"
      n_U_busmv = "n:U:busmv", "kV, Line-Ground Voltage, Magnitude"
      n_U_buslv = "n:U:buslv", "kV, Line-Ground Voltage, Magnitude"
      n_Ul_bushv = "n:Ul:bushv", "kV, Line-Line Voltage, Magnitude"
      n_Ul_busmv = "n:Ul:busmv", "kV, Line-Line Voltage, Magnitude"
      n_Ul_buslv = "n:Ul:buslv", "kV, Line-Line Voltage, Magnitude"
      n_phiu_bushv = "n:phiu:bushv", "deg, Voltage, Angle"
      n_phiu_busmv = "n:phiu:busmv", "deg, Voltage, Angle"
      n_phiu_buslv = "n:phiu:buslv", "deg, Voltage, Angle"
      n_phiurel_bushv = "n:phiurel:bushv", "deg, Voltage, Relative Angle"
      n_phiurel_busmv = "n:phiurel:busmv", "deg, Voltage, Relative Angle"
      n_phiurel_buslv = "n:phiurel:buslv", "deg, Voltage, Relative Angle"
      n_du_bushv = "n:du:bushv", "%, Voltage Deviation"
      n_du_busmv = "n:du:busmv", "%, Voltage Deviation"
      n_du_buslv = "n:du:buslv", "%, Voltage Deviation"
      n_Pgen_bushv = "n:Pgen:bushv", "MW, Generation, Active Power"
      n_Pgen_busmv = "n:Pgen:busmv", "MW, Generation, Active Power"
      n_Pgen_buslv = "n:Pgen:buslv", "MW, Generation, Active Power"
      n_Qgen_bushv = "n:Qgen:bushv", "Mvar, Generation, Reactive Power"
      n_Qgen_busmv = "n:Qgen:busmv", "Mvar, Generation, Reactive Power"
      n_Qgen_buslv = "n:Qgen:buslv", "Mvar, Generation, Reactive Power"
      n_Pmot_bushv = "n:Pmot:bushv", "MW, Motor Load, Active Power"
      n_Pmot_busmv = "n:Pmot:busmv", "MW, Motor Load, Active Power"
      n_Pmot_buslv = "n:Pmot:buslv", "MW, Motor Load, Active Power"
      n_Qmot_bushv = "n:Qmot:bushv", "Mvar, Motor Load, Reactive Power"
      n_Qmot_busmv = "n:Qmot:busmv", "Mvar, Motor Load, Reactive Power"
      n_Qmot_buslv = "n:Qmot:buslv", "Mvar, Motor Load, Reactive Power"
      n_Pload_bushv = "n:Pload:bushv", "MW, General Load, Active Power"
      n_Pload_busmv = "n:Pload:busmv", "MW, General Load, Active Power"
      n_Pload_buslv = "n:Pload:buslv", "MW, General Load, Active Power"
      n_Qload_bushv = "n:Qload:bushv", "Mvar, General Load, Reactive Power"
      n_Qload_busmv = "n:Qload:busmv", "Mvar, General Load, Reactive Power"
      n_Qload_buslv = "n:Qload:buslv", "Mvar, General Load, Reactive Power"
      n_Pcomp_bushv = "n:Pcomp:bushv", "MW, Compensation (Losses)"
      n_Pcomp_busmv = "n:Pcomp:busmv", "MW, Compensation (Losses)"
      n_Pcomp_buslv = "n:Pcomp:buslv", "MW, Compensation (Losses)"
      n_Qcomp_bushv = "n:Qcomp:bushv", "Mvar, Compensation"
      n_Qcomp_busmv = "n:Qcomp:busmv", "Mvar, Compensation"
      n_Qcomp_buslv = "n:Qcomp:buslv", "Mvar, Compensation"
      n_Pnet_bushv = "n:Pnet:bushv", "MW, External Networks, Active Power"
      n_Pnet_busmv = "n:Pnet:busmv", "MW, External Networks, Active Power"
      n_Pnet_buslv = "n:Pnet:buslv", "MW, External Networks, Active Power"
      n_Qnet_bushv = "n:Qnet:bushv", "Mvar, External Networks, Reactive Power"
      n_Qnet_busmv = "n:Qnet:busmv", "Mvar, External Networks, Reactive Power"
      n_Qnet_buslv = "n:Qnet:buslv", "Mvar, External Networks, Reactive Power"
      n_Pflow_bushv = "n:Pflow:bushv", "MW, Power Flow, Active Power"
      n_Pflow_busmv = "n:Pflow:busmv", "MW, Power Flow, Active Power"
      n_Pflow_buslv = "n:Pflow:buslv", "MW, Power Flow, Active Power"
      n_Qflow_bushv = "n:Qflow:bushv", "Mvar, Power Flow, Reactive Power"
      n_Qflow_busmv = "n:Qflow:busmv", "Mvar, Power Flow, Reactive Power"
      n_Qflow_buslv = "n:Qflow:buslv", "Mvar, Power Flow, Reactive Power"
      n_Pout_bushv = "n:Pout:bushv", "MW, Outgoing Flow, Active Power"
      n_Pout_busmv = "n:Pout:busmv", "MW, Outgoing Flow, Active Power"
      n_Pout_buslv = "n:Pout:buslv", "MW, Outgoing Flow, Active Power"
      n_Qout_bushv = "n:Qout:bushv", "Mvar, Outgoing Power, Reactive Power"
      n_Qout_busmv = "n:Qout:busmv", "Mvar, Outgoing Power, Reactive Power"
      n_Qout_buslv = "n:Qout:buslv", "Mvar, Outgoing Power, Reactive Power"
      n_Sout_bushv = "n:Sout:bushv", "MVA, Outgoing Power, Apparent Power"
      n_Sout_busmv = "n:Sout:busmv", "MVA, Outgoing Power, Apparent Power"
      n_Sout_buslv = "n:Sout:buslv", "MVA, Outgoing Power, Apparent Power"
      n_cosphiout_bushv = "n:cosphiout:bushv", "Outgoing Power, Power Factor"
      n_cosphiout_busmv = "n:cosphiout:busmv", "Outgoing Power, Power Factor"
      n_cosphiout_buslv = "n:cosphiout:buslv", "Outgoing Power, Power Factor"
      n_Pbalance_bushv = "n:Pbalance:bushv", "MW, Active Power Balance (=0)"
      n_Pbalance_busmv = "n:Pbalance:busmv", "MW, Active Power Balance (=0)"
      n_Pbalance_buslv = "n:Pbalance:buslv", "MW, Active Power Balance (=0)"
      n_Qbalance_bushv = "n:Qbalance:bushv", "Mvar, Reactive Power Balance (=0)"
      n_Qbalance_busmv = "n:Qbalance:busmv", "Mvar, Reactive Power Balance (=0)"
      n_Qbalance_buslv = "n:Qbalance:buslv", "Mvar, Reactive Power Balance (=0)"
      n_LossPdown_bushv = "n:LossPdown:bushv", "MW, Losses, downstream"
      n_LossPdown_busmv = "n:LossPdown:busmv", "MW, Losses, downstream"
      n_LossPdown_buslv = "n:LossPdown:buslv", "MW, Losses, downstream"
      n_LossQdown_bushv = "n:LossQdown:bushv", "Mvar, Losses, downstream (Reactive Power)"
      n_LossQdown_busmv = "n:LossQdown:busmv", "Mvar, Losses, downstream (Reactive Power)"
      n_LossQdown_buslv = "n:LossQdown:buslv", "Mvar, Losses, downstream (Reactive Power)"
      n_LossPdownload_bushv = "n:LossPdownload:bushv", "MW, Load losses, downstream"
      n_LossPdownload_busmv = "n:LossPdownload:busmv", "MW, Load losses, downstream"
      n_LossPdownload_buslv = "n:LossPdownload:buslv", "MW, Load losses, downstream"
      n_LossQdownload_bushv = "n:LossQdownload:bushv", "Mvar, Load losses, downstream"
      n_LossQdownload_busmv = "n:LossQdownload:busmv", "Mvar, Load losses, downstream"
      n_LossQdownload_buslv = "n:LossQdownload:buslv", "Mvar, Load losses, downstream"
      n_LossPdownnoload_bushv = "n:LossPdownnoload:bushv", "MW, No load losses, downstream"
      n_LossPdownnoload_busmv = "n:LossPdownnoload:busmv", "MW, No load losses, downstream"
      n_LossPdownnoload_buslv = "n:LossPdownnoload:buslv", "MW, No load losses, downstream"
      n_LossQdownnoload_bushv = "n:LossQdownnoload:bushv", "Mvar, No load losses, downstream"
      n_LossQdownnoload_busmv = "n:LossQdownnoload:busmv", "Mvar, No load losses, downstream"
      n_LossQdownnoload_buslv = "n:LossQdownnoload:buslv", "Mvar, No load losses, downstream"
      n_du_feed_bushv = "n:du_feed:bushv", "%, Voltage difference relative to feeder begin"
      n_du_feed_busmv = "n:du_feed:busmv", "%, Voltage difference relative to feeder begin"
      n_du_feed_buslv = "n:du_feed:buslv", "%, Voltage difference relative to feeder begin"
      s_nt_in_h = "s:nt_in_h", "IN, Tap Position (HV), Input, continuous, double"
      s_nt_in_m = "s:nt_in_m", "IN, Tap Position (MV), Input, continuous, double"
      s_nt_in_l = "s:nt_in_l", "IN, Tap Position (LV), Input, continuous, double"
      s_du_ldc = "s:du_ldc", "IN, LDC Input, continuous, double"
      c_dLoadingdP = "c:dLoadingdP", "%/MW, Branch Loading Sensitivity: dLbranch/dPbus"
      c_dLoadingdtap = "c:dLoadingdtap", "%/tap, Branch Loading Sensitivity: dLbranch/dtap"
      c_dPlossdP = "c:dPlossdP", "kW/MW, Active Power Losses Sensitivity: dPloss/dP"
      c_dPlossdQ = "c:dPlossdQ", "kW/Mvar, Active Power Losses Sensitivity: dPloss/dQ"
      c_dQlossdP = "c:dQlossdP", "kvar/MW, Reactive Power Losses Sensitivity: dQloss/dP"
      c_dQlossdQ = "c:dQlossdQ", "kvar/Mvar, Reactive Power Losses Sensitivity: dQloss/dQ"
      c_dPlossdtap = "c:dPlossdtap", "kW/tap, Active Power Losses Sensitivity: dPloss/dtap"
      c_dQlossdtap = "c:dQlossdtap", "kvar/tap, Reactive Power Losses Sensitivity: dQloss/dtap"
      c_loading_h = "c:loading_h", "%, Loading (HV)"
      c_loading_m = "c:loading_m", "%, Loading (MV)"
      c_loading_l = "c:loading_l", "%, Loading (LV)"
      c_loading = "c:loading", "%, Maximum Loading"
      c_maxload = "c:maxload", "%, Max. Loading"
      c_maxloadpu = "c:maxloadpu", "p.u., Max. Loading"
      c_n3tap_h = "c:n3tap_h", "Current Tap Position (HV)"
      c_n3tap_m = "c:n3tap_m", "Current Tap Position (MV)"
      c_n3tap_l = "c:n3tap_l", "Current Tap Position (LV)"
      c_nntap_int_h = "c:nntap_int_h", "Tap (HV), relative Position"
      c_nntap_int_m = "c:nntap_int_m", "Tap (MV), relative Position"
      c_nntap_int_l = "c:nntap_int_l", "Tap (LV), relative Position"
      c_zs_h_r = "c:zs_h:r", "p.u., Short Circuit Impedance (HV), Real Part"
      c_zs_h_i = "c:zs_h:i", "p.u., Short Circuit Impedance (HV), Imaginary Part"
      c_zs_h_phi = "c:zs_h:phi", "deg, Short Circuit Impedance (HV), Angle"
      c_zs_h_mag = "c:zs_h:mag", "p.u., Short Circuit Impedance (HV), Magnitude"
      c_zs_h_phirad = "c:zs_h:phirad", "rad, Short Circuit Impedance (HV), Angle"
      c_zs_m_r = "c:zs_m:r", "p.u., Short Circuit Impedance (MV), Real Part"
      c_zs_m_i = "c:zs_m:i", "p.u., Short Circuit Impedance (MV), Imaginary Part"
      c_zs_m_phi = "c:zs_m:phi", "deg, Short Circuit Impedance (MV), Angle"
      c_zs_m_mag = "c:zs_m:mag", "p.u., Short Circuit Impedance (MV), Magnitude"
      c_zs_m_phirad = "c:zs_m:phirad", "rad, Short Circuit Impedance (MV), Angle"
      c_zs_l_r = "c:zs_l:r", "p.u., Short Circuit Impedance (LV), Real Part"
      c_zs_l_i = "c:zs_l:i", "p.u., Short Circuit Impedance (LV), Imaginary Part"
      c_zs_l_phi = "c:zs_l:phi", "deg, Short Circuit Impedance (LV), Angle"
      c_zs_l_mag = "c:zs_l:mag", "p.u., Short Circuit Impedance (LV), Magnitude"
      c_zs_l_phirad = "c:zs_l:phirad", "rad, Short Circuit Impedance (LV), Angle"
      c_ym_r = "c:ym:r", "p.u., Magnetising Admittance, Real Part"
      c_ym_i = "c:ym:i", "p.u., Magnetising Admittance, Imaginary Part"
      c_ym_phi = "c:ym:phi", "deg, Magnetising Admittance, Angle"
      c_ym_mag = "c:ym:mag", "p.u., Magnetising Admittance, Magnitude"
      c_ym_phirad = "c:ym:phirad", "rad, Magnetising Admittance, Angle"
      c_t_h_r = "c:t_h:r", "Complex Winding Ratio (HV/S), Real Part"
      c_t_h_i = "c:t_h:i", "Complex Winding Ratio (HV/S), Imaginary Part"
      c_t_h_phi = "c:t_h:phi", "Complex Winding Ratio (HV/S), Angle"
      c_t_h_mag = "c:t_h:mag", "Complex Winding Ratio (HV/S), Magnitude"
      c_t_h_phirad = "c:t_h:phirad", "Complex Winding Ratio (HV/S), Angle"
      c_t_m_r = "c:t_m:r", "Complex Winding Ratio (MV/S), Real Part"
      c_t_m_i = "c:t_m:i", "Complex Winding Ratio (MV/S), Imaginary Part"
      c_t_m_phi = "c:t_m:phi", "Complex Winding Ratio (MV/S), Angle"
      c_t_m_mag = "c:t_m:mag", "Complex Winding Ratio (MV/S), Magnitude"
      c_t_m_phirad = "c:t_m:phirad", "Complex Winding Ratio (MV/S), Angle"
      c_t_l_r = "c:t_l:r", "Complex Winding Ratio (LV/S), Real Part"
      c_t_l_i = "c:t_l:i", "Complex Winding Ratio (LV/S), Imaginary Part"
      c_t_l_phi = "c:t_l:phi", "Complex Winding Ratio (LV/S), Angle"
      c_t_l_mag = "c:t_l:mag", "Complex Winding Ratio (LV/S), Magnitude"
      c_t_l_phirad = "c:t_l:phirad", "Complex Winding Ratio (LV/S), Angle"
      c_i_uplim = "c:i_uplim", "Tap at Maximum"
      c_i_lowlim = "c:i_lowlim", "Tap at Minimum"
      c_nCtrlIter = "c:nCtrlIter", "No. of CTRL-iterations"
      c_Pmeas = "c:Pmeas", "MW, Measured active power"
      c_i_fixedtap = "c:i_fixedtap", "Flag"
      c_um_1_r = "c:um_1:r", "Magnetising Flux, Real Part"
      c_um_1_i = "c:um_1:i", "Magnetising Flux, Imaginary Part"
      c_um_1_phi = "c:um_1:phi", "Magnetising Flux, Angle"
      c_um_1_mag = "c:um_1:mag", "Magnetising Flux, Magnitude"
      c_um_1_phirad = "c:um_1:phirad", "Magnetising Flux, Angle"
      c_u_h_r = "c:u_h:r", "Internal, Real Part"
      c_u_h_i = "c:u_h:i", "Internal, Imaginary Part"
      c_u_h_phi = "c:u_h:phi", "Internal, Angle"
      c_u_h_mag = "c:u_h:mag", "Internal, Magnitude"
      c_u_h_phirad = "c:u_h:phirad", "Internal, Angle"
      c_u_m_r = "c:u_m:r", "Internal, Real Part"
      c_u_m_i = "c:u_m:i", "Internal, Imaginary Part"
      c_u_m_phi = "c:u_m:phi", "Internal, Angle"
      c_u_m_mag = "c:u_m:mag", "Internal, Magnitude"
      c_u_m_phirad = "c:u_m:phirad", "Internal, Angle"
      c_u_l_r = "c:u_l:r", "Internal, Real Part"
      c_u_l_i = "c:u_l:i", "Internal, Imaginary Part"
      c_u_l_phi = "c:u_l:phi", "Internal, Angle"
      c_u_l_mag = "c:u_l:mag", "Internal, Magnitude"
      c_u_l_phirad = "c:u_l:phirad", "Internal, Angle"
      c_w_0 = "c:w_0", "Nominal winding ratio"
      c_Gmload = "c:Gmload", "kW, Iron Losses"
      c_Xmload = "c:Xmload", "kvar, Magnetising Losses"
      c_Ploss = "c:Ploss", "MW, Losses (total)"
      c_Qloss = "c:Qloss", "Mvar, Reactive-Losses (total)"
      c_Plossld = "c:Plossld", "MW, Losses (load)"
      c_Qlossld = "c:Qlossld", "Mvar, Reactive-Losses (load)"
      c_Plossnld = "c:Plossnld", "MW, Losses (no load)"
      c_Qlossnld = "c:Qlossnld", "Mvar, Reactive-Losses (no load)"
      c_itrcn = "c:itrcn", "Flag f. auto. Tap"
      c_i3loc = "c:i3loc", "Position of Magn.ind."
      c_isat = "c:isat", "Saturated Model"
      c_dphiu_hm = "c:dphiu_hm", "deg, Voltage Drop Angle (HV-MV)"
      c_dphiu_hl = "c:dphiu_hl", "deg, Voltage Drop Angle (HV-LV)"
      c_dphiu_ml = "c:dphiu_ml", "deg, Voltage Drop Angle (MV-LV)"
      c_Inom_HV = "c:Inom_HV", "kA, Nominal Current (HV-Side)"
      c_Inom_MV = "c:Inom_MV", "kA, Nominal Current (MV-Side)"
      c_Inom_LV = "c:Inom_LV", "kA, Nominal Current (LV-Side)"

    class ElmVac(Enum):
      _init_ = 'value __doc__'

      m_dPdQ_bus1 = "m:dPdQ:bus1", "MW/Mvar, Branch Sensitivity dPbranch/dQbus"
      m_dQdQ_bus1 = "m:dQdQ:bus1", "Mvar/Mvar, Branch Sensitivity dQbranch/dQbus"
      m_dPdP_bus1 = "m:dPdP:bus1", "MW/MW, Branch Sensitivity dPbranch/dPbus"
      m_dQdP_bus1 = "m:dQdP:bus1", "Mvar/MW, Branch Sensitivity dQbranch/dPbus"
      m_dPdtap_bus1 = "m:dPdtap:bus1", "MW/tap, Branch Sensitivity dPbranch/dtap"
      m_dQdtap_bus1 = "m:dQdtap:bus1", "Mvar/tap, Branch Sensitivity dQbranch/dtap"
      m_LODF_bus1 = "m:LODF:bus1", "%, Line Outage Distribution Factor LODF"
      m_u1r_bus1 = "m:u1r:bus1", "p.u., Positive-Sequence-Voltage, Real Part"
      m_u1i_bus1 = "m:u1i:bus1", "p.u., Positive-Sequence-Voltage, Imaginary Part"
      m_u1_bus1 = "m:u1:bus1", "p.u., Positive-Sequence-Voltage, Magnitude"
      m_U1_bus1 = "m:U1:bus1", "kV, Line-Ground Positive-Sequence-Voltage, Magnitude"
      m_U1l_bus1 = "m:U1l:bus1", "kV, Line-Line Positive-Sequence-Voltage, Magnitude"
      m_phiu1_bus1 = "m:phiu1:bus1", "deg, Positive-Sequence-Voltage, Angle"
      m_inet_bus1 = "m:inet:bus1", "p.u., Current, Magnitude, referred to network"
      m_I_bus1 = "m:I:bus1", "kA, Current, Magnitude"
      m_phii_bus1 = "m:phii:bus1", "deg, Current, Angle"
      m_i1r_bus1 = "m:i1r:bus1", "p.u., Positive-Sequence Current, Real Part"
      m_i1i_bus1 = "m:i1i:bus1", "p.u., Positive-Sequence Current, Imaginary Part"
      m_i1_bus1 = "m:i1:bus1", "p.u., Positive-Sequence Current, Magnitude"
      m_I1_bus1 = "m:I1:bus1", "kA, Positive-Sequence Current, Magnitude"
      m_phii1_bus1 = "m:phii1:bus1", "deg, Positive-Sequence Current, Angle"
      m_P_bus1 = "m:P:bus1", "MW, Active Power"
      m_Q_bus1 = "m:Q:bus1", "Mvar, Reactive Power"
      m_S_bus1 = "m:S:bus1", "MVA, Apparent Power"
      m_cosphi_bus1 = "m:cosphi:bus1", "Power Factor"
      m_tanphi_bus1 = "m:tanphi:bus1", "tan(phi)"
      m_Psum_bus1 = "m:Psum:bus1", "MW, Total Active Power"
      m_Qsum_bus1 = "m:Qsum:bus1", "Mvar, Total Reactive Power"
      m_Ssum_bus1 = "m:Ssum:bus1", "MVA, Total Apparent Power"
      m_cosphisum_bus1 = "m:cosphisum:bus1", "Total Power Factor"
      m_tanphisum_bus1 = "m:tanphisum:bus1", "Total tan(phi)"
      m_Spu_bus1 = "m:Spu:bus1", "MVA/p.u., Apparent Power per p.u. Voltage"
      m_i1P_bus1 = "m:i1P:bus1", "p.u., Positive-Sequence Active Current"
      m_i1Q_bus1 = "m:i1Q:bus1", "p.u., Positive-Sequence Reactive Current"
      m_I1P_bus1 = "m:I1P:bus1", "kA, Positive-Sequence Active Current"
      m_I1Q_bus1 = "m:I1Q:bus1", "kA, Positive-Sequence Reactive Current"
      m_ur_bus1 = "m:ur:bus1", "p.u., Voltage, Real Part"
      m_ui_bus1 = "m:ui:bus1", "p.u., Voltage, Imaginary Part"
      m_u_bus1 = "m:u:bus1", "p.u., Voltage, Magnitude"
      m_ir_bus1 = "m:ir:bus1", "p.u., Current, Real Part"
      m_ii_bus1 = "m:ii:bus1", "p.u., Current, Imaginary Part"
      m_i_bus1 = "m:i:bus1", "p.u., Current, Magnitude"
      m_phiui_bus1 = "m:phiui:bus1", "deg, Angle between Voltage and Current"
      m_phiu1i1_bus1 = "m:phiu1i1:bus1", "deg, Angle between Voltage and Current in positive sequence system"
      m_Tfct_bus1 = "m:Tfct:bus1", "s, Fault Clearing Time"
      m_Brkload_bus1 = "m:Brkload:bus1", "%, Breaker Loading"
      n_dphidP_bus1 = "n:dphidP:bus1", "deg/MW, dphi/dP Sensitivity"
      n_dphidQ_bus1 = "n:dphidQ:bus1", "deg/Mvar, dphi/dQ Sensitivity"
      n_dphidtap_bus1 = "n:dphidtap:bus1", "deg/tap, dphi/dtap Sensitivity"
      n_dvdP_bus1 = "n:dvdP:bus1", "p.u./MW, dv/dP Sensitivity"
      n_dvdQ_bus1 = "n:dvdQ:bus1", "p.u./Mvar, dv/dQ Sensitivity"
      n_dvdtap_bus1 = "n:dvdtap:bus1", "p.u./tap, dv/dtap Sensitivity"
      n_levecQ_bus1 = "n:levecQ:bus1", "Left Eigenvector of selected Q-Mode"
      n_revecQ_bus1 = "n:revecQ:bus1", "Right Eigenvector of selected Q-Mode"
      n_partQ_bus1 = "n:partQ:bus1", "Participation Factor of selected Q-Mode"
      n_ur_bus1 = "n:ur:bus1", "p.u., Voltage, Real Part"
      n_ui_bus1 = "n:ui:bus1", "p.u., Voltage, Imaginary Part"
      n_u_bus1 = "n:u:bus1", "p.u., Voltage, Magnitude"
      n_upc_bus1 = "n:upc:bus1", "%, Voltage, Magnitude"
      n_u1_bus1 = "n:u1:bus1", "p.u., Voltage, Magnitude"
      n_u1pc_bus1 = "n:u1pc:bus1", "%, Voltage, Magnitude"
      n_u1r_bus1 = "n:u1r:bus1", "p.u., Positive-Sequence Voltage, Real Part"
      n_u1i_bus1 = "n:u1i:bus1", "p.u., Positive-Sequence Voltage, Imaginary Part"
      n_U_bus1 = "n:U:bus1", "kV, Line-Ground Voltage, Magnitude"
      n_Ul_bus1 = "n:Ul:bus1", "kV, Line-Line Voltage, Magnitude"
      n_phiu_bus1 = "n:phiu:bus1", "deg, Voltage, Angle"
      n_phiurel_bus1 = "n:phiurel:bus1", "deg, Voltage, Relative Angle"
      n_du_bus1 = "n:du:bus1", "%, Voltage Deviation"
      n_Pgen_bus1 = "n:Pgen:bus1", "MW, Generation, Active Power"
      n_Qgen_bus1 = "n:Qgen:bus1", "Mvar, Generation, Reactive Power"
      n_Pmot_bus1 = "n:Pmot:bus1", "MW, Motor Load, Active Power"
      n_Qmot_bus1 = "n:Qmot:bus1", "Mvar, Motor Load, Reactive Power"
      n_Pload_bus1 = "n:Pload:bus1", "MW, General Load, Active Power"
      n_Qload_bus1 = "n:Qload:bus1", "Mvar, General Load, Reactive Power"
      n_Pcomp_bus1 = "n:Pcomp:bus1", "MW, Compensation (Losses)"
      n_Qcomp_bus1 = "n:Qcomp:bus1", "Mvar, Compensation"
      n_Pnet_bus1 = "n:Pnet:bus1", "MW, External Networks, Active Power"
      n_Qnet_bus1 = "n:Qnet:bus1", "Mvar, External Networks, Reactive Power"
      n_Pflow_bus1 = "n:Pflow:bus1", "MW, Power Flow, Active Power"
      n_Qflow_bus1 = "n:Qflow:bus1", "Mvar, Power Flow, Reactive Power"
      n_Pout_bus1 = "n:Pout:bus1", "MW, Outgoing Flow, Active Power"
      n_Qout_bus1 = "n:Qout:bus1", "Mvar, Outgoing Power, Reactive Power"
      n_Sout_bus1 = "n:Sout:bus1", "MVA, Outgoing Power, Apparent Power"
      n_cosphiout_bus1 = "n:cosphiout:bus1", "Outgoing Power, Power Factor"
      n_Pbalance_bus1 = "n:Pbalance:bus1", "MW, Active Power Balance (=0)"
      n_Qbalance_bus1 = "n:Qbalance:bus1", "Mvar, Reactive Power Balance (=0)"
      n_LossPdown_bus1 = "n:LossPdown:bus1", "MW, Losses, downstream"
      n_LossQdown_bus1 = "n:LossQdown:bus1", "Mvar, Losses, downstream (Reactive Power)"
      n_LossPdownload_bus1 = "n:LossPdownload:bus1", "MW, Load losses, downstream"
      n_LossQdownload_bus1 = "n:LossQdownload:bus1", "Mvar, Load losses, downstream"
      n_LossPdownnoload_bus1 = "n:LossPdownnoload:bus1", "MW, No load losses, downstream"
      n_LossQdownnoload_bus1 = "n:LossQdownnoload:bus1", "Mvar, No load losses, downstream"
      n_du_feed_bus1 = "n:du_feed:bus1", "%, Voltage difference relative to feeder begin"
      s_u0 = "s:u0", "p.u., IN, Voltage-Input, continuous, double"
      s_Ul0 = "s:Ul0", "kV, IN, Voltage-Input (Line-Line), continuous, double"
      s_U_A = "s:U_A", "kV, IN, Voltage, Magnitude, Phase a, continuous, double"
      s_U_B = "s:U_B", "kV, IN, Voltage, Magnitude, Phase b, continuous, double"
      s_U_C = "s:U_C", "kV, IN, Voltage, Magnitude, Phase c, continuous, double"
      s_phiu_B = "s:phiu_B", "deg, IN, Voltage, Angle, Phase b continuous, double"
      s_phiu_C = "s:phiu_C", "deg, IN, Voltage, Angle, Phase c continuous, double"
      s_dphiu = "s:dphiu", "rad, IN, Voltage Angle Input, continuous, double"
      s_dFin = "s:dFin", "Hz, IN, Frequency Deviation, continuous, double"
      s_dF = "s:dF", "Hz, OUT, Frequency Deviation Output, continuous, double"
      c_uini = "c:uini", "p.u., Voltage"
      c_phiu = "c:phiu", "rad, Voltage Angle"
      c_Kpf = "c:Kpf", "MW/Hz, Primary Frequency Bias"
      c_Ksf = "c:Ksf", "MW/Hz, Secondary Frequency Bias"
      c_Kp = "c:Kp", "Secondary Factor"
      c_iRefElement = "c:iRefElement", "Reference Element"
      c_Ploss = "c:Ploss", "MW, Losses (total)"
      c_Qloss = "c:Qloss", "Mvar, Reactive-Losses (total)"
      c_Plossld = "c:Plossld", "MW, Losses (load)"
      c_Qlossld = "c:Qlossld", "Mvar, Reactive-Losses (load)"
      c_Plossnld = "c:Plossnld", "MW, Losses (no load)"
      c_Qlossnld = "c:Qlossnld", "Mvar, Reactive-Losses (no load)"

    class ElmVsc(Enum):
      _init_ = 'value __doc__'

      m_dPdQ_busac = "m:dPdQ:busac", "MW/Mvar, Branch Sensitivity dPbranch/dQbus"
      m_dPdQ_busdp = "m:dPdQ:busdp", "MW/Mvar, Branch Sensitivity dPbranch/dQbus"
      m_dPdQ_busdm = "m:dPdQ:busdm", "MW/Mvar, Branch Sensitivity dPbranch/dQbus"
      m_dQdQ_busac = "m:dQdQ:busac", "Mvar/Mvar, Branch Sensitivity dQbranch/dQbus"
      m_dQdQ_busdp = "m:dQdQ:busdp", "Mvar/Mvar, Branch Sensitivity dQbranch/dQbus"
      m_dQdQ_busdm = "m:dQdQ:busdm", "Mvar/Mvar, Branch Sensitivity dQbranch/dQbus"
      m_dPdP_busac = "m:dPdP:busac", "MW/MW, Branch Sensitivity dPbranch/dPbus"
      m_dPdP_busdp = "m:dPdP:busdp", "MW/MW, Branch Sensitivity dPbranch/dPbus"
      m_dPdP_busdm = "m:dPdP:busdm", "MW/MW, Branch Sensitivity dPbranch/dPbus"
      m_dQdP_busac = "m:dQdP:busac", "Mvar/MW, Branch Sensitivity dQbranch/dPbus"
      m_dQdP_busdp = "m:dQdP:busdp", "Mvar/MW, Branch Sensitivity dQbranch/dPbus"
      m_dQdP_busdm = "m:dQdP:busdm", "Mvar/MW, Branch Sensitivity dQbranch/dPbus"
      m_dPdtap_busac = "m:dPdtap:busac", "MW/tap, Branch Sensitivity dPbranch/dtap"
      m_dPdtap_busdp = "m:dPdtap:busdp", "MW/tap, Branch Sensitivity dPbranch/dtap"
      m_dPdtap_busdm = "m:dPdtap:busdm", "MW/tap, Branch Sensitivity dPbranch/dtap"
      m_dQdtap_busac = "m:dQdtap:busac", "Mvar/tap, Branch Sensitivity dQbranch/dtap"
      m_dQdtap_busdp = "m:dQdtap:busdp", "Mvar/tap, Branch Sensitivity dQbranch/dtap"
      m_dQdtap_busdm = "m:dQdtap:busdm", "Mvar/tap, Branch Sensitivity dQbranch/dtap"
      m_LODF_busac = "m:LODF:busac", "%, Line Outage Distribution Factor LODF"
      m_LODF_busdp = "m:LODF:busdp", "%, Line Outage Distribution Factor LODF"
      m_LODF_busdm = "m:LODF:busdm", "%, Line Outage Distribution Factor LODF"
      m_u1r_busac = "m:u1r:busac", "p.u., Positive-Sequence-Voltage, Real Part"
      m_u1r_busdp = "m:u1r:busdp", "p.u., Positive-Sequence-Voltage, Real Part"
      m_u1r_busdm = "m:u1r:busdm", "p.u., Positive-Sequence-Voltage, Real Part"
      m_u1i_busac = "m:u1i:busac", "p.u., Positive-Sequence-Voltage, Imaginary Part"
      m_u1i_busdp = "m:u1i:busdp", "p.u., Positive-Sequence-Voltage, Imaginary Part"
      m_u1i_busdm = "m:u1i:busdm", "p.u., Positive-Sequence-Voltage, Imaginary Part"
      m_u1_busac = "m:u1:busac", "p.u., Positive-Sequence-Voltage, Magnitude"
      m_u1_busdp = "m:u1:busdp", "p.u., Positive-Sequence-Voltage, Magnitude"
      m_u1_busdm = "m:u1:busdm", "p.u., Positive-Sequence-Voltage, Magnitude"
      m_U1_busac = "m:U1:busac", "kV, Line-Ground Positive-Sequence-Voltage, Magnitude"
      m_U1_busdp = "m:U1:busdp", "kV, Line-Ground Positive-Sequence-Voltage, Magnitude"
      m_U1_busdm = "m:U1:busdm", "kV, Line-Ground Positive-Sequence-Voltage, Magnitude"
      m_U1l_busac = "m:U1l:busac", "kV, Line-Line Positive-Sequence-Voltage, Magnitude"
      m_U1l_busdp = "m:U1l:busdp", "kV, Line-Line Positive-Sequence-Voltage, Magnitude"
      m_U1l_busdm = "m:U1l:busdm", "kV, Line-Line Positive-Sequence-Voltage, Magnitude"
      m_phiu1_busac = "m:phiu1:busac", "deg, Positive-Sequence-Voltage, Angle"
      m_phiu1_busdp = "m:phiu1:busdp", "deg, Positive-Sequence-Voltage, Angle"
      m_phiu1_busdm = "m:phiu1:busdm", "deg, Positive-Sequence-Voltage, Angle"
      m_inet_busac = "m:inet:busac", "p.u., Current, Magnitude, referred to network"
      m_inet_busdp = "m:inet:busdp", "p.u., Current, Magnitude, referred to network"
      m_inet_busdm = "m:inet:busdm", "p.u., Current, Magnitude, referred to network"
      m_I_busac = "m:I:busac", "kA, Current, Magnitude"
      m_I_busdp = "m:I:busdp", "kA, Current, Magnitude"
      m_I_busdm = "m:I:busdm", "kA, Current, Magnitude"
      m_phii_busac = "m:phii:busac", "deg, Current, Angle"
      m_phii_busdp = "m:phii:busdp", "deg, Current, Angle"
      m_phii_busdm = "m:phii:busdm", "deg, Current, Angle"
      m_i1r_busac = "m:i1r:busac", "p.u., Positive-Sequence Current, Real Part"
      m_i1r_busdp = "m:i1r:busdp", "p.u., Positive-Sequence Current, Real Part"
      m_i1r_busdm = "m:i1r:busdm", "p.u., Positive-Sequence Current, Real Part"
      m_i1i_busac = "m:i1i:busac", "p.u., Positive-Sequence Current, Imaginary Part"
      m_i1i_busdp = "m:i1i:busdp", "p.u., Positive-Sequence Current, Imaginary Part"
      m_i1i_busdm = "m:i1i:busdm", "p.u., Positive-Sequence Current, Imaginary Part"
      m_i1_busac = "m:i1:busac", "p.u., Positive-Sequence Current, Magnitude"
      m_i1_busdp = "m:i1:busdp", "p.u., Positive-Sequence Current, Magnitude"
      m_i1_busdm = "m:i1:busdm", "p.u., Positive-Sequence Current, Magnitude"
      m_I1_busac = "m:I1:busac", "kA, Positive-Sequence Current, Magnitude"
      m_I1_busdp = "m:I1:busdp", "kA, Positive-Sequence Current, Magnitude"
      m_I1_busdm = "m:I1:busdm", "kA, Positive-Sequence Current, Magnitude"
      m_phii1_busac = "m:phii1:busac", "deg, Positive-Sequence Current, Angle"
      m_phii1_busdp = "m:phii1:busdp", "deg, Positive-Sequence Current, Angle"
      m_phii1_busdm = "m:phii1:busdm", "deg, Positive-Sequence Current, Angle"
      m_P_busac = "m:P:busac", "MW, Active Power"
      m_P_busdp = "m:P:busdp", "MW, Active Power"
      m_P_busdm = "m:P:busdm", "MW, Active Power"
      m_Q_busac = "m:Q:busac", "Mvar, Reactive Power"
      m_Q_busdp = "m:Q:busdp", "Mvar, Reactive Power"
      m_Q_busdm = "m:Q:busdm", "Mvar, Reactive Power"
      m_S_busac = "m:S:busac", "MVA, Apparent Power"
      m_S_busdp = "m:S:busdp", "MVA, Apparent Power"
      m_S_busdm = "m:S:busdm", "MVA, Apparent Power"
      m_cosphi_busac = "m:cosphi:busac", "Power Factor"
      m_cosphi_busdp = "m:cosphi:busdp", "Power Factor"
      m_cosphi_busdm = "m:cosphi:busdm", "Power Factor"
      m_tanphi_busac = "m:tanphi:busac", "tan(phi)"
      m_tanphi_busdp = "m:tanphi:busdp", "tan(phi)"
      m_tanphi_busdm = "m:tanphi:busdm", "tan(phi)"
      m_Psum_busac = "m:Psum:busac", "MW, Total Active Power"
      m_Psum_busdp = "m:Psum:busdp", "MW, Total Active Power"
      m_Psum_busdm = "m:Psum:busdm", "MW, Total Active Power"
      m_Qsum_busac = "m:Qsum:busac", "Mvar, Total Reactive Power"
      m_Qsum_busdp = "m:Qsum:busdp", "Mvar, Total Reactive Power"
      m_Qsum_busdm = "m:Qsum:busdm", "Mvar, Total Reactive Power"
      m_Ssum_busac = "m:Ssum:busac", "MVA, Total Apparent Power"
      m_Ssum_busdp = "m:Ssum:busdp", "MVA, Total Apparent Power"
      m_Ssum_busdm = "m:Ssum:busdm", "MVA, Total Apparent Power"
      m_cosphisum_busac = "m:cosphisum:busac", "Total Power Factor"
      m_cosphisum_busdp = "m:cosphisum:busdp", "Total Power Factor"
      m_cosphisum_busdm = "m:cosphisum:busdm", "Total Power Factor"
      m_tanphisum_busac = "m:tanphisum:busac", "Total tan(phi)"
      m_tanphisum_busdp = "m:tanphisum:busdp", "Total tan(phi)"
      m_tanphisum_busdm = "m:tanphisum:busdm", "Total tan(phi)"
      m_Spu_busac = "m:Spu:busac", "MVA/p.u., Apparent Power per p.u. Voltage"
      m_Spu_busdp = "m:Spu:busdp", "MVA/p.u., Apparent Power per p.u. Voltage"
      m_Spu_busdm = "m:Spu:busdm", "MVA/p.u., Apparent Power per p.u. Voltage"
      m_i1P_busac = "m:i1P:busac", "p.u., Positive-Sequence Active Current"
      m_i1P_busdp = "m:i1P:busdp", "p.u., Positive-Sequence Active Current"
      m_i1P_busdm = "m:i1P:busdm", "p.u., Positive-Sequence Active Current"
      m_i1Q_busac = "m:i1Q:busac", "p.u., Positive-Sequence Reactive Current"
      m_i1Q_busdp = "m:i1Q:busdp", "p.u., Positive-Sequence Reactive Current"
      m_i1Q_busdm = "m:i1Q:busdm", "p.u., Positive-Sequence Reactive Current"
      m_I1P_busac = "m:I1P:busac", "kA, Positive-Sequence Active Current"
      m_I1P_busdp = "m:I1P:busdp", "kA, Positive-Sequence Active Current"
      m_I1P_busdm = "m:I1P:busdm", "kA, Positive-Sequence Active Current"
      m_I1Q_busac = "m:I1Q:busac", "kA, Positive-Sequence Reactive Current"
      m_I1Q_busdp = "m:I1Q:busdp", "kA, Positive-Sequence Reactive Current"
      m_I1Q_busdm = "m:I1Q:busdm", "kA, Positive-Sequence Reactive Current"
      m_ur_busac = "m:ur:busac", "p.u., Voltage, Real Part"
      m_ur_busdp = "m:ur:busdp", "p.u., Voltage, Real Part"
      m_ur_busdm = "m:ur:busdm", "p.u., Voltage, Real Part"
      m_ui_busac = "m:ui:busac", "p.u., Voltage, Imaginary Part"
      m_ui_busdp = "m:ui:busdp", "p.u., Voltage, Imaginary Part"
      m_ui_busdm = "m:ui:busdm", "p.u., Voltage, Imaginary Part"
      m_u_busac = "m:u:busac", "p.u., Voltage, Magnitude"
      m_u_busdp = "m:u:busdp", "p.u., Voltage, Magnitude"
      m_u_busdm = "m:u:busdm", "p.u., Voltage, Magnitude"
      m_ir_busac = "m:ir:busac", "p.u., Current, Real Part"
      m_ir_busdp = "m:ir:busdp", "p.u., Current, Real Part"
      m_ir_busdm = "m:ir:busdm", "p.u., Current, Real Part"
      m_ii_busac = "m:ii:busac", "p.u., Current, Imaginary Part"
      m_ii_busdp = "m:ii:busdp", "p.u., Current, Imaginary Part"
      m_ii_busdm = "m:ii:busdm", "p.u., Current, Imaginary Part"
      m_i_busac = "m:i:busac", "p.u., Current, Magnitude"
      m_i_busdp = "m:i:busdp", "p.u., Current, Magnitude"
      m_i_busdm = "m:i:busdm", "p.u., Current, Magnitude"
      m_phiui_busac = "m:phiui:busac", "deg, Angle between Voltage and Current"
      m_phiui_busdp = "m:phiui:busdp", "deg, Angle between Voltage and Current"
      m_phiui_busdm = "m:phiui:busdm", "deg, Angle between Voltage and Current"
      m_phiu1i1_busac = "m:phiu1i1:busac", "deg, Angle between Voltage and Current in positive sequence system"
      m_phiu1i1_busdp = "m:phiu1i1:busdp", "deg, Angle between Voltage and Current in positive sequence system"
      m_phiu1i1_busdm = "m:phiu1i1:busdm", "deg, Angle between Voltage and Current in positive sequence system"
      m_Tfct_busac = "m:Tfct:busac", "s, Fault Clearing Time"
      m_Tfct_busdp = "m:Tfct:busdp", "s, Fault Clearing Time"
      m_Tfct_busdm = "m:Tfct:busdm", "s, Fault Clearing Time"
      m_Brkload_busac = "m:Brkload:busac", "%, Breaker Loading"
      m_Brkload_busdp = "m:Brkload:busdp", "%, Breaker Loading"
      m_Brkload_busdm = "m:Brkload:busdm", "%, Breaker Loading"
      n_dphidP_busac = "n:dphidP:busac", "deg/MW, dphi/dP Sensitivity"
      n_dphidP_busdp = "n:dphidP:busdp", "deg/MW, dphi/dP Sensitivity"
      n_dphidP_busdm = "n:dphidP:busdm", "deg/MW, dphi/dP Sensitivity"
      n_dphidQ_busac = "n:dphidQ:busac", "deg/Mvar, dphi/dQ Sensitivity"
      n_dphidQ_busdp = "n:dphidQ:busdp", "deg/Mvar, dphi/dQ Sensitivity"
      n_dphidQ_busdm = "n:dphidQ:busdm", "deg/Mvar, dphi/dQ Sensitivity"
      n_dphidtap_busac = "n:dphidtap:busac", "deg/tap, dphi/dtap Sensitivity"
      n_dphidtap_busdp = "n:dphidtap:busdp", "deg/tap, dphi/dtap Sensitivity"
      n_dphidtap_busdm = "n:dphidtap:busdm", "deg/tap, dphi/dtap Sensitivity"
      n_dvdP_busac = "n:dvdP:busac", "p.u./MW, dv/dP Sensitivity"
      n_dvdP_busdp = "n:dvdP:busdp", "p.u./MW, dv/dP Sensitivity"
      n_dvdP_busdm = "n:dvdP:busdm", "p.u./MW, dv/dP Sensitivity"
      n_dvdQ_busac = "n:dvdQ:busac", "p.u./Mvar, dv/dQ Sensitivity"
      n_dvdQ_busdp = "n:dvdQ:busdp", "p.u./Mvar, dv/dQ Sensitivity"
      n_dvdQ_busdm = "n:dvdQ:busdm", "p.u./Mvar, dv/dQ Sensitivity"
      n_dvdtap_busac = "n:dvdtap:busac", "p.u./tap, dv/dtap Sensitivity"
      n_dvdtap_busdp = "n:dvdtap:busdp", "p.u./tap, dv/dtap Sensitivity"
      n_dvdtap_busdm = "n:dvdtap:busdm", "p.u./tap, dv/dtap Sensitivity"
      n_levecQ_busac = "n:levecQ:busac", "Left Eigenvector of selected Q-Mode"
      n_levecQ_busdp = "n:levecQ:busdp", "Left Eigenvector of selected Q-Mode"
      n_levecQ_busdm = "n:levecQ:busdm", "Left Eigenvector of selected Q-Mode"
      n_revecQ_busac = "n:revecQ:busac", "Right Eigenvector of selected Q-Mode"
      n_revecQ_busdp = "n:revecQ:busdp", "Right Eigenvector of selected Q-Mode"
      n_revecQ_busdm = "n:revecQ:busdm", "Right Eigenvector of selected Q-Mode"
      n_partQ_busac = "n:partQ:busac", "Participation Factor of selected Q-Mode"
      n_partQ_busdp = "n:partQ:busdp", "Participation Factor of selected Q-Mode"
      n_partQ_busdm = "n:partQ:busdm", "Participation Factor of selected Q-Mode"
      n_ur_busac = "n:ur:busac", "p.u., Voltage, Real Part"
      n_ur_busdp = "n:ur:busdp", "p.u., Voltage, Real Part"
      n_ur_busdm = "n:ur:busdm", "p.u., Voltage, Real Part"
      n_ui_busac = "n:ui:busac", "p.u., Voltage, Imaginary Part"
      n_ui_busdp = "n:ui:busdp", "p.u., Voltage, Imaginary Part"
      n_ui_busdm = "n:ui:busdm", "p.u., Voltage, Imaginary Part"
      n_u_busac = "n:u:busac", "p.u., Voltage, Magnitude"
      n_u_busdp = "n:u:busdp", "p.u., Voltage, Magnitude"
      n_u_busdm = "n:u:busdm", "p.u., Voltage, Magnitude"
      n_upc_busac = "n:upc:busac", "%, Voltage, Magnitude"
      n_upc_busdp = "n:upc:busdp", "%, Voltage, Magnitude"
      n_upc_busdm = "n:upc:busdm", "%, Voltage, Magnitude"
      n_u1_busac = "n:u1:busac", "p.u., Voltage, Magnitude"
      n_u1_busdp = "n:u1:busdp", "p.u., Voltage, Magnitude"
      n_u1_busdm = "n:u1:busdm", "p.u., Voltage, Magnitude"
      n_u1pc_busac = "n:u1pc:busac", "%, Voltage, Magnitude"
      n_u1pc_busdp = "n:u1pc:busdp", "%, Voltage, Magnitude"
      n_u1pc_busdm = "n:u1pc:busdm", "%, Voltage, Magnitude"
      n_u1r_busac = "n:u1r:busac", "p.u., Positive-Sequence Voltage, Real Part"
      n_u1r_busdp = "n:u1r:busdp", "p.u., Positive-Sequence Voltage, Real Part"
      n_u1r_busdm = "n:u1r:busdm", "p.u., Positive-Sequence Voltage, Real Part"
      n_u1i_busac = "n:u1i:busac", "p.u., Positive-Sequence Voltage, Imaginary Part"
      n_u1i_busdp = "n:u1i:busdp", "p.u., Positive-Sequence Voltage, Imaginary Part"
      n_u1i_busdm = "n:u1i:busdm", "p.u., Positive-Sequence Voltage, Imaginary Part"
      n_U_busac = "n:U:busac", "kV, Line-Ground Voltage, Magnitude"
      n_U_busdp = "n:U:busdp", "kV, Line-Ground Voltage, Magnitude"
      n_U_busdm = "n:U:busdm", "kV, Line-Ground Voltage, Magnitude"
      n_Ul_busac = "n:Ul:busac", "kV, Line-Line Voltage, Magnitude"
      n_Ul_busdp = "n:Ul:busdp", "kV, Line-Line Voltage, Magnitude"
      n_Ul_busdm = "n:Ul:busdm", "kV, Line-Line Voltage, Magnitude"
      n_phiu_busac = "n:phiu:busac", "deg, Voltage, Angle"
      n_phiu_busdp = "n:phiu:busdp", "deg, Voltage, Angle"
      n_phiu_busdm = "n:phiu:busdm", "deg, Voltage, Angle"
      n_phiurel_busac = "n:phiurel:busac", "deg, Voltage, Relative Angle"
      n_phiurel_busdp = "n:phiurel:busdp", "deg, Voltage, Relative Angle"
      n_phiurel_busdm = "n:phiurel:busdm", "deg, Voltage, Relative Angle"
      n_du_busac = "n:du:busac", "%, Voltage Deviation"
      n_du_busdp = "n:du:busdp", "%, Voltage Deviation"
      n_du_busdm = "n:du:busdm", "%, Voltage Deviation"
      n_Pgen_busac = "n:Pgen:busac", "MW, Generation, Active Power"
      n_Pgen_busdp = "n:Pgen:busdp", "MW, Generation, Active Power"
      n_Pgen_busdm = "n:Pgen:busdm", "MW, Generation, Active Power"
      n_Qgen_busac = "n:Qgen:busac", "Mvar, Generation, Reactive Power"
      n_Qgen_busdp = "n:Qgen:busdp", "Mvar, Generation, Reactive Power"
      n_Qgen_busdm = "n:Qgen:busdm", "Mvar, Generation, Reactive Power"
      n_Pmot_busac = "n:Pmot:busac", "MW, Motor Load, Active Power"
      n_Pmot_busdp = "n:Pmot:busdp", "MW, Motor Load, Active Power"
      n_Pmot_busdm = "n:Pmot:busdm", "MW, Motor Load, Active Power"
      n_Qmot_busac = "n:Qmot:busac", "Mvar, Motor Load, Reactive Power"
      n_Qmot_busdp = "n:Qmot:busdp", "Mvar, Motor Load, Reactive Power"
      n_Qmot_busdm = "n:Qmot:busdm", "Mvar, Motor Load, Reactive Power"
      n_Pload_busac = "n:Pload:busac", "MW, General Load, Active Power"
      n_Pload_busdp = "n:Pload:busdp", "MW, General Load, Active Power"
      n_Pload_busdm = "n:Pload:busdm", "MW, General Load, Active Power"
      n_Qload_busac = "n:Qload:busac", "Mvar, General Load, Reactive Power"
      n_Qload_busdp = "n:Qload:busdp", "Mvar, General Load, Reactive Power"
      n_Qload_busdm = "n:Qload:busdm", "Mvar, General Load, Reactive Power"
      n_Pcomp_busac = "n:Pcomp:busac", "MW, Compensation (Losses)"
      n_Pcomp_busdp = "n:Pcomp:busdp", "MW, Compensation (Losses)"
      n_Pcomp_busdm = "n:Pcomp:busdm", "MW, Compensation (Losses)"
      n_Qcomp_busac = "n:Qcomp:busac", "Mvar, Compensation"
      n_Qcomp_busdp = "n:Qcomp:busdp", "Mvar, Compensation"
      n_Qcomp_busdm = "n:Qcomp:busdm", "Mvar, Compensation"
      n_Pnet_busac = "n:Pnet:busac", "MW, External Networks, Active Power"
      n_Pnet_busdp = "n:Pnet:busdp", "MW, External Networks, Active Power"
      n_Pnet_busdm = "n:Pnet:busdm", "MW, External Networks, Active Power"
      n_Qnet_busac = "n:Qnet:busac", "Mvar, External Networks, Reactive Power"
      n_Qnet_busdp = "n:Qnet:busdp", "Mvar, External Networks, Reactive Power"
      n_Qnet_busdm = "n:Qnet:busdm", "Mvar, External Networks, Reactive Power"
      n_Pflow_busac = "n:Pflow:busac", "MW, Power Flow, Active Power"
      n_Pflow_busdp = "n:Pflow:busdp", "MW, Power Flow, Active Power"
      n_Pflow_busdm = "n:Pflow:busdm", "MW, Power Flow, Active Power"
      n_Qflow_busac = "n:Qflow:busac", "Mvar, Power Flow, Reactive Power"
      n_Qflow_busdp = "n:Qflow:busdp", "Mvar, Power Flow, Reactive Power"
      n_Qflow_busdm = "n:Qflow:busdm", "Mvar, Power Flow, Reactive Power"
      n_Pout_busac = "n:Pout:busac", "MW, Outgoing Flow, Active Power"
      n_Pout_busdp = "n:Pout:busdp", "MW, Outgoing Flow, Active Power"
      n_Pout_busdm = "n:Pout:busdm", "MW, Outgoing Flow, Active Power"
      n_Qout_busac = "n:Qout:busac", "Mvar, Outgoing Power, Reactive Power"
      n_Qout_busdp = "n:Qout:busdp", "Mvar, Outgoing Power, Reactive Power"
      n_Qout_busdm = "n:Qout:busdm", "Mvar, Outgoing Power, Reactive Power"
      n_Sout_busac = "n:Sout:busac", "MVA, Outgoing Power, Apparent Power"
      n_Sout_busdp = "n:Sout:busdp", "MVA, Outgoing Power, Apparent Power"
      n_Sout_busdm = "n:Sout:busdm", "MVA, Outgoing Power, Apparent Power"
      n_cosphiout_busac = "n:cosphiout:busac", "Outgoing Power, Power Factor"
      n_cosphiout_busdp = "n:cosphiout:busdp", "Outgoing Power, Power Factor"
      n_cosphiout_busdm = "n:cosphiout:busdm", "Outgoing Power, Power Factor"
      n_Pbalance_busac = "n:Pbalance:busac", "MW, Active Power Balance (=0)"
      n_Pbalance_busdp = "n:Pbalance:busdp", "MW, Active Power Balance (=0)"
      n_Pbalance_busdm = "n:Pbalance:busdm", "MW, Active Power Balance (=0)"
      n_Qbalance_busac = "n:Qbalance:busac", "Mvar, Reactive Power Balance (=0)"
      n_Qbalance_busdp = "n:Qbalance:busdp", "Mvar, Reactive Power Balance (=0)"
      n_Qbalance_busdm = "n:Qbalance:busdm", "Mvar, Reactive Power Balance (=0)"
      n_LossPdown_busac = "n:LossPdown:busac", "MW, Losses, downstream"
      n_LossPdown_busdp = "n:LossPdown:busdp", "MW, Losses, downstream"
      n_LossPdown_busdm = "n:LossPdown:busdm", "MW, Losses, downstream"
      n_LossQdown_busac = "n:LossQdown:busac", "Mvar, Losses, downstream (Reactive Power)"
      n_LossQdown_busdp = "n:LossQdown:busdp", "Mvar, Losses, downstream (Reactive Power)"
      n_LossQdown_busdm = "n:LossQdown:busdm", "Mvar, Losses, downstream (Reactive Power)"
      n_LossPdownload_busac = "n:LossPdownload:busac", "MW, Load losses, downstream"
      n_LossPdownload_busdp = "n:LossPdownload:busdp", "MW, Load losses, downstream"
      n_LossPdownload_busdm = "n:LossPdownload:busdm", "MW, Load losses, downstream"
      n_LossQdownload_busac = "n:LossQdownload:busac", "Mvar, Load losses, downstream"
      n_LossQdownload_busdp = "n:LossQdownload:busdp", "Mvar, Load losses, downstream"
      n_LossQdownload_busdm = "n:LossQdownload:busdm", "Mvar, Load losses, downstream"
      n_LossPdownnoload_busac = "n:LossPdownnoload:busac", "MW, No load losses, downstream"
      n_LossPdownnoload_busdp = "n:LossPdownnoload:busdp", "MW, No load losses, downstream"
      n_LossPdownnoload_busdm = "n:LossPdownnoload:busdm", "MW, No load losses, downstream"
      n_LossQdownnoload_busac = "n:LossQdownnoload:busac", "Mvar, No load losses, downstream"
      n_LossQdownnoload_busdp = "n:LossQdownnoload:busdp", "Mvar, No load losses, downstream"
      n_LossQdownnoload_busdm = "n:LossQdownnoload:busdm", "Mvar, No load losses, downstream"
      n_du_feed_busac = "n:du_feed:busac", "%, Voltage difference relative to feeder begin"
      n_du_feed_busdp = "n:du_feed:busdp", "%, Voltage difference relative to feeder begin"
      n_du_feed_busdm = "n:du_feed:busdm", "%, Voltage difference relative to feeder begin"
      s_pset = "s:pset", "MW, IN, Active power setpoint, continuous, double"
      s_qset = "s:qset", "Mvar, IN, Reactive power setpoint continuous, double"
      s_udcset = "s:udcset", "p.u., IN, DC Voltage setpoint, continuous, double"
      s_dFin = "s:dFin", "Hz, IN, Frequency Deviation, continuous, double"
      c_phiu = "c:phiu", "rad, Voltage angle"
      c_uini = "c:uini", "p.u., Voltage setpoint"
      c_snomn = "c:snomn", "MVA, Total rated apparent power (nparnum*Snom)"
      c_uinidc = "c:uinidc", "p.u., Voltage setpoint (DC)"
      c_pmsetp = "c:pmsetp", "PWM factor"
      c_i_acdc = "c:i_acdc", "Control mode"
      c_i_mod = "c:i_mod", "Modulation"
      c_K0 = "c:K0", "Vac/Vdc at PWM=1"
      c_Pm = "c:Pm", "Pulse-modulation factor"
      c_uDC = "c:uDC", "p.u., DC-voltage"
      c_pini = "c:pini", "Active power setpoint"
      c_qini = "c:qini", "Reactive power setpoint"
      c_DCPoleLosses = "c:DCPoleLosses", "MW, DC pole losses"
      c_Qmax = "c:Qmax", "Mvar, Upper limit of reactive power"
      c_Qmin = "c:Qmin", "Mvar, Lower limit of reactive power"
      c_Pmin = "c:Pmin", "MW, Lower limit of active power"
      c_Pmax = "c:Pmax", "MW, Upper limit of active power"
      c_i_qlim = "c:i_qlim", "Reactive power limited"
      c_i_plim = "c:i_plim", "Active power limited"
      c_Kpf = "c:Kpf", "MW/Hz, Frequency bias"
      c_Kp = "c:Kp", "p.u., Active power participation"
      c_zonewscale = "c:zonewscale", "Wind generation zone scaling factor"
      c_iRefElement = "c:iRefElement", "Reference element"
      c_Kq = "c:Kq", "p.u., Reactive power participation"
      c_iPowerOn = "c:iPowerOn", "Flag"
      c_i_vlim = "c:i_vlim", "Voltage setpoint limit reached"
      c_dphi = "c:dphi", "deg, Measured angle difference"
      c_Pmeas = "c:Pmeas", "MW, Measured active power"
      c_nCtrlIter = "c:nCtrlIter", "No. of CTRL-iterations"
      c_Ploss = "c:Ploss", "MW, Losses (total)"
      c_Qloss = "c:Qloss", "Mvar, Reactive-Losses (total)"
      c_Plossld = "c:Plossld", "MW, Losses (load)"
      c_Qlossld = "c:Qlossld", "Mvar, Reactive-Losses (load)"
      c_Plossnld = "c:Plossnld", "MW, Losses (no load)"
      c_Qlossnld = "c:Qlossnld", "Mvar, Reactive-Losses (no load)"

    class ElmVscmono(Enum):
      _init_ = 'value __doc__'

      m_dPdQ_busac = "m:dPdQ:busac", "MW/Mvar, Branch Sensitivity dPbranch/dQbus"
      m_dPdQ_busdc = "m:dPdQ:busdc", "MW/Mvar, Branch Sensitivity dPbranch/dQbus"
      m_dQdQ_busac = "m:dQdQ:busac", "Mvar/Mvar, Branch Sensitivity dQbranch/dQbus"
      m_dQdQ_busdc = "m:dQdQ:busdc", "Mvar/Mvar, Branch Sensitivity dQbranch/dQbus"
      m_dPdP_busac = "m:dPdP:busac", "MW/MW, Branch Sensitivity dPbranch/dPbus"
      m_dPdP_busdc = "m:dPdP:busdc", "MW/MW, Branch Sensitivity dPbranch/dPbus"
      m_dQdP_busac = "m:dQdP:busac", "Mvar/MW, Branch Sensitivity dQbranch/dPbus"
      m_dQdP_busdc = "m:dQdP:busdc", "Mvar/MW, Branch Sensitivity dQbranch/dPbus"
      m_dPdtap_busac = "m:dPdtap:busac", "MW/tap, Branch Sensitivity dPbranch/dtap"
      m_dPdtap_busdc = "m:dPdtap:busdc", "MW/tap, Branch Sensitivity dPbranch/dtap"
      m_dQdtap_busac = "m:dQdtap:busac", "Mvar/tap, Branch Sensitivity dQbranch/dtap"
      m_dQdtap_busdc = "m:dQdtap:busdc", "Mvar/tap, Branch Sensitivity dQbranch/dtap"
      m_LODF_busac = "m:LODF:busac", "%, Line Outage Distribution Factor LODF"
      m_LODF_busdc = "m:LODF:busdc", "%, Line Outage Distribution Factor LODF"
      m_u1r_busac = "m:u1r:busac", "p.u., Positive-Sequence-Voltage, Real Part"
      m_u1r_busdc = "m:u1r:busdc", "p.u., Positive-Sequence-Voltage, Real Part"
      m_u1i_busac = "m:u1i:busac", "p.u., Positive-Sequence-Voltage, Imaginary Part"
      m_u1i_busdc = "m:u1i:busdc", "p.u., Positive-Sequence-Voltage, Imaginary Part"
      m_u1_busac = "m:u1:busac", "p.u., Positive-Sequence-Voltage, Magnitude"
      m_u1_busdc = "m:u1:busdc", "p.u., Positive-Sequence-Voltage, Magnitude"
      m_U1_busac = "m:U1:busac", "kV, Line-Ground Positive-Sequence-Voltage, Magnitude"
      m_U1_busdc = "m:U1:busdc", "kV, Line-Ground Positive-Sequence-Voltage, Magnitude"
      m_U1l_busac = "m:U1l:busac", "kV, Line-Line Positive-Sequence-Voltage, Magnitude"
      m_U1l_busdc = "m:U1l:busdc", "kV, Line-Line Positive-Sequence-Voltage, Magnitude"
      m_phiu1_busac = "m:phiu1:busac", "deg, Positive-Sequence-Voltage, Angle"
      m_phiu1_busdc = "m:phiu1:busdc", "deg, Positive-Sequence-Voltage, Angle"
      m_inet_busac = "m:inet:busac", "p.u., Current, Magnitude, referred to network"
      m_inet_busdc = "m:inet:busdc", "p.u., Current, Magnitude, referred to network"
      m_I_busac = "m:I:busac", "kA, Current, Magnitude"
      m_I_busdc = "m:I:busdc", "kA, Current, Magnitude"
      m_phii_busac = "m:phii:busac", "deg, Current, Angle"
      m_phii_busdc = "m:phii:busdc", "deg, Current, Angle"
      m_i1r_busac = "m:i1r:busac", "p.u., Positive-Sequence Current, Real Part"
      m_i1r_busdc = "m:i1r:busdc", "p.u., Positive-Sequence Current, Real Part"
      m_i1i_busac = "m:i1i:busac", "p.u., Positive-Sequence Current, Imaginary Part"
      m_i1i_busdc = "m:i1i:busdc", "p.u., Positive-Sequence Current, Imaginary Part"
      m_i1_busac = "m:i1:busac", "p.u., Positive-Sequence Current, Magnitude"
      m_i1_busdc = "m:i1:busdc", "p.u., Positive-Sequence Current, Magnitude"
      m_I1_busac = "m:I1:busac", "kA, Positive-Sequence Current, Magnitude"
      m_I1_busdc = "m:I1:busdc", "kA, Positive-Sequence Current, Magnitude"
      m_phii1_busac = "m:phii1:busac", "deg, Positive-Sequence Current, Angle"
      m_phii1_busdc = "m:phii1:busdc", "deg, Positive-Sequence Current, Angle"
      m_P_busac = "m:P:busac", "MW, Active Power"
      m_P_busdc = "m:P:busdc", "MW, Active Power"
      m_Q_busac = "m:Q:busac", "Mvar, Reactive Power"
      m_Q_busdc = "m:Q:busdc", "Mvar, Reactive Power"
      m_S_busac = "m:S:busac", "MVA, Apparent Power"
      m_S_busdc = "m:S:busdc", "MVA, Apparent Power"
      m_cosphi_busac = "m:cosphi:busac", "Power Factor"
      m_cosphi_busdc = "m:cosphi:busdc", "Power Factor"
      m_tanphi_busac = "m:tanphi:busac", "tan(phi)"
      m_tanphi_busdc = "m:tanphi:busdc", "tan(phi)"
      m_Psum_busac = "m:Psum:busac", "MW, Total Active Power"
      m_Psum_busdc = "m:Psum:busdc", "MW, Total Active Power"
      m_Qsum_busac = "m:Qsum:busac", "Mvar, Total Reactive Power"
      m_Qsum_busdc = "m:Qsum:busdc", "Mvar, Total Reactive Power"
      m_Ssum_busac = "m:Ssum:busac", "MVA, Total Apparent Power"
      m_Ssum_busdc = "m:Ssum:busdc", "MVA, Total Apparent Power"
      m_cosphisum_busac = "m:cosphisum:busac", "Total Power Factor"
      m_cosphisum_busdc = "m:cosphisum:busdc", "Total Power Factor"
      m_tanphisum_busac = "m:tanphisum:busac", "Total tan(phi)"
      m_tanphisum_busdc = "m:tanphisum:busdc", "Total tan(phi)"
      m_Spu_busac = "m:Spu:busac", "MVA/p.u., Apparent Power per p.u. Voltage"
      m_Spu_busdc = "m:Spu:busdc", "MVA/p.u., Apparent Power per p.u. Voltage"
      m_i1P_busac = "m:i1P:busac", "p.u., Positive-Sequence Active Current"
      m_i1P_busdc = "m:i1P:busdc", "p.u., Positive-Sequence Active Current"
      m_i1Q_busac = "m:i1Q:busac", "p.u., Positive-Sequence Reactive Current"
      m_i1Q_busdc = "m:i1Q:busdc", "p.u., Positive-Sequence Reactive Current"
      m_I1P_busac = "m:I1P:busac", "kA, Positive-Sequence Active Current"
      m_I1P_busdc = "m:I1P:busdc", "kA, Positive-Sequence Active Current"
      m_I1Q_busac = "m:I1Q:busac", "kA, Positive-Sequence Reactive Current"
      m_I1Q_busdc = "m:I1Q:busdc", "kA, Positive-Sequence Reactive Current"
      m_ur_busac = "m:ur:busac", "p.u., Voltage, Real Part"
      m_ur_busdc = "m:ur:busdc", "p.u., Voltage, Real Part"
      m_ui_busac = "m:ui:busac", "p.u., Voltage, Imaginary Part"
      m_ui_busdc = "m:ui:busdc", "p.u., Voltage, Imaginary Part"
      m_u_busac = "m:u:busac", "p.u., Voltage, Magnitude"
      m_u_busdc = "m:u:busdc", "p.u., Voltage, Magnitude"
      m_ir_busac = "m:ir:busac", "p.u., Current, Real Part"
      m_ir_busdc = "m:ir:busdc", "p.u., Current, Real Part"
      m_ii_busac = "m:ii:busac", "p.u., Current, Imaginary Part"
      m_ii_busdc = "m:ii:busdc", "p.u., Current, Imaginary Part"
      m_i_busac = "m:i:busac", "p.u., Current, Magnitude"
      m_i_busdc = "m:i:busdc", "p.u., Current, Magnitude"
      m_phiui_busac = "m:phiui:busac", "deg, Angle between Voltage and Current"
      m_phiui_busdc = "m:phiui:busdc", "deg, Angle between Voltage and Current"
      m_phiu1i1_busac = "m:phiu1i1:busac", "deg, Angle between Voltage and Current in positive sequence system"
      m_phiu1i1_busdc = "m:phiu1i1:busdc", "deg, Angle between Voltage and Current in positive sequence system"
      m_Tfct_busac = "m:Tfct:busac", "s, Fault Clearing Time"
      m_Tfct_busdc = "m:Tfct:busdc", "s, Fault Clearing Time"
      m_Brkload_busac = "m:Brkload:busac", "%, Breaker Loading"
      m_Brkload_busdc = "m:Brkload:busdc", "%, Breaker Loading"
      n_dphidP_busac = "n:dphidP:busac", "deg/MW, dphi/dP Sensitivity"
      n_dphidP_busdc = "n:dphidP:busdc", "deg/MW, dphi/dP Sensitivity"
      n_dphidQ_busac = "n:dphidQ:busac", "deg/Mvar, dphi/dQ Sensitivity"
      n_dphidQ_busdc = "n:dphidQ:busdc", "deg/Mvar, dphi/dQ Sensitivity"
      n_dphidtap_busac = "n:dphidtap:busac", "deg/tap, dphi/dtap Sensitivity"
      n_dphidtap_busdc = "n:dphidtap:busdc", "deg/tap, dphi/dtap Sensitivity"
      n_dvdP_busac = "n:dvdP:busac", "p.u./MW, dv/dP Sensitivity"
      n_dvdP_busdc = "n:dvdP:busdc", "p.u./MW, dv/dP Sensitivity"
      n_dvdQ_busac = "n:dvdQ:busac", "p.u./Mvar, dv/dQ Sensitivity"
      n_dvdQ_busdc = "n:dvdQ:busdc", "p.u./Mvar, dv/dQ Sensitivity"
      n_dvdtap_busac = "n:dvdtap:busac", "p.u./tap, dv/dtap Sensitivity"
      n_dvdtap_busdc = "n:dvdtap:busdc", "p.u./tap, dv/dtap Sensitivity"
      n_levecQ_busac = "n:levecQ:busac", "Left Eigenvector of selected Q-Mode"
      n_levecQ_busdc = "n:levecQ:busdc", "Left Eigenvector of selected Q-Mode"
      n_revecQ_busac = "n:revecQ:busac", "Right Eigenvector of selected Q-Mode"
      n_revecQ_busdc = "n:revecQ:busdc", "Right Eigenvector of selected Q-Mode"
      n_partQ_busac = "n:partQ:busac", "Participation Factor of selected Q-Mode"
      n_partQ_busdc = "n:partQ:busdc", "Participation Factor of selected Q-Mode"
      n_ur_busac = "n:ur:busac", "p.u., Voltage, Real Part"
      n_ur_busdc = "n:ur:busdc", "p.u., Voltage, Real Part"
      n_ui_busac = "n:ui:busac", "p.u., Voltage, Imaginary Part"
      n_ui_busdc = "n:ui:busdc", "p.u., Voltage, Imaginary Part"
      n_u_busac = "n:u:busac", "p.u., Voltage, Magnitude"
      n_u_busdc = "n:u:busdc", "p.u., Voltage, Magnitude"
      n_upc_busac = "n:upc:busac", "%, Voltage, Magnitude"
      n_upc_busdc = "n:upc:busdc", "%, Voltage, Magnitude"
      n_u1_busac = "n:u1:busac", "p.u., Voltage, Magnitude"
      n_u1_busdc = "n:u1:busdc", "p.u., Voltage, Magnitude"
      n_u1pc_busac = "n:u1pc:busac", "%, Voltage, Magnitude"
      n_u1pc_busdc = "n:u1pc:busdc", "%, Voltage, Magnitude"
      n_u1r_busac = "n:u1r:busac", "p.u., Positive-Sequence Voltage, Real Part"
      n_u1r_busdc = "n:u1r:busdc", "p.u., Positive-Sequence Voltage, Real Part"
      n_u1i_busac = "n:u1i:busac", "p.u., Positive-Sequence Voltage, Imaginary Part"
      n_u1i_busdc = "n:u1i:busdc", "p.u., Positive-Sequence Voltage, Imaginary Part"
      n_U_busac = "n:U:busac", "kV, Line-Ground Voltage, Magnitude"
      n_U_busdc = "n:U:busdc", "kV, Line-Ground Voltage, Magnitude"
      n_Ul_busac = "n:Ul:busac", "kV, Line-Line Voltage, Magnitude"
      n_Ul_busdc = "n:Ul:busdc", "kV, Line-Line Voltage, Magnitude"
      n_phiu_busac = "n:phiu:busac", "deg, Voltage, Angle"
      n_phiu_busdc = "n:phiu:busdc", "deg, Voltage, Angle"
      n_phiurel_busac = "n:phiurel:busac", "deg, Voltage, Relative Angle"
      n_phiurel_busdc = "n:phiurel:busdc", "deg, Voltage, Relative Angle"
      n_du_busac = "n:du:busac", "%, Voltage Deviation"
      n_du_busdc = "n:du:busdc", "%, Voltage Deviation"
      n_Pgen_busac = "n:Pgen:busac", "MW, Generation, Active Power"
      n_Pgen_busdc = "n:Pgen:busdc", "MW, Generation, Active Power"
      n_Qgen_busac = "n:Qgen:busac", "Mvar, Generation, Reactive Power"
      n_Qgen_busdc = "n:Qgen:busdc", "Mvar, Generation, Reactive Power"
      n_Pmot_busac = "n:Pmot:busac", "MW, Motor Load, Active Power"
      n_Pmot_busdc = "n:Pmot:busdc", "MW, Motor Load, Active Power"
      n_Qmot_busac = "n:Qmot:busac", "Mvar, Motor Load, Reactive Power"
      n_Qmot_busdc = "n:Qmot:busdc", "Mvar, Motor Load, Reactive Power"
      n_Pload_busac = "n:Pload:busac", "MW, General Load, Active Power"
      n_Pload_busdc = "n:Pload:busdc", "MW, General Load, Active Power"
      n_Qload_busac = "n:Qload:busac", "Mvar, General Load, Reactive Power"
      n_Qload_busdc = "n:Qload:busdc", "Mvar, General Load, Reactive Power"
      n_Pcomp_busac = "n:Pcomp:busac", "MW, Compensation (Losses)"
      n_Pcomp_busdc = "n:Pcomp:busdc", "MW, Compensation (Losses)"
      n_Qcomp_busac = "n:Qcomp:busac", "Mvar, Compensation"
      n_Qcomp_busdc = "n:Qcomp:busdc", "Mvar, Compensation"
      n_Pnet_busac = "n:Pnet:busac", "MW, External Networks, Active Power"
      n_Pnet_busdc = "n:Pnet:busdc", "MW, External Networks, Active Power"
      n_Qnet_busac = "n:Qnet:busac", "Mvar, External Networks, Reactive Power"
      n_Qnet_busdc = "n:Qnet:busdc", "Mvar, External Networks, Reactive Power"
      n_Pflow_busac = "n:Pflow:busac", "MW, Power Flow, Active Power"
      n_Pflow_busdc = "n:Pflow:busdc", "MW, Power Flow, Active Power"
      n_Qflow_busac = "n:Qflow:busac", "Mvar, Power Flow, Reactive Power"
      n_Qflow_busdc = "n:Qflow:busdc", "Mvar, Power Flow, Reactive Power"
      n_Pout_busac = "n:Pout:busac", "MW, Outgoing Flow, Active Power"
      n_Pout_busdc = "n:Pout:busdc", "MW, Outgoing Flow, Active Power"
      n_Qout_busac = "n:Qout:busac", "Mvar, Outgoing Power, Reactive Power"
      n_Qout_busdc = "n:Qout:busdc", "Mvar, Outgoing Power, Reactive Power"
      n_Sout_busac = "n:Sout:busac", "MVA, Outgoing Power, Apparent Power"
      n_Sout_busdc = "n:Sout:busdc", "MVA, Outgoing Power, Apparent Power"
      n_cosphiout_busac = "n:cosphiout:busac", "Outgoing Power, Power Factor"
      n_cosphiout_busdc = "n:cosphiout:busdc", "Outgoing Power, Power Factor"
      n_Pbalance_busac = "n:Pbalance:busac", "MW, Active Power Balance (=0)"
      n_Pbalance_busdc = "n:Pbalance:busdc", "MW, Active Power Balance (=0)"
      n_Qbalance_busac = "n:Qbalance:busac", "Mvar, Reactive Power Balance (=0)"
      n_Qbalance_busdc = "n:Qbalance:busdc", "Mvar, Reactive Power Balance (=0)"
      n_LossPdown_busac = "n:LossPdown:busac", "MW, Losses, downstream"
      n_LossPdown_busdc = "n:LossPdown:busdc", "MW, Losses, downstream"
      n_LossQdown_busac = "n:LossQdown:busac", "Mvar, Losses, downstream (Reactive Power)"
      n_LossQdown_busdc = "n:LossQdown:busdc", "Mvar, Losses, downstream (Reactive Power)"
      n_LossPdownload_busac = "n:LossPdownload:busac", "MW, Load losses, downstream"
      n_LossPdownload_busdc = "n:LossPdownload:busdc", "MW, Load losses, downstream"
      n_LossQdownload_busac = "n:LossQdownload:busac", "Mvar, Load losses, downstream"
      n_LossQdownload_busdc = "n:LossQdownload:busdc", "Mvar, Load losses, downstream"
      n_LossPdownnoload_busac = "n:LossPdownnoload:busac", "MW, No load losses, downstream"
      n_LossPdownnoload_busdc = "n:LossPdownnoload:busdc", "MW, No load losses, downstream"
      n_LossQdownnoload_busac = "n:LossQdownnoload:busac", "Mvar, No load losses, downstream"
      n_LossQdownnoload_busdc = "n:LossQdownnoload:busdc", "Mvar, No load losses, downstream"
      n_du_feed_busac = "n:du_feed:busac", "%, Voltage difference relative to feeder begin"
      n_du_feed_busdc = "n:du_feed:busdc", "%, Voltage difference relative to feeder begin"
      s_pset = "s:pset", "MW, IN, Active power setpoint, continuous, double"
      s_qset = "s:qset", "Mvar, IN, Reactive power setpoint continuous, double"
      s_udcset = "s:udcset", "p.u., IN, DC Voltage setpoint, continuous, double"
      s_dFin = "s:dFin", "Hz, IN, Frequency Deviation, continuous, double"
      c_phiu = "c:phiu", "rad, Voltage angle"
      c_uini = "c:uini", "p.u., Voltage setpoint"
      c_snomn = "c:snomn", "MVA, Total rated apparent power (nparnum*Snom)"
      c_uinidc = "c:uinidc", "p.u., Voltage setpoint (DC)"
      c_pmsetp = "c:pmsetp", "PWM factor"
      c_i_acdc = "c:i_acdc", "Control mode"
      c_i_mod = "c:i_mod", "Modulation"
      c_K0 = "c:K0", "Vac/Vdc at PWM=1"
      c_Pm = "c:Pm", "Pulse-modulation factor"
      c_uDC = "c:uDC", "p.u., DC-voltage"
      c_pini = "c:pini", "Active power setpoint"
      c_qini = "c:qini", "Reactive power setpoint"
      c_DCPoleLosses = "c:DCPoleLosses", "MW, DC pole losses"
      c_Qmax = "c:Qmax", "Mvar, Upper limit of reactive power"
      c_Qmin = "c:Qmin", "Mvar, Lower limit of reactive power"
      c_Pmin = "c:Pmin", "MW, Lower limit of active power"
      c_Pmax = "c:Pmax", "MW, Upper limit of active power"
      c_i_qlim = "c:i_qlim", "Reactive power limited"
      c_i_plim = "c:i_plim", "Active power limited"
      c_Kpf = "c:Kpf", "MW/Hz, Frequency bias"
      c_Kp = "c:Kp", "p.u., Active power participation"
      c_zonewscale = "c:zonewscale", "Wind generation zone scaling factor"
      c_iRefElement = "c:iRefElement", "Reference element"
      c_Kq = "c:Kq", "p.u., Reactive power participation"
      c_iPowerOn = "c:iPowerOn", "Flag"
      c_i_vlim = "c:i_vlim", "Voltage setpoint limit reached"
      c_dphi = "c:dphi", "deg, Measured angle difference"
      c_Pmeas = "c:Pmeas", "MW, Measured active power"
      c_nCtrlIter = "c:nCtrlIter", "No. of CTRL-iterations"
      c_Ploss = "c:Ploss", "MW, Losses (total)"
      c_Qloss = "c:Qloss", "Mvar, Reactive-Losses (total)"
      c_Plossld = "c:Plossld", "MW, Losses (load)"
      c_Qlossld = "c:Qlossld", "Mvar, Reactive-Losses (load)"
      c_Plossnld = "c:Plossnld", "MW, Losses (no load)"
      c_Qlossnld = "c:Qlossnld", "Mvar, Reactive-Losses (no load)"

    class ElmXnet(Enum):
      _init_ = 'value __doc__'

      m_dPdQ_bus1 = "m:dPdQ:bus1", "MW/Mvar, Branch Sensitivity dPbranch/dQbus"
      m_dQdQ_bus1 = "m:dQdQ:bus1", "Mvar/Mvar, Branch Sensitivity dQbranch/dQbus"
      m_dPdP_bus1 = "m:dPdP:bus1", "MW/MW, Branch Sensitivity dPbranch/dPbus"
      m_dQdP_bus1 = "m:dQdP:bus1", "Mvar/MW, Branch Sensitivity dQbranch/dPbus"
      m_dPdtap_bus1 = "m:dPdtap:bus1", "MW/tap, Branch Sensitivity dPbranch/dtap"
      m_dQdtap_bus1 = "m:dQdtap:bus1", "Mvar/tap, Branch Sensitivity dQbranch/dtap"
      m_LODF_bus1 = "m:LODF:bus1", "%, Line Outage Distribution Factor LODF"
      m_u1r_bus1 = "m:u1r:bus1", "p.u., Positive-Sequence-Voltage, Real Part"
      m_u1i_bus1 = "m:u1i:bus1", "p.u., Positive-Sequence-Voltage, Imaginary Part"
      m_u1_bus1 = "m:u1:bus1", "p.u., Positive-Sequence-Voltage, Magnitude"
      m_U1_bus1 = "m:U1:bus1", "kV, Line-Ground Positive-Sequence-Voltage, Magnitude"
      m_U1l_bus1 = "m:U1l:bus1", "kV, Line-Line Positive-Sequence-Voltage, Magnitude"
      m_phiu1_bus1 = "m:phiu1:bus1", "deg, Positive-Sequence-Voltage, Angle"
      m_inet_bus1 = "m:inet:bus1", "p.u., Current, Magnitude, referred to network"
      m_I_bus1 = "m:I:bus1", "kA, Current, Magnitude"
      m_phii_bus1 = "m:phii:bus1", "deg, Current, Angle"
      m_i1r_bus1 = "m:i1r:bus1", "p.u., Positive-Sequence Current, Real Part"
      m_i1i_bus1 = "m:i1i:bus1", "p.u., Positive-Sequence Current, Imaginary Part"
      m_i1_bus1 = "m:i1:bus1", "p.u., Positive-Sequence Current, Magnitude"
      m_I1_bus1 = "m:I1:bus1", "kA, Positive-Sequence Current, Magnitude"
      m_phii1_bus1 = "m:phii1:bus1", "deg, Positive-Sequence Current, Angle"
      m_P_bus1 = "m:P:bus1", "MW, Active Power"
      m_Q_bus1 = "m:Q:bus1", "Mvar, Reactive Power"
      m_S_bus1 = "m:S:bus1", "MVA, Apparent Power"
      m_cosphi_bus1 = "m:cosphi:bus1", "Power Factor"
      m_tanphi_bus1 = "m:tanphi:bus1", "tan(phi)"
      m_Psum_bus1 = "m:Psum:bus1", "MW, Total Active Power"
      m_Qsum_bus1 = "m:Qsum:bus1", "Mvar, Total Reactive Power"
      m_Ssum_bus1 = "m:Ssum:bus1", "MVA, Total Apparent Power"
      m_cosphisum_bus1 = "m:cosphisum:bus1", "Total Power Factor"
      m_tanphisum_bus1 = "m:tanphisum:bus1", "Total tan(phi)"
      m_Spu_bus1 = "m:Spu:bus1", "MVA/p.u., Apparent Power per p.u. Voltage"
      m_i1P_bus1 = "m:i1P:bus1", "p.u., Positive-Sequence Active Current"
      m_i1Q_bus1 = "m:i1Q:bus1", "p.u., Positive-Sequence Reactive Current"
      m_I1P_bus1 = "m:I1P:bus1", "kA, Positive-Sequence Active Current"
      m_I1Q_bus1 = "m:I1Q:bus1", "kA, Positive-Sequence Reactive Current"
      m_ur_bus1 = "m:ur:bus1", "p.u., Voltage, Real Part"
      m_ui_bus1 = "m:ui:bus1", "p.u., Voltage, Imaginary Part"
      m_u_bus1 = "m:u:bus1", "p.u., Voltage, Magnitude"
      m_ir_bus1 = "m:ir:bus1", "p.u., Current, Real Part"
      m_ii_bus1 = "m:ii:bus1", "p.u., Current, Imaginary Part"
      m_i_bus1 = "m:i:bus1", "p.u., Current, Magnitude"
      m_phiui_bus1 = "m:phiui:bus1", "deg, Angle between Voltage and Current"
      m_phiu1i1_bus1 = "m:phiu1i1:bus1", "deg, Angle between Voltage and Current in positive sequence system"
      m_Tfct_bus1 = "m:Tfct:bus1", "s, Fault Clearing Time"
      m_Brkload_bus1 = "m:Brkload:bus1", "%, Breaker Loading"
      n_dphidP_bus1 = "n:dphidP:bus1", "deg/MW, dphi/dP Sensitivity"
      n_dphidQ_bus1 = "n:dphidQ:bus1", "deg/Mvar, dphi/dQ Sensitivity"
      n_dphidtap_bus1 = "n:dphidtap:bus1", "deg/tap, dphi/dtap Sensitivity"
      n_dvdP_bus1 = "n:dvdP:bus1", "p.u./MW, dv/dP Sensitivity"
      n_dvdQ_bus1 = "n:dvdQ:bus1", "p.u./Mvar, dv/dQ Sensitivity"
      n_dvdtap_bus1 = "n:dvdtap:bus1", "p.u./tap, dv/dtap Sensitivity"
      n_levecQ_bus1 = "n:levecQ:bus1", "Left Eigenvector of selected Q-Mode"
      n_revecQ_bus1 = "n:revecQ:bus1", "Right Eigenvector of selected Q-Mode"
      n_partQ_bus1 = "n:partQ:bus1", "Participation Factor of selected Q-Mode"
      n_ur_bus1 = "n:ur:bus1", "p.u., Voltage, Real Part"
      n_ui_bus1 = "n:ui:bus1", "p.u., Voltage, Imaginary Part"
      n_u_bus1 = "n:u:bus1", "p.u., Voltage, Magnitude"
      n_upc_bus1 = "n:upc:bus1", "%, Voltage, Magnitude"
      n_u1_bus1 = "n:u1:bus1", "p.u., Voltage, Magnitude"
      n_u1pc_bus1 = "n:u1pc:bus1", "%, Voltage, Magnitude"
      n_u1r_bus1 = "n:u1r:bus1", "p.u., Positive-Sequence Voltage, Real Part"
      n_u1i_bus1 = "n:u1i:bus1", "p.u., Positive-Sequence Voltage, Imaginary Part"
      n_U_bus1 = "n:U:bus1", "kV, Line-Ground Voltage, Magnitude"
      n_Ul_bus1 = "n:Ul:bus1", "kV, Line-Line Voltage, Magnitude"
      n_phiu_bus1 = "n:phiu:bus1", "deg, Voltage, Angle"
      n_phiurel_bus1 = "n:phiurel:bus1", "deg, Voltage, Relative Angle"
      n_du_bus1 = "n:du:bus1", "%, Voltage Deviation"
      n_Pgen_bus1 = "n:Pgen:bus1", "MW, Generation, Active Power"
      n_Qgen_bus1 = "n:Qgen:bus1", "Mvar, Generation, Reactive Power"
      n_Pmot_bus1 = "n:Pmot:bus1", "MW, Motor Load, Active Power"
      n_Qmot_bus1 = "n:Qmot:bus1", "Mvar, Motor Load, Reactive Power"
      n_Pload_bus1 = "n:Pload:bus1", "MW, General Load, Active Power"
      n_Qload_bus1 = "n:Qload:bus1", "Mvar, General Load, Reactive Power"
      n_Pcomp_bus1 = "n:Pcomp:bus1", "MW, Compensation (Losses)"
      n_Qcomp_bus1 = "n:Qcomp:bus1", "Mvar, Compensation"
      n_Pnet_bus1 = "n:Pnet:bus1", "MW, External Networks, Active Power"
      n_Qnet_bus1 = "n:Qnet:bus1", "Mvar, External Networks, Reactive Power"
      n_Pflow_bus1 = "n:Pflow:bus1", "MW, Power Flow, Active Power"
      n_Qflow_bus1 = "n:Qflow:bus1", "Mvar, Power Flow, Reactive Power"
      n_Pout_bus1 = "n:Pout:bus1", "MW, Outgoing Flow, Active Power"
      n_Qout_bus1 = "n:Qout:bus1", "Mvar, Outgoing Power, Reactive Power"
      n_Sout_bus1 = "n:Sout:bus1", "MVA, Outgoing Power, Apparent Power"
      n_cosphiout_bus1 = "n:cosphiout:bus1", "Outgoing Power, Power Factor"
      n_Pbalance_bus1 = "n:Pbalance:bus1", "MW, Active Power Balance (=0)"
      n_Qbalance_bus1 = "n:Qbalance:bus1", "Mvar, Reactive Power Balance (=0)"
      n_LossPdown_bus1 = "n:LossPdown:bus1", "MW, Losses, downstream"
      n_LossQdown_bus1 = "n:LossQdown:bus1", "Mvar, Losses, downstream (Reactive Power)"
      n_LossPdownload_bus1 = "n:LossPdownload:bus1", "MW, Load losses, downstream"
      n_LossQdownload_bus1 = "n:LossQdownload:bus1", "Mvar, Load losses, downstream"
      n_LossPdownnoload_bus1 = "n:LossPdownnoload:bus1", "MW, No load losses, downstream"
      n_LossQdownnoload_bus1 = "n:LossQdownnoload:bus1", "Mvar, No load losses, downstream"
      n_du_feed_bus1 = "n:du_feed:bus1", "%, Voltage difference relative to feeder begin"
      s_pset = "s:pset", "MW, IN, Active Power Setpoint, continuous, double"
      s_qset = "s:qset", "Mvar, IN, Reactive Power Setpoint continuous, double"
      s_dFin = "s:dFin", "Hz, IN, Frequency Deviation, continuous, double"
      s_dF = "s:dF", "Hz, OUT, Frequency Deviation Output, continuous, double"
      c_phiini = "c:phiini", "Initial Angle"
      c_uini_r = "c:uini:r", "p.u., Voltage Setpoint for PV and Slack, Real Part"
      c_uini_i = "c:uini:i", "p.u., Voltage Setpoint for PV and Slack, Imaginary Part"
      c_uini_phi = "c:uini:phi", "deg, Voltage Setpoint for PV and Slack, Angle"
      c_uini_mag = "c:uini:mag", "p.u., Voltage Setpoint for PV and Slack, Magnitude"
      c_uini_phirad = "c:uini:phirad", "rad, Voltage Setpoint for PV and Slack, Angle"
      c_p = "c:p", "MW, Total Active Power"
      c_q = "c:q", "Mvar, Total Reactive Power"
      c_sgnn = "c:sgnn", "MVA, Total rated apparent power (ngnum*sgn)"
      c_cosn = "c:cosn", "Nominal-Power factor"
      c_c2 = "c:c2", "Voltage Coefficient (uknom/ugn)"
      c_iPowerOn = "c:iPowerOn", "Flag"
      c_Pdisp = "c:Pdisp", "MW, Power (Eco-Dispatch)"
      c_xdq = "c:xdq", "p.u., Synchronous Reactance"
      c_rs = "c:rs", "p.u., Resistance"
      c_i_qlim = "c:i_qlim", "Reactive power limit reached"
      c_Qmin = "c:Qmin", "Mvar, Lower limit of reactive power"
      c_Qmax = "c:Qmax", "Mvar, Upper limit of reactive power"
      c_sm_r = "c:sm:r", "MW, Complex Power, Real Part"
      c_sm_i = "c:sm:i", "Mvar, Complex Power, Imaginary Part"
      c_sm_phi = "c:sm:phi", "deg, Complex Power, Angle"
      c_sm_mag = "c:sm:mag", "MVA, Complex Power, Magnitude"
      c_sm_phirad = "c:sm:phirad", "rad, Complex Power, Angle"
      c_Kf = "c:Kf", "MW/Hz, Frequency Bias"
      c_Kp = "c:Kp", "p.u., Active Power Participation"
      c_iRefElement = "c:iRefElement", "Reference Element"
      c_nCtrlIter = "c:nCtrlIter", "No. of CTRL-iterations"
      c_Ploss = "c:Ploss", "MW, Losses (total)"
      c_Qloss = "c:Qloss", "Mvar, Reactive-Losses (total)"
      c_Plossld = "c:Plossld", "MW, Losses (load)"
      c_Qlossld = "c:Qlossld", "Mvar, Reactive-Losses (load)"
      c_Plossnld = "c:Plossnld", "MW, Losses (no load)"
      c_Qlossnld = "c:Qlossnld", "Mvar, Reactive-Losses (no load)"

    class ElmZone(Enum):
      _init_ = 'value __doc__'

      c_n_ele = "c:n_ele", "Number of Elements"
      c_n_zones = "c:n_zones", "Number of Connected Zones"
      c_n_volt = "c:n_volt", "Number of Voltage Levels"
      c_n_bar = "c:n_bar", "No. of Busbars"
      c_n_term = "c:n_term", "No. of Terminals"
      c_n_lne = "c:n_lne", "No. of Lines"
      c_n_sym = "c:n_sym", "No. of syn. Machines"
      c_n_asm = "c:n_asm", "No. of asyn. Machines"
      c_n_stg = "c:n_stg", "No. of Static Generators"
      c_n_pvs = "c:n_pvs", "No. of PV Systems"
      c_n_lod = "c:n_lod", "No. of Loads"
      c_n_cust = "c:n_cust", "No. of Customers"
      c_TotgenP = "c:TotgenP", "MW, Total Generation, Active Power"
      c_TotgenQ = "c:TotgenQ", "Mvar, Total Generation, Reactive Power"
      c_TotconsP = "c:TotconsP", "MW, Total Consumption, Active Power"
      c_TotconsQ = "c:TotconsQ", "Mvar, Total Consumption, Reactive Power"
      c_GenP = "c:GenP", "MW, Generators, Active Power"
      c_GenQ = "c:GenQ", "Mvar, Generators, Reactive Power"
      c_GenS = "c:GenS", "MVA, Generators, Apparent Power"
      c_Gencos = "c:Gencos", "Generators, Power Factor"
      c_GenPnom = "c:GenPnom", "MW, Generators, Nominal Active Power"
      c_GenQnom = "c:GenQnom", "Mvar, Generators, Nominal Reactive Power"
      c_GenSnom = "c:GenSnom", "MVA, Generators, Nominal Apparent Power"
      c_GenPdiff = "c:GenPdiff", "MW, Generators, difference between nominal and actual active power"
      c_GenQdiff = "c:GenQdiff", "Mvar, Generators, difference between nominal and actual reactive power"
      c_Gencosnom = "c:Gencosnom", "Generators, Nominal Power Factor"
      c_GenPdisp = "c:GenPdisp", "MW, Generators, Dispatch Active Power"
      c_GenPmism = "c:GenPmism", "MW, Generators, difference between actual and dispatched active power"
      c_GenPspinres = "c:GenPspinres", "MW, Generators, difference between maximum and actual active power"
      c_GenQspinres = "c:GenQspinres", "Mvar, Generators, difference between maximum and actual reactive power"
      c_MotP = "c:MotP", "MW, Motor Loads, Active Power"
      c_MotQ = "c:MotQ", "Mvar, Motor Loads, Reactive Power"
      c_MotS = "c:MotS", "MVA, Motor Loads, Apparent Power"
      c_Motcos = "c:Motcos", "Motor Loads, Power Factor"
      c_LoadP = "c:LoadP", "MW, Loads, Active Power"
      c_LoadQ = "c:LoadQ", "Mvar, Loads, Reactive Power"
      c_LoadS = "c:LoadS", "MVA, Loads, Apparent Power"
      c_Loadcos = "c:Loadcos", "Loads, Power Factor"
      c_LoadPnom = "c:LoadPnom", "MW, Loads, Nominal Active Power"
      c_LoadQnom = "c:LoadQnom", "Mvar, Loads, Nominal Reactive Power"
      c_LoadSnom = "c:LoadSnom", "MVA, Loads, Nominal Apparent Power"
      c_Loadcosnom = "c:Loadcosnom", "Loads, Nominal Power Factor"
      c_LoadPdiff = "c:LoadPdiff", "MW, Loads, difference between nominal and actual active power"
      c_LoadQdiff = "c:LoadQdiff", "Mvar, Loads, difference between nominal and actual reactive power"
      c_LoadPdem = "c:LoadPdem", "MW, Loads, Active Power Demand"
      c_LoadPmism = "c:LoadPmism", "MW, Loads, difference between actual active power and demand"
      c_CompP = "c:CompP", "MW, Compensation (Losses)"
      c_CompQ = "c:CompQ", "Mvar, Compensation, C"
      c_ComprQ = "c:ComprQ", "Mvar, Compensation, L"
      c_NetP = "c:NetP", "MW, External Networks, Active Power"
      c_NetQ = "c:NetQ", "Mvar, External Networks, Reactive Power"
      c_NetS = "c:NetS", "MVA, External Networks, Apparent Power"
      c_Netcos = "c:Netcos", "External Networks, Power Factor"
      c_InterP = "c:InterP", "MW, Interchange Flow, Active Power"
      c_InterQ = "c:InterQ", "Mvar, Interchange Flow, Reactive Power"
      c_LossP = "c:LossP", "MW, Losses, Active Power"
      c_LossQ = "c:LossQ", "Mvar, Losses, Reactive Power"
      c_LossPld = "c:LossPld", "MW, Losses, Active Power (load)"
      c_LossQld = "c:LossQld", "Mvar, Losses, Reactive Power (load)"
      c_LossPnld = "c:LossPnld", "MW, Losses, Active Power (no load)"
      c_LossQnld = "c:LossQnld", "Mvar, Losses, Reactive Power (no load)"
      c_LossPlne = "c:LossPlne", "MW, Line Losses, Active Power"
      c_LossQlne = "c:LossQlne", "Mvar, Line Losses, Reactive Power"
      c_LossPldlne = "c:LossPldlne", "MW, Line Losses, Active Power (load)"
      c_LossQldlne = "c:LossQldlne", "Mvar, Line Losses, Reactive Power (load)"
      c_LossPnldlne = "c:LossPnldlne", "MW, Line Losses, Active Power (no load)"
      c_LossQnldlne = "c:LossQnldlne", "Mvar, Line Losses, Reactive Power (no load)"
      c_LossPtrf = "c:LossPtrf", "MW, Transformer Losses, Active Power"
      c_LossQtrf = "c:LossQtrf", "Mvar, Transformer Losses, Reactive Power"
      c_LossPldtrf = "c:LossPldtrf", "MW, Transformer Losses, Active Power (load)"
      c_LossQldtrf = "c:LossQldtrf", "Mvar, Transformer Losses, Reactive Power (load)"
      c_LossPnldtrf = "c:LossPnldtrf", "MW, Transformer Losses, Active Power (no load)"
      c_LossQnldtrf = "c:LossQnldtrf", "Mvar, Transformer Losses, Reactive Power (no load)"
      c_TotLoadP = "c:TotLoadP", "MW, Total Load, Active Power"
      c_TotLoadQ = "c:TotLoadQ", "Mvar, Total Load, Reactive Power"
      c_TotLoadS = "c:TotLoadS", "MVA, Total Load, Apparent Power"
      c_TotLoadcos = "c:TotLoadcos", "Total Load, Power Factor"
      c_TotP = "c:TotP", "MW, Total Demand"
      c_UnsupP = "c:UnsupP", "MW, Unsupplied Demand"
      c_SupP = "c:SupP", "MW, Supplied Demand"
      c_ExportP = "c:ExportP", "MW, Export, Active Power"
      c_ExportQ = "c:ExportQ", "Mvar, Export, Reactive Power"
      c_ImportP = "c:ImportP", "MW, Import, Active Power"
      c_ImportQ = "c:ImportQ", "Mvar, Import, Reactive Power"
      c_cst_disp = "c:cst_disp", "USD/h, Total Production Costs"
      c_NrOvlBranch = "c:NrOvlBranch", "Number of overloaded branch components"
      c_NrGenQMax = "c:NrGenQMax", "Number of generators above their maximum reactive power limit"
      c_NrGenQMin = "c:NrGenQMin", "Number of generators below their minimum reactive power limit"
      c_NrTermMin = "c:NrTermMin", "Number of terminals below their minimum voltage limit"
      c_NrTermMax = "c:NrTermMax", "Number of terminals above their maximum voltage limit"
      c_RUC_Lead = "c:RUC_Lead", "Mvar, Used Reactive Capacity Lead"
      c_RUC_Lag = "c:RUC_Lag", "Mvar, Used Reactive Capacity Lag"
      c_RSC_Lead = "c:RSC_Lead", "Mvar, Spare Reactive Capacity Lead"
      c_RSC_Lag = "c:RSC_Lag", "Mvar, Spare Reactive Capacity Lag"
      c_Q_shgain = "c:Q_shgain", "Mvar, Reactive Power Shunt Gain"
      c_ConnMvar_max = "c:ConnMvar_max", "Mvar, Connected MVAR Capacity (max)"
      c_ConnMvar_min = "c:ConnMvar_min", "Mvar, Connected MVAR Capacity (min)"
      c_Maxloading = "c:Maxloading", "%, Maximum Loading"
      c_umin = "c:umin", "p.u., Minimum voltage of all terminals"
      c_umax = "c:umax", "p.u., Maximum voltage of all terminals"

    class ElmZpu(Enum):
      _init_ = 'value __doc__'

      m_dPdQ_bus1 = "m:dPdQ:bus1", "MW/Mvar, Branch Sensitivity dPbranch/dQbus"
      m_dPdQ_bus2 = "m:dPdQ:bus2", "MW/Mvar, Branch Sensitivity dPbranch/dQbus"
      m_dQdQ_bus1 = "m:dQdQ:bus1", "Mvar/Mvar, Branch Sensitivity dQbranch/dQbus"
      m_dQdQ_bus2 = "m:dQdQ:bus2", "Mvar/Mvar, Branch Sensitivity dQbranch/dQbus"
      m_dPdP_bus1 = "m:dPdP:bus1", "MW/MW, Branch Sensitivity dPbranch/dPbus"
      m_dPdP_bus2 = "m:dPdP:bus2", "MW/MW, Branch Sensitivity dPbranch/dPbus"
      m_dQdP_bus1 = "m:dQdP:bus1", "Mvar/MW, Branch Sensitivity dQbranch/dPbus"
      m_dQdP_bus2 = "m:dQdP:bus2", "Mvar/MW, Branch Sensitivity dQbranch/dPbus"
      m_dPdtap_bus1 = "m:dPdtap:bus1", "MW/tap, Branch Sensitivity dPbranch/dtap"
      m_dPdtap_bus2 = "m:dPdtap:bus2", "MW/tap, Branch Sensitivity dPbranch/dtap"
      m_dQdtap_bus1 = "m:dQdtap:bus1", "Mvar/tap, Branch Sensitivity dQbranch/dtap"
      m_dQdtap_bus2 = "m:dQdtap:bus2", "Mvar/tap, Branch Sensitivity dQbranch/dtap"
      m_LODF_bus1 = "m:LODF:bus1", "%, Line Outage Distribution Factor LODF"
      m_LODF_bus2 = "m:LODF:bus2", "%, Line Outage Distribution Factor LODF"
      m_u1r_bus1 = "m:u1r:bus1", "p.u., Positive-Sequence-Voltage, Real Part"
      m_u1r_bus2 = "m:u1r:bus2", "p.u., Positive-Sequence-Voltage, Real Part"
      m_u1i_bus1 = "m:u1i:bus1", "p.u., Positive-Sequence-Voltage, Imaginary Part"
      m_u1i_bus2 = "m:u1i:bus2", "p.u., Positive-Sequence-Voltage, Imaginary Part"
      m_u1_bus1 = "m:u1:bus1", "p.u., Positive-Sequence-Voltage, Magnitude"
      m_u1_bus2 = "m:u1:bus2", "p.u., Positive-Sequence-Voltage, Magnitude"
      m_U1_bus1 = "m:U1:bus1", "kV, Line-Ground Positive-Sequence-Voltage, Magnitude"
      m_U1_bus2 = "m:U1:bus2", "kV, Line-Ground Positive-Sequence-Voltage, Magnitude"
      m_U1l_bus1 = "m:U1l:bus1", "kV, Line-Line Positive-Sequence-Voltage, Magnitude"
      m_U1l_bus2 = "m:U1l:bus2", "kV, Line-Line Positive-Sequence-Voltage, Magnitude"
      m_phiu1_bus1 = "m:phiu1:bus1", "deg, Positive-Sequence-Voltage, Angle"
      m_phiu1_bus2 = "m:phiu1:bus2", "deg, Positive-Sequence-Voltage, Angle"
      m_inet_bus1 = "m:inet:bus1", "p.u., Current, Magnitude, referred to network"
      m_inet_bus2 = "m:inet:bus2", "p.u., Current, Magnitude, referred to network"
      m_I_bus1 = "m:I:bus1", "kA, Current, Magnitude"
      m_I_bus2 = "m:I:bus2", "kA, Current, Magnitude"
      m_phii_bus1 = "m:phii:bus1", "deg, Current, Angle"
      m_phii_bus2 = "m:phii:bus2", "deg, Current, Angle"
      m_i1r_bus1 = "m:i1r:bus1", "p.u., Positive-Sequence Current, Real Part"
      m_i1r_bus2 = "m:i1r:bus2", "p.u., Positive-Sequence Current, Real Part"
      m_i1i_bus1 = "m:i1i:bus1", "p.u., Positive-Sequence Current, Imaginary Part"
      m_i1i_bus2 = "m:i1i:bus2", "p.u., Positive-Sequence Current, Imaginary Part"
      m_i1_bus1 = "m:i1:bus1", "p.u., Positive-Sequence Current, Magnitude"
      m_i1_bus2 = "m:i1:bus2", "p.u., Positive-Sequence Current, Magnitude"
      m_I1_bus1 = "m:I1:bus1", "kA, Positive-Sequence Current, Magnitude"
      m_I1_bus2 = "m:I1:bus2", "kA, Positive-Sequence Current, Magnitude"
      m_phii1_bus1 = "m:phii1:bus1", "deg, Positive-Sequence Current, Angle"
      m_phii1_bus2 = "m:phii1:bus2", "deg, Positive-Sequence Current, Angle"
      m_P_bus1 = "m:P:bus1", "MW, Active Power"
      m_P_bus2 = "m:P:bus2", "MW, Active Power"
      m_Q_bus1 = "m:Q:bus1", "Mvar, Reactive Power"
      m_Q_bus2 = "m:Q:bus2", "Mvar, Reactive Power"
      m_S_bus1 = "m:S:bus1", "MVA, Apparent Power"
      m_S_bus2 = "m:S:bus2", "MVA, Apparent Power"
      m_cosphi_bus1 = "m:cosphi:bus1", "Power Factor"
      m_cosphi_bus2 = "m:cosphi:bus2", "Power Factor"
      m_tanphi_bus1 = "m:tanphi:bus1", "tan(phi)"
      m_tanphi_bus2 = "m:tanphi:bus2", "tan(phi)"
      m_Psum_bus1 = "m:Psum:bus1", "MW, Total Active Power"
      m_Psum_bus2 = "m:Psum:bus2", "MW, Total Active Power"
      m_Qsum_bus1 = "m:Qsum:bus1", "Mvar, Total Reactive Power"
      m_Qsum_bus2 = "m:Qsum:bus2", "Mvar, Total Reactive Power"
      m_Ssum_bus1 = "m:Ssum:bus1", "MVA, Total Apparent Power"
      m_Ssum_bus2 = "m:Ssum:bus2", "MVA, Total Apparent Power"
      m_cosphisum_bus1 = "m:cosphisum:bus1", "Total Power Factor"
      m_cosphisum_bus2 = "m:cosphisum:bus2", "Total Power Factor"
      m_tanphisum_bus1 = "m:tanphisum:bus1", "Total tan(phi)"
      m_tanphisum_bus2 = "m:tanphisum:bus2", "Total tan(phi)"
      m_Spu_bus1 = "m:Spu:bus1", "MVA/p.u., Apparent Power per p.u. Voltage"
      m_Spu_bus2 = "m:Spu:bus2", "MVA/p.u., Apparent Power per p.u. Voltage"
      m_i1P_bus1 = "m:i1P:bus1", "p.u., Positive-Sequence Active Current"
      m_i1P_bus2 = "m:i1P:bus2", "p.u., Positive-Sequence Active Current"
      m_i1Q_bus1 = "m:i1Q:bus1", "p.u., Positive-Sequence Reactive Current"
      m_i1Q_bus2 = "m:i1Q:bus2", "p.u., Positive-Sequence Reactive Current"
      m_I1P_bus1 = "m:I1P:bus1", "kA, Positive-Sequence Active Current"
      m_I1P_bus2 = "m:I1P:bus2", "kA, Positive-Sequence Active Current"
      m_I1Q_bus1 = "m:I1Q:bus1", "kA, Positive-Sequence Reactive Current"
      m_I1Q_bus2 = "m:I1Q:bus2", "kA, Positive-Sequence Reactive Current"
      m_ur_bus1 = "m:ur:bus1", "p.u., Voltage, Real Part"
      m_ur_bus2 = "m:ur:bus2", "p.u., Voltage, Real Part"
      m_ui_bus1 = "m:ui:bus1", "p.u., Voltage, Imaginary Part"
      m_ui_bus2 = "m:ui:bus2", "p.u., Voltage, Imaginary Part"
      m_u_bus1 = "m:u:bus1", "p.u., Voltage, Magnitude"
      m_u_bus2 = "m:u:bus2", "p.u., Voltage, Magnitude"
      m_ir_bus1 = "m:ir:bus1", "p.u., Current, Real Part"
      m_ir_bus2 = "m:ir:bus2", "p.u., Current, Real Part"
      m_ii_bus1 = "m:ii:bus1", "p.u., Current, Imaginary Part"
      m_ii_bus2 = "m:ii:bus2", "p.u., Current, Imaginary Part"
      m_i_bus1 = "m:i:bus1", "p.u., Current, Magnitude"
      m_i_bus2 = "m:i:bus2", "p.u., Current, Magnitude"
      m_phiui_bus1 = "m:phiui:bus1", "deg, Angle between Voltage and Current"
      m_phiui_bus2 = "m:phiui:bus2", "deg, Angle between Voltage and Current"
      m_phiu1i1_bus1 = "m:phiu1i1:bus1", "deg, Angle between Voltage and Current in positive sequence system"
      m_phiu1i1_bus2 = "m:phiu1i1:bus2", "deg, Angle between Voltage and Current in positive sequence system"
      m_Tfct_bus1 = "m:Tfct:bus1", "s, Fault Clearing Time"
      m_Tfct_bus2 = "m:Tfct:bus2", "s, Fault Clearing Time"
      m_Brkload_bus1 = "m:Brkload:bus1", "%, Breaker Loading"
      m_Brkload_bus2 = "m:Brkload:bus2", "%, Breaker Loading"
      n_dphidP_bus1 = "n:dphidP:bus1", "deg/MW, dphi/dP Sensitivity"
      n_dphidP_bus2 = "n:dphidP:bus2", "deg/MW, dphi/dP Sensitivity"
      n_dphidQ_bus1 = "n:dphidQ:bus1", "deg/Mvar, dphi/dQ Sensitivity"
      n_dphidQ_bus2 = "n:dphidQ:bus2", "deg/Mvar, dphi/dQ Sensitivity"
      n_dphidtap_bus1 = "n:dphidtap:bus1", "deg/tap, dphi/dtap Sensitivity"
      n_dphidtap_bus2 = "n:dphidtap:bus2", "deg/tap, dphi/dtap Sensitivity"
      n_dvdP_bus1 = "n:dvdP:bus1", "p.u./MW, dv/dP Sensitivity"
      n_dvdP_bus2 = "n:dvdP:bus2", "p.u./MW, dv/dP Sensitivity"
      n_dvdQ_bus1 = "n:dvdQ:bus1", "p.u./Mvar, dv/dQ Sensitivity"
      n_dvdQ_bus2 = "n:dvdQ:bus2", "p.u./Mvar, dv/dQ Sensitivity"
      n_dvdtap_bus1 = "n:dvdtap:bus1", "p.u./tap, dv/dtap Sensitivity"
      n_dvdtap_bus2 = "n:dvdtap:bus2", "p.u./tap, dv/dtap Sensitivity"
      n_levecQ_bus1 = "n:levecQ:bus1", "Left Eigenvector of selected Q-Mode"
      n_levecQ_bus2 = "n:levecQ:bus2", "Left Eigenvector of selected Q-Mode"
      n_revecQ_bus1 = "n:revecQ:bus1", "Right Eigenvector of selected Q-Mode"
      n_revecQ_bus2 = "n:revecQ:bus2", "Right Eigenvector of selected Q-Mode"
      n_partQ_bus1 = "n:partQ:bus1", "Participation Factor of selected Q-Mode"
      n_partQ_bus2 = "n:partQ:bus2", "Participation Factor of selected Q-Mode"
      n_ur_bus1 = "n:ur:bus1", "p.u., Voltage, Real Part"
      n_ur_bus2 = "n:ur:bus2", "p.u., Voltage, Real Part"
      n_ui_bus1 = "n:ui:bus1", "p.u., Voltage, Imaginary Part"
      n_ui_bus2 = "n:ui:bus2", "p.u., Voltage, Imaginary Part"
      n_u_bus1 = "n:u:bus1", "p.u., Voltage, Magnitude"
      n_u_bus2 = "n:u:bus2", "p.u., Voltage, Magnitude"
      n_upc_bus1 = "n:upc:bus1", "%, Voltage, Magnitude"
      n_upc_bus2 = "n:upc:bus2", "%, Voltage, Magnitude"
      n_u1_bus1 = "n:u1:bus1", "p.u., Voltage, Magnitude"
      n_u1_bus2 = "n:u1:bus2", "p.u., Voltage, Magnitude"
      n_u1pc_bus1 = "n:u1pc:bus1", "%, Voltage, Magnitude"
      n_u1pc_bus2 = "n:u1pc:bus2", "%, Voltage, Magnitude"
      n_u1r_bus1 = "n:u1r:bus1", "p.u., Positive-Sequence Voltage, Real Part"
      n_u1r_bus2 = "n:u1r:bus2", "p.u., Positive-Sequence Voltage, Real Part"
      n_u1i_bus1 = "n:u1i:bus1", "p.u., Positive-Sequence Voltage, Imaginary Part"
      n_u1i_bus2 = "n:u1i:bus2", "p.u., Positive-Sequence Voltage, Imaginary Part"
      n_U_bus1 = "n:U:bus1", "kV, Line-Ground Voltage, Magnitude"
      n_U_bus2 = "n:U:bus2", "kV, Line-Ground Voltage, Magnitude"
      n_Ul_bus1 = "n:Ul:bus1", "kV, Line-Line Voltage, Magnitude"
      n_Ul_bus2 = "n:Ul:bus2", "kV, Line-Line Voltage, Magnitude"
      n_phiu_bus1 = "n:phiu:bus1", "deg, Voltage, Angle"
      n_phiu_bus2 = "n:phiu:bus2", "deg, Voltage, Angle"
      n_phiurel_bus1 = "n:phiurel:bus1", "deg, Voltage, Relative Angle"
      n_phiurel_bus2 = "n:phiurel:bus2", "deg, Voltage, Relative Angle"
      n_du_bus1 = "n:du:bus1", "%, Voltage Deviation"
      n_du_bus2 = "n:du:bus2", "%, Voltage Deviation"
      n_Pgen_bus1 = "n:Pgen:bus1", "MW, Generation, Active Power"
      n_Pgen_bus2 = "n:Pgen:bus2", "MW, Generation, Active Power"
      n_Qgen_bus1 = "n:Qgen:bus1", "Mvar, Generation, Reactive Power"
      n_Qgen_bus2 = "n:Qgen:bus2", "Mvar, Generation, Reactive Power"
      n_Pmot_bus1 = "n:Pmot:bus1", "MW, Motor Load, Active Power"
      n_Pmot_bus2 = "n:Pmot:bus2", "MW, Motor Load, Active Power"
      n_Qmot_bus1 = "n:Qmot:bus1", "Mvar, Motor Load, Reactive Power"
      n_Qmot_bus2 = "n:Qmot:bus2", "Mvar, Motor Load, Reactive Power"
      n_Pload_bus1 = "n:Pload:bus1", "MW, General Load, Active Power"
      n_Pload_bus2 = "n:Pload:bus2", "MW, General Load, Active Power"
      n_Qload_bus1 = "n:Qload:bus1", "Mvar, General Load, Reactive Power"
      n_Qload_bus2 = "n:Qload:bus2", "Mvar, General Load, Reactive Power"
      n_Pcomp_bus1 = "n:Pcomp:bus1", "MW, Compensation (Losses)"
      n_Pcomp_bus2 = "n:Pcomp:bus2", "MW, Compensation (Losses)"
      n_Qcomp_bus1 = "n:Qcomp:bus1", "Mvar, Compensation"
      n_Qcomp_bus2 = "n:Qcomp:bus2", "Mvar, Compensation"
      n_Pnet_bus1 = "n:Pnet:bus1", "MW, External Networks, Active Power"
      n_Pnet_bus2 = "n:Pnet:bus2", "MW, External Networks, Active Power"
      n_Qnet_bus1 = "n:Qnet:bus1", "Mvar, External Networks, Reactive Power"
      n_Qnet_bus2 = "n:Qnet:bus2", "Mvar, External Networks, Reactive Power"
      n_Pflow_bus1 = "n:Pflow:bus1", "MW, Power Flow, Active Power"
      n_Pflow_bus2 = "n:Pflow:bus2", "MW, Power Flow, Active Power"
      n_Qflow_bus1 = "n:Qflow:bus1", "Mvar, Power Flow, Reactive Power"
      n_Qflow_bus2 = "n:Qflow:bus2", "Mvar, Power Flow, Reactive Power"
      n_Pout_bus1 = "n:Pout:bus1", "MW, Outgoing Flow, Active Power"
      n_Pout_bus2 = "n:Pout:bus2", "MW, Outgoing Flow, Active Power"
      n_Qout_bus1 = "n:Qout:bus1", "Mvar, Outgoing Power, Reactive Power"
      n_Qout_bus2 = "n:Qout:bus2", "Mvar, Outgoing Power, Reactive Power"
      n_Sout_bus1 = "n:Sout:bus1", "MVA, Outgoing Power, Apparent Power"
      n_Sout_bus2 = "n:Sout:bus2", "MVA, Outgoing Power, Apparent Power"
      n_cosphiout_bus1 = "n:cosphiout:bus1", "Outgoing Power, Power Factor"
      n_cosphiout_bus2 = "n:cosphiout:bus2", "Outgoing Power, Power Factor"
      n_Pbalance_bus1 = "n:Pbalance:bus1", "MW, Active Power Balance (=0)"
      n_Pbalance_bus2 = "n:Pbalance:bus2", "MW, Active Power Balance (=0)"
      n_Qbalance_bus1 = "n:Qbalance:bus1", "Mvar, Reactive Power Balance (=0)"
      n_Qbalance_bus2 = "n:Qbalance:bus2", "Mvar, Reactive Power Balance (=0)"
      n_LossPdown_bus1 = "n:LossPdown:bus1", "MW, Losses, downstream"
      n_LossPdown_bus2 = "n:LossPdown:bus2", "MW, Losses, downstream"
      n_LossQdown_bus1 = "n:LossQdown:bus1", "Mvar, Losses, downstream (Reactive Power)"
      n_LossQdown_bus2 = "n:LossQdown:bus2", "Mvar, Losses, downstream (Reactive Power)"
      n_LossPdownload_bus1 = "n:LossPdownload:bus1", "MW, Load losses, downstream"
      n_LossPdownload_bus2 = "n:LossPdownload:bus2", "MW, Load losses, downstream"
      n_LossQdownload_bus1 = "n:LossQdownload:bus1", "Mvar, Load losses, downstream"
      n_LossQdownload_bus2 = "n:LossQdownload:bus2", "Mvar, Load losses, downstream"
      n_LossPdownnoload_bus1 = "n:LossPdownnoload:bus1", "MW, No load losses, downstream"
      n_LossPdownnoload_bus2 = "n:LossPdownnoload:bus2", "MW, No load losses, downstream"
      n_LossQdownnoload_bus1 = "n:LossQdownnoload:bus1", "Mvar, No load losses, downstream"
      n_LossQdownnoload_bus2 = "n:LossQdownnoload:bus2", "Mvar, No load losses, downstream"
      n_du_feed_bus1 = "n:du_feed:bus1", "%, Voltage difference relative to feeder begin"
      n_du_feed_bus2 = "n:du_feed:bus2", "%, Voltage difference relative to feeder begin"
      c_z1_ij_r = "c:z1_ij:r", "p.u., Impedance i-j, Real Part"
      c_z1_ij_i = "c:z1_ij:i", "p.u., Impedance i-j, Imaginary Part"
      c_z1_ij_phi = "c:z1_ij:phi", "deg, Impedance i-j, Angle"
      c_z1_ij_mag = "c:z1_ij:mag", "p.u., Impedance i-j, Magnitude"
      c_z1_ij_phirad = "c:z1_ij:phirad", "rad, Impedance i-j, Angle"
      c_z1_ji_r = "c:z1_ji:r", "p.u., Impedance j-i, Real Part"
      c_z1_ji_i = "c:z1_ji:i", "p.u., Impedance j-i, Imaginary Part"
      c_z1_ji_phi = "c:z1_ji:phi", "deg, Impedance j-i, Angle"
      c_z1_ji_mag = "c:z1_ji:mag", "p.u., Impedance j-i, Magnitude"
      c_z1_ji_phirad = "c:z1_ji:phirad", "rad, Impedance j-i, Angle"
      c_y1_ij_r = "c:y1_ij:r", "p.u., Admittance i-j, Real Part"
      c_y1_ij_i = "c:y1_ij:i", "p.u., Admittance i-j, Imaginary Part"
      c_y1_ij_phi = "c:y1_ij:phi", "deg, Admittance i-j, Angle"
      c_y1_ij_mag = "c:y1_ij:mag", "p.u., Admittance i-j, Magnitude"
      c_y1_ij_phirad = "c:y1_ij:phirad", "rad, Admittance i-j, Angle"
      c_y1_ji_r = "c:y1_ji:r", "p.u., Admittance j-i, Real Part"
      c_y1_ji_i = "c:y1_ji:i", "p.u., Admittance j-i, Imaginary Part"
      c_y1_ji_phi = "c:y1_ji:phi", "deg, Admittance j-i, Angle"
      c_y1_ji_mag = "c:y1_ji:mag", "p.u., Admittance j-i, Magnitude"
      c_y1_ji_phirad = "c:y1_ji:phirad", "rad, Admittance j-i, Angle"
      c_du = "c:du", "p.u., Voltage Drop"
      c_dupc = "c:dupc", "%, Voltage Drop"
      c_dphiu = "c:dphiu", "deg, Voltage Drop Angle"
      c_du1 = "c:du1", "p.u., Positive Sequence Voltage Drop"
      c_du1pc = "c:du1pc", "%, Positive Sequence Voltage Drop"
      c_dphiu1 = "c:dphiu1", "deg, Positive Sequence Voltage Drop Angle"
      c_Ploss = "c:Ploss", "MW, Losses (total)"
      c_Qloss = "c:Qloss", "Mvar, Reactive-Losses (total)"
      c_Plossld = "c:Plossld", "MW, Losses (load)"
      c_Qlossld = "c:Qlossld", "Mvar, Reactive-Losses (load)"
      c_Plossnld = "c:Plossnld", "MW, Losses (no load)"
      c_Qlossnld = "c:Qlossnld", "Mvar, Reactive-Losses (no load)"
